/* Generated by Cython 3.2.5 */

/* BEGIN: Cython Metadata
{
    "distutils": {
        "define_macros": [
            [
                "_FORCE_INLINES",
                "1"
            ],
            [
                "CYTHON_EXTERN_C",
                "extern \"C\""
            ],
            [
                "NPY_TARGET_VERSION",
                "NPY_2_0_API_VERSION"
            ],
            [
                "CUPY_USE_HIP",
                "1"
            ],
            [
                "__HIP_PLATFORM_AMD__",
                "1"
            ],
            [
                "__HIP_PLATFORM_HCC__",
                "1"
            ],
            [
                "THRUST_DEVICE_SYSTEM",
                "THRUST_DEVICE_SYSTEM_HIP"
            ],
            [
                "CUPY_CACHE_KEY",
                "28a6d0d64fe089c15e4ff802c7b0f8edb50d2a99"
            ],
            [
                "CUPY_CUB_VERSION_CODE",
                400200
            ]
        ],
        "depends": [
            "cupy_backends/cupy_complex.h",
            "cupy_backends/cupy_lapack.h"
        ],
        "extra_compile_args": [
            "-std=c++14"
        ],
        "extra_link_args": [
            "-Wl,--disable-new-dtags,-rpath,/opt/rocm/hipfft/lib,-rpath,/opt/rocm/lib"
        ],
        "include_dirs": [
            "cupyx",
            "/opt/rocm/hipfft/include",
            "/opt/rocm/include/hipcub",
            "/build/python-cupy/src/python-cupy-rocm-gfx11/cupy/_core/include",
            "/opt/rocm/include",
            "/opt/rocm/include/hip",
            "/opt/rocm/include/rocrand",
            "/opt/rocm/include/hiprand",
            "/opt/rocm/include/roctracer",
            "/opt/rocm/include/hipblas",
            "/opt/rocm/include/hipsparse",
            "/opt/rocm/include/hipfft",
            "/opt/rocm/include/rocsolver",
            "/opt/rocm/include/rccl",
            "/usr/lib/python3.14/site-packages/numpy/_core/include"
        ],
        "language": "c++",
        "libraries": [
            "amdhip64",
            "hipblas",
            "hipfft",
            "hiprand",
            "hipsparse",
            "rocfft",
            "roctx64",
            "rocblas",
            "rocsolver",
            "rocsparse"
        ],
        "library_dirs": [
            "/opt/rocm/hipfft/lib",
            "/opt/rocm/lib"
        ],
        "name": "cupyx.cusolver",
        "sources": [
            "cupyx/cusolver.pyx"
        ]
    },
    "module_name": "cupyx.cusolver"
}
END: Cython Metadata */

#ifndef PY_SSIZE_T_CLEAN
#define PY_SSIZE_T_CLEAN
#endif /* PY_SSIZE_T_CLEAN */
/* InitLimitedAPI */
#if defined(Py_LIMITED_API)
  #if !defined(CYTHON_LIMITED_API)
  #define CYTHON_LIMITED_API 1
  #endif
#elif defined(CYTHON_LIMITED_API)
  #ifdef _MSC_VER
  #pragma message ("Limited API usage is enabled with 'CYTHON_LIMITED_API' but 'Py_LIMITED_API' does not define a Python target version. Consider setting 'Py_LIMITED_API' instead.")
  #else
  #warning Limited API usage is enabled with 'CYTHON_LIMITED_API' but 'Py_LIMITED_API' does not define a Python target version. Consider setting 'Py_LIMITED_API' instead.
  #endif
#endif

#include "Python.h"
#ifndef Py_PYTHON_H
    #error Python headers needed to compile C extensions, please install development version of Python.
#elif PY_VERSION_HEX < 0x03080000
    #error Cython requires Python 3.8+.
#else
#define __PYX_ABI_VERSION "3_2_5"
#define CYTHON_HEX_VERSION 0x030205F0
#define CYTHON_FUTURE_DIVISION 1
/* CModulePreamble */
#include <stddef.h>
#ifndef offsetof
  #define offsetof(type, member) ( (size_t) & ((type*)0) -> member )
#endif
#if !defined(_WIN32) && !defined(WIN32) && !defined(MS_WINDOWS)
  #ifndef __stdcall
    #define __stdcall
  #endif
  #ifndef __cdecl
    #define __cdecl
  #endif
  #ifndef __fastcall
    #define __fastcall
  #endif
#endif
#ifndef DL_IMPORT
  #define DL_IMPORT(t) t
#endif
#ifndef DL_EXPORT
  #define DL_EXPORT(t) t
#endif
#define __PYX_COMMA ,
#ifndef PY_LONG_LONG
  #define PY_LONG_LONG LONG_LONG
#endif
#ifndef Py_HUGE_VAL
  #define Py_HUGE_VAL HUGE_VAL
#endif
#define __PYX_LIMITED_VERSION_HEX PY_VERSION_HEX
#if defined(GRAALVM_PYTHON)
  /* For very preliminary testing purposes. Most variables are set the same as PyPy.
     The existence of this section does not imply that anything works or is even tested */
  #define CYTHON_COMPILING_IN_PYPY 0
  #define CYTHON_COMPILING_IN_CPYTHON 0
  #define CYTHON_COMPILING_IN_LIMITED_API 0
  #define CYTHON_COMPILING_IN_GRAAL 1
  #define CYTHON_COMPILING_IN_CPYTHON_FREETHREADING 0
  #undef CYTHON_USE_TYPE_SLOTS
  #define CYTHON_USE_TYPE_SLOTS 0
  #undef CYTHON_USE_TYPE_SPECS
  #define CYTHON_USE_TYPE_SPECS 0
  #undef CYTHON_USE_PYTYPE_LOOKUP
  #define CYTHON_USE_PYTYPE_LOOKUP 0
  #undef CYTHON_USE_PYLIST_INTERNALS
  #define CYTHON_USE_PYLIST_INTERNALS 0
  #undef CYTHON_USE_UNICODE_INTERNALS
  #define CYTHON_USE_UNICODE_INTERNALS 0
  #undef CYTHON_USE_UNICODE_WRITER
  #define CYTHON_USE_UNICODE_WRITER 0
  #undef CYTHON_USE_PYLONG_INTERNALS
  #define CYTHON_USE_PYLONG_INTERNALS 0
  #undef CYTHON_AVOID_BORROWED_REFS
  #define CYTHON_AVOID_BORROWED_REFS 1
  #undef CYTHON_AVOID_THREAD_UNSAFE_BORROWED_REFS
  #define CYTHON_AVOID_THREAD_UNSAFE_BORROWED_REFS 0
  #undef CYTHON_ASSUME_SAFE_MACROS
  #define CYTHON_ASSUME_SAFE_MACROS 0
  #undef CYTHON_ASSUME_SAFE_SIZE
  #define CYTHON_ASSUME_SAFE_SIZE 0
  #undef CYTHON_UNPACK_METHODS
  #define CYTHON_UNPACK_METHODS 0
  #undef CYTHON_FAST_THREAD_STATE
  #define CYTHON_FAST_THREAD_STATE 0
  #undef CYTHON_FAST_GIL
  #define CYTHON_FAST_GIL 0
  #undef CYTHON_METH_FASTCALL
  #define CYTHON_METH_FASTCALL 0
  #undef CYTHON_FAST_PYCALL
  #define CYTHON_FAST_PYCALL 0
  #ifndef CYTHON_PEP487_INIT_SUBCLASS
    #define CYTHON_PEP487_INIT_SUBCLASS 1
  #endif
  #undef CYTHON_PEP489_MULTI_PHASE_INIT
  #define CYTHON_PEP489_MULTI_PHASE_INIT 1
  #undef CYTHON_USE_MODULE_STATE
  #define CYTHON_USE_MODULE_STATE 0
  #undef CYTHON_USE_SYS_MONITORING
  #define CYTHON_USE_SYS_MONITORING 0
  #undef CYTHON_USE_TP_FINALIZE
  #define CYTHON_USE_TP_FINALIZE 0
  #undef CYTHON_USE_AM_SEND
  #define CYTHON_USE_AM_SEND 0
  #undef CYTHON_USE_DICT_VERSIONS
  #define CYTHON_USE_DICT_VERSIONS 0
  #undef CYTHON_USE_EXC_INFO_STACK
  #define CYTHON_USE_EXC_INFO_STACK 1
  #ifndef CYTHON_UPDATE_DESCRIPTOR_DOC
    #define CYTHON_UPDATE_DESCRIPTOR_DOC 0
  #endif
  #undef CYTHON_USE_FREELISTS
  #define CYTHON_USE_FREELISTS 0
  #undef CYTHON_IMMORTAL_CONSTANTS
  #define CYTHON_IMMORTAL_CONSTANTS 0
#elif defined(PYPY_VERSION)
  #define CYTHON_COMPILING_IN_PYPY 1
  #define CYTHON_COMPILING_IN_CPYTHON 0
  #define CYTHON_COMPILING_IN_LIMITED_API 0
  #define CYTHON_COMPILING_IN_GRAAL 0
  #define CYTHON_COMPILING_IN_CPYTHON_FREETHREADING 0
  #undef CYTHON_USE_TYPE_SLOTS
  #define CYTHON_USE_TYPE_SLOTS 1
  #ifndef CYTHON_USE_TYPE_SPECS
    #define CYTHON_USE_TYPE_SPECS 0
  #endif
  #undef CYTHON_USE_PYTYPE_LOOKUP
  #define CYTHON_USE_PYTYPE_LOOKUP 0
  #undef CYTHON_USE_PYLIST_INTERNALS
  #define CYTHON_USE_PYLIST_INTERNALS 0
  #undef CYTHON_USE_UNICODE_INTERNALS
  #define CYTHON_USE_UNICODE_INTERNALS 0
  #undef CYTHON_USE_UNICODE_WRITER
  #define CYTHON_USE_UNICODE_WRITER 0
  #undef CYTHON_USE_PYLONG_INTERNALS
  #define CYTHON_USE_PYLONG_INTERNALS 0
  #undef CYTHON_AVOID_BORROWED_REFS
  #define CYTHON_AVOID_BORROWED_REFS 1
  #undef CYTHON_AVOID_THREAD_UNSAFE_BORROWED_REFS
  #define CYTHON_AVOID_THREAD_UNSAFE_BORROWED_REFS 1
  #undef CYTHON_ASSUME_SAFE_MACROS
  #define CYTHON_ASSUME_SAFE_MACROS 0
  #ifndef CYTHON_ASSUME_SAFE_SIZE
    #define CYTHON_ASSUME_SAFE_SIZE 1
  #endif
  #undef CYTHON_UNPACK_METHODS
  #define CYTHON_UNPACK_METHODS 0
  #undef CYTHON_FAST_THREAD_STATE
  #define CYTHON_FAST_THREAD_STATE 0
  #undef CYTHON_FAST_GIL
  #define CYTHON_FAST_GIL 0
  #undef CYTHON_METH_FASTCALL
  #define CYTHON_METH_FASTCALL 0
  #undef CYTHON_FAST_PYCALL
  #define CYTHON_FAST_PYCALL 0
  #ifndef CYTHON_PEP487_INIT_SUBCLASS
    #define CYTHON_PEP487_INIT_SUBCLASS 1
  #endif
  #if PY_VERSION_HEX < 0x03090000
    #undef CYTHON_PEP489_MULTI_PHASE_INIT
    #define CYTHON_PEP489_MULTI_PHASE_INIT 0
  #elif !defined(CYTHON_PEP489_MULTI_PHASE_INIT)
    #define CYTHON_PEP489_MULTI_PHASE_INIT 1
  #endif
  #undef CYTHON_USE_MODULE_STATE
  #define CYTHON_USE_MODULE_STATE 0
  #undef CYTHON_USE_SYS_MONITORING
  #define CYTHON_USE_SYS_MONITORING 0
  #ifndef CYTHON_USE_TP_FINALIZE
    #define CYTHON_USE_TP_FINALIZE (PYPY_VERSION_NUM >= 0x07030C00)
  #endif
  #undef CYTHON_USE_AM_SEND
  #define CYTHON_USE_AM_SEND 0
  #undef CYTHON_USE_DICT_VERSIONS
  #define CYTHON_USE_DICT_VERSIONS 0
  #undef CYTHON_USE_EXC_INFO_STACK
  #define CYTHON_USE_EXC_INFO_STACK 0
  #ifndef CYTHON_UPDATE_DESCRIPTOR_DOC
    #define CYTHON_UPDATE_DESCRIPTOR_DOC (PYPY_VERSION_NUM >= 0x07031100)
  #endif
  #undef CYTHON_USE_FREELISTS
  #define CYTHON_USE_FREELISTS 0
  #undef CYTHON_IMMORTAL_CONSTANTS
  #define CYTHON_IMMORTAL_CONSTANTS 0
#elif defined(CYTHON_LIMITED_API)
  #ifdef Py_LIMITED_API
    #undef __PYX_LIMITED_VERSION_HEX
    #define __PYX_LIMITED_VERSION_HEX Py_LIMITED_API
  #endif
  #define CYTHON_COMPILING_IN_PYPY 0
  #define CYTHON_COMPILING_IN_CPYTHON 0
  #define CYTHON_COMPILING_IN_LIMITED_API 1
  #define CYTHON_COMPILING_IN_GRAAL 0
  #define CYTHON_COMPILING_IN_CPYTHON_FREETHREADING 0
  #undef CYTHON_USE_TYPE_SLOTS
  #define CYTHON_USE_TYPE_SLOTS 0
  #undef CYTHON_USE_TYPE_SPECS
  #define CYTHON_USE_TYPE_SPECS 1
  #undef CYTHON_USE_PYTYPE_LOOKUP
  #define CYTHON_USE_PYTYPE_LOOKUP 0
  #undef CYTHON_USE_PYLIST_INTERNALS
  #define CYTHON_USE_PYLIST_INTERNALS 0
  #undef CYTHON_USE_UNICODE_INTERNALS
  #define CYTHON_USE_UNICODE_INTERNALS 0
  #ifndef CYTHON_USE_UNICODE_WRITER
    #define CYTHON_USE_UNICODE_WRITER 0
  #endif
  #undef CYTHON_USE_PYLONG_INTERNALS
  #define CYTHON_USE_PYLONG_INTERNALS 0
  #ifndef CYTHON_AVOID_BORROWED_REFS
    #define CYTHON_AVOID_BORROWED_REFS 0
  #endif
  #ifndef CYTHON_AVOID_THREAD_UNSAFE_BORROWED_REFS
    #define CYTHON_AVOID_THREAD_UNSAFE_BORROWED_REFS 0
  #endif
  #undef CYTHON_ASSUME_SAFE_MACROS
  #define CYTHON_ASSUME_SAFE_MACROS 0
  #undef CYTHON_ASSUME_SAFE_SIZE
  #define CYTHON_ASSUME_SAFE_SIZE 0
  #undef CYTHON_UNPACK_METHODS
  #define CYTHON_UNPACK_METHODS 0
  #undef CYTHON_FAST_THREAD_STATE
  #define CYTHON_FAST_THREAD_STATE 0
  #undef CYTHON_FAST_GIL
  #define CYTHON_FAST_GIL 0
  #undef CYTHON_METH_FASTCALL
  #define CYTHON_METH_FASTCALL (__PYX_LIMITED_VERSION_HEX >= 0x030C0000)
  #undef CYTHON_FAST_PYCALL
  #define CYTHON_FAST_PYCALL 0
  #ifndef CYTHON_PEP487_INIT_SUBCLASS
    #define CYTHON_PEP487_INIT_SUBCLASS 1
  #endif
  #ifndef CYTHON_PEP489_MULTI_PHASE_INIT
    #define CYTHON_PEP489_MULTI_PHASE_INIT 1
  #endif
  #ifndef CYTHON_USE_MODULE_STATE
    #define CYTHON_USE_MODULE_STATE 0
  #endif
  #undef CYTHON_USE_SYS_MONITORING
  #define CYTHON_USE_SYS_MONITORING 0
  #ifndef CYTHON_USE_TP_FINALIZE
    #define CYTHON_USE_TP_FINALIZE 0
  #endif
  #ifndef CYTHON_USE_AM_SEND
    #define CYTHON_USE_AM_SEND (__PYX_LIMITED_VERSION_HEX >= 0x030A0000)
  #endif
  #undef CYTHON_USE_DICT_VERSIONS
  #define CYTHON_USE_DICT_VERSIONS 0
  #undef CYTHON_USE_EXC_INFO_STACK
  #define CYTHON_USE_EXC_INFO_STACK 0
  #ifndef CYTHON_UPDATE_DESCRIPTOR_DOC
    #define CYTHON_UPDATE_DESCRIPTOR_DOC 0
  #endif
  #ifndef CYTHON_USE_FREELISTS
  #define CYTHON_USE_FREELISTS 1
  #endif
  #undef CYTHON_IMMORTAL_CONSTANTS
  #define CYTHON_IMMORTAL_CONSTANTS 0
#else
  #define CYTHON_COMPILING_IN_PYPY 0
  #define CYTHON_COMPILING_IN_CPYTHON 1
  #define CYTHON_COMPILING_IN_LIMITED_API 0
  #define CYTHON_COMPILING_IN_GRAAL 0
  #ifdef Py_GIL_DISABLED
    #define CYTHON_COMPILING_IN_CPYTHON_FREETHREADING 1
  #else
    #define CYTHON_COMPILING_IN_CPYTHON_FREETHREADING 0
  #endif
  #if PY_VERSION_HEX < 0x030A0000
    #undef CYTHON_USE_TYPE_SLOTS
    #define CYTHON_USE_TYPE_SLOTS 1
  #elif !defined(CYTHON_USE_TYPE_SLOTS)
    #define CYTHON_USE_TYPE_SLOTS 1
  #endif
  #ifndef CYTHON_USE_TYPE_SPECS
    #define CYTHON_USE_TYPE_SPECS 0
  #endif
  #ifndef CYTHON_USE_PYTYPE_LOOKUP
    #define CYTHON_USE_PYTYPE_LOOKUP 1
  #endif
  #ifndef CYTHON_USE_PYLONG_INTERNALS
    #define CYTHON_USE_PYLONG_INTERNALS 1
  #endif
  #if CYTHON_COMPILING_IN_CPYTHON_FREETHREADING
    #undef CYTHON_USE_PYLIST_INTERNALS
    #define CYTHON_USE_PYLIST_INTERNALS 0
  #elif !defined(CYTHON_USE_PYLIST_INTERNALS)
    #define CYTHON_USE_PYLIST_INTERNALS 1
  #endif
  #ifndef CYTHON_USE_UNICODE_INTERNALS
    #define CYTHON_USE_UNICODE_INTERNALS 1
  #endif
  #if CYTHON_COMPILING_IN_CPYTHON_FREETHREADING || PY_VERSION_HEX >= 0x030B00A2
    #undef CYTHON_USE_UNICODE_WRITER
    #define CYTHON_USE_UNICODE_WRITER 0
  #elif !defined(CYTHON_USE_UNICODE_WRITER)
    #define CYTHON_USE_UNICODE_WRITER 1
  #endif
  #ifndef CYTHON_AVOID_BORROWED_REFS
    #define CYTHON_AVOID_BORROWED_REFS 0
  #endif
  #if CYTHON_COMPILING_IN_CPYTHON_FREETHREADING
    #undef CYTHON_AVOID_THREAD_UNSAFE_BORROWED_REFS
    #define CYTHON_AVOID_THREAD_UNSAFE_BORROWED_REFS 1
  #elif !defined(CYTHON_AVOID_THREAD_UNSAFE_BORROWED_REFS)
    #define CYTHON_AVOID_THREAD_UNSAFE_BORROWED_REFS 0
  #endif
  #ifndef CYTHON_ASSUME_SAFE_MACROS
    #define CYTHON_ASSUME_SAFE_MACROS 1
  #endif
  #ifndef CYTHON_ASSUME_SAFE_SIZE
    #define CYTHON_ASSUME_SAFE_SIZE 1
  #endif
  #ifndef CYTHON_UNPACK_METHODS
    #define CYTHON_UNPACK_METHODS 1
  #endif
  #ifndef CYTHON_FAST_THREAD_STATE
    #define CYTHON_FAST_THREAD_STATE 1
  #endif
  #if CYTHON_COMPILING_IN_CPYTHON_FREETHREADING
    #undef CYTHON_FAST_GIL
    #define CYTHON_FAST_GIL 0
  #elif !defined(CYTHON_FAST_GIL)
    #define CYTHON_FAST_GIL (PY_VERSION_HEX < 0x030C00A6)
  #endif
  #ifndef CYTHON_METH_FASTCALL
    #define CYTHON_METH_FASTCALL 1
  #endif
  #ifndef CYTHON_FAST_PYCALL
    #define CYTHON_FAST_PYCALL 1
  #endif
  #ifndef CYTHON_PEP487_INIT_SUBCLASS
    #define CYTHON_PEP487_INIT_SUBCLASS 1
  #endif
  #ifndef CYTHON_PEP489_MULTI_PHASE_INIT
    #define CYTHON_PEP489_MULTI_PHASE_INIT 1
  #endif
  #ifndef CYTHON_USE_MODULE_STATE
    #define CYTHON_USE_MODULE_STATE 0
  #endif
  #ifndef CYTHON_USE_SYS_MONITORING
    #define CYTHON_USE_SYS_MONITORING (PY_VERSION_HEX >= 0x030d00B1)
  #endif
  #ifndef CYTHON_USE_TP_FINALIZE
    #define CYTHON_USE_TP_FINALIZE 1
  #endif
  #ifndef CYTHON_USE_AM_SEND
    #define CYTHON_USE_AM_SEND 1
  #endif
  #if CYTHON_COMPILING_IN_CPYTHON_FREETHREADING
    #undef CYTHON_USE_DICT_VERSIONS
    #define CYTHON_USE_DICT_VERSIONS 0
  #elif !defined(CYTHON_USE_DICT_VERSIONS)
    #define CYTHON_USE_DICT_VERSIONS  (PY_VERSION_HEX < 0x030C00A5 && !CYTHON_USE_MODULE_STATE)
  #endif
  #ifndef CYTHON_USE_EXC_INFO_STACK
    #define CYTHON_USE_EXC_INFO_STACK 1
  #endif
  #ifndef CYTHON_UPDATE_DESCRIPTOR_DOC
    #define CYTHON_UPDATE_DESCRIPTOR_DOC 1
  #endif
  #ifndef CYTHON_USE_FREELISTS
    #define CYTHON_USE_FREELISTS (!CYTHON_COMPILING_IN_CPYTHON_FREETHREADING)
  #endif
  #if defined(CYTHON_IMMORTAL_CONSTANTS) && PY_VERSION_HEX < 0x030C0000
    #undef CYTHON_IMMORTAL_CONSTANTS
    #define CYTHON_IMMORTAL_CONSTANTS 0  // definitely won't work
  #elif !defined(CYTHON_IMMORTAL_CONSTANTS)
    #define CYTHON_IMMORTAL_CONSTANTS (PY_VERSION_HEX >= 0x030C0000 && !CYTHON_USE_MODULE_STATE && CYTHON_COMPILING_IN_CPYTHON_FREETHREADING)
  #endif
#endif
#ifndef CYTHON_COMPRESS_STRINGS
  #define CYTHON_COMPRESS_STRINGS 1
#endif
#ifndef CYTHON_FAST_PYCCALL
#define CYTHON_FAST_PYCCALL  CYTHON_FAST_PYCALL
#endif
#ifndef CYTHON_VECTORCALL
#if CYTHON_COMPILING_IN_LIMITED_API
#define CYTHON_VECTORCALL  (__PYX_LIMITED_VERSION_HEX >= 0x030C0000)
#else
#define CYTHON_VECTORCALL  (CYTHON_FAST_PYCCALL)
#endif
#endif
#if CYTHON_USE_PYLONG_INTERNALS
  #undef SHIFT
  #undef BASE
  #undef MASK
  #ifdef SIZEOF_VOID_P
    enum { __pyx_check_sizeof_voidp = 1 / (int)(SIZEOF_VOID_P == sizeof(void*)) };
  #endif
#endif
#ifndef __has_attribute
  #define __has_attribute(x) 0
#endif
#ifndef __has_cpp_attribute
  #define __has_cpp_attribute(x) 0
#endif
#ifndef CYTHON_RESTRICT
  #if defined(__GNUC__)
    #define CYTHON_RESTRICT __restrict__
  #elif defined(_MSC_VER) && _MSC_VER >= 1400
    #define CYTHON_RESTRICT __restrict
  #elif defined (__STDC_VERSION__) && __STDC_VERSION__ >= 199901L
    #define CYTHON_RESTRICT restrict
  #else
    #define CYTHON_RESTRICT
  #endif
#endif
#ifndef CYTHON_UNUSED
  #if defined(__cplusplus)
    /* for clang __has_cpp_attribute(maybe_unused) is true even before C++17
     * but leads to warnings with -pedantic, since it is a C++17 feature */
    #if ((defined(_MSVC_LANG) && _MSVC_LANG >= 201703L) || __cplusplus >= 201703L)
      #if __has_cpp_attribute(maybe_unused)
        #define CYTHON_UNUSED [[maybe_unused]]
      #endif
    #endif
  #endif
#endif
#ifndef CYTHON_UNUSED
# if defined(__GNUC__)
#   if !(defined(__cplusplus)) || (__GNUC__ > 3 || (__GNUC__ == 3 && __GNUC_MINOR__ >= 4))
#     define CYTHON_UNUSED __attribute__ ((__unused__))
#   else
#     define CYTHON_UNUSED
#   endif
# elif defined(__ICC) || (defined(__INTEL_COMPILER) && !defined(_MSC_VER))
#   define CYTHON_UNUSED __attribute__ ((__unused__))
# else
#   define CYTHON_UNUSED
# endif
#endif
#ifndef CYTHON_UNUSED_VAR
#  if defined(__cplusplus)
     template<class T> void CYTHON_UNUSED_VAR( const T& ) { }
#  else
#    define CYTHON_UNUSED_VAR(x) (void)(x)
#  endif
#endif
#ifndef CYTHON_MAYBE_UNUSED_VAR
  #define CYTHON_MAYBE_UNUSED_VAR(x) CYTHON_UNUSED_VAR(x)
#endif
#ifndef CYTHON_NCP_UNUSED
# if CYTHON_COMPILING_IN_CPYTHON && !CYTHON_COMPILING_IN_CPYTHON_FREETHREADING
#  define CYTHON_NCP_UNUSED
# else
#  define CYTHON_NCP_UNUSED CYTHON_UNUSED
# endif
#endif
#ifndef CYTHON_USE_CPP_STD_MOVE
  #if defined(__cplusplus) && (\
    __cplusplus >= 201103L || (defined(_MSC_VER) && _MSC_VER >= 1600))
    #define CYTHON_USE_CPP_STD_MOVE 1
  #else
    #define CYTHON_USE_CPP_STD_MOVE 0
  #endif
#endif
#define __Pyx_void_to_None(void_result) ((void)(void_result), Py_INCREF(Py_None), Py_None)
#include <stdint.h>
typedef uintptr_t  __pyx_uintptr_t;
#ifndef CYTHON_FALLTHROUGH
  #if defined(__cplusplus)
    /* for clang __has_cpp_attribute(fallthrough) is true even before C++17
     * but leads to warnings with -pedantic, since it is a C++17 feature */
    #if ((defined(_MSVC_LANG) && _MSVC_LANG >= 201703L) || __cplusplus >= 201703L)
      #if __has_cpp_attribute(fallthrough)
        #define CYTHON_FALLTHROUGH [[fallthrough]]
      #endif
    #endif
    #ifndef CYTHON_FALLTHROUGH
      #if __has_cpp_attribute(clang::fallthrough)
        #define CYTHON_FALLTHROUGH [[clang::fallthrough]]
      #elif __has_cpp_attribute(gnu::fallthrough)
        #define CYTHON_FALLTHROUGH [[gnu::fallthrough]]
      #endif
    #endif
  #endif
  #ifndef CYTHON_FALLTHROUGH
    #if __has_attribute(fallthrough)
      #define CYTHON_FALLTHROUGH __attribute__((fallthrough))
    #else
      #define CYTHON_FALLTHROUGH
    #endif
  #endif
  #if defined(__clang__) && defined(__apple_build_version__)
    #if __apple_build_version__ < 7000000
      #undef  CYTHON_FALLTHROUGH
      #define CYTHON_FALLTHROUGH
    #endif
  #endif
#endif
#ifndef Py_UNREACHABLE
  #define Py_UNREACHABLE()  assert(0); abort()
#endif
#ifdef __cplusplus
  template <typename T>
  struct __PYX_IS_UNSIGNED_IMPL {static const bool value = T(0) < T(-1);};
  #define __PYX_IS_UNSIGNED(type) (__PYX_IS_UNSIGNED_IMPL<type>::value)
#else
  #define __PYX_IS_UNSIGNED(type) (((type)-1) > 0)
#endif
#if CYTHON_COMPILING_IN_PYPY == 1
  #define __PYX_NEED_TP_PRINT_SLOT  (PY_VERSION_HEX < 0x030A0000)
#else
  #define __PYX_NEED_TP_PRINT_SLOT  (PY_VERSION_HEX < 0x03090000)
#endif
#define __PYX_REINTERPRET_FUNCION(func_pointer, other_pointer) ((func_pointer)(void(*)(void))(other_pointer))

/* CppInitCode */
#ifndef __cplusplus
  #error "Cython files generated with the C++ option must be compiled with a C++ compiler."
#endif
#ifndef CYTHON_INLINE
  #if defined(__clang__)
    #define CYTHON_INLINE __inline__ __attribute__ ((__unused__))
  #else
    #define CYTHON_INLINE inline
  #endif
#endif
template<typename T>
void __Pyx_call_destructor(T& x) {
    x.~T();
}
template<typename T>
class __Pyx_FakeReference {
  public:
    __Pyx_FakeReference() : ptr(NULL) { }
    __Pyx_FakeReference(const T& ref) : ptr(const_cast<T*>(&ref)) { }
    T *operator->() { return ptr; }
    T *operator&() { return ptr; }
    operator T&() { return *ptr; }
    template<typename U> bool operator ==(const U& other) const { return *ptr == other; }
    template<typename U> bool operator !=(const U& other) const { return *ptr != other; }
    template<typename U> bool operator==(const __Pyx_FakeReference<U>& other) const { return *ptr == *other.ptr; }
    template<typename U> bool operator!=(const __Pyx_FakeReference<U>& other) const { return *ptr != *other.ptr; }
  private:
    T *ptr;
};

/* PythonCompatibility */
#define __PYX_BUILD_PY_SSIZE_T "n"
#define CYTHON_FORMAT_SSIZE_T "z"
#define __Pyx_BUILTIN_MODULE_NAME "builtins"
#define __Pyx_DefaultClassType PyType_Type
#if CYTHON_COMPILING_IN_LIMITED_API
    #ifndef CO_OPTIMIZED
    static int CO_OPTIMIZED;
    #endif
    #ifndef CO_NEWLOCALS
    static int CO_NEWLOCALS;
    #endif
    #ifndef CO_VARARGS
    static int CO_VARARGS;
    #endif
    #ifndef CO_VARKEYWORDS
    static int CO_VARKEYWORDS;
    #endif
    #ifndef CO_ASYNC_GENERATOR
    static int CO_ASYNC_GENERATOR;
    #endif
    #ifndef CO_GENERATOR
    static int CO_GENERATOR;
    #endif
    #ifndef CO_COROUTINE
    static int CO_COROUTINE;
    #endif
#else
    #ifndef CO_COROUTINE
      #define CO_COROUTINE 0x80
    #endif
    #ifndef CO_ASYNC_GENERATOR
      #define CO_ASYNC_GENERATOR 0x200
    #endif
#endif
static int __Pyx_init_co_variables(void);
#if PY_VERSION_HEX >= 0x030900A4 || defined(Py_IS_TYPE)
  #define __Pyx_IS_TYPE(ob, type) Py_IS_TYPE(ob, type)
#else
  #define __Pyx_IS_TYPE(ob, type) (((const PyObject*)ob)->ob_type == (type))
#endif
#if PY_VERSION_HEX >= 0x030A00B1 || defined(Py_Is)
  #define __Pyx_Py_Is(x, y)  Py_Is(x, y)
#else
  #define __Pyx_Py_Is(x, y) ((x) == (y))
#endif
#if PY_VERSION_HEX >= 0x030A00B1 || defined(Py_IsNone)
  #define __Pyx_Py_IsNone(ob) Py_IsNone(ob)
#else
  #define __Pyx_Py_IsNone(ob) __Pyx_Py_Is((ob), Py_None)
#endif
#if PY_VERSION_HEX >= 0x030A00B1 || defined(Py_IsTrue)
  #define __Pyx_Py_IsTrue(ob) Py_IsTrue(ob)
#else
  #define __Pyx_Py_IsTrue(ob) __Pyx_Py_Is((ob), Py_True)
#endif
#if PY_VERSION_HEX >= 0x030A00B1 || defined(Py_IsFalse)
  #define __Pyx_Py_IsFalse(ob) Py_IsFalse(ob)
#else
  #define __Pyx_Py_IsFalse(ob) __Pyx_Py_Is((ob), Py_False)
#endif
#define __Pyx_NoneAsNull(obj)  (__Pyx_Py_IsNone(obj) ? NULL : (obj))
#if PY_VERSION_HEX >= 0x030900F0 && !CYTHON_COMPILING_IN_PYPY
  #define __Pyx_PyObject_GC_IsFinalized(o) PyObject_GC_IsFinalized(o)
#else
  #define __Pyx_PyObject_GC_IsFinalized(o) _PyGC_FINALIZED(o)
#endif
#ifndef Py_TPFLAGS_CHECKTYPES
  #define Py_TPFLAGS_CHECKTYPES 0
#endif
#ifndef Py_TPFLAGS_HAVE_INDEX
  #define Py_TPFLAGS_HAVE_INDEX 0
#endif
#ifndef Py_TPFLAGS_HAVE_NEWBUFFER
  #define Py_TPFLAGS_HAVE_NEWBUFFER 0
#endif
#ifndef Py_TPFLAGS_HAVE_FINALIZE
  #define Py_TPFLAGS_HAVE_FINALIZE 0
#endif
#ifndef Py_TPFLAGS_SEQUENCE
  #define Py_TPFLAGS_SEQUENCE 0
#endif
#ifndef Py_TPFLAGS_MAPPING
  #define Py_TPFLAGS_MAPPING 0
#endif
#ifndef Py_TPFLAGS_IMMUTABLETYPE
  #define Py_TPFLAGS_IMMUTABLETYPE (1UL << 8)
#endif
#ifndef Py_TPFLAGS_DISALLOW_INSTANTIATION
  #define Py_TPFLAGS_DISALLOW_INSTANTIATION (1UL << 7)
#endif
#ifndef METH_STACKLESS
  #define METH_STACKLESS 0
#endif
#ifndef METH_FASTCALL
  #ifndef METH_FASTCALL
     #define METH_FASTCALL 0x80
  #endif
  typedef PyObject *(*__Pyx_PyCFunctionFast) (PyObject *self, PyObject *const *args, Py_ssize_t nargs);
  typedef PyObject *(*__Pyx_PyCFunctionFastWithKeywords) (PyObject *self, PyObject *const *args,
                                                          Py_ssize_t nargs, PyObject *kwnames);
#else
  #if PY_VERSION_HEX >= 0x030d00A4
  #  define __Pyx_PyCFunctionFast PyCFunctionFast
  #  define __Pyx_PyCFunctionFastWithKeywords PyCFunctionFastWithKeywords
  #else
  #  define __Pyx_PyCFunctionFast _PyCFunctionFast
  #  define __Pyx_PyCFunctionFastWithKeywords _PyCFunctionFastWithKeywords
  #endif
#endif
#if CYTHON_METH_FASTCALL
  #define __Pyx_METH_FASTCALL METH_FASTCALL
  #define __Pyx_PyCFunction_FastCall __Pyx_PyCFunctionFast
  #define __Pyx_PyCFunction_FastCallWithKeywords __Pyx_PyCFunctionFastWithKeywords
#else
  #define __Pyx_METH_FASTCALL METH_VARARGS
  #define __Pyx_PyCFunction_FastCall PyCFunction
  #define __Pyx_PyCFunction_FastCallWithKeywords PyCFunctionWithKeywords
#endif
#if CYTHON_VECTORCALL
  #define __pyx_vectorcallfunc vectorcallfunc
  #define __Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET  PY_VECTORCALL_ARGUMENTS_OFFSET
  #define __Pyx_PyVectorcall_NARGS(n)  PyVectorcall_NARGS((size_t)(n))
#else
  #define __Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET  0
  #define __Pyx_PyVectorcall_NARGS(n)  ((Py_ssize_t)(n))
#endif
#if PY_VERSION_HEX >= 0x030900B1
#define __Pyx_PyCFunction_CheckExact(func)  PyCFunction_CheckExact(func)
#else
#define __Pyx_PyCFunction_CheckExact(func)  PyCFunction_Check(func)
#endif
#define __Pyx_CyOrPyCFunction_Check(func)  PyCFunction_Check(func)
#if CYTHON_COMPILING_IN_CPYTHON
#define __Pyx_CyOrPyCFunction_GET_FUNCTION(func)  (((PyCFunctionObject*)(func))->m_ml->ml_meth)
#elif !CYTHON_COMPILING_IN_LIMITED_API
#define __Pyx_CyOrPyCFunction_GET_FUNCTION(func)  PyCFunction_GET_FUNCTION(func)
#endif
#if CYTHON_COMPILING_IN_CPYTHON
#define __Pyx_CyOrPyCFunction_GET_FLAGS(func)  (((PyCFunctionObject*)(func))->m_ml->ml_flags)
static CYTHON_INLINE PyObject* __Pyx_CyOrPyCFunction_GET_SELF(PyObject *func) {
    return (__Pyx_CyOrPyCFunction_GET_FLAGS(func) & METH_STATIC) ? NULL : ((PyCFunctionObject*)func)->m_self;
}
#endif
static CYTHON_INLINE int __Pyx__IsSameCFunction(PyObject *func, void (*cfunc)(void)) {
#if CYTHON_COMPILING_IN_LIMITED_API
    return PyCFunction_Check(func) && PyCFunction_GetFunction(func) == (PyCFunction) cfunc;
#else
    return PyCFunction_Check(func) && PyCFunction_GET_FUNCTION(func) == (PyCFunction) cfunc;
#endif
}
#define __Pyx_IsSameCFunction(func, cfunc)   __Pyx__IsSameCFunction(func, cfunc)
#if PY_VERSION_HEX < 0x03090000 || (CYTHON_COMPILING_IN_LIMITED_API && __PYX_LIMITED_VERSION_HEX < 0x030A0000)
  #define __Pyx_PyType_FromModuleAndSpec(m, s, b)  ((void)m, PyType_FromSpecWithBases(s, b))
  typedef PyObject *(*__Pyx_PyCMethod)(PyObject *, PyTypeObject *, PyObject *const *, size_t, PyObject *);
#else
  #define __Pyx_PyType_FromModuleAndSpec(m, s, b)  PyType_FromModuleAndSpec(m, s, b)
  #define __Pyx_PyCMethod  PyCMethod
#endif
#ifndef METH_METHOD
  #define METH_METHOD 0x200
#endif
#if CYTHON_COMPILING_IN_PYPY && !defined(PyObject_Malloc)
  #define PyObject_Malloc(s)   PyMem_Malloc(s)
  #define PyObject_Free(p)     PyMem_Free(p)
  #define PyObject_Realloc(p)  PyMem_Realloc(p)
#endif
#if CYTHON_COMPILING_IN_LIMITED_API
  #define __Pyx_PyFrame_SetLineNumber(frame, lineno)
#elif CYTHON_COMPILING_IN_GRAAL && defined(GRAALPY_VERSION_NUM) && GRAALPY_VERSION_NUM > 0x19000000
  #define __Pyx_PyCode_HasFreeVars(co)  (PyCode_GetNumFree(co) > 0)
  #define __Pyx_PyFrame_SetLineNumber(frame, lineno) GraalPyFrame_SetLineNumber((frame), (lineno))
#elif CYTHON_COMPILING_IN_GRAAL
  #define __Pyx_PyCode_HasFreeVars(co)  (PyCode_GetNumFree(co) > 0)
  #define __Pyx_PyFrame_SetLineNumber(frame, lineno) _PyFrame_SetLineNumber((frame), (lineno))
#else
  #define __Pyx_PyCode_HasFreeVars(co)  (PyCode_GetNumFree(co) > 0)
  #define __Pyx_PyFrame_SetLineNumber(frame, lineno)  (frame)->f_lineno = (lineno)
#endif
#if CYTHON_COMPILING_IN_LIMITED_API
  #define __Pyx_PyThreadState_Current PyThreadState_Get()
#elif !CYTHON_FAST_THREAD_STATE
  #define __Pyx_PyThreadState_Current PyThreadState_GET()
#elif PY_VERSION_HEX >= 0x030d00A1
  #define __Pyx_PyThreadState_Current PyThreadState_GetUnchecked()
#else
  #define __Pyx_PyThreadState_Current _PyThreadState_UncheckedGet()
#endif
#if CYTHON_USE_MODULE_STATE
static CYTHON_INLINE void *__Pyx__PyModule_GetState(PyObject *op)
{
    void *result;
    result = PyModule_GetState(op);
    if (!result)
        Py_FatalError("Couldn't find the module state");
    return result;
}
#define __Pyx_PyModule_GetState(o) (__pyx_mstatetype *)__Pyx__PyModule_GetState(o)
#else
#define __Pyx_PyModule_GetState(op) ((void)op,__pyx_mstate_global)
#endif
#define __Pyx_PyObject_GetSlot(obj, name, func_ctype)  __Pyx_PyType_GetSlot(Py_TYPE((PyObject *) obj), name, func_ctype)
#define __Pyx_PyObject_TryGetSlot(obj, name, func_ctype) __Pyx_PyType_TryGetSlot(Py_TYPE(obj), name, func_ctype)
#define __Pyx_PyObject_GetSubSlot(obj, sub, name, func_ctype) __Pyx_PyType_GetSubSlot(Py_TYPE(obj), sub, name, func_ctype)
#define __Pyx_PyObject_TryGetSubSlot(obj, sub, name, func_ctype) __Pyx_PyType_TryGetSubSlot(Py_TYPE(obj), sub, name, func_ctype)
#if CYTHON_USE_TYPE_SLOTS
  #define __Pyx_PyType_GetSlot(type, name, func_ctype)  ((type)->name)
  #define __Pyx_PyType_TryGetSlot(type, name, func_ctype) __Pyx_PyType_GetSlot(type, name, func_ctype)
  #define __Pyx_PyType_GetSubSlot(type, sub, name, func_ctype) (((type)->sub) ? ((type)->sub->name) : NULL)
  #define __Pyx_PyType_TryGetSubSlot(type, sub, name, func_ctype) __Pyx_PyType_GetSubSlot(type, sub, name, func_ctype)
#else
  #define __Pyx_PyType_GetSlot(type, name, func_ctype)  ((func_ctype) PyType_GetSlot((type), Py_##name))
  #define __Pyx_PyType_TryGetSlot(type, name, func_ctype)\
    ((__PYX_LIMITED_VERSION_HEX >= 0x030A0000 ||\
     (PyType_GetFlags(type) & Py_TPFLAGS_HEAPTYPE) || __Pyx_get_runtime_version() >= 0x030A0000) ?\
     __Pyx_PyType_GetSlot(type, name, func_ctype) : NULL)
  #define __Pyx_PyType_GetSubSlot(obj, sub, name, func_ctype) __Pyx_PyType_GetSlot(obj, name, func_ctype)
  #define __Pyx_PyType_TryGetSubSlot(obj, sub, name, func_ctype) __Pyx_PyType_TryGetSlot(obj, name, func_ctype)
#endif
#if CYTHON_COMPILING_IN_CPYTHON || defined(_PyDict_NewPresized)
#define __Pyx_PyDict_NewPresized(n)  ((n <= 8) ? PyDict_New() : _PyDict_NewPresized(n))
#else
#define __Pyx_PyDict_NewPresized(n)  PyDict_New()
#endif
#define __Pyx_PyNumber_Divide(x,y)         PyNumber_TrueDivide(x,y)
#define __Pyx_PyNumber_InPlaceDivide(x,y)  PyNumber_InPlaceTrueDivide(x,y)
#if CYTHON_COMPILING_IN_CPYTHON && CYTHON_USE_UNICODE_INTERNALS
#define __Pyx_PyDict_GetItemStrWithError(dict, name)  _PyDict_GetItem_KnownHash(dict, name, ((PyASCIIObject *) name)->hash)
static CYTHON_INLINE PyObject * __Pyx_PyDict_GetItemStr(PyObject *dict, PyObject *name) {
    PyObject *res = __Pyx_PyDict_GetItemStrWithError(dict, name);
    if (res == NULL) PyErr_Clear();
    return res;
}
#elif !CYTHON_COMPILING_IN_PYPY || PYPY_VERSION_NUM >= 0x07020000
#define __Pyx_PyDict_GetItemStrWithError  PyDict_GetItemWithError
#define __Pyx_PyDict_GetItemStr           PyDict_GetItem
#else
static CYTHON_INLINE PyObject * __Pyx_PyDict_GetItemStrWithError(PyObject *dict, PyObject *name) {
#if CYTHON_COMPILING_IN_PYPY
    return PyDict_GetItem(dict, name);
#else
    PyDictEntry *ep;
    PyDictObject *mp = (PyDictObject*) dict;
    long hash = ((PyStringObject *) name)->ob_shash;
    assert(hash != -1);
    ep = (mp->ma_lookup)(mp, name, hash);
    if (ep == NULL) {
        return NULL;
    }
    return ep->me_value;
#endif
}
#define __Pyx_PyDict_GetItemStr           PyDict_GetItem
#endif
#if CYTHON_USE_TYPE_SLOTS
  #define __Pyx_PyType_GetFlags(tp)   (((PyTypeObject *)tp)->tp_flags)
  #define __Pyx_PyType_HasFeature(type, feature)  ((__Pyx_PyType_GetFlags(type) & (feature)) != 0)
#else
  #define __Pyx_PyType_GetFlags(tp)   (PyType_GetFlags((PyTypeObject *)tp))
  #define __Pyx_PyType_HasFeature(type, feature)  PyType_HasFeature(type, feature)
#endif
#define __Pyx_PyObject_GetIterNextFunc(iterator)  __Pyx_PyObject_GetSlot(iterator, tp_iternext, iternextfunc)
#if CYTHON_USE_TYPE_SPECS
#define __Pyx_PyHeapTypeObject_GC_Del(obj)  {\
    PyTypeObject *type = Py_TYPE((PyObject*)obj);\
    assert(__Pyx_PyType_HasFeature(type, Py_TPFLAGS_HEAPTYPE));\
    PyObject_GC_Del(obj);\
    Py_DECREF(type);\
}
#else
#define __Pyx_PyHeapTypeObject_GC_Del(obj)  PyObject_GC_Del(obj)
#endif
#if CYTHON_COMPILING_IN_LIMITED_API
  #define __Pyx_PyUnicode_READY(op)       (0)
  #define __Pyx_PyUnicode_READ_CHAR(u, i) PyUnicode_ReadChar(u, i)
  #define __Pyx_PyUnicode_MAX_CHAR_VALUE(u)   ((void)u, 1114111U)
  #define __Pyx_PyUnicode_KIND(u)         ((void)u, (0))
  #define __Pyx_PyUnicode_DATA(u)         ((void*)u)
  #define __Pyx_PyUnicode_READ(k, d, i)   ((void)k, PyUnicode_ReadChar((PyObject*)(d), i))
  #define __Pyx_PyUnicode_IS_TRUE(u)      (0 != PyUnicode_GetLength(u))
#else
  #if PY_VERSION_HEX >= 0x030C0000
    #define __Pyx_PyUnicode_READY(op)       (0)
  #else
    #define __Pyx_PyUnicode_READY(op)       (likely(PyUnicode_IS_READY(op)) ?\
                                                0 : _PyUnicode_Ready((PyObject *)(op)))
  #endif
  #define __Pyx_PyUnicode_READ_CHAR(u, i) PyUnicode_READ_CHAR(u, i)
  #define __Pyx_PyUnicode_MAX_CHAR_VALUE(u)   PyUnicode_MAX_CHAR_VALUE(u)
  #define __Pyx_PyUnicode_KIND(u)         ((int)PyUnicode_KIND(u))
  #define __Pyx_PyUnicode_DATA(u)         PyUnicode_DATA(u)
  #define __Pyx_PyUnicode_READ(k, d, i)   PyUnicode_READ(k, d, i)
  #define __Pyx_PyUnicode_WRITE(k, d, i, ch)  PyUnicode_WRITE(k, d, i, (Py_UCS4) ch)
  #if PY_VERSION_HEX >= 0x030C0000
    #define __Pyx_PyUnicode_IS_TRUE(u)      (0 != PyUnicode_GET_LENGTH(u))
  #else
    #if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x03090000
    #define __Pyx_PyUnicode_IS_TRUE(u)      (0 != (likely(PyUnicode_IS_READY(u)) ? PyUnicode_GET_LENGTH(u) : ((PyCompactUnicodeObject *)(u))->wstr_length))
    #else
    #define __Pyx_PyUnicode_IS_TRUE(u)      (0 != (likely(PyUnicode_IS_READY(u)) ? PyUnicode_GET_LENGTH(u) : PyUnicode_GET_SIZE(u)))
    #endif
  #endif
#endif
#if CYTHON_COMPILING_IN_PYPY
  #define __Pyx_PyUnicode_Concat(a, b)      PyNumber_Add(a, b)
  #define __Pyx_PyUnicode_ConcatSafe(a, b)  PyNumber_Add(a, b)
#else
  #define __Pyx_PyUnicode_Concat(a, b)      PyUnicode_Concat(a, b)
  #define __Pyx_PyUnicode_ConcatSafe(a, b)  ((unlikely((a) == Py_None) || unlikely((b) == Py_None)) ?\
      PyNumber_Add(a, b) : __Pyx_PyUnicode_Concat(a, b))
#endif
#if CYTHON_COMPILING_IN_PYPY
  #if !defined(PyUnicode_DecodeUnicodeEscape)
    #define PyUnicode_DecodeUnicodeEscape(s, size, errors)  PyUnicode_Decode(s, size, "unicode_escape", errors)
  #endif
  #if !defined(PyUnicode_Contains)
    #define PyUnicode_Contains(u, s)  PySequence_Contains(u, s)
  #endif
  #if !defined(PyByteArray_Check)
    #define PyByteArray_Check(obj)  PyObject_TypeCheck(obj, &PyByteArray_Type)
  #endif
  #if !defined(PyObject_Format)
    #define PyObject_Format(obj, fmt)  PyObject_CallMethod(obj, "__format__", "O", fmt)
  #endif
#endif
#define __Pyx_PyUnicode_FormatSafe(a, b)  ((unlikely((a) == Py_None || (PyUnicode_Check(b) && !PyUnicode_CheckExact(b)))) ? PyNumber_Remainder(a, b) : PyUnicode_Format(a, b))
#if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x030E0000
  #define __Pyx_PySequence_ListKeepNew(obj)\
    (likely(PyList_CheckExact(obj) && PyUnstable_Object_IsUniquelyReferenced(obj)) ? __Pyx_NewRef(obj) : PySequence_List(obj))
#elif CYTHON_COMPILING_IN_CPYTHON
  #define __Pyx_PySequence_ListKeepNew(obj)\
    (likely(PyList_CheckExact(obj) && Py_REFCNT(obj) == 1) ? __Pyx_NewRef(obj) : PySequence_List(obj))
#else
  #define __Pyx_PySequence_ListKeepNew(obj)  PySequence_List(obj)
#endif
#ifndef PySet_CheckExact
  #define PySet_CheckExact(obj)        __Pyx_IS_TYPE(obj, &PySet_Type)
#endif
#if PY_VERSION_HEX >= 0x030900A4
  #define __Pyx_SET_REFCNT(obj, refcnt) Py_SET_REFCNT(obj, refcnt)
  #define __Pyx_SET_SIZE(obj, size) Py_SET_SIZE(obj, size)
#else
  #define __Pyx_SET_REFCNT(obj, refcnt) Py_REFCNT(obj) = (refcnt)
  #define __Pyx_SET_SIZE(obj, size) Py_SIZE(obj) = (size)
#endif
enum __Pyx_ReferenceSharing {
  __Pyx_ReferenceSharing_DefinitelyUnique, // We created it so we know it's unshared - no need to check
  __Pyx_ReferenceSharing_OwnStrongReference,
  __Pyx_ReferenceSharing_FunctionArgument,
  __Pyx_ReferenceSharing_SharedReference, // Never trust it to be unshared because it's a global or similar
};
#if CYTHON_COMPILING_IN_CPYTHON_FREETHREADING && PY_VERSION_HEX >= 0x030E0000
#define __Pyx_IS_UNIQUELY_REFERENCED(o, sharing)\
    (sharing == __Pyx_ReferenceSharing_DefinitelyUnique ? 1 :\
      (sharing == __Pyx_ReferenceSharing_FunctionArgument ? PyUnstable_Object_IsUniqueReferencedTemporary(o) :\
      (sharing == __Pyx_ReferenceSharing_OwnStrongReference ? PyUnstable_Object_IsUniquelyReferenced(o) : 0)))
#elif (CYTHON_COMPILING_IN_CPYTHON && !CYTHON_COMPILING_IN_CPYTHON_FREETHREADING) || CYTHON_COMPILING_IN_LIMITED_API
#define __Pyx_IS_UNIQUELY_REFERENCED(o, sharing) (((void)sharing), Py_REFCNT(o) == 1)
#else
#define __Pyx_IS_UNIQUELY_REFERENCED(o, sharing) (((void)o), ((void)sharing), 0)
#endif
#if CYTHON_AVOID_BORROWED_REFS || CYTHON_AVOID_THREAD_UNSAFE_BORROWED_REFS
  #if __PYX_LIMITED_VERSION_HEX >= 0x030d0000
    #define __Pyx_PyList_GetItemRef(o, i) PyList_GetItemRef(o, i)
  #elif CYTHON_COMPILING_IN_LIMITED_API || !CYTHON_ASSUME_SAFE_MACROS
    #define __Pyx_PyList_GetItemRef(o, i) (likely((i) >= 0) ? PySequence_GetItem(o, i) : (PyErr_SetString(PyExc_IndexError, "list index out of range"), (PyObject*)NULL))
  #else
    #define __Pyx_PyList_GetItemRef(o, i) PySequence_ITEM(o, i)
  #endif
#elif CYTHON_COMPILING_IN_LIMITED_API || !CYTHON_ASSUME_SAFE_MACROS
  #if __PYX_LIMITED_VERSION_HEX >= 0x030d0000
    #define __Pyx_PyList_GetItemRef(o, i) PyList_GetItemRef(o, i)
  #else
    #define __Pyx_PyList_GetItemRef(o, i) __Pyx_XNewRef(PyList_GetItem(o, i))
  #endif
#else
  #define __Pyx_PyList_GetItemRef(o, i) __Pyx_NewRef(PyList_GET_ITEM(o, i))
#endif
#if CYTHON_AVOID_THREAD_UNSAFE_BORROWED_REFS && !CYTHON_COMPILING_IN_LIMITED_API && CYTHON_ASSUME_SAFE_MACROS
  #define __Pyx_PyList_GetItemRefFast(o, i, unsafe_shared) (__Pyx_IS_UNIQUELY_REFERENCED(o, unsafe_shared) ?\
    __Pyx_NewRef(PyList_GET_ITEM(o, i)) : __Pyx_PyList_GetItemRef(o, i))
#else
  #define __Pyx_PyList_GetItemRefFast(o, i, unsafe_shared) __Pyx_PyList_GetItemRef(o, i)
#endif
#if __PYX_LIMITED_VERSION_HEX >= 0x030d0000
#define __Pyx_PyDict_GetItemRef(dict, key, result) PyDict_GetItemRef(dict, key, result)
#elif CYTHON_AVOID_BORROWED_REFS || CYTHON_AVOID_THREAD_UNSAFE_BORROWED_REFS
static CYTHON_INLINE int __Pyx_PyDict_GetItemRef(PyObject *dict, PyObject *key, PyObject **result) {
  *result = PyObject_GetItem(dict, key);
  if (*result == NULL) {
    if (PyErr_ExceptionMatches(PyExc_KeyError)) {
      PyErr_Clear();
      return 0;
    }
    return -1;
  }
  return 1;
}
#else
static CYTHON_INLINE int __Pyx_PyDict_GetItemRef(PyObject *dict, PyObject *key, PyObject **result) {
  *result = PyDict_GetItemWithError(dict, key);
  if (*result == NULL) {
    return PyErr_Occurred() ? -1 : 0;
  }
  Py_INCREF(*result);
  return 1;
}
#endif
#if defined(CYTHON_DEBUG_VISIT_CONST) && CYTHON_DEBUG_VISIT_CONST
  #define __Pyx_VISIT_CONST(obj)  Py_VISIT(obj)
#else
  #define __Pyx_VISIT_CONST(obj)
#endif
#if CYTHON_ASSUME_SAFE_MACROS
  #define __Pyx_PySequence_ITEM(o, i) PySequence_ITEM(o, i)
  #define __Pyx_PySequence_SIZE(seq)  Py_SIZE(seq)
  #define __Pyx_PyTuple_SET_ITEM(o, i, v) (PyTuple_SET_ITEM(o, i, v), (0))
  #define __Pyx_PyTuple_GET_ITEM(o, i) PyTuple_GET_ITEM(o, i)
  #define __Pyx_PyList_SET_ITEM(o, i, v) (PyList_SET_ITEM(o, i, v), (0))
  #define __Pyx_PyList_GET_ITEM(o, i) PyList_GET_ITEM(o, i)
#else
  #define __Pyx_PySequence_ITEM(o, i) PySequence_GetItem(o, i)
  #define __Pyx_PySequence_SIZE(seq)  PySequence_Size(seq)
  #define __Pyx_PyTuple_SET_ITEM(o, i, v) PyTuple_SetItem(o, i, v)
  #define __Pyx_PyTuple_GET_ITEM(o, i) PyTuple_GetItem(o, i)
  #define __Pyx_PyList_SET_ITEM(o, i, v) PyList_SetItem(o, i, v)
  #define __Pyx_PyList_GET_ITEM(o, i) PyList_GetItem(o, i)
#endif
#if CYTHON_ASSUME_SAFE_SIZE
  #define __Pyx_PyTuple_GET_SIZE(o) PyTuple_GET_SIZE(o)
  #define __Pyx_PyList_GET_SIZE(o) PyList_GET_SIZE(o)
  #define __Pyx_PySet_GET_SIZE(o) PySet_GET_SIZE(o)
  #define __Pyx_PyBytes_GET_SIZE(o) PyBytes_GET_SIZE(o)
  #define __Pyx_PyByteArray_GET_SIZE(o) PyByteArray_GET_SIZE(o)
  #define __Pyx_PyUnicode_GET_LENGTH(o) PyUnicode_GET_LENGTH(o)
#else
  #define __Pyx_PyTuple_GET_SIZE(o) PyTuple_Size(o)
  #define __Pyx_PyList_GET_SIZE(o) PyList_Size(o)
  #define __Pyx_PySet_GET_SIZE(o) PySet_Size(o)
  #define __Pyx_PyBytes_GET_SIZE(o) PyBytes_Size(o)
  #define __Pyx_PyByteArray_GET_SIZE(o) PyByteArray_Size(o)
  #define __Pyx_PyUnicode_GET_LENGTH(o) PyUnicode_GetLength(o)
#endif
#if CYTHON_COMPILING_IN_PYPY && !defined(PyUnicode_InternFromString)
  #define PyUnicode_InternFromString(s) PyUnicode_FromString(s)
#endif
#define __Pyx_PyLong_FromHash_t PyLong_FromSsize_t
#define __Pyx_PyLong_AsHash_t   __Pyx_PyIndex_AsSsize_t
#if __PYX_LIMITED_VERSION_HEX >= 0x030A0000
    #define __Pyx_PySendResult PySendResult
#else
    typedef enum {
        PYGEN_RETURN = 0,
        PYGEN_ERROR = -1,
        PYGEN_NEXT = 1,
    } __Pyx_PySendResult;
#endif
#if CYTHON_COMPILING_IN_LIMITED_API || PY_VERSION_HEX < 0x030A00A3
  typedef __Pyx_PySendResult (*__Pyx_pyiter_sendfunc)(PyObject *iter, PyObject *value, PyObject **result);
#else
  #define __Pyx_pyiter_sendfunc sendfunc
#endif
#if !CYTHON_USE_AM_SEND
#define __PYX_HAS_PY_AM_SEND 0
#elif __PYX_LIMITED_VERSION_HEX >= 0x030A0000
#define __PYX_HAS_PY_AM_SEND 1
#else
#define __PYX_HAS_PY_AM_SEND 2  // our own backported implementation
#endif
#if __PYX_HAS_PY_AM_SEND < 2
    #define __Pyx_PyAsyncMethodsStruct PyAsyncMethods
#else
    typedef struct {
        unaryfunc am_await;
        unaryfunc am_aiter;
        unaryfunc am_anext;
        __Pyx_pyiter_sendfunc am_send;
    } __Pyx_PyAsyncMethodsStruct;
    #define __Pyx_SlotTpAsAsync(s) ((PyAsyncMethods*)(s))
#endif
#if CYTHON_USE_AM_SEND && PY_VERSION_HEX < 0x030A00F0
    #define __Pyx_TPFLAGS_HAVE_AM_SEND (1UL << 21)
#else
    #define __Pyx_TPFLAGS_HAVE_AM_SEND (0)
#endif
#if PY_VERSION_HEX >= 0x03090000
#define __Pyx_PyInterpreterState_Get() PyInterpreterState_Get()
#else
#define __Pyx_PyInterpreterState_Get() PyThreadState_Get()->interp
#endif
#if CYTHON_COMPILING_IN_LIMITED_API && PY_VERSION_HEX < 0x030A0000
#ifdef __cplusplus
extern "C"
#endif
PyAPI_FUNC(void *) PyMem_Calloc(size_t nelem, size_t elsize);
#endif
#if CYTHON_COMPILING_IN_LIMITED_API
static int __Pyx_init_co_variable(PyObject *inspect, const char* name, int *write_to) {
    int value;
    PyObject *py_value = PyObject_GetAttrString(inspect, name);
    if (!py_value) return 0;
    value = (int) PyLong_AsLong(py_value);
    Py_DECREF(py_value);
    *write_to = value;
    return value != -1 || !PyErr_Occurred();
}
static int __Pyx_init_co_variables(void) {
    PyObject *inspect;
    int result;
    inspect = PyImport_ImportModule("inspect");
    result =
#if !defined(CO_OPTIMIZED)
        __Pyx_init_co_variable(inspect, "CO_OPTIMIZED", &CO_OPTIMIZED) &&
#endif
#if !defined(CO_NEWLOCALS)
        __Pyx_init_co_variable(inspect, "CO_NEWLOCALS", &CO_NEWLOCALS) &&
#endif
#if !defined(CO_VARARGS)
        __Pyx_init_co_variable(inspect, "CO_VARARGS", &CO_VARARGS) &&
#endif
#if !defined(CO_VARKEYWORDS)
        __Pyx_init_co_variable(inspect, "CO_VARKEYWORDS", &CO_VARKEYWORDS) &&
#endif
#if !defined(CO_ASYNC_GENERATOR)
        __Pyx_init_co_variable(inspect, "CO_ASYNC_GENERATOR", &CO_ASYNC_GENERATOR) &&
#endif
#if !defined(CO_GENERATOR)
        __Pyx_init_co_variable(inspect, "CO_GENERATOR", &CO_GENERATOR) &&
#endif
#if !defined(CO_COROUTINE)
        __Pyx_init_co_variable(inspect, "CO_COROUTINE", &CO_COROUTINE) &&
#endif
        1;
    Py_DECREF(inspect);
    return result ? 0 : -1;
}
#else
static int __Pyx_init_co_variables(void) {
    return 0;  // It's a limited API-only feature
}
#endif

/* MathInitCode */
#if defined(_WIN32) || defined(WIN32) || defined(MS_WINDOWS)
  #ifndef _USE_MATH_DEFINES
    #define _USE_MATH_DEFINES
  #endif
#endif
#include <math.h>
#if defined(__CYGWIN__) && defined(_LDBL_EQ_DBL)
#define __Pyx_truncl trunc
#else
#define __Pyx_truncl truncl
#endif

#ifndef CYTHON_CLINE_IN_TRACEBACK_RUNTIME
#define CYTHON_CLINE_IN_TRACEBACK_RUNTIME 0
#endif
#ifndef CYTHON_CLINE_IN_TRACEBACK
#define CYTHON_CLINE_IN_TRACEBACK CYTHON_CLINE_IN_TRACEBACK_RUNTIME
#endif
#if CYTHON_CLINE_IN_TRACEBACK
#define __PYX_MARK_ERR_POS(f_index, lineno)  { __pyx_filename = __pyx_f[f_index]; (void) __pyx_filename; __pyx_lineno = lineno; (void) __pyx_lineno; __pyx_clineno = __LINE__; (void) __pyx_clineno; }
#else
#define __PYX_MARK_ERR_POS(f_index, lineno)  { __pyx_filename = __pyx_f[f_index]; (void) __pyx_filename; __pyx_lineno = lineno; (void) __pyx_lineno; (void) __pyx_clineno; }
#endif
#define __PYX_ERR(f_index, lineno, Ln_error) \
    { __PYX_MARK_ERR_POS(f_index, lineno) goto Ln_error; }

#ifdef CYTHON_EXTERN_C
    #undef __PYX_EXTERN_C
    #define __PYX_EXTERN_C CYTHON_EXTERN_C
#elif defined(__PYX_EXTERN_C)
    #ifdef _MSC_VER
    #pragma message ("Please do not define the '__PYX_EXTERN_C' macro externally. Use 'CYTHON_EXTERN_C' instead.")
    #else
    #warning Please do not define the '__PYX_EXTERN_C' macro externally. Use 'CYTHON_EXTERN_C' instead.
    #endif
#else
    #define __PYX_EXTERN_C extern "C++"
#endif

#define __PYX_HAVE__cupyx__cusolver
#define __PYX_HAVE_API__cupyx__cusolver
/* Early includes */
#include <stdint.h>
#include "ios"
#include "new"
#include "stdexcept"
#include "typeinfo"
#include <vector>
#include <utility>

    #if __cplusplus >= 201103L || (defined(_MSC_VER) && _MSC_VER >= 1600)
    // move should be defined for these versions of MSVC, but __cplusplus isn't set usefully
    #include <type_traits>

    namespace cython_std {
    template <typename T> typename std::remove_reference<T>::type&& move(T& t) noexcept { return std::move(t); }
    template <typename T> typename std::remove_reference<T>::type&& move(T&& t) noexcept { return std::move(t); }
    }

    #endif
    
#include <map>
#include "../cupy_backends/cupy_complex.h"
#include "../cupy_backends/cupy_lapack.h"
#ifdef _OPENMP
#include <omp.h>
#endif /* _OPENMP */

#if defined(PYREX_WITHOUT_ASSERTIONS) && !defined(CYTHON_WITHOUT_ASSERTIONS)
#define CYTHON_WITHOUT_ASSERTIONS
#endif

#ifdef CYTHON_FREETHREADING_COMPATIBLE
#if CYTHON_FREETHREADING_COMPATIBLE
#define __Pyx_FREETHREADING_COMPATIBLE Py_MOD_GIL_NOT_USED
#else
#define __Pyx_FREETHREADING_COMPATIBLE Py_MOD_GIL_USED
#endif
#else
#define __Pyx_FREETHREADING_COMPATIBLE Py_MOD_GIL_NOT_USED
#endif
#define __PYX_DEFAULT_STRING_ENCODING_IS_ASCII 0
#define __PYX_DEFAULT_STRING_ENCODING_IS_UTF8 0
#define __PYX_DEFAULT_STRING_ENCODING ""
#define __Pyx_PyObject_FromString __Pyx_PyBytes_FromString
#define __Pyx_PyObject_FromStringAndSize __Pyx_PyBytes_FromStringAndSize
#define __Pyx_uchar_cast(c) ((unsigned char)c)
#define __Pyx_long_cast(x) ((long)x)
#define __Pyx_fits_Py_ssize_t(v, type, is_signed)  (\
    (sizeof(type) < sizeof(Py_ssize_t))  ||\
    (sizeof(type) > sizeof(Py_ssize_t) &&\
          likely(v < (type)PY_SSIZE_T_MAX ||\
                 v == (type)PY_SSIZE_T_MAX)  &&\
          (!is_signed || likely(v > (type)PY_SSIZE_T_MIN ||\
                                v == (type)PY_SSIZE_T_MIN)))  ||\
    (sizeof(type) == sizeof(Py_ssize_t) &&\
          (is_signed || likely(v < (type)PY_SSIZE_T_MAX ||\
                               v == (type)PY_SSIZE_T_MAX)))  )
static CYTHON_INLINE int __Pyx_is_valid_index(Py_ssize_t i, Py_ssize_t limit) {
    return (size_t) i < (size_t) limit;
}
#if defined (__cplusplus) && __cplusplus >= 201103L
    #include <cstdlib>
    #define __Pyx_sst_abs(value) std::abs(value)
#elif SIZEOF_INT >= SIZEOF_SIZE_T
    #define __Pyx_sst_abs(value) abs(value)
#elif SIZEOF_LONG >= SIZEOF_SIZE_T
    #define __Pyx_sst_abs(value) labs(value)
#elif defined (_MSC_VER)
    #define __Pyx_sst_abs(value) ((Py_ssize_t)_abs64(value))
#elif defined (__STDC_VERSION__) && __STDC_VERSION__ >= 199901L
    #define __Pyx_sst_abs(value) llabs(value)
#elif defined (__GNUC__)
    #define __Pyx_sst_abs(value) __builtin_llabs(value)
#else
    #define __Pyx_sst_abs(value) ((value<0) ? -value : value)
#endif
static CYTHON_INLINE Py_ssize_t __Pyx_ssize_strlen(const char *s);
static CYTHON_INLINE const char* __Pyx_PyObject_AsString(PyObject*);
static CYTHON_INLINE const char* __Pyx_PyObject_AsStringAndSize(PyObject*, Py_ssize_t* length);
static CYTHON_INLINE PyObject* __Pyx_PyByteArray_FromString(const char*);
#define __Pyx_PyByteArray_FromStringAndSize(s, l) PyByteArray_FromStringAndSize((const char*)s, l)
#define __Pyx_PyBytes_FromString        PyBytes_FromString
#define __Pyx_PyBytes_FromStringAndSize PyBytes_FromStringAndSize
static CYTHON_INLINE PyObject* __Pyx_PyUnicode_FromString(const char*);
#if CYTHON_ASSUME_SAFE_MACROS
    #define __Pyx_PyBytes_AsWritableString(s)     ((char*) PyBytes_AS_STRING(s))
    #define __Pyx_PyBytes_AsWritableSString(s)    ((signed char*) PyBytes_AS_STRING(s))
    #define __Pyx_PyBytes_AsWritableUString(s)    ((unsigned char*) PyBytes_AS_STRING(s))
    #define __Pyx_PyBytes_AsString(s)     ((const char*) PyBytes_AS_STRING(s))
    #define __Pyx_PyBytes_AsSString(s)    ((const signed char*) PyBytes_AS_STRING(s))
    #define __Pyx_PyBytes_AsUString(s)    ((const unsigned char*) PyBytes_AS_STRING(s))
    #define __Pyx_PyByteArray_AsString(s) PyByteArray_AS_STRING(s)
#else
    #define __Pyx_PyBytes_AsWritableString(s)     ((char*) PyBytes_AsString(s))
    #define __Pyx_PyBytes_AsWritableSString(s)    ((signed char*) PyBytes_AsString(s))
    #define __Pyx_PyBytes_AsWritableUString(s)    ((unsigned char*) PyBytes_AsString(s))
    #define __Pyx_PyBytes_AsString(s)     ((const char*) PyBytes_AsString(s))
    #define __Pyx_PyBytes_AsSString(s)    ((const signed char*) PyBytes_AsString(s))
    #define __Pyx_PyBytes_AsUString(s)    ((const unsigned char*) PyBytes_AsString(s))
    #define __Pyx_PyByteArray_AsString(s) PyByteArray_AsString(s)
#endif
#define __Pyx_PyObject_AsWritableString(s)    ((char*)(__pyx_uintptr_t) __Pyx_PyObject_AsString(s))
#define __Pyx_PyObject_AsWritableSString(s)    ((signed char*)(__pyx_uintptr_t) __Pyx_PyObject_AsString(s))
#define __Pyx_PyObject_AsWritableUString(s)    ((unsigned char*)(__pyx_uintptr_t) __Pyx_PyObject_AsString(s))
#define __Pyx_PyObject_AsSString(s)    ((const signed char*) __Pyx_PyObject_AsString(s))
#define __Pyx_PyObject_AsUString(s)    ((const unsigned char*) __Pyx_PyObject_AsString(s))
#define __Pyx_PyObject_FromCString(s)  __Pyx_PyObject_FromString((const char*)s)
#define __Pyx_PyBytes_FromCString(s)   __Pyx_PyBytes_FromString((const char*)s)
#define __Pyx_PyByteArray_FromCString(s)   __Pyx_PyByteArray_FromString((const char*)s)
#define __Pyx_PyUnicode_FromCString(s) __Pyx_PyUnicode_FromString((const char*)s)
#define __Pyx_PyUnicode_FromOrdinal(o)       PyUnicode_FromOrdinal((int)o)
#define __Pyx_PyUnicode_AsUnicode            PyUnicode_AsUnicode
static CYTHON_INLINE PyObject *__Pyx_NewRef(PyObject *obj) {
#if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x030a0000 || defined(Py_NewRef)
    return Py_NewRef(obj);
#else
    Py_INCREF(obj);
    return obj;
#endif
}
static CYTHON_INLINE PyObject *__Pyx_XNewRef(PyObject *obj) {
#if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x030a0000 || defined(Py_XNewRef)
    return Py_XNewRef(obj);
#else
    Py_XINCREF(obj);
    return obj;
#endif
}
static CYTHON_INLINE PyObject *__Pyx_Owned_Py_None(int b);
static CYTHON_INLINE PyObject * __Pyx_PyBool_FromLong(long b);
static CYTHON_INLINE int __Pyx_PyObject_IsTrue(PyObject*);
static CYTHON_INLINE int __Pyx_PyObject_IsTrueAndDecref(PyObject*);
static CYTHON_INLINE PyObject* __Pyx_PyNumber_Long(PyObject* x);
#define __Pyx_PySequence_Tuple(obj)\
    (likely(PyTuple_CheckExact(obj)) ? __Pyx_NewRef(obj) : PySequence_Tuple(obj))
static CYTHON_INLINE Py_ssize_t __Pyx_PyIndex_AsSsize_t(PyObject*);
static CYTHON_INLINE PyObject * __Pyx_PyLong_FromSize_t(size_t);
static CYTHON_INLINE Py_hash_t __Pyx_PyIndex_AsHash_t(PyObject*);
#if CYTHON_ASSUME_SAFE_MACROS
#define __Pyx_PyFloat_AsDouble(x) (PyFloat_CheckExact(x) ? PyFloat_AS_DOUBLE(x) : PyFloat_AsDouble(x))
#define __Pyx_PyFloat_AS_DOUBLE(x) PyFloat_AS_DOUBLE(x)
#else
#define __Pyx_PyFloat_AsDouble(x) PyFloat_AsDouble(x)
#define __Pyx_PyFloat_AS_DOUBLE(x) PyFloat_AsDouble(x)
#endif
#define __Pyx_PyFloat_AsFloat(x) ((float) __Pyx_PyFloat_AsDouble(x))
#define __Pyx_PyNumber_Int(x) (PyLong_CheckExact(x) ? __Pyx_NewRef(x) : PyNumber_Long(x))
#if CYTHON_USE_PYLONG_INTERNALS
  #if PY_VERSION_HEX >= 0x030C00A7
  #ifndef _PyLong_SIGN_MASK
    #define _PyLong_SIGN_MASK 3
  #endif
  #ifndef _PyLong_NON_SIZE_BITS
    #define _PyLong_NON_SIZE_BITS 3
  #endif
  #define __Pyx_PyLong_Sign(x)  (((PyLongObject*)x)->long_value.lv_tag & _PyLong_SIGN_MASK)
  #define __Pyx_PyLong_IsNeg(x)  ((__Pyx_PyLong_Sign(x) & 2) != 0)
  #define __Pyx_PyLong_IsNonNeg(x)  (!__Pyx_PyLong_IsNeg(x))
  #define __Pyx_PyLong_IsZero(x)  (__Pyx_PyLong_Sign(x) & 1)
  #define __Pyx_PyLong_IsPos(x)  (__Pyx_PyLong_Sign(x) == 0)
  #define __Pyx_PyLong_CompactValueUnsigned(x)  (__Pyx_PyLong_Digits(x)[0])
  #define __Pyx_PyLong_DigitCount(x)  ((Py_ssize_t) (((PyLongObject*)x)->long_value.lv_tag >> _PyLong_NON_SIZE_BITS))
  #define __Pyx_PyLong_SignedDigitCount(x)\
        ((1 - (Py_ssize_t) __Pyx_PyLong_Sign(x)) * __Pyx_PyLong_DigitCount(x))
  #if defined(PyUnstable_Long_IsCompact) && defined(PyUnstable_Long_CompactValue)
    #define __Pyx_PyLong_IsCompact(x)     PyUnstable_Long_IsCompact((PyLongObject*) x)
    #define __Pyx_PyLong_CompactValue(x)  PyUnstable_Long_CompactValue((PyLongObject*) x)
  #else
    #define __Pyx_PyLong_IsCompact(x)     (((PyLongObject*)x)->long_value.lv_tag < (2 << _PyLong_NON_SIZE_BITS))
    #define __Pyx_PyLong_CompactValue(x)  ((1 - (Py_ssize_t) __Pyx_PyLong_Sign(x)) * (Py_ssize_t) __Pyx_PyLong_Digits(x)[0])
  #endif
  typedef Py_ssize_t  __Pyx_compact_pylong;
  typedef size_t  __Pyx_compact_upylong;
  #else
  #define __Pyx_PyLong_IsNeg(x)  (Py_SIZE(x) < 0)
  #define __Pyx_PyLong_IsNonNeg(x)  (Py_SIZE(x) >= 0)
  #define __Pyx_PyLong_IsZero(x)  (Py_SIZE(x) == 0)
  #define __Pyx_PyLong_IsPos(x)  (Py_SIZE(x) > 0)
  #define __Pyx_PyLong_CompactValueUnsigned(x)  ((Py_SIZE(x) == 0) ? 0 : __Pyx_PyLong_Digits(x)[0])
  #define __Pyx_PyLong_DigitCount(x)  __Pyx_sst_abs(Py_SIZE(x))
  #define __Pyx_PyLong_SignedDigitCount(x)  Py_SIZE(x)
  #define __Pyx_PyLong_IsCompact(x)  (Py_SIZE(x) == 0 || Py_SIZE(x) == 1 || Py_SIZE(x) == -1)
  #define __Pyx_PyLong_CompactValue(x)\
        ((Py_SIZE(x) == 0) ? (sdigit) 0 : ((Py_SIZE(x) < 0) ? -(sdigit)__Pyx_PyLong_Digits(x)[0] : (sdigit)__Pyx_PyLong_Digits(x)[0]))
  typedef sdigit  __Pyx_compact_pylong;
  typedef digit  __Pyx_compact_upylong;
  #endif
  #if PY_VERSION_HEX >= 0x030C00A5
  #define __Pyx_PyLong_Digits(x)  (((PyLongObject*)x)->long_value.ob_digit)
  #else
  #define __Pyx_PyLong_Digits(x)  (((PyLongObject*)x)->ob_digit)
  #endif
#endif
#if __PYX_DEFAULT_STRING_ENCODING_IS_UTF8
  #define __Pyx_PyUnicode_FromStringAndSize(c_str, size) PyUnicode_DecodeUTF8(c_str, size, NULL)
#elif __PYX_DEFAULT_STRING_ENCODING_IS_ASCII
  #define __Pyx_PyUnicode_FromStringAndSize(c_str, size) PyUnicode_DecodeASCII(c_str, size, NULL)
#else
  #define __Pyx_PyUnicode_FromStringAndSize(c_str, size) PyUnicode_Decode(c_str, size, __PYX_DEFAULT_STRING_ENCODING, NULL)
#endif


/* Test for GCC > 2.95 */
#if defined(__GNUC__)     && (__GNUC__ > 2 || (__GNUC__ == 2 && (__GNUC_MINOR__ > 95)))
  #define likely(x)   __builtin_expect(!!(x), 1)
  #define unlikely(x) __builtin_expect(!!(x), 0)
#else /* !__GNUC__ or GCC < 2.95 */
  #define likely(x)   (x)
  #define unlikely(x) (x)
#endif /* __GNUC__ */
/* PretendToInitialize */
#ifdef __cplusplus
#if __cplusplus > 201103L
#include <type_traits>
#endif
template <typename T>
static void __Pyx_pretend_to_initialize(T* ptr) {
#if __cplusplus > 201103L
    if ((std::is_trivially_default_constructible<T>::value))
#endif
        *ptr = T();
    (void)ptr;
}
#else
static CYTHON_INLINE void __Pyx_pretend_to_initialize(void* ptr) { (void)ptr; }
#endif


#if !CYTHON_USE_MODULE_STATE
static PyObject *__pyx_m = NULL;
#endif
static int __pyx_lineno;
static int __pyx_clineno = 0;
static const char * const __pyx_cfilenm = __FILE__;
static const char *__pyx_filename;

/* #### Code section: filename_table ### */

static const char* const __pyx_f[] = {
  "cupyx/cusolver.pyx",
  "vector.from_py",
  "cupy/cuda/memory.pxd",
  "cupy/_core/core.pxd",
  "cupy_backends/cuda/api/runtime.pxd",
  "cupy/cuda/device.pxd",
  "cupy/cuda/function.pxd",
  "cupy/_core/_carray.pxd",
};
/* #### Code section: utility_code_proto_before_types ### */
/* CriticalSectionsDefinition.proto (used by CriticalSections) */
#if !CYTHON_COMPILING_IN_CPYTHON_FREETHREADING
#define __Pyx_PyCriticalSection void*
#define __Pyx_PyCriticalSection2 void*
#define __Pyx_PyCriticalSection_End(cs)
#define __Pyx_PyCriticalSection2_End(cs)
#else
#define __Pyx_PyCriticalSection PyCriticalSection
#define __Pyx_PyCriticalSection2 PyCriticalSection2
#define __Pyx_PyCriticalSection_End PyCriticalSection_End
#define __Pyx_PyCriticalSection2_End PyCriticalSection2_End
#endif

/* CriticalSections.proto */
#if !CYTHON_COMPILING_IN_CPYTHON_FREETHREADING
#define __Pyx_PyCriticalSection_Begin(cs, arg) (void)(cs)
#define __Pyx_PyCriticalSection2_Begin(cs, arg1, arg2) (void)(cs)
#else
#define __Pyx_PyCriticalSection_Begin PyCriticalSection_Begin
#define __Pyx_PyCriticalSection2_Begin PyCriticalSection2_Begin
#endif
#if PY_VERSION_HEX < 0x030d0000 || CYTHON_COMPILING_IN_LIMITED_API
#define __Pyx_BEGIN_CRITICAL_SECTION(o) {
#define __Pyx_END_CRITICAL_SECTION() }
#else
#define __Pyx_BEGIN_CRITICAL_SECTION Py_BEGIN_CRITICAL_SECTION
#define __Pyx_END_CRITICAL_SECTION Py_END_CRITICAL_SECTION
#endif

/* Atomics.proto (used by UnpackUnboundCMethod) */
#include <pythread.h>
#ifndef CYTHON_ATOMICS
    #define CYTHON_ATOMICS 1
#endif
#define __PYX_CYTHON_ATOMICS_ENABLED() CYTHON_ATOMICS
#define __PYX_GET_CYTHON_COMPILING_IN_CPYTHON_FREETHREADING() CYTHON_COMPILING_IN_CPYTHON_FREETHREADING
#define __pyx_atomic_int_type int
#define __pyx_nonatomic_int_type int
#if CYTHON_ATOMICS && (defined(__STDC_VERSION__) &&\
                        (__STDC_VERSION__ >= 201112L) &&\
                        !defined(__STDC_NO_ATOMICS__))
    #include <stdatomic.h>
#elif CYTHON_ATOMICS && (defined(__cplusplus) && (\
                    (__cplusplus >= 201103L) ||\
                    (defined(_MSC_VER) && _MSC_VER >= 1700)))
    #include <atomic>
#endif
#if CYTHON_ATOMICS && (defined(__STDC_VERSION__) &&\
                        (__STDC_VERSION__ >= 201112L) &&\
                        !defined(__STDC_NO_ATOMICS__) &&\
                       ATOMIC_INT_LOCK_FREE == 2)
    #undef __pyx_atomic_int_type
    #define __pyx_atomic_int_type atomic_int
    #define __pyx_atomic_ptr_type atomic_uintptr_t
    #define __pyx_nonatomic_ptr_type uintptr_t
    #define __pyx_atomic_incr_relaxed(value) atomic_fetch_add_explicit(value, 1, memory_order_relaxed)
    #define __pyx_atomic_incr_acq_rel(value) atomic_fetch_add_explicit(value, 1, memory_order_acq_rel)
    #define __pyx_atomic_decr_acq_rel(value) atomic_fetch_sub_explicit(value, 1, memory_order_acq_rel)
    #define __pyx_atomic_sub(value, arg) atomic_fetch_sub(value, arg)
    #define __pyx_atomic_int_cmp_exchange(value, expected, desired) atomic_compare_exchange_strong(value, expected, desired)
    #define __pyx_atomic_load(value) atomic_load(value)
    #define __pyx_atomic_store(value, new_value) atomic_store(value, new_value)
    #define __pyx_atomic_pointer_load_relaxed(value) atomic_load_explicit(value, memory_order_relaxed)
    #define __pyx_atomic_pointer_load_acquire(value) atomic_load_explicit(value, memory_order_acquire)
    #define __pyx_atomic_pointer_exchange(value, new_value) atomic_exchange(value, (__pyx_nonatomic_ptr_type)new_value)
    #define __pyx_atomic_pointer_cmp_exchange(value, expected, desired) atomic_compare_exchange_strong(value, expected, desired)
    #if defined(__PYX_DEBUG_ATOMICS) && defined(_MSC_VER)
        #pragma message ("Using standard C atomics")
    #elif defined(__PYX_DEBUG_ATOMICS)
        #warning "Using standard C atomics"
    #endif
#elif CYTHON_ATOMICS && (defined(__cplusplus) && (\
                    (__cplusplus >= 201103L) ||\
\
                    (defined(_MSC_VER) && _MSC_VER >= 1700)) &&\
                    ATOMIC_INT_LOCK_FREE == 2)
    #undef __pyx_atomic_int_type
    #define __pyx_atomic_int_type std::atomic_int
    #define __pyx_atomic_ptr_type std::atomic_uintptr_t
    #define __pyx_nonatomic_ptr_type uintptr_t
    #define __pyx_atomic_incr_relaxed(value) std::atomic_fetch_add_explicit(value, 1, std::memory_order_relaxed)
    #define __pyx_atomic_incr_acq_rel(value) std::atomic_fetch_add_explicit(value, 1, std::memory_order_acq_rel)
    #define __pyx_atomic_decr_acq_rel(value) std::atomic_fetch_sub_explicit(value, 1, std::memory_order_acq_rel)
    #define __pyx_atomic_sub(value, arg) std::atomic_fetch_sub(value, arg)
    #define __pyx_atomic_int_cmp_exchange(value, expected, desired) std::atomic_compare_exchange_strong(value, expected, desired)
    #define __pyx_atomic_load(value) std::atomic_load(value)
    #define __pyx_atomic_store(value, new_value) std::atomic_store(value, new_value)
    #define __pyx_atomic_pointer_load_relaxed(value) std::atomic_load_explicit(value, std::memory_order_relaxed)
    #define __pyx_atomic_pointer_load_acquire(value) std::atomic_load_explicit(value, std::memory_order_acquire)
    #define __pyx_atomic_pointer_exchange(value, new_value) std::atomic_exchange(value, (__pyx_nonatomic_ptr_type)new_value)
    #define __pyx_atomic_pointer_cmp_exchange(value, expected, desired) std::atomic_compare_exchange_strong(value, expected, desired)
    #if defined(__PYX_DEBUG_ATOMICS) && defined(_MSC_VER)
        #pragma message ("Using standard C++ atomics")
    #elif defined(__PYX_DEBUG_ATOMICS)
        #warning "Using standard C++ atomics"
    #endif
#elif CYTHON_ATOMICS && (__GNUC__ >= 5 || (__GNUC__ == 4 &&\
                    (__GNUC_MINOR__ > 1 ||\
                    (__GNUC_MINOR__ == 1 && __GNUC_PATCHLEVEL__ >= 2))))
    #define __pyx_atomic_ptr_type void*
    #define __pyx_nonatomic_ptr_type void*
    #define __pyx_atomic_incr_relaxed(value) __sync_fetch_and_add(value, 1)
    #define __pyx_atomic_incr_acq_rel(value) __sync_fetch_and_add(value, 1)
    #define __pyx_atomic_decr_acq_rel(value) __sync_fetch_and_sub(value, 1)
    #define __pyx_atomic_sub(value, arg) __sync_fetch_and_sub(value, arg)
    static CYTHON_INLINE int __pyx_atomic_int_cmp_exchange(__pyx_atomic_int_type* value, __pyx_nonatomic_int_type* expected, __pyx_nonatomic_int_type desired) {
        __pyx_nonatomic_int_type old = __sync_val_compare_and_swap(value, *expected, desired);
        int result = old == *expected;
        *expected = old;
        return result;
    }
    #define __pyx_atomic_load(value) __sync_fetch_and_add(value, 0)
    #define __pyx_atomic_store(value, new_value) __sync_lock_test_and_set(value, new_value)
    #define __pyx_atomic_pointer_load_relaxed(value) __sync_fetch_and_add(value, 0)
    #define __pyx_atomic_pointer_load_acquire(value) __sync_fetch_and_add(value, 0)
    #define __pyx_atomic_pointer_exchange(value, new_value) __sync_lock_test_and_set(value, (__pyx_atomic_ptr_type)new_value)
    static CYTHON_INLINE int __pyx_atomic_pointer_cmp_exchange(__pyx_atomic_ptr_type* value, __pyx_nonatomic_ptr_type* expected, __pyx_nonatomic_ptr_type desired) {
        __pyx_nonatomic_ptr_type old = __sync_val_compare_and_swap(value, *expected, desired);
        int result = old == *expected;
        *expected = old;
        return result;
    }
    #ifdef __PYX_DEBUG_ATOMICS
        #warning "Using GNU atomics"
    #endif
#elif CYTHON_ATOMICS && defined(_MSC_VER)
    #include <intrin.h>
    #undef __pyx_atomic_int_type
    #define __pyx_atomic_int_type long
    #define __pyx_atomic_ptr_type void*
    #undef __pyx_nonatomic_int_type
    #define __pyx_nonatomic_int_type long
    #define __pyx_nonatomic_ptr_type void*
    #pragma intrinsic (_InterlockedExchangeAdd, _InterlockedExchange, _InterlockedCompareExchange, _InterlockedCompareExchangePointer, _InterlockedExchangePointer)
    #define __pyx_atomic_incr_relaxed(value) _InterlockedExchangeAdd(value, 1)
    #define __pyx_atomic_incr_acq_rel(value) _InterlockedExchangeAdd(value, 1)
    #define __pyx_atomic_decr_acq_rel(value) _InterlockedExchangeAdd(value, -1)
    #define __pyx_atomic_sub(value, arg) _InterlockedExchangeAdd(value, -arg)
    static CYTHON_INLINE int __pyx_atomic_int_cmp_exchange(__pyx_atomic_int_type* value, __pyx_nonatomic_int_type* expected, __pyx_nonatomic_int_type desired) {
        __pyx_nonatomic_int_type old = _InterlockedCompareExchange(value, desired, *expected);
        int result = old == *expected;
        *expected = old;
        return result;
    }
    #define __pyx_atomic_load(value) _InterlockedExchangeAdd(value, 0)
    #define __pyx_atomic_store(value, new_value) _InterlockedExchange(value, new_value)
    #define __pyx_atomic_pointer_load_relaxed(value) *(void * volatile *)value
    #define __pyx_atomic_pointer_load_acquire(value) _InterlockedCompareExchangePointer(value, 0, 0)
    #define __pyx_atomic_pointer_exchange(value, new_value) _InterlockedExchangePointer(value, (__pyx_atomic_ptr_type)new_value)
    static CYTHON_INLINE int __pyx_atomic_pointer_cmp_exchange(__pyx_atomic_ptr_type* value, __pyx_nonatomic_ptr_type* expected, __pyx_nonatomic_ptr_type desired) {
        __pyx_atomic_ptr_type old = _InterlockedCompareExchangePointer(value, desired, *expected);
        int result = old == *expected;
        *expected = old;
        return result;
    }
    #ifdef __PYX_DEBUG_ATOMICS
        #pragma message ("Using MSVC atomics")
    #endif
#else
    #undef CYTHON_ATOMICS
    #define CYTHON_ATOMICS 0
    #ifdef __PYX_DEBUG_ATOMICS
        #warning "Not using atomics"
    #endif
#endif

/* NoFastGil.proto */
#define __Pyx_PyGILState_Ensure PyGILState_Ensure
#define __Pyx_PyGILState_Release PyGILState_Release
#define __Pyx_FastGIL_Remember()
#define __Pyx_FastGIL_Forget()
#define __Pyx_FastGilFuncInit()

/* ForceInitThreads.proto */
#ifndef __PYX_FORCE_INIT_THREADS
  #define __PYX_FORCE_INIT_THREADS 0
#endif

/* #### Code section: numeric_typedefs ### */
/* #### Code section: complex_type_declarations ### */
/* #### Code section: type_declarations ### */

/*--- Type declarations ---*/
struct __pyx_obj_13cupy_backends_4cuda_3api_7runtime_PointerAttributes;
struct __pyx_obj_13cupy_backends_4cuda_3api_7runtime_MemPoolProps;
struct __pyx_obj_4cupy_4cuda_6device_Handle;
struct __pyx_obj_4cupy_4cuda_6device_Device;
struct __pyx_obj_4cupy_4cuda_6memory_BaseMemory;
struct __pyx_obj_4cupy_4cuda_6memory_SystemMemory;
struct __pyx_obj_4cupy_4cuda_6memory_MemoryPointer;
struct __pyx_obj_4cupy_4cuda_6memory_MemoryPool;
struct __pyx_obj_4cupy_4cuda_6memory_CFunctionAllocatorMemory;
struct __pyx_obj_4cupy_4cuda_6memory_CFunctionAllocator;
struct __pyx_obj_4cupy_4cuda_6memory_PythonFunctionAllocatorMemory;
struct __pyx_obj_4cupy_4cuda_6memory_PythonFunctionAllocator;
struct __pyx_obj_4cupy_4cuda_8function_CPointer;
struct __pyx_obj_4cupy_4cuda_8function_Function;
struct __pyx_obj_4cupy_4cuda_8function_Module;
struct __pyx_obj_4cupy_4cuda_8function_LinkState;
struct __pyx_obj_4cupy_5_core_7_carray_mdspan;
struct __pyx_obj_4cupy_5_core_7_carray_CArray;
struct __pyx_obj_4cupy_5_core_7_carray_CIndexer;
struct __pyx_obj_4cupy_5_core_7_carray_Indexer;
struct __pyx_obj_4cupy_5_core_4core__ndarray_base;

/* "cupy_backends/cuda/api/_driver_enum.pxd":1
 * cpdef enum:             # <<<<<<<<<<<<<<
 *     # CUjitInputType
 *     CU_JIT_INPUT_CUBIN = 0
*/
enum  {
  __pyx_e_13cupy_backends_4cuda_3api_12_driver_enum_CU_JIT_INPUT_CUBIN = 0,
  __pyx_e_13cupy_backends_4cuda_3api_12_driver_enum_CU_JIT_INPUT_PTX = 1,
  __pyx_e_13cupy_backends_4cuda_3api_12_driver_enum_CU_JIT_INPUT_FATBINARY = 2,
  __pyx_e_13cupy_backends_4cuda_3api_12_driver_enum_CU_JIT_INPUT_OBJECT = 3,
  __pyx_e_13cupy_backends_4cuda_3api_12_driver_enum_CU_JIT_INPUT_LIBRARY = 4,
  __pyx_e_13cupy_backends_4cuda_3api_12_driver_enum_CU_JIT_INPUT_NVVM = 5,
  __pyx_e_13cupy_backends_4cuda_3api_12_driver_enum_CU_JIT_MAX_REGISTERS = 0,
  __pyx_e_13cupy_backends_4cuda_3api_12_driver_enum_CU_JIT_THREADS_PER_BLOCK = 1,
  __pyx_e_13cupy_backends_4cuda_3api_12_driver_enum_CU_JIT_WALL_TIME = 2,
  __pyx_e_13cupy_backends_4cuda_3api_12_driver_enum_CU_JIT_INFO_LOG_BUFFER = 3,
  __pyx_e_13cupy_backends_4cuda_3api_12_driver_enum_CU_JIT_INFO_LOG_BUFFER_SIZE_BYTES = 4,
  __pyx_e_13cupy_backends_4cuda_3api_12_driver_enum_CU_JIT_ERROR_LOG_BUFFER = 5,
  __pyx_e_13cupy_backends_4cuda_3api_12_driver_enum_CU_JIT_ERROR_LOG_BUFFER_SIZE_BYTES = 6,
  __pyx_e_13cupy_backends_4cuda_3api_12_driver_enum_CU_JIT_OPTIMIZATION_LEVEL = 7,
  __pyx_e_13cupy_backends_4cuda_3api_12_driver_enum_CU_JIT_TARGET_FROM_CUCONTEXT = 8,
  __pyx_e_13cupy_backends_4cuda_3api_12_driver_enum_CU_JIT_TARGET = 9,
  __pyx_e_13cupy_backends_4cuda_3api_12_driver_enum_CU_JIT_FALLBACK_STRATEGY = 10,
  __pyx_e_13cupy_backends_4cuda_3api_12_driver_enum_CU_JIT_GENERATE_DEBUG_INFO = 11,
  __pyx_e_13cupy_backends_4cuda_3api_12_driver_enum_CU_JIT_LOG_VERBOSE = 12,
  __pyx_e_13cupy_backends_4cuda_3api_12_driver_enum_CU_JIT_GENERATE_LINE_INFO = 13,
  __pyx_e_13cupy_backends_4cuda_3api_12_driver_enum_CU_JIT_CACHE_MODE = 14,
  __pyx_e_13cupy_backends_4cuda_3api_12_driver_enum_CU_JIT_NEW_SM3X_OPT = 15,
  __pyx_e_13cupy_backends_4cuda_3api_12_driver_enum_CU_JIT_FAST_COMPILE = 16,
  __pyx_e_13cupy_backends_4cuda_3api_12_driver_enum_CU_JIT_GLOBAL_SYMBOL_NAMES = 17,
  __pyx_e_13cupy_backends_4cuda_3api_12_driver_enum_CU_JIT_GLOBAL_SYMBOL_ADDRESSES = 18,
  __pyx_e_13cupy_backends_4cuda_3api_12_driver_enum_CU_JIT_GLOBAL_SYMBOL_COUNT = 19,
  __pyx_e_13cupy_backends_4cuda_3api_12_driver_enum_CU_JIT_LTO = 20,
  __pyx_e_13cupy_backends_4cuda_3api_12_driver_enum_CU_JIT_FTZ = 21,
  __pyx_e_13cupy_backends_4cuda_3api_12_driver_enum_CU_JIT_PREC_DIV = 22,
  __pyx_e_13cupy_backends_4cuda_3api_12_driver_enum_CU_JIT_PREC_SQRT = 23,
  __pyx_e_13cupy_backends_4cuda_3api_12_driver_enum_CU_JIT_FMA = 24,
  __pyx_e_13cupy_backends_4cuda_3api_12_driver_enum_CU_JIT_NUM_OPTIONS = 25,
  __pyx_e_13cupy_backends_4cuda_3api_12_driver_enum_CU_FUNC_ATTRIBUTE_MAX_THREADS_PER_BLOCK = 0,
  __pyx_e_13cupy_backends_4cuda_3api_12_driver_enum_CU_FUNC_ATTRIBUTE_SHARED_SIZE_BYTES = 1,
  __pyx_e_13cupy_backends_4cuda_3api_12_driver_enum_CU_FUNC_ATTRIBUTE_CONST_SIZE_BYTES = 2,
  __pyx_e_13cupy_backends_4cuda_3api_12_driver_enum_CU_FUNC_ATTRIBUTE_LOCAL_SIZE_BYTES = 3,
  __pyx_e_13cupy_backends_4cuda_3api_12_driver_enum_CU_FUNC_ATTRIBUTE_NUM_REGS = 4,
  __pyx_e_13cupy_backends_4cuda_3api_12_driver_enum_CU_FUNC_ATTRIBUTE_PTX_VERSION = 5,
  __pyx_e_13cupy_backends_4cuda_3api_12_driver_enum_CU_FUNC_ATTRIBUTE_BINARY_VERSION = 6,
  __pyx_e_13cupy_backends_4cuda_3api_12_driver_enum_CU_FUNC_ATTRIBUTE_CACHE_MODE_CA = 7,
  __pyx_e_13cupy_backends_4cuda_3api_12_driver_enum_CU_FUNC_ATTRIBUTE_MAX_DYNAMIC_SHARED_SIZE_BYTES = 8,
  __pyx_e_13cupy_backends_4cuda_3api_12_driver_enum_CU_FUNC_ATTRIBUTE_PREFERRED_SHARED_MEMORY_CARVEOUT = 9,
  __pyx_e_13cupy_backends_4cuda_3api_12_driver_enum_CUDA_ERROR_INVALID_VALUE = 1,
  __pyx_e_13cupy_backends_4cuda_3api_12_driver_enum_CU_AD_FORMAT_UNSIGNED_INT8 = 0x01,
  __pyx_e_13cupy_backends_4cuda_3api_12_driver_enum_CU_AD_FORMAT_UNSIGNED_INT16 = 0x02,
  __pyx_e_13cupy_backends_4cuda_3api_12_driver_enum_CU_AD_FORMAT_UNSIGNED_INT32 = 0x03,
  __pyx_e_13cupy_backends_4cuda_3api_12_driver_enum_CU_AD_FORMAT_SIGNED_INT8 = 0x08,
  __pyx_e_13cupy_backends_4cuda_3api_12_driver_enum_CU_AD_FORMAT_SIGNED_INT16 = 0x09,
  __pyx_e_13cupy_backends_4cuda_3api_12_driver_enum_CU_AD_FORMAT_SIGNED_INT32 = 0x0a,
  __pyx_e_13cupy_backends_4cuda_3api_12_driver_enum_CU_AD_FORMAT_HALF = 0x10,
  __pyx_e_13cupy_backends_4cuda_3api_12_driver_enum_CU_AD_FORMAT_FLOAT = 0x20,
  __pyx_e_13cupy_backends_4cuda_3api_12_driver_enum_CU_TR_ADDRESS_MODE_WRAP = 0,
  __pyx_e_13cupy_backends_4cuda_3api_12_driver_enum_CU_TR_ADDRESS_MODE_CLAMP = 1,
  __pyx_e_13cupy_backends_4cuda_3api_12_driver_enum_CU_TR_ADDRESS_MODE_MIRROR = 2,
  __pyx_e_13cupy_backends_4cuda_3api_12_driver_enum_CU_TR_ADDRESS_MODE_BORDER = 3,
  __pyx_e_13cupy_backends_4cuda_3api_12_driver_enum_CU_TR_FILTER_MODE_POINT = 0,
  __pyx_e_13cupy_backends_4cuda_3api_12_driver_enum_CU_TR_FILTER_MODE_LINEAR = 1,
  __pyx_e_13cupy_backends_4cuda_3api_12_driver_enum_CU_TRSA_OVERRIDE_FORMAT = 0x01,
  __pyx_e_13cupy_backends_4cuda_3api_12_driver_enum_CU_TRSF_READ_AS_INTEGER = 0x01,
  __pyx_e_13cupy_backends_4cuda_3api_12_driver_enum_CU_TRSF_NORMALIZED_COORDINATES = 0x02,
  __pyx_e_13cupy_backends_4cuda_3api_12_driver_enum_CU_TRSF_SRGB = 0x10
};

/* "cupy_backends/cuda/api/_runtime_enum.pxd":1
 * cpdef enum:             # <<<<<<<<<<<<<<
 *     # need to revisit this when cython supports C++ enums (in 3.0)
 *     # https://stackoverflow.com/a/67138945
*/
enum  {

  /* "cupy_backends/cuda/api/_runtime_enum.pxd":116
 *     cudaGraphDebugDotFlagsKernelNodeAttributes = 1<<9
 *     cudaGraphDebugDotFlagsHandles = 1<<10
 *     cudaGraphDebugDotFlagsConditionalNodeParams = 1<<15             # <<<<<<<<<<<<<<
 * 
 * # This was a legacy mistake: the prefix "cuda" should have been removed
*/
  __pyx_e_13cupy_backends_4cuda_3api_13_runtime_enum_cudaMemoryTypeHost = 1,
  __pyx_e_13cupy_backends_4cuda_3api_13_runtime_enum_cudaMemoryTypeDevice = 2,
  __pyx_e_13cupy_backends_4cuda_3api_13_runtime_enum_cudaIpcMemLazyEnablePeerAccess = 1,
  __pyx_e_13cupy_backends_4cuda_3api_13_runtime_enum_cudaMemAttachGlobal = 1,
  __pyx_e_13cupy_backends_4cuda_3api_13_runtime_enum_cudaMemAttachHost = 2,
  __pyx_e_13cupy_backends_4cuda_3api_13_runtime_enum_cudaMemAttachSingle = 4,
  __pyx_e_13cupy_backends_4cuda_3api_13_runtime_enum_cudaCpuDeviceId = -1L,
  __pyx_e_13cupy_backends_4cuda_3api_13_runtime_enum_cudaInvalidDeviceId = -2L,
  __pyx_e_13cupy_backends_4cuda_3api_13_runtime_enum_cudaMemAdviseSetReadMostly = 1,
  __pyx_e_13cupy_backends_4cuda_3api_13_runtime_enum_cudaMemAdviseUnsetReadMostly = 2,
  __pyx_e_13cupy_backends_4cuda_3api_13_runtime_enum_cudaMemAdviseSetPreferredLocation = 3,
  __pyx_e_13cupy_backends_4cuda_3api_13_runtime_enum_cudaMemAdviseUnsetPreferredLocation = 4,
  __pyx_e_13cupy_backends_4cuda_3api_13_runtime_enum_cudaMemAdviseSetAccessedBy = 5,
  __pyx_e_13cupy_backends_4cuda_3api_13_runtime_enum_cudaMemAdviseUnsetAccessedBy = 6,
  __pyx_e_13cupy_backends_4cuda_3api_13_runtime_enum_CUDA_R_32F = 0,
  __pyx_e_13cupy_backends_4cuda_3api_13_runtime_enum_CUDA_R_64F = 1,
  __pyx_e_13cupy_backends_4cuda_3api_13_runtime_enum_CUDA_R_16F = 2,
  __pyx_e_13cupy_backends_4cuda_3api_13_runtime_enum_CUDA_R_8I = 3,
  __pyx_e_13cupy_backends_4cuda_3api_13_runtime_enum_CUDA_C_32F = 4,
  __pyx_e_13cupy_backends_4cuda_3api_13_runtime_enum_CUDA_C_64F = 5,
  __pyx_e_13cupy_backends_4cuda_3api_13_runtime_enum_CUDA_C_16F = 6,
  __pyx_e_13cupy_backends_4cuda_3api_13_runtime_enum_CUDA_C_8I = 7,
  __pyx_e_13cupy_backends_4cuda_3api_13_runtime_enum_CUDA_R_8U = 8,
  __pyx_e_13cupy_backends_4cuda_3api_13_runtime_enum_CUDA_C_8U = 9,
  __pyx_e_13cupy_backends_4cuda_3api_13_runtime_enum_CUDA_R_16BF = 14,
  __pyx_e_13cupy_backends_4cuda_3api_13_runtime_enum_cudaLimitStackSize = 0x00,
  __pyx_e_13cupy_backends_4cuda_3api_13_runtime_enum_cudaLimitPrintfFifoSize = 0x01,
  __pyx_e_13cupy_backends_4cuda_3api_13_runtime_enum_cudaLimitMallocHeapSize = 0x02,
  __pyx_e_13cupy_backends_4cuda_3api_13_runtime_enum_cudaLimitDevRuntimeSyncDepth = 0x03,
  __pyx_e_13cupy_backends_4cuda_3api_13_runtime_enum_cudaLimitDevRuntimePendingLaunchCount = 0x04,
  __pyx_e_13cupy_backends_4cuda_3api_13_runtime_enum_cudaLimitMaxL2FetchGranularity = 0x05,
  __pyx_e_13cupy_backends_4cuda_3api_13_runtime_enum_cudaChannelFormatKindSigned = 0,
  __pyx_e_13cupy_backends_4cuda_3api_13_runtime_enum_cudaChannelFormatKindUnsigned = 1,
  __pyx_e_13cupy_backends_4cuda_3api_13_runtime_enum_cudaChannelFormatKindFloat = 2,
  __pyx_e_13cupy_backends_4cuda_3api_13_runtime_enum_cudaChannelFormatKindNone = 3,
  __pyx_e_13cupy_backends_4cuda_3api_13_runtime_enum_cudaArrayDefault = 0,
  __pyx_e_13cupy_backends_4cuda_3api_13_runtime_enum_cudaArraySurfaceLoadStore = 2,
  __pyx_e_13cupy_backends_4cuda_3api_13_runtime_enum_cudaResourceTypeArray = 0,
  __pyx_e_13cupy_backends_4cuda_3api_13_runtime_enum_cudaResourceTypeMipmappedArray = 1,
  __pyx_e_13cupy_backends_4cuda_3api_13_runtime_enum_cudaResourceTypeLinear = 2,
  __pyx_e_13cupy_backends_4cuda_3api_13_runtime_enum_cudaResourceTypePitch2D = 3,
  __pyx_e_13cupy_backends_4cuda_3api_13_runtime_enum_cudaAddressModeWrap = 0,
  __pyx_e_13cupy_backends_4cuda_3api_13_runtime_enum_cudaAddressModeClamp = 1,
  __pyx_e_13cupy_backends_4cuda_3api_13_runtime_enum_cudaAddressModeMirror = 2,
  __pyx_e_13cupy_backends_4cuda_3api_13_runtime_enum_cudaAddressModeBorder = 3,
  __pyx_e_13cupy_backends_4cuda_3api_13_runtime_enum_cudaFilterModePoint = 0,
  __pyx_e_13cupy_backends_4cuda_3api_13_runtime_enum_cudaFilterModeLinear = 1,
  __pyx_e_13cupy_backends_4cuda_3api_13_runtime_enum_cudaReadModeElementType = 0,
  __pyx_e_13cupy_backends_4cuda_3api_13_runtime_enum_cudaReadModeNormalizedFloat = 1,
  __pyx_e_13cupy_backends_4cuda_3api_13_runtime_enum_cudaMemPoolReuseFollowEventDependencies = 0x1,
  __pyx_e_13cupy_backends_4cuda_3api_13_runtime_enum_cudaMemPoolReuseAllowOpportunistic = 0x2,
  __pyx_e_13cupy_backends_4cuda_3api_13_runtime_enum_cudaMemPoolReuseAllowInternalDependencies = 0x3,
  __pyx_e_13cupy_backends_4cuda_3api_13_runtime_enum_cudaMemPoolAttrReleaseThreshold = 0x4,
  __pyx_e_13cupy_backends_4cuda_3api_13_runtime_enum_cudaMemPoolAttrReservedMemCurrent = 0x5,
  __pyx_e_13cupy_backends_4cuda_3api_13_runtime_enum_cudaMemPoolAttrReservedMemHigh = 0x6,
  __pyx_e_13cupy_backends_4cuda_3api_13_runtime_enum_cudaMemPoolAttrUsedMemCurrent = 0x7,
  __pyx_e_13cupy_backends_4cuda_3api_13_runtime_enum_cudaMemPoolAttrUsedMemHigh = 0x8,
  __pyx_e_13cupy_backends_4cuda_3api_13_runtime_enum_cudaMemAllocationTypePinned = 0x1,
  __pyx_e_13cupy_backends_4cuda_3api_13_runtime_enum_cudaMemHandleTypeNone = 0x0,
  __pyx_e_13cupy_backends_4cuda_3api_13_runtime_enum_cudaMemHandleTypePosixFileDescriptor = 0x1,
  __pyx_e_13cupy_backends_4cuda_3api_13_runtime_enum_cudaMemLocationTypeInvalid = 0,
  __pyx_e_13cupy_backends_4cuda_3api_13_runtime_enum_cudaMemLocationTypeDevice = 1,
  __pyx_e_13cupy_backends_4cuda_3api_13_runtime_enum_cudaMemLocationTypeHost = 2,
  __pyx_e_13cupy_backends_4cuda_3api_13_runtime_enum_cudaMemLocationTypeHostNuma = 3,
  __pyx_e_13cupy_backends_4cuda_3api_13_runtime_enum_cudaMemLocationTypeHostNumaCurrent = 4,
  __pyx_e_13cupy_backends_4cuda_3api_13_runtime_enum_cudaGraphDebugDotFlagsVerbose = (1 << 0),
  __pyx_e_13cupy_backends_4cuda_3api_13_runtime_enum_cudaGraphDebugDotFlagsKernelNodeParams = (1 << 2),
  __pyx_e_13cupy_backends_4cuda_3api_13_runtime_enum_cudaGraphDebugDotFlagsMemcpyNodeParams = (1 << 3),
  __pyx_e_13cupy_backends_4cuda_3api_13_runtime_enum_cudaGraphDebugDotFlagsMemsetNodeParams = (1 << 4),
  __pyx_e_13cupy_backends_4cuda_3api_13_runtime_enum_cudaGraphDebugDotFlagsHostNodeParams = (1 << 5),
  __pyx_e_13cupy_backends_4cuda_3api_13_runtime_enum_cudaGraphDebugDotFlagsEventNodeParams = (1 << 6),
  __pyx_e_13cupy_backends_4cuda_3api_13_runtime_enum_cudaGraphDebugDotFlagsExtSemasSignalNodeParams = (1 << 7),
  __pyx_e_13cupy_backends_4cuda_3api_13_runtime_enum_cudaGraphDebugDotFlagsExtSemasWaitNodeParams = (1 << 8),
  __pyx_e_13cupy_backends_4cuda_3api_13_runtime_enum_cudaGraphDebugDotFlagsKernelNodeAttributes = (1 << 9),
  __pyx_e_13cupy_backends_4cuda_3api_13_runtime_enum_cudaGraphDebugDotFlagsHandles = (1 << 10),
  __pyx_e_13cupy_backends_4cuda_3api_13_runtime_enum_cudaGraphDebugDotFlagsConditionalNodeParams = (1 << 15)
};

/* "cupy_backends/cuda/api/_runtime_enum.pxd":124
 * IF CUPY_HIP_VERSION > 0:
 *     # separate in groups of 10 for easier counting...
 *     cpdef enum:             # <<<<<<<<<<<<<<
 *         cudaDevAttrMaxThreadsPerBlock = 0
 *         cudaDevAttrMaxBlockDimX
*/
enum  {
  __pyx_e_13cupy_backends_4cuda_3api_13_runtime_enum_cudaDevAttrMaxThreadsPerBlock = 0,
  __pyx_e_13cupy_backends_4cuda_3api_13_runtime_enum_cudaDevAttrMaxBlockDimX,
  __pyx_e_13cupy_backends_4cuda_3api_13_runtime_enum_cudaDevAttrMaxBlockDimY,
  __pyx_e_13cupy_backends_4cuda_3api_13_runtime_enum_cudaDevAttrMaxBlockDimZ,
  __pyx_e_13cupy_backends_4cuda_3api_13_runtime_enum_cudaDevAttrMaxGridDimX,
  __pyx_e_13cupy_backends_4cuda_3api_13_runtime_enum_cudaDevAttrMaxGridDimY,
  __pyx_e_13cupy_backends_4cuda_3api_13_runtime_enum_cudaDevAttrMaxGridDimZ,
  __pyx_e_13cupy_backends_4cuda_3api_13_runtime_enum_cudaDevAttrMaxSharedMemoryPerBlock,
  __pyx_e_13cupy_backends_4cuda_3api_13_runtime_enum_cudaDevAttrTotalConstantMemory,
  __pyx_e_13cupy_backends_4cuda_3api_13_runtime_enum_cudaDevAttrWarpSize,
  __pyx_e_13cupy_backends_4cuda_3api_13_runtime_enum_cudaDevAttrMaxRegistersPerBlock,
  __pyx_e_13cupy_backends_4cuda_3api_13_runtime_enum_cudaDevAttrClockRate,
  __pyx_e_13cupy_backends_4cuda_3api_13_runtime_enum_cudaDevAttrMemoryClockRate,
  __pyx_e_13cupy_backends_4cuda_3api_13_runtime_enum_cudaDevAttrGlobalMemoryBusWidth,
  __pyx_e_13cupy_backends_4cuda_3api_13_runtime_enum_cudaDevAttrMultiProcessorCount,
  __pyx_e_13cupy_backends_4cuda_3api_13_runtime_enum_cudaDevAttrComputeMode,
  __pyx_e_13cupy_backends_4cuda_3api_13_runtime_enum_cudaDevAttrL2CacheSize,
  __pyx_e_13cupy_backends_4cuda_3api_13_runtime_enum_cudaDevAttrMaxThreadsPerMultiProcessor,
  __pyx_e_13cupy_backends_4cuda_3api_13_runtime_enum_cudaDevAttrConcurrentKernels = 20,
  __pyx_e_13cupy_backends_4cuda_3api_13_runtime_enum_cudaDevAttrPciBusId,
  __pyx_e_13cupy_backends_4cuda_3api_13_runtime_enum_cudaDevAttrPciDeviceId,
  __pyx_e_13cupy_backends_4cuda_3api_13_runtime_enum_cudaDevAttrMaxSharedMemoryPerMultiprocessor,
  __pyx_e_13cupy_backends_4cuda_3api_13_runtime_enum_cudaDevAttrIsMultiGpuBoard,
  __pyx_e_13cupy_backends_4cuda_3api_13_runtime_enum_cudaDevAttrIntegrated,
  __pyx_e_13cupy_backends_4cuda_3api_13_runtime_enum_cudaDevAttrCooperativeLaunch,
  __pyx_e_13cupy_backends_4cuda_3api_13_runtime_enum_cudaDevAttrCooperativeMultiDeviceLaunch,
  __pyx_e_13cupy_backends_4cuda_3api_13_runtime_enum_cudaDevAttrMaxTexture1DWidth,
  __pyx_e_13cupy_backends_4cuda_3api_13_runtime_enum_cudaDevAttrMaxTexture2DWidth,
  __pyx_e_13cupy_backends_4cuda_3api_13_runtime_enum_cudaDevAttrMaxTexture2DHeight,
  __pyx_e_13cupy_backends_4cuda_3api_13_runtime_enum_cudaDevAttrMaxTexture3DWidth,
  __pyx_e_13cupy_backends_4cuda_3api_13_runtime_enum_cudaDevAttrMaxTexture3DHeight,
  __pyx_e_13cupy_backends_4cuda_3api_13_runtime_enum_cudaDevAttrMaxTexture3DDepth,
  __pyx_e_13cupy_backends_4cuda_3api_13_runtime_enum_cudaDevAttrMaxPitch = 36,
  __pyx_e_13cupy_backends_4cuda_3api_13_runtime_enum_cudaDevAttrTextureAlignment,
  __pyx_e_13cupy_backends_4cuda_3api_13_runtime_enum_cudaDevAttrTexturePitchAlignment,
  __pyx_e_13cupy_backends_4cuda_3api_13_runtime_enum_cudaDevAttrKernelExecTimeout,
  __pyx_e_13cupy_backends_4cuda_3api_13_runtime_enum_cudaDevAttrCanMapHostMemory,
  __pyx_e_13cupy_backends_4cuda_3api_13_runtime_enum_cudaDevAttrEccEnabled,
  __pyx_e_13cupy_backends_4cuda_3api_13_runtime_enum_cudaDevAttrMemoryPoolsSupported = 0
};

/* "cupy_backends/cuda/api/_runtime_enum.pxd":240
 *         # cudaDevAttrHostRegisterSupported
 *     IF CUPY_HIP_VERSION >= 310:
 *         cpdef enum:             # <<<<<<<<<<<<<<
 *             # hipDeviceAttributeAsicRevision  # does not exist in CUDA
 *             cudaDevAttrManagedMemory = 47
*/
enum  {
  __pyx_e_13cupy_backends_4cuda_3api_13_runtime_enum_cudaDevAttrManagedMemory = 47,
  __pyx_e_13cupy_backends_4cuda_3api_13_runtime_enum_cudaDevAttrDirectManagedMemAccessFromHost,
  __pyx_e_13cupy_backends_4cuda_3api_13_runtime_enum_cudaDevAttrConcurrentManagedAccess,
  __pyx_e_13cupy_backends_4cuda_3api_13_runtime_enum_cudaDevAttrPageableMemoryAccess,
  __pyx_e_13cupy_backends_4cuda_3api_13_runtime_enum_cudaDevAttrPageableMemoryAccessUsesHostPageTables
};
struct __pyx_opt_args_13cupy_backends_4cuda_3api_7runtime_mallocManaged;
struct __pyx_opt_args_13cupy_backends_4cuda_3api_7runtime_malloc3DArray;
struct __pyx_opt_args_13cupy_backends_4cuda_3api_7runtime_mallocArray;
struct __pyx_opt_args_13cupy_backends_4cuda_3api_7runtime_streamAddCallback;
struct __pyx_opt_args_13cupy_backends_4cuda_3api_7runtime_streamWaitEvent;
struct __pyx_opt_args_13cupy_backends_4cuda_3api_7runtime_streamBeginCapture;

/* "runtime.pxd":99
 * 
 * # For backward compatibility, keep APIs not prefixed with "cuda".
 * cpdef enum:             # <<<<<<<<<<<<<<
 *     memcpyHostToHost = 0
 *     memcpyHostToDevice = 1
*/
enum  {
  __pyx_e_13cupy_backends_4cuda_3api_7runtime_memcpyHostToHost = 0,
  __pyx_e_13cupy_backends_4cuda_3api_7runtime_memcpyHostToDevice = 1,
  __pyx_e_13cupy_backends_4cuda_3api_7runtime_memcpyDeviceToHost = 2,
  __pyx_e_13cupy_backends_4cuda_3api_7runtime_memcpyDeviceToDevice = 3,
  __pyx_e_13cupy_backends_4cuda_3api_7runtime_memcpyDefault = 4,
  __pyx_e_13cupy_backends_4cuda_3api_7runtime_hostAllocDefault = 0,
  __pyx_e_13cupy_backends_4cuda_3api_7runtime_hostAllocPortable = 1,
  __pyx_e_13cupy_backends_4cuda_3api_7runtime_hostAllocMapped = 2,
  __pyx_e_13cupy_backends_4cuda_3api_7runtime_hostAllocWriteCombined = 4,
  __pyx_e_13cupy_backends_4cuda_3api_7runtime_streamDefault = 0,
  __pyx_e_13cupy_backends_4cuda_3api_7runtime_streamNonBlocking = 1,
  __pyx_e_13cupy_backends_4cuda_3api_7runtime_streamLegacy = 1,
  __pyx_e_13cupy_backends_4cuda_3api_7runtime_streamPerThread = 2,
  __pyx_e_13cupy_backends_4cuda_3api_7runtime_eventDefault = 0,
  __pyx_e_13cupy_backends_4cuda_3api_7runtime_eventBlockingSync = 1,
  __pyx_e_13cupy_backends_4cuda_3api_7runtime_eventDisableTiming = 2,
  __pyx_e_13cupy_backends_4cuda_3api_7runtime_eventInterprocess = 4,
  __pyx_e_13cupy_backends_4cuda_3api_7runtime_streamCaptureModeGlobal = 0,
  __pyx_e_13cupy_backends_4cuda_3api_7runtime_streamCaptureModeThreadLocal = 1,
  __pyx_e_13cupy_backends_4cuda_3api_7runtime_streamCaptureModeRelaxed = 2,
  __pyx_e_13cupy_backends_4cuda_3api_7runtime_streamCaptureStatusNone = 0,
  __pyx_e_13cupy_backends_4cuda_3api_7runtime_streamCaptureStatusActive = 1,
  __pyx_e_13cupy_backends_4cuda_3api_7runtime_streamCaptureStatusInvalidated = 2,
  __pyx_e_13cupy_backends_4cuda_3api_7runtime_memoryTypeUnregistered = 0,
  __pyx_e_13cupy_backends_4cuda_3api_7runtime_memoryTypeHost = 1,
  __pyx_e_13cupy_backends_4cuda_3api_7runtime_memoryTypeDevice = 2,
  __pyx_e_13cupy_backends_4cuda_3api_7runtime_memoryTypeManaged = 3
};

/* "runtime.pxd":205
 * 
 * cpdef intptr_t malloc(size_t size) except? 0
 * cpdef intptr_t mallocManaged(size_t size, unsigned int flags=*) except? 0             # <<<<<<<<<<<<<<
 * cpdef intptr_t malloc3DArray(intptr_t desc, size_t width, size_t height,
 *                              size_t depth, unsigned int flags=*) except? 0
*/
struct __pyx_opt_args_13cupy_backends_4cuda_3api_7runtime_mallocManaged {
  int __pyx_n;
  unsigned int flags;
};

/* "runtime.pxd":206
 * cpdef intptr_t malloc(size_t size) except? 0
 * cpdef intptr_t mallocManaged(size_t size, unsigned int flags=*) except? 0
 * cpdef intptr_t malloc3DArray(intptr_t desc, size_t width, size_t height,             # <<<<<<<<<<<<<<
 *                              size_t depth, unsigned int flags=*) except? 0
 * cpdef intptr_t mallocArray(intptr_t desc, size_t width, size_t height,
*/
struct __pyx_opt_args_13cupy_backends_4cuda_3api_7runtime_malloc3DArray {
  int __pyx_n;
  unsigned int flags;
};

/* "runtime.pxd":208
 * cpdef intptr_t malloc3DArray(intptr_t desc, size_t width, size_t height,
 *                              size_t depth, unsigned int flags=*) except? 0
 * cpdef intptr_t mallocArray(intptr_t desc, size_t width, size_t height,             # <<<<<<<<<<<<<<
 *                            unsigned int flags=*) except? 0
 * cpdef intptr_t mallocAsync(size_t size, intptr_t stream) except? 0
*/
struct __pyx_opt_args_13cupy_backends_4cuda_3api_7runtime_mallocArray {
  int __pyx_n;
  unsigned int flags;
};

/* "runtime.pxd":275
 * cpdef streamDestroy(intptr_t stream)
 * cpdef streamSynchronize(intptr_t stream)
 * cpdef streamAddCallback(intptr_t stream, callback, intptr_t arg,             # <<<<<<<<<<<<<<
 *                         unsigned int flags=*)
 * cpdef launchHostFunc(intptr_t stream, callback, intptr_t arg)
*/
struct __pyx_opt_args_13cupy_backends_4cuda_3api_7runtime_streamAddCallback {
  int __pyx_n;
  unsigned int flags;
};

/* "runtime.pxd":280
 * cpdef _launchHostFuncUnmanaged(intptr_t stream, tuple func_arg)
 * cpdef streamQuery(intptr_t stream)
 * cpdef streamWaitEvent(intptr_t stream, intptr_t event, unsigned int flags=*)             # <<<<<<<<<<<<<<
 * cpdef streamBeginCapture(intptr_t stream, int mode=*)
 * cpdef intptr_t streamEndCapture(intptr_t stream) except? 0
*/
struct __pyx_opt_args_13cupy_backends_4cuda_3api_7runtime_streamWaitEvent {
  int __pyx_n;
  unsigned int flags;
};

/* "runtime.pxd":281
 * cpdef streamQuery(intptr_t stream)
 * cpdef streamWaitEvent(intptr_t stream, intptr_t event, unsigned int flags=*)
 * cpdef streamBeginCapture(intptr_t stream, int mode=*)             # <<<<<<<<<<<<<<
 * cpdef intptr_t streamEndCapture(intptr_t stream) except? 0
 * cpdef bint streamIsCapturing(intptr_t stream) except*
*/
struct __pyx_opt_args_13cupy_backends_4cuda_3api_7runtime_streamBeginCapture {
  int __pyx_n;
  int mode;
};
struct __pyx_ctuple_size_t__and_size_t;
typedef struct __pyx_ctuple_size_t__and_size_t __pyx_ctuple_size_t__and_size_t;

/* "cusolver.pxd":37
 * ###############################################################################
 * 
 * cpdef enum:             # <<<<<<<<<<<<<<
 *     CUSOLVER_EIG_TYPE_1 = 1
 *     CUSOLVER_EIG_TYPE_2 = 2
*/
enum  {
  __pyx_e_13cupy_backends_4cuda_4libs_8cusolver_CUSOLVER_EIG_TYPE_1 = 1,
  __pyx_e_13cupy_backends_4cuda_4libs_8cusolver_CUSOLVER_EIG_TYPE_2 = 2,
  __pyx_e_13cupy_backends_4cuda_4libs_8cusolver_CUSOLVER_EIG_TYPE_3 = 3,
  __pyx_e_13cupy_backends_4cuda_4libs_8cusolver_CUSOLVER_EIG_MODE_NOVECTOR = 0,
  __pyx_e_13cupy_backends_4cuda_4libs_8cusolver_CUSOLVER_EIG_MODE_VECTOR = 1
};

/* "cusolver.pxd":669
 * 
 * # dense eigenvalue solver (64bit)
 * cpdef (size_t, size_t) xsyevd_bufferSize(  # noqa             # <<<<<<<<<<<<<<
 *     intptr_t handle, intptr_t params, int jobz, int uplo,
 *     int64_t n, int dataTypeA, intptr_t A, int64_t lda,
*/
struct __pyx_ctuple_size_t__and_size_t {
  size_t f0;
  size_t f1;
};
struct __pyx_opt_args_4cupy_4cuda_6memory_13MemoryPointer_copy_from_device_async;
struct __pyx_opt_args_4cupy_4cuda_6memory_13MemoryPointer_copy_from_host_async;
struct __pyx_opt_args_4cupy_4cuda_6memory_13MemoryPointer_copy_from_async;
struct __pyx_opt_args_4cupy_4cuda_6memory_13MemoryPointer_copy_to_host_async;
struct __pyx_opt_args_4cupy_4cuda_6memory_13MemoryPointer_memset_async;
struct __pyx_opt_args_4cupy_4cuda_6memory_set_allocator;
struct __pyx_opt_args_4cupy_4cuda_6memory_10MemoryPool_free_all_blocks;
struct __pyx_opt_args_4cupy_4cuda_6memory_10MemoryPool_set_limit;

/* "memory.pxd":39
 * 
 *     cpdef copy_from_device(self, MemoryPointer src, size_t size)
 *     cpdef copy_from_device_async(self, MemoryPointer src, size_t size,             # <<<<<<<<<<<<<<
 *                                  stream=?)
 *     cpdef copy_from_host(self, mem, size_t size)
*/
struct __pyx_opt_args_4cupy_4cuda_6memory_13MemoryPointer_copy_from_device_async {
  int __pyx_n;
  PyObject *stream;
};

/* "memory.pxd":42
 *                                  stream=?)
 *     cpdef copy_from_host(self, mem, size_t size)
 *     cpdef copy_from_host_async(self, mem, size_t size, stream=?)             # <<<<<<<<<<<<<<
 *     cpdef copy_from(self, mem, size_t size)
 *     cpdef copy_from_async(self, mem, size_t size, stream=?)
*/
struct __pyx_opt_args_4cupy_4cuda_6memory_13MemoryPointer_copy_from_host_async {
  int __pyx_n;
  PyObject *stream;
};

/* "memory.pxd":44
 *     cpdef copy_from_host_async(self, mem, size_t size, stream=?)
 *     cpdef copy_from(self, mem, size_t size)
 *     cpdef copy_from_async(self, mem, size_t size, stream=?)             # <<<<<<<<<<<<<<
 *     cpdef copy_to_host(self, mem, size_t size)
 *     cpdef copy_to_host_async(self, mem, size_t size, stream=?)
*/
struct __pyx_opt_args_4cupy_4cuda_6memory_13MemoryPointer_copy_from_async {
  int __pyx_n;
  PyObject *stream;
};

/* "memory.pxd":46
 *     cpdef copy_from_async(self, mem, size_t size, stream=?)
 *     cpdef copy_to_host(self, mem, size_t size)
 *     cpdef copy_to_host_async(self, mem, size_t size, stream=?)             # <<<<<<<<<<<<<<
 *     cpdef memset(self, int value, size_t size)
 *     cpdef memset_async(self, int value, size_t size, stream=?)
*/
struct __pyx_opt_args_4cupy_4cuda_6memory_13MemoryPointer_copy_to_host_async {
  int __pyx_n;
  PyObject *stream;
};

/* "memory.pxd":48
 *     cpdef copy_to_host_async(self, mem, size_t size, stream=?)
 *     cpdef memset(self, int value, size_t size)
 *     cpdef memset_async(self, int value, size_t size, stream=?)             # <<<<<<<<<<<<<<
 * 
 * 
*/
struct __pyx_opt_args_4cupy_4cuda_6memory_13MemoryPointer_memset_async {
  int __pyx_n;
  PyObject *stream;
};

/* "memory.pxd":54
 * 
 * 
 * cpdef set_allocator(allocator=*)             # <<<<<<<<<<<<<<
 * cpdef get_allocator()
 * 
*/
struct __pyx_opt_args_4cupy_4cuda_6memory_set_allocator {
  int __pyx_n;
  PyObject *allocator;
};

/* "memory.pxd":65
 * 
 *     cpdef MemoryPointer malloc(self, size_t size)
 *     cpdef free_all_blocks(self, stream=?)             # <<<<<<<<<<<<<<
 *     cpdef free_all_free(self)
 *     # they call device_pool so they may raise error.
*/
struct __pyx_opt_args_4cupy_4cuda_6memory_10MemoryPool_free_all_blocks {
  int __pyx_n;
  PyObject *stream;
};

/* "memory.pxd":72
 *     cpdef size_t free_bytes(self)
 *     cpdef size_t total_bytes(self)
 *     cpdef set_limit(self, size=?, fraction=?)             # <<<<<<<<<<<<<<
 *     cpdef size_t get_limit(self)
 * 
*/
struct __pyx_opt_args_4cupy_4cuda_6memory_10MemoryPool_set_limit {
  int __pyx_n;
  PyObject *size;
  PyObject *fraction;
};
struct __pyx_opt_args_4cupy_4cuda_8function_8Function_linear_launch;

/* "cupy/cuda/function.pxd":15
 *         public intptr_t ptr
 * 
 *     cpdef linear_launch(self, size_t size, args, size_t shared_mem=*,             # <<<<<<<<<<<<<<
 *                         size_t block_max_size=*, stream=*,
 *                         bint enable_cooperative_groups=*)
*/
struct __pyx_opt_args_4cupy_4cuda_8function_8Function_linear_launch {
  int __pyx_n;
  size_t shared_mem;
  size_t block_max_size;
  PyObject *stream;
  int enable_cooperative_groups;
};
struct __pyx_t_4cupy_5_core_7_carray__CArray;
struct __pyx_t_4cupy_5_core_7_carray__CIndexer;

/* "cupy/_core/_carray.pxd":13
 * # this matches NPY_MAXDIMS
 * # Note: we make it an enum to work around cython/cython#4369
 * cdef enum: MAX_NDIM = 64             # <<<<<<<<<<<<<<
 * 
 * 
*/
enum  {
  __pyx_e_4cupy_5_core_7_carray_MAX_NDIM = 64
};

/* "cupy/_core/_carray.pxd":8
 * 
 * 
 * ctypedef vector.vector[Py_ssize_t] shape_t             # <<<<<<<<<<<<<<
 * ctypedef vector.vector[Py_ssize_t] strides_t
 * 
*/
typedef std::vector<Py_ssize_t>  __pyx_t_4cupy_5_core_7_carray_shape_t;

/* "cupy/_core/_carray.pxd":9
 * 
 * ctypedef vector.vector[Py_ssize_t] shape_t
 * ctypedef vector.vector[Py_ssize_t] strides_t             # <<<<<<<<<<<<<<
 * 
 * # this matches NPY_MAXDIMS
*/
typedef std::vector<Py_ssize_t>  __pyx_t_4cupy_5_core_7_carray_strides_t;

/* "cupy/_core/_carray.pxd":17
 * 
 * # This is now excluding shape/strides only for getting the compiler-time size
 * cdef struct _CArray:             # <<<<<<<<<<<<<<
 *     void* data
 *     Py_ssize_t size
*/
struct __pyx_t_4cupy_5_core_7_carray__CArray {
  void *data;
  Py_ssize_t size;
};

/* "cupy/_core/_carray.pxd":40
 * 
 * # This is now excluding shape/strides only for getting the compiler-time size
 * cdef struct _CIndexer:             # <<<<<<<<<<<<<<
 *     Py_ssize_t size
 * 
*/
struct __pyx_t_4cupy_5_core_7_carray__CIndexer {
  Py_ssize_t size;
};
struct __pyx_opt_args_4cupy_5_core_4core_13_ndarray_base_tobytes;
struct __pyx_opt_args_4cupy_5_core_4core_13_ndarray_base_tofile;
struct __pyx_opt_args_4cupy_5_core_4core_13_ndarray_base__astype;
struct __pyx_opt_args_4cupy_5_core_4core_13_ndarray_base_astype;
struct __pyx_opt_args_4cupy_5_core_4core_13_ndarray_base_byteswap;
struct __pyx_opt_args_4cupy_5_core_4core_13_ndarray_base_copy;
struct __pyx_opt_args_4cupy_5_core_4core_13_ndarray_base_view;
struct __pyx_opt_args_4cupy_5_core_4core_13_ndarray_base_flatten;
struct __pyx_opt_args_4cupy_5_core_4core_13_ndarray_base_ravel;
struct __pyx_opt_args_4cupy_5_core_4core_13_ndarray_base_squeeze;
struct __pyx_opt_args_4cupy_5_core_4core_13_ndarray_base_take;
struct __pyx_opt_args_4cupy_5_core_4core_13_ndarray_base_put;
struct __pyx_opt_args_4cupy_5_core_4core_13_ndarray_base_repeat;
struct __pyx_opt_args_4cupy_5_core_4core_13_ndarray_base_choose;
struct __pyx_opt_args_4cupy_5_core_4core_13_ndarray_base_sort;
struct __pyx_opt_args_4cupy_5_core_4core_13_ndarray_base_argsort;
struct __pyx_opt_args_4cupy_5_core_4core_13_ndarray_base_partition;
struct __pyx_opt_args_4cupy_5_core_4core_13_ndarray_base_argpartition;
struct __pyx_opt_args_4cupy_5_core_4core_13_ndarray_base_compress;
struct __pyx_opt_args_4cupy_5_core_4core_13_ndarray_base_diagonal;
struct __pyx_opt_args_4cupy_5_core_4core_13_ndarray_base_max;
struct __pyx_opt_args_4cupy_5_core_4core_13_ndarray_base_argmax;
struct __pyx_opt_args_4cupy_5_core_4core_13_ndarray_base_min;
struct __pyx_opt_args_4cupy_5_core_4core_13_ndarray_base_argmin;
struct __pyx_opt_args_4cupy_5_core_4core_13_ndarray_base_ptp;
struct __pyx_opt_args_4cupy_5_core_4core_13_ndarray_base_clip;
struct __pyx_opt_args_4cupy_5_core_4core_13_ndarray_base_round;
struct __pyx_opt_args_4cupy_5_core_4core_13_ndarray_base_trace;
struct __pyx_opt_args_4cupy_5_core_4core_13_ndarray_base_sum;
struct __pyx_opt_args_4cupy_5_core_4core_13_ndarray_base_cumsum;
struct __pyx_opt_args_4cupy_5_core_4core_13_ndarray_base_mean;
struct __pyx_opt_args_4cupy_5_core_4core_13_ndarray_base_var;
struct __pyx_opt_args_4cupy_5_core_4core_13_ndarray_base_std;
struct __pyx_opt_args_4cupy_5_core_4core_13_ndarray_base_prod;
struct __pyx_opt_args_4cupy_5_core_4core_13_ndarray_base_cumprod;
struct __pyx_opt_args_4cupy_5_core_4core_13_ndarray_base_all;
struct __pyx_opt_args_4cupy_5_core_4core_13_ndarray_base_any;
struct __pyx_opt_args_4cupy_5_core_4core_13_ndarray_base_get;
struct __pyx_opt_args_4cupy_5_core_4core_13_ndarray_base_set;
struct __pyx_opt_args_4cupy_5_core_4core_13_ndarray_base_reduced_view;
struct __pyx_opt_args_4cupy_5_core_4core_ascontiguousarray;
struct __pyx_opt_args_4cupy_5_core_4core_asfortranarray;
struct __pyx_opt_args_4cupy_5_core_4core_compile_with_cache;
struct __pyx_opt_args_4cupy_5_core_4core_array;

/* "cupy/_core/core.pxd":31
 *     cpdef item(self)
 *     cpdef tolist(self)
 *     cpdef bytes tobytes(self, order=*)             # <<<<<<<<<<<<<<
 *     cpdef tofile(self, fid, sep=*, format=*)
 *     cpdef dump(self, file)
*/
struct __pyx_opt_args_4cupy_5_core_4core_13_ndarray_base_tobytes {
  int __pyx_n;
  PyObject *order;
};

/* "cupy/_core/core.pxd":32
 *     cpdef tolist(self)
 *     cpdef bytes tobytes(self, order=*)
 *     cpdef tofile(self, fid, sep=*, format=*)             # <<<<<<<<<<<<<<
 *     cpdef dump(self, file)
 *     cpdef bytes dumps(self)
*/
struct __pyx_opt_args_4cupy_5_core_4core_13_ndarray_base_tofile {
  int __pyx_n;
  PyObject *sep;
  PyObject *format;
};

/* "cupy/_core/core.pxd":35
 *     cpdef dump(self, file)
 *     cpdef bytes dumps(self)
 *     cpdef _ndarray_base _astype(             # <<<<<<<<<<<<<<
 *         self, dtype, order=*, casting=*, subok=*, copy=*)
 *     cpdef _ndarray_base astype(
*/
struct __pyx_opt_args_4cupy_5_core_4core_13_ndarray_base__astype {
  int __pyx_n;
  PyObject *order;
  PyObject *casting;
  PyObject *subok;
  PyObject *copy;
};

/* "cupy/_core/core.pxd":37
 *     cpdef _ndarray_base _astype(
 *         self, dtype, order=*, casting=*, subok=*, copy=*)
 *     cpdef _ndarray_base astype(             # <<<<<<<<<<<<<<
 *         self, dtype, order=*, casting=*, subok=*, copy=*)
 *     cpdef _ndarray_base byteswap(self, inplace=*)
*/
struct __pyx_opt_args_4cupy_5_core_4core_13_ndarray_base_astype {
  int __pyx_n;
  PyObject *order;
  PyObject *casting;
  PyObject *subok;
  PyObject *copy;
};

/* "cupy/_core/core.pxd":39
 *     cpdef _ndarray_base astype(
 *         self, dtype, order=*, casting=*, subok=*, copy=*)
 *     cpdef _ndarray_base byteswap(self, inplace=*)             # <<<<<<<<<<<<<<
 *     cpdef _ndarray_base copy(self, order=*)
 *     cpdef _ndarray_base view(self, dtype=*, array_class=*)
*/
struct __pyx_opt_args_4cupy_5_core_4core_13_ndarray_base_byteswap {
  int __pyx_n;
  PyObject *inplace;
};

/* "cupy/_core/core.pxd":40
 *         self, dtype, order=*, casting=*, subok=*, copy=*)
 *     cpdef _ndarray_base byteswap(self, inplace=*)
 *     cpdef _ndarray_base copy(self, order=*)             # <<<<<<<<<<<<<<
 *     cpdef _ndarray_base view(self, dtype=*, array_class=*)
 *     cpdef fill(self, value)
*/
struct __pyx_opt_args_4cupy_5_core_4core_13_ndarray_base_copy {
  int __pyx_n;
  PyObject *order;
};

/* "cupy/_core/core.pxd":41
 *     cpdef _ndarray_base byteswap(self, inplace=*)
 *     cpdef _ndarray_base copy(self, order=*)
 *     cpdef _ndarray_base view(self, dtype=*, array_class=*)             # <<<<<<<<<<<<<<
 *     cpdef fill(self, value)
 *     cpdef _ndarray_base swapaxes(self, Py_ssize_t axis1, Py_ssize_t axis2)
*/
struct __pyx_opt_args_4cupy_5_core_4core_13_ndarray_base_view {
  int __pyx_n;
  PyObject *dtype;
  PyObject *array_class;
};

/* "cupy/_core/core.pxd":44
 *     cpdef fill(self, value)
 *     cpdef _ndarray_base swapaxes(self, Py_ssize_t axis1, Py_ssize_t axis2)
 *     cpdef _ndarray_base flatten(self, order=*)             # <<<<<<<<<<<<<<
 *     cpdef _ndarray_base ravel(self, order=*)
 *     cpdef _ndarray_base squeeze(self, axis=*)
*/
struct __pyx_opt_args_4cupy_5_core_4core_13_ndarray_base_flatten {
  int __pyx_n;
  PyObject *order;
};

/* "cupy/_core/core.pxd":45
 *     cpdef _ndarray_base swapaxes(self, Py_ssize_t axis1, Py_ssize_t axis2)
 *     cpdef _ndarray_base flatten(self, order=*)
 *     cpdef _ndarray_base ravel(self, order=*)             # <<<<<<<<<<<<<<
 *     cpdef _ndarray_base squeeze(self, axis=*)
 *     cpdef _ndarray_base take(self, indices, axis=*, out=*)
*/
struct __pyx_opt_args_4cupy_5_core_4core_13_ndarray_base_ravel {
  int __pyx_n;
  PyObject *order;
};

/* "cupy/_core/core.pxd":46
 *     cpdef _ndarray_base flatten(self, order=*)
 *     cpdef _ndarray_base ravel(self, order=*)
 *     cpdef _ndarray_base squeeze(self, axis=*)             # <<<<<<<<<<<<<<
 *     cpdef _ndarray_base take(self, indices, axis=*, out=*)
 *     cpdef put(self, indices, values, mode=*)
*/
struct __pyx_opt_args_4cupy_5_core_4core_13_ndarray_base_squeeze {
  int __pyx_n;
  PyObject *axis;
};

/* "cupy/_core/core.pxd":47
 *     cpdef _ndarray_base ravel(self, order=*)
 *     cpdef _ndarray_base squeeze(self, axis=*)
 *     cpdef _ndarray_base take(self, indices, axis=*, out=*)             # <<<<<<<<<<<<<<
 *     cpdef put(self, indices, values, mode=*)
 *     cpdef repeat(self, repeats, axis=*)
*/
struct __pyx_opt_args_4cupy_5_core_4core_13_ndarray_base_take {
  int __pyx_n;
  PyObject *axis;
  PyObject *out;
};

/* "cupy/_core/core.pxd":48
 *     cpdef _ndarray_base squeeze(self, axis=*)
 *     cpdef _ndarray_base take(self, indices, axis=*, out=*)
 *     cpdef put(self, indices, values, mode=*)             # <<<<<<<<<<<<<<
 *     cpdef repeat(self, repeats, axis=*)
 *     cpdef choose(self, choices, out=*, mode=*)
*/
struct __pyx_opt_args_4cupy_5_core_4core_13_ndarray_base_put {
  int __pyx_n;
  PyObject *mode;
};

/* "cupy/_core/core.pxd":49
 *     cpdef _ndarray_base take(self, indices, axis=*, out=*)
 *     cpdef put(self, indices, values, mode=*)
 *     cpdef repeat(self, repeats, axis=*)             # <<<<<<<<<<<<<<
 *     cpdef choose(self, choices, out=*, mode=*)
 *     cpdef sort(self, int axis=*, kind=*)
*/
struct __pyx_opt_args_4cupy_5_core_4core_13_ndarray_base_repeat {
  int __pyx_n;
  PyObject *axis;
};

/* "cupy/_core/core.pxd":50
 *     cpdef put(self, indices, values, mode=*)
 *     cpdef repeat(self, repeats, axis=*)
 *     cpdef choose(self, choices, out=*, mode=*)             # <<<<<<<<<<<<<<
 *     cpdef sort(self, int axis=*, kind=*)
 *     cpdef _ndarray_base argsort(self, axis=*, kind=*)
*/
struct __pyx_opt_args_4cupy_5_core_4core_13_ndarray_base_choose {
  int __pyx_n;
  PyObject *out;
  PyObject *mode;
};

/* "cupy/_core/core.pxd":51
 *     cpdef repeat(self, repeats, axis=*)
 *     cpdef choose(self, choices, out=*, mode=*)
 *     cpdef sort(self, int axis=*, kind=*)             # <<<<<<<<<<<<<<
 *     cpdef _ndarray_base argsort(self, axis=*, kind=*)
 *     cpdef partition(self, kth, int axis=*)
*/
struct __pyx_opt_args_4cupy_5_core_4core_13_ndarray_base_sort {
  int __pyx_n;
  int axis;
  PyObject *kind;
};

/* "cupy/_core/core.pxd":52
 *     cpdef choose(self, choices, out=*, mode=*)
 *     cpdef sort(self, int axis=*, kind=*)
 *     cpdef _ndarray_base argsort(self, axis=*, kind=*)             # <<<<<<<<<<<<<<
 *     cpdef partition(self, kth, int axis=*)
 *     cpdef _ndarray_base argpartition(self, kth, axis=*)
*/
struct __pyx_opt_args_4cupy_5_core_4core_13_ndarray_base_argsort {
  int __pyx_n;
  PyObject *axis;
  PyObject *kind;
};

/* "cupy/_core/core.pxd":53
 *     cpdef sort(self, int axis=*, kind=*)
 *     cpdef _ndarray_base argsort(self, axis=*, kind=*)
 *     cpdef partition(self, kth, int axis=*)             # <<<<<<<<<<<<<<
 *     cpdef _ndarray_base argpartition(self, kth, axis=*)
 *     cpdef tuple nonzero(self)
*/
struct __pyx_opt_args_4cupy_5_core_4core_13_ndarray_base_partition {
  int __pyx_n;
  int axis;
};

/* "cupy/_core/core.pxd":54
 *     cpdef _ndarray_base argsort(self, axis=*, kind=*)
 *     cpdef partition(self, kth, int axis=*)
 *     cpdef _ndarray_base argpartition(self, kth, axis=*)             # <<<<<<<<<<<<<<
 *     cpdef tuple nonzero(self)
 *     cpdef _ndarray_base compress(self, condition, axis=*, out=*)
*/
struct __pyx_opt_args_4cupy_5_core_4core_13_ndarray_base_argpartition {
  int __pyx_n;
  PyObject *axis;
};

/* "cupy/_core/core.pxd":56
 *     cpdef _ndarray_base argpartition(self, kth, axis=*)
 *     cpdef tuple nonzero(self)
 *     cpdef _ndarray_base compress(self, condition, axis=*, out=*)             # <<<<<<<<<<<<<<
 *     cpdef _ndarray_base diagonal(self, offset=*, axis1=*, axis2=*)
 *     cpdef _ndarray_base max(self, axis=*, out=*, keepdims=*)
*/
struct __pyx_opt_args_4cupy_5_core_4core_13_ndarray_base_compress {
  int __pyx_n;
  PyObject *axis;
  PyObject *out;
};

/* "cupy/_core/core.pxd":57
 *     cpdef tuple nonzero(self)
 *     cpdef _ndarray_base compress(self, condition, axis=*, out=*)
 *     cpdef _ndarray_base diagonal(self, offset=*, axis1=*, axis2=*)             # <<<<<<<<<<<<<<
 *     cpdef _ndarray_base max(self, axis=*, out=*, keepdims=*)
 *     cpdef _ndarray_base argmax(self, axis=*, out=*, dtype=*, keepdims=*)
*/
struct __pyx_opt_args_4cupy_5_core_4core_13_ndarray_base_diagonal {
  int __pyx_n;
  PyObject *offset;
  PyObject *axis1;
  PyObject *axis2;
};

/* "cupy/_core/core.pxd":58
 *     cpdef _ndarray_base compress(self, condition, axis=*, out=*)
 *     cpdef _ndarray_base diagonal(self, offset=*, axis1=*, axis2=*)
 *     cpdef _ndarray_base max(self, axis=*, out=*, keepdims=*)             # <<<<<<<<<<<<<<
 *     cpdef _ndarray_base argmax(self, axis=*, out=*, dtype=*, keepdims=*)
 *     cpdef _ndarray_base min(self, axis=*, out=*, keepdims=*)
*/
struct __pyx_opt_args_4cupy_5_core_4core_13_ndarray_base_max {
  int __pyx_n;
  PyObject *axis;
  PyObject *out;
  PyObject *keepdims;
};

/* "cupy/_core/core.pxd":59
 *     cpdef _ndarray_base diagonal(self, offset=*, axis1=*, axis2=*)
 *     cpdef _ndarray_base max(self, axis=*, out=*, keepdims=*)
 *     cpdef _ndarray_base argmax(self, axis=*, out=*, dtype=*, keepdims=*)             # <<<<<<<<<<<<<<
 *     cpdef _ndarray_base min(self, axis=*, out=*, keepdims=*)
 *     cpdef _ndarray_base argmin(self, axis=*, out=*, dtype=*, keepdims=*)
*/
struct __pyx_opt_args_4cupy_5_core_4core_13_ndarray_base_argmax {
  int __pyx_n;
  PyObject *axis;
  PyObject *out;
  PyObject *dtype;
  PyObject *keepdims;
};

/* "cupy/_core/core.pxd":60
 *     cpdef _ndarray_base max(self, axis=*, out=*, keepdims=*)
 *     cpdef _ndarray_base argmax(self, axis=*, out=*, dtype=*, keepdims=*)
 *     cpdef _ndarray_base min(self, axis=*, out=*, keepdims=*)             # <<<<<<<<<<<<<<
 *     cpdef _ndarray_base argmin(self, axis=*, out=*, dtype=*, keepdims=*)
 *     cpdef _ndarray_base ptp(self, axis=*, out=*, keepdims=*)
*/
struct __pyx_opt_args_4cupy_5_core_4core_13_ndarray_base_min {
  int __pyx_n;
  PyObject *axis;
  PyObject *out;
  PyObject *keepdims;
};

/* "cupy/_core/core.pxd":61
 *     cpdef _ndarray_base argmax(self, axis=*, out=*, dtype=*, keepdims=*)
 *     cpdef _ndarray_base min(self, axis=*, out=*, keepdims=*)
 *     cpdef _ndarray_base argmin(self, axis=*, out=*, dtype=*, keepdims=*)             # <<<<<<<<<<<<<<
 *     cpdef _ndarray_base ptp(self, axis=*, out=*, keepdims=*)
 *     cpdef _ndarray_base clip(self, min=*, max=*, out=*)
*/
struct __pyx_opt_args_4cupy_5_core_4core_13_ndarray_base_argmin {
  int __pyx_n;
  PyObject *axis;
  PyObject *out;
  PyObject *dtype;
  PyObject *keepdims;
};

/* "cupy/_core/core.pxd":62
 *     cpdef _ndarray_base min(self, axis=*, out=*, keepdims=*)
 *     cpdef _ndarray_base argmin(self, axis=*, out=*, dtype=*, keepdims=*)
 *     cpdef _ndarray_base ptp(self, axis=*, out=*, keepdims=*)             # <<<<<<<<<<<<<<
 *     cpdef _ndarray_base clip(self, min=*, max=*, out=*)
 *     cpdef _ndarray_base round(self, decimals=*, out=*)
*/
struct __pyx_opt_args_4cupy_5_core_4core_13_ndarray_base_ptp {
  int __pyx_n;
  PyObject *axis;
  PyObject *out;
  PyObject *keepdims;
};

/* "cupy/_core/core.pxd":63
 *     cpdef _ndarray_base argmin(self, axis=*, out=*, dtype=*, keepdims=*)
 *     cpdef _ndarray_base ptp(self, axis=*, out=*, keepdims=*)
 *     cpdef _ndarray_base clip(self, min=*, max=*, out=*)             # <<<<<<<<<<<<<<
 *     cpdef _ndarray_base round(self, decimals=*, out=*)
 * 
*/
struct __pyx_opt_args_4cupy_5_core_4core_13_ndarray_base_clip {
  int __pyx_n;
  PyObject *min;
  PyObject *max;
  PyObject *out;
};

/* "cupy/_core/core.pxd":64
 *     cpdef _ndarray_base ptp(self, axis=*, out=*, keepdims=*)
 *     cpdef _ndarray_base clip(self, min=*, max=*, out=*)
 *     cpdef _ndarray_base round(self, decimals=*, out=*)             # <<<<<<<<<<<<<<
 * 
 *     cpdef _ndarray_base trace(self, offset=*, axis1=*, axis2=*, dtype=*, out=*)
*/
struct __pyx_opt_args_4cupy_5_core_4core_13_ndarray_base_round {
  int __pyx_n;
  PyObject *decimals;
  PyObject *out;
};

/* "cupy/_core/core.pxd":66
 *     cpdef _ndarray_base round(self, decimals=*, out=*)
 * 
 *     cpdef _ndarray_base trace(self, offset=*, axis1=*, axis2=*, dtype=*, out=*)             # <<<<<<<<<<<<<<
 *     cpdef _ndarray_base sum(self, axis=*, dtype=*, out=*, keepdims=*)
 *     cpdef _ndarray_base cumsum(self, axis=*, dtype=*, out=*)
*/
struct __pyx_opt_args_4cupy_5_core_4core_13_ndarray_base_trace {
  int __pyx_n;
  PyObject *offset;
  PyObject *axis1;
  PyObject *axis2;
  PyObject *dtype;
  PyObject *out;
};

/* "cupy/_core/core.pxd":67
 * 
 *     cpdef _ndarray_base trace(self, offset=*, axis1=*, axis2=*, dtype=*, out=*)
 *     cpdef _ndarray_base sum(self, axis=*, dtype=*, out=*, keepdims=*)             # <<<<<<<<<<<<<<
 *     cpdef _ndarray_base cumsum(self, axis=*, dtype=*, out=*)
 *     cpdef _ndarray_base mean(self, axis=*, dtype=*, out=*, keepdims=*)
*/
struct __pyx_opt_args_4cupy_5_core_4core_13_ndarray_base_sum {
  int __pyx_n;
  PyObject *axis;
  PyObject *dtype;
  PyObject *out;
  PyObject *keepdims;
};

/* "cupy/_core/core.pxd":68
 *     cpdef _ndarray_base trace(self, offset=*, axis1=*, axis2=*, dtype=*, out=*)
 *     cpdef _ndarray_base sum(self, axis=*, dtype=*, out=*, keepdims=*)
 *     cpdef _ndarray_base cumsum(self, axis=*, dtype=*, out=*)             # <<<<<<<<<<<<<<
 *     cpdef _ndarray_base mean(self, axis=*, dtype=*, out=*, keepdims=*)
 *     cpdef _ndarray_base var(self, axis=*, dtype=*, out=*, ddof=*, keepdims=*)
*/
struct __pyx_opt_args_4cupy_5_core_4core_13_ndarray_base_cumsum {
  int __pyx_n;
  PyObject *axis;
  PyObject *dtype;
  PyObject *out;
};

/* "cupy/_core/core.pxd":69
 *     cpdef _ndarray_base sum(self, axis=*, dtype=*, out=*, keepdims=*)
 *     cpdef _ndarray_base cumsum(self, axis=*, dtype=*, out=*)
 *     cpdef _ndarray_base mean(self, axis=*, dtype=*, out=*, keepdims=*)             # <<<<<<<<<<<<<<
 *     cpdef _ndarray_base var(self, axis=*, dtype=*, out=*, ddof=*, keepdims=*)
 *     cpdef _ndarray_base std(self, axis=*, dtype=*, out=*, ddof=*, keepdims=*)
*/
struct __pyx_opt_args_4cupy_5_core_4core_13_ndarray_base_mean {
  int __pyx_n;
  PyObject *axis;
  PyObject *dtype;
  PyObject *out;
  PyObject *keepdims;
};

/* "cupy/_core/core.pxd":70
 *     cpdef _ndarray_base cumsum(self, axis=*, dtype=*, out=*)
 *     cpdef _ndarray_base mean(self, axis=*, dtype=*, out=*, keepdims=*)
 *     cpdef _ndarray_base var(self, axis=*, dtype=*, out=*, ddof=*, keepdims=*)             # <<<<<<<<<<<<<<
 *     cpdef _ndarray_base std(self, axis=*, dtype=*, out=*, ddof=*, keepdims=*)
 *     cpdef _ndarray_base prod(self, axis=*, dtype=*, out=*, keepdims=*)
*/
struct __pyx_opt_args_4cupy_5_core_4core_13_ndarray_base_var {
  int __pyx_n;
  PyObject *axis;
  PyObject *dtype;
  PyObject *out;
  PyObject *ddof;
  PyObject *keepdims;
};

/* "cupy/_core/core.pxd":71
 *     cpdef _ndarray_base mean(self, axis=*, dtype=*, out=*, keepdims=*)
 *     cpdef _ndarray_base var(self, axis=*, dtype=*, out=*, ddof=*, keepdims=*)
 *     cpdef _ndarray_base std(self, axis=*, dtype=*, out=*, ddof=*, keepdims=*)             # <<<<<<<<<<<<<<
 *     cpdef _ndarray_base prod(self, axis=*, dtype=*, out=*, keepdims=*)
 *     cpdef _ndarray_base cumprod(self, axis=*, dtype=*, out=*)
*/
struct __pyx_opt_args_4cupy_5_core_4core_13_ndarray_base_std {
  int __pyx_n;
  PyObject *axis;
  PyObject *dtype;
  PyObject *out;
  PyObject *ddof;
  PyObject *keepdims;
};

/* "cupy/_core/core.pxd":72
 *     cpdef _ndarray_base var(self, axis=*, dtype=*, out=*, ddof=*, keepdims=*)
 *     cpdef _ndarray_base std(self, axis=*, dtype=*, out=*, ddof=*, keepdims=*)
 *     cpdef _ndarray_base prod(self, axis=*, dtype=*, out=*, keepdims=*)             # <<<<<<<<<<<<<<
 *     cpdef _ndarray_base cumprod(self, axis=*, dtype=*, out=*)
 *     cpdef _ndarray_base _add_reduceat(self, indices, axis, dtype, out)
*/
struct __pyx_opt_args_4cupy_5_core_4core_13_ndarray_base_prod {
  int __pyx_n;
  PyObject *axis;
  PyObject *dtype;
  PyObject *out;
  PyObject *keepdims;
};

/* "cupy/_core/core.pxd":73
 *     cpdef _ndarray_base std(self, axis=*, dtype=*, out=*, ddof=*, keepdims=*)
 *     cpdef _ndarray_base prod(self, axis=*, dtype=*, out=*, keepdims=*)
 *     cpdef _ndarray_base cumprod(self, axis=*, dtype=*, out=*)             # <<<<<<<<<<<<<<
 *     cpdef _ndarray_base _add_reduceat(self, indices, axis, dtype, out)
 *     cpdef _ndarray_base all(self, axis=*, out=*, keepdims=*)
*/
struct __pyx_opt_args_4cupy_5_core_4core_13_ndarray_base_cumprod {
  int __pyx_n;
  PyObject *axis;
  PyObject *dtype;
  PyObject *out;
};

/* "cupy/_core/core.pxd":75
 *     cpdef _ndarray_base cumprod(self, axis=*, dtype=*, out=*)
 *     cpdef _ndarray_base _add_reduceat(self, indices, axis, dtype, out)
 *     cpdef _ndarray_base all(self, axis=*, out=*, keepdims=*)             # <<<<<<<<<<<<<<
 *     cpdef _ndarray_base any(self, axis=*, out=*, keepdims=*)
 *     cpdef _ndarray_base conj(self)
*/
struct __pyx_opt_args_4cupy_5_core_4core_13_ndarray_base_all {
  int __pyx_n;
  PyObject *axis;
  PyObject *out;
  PyObject *keepdims;
};

/* "cupy/_core/core.pxd":76
 *     cpdef _ndarray_base _add_reduceat(self, indices, axis, dtype, out)
 *     cpdef _ndarray_base all(self, axis=*, out=*, keepdims=*)
 *     cpdef _ndarray_base any(self, axis=*, out=*, keepdims=*)             # <<<<<<<<<<<<<<
 *     cpdef _ndarray_base conj(self)
 *     cpdef _ndarray_base conjugate(self)
*/
struct __pyx_opt_args_4cupy_5_core_4core_13_ndarray_base_any {
  int __pyx_n;
  PyObject *axis;
  PyObject *out;
  PyObject *keepdims;
};

/* "cupy/_core/core.pxd":79
 *     cpdef _ndarray_base conj(self)
 *     cpdef _ndarray_base conjugate(self)
 *     cpdef get(self, stream=*, order=*, out=*, blocking=*)             # <<<<<<<<<<<<<<
 *     cpdef set(self, arr, stream=*)
 *     cpdef _ndarray_base reduced_view(self, dtype=*)
*/
struct __pyx_opt_args_4cupy_5_core_4core_13_ndarray_base_get {
  int __pyx_n;
  PyObject *stream;
  PyObject *order;
  PyObject *out;
  PyObject *blocking;
};

/* "cupy/_core/core.pxd":80
 *     cpdef _ndarray_base conjugate(self)
 *     cpdef get(self, stream=*, order=*, out=*, blocking=*)
 *     cpdef set(self, arr, stream=*)             # <<<<<<<<<<<<<<
 *     cpdef _ndarray_base reduced_view(self, dtype=*)
 *     cdef _update_c_contiguity(self)
*/
struct __pyx_opt_args_4cupy_5_core_4core_13_ndarray_base_set {
  int __pyx_n;
  PyObject *stream;
};

/* "cupy/_core/core.pxd":81
 *     cpdef get(self, stream=*, order=*, out=*, blocking=*)
 *     cpdef set(self, arr, stream=*)
 *     cpdef _ndarray_base reduced_view(self, dtype=*)             # <<<<<<<<<<<<<<
 *     cdef _update_c_contiguity(self)
 *     cdef _update_f_contiguity(self)
*/
struct __pyx_opt_args_4cupy_5_core_4core_13_ndarray_base_reduced_view {
  int __pyx_n;
  PyObject *dtype;
};

/* "cupy/_core/core.pxd":101
 * cpdef _ndarray_base _internal_ascontiguousarray(_ndarray_base a)
 * cpdef _ndarray_base _internal_asfortranarray(_ndarray_base a)
 * cpdef _ndarray_base ascontiguousarray(_ndarray_base a, dtype=*)             # <<<<<<<<<<<<<<
 * cpdef _ndarray_base asfortranarray(_ndarray_base a, dtype=*)
 * 
*/
struct __pyx_opt_args_4cupy_5_core_4core_ascontiguousarray {
  int __pyx_n;
  PyObject *dtype;
};

/* "cupy/_core/core.pxd":102
 * cpdef _ndarray_base _internal_asfortranarray(_ndarray_base a)
 * cpdef _ndarray_base ascontiguousarray(_ndarray_base a, dtype=*)
 * cpdef _ndarray_base asfortranarray(_ndarray_base a, dtype=*)             # <<<<<<<<<<<<<<
 * 
 * cpdef Module compile_with_cache(str source, tuple options=*, arch=*,
*/
struct __pyx_opt_args_4cupy_5_core_4core_asfortranarray {
  int __pyx_n;
  PyObject *dtype;
};

/* "cupy/_core/core.pxd":104
 * cpdef _ndarray_base asfortranarray(_ndarray_base a, dtype=*)
 * 
 * cpdef Module compile_with_cache(str source, tuple options=*, arch=*,             # <<<<<<<<<<<<<<
 *                                 prepend_cupy_headers=*,
 *                                 backend=*, translate_cucomplex=*,
*/
struct __pyx_opt_args_4cupy_5_core_4core_compile_with_cache {
  int __pyx_n;
  PyObject *options;
  PyObject *arch;
  PyObject *prepend_cupy_headers;
  PyObject *backend;
  PyObject *translate_cucomplex;
  PyObject *enable_cooperative_groups;
  PyObject *name_expressions;
  PyObject *log_stream;
  int jitify;
};

/* "cupy/_core/core.pxd":114
 * # TODO(niboshi): Move to _routines_creation.pyx
 * 
 * cpdef _ndarray_base array(             # <<<<<<<<<<<<<<
 *     obj, dtype=*, copy=*, order=*, bint subok=*, Py_ssize_t ndmin=*,
 *     bint blocking=*)
*/
struct __pyx_opt_args_4cupy_5_core_4core_array {
  int __pyx_n;
  PyObject *dtype;
  PyObject *copy;
  PyObject *order;
  int subok;
  Py_ssize_t ndmin;
  int blocking;
};

/* "cupyx/cusolver.pyx":60
 *         int batch_size, int origin_n)
 * 
 * ctypedef int(*gesvd_ptr)(intptr_t, char, char, int, int, intptr_t,             # <<<<<<<<<<<<<<
 *                          intptr_t, intptr_t, intptr_t,
 *                          intptr_t, int, intptr_t, int) noexcept nogil
*/
typedef int (*__pyx_t_5cupyx_8cusolver_gesvd_ptr)(intptr_t, char, char, int, int, intptr_t, intptr_t, intptr_t, intptr_t, intptr_t, int, intptr_t, int);

/* "cupyx/cusolver.pyx":63
 *                          intptr_t, intptr_t, intptr_t,
 *                          intptr_t, int, intptr_t, int) noexcept nogil
 * ctypedef int(*geqrf_ptr)(intptr_t, int, int, intptr_t, int, intptr_t,             # <<<<<<<<<<<<<<
 *                          intptr_t, int, intptr_t, int) noexcept nogil
 * ctypedef int(*orgqr_ptr)(intptr_t, int, int, int, intptr_t, int, intptr_t,
*/
typedef int (*__pyx_t_5cupyx_8cusolver_geqrf_ptr)(intptr_t, int, int, intptr_t, int, intptr_t, intptr_t, int, intptr_t, int);

/* "cupyx/cusolver.pyx":65
 * ctypedef int(*geqrf_ptr)(intptr_t, int, int, intptr_t, int, intptr_t,
 *                          intptr_t, int, intptr_t, int) noexcept nogil
 * ctypedef int(*orgqr_ptr)(intptr_t, int, int, int, intptr_t, int, intptr_t,             # <<<<<<<<<<<<<<
 *                          intptr_t, int, intptr_t, int, int) noexcept nogil
 * 
*/
typedef int (*__pyx_t_5cupyx_8cusolver_orgqr_ptr)(intptr_t, int, int, int, intptr_t, int, intptr_t, intptr_t, int, intptr_t, int, int);

/* "runtime.pxd":8
 * ###############################################################################
 * 
 * cdef class PointerAttributes:             # <<<<<<<<<<<<<<
 *     cdef:
 *         readonly int device
*/
struct __pyx_obj_13cupy_backends_4cuda_3api_7runtime_PointerAttributes {
  PyObject_HEAD
  int device;
  intptr_t devicePointer;
  intptr_t hostPointer;
  int type;
};


/* "runtime.pxd":15
 *         readonly int type
 * 
 * cdef class MemPoolProps:             # <<<<<<<<<<<<<<
 *     # flatten the struct & list meaningful members
 *     cdef:
*/
struct __pyx_obj_13cupy_backends_4cuda_3api_7runtime_MemPoolProps {
  PyObject_HEAD
  int allocType;
  int handleType;
  int locationType;
  int devId;
};


/* "device.pxd":13
 * cdef bint _enable_peer_access(int device, int peer) except -1
 * 
 * cdef class Handle:             # <<<<<<<<<<<<<<
 *     cdef:
 *         public size_t handle
*/
struct __pyx_obj_4cupy_4cuda_6device_Handle {
  PyObject_HEAD
  size_t handle;
  PyObject *_destroy_func;
};


/* "device.pxd":18
 *         object _destroy_func
 * 
 * cdef class Device:             # <<<<<<<<<<<<<<
 *     cdef:
 *         public int id
*/
struct __pyx_obj_4cupy_4cuda_6device_Device {
  PyObject_HEAD
  struct __pyx_vtabstruct_4cupy_4cuda_6device_Device *__pyx_vtab;
  int id;
  PyObject *_device_stack;
};


/* "memory.pxd":10
 * 
 * 
 * @cython.no_gc             # <<<<<<<<<<<<<<
 * cdef class BaseMemory:
 * 
*/
struct __pyx_obj_4cupy_4cuda_6memory_BaseMemory {
  PyObject_HEAD
  intptr_t ptr;
  size_t size;
  int device_id;
};


/* "memory.pxd":19
 * 
 * 
 * @cython.no_gc             # <<<<<<<<<<<<<<
 * cdef class SystemMemory(BaseMemory):
 * 
*/
struct __pyx_obj_4cupy_4cuda_6memory_SystemMemory {
  struct __pyx_obj_4cupy_4cuda_6memory_BaseMemory __pyx_base;
  struct __pyx_vtabstruct_4cupy_4cuda_6memory_SystemMemory *__pyx_vtab;
  PyObject *_owner;
};


/* "memory.pxd":29
 * 
 * 
 * cdef class MemoryPointer:             # <<<<<<<<<<<<<<
 * 
 *     cdef:
*/
struct __pyx_obj_4cupy_4cuda_6memory_MemoryPointer {
  PyObject_HEAD
  struct __pyx_vtabstruct_4cupy_4cuda_6memory_MemoryPointer *__pyx_vtab;
  intptr_t ptr;
  int device_id;
  struct __pyx_obj_4cupy_4cuda_6memory_BaseMemory *mem;
};


/* "memory.pxd":58
 * 
 * 
 * cdef class MemoryPool:             # <<<<<<<<<<<<<<
 * 
 *     cdef:
*/
struct __pyx_obj_4cupy_4cuda_6memory_MemoryPool {
  PyObject_HEAD
  struct __pyx_vtabstruct_4cupy_4cuda_6memory_MemoryPool *__pyx_vtab;
  PyObject *_pools;
  PyObject *_allocator;
};


/* "memory.pxd":89
 * 
 * 
 * @cython.no_gc             # <<<<<<<<<<<<<<
 * cdef class CFunctionAllocatorMemory(BaseMemory):
 * 
*/
struct __pyx_obj_4cupy_4cuda_6memory_CFunctionAllocatorMemory {
  struct __pyx_obj_4cupy_4cuda_6memory_BaseMemory __pyx_base;
  intptr_t _param;
  intptr_t _free_func;
};


/* "memory.pxd":97
 * 
 * 
 * cdef class CFunctionAllocator:             # <<<<<<<<<<<<<<
 * 
 *     cdef:
*/
struct __pyx_obj_4cupy_4cuda_6memory_CFunctionAllocator {
  PyObject_HEAD
  struct __pyx_vtabstruct_4cupy_4cuda_6memory_CFunctionAllocator *__pyx_vtab;
  intptr_t _param;
  intptr_t _malloc_func;
  intptr_t _free_func;
  PyObject *_owner;
};


/* "memory.pxd":108
 * 
 * 
 * cdef class PythonFunctionAllocatorMemory(BaseMemory):             # <<<<<<<<<<<<<<
 * 
 *     cdef:
*/
struct __pyx_obj_4cupy_4cuda_6memory_PythonFunctionAllocatorMemory {
  struct __pyx_obj_4cupy_4cuda_6memory_BaseMemory __pyx_base;
  PyObject *_free_func;
};


/* "memory.pxd":114
 * 
 * 
 * cdef class PythonFunctionAllocator:             # <<<<<<<<<<<<<<
 * 
 *     cdef:
*/
struct __pyx_obj_4cupy_4cuda_6memory_PythonFunctionAllocator {
  PyObject_HEAD
  struct __pyx_vtabstruct_4cupy_4cuda_6memory_PythonFunctionAllocator *__pyx_vtab;
  PyObject *_malloc_func;
  PyObject *_free_func;
};


/* "cupy/cuda/function.pxd":4
 * 
 * 
 * cdef class CPointer:             # <<<<<<<<<<<<<<
 *     cdef:
 *         readonly intptr_t ptr
*/
struct __pyx_obj_4cupy_4cuda_8function_CPointer {
  PyObject_HEAD
  intptr_t ptr;
};


/* "cupy/cuda/function.pxd":9
 * 
 * 
 * cdef class Function:             # <<<<<<<<<<<<<<
 * 
 *     cdef:
*/
struct __pyx_obj_4cupy_4cuda_8function_Function {
  PyObject_HEAD
  struct __pyx_vtabstruct_4cupy_4cuda_8function_Function *__pyx_vtab;
  struct __pyx_obj_4cupy_4cuda_8function_Module *module;
  intptr_t ptr;
};


/* "cupy/cuda/function.pxd":20
 * 
 * 
 * cdef class Module:             # <<<<<<<<<<<<<<
 * 
 *     cdef:
*/
struct __pyx_obj_4cupy_4cuda_8function_Module {
  PyObject_HEAD
  struct __pyx_vtabstruct_4cupy_4cuda_8function_Module *__pyx_vtab;
  intptr_t ptr;
  PyObject *mapping;
};


/* "cupy/cuda/function.pxd":34
 * 
 * 
 * cdef class LinkState:             # <<<<<<<<<<<<<<
 * 
 *     cdef:
*/
struct __pyx_obj_4cupy_4cuda_8function_LinkState {
  PyObject_HEAD
  struct __pyx_vtabstruct_4cupy_4cuda_8function_LinkState *__pyx_vtab;
  intptr_t ptr;
};


/* "cupy/_core/_carray.pxd":22
 * 
 * 
 * @cython.final             # <<<<<<<<<<<<<<
 * cdef class mdspan(function.CPointer):
 * 
*/
struct __pyx_obj_4cupy_5_core_7_carray_mdspan {
  struct __pyx_obj_4cupy_4cuda_8function_CPointer __pyx_base;
  struct __pyx_vtabstruct_4cupy_5_core_7_carray_mdspan *__pyx_vtab;
};


/* "cupy/_core/_carray.pxd":31
 * 
 * 
 * @cython.final             # <<<<<<<<<<<<<<
 * cdef class CArray(function.CPointer):
 * 
*/
struct __pyx_obj_4cupy_5_core_7_carray_CArray {
  struct __pyx_obj_4cupy_4cuda_8function_CPointer __pyx_base;
  struct __pyx_vtabstruct_4cupy_5_core_7_carray_CArray *__pyx_vtab;
};


/* "cupy/_core/_carray.pxd":44
 * 
 * 
 * cdef class CIndexer(function.CPointer):             # <<<<<<<<<<<<<<
 * 
 *     cdef void init(self, Py_ssize_t size, const shape_t &shape) except*
*/
struct __pyx_obj_4cupy_5_core_7_carray_CIndexer {
  struct __pyx_obj_4cupy_4cuda_8function_CPointer __pyx_base;
  struct __pyx_vtabstruct_4cupy_5_core_7_carray_CIndexer *__pyx_vtab;
};


/* "cupy/_core/_carray.pxd":49
 * 
 * 
 * cdef class Indexer:             # <<<<<<<<<<<<<<
 *     cdef:
 *         readonly Py_ssize_t size
*/
struct __pyx_obj_4cupy_5_core_7_carray_Indexer {
  PyObject_HEAD
  struct __pyx_vtabstruct_4cupy_5_core_7_carray_Indexer *__pyx_vtab;
  Py_ssize_t size;
  __pyx_t_4cupy_5_core_7_carray_shape_t shape;
  int _index_32_bits;
};


/* "cupy/_core/core.pxd":10
 * 
 * 
 * cdef class _ndarray_base:             # <<<<<<<<<<<<<<
 *     cdef:
 *         object __weakref__
*/
struct __pyx_obj_4cupy_5_core_4core__ndarray_base {
  PyObject_HEAD
  struct __pyx_vtabstruct_4cupy_5_core_4core__ndarray_base *__pyx_vtab;
  PyObject *__weakref__;
  Py_ssize_t size;
  __pyx_t_4cupy_5_core_7_carray_shape_t _shape;
  __pyx_t_4cupy_5_core_7_carray_strides_t _strides;
  int _c_contiguous;
  int _f_contiguous;
  int _index_32_bits;
  PyObject *dtype;
  struct __pyx_obj_4cupy_4cuda_6memory_MemoryPointer *data;
  struct __pyx_obj_4cupy_5_core_4core__ndarray_base *base;
};



/* "device.pxd":18
 *         object _destroy_func
 * 
 * cdef class Device:             # <<<<<<<<<<<<<<
 *     cdef:
 *         public int id
*/

struct __pyx_vtabstruct_4cupy_4cuda_6device_Device {
  PyObject *(*use)(struct __pyx_obj_4cupy_4cuda_6device_Device *, int __pyx_skip_dispatch);
  PyObject *(*synchronize)(struct __pyx_obj_4cupy_4cuda_6device_Device *, int __pyx_skip_dispatch);
};
static struct __pyx_vtabstruct_4cupy_4cuda_6device_Device *__pyx_vtabptr_4cupy_4cuda_6device_Device;


/* "memory.pxd":19
 * 
 * 
 * @cython.no_gc             # <<<<<<<<<<<<<<
 * cdef class SystemMemory(BaseMemory):
 * 
*/

struct __pyx_vtabstruct_4cupy_4cuda_6memory_SystemMemory {
  PyObject *(*from_external)(intptr_t, size_t, PyObject *);
};
static struct __pyx_vtabstruct_4cupy_4cuda_6memory_SystemMemory *__pyx_vtabptr_4cupy_4cuda_6memory_SystemMemory;


/* "memory.pxd":29
 * 
 * 
 * cdef class MemoryPointer:             # <<<<<<<<<<<<<<
 * 
 *     cdef:
*/

struct __pyx_vtabstruct_4cupy_4cuda_6memory_MemoryPointer {
  PyObject *(*_init)(struct __pyx_obj_4cupy_4cuda_6memory_MemoryPointer *, struct __pyx_obj_4cupy_4cuda_6memory_BaseMemory *, ptrdiff_t);
  PyObject *(*copy_from_device)(struct __pyx_obj_4cupy_4cuda_6memory_MemoryPointer *, struct __pyx_obj_4cupy_4cuda_6memory_MemoryPointer *, size_t, int __pyx_skip_dispatch);
  PyObject *(*copy_from_device_async)(struct __pyx_obj_4cupy_4cuda_6memory_MemoryPointer *, struct __pyx_obj_4cupy_4cuda_6memory_MemoryPointer *, size_t, int __pyx_skip_dispatch, struct __pyx_opt_args_4cupy_4cuda_6memory_13MemoryPointer_copy_from_device_async *__pyx_optional_args);
  PyObject *(*copy_from_host)(struct __pyx_obj_4cupy_4cuda_6memory_MemoryPointer *, PyObject *, size_t, int __pyx_skip_dispatch);
  PyObject *(*copy_from_host_async)(struct __pyx_obj_4cupy_4cuda_6memory_MemoryPointer *, PyObject *, size_t, int __pyx_skip_dispatch, struct __pyx_opt_args_4cupy_4cuda_6memory_13MemoryPointer_copy_from_host_async *__pyx_optional_args);
  PyObject *(*copy_from)(struct __pyx_obj_4cupy_4cuda_6memory_MemoryPointer *, PyObject *, size_t, int __pyx_skip_dispatch);
  PyObject *(*copy_from_async)(struct __pyx_obj_4cupy_4cuda_6memory_MemoryPointer *, PyObject *, size_t, int __pyx_skip_dispatch, struct __pyx_opt_args_4cupy_4cuda_6memory_13MemoryPointer_copy_from_async *__pyx_optional_args);
  PyObject *(*copy_to_host)(struct __pyx_obj_4cupy_4cuda_6memory_MemoryPointer *, PyObject *, size_t, int __pyx_skip_dispatch);
  PyObject *(*copy_to_host_async)(struct __pyx_obj_4cupy_4cuda_6memory_MemoryPointer *, PyObject *, size_t, int __pyx_skip_dispatch, struct __pyx_opt_args_4cupy_4cuda_6memory_13MemoryPointer_copy_to_host_async *__pyx_optional_args);
  PyObject *(*memset)(struct __pyx_obj_4cupy_4cuda_6memory_MemoryPointer *, int, size_t, int __pyx_skip_dispatch);
  PyObject *(*memset_async)(struct __pyx_obj_4cupy_4cuda_6memory_MemoryPointer *, int, size_t, int __pyx_skip_dispatch, struct __pyx_opt_args_4cupy_4cuda_6memory_13MemoryPointer_memset_async *__pyx_optional_args);
};
static struct __pyx_vtabstruct_4cupy_4cuda_6memory_MemoryPointer *__pyx_vtabptr_4cupy_4cuda_6memory_MemoryPointer;


/* "memory.pxd":58
 * 
 * 
 * cdef class MemoryPool:             # <<<<<<<<<<<<<<
 * 
 *     cdef:
*/

struct __pyx_vtabstruct_4cupy_4cuda_6memory_MemoryPool {
  struct __pyx_obj_4cupy_4cuda_6memory_MemoryPointer *(*malloc)(struct __pyx_obj_4cupy_4cuda_6memory_MemoryPool *, size_t, int __pyx_skip_dispatch);
  PyObject *(*free_all_blocks)(struct __pyx_obj_4cupy_4cuda_6memory_MemoryPool *, int __pyx_skip_dispatch, struct __pyx_opt_args_4cupy_4cuda_6memory_10MemoryPool_free_all_blocks *__pyx_optional_args);
  PyObject *(*free_all_free)(struct __pyx_obj_4cupy_4cuda_6memory_MemoryPool *, int __pyx_skip_dispatch);
  size_t (*n_free_blocks)(struct __pyx_obj_4cupy_4cuda_6memory_MemoryPool *, int __pyx_skip_dispatch);
  size_t (*used_bytes)(struct __pyx_obj_4cupy_4cuda_6memory_MemoryPool *, int __pyx_skip_dispatch);
  size_t (*free_bytes)(struct __pyx_obj_4cupy_4cuda_6memory_MemoryPool *, int __pyx_skip_dispatch);
  size_t (*total_bytes)(struct __pyx_obj_4cupy_4cuda_6memory_MemoryPool *, int __pyx_skip_dispatch);
  PyObject *(*set_limit)(struct __pyx_obj_4cupy_4cuda_6memory_MemoryPool *, int __pyx_skip_dispatch, struct __pyx_opt_args_4cupy_4cuda_6memory_10MemoryPool_set_limit *__pyx_optional_args);
  size_t (*get_limit)(struct __pyx_obj_4cupy_4cuda_6memory_MemoryPool *, int __pyx_skip_dispatch);
  PyObject *(*_ensure_pools_and_return_device_pool)(struct __pyx_obj_4cupy_4cuda_6memory_MemoryPool *);
  PyObject *(*device_pool)(struct __pyx_obj_4cupy_4cuda_6memory_MemoryPool *);
};
static struct __pyx_vtabstruct_4cupy_4cuda_6memory_MemoryPool *__pyx_vtabptr_4cupy_4cuda_6memory_MemoryPool;
static CYTHON_INLINE PyObject *__pyx_f_4cupy_4cuda_6memory_10MemoryPool_device_pool(struct __pyx_obj_4cupy_4cuda_6memory_MemoryPool *);


/* "memory.pxd":97
 * 
 * 
 * cdef class CFunctionAllocator:             # <<<<<<<<<<<<<<
 * 
 *     cdef:
*/

struct __pyx_vtabstruct_4cupy_4cuda_6memory_CFunctionAllocator {
  struct __pyx_obj_4cupy_4cuda_6memory_MemoryPointer *(*malloc)(struct __pyx_obj_4cupy_4cuda_6memory_CFunctionAllocator *, size_t, int __pyx_skip_dispatch);
};
static struct __pyx_vtabstruct_4cupy_4cuda_6memory_CFunctionAllocator *__pyx_vtabptr_4cupy_4cuda_6memory_CFunctionAllocator;


/* "memory.pxd":114
 * 
 * 
 * cdef class PythonFunctionAllocator:             # <<<<<<<<<<<<<<
 * 
 *     cdef:
*/

struct __pyx_vtabstruct_4cupy_4cuda_6memory_PythonFunctionAllocator {
  struct __pyx_obj_4cupy_4cuda_6memory_MemoryPointer *(*malloc)(struct __pyx_obj_4cupy_4cuda_6memory_PythonFunctionAllocator *, size_t, int __pyx_skip_dispatch);
};
static struct __pyx_vtabstruct_4cupy_4cuda_6memory_PythonFunctionAllocator *__pyx_vtabptr_4cupy_4cuda_6memory_PythonFunctionAllocator;


/* "cupy/cuda/function.pxd":9
 * 
 * 
 * cdef class Function:             # <<<<<<<<<<<<<<
 * 
 *     cdef:
*/

struct __pyx_vtabstruct_4cupy_4cuda_8function_Function {
  PyObject *(*linear_launch)(struct __pyx_obj_4cupy_4cuda_8function_Function *, size_t, PyObject *, int __pyx_skip_dispatch, struct __pyx_opt_args_4cupy_4cuda_8function_8Function_linear_launch *__pyx_optional_args);
};
static struct __pyx_vtabstruct_4cupy_4cuda_8function_Function *__pyx_vtabptr_4cupy_4cuda_8function_Function;


/* "cupy/cuda/function.pxd":20
 * 
 * 
 * cdef class Module:             # <<<<<<<<<<<<<<
 * 
 *     cdef:
*/

struct __pyx_vtabstruct_4cupy_4cuda_8function_Module {
  PyObject *(*load_file)(struct __pyx_obj_4cupy_4cuda_8function_Module *, PyObject *, int __pyx_skip_dispatch);
  PyObject *(*load)(struct __pyx_obj_4cupy_4cuda_8function_Module *, PyObject *, int __pyx_skip_dispatch);
  PyObject *(*get_global_var)(struct __pyx_obj_4cupy_4cuda_8function_Module *, PyObject *, int __pyx_skip_dispatch);
  PyObject *(*get_function)(struct __pyx_obj_4cupy_4cuda_8function_Module *, PyObject *, int __pyx_skip_dispatch);
  PyObject *(*_set_mapping)(struct __pyx_obj_4cupy_4cuda_8function_Module *, PyObject *, int __pyx_skip_dispatch);
  PyObject *(*_enumerate_and_build_mapping)(struct __pyx_obj_4cupy_4cuda_8function_Module *, PyObject *, int __pyx_skip_dispatch);
};
static struct __pyx_vtabstruct_4cupy_4cuda_8function_Module *__pyx_vtabptr_4cupy_4cuda_8function_Module;


/* "cupy/cuda/function.pxd":34
 * 
 * 
 * cdef class LinkState:             # <<<<<<<<<<<<<<
 * 
 *     cdef:
*/

struct __pyx_vtabstruct_4cupy_4cuda_8function_LinkState {
  PyObject *(*add_ptr_data)(struct __pyx_obj_4cupy_4cuda_8function_LinkState *, PyObject *, PyObject *, int __pyx_skip_dispatch);
  PyObject *(*add_ptr_file)(struct __pyx_obj_4cupy_4cuda_8function_LinkState *, PyObject *, int __pyx_skip_dispatch);
  PyObject *(*complete)(struct __pyx_obj_4cupy_4cuda_8function_LinkState *, int __pyx_skip_dispatch);
};
static struct __pyx_vtabstruct_4cupy_4cuda_8function_LinkState *__pyx_vtabptr_4cupy_4cuda_8function_LinkState;


/* "cupy/_core/_carray.pxd":22
 * 
 * 
 * @cython.final             # <<<<<<<<<<<<<<
 * cdef class mdspan(function.CPointer):
 * 
*/

struct __pyx_vtabstruct_4cupy_5_core_7_carray_mdspan {
  int (*init)(struct __pyx_obj_4cupy_5_core_7_carray_mdspan *, void *, int, __pyx_t_4cupy_5_core_7_carray_shape_t const &, __pyx_t_4cupy_5_core_7_carray_strides_t const &, int, int);
};
static struct __pyx_vtabstruct_4cupy_5_core_7_carray_mdspan *__pyx_vtabptr_4cupy_5_core_7_carray_mdspan;


/* "cupy/_core/_carray.pxd":31
 * 
 * 
 * @cython.final             # <<<<<<<<<<<<<<
 * cdef class CArray(function.CPointer):
 * 
*/

struct __pyx_vtabstruct_4cupy_5_core_7_carray_CArray {
  void (*init)(struct __pyx_obj_4cupy_5_core_7_carray_CArray *, void *, Py_ssize_t, __pyx_t_4cupy_5_core_7_carray_shape_t const &, __pyx_t_4cupy_5_core_7_carray_strides_t const &);
};
static struct __pyx_vtabstruct_4cupy_5_core_7_carray_CArray *__pyx_vtabptr_4cupy_5_core_7_carray_CArray;


/* "cupy/_core/_carray.pxd":44
 * 
 * 
 * cdef class CIndexer(function.CPointer):             # <<<<<<<<<<<<<<
 * 
 *     cdef void init(self, Py_ssize_t size, const shape_t &shape) except*
*/

struct __pyx_vtabstruct_4cupy_5_core_7_carray_CIndexer {
  void (*init)(struct __pyx_obj_4cupy_5_core_7_carray_CIndexer *, Py_ssize_t, __pyx_t_4cupy_5_core_7_carray_shape_t const &);
};
static struct __pyx_vtabstruct_4cupy_5_core_7_carray_CIndexer *__pyx_vtabptr_4cupy_5_core_7_carray_CIndexer;


/* "cupy/_core/_carray.pxd":49
 * 
 * 
 * cdef class Indexer:             # <<<<<<<<<<<<<<
 *     cdef:
 *         readonly Py_ssize_t size
*/

struct __pyx_vtabstruct_4cupy_5_core_7_carray_Indexer {
  void (*init)(struct __pyx_obj_4cupy_5_core_7_carray_Indexer *, __pyx_t_4cupy_5_core_7_carray_shape_t const &);
  struct __pyx_obj_4cupy_4cuda_8function_CPointer *(*get_pointer)(struct __pyx_obj_4cupy_5_core_7_carray_Indexer *);
};
static struct __pyx_vtabstruct_4cupy_5_core_7_carray_Indexer *__pyx_vtabptr_4cupy_5_core_7_carray_Indexer;


/* "cupy/_core/core.pxd":10
 * 
 * 
 * cdef class _ndarray_base:             # <<<<<<<<<<<<<<
 *     cdef:
 *         object __weakref__
*/

struct __pyx_vtabstruct_4cupy_5_core_4core__ndarray_base {
  PyObject *(*_init_fast)(struct __pyx_obj_4cupy_5_core_4core__ndarray_base *, __pyx_t_4cupy_5_core_7_carray_shape_t const &, PyObject *, int);
  int (*is_host_accessible)(struct __pyx_obj_4cupy_5_core_4core__ndarray_base *);
  PyObject *(*item)(struct __pyx_obj_4cupy_5_core_4core__ndarray_base *, int __pyx_skip_dispatch);
  PyObject *(*tolist)(struct __pyx_obj_4cupy_5_core_4core__ndarray_base *, int __pyx_skip_dispatch);
  PyObject *(*tobytes)(struct __pyx_obj_4cupy_5_core_4core__ndarray_base *, int __pyx_skip_dispatch, struct __pyx_opt_args_4cupy_5_core_4core_13_ndarray_base_tobytes *__pyx_optional_args);
  PyObject *(*tofile)(struct __pyx_obj_4cupy_5_core_4core__ndarray_base *, PyObject *, int __pyx_skip_dispatch, struct __pyx_opt_args_4cupy_5_core_4core_13_ndarray_base_tofile *__pyx_optional_args);
  PyObject *(*dump)(struct __pyx_obj_4cupy_5_core_4core__ndarray_base *, PyObject *, int __pyx_skip_dispatch);
  PyObject *(*dumps)(struct __pyx_obj_4cupy_5_core_4core__ndarray_base *, int __pyx_skip_dispatch);
  struct __pyx_obj_4cupy_5_core_4core__ndarray_base *(*_astype)(struct __pyx_obj_4cupy_5_core_4core__ndarray_base *, PyObject *, int __pyx_skip_dispatch, struct __pyx_opt_args_4cupy_5_core_4core_13_ndarray_base__astype *__pyx_optional_args);
  struct __pyx_obj_4cupy_5_core_4core__ndarray_base *(*astype)(struct __pyx_obj_4cupy_5_core_4core__ndarray_base *, PyObject *, int __pyx_skip_dispatch, struct __pyx_opt_args_4cupy_5_core_4core_13_ndarray_base_astype *__pyx_optional_args);
  struct __pyx_obj_4cupy_5_core_4core__ndarray_base *(*byteswap)(struct __pyx_obj_4cupy_5_core_4core__ndarray_base *, int __pyx_skip_dispatch, struct __pyx_opt_args_4cupy_5_core_4core_13_ndarray_base_byteswap *__pyx_optional_args);
  struct __pyx_obj_4cupy_5_core_4core__ndarray_base *(*copy)(struct __pyx_obj_4cupy_5_core_4core__ndarray_base *, int __pyx_skip_dispatch, struct __pyx_opt_args_4cupy_5_core_4core_13_ndarray_base_copy *__pyx_optional_args);
  struct __pyx_obj_4cupy_5_core_4core__ndarray_base *(*view)(struct __pyx_obj_4cupy_5_core_4core__ndarray_base *, int __pyx_skip_dispatch, struct __pyx_opt_args_4cupy_5_core_4core_13_ndarray_base_view *__pyx_optional_args);
  PyObject *(*fill)(struct __pyx_obj_4cupy_5_core_4core__ndarray_base *, PyObject *, int __pyx_skip_dispatch);
  struct __pyx_obj_4cupy_5_core_4core__ndarray_base *(*swapaxes)(struct __pyx_obj_4cupy_5_core_4core__ndarray_base *, Py_ssize_t, Py_ssize_t, int __pyx_skip_dispatch);
  struct __pyx_obj_4cupy_5_core_4core__ndarray_base *(*flatten)(struct __pyx_obj_4cupy_5_core_4core__ndarray_base *, int __pyx_skip_dispatch, struct __pyx_opt_args_4cupy_5_core_4core_13_ndarray_base_flatten *__pyx_optional_args);
  struct __pyx_obj_4cupy_5_core_4core__ndarray_base *(*ravel)(struct __pyx_obj_4cupy_5_core_4core__ndarray_base *, int __pyx_skip_dispatch, struct __pyx_opt_args_4cupy_5_core_4core_13_ndarray_base_ravel *__pyx_optional_args);
  struct __pyx_obj_4cupy_5_core_4core__ndarray_base *(*squeeze)(struct __pyx_obj_4cupy_5_core_4core__ndarray_base *, int __pyx_skip_dispatch, struct __pyx_opt_args_4cupy_5_core_4core_13_ndarray_base_squeeze *__pyx_optional_args);
  struct __pyx_obj_4cupy_5_core_4core__ndarray_base *(*take)(struct __pyx_obj_4cupy_5_core_4core__ndarray_base *, PyObject *, int __pyx_skip_dispatch, struct __pyx_opt_args_4cupy_5_core_4core_13_ndarray_base_take *__pyx_optional_args);
  PyObject *(*put)(struct __pyx_obj_4cupy_5_core_4core__ndarray_base *, PyObject *, PyObject *, int __pyx_skip_dispatch, struct __pyx_opt_args_4cupy_5_core_4core_13_ndarray_base_put *__pyx_optional_args);
  PyObject *(*repeat)(struct __pyx_obj_4cupy_5_core_4core__ndarray_base *, PyObject *, int __pyx_skip_dispatch, struct __pyx_opt_args_4cupy_5_core_4core_13_ndarray_base_repeat *__pyx_optional_args);
  PyObject *(*choose)(struct __pyx_obj_4cupy_5_core_4core__ndarray_base *, PyObject *, int __pyx_skip_dispatch, struct __pyx_opt_args_4cupy_5_core_4core_13_ndarray_base_choose *__pyx_optional_args);
  PyObject *(*sort)(struct __pyx_obj_4cupy_5_core_4core__ndarray_base *, int __pyx_skip_dispatch, struct __pyx_opt_args_4cupy_5_core_4core_13_ndarray_base_sort *__pyx_optional_args);
  struct __pyx_obj_4cupy_5_core_4core__ndarray_base *(*argsort)(struct __pyx_obj_4cupy_5_core_4core__ndarray_base *, int __pyx_skip_dispatch, struct __pyx_opt_args_4cupy_5_core_4core_13_ndarray_base_argsort *__pyx_optional_args);
  PyObject *(*partition)(struct __pyx_obj_4cupy_5_core_4core__ndarray_base *, PyObject *, int __pyx_skip_dispatch, struct __pyx_opt_args_4cupy_5_core_4core_13_ndarray_base_partition *__pyx_optional_args);
  struct __pyx_obj_4cupy_5_core_4core__ndarray_base *(*argpartition)(struct __pyx_obj_4cupy_5_core_4core__ndarray_base *, PyObject *, int __pyx_skip_dispatch, struct __pyx_opt_args_4cupy_5_core_4core_13_ndarray_base_argpartition *__pyx_optional_args);
  PyObject *(*nonzero)(struct __pyx_obj_4cupy_5_core_4core__ndarray_base *, int __pyx_skip_dispatch);
  struct __pyx_obj_4cupy_5_core_4core__ndarray_base *(*compress)(struct __pyx_obj_4cupy_5_core_4core__ndarray_base *, PyObject *, int __pyx_skip_dispatch, struct __pyx_opt_args_4cupy_5_core_4core_13_ndarray_base_compress *__pyx_optional_args);
  struct __pyx_obj_4cupy_5_core_4core__ndarray_base *(*diagonal)(struct __pyx_obj_4cupy_5_core_4core__ndarray_base *, int __pyx_skip_dispatch, struct __pyx_opt_args_4cupy_5_core_4core_13_ndarray_base_diagonal *__pyx_optional_args);
  struct __pyx_obj_4cupy_5_core_4core__ndarray_base *(*max)(struct __pyx_obj_4cupy_5_core_4core__ndarray_base *, int __pyx_skip_dispatch, struct __pyx_opt_args_4cupy_5_core_4core_13_ndarray_base_max *__pyx_optional_args);
  struct __pyx_obj_4cupy_5_core_4core__ndarray_base *(*argmax)(struct __pyx_obj_4cupy_5_core_4core__ndarray_base *, int __pyx_skip_dispatch, struct __pyx_opt_args_4cupy_5_core_4core_13_ndarray_base_argmax *__pyx_optional_args);
  struct __pyx_obj_4cupy_5_core_4core__ndarray_base *(*min)(struct __pyx_obj_4cupy_5_core_4core__ndarray_base *, int __pyx_skip_dispatch, struct __pyx_opt_args_4cupy_5_core_4core_13_ndarray_base_min *__pyx_optional_args);
  struct __pyx_obj_4cupy_5_core_4core__ndarray_base *(*argmin)(struct __pyx_obj_4cupy_5_core_4core__ndarray_base *, int __pyx_skip_dispatch, struct __pyx_opt_args_4cupy_5_core_4core_13_ndarray_base_argmin *__pyx_optional_args);
  struct __pyx_obj_4cupy_5_core_4core__ndarray_base *(*ptp)(struct __pyx_obj_4cupy_5_core_4core__ndarray_base *, int __pyx_skip_dispatch, struct __pyx_opt_args_4cupy_5_core_4core_13_ndarray_base_ptp *__pyx_optional_args);
  struct __pyx_obj_4cupy_5_core_4core__ndarray_base *(*clip)(struct __pyx_obj_4cupy_5_core_4core__ndarray_base *, int __pyx_skip_dispatch, struct __pyx_opt_args_4cupy_5_core_4core_13_ndarray_base_clip *__pyx_optional_args);
  struct __pyx_obj_4cupy_5_core_4core__ndarray_base *(*round)(struct __pyx_obj_4cupy_5_core_4core__ndarray_base *, int __pyx_skip_dispatch, struct __pyx_opt_args_4cupy_5_core_4core_13_ndarray_base_round *__pyx_optional_args);
  struct __pyx_obj_4cupy_5_core_4core__ndarray_base *(*trace)(struct __pyx_obj_4cupy_5_core_4core__ndarray_base *, int __pyx_skip_dispatch, struct __pyx_opt_args_4cupy_5_core_4core_13_ndarray_base_trace *__pyx_optional_args);
  struct __pyx_obj_4cupy_5_core_4core__ndarray_base *(*sum)(struct __pyx_obj_4cupy_5_core_4core__ndarray_base *, int __pyx_skip_dispatch, struct __pyx_opt_args_4cupy_5_core_4core_13_ndarray_base_sum *__pyx_optional_args);
  struct __pyx_obj_4cupy_5_core_4core__ndarray_base *(*cumsum)(struct __pyx_obj_4cupy_5_core_4core__ndarray_base *, int __pyx_skip_dispatch, struct __pyx_opt_args_4cupy_5_core_4core_13_ndarray_base_cumsum *__pyx_optional_args);
  struct __pyx_obj_4cupy_5_core_4core__ndarray_base *(*mean)(struct __pyx_obj_4cupy_5_core_4core__ndarray_base *, int __pyx_skip_dispatch, struct __pyx_opt_args_4cupy_5_core_4core_13_ndarray_base_mean *__pyx_optional_args);
  struct __pyx_obj_4cupy_5_core_4core__ndarray_base *(*var)(struct __pyx_obj_4cupy_5_core_4core__ndarray_base *, int __pyx_skip_dispatch, struct __pyx_opt_args_4cupy_5_core_4core_13_ndarray_base_var *__pyx_optional_args);
  struct __pyx_obj_4cupy_5_core_4core__ndarray_base *(*std)(struct __pyx_obj_4cupy_5_core_4core__ndarray_base *, int __pyx_skip_dispatch, struct __pyx_opt_args_4cupy_5_core_4core_13_ndarray_base_std *__pyx_optional_args);
  struct __pyx_obj_4cupy_5_core_4core__ndarray_base *(*prod)(struct __pyx_obj_4cupy_5_core_4core__ndarray_base *, int __pyx_skip_dispatch, struct __pyx_opt_args_4cupy_5_core_4core_13_ndarray_base_prod *__pyx_optional_args);
  struct __pyx_obj_4cupy_5_core_4core__ndarray_base *(*cumprod)(struct __pyx_obj_4cupy_5_core_4core__ndarray_base *, int __pyx_skip_dispatch, struct __pyx_opt_args_4cupy_5_core_4core_13_ndarray_base_cumprod *__pyx_optional_args);
  struct __pyx_obj_4cupy_5_core_4core__ndarray_base *(*_add_reduceat)(struct __pyx_obj_4cupy_5_core_4core__ndarray_base *, PyObject *, PyObject *, PyObject *, PyObject *, int __pyx_skip_dispatch);
  struct __pyx_obj_4cupy_5_core_4core__ndarray_base *(*all)(struct __pyx_obj_4cupy_5_core_4core__ndarray_base *, int __pyx_skip_dispatch, struct __pyx_opt_args_4cupy_5_core_4core_13_ndarray_base_all *__pyx_optional_args);
  struct __pyx_obj_4cupy_5_core_4core__ndarray_base *(*any)(struct __pyx_obj_4cupy_5_core_4core__ndarray_base *, int __pyx_skip_dispatch, struct __pyx_opt_args_4cupy_5_core_4core_13_ndarray_base_any *__pyx_optional_args);
  struct __pyx_obj_4cupy_5_core_4core__ndarray_base *(*conj)(struct __pyx_obj_4cupy_5_core_4core__ndarray_base *, int __pyx_skip_dispatch);
  struct __pyx_obj_4cupy_5_core_4core__ndarray_base *(*conjugate)(struct __pyx_obj_4cupy_5_core_4core__ndarray_base *, int __pyx_skip_dispatch);
  PyObject *(*get)(struct __pyx_obj_4cupy_5_core_4core__ndarray_base *, int __pyx_skip_dispatch, struct __pyx_opt_args_4cupy_5_core_4core_13_ndarray_base_get *__pyx_optional_args);
  PyObject *(*set)(struct __pyx_obj_4cupy_5_core_4core__ndarray_base *, PyObject *, int __pyx_skip_dispatch, struct __pyx_opt_args_4cupy_5_core_4core_13_ndarray_base_set *__pyx_optional_args);
  struct __pyx_obj_4cupy_5_core_4core__ndarray_base *(*reduced_view)(struct __pyx_obj_4cupy_5_core_4core__ndarray_base *, int __pyx_skip_dispatch, struct __pyx_opt_args_4cupy_5_core_4core_13_ndarray_base_reduced_view *__pyx_optional_args);
  PyObject *(*_update_c_contiguity)(struct __pyx_obj_4cupy_5_core_4core__ndarray_base *);
  PyObject *(*_update_f_contiguity)(struct __pyx_obj_4cupy_5_core_4core__ndarray_base *);
  PyObject *(*_update_contiguity)(struct __pyx_obj_4cupy_5_core_4core__ndarray_base *);
  PyObject *(*_set_shape_and_strides)(struct __pyx_obj_4cupy_5_core_4core__ndarray_base *, __pyx_t_4cupy_5_core_7_carray_shape_t const &, __pyx_t_4cupy_5_core_7_carray_strides_t const &, int, int, int __pyx_skip_dispatch);
  struct __pyx_obj_4cupy_5_core_4core__ndarray_base *(*_view)(struct __pyx_obj_4cupy_5_core_4core__ndarray_base *, PyObject *, __pyx_t_4cupy_5_core_7_carray_shape_t const &, __pyx_t_4cupy_5_core_7_carray_strides_t const &, int, int, PyObject *);
  PyObject *(*_set_contiguous_strides)(struct __pyx_obj_4cupy_5_core_4core__ndarray_base *, Py_ssize_t, int);
  struct __pyx_obj_4cupy_4cuda_8function_CPointer *(*get_pointer)(struct __pyx_obj_4cupy_5_core_4core__ndarray_base *);
  PyObject *(*toDlpack)(struct __pyx_obj_4cupy_5_core_4core__ndarray_base *, int __pyx_skip_dispatch);
};
static struct __pyx_vtabstruct_4cupy_5_core_4core__ndarray_base *__pyx_vtabptr_4cupy_5_core_4core__ndarray_base;
/* #### Code section: utility_code_proto ### */

/* --- Runtime support code (head) --- */
/* Refnanny.proto */
#ifndef CYTHON_REFNANNY
  #define CYTHON_REFNANNY 0
#endif
#if CYTHON_REFNANNY
  typedef struct {
    void (*INCREF)(void*, PyObject*, Py_ssize_t);
    void (*DECREF)(void*, PyObject*, Py_ssize_t);
    void (*GOTREF)(void*, PyObject*, Py_ssize_t);
    void (*GIVEREF)(void*, PyObject*, Py_ssize_t);
    void* (*SetupContext)(const char*, Py_ssize_t, const char*);
    void (*FinishContext)(void**);
  } __Pyx_RefNannyAPIStruct;
  static __Pyx_RefNannyAPIStruct *__Pyx_RefNanny = NULL;
  static __Pyx_RefNannyAPIStruct *__Pyx_RefNannyImportAPI(const char *modname);
  #define __Pyx_RefNannyDeclarations void *__pyx_refnanny = NULL;
  #define __Pyx_RefNannySetupContext(name, acquire_gil)\
          if (acquire_gil) {\
              PyGILState_STATE __pyx_gilstate_save = PyGILState_Ensure();\
              __pyx_refnanny = __Pyx_RefNanny->SetupContext((name), (__LINE__), (__FILE__));\
              PyGILState_Release(__pyx_gilstate_save);\
          } else {\
              __pyx_refnanny = __Pyx_RefNanny->SetupContext((name), (__LINE__), (__FILE__));\
          }
  #define __Pyx_RefNannyFinishContextNogil() {\
              PyGILState_STATE __pyx_gilstate_save = PyGILState_Ensure();\
              __Pyx_RefNannyFinishContext();\
              PyGILState_Release(__pyx_gilstate_save);\
          }
  #define __Pyx_RefNannyFinishContextNogil() {\
              PyGILState_STATE __pyx_gilstate_save = PyGILState_Ensure();\
              __Pyx_RefNannyFinishContext();\
              PyGILState_Release(__pyx_gilstate_save);\
          }
  #define __Pyx_RefNannyFinishContext()\
          __Pyx_RefNanny->FinishContext(&__pyx_refnanny)
  #define __Pyx_INCREF(r)  __Pyx_RefNanny->INCREF(__pyx_refnanny, (PyObject *)(r), (__LINE__))
  #define __Pyx_DECREF(r)  __Pyx_RefNanny->DECREF(__pyx_refnanny, (PyObject *)(r), (__LINE__))
  #define __Pyx_GOTREF(r)  __Pyx_RefNanny->GOTREF(__pyx_refnanny, (PyObject *)(r), (__LINE__))
  #define __Pyx_GIVEREF(r) __Pyx_RefNanny->GIVEREF(__pyx_refnanny, (PyObject *)(r), (__LINE__))
  #define __Pyx_XINCREF(r)  do { if((r) == NULL); else {__Pyx_INCREF(r); }} while(0)
  #define __Pyx_XDECREF(r)  do { if((r) == NULL); else {__Pyx_DECREF(r); }} while(0)
  #define __Pyx_XGOTREF(r)  do { if((r) == NULL); else {__Pyx_GOTREF(r); }} while(0)
  #define __Pyx_XGIVEREF(r) do { if((r) == NULL); else {__Pyx_GIVEREF(r);}} while(0)
#else
  #define __Pyx_RefNannyDeclarations
  #define __Pyx_RefNannySetupContext(name, acquire_gil)
  #define __Pyx_RefNannyFinishContextNogil()
  #define __Pyx_RefNannyFinishContext()
  #define __Pyx_INCREF(r) Py_INCREF(r)
  #define __Pyx_DECREF(r) Py_DECREF(r)
  #define __Pyx_GOTREF(r)
  #define __Pyx_GIVEREF(r)
  #define __Pyx_XINCREF(r) Py_XINCREF(r)
  #define __Pyx_XDECREF(r) Py_XDECREF(r)
  #define __Pyx_XGOTREF(r)
  #define __Pyx_XGIVEREF(r)
#endif
#define __Pyx_Py_XDECREF_SET(r, v) do {\
        PyObject *tmp = (PyObject *) r;\
        r = v; Py_XDECREF(tmp);\
    } while (0)
#define __Pyx_XDECREF_SET(r, v) do {\
        PyObject *tmp = (PyObject *) r;\
        r = v; __Pyx_XDECREF(tmp);\
    } while (0)
#define __Pyx_DECREF_SET(r, v) do {\
        PyObject *tmp = (PyObject *) r;\
        r = v; __Pyx_DECREF(tmp);\
    } while (0)
#define __Pyx_CLEAR(r)    do { PyObject* tmp = ((PyObject*)(r)); r = NULL; __Pyx_DECREF(tmp);} while(0)
#define __Pyx_XCLEAR(r)   do { if((r) != NULL) {PyObject* tmp = ((PyObject*)(r)); r = NULL; __Pyx_DECREF(tmp);}} while(0)

/* GetItemInt.proto */
#define __Pyx_GetItemInt(o, i, type, is_signed, to_py_func, is_list, wraparound, boundscheck, has_gil, unsafe_shared)\
    (__Pyx_fits_Py_ssize_t(i, type, is_signed) ?\
    __Pyx_GetItemInt_Fast(o, (Py_ssize_t)i, is_list, wraparound, boundscheck, unsafe_shared) :\
    (is_list ? (PyErr_SetString(PyExc_IndexError, "list index out of range"), (PyObject*)NULL) :\
               __Pyx_GetItemInt_Generic(o, to_py_func(i))))
#define __Pyx_GetItemInt_List(o, i, type, is_signed, to_py_func, is_list, wraparound, boundscheck, has_gil, unsafe_shared)\
    (__Pyx_fits_Py_ssize_t(i, type, is_signed) ?\
    __Pyx_GetItemInt_List_Fast(o, (Py_ssize_t)i, wraparound, boundscheck, unsafe_shared) :\
    (PyErr_SetString(PyExc_IndexError, "list index out of range"), (PyObject*)NULL))
static CYTHON_INLINE PyObject *__Pyx_GetItemInt_List_Fast(PyObject *o, Py_ssize_t i,
                                                              int wraparound, int boundscheck, int unsafe_shared);
#define __Pyx_GetItemInt_Tuple(o, i, type, is_signed, to_py_func, is_list, wraparound, boundscheck, has_gil, unsafe_shared)\
    (__Pyx_fits_Py_ssize_t(i, type, is_signed) ?\
    __Pyx_GetItemInt_Tuple_Fast(o, (Py_ssize_t)i, wraparound, boundscheck, unsafe_shared) :\
    (PyErr_SetString(PyExc_IndexError, "tuple index out of range"), (PyObject*)NULL))
static CYTHON_INLINE PyObject *__Pyx_GetItemInt_Tuple_Fast(PyObject *o, Py_ssize_t i,
                                                              int wraparound, int boundscheck, int unsafe_shared);
static PyObject *__Pyx_GetItemInt_Generic(PyObject *o, PyObject* j);
static CYTHON_INLINE PyObject *__Pyx_GetItemInt_Fast(PyObject *o, Py_ssize_t i,
                                                     int is_list, int wraparound, int boundscheck, int unsafe_shared);

/* PyErrExceptionMatches.proto (used by PyObjectGetAttrStrNoError) */
#if CYTHON_FAST_THREAD_STATE
#define __Pyx_PyErr_ExceptionMatches(err) __Pyx_PyErr_ExceptionMatchesInState(__pyx_tstate, err)
static CYTHON_INLINE int __Pyx_PyErr_ExceptionMatchesInState(PyThreadState* tstate, PyObject* err);
#else
#define __Pyx_PyErr_ExceptionMatches(err)  PyErr_ExceptionMatches(err)
#endif

/* PyThreadStateGet.proto (used by PyErrFetchRestore) */
#if CYTHON_FAST_THREAD_STATE
#define __Pyx_PyThreadState_declare  PyThreadState *__pyx_tstate;
#define __Pyx_PyThreadState_assign  __pyx_tstate = __Pyx_PyThreadState_Current;
#if PY_VERSION_HEX >= 0x030C00A6
#define __Pyx_PyErr_Occurred()  (__pyx_tstate->current_exception != NULL)
#define __Pyx_PyErr_CurrentExceptionType()  (__pyx_tstate->current_exception ? (PyObject*) Py_TYPE(__pyx_tstate->current_exception) : (PyObject*) NULL)
#else
#define __Pyx_PyErr_Occurred()  (__pyx_tstate->curexc_type != NULL)
#define __Pyx_PyErr_CurrentExceptionType()  (__pyx_tstate->curexc_type)
#endif
#else
#define __Pyx_PyThreadState_declare
#define __Pyx_PyThreadState_assign
#define __Pyx_PyErr_Occurred()  (PyErr_Occurred() != NULL)
#define __Pyx_PyErr_CurrentExceptionType()  PyErr_Occurred()
#endif

/* PyErrFetchRestore.proto (used by PyObjectGetAttrStrNoError) */
#if CYTHON_FAST_THREAD_STATE
#define __Pyx_PyErr_Clear() __Pyx_ErrRestore(NULL, NULL, NULL)
#define __Pyx_ErrRestoreWithState(type, value, tb)  __Pyx_ErrRestoreInState(PyThreadState_GET(), type, value, tb)
#define __Pyx_ErrFetchWithState(type, value, tb)    __Pyx_ErrFetchInState(PyThreadState_GET(), type, value, tb)
#define __Pyx_ErrRestore(type, value, tb)  __Pyx_ErrRestoreInState(__pyx_tstate, type, value, tb)
#define __Pyx_ErrFetch(type, value, tb)    __Pyx_ErrFetchInState(__pyx_tstate, type, value, tb)
static CYTHON_INLINE void __Pyx_ErrRestoreInState(PyThreadState *tstate, PyObject *type, PyObject *value, PyObject *tb);
static CYTHON_INLINE void __Pyx_ErrFetchInState(PyThreadState *tstate, PyObject **type, PyObject **value, PyObject **tb);
#if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX < 0x030C00A6
#define __Pyx_PyErr_SetNone(exc) (Py_INCREF(exc), __Pyx_ErrRestore((exc), NULL, NULL))
#else
#define __Pyx_PyErr_SetNone(exc) PyErr_SetNone(exc)
#endif
#else
#define __Pyx_PyErr_Clear() PyErr_Clear()
#define __Pyx_PyErr_SetNone(exc) PyErr_SetNone(exc)
#define __Pyx_ErrRestoreWithState(type, value, tb)  PyErr_Restore(type, value, tb)
#define __Pyx_ErrFetchWithState(type, value, tb)  PyErr_Fetch(type, value, tb)
#define __Pyx_ErrRestoreInState(tstate, type, value, tb)  PyErr_Restore(type, value, tb)
#define __Pyx_ErrFetchInState(tstate, type, value, tb)  PyErr_Fetch(type, value, tb)
#define __Pyx_ErrRestore(type, value, tb)  PyErr_Restore(type, value, tb)
#define __Pyx_ErrFetch(type, value, tb)  PyErr_Fetch(type, value, tb)
#endif

/* PyObjectGetAttrStr.proto (used by PyObjectGetAttrStrNoError) */
#if CYTHON_USE_TYPE_SLOTS
static CYTHON_INLINE PyObject* __Pyx_PyObject_GetAttrStr(PyObject* obj, PyObject* attr_name);
#else
#define __Pyx_PyObject_GetAttrStr(o,n) PyObject_GetAttr(o,n)
#endif

/* PyObjectGetAttrStrNoError.proto (used by HasAttr) */
static CYTHON_INLINE PyObject* __Pyx_PyObject_GetAttrStrNoError(PyObject* obj, PyObject* attr_name);

/* HasAttr.proto */
#if __PYX_LIMITED_VERSION_HEX >= 0x030d0000
#define __Pyx_HasAttr(o, n)  PyObject_HasAttrWithError(o, n)
#else
static CYTHON_INLINE int __Pyx_HasAttr(PyObject *, PyObject *);
#endif

/* PyObjectCall.proto (used by PyObjectFastCall) */
#if CYTHON_COMPILING_IN_CPYTHON
static CYTHON_INLINE PyObject* __Pyx_PyObject_Call(PyObject *func, PyObject *arg, PyObject *kw);
#else
#define __Pyx_PyObject_Call(func, arg, kw) PyObject_Call(func, arg, kw)
#endif

/* PyObjectCallMethO.proto (used by PyObjectFastCall) */
#if CYTHON_COMPILING_IN_CPYTHON
static CYTHON_INLINE PyObject* __Pyx_PyObject_CallMethO(PyObject *func, PyObject *arg);
#endif

/* PyObjectFastCall.proto (used by PyObjectFastCallMethod) */
#define __Pyx_PyObject_FastCall(func, args, nargs)  __Pyx_PyObject_FastCallDict(func, args, (size_t)(nargs), NULL)
static CYTHON_INLINE PyObject* __Pyx_PyObject_FastCallDict(PyObject *func, PyObject * const*args, size_t nargs, PyObject *kwargs);

/* PyObjectFastCallMethod.proto */
#if CYTHON_VECTORCALL && PY_VERSION_HEX >= 0x03090000
#define __Pyx_PyObject_FastCallMethod(name, args, nargsf) PyObject_VectorcallMethod(name, args, nargsf, NULL)
#else
static PyObject *__Pyx_PyObject_FastCallMethod(PyObject *name, PyObject *const *args, size_t nargsf);
#endif

/* ExtTypeTest.proto */
static CYTHON_INLINE int __Pyx_TypeTest(PyObject *obj, PyTypeObject *type);

/* PyTypeError_Check.proto */
#define __Pyx_PyExc_TypeError_Check(obj)  __Pyx_TypeCheck(obj, PyExc_TypeError)

/* PyObjectFormatSimple.proto */
#if CYTHON_COMPILING_IN_PYPY
    #define __Pyx_PyObject_FormatSimple(s, f) (\
        likely(PyUnicode_CheckExact(s)) ? (Py_INCREF(s), s) :\
        PyObject_Format(s, f))
#elif CYTHON_USE_TYPE_SLOTS
    #define __Pyx_PyObject_FormatSimple(s, f) (\
        likely(PyUnicode_CheckExact(s)) ? (Py_INCREF(s), s) :\
        likely(PyLong_CheckExact(s)) ? PyLong_Type.tp_repr(s) :\
        likely(PyFloat_CheckExact(s)) ? PyFloat_Type.tp_repr(s) :\
        PyObject_Format(s, f))
#else
    #define __Pyx_PyObject_FormatSimple(s, f) (\
        likely(PyUnicode_CheckExact(s)) ? (Py_INCREF(s), s) :\
        PyObject_Format(s, f))
#endif

/* IncludeStringH.proto (used by JoinPyUnicode) */
#include <string.h>

/* JoinPyUnicode.export */
static PyObject* __Pyx_PyUnicode_Join(PyObject** values, Py_ssize_t value_count, Py_ssize_t result_ulength,
                                      Py_UCS4 max_char);

/* RaiseException.export */
static void __Pyx_Raise(PyObject *type, PyObject *value, PyObject *tb, PyObject *cause);

/* TupleAndListFromArray.proto (used by fastcall) */
#if CYTHON_COMPILING_IN_CPYTHON
static CYTHON_INLINE PyObject* __Pyx_PyList_FromArray(PyObject *const *src, Py_ssize_t n);
#endif
#if CYTHON_COMPILING_IN_CPYTHON || CYTHON_METH_FASTCALL
static CYTHON_INLINE PyObject* __Pyx_PyTuple_FromArray(PyObject *const *src, Py_ssize_t n);
#endif

/* BytesEquals.proto (used by UnicodeEquals) */
static CYTHON_INLINE int __Pyx_PyBytes_Equals(PyObject* s1, PyObject* s2, int equals);

/* UnicodeEquals.proto (used by fastcall) */
static CYTHON_INLINE int __Pyx_PyUnicode_Equals(PyObject* s1, PyObject* s2, int equals);

/* fastcall.proto */
#if CYTHON_AVOID_BORROWED_REFS
    #define __Pyx_ArgRef_VARARGS(args, i) __Pyx_PySequence_ITEM(args, i)
#elif CYTHON_ASSUME_SAFE_MACROS
    #define __Pyx_ArgRef_VARARGS(args, i) __Pyx_NewRef(__Pyx_PyTuple_GET_ITEM(args, i))
#else
    #define __Pyx_ArgRef_VARARGS(args, i) __Pyx_XNewRef(PyTuple_GetItem(args, i))
#endif
#define __Pyx_NumKwargs_VARARGS(kwds) PyDict_Size(kwds)
#define __Pyx_KwValues_VARARGS(args, nargs) NULL
#define __Pyx_GetKwValue_VARARGS(kw, kwvalues, s) __Pyx_PyDict_GetItemStrWithError(kw, s)
#define __Pyx_KwargsAsDict_VARARGS(kw, kwvalues) PyDict_Copy(kw)
#if CYTHON_METH_FASTCALL
    #define __Pyx_ArgRef_FASTCALL(args, i) __Pyx_NewRef(args[i])
    #define __Pyx_NumKwargs_FASTCALL(kwds) __Pyx_PyTuple_GET_SIZE(kwds)
    #define __Pyx_KwValues_FASTCALL(args, nargs) ((args) + (nargs))
    static CYTHON_INLINE PyObject * __Pyx_GetKwValue_FASTCALL(PyObject *kwnames, PyObject *const *kwvalues, PyObject *s);
  #if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x030d0000 || CYTHON_COMPILING_IN_LIMITED_API
    CYTHON_UNUSED static PyObject *__Pyx_KwargsAsDict_FASTCALL(PyObject *kwnames, PyObject *const *kwvalues);
  #else
    #define __Pyx_KwargsAsDict_FASTCALL(kw, kwvalues) _PyStack_AsDict(kwvalues, kw)
  #endif
#else
    #define __Pyx_ArgRef_FASTCALL __Pyx_ArgRef_VARARGS
    #define __Pyx_NumKwargs_FASTCALL __Pyx_NumKwargs_VARARGS
    #define __Pyx_KwValues_FASTCALL __Pyx_KwValues_VARARGS
    #define __Pyx_GetKwValue_FASTCALL __Pyx_GetKwValue_VARARGS
    #define __Pyx_KwargsAsDict_FASTCALL __Pyx_KwargsAsDict_VARARGS
#endif
#define __Pyx_ArgsSlice_VARARGS(args, start, stop) PyTuple_GetSlice(args, start, stop)
#if CYTHON_METH_FASTCALL || (CYTHON_COMPILING_IN_CPYTHON && CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS)
#define __Pyx_ArgsSlice_FASTCALL(args, start, stop) __Pyx_PyTuple_FromArray(args + start, stop - start)
#else
#define __Pyx_ArgsSlice_FASTCALL(args, start, stop) PyTuple_GetSlice(args, start, stop)
#endif

/* py_dict_items.proto (used by OwnedDictNext) */
static CYTHON_INLINE PyObject* __Pyx_PyDict_Items(PyObject* d);

/* CallCFunction.proto (used by CallUnboundCMethod0) */
#define __Pyx_CallCFunction(cfunc, self, args)\
    ((PyCFunction)(void(*)(void))(cfunc)->func)(self, args)
#define __Pyx_CallCFunctionWithKeywords(cfunc, self, args, kwargs)\
    ((PyCFunctionWithKeywords)(void(*)(void))(cfunc)->func)(self, args, kwargs)
#define __Pyx_CallCFunctionFast(cfunc, self, args, nargs)\
    ((__Pyx_PyCFunctionFast)(void(*)(void))(PyCFunction)(cfunc)->func)(self, args, nargs)
#define __Pyx_CallCFunctionFastWithKeywords(cfunc, self, args, nargs, kwnames)\
    ((__Pyx_PyCFunctionFastWithKeywords)(void(*)(void))(PyCFunction)(cfunc)->func)(self, args, nargs, kwnames)

/* PyObjectCallOneArg.proto (used by CallUnboundCMethod0) */
static CYTHON_INLINE PyObject* __Pyx_PyObject_CallOneArg(PyObject *func, PyObject *arg);

/* UnpackUnboundCMethod.proto (used by CallUnboundCMethod0) */
typedef struct {
    PyObject *type;
    PyObject **method_name;
#if CYTHON_COMPILING_IN_CPYTHON_FREETHREADING && CYTHON_ATOMICS
    __pyx_atomic_int_type initialized;
#endif
    PyCFunction func;
    PyObject *method;
    int flag;
} __Pyx_CachedCFunction;
#if CYTHON_COMPILING_IN_CPYTHON_FREETHREADING
static CYTHON_INLINE int __Pyx_CachedCFunction_GetAndSetInitializing(__Pyx_CachedCFunction *cfunc) {
#if !CYTHON_ATOMICS
    return 1;
#else
    __pyx_nonatomic_int_type expected = 0;
    if (__pyx_atomic_int_cmp_exchange(&cfunc->initialized, &expected, 1)) {
        return 0;
    }
    return expected;
#endif
}
static CYTHON_INLINE void __Pyx_CachedCFunction_SetFinishedInitializing(__Pyx_CachedCFunction *cfunc) {
#if CYTHON_ATOMICS
    __pyx_atomic_store(&cfunc->initialized, 2);
#endif
}
#else
#define __Pyx_CachedCFunction_GetAndSetInitializing(cfunc) 2
#define __Pyx_CachedCFunction_SetFinishedInitializing(cfunc)
#endif

/* CallUnboundCMethod0.proto */
CYTHON_UNUSED
static PyObject* __Pyx__CallUnboundCMethod0(__Pyx_CachedCFunction* cfunc, PyObject* self);
#if CYTHON_COMPILING_IN_CPYTHON
static CYTHON_INLINE PyObject* __Pyx_CallUnboundCMethod0(__Pyx_CachedCFunction* cfunc, PyObject* self);
#else
#define __Pyx_CallUnboundCMethod0(cfunc, self)  __Pyx__CallUnboundCMethod0(cfunc, self)
#endif

/* py_dict_values.proto (used by OwnedDictNext) */
static CYTHON_INLINE PyObject* __Pyx_PyDict_Values(PyObject* d);

/* OwnedDictNext.proto (used by ParseKeywordsImpl) */
#if CYTHON_AVOID_BORROWED_REFS
static int __Pyx_PyDict_NextRef(PyObject *p, PyObject **ppos, PyObject **pkey, PyObject **pvalue);
#else
CYTHON_INLINE
static int __Pyx_PyDict_NextRef(PyObject *p, Py_ssize_t *ppos, PyObject **pkey, PyObject **pvalue);
#endif

/* RaiseDoubleKeywords.proto (used by ParseKeywordsImpl) */
static void __Pyx_RaiseDoubleKeywordsError(const char* func_name, PyObject* kw_name);

/* ParseKeywordsImpl.export */
static int __Pyx_ParseKeywordsTuple(
    PyObject *kwds,
    PyObject * const *kwvalues,
    PyObject ** const argnames[],
    PyObject *kwds2,
    PyObject *values[],
    Py_ssize_t num_pos_args,
    Py_ssize_t num_kwargs,
    const char* function_name,
    int ignore_unknown_kwargs
);
static int __Pyx_ParseKeywordDictToDict(
    PyObject *kwds,
    PyObject ** const argnames[],
    PyObject *kwds2,
    PyObject *values[],
    Py_ssize_t num_pos_args,
    const char* function_name
);
static int __Pyx_ParseKeywordDict(
    PyObject *kwds,
    PyObject ** const argnames[],
    PyObject *values[],
    Py_ssize_t num_pos_args,
    Py_ssize_t num_kwargs,
    const char* function_name,
    int ignore_unknown_kwargs
);

/* CallUnboundCMethod2.proto */
CYTHON_UNUSED
static PyObject* __Pyx__CallUnboundCMethod2(__Pyx_CachedCFunction* cfunc, PyObject* self, PyObject* arg1, PyObject* arg2);
#if CYTHON_COMPILING_IN_CPYTHON
static CYTHON_INLINE PyObject *__Pyx_CallUnboundCMethod2(__Pyx_CachedCFunction *cfunc, PyObject *self, PyObject *arg1, PyObject *arg2);
#else
#define __Pyx_CallUnboundCMethod2(cfunc, self, arg1, arg2)  __Pyx__CallUnboundCMethod2(cfunc, self, arg1, arg2)
#endif

/* ParseKeywords.proto */
static CYTHON_INLINE int __Pyx_ParseKeywords(
    PyObject *kwds, PyObject *const *kwvalues, PyObject ** const argnames[],
    PyObject *kwds2, PyObject *values[],
    Py_ssize_t num_pos_args, Py_ssize_t num_kwargs,
    const char* function_name,
    int ignore_unknown_kwargs
);

/* RaiseArgTupleInvalid.proto */
static void __Pyx_RaiseArgtupleInvalid(const char* func_name, int exact,
    Py_ssize_t num_min, Py_ssize_t num_max, Py_ssize_t num_found);

/* GetBuiltinName.proto (used by GetModuleGlobalName) */
static PyObject *__Pyx_GetBuiltinName(PyObject *name);

/* PyDictVersioning.proto (used by GetModuleGlobalName) */
#if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_TYPE_SLOTS
#define __PYX_DICT_VERSION_INIT  ((PY_UINT64_T) -1)
#define __PYX_GET_DICT_VERSION(dict)  (((PyDictObject*)(dict))->ma_version_tag)
#define __PYX_UPDATE_DICT_CACHE(dict, value, cache_var, version_var)\
    (version_var) = __PYX_GET_DICT_VERSION(dict);\
    (cache_var) = (value);
#define __PYX_PY_DICT_LOOKUP_IF_MODIFIED(VAR, DICT, LOOKUP) {\
    static PY_UINT64_T __pyx_dict_version = 0;\
    static PyObject *__pyx_dict_cached_value = NULL;\
    if (likely(__PYX_GET_DICT_VERSION(DICT) == __pyx_dict_version)) {\
        (VAR) = __Pyx_XNewRef(__pyx_dict_cached_value);\
    } else {\
        (VAR) = __pyx_dict_cached_value = (LOOKUP);\
        __pyx_dict_version = __PYX_GET_DICT_VERSION(DICT);\
    }\
}
static CYTHON_INLINE PY_UINT64_T __Pyx_get_tp_dict_version(PyObject *obj);
static CYTHON_INLINE PY_UINT64_T __Pyx_get_object_dict_version(PyObject *obj);
static CYTHON_INLINE int __Pyx_object_dict_version_matches(PyObject* obj, PY_UINT64_T tp_dict_version, PY_UINT64_T obj_dict_version);
#else
#define __PYX_GET_DICT_VERSION(dict)  (0)
#define __PYX_UPDATE_DICT_CACHE(dict, value, cache_var, version_var)
#define __PYX_PY_DICT_LOOKUP_IF_MODIFIED(VAR, DICT, LOOKUP)  (VAR) = (LOOKUP);
#endif

/* GetModuleGlobalName.proto */
#if CYTHON_USE_DICT_VERSIONS
#define __Pyx_GetModuleGlobalName(var, name)  do {\
    static PY_UINT64_T __pyx_dict_version = 0;\
    static PyObject *__pyx_dict_cached_value = NULL;\
    (var) = (likely(__pyx_dict_version == __PYX_GET_DICT_VERSION(__pyx_mstate_global->__pyx_d))) ?\
        (likely(__pyx_dict_cached_value) ? __Pyx_NewRef(__pyx_dict_cached_value) : __Pyx_GetBuiltinName(name)) :\
        __Pyx__GetModuleGlobalName(name, &__pyx_dict_version, &__pyx_dict_cached_value);\
} while(0)
#define __Pyx_GetModuleGlobalNameUncached(var, name)  do {\
    PY_UINT64_T __pyx_dict_version;\
    PyObject *__pyx_dict_cached_value;\
    (var) = __Pyx__GetModuleGlobalName(name, &__pyx_dict_version, &__pyx_dict_cached_value);\
} while(0)
static PyObject *__Pyx__GetModuleGlobalName(PyObject *name, PY_UINT64_T *dict_version, PyObject **dict_cached_value);
#else
#define __Pyx_GetModuleGlobalName(var, name)  (var) = __Pyx__GetModuleGlobalName(name)
#define __Pyx_GetModuleGlobalNameUncached(var, name)  (var) = __Pyx__GetModuleGlobalName(name)
static CYTHON_INLINE PyObject *__Pyx__GetModuleGlobalName(PyObject *name);
#endif

/* PySequenceContains.proto */
static CYTHON_INLINE int __Pyx_PySequence_ContainsTF(PyObject* item, PyObject* seq, int eq) {
    int result = PySequence_Contains(seq, item);
    return unlikely(result < 0) ? result : (result == (eq == Py_EQ));
}

/* PyValueError_Check.proto */
#define __Pyx_PyExc_ValueError_Check(obj)  __Pyx_TypeCheck(obj, PyExc_ValueError)

/* ObjectGetItem.proto */
#if CYTHON_USE_TYPE_SLOTS
static CYTHON_INLINE PyObject *__Pyx_PyObject_GetItem(PyObject *obj, PyObject *key);
#else
#define __Pyx_PyObject_GetItem(obj, key)  PyObject_GetItem(obj, key)
#endif

/* RaiseTooManyValuesToUnpack.proto */
static CYTHON_INLINE void __Pyx_RaiseTooManyValuesError(Py_ssize_t expected);

/* RaiseNeedMoreValuesToUnpack.proto */
static CYTHON_INLINE void __Pyx_RaiseNeedMoreValuesError(Py_ssize_t index);

/* IterFinish.proto */
static CYTHON_INLINE int __Pyx_IterFinish(void);

/* UnpackItemEndCheck.proto */
static int __Pyx_IternextUnpackEndCheck(PyObject *retval, Py_ssize_t expected);

/* PyLongCompare.proto */
static CYTHON_INLINE int __Pyx_PyLong_BoolEqObjC(PyObject *op1, PyObject *op2, long intval, long inplace);

/* PyRuntimeError_Check.proto */
#define __Pyx_PyExc_RuntimeError_Check(obj)  __Pyx_TypeCheck(obj, PyExc_RuntimeError)

/* AssertionsEnabled.proto */
#if CYTHON_COMPILING_IN_LIMITED_API  ||  PY_VERSION_HEX >= 0x030C0000
  static int __pyx_assertions_enabled_flag;
  #define __pyx_assertions_enabled() (__pyx_assertions_enabled_flag)
  #if __clang__ || __GNUC__
  __attribute__((no_sanitize("thread")))
  #endif
  static int __Pyx_init_assertions_enabled(void) {
    PyObject *builtins, *debug, *debug_str;
    int flag;
    builtins = PyEval_GetBuiltins();
    if (!builtins) goto bad;
    debug_str = PyUnicode_FromStringAndSize("__debug__", 9);
    if (!debug_str) goto bad;
    debug = PyObject_GetItem(builtins, debug_str);
    Py_DECREF(debug_str);
    if (!debug) goto bad;
    flag = PyObject_IsTrue(debug);
    Py_DECREF(debug);
    if (flag == -1) goto bad;
    __pyx_assertions_enabled_flag = flag;
    return 0;
  bad:
    __pyx_assertions_enabled_flag = 1;
    return -1;
  }
#else
  #define __Pyx_init_assertions_enabled()  (0)
  #define __pyx_assertions_enabled()  (!Py_OptimizeFlag)
#endif

/* PyAssertionError_Check.proto */
#define __Pyx_PyExc_AssertionError_Check(obj)  __Pyx_TypeCheck(obj, PyExc_AssertionError)

/* PyObjectVectorCallKwBuilder.proto */
CYTHON_UNUSED static int __Pyx_VectorcallBuilder_AddArg_Check(PyObject *key, PyObject *value, PyObject *builder, PyObject **args, int n);
#if CYTHON_VECTORCALL
#if PY_VERSION_HEX >= 0x03090000
#define __Pyx_Object_Vectorcall_CallFromBuilder PyObject_Vectorcall
#else
#define __Pyx_Object_Vectorcall_CallFromBuilder _PyObject_Vectorcall
#endif
#define __Pyx_MakeVectorcallBuilderKwds(n) PyTuple_New(n)
static int __Pyx_VectorcallBuilder_AddArg(PyObject *key, PyObject *value, PyObject *builder, PyObject **args, int n);
static int __Pyx_VectorcallBuilder_AddArgStr(const char *key, PyObject *value, PyObject *builder, PyObject **args, int n);
#else
#define __Pyx_Object_Vectorcall_CallFromBuilder __Pyx_PyObject_FastCallDict
#define __Pyx_MakeVectorcallBuilderKwds(n) __Pyx_PyDict_NewPresized(n)
#define __Pyx_VectorcallBuilder_AddArg(key, value, builder, args, n) PyDict_SetItem(builder, key, value)
#define __Pyx_VectorcallBuilder_AddArgStr(key, value, builder, args, n) PyDict_SetItemString(builder, key, value)
#endif

/* RaiseUnexpectedTypeError.proto */
static int __Pyx_RaiseUnexpectedTypeError(const char *expected, PyObject *obj);

/* PyObjectVectorCallMethodKwBuilder.proto */
#if CYTHON_VECTORCALL && PY_VERSION_HEX >= 0x03090000
#define __Pyx_Object_VectorcallMethod_CallFromBuilder PyObject_VectorcallMethod
#else
static PyObject *__Pyx_Object_VectorcallMethod_CallFromBuilder(PyObject *name, PyObject *const *args, size_t nargsf, PyObject *kwnames);
#endif

/* SliceObject.proto */
static CYTHON_INLINE PyObject* __Pyx_PyObject_GetSlice(
        PyObject* obj, Py_ssize_t cstart, Py_ssize_t cstop,
        PyObject** py_start, PyObject** py_stop, PyObject** py_slice,
        int has_cstart, int has_cstop, int wraparound);

/* RaiseUnboundLocalError.proto */
static void __Pyx_RaiseUnboundLocalError(const char *varname);

/* PyLongCompare.proto */
static CYTHON_INLINE int __Pyx_PyLong_BoolNeObjC(PyObject *op1, PyObject *op2, long intval, long inplace);

/* PyUnicodeContains.proto */
static CYTHON_INLINE int __Pyx_PyUnicode_ContainsTF(PyObject* substring, PyObject* text, int eq) {
    int result = PyUnicode_Contains(text, substring);
    return unlikely(result < 0) ? result : (result == (eq == Py_EQ));
}

/* UnicodeConcatInPlace.proto */
# if CYTHON_COMPILING_IN_CPYTHON
    #if CYTHON_REFNANNY
        #define __Pyx_PyUnicode_ConcatInPlace(left, right, unsafe_shared) __Pyx_PyUnicode_ConcatInPlaceImpl(&left, right, unsafe_shared, __pyx_refnanny)
    #else
        #define __Pyx_PyUnicode_ConcatInPlace(left, right, unsafe_shared) __Pyx_PyUnicode_ConcatInPlaceImpl(&left, right, unsafe_shared)
    #endif
    #define __Pyx_PyUnicode_Concat__Pyx_ReferenceSharing_DefinitelyUniqueInPlace(left, right) __Pyx_PyUnicode_ConcatInPlace(left, right, __Pyx_ReferenceSharing_DefinitelyUnique)
    #define __Pyx_PyUnicode_Concat__Pyx_ReferenceSharing_OwnStrongReferenceInPlace(left, right) __Pyx_PyUnicode_ConcatInPlace(left, right, __Pyx_ReferenceSharing_OwnStrongReference)
    #define __Pyx_PyUnicode_Concat__Pyx_ReferenceSharing_FunctionArgumentInPlace(left, right) __Pyx_PyUnicode_ConcatInPlace(left, right, __Pyx_ReferenceSharing_DefinitelyUnique)
    #define __Pyx_PyUnicode_Concat__Pyx_ReferenceSharing_SharedReferenceInPlace(left, right) __Pyx_PyUnicode_ConcatInPlace(left, right, __Pyx_ReferenceSharing_SharedReference)
    static CYTHON_INLINE PyObject *__Pyx_PyUnicode_ConcatInPlaceImpl(PyObject **p_left, PyObject *right, int unsafe_shared
        #if CYTHON_REFNANNY
        , void* __pyx_refnanny
        #endif
    );
#else
#define __Pyx_PyUnicode_Concat__Pyx_ReferenceSharing_DefinitelyUniqueInPlace __Pyx_PyUnicode_Concat
#define __Pyx_PyUnicode_Concat__Pyx_ReferenceSharing_OwnStrongReferenceInPlace __Pyx_PyUnicode_Concat
#define __Pyx_PyUnicode_Concat__Pyx_ReferenceSharing_FunctionArgumentInPlace __Pyx_PyUnicode_Concat
#define __Pyx_PyUnicode_Concat__Pyx_ReferenceSharing_SharedReferenceInPlace __Pyx_PyUnicode_Concat
#endif
#define __Pyx_PyUnicode_Concat__Pyx_ReferenceSharing_DefinitelyUniqueInPlaceSafe(left, right)\
    ((unlikely((left) == Py_None) || unlikely((right) == Py_None)) ?\
    PyNumber_InPlaceAdd(left, right) : __Pyx_PyUnicode_Concat__Pyx_ReferenceSharing_DefinitelyUniqueInPlace(left, right))
#define __Pyx_PyUnicode_Concat__Pyx_ReferenceSharing_OwnStrongReferenceInPlaceSafe(left, right)\
    ((unlikely((left) == Py_None) || unlikely((right) == Py_None)) ?\
    PyNumber_InPlaceAdd(left, right) : __Pyx_PyUnicode_Concat__Pyx_ReferenceSharing_OwnStrongReferenceInPlace(left, right))
#define __Pyx_PyUnicode_Concat__Pyx_ReferenceSharing_FunctionArgumentInPlaceSafe(left, right)\
    ((unlikely((left) == Py_None) || unlikely((right) == Py_None)) ?\
    PyNumber_InPlaceAdd(left, right) : __Pyx_PyUnicode_Concat__Pyx_ReferenceSharing_FunctionArgumentInPlace(left, right))
#define __Pyx_PyUnicode_Concat__Pyx_ReferenceSharing_SharedReferenceInPlaceSafe(left, right)\
    ((unlikely((left) == Py_None) || unlikely((right) == Py_None)) ?\
    PyNumber_InPlaceAdd(left, right) : __Pyx_PyUnicode_Concat__Pyx_ReferenceSharing_SharedReferenceInPlace(left, right))

/* GetAttr.proto */
static CYTHON_INLINE PyObject *__Pyx_GetAttr(PyObject *, PyObject *);

/* TypeImport.proto */
#ifndef __PYX_HAVE_RT_ImportType_proto_3_2_5
#define __PYX_HAVE_RT_ImportType_proto_3_2_5
#if defined (__STDC_VERSION__) && __STDC_VERSION__ >= 201112L
#include <stdalign.h>
#endif
#if (defined (__STDC_VERSION__) && __STDC_VERSION__ >= 201112L) || __cplusplus >= 201103L
#define __PYX_GET_STRUCT_ALIGNMENT_3_2_5(s) alignof(s)
#else
#define __PYX_GET_STRUCT_ALIGNMENT_3_2_5(s) sizeof(void*)
#endif
enum __Pyx_ImportType_CheckSize_3_2_5 {
   __Pyx_ImportType_CheckSize_Error_3_2_5 = 0,
   __Pyx_ImportType_CheckSize_Warn_3_2_5 = 1,
   __Pyx_ImportType_CheckSize_Ignore_3_2_5 = 2
};
static PyTypeObject *__Pyx_ImportType_3_2_5(PyObject* module, const char *module_name, const char *class_name, size_t size, size_t alignment, enum __Pyx_ImportType_CheckSize_3_2_5 check_size);
#endif

/* GetVTable.proto */
static void* __Pyx_GetVtable(PyTypeObject *type);

/* VoidPtrImport.proto */
static int __Pyx_ImportVoidPtr_3_2_5(PyObject *module, const char *name, void **p, const char *sig);

/* FunctionImport.proto */
static int __Pyx_ImportFunction_3_2_5(PyObject *module, const char *funcname, void (**f)(void), const char *sig);

/* ImportImpl.export */
static PyObject *__Pyx__Import(PyObject *name, PyObject *const *imported_names, Py_ssize_t len_imported_names, PyObject *qualname, PyObject *moddict, int level);

/* Import.proto */
static CYTHON_INLINE PyObject *__Pyx_Import(PyObject *name, PyObject *const *imported_names, Py_ssize_t len_imported_names, PyObject *qualname, int level);

/* ImportFrom.proto */
static PyObject* __Pyx_ImportFrom(PyObject* module, PyObject* name);

/* dict_setdefault.proto (used by CLineInTraceback) */
static CYTHON_INLINE PyObject *__Pyx_PyDict_SetDefault(PyObject *d, PyObject *key, PyObject *default_value);

/* CLineInTraceback.proto (used by AddTraceback) */
#if CYTHON_CLINE_IN_TRACEBACK && CYTHON_CLINE_IN_TRACEBACK_RUNTIME
static int __Pyx_CLineForTraceback(PyThreadState *tstate, int c_line);
#else
#define __Pyx_CLineForTraceback(tstate, c_line)  (((CYTHON_CLINE_IN_TRACEBACK)) ? c_line : 0)
#endif

/* CodeObjectCache.proto (used by AddTraceback) */
#if CYTHON_COMPILING_IN_LIMITED_API
typedef PyObject __Pyx_CachedCodeObjectType;
#else
typedef PyCodeObject __Pyx_CachedCodeObjectType;
#endif
typedef struct {
    __Pyx_CachedCodeObjectType* code_object;
    int code_line;
} __Pyx_CodeObjectCacheEntry;
struct __Pyx_CodeObjectCache {
    int count;
    int max_count;
    __Pyx_CodeObjectCacheEntry* entries;
  #if CYTHON_COMPILING_IN_CPYTHON_FREETHREADING
    __pyx_atomic_int_type accessor_count;
  #endif
};
static int __pyx_bisect_code_objects(__Pyx_CodeObjectCacheEntry* entries, int count, int code_line);
static __Pyx_CachedCodeObjectType *__pyx_find_code_object(int code_line);
static void __pyx_insert_code_object(int code_line, __Pyx_CachedCodeObjectType* code_object);

/* AddTraceback.proto */
static void __Pyx_AddTraceback(const char *funcname, int c_line,
                               int py_line, const char *filename);

/* GCCDiagnostics.proto */
#if !defined(__INTEL_COMPILER) && defined(__GNUC__) && (__GNUC__ > 4 || (__GNUC__ == 4 && __GNUC_MINOR__ >= 6))
#define __Pyx_HAS_GCC_DIAGNOSTIC
#endif

/* LengthHint.proto */
#if CYTHON_COMPILING_IN_LIMITED_API
#define __Pyx_PyObject_LengthHint(o, defaultval)  (defaultval)
#else
#define __Pyx_PyObject_LengthHint(o, defaultval)  PyObject_LengthHint(o, defaultval)
#endif

/* CIntToPy.proto */
static CYTHON_INLINE PyObject* __Pyx_PyLong_From_long(long value);

/* CIntFromPy.proto */
static CYTHON_INLINE int __Pyx_PyLong_As_int(PyObject *);

/* CIntToPy.proto */
static CYTHON_INLINE PyObject* __Pyx_PyLong_From_int(int value);

/* CIntFromPy.proto */
static CYTHON_INLINE size_t __Pyx_PyLong_As_size_t(PyObject *);

/* CppExceptionConversion.proto */
#ifndef __Pyx_CppExn2PyErr
#include <new>
#include <typeinfo>
#include <stdexcept>
#include <ios>
static void __Pyx_CppExn2PyErr() {
  try {
    if (PyErr_Occurred())
      ; // let the latest Python exn pass through and ignore the current one
    else
      throw;
  } catch (const std::bad_alloc& exn) {
    PyErr_SetString(PyExc_MemoryError, exn.what());
  } catch (const std::bad_cast& exn) {
    PyErr_SetString(PyExc_TypeError, exn.what());
  } catch (const std::bad_typeid& exn) {
    PyErr_SetString(PyExc_TypeError, exn.what());
  } catch (const std::domain_error& exn) {
    PyErr_SetString(PyExc_ValueError, exn.what());
  } catch (const std::invalid_argument& exn) {
    PyErr_SetString(PyExc_ValueError, exn.what());
  } catch (const std::ios_base::failure& exn) {
    PyErr_SetString(PyExc_IOError, exn.what());
  } catch (const std::out_of_range& exn) {
    PyErr_SetString(PyExc_IndexError, exn.what());
  } catch (const std::overflow_error& exn) {
    PyErr_SetString(PyExc_OverflowError, exn.what());
  } catch (const std::range_error& exn) {
    PyErr_SetString(PyExc_ArithmeticError, exn.what());
  } catch (const std::underflow_error& exn) {
    PyErr_SetString(PyExc_ArithmeticError, exn.what());
  } catch (const std::exception& exn) {
    PyErr_SetString(PyExc_RuntimeError, exn.what());
  }
  catch (...)
  {
    PyErr_SetString(PyExc_RuntimeError, "Unknown exception");
  }
}
#endif

/* FormatTypeName.proto */
#if CYTHON_COMPILING_IN_LIMITED_API
typedef PyObject *__Pyx_TypeName;
#define __Pyx_FMT_TYPENAME "%U"
#define __Pyx_DECREF_TypeName(obj) Py_XDECREF(obj)
#if __PYX_LIMITED_VERSION_HEX >= 0x030d0000
#define __Pyx_PyType_GetFullyQualifiedName PyType_GetFullyQualifiedName
#else
static __Pyx_TypeName __Pyx_PyType_GetFullyQualifiedName(PyTypeObject* tp);
#endif
#else  // !LIMITED_API
typedef const char *__Pyx_TypeName;
#define __Pyx_FMT_TYPENAME "%.200s"
#define __Pyx_PyType_GetFullyQualifiedName(tp) ((tp)->tp_name)
#define __Pyx_DECREF_TypeName(obj)
#endif

/* CIntFromPy.proto */
static CYTHON_INLINE long __Pyx_PyLong_As_long(PyObject *);

/* FastTypeChecks.proto */
#if CYTHON_COMPILING_IN_CPYTHON
#define __Pyx_TypeCheck(obj, type) __Pyx_IsSubtype(Py_TYPE(obj), (PyTypeObject *)type)
#define __Pyx_TypeCheck2(obj, type1, type2) __Pyx_IsAnySubtype2(Py_TYPE(obj), (PyTypeObject *)type1, (PyTypeObject *)type2)
static CYTHON_INLINE int __Pyx_IsSubtype(PyTypeObject *a, PyTypeObject *b);
static CYTHON_INLINE int __Pyx_IsAnySubtype2(PyTypeObject *cls, PyTypeObject *a, PyTypeObject *b);
static CYTHON_INLINE int __Pyx_PyErr_GivenExceptionMatches(PyObject *err, PyObject *type);
static CYTHON_INLINE int __Pyx_PyErr_GivenExceptionMatches2(PyObject *err, PyObject *type1, PyObject *type2);
#else
#define __Pyx_TypeCheck(obj, type) PyObject_TypeCheck(obj, (PyTypeObject *)type)
#define __Pyx_TypeCheck2(obj, type1, type2) (PyObject_TypeCheck(obj, (PyTypeObject *)type1) || PyObject_TypeCheck(obj, (PyTypeObject *)type2))
#define __Pyx_PyErr_GivenExceptionMatches(err, type) PyErr_GivenExceptionMatches(err, type)
static CYTHON_INLINE int __Pyx_PyErr_GivenExceptionMatches2(PyObject *err, PyObject *type1, PyObject *type2) {
    return PyErr_GivenExceptionMatches(err, type1) || PyErr_GivenExceptionMatches(err, type2);
}
#endif
#define __Pyx_PyErr_ExceptionMatches2(err1, err2)  __Pyx_PyErr_GivenExceptionMatches2(__Pyx_PyErr_CurrentExceptionType(), err1, err2)
#define __Pyx_PyException_Check(obj) __Pyx_TypeCheck(obj, PyExc_Exception)
#ifdef PyExceptionInstance_Check
  #define __Pyx_PyBaseException_Check(obj) PyExceptionInstance_Check(obj)
#else
  #define __Pyx_PyBaseException_Check(obj) __Pyx_TypeCheck(obj, PyExc_BaseException)
#endif

/* GetRuntimeVersion.proto */
#if __PYX_LIMITED_VERSION_HEX < 0x030b0000
static unsigned long __Pyx_cached_runtime_version = 0;
static void __Pyx_init_runtime_version(void);
#else
#define __Pyx_init_runtime_version()
#endif
static unsigned long __Pyx_get_runtime_version(void);

/* AddModuleRef.proto */
#if ((CYTHON_COMPILING_IN_CPYTHON_FREETHREADING ) ||\
     __PYX_LIMITED_VERSION_HEX < 0x030d0000)
  static PyObject *__Pyx_PyImport_AddModuleRef(const char *name);
#else
  #define __Pyx_PyImport_AddModuleRef(name) PyImport_AddModuleRef(name)
#endif

/* CheckBinaryVersion.proto */
static int __Pyx_check_binary_version(unsigned long ct_version, unsigned long rt_version, int allow_newer);

/* DecompressString.proto */
static PyObject *__Pyx_DecompressString(const char *s, Py_ssize_t length, int algo);

/* MultiPhaseInitModuleState.proto */
#if CYTHON_PEP489_MULTI_PHASE_INIT && CYTHON_USE_MODULE_STATE
static PyObject *__Pyx_State_FindModule(void*);
static int __Pyx_State_AddModule(PyObject* module, void*);
static int __Pyx_State_RemoveModule(void*);
#elif CYTHON_USE_MODULE_STATE
#define __Pyx_State_FindModule PyState_FindModule
#define __Pyx_State_AddModule PyState_AddModule
#define __Pyx_State_RemoveModule PyState_RemoveModule
#endif

/* #### Code section: module_declarations ### */
/* CythonABIVersion.proto */
#if CYTHON_COMPILING_IN_LIMITED_API
    #if CYTHON_METH_FASTCALL
        #define __PYX_FASTCALL_ABI_SUFFIX  "_fastcall"
    #else
        #define __PYX_FASTCALL_ABI_SUFFIX
    #endif
    #define __PYX_LIMITED_ABI_SUFFIX "limited" __PYX_FASTCALL_ABI_SUFFIX __PYX_AM_SEND_ABI_SUFFIX
#else
    #define __PYX_LIMITED_ABI_SUFFIX
#endif
#if __PYX_HAS_PY_AM_SEND == 1
    #define __PYX_AM_SEND_ABI_SUFFIX
#elif __PYX_HAS_PY_AM_SEND == 2
    #define __PYX_AM_SEND_ABI_SUFFIX "amsendbackport"
#else
    #define __PYX_AM_SEND_ABI_SUFFIX "noamsend"
#endif
#ifndef __PYX_MONITORING_ABI_SUFFIX
    #define __PYX_MONITORING_ABI_SUFFIX
#endif
#if CYTHON_USE_TP_FINALIZE
    #define __PYX_TP_FINALIZE_ABI_SUFFIX
#else
    #define __PYX_TP_FINALIZE_ABI_SUFFIX "nofinalize"
#endif
#if CYTHON_USE_FREELISTS || !defined(__Pyx_AsyncGen_USED)
    #define __PYX_FREELISTS_ABI_SUFFIX
#else
    #define __PYX_FREELISTS_ABI_SUFFIX "nofreelists"
#endif
#define CYTHON_ABI  __PYX_ABI_VERSION __PYX_LIMITED_ABI_SUFFIX __PYX_MONITORING_ABI_SUFFIX __PYX_TP_FINALIZE_ABI_SUFFIX __PYX_FREELISTS_ABI_SUFFIX __PYX_AM_SEND_ABI_SUFFIX
#define __PYX_ABI_MODULE_NAME "_cython_" CYTHON_ABI
#define __PYX_TYPE_MODULE_PREFIX __PYX_ABI_MODULE_NAME "."

static CYTHON_INLINE PyObject *__pyx_f_4cupy_4cuda_6memory_10MemoryPool_device_pool(struct __pyx_obj_4cupy_4cuda_6memory_MemoryPool *__pyx_v_self); /* proto*/

/* Module declarations from "libc.stdint" */

/* Module declarations from "cupy_backends.cuda.api" */

/* Module declarations from "libcpp.vector" */

/* Module declarations from "cupy_backends.cuda.api._driver_enum" */

/* Module declarations from "cupy_backends.cuda.api.driver" */

/* Module declarations from "cupy_backends.cuda.api._runtime_enum" */

/* Module declarations from "cupy_backends.cuda.api.runtime" */
static int *__pyx_vp_13cupy_backends_4cuda_3api_7runtime_errorMemoryAllocation = 0;
#define __pyx_v_13cupy_backends_4cuda_3api_7runtime_errorMemoryAllocation (*__pyx_vp_13cupy_backends_4cuda_3api_7runtime_errorMemoryAllocation)
static int *__pyx_vp_13cupy_backends_4cuda_3api_7runtime_errorInvalidValue = 0;
#define __pyx_v_13cupy_backends_4cuda_3api_7runtime_errorInvalidValue (*__pyx_vp_13cupy_backends_4cuda_3api_7runtime_errorInvalidValue)
static int *__pyx_vp_13cupy_backends_4cuda_3api_7runtime_errorPeerAccessAlreadyEnabled = 0;
#define __pyx_v_13cupy_backends_4cuda_3api_7runtime_errorPeerAccessAlreadyEnabled (*__pyx_vp_13cupy_backends_4cuda_3api_7runtime_errorPeerAccessAlreadyEnabled)
static int *__pyx_vp_13cupy_backends_4cuda_3api_7runtime_errorContextIsDestroyed = 0;
#define __pyx_v_13cupy_backends_4cuda_3api_7runtime_errorContextIsDestroyed (*__pyx_vp_13cupy_backends_4cuda_3api_7runtime_errorContextIsDestroyed)
static int *__pyx_vp_13cupy_backends_4cuda_3api_7runtime_errorInvalidResourceHandle = 0;
#define __pyx_v_13cupy_backends_4cuda_3api_7runtime_errorInvalidResourceHandle (*__pyx_vp_13cupy_backends_4cuda_3api_7runtime_errorInvalidResourceHandle)
static int *__pyx_vp_13cupy_backends_4cuda_3api_7runtime_deviceAttributeComputeCapabilityMajor = 0;
#define __pyx_v_13cupy_backends_4cuda_3api_7runtime_deviceAttributeComputeCapabilityMajor (*__pyx_vp_13cupy_backends_4cuda_3api_7runtime_deviceAttributeComputeCapabilityMajor)
static int *__pyx_vp_13cupy_backends_4cuda_3api_7runtime_deviceAttributeComputeCapabilityMinor = 0;
#define __pyx_v_13cupy_backends_4cuda_3api_7runtime_deviceAttributeComputeCapabilityMinor (*__pyx_vp_13cupy_backends_4cuda_3api_7runtime_deviceAttributeComputeCapabilityMinor)
static int *__pyx_vp_13cupy_backends_4cuda_3api_7runtime__is_hip_environment = 0;
#define __pyx_v_13cupy_backends_4cuda_3api_7runtime__is_hip_environment (*__pyx_vp_13cupy_backends_4cuda_3api_7runtime__is_hip_environment)

/* Module declarations from "cupy_backends.cuda.libs" */

/* Module declarations from "cupy_backends.cuda.libs.cusolver" */
static int (*__pyx_f_13cupy_backends_4cuda_4libs_8cusolver__getVersionNumber)(int __pyx_skip_dispatch); /*proto*/
static int (*__pyx_f_13cupy_backends_4cuda_4libs_8cusolver_sgeqrf_bufferSize)(intptr_t, int, int, size_t, int, int __pyx_skip_dispatch); /*proto*/
static int (*__pyx_f_13cupy_backends_4cuda_4libs_8cusolver_dgeqrf_bufferSize)(intptr_t, int, int, size_t, int, int __pyx_skip_dispatch); /*proto*/
static int (*__pyx_f_13cupy_backends_4cuda_4libs_8cusolver_cgeqrf_bufferSize)(intptr_t, int, int, size_t, int, int __pyx_skip_dispatch); /*proto*/
static int (*__pyx_f_13cupy_backends_4cuda_4libs_8cusolver_zgeqrf_bufferSize)(intptr_t, int, int, size_t, int, int __pyx_skip_dispatch); /*proto*/
static int (*__pyx_f_13cupy_backends_4cuda_4libs_8cusolver_sorgqr_bufferSize)(intptr_t, int, int, int, size_t, int, size_t, int __pyx_skip_dispatch); /*proto*/
static int (*__pyx_f_13cupy_backends_4cuda_4libs_8cusolver_dorgqr_bufferSize)(intptr_t, int, int, int, size_t, int, size_t, int __pyx_skip_dispatch); /*proto*/
static int (*__pyx_f_13cupy_backends_4cuda_4libs_8cusolver_cungqr_bufferSize)(intptr_t, int, int, int, size_t, int, size_t, int __pyx_skip_dispatch); /*proto*/
static int (*__pyx_f_13cupy_backends_4cuda_4libs_8cusolver_zungqr_bufferSize)(intptr_t, int, int, int, size_t, int, size_t, int __pyx_skip_dispatch); /*proto*/
static int (*__pyx_f_13cupy_backends_4cuda_4libs_8cusolver_sgesvd_bufferSize)(intptr_t, int, int, int __pyx_skip_dispatch); /*proto*/
static int (*__pyx_f_13cupy_backends_4cuda_4libs_8cusolver_dgesvd_bufferSize)(intptr_t, int, int, int __pyx_skip_dispatch); /*proto*/
static int (*__pyx_f_13cupy_backends_4cuda_4libs_8cusolver_cgesvd_bufferSize)(intptr_t, int, int, int __pyx_skip_dispatch); /*proto*/
static int (*__pyx_f_13cupy_backends_4cuda_4libs_8cusolver_zgesvd_bufferSize)(intptr_t, int, int, int __pyx_skip_dispatch); /*proto*/

/* Module declarations from "cupy.cuda" */

/* Module declarations from "cython" */

/* Module declarations from "libcpp" */

/* Module declarations from "libcpp.utility" */

/* Module declarations from "libcpp.map" */

/* Module declarations from "cupy.cuda.device" */
static int (*__pyx_f_4cupy_4cuda_6device_get_device_id)(int __pyx_skip_dispatch); /*proto*/

/* Module declarations from "cupy.cuda.memory" */
static struct __pyx_obj_4cupy_4cuda_6memory_MemoryPointer *(*__pyx_f_4cupy_4cuda_6memory_alloc)(PyObject *, int __pyx_skip_dispatch); /*proto*/

/* Module declarations from "cupy.cuda.function" */

/* Module declarations from "cupy._core._carray" */

/* Module declarations from "cupy._core.core" */
static struct __pyx_obj_4cupy_5_core_4core__ndarray_base *(*__pyx_f_4cupy_5_core_4core__internal_ascontiguousarray)(struct __pyx_obj_4cupy_5_core_4core__ndarray_base *, int __pyx_skip_dispatch); /*proto*/
static struct __pyx_obj_4cupy_5_core_4core__ndarray_base *(*__pyx_f_4cupy_5_core_4core__convert_object_with_cuda_array_interface)(PyObject *, int __pyx_skip_dispatch); /*proto*/
static struct __pyx_obj_4cupy_5_core_4core__ndarray_base *(*__pyx_f_4cupy_5_core_4core__ndarray_init)(PyObject *, __pyx_t_4cupy_5_core_7_carray_shape_t const &, PyObject *, PyObject *); /*proto*/

/* Module declarations from "cupyx.cusolver" */
static PyObject *__pyx_f_5cupyx_8cusolver__gesvdj_batched(PyObject *, PyObject *, PyObject *, PyObject *, int __pyx_skip_dispatch); /*proto*/
static PyObject *__pyx_f_5cupyx_8cusolver__gesvd_batched(PyObject *, PyObject *, PyObject *, PyObject *, PyObject *, int __pyx_skip_dispatch); /*proto*/
static PyObject *__pyx_f_5cupyx_8cusolver__geqrf_orgqr_batched(PyObject *, PyObject *, int __pyx_skip_dispatch); /*proto*/
static std::vector<Py_ssize_t>  __pyx_convert_vector_from_py_Py_ssize_t(PyObject *); /*proto*/
/* #### Code section: typeinfo ### */
/* #### Code section: before_global_var ### */
#define __Pyx_MODULE_NAME "cupyx.cusolver"
extern int __pyx_module_is_main_cupyx__cusolver;
int __pyx_module_is_main_cupyx__cusolver = 0;

/* Implementation of "cupyx.cusolver" */
/* #### Code section: global_var ### */
/* #### Code section: string_decls ### */
/* #### Code section: decls ### */
static PyObject *__pyx_pf_5cupyx_8cusolver_check_availability(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_name); /* proto */
static PyObject *__pyx_pf_5cupyx_8cusolver_2gesvdj(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_a, PyObject *__pyx_v_full_matrices, PyObject *__pyx_v_compute_uv, PyObject *__pyx_v_overwrite_a); /* proto */
static PyObject *__pyx_pf_5cupyx_8cusolver_4_gesvdj_batched(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_a, PyObject *__pyx_v_full_matrices, PyObject *__pyx_v_compute_uv, PyObject *__pyx_v_overwrite_a); /* proto */
static PyObject *__pyx_pf_5cupyx_8cusolver_6_gesvd_batched(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_a, PyObject *__pyx_v_a_dtype, PyObject *__pyx_v_full_matrices, PyObject *__pyx_v_compute_uv, PyObject *__pyx_v_overwrite_a); /* proto */
static PyObject *__pyx_pf_5cupyx_8cusolver_8gesvda(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_a, PyObject *__pyx_v_compute_uv); /* proto */
static PyObject *__pyx_pf_5cupyx_8cusolver_10syevj(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_a, PyObject *__pyx_v_UPLO, PyObject *__pyx_v_with_eigen_vector); /* proto */
static PyObject *__pyx_pf_5cupyx_8cusolver_12_syevj_batched(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_a, PyObject *__pyx_v_UPLO, PyObject *__pyx_v_with_eigen_vector); /* proto */
static PyObject *__pyx_pf_5cupyx_8cusolver_14gesv(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_a, PyObject *__pyx_v_b); /* proto */
static PyObject *__pyx_pf_5cupyx_8cusolver_16gels(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_a, PyObject *__pyx_v_b); /* proto */
static PyObject *__pyx_pf_5cupyx_8cusolver_18csrlsvqr(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_A, PyObject *__pyx_v_b, PyObject *__pyx_v_tol, PyObject *__pyx_v_reorder); /* proto */
static PyObject *__pyx_pf_5cupyx_8cusolver_20_geqrf_orgqr_batched(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_a, PyObject *__pyx_v_mode); /* proto */
/* #### Code section: late_includes ### */
/* #### Code section: module_state ### */
/* SmallCodeConfig */
#ifndef CYTHON_SMALL_CODE
#if defined(__clang__)
    #define CYTHON_SMALL_CODE
#elif defined(__GNUC__) && (__GNUC__ > 4 || (__GNUC__ == 4 && __GNUC_MINOR__ >= 3))
    #define CYTHON_SMALL_CODE __attribute__((cold))
#else
    #define CYTHON_SMALL_CODE
#endif
#endif

typedef struct {
  PyObject *__pyx_d;
  PyObject *__pyx_b;
  PyObject *__pyx_cython_runtime;
  PyObject *__pyx_empty_tuple;
  PyObject *__pyx_empty_bytes;
  PyObject *__pyx_empty_unicode;
  PyTypeObject *__pyx_ptype_13cupy_backends_4cuda_3api_7runtime_PointerAttributes;
  PyTypeObject *__pyx_ptype_13cupy_backends_4cuda_3api_7runtime_MemPoolProps;
  PyTypeObject *__pyx_ptype_4cupy_4cuda_6device_Handle;
  PyTypeObject *__pyx_ptype_4cupy_4cuda_6device_Device;
  PyTypeObject *__pyx_ptype_4cupy_4cuda_6memory_BaseMemory;
  PyTypeObject *__pyx_ptype_4cupy_4cuda_6memory_SystemMemory;
  PyTypeObject *__pyx_ptype_4cupy_4cuda_6memory_MemoryPointer;
  PyTypeObject *__pyx_ptype_4cupy_4cuda_6memory_MemoryPool;
  PyTypeObject *__pyx_ptype_4cupy_4cuda_6memory_CFunctionAllocatorMemory;
  PyTypeObject *__pyx_ptype_4cupy_4cuda_6memory_CFunctionAllocator;
  PyTypeObject *__pyx_ptype_4cupy_4cuda_6memory_PythonFunctionAllocatorMemory;
  PyTypeObject *__pyx_ptype_4cupy_4cuda_6memory_PythonFunctionAllocator;
  PyTypeObject *__pyx_ptype_4cupy_4cuda_8function_CPointer;
  PyTypeObject *__pyx_ptype_4cupy_4cuda_8function_Function;
  PyTypeObject *__pyx_ptype_4cupy_4cuda_8function_Module;
  PyTypeObject *__pyx_ptype_4cupy_4cuda_8function_LinkState;
  PyTypeObject *__pyx_ptype_4cupy_5_core_7_carray_mdspan;
  PyTypeObject *__pyx_ptype_4cupy_5_core_7_carray_CArray;
  PyTypeObject *__pyx_ptype_4cupy_5_core_7_carray_CIndexer;
  PyTypeObject *__pyx_ptype_4cupy_5_core_7_carray_Indexer;
  PyTypeObject *__pyx_ptype_4cupy_5_core_4core__ndarray_base;
  __Pyx_CachedCFunction __pyx_umethod_PyDict_Type_items;
  __Pyx_CachedCFunction __pyx_umethod_PyDict_Type_pop;
  __Pyx_CachedCFunction __pyx_umethod_PyDict_Type_values;
  PyObject *__pyx_slice[3];
  PyObject *__pyx_tuple[9];
  PyObject *__pyx_string_tab[233];
  PyObject *__pyx_number_tab[12];
/* #### Code section: module_state_contents ### */
/* CodeObjectCache.module_state_decls */
struct __Pyx_CodeObjectCache __pyx_code_cache;

/* #### Code section: module_state_end ### */
} __pyx_mstatetype;

#if CYTHON_USE_MODULE_STATE
#ifdef __cplusplus
namespace {
extern struct PyModuleDef __pyx_moduledef;
} /* anonymous namespace */
#else
static struct PyModuleDef __pyx_moduledef;
#endif

#define __pyx_mstate_global (__Pyx_PyModule_GetState(__Pyx_State_FindModule(&__pyx_moduledef)))

#define __pyx_m (__Pyx_State_FindModule(&__pyx_moduledef))
#else
static __pyx_mstatetype __pyx_mstate_global_static =
#ifdef __cplusplus
    {};
#else
    {0};
#endif
static __pyx_mstatetype * const __pyx_mstate_global = &__pyx_mstate_global_static;
#endif
/* #### Code section: constant_name_defines ### */
#define __pyx_kp_u_ __pyx_string_tab[0]
#define __pyx_kp_u_A_is_not_positive_definite_or_ne __pyx_string_tab[1]
#define __pyx_kp_u_A_must_be_CSR_sparse_matrix __pyx_string_tab[2]
#define __pyx_kp_u_Invalid_dtype_actual __pyx_string_tab[3]
#define __pyx_kp_u_No_available_version_information __pyx_string_tab[4]
#define __pyx_kp_u_Only_float_and_double_and_cuComp __pyx_string_tab[5]
#define __pyx_kp_u_This_function_is_disabled_on_HIP __pyx_string_tab[6]
#define __pyx_kp_u_UPLO_argument_must_be_L_or_U __pyx_string_tab[7]
#define __pyx_kp_u__2 __pyx_string_tab[8]
#define __pyx_kp_u__3 __pyx_string_tab[9]
#define __pyx_kp_u_a_must_be_a_square_matrix __pyx_string_tab[10]
#define __pyx_kp_u_a_ndim_must_be_2_actual __pyx_string_tab[11]
#define __pyx_kp_u_b_must_be_cupy_ndarray __pyx_string_tab[12]
#define __pyx_kp_u_b_ndim_must_be_1_actual __pyx_string_tab[13]
#define __pyx_kp_u_b_ndim_must_be_1_or_2_actual __pyx_string_tab[14]
#define __pyx_kp_u_b_ndim_must_be_1_or_2_actual_2 __pyx_string_tab[15]
#define __pyx_kp_u_csrlsvqr_is_not_available __pyx_string_tab[16]
#define __pyx_kp_u_dtype_mismatch __pyx_string_tab[17]
#define __pyx_kp_u_dtype_mismatch_a_b __pyx_string_tab[18]
#define __pyx_kp_u_dtype_must_be_float32_float64_co __pyx_string_tab[19]
#define __pyx_kp_u_gels_has_failed __pyx_string_tab[20]
#define __pyx_kp_u_gels_is_not_available __pyx_string_tab[21]
#define __pyx_kp_u_gels_reached_maximum_allowed_ite __pyx_string_tab[22]
#define __pyx_kp_u_gesv_has_failed __pyx_string_tab[23]
#define __pyx_kp_u_gesv_is_not_available __pyx_string_tab[24]
#define __pyx_kp_u_gesvda_is_not_available __pyx_string_tab[25]
#define __pyx_kp_u_gesvdj_is_not_available __pyx_string_tab[26]
#define __pyx_kp_u_got __pyx_string_tab[27]
#define __pyx_kp_u_has_incorrect_type_expected __pyx_string_tab[28]
#define __pyx_kp_u_invalid_shape __pyx_string_tab[29]
#define __pyx_kp_u_m_must_be_equal_to_or_greater_th __pyx_string_tab[30]
#define __pyx_kp_u_shape_mismatch_a_b __pyx_string_tab[31]
#define __pyx_kp_u_syevj_is_not_available __pyx_string_tab[32]
#define __pyx_kp_u_unsupported_dtype_actual __pyx_string_tab[33]
#define __pyx_n_u_A __pyx_string_tab[34]
#define __pyx_n_u_C __pyx_string_tab[35]
#define __pyx_n_u_COMPUTE_TYPE_FP16 __pyx_string_tab[36]
#define __pyx_n_u_COMPUTE_TYPE_FP32 __pyx_string_tab[37]
#define __pyx_n_u_COMPUTE_TYPE_TF32 __pyx_string_tab[38]
#define __pyx_n_u_CUBLAS_FILL_MODE_LOWER __pyx_string_tab[39]
#define __pyx_n_u_CUBLAS_FILL_MODE_UPPER __pyx_string_tab[40]
#define __pyx_n_u_CUSOLVERError __pyx_string_tab[41]
#define __pyx_n_u_CUSOLVER_EIG_MODE_NOVECTOR __pyx_string_tab[42]
#define __pyx_n_u_CUSOLVER_EIG_MODE_VECTOR __pyx_string_tab[43]
#define __pyx_n_u_D __pyx_string_tab[44]
#define __pyx_n_u_Device __pyx_string_tab[45]
#define __pyx_n_u_F __pyx_string_tab[46]
#define __pyx_n_u_FD __pyx_string_tab[47]
#define __pyx_n_u_L __pyx_string_tab[48]
#define __pyx_n_u_Pyx_PyDict_NextRef __pyx_string_tab[49]
#define __pyx_n_u_U __pyx_string_tab[50]
#define __pyx_n_u_UPLO __pyx_string_tab[51]
#define __pyx_n_u_a __pyx_string_tab[52]
#define __pyx_n_u_a_dtype __pyx_string_tab[53]
#define __pyx_n_u_array __pyx_string_tab[54]
#define __pyx_n_u_ascontiguousarray __pyx_string_tab[55]
#define __pyx_n_u_astype __pyx_string_tab[56]
#define __pyx_n_u_available_compute_capability __pyx_string_tab[57]
#define __pyx_n_u_available_cuda_version __pyx_string_tab[58]
#define __pyx_n_u_available_hip_version __pyx_string_tab[59]
#define __pyx_n_u_b __pyx_string_tab[60]
#define __pyx_n_u_bufferSize __pyx_string_tab[61]
#define __pyx_n_u_c __pyx_string_tab[62]
#define __pyx_n_u_cgesvdaStridedBatched __pyx_string_tab[63]
#define __pyx_n_u_cgesvdaStridedBatched_bufferSize __pyx_string_tab[64]
#define __pyx_n_u_cgesvdj __pyx_string_tab[65]
#define __pyx_n_u_cgesvdjBatched __pyx_string_tab[66]
#define __pyx_n_u_cgesvdjBatched_bufferSize __pyx_string_tab[67]
#define __pyx_n_u_cgesvdj_bufferSize __pyx_string_tab[68]
#define __pyx_n_u_char __pyx_string_tab[69]
#define __pyx_n_u_check_availability __pyx_string_tab[70]
#define __pyx_n_u_check_cusolver_dev_info_if_sync __pyx_string_tab[71]
#define __pyx_n_u_cheevj __pyx_string_tab[72]
#define __pyx_n_u_cheevjBatched __pyx_string_tab[73]
#define __pyx_n_u_cheevjBatched_bufferSize __pyx_string_tab[74]
#define __pyx_n_u_cheevj_bufferSize __pyx_string_tab[75]
#define __pyx_n_u_class_getitem __pyx_string_tab[76]
#define __pyx_n_u_cline_in_traceback __pyx_string_tab[77]
#define __pyx_n_u_complete __pyx_string_tab[78]
#define __pyx_n_u_compute_uv __pyx_string_tab[79]
#define __pyx_n_u_conj __pyx_string_tab[80]
#define __pyx_n_u_copy __pyx_string_tab[81]
#define __pyx_n_u_createGesvdjInfo __pyx_string_tab[82]
#define __pyx_n_u_createSyevjInfo __pyx_string_tab[83]
#define __pyx_n_u_csrlsvqr __pyx_string_tab[84]
#define __pyx_n_u_ctypes __pyx_string_tab[85]
#define __pyx_n_u_cublas __pyx_string_tab[86]
#define __pyx_n_u_cublas_2 __pyx_string_tab[87]
#define __pyx_n_u_cuda_array_interface __pyx_string_tab[88]
#define __pyx_n_u_cupy __pyx_string_tab[89]
#define __pyx_n_u_cupy_2 __pyx_string_tab[90]
#define __pyx_n_u_cupy__core __pyx_string_tab[91]
#define __pyx_n_u_cupy_backends_cuda_api __pyx_string_tab[92]
#define __pyx_n_u_cupy_backends_cuda_libs __pyx_string_tab[93]
#define __pyx_n_u_cupy_cuda __pyx_string_tab[94]
#define __pyx_n_u_cupy_get_ndarray __pyx_string_tab[95]
#define __pyx_n_u_cupyx __pyx_string_tab[96]
#define __pyx_n_u_cupyx_2 __pyx_string_tab[97]
#define __pyx_n_u_cupyx_cusolver __pyx_string_tab[98]
#define __pyx_n_u_cusolver __pyx_string_tab[99]
#define __pyx_n_u_cusolver_2 __pyx_string_tab[100]
#define __pyx_n_u_cusolver_handle __pyx_string_tab[101]
#define __pyx_n_u_d __pyx_string_tab[102]
#define __pyx_n_u_data __pyx_string_tab[103]
#define __pyx_n_u_descr __pyx_string_tab[104]
#define __pyx_n_u_descriptor __pyx_string_tab[105]
#define __pyx_n_u_destroyGesvdjInfo __pyx_string_tab[106]
#define __pyx_n_u_destroySyevjInfo __pyx_string_tab[107]
#define __pyx_n_u_device __pyx_string_tab[108]
#define __pyx_n_u_device_2 __pyx_string_tab[109]
#define __pyx_n_u_dgesvdaStridedBatched __pyx_string_tab[110]
#define __pyx_n_u_dgesvdaStridedBatched_bufferSize __pyx_string_tab[111]
#define __pyx_n_u_dgesvdj __pyx_string_tab[112]
#define __pyx_n_u_dgesvdjBatched __pyx_string_tab[113]
#define __pyx_n_u_dgesvdjBatched_bufferSize __pyx_string_tab[114]
#define __pyx_n_u_dgesvdj_bufferSize __pyx_string_tab[115]
#define __pyx_n_u_dsyevj __pyx_string_tab[116]
#define __pyx_n_u_dsyevjBatched __pyx_string_tab[117]
#define __pyx_n_u_dsyevjBatched_bufferSize __pyx_string_tab[118]
#define __pyx_n_u_dsyevj_bufferSize __pyx_string_tab[119]
#define __pyx_n_u_dtype __pyx_string_tab[120]
#define __pyx_n_u_empty __pyx_string_tab[121]
#define __pyx_n_u_empty_like __pyx_string_tab[122]
#define __pyx_n_u_f __pyx_string_tab[123]
#define __pyx_n_u_fd __pyx_string_tab[124]
#define __pyx_n_u_float64 __pyx_string_tab[125]
#define __pyx_n_u_for_each_device __pyx_string_tab[126]
#define __pyx_n_u_format __pyx_string_tab[127]
#define __pyx_n_u_full_matrices __pyx_string_tab[128]
#define __pyx_n_u_geev __pyx_string_tab[129]
#define __pyx_n_u_gels __pyx_string_tab[130]
#define __pyx_n_u_geqrf __pyx_string_tab[131]
#define __pyx_n_u_gesv __pyx_string_tab[132]
#define __pyx_n_u_gesvd __pyx_string_tab[133]
#define __pyx_n_u_gesvda __pyx_string_tab[134]
#define __pyx_n_u_gesvdj __pyx_string_tab[135]
#define __pyx_n_u_gesvdjBatched __pyx_string_tab[136]
#define __pyx_n_u_get_compute_capability __pyx_string_tab[137]
#define __pyx_n_u_get_compute_type __pyx_string_tab[138]
#define __pyx_n_u_get_cusolver_handle __pyx_string_tab[139]
#define __pyx_n_u_get_cusolver_sp_handle __pyx_string_tab[140]
#define __pyx_n_u_h __pyx_string_tab[141]
#define __pyx_n_u_indices __pyx_string_tab[142]
#define __pyx_n_u_indptr __pyx_string_tab[143]
#define __pyx_n_u_inf __pyx_string_tab[144]
#define __pyx_n_u_int32 __pyx_string_tab[145]
#define __pyx_n_u_int8 __pyx_string_tab[146]
#define __pyx_n_u_is_hip __pyx_string_tab[147]
#define __pyx_n_u_isspmatrix_csr __pyx_string_tab[148]
#define __pyx_n_u_item __pyx_string_tab[149]
#define __pyx_n_u_items __pyx_string_tab[150]
#define __pyx_n_u_itemsize __pyx_string_tab[151]
#define __pyx_n_u_k __pyx_string_tab[152]
#define __pyx_n_u_linalg __pyx_string_tab[153]
#define __pyx_n_u_linalg_2 __pyx_string_tab[154]
#define __pyx_n_u_linalg_common_type __pyx_string_tab[155]
#define __pyx_n_u_lower __pyx_string_tab[156]
#define __pyx_n_u_main __pyx_string_tab[157]
#define __pyx_n_u_mat_ptrs __pyx_string_tab[158]
#define __pyx_n_u_memoize __pyx_string_tab[159]
#define __pyx_n_u_mode __pyx_string_tab[160]
#define __pyx_n_u_module __pyx_string_tab[161]
#define __pyx_n_u_name __pyx_string_tab[162]
#define __pyx_n_u_name_2 __pyx_string_tab[163]
#define __pyx_n_u_ndarray __pyx_string_tab[164]
#define __pyx_n_u_ndim __pyx_string_tab[165]
#define __pyx_n_u_nnz __pyx_string_tab[166]
#define __pyx_n_u_numpy __pyx_string_tab[167]
#define __pyx_n_u_numpy_2 __pyx_string_tab[168]
#define __pyx_n_u_order __pyx_string_tab[169]
#define __pyx_n_u_orgqr __pyx_string_tab[170]
#define __pyx_n_u_overwrite_a __pyx_string_tab[171]
#define __pyx_n_u_pop __pyx_string_tab[172]
#define __pyx_n_u_potrfBatched __pyx_string_tab[173]
#define __pyx_n_u_potrsBatched __pyx_string_tab[174]
#define __pyx_n_u_prod __pyx_string_tab[175]
#define __pyx_n_u_ptr __pyx_string_tab[176]
#define __pyx_n_u_pyx_vtable __pyx_string_tab[177]
#define __pyx_n_u_qualname __pyx_string_tab[178]
#define __pyx_n_u_r __pyx_string_tab[179]
#define __pyx_n_u_raw __pyx_string_tab[180]
#define __pyx_n_u_reject_float16 __pyx_string_tab[181]
#define __pyx_n_u_reorder __pyx_string_tab[182]
#define __pyx_n_u_reshape __pyx_string_tab[183]
#define __pyx_n_u_routines_linalg __pyx_string_tab[184]
#define __pyx_n_u_runtime __pyx_string_tab[185]
#define __pyx_n_u_runtime_2 __pyx_string_tab[186]
#define __pyx_n_u_s __pyx_string_tab[187]
#define __pyx_n_u_scipy __pyx_string_tab[188]
#define __pyx_n_u_setdefault __pyx_string_tab[189]
#define __pyx_n_u_sgesvdaStridedBatched __pyx_string_tab[190]
#define __pyx_n_u_sgesvdaStridedBatched_bufferSize __pyx_string_tab[191]
#define __pyx_n_u_sgesvdj __pyx_string_tab[192]
#define __pyx_n_u_sgesvdjBatched __pyx_string_tab[193]
#define __pyx_n_u_sgesvdjBatched_bufferSize __pyx_string_tab[194]
#define __pyx_n_u_sgesvdj_bufferSize __pyx_string_tab[195]
#define __pyx_n_u_shape __pyx_string_tab[196]
#define __pyx_n_u_sparse __pyx_string_tab[197]
#define __pyx_n_u_ssyevj __pyx_string_tab[198]
#define __pyx_n_u_ssyevjBatched __pyx_string_tab[199]
#define __pyx_n_u_ssyevjBatched_bufferSize __pyx_string_tab[200]
#define __pyx_n_u_ssyevj_bufferSize __pyx_string_tab[201]
#define __pyx_n_u_swapaxes __pyx_string_tab[202]
#define __pyx_n_u_syevj __pyx_string_tab[203]
#define __pyx_n_u_syevj_batched __pyx_string_tab[204]
#define __pyx_n_u_test __pyx_string_tab[205]
#define __pyx_n_u_tol __pyx_string_tab[206]
#define __pyx_n_u_transpose __pyx_string_tab[207]
#define __pyx_n_u_triu __pyx_string_tab[208]
#define __pyx_n_u_util __pyx_string_tab[209]
#define __pyx_n_u_values __pyx_string_tab[210]
#define __pyx_n_u_warn __pyx_string_tab[211]
#define __pyx_n_u_warnings __pyx_string_tab[212]
#define __pyx_n_u_warnings_2 __pyx_string_tab[213]
#define __pyx_n_u_with_eigen_vector __pyx_string_tab[214]
#define __pyx_n_u_x __pyx_string_tab[215]
#define __pyx_n_u_y __pyx_string_tab[216]
#define __pyx_n_u_z __pyx_string_tab[217]
#define __pyx_n_u_zgesvdaStridedBatched __pyx_string_tab[218]
#define __pyx_n_u_zgesvdaStridedBatched_bufferSize __pyx_string_tab[219]
#define __pyx_n_u_zgesvdj __pyx_string_tab[220]
#define __pyx_n_u_zgesvdjBatched __pyx_string_tab[221]
#define __pyx_n_u_zgesvdjBatched_bufferSize __pyx_string_tab[222]
#define __pyx_n_u_zgesvdj_bufferSize __pyx_string_tab[223]
#define __pyx_n_u_zheevj __pyx_string_tab[224]
#define __pyx_n_u_zheevjBatched __pyx_string_tab[225]
#define __pyx_n_u_zheevjBatched_bufferSize __pyx_string_tab[226]
#define __pyx_n_u_zheevj_bufferSize __pyx_string_tab[227]
#define __pyx_kp_b_int__is_hip_environment_deviceAt __pyx_string_tab[228]
#define __pyx_kp_b_int_int___pyx_skip_dispatch_get __pyx_string_tab[229]
#define __pyx_kp_b_int_int___pyx_skip_dispatch_int __pyx_string_tab[230]
#define __pyx_kp_b_struct___pyx_obj_4cupy_4cuda_6me __pyx_string_tab[231]
#define __pyx_kp_b_struct___pyx_obj_4cupy_5_core_4c __pyx_string_tab[232]
#define __pyx_int_0 __pyx_number_tab[0]
#define __pyx_int_neg_1 __pyx_number_tab[1]
#define __pyx_int_1 __pyx_number_tab[2]
#define __pyx_int_neg_2 __pyx_number_tab[3]
#define __pyx_int_2 __pyx_number_tab[4]
#define __pyx_int_3 __pyx_number_tab[5]
#define __pyx_int_neg_50 __pyx_number_tab[6]
#define __pyx_int_70 __pyx_number_tab[7]
#define __pyx_int_306 __pyx_number_tab[8]
#define __pyx_int_309 __pyx_number_tab[9]
#define __pyx_int_402 __pyx_number_tab[10]
#define __pyx_int_11700 __pyx_number_tab[11]
/* #### Code section: module_state_clear ### */
#if CYTHON_USE_MODULE_STATE
static CYTHON_SMALL_CODE int __pyx_m_clear(PyObject *m) {
  __pyx_mstatetype *clear_module_state = __Pyx_PyModule_GetState(m);
  if (!clear_module_state) return 0;
  Py_CLEAR(clear_module_state->__pyx_d);
  Py_CLEAR(clear_module_state->__pyx_b);
  Py_CLEAR(clear_module_state->__pyx_cython_runtime);
  Py_CLEAR(clear_module_state->__pyx_empty_tuple);
  Py_CLEAR(clear_module_state->__pyx_empty_bytes);
  Py_CLEAR(clear_module_state->__pyx_empty_unicode);
  #if CYTHON_PEP489_MULTI_PHASE_INIT
  __Pyx_State_RemoveModule(NULL);
  #endif
  Py_CLEAR(clear_module_state->__pyx_ptype_13cupy_backends_4cuda_3api_7runtime_PointerAttributes);
  Py_CLEAR(clear_module_state->__pyx_ptype_13cupy_backends_4cuda_3api_7runtime_MemPoolProps);
  Py_CLEAR(clear_module_state->__pyx_ptype_4cupy_4cuda_6device_Handle);
  Py_CLEAR(clear_module_state->__pyx_ptype_4cupy_4cuda_6device_Device);
  Py_CLEAR(clear_module_state->__pyx_ptype_4cupy_4cuda_6memory_BaseMemory);
  Py_CLEAR(clear_module_state->__pyx_ptype_4cupy_4cuda_6memory_SystemMemory);
  Py_CLEAR(clear_module_state->__pyx_ptype_4cupy_4cuda_6memory_MemoryPointer);
  Py_CLEAR(clear_module_state->__pyx_ptype_4cupy_4cuda_6memory_MemoryPool);
  Py_CLEAR(clear_module_state->__pyx_ptype_4cupy_4cuda_6memory_CFunctionAllocatorMemory);
  Py_CLEAR(clear_module_state->__pyx_ptype_4cupy_4cuda_6memory_CFunctionAllocator);
  Py_CLEAR(clear_module_state->__pyx_ptype_4cupy_4cuda_6memory_PythonFunctionAllocatorMemory);
  Py_CLEAR(clear_module_state->__pyx_ptype_4cupy_4cuda_6memory_PythonFunctionAllocator);
  Py_CLEAR(clear_module_state->__pyx_ptype_4cupy_4cuda_8function_CPointer);
  Py_CLEAR(clear_module_state->__pyx_ptype_4cupy_4cuda_8function_Function);
  Py_CLEAR(clear_module_state->__pyx_ptype_4cupy_4cuda_8function_Module);
  Py_CLEAR(clear_module_state->__pyx_ptype_4cupy_4cuda_8function_LinkState);
  Py_CLEAR(clear_module_state->__pyx_ptype_4cupy_5_core_7_carray_mdspan);
  Py_CLEAR(clear_module_state->__pyx_ptype_4cupy_5_core_7_carray_CArray);
  Py_CLEAR(clear_module_state->__pyx_ptype_4cupy_5_core_7_carray_CIndexer);
  Py_CLEAR(clear_module_state->__pyx_ptype_4cupy_5_core_7_carray_Indexer);
  Py_CLEAR(clear_module_state->__pyx_ptype_4cupy_5_core_4core__ndarray_base);
  for (int i=0; i<3; ++i) { Py_CLEAR(clear_module_state->__pyx_slice[i]); }
  for (int i=0; i<9; ++i) { Py_CLEAR(clear_module_state->__pyx_tuple[i]); }
  for (int i=0; i<233; ++i) { Py_CLEAR(clear_module_state->__pyx_string_tab[i]); }
  for (int i=0; i<12; ++i) { Py_CLEAR(clear_module_state->__pyx_number_tab[i]); }
/* #### Code section: module_state_clear_contents ### */
/* #### Code section: module_state_clear_end ### */
return 0;
}
#endif
/* #### Code section: module_state_traverse ### */
#if CYTHON_USE_MODULE_STATE
static CYTHON_SMALL_CODE int __pyx_m_traverse(PyObject *m, visitproc visit, void *arg) {
  __pyx_mstatetype *traverse_module_state = __Pyx_PyModule_GetState(m);
  if (!traverse_module_state) return 0;
  Py_VISIT(traverse_module_state->__pyx_d);
  Py_VISIT(traverse_module_state->__pyx_b);
  Py_VISIT(traverse_module_state->__pyx_cython_runtime);
  __Pyx_VISIT_CONST(traverse_module_state->__pyx_empty_tuple);
  __Pyx_VISIT_CONST(traverse_module_state->__pyx_empty_bytes);
  __Pyx_VISIT_CONST(traverse_module_state->__pyx_empty_unicode);
  Py_VISIT(traverse_module_state->__pyx_ptype_13cupy_backends_4cuda_3api_7runtime_PointerAttributes);
  Py_VISIT(traverse_module_state->__pyx_ptype_13cupy_backends_4cuda_3api_7runtime_MemPoolProps);
  Py_VISIT(traverse_module_state->__pyx_ptype_4cupy_4cuda_6device_Handle);
  Py_VISIT(traverse_module_state->__pyx_ptype_4cupy_4cuda_6device_Device);
  Py_VISIT(traverse_module_state->__pyx_ptype_4cupy_4cuda_6memory_BaseMemory);
  Py_VISIT(traverse_module_state->__pyx_ptype_4cupy_4cuda_6memory_SystemMemory);
  Py_VISIT(traverse_module_state->__pyx_ptype_4cupy_4cuda_6memory_MemoryPointer);
  Py_VISIT(traverse_module_state->__pyx_ptype_4cupy_4cuda_6memory_MemoryPool);
  Py_VISIT(traverse_module_state->__pyx_ptype_4cupy_4cuda_6memory_CFunctionAllocatorMemory);
  Py_VISIT(traverse_module_state->__pyx_ptype_4cupy_4cuda_6memory_CFunctionAllocator);
  Py_VISIT(traverse_module_state->__pyx_ptype_4cupy_4cuda_6memory_PythonFunctionAllocatorMemory);
  Py_VISIT(traverse_module_state->__pyx_ptype_4cupy_4cuda_6memory_PythonFunctionAllocator);
  Py_VISIT(traverse_module_state->__pyx_ptype_4cupy_4cuda_8function_CPointer);
  Py_VISIT(traverse_module_state->__pyx_ptype_4cupy_4cuda_8function_Function);
  Py_VISIT(traverse_module_state->__pyx_ptype_4cupy_4cuda_8function_Module);
  Py_VISIT(traverse_module_state->__pyx_ptype_4cupy_4cuda_8function_LinkState);
  Py_VISIT(traverse_module_state->__pyx_ptype_4cupy_5_core_7_carray_mdspan);
  Py_VISIT(traverse_module_state->__pyx_ptype_4cupy_5_core_7_carray_CArray);
  Py_VISIT(traverse_module_state->__pyx_ptype_4cupy_5_core_7_carray_CIndexer);
  Py_VISIT(traverse_module_state->__pyx_ptype_4cupy_5_core_7_carray_Indexer);
  Py_VISIT(traverse_module_state->__pyx_ptype_4cupy_5_core_4core__ndarray_base);
  for (int i=0; i<3; ++i) { __Pyx_VISIT_CONST(traverse_module_state->__pyx_slice[i]); }
  for (int i=0; i<9; ++i) { __Pyx_VISIT_CONST(traverse_module_state->__pyx_tuple[i]); }
  for (int i=0; i<233; ++i) { __Pyx_VISIT_CONST(traverse_module_state->__pyx_string_tab[i]); }
  for (int i=0; i<12; ++i) { __Pyx_VISIT_CONST(traverse_module_state->__pyx_number_tab[i]); }
/* #### Code section: module_state_traverse_contents ### */
/* #### Code section: module_state_traverse_end ### */
return 0;
}
#endif
/* #### Code section: module_code ### */

/* "vector.from_py":51
 *     cdef Py_ssize_t __Pyx_PyObject_LengthHint(object o, Py_ssize_t defaultval) except -1
 * 
 * @cname("__pyx_convert_vector_from_py_Py_ssize_t")             # <<<<<<<<<<<<<<
 * cdef vector[X] __pyx_convert_vector_from_py_Py_ssize_t(object o) except *:
 * 
*/

static std::vector<Py_ssize_t>  __pyx_convert_vector_from_py_Py_ssize_t(PyObject *__pyx_v_o) {
  std::vector<Py_ssize_t>  __pyx_v_v;
  Py_ssize_t __pyx_v_s;
  PyObject *__pyx_v_item = NULL;
  std::vector<Py_ssize_t>  __pyx_r;
  __Pyx_RefNannyDeclarations
  Py_ssize_t __pyx_t_1;
  int __pyx_t_2;
  PyObject *__pyx_t_3 = NULL;
  PyObject *(*__pyx_t_4)(PyObject *);
  PyObject *__pyx_t_5 = NULL;
  Py_ssize_t __pyx_t_6;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__pyx_convert_vector_from_py_Py_ssize_t", 0);

  /* "vector.from_py":55
 * 
 *     cdef vector[X] v
 *     cdef Py_ssize_t s = __Pyx_PyObject_LengthHint(o, 0)             # <<<<<<<<<<<<<<
 * 
 *     if s > 0:
*/
  __pyx_t_1 = __Pyx_PyObject_LengthHint(__pyx_v_o, 0); if (unlikely(__pyx_t_1 == ((Py_ssize_t)-1L))) __PYX_ERR(1, 55, __pyx_L1_error)
  __pyx_v_s = __pyx_t_1;

  /* "vector.from_py":57
 *     cdef Py_ssize_t s = __Pyx_PyObject_LengthHint(o, 0)
 * 
 *     if s > 0:             # <<<<<<<<<<<<<<
 *         v.reserve(<size_t> s)
 * 
*/
  __pyx_t_2 = (__pyx_v_s > 0);
  if (__pyx_t_2) {

    /* "vector.from_py":58
 * 
 *     if s > 0:
 *         v.reserve(<size_t> s)             # <<<<<<<<<<<<<<
 * 
 *     for item in o:
*/
    try {
      __pyx_v_v.reserve(((size_t)__pyx_v_s));
    } catch(...) {
      __Pyx_CppExn2PyErr();
      __PYX_ERR(1, 58, __pyx_L1_error)
    }

    /* "vector.from_py":57
 *     cdef Py_ssize_t s = __Pyx_PyObject_LengthHint(o, 0)
 * 
 *     if s > 0:             # <<<<<<<<<<<<<<
 *         v.reserve(<size_t> s)
 * 
*/
  }

  /* "vector.from_py":60
 *         v.reserve(<size_t> s)
 * 
 *     for item in o:             # <<<<<<<<<<<<<<
 *         v.push_back(<X>item)
 * 
*/
  if (likely(PyList_CheckExact(__pyx_v_o)) || PyTuple_CheckExact(__pyx_v_o)) {
    __pyx_t_3 = __pyx_v_o; __Pyx_INCREF(__pyx_t_3);
    __pyx_t_1 = 0;
    __pyx_t_4 = NULL;
  } else {
    __pyx_t_1 = -1; __pyx_t_3 = PyObject_GetIter(__pyx_v_o); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 60, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_t_4 = (CYTHON_COMPILING_IN_LIMITED_API) ? PyIter_Next : __Pyx_PyObject_GetIterNextFunc(__pyx_t_3); if (unlikely(!__pyx_t_4)) __PYX_ERR(1, 60, __pyx_L1_error)
  }
  for (;;) {
    if (likely(!__pyx_t_4)) {
      if (likely(PyList_CheckExact(__pyx_t_3))) {
        {
          Py_ssize_t __pyx_temp = __Pyx_PyList_GET_SIZE(__pyx_t_3);
          #if !CYTHON_ASSUME_SAFE_SIZE
          if (unlikely((__pyx_temp < 0))) __PYX_ERR(1, 60, __pyx_L1_error)
          #endif
          if (__pyx_t_1 >= __pyx_temp) break;
        }
        __pyx_t_5 = __Pyx_PyList_GetItemRefFast(__pyx_t_3, __pyx_t_1, __Pyx_ReferenceSharing_OwnStrongReference);
        ++__pyx_t_1;
      } else {
        {
          Py_ssize_t __pyx_temp = __Pyx_PyTuple_GET_SIZE(__pyx_t_3);
          #if !CYTHON_ASSUME_SAFE_SIZE
          if (unlikely((__pyx_temp < 0))) __PYX_ERR(1, 60, __pyx_L1_error)
          #endif
          if (__pyx_t_1 >= __pyx_temp) break;
        }
        #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
        __pyx_t_5 = __Pyx_NewRef(PyTuple_GET_ITEM(__pyx_t_3, __pyx_t_1));
        #else
        __pyx_t_5 = __Pyx_PySequence_ITEM(__pyx_t_3, __pyx_t_1);
        #endif
        ++__pyx_t_1;
      }
      if (unlikely(!__pyx_t_5)) __PYX_ERR(1, 60, __pyx_L1_error)
    } else {
      __pyx_t_5 = __pyx_t_4(__pyx_t_3);
      if (unlikely(!__pyx_t_5)) {
        PyObject* exc_type = PyErr_Occurred();
        if (exc_type) {
          if (unlikely(!__Pyx_PyErr_GivenExceptionMatches(exc_type, PyExc_StopIteration))) __PYX_ERR(1, 60, __pyx_L1_error)
          PyErr_Clear();
        }
        break;
      }
    }
    __Pyx_GOTREF(__pyx_t_5);
    __Pyx_XDECREF_SET(__pyx_v_item, __pyx_t_5);
    __pyx_t_5 = 0;

    /* "vector.from_py":61
 * 
 *     for item in o:
 *         v.push_back(<X>item)             # <<<<<<<<<<<<<<
 * 
 *     return v
*/
    __pyx_t_6 = __Pyx_PyIndex_AsSsize_t(__pyx_v_item); if (unlikely((__pyx_t_6 == (Py_ssize_t)-1) && PyErr_Occurred())) __PYX_ERR(1, 61, __pyx_L1_error)
    try {
      __pyx_v_v.push_back(((Py_ssize_t)__pyx_t_6));
    } catch(...) {
      __Pyx_CppExn2PyErr();
      __PYX_ERR(1, 61, __pyx_L1_error)
    }

    /* "vector.from_py":60
 *         v.reserve(<size_t> s)
 * 
 *     for item in o:             # <<<<<<<<<<<<<<
 *         v.push_back(<X>item)
 * 
*/
  }
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "vector.from_py":63
 *         v.push_back(<X>item)
 * 
 *     return v             # <<<<<<<<<<<<<<
 * 
*/
  __pyx_r = __pyx_v_v;
  goto __pyx_L0;

  /* "vector.from_py":51
 *     cdef Py_ssize_t __Pyx_PyObject_LengthHint(object o, Py_ssize_t defaultval) except -1
 * 
 * @cname("__pyx_convert_vector_from_py_Py_ssize_t")             # <<<<<<<<<<<<<<
 * cdef vector[X] __pyx_convert_vector_from_py_Py_ssize_t(object o) except *:
 * 
*/

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_AddTraceback("vector.from_py.__pyx_convert_vector_from_py_Py_ssize_t", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_pretend_to_initialize(&__pyx_r);
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_item);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "memory.pxd":77
 *     cdef _ensure_pools_and_return_device_pool(self)
 * 
 *     cdef inline device_pool(self):             # <<<<<<<<<<<<<<
 *         # This criticial section may not be needed in practice. But without it
 *         # `self._pools` could be in an inconsistent state (the `PyObject *`
*/

static CYTHON_INLINE PyObject *__pyx_f_4cupy_4cuda_6memory_10MemoryPool_device_pool(struct __pyx_obj_4cupy_4cuda_6memory_MemoryPool *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_t_2;
  int __pyx_t_3;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("device_pool", 0);

  /* "memory.pxd":82
 *         # or the tuple not fully written due to CPU/compiler optimizations).
 *         # Atomics would be another way to ensure complete safety here.
 *         with cython.critical_section(self):             # <<<<<<<<<<<<<<
 *             if self._pools is not None:
 *                 return self._pools[device.get_device_id()]
*/
  {
      __Pyx_PyCriticalSection __pyx_cs;
      __pyx_t_1 = ((PyObject *)__pyx_v_self);
      __Pyx_INCREF(__pyx_t_1);
      __Pyx_PyCriticalSection_Begin(&__pyx_cs, (PyObject*)__pyx_t_1);
      /*try:*/ {

        /* "memory.pxd":83
 *         # Atomics would be another way to ensure complete safety here.
 *         with cython.critical_section(self):
 *             if self._pools is not None:             # <<<<<<<<<<<<<<
 *                 return self._pools[device.get_device_id()]
 * 
*/
        __pyx_t_2 = (__pyx_v_self->_pools != ((PyObject*)Py_None));
        if (__pyx_t_2) {

          /* "memory.pxd":84
 *         with cython.critical_section(self):
 *             if self._pools is not None:
 *                 return self._pools[device.get_device_id()]             # <<<<<<<<<<<<<<
 * 
 *         return self._ensure_pools_and_return_device_pool()
*/
          __Pyx_XDECREF(__pyx_r);
          if (unlikely(__pyx_v_self->_pools == Py_None)) {
            PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
            __PYX_ERR(2, 84, __pyx_L4_error)
          }
          __pyx_t_3 = __pyx_f_4cupy_4cuda_6device_get_device_id(0); if (unlikely(__pyx_t_3 == ((int)-1) && PyErr_Occurred())) __PYX_ERR(2, 84, __pyx_L4_error)
          __pyx_t_4 = __Pyx_GetItemInt_Tuple(__pyx_v_self->_pools, __pyx_t_3, int, 1, __Pyx_PyLong_From_int, 0, 1, 1, 1, __Pyx_ReferenceSharing_SharedReference); if (unlikely(!__pyx_t_4)) __PYX_ERR(2, 84, __pyx_L4_error)
          __Pyx_GOTREF(__pyx_t_4);
          __pyx_r = __pyx_t_4;
          __pyx_t_4 = 0;
          __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
          goto __pyx_L3_return;

          /* "memory.pxd":83
 *         # Atomics would be another way to ensure complete safety here.
 *         with cython.critical_section(self):
 *             if self._pools is not None:             # <<<<<<<<<<<<<<
 *                 return self._pools[device.get_device_id()]
 * 
*/
        }
      }

      /* "memory.pxd":82
 *         # or the tuple not fully written due to CPU/compiler optimizations).
 *         # Atomics would be another way to ensure complete safety here.
 *         with cython.critical_section(self):             # <<<<<<<<<<<<<<
 *             if self._pools is not None:
 *                 return self._pools[device.get_device_id()]
*/
      /*finally:*/ {
        /*normal exit:*/{
          __Pyx_PyCriticalSection_End(&__pyx_cs);
          goto __pyx_L5;
        }
        __pyx_L3_return: {
          __pyx_t_5 = __pyx_r;
          __pyx_r = 0;
          __Pyx_PyCriticalSection_End(&__pyx_cs);
          __pyx_r = __pyx_t_5;
          __pyx_t_5 = 0;
          goto __pyx_L0;
        }
        __pyx_L4_error: {
          __Pyx_PyCriticalSection_End(&__pyx_cs);
          goto __pyx_L1_error;
        }
        __pyx_L5:;
      }
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  }

  /* "memory.pxd":86
 *                 return self._pools[device.get_device_id()]
 * 
 *         return self._ensure_pools_and_return_device_pool()             # <<<<<<<<<<<<<<
 * 
 * 
*/
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = ((struct __pyx_vtabstruct_4cupy_4cuda_6memory_MemoryPool *)__pyx_v_self->__pyx_vtab)->_ensure_pools_and_return_device_pool(__pyx_v_self); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 86, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "memory.pxd":77
 *     cdef _ensure_pools_and_return_device_pool(self)
 * 
 *     cdef inline device_pool(self):             # <<<<<<<<<<<<<<
 *         # This criticial section may not be needed in practice. But without it
 *         # `self._pools` could be in an inconsistent state (the `PyObject *`
*/

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_AddTraceback("cupy.cuda.memory.MemoryPool.device_pool", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "cupy/_core/core.pxd":125
 * 
 * 
 * cdef inline _ndarray_base _convert_from_cupy_like(             # <<<<<<<<<<<<<<
 *         obj, error):
 *     """Inline function to convert cupy array-likes to cupy ndarray.
*/

static CYTHON_INLINE struct __pyx_obj_4cupy_5_core_4core__ndarray_base *__pyx_f_4cupy_5_core_4core__convert_from_cupy_like(PyObject *__pyx_v_obj, PyObject *__pyx_v_error) {
  struct __pyx_obj_4cupy_5_core_4core__ndarray_base *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  size_t __pyx_t_4;
  PyObject *__pyx_t_5 = NULL;
  PyObject *__pyx_t_6 = NULL;
  PyObject *__pyx_t_7 = NULL;
  PyObject *__pyx_t_8[6];
  PyObject *__pyx_t_9 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("_convert_from_cupy_like", 0);

  /* "cupy/_core/core.pxd":137
 *     is not cupy-like and `None` is returned instead.
 *     """
 *     if isinstance(obj, _ndarray_base):             # <<<<<<<<<<<<<<
 *         return <_ndarray_base>obj
 *     elif hasattr(obj, "__cuda_array_interface__"):
*/
  __pyx_t_1 = __Pyx_TypeCheck(__pyx_v_obj, __pyx_mstate_global->__pyx_ptype_4cupy_5_core_4core__ndarray_base); 
  if (__pyx_t_1) {

    /* "cupy/_core/core.pxd":138
 *     """
 *     if isinstance(obj, _ndarray_base):
 *         return <_ndarray_base>obj             # <<<<<<<<<<<<<<
 *     elif hasattr(obj, "__cuda_array_interface__"):
 *         return _convert_object_with_cuda_array_interface(obj)
*/
    __Pyx_XDECREF((PyObject *)__pyx_r);
    __Pyx_INCREF((PyObject *)((struct __pyx_obj_4cupy_5_core_4core__ndarray_base *)__pyx_v_obj));
    __pyx_r = ((struct __pyx_obj_4cupy_5_core_4core__ndarray_base *)__pyx_v_obj);
    goto __pyx_L0;

    /* "cupy/_core/core.pxd":137
 *     is not cupy-like and `None` is returned instead.
 *     """
 *     if isinstance(obj, _ndarray_base):             # <<<<<<<<<<<<<<
 *         return <_ndarray_base>obj
 *     elif hasattr(obj, "__cuda_array_interface__"):
*/
  }

  /* "cupy/_core/core.pxd":139
 *     if isinstance(obj, _ndarray_base):
 *         return <_ndarray_base>obj
 *     elif hasattr(obj, "__cuda_array_interface__"):             # <<<<<<<<<<<<<<
 *         return _convert_object_with_cuda_array_interface(obj)
 *     elif hasattr(obj, "__cupy_get_ndarray__"):
*/
  __pyx_t_1 = __Pyx_HasAttr(__pyx_v_obj, __pyx_mstate_global->__pyx_n_u_cuda_array_interface); if (unlikely(__pyx_t_1 == ((int)-1))) __PYX_ERR(3, 139, __pyx_L1_error)
  if (__pyx_t_1) {

    /* "cupy/_core/core.pxd":140
 *         return <_ndarray_base>obj
 *     elif hasattr(obj, "__cuda_array_interface__"):
 *         return _convert_object_with_cuda_array_interface(obj)             # <<<<<<<<<<<<<<
 *     elif hasattr(obj, "__cupy_get_ndarray__"):
 *         return obj.__cupy_get_ndarray__()
*/
    __Pyx_XDECREF((PyObject *)__pyx_r);
    __pyx_t_2 = ((PyObject *)__pyx_f_4cupy_5_core_4core__convert_object_with_cuda_array_interface(__pyx_v_obj, 0)); if (unlikely(!__pyx_t_2)) __PYX_ERR(3, 140, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_r = ((struct __pyx_obj_4cupy_5_core_4core__ndarray_base *)__pyx_t_2);
    __pyx_t_2 = 0;
    goto __pyx_L0;

    /* "cupy/_core/core.pxd":139
 *     if isinstance(obj, _ndarray_base):
 *         return <_ndarray_base>obj
 *     elif hasattr(obj, "__cuda_array_interface__"):             # <<<<<<<<<<<<<<
 *         return _convert_object_with_cuda_array_interface(obj)
 *     elif hasattr(obj, "__cupy_get_ndarray__"):
*/
  }

  /* "cupy/_core/core.pxd":141
 *     elif hasattr(obj, "__cuda_array_interface__"):
 *         return _convert_object_with_cuda_array_interface(obj)
 *     elif hasattr(obj, "__cupy_get_ndarray__"):             # <<<<<<<<<<<<<<
 *         return obj.__cupy_get_ndarray__()
 * 
*/
  __pyx_t_1 = __Pyx_HasAttr(__pyx_v_obj, __pyx_mstate_global->__pyx_n_u_cupy_get_ndarray); if (unlikely(__pyx_t_1 == ((int)-1))) __PYX_ERR(3, 141, __pyx_L1_error)
  if (__pyx_t_1) {

    /* "cupy/_core/core.pxd":142
 *         return _convert_object_with_cuda_array_interface(obj)
 *     elif hasattr(obj, "__cupy_get_ndarray__"):
 *         return obj.__cupy_get_ndarray__()             # <<<<<<<<<<<<<<
 * 
 *     if error is not False:
*/
    __Pyx_XDECREF((PyObject *)__pyx_r);
    __pyx_t_3 = __pyx_v_obj;
    __Pyx_INCREF(__pyx_t_3);
    __pyx_t_4 = 0;
    {
      PyObject *__pyx_callargs[2] = {__pyx_t_3, NULL};
      __pyx_t_2 = __Pyx_PyObject_FastCallMethod((PyObject*)__pyx_mstate_global->__pyx_n_u_cupy_get_ndarray, __pyx_callargs+__pyx_t_4, (1-__pyx_t_4) | (1*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
      __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
      if (unlikely(!__pyx_t_2)) __PYX_ERR(3, 142, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_2);
    }
    if (!(likely(((__pyx_t_2) == Py_None) || likely(__Pyx_TypeTest(__pyx_t_2, __pyx_mstate_global->__pyx_ptype_4cupy_5_core_4core__ndarray_base))))) __PYX_ERR(3, 142, __pyx_L1_error)
    __pyx_r = ((struct __pyx_obj_4cupy_5_core_4core__ndarray_base *)__pyx_t_2);
    __pyx_t_2 = 0;
    goto __pyx_L0;

    /* "cupy/_core/core.pxd":141
 *     elif hasattr(obj, "__cuda_array_interface__"):
 *         return _convert_object_with_cuda_array_interface(obj)
 *     elif hasattr(obj, "__cupy_get_ndarray__"):             # <<<<<<<<<<<<<<
 *         return obj.__cupy_get_ndarray__()
 * 
*/
  }

  /* "cupy/_core/core.pxd":144
 *         return obj.__cupy_get_ndarray__()
 * 
 *     if error is not False:             # <<<<<<<<<<<<<<
 *         raise TypeError(
 *             f'{error} has incorrect type '
*/
  __pyx_t_1 = (__pyx_v_error != Py_False);
  if (unlikely(__pyx_t_1)) {

    /* "cupy/_core/core.pxd":145
 * 
 *     if error is not False:
 *         raise TypeError(             # <<<<<<<<<<<<<<
 *             f'{error} has incorrect type '
 *             f'(expected {_ndarray_base}, got {type(obj)}).'
*/
    __pyx_t_3 = NULL;

    /* "cupy/_core/core.pxd":146
 *     if error is not False:
 *         raise TypeError(
 *             f'{error} has incorrect type '             # <<<<<<<<<<<<<<
 *             f'(expected {_ndarray_base}, got {type(obj)}).'
 *         )
*/
    __pyx_t_5 = __Pyx_PyObject_FormatSimple(__pyx_v_error, __pyx_mstate_global->__pyx_empty_unicode); if (unlikely(!__pyx_t_5)) __PYX_ERR(3, 146, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);

    /* "cupy/_core/core.pxd":147
 *         raise TypeError(
 *             f'{error} has incorrect type '
 *             f'(expected {_ndarray_base}, got {type(obj)}).'             # <<<<<<<<<<<<<<
 *         )
 * 
*/
    __pyx_t_6 = __Pyx_PyObject_FormatSimple(((PyObject *)__pyx_mstate_global->__pyx_ptype_4cupy_5_core_4core__ndarray_base), __pyx_mstate_global->__pyx_empty_unicode); if (unlikely(!__pyx_t_6)) __PYX_ERR(3, 147, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_6);
    __pyx_t_7 = __Pyx_PyObject_FormatSimple(((PyObject *)Py_TYPE(__pyx_v_obj)), __pyx_mstate_global->__pyx_empty_unicode); if (unlikely(!__pyx_t_7)) __PYX_ERR(3, 147, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_7);
    __pyx_t_8[0] = __pyx_t_5;
    __pyx_t_8[1] = __pyx_mstate_global->__pyx_kp_u_has_incorrect_type_expected;
    __pyx_t_8[2] = __pyx_t_6;
    __pyx_t_8[3] = __pyx_mstate_global->__pyx_kp_u_got;
    __pyx_t_8[4] = __pyx_t_7;
    __pyx_t_8[5] = __pyx_mstate_global->__pyx_kp_u_;

    /* "cupy/_core/core.pxd":146
 *     if error is not False:
 *         raise TypeError(
 *             f'{error} has incorrect type '             # <<<<<<<<<<<<<<
 *             f'(expected {_ndarray_base}, got {type(obj)}).'
 *         )
*/
    __pyx_t_9 = __Pyx_PyUnicode_Join(__pyx_t_8, 6, __Pyx_PyUnicode_GET_LENGTH(__pyx_t_5) + 30 + __Pyx_PyUnicode_GET_LENGTH(__pyx_t_6) + 6 + __Pyx_PyUnicode_GET_LENGTH(__pyx_t_7) + 2, 127 | __Pyx_PyUnicode_MAX_CHAR_VALUE(__pyx_t_5) | __Pyx_PyUnicode_MAX_CHAR_VALUE(__pyx_t_6) | __Pyx_PyUnicode_MAX_CHAR_VALUE(__pyx_t_7));
    if (unlikely(!__pyx_t_9)) __PYX_ERR(3, 146, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_9);
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
    __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
    __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
    __pyx_t_4 = 1;
    {
      PyObject *__pyx_callargs[2] = {__pyx_t_3, __pyx_t_9};
      __pyx_t_2 = __Pyx_PyObject_FastCall((PyObject*)(((PyTypeObject*)PyExc_TypeError)), __pyx_callargs+__pyx_t_4, (2-__pyx_t_4) | (__pyx_t_4*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
      __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
      __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
      if (unlikely(!__pyx_t_2)) __PYX_ERR(3, 145, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_2);
    }
    __Pyx_Raise(__pyx_t_2, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __PYX_ERR(3, 145, __pyx_L1_error)

    /* "cupy/_core/core.pxd":144
 *         return obj.__cupy_get_ndarray__()
 * 
 *     if error is not False:             # <<<<<<<<<<<<<<
 *         raise TypeError(
 *             f'{error} has incorrect type '
*/
  }

  /* "cupy/_core/core.pxd":150
 *         )
 * 
 *     return None             # <<<<<<<<<<<<<<
*/
  __Pyx_XDECREF((PyObject *)__pyx_r);
  __pyx_r = ((struct __pyx_obj_4cupy_5_core_4core__ndarray_base *)Py_None); __Pyx_INCREF(Py_None);
  goto __pyx_L0;

  /* "cupy/_core/core.pxd":125
 * 
 * 
 * cdef inline _ndarray_base _convert_from_cupy_like(             # <<<<<<<<<<<<<<
 *         obj, error):
 *     """Inline function to convert cupy array-likes to cupy ndarray.
*/

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_XDECREF(__pyx_t_7);
  __Pyx_XDECREF(__pyx_t_9);
  __Pyx_AddTraceback("cupy._core.core._convert_from_cupy_like", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF((PyObject *)__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "cupyx/cusolver.pyx":103
 * 
 * 
 * @_util.memoize(for_each_device=True)             # <<<<<<<<<<<<<<
 * def check_availability(name):
 *     if not _runtime.is_hip:
*/

/* Python wrapper */
static PyObject *__pyx_pw_5cupyx_8cusolver_1check_availability(PyObject *__pyx_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
PyDoc_STRVAR(__pyx_doc_5cupyx_8cusolver_check_availability, "check_availability(name)");
static PyMethodDef __pyx_mdef_5cupyx_8cusolver_1check_availability = {"check_availability", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_5cupyx_8cusolver_1check_availability, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_5cupyx_8cusolver_check_availability};
static PyObject *__pyx_pw_5cupyx_8cusolver_1check_availability(PyObject *__pyx_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
) {
  PyObject *__pyx_v_name = 0;
  #if !CYTHON_METH_FASTCALL
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  #endif
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject* values[1] = {0};
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("check_availability (wrapper)", 0);
  #if !CYTHON_METH_FASTCALL
  #if CYTHON_ASSUME_SAFE_SIZE
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return NULL;
  #endif
  #endif
  __pyx_kwvalues = __Pyx_KwValues_FASTCALL(__pyx_args, __pyx_nargs);
  {
    PyObject ** const __pyx_pyargnames[] = {&__pyx_mstate_global->__pyx_n_u_name,0};
    const Py_ssize_t __pyx_kwds_len = (__pyx_kwds) ? __Pyx_NumKwargs_FASTCALL(__pyx_kwds) : 0;
    if (unlikely(__pyx_kwds_len < 0)) __PYX_ERR(0, 103, __pyx_L3_error)
    if (__pyx_kwds_len > 0) {
      switch (__pyx_nargs) {
        case  1:
        values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(0, 103, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      const Py_ssize_t kwd_pos_args = __pyx_nargs;
      if (__Pyx_ParseKeywords(__pyx_kwds, __pyx_kwvalues, __pyx_pyargnames, 0, values, kwd_pos_args, __pyx_kwds_len, "check_availability", 0) < (0)) __PYX_ERR(0, 103, __pyx_L3_error)
      for (Py_ssize_t i = __pyx_nargs; i < 1; i++) {
        if (unlikely(!values[i])) { __Pyx_RaiseArgtupleInvalid("check_availability", 1, 1, 1, i); __PYX_ERR(0, 103, __pyx_L3_error) }
      }
    } else if (unlikely(__pyx_nargs != 1)) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
      if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(0, 103, __pyx_L3_error)
    }
    __pyx_v_name = values[0];
  }
  goto __pyx_L6_skip;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("check_availability", 1, 1, 1, __pyx_nargs); __PYX_ERR(0, 103, __pyx_L3_error)
  __pyx_L6_skip:;
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_AddTraceback("cupyx.cusolver.check_availability", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_5cupyx_8cusolver_check_availability(__pyx_self, __pyx_v_name);

  /* function exit code */
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5cupyx_8cusolver_check_availability(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_name) {
  PyObject *__pyx_v_available_version = NULL;
  long __pyx_v_version;
  PyObject *__pyx_v_msg = NULL;
  PyObject *__pyx_v_version_added = NULL;
  PyObject *__pyx_v_version_removed = NULL;
  PyObject *__pyx_v_compute_capability = NULL;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  int __pyx_t_3;
  int __pyx_t_4;
  int __pyx_t_5;
  size_t __pyx_t_6;
  PyObject *__pyx_t_7 = NULL;
  PyObject *__pyx_t_8 = NULL;
  PyObject *(*__pyx_t_9)(PyObject *);
  int __pyx_t_10;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("check_availability", 0);

  /* "cupyx/cusolver.pyx":105
 * @_util.memoize(for_each_device=True)
 * def check_availability(name):
 *     if not _runtime.is_hip:             # <<<<<<<<<<<<<<
 *         available_version = _available_cuda_version
 *         version = cusolver._getVersionNumber()
*/
  __Pyx_GetModuleGlobalName(__pyx_t_1, __pyx_mstate_global->__pyx_n_u_runtime); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 105, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_t_1, __pyx_mstate_global->__pyx_n_u_is_hip); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 105, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_3 = __Pyx_PyObject_IsTrue(__pyx_t_2); if (unlikely((__pyx_t_3 < 0))) __PYX_ERR(0, 105, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_4 = (!__pyx_t_3);
  if (__pyx_t_4) {

    /* "cupyx/cusolver.pyx":106
 * def check_availability(name):
 *     if not _runtime.is_hip:
 *         available_version = _available_cuda_version             # <<<<<<<<<<<<<<
 *         version = cusolver._getVersionNumber()
 *     else:
*/
    __Pyx_GetModuleGlobalName(__pyx_t_2, __pyx_mstate_global->__pyx_n_u_available_cuda_version); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 106, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_v_available_version = __pyx_t_2;
    __pyx_t_2 = 0;

    /* "cupyx/cusolver.pyx":107
 *     if not _runtime.is_hip:
 *         available_version = _available_cuda_version
 *         version = cusolver._getVersionNumber()             # <<<<<<<<<<<<<<
 *     else:
 *         available_version = _available_hip_version
*/
    __pyx_t_5 = __pyx_f_13cupy_backends_4cuda_4libs_8cusolver__getVersionNumber(0); if (unlikely(__pyx_t_5 == ((int)-1) && PyErr_Occurred())) __PYX_ERR(0, 107, __pyx_L1_error)
    __pyx_v_version = __pyx_t_5;

    /* "cupyx/cusolver.pyx":105
 * @_util.memoize(for_each_device=True)
 * def check_availability(name):
 *     if not _runtime.is_hip:             # <<<<<<<<<<<<<<
 *         available_version = _available_cuda_version
 *         version = cusolver._getVersionNumber()
*/
    goto __pyx_L3;
  }

  /* "cupyx/cusolver.pyx":109
 *         version = cusolver._getVersionNumber()
 *     else:
 *         available_version = _available_hip_version             # <<<<<<<<<<<<<<
 *         version = CUPY_HIP_VERSION
 *     if name not in available_version:
*/
  /*else*/ {
    __Pyx_GetModuleGlobalName(__pyx_t_2, __pyx_mstate_global->__pyx_n_u_available_hip_version); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 109, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_v_available_version = __pyx_t_2;
    __pyx_t_2 = 0;

    /* "cupyx/cusolver.pyx":110
 *     else:
 *         available_version = _available_hip_version
 *         version = CUPY_HIP_VERSION             # <<<<<<<<<<<<<<
 *     if name not in available_version:
 *         msg = 'No available version information specified for {}'.format(name)
*/
    __pyx_v_version = 0x42FFA9B;
  }
  __pyx_L3:;

  /* "cupyx/cusolver.pyx":111
 *         available_version = _available_hip_version
 *         version = CUPY_HIP_VERSION
 *     if name not in available_version:             # <<<<<<<<<<<<<<
 *         msg = 'No available version information specified for {}'.format(name)
 *         raise ValueError(msg)
*/
  __pyx_t_4 = (__Pyx_PySequence_ContainsTF(__pyx_v_name, __pyx_v_available_version, Py_NE)); if (unlikely((__pyx_t_4 < 0))) __PYX_ERR(0, 111, __pyx_L1_error)
  if (unlikely(__pyx_t_4)) {

    /* "cupyx/cusolver.pyx":112
 *         version = CUPY_HIP_VERSION
 *     if name not in available_version:
 *         msg = 'No available version information specified for {}'.format(name)             # <<<<<<<<<<<<<<
 *         raise ValueError(msg)
 *     version_added, version_removed = available_version[name]
*/
    __pyx_t_1 = __pyx_mstate_global->__pyx_kp_u_No_available_version_information;
    __Pyx_INCREF(__pyx_t_1);
    __pyx_t_6 = 0;
    {
      PyObject *__pyx_callargs[2] = {__pyx_t_1, __pyx_v_name};
      __pyx_t_2 = __Pyx_PyObject_FastCallMethod((PyObject*)__pyx_mstate_global->__pyx_n_u_format, __pyx_callargs+__pyx_t_6, (2-__pyx_t_6) | (1*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
      __Pyx_XDECREF(__pyx_t_1); __pyx_t_1 = 0;
      if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 112, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_2);
    }
    __pyx_v_msg = ((PyObject*)__pyx_t_2);
    __pyx_t_2 = 0;

    /* "cupyx/cusolver.pyx":113
 *     if name not in available_version:
 *         msg = 'No available version information specified for {}'.format(name)
 *         raise ValueError(msg)             # <<<<<<<<<<<<<<
 *     version_added, version_removed = available_version[name]
 *     if version_added is not None and version < version_added:
*/
    __pyx_t_1 = NULL;
    __pyx_t_6 = 1;
    {
      PyObject *__pyx_callargs[2] = {__pyx_t_1, __pyx_v_msg};
      __pyx_t_2 = __Pyx_PyObject_FastCall((PyObject*)(((PyTypeObject*)PyExc_ValueError)), __pyx_callargs+__pyx_t_6, (2-__pyx_t_6) | (__pyx_t_6*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
      __Pyx_XDECREF(__pyx_t_1); __pyx_t_1 = 0;
      if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 113, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_2);
    }
    __Pyx_Raise(__pyx_t_2, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __PYX_ERR(0, 113, __pyx_L1_error)

    /* "cupyx/cusolver.pyx":111
 *         available_version = _available_hip_version
 *         version = CUPY_HIP_VERSION
 *     if name not in available_version:             # <<<<<<<<<<<<<<
 *         msg = 'No available version information specified for {}'.format(name)
 *         raise ValueError(msg)
*/
  }

  /* "cupyx/cusolver.pyx":114
 *         msg = 'No available version information specified for {}'.format(name)
 *         raise ValueError(msg)
 *     version_added, version_removed = available_version[name]             # <<<<<<<<<<<<<<
 *     if version_added is not None and version < version_added:
 *         return False
*/
  __pyx_t_2 = __Pyx_PyObject_GetItem(__pyx_v_available_version, __pyx_v_name); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 114, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if ((likely(PyTuple_CheckExact(__pyx_t_2))) || (PyList_CheckExact(__pyx_t_2))) {
    PyObject* sequence = __pyx_t_2;
    Py_ssize_t size = __Pyx_PySequence_SIZE(sequence);
    if (unlikely(size != 2)) {
      if (size > 2) __Pyx_RaiseTooManyValuesError(2);
      else if (size >= 0) __Pyx_RaiseNeedMoreValuesError(size);
      __PYX_ERR(0, 114, __pyx_L1_error)
    }
    #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
    if (likely(PyTuple_CheckExact(sequence))) {
      __pyx_t_1 = PyTuple_GET_ITEM(sequence, 0);
      __Pyx_INCREF(__pyx_t_1);
      __pyx_t_7 = PyTuple_GET_ITEM(sequence, 1);
      __Pyx_INCREF(__pyx_t_7);
    } else {
      __pyx_t_1 = __Pyx_PyList_GetItemRefFast(sequence, 0, __Pyx_ReferenceSharing_SharedReference);
      if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 114, __pyx_L1_error)
      __Pyx_XGOTREF(__pyx_t_1);
      __pyx_t_7 = __Pyx_PyList_GetItemRefFast(sequence, 1, __Pyx_ReferenceSharing_SharedReference);
      if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 114, __pyx_L1_error)
      __Pyx_XGOTREF(__pyx_t_7);
    }
    #else
    __pyx_t_1 = __Pyx_PySequence_ITEM(sequence, 0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 114, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_t_7 = __Pyx_PySequence_ITEM(sequence, 1); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 114, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_7);
    #endif
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  } else {
    Py_ssize_t index = -1;
    __pyx_t_8 = PyObject_GetIter(__pyx_t_2); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 114, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_8);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __pyx_t_9 = (CYTHON_COMPILING_IN_LIMITED_API) ? PyIter_Next : __Pyx_PyObject_GetIterNextFunc(__pyx_t_8);
    index = 0; __pyx_t_1 = __pyx_t_9(__pyx_t_8); if (unlikely(!__pyx_t_1)) goto __pyx_L5_unpacking_failed;
    __Pyx_GOTREF(__pyx_t_1);
    index = 1; __pyx_t_7 = __pyx_t_9(__pyx_t_8); if (unlikely(!__pyx_t_7)) goto __pyx_L5_unpacking_failed;
    __Pyx_GOTREF(__pyx_t_7);
    if (__Pyx_IternextUnpackEndCheck(__pyx_t_9(__pyx_t_8), 2) < (0)) __PYX_ERR(0, 114, __pyx_L1_error)
    __pyx_t_9 = NULL;
    __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
    goto __pyx_L6_unpacking_done;
    __pyx_L5_unpacking_failed:;
    __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
    __pyx_t_9 = NULL;
    if (__Pyx_IterFinish() == 0) __Pyx_RaiseNeedMoreValuesError(index);
    __PYX_ERR(0, 114, __pyx_L1_error)
    __pyx_L6_unpacking_done:;
  }
  __pyx_v_version_added = __pyx_t_1;
  __pyx_t_1 = 0;
  __pyx_v_version_removed = __pyx_t_7;
  __pyx_t_7 = 0;

  /* "cupyx/cusolver.pyx":115
 *         raise ValueError(msg)
 *     version_added, version_removed = available_version[name]
 *     if version_added is not None and version < version_added:             # <<<<<<<<<<<<<<
 *         return False
 *     if version_removed is not None and version >= version_removed:
*/
  __pyx_t_3 = (__pyx_v_version_added != Py_None);
  if (__pyx_t_3) {
  } else {
    __pyx_t_4 = __pyx_t_3;
    goto __pyx_L8_bool_binop_done;
  }
  __pyx_t_2 = __Pyx_PyLong_From_long(__pyx_v_version); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 115, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_7 = PyObject_RichCompare(__pyx_t_2, __pyx_v_version_added, Py_LT); __Pyx_XGOTREF(__pyx_t_7); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 115, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_3 = __Pyx_PyObject_IsTrue(__pyx_t_7); if (unlikely((__pyx_t_3 < 0))) __PYX_ERR(0, 115, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
  __pyx_t_4 = __pyx_t_3;
  __pyx_L8_bool_binop_done:;
  if (__pyx_t_4) {

    /* "cupyx/cusolver.pyx":116
 *     version_added, version_removed = available_version[name]
 *     if version_added is not None and version < version_added:
 *         return False             # <<<<<<<<<<<<<<
 *     if version_removed is not None and version >= version_removed:
 *         return False
*/
    __Pyx_XDECREF(__pyx_r);
    __Pyx_INCREF(Py_False);
    __pyx_r = Py_False;
    goto __pyx_L0;

    /* "cupyx/cusolver.pyx":115
 *         raise ValueError(msg)
 *     version_added, version_removed = available_version[name]
 *     if version_added is not None and version < version_added:             # <<<<<<<<<<<<<<
 *         return False
 *     if version_removed is not None and version >= version_removed:
*/
  }

  /* "cupyx/cusolver.pyx":117
 *     if version_added is not None and version < version_added:
 *         return False
 *     if version_removed is not None and version >= version_removed:             # <<<<<<<<<<<<<<
 *         return False
 *     # CUDA specific stuff
*/
  __pyx_t_3 = (__pyx_v_version_removed != Py_None);
  if (__pyx_t_3) {
  } else {
    __pyx_t_4 = __pyx_t_3;
    goto __pyx_L11_bool_binop_done;
  }
  __pyx_t_7 = __Pyx_PyLong_From_long(__pyx_v_version); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 117, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_7);
  __pyx_t_2 = PyObject_RichCompare(__pyx_t_7, __pyx_v_version_removed, Py_GE); __Pyx_XGOTREF(__pyx_t_2); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 117, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
  __pyx_t_3 = __Pyx_PyObject_IsTrue(__pyx_t_2); if (unlikely((__pyx_t_3 < 0))) __PYX_ERR(0, 117, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_4 = __pyx_t_3;
  __pyx_L11_bool_binop_done:;
  if (__pyx_t_4) {

    /* "cupyx/cusolver.pyx":118
 *         return False
 *     if version_removed is not None and version >= version_removed:
 *         return False             # <<<<<<<<<<<<<<
 *     # CUDA specific stuff
 *     if not _runtime.is_hip and name in _available_compute_capability:
*/
    __Pyx_XDECREF(__pyx_r);
    __Pyx_INCREF(Py_False);
    __pyx_r = Py_False;
    goto __pyx_L0;

    /* "cupyx/cusolver.pyx":117
 *     if version_added is not None and version < version_added:
 *         return False
 *     if version_removed is not None and version >= version_removed:             # <<<<<<<<<<<<<<
 *         return False
 *     # CUDA specific stuff
*/
  }

  /* "cupyx/cusolver.pyx":120
 *         return False
 *     # CUDA specific stuff
 *     if not _runtime.is_hip and name in _available_compute_capability:             # <<<<<<<<<<<<<<
 *         compute_capability = int(_device.get_compute_capability())
 *         if compute_capability < _available_compute_capability[name]:
*/
  __Pyx_GetModuleGlobalName(__pyx_t_2, __pyx_mstate_global->__pyx_n_u_runtime); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 120, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_7 = __Pyx_PyObject_GetAttrStr(__pyx_t_2, __pyx_mstate_global->__pyx_n_u_is_hip); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 120, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_7);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_3 = __Pyx_PyObject_IsTrue(__pyx_t_7); if (unlikely((__pyx_t_3 < 0))) __PYX_ERR(0, 120, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
  __pyx_t_10 = (!__pyx_t_3);
  if (__pyx_t_10) {
  } else {
    __pyx_t_4 = __pyx_t_10;
    goto __pyx_L14_bool_binop_done;
  }
  __Pyx_GetModuleGlobalName(__pyx_t_7, __pyx_mstate_global->__pyx_n_u_available_compute_capability); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 120, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_7);
  __pyx_t_10 = (__Pyx_PySequence_ContainsTF(__pyx_v_name, __pyx_t_7, Py_EQ)); if (unlikely((__pyx_t_10 < 0))) __PYX_ERR(0, 120, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
  __pyx_t_4 = __pyx_t_10;
  __pyx_L14_bool_binop_done:;
  if (__pyx_t_4) {

    /* "cupyx/cusolver.pyx":121
 *     # CUDA specific stuff
 *     if not _runtime.is_hip and name in _available_compute_capability:
 *         compute_capability = int(_device.get_compute_capability())             # <<<<<<<<<<<<<<
 *         if compute_capability < _available_compute_capability[name]:
 *             return False
*/
    __pyx_t_2 = NULL;
    __Pyx_GetModuleGlobalName(__pyx_t_1, __pyx_mstate_global->__pyx_n_u_device); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 121, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_t_8 = __Pyx_PyObject_GetAttrStr(__pyx_t_1, __pyx_mstate_global->__pyx_n_u_get_compute_capability); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 121, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_8);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __pyx_t_6 = 1;
    #if CYTHON_UNPACK_METHODS
    if (unlikely(PyMethod_Check(__pyx_t_8))) {
      __pyx_t_2 = PyMethod_GET_SELF(__pyx_t_8);
      assert(__pyx_t_2);
      PyObject* __pyx__function = PyMethod_GET_FUNCTION(__pyx_t_8);
      __Pyx_INCREF(__pyx_t_2);
      __Pyx_INCREF(__pyx__function);
      __Pyx_DECREF_SET(__pyx_t_8, __pyx__function);
      __pyx_t_6 = 0;
    }
    #endif
    {
      PyObject *__pyx_callargs[2] = {__pyx_t_2, NULL};
      __pyx_t_7 = __Pyx_PyObject_FastCall((PyObject*)__pyx_t_8, __pyx_callargs+__pyx_t_6, (1-__pyx_t_6) | (__pyx_t_6*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
      __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
      __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
      if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 121, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_7);
    }
    __pyx_t_8 = __Pyx_PyNumber_Int(__pyx_t_7); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 121, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_8);
    __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
    __pyx_v_compute_capability = ((PyObject*)__pyx_t_8);
    __pyx_t_8 = 0;

    /* "cupyx/cusolver.pyx":122
 *     if not _runtime.is_hip and name in _available_compute_capability:
 *         compute_capability = int(_device.get_compute_capability())
 *         if compute_capability < _available_compute_capability[name]:             # <<<<<<<<<<<<<<
 *             return False
 *     return True
*/
    __Pyx_GetModuleGlobalName(__pyx_t_8, __pyx_mstate_global->__pyx_n_u_available_compute_capability); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 122, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_8);
    __pyx_t_7 = __Pyx_PyObject_GetItem(__pyx_t_8, __pyx_v_name); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 122, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_7);
    __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
    __pyx_t_8 = PyObject_RichCompare(__pyx_v_compute_capability, __pyx_t_7, Py_LT); __Pyx_XGOTREF(__pyx_t_8); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 122, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
    __pyx_t_4 = __Pyx_PyObject_IsTrue(__pyx_t_8); if (unlikely((__pyx_t_4 < 0))) __PYX_ERR(0, 122, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
    if (__pyx_t_4) {

      /* "cupyx/cusolver.pyx":123
 *         compute_capability = int(_device.get_compute_capability())
 *         if compute_capability < _available_compute_capability[name]:
 *             return False             # <<<<<<<<<<<<<<
 *     return True
 * 
*/
      __Pyx_XDECREF(__pyx_r);
      __Pyx_INCREF(Py_False);
      __pyx_r = Py_False;
      goto __pyx_L0;

      /* "cupyx/cusolver.pyx":122
 *     if not _runtime.is_hip and name in _available_compute_capability:
 *         compute_capability = int(_device.get_compute_capability())
 *         if compute_capability < _available_compute_capability[name]:             # <<<<<<<<<<<<<<
 *             return False
 *     return True
*/
    }

    /* "cupyx/cusolver.pyx":120
 *         return False
 *     # CUDA specific stuff
 *     if not _runtime.is_hip and name in _available_compute_capability:             # <<<<<<<<<<<<<<
 *         compute_capability = int(_device.get_compute_capability())
 *         if compute_capability < _available_compute_capability[name]:
*/
  }

  /* "cupyx/cusolver.pyx":124
 *         if compute_capability < _available_compute_capability[name]:
 *             return False
 *     return True             # <<<<<<<<<<<<<<
 * 
 * 
*/
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(Py_True);
  __pyx_r = Py_True;
  goto __pyx_L0;

  /* "cupyx/cusolver.pyx":103
 * 
 * 
 * @_util.memoize(for_each_device=True)             # <<<<<<<<<<<<<<
 * def check_availability(name):
 *     if not _runtime.is_hip:
*/

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_7);
  __Pyx_XDECREF(__pyx_t_8);
  __Pyx_AddTraceback("cupyx.cusolver.check_availability", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_available_version);
  __Pyx_XDECREF(__pyx_v_msg);
  __Pyx_XDECREF(__pyx_v_version_added);
  __Pyx_XDECREF(__pyx_v_version_removed);
  __Pyx_XDECREF(__pyx_v_compute_capability);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "cupyx/cusolver.pyx":127
 * 
 * 
 * def gesvdj(a, full_matrices=True, compute_uv=True, overwrite_a=False):             # <<<<<<<<<<<<<<
 *     """Singular value decomposition using cusolverDn<t>gesvdj().
 * 
*/

/* Python wrapper */
static PyObject *__pyx_pw_5cupyx_8cusolver_3gesvdj(PyObject *__pyx_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
PyDoc_STRVAR(__pyx_doc_5cupyx_8cusolver_2gesvdj, "gesvdj(a, full_matrices=True, compute_uv=True, overwrite_a=False)\n\nSingular value decomposition using cusolverDn<t>gesvdj().\n\nFactorizes the matrix ``a`` into two unitary matrices ``u`` and ``v`` and\na singular values vector ``s`` such that ``a == u @ diag(s) @ v*``.\n\nArgs:\n    a (cupy.ndarray): The input matrix with dimension ``(M, N)``.\n    full_matrices (bool): If True, it returns u and v with dimensions\n        ``(M, M)`` and ``(N, N)``. Otherwise, the dimensions of u and v\n        are respectively ``(M, K)`` and ``(K, N)``, where\n        ``K = min(M, N)``.\n    compute_uv (bool): If ``False``, it only returns singular values.\n    overwrite_a (bool): If ``True``, matrix ``a`` might be overwritten.\n\nReturns:\n    tuple of :class:`cupy.ndarray`:\n        A tuple of ``(u, s, v)``.");
static PyMethodDef __pyx_mdef_5cupyx_8cusolver_3gesvdj = {"gesvdj", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_5cupyx_8cusolver_3gesvdj, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_5cupyx_8cusolver_2gesvdj};
static PyObject *__pyx_pw_5cupyx_8cusolver_3gesvdj(PyObject *__pyx_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
) {
  PyObject *__pyx_v_a = 0;
  PyObject *__pyx_v_full_matrices = 0;
  PyObject *__pyx_v_compute_uv = 0;
  PyObject *__pyx_v_overwrite_a = 0;
  #if !CYTHON_METH_FASTCALL
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  #endif
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject* values[4] = {0,0,0,0};
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("gesvdj (wrapper)", 0);
  #if !CYTHON_METH_FASTCALL
  #if CYTHON_ASSUME_SAFE_SIZE
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return NULL;
  #endif
  #endif
  __pyx_kwvalues = __Pyx_KwValues_FASTCALL(__pyx_args, __pyx_nargs);
  {
    PyObject ** const __pyx_pyargnames[] = {&__pyx_mstate_global->__pyx_n_u_a,&__pyx_mstate_global->__pyx_n_u_full_matrices,&__pyx_mstate_global->__pyx_n_u_compute_uv,&__pyx_mstate_global->__pyx_n_u_overwrite_a,0};
    const Py_ssize_t __pyx_kwds_len = (__pyx_kwds) ? __Pyx_NumKwargs_FASTCALL(__pyx_kwds) : 0;
    if (unlikely(__pyx_kwds_len < 0)) __PYX_ERR(0, 127, __pyx_L3_error)
    if (__pyx_kwds_len > 0) {
      switch (__pyx_nargs) {
        case  4:
        values[3] = __Pyx_ArgRef_FASTCALL(__pyx_args, 3);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[3])) __PYX_ERR(0, 127, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  3:
        values[2] = __Pyx_ArgRef_FASTCALL(__pyx_args, 2);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[2])) __PYX_ERR(0, 127, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  2:
        values[1] = __Pyx_ArgRef_FASTCALL(__pyx_args, 1);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[1])) __PYX_ERR(0, 127, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  1:
        values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(0, 127, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      const Py_ssize_t kwd_pos_args = __pyx_nargs;
      if (__Pyx_ParseKeywords(__pyx_kwds, __pyx_kwvalues, __pyx_pyargnames, 0, values, kwd_pos_args, __pyx_kwds_len, "gesvdj", 0) < (0)) __PYX_ERR(0, 127, __pyx_L3_error)
      if (!values[1]) values[1] = __Pyx_NewRef(((PyObject *)Py_True));
      if (!values[2]) values[2] = __Pyx_NewRef(((PyObject *)Py_True));
      if (!values[3]) values[3] = __Pyx_NewRef(((PyObject *)Py_False));
      for (Py_ssize_t i = __pyx_nargs; i < 1; i++) {
        if (unlikely(!values[i])) { __Pyx_RaiseArgtupleInvalid("gesvdj", 0, 1, 4, i); __PYX_ERR(0, 127, __pyx_L3_error) }
      }
    } else {
      switch (__pyx_nargs) {
        case  4:
        values[3] = __Pyx_ArgRef_FASTCALL(__pyx_args, 3);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[3])) __PYX_ERR(0, 127, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  3:
        values[2] = __Pyx_ArgRef_FASTCALL(__pyx_args, 2);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[2])) __PYX_ERR(0, 127, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  2:
        values[1] = __Pyx_ArgRef_FASTCALL(__pyx_args, 1);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[1])) __PYX_ERR(0, 127, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  1:
        values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(0, 127, __pyx_L3_error)
        break;
        default: goto __pyx_L5_argtuple_error;
      }
      if (!values[1]) values[1] = __Pyx_NewRef(((PyObject *)Py_True));
      if (!values[2]) values[2] = __Pyx_NewRef(((PyObject *)Py_True));
      if (!values[3]) values[3] = __Pyx_NewRef(((PyObject *)Py_False));
    }
    __pyx_v_a = values[0];
    __pyx_v_full_matrices = values[1];
    __pyx_v_compute_uv = values[2];
    __pyx_v_overwrite_a = values[3];
  }
  goto __pyx_L6_skip;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("gesvdj", 0, 1, 4, __pyx_nargs); __PYX_ERR(0, 127, __pyx_L3_error)
  __pyx_L6_skip:;
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_AddTraceback("cupyx.cusolver.gesvdj", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_5cupyx_8cusolver_2gesvdj(__pyx_self, __pyx_v_a, __pyx_v_full_matrices, __pyx_v_compute_uv, __pyx_v_overwrite_a);

  /* function exit code */
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5cupyx_8cusolver_2gesvdj(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_a, PyObject *__pyx_v_full_matrices, PyObject *__pyx_v_compute_uv, PyObject *__pyx_v_overwrite_a) {
  PyObject *__pyx_v_helper = NULL;
  PyObject *__pyx_v_solver = NULL;
  PyObject *__pyx_v_s_dtype = NULL;
  PyObject *__pyx_v_handle = NULL;
  PyObject *__pyx_v_m = NULL;
  PyObject *__pyx_v_n = NULL;
  PyObject *__pyx_v_lda = NULL;
  PyObject *__pyx_v_mn = NULL;
  PyObject *__pyx_v_s = NULL;
  PyObject *__pyx_v_ldu = NULL;
  PyObject *__pyx_v_ldv = NULL;
  PyObject *__pyx_v_jobz = NULL;
  long __pyx_v_econ;
  PyObject *__pyx_v_u = NULL;
  PyObject *__pyx_v_v = NULL;
  PyObject *__pyx_v_params = NULL;
  PyObject *__pyx_v_lwork = NULL;
  PyObject *__pyx_v_work = NULL;
  PyObject *__pyx_v_info = NULL;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_t_2;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  size_t __pyx_t_5;
  int __pyx_t_6;
  PyObject *__pyx_t_7 = NULL;
  PyObject *(*__pyx_t_8)(PyObject *);
  PyObject *__pyx_t_9 = NULL;
  PyObject *__pyx_t_10 = NULL;
  PyObject *__pyx_t_11 = NULL;
  PyObject *__pyx_t_12 = NULL;
  PyObject *__pyx_t_13 = NULL;
  PyObject *__pyx_t_14 = NULL;
  PyObject *__pyx_t_15 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("gesvdj", 0);
  __Pyx_INCREF(__pyx_v_a);
  __Pyx_INCREF(__pyx_v_full_matrices);

  /* "cupyx/cusolver.pyx":146
 *             A tuple of ``(u, s, v)``.
 *     """
 *     if a.ndim == 3:             # <<<<<<<<<<<<<<
 *         return _gesvdj_batched(a, full_matrices, compute_uv, overwrite_a)
 * 
*/
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_v_a, __pyx_mstate_global->__pyx_n_u_ndim); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 146, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = (__Pyx_PyLong_BoolEqObjC(__pyx_t_1, __pyx_mstate_global->__pyx_int_3, 3, 0)); if (unlikely((__pyx_t_2 < 0))) __PYX_ERR(0, 146, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  if (__pyx_t_2) {

    /* "cupyx/cusolver.pyx":147
 *     """
 *     if a.ndim == 3:
 *         return _gesvdj_batched(a, full_matrices, compute_uv, overwrite_a)             # <<<<<<<<<<<<<<
 * 
 *     if not check_availability('gesvdj'):
*/
    __Pyx_XDECREF(__pyx_r);
    __pyx_t_1 = __pyx_f_5cupyx_8cusolver__gesvdj_batched(__pyx_v_a, __pyx_v_full_matrices, __pyx_v_compute_uv, __pyx_v_overwrite_a, 0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 147, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_r = __pyx_t_1;
    __pyx_t_1 = 0;
    goto __pyx_L0;

    /* "cupyx/cusolver.pyx":146
 *             A tuple of ``(u, s, v)``.
 *     """
 *     if a.ndim == 3:             # <<<<<<<<<<<<<<
 *         return _gesvdj_batched(a, full_matrices, compute_uv, overwrite_a)
 * 
*/
  }

  /* "cupyx/cusolver.pyx":149
 *         return _gesvdj_batched(a, full_matrices, compute_uv, overwrite_a)
 * 
 *     if not check_availability('gesvdj'):             # <<<<<<<<<<<<<<
 *         raise RuntimeError('gesvdj is not available.')
 * 
*/
  __pyx_t_3 = NULL;
  __Pyx_GetModuleGlobalName(__pyx_t_4, __pyx_mstate_global->__pyx_n_u_check_availability); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 149, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_t_5 = 1;
  #if CYTHON_UNPACK_METHODS
  if (unlikely(PyMethod_Check(__pyx_t_4))) {
    __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_4);
    assert(__pyx_t_3);
    PyObject* __pyx__function = PyMethod_GET_FUNCTION(__pyx_t_4);
    __Pyx_INCREF(__pyx_t_3);
    __Pyx_INCREF(__pyx__function);
    __Pyx_DECREF_SET(__pyx_t_4, __pyx__function);
    __pyx_t_5 = 0;
  }
  #endif
  {
    PyObject *__pyx_callargs[2] = {__pyx_t_3, __pyx_mstate_global->__pyx_n_u_gesvdj};
    __pyx_t_1 = __Pyx_PyObject_FastCall((PyObject*)__pyx_t_4, __pyx_callargs+__pyx_t_5, (2-__pyx_t_5) | (__pyx_t_5*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
    __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 149, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
  }
  __pyx_t_2 = __Pyx_PyObject_IsTrue(__pyx_t_1); if (unlikely((__pyx_t_2 < 0))) __PYX_ERR(0, 149, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_6 = (!__pyx_t_2);
  if (unlikely(__pyx_t_6)) {

    /* "cupyx/cusolver.pyx":150
 * 
 *     if not check_availability('gesvdj'):
 *         raise RuntimeError('gesvdj is not available.')             # <<<<<<<<<<<<<<
 * 
 *     assert a.ndim == 2
*/
    __pyx_t_4 = NULL;
    __pyx_t_5 = 1;
    {
      PyObject *__pyx_callargs[2] = {__pyx_t_4, __pyx_mstate_global->__pyx_kp_u_gesvdj_is_not_available};
      __pyx_t_1 = __Pyx_PyObject_FastCall((PyObject*)(((PyTypeObject*)PyExc_RuntimeError)), __pyx_callargs+__pyx_t_5, (2-__pyx_t_5) | (__pyx_t_5*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
      __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
      if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 150, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
    }
    __Pyx_Raise(__pyx_t_1, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __PYX_ERR(0, 150, __pyx_L1_error)

    /* "cupyx/cusolver.pyx":149
 *         return _gesvdj_batched(a, full_matrices, compute_uv, overwrite_a)
 * 
 *     if not check_availability('gesvdj'):             # <<<<<<<<<<<<<<
 *         raise RuntimeError('gesvdj is not available.')
 * 
*/
  }

  /* "cupyx/cusolver.pyx":152
 *         raise RuntimeError('gesvdj is not available.')
 * 
 *     assert a.ndim == 2             # <<<<<<<<<<<<<<
 * 
 *     if a.dtype == 'f':
*/
  #ifndef CYTHON_WITHOUT_ASSERTIONS
  if (unlikely(__pyx_assertions_enabled())) {
    __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_v_a, __pyx_mstate_global->__pyx_n_u_ndim); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 152, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_t_6 = (__Pyx_PyLong_BoolEqObjC(__pyx_t_1, __pyx_mstate_global->__pyx_int_2, 2, 0)); if (unlikely((__pyx_t_6 < 0))) __PYX_ERR(0, 152, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    if (unlikely(!__pyx_t_6)) {
      __Pyx_Raise(((PyObject *)(((PyTypeObject*)PyExc_AssertionError))), 0, 0, 0);
      __PYX_ERR(0, 152, __pyx_L1_error)
    }
  }
  #else
  if ((1)); else __PYX_ERR(0, 152, __pyx_L1_error)
  #endif

  /* "cupyx/cusolver.pyx":154
 *     assert a.ndim == 2
 * 
 *     if a.dtype == 'f':             # <<<<<<<<<<<<<<
 *         helper = _cusolver.sgesvdj_bufferSize
 *         solver = _cusolver.sgesvdj
*/
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_v_a, __pyx_mstate_global->__pyx_n_u_dtype); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 154, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_6 = (__Pyx_PyUnicode_Equals(__pyx_t_1, __pyx_mstate_global->__pyx_n_u_f, Py_EQ)); if (unlikely((__pyx_t_6 < 0))) __PYX_ERR(0, 154, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  if (__pyx_t_6) {

    /* "cupyx/cusolver.pyx":155
 * 
 *     if a.dtype == 'f':
 *         helper = _cusolver.sgesvdj_bufferSize             # <<<<<<<<<<<<<<
 *         solver = _cusolver.sgesvdj
 *         s_dtype = 'f'
*/
    __Pyx_GetModuleGlobalName(__pyx_t_1, __pyx_mstate_global->__pyx_n_u_cusolver); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 155, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_t_4 = __Pyx_PyObject_GetAttrStr(__pyx_t_1, __pyx_mstate_global->__pyx_n_u_sgesvdj_bufferSize); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 155, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __pyx_v_helper = __pyx_t_4;
    __pyx_t_4 = 0;

    /* "cupyx/cusolver.pyx":156
 *     if a.dtype == 'f':
 *         helper = _cusolver.sgesvdj_bufferSize
 *         solver = _cusolver.sgesvdj             # <<<<<<<<<<<<<<
 *         s_dtype = 'f'
 *     elif a.dtype == 'd':
*/
    __Pyx_GetModuleGlobalName(__pyx_t_4, __pyx_mstate_global->__pyx_n_u_cusolver); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 156, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_t_4, __pyx_mstate_global->__pyx_n_u_sgesvdj); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 156, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __pyx_v_solver = __pyx_t_1;
    __pyx_t_1 = 0;

    /* "cupyx/cusolver.pyx":157
 *         helper = _cusolver.sgesvdj_bufferSize
 *         solver = _cusolver.sgesvdj
 *         s_dtype = 'f'             # <<<<<<<<<<<<<<
 *     elif a.dtype == 'd':
 *         helper = _cusolver.dgesvdj_bufferSize
*/
    __Pyx_INCREF(__pyx_mstate_global->__pyx_n_u_f);
    __pyx_v_s_dtype = __pyx_mstate_global->__pyx_n_u_f;

    /* "cupyx/cusolver.pyx":154
 *     assert a.ndim == 2
 * 
 *     if a.dtype == 'f':             # <<<<<<<<<<<<<<
 *         helper = _cusolver.sgesvdj_bufferSize
 *         solver = _cusolver.sgesvdj
*/
    goto __pyx_L5;
  }

  /* "cupyx/cusolver.pyx":158
 *         solver = _cusolver.sgesvdj
 *         s_dtype = 'f'
 *     elif a.dtype == 'd':             # <<<<<<<<<<<<<<
 *         helper = _cusolver.dgesvdj_bufferSize
 *         solver = _cusolver.dgesvdj
*/
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_v_a, __pyx_mstate_global->__pyx_n_u_dtype); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 158, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_6 = (__Pyx_PyUnicode_Equals(__pyx_t_1, __pyx_mstate_global->__pyx_n_u_d, Py_EQ)); if (unlikely((__pyx_t_6 < 0))) __PYX_ERR(0, 158, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  if (__pyx_t_6) {

    /* "cupyx/cusolver.pyx":159
 *         s_dtype = 'f'
 *     elif a.dtype == 'd':
 *         helper = _cusolver.dgesvdj_bufferSize             # <<<<<<<<<<<<<<
 *         solver = _cusolver.dgesvdj
 *         s_dtype = 'd'
*/
    __Pyx_GetModuleGlobalName(__pyx_t_1, __pyx_mstate_global->__pyx_n_u_cusolver); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 159, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_t_4 = __Pyx_PyObject_GetAttrStr(__pyx_t_1, __pyx_mstate_global->__pyx_n_u_dgesvdj_bufferSize); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 159, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __pyx_v_helper = __pyx_t_4;
    __pyx_t_4 = 0;

    /* "cupyx/cusolver.pyx":160
 *     elif a.dtype == 'd':
 *         helper = _cusolver.dgesvdj_bufferSize
 *         solver = _cusolver.dgesvdj             # <<<<<<<<<<<<<<
 *         s_dtype = 'd'
 *     elif a.dtype == 'F':
*/
    __Pyx_GetModuleGlobalName(__pyx_t_4, __pyx_mstate_global->__pyx_n_u_cusolver); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 160, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_t_4, __pyx_mstate_global->__pyx_n_u_dgesvdj); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 160, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __pyx_v_solver = __pyx_t_1;
    __pyx_t_1 = 0;

    /* "cupyx/cusolver.pyx":161
 *         helper = _cusolver.dgesvdj_bufferSize
 *         solver = _cusolver.dgesvdj
 *         s_dtype = 'd'             # <<<<<<<<<<<<<<
 *     elif a.dtype == 'F':
 *         helper = _cusolver.cgesvdj_bufferSize
*/
    __Pyx_INCREF(__pyx_mstate_global->__pyx_n_u_d);
    __pyx_v_s_dtype = __pyx_mstate_global->__pyx_n_u_d;

    /* "cupyx/cusolver.pyx":158
 *         solver = _cusolver.sgesvdj
 *         s_dtype = 'f'
 *     elif a.dtype == 'd':             # <<<<<<<<<<<<<<
 *         helper = _cusolver.dgesvdj_bufferSize
 *         solver = _cusolver.dgesvdj
*/
    goto __pyx_L5;
  }

  /* "cupyx/cusolver.pyx":162
 *         solver = _cusolver.dgesvdj
 *         s_dtype = 'd'
 *     elif a.dtype == 'F':             # <<<<<<<<<<<<<<
 *         helper = _cusolver.cgesvdj_bufferSize
 *         solver = _cusolver.cgesvdj
*/
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_v_a, __pyx_mstate_global->__pyx_n_u_dtype); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 162, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_6 = (__Pyx_PyUnicode_Equals(__pyx_t_1, __pyx_mstate_global->__pyx_n_u_F, Py_EQ)); if (unlikely((__pyx_t_6 < 0))) __PYX_ERR(0, 162, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  if (__pyx_t_6) {

    /* "cupyx/cusolver.pyx":163
 *         s_dtype = 'd'
 *     elif a.dtype == 'F':
 *         helper = _cusolver.cgesvdj_bufferSize             # <<<<<<<<<<<<<<
 *         solver = _cusolver.cgesvdj
 *         s_dtype = 'f'
*/
    __Pyx_GetModuleGlobalName(__pyx_t_1, __pyx_mstate_global->__pyx_n_u_cusolver); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 163, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_t_4 = __Pyx_PyObject_GetAttrStr(__pyx_t_1, __pyx_mstate_global->__pyx_n_u_cgesvdj_bufferSize); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 163, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __pyx_v_helper = __pyx_t_4;
    __pyx_t_4 = 0;

    /* "cupyx/cusolver.pyx":164
 *     elif a.dtype == 'F':
 *         helper = _cusolver.cgesvdj_bufferSize
 *         solver = _cusolver.cgesvdj             # <<<<<<<<<<<<<<
 *         s_dtype = 'f'
 *     elif a.dtype == 'D':
*/
    __Pyx_GetModuleGlobalName(__pyx_t_4, __pyx_mstate_global->__pyx_n_u_cusolver); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 164, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_t_4, __pyx_mstate_global->__pyx_n_u_cgesvdj); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 164, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __pyx_v_solver = __pyx_t_1;
    __pyx_t_1 = 0;

    /* "cupyx/cusolver.pyx":165
 *         helper = _cusolver.cgesvdj_bufferSize
 *         solver = _cusolver.cgesvdj
 *         s_dtype = 'f'             # <<<<<<<<<<<<<<
 *     elif a.dtype == 'D':
 *         helper = _cusolver.zgesvdj_bufferSize
*/
    __Pyx_INCREF(__pyx_mstate_global->__pyx_n_u_f);
    __pyx_v_s_dtype = __pyx_mstate_global->__pyx_n_u_f;

    /* "cupyx/cusolver.pyx":162
 *         solver = _cusolver.dgesvdj
 *         s_dtype = 'd'
 *     elif a.dtype == 'F':             # <<<<<<<<<<<<<<
 *         helper = _cusolver.cgesvdj_bufferSize
 *         solver = _cusolver.cgesvdj
*/
    goto __pyx_L5;
  }

  /* "cupyx/cusolver.pyx":166
 *         solver = _cusolver.cgesvdj
 *         s_dtype = 'f'
 *     elif a.dtype == 'D':             # <<<<<<<<<<<<<<
 *         helper = _cusolver.zgesvdj_bufferSize
 *         solver = _cusolver.zgesvdj
*/
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_v_a, __pyx_mstate_global->__pyx_n_u_dtype); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 166, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_6 = (__Pyx_PyUnicode_Equals(__pyx_t_1, __pyx_mstate_global->__pyx_n_u_D, Py_EQ)); if (unlikely((__pyx_t_6 < 0))) __PYX_ERR(0, 166, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  if (likely(__pyx_t_6)) {

    /* "cupyx/cusolver.pyx":167
 *         s_dtype = 'f'
 *     elif a.dtype == 'D':
 *         helper = _cusolver.zgesvdj_bufferSize             # <<<<<<<<<<<<<<
 *         solver = _cusolver.zgesvdj
 *         s_dtype = 'd'
*/
    __Pyx_GetModuleGlobalName(__pyx_t_1, __pyx_mstate_global->__pyx_n_u_cusolver); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 167, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_t_4 = __Pyx_PyObject_GetAttrStr(__pyx_t_1, __pyx_mstate_global->__pyx_n_u_zgesvdj_bufferSize); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 167, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __pyx_v_helper = __pyx_t_4;
    __pyx_t_4 = 0;

    /* "cupyx/cusolver.pyx":168
 *     elif a.dtype == 'D':
 *         helper = _cusolver.zgesvdj_bufferSize
 *         solver = _cusolver.zgesvdj             # <<<<<<<<<<<<<<
 *         s_dtype = 'd'
 *     else:
*/
    __Pyx_GetModuleGlobalName(__pyx_t_4, __pyx_mstate_global->__pyx_n_u_cusolver); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 168, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_t_4, __pyx_mstate_global->__pyx_n_u_zgesvdj); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 168, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __pyx_v_solver = __pyx_t_1;
    __pyx_t_1 = 0;

    /* "cupyx/cusolver.pyx":169
 *         helper = _cusolver.zgesvdj_bufferSize
 *         solver = _cusolver.zgesvdj
 *         s_dtype = 'd'             # <<<<<<<<<<<<<<
 *     else:
 *         raise TypeError
*/
    __Pyx_INCREF(__pyx_mstate_global->__pyx_n_u_d);
    __pyx_v_s_dtype = __pyx_mstate_global->__pyx_n_u_d;

    /* "cupyx/cusolver.pyx":166
 *         solver = _cusolver.cgesvdj
 *         s_dtype = 'f'
 *     elif a.dtype == 'D':             # <<<<<<<<<<<<<<
 *         helper = _cusolver.zgesvdj_bufferSize
 *         solver = _cusolver.zgesvdj
*/
    goto __pyx_L5;
  }

  /* "cupyx/cusolver.pyx":171
 *         s_dtype = 'd'
 *     else:
 *         raise TypeError             # <<<<<<<<<<<<<<
 * 
 *     handle = _device.get_cusolver_handle()
*/
  /*else*/ {
    __Pyx_Raise(((PyObject *)(((PyTypeObject*)PyExc_TypeError))), 0, 0, 0);
    __PYX_ERR(0, 171, __pyx_L1_error)
  }
  __pyx_L5:;

  /* "cupyx/cusolver.pyx":173
 *         raise TypeError
 * 
 *     handle = _device.get_cusolver_handle()             # <<<<<<<<<<<<<<
 *     m, n = a.shape
 *     a = _cupy.array(a, order='F', copy=(None if overwrite_a else True))
*/
  __pyx_t_4 = NULL;
  __Pyx_GetModuleGlobalName(__pyx_t_3, __pyx_mstate_global->__pyx_n_u_device); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 173, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_7 = __Pyx_PyObject_GetAttrStr(__pyx_t_3, __pyx_mstate_global->__pyx_n_u_get_cusolver_handle); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 173, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_7);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_t_5 = 1;
  #if CYTHON_UNPACK_METHODS
  if (unlikely(PyMethod_Check(__pyx_t_7))) {
    __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_7);
    assert(__pyx_t_4);
    PyObject* __pyx__function = PyMethod_GET_FUNCTION(__pyx_t_7);
    __Pyx_INCREF(__pyx_t_4);
    __Pyx_INCREF(__pyx__function);
    __Pyx_DECREF_SET(__pyx_t_7, __pyx__function);
    __pyx_t_5 = 0;
  }
  #endif
  {
    PyObject *__pyx_callargs[2] = {__pyx_t_4, NULL};
    __pyx_t_1 = __Pyx_PyObject_FastCall((PyObject*)__pyx_t_7, __pyx_callargs+__pyx_t_5, (1-__pyx_t_5) | (__pyx_t_5*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
    __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
    __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
    if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 173, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
  }
  __pyx_v_handle = __pyx_t_1;
  __pyx_t_1 = 0;

  /* "cupyx/cusolver.pyx":174
 * 
 *     handle = _device.get_cusolver_handle()
 *     m, n = a.shape             # <<<<<<<<<<<<<<
 *     a = _cupy.array(a, order='F', copy=(None if overwrite_a else True))
 *     lda = m
*/
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_v_a, __pyx_mstate_global->__pyx_n_u_shape); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 174, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if ((likely(PyTuple_CheckExact(__pyx_t_1))) || (PyList_CheckExact(__pyx_t_1))) {
    PyObject* sequence = __pyx_t_1;
    Py_ssize_t size = __Pyx_PySequence_SIZE(sequence);
    if (unlikely(size != 2)) {
      if (size > 2) __Pyx_RaiseTooManyValuesError(2);
      else if (size >= 0) __Pyx_RaiseNeedMoreValuesError(size);
      __PYX_ERR(0, 174, __pyx_L1_error)
    }
    #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
    if (likely(PyTuple_CheckExact(sequence))) {
      __pyx_t_7 = PyTuple_GET_ITEM(sequence, 0);
      __Pyx_INCREF(__pyx_t_7);
      __pyx_t_4 = PyTuple_GET_ITEM(sequence, 1);
      __Pyx_INCREF(__pyx_t_4);
    } else {
      __pyx_t_7 = __Pyx_PyList_GetItemRefFast(sequence, 0, __Pyx_ReferenceSharing_SharedReference);
      if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 174, __pyx_L1_error)
      __Pyx_XGOTREF(__pyx_t_7);
      __pyx_t_4 = __Pyx_PyList_GetItemRefFast(sequence, 1, __Pyx_ReferenceSharing_SharedReference);
      if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 174, __pyx_L1_error)
      __Pyx_XGOTREF(__pyx_t_4);
    }
    #else
    __pyx_t_7 = __Pyx_PySequence_ITEM(sequence, 0); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 174, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_7);
    __pyx_t_4 = __Pyx_PySequence_ITEM(sequence, 1); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 174, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    #endif
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  } else {
    Py_ssize_t index = -1;
    __pyx_t_3 = PyObject_GetIter(__pyx_t_1); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 174, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __pyx_t_8 = (CYTHON_COMPILING_IN_LIMITED_API) ? PyIter_Next : __Pyx_PyObject_GetIterNextFunc(__pyx_t_3);
    index = 0; __pyx_t_7 = __pyx_t_8(__pyx_t_3); if (unlikely(!__pyx_t_7)) goto __pyx_L6_unpacking_failed;
    __Pyx_GOTREF(__pyx_t_7);
    index = 1; __pyx_t_4 = __pyx_t_8(__pyx_t_3); if (unlikely(!__pyx_t_4)) goto __pyx_L6_unpacking_failed;
    __Pyx_GOTREF(__pyx_t_4);
    if (__Pyx_IternextUnpackEndCheck(__pyx_t_8(__pyx_t_3), 2) < (0)) __PYX_ERR(0, 174, __pyx_L1_error)
    __pyx_t_8 = NULL;
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    goto __pyx_L7_unpacking_done;
    __pyx_L6_unpacking_failed:;
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __pyx_t_8 = NULL;
    if (__Pyx_IterFinish() == 0) __Pyx_RaiseNeedMoreValuesError(index);
    __PYX_ERR(0, 174, __pyx_L1_error)
    __pyx_L7_unpacking_done:;
  }
  __pyx_v_m = __pyx_t_7;
  __pyx_t_7 = 0;
  __pyx_v_n = __pyx_t_4;
  __pyx_t_4 = 0;

  /* "cupyx/cusolver.pyx":175
 *     handle = _device.get_cusolver_handle()
 *     m, n = a.shape
 *     a = _cupy.array(a, order='F', copy=(None if overwrite_a else True))             # <<<<<<<<<<<<<<
 *     lda = m
 *     mn = min(m, n)
*/
  __pyx_t_4 = NULL;
  __Pyx_GetModuleGlobalName(__pyx_t_7, __pyx_mstate_global->__pyx_n_u_cupy); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 175, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_7);
  __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_t_7, __pyx_mstate_global->__pyx_n_u_array); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 175, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
  __pyx_t_6 = __Pyx_PyObject_IsTrue(__pyx_v_overwrite_a); if (unlikely((__pyx_t_6 < 0))) __PYX_ERR(0, 175, __pyx_L1_error)
  if (__pyx_t_6) {
    __Pyx_INCREF(Py_None);
    __pyx_t_7 = Py_None;
  } else {
    __Pyx_INCREF(Py_True);
    __pyx_t_7 = Py_True;
  }
  __pyx_t_5 = 1;
  #if CYTHON_UNPACK_METHODS
  if (unlikely(PyMethod_Check(__pyx_t_3))) {
    __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_3);
    assert(__pyx_t_4);
    PyObject* __pyx__function = PyMethod_GET_FUNCTION(__pyx_t_3);
    __Pyx_INCREF(__pyx_t_4);
    __Pyx_INCREF(__pyx__function);
    __Pyx_DECREF_SET(__pyx_t_3, __pyx__function);
    __pyx_t_5 = 0;
  }
  #endif
  {
    PyObject *__pyx_callargs[2 + ((CYTHON_VECTORCALL) ? 2 : 0)] = {__pyx_t_4, __pyx_v_a};
    __pyx_t_9 = __Pyx_MakeVectorcallBuilderKwds(2); if (unlikely(!__pyx_t_9)) __PYX_ERR(0, 175, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_9);
    if (__Pyx_VectorcallBuilder_AddArg(__pyx_mstate_global->__pyx_n_u_order, __pyx_mstate_global->__pyx_n_u_F, __pyx_t_9, __pyx_callargs+2, 0) < (0)) __PYX_ERR(0, 175, __pyx_L1_error)
    if (__Pyx_VectorcallBuilder_AddArg(__pyx_mstate_global->__pyx_n_u_copy, __pyx_t_7, __pyx_t_9, __pyx_callargs+2, 1) < (0)) __PYX_ERR(0, 175, __pyx_L1_error)
    __pyx_t_1 = __Pyx_Object_Vectorcall_CallFromBuilder((PyObject*)__pyx_t_3, __pyx_callargs+__pyx_t_5, (2-__pyx_t_5) | (__pyx_t_5*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET), __pyx_t_9);
    __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
    __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
    __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 175, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
  }
  __Pyx_DECREF_SET(__pyx_v_a, __pyx_t_1);
  __pyx_t_1 = 0;

  /* "cupyx/cusolver.pyx":176
 *     m, n = a.shape
 *     a = _cupy.array(a, order='F', copy=(None if overwrite_a else True))
 *     lda = m             # <<<<<<<<<<<<<<
 *     mn = min(m, n)
 *     s = _cupy.empty(mn, dtype=s_dtype)
*/
  __Pyx_INCREF(__pyx_v_m);
  __pyx_v_lda = __pyx_v_m;

  /* "cupyx/cusolver.pyx":177
 *     a = _cupy.array(a, order='F', copy=(None if overwrite_a else True))
 *     lda = m
 *     mn = min(m, n)             # <<<<<<<<<<<<<<
 *     s = _cupy.empty(mn, dtype=s_dtype)
 *     ldu = m
*/
  __Pyx_INCREF(__pyx_v_n);
  __pyx_t_1 = __pyx_v_n;
  __Pyx_INCREF(__pyx_v_m);
  __pyx_t_3 = __pyx_v_m;
  __pyx_t_7 = PyObject_RichCompare(__pyx_t_1, __pyx_t_3, Py_LT); __Pyx_XGOTREF(__pyx_t_7); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 177, __pyx_L1_error)
  __pyx_t_6 = __Pyx_PyObject_IsTrue(__pyx_t_7); if (unlikely((__pyx_t_6 < 0))) __PYX_ERR(0, 177, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
  if (__pyx_t_6) {
    __Pyx_INCREF(__pyx_t_1);
    __pyx_t_9 = __pyx_t_1;
  } else {
    __Pyx_INCREF(__pyx_t_3);
    __pyx_t_9 = __pyx_t_3;
  }
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_1 = __pyx_t_9;
  __Pyx_INCREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
  __pyx_v_mn = __pyx_t_1;
  __pyx_t_1 = 0;

  /* "cupyx/cusolver.pyx":178
 *     lda = m
 *     mn = min(m, n)
 *     s = _cupy.empty(mn, dtype=s_dtype)             # <<<<<<<<<<<<<<
 *     ldu = m
 *     ldv = n
*/
  __pyx_t_9 = NULL;
  __Pyx_GetModuleGlobalName(__pyx_t_3, __pyx_mstate_global->__pyx_n_u_cupy); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 178, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_7 = __Pyx_PyObject_GetAttrStr(__pyx_t_3, __pyx_mstate_global->__pyx_n_u_empty); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 178, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_7);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_t_5 = 1;
  #if CYTHON_UNPACK_METHODS
  if (unlikely(PyMethod_Check(__pyx_t_7))) {
    __pyx_t_9 = PyMethod_GET_SELF(__pyx_t_7);
    assert(__pyx_t_9);
    PyObject* __pyx__function = PyMethod_GET_FUNCTION(__pyx_t_7);
    __Pyx_INCREF(__pyx_t_9);
    __Pyx_INCREF(__pyx__function);
    __Pyx_DECREF_SET(__pyx_t_7, __pyx__function);
    __pyx_t_5 = 0;
  }
  #endif
  {
    PyObject *__pyx_callargs[2 + ((CYTHON_VECTORCALL) ? 1 : 0)] = {__pyx_t_9, __pyx_v_mn};
    __pyx_t_3 = __Pyx_MakeVectorcallBuilderKwds(1); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 178, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    if (__Pyx_VectorcallBuilder_AddArg(__pyx_mstate_global->__pyx_n_u_dtype, __pyx_v_s_dtype, __pyx_t_3, __pyx_callargs+2, 0) < (0)) __PYX_ERR(0, 178, __pyx_L1_error)
    __pyx_t_1 = __Pyx_Object_Vectorcall_CallFromBuilder((PyObject*)__pyx_t_7, __pyx_callargs+__pyx_t_5, (2-__pyx_t_5) | (__pyx_t_5*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET), __pyx_t_3);
    __Pyx_XDECREF(__pyx_t_9); __pyx_t_9 = 0;
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
    if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 178, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
  }
  __pyx_v_s = __pyx_t_1;
  __pyx_t_1 = 0;

  /* "cupyx/cusolver.pyx":179
 *     mn = min(m, n)
 *     s = _cupy.empty(mn, dtype=s_dtype)
 *     ldu = m             # <<<<<<<<<<<<<<
 *     ldv = n
 *     if compute_uv:
*/
  __Pyx_INCREF(__pyx_v_m);
  __pyx_v_ldu = __pyx_v_m;

  /* "cupyx/cusolver.pyx":180
 *     s = _cupy.empty(mn, dtype=s_dtype)
 *     ldu = m
 *     ldv = n             # <<<<<<<<<<<<<<
 *     if compute_uv:
 *         jobz = _cusolver.CUSOLVER_EIG_MODE_VECTOR
*/
  __Pyx_INCREF(__pyx_v_n);
  __pyx_v_ldv = __pyx_v_n;

  /* "cupyx/cusolver.pyx":181
 *     ldu = m
 *     ldv = n
 *     if compute_uv:             # <<<<<<<<<<<<<<
 *         jobz = _cusolver.CUSOLVER_EIG_MODE_VECTOR
 *     else:
*/
  __pyx_t_6 = __Pyx_PyObject_IsTrue(__pyx_v_compute_uv); if (unlikely((__pyx_t_6 < 0))) __PYX_ERR(0, 181, __pyx_L1_error)
  if (__pyx_t_6) {

    /* "cupyx/cusolver.pyx":182
 *     ldv = n
 *     if compute_uv:
 *         jobz = _cusolver.CUSOLVER_EIG_MODE_VECTOR             # <<<<<<<<<<<<<<
 *     else:
 *         jobz = _cusolver.CUSOLVER_EIG_MODE_NOVECTOR
*/
    __Pyx_GetModuleGlobalName(__pyx_t_1, __pyx_mstate_global->__pyx_n_u_cusolver); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 182, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_t_7 = __Pyx_PyObject_GetAttrStr(__pyx_t_1, __pyx_mstate_global->__pyx_n_u_CUSOLVER_EIG_MODE_VECTOR); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 182, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_7);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __pyx_v_jobz = __pyx_t_7;
    __pyx_t_7 = 0;

    /* "cupyx/cusolver.pyx":181
 *     ldu = m
 *     ldv = n
 *     if compute_uv:             # <<<<<<<<<<<<<<
 *         jobz = _cusolver.CUSOLVER_EIG_MODE_VECTOR
 *     else:
*/
    goto __pyx_L8;
  }

  /* "cupyx/cusolver.pyx":184
 *         jobz = _cusolver.CUSOLVER_EIG_MODE_VECTOR
 *     else:
 *         jobz = _cusolver.CUSOLVER_EIG_MODE_NOVECTOR             # <<<<<<<<<<<<<<
 *         full_matrices = False
 *     if full_matrices:
*/
  /*else*/ {
    __Pyx_GetModuleGlobalName(__pyx_t_7, __pyx_mstate_global->__pyx_n_u_cusolver); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 184, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_7);
    __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_t_7, __pyx_mstate_global->__pyx_n_u_CUSOLVER_EIG_MODE_NOVECTOR); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 184, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
    __pyx_v_jobz = __pyx_t_1;
    __pyx_t_1 = 0;

    /* "cupyx/cusolver.pyx":185
 *     else:
 *         jobz = _cusolver.CUSOLVER_EIG_MODE_NOVECTOR
 *         full_matrices = False             # <<<<<<<<<<<<<<
 *     if full_matrices:
 *         econ = 0
*/
    __Pyx_INCREF(Py_False);
    __Pyx_DECREF_SET(__pyx_v_full_matrices, Py_False);
  }
  __pyx_L8:;

  /* "cupyx/cusolver.pyx":186
 *         jobz = _cusolver.CUSOLVER_EIG_MODE_NOVECTOR
 *         full_matrices = False
 *     if full_matrices:             # <<<<<<<<<<<<<<
 *         econ = 0
 *         u = _cupy.empty((ldu, m), dtype=a.dtype, order='F')
*/
  __pyx_t_6 = __Pyx_PyObject_IsTrue(__pyx_v_full_matrices); if (unlikely((__pyx_t_6 < 0))) __PYX_ERR(0, 186, __pyx_L1_error)
  if (__pyx_t_6) {

    /* "cupyx/cusolver.pyx":187
 *         full_matrices = False
 *     if full_matrices:
 *         econ = 0             # <<<<<<<<<<<<<<
 *         u = _cupy.empty((ldu, m), dtype=a.dtype, order='F')
 *         v = _cupy.empty((ldv, n), dtype=a.dtype, order='F')
*/
    __pyx_v_econ = 0;

    /* "cupyx/cusolver.pyx":188
 *     if full_matrices:
 *         econ = 0
 *         u = _cupy.empty((ldu, m), dtype=a.dtype, order='F')             # <<<<<<<<<<<<<<
 *         v = _cupy.empty((ldv, n), dtype=a.dtype, order='F')
 *     else:
*/
    __pyx_t_7 = NULL;
    __Pyx_GetModuleGlobalName(__pyx_t_3, __pyx_mstate_global->__pyx_n_u_cupy); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 188, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_t_9 = __Pyx_PyObject_GetAttrStr(__pyx_t_3, __pyx_mstate_global->__pyx_n_u_empty); if (unlikely(!__pyx_t_9)) __PYX_ERR(0, 188, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_9);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __pyx_t_3 = PyTuple_New(2); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 188, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_INCREF(__pyx_v_ldu);
    __Pyx_GIVEREF(__pyx_v_ldu);
    if (__Pyx_PyTuple_SET_ITEM(__pyx_t_3, 0, __pyx_v_ldu) != (0)) __PYX_ERR(0, 188, __pyx_L1_error);
    __Pyx_INCREF(__pyx_v_m);
    __Pyx_GIVEREF(__pyx_v_m);
    if (__Pyx_PyTuple_SET_ITEM(__pyx_t_3, 1, __pyx_v_m) != (0)) __PYX_ERR(0, 188, __pyx_L1_error);
    __pyx_t_4 = __Pyx_PyObject_GetAttrStr(__pyx_v_a, __pyx_mstate_global->__pyx_n_u_dtype); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 188, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __pyx_t_5 = 1;
    #if CYTHON_UNPACK_METHODS
    if (unlikely(PyMethod_Check(__pyx_t_9))) {
      __pyx_t_7 = PyMethod_GET_SELF(__pyx_t_9);
      assert(__pyx_t_7);
      PyObject* __pyx__function = PyMethod_GET_FUNCTION(__pyx_t_9);
      __Pyx_INCREF(__pyx_t_7);
      __Pyx_INCREF(__pyx__function);
      __Pyx_DECREF_SET(__pyx_t_9, __pyx__function);
      __pyx_t_5 = 0;
    }
    #endif
    {
      PyObject *__pyx_callargs[2 + ((CYTHON_VECTORCALL) ? 2 : 0)] = {__pyx_t_7, __pyx_t_3};
      __pyx_t_10 = __Pyx_MakeVectorcallBuilderKwds(2); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 188, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_10);
      if (__Pyx_VectorcallBuilder_AddArg(__pyx_mstate_global->__pyx_n_u_dtype, __pyx_t_4, __pyx_t_10, __pyx_callargs+2, 0) < (0)) __PYX_ERR(0, 188, __pyx_L1_error)
      if (__Pyx_VectorcallBuilder_AddArg(__pyx_mstate_global->__pyx_n_u_order, __pyx_mstate_global->__pyx_n_u_F, __pyx_t_10, __pyx_callargs+2, 1) < (0)) __PYX_ERR(0, 188, __pyx_L1_error)
      __pyx_t_1 = __Pyx_Object_Vectorcall_CallFromBuilder((PyObject*)__pyx_t_9, __pyx_callargs+__pyx_t_5, (2-__pyx_t_5) | (__pyx_t_5*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET), __pyx_t_10);
      __Pyx_XDECREF(__pyx_t_7); __pyx_t_7 = 0;
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
      __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
      if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 188, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
    }
    __pyx_v_u = __pyx_t_1;
    __pyx_t_1 = 0;

    /* "cupyx/cusolver.pyx":189
 *         econ = 0
 *         u = _cupy.empty((ldu, m), dtype=a.dtype, order='F')
 *         v = _cupy.empty((ldv, n), dtype=a.dtype, order='F')             # <<<<<<<<<<<<<<
 *     else:
 *         econ = 1
*/
    __pyx_t_9 = NULL;
    __Pyx_GetModuleGlobalName(__pyx_t_10, __pyx_mstate_global->__pyx_n_u_cupy); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 189, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_10);
    __pyx_t_4 = __Pyx_PyObject_GetAttrStr(__pyx_t_10, __pyx_mstate_global->__pyx_n_u_empty); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 189, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
    __pyx_t_10 = PyTuple_New(2); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 189, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_10);
    __Pyx_INCREF(__pyx_v_ldv);
    __Pyx_GIVEREF(__pyx_v_ldv);
    if (__Pyx_PyTuple_SET_ITEM(__pyx_t_10, 0, __pyx_v_ldv) != (0)) __PYX_ERR(0, 189, __pyx_L1_error);
    __Pyx_INCREF(__pyx_v_n);
    __Pyx_GIVEREF(__pyx_v_n);
    if (__Pyx_PyTuple_SET_ITEM(__pyx_t_10, 1, __pyx_v_n) != (0)) __PYX_ERR(0, 189, __pyx_L1_error);
    __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_v_a, __pyx_mstate_global->__pyx_n_u_dtype); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 189, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_t_5 = 1;
    #if CYTHON_UNPACK_METHODS
    if (unlikely(PyMethod_Check(__pyx_t_4))) {
      __pyx_t_9 = PyMethod_GET_SELF(__pyx_t_4);
      assert(__pyx_t_9);
      PyObject* __pyx__function = PyMethod_GET_FUNCTION(__pyx_t_4);
      __Pyx_INCREF(__pyx_t_9);
      __Pyx_INCREF(__pyx__function);
      __Pyx_DECREF_SET(__pyx_t_4, __pyx__function);
      __pyx_t_5 = 0;
    }
    #endif
    {
      PyObject *__pyx_callargs[2 + ((CYTHON_VECTORCALL) ? 2 : 0)] = {__pyx_t_9, __pyx_t_10};
      __pyx_t_7 = __Pyx_MakeVectorcallBuilderKwds(2); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 189, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_7);
      if (__Pyx_VectorcallBuilder_AddArg(__pyx_mstate_global->__pyx_n_u_dtype, __pyx_t_3, __pyx_t_7, __pyx_callargs+2, 0) < (0)) __PYX_ERR(0, 189, __pyx_L1_error)
      if (__Pyx_VectorcallBuilder_AddArg(__pyx_mstate_global->__pyx_n_u_order, __pyx_mstate_global->__pyx_n_u_F, __pyx_t_7, __pyx_callargs+2, 1) < (0)) __PYX_ERR(0, 189, __pyx_L1_error)
      __pyx_t_1 = __Pyx_Object_Vectorcall_CallFromBuilder((PyObject*)__pyx_t_4, __pyx_callargs+__pyx_t_5, (2-__pyx_t_5) | (__pyx_t_5*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET), __pyx_t_7);
      __Pyx_XDECREF(__pyx_t_9); __pyx_t_9 = 0;
      __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
      __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 189, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
    }
    __pyx_v_v = __pyx_t_1;
    __pyx_t_1 = 0;

    /* "cupyx/cusolver.pyx":186
 *         jobz = _cusolver.CUSOLVER_EIG_MODE_NOVECTOR
 *         full_matrices = False
 *     if full_matrices:             # <<<<<<<<<<<<<<
 *         econ = 0
 *         u = _cupy.empty((ldu, m), dtype=a.dtype, order='F')
*/
    goto __pyx_L9;
  }

  /* "cupyx/cusolver.pyx":191
 *         v = _cupy.empty((ldv, n), dtype=a.dtype, order='F')
 *     else:
 *         econ = 1             # <<<<<<<<<<<<<<
 *         u = _cupy.empty((ldu, mn), dtype=a.dtype, order='F')
 *         v = _cupy.empty((ldv, mn), dtype=a.dtype, order='F')
*/
  /*else*/ {
    __pyx_v_econ = 1;

    /* "cupyx/cusolver.pyx":192
 *     else:
 *         econ = 1
 *         u = _cupy.empty((ldu, mn), dtype=a.dtype, order='F')             # <<<<<<<<<<<<<<
 *         v = _cupy.empty((ldv, mn), dtype=a.dtype, order='F')
 *     params = _cusolver.createGesvdjInfo()
*/
    __pyx_t_4 = NULL;
    __Pyx_GetModuleGlobalName(__pyx_t_7, __pyx_mstate_global->__pyx_n_u_cupy); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 192, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_7);
    __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_t_7, __pyx_mstate_global->__pyx_n_u_empty); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 192, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
    __pyx_t_7 = PyTuple_New(2); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 192, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_7);
    __Pyx_INCREF(__pyx_v_ldu);
    __Pyx_GIVEREF(__pyx_v_ldu);
    if (__Pyx_PyTuple_SET_ITEM(__pyx_t_7, 0, __pyx_v_ldu) != (0)) __PYX_ERR(0, 192, __pyx_L1_error);
    __Pyx_INCREF(__pyx_v_mn);
    __Pyx_GIVEREF(__pyx_v_mn);
    if (__Pyx_PyTuple_SET_ITEM(__pyx_t_7, 1, __pyx_v_mn) != (0)) __PYX_ERR(0, 192, __pyx_L1_error);
    __pyx_t_10 = __Pyx_PyObject_GetAttrStr(__pyx_v_a, __pyx_mstate_global->__pyx_n_u_dtype); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 192, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_10);
    __pyx_t_5 = 1;
    #if CYTHON_UNPACK_METHODS
    if (unlikely(PyMethod_Check(__pyx_t_3))) {
      __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_3);
      assert(__pyx_t_4);
      PyObject* __pyx__function = PyMethod_GET_FUNCTION(__pyx_t_3);
      __Pyx_INCREF(__pyx_t_4);
      __Pyx_INCREF(__pyx__function);
      __Pyx_DECREF_SET(__pyx_t_3, __pyx__function);
      __pyx_t_5 = 0;
    }
    #endif
    {
      PyObject *__pyx_callargs[2 + ((CYTHON_VECTORCALL) ? 2 : 0)] = {__pyx_t_4, __pyx_t_7};
      __pyx_t_9 = __Pyx_MakeVectorcallBuilderKwds(2); if (unlikely(!__pyx_t_9)) __PYX_ERR(0, 192, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_9);
      if (__Pyx_VectorcallBuilder_AddArg(__pyx_mstate_global->__pyx_n_u_dtype, __pyx_t_10, __pyx_t_9, __pyx_callargs+2, 0) < (0)) __PYX_ERR(0, 192, __pyx_L1_error)
      if (__Pyx_VectorcallBuilder_AddArg(__pyx_mstate_global->__pyx_n_u_order, __pyx_mstate_global->__pyx_n_u_F, __pyx_t_9, __pyx_callargs+2, 1) < (0)) __PYX_ERR(0, 192, __pyx_L1_error)
      __pyx_t_1 = __Pyx_Object_Vectorcall_CallFromBuilder((PyObject*)__pyx_t_3, __pyx_callargs+__pyx_t_5, (2-__pyx_t_5) | (__pyx_t_5*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET), __pyx_t_9);
      __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
      __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
      __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
      __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
      if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 192, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
    }
    __pyx_v_u = __pyx_t_1;
    __pyx_t_1 = 0;

    /* "cupyx/cusolver.pyx":193
 *         econ = 1
 *         u = _cupy.empty((ldu, mn), dtype=a.dtype, order='F')
 *         v = _cupy.empty((ldv, mn), dtype=a.dtype, order='F')             # <<<<<<<<<<<<<<
 *     params = _cusolver.createGesvdjInfo()
 *     lwork = helper(handle, jobz, econ, m, n, a.data.ptr, lda, s.data.ptr,
*/
    __pyx_t_3 = NULL;
    __Pyx_GetModuleGlobalName(__pyx_t_9, __pyx_mstate_global->__pyx_n_u_cupy); if (unlikely(!__pyx_t_9)) __PYX_ERR(0, 193, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_9);
    __pyx_t_10 = __Pyx_PyObject_GetAttrStr(__pyx_t_9, __pyx_mstate_global->__pyx_n_u_empty); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 193, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_10);
    __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
    __pyx_t_9 = PyTuple_New(2); if (unlikely(!__pyx_t_9)) __PYX_ERR(0, 193, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_9);
    __Pyx_INCREF(__pyx_v_ldv);
    __Pyx_GIVEREF(__pyx_v_ldv);
    if (__Pyx_PyTuple_SET_ITEM(__pyx_t_9, 0, __pyx_v_ldv) != (0)) __PYX_ERR(0, 193, __pyx_L1_error);
    __Pyx_INCREF(__pyx_v_mn);
    __Pyx_GIVEREF(__pyx_v_mn);
    if (__Pyx_PyTuple_SET_ITEM(__pyx_t_9, 1, __pyx_v_mn) != (0)) __PYX_ERR(0, 193, __pyx_L1_error);
    __pyx_t_7 = __Pyx_PyObject_GetAttrStr(__pyx_v_a, __pyx_mstate_global->__pyx_n_u_dtype); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 193, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_7);
    __pyx_t_5 = 1;
    #if CYTHON_UNPACK_METHODS
    if (unlikely(PyMethod_Check(__pyx_t_10))) {
      __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_10);
      assert(__pyx_t_3);
      PyObject* __pyx__function = PyMethod_GET_FUNCTION(__pyx_t_10);
      __Pyx_INCREF(__pyx_t_3);
      __Pyx_INCREF(__pyx__function);
      __Pyx_DECREF_SET(__pyx_t_10, __pyx__function);
      __pyx_t_5 = 0;
    }
    #endif
    {
      PyObject *__pyx_callargs[2 + ((CYTHON_VECTORCALL) ? 2 : 0)] = {__pyx_t_3, __pyx_t_9};
      __pyx_t_4 = __Pyx_MakeVectorcallBuilderKwds(2); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 193, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_4);
      if (__Pyx_VectorcallBuilder_AddArg(__pyx_mstate_global->__pyx_n_u_dtype, __pyx_t_7, __pyx_t_4, __pyx_callargs+2, 0) < (0)) __PYX_ERR(0, 193, __pyx_L1_error)
      if (__Pyx_VectorcallBuilder_AddArg(__pyx_mstate_global->__pyx_n_u_order, __pyx_mstate_global->__pyx_n_u_F, __pyx_t_4, __pyx_callargs+2, 1) < (0)) __PYX_ERR(0, 193, __pyx_L1_error)
      __pyx_t_1 = __Pyx_Object_Vectorcall_CallFromBuilder((PyObject*)__pyx_t_10, __pyx_callargs+__pyx_t_5, (2-__pyx_t_5) | (__pyx_t_5*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET), __pyx_t_4);
      __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
      __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
      __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
      if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 193, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
    }
    __pyx_v_v = __pyx_t_1;
    __pyx_t_1 = 0;
  }
  __pyx_L9:;

  /* "cupyx/cusolver.pyx":194
 *         u = _cupy.empty((ldu, mn), dtype=a.dtype, order='F')
 *         v = _cupy.empty((ldv, mn), dtype=a.dtype, order='F')
 *     params = _cusolver.createGesvdjInfo()             # <<<<<<<<<<<<<<
 *     lwork = helper(handle, jobz, econ, m, n, a.data.ptr, lda, s.data.ptr,
 *                    u.data.ptr, ldu, v.data.ptr, ldv, params)
*/
  __pyx_t_10 = NULL;
  __Pyx_GetModuleGlobalName(__pyx_t_4, __pyx_mstate_global->__pyx_n_u_cusolver); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 194, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_t_7 = __Pyx_PyObject_GetAttrStr(__pyx_t_4, __pyx_mstate_global->__pyx_n_u_createGesvdjInfo); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 194, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_7);
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __pyx_t_5 = 1;
  #if CYTHON_UNPACK_METHODS
  if (unlikely(PyMethod_Check(__pyx_t_7))) {
    __pyx_t_10 = PyMethod_GET_SELF(__pyx_t_7);
    assert(__pyx_t_10);
    PyObject* __pyx__function = PyMethod_GET_FUNCTION(__pyx_t_7);
    __Pyx_INCREF(__pyx_t_10);
    __Pyx_INCREF(__pyx__function);
    __Pyx_DECREF_SET(__pyx_t_7, __pyx__function);
    __pyx_t_5 = 0;
  }
  #endif
  {
    PyObject *__pyx_callargs[2] = {__pyx_t_10, NULL};
    __pyx_t_1 = __Pyx_PyObject_FastCall((PyObject*)__pyx_t_7, __pyx_callargs+__pyx_t_5, (1-__pyx_t_5) | (__pyx_t_5*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
    __Pyx_XDECREF(__pyx_t_10); __pyx_t_10 = 0;
    __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
    if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 194, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
  }
  __pyx_v_params = __pyx_t_1;
  __pyx_t_1 = 0;

  /* "cupyx/cusolver.pyx":195
 *         v = _cupy.empty((ldv, mn), dtype=a.dtype, order='F')
 *     params = _cusolver.createGesvdjInfo()
 *     lwork = helper(handle, jobz, econ, m, n, a.data.ptr, lda, s.data.ptr,             # <<<<<<<<<<<<<<
 *                    u.data.ptr, ldu, v.data.ptr, ldv, params)
 *     work = _cupy.empty(lwork, dtype=a.dtype)
*/
  __pyx_t_7 = NULL;
  __Pyx_INCREF(__pyx_v_helper);
  __pyx_t_10 = __pyx_v_helper; 
  __pyx_t_4 = __Pyx_PyLong_From_long(__pyx_v_econ); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 195, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_t_9 = __Pyx_PyObject_GetAttrStr(__pyx_v_a, __pyx_mstate_global->__pyx_n_u_data); if (unlikely(!__pyx_t_9)) __PYX_ERR(0, 195, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_9);
  __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_t_9, __pyx_mstate_global->__pyx_n_u_ptr); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 195, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
  __pyx_t_9 = __Pyx_PyObject_GetAttrStr(__pyx_v_s, __pyx_mstate_global->__pyx_n_u_data); if (unlikely(!__pyx_t_9)) __PYX_ERR(0, 195, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_9);
  __pyx_t_11 = __Pyx_PyObject_GetAttrStr(__pyx_t_9, __pyx_mstate_global->__pyx_n_u_ptr); if (unlikely(!__pyx_t_11)) __PYX_ERR(0, 195, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_11);
  __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;

  /* "cupyx/cusolver.pyx":196
 *     params = _cusolver.createGesvdjInfo()
 *     lwork = helper(handle, jobz, econ, m, n, a.data.ptr, lda, s.data.ptr,
 *                    u.data.ptr, ldu, v.data.ptr, ldv, params)             # <<<<<<<<<<<<<<
 *     work = _cupy.empty(lwork, dtype=a.dtype)
 *     info = _cupy.empty(1, dtype=_numpy.int32)
*/
  __pyx_t_9 = __Pyx_PyObject_GetAttrStr(__pyx_v_u, __pyx_mstate_global->__pyx_n_u_data); if (unlikely(!__pyx_t_9)) __PYX_ERR(0, 196, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_9);
  __pyx_t_12 = __Pyx_PyObject_GetAttrStr(__pyx_t_9, __pyx_mstate_global->__pyx_n_u_ptr); if (unlikely(!__pyx_t_12)) __PYX_ERR(0, 196, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_12);
  __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
  __pyx_t_9 = __Pyx_PyObject_GetAttrStr(__pyx_v_v, __pyx_mstate_global->__pyx_n_u_data); if (unlikely(!__pyx_t_9)) __PYX_ERR(0, 196, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_9);
  __pyx_t_13 = __Pyx_PyObject_GetAttrStr(__pyx_t_9, __pyx_mstate_global->__pyx_n_u_ptr); if (unlikely(!__pyx_t_13)) __PYX_ERR(0, 196, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_13);
  __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
  __pyx_t_5 = 1;
  #if CYTHON_UNPACK_METHODS
  if (unlikely(PyMethod_Check(__pyx_t_10))) {
    __pyx_t_7 = PyMethod_GET_SELF(__pyx_t_10);
    assert(__pyx_t_7);
    PyObject* __pyx__function = PyMethod_GET_FUNCTION(__pyx_t_10);
    __Pyx_INCREF(__pyx_t_7);
    __Pyx_INCREF(__pyx__function);
    __Pyx_DECREF_SET(__pyx_t_10, __pyx__function);
    __pyx_t_5 = 0;
  }
  #endif
  {
    PyObject *__pyx_callargs[14] = {__pyx_t_7, __pyx_v_handle, __pyx_v_jobz, __pyx_t_4, __pyx_v_m, __pyx_v_n, __pyx_t_3, __pyx_v_lda, __pyx_t_11, __pyx_t_12, __pyx_v_ldu, __pyx_t_13, __pyx_v_ldv, __pyx_v_params};
    __pyx_t_1 = __Pyx_PyObject_FastCall((PyObject*)__pyx_t_10, __pyx_callargs+__pyx_t_5, (14-__pyx_t_5) | (__pyx_t_5*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
    __Pyx_XDECREF(__pyx_t_7); __pyx_t_7 = 0;
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __Pyx_DECREF(__pyx_t_11); __pyx_t_11 = 0;
    __Pyx_DECREF(__pyx_t_12); __pyx_t_12 = 0;
    __Pyx_DECREF(__pyx_t_13); __pyx_t_13 = 0;
    __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
    if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 195, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
  }
  __pyx_v_lwork = __pyx_t_1;
  __pyx_t_1 = 0;

  /* "cupyx/cusolver.pyx":197
 *     lwork = helper(handle, jobz, econ, m, n, a.data.ptr, lda, s.data.ptr,
 *                    u.data.ptr, ldu, v.data.ptr, ldv, params)
 *     work = _cupy.empty(lwork, dtype=a.dtype)             # <<<<<<<<<<<<<<
 *     info = _cupy.empty(1, dtype=_numpy.int32)
 *     solver(handle, jobz, econ, m, n, a.data.ptr, lda, s.data.ptr,
*/
  __pyx_t_10 = NULL;
  __Pyx_GetModuleGlobalName(__pyx_t_13, __pyx_mstate_global->__pyx_n_u_cupy); if (unlikely(!__pyx_t_13)) __PYX_ERR(0, 197, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_13);
  __pyx_t_12 = __Pyx_PyObject_GetAttrStr(__pyx_t_13, __pyx_mstate_global->__pyx_n_u_empty); if (unlikely(!__pyx_t_12)) __PYX_ERR(0, 197, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_12);
  __Pyx_DECREF(__pyx_t_13); __pyx_t_13 = 0;
  __pyx_t_13 = __Pyx_PyObject_GetAttrStr(__pyx_v_a, __pyx_mstate_global->__pyx_n_u_dtype); if (unlikely(!__pyx_t_13)) __PYX_ERR(0, 197, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_13);
  __pyx_t_5 = 1;
  #if CYTHON_UNPACK_METHODS
  if (unlikely(PyMethod_Check(__pyx_t_12))) {
    __pyx_t_10 = PyMethod_GET_SELF(__pyx_t_12);
    assert(__pyx_t_10);
    PyObject* __pyx__function = PyMethod_GET_FUNCTION(__pyx_t_12);
    __Pyx_INCREF(__pyx_t_10);
    __Pyx_INCREF(__pyx__function);
    __Pyx_DECREF_SET(__pyx_t_12, __pyx__function);
    __pyx_t_5 = 0;
  }
  #endif
  {
    PyObject *__pyx_callargs[2 + ((CYTHON_VECTORCALL) ? 1 : 0)] = {__pyx_t_10, __pyx_v_lwork};
    __pyx_t_11 = __Pyx_MakeVectorcallBuilderKwds(1); if (unlikely(!__pyx_t_11)) __PYX_ERR(0, 197, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_11);
    if (__Pyx_VectorcallBuilder_AddArg(__pyx_mstate_global->__pyx_n_u_dtype, __pyx_t_13, __pyx_t_11, __pyx_callargs+2, 0) < (0)) __PYX_ERR(0, 197, __pyx_L1_error)
    __pyx_t_1 = __Pyx_Object_Vectorcall_CallFromBuilder((PyObject*)__pyx_t_12, __pyx_callargs+__pyx_t_5, (2-__pyx_t_5) | (__pyx_t_5*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET), __pyx_t_11);
    __Pyx_XDECREF(__pyx_t_10); __pyx_t_10 = 0;
    __Pyx_DECREF(__pyx_t_13); __pyx_t_13 = 0;
    __Pyx_DECREF(__pyx_t_11); __pyx_t_11 = 0;
    __Pyx_DECREF(__pyx_t_12); __pyx_t_12 = 0;
    if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 197, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
  }
  __pyx_v_work = __pyx_t_1;
  __pyx_t_1 = 0;

  /* "cupyx/cusolver.pyx":198
 *                    u.data.ptr, ldu, v.data.ptr, ldv, params)
 *     work = _cupy.empty(lwork, dtype=a.dtype)
 *     info = _cupy.empty(1, dtype=_numpy.int32)             # <<<<<<<<<<<<<<
 *     solver(handle, jobz, econ, m, n, a.data.ptr, lda, s.data.ptr,
 *            u.data.ptr, ldu, v.data.ptr, ldv, work.data.ptr, lwork,
*/
  __pyx_t_12 = NULL;
  __Pyx_GetModuleGlobalName(__pyx_t_11, __pyx_mstate_global->__pyx_n_u_cupy); if (unlikely(!__pyx_t_11)) __PYX_ERR(0, 198, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_11);
  __pyx_t_13 = __Pyx_PyObject_GetAttrStr(__pyx_t_11, __pyx_mstate_global->__pyx_n_u_empty); if (unlikely(!__pyx_t_13)) __PYX_ERR(0, 198, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_13);
  __Pyx_DECREF(__pyx_t_11); __pyx_t_11 = 0;
  __Pyx_GetModuleGlobalName(__pyx_t_11, __pyx_mstate_global->__pyx_n_u_numpy); if (unlikely(!__pyx_t_11)) __PYX_ERR(0, 198, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_11);
  __pyx_t_10 = __Pyx_PyObject_GetAttrStr(__pyx_t_11, __pyx_mstate_global->__pyx_n_u_int32); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 198, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_10);
  __Pyx_DECREF(__pyx_t_11); __pyx_t_11 = 0;
  __pyx_t_5 = 1;
  #if CYTHON_UNPACK_METHODS
  if (unlikely(PyMethod_Check(__pyx_t_13))) {
    __pyx_t_12 = PyMethod_GET_SELF(__pyx_t_13);
    assert(__pyx_t_12);
    PyObject* __pyx__function = PyMethod_GET_FUNCTION(__pyx_t_13);
    __Pyx_INCREF(__pyx_t_12);
    __Pyx_INCREF(__pyx__function);
    __Pyx_DECREF_SET(__pyx_t_13, __pyx__function);
    __pyx_t_5 = 0;
  }
  #endif
  {
    PyObject *__pyx_callargs[2 + ((CYTHON_VECTORCALL) ? 1 : 0)] = {__pyx_t_12, __pyx_mstate_global->__pyx_int_1};
    __pyx_t_11 = __Pyx_MakeVectorcallBuilderKwds(1); if (unlikely(!__pyx_t_11)) __PYX_ERR(0, 198, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_11);
    if (__Pyx_VectorcallBuilder_AddArg(__pyx_mstate_global->__pyx_n_u_dtype, __pyx_t_10, __pyx_t_11, __pyx_callargs+2, 0) < (0)) __PYX_ERR(0, 198, __pyx_L1_error)
    __pyx_t_1 = __Pyx_Object_Vectorcall_CallFromBuilder((PyObject*)__pyx_t_13, __pyx_callargs+__pyx_t_5, (2-__pyx_t_5) | (__pyx_t_5*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET), __pyx_t_11);
    __Pyx_XDECREF(__pyx_t_12); __pyx_t_12 = 0;
    __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
    __Pyx_DECREF(__pyx_t_11); __pyx_t_11 = 0;
    __Pyx_DECREF(__pyx_t_13); __pyx_t_13 = 0;
    if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 198, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
  }
  __pyx_v_info = __pyx_t_1;
  __pyx_t_1 = 0;

  /* "cupyx/cusolver.pyx":199
 *     work = _cupy.empty(lwork, dtype=a.dtype)
 *     info = _cupy.empty(1, dtype=_numpy.int32)
 *     solver(handle, jobz, econ, m, n, a.data.ptr, lda, s.data.ptr,             # <<<<<<<<<<<<<<
 *            u.data.ptr, ldu, v.data.ptr, ldv, work.data.ptr, lwork,
 *            info.data.ptr, params)
*/
  __pyx_t_13 = NULL;
  __Pyx_INCREF(__pyx_v_solver);
  __pyx_t_11 = __pyx_v_solver; 
  __pyx_t_10 = __Pyx_PyLong_From_long(__pyx_v_econ); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 199, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_10);
  __pyx_t_12 = __Pyx_PyObject_GetAttrStr(__pyx_v_a, __pyx_mstate_global->__pyx_n_u_data); if (unlikely(!__pyx_t_12)) __PYX_ERR(0, 199, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_12);
  __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_t_12, __pyx_mstate_global->__pyx_n_u_ptr); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 199, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_12); __pyx_t_12 = 0;
  __pyx_t_12 = __Pyx_PyObject_GetAttrStr(__pyx_v_s, __pyx_mstate_global->__pyx_n_u_data); if (unlikely(!__pyx_t_12)) __PYX_ERR(0, 199, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_12);
  __pyx_t_4 = __Pyx_PyObject_GetAttrStr(__pyx_t_12, __pyx_mstate_global->__pyx_n_u_ptr); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 199, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_DECREF(__pyx_t_12); __pyx_t_12 = 0;

  /* "cupyx/cusolver.pyx":200
 *     info = _cupy.empty(1, dtype=_numpy.int32)
 *     solver(handle, jobz, econ, m, n, a.data.ptr, lda, s.data.ptr,
 *            u.data.ptr, ldu, v.data.ptr, ldv, work.data.ptr, lwork,             # <<<<<<<<<<<<<<
 *            info.data.ptr, params)
 *     _cupy.linalg._util._check_cusolver_dev_info_if_synchronization_allowed(
*/
  __pyx_t_12 = __Pyx_PyObject_GetAttrStr(__pyx_v_u, __pyx_mstate_global->__pyx_n_u_data); if (unlikely(!__pyx_t_12)) __PYX_ERR(0, 200, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_12);
  __pyx_t_7 = __Pyx_PyObject_GetAttrStr(__pyx_t_12, __pyx_mstate_global->__pyx_n_u_ptr); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 200, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_7);
  __Pyx_DECREF(__pyx_t_12); __pyx_t_12 = 0;
  __pyx_t_12 = __Pyx_PyObject_GetAttrStr(__pyx_v_v, __pyx_mstate_global->__pyx_n_u_data); if (unlikely(!__pyx_t_12)) __PYX_ERR(0, 200, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_12);
  __pyx_t_9 = __Pyx_PyObject_GetAttrStr(__pyx_t_12, __pyx_mstate_global->__pyx_n_u_ptr); if (unlikely(!__pyx_t_9)) __PYX_ERR(0, 200, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_9);
  __Pyx_DECREF(__pyx_t_12); __pyx_t_12 = 0;
  __pyx_t_12 = __Pyx_PyObject_GetAttrStr(__pyx_v_work, __pyx_mstate_global->__pyx_n_u_data); if (unlikely(!__pyx_t_12)) __PYX_ERR(0, 200, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_12);
  __pyx_t_14 = __Pyx_PyObject_GetAttrStr(__pyx_t_12, __pyx_mstate_global->__pyx_n_u_ptr); if (unlikely(!__pyx_t_14)) __PYX_ERR(0, 200, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_14);
  __Pyx_DECREF(__pyx_t_12); __pyx_t_12 = 0;

  /* "cupyx/cusolver.pyx":201
 *     solver(handle, jobz, econ, m, n, a.data.ptr, lda, s.data.ptr,
 *            u.data.ptr, ldu, v.data.ptr, ldv, work.data.ptr, lwork,
 *            info.data.ptr, params)             # <<<<<<<<<<<<<<
 *     _cupy.linalg._util._check_cusolver_dev_info_if_synchronization_allowed(
 *         gesvdj, info)
*/
  __pyx_t_12 = __Pyx_PyObject_GetAttrStr(__pyx_v_info, __pyx_mstate_global->__pyx_n_u_data); if (unlikely(!__pyx_t_12)) __PYX_ERR(0, 201, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_12);
  __pyx_t_15 = __Pyx_PyObject_GetAttrStr(__pyx_t_12, __pyx_mstate_global->__pyx_n_u_ptr); if (unlikely(!__pyx_t_15)) __PYX_ERR(0, 201, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_15);
  __Pyx_DECREF(__pyx_t_12); __pyx_t_12 = 0;
  __pyx_t_5 = 1;
  #if CYTHON_UNPACK_METHODS
  if (unlikely(PyMethod_Check(__pyx_t_11))) {
    __pyx_t_13 = PyMethod_GET_SELF(__pyx_t_11);
    assert(__pyx_t_13);
    PyObject* __pyx__function = PyMethod_GET_FUNCTION(__pyx_t_11);
    __Pyx_INCREF(__pyx_t_13);
    __Pyx_INCREF(__pyx__function);
    __Pyx_DECREF_SET(__pyx_t_11, __pyx__function);
    __pyx_t_5 = 0;
  }
  #endif
  {
    PyObject *__pyx_callargs[17] = {__pyx_t_13, __pyx_v_handle, __pyx_v_jobz, __pyx_t_10, __pyx_v_m, __pyx_v_n, __pyx_t_3, __pyx_v_lda, __pyx_t_4, __pyx_t_7, __pyx_v_ldu, __pyx_t_9, __pyx_v_ldv, __pyx_t_14, __pyx_v_lwork, __pyx_t_15, __pyx_v_params};
    __pyx_t_1 = __Pyx_PyObject_FastCall((PyObject*)__pyx_t_11, __pyx_callargs+__pyx_t_5, (17-__pyx_t_5) | (__pyx_t_5*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
    __Pyx_XDECREF(__pyx_t_13); __pyx_t_13 = 0;
    __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
    __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
    __Pyx_DECREF(__pyx_t_14); __pyx_t_14 = 0;
    __Pyx_DECREF(__pyx_t_15); __pyx_t_15 = 0;
    __Pyx_DECREF(__pyx_t_11); __pyx_t_11 = 0;
    if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 199, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
  }
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "cupyx/cusolver.pyx":202
 *            u.data.ptr, ldu, v.data.ptr, ldv, work.data.ptr, lwork,
 *            info.data.ptr, params)
 *     _cupy.linalg._util._check_cusolver_dev_info_if_synchronization_allowed(             # <<<<<<<<<<<<<<
 *         gesvdj, info)
 * 
*/
  __Pyx_GetModuleGlobalName(__pyx_t_15, __pyx_mstate_global->__pyx_n_u_cupy); if (unlikely(!__pyx_t_15)) __PYX_ERR(0, 202, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_15);
  __pyx_t_14 = __Pyx_PyObject_GetAttrStr(__pyx_t_15, __pyx_mstate_global->__pyx_n_u_linalg); if (unlikely(!__pyx_t_14)) __PYX_ERR(0, 202, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_14);
  __Pyx_DECREF(__pyx_t_15); __pyx_t_15 = 0;
  __pyx_t_15 = __Pyx_PyObject_GetAttrStr(__pyx_t_14, __pyx_mstate_global->__pyx_n_u_util); if (unlikely(!__pyx_t_15)) __PYX_ERR(0, 202, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_15);
  __Pyx_DECREF(__pyx_t_14); __pyx_t_14 = 0;
  __pyx_t_11 = __pyx_t_15;
  __Pyx_INCREF(__pyx_t_11);

  /* "cupyx/cusolver.pyx":203
 *            info.data.ptr, params)
 *     _cupy.linalg._util._check_cusolver_dev_info_if_synchronization_allowed(
 *         gesvdj, info)             # <<<<<<<<<<<<<<
 * 
 *     _cusolver.destroyGesvdjInfo(params)
*/
  __Pyx_GetModuleGlobalName(__pyx_t_14, __pyx_mstate_global->__pyx_n_u_gesvdj); if (unlikely(!__pyx_t_14)) __PYX_ERR(0, 203, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_14);
  __pyx_t_5 = 0;
  {
    PyObject *__pyx_callargs[3] = {__pyx_t_11, __pyx_t_14, __pyx_v_info};
    __pyx_t_1 = __Pyx_PyObject_FastCallMethod((PyObject*)__pyx_mstate_global->__pyx_n_u_check_cusolver_dev_info_if_sync, __pyx_callargs+__pyx_t_5, (3-__pyx_t_5) | (1*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
    __Pyx_XDECREF(__pyx_t_11); __pyx_t_11 = 0;
    __Pyx_DECREF(__pyx_t_14); __pyx_t_14 = 0;
    __Pyx_DECREF(__pyx_t_15); __pyx_t_15 = 0;
    if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 202, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
  }
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "cupyx/cusolver.pyx":205
 *         gesvdj, info)
 * 
 *     _cusolver.destroyGesvdjInfo(params)             # <<<<<<<<<<<<<<
 *     if compute_uv:
 *         return u, s, v
*/
  __pyx_t_15 = NULL;
  __Pyx_GetModuleGlobalName(__pyx_t_14, __pyx_mstate_global->__pyx_n_u_cusolver); if (unlikely(!__pyx_t_14)) __PYX_ERR(0, 205, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_14);
  __pyx_t_11 = __Pyx_PyObject_GetAttrStr(__pyx_t_14, __pyx_mstate_global->__pyx_n_u_destroyGesvdjInfo); if (unlikely(!__pyx_t_11)) __PYX_ERR(0, 205, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_11);
  __Pyx_DECREF(__pyx_t_14); __pyx_t_14 = 0;
  __pyx_t_5 = 1;
  #if CYTHON_UNPACK_METHODS
  if (unlikely(PyMethod_Check(__pyx_t_11))) {
    __pyx_t_15 = PyMethod_GET_SELF(__pyx_t_11);
    assert(__pyx_t_15);
    PyObject* __pyx__function = PyMethod_GET_FUNCTION(__pyx_t_11);
    __Pyx_INCREF(__pyx_t_15);
    __Pyx_INCREF(__pyx__function);
    __Pyx_DECREF_SET(__pyx_t_11, __pyx__function);
    __pyx_t_5 = 0;
  }
  #endif
  {
    PyObject *__pyx_callargs[2] = {__pyx_t_15, __pyx_v_params};
    __pyx_t_1 = __Pyx_PyObject_FastCall((PyObject*)__pyx_t_11, __pyx_callargs+__pyx_t_5, (2-__pyx_t_5) | (__pyx_t_5*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
    __Pyx_XDECREF(__pyx_t_15); __pyx_t_15 = 0;
    __Pyx_DECREF(__pyx_t_11); __pyx_t_11 = 0;
    if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 205, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
  }
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "cupyx/cusolver.pyx":206
 * 
 *     _cusolver.destroyGesvdjInfo(params)
 *     if compute_uv:             # <<<<<<<<<<<<<<
 *         return u, s, v
 *     else:
*/
  __pyx_t_6 = __Pyx_PyObject_IsTrue(__pyx_v_compute_uv); if (unlikely((__pyx_t_6 < 0))) __PYX_ERR(0, 206, __pyx_L1_error)
  if (__pyx_t_6) {

    /* "cupyx/cusolver.pyx":207
 *     _cusolver.destroyGesvdjInfo(params)
 *     if compute_uv:
 *         return u, s, v             # <<<<<<<<<<<<<<
 *     else:
 *         return s
*/
    __Pyx_XDECREF(__pyx_r);
    __pyx_t_1 = PyTuple_New(3); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 207, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_INCREF(__pyx_v_u);
    __Pyx_GIVEREF(__pyx_v_u);
    if (__Pyx_PyTuple_SET_ITEM(__pyx_t_1, 0, __pyx_v_u) != (0)) __PYX_ERR(0, 207, __pyx_L1_error);
    __Pyx_INCREF(__pyx_v_s);
    __Pyx_GIVEREF(__pyx_v_s);
    if (__Pyx_PyTuple_SET_ITEM(__pyx_t_1, 1, __pyx_v_s) != (0)) __PYX_ERR(0, 207, __pyx_L1_error);
    __Pyx_INCREF(__pyx_v_v);
    __Pyx_GIVEREF(__pyx_v_v);
    if (__Pyx_PyTuple_SET_ITEM(__pyx_t_1, 2, __pyx_v_v) != (0)) __PYX_ERR(0, 207, __pyx_L1_error);
    __pyx_r = __pyx_t_1;
    __pyx_t_1 = 0;
    goto __pyx_L0;

    /* "cupyx/cusolver.pyx":206
 * 
 *     _cusolver.destroyGesvdjInfo(params)
 *     if compute_uv:             # <<<<<<<<<<<<<<
 *         return u, s, v
 *     else:
*/
  }

  /* "cupyx/cusolver.pyx":209
 *         return u, s, v
 *     else:
 *         return s             # <<<<<<<<<<<<<<
 * 
 * 
*/
  /*else*/ {
    __Pyx_XDECREF(__pyx_r);
    __Pyx_INCREF(__pyx_v_s);
    __pyx_r = __pyx_v_s;
    goto __pyx_L0;
  }

  /* "cupyx/cusolver.pyx":127
 * 
 * 
 * def gesvdj(a, full_matrices=True, compute_uv=True, overwrite_a=False):             # <<<<<<<<<<<<<<
 *     """Singular value decomposition using cusolverDn<t>gesvdj().
 * 
*/

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_7);
  __Pyx_XDECREF(__pyx_t_9);
  __Pyx_XDECREF(__pyx_t_10);
  __Pyx_XDECREF(__pyx_t_11);
  __Pyx_XDECREF(__pyx_t_12);
  __Pyx_XDECREF(__pyx_t_13);
  __Pyx_XDECREF(__pyx_t_14);
  __Pyx_XDECREF(__pyx_t_15);
  __Pyx_AddTraceback("cupyx.cusolver.gesvdj", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_helper);
  __Pyx_XDECREF(__pyx_v_solver);
  __Pyx_XDECREF(__pyx_v_s_dtype);
  __Pyx_XDECREF(__pyx_v_handle);
  __Pyx_XDECREF(__pyx_v_m);
  __Pyx_XDECREF(__pyx_v_n);
  __Pyx_XDECREF(__pyx_v_lda);
  __Pyx_XDECREF(__pyx_v_mn);
  __Pyx_XDECREF(__pyx_v_s);
  __Pyx_XDECREF(__pyx_v_ldu);
  __Pyx_XDECREF(__pyx_v_ldv);
  __Pyx_XDECREF(__pyx_v_jobz);
  __Pyx_XDECREF(__pyx_v_u);
  __Pyx_XDECREF(__pyx_v_v);
  __Pyx_XDECREF(__pyx_v_params);
  __Pyx_XDECREF(__pyx_v_lwork);
  __Pyx_XDECREF(__pyx_v_work);
  __Pyx_XDECREF(__pyx_v_info);
  __Pyx_XDECREF(__pyx_v_a);
  __Pyx_XDECREF(__pyx_v_full_matrices);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "cupyx/cusolver.pyx":212
 * 
 * 
 * cpdef _gesvdj_batched(a, full_matrices, compute_uv, overwrite_a):             # <<<<<<<<<<<<<<
 *     if not check_availability('gesvdjBatched'):
 *         raise RuntimeError('gesvdj is not available.')
*/

static PyObject *__pyx_pw_5cupyx_8cusolver_5_gesvdj_batched(PyObject *__pyx_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
static PyObject *__pyx_f_5cupyx_8cusolver__gesvdj_batched(PyObject *__pyx_v_a, PyObject *__pyx_v_full_matrices, PyObject *__pyx_v_compute_uv, PyObject *__pyx_v_overwrite_a, CYTHON_UNUSED int __pyx_skip_dispatch) {
  PyObject *__pyx_v_helper = NULL;
  PyObject *__pyx_v_solver = NULL;
  PyObject *__pyx_v_s_dtype = NULL;
  PyObject *__pyx_v_handle = NULL;
  PyObject *__pyx_v_batch_size = NULL;
  PyObject *__pyx_v_m = NULL;
  PyObject *__pyx_v_n = NULL;
  PyObject *__pyx_v_ap = NULL;
  PyObject *__pyx_v_lda = NULL;
  PyObject *__pyx_v_mn = NULL;
  PyObject *__pyx_v_s = NULL;
  PyObject *__pyx_v_ldu = NULL;
  PyObject *__pyx_v_ldv = NULL;
  PyObject *__pyx_v_jobz = NULL;
  PyObject *__pyx_v_u = NULL;
  PyObject *__pyx_v_v = NULL;
  PyObject *__pyx_v_params = NULL;
  PyObject *__pyx_v_lwork = NULL;
  PyObject *__pyx_v_work = NULL;
  PyObject *__pyx_v_info = NULL;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  size_t __pyx_t_4;
  int __pyx_t_5;
  int __pyx_t_6;
  PyObject *__pyx_t_7 = NULL;
  PyObject *__pyx_t_8 = NULL;
  PyObject *(*__pyx_t_9)(PyObject *);
  PyObject *__pyx_t_10 = NULL;
  PyObject *__pyx_t_11 = NULL;
  PyObject *__pyx_t_12 = NULL;
  PyObject *__pyx_t_13 = NULL;
  PyObject *__pyx_t_14 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("_gesvdj_batched", 0);
  __Pyx_INCREF(__pyx_v_a);

  /* "cupyx/cusolver.pyx":213
 * 
 * cpdef _gesvdj_batched(a, full_matrices, compute_uv, overwrite_a):
 *     if not check_availability('gesvdjBatched'):             # <<<<<<<<<<<<<<
 *         raise RuntimeError('gesvdj is not available.')
 * 
*/
  __pyx_t_2 = NULL;
  __Pyx_GetModuleGlobalName(__pyx_t_3, __pyx_mstate_global->__pyx_n_u_check_availability); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 213, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_4 = 1;
  #if CYTHON_UNPACK_METHODS
  if (unlikely(PyMethod_Check(__pyx_t_3))) {
    __pyx_t_2 = PyMethod_GET_SELF(__pyx_t_3);
    assert(__pyx_t_2);
    PyObject* __pyx__function = PyMethod_GET_FUNCTION(__pyx_t_3);
    __Pyx_INCREF(__pyx_t_2);
    __Pyx_INCREF(__pyx__function);
    __Pyx_DECREF_SET(__pyx_t_3, __pyx__function);
    __pyx_t_4 = 0;
  }
  #endif
  {
    PyObject *__pyx_callargs[2] = {__pyx_t_2, __pyx_mstate_global->__pyx_n_u_gesvdjBatched};
    __pyx_t_1 = __Pyx_PyObject_FastCall((PyObject*)__pyx_t_3, __pyx_callargs+__pyx_t_4, (2-__pyx_t_4) | (__pyx_t_4*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
    __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 213, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
  }
  __pyx_t_5 = __Pyx_PyObject_IsTrue(__pyx_t_1); if (unlikely((__pyx_t_5 < 0))) __PYX_ERR(0, 213, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_6 = (!__pyx_t_5);
  if (unlikely(__pyx_t_6)) {

    /* "cupyx/cusolver.pyx":214
 * cpdef _gesvdj_batched(a, full_matrices, compute_uv, overwrite_a):
 *     if not check_availability('gesvdjBatched'):
 *         raise RuntimeError('gesvdj is not available.')             # <<<<<<<<<<<<<<
 * 
 *     if a.dtype == 'f':
*/
    __pyx_t_3 = NULL;
    __pyx_t_4 = 1;
    {
      PyObject *__pyx_callargs[2] = {__pyx_t_3, __pyx_mstate_global->__pyx_kp_u_gesvdj_is_not_available};
      __pyx_t_1 = __Pyx_PyObject_FastCall((PyObject*)(((PyTypeObject*)PyExc_RuntimeError)), __pyx_callargs+__pyx_t_4, (2-__pyx_t_4) | (__pyx_t_4*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
      __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
      if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 214, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
    }
    __Pyx_Raise(__pyx_t_1, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __PYX_ERR(0, 214, __pyx_L1_error)

    /* "cupyx/cusolver.pyx":213
 * 
 * cpdef _gesvdj_batched(a, full_matrices, compute_uv, overwrite_a):
 *     if not check_availability('gesvdjBatched'):             # <<<<<<<<<<<<<<
 *         raise RuntimeError('gesvdj is not available.')
 * 
*/
  }

  /* "cupyx/cusolver.pyx":216
 *         raise RuntimeError('gesvdj is not available.')
 * 
 *     if a.dtype == 'f':             # <<<<<<<<<<<<<<
 *         helper = _cusolver.sgesvdjBatched_bufferSize
 *         solver = _cusolver.sgesvdjBatched
*/
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_v_a, __pyx_mstate_global->__pyx_n_u_dtype); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 216, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_6 = (__Pyx_PyUnicode_Equals(__pyx_t_1, __pyx_mstate_global->__pyx_n_u_f, Py_EQ)); if (unlikely((__pyx_t_6 < 0))) __PYX_ERR(0, 216, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  if (__pyx_t_6) {

    /* "cupyx/cusolver.pyx":217
 * 
 *     if a.dtype == 'f':
 *         helper = _cusolver.sgesvdjBatched_bufferSize             # <<<<<<<<<<<<<<
 *         solver = _cusolver.sgesvdjBatched
 *         s_dtype = 'f'
*/
    __Pyx_GetModuleGlobalName(__pyx_t_1, __pyx_mstate_global->__pyx_n_u_cusolver); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 217, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_t_1, __pyx_mstate_global->__pyx_n_u_sgesvdjBatched_bufferSize); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 217, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __pyx_v_helper = __pyx_t_3;
    __pyx_t_3 = 0;

    /* "cupyx/cusolver.pyx":218
 *     if a.dtype == 'f':
 *         helper = _cusolver.sgesvdjBatched_bufferSize
 *         solver = _cusolver.sgesvdjBatched             # <<<<<<<<<<<<<<
 *         s_dtype = 'f'
 *     elif a.dtype == 'd':
*/
    __Pyx_GetModuleGlobalName(__pyx_t_3, __pyx_mstate_global->__pyx_n_u_cusolver); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 218, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_t_3, __pyx_mstate_global->__pyx_n_u_sgesvdjBatched); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 218, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __pyx_v_solver = __pyx_t_1;
    __pyx_t_1 = 0;

    /* "cupyx/cusolver.pyx":219
 *         helper = _cusolver.sgesvdjBatched_bufferSize
 *         solver = _cusolver.sgesvdjBatched
 *         s_dtype = 'f'             # <<<<<<<<<<<<<<
 *     elif a.dtype == 'd':
 *         helper = _cusolver.dgesvdjBatched_bufferSize
*/
    __Pyx_INCREF(__pyx_mstate_global->__pyx_n_u_f);
    __pyx_v_s_dtype = __pyx_mstate_global->__pyx_n_u_f;

    /* "cupyx/cusolver.pyx":216
 *         raise RuntimeError('gesvdj is not available.')
 * 
 *     if a.dtype == 'f':             # <<<<<<<<<<<<<<
 *         helper = _cusolver.sgesvdjBatched_bufferSize
 *         solver = _cusolver.sgesvdjBatched
*/
    goto __pyx_L4;
  }

  /* "cupyx/cusolver.pyx":220
 *         solver = _cusolver.sgesvdjBatched
 *         s_dtype = 'f'
 *     elif a.dtype == 'd':             # <<<<<<<<<<<<<<
 *         helper = _cusolver.dgesvdjBatched_bufferSize
 *         solver = _cusolver.dgesvdjBatched
*/
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_v_a, __pyx_mstate_global->__pyx_n_u_dtype); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 220, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_6 = (__Pyx_PyUnicode_Equals(__pyx_t_1, __pyx_mstate_global->__pyx_n_u_d, Py_EQ)); if (unlikely((__pyx_t_6 < 0))) __PYX_ERR(0, 220, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  if (__pyx_t_6) {

    /* "cupyx/cusolver.pyx":221
 *         s_dtype = 'f'
 *     elif a.dtype == 'd':
 *         helper = _cusolver.dgesvdjBatched_bufferSize             # <<<<<<<<<<<<<<
 *         solver = _cusolver.dgesvdjBatched
 *         s_dtype = 'd'
*/
    __Pyx_GetModuleGlobalName(__pyx_t_1, __pyx_mstate_global->__pyx_n_u_cusolver); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 221, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_t_1, __pyx_mstate_global->__pyx_n_u_dgesvdjBatched_bufferSize); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 221, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __pyx_v_helper = __pyx_t_3;
    __pyx_t_3 = 0;

    /* "cupyx/cusolver.pyx":222
 *     elif a.dtype == 'd':
 *         helper = _cusolver.dgesvdjBatched_bufferSize
 *         solver = _cusolver.dgesvdjBatched             # <<<<<<<<<<<<<<
 *         s_dtype = 'd'
 *     elif a.dtype == 'F':
*/
    __Pyx_GetModuleGlobalName(__pyx_t_3, __pyx_mstate_global->__pyx_n_u_cusolver); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 222, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_t_3, __pyx_mstate_global->__pyx_n_u_dgesvdjBatched); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 222, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __pyx_v_solver = __pyx_t_1;
    __pyx_t_1 = 0;

    /* "cupyx/cusolver.pyx":223
 *         helper = _cusolver.dgesvdjBatched_bufferSize
 *         solver = _cusolver.dgesvdjBatched
 *         s_dtype = 'd'             # <<<<<<<<<<<<<<
 *     elif a.dtype == 'F':
 *         helper = _cusolver.cgesvdjBatched_bufferSize
*/
    __Pyx_INCREF(__pyx_mstate_global->__pyx_n_u_d);
    __pyx_v_s_dtype = __pyx_mstate_global->__pyx_n_u_d;

    /* "cupyx/cusolver.pyx":220
 *         solver = _cusolver.sgesvdjBatched
 *         s_dtype = 'f'
 *     elif a.dtype == 'd':             # <<<<<<<<<<<<<<
 *         helper = _cusolver.dgesvdjBatched_bufferSize
 *         solver = _cusolver.dgesvdjBatched
*/
    goto __pyx_L4;
  }

  /* "cupyx/cusolver.pyx":224
 *         solver = _cusolver.dgesvdjBatched
 *         s_dtype = 'd'
 *     elif a.dtype == 'F':             # <<<<<<<<<<<<<<
 *         helper = _cusolver.cgesvdjBatched_bufferSize
 *         solver = _cusolver.cgesvdjBatched
*/
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_v_a, __pyx_mstate_global->__pyx_n_u_dtype); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 224, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_6 = (__Pyx_PyUnicode_Equals(__pyx_t_1, __pyx_mstate_global->__pyx_n_u_F, Py_EQ)); if (unlikely((__pyx_t_6 < 0))) __PYX_ERR(0, 224, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  if (__pyx_t_6) {

    /* "cupyx/cusolver.pyx":225
 *         s_dtype = 'd'
 *     elif a.dtype == 'F':
 *         helper = _cusolver.cgesvdjBatched_bufferSize             # <<<<<<<<<<<<<<
 *         solver = _cusolver.cgesvdjBatched
 *         s_dtype = 'f'
*/
    __Pyx_GetModuleGlobalName(__pyx_t_1, __pyx_mstate_global->__pyx_n_u_cusolver); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 225, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_t_1, __pyx_mstate_global->__pyx_n_u_cgesvdjBatched_bufferSize); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 225, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __pyx_v_helper = __pyx_t_3;
    __pyx_t_3 = 0;

    /* "cupyx/cusolver.pyx":226
 *     elif a.dtype == 'F':
 *         helper = _cusolver.cgesvdjBatched_bufferSize
 *         solver = _cusolver.cgesvdjBatched             # <<<<<<<<<<<<<<
 *         s_dtype = 'f'
 *     elif a.dtype == 'D':
*/
    __Pyx_GetModuleGlobalName(__pyx_t_3, __pyx_mstate_global->__pyx_n_u_cusolver); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 226, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_t_3, __pyx_mstate_global->__pyx_n_u_cgesvdjBatched); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 226, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __pyx_v_solver = __pyx_t_1;
    __pyx_t_1 = 0;

    /* "cupyx/cusolver.pyx":227
 *         helper = _cusolver.cgesvdjBatched_bufferSize
 *         solver = _cusolver.cgesvdjBatched
 *         s_dtype = 'f'             # <<<<<<<<<<<<<<
 *     elif a.dtype == 'D':
 *         helper = _cusolver.zgesvdjBatched_bufferSize
*/
    __Pyx_INCREF(__pyx_mstate_global->__pyx_n_u_f);
    __pyx_v_s_dtype = __pyx_mstate_global->__pyx_n_u_f;

    /* "cupyx/cusolver.pyx":224
 *         solver = _cusolver.dgesvdjBatched
 *         s_dtype = 'd'
 *     elif a.dtype == 'F':             # <<<<<<<<<<<<<<
 *         helper = _cusolver.cgesvdjBatched_bufferSize
 *         solver = _cusolver.cgesvdjBatched
*/
    goto __pyx_L4;
  }

  /* "cupyx/cusolver.pyx":228
 *         solver = _cusolver.cgesvdjBatched
 *         s_dtype = 'f'
 *     elif a.dtype == 'D':             # <<<<<<<<<<<<<<
 *         helper = _cusolver.zgesvdjBatched_bufferSize
 *         solver = _cusolver.zgesvdjBatched
*/
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_v_a, __pyx_mstate_global->__pyx_n_u_dtype); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 228, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_6 = (__Pyx_PyUnicode_Equals(__pyx_t_1, __pyx_mstate_global->__pyx_n_u_D, Py_EQ)); if (unlikely((__pyx_t_6 < 0))) __PYX_ERR(0, 228, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  if (likely(__pyx_t_6)) {

    /* "cupyx/cusolver.pyx":229
 *         s_dtype = 'f'
 *     elif a.dtype == 'D':
 *         helper = _cusolver.zgesvdjBatched_bufferSize             # <<<<<<<<<<<<<<
 *         solver = _cusolver.zgesvdjBatched
 *         s_dtype = 'd'
*/
    __Pyx_GetModuleGlobalName(__pyx_t_1, __pyx_mstate_global->__pyx_n_u_cusolver); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 229, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_t_1, __pyx_mstate_global->__pyx_n_u_zgesvdjBatched_bufferSize); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 229, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __pyx_v_helper = __pyx_t_3;
    __pyx_t_3 = 0;

    /* "cupyx/cusolver.pyx":230
 *     elif a.dtype == 'D':
 *         helper = _cusolver.zgesvdjBatched_bufferSize
 *         solver = _cusolver.zgesvdjBatched             # <<<<<<<<<<<<<<
 *         s_dtype = 'd'
 *     else:
*/
    __Pyx_GetModuleGlobalName(__pyx_t_3, __pyx_mstate_global->__pyx_n_u_cusolver); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 230, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_t_3, __pyx_mstate_global->__pyx_n_u_zgesvdjBatched); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 230, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __pyx_v_solver = __pyx_t_1;
    __pyx_t_1 = 0;

    /* "cupyx/cusolver.pyx":231
 *         helper = _cusolver.zgesvdjBatched_bufferSize
 *         solver = _cusolver.zgesvdjBatched
 *         s_dtype = 'd'             # <<<<<<<<<<<<<<
 *     else:
 *         raise TypeError
*/
    __Pyx_INCREF(__pyx_mstate_global->__pyx_n_u_d);
    __pyx_v_s_dtype = __pyx_mstate_global->__pyx_n_u_d;

    /* "cupyx/cusolver.pyx":228
 *         solver = _cusolver.cgesvdjBatched
 *         s_dtype = 'f'
 *     elif a.dtype == 'D':             # <<<<<<<<<<<<<<
 *         helper = _cusolver.zgesvdjBatched_bufferSize
 *         solver = _cusolver.zgesvdjBatched
*/
    goto __pyx_L4;
  }

  /* "cupyx/cusolver.pyx":233
 *         s_dtype = 'd'
 *     else:
 *         raise TypeError             # <<<<<<<<<<<<<<
 * 
 *     handle = _device.get_cusolver_handle()
*/
  /*else*/ {
    __Pyx_Raise(((PyObject *)(((PyTypeObject*)PyExc_TypeError))), 0, 0, 0);
    __PYX_ERR(0, 233, __pyx_L1_error)
  }
  __pyx_L4:;

  /* "cupyx/cusolver.pyx":235
 *         raise TypeError
 * 
 *     handle = _device.get_cusolver_handle()             # <<<<<<<<<<<<<<
 *     batch_size, m, n = a.shape
 *     a = _cupy.array(a.swapaxes(-2, -1), order='C',
*/
  __pyx_t_3 = NULL;
  __Pyx_GetModuleGlobalName(__pyx_t_2, __pyx_mstate_global->__pyx_n_u_device); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 235, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_7 = __Pyx_PyObject_GetAttrStr(__pyx_t_2, __pyx_mstate_global->__pyx_n_u_get_cusolver_handle); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 235, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_7);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_4 = 1;
  #if CYTHON_UNPACK_METHODS
  if (unlikely(PyMethod_Check(__pyx_t_7))) {
    __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_7);
    assert(__pyx_t_3);
    PyObject* __pyx__function = PyMethod_GET_FUNCTION(__pyx_t_7);
    __Pyx_INCREF(__pyx_t_3);
    __Pyx_INCREF(__pyx__function);
    __Pyx_DECREF_SET(__pyx_t_7, __pyx__function);
    __pyx_t_4 = 0;
  }
  #endif
  {
    PyObject *__pyx_callargs[2] = {__pyx_t_3, NULL};
    __pyx_t_1 = __Pyx_PyObject_FastCall((PyObject*)__pyx_t_7, __pyx_callargs+__pyx_t_4, (1-__pyx_t_4) | (__pyx_t_4*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
    __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
    __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
    if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 235, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
  }
  __pyx_v_handle = __pyx_t_1;
  __pyx_t_1 = 0;

  /* "cupyx/cusolver.pyx":236
 * 
 *     handle = _device.get_cusolver_handle()
 *     batch_size, m, n = a.shape             # <<<<<<<<<<<<<<
 *     a = _cupy.array(a.swapaxes(-2, -1), order='C',
 *                     copy=(None if overwrite_a else True))
*/
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_v_a, __pyx_mstate_global->__pyx_n_u_shape); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 236, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if ((likely(PyTuple_CheckExact(__pyx_t_1))) || (PyList_CheckExact(__pyx_t_1))) {
    PyObject* sequence = __pyx_t_1;
    Py_ssize_t size = __Pyx_PySequence_SIZE(sequence);
    if (unlikely(size != 3)) {
      if (size > 3) __Pyx_RaiseTooManyValuesError(3);
      else if (size >= 0) __Pyx_RaiseNeedMoreValuesError(size);
      __PYX_ERR(0, 236, __pyx_L1_error)
    }
    #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
    if (likely(PyTuple_CheckExact(sequence))) {
      __pyx_t_7 = PyTuple_GET_ITEM(sequence, 0);
      __Pyx_INCREF(__pyx_t_7);
      __pyx_t_3 = PyTuple_GET_ITEM(sequence, 1);
      __Pyx_INCREF(__pyx_t_3);
      __pyx_t_2 = PyTuple_GET_ITEM(sequence, 2);
      __Pyx_INCREF(__pyx_t_2);
    } else {
      __pyx_t_7 = __Pyx_PyList_GetItemRefFast(sequence, 0, __Pyx_ReferenceSharing_SharedReference);
      if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 236, __pyx_L1_error)
      __Pyx_XGOTREF(__pyx_t_7);
      __pyx_t_3 = __Pyx_PyList_GetItemRefFast(sequence, 1, __Pyx_ReferenceSharing_SharedReference);
      if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 236, __pyx_L1_error)
      __Pyx_XGOTREF(__pyx_t_3);
      __pyx_t_2 = __Pyx_PyList_GetItemRefFast(sequence, 2, __Pyx_ReferenceSharing_SharedReference);
      if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 236, __pyx_L1_error)
      __Pyx_XGOTREF(__pyx_t_2);
    }
    #else
    __pyx_t_7 = __Pyx_PySequence_ITEM(sequence, 0); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 236, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_7);
    __pyx_t_3 = __Pyx_PySequence_ITEM(sequence, 1); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 236, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_t_2 = __Pyx_PySequence_ITEM(sequence, 2); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 236, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    #endif
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  } else {
    Py_ssize_t index = -1;
    __pyx_t_8 = PyObject_GetIter(__pyx_t_1); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 236, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_8);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __pyx_t_9 = (CYTHON_COMPILING_IN_LIMITED_API) ? PyIter_Next : __Pyx_PyObject_GetIterNextFunc(__pyx_t_8);
    index = 0; __pyx_t_7 = __pyx_t_9(__pyx_t_8); if (unlikely(!__pyx_t_7)) goto __pyx_L5_unpacking_failed;
    __Pyx_GOTREF(__pyx_t_7);
    index = 1; __pyx_t_3 = __pyx_t_9(__pyx_t_8); if (unlikely(!__pyx_t_3)) goto __pyx_L5_unpacking_failed;
    __Pyx_GOTREF(__pyx_t_3);
    index = 2; __pyx_t_2 = __pyx_t_9(__pyx_t_8); if (unlikely(!__pyx_t_2)) goto __pyx_L5_unpacking_failed;
    __Pyx_GOTREF(__pyx_t_2);
    if (__Pyx_IternextUnpackEndCheck(__pyx_t_9(__pyx_t_8), 3) < (0)) __PYX_ERR(0, 236, __pyx_L1_error)
    __pyx_t_9 = NULL;
    __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
    goto __pyx_L6_unpacking_done;
    __pyx_L5_unpacking_failed:;
    __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
    __pyx_t_9 = NULL;
    if (__Pyx_IterFinish() == 0) __Pyx_RaiseNeedMoreValuesError(index);
    __PYX_ERR(0, 236, __pyx_L1_error)
    __pyx_L6_unpacking_done:;
  }
  __pyx_v_batch_size = __pyx_t_7;
  __pyx_t_7 = 0;
  __pyx_v_m = __pyx_t_3;
  __pyx_t_3 = 0;
  __pyx_v_n = __pyx_t_2;
  __pyx_t_2 = 0;

  /* "cupyx/cusolver.pyx":237
 *     handle = _device.get_cusolver_handle()
 *     batch_size, m, n = a.shape
 *     a = _cupy.array(a.swapaxes(-2, -1), order='C',             # <<<<<<<<<<<<<<
 *                     copy=(None if overwrite_a else True))
 *     if runtime._is_hip_environment:
*/
  __pyx_t_2 = NULL;
  __Pyx_GetModuleGlobalName(__pyx_t_3, __pyx_mstate_global->__pyx_n_u_cupy); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 237, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_7 = __Pyx_PyObject_GetAttrStr(__pyx_t_3, __pyx_mstate_global->__pyx_n_u_array); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 237, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_7);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_v_a, __pyx_mstate_global->__pyx_n_u_swapaxes); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 237, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_8 = __Pyx_PyObject_Call(__pyx_t_3, __pyx_mstate_global->__pyx_tuple[0], NULL); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 237, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_8);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "cupyx/cusolver.pyx":238
 *     batch_size, m, n = a.shape
 *     a = _cupy.array(a.swapaxes(-2, -1), order='C',
 *                     copy=(None if overwrite_a else True))             # <<<<<<<<<<<<<<
 *     if runtime._is_hip_environment:
 *         # rocsolver_<t>gesvd_batched has a different signature...
*/
  __pyx_t_6 = __Pyx_PyObject_IsTrue(__pyx_v_overwrite_a); if (unlikely((__pyx_t_6 < 0))) __PYX_ERR(0, 238, __pyx_L1_error)
  if (__pyx_t_6) {
    __Pyx_INCREF(Py_None);
    __pyx_t_3 = Py_None;
  } else {
    __Pyx_INCREF(Py_True);
    __pyx_t_3 = Py_True;
  }
  __pyx_t_4 = 1;
  #if CYTHON_UNPACK_METHODS
  if (unlikely(PyMethod_Check(__pyx_t_7))) {
    __pyx_t_2 = PyMethod_GET_SELF(__pyx_t_7);
    assert(__pyx_t_2);
    PyObject* __pyx__function = PyMethod_GET_FUNCTION(__pyx_t_7);
    __Pyx_INCREF(__pyx_t_2);
    __Pyx_INCREF(__pyx__function);
    __Pyx_DECREF_SET(__pyx_t_7, __pyx__function);
    __pyx_t_4 = 0;
  }
  #endif
  {
    PyObject *__pyx_callargs[2 + ((CYTHON_VECTORCALL) ? 2 : 0)] = {__pyx_t_2, __pyx_t_8};
    __pyx_t_10 = __Pyx_MakeVectorcallBuilderKwds(2); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 237, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_10);
    if (__Pyx_VectorcallBuilder_AddArg(__pyx_mstate_global->__pyx_n_u_order, __pyx_mstate_global->__pyx_n_u_C, __pyx_t_10, __pyx_callargs+2, 0) < (0)) __PYX_ERR(0, 237, __pyx_L1_error)
    if (__Pyx_VectorcallBuilder_AddArg(__pyx_mstate_global->__pyx_n_u_copy, __pyx_t_3, __pyx_t_10, __pyx_callargs+2, 1) < (0)) __PYX_ERR(0, 237, __pyx_L1_error)
    __pyx_t_1 = __Pyx_Object_Vectorcall_CallFromBuilder((PyObject*)__pyx_t_7, __pyx_callargs+__pyx_t_4, (2-__pyx_t_4) | (__pyx_t_4*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET), __pyx_t_10);
    __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
    __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
    if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 237, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
  }
  __Pyx_DECREF_SET(__pyx_v_a, __pyx_t_1);
  __pyx_t_1 = 0;

  /* "cupyx/cusolver.pyx":239
 *     a = _cupy.array(a.swapaxes(-2, -1), order='C',
 *                     copy=(None if overwrite_a else True))
 *     if runtime._is_hip_environment:             # <<<<<<<<<<<<<<
 *         # rocsolver_<t>gesvd_batched has a different signature...
 *         ap = _linalg._mat_ptrs(a)
*/
  if (__pyx_v_13cupy_backends_4cuda_3api_7runtime__is_hip_environment) {

    /* "cupyx/cusolver.pyx":241
 *     if runtime._is_hip_environment:
 *         # rocsolver_<t>gesvd_batched has a different signature...
 *         ap = _linalg._mat_ptrs(a)             # <<<<<<<<<<<<<<
 *     else:
 *         ap = a
*/
    __pyx_t_7 = NULL;
    __Pyx_GetModuleGlobalName(__pyx_t_10, __pyx_mstate_global->__pyx_n_u_linalg_2); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 241, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_10);
    __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_t_10, __pyx_mstate_global->__pyx_n_u_mat_ptrs); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 241, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
    __pyx_t_4 = 1;
    #if CYTHON_UNPACK_METHODS
    if (unlikely(PyMethod_Check(__pyx_t_3))) {
      __pyx_t_7 = PyMethod_GET_SELF(__pyx_t_3);
      assert(__pyx_t_7);
      PyObject* __pyx__function = PyMethod_GET_FUNCTION(__pyx_t_3);
      __Pyx_INCREF(__pyx_t_7);
      __Pyx_INCREF(__pyx__function);
      __Pyx_DECREF_SET(__pyx_t_3, __pyx__function);
      __pyx_t_4 = 0;
    }
    #endif
    {
      PyObject *__pyx_callargs[2] = {__pyx_t_7, __pyx_v_a};
      __pyx_t_1 = __Pyx_PyObject_FastCall((PyObject*)__pyx_t_3, __pyx_callargs+__pyx_t_4, (2-__pyx_t_4) | (__pyx_t_4*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
      __Pyx_XDECREF(__pyx_t_7); __pyx_t_7 = 0;
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
      if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 241, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
    }
    __pyx_v_ap = __pyx_t_1;
    __pyx_t_1 = 0;

    /* "cupyx/cusolver.pyx":239
 *     a = _cupy.array(a.swapaxes(-2, -1), order='C',
 *                     copy=(None if overwrite_a else True))
 *     if runtime._is_hip_environment:             # <<<<<<<<<<<<<<
 *         # rocsolver_<t>gesvd_batched has a different signature...
 *         ap = _linalg._mat_ptrs(a)
*/
    goto __pyx_L7;
  }

  /* "cupyx/cusolver.pyx":243
 *         ap = _linalg._mat_ptrs(a)
 *     else:
 *         ap = a             # <<<<<<<<<<<<<<
 *     lda = m
 *     mn = min(m, n)
*/
  /*else*/ {
    __Pyx_INCREF(__pyx_v_a);
    __pyx_v_ap = __pyx_v_a;
  }
  __pyx_L7:;

  /* "cupyx/cusolver.pyx":244
 *     else:
 *         ap = a
 *     lda = m             # <<<<<<<<<<<<<<
 *     mn = min(m, n)
 *     s = _cupy.empty((batch_size, mn), dtype=s_dtype)
*/
  __Pyx_INCREF(__pyx_v_m);
  __pyx_v_lda = __pyx_v_m;

  /* "cupyx/cusolver.pyx":245
 *         ap = a
 *     lda = m
 *     mn = min(m, n)             # <<<<<<<<<<<<<<
 *     s = _cupy.empty((batch_size, mn), dtype=s_dtype)
 *     ldu = m
*/
  __Pyx_INCREF(__pyx_v_n);
  __pyx_t_1 = __pyx_v_n;
  __Pyx_INCREF(__pyx_v_m);
  __pyx_t_3 = __pyx_v_m;
  __pyx_t_10 = PyObject_RichCompare(__pyx_t_1, __pyx_t_3, Py_LT); __Pyx_XGOTREF(__pyx_t_10); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 245, __pyx_L1_error)
  __pyx_t_6 = __Pyx_PyObject_IsTrue(__pyx_t_10); if (unlikely((__pyx_t_6 < 0))) __PYX_ERR(0, 245, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
  if (__pyx_t_6) {
    __Pyx_INCREF(__pyx_t_1);
    __pyx_t_7 = __pyx_t_1;
  } else {
    __Pyx_INCREF(__pyx_t_3);
    __pyx_t_7 = __pyx_t_3;
  }
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_1 = __pyx_t_7;
  __Pyx_INCREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
  __pyx_v_mn = __pyx_t_1;
  __pyx_t_1 = 0;

  /* "cupyx/cusolver.pyx":246
 *     lda = m
 *     mn = min(m, n)
 *     s = _cupy.empty((batch_size, mn), dtype=s_dtype)             # <<<<<<<<<<<<<<
 *     ldu = m
 *     ldv = n
*/
  __pyx_t_7 = NULL;
  __Pyx_GetModuleGlobalName(__pyx_t_3, __pyx_mstate_global->__pyx_n_u_cupy); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 246, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_10 = __Pyx_PyObject_GetAttrStr(__pyx_t_3, __pyx_mstate_global->__pyx_n_u_empty); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 246, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_10);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_t_3 = PyTuple_New(2); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 246, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_INCREF(__pyx_v_batch_size);
  __Pyx_GIVEREF(__pyx_v_batch_size);
  if (__Pyx_PyTuple_SET_ITEM(__pyx_t_3, 0, __pyx_v_batch_size) != (0)) __PYX_ERR(0, 246, __pyx_L1_error);
  __Pyx_INCREF(__pyx_v_mn);
  __Pyx_GIVEREF(__pyx_v_mn);
  if (__Pyx_PyTuple_SET_ITEM(__pyx_t_3, 1, __pyx_v_mn) != (0)) __PYX_ERR(0, 246, __pyx_L1_error);
  __pyx_t_4 = 1;
  #if CYTHON_UNPACK_METHODS
  if (unlikely(PyMethod_Check(__pyx_t_10))) {
    __pyx_t_7 = PyMethod_GET_SELF(__pyx_t_10);
    assert(__pyx_t_7);
    PyObject* __pyx__function = PyMethod_GET_FUNCTION(__pyx_t_10);
    __Pyx_INCREF(__pyx_t_7);
    __Pyx_INCREF(__pyx__function);
    __Pyx_DECREF_SET(__pyx_t_10, __pyx__function);
    __pyx_t_4 = 0;
  }
  #endif
  {
    PyObject *__pyx_callargs[2 + ((CYTHON_VECTORCALL) ? 1 : 0)] = {__pyx_t_7, __pyx_t_3};
    __pyx_t_8 = __Pyx_MakeVectorcallBuilderKwds(1); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 246, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_8);
    if (__Pyx_VectorcallBuilder_AddArg(__pyx_mstate_global->__pyx_n_u_dtype, __pyx_v_s_dtype, __pyx_t_8, __pyx_callargs+2, 0) < (0)) __PYX_ERR(0, 246, __pyx_L1_error)
    __pyx_t_1 = __Pyx_Object_Vectorcall_CallFromBuilder((PyObject*)__pyx_t_10, __pyx_callargs+__pyx_t_4, (2-__pyx_t_4) | (__pyx_t_4*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET), __pyx_t_8);
    __Pyx_XDECREF(__pyx_t_7); __pyx_t_7 = 0;
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
    __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
    if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 246, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
  }
  __pyx_v_s = __pyx_t_1;
  __pyx_t_1 = 0;

  /* "cupyx/cusolver.pyx":247
 *     mn = min(m, n)
 *     s = _cupy.empty((batch_size, mn), dtype=s_dtype)
 *     ldu = m             # <<<<<<<<<<<<<<
 *     ldv = n
 *     if compute_uv:
*/
  __Pyx_INCREF(__pyx_v_m);
  __pyx_v_ldu = __pyx_v_m;

  /* "cupyx/cusolver.pyx":248
 *     s = _cupy.empty((batch_size, mn), dtype=s_dtype)
 *     ldu = m
 *     ldv = n             # <<<<<<<<<<<<<<
 *     if compute_uv:
 *         jobz = _cusolver.CUSOLVER_EIG_MODE_VECTOR
*/
  __Pyx_INCREF(__pyx_v_n);
  __pyx_v_ldv = __pyx_v_n;

  /* "cupyx/cusolver.pyx":249
 *     ldu = m
 *     ldv = n
 *     if compute_uv:             # <<<<<<<<<<<<<<
 *         jobz = _cusolver.CUSOLVER_EIG_MODE_VECTOR
 *     else:
*/
  __pyx_t_6 = __Pyx_PyObject_IsTrue(__pyx_v_compute_uv); if (unlikely((__pyx_t_6 < 0))) __PYX_ERR(0, 249, __pyx_L1_error)
  if (__pyx_t_6) {

    /* "cupyx/cusolver.pyx":250
 *     ldv = n
 *     if compute_uv:
 *         jobz = _cusolver.CUSOLVER_EIG_MODE_VECTOR             # <<<<<<<<<<<<<<
 *     else:
 *         jobz = _cusolver.CUSOLVER_EIG_MODE_NOVECTOR
*/
    __Pyx_GetModuleGlobalName(__pyx_t_1, __pyx_mstate_global->__pyx_n_u_cusolver); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 250, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_t_10 = __Pyx_PyObject_GetAttrStr(__pyx_t_1, __pyx_mstate_global->__pyx_n_u_CUSOLVER_EIG_MODE_VECTOR); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 250, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_10);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __pyx_v_jobz = __pyx_t_10;
    __pyx_t_10 = 0;

    /* "cupyx/cusolver.pyx":249
 *     ldu = m
 *     ldv = n
 *     if compute_uv:             # <<<<<<<<<<<<<<
 *         jobz = _cusolver.CUSOLVER_EIG_MODE_VECTOR
 *     else:
*/
    goto __pyx_L8;
  }

  /* "cupyx/cusolver.pyx":252
 *         jobz = _cusolver.CUSOLVER_EIG_MODE_VECTOR
 *     else:
 *         jobz = _cusolver.CUSOLVER_EIG_MODE_NOVECTOR             # <<<<<<<<<<<<<<
 *         # if not batched, `full_matrices = False` could speedup.
 * 
*/
  /*else*/ {
    __Pyx_GetModuleGlobalName(__pyx_t_10, __pyx_mstate_global->__pyx_n_u_cusolver); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 252, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_10);
    __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_t_10, __pyx_mstate_global->__pyx_n_u_CUSOLVER_EIG_MODE_NOVECTOR); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 252, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
    __pyx_v_jobz = __pyx_t_1;
    __pyx_t_1 = 0;
  }
  __pyx_L8:;

  /* "cupyx/cusolver.pyx":255
 *         # if not batched, `full_matrices = False` could speedup.
 * 
 *     u = _cupy.empty((batch_size, m, ldu), dtype=a.dtype).swapaxes(-2, -1)             # <<<<<<<<<<<<<<
 *     v = _cupy.empty((batch_size, n, ldv), dtype=a.dtype).swapaxes(-2, -1)
 *     params = _cusolver.createGesvdjInfo()
*/
  __pyx_t_10 = NULL;
  __Pyx_GetModuleGlobalName(__pyx_t_8, __pyx_mstate_global->__pyx_n_u_cupy); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 255, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_8);
  __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_t_8, __pyx_mstate_global->__pyx_n_u_empty); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 255, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
  __pyx_t_8 = PyTuple_New(3); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 255, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_8);
  __Pyx_INCREF(__pyx_v_batch_size);
  __Pyx_GIVEREF(__pyx_v_batch_size);
  if (__Pyx_PyTuple_SET_ITEM(__pyx_t_8, 0, __pyx_v_batch_size) != (0)) __PYX_ERR(0, 255, __pyx_L1_error);
  __Pyx_INCREF(__pyx_v_m);
  __Pyx_GIVEREF(__pyx_v_m);
  if (__Pyx_PyTuple_SET_ITEM(__pyx_t_8, 1, __pyx_v_m) != (0)) __PYX_ERR(0, 255, __pyx_L1_error);
  __Pyx_INCREF(__pyx_v_ldu);
  __Pyx_GIVEREF(__pyx_v_ldu);
  if (__Pyx_PyTuple_SET_ITEM(__pyx_t_8, 2, __pyx_v_ldu) != (0)) __PYX_ERR(0, 255, __pyx_L1_error);
  __pyx_t_7 = __Pyx_PyObject_GetAttrStr(__pyx_v_a, __pyx_mstate_global->__pyx_n_u_dtype); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 255, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_7);
  __pyx_t_4 = 1;
  #if CYTHON_UNPACK_METHODS
  if (unlikely(PyMethod_Check(__pyx_t_3))) {
    __pyx_t_10 = PyMethod_GET_SELF(__pyx_t_3);
    assert(__pyx_t_10);
    PyObject* __pyx__function = PyMethod_GET_FUNCTION(__pyx_t_3);
    __Pyx_INCREF(__pyx_t_10);
    __Pyx_INCREF(__pyx__function);
    __Pyx_DECREF_SET(__pyx_t_3, __pyx__function);
    __pyx_t_4 = 0;
  }
  #endif
  {
    PyObject *__pyx_callargs[2 + ((CYTHON_VECTORCALL) ? 1 : 0)] = {__pyx_t_10, __pyx_t_8};
    __pyx_t_2 = __Pyx_MakeVectorcallBuilderKwds(1); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 255, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    if (__Pyx_VectorcallBuilder_AddArg(__pyx_mstate_global->__pyx_n_u_dtype, __pyx_t_7, __pyx_t_2, __pyx_callargs+2, 0) < (0)) __PYX_ERR(0, 255, __pyx_L1_error)
    __pyx_t_1 = __Pyx_Object_Vectorcall_CallFromBuilder((PyObject*)__pyx_t_3, __pyx_callargs+__pyx_t_4, (2-__pyx_t_4) | (__pyx_t_4*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET), __pyx_t_2);
    __Pyx_XDECREF(__pyx_t_10); __pyx_t_10 = 0;
    __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
    __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 255, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
  }
  __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_t_1, __pyx_mstate_global->__pyx_n_u_swapaxes); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 255, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_3, __pyx_mstate_global->__pyx_tuple[0], NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 255, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_v_u = __pyx_t_1;
  __pyx_t_1 = 0;

  /* "cupyx/cusolver.pyx":256
 * 
 *     u = _cupy.empty((batch_size, m, ldu), dtype=a.dtype).swapaxes(-2, -1)
 *     v = _cupy.empty((batch_size, n, ldv), dtype=a.dtype).swapaxes(-2, -1)             # <<<<<<<<<<<<<<
 *     params = _cusolver.createGesvdjInfo()
 *     lwork = helper(handle, jobz, m, n, ap.data.ptr, lda, s.data.ptr,
*/
  __pyx_t_3 = NULL;
  __Pyx_GetModuleGlobalName(__pyx_t_2, __pyx_mstate_global->__pyx_n_u_cupy); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 256, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_7 = __Pyx_PyObject_GetAttrStr(__pyx_t_2, __pyx_mstate_global->__pyx_n_u_empty); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 256, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_7);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_2 = PyTuple_New(3); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 256, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_INCREF(__pyx_v_batch_size);
  __Pyx_GIVEREF(__pyx_v_batch_size);
  if (__Pyx_PyTuple_SET_ITEM(__pyx_t_2, 0, __pyx_v_batch_size) != (0)) __PYX_ERR(0, 256, __pyx_L1_error);
  __Pyx_INCREF(__pyx_v_n);
  __Pyx_GIVEREF(__pyx_v_n);
  if (__Pyx_PyTuple_SET_ITEM(__pyx_t_2, 1, __pyx_v_n) != (0)) __PYX_ERR(0, 256, __pyx_L1_error);
  __Pyx_INCREF(__pyx_v_ldv);
  __Pyx_GIVEREF(__pyx_v_ldv);
  if (__Pyx_PyTuple_SET_ITEM(__pyx_t_2, 2, __pyx_v_ldv) != (0)) __PYX_ERR(0, 256, __pyx_L1_error);
  __pyx_t_8 = __Pyx_PyObject_GetAttrStr(__pyx_v_a, __pyx_mstate_global->__pyx_n_u_dtype); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 256, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_8);
  __pyx_t_4 = 1;
  #if CYTHON_UNPACK_METHODS
  if (unlikely(PyMethod_Check(__pyx_t_7))) {
    __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_7);
    assert(__pyx_t_3);
    PyObject* __pyx__function = PyMethod_GET_FUNCTION(__pyx_t_7);
    __Pyx_INCREF(__pyx_t_3);
    __Pyx_INCREF(__pyx__function);
    __Pyx_DECREF_SET(__pyx_t_7, __pyx__function);
    __pyx_t_4 = 0;
  }
  #endif
  {
    PyObject *__pyx_callargs[2 + ((CYTHON_VECTORCALL) ? 1 : 0)] = {__pyx_t_3, __pyx_t_2};
    __pyx_t_10 = __Pyx_MakeVectorcallBuilderKwds(1); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 256, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_10);
    if (__Pyx_VectorcallBuilder_AddArg(__pyx_mstate_global->__pyx_n_u_dtype, __pyx_t_8, __pyx_t_10, __pyx_callargs+2, 0) < (0)) __PYX_ERR(0, 256, __pyx_L1_error)
    __pyx_t_1 = __Pyx_Object_Vectorcall_CallFromBuilder((PyObject*)__pyx_t_7, __pyx_callargs+__pyx_t_4, (2-__pyx_t_4) | (__pyx_t_4*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET), __pyx_t_10);
    __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
    __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
    __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
    if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 256, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
  }
  __pyx_t_7 = __Pyx_PyObject_GetAttrStr(__pyx_t_1, __pyx_mstate_global->__pyx_n_u_swapaxes); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 256, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_7);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_7, __pyx_mstate_global->__pyx_tuple[0], NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 256, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
  __pyx_v_v = __pyx_t_1;
  __pyx_t_1 = 0;

  /* "cupyx/cusolver.pyx":257
 *     u = _cupy.empty((batch_size, m, ldu), dtype=a.dtype).swapaxes(-2, -1)
 *     v = _cupy.empty((batch_size, n, ldv), dtype=a.dtype).swapaxes(-2, -1)
 *     params = _cusolver.createGesvdjInfo()             # <<<<<<<<<<<<<<
 *     lwork = helper(handle, jobz, m, n, ap.data.ptr, lda, s.data.ptr,
 *                    u.data.ptr, ldu, v.data.ptr, ldv, params, batch_size)
*/
  __pyx_t_7 = NULL;
  __Pyx_GetModuleGlobalName(__pyx_t_10, __pyx_mstate_global->__pyx_n_u_cusolver); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 257, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_10);
  __pyx_t_8 = __Pyx_PyObject_GetAttrStr(__pyx_t_10, __pyx_mstate_global->__pyx_n_u_createGesvdjInfo); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 257, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_8);
  __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
  __pyx_t_4 = 1;
  #if CYTHON_UNPACK_METHODS
  if (unlikely(PyMethod_Check(__pyx_t_8))) {
    __pyx_t_7 = PyMethod_GET_SELF(__pyx_t_8);
    assert(__pyx_t_7);
    PyObject* __pyx__function = PyMethod_GET_FUNCTION(__pyx_t_8);
    __Pyx_INCREF(__pyx_t_7);
    __Pyx_INCREF(__pyx__function);
    __Pyx_DECREF_SET(__pyx_t_8, __pyx__function);
    __pyx_t_4 = 0;
  }
  #endif
  {
    PyObject *__pyx_callargs[2] = {__pyx_t_7, NULL};
    __pyx_t_1 = __Pyx_PyObject_FastCall((PyObject*)__pyx_t_8, __pyx_callargs+__pyx_t_4, (1-__pyx_t_4) | (__pyx_t_4*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
    __Pyx_XDECREF(__pyx_t_7); __pyx_t_7 = 0;
    __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
    if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 257, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
  }
  __pyx_v_params = __pyx_t_1;
  __pyx_t_1 = 0;

  /* "cupyx/cusolver.pyx":258
 *     v = _cupy.empty((batch_size, n, ldv), dtype=a.dtype).swapaxes(-2, -1)
 *     params = _cusolver.createGesvdjInfo()
 *     lwork = helper(handle, jobz, m, n, ap.data.ptr, lda, s.data.ptr,             # <<<<<<<<<<<<<<
 *                    u.data.ptr, ldu, v.data.ptr, ldv, params, batch_size)
 *     work = _cupy.empty(lwork, dtype=a.dtype)
*/
  __pyx_t_8 = NULL;
  __Pyx_INCREF(__pyx_v_helper);
  __pyx_t_7 = __pyx_v_helper; 
  __pyx_t_10 = __Pyx_PyObject_GetAttrStr(__pyx_v_ap, __pyx_mstate_global->__pyx_n_u_data); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 258, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_10);
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_t_10, __pyx_mstate_global->__pyx_n_u_ptr); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 258, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
  __pyx_t_10 = __Pyx_PyObject_GetAttrStr(__pyx_v_s, __pyx_mstate_global->__pyx_n_u_data); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 258, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_10);
  __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_t_10, __pyx_mstate_global->__pyx_n_u_ptr); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 258, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;

  /* "cupyx/cusolver.pyx":259
 *     params = _cusolver.createGesvdjInfo()
 *     lwork = helper(handle, jobz, m, n, ap.data.ptr, lda, s.data.ptr,
 *                    u.data.ptr, ldu, v.data.ptr, ldv, params, batch_size)             # <<<<<<<<<<<<<<
 *     work = _cupy.empty(lwork, dtype=a.dtype)
 *     info = _cupy.empty(batch_size, dtype=_numpy.int32)
*/
  __pyx_t_10 = __Pyx_PyObject_GetAttrStr(__pyx_v_u, __pyx_mstate_global->__pyx_n_u_data); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 259, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_10);
  __pyx_t_11 = __Pyx_PyObject_GetAttrStr(__pyx_t_10, __pyx_mstate_global->__pyx_n_u_ptr); if (unlikely(!__pyx_t_11)) __PYX_ERR(0, 259, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_11);
  __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
  __pyx_t_10 = __Pyx_PyObject_GetAttrStr(__pyx_v_v, __pyx_mstate_global->__pyx_n_u_data); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 259, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_10);
  __pyx_t_12 = __Pyx_PyObject_GetAttrStr(__pyx_t_10, __pyx_mstate_global->__pyx_n_u_ptr); if (unlikely(!__pyx_t_12)) __PYX_ERR(0, 259, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_12);
  __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
  __pyx_t_4 = 1;
  #if CYTHON_UNPACK_METHODS
  if (unlikely(PyMethod_Check(__pyx_t_7))) {
    __pyx_t_8 = PyMethod_GET_SELF(__pyx_t_7);
    assert(__pyx_t_8);
    PyObject* __pyx__function = PyMethod_GET_FUNCTION(__pyx_t_7);
    __Pyx_INCREF(__pyx_t_8);
    __Pyx_INCREF(__pyx__function);
    __Pyx_DECREF_SET(__pyx_t_7, __pyx__function);
    __pyx_t_4 = 0;
  }
  #endif
  {
    PyObject *__pyx_callargs[14] = {__pyx_t_8, __pyx_v_handle, __pyx_v_jobz, __pyx_v_m, __pyx_v_n, __pyx_t_2, __pyx_v_lda, __pyx_t_3, __pyx_t_11, __pyx_v_ldu, __pyx_t_12, __pyx_v_ldv, __pyx_v_params, __pyx_v_batch_size};
    __pyx_t_1 = __Pyx_PyObject_FastCall((PyObject*)__pyx_t_7, __pyx_callargs+__pyx_t_4, (14-__pyx_t_4) | (__pyx_t_4*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
    __Pyx_XDECREF(__pyx_t_8); __pyx_t_8 = 0;
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __Pyx_DECREF(__pyx_t_11); __pyx_t_11 = 0;
    __Pyx_DECREF(__pyx_t_12); __pyx_t_12 = 0;
    __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
    if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 258, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
  }
  __pyx_v_lwork = __pyx_t_1;
  __pyx_t_1 = 0;

  /* "cupyx/cusolver.pyx":260
 *     lwork = helper(handle, jobz, m, n, ap.data.ptr, lda, s.data.ptr,
 *                    u.data.ptr, ldu, v.data.ptr, ldv, params, batch_size)
 *     work = _cupy.empty(lwork, dtype=a.dtype)             # <<<<<<<<<<<<<<
 *     info = _cupy.empty(batch_size, dtype=_numpy.int32)
 *     solver(handle, jobz, m, n, ap.data.ptr, lda, s.data.ptr,
*/
  __pyx_t_7 = NULL;
  __Pyx_GetModuleGlobalName(__pyx_t_12, __pyx_mstate_global->__pyx_n_u_cupy); if (unlikely(!__pyx_t_12)) __PYX_ERR(0, 260, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_12);
  __pyx_t_11 = __Pyx_PyObject_GetAttrStr(__pyx_t_12, __pyx_mstate_global->__pyx_n_u_empty); if (unlikely(!__pyx_t_11)) __PYX_ERR(0, 260, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_11);
  __Pyx_DECREF(__pyx_t_12); __pyx_t_12 = 0;
  __pyx_t_12 = __Pyx_PyObject_GetAttrStr(__pyx_v_a, __pyx_mstate_global->__pyx_n_u_dtype); if (unlikely(!__pyx_t_12)) __PYX_ERR(0, 260, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_12);
  __pyx_t_4 = 1;
  #if CYTHON_UNPACK_METHODS
  if (unlikely(PyMethod_Check(__pyx_t_11))) {
    __pyx_t_7 = PyMethod_GET_SELF(__pyx_t_11);
    assert(__pyx_t_7);
    PyObject* __pyx__function = PyMethod_GET_FUNCTION(__pyx_t_11);
    __Pyx_INCREF(__pyx_t_7);
    __Pyx_INCREF(__pyx__function);
    __Pyx_DECREF_SET(__pyx_t_11, __pyx__function);
    __pyx_t_4 = 0;
  }
  #endif
  {
    PyObject *__pyx_callargs[2 + ((CYTHON_VECTORCALL) ? 1 : 0)] = {__pyx_t_7, __pyx_v_lwork};
    __pyx_t_3 = __Pyx_MakeVectorcallBuilderKwds(1); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 260, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    if (__Pyx_VectorcallBuilder_AddArg(__pyx_mstate_global->__pyx_n_u_dtype, __pyx_t_12, __pyx_t_3, __pyx_callargs+2, 0) < (0)) __PYX_ERR(0, 260, __pyx_L1_error)
    __pyx_t_1 = __Pyx_Object_Vectorcall_CallFromBuilder((PyObject*)__pyx_t_11, __pyx_callargs+__pyx_t_4, (2-__pyx_t_4) | (__pyx_t_4*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET), __pyx_t_3);
    __Pyx_XDECREF(__pyx_t_7); __pyx_t_7 = 0;
    __Pyx_DECREF(__pyx_t_12); __pyx_t_12 = 0;
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __Pyx_DECREF(__pyx_t_11); __pyx_t_11 = 0;
    if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 260, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
  }
  __pyx_v_work = __pyx_t_1;
  __pyx_t_1 = 0;

  /* "cupyx/cusolver.pyx":261
 *                    u.data.ptr, ldu, v.data.ptr, ldv, params, batch_size)
 *     work = _cupy.empty(lwork, dtype=a.dtype)
 *     info = _cupy.empty(batch_size, dtype=_numpy.int32)             # <<<<<<<<<<<<<<
 *     solver(handle, jobz, m, n, ap.data.ptr, lda, s.data.ptr,
 *            u.data.ptr, ldu, v.data.ptr, ldv, work.data.ptr, lwork,
*/
  __pyx_t_11 = NULL;
  __Pyx_GetModuleGlobalName(__pyx_t_3, __pyx_mstate_global->__pyx_n_u_cupy); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 261, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_12 = __Pyx_PyObject_GetAttrStr(__pyx_t_3, __pyx_mstate_global->__pyx_n_u_empty); if (unlikely(!__pyx_t_12)) __PYX_ERR(0, 261, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_12);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __Pyx_GetModuleGlobalName(__pyx_t_3, __pyx_mstate_global->__pyx_n_u_numpy); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 261, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_7 = __Pyx_PyObject_GetAttrStr(__pyx_t_3, __pyx_mstate_global->__pyx_n_u_int32); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 261, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_7);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_t_4 = 1;
  #if CYTHON_UNPACK_METHODS
  if (unlikely(PyMethod_Check(__pyx_t_12))) {
    __pyx_t_11 = PyMethod_GET_SELF(__pyx_t_12);
    assert(__pyx_t_11);
    PyObject* __pyx__function = PyMethod_GET_FUNCTION(__pyx_t_12);
    __Pyx_INCREF(__pyx_t_11);
    __Pyx_INCREF(__pyx__function);
    __Pyx_DECREF_SET(__pyx_t_12, __pyx__function);
    __pyx_t_4 = 0;
  }
  #endif
  {
    PyObject *__pyx_callargs[2 + ((CYTHON_VECTORCALL) ? 1 : 0)] = {__pyx_t_11, __pyx_v_batch_size};
    __pyx_t_3 = __Pyx_MakeVectorcallBuilderKwds(1); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 261, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    if (__Pyx_VectorcallBuilder_AddArg(__pyx_mstate_global->__pyx_n_u_dtype, __pyx_t_7, __pyx_t_3, __pyx_callargs+2, 0) < (0)) __PYX_ERR(0, 261, __pyx_L1_error)
    __pyx_t_1 = __Pyx_Object_Vectorcall_CallFromBuilder((PyObject*)__pyx_t_12, __pyx_callargs+__pyx_t_4, (2-__pyx_t_4) | (__pyx_t_4*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET), __pyx_t_3);
    __Pyx_XDECREF(__pyx_t_11); __pyx_t_11 = 0;
    __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __Pyx_DECREF(__pyx_t_12); __pyx_t_12 = 0;
    if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 261, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
  }
  __pyx_v_info = __pyx_t_1;
  __pyx_t_1 = 0;

  /* "cupyx/cusolver.pyx":262
 *     work = _cupy.empty(lwork, dtype=a.dtype)
 *     info = _cupy.empty(batch_size, dtype=_numpy.int32)
 *     solver(handle, jobz, m, n, ap.data.ptr, lda, s.data.ptr,             # <<<<<<<<<<<<<<
 *            u.data.ptr, ldu, v.data.ptr, ldv, work.data.ptr, lwork,
 *            info.data.ptr, params, batch_size)
*/
  __pyx_t_12 = NULL;
  __Pyx_INCREF(__pyx_v_solver);
  __pyx_t_3 = __pyx_v_solver; 
  __pyx_t_7 = __Pyx_PyObject_GetAttrStr(__pyx_v_ap, __pyx_mstate_global->__pyx_n_u_data); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 262, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_7);
  __pyx_t_11 = __Pyx_PyObject_GetAttrStr(__pyx_t_7, __pyx_mstate_global->__pyx_n_u_ptr); if (unlikely(!__pyx_t_11)) __PYX_ERR(0, 262, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_11);
  __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
  __pyx_t_7 = __Pyx_PyObject_GetAttrStr(__pyx_v_s, __pyx_mstate_global->__pyx_n_u_data); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 262, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_7);
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_t_7, __pyx_mstate_global->__pyx_n_u_ptr); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 262, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;

  /* "cupyx/cusolver.pyx":263
 *     info = _cupy.empty(batch_size, dtype=_numpy.int32)
 *     solver(handle, jobz, m, n, ap.data.ptr, lda, s.data.ptr,
 *            u.data.ptr, ldu, v.data.ptr, ldv, work.data.ptr, lwork,             # <<<<<<<<<<<<<<
 *            info.data.ptr, params, batch_size)
 *     _cupy.linalg._util._check_cusolver_dev_info_if_synchronization_allowed(
*/
  __pyx_t_7 = __Pyx_PyObject_GetAttrStr(__pyx_v_u, __pyx_mstate_global->__pyx_n_u_data); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 263, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_7);
  __pyx_t_8 = __Pyx_PyObject_GetAttrStr(__pyx_t_7, __pyx_mstate_global->__pyx_n_u_ptr); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 263, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_8);
  __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
  __pyx_t_7 = __Pyx_PyObject_GetAttrStr(__pyx_v_v, __pyx_mstate_global->__pyx_n_u_data); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 263, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_7);
  __pyx_t_10 = __Pyx_PyObject_GetAttrStr(__pyx_t_7, __pyx_mstate_global->__pyx_n_u_ptr); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 263, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_10);
  __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
  __pyx_t_7 = __Pyx_PyObject_GetAttrStr(__pyx_v_work, __pyx_mstate_global->__pyx_n_u_data); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 263, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_7);
  __pyx_t_13 = __Pyx_PyObject_GetAttrStr(__pyx_t_7, __pyx_mstate_global->__pyx_n_u_ptr); if (unlikely(!__pyx_t_13)) __PYX_ERR(0, 263, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_13);
  __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;

  /* "cupyx/cusolver.pyx":264
 *     solver(handle, jobz, m, n, ap.data.ptr, lda, s.data.ptr,
 *            u.data.ptr, ldu, v.data.ptr, ldv, work.data.ptr, lwork,
 *            info.data.ptr, params, batch_size)             # <<<<<<<<<<<<<<
 *     _cupy.linalg._util._check_cusolver_dev_info_if_synchronization_allowed(
 *         gesvdj, info)
*/
  __pyx_t_7 = __Pyx_PyObject_GetAttrStr(__pyx_v_info, __pyx_mstate_global->__pyx_n_u_data); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 264, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_7);
  __pyx_t_14 = __Pyx_PyObject_GetAttrStr(__pyx_t_7, __pyx_mstate_global->__pyx_n_u_ptr); if (unlikely(!__pyx_t_14)) __PYX_ERR(0, 264, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_14);
  __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
  __pyx_t_4 = 1;
  #if CYTHON_UNPACK_METHODS
  if (unlikely(PyMethod_Check(__pyx_t_3))) {
    __pyx_t_12 = PyMethod_GET_SELF(__pyx_t_3);
    assert(__pyx_t_12);
    PyObject* __pyx__function = PyMethod_GET_FUNCTION(__pyx_t_3);
    __Pyx_INCREF(__pyx_t_12);
    __Pyx_INCREF(__pyx__function);
    __Pyx_DECREF_SET(__pyx_t_3, __pyx__function);
    __pyx_t_4 = 0;
  }
  #endif
  {
    PyObject *__pyx_callargs[17] = {__pyx_t_12, __pyx_v_handle, __pyx_v_jobz, __pyx_v_m, __pyx_v_n, __pyx_t_11, __pyx_v_lda, __pyx_t_2, __pyx_t_8, __pyx_v_ldu, __pyx_t_10, __pyx_v_ldv, __pyx_t_13, __pyx_v_lwork, __pyx_t_14, __pyx_v_params, __pyx_v_batch_size};
    __pyx_t_1 = __Pyx_PyObject_FastCall((PyObject*)__pyx_t_3, __pyx_callargs+__pyx_t_4, (17-__pyx_t_4) | (__pyx_t_4*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
    __Pyx_XDECREF(__pyx_t_12); __pyx_t_12 = 0;
    __Pyx_DECREF(__pyx_t_11); __pyx_t_11 = 0;
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
    __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
    __Pyx_DECREF(__pyx_t_13); __pyx_t_13 = 0;
    __Pyx_DECREF(__pyx_t_14); __pyx_t_14 = 0;
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 262, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
  }
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "cupyx/cusolver.pyx":265
 *            u.data.ptr, ldu, v.data.ptr, ldv, work.data.ptr, lwork,
 *            info.data.ptr, params, batch_size)
 *     _cupy.linalg._util._check_cusolver_dev_info_if_synchronization_allowed(             # <<<<<<<<<<<<<<
 *         gesvdj, info)
 * 
*/
  __Pyx_GetModuleGlobalName(__pyx_t_14, __pyx_mstate_global->__pyx_n_u_cupy); if (unlikely(!__pyx_t_14)) __PYX_ERR(0, 265, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_14);
  __pyx_t_13 = __Pyx_PyObject_GetAttrStr(__pyx_t_14, __pyx_mstate_global->__pyx_n_u_linalg); if (unlikely(!__pyx_t_13)) __PYX_ERR(0, 265, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_13);
  __Pyx_DECREF(__pyx_t_14); __pyx_t_14 = 0;
  __pyx_t_14 = __Pyx_PyObject_GetAttrStr(__pyx_t_13, __pyx_mstate_global->__pyx_n_u_util); if (unlikely(!__pyx_t_14)) __PYX_ERR(0, 265, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_14);
  __Pyx_DECREF(__pyx_t_13); __pyx_t_13 = 0;
  __pyx_t_3 = __pyx_t_14;
  __Pyx_INCREF(__pyx_t_3);

  /* "cupyx/cusolver.pyx":266
 *            info.data.ptr, params, batch_size)
 *     _cupy.linalg._util._check_cusolver_dev_info_if_synchronization_allowed(
 *         gesvdj, info)             # <<<<<<<<<<<<<<
 * 
 *     _cusolver.destroyGesvdjInfo(params)
*/
  __Pyx_GetModuleGlobalName(__pyx_t_13, __pyx_mstate_global->__pyx_n_u_gesvdj); if (unlikely(!__pyx_t_13)) __PYX_ERR(0, 266, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_13);
  __pyx_t_4 = 0;
  {
    PyObject *__pyx_callargs[3] = {__pyx_t_3, __pyx_t_13, __pyx_v_info};
    __pyx_t_1 = __Pyx_PyObject_FastCallMethod((PyObject*)__pyx_mstate_global->__pyx_n_u_check_cusolver_dev_info_if_sync, __pyx_callargs+__pyx_t_4, (3-__pyx_t_4) | (1*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
    __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
    __Pyx_DECREF(__pyx_t_13); __pyx_t_13 = 0;
    __Pyx_DECREF(__pyx_t_14); __pyx_t_14 = 0;
    if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 265, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
  }
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "cupyx/cusolver.pyx":268
 *         gesvdj, info)
 * 
 *     _cusolver.destroyGesvdjInfo(params)             # <<<<<<<<<<<<<<
 *     if runtime._is_hip_environment:
 *         v = v.swapaxes(-1, -2).conj()
*/
  __pyx_t_14 = NULL;
  __Pyx_GetModuleGlobalName(__pyx_t_13, __pyx_mstate_global->__pyx_n_u_cusolver); if (unlikely(!__pyx_t_13)) __PYX_ERR(0, 268, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_13);
  __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_t_13, __pyx_mstate_global->__pyx_n_u_destroyGesvdjInfo); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 268, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_13); __pyx_t_13 = 0;
  __pyx_t_4 = 1;
  #if CYTHON_UNPACK_METHODS
  if (unlikely(PyMethod_Check(__pyx_t_3))) {
    __pyx_t_14 = PyMethod_GET_SELF(__pyx_t_3);
    assert(__pyx_t_14);
    PyObject* __pyx__function = PyMethod_GET_FUNCTION(__pyx_t_3);
    __Pyx_INCREF(__pyx_t_14);
    __Pyx_INCREF(__pyx__function);
    __Pyx_DECREF_SET(__pyx_t_3, __pyx__function);
    __pyx_t_4 = 0;
  }
  #endif
  {
    PyObject *__pyx_callargs[2] = {__pyx_t_14, __pyx_v_params};
    __pyx_t_1 = __Pyx_PyObject_FastCall((PyObject*)__pyx_t_3, __pyx_callargs+__pyx_t_4, (2-__pyx_t_4) | (__pyx_t_4*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
    __Pyx_XDECREF(__pyx_t_14); __pyx_t_14 = 0;
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 268, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
  }
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "cupyx/cusolver.pyx":269
 * 
 *     _cusolver.destroyGesvdjInfo(params)
 *     if runtime._is_hip_environment:             # <<<<<<<<<<<<<<
 *         v = v.swapaxes(-1, -2).conj()
 *     if not full_matrices:
*/
  if (__pyx_v_13cupy_backends_4cuda_3api_7runtime__is_hip_environment) {

    /* "cupyx/cusolver.pyx":270
 *     _cusolver.destroyGesvdjInfo(params)
 *     if runtime._is_hip_environment:
 *         v = v.swapaxes(-1, -2).conj()             # <<<<<<<<<<<<<<
 *     if not full_matrices:
 *         u = u[..., :mn]
*/
    __pyx_t_14 = __Pyx_PyObject_GetAttrStr(__pyx_v_v, __pyx_mstate_global->__pyx_n_u_swapaxes); if (unlikely(!__pyx_t_14)) __PYX_ERR(0, 270, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_14);
    __pyx_t_13 = __Pyx_PyObject_Call(__pyx_t_14, __pyx_mstate_global->__pyx_tuple[1], NULL); if (unlikely(!__pyx_t_13)) __PYX_ERR(0, 270, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_13);
    __Pyx_DECREF(__pyx_t_14); __pyx_t_14 = 0;
    __pyx_t_3 = __pyx_t_13;
    __Pyx_INCREF(__pyx_t_3);
    __pyx_t_4 = 0;
    {
      PyObject *__pyx_callargs[2] = {__pyx_t_3, NULL};
      __pyx_t_1 = __Pyx_PyObject_FastCallMethod((PyObject*)__pyx_mstate_global->__pyx_n_u_conj, __pyx_callargs+__pyx_t_4, (1-__pyx_t_4) | (1*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
      __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
      __Pyx_DECREF(__pyx_t_13); __pyx_t_13 = 0;
      if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 270, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
    }
    __Pyx_DECREF_SET(__pyx_v_v, __pyx_t_1);
    __pyx_t_1 = 0;

    /* "cupyx/cusolver.pyx":269
 * 
 *     _cusolver.destroyGesvdjInfo(params)
 *     if runtime._is_hip_environment:             # <<<<<<<<<<<<<<
 *         v = v.swapaxes(-1, -2).conj()
 *     if not full_matrices:
*/
  }

  /* "cupyx/cusolver.pyx":271
 *     if runtime._is_hip_environment:
 *         v = v.swapaxes(-1, -2).conj()
 *     if not full_matrices:             # <<<<<<<<<<<<<<
 *         u = u[..., :mn]
 *         v = v[..., :mn]
*/
  __pyx_t_6 = __Pyx_PyObject_IsTrue(__pyx_v_full_matrices); if (unlikely((__pyx_t_6 < 0))) __PYX_ERR(0, 271, __pyx_L1_error)
  __pyx_t_5 = (!__pyx_t_6);
  if (__pyx_t_5) {

    /* "cupyx/cusolver.pyx":272
 *         v = v.swapaxes(-1, -2).conj()
 *     if not full_matrices:
 *         u = u[..., :mn]             # <<<<<<<<<<<<<<
 *         v = v[..., :mn]
 *     if compute_uv:
*/
    __pyx_t_1 = PySlice_New(Py_None, __pyx_v_mn, Py_None); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 272, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_t_13 = PyTuple_New(2); if (unlikely(!__pyx_t_13)) __PYX_ERR(0, 272, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_13);
    __Pyx_INCREF(Py_Ellipsis);
    __Pyx_GIVEREF(Py_Ellipsis);
    if (__Pyx_PyTuple_SET_ITEM(__pyx_t_13, 0, Py_Ellipsis) != (0)) __PYX_ERR(0, 272, __pyx_L1_error);
    __Pyx_GIVEREF(__pyx_t_1);
    if (__Pyx_PyTuple_SET_ITEM(__pyx_t_13, 1, __pyx_t_1) != (0)) __PYX_ERR(0, 272, __pyx_L1_error);
    __pyx_t_1 = 0;
    __pyx_t_1 = __Pyx_PyObject_GetItem(__pyx_v_u, __pyx_t_13); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 272, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_13); __pyx_t_13 = 0;
    __Pyx_DECREF_SET(__pyx_v_u, __pyx_t_1);
    __pyx_t_1 = 0;

    /* "cupyx/cusolver.pyx":273
 *     if not full_matrices:
 *         u = u[..., :mn]
 *         v = v[..., :mn]             # <<<<<<<<<<<<<<
 *     if compute_uv:
 *         return u, s, v
*/
    __pyx_t_1 = PySlice_New(Py_None, __pyx_v_mn, Py_None); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 273, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_t_13 = PyTuple_New(2); if (unlikely(!__pyx_t_13)) __PYX_ERR(0, 273, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_13);
    __Pyx_INCREF(Py_Ellipsis);
    __Pyx_GIVEREF(Py_Ellipsis);
    if (__Pyx_PyTuple_SET_ITEM(__pyx_t_13, 0, Py_Ellipsis) != (0)) __PYX_ERR(0, 273, __pyx_L1_error);
    __Pyx_GIVEREF(__pyx_t_1);
    if (__Pyx_PyTuple_SET_ITEM(__pyx_t_13, 1, __pyx_t_1) != (0)) __PYX_ERR(0, 273, __pyx_L1_error);
    __pyx_t_1 = 0;
    __pyx_t_1 = __Pyx_PyObject_GetItem(__pyx_v_v, __pyx_t_13); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 273, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_13); __pyx_t_13 = 0;
    __Pyx_DECREF_SET(__pyx_v_v, __pyx_t_1);
    __pyx_t_1 = 0;

    /* "cupyx/cusolver.pyx":271
 *     if runtime._is_hip_environment:
 *         v = v.swapaxes(-1, -2).conj()
 *     if not full_matrices:             # <<<<<<<<<<<<<<
 *         u = u[..., :mn]
 *         v = v[..., :mn]
*/
  }

  /* "cupyx/cusolver.pyx":274
 *         u = u[..., :mn]
 *         v = v[..., :mn]
 *     if compute_uv:             # <<<<<<<<<<<<<<
 *         return u, s, v
 *     else:
*/
  __pyx_t_5 = __Pyx_PyObject_IsTrue(__pyx_v_compute_uv); if (unlikely((__pyx_t_5 < 0))) __PYX_ERR(0, 274, __pyx_L1_error)
  if (__pyx_t_5) {

    /* "cupyx/cusolver.pyx":275
 *         v = v[..., :mn]
 *     if compute_uv:
 *         return u, s, v             # <<<<<<<<<<<<<<
 *     else:
 *         return s
*/
    __Pyx_XDECREF(__pyx_r);
    __pyx_t_1 = PyTuple_New(3); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 275, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_INCREF(__pyx_v_u);
    __Pyx_GIVEREF(__pyx_v_u);
    if (__Pyx_PyTuple_SET_ITEM(__pyx_t_1, 0, __pyx_v_u) != (0)) __PYX_ERR(0, 275, __pyx_L1_error);
    __Pyx_INCREF(__pyx_v_s);
    __Pyx_GIVEREF(__pyx_v_s);
    if (__Pyx_PyTuple_SET_ITEM(__pyx_t_1, 1, __pyx_v_s) != (0)) __PYX_ERR(0, 275, __pyx_L1_error);
    __Pyx_INCREF(__pyx_v_v);
    __Pyx_GIVEREF(__pyx_v_v);
    if (__Pyx_PyTuple_SET_ITEM(__pyx_t_1, 2, __pyx_v_v) != (0)) __PYX_ERR(0, 275, __pyx_L1_error);
    __pyx_r = __pyx_t_1;
    __pyx_t_1 = 0;
    goto __pyx_L0;

    /* "cupyx/cusolver.pyx":274
 *         u = u[..., :mn]
 *         v = v[..., :mn]
 *     if compute_uv:             # <<<<<<<<<<<<<<
 *         return u, s, v
 *     else:
*/
  }

  /* "cupyx/cusolver.pyx":277
 *         return u, s, v
 *     else:
 *         return s             # <<<<<<<<<<<<<<
 * 
 * 
*/
  /*else*/ {
    __Pyx_XDECREF(__pyx_r);
    __Pyx_INCREF(__pyx_v_s);
    __pyx_r = __pyx_v_s;
    goto __pyx_L0;
  }

  /* "cupyx/cusolver.pyx":212
 * 
 * 
 * cpdef _gesvdj_batched(a, full_matrices, compute_uv, overwrite_a):             # <<<<<<<<<<<<<<
 *     if not check_availability('gesvdjBatched'):
 *         raise RuntimeError('gesvdj is not available.')
*/

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_7);
  __Pyx_XDECREF(__pyx_t_8);
  __Pyx_XDECREF(__pyx_t_10);
  __Pyx_XDECREF(__pyx_t_11);
  __Pyx_XDECREF(__pyx_t_12);
  __Pyx_XDECREF(__pyx_t_13);
  __Pyx_XDECREF(__pyx_t_14);
  __Pyx_AddTraceback("cupyx.cusolver._gesvdj_batched", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_helper);
  __Pyx_XDECREF(__pyx_v_solver);
  __Pyx_XDECREF(__pyx_v_s_dtype);
  __Pyx_XDECREF(__pyx_v_handle);
  __Pyx_XDECREF(__pyx_v_batch_size);
  __Pyx_XDECREF(__pyx_v_m);
  __Pyx_XDECREF(__pyx_v_n);
  __Pyx_XDECREF(__pyx_v_ap);
  __Pyx_XDECREF(__pyx_v_lda);
  __Pyx_XDECREF(__pyx_v_mn);
  __Pyx_XDECREF(__pyx_v_s);
  __Pyx_XDECREF(__pyx_v_ldu);
  __Pyx_XDECREF(__pyx_v_ldv);
  __Pyx_XDECREF(__pyx_v_jobz);
  __Pyx_XDECREF(__pyx_v_u);
  __Pyx_XDECREF(__pyx_v_v);
  __Pyx_XDECREF(__pyx_v_params);
  __Pyx_XDECREF(__pyx_v_lwork);
  __Pyx_XDECREF(__pyx_v_work);
  __Pyx_XDECREF(__pyx_v_info);
  __Pyx_XDECREF(__pyx_v_a);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_pw_5cupyx_8cusolver_5_gesvdj_batched(PyObject *__pyx_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
PyDoc_STRVAR(__pyx_doc_5cupyx_8cusolver_4_gesvdj_batched, "_gesvdj_batched(a, full_matrices, compute_uv, overwrite_a)");
static PyObject *__pyx_pw_5cupyx_8cusolver_5_gesvdj_batched(PyObject *__pyx_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
) {
  PyObject *__pyx_v_a = 0;
  PyObject *__pyx_v_full_matrices = 0;
  PyObject *__pyx_v_compute_uv = 0;
  PyObject *__pyx_v_overwrite_a = 0;
  #if !CYTHON_METH_FASTCALL
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  #endif
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject* values[4] = {0,0,0,0};
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("_gesvdj_batched (wrapper)", 0);
  #if !CYTHON_METH_FASTCALL
  #if CYTHON_ASSUME_SAFE_SIZE
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return NULL;
  #endif
  #endif
  __pyx_kwvalues = __Pyx_KwValues_FASTCALL(__pyx_args, __pyx_nargs);
  {
    PyObject ** const __pyx_pyargnames[] = {&__pyx_mstate_global->__pyx_n_u_a,&__pyx_mstate_global->__pyx_n_u_full_matrices,&__pyx_mstate_global->__pyx_n_u_compute_uv,&__pyx_mstate_global->__pyx_n_u_overwrite_a,0};
    const Py_ssize_t __pyx_kwds_len = (__pyx_kwds) ? __Pyx_NumKwargs_FASTCALL(__pyx_kwds) : 0;
    if (unlikely(__pyx_kwds_len < 0)) __PYX_ERR(0, 212, __pyx_L3_error)
    if (__pyx_kwds_len > 0) {
      switch (__pyx_nargs) {
        case  4:
        values[3] = __Pyx_ArgRef_FASTCALL(__pyx_args, 3);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[3])) __PYX_ERR(0, 212, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  3:
        values[2] = __Pyx_ArgRef_FASTCALL(__pyx_args, 2);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[2])) __PYX_ERR(0, 212, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  2:
        values[1] = __Pyx_ArgRef_FASTCALL(__pyx_args, 1);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[1])) __PYX_ERR(0, 212, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  1:
        values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(0, 212, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      const Py_ssize_t kwd_pos_args = __pyx_nargs;
      if (__Pyx_ParseKeywords(__pyx_kwds, __pyx_kwvalues, __pyx_pyargnames, 0, values, kwd_pos_args, __pyx_kwds_len, "_gesvdj_batched", 0) < (0)) __PYX_ERR(0, 212, __pyx_L3_error)
      for (Py_ssize_t i = __pyx_nargs; i < 4; i++) {
        if (unlikely(!values[i])) { __Pyx_RaiseArgtupleInvalid("_gesvdj_batched", 1, 4, 4, i); __PYX_ERR(0, 212, __pyx_L3_error) }
      }
    } else if (unlikely(__pyx_nargs != 4)) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
      if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(0, 212, __pyx_L3_error)
      values[1] = __Pyx_ArgRef_FASTCALL(__pyx_args, 1);
      if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[1])) __PYX_ERR(0, 212, __pyx_L3_error)
      values[2] = __Pyx_ArgRef_FASTCALL(__pyx_args, 2);
      if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[2])) __PYX_ERR(0, 212, __pyx_L3_error)
      values[3] = __Pyx_ArgRef_FASTCALL(__pyx_args, 3);
      if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[3])) __PYX_ERR(0, 212, __pyx_L3_error)
    }
    __pyx_v_a = values[0];
    __pyx_v_full_matrices = values[1];
    __pyx_v_compute_uv = values[2];
    __pyx_v_overwrite_a = values[3];
  }
  goto __pyx_L6_skip;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("_gesvdj_batched", 1, 4, 4, __pyx_nargs); __PYX_ERR(0, 212, __pyx_L3_error)
  __pyx_L6_skip:;
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_AddTraceback("cupyx.cusolver._gesvdj_batched", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_5cupyx_8cusolver_4_gesvdj_batched(__pyx_self, __pyx_v_a, __pyx_v_full_matrices, __pyx_v_compute_uv, __pyx_v_overwrite_a);

  /* function exit code */
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5cupyx_8cusolver_4_gesvdj_batched(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_a, PyObject *__pyx_v_full_matrices, PyObject *__pyx_v_compute_uv, PyObject *__pyx_v_overwrite_a) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("_gesvdj_batched", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __pyx_f_5cupyx_8cusolver__gesvdj_batched(__pyx_v_a, __pyx_v_full_matrices, __pyx_v_compute_uv, __pyx_v_overwrite_a, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 212, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("cupyx.cusolver._gesvdj_batched", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "cupyx/cusolver.pyx":280
 * 
 * 
 * cpdef _gesvd_batched(a, a_dtype, full_matrices, compute_uv, overwrite_a):             # <<<<<<<<<<<<<<
 *     """A loop-based gesvd wrapper to support batched SVD."""
 *     # This function follows more closely with gesvd() in
*/

static PyObject *__pyx_pw_5cupyx_8cusolver_7_gesvd_batched(PyObject *__pyx_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
static PyObject *__pyx_f_5cupyx_8cusolver__gesvd_batched(PyObject *__pyx_v_a, PyObject *__pyx_v_a_dtype, PyObject *__pyx_v_full_matrices, PyObject *__pyx_v_compute_uv, PyObject *__pyx_v_overwrite_a, CYTHON_UNUSED int __pyx_skip_dispatch) {
  struct __pyx_obj_4cupy_5_core_4core__ndarray_base *__pyx_v_x = 0;
  struct __pyx_obj_4cupy_5_core_4core__ndarray_base *__pyx_v_s = 0;
  struct __pyx_obj_4cupy_5_core_4core__ndarray_base *__pyx_v_u = 0;
  struct __pyx_obj_4cupy_5_core_4core__ndarray_base *__pyx_v_vt = 0;
  struct __pyx_obj_4cupy_5_core_4core__ndarray_base *__pyx_v_dev_info = 0;
  int __pyx_v_n;
  int __pyx_v_m;
  int __pyx_v_k;
  int __pyx_v_batch_size;
  int __pyx_v_buffersize;
  int __pyx_v_status;
  intptr_t __pyx_v_a_ptr;
  intptr_t __pyx_v_s_ptr;
  intptr_t __pyx_v_u_ptr;
  intptr_t __pyx_v_vt_ptr;
  intptr_t __pyx_v_w_ptr;
  intptr_t __pyx_v_info_ptr;
  PyObject *__pyx_v_s_dtype = 0;
  char __pyx_v_job_u;
  char __pyx_v_job_vt;
  int __pyx_v_trans_flag;
  __pyx_t_5cupyx_8cusolver_gesvd_ptr __pyx_v_gesvd;
  intptr_t __pyx_v_handle;
  int (*__pyx_v_gesvd_bufferSize)(intptr_t, int, int, int __pyx_skip_dispatch);
  struct __pyx_obj_4cupy_4cuda_6memory_MemoryPointer *__pyx_v_workspace = NULL;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  size_t __pyx_t_3;
  int __pyx_t_4;
  int __pyx_t_5;
  PyObject *__pyx_t_6 = NULL;
  PyObject *__pyx_t_7 = NULL;
  PyObject *__pyx_t_8 = NULL;
  PyObject *(*__pyx_t_9)(PyObject *);
  int __pyx_t_10;
  int __pyx_t_11;
  int __pyx_t_12;
  intptr_t __pyx_t_13;
  __pyx_t_4cupy_5_core_7_carray_shape_t __pyx_t_14;
  intptr_t __pyx_t_15;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("_gesvd_batched", 0);

  /* "cupyx/cusolver.pyx":288
 *     # it here and unify with rocSOLVER's counterpart (it's currently wrapped
 *     # as gesvdj, not gesvd) here.
 *     if runtime._is_hip_environment:             # <<<<<<<<<<<<<<
 *         # we never arrive here, so just raise
 *         raise RuntimeError("This function is disabled on HIP as "
*/
  if (unlikely(__pyx_v_13cupy_backends_4cuda_3api_7runtime__is_hip_environment)) {

    /* "cupyx/cusolver.pyx":290
 *     if runtime._is_hip_environment:
 *         # we never arrive here, so just raise
 *         raise RuntimeError("This function is disabled on HIP as "             # <<<<<<<<<<<<<<
 *                            "it is not needed")
 * 
*/
    __pyx_t_2 = NULL;
    __pyx_t_3 = 1;
    {
      PyObject *__pyx_callargs[2] = {__pyx_t_2, __pyx_mstate_global->__pyx_kp_u_This_function_is_disabled_on_HIP};
      __pyx_t_1 = __Pyx_PyObject_FastCall((PyObject*)(((PyTypeObject*)PyExc_RuntimeError)), __pyx_callargs+__pyx_t_3, (2-__pyx_t_3) | (__pyx_t_3*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
      __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
      if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 290, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
    }
    __Pyx_Raise(__pyx_t_1, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __PYX_ERR(0, 290, __pyx_L1_error)

    /* "cupyx/cusolver.pyx":288
 *     # it here and unify with rocSOLVER's counterpart (it's currently wrapped
 *     # as gesvdj, not gesvd) here.
 *     if runtime._is_hip_environment:             # <<<<<<<<<<<<<<
 *         # we never arrive here, so just raise
 *         raise RuntimeError("This function is disabled on HIP as "
*/
  }

  /* "cupyx/cusolver.pyx":303
 *     cdef gesvd_ptr gesvd
 * 
 *     assert a.ndim > 2             # <<<<<<<<<<<<<<
 *     assert not overwrite_a  # TODO(leofang): handle this?
 *     batch_size, n, m = a.shape
*/
  #ifndef CYTHON_WITHOUT_ASSERTIONS
  if (unlikely(__pyx_assertions_enabled())) {
    __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_v_a, __pyx_mstate_global->__pyx_n_u_ndim); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 303, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_t_2 = PyObject_RichCompare(__pyx_t_1, __pyx_mstate_global->__pyx_int_2, Py_GT); __Pyx_XGOTREF(__pyx_t_2); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 303, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __pyx_t_4 = __Pyx_PyObject_IsTrue(__pyx_t_2); if (unlikely((__pyx_t_4 < 0))) __PYX_ERR(0, 303, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    if (unlikely(!__pyx_t_4)) {
      __Pyx_Raise(((PyObject *)(((PyTypeObject*)PyExc_AssertionError))), 0, 0, 0);
      __PYX_ERR(0, 303, __pyx_L1_error)
    }
  }
  #else
  if ((1)); else __PYX_ERR(0, 303, __pyx_L1_error)
  #endif

  /* "cupyx/cusolver.pyx":304
 * 
 *     assert a.ndim > 2
 *     assert not overwrite_a  # TODO(leofang): handle this?             # <<<<<<<<<<<<<<
 *     batch_size, n, m = a.shape
 *     s_dtype = a_dtype.lower()
*/
  #ifndef CYTHON_WITHOUT_ASSERTIONS
  if (unlikely(__pyx_assertions_enabled())) {
    __pyx_t_4 = __Pyx_PyObject_IsTrue(__pyx_v_overwrite_a); if (unlikely((__pyx_t_4 < 0))) __PYX_ERR(0, 304, __pyx_L1_error)
    __pyx_t_5 = (!__pyx_t_4);
    if (unlikely(!__pyx_t_5)) {
      __Pyx_Raise(((PyObject *)(((PyTypeObject*)PyExc_AssertionError))), 0, 0, 0);
      __PYX_ERR(0, 304, __pyx_L1_error)
    }
  }
  #else
  if ((1)); else __PYX_ERR(0, 304, __pyx_L1_error)
  #endif

  /* "cupyx/cusolver.pyx":305
 *     assert a.ndim > 2
 *     assert not overwrite_a  # TODO(leofang): handle this?
 *     batch_size, n, m = a.shape             # <<<<<<<<<<<<<<
 *     s_dtype = a_dtype.lower()
 * 
*/
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_v_a, __pyx_mstate_global->__pyx_n_u_shape); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 305, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if ((likely(PyTuple_CheckExact(__pyx_t_2))) || (PyList_CheckExact(__pyx_t_2))) {
    PyObject* sequence = __pyx_t_2;
    Py_ssize_t size = __Pyx_PySequence_SIZE(sequence);
    if (unlikely(size != 3)) {
      if (size > 3) __Pyx_RaiseTooManyValuesError(3);
      else if (size >= 0) __Pyx_RaiseNeedMoreValuesError(size);
      __PYX_ERR(0, 305, __pyx_L1_error)
    }
    #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
    if (likely(PyTuple_CheckExact(sequence))) {
      __pyx_t_1 = PyTuple_GET_ITEM(sequence, 0);
      __Pyx_INCREF(__pyx_t_1);
      __pyx_t_6 = PyTuple_GET_ITEM(sequence, 1);
      __Pyx_INCREF(__pyx_t_6);
      __pyx_t_7 = PyTuple_GET_ITEM(sequence, 2);
      __Pyx_INCREF(__pyx_t_7);
    } else {
      __pyx_t_1 = __Pyx_PyList_GetItemRefFast(sequence, 0, __Pyx_ReferenceSharing_SharedReference);
      if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 305, __pyx_L1_error)
      __Pyx_XGOTREF(__pyx_t_1);
      __pyx_t_6 = __Pyx_PyList_GetItemRefFast(sequence, 1, __Pyx_ReferenceSharing_SharedReference);
      if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 305, __pyx_L1_error)
      __Pyx_XGOTREF(__pyx_t_6);
      __pyx_t_7 = __Pyx_PyList_GetItemRefFast(sequence, 2, __Pyx_ReferenceSharing_SharedReference);
      if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 305, __pyx_L1_error)
      __Pyx_XGOTREF(__pyx_t_7);
    }
    #else
    __pyx_t_1 = __Pyx_PySequence_ITEM(sequence, 0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 305, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_t_6 = __Pyx_PySequence_ITEM(sequence, 1); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 305, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_6);
    __pyx_t_7 = __Pyx_PySequence_ITEM(sequence, 2); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 305, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_7);
    #endif
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  } else {
    Py_ssize_t index = -1;
    __pyx_t_8 = PyObject_GetIter(__pyx_t_2); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 305, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_8);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __pyx_t_9 = (CYTHON_COMPILING_IN_LIMITED_API) ? PyIter_Next : __Pyx_PyObject_GetIterNextFunc(__pyx_t_8);
    index = 0; __pyx_t_1 = __pyx_t_9(__pyx_t_8); if (unlikely(!__pyx_t_1)) goto __pyx_L4_unpacking_failed;
    __Pyx_GOTREF(__pyx_t_1);
    index = 1; __pyx_t_6 = __pyx_t_9(__pyx_t_8); if (unlikely(!__pyx_t_6)) goto __pyx_L4_unpacking_failed;
    __Pyx_GOTREF(__pyx_t_6);
    index = 2; __pyx_t_7 = __pyx_t_9(__pyx_t_8); if (unlikely(!__pyx_t_7)) goto __pyx_L4_unpacking_failed;
    __Pyx_GOTREF(__pyx_t_7);
    if (__Pyx_IternextUnpackEndCheck(__pyx_t_9(__pyx_t_8), 3) < (0)) __PYX_ERR(0, 305, __pyx_L1_error)
    __pyx_t_9 = NULL;
    __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
    goto __pyx_L5_unpacking_done;
    __pyx_L4_unpacking_failed:;
    __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
    __pyx_t_9 = NULL;
    if (__Pyx_IterFinish() == 0) __Pyx_RaiseNeedMoreValuesError(index);
    __PYX_ERR(0, 305, __pyx_L1_error)
    __pyx_L5_unpacking_done:;
  }
  __pyx_t_10 = __Pyx_PyLong_As_int(__pyx_t_1); if (unlikely((__pyx_t_10 == (int)-1) && PyErr_Occurred())) __PYX_ERR(0, 305, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_11 = __Pyx_PyLong_As_int(__pyx_t_6); if (unlikely((__pyx_t_11 == (int)-1) && PyErr_Occurred())) __PYX_ERR(0, 305, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
  __pyx_t_12 = __Pyx_PyLong_As_int(__pyx_t_7); if (unlikely((__pyx_t_12 == (int)-1) && PyErr_Occurred())) __PYX_ERR(0, 305, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
  __pyx_v_batch_size = __pyx_t_10;
  __pyx_v_n = __pyx_t_11;
  __pyx_v_m = __pyx_t_12;

  /* "cupyx/cusolver.pyx":306
 *     assert not overwrite_a  # TODO(leofang): handle this?
 *     batch_size, n, m = a.shape
 *     s_dtype = a_dtype.lower()             # <<<<<<<<<<<<<<
 * 
 *     # `a` must be copied because xgesvd destroys the matrix
*/
  __pyx_t_7 = __pyx_v_a_dtype;
  __Pyx_INCREF(__pyx_t_7);
  __pyx_t_3 = 0;
  {
    PyObject *__pyx_callargs[2] = {__pyx_t_7, NULL};
    __pyx_t_2 = __Pyx_PyObject_FastCallMethod((PyObject*)__pyx_mstate_global->__pyx_n_u_lower, __pyx_callargs+__pyx_t_3, (1-__pyx_t_3) | (1*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
    __Pyx_XDECREF(__pyx_t_7); __pyx_t_7 = 0;
    if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 306, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
  }
  if (!(likely(PyUnicode_CheckExact(__pyx_t_2))||((__pyx_t_2) == Py_None) || __Pyx_RaiseUnexpectedTypeError("str", __pyx_t_2))) __PYX_ERR(0, 306, __pyx_L1_error)
  __pyx_v_s_dtype = ((PyObject*)__pyx_t_2);
  __pyx_t_2 = 0;

  /* "cupyx/cusolver.pyx":309
 * 
 *     # `a` must be copied because xgesvd destroys the matrix
 *     if m >= n:             # <<<<<<<<<<<<<<
 *         x = a.astype(a_dtype, order='C', copy=True)
 *         trans_flag = False
*/
  __pyx_t_5 = (__pyx_v_m >= __pyx_v_n);
  if (__pyx_t_5) {

    /* "cupyx/cusolver.pyx":310
 *     # `a` must be copied because xgesvd destroys the matrix
 *     if m >= n:
 *         x = a.astype(a_dtype, order='C', copy=True)             # <<<<<<<<<<<<<<
 *         trans_flag = False
 *     else:
*/
    __pyx_t_7 = __pyx_v_a;
    __Pyx_INCREF(__pyx_t_7);
    __pyx_t_3 = 0;
    {
      PyObject *__pyx_callargs[2 + ((CYTHON_VECTORCALL) ? 2 : 0)] = {__pyx_t_7, __pyx_v_a_dtype};
      __pyx_t_6 = __Pyx_MakeVectorcallBuilderKwds(2); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 310, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_6);
      if (__Pyx_VectorcallBuilder_AddArg(__pyx_mstate_global->__pyx_n_u_order, __pyx_mstate_global->__pyx_n_u_C, __pyx_t_6, __pyx_callargs+2, 0) < (0)) __PYX_ERR(0, 310, __pyx_L1_error)
      if (__Pyx_VectorcallBuilder_AddArg(__pyx_mstate_global->__pyx_n_u_copy, Py_True, __pyx_t_6, __pyx_callargs+2, 1) < (0)) __PYX_ERR(0, 310, __pyx_L1_error)
      __pyx_t_2 = __Pyx_Object_VectorcallMethod_CallFromBuilder((PyObject*)__pyx_mstate_global->__pyx_n_u_astype, __pyx_callargs+__pyx_t_3, (2-__pyx_t_3) | (1*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET), __pyx_t_6);
      __Pyx_XDECREF(__pyx_t_7); __pyx_t_7 = 0;
      __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
      if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 310, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_2);
    }
    if (!(likely(((__pyx_t_2) == Py_None) || likely(__Pyx_TypeTest(__pyx_t_2, __pyx_mstate_global->__pyx_ptype_4cupy_5_core_4core__ndarray_base))))) __PYX_ERR(0, 310, __pyx_L1_error)
    __pyx_v_x = ((struct __pyx_obj_4cupy_5_core_4core__ndarray_base *)__pyx_t_2);
    __pyx_t_2 = 0;

    /* "cupyx/cusolver.pyx":311
 *     if m >= n:
 *         x = a.astype(a_dtype, order='C', copy=True)
 *         trans_flag = False             # <<<<<<<<<<<<<<
 *     else:
 *         m, n = a.shape[-2:]
*/
    __pyx_v_trans_flag = 0;

    /* "cupyx/cusolver.pyx":309
 * 
 *     # `a` must be copied because xgesvd destroys the matrix
 *     if m >= n:             # <<<<<<<<<<<<<<
 *         x = a.astype(a_dtype, order='C', copy=True)
 *         trans_flag = False
*/
    goto __pyx_L6;
  }

  /* "cupyx/cusolver.pyx":313
 *         trans_flag = False
 *     else:
 *         m, n = a.shape[-2:]             # <<<<<<<<<<<<<<
 *         x = a.swapaxes(-2, -1).astype(a_dtype, order='C', copy=True)
 *         trans_flag = True
*/
  /*else*/ {
    __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_v_a, __pyx_mstate_global->__pyx_n_u_shape); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 313, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_6 = __Pyx_PyObject_GetSlice(__pyx_t_2, -2L, 0, NULL, NULL, &__pyx_mstate_global->__pyx_slice[0], 1, 0, 1); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 313, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_6);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    if ((likely(PyTuple_CheckExact(__pyx_t_6))) || (PyList_CheckExact(__pyx_t_6))) {
      PyObject* sequence = __pyx_t_6;
      Py_ssize_t size = __Pyx_PySequence_SIZE(sequence);
      if (unlikely(size != 2)) {
        if (size > 2) __Pyx_RaiseTooManyValuesError(2);
        else if (size >= 0) __Pyx_RaiseNeedMoreValuesError(size);
        __PYX_ERR(0, 313, __pyx_L1_error)
      }
      #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
      if (likely(PyTuple_CheckExact(sequence))) {
        __pyx_t_2 = PyTuple_GET_ITEM(sequence, 0);
        __Pyx_INCREF(__pyx_t_2);
        __pyx_t_7 = PyTuple_GET_ITEM(sequence, 1);
        __Pyx_INCREF(__pyx_t_7);
      } else {
        __pyx_t_2 = __Pyx_PyList_GetItemRefFast(sequence, 0, __Pyx_ReferenceSharing_SharedReference);
        if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 313, __pyx_L1_error)
        __Pyx_XGOTREF(__pyx_t_2);
        __pyx_t_7 = __Pyx_PyList_GetItemRefFast(sequence, 1, __Pyx_ReferenceSharing_SharedReference);
        if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 313, __pyx_L1_error)
        __Pyx_XGOTREF(__pyx_t_7);
      }
      #else
      __pyx_t_2 = __Pyx_PySequence_ITEM(sequence, 0); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 313, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_2);
      __pyx_t_7 = __Pyx_PySequence_ITEM(sequence, 1); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 313, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_7);
      #endif
      __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
    } else {
      Py_ssize_t index = -1;
      __pyx_t_1 = PyObject_GetIter(__pyx_t_6); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 313, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
      __pyx_t_9 = (CYTHON_COMPILING_IN_LIMITED_API) ? PyIter_Next : __Pyx_PyObject_GetIterNextFunc(__pyx_t_1);
      index = 0; __pyx_t_2 = __pyx_t_9(__pyx_t_1); if (unlikely(!__pyx_t_2)) goto __pyx_L7_unpacking_failed;
      __Pyx_GOTREF(__pyx_t_2);
      index = 1; __pyx_t_7 = __pyx_t_9(__pyx_t_1); if (unlikely(!__pyx_t_7)) goto __pyx_L7_unpacking_failed;
      __Pyx_GOTREF(__pyx_t_7);
      if (__Pyx_IternextUnpackEndCheck(__pyx_t_9(__pyx_t_1), 2) < (0)) __PYX_ERR(0, 313, __pyx_L1_error)
      __pyx_t_9 = NULL;
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      goto __pyx_L8_unpacking_done;
      __pyx_L7_unpacking_failed:;
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      __pyx_t_9 = NULL;
      if (__Pyx_IterFinish() == 0) __Pyx_RaiseNeedMoreValuesError(index);
      __PYX_ERR(0, 313, __pyx_L1_error)
      __pyx_L8_unpacking_done:;
    }
    __pyx_t_12 = __Pyx_PyLong_As_int(__pyx_t_2); if (unlikely((__pyx_t_12 == (int)-1) && PyErr_Occurred())) __PYX_ERR(0, 313, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __pyx_t_11 = __Pyx_PyLong_As_int(__pyx_t_7); if (unlikely((__pyx_t_11 == (int)-1) && PyErr_Occurred())) __PYX_ERR(0, 313, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
    __pyx_v_m = __pyx_t_12;
    __pyx_v_n = __pyx_t_11;

    /* "cupyx/cusolver.pyx":314
 *     else:
 *         m, n = a.shape[-2:]
 *         x = a.swapaxes(-2, -1).astype(a_dtype, order='C', copy=True)             # <<<<<<<<<<<<<<
 *         trans_flag = True
 *     a_ptr = x.data.ptr
*/
    __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_v_a, __pyx_mstate_global->__pyx_n_u_swapaxes); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 314, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_2, __pyx_mstate_global->__pyx_tuple[0], NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 314, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __pyx_t_7 = __pyx_t_1;
    __Pyx_INCREF(__pyx_t_7);
    __pyx_t_3 = 0;
    {
      PyObject *__pyx_callargs[2 + ((CYTHON_VECTORCALL) ? 2 : 0)] = {__pyx_t_7, __pyx_v_a_dtype};
      __pyx_t_2 = __Pyx_MakeVectorcallBuilderKwds(2); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 314, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_2);
      if (__Pyx_VectorcallBuilder_AddArg(__pyx_mstate_global->__pyx_n_u_order, __pyx_mstate_global->__pyx_n_u_C, __pyx_t_2, __pyx_callargs+2, 0) < (0)) __PYX_ERR(0, 314, __pyx_L1_error)
      if (__Pyx_VectorcallBuilder_AddArg(__pyx_mstate_global->__pyx_n_u_copy, Py_True, __pyx_t_2, __pyx_callargs+2, 1) < (0)) __PYX_ERR(0, 314, __pyx_L1_error)
      __pyx_t_6 = __Pyx_Object_VectorcallMethod_CallFromBuilder((PyObject*)__pyx_mstate_global->__pyx_n_u_astype, __pyx_callargs+__pyx_t_3, (2-__pyx_t_3) | (1*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET), __pyx_t_2);
      __Pyx_XDECREF(__pyx_t_7); __pyx_t_7 = 0;
      __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 314, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_6);
    }
    if (!(likely(((__pyx_t_6) == Py_None) || likely(__Pyx_TypeTest(__pyx_t_6, __pyx_mstate_global->__pyx_ptype_4cupy_5_core_4core__ndarray_base))))) __PYX_ERR(0, 314, __pyx_L1_error)
    __pyx_v_x = ((struct __pyx_obj_4cupy_5_core_4core__ndarray_base *)__pyx_t_6);
    __pyx_t_6 = 0;

    /* "cupyx/cusolver.pyx":315
 *         m, n = a.shape[-2:]
 *         x = a.swapaxes(-2, -1).astype(a_dtype, order='C', copy=True)
 *         trans_flag = True             # <<<<<<<<<<<<<<
 *     a_ptr = x.data.ptr
 * 
*/
    __pyx_v_trans_flag = 1;
  }
  __pyx_L6:;

  /* "cupyx/cusolver.pyx":316
 *         x = a.swapaxes(-2, -1).astype(a_dtype, order='C', copy=True)
 *         trans_flag = True
 *     a_ptr = x.data.ptr             # <<<<<<<<<<<<<<
 * 
 *     k = n  # = min(m, n) where m >= n is ensured above
*/
  __pyx_t_13 = __pyx_v_x->data->ptr;
  __pyx_v_a_ptr = __pyx_t_13;

  /* "cupyx/cusolver.pyx":318
 *     a_ptr = x.data.ptr
 * 
 *     k = n  # = min(m, n) where m >= n is ensured above             # <<<<<<<<<<<<<<
 *     if compute_uv:
 *         if full_matrices:
*/
  __pyx_v_k = __pyx_v_n;

  /* "cupyx/cusolver.pyx":319
 * 
 *     k = n  # = min(m, n) where m >= n is ensured above
 *     if compute_uv:             # <<<<<<<<<<<<<<
 *         if full_matrices:
 *             u = _ndarray_init(_cupy.ndarray, (batch_size, m, m), a_dtype, None)
*/
  __pyx_t_5 = __Pyx_PyObject_IsTrue(__pyx_v_compute_uv); if (unlikely((__pyx_t_5 < 0))) __PYX_ERR(0, 319, __pyx_L1_error)
  if (__pyx_t_5) {

    /* "cupyx/cusolver.pyx":320
 *     k = n  # = min(m, n) where m >= n is ensured above
 *     if compute_uv:
 *         if full_matrices:             # <<<<<<<<<<<<<<
 *             u = _ndarray_init(_cupy.ndarray, (batch_size, m, m), a_dtype, None)
 *             vt = x[..., :n]
*/
    __pyx_t_5 = __Pyx_PyObject_IsTrue(__pyx_v_full_matrices); if (unlikely((__pyx_t_5 < 0))) __PYX_ERR(0, 320, __pyx_L1_error)
    if (__pyx_t_5) {

      /* "cupyx/cusolver.pyx":321
 *     if compute_uv:
 *         if full_matrices:
 *             u = _ndarray_init(_cupy.ndarray, (batch_size, m, m), a_dtype, None)             # <<<<<<<<<<<<<<
 *             vt = x[..., :n]
 *             job_u = b'A'
*/
      __Pyx_GetModuleGlobalName(__pyx_t_6, __pyx_mstate_global->__pyx_n_u_cupy); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 321, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_6);
      __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_t_6, __pyx_mstate_global->__pyx_n_u_ndarray); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 321, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
      __pyx_t_6 = __Pyx_PyLong_From_int(__pyx_v_batch_size); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 321, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_6);
      __pyx_t_2 = __Pyx_PyLong_From_int(__pyx_v_m); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 321, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_2);
      __pyx_t_7 = __Pyx_PyLong_From_int(__pyx_v_m); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 321, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_7);
      __pyx_t_8 = PyTuple_New(3); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 321, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_8);
      __Pyx_GIVEREF(__pyx_t_6);
      if (__Pyx_PyTuple_SET_ITEM(__pyx_t_8, 0, __pyx_t_6) != (0)) __PYX_ERR(0, 321, __pyx_L1_error);
      __Pyx_GIVEREF(__pyx_t_2);
      if (__Pyx_PyTuple_SET_ITEM(__pyx_t_8, 1, __pyx_t_2) != (0)) __PYX_ERR(0, 321, __pyx_L1_error);
      __Pyx_GIVEREF(__pyx_t_7);
      if (__Pyx_PyTuple_SET_ITEM(__pyx_t_8, 2, __pyx_t_7) != (0)) __PYX_ERR(0, 321, __pyx_L1_error);
      __pyx_t_6 = 0;
      __pyx_t_2 = 0;
      __pyx_t_7 = 0;
      __pyx_t_14 = __pyx_convert_vector_from_py_Py_ssize_t(__pyx_t_8); if (unlikely(PyErr_Occurred())) __PYX_ERR(0, 321, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
      __pyx_t_8 = ((PyObject *)__pyx_f_4cupy_5_core_4core__ndarray_init(__pyx_t_1, __pyx_t_14, __pyx_v_a_dtype, Py_None)); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 321, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_8);
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      __pyx_v_u = ((struct __pyx_obj_4cupy_5_core_4core__ndarray_base *)__pyx_t_8);
      __pyx_t_8 = 0;

      /* "cupyx/cusolver.pyx":322
 *         if full_matrices:
 *             u = _ndarray_init(_cupy.ndarray, (batch_size, m, m), a_dtype, None)
 *             vt = x[..., :n]             # <<<<<<<<<<<<<<
 *             job_u = b'A'
 *             job_vt = b'O'
*/
      __pyx_t_8 = __Pyx_PyLong_From_int(__pyx_v_n); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 322, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_8);
      __pyx_t_1 = PySlice_New(Py_None, __pyx_t_8, Py_None); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 322, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
      __pyx_t_8 = PyTuple_New(2); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 322, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_8);
      __Pyx_INCREF(Py_Ellipsis);
      __Pyx_GIVEREF(Py_Ellipsis);
      if (__Pyx_PyTuple_SET_ITEM(__pyx_t_8, 0, Py_Ellipsis) != (0)) __PYX_ERR(0, 322, __pyx_L1_error);
      __Pyx_GIVEREF(__pyx_t_1);
      if (__Pyx_PyTuple_SET_ITEM(__pyx_t_8, 1, __pyx_t_1) != (0)) __PYX_ERR(0, 322, __pyx_L1_error);
      __pyx_t_1 = 0;
      __pyx_t_1 = __Pyx_PyObject_GetItem(((PyObject *)__pyx_v_x), __pyx_t_8); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 322, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
      if (!(likely(((__pyx_t_1) == Py_None) || likely(__Pyx_TypeTest(__pyx_t_1, __pyx_mstate_global->__pyx_ptype_4cupy_5_core_4core__ndarray_base))))) __PYX_ERR(0, 322, __pyx_L1_error)
      __pyx_v_vt = ((struct __pyx_obj_4cupy_5_core_4core__ndarray_base *)__pyx_t_1);
      __pyx_t_1 = 0;

      /* "cupyx/cusolver.pyx":323
 *             u = _ndarray_init(_cupy.ndarray, (batch_size, m, m), a_dtype, None)
 *             vt = x[..., :n]
 *             job_u = b'A'             # <<<<<<<<<<<<<<
 *             job_vt = b'O'
 *             u_ptr, vt_ptr = u.data.ptr, 0
*/
      __pyx_v_job_u = 'A';

      /* "cupyx/cusolver.pyx":324
 *             vt = x[..., :n]
 *             job_u = b'A'
 *             job_vt = b'O'             # <<<<<<<<<<<<<<
 *             u_ptr, vt_ptr = u.data.ptr, 0
 *         else:
*/
      __pyx_v_job_vt = 'O';

      /* "cupyx/cusolver.pyx":325
 *             job_u = b'A'
 *             job_vt = b'O'
 *             u_ptr, vt_ptr = u.data.ptr, 0             # <<<<<<<<<<<<<<
 *         else:
 *             u = x
*/
      __pyx_t_13 = __pyx_v_u->data->ptr;
      __pyx_t_15 = 0;
      __pyx_v_u_ptr = __pyx_t_13;
      __pyx_v_vt_ptr = __pyx_t_15;

      /* "cupyx/cusolver.pyx":320
 *     k = n  # = min(m, n) where m >= n is ensured above
 *     if compute_uv:
 *         if full_matrices:             # <<<<<<<<<<<<<<
 *             u = _ndarray_init(_cupy.ndarray, (batch_size, m, m), a_dtype, None)
 *             vt = x[..., :n]
*/
      goto __pyx_L10;
    }

    /* "cupyx/cusolver.pyx":327
 *             u_ptr, vt_ptr = u.data.ptr, 0
 *         else:
 *             u = x             # <<<<<<<<<<<<<<
 *             vt = _ndarray_init(
 *                 _cupy.ndarray, (batch_size, k, n), a_dtype, None)
*/
    /*else*/ {
      __Pyx_INCREF((PyObject *)__pyx_v_x);
      __pyx_v_u = __pyx_v_x;

      /* "cupyx/cusolver.pyx":329
 *             u = x
 *             vt = _ndarray_init(
 *                 _cupy.ndarray, (batch_size, k, n), a_dtype, None)             # <<<<<<<<<<<<<<
 *             job_u = b'O'
 *             job_vt = b'S'
*/
      __Pyx_GetModuleGlobalName(__pyx_t_1, __pyx_mstate_global->__pyx_n_u_cupy); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 329, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      __pyx_t_8 = __Pyx_PyObject_GetAttrStr(__pyx_t_1, __pyx_mstate_global->__pyx_n_u_ndarray); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 329, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_8);
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      __pyx_t_1 = __Pyx_PyLong_From_int(__pyx_v_batch_size); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 329, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      __pyx_t_7 = __Pyx_PyLong_From_int(__pyx_v_k); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 329, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_7);
      __pyx_t_2 = __Pyx_PyLong_From_int(__pyx_v_n); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 329, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_2);
      __pyx_t_6 = PyTuple_New(3); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 329, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_6);
      __Pyx_GIVEREF(__pyx_t_1);
      if (__Pyx_PyTuple_SET_ITEM(__pyx_t_6, 0, __pyx_t_1) != (0)) __PYX_ERR(0, 329, __pyx_L1_error);
      __Pyx_GIVEREF(__pyx_t_7);
      if (__Pyx_PyTuple_SET_ITEM(__pyx_t_6, 1, __pyx_t_7) != (0)) __PYX_ERR(0, 329, __pyx_L1_error);
      __Pyx_GIVEREF(__pyx_t_2);
      if (__Pyx_PyTuple_SET_ITEM(__pyx_t_6, 2, __pyx_t_2) != (0)) __PYX_ERR(0, 329, __pyx_L1_error);
      __pyx_t_1 = 0;
      __pyx_t_7 = 0;
      __pyx_t_2 = 0;
      __pyx_t_14 = __pyx_convert_vector_from_py_Py_ssize_t(__pyx_t_6); if (unlikely(PyErr_Occurred())) __PYX_ERR(0, 329, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;

      /* "cupyx/cusolver.pyx":328
 *         else:
 *             u = x
 *             vt = _ndarray_init(             # <<<<<<<<<<<<<<
 *                 _cupy.ndarray, (batch_size, k, n), a_dtype, None)
 *             job_u = b'O'
*/
      __pyx_t_6 = ((PyObject *)__pyx_f_4cupy_5_core_4core__ndarray_init(__pyx_t_8, __pyx_t_14, __pyx_v_a_dtype, Py_None)); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 328, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_6);
      __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
      __pyx_v_vt = ((struct __pyx_obj_4cupy_5_core_4core__ndarray_base *)__pyx_t_6);
      __pyx_t_6 = 0;

      /* "cupyx/cusolver.pyx":330
 *             vt = _ndarray_init(
 *                 _cupy.ndarray, (batch_size, k, n), a_dtype, None)
 *             job_u = b'O'             # <<<<<<<<<<<<<<
 *             job_vt = b'S'
 *             u_ptr, vt_ptr = 0, vt.data.ptr
*/
      __pyx_v_job_u = 'O';

      /* "cupyx/cusolver.pyx":331
 *                 _cupy.ndarray, (batch_size, k, n), a_dtype, None)
 *             job_u = b'O'
 *             job_vt = b'S'             # <<<<<<<<<<<<<<
 *             u_ptr, vt_ptr = 0, vt.data.ptr
 *     else:
*/
      __pyx_v_job_vt = 'S';

      /* "cupyx/cusolver.pyx":332
 *             job_u = b'O'
 *             job_vt = b'S'
 *             u_ptr, vt_ptr = 0, vt.data.ptr             # <<<<<<<<<<<<<<
 *     else:
 *         u_ptr, vt_ptr = 0, 0  # Use nullptr
*/
      __pyx_t_15 = 0;
      __pyx_t_13 = __pyx_v_vt->data->ptr;
      __pyx_v_u_ptr = __pyx_t_15;
      __pyx_v_vt_ptr = __pyx_t_13;
    }
    __pyx_L10:;

    /* "cupyx/cusolver.pyx":319
 * 
 *     k = n  # = min(m, n) where m >= n is ensured above
 *     if compute_uv:             # <<<<<<<<<<<<<<
 *         if full_matrices:
 *             u = _ndarray_init(_cupy.ndarray, (batch_size, m, m), a_dtype, None)
*/
    goto __pyx_L9;
  }

  /* "cupyx/cusolver.pyx":334
 *             u_ptr, vt_ptr = 0, vt.data.ptr
 *     else:
 *         u_ptr, vt_ptr = 0, 0  # Use nullptr             # <<<<<<<<<<<<<<
 *         job_u = b'N'
 *         job_vt = b'N'
*/
  /*else*/ {
    __pyx_t_13 = 0;
    __pyx_t_15 = 0;
    __pyx_v_u_ptr = __pyx_t_13;
    __pyx_v_vt_ptr = __pyx_t_15;

    /* "cupyx/cusolver.pyx":335
 *     else:
 *         u_ptr, vt_ptr = 0, 0  # Use nullptr
 *         job_u = b'N'             # <<<<<<<<<<<<<<
 *         job_vt = b'N'
 *     s = _ndarray_init(_cupy.ndarray, (batch_size, k), s_dtype, None)
*/
    __pyx_v_job_u = 'N';

    /* "cupyx/cusolver.pyx":336
 *         u_ptr, vt_ptr = 0, 0  # Use nullptr
 *         job_u = b'N'
 *         job_vt = b'N'             # <<<<<<<<<<<<<<
 *     s = _ndarray_init(_cupy.ndarray, (batch_size, k), s_dtype, None)
 *     s_ptr = s.data.ptr
*/
    __pyx_v_job_vt = 'N';
  }
  __pyx_L9:;

  /* "cupyx/cusolver.pyx":337
 *         job_u = b'N'
 *         job_vt = b'N'
 *     s = _ndarray_init(_cupy.ndarray, (batch_size, k), s_dtype, None)             # <<<<<<<<<<<<<<
 *     s_ptr = s.data.ptr
 *     cdef intptr_t handle = _device.get_cusolver_handle()
*/
  __Pyx_GetModuleGlobalName(__pyx_t_6, __pyx_mstate_global->__pyx_n_u_cupy); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 337, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_6);
  __pyx_t_8 = __Pyx_PyObject_GetAttrStr(__pyx_t_6, __pyx_mstate_global->__pyx_n_u_ndarray); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 337, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_8);
  __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
  __pyx_t_6 = __Pyx_PyLong_From_int(__pyx_v_batch_size); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 337, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_6);
  __pyx_t_2 = __Pyx_PyLong_From_int(__pyx_v_k); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 337, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_7 = PyTuple_New(2); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 337, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_7);
  __Pyx_GIVEREF(__pyx_t_6);
  if (__Pyx_PyTuple_SET_ITEM(__pyx_t_7, 0, __pyx_t_6) != (0)) __PYX_ERR(0, 337, __pyx_L1_error);
  __Pyx_GIVEREF(__pyx_t_2);
  if (__Pyx_PyTuple_SET_ITEM(__pyx_t_7, 1, __pyx_t_2) != (0)) __PYX_ERR(0, 337, __pyx_L1_error);
  __pyx_t_6 = 0;
  __pyx_t_2 = 0;
  __pyx_t_14 = __pyx_convert_vector_from_py_Py_ssize_t(__pyx_t_7); if (unlikely(PyErr_Occurred())) __PYX_ERR(0, 337, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
  __pyx_t_7 = ((PyObject *)__pyx_f_4cupy_5_core_4core__ndarray_init(__pyx_t_8, __pyx_t_14, __pyx_v_s_dtype, Py_None)); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 337, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_7);
  __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
  __pyx_v_s = ((struct __pyx_obj_4cupy_5_core_4core__ndarray_base *)__pyx_t_7);
  __pyx_t_7 = 0;

  /* "cupyx/cusolver.pyx":338
 *         job_vt = b'N'
 *     s = _ndarray_init(_cupy.ndarray, (batch_size, k), s_dtype, None)
 *     s_ptr = s.data.ptr             # <<<<<<<<<<<<<<
 *     cdef intptr_t handle = _device.get_cusolver_handle()
 *     dev_info = _ndarray_init(_cupy.ndarray, (batch_size,), _numpy.int32, None)
*/
  __pyx_t_15 = __pyx_v_s->data->ptr;
  __pyx_v_s_ptr = __pyx_t_15;

  /* "cupyx/cusolver.pyx":339
 *     s = _ndarray_init(_cupy.ndarray, (batch_size, k), s_dtype, None)
 *     s_ptr = s.data.ptr
 *     cdef intptr_t handle = _device.get_cusolver_handle()             # <<<<<<<<<<<<<<
 *     dev_info = _ndarray_init(_cupy.ndarray, (batch_size,), _numpy.int32, None)
 *     info_ptr = dev_info.data.ptr
*/
  __pyx_t_8 = NULL;
  __Pyx_GetModuleGlobalName(__pyx_t_2, __pyx_mstate_global->__pyx_n_u_device); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 339, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_6 = __Pyx_PyObject_GetAttrStr(__pyx_t_2, __pyx_mstate_global->__pyx_n_u_get_cusolver_handle); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 339, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_6);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_3 = 1;
  #if CYTHON_UNPACK_METHODS
  if (unlikely(PyMethod_Check(__pyx_t_6))) {
    __pyx_t_8 = PyMethod_GET_SELF(__pyx_t_6);
    assert(__pyx_t_8);
    PyObject* __pyx__function = PyMethod_GET_FUNCTION(__pyx_t_6);
    __Pyx_INCREF(__pyx_t_8);
    __Pyx_INCREF(__pyx__function);
    __Pyx_DECREF_SET(__pyx_t_6, __pyx__function);
    __pyx_t_3 = 0;
  }
  #endif
  {
    PyObject *__pyx_callargs[2] = {__pyx_t_8, NULL};
    __pyx_t_7 = __Pyx_PyObject_FastCall((PyObject*)__pyx_t_6, __pyx_callargs+__pyx_t_3, (1-__pyx_t_3) | (__pyx_t_3*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
    __Pyx_XDECREF(__pyx_t_8); __pyx_t_8 = 0;
    __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
    if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 339, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_7);
  }
  __pyx_t_15 = PyLong_AsSsize_t(__pyx_t_7); if (unlikely((__pyx_t_15 == ((intptr_t)-1)) && PyErr_Occurred())) __PYX_ERR(0, 339, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
  __pyx_v_handle = __pyx_t_15;

  /* "cupyx/cusolver.pyx":340
 *     s_ptr = s.data.ptr
 *     cdef intptr_t handle = _device.get_cusolver_handle()
 *     dev_info = _ndarray_init(_cupy.ndarray, (batch_size,), _numpy.int32, None)             # <<<<<<<<<<<<<<
 *     info_ptr = dev_info.data.ptr
 * 
*/
  __Pyx_GetModuleGlobalName(__pyx_t_7, __pyx_mstate_global->__pyx_n_u_cupy); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 340, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_7);
  __pyx_t_6 = __Pyx_PyObject_GetAttrStr(__pyx_t_7, __pyx_mstate_global->__pyx_n_u_ndarray); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 340, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_6);
  __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
  __pyx_t_7 = __Pyx_PyLong_From_int(__pyx_v_batch_size); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 340, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_7);
  __pyx_t_8 = PyTuple_New(1); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 340, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_8);
  __Pyx_GIVEREF(__pyx_t_7);
  if (__Pyx_PyTuple_SET_ITEM(__pyx_t_8, 0, __pyx_t_7) != (0)) __PYX_ERR(0, 340, __pyx_L1_error);
  __pyx_t_7 = 0;
  __pyx_t_14 = __pyx_convert_vector_from_py_Py_ssize_t(__pyx_t_8); if (unlikely(PyErr_Occurred())) __PYX_ERR(0, 340, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
  __Pyx_GetModuleGlobalName(__pyx_t_8, __pyx_mstate_global->__pyx_n_u_numpy); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 340, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_8);
  __pyx_t_7 = __Pyx_PyObject_GetAttrStr(__pyx_t_8, __pyx_mstate_global->__pyx_n_u_int32); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 340, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_7);
  __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
  __pyx_t_8 = ((PyObject *)__pyx_f_4cupy_5_core_4core__ndarray_init(__pyx_t_6, __pyx_t_14, __pyx_t_7, Py_None)); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 340, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_8);
  __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
  __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
  __pyx_v_dev_info = ((struct __pyx_obj_4cupy_5_core_4core__ndarray_base *)__pyx_t_8);
  __pyx_t_8 = 0;

  /* "cupyx/cusolver.pyx":341
 *     cdef intptr_t handle = _device.get_cusolver_handle()
 *     dev_info = _ndarray_init(_cupy.ndarray, (batch_size,), _numpy.int32, None)
 *     info_ptr = dev_info.data.ptr             # <<<<<<<<<<<<<<
 * 
 *     if a_dtype == 'f':
*/
  __pyx_t_15 = __pyx_v_dev_info->data->ptr;
  __pyx_v_info_ptr = __pyx_t_15;

  /* "cupyx/cusolver.pyx":343
 *     info_ptr = dev_info.data.ptr
 * 
 *     if a_dtype == 'f':             # <<<<<<<<<<<<<<
 *         gesvd_bufferSize = sgesvd_bufferSize
 *         gesvd = gesvd_loop[float]
*/
  __pyx_t_5 = (__Pyx_PyUnicode_Equals(__pyx_v_a_dtype, __pyx_mstate_global->__pyx_n_u_f, Py_EQ)); if (unlikely((__pyx_t_5 < 0))) __PYX_ERR(0, 343, __pyx_L1_error)
  if (__pyx_t_5) {

    /* "cupyx/cusolver.pyx":344
 * 
 *     if a_dtype == 'f':
 *         gesvd_bufferSize = sgesvd_bufferSize             # <<<<<<<<<<<<<<
 *         gesvd = gesvd_loop[float]
 *     elif a_dtype == 'd':
*/
    __pyx_v_gesvd_bufferSize = __pyx_f_13cupy_backends_4cuda_4libs_8cusolver_sgesvd_bufferSize;

    /* "cupyx/cusolver.pyx":345
 *     if a_dtype == 'f':
 *         gesvd_bufferSize = sgesvd_bufferSize
 *         gesvd = gesvd_loop[float]             # <<<<<<<<<<<<<<
 *     elif a_dtype == 'd':
 *         gesvd_bufferSize = dgesvd_bufferSize
*/
    __pyx_v_gesvd = gesvd_loop<float>;

    /* "cupyx/cusolver.pyx":343
 *     info_ptr = dev_info.data.ptr
 * 
 *     if a_dtype == 'f':             # <<<<<<<<<<<<<<
 *         gesvd_bufferSize = sgesvd_bufferSize
 *         gesvd = gesvd_loop[float]
*/
    goto __pyx_L11;
  }

  /* "cupyx/cusolver.pyx":346
 *         gesvd_bufferSize = sgesvd_bufferSize
 *         gesvd = gesvd_loop[float]
 *     elif a_dtype == 'd':             # <<<<<<<<<<<<<<
 *         gesvd_bufferSize = dgesvd_bufferSize
 *         gesvd = gesvd_loop[double]
*/
  __pyx_t_5 = (__Pyx_PyUnicode_Equals(__pyx_v_a_dtype, __pyx_mstate_global->__pyx_n_u_d, Py_EQ)); if (unlikely((__pyx_t_5 < 0))) __PYX_ERR(0, 346, __pyx_L1_error)
  if (__pyx_t_5) {

    /* "cupyx/cusolver.pyx":347
 *         gesvd = gesvd_loop[float]
 *     elif a_dtype == 'd':
 *         gesvd_bufferSize = dgesvd_bufferSize             # <<<<<<<<<<<<<<
 *         gesvd = gesvd_loop[double]
 *     elif a_dtype == 'F':
*/
    __pyx_v_gesvd_bufferSize = __pyx_f_13cupy_backends_4cuda_4libs_8cusolver_dgesvd_bufferSize;

    /* "cupyx/cusolver.pyx":348
 *     elif a_dtype == 'd':
 *         gesvd_bufferSize = dgesvd_bufferSize
 *         gesvd = gesvd_loop[double]             # <<<<<<<<<<<<<<
 *     elif a_dtype == 'F':
 *         gesvd_bufferSize = cgesvd_bufferSize
*/
    __pyx_v_gesvd = gesvd_loop<double>;

    /* "cupyx/cusolver.pyx":346
 *         gesvd_bufferSize = sgesvd_bufferSize
 *         gesvd = gesvd_loop[float]
 *     elif a_dtype == 'd':             # <<<<<<<<<<<<<<
 *         gesvd_bufferSize = dgesvd_bufferSize
 *         gesvd = gesvd_loop[double]
*/
    goto __pyx_L11;
  }

  /* "cupyx/cusolver.pyx":349
 *         gesvd_bufferSize = dgesvd_bufferSize
 *         gesvd = gesvd_loop[double]
 *     elif a_dtype == 'F':             # <<<<<<<<<<<<<<
 *         gesvd_bufferSize = cgesvd_bufferSize
 *         gesvd = gesvd_loop[cuComplex]
*/
  __pyx_t_5 = (__Pyx_PyUnicode_Equals(__pyx_v_a_dtype, __pyx_mstate_global->__pyx_n_u_F, Py_EQ)); if (unlikely((__pyx_t_5 < 0))) __PYX_ERR(0, 349, __pyx_L1_error)
  if (__pyx_t_5) {

    /* "cupyx/cusolver.pyx":350
 *         gesvd = gesvd_loop[double]
 *     elif a_dtype == 'F':
 *         gesvd_bufferSize = cgesvd_bufferSize             # <<<<<<<<<<<<<<
 *         gesvd = gesvd_loop[cuComplex]
 *     elif a_dtype == 'D':
*/
    __pyx_v_gesvd_bufferSize = __pyx_f_13cupy_backends_4cuda_4libs_8cusolver_cgesvd_bufferSize;

    /* "cupyx/cusolver.pyx":351
 *     elif a_dtype == 'F':
 *         gesvd_bufferSize = cgesvd_bufferSize
 *         gesvd = gesvd_loop[cuComplex]             # <<<<<<<<<<<<<<
 *     elif a_dtype == 'D':
 *         gesvd_bufferSize = zgesvd_bufferSize
*/
    __pyx_v_gesvd = gesvd_loop<cuComplex>;

    /* "cupyx/cusolver.pyx":349
 *         gesvd_bufferSize = dgesvd_bufferSize
 *         gesvd = gesvd_loop[double]
 *     elif a_dtype == 'F':             # <<<<<<<<<<<<<<
 *         gesvd_bufferSize = cgesvd_bufferSize
 *         gesvd = gesvd_loop[cuComplex]
*/
    goto __pyx_L11;
  }

  /* "cupyx/cusolver.pyx":352
 *         gesvd_bufferSize = cgesvd_bufferSize
 *         gesvd = gesvd_loop[cuComplex]
 *     elif a_dtype == 'D':             # <<<<<<<<<<<<<<
 *         gesvd_bufferSize = zgesvd_bufferSize
 *         gesvd = gesvd_loop[cuDoubleComplex]
*/
  __pyx_t_5 = (__Pyx_PyUnicode_Equals(__pyx_v_a_dtype, __pyx_mstate_global->__pyx_n_u_D, Py_EQ)); if (unlikely((__pyx_t_5 < 0))) __PYX_ERR(0, 352, __pyx_L1_error)
  if (likely(__pyx_t_5)) {

    /* "cupyx/cusolver.pyx":353
 *         gesvd = gesvd_loop[cuComplex]
 *     elif a_dtype == 'D':
 *         gesvd_bufferSize = zgesvd_bufferSize             # <<<<<<<<<<<<<<
 *         gesvd = gesvd_loop[cuDoubleComplex]
 *     else:
*/
    __pyx_v_gesvd_bufferSize = __pyx_f_13cupy_backends_4cuda_4libs_8cusolver_zgesvd_bufferSize;

    /* "cupyx/cusolver.pyx":354
 *     elif a_dtype == 'D':
 *         gesvd_bufferSize = zgesvd_bufferSize
 *         gesvd = gesvd_loop[cuDoubleComplex]             # <<<<<<<<<<<<<<
 *     else:
 *         raise TypeError
*/
    __pyx_v_gesvd = gesvd_loop<cuDoubleComplex>;

    /* "cupyx/cusolver.pyx":352
 *         gesvd_bufferSize = cgesvd_bufferSize
 *         gesvd = gesvd_loop[cuComplex]
 *     elif a_dtype == 'D':             # <<<<<<<<<<<<<<
 *         gesvd_bufferSize = zgesvd_bufferSize
 *         gesvd = gesvd_loop[cuDoubleComplex]
*/
    goto __pyx_L11;
  }

  /* "cupyx/cusolver.pyx":356
 *         gesvd = gesvd_loop[cuDoubleComplex]
 *     else:
 *         raise TypeError             # <<<<<<<<<<<<<<
 * 
 *     # this wrapper also sets the stream for us
*/
  /*else*/ {
    __Pyx_Raise(((PyObject *)(((PyTypeObject*)PyExc_TypeError))), 0, 0, 0);
    __PYX_ERR(0, 356, __pyx_L1_error)
  }
  __pyx_L11:;

  /* "cupyx/cusolver.pyx":359
 * 
 *     # this wrapper also sets the stream for us
 *     buffersize = gesvd_bufferSize(handle, m, n)             # <<<<<<<<<<<<<<
 *     # we are on the same stream, so the workspace can be reused in the loop
 *     workspace = memory.alloc(buffersize * x.dtype.itemsize)
*/
  __pyx_t_11 = __pyx_v_gesvd_bufferSize(__pyx_v_handle, __pyx_v_m, __pyx_v_n, 0); if (unlikely(__pyx_t_11 == ((int)-1) && PyErr_Occurred())) __PYX_ERR(0, 359, __pyx_L1_error)
  __pyx_v_buffersize = __pyx_t_11;

  /* "cupyx/cusolver.pyx":361
 *     buffersize = gesvd_bufferSize(handle, m, n)
 *     # we are on the same stream, so the workspace can be reused in the loop
 *     workspace = memory.alloc(buffersize * x.dtype.itemsize)             # <<<<<<<<<<<<<<
 *     w_ptr = workspace.ptr
 * 
*/
  __pyx_t_8 = __Pyx_PyLong_From_int(__pyx_v_buffersize); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 361, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_8);
  __pyx_t_7 = __Pyx_PyObject_GetAttrStr(__pyx_v_x->dtype, __pyx_mstate_global->__pyx_n_u_itemsize); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 361, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_7);
  __pyx_t_6 = PyNumber_Multiply(__pyx_t_8, __pyx_t_7); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 361, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_6);
  __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
  __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
  __pyx_t_7 = ((PyObject *)__pyx_f_4cupy_4cuda_6memory_alloc(__pyx_t_6, 0)); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 361, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_7);
  __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
  __pyx_v_workspace = ((struct __pyx_obj_4cupy_4cuda_6memory_MemoryPointer *)__pyx_t_7);
  __pyx_t_7 = 0;

  /* "cupyx/cusolver.pyx":362
 *     # we are on the same stream, so the workspace can be reused in the loop
 *     workspace = memory.alloc(buffersize * x.dtype.itemsize)
 *     w_ptr = workspace.ptr             # <<<<<<<<<<<<<<
 * 
 *     # the loop starts here, with gil released to reduce overhead
*/
  __pyx_t_15 = __pyx_v_workspace->ptr;
  __pyx_v_w_ptr = __pyx_t_15;

  /* "cupyx/cusolver.pyx":365
 * 
 *     # the loop starts here, with gil released to reduce overhead
 *     with nogil:             # <<<<<<<<<<<<<<
 *         status = gesvd(
 *             handle, job_u, job_vt, m, n, a_ptr,
*/
  {
      PyThreadState * _save;
      _save = PyEval_SaveThread();
      __Pyx_FastGIL_Remember();
      /*try:*/ {

        /* "cupyx/cusolver.pyx":366
 *     # the loop starts here, with gil released to reduce overhead
 *     with nogil:
 *         status = gesvd(             # <<<<<<<<<<<<<<
 *             handle, job_u, job_vt, m, n, a_ptr,
 *             s_ptr, u_ptr, vt_ptr,
*/
        __pyx_v_status = __pyx_v_gesvd(__pyx_v_handle, __pyx_v_job_u, __pyx_v_job_vt, __pyx_v_m, __pyx_v_n, __pyx_v_a_ptr, __pyx_v_s_ptr, __pyx_v_u_ptr, __pyx_v_vt_ptr, __pyx_v_w_ptr, __pyx_v_buffersize, __pyx_v_info_ptr, __pyx_v_batch_size);
      }

      /* "cupyx/cusolver.pyx":365
 * 
 *     # the loop starts here, with gil released to reduce overhead
 *     with nogil:             # <<<<<<<<<<<<<<
 *         status = gesvd(
 *             handle, job_u, job_vt, m, n, a_ptr,
*/
      /*finally:*/ {
        /*normal exit:*/{
          __Pyx_FastGIL_Forget();
          PyEval_RestoreThread(_save);
          goto __pyx_L14;
        }
        __pyx_L14:;
      }
  }

  /* "cupyx/cusolver.pyx":370
 *             s_ptr, u_ptr, vt_ptr,
 *             w_ptr, buffersize, info_ptr, batch_size)
 *     if status != 0:             # <<<<<<<<<<<<<<
 *         raise _cusolver.CUSOLVERError(status)
 * 
*/
  __pyx_t_5 = (__pyx_v_status != 0);
  if (unlikely(__pyx_t_5)) {

    /* "cupyx/cusolver.pyx":371
 *             w_ptr, buffersize, info_ptr, batch_size)
 *     if status != 0:
 *         raise _cusolver.CUSOLVERError(status)             # <<<<<<<<<<<<<<
 * 
 *     # check the full info array
*/
    __pyx_t_6 = NULL;
    __Pyx_GetModuleGlobalName(__pyx_t_8, __pyx_mstate_global->__pyx_n_u_cusolver); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 371, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_8);
    __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_t_8, __pyx_mstate_global->__pyx_n_u_CUSOLVERError); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 371, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
    __pyx_t_8 = __Pyx_PyLong_From_int(__pyx_v_status); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 371, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_8);
    __pyx_t_3 = 1;
    #if CYTHON_UNPACK_METHODS
    if (unlikely(PyMethod_Check(__pyx_t_2))) {
      __pyx_t_6 = PyMethod_GET_SELF(__pyx_t_2);
      assert(__pyx_t_6);
      PyObject* __pyx__function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_6);
      __Pyx_INCREF(__pyx__function);
      __Pyx_DECREF_SET(__pyx_t_2, __pyx__function);
      __pyx_t_3 = 0;
    }
    #endif
    {
      PyObject *__pyx_callargs[2] = {__pyx_t_6, __pyx_t_8};
      __pyx_t_7 = __Pyx_PyObject_FastCall((PyObject*)__pyx_t_2, __pyx_callargs+__pyx_t_3, (2-__pyx_t_3) | (__pyx_t_3*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
      __Pyx_XDECREF(__pyx_t_6); __pyx_t_6 = 0;
      __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
      __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
      if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 371, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_7);
    }
    __Pyx_Raise(__pyx_t_7, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
    __PYX_ERR(0, 371, __pyx_L1_error)

    /* "cupyx/cusolver.pyx":370
 *             s_ptr, u_ptr, vt_ptr,
 *             w_ptr, buffersize, info_ptr, batch_size)
 *     if status != 0:             # <<<<<<<<<<<<<<
 *         raise _cusolver.CUSOLVERError(status)
 * 
*/
  }

  /* "cupyx/cusolver.pyx":374
 * 
 *     # check the full info array
 *     _cupy.linalg._util._check_cusolver_dev_info_if_synchronization_allowed(             # <<<<<<<<<<<<<<
 *         'gesvd', dev_info)
 * 
*/
  __Pyx_GetModuleGlobalName(__pyx_t_8, __pyx_mstate_global->__pyx_n_u_cupy); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 374, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_8);
  __pyx_t_6 = __Pyx_PyObject_GetAttrStr(__pyx_t_8, __pyx_mstate_global->__pyx_n_u_linalg); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 374, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_6);
  __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
  __pyx_t_8 = __Pyx_PyObject_GetAttrStr(__pyx_t_6, __pyx_mstate_global->__pyx_n_u_util); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 374, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_8);
  __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
  __pyx_t_2 = __pyx_t_8;
  __Pyx_INCREF(__pyx_t_2);

  /* "cupyx/cusolver.pyx":375
 *     # check the full info array
 *     _cupy.linalg._util._check_cusolver_dev_info_if_synchronization_allowed(
 *         'gesvd', dev_info)             # <<<<<<<<<<<<<<
 * 
 *     # Note that the returned array may need to be transposed
*/
  __pyx_t_3 = 0;
  {
    PyObject *__pyx_callargs[3] = {__pyx_t_2, __pyx_mstate_global->__pyx_n_u_gesvd, ((PyObject *)__pyx_v_dev_info)};
    __pyx_t_7 = __Pyx_PyObject_FastCallMethod((PyObject*)__pyx_mstate_global->__pyx_n_u_check_cusolver_dev_info_if_sync, __pyx_callargs+__pyx_t_3, (3-__pyx_t_3) | (1*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
    __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
    if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 374, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_7);
  }
  __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;

  /* "cupyx/cusolver.pyx":379
 *     # Note that the returned array may need to be transposed
 *     # depending on the structure of an input
 *     if compute_uv:             # <<<<<<<<<<<<<<
 *         if trans_flag:
 *             return u.swapaxes(-2, -1), s, vt.conj()
*/
  __pyx_t_5 = __Pyx_PyObject_IsTrue(__pyx_v_compute_uv); if (unlikely((__pyx_t_5 < 0))) __PYX_ERR(0, 379, __pyx_L1_error)
  if (__pyx_t_5) {

    /* "cupyx/cusolver.pyx":380
 *     # depending on the structure of an input
 *     if compute_uv:
 *         if trans_flag:             # <<<<<<<<<<<<<<
 *             return u.swapaxes(-2, -1), s, vt.conj()
 *         else:
*/
    if (__pyx_v_trans_flag) {

      /* "cupyx/cusolver.pyx":381
 *     if compute_uv:
 *         if trans_flag:
 *             return u.swapaxes(-2, -1), s, vt.conj()             # <<<<<<<<<<<<<<
 *         else:
 *             return vt, s, u.swapaxes(-2, -1).conj()
*/
      __Pyx_XDECREF(__pyx_r);
      if (unlikely(!__pyx_v_u)) { __Pyx_RaiseUnboundLocalError("u"); __PYX_ERR(0, 381, __pyx_L1_error) }
      __pyx_t_7 = ((PyObject *)((struct __pyx_vtabstruct_4cupy_5_core_4core__ndarray_base *)__pyx_v_u->__pyx_vtab)->swapaxes(__pyx_v_u, -2L, -1L, 0)); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 381, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_7);
      if (unlikely(!__pyx_v_vt)) { __Pyx_RaiseUnboundLocalError("vt"); __PYX_ERR(0, 381, __pyx_L1_error) }
      __pyx_t_8 = ((PyObject *)((struct __pyx_vtabstruct_4cupy_5_core_4core__ndarray_base *)__pyx_v_vt->__pyx_vtab)->conj(__pyx_v_vt, 0)); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 381, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_8);
      __pyx_t_2 = PyTuple_New(3); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 381, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_2);
      __Pyx_GIVEREF(__pyx_t_7);
      if (__Pyx_PyTuple_SET_ITEM(__pyx_t_2, 0, __pyx_t_7) != (0)) __PYX_ERR(0, 381, __pyx_L1_error);
      __Pyx_INCREF((PyObject *)__pyx_v_s);
      __Pyx_GIVEREF((PyObject *)__pyx_v_s);
      if (__Pyx_PyTuple_SET_ITEM(__pyx_t_2, 1, ((PyObject *)__pyx_v_s)) != (0)) __PYX_ERR(0, 381, __pyx_L1_error);
      __Pyx_GIVEREF(__pyx_t_8);
      if (__Pyx_PyTuple_SET_ITEM(__pyx_t_2, 2, __pyx_t_8) != (0)) __PYX_ERR(0, 381, __pyx_L1_error);
      __pyx_t_7 = 0;
      __pyx_t_8 = 0;
      __pyx_r = __pyx_t_2;
      __pyx_t_2 = 0;
      goto __pyx_L0;

      /* "cupyx/cusolver.pyx":380
 *     # depending on the structure of an input
 *     if compute_uv:
 *         if trans_flag:             # <<<<<<<<<<<<<<
 *             return u.swapaxes(-2, -1), s, vt.conj()
 *         else:
*/
    }

    /* "cupyx/cusolver.pyx":383
 *             return u.swapaxes(-2, -1), s, vt.conj()
 *         else:
 *             return vt, s, u.swapaxes(-2, -1).conj()             # <<<<<<<<<<<<<<
 *     else:
 *         return s
*/
    /*else*/ {
      __Pyx_XDECREF(__pyx_r);
      if (unlikely(!__pyx_v_vt)) { __Pyx_RaiseUnboundLocalError("vt"); __PYX_ERR(0, 383, __pyx_L1_error) }
      if (unlikely(!__pyx_v_u)) { __Pyx_RaiseUnboundLocalError("u"); __PYX_ERR(0, 383, __pyx_L1_error) }
      __pyx_t_2 = ((PyObject *)((struct __pyx_vtabstruct_4cupy_5_core_4core__ndarray_base *)__pyx_v_u->__pyx_vtab)->swapaxes(__pyx_v_u, -2L, -1L, 0)); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 383, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_2);
      __pyx_t_8 = ((PyObject *)((struct __pyx_vtabstruct_4cupy_5_core_4core__ndarray_base *)((struct __pyx_obj_4cupy_5_core_4core__ndarray_base *)__pyx_t_2)->__pyx_vtab)->conj(((struct __pyx_obj_4cupy_5_core_4core__ndarray_base *)__pyx_t_2), 0)); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 383, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_8);
      __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
      __pyx_t_2 = PyTuple_New(3); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 383, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_2);
      __Pyx_INCREF((PyObject *)__pyx_v_vt);
      __Pyx_GIVEREF((PyObject *)__pyx_v_vt);
      if (__Pyx_PyTuple_SET_ITEM(__pyx_t_2, 0, ((PyObject *)__pyx_v_vt)) != (0)) __PYX_ERR(0, 383, __pyx_L1_error);
      __Pyx_INCREF((PyObject *)__pyx_v_s);
      __Pyx_GIVEREF((PyObject *)__pyx_v_s);
      if (__Pyx_PyTuple_SET_ITEM(__pyx_t_2, 1, ((PyObject *)__pyx_v_s)) != (0)) __PYX_ERR(0, 383, __pyx_L1_error);
      __Pyx_GIVEREF(__pyx_t_8);
      if (__Pyx_PyTuple_SET_ITEM(__pyx_t_2, 2, __pyx_t_8) != (0)) __PYX_ERR(0, 383, __pyx_L1_error);
      __pyx_t_8 = 0;
      __pyx_r = __pyx_t_2;
      __pyx_t_2 = 0;
      goto __pyx_L0;
    }

    /* "cupyx/cusolver.pyx":379
 *     # Note that the returned array may need to be transposed
 *     # depending on the structure of an input
 *     if compute_uv:             # <<<<<<<<<<<<<<
 *         if trans_flag:
 *             return u.swapaxes(-2, -1), s, vt.conj()
*/
  }

  /* "cupyx/cusolver.pyx":385
 *             return vt, s, u.swapaxes(-2, -1).conj()
 *     else:
 *         return s             # <<<<<<<<<<<<<<
 * 
 * 
*/
  /*else*/ {
    __Pyx_XDECREF(__pyx_r);
    __Pyx_INCREF((PyObject *)__pyx_v_s);
    __pyx_r = ((PyObject *)__pyx_v_s);
    goto __pyx_L0;
  }

  /* "cupyx/cusolver.pyx":280
 * 
 * 
 * cpdef _gesvd_batched(a, a_dtype, full_matrices, compute_uv, overwrite_a):             # <<<<<<<<<<<<<<
 *     """A loop-based gesvd wrapper to support batched SVD."""
 *     # This function follows more closely with gesvd() in
*/

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_XDECREF(__pyx_t_7);
  __Pyx_XDECREF(__pyx_t_8);
  __Pyx_AddTraceback("cupyx.cusolver._gesvd_batched", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XDECREF((PyObject *)__pyx_v_x);
  __Pyx_XDECREF((PyObject *)__pyx_v_s);
  __Pyx_XDECREF((PyObject *)__pyx_v_u);
  __Pyx_XDECREF((PyObject *)__pyx_v_vt);
  __Pyx_XDECREF((PyObject *)__pyx_v_dev_info);
  __Pyx_XDECREF(__pyx_v_s_dtype);
  __Pyx_XDECREF((PyObject *)__pyx_v_workspace);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_pw_5cupyx_8cusolver_7_gesvd_batched(PyObject *__pyx_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
PyDoc_STRVAR(__pyx_doc_5cupyx_8cusolver_6_gesvd_batched, "_gesvd_batched(a, a_dtype, full_matrices, compute_uv, overwrite_a)\n\nA loop-based gesvd wrapper to support batched SVD.");
static PyObject *__pyx_pw_5cupyx_8cusolver_7_gesvd_batched(PyObject *__pyx_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
) {
  PyObject *__pyx_v_a = 0;
  PyObject *__pyx_v_a_dtype = 0;
  PyObject *__pyx_v_full_matrices = 0;
  PyObject *__pyx_v_compute_uv = 0;
  PyObject *__pyx_v_overwrite_a = 0;
  #if !CYTHON_METH_FASTCALL
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  #endif
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject* values[5] = {0,0,0,0,0};
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("_gesvd_batched (wrapper)", 0);
  #if !CYTHON_METH_FASTCALL
  #if CYTHON_ASSUME_SAFE_SIZE
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return NULL;
  #endif
  #endif
  __pyx_kwvalues = __Pyx_KwValues_FASTCALL(__pyx_args, __pyx_nargs);
  {
    PyObject ** const __pyx_pyargnames[] = {&__pyx_mstate_global->__pyx_n_u_a,&__pyx_mstate_global->__pyx_n_u_a_dtype,&__pyx_mstate_global->__pyx_n_u_full_matrices,&__pyx_mstate_global->__pyx_n_u_compute_uv,&__pyx_mstate_global->__pyx_n_u_overwrite_a,0};
    const Py_ssize_t __pyx_kwds_len = (__pyx_kwds) ? __Pyx_NumKwargs_FASTCALL(__pyx_kwds) : 0;
    if (unlikely(__pyx_kwds_len < 0)) __PYX_ERR(0, 280, __pyx_L3_error)
    if (__pyx_kwds_len > 0) {
      switch (__pyx_nargs) {
        case  5:
        values[4] = __Pyx_ArgRef_FASTCALL(__pyx_args, 4);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[4])) __PYX_ERR(0, 280, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  4:
        values[3] = __Pyx_ArgRef_FASTCALL(__pyx_args, 3);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[3])) __PYX_ERR(0, 280, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  3:
        values[2] = __Pyx_ArgRef_FASTCALL(__pyx_args, 2);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[2])) __PYX_ERR(0, 280, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  2:
        values[1] = __Pyx_ArgRef_FASTCALL(__pyx_args, 1);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[1])) __PYX_ERR(0, 280, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  1:
        values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(0, 280, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      const Py_ssize_t kwd_pos_args = __pyx_nargs;
      if (__Pyx_ParseKeywords(__pyx_kwds, __pyx_kwvalues, __pyx_pyargnames, 0, values, kwd_pos_args, __pyx_kwds_len, "_gesvd_batched", 0) < (0)) __PYX_ERR(0, 280, __pyx_L3_error)
      for (Py_ssize_t i = __pyx_nargs; i < 5; i++) {
        if (unlikely(!values[i])) { __Pyx_RaiseArgtupleInvalid("_gesvd_batched", 1, 5, 5, i); __PYX_ERR(0, 280, __pyx_L3_error) }
      }
    } else if (unlikely(__pyx_nargs != 5)) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
      if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(0, 280, __pyx_L3_error)
      values[1] = __Pyx_ArgRef_FASTCALL(__pyx_args, 1);
      if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[1])) __PYX_ERR(0, 280, __pyx_L3_error)
      values[2] = __Pyx_ArgRef_FASTCALL(__pyx_args, 2);
      if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[2])) __PYX_ERR(0, 280, __pyx_L3_error)
      values[3] = __Pyx_ArgRef_FASTCALL(__pyx_args, 3);
      if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[3])) __PYX_ERR(0, 280, __pyx_L3_error)
      values[4] = __Pyx_ArgRef_FASTCALL(__pyx_args, 4);
      if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[4])) __PYX_ERR(0, 280, __pyx_L3_error)
    }
    __pyx_v_a = values[0];
    __pyx_v_a_dtype = values[1];
    __pyx_v_full_matrices = values[2];
    __pyx_v_compute_uv = values[3];
    __pyx_v_overwrite_a = values[4];
  }
  goto __pyx_L6_skip;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("_gesvd_batched", 1, 5, 5, __pyx_nargs); __PYX_ERR(0, 280, __pyx_L3_error)
  __pyx_L6_skip:;
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_AddTraceback("cupyx.cusolver._gesvd_batched", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_5cupyx_8cusolver_6_gesvd_batched(__pyx_self, __pyx_v_a, __pyx_v_a_dtype, __pyx_v_full_matrices, __pyx_v_compute_uv, __pyx_v_overwrite_a);

  /* function exit code */
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5cupyx_8cusolver_6_gesvd_batched(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_a, PyObject *__pyx_v_a_dtype, PyObject *__pyx_v_full_matrices, PyObject *__pyx_v_compute_uv, PyObject *__pyx_v_overwrite_a) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("_gesvd_batched", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __pyx_f_5cupyx_8cusolver__gesvd_batched(__pyx_v_a, __pyx_v_a_dtype, __pyx_v_full_matrices, __pyx_v_compute_uv, __pyx_v_overwrite_a, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 280, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("cupyx.cusolver._gesvd_batched", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "cupyx/cusolver.pyx":388
 * 
 * 
 * def gesvda(a, compute_uv=True):             # <<<<<<<<<<<<<<
 *     """Singular value decomposition using cusolverDn<t>gesvdaStridedBatched().
 * 
*/

/* Python wrapper */
static PyObject *__pyx_pw_5cupyx_8cusolver_9gesvda(PyObject *__pyx_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
PyDoc_STRVAR(__pyx_doc_5cupyx_8cusolver_8gesvda, "gesvda(a, compute_uv=True)\n\nSingular value decomposition using cusolverDn<t>gesvdaStridedBatched().\n\nFactorizes the matrix ``a`` into two unitary matrices ``u`` and ``v`` and\na singular values vector ``s`` such that ``a == u @ diag(s) @ v*``.\n\nArgs:\n    a (cupy.ndarray): The input matrix with dimension ``(.., M, N)``.\n    compute_uv (bool): If ``False``, it only returns singular values.\n\nReturns:\n    tuple of :class:`cupy.ndarray`:\n        A tuple of ``(u, s, v)``.");
static PyMethodDef __pyx_mdef_5cupyx_8cusolver_9gesvda = {"gesvda", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_5cupyx_8cusolver_9gesvda, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_5cupyx_8cusolver_8gesvda};
static PyObject *__pyx_pw_5cupyx_8cusolver_9gesvda(PyObject *__pyx_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
) {
  PyObject *__pyx_v_a = 0;
  PyObject *__pyx_v_compute_uv = 0;
  #if !CYTHON_METH_FASTCALL
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  #endif
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject* values[2] = {0,0};
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("gesvda (wrapper)", 0);
  #if !CYTHON_METH_FASTCALL
  #if CYTHON_ASSUME_SAFE_SIZE
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return NULL;
  #endif
  #endif
  __pyx_kwvalues = __Pyx_KwValues_FASTCALL(__pyx_args, __pyx_nargs);
  {
    PyObject ** const __pyx_pyargnames[] = {&__pyx_mstate_global->__pyx_n_u_a,&__pyx_mstate_global->__pyx_n_u_compute_uv,0};
    const Py_ssize_t __pyx_kwds_len = (__pyx_kwds) ? __Pyx_NumKwargs_FASTCALL(__pyx_kwds) : 0;
    if (unlikely(__pyx_kwds_len < 0)) __PYX_ERR(0, 388, __pyx_L3_error)
    if (__pyx_kwds_len > 0) {
      switch (__pyx_nargs) {
        case  2:
        values[1] = __Pyx_ArgRef_FASTCALL(__pyx_args, 1);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[1])) __PYX_ERR(0, 388, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  1:
        values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(0, 388, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      const Py_ssize_t kwd_pos_args = __pyx_nargs;
      if (__Pyx_ParseKeywords(__pyx_kwds, __pyx_kwvalues, __pyx_pyargnames, 0, values, kwd_pos_args, __pyx_kwds_len, "gesvda", 0) < (0)) __PYX_ERR(0, 388, __pyx_L3_error)
      if (!values[1]) values[1] = __Pyx_NewRef(((PyObject *)Py_True));
      for (Py_ssize_t i = __pyx_nargs; i < 1; i++) {
        if (unlikely(!values[i])) { __Pyx_RaiseArgtupleInvalid("gesvda", 0, 1, 2, i); __PYX_ERR(0, 388, __pyx_L3_error) }
      }
    } else {
      switch (__pyx_nargs) {
        case  2:
        values[1] = __Pyx_ArgRef_FASTCALL(__pyx_args, 1);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[1])) __PYX_ERR(0, 388, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  1:
        values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(0, 388, __pyx_L3_error)
        break;
        default: goto __pyx_L5_argtuple_error;
      }
      if (!values[1]) values[1] = __Pyx_NewRef(((PyObject *)Py_True));
    }
    __pyx_v_a = values[0];
    __pyx_v_compute_uv = values[1];
  }
  goto __pyx_L6_skip;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("gesvda", 0, 1, 2, __pyx_nargs); __PYX_ERR(0, 388, __pyx_L3_error)
  __pyx_L6_skip:;
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_AddTraceback("cupyx.cusolver.gesvda", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_5cupyx_8cusolver_8gesvda(__pyx_self, __pyx_v_a, __pyx_v_compute_uv);

  /* function exit code */
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5cupyx_8cusolver_8gesvda(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_a, PyObject *__pyx_v_compute_uv) {
  PyObject *__pyx_v_a_ndim = NULL;
  PyObject *__pyx_v_a_shape = NULL;
  PyObject *__pyx_v_m = NULL;
  PyObject *__pyx_v_n = NULL;
  PyObject *__pyx_v_helper = NULL;
  PyObject *__pyx_v_solver = NULL;
  PyObject *__pyx_v_s_dtype = NULL;
  PyObject *__pyx_v_handle = NULL;
  PyObject *__pyx_v_jobz = NULL;
  PyObject *__pyx_v_rank = NULL;
  PyObject *__pyx_v_batch_size = NULL;
  PyObject *__pyx_v_lda = NULL;
  PyObject *__pyx_v_stride_a = NULL;
  PyObject *__pyx_v_s = NULL;
  PyObject *__pyx_v_stride_s = NULL;
  PyObject *__pyx_v_ldu = NULL;
  PyObject *__pyx_v_ldv = NULL;
  PyObject *__pyx_v_u = NULL;
  PyObject *__pyx_v_v = NULL;
  PyObject *__pyx_v_stride_u = NULL;
  PyObject *__pyx_v_stride_v = NULL;
  PyObject *__pyx_v_lwork = NULL;
  PyObject *__pyx_v_work = NULL;
  PyObject *__pyx_v_info = NULL;
  PyObject *__pyx_v_r_norm = NULL;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  size_t __pyx_t_4;
  int __pyx_t_5;
  int __pyx_t_6;
  PyObject *__pyx_t_7 = NULL;
  PyObject *(*__pyx_t_8)(PyObject *);
  PyObject *__pyx_t_9 = NULL;
  PyObject *__pyx_t_10 = NULL;
  PyObject *__pyx_t_11 = NULL;
  PyObject *__pyx_t_12 = NULL;
  PyObject *__pyx_t_13 = NULL;
  PyObject *__pyx_t_14 = NULL;
  PyObject *__pyx_t_15 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("gesvda", 0);
  __Pyx_INCREF(__pyx_v_a);

  /* "cupyx/cusolver.pyx":402
 *             A tuple of ``(u, s, v)``.
 *     """
 *     if not check_availability('gesvda'):             # <<<<<<<<<<<<<<
 *         raise RuntimeError('gesvda is not available.')
 * 
*/
  __pyx_t_2 = NULL;
  __Pyx_GetModuleGlobalName(__pyx_t_3, __pyx_mstate_global->__pyx_n_u_check_availability); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 402, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_4 = 1;
  #if CYTHON_UNPACK_METHODS
  if (unlikely(PyMethod_Check(__pyx_t_3))) {
    __pyx_t_2 = PyMethod_GET_SELF(__pyx_t_3);
    assert(__pyx_t_2);
    PyObject* __pyx__function = PyMethod_GET_FUNCTION(__pyx_t_3);
    __Pyx_INCREF(__pyx_t_2);
    __Pyx_INCREF(__pyx__function);
    __Pyx_DECREF_SET(__pyx_t_3, __pyx__function);
    __pyx_t_4 = 0;
  }
  #endif
  {
    PyObject *__pyx_callargs[2] = {__pyx_t_2, __pyx_mstate_global->__pyx_n_u_gesvda};
    __pyx_t_1 = __Pyx_PyObject_FastCall((PyObject*)__pyx_t_3, __pyx_callargs+__pyx_t_4, (2-__pyx_t_4) | (__pyx_t_4*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
    __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 402, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
  }
  __pyx_t_5 = __Pyx_PyObject_IsTrue(__pyx_t_1); if (unlikely((__pyx_t_5 < 0))) __PYX_ERR(0, 402, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_6 = (!__pyx_t_5);
  if (unlikely(__pyx_t_6)) {

    /* "cupyx/cusolver.pyx":403
 *     """
 *     if not check_availability('gesvda'):
 *         raise RuntimeError('gesvda is not available.')             # <<<<<<<<<<<<<<
 * 
 *     assert a.ndim >= 2
*/
    __pyx_t_3 = NULL;
    __pyx_t_4 = 1;
    {
      PyObject *__pyx_callargs[2] = {__pyx_t_3, __pyx_mstate_global->__pyx_kp_u_gesvda_is_not_available};
      __pyx_t_1 = __Pyx_PyObject_FastCall((PyObject*)(((PyTypeObject*)PyExc_RuntimeError)), __pyx_callargs+__pyx_t_4, (2-__pyx_t_4) | (__pyx_t_4*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
      __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
      if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 403, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
    }
    __Pyx_Raise(__pyx_t_1, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __PYX_ERR(0, 403, __pyx_L1_error)

    /* "cupyx/cusolver.pyx":402
 *             A tuple of ``(u, s, v)``.
 *     """
 *     if not check_availability('gesvda'):             # <<<<<<<<<<<<<<
 *         raise RuntimeError('gesvda is not available.')
 * 
*/
  }

  /* "cupyx/cusolver.pyx":405
 *         raise RuntimeError('gesvda is not available.')
 * 
 *     assert a.ndim >= 2             # <<<<<<<<<<<<<<
 *     a_ndim = a.ndim
 *     a_shape = a.shape
*/
  #ifndef CYTHON_WITHOUT_ASSERTIONS
  if (unlikely(__pyx_assertions_enabled())) {
    __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_v_a, __pyx_mstate_global->__pyx_n_u_ndim); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 405, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_t_3 = PyObject_RichCompare(__pyx_t_1, __pyx_mstate_global->__pyx_int_2, Py_GE); __Pyx_XGOTREF(__pyx_t_3); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 405, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __pyx_t_6 = __Pyx_PyObject_IsTrue(__pyx_t_3); if (unlikely((__pyx_t_6 < 0))) __PYX_ERR(0, 405, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    if (unlikely(!__pyx_t_6)) {
      __Pyx_Raise(((PyObject *)(((PyTypeObject*)PyExc_AssertionError))), 0, 0, 0);
      __PYX_ERR(0, 405, __pyx_L1_error)
    }
  }
  #else
  if ((1)); else __PYX_ERR(0, 405, __pyx_L1_error)
  #endif

  /* "cupyx/cusolver.pyx":406
 * 
 *     assert a.ndim >= 2
 *     a_ndim = a.ndim             # <<<<<<<<<<<<<<
 *     a_shape = a.shape
 *     m, n = a_shape[-2:]
*/
  __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_v_a, __pyx_mstate_global->__pyx_n_u_ndim); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 406, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_v_a_ndim = __pyx_t_3;
  __pyx_t_3 = 0;

  /* "cupyx/cusolver.pyx":407
 *     assert a.ndim >= 2
 *     a_ndim = a.ndim
 *     a_shape = a.shape             # <<<<<<<<<<<<<<
 *     m, n = a_shape[-2:]
 *     assert m >= n
*/
  __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_v_a, __pyx_mstate_global->__pyx_n_u_shape); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 407, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_v_a_shape = __pyx_t_3;
  __pyx_t_3 = 0;

  /* "cupyx/cusolver.pyx":408
 *     a_ndim = a.ndim
 *     a_shape = a.shape
 *     m, n = a_shape[-2:]             # <<<<<<<<<<<<<<
 *     assert m >= n
 * 
*/
  __pyx_t_3 = __Pyx_PyObject_GetSlice(__pyx_v_a_shape, -2L, 0, NULL, NULL, &__pyx_mstate_global->__pyx_slice[0], 1, 0, 1); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 408, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  if ((likely(PyTuple_CheckExact(__pyx_t_3))) || (PyList_CheckExact(__pyx_t_3))) {
    PyObject* sequence = __pyx_t_3;
    Py_ssize_t size = __Pyx_PySequence_SIZE(sequence);
    if (unlikely(size != 2)) {
      if (size > 2) __Pyx_RaiseTooManyValuesError(2);
      else if (size >= 0) __Pyx_RaiseNeedMoreValuesError(size);
      __PYX_ERR(0, 408, __pyx_L1_error)
    }
    #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
    if (likely(PyTuple_CheckExact(sequence))) {
      __pyx_t_1 = PyTuple_GET_ITEM(sequence, 0);
      __Pyx_INCREF(__pyx_t_1);
      __pyx_t_2 = PyTuple_GET_ITEM(sequence, 1);
      __Pyx_INCREF(__pyx_t_2);
    } else {
      __pyx_t_1 = __Pyx_PyList_GetItemRefFast(sequence, 0, __Pyx_ReferenceSharing_SharedReference);
      if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 408, __pyx_L1_error)
      __Pyx_XGOTREF(__pyx_t_1);
      __pyx_t_2 = __Pyx_PyList_GetItemRefFast(sequence, 1, __Pyx_ReferenceSharing_SharedReference);
      if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 408, __pyx_L1_error)
      __Pyx_XGOTREF(__pyx_t_2);
    }
    #else
    __pyx_t_1 = __Pyx_PySequence_ITEM(sequence, 0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 408, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_t_2 = __Pyx_PySequence_ITEM(sequence, 1); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 408, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    #endif
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  } else {
    Py_ssize_t index = -1;
    __pyx_t_7 = PyObject_GetIter(__pyx_t_3); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 408, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_7);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __pyx_t_8 = (CYTHON_COMPILING_IN_LIMITED_API) ? PyIter_Next : __Pyx_PyObject_GetIterNextFunc(__pyx_t_7);
    index = 0; __pyx_t_1 = __pyx_t_8(__pyx_t_7); if (unlikely(!__pyx_t_1)) goto __pyx_L4_unpacking_failed;
    __Pyx_GOTREF(__pyx_t_1);
    index = 1; __pyx_t_2 = __pyx_t_8(__pyx_t_7); if (unlikely(!__pyx_t_2)) goto __pyx_L4_unpacking_failed;
    __Pyx_GOTREF(__pyx_t_2);
    if (__Pyx_IternextUnpackEndCheck(__pyx_t_8(__pyx_t_7), 2) < (0)) __PYX_ERR(0, 408, __pyx_L1_error)
    __pyx_t_8 = NULL;
    __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
    goto __pyx_L5_unpacking_done;
    __pyx_L4_unpacking_failed:;
    __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
    __pyx_t_8 = NULL;
    if (__Pyx_IterFinish() == 0) __Pyx_RaiseNeedMoreValuesError(index);
    __PYX_ERR(0, 408, __pyx_L1_error)
    __pyx_L5_unpacking_done:;
  }
  __pyx_v_m = __pyx_t_1;
  __pyx_t_1 = 0;
  __pyx_v_n = __pyx_t_2;
  __pyx_t_2 = 0;

  /* "cupyx/cusolver.pyx":409
 *     a_shape = a.shape
 *     m, n = a_shape[-2:]
 *     assert m >= n             # <<<<<<<<<<<<<<
 * 
 *     if a.dtype == 'f':
*/
  #ifndef CYTHON_WITHOUT_ASSERTIONS
  if (unlikely(__pyx_assertions_enabled())) {
    __pyx_t_3 = PyObject_RichCompare(__pyx_v_m, __pyx_v_n, Py_GE); __Pyx_XGOTREF(__pyx_t_3); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 409, __pyx_L1_error)
    __pyx_t_6 = __Pyx_PyObject_IsTrue(__pyx_t_3); if (unlikely((__pyx_t_6 < 0))) __PYX_ERR(0, 409, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    if (unlikely(!__pyx_t_6)) {
      __Pyx_Raise(((PyObject *)(((PyTypeObject*)PyExc_AssertionError))), 0, 0, 0);
      __PYX_ERR(0, 409, __pyx_L1_error)
    }
  }
  #else
  if ((1)); else __PYX_ERR(0, 409, __pyx_L1_error)
  #endif

  /* "cupyx/cusolver.pyx":411
 *     assert m >= n
 * 
 *     if a.dtype == 'f':             # <<<<<<<<<<<<<<
 *         helper = _cusolver.sgesvdaStridedBatched_bufferSize
 *         solver = _cusolver.sgesvdaStridedBatched
*/
  __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_v_a, __pyx_mstate_global->__pyx_n_u_dtype); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 411, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_6 = (__Pyx_PyUnicode_Equals(__pyx_t_3, __pyx_mstate_global->__pyx_n_u_f, Py_EQ)); if (unlikely((__pyx_t_6 < 0))) __PYX_ERR(0, 411, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  if (__pyx_t_6) {

    /* "cupyx/cusolver.pyx":412
 * 
 *     if a.dtype == 'f':
 *         helper = _cusolver.sgesvdaStridedBatched_bufferSize             # <<<<<<<<<<<<<<
 *         solver = _cusolver.sgesvdaStridedBatched
 *         s_dtype = 'f'
*/
    __Pyx_GetModuleGlobalName(__pyx_t_3, __pyx_mstate_global->__pyx_n_u_cusolver); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 412, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_t_3, __pyx_mstate_global->__pyx_n_u_sgesvdaStridedBatched_bufferSize); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 412, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __pyx_v_helper = __pyx_t_2;
    __pyx_t_2 = 0;

    /* "cupyx/cusolver.pyx":413
 *     if a.dtype == 'f':
 *         helper = _cusolver.sgesvdaStridedBatched_bufferSize
 *         solver = _cusolver.sgesvdaStridedBatched             # <<<<<<<<<<<<<<
 *         s_dtype = 'f'
 *     elif a.dtype == 'd':
*/
    __Pyx_GetModuleGlobalName(__pyx_t_2, __pyx_mstate_global->__pyx_n_u_cusolver); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 413, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_t_2, __pyx_mstate_global->__pyx_n_u_sgesvdaStridedBatched); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 413, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __pyx_v_solver = __pyx_t_3;
    __pyx_t_3 = 0;

    /* "cupyx/cusolver.pyx":414
 *         helper = _cusolver.sgesvdaStridedBatched_bufferSize
 *         solver = _cusolver.sgesvdaStridedBatched
 *         s_dtype = 'f'             # <<<<<<<<<<<<<<
 *     elif a.dtype == 'd':
 *         helper = _cusolver.dgesvdaStridedBatched_bufferSize
*/
    __Pyx_INCREF(__pyx_mstate_global->__pyx_n_u_f);
    __pyx_v_s_dtype = __pyx_mstate_global->__pyx_n_u_f;

    /* "cupyx/cusolver.pyx":411
 *     assert m >= n
 * 
 *     if a.dtype == 'f':             # <<<<<<<<<<<<<<
 *         helper = _cusolver.sgesvdaStridedBatched_bufferSize
 *         solver = _cusolver.sgesvdaStridedBatched
*/
    goto __pyx_L6;
  }

  /* "cupyx/cusolver.pyx":415
 *         solver = _cusolver.sgesvdaStridedBatched
 *         s_dtype = 'f'
 *     elif a.dtype == 'd':             # <<<<<<<<<<<<<<
 *         helper = _cusolver.dgesvdaStridedBatched_bufferSize
 *         solver = _cusolver.dgesvdaStridedBatched
*/
  __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_v_a, __pyx_mstate_global->__pyx_n_u_dtype); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 415, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_6 = (__Pyx_PyUnicode_Equals(__pyx_t_3, __pyx_mstate_global->__pyx_n_u_d, Py_EQ)); if (unlikely((__pyx_t_6 < 0))) __PYX_ERR(0, 415, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  if (__pyx_t_6) {

    /* "cupyx/cusolver.pyx":416
 *         s_dtype = 'f'
 *     elif a.dtype == 'd':
 *         helper = _cusolver.dgesvdaStridedBatched_bufferSize             # <<<<<<<<<<<<<<
 *         solver = _cusolver.dgesvdaStridedBatched
 *         s_dtype = 'd'
*/
    __Pyx_GetModuleGlobalName(__pyx_t_3, __pyx_mstate_global->__pyx_n_u_cusolver); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 416, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_t_3, __pyx_mstate_global->__pyx_n_u_dgesvdaStridedBatched_bufferSize); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 416, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __pyx_v_helper = __pyx_t_2;
    __pyx_t_2 = 0;

    /* "cupyx/cusolver.pyx":417
 *     elif a.dtype == 'd':
 *         helper = _cusolver.dgesvdaStridedBatched_bufferSize
 *         solver = _cusolver.dgesvdaStridedBatched             # <<<<<<<<<<<<<<
 *         s_dtype = 'd'
 *     elif a.dtype == 'F':
*/
    __Pyx_GetModuleGlobalName(__pyx_t_2, __pyx_mstate_global->__pyx_n_u_cusolver); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 417, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_t_2, __pyx_mstate_global->__pyx_n_u_dgesvdaStridedBatched); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 417, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __pyx_v_solver = __pyx_t_3;
    __pyx_t_3 = 0;

    /* "cupyx/cusolver.pyx":418
 *         helper = _cusolver.dgesvdaStridedBatched_bufferSize
 *         solver = _cusolver.dgesvdaStridedBatched
 *         s_dtype = 'd'             # <<<<<<<<<<<<<<
 *     elif a.dtype == 'F':
 *         helper = _cusolver.cgesvdaStridedBatched_bufferSize
*/
    __Pyx_INCREF(__pyx_mstate_global->__pyx_n_u_d);
    __pyx_v_s_dtype = __pyx_mstate_global->__pyx_n_u_d;

    /* "cupyx/cusolver.pyx":415
 *         solver = _cusolver.sgesvdaStridedBatched
 *         s_dtype = 'f'
 *     elif a.dtype == 'd':             # <<<<<<<<<<<<<<
 *         helper = _cusolver.dgesvdaStridedBatched_bufferSize
 *         solver = _cusolver.dgesvdaStridedBatched
*/
    goto __pyx_L6;
  }

  /* "cupyx/cusolver.pyx":419
 *         solver = _cusolver.dgesvdaStridedBatched
 *         s_dtype = 'd'
 *     elif a.dtype == 'F':             # <<<<<<<<<<<<<<
 *         helper = _cusolver.cgesvdaStridedBatched_bufferSize
 *         solver = _cusolver.cgesvdaStridedBatched
*/
  __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_v_a, __pyx_mstate_global->__pyx_n_u_dtype); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 419, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_6 = (__Pyx_PyUnicode_Equals(__pyx_t_3, __pyx_mstate_global->__pyx_n_u_F, Py_EQ)); if (unlikely((__pyx_t_6 < 0))) __PYX_ERR(0, 419, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  if (__pyx_t_6) {

    /* "cupyx/cusolver.pyx":420
 *         s_dtype = 'd'
 *     elif a.dtype == 'F':
 *         helper = _cusolver.cgesvdaStridedBatched_bufferSize             # <<<<<<<<<<<<<<
 *         solver = _cusolver.cgesvdaStridedBatched
 *         s_dtype = 'f'
*/
    __Pyx_GetModuleGlobalName(__pyx_t_3, __pyx_mstate_global->__pyx_n_u_cusolver); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 420, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_t_3, __pyx_mstate_global->__pyx_n_u_cgesvdaStridedBatched_bufferSize); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 420, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __pyx_v_helper = __pyx_t_2;
    __pyx_t_2 = 0;

    /* "cupyx/cusolver.pyx":421
 *     elif a.dtype == 'F':
 *         helper = _cusolver.cgesvdaStridedBatched_bufferSize
 *         solver = _cusolver.cgesvdaStridedBatched             # <<<<<<<<<<<<<<
 *         s_dtype = 'f'
 *     elif a.dtype == 'D':
*/
    __Pyx_GetModuleGlobalName(__pyx_t_2, __pyx_mstate_global->__pyx_n_u_cusolver); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 421, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_t_2, __pyx_mstate_global->__pyx_n_u_cgesvdaStridedBatched); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 421, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __pyx_v_solver = __pyx_t_3;
    __pyx_t_3 = 0;

    /* "cupyx/cusolver.pyx":422
 *         helper = _cusolver.cgesvdaStridedBatched_bufferSize
 *         solver = _cusolver.cgesvdaStridedBatched
 *         s_dtype = 'f'             # <<<<<<<<<<<<<<
 *     elif a.dtype == 'D':
 *         helper = _cusolver.zgesvdaStridedBatched_bufferSize
*/
    __Pyx_INCREF(__pyx_mstate_global->__pyx_n_u_f);
    __pyx_v_s_dtype = __pyx_mstate_global->__pyx_n_u_f;

    /* "cupyx/cusolver.pyx":419
 *         solver = _cusolver.dgesvdaStridedBatched
 *         s_dtype = 'd'
 *     elif a.dtype == 'F':             # <<<<<<<<<<<<<<
 *         helper = _cusolver.cgesvdaStridedBatched_bufferSize
 *         solver = _cusolver.cgesvdaStridedBatched
*/
    goto __pyx_L6;
  }

  /* "cupyx/cusolver.pyx":423
 *         solver = _cusolver.cgesvdaStridedBatched
 *         s_dtype = 'f'
 *     elif a.dtype == 'D':             # <<<<<<<<<<<<<<
 *         helper = _cusolver.zgesvdaStridedBatched_bufferSize
 *         solver = _cusolver.zgesvdaStridedBatched
*/
  __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_v_a, __pyx_mstate_global->__pyx_n_u_dtype); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 423, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_6 = (__Pyx_PyUnicode_Equals(__pyx_t_3, __pyx_mstate_global->__pyx_n_u_D, Py_EQ)); if (unlikely((__pyx_t_6 < 0))) __PYX_ERR(0, 423, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  if (likely(__pyx_t_6)) {

    /* "cupyx/cusolver.pyx":424
 *         s_dtype = 'f'
 *     elif a.dtype == 'D':
 *         helper = _cusolver.zgesvdaStridedBatched_bufferSize             # <<<<<<<<<<<<<<
 *         solver = _cusolver.zgesvdaStridedBatched
 *         s_dtype = 'd'
*/
    __Pyx_GetModuleGlobalName(__pyx_t_3, __pyx_mstate_global->__pyx_n_u_cusolver); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 424, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_t_3, __pyx_mstate_global->__pyx_n_u_zgesvdaStridedBatched_bufferSize); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 424, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __pyx_v_helper = __pyx_t_2;
    __pyx_t_2 = 0;

    /* "cupyx/cusolver.pyx":425
 *     elif a.dtype == 'D':
 *         helper = _cusolver.zgesvdaStridedBatched_bufferSize
 *         solver = _cusolver.zgesvdaStridedBatched             # <<<<<<<<<<<<<<
 *         s_dtype = 'd'
 *     else:
*/
    __Pyx_GetModuleGlobalName(__pyx_t_2, __pyx_mstate_global->__pyx_n_u_cusolver); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 425, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_t_2, __pyx_mstate_global->__pyx_n_u_zgesvdaStridedBatched); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 425, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __pyx_v_solver = __pyx_t_3;
    __pyx_t_3 = 0;

    /* "cupyx/cusolver.pyx":426
 *         helper = _cusolver.zgesvdaStridedBatched_bufferSize
 *         solver = _cusolver.zgesvdaStridedBatched
 *         s_dtype = 'd'             # <<<<<<<<<<<<<<
 *     else:
 *         raise TypeError
*/
    __Pyx_INCREF(__pyx_mstate_global->__pyx_n_u_d);
    __pyx_v_s_dtype = __pyx_mstate_global->__pyx_n_u_d;

    /* "cupyx/cusolver.pyx":423
 *         solver = _cusolver.cgesvdaStridedBatched
 *         s_dtype = 'f'
 *     elif a.dtype == 'D':             # <<<<<<<<<<<<<<
 *         helper = _cusolver.zgesvdaStridedBatched_bufferSize
 *         solver = _cusolver.zgesvdaStridedBatched
*/
    goto __pyx_L6;
  }

  /* "cupyx/cusolver.pyx":428
 *         s_dtype = 'd'
 *     else:
 *         raise TypeError             # <<<<<<<<<<<<<<
 * 
 *     handle = _device.get_cusolver_handle()
*/
  /*else*/ {
    __Pyx_Raise(((PyObject *)(((PyTypeObject*)PyExc_TypeError))), 0, 0, 0);
    __PYX_ERR(0, 428, __pyx_L1_error)
  }
  __pyx_L6:;

  /* "cupyx/cusolver.pyx":430
 *         raise TypeError
 * 
 *     handle = _device.get_cusolver_handle()             # <<<<<<<<<<<<<<
 *     if compute_uv:
 *         jobz = _cusolver.CUSOLVER_EIG_MODE_VECTOR
*/
  __pyx_t_2 = NULL;
  __Pyx_GetModuleGlobalName(__pyx_t_1, __pyx_mstate_global->__pyx_n_u_device); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 430, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_7 = __Pyx_PyObject_GetAttrStr(__pyx_t_1, __pyx_mstate_global->__pyx_n_u_get_cusolver_handle); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 430, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_7);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_4 = 1;
  #if CYTHON_UNPACK_METHODS
  if (unlikely(PyMethod_Check(__pyx_t_7))) {
    __pyx_t_2 = PyMethod_GET_SELF(__pyx_t_7);
    assert(__pyx_t_2);
    PyObject* __pyx__function = PyMethod_GET_FUNCTION(__pyx_t_7);
    __Pyx_INCREF(__pyx_t_2);
    __Pyx_INCREF(__pyx__function);
    __Pyx_DECREF_SET(__pyx_t_7, __pyx__function);
    __pyx_t_4 = 0;
  }
  #endif
  {
    PyObject *__pyx_callargs[2] = {__pyx_t_2, NULL};
    __pyx_t_3 = __Pyx_PyObject_FastCall((PyObject*)__pyx_t_7, __pyx_callargs+__pyx_t_4, (1-__pyx_t_4) | (__pyx_t_4*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
    __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
    if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 430, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
  }
  __pyx_v_handle = __pyx_t_3;
  __pyx_t_3 = 0;

  /* "cupyx/cusolver.pyx":431
 * 
 *     handle = _device.get_cusolver_handle()
 *     if compute_uv:             # <<<<<<<<<<<<<<
 *         jobz = _cusolver.CUSOLVER_EIG_MODE_VECTOR
 *     else:
*/
  __pyx_t_6 = __Pyx_PyObject_IsTrue(__pyx_v_compute_uv); if (unlikely((__pyx_t_6 < 0))) __PYX_ERR(0, 431, __pyx_L1_error)
  if (__pyx_t_6) {

    /* "cupyx/cusolver.pyx":432
 *     handle = _device.get_cusolver_handle()
 *     if compute_uv:
 *         jobz = _cusolver.CUSOLVER_EIG_MODE_VECTOR             # <<<<<<<<<<<<<<
 *     else:
 *         jobz = _cusolver.CUSOLVER_EIG_MODE_NOVECTOR
*/
    __Pyx_GetModuleGlobalName(__pyx_t_3, __pyx_mstate_global->__pyx_n_u_cusolver); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 432, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_t_7 = __Pyx_PyObject_GetAttrStr(__pyx_t_3, __pyx_mstate_global->__pyx_n_u_CUSOLVER_EIG_MODE_VECTOR); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 432, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_7);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __pyx_v_jobz = __pyx_t_7;
    __pyx_t_7 = 0;

    /* "cupyx/cusolver.pyx":431
 * 
 *     handle = _device.get_cusolver_handle()
 *     if compute_uv:             # <<<<<<<<<<<<<<
 *         jobz = _cusolver.CUSOLVER_EIG_MODE_VECTOR
 *     else:
*/
    goto __pyx_L7;
  }

  /* "cupyx/cusolver.pyx":434
 *         jobz = _cusolver.CUSOLVER_EIG_MODE_VECTOR
 *     else:
 *         jobz = _cusolver.CUSOLVER_EIG_MODE_NOVECTOR             # <<<<<<<<<<<<<<
 *     rank = min(m, n)
 *     if a_ndim == 2:
*/
  /*else*/ {
    __Pyx_GetModuleGlobalName(__pyx_t_7, __pyx_mstate_global->__pyx_n_u_cusolver); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 434, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_7);
    __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_t_7, __pyx_mstate_global->__pyx_n_u_CUSOLVER_EIG_MODE_NOVECTOR); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 434, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
    __pyx_v_jobz = __pyx_t_3;
    __pyx_t_3 = 0;
  }
  __pyx_L7:;

  /* "cupyx/cusolver.pyx":435
 *     else:
 *         jobz = _cusolver.CUSOLVER_EIG_MODE_NOVECTOR
 *     rank = min(m, n)             # <<<<<<<<<<<<<<
 *     if a_ndim == 2:
 *         batch_size = 1
*/
  __Pyx_INCREF(__pyx_v_n);
  __pyx_t_3 = __pyx_v_n;
  __Pyx_INCREF(__pyx_v_m);
  __pyx_t_7 = __pyx_v_m;
  __pyx_t_1 = PyObject_RichCompare(__pyx_t_3, __pyx_t_7, Py_LT); __Pyx_XGOTREF(__pyx_t_1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 435, __pyx_L1_error)
  __pyx_t_6 = __Pyx_PyObject_IsTrue(__pyx_t_1); if (unlikely((__pyx_t_6 < 0))) __PYX_ERR(0, 435, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  if (__pyx_t_6) {
    __Pyx_INCREF(__pyx_t_3);
    __pyx_t_2 = __pyx_t_3;
  } else {
    __Pyx_INCREF(__pyx_t_7);
    __pyx_t_2 = __pyx_t_7;
  }
  __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_t_3 = __pyx_t_2;
  __Pyx_INCREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_v_rank = __pyx_t_3;
  __pyx_t_3 = 0;

  /* "cupyx/cusolver.pyx":436
 *         jobz = _cusolver.CUSOLVER_EIG_MODE_NOVECTOR
 *     rank = min(m, n)
 *     if a_ndim == 2:             # <<<<<<<<<<<<<<
 *         batch_size = 1
 *     else:
*/
  __pyx_t_6 = (__Pyx_PyLong_BoolEqObjC(__pyx_v_a_ndim, __pyx_mstate_global->__pyx_int_2, 2, 0)); if (unlikely((__pyx_t_6 < 0))) __PYX_ERR(0, 436, __pyx_L1_error)
  if (__pyx_t_6) {

    /* "cupyx/cusolver.pyx":437
 *     rank = min(m, n)
 *     if a_ndim == 2:
 *         batch_size = 1             # <<<<<<<<<<<<<<
 *     else:
 *         batch_size = _numpy.array(a_shape[:-2]).prod().item()
*/
    __Pyx_INCREF(__pyx_mstate_global->__pyx_int_1);
    __pyx_v_batch_size = __pyx_mstate_global->__pyx_int_1;

    /* "cupyx/cusolver.pyx":436
 *         jobz = _cusolver.CUSOLVER_EIG_MODE_NOVECTOR
 *     rank = min(m, n)
 *     if a_ndim == 2:             # <<<<<<<<<<<<<<
 *         batch_size = 1
 *     else:
*/
    goto __pyx_L8;
  }

  /* "cupyx/cusolver.pyx":439
 *         batch_size = 1
 *     else:
 *         batch_size = _numpy.array(a_shape[:-2]).prod().item()             # <<<<<<<<<<<<<<
 *     a = a.reshape((batch_size, m, n))
 *     a = _cupy.ascontiguousarray(a.transpose(0, 2, 1))
*/
  /*else*/ {
    __pyx_t_10 = NULL;
    __Pyx_GetModuleGlobalName(__pyx_t_11, __pyx_mstate_global->__pyx_n_u_numpy); if (unlikely(!__pyx_t_11)) __PYX_ERR(0, 439, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_11);
    __pyx_t_12 = __Pyx_PyObject_GetAttrStr(__pyx_t_11, __pyx_mstate_global->__pyx_n_u_array); if (unlikely(!__pyx_t_12)) __PYX_ERR(0, 439, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_12);
    __Pyx_DECREF(__pyx_t_11); __pyx_t_11 = 0;
    __pyx_t_11 = __Pyx_PyObject_GetSlice(__pyx_v_a_shape, 0, -2L, NULL, NULL, &__pyx_mstate_global->__pyx_slice[1], 0, 1, 1); if (unlikely(!__pyx_t_11)) __PYX_ERR(0, 439, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_11);
    __pyx_t_4 = 1;
    #if CYTHON_UNPACK_METHODS
    if (unlikely(PyMethod_Check(__pyx_t_12))) {
      __pyx_t_10 = PyMethod_GET_SELF(__pyx_t_12);
      assert(__pyx_t_10);
      PyObject* __pyx__function = PyMethod_GET_FUNCTION(__pyx_t_12);
      __Pyx_INCREF(__pyx_t_10);
      __Pyx_INCREF(__pyx__function);
      __Pyx_DECREF_SET(__pyx_t_12, __pyx__function);
      __pyx_t_4 = 0;
    }
    #endif
    {
      PyObject *__pyx_callargs[2] = {__pyx_t_10, __pyx_t_11};
      __pyx_t_9 = __Pyx_PyObject_FastCall((PyObject*)__pyx_t_12, __pyx_callargs+__pyx_t_4, (2-__pyx_t_4) | (__pyx_t_4*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
      __Pyx_XDECREF(__pyx_t_10); __pyx_t_10 = 0;
      __Pyx_DECREF(__pyx_t_11); __pyx_t_11 = 0;
      __Pyx_DECREF(__pyx_t_12); __pyx_t_12 = 0;
      if (unlikely(!__pyx_t_9)) __PYX_ERR(0, 439, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_9);
    }
    __pyx_t_1 = __pyx_t_9;
    __Pyx_INCREF(__pyx_t_1);
    __pyx_t_4 = 0;
    {
      PyObject *__pyx_callargs[2] = {__pyx_t_1, NULL};
      __pyx_t_7 = __Pyx_PyObject_FastCallMethod((PyObject*)__pyx_mstate_global->__pyx_n_u_prod, __pyx_callargs+__pyx_t_4, (1-__pyx_t_4) | (1*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
      __Pyx_XDECREF(__pyx_t_1); __pyx_t_1 = 0;
      __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
      if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 439, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_7);
    }
    __pyx_t_2 = __pyx_t_7;
    __Pyx_INCREF(__pyx_t_2);
    __pyx_t_4 = 0;
    {
      PyObject *__pyx_callargs[2] = {__pyx_t_2, NULL};
      __pyx_t_3 = __Pyx_PyObject_FastCallMethod((PyObject*)__pyx_mstate_global->__pyx_n_u_item, __pyx_callargs+__pyx_t_4, (1-__pyx_t_4) | (1*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
      __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
      __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
      if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 439, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_3);
    }
    __pyx_v_batch_size = __pyx_t_3;
    __pyx_t_3 = 0;
  }
  __pyx_L8:;

  /* "cupyx/cusolver.pyx":440
 *     else:
 *         batch_size = _numpy.array(a_shape[:-2]).prod().item()
 *     a = a.reshape((batch_size, m, n))             # <<<<<<<<<<<<<<
 *     a = _cupy.ascontiguousarray(a.transpose(0, 2, 1))
 *     lda = m
*/
  __pyx_t_7 = __pyx_v_a;
  __Pyx_INCREF(__pyx_t_7);
  __pyx_t_2 = PyTuple_New(3); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 440, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_INCREF(__pyx_v_batch_size);
  __Pyx_GIVEREF(__pyx_v_batch_size);
  if (__Pyx_PyTuple_SET_ITEM(__pyx_t_2, 0, __pyx_v_batch_size) != (0)) __PYX_ERR(0, 440, __pyx_L1_error);
  __Pyx_INCREF(__pyx_v_m);
  __Pyx_GIVEREF(__pyx_v_m);
  if (__Pyx_PyTuple_SET_ITEM(__pyx_t_2, 1, __pyx_v_m) != (0)) __PYX_ERR(0, 440, __pyx_L1_error);
  __Pyx_INCREF(__pyx_v_n);
  __Pyx_GIVEREF(__pyx_v_n);
  if (__Pyx_PyTuple_SET_ITEM(__pyx_t_2, 2, __pyx_v_n) != (0)) __PYX_ERR(0, 440, __pyx_L1_error);
  __pyx_t_4 = 0;
  {
    PyObject *__pyx_callargs[2] = {__pyx_t_7, __pyx_t_2};
    __pyx_t_3 = __Pyx_PyObject_FastCallMethod((PyObject*)__pyx_mstate_global->__pyx_n_u_reshape, __pyx_callargs+__pyx_t_4, (2-__pyx_t_4) | (1*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
    __Pyx_XDECREF(__pyx_t_7); __pyx_t_7 = 0;
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 440, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
  }
  __Pyx_DECREF_SET(__pyx_v_a, __pyx_t_3);
  __pyx_t_3 = 0;

  /* "cupyx/cusolver.pyx":441
 *         batch_size = _numpy.array(a_shape[:-2]).prod().item()
 *     a = a.reshape((batch_size, m, n))
 *     a = _cupy.ascontiguousarray(a.transpose(0, 2, 1))             # <<<<<<<<<<<<<<
 *     lda = m
 *     stride_a = lda * n
*/
  __pyx_t_2 = NULL;
  __Pyx_GetModuleGlobalName(__pyx_t_7, __pyx_mstate_global->__pyx_n_u_cupy); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 441, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_7);
  __pyx_t_9 = __Pyx_PyObject_GetAttrStr(__pyx_t_7, __pyx_mstate_global->__pyx_n_u_ascontiguousarray); if (unlikely(!__pyx_t_9)) __PYX_ERR(0, 441, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_9);
  __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
  __pyx_t_7 = __Pyx_PyObject_GetAttrStr(__pyx_v_a, __pyx_mstate_global->__pyx_n_u_transpose); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 441, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_7);
  __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_7, __pyx_mstate_global->__pyx_tuple[2], NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 441, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
  __pyx_t_4 = 1;
  #if CYTHON_UNPACK_METHODS
  if (unlikely(PyMethod_Check(__pyx_t_9))) {
    __pyx_t_2 = PyMethod_GET_SELF(__pyx_t_9);
    assert(__pyx_t_2);
    PyObject* __pyx__function = PyMethod_GET_FUNCTION(__pyx_t_9);
    __Pyx_INCREF(__pyx_t_2);
    __Pyx_INCREF(__pyx__function);
    __Pyx_DECREF_SET(__pyx_t_9, __pyx__function);
    __pyx_t_4 = 0;
  }
  #endif
  {
    PyObject *__pyx_callargs[2] = {__pyx_t_2, __pyx_t_1};
    __pyx_t_3 = __Pyx_PyObject_FastCall((PyObject*)__pyx_t_9, __pyx_callargs+__pyx_t_4, (2-__pyx_t_4) | (__pyx_t_4*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
    __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
    if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 441, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
  }
  __Pyx_DECREF_SET(__pyx_v_a, __pyx_t_3);
  __pyx_t_3 = 0;

  /* "cupyx/cusolver.pyx":442
 *     a = a.reshape((batch_size, m, n))
 *     a = _cupy.ascontiguousarray(a.transpose(0, 2, 1))
 *     lda = m             # <<<<<<<<<<<<<<
 *     stride_a = lda * n
 *     s = _cupy.empty((batch_size, rank), dtype=s_dtype)
*/
  __Pyx_INCREF(__pyx_v_m);
  __pyx_v_lda = __pyx_v_m;

  /* "cupyx/cusolver.pyx":443
 *     a = _cupy.ascontiguousarray(a.transpose(0, 2, 1))
 *     lda = m
 *     stride_a = lda * n             # <<<<<<<<<<<<<<
 *     s = _cupy.empty((batch_size, rank), dtype=s_dtype)
 *     stride_s = rank
*/
  __pyx_t_3 = PyNumber_Multiply(__pyx_v_lda, __pyx_v_n); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 443, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_v_stride_a = __pyx_t_3;
  __pyx_t_3 = 0;

  /* "cupyx/cusolver.pyx":444
 *     lda = m
 *     stride_a = lda * n
 *     s = _cupy.empty((batch_size, rank), dtype=s_dtype)             # <<<<<<<<<<<<<<
 *     stride_s = rank
 *     ldu = m
*/
  __pyx_t_9 = NULL;
  __Pyx_GetModuleGlobalName(__pyx_t_1, __pyx_mstate_global->__pyx_n_u_cupy); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 444, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_t_1, __pyx_mstate_global->__pyx_n_u_empty); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 444, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_1 = PyTuple_New(2); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 444, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_INCREF(__pyx_v_batch_size);
  __Pyx_GIVEREF(__pyx_v_batch_size);
  if (__Pyx_PyTuple_SET_ITEM(__pyx_t_1, 0, __pyx_v_batch_size) != (0)) __PYX_ERR(0, 444, __pyx_L1_error);
  __Pyx_INCREF(__pyx_v_rank);
  __Pyx_GIVEREF(__pyx_v_rank);
  if (__Pyx_PyTuple_SET_ITEM(__pyx_t_1, 1, __pyx_v_rank) != (0)) __PYX_ERR(0, 444, __pyx_L1_error);
  __pyx_t_4 = 1;
  #if CYTHON_UNPACK_METHODS
  if (unlikely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_9 = PyMethod_GET_SELF(__pyx_t_2);
    assert(__pyx_t_9);
    PyObject* __pyx__function = PyMethod_GET_FUNCTION(__pyx_t_2);
    __Pyx_INCREF(__pyx_t_9);
    __Pyx_INCREF(__pyx__function);
    __Pyx_DECREF_SET(__pyx_t_2, __pyx__function);
    __pyx_t_4 = 0;
  }
  #endif
  {
    PyObject *__pyx_callargs[2 + ((CYTHON_VECTORCALL) ? 1 : 0)] = {__pyx_t_9, __pyx_t_1};
    __pyx_t_7 = __Pyx_MakeVectorcallBuilderKwds(1); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 444, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_7);
    if (__Pyx_VectorcallBuilder_AddArg(__pyx_mstate_global->__pyx_n_u_dtype, __pyx_v_s_dtype, __pyx_t_7, __pyx_callargs+2, 0) < (0)) __PYX_ERR(0, 444, __pyx_L1_error)
    __pyx_t_3 = __Pyx_Object_Vectorcall_CallFromBuilder((PyObject*)__pyx_t_2, __pyx_callargs+__pyx_t_4, (2-__pyx_t_4) | (__pyx_t_4*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET), __pyx_t_7);
    __Pyx_XDECREF(__pyx_t_9); __pyx_t_9 = 0;
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 444, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
  }
  __pyx_v_s = __pyx_t_3;
  __pyx_t_3 = 0;

  /* "cupyx/cusolver.pyx":445
 *     stride_a = lda * n
 *     s = _cupy.empty((batch_size, rank), dtype=s_dtype)
 *     stride_s = rank             # <<<<<<<<<<<<<<
 *     ldu = m
 *     ldv = n
*/
  __Pyx_INCREF(__pyx_v_rank);
  __pyx_v_stride_s = __pyx_v_rank;

  /* "cupyx/cusolver.pyx":446
 *     s = _cupy.empty((batch_size, rank), dtype=s_dtype)
 *     stride_s = rank
 *     ldu = m             # <<<<<<<<<<<<<<
 *     ldv = n
 *     u = _cupy.empty((batch_size, rank, ldu), dtype=a.dtype, order='C')
*/
  __Pyx_INCREF(__pyx_v_m);
  __pyx_v_ldu = __pyx_v_m;

  /* "cupyx/cusolver.pyx":447
 *     stride_s = rank
 *     ldu = m
 *     ldv = n             # <<<<<<<<<<<<<<
 *     u = _cupy.empty((batch_size, rank, ldu), dtype=a.dtype, order='C')
 *     v = _cupy.empty((batch_size, rank, ldv), dtype=a.dtype, order='C')
*/
  __Pyx_INCREF(__pyx_v_n);
  __pyx_v_ldv = __pyx_v_n;

  /* "cupyx/cusolver.pyx":448
 *     ldu = m
 *     ldv = n
 *     u = _cupy.empty((batch_size, rank, ldu), dtype=a.dtype, order='C')             # <<<<<<<<<<<<<<
 *     v = _cupy.empty((batch_size, rank, ldv), dtype=a.dtype, order='C')
 *     stride_u = rank * ldu
*/
  __pyx_t_2 = NULL;
  __Pyx_GetModuleGlobalName(__pyx_t_7, __pyx_mstate_global->__pyx_n_u_cupy); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 448, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_7);
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_t_7, __pyx_mstate_global->__pyx_n_u_empty); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 448, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
  __pyx_t_7 = PyTuple_New(3); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 448, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_7);
  __Pyx_INCREF(__pyx_v_batch_size);
  __Pyx_GIVEREF(__pyx_v_batch_size);
  if (__Pyx_PyTuple_SET_ITEM(__pyx_t_7, 0, __pyx_v_batch_size) != (0)) __PYX_ERR(0, 448, __pyx_L1_error);
  __Pyx_INCREF(__pyx_v_rank);
  __Pyx_GIVEREF(__pyx_v_rank);
  if (__Pyx_PyTuple_SET_ITEM(__pyx_t_7, 1, __pyx_v_rank) != (0)) __PYX_ERR(0, 448, __pyx_L1_error);
  __Pyx_INCREF(__pyx_v_ldu);
  __Pyx_GIVEREF(__pyx_v_ldu);
  if (__Pyx_PyTuple_SET_ITEM(__pyx_t_7, 2, __pyx_v_ldu) != (0)) __PYX_ERR(0, 448, __pyx_L1_error);
  __pyx_t_9 = __Pyx_PyObject_GetAttrStr(__pyx_v_a, __pyx_mstate_global->__pyx_n_u_dtype); if (unlikely(!__pyx_t_9)) __PYX_ERR(0, 448, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_9);
  __pyx_t_4 = 1;
  #if CYTHON_UNPACK_METHODS
  if (unlikely(PyMethod_Check(__pyx_t_1))) {
    __pyx_t_2 = PyMethod_GET_SELF(__pyx_t_1);
    assert(__pyx_t_2);
    PyObject* __pyx__function = PyMethod_GET_FUNCTION(__pyx_t_1);
    __Pyx_INCREF(__pyx_t_2);
    __Pyx_INCREF(__pyx__function);
    __Pyx_DECREF_SET(__pyx_t_1, __pyx__function);
    __pyx_t_4 = 0;
  }
  #endif
  {
    PyObject *__pyx_callargs[2 + ((CYTHON_VECTORCALL) ? 2 : 0)] = {__pyx_t_2, __pyx_t_7};
    __pyx_t_12 = __Pyx_MakeVectorcallBuilderKwds(2); if (unlikely(!__pyx_t_12)) __PYX_ERR(0, 448, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_12);
    if (__Pyx_VectorcallBuilder_AddArg(__pyx_mstate_global->__pyx_n_u_dtype, __pyx_t_9, __pyx_t_12, __pyx_callargs+2, 0) < (0)) __PYX_ERR(0, 448, __pyx_L1_error)
    if (__Pyx_VectorcallBuilder_AddArg(__pyx_mstate_global->__pyx_n_u_order, __pyx_mstate_global->__pyx_n_u_C, __pyx_t_12, __pyx_callargs+2, 1) < (0)) __PYX_ERR(0, 448, __pyx_L1_error)
    __pyx_t_3 = __Pyx_Object_Vectorcall_CallFromBuilder((PyObject*)__pyx_t_1, __pyx_callargs+__pyx_t_4, (2-__pyx_t_4) | (__pyx_t_4*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET), __pyx_t_12);
    __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
    __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
    __Pyx_DECREF(__pyx_t_12); __pyx_t_12 = 0;
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 448, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
  }
  __pyx_v_u = __pyx_t_3;
  __pyx_t_3 = 0;

  /* "cupyx/cusolver.pyx":449
 *     ldv = n
 *     u = _cupy.empty((batch_size, rank, ldu), dtype=a.dtype, order='C')
 *     v = _cupy.empty((batch_size, rank, ldv), dtype=a.dtype, order='C')             # <<<<<<<<<<<<<<
 *     stride_u = rank * ldu
 *     stride_v = rank * ldv
*/
  __pyx_t_1 = NULL;
  __Pyx_GetModuleGlobalName(__pyx_t_12, __pyx_mstate_global->__pyx_n_u_cupy); if (unlikely(!__pyx_t_12)) __PYX_ERR(0, 449, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_12);
  __pyx_t_9 = __Pyx_PyObject_GetAttrStr(__pyx_t_12, __pyx_mstate_global->__pyx_n_u_empty); if (unlikely(!__pyx_t_9)) __PYX_ERR(0, 449, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_9);
  __Pyx_DECREF(__pyx_t_12); __pyx_t_12 = 0;
  __pyx_t_12 = PyTuple_New(3); if (unlikely(!__pyx_t_12)) __PYX_ERR(0, 449, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_12);
  __Pyx_INCREF(__pyx_v_batch_size);
  __Pyx_GIVEREF(__pyx_v_batch_size);
  if (__Pyx_PyTuple_SET_ITEM(__pyx_t_12, 0, __pyx_v_batch_size) != (0)) __PYX_ERR(0, 449, __pyx_L1_error);
  __Pyx_INCREF(__pyx_v_rank);
  __Pyx_GIVEREF(__pyx_v_rank);
  if (__Pyx_PyTuple_SET_ITEM(__pyx_t_12, 1, __pyx_v_rank) != (0)) __PYX_ERR(0, 449, __pyx_L1_error);
  __Pyx_INCREF(__pyx_v_ldv);
  __Pyx_GIVEREF(__pyx_v_ldv);
  if (__Pyx_PyTuple_SET_ITEM(__pyx_t_12, 2, __pyx_v_ldv) != (0)) __PYX_ERR(0, 449, __pyx_L1_error);
  __pyx_t_7 = __Pyx_PyObject_GetAttrStr(__pyx_v_a, __pyx_mstate_global->__pyx_n_u_dtype); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 449, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_7);
  __pyx_t_4 = 1;
  #if CYTHON_UNPACK_METHODS
  if (unlikely(PyMethod_Check(__pyx_t_9))) {
    __pyx_t_1 = PyMethod_GET_SELF(__pyx_t_9);
    assert(__pyx_t_1);
    PyObject* __pyx__function = PyMethod_GET_FUNCTION(__pyx_t_9);
    __Pyx_INCREF(__pyx_t_1);
    __Pyx_INCREF(__pyx__function);
    __Pyx_DECREF_SET(__pyx_t_9, __pyx__function);
    __pyx_t_4 = 0;
  }
  #endif
  {
    PyObject *__pyx_callargs[2 + ((CYTHON_VECTORCALL) ? 2 : 0)] = {__pyx_t_1, __pyx_t_12};
    __pyx_t_2 = __Pyx_MakeVectorcallBuilderKwds(2); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 449, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    if (__Pyx_VectorcallBuilder_AddArg(__pyx_mstate_global->__pyx_n_u_dtype, __pyx_t_7, __pyx_t_2, __pyx_callargs+2, 0) < (0)) __PYX_ERR(0, 449, __pyx_L1_error)
    if (__Pyx_VectorcallBuilder_AddArg(__pyx_mstate_global->__pyx_n_u_order, __pyx_mstate_global->__pyx_n_u_C, __pyx_t_2, __pyx_callargs+2, 1) < (0)) __PYX_ERR(0, 449, __pyx_L1_error)
    __pyx_t_3 = __Pyx_Object_Vectorcall_CallFromBuilder((PyObject*)__pyx_t_9, __pyx_callargs+__pyx_t_4, (2-__pyx_t_4) | (__pyx_t_4*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET), __pyx_t_2);
    __Pyx_XDECREF(__pyx_t_1); __pyx_t_1 = 0;
    __Pyx_DECREF(__pyx_t_12); __pyx_t_12 = 0;
    __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
    if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 449, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
  }
  __pyx_v_v = __pyx_t_3;
  __pyx_t_3 = 0;

  /* "cupyx/cusolver.pyx":450
 *     u = _cupy.empty((batch_size, rank, ldu), dtype=a.dtype, order='C')
 *     v = _cupy.empty((batch_size, rank, ldv), dtype=a.dtype, order='C')
 *     stride_u = rank * ldu             # <<<<<<<<<<<<<<
 *     stride_v = rank * ldv
 *     lwork = helper(handle, jobz, rank, m, n, a.data.ptr, lda, stride_a,
*/
  __pyx_t_3 = PyNumber_Multiply(__pyx_v_rank, __pyx_v_ldu); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 450, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_v_stride_u = __pyx_t_3;
  __pyx_t_3 = 0;

  /* "cupyx/cusolver.pyx":451
 *     v = _cupy.empty((batch_size, rank, ldv), dtype=a.dtype, order='C')
 *     stride_u = rank * ldu
 *     stride_v = rank * ldv             # <<<<<<<<<<<<<<
 *     lwork = helper(handle, jobz, rank, m, n, a.data.ptr, lda, stride_a,
 *                    s.data.ptr, stride_s, u.data.ptr, ldu, stride_u,
*/
  __pyx_t_3 = PyNumber_Multiply(__pyx_v_rank, __pyx_v_ldv); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 451, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_v_stride_v = __pyx_t_3;
  __pyx_t_3 = 0;

  /* "cupyx/cusolver.pyx":452
 *     stride_u = rank * ldu
 *     stride_v = rank * ldv
 *     lwork = helper(handle, jobz, rank, m, n, a.data.ptr, lda, stride_a,             # <<<<<<<<<<<<<<
 *                    s.data.ptr, stride_s, u.data.ptr, ldu, stride_u,
 *                    v.data.ptr, ldv, stride_v, batch_size)
*/
  __pyx_t_9 = NULL;
  __Pyx_INCREF(__pyx_v_helper);
  __pyx_t_2 = __pyx_v_helper; 
  __pyx_t_7 = __Pyx_PyObject_GetAttrStr(__pyx_v_a, __pyx_mstate_global->__pyx_n_u_data); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 452, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_7);
  __pyx_t_12 = __Pyx_PyObject_GetAttrStr(__pyx_t_7, __pyx_mstate_global->__pyx_n_u_ptr); if (unlikely(!__pyx_t_12)) __PYX_ERR(0, 452, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_12);
  __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;

  /* "cupyx/cusolver.pyx":453
 *     stride_v = rank * ldv
 *     lwork = helper(handle, jobz, rank, m, n, a.data.ptr, lda, stride_a,
 *                    s.data.ptr, stride_s, u.data.ptr, ldu, stride_u,             # <<<<<<<<<<<<<<
 *                    v.data.ptr, ldv, stride_v, batch_size)
 *     work = _cupy.empty((lwork,), dtype=a.dtype)
*/
  __pyx_t_7 = __Pyx_PyObject_GetAttrStr(__pyx_v_s, __pyx_mstate_global->__pyx_n_u_data); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 453, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_7);
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_t_7, __pyx_mstate_global->__pyx_n_u_ptr); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 453, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
  __pyx_t_7 = __Pyx_PyObject_GetAttrStr(__pyx_v_u, __pyx_mstate_global->__pyx_n_u_data); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 453, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_7);
  __pyx_t_11 = __Pyx_PyObject_GetAttrStr(__pyx_t_7, __pyx_mstate_global->__pyx_n_u_ptr); if (unlikely(!__pyx_t_11)) __PYX_ERR(0, 453, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_11);
  __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;

  /* "cupyx/cusolver.pyx":454
 *     lwork = helper(handle, jobz, rank, m, n, a.data.ptr, lda, stride_a,
 *                    s.data.ptr, stride_s, u.data.ptr, ldu, stride_u,
 *                    v.data.ptr, ldv, stride_v, batch_size)             # <<<<<<<<<<<<<<
 *     work = _cupy.empty((lwork,), dtype=a.dtype)
 *     info = _cupy.empty((batch_size,), dtype=_numpy.int32)
*/
  __pyx_t_7 = __Pyx_PyObject_GetAttrStr(__pyx_v_v, __pyx_mstate_global->__pyx_n_u_data); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 454, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_7);
  __pyx_t_10 = __Pyx_PyObject_GetAttrStr(__pyx_t_7, __pyx_mstate_global->__pyx_n_u_ptr); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 454, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_10);
  __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
  __pyx_t_4 = 1;
  #if CYTHON_UNPACK_METHODS
  if (unlikely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_9 = PyMethod_GET_SELF(__pyx_t_2);
    assert(__pyx_t_9);
    PyObject* __pyx__function = PyMethod_GET_FUNCTION(__pyx_t_2);
    __Pyx_INCREF(__pyx_t_9);
    __Pyx_INCREF(__pyx__function);
    __Pyx_DECREF_SET(__pyx_t_2, __pyx__function);
    __pyx_t_4 = 0;
  }
  #endif
  {
    PyObject *__pyx_callargs[18] = {__pyx_t_9, __pyx_v_handle, __pyx_v_jobz, __pyx_v_rank, __pyx_v_m, __pyx_v_n, __pyx_t_12, __pyx_v_lda, __pyx_v_stride_a, __pyx_t_1, __pyx_v_stride_s, __pyx_t_11, __pyx_v_ldu, __pyx_v_stride_u, __pyx_t_10, __pyx_v_ldv, __pyx_v_stride_v, __pyx_v_batch_size};
    __pyx_t_3 = __Pyx_PyObject_FastCall((PyObject*)__pyx_t_2, __pyx_callargs+__pyx_t_4, (18-__pyx_t_4) | (__pyx_t_4*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
    __Pyx_XDECREF(__pyx_t_9); __pyx_t_9 = 0;
    __Pyx_DECREF(__pyx_t_12); __pyx_t_12 = 0;
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __Pyx_DECREF(__pyx_t_11); __pyx_t_11 = 0;
    __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 452, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
  }
  __pyx_v_lwork = __pyx_t_3;
  __pyx_t_3 = 0;

  /* "cupyx/cusolver.pyx":455
 *                    s.data.ptr, stride_s, u.data.ptr, ldu, stride_u,
 *                    v.data.ptr, ldv, stride_v, batch_size)
 *     work = _cupy.empty((lwork,), dtype=a.dtype)             # <<<<<<<<<<<<<<
 *     info = _cupy.empty((batch_size,), dtype=_numpy.int32)
 *     r_norm = _numpy.empty((batch_size,), dtype=_numpy.float64)
*/
  __pyx_t_2 = NULL;
  __Pyx_GetModuleGlobalName(__pyx_t_10, __pyx_mstate_global->__pyx_n_u_cupy); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 455, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_10);
  __pyx_t_11 = __Pyx_PyObject_GetAttrStr(__pyx_t_10, __pyx_mstate_global->__pyx_n_u_empty); if (unlikely(!__pyx_t_11)) __PYX_ERR(0, 455, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_11);
  __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
  __pyx_t_10 = PyTuple_New(1); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 455, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_10);
  __Pyx_INCREF(__pyx_v_lwork);
  __Pyx_GIVEREF(__pyx_v_lwork);
  if (__Pyx_PyTuple_SET_ITEM(__pyx_t_10, 0, __pyx_v_lwork) != (0)) __PYX_ERR(0, 455, __pyx_L1_error);
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_v_a, __pyx_mstate_global->__pyx_n_u_dtype); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 455, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_4 = 1;
  #if CYTHON_UNPACK_METHODS
  if (unlikely(PyMethod_Check(__pyx_t_11))) {
    __pyx_t_2 = PyMethod_GET_SELF(__pyx_t_11);
    assert(__pyx_t_2);
    PyObject* __pyx__function = PyMethod_GET_FUNCTION(__pyx_t_11);
    __Pyx_INCREF(__pyx_t_2);
    __Pyx_INCREF(__pyx__function);
    __Pyx_DECREF_SET(__pyx_t_11, __pyx__function);
    __pyx_t_4 = 0;
  }
  #endif
  {
    PyObject *__pyx_callargs[2 + ((CYTHON_VECTORCALL) ? 1 : 0)] = {__pyx_t_2, __pyx_t_10};
    __pyx_t_12 = __Pyx_MakeVectorcallBuilderKwds(1); if (unlikely(!__pyx_t_12)) __PYX_ERR(0, 455, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_12);
    if (__Pyx_VectorcallBuilder_AddArg(__pyx_mstate_global->__pyx_n_u_dtype, __pyx_t_1, __pyx_t_12, __pyx_callargs+2, 0) < (0)) __PYX_ERR(0, 455, __pyx_L1_error)
    __pyx_t_3 = __Pyx_Object_Vectorcall_CallFromBuilder((PyObject*)__pyx_t_11, __pyx_callargs+__pyx_t_4, (2-__pyx_t_4) | (__pyx_t_4*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET), __pyx_t_12);
    __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __Pyx_DECREF(__pyx_t_12); __pyx_t_12 = 0;
    __Pyx_DECREF(__pyx_t_11); __pyx_t_11 = 0;
    if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 455, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
  }
  __pyx_v_work = __pyx_t_3;
  __pyx_t_3 = 0;

  /* "cupyx/cusolver.pyx":456
 *                    v.data.ptr, ldv, stride_v, batch_size)
 *     work = _cupy.empty((lwork,), dtype=a.dtype)
 *     info = _cupy.empty((batch_size,), dtype=_numpy.int32)             # <<<<<<<<<<<<<<
 *     r_norm = _numpy.empty((batch_size,), dtype=_numpy.float64)
 *     solver(handle, jobz, rank, m, n, a.data.ptr, lda, stride_a, s.data.ptr,
*/
  __pyx_t_11 = NULL;
  __Pyx_GetModuleGlobalName(__pyx_t_12, __pyx_mstate_global->__pyx_n_u_cupy); if (unlikely(!__pyx_t_12)) __PYX_ERR(0, 456, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_12);
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_t_12, __pyx_mstate_global->__pyx_n_u_empty); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 456, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_12); __pyx_t_12 = 0;
  __pyx_t_12 = PyTuple_New(1); if (unlikely(!__pyx_t_12)) __PYX_ERR(0, 456, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_12);
  __Pyx_INCREF(__pyx_v_batch_size);
  __Pyx_GIVEREF(__pyx_v_batch_size);
  if (__Pyx_PyTuple_SET_ITEM(__pyx_t_12, 0, __pyx_v_batch_size) != (0)) __PYX_ERR(0, 456, __pyx_L1_error);
  __Pyx_GetModuleGlobalName(__pyx_t_10, __pyx_mstate_global->__pyx_n_u_numpy); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 456, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_10);
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_t_10, __pyx_mstate_global->__pyx_n_u_int32); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 456, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
  __pyx_t_4 = 1;
  #if CYTHON_UNPACK_METHODS
  if (unlikely(PyMethod_Check(__pyx_t_1))) {
    __pyx_t_11 = PyMethod_GET_SELF(__pyx_t_1);
    assert(__pyx_t_11);
    PyObject* __pyx__function = PyMethod_GET_FUNCTION(__pyx_t_1);
    __Pyx_INCREF(__pyx_t_11);
    __Pyx_INCREF(__pyx__function);
    __Pyx_DECREF_SET(__pyx_t_1, __pyx__function);
    __pyx_t_4 = 0;
  }
  #endif
  {
    PyObject *__pyx_callargs[2 + ((CYTHON_VECTORCALL) ? 1 : 0)] = {__pyx_t_11, __pyx_t_12};
    __pyx_t_10 = __Pyx_MakeVectorcallBuilderKwds(1); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 456, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_10);
    if (__Pyx_VectorcallBuilder_AddArg(__pyx_mstate_global->__pyx_n_u_dtype, __pyx_t_2, __pyx_t_10, __pyx_callargs+2, 0) < (0)) __PYX_ERR(0, 456, __pyx_L1_error)
    __pyx_t_3 = __Pyx_Object_Vectorcall_CallFromBuilder((PyObject*)__pyx_t_1, __pyx_callargs+__pyx_t_4, (2-__pyx_t_4) | (__pyx_t_4*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET), __pyx_t_10);
    __Pyx_XDECREF(__pyx_t_11); __pyx_t_11 = 0;
    __Pyx_DECREF(__pyx_t_12); __pyx_t_12 = 0;
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 456, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
  }
  __pyx_v_info = __pyx_t_3;
  __pyx_t_3 = 0;

  /* "cupyx/cusolver.pyx":457
 *     work = _cupy.empty((lwork,), dtype=a.dtype)
 *     info = _cupy.empty((batch_size,), dtype=_numpy.int32)
 *     r_norm = _numpy.empty((batch_size,), dtype=_numpy.float64)             # <<<<<<<<<<<<<<
 *     solver(handle, jobz, rank, m, n, a.data.ptr, lda, stride_a, s.data.ptr,
 *            stride_s, u.data.ptr, ldu, stride_u, v.data.ptr, ldv, stride_v,
*/
  __pyx_t_1 = NULL;
  __Pyx_GetModuleGlobalName(__pyx_t_10, __pyx_mstate_global->__pyx_n_u_numpy); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 457, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_10);
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_t_10, __pyx_mstate_global->__pyx_n_u_empty); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 457, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
  __pyx_t_10 = PyTuple_New(1); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 457, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_10);
  __Pyx_INCREF(__pyx_v_batch_size);
  __Pyx_GIVEREF(__pyx_v_batch_size);
  if (__Pyx_PyTuple_SET_ITEM(__pyx_t_10, 0, __pyx_v_batch_size) != (0)) __PYX_ERR(0, 457, __pyx_L1_error);
  __Pyx_GetModuleGlobalName(__pyx_t_12, __pyx_mstate_global->__pyx_n_u_numpy); if (unlikely(!__pyx_t_12)) __PYX_ERR(0, 457, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_12);
  __pyx_t_11 = __Pyx_PyObject_GetAttrStr(__pyx_t_12, __pyx_mstate_global->__pyx_n_u_float64); if (unlikely(!__pyx_t_11)) __PYX_ERR(0, 457, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_11);
  __Pyx_DECREF(__pyx_t_12); __pyx_t_12 = 0;
  __pyx_t_4 = 1;
  #if CYTHON_UNPACK_METHODS
  if (unlikely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_1 = PyMethod_GET_SELF(__pyx_t_2);
    assert(__pyx_t_1);
    PyObject* __pyx__function = PyMethod_GET_FUNCTION(__pyx_t_2);
    __Pyx_INCREF(__pyx_t_1);
    __Pyx_INCREF(__pyx__function);
    __Pyx_DECREF_SET(__pyx_t_2, __pyx__function);
    __pyx_t_4 = 0;
  }
  #endif
  {
    PyObject *__pyx_callargs[2 + ((CYTHON_VECTORCALL) ? 1 : 0)] = {__pyx_t_1, __pyx_t_10};
    __pyx_t_12 = __Pyx_MakeVectorcallBuilderKwds(1); if (unlikely(!__pyx_t_12)) __PYX_ERR(0, 457, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_12);
    if (__Pyx_VectorcallBuilder_AddArg(__pyx_mstate_global->__pyx_n_u_dtype, __pyx_t_11, __pyx_t_12, __pyx_callargs+2, 0) < (0)) __PYX_ERR(0, 457, __pyx_L1_error)
    __pyx_t_3 = __Pyx_Object_Vectorcall_CallFromBuilder((PyObject*)__pyx_t_2, __pyx_callargs+__pyx_t_4, (2-__pyx_t_4) | (__pyx_t_4*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET), __pyx_t_12);
    __Pyx_XDECREF(__pyx_t_1); __pyx_t_1 = 0;
    __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
    __Pyx_DECREF(__pyx_t_11); __pyx_t_11 = 0;
    __Pyx_DECREF(__pyx_t_12); __pyx_t_12 = 0;
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 457, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
  }
  __pyx_v_r_norm = __pyx_t_3;
  __pyx_t_3 = 0;

  /* "cupyx/cusolver.pyx":458
 *     info = _cupy.empty((batch_size,), dtype=_numpy.int32)
 *     r_norm = _numpy.empty((batch_size,), dtype=_numpy.float64)
 *     solver(handle, jobz, rank, m, n, a.data.ptr, lda, stride_a, s.data.ptr,             # <<<<<<<<<<<<<<
 *            stride_s, u.data.ptr, ldu, stride_u, v.data.ptr, ldv, stride_v,
 *            work.data.ptr, lwork, info.data.ptr, r_norm.ctypes.data, batch_size)
*/
  __pyx_t_2 = NULL;
  __Pyx_INCREF(__pyx_v_solver);
  __pyx_t_12 = __pyx_v_solver; 
  __pyx_t_11 = __Pyx_PyObject_GetAttrStr(__pyx_v_a, __pyx_mstate_global->__pyx_n_u_data); if (unlikely(!__pyx_t_11)) __PYX_ERR(0, 458, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_11);
  __pyx_t_10 = __Pyx_PyObject_GetAttrStr(__pyx_t_11, __pyx_mstate_global->__pyx_n_u_ptr); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 458, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_10);
  __Pyx_DECREF(__pyx_t_11); __pyx_t_11 = 0;
  __pyx_t_11 = __Pyx_PyObject_GetAttrStr(__pyx_v_s, __pyx_mstate_global->__pyx_n_u_data); if (unlikely(!__pyx_t_11)) __PYX_ERR(0, 458, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_11);
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_t_11, __pyx_mstate_global->__pyx_n_u_ptr); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 458, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_11); __pyx_t_11 = 0;

  /* "cupyx/cusolver.pyx":459
 *     r_norm = _numpy.empty((batch_size,), dtype=_numpy.float64)
 *     solver(handle, jobz, rank, m, n, a.data.ptr, lda, stride_a, s.data.ptr,
 *            stride_s, u.data.ptr, ldu, stride_u, v.data.ptr, ldv, stride_v,             # <<<<<<<<<<<<<<
 *            work.data.ptr, lwork, info.data.ptr, r_norm.ctypes.data, batch_size)
 * 
*/
  __pyx_t_11 = __Pyx_PyObject_GetAttrStr(__pyx_v_u, __pyx_mstate_global->__pyx_n_u_data); if (unlikely(!__pyx_t_11)) __PYX_ERR(0, 459, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_11);
  __pyx_t_9 = __Pyx_PyObject_GetAttrStr(__pyx_t_11, __pyx_mstate_global->__pyx_n_u_ptr); if (unlikely(!__pyx_t_9)) __PYX_ERR(0, 459, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_9);
  __Pyx_DECREF(__pyx_t_11); __pyx_t_11 = 0;
  __pyx_t_11 = __Pyx_PyObject_GetAttrStr(__pyx_v_v, __pyx_mstate_global->__pyx_n_u_data); if (unlikely(!__pyx_t_11)) __PYX_ERR(0, 459, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_11);
  __pyx_t_7 = __Pyx_PyObject_GetAttrStr(__pyx_t_11, __pyx_mstate_global->__pyx_n_u_ptr); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 459, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_7);
  __Pyx_DECREF(__pyx_t_11); __pyx_t_11 = 0;

  /* "cupyx/cusolver.pyx":460
 *     solver(handle, jobz, rank, m, n, a.data.ptr, lda, stride_a, s.data.ptr,
 *            stride_s, u.data.ptr, ldu, stride_u, v.data.ptr, ldv, stride_v,
 *            work.data.ptr, lwork, info.data.ptr, r_norm.ctypes.data, batch_size)             # <<<<<<<<<<<<<<
 * 
 *     s = s.reshape(a_shape[:-2] + (s.shape[-1],))
*/
  __pyx_t_11 = __Pyx_PyObject_GetAttrStr(__pyx_v_work, __pyx_mstate_global->__pyx_n_u_data); if (unlikely(!__pyx_t_11)) __PYX_ERR(0, 460, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_11);
  __pyx_t_13 = __Pyx_PyObject_GetAttrStr(__pyx_t_11, __pyx_mstate_global->__pyx_n_u_ptr); if (unlikely(!__pyx_t_13)) __PYX_ERR(0, 460, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_13);
  __Pyx_DECREF(__pyx_t_11); __pyx_t_11 = 0;
  __pyx_t_11 = __Pyx_PyObject_GetAttrStr(__pyx_v_info, __pyx_mstate_global->__pyx_n_u_data); if (unlikely(!__pyx_t_11)) __PYX_ERR(0, 460, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_11);
  __pyx_t_14 = __Pyx_PyObject_GetAttrStr(__pyx_t_11, __pyx_mstate_global->__pyx_n_u_ptr); if (unlikely(!__pyx_t_14)) __PYX_ERR(0, 460, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_14);
  __Pyx_DECREF(__pyx_t_11); __pyx_t_11 = 0;
  __pyx_t_11 = __Pyx_PyObject_GetAttrStr(__pyx_v_r_norm, __pyx_mstate_global->__pyx_n_u_ctypes); if (unlikely(!__pyx_t_11)) __PYX_ERR(0, 460, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_11);
  __pyx_t_15 = __Pyx_PyObject_GetAttrStr(__pyx_t_11, __pyx_mstate_global->__pyx_n_u_data); if (unlikely(!__pyx_t_15)) __PYX_ERR(0, 460, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_15);
  __Pyx_DECREF(__pyx_t_11); __pyx_t_11 = 0;
  __pyx_t_4 = 1;
  #if CYTHON_UNPACK_METHODS
  if (unlikely(PyMethod_Check(__pyx_t_12))) {
    __pyx_t_2 = PyMethod_GET_SELF(__pyx_t_12);
    assert(__pyx_t_2);
    PyObject* __pyx__function = PyMethod_GET_FUNCTION(__pyx_t_12);
    __Pyx_INCREF(__pyx_t_2);
    __Pyx_INCREF(__pyx__function);
    __Pyx_DECREF_SET(__pyx_t_12, __pyx__function);
    __pyx_t_4 = 0;
  }
  #endif
  {
    PyObject *__pyx_callargs[22] = {__pyx_t_2, __pyx_v_handle, __pyx_v_jobz, __pyx_v_rank, __pyx_v_m, __pyx_v_n, __pyx_t_10, __pyx_v_lda, __pyx_v_stride_a, __pyx_t_1, __pyx_v_stride_s, __pyx_t_9, __pyx_v_ldu, __pyx_v_stride_u, __pyx_t_7, __pyx_v_ldv, __pyx_v_stride_v, __pyx_t_13, __pyx_v_lwork, __pyx_t_14, __pyx_t_15, __pyx_v_batch_size};
    __pyx_t_3 = __Pyx_PyObject_FastCall((PyObject*)__pyx_t_12, __pyx_callargs+__pyx_t_4, (22-__pyx_t_4) | (__pyx_t_4*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
    __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
    __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
    __Pyx_DECREF(__pyx_t_13); __pyx_t_13 = 0;
    __Pyx_DECREF(__pyx_t_14); __pyx_t_14 = 0;
    __Pyx_DECREF(__pyx_t_15); __pyx_t_15 = 0;
    __Pyx_DECREF(__pyx_t_12); __pyx_t_12 = 0;
    if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 458, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
  }
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "cupyx/cusolver.pyx":462
 *            work.data.ptr, lwork, info.data.ptr, r_norm.ctypes.data, batch_size)
 * 
 *     s = s.reshape(a_shape[:-2] + (s.shape[-1],))             # <<<<<<<<<<<<<<
 *     if not compute_uv:
 *         return s
*/
  __pyx_t_12 = __pyx_v_s;
  __Pyx_INCREF(__pyx_t_12);
  __pyx_t_15 = __Pyx_PyObject_GetSlice(__pyx_v_a_shape, 0, -2L, NULL, NULL, &__pyx_mstate_global->__pyx_slice[1], 0, 1, 1); if (unlikely(!__pyx_t_15)) __PYX_ERR(0, 462, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_15);
  __pyx_t_14 = __Pyx_PyObject_GetAttrStr(__pyx_v_s, __pyx_mstate_global->__pyx_n_u_shape); if (unlikely(!__pyx_t_14)) __PYX_ERR(0, 462, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_14);
  __pyx_t_13 = __Pyx_GetItemInt(__pyx_t_14, -1L, long, 1, __Pyx_PyLong_From_long, 0, 1, 1, 1, __Pyx_ReferenceSharing_OwnStrongReference); if (unlikely(!__pyx_t_13)) __PYX_ERR(0, 462, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_13);
  __Pyx_DECREF(__pyx_t_14); __pyx_t_14 = 0;
  __pyx_t_14 = PyTuple_New(1); if (unlikely(!__pyx_t_14)) __PYX_ERR(0, 462, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_14);
  __Pyx_GIVEREF(__pyx_t_13);
  if (__Pyx_PyTuple_SET_ITEM(__pyx_t_14, 0, __pyx_t_13) != (0)) __PYX_ERR(0, 462, __pyx_L1_error);
  __pyx_t_13 = 0;
  __pyx_t_13 = PyNumber_Add(__pyx_t_15, __pyx_t_14); if (unlikely(!__pyx_t_13)) __PYX_ERR(0, 462, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_13);
  __Pyx_DECREF(__pyx_t_15); __pyx_t_15 = 0;
  __Pyx_DECREF(__pyx_t_14); __pyx_t_14 = 0;
  __pyx_t_4 = 0;
  {
    PyObject *__pyx_callargs[2] = {__pyx_t_12, __pyx_t_13};
    __pyx_t_3 = __Pyx_PyObject_FastCallMethod((PyObject*)__pyx_mstate_global->__pyx_n_u_reshape, __pyx_callargs+__pyx_t_4, (2-__pyx_t_4) | (1*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
    __Pyx_XDECREF(__pyx_t_12); __pyx_t_12 = 0;
    __Pyx_DECREF(__pyx_t_13); __pyx_t_13 = 0;
    if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 462, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
  }
  __Pyx_DECREF_SET(__pyx_v_s, __pyx_t_3);
  __pyx_t_3 = 0;

  /* "cupyx/cusolver.pyx":463
 * 
 *     s = s.reshape(a_shape[:-2] + (s.shape[-1],))
 *     if not compute_uv:             # <<<<<<<<<<<<<<
 *         return s
 * 
*/
  __pyx_t_6 = __Pyx_PyObject_IsTrue(__pyx_v_compute_uv); if (unlikely((__pyx_t_6 < 0))) __PYX_ERR(0, 463, __pyx_L1_error)
  __pyx_t_5 = (!__pyx_t_6);
  if (__pyx_t_5) {

    /* "cupyx/cusolver.pyx":464
 *     s = s.reshape(a_shape[:-2] + (s.shape[-1],))
 *     if not compute_uv:
 *         return s             # <<<<<<<<<<<<<<
 * 
 *     u = u.transpose(0, 2, 1)
*/
    __Pyx_XDECREF(__pyx_r);
    __Pyx_INCREF(__pyx_v_s);
    __pyx_r = __pyx_v_s;
    goto __pyx_L0;

    /* "cupyx/cusolver.pyx":463
 * 
 *     s = s.reshape(a_shape[:-2] + (s.shape[-1],))
 *     if not compute_uv:             # <<<<<<<<<<<<<<
 *         return s
 * 
*/
  }

  /* "cupyx/cusolver.pyx":466
 *         return s
 * 
 *     u = u.transpose(0, 2, 1)             # <<<<<<<<<<<<<<
 *     v = v.transpose(0, 2, 1)
 *     u = u.reshape(a_shape[:-2] + (u.shape[-2:]))
*/
  __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_v_u, __pyx_mstate_global->__pyx_n_u_transpose); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 466, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_13 = __Pyx_PyObject_Call(__pyx_t_3, __pyx_mstate_global->__pyx_tuple[2], NULL); if (unlikely(!__pyx_t_13)) __PYX_ERR(0, 466, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_13);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __Pyx_DECREF_SET(__pyx_v_u, __pyx_t_13);
  __pyx_t_13 = 0;

  /* "cupyx/cusolver.pyx":467
 * 
 *     u = u.transpose(0, 2, 1)
 *     v = v.transpose(0, 2, 1)             # <<<<<<<<<<<<<<
 *     u = u.reshape(a_shape[:-2] + (u.shape[-2:]))
 *     v = v.reshape(a_shape[:-2] + (v.shape[-2:]))
*/
  __pyx_t_13 = __Pyx_PyObject_GetAttrStr(__pyx_v_v, __pyx_mstate_global->__pyx_n_u_transpose); if (unlikely(!__pyx_t_13)) __PYX_ERR(0, 467, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_13);
  __pyx_t_3 = __Pyx_PyObject_Call(__pyx_t_13, __pyx_mstate_global->__pyx_tuple[2], NULL); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 467, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_13); __pyx_t_13 = 0;
  __Pyx_DECREF_SET(__pyx_v_v, __pyx_t_3);
  __pyx_t_3 = 0;

  /* "cupyx/cusolver.pyx":468
 *     u = u.transpose(0, 2, 1)
 *     v = v.transpose(0, 2, 1)
 *     u = u.reshape(a_shape[:-2] + (u.shape[-2:]))             # <<<<<<<<<<<<<<
 *     v = v.reshape(a_shape[:-2] + (v.shape[-2:]))
 *     return u, s, v
*/
  __pyx_t_13 = __pyx_v_u;
  __Pyx_INCREF(__pyx_t_13);
  __pyx_t_12 = __Pyx_PyObject_GetSlice(__pyx_v_a_shape, 0, -2L, NULL, NULL, &__pyx_mstate_global->__pyx_slice[1], 0, 1, 1); if (unlikely(!__pyx_t_12)) __PYX_ERR(0, 468, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_12);
  __pyx_t_14 = __Pyx_PyObject_GetAttrStr(__pyx_v_u, __pyx_mstate_global->__pyx_n_u_shape); if (unlikely(!__pyx_t_14)) __PYX_ERR(0, 468, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_14);
  __pyx_t_15 = __Pyx_PyObject_GetSlice(__pyx_t_14, -2L, 0, NULL, NULL, &__pyx_mstate_global->__pyx_slice[0], 1, 0, 1); if (unlikely(!__pyx_t_15)) __PYX_ERR(0, 468, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_15);
  __Pyx_DECREF(__pyx_t_14); __pyx_t_14 = 0;
  __pyx_t_14 = PyNumber_Add(__pyx_t_12, __pyx_t_15); if (unlikely(!__pyx_t_14)) __PYX_ERR(0, 468, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_14);
  __Pyx_DECREF(__pyx_t_12); __pyx_t_12 = 0;
  __Pyx_DECREF(__pyx_t_15); __pyx_t_15 = 0;
  __pyx_t_4 = 0;
  {
    PyObject *__pyx_callargs[2] = {__pyx_t_13, __pyx_t_14};
    __pyx_t_3 = __Pyx_PyObject_FastCallMethod((PyObject*)__pyx_mstate_global->__pyx_n_u_reshape, __pyx_callargs+__pyx_t_4, (2-__pyx_t_4) | (1*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
    __Pyx_XDECREF(__pyx_t_13); __pyx_t_13 = 0;
    __Pyx_DECREF(__pyx_t_14); __pyx_t_14 = 0;
    if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 468, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
  }
  __Pyx_DECREF_SET(__pyx_v_u, __pyx_t_3);
  __pyx_t_3 = 0;

  /* "cupyx/cusolver.pyx":469
 *     v = v.transpose(0, 2, 1)
 *     u = u.reshape(a_shape[:-2] + (u.shape[-2:]))
 *     v = v.reshape(a_shape[:-2] + (v.shape[-2:]))             # <<<<<<<<<<<<<<
 *     return u, s, v
 * 
*/
  __pyx_t_14 = __pyx_v_v;
  __Pyx_INCREF(__pyx_t_14);
  __pyx_t_13 = __Pyx_PyObject_GetSlice(__pyx_v_a_shape, 0, -2L, NULL, NULL, &__pyx_mstate_global->__pyx_slice[1], 0, 1, 1); if (unlikely(!__pyx_t_13)) __PYX_ERR(0, 469, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_13);
  __pyx_t_15 = __Pyx_PyObject_GetAttrStr(__pyx_v_v, __pyx_mstate_global->__pyx_n_u_shape); if (unlikely(!__pyx_t_15)) __PYX_ERR(0, 469, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_15);
  __pyx_t_12 = __Pyx_PyObject_GetSlice(__pyx_t_15, -2L, 0, NULL, NULL, &__pyx_mstate_global->__pyx_slice[0], 1, 0, 1); if (unlikely(!__pyx_t_12)) __PYX_ERR(0, 469, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_12);
  __Pyx_DECREF(__pyx_t_15); __pyx_t_15 = 0;
  __pyx_t_15 = PyNumber_Add(__pyx_t_13, __pyx_t_12); if (unlikely(!__pyx_t_15)) __PYX_ERR(0, 469, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_15);
  __Pyx_DECREF(__pyx_t_13); __pyx_t_13 = 0;
  __Pyx_DECREF(__pyx_t_12); __pyx_t_12 = 0;
  __pyx_t_4 = 0;
  {
    PyObject *__pyx_callargs[2] = {__pyx_t_14, __pyx_t_15};
    __pyx_t_3 = __Pyx_PyObject_FastCallMethod((PyObject*)__pyx_mstate_global->__pyx_n_u_reshape, __pyx_callargs+__pyx_t_4, (2-__pyx_t_4) | (1*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
    __Pyx_XDECREF(__pyx_t_14); __pyx_t_14 = 0;
    __Pyx_DECREF(__pyx_t_15); __pyx_t_15 = 0;
    if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 469, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
  }
  __Pyx_DECREF_SET(__pyx_v_v, __pyx_t_3);
  __pyx_t_3 = 0;

  /* "cupyx/cusolver.pyx":470
 *     u = u.reshape(a_shape[:-2] + (u.shape[-2:]))
 *     v = v.reshape(a_shape[:-2] + (v.shape[-2:]))
 *     return u, s, v             # <<<<<<<<<<<<<<
 * 
 * 
*/
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_3 = PyTuple_New(3); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 470, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_INCREF(__pyx_v_u);
  __Pyx_GIVEREF(__pyx_v_u);
  if (__Pyx_PyTuple_SET_ITEM(__pyx_t_3, 0, __pyx_v_u) != (0)) __PYX_ERR(0, 470, __pyx_L1_error);
  __Pyx_INCREF(__pyx_v_s);
  __Pyx_GIVEREF(__pyx_v_s);
  if (__Pyx_PyTuple_SET_ITEM(__pyx_t_3, 1, __pyx_v_s) != (0)) __PYX_ERR(0, 470, __pyx_L1_error);
  __Pyx_INCREF(__pyx_v_v);
  __Pyx_GIVEREF(__pyx_v_v);
  if (__Pyx_PyTuple_SET_ITEM(__pyx_t_3, 2, __pyx_v_v) != (0)) __PYX_ERR(0, 470, __pyx_L1_error);
  __pyx_r = __pyx_t_3;
  __pyx_t_3 = 0;
  goto __pyx_L0;

  /* "cupyx/cusolver.pyx":388
 * 
 * 
 * def gesvda(a, compute_uv=True):             # <<<<<<<<<<<<<<
 *     """Singular value decomposition using cusolverDn<t>gesvdaStridedBatched().
 * 
*/

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_7);
  __Pyx_XDECREF(__pyx_t_9);
  __Pyx_XDECREF(__pyx_t_10);
  __Pyx_XDECREF(__pyx_t_11);
  __Pyx_XDECREF(__pyx_t_12);
  __Pyx_XDECREF(__pyx_t_13);
  __Pyx_XDECREF(__pyx_t_14);
  __Pyx_XDECREF(__pyx_t_15);
  __Pyx_AddTraceback("cupyx.cusolver.gesvda", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_a_ndim);
  __Pyx_XDECREF(__pyx_v_a_shape);
  __Pyx_XDECREF(__pyx_v_m);
  __Pyx_XDECREF(__pyx_v_n);
  __Pyx_XDECREF(__pyx_v_helper);
  __Pyx_XDECREF(__pyx_v_solver);
  __Pyx_XDECREF(__pyx_v_s_dtype);
  __Pyx_XDECREF(__pyx_v_handle);
  __Pyx_XDECREF(__pyx_v_jobz);
  __Pyx_XDECREF(__pyx_v_rank);
  __Pyx_XDECREF(__pyx_v_batch_size);
  __Pyx_XDECREF(__pyx_v_lda);
  __Pyx_XDECREF(__pyx_v_stride_a);
  __Pyx_XDECREF(__pyx_v_s);
  __Pyx_XDECREF(__pyx_v_stride_s);
  __Pyx_XDECREF(__pyx_v_ldu);
  __Pyx_XDECREF(__pyx_v_ldv);
  __Pyx_XDECREF(__pyx_v_u);
  __Pyx_XDECREF(__pyx_v_v);
  __Pyx_XDECREF(__pyx_v_stride_u);
  __Pyx_XDECREF(__pyx_v_stride_v);
  __Pyx_XDECREF(__pyx_v_lwork);
  __Pyx_XDECREF(__pyx_v_work);
  __Pyx_XDECREF(__pyx_v_info);
  __Pyx_XDECREF(__pyx_v_r_norm);
  __Pyx_XDECREF(__pyx_v_a);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "cupyx/cusolver.pyx":473
 * 
 * 
 * def syevj(a, UPLO='L', with_eigen_vector=True):             # <<<<<<<<<<<<<<
 *     """Eigenvalue decomposition of symmetric matrix using cusolverDn<t>syevj().
 * 
*/

/* Python wrapper */
static PyObject *__pyx_pw_5cupyx_8cusolver_11syevj(PyObject *__pyx_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
PyDoc_STRVAR(__pyx_doc_5cupyx_8cusolver_10syevj, "syevj(a, UPLO='L', with_eigen_vector=True)\n\nEigenvalue decomposition of symmetric matrix using cusolverDn<t>syevj().\n\nComputes eigenvalues ``w`` and (optionally) eigenvectors ``v`` of a complex\nHermitian or a real symmetric matrix.\n\nArgs:\n    a (cupy.ndarray): A symmetric 2-D square matrix ``(M, M)`` or a batch\n        of symmetric 2-D square matrices ``(..., M, M)``.\n    UPLO (str): Select from ``'L'`` or ``'U'``. It specifies which\n        part of ``a`` is used. ``'L'`` uses the lower triangular part of\n        ``a``, and ``'U'`` uses the upper triangular part of ``a``.\n    with_eigen_vector (bool): Indicates whether or not eigenvectors\n        are computed.\n\nReturns:\n    tuple of :class:`~cupy.ndarray`:\n        Returns a tuple ``(w, v)``. ``w`` contains eigenvalues and\n        ``v`` contains eigenvectors. ``v[:, i]`` is an eigenvector\n        corresponding to an eigenvalue ``w[i]``. For batch input,\n        ``v[k, :, i]`` is an eigenvector corresponding to an eigenvalue\n        ``w[k, i]`` of ``a[k]``.");
static PyMethodDef __pyx_mdef_5cupyx_8cusolver_11syevj = {"syevj", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_5cupyx_8cusolver_11syevj, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_5cupyx_8cusolver_10syevj};
static PyObject *__pyx_pw_5cupyx_8cusolver_11syevj(PyObject *__pyx_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
) {
  PyObject *__pyx_v_a = 0;
  PyObject *__pyx_v_UPLO = 0;
  PyObject *__pyx_v_with_eigen_vector = 0;
  #if !CYTHON_METH_FASTCALL
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  #endif
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject* values[3] = {0,0,0};
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("syevj (wrapper)", 0);
  #if !CYTHON_METH_FASTCALL
  #if CYTHON_ASSUME_SAFE_SIZE
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return NULL;
  #endif
  #endif
  __pyx_kwvalues = __Pyx_KwValues_FASTCALL(__pyx_args, __pyx_nargs);
  {
    PyObject ** const __pyx_pyargnames[] = {&__pyx_mstate_global->__pyx_n_u_a,&__pyx_mstate_global->__pyx_n_u_UPLO,&__pyx_mstate_global->__pyx_n_u_with_eigen_vector,0};
    const Py_ssize_t __pyx_kwds_len = (__pyx_kwds) ? __Pyx_NumKwargs_FASTCALL(__pyx_kwds) : 0;
    if (unlikely(__pyx_kwds_len < 0)) __PYX_ERR(0, 473, __pyx_L3_error)
    if (__pyx_kwds_len > 0) {
      switch (__pyx_nargs) {
        case  3:
        values[2] = __Pyx_ArgRef_FASTCALL(__pyx_args, 2);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[2])) __PYX_ERR(0, 473, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  2:
        values[1] = __Pyx_ArgRef_FASTCALL(__pyx_args, 1);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[1])) __PYX_ERR(0, 473, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  1:
        values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(0, 473, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      const Py_ssize_t kwd_pos_args = __pyx_nargs;
      if (__Pyx_ParseKeywords(__pyx_kwds, __pyx_kwvalues, __pyx_pyargnames, 0, values, kwd_pos_args, __pyx_kwds_len, "syevj", 0) < (0)) __PYX_ERR(0, 473, __pyx_L3_error)
      if (!values[1]) values[1] = __Pyx_NewRef(((PyObject *)__pyx_mstate_global->__pyx_n_u_L));
      if (!values[2]) values[2] = __Pyx_NewRef(((PyObject *)Py_True));
      for (Py_ssize_t i = __pyx_nargs; i < 1; i++) {
        if (unlikely(!values[i])) { __Pyx_RaiseArgtupleInvalid("syevj", 0, 1, 3, i); __PYX_ERR(0, 473, __pyx_L3_error) }
      }
    } else {
      switch (__pyx_nargs) {
        case  3:
        values[2] = __Pyx_ArgRef_FASTCALL(__pyx_args, 2);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[2])) __PYX_ERR(0, 473, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  2:
        values[1] = __Pyx_ArgRef_FASTCALL(__pyx_args, 1);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[1])) __PYX_ERR(0, 473, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  1:
        values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(0, 473, __pyx_L3_error)
        break;
        default: goto __pyx_L5_argtuple_error;
      }
      if (!values[1]) values[1] = __Pyx_NewRef(((PyObject *)__pyx_mstate_global->__pyx_n_u_L));
      if (!values[2]) values[2] = __Pyx_NewRef(((PyObject *)Py_True));
    }
    __pyx_v_a = values[0];
    __pyx_v_UPLO = values[1];
    __pyx_v_with_eigen_vector = values[2];
  }
  goto __pyx_L6_skip;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("syevj", 0, 1, 3, __pyx_nargs); __PYX_ERR(0, 473, __pyx_L3_error)
  __pyx_L6_skip:;
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_AddTraceback("cupyx.cusolver.syevj", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_5cupyx_8cusolver_10syevj(__pyx_self, __pyx_v_a, __pyx_v_UPLO, __pyx_v_with_eigen_vector);

  /* function exit code */
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5cupyx_8cusolver_10syevj(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_a, PyObject *__pyx_v_UPLO, PyObject *__pyx_v_with_eigen_vector) {
  PyObject *__pyx_v_dtype = NULL;
  PyObject *__pyx_v_v_dtype = NULL;
  PyObject *__pyx_v_real_dtype = NULL;
  PyObject *__pyx_v_w_dtype = NULL;
  PyObject *__pyx_v_v = NULL;
  PyObject *__pyx_v_m = NULL;
  PyObject *__pyx_v_lda = NULL;
  PyObject *__pyx_v_w = NULL;
  PyObject *__pyx_v_dev_info = NULL;
  PyObject *__pyx_v_handle = NULL;
  PyObject *__pyx_v_jobz = NULL;
  PyObject *__pyx_v_uplo = NULL;
  PyObject *__pyx_v_buffer_size = NULL;
  PyObject *__pyx_v_syevj = NULL;
  PyObject *__pyx_v_params = NULL;
  PyObject *__pyx_v_work_size = NULL;
  PyObject *__pyx_v_work = NULL;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  size_t __pyx_t_4;
  int __pyx_t_5;
  int __pyx_t_6;
  PyObject *__pyx_t_7 = NULL;
  PyObject *(*__pyx_t_8)(PyObject *);
  PyObject *__pyx_t_9 = NULL;
  PyObject *__pyx_t_10 = NULL;
  PyObject *__pyx_t_11 = NULL;
  PyObject *__pyx_t_12 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("syevj", 0);

  /* "cupyx/cusolver.pyx":496
 *             ``w[k, i]`` of ``a[k]``.
 *     """
 *     if not check_availability('syevj'):             # <<<<<<<<<<<<<<
 *         raise RuntimeError('syevj is not available.')
 * 
*/
  __pyx_t_2 = NULL;
  __Pyx_GetModuleGlobalName(__pyx_t_3, __pyx_mstate_global->__pyx_n_u_check_availability); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 496, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_4 = 1;
  #if CYTHON_UNPACK_METHODS
  if (unlikely(PyMethod_Check(__pyx_t_3))) {
    __pyx_t_2 = PyMethod_GET_SELF(__pyx_t_3);
    assert(__pyx_t_2);
    PyObject* __pyx__function = PyMethod_GET_FUNCTION(__pyx_t_3);
    __Pyx_INCREF(__pyx_t_2);
    __Pyx_INCREF(__pyx__function);
    __Pyx_DECREF_SET(__pyx_t_3, __pyx__function);
    __pyx_t_4 = 0;
  }
  #endif
  {
    PyObject *__pyx_callargs[2] = {__pyx_t_2, __pyx_mstate_global->__pyx_n_u_syevj};
    __pyx_t_1 = __Pyx_PyObject_FastCall((PyObject*)__pyx_t_3, __pyx_callargs+__pyx_t_4, (2-__pyx_t_4) | (__pyx_t_4*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
    __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 496, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
  }
  __pyx_t_5 = __Pyx_PyObject_IsTrue(__pyx_t_1); if (unlikely((__pyx_t_5 < 0))) __PYX_ERR(0, 496, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_6 = (!__pyx_t_5);
  if (unlikely(__pyx_t_6)) {

    /* "cupyx/cusolver.pyx":497
 *     """
 *     if not check_availability('syevj'):
 *         raise RuntimeError('syevj is not available.')             # <<<<<<<<<<<<<<
 * 
 *     if UPLO not in ('L', 'U'):
*/
    __pyx_t_3 = NULL;
    __pyx_t_4 = 1;
    {
      PyObject *__pyx_callargs[2] = {__pyx_t_3, __pyx_mstate_global->__pyx_kp_u_syevj_is_not_available};
      __pyx_t_1 = __Pyx_PyObject_FastCall((PyObject*)(((PyTypeObject*)PyExc_RuntimeError)), __pyx_callargs+__pyx_t_4, (2-__pyx_t_4) | (__pyx_t_4*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
      __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
      if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 497, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
    }
    __Pyx_Raise(__pyx_t_1, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __PYX_ERR(0, 497, __pyx_L1_error)

    /* "cupyx/cusolver.pyx":496
 *             ``w[k, i]`` of ``a[k]``.
 *     """
 *     if not check_availability('syevj'):             # <<<<<<<<<<<<<<
 *         raise RuntimeError('syevj is not available.')
 * 
*/
  }

  /* "cupyx/cusolver.pyx":499
 *         raise RuntimeError('syevj is not available.')
 * 
 *     if UPLO not in ('L', 'U'):             # <<<<<<<<<<<<<<
 *         raise ValueError('UPLO argument must be \'L\' or \'U\'')
 * 
*/
  __Pyx_INCREF(__pyx_v_UPLO);
  __pyx_t_1 = __pyx_v_UPLO;
  __pyx_t_5 = (__Pyx_PyUnicode_Equals(__pyx_t_1, __pyx_mstate_global->__pyx_n_u_L, Py_NE)); if (unlikely((__pyx_t_5 < 0))) __PYX_ERR(0, 499, __pyx_L1_error)
  if (__pyx_t_5) {
  } else {
    __pyx_t_6 = __pyx_t_5;
    goto __pyx_L5_bool_binop_done;
  }
  __pyx_t_5 = (__Pyx_PyUnicode_Equals(__pyx_t_1, __pyx_mstate_global->__pyx_n_u_U, Py_NE)); if (unlikely((__pyx_t_5 < 0))) __PYX_ERR(0, 499, __pyx_L1_error)
  __pyx_t_6 = __pyx_t_5;
  __pyx_L5_bool_binop_done:;
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_5 = __pyx_t_6;
  if (unlikely(__pyx_t_5)) {

    /* "cupyx/cusolver.pyx":500
 * 
 *     if UPLO not in ('L', 'U'):
 *         raise ValueError('UPLO argument must be \'L\' or \'U\'')             # <<<<<<<<<<<<<<
 * 
 *     if a.ndim > 2:
*/
    __pyx_t_3 = NULL;
    __pyx_t_4 = 1;
    {
      PyObject *__pyx_callargs[2] = {__pyx_t_3, __pyx_mstate_global->__pyx_kp_u_UPLO_argument_must_be_L_or_U};
      __pyx_t_1 = __Pyx_PyObject_FastCall((PyObject*)(((PyTypeObject*)PyExc_ValueError)), __pyx_callargs+__pyx_t_4, (2-__pyx_t_4) | (__pyx_t_4*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
      __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
      if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 500, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
    }
    __Pyx_Raise(__pyx_t_1, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __PYX_ERR(0, 500, __pyx_L1_error)

    /* "cupyx/cusolver.pyx":499
 *         raise RuntimeError('syevj is not available.')
 * 
 *     if UPLO not in ('L', 'U'):             # <<<<<<<<<<<<<<
 *         raise ValueError('UPLO argument must be \'L\' or \'U\'')
 * 
*/
  }

  /* "cupyx/cusolver.pyx":502
 *         raise ValueError('UPLO argument must be \'L\' or \'U\'')
 * 
 *     if a.ndim > 2:             # <<<<<<<<<<<<<<
 *         return _syevj_batched(a, UPLO, with_eigen_vector)
 * 
*/
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_v_a, __pyx_mstate_global->__pyx_n_u_ndim); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 502, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_3 = PyObject_RichCompare(__pyx_t_1, __pyx_mstate_global->__pyx_int_2, Py_GT); __Pyx_XGOTREF(__pyx_t_3); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 502, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_5 = __Pyx_PyObject_IsTrue(__pyx_t_3); if (unlikely((__pyx_t_5 < 0))) __PYX_ERR(0, 502, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  if (__pyx_t_5) {

    /* "cupyx/cusolver.pyx":503
 * 
 *     if a.ndim > 2:
 *         return _syevj_batched(a, UPLO, with_eigen_vector)             # <<<<<<<<<<<<<<
 * 
 *     assert a.ndim == 2
*/
    __Pyx_XDECREF(__pyx_r);
    __pyx_t_1 = NULL;
    __Pyx_GetModuleGlobalName(__pyx_t_2, __pyx_mstate_global->__pyx_n_u_syevj_batched); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 503, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_4 = 1;
    #if CYTHON_UNPACK_METHODS
    if (unlikely(PyMethod_Check(__pyx_t_2))) {
      __pyx_t_1 = PyMethod_GET_SELF(__pyx_t_2);
      assert(__pyx_t_1);
      PyObject* __pyx__function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_1);
      __Pyx_INCREF(__pyx__function);
      __Pyx_DECREF_SET(__pyx_t_2, __pyx__function);
      __pyx_t_4 = 0;
    }
    #endif
    {
      PyObject *__pyx_callargs[4] = {__pyx_t_1, __pyx_v_a, __pyx_v_UPLO, __pyx_v_with_eigen_vector};
      __pyx_t_3 = __Pyx_PyObject_FastCall((PyObject*)__pyx_t_2, __pyx_callargs+__pyx_t_4, (4-__pyx_t_4) | (__pyx_t_4*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
      __Pyx_XDECREF(__pyx_t_1); __pyx_t_1 = 0;
      __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
      if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 503, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_3);
    }
    __pyx_r = __pyx_t_3;
    __pyx_t_3 = 0;
    goto __pyx_L0;

    /* "cupyx/cusolver.pyx":502
 *         raise ValueError('UPLO argument must be \'L\' or \'U\'')
 * 
 *     if a.ndim > 2:             # <<<<<<<<<<<<<<
 *         return _syevj_batched(a, UPLO, with_eigen_vector)
 * 
*/
  }

  /* "cupyx/cusolver.pyx":505
 *         return _syevj_batched(a, UPLO, with_eigen_vector)
 * 
 *     assert a.ndim == 2             # <<<<<<<<<<<<<<
 * 
 *     # reject_float16=False for backward compatibility
*/
  #ifndef CYTHON_WITHOUT_ASSERTIONS
  if (unlikely(__pyx_assertions_enabled())) {
    __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_v_a, __pyx_mstate_global->__pyx_n_u_ndim); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 505, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_t_5 = (__Pyx_PyLong_BoolEqObjC(__pyx_t_3, __pyx_mstate_global->__pyx_int_2, 2, 0)); if (unlikely((__pyx_t_5 < 0))) __PYX_ERR(0, 505, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    if (unlikely(!__pyx_t_5)) {
      __Pyx_Raise(((PyObject *)(((PyTypeObject*)PyExc_AssertionError))), 0, 0, 0);
      __PYX_ERR(0, 505, __pyx_L1_error)
    }
  }
  #else
  if ((1)); else __PYX_ERR(0, 505, __pyx_L1_error)
  #endif

  /* "cupyx/cusolver.pyx":508
 * 
 *     # reject_float16=False for backward compatibility
 *     dtype, v_dtype = _cupy.linalg._util.linalg_common_type(             # <<<<<<<<<<<<<<
 *         a, reject_float16=False)
 *     real_dtype = dtype.char.lower()
*/
  __Pyx_GetModuleGlobalName(__pyx_t_1, __pyx_mstate_global->__pyx_n_u_cupy); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 508, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_7 = __Pyx_PyObject_GetAttrStr(__pyx_t_1, __pyx_mstate_global->__pyx_n_u_linalg); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 508, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_7);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_t_7, __pyx_mstate_global->__pyx_n_u_util); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 508, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
  __pyx_t_2 = __pyx_t_1;
  __Pyx_INCREF(__pyx_t_2);

  /* "cupyx/cusolver.pyx":509
 *     # reject_float16=False for backward compatibility
 *     dtype, v_dtype = _cupy.linalg._util.linalg_common_type(
 *         a, reject_float16=False)             # <<<<<<<<<<<<<<
 *     real_dtype = dtype.char.lower()
 *     w_dtype = v_dtype.char.lower()
*/
  __pyx_t_4 = 0;
  {
    PyObject *__pyx_callargs[2 + ((CYTHON_VECTORCALL) ? 1 : 0)] = {__pyx_t_2, __pyx_v_a};
    __pyx_t_7 = __Pyx_MakeVectorcallBuilderKwds(1); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 508, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_7);
    if (__Pyx_VectorcallBuilder_AddArg(__pyx_mstate_global->__pyx_n_u_reject_float16, Py_False, __pyx_t_7, __pyx_callargs+2, 0) < (0)) __PYX_ERR(0, 508, __pyx_L1_error)
    __pyx_t_3 = __Pyx_Object_VectorcallMethod_CallFromBuilder((PyObject*)__pyx_mstate_global->__pyx_n_u_linalg_common_type, __pyx_callargs+__pyx_t_4, (2-__pyx_t_4) | (1*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET), __pyx_t_7);
    __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 508, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
  }
  if ((likely(PyTuple_CheckExact(__pyx_t_3))) || (PyList_CheckExact(__pyx_t_3))) {
    PyObject* sequence = __pyx_t_3;
    Py_ssize_t size = __Pyx_PySequence_SIZE(sequence);
    if (unlikely(size != 2)) {
      if (size > 2) __Pyx_RaiseTooManyValuesError(2);
      else if (size >= 0) __Pyx_RaiseNeedMoreValuesError(size);
      __PYX_ERR(0, 508, __pyx_L1_error)
    }
    #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
    if (likely(PyTuple_CheckExact(sequence))) {
      __pyx_t_1 = PyTuple_GET_ITEM(sequence, 0);
      __Pyx_INCREF(__pyx_t_1);
      __pyx_t_7 = PyTuple_GET_ITEM(sequence, 1);
      __Pyx_INCREF(__pyx_t_7);
    } else {
      __pyx_t_1 = __Pyx_PyList_GetItemRefFast(sequence, 0, __Pyx_ReferenceSharing_SharedReference);
      if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 508, __pyx_L1_error)
      __Pyx_XGOTREF(__pyx_t_1);
      __pyx_t_7 = __Pyx_PyList_GetItemRefFast(sequence, 1, __Pyx_ReferenceSharing_SharedReference);
      if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 508, __pyx_L1_error)
      __Pyx_XGOTREF(__pyx_t_7);
    }
    #else
    __pyx_t_1 = __Pyx_PySequence_ITEM(sequence, 0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 508, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_t_7 = __Pyx_PySequence_ITEM(sequence, 1); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 508, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_7);
    #endif
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  } else {
    Py_ssize_t index = -1;
    __pyx_t_2 = PyObject_GetIter(__pyx_t_3); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 508, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __pyx_t_8 = (CYTHON_COMPILING_IN_LIMITED_API) ? PyIter_Next : __Pyx_PyObject_GetIterNextFunc(__pyx_t_2);
    index = 0; __pyx_t_1 = __pyx_t_8(__pyx_t_2); if (unlikely(!__pyx_t_1)) goto __pyx_L8_unpacking_failed;
    __Pyx_GOTREF(__pyx_t_1);
    index = 1; __pyx_t_7 = __pyx_t_8(__pyx_t_2); if (unlikely(!__pyx_t_7)) goto __pyx_L8_unpacking_failed;
    __Pyx_GOTREF(__pyx_t_7);
    if (__Pyx_IternextUnpackEndCheck(__pyx_t_8(__pyx_t_2), 2) < (0)) __PYX_ERR(0, 508, __pyx_L1_error)
    __pyx_t_8 = NULL;
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    goto __pyx_L9_unpacking_done;
    __pyx_L8_unpacking_failed:;
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __pyx_t_8 = NULL;
    if (__Pyx_IterFinish() == 0) __Pyx_RaiseNeedMoreValuesError(index);
    __PYX_ERR(0, 508, __pyx_L1_error)
    __pyx_L9_unpacking_done:;
  }

  /* "cupyx/cusolver.pyx":508
 * 
 *     # reject_float16=False for backward compatibility
 *     dtype, v_dtype = _cupy.linalg._util.linalg_common_type(             # <<<<<<<<<<<<<<
 *         a, reject_float16=False)
 *     real_dtype = dtype.char.lower()
*/
  __pyx_v_dtype = __pyx_t_1;
  __pyx_t_1 = 0;
  __pyx_v_v_dtype = __pyx_t_7;
  __pyx_t_7 = 0;

  /* "cupyx/cusolver.pyx":510
 *     dtype, v_dtype = _cupy.linalg._util.linalg_common_type(
 *         a, reject_float16=False)
 *     real_dtype = dtype.char.lower()             # <<<<<<<<<<<<<<
 *     w_dtype = v_dtype.char.lower()
 * 
*/
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_v_dtype, __pyx_mstate_global->__pyx_n_u_char); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 510, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_7 = __pyx_t_1;
  __Pyx_INCREF(__pyx_t_7);
  __pyx_t_4 = 0;
  {
    PyObject *__pyx_callargs[2] = {__pyx_t_7, NULL};
    __pyx_t_3 = __Pyx_PyObject_FastCallMethod((PyObject*)__pyx_mstate_global->__pyx_n_u_lower, __pyx_callargs+__pyx_t_4, (1-__pyx_t_4) | (1*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
    __Pyx_XDECREF(__pyx_t_7); __pyx_t_7 = 0;
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 510, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
  }
  __pyx_v_real_dtype = __pyx_t_3;
  __pyx_t_3 = 0;

  /* "cupyx/cusolver.pyx":511
 *         a, reject_float16=False)
 *     real_dtype = dtype.char.lower()
 *     w_dtype = v_dtype.char.lower()             # <<<<<<<<<<<<<<
 * 
 *     # Note that cuSolver assumes fortran array
*/
  __pyx_t_7 = __Pyx_PyObject_GetAttrStr(__pyx_v_v_dtype, __pyx_mstate_global->__pyx_n_u_char); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 511, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_7);
  __pyx_t_1 = __pyx_t_7;
  __Pyx_INCREF(__pyx_t_1);
  __pyx_t_4 = 0;
  {
    PyObject *__pyx_callargs[2] = {__pyx_t_1, NULL};
    __pyx_t_3 = __Pyx_PyObject_FastCallMethod((PyObject*)__pyx_mstate_global->__pyx_n_u_lower, __pyx_callargs+__pyx_t_4, (1-__pyx_t_4) | (1*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
    __Pyx_XDECREF(__pyx_t_1); __pyx_t_1 = 0;
    __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
    if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 511, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
  }
  __pyx_v_w_dtype = __pyx_t_3;
  __pyx_t_3 = 0;

  /* "cupyx/cusolver.pyx":514
 * 
 *     # Note that cuSolver assumes fortran array
 *     v = a.astype(dtype, order='F', copy=True)             # <<<<<<<<<<<<<<
 * 
 *     m, lda = a.shape
*/
  __pyx_t_7 = __pyx_v_a;
  __Pyx_INCREF(__pyx_t_7);
  __pyx_t_4 = 0;
  {
    PyObject *__pyx_callargs[2 + ((CYTHON_VECTORCALL) ? 2 : 0)] = {__pyx_t_7, __pyx_v_dtype};
    __pyx_t_1 = __Pyx_MakeVectorcallBuilderKwds(2); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 514, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    if (__Pyx_VectorcallBuilder_AddArg(__pyx_mstate_global->__pyx_n_u_order, __pyx_mstate_global->__pyx_n_u_F, __pyx_t_1, __pyx_callargs+2, 0) < (0)) __PYX_ERR(0, 514, __pyx_L1_error)
    if (__Pyx_VectorcallBuilder_AddArg(__pyx_mstate_global->__pyx_n_u_copy, Py_True, __pyx_t_1, __pyx_callargs+2, 1) < (0)) __PYX_ERR(0, 514, __pyx_L1_error)
    __pyx_t_3 = __Pyx_Object_VectorcallMethod_CallFromBuilder((PyObject*)__pyx_mstate_global->__pyx_n_u_astype, __pyx_callargs+__pyx_t_4, (2-__pyx_t_4) | (1*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET), __pyx_t_1);
    __Pyx_XDECREF(__pyx_t_7); __pyx_t_7 = 0;
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 514, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
  }
  __pyx_v_v = __pyx_t_3;
  __pyx_t_3 = 0;

  /* "cupyx/cusolver.pyx":516
 *     v = a.astype(dtype, order='F', copy=True)
 * 
 *     m, lda = a.shape             # <<<<<<<<<<<<<<
 *     w = _cupy.empty(m, real_dtype)
 *     dev_info = _cupy.empty((1,), _cupy.int32)
*/
  __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_v_a, __pyx_mstate_global->__pyx_n_u_shape); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 516, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  if ((likely(PyTuple_CheckExact(__pyx_t_3))) || (PyList_CheckExact(__pyx_t_3))) {
    PyObject* sequence = __pyx_t_3;
    Py_ssize_t size = __Pyx_PySequence_SIZE(sequence);
    if (unlikely(size != 2)) {
      if (size > 2) __Pyx_RaiseTooManyValuesError(2);
      else if (size >= 0) __Pyx_RaiseNeedMoreValuesError(size);
      __PYX_ERR(0, 516, __pyx_L1_error)
    }
    #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
    if (likely(PyTuple_CheckExact(sequence))) {
      __pyx_t_1 = PyTuple_GET_ITEM(sequence, 0);
      __Pyx_INCREF(__pyx_t_1);
      __pyx_t_7 = PyTuple_GET_ITEM(sequence, 1);
      __Pyx_INCREF(__pyx_t_7);
    } else {
      __pyx_t_1 = __Pyx_PyList_GetItemRefFast(sequence, 0, __Pyx_ReferenceSharing_SharedReference);
      if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 516, __pyx_L1_error)
      __Pyx_XGOTREF(__pyx_t_1);
      __pyx_t_7 = __Pyx_PyList_GetItemRefFast(sequence, 1, __Pyx_ReferenceSharing_SharedReference);
      if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 516, __pyx_L1_error)
      __Pyx_XGOTREF(__pyx_t_7);
    }
    #else
    __pyx_t_1 = __Pyx_PySequence_ITEM(sequence, 0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 516, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_t_7 = __Pyx_PySequence_ITEM(sequence, 1); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 516, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_7);
    #endif
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  } else {
    Py_ssize_t index = -1;
    __pyx_t_2 = PyObject_GetIter(__pyx_t_3); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 516, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __pyx_t_8 = (CYTHON_COMPILING_IN_LIMITED_API) ? PyIter_Next : __Pyx_PyObject_GetIterNextFunc(__pyx_t_2);
    index = 0; __pyx_t_1 = __pyx_t_8(__pyx_t_2); if (unlikely(!__pyx_t_1)) goto __pyx_L10_unpacking_failed;
    __Pyx_GOTREF(__pyx_t_1);
    index = 1; __pyx_t_7 = __pyx_t_8(__pyx_t_2); if (unlikely(!__pyx_t_7)) goto __pyx_L10_unpacking_failed;
    __Pyx_GOTREF(__pyx_t_7);
    if (__Pyx_IternextUnpackEndCheck(__pyx_t_8(__pyx_t_2), 2) < (0)) __PYX_ERR(0, 516, __pyx_L1_error)
    __pyx_t_8 = NULL;
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    goto __pyx_L11_unpacking_done;
    __pyx_L10_unpacking_failed:;
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __pyx_t_8 = NULL;
    if (__Pyx_IterFinish() == 0) __Pyx_RaiseNeedMoreValuesError(index);
    __PYX_ERR(0, 516, __pyx_L1_error)
    __pyx_L11_unpacking_done:;
  }
  __pyx_v_m = __pyx_t_1;
  __pyx_t_1 = 0;
  __pyx_v_lda = __pyx_t_7;
  __pyx_t_7 = 0;

  /* "cupyx/cusolver.pyx":517
 * 
 *     m, lda = a.shape
 *     w = _cupy.empty(m, real_dtype)             # <<<<<<<<<<<<<<
 *     dev_info = _cupy.empty((1,), _cupy.int32)
 *     handle = _device.Device().cusolver_handle
*/
  __pyx_t_7 = NULL;
  __Pyx_GetModuleGlobalName(__pyx_t_1, __pyx_mstate_global->__pyx_n_u_cupy); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 517, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_t_1, __pyx_mstate_global->__pyx_n_u_empty); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 517, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_4 = 1;
  #if CYTHON_UNPACK_METHODS
  if (unlikely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_7 = PyMethod_GET_SELF(__pyx_t_2);
    assert(__pyx_t_7);
    PyObject* __pyx__function = PyMethod_GET_FUNCTION(__pyx_t_2);
    __Pyx_INCREF(__pyx_t_7);
    __Pyx_INCREF(__pyx__function);
    __Pyx_DECREF_SET(__pyx_t_2, __pyx__function);
    __pyx_t_4 = 0;
  }
  #endif
  {
    PyObject *__pyx_callargs[3] = {__pyx_t_7, __pyx_v_m, __pyx_v_real_dtype};
    __pyx_t_3 = __Pyx_PyObject_FastCall((PyObject*)__pyx_t_2, __pyx_callargs+__pyx_t_4, (3-__pyx_t_4) | (__pyx_t_4*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
    __Pyx_XDECREF(__pyx_t_7); __pyx_t_7 = 0;
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 517, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
  }
  __pyx_v_w = __pyx_t_3;
  __pyx_t_3 = 0;

  /* "cupyx/cusolver.pyx":518
 *     m, lda = a.shape
 *     w = _cupy.empty(m, real_dtype)
 *     dev_info = _cupy.empty((1,), _cupy.int32)             # <<<<<<<<<<<<<<
 *     handle = _device.Device().cusolver_handle
 * 
*/
  __pyx_t_2 = NULL;
  __Pyx_GetModuleGlobalName(__pyx_t_7, __pyx_mstate_global->__pyx_n_u_cupy); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 518, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_7);
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_t_7, __pyx_mstate_global->__pyx_n_u_empty); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 518, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
  __Pyx_GetModuleGlobalName(__pyx_t_7, __pyx_mstate_global->__pyx_n_u_cupy); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 518, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_7);
  __pyx_t_9 = __Pyx_PyObject_GetAttrStr(__pyx_t_7, __pyx_mstate_global->__pyx_n_u_int32); if (unlikely(!__pyx_t_9)) __PYX_ERR(0, 518, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_9);
  __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
  __pyx_t_4 = 1;
  #if CYTHON_UNPACK_METHODS
  if (unlikely(PyMethod_Check(__pyx_t_1))) {
    __pyx_t_2 = PyMethod_GET_SELF(__pyx_t_1);
    assert(__pyx_t_2);
    PyObject* __pyx__function = PyMethod_GET_FUNCTION(__pyx_t_1);
    __Pyx_INCREF(__pyx_t_2);
    __Pyx_INCREF(__pyx__function);
    __Pyx_DECREF_SET(__pyx_t_1, __pyx__function);
    __pyx_t_4 = 0;
  }
  #endif
  {
    PyObject *__pyx_callargs[3] = {__pyx_t_2, __pyx_mstate_global->__pyx_tuple[3], __pyx_t_9};
    __pyx_t_3 = __Pyx_PyObject_FastCall((PyObject*)__pyx_t_1, __pyx_callargs+__pyx_t_4, (3-__pyx_t_4) | (__pyx_t_4*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
    __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 518, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
  }
  __pyx_v_dev_info = __pyx_t_3;
  __pyx_t_3 = 0;

  /* "cupyx/cusolver.pyx":519
 *     w = _cupy.empty(m, real_dtype)
 *     dev_info = _cupy.empty((1,), _cupy.int32)
 *     handle = _device.Device().cusolver_handle             # <<<<<<<<<<<<<<
 * 
 *     if with_eigen_vector:
*/
  __pyx_t_1 = NULL;
  __Pyx_GetModuleGlobalName(__pyx_t_9, __pyx_mstate_global->__pyx_n_u_device); if (unlikely(!__pyx_t_9)) __PYX_ERR(0, 519, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_9);
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_t_9, __pyx_mstate_global->__pyx_n_u_Device); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 519, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
  __pyx_t_4 = 1;
  #if CYTHON_UNPACK_METHODS
  if (unlikely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_1 = PyMethod_GET_SELF(__pyx_t_2);
    assert(__pyx_t_1);
    PyObject* __pyx__function = PyMethod_GET_FUNCTION(__pyx_t_2);
    __Pyx_INCREF(__pyx_t_1);
    __Pyx_INCREF(__pyx__function);
    __Pyx_DECREF_SET(__pyx_t_2, __pyx__function);
    __pyx_t_4 = 0;
  }
  #endif
  {
    PyObject *__pyx_callargs[2] = {__pyx_t_1, NULL};
    __pyx_t_3 = __Pyx_PyObject_FastCall((PyObject*)__pyx_t_2, __pyx_callargs+__pyx_t_4, (1-__pyx_t_4) | (__pyx_t_4*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
    __Pyx_XDECREF(__pyx_t_1); __pyx_t_1 = 0;
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 519, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
  }
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_t_3, __pyx_mstate_global->__pyx_n_u_cusolver_handle); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 519, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_v_handle = __pyx_t_2;
  __pyx_t_2 = 0;

  /* "cupyx/cusolver.pyx":521
 *     handle = _device.Device().cusolver_handle
 * 
 *     if with_eigen_vector:             # <<<<<<<<<<<<<<
 *         jobz = _cusolver.CUSOLVER_EIG_MODE_VECTOR
 *     else:
*/
  __pyx_t_5 = __Pyx_PyObject_IsTrue(__pyx_v_with_eigen_vector); if (unlikely((__pyx_t_5 < 0))) __PYX_ERR(0, 521, __pyx_L1_error)
  if (__pyx_t_5) {

    /* "cupyx/cusolver.pyx":522
 * 
 *     if with_eigen_vector:
 *         jobz = _cusolver.CUSOLVER_EIG_MODE_VECTOR             # <<<<<<<<<<<<<<
 *     else:
 *         jobz = _cusolver.CUSOLVER_EIG_MODE_NOVECTOR
*/
    __Pyx_GetModuleGlobalName(__pyx_t_2, __pyx_mstate_global->__pyx_n_u_cusolver); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 522, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_t_2, __pyx_mstate_global->__pyx_n_u_CUSOLVER_EIG_MODE_VECTOR); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 522, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __pyx_v_jobz = __pyx_t_3;
    __pyx_t_3 = 0;

    /* "cupyx/cusolver.pyx":521
 *     handle = _device.Device().cusolver_handle
 * 
 *     if with_eigen_vector:             # <<<<<<<<<<<<<<
 *         jobz = _cusolver.CUSOLVER_EIG_MODE_VECTOR
 *     else:
*/
    goto __pyx_L12;
  }

  /* "cupyx/cusolver.pyx":524
 *         jobz = _cusolver.CUSOLVER_EIG_MODE_VECTOR
 *     else:
 *         jobz = _cusolver.CUSOLVER_EIG_MODE_NOVECTOR             # <<<<<<<<<<<<<<
 * 
 *     if UPLO == 'L':
*/
  /*else*/ {
    __Pyx_GetModuleGlobalName(__pyx_t_3, __pyx_mstate_global->__pyx_n_u_cusolver); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 524, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_t_3, __pyx_mstate_global->__pyx_n_u_CUSOLVER_EIG_MODE_NOVECTOR); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 524, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __pyx_v_jobz = __pyx_t_2;
    __pyx_t_2 = 0;
  }
  __pyx_L12:;

  /* "cupyx/cusolver.pyx":526
 *         jobz = _cusolver.CUSOLVER_EIG_MODE_NOVECTOR
 * 
 *     if UPLO == 'L':             # <<<<<<<<<<<<<<
 *         uplo = _cublas.CUBLAS_FILL_MODE_LOWER
 *     else:  # UPLO == 'U'
*/
  __pyx_t_5 = (__Pyx_PyUnicode_Equals(__pyx_v_UPLO, __pyx_mstate_global->__pyx_n_u_L, Py_EQ)); if (unlikely((__pyx_t_5 < 0))) __PYX_ERR(0, 526, __pyx_L1_error)
  if (__pyx_t_5) {

    /* "cupyx/cusolver.pyx":527
 * 
 *     if UPLO == 'L':
 *         uplo = _cublas.CUBLAS_FILL_MODE_LOWER             # <<<<<<<<<<<<<<
 *     else:  # UPLO == 'U'
 *         uplo = _cublas.CUBLAS_FILL_MODE_UPPER
*/
    __Pyx_GetModuleGlobalName(__pyx_t_2, __pyx_mstate_global->__pyx_n_u_cublas); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 527, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_t_2, __pyx_mstate_global->__pyx_n_u_CUBLAS_FILL_MODE_LOWER); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 527, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __pyx_v_uplo = __pyx_t_3;
    __pyx_t_3 = 0;

    /* "cupyx/cusolver.pyx":526
 *         jobz = _cusolver.CUSOLVER_EIG_MODE_NOVECTOR
 * 
 *     if UPLO == 'L':             # <<<<<<<<<<<<<<
 *         uplo = _cublas.CUBLAS_FILL_MODE_LOWER
 *     else:  # UPLO == 'U'
*/
    goto __pyx_L13;
  }

  /* "cupyx/cusolver.pyx":529
 *         uplo = _cublas.CUBLAS_FILL_MODE_LOWER
 *     else:  # UPLO == 'U'
 *         uplo = _cublas.CUBLAS_FILL_MODE_UPPER             # <<<<<<<<<<<<<<
 * 
 *     if dtype == 'f':
*/
  /*else*/ {
    __Pyx_GetModuleGlobalName(__pyx_t_3, __pyx_mstate_global->__pyx_n_u_cublas); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 529, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_t_3, __pyx_mstate_global->__pyx_n_u_CUBLAS_FILL_MODE_UPPER); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 529, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __pyx_v_uplo = __pyx_t_2;
    __pyx_t_2 = 0;
  }
  __pyx_L13:;

  /* "cupyx/cusolver.pyx":531
 *         uplo = _cublas.CUBLAS_FILL_MODE_UPPER
 * 
 *     if dtype == 'f':             # <<<<<<<<<<<<<<
 *         buffer_size = _cusolver.ssyevj_bufferSize
 *         syevj = _cusolver.ssyevj
*/
  __pyx_t_5 = (__Pyx_PyUnicode_Equals(__pyx_v_dtype, __pyx_mstate_global->__pyx_n_u_f, Py_EQ)); if (unlikely((__pyx_t_5 < 0))) __PYX_ERR(0, 531, __pyx_L1_error)
  if (__pyx_t_5) {

    /* "cupyx/cusolver.pyx":532
 * 
 *     if dtype == 'f':
 *         buffer_size = _cusolver.ssyevj_bufferSize             # <<<<<<<<<<<<<<
 *         syevj = _cusolver.ssyevj
 *     elif dtype == 'd':
*/
    __Pyx_GetModuleGlobalName(__pyx_t_2, __pyx_mstate_global->__pyx_n_u_cusolver); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 532, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_t_2, __pyx_mstate_global->__pyx_n_u_ssyevj_bufferSize); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 532, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __pyx_v_buffer_size = __pyx_t_3;
    __pyx_t_3 = 0;

    /* "cupyx/cusolver.pyx":533
 *     if dtype == 'f':
 *         buffer_size = _cusolver.ssyevj_bufferSize
 *         syevj = _cusolver.ssyevj             # <<<<<<<<<<<<<<
 *     elif dtype == 'd':
 *         buffer_size = _cusolver.dsyevj_bufferSize
*/
    __Pyx_GetModuleGlobalName(__pyx_t_3, __pyx_mstate_global->__pyx_n_u_cusolver); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 533, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_t_3, __pyx_mstate_global->__pyx_n_u_ssyevj); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 533, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __pyx_v_syevj = __pyx_t_2;
    __pyx_t_2 = 0;

    /* "cupyx/cusolver.pyx":531
 *         uplo = _cublas.CUBLAS_FILL_MODE_UPPER
 * 
 *     if dtype == 'f':             # <<<<<<<<<<<<<<
 *         buffer_size = _cusolver.ssyevj_bufferSize
 *         syevj = _cusolver.ssyevj
*/
    goto __pyx_L14;
  }

  /* "cupyx/cusolver.pyx":534
 *         buffer_size = _cusolver.ssyevj_bufferSize
 *         syevj = _cusolver.ssyevj
 *     elif dtype == 'd':             # <<<<<<<<<<<<<<
 *         buffer_size = _cusolver.dsyevj_bufferSize
 *         syevj = _cusolver.dsyevj
*/
  __pyx_t_5 = (__Pyx_PyUnicode_Equals(__pyx_v_dtype, __pyx_mstate_global->__pyx_n_u_d, Py_EQ)); if (unlikely((__pyx_t_5 < 0))) __PYX_ERR(0, 534, __pyx_L1_error)
  if (__pyx_t_5) {

    /* "cupyx/cusolver.pyx":535
 *         syevj = _cusolver.ssyevj
 *     elif dtype == 'd':
 *         buffer_size = _cusolver.dsyevj_bufferSize             # <<<<<<<<<<<<<<
 *         syevj = _cusolver.dsyevj
 *     elif dtype == 'F':
*/
    __Pyx_GetModuleGlobalName(__pyx_t_2, __pyx_mstate_global->__pyx_n_u_cusolver); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 535, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_t_2, __pyx_mstate_global->__pyx_n_u_dsyevj_bufferSize); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 535, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __pyx_v_buffer_size = __pyx_t_3;
    __pyx_t_3 = 0;

    /* "cupyx/cusolver.pyx":536
 *     elif dtype == 'd':
 *         buffer_size = _cusolver.dsyevj_bufferSize
 *         syevj = _cusolver.dsyevj             # <<<<<<<<<<<<<<
 *     elif dtype == 'F':
 *         buffer_size = _cusolver.cheevj_bufferSize
*/
    __Pyx_GetModuleGlobalName(__pyx_t_3, __pyx_mstate_global->__pyx_n_u_cusolver); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 536, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_t_3, __pyx_mstate_global->__pyx_n_u_dsyevj); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 536, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __pyx_v_syevj = __pyx_t_2;
    __pyx_t_2 = 0;

    /* "cupyx/cusolver.pyx":534
 *         buffer_size = _cusolver.ssyevj_bufferSize
 *         syevj = _cusolver.ssyevj
 *     elif dtype == 'd':             # <<<<<<<<<<<<<<
 *         buffer_size = _cusolver.dsyevj_bufferSize
 *         syevj = _cusolver.dsyevj
*/
    goto __pyx_L14;
  }

  /* "cupyx/cusolver.pyx":537
 *         buffer_size = _cusolver.dsyevj_bufferSize
 *         syevj = _cusolver.dsyevj
 *     elif dtype == 'F':             # <<<<<<<<<<<<<<
 *         buffer_size = _cusolver.cheevj_bufferSize
 *         syevj = _cusolver.cheevj
*/
  __pyx_t_5 = (__Pyx_PyUnicode_Equals(__pyx_v_dtype, __pyx_mstate_global->__pyx_n_u_F, Py_EQ)); if (unlikely((__pyx_t_5 < 0))) __PYX_ERR(0, 537, __pyx_L1_error)
  if (__pyx_t_5) {

    /* "cupyx/cusolver.pyx":538
 *         syevj = _cusolver.dsyevj
 *     elif dtype == 'F':
 *         buffer_size = _cusolver.cheevj_bufferSize             # <<<<<<<<<<<<<<
 *         syevj = _cusolver.cheevj
 *     elif dtype == 'D':
*/
    __Pyx_GetModuleGlobalName(__pyx_t_2, __pyx_mstate_global->__pyx_n_u_cusolver); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 538, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_t_2, __pyx_mstate_global->__pyx_n_u_cheevj_bufferSize); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 538, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __pyx_v_buffer_size = __pyx_t_3;
    __pyx_t_3 = 0;

    /* "cupyx/cusolver.pyx":539
 *     elif dtype == 'F':
 *         buffer_size = _cusolver.cheevj_bufferSize
 *         syevj = _cusolver.cheevj             # <<<<<<<<<<<<<<
 *     elif dtype == 'D':
 *         buffer_size = _cusolver.zheevj_bufferSize
*/
    __Pyx_GetModuleGlobalName(__pyx_t_3, __pyx_mstate_global->__pyx_n_u_cusolver); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 539, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_t_3, __pyx_mstate_global->__pyx_n_u_cheevj); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 539, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __pyx_v_syevj = __pyx_t_2;
    __pyx_t_2 = 0;

    /* "cupyx/cusolver.pyx":537
 *         buffer_size = _cusolver.dsyevj_bufferSize
 *         syevj = _cusolver.dsyevj
 *     elif dtype == 'F':             # <<<<<<<<<<<<<<
 *         buffer_size = _cusolver.cheevj_bufferSize
 *         syevj = _cusolver.cheevj
*/
    goto __pyx_L14;
  }

  /* "cupyx/cusolver.pyx":540
 *         buffer_size = _cusolver.cheevj_bufferSize
 *         syevj = _cusolver.cheevj
 *     elif dtype == 'D':             # <<<<<<<<<<<<<<
 *         buffer_size = _cusolver.zheevj_bufferSize
 *         syevj = _cusolver.zheevj
*/
  __pyx_t_5 = (__Pyx_PyUnicode_Equals(__pyx_v_dtype, __pyx_mstate_global->__pyx_n_u_D, Py_EQ)); if (unlikely((__pyx_t_5 < 0))) __PYX_ERR(0, 540, __pyx_L1_error)
  if (likely(__pyx_t_5)) {

    /* "cupyx/cusolver.pyx":541
 *         syevj = _cusolver.cheevj
 *     elif dtype == 'D':
 *         buffer_size = _cusolver.zheevj_bufferSize             # <<<<<<<<<<<<<<
 *         syevj = _cusolver.zheevj
 *     else:
*/
    __Pyx_GetModuleGlobalName(__pyx_t_2, __pyx_mstate_global->__pyx_n_u_cusolver); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 541, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_t_2, __pyx_mstate_global->__pyx_n_u_zheevj_bufferSize); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 541, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __pyx_v_buffer_size = __pyx_t_3;
    __pyx_t_3 = 0;

    /* "cupyx/cusolver.pyx":542
 *     elif dtype == 'D':
 *         buffer_size = _cusolver.zheevj_bufferSize
 *         syevj = _cusolver.zheevj             # <<<<<<<<<<<<<<
 *     else:
 *         raise RuntimeError('Only float and double and cuComplex and '
*/
    __Pyx_GetModuleGlobalName(__pyx_t_3, __pyx_mstate_global->__pyx_n_u_cusolver); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 542, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_t_3, __pyx_mstate_global->__pyx_n_u_zheevj); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 542, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __pyx_v_syevj = __pyx_t_2;
    __pyx_t_2 = 0;

    /* "cupyx/cusolver.pyx":540
 *         buffer_size = _cusolver.cheevj_bufferSize
 *         syevj = _cusolver.cheevj
 *     elif dtype == 'D':             # <<<<<<<<<<<<<<
 *         buffer_size = _cusolver.zheevj_bufferSize
 *         syevj = _cusolver.zheevj
*/
    goto __pyx_L14;
  }

  /* "cupyx/cusolver.pyx":544
 *         syevj = _cusolver.zheevj
 *     else:
 *         raise RuntimeError('Only float and double and cuComplex and '             # <<<<<<<<<<<<<<
 *                            + 'cuDoubleComplex are supported')
 * 
*/
  /*else*/ {
    __pyx_t_3 = NULL;
    __pyx_t_4 = 1;
    {
      PyObject *__pyx_callargs[2] = {__pyx_t_3, __pyx_mstate_global->__pyx_kp_u_Only_float_and_double_and_cuComp};
      __pyx_t_2 = __Pyx_PyObject_FastCall((PyObject*)(((PyTypeObject*)PyExc_RuntimeError)), __pyx_callargs+__pyx_t_4, (2-__pyx_t_4) | (__pyx_t_4*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
      __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
      if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 544, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_2);
    }
    __Pyx_Raise(__pyx_t_2, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __PYX_ERR(0, 544, __pyx_L1_error)
  }
  __pyx_L14:;

  /* "cupyx/cusolver.pyx":547
 *                            + 'cuDoubleComplex are supported')
 * 
 *     params = _cusolver.createSyevjInfo()             # <<<<<<<<<<<<<<
 *     work_size = buffer_size(
 *         handle, jobz, uplo, m, v.data.ptr, lda, w.data.ptr, params)
*/
  __pyx_t_3 = NULL;
  __Pyx_GetModuleGlobalName(__pyx_t_1, __pyx_mstate_global->__pyx_n_u_cusolver); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 547, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_9 = __Pyx_PyObject_GetAttrStr(__pyx_t_1, __pyx_mstate_global->__pyx_n_u_createSyevjInfo); if (unlikely(!__pyx_t_9)) __PYX_ERR(0, 547, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_9);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_4 = 1;
  #if CYTHON_UNPACK_METHODS
  if (unlikely(PyMethod_Check(__pyx_t_9))) {
    __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_9);
    assert(__pyx_t_3);
    PyObject* __pyx__function = PyMethod_GET_FUNCTION(__pyx_t_9);
    __Pyx_INCREF(__pyx_t_3);
    __Pyx_INCREF(__pyx__function);
    __Pyx_DECREF_SET(__pyx_t_9, __pyx__function);
    __pyx_t_4 = 0;
  }
  #endif
  {
    PyObject *__pyx_callargs[2] = {__pyx_t_3, NULL};
    __pyx_t_2 = __Pyx_PyObject_FastCall((PyObject*)__pyx_t_9, __pyx_callargs+__pyx_t_4, (1-__pyx_t_4) | (__pyx_t_4*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
    __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
    __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
    if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 547, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
  }
  __pyx_v_params = __pyx_t_2;
  __pyx_t_2 = 0;

  /* "cupyx/cusolver.pyx":548
 * 
 *     params = _cusolver.createSyevjInfo()
 *     work_size = buffer_size(             # <<<<<<<<<<<<<<
 *         handle, jobz, uplo, m, v.data.ptr, lda, w.data.ptr, params)
 *     work = _cupy.empty(work_size, dtype)
*/
  __pyx_t_9 = NULL;
  __Pyx_INCREF(__pyx_v_buffer_size);
  __pyx_t_3 = __pyx_v_buffer_size; 

  /* "cupyx/cusolver.pyx":549
 *     params = _cusolver.createSyevjInfo()
 *     work_size = buffer_size(
 *         handle, jobz, uplo, m, v.data.ptr, lda, w.data.ptr, params)             # <<<<<<<<<<<<<<
 *     work = _cupy.empty(work_size, dtype)
 *     syevj(
*/
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_v_v, __pyx_mstate_global->__pyx_n_u_data); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 549, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_7 = __Pyx_PyObject_GetAttrStr(__pyx_t_1, __pyx_mstate_global->__pyx_n_u_ptr); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 549, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_7);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_v_w, __pyx_mstate_global->__pyx_n_u_data); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 549, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_10 = __Pyx_PyObject_GetAttrStr(__pyx_t_1, __pyx_mstate_global->__pyx_n_u_ptr); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 549, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_10);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_4 = 1;
  #if CYTHON_UNPACK_METHODS
  if (unlikely(PyMethod_Check(__pyx_t_3))) {
    __pyx_t_9 = PyMethod_GET_SELF(__pyx_t_3);
    assert(__pyx_t_9);
    PyObject* __pyx__function = PyMethod_GET_FUNCTION(__pyx_t_3);
    __Pyx_INCREF(__pyx_t_9);
    __Pyx_INCREF(__pyx__function);
    __Pyx_DECREF_SET(__pyx_t_3, __pyx__function);
    __pyx_t_4 = 0;
  }
  #endif
  {
    PyObject *__pyx_callargs[9] = {__pyx_t_9, __pyx_v_handle, __pyx_v_jobz, __pyx_v_uplo, __pyx_v_m, __pyx_t_7, __pyx_v_lda, __pyx_t_10, __pyx_v_params};
    __pyx_t_2 = __Pyx_PyObject_FastCall((PyObject*)__pyx_t_3, __pyx_callargs+__pyx_t_4, (9-__pyx_t_4) | (__pyx_t_4*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
    __Pyx_XDECREF(__pyx_t_9); __pyx_t_9 = 0;
    __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
    __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 548, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
  }
  __pyx_v_work_size = __pyx_t_2;
  __pyx_t_2 = 0;

  /* "cupyx/cusolver.pyx":550
 *     work_size = buffer_size(
 *         handle, jobz, uplo, m, v.data.ptr, lda, w.data.ptr, params)
 *     work = _cupy.empty(work_size, dtype)             # <<<<<<<<<<<<<<
 *     syevj(
 *         handle, jobz, uplo, m, v.data.ptr, lda,
*/
  __pyx_t_3 = NULL;
  __Pyx_GetModuleGlobalName(__pyx_t_10, __pyx_mstate_global->__pyx_n_u_cupy); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 550, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_10);
  __pyx_t_7 = __Pyx_PyObject_GetAttrStr(__pyx_t_10, __pyx_mstate_global->__pyx_n_u_empty); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 550, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_7);
  __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
  __pyx_t_4 = 1;
  #if CYTHON_UNPACK_METHODS
  if (unlikely(PyMethod_Check(__pyx_t_7))) {
    __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_7);
    assert(__pyx_t_3);
    PyObject* __pyx__function = PyMethod_GET_FUNCTION(__pyx_t_7);
    __Pyx_INCREF(__pyx_t_3);
    __Pyx_INCREF(__pyx__function);
    __Pyx_DECREF_SET(__pyx_t_7, __pyx__function);
    __pyx_t_4 = 0;
  }
  #endif
  {
    PyObject *__pyx_callargs[3] = {__pyx_t_3, __pyx_v_work_size, __pyx_v_dtype};
    __pyx_t_2 = __Pyx_PyObject_FastCall((PyObject*)__pyx_t_7, __pyx_callargs+__pyx_t_4, (3-__pyx_t_4) | (__pyx_t_4*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
    __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
    __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
    if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 550, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
  }
  __pyx_v_work = __pyx_t_2;
  __pyx_t_2 = 0;

  /* "cupyx/cusolver.pyx":551
 *         handle, jobz, uplo, m, v.data.ptr, lda, w.data.ptr, params)
 *     work = _cupy.empty(work_size, dtype)
 *     syevj(             # <<<<<<<<<<<<<<
 *         handle, jobz, uplo, m, v.data.ptr, lda,
 *         w.data.ptr, work.data.ptr, work_size, dev_info.data.ptr, params)
*/
  __pyx_t_7 = NULL;
  __Pyx_INCREF(__pyx_v_syevj);
  __pyx_t_3 = __pyx_v_syevj; 

  /* "cupyx/cusolver.pyx":552
 *     work = _cupy.empty(work_size, dtype)
 *     syevj(
 *         handle, jobz, uplo, m, v.data.ptr, lda,             # <<<<<<<<<<<<<<
 *         w.data.ptr, work.data.ptr, work_size, dev_info.data.ptr, params)
 *     _cupy.linalg._util._check_cusolver_dev_info_if_synchronization_allowed(
*/
  __pyx_t_10 = __Pyx_PyObject_GetAttrStr(__pyx_v_v, __pyx_mstate_global->__pyx_n_u_data); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 552, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_10);
  __pyx_t_9 = __Pyx_PyObject_GetAttrStr(__pyx_t_10, __pyx_mstate_global->__pyx_n_u_ptr); if (unlikely(!__pyx_t_9)) __PYX_ERR(0, 552, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_9);
  __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;

  /* "cupyx/cusolver.pyx":553
 *     syevj(
 *         handle, jobz, uplo, m, v.data.ptr, lda,
 *         w.data.ptr, work.data.ptr, work_size, dev_info.data.ptr, params)             # <<<<<<<<<<<<<<
 *     _cupy.linalg._util._check_cusolver_dev_info_if_synchronization_allowed(
 *         syevj, dev_info)
*/
  __pyx_t_10 = __Pyx_PyObject_GetAttrStr(__pyx_v_w, __pyx_mstate_global->__pyx_n_u_data); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 553, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_10);
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_t_10, __pyx_mstate_global->__pyx_n_u_ptr); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 553, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
  __pyx_t_10 = __Pyx_PyObject_GetAttrStr(__pyx_v_work, __pyx_mstate_global->__pyx_n_u_data); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 553, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_10);
  __pyx_t_11 = __Pyx_PyObject_GetAttrStr(__pyx_t_10, __pyx_mstate_global->__pyx_n_u_ptr); if (unlikely(!__pyx_t_11)) __PYX_ERR(0, 553, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_11);
  __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
  __pyx_t_10 = __Pyx_PyObject_GetAttrStr(__pyx_v_dev_info, __pyx_mstate_global->__pyx_n_u_data); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 553, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_10);
  __pyx_t_12 = __Pyx_PyObject_GetAttrStr(__pyx_t_10, __pyx_mstate_global->__pyx_n_u_ptr); if (unlikely(!__pyx_t_12)) __PYX_ERR(0, 553, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_12);
  __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
  __pyx_t_4 = 1;
  #if CYTHON_UNPACK_METHODS
  if (unlikely(PyMethod_Check(__pyx_t_3))) {
    __pyx_t_7 = PyMethod_GET_SELF(__pyx_t_3);
    assert(__pyx_t_7);
    PyObject* __pyx__function = PyMethod_GET_FUNCTION(__pyx_t_3);
    __Pyx_INCREF(__pyx_t_7);
    __Pyx_INCREF(__pyx__function);
    __Pyx_DECREF_SET(__pyx_t_3, __pyx__function);
    __pyx_t_4 = 0;
  }
  #endif
  {
    PyObject *__pyx_callargs[12] = {__pyx_t_7, __pyx_v_handle, __pyx_v_jobz, __pyx_v_uplo, __pyx_v_m, __pyx_t_9, __pyx_v_lda, __pyx_t_1, __pyx_t_11, __pyx_v_work_size, __pyx_t_12, __pyx_v_params};
    __pyx_t_2 = __Pyx_PyObject_FastCall((PyObject*)__pyx_t_3, __pyx_callargs+__pyx_t_4, (12-__pyx_t_4) | (__pyx_t_4*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
    __Pyx_XDECREF(__pyx_t_7); __pyx_t_7 = 0;
    __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __Pyx_DECREF(__pyx_t_11); __pyx_t_11 = 0;
    __Pyx_DECREF(__pyx_t_12); __pyx_t_12 = 0;
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 551, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
  }
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "cupyx/cusolver.pyx":554
 *         handle, jobz, uplo, m, v.data.ptr, lda,
 *         w.data.ptr, work.data.ptr, work_size, dev_info.data.ptr, params)
 *     _cupy.linalg._util._check_cusolver_dev_info_if_synchronization_allowed(             # <<<<<<<<<<<<<<
 *         syevj, dev_info)
 * 
*/
  __Pyx_GetModuleGlobalName(__pyx_t_12, __pyx_mstate_global->__pyx_n_u_cupy); if (unlikely(!__pyx_t_12)) __PYX_ERR(0, 554, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_12);
  __pyx_t_11 = __Pyx_PyObject_GetAttrStr(__pyx_t_12, __pyx_mstate_global->__pyx_n_u_linalg); if (unlikely(!__pyx_t_11)) __PYX_ERR(0, 554, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_11);
  __Pyx_DECREF(__pyx_t_12); __pyx_t_12 = 0;
  __pyx_t_12 = __Pyx_PyObject_GetAttrStr(__pyx_t_11, __pyx_mstate_global->__pyx_n_u_util); if (unlikely(!__pyx_t_12)) __PYX_ERR(0, 554, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_12);
  __Pyx_DECREF(__pyx_t_11); __pyx_t_11 = 0;
  __pyx_t_3 = __pyx_t_12;
  __Pyx_INCREF(__pyx_t_3);

  /* "cupyx/cusolver.pyx":555
 *         w.data.ptr, work.data.ptr, work_size, dev_info.data.ptr, params)
 *     _cupy.linalg._util._check_cusolver_dev_info_if_synchronization_allowed(
 *         syevj, dev_info)             # <<<<<<<<<<<<<<
 * 
 *     _cusolver.destroySyevjInfo(params)
*/
  __pyx_t_4 = 0;
  {
    PyObject *__pyx_callargs[3] = {__pyx_t_3, __pyx_v_syevj, __pyx_v_dev_info};
    __pyx_t_2 = __Pyx_PyObject_FastCallMethod((PyObject*)__pyx_mstate_global->__pyx_n_u_check_cusolver_dev_info_if_sync, __pyx_callargs+__pyx_t_4, (3-__pyx_t_4) | (1*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
    __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
    __Pyx_DECREF(__pyx_t_12); __pyx_t_12 = 0;
    if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 554, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
  }
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "cupyx/cusolver.pyx":557
 *         syevj, dev_info)
 * 
 *     _cusolver.destroySyevjInfo(params)             # <<<<<<<<<<<<<<
 * 
 *     w = w.astype(w_dtype, copy=False)
*/
  __pyx_t_12 = NULL;
  __Pyx_GetModuleGlobalName(__pyx_t_3, __pyx_mstate_global->__pyx_n_u_cusolver); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 557, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_11 = __Pyx_PyObject_GetAttrStr(__pyx_t_3, __pyx_mstate_global->__pyx_n_u_destroySyevjInfo); if (unlikely(!__pyx_t_11)) __PYX_ERR(0, 557, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_11);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_t_4 = 1;
  #if CYTHON_UNPACK_METHODS
  if (unlikely(PyMethod_Check(__pyx_t_11))) {
    __pyx_t_12 = PyMethod_GET_SELF(__pyx_t_11);
    assert(__pyx_t_12);
    PyObject* __pyx__function = PyMethod_GET_FUNCTION(__pyx_t_11);
    __Pyx_INCREF(__pyx_t_12);
    __Pyx_INCREF(__pyx__function);
    __Pyx_DECREF_SET(__pyx_t_11, __pyx__function);
    __pyx_t_4 = 0;
  }
  #endif
  {
    PyObject *__pyx_callargs[2] = {__pyx_t_12, __pyx_v_params};
    __pyx_t_2 = __Pyx_PyObject_FastCall((PyObject*)__pyx_t_11, __pyx_callargs+__pyx_t_4, (2-__pyx_t_4) | (__pyx_t_4*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
    __Pyx_XDECREF(__pyx_t_12); __pyx_t_12 = 0;
    __Pyx_DECREF(__pyx_t_11); __pyx_t_11 = 0;
    if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 557, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
  }
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "cupyx/cusolver.pyx":559
 *     _cusolver.destroySyevjInfo(params)
 * 
 *     w = w.astype(w_dtype, copy=False)             # <<<<<<<<<<<<<<
 *     if not with_eigen_vector:
 *         return w
*/
  __pyx_t_11 = __pyx_v_w;
  __Pyx_INCREF(__pyx_t_11);
  __pyx_t_4 = 0;
  {
    PyObject *__pyx_callargs[2 + ((CYTHON_VECTORCALL) ? 1 : 0)] = {__pyx_t_11, __pyx_v_w_dtype};
    __pyx_t_12 = __Pyx_MakeVectorcallBuilderKwds(1); if (unlikely(!__pyx_t_12)) __PYX_ERR(0, 559, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_12);
    if (__Pyx_VectorcallBuilder_AddArg(__pyx_mstate_global->__pyx_n_u_copy, Py_False, __pyx_t_12, __pyx_callargs+2, 0) < (0)) __PYX_ERR(0, 559, __pyx_L1_error)
    __pyx_t_2 = __Pyx_Object_VectorcallMethod_CallFromBuilder((PyObject*)__pyx_mstate_global->__pyx_n_u_astype, __pyx_callargs+__pyx_t_4, (2-__pyx_t_4) | (1*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET), __pyx_t_12);
    __Pyx_XDECREF(__pyx_t_11); __pyx_t_11 = 0;
    __Pyx_DECREF(__pyx_t_12); __pyx_t_12 = 0;
    if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 559, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
  }
  __Pyx_DECREF_SET(__pyx_v_w, __pyx_t_2);
  __pyx_t_2 = 0;

  /* "cupyx/cusolver.pyx":560
 * 
 *     w = w.astype(w_dtype, copy=False)
 *     if not with_eigen_vector:             # <<<<<<<<<<<<<<
 *         return w
 *     v = v.astype(v_dtype, copy=False)
*/
  __pyx_t_5 = __Pyx_PyObject_IsTrue(__pyx_v_with_eigen_vector); if (unlikely((__pyx_t_5 < 0))) __PYX_ERR(0, 560, __pyx_L1_error)
  __pyx_t_6 = (!__pyx_t_5);
  if (__pyx_t_6) {

    /* "cupyx/cusolver.pyx":561
 *     w = w.astype(w_dtype, copy=False)
 *     if not with_eigen_vector:
 *         return w             # <<<<<<<<<<<<<<
 *     v = v.astype(v_dtype, copy=False)
 *     return w, v
*/
    __Pyx_XDECREF(__pyx_r);
    __Pyx_INCREF(__pyx_v_w);
    __pyx_r = __pyx_v_w;
    goto __pyx_L0;

    /* "cupyx/cusolver.pyx":560
 * 
 *     w = w.astype(w_dtype, copy=False)
 *     if not with_eigen_vector:             # <<<<<<<<<<<<<<
 *         return w
 *     v = v.astype(v_dtype, copy=False)
*/
  }

  /* "cupyx/cusolver.pyx":562
 *     if not with_eigen_vector:
 *         return w
 *     v = v.astype(v_dtype, copy=False)             # <<<<<<<<<<<<<<
 *     return w, v
 * 
*/
  __pyx_t_12 = __pyx_v_v;
  __Pyx_INCREF(__pyx_t_12);
  __pyx_t_4 = 0;
  {
    PyObject *__pyx_callargs[2 + ((CYTHON_VECTORCALL) ? 1 : 0)] = {__pyx_t_12, __pyx_v_v_dtype};
    __pyx_t_11 = __Pyx_MakeVectorcallBuilderKwds(1); if (unlikely(!__pyx_t_11)) __PYX_ERR(0, 562, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_11);
    if (__Pyx_VectorcallBuilder_AddArg(__pyx_mstate_global->__pyx_n_u_copy, Py_False, __pyx_t_11, __pyx_callargs+2, 0) < (0)) __PYX_ERR(0, 562, __pyx_L1_error)
    __pyx_t_2 = __Pyx_Object_VectorcallMethod_CallFromBuilder((PyObject*)__pyx_mstate_global->__pyx_n_u_astype, __pyx_callargs+__pyx_t_4, (2-__pyx_t_4) | (1*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET), __pyx_t_11);
    __Pyx_XDECREF(__pyx_t_12); __pyx_t_12 = 0;
    __Pyx_DECREF(__pyx_t_11); __pyx_t_11 = 0;
    if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 562, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
  }
  __Pyx_DECREF_SET(__pyx_v_v, __pyx_t_2);
  __pyx_t_2 = 0;

  /* "cupyx/cusolver.pyx":563
 *         return w
 *     v = v.astype(v_dtype, copy=False)
 *     return w, v             # <<<<<<<<<<<<<<
 * 
 * 
*/
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_2 = PyTuple_New(2); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 563, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_INCREF(__pyx_v_w);
  __Pyx_GIVEREF(__pyx_v_w);
  if (__Pyx_PyTuple_SET_ITEM(__pyx_t_2, 0, __pyx_v_w) != (0)) __PYX_ERR(0, 563, __pyx_L1_error);
  __Pyx_INCREF(__pyx_v_v);
  __Pyx_GIVEREF(__pyx_v_v);
  if (__Pyx_PyTuple_SET_ITEM(__pyx_t_2, 1, __pyx_v_v) != (0)) __PYX_ERR(0, 563, __pyx_L1_error);
  __pyx_r = __pyx_t_2;
  __pyx_t_2 = 0;
  goto __pyx_L0;

  /* "cupyx/cusolver.pyx":473
 * 
 * 
 * def syevj(a, UPLO='L', with_eigen_vector=True):             # <<<<<<<<<<<<<<
 *     """Eigenvalue decomposition of symmetric matrix using cusolverDn<t>syevj().
 * 
*/

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_7);
  __Pyx_XDECREF(__pyx_t_9);
  __Pyx_XDECREF(__pyx_t_10);
  __Pyx_XDECREF(__pyx_t_11);
  __Pyx_XDECREF(__pyx_t_12);
  __Pyx_AddTraceback("cupyx.cusolver.syevj", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_dtype);
  __Pyx_XDECREF(__pyx_v_v_dtype);
  __Pyx_XDECREF(__pyx_v_real_dtype);
  __Pyx_XDECREF(__pyx_v_w_dtype);
  __Pyx_XDECREF(__pyx_v_v);
  __Pyx_XDECREF(__pyx_v_m);
  __Pyx_XDECREF(__pyx_v_lda);
  __Pyx_XDECREF(__pyx_v_w);
  __Pyx_XDECREF(__pyx_v_dev_info);
  __Pyx_XDECREF(__pyx_v_handle);
  __Pyx_XDECREF(__pyx_v_jobz);
  __Pyx_XDECREF(__pyx_v_uplo);
  __Pyx_XDECREF(__pyx_v_buffer_size);
  __Pyx_XDECREF(__pyx_v_syevj);
  __Pyx_XDECREF(__pyx_v_params);
  __Pyx_XDECREF(__pyx_v_work_size);
  __Pyx_XDECREF(__pyx_v_work);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "cupyx/cusolver.pyx":566
 * 
 * 
 * def _syevj_batched(a, UPLO, with_eigen_vector):             # <<<<<<<<<<<<<<
 *     # reject_float16=False for backward compatibility
 *     dtype, v_dtype = _cupy.linalg._util.linalg_common_type(
*/

/* Python wrapper */
static PyObject *__pyx_pw_5cupyx_8cusolver_13_syevj_batched(PyObject *__pyx_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
PyDoc_STRVAR(__pyx_doc_5cupyx_8cusolver_12_syevj_batched, "_syevj_batched(a, UPLO, with_eigen_vector)");
static PyMethodDef __pyx_mdef_5cupyx_8cusolver_13_syevj_batched = {"_syevj_batched", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_5cupyx_8cusolver_13_syevj_batched, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_5cupyx_8cusolver_12_syevj_batched};
static PyObject *__pyx_pw_5cupyx_8cusolver_13_syevj_batched(PyObject *__pyx_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
) {
  PyObject *__pyx_v_a = 0;
  PyObject *__pyx_v_UPLO = 0;
  PyObject *__pyx_v_with_eigen_vector = 0;
  #if !CYTHON_METH_FASTCALL
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  #endif
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject* values[3] = {0,0,0};
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("_syevj_batched (wrapper)", 0);
  #if !CYTHON_METH_FASTCALL
  #if CYTHON_ASSUME_SAFE_SIZE
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return NULL;
  #endif
  #endif
  __pyx_kwvalues = __Pyx_KwValues_FASTCALL(__pyx_args, __pyx_nargs);
  {
    PyObject ** const __pyx_pyargnames[] = {&__pyx_mstate_global->__pyx_n_u_a,&__pyx_mstate_global->__pyx_n_u_UPLO,&__pyx_mstate_global->__pyx_n_u_with_eigen_vector,0};
    const Py_ssize_t __pyx_kwds_len = (__pyx_kwds) ? __Pyx_NumKwargs_FASTCALL(__pyx_kwds) : 0;
    if (unlikely(__pyx_kwds_len < 0)) __PYX_ERR(0, 566, __pyx_L3_error)
    if (__pyx_kwds_len > 0) {
      switch (__pyx_nargs) {
        case  3:
        values[2] = __Pyx_ArgRef_FASTCALL(__pyx_args, 2);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[2])) __PYX_ERR(0, 566, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  2:
        values[1] = __Pyx_ArgRef_FASTCALL(__pyx_args, 1);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[1])) __PYX_ERR(0, 566, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  1:
        values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(0, 566, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      const Py_ssize_t kwd_pos_args = __pyx_nargs;
      if (__Pyx_ParseKeywords(__pyx_kwds, __pyx_kwvalues, __pyx_pyargnames, 0, values, kwd_pos_args, __pyx_kwds_len, "_syevj_batched", 0) < (0)) __PYX_ERR(0, 566, __pyx_L3_error)
      for (Py_ssize_t i = __pyx_nargs; i < 3; i++) {
        if (unlikely(!values[i])) { __Pyx_RaiseArgtupleInvalid("_syevj_batched", 1, 3, 3, i); __PYX_ERR(0, 566, __pyx_L3_error) }
      }
    } else if (unlikely(__pyx_nargs != 3)) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
      if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(0, 566, __pyx_L3_error)
      values[1] = __Pyx_ArgRef_FASTCALL(__pyx_args, 1);
      if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[1])) __PYX_ERR(0, 566, __pyx_L3_error)
      values[2] = __Pyx_ArgRef_FASTCALL(__pyx_args, 2);
      if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[2])) __PYX_ERR(0, 566, __pyx_L3_error)
    }
    __pyx_v_a = values[0];
    __pyx_v_UPLO = values[1];
    __pyx_v_with_eigen_vector = values[2];
  }
  goto __pyx_L6_skip;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("_syevj_batched", 1, 3, 3, __pyx_nargs); __PYX_ERR(0, 566, __pyx_L3_error)
  __pyx_L6_skip:;
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_AddTraceback("cupyx.cusolver._syevj_batched", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_5cupyx_8cusolver_12_syevj_batched(__pyx_self, __pyx_v_a, __pyx_v_UPLO, __pyx_v_with_eigen_vector);

  /* function exit code */
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5cupyx_8cusolver_12_syevj_batched(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_a, PyObject *__pyx_v_UPLO, PyObject *__pyx_v_with_eigen_vector) {
  PyObject *__pyx_v_dtype = NULL;
  PyObject *__pyx_v_v_dtype = NULL;
  PyObject *__pyx_v_real_dtype = NULL;
  PyObject *__pyx_v_w_dtype = NULL;
  PyObject *__pyx_v_batch_shape = NULL;
  PyObject *__pyx_v_m = NULL;
  PyObject *__pyx_v_lda = NULL;
  PyObject *__pyx_v_batch_size = NULL;
  PyObject *__pyx_v_v = NULL;
  PyObject *__pyx_v_vp = NULL;
  PyObject *__pyx_v_w = NULL;
  PyObject *__pyx_v_dev_info = NULL;
  PyObject *__pyx_v_handle = NULL;
  PyObject *__pyx_v_jobz = NULL;
  PyObject *__pyx_v_uplo = NULL;
  PyObject *__pyx_v_buffer_size = NULL;
  PyObject *__pyx_v_syevjBatched = NULL;
  PyObject *__pyx_v_params = NULL;
  PyObject *__pyx_v_work_size = NULL;
  PyObject *__pyx_v_work = NULL;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  size_t __pyx_t_5;
  PyObject *(*__pyx_t_6)(PyObject *);
  Py_ssize_t __pyx_t_7;
  PyObject *__pyx_t_8 = NULL;
  PyObject *__pyx_t_9 = NULL;
  int __pyx_t_10;
  PyObject *__pyx_t_11 = NULL;
  PyObject *__pyx_t_12 = NULL;
  int __pyx_t_13;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("_syevj_batched", 0);
  __Pyx_INCREF(__pyx_v_a);

  /* "cupyx/cusolver.pyx":568
 * def _syevj_batched(a, UPLO, with_eigen_vector):
 *     # reject_float16=False for backward compatibility
 *     dtype, v_dtype = _cupy.linalg._util.linalg_common_type(             # <<<<<<<<<<<<<<
 *         a, reject_float16=False)
 *     real_dtype = dtype.char.lower()
*/
  __Pyx_GetModuleGlobalName(__pyx_t_3, __pyx_mstate_global->__pyx_n_u_cupy); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 568, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_4 = __Pyx_PyObject_GetAttrStr(__pyx_t_3, __pyx_mstate_global->__pyx_n_u_linalg); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 568, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_t_4, __pyx_mstate_global->__pyx_n_u_util); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 568, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __pyx_t_2 = __pyx_t_3;
  __Pyx_INCREF(__pyx_t_2);

  /* "cupyx/cusolver.pyx":569
 *     # reject_float16=False for backward compatibility
 *     dtype, v_dtype = _cupy.linalg._util.linalg_common_type(
 *         a, reject_float16=False)             # <<<<<<<<<<<<<<
 *     real_dtype = dtype.char.lower()
 *     w_dtype = v_dtype.char.lower()
*/
  __pyx_t_5 = 0;
  {
    PyObject *__pyx_callargs[2 + ((CYTHON_VECTORCALL) ? 1 : 0)] = {__pyx_t_2, __pyx_v_a};
    __pyx_t_4 = __Pyx_MakeVectorcallBuilderKwds(1); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 568, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    if (__Pyx_VectorcallBuilder_AddArg(__pyx_mstate_global->__pyx_n_u_reject_float16, Py_False, __pyx_t_4, __pyx_callargs+2, 0) < (0)) __PYX_ERR(0, 568, __pyx_L1_error)
    __pyx_t_1 = __Pyx_Object_VectorcallMethod_CallFromBuilder((PyObject*)__pyx_mstate_global->__pyx_n_u_linalg_common_type, __pyx_callargs+__pyx_t_5, (2-__pyx_t_5) | (1*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET), __pyx_t_4);
    __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 568, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
  }
  if ((likely(PyTuple_CheckExact(__pyx_t_1))) || (PyList_CheckExact(__pyx_t_1))) {
    PyObject* sequence = __pyx_t_1;
    Py_ssize_t size = __Pyx_PySequence_SIZE(sequence);
    if (unlikely(size != 2)) {
      if (size > 2) __Pyx_RaiseTooManyValuesError(2);
      else if (size >= 0) __Pyx_RaiseNeedMoreValuesError(size);
      __PYX_ERR(0, 568, __pyx_L1_error)
    }
    #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
    if (likely(PyTuple_CheckExact(sequence))) {
      __pyx_t_3 = PyTuple_GET_ITEM(sequence, 0);
      __Pyx_INCREF(__pyx_t_3);
      __pyx_t_4 = PyTuple_GET_ITEM(sequence, 1);
      __Pyx_INCREF(__pyx_t_4);
    } else {
      __pyx_t_3 = __Pyx_PyList_GetItemRefFast(sequence, 0, __Pyx_ReferenceSharing_SharedReference);
      if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 568, __pyx_L1_error)
      __Pyx_XGOTREF(__pyx_t_3);
      __pyx_t_4 = __Pyx_PyList_GetItemRefFast(sequence, 1, __Pyx_ReferenceSharing_SharedReference);
      if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 568, __pyx_L1_error)
      __Pyx_XGOTREF(__pyx_t_4);
    }
    #else
    __pyx_t_3 = __Pyx_PySequence_ITEM(sequence, 0); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 568, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_t_4 = __Pyx_PySequence_ITEM(sequence, 1); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 568, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    #endif
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  } else {
    Py_ssize_t index = -1;
    __pyx_t_2 = PyObject_GetIter(__pyx_t_1); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 568, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __pyx_t_6 = (CYTHON_COMPILING_IN_LIMITED_API) ? PyIter_Next : __Pyx_PyObject_GetIterNextFunc(__pyx_t_2);
    index = 0; __pyx_t_3 = __pyx_t_6(__pyx_t_2); if (unlikely(!__pyx_t_3)) goto __pyx_L3_unpacking_failed;
    __Pyx_GOTREF(__pyx_t_3);
    index = 1; __pyx_t_4 = __pyx_t_6(__pyx_t_2); if (unlikely(!__pyx_t_4)) goto __pyx_L3_unpacking_failed;
    __Pyx_GOTREF(__pyx_t_4);
    if (__Pyx_IternextUnpackEndCheck(__pyx_t_6(__pyx_t_2), 2) < (0)) __PYX_ERR(0, 568, __pyx_L1_error)
    __pyx_t_6 = NULL;
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    goto __pyx_L4_unpacking_done;
    __pyx_L3_unpacking_failed:;
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __pyx_t_6 = NULL;
    if (__Pyx_IterFinish() == 0) __Pyx_RaiseNeedMoreValuesError(index);
    __PYX_ERR(0, 568, __pyx_L1_error)
    __pyx_L4_unpacking_done:;
  }

  /* "cupyx/cusolver.pyx":568
 * def _syevj_batched(a, UPLO, with_eigen_vector):
 *     # reject_float16=False for backward compatibility
 *     dtype, v_dtype = _cupy.linalg._util.linalg_common_type(             # <<<<<<<<<<<<<<
 *         a, reject_float16=False)
 *     real_dtype = dtype.char.lower()
*/
  __pyx_v_dtype = __pyx_t_3;
  __pyx_t_3 = 0;
  __pyx_v_v_dtype = __pyx_t_4;
  __pyx_t_4 = 0;

  /* "cupyx/cusolver.pyx":570
 *     dtype, v_dtype = _cupy.linalg._util.linalg_common_type(
 *         a, reject_float16=False)
 *     real_dtype = dtype.char.lower()             # <<<<<<<<<<<<<<
 *     w_dtype = v_dtype.char.lower()
 * 
*/
  __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_v_dtype, __pyx_mstate_global->__pyx_n_u_char); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 570, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_4 = __pyx_t_3;
  __Pyx_INCREF(__pyx_t_4);
  __pyx_t_5 = 0;
  {
    PyObject *__pyx_callargs[2] = {__pyx_t_4, NULL};
    __pyx_t_1 = __Pyx_PyObject_FastCallMethod((PyObject*)__pyx_mstate_global->__pyx_n_u_lower, __pyx_callargs+__pyx_t_5, (1-__pyx_t_5) | (1*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
    __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 570, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
  }
  __pyx_v_real_dtype = __pyx_t_1;
  __pyx_t_1 = 0;

  /* "cupyx/cusolver.pyx":571
 *         a, reject_float16=False)
 *     real_dtype = dtype.char.lower()
 *     w_dtype = v_dtype.char.lower()             # <<<<<<<<<<<<<<
 * 
 *     *batch_shape, m, lda = a.shape
*/
  __pyx_t_4 = __Pyx_PyObject_GetAttrStr(__pyx_v_v_dtype, __pyx_mstate_global->__pyx_n_u_char); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 571, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_t_3 = __pyx_t_4;
  __Pyx_INCREF(__pyx_t_3);
  __pyx_t_5 = 0;
  {
    PyObject *__pyx_callargs[2] = {__pyx_t_3, NULL};
    __pyx_t_1 = __Pyx_PyObject_FastCallMethod((PyObject*)__pyx_mstate_global->__pyx_n_u_lower, __pyx_callargs+__pyx_t_5, (1-__pyx_t_5) | (1*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
    __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 571, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
  }
  __pyx_v_w_dtype = __pyx_t_1;
  __pyx_t_1 = 0;

  /* "cupyx/cusolver.pyx":573
 *     w_dtype = v_dtype.char.lower()
 * 
 *     *batch_shape, m, lda = a.shape             # <<<<<<<<<<<<<<
 *     batch_size = _numpy.prod(batch_shape)
 *     a = a.reshape(batch_size, m, lda)
*/
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_v_a, __pyx_mstate_global->__pyx_n_u_shape); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 573, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_4 = __Pyx_PySequence_ListKeepNew(__pyx_t_1); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 573, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_7 = __Pyx_PyList_GET_SIZE(__pyx_t_4);
  if (unlikely(__pyx_t_7 < 2)) {
    __Pyx_RaiseNeedMoreValuesError(0+__pyx_t_7); __PYX_ERR(0, 573, __pyx_L1_error)
  }
  #if CYTHON_COMPILING_IN_CPYTHON
  __pyx_t_3 = PyList_GET_ITEM(__pyx_t_4, __pyx_t_7-1); 
  ((PyVarObject*)__pyx_t_4)->ob_size--;
  #else
  __pyx_t_3 = __Pyx_PySequence_ITEM(__pyx_t_4, __pyx_t_7-1); 
  #endif
  __Pyx_GOTREF(__pyx_t_3);
  #if CYTHON_COMPILING_IN_CPYTHON
  __pyx_t_2 = PyList_GET_ITEM(__pyx_t_4, __pyx_t_7-2); 
  ((PyVarObject*)__pyx_t_4)->ob_size--;
  #else
  __pyx_t_2 = __Pyx_PySequence_ITEM(__pyx_t_4, __pyx_t_7-2); 
  #endif
  __Pyx_GOTREF(__pyx_t_2);
  #if !CYTHON_COMPILING_IN_CPYTHON
  __pyx_t_8 = PySequence_GetSlice(__pyx_t_4, 0, __pyx_t_7-2); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 573, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_8);
  __Pyx_DECREF(__pyx_t_4);
  __pyx_t_4 = __pyx_t_8; __pyx_t_8 = NULL;
  #else
  CYTHON_UNUSED_VAR(__pyx_t_8);
  #endif
  __pyx_v_batch_shape = ((PyObject*)__pyx_t_4);
  __pyx_t_4 = 0;
  __pyx_v_m = __pyx_t_2;
  __pyx_t_2 = 0;
  __pyx_v_lda = __pyx_t_3;
  __pyx_t_3 = 0;

  /* "cupyx/cusolver.pyx":574
 * 
 *     *batch_shape, m, lda = a.shape
 *     batch_size = _numpy.prod(batch_shape)             # <<<<<<<<<<<<<<
 *     a = a.reshape(batch_size, m, lda)
 *     v = _cupy.array(
*/
  __pyx_t_3 = NULL;
  __Pyx_GetModuleGlobalName(__pyx_t_2, __pyx_mstate_global->__pyx_n_u_numpy); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 574, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_4 = __Pyx_PyObject_GetAttrStr(__pyx_t_2, __pyx_mstate_global->__pyx_n_u_prod); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 574, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_5 = 1;
  #if CYTHON_UNPACK_METHODS
  if (unlikely(PyMethod_Check(__pyx_t_4))) {
    __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_4);
    assert(__pyx_t_3);
    PyObject* __pyx__function = PyMethod_GET_FUNCTION(__pyx_t_4);
    __Pyx_INCREF(__pyx_t_3);
    __Pyx_INCREF(__pyx__function);
    __Pyx_DECREF_SET(__pyx_t_4, __pyx__function);
    __pyx_t_5 = 0;
  }
  #endif
  {
    PyObject *__pyx_callargs[2] = {__pyx_t_3, __pyx_v_batch_shape};
    __pyx_t_1 = __Pyx_PyObject_FastCall((PyObject*)__pyx_t_4, __pyx_callargs+__pyx_t_5, (2-__pyx_t_5) | (__pyx_t_5*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
    __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 574, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
  }
  __pyx_v_batch_size = __pyx_t_1;
  __pyx_t_1 = 0;

  /* "cupyx/cusolver.pyx":575
 *     *batch_shape, m, lda = a.shape
 *     batch_size = _numpy.prod(batch_shape)
 *     a = a.reshape(batch_size, m, lda)             # <<<<<<<<<<<<<<
 *     v = _cupy.array(
 *         a.swapaxes(-2, -1), order='C', copy=True, dtype=dtype)
*/
  __pyx_t_4 = __pyx_v_a;
  __Pyx_INCREF(__pyx_t_4);
  __pyx_t_5 = 0;
  {
    PyObject *__pyx_callargs[4] = {__pyx_t_4, __pyx_v_batch_size, __pyx_v_m, __pyx_v_lda};
    __pyx_t_1 = __Pyx_PyObject_FastCallMethod((PyObject*)__pyx_mstate_global->__pyx_n_u_reshape, __pyx_callargs+__pyx_t_5, (4-__pyx_t_5) | (1*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
    __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
    if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 575, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
  }
  __Pyx_DECREF_SET(__pyx_v_a, __pyx_t_1);
  __pyx_t_1 = 0;

  /* "cupyx/cusolver.pyx":576
 *     batch_size = _numpy.prod(batch_shape)
 *     a = a.reshape(batch_size, m, lda)
 *     v = _cupy.array(             # <<<<<<<<<<<<<<
 *         a.swapaxes(-2, -1), order='C', copy=True, dtype=dtype)
 *     if runtime._is_hip_environment:
*/
  __pyx_t_4 = NULL;
  __Pyx_GetModuleGlobalName(__pyx_t_3, __pyx_mstate_global->__pyx_n_u_cupy); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 576, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_t_3, __pyx_mstate_global->__pyx_n_u_array); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 576, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "cupyx/cusolver.pyx":577
 *     a = a.reshape(batch_size, m, lda)
 *     v = _cupy.array(
 *         a.swapaxes(-2, -1), order='C', copy=True, dtype=dtype)             # <<<<<<<<<<<<<<
 *     if runtime._is_hip_environment:
 *         # the batched syev/heev has a different signature...
*/
  __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_v_a, __pyx_mstate_global->__pyx_n_u_swapaxes); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 577, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_8 = __Pyx_PyObject_Call(__pyx_t_3, __pyx_mstate_global->__pyx_tuple[0], NULL); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 577, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_8);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_t_5 = 1;
  #if CYTHON_UNPACK_METHODS
  if (unlikely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_2);
    assert(__pyx_t_4);
    PyObject* __pyx__function = PyMethod_GET_FUNCTION(__pyx_t_2);
    __Pyx_INCREF(__pyx_t_4);
    __Pyx_INCREF(__pyx__function);
    __Pyx_DECREF_SET(__pyx_t_2, __pyx__function);
    __pyx_t_5 = 0;
  }
  #endif
  {
    PyObject *__pyx_callargs[2 + ((CYTHON_VECTORCALL) ? 3 : 0)] = {__pyx_t_4, __pyx_t_8};
    __pyx_t_3 = __Pyx_MakeVectorcallBuilderKwds(3); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 576, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    if (__Pyx_VectorcallBuilder_AddArg(__pyx_mstate_global->__pyx_n_u_order, __pyx_mstate_global->__pyx_n_u_C, __pyx_t_3, __pyx_callargs+2, 0) < (0)) __PYX_ERR(0, 576, __pyx_L1_error)
    if (__Pyx_VectorcallBuilder_AddArg(__pyx_mstate_global->__pyx_n_u_copy, Py_True, __pyx_t_3, __pyx_callargs+2, 1) < (0)) __PYX_ERR(0, 576, __pyx_L1_error)
    if (__Pyx_VectorcallBuilder_AddArg(__pyx_mstate_global->__pyx_n_u_dtype, __pyx_v_dtype, __pyx_t_3, __pyx_callargs+2, 2) < (0)) __PYX_ERR(0, 576, __pyx_L1_error)
    __pyx_t_1 = __Pyx_Object_Vectorcall_CallFromBuilder((PyObject*)__pyx_t_2, __pyx_callargs+__pyx_t_5, (2-__pyx_t_5) | (__pyx_t_5*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET), __pyx_t_3);
    __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
    __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 576, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
  }
  __pyx_v_v = __pyx_t_1;
  __pyx_t_1 = 0;

  /* "cupyx/cusolver.pyx":578
 *     v = _cupy.array(
 *         a.swapaxes(-2, -1), order='C', copy=True, dtype=dtype)
 *     if runtime._is_hip_environment:             # <<<<<<<<<<<<<<
 *         # the batched syev/heev has a different signature...
 *         vp = _linalg._mat_ptrs(v)
*/
  if (__pyx_v_13cupy_backends_4cuda_3api_7runtime__is_hip_environment) {

    /* "cupyx/cusolver.pyx":580
 *     if runtime._is_hip_environment:
 *         # the batched syev/heev has a different signature...
 *         vp = _linalg._mat_ptrs(v)             # <<<<<<<<<<<<<<
 *     else:
 *         vp = v
*/
    __pyx_t_2 = NULL;
    __Pyx_GetModuleGlobalName(__pyx_t_3, __pyx_mstate_global->__pyx_n_u_linalg_2); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 580, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_t_8 = __Pyx_PyObject_GetAttrStr(__pyx_t_3, __pyx_mstate_global->__pyx_n_u_mat_ptrs); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 580, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_8);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __pyx_t_5 = 1;
    #if CYTHON_UNPACK_METHODS
    if (unlikely(PyMethod_Check(__pyx_t_8))) {
      __pyx_t_2 = PyMethod_GET_SELF(__pyx_t_8);
      assert(__pyx_t_2);
      PyObject* __pyx__function = PyMethod_GET_FUNCTION(__pyx_t_8);
      __Pyx_INCREF(__pyx_t_2);
      __Pyx_INCREF(__pyx__function);
      __Pyx_DECREF_SET(__pyx_t_8, __pyx__function);
      __pyx_t_5 = 0;
    }
    #endif
    {
      PyObject *__pyx_callargs[2] = {__pyx_t_2, __pyx_v_v};
      __pyx_t_1 = __Pyx_PyObject_FastCall((PyObject*)__pyx_t_8, __pyx_callargs+__pyx_t_5, (2-__pyx_t_5) | (__pyx_t_5*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
      __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
      __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
      if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 580, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
    }
    __pyx_v_vp = __pyx_t_1;
    __pyx_t_1 = 0;

    /* "cupyx/cusolver.pyx":578
 *     v = _cupy.array(
 *         a.swapaxes(-2, -1), order='C', copy=True, dtype=dtype)
 *     if runtime._is_hip_environment:             # <<<<<<<<<<<<<<
 *         # the batched syev/heev has a different signature...
 *         vp = _linalg._mat_ptrs(v)
*/
    goto __pyx_L5;
  }

  /* "cupyx/cusolver.pyx":582
 *         vp = _linalg._mat_ptrs(v)
 *     else:
 *         vp = v             # <<<<<<<<<<<<<<
 * 
 *     w = _cupy.empty((batch_size, m), real_dtype).swapaxes(-2, -1)
*/
  /*else*/ {
    __Pyx_INCREF(__pyx_v_v);
    __pyx_v_vp = __pyx_v_v;
  }
  __pyx_L5:;

  /* "cupyx/cusolver.pyx":584
 *         vp = v
 * 
 *     w = _cupy.empty((batch_size, m), real_dtype).swapaxes(-2, -1)             # <<<<<<<<<<<<<<
 *     dev_info = _cupy.empty((batch_size,), _cupy.int32)
 *     handle = _device.Device().cusolver_handle
*/
  __pyx_t_8 = NULL;
  __Pyx_GetModuleGlobalName(__pyx_t_2, __pyx_mstate_global->__pyx_n_u_cupy); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 584, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_t_2, __pyx_mstate_global->__pyx_n_u_empty); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 584, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_2 = PyTuple_New(2); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 584, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_INCREF(__pyx_v_batch_size);
  __Pyx_GIVEREF(__pyx_v_batch_size);
  if (__Pyx_PyTuple_SET_ITEM(__pyx_t_2, 0, __pyx_v_batch_size) != (0)) __PYX_ERR(0, 584, __pyx_L1_error);
  __Pyx_INCREF(__pyx_v_m);
  __Pyx_GIVEREF(__pyx_v_m);
  if (__Pyx_PyTuple_SET_ITEM(__pyx_t_2, 1, __pyx_v_m) != (0)) __PYX_ERR(0, 584, __pyx_L1_error);
  __pyx_t_5 = 1;
  #if CYTHON_UNPACK_METHODS
  if (unlikely(PyMethod_Check(__pyx_t_3))) {
    __pyx_t_8 = PyMethod_GET_SELF(__pyx_t_3);
    assert(__pyx_t_8);
    PyObject* __pyx__function = PyMethod_GET_FUNCTION(__pyx_t_3);
    __Pyx_INCREF(__pyx_t_8);
    __Pyx_INCREF(__pyx__function);
    __Pyx_DECREF_SET(__pyx_t_3, __pyx__function);
    __pyx_t_5 = 0;
  }
  #endif
  {
    PyObject *__pyx_callargs[3] = {__pyx_t_8, __pyx_t_2, __pyx_v_real_dtype};
    __pyx_t_1 = __Pyx_PyObject_FastCall((PyObject*)__pyx_t_3, __pyx_callargs+__pyx_t_5, (3-__pyx_t_5) | (__pyx_t_5*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
    __Pyx_XDECREF(__pyx_t_8); __pyx_t_8 = 0;
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 584, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
  }
  __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_t_1, __pyx_mstate_global->__pyx_n_u_swapaxes); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 584, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_3, __pyx_mstate_global->__pyx_tuple[0], NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 584, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_v_w = __pyx_t_1;
  __pyx_t_1 = 0;

  /* "cupyx/cusolver.pyx":585
 * 
 *     w = _cupy.empty((batch_size, m), real_dtype).swapaxes(-2, -1)
 *     dev_info = _cupy.empty((batch_size,), _cupy.int32)             # <<<<<<<<<<<<<<
 *     handle = _device.Device().cusolver_handle
 * 
*/
  __pyx_t_3 = NULL;
  __Pyx_GetModuleGlobalName(__pyx_t_2, __pyx_mstate_global->__pyx_n_u_cupy); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 585, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_8 = __Pyx_PyObject_GetAttrStr(__pyx_t_2, __pyx_mstate_global->__pyx_n_u_empty); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 585, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_8);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_2 = PyTuple_New(1); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 585, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_INCREF(__pyx_v_batch_size);
  __Pyx_GIVEREF(__pyx_v_batch_size);
  if (__Pyx_PyTuple_SET_ITEM(__pyx_t_2, 0, __pyx_v_batch_size) != (0)) __PYX_ERR(0, 585, __pyx_L1_error);
  __Pyx_GetModuleGlobalName(__pyx_t_4, __pyx_mstate_global->__pyx_n_u_cupy); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 585, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_t_9 = __Pyx_PyObject_GetAttrStr(__pyx_t_4, __pyx_mstate_global->__pyx_n_u_int32); if (unlikely(!__pyx_t_9)) __PYX_ERR(0, 585, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_9);
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __pyx_t_5 = 1;
  #if CYTHON_UNPACK_METHODS
  if (unlikely(PyMethod_Check(__pyx_t_8))) {
    __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_8);
    assert(__pyx_t_3);
    PyObject* __pyx__function = PyMethod_GET_FUNCTION(__pyx_t_8);
    __Pyx_INCREF(__pyx_t_3);
    __Pyx_INCREF(__pyx__function);
    __Pyx_DECREF_SET(__pyx_t_8, __pyx__function);
    __pyx_t_5 = 0;
  }
  #endif
  {
    PyObject *__pyx_callargs[3] = {__pyx_t_3, __pyx_t_2, __pyx_t_9};
    __pyx_t_1 = __Pyx_PyObject_FastCall((PyObject*)__pyx_t_8, __pyx_callargs+__pyx_t_5, (3-__pyx_t_5) | (__pyx_t_5*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
    __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
    __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
    if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 585, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
  }
  __pyx_v_dev_info = __pyx_t_1;
  __pyx_t_1 = 0;

  /* "cupyx/cusolver.pyx":586
 *     w = _cupy.empty((batch_size, m), real_dtype).swapaxes(-2, -1)
 *     dev_info = _cupy.empty((batch_size,), _cupy.int32)
 *     handle = _device.Device().cusolver_handle             # <<<<<<<<<<<<<<
 * 
 *     if with_eigen_vector:
*/
  __pyx_t_8 = NULL;
  __Pyx_GetModuleGlobalName(__pyx_t_9, __pyx_mstate_global->__pyx_n_u_device); if (unlikely(!__pyx_t_9)) __PYX_ERR(0, 586, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_9);
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_t_9, __pyx_mstate_global->__pyx_n_u_Device); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 586, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
  __pyx_t_5 = 1;
  #if CYTHON_UNPACK_METHODS
  if (unlikely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_8 = PyMethod_GET_SELF(__pyx_t_2);
    assert(__pyx_t_8);
    PyObject* __pyx__function = PyMethod_GET_FUNCTION(__pyx_t_2);
    __Pyx_INCREF(__pyx_t_8);
    __Pyx_INCREF(__pyx__function);
    __Pyx_DECREF_SET(__pyx_t_2, __pyx__function);
    __pyx_t_5 = 0;
  }
  #endif
  {
    PyObject *__pyx_callargs[2] = {__pyx_t_8, NULL};
    __pyx_t_1 = __Pyx_PyObject_FastCall((PyObject*)__pyx_t_2, __pyx_callargs+__pyx_t_5, (1-__pyx_t_5) | (__pyx_t_5*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
    __Pyx_XDECREF(__pyx_t_8); __pyx_t_8 = 0;
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 586, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
  }
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_t_1, __pyx_mstate_global->__pyx_n_u_cusolver_handle); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 586, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v_handle = __pyx_t_2;
  __pyx_t_2 = 0;

  /* "cupyx/cusolver.pyx":588
 *     handle = _device.Device().cusolver_handle
 * 
 *     if with_eigen_vector:             # <<<<<<<<<<<<<<
 *         jobz = _cusolver.CUSOLVER_EIG_MODE_VECTOR
 *     else:
*/
  __pyx_t_10 = __Pyx_PyObject_IsTrue(__pyx_v_with_eigen_vector); if (unlikely((__pyx_t_10 < 0))) __PYX_ERR(0, 588, __pyx_L1_error)
  if (__pyx_t_10) {

    /* "cupyx/cusolver.pyx":589
 * 
 *     if with_eigen_vector:
 *         jobz = _cusolver.CUSOLVER_EIG_MODE_VECTOR             # <<<<<<<<<<<<<<
 *     else:
 *         jobz = _cusolver.CUSOLVER_EIG_MODE_NOVECTOR
*/
    __Pyx_GetModuleGlobalName(__pyx_t_2, __pyx_mstate_global->__pyx_n_u_cusolver); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 589, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_t_2, __pyx_mstate_global->__pyx_n_u_CUSOLVER_EIG_MODE_VECTOR); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 589, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __pyx_v_jobz = __pyx_t_1;
    __pyx_t_1 = 0;

    /* "cupyx/cusolver.pyx":588
 *     handle = _device.Device().cusolver_handle
 * 
 *     if with_eigen_vector:             # <<<<<<<<<<<<<<
 *         jobz = _cusolver.CUSOLVER_EIG_MODE_VECTOR
 *     else:
*/
    goto __pyx_L6;
  }

  /* "cupyx/cusolver.pyx":591
 *         jobz = _cusolver.CUSOLVER_EIG_MODE_VECTOR
 *     else:
 *         jobz = _cusolver.CUSOLVER_EIG_MODE_NOVECTOR             # <<<<<<<<<<<<<<
 * 
 *     if UPLO == 'L':
*/
  /*else*/ {
    __Pyx_GetModuleGlobalName(__pyx_t_1, __pyx_mstate_global->__pyx_n_u_cusolver); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 591, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_t_1, __pyx_mstate_global->__pyx_n_u_CUSOLVER_EIG_MODE_NOVECTOR); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 591, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __pyx_v_jobz = __pyx_t_2;
    __pyx_t_2 = 0;
  }
  __pyx_L6:;

  /* "cupyx/cusolver.pyx":593
 *         jobz = _cusolver.CUSOLVER_EIG_MODE_NOVECTOR
 * 
 *     if UPLO == 'L':             # <<<<<<<<<<<<<<
 *         uplo = _cublas.CUBLAS_FILL_MODE_LOWER
 *     else:  # UPLO == 'U'
*/
  __pyx_t_10 = (__Pyx_PyUnicode_Equals(__pyx_v_UPLO, __pyx_mstate_global->__pyx_n_u_L, Py_EQ)); if (unlikely((__pyx_t_10 < 0))) __PYX_ERR(0, 593, __pyx_L1_error)
  if (__pyx_t_10) {

    /* "cupyx/cusolver.pyx":594
 * 
 *     if UPLO == 'L':
 *         uplo = _cublas.CUBLAS_FILL_MODE_LOWER             # <<<<<<<<<<<<<<
 *     else:  # UPLO == 'U'
 *         uplo = _cublas.CUBLAS_FILL_MODE_UPPER
*/
    __Pyx_GetModuleGlobalName(__pyx_t_2, __pyx_mstate_global->__pyx_n_u_cublas); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 594, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_t_2, __pyx_mstate_global->__pyx_n_u_CUBLAS_FILL_MODE_LOWER); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 594, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __pyx_v_uplo = __pyx_t_1;
    __pyx_t_1 = 0;

    /* "cupyx/cusolver.pyx":593
 *         jobz = _cusolver.CUSOLVER_EIG_MODE_NOVECTOR
 * 
 *     if UPLO == 'L':             # <<<<<<<<<<<<<<
 *         uplo = _cublas.CUBLAS_FILL_MODE_LOWER
 *     else:  # UPLO == 'U'
*/
    goto __pyx_L7;
  }

  /* "cupyx/cusolver.pyx":596
 *         uplo = _cublas.CUBLAS_FILL_MODE_LOWER
 *     else:  # UPLO == 'U'
 *         uplo = _cublas.CUBLAS_FILL_MODE_UPPER             # <<<<<<<<<<<<<<
 * 
 *     if dtype == 'f':
*/
  /*else*/ {
    __Pyx_GetModuleGlobalName(__pyx_t_1, __pyx_mstate_global->__pyx_n_u_cublas); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 596, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_t_1, __pyx_mstate_global->__pyx_n_u_CUBLAS_FILL_MODE_UPPER); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 596, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __pyx_v_uplo = __pyx_t_2;
    __pyx_t_2 = 0;
  }
  __pyx_L7:;

  /* "cupyx/cusolver.pyx":598
 *         uplo = _cublas.CUBLAS_FILL_MODE_UPPER
 * 
 *     if dtype == 'f':             # <<<<<<<<<<<<<<
 *         buffer_size = _cusolver.ssyevjBatched_bufferSize
 *         syevjBatched = _cusolver.ssyevjBatched
*/
  __pyx_t_10 = (__Pyx_PyUnicode_Equals(__pyx_v_dtype, __pyx_mstate_global->__pyx_n_u_f, Py_EQ)); if (unlikely((__pyx_t_10 < 0))) __PYX_ERR(0, 598, __pyx_L1_error)
  if (__pyx_t_10) {

    /* "cupyx/cusolver.pyx":599
 * 
 *     if dtype == 'f':
 *         buffer_size = _cusolver.ssyevjBatched_bufferSize             # <<<<<<<<<<<<<<
 *         syevjBatched = _cusolver.ssyevjBatched
 *     elif dtype == 'd':
*/
    __Pyx_GetModuleGlobalName(__pyx_t_2, __pyx_mstate_global->__pyx_n_u_cusolver); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 599, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_t_2, __pyx_mstate_global->__pyx_n_u_ssyevjBatched_bufferSize); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 599, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __pyx_v_buffer_size = __pyx_t_1;
    __pyx_t_1 = 0;

    /* "cupyx/cusolver.pyx":600
 *     if dtype == 'f':
 *         buffer_size = _cusolver.ssyevjBatched_bufferSize
 *         syevjBatched = _cusolver.ssyevjBatched             # <<<<<<<<<<<<<<
 *     elif dtype == 'd':
 *         buffer_size = _cusolver.dsyevjBatched_bufferSize
*/
    __Pyx_GetModuleGlobalName(__pyx_t_1, __pyx_mstate_global->__pyx_n_u_cusolver); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 600, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_t_1, __pyx_mstate_global->__pyx_n_u_ssyevjBatched); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 600, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __pyx_v_syevjBatched = __pyx_t_2;
    __pyx_t_2 = 0;

    /* "cupyx/cusolver.pyx":598
 *         uplo = _cublas.CUBLAS_FILL_MODE_UPPER
 * 
 *     if dtype == 'f':             # <<<<<<<<<<<<<<
 *         buffer_size = _cusolver.ssyevjBatched_bufferSize
 *         syevjBatched = _cusolver.ssyevjBatched
*/
    goto __pyx_L8;
  }

  /* "cupyx/cusolver.pyx":601
 *         buffer_size = _cusolver.ssyevjBatched_bufferSize
 *         syevjBatched = _cusolver.ssyevjBatched
 *     elif dtype == 'd':             # <<<<<<<<<<<<<<
 *         buffer_size = _cusolver.dsyevjBatched_bufferSize
 *         syevjBatched = _cusolver.dsyevjBatched
*/
  __pyx_t_10 = (__Pyx_PyUnicode_Equals(__pyx_v_dtype, __pyx_mstate_global->__pyx_n_u_d, Py_EQ)); if (unlikely((__pyx_t_10 < 0))) __PYX_ERR(0, 601, __pyx_L1_error)
  if (__pyx_t_10) {

    /* "cupyx/cusolver.pyx":602
 *         syevjBatched = _cusolver.ssyevjBatched
 *     elif dtype == 'd':
 *         buffer_size = _cusolver.dsyevjBatched_bufferSize             # <<<<<<<<<<<<<<
 *         syevjBatched = _cusolver.dsyevjBatched
 *     elif dtype == 'F':
*/
    __Pyx_GetModuleGlobalName(__pyx_t_2, __pyx_mstate_global->__pyx_n_u_cusolver); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 602, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_t_2, __pyx_mstate_global->__pyx_n_u_dsyevjBatched_bufferSize); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 602, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __pyx_v_buffer_size = __pyx_t_1;
    __pyx_t_1 = 0;

    /* "cupyx/cusolver.pyx":603
 *     elif dtype == 'd':
 *         buffer_size = _cusolver.dsyevjBatched_bufferSize
 *         syevjBatched = _cusolver.dsyevjBatched             # <<<<<<<<<<<<<<
 *     elif dtype == 'F':
 *         buffer_size = _cusolver.cheevjBatched_bufferSize
*/
    __Pyx_GetModuleGlobalName(__pyx_t_1, __pyx_mstate_global->__pyx_n_u_cusolver); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 603, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_t_1, __pyx_mstate_global->__pyx_n_u_dsyevjBatched); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 603, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __pyx_v_syevjBatched = __pyx_t_2;
    __pyx_t_2 = 0;

    /* "cupyx/cusolver.pyx":601
 *         buffer_size = _cusolver.ssyevjBatched_bufferSize
 *         syevjBatched = _cusolver.ssyevjBatched
 *     elif dtype == 'd':             # <<<<<<<<<<<<<<
 *         buffer_size = _cusolver.dsyevjBatched_bufferSize
 *         syevjBatched = _cusolver.dsyevjBatched
*/
    goto __pyx_L8;
  }

  /* "cupyx/cusolver.pyx":604
 *         buffer_size = _cusolver.dsyevjBatched_bufferSize
 *         syevjBatched = _cusolver.dsyevjBatched
 *     elif dtype == 'F':             # <<<<<<<<<<<<<<
 *         buffer_size = _cusolver.cheevjBatched_bufferSize
 *         syevjBatched = _cusolver.cheevjBatched
*/
  __pyx_t_10 = (__Pyx_PyUnicode_Equals(__pyx_v_dtype, __pyx_mstate_global->__pyx_n_u_F, Py_EQ)); if (unlikely((__pyx_t_10 < 0))) __PYX_ERR(0, 604, __pyx_L1_error)
  if (__pyx_t_10) {

    /* "cupyx/cusolver.pyx":605
 *         syevjBatched = _cusolver.dsyevjBatched
 *     elif dtype == 'F':
 *         buffer_size = _cusolver.cheevjBatched_bufferSize             # <<<<<<<<<<<<<<
 *         syevjBatched = _cusolver.cheevjBatched
 *     elif dtype == 'D':
*/
    __Pyx_GetModuleGlobalName(__pyx_t_2, __pyx_mstate_global->__pyx_n_u_cusolver); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 605, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_t_2, __pyx_mstate_global->__pyx_n_u_cheevjBatched_bufferSize); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 605, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __pyx_v_buffer_size = __pyx_t_1;
    __pyx_t_1 = 0;

    /* "cupyx/cusolver.pyx":606
 *     elif dtype == 'F':
 *         buffer_size = _cusolver.cheevjBatched_bufferSize
 *         syevjBatched = _cusolver.cheevjBatched             # <<<<<<<<<<<<<<
 *     elif dtype == 'D':
 *         buffer_size = _cusolver.zheevjBatched_bufferSize
*/
    __Pyx_GetModuleGlobalName(__pyx_t_1, __pyx_mstate_global->__pyx_n_u_cusolver); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 606, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_t_1, __pyx_mstate_global->__pyx_n_u_cheevjBatched); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 606, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __pyx_v_syevjBatched = __pyx_t_2;
    __pyx_t_2 = 0;

    /* "cupyx/cusolver.pyx":604
 *         buffer_size = _cusolver.dsyevjBatched_bufferSize
 *         syevjBatched = _cusolver.dsyevjBatched
 *     elif dtype == 'F':             # <<<<<<<<<<<<<<
 *         buffer_size = _cusolver.cheevjBatched_bufferSize
 *         syevjBatched = _cusolver.cheevjBatched
*/
    goto __pyx_L8;
  }

  /* "cupyx/cusolver.pyx":607
 *         buffer_size = _cusolver.cheevjBatched_bufferSize
 *         syevjBatched = _cusolver.cheevjBatched
 *     elif dtype == 'D':             # <<<<<<<<<<<<<<
 *         buffer_size = _cusolver.zheevjBatched_bufferSize
 *         syevjBatched = _cusolver.zheevjBatched
*/
  __pyx_t_10 = (__Pyx_PyUnicode_Equals(__pyx_v_dtype, __pyx_mstate_global->__pyx_n_u_D, Py_EQ)); if (unlikely((__pyx_t_10 < 0))) __PYX_ERR(0, 607, __pyx_L1_error)
  if (likely(__pyx_t_10)) {

    /* "cupyx/cusolver.pyx":608
 *         syevjBatched = _cusolver.cheevjBatched
 *     elif dtype == 'D':
 *         buffer_size = _cusolver.zheevjBatched_bufferSize             # <<<<<<<<<<<<<<
 *         syevjBatched = _cusolver.zheevjBatched
 *     else:
*/
    __Pyx_GetModuleGlobalName(__pyx_t_2, __pyx_mstate_global->__pyx_n_u_cusolver); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 608, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_t_2, __pyx_mstate_global->__pyx_n_u_zheevjBatched_bufferSize); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 608, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __pyx_v_buffer_size = __pyx_t_1;
    __pyx_t_1 = 0;

    /* "cupyx/cusolver.pyx":609
 *     elif dtype == 'D':
 *         buffer_size = _cusolver.zheevjBatched_bufferSize
 *         syevjBatched = _cusolver.zheevjBatched             # <<<<<<<<<<<<<<
 *     else:
 *         raise RuntimeError('Only float and double and cuComplex and '
*/
    __Pyx_GetModuleGlobalName(__pyx_t_1, __pyx_mstate_global->__pyx_n_u_cusolver); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 609, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_t_1, __pyx_mstate_global->__pyx_n_u_zheevjBatched); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 609, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __pyx_v_syevjBatched = __pyx_t_2;
    __pyx_t_2 = 0;

    /* "cupyx/cusolver.pyx":607
 *         buffer_size = _cusolver.cheevjBatched_bufferSize
 *         syevjBatched = _cusolver.cheevjBatched
 *     elif dtype == 'D':             # <<<<<<<<<<<<<<
 *         buffer_size = _cusolver.zheevjBatched_bufferSize
 *         syevjBatched = _cusolver.zheevjBatched
*/
    goto __pyx_L8;
  }

  /* "cupyx/cusolver.pyx":611
 *         syevjBatched = _cusolver.zheevjBatched
 *     else:
 *         raise RuntimeError('Only float and double and cuComplex and '             # <<<<<<<<<<<<<<
 *                            + 'cuDoubleComplex are supported')
 * 
*/
  /*else*/ {
    __pyx_t_1 = NULL;
    __pyx_t_5 = 1;
    {
      PyObject *__pyx_callargs[2] = {__pyx_t_1, __pyx_mstate_global->__pyx_kp_u_Only_float_and_double_and_cuComp};
      __pyx_t_2 = __Pyx_PyObject_FastCall((PyObject*)(((PyTypeObject*)PyExc_RuntimeError)), __pyx_callargs+__pyx_t_5, (2-__pyx_t_5) | (__pyx_t_5*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
      __Pyx_XDECREF(__pyx_t_1); __pyx_t_1 = 0;
      if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 611, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_2);
    }
    __Pyx_Raise(__pyx_t_2, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __PYX_ERR(0, 611, __pyx_L1_error)
  }
  __pyx_L8:;

  /* "cupyx/cusolver.pyx":614
 *                            + 'cuDoubleComplex are supported')
 * 
 *     params = _cusolver.createSyevjInfo()             # <<<<<<<<<<<<<<
 *     work_size = buffer_size(
 *         handle, jobz, uplo, m, vp.data.ptr, lda,
*/
  __pyx_t_1 = NULL;
  __Pyx_GetModuleGlobalName(__pyx_t_8, __pyx_mstate_global->__pyx_n_u_cusolver); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 614, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_8);
  __pyx_t_9 = __Pyx_PyObject_GetAttrStr(__pyx_t_8, __pyx_mstate_global->__pyx_n_u_createSyevjInfo); if (unlikely(!__pyx_t_9)) __PYX_ERR(0, 614, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_9);
  __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
  __pyx_t_5 = 1;
  #if CYTHON_UNPACK_METHODS
  if (unlikely(PyMethod_Check(__pyx_t_9))) {
    __pyx_t_1 = PyMethod_GET_SELF(__pyx_t_9);
    assert(__pyx_t_1);
    PyObject* __pyx__function = PyMethod_GET_FUNCTION(__pyx_t_9);
    __Pyx_INCREF(__pyx_t_1);
    __Pyx_INCREF(__pyx__function);
    __Pyx_DECREF_SET(__pyx_t_9, __pyx__function);
    __pyx_t_5 = 0;
  }
  #endif
  {
    PyObject *__pyx_callargs[2] = {__pyx_t_1, NULL};
    __pyx_t_2 = __Pyx_PyObject_FastCall((PyObject*)__pyx_t_9, __pyx_callargs+__pyx_t_5, (1-__pyx_t_5) | (__pyx_t_5*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
    __Pyx_XDECREF(__pyx_t_1); __pyx_t_1 = 0;
    __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
    if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 614, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
  }
  __pyx_v_params = __pyx_t_2;
  __pyx_t_2 = 0;

  /* "cupyx/cusolver.pyx":615
 * 
 *     params = _cusolver.createSyevjInfo()
 *     work_size = buffer_size(             # <<<<<<<<<<<<<<
 *         handle, jobz, uplo, m, vp.data.ptr, lda,
 *         w.data.ptr, params, batch_size)
*/
  __pyx_t_9 = NULL;
  __Pyx_INCREF(__pyx_v_buffer_size);
  __pyx_t_1 = __pyx_v_buffer_size; 

  /* "cupyx/cusolver.pyx":616
 *     params = _cusolver.createSyevjInfo()
 *     work_size = buffer_size(
 *         handle, jobz, uplo, m, vp.data.ptr, lda,             # <<<<<<<<<<<<<<
 *         w.data.ptr, params, batch_size)
 *     work = _cupy.empty(work_size, dtype)
*/
  __pyx_t_8 = __Pyx_PyObject_GetAttrStr(__pyx_v_vp, __pyx_mstate_global->__pyx_n_u_data); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 616, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_8);
  __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_t_8, __pyx_mstate_global->__pyx_n_u_ptr); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 616, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;

  /* "cupyx/cusolver.pyx":617
 *     work_size = buffer_size(
 *         handle, jobz, uplo, m, vp.data.ptr, lda,
 *         w.data.ptr, params, batch_size)             # <<<<<<<<<<<<<<
 *     work = _cupy.empty(work_size, dtype)
 *     syevjBatched(
*/
  __pyx_t_8 = __Pyx_PyObject_GetAttrStr(__pyx_v_w, __pyx_mstate_global->__pyx_n_u_data); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 617, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_8);
  __pyx_t_4 = __Pyx_PyObject_GetAttrStr(__pyx_t_8, __pyx_mstate_global->__pyx_n_u_ptr); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 617, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
  __pyx_t_5 = 1;
  #if CYTHON_UNPACK_METHODS
  if (unlikely(PyMethod_Check(__pyx_t_1))) {
    __pyx_t_9 = PyMethod_GET_SELF(__pyx_t_1);
    assert(__pyx_t_9);
    PyObject* __pyx__function = PyMethod_GET_FUNCTION(__pyx_t_1);
    __Pyx_INCREF(__pyx_t_9);
    __Pyx_INCREF(__pyx__function);
    __Pyx_DECREF_SET(__pyx_t_1, __pyx__function);
    __pyx_t_5 = 0;
  }
  #endif
  {
    PyObject *__pyx_callargs[10] = {__pyx_t_9, __pyx_v_handle, __pyx_v_jobz, __pyx_v_uplo, __pyx_v_m, __pyx_t_3, __pyx_v_lda, __pyx_t_4, __pyx_v_params, __pyx_v_batch_size};
    __pyx_t_2 = __Pyx_PyObject_FastCall((PyObject*)__pyx_t_1, __pyx_callargs+__pyx_t_5, (10-__pyx_t_5) | (__pyx_t_5*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
    __Pyx_XDECREF(__pyx_t_9); __pyx_t_9 = 0;
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 615, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
  }
  __pyx_v_work_size = __pyx_t_2;
  __pyx_t_2 = 0;

  /* "cupyx/cusolver.pyx":618
 *         handle, jobz, uplo, m, vp.data.ptr, lda,
 *         w.data.ptr, params, batch_size)
 *     work = _cupy.empty(work_size, dtype)             # <<<<<<<<<<<<<<
 *     syevjBatched(
 *         handle, jobz, uplo, m, vp.data.ptr, lda,
*/
  __pyx_t_1 = NULL;
  __Pyx_GetModuleGlobalName(__pyx_t_4, __pyx_mstate_global->__pyx_n_u_cupy); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 618, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_t_4, __pyx_mstate_global->__pyx_n_u_empty); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 618, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __pyx_t_5 = 1;
  #if CYTHON_UNPACK_METHODS
  if (unlikely(PyMethod_Check(__pyx_t_3))) {
    __pyx_t_1 = PyMethod_GET_SELF(__pyx_t_3);
    assert(__pyx_t_1);
    PyObject* __pyx__function = PyMethod_GET_FUNCTION(__pyx_t_3);
    __Pyx_INCREF(__pyx_t_1);
    __Pyx_INCREF(__pyx__function);
    __Pyx_DECREF_SET(__pyx_t_3, __pyx__function);
    __pyx_t_5 = 0;
  }
  #endif
  {
    PyObject *__pyx_callargs[3] = {__pyx_t_1, __pyx_v_work_size, __pyx_v_dtype};
    __pyx_t_2 = __Pyx_PyObject_FastCall((PyObject*)__pyx_t_3, __pyx_callargs+__pyx_t_5, (3-__pyx_t_5) | (__pyx_t_5*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
    __Pyx_XDECREF(__pyx_t_1); __pyx_t_1 = 0;
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 618, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
  }
  __pyx_v_work = __pyx_t_2;
  __pyx_t_2 = 0;

  /* "cupyx/cusolver.pyx":619
 *         w.data.ptr, params, batch_size)
 *     work = _cupy.empty(work_size, dtype)
 *     syevjBatched(             # <<<<<<<<<<<<<<
 *         handle, jobz, uplo, m, vp.data.ptr, lda,
 *         w.data.ptr, work.data.ptr, work_size, dev_info.data.ptr, params,
*/
  __pyx_t_3 = NULL;
  __Pyx_INCREF(__pyx_v_syevjBatched);
  __pyx_t_1 = __pyx_v_syevjBatched; 

  /* "cupyx/cusolver.pyx":620
 *     work = _cupy.empty(work_size, dtype)
 *     syevjBatched(
 *         handle, jobz, uplo, m, vp.data.ptr, lda,             # <<<<<<<<<<<<<<
 *         w.data.ptr, work.data.ptr, work_size, dev_info.data.ptr, params,
 *         batch_size)
*/
  __pyx_t_4 = __Pyx_PyObject_GetAttrStr(__pyx_v_vp, __pyx_mstate_global->__pyx_n_u_data); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 620, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_t_9 = __Pyx_PyObject_GetAttrStr(__pyx_t_4, __pyx_mstate_global->__pyx_n_u_ptr); if (unlikely(!__pyx_t_9)) __PYX_ERR(0, 620, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_9);
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

  /* "cupyx/cusolver.pyx":621
 *     syevjBatched(
 *         handle, jobz, uplo, m, vp.data.ptr, lda,
 *         w.data.ptr, work.data.ptr, work_size, dev_info.data.ptr, params,             # <<<<<<<<<<<<<<
 *         batch_size)
 *     _cupy.linalg._util._check_cusolver_dev_info_if_synchronization_allowed(
*/
  __pyx_t_4 = __Pyx_PyObject_GetAttrStr(__pyx_v_w, __pyx_mstate_global->__pyx_n_u_data); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 621, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_t_8 = __Pyx_PyObject_GetAttrStr(__pyx_t_4, __pyx_mstate_global->__pyx_n_u_ptr); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 621, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_8);
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __pyx_t_4 = __Pyx_PyObject_GetAttrStr(__pyx_v_work, __pyx_mstate_global->__pyx_n_u_data); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 621, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_t_11 = __Pyx_PyObject_GetAttrStr(__pyx_t_4, __pyx_mstate_global->__pyx_n_u_ptr); if (unlikely(!__pyx_t_11)) __PYX_ERR(0, 621, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_11);
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __pyx_t_4 = __Pyx_PyObject_GetAttrStr(__pyx_v_dev_info, __pyx_mstate_global->__pyx_n_u_data); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 621, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_t_12 = __Pyx_PyObject_GetAttrStr(__pyx_t_4, __pyx_mstate_global->__pyx_n_u_ptr); if (unlikely(!__pyx_t_12)) __PYX_ERR(0, 621, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_12);
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

  /* "cupyx/cusolver.pyx":622
 *         handle, jobz, uplo, m, vp.data.ptr, lda,
 *         w.data.ptr, work.data.ptr, work_size, dev_info.data.ptr, params,
 *         batch_size)             # <<<<<<<<<<<<<<
 *     _cupy.linalg._util._check_cusolver_dev_info_if_synchronization_allowed(
 *         syevjBatched, dev_info)
*/
  __pyx_t_5 = 1;
  #if CYTHON_UNPACK_METHODS
  if (unlikely(PyMethod_Check(__pyx_t_1))) {
    __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_1);
    assert(__pyx_t_3);
    PyObject* __pyx__function = PyMethod_GET_FUNCTION(__pyx_t_1);
    __Pyx_INCREF(__pyx_t_3);
    __Pyx_INCREF(__pyx__function);
    __Pyx_DECREF_SET(__pyx_t_1, __pyx__function);
    __pyx_t_5 = 0;
  }
  #endif
  {
    PyObject *__pyx_callargs[13] = {__pyx_t_3, __pyx_v_handle, __pyx_v_jobz, __pyx_v_uplo, __pyx_v_m, __pyx_t_9, __pyx_v_lda, __pyx_t_8, __pyx_t_11, __pyx_v_work_size, __pyx_t_12, __pyx_v_params, __pyx_v_batch_size};
    __pyx_t_2 = __Pyx_PyObject_FastCall((PyObject*)__pyx_t_1, __pyx_callargs+__pyx_t_5, (13-__pyx_t_5) | (__pyx_t_5*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
    __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
    __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
    __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
    __Pyx_DECREF(__pyx_t_11); __pyx_t_11 = 0;
    __Pyx_DECREF(__pyx_t_12); __pyx_t_12 = 0;
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 619, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
  }
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "cupyx/cusolver.pyx":623
 *         w.data.ptr, work.data.ptr, work_size, dev_info.data.ptr, params,
 *         batch_size)
 *     _cupy.linalg._util._check_cusolver_dev_info_if_synchronization_allowed(             # <<<<<<<<<<<<<<
 *         syevjBatched, dev_info)
 * 
*/
  __Pyx_GetModuleGlobalName(__pyx_t_12, __pyx_mstate_global->__pyx_n_u_cupy); if (unlikely(!__pyx_t_12)) __PYX_ERR(0, 623, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_12);
  __pyx_t_11 = __Pyx_PyObject_GetAttrStr(__pyx_t_12, __pyx_mstate_global->__pyx_n_u_linalg); if (unlikely(!__pyx_t_11)) __PYX_ERR(0, 623, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_11);
  __Pyx_DECREF(__pyx_t_12); __pyx_t_12 = 0;
  __pyx_t_12 = __Pyx_PyObject_GetAttrStr(__pyx_t_11, __pyx_mstate_global->__pyx_n_u_util); if (unlikely(!__pyx_t_12)) __PYX_ERR(0, 623, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_12);
  __Pyx_DECREF(__pyx_t_11); __pyx_t_11 = 0;
  __pyx_t_1 = __pyx_t_12;
  __Pyx_INCREF(__pyx_t_1);

  /* "cupyx/cusolver.pyx":624
 *         batch_size)
 *     _cupy.linalg._util._check_cusolver_dev_info_if_synchronization_allowed(
 *         syevjBatched, dev_info)             # <<<<<<<<<<<<<<
 * 
 *     _cusolver.destroySyevjInfo(params)
*/
  __pyx_t_5 = 0;
  {
    PyObject *__pyx_callargs[3] = {__pyx_t_1, __pyx_v_syevjBatched, __pyx_v_dev_info};
    __pyx_t_2 = __Pyx_PyObject_FastCallMethod((PyObject*)__pyx_mstate_global->__pyx_n_u_check_cusolver_dev_info_if_sync, __pyx_callargs+__pyx_t_5, (3-__pyx_t_5) | (1*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
    __Pyx_XDECREF(__pyx_t_1); __pyx_t_1 = 0;
    __Pyx_DECREF(__pyx_t_12); __pyx_t_12 = 0;
    if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 623, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
  }
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "cupyx/cusolver.pyx":626
 *         syevjBatched, dev_info)
 * 
 *     _cusolver.destroySyevjInfo(params)             # <<<<<<<<<<<<<<
 * 
 *     w = w.astype(w_dtype, copy=False)
*/
  __pyx_t_12 = NULL;
  __Pyx_GetModuleGlobalName(__pyx_t_1, __pyx_mstate_global->__pyx_n_u_cusolver); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 626, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_11 = __Pyx_PyObject_GetAttrStr(__pyx_t_1, __pyx_mstate_global->__pyx_n_u_destroySyevjInfo); if (unlikely(!__pyx_t_11)) __PYX_ERR(0, 626, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_11);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_5 = 1;
  #if CYTHON_UNPACK_METHODS
  if (unlikely(PyMethod_Check(__pyx_t_11))) {
    __pyx_t_12 = PyMethod_GET_SELF(__pyx_t_11);
    assert(__pyx_t_12);
    PyObject* __pyx__function = PyMethod_GET_FUNCTION(__pyx_t_11);
    __Pyx_INCREF(__pyx_t_12);
    __Pyx_INCREF(__pyx__function);
    __Pyx_DECREF_SET(__pyx_t_11, __pyx__function);
    __pyx_t_5 = 0;
  }
  #endif
  {
    PyObject *__pyx_callargs[2] = {__pyx_t_12, __pyx_v_params};
    __pyx_t_2 = __Pyx_PyObject_FastCall((PyObject*)__pyx_t_11, __pyx_callargs+__pyx_t_5, (2-__pyx_t_5) | (__pyx_t_5*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
    __Pyx_XDECREF(__pyx_t_12); __pyx_t_12 = 0;
    __Pyx_DECREF(__pyx_t_11); __pyx_t_11 = 0;
    if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 626, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
  }
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "cupyx/cusolver.pyx":628
 *     _cusolver.destroySyevjInfo(params)
 * 
 *     w = w.astype(w_dtype, copy=False)             # <<<<<<<<<<<<<<
 *     w = w.swapaxes(-2, -1).reshape(*batch_shape, m)
 *     if not with_eigen_vector:
*/
  __pyx_t_11 = __pyx_v_w;
  __Pyx_INCREF(__pyx_t_11);
  __pyx_t_5 = 0;
  {
    PyObject *__pyx_callargs[2 + ((CYTHON_VECTORCALL) ? 1 : 0)] = {__pyx_t_11, __pyx_v_w_dtype};
    __pyx_t_12 = __Pyx_MakeVectorcallBuilderKwds(1); if (unlikely(!__pyx_t_12)) __PYX_ERR(0, 628, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_12);
    if (__Pyx_VectorcallBuilder_AddArg(__pyx_mstate_global->__pyx_n_u_copy, Py_False, __pyx_t_12, __pyx_callargs+2, 0) < (0)) __PYX_ERR(0, 628, __pyx_L1_error)
    __pyx_t_2 = __Pyx_Object_VectorcallMethod_CallFromBuilder((PyObject*)__pyx_mstate_global->__pyx_n_u_astype, __pyx_callargs+__pyx_t_5, (2-__pyx_t_5) | (1*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET), __pyx_t_12);
    __Pyx_XDECREF(__pyx_t_11); __pyx_t_11 = 0;
    __Pyx_DECREF(__pyx_t_12); __pyx_t_12 = 0;
    if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 628, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
  }
  __Pyx_DECREF_SET(__pyx_v_w, __pyx_t_2);
  __pyx_t_2 = 0;

  /* "cupyx/cusolver.pyx":629
 * 
 *     w = w.astype(w_dtype, copy=False)
 *     w = w.swapaxes(-2, -1).reshape(*batch_shape, m)             # <<<<<<<<<<<<<<
 *     if not with_eigen_vector:
 *         return w
*/
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_v_w, __pyx_mstate_global->__pyx_n_u_swapaxes); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 629, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_12 = __Pyx_PyObject_Call(__pyx_t_2, __pyx_mstate_global->__pyx_tuple[0], NULL); if (unlikely(!__pyx_t_12)) __PYX_ERR(0, 629, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_12);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_t_12, __pyx_mstate_global->__pyx_n_u_reshape); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 629, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_12); __pyx_t_12 = 0;
  __pyx_t_12 = PySequence_Tuple(__pyx_v_batch_shape); if (unlikely(!__pyx_t_12)) __PYX_ERR(0, 629, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_12);
  __pyx_t_11 = PyTuple_New(1); if (unlikely(!__pyx_t_11)) __PYX_ERR(0, 629, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_11);
  __Pyx_INCREF(__pyx_v_m);
  __Pyx_GIVEREF(__pyx_v_m);
  if (__Pyx_PyTuple_SET_ITEM(__pyx_t_11, 0, __pyx_v_m) != (0)) __PYX_ERR(0, 629, __pyx_L1_error);
  __pyx_t_1 = PyNumber_Add(__pyx_t_12, __pyx_t_11); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 629, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_12); __pyx_t_12 = 0;
  __Pyx_DECREF(__pyx_t_11); __pyx_t_11 = 0;
  __pyx_t_11 = __Pyx_PyObject_Call(__pyx_t_2, __pyx_t_1, NULL); if (unlikely(!__pyx_t_11)) __PYX_ERR(0, 629, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_11);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_DECREF_SET(__pyx_v_w, __pyx_t_11);
  __pyx_t_11 = 0;

  /* "cupyx/cusolver.pyx":630
 *     w = w.astype(w_dtype, copy=False)
 *     w = w.swapaxes(-2, -1).reshape(*batch_shape, m)
 *     if not with_eigen_vector:             # <<<<<<<<<<<<<<
 *         return w
 *     v = v.astype(v_dtype, copy=False)
*/
  __pyx_t_10 = __Pyx_PyObject_IsTrue(__pyx_v_with_eigen_vector); if (unlikely((__pyx_t_10 < 0))) __PYX_ERR(0, 630, __pyx_L1_error)
  __pyx_t_13 = (!__pyx_t_10);
  if (__pyx_t_13) {

    /* "cupyx/cusolver.pyx":631
 *     w = w.swapaxes(-2, -1).reshape(*batch_shape, m)
 *     if not with_eigen_vector:
 *         return w             # <<<<<<<<<<<<<<
 *     v = v.astype(v_dtype, copy=False)
 *     v = v.swapaxes(-2, -1).reshape(*batch_shape, m, m)
*/
    __Pyx_XDECREF(__pyx_r);
    __Pyx_INCREF(__pyx_v_w);
    __pyx_r = __pyx_v_w;
    goto __pyx_L0;

    /* "cupyx/cusolver.pyx":630
 *     w = w.astype(w_dtype, copy=False)
 *     w = w.swapaxes(-2, -1).reshape(*batch_shape, m)
 *     if not with_eigen_vector:             # <<<<<<<<<<<<<<
 *         return w
 *     v = v.astype(v_dtype, copy=False)
*/
  }

  /* "cupyx/cusolver.pyx":632
 *     if not with_eigen_vector:
 *         return w
 *     v = v.astype(v_dtype, copy=False)             # <<<<<<<<<<<<<<
 *     v = v.swapaxes(-2, -1).reshape(*batch_shape, m, m)
 *     return w, v
*/
  __pyx_t_1 = __pyx_v_v;
  __Pyx_INCREF(__pyx_t_1);
  __pyx_t_5 = 0;
  {
    PyObject *__pyx_callargs[2 + ((CYTHON_VECTORCALL) ? 1 : 0)] = {__pyx_t_1, __pyx_v_v_dtype};
    __pyx_t_2 = __Pyx_MakeVectorcallBuilderKwds(1); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 632, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    if (__Pyx_VectorcallBuilder_AddArg(__pyx_mstate_global->__pyx_n_u_copy, Py_False, __pyx_t_2, __pyx_callargs+2, 0) < (0)) __PYX_ERR(0, 632, __pyx_L1_error)
    __pyx_t_11 = __Pyx_Object_VectorcallMethod_CallFromBuilder((PyObject*)__pyx_mstate_global->__pyx_n_u_astype, __pyx_callargs+__pyx_t_5, (2-__pyx_t_5) | (1*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET), __pyx_t_2);
    __Pyx_XDECREF(__pyx_t_1); __pyx_t_1 = 0;
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    if (unlikely(!__pyx_t_11)) __PYX_ERR(0, 632, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_11);
  }
  __Pyx_DECREF_SET(__pyx_v_v, __pyx_t_11);
  __pyx_t_11 = 0;

  /* "cupyx/cusolver.pyx":633
 *         return w
 *     v = v.astype(v_dtype, copy=False)
 *     v = v.swapaxes(-2, -1).reshape(*batch_shape, m, m)             # <<<<<<<<<<<<<<
 *     return w, v
 * 
*/
  __pyx_t_11 = __Pyx_PyObject_GetAttrStr(__pyx_v_v, __pyx_mstate_global->__pyx_n_u_swapaxes); if (unlikely(!__pyx_t_11)) __PYX_ERR(0, 633, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_11);
  __pyx_t_2 = __Pyx_PyObject_Call(__pyx_t_11, __pyx_mstate_global->__pyx_tuple[0], NULL); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 633, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_11); __pyx_t_11 = 0;
  __pyx_t_11 = __Pyx_PyObject_GetAttrStr(__pyx_t_2, __pyx_mstate_global->__pyx_n_u_reshape); if (unlikely(!__pyx_t_11)) __PYX_ERR(0, 633, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_11);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_2 = PySequence_Tuple(__pyx_v_batch_shape); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 633, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_1 = PyTuple_New(2); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 633, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_INCREF(__pyx_v_m);
  __Pyx_GIVEREF(__pyx_v_m);
  if (__Pyx_PyTuple_SET_ITEM(__pyx_t_1, 0, __pyx_v_m) != (0)) __PYX_ERR(0, 633, __pyx_L1_error);
  __Pyx_INCREF(__pyx_v_m);
  __Pyx_GIVEREF(__pyx_v_m);
  if (__Pyx_PyTuple_SET_ITEM(__pyx_t_1, 1, __pyx_v_m) != (0)) __PYX_ERR(0, 633, __pyx_L1_error);
  __pyx_t_12 = PyNumber_Add(__pyx_t_2, __pyx_t_1); if (unlikely(!__pyx_t_12)) __PYX_ERR(0, 633, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_12);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_11, __pyx_t_12, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 633, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_11); __pyx_t_11 = 0;
  __Pyx_DECREF(__pyx_t_12); __pyx_t_12 = 0;
  __Pyx_DECREF_SET(__pyx_v_v, __pyx_t_1);
  __pyx_t_1 = 0;

  /* "cupyx/cusolver.pyx":634
 *     v = v.astype(v_dtype, copy=False)
 *     v = v.swapaxes(-2, -1).reshape(*batch_shape, m, m)
 *     return w, v             # <<<<<<<<<<<<<<
 * 
 * 
*/
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = PyTuple_New(2); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 634, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_INCREF(__pyx_v_w);
  __Pyx_GIVEREF(__pyx_v_w);
  if (__Pyx_PyTuple_SET_ITEM(__pyx_t_1, 0, __pyx_v_w) != (0)) __PYX_ERR(0, 634, __pyx_L1_error);
  __Pyx_INCREF(__pyx_v_v);
  __Pyx_GIVEREF(__pyx_v_v);
  if (__Pyx_PyTuple_SET_ITEM(__pyx_t_1, 1, __pyx_v_v) != (0)) __PYX_ERR(0, 634, __pyx_L1_error);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "cupyx/cusolver.pyx":566
 * 
 * 
 * def _syevj_batched(a, UPLO, with_eigen_vector):             # <<<<<<<<<<<<<<
 *     # reject_float16=False for backward compatibility
 *     dtype, v_dtype = _cupy.linalg._util.linalg_common_type(
*/

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_8);
  __Pyx_XDECREF(__pyx_t_9);
  __Pyx_XDECREF(__pyx_t_11);
  __Pyx_XDECREF(__pyx_t_12);
  __Pyx_AddTraceback("cupyx.cusolver._syevj_batched", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_dtype);
  __Pyx_XDECREF(__pyx_v_v_dtype);
  __Pyx_XDECREF(__pyx_v_real_dtype);
  __Pyx_XDECREF(__pyx_v_w_dtype);
  __Pyx_XDECREF(__pyx_v_batch_shape);
  __Pyx_XDECREF(__pyx_v_m);
  __Pyx_XDECREF(__pyx_v_lda);
  __Pyx_XDECREF(__pyx_v_batch_size);
  __Pyx_XDECREF(__pyx_v_v);
  __Pyx_XDECREF(__pyx_v_vp);
  __Pyx_XDECREF(__pyx_v_w);
  __Pyx_XDECREF(__pyx_v_dev_info);
  __Pyx_XDECREF(__pyx_v_handle);
  __Pyx_XDECREF(__pyx_v_jobz);
  __Pyx_XDECREF(__pyx_v_uplo);
  __Pyx_XDECREF(__pyx_v_buffer_size);
  __Pyx_XDECREF(__pyx_v_syevjBatched);
  __Pyx_XDECREF(__pyx_v_params);
  __Pyx_XDECREF(__pyx_v_work_size);
  __Pyx_XDECREF(__pyx_v_work);
  __Pyx_XDECREF(__pyx_v_a);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "cupyx/cusolver.pyx":637
 * 
 * 
 * def gesv(a, b):             # <<<<<<<<<<<<<<
 *     """Solve a linear matrix equation using cusolverDn<t1><t2>gesv().
 *     Computes the solution to a system of linear equation ``ax = b``.
*/

/* Python wrapper */
static PyObject *__pyx_pw_5cupyx_8cusolver_15gesv(PyObject *__pyx_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
PyDoc_STRVAR(__pyx_doc_5cupyx_8cusolver_14gesv, "gesv(a, b)\n\nSolve a linear matrix equation using cusolverDn<t1><t2>gesv().\nComputes the solution to a system of linear equation ``ax = b``.\nArgs:\n    a (cupy.ndarray): The matrix with dimension ``(M, M)``.\n    b (cupy.ndarray): The matrix with dimension ``(M)`` or ``(M, K)``.\nReturns:\n    cupy.ndarray:\n        The matrix with dimension ``(M)`` or ``(M, K)``.");
static PyMethodDef __pyx_mdef_5cupyx_8cusolver_15gesv = {"gesv", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_5cupyx_8cusolver_15gesv, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_5cupyx_8cusolver_14gesv};
static PyObject *__pyx_pw_5cupyx_8cusolver_15gesv(PyObject *__pyx_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
) {
  PyObject *__pyx_v_a = 0;
  PyObject *__pyx_v_b = 0;
  #if !CYTHON_METH_FASTCALL
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  #endif
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject* values[2] = {0,0};
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("gesv (wrapper)", 0);
  #if !CYTHON_METH_FASTCALL
  #if CYTHON_ASSUME_SAFE_SIZE
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return NULL;
  #endif
  #endif
  __pyx_kwvalues = __Pyx_KwValues_FASTCALL(__pyx_args, __pyx_nargs);
  {
    PyObject ** const __pyx_pyargnames[] = {&__pyx_mstate_global->__pyx_n_u_a,&__pyx_mstate_global->__pyx_n_u_b,0};
    const Py_ssize_t __pyx_kwds_len = (__pyx_kwds) ? __Pyx_NumKwargs_FASTCALL(__pyx_kwds) : 0;
    if (unlikely(__pyx_kwds_len < 0)) __PYX_ERR(0, 637, __pyx_L3_error)
    if (__pyx_kwds_len > 0) {
      switch (__pyx_nargs) {
        case  2:
        values[1] = __Pyx_ArgRef_FASTCALL(__pyx_args, 1);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[1])) __PYX_ERR(0, 637, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  1:
        values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(0, 637, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      const Py_ssize_t kwd_pos_args = __pyx_nargs;
      if (__Pyx_ParseKeywords(__pyx_kwds, __pyx_kwvalues, __pyx_pyargnames, 0, values, kwd_pos_args, __pyx_kwds_len, "gesv", 0) < (0)) __PYX_ERR(0, 637, __pyx_L3_error)
      for (Py_ssize_t i = __pyx_nargs; i < 2; i++) {
        if (unlikely(!values[i])) { __Pyx_RaiseArgtupleInvalid("gesv", 1, 2, 2, i); __PYX_ERR(0, 637, __pyx_L3_error) }
      }
    } else if (unlikely(__pyx_nargs != 2)) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
      if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(0, 637, __pyx_L3_error)
      values[1] = __Pyx_ArgRef_FASTCALL(__pyx_args, 1);
      if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[1])) __PYX_ERR(0, 637, __pyx_L3_error)
    }
    __pyx_v_a = values[0];
    __pyx_v_b = values[1];
  }
  goto __pyx_L6_skip;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("gesv", 1, 2, 2, __pyx_nargs); __PYX_ERR(0, 637, __pyx_L3_error)
  __pyx_L6_skip:;
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_AddTraceback("cupyx.cusolver.gesv", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_5cupyx_8cusolver_14gesv(__pyx_self, __pyx_v_a, __pyx_v_b);

  /* function exit code */
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5cupyx_8cusolver_14gesv(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_a, PyObject *__pyx_v_b) {
  PyObject *__pyx_v_n = NULL;
  PyObject *__pyx_v_nrhs = NULL;
  PyObject *__pyx_v_compute_type = NULL;
  PyObject *__pyx_v_t1 = NULL;
  PyObject *__pyx_v_t2 = NULL;
  PyObject *__pyx_v_solver_name = NULL;
  PyObject *__pyx_v_solver = NULL;
  PyObject *__pyx_v_helper = NULL;
  PyObject *__pyx_v_x = NULL;
  PyObject *__pyx_v_dipiv = NULL;
  PyObject *__pyx_v_dinfo = NULL;
  PyObject *__pyx_v_handle = NULL;
  PyObject *__pyx_v_lwork = NULL;
  PyObject *__pyx_v_dwork = NULL;
  PyObject *__pyx_v_niters = NULL;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  size_t __pyx_t_4;
  int __pyx_t_5;
  int __pyx_t_6;
  PyObject *__pyx_t_7 = NULL;
  PyObject *__pyx_t_8 = NULL;
  PyObject *__pyx_t_9 = NULL;
  PyObject *(*__pyx_t_10)(PyObject *);
  PyObject *__pyx_t_11 = NULL;
  PyObject *__pyx_t_12 = NULL;
  PyObject *__pyx_t_13 = NULL;
  PyObject *__pyx_t_14 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("gesv", 0);
  __Pyx_INCREF(__pyx_v_a);
  __Pyx_INCREF(__pyx_v_b);

  /* "cupyx/cusolver.pyx":647
 *             The matrix with dimension ``(M)`` or ``(M, K)``.
 *     """
 *     if not check_availability('gesv'):             # <<<<<<<<<<<<<<
 *         raise RuntimeError('gesv is not available.')
 * 
*/
  __pyx_t_2 = NULL;
  __Pyx_GetModuleGlobalName(__pyx_t_3, __pyx_mstate_global->__pyx_n_u_check_availability); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 647, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_4 = 1;
  #if CYTHON_UNPACK_METHODS
  if (unlikely(PyMethod_Check(__pyx_t_3))) {
    __pyx_t_2 = PyMethod_GET_SELF(__pyx_t_3);
    assert(__pyx_t_2);
    PyObject* __pyx__function = PyMethod_GET_FUNCTION(__pyx_t_3);
    __Pyx_INCREF(__pyx_t_2);
    __Pyx_INCREF(__pyx__function);
    __Pyx_DECREF_SET(__pyx_t_3, __pyx__function);
    __pyx_t_4 = 0;
  }
  #endif
  {
    PyObject *__pyx_callargs[2] = {__pyx_t_2, __pyx_mstate_global->__pyx_n_u_gesv};
    __pyx_t_1 = __Pyx_PyObject_FastCall((PyObject*)__pyx_t_3, __pyx_callargs+__pyx_t_4, (2-__pyx_t_4) | (__pyx_t_4*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
    __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 647, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
  }
  __pyx_t_5 = __Pyx_PyObject_IsTrue(__pyx_t_1); if (unlikely((__pyx_t_5 < 0))) __PYX_ERR(0, 647, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_6 = (!__pyx_t_5);
  if (unlikely(__pyx_t_6)) {

    /* "cupyx/cusolver.pyx":648
 *     """
 *     if not check_availability('gesv'):
 *         raise RuntimeError('gesv is not available.')             # <<<<<<<<<<<<<<
 * 
 *     if a.ndim != 2:
*/
    __pyx_t_3 = NULL;
    __pyx_t_4 = 1;
    {
      PyObject *__pyx_callargs[2] = {__pyx_t_3, __pyx_mstate_global->__pyx_kp_u_gesv_is_not_available};
      __pyx_t_1 = __Pyx_PyObject_FastCall((PyObject*)(((PyTypeObject*)PyExc_RuntimeError)), __pyx_callargs+__pyx_t_4, (2-__pyx_t_4) | (__pyx_t_4*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
      __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
      if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 648, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
    }
    __Pyx_Raise(__pyx_t_1, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __PYX_ERR(0, 648, __pyx_L1_error)

    /* "cupyx/cusolver.pyx":647
 *             The matrix with dimension ``(M)`` or ``(M, K)``.
 *     """
 *     if not check_availability('gesv'):             # <<<<<<<<<<<<<<
 *         raise RuntimeError('gesv is not available.')
 * 
*/
  }

  /* "cupyx/cusolver.pyx":650
 *         raise RuntimeError('gesv is not available.')
 * 
 *     if a.ndim != 2:             # <<<<<<<<<<<<<<
 *         raise ValueError('a.ndim must be 2 (actual:{})'.format(a.ndim))
 *     if b.ndim not in (1, 2):
*/
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_v_a, __pyx_mstate_global->__pyx_n_u_ndim); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 650, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_6 = (__Pyx_PyLong_BoolNeObjC(__pyx_t_1, __pyx_mstate_global->__pyx_int_2, 2, 0)); if (unlikely((__pyx_t_6 < 0))) __PYX_ERR(0, 650, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  if (unlikely(__pyx_t_6)) {

    /* "cupyx/cusolver.pyx":651
 * 
 *     if a.ndim != 2:
 *         raise ValueError('a.ndim must be 2 (actual:{})'.format(a.ndim))             # <<<<<<<<<<<<<<
 *     if b.ndim not in (1, 2):
 *         raise ValueError('b.ndim must be 1 or 2 (actual:{})'.format(b.ndim))
*/
    __pyx_t_3 = NULL;
    __pyx_t_7 = __pyx_mstate_global->__pyx_kp_u_a_ndim_must_be_2_actual;
    __Pyx_INCREF(__pyx_t_7);
    __pyx_t_8 = __Pyx_PyObject_GetAttrStr(__pyx_v_a, __pyx_mstate_global->__pyx_n_u_ndim); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 651, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_8);
    __pyx_t_4 = 0;
    {
      PyObject *__pyx_callargs[2] = {__pyx_t_7, __pyx_t_8};
      __pyx_t_2 = __Pyx_PyObject_FastCallMethod((PyObject*)__pyx_mstate_global->__pyx_n_u_format, __pyx_callargs+__pyx_t_4, (2-__pyx_t_4) | (1*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
      __Pyx_XDECREF(__pyx_t_7); __pyx_t_7 = 0;
      __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
      if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 651, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_2);
    }
    __pyx_t_4 = 1;
    {
      PyObject *__pyx_callargs[2] = {__pyx_t_3, __pyx_t_2};
      __pyx_t_1 = __Pyx_PyObject_FastCall((PyObject*)(((PyTypeObject*)PyExc_ValueError)), __pyx_callargs+__pyx_t_4, (2-__pyx_t_4) | (__pyx_t_4*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
      __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
      __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
      if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 651, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
    }
    __Pyx_Raise(__pyx_t_1, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __PYX_ERR(0, 651, __pyx_L1_error)

    /* "cupyx/cusolver.pyx":650
 *         raise RuntimeError('gesv is not available.')
 * 
 *     if a.ndim != 2:             # <<<<<<<<<<<<<<
 *         raise ValueError('a.ndim must be 2 (actual:{})'.format(a.ndim))
 *     if b.ndim not in (1, 2):
*/
  }

  /* "cupyx/cusolver.pyx":652
 *     if a.ndim != 2:
 *         raise ValueError('a.ndim must be 2 (actual:{})'.format(a.ndim))
 *     if b.ndim not in (1, 2):             # <<<<<<<<<<<<<<
 *         raise ValueError('b.ndim must be 1 or 2 (actual:{})'.format(b.ndim))
 *     if a.shape[0] != a.shape[1]:
*/
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_v_b, __pyx_mstate_global->__pyx_n_u_ndim); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 652, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_5 = (__Pyx_PyLong_BoolNeObjC(__pyx_t_1, __pyx_mstate_global->__pyx_int_1, 1, 0)); if (unlikely((__pyx_t_5 < 0))) __PYX_ERR(0, 652, __pyx_L1_error)
  if (__pyx_t_5) {
  } else {
    __pyx_t_6 = __pyx_t_5;
    goto __pyx_L6_bool_binop_done;
  }
  __pyx_t_5 = (__Pyx_PyLong_BoolNeObjC(__pyx_t_1, __pyx_mstate_global->__pyx_int_2, 2, 0)); if (unlikely((__pyx_t_5 < 0))) __PYX_ERR(0, 652, __pyx_L1_error)
  __pyx_t_6 = __pyx_t_5;
  __pyx_L6_bool_binop_done:;
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_5 = __pyx_t_6;
  if (unlikely(__pyx_t_5)) {

    /* "cupyx/cusolver.pyx":653
 *         raise ValueError('a.ndim must be 2 (actual:{})'.format(a.ndim))
 *     if b.ndim not in (1, 2):
 *         raise ValueError('b.ndim must be 1 or 2 (actual:{})'.format(b.ndim))             # <<<<<<<<<<<<<<
 *     if a.shape[0] != a.shape[1]:
 *         raise ValueError('a must be a square matrix.')
*/
    __pyx_t_2 = NULL;
    __pyx_t_8 = __pyx_mstate_global->__pyx_kp_u_b_ndim_must_be_1_or_2_actual;
    __Pyx_INCREF(__pyx_t_8);
    __pyx_t_7 = __Pyx_PyObject_GetAttrStr(__pyx_v_b, __pyx_mstate_global->__pyx_n_u_ndim); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 653, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_7);
    __pyx_t_4 = 0;
    {
      PyObject *__pyx_callargs[2] = {__pyx_t_8, __pyx_t_7};
      __pyx_t_3 = __Pyx_PyObject_FastCallMethod((PyObject*)__pyx_mstate_global->__pyx_n_u_format, __pyx_callargs+__pyx_t_4, (2-__pyx_t_4) | (1*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
      __Pyx_XDECREF(__pyx_t_8); __pyx_t_8 = 0;
      __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
      if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 653, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_3);
    }
    __pyx_t_4 = 1;
    {
      PyObject *__pyx_callargs[2] = {__pyx_t_2, __pyx_t_3};
      __pyx_t_1 = __Pyx_PyObject_FastCall((PyObject*)(((PyTypeObject*)PyExc_ValueError)), __pyx_callargs+__pyx_t_4, (2-__pyx_t_4) | (__pyx_t_4*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
      __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
      if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 653, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
    }
    __Pyx_Raise(__pyx_t_1, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __PYX_ERR(0, 653, __pyx_L1_error)

    /* "cupyx/cusolver.pyx":652
 *     if a.ndim != 2:
 *         raise ValueError('a.ndim must be 2 (actual:{})'.format(a.ndim))
 *     if b.ndim not in (1, 2):             # <<<<<<<<<<<<<<
 *         raise ValueError('b.ndim must be 1 or 2 (actual:{})'.format(b.ndim))
 *     if a.shape[0] != a.shape[1]:
*/
  }

  /* "cupyx/cusolver.pyx":654
 *     if b.ndim not in (1, 2):
 *         raise ValueError('b.ndim must be 1 or 2 (actual:{})'.format(b.ndim))
 *     if a.shape[0] != a.shape[1]:             # <<<<<<<<<<<<<<
 *         raise ValueError('a must be a square matrix.')
 *     if a.shape[0] != b.shape[0]:
*/
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_v_a, __pyx_mstate_global->__pyx_n_u_shape); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 654, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_3 = __Pyx_GetItemInt(__pyx_t_1, 0, long, 1, __Pyx_PyLong_From_long, 0, 0, 1, 1, __Pyx_ReferenceSharing_OwnStrongReference); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 654, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_v_a, __pyx_mstate_global->__pyx_n_u_shape); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 654, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_GetItemInt(__pyx_t_1, 1, long, 1, __Pyx_PyLong_From_long, 0, 0, 1, 1, __Pyx_ReferenceSharing_OwnStrongReference); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 654, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_1 = PyObject_RichCompare(__pyx_t_3, __pyx_t_2, Py_NE); __Pyx_XGOTREF(__pyx_t_1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 654, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_5 = __Pyx_PyObject_IsTrue(__pyx_t_1); if (unlikely((__pyx_t_5 < 0))) __PYX_ERR(0, 654, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  if (unlikely(__pyx_t_5)) {

    /* "cupyx/cusolver.pyx":655
 *         raise ValueError('b.ndim must be 1 or 2 (actual:{})'.format(b.ndim))
 *     if a.shape[0] != a.shape[1]:
 *         raise ValueError('a must be a square matrix.')             # <<<<<<<<<<<<<<
 *     if a.shape[0] != b.shape[0]:
 *         raise ValueError('shape mismatch (a:{}, b:{}).'.
*/
    __pyx_t_2 = NULL;
    __pyx_t_4 = 1;
    {
      PyObject *__pyx_callargs[2] = {__pyx_t_2, __pyx_mstate_global->__pyx_kp_u_a_must_be_a_square_matrix};
      __pyx_t_1 = __Pyx_PyObject_FastCall((PyObject*)(((PyTypeObject*)PyExc_ValueError)), __pyx_callargs+__pyx_t_4, (2-__pyx_t_4) | (__pyx_t_4*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
      __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
      if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 655, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
    }
    __Pyx_Raise(__pyx_t_1, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __PYX_ERR(0, 655, __pyx_L1_error)

    /* "cupyx/cusolver.pyx":654
 *     if b.ndim not in (1, 2):
 *         raise ValueError('b.ndim must be 1 or 2 (actual:{})'.format(b.ndim))
 *     if a.shape[0] != a.shape[1]:             # <<<<<<<<<<<<<<
 *         raise ValueError('a must be a square matrix.')
 *     if a.shape[0] != b.shape[0]:
*/
  }

  /* "cupyx/cusolver.pyx":656
 *     if a.shape[0] != a.shape[1]:
 *         raise ValueError('a must be a square matrix.')
 *     if a.shape[0] != b.shape[0]:             # <<<<<<<<<<<<<<
 *         raise ValueError('shape mismatch (a:{}, b:{}).'.
 *                          format(a.shape, b.shape))
*/
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_v_a, __pyx_mstate_global->__pyx_n_u_shape); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 656, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_GetItemInt(__pyx_t_1, 0, long, 1, __Pyx_PyLong_From_long, 0, 0, 1, 1, __Pyx_ReferenceSharing_OwnStrongReference); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 656, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_v_b, __pyx_mstate_global->__pyx_n_u_shape); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 656, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_3 = __Pyx_GetItemInt(__pyx_t_1, 0, long, 1, __Pyx_PyLong_From_long, 0, 0, 1, 1, __Pyx_ReferenceSharing_OwnStrongReference); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 656, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_1 = PyObject_RichCompare(__pyx_t_2, __pyx_t_3, Py_NE); __Pyx_XGOTREF(__pyx_t_1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 656, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_t_5 = __Pyx_PyObject_IsTrue(__pyx_t_1); if (unlikely((__pyx_t_5 < 0))) __PYX_ERR(0, 656, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  if (unlikely(__pyx_t_5)) {

    /* "cupyx/cusolver.pyx":657
 *         raise ValueError('a must be a square matrix.')
 *     if a.shape[0] != b.shape[0]:
 *         raise ValueError('shape mismatch (a:{}, b:{}).'.             # <<<<<<<<<<<<<<
 *                          format(a.shape, b.shape))
 *     if a.dtype != b.dtype:
*/
    __pyx_t_3 = NULL;

    /* "cupyx/cusolver.pyx":658
 *     if a.shape[0] != b.shape[0]:
 *         raise ValueError('shape mismatch (a:{}, b:{}).'.
 *                          format(a.shape, b.shape))             # <<<<<<<<<<<<<<
 *     if a.dtype != b.dtype:
 *         raise ValueError('dtype mismatch (a:{}, b:{}).'.
*/
    __pyx_t_7 = __pyx_mstate_global->__pyx_kp_u_shape_mismatch_a_b;
    __Pyx_INCREF(__pyx_t_7);
    __pyx_t_8 = __Pyx_PyObject_GetAttrStr(__pyx_v_a, __pyx_mstate_global->__pyx_n_u_shape); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 658, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_8);
    __pyx_t_9 = __Pyx_PyObject_GetAttrStr(__pyx_v_b, __pyx_mstate_global->__pyx_n_u_shape); if (unlikely(!__pyx_t_9)) __PYX_ERR(0, 658, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_9);
    __pyx_t_4 = 0;
    {
      PyObject *__pyx_callargs[3] = {__pyx_t_7, __pyx_t_8, __pyx_t_9};
      __pyx_t_2 = __Pyx_PyObject_FastCallMethod((PyObject*)__pyx_mstate_global->__pyx_n_u_format, __pyx_callargs+__pyx_t_4, (3-__pyx_t_4) | (1*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
      __Pyx_XDECREF(__pyx_t_7); __pyx_t_7 = 0;
      __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
      __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
      if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 658, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_2);
    }
    __pyx_t_4 = 1;
    {
      PyObject *__pyx_callargs[2] = {__pyx_t_3, __pyx_t_2};
      __pyx_t_1 = __Pyx_PyObject_FastCall((PyObject*)(((PyTypeObject*)PyExc_ValueError)), __pyx_callargs+__pyx_t_4, (2-__pyx_t_4) | (__pyx_t_4*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
      __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
      __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
      if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 657, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
    }
    __Pyx_Raise(__pyx_t_1, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __PYX_ERR(0, 657, __pyx_L1_error)

    /* "cupyx/cusolver.pyx":656
 *     if a.shape[0] != a.shape[1]:
 *         raise ValueError('a must be a square matrix.')
 *     if a.shape[0] != b.shape[0]:             # <<<<<<<<<<<<<<
 *         raise ValueError('shape mismatch (a:{}, b:{}).'.
 *                          format(a.shape, b.shape))
*/
  }

  /* "cupyx/cusolver.pyx":659
 *         raise ValueError('shape mismatch (a:{}, b:{}).'.
 *                          format(a.shape, b.shape))
 *     if a.dtype != b.dtype:             # <<<<<<<<<<<<<<
 *         raise ValueError('dtype mismatch (a:{}, b:{}).'.
 *                          format(a.dtype, b.dtype))
*/
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_v_a, __pyx_mstate_global->__pyx_n_u_dtype); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 659, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_v_b, __pyx_mstate_global->__pyx_n_u_dtype); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 659, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = PyObject_RichCompare(__pyx_t_1, __pyx_t_2, Py_NE); __Pyx_XGOTREF(__pyx_t_3); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 659, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_5 = __Pyx_PyObject_IsTrue(__pyx_t_3); if (unlikely((__pyx_t_5 < 0))) __PYX_ERR(0, 659, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  if (unlikely(__pyx_t_5)) {

    /* "cupyx/cusolver.pyx":660
 *                          format(a.shape, b.shape))
 *     if a.dtype != b.dtype:
 *         raise ValueError('dtype mismatch (a:{}, b:{}).'.             # <<<<<<<<<<<<<<
 *                          format(a.dtype, b.dtype))
 * 
*/
    __pyx_t_2 = NULL;

    /* "cupyx/cusolver.pyx":661
 *     if a.dtype != b.dtype:
 *         raise ValueError('dtype mismatch (a:{}, b:{}).'.
 *                          format(a.dtype, b.dtype))             # <<<<<<<<<<<<<<
 * 
 *     if b.ndim == 2:
*/
    __pyx_t_9 = __pyx_mstate_global->__pyx_kp_u_dtype_mismatch_a_b;
    __Pyx_INCREF(__pyx_t_9);
    __pyx_t_8 = __Pyx_PyObject_GetAttrStr(__pyx_v_a, __pyx_mstate_global->__pyx_n_u_dtype); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 661, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_8);
    __pyx_t_7 = __Pyx_PyObject_GetAttrStr(__pyx_v_b, __pyx_mstate_global->__pyx_n_u_dtype); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 661, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_7);
    __pyx_t_4 = 0;
    {
      PyObject *__pyx_callargs[3] = {__pyx_t_9, __pyx_t_8, __pyx_t_7};
      __pyx_t_1 = __Pyx_PyObject_FastCallMethod((PyObject*)__pyx_mstate_global->__pyx_n_u_format, __pyx_callargs+__pyx_t_4, (3-__pyx_t_4) | (1*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
      __Pyx_XDECREF(__pyx_t_9); __pyx_t_9 = 0;
      __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
      __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
      if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 661, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
    }
    __pyx_t_4 = 1;
    {
      PyObject *__pyx_callargs[2] = {__pyx_t_2, __pyx_t_1};
      __pyx_t_3 = __Pyx_PyObject_FastCall((PyObject*)(((PyTypeObject*)PyExc_ValueError)), __pyx_callargs+__pyx_t_4, (2-__pyx_t_4) | (__pyx_t_4*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
      __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 660, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_3);
    }
    __Pyx_Raise(__pyx_t_3, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __PYX_ERR(0, 660, __pyx_L1_error)

    /* "cupyx/cusolver.pyx":659
 *         raise ValueError('shape mismatch (a:{}, b:{}).'.
 *                          format(a.shape, b.shape))
 *     if a.dtype != b.dtype:             # <<<<<<<<<<<<<<
 *         raise ValueError('dtype mismatch (a:{}, b:{}).'.
 *                          format(a.dtype, b.dtype))
*/
  }

  /* "cupyx/cusolver.pyx":663
 *                          format(a.dtype, b.dtype))
 * 
 *     if b.ndim == 2:             # <<<<<<<<<<<<<<
 *         n, nrhs = b.shape
 *     else:
*/
  __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_v_b, __pyx_mstate_global->__pyx_n_u_ndim); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 663, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_5 = (__Pyx_PyLong_BoolEqObjC(__pyx_t_3, __pyx_mstate_global->__pyx_int_2, 2, 0)); if (unlikely((__pyx_t_5 < 0))) __PYX_ERR(0, 663, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  if (__pyx_t_5) {

    /* "cupyx/cusolver.pyx":664
 * 
 *     if b.ndim == 2:
 *         n, nrhs = b.shape             # <<<<<<<<<<<<<<
 *     else:
 *         n, nrhs = b.shape[0], 1
*/
    __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_v_b, __pyx_mstate_global->__pyx_n_u_shape); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 664, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    if ((likely(PyTuple_CheckExact(__pyx_t_3))) || (PyList_CheckExact(__pyx_t_3))) {
      PyObject* sequence = __pyx_t_3;
      Py_ssize_t size = __Pyx_PySequence_SIZE(sequence);
      if (unlikely(size != 2)) {
        if (size > 2) __Pyx_RaiseTooManyValuesError(2);
        else if (size >= 0) __Pyx_RaiseNeedMoreValuesError(size);
        __PYX_ERR(0, 664, __pyx_L1_error)
      }
      #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
      if (likely(PyTuple_CheckExact(sequence))) {
        __pyx_t_1 = PyTuple_GET_ITEM(sequence, 0);
        __Pyx_INCREF(__pyx_t_1);
        __pyx_t_2 = PyTuple_GET_ITEM(sequence, 1);
        __Pyx_INCREF(__pyx_t_2);
      } else {
        __pyx_t_1 = __Pyx_PyList_GetItemRefFast(sequence, 0, __Pyx_ReferenceSharing_SharedReference);
        if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 664, __pyx_L1_error)
        __Pyx_XGOTREF(__pyx_t_1);
        __pyx_t_2 = __Pyx_PyList_GetItemRefFast(sequence, 1, __Pyx_ReferenceSharing_SharedReference);
        if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 664, __pyx_L1_error)
        __Pyx_XGOTREF(__pyx_t_2);
      }
      #else
      __pyx_t_1 = __Pyx_PySequence_ITEM(sequence, 0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 664, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      __pyx_t_2 = __Pyx_PySequence_ITEM(sequence, 1); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 664, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_2);
      #endif
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    } else {
      Py_ssize_t index = -1;
      __pyx_t_7 = PyObject_GetIter(__pyx_t_3); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 664, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_7);
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
      __pyx_t_10 = (CYTHON_COMPILING_IN_LIMITED_API) ? PyIter_Next : __Pyx_PyObject_GetIterNextFunc(__pyx_t_7);
      index = 0; __pyx_t_1 = __pyx_t_10(__pyx_t_7); if (unlikely(!__pyx_t_1)) goto __pyx_L12_unpacking_failed;
      __Pyx_GOTREF(__pyx_t_1);
      index = 1; __pyx_t_2 = __pyx_t_10(__pyx_t_7); if (unlikely(!__pyx_t_2)) goto __pyx_L12_unpacking_failed;
      __Pyx_GOTREF(__pyx_t_2);
      if (__Pyx_IternextUnpackEndCheck(__pyx_t_10(__pyx_t_7), 2) < (0)) __PYX_ERR(0, 664, __pyx_L1_error)
      __pyx_t_10 = NULL;
      __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
      goto __pyx_L13_unpacking_done;
      __pyx_L12_unpacking_failed:;
      __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
      __pyx_t_10 = NULL;
      if (__Pyx_IterFinish() == 0) __Pyx_RaiseNeedMoreValuesError(index);
      __PYX_ERR(0, 664, __pyx_L1_error)
      __pyx_L13_unpacking_done:;
    }
    __pyx_v_n = __pyx_t_1;
    __pyx_t_1 = 0;
    __pyx_v_nrhs = __pyx_t_2;
    __pyx_t_2 = 0;

    /* "cupyx/cusolver.pyx":663
 *                          format(a.dtype, b.dtype))
 * 
 *     if b.ndim == 2:             # <<<<<<<<<<<<<<
 *         n, nrhs = b.shape
 *     else:
*/
    goto __pyx_L11;
  }

  /* "cupyx/cusolver.pyx":666
 *         n, nrhs = b.shape
 *     else:
 *         n, nrhs = b.shape[0], 1             # <<<<<<<<<<<<<<
 * 
 *     compute_type = _linalg.get_compute_type(a.dtype)
*/
  /*else*/ {
    __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_v_b, __pyx_mstate_global->__pyx_n_u_shape); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 666, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_t_2 = __Pyx_GetItemInt(__pyx_t_3, 0, long, 1, __Pyx_PyLong_From_long, 0, 0, 1, 1, __Pyx_ReferenceSharing_OwnStrongReference); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 666, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __pyx_t_3 = __pyx_mstate_global->__pyx_int_1;
    __Pyx_INCREF(__pyx_t_3);
    __pyx_v_n = __pyx_t_2;
    __pyx_t_2 = 0;
    __pyx_v_nrhs = __pyx_t_3;
    __pyx_t_3 = 0;
  }
  __pyx_L11:;

  /* "cupyx/cusolver.pyx":668
 *         n, nrhs = b.shape[0], 1
 * 
 *     compute_type = _linalg.get_compute_type(a.dtype)             # <<<<<<<<<<<<<<
 *     if a.dtype.char in 'fd':
 *         if a.dtype.char == 'f':
*/
  __pyx_t_2 = NULL;
  __Pyx_GetModuleGlobalName(__pyx_t_1, __pyx_mstate_global->__pyx_n_u_linalg_2); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 668, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_7 = __Pyx_PyObject_GetAttrStr(__pyx_t_1, __pyx_mstate_global->__pyx_n_u_get_compute_type); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 668, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_7);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_v_a, __pyx_mstate_global->__pyx_n_u_dtype); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 668, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_4 = 1;
  #if CYTHON_UNPACK_METHODS
  if (unlikely(PyMethod_Check(__pyx_t_7))) {
    __pyx_t_2 = PyMethod_GET_SELF(__pyx_t_7);
    assert(__pyx_t_2);
    PyObject* __pyx__function = PyMethod_GET_FUNCTION(__pyx_t_7);
    __Pyx_INCREF(__pyx_t_2);
    __Pyx_INCREF(__pyx__function);
    __Pyx_DECREF_SET(__pyx_t_7, __pyx__function);
    __pyx_t_4 = 0;
  }
  #endif
  {
    PyObject *__pyx_callargs[2] = {__pyx_t_2, __pyx_t_1};
    __pyx_t_3 = __Pyx_PyObject_FastCall((PyObject*)__pyx_t_7, __pyx_callargs+__pyx_t_4, (2-__pyx_t_4) | (__pyx_t_4*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
    __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
    if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 668, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
  }
  __pyx_v_compute_type = __pyx_t_3;
  __pyx_t_3 = 0;

  /* "cupyx/cusolver.pyx":669
 * 
 *     compute_type = _linalg.get_compute_type(a.dtype)
 *     if a.dtype.char in 'fd':             # <<<<<<<<<<<<<<
 *         if a.dtype.char == 'f':
 *             t1 = t2 = 's'
*/
  __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_v_a, __pyx_mstate_global->__pyx_n_u_dtype); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 669, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_7 = __Pyx_PyObject_GetAttrStr(__pyx_t_3, __pyx_mstate_global->__pyx_n_u_char); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 669, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_7);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_t_5 = (__Pyx_PyUnicode_ContainsTF(__pyx_t_7, __pyx_mstate_global->__pyx_n_u_fd, Py_EQ)); if (unlikely((__pyx_t_5 < 0))) __PYX_ERR(0, 669, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
  if (__pyx_t_5) {

    /* "cupyx/cusolver.pyx":670
 *     compute_type = _linalg.get_compute_type(a.dtype)
 *     if a.dtype.char in 'fd':
 *         if a.dtype.char == 'f':             # <<<<<<<<<<<<<<
 *             t1 = t2 = 's'
 *         else:
*/
    __pyx_t_7 = __Pyx_PyObject_GetAttrStr(__pyx_v_a, __pyx_mstate_global->__pyx_n_u_dtype); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 670, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_7);
    __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_t_7, __pyx_mstate_global->__pyx_n_u_char); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 670, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
    __pyx_t_5 = (__Pyx_PyUnicode_Equals(__pyx_t_3, __pyx_mstate_global->__pyx_n_u_f, Py_EQ)); if (unlikely((__pyx_t_5 < 0))) __PYX_ERR(0, 670, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    if (__pyx_t_5) {

      /* "cupyx/cusolver.pyx":671
 *     if a.dtype.char in 'fd':
 *         if a.dtype.char == 'f':
 *             t1 = t2 = 's'             # <<<<<<<<<<<<<<
 *         else:
 *             t1 = t2 = 'd'
*/
      __Pyx_INCREF(__pyx_mstate_global->__pyx_n_u_s);
      __pyx_v_t1 = __pyx_mstate_global->__pyx_n_u_s;
      __Pyx_INCREF(__pyx_mstate_global->__pyx_n_u_s);
      __pyx_v_t2 = __pyx_mstate_global->__pyx_n_u_s;

      /* "cupyx/cusolver.pyx":670
 *     compute_type = _linalg.get_compute_type(a.dtype)
 *     if a.dtype.char in 'fd':
 *         if a.dtype.char == 'f':             # <<<<<<<<<<<<<<
 *             t1 = t2 = 's'
 *         else:
*/
      goto __pyx_L15;
    }

    /* "cupyx/cusolver.pyx":673
 *             t1 = t2 = 's'
 *         else:
 *             t1 = t2 = 'd'             # <<<<<<<<<<<<<<
 *         if compute_type == _linalg.COMPUTE_TYPE_FP16:
 *             t2 = 'h'
*/
    /*else*/ {
      __Pyx_INCREF(__pyx_mstate_global->__pyx_n_u_d);
      __pyx_v_t1 = __pyx_mstate_global->__pyx_n_u_d;
      __Pyx_INCREF(__pyx_mstate_global->__pyx_n_u_d);
      __pyx_v_t2 = __pyx_mstate_global->__pyx_n_u_d;
    }
    __pyx_L15:;

    /* "cupyx/cusolver.pyx":674
 *         else:
 *             t1 = t2 = 'd'
 *         if compute_type == _linalg.COMPUTE_TYPE_FP16:             # <<<<<<<<<<<<<<
 *             t2 = 'h'
 *         elif compute_type == _linalg.COMPUTE_TYPE_TF32:
*/
    __Pyx_GetModuleGlobalName(__pyx_t_3, __pyx_mstate_global->__pyx_n_u_linalg_2); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 674, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_t_7 = __Pyx_PyObject_GetAttrStr(__pyx_t_3, __pyx_mstate_global->__pyx_n_u_COMPUTE_TYPE_FP16); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 674, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_7);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __pyx_t_3 = PyObject_RichCompare(__pyx_v_compute_type, __pyx_t_7, Py_EQ); __Pyx_XGOTREF(__pyx_t_3); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 674, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
    __pyx_t_5 = __Pyx_PyObject_IsTrue(__pyx_t_3); if (unlikely((__pyx_t_5 < 0))) __PYX_ERR(0, 674, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    if (__pyx_t_5) {

      /* "cupyx/cusolver.pyx":675
 *             t1 = t2 = 'd'
 *         if compute_type == _linalg.COMPUTE_TYPE_FP16:
 *             t2 = 'h'             # <<<<<<<<<<<<<<
 *         elif compute_type == _linalg.COMPUTE_TYPE_TF32:
 *             t2 = 'x'
*/
      __Pyx_INCREF(__pyx_mstate_global->__pyx_n_u_h);
      __Pyx_DECREF_SET(__pyx_v_t2, __pyx_mstate_global->__pyx_n_u_h);

      /* "cupyx/cusolver.pyx":674
 *         else:
 *             t1 = t2 = 'd'
 *         if compute_type == _linalg.COMPUTE_TYPE_FP16:             # <<<<<<<<<<<<<<
 *             t2 = 'h'
 *         elif compute_type == _linalg.COMPUTE_TYPE_TF32:
*/
      goto __pyx_L16;
    }

    /* "cupyx/cusolver.pyx":676
 *         if compute_type == _linalg.COMPUTE_TYPE_FP16:
 *             t2 = 'h'
 *         elif compute_type == _linalg.COMPUTE_TYPE_TF32:             # <<<<<<<<<<<<<<
 *             t2 = 'x'
 *         elif compute_type == _linalg.COMPUTE_TYPE_FP32:
*/
    __Pyx_GetModuleGlobalName(__pyx_t_3, __pyx_mstate_global->__pyx_n_u_linalg_2); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 676, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_t_7 = __Pyx_PyObject_GetAttrStr(__pyx_t_3, __pyx_mstate_global->__pyx_n_u_COMPUTE_TYPE_TF32); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 676, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_7);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __pyx_t_3 = PyObject_RichCompare(__pyx_v_compute_type, __pyx_t_7, Py_EQ); __Pyx_XGOTREF(__pyx_t_3); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 676, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
    __pyx_t_5 = __Pyx_PyObject_IsTrue(__pyx_t_3); if (unlikely((__pyx_t_5 < 0))) __PYX_ERR(0, 676, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    if (__pyx_t_5) {

      /* "cupyx/cusolver.pyx":677
 *             t2 = 'h'
 *         elif compute_type == _linalg.COMPUTE_TYPE_TF32:
 *             t2 = 'x'             # <<<<<<<<<<<<<<
 *         elif compute_type == _linalg.COMPUTE_TYPE_FP32:
 *             t2 = 's'
*/
      __Pyx_INCREF(__pyx_mstate_global->__pyx_n_u_x);
      __Pyx_DECREF_SET(__pyx_v_t2, __pyx_mstate_global->__pyx_n_u_x);

      /* "cupyx/cusolver.pyx":676
 *         if compute_type == _linalg.COMPUTE_TYPE_FP16:
 *             t2 = 'h'
 *         elif compute_type == _linalg.COMPUTE_TYPE_TF32:             # <<<<<<<<<<<<<<
 *             t2 = 'x'
 *         elif compute_type == _linalg.COMPUTE_TYPE_FP32:
*/
      goto __pyx_L16;
    }

    /* "cupyx/cusolver.pyx":678
 *         elif compute_type == _linalg.COMPUTE_TYPE_TF32:
 *             t2 = 'x'
 *         elif compute_type == _linalg.COMPUTE_TYPE_FP32:             # <<<<<<<<<<<<<<
 *             t2 = 's'
 *     elif a.dtype.char in 'FD':
*/
    __Pyx_GetModuleGlobalName(__pyx_t_3, __pyx_mstate_global->__pyx_n_u_linalg_2); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 678, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_t_7 = __Pyx_PyObject_GetAttrStr(__pyx_t_3, __pyx_mstate_global->__pyx_n_u_COMPUTE_TYPE_FP32); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 678, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_7);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __pyx_t_3 = PyObject_RichCompare(__pyx_v_compute_type, __pyx_t_7, Py_EQ); __Pyx_XGOTREF(__pyx_t_3); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 678, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
    __pyx_t_5 = __Pyx_PyObject_IsTrue(__pyx_t_3); if (unlikely((__pyx_t_5 < 0))) __PYX_ERR(0, 678, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    if (__pyx_t_5) {

      /* "cupyx/cusolver.pyx":679
 *             t2 = 'x'
 *         elif compute_type == _linalg.COMPUTE_TYPE_FP32:
 *             t2 = 's'             # <<<<<<<<<<<<<<
 *     elif a.dtype.char in 'FD':
 *         if a.dtype.char == 'F':
*/
      __Pyx_INCREF(__pyx_mstate_global->__pyx_n_u_s);
      __Pyx_DECREF_SET(__pyx_v_t2, __pyx_mstate_global->__pyx_n_u_s);

      /* "cupyx/cusolver.pyx":678
 *         elif compute_type == _linalg.COMPUTE_TYPE_TF32:
 *             t2 = 'x'
 *         elif compute_type == _linalg.COMPUTE_TYPE_FP32:             # <<<<<<<<<<<<<<
 *             t2 = 's'
 *     elif a.dtype.char in 'FD':
*/
    }
    __pyx_L16:;

    /* "cupyx/cusolver.pyx":669
 * 
 *     compute_type = _linalg.get_compute_type(a.dtype)
 *     if a.dtype.char in 'fd':             # <<<<<<<<<<<<<<
 *         if a.dtype.char == 'f':
 *             t1 = t2 = 's'
*/
    goto __pyx_L14;
  }

  /* "cupyx/cusolver.pyx":680
 *         elif compute_type == _linalg.COMPUTE_TYPE_FP32:
 *             t2 = 's'
 *     elif a.dtype.char in 'FD':             # <<<<<<<<<<<<<<
 *         if a.dtype.char == 'F':
 *             t1 = t2 = 'c'
*/
  __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_v_a, __pyx_mstate_global->__pyx_n_u_dtype); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 680, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_7 = __Pyx_PyObject_GetAttrStr(__pyx_t_3, __pyx_mstate_global->__pyx_n_u_char); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 680, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_7);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_t_5 = (__Pyx_PyUnicode_ContainsTF(__pyx_t_7, __pyx_mstate_global->__pyx_n_u_FD, Py_EQ)); if (unlikely((__pyx_t_5 < 0))) __PYX_ERR(0, 680, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
  if (likely(__pyx_t_5)) {

    /* "cupyx/cusolver.pyx":681
 *             t2 = 's'
 *     elif a.dtype.char in 'FD':
 *         if a.dtype.char == 'F':             # <<<<<<<<<<<<<<
 *             t1 = t2 = 'c'
 *         else:
*/
    __pyx_t_7 = __Pyx_PyObject_GetAttrStr(__pyx_v_a, __pyx_mstate_global->__pyx_n_u_dtype); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 681, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_7);
    __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_t_7, __pyx_mstate_global->__pyx_n_u_char); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 681, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
    __pyx_t_5 = (__Pyx_PyUnicode_Equals(__pyx_t_3, __pyx_mstate_global->__pyx_n_u_F, Py_EQ)); if (unlikely((__pyx_t_5 < 0))) __PYX_ERR(0, 681, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    if (__pyx_t_5) {

      /* "cupyx/cusolver.pyx":682
 *     elif a.dtype.char in 'FD':
 *         if a.dtype.char == 'F':
 *             t1 = t2 = 'c'             # <<<<<<<<<<<<<<
 *         else:
 *             t1 = t2 = 'z'
*/
      __Pyx_INCREF(__pyx_mstate_global->__pyx_n_u_c);
      __pyx_v_t1 = __pyx_mstate_global->__pyx_n_u_c;
      __Pyx_INCREF(__pyx_mstate_global->__pyx_n_u_c);
      __pyx_v_t2 = __pyx_mstate_global->__pyx_n_u_c;

      /* "cupyx/cusolver.pyx":681
 *             t2 = 's'
 *     elif a.dtype.char in 'FD':
 *         if a.dtype.char == 'F':             # <<<<<<<<<<<<<<
 *             t1 = t2 = 'c'
 *         else:
*/
      goto __pyx_L17;
    }

    /* "cupyx/cusolver.pyx":684
 *             t1 = t2 = 'c'
 *         else:
 *             t1 = t2 = 'z'             # <<<<<<<<<<<<<<
 *         if compute_type == _linalg.COMPUTE_TYPE_FP16:
 *             t2 = 'k'
*/
    /*else*/ {
      __Pyx_INCREF(__pyx_mstate_global->__pyx_n_u_z);
      __pyx_v_t1 = __pyx_mstate_global->__pyx_n_u_z;
      __Pyx_INCREF(__pyx_mstate_global->__pyx_n_u_z);
      __pyx_v_t2 = __pyx_mstate_global->__pyx_n_u_z;
    }
    __pyx_L17:;

    /* "cupyx/cusolver.pyx":685
 *         else:
 *             t1 = t2 = 'z'
 *         if compute_type == _linalg.COMPUTE_TYPE_FP16:             # <<<<<<<<<<<<<<
 *             t2 = 'k'
 *         elif compute_type == _linalg.COMPUTE_TYPE_TF32:
*/
    __Pyx_GetModuleGlobalName(__pyx_t_3, __pyx_mstate_global->__pyx_n_u_linalg_2); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 685, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_t_7 = __Pyx_PyObject_GetAttrStr(__pyx_t_3, __pyx_mstate_global->__pyx_n_u_COMPUTE_TYPE_FP16); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 685, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_7);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __pyx_t_3 = PyObject_RichCompare(__pyx_v_compute_type, __pyx_t_7, Py_EQ); __Pyx_XGOTREF(__pyx_t_3); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 685, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
    __pyx_t_5 = __Pyx_PyObject_IsTrue(__pyx_t_3); if (unlikely((__pyx_t_5 < 0))) __PYX_ERR(0, 685, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    if (__pyx_t_5) {

      /* "cupyx/cusolver.pyx":686
 *             t1 = t2 = 'z'
 *         if compute_type == _linalg.COMPUTE_TYPE_FP16:
 *             t2 = 'k'             # <<<<<<<<<<<<<<
 *         elif compute_type == _linalg.COMPUTE_TYPE_TF32:
 *             t2 = 'y'
*/
      __Pyx_INCREF(__pyx_mstate_global->__pyx_n_u_k);
      __Pyx_DECREF_SET(__pyx_v_t2, __pyx_mstate_global->__pyx_n_u_k);

      /* "cupyx/cusolver.pyx":685
 *         else:
 *             t1 = t2 = 'z'
 *         if compute_type == _linalg.COMPUTE_TYPE_FP16:             # <<<<<<<<<<<<<<
 *             t2 = 'k'
 *         elif compute_type == _linalg.COMPUTE_TYPE_TF32:
*/
      goto __pyx_L18;
    }

    /* "cupyx/cusolver.pyx":687
 *         if compute_type == _linalg.COMPUTE_TYPE_FP16:
 *             t2 = 'k'
 *         elif compute_type == _linalg.COMPUTE_TYPE_TF32:             # <<<<<<<<<<<<<<
 *             t2 = 'y'
 *         elif compute_type == _linalg.COMPUTE_TYPE_FP32:
*/
    __Pyx_GetModuleGlobalName(__pyx_t_3, __pyx_mstate_global->__pyx_n_u_linalg_2); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 687, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_t_7 = __Pyx_PyObject_GetAttrStr(__pyx_t_3, __pyx_mstate_global->__pyx_n_u_COMPUTE_TYPE_TF32); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 687, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_7);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __pyx_t_3 = PyObject_RichCompare(__pyx_v_compute_type, __pyx_t_7, Py_EQ); __Pyx_XGOTREF(__pyx_t_3); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 687, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
    __pyx_t_5 = __Pyx_PyObject_IsTrue(__pyx_t_3); if (unlikely((__pyx_t_5 < 0))) __PYX_ERR(0, 687, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    if (__pyx_t_5) {

      /* "cupyx/cusolver.pyx":688
 *             t2 = 'k'
 *         elif compute_type == _linalg.COMPUTE_TYPE_TF32:
 *             t2 = 'y'             # <<<<<<<<<<<<<<
 *         elif compute_type == _linalg.COMPUTE_TYPE_FP32:
 *             t2 = 'c'
*/
      __Pyx_INCREF(__pyx_mstate_global->__pyx_n_u_y);
      __Pyx_DECREF_SET(__pyx_v_t2, __pyx_mstate_global->__pyx_n_u_y);

      /* "cupyx/cusolver.pyx":687
 *         if compute_type == _linalg.COMPUTE_TYPE_FP16:
 *             t2 = 'k'
 *         elif compute_type == _linalg.COMPUTE_TYPE_TF32:             # <<<<<<<<<<<<<<
 *             t2 = 'y'
 *         elif compute_type == _linalg.COMPUTE_TYPE_FP32:
*/
      goto __pyx_L18;
    }

    /* "cupyx/cusolver.pyx":689
 *         elif compute_type == _linalg.COMPUTE_TYPE_TF32:
 *             t2 = 'y'
 *         elif compute_type == _linalg.COMPUTE_TYPE_FP32:             # <<<<<<<<<<<<<<
 *             t2 = 'c'
 *     else:
*/
    __Pyx_GetModuleGlobalName(__pyx_t_3, __pyx_mstate_global->__pyx_n_u_linalg_2); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 689, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_t_7 = __Pyx_PyObject_GetAttrStr(__pyx_t_3, __pyx_mstate_global->__pyx_n_u_COMPUTE_TYPE_FP32); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 689, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_7);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __pyx_t_3 = PyObject_RichCompare(__pyx_v_compute_type, __pyx_t_7, Py_EQ); __Pyx_XGOTREF(__pyx_t_3); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 689, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
    __pyx_t_5 = __Pyx_PyObject_IsTrue(__pyx_t_3); if (unlikely((__pyx_t_5 < 0))) __PYX_ERR(0, 689, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    if (__pyx_t_5) {

      /* "cupyx/cusolver.pyx":690
 *             t2 = 'y'
 *         elif compute_type == _linalg.COMPUTE_TYPE_FP32:
 *             t2 = 'c'             # <<<<<<<<<<<<<<
 *     else:
 *         raise ValueError('unsupported dtype (actual:{})'.format(a.dtype))
*/
      __Pyx_INCREF(__pyx_mstate_global->__pyx_n_u_c);
      __Pyx_DECREF_SET(__pyx_v_t2, __pyx_mstate_global->__pyx_n_u_c);

      /* "cupyx/cusolver.pyx":689
 *         elif compute_type == _linalg.COMPUTE_TYPE_TF32:
 *             t2 = 'y'
 *         elif compute_type == _linalg.COMPUTE_TYPE_FP32:             # <<<<<<<<<<<<<<
 *             t2 = 'c'
 *     else:
*/
    }
    __pyx_L18:;

    /* "cupyx/cusolver.pyx":680
 *         elif compute_type == _linalg.COMPUTE_TYPE_FP32:
 *             t2 = 's'
 *     elif a.dtype.char in 'FD':             # <<<<<<<<<<<<<<
 *         if a.dtype.char == 'F':
 *             t1 = t2 = 'c'
*/
    goto __pyx_L14;
  }

  /* "cupyx/cusolver.pyx":692
 *             t2 = 'c'
 *     else:
 *         raise ValueError('unsupported dtype (actual:{})'.format(a.dtype))             # <<<<<<<<<<<<<<
 *     solver_name = t1 + t2 + 'gesv'
 *     solver = getattr(_cusolver, solver_name)
*/
  /*else*/ {
    __pyx_t_7 = NULL;
    __pyx_t_2 = __pyx_mstate_global->__pyx_kp_u_unsupported_dtype_actual;
    __Pyx_INCREF(__pyx_t_2);
    __pyx_t_8 = __Pyx_PyObject_GetAttrStr(__pyx_v_a, __pyx_mstate_global->__pyx_n_u_dtype); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 692, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_8);
    __pyx_t_4 = 0;
    {
      PyObject *__pyx_callargs[2] = {__pyx_t_2, __pyx_t_8};
      __pyx_t_1 = __Pyx_PyObject_FastCallMethod((PyObject*)__pyx_mstate_global->__pyx_n_u_format, __pyx_callargs+__pyx_t_4, (2-__pyx_t_4) | (1*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
      __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
      __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
      if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 692, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
    }
    __pyx_t_4 = 1;
    {
      PyObject *__pyx_callargs[2] = {__pyx_t_7, __pyx_t_1};
      __pyx_t_3 = __Pyx_PyObject_FastCall((PyObject*)(((PyTypeObject*)PyExc_ValueError)), __pyx_callargs+__pyx_t_4, (2-__pyx_t_4) | (__pyx_t_4*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
      __Pyx_XDECREF(__pyx_t_7); __pyx_t_7 = 0;
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 692, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_3);
    }
    __Pyx_Raise(__pyx_t_3, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __PYX_ERR(0, 692, __pyx_L1_error)
  }
  __pyx_L14:;

  /* "cupyx/cusolver.pyx":693
 *     else:
 *         raise ValueError('unsupported dtype (actual:{})'.format(a.dtype))
 *     solver_name = t1 + t2 + 'gesv'             # <<<<<<<<<<<<<<
 *     solver = getattr(_cusolver, solver_name)
 *     helper = getattr(_cusolver, solver_name + '_bufferSize')
*/
  __pyx_t_3 = __Pyx_PyUnicode_Concat(__pyx_v_t1, __pyx_v_t2); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 693, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_1 = __Pyx_PyUnicode_Concat__Pyx_ReferenceSharing_OwnStrongReferenceInPlace(__pyx_t_3, __pyx_mstate_global->__pyx_n_u_gesv); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 693, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_v_solver_name = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "cupyx/cusolver.pyx":694
 *         raise ValueError('unsupported dtype (actual:{})'.format(a.dtype))
 *     solver_name = t1 + t2 + 'gesv'
 *     solver = getattr(_cusolver, solver_name)             # <<<<<<<<<<<<<<
 *     helper = getattr(_cusolver, solver_name + '_bufferSize')
 * 
*/
  __Pyx_GetModuleGlobalName(__pyx_t_1, __pyx_mstate_global->__pyx_n_u_cusolver); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 694, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_3 = __Pyx_GetAttr(__pyx_t_1, __pyx_v_solver_name); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 694, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v_solver = __pyx_t_3;
  __pyx_t_3 = 0;

  /* "cupyx/cusolver.pyx":695
 *     solver_name = t1 + t2 + 'gesv'
 *     solver = getattr(_cusolver, solver_name)
 *     helper = getattr(_cusolver, solver_name + '_bufferSize')             # <<<<<<<<<<<<<<
 * 
 *     a = a.copy(order='F')
*/
  __Pyx_GetModuleGlobalName(__pyx_t_3, __pyx_mstate_global->__pyx_n_u_cusolver); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 695, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_1 = __Pyx_PyUnicode_Concat(__pyx_v_solver_name, __pyx_mstate_global->__pyx_n_u_bufferSize); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 695, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_7 = __Pyx_GetAttr(__pyx_t_3, __pyx_t_1); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 695, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_7);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v_helper = __pyx_t_7;
  __pyx_t_7 = 0;

  /* "cupyx/cusolver.pyx":697
 *     helper = getattr(_cusolver, solver_name + '_bufferSize')
 * 
 *     a = a.copy(order='F')             # <<<<<<<<<<<<<<
 *     b = b.copy(order='F')
 *     x = _cupy.empty_like(b)
*/
  __pyx_t_1 = __pyx_v_a;
  __Pyx_INCREF(__pyx_t_1);
  __pyx_t_4 = 0;
  {
    PyObject *__pyx_callargs[2 + ((CYTHON_VECTORCALL) ? 1 : 0)] = {__pyx_t_1, NULL};
    __pyx_t_3 = __Pyx_MakeVectorcallBuilderKwds(1); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 697, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    if (__Pyx_VectorcallBuilder_AddArg(__pyx_mstate_global->__pyx_n_u_order, __pyx_mstate_global->__pyx_n_u_F, __pyx_t_3, __pyx_callargs+1, 0) < (0)) __PYX_ERR(0, 697, __pyx_L1_error)
    __pyx_t_7 = __Pyx_Object_VectorcallMethod_CallFromBuilder((PyObject*)__pyx_mstate_global->__pyx_n_u_copy, __pyx_callargs+__pyx_t_4, (1-__pyx_t_4) | (1*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET), __pyx_t_3);
    __Pyx_XDECREF(__pyx_t_1); __pyx_t_1 = 0;
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 697, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_7);
  }
  __Pyx_DECREF_SET(__pyx_v_a, __pyx_t_7);
  __pyx_t_7 = 0;

  /* "cupyx/cusolver.pyx":698
 * 
 *     a = a.copy(order='F')
 *     b = b.copy(order='F')             # <<<<<<<<<<<<<<
 *     x = _cupy.empty_like(b)
 *     dipiv = _cupy.empty(n, dtype=_numpy.int32)
*/
  __pyx_t_3 = __pyx_v_b;
  __Pyx_INCREF(__pyx_t_3);
  __pyx_t_4 = 0;
  {
    PyObject *__pyx_callargs[2 + ((CYTHON_VECTORCALL) ? 1 : 0)] = {__pyx_t_3, NULL};
    __pyx_t_1 = __Pyx_MakeVectorcallBuilderKwds(1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 698, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    if (__Pyx_VectorcallBuilder_AddArg(__pyx_mstate_global->__pyx_n_u_order, __pyx_mstate_global->__pyx_n_u_F, __pyx_t_1, __pyx_callargs+1, 0) < (0)) __PYX_ERR(0, 698, __pyx_L1_error)
    __pyx_t_7 = __Pyx_Object_VectorcallMethod_CallFromBuilder((PyObject*)__pyx_mstate_global->__pyx_n_u_copy, __pyx_callargs+__pyx_t_4, (1-__pyx_t_4) | (1*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET), __pyx_t_1);
    __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 698, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_7);
  }
  __Pyx_DECREF_SET(__pyx_v_b, __pyx_t_7);
  __pyx_t_7 = 0;

  /* "cupyx/cusolver.pyx":699
 *     a = a.copy(order='F')
 *     b = b.copy(order='F')
 *     x = _cupy.empty_like(b)             # <<<<<<<<<<<<<<
 *     dipiv = _cupy.empty(n, dtype=_numpy.int32)
 *     dinfo = _cupy.empty(1, dtype=_numpy.int32)
*/
  __pyx_t_1 = NULL;
  __Pyx_GetModuleGlobalName(__pyx_t_3, __pyx_mstate_global->__pyx_n_u_cupy); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 699, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_8 = __Pyx_PyObject_GetAttrStr(__pyx_t_3, __pyx_mstate_global->__pyx_n_u_empty_like); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 699, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_8);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_t_4 = 1;
  #if CYTHON_UNPACK_METHODS
  if (unlikely(PyMethod_Check(__pyx_t_8))) {
    __pyx_t_1 = PyMethod_GET_SELF(__pyx_t_8);
    assert(__pyx_t_1);
    PyObject* __pyx__function = PyMethod_GET_FUNCTION(__pyx_t_8);
    __Pyx_INCREF(__pyx_t_1);
    __Pyx_INCREF(__pyx__function);
    __Pyx_DECREF_SET(__pyx_t_8, __pyx__function);
    __pyx_t_4 = 0;
  }
  #endif
  {
    PyObject *__pyx_callargs[2] = {__pyx_t_1, __pyx_v_b};
    __pyx_t_7 = __Pyx_PyObject_FastCall((PyObject*)__pyx_t_8, __pyx_callargs+__pyx_t_4, (2-__pyx_t_4) | (__pyx_t_4*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
    __Pyx_XDECREF(__pyx_t_1); __pyx_t_1 = 0;
    __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
    if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 699, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_7);
  }
  __pyx_v_x = __pyx_t_7;
  __pyx_t_7 = 0;

  /* "cupyx/cusolver.pyx":700
 *     b = b.copy(order='F')
 *     x = _cupy.empty_like(b)
 *     dipiv = _cupy.empty(n, dtype=_numpy.int32)             # <<<<<<<<<<<<<<
 *     dinfo = _cupy.empty(1, dtype=_numpy.int32)
 *     handle = _device.get_cusolver_handle()
*/
  __pyx_t_8 = NULL;
  __Pyx_GetModuleGlobalName(__pyx_t_1, __pyx_mstate_global->__pyx_n_u_cupy); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 700, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_t_1, __pyx_mstate_global->__pyx_n_u_empty); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 700, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_GetModuleGlobalName(__pyx_t_1, __pyx_mstate_global->__pyx_n_u_numpy); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 700, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_t_1, __pyx_mstate_global->__pyx_n_u_int32); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 700, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_4 = 1;
  #if CYTHON_UNPACK_METHODS
  if (unlikely(PyMethod_Check(__pyx_t_3))) {
    __pyx_t_8 = PyMethod_GET_SELF(__pyx_t_3);
    assert(__pyx_t_8);
    PyObject* __pyx__function = PyMethod_GET_FUNCTION(__pyx_t_3);
    __Pyx_INCREF(__pyx_t_8);
    __Pyx_INCREF(__pyx__function);
    __Pyx_DECREF_SET(__pyx_t_3, __pyx__function);
    __pyx_t_4 = 0;
  }
  #endif
  {
    PyObject *__pyx_callargs[2 + ((CYTHON_VECTORCALL) ? 1 : 0)] = {__pyx_t_8, __pyx_v_n};
    __pyx_t_1 = __Pyx_MakeVectorcallBuilderKwds(1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 700, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    if (__Pyx_VectorcallBuilder_AddArg(__pyx_mstate_global->__pyx_n_u_dtype, __pyx_t_2, __pyx_t_1, __pyx_callargs+2, 0) < (0)) __PYX_ERR(0, 700, __pyx_L1_error)
    __pyx_t_7 = __Pyx_Object_Vectorcall_CallFromBuilder((PyObject*)__pyx_t_3, __pyx_callargs+__pyx_t_4, (2-__pyx_t_4) | (__pyx_t_4*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET), __pyx_t_1);
    __Pyx_XDECREF(__pyx_t_8); __pyx_t_8 = 0;
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 700, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_7);
  }
  __pyx_v_dipiv = __pyx_t_7;
  __pyx_t_7 = 0;

  /* "cupyx/cusolver.pyx":701
 *     x = _cupy.empty_like(b)
 *     dipiv = _cupy.empty(n, dtype=_numpy.int32)
 *     dinfo = _cupy.empty(1, dtype=_numpy.int32)             # <<<<<<<<<<<<<<
 *     handle = _device.get_cusolver_handle()
 *     lwork = helper(handle, n, nrhs, a.data.ptr, n, dipiv.data.ptr,
*/
  __pyx_t_3 = NULL;
  __Pyx_GetModuleGlobalName(__pyx_t_1, __pyx_mstate_global->__pyx_n_u_cupy); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 701, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_t_1, __pyx_mstate_global->__pyx_n_u_empty); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 701, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_GetModuleGlobalName(__pyx_t_1, __pyx_mstate_global->__pyx_n_u_numpy); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 701, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_8 = __Pyx_PyObject_GetAttrStr(__pyx_t_1, __pyx_mstate_global->__pyx_n_u_int32); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 701, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_8);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_4 = 1;
  #if CYTHON_UNPACK_METHODS
  if (unlikely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_2);
    assert(__pyx_t_3);
    PyObject* __pyx__function = PyMethod_GET_FUNCTION(__pyx_t_2);
    __Pyx_INCREF(__pyx_t_3);
    __Pyx_INCREF(__pyx__function);
    __Pyx_DECREF_SET(__pyx_t_2, __pyx__function);
    __pyx_t_4 = 0;
  }
  #endif
  {
    PyObject *__pyx_callargs[2 + ((CYTHON_VECTORCALL) ? 1 : 0)] = {__pyx_t_3, __pyx_mstate_global->__pyx_int_1};
    __pyx_t_1 = __Pyx_MakeVectorcallBuilderKwds(1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 701, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    if (__Pyx_VectorcallBuilder_AddArg(__pyx_mstate_global->__pyx_n_u_dtype, __pyx_t_8, __pyx_t_1, __pyx_callargs+2, 0) < (0)) __PYX_ERR(0, 701, __pyx_L1_error)
    __pyx_t_7 = __Pyx_Object_Vectorcall_CallFromBuilder((PyObject*)__pyx_t_2, __pyx_callargs+__pyx_t_4, (2-__pyx_t_4) | (__pyx_t_4*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET), __pyx_t_1);
    __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
    __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 701, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_7);
  }
  __pyx_v_dinfo = __pyx_t_7;
  __pyx_t_7 = 0;

  /* "cupyx/cusolver.pyx":702
 *     dipiv = _cupy.empty(n, dtype=_numpy.int32)
 *     dinfo = _cupy.empty(1, dtype=_numpy.int32)
 *     handle = _device.get_cusolver_handle()             # <<<<<<<<<<<<<<
 *     lwork = helper(handle, n, nrhs, a.data.ptr, n, dipiv.data.ptr,
 *                    b.data.ptr, n, x.data.ptr, n, 0)
*/
  __pyx_t_2 = NULL;
  __Pyx_GetModuleGlobalName(__pyx_t_1, __pyx_mstate_global->__pyx_n_u_device); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 702, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_8 = __Pyx_PyObject_GetAttrStr(__pyx_t_1, __pyx_mstate_global->__pyx_n_u_get_cusolver_handle); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 702, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_8);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_4 = 1;
  #if CYTHON_UNPACK_METHODS
  if (unlikely(PyMethod_Check(__pyx_t_8))) {
    __pyx_t_2 = PyMethod_GET_SELF(__pyx_t_8);
    assert(__pyx_t_2);
    PyObject* __pyx__function = PyMethod_GET_FUNCTION(__pyx_t_8);
    __Pyx_INCREF(__pyx_t_2);
    __Pyx_INCREF(__pyx__function);
    __Pyx_DECREF_SET(__pyx_t_8, __pyx__function);
    __pyx_t_4 = 0;
  }
  #endif
  {
    PyObject *__pyx_callargs[2] = {__pyx_t_2, NULL};
    __pyx_t_7 = __Pyx_PyObject_FastCall((PyObject*)__pyx_t_8, __pyx_callargs+__pyx_t_4, (1-__pyx_t_4) | (__pyx_t_4*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
    __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
    if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 702, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_7);
  }
  __pyx_v_handle = __pyx_t_7;
  __pyx_t_7 = 0;

  /* "cupyx/cusolver.pyx":703
 *     dinfo = _cupy.empty(1, dtype=_numpy.int32)
 *     handle = _device.get_cusolver_handle()
 *     lwork = helper(handle, n, nrhs, a.data.ptr, n, dipiv.data.ptr,             # <<<<<<<<<<<<<<
 *                    b.data.ptr, n, x.data.ptr, n, 0)
 *     dwork = _cupy.empty(lwork, dtype=_numpy.int8)
*/
  __pyx_t_8 = NULL;
  __Pyx_INCREF(__pyx_v_helper);
  __pyx_t_2 = __pyx_v_helper; 
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_v_a, __pyx_mstate_global->__pyx_n_u_data); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 703, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_t_1, __pyx_mstate_global->__pyx_n_u_ptr); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 703, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_v_dipiv, __pyx_mstate_global->__pyx_n_u_data); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 703, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_9 = __Pyx_PyObject_GetAttrStr(__pyx_t_1, __pyx_mstate_global->__pyx_n_u_ptr); if (unlikely(!__pyx_t_9)) __PYX_ERR(0, 703, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_9);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "cupyx/cusolver.pyx":704
 *     handle = _device.get_cusolver_handle()
 *     lwork = helper(handle, n, nrhs, a.data.ptr, n, dipiv.data.ptr,
 *                    b.data.ptr, n, x.data.ptr, n, 0)             # <<<<<<<<<<<<<<
 *     dwork = _cupy.empty(lwork, dtype=_numpy.int8)
 *     niters = solver(handle, n, nrhs, a.data.ptr, n, dipiv.data.ptr,
*/
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_v_b, __pyx_mstate_global->__pyx_n_u_data); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 704, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_11 = __Pyx_PyObject_GetAttrStr(__pyx_t_1, __pyx_mstate_global->__pyx_n_u_ptr); if (unlikely(!__pyx_t_11)) __PYX_ERR(0, 704, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_11);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_v_x, __pyx_mstate_global->__pyx_n_u_data); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 704, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_12 = __Pyx_PyObject_GetAttrStr(__pyx_t_1, __pyx_mstate_global->__pyx_n_u_ptr); if (unlikely(!__pyx_t_12)) __PYX_ERR(0, 704, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_12);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_4 = 1;
  #if CYTHON_UNPACK_METHODS
  if (unlikely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_8 = PyMethod_GET_SELF(__pyx_t_2);
    assert(__pyx_t_8);
    PyObject* __pyx__function = PyMethod_GET_FUNCTION(__pyx_t_2);
    __Pyx_INCREF(__pyx_t_8);
    __Pyx_INCREF(__pyx__function);
    __Pyx_DECREF_SET(__pyx_t_2, __pyx__function);
    __pyx_t_4 = 0;
  }
  #endif
  {
    PyObject *__pyx_callargs[12] = {__pyx_t_8, __pyx_v_handle, __pyx_v_n, __pyx_v_nrhs, __pyx_t_3, __pyx_v_n, __pyx_t_9, __pyx_t_11, __pyx_v_n, __pyx_t_12, __pyx_v_n, __pyx_mstate_global->__pyx_int_0};
    __pyx_t_7 = __Pyx_PyObject_FastCall((PyObject*)__pyx_t_2, __pyx_callargs+__pyx_t_4, (12-__pyx_t_4) | (__pyx_t_4*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
    __Pyx_XDECREF(__pyx_t_8); __pyx_t_8 = 0;
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
    __Pyx_DECREF(__pyx_t_11); __pyx_t_11 = 0;
    __Pyx_DECREF(__pyx_t_12); __pyx_t_12 = 0;
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 703, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_7);
  }
  __pyx_v_lwork = __pyx_t_7;
  __pyx_t_7 = 0;

  /* "cupyx/cusolver.pyx":705
 *     lwork = helper(handle, n, nrhs, a.data.ptr, n, dipiv.data.ptr,
 *                    b.data.ptr, n, x.data.ptr, n, 0)
 *     dwork = _cupy.empty(lwork, dtype=_numpy.int8)             # <<<<<<<<<<<<<<
 *     niters = solver(handle, n, nrhs, a.data.ptr, n, dipiv.data.ptr,
 *                     b.data.ptr, n, x.data.ptr, n, dwork.data.ptr, lwork,
*/
  __pyx_t_2 = NULL;
  __Pyx_GetModuleGlobalName(__pyx_t_12, __pyx_mstate_global->__pyx_n_u_cupy); if (unlikely(!__pyx_t_12)) __PYX_ERR(0, 705, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_12);
  __pyx_t_11 = __Pyx_PyObject_GetAttrStr(__pyx_t_12, __pyx_mstate_global->__pyx_n_u_empty); if (unlikely(!__pyx_t_11)) __PYX_ERR(0, 705, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_11);
  __Pyx_DECREF(__pyx_t_12); __pyx_t_12 = 0;
  __Pyx_GetModuleGlobalName(__pyx_t_12, __pyx_mstate_global->__pyx_n_u_numpy); if (unlikely(!__pyx_t_12)) __PYX_ERR(0, 705, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_12);
  __pyx_t_9 = __Pyx_PyObject_GetAttrStr(__pyx_t_12, __pyx_mstate_global->__pyx_n_u_int8); if (unlikely(!__pyx_t_9)) __PYX_ERR(0, 705, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_9);
  __Pyx_DECREF(__pyx_t_12); __pyx_t_12 = 0;
  __pyx_t_4 = 1;
  #if CYTHON_UNPACK_METHODS
  if (unlikely(PyMethod_Check(__pyx_t_11))) {
    __pyx_t_2 = PyMethod_GET_SELF(__pyx_t_11);
    assert(__pyx_t_2);
    PyObject* __pyx__function = PyMethod_GET_FUNCTION(__pyx_t_11);
    __Pyx_INCREF(__pyx_t_2);
    __Pyx_INCREF(__pyx__function);
    __Pyx_DECREF_SET(__pyx_t_11, __pyx__function);
    __pyx_t_4 = 0;
  }
  #endif
  {
    PyObject *__pyx_callargs[2 + ((CYTHON_VECTORCALL) ? 1 : 0)] = {__pyx_t_2, __pyx_v_lwork};
    __pyx_t_12 = __Pyx_MakeVectorcallBuilderKwds(1); if (unlikely(!__pyx_t_12)) __PYX_ERR(0, 705, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_12);
    if (__Pyx_VectorcallBuilder_AddArg(__pyx_mstate_global->__pyx_n_u_dtype, __pyx_t_9, __pyx_t_12, __pyx_callargs+2, 0) < (0)) __PYX_ERR(0, 705, __pyx_L1_error)
    __pyx_t_7 = __Pyx_Object_Vectorcall_CallFromBuilder((PyObject*)__pyx_t_11, __pyx_callargs+__pyx_t_4, (2-__pyx_t_4) | (__pyx_t_4*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET), __pyx_t_12);
    __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
    __Pyx_DECREF(__pyx_t_12); __pyx_t_12 = 0;
    __Pyx_DECREF(__pyx_t_11); __pyx_t_11 = 0;
    if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 705, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_7);
  }
  __pyx_v_dwork = __pyx_t_7;
  __pyx_t_7 = 0;

  /* "cupyx/cusolver.pyx":706
 *                    b.data.ptr, n, x.data.ptr, n, 0)
 *     dwork = _cupy.empty(lwork, dtype=_numpy.int8)
 *     niters = solver(handle, n, nrhs, a.data.ptr, n, dipiv.data.ptr,             # <<<<<<<<<<<<<<
 *                     b.data.ptr, n, x.data.ptr, n, dwork.data.ptr, lwork,
 *                     dinfo.data.ptr)
*/
  __pyx_t_11 = NULL;
  __Pyx_INCREF(__pyx_v_solver);
  __pyx_t_12 = __pyx_v_solver; 
  __pyx_t_9 = __Pyx_PyObject_GetAttrStr(__pyx_v_a, __pyx_mstate_global->__pyx_n_u_data); if (unlikely(!__pyx_t_9)) __PYX_ERR(0, 706, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_9);
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_t_9, __pyx_mstate_global->__pyx_n_u_ptr); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 706, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
  __pyx_t_9 = __Pyx_PyObject_GetAttrStr(__pyx_v_dipiv, __pyx_mstate_global->__pyx_n_u_data); if (unlikely(!__pyx_t_9)) __PYX_ERR(0, 706, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_9);
  __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_t_9, __pyx_mstate_global->__pyx_n_u_ptr); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 706, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;

  /* "cupyx/cusolver.pyx":707
 *     dwork = _cupy.empty(lwork, dtype=_numpy.int8)
 *     niters = solver(handle, n, nrhs, a.data.ptr, n, dipiv.data.ptr,
 *                     b.data.ptr, n, x.data.ptr, n, dwork.data.ptr, lwork,             # <<<<<<<<<<<<<<
 *                     dinfo.data.ptr)
 *     if niters < 0:
*/
  __pyx_t_9 = __Pyx_PyObject_GetAttrStr(__pyx_v_b, __pyx_mstate_global->__pyx_n_u_data); if (unlikely(!__pyx_t_9)) __PYX_ERR(0, 707, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_9);
  __pyx_t_8 = __Pyx_PyObject_GetAttrStr(__pyx_t_9, __pyx_mstate_global->__pyx_n_u_ptr); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 707, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_8);
  __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
  __pyx_t_9 = __Pyx_PyObject_GetAttrStr(__pyx_v_x, __pyx_mstate_global->__pyx_n_u_data); if (unlikely(!__pyx_t_9)) __PYX_ERR(0, 707, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_9);
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_t_9, __pyx_mstate_global->__pyx_n_u_ptr); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 707, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
  __pyx_t_9 = __Pyx_PyObject_GetAttrStr(__pyx_v_dwork, __pyx_mstate_global->__pyx_n_u_data); if (unlikely(!__pyx_t_9)) __PYX_ERR(0, 707, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_9);
  __pyx_t_13 = __Pyx_PyObject_GetAttrStr(__pyx_t_9, __pyx_mstate_global->__pyx_n_u_ptr); if (unlikely(!__pyx_t_13)) __PYX_ERR(0, 707, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_13);
  __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;

  /* "cupyx/cusolver.pyx":708
 *     niters = solver(handle, n, nrhs, a.data.ptr, n, dipiv.data.ptr,
 *                     b.data.ptr, n, x.data.ptr, n, dwork.data.ptr, lwork,
 *                     dinfo.data.ptr)             # <<<<<<<<<<<<<<
 *     if niters < 0:
 *         raise RuntimeError('gesv has failed ({}).'.format(niters))
*/
  __pyx_t_9 = __Pyx_PyObject_GetAttrStr(__pyx_v_dinfo, __pyx_mstate_global->__pyx_n_u_data); if (unlikely(!__pyx_t_9)) __PYX_ERR(0, 708, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_9);
  __pyx_t_14 = __Pyx_PyObject_GetAttrStr(__pyx_t_9, __pyx_mstate_global->__pyx_n_u_ptr); if (unlikely(!__pyx_t_14)) __PYX_ERR(0, 708, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_14);
  __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
  __pyx_t_4 = 1;
  #if CYTHON_UNPACK_METHODS
  if (unlikely(PyMethod_Check(__pyx_t_12))) {
    __pyx_t_11 = PyMethod_GET_SELF(__pyx_t_12);
    assert(__pyx_t_11);
    PyObject* __pyx__function = PyMethod_GET_FUNCTION(__pyx_t_12);
    __Pyx_INCREF(__pyx_t_11);
    __Pyx_INCREF(__pyx__function);
    __Pyx_DECREF_SET(__pyx_t_12, __pyx__function);
    __pyx_t_4 = 0;
  }
  #endif
  {
    PyObject *__pyx_callargs[14] = {__pyx_t_11, __pyx_v_handle, __pyx_v_n, __pyx_v_nrhs, __pyx_t_2, __pyx_v_n, __pyx_t_3, __pyx_t_8, __pyx_v_n, __pyx_t_1, __pyx_v_n, __pyx_t_13, __pyx_v_lwork, __pyx_t_14};
    __pyx_t_7 = __Pyx_PyObject_FastCall((PyObject*)__pyx_t_12, __pyx_callargs+__pyx_t_4, (14-__pyx_t_4) | (__pyx_t_4*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
    __Pyx_XDECREF(__pyx_t_11); __pyx_t_11 = 0;
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __Pyx_DECREF(__pyx_t_13); __pyx_t_13 = 0;
    __Pyx_DECREF(__pyx_t_14); __pyx_t_14 = 0;
    __Pyx_DECREF(__pyx_t_12); __pyx_t_12 = 0;
    if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 706, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_7);
  }
  __pyx_v_niters = __pyx_t_7;
  __pyx_t_7 = 0;

  /* "cupyx/cusolver.pyx":709
 *                     b.data.ptr, n, x.data.ptr, n, dwork.data.ptr, lwork,
 *                     dinfo.data.ptr)
 *     if niters < 0:             # <<<<<<<<<<<<<<
 *         raise RuntimeError('gesv has failed ({}).'.format(niters))
 *     return x
*/
  __pyx_t_7 = PyObject_RichCompare(__pyx_v_niters, __pyx_mstate_global->__pyx_int_0, Py_LT); __Pyx_XGOTREF(__pyx_t_7); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 709, __pyx_L1_error)
  __pyx_t_5 = __Pyx_PyObject_IsTrue(__pyx_t_7); if (unlikely((__pyx_t_5 < 0))) __PYX_ERR(0, 709, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
  if (unlikely(__pyx_t_5)) {

    /* "cupyx/cusolver.pyx":710
 *                     dinfo.data.ptr)
 *     if niters < 0:
 *         raise RuntimeError('gesv has failed ({}).'.format(niters))             # <<<<<<<<<<<<<<
 *     return x
 * 
*/
    __pyx_t_12 = NULL;
    __pyx_t_13 = __pyx_mstate_global->__pyx_kp_u_gesv_has_failed;
    __Pyx_INCREF(__pyx_t_13);
    __pyx_t_4 = 0;
    {
      PyObject *__pyx_callargs[2] = {__pyx_t_13, __pyx_v_niters};
      __pyx_t_14 = __Pyx_PyObject_FastCallMethod((PyObject*)__pyx_mstate_global->__pyx_n_u_format, __pyx_callargs+__pyx_t_4, (2-__pyx_t_4) | (1*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
      __Pyx_XDECREF(__pyx_t_13); __pyx_t_13 = 0;
      if (unlikely(!__pyx_t_14)) __PYX_ERR(0, 710, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_14);
    }
    __pyx_t_4 = 1;
    {
      PyObject *__pyx_callargs[2] = {__pyx_t_12, __pyx_t_14};
      __pyx_t_7 = __Pyx_PyObject_FastCall((PyObject*)(((PyTypeObject*)PyExc_RuntimeError)), __pyx_callargs+__pyx_t_4, (2-__pyx_t_4) | (__pyx_t_4*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
      __Pyx_XDECREF(__pyx_t_12); __pyx_t_12 = 0;
      __Pyx_DECREF(__pyx_t_14); __pyx_t_14 = 0;
      if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 710, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_7);
    }
    __Pyx_Raise(__pyx_t_7, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
    __PYX_ERR(0, 710, __pyx_L1_error)

    /* "cupyx/cusolver.pyx":709
 *                     b.data.ptr, n, x.data.ptr, n, dwork.data.ptr, lwork,
 *                     dinfo.data.ptr)
 *     if niters < 0:             # <<<<<<<<<<<<<<
 *         raise RuntimeError('gesv has failed ({}).'.format(niters))
 *     return x
*/
  }

  /* "cupyx/cusolver.pyx":711
 *     if niters < 0:
 *         raise RuntimeError('gesv has failed ({}).'.format(niters))
 *     return x             # <<<<<<<<<<<<<<
 * 
 * 
*/
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_v_x);
  __pyx_r = __pyx_v_x;
  goto __pyx_L0;

  /* "cupyx/cusolver.pyx":637
 * 
 * 
 * def gesv(a, b):             # <<<<<<<<<<<<<<
 *     """Solve a linear matrix equation using cusolverDn<t1><t2>gesv().
 *     Computes the solution to a system of linear equation ``ax = b``.
*/

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_7);
  __Pyx_XDECREF(__pyx_t_8);
  __Pyx_XDECREF(__pyx_t_9);
  __Pyx_XDECREF(__pyx_t_11);
  __Pyx_XDECREF(__pyx_t_12);
  __Pyx_XDECREF(__pyx_t_13);
  __Pyx_XDECREF(__pyx_t_14);
  __Pyx_AddTraceback("cupyx.cusolver.gesv", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_n);
  __Pyx_XDECREF(__pyx_v_nrhs);
  __Pyx_XDECREF(__pyx_v_compute_type);
  __Pyx_XDECREF(__pyx_v_t1);
  __Pyx_XDECREF(__pyx_v_t2);
  __Pyx_XDECREF(__pyx_v_solver_name);
  __Pyx_XDECREF(__pyx_v_solver);
  __Pyx_XDECREF(__pyx_v_helper);
  __Pyx_XDECREF(__pyx_v_x);
  __Pyx_XDECREF(__pyx_v_dipiv);
  __Pyx_XDECREF(__pyx_v_dinfo);
  __Pyx_XDECREF(__pyx_v_handle);
  __Pyx_XDECREF(__pyx_v_lwork);
  __Pyx_XDECREF(__pyx_v_dwork);
  __Pyx_XDECREF(__pyx_v_niters);
  __Pyx_XDECREF(__pyx_v_a);
  __Pyx_XDECREF(__pyx_v_b);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "cupyx/cusolver.pyx":714
 * 
 * 
 * def gels(a, b):             # <<<<<<<<<<<<<<
 *     """Compute least square solution using cusolverDn<t1><t2>gels().
 * 
*/

/* Python wrapper */
static PyObject *__pyx_pw_5cupyx_8cusolver_17gels(PyObject *__pyx_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
PyDoc_STRVAR(__pyx_doc_5cupyx_8cusolver_16gels, "gels(a, b)\n\nCompute least square solution using cusolverDn<t1><t2>gels().\n\nComputes the least square solution to a system of ``ax = b``.\n\nArgs:\n    a (cupy.ndarray): The matrix with dimension ``(M, N)``.\n    b (cupy.ndarray): The matrix with dimension ``(M)`` or ``(M, K)``.\n\nReturns:\n    cupy.ndarray:\n        The matrix with dimension ``(N)`` or ``(N, K)``.");
static PyMethodDef __pyx_mdef_5cupyx_8cusolver_17gels = {"gels", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_5cupyx_8cusolver_17gels, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_5cupyx_8cusolver_16gels};
static PyObject *__pyx_pw_5cupyx_8cusolver_17gels(PyObject *__pyx_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
) {
  PyObject *__pyx_v_a = 0;
  PyObject *__pyx_v_b = 0;
  #if !CYTHON_METH_FASTCALL
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  #endif
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject* values[2] = {0,0};
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("gels (wrapper)", 0);
  #if !CYTHON_METH_FASTCALL
  #if CYTHON_ASSUME_SAFE_SIZE
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return NULL;
  #endif
  #endif
  __pyx_kwvalues = __Pyx_KwValues_FASTCALL(__pyx_args, __pyx_nargs);
  {
    PyObject ** const __pyx_pyargnames[] = {&__pyx_mstate_global->__pyx_n_u_a,&__pyx_mstate_global->__pyx_n_u_b,0};
    const Py_ssize_t __pyx_kwds_len = (__pyx_kwds) ? __Pyx_NumKwargs_FASTCALL(__pyx_kwds) : 0;
    if (unlikely(__pyx_kwds_len < 0)) __PYX_ERR(0, 714, __pyx_L3_error)
    if (__pyx_kwds_len > 0) {
      switch (__pyx_nargs) {
        case  2:
        values[1] = __Pyx_ArgRef_FASTCALL(__pyx_args, 1);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[1])) __PYX_ERR(0, 714, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  1:
        values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(0, 714, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      const Py_ssize_t kwd_pos_args = __pyx_nargs;
      if (__Pyx_ParseKeywords(__pyx_kwds, __pyx_kwvalues, __pyx_pyargnames, 0, values, kwd_pos_args, __pyx_kwds_len, "gels", 0) < (0)) __PYX_ERR(0, 714, __pyx_L3_error)
      for (Py_ssize_t i = __pyx_nargs; i < 2; i++) {
        if (unlikely(!values[i])) { __Pyx_RaiseArgtupleInvalid("gels", 1, 2, 2, i); __PYX_ERR(0, 714, __pyx_L3_error) }
      }
    } else if (unlikely(__pyx_nargs != 2)) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
      if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(0, 714, __pyx_L3_error)
      values[1] = __Pyx_ArgRef_FASTCALL(__pyx_args, 1);
      if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[1])) __PYX_ERR(0, 714, __pyx_L3_error)
    }
    __pyx_v_a = values[0];
    __pyx_v_b = values[1];
  }
  goto __pyx_L6_skip;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("gels", 1, 2, 2, __pyx_nargs); __PYX_ERR(0, 714, __pyx_L3_error)
  __pyx_L6_skip:;
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_AddTraceback("cupyx.cusolver.gels", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_5cupyx_8cusolver_16gels(__pyx_self, __pyx_v_a, __pyx_v_b);

  /* function exit code */
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5cupyx_8cusolver_16gels(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_a, PyObject *__pyx_v_b) {
  PyObject *__pyx_v_nrhs = NULL;
  PyObject *__pyx_v_m = NULL;
  PyObject *__pyx_v_n = NULL;
  PyObject *__pyx_v_max_mn = NULL;
  PyObject *__pyx_v_b_ndim = NULL;
  PyObject *__pyx_v_compute_type = NULL;
  PyObject *__pyx_v_t1 = NULL;
  PyObject *__pyx_v_t2 = NULL;
  PyObject *__pyx_v_solver_name = NULL;
  PyObject *__pyx_v_solver = NULL;
  PyObject *__pyx_v_helper = NULL;
  PyObject *__pyx_v_org_nrhs = NULL;
  PyObject *__pyx_v_bb = NULL;
  PyObject *__pyx_v_x = NULL;
  PyObject *__pyx_v_dinfo = NULL;
  PyObject *__pyx_v_handle = NULL;
  PyObject *__pyx_v_lwork = NULL;
  PyObject *__pyx_v_dwork = NULL;
  PyObject *__pyx_v_niters = NULL;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  size_t __pyx_t_4;
  int __pyx_t_5;
  int __pyx_t_6;
  PyObject *__pyx_t_7 = NULL;
  PyObject *__pyx_t_8 = NULL;
  PyObject *__pyx_t_9 = NULL;
  PyObject *(*__pyx_t_10)(PyObject *);
  PyObject *__pyx_t_11 = NULL;
  PyObject *__pyx_t_12 = NULL;
  PyObject *__pyx_t_13 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("gels", 0);
  __Pyx_INCREF(__pyx_v_a);
  __Pyx_INCREF(__pyx_v_b);

  /* "cupyx/cusolver.pyx":728
 * 
 *     """
 *     if not check_availability('gels'):             # <<<<<<<<<<<<<<
 *         raise RuntimeError('gels is not available.')
 * 
*/
  __pyx_t_2 = NULL;
  __Pyx_GetModuleGlobalName(__pyx_t_3, __pyx_mstate_global->__pyx_n_u_check_availability); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 728, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_4 = 1;
  #if CYTHON_UNPACK_METHODS
  if (unlikely(PyMethod_Check(__pyx_t_3))) {
    __pyx_t_2 = PyMethod_GET_SELF(__pyx_t_3);
    assert(__pyx_t_2);
    PyObject* __pyx__function = PyMethod_GET_FUNCTION(__pyx_t_3);
    __Pyx_INCREF(__pyx_t_2);
    __Pyx_INCREF(__pyx__function);
    __Pyx_DECREF_SET(__pyx_t_3, __pyx__function);
    __pyx_t_4 = 0;
  }
  #endif
  {
    PyObject *__pyx_callargs[2] = {__pyx_t_2, __pyx_mstate_global->__pyx_n_u_gels};
    __pyx_t_1 = __Pyx_PyObject_FastCall((PyObject*)__pyx_t_3, __pyx_callargs+__pyx_t_4, (2-__pyx_t_4) | (__pyx_t_4*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
    __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 728, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
  }
  __pyx_t_5 = __Pyx_PyObject_IsTrue(__pyx_t_1); if (unlikely((__pyx_t_5 < 0))) __PYX_ERR(0, 728, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_6 = (!__pyx_t_5);
  if (unlikely(__pyx_t_6)) {

    /* "cupyx/cusolver.pyx":729
 *     """
 *     if not check_availability('gels'):
 *         raise RuntimeError('gels is not available.')             # <<<<<<<<<<<<<<
 * 
 *     if a.ndim != 2:
*/
    __pyx_t_3 = NULL;
    __pyx_t_4 = 1;
    {
      PyObject *__pyx_callargs[2] = {__pyx_t_3, __pyx_mstate_global->__pyx_kp_u_gels_is_not_available};
      __pyx_t_1 = __Pyx_PyObject_FastCall((PyObject*)(((PyTypeObject*)PyExc_RuntimeError)), __pyx_callargs+__pyx_t_4, (2-__pyx_t_4) | (__pyx_t_4*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
      __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
      if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 729, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
    }
    __Pyx_Raise(__pyx_t_1, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __PYX_ERR(0, 729, __pyx_L1_error)

    /* "cupyx/cusolver.pyx":728
 * 
 *     """
 *     if not check_availability('gels'):             # <<<<<<<<<<<<<<
 *         raise RuntimeError('gels is not available.')
 * 
*/
  }

  /* "cupyx/cusolver.pyx":731
 *         raise RuntimeError('gels is not available.')
 * 
 *     if a.ndim != 2:             # <<<<<<<<<<<<<<
 *         raise ValueError('a.ndim must be 2 (actual:{})'.format(a.ndim))
 *     if b.ndim == 1:
*/
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_v_a, __pyx_mstate_global->__pyx_n_u_ndim); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 731, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_6 = (__Pyx_PyLong_BoolNeObjC(__pyx_t_1, __pyx_mstate_global->__pyx_int_2, 2, 0)); if (unlikely((__pyx_t_6 < 0))) __PYX_ERR(0, 731, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  if (unlikely(__pyx_t_6)) {

    /* "cupyx/cusolver.pyx":732
 * 
 *     if a.ndim != 2:
 *         raise ValueError('a.ndim must be 2 (actual:{})'.format(a.ndim))             # <<<<<<<<<<<<<<
 *     if b.ndim == 1:
 *         nrhs = 1
*/
    __pyx_t_3 = NULL;
    __pyx_t_7 = __pyx_mstate_global->__pyx_kp_u_a_ndim_must_be_2_actual;
    __Pyx_INCREF(__pyx_t_7);
    __pyx_t_8 = __Pyx_PyObject_GetAttrStr(__pyx_v_a, __pyx_mstate_global->__pyx_n_u_ndim); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 732, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_8);
    __pyx_t_4 = 0;
    {
      PyObject *__pyx_callargs[2] = {__pyx_t_7, __pyx_t_8};
      __pyx_t_2 = __Pyx_PyObject_FastCallMethod((PyObject*)__pyx_mstate_global->__pyx_n_u_format, __pyx_callargs+__pyx_t_4, (2-__pyx_t_4) | (1*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
      __Pyx_XDECREF(__pyx_t_7); __pyx_t_7 = 0;
      __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
      if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 732, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_2);
    }
    __pyx_t_4 = 1;
    {
      PyObject *__pyx_callargs[2] = {__pyx_t_3, __pyx_t_2};
      __pyx_t_1 = __Pyx_PyObject_FastCall((PyObject*)(((PyTypeObject*)PyExc_ValueError)), __pyx_callargs+__pyx_t_4, (2-__pyx_t_4) | (__pyx_t_4*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
      __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
      __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
      if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 732, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
    }
    __Pyx_Raise(__pyx_t_1, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __PYX_ERR(0, 732, __pyx_L1_error)

    /* "cupyx/cusolver.pyx":731
 *         raise RuntimeError('gels is not available.')
 * 
 *     if a.ndim != 2:             # <<<<<<<<<<<<<<
 *         raise ValueError('a.ndim must be 2 (actual:{})'.format(a.ndim))
 *     if b.ndim == 1:
*/
  }

  /* "cupyx/cusolver.pyx":733
 *     if a.ndim != 2:
 *         raise ValueError('a.ndim must be 2 (actual:{})'.format(a.ndim))
 *     if b.ndim == 1:             # <<<<<<<<<<<<<<
 *         nrhs = 1
 *     elif b.ndim == 2:
*/
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_v_b, __pyx_mstate_global->__pyx_n_u_ndim); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 733, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_6 = (__Pyx_PyLong_BoolEqObjC(__pyx_t_1, __pyx_mstate_global->__pyx_int_1, 1, 0)); if (unlikely((__pyx_t_6 < 0))) __PYX_ERR(0, 733, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  if (__pyx_t_6) {

    /* "cupyx/cusolver.pyx":734
 *         raise ValueError('a.ndim must be 2 (actual:{})'.format(a.ndim))
 *     if b.ndim == 1:
 *         nrhs = 1             # <<<<<<<<<<<<<<
 *     elif b.ndim == 2:
 *         nrhs = b.shape[1]
*/
    __Pyx_INCREF(__pyx_mstate_global->__pyx_int_1);
    __pyx_v_nrhs = __pyx_mstate_global->__pyx_int_1;

    /* "cupyx/cusolver.pyx":733
 *     if a.ndim != 2:
 *         raise ValueError('a.ndim must be 2 (actual:{})'.format(a.ndim))
 *     if b.ndim == 1:             # <<<<<<<<<<<<<<
 *         nrhs = 1
 *     elif b.ndim == 2:
*/
    goto __pyx_L5;
  }

  /* "cupyx/cusolver.pyx":735
 *     if b.ndim == 1:
 *         nrhs = 1
 *     elif b.ndim == 2:             # <<<<<<<<<<<<<<
 *         nrhs = b.shape[1]
 *     else:
*/
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_v_b, __pyx_mstate_global->__pyx_n_u_ndim); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 735, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_6 = (__Pyx_PyLong_BoolEqObjC(__pyx_t_1, __pyx_mstate_global->__pyx_int_2, 2, 0)); if (unlikely((__pyx_t_6 < 0))) __PYX_ERR(0, 735, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  if (likely(__pyx_t_6)) {

    /* "cupyx/cusolver.pyx":736
 *         nrhs = 1
 *     elif b.ndim == 2:
 *         nrhs = b.shape[1]             # <<<<<<<<<<<<<<
 *     else:
 *         raise ValueError('b.ndim must be 1 or 2 (actual: {})'.format(b.ndim))
*/
    __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_v_b, __pyx_mstate_global->__pyx_n_u_shape); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 736, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_t_2 = __Pyx_GetItemInt(__pyx_t_1, 1, long, 1, __Pyx_PyLong_From_long, 0, 0, 1, 1, __Pyx_ReferenceSharing_OwnStrongReference); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 736, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __pyx_v_nrhs = __pyx_t_2;
    __pyx_t_2 = 0;

    /* "cupyx/cusolver.pyx":735
 *     if b.ndim == 1:
 *         nrhs = 1
 *     elif b.ndim == 2:             # <<<<<<<<<<<<<<
 *         nrhs = b.shape[1]
 *     else:
*/
    goto __pyx_L5;
  }

  /* "cupyx/cusolver.pyx":738
 *         nrhs = b.shape[1]
 *     else:
 *         raise ValueError('b.ndim must be 1 or 2 (actual: {})'.format(b.ndim))             # <<<<<<<<<<<<<<
 *     if a.shape[0] != b.shape[0]:
 *         raise ValueError('shape mismatch (a:{}, b:{}).'.
*/
  /*else*/ {
    __pyx_t_1 = NULL;
    __pyx_t_8 = __pyx_mstate_global->__pyx_kp_u_b_ndim_must_be_1_or_2_actual_2;
    __Pyx_INCREF(__pyx_t_8);
    __pyx_t_7 = __Pyx_PyObject_GetAttrStr(__pyx_v_b, __pyx_mstate_global->__pyx_n_u_ndim); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 738, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_7);
    __pyx_t_4 = 0;
    {
      PyObject *__pyx_callargs[2] = {__pyx_t_8, __pyx_t_7};
      __pyx_t_3 = __Pyx_PyObject_FastCallMethod((PyObject*)__pyx_mstate_global->__pyx_n_u_format, __pyx_callargs+__pyx_t_4, (2-__pyx_t_4) | (1*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
      __Pyx_XDECREF(__pyx_t_8); __pyx_t_8 = 0;
      __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
      if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 738, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_3);
    }
    __pyx_t_4 = 1;
    {
      PyObject *__pyx_callargs[2] = {__pyx_t_1, __pyx_t_3};
      __pyx_t_2 = __Pyx_PyObject_FastCall((PyObject*)(((PyTypeObject*)PyExc_ValueError)), __pyx_callargs+__pyx_t_4, (2-__pyx_t_4) | (__pyx_t_4*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
      __Pyx_XDECREF(__pyx_t_1); __pyx_t_1 = 0;
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
      if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 738, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_2);
    }
    __Pyx_Raise(__pyx_t_2, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __PYX_ERR(0, 738, __pyx_L1_error)
  }
  __pyx_L5:;

  /* "cupyx/cusolver.pyx":739
 *     else:
 *         raise ValueError('b.ndim must be 1 or 2 (actual: {})'.format(b.ndim))
 *     if a.shape[0] != b.shape[0]:             # <<<<<<<<<<<<<<
 *         raise ValueError('shape mismatch (a:{}, b:{}).'.
 *                          format(a.shape, b.shape))
*/
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_v_a, __pyx_mstate_global->__pyx_n_u_shape); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 739, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = __Pyx_GetItemInt(__pyx_t_2, 0, long, 1, __Pyx_PyLong_From_long, 0, 0, 1, 1, __Pyx_ReferenceSharing_OwnStrongReference); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 739, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_v_b, __pyx_mstate_global->__pyx_n_u_shape); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 739, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_1 = __Pyx_GetItemInt(__pyx_t_2, 0, long, 1, __Pyx_PyLong_From_long, 0, 0, 1, 1, __Pyx_ReferenceSharing_OwnStrongReference); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 739, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_2 = PyObject_RichCompare(__pyx_t_3, __pyx_t_1, Py_NE); __Pyx_XGOTREF(__pyx_t_2); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 739, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_6 = __Pyx_PyObject_IsTrue(__pyx_t_2); if (unlikely((__pyx_t_6 < 0))) __PYX_ERR(0, 739, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  if (unlikely(__pyx_t_6)) {

    /* "cupyx/cusolver.pyx":740
 *         raise ValueError('b.ndim must be 1 or 2 (actual: {})'.format(b.ndim))
 *     if a.shape[0] != b.shape[0]:
 *         raise ValueError('shape mismatch (a:{}, b:{}).'.             # <<<<<<<<<<<<<<
 *                          format(a.shape, b.shape))
 *     if a.dtype != b.dtype:
*/
    __pyx_t_1 = NULL;

    /* "cupyx/cusolver.pyx":741
 *     if a.shape[0] != b.shape[0]:
 *         raise ValueError('shape mismatch (a:{}, b:{}).'.
 *                          format(a.shape, b.shape))             # <<<<<<<<<<<<<<
 *     if a.dtype != b.dtype:
 *         raise ValueError('dtype mismatch (a:{}, b:{}).'.
*/
    __pyx_t_7 = __pyx_mstate_global->__pyx_kp_u_shape_mismatch_a_b;
    __Pyx_INCREF(__pyx_t_7);
    __pyx_t_8 = __Pyx_PyObject_GetAttrStr(__pyx_v_a, __pyx_mstate_global->__pyx_n_u_shape); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 741, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_8);
    __pyx_t_9 = __Pyx_PyObject_GetAttrStr(__pyx_v_b, __pyx_mstate_global->__pyx_n_u_shape); if (unlikely(!__pyx_t_9)) __PYX_ERR(0, 741, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_9);
    __pyx_t_4 = 0;
    {
      PyObject *__pyx_callargs[3] = {__pyx_t_7, __pyx_t_8, __pyx_t_9};
      __pyx_t_3 = __Pyx_PyObject_FastCallMethod((PyObject*)__pyx_mstate_global->__pyx_n_u_format, __pyx_callargs+__pyx_t_4, (3-__pyx_t_4) | (1*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
      __Pyx_XDECREF(__pyx_t_7); __pyx_t_7 = 0;
      __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
      __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
      if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 741, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_3);
    }
    __pyx_t_4 = 1;
    {
      PyObject *__pyx_callargs[2] = {__pyx_t_1, __pyx_t_3};
      __pyx_t_2 = __Pyx_PyObject_FastCall((PyObject*)(((PyTypeObject*)PyExc_ValueError)), __pyx_callargs+__pyx_t_4, (2-__pyx_t_4) | (__pyx_t_4*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
      __Pyx_XDECREF(__pyx_t_1); __pyx_t_1 = 0;
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
      if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 740, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_2);
    }
    __Pyx_Raise(__pyx_t_2, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __PYX_ERR(0, 740, __pyx_L1_error)

    /* "cupyx/cusolver.pyx":739
 *     else:
 *         raise ValueError('b.ndim must be 1 or 2 (actual: {})'.format(b.ndim))
 *     if a.shape[0] != b.shape[0]:             # <<<<<<<<<<<<<<
 *         raise ValueError('shape mismatch (a:{}, b:{}).'.
 *                          format(a.shape, b.shape))
*/
  }

  /* "cupyx/cusolver.pyx":742
 *         raise ValueError('shape mismatch (a:{}, b:{}).'.
 *                          format(a.shape, b.shape))
 *     if a.dtype != b.dtype:             # <<<<<<<<<<<<<<
 *         raise ValueError('dtype mismatch (a:{}, b:{}).'.
 *                          format(a.dtype, b.dtype))
*/
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_v_a, __pyx_mstate_global->__pyx_n_u_dtype); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 742, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_v_b, __pyx_mstate_global->__pyx_n_u_dtype); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 742, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_1 = PyObject_RichCompare(__pyx_t_2, __pyx_t_3, Py_NE); __Pyx_XGOTREF(__pyx_t_1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 742, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_t_6 = __Pyx_PyObject_IsTrue(__pyx_t_1); if (unlikely((__pyx_t_6 < 0))) __PYX_ERR(0, 742, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  if (unlikely(__pyx_t_6)) {

    /* "cupyx/cusolver.pyx":743
 *                          format(a.shape, b.shape))
 *     if a.dtype != b.dtype:
 *         raise ValueError('dtype mismatch (a:{}, b:{}).'.             # <<<<<<<<<<<<<<
 *                          format(a.dtype, b.dtype))
 * 
*/
    __pyx_t_3 = NULL;

    /* "cupyx/cusolver.pyx":744
 *     if a.dtype != b.dtype:
 *         raise ValueError('dtype mismatch (a:{}, b:{}).'.
 *                          format(a.dtype, b.dtype))             # <<<<<<<<<<<<<<
 * 
 *     m, n = a.shape
*/
    __pyx_t_9 = __pyx_mstate_global->__pyx_kp_u_dtype_mismatch_a_b;
    __Pyx_INCREF(__pyx_t_9);
    __pyx_t_8 = __Pyx_PyObject_GetAttrStr(__pyx_v_a, __pyx_mstate_global->__pyx_n_u_dtype); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 744, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_8);
    __pyx_t_7 = __Pyx_PyObject_GetAttrStr(__pyx_v_b, __pyx_mstate_global->__pyx_n_u_dtype); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 744, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_7);
    __pyx_t_4 = 0;
    {
      PyObject *__pyx_callargs[3] = {__pyx_t_9, __pyx_t_8, __pyx_t_7};
      __pyx_t_2 = __Pyx_PyObject_FastCallMethod((PyObject*)__pyx_mstate_global->__pyx_n_u_format, __pyx_callargs+__pyx_t_4, (3-__pyx_t_4) | (1*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
      __Pyx_XDECREF(__pyx_t_9); __pyx_t_9 = 0;
      __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
      __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
      if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 744, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_2);
    }
    __pyx_t_4 = 1;
    {
      PyObject *__pyx_callargs[2] = {__pyx_t_3, __pyx_t_2};
      __pyx_t_1 = __Pyx_PyObject_FastCall((PyObject*)(((PyTypeObject*)PyExc_ValueError)), __pyx_callargs+__pyx_t_4, (2-__pyx_t_4) | (__pyx_t_4*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
      __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
      __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
      if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 743, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
    }
    __Pyx_Raise(__pyx_t_1, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __PYX_ERR(0, 743, __pyx_L1_error)

    /* "cupyx/cusolver.pyx":742
 *         raise ValueError('shape mismatch (a:{}, b:{}).'.
 *                          format(a.shape, b.shape))
 *     if a.dtype != b.dtype:             # <<<<<<<<<<<<<<
 *         raise ValueError('dtype mismatch (a:{}, b:{}).'.
 *                          format(a.dtype, b.dtype))
*/
  }

  /* "cupyx/cusolver.pyx":746
 *                          format(a.dtype, b.dtype))
 * 
 *     m, n = a.shape             # <<<<<<<<<<<<<<
 *     if m < n:
 *         raise ValueError('m must be equal to or greater than n.')
*/
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_v_a, __pyx_mstate_global->__pyx_n_u_shape); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 746, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if ((likely(PyTuple_CheckExact(__pyx_t_1))) || (PyList_CheckExact(__pyx_t_1))) {
    PyObject* sequence = __pyx_t_1;
    Py_ssize_t size = __Pyx_PySequence_SIZE(sequence);
    if (unlikely(size != 2)) {
      if (size > 2) __Pyx_RaiseTooManyValuesError(2);
      else if (size >= 0) __Pyx_RaiseNeedMoreValuesError(size);
      __PYX_ERR(0, 746, __pyx_L1_error)
    }
    #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
    if (likely(PyTuple_CheckExact(sequence))) {
      __pyx_t_2 = PyTuple_GET_ITEM(sequence, 0);
      __Pyx_INCREF(__pyx_t_2);
      __pyx_t_3 = PyTuple_GET_ITEM(sequence, 1);
      __Pyx_INCREF(__pyx_t_3);
    } else {
      __pyx_t_2 = __Pyx_PyList_GetItemRefFast(sequence, 0, __Pyx_ReferenceSharing_SharedReference);
      if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 746, __pyx_L1_error)
      __Pyx_XGOTREF(__pyx_t_2);
      __pyx_t_3 = __Pyx_PyList_GetItemRefFast(sequence, 1, __Pyx_ReferenceSharing_SharedReference);
      if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 746, __pyx_L1_error)
      __Pyx_XGOTREF(__pyx_t_3);
    }
    #else
    __pyx_t_2 = __Pyx_PySequence_ITEM(sequence, 0); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 746, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_3 = __Pyx_PySequence_ITEM(sequence, 1); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 746, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    #endif
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  } else {
    Py_ssize_t index = -1;
    __pyx_t_7 = PyObject_GetIter(__pyx_t_1); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 746, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_7);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __pyx_t_10 = (CYTHON_COMPILING_IN_LIMITED_API) ? PyIter_Next : __Pyx_PyObject_GetIterNextFunc(__pyx_t_7);
    index = 0; __pyx_t_2 = __pyx_t_10(__pyx_t_7); if (unlikely(!__pyx_t_2)) goto __pyx_L8_unpacking_failed;
    __Pyx_GOTREF(__pyx_t_2);
    index = 1; __pyx_t_3 = __pyx_t_10(__pyx_t_7); if (unlikely(!__pyx_t_3)) goto __pyx_L8_unpacking_failed;
    __Pyx_GOTREF(__pyx_t_3);
    if (__Pyx_IternextUnpackEndCheck(__pyx_t_10(__pyx_t_7), 2) < (0)) __PYX_ERR(0, 746, __pyx_L1_error)
    __pyx_t_10 = NULL;
    __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
    goto __pyx_L9_unpacking_done;
    __pyx_L8_unpacking_failed:;
    __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
    __pyx_t_10 = NULL;
    if (__Pyx_IterFinish() == 0) __Pyx_RaiseNeedMoreValuesError(index);
    __PYX_ERR(0, 746, __pyx_L1_error)
    __pyx_L9_unpacking_done:;
  }
  __pyx_v_m = __pyx_t_2;
  __pyx_t_2 = 0;
  __pyx_v_n = __pyx_t_3;
  __pyx_t_3 = 0;

  /* "cupyx/cusolver.pyx":747
 * 
 *     m, n = a.shape
 *     if m < n:             # <<<<<<<<<<<<<<
 *         raise ValueError('m must be equal to or greater than n.')
 *     max_mn = max(m, n)
*/
  __pyx_t_1 = PyObject_RichCompare(__pyx_v_m, __pyx_v_n, Py_LT); __Pyx_XGOTREF(__pyx_t_1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 747, __pyx_L1_error)
  __pyx_t_6 = __Pyx_PyObject_IsTrue(__pyx_t_1); if (unlikely((__pyx_t_6 < 0))) __PYX_ERR(0, 747, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  if (unlikely(__pyx_t_6)) {

    /* "cupyx/cusolver.pyx":748
 *     m, n = a.shape
 *     if m < n:
 *         raise ValueError('m must be equal to or greater than n.')             # <<<<<<<<<<<<<<
 *     max_mn = max(m, n)
 *     b_ndim = b.ndim
*/
    __pyx_t_3 = NULL;
    __pyx_t_4 = 1;
    {
      PyObject *__pyx_callargs[2] = {__pyx_t_3, __pyx_mstate_global->__pyx_kp_u_m_must_be_equal_to_or_greater_th};
      __pyx_t_1 = __Pyx_PyObject_FastCall((PyObject*)(((PyTypeObject*)PyExc_ValueError)), __pyx_callargs+__pyx_t_4, (2-__pyx_t_4) | (__pyx_t_4*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
      __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
      if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 748, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
    }
    __Pyx_Raise(__pyx_t_1, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __PYX_ERR(0, 748, __pyx_L1_error)

    /* "cupyx/cusolver.pyx":747
 * 
 *     m, n = a.shape
 *     if m < n:             # <<<<<<<<<<<<<<
 *         raise ValueError('m must be equal to or greater than n.')
 *     max_mn = max(m, n)
*/
  }

  /* "cupyx/cusolver.pyx":749
 *     if m < n:
 *         raise ValueError('m must be equal to or greater than n.')
 *     max_mn = max(m, n)             # <<<<<<<<<<<<<<
 *     b_ndim = b.ndim
 * 
*/
  __Pyx_INCREF(__pyx_v_n);
  __pyx_t_1 = __pyx_v_n;
  __Pyx_INCREF(__pyx_v_m);
  __pyx_t_3 = __pyx_v_m;
  __pyx_t_7 = PyObject_RichCompare(__pyx_t_1, __pyx_t_3, Py_GT); __Pyx_XGOTREF(__pyx_t_7); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 749, __pyx_L1_error)
  __pyx_t_6 = __Pyx_PyObject_IsTrue(__pyx_t_7); if (unlikely((__pyx_t_6 < 0))) __PYX_ERR(0, 749, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
  if (__pyx_t_6) {
    __Pyx_INCREF(__pyx_t_1);
    __pyx_t_2 = __pyx_t_1;
  } else {
    __Pyx_INCREF(__pyx_t_3);
    __pyx_t_2 = __pyx_t_3;
  }
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_1 = __pyx_t_2;
  __Pyx_INCREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_v_max_mn = __pyx_t_1;
  __pyx_t_1 = 0;

  /* "cupyx/cusolver.pyx":750
 *         raise ValueError('m must be equal to or greater than n.')
 *     max_mn = max(m, n)
 *     b_ndim = b.ndim             # <<<<<<<<<<<<<<
 * 
 *     compute_type = _linalg.get_compute_type(a.dtype)
*/
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_v_b, __pyx_mstate_global->__pyx_n_u_ndim); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 750, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_v_b_ndim = __pyx_t_1;
  __pyx_t_1 = 0;

  /* "cupyx/cusolver.pyx":752
 *     b_ndim = b.ndim
 * 
 *     compute_type = _linalg.get_compute_type(a.dtype)             # <<<<<<<<<<<<<<
 *     if a.dtype.char in 'fd':
 *         if a.dtype.char == 'f':
*/
  __pyx_t_2 = NULL;
  __Pyx_GetModuleGlobalName(__pyx_t_3, __pyx_mstate_global->__pyx_n_u_linalg_2); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 752, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_7 = __Pyx_PyObject_GetAttrStr(__pyx_t_3, __pyx_mstate_global->__pyx_n_u_get_compute_type); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 752, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_7);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_v_a, __pyx_mstate_global->__pyx_n_u_dtype); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 752, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_4 = 1;
  #if CYTHON_UNPACK_METHODS
  if (unlikely(PyMethod_Check(__pyx_t_7))) {
    __pyx_t_2 = PyMethod_GET_SELF(__pyx_t_7);
    assert(__pyx_t_2);
    PyObject* __pyx__function = PyMethod_GET_FUNCTION(__pyx_t_7);
    __Pyx_INCREF(__pyx_t_2);
    __Pyx_INCREF(__pyx__function);
    __Pyx_DECREF_SET(__pyx_t_7, __pyx__function);
    __pyx_t_4 = 0;
  }
  #endif
  {
    PyObject *__pyx_callargs[2] = {__pyx_t_2, __pyx_t_3};
    __pyx_t_1 = __Pyx_PyObject_FastCall((PyObject*)__pyx_t_7, __pyx_callargs+__pyx_t_4, (2-__pyx_t_4) | (__pyx_t_4*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
    __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
    if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 752, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
  }
  __pyx_v_compute_type = __pyx_t_1;
  __pyx_t_1 = 0;

  /* "cupyx/cusolver.pyx":753
 * 
 *     compute_type = _linalg.get_compute_type(a.dtype)
 *     if a.dtype.char in 'fd':             # <<<<<<<<<<<<<<
 *         if a.dtype.char == 'f':
 *             t1 = t2 = 's'
*/
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_v_a, __pyx_mstate_global->__pyx_n_u_dtype); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 753, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_7 = __Pyx_PyObject_GetAttrStr(__pyx_t_1, __pyx_mstate_global->__pyx_n_u_char); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 753, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_7);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_6 = (__Pyx_PyUnicode_ContainsTF(__pyx_t_7, __pyx_mstate_global->__pyx_n_u_fd, Py_EQ)); if (unlikely((__pyx_t_6 < 0))) __PYX_ERR(0, 753, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
  if (__pyx_t_6) {

    /* "cupyx/cusolver.pyx":754
 *     compute_type = _linalg.get_compute_type(a.dtype)
 *     if a.dtype.char in 'fd':
 *         if a.dtype.char == 'f':             # <<<<<<<<<<<<<<
 *             t1 = t2 = 's'
 *         else:
*/
    __pyx_t_7 = __Pyx_PyObject_GetAttrStr(__pyx_v_a, __pyx_mstate_global->__pyx_n_u_dtype); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 754, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_7);
    __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_t_7, __pyx_mstate_global->__pyx_n_u_char); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 754, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
    __pyx_t_6 = (__Pyx_PyUnicode_Equals(__pyx_t_1, __pyx_mstate_global->__pyx_n_u_f, Py_EQ)); if (unlikely((__pyx_t_6 < 0))) __PYX_ERR(0, 754, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    if (__pyx_t_6) {

      /* "cupyx/cusolver.pyx":755
 *     if a.dtype.char in 'fd':
 *         if a.dtype.char == 'f':
 *             t1 = t2 = 's'             # <<<<<<<<<<<<<<
 *         else:
 *             t1 = t2 = 'd'
*/
      __Pyx_INCREF(__pyx_mstate_global->__pyx_n_u_s);
      __pyx_v_t1 = __pyx_mstate_global->__pyx_n_u_s;
      __Pyx_INCREF(__pyx_mstate_global->__pyx_n_u_s);
      __pyx_v_t2 = __pyx_mstate_global->__pyx_n_u_s;

      /* "cupyx/cusolver.pyx":754
 *     compute_type = _linalg.get_compute_type(a.dtype)
 *     if a.dtype.char in 'fd':
 *         if a.dtype.char == 'f':             # <<<<<<<<<<<<<<
 *             t1 = t2 = 's'
 *         else:
*/
      goto __pyx_L12;
    }

    /* "cupyx/cusolver.pyx":757
 *             t1 = t2 = 's'
 *         else:
 *             t1 = t2 = 'd'             # <<<<<<<<<<<<<<
 *         if compute_type == _linalg.COMPUTE_TYPE_FP16:
 *             t2 = 'h'
*/
    /*else*/ {
      __Pyx_INCREF(__pyx_mstate_global->__pyx_n_u_d);
      __pyx_v_t1 = __pyx_mstate_global->__pyx_n_u_d;
      __Pyx_INCREF(__pyx_mstate_global->__pyx_n_u_d);
      __pyx_v_t2 = __pyx_mstate_global->__pyx_n_u_d;
    }
    __pyx_L12:;

    /* "cupyx/cusolver.pyx":758
 *         else:
 *             t1 = t2 = 'd'
 *         if compute_type == _linalg.COMPUTE_TYPE_FP16:             # <<<<<<<<<<<<<<
 *             t2 = 'h'
 *         elif compute_type == _linalg.COMPUTE_TYPE_TF32:
*/
    __Pyx_GetModuleGlobalName(__pyx_t_1, __pyx_mstate_global->__pyx_n_u_linalg_2); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 758, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_t_7 = __Pyx_PyObject_GetAttrStr(__pyx_t_1, __pyx_mstate_global->__pyx_n_u_COMPUTE_TYPE_FP16); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 758, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_7);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __pyx_t_1 = PyObject_RichCompare(__pyx_v_compute_type, __pyx_t_7, Py_EQ); __Pyx_XGOTREF(__pyx_t_1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 758, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
    __pyx_t_6 = __Pyx_PyObject_IsTrue(__pyx_t_1); if (unlikely((__pyx_t_6 < 0))) __PYX_ERR(0, 758, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    if (__pyx_t_6) {

      /* "cupyx/cusolver.pyx":759
 *             t1 = t2 = 'd'
 *         if compute_type == _linalg.COMPUTE_TYPE_FP16:
 *             t2 = 'h'             # <<<<<<<<<<<<<<
 *         elif compute_type == _linalg.COMPUTE_TYPE_TF32:
 *             t2 = 'x'
*/
      __Pyx_INCREF(__pyx_mstate_global->__pyx_n_u_h);
      __Pyx_DECREF_SET(__pyx_v_t2, __pyx_mstate_global->__pyx_n_u_h);

      /* "cupyx/cusolver.pyx":758
 *         else:
 *             t1 = t2 = 'd'
 *         if compute_type == _linalg.COMPUTE_TYPE_FP16:             # <<<<<<<<<<<<<<
 *             t2 = 'h'
 *         elif compute_type == _linalg.COMPUTE_TYPE_TF32:
*/
      goto __pyx_L13;
    }

    /* "cupyx/cusolver.pyx":760
 *         if compute_type == _linalg.COMPUTE_TYPE_FP16:
 *             t2 = 'h'
 *         elif compute_type == _linalg.COMPUTE_TYPE_TF32:             # <<<<<<<<<<<<<<
 *             t2 = 'x'
 *         elif compute_type == _linalg.COMPUTE_TYPE_FP32:
*/
    __Pyx_GetModuleGlobalName(__pyx_t_1, __pyx_mstate_global->__pyx_n_u_linalg_2); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 760, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_t_7 = __Pyx_PyObject_GetAttrStr(__pyx_t_1, __pyx_mstate_global->__pyx_n_u_COMPUTE_TYPE_TF32); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 760, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_7);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __pyx_t_1 = PyObject_RichCompare(__pyx_v_compute_type, __pyx_t_7, Py_EQ); __Pyx_XGOTREF(__pyx_t_1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 760, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
    __pyx_t_6 = __Pyx_PyObject_IsTrue(__pyx_t_1); if (unlikely((__pyx_t_6 < 0))) __PYX_ERR(0, 760, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    if (__pyx_t_6) {

      /* "cupyx/cusolver.pyx":761
 *             t2 = 'h'
 *         elif compute_type == _linalg.COMPUTE_TYPE_TF32:
 *             t2 = 'x'             # <<<<<<<<<<<<<<
 *         elif compute_type == _linalg.COMPUTE_TYPE_FP32:
 *             t2 = 's'
*/
      __Pyx_INCREF(__pyx_mstate_global->__pyx_n_u_x);
      __Pyx_DECREF_SET(__pyx_v_t2, __pyx_mstate_global->__pyx_n_u_x);

      /* "cupyx/cusolver.pyx":760
 *         if compute_type == _linalg.COMPUTE_TYPE_FP16:
 *             t2 = 'h'
 *         elif compute_type == _linalg.COMPUTE_TYPE_TF32:             # <<<<<<<<<<<<<<
 *             t2 = 'x'
 *         elif compute_type == _linalg.COMPUTE_TYPE_FP32:
*/
      goto __pyx_L13;
    }

    /* "cupyx/cusolver.pyx":762
 *         elif compute_type == _linalg.COMPUTE_TYPE_TF32:
 *             t2 = 'x'
 *         elif compute_type == _linalg.COMPUTE_TYPE_FP32:             # <<<<<<<<<<<<<<
 *             t2 = 's'
 *     elif a.dtype.char in 'FD':
*/
    __Pyx_GetModuleGlobalName(__pyx_t_1, __pyx_mstate_global->__pyx_n_u_linalg_2); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 762, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_t_7 = __Pyx_PyObject_GetAttrStr(__pyx_t_1, __pyx_mstate_global->__pyx_n_u_COMPUTE_TYPE_FP32); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 762, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_7);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __pyx_t_1 = PyObject_RichCompare(__pyx_v_compute_type, __pyx_t_7, Py_EQ); __Pyx_XGOTREF(__pyx_t_1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 762, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
    __pyx_t_6 = __Pyx_PyObject_IsTrue(__pyx_t_1); if (unlikely((__pyx_t_6 < 0))) __PYX_ERR(0, 762, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    if (__pyx_t_6) {

      /* "cupyx/cusolver.pyx":763
 *             t2 = 'x'
 *         elif compute_type == _linalg.COMPUTE_TYPE_FP32:
 *             t2 = 's'             # <<<<<<<<<<<<<<
 *     elif a.dtype.char in 'FD':
 *         if a.dtype.char == 'F':
*/
      __Pyx_INCREF(__pyx_mstate_global->__pyx_n_u_s);
      __Pyx_DECREF_SET(__pyx_v_t2, __pyx_mstate_global->__pyx_n_u_s);

      /* "cupyx/cusolver.pyx":762
 *         elif compute_type == _linalg.COMPUTE_TYPE_TF32:
 *             t2 = 'x'
 *         elif compute_type == _linalg.COMPUTE_TYPE_FP32:             # <<<<<<<<<<<<<<
 *             t2 = 's'
 *     elif a.dtype.char in 'FD':
*/
    }
    __pyx_L13:;

    /* "cupyx/cusolver.pyx":753
 * 
 *     compute_type = _linalg.get_compute_type(a.dtype)
 *     if a.dtype.char in 'fd':             # <<<<<<<<<<<<<<
 *         if a.dtype.char == 'f':
 *             t1 = t2 = 's'
*/
    goto __pyx_L11;
  }

  /* "cupyx/cusolver.pyx":764
 *         elif compute_type == _linalg.COMPUTE_TYPE_FP32:
 *             t2 = 's'
 *     elif a.dtype.char in 'FD':             # <<<<<<<<<<<<<<
 *         if a.dtype.char == 'F':
 *             t1 = t2 = 'c'
*/
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_v_a, __pyx_mstate_global->__pyx_n_u_dtype); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 764, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_7 = __Pyx_PyObject_GetAttrStr(__pyx_t_1, __pyx_mstate_global->__pyx_n_u_char); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 764, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_7);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_6 = (__Pyx_PyUnicode_ContainsTF(__pyx_t_7, __pyx_mstate_global->__pyx_n_u_FD, Py_EQ)); if (unlikely((__pyx_t_6 < 0))) __PYX_ERR(0, 764, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
  if (likely(__pyx_t_6)) {

    /* "cupyx/cusolver.pyx":765
 *             t2 = 's'
 *     elif a.dtype.char in 'FD':
 *         if a.dtype.char == 'F':             # <<<<<<<<<<<<<<
 *             t1 = t2 = 'c'
 *         else:
*/
    __pyx_t_7 = __Pyx_PyObject_GetAttrStr(__pyx_v_a, __pyx_mstate_global->__pyx_n_u_dtype); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 765, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_7);
    __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_t_7, __pyx_mstate_global->__pyx_n_u_char); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 765, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
    __pyx_t_6 = (__Pyx_PyUnicode_Equals(__pyx_t_1, __pyx_mstate_global->__pyx_n_u_F, Py_EQ)); if (unlikely((__pyx_t_6 < 0))) __PYX_ERR(0, 765, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    if (__pyx_t_6) {

      /* "cupyx/cusolver.pyx":766
 *     elif a.dtype.char in 'FD':
 *         if a.dtype.char == 'F':
 *             t1 = t2 = 'c'             # <<<<<<<<<<<<<<
 *         else:
 *             t1 = t2 = 'z'
*/
      __Pyx_INCREF(__pyx_mstate_global->__pyx_n_u_c);
      __pyx_v_t1 = __pyx_mstate_global->__pyx_n_u_c;
      __Pyx_INCREF(__pyx_mstate_global->__pyx_n_u_c);
      __pyx_v_t2 = __pyx_mstate_global->__pyx_n_u_c;

      /* "cupyx/cusolver.pyx":765
 *             t2 = 's'
 *     elif a.dtype.char in 'FD':
 *         if a.dtype.char == 'F':             # <<<<<<<<<<<<<<
 *             t1 = t2 = 'c'
 *         else:
*/
      goto __pyx_L14;
    }

    /* "cupyx/cusolver.pyx":768
 *             t1 = t2 = 'c'
 *         else:
 *             t1 = t2 = 'z'             # <<<<<<<<<<<<<<
 *         if compute_type == _linalg.COMPUTE_TYPE_FP16:
 *             t2 = 'k'
*/
    /*else*/ {
      __Pyx_INCREF(__pyx_mstate_global->__pyx_n_u_z);
      __pyx_v_t1 = __pyx_mstate_global->__pyx_n_u_z;
      __Pyx_INCREF(__pyx_mstate_global->__pyx_n_u_z);
      __pyx_v_t2 = __pyx_mstate_global->__pyx_n_u_z;
    }
    __pyx_L14:;

    /* "cupyx/cusolver.pyx":769
 *         else:
 *             t1 = t2 = 'z'
 *         if compute_type == _linalg.COMPUTE_TYPE_FP16:             # <<<<<<<<<<<<<<
 *             t2 = 'k'
 *         elif compute_type == _linalg.COMPUTE_TYPE_TF32:
*/
    __Pyx_GetModuleGlobalName(__pyx_t_1, __pyx_mstate_global->__pyx_n_u_linalg_2); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 769, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_t_7 = __Pyx_PyObject_GetAttrStr(__pyx_t_1, __pyx_mstate_global->__pyx_n_u_COMPUTE_TYPE_FP16); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 769, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_7);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __pyx_t_1 = PyObject_RichCompare(__pyx_v_compute_type, __pyx_t_7, Py_EQ); __Pyx_XGOTREF(__pyx_t_1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 769, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
    __pyx_t_6 = __Pyx_PyObject_IsTrue(__pyx_t_1); if (unlikely((__pyx_t_6 < 0))) __PYX_ERR(0, 769, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    if (__pyx_t_6) {

      /* "cupyx/cusolver.pyx":770
 *             t1 = t2 = 'z'
 *         if compute_type == _linalg.COMPUTE_TYPE_FP16:
 *             t2 = 'k'             # <<<<<<<<<<<<<<
 *         elif compute_type == _linalg.COMPUTE_TYPE_TF32:
 *             t2 = 'y'
*/
      __Pyx_INCREF(__pyx_mstate_global->__pyx_n_u_k);
      __Pyx_DECREF_SET(__pyx_v_t2, __pyx_mstate_global->__pyx_n_u_k);

      /* "cupyx/cusolver.pyx":769
 *         else:
 *             t1 = t2 = 'z'
 *         if compute_type == _linalg.COMPUTE_TYPE_FP16:             # <<<<<<<<<<<<<<
 *             t2 = 'k'
 *         elif compute_type == _linalg.COMPUTE_TYPE_TF32:
*/
      goto __pyx_L15;
    }

    /* "cupyx/cusolver.pyx":771
 *         if compute_type == _linalg.COMPUTE_TYPE_FP16:
 *             t2 = 'k'
 *         elif compute_type == _linalg.COMPUTE_TYPE_TF32:             # <<<<<<<<<<<<<<
 *             t2 = 'y'
 *         elif compute_type == _linalg.COMPUTE_TYPE_FP32:
*/
    __Pyx_GetModuleGlobalName(__pyx_t_1, __pyx_mstate_global->__pyx_n_u_linalg_2); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 771, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_t_7 = __Pyx_PyObject_GetAttrStr(__pyx_t_1, __pyx_mstate_global->__pyx_n_u_COMPUTE_TYPE_TF32); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 771, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_7);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __pyx_t_1 = PyObject_RichCompare(__pyx_v_compute_type, __pyx_t_7, Py_EQ); __Pyx_XGOTREF(__pyx_t_1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 771, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
    __pyx_t_6 = __Pyx_PyObject_IsTrue(__pyx_t_1); if (unlikely((__pyx_t_6 < 0))) __PYX_ERR(0, 771, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    if (__pyx_t_6) {

      /* "cupyx/cusolver.pyx":772
 *             t2 = 'k'
 *         elif compute_type == _linalg.COMPUTE_TYPE_TF32:
 *             t2 = 'y'             # <<<<<<<<<<<<<<
 *         elif compute_type == _linalg.COMPUTE_TYPE_FP32:
 *             t2 = 'c'
*/
      __Pyx_INCREF(__pyx_mstate_global->__pyx_n_u_y);
      __Pyx_DECREF_SET(__pyx_v_t2, __pyx_mstate_global->__pyx_n_u_y);

      /* "cupyx/cusolver.pyx":771
 *         if compute_type == _linalg.COMPUTE_TYPE_FP16:
 *             t2 = 'k'
 *         elif compute_type == _linalg.COMPUTE_TYPE_TF32:             # <<<<<<<<<<<<<<
 *             t2 = 'y'
 *         elif compute_type == _linalg.COMPUTE_TYPE_FP32:
*/
      goto __pyx_L15;
    }

    /* "cupyx/cusolver.pyx":773
 *         elif compute_type == _linalg.COMPUTE_TYPE_TF32:
 *             t2 = 'y'
 *         elif compute_type == _linalg.COMPUTE_TYPE_FP32:             # <<<<<<<<<<<<<<
 *             t2 = 'c'
 *     else:
*/
    __Pyx_GetModuleGlobalName(__pyx_t_1, __pyx_mstate_global->__pyx_n_u_linalg_2); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 773, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_t_7 = __Pyx_PyObject_GetAttrStr(__pyx_t_1, __pyx_mstate_global->__pyx_n_u_COMPUTE_TYPE_FP32); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 773, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_7);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __pyx_t_1 = PyObject_RichCompare(__pyx_v_compute_type, __pyx_t_7, Py_EQ); __Pyx_XGOTREF(__pyx_t_1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 773, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
    __pyx_t_6 = __Pyx_PyObject_IsTrue(__pyx_t_1); if (unlikely((__pyx_t_6 < 0))) __PYX_ERR(0, 773, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    if (__pyx_t_6) {

      /* "cupyx/cusolver.pyx":774
 *             t2 = 'y'
 *         elif compute_type == _linalg.COMPUTE_TYPE_FP32:
 *             t2 = 'c'             # <<<<<<<<<<<<<<
 *     else:
 *         raise ValueError('unsupported dtype (actual:{})'.format(a.dtype))
*/
      __Pyx_INCREF(__pyx_mstate_global->__pyx_n_u_c);
      __Pyx_DECREF_SET(__pyx_v_t2, __pyx_mstate_global->__pyx_n_u_c);

      /* "cupyx/cusolver.pyx":773
 *         elif compute_type == _linalg.COMPUTE_TYPE_TF32:
 *             t2 = 'y'
 *         elif compute_type == _linalg.COMPUTE_TYPE_FP32:             # <<<<<<<<<<<<<<
 *             t2 = 'c'
 *     else:
*/
    }
    __pyx_L15:;

    /* "cupyx/cusolver.pyx":764
 *         elif compute_type == _linalg.COMPUTE_TYPE_FP32:
 *             t2 = 's'
 *     elif a.dtype.char in 'FD':             # <<<<<<<<<<<<<<
 *         if a.dtype.char == 'F':
 *             t1 = t2 = 'c'
*/
    goto __pyx_L11;
  }

  /* "cupyx/cusolver.pyx":776
 *             t2 = 'c'
 *     else:
 *         raise ValueError('unsupported dtype (actual:{})'.format(a.dtype))             # <<<<<<<<<<<<<<
 *     solver_name = t1 + t2 + 'gels'
 *     solver = getattr(_cusolver, solver_name)
*/
  /*else*/ {
    __pyx_t_7 = NULL;
    __pyx_t_2 = __pyx_mstate_global->__pyx_kp_u_unsupported_dtype_actual;
    __Pyx_INCREF(__pyx_t_2);
    __pyx_t_8 = __Pyx_PyObject_GetAttrStr(__pyx_v_a, __pyx_mstate_global->__pyx_n_u_dtype); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 776, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_8);
    __pyx_t_4 = 0;
    {
      PyObject *__pyx_callargs[2] = {__pyx_t_2, __pyx_t_8};
      __pyx_t_3 = __Pyx_PyObject_FastCallMethod((PyObject*)__pyx_mstate_global->__pyx_n_u_format, __pyx_callargs+__pyx_t_4, (2-__pyx_t_4) | (1*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
      __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
      __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
      if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 776, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_3);
    }
    __pyx_t_4 = 1;
    {
      PyObject *__pyx_callargs[2] = {__pyx_t_7, __pyx_t_3};
      __pyx_t_1 = __Pyx_PyObject_FastCall((PyObject*)(((PyTypeObject*)PyExc_ValueError)), __pyx_callargs+__pyx_t_4, (2-__pyx_t_4) | (__pyx_t_4*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
      __Pyx_XDECREF(__pyx_t_7); __pyx_t_7 = 0;
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
      if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 776, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
    }
    __Pyx_Raise(__pyx_t_1, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __PYX_ERR(0, 776, __pyx_L1_error)
  }
  __pyx_L11:;

  /* "cupyx/cusolver.pyx":777
 *     else:
 *         raise ValueError('unsupported dtype (actual:{})'.format(a.dtype))
 *     solver_name = t1 + t2 + 'gels'             # <<<<<<<<<<<<<<
 *     solver = getattr(_cusolver, solver_name)
 *     helper = getattr(_cusolver, solver_name + '_bufferSize')
*/
  __pyx_t_1 = __Pyx_PyUnicode_Concat(__pyx_v_t1, __pyx_v_t2); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 777, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_3 = __Pyx_PyUnicode_Concat__Pyx_ReferenceSharing_OwnStrongReferenceInPlace(__pyx_t_1, __pyx_mstate_global->__pyx_n_u_gels); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 777, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v_solver_name = ((PyObject*)__pyx_t_3);
  __pyx_t_3 = 0;

  /* "cupyx/cusolver.pyx":778
 *         raise ValueError('unsupported dtype (actual:{})'.format(a.dtype))
 *     solver_name = t1 + t2 + 'gels'
 *     solver = getattr(_cusolver, solver_name)             # <<<<<<<<<<<<<<
 *     helper = getattr(_cusolver, solver_name + '_bufferSize')
 * 
*/
  __Pyx_GetModuleGlobalName(__pyx_t_3, __pyx_mstate_global->__pyx_n_u_cusolver); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 778, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_1 = __Pyx_GetAttr(__pyx_t_3, __pyx_v_solver_name); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 778, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_v_solver = __pyx_t_1;
  __pyx_t_1 = 0;

  /* "cupyx/cusolver.pyx":779
 *     solver_name = t1 + t2 + 'gels'
 *     solver = getattr(_cusolver, solver_name)
 *     helper = getattr(_cusolver, solver_name + '_bufferSize')             # <<<<<<<<<<<<<<
 * 
 *     a = a.copy(order='F')
*/
  __Pyx_GetModuleGlobalName(__pyx_t_1, __pyx_mstate_global->__pyx_n_u_cusolver); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 779, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_3 = __Pyx_PyUnicode_Concat(__pyx_v_solver_name, __pyx_mstate_global->__pyx_n_u_bufferSize); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 779, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_7 = __Pyx_GetAttr(__pyx_t_1, __pyx_t_3); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 779, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_7);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_v_helper = __pyx_t_7;
  __pyx_t_7 = 0;

  /* "cupyx/cusolver.pyx":781
 *     helper = getattr(_cusolver, solver_name + '_bufferSize')
 * 
 *     a = a.copy(order='F')             # <<<<<<<<<<<<<<
 *     org_nrhs = nrhs
 *     if m > n and nrhs == 1:
*/
  __pyx_t_3 = __pyx_v_a;
  __Pyx_INCREF(__pyx_t_3);
  __pyx_t_4 = 0;
  {
    PyObject *__pyx_callargs[2 + ((CYTHON_VECTORCALL) ? 1 : 0)] = {__pyx_t_3, NULL};
    __pyx_t_1 = __Pyx_MakeVectorcallBuilderKwds(1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 781, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    if (__Pyx_VectorcallBuilder_AddArg(__pyx_mstate_global->__pyx_n_u_order, __pyx_mstate_global->__pyx_n_u_F, __pyx_t_1, __pyx_callargs+1, 0) < (0)) __PYX_ERR(0, 781, __pyx_L1_error)
    __pyx_t_7 = __Pyx_Object_VectorcallMethod_CallFromBuilder((PyObject*)__pyx_mstate_global->__pyx_n_u_copy, __pyx_callargs+__pyx_t_4, (1-__pyx_t_4) | (1*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET), __pyx_t_1);
    __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 781, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_7);
  }
  __Pyx_DECREF_SET(__pyx_v_a, __pyx_t_7);
  __pyx_t_7 = 0;

  /* "cupyx/cusolver.pyx":782
 * 
 *     a = a.copy(order='F')
 *     org_nrhs = nrhs             # <<<<<<<<<<<<<<
 *     if m > n and nrhs == 1:
 *         # Note: this is workaround as there is bug in cusolverDn<T1><T2>gels()
*/
  __Pyx_INCREF(__pyx_v_nrhs);
  __pyx_v_org_nrhs = __pyx_v_nrhs;

  /* "cupyx/cusolver.pyx":783
 *     a = a.copy(order='F')
 *     org_nrhs = nrhs
 *     if m > n and nrhs == 1:             # <<<<<<<<<<<<<<
 *         # Note: this is workaround as there is bug in cusolverDn<T1><T2>gels()
 *         # of CUDA 11.0/11.1 and it returns CUSOLVER_STATUS_IRS_NOT_SUPPORTED
*/
  __pyx_t_7 = PyObject_RichCompare(__pyx_v_m, __pyx_v_n, Py_GT); __Pyx_XGOTREF(__pyx_t_7); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 783, __pyx_L1_error)
  __pyx_t_5 = __Pyx_PyObject_IsTrue(__pyx_t_7); if (unlikely((__pyx_t_5 < 0))) __PYX_ERR(0, 783, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
  if (__pyx_t_5) {
  } else {
    __pyx_t_6 = __pyx_t_5;
    goto __pyx_L17_bool_binop_done;
  }
  __pyx_t_5 = (__Pyx_PyLong_BoolEqObjC(__pyx_v_nrhs, __pyx_mstate_global->__pyx_int_1, 1, 0)); if (unlikely((__pyx_t_5 < 0))) __PYX_ERR(0, 783, __pyx_L1_error)
  __pyx_t_6 = __pyx_t_5;
  __pyx_L17_bool_binop_done:;
  if (__pyx_t_6) {

    /* "cupyx/cusolver.pyx":787
 *         # of CUDA 11.0/11.1 and it returns CUSOLVER_STATUS_IRS_NOT_SUPPORTED
 *         # when m > n and nrhs == 1.
 *         nrhs = 2             # <<<<<<<<<<<<<<
 *         bb = b.reshape(m, 1)
 *         b = _cupy.empty((max_mn, nrhs), dtype=a.dtype, order='F')
*/
    __Pyx_INCREF(__pyx_mstate_global->__pyx_int_2);
    __Pyx_DECREF_SET(__pyx_v_nrhs, __pyx_mstate_global->__pyx_int_2);

    /* "cupyx/cusolver.pyx":788
 *         # when m > n and nrhs == 1.
 *         nrhs = 2
 *         bb = b.reshape(m, 1)             # <<<<<<<<<<<<<<
 *         b = _cupy.empty((max_mn, nrhs), dtype=a.dtype, order='F')
 *         b[:m, :] = bb
*/
    __pyx_t_1 = __pyx_v_b;
    __Pyx_INCREF(__pyx_t_1);
    __pyx_t_4 = 0;
    {
      PyObject *__pyx_callargs[3] = {__pyx_t_1, __pyx_v_m, __pyx_mstate_global->__pyx_int_1};
      __pyx_t_7 = __Pyx_PyObject_FastCallMethod((PyObject*)__pyx_mstate_global->__pyx_n_u_reshape, __pyx_callargs+__pyx_t_4, (3-__pyx_t_4) | (1*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
      __Pyx_XDECREF(__pyx_t_1); __pyx_t_1 = 0;
      if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 788, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_7);
    }
    __pyx_v_bb = __pyx_t_7;
    __pyx_t_7 = 0;

    /* "cupyx/cusolver.pyx":789
 *         nrhs = 2
 *         bb = b.reshape(m, 1)
 *         b = _cupy.empty((max_mn, nrhs), dtype=a.dtype, order='F')             # <<<<<<<<<<<<<<
 *         b[:m, :] = bb
 *     else:
*/
    __pyx_t_1 = NULL;
    __Pyx_GetModuleGlobalName(__pyx_t_3, __pyx_mstate_global->__pyx_n_u_cupy); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 789, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_t_8 = __Pyx_PyObject_GetAttrStr(__pyx_t_3, __pyx_mstate_global->__pyx_n_u_empty); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 789, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_8);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __pyx_t_3 = PyTuple_New(2); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 789, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_INCREF(__pyx_v_max_mn);
    __Pyx_GIVEREF(__pyx_v_max_mn);
    if (__Pyx_PyTuple_SET_ITEM(__pyx_t_3, 0, __pyx_v_max_mn) != (0)) __PYX_ERR(0, 789, __pyx_L1_error);
    __Pyx_INCREF(__pyx_v_nrhs);
    __Pyx_GIVEREF(__pyx_v_nrhs);
    if (__Pyx_PyTuple_SET_ITEM(__pyx_t_3, 1, __pyx_v_nrhs) != (0)) __PYX_ERR(0, 789, __pyx_L1_error);
    __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_v_a, __pyx_mstate_global->__pyx_n_u_dtype); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 789, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_4 = 1;
    #if CYTHON_UNPACK_METHODS
    if (unlikely(PyMethod_Check(__pyx_t_8))) {
      __pyx_t_1 = PyMethod_GET_SELF(__pyx_t_8);
      assert(__pyx_t_1);
      PyObject* __pyx__function = PyMethod_GET_FUNCTION(__pyx_t_8);
      __Pyx_INCREF(__pyx_t_1);
      __Pyx_INCREF(__pyx__function);
      __Pyx_DECREF_SET(__pyx_t_8, __pyx__function);
      __pyx_t_4 = 0;
    }
    #endif
    {
      PyObject *__pyx_callargs[2 + ((CYTHON_VECTORCALL) ? 2 : 0)] = {__pyx_t_1, __pyx_t_3};
      __pyx_t_9 = __Pyx_MakeVectorcallBuilderKwds(2); if (unlikely(!__pyx_t_9)) __PYX_ERR(0, 789, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_9);
      if (__Pyx_VectorcallBuilder_AddArg(__pyx_mstate_global->__pyx_n_u_dtype, __pyx_t_2, __pyx_t_9, __pyx_callargs+2, 0) < (0)) __PYX_ERR(0, 789, __pyx_L1_error)
      if (__Pyx_VectorcallBuilder_AddArg(__pyx_mstate_global->__pyx_n_u_order, __pyx_mstate_global->__pyx_n_u_F, __pyx_t_9, __pyx_callargs+2, 1) < (0)) __PYX_ERR(0, 789, __pyx_L1_error)
      __pyx_t_7 = __Pyx_Object_Vectorcall_CallFromBuilder((PyObject*)__pyx_t_8, __pyx_callargs+__pyx_t_4, (2-__pyx_t_4) | (__pyx_t_4*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET), __pyx_t_9);
      __Pyx_XDECREF(__pyx_t_1); __pyx_t_1 = 0;
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
      __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
      __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
      __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
      if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 789, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_7);
    }
    __Pyx_DECREF_SET(__pyx_v_b, __pyx_t_7);
    __pyx_t_7 = 0;

    /* "cupyx/cusolver.pyx":790
 *         bb = b.reshape(m, 1)
 *         b = _cupy.empty((max_mn, nrhs), dtype=a.dtype, order='F')
 *         b[:m, :] = bb             # <<<<<<<<<<<<<<
 *     else:
 *         b = b.copy(order='F')
*/
    __pyx_t_7 = PySlice_New(Py_None, __pyx_v_m, Py_None); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 790, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_7);
    __pyx_t_8 = PyTuple_New(2); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 790, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_8);
    __Pyx_GIVEREF(__pyx_t_7);
    if (__Pyx_PyTuple_SET_ITEM(__pyx_t_8, 0, __pyx_t_7) != (0)) __PYX_ERR(0, 790, __pyx_L1_error);
    __Pyx_INCREF(__pyx_mstate_global->__pyx_slice[2]);
    __Pyx_GIVEREF(__pyx_mstate_global->__pyx_slice[2]);
    if (__Pyx_PyTuple_SET_ITEM(__pyx_t_8, 1, __pyx_mstate_global->__pyx_slice[2]) != (0)) __PYX_ERR(0, 790, __pyx_L1_error);
    __pyx_t_7 = 0;
    if (unlikely((PyObject_SetItem(__pyx_v_b, __pyx_t_8, __pyx_v_bb) < 0))) __PYX_ERR(0, 790, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;

    /* "cupyx/cusolver.pyx":783
 *     a = a.copy(order='F')
 *     org_nrhs = nrhs
 *     if m > n and nrhs == 1:             # <<<<<<<<<<<<<<
 *         # Note: this is workaround as there is bug in cusolverDn<T1><T2>gels()
 *         # of CUDA 11.0/11.1 and it returns CUSOLVER_STATUS_IRS_NOT_SUPPORTED
*/
    goto __pyx_L16;
  }

  /* "cupyx/cusolver.pyx":792
 *         b[:m, :] = bb
 *     else:
 *         b = b.copy(order='F')             # <<<<<<<<<<<<<<
 *     x = _cupy.empty((max_mn, nrhs), dtype=a.dtype, order='F')
 *     dinfo = _cupy.empty(1, dtype=_numpy.int32)
*/
  /*else*/ {
    __pyx_t_7 = __pyx_v_b;
    __Pyx_INCREF(__pyx_t_7);
    __pyx_t_4 = 0;
    {
      PyObject *__pyx_callargs[2 + ((CYTHON_VECTORCALL) ? 1 : 0)] = {__pyx_t_7, NULL};
      __pyx_t_9 = __Pyx_MakeVectorcallBuilderKwds(1); if (unlikely(!__pyx_t_9)) __PYX_ERR(0, 792, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_9);
      if (__Pyx_VectorcallBuilder_AddArg(__pyx_mstate_global->__pyx_n_u_order, __pyx_mstate_global->__pyx_n_u_F, __pyx_t_9, __pyx_callargs+1, 0) < (0)) __PYX_ERR(0, 792, __pyx_L1_error)
      __pyx_t_8 = __Pyx_Object_VectorcallMethod_CallFromBuilder((PyObject*)__pyx_mstate_global->__pyx_n_u_copy, __pyx_callargs+__pyx_t_4, (1-__pyx_t_4) | (1*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET), __pyx_t_9);
      __Pyx_XDECREF(__pyx_t_7); __pyx_t_7 = 0;
      __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
      if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 792, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_8);
    }
    __Pyx_DECREF_SET(__pyx_v_b, __pyx_t_8);
    __pyx_t_8 = 0;
  }
  __pyx_L16:;

  /* "cupyx/cusolver.pyx":793
 *     else:
 *         b = b.copy(order='F')
 *     x = _cupy.empty((max_mn, nrhs), dtype=a.dtype, order='F')             # <<<<<<<<<<<<<<
 *     dinfo = _cupy.empty(1, dtype=_numpy.int32)
 *     handle = _device.get_cusolver_handle()
*/
  __pyx_t_9 = NULL;
  __Pyx_GetModuleGlobalName(__pyx_t_7, __pyx_mstate_global->__pyx_n_u_cupy); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 793, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_7);
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_t_7, __pyx_mstate_global->__pyx_n_u_empty); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 793, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
  __pyx_t_7 = PyTuple_New(2); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 793, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_7);
  __Pyx_INCREF(__pyx_v_max_mn);
  __Pyx_GIVEREF(__pyx_v_max_mn);
  if (__Pyx_PyTuple_SET_ITEM(__pyx_t_7, 0, __pyx_v_max_mn) != (0)) __PYX_ERR(0, 793, __pyx_L1_error);
  __Pyx_INCREF(__pyx_v_nrhs);
  __Pyx_GIVEREF(__pyx_v_nrhs);
  if (__Pyx_PyTuple_SET_ITEM(__pyx_t_7, 1, __pyx_v_nrhs) != (0)) __PYX_ERR(0, 793, __pyx_L1_error);
  __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_v_a, __pyx_mstate_global->__pyx_n_u_dtype); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 793, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_4 = 1;
  #if CYTHON_UNPACK_METHODS
  if (unlikely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_9 = PyMethod_GET_SELF(__pyx_t_2);
    assert(__pyx_t_9);
    PyObject* __pyx__function = PyMethod_GET_FUNCTION(__pyx_t_2);
    __Pyx_INCREF(__pyx_t_9);
    __Pyx_INCREF(__pyx__function);
    __Pyx_DECREF_SET(__pyx_t_2, __pyx__function);
    __pyx_t_4 = 0;
  }
  #endif
  {
    PyObject *__pyx_callargs[2 + ((CYTHON_VECTORCALL) ? 2 : 0)] = {__pyx_t_9, __pyx_t_7};
    __pyx_t_1 = __Pyx_MakeVectorcallBuilderKwds(2); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 793, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    if (__Pyx_VectorcallBuilder_AddArg(__pyx_mstate_global->__pyx_n_u_dtype, __pyx_t_3, __pyx_t_1, __pyx_callargs+2, 0) < (0)) __PYX_ERR(0, 793, __pyx_L1_error)
    if (__Pyx_VectorcallBuilder_AddArg(__pyx_mstate_global->__pyx_n_u_order, __pyx_mstate_global->__pyx_n_u_F, __pyx_t_1, __pyx_callargs+2, 1) < (0)) __PYX_ERR(0, 793, __pyx_L1_error)
    __pyx_t_8 = __Pyx_Object_Vectorcall_CallFromBuilder((PyObject*)__pyx_t_2, __pyx_callargs+__pyx_t_4, (2-__pyx_t_4) | (__pyx_t_4*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET), __pyx_t_1);
    __Pyx_XDECREF(__pyx_t_9); __pyx_t_9 = 0;
    __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 793, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_8);
  }
  __pyx_v_x = __pyx_t_8;
  __pyx_t_8 = 0;

  /* "cupyx/cusolver.pyx":794
 *         b = b.copy(order='F')
 *     x = _cupy.empty((max_mn, nrhs), dtype=a.dtype, order='F')
 *     dinfo = _cupy.empty(1, dtype=_numpy.int32)             # <<<<<<<<<<<<<<
 *     handle = _device.get_cusolver_handle()
 *     lwork = helper(handle, m, n, nrhs, a.data.ptr, m, b.data.ptr, m,
*/
  __pyx_t_2 = NULL;
  __Pyx_GetModuleGlobalName(__pyx_t_1, __pyx_mstate_global->__pyx_n_u_cupy); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 794, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_t_1, __pyx_mstate_global->__pyx_n_u_empty); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 794, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_GetModuleGlobalName(__pyx_t_1, __pyx_mstate_global->__pyx_n_u_numpy); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 794, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_7 = __Pyx_PyObject_GetAttrStr(__pyx_t_1, __pyx_mstate_global->__pyx_n_u_int32); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 794, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_7);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_4 = 1;
  #if CYTHON_UNPACK_METHODS
  if (unlikely(PyMethod_Check(__pyx_t_3))) {
    __pyx_t_2 = PyMethod_GET_SELF(__pyx_t_3);
    assert(__pyx_t_2);
    PyObject* __pyx__function = PyMethod_GET_FUNCTION(__pyx_t_3);
    __Pyx_INCREF(__pyx_t_2);
    __Pyx_INCREF(__pyx__function);
    __Pyx_DECREF_SET(__pyx_t_3, __pyx__function);
    __pyx_t_4 = 0;
  }
  #endif
  {
    PyObject *__pyx_callargs[2 + ((CYTHON_VECTORCALL) ? 1 : 0)] = {__pyx_t_2, __pyx_mstate_global->__pyx_int_1};
    __pyx_t_1 = __Pyx_MakeVectorcallBuilderKwds(1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 794, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    if (__Pyx_VectorcallBuilder_AddArg(__pyx_mstate_global->__pyx_n_u_dtype, __pyx_t_7, __pyx_t_1, __pyx_callargs+2, 0) < (0)) __PYX_ERR(0, 794, __pyx_L1_error)
    __pyx_t_8 = __Pyx_Object_Vectorcall_CallFromBuilder((PyObject*)__pyx_t_3, __pyx_callargs+__pyx_t_4, (2-__pyx_t_4) | (__pyx_t_4*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET), __pyx_t_1);
    __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 794, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_8);
  }
  __pyx_v_dinfo = __pyx_t_8;
  __pyx_t_8 = 0;

  /* "cupyx/cusolver.pyx":795
 *     x = _cupy.empty((max_mn, nrhs), dtype=a.dtype, order='F')
 *     dinfo = _cupy.empty(1, dtype=_numpy.int32)
 *     handle = _device.get_cusolver_handle()             # <<<<<<<<<<<<<<
 *     lwork = helper(handle, m, n, nrhs, a.data.ptr, m, b.data.ptr, m,
 *                    x.data.ptr, max_mn, 0)
*/
  __pyx_t_3 = NULL;
  __Pyx_GetModuleGlobalName(__pyx_t_1, __pyx_mstate_global->__pyx_n_u_device); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 795, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_7 = __Pyx_PyObject_GetAttrStr(__pyx_t_1, __pyx_mstate_global->__pyx_n_u_get_cusolver_handle); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 795, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_7);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_4 = 1;
  #if CYTHON_UNPACK_METHODS
  if (unlikely(PyMethod_Check(__pyx_t_7))) {
    __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_7);
    assert(__pyx_t_3);
    PyObject* __pyx__function = PyMethod_GET_FUNCTION(__pyx_t_7);
    __Pyx_INCREF(__pyx_t_3);
    __Pyx_INCREF(__pyx__function);
    __Pyx_DECREF_SET(__pyx_t_7, __pyx__function);
    __pyx_t_4 = 0;
  }
  #endif
  {
    PyObject *__pyx_callargs[2] = {__pyx_t_3, NULL};
    __pyx_t_8 = __Pyx_PyObject_FastCall((PyObject*)__pyx_t_7, __pyx_callargs+__pyx_t_4, (1-__pyx_t_4) | (__pyx_t_4*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
    __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
    __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
    if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 795, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_8);
  }
  __pyx_v_handle = __pyx_t_8;
  __pyx_t_8 = 0;

  /* "cupyx/cusolver.pyx":796
 *     dinfo = _cupy.empty(1, dtype=_numpy.int32)
 *     handle = _device.get_cusolver_handle()
 *     lwork = helper(handle, m, n, nrhs, a.data.ptr, m, b.data.ptr, m,             # <<<<<<<<<<<<<<
 *                    x.data.ptr, max_mn, 0)
 *     dwork = _cupy.empty(lwork, dtype=_numpy.int8)
*/
  __pyx_t_7 = NULL;
  __Pyx_INCREF(__pyx_v_helper);
  __pyx_t_3 = __pyx_v_helper; 
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_v_a, __pyx_mstate_global->__pyx_n_u_data); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 796, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_t_1, __pyx_mstate_global->__pyx_n_u_ptr); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 796, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_v_b, __pyx_mstate_global->__pyx_n_u_data); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 796, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_9 = __Pyx_PyObject_GetAttrStr(__pyx_t_1, __pyx_mstate_global->__pyx_n_u_ptr); if (unlikely(!__pyx_t_9)) __PYX_ERR(0, 796, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_9);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "cupyx/cusolver.pyx":797
 *     handle = _device.get_cusolver_handle()
 *     lwork = helper(handle, m, n, nrhs, a.data.ptr, m, b.data.ptr, m,
 *                    x.data.ptr, max_mn, 0)             # <<<<<<<<<<<<<<
 *     dwork = _cupy.empty(lwork, dtype=_numpy.int8)
 *     niters = solver(handle, m, n, nrhs, a.data.ptr, m, b.data.ptr, m,
*/
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_v_x, __pyx_mstate_global->__pyx_n_u_data); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 797, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_11 = __Pyx_PyObject_GetAttrStr(__pyx_t_1, __pyx_mstate_global->__pyx_n_u_ptr); if (unlikely(!__pyx_t_11)) __PYX_ERR(0, 797, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_11);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_4 = 1;
  #if CYTHON_UNPACK_METHODS
  if (unlikely(PyMethod_Check(__pyx_t_3))) {
    __pyx_t_7 = PyMethod_GET_SELF(__pyx_t_3);
    assert(__pyx_t_7);
    PyObject* __pyx__function = PyMethod_GET_FUNCTION(__pyx_t_3);
    __Pyx_INCREF(__pyx_t_7);
    __Pyx_INCREF(__pyx__function);
    __Pyx_DECREF_SET(__pyx_t_3, __pyx__function);
    __pyx_t_4 = 0;
  }
  #endif
  {
    PyObject *__pyx_callargs[12] = {__pyx_t_7, __pyx_v_handle, __pyx_v_m, __pyx_v_n, __pyx_v_nrhs, __pyx_t_2, __pyx_v_m, __pyx_t_9, __pyx_v_m, __pyx_t_11, __pyx_v_max_mn, __pyx_mstate_global->__pyx_int_0};
    __pyx_t_8 = __Pyx_PyObject_FastCall((PyObject*)__pyx_t_3, __pyx_callargs+__pyx_t_4, (12-__pyx_t_4) | (__pyx_t_4*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
    __Pyx_XDECREF(__pyx_t_7); __pyx_t_7 = 0;
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
    __Pyx_DECREF(__pyx_t_11); __pyx_t_11 = 0;
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 796, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_8);
  }
  __pyx_v_lwork = __pyx_t_8;
  __pyx_t_8 = 0;

  /* "cupyx/cusolver.pyx":798
 *     lwork = helper(handle, m, n, nrhs, a.data.ptr, m, b.data.ptr, m,
 *                    x.data.ptr, max_mn, 0)
 *     dwork = _cupy.empty(lwork, dtype=_numpy.int8)             # <<<<<<<<<<<<<<
 *     niters = solver(handle, m, n, nrhs, a.data.ptr, m, b.data.ptr, m,
 *                     x.data.ptr, max_mn, dwork.data.ptr, lwork, dinfo.data.ptr)
*/
  __pyx_t_3 = NULL;
  __Pyx_GetModuleGlobalName(__pyx_t_11, __pyx_mstate_global->__pyx_n_u_cupy); if (unlikely(!__pyx_t_11)) __PYX_ERR(0, 798, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_11);
  __pyx_t_9 = __Pyx_PyObject_GetAttrStr(__pyx_t_11, __pyx_mstate_global->__pyx_n_u_empty); if (unlikely(!__pyx_t_9)) __PYX_ERR(0, 798, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_9);
  __Pyx_DECREF(__pyx_t_11); __pyx_t_11 = 0;
  __Pyx_GetModuleGlobalName(__pyx_t_11, __pyx_mstate_global->__pyx_n_u_numpy); if (unlikely(!__pyx_t_11)) __PYX_ERR(0, 798, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_11);
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_t_11, __pyx_mstate_global->__pyx_n_u_int8); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 798, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_11); __pyx_t_11 = 0;
  __pyx_t_4 = 1;
  #if CYTHON_UNPACK_METHODS
  if (unlikely(PyMethod_Check(__pyx_t_9))) {
    __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_9);
    assert(__pyx_t_3);
    PyObject* __pyx__function = PyMethod_GET_FUNCTION(__pyx_t_9);
    __Pyx_INCREF(__pyx_t_3);
    __Pyx_INCREF(__pyx__function);
    __Pyx_DECREF_SET(__pyx_t_9, __pyx__function);
    __pyx_t_4 = 0;
  }
  #endif
  {
    PyObject *__pyx_callargs[2 + ((CYTHON_VECTORCALL) ? 1 : 0)] = {__pyx_t_3, __pyx_v_lwork};
    __pyx_t_11 = __Pyx_MakeVectorcallBuilderKwds(1); if (unlikely(!__pyx_t_11)) __PYX_ERR(0, 798, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_11);
    if (__Pyx_VectorcallBuilder_AddArg(__pyx_mstate_global->__pyx_n_u_dtype, __pyx_t_2, __pyx_t_11, __pyx_callargs+2, 0) < (0)) __PYX_ERR(0, 798, __pyx_L1_error)
    __pyx_t_8 = __Pyx_Object_Vectorcall_CallFromBuilder((PyObject*)__pyx_t_9, __pyx_callargs+__pyx_t_4, (2-__pyx_t_4) | (__pyx_t_4*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET), __pyx_t_11);
    __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_DECREF(__pyx_t_11); __pyx_t_11 = 0;
    __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
    if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 798, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_8);
  }
  __pyx_v_dwork = __pyx_t_8;
  __pyx_t_8 = 0;

  /* "cupyx/cusolver.pyx":799
 *                    x.data.ptr, max_mn, 0)
 *     dwork = _cupy.empty(lwork, dtype=_numpy.int8)
 *     niters = solver(handle, m, n, nrhs, a.data.ptr, m, b.data.ptr, m,             # <<<<<<<<<<<<<<
 *                     x.data.ptr, max_mn, dwork.data.ptr, lwork, dinfo.data.ptr)
 *     if niters < 0:
*/
  __pyx_t_9 = NULL;
  __Pyx_INCREF(__pyx_v_solver);
  __pyx_t_11 = __pyx_v_solver; 
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_v_a, __pyx_mstate_global->__pyx_n_u_data); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 799, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_t_2, __pyx_mstate_global->__pyx_n_u_ptr); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 799, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_v_b, __pyx_mstate_global->__pyx_n_u_data); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 799, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_7 = __Pyx_PyObject_GetAttrStr(__pyx_t_2, __pyx_mstate_global->__pyx_n_u_ptr); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 799, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_7);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "cupyx/cusolver.pyx":800
 *     dwork = _cupy.empty(lwork, dtype=_numpy.int8)
 *     niters = solver(handle, m, n, nrhs, a.data.ptr, m, b.data.ptr, m,
 *                     x.data.ptr, max_mn, dwork.data.ptr, lwork, dinfo.data.ptr)             # <<<<<<<<<<<<<<
 *     if niters < 0:
 *         if niters <= -50:
*/
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_v_x, __pyx_mstate_global->__pyx_n_u_data); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 800, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_t_2, __pyx_mstate_global->__pyx_n_u_ptr); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 800, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_v_dwork, __pyx_mstate_global->__pyx_n_u_data); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 800, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_12 = __Pyx_PyObject_GetAttrStr(__pyx_t_2, __pyx_mstate_global->__pyx_n_u_ptr); if (unlikely(!__pyx_t_12)) __PYX_ERR(0, 800, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_12);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_v_dinfo, __pyx_mstate_global->__pyx_n_u_data); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 800, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_13 = __Pyx_PyObject_GetAttrStr(__pyx_t_2, __pyx_mstate_global->__pyx_n_u_ptr); if (unlikely(!__pyx_t_13)) __PYX_ERR(0, 800, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_13);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_4 = 1;
  #if CYTHON_UNPACK_METHODS
  if (unlikely(PyMethod_Check(__pyx_t_11))) {
    __pyx_t_9 = PyMethod_GET_SELF(__pyx_t_11);
    assert(__pyx_t_9);
    PyObject* __pyx__function = PyMethod_GET_FUNCTION(__pyx_t_11);
    __Pyx_INCREF(__pyx_t_9);
    __Pyx_INCREF(__pyx__function);
    __Pyx_DECREF_SET(__pyx_t_11, __pyx__function);
    __pyx_t_4 = 0;
  }
  #endif
  {
    PyObject *__pyx_callargs[14] = {__pyx_t_9, __pyx_v_handle, __pyx_v_m, __pyx_v_n, __pyx_v_nrhs, __pyx_t_3, __pyx_v_m, __pyx_t_7, __pyx_v_m, __pyx_t_1, __pyx_v_max_mn, __pyx_t_12, __pyx_v_lwork, __pyx_t_13};
    __pyx_t_8 = __Pyx_PyObject_FastCall((PyObject*)__pyx_t_11, __pyx_callargs+__pyx_t_4, (14-__pyx_t_4) | (__pyx_t_4*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
    __Pyx_XDECREF(__pyx_t_9); __pyx_t_9 = 0;
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __Pyx_DECREF(__pyx_t_12); __pyx_t_12 = 0;
    __Pyx_DECREF(__pyx_t_13); __pyx_t_13 = 0;
    __Pyx_DECREF(__pyx_t_11); __pyx_t_11 = 0;
    if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 799, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_8);
  }
  __pyx_v_niters = __pyx_t_8;
  __pyx_t_8 = 0;

  /* "cupyx/cusolver.pyx":801
 *     niters = solver(handle, m, n, nrhs, a.data.ptr, m, b.data.ptr, m,
 *                     x.data.ptr, max_mn, dwork.data.ptr, lwork, dinfo.data.ptr)
 *     if niters < 0:             # <<<<<<<<<<<<<<
 *         if niters <= -50:
 *             _warnings.warn('gels reached maximum allowed iterations.')
*/
  __pyx_t_8 = PyObject_RichCompare(__pyx_v_niters, __pyx_mstate_global->__pyx_int_0, Py_LT); __Pyx_XGOTREF(__pyx_t_8); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 801, __pyx_L1_error)
  __pyx_t_6 = __Pyx_PyObject_IsTrue(__pyx_t_8); if (unlikely((__pyx_t_6 < 0))) __PYX_ERR(0, 801, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
  if (__pyx_t_6) {

    /* "cupyx/cusolver.pyx":802
 *                     x.data.ptr, max_mn, dwork.data.ptr, lwork, dinfo.data.ptr)
 *     if niters < 0:
 *         if niters <= -50:             # <<<<<<<<<<<<<<
 *             _warnings.warn('gels reached maximum allowed iterations.')
 *         else:
*/
    __pyx_t_8 = PyObject_RichCompare(__pyx_v_niters, __pyx_mstate_global->__pyx_int_neg_50, Py_LE); __Pyx_XGOTREF(__pyx_t_8); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 802, __pyx_L1_error)
    __pyx_t_6 = __Pyx_PyObject_IsTrue(__pyx_t_8); if (unlikely((__pyx_t_6 < 0))) __PYX_ERR(0, 802, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
    if (likely(__pyx_t_6)) {

      /* "cupyx/cusolver.pyx":803
 *     if niters < 0:
 *         if niters <= -50:
 *             _warnings.warn('gels reached maximum allowed iterations.')             # <<<<<<<<<<<<<<
 *         else:
 *             raise RuntimeError('gels has failed ({}).'.format(niters))
*/
      __pyx_t_11 = NULL;
      __Pyx_GetModuleGlobalName(__pyx_t_13, __pyx_mstate_global->__pyx_n_u_warnings); if (unlikely(!__pyx_t_13)) __PYX_ERR(0, 803, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_13);
      __pyx_t_12 = __Pyx_PyObject_GetAttrStr(__pyx_t_13, __pyx_mstate_global->__pyx_n_u_warn); if (unlikely(!__pyx_t_12)) __PYX_ERR(0, 803, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_12);
      __Pyx_DECREF(__pyx_t_13); __pyx_t_13 = 0;
      __pyx_t_4 = 1;
      #if CYTHON_UNPACK_METHODS
      if (unlikely(PyMethod_Check(__pyx_t_12))) {
        __pyx_t_11 = PyMethod_GET_SELF(__pyx_t_12);
        assert(__pyx_t_11);
        PyObject* __pyx__function = PyMethod_GET_FUNCTION(__pyx_t_12);
        __Pyx_INCREF(__pyx_t_11);
        __Pyx_INCREF(__pyx__function);
        __Pyx_DECREF_SET(__pyx_t_12, __pyx__function);
        __pyx_t_4 = 0;
      }
      #endif
      {
        PyObject *__pyx_callargs[2] = {__pyx_t_11, __pyx_mstate_global->__pyx_kp_u_gels_reached_maximum_allowed_ite};
        __pyx_t_8 = __Pyx_PyObject_FastCall((PyObject*)__pyx_t_12, __pyx_callargs+__pyx_t_4, (2-__pyx_t_4) | (__pyx_t_4*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
        __Pyx_XDECREF(__pyx_t_11); __pyx_t_11 = 0;
        __Pyx_DECREF(__pyx_t_12); __pyx_t_12 = 0;
        if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 803, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_8);
      }
      __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;

      /* "cupyx/cusolver.pyx":802
 *                     x.data.ptr, max_mn, dwork.data.ptr, lwork, dinfo.data.ptr)
 *     if niters < 0:
 *         if niters <= -50:             # <<<<<<<<<<<<<<
 *             _warnings.warn('gels reached maximum allowed iterations.')
 *         else:
*/
      goto __pyx_L20;
    }

    /* "cupyx/cusolver.pyx":805
 *             _warnings.warn('gels reached maximum allowed iterations.')
 *         else:
 *             raise RuntimeError('gels has failed ({}).'.format(niters))             # <<<<<<<<<<<<<<
 *     x = x[:n]
 *     if org_nrhs != nrhs:
*/
    /*else*/ {
      __pyx_t_12 = NULL;
      __pyx_t_13 = __pyx_mstate_global->__pyx_kp_u_gels_has_failed;
      __Pyx_INCREF(__pyx_t_13);
      __pyx_t_4 = 0;
      {
        PyObject *__pyx_callargs[2] = {__pyx_t_13, __pyx_v_niters};
        __pyx_t_11 = __Pyx_PyObject_FastCallMethod((PyObject*)__pyx_mstate_global->__pyx_n_u_format, __pyx_callargs+__pyx_t_4, (2-__pyx_t_4) | (1*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
        __Pyx_XDECREF(__pyx_t_13); __pyx_t_13 = 0;
        if (unlikely(!__pyx_t_11)) __PYX_ERR(0, 805, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_11);
      }
      __pyx_t_4 = 1;
      {
        PyObject *__pyx_callargs[2] = {__pyx_t_12, __pyx_t_11};
        __pyx_t_8 = __Pyx_PyObject_FastCall((PyObject*)(((PyTypeObject*)PyExc_RuntimeError)), __pyx_callargs+__pyx_t_4, (2-__pyx_t_4) | (__pyx_t_4*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
        __Pyx_XDECREF(__pyx_t_12); __pyx_t_12 = 0;
        __Pyx_DECREF(__pyx_t_11); __pyx_t_11 = 0;
        if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 805, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_8);
      }
      __Pyx_Raise(__pyx_t_8, 0, 0, 0);
      __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
      __PYX_ERR(0, 805, __pyx_L1_error)
    }
    __pyx_L20:;

    /* "cupyx/cusolver.pyx":801
 *     niters = solver(handle, m, n, nrhs, a.data.ptr, m, b.data.ptr, m,
 *                     x.data.ptr, max_mn, dwork.data.ptr, lwork, dinfo.data.ptr)
 *     if niters < 0:             # <<<<<<<<<<<<<<
 *         if niters <= -50:
 *             _warnings.warn('gels reached maximum allowed iterations.')
*/
  }

  /* "cupyx/cusolver.pyx":806
 *         else:
 *             raise RuntimeError('gels has failed ({}).'.format(niters))
 *     x = x[:n]             # <<<<<<<<<<<<<<
 *     if org_nrhs != nrhs:
 *         x = x[:, :org_nrhs]
*/
  __pyx_t_8 = __Pyx_PyObject_GetSlice(__pyx_v_x, 0, 0, NULL, &__pyx_v_n, NULL, 0, 0, 1); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 806, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_8);
  __Pyx_DECREF_SET(__pyx_v_x, __pyx_t_8);
  __pyx_t_8 = 0;

  /* "cupyx/cusolver.pyx":807
 *             raise RuntimeError('gels has failed ({}).'.format(niters))
 *     x = x[:n]
 *     if org_nrhs != nrhs:             # <<<<<<<<<<<<<<
 *         x = x[:, :org_nrhs]
 *     if b_ndim == 1:
*/
  __pyx_t_8 = PyObject_RichCompare(__pyx_v_org_nrhs, __pyx_v_nrhs, Py_NE); __Pyx_XGOTREF(__pyx_t_8); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 807, __pyx_L1_error)
  __pyx_t_6 = __Pyx_PyObject_IsTrue(__pyx_t_8); if (unlikely((__pyx_t_6 < 0))) __PYX_ERR(0, 807, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
  if (__pyx_t_6) {

    /* "cupyx/cusolver.pyx":808
 *     x = x[:n]
 *     if org_nrhs != nrhs:
 *         x = x[:, :org_nrhs]             # <<<<<<<<<<<<<<
 *     if b_ndim == 1:
 *         x = x.reshape(n)
*/
    __pyx_t_8 = PySlice_New(Py_None, __pyx_v_org_nrhs, Py_None); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 808, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_8);
    __pyx_t_11 = PyTuple_New(2); if (unlikely(!__pyx_t_11)) __PYX_ERR(0, 808, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_11);
    __Pyx_INCREF(__pyx_mstate_global->__pyx_slice[2]);
    __Pyx_GIVEREF(__pyx_mstate_global->__pyx_slice[2]);
    if (__Pyx_PyTuple_SET_ITEM(__pyx_t_11, 0, __pyx_mstate_global->__pyx_slice[2]) != (0)) __PYX_ERR(0, 808, __pyx_L1_error);
    __Pyx_GIVEREF(__pyx_t_8);
    if (__Pyx_PyTuple_SET_ITEM(__pyx_t_11, 1, __pyx_t_8) != (0)) __PYX_ERR(0, 808, __pyx_L1_error);
    __pyx_t_8 = 0;
    __pyx_t_8 = __Pyx_PyObject_GetItem(__pyx_v_x, __pyx_t_11); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 808, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_8);
    __Pyx_DECREF(__pyx_t_11); __pyx_t_11 = 0;
    __Pyx_DECREF_SET(__pyx_v_x, __pyx_t_8);
    __pyx_t_8 = 0;

    /* "cupyx/cusolver.pyx":807
 *             raise RuntimeError('gels has failed ({}).'.format(niters))
 *     x = x[:n]
 *     if org_nrhs != nrhs:             # <<<<<<<<<<<<<<
 *         x = x[:, :org_nrhs]
 *     if b_ndim == 1:
*/
  }

  /* "cupyx/cusolver.pyx":809
 *     if org_nrhs != nrhs:
 *         x = x[:, :org_nrhs]
 *     if b_ndim == 1:             # <<<<<<<<<<<<<<
 *         x = x.reshape(n)
 *     return x
*/
  __pyx_t_6 = (__Pyx_PyLong_BoolEqObjC(__pyx_v_b_ndim, __pyx_mstate_global->__pyx_int_1, 1, 0)); if (unlikely((__pyx_t_6 < 0))) __PYX_ERR(0, 809, __pyx_L1_error)
  if (__pyx_t_6) {

    /* "cupyx/cusolver.pyx":810
 *         x = x[:, :org_nrhs]
 *     if b_ndim == 1:
 *         x = x.reshape(n)             # <<<<<<<<<<<<<<
 *     return x
 * 
*/
    __pyx_t_11 = __pyx_v_x;
    __Pyx_INCREF(__pyx_t_11);
    __pyx_t_4 = 0;
    {
      PyObject *__pyx_callargs[2] = {__pyx_t_11, __pyx_v_n};
      __pyx_t_8 = __Pyx_PyObject_FastCallMethod((PyObject*)__pyx_mstate_global->__pyx_n_u_reshape, __pyx_callargs+__pyx_t_4, (2-__pyx_t_4) | (1*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
      __Pyx_XDECREF(__pyx_t_11); __pyx_t_11 = 0;
      if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 810, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_8);
    }
    __Pyx_DECREF_SET(__pyx_v_x, __pyx_t_8);
    __pyx_t_8 = 0;

    /* "cupyx/cusolver.pyx":809
 *     if org_nrhs != nrhs:
 *         x = x[:, :org_nrhs]
 *     if b_ndim == 1:             # <<<<<<<<<<<<<<
 *         x = x.reshape(n)
 *     return x
*/
  }

  /* "cupyx/cusolver.pyx":811
 *     if b_ndim == 1:
 *         x = x.reshape(n)
 *     return x             # <<<<<<<<<<<<<<
 * 
 * 
*/
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_v_x);
  __pyx_r = __pyx_v_x;
  goto __pyx_L0;

  /* "cupyx/cusolver.pyx":714
 * 
 * 
 * def gels(a, b):             # <<<<<<<<<<<<<<
 *     """Compute least square solution using cusolverDn<t1><t2>gels().
 * 
*/

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_7);
  __Pyx_XDECREF(__pyx_t_8);
  __Pyx_XDECREF(__pyx_t_9);
  __Pyx_XDECREF(__pyx_t_11);
  __Pyx_XDECREF(__pyx_t_12);
  __Pyx_XDECREF(__pyx_t_13);
  __Pyx_AddTraceback("cupyx.cusolver.gels", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_nrhs);
  __Pyx_XDECREF(__pyx_v_m);
  __Pyx_XDECREF(__pyx_v_n);
  __Pyx_XDECREF(__pyx_v_max_mn);
  __Pyx_XDECREF(__pyx_v_b_ndim);
  __Pyx_XDECREF(__pyx_v_compute_type);
  __Pyx_XDECREF(__pyx_v_t1);
  __Pyx_XDECREF(__pyx_v_t2);
  __Pyx_XDECREF(__pyx_v_solver_name);
  __Pyx_XDECREF(__pyx_v_solver);
  __Pyx_XDECREF(__pyx_v_helper);
  __Pyx_XDECREF(__pyx_v_org_nrhs);
  __Pyx_XDECREF(__pyx_v_bb);
  __Pyx_XDECREF(__pyx_v_x);
  __Pyx_XDECREF(__pyx_v_dinfo);
  __Pyx_XDECREF(__pyx_v_handle);
  __Pyx_XDECREF(__pyx_v_lwork);
  __Pyx_XDECREF(__pyx_v_dwork);
  __Pyx_XDECREF(__pyx_v_niters);
  __Pyx_XDECREF(__pyx_v_a);
  __Pyx_XDECREF(__pyx_v_b);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "cupyx/cusolver.pyx":814
 * 
 * 
 * def csrlsvqr(A, b, tol=0, reorder=1):             # <<<<<<<<<<<<<<
 *     """Solves the linear system ``Ax = b`` using QR factorization.
 * 
*/

/* Python wrapper */
static PyObject *__pyx_pw_5cupyx_8cusolver_19csrlsvqr(PyObject *__pyx_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
PyDoc_STRVAR(__pyx_doc_5cupyx_8cusolver_18csrlsvqr, "csrlsvqr(A, b, tol=0, reorder=1)\n\nSolves the linear system ``Ax = b`` using QR factorization.\n\nArgs:\n    A (cupyx.scipy.sparse.csr_matrix): Sparse matrix with dimension\n        ``(M, M)``.\n    b (cupy.ndarray): Dense vector with dimension ``(M,)``.\n    tol (float): Tolerance to decide if singular or not.\n    reorder (int): Reordering scheme to reduce zero fill-in.\n        1: symrcm is used.\n        2: symamd is used.\n        3: csrmetisnd is used.\n        else: no reordering.");
static PyMethodDef __pyx_mdef_5cupyx_8cusolver_19csrlsvqr = {"csrlsvqr", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_5cupyx_8cusolver_19csrlsvqr, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_5cupyx_8cusolver_18csrlsvqr};
static PyObject *__pyx_pw_5cupyx_8cusolver_19csrlsvqr(PyObject *__pyx_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
) {
  PyObject *__pyx_v_A = 0;
  PyObject *__pyx_v_b = 0;
  PyObject *__pyx_v_tol = 0;
  PyObject *__pyx_v_reorder = 0;
  #if !CYTHON_METH_FASTCALL
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  #endif
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject* values[4] = {0,0,0,0};
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("csrlsvqr (wrapper)", 0);
  #if !CYTHON_METH_FASTCALL
  #if CYTHON_ASSUME_SAFE_SIZE
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return NULL;
  #endif
  #endif
  __pyx_kwvalues = __Pyx_KwValues_FASTCALL(__pyx_args, __pyx_nargs);
  {
    PyObject ** const __pyx_pyargnames[] = {&__pyx_mstate_global->__pyx_n_u_A,&__pyx_mstate_global->__pyx_n_u_b,&__pyx_mstate_global->__pyx_n_u_tol,&__pyx_mstate_global->__pyx_n_u_reorder,0};
    const Py_ssize_t __pyx_kwds_len = (__pyx_kwds) ? __Pyx_NumKwargs_FASTCALL(__pyx_kwds) : 0;
    if (unlikely(__pyx_kwds_len < 0)) __PYX_ERR(0, 814, __pyx_L3_error)
    if (__pyx_kwds_len > 0) {
      switch (__pyx_nargs) {
        case  4:
        values[3] = __Pyx_ArgRef_FASTCALL(__pyx_args, 3);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[3])) __PYX_ERR(0, 814, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  3:
        values[2] = __Pyx_ArgRef_FASTCALL(__pyx_args, 2);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[2])) __PYX_ERR(0, 814, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  2:
        values[1] = __Pyx_ArgRef_FASTCALL(__pyx_args, 1);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[1])) __PYX_ERR(0, 814, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  1:
        values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(0, 814, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      const Py_ssize_t kwd_pos_args = __pyx_nargs;
      if (__Pyx_ParseKeywords(__pyx_kwds, __pyx_kwvalues, __pyx_pyargnames, 0, values, kwd_pos_args, __pyx_kwds_len, "csrlsvqr", 0) < (0)) __PYX_ERR(0, 814, __pyx_L3_error)
      if (!values[2]) values[2] = __Pyx_NewRef(((PyObject *)__pyx_mstate_global->__pyx_int_0));
      if (!values[3]) values[3] = __Pyx_NewRef(((PyObject *)__pyx_mstate_global->__pyx_int_1));
      for (Py_ssize_t i = __pyx_nargs; i < 2; i++) {
        if (unlikely(!values[i])) { __Pyx_RaiseArgtupleInvalid("csrlsvqr", 0, 2, 4, i); __PYX_ERR(0, 814, __pyx_L3_error) }
      }
    } else {
      switch (__pyx_nargs) {
        case  4:
        values[3] = __Pyx_ArgRef_FASTCALL(__pyx_args, 3);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[3])) __PYX_ERR(0, 814, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  3:
        values[2] = __Pyx_ArgRef_FASTCALL(__pyx_args, 2);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[2])) __PYX_ERR(0, 814, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  2:
        values[1] = __Pyx_ArgRef_FASTCALL(__pyx_args, 1);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[1])) __PYX_ERR(0, 814, __pyx_L3_error)
        values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(0, 814, __pyx_L3_error)
        break;
        default: goto __pyx_L5_argtuple_error;
      }
      if (!values[2]) values[2] = __Pyx_NewRef(((PyObject *)__pyx_mstate_global->__pyx_int_0));
      if (!values[3]) values[3] = __Pyx_NewRef(((PyObject *)__pyx_mstate_global->__pyx_int_1));
    }
    __pyx_v_A = values[0];
    __pyx_v_b = values[1];
    __pyx_v_tol = values[2];
    __pyx_v_reorder = values[3];
  }
  goto __pyx_L6_skip;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("csrlsvqr", 0, 2, 4, __pyx_nargs); __PYX_ERR(0, 814, __pyx_L3_error)
  __pyx_L6_skip:;
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_AddTraceback("cupyx.cusolver.csrlsvqr", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_5cupyx_8cusolver_18csrlsvqr(__pyx_self, __pyx_v_A, __pyx_v_b, __pyx_v_tol, __pyx_v_reorder);

  /* function exit code */
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5cupyx_8cusolver_18csrlsvqr(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_A, PyObject *__pyx_v_b, PyObject *__pyx_v_tol, PyObject *__pyx_v_reorder) {
  PyObject *__pyx_v_dtype = NULL;
  PyObject *__pyx_v_t = NULL;
  PyObject *__pyx_v_solve = NULL;
  PyObject *__pyx_v_m = NULL;
  PyObject *__pyx_v_x = NULL;
  PyObject *__pyx_v_singularity = NULL;
  PyObject *__pyx_v_handle = NULL;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  size_t __pyx_t_4;
  int __pyx_t_5;
  int __pyx_t_6;
  PyObject *__pyx_t_7 = NULL;
  PyObject *__pyx_t_8 = NULL;
  long __pyx_t_9;
  PyObject *__pyx_t_10 = NULL;
  PyObject *__pyx_t_11 = NULL;
  PyObject *__pyx_t_12 = NULL;
  PyObject *__pyx_t_13 = NULL;
  PyObject *__pyx_t_14 = NULL;
  PyObject *__pyx_t_15 = NULL;
  PyObject *__pyx_t_16 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("csrlsvqr", 0);
  __Pyx_INCREF(__pyx_v_b);
  __Pyx_INCREF(__pyx_v_tol);

  /* "cupyx/cusolver.pyx":828
 *             else: no reordering.
 *     """
 *     if not check_availability('csrlsvqr'):             # <<<<<<<<<<<<<<
 *         raise RuntimeError('csrlsvqr is not available.')
 * 
*/
  __pyx_t_2 = NULL;
  __Pyx_GetModuleGlobalName(__pyx_t_3, __pyx_mstate_global->__pyx_n_u_check_availability); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 828, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_4 = 1;
  #if CYTHON_UNPACK_METHODS
  if (unlikely(PyMethod_Check(__pyx_t_3))) {
    __pyx_t_2 = PyMethod_GET_SELF(__pyx_t_3);
    assert(__pyx_t_2);
    PyObject* __pyx__function = PyMethod_GET_FUNCTION(__pyx_t_3);
    __Pyx_INCREF(__pyx_t_2);
    __Pyx_INCREF(__pyx__function);
    __Pyx_DECREF_SET(__pyx_t_3, __pyx__function);
    __pyx_t_4 = 0;
  }
  #endif
  {
    PyObject *__pyx_callargs[2] = {__pyx_t_2, __pyx_mstate_global->__pyx_n_u_csrlsvqr};
    __pyx_t_1 = __Pyx_PyObject_FastCall((PyObject*)__pyx_t_3, __pyx_callargs+__pyx_t_4, (2-__pyx_t_4) | (__pyx_t_4*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
    __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 828, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
  }
  __pyx_t_5 = __Pyx_PyObject_IsTrue(__pyx_t_1); if (unlikely((__pyx_t_5 < 0))) __PYX_ERR(0, 828, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_6 = (!__pyx_t_5);
  if (unlikely(__pyx_t_6)) {

    /* "cupyx/cusolver.pyx":829
 *     """
 *     if not check_availability('csrlsvqr'):
 *         raise RuntimeError('csrlsvqr is not available.')             # <<<<<<<<<<<<<<
 * 
 *     if not _cupyx.scipy.sparse.isspmatrix_csr(A):
*/
    __pyx_t_3 = NULL;
    __pyx_t_4 = 1;
    {
      PyObject *__pyx_callargs[2] = {__pyx_t_3, __pyx_mstate_global->__pyx_kp_u_csrlsvqr_is_not_available};
      __pyx_t_1 = __Pyx_PyObject_FastCall((PyObject*)(((PyTypeObject*)PyExc_RuntimeError)), __pyx_callargs+__pyx_t_4, (2-__pyx_t_4) | (__pyx_t_4*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
      __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
      if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 829, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
    }
    __Pyx_Raise(__pyx_t_1, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __PYX_ERR(0, 829, __pyx_L1_error)

    /* "cupyx/cusolver.pyx":828
 *             else: no reordering.
 *     """
 *     if not check_availability('csrlsvqr'):             # <<<<<<<<<<<<<<
 *         raise RuntimeError('csrlsvqr is not available.')
 * 
*/
  }

  /* "cupyx/cusolver.pyx":831
 *         raise RuntimeError('csrlsvqr is not available.')
 * 
 *     if not _cupyx.scipy.sparse.isspmatrix_csr(A):             # <<<<<<<<<<<<<<
 *         raise ValueError('A must be CSR sparse matrix')
 *     if not isinstance(b, _cupy.ndarray):
*/
  __Pyx_GetModuleGlobalName(__pyx_t_2, __pyx_mstate_global->__pyx_n_u_cupyx); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 831, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_7 = __Pyx_PyObject_GetAttrStr(__pyx_t_2, __pyx_mstate_global->__pyx_n_u_scipy); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 831, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_7);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_t_7, __pyx_mstate_global->__pyx_n_u_sparse); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 831, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
  __pyx_t_3 = __pyx_t_2;
  __Pyx_INCREF(__pyx_t_3);
  __pyx_t_4 = 0;
  {
    PyObject *__pyx_callargs[2] = {__pyx_t_3, __pyx_v_A};
    __pyx_t_1 = __Pyx_PyObject_FastCallMethod((PyObject*)__pyx_mstate_global->__pyx_n_u_isspmatrix_csr, __pyx_callargs+__pyx_t_4, (2-__pyx_t_4) | (1*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
    __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 831, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
  }
  __pyx_t_6 = __Pyx_PyObject_IsTrue(__pyx_t_1); if (unlikely((__pyx_t_6 < 0))) __PYX_ERR(0, 831, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_5 = (!__pyx_t_6);
  if (unlikely(__pyx_t_5)) {

    /* "cupyx/cusolver.pyx":832
 * 
 *     if not _cupyx.scipy.sparse.isspmatrix_csr(A):
 *         raise ValueError('A must be CSR sparse matrix')             # <<<<<<<<<<<<<<
 *     if not isinstance(b, _cupy.ndarray):
 *         raise ValueError('b must be cupy.ndarray')
*/
    __pyx_t_2 = NULL;
    __pyx_t_4 = 1;
    {
      PyObject *__pyx_callargs[2] = {__pyx_t_2, __pyx_mstate_global->__pyx_kp_u_A_must_be_CSR_sparse_matrix};
      __pyx_t_1 = __Pyx_PyObject_FastCall((PyObject*)(((PyTypeObject*)PyExc_ValueError)), __pyx_callargs+__pyx_t_4, (2-__pyx_t_4) | (__pyx_t_4*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
      __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
      if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 832, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
    }
    __Pyx_Raise(__pyx_t_1, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __PYX_ERR(0, 832, __pyx_L1_error)

    /* "cupyx/cusolver.pyx":831
 *         raise RuntimeError('csrlsvqr is not available.')
 * 
 *     if not _cupyx.scipy.sparse.isspmatrix_csr(A):             # <<<<<<<<<<<<<<
 *         raise ValueError('A must be CSR sparse matrix')
 *     if not isinstance(b, _cupy.ndarray):
*/
  }

  /* "cupyx/cusolver.pyx":833
 *     if not _cupyx.scipy.sparse.isspmatrix_csr(A):
 *         raise ValueError('A must be CSR sparse matrix')
 *     if not isinstance(b, _cupy.ndarray):             # <<<<<<<<<<<<<<
 *         raise ValueError('b must be cupy.ndarray')
 *     if b.ndim != 1:
*/
  __Pyx_GetModuleGlobalName(__pyx_t_1, __pyx_mstate_global->__pyx_n_u_cupy); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 833, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_t_1, __pyx_mstate_global->__pyx_n_u_ndarray); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 833, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_5 = PyObject_IsInstance(__pyx_v_b, __pyx_t_2); if (unlikely(__pyx_t_5 == ((int)-1))) __PYX_ERR(0, 833, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_6 = (!__pyx_t_5);
  if (unlikely(__pyx_t_6)) {

    /* "cupyx/cusolver.pyx":834
 *         raise ValueError('A must be CSR sparse matrix')
 *     if not isinstance(b, _cupy.ndarray):
 *         raise ValueError('b must be cupy.ndarray')             # <<<<<<<<<<<<<<
 *     if b.ndim != 1:
 *         raise ValueError('b.ndim must be 1 (actual: {})'.format(b.ndim))
*/
    __pyx_t_1 = NULL;
    __pyx_t_4 = 1;
    {
      PyObject *__pyx_callargs[2] = {__pyx_t_1, __pyx_mstate_global->__pyx_kp_u_b_must_be_cupy_ndarray};
      __pyx_t_2 = __Pyx_PyObject_FastCall((PyObject*)(((PyTypeObject*)PyExc_ValueError)), __pyx_callargs+__pyx_t_4, (2-__pyx_t_4) | (__pyx_t_4*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
      __Pyx_XDECREF(__pyx_t_1); __pyx_t_1 = 0;
      if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 834, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_2);
    }
    __Pyx_Raise(__pyx_t_2, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __PYX_ERR(0, 834, __pyx_L1_error)

    /* "cupyx/cusolver.pyx":833
 *     if not _cupyx.scipy.sparse.isspmatrix_csr(A):
 *         raise ValueError('A must be CSR sparse matrix')
 *     if not isinstance(b, _cupy.ndarray):             # <<<<<<<<<<<<<<
 *         raise ValueError('b must be cupy.ndarray')
 *     if b.ndim != 1:
*/
  }

  /* "cupyx/cusolver.pyx":835
 *     if not isinstance(b, _cupy.ndarray):
 *         raise ValueError('b must be cupy.ndarray')
 *     if b.ndim != 1:             # <<<<<<<<<<<<<<
 *         raise ValueError('b.ndim must be 1 (actual: {})'.format(b.ndim))
 *     if not (A.shape[0] == A.shape[1] == b.shape[0]):
*/
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_v_b, __pyx_mstate_global->__pyx_n_u_ndim); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 835, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_6 = (__Pyx_PyLong_BoolNeObjC(__pyx_t_2, __pyx_mstate_global->__pyx_int_1, 1, 0)); if (unlikely((__pyx_t_6 < 0))) __PYX_ERR(0, 835, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  if (unlikely(__pyx_t_6)) {

    /* "cupyx/cusolver.pyx":836
 *         raise ValueError('b must be cupy.ndarray')
 *     if b.ndim != 1:
 *         raise ValueError('b.ndim must be 1 (actual: {})'.format(b.ndim))             # <<<<<<<<<<<<<<
 *     if not (A.shape[0] == A.shape[1] == b.shape[0]):
 *         raise ValueError('invalid shape')
*/
    __pyx_t_1 = NULL;
    __pyx_t_7 = __pyx_mstate_global->__pyx_kp_u_b_ndim_must_be_1_actual;
    __Pyx_INCREF(__pyx_t_7);
    __pyx_t_8 = __Pyx_PyObject_GetAttrStr(__pyx_v_b, __pyx_mstate_global->__pyx_n_u_ndim); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 836, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_8);
    __pyx_t_4 = 0;
    {
      PyObject *__pyx_callargs[2] = {__pyx_t_7, __pyx_t_8};
      __pyx_t_3 = __Pyx_PyObject_FastCallMethod((PyObject*)__pyx_mstate_global->__pyx_n_u_format, __pyx_callargs+__pyx_t_4, (2-__pyx_t_4) | (1*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
      __Pyx_XDECREF(__pyx_t_7); __pyx_t_7 = 0;
      __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
      if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 836, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_3);
    }
    __pyx_t_4 = 1;
    {
      PyObject *__pyx_callargs[2] = {__pyx_t_1, __pyx_t_3};
      __pyx_t_2 = __Pyx_PyObject_FastCall((PyObject*)(((PyTypeObject*)PyExc_ValueError)), __pyx_callargs+__pyx_t_4, (2-__pyx_t_4) | (__pyx_t_4*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
      __Pyx_XDECREF(__pyx_t_1); __pyx_t_1 = 0;
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
      if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 836, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_2);
    }
    __Pyx_Raise(__pyx_t_2, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __PYX_ERR(0, 836, __pyx_L1_error)

    /* "cupyx/cusolver.pyx":835
 *     if not isinstance(b, _cupy.ndarray):
 *         raise ValueError('b must be cupy.ndarray')
 *     if b.ndim != 1:             # <<<<<<<<<<<<<<
 *         raise ValueError('b.ndim must be 1 (actual: {})'.format(b.ndim))
 *     if not (A.shape[0] == A.shape[1] == b.shape[0]):
*/
  }

  /* "cupyx/cusolver.pyx":837
 *     if b.ndim != 1:
 *         raise ValueError('b.ndim must be 1 (actual: {})'.format(b.ndim))
 *     if not (A.shape[0] == A.shape[1] == b.shape[0]):             # <<<<<<<<<<<<<<
 *         raise ValueError('invalid shape')
 *     if A.dtype != b.dtype:
*/
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_v_A, __pyx_mstate_global->__pyx_n_u_shape); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 837, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = __Pyx_GetItemInt(__pyx_t_2, 0, long, 1, __Pyx_PyLong_From_long, 0, 0, 1, 1, __Pyx_ReferenceSharing_OwnStrongReference); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 837, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_v_A, __pyx_mstate_global->__pyx_n_u_shape); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 837, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_1 = __Pyx_GetItemInt(__pyx_t_2, 1, long, 1, __Pyx_PyLong_From_long, 0, 0, 1, 1, __Pyx_ReferenceSharing_OwnStrongReference); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 837, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_2 = PyObject_RichCompare(__pyx_t_3, __pyx_t_1, Py_EQ); __Pyx_XGOTREF(__pyx_t_2); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 837, __pyx_L1_error)
  if (__Pyx_PyObject_IsTrue(__pyx_t_2)) {
    __Pyx_DECREF(__pyx_t_2);
    __pyx_t_8 = __Pyx_PyObject_GetAttrStr(__pyx_v_b, __pyx_mstate_global->__pyx_n_u_shape); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 837, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_8);
    __pyx_t_7 = __Pyx_GetItemInt(__pyx_t_8, 0, long, 1, __Pyx_PyLong_From_long, 0, 0, 1, 1, __Pyx_ReferenceSharing_OwnStrongReference); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 837, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_7);
    __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
    __pyx_t_2 = PyObject_RichCompare(__pyx_t_1, __pyx_t_7, Py_EQ); __Pyx_XGOTREF(__pyx_t_2); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 837, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
  }
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_6 = __Pyx_PyObject_IsTrue(__pyx_t_2); if (unlikely((__pyx_t_6 < 0))) __PYX_ERR(0, 837, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_5 = (!__pyx_t_6);
  if (unlikely(__pyx_t_5)) {

    /* "cupyx/cusolver.pyx":838
 *         raise ValueError('b.ndim must be 1 (actual: {})'.format(b.ndim))
 *     if not (A.shape[0] == A.shape[1] == b.shape[0]):
 *         raise ValueError('invalid shape')             # <<<<<<<<<<<<<<
 *     if A.dtype != b.dtype:
 *         raise TypeError('dtype mismatch')
*/
    __pyx_t_1 = NULL;
    __pyx_t_4 = 1;
    {
      PyObject *__pyx_callargs[2] = {__pyx_t_1, __pyx_mstate_global->__pyx_kp_u_invalid_shape};
      __pyx_t_2 = __Pyx_PyObject_FastCall((PyObject*)(((PyTypeObject*)PyExc_ValueError)), __pyx_callargs+__pyx_t_4, (2-__pyx_t_4) | (__pyx_t_4*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
      __Pyx_XDECREF(__pyx_t_1); __pyx_t_1 = 0;
      if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 838, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_2);
    }
    __Pyx_Raise(__pyx_t_2, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __PYX_ERR(0, 838, __pyx_L1_error)

    /* "cupyx/cusolver.pyx":837
 *     if b.ndim != 1:
 *         raise ValueError('b.ndim must be 1 (actual: {})'.format(b.ndim))
 *     if not (A.shape[0] == A.shape[1] == b.shape[0]):             # <<<<<<<<<<<<<<
 *         raise ValueError('invalid shape')
 *     if A.dtype != b.dtype:
*/
  }

  /* "cupyx/cusolver.pyx":839
 *     if not (A.shape[0] == A.shape[1] == b.shape[0]):
 *         raise ValueError('invalid shape')
 *     if A.dtype != b.dtype:             # <<<<<<<<<<<<<<
 *         raise TypeError('dtype mismatch')
 * 
*/
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_v_A, __pyx_mstate_global->__pyx_n_u_dtype); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 839, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_v_b, __pyx_mstate_global->__pyx_n_u_dtype); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 839, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_3 = PyObject_RichCompare(__pyx_t_2, __pyx_t_1, Py_NE); __Pyx_XGOTREF(__pyx_t_3); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 839, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_5 = __Pyx_PyObject_IsTrue(__pyx_t_3); if (unlikely((__pyx_t_5 < 0))) __PYX_ERR(0, 839, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  if (unlikely(__pyx_t_5)) {

    /* "cupyx/cusolver.pyx":840
 *         raise ValueError('invalid shape')
 *     if A.dtype != b.dtype:
 *         raise TypeError('dtype mismatch')             # <<<<<<<<<<<<<<
 * 
 *     dtype = A.dtype
*/
    __pyx_t_1 = NULL;
    __pyx_t_4 = 1;
    {
      PyObject *__pyx_callargs[2] = {__pyx_t_1, __pyx_mstate_global->__pyx_kp_u_dtype_mismatch};
      __pyx_t_3 = __Pyx_PyObject_FastCall((PyObject*)(((PyTypeObject*)PyExc_TypeError)), __pyx_callargs+__pyx_t_4, (2-__pyx_t_4) | (__pyx_t_4*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
      __Pyx_XDECREF(__pyx_t_1); __pyx_t_1 = 0;
      if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 840, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_3);
    }
    __Pyx_Raise(__pyx_t_3, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __PYX_ERR(0, 840, __pyx_L1_error)

    /* "cupyx/cusolver.pyx":839
 *     if not (A.shape[0] == A.shape[1] == b.shape[0]):
 *         raise ValueError('invalid shape')
 *     if A.dtype != b.dtype:             # <<<<<<<<<<<<<<
 *         raise TypeError('dtype mismatch')
 * 
*/
  }

  /* "cupyx/cusolver.pyx":842
 *         raise TypeError('dtype mismatch')
 * 
 *     dtype = A.dtype             # <<<<<<<<<<<<<<
 *     if dtype.char == 'f':
 *         t = 's'
*/
  __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_v_A, __pyx_mstate_global->__pyx_n_u_dtype); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 842, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_v_dtype = __pyx_t_3;
  __pyx_t_3 = 0;

  /* "cupyx/cusolver.pyx":843
 * 
 *     dtype = A.dtype
 *     if dtype.char == 'f':             # <<<<<<<<<<<<<<
 *         t = 's'
 *     elif dtype.char == 'd':
*/
  __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_v_dtype, __pyx_mstate_global->__pyx_n_u_char); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 843, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_5 = (__Pyx_PyUnicode_Equals(__pyx_t_3, __pyx_mstate_global->__pyx_n_u_f, Py_EQ)); if (unlikely((__pyx_t_5 < 0))) __PYX_ERR(0, 843, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  if (__pyx_t_5) {

    /* "cupyx/cusolver.pyx":844
 *     dtype = A.dtype
 *     if dtype.char == 'f':
 *         t = 's'             # <<<<<<<<<<<<<<
 *     elif dtype.char == 'd':
 *         t = 'd'
*/
    __Pyx_INCREF(__pyx_mstate_global->__pyx_n_u_s);
    __pyx_v_t = __pyx_mstate_global->__pyx_n_u_s;

    /* "cupyx/cusolver.pyx":843
 * 
 *     dtype = A.dtype
 *     if dtype.char == 'f':             # <<<<<<<<<<<<<<
 *         t = 's'
 *     elif dtype.char == 'd':
*/
    goto __pyx_L9;
  }

  /* "cupyx/cusolver.pyx":845
 *     if dtype.char == 'f':
 *         t = 's'
 *     elif dtype.char == 'd':             # <<<<<<<<<<<<<<
 *         t = 'd'
 *     elif dtype.char == 'F':
*/
  __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_v_dtype, __pyx_mstate_global->__pyx_n_u_char); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 845, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_5 = (__Pyx_PyUnicode_Equals(__pyx_t_3, __pyx_mstate_global->__pyx_n_u_d, Py_EQ)); if (unlikely((__pyx_t_5 < 0))) __PYX_ERR(0, 845, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  if (__pyx_t_5) {

    /* "cupyx/cusolver.pyx":846
 *         t = 's'
 *     elif dtype.char == 'd':
 *         t = 'd'             # <<<<<<<<<<<<<<
 *     elif dtype.char == 'F':
 *         t = 'c'
*/
    __Pyx_INCREF(__pyx_mstate_global->__pyx_n_u_d);
    __pyx_v_t = __pyx_mstate_global->__pyx_n_u_d;

    /* "cupyx/cusolver.pyx":845
 *     if dtype.char == 'f':
 *         t = 's'
 *     elif dtype.char == 'd':             # <<<<<<<<<<<<<<
 *         t = 'd'
 *     elif dtype.char == 'F':
*/
    goto __pyx_L9;
  }

  /* "cupyx/cusolver.pyx":847
 *     elif dtype.char == 'd':
 *         t = 'd'
 *     elif dtype.char == 'F':             # <<<<<<<<<<<<<<
 *         t = 'c'
 *     elif dtype.char == 'D':
*/
  __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_v_dtype, __pyx_mstate_global->__pyx_n_u_char); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 847, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_5 = (__Pyx_PyUnicode_Equals(__pyx_t_3, __pyx_mstate_global->__pyx_n_u_F, Py_EQ)); if (unlikely((__pyx_t_5 < 0))) __PYX_ERR(0, 847, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  if (__pyx_t_5) {

    /* "cupyx/cusolver.pyx":848
 *         t = 'd'
 *     elif dtype.char == 'F':
 *         t = 'c'             # <<<<<<<<<<<<<<
 *     elif dtype.char == 'D':
 *         t = 'z'
*/
    __Pyx_INCREF(__pyx_mstate_global->__pyx_n_u_c);
    __pyx_v_t = __pyx_mstate_global->__pyx_n_u_c;

    /* "cupyx/cusolver.pyx":847
 *     elif dtype.char == 'd':
 *         t = 'd'
 *     elif dtype.char == 'F':             # <<<<<<<<<<<<<<
 *         t = 'c'
 *     elif dtype.char == 'D':
*/
    goto __pyx_L9;
  }

  /* "cupyx/cusolver.pyx":849
 *     elif dtype.char == 'F':
 *         t = 'c'
 *     elif dtype.char == 'D':             # <<<<<<<<<<<<<<
 *         t = 'z'
 *     else:
*/
  __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_v_dtype, __pyx_mstate_global->__pyx_n_u_char); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 849, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_5 = (__Pyx_PyUnicode_Equals(__pyx_t_3, __pyx_mstate_global->__pyx_n_u_D, Py_EQ)); if (unlikely((__pyx_t_5 < 0))) __PYX_ERR(0, 849, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  if (likely(__pyx_t_5)) {

    /* "cupyx/cusolver.pyx":850
 *         t = 'c'
 *     elif dtype.char == 'D':
 *         t = 'z'             # <<<<<<<<<<<<<<
 *     else:
 *         raise TypeError('Invalid dtype (actual: {})'.format(dtype))
*/
    __Pyx_INCREF(__pyx_mstate_global->__pyx_n_u_z);
    __pyx_v_t = __pyx_mstate_global->__pyx_n_u_z;

    /* "cupyx/cusolver.pyx":849
 *     elif dtype.char == 'F':
 *         t = 'c'
 *     elif dtype.char == 'D':             # <<<<<<<<<<<<<<
 *         t = 'z'
 *     else:
*/
    goto __pyx_L9;
  }

  /* "cupyx/cusolver.pyx":852
 *         t = 'z'
 *     else:
 *         raise TypeError('Invalid dtype (actual: {})'.format(dtype))             # <<<<<<<<<<<<<<
 *     solve = getattr(_cusolver, t + 'csrlsvqr')
 * 
*/
  /*else*/ {
    __pyx_t_1 = NULL;
    __pyx_t_7 = __pyx_mstate_global->__pyx_kp_u_Invalid_dtype_actual;
    __Pyx_INCREF(__pyx_t_7);
    __pyx_t_4 = 0;
    {
      PyObject *__pyx_callargs[2] = {__pyx_t_7, __pyx_v_dtype};
      __pyx_t_2 = __Pyx_PyObject_FastCallMethod((PyObject*)__pyx_mstate_global->__pyx_n_u_format, __pyx_callargs+__pyx_t_4, (2-__pyx_t_4) | (1*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
      __Pyx_XDECREF(__pyx_t_7); __pyx_t_7 = 0;
      if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 852, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_2);
    }
    __pyx_t_4 = 1;
    {
      PyObject *__pyx_callargs[2] = {__pyx_t_1, __pyx_t_2};
      __pyx_t_3 = __Pyx_PyObject_FastCall((PyObject*)(((PyTypeObject*)PyExc_TypeError)), __pyx_callargs+__pyx_t_4, (2-__pyx_t_4) | (__pyx_t_4*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
      __Pyx_XDECREF(__pyx_t_1); __pyx_t_1 = 0;
      __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
      if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 852, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_3);
    }
    __Pyx_Raise(__pyx_t_3, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __PYX_ERR(0, 852, __pyx_L1_error)
  }
  __pyx_L9:;

  /* "cupyx/cusolver.pyx":853
 *     else:
 *         raise TypeError('Invalid dtype (actual: {})'.format(dtype))
 *     solve = getattr(_cusolver, t + 'csrlsvqr')             # <<<<<<<<<<<<<<
 * 
 *     tol = max(tol, 0)
*/
  __Pyx_GetModuleGlobalName(__pyx_t_3, __pyx_mstate_global->__pyx_n_u_cusolver); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 853, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_2 = __Pyx_PyUnicode_Concat(__pyx_v_t, __pyx_mstate_global->__pyx_n_u_csrlsvqr); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 853, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_1 = __Pyx_GetAttr(__pyx_t_3, __pyx_t_2); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 853, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_v_solve = __pyx_t_1;
  __pyx_t_1 = 0;

  /* "cupyx/cusolver.pyx":855
 *     solve = getattr(_cusolver, t + 'csrlsvqr')
 * 
 *     tol = max(tol, 0)             # <<<<<<<<<<<<<<
 *     m = A.shape[0]
 *     b = _internal_ascontiguousarray(b)
*/
  __pyx_t_9 = 0;
  __Pyx_INCREF(__pyx_v_tol);
  __pyx_t_1 = __pyx_v_tol;
  __pyx_t_3 = __Pyx_PyLong_From_long(__pyx_t_9); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 855, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_7 = PyObject_RichCompare(__pyx_t_3, __pyx_t_1, Py_GT); __Pyx_XGOTREF(__pyx_t_7); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 855, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_t_5 = __Pyx_PyObject_IsTrue(__pyx_t_7); if (unlikely((__pyx_t_5 < 0))) __PYX_ERR(0, 855, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
  if (__pyx_t_5) {
    __pyx_t_7 = __Pyx_PyLong_From_long(__pyx_t_9); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 855, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_7);
    __pyx_t_2 = __pyx_t_7;
    __pyx_t_7 = 0;
  } else {
    __Pyx_INCREF(__pyx_t_1);
    __pyx_t_2 = __pyx_t_1;
  }
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_1 = __pyx_t_2;
  __Pyx_INCREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __Pyx_DECREF_SET(__pyx_v_tol, __pyx_t_1);
  __pyx_t_1 = 0;

  /* "cupyx/cusolver.pyx":856
 * 
 *     tol = max(tol, 0)
 *     m = A.shape[0]             # <<<<<<<<<<<<<<
 *     b = _internal_ascontiguousarray(b)
 *     x = _cupy.empty((m,), dtype=dtype)
*/
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_v_A, __pyx_mstate_global->__pyx_n_u_shape); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 856, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_GetItemInt(__pyx_t_1, 0, long, 1, __Pyx_PyLong_From_long, 0, 0, 1, 1, __Pyx_ReferenceSharing_OwnStrongReference); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 856, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v_m = __pyx_t_2;
  __pyx_t_2 = 0;

  /* "cupyx/cusolver.pyx":857
 *     tol = max(tol, 0)
 *     m = A.shape[0]
 *     b = _internal_ascontiguousarray(b)             # <<<<<<<<<<<<<<
 *     x = _cupy.empty((m,), dtype=dtype)
 *     singularity = _numpy.empty((1,), _numpy.int32)
*/
  if (!(likely(((__pyx_v_b) == Py_None) || likely(__Pyx_TypeTest(__pyx_v_b, __pyx_mstate_global->__pyx_ptype_4cupy_5_core_4core__ndarray_base))))) __PYX_ERR(0, 857, __pyx_L1_error)
  __pyx_t_2 = ((PyObject *)__pyx_f_4cupy_5_core_4core__internal_ascontiguousarray(((struct __pyx_obj_4cupy_5_core_4core__ndarray_base *)__pyx_v_b), 0)); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 857, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF_SET(__pyx_v_b, __pyx_t_2);
  __pyx_t_2 = 0;

  /* "cupyx/cusolver.pyx":858
 *     m = A.shape[0]
 *     b = _internal_ascontiguousarray(b)
 *     x = _cupy.empty((m,), dtype=dtype)             # <<<<<<<<<<<<<<
 *     singularity = _numpy.empty((1,), _numpy.int32)
 * 
*/
  __pyx_t_1 = NULL;
  __Pyx_GetModuleGlobalName(__pyx_t_7, __pyx_mstate_global->__pyx_n_u_cupy); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 858, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_7);
  __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_t_7, __pyx_mstate_global->__pyx_n_u_empty); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 858, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
  __pyx_t_7 = PyTuple_New(1); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 858, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_7);
  __Pyx_INCREF(__pyx_v_m);
  __Pyx_GIVEREF(__pyx_v_m);
  if (__Pyx_PyTuple_SET_ITEM(__pyx_t_7, 0, __pyx_v_m) != (0)) __PYX_ERR(0, 858, __pyx_L1_error);
  __pyx_t_4 = 1;
  #if CYTHON_UNPACK_METHODS
  if (unlikely(PyMethod_Check(__pyx_t_3))) {
    __pyx_t_1 = PyMethod_GET_SELF(__pyx_t_3);
    assert(__pyx_t_1);
    PyObject* __pyx__function = PyMethod_GET_FUNCTION(__pyx_t_3);
    __Pyx_INCREF(__pyx_t_1);
    __Pyx_INCREF(__pyx__function);
    __Pyx_DECREF_SET(__pyx_t_3, __pyx__function);
    __pyx_t_4 = 0;
  }
  #endif
  {
    PyObject *__pyx_callargs[2 + ((CYTHON_VECTORCALL) ? 1 : 0)] = {__pyx_t_1, __pyx_t_7};
    __pyx_t_8 = __Pyx_MakeVectorcallBuilderKwds(1); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 858, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_8);
    if (__Pyx_VectorcallBuilder_AddArg(__pyx_mstate_global->__pyx_n_u_dtype, __pyx_v_dtype, __pyx_t_8, __pyx_callargs+2, 0) < (0)) __PYX_ERR(0, 858, __pyx_L1_error)
    __pyx_t_2 = __Pyx_Object_Vectorcall_CallFromBuilder((PyObject*)__pyx_t_3, __pyx_callargs+__pyx_t_4, (2-__pyx_t_4) | (__pyx_t_4*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET), __pyx_t_8);
    __Pyx_XDECREF(__pyx_t_1); __pyx_t_1 = 0;
    __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
    __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 858, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
  }
  __pyx_v_x = __pyx_t_2;
  __pyx_t_2 = 0;

  /* "cupyx/cusolver.pyx":859
 *     b = _internal_ascontiguousarray(b)
 *     x = _cupy.empty((m,), dtype=dtype)
 *     singularity = _numpy.empty((1,), _numpy.int32)             # <<<<<<<<<<<<<<
 * 
 *     handle = _device.get_cusolver_sp_handle()
*/
  __pyx_t_3 = NULL;
  __Pyx_GetModuleGlobalName(__pyx_t_8, __pyx_mstate_global->__pyx_n_u_numpy); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 859, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_8);
  __pyx_t_7 = __Pyx_PyObject_GetAttrStr(__pyx_t_8, __pyx_mstate_global->__pyx_n_u_empty); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 859, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_7);
  __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
  __Pyx_GetModuleGlobalName(__pyx_t_8, __pyx_mstate_global->__pyx_n_u_numpy); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 859, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_8);
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_t_8, __pyx_mstate_global->__pyx_n_u_int32); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 859, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
  __pyx_t_4 = 1;
  #if CYTHON_UNPACK_METHODS
  if (unlikely(PyMethod_Check(__pyx_t_7))) {
    __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_7);
    assert(__pyx_t_3);
    PyObject* __pyx__function = PyMethod_GET_FUNCTION(__pyx_t_7);
    __Pyx_INCREF(__pyx_t_3);
    __Pyx_INCREF(__pyx__function);
    __Pyx_DECREF_SET(__pyx_t_7, __pyx__function);
    __pyx_t_4 = 0;
  }
  #endif
  {
    PyObject *__pyx_callargs[3] = {__pyx_t_3, __pyx_mstate_global->__pyx_tuple[3], __pyx_t_1};
    __pyx_t_2 = __Pyx_PyObject_FastCall((PyObject*)__pyx_t_7, __pyx_callargs+__pyx_t_4, (3-__pyx_t_4) | (__pyx_t_4*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
    __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
    if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 859, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
  }
  __pyx_v_singularity = __pyx_t_2;
  __pyx_t_2 = 0;

  /* "cupyx/cusolver.pyx":861
 *     singularity = _numpy.empty((1,), _numpy.int32)
 * 
 *     handle = _device.get_cusolver_sp_handle()             # <<<<<<<<<<<<<<
 *     solve(handle, m, A.nnz, A._descr.descriptor, A.data.data.ptr,
 *           A.indptr.data.ptr, A.indices.data.ptr, b.data.ptr, tol, reorder,
*/
  __pyx_t_7 = NULL;
  __Pyx_GetModuleGlobalName(__pyx_t_1, __pyx_mstate_global->__pyx_n_u_device); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 861, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_t_1, __pyx_mstate_global->__pyx_n_u_get_cusolver_sp_handle); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 861, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_4 = 1;
  #if CYTHON_UNPACK_METHODS
  if (unlikely(PyMethod_Check(__pyx_t_3))) {
    __pyx_t_7 = PyMethod_GET_SELF(__pyx_t_3);
    assert(__pyx_t_7);
    PyObject* __pyx__function = PyMethod_GET_FUNCTION(__pyx_t_3);
    __Pyx_INCREF(__pyx_t_7);
    __Pyx_INCREF(__pyx__function);
    __Pyx_DECREF_SET(__pyx_t_3, __pyx__function);
    __pyx_t_4 = 0;
  }
  #endif
  {
    PyObject *__pyx_callargs[2] = {__pyx_t_7, NULL};
    __pyx_t_2 = __Pyx_PyObject_FastCall((PyObject*)__pyx_t_3, __pyx_callargs+__pyx_t_4, (1-__pyx_t_4) | (__pyx_t_4*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
    __Pyx_XDECREF(__pyx_t_7); __pyx_t_7 = 0;
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 861, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
  }
  __pyx_v_handle = __pyx_t_2;
  __pyx_t_2 = 0;

  /* "cupyx/cusolver.pyx":862
 * 
 *     handle = _device.get_cusolver_sp_handle()
 *     solve(handle, m, A.nnz, A._descr.descriptor, A.data.data.ptr,             # <<<<<<<<<<<<<<
 *           A.indptr.data.ptr, A.indices.data.ptr, b.data.ptr, tol, reorder,
 *           x.data.ptr, singularity.ctypes.data)
*/
  __pyx_t_3 = NULL;
  __Pyx_INCREF(__pyx_v_solve);
  __pyx_t_7 = __pyx_v_solve; 
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_v_A, __pyx_mstate_global->__pyx_n_u_nnz); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 862, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_8 = __Pyx_PyObject_GetAttrStr(__pyx_v_A, __pyx_mstate_global->__pyx_n_u_descr); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 862, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_8);
  __pyx_t_10 = __Pyx_PyObject_GetAttrStr(__pyx_t_8, __pyx_mstate_global->__pyx_n_u_descriptor); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 862, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_10);
  __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
  __pyx_t_8 = __Pyx_PyObject_GetAttrStr(__pyx_v_A, __pyx_mstate_global->__pyx_n_u_data); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 862, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_8);
  __pyx_t_11 = __Pyx_PyObject_GetAttrStr(__pyx_t_8, __pyx_mstate_global->__pyx_n_u_data); if (unlikely(!__pyx_t_11)) __PYX_ERR(0, 862, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_11);
  __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
  __pyx_t_8 = __Pyx_PyObject_GetAttrStr(__pyx_t_11, __pyx_mstate_global->__pyx_n_u_ptr); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 862, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_8);
  __Pyx_DECREF(__pyx_t_11); __pyx_t_11 = 0;

  /* "cupyx/cusolver.pyx":863
 *     handle = _device.get_cusolver_sp_handle()
 *     solve(handle, m, A.nnz, A._descr.descriptor, A.data.data.ptr,
 *           A.indptr.data.ptr, A.indices.data.ptr, b.data.ptr, tol, reorder,             # <<<<<<<<<<<<<<
 *           x.data.ptr, singularity.ctypes.data)
 * 
*/
  __pyx_t_11 = __Pyx_PyObject_GetAttrStr(__pyx_v_A, __pyx_mstate_global->__pyx_n_u_indptr); if (unlikely(!__pyx_t_11)) __PYX_ERR(0, 863, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_11);
  __pyx_t_12 = __Pyx_PyObject_GetAttrStr(__pyx_t_11, __pyx_mstate_global->__pyx_n_u_data); if (unlikely(!__pyx_t_12)) __PYX_ERR(0, 863, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_12);
  __Pyx_DECREF(__pyx_t_11); __pyx_t_11 = 0;
  __pyx_t_11 = __Pyx_PyObject_GetAttrStr(__pyx_t_12, __pyx_mstate_global->__pyx_n_u_ptr); if (unlikely(!__pyx_t_11)) __PYX_ERR(0, 863, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_11);
  __Pyx_DECREF(__pyx_t_12); __pyx_t_12 = 0;
  __pyx_t_12 = __Pyx_PyObject_GetAttrStr(__pyx_v_A, __pyx_mstate_global->__pyx_n_u_indices); if (unlikely(!__pyx_t_12)) __PYX_ERR(0, 863, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_12);
  __pyx_t_13 = __Pyx_PyObject_GetAttrStr(__pyx_t_12, __pyx_mstate_global->__pyx_n_u_data); if (unlikely(!__pyx_t_13)) __PYX_ERR(0, 863, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_13);
  __Pyx_DECREF(__pyx_t_12); __pyx_t_12 = 0;
  __pyx_t_12 = __Pyx_PyObject_GetAttrStr(__pyx_t_13, __pyx_mstate_global->__pyx_n_u_ptr); if (unlikely(!__pyx_t_12)) __PYX_ERR(0, 863, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_12);
  __Pyx_DECREF(__pyx_t_13); __pyx_t_13 = 0;
  __pyx_t_13 = __Pyx_PyObject_GetAttrStr(__pyx_v_b, __pyx_mstate_global->__pyx_n_u_data); if (unlikely(!__pyx_t_13)) __PYX_ERR(0, 863, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_13);
  __pyx_t_14 = __Pyx_PyObject_GetAttrStr(__pyx_t_13, __pyx_mstate_global->__pyx_n_u_ptr); if (unlikely(!__pyx_t_14)) __PYX_ERR(0, 863, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_14);
  __Pyx_DECREF(__pyx_t_13); __pyx_t_13 = 0;

  /* "cupyx/cusolver.pyx":864
 *     solve(handle, m, A.nnz, A._descr.descriptor, A.data.data.ptr,
 *           A.indptr.data.ptr, A.indices.data.ptr, b.data.ptr, tol, reorder,
 *           x.data.ptr, singularity.ctypes.data)             # <<<<<<<<<<<<<<
 * 
 *     if singularity[0] >= 0:
*/
  __pyx_t_13 = __Pyx_PyObject_GetAttrStr(__pyx_v_x, __pyx_mstate_global->__pyx_n_u_data); if (unlikely(!__pyx_t_13)) __PYX_ERR(0, 864, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_13);
  __pyx_t_15 = __Pyx_PyObject_GetAttrStr(__pyx_t_13, __pyx_mstate_global->__pyx_n_u_ptr); if (unlikely(!__pyx_t_15)) __PYX_ERR(0, 864, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_15);
  __Pyx_DECREF(__pyx_t_13); __pyx_t_13 = 0;
  __pyx_t_13 = __Pyx_PyObject_GetAttrStr(__pyx_v_singularity, __pyx_mstate_global->__pyx_n_u_ctypes); if (unlikely(!__pyx_t_13)) __PYX_ERR(0, 864, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_13);
  __pyx_t_16 = __Pyx_PyObject_GetAttrStr(__pyx_t_13, __pyx_mstate_global->__pyx_n_u_data); if (unlikely(!__pyx_t_16)) __PYX_ERR(0, 864, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_16);
  __Pyx_DECREF(__pyx_t_13); __pyx_t_13 = 0;
  __pyx_t_4 = 1;
  #if CYTHON_UNPACK_METHODS
  if (unlikely(PyMethod_Check(__pyx_t_7))) {
    __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_7);
    assert(__pyx_t_3);
    PyObject* __pyx__function = PyMethod_GET_FUNCTION(__pyx_t_7);
    __Pyx_INCREF(__pyx_t_3);
    __Pyx_INCREF(__pyx__function);
    __Pyx_DECREF_SET(__pyx_t_7, __pyx__function);
    __pyx_t_4 = 0;
  }
  #endif
  {
    PyObject *__pyx_callargs[13] = {__pyx_t_3, __pyx_v_handle, __pyx_v_m, __pyx_t_1, __pyx_t_10, __pyx_t_8, __pyx_t_11, __pyx_t_12, __pyx_t_14, __pyx_v_tol, __pyx_v_reorder, __pyx_t_15, __pyx_t_16};
    __pyx_t_2 = __Pyx_PyObject_FastCall((PyObject*)__pyx_t_7, __pyx_callargs+__pyx_t_4, (13-__pyx_t_4) | (__pyx_t_4*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
    __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
    __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
    __Pyx_DECREF(__pyx_t_11); __pyx_t_11 = 0;
    __Pyx_DECREF(__pyx_t_12); __pyx_t_12 = 0;
    __Pyx_DECREF(__pyx_t_14); __pyx_t_14 = 0;
    __Pyx_DECREF(__pyx_t_15); __pyx_t_15 = 0;
    __Pyx_DECREF(__pyx_t_16); __pyx_t_16 = 0;
    __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
    if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 862, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
  }
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "cupyx/cusolver.pyx":866
 *           x.data.ptr, singularity.ctypes.data)
 * 
 *     if singularity[0] >= 0:             # <<<<<<<<<<<<<<
 *         _warnings.warn('A is not positive definite or near singular under '
 *                        'tolerance {} (singularity: {})'.
*/
  __pyx_t_2 = __Pyx_GetItemInt(__pyx_v_singularity, 0, long, 1, __Pyx_PyLong_From_long, 0, 0, 1, 1, __Pyx_ReferenceSharing_OwnStrongReference); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 866, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_7 = PyObject_RichCompare(__pyx_t_2, __pyx_mstate_global->__pyx_int_0, Py_GE); __Pyx_XGOTREF(__pyx_t_7); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 866, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_5 = __Pyx_PyObject_IsTrue(__pyx_t_7); if (unlikely((__pyx_t_5 < 0))) __PYX_ERR(0, 866, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
  if (__pyx_t_5) {

    /* "cupyx/cusolver.pyx":867
 * 
 *     if singularity[0] >= 0:
 *         _warnings.warn('A is not positive definite or near singular under '             # <<<<<<<<<<<<<<
 *                        'tolerance {} (singularity: {})'.
 *                        format(tol, singularity))
*/
    __pyx_t_2 = NULL;
    __Pyx_GetModuleGlobalName(__pyx_t_16, __pyx_mstate_global->__pyx_n_u_warnings); if (unlikely(!__pyx_t_16)) __PYX_ERR(0, 867, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_16);
    __pyx_t_15 = __Pyx_PyObject_GetAttrStr(__pyx_t_16, __pyx_mstate_global->__pyx_n_u_warn); if (unlikely(!__pyx_t_15)) __PYX_ERR(0, 867, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_15);
    __Pyx_DECREF(__pyx_t_16); __pyx_t_16 = 0;

    /* "cupyx/cusolver.pyx":869
 *         _warnings.warn('A is not positive definite or near singular under '
 *                        'tolerance {} (singularity: {})'.
 *                        format(tol, singularity))             # <<<<<<<<<<<<<<
 *     return x
 * 
*/
    __pyx_t_14 = __pyx_mstate_global->__pyx_kp_u_A_is_not_positive_definite_or_ne;
    __Pyx_INCREF(__pyx_t_14);
    __pyx_t_4 = 0;
    {
      PyObject *__pyx_callargs[3] = {__pyx_t_14, __pyx_v_tol, __pyx_v_singularity};
      __pyx_t_16 = __Pyx_PyObject_FastCallMethod((PyObject*)__pyx_mstate_global->__pyx_n_u_format, __pyx_callargs+__pyx_t_4, (3-__pyx_t_4) | (1*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
      __Pyx_XDECREF(__pyx_t_14); __pyx_t_14 = 0;
      if (unlikely(!__pyx_t_16)) __PYX_ERR(0, 869, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_16);
    }
    __pyx_t_4 = 1;
    #if CYTHON_UNPACK_METHODS
    if (unlikely(PyMethod_Check(__pyx_t_15))) {
      __pyx_t_2 = PyMethod_GET_SELF(__pyx_t_15);
      assert(__pyx_t_2);
      PyObject* __pyx__function = PyMethod_GET_FUNCTION(__pyx_t_15);
      __Pyx_INCREF(__pyx_t_2);
      __Pyx_INCREF(__pyx__function);
      __Pyx_DECREF_SET(__pyx_t_15, __pyx__function);
      __pyx_t_4 = 0;
    }
    #endif
    {
      PyObject *__pyx_callargs[2] = {__pyx_t_2, __pyx_t_16};
      __pyx_t_7 = __Pyx_PyObject_FastCall((PyObject*)__pyx_t_15, __pyx_callargs+__pyx_t_4, (2-__pyx_t_4) | (__pyx_t_4*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
      __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
      __Pyx_DECREF(__pyx_t_16); __pyx_t_16 = 0;
      __Pyx_DECREF(__pyx_t_15); __pyx_t_15 = 0;
      if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 867, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_7);
    }
    __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;

    /* "cupyx/cusolver.pyx":866
 *           x.data.ptr, singularity.ctypes.data)
 * 
 *     if singularity[0] >= 0:             # <<<<<<<<<<<<<<
 *         _warnings.warn('A is not positive definite or near singular under '
 *                        'tolerance {} (singularity: {})'.
*/
  }

  /* "cupyx/cusolver.pyx":870
 *                        'tolerance {} (singularity: {})'.
 *                        format(tol, singularity))
 *     return x             # <<<<<<<<<<<<<<
 * 
 * 
*/
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_v_x);
  __pyx_r = __pyx_v_x;
  goto __pyx_L0;

  /* "cupyx/cusolver.pyx":814
 * 
 * 
 * def csrlsvqr(A, b, tol=0, reorder=1):             # <<<<<<<<<<<<<<
 *     """Solves the linear system ``Ax = b`` using QR factorization.
 * 
*/

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_7);
  __Pyx_XDECREF(__pyx_t_8);
  __Pyx_XDECREF(__pyx_t_10);
  __Pyx_XDECREF(__pyx_t_11);
  __Pyx_XDECREF(__pyx_t_12);
  __Pyx_XDECREF(__pyx_t_13);
  __Pyx_XDECREF(__pyx_t_14);
  __Pyx_XDECREF(__pyx_t_15);
  __Pyx_XDECREF(__pyx_t_16);
  __Pyx_AddTraceback("cupyx.cusolver.csrlsvqr", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_dtype);
  __Pyx_XDECREF(__pyx_v_t);
  __Pyx_XDECREF(__pyx_v_solve);
  __Pyx_XDECREF(__pyx_v_m);
  __Pyx_XDECREF(__pyx_v_x);
  __Pyx_XDECREF(__pyx_v_singularity);
  __Pyx_XDECREF(__pyx_v_handle);
  __Pyx_XDECREF(__pyx_v_b);
  __Pyx_XDECREF(__pyx_v_tol);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "cupyx/cusolver.pyx":873
 * 
 * 
 * cpdef _geqrf_orgqr_batched(a, mode):             # <<<<<<<<<<<<<<
 *     '''Internal helper for batched QR solver. The input array ``a''
 *     is of shape (batch_size, m, n)
*/

static PyObject *__pyx_pw_5cupyx_8cusolver_21_geqrf_orgqr_batched(PyObject *__pyx_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
static PyObject *__pyx_f_5cupyx_8cusolver__geqrf_orgqr_batched(PyObject *__pyx_v_a, PyObject *__pyx_v_mode, CYTHON_UNUSED int __pyx_skip_dispatch) {
  intptr_t __pyx_v_x_ptr;
  intptr_t __pyx_v_tau_ptr;
  intptr_t __pyx_v_w_ptr;
  intptr_t __pyx_v_info_ptr;
  int __pyx_v_m;
  int __pyx_v_n;
  int __pyx_v_mn;
  int __pyx_v_mc;
  int __pyx_v_batch_size;
  int __pyx_v_buffersize;
  int __pyx_v_orig_n;
  PyObject *__pyx_v_dtype = NULL;
  PyObject *__pyx_v_out_dtype = NULL;
  PyObject *__pyx_v_x = NULL;
  PyObject *__pyx_v_ap = NULL;
  intptr_t __pyx_v_handle;
  struct __pyx_obj_4cupy_5_core_4core__ndarray_base *__pyx_v_dev_info = NULL;
  __pyx_t_5cupyx_8cusolver_geqrf_ptr __pyx_v_geqrf;
  int (*__pyx_v_geqrf_bufferSize)(intptr_t, int, int, size_t, int, int __pyx_skip_dispatch);
  PyObject *__pyx_v_msg = NULL;
  struct __pyx_obj_4cupy_4cuda_6memory_MemoryPointer *__pyx_v_workspace = NULL;
  PyObject *__pyx_v_tau = NULL;
  int __pyx_v_status;
  PyObject *__pyx_v_r = NULL;
  PyObject *__pyx_v_q = NULL;
  __pyx_t_5cupyx_8cusolver_orgqr_ptr __pyx_v_orgqr;
  int (*__pyx_v_orgqr_bufferSize)(intptr_t, int, int, int, size_t, int, size_t, int __pyx_skip_dispatch);
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  size_t __pyx_t_5;
  PyObject *(*__pyx_t_6)(PyObject *);
  PyObject *__pyx_t_7 = NULL;
  int __pyx_t_8;
  int __pyx_t_9;
  int __pyx_t_10;
  int __pyx_t_11;
  intptr_t __pyx_t_12;
  __pyx_t_4cupy_5_core_7_carray_shape_t __pyx_t_13;
  PyObject *__pyx_t_14 = NULL;
  int __pyx_t_15;
  PyObject *__pyx_t_16 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("_geqrf_orgqr_batched", 0);

  /* "cupyx/cusolver.pyx":881
 * 
 *     # support float32, float64, complex64, and complex128
 *     dtype, out_dtype = _cupy.linalg._util.linalg_common_type(a)             # <<<<<<<<<<<<<<
 * 
 *     batch_size, m, n = a.shape
*/
  __Pyx_GetModuleGlobalName(__pyx_t_3, __pyx_mstate_global->__pyx_n_u_cupy); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 881, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_4 = __Pyx_PyObject_GetAttrStr(__pyx_t_3, __pyx_mstate_global->__pyx_n_u_linalg); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 881, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_t_4, __pyx_mstate_global->__pyx_n_u_util); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 881, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __pyx_t_2 = __pyx_t_3;
  __Pyx_INCREF(__pyx_t_2);
  __pyx_t_5 = 0;
  {
    PyObject *__pyx_callargs[2] = {__pyx_t_2, __pyx_v_a};
    __pyx_t_1 = __Pyx_PyObject_FastCallMethod((PyObject*)__pyx_mstate_global->__pyx_n_u_linalg_common_type, __pyx_callargs+__pyx_t_5, (2-__pyx_t_5) | (1*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
    __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 881, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
  }
  if ((likely(PyTuple_CheckExact(__pyx_t_1))) || (PyList_CheckExact(__pyx_t_1))) {
    PyObject* sequence = __pyx_t_1;
    Py_ssize_t size = __Pyx_PySequence_SIZE(sequence);
    if (unlikely(size != 2)) {
      if (size > 2) __Pyx_RaiseTooManyValuesError(2);
      else if (size >= 0) __Pyx_RaiseNeedMoreValuesError(size);
      __PYX_ERR(0, 881, __pyx_L1_error)
    }
    #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
    if (likely(PyTuple_CheckExact(sequence))) {
      __pyx_t_3 = PyTuple_GET_ITEM(sequence, 0);
      __Pyx_INCREF(__pyx_t_3);
      __pyx_t_2 = PyTuple_GET_ITEM(sequence, 1);
      __Pyx_INCREF(__pyx_t_2);
    } else {
      __pyx_t_3 = __Pyx_PyList_GetItemRefFast(sequence, 0, __Pyx_ReferenceSharing_SharedReference);
      if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 881, __pyx_L1_error)
      __Pyx_XGOTREF(__pyx_t_3);
      __pyx_t_2 = __Pyx_PyList_GetItemRefFast(sequence, 1, __Pyx_ReferenceSharing_SharedReference);
      if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 881, __pyx_L1_error)
      __Pyx_XGOTREF(__pyx_t_2);
    }
    #else
    __pyx_t_3 = __Pyx_PySequence_ITEM(sequence, 0); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 881, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_t_2 = __Pyx_PySequence_ITEM(sequence, 1); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 881, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    #endif
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  } else {
    Py_ssize_t index = -1;
    __pyx_t_4 = PyObject_GetIter(__pyx_t_1); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 881, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __pyx_t_6 = (CYTHON_COMPILING_IN_LIMITED_API) ? PyIter_Next : __Pyx_PyObject_GetIterNextFunc(__pyx_t_4);
    index = 0; __pyx_t_3 = __pyx_t_6(__pyx_t_4); if (unlikely(!__pyx_t_3)) goto __pyx_L3_unpacking_failed;
    __Pyx_GOTREF(__pyx_t_3);
    index = 1; __pyx_t_2 = __pyx_t_6(__pyx_t_4); if (unlikely(!__pyx_t_2)) goto __pyx_L3_unpacking_failed;
    __Pyx_GOTREF(__pyx_t_2);
    if (__Pyx_IternextUnpackEndCheck(__pyx_t_6(__pyx_t_4), 2) < (0)) __PYX_ERR(0, 881, __pyx_L1_error)
    __pyx_t_6 = NULL;
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    goto __pyx_L4_unpacking_done;
    __pyx_L3_unpacking_failed:;
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __pyx_t_6 = NULL;
    if (__Pyx_IterFinish() == 0) __Pyx_RaiseNeedMoreValuesError(index);
    __PYX_ERR(0, 881, __pyx_L1_error)
    __pyx_L4_unpacking_done:;
  }
  __pyx_v_dtype = __pyx_t_3;
  __pyx_t_3 = 0;
  __pyx_v_out_dtype = __pyx_t_2;
  __pyx_t_2 = 0;

  /* "cupyx/cusolver.pyx":883
 *     dtype, out_dtype = _cupy.linalg._util.linalg_common_type(a)
 * 
 *     batch_size, m, n = a.shape             # <<<<<<<<<<<<<<
 *     mn = min(m, n)
 * 
*/
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_v_a, __pyx_mstate_global->__pyx_n_u_shape); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 883, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if ((likely(PyTuple_CheckExact(__pyx_t_1))) || (PyList_CheckExact(__pyx_t_1))) {
    PyObject* sequence = __pyx_t_1;
    Py_ssize_t size = __Pyx_PySequence_SIZE(sequence);
    if (unlikely(size != 3)) {
      if (size > 3) __Pyx_RaiseTooManyValuesError(3);
      else if (size >= 0) __Pyx_RaiseNeedMoreValuesError(size);
      __PYX_ERR(0, 883, __pyx_L1_error)
    }
    #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
    if (likely(PyTuple_CheckExact(sequence))) {
      __pyx_t_2 = PyTuple_GET_ITEM(sequence, 0);
      __Pyx_INCREF(__pyx_t_2);
      __pyx_t_3 = PyTuple_GET_ITEM(sequence, 1);
      __Pyx_INCREF(__pyx_t_3);
      __pyx_t_4 = PyTuple_GET_ITEM(sequence, 2);
      __Pyx_INCREF(__pyx_t_4);
    } else {
      __pyx_t_2 = __Pyx_PyList_GetItemRefFast(sequence, 0, __Pyx_ReferenceSharing_SharedReference);
      if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 883, __pyx_L1_error)
      __Pyx_XGOTREF(__pyx_t_2);
      __pyx_t_3 = __Pyx_PyList_GetItemRefFast(sequence, 1, __Pyx_ReferenceSharing_SharedReference);
      if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 883, __pyx_L1_error)
      __Pyx_XGOTREF(__pyx_t_3);
      __pyx_t_4 = __Pyx_PyList_GetItemRefFast(sequence, 2, __Pyx_ReferenceSharing_SharedReference);
      if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 883, __pyx_L1_error)
      __Pyx_XGOTREF(__pyx_t_4);
    }
    #else
    __pyx_t_2 = __Pyx_PySequence_ITEM(sequence, 0); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 883, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_3 = __Pyx_PySequence_ITEM(sequence, 1); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 883, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_t_4 = __Pyx_PySequence_ITEM(sequence, 2); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 883, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    #endif
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  } else {
    Py_ssize_t index = -1;
    __pyx_t_7 = PyObject_GetIter(__pyx_t_1); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 883, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_7);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __pyx_t_6 = (CYTHON_COMPILING_IN_LIMITED_API) ? PyIter_Next : __Pyx_PyObject_GetIterNextFunc(__pyx_t_7);
    index = 0; __pyx_t_2 = __pyx_t_6(__pyx_t_7); if (unlikely(!__pyx_t_2)) goto __pyx_L5_unpacking_failed;
    __Pyx_GOTREF(__pyx_t_2);
    index = 1; __pyx_t_3 = __pyx_t_6(__pyx_t_7); if (unlikely(!__pyx_t_3)) goto __pyx_L5_unpacking_failed;
    __Pyx_GOTREF(__pyx_t_3);
    index = 2; __pyx_t_4 = __pyx_t_6(__pyx_t_7); if (unlikely(!__pyx_t_4)) goto __pyx_L5_unpacking_failed;
    __Pyx_GOTREF(__pyx_t_4);
    if (__Pyx_IternextUnpackEndCheck(__pyx_t_6(__pyx_t_7), 3) < (0)) __PYX_ERR(0, 883, __pyx_L1_error)
    __pyx_t_6 = NULL;
    __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
    goto __pyx_L6_unpacking_done;
    __pyx_L5_unpacking_failed:;
    __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
    __pyx_t_6 = NULL;
    if (__Pyx_IterFinish() == 0) __Pyx_RaiseNeedMoreValuesError(index);
    __PYX_ERR(0, 883, __pyx_L1_error)
    __pyx_L6_unpacking_done:;
  }
  __pyx_t_8 = __Pyx_PyLong_As_int(__pyx_t_2); if (unlikely((__pyx_t_8 == (int)-1) && PyErr_Occurred())) __PYX_ERR(0, 883, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_9 = __Pyx_PyLong_As_int(__pyx_t_3); if (unlikely((__pyx_t_9 == (int)-1) && PyErr_Occurred())) __PYX_ERR(0, 883, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_t_10 = __Pyx_PyLong_As_int(__pyx_t_4); if (unlikely((__pyx_t_10 == (int)-1) && PyErr_Occurred())) __PYX_ERR(0, 883, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __pyx_v_batch_size = __pyx_t_8;
  __pyx_v_m = __pyx_t_9;
  __pyx_v_n = __pyx_t_10;

  /* "cupyx/cusolver.pyx":884
 * 
 *     batch_size, m, n = a.shape
 *     mn = min(m, n)             # <<<<<<<<<<<<<<
 * 
 *     x = a.swapaxes(-2, -1).astype(dtype, order='C', copy=True)
*/
  __pyx_t_10 = __pyx_v_n;
  __pyx_t_9 = __pyx_v_m;
  __pyx_t_11 = (__pyx_t_10 < __pyx_t_9);
  if (__pyx_t_11) {
    __pyx_t_8 = __pyx_t_10;
  } else {
    __pyx_t_8 = __pyx_t_9;
  }
  __pyx_v_mn = __pyx_t_8;

  /* "cupyx/cusolver.pyx":886
 *     mn = min(m, n)
 * 
 *     x = a.swapaxes(-2, -1).astype(dtype, order='C', copy=True)             # <<<<<<<<<<<<<<
 *     if runtime._is_hip_environment:
 *         # rocsolver_<t>geqrf_batched has a different signature...
*/
  __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_v_a, __pyx_mstate_global->__pyx_n_u_swapaxes); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 886, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_2 = __Pyx_PyObject_Call(__pyx_t_3, __pyx_mstate_global->__pyx_tuple[0], NULL); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 886, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_t_4 = __pyx_t_2;
  __Pyx_INCREF(__pyx_t_4);
  __pyx_t_5 = 0;
  {
    PyObject *__pyx_callargs[2 + ((CYTHON_VECTORCALL) ? 2 : 0)] = {__pyx_t_4, __pyx_v_dtype};
    __pyx_t_3 = __Pyx_MakeVectorcallBuilderKwds(2); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 886, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    if (__Pyx_VectorcallBuilder_AddArg(__pyx_mstate_global->__pyx_n_u_order, __pyx_mstate_global->__pyx_n_u_C, __pyx_t_3, __pyx_callargs+2, 0) < (0)) __PYX_ERR(0, 886, __pyx_L1_error)
    if (__Pyx_VectorcallBuilder_AddArg(__pyx_mstate_global->__pyx_n_u_copy, Py_True, __pyx_t_3, __pyx_callargs+2, 1) < (0)) __PYX_ERR(0, 886, __pyx_L1_error)
    __pyx_t_1 = __Pyx_Object_VectorcallMethod_CallFromBuilder((PyObject*)__pyx_mstate_global->__pyx_n_u_astype, __pyx_callargs+__pyx_t_5, (2-__pyx_t_5) | (1*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET), __pyx_t_3);
    __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 886, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
  }
  __pyx_v_x = __pyx_t_1;
  __pyx_t_1 = 0;

  /* "cupyx/cusolver.pyx":887
 * 
 *     x = a.swapaxes(-2, -1).astype(dtype, order='C', copy=True)
 *     if runtime._is_hip_environment:             # <<<<<<<<<<<<<<
 *         # rocsolver_<t>geqrf_batched has a different signature...
 *         ap = _linalg._mat_ptrs(x)
*/
  if (__pyx_v_13cupy_backends_4cuda_3api_7runtime__is_hip_environment) {

    /* "cupyx/cusolver.pyx":889
 *     if runtime._is_hip_environment:
 *         # rocsolver_<t>geqrf_batched has a different signature...
 *         ap = _linalg._mat_ptrs(x)             # <<<<<<<<<<<<<<
 *     else:
 *         ap = x
*/
    __pyx_t_2 = NULL;
    __Pyx_GetModuleGlobalName(__pyx_t_3, __pyx_mstate_global->__pyx_n_u_linalg_2); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 889, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_t_4 = __Pyx_PyObject_GetAttrStr(__pyx_t_3, __pyx_mstate_global->__pyx_n_u_mat_ptrs); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 889, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __pyx_t_5 = 1;
    #if CYTHON_UNPACK_METHODS
    if (unlikely(PyMethod_Check(__pyx_t_4))) {
      __pyx_t_2 = PyMethod_GET_SELF(__pyx_t_4);
      assert(__pyx_t_2);
      PyObject* __pyx__function = PyMethod_GET_FUNCTION(__pyx_t_4);
      __Pyx_INCREF(__pyx_t_2);
      __Pyx_INCREF(__pyx__function);
      __Pyx_DECREF_SET(__pyx_t_4, __pyx__function);
      __pyx_t_5 = 0;
    }
    #endif
    {
      PyObject *__pyx_callargs[2] = {__pyx_t_2, __pyx_v_x};
      __pyx_t_1 = __Pyx_PyObject_FastCall((PyObject*)__pyx_t_4, __pyx_callargs+__pyx_t_5, (2-__pyx_t_5) | (__pyx_t_5*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
      __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 889, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
    }
    __pyx_v_ap = __pyx_t_1;
    __pyx_t_1 = 0;

    /* "cupyx/cusolver.pyx":887
 * 
 *     x = a.swapaxes(-2, -1).astype(dtype, order='C', copy=True)
 *     if runtime._is_hip_environment:             # <<<<<<<<<<<<<<
 *         # rocsolver_<t>geqrf_batched has a different signature...
 *         ap = _linalg._mat_ptrs(x)
*/
    goto __pyx_L7;
  }

  /* "cupyx/cusolver.pyx":891
 *         ap = _linalg._mat_ptrs(x)
 *     else:
 *         ap = x             # <<<<<<<<<<<<<<
 *     x_ptr = ap.data.ptr
 * 
*/
  /*else*/ {
    __Pyx_INCREF(__pyx_v_x);
    __pyx_v_ap = __pyx_v_x;
  }
  __pyx_L7:;

  /* "cupyx/cusolver.pyx":892
 *     else:
 *         ap = x
 *     x_ptr = ap.data.ptr             # <<<<<<<<<<<<<<
 * 
 *     cdef intptr_t handle = _device.get_cusolver_handle()
*/
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_v_ap, __pyx_mstate_global->__pyx_n_u_data); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 892, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_4 = __Pyx_PyObject_GetAttrStr(__pyx_t_1, __pyx_mstate_global->__pyx_n_u_ptr); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 892, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_12 = PyLong_AsSsize_t(__pyx_t_4); if (unlikely((__pyx_t_12 == ((intptr_t)-1)) && PyErr_Occurred())) __PYX_ERR(0, 892, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __pyx_v_x_ptr = __pyx_t_12;

  /* "cupyx/cusolver.pyx":894
 *     x_ptr = ap.data.ptr
 * 
 *     cdef intptr_t handle = _device.get_cusolver_handle()             # <<<<<<<<<<<<<<
 *     dev_info = _ndarray_init(_cupy.ndarray, (batch_size,), _numpy.int32, None)
 *     info_ptr = dev_info.data.ptr
*/
  __pyx_t_1 = NULL;
  __Pyx_GetModuleGlobalName(__pyx_t_2, __pyx_mstate_global->__pyx_n_u_device); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 894, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_t_2, __pyx_mstate_global->__pyx_n_u_get_cusolver_handle); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 894, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_5 = 1;
  #if CYTHON_UNPACK_METHODS
  if (unlikely(PyMethod_Check(__pyx_t_3))) {
    __pyx_t_1 = PyMethod_GET_SELF(__pyx_t_3);
    assert(__pyx_t_1);
    PyObject* __pyx__function = PyMethod_GET_FUNCTION(__pyx_t_3);
    __Pyx_INCREF(__pyx_t_1);
    __Pyx_INCREF(__pyx__function);
    __Pyx_DECREF_SET(__pyx_t_3, __pyx__function);
    __pyx_t_5 = 0;
  }
  #endif
  {
    PyObject *__pyx_callargs[2] = {__pyx_t_1, NULL};
    __pyx_t_4 = __Pyx_PyObject_FastCall((PyObject*)__pyx_t_3, __pyx_callargs+__pyx_t_5, (1-__pyx_t_5) | (__pyx_t_5*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
    __Pyx_XDECREF(__pyx_t_1); __pyx_t_1 = 0;
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 894, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
  }
  __pyx_t_12 = PyLong_AsSsize_t(__pyx_t_4); if (unlikely((__pyx_t_12 == ((intptr_t)-1)) && PyErr_Occurred())) __PYX_ERR(0, 894, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __pyx_v_handle = __pyx_t_12;

  /* "cupyx/cusolver.pyx":895
 * 
 *     cdef intptr_t handle = _device.get_cusolver_handle()
 *     dev_info = _ndarray_init(_cupy.ndarray, (batch_size,), _numpy.int32, None)             # <<<<<<<<<<<<<<
 *     info_ptr = dev_info.data.ptr
 * 
*/
  __Pyx_GetModuleGlobalName(__pyx_t_4, __pyx_mstate_global->__pyx_n_u_cupy); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 895, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_t_4, __pyx_mstate_global->__pyx_n_u_ndarray); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 895, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __pyx_t_4 = __Pyx_PyLong_From_int(__pyx_v_batch_size); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 895, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_t_1 = PyTuple_New(1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 895, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_4);
  if (__Pyx_PyTuple_SET_ITEM(__pyx_t_1, 0, __pyx_t_4) != (0)) __PYX_ERR(0, 895, __pyx_L1_error);
  __pyx_t_4 = 0;
  __pyx_t_13 = __pyx_convert_vector_from_py_Py_ssize_t(__pyx_t_1); if (unlikely(PyErr_Occurred())) __PYX_ERR(0, 895, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_GetModuleGlobalName(__pyx_t_1, __pyx_mstate_global->__pyx_n_u_numpy); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 895, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_4 = __Pyx_PyObject_GetAttrStr(__pyx_t_1, __pyx_mstate_global->__pyx_n_u_int32); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 895, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_1 = ((PyObject *)__pyx_f_4cupy_5_core_4core__ndarray_init(__pyx_t_3, __pyx_t_13, __pyx_t_4, Py_None)); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 895, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __pyx_v_dev_info = ((struct __pyx_obj_4cupy_5_core_4core__ndarray_base *)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "cupyx/cusolver.pyx":896
 *     cdef intptr_t handle = _device.get_cusolver_handle()
 *     dev_info = _ndarray_init(_cupy.ndarray, (batch_size,), _numpy.int32, None)
 *     info_ptr = dev_info.data.ptr             # <<<<<<<<<<<<<<
 * 
 *     cdef geqrf_ptr geqrf
*/
  __pyx_t_12 = __pyx_v_dev_info->data->ptr;
  __pyx_v_info_ptr = __pyx_t_12;

  /* "cupyx/cusolver.pyx":899
 * 
 *     cdef geqrf_ptr geqrf
 *     if dtype == 'f':             # <<<<<<<<<<<<<<
 *         geqrf_bufferSize = sgeqrf_bufferSize
 *         geqrf = geqrf_loop[float]
*/
  __pyx_t_11 = (__Pyx_PyUnicode_Equals(__pyx_v_dtype, __pyx_mstate_global->__pyx_n_u_f, Py_EQ)); if (unlikely((__pyx_t_11 < 0))) __PYX_ERR(0, 899, __pyx_L1_error)
  if (__pyx_t_11) {

    /* "cupyx/cusolver.pyx":900
 *     cdef geqrf_ptr geqrf
 *     if dtype == 'f':
 *         geqrf_bufferSize = sgeqrf_bufferSize             # <<<<<<<<<<<<<<
 *         geqrf = geqrf_loop[float]
 *     elif dtype == 'd':
*/
    __pyx_v_geqrf_bufferSize = __pyx_f_13cupy_backends_4cuda_4libs_8cusolver_sgeqrf_bufferSize;

    /* "cupyx/cusolver.pyx":901
 *     if dtype == 'f':
 *         geqrf_bufferSize = sgeqrf_bufferSize
 *         geqrf = geqrf_loop[float]             # <<<<<<<<<<<<<<
 *     elif dtype == 'd':
 *         geqrf_bufferSize = dgeqrf_bufferSize
*/
    __pyx_v_geqrf = geqrf_loop<float>;

    /* "cupyx/cusolver.pyx":899
 * 
 *     cdef geqrf_ptr geqrf
 *     if dtype == 'f':             # <<<<<<<<<<<<<<
 *         geqrf_bufferSize = sgeqrf_bufferSize
 *         geqrf = geqrf_loop[float]
*/
    goto __pyx_L8;
  }

  /* "cupyx/cusolver.pyx":902
 *         geqrf_bufferSize = sgeqrf_bufferSize
 *         geqrf = geqrf_loop[float]
 *     elif dtype == 'd':             # <<<<<<<<<<<<<<
 *         geqrf_bufferSize = dgeqrf_bufferSize
 *         geqrf = geqrf_loop[double]
*/
  __pyx_t_11 = (__Pyx_PyUnicode_Equals(__pyx_v_dtype, __pyx_mstate_global->__pyx_n_u_d, Py_EQ)); if (unlikely((__pyx_t_11 < 0))) __PYX_ERR(0, 902, __pyx_L1_error)
  if (__pyx_t_11) {

    /* "cupyx/cusolver.pyx":903
 *         geqrf = geqrf_loop[float]
 *     elif dtype == 'd':
 *         geqrf_bufferSize = dgeqrf_bufferSize             # <<<<<<<<<<<<<<
 *         geqrf = geqrf_loop[double]
 *     elif dtype == 'F':
*/
    __pyx_v_geqrf_bufferSize = __pyx_f_13cupy_backends_4cuda_4libs_8cusolver_dgeqrf_bufferSize;

    /* "cupyx/cusolver.pyx":904
 *     elif dtype == 'd':
 *         geqrf_bufferSize = dgeqrf_bufferSize
 *         geqrf = geqrf_loop[double]             # <<<<<<<<<<<<<<
 *     elif dtype == 'F':
 *         geqrf_bufferSize = cgeqrf_bufferSize
*/
    __pyx_v_geqrf = geqrf_loop<double>;

    /* "cupyx/cusolver.pyx":902
 *         geqrf_bufferSize = sgeqrf_bufferSize
 *         geqrf = geqrf_loop[float]
 *     elif dtype == 'd':             # <<<<<<<<<<<<<<
 *         geqrf_bufferSize = dgeqrf_bufferSize
 *         geqrf = geqrf_loop[double]
*/
    goto __pyx_L8;
  }

  /* "cupyx/cusolver.pyx":905
 *         geqrf_bufferSize = dgeqrf_bufferSize
 *         geqrf = geqrf_loop[double]
 *     elif dtype == 'F':             # <<<<<<<<<<<<<<
 *         geqrf_bufferSize = cgeqrf_bufferSize
 *         geqrf = geqrf_loop[cuComplex]
*/
  __pyx_t_11 = (__Pyx_PyUnicode_Equals(__pyx_v_dtype, __pyx_mstate_global->__pyx_n_u_F, Py_EQ)); if (unlikely((__pyx_t_11 < 0))) __PYX_ERR(0, 905, __pyx_L1_error)
  if (__pyx_t_11) {

    /* "cupyx/cusolver.pyx":906
 *         geqrf = geqrf_loop[double]
 *     elif dtype == 'F':
 *         geqrf_bufferSize = cgeqrf_bufferSize             # <<<<<<<<<<<<<<
 *         geqrf = geqrf_loop[cuComplex]
 *     elif dtype == 'D':
*/
    __pyx_v_geqrf_bufferSize = __pyx_f_13cupy_backends_4cuda_4libs_8cusolver_cgeqrf_bufferSize;

    /* "cupyx/cusolver.pyx":907
 *     elif dtype == 'F':
 *         geqrf_bufferSize = cgeqrf_bufferSize
 *         geqrf = geqrf_loop[cuComplex]             # <<<<<<<<<<<<<<
 *     elif dtype == 'D':
 *         geqrf_bufferSize = zgeqrf_bufferSize
*/
    __pyx_v_geqrf = geqrf_loop<cuComplex>;

    /* "cupyx/cusolver.pyx":905
 *         geqrf_bufferSize = dgeqrf_bufferSize
 *         geqrf = geqrf_loop[double]
 *     elif dtype == 'F':             # <<<<<<<<<<<<<<
 *         geqrf_bufferSize = cgeqrf_bufferSize
 *         geqrf = geqrf_loop[cuComplex]
*/
    goto __pyx_L8;
  }

  /* "cupyx/cusolver.pyx":908
 *         geqrf_bufferSize = cgeqrf_bufferSize
 *         geqrf = geqrf_loop[cuComplex]
 *     elif dtype == 'D':             # <<<<<<<<<<<<<<
 *         geqrf_bufferSize = zgeqrf_bufferSize
 *         geqrf = geqrf_loop[cuDoubleComplex]
*/
  __pyx_t_11 = (__Pyx_PyUnicode_Equals(__pyx_v_dtype, __pyx_mstate_global->__pyx_n_u_D, Py_EQ)); if (unlikely((__pyx_t_11 < 0))) __PYX_ERR(0, 908, __pyx_L1_error)
  if (likely(__pyx_t_11)) {

    /* "cupyx/cusolver.pyx":909
 *         geqrf = geqrf_loop[cuComplex]
 *     elif dtype == 'D':
 *         geqrf_bufferSize = zgeqrf_bufferSize             # <<<<<<<<<<<<<<
 *         geqrf = geqrf_loop[cuDoubleComplex]
 *     else:
*/
    __pyx_v_geqrf_bufferSize = __pyx_f_13cupy_backends_4cuda_4libs_8cusolver_zgeqrf_bufferSize;

    /* "cupyx/cusolver.pyx":910
 *     elif dtype == 'D':
 *         geqrf_bufferSize = zgeqrf_bufferSize
 *         geqrf = geqrf_loop[cuDoubleComplex]             # <<<<<<<<<<<<<<
 *     else:
 *         msg = ('dtype must be float32, float64, complex64 or complex128'
*/
    __pyx_v_geqrf = geqrf_loop<cuDoubleComplex>;

    /* "cupyx/cusolver.pyx":908
 *         geqrf_bufferSize = cgeqrf_bufferSize
 *         geqrf = geqrf_loop[cuComplex]
 *     elif dtype == 'D':             # <<<<<<<<<<<<<<
 *         geqrf_bufferSize = zgeqrf_bufferSize
 *         geqrf = geqrf_loop[cuDoubleComplex]
*/
    goto __pyx_L8;
  }

  /* "cupyx/cusolver.pyx":912
 *         geqrf = geqrf_loop[cuDoubleComplex]
 *     else:
 *         msg = ('dtype must be float32, float64, complex64 or complex128'             # <<<<<<<<<<<<<<
 *                ' (actual: {})'.format(a.dtype))
 *         raise ValueError(msg)
*/
  /*else*/ {

    /* "cupyx/cusolver.pyx":913
 *     else:
 *         msg = ('dtype must be float32, float64, complex64 or complex128'
 *                ' (actual: {})'.format(a.dtype))             # <<<<<<<<<<<<<<
 *         raise ValueError(msg)
 * 
*/
    __pyx_t_4 = __pyx_mstate_global->__pyx_kp_u_dtype_must_be_float32_float64_co;
    __Pyx_INCREF(__pyx_t_4);
    __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_v_a, __pyx_mstate_global->__pyx_n_u_dtype); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 913, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_t_5 = 0;
    {
      PyObject *__pyx_callargs[2] = {__pyx_t_4, __pyx_t_3};
      __pyx_t_1 = __Pyx_PyObject_FastCallMethod((PyObject*)__pyx_mstate_global->__pyx_n_u_format, __pyx_callargs+__pyx_t_5, (2-__pyx_t_5) | (1*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
      __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
      if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 913, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
    }
    __pyx_v_msg = ((PyObject*)__pyx_t_1);
    __pyx_t_1 = 0;

    /* "cupyx/cusolver.pyx":914
 *         msg = ('dtype must be float32, float64, complex64 or complex128'
 *                ' (actual: {})'.format(a.dtype))
 *         raise ValueError(msg)             # <<<<<<<<<<<<<<
 * 
 *     # this wrapper also sets the stream for us
*/
    __pyx_t_3 = NULL;
    __pyx_t_5 = 1;
    {
      PyObject *__pyx_callargs[2] = {__pyx_t_3, __pyx_v_msg};
      __pyx_t_1 = __Pyx_PyObject_FastCall((PyObject*)(((PyTypeObject*)PyExc_ValueError)), __pyx_callargs+__pyx_t_5, (2-__pyx_t_5) | (__pyx_t_5*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
      __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
      if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 914, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
    }
    __Pyx_Raise(__pyx_t_1, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __PYX_ERR(0, 914, __pyx_L1_error)
  }
  __pyx_L8:;

  /* "cupyx/cusolver.pyx":917
 * 
 *     # this wrapper also sets the stream for us
 *     buffersize = geqrf_bufferSize(handle, m, n, x.data.ptr, n)             # <<<<<<<<<<<<<<
 *     # we are on the same stream, so the workspace can be reused in the loop
 *     workspace = memory.alloc(buffersize * a.dtype.itemsize)
*/
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_v_x, __pyx_mstate_global->__pyx_n_u_data); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 917, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_t_1, __pyx_mstate_global->__pyx_n_u_ptr); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 917, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_5 = __Pyx_PyLong_As_size_t(__pyx_t_3); if (unlikely((__pyx_t_5 == (size_t)-1) && PyErr_Occurred())) __PYX_ERR(0, 917, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_t_8 = __pyx_v_geqrf_bufferSize(__pyx_v_handle, __pyx_v_m, __pyx_v_n, __pyx_t_5, __pyx_v_n, 0); if (unlikely(__pyx_t_8 == ((int)-1) && PyErr_Occurred())) __PYX_ERR(0, 917, __pyx_L1_error)
  __pyx_v_buffersize = __pyx_t_8;

  /* "cupyx/cusolver.pyx":919
 *     buffersize = geqrf_bufferSize(handle, m, n, x.data.ptr, n)
 *     # we are on the same stream, so the workspace can be reused in the loop
 *     workspace = memory.alloc(buffersize * a.dtype.itemsize)             # <<<<<<<<<<<<<<
 *     w_ptr = workspace.ptr
 *     tau = _cupy.empty((batch_size, mn), dtype=dtype)
*/
  __pyx_t_3 = __Pyx_PyLong_From_int(__pyx_v_buffersize); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 919, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_v_a, __pyx_mstate_global->__pyx_n_u_dtype); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 919, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_4 = __Pyx_PyObject_GetAttrStr(__pyx_t_1, __pyx_mstate_global->__pyx_n_u_itemsize); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 919, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_1 = PyNumber_Multiply(__pyx_t_3, __pyx_t_4); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 919, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __pyx_t_4 = ((PyObject *)__pyx_f_4cupy_4cuda_6memory_alloc(__pyx_t_1, 0)); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 919, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v_workspace = ((struct __pyx_obj_4cupy_4cuda_6memory_MemoryPointer *)__pyx_t_4);
  __pyx_t_4 = 0;

  /* "cupyx/cusolver.pyx":920
 *     # we are on the same stream, so the workspace can be reused in the loop
 *     workspace = memory.alloc(buffersize * a.dtype.itemsize)
 *     w_ptr = workspace.ptr             # <<<<<<<<<<<<<<
 *     tau = _cupy.empty((batch_size, mn), dtype=dtype)
 *     tau_ptr = tau.data.ptr
*/
  __pyx_t_12 = __pyx_v_workspace->ptr;
  __pyx_v_w_ptr = __pyx_t_12;

  /* "cupyx/cusolver.pyx":921
 *     workspace = memory.alloc(buffersize * a.dtype.itemsize)
 *     w_ptr = workspace.ptr
 *     tau = _cupy.empty((batch_size, mn), dtype=dtype)             # <<<<<<<<<<<<<<
 *     tau_ptr = tau.data.ptr
 * 
*/
  __pyx_t_1 = NULL;
  __Pyx_GetModuleGlobalName(__pyx_t_3, __pyx_mstate_global->__pyx_n_u_cupy); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 921, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_t_3, __pyx_mstate_global->__pyx_n_u_empty); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 921, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_t_3 = __Pyx_PyLong_From_int(__pyx_v_batch_size); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 921, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_7 = __Pyx_PyLong_From_int(__pyx_v_mn); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 921, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_7);
  __pyx_t_14 = PyTuple_New(2); if (unlikely(!__pyx_t_14)) __PYX_ERR(0, 921, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_14);
  __Pyx_GIVEREF(__pyx_t_3);
  if (__Pyx_PyTuple_SET_ITEM(__pyx_t_14, 0, __pyx_t_3) != (0)) __PYX_ERR(0, 921, __pyx_L1_error);
  __Pyx_GIVEREF(__pyx_t_7);
  if (__Pyx_PyTuple_SET_ITEM(__pyx_t_14, 1, __pyx_t_7) != (0)) __PYX_ERR(0, 921, __pyx_L1_error);
  __pyx_t_3 = 0;
  __pyx_t_7 = 0;
  __pyx_t_5 = 1;
  #if CYTHON_UNPACK_METHODS
  if (unlikely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_1 = PyMethod_GET_SELF(__pyx_t_2);
    assert(__pyx_t_1);
    PyObject* __pyx__function = PyMethod_GET_FUNCTION(__pyx_t_2);
    __Pyx_INCREF(__pyx_t_1);
    __Pyx_INCREF(__pyx__function);
    __Pyx_DECREF_SET(__pyx_t_2, __pyx__function);
    __pyx_t_5 = 0;
  }
  #endif
  {
    PyObject *__pyx_callargs[2 + ((CYTHON_VECTORCALL) ? 1 : 0)] = {__pyx_t_1, __pyx_t_14};
    __pyx_t_7 = __Pyx_MakeVectorcallBuilderKwds(1); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 921, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_7);
    if (__Pyx_VectorcallBuilder_AddArg(__pyx_mstate_global->__pyx_n_u_dtype, __pyx_v_dtype, __pyx_t_7, __pyx_callargs+2, 0) < (0)) __PYX_ERR(0, 921, __pyx_L1_error)
    __pyx_t_4 = __Pyx_Object_Vectorcall_CallFromBuilder((PyObject*)__pyx_t_2, __pyx_callargs+__pyx_t_5, (2-__pyx_t_5) | (__pyx_t_5*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET), __pyx_t_7);
    __Pyx_XDECREF(__pyx_t_1); __pyx_t_1 = 0;
    __Pyx_DECREF(__pyx_t_14); __pyx_t_14 = 0;
    __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 921, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
  }
  __pyx_v_tau = __pyx_t_4;
  __pyx_t_4 = 0;

  /* "cupyx/cusolver.pyx":922
 *     w_ptr = workspace.ptr
 *     tau = _cupy.empty((batch_size, mn), dtype=dtype)
 *     tau_ptr = tau.data.ptr             # <<<<<<<<<<<<<<
 * 
 *     # compute working space of geqrf and solve R
*/
  __pyx_t_4 = __Pyx_PyObject_GetAttrStr(__pyx_v_tau, __pyx_mstate_global->__pyx_n_u_data); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 922, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_t_4, __pyx_mstate_global->__pyx_n_u_ptr); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 922, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __pyx_t_12 = PyLong_AsSsize_t(__pyx_t_2); if (unlikely((__pyx_t_12 == ((intptr_t)-1)) && PyErr_Occurred())) __PYX_ERR(0, 922, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_v_tau_ptr = __pyx_t_12;

  /* "cupyx/cusolver.pyx":926
 *     # compute working space of geqrf and solve R
 *     # the loop starts here, with gil released to reduce overhead
 *     with nogil:             # <<<<<<<<<<<<<<
 *         status = geqrf(handle, m, n, x_ptr, m, tau_ptr,
 *                        w_ptr, buffersize, info_ptr, batch_size)
*/
  {
      PyThreadState * _save;
      _save = PyEval_SaveThread();
      __Pyx_FastGIL_Remember();
      /*try:*/ {

        /* "cupyx/cusolver.pyx":927
 *     # the loop starts here, with gil released to reduce overhead
 *     with nogil:
 *         status = geqrf(handle, m, n, x_ptr, m, tau_ptr,             # <<<<<<<<<<<<<<
 *                        w_ptr, buffersize, info_ptr, batch_size)
 *     if status != 0:
*/
        __pyx_v_status = __pyx_v_geqrf(__pyx_v_handle, __pyx_v_m, __pyx_v_n, __pyx_v_x_ptr, __pyx_v_m, __pyx_v_tau_ptr, __pyx_v_w_ptr, __pyx_v_buffersize, __pyx_v_info_ptr, __pyx_v_batch_size);
      }

      /* "cupyx/cusolver.pyx":926
 *     # compute working space of geqrf and solve R
 *     # the loop starts here, with gil released to reduce overhead
 *     with nogil:             # <<<<<<<<<<<<<<
 *         status = geqrf(handle, m, n, x_ptr, m, tau_ptr,
 *                        w_ptr, buffersize, info_ptr, batch_size)
*/
      /*finally:*/ {
        /*normal exit:*/{
          __Pyx_FastGIL_Forget();
          PyEval_RestoreThread(_save);
          goto __pyx_L11;
        }
        __pyx_L11:;
      }
  }

  /* "cupyx/cusolver.pyx":929
 *         status = geqrf(handle, m, n, x_ptr, m, tau_ptr,
 *                        w_ptr, buffersize, info_ptr, batch_size)
 *     if status != 0:             # <<<<<<<<<<<<<<
 *         raise _cusolver.CUSOLVERError(status)
 *     _cupy.linalg._util._check_cusolver_dev_info_if_synchronization_allowed(
*/
  __pyx_t_11 = (__pyx_v_status != 0);
  if (unlikely(__pyx_t_11)) {

    /* "cupyx/cusolver.pyx":930
 *                        w_ptr, buffersize, info_ptr, batch_size)
 *     if status != 0:
 *         raise _cusolver.CUSOLVERError(status)             # <<<<<<<<<<<<<<
 *     _cupy.linalg._util._check_cusolver_dev_info_if_synchronization_allowed(
 *         'geqrf', dev_info)
*/
    __pyx_t_4 = NULL;
    __Pyx_GetModuleGlobalName(__pyx_t_7, __pyx_mstate_global->__pyx_n_u_cusolver); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 930, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_7);
    __pyx_t_14 = __Pyx_PyObject_GetAttrStr(__pyx_t_7, __pyx_mstate_global->__pyx_n_u_CUSOLVERError); if (unlikely(!__pyx_t_14)) __PYX_ERR(0, 930, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_14);
    __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
    __pyx_t_7 = __Pyx_PyLong_From_int(__pyx_v_status); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 930, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_7);
    __pyx_t_5 = 1;
    #if CYTHON_UNPACK_METHODS
    if (unlikely(PyMethod_Check(__pyx_t_14))) {
      __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_14);
      assert(__pyx_t_4);
      PyObject* __pyx__function = PyMethod_GET_FUNCTION(__pyx_t_14);
      __Pyx_INCREF(__pyx_t_4);
      __Pyx_INCREF(__pyx__function);
      __Pyx_DECREF_SET(__pyx_t_14, __pyx__function);
      __pyx_t_5 = 0;
    }
    #endif
    {
      PyObject *__pyx_callargs[2] = {__pyx_t_4, __pyx_t_7};
      __pyx_t_2 = __Pyx_PyObject_FastCall((PyObject*)__pyx_t_14, __pyx_callargs+__pyx_t_5, (2-__pyx_t_5) | (__pyx_t_5*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
      __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
      __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
      __Pyx_DECREF(__pyx_t_14); __pyx_t_14 = 0;
      if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 930, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_2);
    }
    __Pyx_Raise(__pyx_t_2, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __PYX_ERR(0, 930, __pyx_L1_error)

    /* "cupyx/cusolver.pyx":929
 *         status = geqrf(handle, m, n, x_ptr, m, tau_ptr,
 *                        w_ptr, buffersize, info_ptr, batch_size)
 *     if status != 0:             # <<<<<<<<<<<<<<
 *         raise _cusolver.CUSOLVERError(status)
 *     _cupy.linalg._util._check_cusolver_dev_info_if_synchronization_allowed(
*/
  }

  /* "cupyx/cusolver.pyx":931
 *     if status != 0:
 *         raise _cusolver.CUSOLVERError(status)
 *     _cupy.linalg._util._check_cusolver_dev_info_if_synchronization_allowed(             # <<<<<<<<<<<<<<
 *         'geqrf', dev_info)
 * 
*/
  __Pyx_GetModuleGlobalName(__pyx_t_7, __pyx_mstate_global->__pyx_n_u_cupy); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 931, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_7);
  __pyx_t_4 = __Pyx_PyObject_GetAttrStr(__pyx_t_7, __pyx_mstate_global->__pyx_n_u_linalg); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 931, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
  __pyx_t_7 = __Pyx_PyObject_GetAttrStr(__pyx_t_4, __pyx_mstate_global->__pyx_n_u_util); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 931, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_7);
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __pyx_t_14 = __pyx_t_7;
  __Pyx_INCREF(__pyx_t_14);

  /* "cupyx/cusolver.pyx":932
 *         raise _cusolver.CUSOLVERError(status)
 *     _cupy.linalg._util._check_cusolver_dev_info_if_synchronization_allowed(
 *         'geqrf', dev_info)             # <<<<<<<<<<<<<<
 * 
 *     if mode == 'r':
*/
  __pyx_t_5 = 0;
  {
    PyObject *__pyx_callargs[3] = {__pyx_t_14, __pyx_mstate_global->__pyx_n_u_geqrf, ((PyObject *)__pyx_v_dev_info)};
    __pyx_t_2 = __Pyx_PyObject_FastCallMethod((PyObject*)__pyx_mstate_global->__pyx_n_u_check_cusolver_dev_info_if_sync, __pyx_callargs+__pyx_t_5, (3-__pyx_t_5) | (1*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
    __Pyx_XDECREF(__pyx_t_14); __pyx_t_14 = 0;
    __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
    if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 931, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
  }
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "cupyx/cusolver.pyx":934
 *         'geqrf', dev_info)
 * 
 *     if mode == 'r':             # <<<<<<<<<<<<<<
 *         r = x[..., :mn].swapaxes(-2, -1)
 *         return _cupy.linalg._util._triu(r).astype(out_dtype, copy=False)
*/
  __pyx_t_11 = (__Pyx_PyUnicode_Equals(__pyx_v_mode, __pyx_mstate_global->__pyx_n_u_r, Py_EQ)); if (unlikely((__pyx_t_11 < 0))) __PYX_ERR(0, 934, __pyx_L1_error)
  if (__pyx_t_11) {

    /* "cupyx/cusolver.pyx":935
 * 
 *     if mode == 'r':
 *         r = x[..., :mn].swapaxes(-2, -1)             # <<<<<<<<<<<<<<
 *         return _cupy.linalg._util._triu(r).astype(out_dtype, copy=False)
 * 
*/
    __pyx_t_2 = __Pyx_PyLong_From_int(__pyx_v_mn); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 935, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_7 = PySlice_New(Py_None, __pyx_t_2, Py_None); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 935, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_7);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __pyx_t_2 = PyTuple_New(2); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 935, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_INCREF(Py_Ellipsis);
    __Pyx_GIVEREF(Py_Ellipsis);
    if (__Pyx_PyTuple_SET_ITEM(__pyx_t_2, 0, Py_Ellipsis) != (0)) __PYX_ERR(0, 935, __pyx_L1_error);
    __Pyx_GIVEREF(__pyx_t_7);
    if (__Pyx_PyTuple_SET_ITEM(__pyx_t_2, 1, __pyx_t_7) != (0)) __PYX_ERR(0, 935, __pyx_L1_error);
    __pyx_t_7 = 0;
    __pyx_t_7 = __Pyx_PyObject_GetItem(__pyx_v_x, __pyx_t_2); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 935, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_7);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_t_7, __pyx_mstate_global->__pyx_n_u_swapaxes); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 935, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
    __pyx_t_7 = __Pyx_PyObject_Call(__pyx_t_2, __pyx_mstate_global->__pyx_tuple[0], NULL); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 935, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_7);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __pyx_v_r = __pyx_t_7;
    __pyx_t_7 = 0;

    /* "cupyx/cusolver.pyx":936
 *     if mode == 'r':
 *         r = x[..., :mn].swapaxes(-2, -1)
 *         return _cupy.linalg._util._triu(r).astype(out_dtype, copy=False)             # <<<<<<<<<<<<<<
 * 
 *     if mode == 'raw':
*/
    __Pyx_XDECREF(__pyx_r);
    __Pyx_GetModuleGlobalName(__pyx_t_1, __pyx_mstate_global->__pyx_n_u_cupy); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 936, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_t_1, __pyx_mstate_global->__pyx_n_u_linalg); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 936, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_t_3, __pyx_mstate_global->__pyx_n_u_util); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 936, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __pyx_t_4 = __pyx_t_1;
    __Pyx_INCREF(__pyx_t_4);
    __pyx_t_5 = 0;
    {
      PyObject *__pyx_callargs[2] = {__pyx_t_4, __pyx_v_r};
      __pyx_t_14 = __Pyx_PyObject_FastCallMethod((PyObject*)__pyx_mstate_global->__pyx_n_u_triu, __pyx_callargs+__pyx_t_5, (2-__pyx_t_5) | (1*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
      __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      if (unlikely(!__pyx_t_14)) __PYX_ERR(0, 936, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_14);
    }
    __pyx_t_2 = __pyx_t_14;
    __Pyx_INCREF(__pyx_t_2);
    __pyx_t_5 = 0;
    {
      PyObject *__pyx_callargs[2 + ((CYTHON_VECTORCALL) ? 1 : 0)] = {__pyx_t_2, __pyx_v_out_dtype};
      __pyx_t_1 = __Pyx_MakeVectorcallBuilderKwds(1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 936, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      if (__Pyx_VectorcallBuilder_AddArg(__pyx_mstate_global->__pyx_n_u_copy, Py_False, __pyx_t_1, __pyx_callargs+2, 0) < (0)) __PYX_ERR(0, 936, __pyx_L1_error)
      __pyx_t_7 = __Pyx_Object_VectorcallMethod_CallFromBuilder((PyObject*)__pyx_mstate_global->__pyx_n_u_astype, __pyx_callargs+__pyx_t_5, (2-__pyx_t_5) | (1*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET), __pyx_t_1);
      __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      __Pyx_DECREF(__pyx_t_14); __pyx_t_14 = 0;
      if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 936, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_7);
    }
    __pyx_r = __pyx_t_7;
    __pyx_t_7 = 0;
    goto __pyx_L0;

    /* "cupyx/cusolver.pyx":934
 *         'geqrf', dev_info)
 * 
 *     if mode == 'r':             # <<<<<<<<<<<<<<
 *         r = x[..., :mn].swapaxes(-2, -1)
 *         return _cupy.linalg._util._triu(r).astype(out_dtype, copy=False)
*/
  }

  /* "cupyx/cusolver.pyx":938
 *         return _cupy.linalg._util._triu(r).astype(out_dtype, copy=False)
 * 
 *     if mode == 'raw':             # <<<<<<<<<<<<<<
 *         return (x.astype(out_dtype, copy=False),
 *                 tau.astype(out_dtype, copy=False))
*/
  __pyx_t_11 = (__Pyx_PyUnicode_Equals(__pyx_v_mode, __pyx_mstate_global->__pyx_n_u_raw, Py_EQ)); if (unlikely((__pyx_t_11 < 0))) __PYX_ERR(0, 938, __pyx_L1_error)
  if (__pyx_t_11) {

    /* "cupyx/cusolver.pyx":939
 * 
 *     if mode == 'raw':
 *         return (x.astype(out_dtype, copy=False),             # <<<<<<<<<<<<<<
 *                 tau.astype(out_dtype, copy=False))
 * 
*/
    __Pyx_XDECREF(__pyx_r);
    __pyx_t_14 = __pyx_v_x;
    __Pyx_INCREF(__pyx_t_14);
    __pyx_t_5 = 0;
    {
      PyObject *__pyx_callargs[2 + ((CYTHON_VECTORCALL) ? 1 : 0)] = {__pyx_t_14, __pyx_v_out_dtype};
      __pyx_t_1 = __Pyx_MakeVectorcallBuilderKwds(1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 939, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      if (__Pyx_VectorcallBuilder_AddArg(__pyx_mstate_global->__pyx_n_u_copy, Py_False, __pyx_t_1, __pyx_callargs+2, 0) < (0)) __PYX_ERR(0, 939, __pyx_L1_error)
      __pyx_t_7 = __Pyx_Object_VectorcallMethod_CallFromBuilder((PyObject*)__pyx_mstate_global->__pyx_n_u_astype, __pyx_callargs+__pyx_t_5, (2-__pyx_t_5) | (1*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET), __pyx_t_1);
      __Pyx_XDECREF(__pyx_t_14); __pyx_t_14 = 0;
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 939, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_7);
    }

    /* "cupyx/cusolver.pyx":940
 *     if mode == 'raw':
 *         return (x.astype(out_dtype, copy=False),
 *                 tau.astype(out_dtype, copy=False))             # <<<<<<<<<<<<<<
 * 
 *     if mode == 'complete' and m > n:
*/
    __pyx_t_14 = __pyx_v_tau;
    __Pyx_INCREF(__pyx_t_14);
    __pyx_t_5 = 0;
    {
      PyObject *__pyx_callargs[2 + ((CYTHON_VECTORCALL) ? 1 : 0)] = {__pyx_t_14, __pyx_v_out_dtype};
      __pyx_t_2 = __Pyx_MakeVectorcallBuilderKwds(1); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 940, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_2);
      if (__Pyx_VectorcallBuilder_AddArg(__pyx_mstate_global->__pyx_n_u_copy, Py_False, __pyx_t_2, __pyx_callargs+2, 0) < (0)) __PYX_ERR(0, 940, __pyx_L1_error)
      __pyx_t_1 = __Pyx_Object_VectorcallMethod_CallFromBuilder((PyObject*)__pyx_mstate_global->__pyx_n_u_astype, __pyx_callargs+__pyx_t_5, (2-__pyx_t_5) | (1*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET), __pyx_t_2);
      __Pyx_XDECREF(__pyx_t_14); __pyx_t_14 = 0;
      __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
      if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 940, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
    }

    /* "cupyx/cusolver.pyx":939
 * 
 *     if mode == 'raw':
 *         return (x.astype(out_dtype, copy=False),             # <<<<<<<<<<<<<<
 *                 tau.astype(out_dtype, copy=False))
 * 
*/
    __pyx_t_2 = PyTuple_New(2); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 939, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_GIVEREF(__pyx_t_7);
    if (__Pyx_PyTuple_SET_ITEM(__pyx_t_2, 0, __pyx_t_7) != (0)) __PYX_ERR(0, 939, __pyx_L1_error);
    __Pyx_GIVEREF(__pyx_t_1);
    if (__Pyx_PyTuple_SET_ITEM(__pyx_t_2, 1, __pyx_t_1) != (0)) __PYX_ERR(0, 939, __pyx_L1_error);
    __pyx_t_7 = 0;
    __pyx_t_1 = 0;
    __pyx_r = __pyx_t_2;
    __pyx_t_2 = 0;
    goto __pyx_L0;

    /* "cupyx/cusolver.pyx":938
 *         return _cupy.linalg._util._triu(r).astype(out_dtype, copy=False)
 * 
 *     if mode == 'raw':             # <<<<<<<<<<<<<<
 *         return (x.astype(out_dtype, copy=False),
 *                 tau.astype(out_dtype, copy=False))
*/
  }

  /* "cupyx/cusolver.pyx":942
 *                 tau.astype(out_dtype, copy=False))
 * 
 *     if mode == 'complete' and m > n:             # <<<<<<<<<<<<<<
 *         mc = m
 *         orig_n = m
*/
  __pyx_t_15 = (__Pyx_PyUnicode_Equals(__pyx_v_mode, __pyx_mstate_global->__pyx_n_u_complete, Py_EQ)); if (unlikely((__pyx_t_15 < 0))) __PYX_ERR(0, 942, __pyx_L1_error)
  if (__pyx_t_15) {
  } else {
    __pyx_t_11 = __pyx_t_15;
    goto __pyx_L16_bool_binop_done;
  }
  __pyx_t_15 = (__pyx_v_m > __pyx_v_n);
  __pyx_t_11 = __pyx_t_15;
  __pyx_L16_bool_binop_done:;
  if (__pyx_t_11) {

    /* "cupyx/cusolver.pyx":943
 * 
 *     if mode == 'complete' and m > n:
 *         mc = m             # <<<<<<<<<<<<<<
 *         orig_n = m
 *         q = _cupy.empty((batch_size, m, m), dtype)
*/
    __pyx_v_mc = __pyx_v_m;

    /* "cupyx/cusolver.pyx":944
 *     if mode == 'complete' and m > n:
 *         mc = m
 *         orig_n = m             # <<<<<<<<<<<<<<
 *         q = _cupy.empty((batch_size, m, m), dtype)
 *     else:
*/
    __pyx_v_orig_n = __pyx_v_m;

    /* "cupyx/cusolver.pyx":945
 *         mc = m
 *         orig_n = m
 *         q = _cupy.empty((batch_size, m, m), dtype)             # <<<<<<<<<<<<<<
 *     else:
 *         mc = mn
*/
    __pyx_t_1 = NULL;
    __Pyx_GetModuleGlobalName(__pyx_t_7, __pyx_mstate_global->__pyx_n_u_cupy); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 945, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_7);
    __pyx_t_14 = __Pyx_PyObject_GetAttrStr(__pyx_t_7, __pyx_mstate_global->__pyx_n_u_empty); if (unlikely(!__pyx_t_14)) __PYX_ERR(0, 945, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_14);
    __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
    __pyx_t_7 = __Pyx_PyLong_From_int(__pyx_v_batch_size); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 945, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_7);
    __pyx_t_4 = __Pyx_PyLong_From_int(__pyx_v_m); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 945, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __pyx_t_3 = __Pyx_PyLong_From_int(__pyx_v_m); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 945, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_t_16 = PyTuple_New(3); if (unlikely(!__pyx_t_16)) __PYX_ERR(0, 945, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_16);
    __Pyx_GIVEREF(__pyx_t_7);
    if (__Pyx_PyTuple_SET_ITEM(__pyx_t_16, 0, __pyx_t_7) != (0)) __PYX_ERR(0, 945, __pyx_L1_error);
    __Pyx_GIVEREF(__pyx_t_4);
    if (__Pyx_PyTuple_SET_ITEM(__pyx_t_16, 1, __pyx_t_4) != (0)) __PYX_ERR(0, 945, __pyx_L1_error);
    __Pyx_GIVEREF(__pyx_t_3);
    if (__Pyx_PyTuple_SET_ITEM(__pyx_t_16, 2, __pyx_t_3) != (0)) __PYX_ERR(0, 945, __pyx_L1_error);
    __pyx_t_7 = 0;
    __pyx_t_4 = 0;
    __pyx_t_3 = 0;
    __pyx_t_5 = 1;
    #if CYTHON_UNPACK_METHODS
    if (unlikely(PyMethod_Check(__pyx_t_14))) {
      __pyx_t_1 = PyMethod_GET_SELF(__pyx_t_14);
      assert(__pyx_t_1);
      PyObject* __pyx__function = PyMethod_GET_FUNCTION(__pyx_t_14);
      __Pyx_INCREF(__pyx_t_1);
      __Pyx_INCREF(__pyx__function);
      __Pyx_DECREF_SET(__pyx_t_14, __pyx__function);
      __pyx_t_5 = 0;
    }
    #endif
    {
      PyObject *__pyx_callargs[3] = {__pyx_t_1, __pyx_t_16, __pyx_v_dtype};
      __pyx_t_2 = __Pyx_PyObject_FastCall((PyObject*)__pyx_t_14, __pyx_callargs+__pyx_t_5, (3-__pyx_t_5) | (__pyx_t_5*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
      __Pyx_XDECREF(__pyx_t_1); __pyx_t_1 = 0;
      __Pyx_DECREF(__pyx_t_16); __pyx_t_16 = 0;
      __Pyx_DECREF(__pyx_t_14); __pyx_t_14 = 0;
      if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 945, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_2);
    }
    __pyx_v_q = __pyx_t_2;
    __pyx_t_2 = 0;

    /* "cupyx/cusolver.pyx":942
 *                 tau.astype(out_dtype, copy=False))
 * 
 *     if mode == 'complete' and m > n:             # <<<<<<<<<<<<<<
 *         mc = m
 *         orig_n = m
*/
    goto __pyx_L15;
  }

  /* "cupyx/cusolver.pyx":947
 *         q = _cupy.empty((batch_size, m, m), dtype)
 *     else:
 *         mc = mn             # <<<<<<<<<<<<<<
 *         orig_n = n
 *         q = _cupy.empty((batch_size, n, m), dtype)
*/
  /*else*/ {
    __pyx_v_mc = __pyx_v_mn;

    /* "cupyx/cusolver.pyx":948
 *     else:
 *         mc = mn
 *         orig_n = n             # <<<<<<<<<<<<<<
 *         q = _cupy.empty((batch_size, n, m), dtype)
 *     q[..., :n, :] = x
*/
    __pyx_v_orig_n = __pyx_v_n;

    /* "cupyx/cusolver.pyx":949
 *         mc = mn
 *         orig_n = n
 *         q = _cupy.empty((batch_size, n, m), dtype)             # <<<<<<<<<<<<<<
 *     q[..., :n, :] = x
 *     x_ptr = q.data.ptr
*/
    __pyx_t_14 = NULL;
    __Pyx_GetModuleGlobalName(__pyx_t_16, __pyx_mstate_global->__pyx_n_u_cupy); if (unlikely(!__pyx_t_16)) __PYX_ERR(0, 949, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_16);
    __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_t_16, __pyx_mstate_global->__pyx_n_u_empty); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 949, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_16); __pyx_t_16 = 0;
    __pyx_t_16 = __Pyx_PyLong_From_int(__pyx_v_batch_size); if (unlikely(!__pyx_t_16)) __PYX_ERR(0, 949, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_16);
    __pyx_t_3 = __Pyx_PyLong_From_int(__pyx_v_n); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 949, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_t_4 = __Pyx_PyLong_From_int(__pyx_v_m); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 949, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __pyx_t_7 = PyTuple_New(3); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 949, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_7);
    __Pyx_GIVEREF(__pyx_t_16);
    if (__Pyx_PyTuple_SET_ITEM(__pyx_t_7, 0, __pyx_t_16) != (0)) __PYX_ERR(0, 949, __pyx_L1_error);
    __Pyx_GIVEREF(__pyx_t_3);
    if (__Pyx_PyTuple_SET_ITEM(__pyx_t_7, 1, __pyx_t_3) != (0)) __PYX_ERR(0, 949, __pyx_L1_error);
    __Pyx_GIVEREF(__pyx_t_4);
    if (__Pyx_PyTuple_SET_ITEM(__pyx_t_7, 2, __pyx_t_4) != (0)) __PYX_ERR(0, 949, __pyx_L1_error);
    __pyx_t_16 = 0;
    __pyx_t_3 = 0;
    __pyx_t_4 = 0;
    __pyx_t_5 = 1;
    #if CYTHON_UNPACK_METHODS
    if (unlikely(PyMethod_Check(__pyx_t_1))) {
      __pyx_t_14 = PyMethod_GET_SELF(__pyx_t_1);
      assert(__pyx_t_14);
      PyObject* __pyx__function = PyMethod_GET_FUNCTION(__pyx_t_1);
      __Pyx_INCREF(__pyx_t_14);
      __Pyx_INCREF(__pyx__function);
      __Pyx_DECREF_SET(__pyx_t_1, __pyx__function);
      __pyx_t_5 = 0;
    }
    #endif
    {
      PyObject *__pyx_callargs[3] = {__pyx_t_14, __pyx_t_7, __pyx_v_dtype};
      __pyx_t_2 = __Pyx_PyObject_FastCall((PyObject*)__pyx_t_1, __pyx_callargs+__pyx_t_5, (3-__pyx_t_5) | (__pyx_t_5*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
      __Pyx_XDECREF(__pyx_t_14); __pyx_t_14 = 0;
      __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 949, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_2);
    }
    __pyx_v_q = __pyx_t_2;
    __pyx_t_2 = 0;
  }
  __pyx_L15:;

  /* "cupyx/cusolver.pyx":950
 *         orig_n = n
 *         q = _cupy.empty((batch_size, n, m), dtype)
 *     q[..., :n, :] = x             # <<<<<<<<<<<<<<
 *     x_ptr = q.data.ptr
 * 
*/
  __pyx_t_2 = __Pyx_PyLong_From_int(__pyx_v_n); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 950, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_1 = PySlice_New(Py_None, __pyx_t_2, Py_None); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 950, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_2 = PyTuple_New(3); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 950, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_INCREF(Py_Ellipsis);
  __Pyx_GIVEREF(Py_Ellipsis);
  if (__Pyx_PyTuple_SET_ITEM(__pyx_t_2, 0, Py_Ellipsis) != (0)) __PYX_ERR(0, 950, __pyx_L1_error);
  __Pyx_GIVEREF(__pyx_t_1);
  if (__Pyx_PyTuple_SET_ITEM(__pyx_t_2, 1, __pyx_t_1) != (0)) __PYX_ERR(0, 950, __pyx_L1_error);
  __Pyx_INCREF(__pyx_mstate_global->__pyx_slice[2]);
  __Pyx_GIVEREF(__pyx_mstate_global->__pyx_slice[2]);
  if (__Pyx_PyTuple_SET_ITEM(__pyx_t_2, 2, __pyx_mstate_global->__pyx_slice[2]) != (0)) __PYX_ERR(0, 950, __pyx_L1_error);
  __pyx_t_1 = 0;
  if (unlikely((PyObject_SetItem(__pyx_v_q, __pyx_t_2, __pyx_v_x) < 0))) __PYX_ERR(0, 950, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "cupyx/cusolver.pyx":951
 *         q = _cupy.empty((batch_size, n, m), dtype)
 *     q[..., :n, :] = x
 *     x_ptr = q.data.ptr             # <<<<<<<<<<<<<<
 * 
 *     # compute working space of orgqr and solve Q
*/
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_v_q, __pyx_mstate_global->__pyx_n_u_data); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 951, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_t_2, __pyx_mstate_global->__pyx_n_u_ptr); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 951, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_12 = PyLong_AsSsize_t(__pyx_t_1); if (unlikely((__pyx_t_12 == ((intptr_t)-1)) && PyErr_Occurred())) __PYX_ERR(0, 951, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v_x_ptr = __pyx_t_12;

  /* "cupyx/cusolver.pyx":954
 * 
 *     # compute working space of orgqr and solve Q
 *     cdef orgqr_ptr orgqr = NULL             # <<<<<<<<<<<<<<
 *     if dtype == 'f':
 *         orgqr_bufferSize = sorgqr_bufferSize
*/
  __pyx_v_orgqr = NULL;

  /* "cupyx/cusolver.pyx":955
 *     # compute working space of orgqr and solve Q
 *     cdef orgqr_ptr orgqr = NULL
 *     if dtype == 'f':             # <<<<<<<<<<<<<<
 *         orgqr_bufferSize = sorgqr_bufferSize
 *         orgqr = orgqr_loop[float]
*/
  __pyx_t_11 = (__Pyx_PyUnicode_Equals(__pyx_v_dtype, __pyx_mstate_global->__pyx_n_u_f, Py_EQ)); if (unlikely((__pyx_t_11 < 0))) __PYX_ERR(0, 955, __pyx_L1_error)
  if (__pyx_t_11) {

    /* "cupyx/cusolver.pyx":956
 *     cdef orgqr_ptr orgqr = NULL
 *     if dtype == 'f':
 *         orgqr_bufferSize = sorgqr_bufferSize             # <<<<<<<<<<<<<<
 *         orgqr = orgqr_loop[float]
 *     elif dtype == 'd':
*/
    __pyx_v_orgqr_bufferSize = __pyx_f_13cupy_backends_4cuda_4libs_8cusolver_sorgqr_bufferSize;

    /* "cupyx/cusolver.pyx":957
 *     if dtype == 'f':
 *         orgqr_bufferSize = sorgqr_bufferSize
 *         orgqr = orgqr_loop[float]             # <<<<<<<<<<<<<<
 *     elif dtype == 'd':
 *         orgqr_bufferSize = dorgqr_bufferSize
*/
    __pyx_v_orgqr = orgqr_loop<float>;

    /* "cupyx/cusolver.pyx":955
 *     # compute working space of orgqr and solve Q
 *     cdef orgqr_ptr orgqr = NULL
 *     if dtype == 'f':             # <<<<<<<<<<<<<<
 *         orgqr_bufferSize = sorgqr_bufferSize
 *         orgqr = orgqr_loop[float]
*/
    goto __pyx_L18;
  }

  /* "cupyx/cusolver.pyx":958
 *         orgqr_bufferSize = sorgqr_bufferSize
 *         orgqr = orgqr_loop[float]
 *     elif dtype == 'd':             # <<<<<<<<<<<<<<
 *         orgqr_bufferSize = dorgqr_bufferSize
 *         orgqr = orgqr_loop[double]
*/
  __pyx_t_11 = (__Pyx_PyUnicode_Equals(__pyx_v_dtype, __pyx_mstate_global->__pyx_n_u_d, Py_EQ)); if (unlikely((__pyx_t_11 < 0))) __PYX_ERR(0, 958, __pyx_L1_error)
  if (__pyx_t_11) {

    /* "cupyx/cusolver.pyx":959
 *         orgqr = orgqr_loop[float]
 *     elif dtype == 'd':
 *         orgqr_bufferSize = dorgqr_bufferSize             # <<<<<<<<<<<<<<
 *         orgqr = orgqr_loop[double]
 *     elif dtype == 'F':
*/
    __pyx_v_orgqr_bufferSize = __pyx_f_13cupy_backends_4cuda_4libs_8cusolver_dorgqr_bufferSize;

    /* "cupyx/cusolver.pyx":960
 *     elif dtype == 'd':
 *         orgqr_bufferSize = dorgqr_bufferSize
 *         orgqr = orgqr_loop[double]             # <<<<<<<<<<<<<<
 *     elif dtype == 'F':
 *         orgqr_bufferSize = cungqr_bufferSize
*/
    __pyx_v_orgqr = orgqr_loop<double>;

    /* "cupyx/cusolver.pyx":958
 *         orgqr_bufferSize = sorgqr_bufferSize
 *         orgqr = orgqr_loop[float]
 *     elif dtype == 'd':             # <<<<<<<<<<<<<<
 *         orgqr_bufferSize = dorgqr_bufferSize
 *         orgqr = orgqr_loop[double]
*/
    goto __pyx_L18;
  }

  /* "cupyx/cusolver.pyx":961
 *         orgqr_bufferSize = dorgqr_bufferSize
 *         orgqr = orgqr_loop[double]
 *     elif dtype == 'F':             # <<<<<<<<<<<<<<
 *         orgqr_bufferSize = cungqr_bufferSize
 *         orgqr = orgqr_loop[cuComplex]
*/
  __pyx_t_11 = (__Pyx_PyUnicode_Equals(__pyx_v_dtype, __pyx_mstate_global->__pyx_n_u_F, Py_EQ)); if (unlikely((__pyx_t_11 < 0))) __PYX_ERR(0, 961, __pyx_L1_error)
  if (__pyx_t_11) {

    /* "cupyx/cusolver.pyx":962
 *         orgqr = orgqr_loop[double]
 *     elif dtype == 'F':
 *         orgqr_bufferSize = cungqr_bufferSize             # <<<<<<<<<<<<<<
 *         orgqr = orgqr_loop[cuComplex]
 *     elif dtype == 'D':
*/
    __pyx_v_orgqr_bufferSize = __pyx_f_13cupy_backends_4cuda_4libs_8cusolver_cungqr_bufferSize;

    /* "cupyx/cusolver.pyx":963
 *     elif dtype == 'F':
 *         orgqr_bufferSize = cungqr_bufferSize
 *         orgqr = orgqr_loop[cuComplex]             # <<<<<<<<<<<<<<
 *     elif dtype == 'D':
 *         orgqr_bufferSize = zungqr_bufferSize
*/
    __pyx_v_orgqr = orgqr_loop<cuComplex>;

    /* "cupyx/cusolver.pyx":961
 *         orgqr_bufferSize = dorgqr_bufferSize
 *         orgqr = orgqr_loop[double]
 *     elif dtype == 'F':             # <<<<<<<<<<<<<<
 *         orgqr_bufferSize = cungqr_bufferSize
 *         orgqr = orgqr_loop[cuComplex]
*/
    goto __pyx_L18;
  }

  /* "cupyx/cusolver.pyx":964
 *         orgqr_bufferSize = cungqr_bufferSize
 *         orgqr = orgqr_loop[cuComplex]
 *     elif dtype == 'D':             # <<<<<<<<<<<<<<
 *         orgqr_bufferSize = zungqr_bufferSize
 *         orgqr = orgqr_loop[cuDoubleComplex]
*/
  __pyx_t_11 = (__Pyx_PyUnicode_Equals(__pyx_v_dtype, __pyx_mstate_global->__pyx_n_u_D, Py_EQ)); if (unlikely((__pyx_t_11 < 0))) __PYX_ERR(0, 964, __pyx_L1_error)
  if (likely(__pyx_t_11)) {

    /* "cupyx/cusolver.pyx":965
 *         orgqr = orgqr_loop[cuComplex]
 *     elif dtype == 'D':
 *         orgqr_bufferSize = zungqr_bufferSize             # <<<<<<<<<<<<<<
 *         orgqr = orgqr_loop[cuDoubleComplex]
 *     else:
*/
    __pyx_v_orgqr_bufferSize = __pyx_f_13cupy_backends_4cuda_4libs_8cusolver_zungqr_bufferSize;

    /* "cupyx/cusolver.pyx":966
 *     elif dtype == 'D':
 *         orgqr_bufferSize = zungqr_bufferSize
 *         orgqr = orgqr_loop[cuDoubleComplex]             # <<<<<<<<<<<<<<
 *     else:
 *         raise ValueError
*/
    __pyx_v_orgqr = orgqr_loop<cuDoubleComplex>;

    /* "cupyx/cusolver.pyx":964
 *         orgqr_bufferSize = cungqr_bufferSize
 *         orgqr = orgqr_loop[cuComplex]
 *     elif dtype == 'D':             # <<<<<<<<<<<<<<
 *         orgqr_bufferSize = zungqr_bufferSize
 *         orgqr = orgqr_loop[cuDoubleComplex]
*/
    goto __pyx_L18;
  }

  /* "cupyx/cusolver.pyx":968
 *         orgqr = orgqr_loop[cuDoubleComplex]
 *     else:
 *         raise ValueError             # <<<<<<<<<<<<<<
 * 
 *     # this wrapper also sets the stream for us
*/
  /*else*/ {
    __Pyx_Raise(((PyObject *)(((PyTypeObject*)PyExc_ValueError))), 0, 0, 0);
    __PYX_ERR(0, 968, __pyx_L1_error)
  }
  __pyx_L18:;

  /* "cupyx/cusolver.pyx":971
 * 
 *     # this wrapper also sets the stream for us
 *     buffersize = orgqr_bufferSize(             # <<<<<<<<<<<<<<
 *         handle, m, mc, mn, x_ptr, m, tau_ptr)
 *     workspace = memory.alloc(buffersize * a.dtype.itemsize)
*/
  __pyx_t_8 = __pyx_v_orgqr_bufferSize(__pyx_v_handle, __pyx_v_m, __pyx_v_mc, __pyx_v_mn, __pyx_v_x_ptr, __pyx_v_m, __pyx_v_tau_ptr, 0); if (unlikely(__pyx_t_8 == ((int)-1) && PyErr_Occurred())) __PYX_ERR(0, 971, __pyx_L1_error)
  __pyx_v_buffersize = __pyx_t_8;

  /* "cupyx/cusolver.pyx":973
 *     buffersize = orgqr_bufferSize(
 *         handle, m, mc, mn, x_ptr, m, tau_ptr)
 *     workspace = memory.alloc(buffersize * a.dtype.itemsize)             # <<<<<<<<<<<<<<
 *     w_ptr = workspace.ptr
 * 
*/
  __pyx_t_1 = __Pyx_PyLong_From_int(__pyx_v_buffersize); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 973, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_v_a, __pyx_mstate_global->__pyx_n_u_dtype); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 973, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_7 = __Pyx_PyObject_GetAttrStr(__pyx_t_2, __pyx_mstate_global->__pyx_n_u_itemsize); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 973, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_7);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_2 = PyNumber_Multiply(__pyx_t_1, __pyx_t_7); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 973, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
  __pyx_t_7 = ((PyObject *)__pyx_f_4cupy_4cuda_6memory_alloc(__pyx_t_2, 0)); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 973, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_7);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __Pyx_DECREF_SET(__pyx_v_workspace, ((struct __pyx_obj_4cupy_4cuda_6memory_MemoryPointer *)__pyx_t_7));
  __pyx_t_7 = 0;

  /* "cupyx/cusolver.pyx":974
 *         handle, m, mc, mn, x_ptr, m, tau_ptr)
 *     workspace = memory.alloc(buffersize * a.dtype.itemsize)
 *     w_ptr = workspace.ptr             # <<<<<<<<<<<<<<
 * 
 *     with nogil:
*/
  __pyx_t_12 = __pyx_v_workspace->ptr;
  __pyx_v_w_ptr = __pyx_t_12;

  /* "cupyx/cusolver.pyx":976
 *     w_ptr = workspace.ptr
 * 
 *     with nogil:             # <<<<<<<<<<<<<<
 *         status = orgqr(
 *             handle, m, mc, mn, x_ptr, m, tau_ptr, w_ptr,
*/
  {
      PyThreadState * _save;
      _save = PyEval_SaveThread();
      __Pyx_FastGIL_Remember();
      /*try:*/ {

        /* "cupyx/cusolver.pyx":977
 * 
 *     with nogil:
 *         status = orgqr(             # <<<<<<<<<<<<<<
 *             handle, m, mc, mn, x_ptr, m, tau_ptr, w_ptr,
 *             buffersize, info_ptr, batch_size, orig_n)
*/
        __pyx_v_status = __pyx_v_orgqr(__pyx_v_handle, __pyx_v_m, __pyx_v_mc, __pyx_v_mn, __pyx_v_x_ptr, __pyx_v_m, __pyx_v_tau_ptr, __pyx_v_w_ptr, __pyx_v_buffersize, __pyx_v_info_ptr, __pyx_v_batch_size, __pyx_v_orig_n);
      }

      /* "cupyx/cusolver.pyx":976
 *     w_ptr = workspace.ptr
 * 
 *     with nogil:             # <<<<<<<<<<<<<<
 *         status = orgqr(
 *             handle, m, mc, mn, x_ptr, m, tau_ptr, w_ptr,
*/
      /*finally:*/ {
        /*normal exit:*/{
          __Pyx_FastGIL_Forget();
          PyEval_RestoreThread(_save);
          goto __pyx_L21;
        }
        __pyx_L21:;
      }
  }

  /* "cupyx/cusolver.pyx":980
 *             handle, m, mc, mn, x_ptr, m, tau_ptr, w_ptr,
 *             buffersize, info_ptr, batch_size, orig_n)
 *     if status != 0:             # <<<<<<<<<<<<<<
 *         raise _cusolver.CUSOLVERError(status)
 *     _cupy.linalg._util._check_cusolver_dev_info_if_synchronization_allowed(
*/
  __pyx_t_11 = (__pyx_v_status != 0);
  if (unlikely(__pyx_t_11)) {

    /* "cupyx/cusolver.pyx":981
 *             buffersize, info_ptr, batch_size, orig_n)
 *     if status != 0:
 *         raise _cusolver.CUSOLVERError(status)             # <<<<<<<<<<<<<<
 *     _cupy.linalg._util._check_cusolver_dev_info_if_synchronization_allowed(
 *         'orgqr', dev_info)
*/
    __pyx_t_2 = NULL;
    __Pyx_GetModuleGlobalName(__pyx_t_1, __pyx_mstate_global->__pyx_n_u_cusolver); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 981, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_t_14 = __Pyx_PyObject_GetAttrStr(__pyx_t_1, __pyx_mstate_global->__pyx_n_u_CUSOLVERError); if (unlikely(!__pyx_t_14)) __PYX_ERR(0, 981, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_14);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __pyx_t_1 = __Pyx_PyLong_From_int(__pyx_v_status); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 981, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_t_5 = 1;
    #if CYTHON_UNPACK_METHODS
    if (unlikely(PyMethod_Check(__pyx_t_14))) {
      __pyx_t_2 = PyMethod_GET_SELF(__pyx_t_14);
      assert(__pyx_t_2);
      PyObject* __pyx__function = PyMethod_GET_FUNCTION(__pyx_t_14);
      __Pyx_INCREF(__pyx_t_2);
      __Pyx_INCREF(__pyx__function);
      __Pyx_DECREF_SET(__pyx_t_14, __pyx__function);
      __pyx_t_5 = 0;
    }
    #endif
    {
      PyObject *__pyx_callargs[2] = {__pyx_t_2, __pyx_t_1};
      __pyx_t_7 = __Pyx_PyObject_FastCall((PyObject*)__pyx_t_14, __pyx_callargs+__pyx_t_5, (2-__pyx_t_5) | (__pyx_t_5*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
      __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      __Pyx_DECREF(__pyx_t_14); __pyx_t_14 = 0;
      if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 981, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_7);
    }
    __Pyx_Raise(__pyx_t_7, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
    __PYX_ERR(0, 981, __pyx_L1_error)

    /* "cupyx/cusolver.pyx":980
 *             handle, m, mc, mn, x_ptr, m, tau_ptr, w_ptr,
 *             buffersize, info_ptr, batch_size, orig_n)
 *     if status != 0:             # <<<<<<<<<<<<<<
 *         raise _cusolver.CUSOLVERError(status)
 *     _cupy.linalg._util._check_cusolver_dev_info_if_synchronization_allowed(
*/
  }

  /* "cupyx/cusolver.pyx":982
 *     if status != 0:
 *         raise _cusolver.CUSOLVERError(status)
 *     _cupy.linalg._util._check_cusolver_dev_info_if_synchronization_allowed(             # <<<<<<<<<<<<<<
 *         'orgqr', dev_info)
 * 
*/
  __Pyx_GetModuleGlobalName(__pyx_t_1, __pyx_mstate_global->__pyx_n_u_cupy); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 982, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_t_1, __pyx_mstate_global->__pyx_n_u_linalg); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 982, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_t_2, __pyx_mstate_global->__pyx_n_u_util); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 982, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_14 = __pyx_t_1;
  __Pyx_INCREF(__pyx_t_14);

  /* "cupyx/cusolver.pyx":983
 *         raise _cusolver.CUSOLVERError(status)
 *     _cupy.linalg._util._check_cusolver_dev_info_if_synchronization_allowed(
 *         'orgqr', dev_info)             # <<<<<<<<<<<<<<
 * 
 *     q = q[..., :mc, :].swapaxes(-2, -1)
*/
  __pyx_t_5 = 0;
  {
    PyObject *__pyx_callargs[3] = {__pyx_t_14, __pyx_mstate_global->__pyx_n_u_orgqr, ((PyObject *)__pyx_v_dev_info)};
    __pyx_t_7 = __Pyx_PyObject_FastCallMethod((PyObject*)__pyx_mstate_global->__pyx_n_u_check_cusolver_dev_info_if_sync, __pyx_callargs+__pyx_t_5, (3-__pyx_t_5) | (1*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
    __Pyx_XDECREF(__pyx_t_14); __pyx_t_14 = 0;
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 982, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_7);
  }
  __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;

  /* "cupyx/cusolver.pyx":985
 *         'orgqr', dev_info)
 * 
 *     q = q[..., :mc, :].swapaxes(-2, -1)             # <<<<<<<<<<<<<<
 *     r = x[..., :mc].swapaxes(-2, -1)
 *     return (q.astype(out_dtype, copy=False),
*/
  __pyx_t_7 = __Pyx_PyLong_From_int(__pyx_v_mc); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 985, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_7);
  __pyx_t_1 = PySlice_New(Py_None, __pyx_t_7, Py_None); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 985, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
  __pyx_t_7 = PyTuple_New(3); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 985, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_7);
  __Pyx_INCREF(Py_Ellipsis);
  __Pyx_GIVEREF(Py_Ellipsis);
  if (__Pyx_PyTuple_SET_ITEM(__pyx_t_7, 0, Py_Ellipsis) != (0)) __PYX_ERR(0, 985, __pyx_L1_error);
  __Pyx_GIVEREF(__pyx_t_1);
  if (__Pyx_PyTuple_SET_ITEM(__pyx_t_7, 1, __pyx_t_1) != (0)) __PYX_ERR(0, 985, __pyx_L1_error);
  __Pyx_INCREF(__pyx_mstate_global->__pyx_slice[2]);
  __Pyx_GIVEREF(__pyx_mstate_global->__pyx_slice[2]);
  if (__Pyx_PyTuple_SET_ITEM(__pyx_t_7, 2, __pyx_mstate_global->__pyx_slice[2]) != (0)) __PYX_ERR(0, 985, __pyx_L1_error);
  __pyx_t_1 = 0;
  __pyx_t_1 = __Pyx_PyObject_GetItem(__pyx_v_q, __pyx_t_7); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 985, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
  __pyx_t_7 = __Pyx_PyObject_GetAttrStr(__pyx_t_1, __pyx_mstate_global->__pyx_n_u_swapaxes); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 985, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_7);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_7, __pyx_mstate_global->__pyx_tuple[0], NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 985, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
  __Pyx_DECREF_SET(__pyx_v_q, __pyx_t_1);
  __pyx_t_1 = 0;

  /* "cupyx/cusolver.pyx":986
 * 
 *     q = q[..., :mc, :].swapaxes(-2, -1)
 *     r = x[..., :mc].swapaxes(-2, -1)             # <<<<<<<<<<<<<<
 *     return (q.astype(out_dtype, copy=False),
 *             _cupy.linalg._util._triu(r).astype(out_dtype, copy=False))
*/
  __pyx_t_1 = __Pyx_PyLong_From_int(__pyx_v_mc); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 986, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_7 = PySlice_New(Py_None, __pyx_t_1, Py_None); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 986, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_7);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_1 = PyTuple_New(2); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 986, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_INCREF(Py_Ellipsis);
  __Pyx_GIVEREF(Py_Ellipsis);
  if (__Pyx_PyTuple_SET_ITEM(__pyx_t_1, 0, Py_Ellipsis) != (0)) __PYX_ERR(0, 986, __pyx_L1_error);
  __Pyx_GIVEREF(__pyx_t_7);
  if (__Pyx_PyTuple_SET_ITEM(__pyx_t_1, 1, __pyx_t_7) != (0)) __PYX_ERR(0, 986, __pyx_L1_error);
  __pyx_t_7 = 0;
  __pyx_t_7 = __Pyx_PyObject_GetItem(__pyx_v_x, __pyx_t_1); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 986, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_7);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_t_7, __pyx_mstate_global->__pyx_n_u_swapaxes); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 986, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
  __pyx_t_7 = __Pyx_PyObject_Call(__pyx_t_1, __pyx_mstate_global->__pyx_tuple[0], NULL); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 986, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_7);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v_r = __pyx_t_7;
  __pyx_t_7 = 0;

  /* "cupyx/cusolver.pyx":987
 *     q = q[..., :mc, :].swapaxes(-2, -1)
 *     r = x[..., :mc].swapaxes(-2, -1)
 *     return (q.astype(out_dtype, copy=False),             # <<<<<<<<<<<<<<
 *             _cupy.linalg._util._triu(r).astype(out_dtype, copy=False))
*/
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __pyx_v_q;
  __Pyx_INCREF(__pyx_t_1);
  __pyx_t_5 = 0;
  {
    PyObject *__pyx_callargs[2 + ((CYTHON_VECTORCALL) ? 1 : 0)] = {__pyx_t_1, __pyx_v_out_dtype};
    __pyx_t_14 = __Pyx_MakeVectorcallBuilderKwds(1); if (unlikely(!__pyx_t_14)) __PYX_ERR(0, 987, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_14);
    if (__Pyx_VectorcallBuilder_AddArg(__pyx_mstate_global->__pyx_n_u_copy, Py_False, __pyx_t_14, __pyx_callargs+2, 0) < (0)) __PYX_ERR(0, 987, __pyx_L1_error)
    __pyx_t_7 = __Pyx_Object_VectorcallMethod_CallFromBuilder((PyObject*)__pyx_mstate_global->__pyx_n_u_astype, __pyx_callargs+__pyx_t_5, (2-__pyx_t_5) | (1*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET), __pyx_t_14);
    __Pyx_XDECREF(__pyx_t_1); __pyx_t_1 = 0;
    __Pyx_DECREF(__pyx_t_14); __pyx_t_14 = 0;
    if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 987, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_7);
  }

  /* "cupyx/cusolver.pyx":988
 *     r = x[..., :mc].swapaxes(-2, -1)
 *     return (q.astype(out_dtype, copy=False),
 *             _cupy.linalg._util._triu(r).astype(out_dtype, copy=False))             # <<<<<<<<<<<<<<
*/
  __Pyx_GetModuleGlobalName(__pyx_t_3, __pyx_mstate_global->__pyx_n_u_cupy); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 988, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_16 = __Pyx_PyObject_GetAttrStr(__pyx_t_3, __pyx_mstate_global->__pyx_n_u_linalg); if (unlikely(!__pyx_t_16)) __PYX_ERR(0, 988, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_16);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_t_16, __pyx_mstate_global->__pyx_n_u_util); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 988, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_16); __pyx_t_16 = 0;
  __pyx_t_4 = __pyx_t_3;
  __Pyx_INCREF(__pyx_t_4);
  __pyx_t_5 = 0;
  {
    PyObject *__pyx_callargs[2] = {__pyx_t_4, __pyx_v_r};
    __pyx_t_2 = __Pyx_PyObject_FastCallMethod((PyObject*)__pyx_mstate_global->__pyx_n_u_triu, __pyx_callargs+__pyx_t_5, (2-__pyx_t_5) | (1*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
    __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 988, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
  }
  __pyx_t_1 = __pyx_t_2;
  __Pyx_INCREF(__pyx_t_1);
  __pyx_t_5 = 0;
  {
    PyObject *__pyx_callargs[2 + ((CYTHON_VECTORCALL) ? 1 : 0)] = {__pyx_t_1, __pyx_v_out_dtype};
    __pyx_t_3 = __Pyx_MakeVectorcallBuilderKwds(1); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 988, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    if (__Pyx_VectorcallBuilder_AddArg(__pyx_mstate_global->__pyx_n_u_copy, Py_False, __pyx_t_3, __pyx_callargs+2, 0) < (0)) __PYX_ERR(0, 988, __pyx_L1_error)
    __pyx_t_14 = __Pyx_Object_VectorcallMethod_CallFromBuilder((PyObject*)__pyx_mstate_global->__pyx_n_u_astype, __pyx_callargs+__pyx_t_5, (2-__pyx_t_5) | (1*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET), __pyx_t_3);
    __Pyx_XDECREF(__pyx_t_1); __pyx_t_1 = 0;
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    if (unlikely(!__pyx_t_14)) __PYX_ERR(0, 988, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_14);
  }

  /* "cupyx/cusolver.pyx":987
 *     q = q[..., :mc, :].swapaxes(-2, -1)
 *     r = x[..., :mc].swapaxes(-2, -1)
 *     return (q.astype(out_dtype, copy=False),             # <<<<<<<<<<<<<<
 *             _cupy.linalg._util._triu(r).astype(out_dtype, copy=False))
*/
  __pyx_t_2 = PyTuple_New(2); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 987, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_GIVEREF(__pyx_t_7);
  if (__Pyx_PyTuple_SET_ITEM(__pyx_t_2, 0, __pyx_t_7) != (0)) __PYX_ERR(0, 987, __pyx_L1_error);
  __Pyx_GIVEREF(__pyx_t_14);
  if (__Pyx_PyTuple_SET_ITEM(__pyx_t_2, 1, __pyx_t_14) != (0)) __PYX_ERR(0, 987, __pyx_L1_error);
  __pyx_t_7 = 0;
  __pyx_t_14 = 0;
  __pyx_r = __pyx_t_2;
  __pyx_t_2 = 0;
  goto __pyx_L0;

  /* "cupyx/cusolver.pyx":873
 * 
 * 
 * cpdef _geqrf_orgqr_batched(a, mode):             # <<<<<<<<<<<<<<
 *     '''Internal helper for batched QR solver. The input array ``a''
 *     is of shape (batch_size, m, n)
*/

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_7);
  __Pyx_XDECREF(__pyx_t_14);
  __Pyx_XDECREF(__pyx_t_16);
  __Pyx_AddTraceback("cupyx.cusolver._geqrf_orgqr_batched", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_dtype);
  __Pyx_XDECREF(__pyx_v_out_dtype);
  __Pyx_XDECREF(__pyx_v_x);
  __Pyx_XDECREF(__pyx_v_ap);
  __Pyx_XDECREF((PyObject *)__pyx_v_dev_info);
  __Pyx_XDECREF(__pyx_v_msg);
  __Pyx_XDECREF((PyObject *)__pyx_v_workspace);
  __Pyx_XDECREF(__pyx_v_tau);
  __Pyx_XDECREF(__pyx_v_r);
  __Pyx_XDECREF(__pyx_v_q);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_pw_5cupyx_8cusolver_21_geqrf_orgqr_batched(PyObject *__pyx_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
PyDoc_STRVAR(__pyx_doc_5cupyx_8cusolver_20_geqrf_orgqr_batched, "_geqrf_orgqr_batched(a, mode)\n\nInternal helper for batched QR solver. The input array ``a''\nis of shape (batch_size, m, n)");
static PyObject *__pyx_pw_5cupyx_8cusolver_21_geqrf_orgqr_batched(PyObject *__pyx_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
) {
  PyObject *__pyx_v_a = 0;
  PyObject *__pyx_v_mode = 0;
  #if !CYTHON_METH_FASTCALL
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  #endif
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject* values[2] = {0,0};
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("_geqrf_orgqr_batched (wrapper)", 0);
  #if !CYTHON_METH_FASTCALL
  #if CYTHON_ASSUME_SAFE_SIZE
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return NULL;
  #endif
  #endif
  __pyx_kwvalues = __Pyx_KwValues_FASTCALL(__pyx_args, __pyx_nargs);
  {
    PyObject ** const __pyx_pyargnames[] = {&__pyx_mstate_global->__pyx_n_u_a,&__pyx_mstate_global->__pyx_n_u_mode,0};
    const Py_ssize_t __pyx_kwds_len = (__pyx_kwds) ? __Pyx_NumKwargs_FASTCALL(__pyx_kwds) : 0;
    if (unlikely(__pyx_kwds_len < 0)) __PYX_ERR(0, 873, __pyx_L3_error)
    if (__pyx_kwds_len > 0) {
      switch (__pyx_nargs) {
        case  2:
        values[1] = __Pyx_ArgRef_FASTCALL(__pyx_args, 1);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[1])) __PYX_ERR(0, 873, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  1:
        values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(0, 873, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      const Py_ssize_t kwd_pos_args = __pyx_nargs;
      if (__Pyx_ParseKeywords(__pyx_kwds, __pyx_kwvalues, __pyx_pyargnames, 0, values, kwd_pos_args, __pyx_kwds_len, "_geqrf_orgqr_batched", 0) < (0)) __PYX_ERR(0, 873, __pyx_L3_error)
      for (Py_ssize_t i = __pyx_nargs; i < 2; i++) {
        if (unlikely(!values[i])) { __Pyx_RaiseArgtupleInvalid("_geqrf_orgqr_batched", 1, 2, 2, i); __PYX_ERR(0, 873, __pyx_L3_error) }
      }
    } else if (unlikely(__pyx_nargs != 2)) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
      if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(0, 873, __pyx_L3_error)
      values[1] = __Pyx_ArgRef_FASTCALL(__pyx_args, 1);
      if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[1])) __PYX_ERR(0, 873, __pyx_L3_error)
    }
    __pyx_v_a = values[0];
    __pyx_v_mode = values[1];
  }
  goto __pyx_L6_skip;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("_geqrf_orgqr_batched", 1, 2, 2, __pyx_nargs); __PYX_ERR(0, 873, __pyx_L3_error)
  __pyx_L6_skip:;
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_AddTraceback("cupyx.cusolver._geqrf_orgqr_batched", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_5cupyx_8cusolver_20_geqrf_orgqr_batched(__pyx_self, __pyx_v_a, __pyx_v_mode);

  /* function exit code */
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5cupyx_8cusolver_20_geqrf_orgqr_batched(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_a, PyObject *__pyx_v_mode) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("_geqrf_orgqr_batched", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __pyx_f_5cupyx_8cusolver__geqrf_orgqr_batched(__pyx_v_a, __pyx_v_mode, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 873, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("cupyx.cusolver._geqrf_orgqr_batched", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}
/* #### Code section: module_exttypes ### */

static PyMethodDef __pyx_methods[] = {
  {"_gesvdj_batched", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_5cupyx_8cusolver_5_gesvdj_batched, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_5cupyx_8cusolver_4_gesvdj_batched},
  {"_gesvd_batched", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_5cupyx_8cusolver_7_gesvd_batched, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_5cupyx_8cusolver_6_gesvd_batched},
  {"_geqrf_orgqr_batched", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_5cupyx_8cusolver_21_geqrf_orgqr_batched, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_5cupyx_8cusolver_20_geqrf_orgqr_batched},
  {0, 0, 0, 0}
};
/* #### Code section: initfunc_declarations ### */
static CYTHON_SMALL_CODE int __Pyx_InitCachedBuiltins(__pyx_mstatetype *__pyx_mstate); /*proto*/
static CYTHON_SMALL_CODE int __Pyx_InitCachedConstants(__pyx_mstatetype *__pyx_mstate); /*proto*/
static CYTHON_SMALL_CODE int __Pyx_InitGlobals(void); /*proto*/
static CYTHON_SMALL_CODE int __Pyx_InitConstants(__pyx_mstatetype *__pyx_mstate); /*proto*/
static CYTHON_SMALL_CODE int __Pyx_modinit_global_init_code(__pyx_mstatetype *__pyx_mstate); /*proto*/
static CYTHON_SMALL_CODE int __Pyx_modinit_variable_export_code(__pyx_mstatetype *__pyx_mstate); /*proto*/
static CYTHON_SMALL_CODE int __Pyx_modinit_function_export_code(__pyx_mstatetype *__pyx_mstate); /*proto*/
static CYTHON_SMALL_CODE int __Pyx_modinit_type_init_code(__pyx_mstatetype *__pyx_mstate); /*proto*/
static CYTHON_SMALL_CODE int __Pyx_modinit_type_import_code(__pyx_mstatetype *__pyx_mstate); /*proto*/
static CYTHON_SMALL_CODE int __Pyx_modinit_variable_import_code(__pyx_mstatetype *__pyx_mstate); /*proto*/
static CYTHON_SMALL_CODE int __Pyx_modinit_function_import_code(__pyx_mstatetype *__pyx_mstate); /*proto*/
static CYTHON_SMALL_CODE int __Pyx_CreateCodeObjects(__pyx_mstatetype *__pyx_mstate); /*proto*/
/* #### Code section: init_module ### */

static int __Pyx_modinit_global_init_code(__pyx_mstatetype *__pyx_mstate) {
  __Pyx_RefNannyDeclarations
  CYTHON_UNUSED_VAR(__pyx_mstate);
  __Pyx_RefNannySetupContext("__Pyx_modinit_global_init_code", 0);
  /*--- Global init code ---*/
  __Pyx_RefNannyFinishContext();
  return 0;
}

static int __Pyx_modinit_variable_export_code(__pyx_mstatetype *__pyx_mstate) {
  __Pyx_RefNannyDeclarations
  CYTHON_UNUSED_VAR(__pyx_mstate);
  __Pyx_RefNannySetupContext("__Pyx_modinit_variable_export_code", 0);
  /*--- Variable export code ---*/
  __Pyx_RefNannyFinishContext();
  return 0;
}

static int __Pyx_modinit_function_export_code(__pyx_mstatetype *__pyx_mstate) {
  __Pyx_RefNannyDeclarations
  CYTHON_UNUSED_VAR(__pyx_mstate);
  __Pyx_RefNannySetupContext("__Pyx_modinit_function_export_code", 0);
  /*--- Function export code ---*/
  __Pyx_RefNannyFinishContext();
  return 0;
}

static int __Pyx_modinit_type_init_code(__pyx_mstatetype *__pyx_mstate) {
  __Pyx_RefNannyDeclarations
  CYTHON_UNUSED_VAR(__pyx_mstate);
  __Pyx_RefNannySetupContext("__Pyx_modinit_type_init_code", 0);
  /*--- Type init code ---*/
  __Pyx_RefNannyFinishContext();
  return 0;
}

static int __Pyx_modinit_type_import_code(__pyx_mstatetype *__pyx_mstate) {
  __Pyx_RefNannyDeclarations
  CYTHON_UNUSED_VAR(__pyx_mstate);
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__Pyx_modinit_type_import_code", 0);
  /*--- Type import code ---*/
  __pyx_t_1 = PyImport_ImportModule("cupy_backends.cuda.api.runtime"); if (unlikely(!__pyx_t_1)) __PYX_ERR(4, 8, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_mstate->__pyx_ptype_13cupy_backends_4cuda_3api_7runtime_PointerAttributes = __Pyx_ImportType_3_2_5(__pyx_t_1, "cupy_backends.cuda.api.runtime", "PointerAttributes",
  #if defined(PYPY_VERSION_NUM) && PYPY_VERSION_NUM < 0x050B0000
  sizeof(struct __pyx_obj_13cupy_backends_4cuda_3api_7runtime_PointerAttributes), __PYX_GET_STRUCT_ALIGNMENT_3_2_5(struct __pyx_obj_13cupy_backends_4cuda_3api_7runtime_PointerAttributes),
  #elif CYTHON_COMPILING_IN_LIMITED_API
  sizeof(struct __pyx_obj_13cupy_backends_4cuda_3api_7runtime_PointerAttributes), __PYX_GET_STRUCT_ALIGNMENT_3_2_5(struct __pyx_obj_13cupy_backends_4cuda_3api_7runtime_PointerAttributes),
  #else
  sizeof(struct __pyx_obj_13cupy_backends_4cuda_3api_7runtime_PointerAttributes), __PYX_GET_STRUCT_ALIGNMENT_3_2_5(struct __pyx_obj_13cupy_backends_4cuda_3api_7runtime_PointerAttributes),
  #endif
  __Pyx_ImportType_CheckSize_Warn_3_2_5); if (!__pyx_mstate->__pyx_ptype_13cupy_backends_4cuda_3api_7runtime_PointerAttributes) __PYX_ERR(4, 8, __pyx_L1_error)
  __pyx_mstate->__pyx_ptype_13cupy_backends_4cuda_3api_7runtime_MemPoolProps = __Pyx_ImportType_3_2_5(__pyx_t_1, "cupy_backends.cuda.api.runtime", "MemPoolProps",
  #if defined(PYPY_VERSION_NUM) && PYPY_VERSION_NUM < 0x050B0000
  sizeof(struct __pyx_obj_13cupy_backends_4cuda_3api_7runtime_MemPoolProps), __PYX_GET_STRUCT_ALIGNMENT_3_2_5(struct __pyx_obj_13cupy_backends_4cuda_3api_7runtime_MemPoolProps),
  #elif CYTHON_COMPILING_IN_LIMITED_API
  sizeof(struct __pyx_obj_13cupy_backends_4cuda_3api_7runtime_MemPoolProps), __PYX_GET_STRUCT_ALIGNMENT_3_2_5(struct __pyx_obj_13cupy_backends_4cuda_3api_7runtime_MemPoolProps),
  #else
  sizeof(struct __pyx_obj_13cupy_backends_4cuda_3api_7runtime_MemPoolProps), __PYX_GET_STRUCT_ALIGNMENT_3_2_5(struct __pyx_obj_13cupy_backends_4cuda_3api_7runtime_MemPoolProps),
  #endif
  __Pyx_ImportType_CheckSize_Warn_3_2_5); if (!__pyx_mstate->__pyx_ptype_13cupy_backends_4cuda_3api_7runtime_MemPoolProps) __PYX_ERR(4, 15, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_1 = PyImport_ImportModule("cupy.cuda.device"); if (unlikely(!__pyx_t_1)) __PYX_ERR(5, 13, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_mstate->__pyx_ptype_4cupy_4cuda_6device_Handle = __Pyx_ImportType_3_2_5(__pyx_t_1, "cupy.cuda.device", "Handle",
  #if defined(PYPY_VERSION_NUM) && PYPY_VERSION_NUM < 0x050B0000
  sizeof(struct __pyx_obj_4cupy_4cuda_6device_Handle), __PYX_GET_STRUCT_ALIGNMENT_3_2_5(struct __pyx_obj_4cupy_4cuda_6device_Handle),
  #elif CYTHON_COMPILING_IN_LIMITED_API
  sizeof(struct __pyx_obj_4cupy_4cuda_6device_Handle), __PYX_GET_STRUCT_ALIGNMENT_3_2_5(struct __pyx_obj_4cupy_4cuda_6device_Handle),
  #else
  sizeof(struct __pyx_obj_4cupy_4cuda_6device_Handle), __PYX_GET_STRUCT_ALIGNMENT_3_2_5(struct __pyx_obj_4cupy_4cuda_6device_Handle),
  #endif
  __Pyx_ImportType_CheckSize_Warn_3_2_5); if (!__pyx_mstate->__pyx_ptype_4cupy_4cuda_6device_Handle) __PYX_ERR(5, 13, __pyx_L1_error)
  __pyx_mstate->__pyx_ptype_4cupy_4cuda_6device_Device = __Pyx_ImportType_3_2_5(__pyx_t_1, "cupy.cuda.device", "Device",
  #if defined(PYPY_VERSION_NUM) && PYPY_VERSION_NUM < 0x050B0000
  sizeof(struct __pyx_obj_4cupy_4cuda_6device_Device), __PYX_GET_STRUCT_ALIGNMENT_3_2_5(struct __pyx_obj_4cupy_4cuda_6device_Device),
  #elif CYTHON_COMPILING_IN_LIMITED_API
  sizeof(struct __pyx_obj_4cupy_4cuda_6device_Device), __PYX_GET_STRUCT_ALIGNMENT_3_2_5(struct __pyx_obj_4cupy_4cuda_6device_Device),
  #else
  sizeof(struct __pyx_obj_4cupy_4cuda_6device_Device), __PYX_GET_STRUCT_ALIGNMENT_3_2_5(struct __pyx_obj_4cupy_4cuda_6device_Device),
  #endif
  __Pyx_ImportType_CheckSize_Warn_3_2_5); if (!__pyx_mstate->__pyx_ptype_4cupy_4cuda_6device_Device) __PYX_ERR(5, 18, __pyx_L1_error)
  __pyx_vtabptr_4cupy_4cuda_6device_Device = (struct __pyx_vtabstruct_4cupy_4cuda_6device_Device*)__Pyx_GetVtable(__pyx_mstate->__pyx_ptype_4cupy_4cuda_6device_Device); if (unlikely(!__pyx_vtabptr_4cupy_4cuda_6device_Device)) __PYX_ERR(5, 18, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_1 = PyImport_ImportModule("cupy.cuda.memory"); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 10, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_mstate->__pyx_ptype_4cupy_4cuda_6memory_BaseMemory = __Pyx_ImportType_3_2_5(__pyx_t_1, "cupy.cuda.memory", "BaseMemory",
  #if defined(PYPY_VERSION_NUM) && PYPY_VERSION_NUM < 0x050B0000
  sizeof(struct __pyx_obj_4cupy_4cuda_6memory_BaseMemory), __PYX_GET_STRUCT_ALIGNMENT_3_2_5(struct __pyx_obj_4cupy_4cuda_6memory_BaseMemory),
  #elif CYTHON_COMPILING_IN_LIMITED_API
  sizeof(struct __pyx_obj_4cupy_4cuda_6memory_BaseMemory), __PYX_GET_STRUCT_ALIGNMENT_3_2_5(struct __pyx_obj_4cupy_4cuda_6memory_BaseMemory),
  #else
  sizeof(struct __pyx_obj_4cupy_4cuda_6memory_BaseMemory), __PYX_GET_STRUCT_ALIGNMENT_3_2_5(struct __pyx_obj_4cupy_4cuda_6memory_BaseMemory),
  #endif
  __Pyx_ImportType_CheckSize_Warn_3_2_5); if (!__pyx_mstate->__pyx_ptype_4cupy_4cuda_6memory_BaseMemory) __PYX_ERR(2, 10, __pyx_L1_error)
  __pyx_mstate->__pyx_ptype_4cupy_4cuda_6memory_SystemMemory = __Pyx_ImportType_3_2_5(__pyx_t_1, "cupy.cuda.memory", "SystemMemory",
  #if defined(PYPY_VERSION_NUM) && PYPY_VERSION_NUM < 0x050B0000
  sizeof(struct __pyx_obj_4cupy_4cuda_6memory_SystemMemory), __PYX_GET_STRUCT_ALIGNMENT_3_2_5(struct __pyx_obj_4cupy_4cuda_6memory_SystemMemory),
  #elif CYTHON_COMPILING_IN_LIMITED_API
  sizeof(struct __pyx_obj_4cupy_4cuda_6memory_SystemMemory), __PYX_GET_STRUCT_ALIGNMENT_3_2_5(struct __pyx_obj_4cupy_4cuda_6memory_SystemMemory),
  #else
  sizeof(struct __pyx_obj_4cupy_4cuda_6memory_SystemMemory), __PYX_GET_STRUCT_ALIGNMENT_3_2_5(struct __pyx_obj_4cupy_4cuda_6memory_SystemMemory),
  #endif
  __Pyx_ImportType_CheckSize_Warn_3_2_5); if (!__pyx_mstate->__pyx_ptype_4cupy_4cuda_6memory_SystemMemory) __PYX_ERR(2, 19, __pyx_L1_error)
  __pyx_vtabptr_4cupy_4cuda_6memory_SystemMemory = (struct __pyx_vtabstruct_4cupy_4cuda_6memory_SystemMemory*)__Pyx_GetVtable(__pyx_mstate->__pyx_ptype_4cupy_4cuda_6memory_SystemMemory); if (unlikely(!__pyx_vtabptr_4cupy_4cuda_6memory_SystemMemory)) __PYX_ERR(2, 19, __pyx_L1_error)
  __pyx_mstate->__pyx_ptype_4cupy_4cuda_6memory_MemoryPointer = __Pyx_ImportType_3_2_5(__pyx_t_1, "cupy.cuda.memory", "MemoryPointer",
  #if defined(PYPY_VERSION_NUM) && PYPY_VERSION_NUM < 0x050B0000
  sizeof(struct __pyx_obj_4cupy_4cuda_6memory_MemoryPointer), __PYX_GET_STRUCT_ALIGNMENT_3_2_5(struct __pyx_obj_4cupy_4cuda_6memory_MemoryPointer),
  #elif CYTHON_COMPILING_IN_LIMITED_API
  sizeof(struct __pyx_obj_4cupy_4cuda_6memory_MemoryPointer), __PYX_GET_STRUCT_ALIGNMENT_3_2_5(struct __pyx_obj_4cupy_4cuda_6memory_MemoryPointer),
  #else
  sizeof(struct __pyx_obj_4cupy_4cuda_6memory_MemoryPointer), __PYX_GET_STRUCT_ALIGNMENT_3_2_5(struct __pyx_obj_4cupy_4cuda_6memory_MemoryPointer),
  #endif
  __Pyx_ImportType_CheckSize_Warn_3_2_5); if (!__pyx_mstate->__pyx_ptype_4cupy_4cuda_6memory_MemoryPointer) __PYX_ERR(2, 29, __pyx_L1_error)
  __pyx_vtabptr_4cupy_4cuda_6memory_MemoryPointer = (struct __pyx_vtabstruct_4cupy_4cuda_6memory_MemoryPointer*)__Pyx_GetVtable(__pyx_mstate->__pyx_ptype_4cupy_4cuda_6memory_MemoryPointer); if (unlikely(!__pyx_vtabptr_4cupy_4cuda_6memory_MemoryPointer)) __PYX_ERR(2, 29, __pyx_L1_error)
  __pyx_mstate->__pyx_ptype_4cupy_4cuda_6memory_MemoryPool = __Pyx_ImportType_3_2_5(__pyx_t_1, "cupy.cuda.memory", "MemoryPool",
  #if defined(PYPY_VERSION_NUM) && PYPY_VERSION_NUM < 0x050B0000
  sizeof(struct __pyx_obj_4cupy_4cuda_6memory_MemoryPool), __PYX_GET_STRUCT_ALIGNMENT_3_2_5(struct __pyx_obj_4cupy_4cuda_6memory_MemoryPool),
  #elif CYTHON_COMPILING_IN_LIMITED_API
  sizeof(struct __pyx_obj_4cupy_4cuda_6memory_MemoryPool), __PYX_GET_STRUCT_ALIGNMENT_3_2_5(struct __pyx_obj_4cupy_4cuda_6memory_MemoryPool),
  #else
  sizeof(struct __pyx_obj_4cupy_4cuda_6memory_MemoryPool), __PYX_GET_STRUCT_ALIGNMENT_3_2_5(struct __pyx_obj_4cupy_4cuda_6memory_MemoryPool),
  #endif
  __Pyx_ImportType_CheckSize_Warn_3_2_5); if (!__pyx_mstate->__pyx_ptype_4cupy_4cuda_6memory_MemoryPool) __PYX_ERR(2, 58, __pyx_L1_error)
  __pyx_vtabptr_4cupy_4cuda_6memory_MemoryPool = (struct __pyx_vtabstruct_4cupy_4cuda_6memory_MemoryPool*)__Pyx_GetVtable(__pyx_mstate->__pyx_ptype_4cupy_4cuda_6memory_MemoryPool); if (unlikely(!__pyx_vtabptr_4cupy_4cuda_6memory_MemoryPool)) __PYX_ERR(2, 58, __pyx_L1_error)
  __pyx_mstate->__pyx_ptype_4cupy_4cuda_6memory_CFunctionAllocatorMemory = __Pyx_ImportType_3_2_5(__pyx_t_1, "cupy.cuda.memory", "CFunctionAllocatorMemory",
  #if defined(PYPY_VERSION_NUM) && PYPY_VERSION_NUM < 0x050B0000
  sizeof(struct __pyx_obj_4cupy_4cuda_6memory_CFunctionAllocatorMemory), __PYX_GET_STRUCT_ALIGNMENT_3_2_5(struct __pyx_obj_4cupy_4cuda_6memory_CFunctionAllocatorMemory),
  #elif CYTHON_COMPILING_IN_LIMITED_API
  sizeof(struct __pyx_obj_4cupy_4cuda_6memory_CFunctionAllocatorMemory), __PYX_GET_STRUCT_ALIGNMENT_3_2_5(struct __pyx_obj_4cupy_4cuda_6memory_CFunctionAllocatorMemory),
  #else
  sizeof(struct __pyx_obj_4cupy_4cuda_6memory_CFunctionAllocatorMemory), __PYX_GET_STRUCT_ALIGNMENT_3_2_5(struct __pyx_obj_4cupy_4cuda_6memory_CFunctionAllocatorMemory),
  #endif
  __Pyx_ImportType_CheckSize_Warn_3_2_5); if (!__pyx_mstate->__pyx_ptype_4cupy_4cuda_6memory_CFunctionAllocatorMemory) __PYX_ERR(2, 89, __pyx_L1_error)
  __pyx_mstate->__pyx_ptype_4cupy_4cuda_6memory_CFunctionAllocator = __Pyx_ImportType_3_2_5(__pyx_t_1, "cupy.cuda.memory", "CFunctionAllocator",
  #if defined(PYPY_VERSION_NUM) && PYPY_VERSION_NUM < 0x050B0000
  sizeof(struct __pyx_obj_4cupy_4cuda_6memory_CFunctionAllocator), __PYX_GET_STRUCT_ALIGNMENT_3_2_5(struct __pyx_obj_4cupy_4cuda_6memory_CFunctionAllocator),
  #elif CYTHON_COMPILING_IN_LIMITED_API
  sizeof(struct __pyx_obj_4cupy_4cuda_6memory_CFunctionAllocator), __PYX_GET_STRUCT_ALIGNMENT_3_2_5(struct __pyx_obj_4cupy_4cuda_6memory_CFunctionAllocator),
  #else
  sizeof(struct __pyx_obj_4cupy_4cuda_6memory_CFunctionAllocator), __PYX_GET_STRUCT_ALIGNMENT_3_2_5(struct __pyx_obj_4cupy_4cuda_6memory_CFunctionAllocator),
  #endif
  __Pyx_ImportType_CheckSize_Warn_3_2_5); if (!__pyx_mstate->__pyx_ptype_4cupy_4cuda_6memory_CFunctionAllocator) __PYX_ERR(2, 97, __pyx_L1_error)
  __pyx_vtabptr_4cupy_4cuda_6memory_CFunctionAllocator = (struct __pyx_vtabstruct_4cupy_4cuda_6memory_CFunctionAllocator*)__Pyx_GetVtable(__pyx_mstate->__pyx_ptype_4cupy_4cuda_6memory_CFunctionAllocator); if (unlikely(!__pyx_vtabptr_4cupy_4cuda_6memory_CFunctionAllocator)) __PYX_ERR(2, 97, __pyx_L1_error)
  __pyx_mstate->__pyx_ptype_4cupy_4cuda_6memory_PythonFunctionAllocatorMemory = __Pyx_ImportType_3_2_5(__pyx_t_1, "cupy.cuda.memory", "PythonFunctionAllocatorMemory",
  #if defined(PYPY_VERSION_NUM) && PYPY_VERSION_NUM < 0x050B0000
  sizeof(struct __pyx_obj_4cupy_4cuda_6memory_PythonFunctionAllocatorMemory), __PYX_GET_STRUCT_ALIGNMENT_3_2_5(struct __pyx_obj_4cupy_4cuda_6memory_PythonFunctionAllocatorMemory),
  #elif CYTHON_COMPILING_IN_LIMITED_API
  sizeof(struct __pyx_obj_4cupy_4cuda_6memory_PythonFunctionAllocatorMemory), __PYX_GET_STRUCT_ALIGNMENT_3_2_5(struct __pyx_obj_4cupy_4cuda_6memory_PythonFunctionAllocatorMemory),
  #else
  sizeof(struct __pyx_obj_4cupy_4cuda_6memory_PythonFunctionAllocatorMemory), __PYX_GET_STRUCT_ALIGNMENT_3_2_5(struct __pyx_obj_4cupy_4cuda_6memory_PythonFunctionAllocatorMemory),
  #endif
  __Pyx_ImportType_CheckSize_Warn_3_2_5); if (!__pyx_mstate->__pyx_ptype_4cupy_4cuda_6memory_PythonFunctionAllocatorMemory) __PYX_ERR(2, 108, __pyx_L1_error)
  __pyx_mstate->__pyx_ptype_4cupy_4cuda_6memory_PythonFunctionAllocator = __Pyx_ImportType_3_2_5(__pyx_t_1, "cupy.cuda.memory", "PythonFunctionAllocator",
  #if defined(PYPY_VERSION_NUM) && PYPY_VERSION_NUM < 0x050B0000
  sizeof(struct __pyx_obj_4cupy_4cuda_6memory_PythonFunctionAllocator), __PYX_GET_STRUCT_ALIGNMENT_3_2_5(struct __pyx_obj_4cupy_4cuda_6memory_PythonFunctionAllocator),
  #elif CYTHON_COMPILING_IN_LIMITED_API
  sizeof(struct __pyx_obj_4cupy_4cuda_6memory_PythonFunctionAllocator), __PYX_GET_STRUCT_ALIGNMENT_3_2_5(struct __pyx_obj_4cupy_4cuda_6memory_PythonFunctionAllocator),
  #else
  sizeof(struct __pyx_obj_4cupy_4cuda_6memory_PythonFunctionAllocator), __PYX_GET_STRUCT_ALIGNMENT_3_2_5(struct __pyx_obj_4cupy_4cuda_6memory_PythonFunctionAllocator),
  #endif
  __Pyx_ImportType_CheckSize_Warn_3_2_5); if (!__pyx_mstate->__pyx_ptype_4cupy_4cuda_6memory_PythonFunctionAllocator) __PYX_ERR(2, 114, __pyx_L1_error)
  __pyx_vtabptr_4cupy_4cuda_6memory_PythonFunctionAllocator = (struct __pyx_vtabstruct_4cupy_4cuda_6memory_PythonFunctionAllocator*)__Pyx_GetVtable(__pyx_mstate->__pyx_ptype_4cupy_4cuda_6memory_PythonFunctionAllocator); if (unlikely(!__pyx_vtabptr_4cupy_4cuda_6memory_PythonFunctionAllocator)) __PYX_ERR(2, 114, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_1 = PyImport_ImportModule("cupy.cuda.function"); if (unlikely(!__pyx_t_1)) __PYX_ERR(6, 4, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_mstate->__pyx_ptype_4cupy_4cuda_8function_CPointer = __Pyx_ImportType_3_2_5(__pyx_t_1, "cupy.cuda.function", "CPointer",
  #if defined(PYPY_VERSION_NUM) && PYPY_VERSION_NUM < 0x050B0000
  sizeof(struct __pyx_obj_4cupy_4cuda_8function_CPointer), __PYX_GET_STRUCT_ALIGNMENT_3_2_5(struct __pyx_obj_4cupy_4cuda_8function_CPointer),
  #elif CYTHON_COMPILING_IN_LIMITED_API
  sizeof(struct __pyx_obj_4cupy_4cuda_8function_CPointer), __PYX_GET_STRUCT_ALIGNMENT_3_2_5(struct __pyx_obj_4cupy_4cuda_8function_CPointer),
  #else
  sizeof(struct __pyx_obj_4cupy_4cuda_8function_CPointer), __PYX_GET_STRUCT_ALIGNMENT_3_2_5(struct __pyx_obj_4cupy_4cuda_8function_CPointer),
  #endif
  __Pyx_ImportType_CheckSize_Warn_3_2_5); if (!__pyx_mstate->__pyx_ptype_4cupy_4cuda_8function_CPointer) __PYX_ERR(6, 4, __pyx_L1_error)
  __pyx_mstate->__pyx_ptype_4cupy_4cuda_8function_Function = __Pyx_ImportType_3_2_5(__pyx_t_1, "cupy.cuda.function", "Function",
  #if defined(PYPY_VERSION_NUM) && PYPY_VERSION_NUM < 0x050B0000
  sizeof(struct __pyx_obj_4cupy_4cuda_8function_Function), __PYX_GET_STRUCT_ALIGNMENT_3_2_5(struct __pyx_obj_4cupy_4cuda_8function_Function),
  #elif CYTHON_COMPILING_IN_LIMITED_API
  sizeof(struct __pyx_obj_4cupy_4cuda_8function_Function), __PYX_GET_STRUCT_ALIGNMENT_3_2_5(struct __pyx_obj_4cupy_4cuda_8function_Function),
  #else
  sizeof(struct __pyx_obj_4cupy_4cuda_8function_Function), __PYX_GET_STRUCT_ALIGNMENT_3_2_5(struct __pyx_obj_4cupy_4cuda_8function_Function),
  #endif
  __Pyx_ImportType_CheckSize_Warn_3_2_5); if (!__pyx_mstate->__pyx_ptype_4cupy_4cuda_8function_Function) __PYX_ERR(6, 9, __pyx_L1_error)
  __pyx_vtabptr_4cupy_4cuda_8function_Function = (struct __pyx_vtabstruct_4cupy_4cuda_8function_Function*)__Pyx_GetVtable(__pyx_mstate->__pyx_ptype_4cupy_4cuda_8function_Function); if (unlikely(!__pyx_vtabptr_4cupy_4cuda_8function_Function)) __PYX_ERR(6, 9, __pyx_L1_error)
  __pyx_mstate->__pyx_ptype_4cupy_4cuda_8function_Module = __Pyx_ImportType_3_2_5(__pyx_t_1, "cupy.cuda.function", "Module",
  #if defined(PYPY_VERSION_NUM) && PYPY_VERSION_NUM < 0x050B0000
  sizeof(struct __pyx_obj_4cupy_4cuda_8function_Module), __PYX_GET_STRUCT_ALIGNMENT_3_2_5(struct __pyx_obj_4cupy_4cuda_8function_Module),
  #elif CYTHON_COMPILING_IN_LIMITED_API
  sizeof(struct __pyx_obj_4cupy_4cuda_8function_Module), __PYX_GET_STRUCT_ALIGNMENT_3_2_5(struct __pyx_obj_4cupy_4cuda_8function_Module),
  #else
  sizeof(struct __pyx_obj_4cupy_4cuda_8function_Module), __PYX_GET_STRUCT_ALIGNMENT_3_2_5(struct __pyx_obj_4cupy_4cuda_8function_Module),
  #endif
  __Pyx_ImportType_CheckSize_Warn_3_2_5); if (!__pyx_mstate->__pyx_ptype_4cupy_4cuda_8function_Module) __PYX_ERR(6, 20, __pyx_L1_error)
  __pyx_vtabptr_4cupy_4cuda_8function_Module = (struct __pyx_vtabstruct_4cupy_4cuda_8function_Module*)__Pyx_GetVtable(__pyx_mstate->__pyx_ptype_4cupy_4cuda_8function_Module); if (unlikely(!__pyx_vtabptr_4cupy_4cuda_8function_Module)) __PYX_ERR(6, 20, __pyx_L1_error)
  __pyx_mstate->__pyx_ptype_4cupy_4cuda_8function_LinkState = __Pyx_ImportType_3_2_5(__pyx_t_1, "cupy.cuda.function", "LinkState",
  #if defined(PYPY_VERSION_NUM) && PYPY_VERSION_NUM < 0x050B0000
  sizeof(struct __pyx_obj_4cupy_4cuda_8function_LinkState), __PYX_GET_STRUCT_ALIGNMENT_3_2_5(struct __pyx_obj_4cupy_4cuda_8function_LinkState),
  #elif CYTHON_COMPILING_IN_LIMITED_API
  sizeof(struct __pyx_obj_4cupy_4cuda_8function_LinkState), __PYX_GET_STRUCT_ALIGNMENT_3_2_5(struct __pyx_obj_4cupy_4cuda_8function_LinkState),
  #else
  sizeof(struct __pyx_obj_4cupy_4cuda_8function_LinkState), __PYX_GET_STRUCT_ALIGNMENT_3_2_5(struct __pyx_obj_4cupy_4cuda_8function_LinkState),
  #endif
  __Pyx_ImportType_CheckSize_Warn_3_2_5); if (!__pyx_mstate->__pyx_ptype_4cupy_4cuda_8function_LinkState) __PYX_ERR(6, 34, __pyx_L1_error)
  __pyx_vtabptr_4cupy_4cuda_8function_LinkState = (struct __pyx_vtabstruct_4cupy_4cuda_8function_LinkState*)__Pyx_GetVtable(__pyx_mstate->__pyx_ptype_4cupy_4cuda_8function_LinkState); if (unlikely(!__pyx_vtabptr_4cupy_4cuda_8function_LinkState)) __PYX_ERR(6, 34, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_1 = PyImport_ImportModule("cupy._core._carray"); if (unlikely(!__pyx_t_1)) __PYX_ERR(7, 22, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_mstate->__pyx_ptype_4cupy_5_core_7_carray_mdspan = __Pyx_ImportType_3_2_5(__pyx_t_1, "cupy._core._carray", "mdspan",
  #if defined(PYPY_VERSION_NUM) && PYPY_VERSION_NUM < 0x050B0000
  sizeof(struct __pyx_obj_4cupy_5_core_7_carray_mdspan), __PYX_GET_STRUCT_ALIGNMENT_3_2_5(struct __pyx_obj_4cupy_5_core_7_carray_mdspan),
  #elif CYTHON_COMPILING_IN_LIMITED_API
  sizeof(struct __pyx_obj_4cupy_5_core_7_carray_mdspan), __PYX_GET_STRUCT_ALIGNMENT_3_2_5(struct __pyx_obj_4cupy_5_core_7_carray_mdspan),
  #else
  sizeof(struct __pyx_obj_4cupy_5_core_7_carray_mdspan), __PYX_GET_STRUCT_ALIGNMENT_3_2_5(struct __pyx_obj_4cupy_5_core_7_carray_mdspan),
  #endif
  __Pyx_ImportType_CheckSize_Warn_3_2_5); if (!__pyx_mstate->__pyx_ptype_4cupy_5_core_7_carray_mdspan) __PYX_ERR(7, 22, __pyx_L1_error)
  __pyx_vtabptr_4cupy_5_core_7_carray_mdspan = (struct __pyx_vtabstruct_4cupy_5_core_7_carray_mdspan*)__Pyx_GetVtable(__pyx_mstate->__pyx_ptype_4cupy_5_core_7_carray_mdspan); if (unlikely(!__pyx_vtabptr_4cupy_5_core_7_carray_mdspan)) __PYX_ERR(7, 22, __pyx_L1_error)
  __pyx_mstate->__pyx_ptype_4cupy_5_core_7_carray_CArray = __Pyx_ImportType_3_2_5(__pyx_t_1, "cupy._core._carray", "CArray",
  #if defined(PYPY_VERSION_NUM) && PYPY_VERSION_NUM < 0x050B0000
  sizeof(struct __pyx_obj_4cupy_5_core_7_carray_CArray), __PYX_GET_STRUCT_ALIGNMENT_3_2_5(struct __pyx_obj_4cupy_5_core_7_carray_CArray),
  #elif CYTHON_COMPILING_IN_LIMITED_API
  sizeof(struct __pyx_obj_4cupy_5_core_7_carray_CArray), __PYX_GET_STRUCT_ALIGNMENT_3_2_5(struct __pyx_obj_4cupy_5_core_7_carray_CArray),
  #else
  sizeof(struct __pyx_obj_4cupy_5_core_7_carray_CArray), __PYX_GET_STRUCT_ALIGNMENT_3_2_5(struct __pyx_obj_4cupy_5_core_7_carray_CArray),
  #endif
  __Pyx_ImportType_CheckSize_Warn_3_2_5); if (!__pyx_mstate->__pyx_ptype_4cupy_5_core_7_carray_CArray) __PYX_ERR(7, 31, __pyx_L1_error)
  __pyx_vtabptr_4cupy_5_core_7_carray_CArray = (struct __pyx_vtabstruct_4cupy_5_core_7_carray_CArray*)__Pyx_GetVtable(__pyx_mstate->__pyx_ptype_4cupy_5_core_7_carray_CArray); if (unlikely(!__pyx_vtabptr_4cupy_5_core_7_carray_CArray)) __PYX_ERR(7, 31, __pyx_L1_error)
  __pyx_mstate->__pyx_ptype_4cupy_5_core_7_carray_CIndexer = __Pyx_ImportType_3_2_5(__pyx_t_1, "cupy._core._carray", "CIndexer",
  #if defined(PYPY_VERSION_NUM) && PYPY_VERSION_NUM < 0x050B0000
  sizeof(struct __pyx_obj_4cupy_5_core_7_carray_CIndexer), __PYX_GET_STRUCT_ALIGNMENT_3_2_5(struct __pyx_obj_4cupy_5_core_7_carray_CIndexer),
  #elif CYTHON_COMPILING_IN_LIMITED_API
  sizeof(struct __pyx_obj_4cupy_5_core_7_carray_CIndexer), __PYX_GET_STRUCT_ALIGNMENT_3_2_5(struct __pyx_obj_4cupy_5_core_7_carray_CIndexer),
  #else
  sizeof(struct __pyx_obj_4cupy_5_core_7_carray_CIndexer), __PYX_GET_STRUCT_ALIGNMENT_3_2_5(struct __pyx_obj_4cupy_5_core_7_carray_CIndexer),
  #endif
  __Pyx_ImportType_CheckSize_Warn_3_2_5); if (!__pyx_mstate->__pyx_ptype_4cupy_5_core_7_carray_CIndexer) __PYX_ERR(7, 44, __pyx_L1_error)
  __pyx_vtabptr_4cupy_5_core_7_carray_CIndexer = (struct __pyx_vtabstruct_4cupy_5_core_7_carray_CIndexer*)__Pyx_GetVtable(__pyx_mstate->__pyx_ptype_4cupy_5_core_7_carray_CIndexer); if (unlikely(!__pyx_vtabptr_4cupy_5_core_7_carray_CIndexer)) __PYX_ERR(7, 44, __pyx_L1_error)
  __pyx_mstate->__pyx_ptype_4cupy_5_core_7_carray_Indexer = __Pyx_ImportType_3_2_5(__pyx_t_1, "cupy._core._carray", "Indexer",
  #if defined(PYPY_VERSION_NUM) && PYPY_VERSION_NUM < 0x050B0000
  sizeof(struct __pyx_obj_4cupy_5_core_7_carray_Indexer), __PYX_GET_STRUCT_ALIGNMENT_3_2_5(struct __pyx_obj_4cupy_5_core_7_carray_Indexer),
  #elif CYTHON_COMPILING_IN_LIMITED_API
  sizeof(struct __pyx_obj_4cupy_5_core_7_carray_Indexer), __PYX_GET_STRUCT_ALIGNMENT_3_2_5(struct __pyx_obj_4cupy_5_core_7_carray_Indexer),
  #else
  sizeof(struct __pyx_obj_4cupy_5_core_7_carray_Indexer), __PYX_GET_STRUCT_ALIGNMENT_3_2_5(struct __pyx_obj_4cupy_5_core_7_carray_Indexer),
  #endif
  __Pyx_ImportType_CheckSize_Warn_3_2_5); if (!__pyx_mstate->__pyx_ptype_4cupy_5_core_7_carray_Indexer) __PYX_ERR(7, 49, __pyx_L1_error)
  __pyx_vtabptr_4cupy_5_core_7_carray_Indexer = (struct __pyx_vtabstruct_4cupy_5_core_7_carray_Indexer*)__Pyx_GetVtable(__pyx_mstate->__pyx_ptype_4cupy_5_core_7_carray_Indexer); if (unlikely(!__pyx_vtabptr_4cupy_5_core_7_carray_Indexer)) __PYX_ERR(7, 49, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_1 = PyImport_ImportModule("cupy._core.core"); if (unlikely(!__pyx_t_1)) __PYX_ERR(3, 10, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_mstate->__pyx_ptype_4cupy_5_core_4core__ndarray_base = __Pyx_ImportType_3_2_5(__pyx_t_1, "cupy._core.core", "_ndarray_base",
  #if defined(PYPY_VERSION_NUM) && PYPY_VERSION_NUM < 0x050B0000
  sizeof(struct __pyx_obj_4cupy_5_core_4core__ndarray_base), __PYX_GET_STRUCT_ALIGNMENT_3_2_5(struct __pyx_obj_4cupy_5_core_4core__ndarray_base),
  #elif CYTHON_COMPILING_IN_LIMITED_API
  sizeof(struct __pyx_obj_4cupy_5_core_4core__ndarray_base), __PYX_GET_STRUCT_ALIGNMENT_3_2_5(struct __pyx_obj_4cupy_5_core_4core__ndarray_base),
  #else
  sizeof(struct __pyx_obj_4cupy_5_core_4core__ndarray_base), __PYX_GET_STRUCT_ALIGNMENT_3_2_5(struct __pyx_obj_4cupy_5_core_4core__ndarray_base),
  #endif
  __Pyx_ImportType_CheckSize_Warn_3_2_5); if (!__pyx_mstate->__pyx_ptype_4cupy_5_core_4core__ndarray_base) __PYX_ERR(3, 10, __pyx_L1_error)
  __pyx_vtabptr_4cupy_5_core_4core__ndarray_base = (struct __pyx_vtabstruct_4cupy_5_core_4core__ndarray_base*)__Pyx_GetVtable(__pyx_mstate->__pyx_ptype_4cupy_5_core_4core__ndarray_base); if (unlikely(!__pyx_vtabptr_4cupy_5_core_4core__ndarray_base)) __PYX_ERR(3, 10, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_RefNannyFinishContext();
  return 0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_RefNannyFinishContext();
  return -1;
}

static int __Pyx_modinit_variable_import_code(__pyx_mstatetype *__pyx_mstate) {
  __Pyx_RefNannyDeclarations
  CYTHON_UNUSED_VAR(__pyx_mstate);
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__Pyx_modinit_variable_import_code", 0);
  /*--- Variable import code ---*/
  {
    __pyx_t_1 = PyImport_ImportModule("cupy_backends.cuda.api.runtime"); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    const char * __pyx_import_signature = __Pyx_PyBytes_AsString(__pyx_mstate_global->__pyx_kp_b_int__is_hip_environment_deviceAt);
    #if !CYTHON_ASSUME_SAFE_MACROS
    if (unlikely(!__pyx_import_signature)) __PYX_ERR(0, 1, __pyx_L1_error)
    #endif
    const char * __pyx_import_name = __pyx_import_signature + 11;
    void **const __pyx_import_pointers[] = {(void **)&__pyx_vp_13cupy_backends_4cuda_3api_7runtime__is_hip_environment, (void **)&__pyx_vp_13cupy_backends_4cuda_3api_7runtime_deviceAttributeComputeCapabilityMajor, (void **)&__pyx_vp_13cupy_backends_4cuda_3api_7runtime_deviceAttributeComputeCapabilityMinor, (void **)&__pyx_vp_13cupy_backends_4cuda_3api_7runtime_errorContextIsDestroyed, (void **)&__pyx_vp_13cupy_backends_4cuda_3api_7runtime_errorInvalidResourceHandle, (void **)&__pyx_vp_13cupy_backends_4cuda_3api_7runtime_errorInvalidValue, (void **)&__pyx_vp_13cupy_backends_4cuda_3api_7runtime_errorMemoryAllocation, (void **)&__pyx_vp_13cupy_backends_4cuda_3api_7runtime_errorPeerAccessAlreadyEnabled, (void **) NULL};
    void **const *__pyx_import_pointer = __pyx_import_pointers;
    const char *__pyx_import_current_signature = __pyx_import_signature;
    while (*__pyx_import_pointer) {
      if (__Pyx_ImportVoidPtr_3_2_5(__pyx_t_1, __pyx_import_name, *__pyx_import_pointer, __pyx_import_current_signature) < (0)) __PYX_ERR(0, 1, __pyx_L1_error)
      ++__pyx_import_pointer;
      __pyx_import_name = strchr(__pyx_import_name, '\0') + 1;
      __pyx_import_signature = strchr(__pyx_import_signature, '\0') + 1;
      if (*__pyx_import_signature != '\0') __pyx_import_current_signature = __pyx_import_signature;
    }
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  }
  __Pyx_RefNannyFinishContext();
  return 0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_RefNannyFinishContext();
  return -1;
}

static int __Pyx_modinit_function_import_code(__pyx_mstatetype *__pyx_mstate) {
  __Pyx_RefNannyDeclarations
  CYTHON_UNUSED_VAR(__pyx_mstate);
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__Pyx_modinit_function_import_code", 0);
  /*--- Function import code ---*/
  {
    __pyx_t_1 = PyImport_ImportModule("cupy_backends.cuda.libs.cusolver"); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    const char * __pyx_import_signature = __Pyx_PyBytes_AsString(__pyx_mstate_global->__pyx_kp_b_int_int___pyx_skip_dispatch_int);
    #if !CYTHON_ASSUME_SAFE_MACROS
    if (unlikely(!__pyx_import_signature)) __PYX_ERR(0, 1, __pyx_L1_error)
    #endif
    const char * __pyx_import_name = __pyx_import_signature + 228;
    void (**const __pyx_import_pointers[])(void) = {(void (**)(void))&__pyx_f_13cupy_backends_4cuda_4libs_8cusolver__getVersionNumber, (void (**)(void))&__pyx_f_13cupy_backends_4cuda_4libs_8cusolver_cgesvd_bufferSize, (void (**)(void))&__pyx_f_13cupy_backends_4cuda_4libs_8cusolver_dgesvd_bufferSize, (void (**)(void))&__pyx_f_13cupy_backends_4cuda_4libs_8cusolver_sgesvd_bufferSize, (void (**)(void))&__pyx_f_13cupy_backends_4cuda_4libs_8cusolver_zgesvd_bufferSize, (void (**)(void))&__pyx_f_13cupy_backends_4cuda_4libs_8cusolver_cungqr_bufferSize, (void (**)(void))&__pyx_f_13cupy_backends_4cuda_4libs_8cusolver_dorgqr_bufferSize, (void (**)(void))&__pyx_f_13cupy_backends_4cuda_4libs_8cusolver_sorgqr_bufferSize, (void (**)(void))&__pyx_f_13cupy_backends_4cuda_4libs_8cusolver_zungqr_bufferSize, (void (**)(void))&__pyx_f_13cupy_backends_4cuda_4libs_8cusolver_cgeqrf_bufferSize, (void (**)(void))&__pyx_f_13cupy_backends_4cuda_4libs_8cusolver_dgeqrf_bufferSize, (void (**)(void))&__pyx_f_13cupy_backends_4cuda_4libs_8cusolver_sgeqrf_bufferSize, (void (**)(void))&__pyx_f_13cupy_backends_4cuda_4libs_8cusolver_zgeqrf_bufferSize, (void (**)(void)) NULL};
    void (**const *__pyx_import_pointer)(void) = __pyx_import_pointers;
    const char *__pyx_import_current_signature = __pyx_import_signature;
    while (*__pyx_import_pointer) {
      if (__Pyx_ImportFunction_3_2_5(__pyx_t_1, __pyx_import_name, *__pyx_import_pointer, __pyx_import_current_signature) < (0)) __PYX_ERR(0, 1, __pyx_L1_error)
      ++__pyx_import_pointer;
      __pyx_import_name = strchr(__pyx_import_name, '\0') + 1;
      __pyx_import_signature = strchr(__pyx_import_signature, '\0') + 1;
      if (*__pyx_import_signature != '\0') __pyx_import_current_signature = __pyx_import_signature;
    }
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  }
  {
    __pyx_t_1 = PyImport_ImportModule("cupy.cuda.device"); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    const char * __pyx_import_signature = __Pyx_PyBytes_AsString(__pyx_mstate_global->__pyx_kp_b_int_int___pyx_skip_dispatch_get);
    #if !CYTHON_ASSUME_SAFE_MACROS
    if (unlikely(!__pyx_import_signature)) __PYX_ERR(0, 1, __pyx_L1_error)
    #endif
    const char * __pyx_import_name = __pyx_import_signature + 30;
    void (**const __pyx_import_pointers[])(void) = {(void (**)(void))&__pyx_f_4cupy_4cuda_6device_get_device_id, (void (**)(void)) NULL};
    void (**const *__pyx_import_pointer)(void) = __pyx_import_pointers;
    const char *__pyx_import_current_signature = __pyx_import_signature;
    while (*__pyx_import_pointer) {
      if (__Pyx_ImportFunction_3_2_5(__pyx_t_1, __pyx_import_name, *__pyx_import_pointer, __pyx_import_current_signature) < (0)) __PYX_ERR(0, 1, __pyx_L1_error)
      ++__pyx_import_pointer;
      __pyx_import_name = strchr(__pyx_import_name, '\0') + 1;
      __pyx_import_signature = strchr(__pyx_import_signature, '\0') + 1;
      if (*__pyx_import_signature != '\0') __pyx_import_current_signature = __pyx_import_signature;
    }
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  }
  {
    __pyx_t_1 = PyImport_ImportModule("cupy.cuda.memory"); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    const char * __pyx_import_signature = __Pyx_PyBytes_AsString(__pyx_mstate_global->__pyx_kp_b_struct___pyx_obj_4cupy_4cuda_6me);
    #if !CYTHON_ASSUME_SAFE_MACROS
    if (unlikely(!__pyx_import_signature)) __PYX_ERR(0, 1, __pyx_L1_error)
    #endif
    const char * __pyx_import_name = __pyx_import_signature + 90;
    void (**const __pyx_import_pointers[])(void) = {(void (**)(void))&__pyx_f_4cupy_4cuda_6memory_alloc, (void (**)(void)) NULL};
    void (**const *__pyx_import_pointer)(void) = __pyx_import_pointers;
    const char *__pyx_import_current_signature = __pyx_import_signature;
    while (*__pyx_import_pointer) {
      if (__Pyx_ImportFunction_3_2_5(__pyx_t_1, __pyx_import_name, *__pyx_import_pointer, __pyx_import_current_signature) < (0)) __PYX_ERR(0, 1, __pyx_L1_error)
      ++__pyx_import_pointer;
      __pyx_import_name = strchr(__pyx_import_name, '\0') + 1;
      __pyx_import_signature = strchr(__pyx_import_signature, '\0') + 1;
      if (*__pyx_import_signature != '\0') __pyx_import_current_signature = __pyx_import_signature;
    }
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  }
  {
    __pyx_t_1 = PyImport_ImportModule("cupy._core.core"); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    const char * __pyx_import_signature = __Pyx_PyBytes_AsString(__pyx_mstate_global->__pyx_kp_b_struct___pyx_obj_4cupy_5_core_4c);
    #if !CYTHON_ASSUME_SAFE_MACROS
    if (unlikely(!__pyx_import_signature)) __PYX_ERR(0, 1, __pyx_L1_error)
    #endif
    const char * __pyx_import_name = __pyx_import_signature + 354;
    void (**const __pyx_import_pointers[])(void) = {(void (**)(void))&__pyx_f_4cupy_5_core_4core__ndarray_init, (void (**)(void))&__pyx_f_4cupy_5_core_4core__convert_object_with_cuda_array_interface, (void (**)(void))&__pyx_f_4cupy_5_core_4core__internal_ascontiguousarray, (void (**)(void)) NULL};
    void (**const *__pyx_import_pointer)(void) = __pyx_import_pointers;
    const char *__pyx_import_current_signature = __pyx_import_signature;
    while (*__pyx_import_pointer) {
      if (__Pyx_ImportFunction_3_2_5(__pyx_t_1, __pyx_import_name, *__pyx_import_pointer, __pyx_import_current_signature) < (0)) __PYX_ERR(0, 1, __pyx_L1_error)
      ++__pyx_import_pointer;
      __pyx_import_name = strchr(__pyx_import_name, '\0') + 1;
      __pyx_import_signature = strchr(__pyx_import_signature, '\0') + 1;
      if (*__pyx_import_signature != '\0') __pyx_import_current_signature = __pyx_import_signature;
    }
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  }
  __Pyx_RefNannyFinishContext();
  return 0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_RefNannyFinishContext();
  return -1;
}

#if CYTHON_PEP489_MULTI_PHASE_INIT
static PyObject* __pyx_pymod_create(PyObject *spec, PyModuleDef *def); /*proto*/
static int __pyx_pymod_exec_cusolver(PyObject* module); /*proto*/
static PyModuleDef_Slot __pyx_moduledef_slots[] = {
  {Py_mod_create, (void*)__pyx_pymod_create},
  {Py_mod_exec, (void*)__pyx_pymod_exec_cusolver},
  #if CYTHON_COMPILING_IN_CPYTHON_FREETHREADING
  {Py_mod_gil, __Pyx_FREETHREADING_COMPATIBLE},
  #endif
  #if PY_VERSION_HEX >= 0x030C0000 && CYTHON_USE_MODULE_STATE
  {Py_mod_multiple_interpreters, Py_MOD_MULTIPLE_INTERPRETERS_NOT_SUPPORTED},
  #endif
  {0, NULL}
};
#endif

#ifdef __cplusplus
namespace {
  struct PyModuleDef __pyx_moduledef =
  #else
  static struct PyModuleDef __pyx_moduledef =
  #endif
  {
      PyModuleDef_HEAD_INIT,
      "cusolver",
      0, /* m_doc */
    #if CYTHON_USE_MODULE_STATE
      sizeof(__pyx_mstatetype), /* m_size */
    #else
      (CYTHON_PEP489_MULTI_PHASE_INIT) ? 0 : -1, /* m_size */
    #endif
      __pyx_methods /* m_methods */,
    #if CYTHON_PEP489_MULTI_PHASE_INIT
      __pyx_moduledef_slots, /* m_slots */
    #else
      NULL, /* m_reload */
    #endif
    #if CYTHON_USE_MODULE_STATE
      __pyx_m_traverse, /* m_traverse */
      __pyx_m_clear, /* m_clear */
      NULL /* m_free */
    #else
      NULL, /* m_traverse */
      NULL, /* m_clear */
      NULL /* m_free */
    #endif
  };
  #ifdef __cplusplus
} /* anonymous namespace */
#endif

/* PyModInitFuncType */
#ifndef CYTHON_NO_PYINIT_EXPORT
  #define __Pyx_PyMODINIT_FUNC PyMODINIT_FUNC
#else
  #ifdef __cplusplus
  #define __Pyx_PyMODINIT_FUNC extern "C" PyObject *
  #else
  #define __Pyx_PyMODINIT_FUNC PyObject *
  #endif
#endif

__Pyx_PyMODINIT_FUNC PyInit_cusolver(void) CYTHON_SMALL_CODE; /*proto*/
__Pyx_PyMODINIT_FUNC PyInit_cusolver(void)
#if CYTHON_PEP489_MULTI_PHASE_INIT
{
  return PyModuleDef_Init(&__pyx_moduledef);
}
/* ModuleCreationPEP489 */
#if CYTHON_COMPILING_IN_LIMITED_API && (__PYX_LIMITED_VERSION_HEX < 0x03090000\
      || ((defined(_WIN32) || defined(WIN32) || defined(MS_WINDOWS)) && __PYX_LIMITED_VERSION_HEX < 0x030A0000))
static PY_INT64_T __Pyx_GetCurrentInterpreterId(void) {
    {
        PyObject *module = PyImport_ImportModule("_interpreters"); // 3.13+ I think
        if (!module) {
            PyErr_Clear(); // just try the 3.8-3.12 version
            module = PyImport_ImportModule("_xxsubinterpreters");
            if (!module) goto bad;
        }
        PyObject *current = PyObject_CallMethod(module, "get_current", NULL);
        Py_DECREF(module);
        if (!current) goto bad;
        if (PyTuple_Check(current)) {
            PyObject *new_current = PySequence_GetItem(current, 0);
            Py_DECREF(current);
            current = new_current;
            if (!new_current) goto bad;
        }
        long long as_c_int = PyLong_AsLongLong(current);
        Py_DECREF(current);
        return as_c_int;
    }
  bad:
    PySys_WriteStderr("__Pyx_GetCurrentInterpreterId failed. Try setting the C define CYTHON_PEP489_MULTI_PHASE_INIT=0\n");
    return -1;
}
#endif
#if !CYTHON_USE_MODULE_STATE
static CYTHON_SMALL_CODE int __Pyx_check_single_interpreter(void) {
    static PY_INT64_T main_interpreter_id = -1;
#if CYTHON_COMPILING_IN_GRAAL && defined(GRAALPY_VERSION_NUM) && GRAALPY_VERSION_NUM > 0x19000000
    PY_INT64_T current_id = GraalPyInterpreterState_GetIDFromThreadState(PyThreadState_Get());
#elif CYTHON_COMPILING_IN_GRAAL
    PY_INT64_T current_id = PyInterpreterState_GetIDFromThreadState(PyThreadState_Get());
#elif CYTHON_COMPILING_IN_LIMITED_API && (__PYX_LIMITED_VERSION_HEX < 0x03090000\
      || ((defined(_WIN32) || defined(WIN32) || defined(MS_WINDOWS)) && __PYX_LIMITED_VERSION_HEX < 0x030A0000))
    PY_INT64_T current_id = __Pyx_GetCurrentInterpreterId();
#elif CYTHON_COMPILING_IN_LIMITED_API
    PY_INT64_T current_id = PyInterpreterState_GetID(PyInterpreterState_Get());
#else
    PY_INT64_T current_id = PyInterpreterState_GetID(PyThreadState_Get()->interp);
#endif
    if (unlikely(current_id == -1)) {
        return -1;
    }
    if (main_interpreter_id == -1) {
        main_interpreter_id = current_id;
        return 0;
    } else if (unlikely(main_interpreter_id != current_id)) {
        PyErr_SetString(
            PyExc_ImportError,
            "Interpreter change detected - this module can only be loaded into one interpreter per process.");
        return -1;
    }
    return 0;
}
#endif
static CYTHON_SMALL_CODE int __Pyx_copy_spec_to_module(PyObject *spec, PyObject *moddict, const char* from_name, const char* to_name, int allow_none)
{
    PyObject *value = PyObject_GetAttrString(spec, from_name);
    int result = 0;
    if (likely(value)) {
        if (allow_none || value != Py_None) {
            result = PyDict_SetItemString(moddict, to_name, value);
        }
        Py_DECREF(value);
    } else if (PyErr_ExceptionMatches(PyExc_AttributeError)) {
        PyErr_Clear();
    } else {
        result = -1;
    }
    return result;
}
static CYTHON_SMALL_CODE PyObject* __pyx_pymod_create(PyObject *spec, PyModuleDef *def) {
    PyObject *module = NULL, *moddict, *modname;
    CYTHON_UNUSED_VAR(def);
    #if !CYTHON_USE_MODULE_STATE
    if (__Pyx_check_single_interpreter())
        return NULL;
    #endif
    if (__pyx_m)
        return __Pyx_NewRef(__pyx_m);
    modname = PyObject_GetAttrString(spec, "name");
    if (unlikely(!modname)) goto bad;
    module = PyModule_NewObject(modname);
    Py_DECREF(modname);
    if (unlikely(!module)) goto bad;
    moddict = PyModule_GetDict(module);
    if (unlikely(!moddict)) goto bad;
    if (unlikely(__Pyx_copy_spec_to_module(spec, moddict, "loader", "__loader__", 1) < 0)) goto bad;
    if (unlikely(__Pyx_copy_spec_to_module(spec, moddict, "origin", "__file__", 1) < 0)) goto bad;
    if (unlikely(__Pyx_copy_spec_to_module(spec, moddict, "parent", "__package__", 1) < 0)) goto bad;
    if (unlikely(__Pyx_copy_spec_to_module(spec, moddict, "submodule_search_locations", "__path__", 0) < 0)) goto bad;
    return module;
bad:
    Py_XDECREF(module);
    return NULL;
}


static CYTHON_SMALL_CODE int __pyx_pymod_exec_cusolver(PyObject *__pyx_pyinit_module)
#endif
{
  int stringtab_initialized = 0;
  #if CYTHON_USE_MODULE_STATE
  int pystate_addmodule_run = 0;
  #endif
  __pyx_mstatetype *__pyx_mstate = NULL;
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  Py_ssize_t __pyx_t_3;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  PyObject *__pyx_t_6 = NULL;
  PyObject *__pyx_t_7 = NULL;
  PyObject *__pyx_t_8 = NULL;
  size_t __pyx_t_9;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannyDeclarations
  #if CYTHON_PEP489_MULTI_PHASE_INIT
  if (__pyx_m) {
    if (__pyx_m == __pyx_pyinit_module) return 0;
    PyErr_SetString(PyExc_RuntimeError, "Module 'cusolver' has already been imported. Re-initialisation is not supported.");
    return -1;
  }
  #else
  if (__pyx_m) return __Pyx_NewRef(__pyx_m);
  #endif
  /*--- Module creation code ---*/
  #if CYTHON_PEP489_MULTI_PHASE_INIT
  __pyx_t_1 = __pyx_pyinit_module;
  Py_INCREF(__pyx_t_1);
  #else
  __pyx_t_1 = PyModule_Create(&__pyx_moduledef); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1, __pyx_L1_error)
  #endif
  #if CYTHON_USE_MODULE_STATE
  {
    int add_module_result = __Pyx_State_AddModule(__pyx_t_1, &__pyx_moduledef);
    __pyx_t_1 = 0; /* transfer ownership from __pyx_t_1 to "cusolver" pseudovariable */
    if (unlikely((add_module_result < 0))) __PYX_ERR(0, 1, __pyx_L1_error)
    pystate_addmodule_run = 1;
  }
  #else
  __pyx_m = __pyx_t_1;
  #endif
  #if CYTHON_COMPILING_IN_CPYTHON_FREETHREADING
  PyUnstable_Module_SetGIL(__pyx_m, Py_MOD_GIL_NOT_USED);
  #endif
  __pyx_mstate = __pyx_mstate_global;
  CYTHON_UNUSED_VAR(__pyx_t_1);
  __pyx_mstate->__pyx_d = PyModule_GetDict(__pyx_m); if (unlikely(!__pyx_mstate->__pyx_d)) __PYX_ERR(0, 1, __pyx_L1_error)
  Py_INCREF(__pyx_mstate->__pyx_d);
  __pyx_mstate->__pyx_b = __Pyx_PyImport_AddModuleRef(__Pyx_BUILTIN_MODULE_NAME); if (unlikely(!__pyx_mstate->__pyx_b)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_mstate->__pyx_cython_runtime = __Pyx_PyImport_AddModuleRef("cython_runtime"); if (unlikely(!__pyx_mstate->__pyx_cython_runtime)) __PYX_ERR(0, 1, __pyx_L1_error)
  if (PyObject_SetAttrString(__pyx_m, "__builtins__", __pyx_mstate->__pyx_b) < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  /* ImportRefnannyAPI */
  #if CYTHON_REFNANNY
  __Pyx_RefNanny = __Pyx_RefNannyImportAPI("refnanny");
  if (!__Pyx_RefNanny) {
    PyErr_Clear();
    __Pyx_RefNanny = __Pyx_RefNannyImportAPI("Cython.Runtime.refnanny");
    if (!__Pyx_RefNanny)
        Py_FatalError("failed to import 'refnanny' module");
  }
  #endif
  
__Pyx_RefNannySetupContext("PyInit_cusolver", 0);
  __Pyx_init_runtime_version();
  if (__Pyx_check_binary_version(__PYX_LIMITED_VERSION_HEX, __Pyx_get_runtime_version(), CYTHON_COMPILING_IN_LIMITED_API) < (0)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_mstate->__pyx_empty_tuple = PyTuple_New(0); if (unlikely(!__pyx_mstate->__pyx_empty_tuple)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_mstate->__pyx_empty_bytes = PyBytes_FromStringAndSize("", 0); if (unlikely(!__pyx_mstate->__pyx_empty_bytes)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_mstate->__pyx_empty_unicode = PyUnicode_FromStringAndSize("", 0); if (unlikely(!__pyx_mstate->__pyx_empty_unicode)) __PYX_ERR(0, 1, __pyx_L1_error)
  /*--- Library function declarations ---*/
  /*--- Initialize various global constants etc. ---*/
  if (__Pyx_InitConstants(__pyx_mstate) < (0)) __PYX_ERR(0, 1, __pyx_L1_error)
  stringtab_initialized = 1;
  if (__Pyx_InitGlobals() < (0)) __PYX_ERR(0, 1, __pyx_L1_error)
  if (__pyx_module_is_main_cupyx__cusolver) {
    if (PyObject_SetAttr(__pyx_m, __pyx_mstate_global->__pyx_n_u_name_2, __pyx_mstate_global->__pyx_n_u_main) < (0)) __PYX_ERR(0, 1, __pyx_L1_error)
  }
  {
    PyObject *modules = PyImport_GetModuleDict(); if (unlikely(!modules)) __PYX_ERR(0, 1, __pyx_L1_error)
    if (!PyDict_GetItemString(modules, "cupyx.cusolver")) {
      if (unlikely((PyDict_SetItemString(modules, "cupyx.cusolver", __pyx_m) < 0))) __PYX_ERR(0, 1, __pyx_L1_error)
    }
  }
  /*--- Builtin init code ---*/
  if (__Pyx_InitCachedBuiltins(__pyx_mstate) < (0)) __PYX_ERR(0, 1, __pyx_L1_error)
  /*--- Constants init code ---*/
  if (__Pyx_InitCachedConstants(__pyx_mstate) < (0)) __PYX_ERR(0, 1, __pyx_L1_error)
  if (__Pyx_CreateCodeObjects(__pyx_mstate) < (0)) __PYX_ERR(0, 1, __pyx_L1_error)
  /*--- Global type/function init code ---*/
  (void)__Pyx_modinit_global_init_code(__pyx_mstate);
  (void)__Pyx_modinit_variable_export_code(__pyx_mstate);
  (void)__Pyx_modinit_function_export_code(__pyx_mstate);
  (void)__Pyx_modinit_type_init_code(__pyx_mstate);
  if (unlikely((__Pyx_modinit_type_import_code(__pyx_mstate) < 0))) __PYX_ERR(0, 1, __pyx_L1_error)
  if (unlikely((__Pyx_modinit_variable_import_code(__pyx_mstate) < 0))) __PYX_ERR(0, 1, __pyx_L1_error)
  if (unlikely((__Pyx_modinit_function_import_code(__pyx_mstate) < 0))) __PYX_ERR(0, 1, __pyx_L1_error)
  /*--- Execution code ---*/

  /* "cupyx/cusolver.pyx":5
 * from libc.stdint cimport intptr_t
 * 
 * import warnings as _warnings             # <<<<<<<<<<<<<<
 * 
 * import numpy as _numpy
*/
  __pyx_t_1 = __Pyx_Import(__pyx_mstate_global->__pyx_n_u_warnings_2, 0, 0, NULL, 0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 5, __pyx_L1_error)
  __pyx_t_2 = __pyx_t_1;
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_mstate_global->__pyx_d, __pyx_mstate_global->__pyx_n_u_warnings, __pyx_t_2) < (0)) __PYX_ERR(0, 5, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "cupyx/cusolver.pyx":7
 * import warnings as _warnings
 * 
 * import numpy as _numpy             # <<<<<<<<<<<<<<
 * 
 * from cupy_backends.cuda.api cimport runtime
*/
  __pyx_t_1 = __Pyx_Import(__pyx_mstate_global->__pyx_n_u_numpy_2, 0, 0, NULL, 0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 7, __pyx_L1_error)
  __pyx_t_2 = __pyx_t_1;
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_mstate_global->__pyx_d, __pyx_mstate_global->__pyx_n_u_numpy, __pyx_t_2) < (0)) __PYX_ERR(0, 7, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "cupyx/cusolver.pyx":21
 * from cupy._core.core cimport _ndarray_init, _ndarray_base
 * 
 * import cupy as _cupy             # <<<<<<<<<<<<<<
 * from cupy_backends.cuda.api import runtime as _runtime
 * from cupy_backends.cuda.libs import cublas as _cublas
*/
  __pyx_t_1 = __Pyx_Import(__pyx_mstate_global->__pyx_n_u_cupy_2, 0, 0, NULL, 0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 21, __pyx_L1_error)
  __pyx_t_2 = __pyx_t_1;
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_mstate_global->__pyx_d, __pyx_mstate_global->__pyx_n_u_cupy, __pyx_t_2) < (0)) __PYX_ERR(0, 21, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "cupyx/cusolver.pyx":22
 * 
 * import cupy as _cupy
 * from cupy_backends.cuda.api import runtime as _runtime             # <<<<<<<<<<<<<<
 * from cupy_backends.cuda.libs import cublas as _cublas
 * from cupy_backends.cuda.libs import cusolver as _cusolver
*/
  {
    PyObject* const __pyx_imported_names[] = {__pyx_mstate_global->__pyx_n_u_runtime_2};
    __pyx_t_1 = __Pyx_Import(__pyx_mstate_global->__pyx_n_u_cupy_backends_cuda_api, __pyx_imported_names, 1, NULL, 0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 22, __pyx_L1_error)
  }
  __pyx_t_2 = __pyx_t_1;
  __Pyx_GOTREF(__pyx_t_2);
  {
    PyObject* const __pyx_imported_names[] = {__pyx_mstate_global->__pyx_n_u_runtime_2};
    __pyx_t_3 = 0; {
      __pyx_t_4 = __Pyx_ImportFrom(__pyx_t_2, __pyx_imported_names[__pyx_t_3]); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 22, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_4);
      switch (__pyx_t_3) {
        case 0:
        if (PyDict_SetItem(__pyx_mstate_global->__pyx_d, __pyx_mstate_global->__pyx_n_u_runtime, __pyx_t_4) < (0)) __PYX_ERR(0, 22, __pyx_L1_error)
        break;
        default:;
      }
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    }
  }
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "cupyx/cusolver.pyx":23
 * import cupy as _cupy
 * from cupy_backends.cuda.api import runtime as _runtime
 * from cupy_backends.cuda.libs import cublas as _cublas             # <<<<<<<<<<<<<<
 * from cupy_backends.cuda.libs import cusolver as _cusolver
 * from cupy.cuda import device as _device
*/
  {
    PyObject* const __pyx_imported_names[] = {__pyx_mstate_global->__pyx_n_u_cublas_2};
    __pyx_t_1 = __Pyx_Import(__pyx_mstate_global->__pyx_n_u_cupy_backends_cuda_libs, __pyx_imported_names, 1, NULL, 0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 23, __pyx_L1_error)
  }
  __pyx_t_2 = __pyx_t_1;
  __Pyx_GOTREF(__pyx_t_2);
  {
    PyObject* const __pyx_imported_names[] = {__pyx_mstate_global->__pyx_n_u_cublas_2};
    __pyx_t_3 = 0; {
      __pyx_t_4 = __Pyx_ImportFrom(__pyx_t_2, __pyx_imported_names[__pyx_t_3]); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 23, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_4);
      switch (__pyx_t_3) {
        case 0:
        if (PyDict_SetItem(__pyx_mstate_global->__pyx_d, __pyx_mstate_global->__pyx_n_u_cublas, __pyx_t_4) < (0)) __PYX_ERR(0, 23, __pyx_L1_error)
        break;
        default:;
      }
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    }
  }
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "cupyx/cusolver.pyx":24
 * from cupy_backends.cuda.api import runtime as _runtime
 * from cupy_backends.cuda.libs import cublas as _cublas
 * from cupy_backends.cuda.libs import cusolver as _cusolver             # <<<<<<<<<<<<<<
 * from cupy.cuda import device as _device
 * from cupy._core import _routines_linalg as _linalg
*/
  {
    PyObject* const __pyx_imported_names[] = {__pyx_mstate_global->__pyx_n_u_cusolver_2};
    __pyx_t_1 = __Pyx_Import(__pyx_mstate_global->__pyx_n_u_cupy_backends_cuda_libs, __pyx_imported_names, 1, NULL, 0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 24, __pyx_L1_error)
  }
  __pyx_t_2 = __pyx_t_1;
  __Pyx_GOTREF(__pyx_t_2);
  {
    PyObject* const __pyx_imported_names[] = {__pyx_mstate_global->__pyx_n_u_cusolver_2};
    __pyx_t_3 = 0; {
      __pyx_t_4 = __Pyx_ImportFrom(__pyx_t_2, __pyx_imported_names[__pyx_t_3]); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 24, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_4);
      switch (__pyx_t_3) {
        case 0:
        if (PyDict_SetItem(__pyx_mstate_global->__pyx_d, __pyx_mstate_global->__pyx_n_u_cusolver, __pyx_t_4) < (0)) __PYX_ERR(0, 24, __pyx_L1_error)
        break;
        default:;
      }
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    }
  }
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "cupyx/cusolver.pyx":25
 * from cupy_backends.cuda.libs import cublas as _cublas
 * from cupy_backends.cuda.libs import cusolver as _cusolver
 * from cupy.cuda import device as _device             # <<<<<<<<<<<<<<
 * from cupy._core import _routines_linalg as _linalg
 * from cupy import _util
*/
  {
    PyObject* const __pyx_imported_names[] = {__pyx_mstate_global->__pyx_n_u_device_2};
    __pyx_t_1 = __Pyx_Import(__pyx_mstate_global->__pyx_n_u_cupy_cuda, __pyx_imported_names, 1, NULL, 0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 25, __pyx_L1_error)
  }
  __pyx_t_2 = __pyx_t_1;
  __Pyx_GOTREF(__pyx_t_2);
  {
    PyObject* const __pyx_imported_names[] = {__pyx_mstate_global->__pyx_n_u_device_2};
    __pyx_t_3 = 0; {
      __pyx_t_4 = __Pyx_ImportFrom(__pyx_t_2, __pyx_imported_names[__pyx_t_3]); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 25, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_4);
      switch (__pyx_t_3) {
        case 0:
        if (PyDict_SetItem(__pyx_mstate_global->__pyx_d, __pyx_mstate_global->__pyx_n_u_device, __pyx_t_4) < (0)) __PYX_ERR(0, 25, __pyx_L1_error)
        break;
        default:;
      }
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    }
  }
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "cupyx/cusolver.pyx":26
 * from cupy_backends.cuda.libs import cusolver as _cusolver
 * from cupy.cuda import device as _device
 * from cupy._core import _routines_linalg as _linalg             # <<<<<<<<<<<<<<
 * from cupy import _util
 * 
*/
  {
    PyObject* const __pyx_imported_names[] = {__pyx_mstate_global->__pyx_n_u_routines_linalg};
    __pyx_t_1 = __Pyx_Import(__pyx_mstate_global->__pyx_n_u_cupy__core, __pyx_imported_names, 1, NULL, 0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 26, __pyx_L1_error)
  }
  __pyx_t_2 = __pyx_t_1;
  __Pyx_GOTREF(__pyx_t_2);
  {
    PyObject* const __pyx_imported_names[] = {__pyx_mstate_global->__pyx_n_u_routines_linalg};
    __pyx_t_3 = 0; {
      __pyx_t_4 = __Pyx_ImportFrom(__pyx_t_2, __pyx_imported_names[__pyx_t_3]); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 26, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_4);
      switch (__pyx_t_3) {
        case 0:
        if (PyDict_SetItem(__pyx_mstate_global->__pyx_d, __pyx_mstate_global->__pyx_n_u_linalg_2, __pyx_t_4) < (0)) __PYX_ERR(0, 26, __pyx_L1_error)
        break;
        default:;
      }
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    }
  }
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "cupyx/cusolver.pyx":27
 * from cupy.cuda import device as _device
 * from cupy._core import _routines_linalg as _linalg
 * from cupy import _util             # <<<<<<<<<<<<<<
 * 
 * import cupyx as _cupyx
*/
  {
    PyObject* const __pyx_imported_names[] = {__pyx_mstate_global->__pyx_n_u_util};
    __pyx_t_1 = __Pyx_Import(__pyx_mstate_global->__pyx_n_u_cupy_2, __pyx_imported_names, 1, NULL, 0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 27, __pyx_L1_error)
  }
  __pyx_t_2 = __pyx_t_1;
  __Pyx_GOTREF(__pyx_t_2);
  {
    PyObject* const __pyx_imported_names[] = {__pyx_mstate_global->__pyx_n_u_util};
    __pyx_t_3 = 0; {
      __pyx_t_4 = __Pyx_ImportFrom(__pyx_t_2, __pyx_imported_names[__pyx_t_3]); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 27, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_4);
      if (PyDict_SetItem(__pyx_mstate_global->__pyx_d, __pyx_imported_names[__pyx_t_3], __pyx_t_4) < (0)) __PYX_ERR(0, 27, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    }
  }
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "cupyx/cusolver.pyx":29
 * from cupy import _util
 * 
 * import cupyx as _cupyx             # <<<<<<<<<<<<<<
 * 
 * 
*/
  __pyx_t_1 = __Pyx_Import(__pyx_mstate_global->__pyx_n_u_cupyx_2, 0, 0, NULL, 0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 29, __pyx_L1_error)
  __pyx_t_2 = __pyx_t_1;
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_mstate_global->__pyx_d, __pyx_mstate_global->__pyx_n_u_cupyx, __pyx_t_2) < (0)) __PYX_ERR(0, 29, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "cupyx/cusolver.pyx":70
 * 
 * _available_cuda_version = {
 *     'gesvdj': (0, None),             # <<<<<<<<<<<<<<
 *     'gesvdjBatched': (0, None),
 *     'gesvda': (0, None),
*/
  __pyx_t_2 = __Pyx_PyDict_NewPresized(10); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 70, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_t_2, __pyx_mstate_global->__pyx_n_u_gesvdj, __pyx_mstate_global->__pyx_tuple[4]) < (0)) __PYX_ERR(0, 70, __pyx_L1_error)

  /* "cupyx/cusolver.pyx":71
 * _available_cuda_version = {
 *     'gesvdj': (0, None),
 *     'gesvdjBatched': (0, None),             # <<<<<<<<<<<<<<
 *     'gesvda': (0, None),
 *     'potrfBatched': (0, None),
*/
  if (PyDict_SetItem(__pyx_t_2, __pyx_mstate_global->__pyx_n_u_gesvdjBatched, __pyx_mstate_global->__pyx_tuple[4]) < (0)) __PYX_ERR(0, 70, __pyx_L1_error)

  /* "cupyx/cusolver.pyx":72
 *     'gesvdj': (0, None),
 *     'gesvdjBatched': (0, None),
 *     'gesvda': (0, None),             # <<<<<<<<<<<<<<
 *     'potrfBatched': (0, None),
 *     'potrsBatched': (0, None),
*/
  if (PyDict_SetItem(__pyx_t_2, __pyx_mstate_global->__pyx_n_u_gesvda, __pyx_mstate_global->__pyx_tuple[4]) < (0)) __PYX_ERR(0, 70, __pyx_L1_error)

  /* "cupyx/cusolver.pyx":73
 *     'gesvdjBatched': (0, None),
 *     'gesvda': (0, None),
 *     'potrfBatched': (0, None),             # <<<<<<<<<<<<<<
 *     'potrsBatched': (0, None),
 *     'syevj': (0, None),
*/
  if (PyDict_SetItem(__pyx_t_2, __pyx_mstate_global->__pyx_n_u_potrfBatched, __pyx_mstate_global->__pyx_tuple[4]) < (0)) __PYX_ERR(0, 70, __pyx_L1_error)

  /* "cupyx/cusolver.pyx":74
 *     'gesvda': (0, None),
 *     'potrfBatched': (0, None),
 *     'potrsBatched': (0, None),             # <<<<<<<<<<<<<<
 *     'syevj': (0, None),
 *     'gesv': (0, None),
*/
  if (PyDict_SetItem(__pyx_t_2, __pyx_mstate_global->__pyx_n_u_potrsBatched, __pyx_mstate_global->__pyx_tuple[4]) < (0)) __PYX_ERR(0, 70, __pyx_L1_error)

  /* "cupyx/cusolver.pyx":75
 *     'potrfBatched': (0, None),
 *     'potrsBatched': (0, None),
 *     'syevj': (0, None),             # <<<<<<<<<<<<<<
 *     'gesv': (0, None),
 *     'gels': (0, None),
*/
  if (PyDict_SetItem(__pyx_t_2, __pyx_mstate_global->__pyx_n_u_syevj, __pyx_mstate_global->__pyx_tuple[4]) < (0)) __PYX_ERR(0, 70, __pyx_L1_error)

  /* "cupyx/cusolver.pyx":76
 *     'potrsBatched': (0, None),
 *     'syevj': (0, None),
 *     'gesv': (0, None),             # <<<<<<<<<<<<<<
 *     'gels': (0, None),
 *     'csrlsvqr': (0, None),
*/
  if (PyDict_SetItem(__pyx_t_2, __pyx_mstate_global->__pyx_n_u_gesv, __pyx_mstate_global->__pyx_tuple[4]) < (0)) __PYX_ERR(0, 70, __pyx_L1_error)

  /* "cupyx/cusolver.pyx":77
 *     'syevj': (0, None),
 *     'gesv': (0, None),
 *     'gels': (0, None),             # <<<<<<<<<<<<<<
 *     'csrlsvqr': (0, None),
 *     'geev': (11700, None),
*/
  if (PyDict_SetItem(__pyx_t_2, __pyx_mstate_global->__pyx_n_u_gels, __pyx_mstate_global->__pyx_tuple[4]) < (0)) __PYX_ERR(0, 70, __pyx_L1_error)

  /* "cupyx/cusolver.pyx":78
 *     'gesv': (0, None),
 *     'gels': (0, None),
 *     'csrlsvqr': (0, None),             # <<<<<<<<<<<<<<
 *     'geev': (11700, None),
 * }
*/
  if (PyDict_SetItem(__pyx_t_2, __pyx_mstate_global->__pyx_n_u_csrlsvqr, __pyx_mstate_global->__pyx_tuple[4]) < (0)) __PYX_ERR(0, 70, __pyx_L1_error)

  /* "cupyx/cusolver.pyx":79
 *     'gels': (0, None),
 *     'csrlsvqr': (0, None),
 *     'geev': (11700, None),             # <<<<<<<<<<<<<<
 * }
 * 
*/
  if (PyDict_SetItem(__pyx_t_2, __pyx_mstate_global->__pyx_n_u_geev, __pyx_mstate_global->__pyx_tuple[5]) < (0)) __PYX_ERR(0, 70, __pyx_L1_error)
  if (PyDict_SetItem(__pyx_mstate_global->__pyx_d, __pyx_mstate_global->__pyx_n_u_available_cuda_version, __pyx_t_2) < (0)) __PYX_ERR(0, 69, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "cupyx/cusolver.pyx":85
 *     # For APIs supported by CUDA but not yet by HIP, we still need them here
 *     # so that our test suite can cover both platforms.
 *     'gesvdj': (_numpy.inf, None),             # <<<<<<<<<<<<<<
 *     'gesvdjBatched': (309, None),  # = rocsolver_<t>gesvd_batched
 *     'gesvda': (_numpy.inf, None),
*/
  __pyx_t_2 = __Pyx_PyDict_NewPresized(10); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 85, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_GetModuleGlobalName(__pyx_t_4, __pyx_mstate_global->__pyx_n_u_numpy); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 85, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_t_5 = __Pyx_PyObject_GetAttrStr(__pyx_t_4, __pyx_mstate_global->__pyx_n_u_inf); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 85, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_5);
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __pyx_t_4 = PyTuple_Pack(2, __pyx_t_5, Py_None); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 85, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
  if (PyDict_SetItem(__pyx_t_2, __pyx_mstate_global->__pyx_n_u_gesvdj, __pyx_t_4) < (0)) __PYX_ERR(0, 85, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

  /* "cupyx/cusolver.pyx":86
 *     # so that our test suite can cover both platforms.
 *     'gesvdj': (_numpy.inf, None),
 *     'gesvdjBatched': (309, None),  # = rocsolver_<t>gesvd_batched             # <<<<<<<<<<<<<<
 *     'gesvda': (_numpy.inf, None),
 *     'potrfBatched': (306, None),
*/
  if (PyDict_SetItem(__pyx_t_2, __pyx_mstate_global->__pyx_n_u_gesvdjBatched, __pyx_mstate_global->__pyx_tuple[6]) < (0)) __PYX_ERR(0, 85, __pyx_L1_error)

  /* "cupyx/cusolver.pyx":87
 *     'gesvdj': (_numpy.inf, None),
 *     'gesvdjBatched': (309, None),  # = rocsolver_<t>gesvd_batched
 *     'gesvda': (_numpy.inf, None),             # <<<<<<<<<<<<<<
 *     'potrfBatched': (306, None),
 *     'potrsBatched': (_numpy.inf, None),
*/
  __Pyx_GetModuleGlobalName(__pyx_t_4, __pyx_mstate_global->__pyx_n_u_numpy); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 87, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_t_5 = __Pyx_PyObject_GetAttrStr(__pyx_t_4, __pyx_mstate_global->__pyx_n_u_inf); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 87, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_5);
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __pyx_t_4 = PyTuple_Pack(2, __pyx_t_5, Py_None); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 87, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
  if (PyDict_SetItem(__pyx_t_2, __pyx_mstate_global->__pyx_n_u_gesvda, __pyx_t_4) < (0)) __PYX_ERR(0, 85, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

  /* "cupyx/cusolver.pyx":88
 *     'gesvdjBatched': (309, None),  # = rocsolver_<t>gesvd_batched
 *     'gesvda': (_numpy.inf, None),
 *     'potrfBatched': (306, None),             # <<<<<<<<<<<<<<
 *     'potrsBatched': (_numpy.inf, None),
 *     'syevj': (402, None),
*/
  if (PyDict_SetItem(__pyx_t_2, __pyx_mstate_global->__pyx_n_u_potrfBatched, __pyx_mstate_global->__pyx_tuple[7]) < (0)) __PYX_ERR(0, 85, __pyx_L1_error)

  /* "cupyx/cusolver.pyx":89
 *     'gesvda': (_numpy.inf, None),
 *     'potrfBatched': (306, None),
 *     'potrsBatched': (_numpy.inf, None),             # <<<<<<<<<<<<<<
 *     'syevj': (402, None),
 *     'gesv': (_numpy.inf, None),
*/
  __Pyx_GetModuleGlobalName(__pyx_t_4, __pyx_mstate_global->__pyx_n_u_numpy); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 89, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_t_5 = __Pyx_PyObject_GetAttrStr(__pyx_t_4, __pyx_mstate_global->__pyx_n_u_inf); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 89, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_5);
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __pyx_t_4 = PyTuple_Pack(2, __pyx_t_5, Py_None); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 89, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
  if (PyDict_SetItem(__pyx_t_2, __pyx_mstate_global->__pyx_n_u_potrsBatched, __pyx_t_4) < (0)) __PYX_ERR(0, 85, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

  /* "cupyx/cusolver.pyx":90
 *     'potrfBatched': (306, None),
 *     'potrsBatched': (_numpy.inf, None),
 *     'syevj': (402, None),             # <<<<<<<<<<<<<<
 *     'gesv': (_numpy.inf, None),
 *     'gels': (_numpy.inf, None),
*/
  if (PyDict_SetItem(__pyx_t_2, __pyx_mstate_global->__pyx_n_u_syevj, __pyx_mstate_global->__pyx_tuple[8]) < (0)) __PYX_ERR(0, 85, __pyx_L1_error)

  /* "cupyx/cusolver.pyx":91
 *     'potrsBatched': (_numpy.inf, None),
 *     'syevj': (402, None),
 *     'gesv': (_numpy.inf, None),             # <<<<<<<<<<<<<<
 *     'gels': (_numpy.inf, None),
 *     'csrlsvqr': (_numpy.inf, None),
*/
  __Pyx_GetModuleGlobalName(__pyx_t_4, __pyx_mstate_global->__pyx_n_u_numpy); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 91, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_t_5 = __Pyx_PyObject_GetAttrStr(__pyx_t_4, __pyx_mstate_global->__pyx_n_u_inf); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 91, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_5);
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __pyx_t_4 = PyTuple_Pack(2, __pyx_t_5, Py_None); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 91, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
  if (PyDict_SetItem(__pyx_t_2, __pyx_mstate_global->__pyx_n_u_gesv, __pyx_t_4) < (0)) __PYX_ERR(0, 85, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

  /* "cupyx/cusolver.pyx":92
 *     'syevj': (402, None),
 *     'gesv': (_numpy.inf, None),
 *     'gels': (_numpy.inf, None),             # <<<<<<<<<<<<<<
 *     'csrlsvqr': (_numpy.inf, None),
 *     'geev': (_numpy.inf, None),
*/
  __Pyx_GetModuleGlobalName(__pyx_t_4, __pyx_mstate_global->__pyx_n_u_numpy); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 92, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_t_5 = __Pyx_PyObject_GetAttrStr(__pyx_t_4, __pyx_mstate_global->__pyx_n_u_inf); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 92, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_5);
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __pyx_t_4 = PyTuple_Pack(2, __pyx_t_5, Py_None); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 92, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
  if (PyDict_SetItem(__pyx_t_2, __pyx_mstate_global->__pyx_n_u_gels, __pyx_t_4) < (0)) __PYX_ERR(0, 85, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

  /* "cupyx/cusolver.pyx":93
 *     'gesv': (_numpy.inf, None),
 *     'gels': (_numpy.inf, None),
 *     'csrlsvqr': (_numpy.inf, None),             # <<<<<<<<<<<<<<
 *     'geev': (_numpy.inf, None),
 * }
*/
  __Pyx_GetModuleGlobalName(__pyx_t_4, __pyx_mstate_global->__pyx_n_u_numpy); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 93, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_t_5 = __Pyx_PyObject_GetAttrStr(__pyx_t_4, __pyx_mstate_global->__pyx_n_u_inf); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 93, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_5);
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __pyx_t_4 = PyTuple_Pack(2, __pyx_t_5, Py_None); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 93, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
  if (PyDict_SetItem(__pyx_t_2, __pyx_mstate_global->__pyx_n_u_csrlsvqr, __pyx_t_4) < (0)) __PYX_ERR(0, 85, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

  /* "cupyx/cusolver.pyx":94
 *     'gels': (_numpy.inf, None),
 *     'csrlsvqr': (_numpy.inf, None),
 *     'geev': (_numpy.inf, None),             # <<<<<<<<<<<<<<
 * }
 * 
*/
  __Pyx_GetModuleGlobalName(__pyx_t_4, __pyx_mstate_global->__pyx_n_u_numpy); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 94, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_t_5 = __Pyx_PyObject_GetAttrStr(__pyx_t_4, __pyx_mstate_global->__pyx_n_u_inf); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 94, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_5);
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __pyx_t_4 = PyTuple_Pack(2, __pyx_t_5, Py_None); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 94, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
  if (PyDict_SetItem(__pyx_t_2, __pyx_mstate_global->__pyx_n_u_geev, __pyx_t_4) < (0)) __PYX_ERR(0, 85, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  if (PyDict_SetItem(__pyx_mstate_global->__pyx_d, __pyx_mstate_global->__pyx_n_u_available_hip_version, __pyx_t_2) < (0)) __PYX_ERR(0, 82, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "cupyx/cusolver.pyx":98
 * 
 * _available_compute_capability = {
 *     'gesv': 70,             # <<<<<<<<<<<<<<
 *     'gels': 70,
 * }
*/
  __pyx_t_2 = __Pyx_PyDict_NewPresized(2); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 98, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_t_2, __pyx_mstate_global->__pyx_n_u_gesv, __pyx_mstate_global->__pyx_int_70) < (0)) __PYX_ERR(0, 98, __pyx_L1_error)
  if (PyDict_SetItem(__pyx_t_2, __pyx_mstate_global->__pyx_n_u_gels, __pyx_mstate_global->__pyx_int_70) < (0)) __PYX_ERR(0, 98, __pyx_L1_error)
  if (PyDict_SetItem(__pyx_mstate_global->__pyx_d, __pyx_mstate_global->__pyx_n_u_available_compute_capability, __pyx_t_2) < (0)) __PYX_ERR(0, 97, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "cupyx/cusolver.pyx":103
 * 
 * 
 * @_util.memoize(for_each_device=True)             # <<<<<<<<<<<<<<
 * def check_availability(name):
 *     if not _runtime.is_hip:
*/
  __pyx_t_4 = NULL;
  __pyx_t_6 = NULL;
  __Pyx_GetModuleGlobalName(__pyx_t_7, __pyx_mstate_global->__pyx_n_u_util); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 103, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_7);
  __pyx_t_8 = __Pyx_PyObject_GetAttrStr(__pyx_t_7, __pyx_mstate_global->__pyx_n_u_memoize); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 103, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_8);
  __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
  __pyx_t_9 = 1;
  {
    PyObject *__pyx_callargs[2 + ((CYTHON_VECTORCALL) ? 1 : 0)] = {__pyx_t_6, NULL};
    __pyx_t_7 = __Pyx_MakeVectorcallBuilderKwds(1); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 103, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_7);
    if (__Pyx_VectorcallBuilder_AddArg(__pyx_mstate_global->__pyx_n_u_for_each_device, Py_True, __pyx_t_7, __pyx_callargs+1, 0) < (0)) __PYX_ERR(0, 103, __pyx_L1_error)
    __pyx_t_5 = __Pyx_Object_Vectorcall_CallFromBuilder((PyObject*)__pyx_t_8, __pyx_callargs+__pyx_t_9, (1-__pyx_t_9) | (__pyx_t_9*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET), __pyx_t_7);
    __Pyx_XDECREF(__pyx_t_6); __pyx_t_6 = 0;
    __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
    __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
    if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 103, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
  }
  __pyx_t_8 = PyCFunction_NewEx(&__pyx_mdef_5cupyx_8cusolver_1check_availability, NULL, __pyx_mstate_global->__pyx_n_u_cupyx_cusolver); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 103, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_8);
  __pyx_t_9 = 1;
  {
    PyObject *__pyx_callargs[2] = {__pyx_t_4, __pyx_t_8};
    __pyx_t_2 = __Pyx_PyObject_FastCall((PyObject*)__pyx_t_5, __pyx_callargs+__pyx_t_9, (2-__pyx_t_9) | (__pyx_t_9*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
    __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
    __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
    if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 103, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
  }
  if (PyDict_SetItem(__pyx_mstate_global->__pyx_d, __pyx_mstate_global->__pyx_n_u_check_availability, __pyx_t_2) < (0)) __PYX_ERR(0, 103, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "cupyx/cusolver.pyx":127
 * 
 * 
 * def gesvdj(a, full_matrices=True, compute_uv=True, overwrite_a=False):             # <<<<<<<<<<<<<<
 *     """Singular value decomposition using cusolverDn<t>gesvdj().
 * 
*/
  __pyx_t_2 = PyCFunction_NewEx(&__pyx_mdef_5cupyx_8cusolver_3gesvdj, NULL, __pyx_mstate_global->__pyx_n_u_cupyx_cusolver); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 127, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_mstate_global->__pyx_d, __pyx_mstate_global->__pyx_n_u_gesvdj, __pyx_t_2) < (0)) __PYX_ERR(0, 127, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "cupyx/cusolver.pyx":388
 * 
 * 
 * def gesvda(a, compute_uv=True):             # <<<<<<<<<<<<<<
 *     """Singular value decomposition using cusolverDn<t>gesvdaStridedBatched().
 * 
*/
  __pyx_t_2 = PyCFunction_NewEx(&__pyx_mdef_5cupyx_8cusolver_9gesvda, NULL, __pyx_mstate_global->__pyx_n_u_cupyx_cusolver); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 388, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_mstate_global->__pyx_d, __pyx_mstate_global->__pyx_n_u_gesvda, __pyx_t_2) < (0)) __PYX_ERR(0, 388, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "cupyx/cusolver.pyx":473
 * 
 * 
 * def syevj(a, UPLO='L', with_eigen_vector=True):             # <<<<<<<<<<<<<<
 *     """Eigenvalue decomposition of symmetric matrix using cusolverDn<t>syevj().
 * 
*/
  __pyx_t_2 = PyCFunction_NewEx(&__pyx_mdef_5cupyx_8cusolver_11syevj, NULL, __pyx_mstate_global->__pyx_n_u_cupyx_cusolver); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 473, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_mstate_global->__pyx_d, __pyx_mstate_global->__pyx_n_u_syevj, __pyx_t_2) < (0)) __PYX_ERR(0, 473, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "cupyx/cusolver.pyx":566
 * 
 * 
 * def _syevj_batched(a, UPLO, with_eigen_vector):             # <<<<<<<<<<<<<<
 *     # reject_float16=False for backward compatibility
 *     dtype, v_dtype = _cupy.linalg._util.linalg_common_type(
*/
  __pyx_t_2 = PyCFunction_NewEx(&__pyx_mdef_5cupyx_8cusolver_13_syevj_batched, NULL, __pyx_mstate_global->__pyx_n_u_cupyx_cusolver); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 566, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_mstate_global->__pyx_d, __pyx_mstate_global->__pyx_n_u_syevj_batched, __pyx_t_2) < (0)) __PYX_ERR(0, 566, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "cupyx/cusolver.pyx":637
 * 
 * 
 * def gesv(a, b):             # <<<<<<<<<<<<<<
 *     """Solve a linear matrix equation using cusolverDn<t1><t2>gesv().
 *     Computes the solution to a system of linear equation ``ax = b``.
*/
  __pyx_t_2 = PyCFunction_NewEx(&__pyx_mdef_5cupyx_8cusolver_15gesv, NULL, __pyx_mstate_global->__pyx_n_u_cupyx_cusolver); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 637, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_mstate_global->__pyx_d, __pyx_mstate_global->__pyx_n_u_gesv, __pyx_t_2) < (0)) __PYX_ERR(0, 637, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "cupyx/cusolver.pyx":714
 * 
 * 
 * def gels(a, b):             # <<<<<<<<<<<<<<
 *     """Compute least square solution using cusolverDn<t1><t2>gels().
 * 
*/
  __pyx_t_2 = PyCFunction_NewEx(&__pyx_mdef_5cupyx_8cusolver_17gels, NULL, __pyx_mstate_global->__pyx_n_u_cupyx_cusolver); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 714, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_mstate_global->__pyx_d, __pyx_mstate_global->__pyx_n_u_gels, __pyx_t_2) < (0)) __PYX_ERR(0, 714, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "cupyx/cusolver.pyx":814
 * 
 * 
 * def csrlsvqr(A, b, tol=0, reorder=1):             # <<<<<<<<<<<<<<
 *     """Solves the linear system ``Ax = b`` using QR factorization.
 * 
*/
  __pyx_t_2 = PyCFunction_NewEx(&__pyx_mdef_5cupyx_8cusolver_19csrlsvqr, NULL, __pyx_mstate_global->__pyx_n_u_cupyx_cusolver); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 814, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_mstate_global->__pyx_d, __pyx_mstate_global->__pyx_n_u_csrlsvqr, __pyx_t_2) < (0)) __PYX_ERR(0, 814, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "cupyx/cusolver.pyx":1
 * # distutils: language = c++             # <<<<<<<<<<<<<<
 * 
 * from libc.stdint cimport intptr_t
*/
  __pyx_t_2 = __Pyx_PyDict_NewPresized(0); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_mstate_global->__pyx_d, __pyx_mstate_global->__pyx_n_u_test, __pyx_t_2) < (0)) __PYX_ERR(0, 1, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /*--- Wrapped vars code ---*/

  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_XDECREF(__pyx_t_7);
  __Pyx_XDECREF(__pyx_t_8);
  if (__pyx_m) {
    if (__pyx_mstate->__pyx_d && stringtab_initialized) {
      __Pyx_AddTraceback("init cupyx.cusolver", __pyx_clineno, __pyx_lineno, __pyx_filename);
    }
    #if !CYTHON_USE_MODULE_STATE
    Py_CLEAR(__pyx_m);
    #else
    Py_DECREF(__pyx_m);
    if (pystate_addmodule_run) {
      PyObject *tp, *value, *tb;
      PyErr_Fetch(&tp, &value, &tb);
      PyState_RemoveModule(&__pyx_moduledef);
      PyErr_Restore(tp, value, tb);
    }
    #endif
  } else if (!PyErr_Occurred()) {
    PyErr_SetString(PyExc_ImportError, "init cupyx.cusolver");
  }
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  #if CYTHON_PEP489_MULTI_PHASE_INIT
  return (__pyx_m != NULL) ? 0 : -1;
  #else
  return __pyx_m;
  #endif
}
/* #### Code section: pystring_table ### */
/* #### Code section: cached_builtins ### */

static int __Pyx_InitCachedBuiltins(__pyx_mstatetype *__pyx_mstate) {
  CYTHON_UNUSED_VAR(__pyx_mstate);

  /* Cached unbound methods */
  __pyx_mstate->__pyx_umethod_PyDict_Type_items.type = (PyObject*)&PyDict_Type;
  __pyx_mstate->__pyx_umethod_PyDict_Type_items.method_name = &__pyx_mstate->__pyx_n_u_items;
  __pyx_mstate->__pyx_umethod_PyDict_Type_pop.type = (PyObject*)&PyDict_Type;
  __pyx_mstate->__pyx_umethod_PyDict_Type_pop.method_name = &__pyx_mstate->__pyx_n_u_pop;
  __pyx_mstate->__pyx_umethod_PyDict_Type_values.type = (PyObject*)&PyDict_Type;
  __pyx_mstate->__pyx_umethod_PyDict_Type_values.method_name = &__pyx_mstate->__pyx_n_u_values;
  return 0;
}
/* #### Code section: cached_constants ### */

static int __Pyx_InitCachedConstants(__pyx_mstatetype *__pyx_mstate) {
  __Pyx_RefNannyDeclarations
  CYTHON_UNUSED_VAR(__pyx_mstate);
  __Pyx_RefNannySetupContext("__Pyx_InitCachedConstants", 0);

  /* "cupyx/cusolver.pyx":237
 *     handle = _device.get_cusolver_handle()
 *     batch_size, m, n = a.shape
 *     a = _cupy.array(a.swapaxes(-2, -1), order='C',             # <<<<<<<<<<<<<<
 *                     copy=(None if overwrite_a else True))
 *     if runtime._is_hip_environment:
*/
  __pyx_mstate_global->__pyx_tuple[0] = PyTuple_Pack(2, __pyx_mstate_global->__pyx_int_neg_2, __pyx_mstate_global->__pyx_int_neg_1); if (unlikely(!__pyx_mstate_global->__pyx_tuple[0])) __PYX_ERR(0, 237, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_mstate_global->__pyx_tuple[0]);
  __Pyx_GIVEREF(__pyx_mstate_global->__pyx_tuple[0]);

  /* "cupyx/cusolver.pyx":270
 *     _cusolver.destroyGesvdjInfo(params)
 *     if runtime._is_hip_environment:
 *         v = v.swapaxes(-1, -2).conj()             # <<<<<<<<<<<<<<
 *     if not full_matrices:
 *         u = u[..., :mn]
*/
  __pyx_mstate_global->__pyx_tuple[1] = PyTuple_Pack(2, __pyx_mstate_global->__pyx_int_neg_1, __pyx_mstate_global->__pyx_int_neg_2); if (unlikely(!__pyx_mstate_global->__pyx_tuple[1])) __PYX_ERR(0, 270, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_mstate_global->__pyx_tuple[1]);
  __Pyx_GIVEREF(__pyx_mstate_global->__pyx_tuple[1]);

  /* "cupyx/cusolver.pyx":313
 *         trans_flag = False
 *     else:
 *         m, n = a.shape[-2:]             # <<<<<<<<<<<<<<
 *         x = a.swapaxes(-2, -1).astype(a_dtype, order='C', copy=True)
 *         trans_flag = True
*/
  __pyx_mstate_global->__pyx_slice[0] = PySlice_New(__pyx_mstate_global->__pyx_int_neg_2, Py_None, Py_None); if (unlikely(!__pyx_mstate_global->__pyx_slice[0])) __PYX_ERR(0, 313, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_mstate_global->__pyx_slice[0]);
  __Pyx_GIVEREF(__pyx_mstate_global->__pyx_slice[0]);

  /* "cupyx/cusolver.pyx":439
 *         batch_size = 1
 *     else:
 *         batch_size = _numpy.array(a_shape[:-2]).prod().item()             # <<<<<<<<<<<<<<
 *     a = a.reshape((batch_size, m, n))
 *     a = _cupy.ascontiguousarray(a.transpose(0, 2, 1))
*/
  __pyx_mstate_global->__pyx_slice[1] = PySlice_New(Py_None, __pyx_mstate_global->__pyx_int_neg_2, Py_None); if (unlikely(!__pyx_mstate_global->__pyx_slice[1])) __PYX_ERR(0, 439, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_mstate_global->__pyx_slice[1]);
  __Pyx_GIVEREF(__pyx_mstate_global->__pyx_slice[1]);

  /* "cupyx/cusolver.pyx":441
 *         batch_size = _numpy.array(a_shape[:-2]).prod().item()
 *     a = a.reshape((batch_size, m, n))
 *     a = _cupy.ascontiguousarray(a.transpose(0, 2, 1))             # <<<<<<<<<<<<<<
 *     lda = m
 *     stride_a = lda * n
*/
  __pyx_mstate_global->__pyx_tuple[2] = PyTuple_Pack(3, __pyx_mstate_global->__pyx_int_0, __pyx_mstate_global->__pyx_int_2, __pyx_mstate_global->__pyx_int_1); if (unlikely(!__pyx_mstate_global->__pyx_tuple[2])) __PYX_ERR(0, 441, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_mstate_global->__pyx_tuple[2]);
  __Pyx_GIVEREF(__pyx_mstate_global->__pyx_tuple[2]);

  /* "cupyx/cusolver.pyx":518
 *     m, lda = a.shape
 *     w = _cupy.empty(m, real_dtype)
 *     dev_info = _cupy.empty((1,), _cupy.int32)             # <<<<<<<<<<<<<<
 *     handle = _device.Device().cusolver_handle
 * 
*/
  __pyx_mstate_global->__pyx_tuple[3] = PyTuple_Pack(1, __pyx_mstate_global->__pyx_int_1); if (unlikely(!__pyx_mstate_global->__pyx_tuple[3])) __PYX_ERR(0, 518, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_mstate_global->__pyx_tuple[3]);
  __Pyx_GIVEREF(__pyx_mstate_global->__pyx_tuple[3]);

  /* "cupyx/cusolver.pyx":790
 *         bb = b.reshape(m, 1)
 *         b = _cupy.empty((max_mn, nrhs), dtype=a.dtype, order='F')
 *         b[:m, :] = bb             # <<<<<<<<<<<<<<
 *     else:
 *         b = b.copy(order='F')
*/
  __pyx_mstate_global->__pyx_slice[2] = PySlice_New(Py_None, Py_None, Py_None); if (unlikely(!__pyx_mstate_global->__pyx_slice[2])) __PYX_ERR(0, 790, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_mstate_global->__pyx_slice[2]);
  __Pyx_GIVEREF(__pyx_mstate_global->__pyx_slice[2]);

  /* "cupyx/cusolver.pyx":70
 * 
 * _available_cuda_version = {
 *     'gesvdj': (0, None),             # <<<<<<<<<<<<<<
 *     'gesvdjBatched': (0, None),
 *     'gesvda': (0, None),
*/
  __pyx_mstate_global->__pyx_tuple[4] = PyTuple_Pack(2, __pyx_mstate_global->__pyx_int_0, Py_None); if (unlikely(!__pyx_mstate_global->__pyx_tuple[4])) __PYX_ERR(0, 70, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_mstate_global->__pyx_tuple[4]);
  __Pyx_GIVEREF(__pyx_mstate_global->__pyx_tuple[4]);

  /* "cupyx/cusolver.pyx":79
 *     'gels': (0, None),
 *     'csrlsvqr': (0, None),
 *     'geev': (11700, None),             # <<<<<<<<<<<<<<
 * }
 * 
*/
  __pyx_mstate_global->__pyx_tuple[5] = PyTuple_Pack(2, __pyx_mstate_global->__pyx_int_11700, Py_None); if (unlikely(!__pyx_mstate_global->__pyx_tuple[5])) __PYX_ERR(0, 79, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_mstate_global->__pyx_tuple[5]);
  __Pyx_GIVEREF(__pyx_mstate_global->__pyx_tuple[5]);

  /* "cupyx/cusolver.pyx":86
 *     # so that our test suite can cover both platforms.
 *     'gesvdj': (_numpy.inf, None),
 *     'gesvdjBatched': (309, None),  # = rocsolver_<t>gesvd_batched             # <<<<<<<<<<<<<<
 *     'gesvda': (_numpy.inf, None),
 *     'potrfBatched': (306, None),
*/
  __pyx_mstate_global->__pyx_tuple[6] = PyTuple_Pack(2, __pyx_mstate_global->__pyx_int_309, Py_None); if (unlikely(!__pyx_mstate_global->__pyx_tuple[6])) __PYX_ERR(0, 86, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_mstate_global->__pyx_tuple[6]);
  __Pyx_GIVEREF(__pyx_mstate_global->__pyx_tuple[6]);

  /* "cupyx/cusolver.pyx":88
 *     'gesvdjBatched': (309, None),  # = rocsolver_<t>gesvd_batched
 *     'gesvda': (_numpy.inf, None),
 *     'potrfBatched': (306, None),             # <<<<<<<<<<<<<<
 *     'potrsBatched': (_numpy.inf, None),
 *     'syevj': (402, None),
*/
  __pyx_mstate_global->__pyx_tuple[7] = PyTuple_Pack(2, __pyx_mstate_global->__pyx_int_306, Py_None); if (unlikely(!__pyx_mstate_global->__pyx_tuple[7])) __PYX_ERR(0, 88, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_mstate_global->__pyx_tuple[7]);
  __Pyx_GIVEREF(__pyx_mstate_global->__pyx_tuple[7]);

  /* "cupyx/cusolver.pyx":90
 *     'potrfBatched': (306, None),
 *     'potrsBatched': (_numpy.inf, None),
 *     'syevj': (402, None),             # <<<<<<<<<<<<<<
 *     'gesv': (_numpy.inf, None),
 *     'gels': (_numpy.inf, None),
*/
  __pyx_mstate_global->__pyx_tuple[8] = PyTuple_Pack(2, __pyx_mstate_global->__pyx_int_402, Py_None); if (unlikely(!__pyx_mstate_global->__pyx_tuple[8])) __PYX_ERR(0, 90, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_mstate_global->__pyx_tuple[8]);
  __Pyx_GIVEREF(__pyx_mstate_global->__pyx_tuple[8]);
  #if CYTHON_IMMORTAL_CONSTANTS
  {
    PyObject **table = __pyx_mstate->__pyx_tuple;
    for (Py_ssize_t i=0; i<9; ++i) {
      #if CYTHON_COMPILING_IN_CPYTHON_FREETHREADING
      #if PY_VERSION_HEX < 0x030E0000
      if (_Py_IsOwnedByCurrentThread(table[i]) && Py_REFCNT(table[i]) == 1)
      #else
      if (PyUnstable_Object_IsUniquelyReferenced(table[i]))
      #endif
      {
        Py_SET_REFCNT(table[i], _Py_IMMORTAL_REFCNT_LOCAL);
      }
      #else
      Py_SET_REFCNT(table[i], _Py_IMMORTAL_INITIAL_REFCNT);
      #endif
    }
  }
  #endif
  #if CYTHON_IMMORTAL_CONSTANTS
  {
    PyObject **table = __pyx_mstate->__pyx_slice;
    for (Py_ssize_t i=0; i<3; ++i) {
      #if CYTHON_COMPILING_IN_CPYTHON_FREETHREADING
      #if PY_VERSION_HEX < 0x030E0000
      if (_Py_IsOwnedByCurrentThread(table[i]) && Py_REFCNT(table[i]) == 1)
      #else
      if (PyUnstable_Object_IsUniquelyReferenced(table[i]))
      #endif
      {
        Py_SET_REFCNT(table[i], _Py_IMMORTAL_REFCNT_LOCAL);
      }
      #else
      Py_SET_REFCNT(table[i], _Py_IMMORTAL_INITIAL_REFCNT);
      #endif
    }
  }
  #endif
  __Pyx_RefNannyFinishContext();
  return 0;
  __pyx_L1_error:;
  __Pyx_RefNannyFinishContext();
  return -1;
}
/* #### Code section: init_constants ### */

static int __Pyx_InitConstants(__pyx_mstatetype *__pyx_mstate) {
  CYTHON_UNUSED_VAR(__pyx_mstate);
  {
    const struct { const unsigned int length: 9; } index[] = {{2},{80},{27},{26},{49},{69},{52},{32},{1},{1},{26},{28},{22},{29},{33},{34},{26},{14},{28},{68},{21},{22},{40},{21},{22},{24},{24},{6},{30},{13},{37},{28},{23},{29},{1},{1},{17},{17},{17},{22},{22},{13},{26},{24},{1},{6},{1},{2},{1},{20},{1},{4},{1},{7},{5},{17},{6},{29},{23},{22},{1},{11},{1},{21},{32},{7},{14},{25},{18},{4},{18},{51},{6},{13},{24},{17},{17},{18},{8},{10},{4},{4},{16},{15},{8},{6},{7},{6},{24},{5},{4},{10},{22},{23},{9},{20},{6},{5},{14},{9},{8},{15},{1},{4},{6},{10},{17},{16},{7},{6},{21},{32},{7},{14},{25},{18},{6},{13},{24},{17},{5},{5},{10},{1},{2},{7},{15},{6},{13},{4},{4},{5},{4},{5},{6},{6},{13},{22},{16},{19},{22},{1},{7},{6},{3},{5},{4},{6},{14},{4},{5},{8},{1},{6},{7},{18},{5},{8},{9},{7},{4},{10},{4},{8},{7},{4},{3},{6},{5},{5},{5},{11},{3},{12},{12},{4},{3},{14},{12},{1},{3},{14},{7},{7},{16},{8},{7},{1},{5},{10},{21},{32},{7},{14},{25},{18},{5},{6},{6},{13},{24},{17},{8},{5},{14},{8},{3},{9},{5},{5},{6},{4},{9},{8},{17},{1},{1},{1},{21},{32},{7},{14},{25},{18},{6},{13},{24},{17},{227},{43},{461},{95},{437}};
    #if (CYTHON_COMPRESS_STRINGS) == 3 && __PYX_LIMITED_VERSION_HEX >= 0x030e0000 /* compression: zstd (1578 bytes) */
const char* const cstring = "(\265/\375`\376\017\0051\000:GH\014$\000\225\332\030\026\342\246,\217\010\005/\366[\310\026\372\006\300\231L\357\361\245y\0050\361\323\200\244\377\377\377\006\033\270\000\313\000\257\000\252`\263\234#J\213H\336\332W?\203g\322n\235`\\\317A\304\314Q\224\322\032E\\\251>P\0028\002\002.\001\3576\212\232V\205\271d8\330\262]\367\254\\U\306\242B1g\266\\\204\224\363f\203\355\367\2030\272\332\226\340l\203\321\213\210+>\247\352\3655HM\315a\024[\206x-\364~\220c\345\036{\257\244\266=\332X%U\017\243i\374\245\002j\341\364L1n*9\013\016\367k\244\326[\241\253\014\215\010\265\217z\217\221\252\0148L\317\002\252\204D\355r\332\333\336\253^\346b\213\032\307\256Z\327\251~j\237\035C\313\324J\374\354\017\252\177N9\253\ne \236\020gJ\313\262Wj\317\343\nY\255\033\220W\010\266\214\3735\326\332\016k\364BP|\362\201\274Ieqzn\374(\324\217\231\036_\324\360\273\267\\S\225\003\332\214\347,F\376b\t\316\224\311\274HLL\354bi%$20\033\021\226\016\014\006\205\205%\373u[2,\035\027\214F&\303\221a\361\300\\\322\"\201\271\310\313\305b\"5*\027\010\216\211\215\003\204\005\203\314DZ2\037\033i\331\340\310\\`\"\"\\\224vr\212SN7\257\333\232\367\252\355B\3358\344\254\372\265\335o9\326\250\002\020e\222{\357\363\310kMU\024#\346h\244\330n\226n\013\260B\312u\243\274\255\246\341\300\240\237JXX\252 a\241\340\000\341\t\357b\247\257\343\217\257s{{\301 \177-\310\366\310K\231\371\332\241\376\007\341\203\232#\275\322k\021M\277\236`\024]v\234\213\324\265\230\271\254\303/\332\310\363\323q\266{Qj\232:n!\333\361\366\225\320\235\235\033\034\367C\346x\003\035j\221\266\032iW\343\235\255E(\373A\014\2026\204[N\017\243e\2141\306XN\257\377=\344^\276,\365\225\261m\314\351&)\234\364\207\033C\245P\234\271\315\355\267\2222\301\334\245P\357\347\343K\303\202bM\360-\020\367\226\024\367C\226>\273\264?\327q\264\317\277\232t\327\2662_;\354^o\226\233\002\232M\335$_~\257\344\336\312\311\263\343\311\200*\000P\357\356-\365+\031~]\037\005%X\220\0073e\004\010\324\313d\264\220\312\022\022\344y+\366\273/\027W\307\331dMz\237(bt\270cU\360\234\223\001\350\314B""\\@9\313\304t\273U\334w\255J\357f\326k\2552\365\213\366\242\243bW\307\347\233\301V\302-\236\263\357\336:\356\007\242\027\037\037\356\377\312F\3565Kn3'\357\270\323\351t:\023\247\337\257\225\322\255\20677\317\256q\234\256cS\251\361\307\214\334\237\353\017\272\323\376\366\336\367\223\243PG\334\335\335\335\375[Th\331\257O\220\227\373F\034\201o\250\201-SjDDD\222$),\007a\010\2018\310\034C\0172\301H\020\222\214\020\003\211\210\211\210\004\321\202\202\302J\235\016D\254\353lF\2442\005\333\222\037\326-\010\013\334\361\322\3418\321BW\330)\330z\203\014_\237\264Hh\001\323A|G\013\357\322\203H>P\316,*^$\353c\340\222\000Z\177\206 \332\235\374\364\010k\366C\277OY(\236\361;5\026\022]%\343\252\371\025^0\270\245\241\272p\001\314O\230:\224B\256\277\214*G\026\210\222\304\304R\320\001w\255\344\001R\201'\343K\\+2\253\252\002\030^\230P\"\272\344\220\312>P\013\362\372\314\335\222oL\232\207\214\337\223\312r8\330\272\216\215z\374\014\210K\336\302\274\327\313\2604\231\034\224\000\371&Q?\273r\322\260\246\333\275\021\374gF)\036\225\t\032w\200\325\211\024\347\351X\226\233\"2\344\306\0244\236\250\303#QK\263\217GS\220\375\030\325\265C\252\355\016\001/f\302\031\332@\205B\370J\252\254B\270_\036W\216V09\350\254n}y\357\340\216\375|\007* k!U\212\3662g\232\220\353\033\177\223\205\333\242\306\3644\256\344\360\024\372(\034\0013\353\0165\212\222<]x)r<6\315!\327\274;\275\206]\2643\310\375\\\212\210\026\257\240\237\334Y\037x\002S\244\313\352\304C\277\3627\321\247\244\303\002\377\254gj\342V\022\264:\"+\266%>\313\227N\276EuF4\256\313V\353\367;\303\031\205\232%=\355q\006\251\270\203\211\215C\005\2613\033t\234\030d\020\332\307\310\323*\323\016\240\304J\252\367\020\n\260P\373!\340{a\007Zn\357\231\002hw\005<@\252b\375\327\232\233g\030\207\361\231>k\270\026\355+\346\260Y\024\021\276\350\256\240\024\233\324\\\236\225\310^(\3518\322\177\030f\212\235\306q{\032\233\345\321\341\214\362\033\327\201\213tI\271:nJ`\211\247\007\255\001Uk\033\346\031.\316\007)\314\252\220c U\201\036\256\201E\024\230\276\2266\364\221o\306HF'""\3443\254\230\263\254\326\361\205w\016\010b\3600\2108\001~\010P\034\304\314\267\330\261m\264\336\316\205a\365`\207'At\226q:\204\265In\324c,d\254w|\306\342ext\266e[\200SM\365\000I\341\326\352\020\327\273\223FD \304kH\300\264\021\262\204j\345\031\300B\237b\221\254MiH\347\314\322o\316@Y89\031\004\227\005D\300\255\375T\312\205K\275\236\366\026\224E<\033\3125\004\373\363;\335\020b\205q\356\032\367 \tx\235\004\351\304YIR\003`\377\272\255\3475x\314b\264\013L\354y!F\261\330mOG\326\220\263\205\345\335k\263\347\324\335\330\205\352\242{\225\251d\225\262\316\343\234e\327\177\340S\264y\000\210\032\213\256\032";
    PyObject *data = __Pyx_DecompressString(cstring, 1578, 3);
    if (unlikely(!data)) __PYX_ERR(0, 1, __pyx_L1_error)
    const char* const bytes = __Pyx_PyBytes_AsString(data);
    #if !CYTHON_ASSUME_SAFE_MACROS
    if (likely(bytes)); else { Py_DECREF(data); __PYX_ERR(0, 1, __pyx_L1_error) }
    #endif
    #elif (CYTHON_COMPRESS_STRINGS) == 2 /* compression: bz2 (1768 bytes) */
const char* const cstring = "BZh91AY&SY\024W\324\367\000\000\213_\200D\000A\365?\320\277\347\337\240\277\377\377\372`\007\234\356\352\3727\255\324\352\227gw\235\353\222\266\323b\3137l$\220MM2\236\247\243\t\224\312{SS\323Dh\006OQ\223i\000\006)\243Bi14\325<\241\262LG\251\352\032\0004\r\000\003I\262I&\332\247\244\364\206\324\r\000\332\200\000\000\000\004\232\2214 \232\233\324\230\024?T\000\323!\246\214\206\217H\006\2070\230\004\300\t\204\302i\200\000\004\311\246\201\204\212\t\243)\211\246Q\224\3642S\324z\230\232\003@\036\220\032\030\236\352\010\276\233\017\213z\250\376\024{r0}\367\226\031l\312\317\326WN\335\326\241\304=\374\013h\227\2122\225\341\324^\021\267\354\377\227|\376\217\033\2231!\255V\021\200|;;\217\307\204\236o\367s\331\014\3157\366\252*\361U\337\030+\022\223+@\212\274\232V\264\352\000\257\r\303\253:\316s\246G\335l)/\302\252a\344\"\032{~0\034J\000\004\306I9\261\364\002\200\210u3\313r\353\251E0*\260\337F\3530\036\371\2424\034\342R\336a\266R\310\323\242\314ypE\243\353\207\230r$$E\030M\222\330\330\357\003 \215e\245gA\256V\330k7\364\260\3143\014\314\3032\345\027\274f:\276\027i\307\344;y\223\304\377\032\214}\241\335\227\277\205\377\261\334zu\212e\303\364P)h\327\034\003\364!v\220\320\230\360\274=6\033\270}JM\0255\353id\001\353Fw\271\214\272\345\342\036Kj\324yD\235\036\336{\312\371.\n\225^j\365\236\t\325\353-\031a\302\205o\002\224\001\352\003\233)\2252\031f\330\361\320\203\326\225\331T5)U\014Th\nrD\316EP\334XX\251\344\3156\247M\230\024\352\241\371\341\360\345\251\306\345 \007|\022\320\331\252\2551\327\371k\344\201\331Z\3665UUUo\272\305 \304[\325G3\307>\203\371\277A\373\373\241\352\355\362\376\232\350\314\3525\350\241\364\237\350\001)3KV6q(Y*\256\3273\373\301\365\340;\250\336\200\232\376\350D\007\244\250iO\020*\273T\223\323BF/\033\213\235m\223>\234\177;\034c\3328\024\360\207\235WciM\256\333hi\231\321nJ\026\365$\\u\367l\005Q0\347\255\361\343\031\226\336y./O\355?G\366\010\350\247M\360\252\201\362W\332,\306y\341\003\003:\236{\372C\2208y#\216\014\300\007w\346\220\267(\342\374\265\200s\307$\232\225g""\001/\313&\\q\265\351DE:\243z:\241\366\250\244\330\2764@\014j^\374'\273\023\215\251a\323\314\353\247\350\320\371\210gU\247\013\330$\3061[\253\30113\237^\304\024\274cq,\032?\220xK\267\204%\333\022\250\331\017V\257y%\246%k\324\305\302yz2\306\t\256\374\360\375C\253\271\027\023\204\316\210\240(\275 \204z\006\003C\220\032\310\r\367\001\r\262\272\245YHo\031G\316%ehm\025\200\236\353`\005\020\211\205\362\270\254\224\026\030@\231\3272\262\277h.=\350\353\254\t\361\363\235\341D\311\331:\357\220v[\032\244K\024\224\255\241ki\265\271\344\010<\320\211\236\221\0245q\225C\274\014\346Cf'\241\016\204pn\365\341\367.\212\307x\022q\000\224\240\265\304\304\214\232 \232\313\037\273\331 >\333x\272\322\246\266\321\312\240\363#\331\353\036\272L\246$\312\326H\357\345C<: \016\316(\322\350\r\\tC\373\031T\326a1u\306\206u\363;4:vsNx\306\344\205\361\030C\262rR\322\351u\246\210xY\021\026*\2129\007\237\177\267xm\204>D\370\013x\020\001,L\000\n\016I\354HXX>h4V6;\357\032\364\325\244\355\224\303\244\221%\022K\316 \304\023\276\222D\223\231-\033c%\243i\014Z\030\306\223;\200\366\213\004j\331E\222#D\320\006M\266\335\246\276\371Z\246\251<\375]&\022\002\231\252]\200}\204\036\242\032\300 \271\343G\003.9\2655;\350\0346\034p\3246\232\023\240\304]\351\354\301MO\0331\266?X\033\326{q\244\230\030|s\363L\016[a\335\301\307\034\274\305_\262\016\031\237\260&Q\220v\242\033\371*\252\252\242\242\276\020\242\204U\\UZ\035\255\273\333\257T\346\233\200\317\0201\232G\314`S\010\010\213R\034[7pE\341\344\303\263\347\345\002\325E\275\324\016\364d\031\311\206\306T\241|R\326Q\013\213\034\025\3448\232:U]\373\326U17o\265n{\331x\"L\024e\202\027\r\3156E\277\017\315\270jr\272\3779\326*\341v8\312!9\204j;\014\364m:\014,2\247\330\362\215\000\rU8\325\020R$\261a\003}\254\310I\361\353\341\264k\"\340T\002\302\031W\245\301\327'q\352\021\020\345\324\223\016\227j\262gs\350\345\2418hG\253\247\216\202<2\245\tX\273%\003\303p\0200\366\327\211cT\327\004\022|\202\002\036\360X\215}\225,\245\370\303+o+j\341\274\356\255\037${w\001\200\230J1W\222'\313\260=p""\247\315\306\265\212\037\363\312\303T\221 \034n\0242,\002K2M\316\361\0166\250\204@\327\244E\365\227\036\226\276\300X'6\210\026\027\343)\004\t\221\2436\365\353aO]\325P\274K\t\3714ND\002\016D\0218\221\013\346=\360n\000C\264\347\226\357\032a+\310>\3022$?6\024\263\347>X\274s}&|\257Np\037\262\301\341\214\224.\003\202\237\004\261\302t\205\300C\206\320\346e\261>Y1\235\201\020\"v\030\325\356rH\031\2047\214\357\301,a\000\224)\213\265\352\035\004\0208\\\227!\214es\024\\X\2540(\3637\337--\nv\316\3176H\027\335\260X\252\013c\r\303v*\246\255\254y,h\202\214\321\243\270\236\367\241\200\r\257\r\242\356@\007v8\227\214\010\237\214\\M\344H|\014Z\243*\\\247\341\212L\211cDA-\307r\000\355+\345\277\273\316{\023\323\351\267<J\032\222@f\232V\021\261\230\303i\273\216;\3223\016\034\246\010;3\274\2354c]sd\263\010Qt[\312\255ydk\316\212\245\200 Ya4k\225\367Q\002\301\030\014\363\202\034\007wg\220\016\310\326\303XHPs\231\020L\243@w\347\222H:\033\271>\273\310B\004\264\327\343\34286\306#\357\000\377\305\334\221N\024$\005\025\365=\300";
    PyObject *data = __Pyx_DecompressString(cstring, 1768, 2);
    if (unlikely(!data)) __PYX_ERR(0, 1, __pyx_L1_error)
    const char* const bytes = __Pyx_PyBytes_AsString(data);
    #if !CYTHON_ASSUME_SAFE_MACROS
    if (likely(bytes)); else { Py_DECREF(data); __PYX_ERR(0, 1, __pyx_L1_error) }
    #endif
    #elif (CYTHON_COMPRESS_STRINGS) != 0 /* compression: zlib (1594 bytes) */
const char* const cstring = "x\332\245X\353O\3438\020\357\237\342O\007\254P%X\216[\335\227\023\013e\027\251l+^\247\373d9\216\323\272$v\360\24349\335\377~3v\032\232\246\005\335\036R\375\370\3313\236\031\317\303\341hxA\244%J;Rj+\235\\\n\222\212L*\351\004\321\206(\301\014\261R\315|\016\003\257Ra\210\323\2710LqA\376\376\207\034\256\027\245\253~\007\340\350\202\024\336:\222\010ry\177Gl\311\214\025\244`\316\310\325\215Z\262\\\246$uU)\310!\343\316\263<\020\375\320\204-\231\314Y\222\013\262\024\306J\255\210T\2316@\211c[\n.3)R\002\030PLT^\221,\327\314\021\246\200\243\366H\211C\356/uQ\346b\325\314\256\302R\213\031A\254/Km\234H\037\346\240z\346\025\017G\3008\225\026%H\tL\277\337L\t\263D\272\265}\224\020\251H\037\247\343\tp\231\371B(\327\252z0>@k\035<\036\014\377`-\312\210}\361xbT\177\310\206*\225E\273|\332\232\000,\220\2640\367e\005\033\2311\254J\272\024'\035\243\365\026A\202.\317w7 \013nMn\227/f\255c{\t\303xG\205\264 ;\237wg\300\002\330\037\223\004\017Y\357l\016\tw\362\371\3648\016\316\317\216\t\217\246\077\077\303\343\233\311\311\351\227\216\0343\221[2\007sgp>\330\377\020\031\007\260'X@\215`|\016\373\n\266\222\205/\010\313s\375\ns\360Z\023\034\306\302>\273\334\301\022\300\035,\3552e{\360E\037?&3\230\007\346Rqm\214\340\216D\237\026+pT\207\2224\276n\347\254\024ow \300!r\210 \264\305\014\264p\030Os\246\210\032\206\235{ll+\261\334!\210W\255/o\005\025\006\342\345\345\344v\372\3700\242\017\177MG\364zzr\276\005|>\355\000\017\327\000<~\035_\334\323\353\233\361\230\336N\256Ft<\371st\327C\037\247SD\357'\343\247\321\335\310\030m\326\023:\272\371\026\367\374\230<\215.\037&w\375\225\210_]\211\245\344\342\372\372jL\351\264Z\301\357JrG\177\210\225\273\023\331#F\032c4\350\025\202\201Y\256\225\2233\257\275m\000\\\243\255=(z\227w\320\263\222%2\207\224\264\271\350SF\233\334\262\001\317e\271F\023\232\370,\023\346^\326\202\363\350\025\367\020\271\020\366_\361ND\272\023\334\244\212.\323t\035\242\305\336\335\233\310\234\031\330\304\237\327\002F%\"\304\275\3259\210JS\261\244\230\034\251\314\250\255\024\237\033\255d\035\374""\2366\221\000\024\3401\261]\213\2619\351\234\211\370\006@)\317\231\265t&\034\304S\201S\251\004\234H\235a\\$\214?\3070vbmp\277\204\253Yp]V<8\365\267\240\330\r\310\030\347\367\350\277a\332$\034\2167gA\245\004\216\212-\2157\024n\026\016\203\310\310\3404\2120\260m~C\270c\0107\030Q\224C\250\324\016\221l\310J\271\003\315eb\003Y`\0358\241Z\264I\257\r\363\325[3l\215\274\036l\367\024\2425\205\204\2222\007\316),7\241\221\245\3238rFWo\3327@\253>^\035\370|\323\356r\246\364\003\017\213\353\213\264\343S\351>\017K{\036\226\206T\022\3335\365\346\244\267u\023\300K\023E\351\252\320\320\\>\213,K\233L\017\205\231bZnt\214\265;\363yNC\371\343\302\316\300\3210}\317\304\213\311P\262 ]T8\n\332\321\003/\252\037\321\233(\312\023\346\335\273\351@\266l\320\271\204j\010R@W\202<\n\0369P\250\240\371\"-f\001im\031\0135\005/E\317\307\237\215M-\236!\010X>\243\261k& H\001A\207r`\330\031\n\272B\234`\347(\234b\013Qh .\300\023\000\324\251\207\214C\025+\336\332\350\211X\250\225\252\251\362EY\205\006\275\t\362\352\354\305hP\343\025\005\242\254\324\220\356\235\311\032\013\341\330\256\307F\243^\224\202\027\323\245\013\271\r\376\260\336\304\223\340\230W#\026P\236h\270\261\223s#\302!F\204\322C\215\366\016\002\3356*R\343!\333\026\242\351\254\345\262\254\254p\360<d>wv\227\243\332\017\2747\256/l\347\236\355>\357\265=\357\r\202\3067\245\r\336i7}\327\356qd\273\355\310\366\025\334i\265\306\233\325\206\214:\210Yhu\016\331NYx\025\203\237\031\351)\030'\207\232\356\221\332(\212\r<}m\333K7\247B\316\204\202r\302!\031\254\252\272\336e\216\372\003\033\305\365E\335\261J\275\317Fu\317Fu\310\347\365f\266\257\367\244\376z;\365C4\014\232?\032\243\202\n\265\224P_\360\271;\210\221}\341@\364\004\342\3572\206\341e\033\233\267l\241\315\307\273\244\202]\"\274\032\240\240C\271\277\261W1Q\2124\342\315\227\302\235\260\332\033.\276\207\000\356,=\341MD\344\026\302\313T\027P\370x\250\201\021\235\na.8\304\273\275\310\241\006\245\325H\205\267=\250H\016\261\211\221b\237AGx\367\227h\234\243\001\246\216(?\225\037l]\257b\320\271cx\360m4\273)\340""\357=\242c\202y\206\356\230\374wv\035N\357q\300r\370\024\237\077\077|\221\0103\210\257\222\r\247\030\244=\304\366\220\272\207p\257 yu\370\204t\326\341\323C\352\036\025\017\345bK\236-\304\366\220z\033\001\377\362|m\007\235,\350Yx\r\234\205\227\301y\021|\210FW\232\352\360\374 \237\016\247\325$\301\214I>\275cE|r\361=\354\177\r\357\025\230`\333>;\022\006\337\304]\356\221\320u\311~\203\272\027\010b\202v\360\371\244\340;\342\227c\262I\3726>\032\374_1\366*\3713\214\177\202\346\035\t\332\235\370\317\211\001\360PP\027\035\362\305\222\026\322\357\256\007\344 \016\241\242\321\336\007\304\277\022\330\202>";
    PyObject *data = __Pyx_DecompressString(cstring, 1594, 1);
    if (unlikely(!data)) __PYX_ERR(0, 1, __pyx_L1_error)
    const char* const bytes = __Pyx_PyBytes_AsString(data);
    #if !CYTHON_ASSUME_SAFE_MACROS
    if (likely(bytes)); else { Py_DECREF(data); __PYX_ERR(0, 1, __pyx_L1_error) }
    #endif
    #else /* compression: none (4350 bytes) */
const char* const bytes = ").A is not positive definite or near singular under tolerance {} (singularity: {})A must be CSR sparse matrixInvalid dtype (actual: {})No available version information specified for {}Only float and double and cuComplex and cuDoubleComplex are supportedThis function is disabled on HIP as it is not neededUPLO argument must be 'L' or 'U'.?a must be a square matrix.a.ndim must be 2 (actual:{})b must be cupy.ndarrayb.ndim must be 1 (actual: {})b.ndim must be 1 or 2 (actual:{})b.ndim must be 1 or 2 (actual: {})csrlsvqr is not available.dtype mismatchdtype mismatch (a:{}, b:{}).dtype must be float32, float64, complex64 or complex128 (actual: {})gels has failed ({}).gels is not available.gels reached maximum allowed iterations.gesv has failed ({}).gesv is not available.gesvda is not available.gesvdj is not available., got  has incorrect type (expected invalid shapem must be equal to or greater than n.shape mismatch (a:{}, b:{}).syevj is not available.unsupported dtype (actual:{})ACCOMPUTE_TYPE_FP16COMPUTE_TYPE_FP32COMPUTE_TYPE_TF32CUBLAS_FILL_MODE_LOWERCUBLAS_FILL_MODE_UPPERCUSOLVERErrorCUSOLVER_EIG_MODE_NOVECTORCUSOLVER_EIG_MODE_VECTORDDeviceFFDL__Pyx_PyDict_NextRefUUPLOaa_dtypearrayascontiguousarrayastype_available_compute_capability_available_cuda_version_available_hip_versionb_bufferSizeccgesvdaStridedBatchedcgesvdaStridedBatched_bufferSizecgesvdjcgesvdjBatchedcgesvdjBatched_bufferSizecgesvdj_bufferSizecharcheck_availability_check_cusolver_dev_info_if_synchronization_allowedcheevjcheevjBatchedcheevjBatched_bufferSizecheevj_bufferSize__class_getitem__cline_in_tracebackcompletecompute_uvconjcopycreateGesvdjInfocreateSyevjInfocsrlsvqrctypes_cublascublas__cuda_array_interface___cupycupycupy._corecupy_backends.cuda.apicupy_backends.cuda.libscupy.cuda__cupy_get_ndarray___cupyxcupyxcupyx.cusolver_cusolvercusolvercusolver_handleddata_descrdescriptordestroyGesvdjInfodestroySyevjInfo_devicedevicedgesvdaStridedBatcheddgesvdaStridedBatched_bufferSizedgesvdjdgesvdjBatcheddgesvdjBat""ched_bufferSizedgesvdj_bufferSizedsyevjdsyevjBatcheddsyevjBatched_bufferSizedsyevj_bufferSizedtypeemptyempty_likeffdfloat64for_each_deviceformatfull_matricesgeevgelsgeqrfgesvgesvdgesvdagesvdjgesvdjBatchedget_compute_capabilityget_compute_typeget_cusolver_handleget_cusolver_sp_handlehindicesindptrinfint32int8is_hipisspmatrix_csritemitemsitemsizeklinalg_linalglinalg_common_typelower__main___mat_ptrsmemoizemode__module__name__name__ndarrayndimnnz_numpynumpyorderorgqroverwrite_apoppotrfBatchedpotrsBatchedprodptr__pyx_vtable____qualname__rrawreject_float16reorderreshape_routines_linalg_runtimeruntimesscipysetdefaultsgesvdaStridedBatchedsgesvdaStridedBatched_bufferSizesgesvdjsgesvdjBatchedsgesvdjBatched_bufferSizesgesvdj_bufferSizeshapesparsessyevjssyevjBatchedssyevjBatched_bufferSizessyevj_bufferSizeswapaxessyevj_syevj_batched__test__toltranspose_triu_utilvalueswarn_warningswarningswith_eigen_vectorxyzzgesvdaStridedBatchedzgesvdaStridedBatched_bufferSizezgesvdjzgesvdjBatchedzgesvdjBatched_bufferSizezgesvdj_bufferSizezheevjzheevjBatchedzheevjBatched_bufferSizezheevj_bufferSizeint\000\000\000\000\000\000\000\000_is_hip_environment\000deviceAttributeComputeCapabilityMajor\000deviceAttributeComputeCapabilityMinor\000errorContextIsDestroyed\000errorInvalidResourceHandle\000errorInvalidValue\000errorMemoryAllocation\000errorPeerAccessAlreadyEnabledint (int __pyx_skip_dispatch)\000get_device_idint (int __pyx_skip_dispatch)\000int (intptr_t, int, int, int __pyx_skip_dispatch)\000\000\000\000int (intptr_t, int, int, int, size_t, int, size_t, int __pyx_skip_dispatch)\000\000\000\000int (intptr_t, int, int, size_t, int, int __pyx_skip_dispatch)\000\000\000\000_getVersionNumber\000cgesvd_bufferSize\000dgesvd_bufferSize\000sgesvd_bufferSize\000zgesvd_bufferSize\000cungqr_bufferSize\000dorgqr_bufferSize\000sorgqr_bufferSize\000zungqr_bufferSize\000cgeqrf_bufferSize\000dgeqrf_bufferSize\000sgeqrf_bufferSize\000zgeqrf_bufferSizestruct __pyx_obj_4cupy_4cuda_6memory_MemoryPointer *(PyObje""ct *, int __pyx_skip_dispatch)\000allocstruct __pyx_obj_4cupy_5_core_4core__ndarray_base *(PyObject *, __pyx_t_4cupy_5_core_7_carray_shape_t const &, PyObject *, PyObject *)\000struct __pyx_obj_4cupy_5_core_4core__ndarray_base *(PyObject *, int __pyx_skip_dispatch)\000struct __pyx_obj_4cupy_5_core_4core__ndarray_base *(struct __pyx_obj_4cupy_5_core_4core__ndarray_base *, int __pyx_skip_dispatch)\000_ndarray_init\000_convert_object_with_cuda_array_interface\000_internal_ascontiguousarray";
    PyObject *data = NULL;
    CYTHON_UNUSED_VAR(__Pyx_DecompressString);
    #endif
    PyObject **stringtab = __pyx_mstate->__pyx_string_tab;
    Py_ssize_t pos = 0;
    for (int i = 0; i < 228; i++) {
      Py_ssize_t bytes_length = index[i].length;
      PyObject *string = PyUnicode_DecodeUTF8(bytes + pos, bytes_length, NULL);
      if (likely(string) && i >= 34) PyUnicode_InternInPlace(&string);
      if (unlikely(!string)) {
        Py_XDECREF(data);
        __PYX_ERR(0, 1, __pyx_L1_error)
      }
      stringtab[i] = string;
      pos += bytes_length;
    }
    for (int i = 228; i < 233; i++) {
      Py_ssize_t bytes_length = index[i].length;
      PyObject *string = PyBytes_FromStringAndSize(bytes + pos, bytes_length);
      stringtab[i] = string;
      pos += bytes_length;
      if (unlikely(!string)) {
        Py_XDECREF(data);
        __PYX_ERR(0, 1, __pyx_L1_error)
      }
    }
    Py_XDECREF(data);
    for (Py_ssize_t i = 0; i < 233; i++) {
      if (unlikely(PyObject_Hash(stringtab[i]) == -1)) {
        __PYX_ERR(0, 1, __pyx_L1_error)
      }
    }
    #if CYTHON_IMMORTAL_CONSTANTS
    {
      PyObject **table = stringtab + 228;
      for (Py_ssize_t i=0; i<5; ++i) {
        #if CYTHON_COMPILING_IN_CPYTHON_FREETHREADING
        #if PY_VERSION_HEX < 0x030E0000
        if (_Py_IsOwnedByCurrentThread(table[i]) && Py_REFCNT(table[i]) == 1)
        #else
        if (PyUnstable_Object_IsUniquelyReferenced(table[i]))
        #endif
        {
          Py_SET_REFCNT(table[i], _Py_IMMORTAL_REFCNT_LOCAL);
        }
        #else
        Py_SET_REFCNT(table[i], _Py_IMMORTAL_INITIAL_REFCNT);
        #endif
      }
    }
    #endif
  }
  {
    PyObject **numbertab = __pyx_mstate->__pyx_number_tab + 0;
    int8_t const cint_constants_1[] = {0,-1,1,-2,2,3,-50,70};
    int16_t const cint_constants_2[] = {306,309,402,11700};
    for (int i = 0; i < 12; i++) {
      numbertab[i] = PyLong_FromLong((i < 8 ? cint_constants_1[i - 0] : cint_constants_2[i - 8]));
      if (unlikely(!numbertab[i])) __PYX_ERR(0, 1, __pyx_L1_error)
    }
  }
  #if CYTHON_IMMORTAL_CONSTANTS
  {
    PyObject **table = __pyx_mstate->__pyx_number_tab;
    for (Py_ssize_t i=0; i<12; ++i) {
      #if CYTHON_COMPILING_IN_CPYTHON_FREETHREADING
      #if PY_VERSION_HEX < 0x030E0000
      if (_Py_IsOwnedByCurrentThread(table[i]) && Py_REFCNT(table[i]) == 1)
      #else
      if (PyUnstable_Object_IsUniquelyReferenced(table[i]))
      #endif
      {
        Py_SET_REFCNT(table[i], _Py_IMMORTAL_REFCNT_LOCAL);
      }
      #else
      Py_SET_REFCNT(table[i], _Py_IMMORTAL_INITIAL_REFCNT);
      #endif
    }
  }
  #endif
  return 0;
  __pyx_L1_error:;
  return -1;
}
/* #### Code section: init_codeobjects ### */

static int __Pyx_CreateCodeObjects(__pyx_mstatetype *__pyx_mstate) {
  CYTHON_UNUSED_VAR(__pyx_mstate);
  return 0;
}
/* #### Code section: init_globals ### */

static int __Pyx_InitGlobals(void) {
  /* PythonCompatibility.init */
  if (likely(__Pyx_init_co_variables() == 0)); else
  
  if (unlikely(PyErr_Occurred())) __PYX_ERR(0, 1, __pyx_L1_error)

  /* AssertionsEnabled.init */
  if (likely(__Pyx_init_assertions_enabled() == 0)); else
  
  if (unlikely(PyErr_Occurred())) __PYX_ERR(0, 1, __pyx_L1_error)

  return 0;
  __pyx_L1_error:;
  return -1;
}
/* #### Code section: cleanup_globals ### */
/* #### Code section: cleanup_module ### */
/* #### Code section: main_method ### */
/* #### Code section: utility_code_pragmas ### */
#ifdef _MSC_VER
#pragma warning( push )
/* Warning 4127: conditional expression is constant
 * Cython uses constant conditional expressions to allow in inline functions to be optimized at
 * compile-time, so this warning is not useful
 */
#pragma warning( disable : 4127 )
#endif



/* #### Code section: utility_code_def ### */

/* --- Runtime support code --- */
/* Refnanny */
#if CYTHON_REFNANNY
static __Pyx_RefNannyAPIStruct *__Pyx_RefNannyImportAPI(const char *modname) {
    PyObject *m = NULL, *p = NULL;
    void *r = NULL;
    m = PyImport_ImportModule(modname);
    if (!m) goto end;
    p = PyObject_GetAttrString(m, "RefNannyAPI");
    if (!p) goto end;
    r = PyLong_AsVoidPtr(p);
end:
    Py_XDECREF(p);
    Py_XDECREF(m);
    return (__Pyx_RefNannyAPIStruct *)r;
}
#endif

/* GetItemInt */
static PyObject *__Pyx_GetItemInt_Generic(PyObject *o, PyObject* j) {
    PyObject *r;
    if (unlikely(!j)) return NULL;
    r = PyObject_GetItem(o, j);
    Py_DECREF(j);
    return r;
}
static CYTHON_INLINE PyObject *__Pyx_GetItemInt_List_Fast(PyObject *o, Py_ssize_t i,
                                                              int wraparound, int boundscheck, int unsafe_shared) {
    CYTHON_MAYBE_UNUSED_VAR(unsafe_shared);
#if CYTHON_ASSUME_SAFE_SIZE
    Py_ssize_t wrapped_i = i;
    if (wraparound & unlikely(i < 0)) {
        wrapped_i += PyList_GET_SIZE(o);
    }
    if ((CYTHON_AVOID_BORROWED_REFS || CYTHON_AVOID_THREAD_UNSAFE_BORROWED_REFS || !CYTHON_ASSUME_SAFE_MACROS)) {
        return __Pyx_PyList_GetItemRefFast(o, wrapped_i, unsafe_shared);
    } else
    if ((!boundscheck) || likely(__Pyx_is_valid_index(wrapped_i, PyList_GET_SIZE(o)))) {
        return __Pyx_NewRef(PyList_GET_ITEM(o, wrapped_i));
    }
    return __Pyx_GetItemInt_Generic(o, PyLong_FromSsize_t(i));
#else
    (void)wraparound;
    (void)boundscheck;
    return PySequence_GetItem(o, i);
#endif
}
static CYTHON_INLINE PyObject *__Pyx_GetItemInt_Tuple_Fast(PyObject *o, Py_ssize_t i,
                                                              int wraparound, int boundscheck, int unsafe_shared) {
    CYTHON_MAYBE_UNUSED_VAR(unsafe_shared);
#if CYTHON_ASSUME_SAFE_SIZE && CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
    Py_ssize_t wrapped_i = i;
    if (wraparound & unlikely(i < 0)) {
        wrapped_i += PyTuple_GET_SIZE(o);
    }
    if ((!boundscheck) || likely(__Pyx_is_valid_index(wrapped_i, PyTuple_GET_SIZE(o)))) {
        return __Pyx_NewRef(PyTuple_GET_ITEM(o, wrapped_i));
    }
    return __Pyx_GetItemInt_Generic(o, PyLong_FromSsize_t(i));
#else
    (void)wraparound;
    (void)boundscheck;
    return PySequence_GetItem(o, i);
#endif
}
static CYTHON_INLINE PyObject *__Pyx_GetItemInt_Fast(PyObject *o, Py_ssize_t i, int is_list,
                                                     int wraparound, int boundscheck, int unsafe_shared) {
    CYTHON_MAYBE_UNUSED_VAR(unsafe_shared);
#if CYTHON_ASSUME_SAFE_MACROS && CYTHON_ASSUME_SAFE_SIZE
    if (is_list || PyList_CheckExact(o)) {
        Py_ssize_t n = ((!wraparound) | likely(i >= 0)) ? i : i + PyList_GET_SIZE(o);
        if ((CYTHON_AVOID_BORROWED_REFS || CYTHON_AVOID_THREAD_UNSAFE_BORROWED_REFS)) {
            return __Pyx_PyList_GetItemRefFast(o, n, unsafe_shared);
        } else if ((!boundscheck) || (likely(__Pyx_is_valid_index(n, PyList_GET_SIZE(o))))) {
            return __Pyx_NewRef(PyList_GET_ITEM(o, n));
        }
    } else
    #if !CYTHON_AVOID_BORROWED_REFS
    if (PyTuple_CheckExact(o)) {
        Py_ssize_t n = ((!wraparound) | likely(i >= 0)) ? i : i + PyTuple_GET_SIZE(o);
        if ((!boundscheck) || likely(__Pyx_is_valid_index(n, PyTuple_GET_SIZE(o)))) {
            return __Pyx_NewRef(PyTuple_GET_ITEM(o, n));
        }
    } else
    #endif
#endif
#if CYTHON_USE_TYPE_SLOTS && !CYTHON_COMPILING_IN_PYPY
    {
        PyMappingMethods *mm = Py_TYPE(o)->tp_as_mapping;
        PySequenceMethods *sm = Py_TYPE(o)->tp_as_sequence;
        if (!is_list && mm && mm->mp_subscript) {
            PyObject *r, *key = PyLong_FromSsize_t(i);
            if (unlikely(!key)) return NULL;
            r = mm->mp_subscript(o, key);
            Py_DECREF(key);
            return r;
        }
        if (is_list || likely(sm && sm->sq_item)) {
            if (wraparound && unlikely(i < 0) && likely(sm->sq_length)) {
                Py_ssize_t l = sm->sq_length(o);
                if (likely(l >= 0)) {
                    i += l;
                } else {
                    if (!PyErr_ExceptionMatches(PyExc_OverflowError))
                        return NULL;
                    PyErr_Clear();
                }
            }
            return sm->sq_item(o, i);
        }
    }
#else
    if (is_list || !PyMapping_Check(o)) {
        return PySequence_GetItem(o, i);
    }
#endif
    (void)wraparound;
    (void)boundscheck;
    return __Pyx_GetItemInt_Generic(o, PyLong_FromSsize_t(i));
}

/* PyErrExceptionMatches (used by PyObjectGetAttrStrNoError) */
#if CYTHON_FAST_THREAD_STATE
static int __Pyx_PyErr_ExceptionMatchesTuple(PyObject *exc_type, PyObject *tuple) {
    Py_ssize_t i, n;
    n = PyTuple_GET_SIZE(tuple);
    for (i=0; i<n; i++) {
        if (exc_type == PyTuple_GET_ITEM(tuple, i)) return 1;
    }
    for (i=0; i<n; i++) {
        if (__Pyx_PyErr_GivenExceptionMatches(exc_type, PyTuple_GET_ITEM(tuple, i))) return 1;
    }
    return 0;
}
static CYTHON_INLINE int __Pyx_PyErr_ExceptionMatchesInState(PyThreadState* tstate, PyObject* err) {
    int result;
    PyObject *exc_type;
#if PY_VERSION_HEX >= 0x030C00A6
    PyObject *current_exception = tstate->current_exception;
    if (unlikely(!current_exception)) return 0;
    exc_type = (PyObject*) Py_TYPE(current_exception);
    if (exc_type == err) return 1;
#else
    exc_type = tstate->curexc_type;
    if (exc_type == err) return 1;
    if (unlikely(!exc_type)) return 0;
#endif
    #if CYTHON_AVOID_BORROWED_REFS
    Py_INCREF(exc_type);
    #endif
    if (unlikely(PyTuple_Check(err))) {
        result = __Pyx_PyErr_ExceptionMatchesTuple(exc_type, err);
    } else {
        result = __Pyx_PyErr_GivenExceptionMatches(exc_type, err);
    }
    #if CYTHON_AVOID_BORROWED_REFS
    Py_DECREF(exc_type);
    #endif
    return result;
}
#endif

/* PyErrFetchRestore (used by PyObjectGetAttrStrNoError) */
#if CYTHON_FAST_THREAD_STATE
static CYTHON_INLINE void __Pyx_ErrRestoreInState(PyThreadState *tstate, PyObject *type, PyObject *value, PyObject *tb) {
#if PY_VERSION_HEX >= 0x030C00A6
    PyObject *tmp_value;
    assert(type == NULL || (value != NULL && type == (PyObject*) Py_TYPE(value)));
    if (value) {
        #if CYTHON_COMPILING_IN_CPYTHON
        if (unlikely(((PyBaseExceptionObject*) value)->traceback != tb))
        #endif
            PyException_SetTraceback(value, tb);
    }
    tmp_value = tstate->current_exception;
    tstate->current_exception = value;
    Py_XDECREF(tmp_value);
    Py_XDECREF(type);
    Py_XDECREF(tb);
#else
    PyObject *tmp_type, *tmp_value, *tmp_tb;
    tmp_type = tstate->curexc_type;
    tmp_value = tstate->curexc_value;
    tmp_tb = tstate->curexc_traceback;
    tstate->curexc_type = type;
    tstate->curexc_value = value;
    tstate->curexc_traceback = tb;
    Py_XDECREF(tmp_type);
    Py_XDECREF(tmp_value);
    Py_XDECREF(tmp_tb);
#endif
}
static CYTHON_INLINE void __Pyx_ErrFetchInState(PyThreadState *tstate, PyObject **type, PyObject **value, PyObject **tb) {
#if PY_VERSION_HEX >= 0x030C00A6
    PyObject* exc_value;
    exc_value = tstate->current_exception;
    tstate->current_exception = 0;
    *value = exc_value;
    *type = NULL;
    *tb = NULL;
    if (exc_value) {
        *type = (PyObject*) Py_TYPE(exc_value);
        Py_INCREF(*type);
        #if CYTHON_COMPILING_IN_CPYTHON
        *tb = ((PyBaseExceptionObject*) exc_value)->traceback;
        Py_XINCREF(*tb);
        #else
        *tb = PyException_GetTraceback(exc_value);
        #endif
    }
#else
    *type = tstate->curexc_type;
    *value = tstate->curexc_value;
    *tb = tstate->curexc_traceback;
    tstate->curexc_type = 0;
    tstate->curexc_value = 0;
    tstate->curexc_traceback = 0;
#endif
}
#endif

/* PyObjectGetAttrStr (used by PyObjectGetAttrStrNoError) */
#if CYTHON_USE_TYPE_SLOTS
static CYTHON_INLINE PyObject* __Pyx_PyObject_GetAttrStr(PyObject* obj, PyObject* attr_name) {
    PyTypeObject* tp = Py_TYPE(obj);
    if (likely(tp->tp_getattro))
        return tp->tp_getattro(obj, attr_name);
    return PyObject_GetAttr(obj, attr_name);
}
#endif

/* PyObjectGetAttrStrNoError (used by HasAttr) */
#if __PYX_LIMITED_VERSION_HEX < 0x030d0000
static void __Pyx_PyObject_GetAttrStr_ClearAttributeError(void) {
    __Pyx_PyThreadState_declare
    __Pyx_PyThreadState_assign
    if (likely(__Pyx_PyErr_ExceptionMatches(PyExc_AttributeError)))
        __Pyx_PyErr_Clear();
}
#endif
static CYTHON_INLINE PyObject* __Pyx_PyObject_GetAttrStrNoError(PyObject* obj, PyObject* attr_name) {
    PyObject *result;
#if __PYX_LIMITED_VERSION_HEX >= 0x030d0000
    (void) PyObject_GetOptionalAttr(obj, attr_name, &result);
    return result;
#else
#if CYTHON_COMPILING_IN_CPYTHON && CYTHON_USE_TYPE_SLOTS
    PyTypeObject* tp = Py_TYPE(obj);
    if (likely(tp->tp_getattro == PyObject_GenericGetAttr)) {
        return _PyObject_GenericGetAttrWithDict(obj, attr_name, NULL, 1);
    }
#endif
    result = __Pyx_PyObject_GetAttrStr(obj, attr_name);
    if (unlikely(!result)) {
        __Pyx_PyObject_GetAttrStr_ClearAttributeError();
    }
    return result;
#endif
}

/* HasAttr */
#if __PYX_LIMITED_VERSION_HEX < 0x030d0000
static CYTHON_INLINE int __Pyx_HasAttr(PyObject *o, PyObject *n) {
    PyObject *r;
    if (unlikely(!PyUnicode_Check(n))) {
        PyErr_SetString(PyExc_TypeError,
                        "hasattr(): attribute name must be string");
        return -1;
    }
    r = __Pyx_PyObject_GetAttrStrNoError(o, n);
    if (!r) {
        return (unlikely(PyErr_Occurred())) ? -1 : 0;
    } else {
        Py_DECREF(r);
        return 1;
    }
}
#endif

/* PyObjectCall (used by PyObjectFastCall) */
#if CYTHON_COMPILING_IN_CPYTHON
static CYTHON_INLINE PyObject* __Pyx_PyObject_Call(PyObject *func, PyObject *arg, PyObject *kw) {
    PyObject *result;
    ternaryfunc call = Py_TYPE(func)->tp_call;
    if (unlikely(!call))
        return PyObject_Call(func, arg, kw);
    if (unlikely(Py_EnterRecursiveCall(" while calling a Python object")))
        return NULL;
    result = (*call)(func, arg, kw);
    Py_LeaveRecursiveCall();
    if (unlikely(!result) && unlikely(!PyErr_Occurred())) {
        PyErr_SetString(
            PyExc_SystemError,
            "NULL result without error in PyObject_Call");
    }
    return result;
}
#endif

/* PyObjectCallMethO (used by PyObjectFastCall) */
#if CYTHON_COMPILING_IN_CPYTHON
static CYTHON_INLINE PyObject* __Pyx_PyObject_CallMethO(PyObject *func, PyObject *arg) {
    PyObject *self, *result;
    PyCFunction cfunc;
    cfunc = __Pyx_CyOrPyCFunction_GET_FUNCTION(func);
    self = __Pyx_CyOrPyCFunction_GET_SELF(func);
    if (unlikely(Py_EnterRecursiveCall(" while calling a Python object")))
        return NULL;
    result = cfunc(self, arg);
    Py_LeaveRecursiveCall();
    if (unlikely(!result) && unlikely(!PyErr_Occurred())) {
        PyErr_SetString(
            PyExc_SystemError,
            "NULL result without error in PyObject_Call");
    }
    return result;
}
#endif

/* PyObjectFastCall (used by PyObjectFastCallMethod) */
#if PY_VERSION_HEX < 0x03090000 || CYTHON_COMPILING_IN_LIMITED_API
static PyObject* __Pyx_PyObject_FastCall_fallback(PyObject *func, PyObject * const*args, size_t nargs, PyObject *kwargs) {
    PyObject *argstuple;
    PyObject *result = 0;
    size_t i;
    argstuple = PyTuple_New((Py_ssize_t)nargs);
    if (unlikely(!argstuple)) return NULL;
    for (i = 0; i < nargs; i++) {
        Py_INCREF(args[i]);
        if (__Pyx_PyTuple_SET_ITEM(argstuple, (Py_ssize_t)i, args[i]) != (0)) goto bad;
    }
    result = __Pyx_PyObject_Call(func, argstuple, kwargs);
  bad:
    Py_DECREF(argstuple);
    return result;
}
#endif
#if CYTHON_VECTORCALL && !CYTHON_COMPILING_IN_LIMITED_API
  #if PY_VERSION_HEX < 0x03090000
    #define __Pyx_PyVectorcall_Function(callable) _PyVectorcall_Function(callable)
  #elif CYTHON_COMPILING_IN_CPYTHON
static CYTHON_INLINE vectorcallfunc __Pyx_PyVectorcall_Function(PyObject *callable) {
    PyTypeObject *tp = Py_TYPE(callable);
    #if defined(__Pyx_CyFunction_USED)
    if (__Pyx_CyFunction_CheckExact(callable)) {
        return __Pyx_CyFunction_func_vectorcall(callable);
    }
    #endif
    if (!PyType_HasFeature(tp, Py_TPFLAGS_HAVE_VECTORCALL)) {
        return NULL;
    }
    assert(PyCallable_Check(callable));
    Py_ssize_t offset = tp->tp_vectorcall_offset;
    assert(offset > 0);
    vectorcallfunc ptr;
    memcpy(&ptr, (char *) callable + offset, sizeof(ptr));
    return ptr;
}
  #else
    #define __Pyx_PyVectorcall_Function(callable) PyVectorcall_Function(callable)
  #endif
#endif
static CYTHON_INLINE PyObject* __Pyx_PyObject_FastCallDict(PyObject *func, PyObject *const *args, size_t _nargs, PyObject *kwargs) {
    Py_ssize_t nargs = __Pyx_PyVectorcall_NARGS(_nargs);
#if CYTHON_COMPILING_IN_CPYTHON
    if (nargs == 0 && kwargs == NULL) {
        if (__Pyx_CyOrPyCFunction_Check(func) && likely( __Pyx_CyOrPyCFunction_GET_FLAGS(func) & METH_NOARGS))
            return __Pyx_PyObject_CallMethO(func, NULL);
    }
    else if (nargs == 1 && kwargs == NULL) {
        if (__Pyx_CyOrPyCFunction_Check(func) && likely( __Pyx_CyOrPyCFunction_GET_FLAGS(func) & METH_O))
            return __Pyx_PyObject_CallMethO(func, args[0]);
    }
#endif
    if (kwargs == NULL) {
        #if CYTHON_VECTORCALL
          #if CYTHON_COMPILING_IN_LIMITED_API
            return PyObject_Vectorcall(func, args, _nargs, NULL);
          #else
            vectorcallfunc f = __Pyx_PyVectorcall_Function(func);
            if (f) {
                return f(func, args, _nargs, NULL);
            }
          #endif
        #endif
    }
    if (nargs == 0) {
        return __Pyx_PyObject_Call(func, __pyx_mstate_global->__pyx_empty_tuple, kwargs);
    }
    #if PY_VERSION_HEX >= 0x03090000 && !CYTHON_COMPILING_IN_LIMITED_API
    return PyObject_VectorcallDict(func, args, (size_t)nargs, kwargs);
    #else
    return __Pyx_PyObject_FastCall_fallback(func, args, (size_t)nargs, kwargs);
    #endif
}

/* PyObjectFastCallMethod */
#if !CYTHON_VECTORCALL || PY_VERSION_HEX < 0x03090000
static PyObject *__Pyx_PyObject_FastCallMethod(PyObject *name, PyObject *const *args, size_t nargsf) {
    PyObject *result;
    PyObject *attr = PyObject_GetAttr(args[0], name);
    if (unlikely(!attr))
        return NULL;
    result = __Pyx_PyObject_FastCall(attr, args+1, nargsf - 1);
    Py_DECREF(attr);
    return result;
}
#endif

/* ExtTypeTest */
static CYTHON_INLINE int __Pyx_TypeTest(PyObject *obj, PyTypeObject *type) {
    __Pyx_TypeName obj_type_name;
    __Pyx_TypeName type_name;
    if (unlikely(!type)) {
        PyErr_SetString(PyExc_SystemError, "Missing type object");
        return 0;
    }
    if (likely(__Pyx_TypeCheck(obj, type)))
        return 1;
    obj_type_name = __Pyx_PyType_GetFullyQualifiedName(Py_TYPE(obj));
    type_name = __Pyx_PyType_GetFullyQualifiedName(type);
    PyErr_Format(PyExc_TypeError,
                 "Cannot convert " __Pyx_FMT_TYPENAME " to " __Pyx_FMT_TYPENAME,
                 obj_type_name, type_name);
    __Pyx_DECREF_TypeName(obj_type_name);
    __Pyx_DECREF_TypeName(type_name);
    return 0;
}

/* JoinPyUnicode */
static PyObject* __Pyx_PyUnicode_Join(PyObject** values, Py_ssize_t value_count, Py_ssize_t result_ulength,
                                      Py_UCS4 max_char) {
#if CYTHON_USE_UNICODE_INTERNALS && CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
    PyObject *result_uval;
    int result_ukind, kind_shift;
    Py_ssize_t i, char_pos;
    void *result_udata;
    if (max_char > 1114111) max_char = 1114111;
    result_uval = PyUnicode_New(result_ulength, max_char);
    if (unlikely(!result_uval)) return NULL;
    result_ukind = (max_char <= 255) ? PyUnicode_1BYTE_KIND : (max_char <= 65535) ? PyUnicode_2BYTE_KIND : PyUnicode_4BYTE_KIND;
    kind_shift = (result_ukind == PyUnicode_4BYTE_KIND) ? 2 : result_ukind - 1;
    result_udata = PyUnicode_DATA(result_uval);
    assert(kind_shift == 2 || kind_shift == 1 || kind_shift == 0);
    if (unlikely((PY_SSIZE_T_MAX >> kind_shift) - result_ulength < 0))
        goto overflow;
    char_pos = 0;
    for (i=0; i < value_count; i++) {
        int ukind;
        Py_ssize_t ulength;
        void *udata;
        PyObject *uval = values[i];
        #if !CYTHON_COMPILING_IN_LIMITED_API
        if (__Pyx_PyUnicode_READY(uval) == (-1))
            goto bad;
        #endif
        ulength = __Pyx_PyUnicode_GET_LENGTH(uval);
        #if !CYTHON_ASSUME_SAFE_SIZE
        if (unlikely(ulength < 0)) goto bad;
        #endif
        if (unlikely(!ulength))
            continue;
        if (unlikely((PY_SSIZE_T_MAX >> kind_shift) - ulength < char_pos))
            goto overflow;
        ukind = __Pyx_PyUnicode_KIND(uval);
        udata = __Pyx_PyUnicode_DATA(uval);
        if (ukind == result_ukind) {
            memcpy((char *)result_udata + (char_pos << kind_shift), udata, (size_t) (ulength << kind_shift));
        } else {
            #if PY_VERSION_HEX >= 0x030d0000
            if (unlikely(PyUnicode_CopyCharacters(result_uval, char_pos, uval, 0, ulength) < 0)) goto bad;
            #elif CYTHON_COMPILING_IN_CPYTHON || defined(_PyUnicode_FastCopyCharacters)
            _PyUnicode_FastCopyCharacters(result_uval, char_pos, uval, 0, ulength);
            #else
            Py_ssize_t j;
            for (j=0; j < ulength; j++) {
                Py_UCS4 uchar = __Pyx_PyUnicode_READ(ukind, udata, j);
                __Pyx_PyUnicode_WRITE(result_ukind, result_udata, char_pos+j, uchar);
            }
            #endif
        }
        char_pos += ulength;
    }
    return result_uval;
overflow:
    PyErr_SetString(PyExc_OverflowError, "join() result is too long for a Python string");
bad:
    Py_DECREF(result_uval);
    return NULL;
#else
    Py_ssize_t i;
    PyObject *result = NULL;
    PyObject *value_tuple = PyTuple_New(value_count);
    if (unlikely(!value_tuple)) return NULL;
    CYTHON_UNUSED_VAR(max_char);
    CYTHON_UNUSED_VAR(result_ulength);
    for (i=0; i<value_count; i++) {
        Py_INCREF(values[i]);
        if (__Pyx_PyTuple_SET_ITEM(value_tuple, i, values[i]) != (0)) goto bad;
    }
    result = PyUnicode_Join(__pyx_mstate_global->__pyx_empty_unicode, value_tuple);
bad:
    Py_DECREF(value_tuple);
    return result;
#endif
}

/* RaiseException */
static void __Pyx_Raise(PyObject *type, PyObject *value, PyObject *tb, PyObject *cause) {
    PyObject* owned_instance = NULL;
    if (tb == Py_None) {
        tb = 0;
    } else if (tb && !PyTraceBack_Check(tb)) {
        PyErr_SetString(PyExc_TypeError,
            "raise: arg 3 must be a traceback or None");
        goto bad;
    }
    if (value == Py_None)
        value = 0;
    if (PyExceptionInstance_Check(type)) {
        if (value) {
            PyErr_SetString(PyExc_TypeError,
                "instance exception may not have a separate value");
            goto bad;
        }
        value = type;
        type = (PyObject*) Py_TYPE(value);
    } else if (PyExceptionClass_Check(type)) {
        PyObject *instance_class = NULL;
        if (value && PyExceptionInstance_Check(value)) {
            instance_class = (PyObject*) Py_TYPE(value);
            if (instance_class != type) {
                int is_subclass = PyObject_IsSubclass(instance_class, type);
                if (!is_subclass) {
                    instance_class = NULL;
                } else if (unlikely(is_subclass == -1)) {
                    goto bad;
                } else {
                    type = instance_class;
                }
            }
        }
        if (!instance_class) {
            PyObject *args;
            if (!value)
                args = PyTuple_New(0);
            else if (PyTuple_Check(value)) {
                Py_INCREF(value);
                args = value;
            } else
                args = PyTuple_Pack(1, value);
            if (!args)
                goto bad;
            owned_instance = PyObject_Call(type, args, NULL);
            Py_DECREF(args);
            if (!owned_instance)
                goto bad;
            value = owned_instance;
            if (!PyExceptionInstance_Check(value)) {
                PyErr_Format(PyExc_TypeError,
                             "calling %R should have returned an instance of "
                             "BaseException, not %R",
                             type, Py_TYPE(value));
                goto bad;
            }
        }
    } else {
        PyErr_SetString(PyExc_TypeError,
            "raise: exception class must be a subclass of BaseException");
        goto bad;
    }
    if (cause) {
        PyObject *fixed_cause;
        if (cause == Py_None) {
            fixed_cause = NULL;
        } else if (PyExceptionClass_Check(cause)) {
            fixed_cause = PyObject_CallObject(cause, NULL);
            if (fixed_cause == NULL)
                goto bad;
        } else if (PyExceptionInstance_Check(cause)) {
            fixed_cause = cause;
            Py_INCREF(fixed_cause);
        } else {
            PyErr_SetString(PyExc_TypeError,
                            "exception causes must derive from "
                            "BaseException");
            goto bad;
        }
        PyException_SetCause(value, fixed_cause);
    }
    PyErr_SetObject(type, value);
    if (tb) {
#if PY_VERSION_HEX >= 0x030C00A6
        PyException_SetTraceback(value, tb);
#elif CYTHON_FAST_THREAD_STATE
        PyThreadState *tstate = __Pyx_PyThreadState_Current;
        PyObject* tmp_tb = tstate->curexc_traceback;
        if (tb != tmp_tb) {
            Py_INCREF(tb);
            tstate->curexc_traceback = tb;
            Py_XDECREF(tmp_tb);
        }
#else
        PyObject *tmp_type, *tmp_value, *tmp_tb;
        PyErr_Fetch(&tmp_type, &tmp_value, &tmp_tb);
        Py_INCREF(tb);
        PyErr_Restore(tmp_type, tmp_value, tb);
        Py_XDECREF(tmp_tb);
#endif
    }
bad:
    Py_XDECREF(owned_instance);
    return;
}

/* TupleAndListFromArray (used by fastcall) */
#if !CYTHON_COMPILING_IN_CPYTHON && CYTHON_METH_FASTCALL
static CYTHON_INLINE PyObject *
__Pyx_PyTuple_FromArray(PyObject *const *src, Py_ssize_t n)
{
    PyObject *res;
    Py_ssize_t i;
    if (n <= 0) {
        return __Pyx_NewRef(__pyx_mstate_global->__pyx_empty_tuple);
    }
    res = PyTuple_New(n);
    if (unlikely(res == NULL)) return NULL;
    for (i = 0; i < n; i++) {
        Py_INCREF(src[i]);
        if (unlikely(__Pyx_PyTuple_SET_ITEM(res, i, src[i]) < (0))) {
            Py_DECREF(res);
            return NULL;
        }
    }
    return res;
}
#elif CYTHON_COMPILING_IN_CPYTHON
static CYTHON_INLINE void __Pyx_copy_object_array(PyObject *const *CYTHON_RESTRICT src, PyObject** CYTHON_RESTRICT dest, Py_ssize_t length) {
    PyObject *v;
    Py_ssize_t i;
    for (i = 0; i < length; i++) {
        v = dest[i] = src[i];
        Py_INCREF(v);
    }
}
static CYTHON_INLINE PyObject *
__Pyx_PyTuple_FromArray(PyObject *const *src, Py_ssize_t n)
{
    PyObject *res;
    if (n <= 0) {
        return __Pyx_NewRef(__pyx_mstate_global->__pyx_empty_tuple);
    }
    res = PyTuple_New(n);
    if (unlikely(res == NULL)) return NULL;
    __Pyx_copy_object_array(src, ((PyTupleObject*)res)->ob_item, n);
    return res;
}
static CYTHON_INLINE PyObject *
__Pyx_PyList_FromArray(PyObject *const *src, Py_ssize_t n)
{
    PyObject *res;
    if (n <= 0) {
        return PyList_New(0);
    }
    res = PyList_New(n);
    if (unlikely(res == NULL)) return NULL;
    __Pyx_copy_object_array(src, ((PyListObject*)res)->ob_item, n);
    return res;
}
#endif

/* BytesEquals (used by UnicodeEquals) */
static CYTHON_INLINE int __Pyx_PyBytes_Equals(PyObject* s1, PyObject* s2, int equals) {
#if CYTHON_COMPILING_IN_PYPY || CYTHON_COMPILING_IN_LIMITED_API || CYTHON_COMPILING_IN_GRAAL ||\
        !(CYTHON_ASSUME_SAFE_SIZE && CYTHON_ASSUME_SAFE_MACROS)
    return PyObject_RichCompareBool(s1, s2, equals);
#else
    if (s1 == s2) {
        return (equals == Py_EQ);
    } else if (PyBytes_CheckExact(s1) & PyBytes_CheckExact(s2)) {
        const char *ps1, *ps2;
        Py_ssize_t length = PyBytes_GET_SIZE(s1);
        if (length != PyBytes_GET_SIZE(s2))
            return (equals == Py_NE);
        ps1 = PyBytes_AS_STRING(s1);
        ps2 = PyBytes_AS_STRING(s2);
        if (ps1[0] != ps2[0]) {
            return (equals == Py_NE);
        } else if (length == 1) {
            return (equals == Py_EQ);
        } else {
            int result;
#if CYTHON_USE_UNICODE_INTERNALS && (PY_VERSION_HEX < 0x030B0000)
            Py_hash_t hash1, hash2;
            hash1 = ((PyBytesObject*)s1)->ob_shash;
            hash2 = ((PyBytesObject*)s2)->ob_shash;
            if (hash1 != hash2 && hash1 != -1 && hash2 != -1) {
                return (equals == Py_NE);
            }
#endif
            result = memcmp(ps1, ps2, (size_t)length);
            return (equals == Py_EQ) ? (result == 0) : (result != 0);
        }
    } else if ((s1 == Py_None) & PyBytes_CheckExact(s2)) {
        return (equals == Py_NE);
    } else if ((s2 == Py_None) & PyBytes_CheckExact(s1)) {
        return (equals == Py_NE);
    } else {
        int result;
        PyObject* py_result = PyObject_RichCompare(s1, s2, equals);
        if (!py_result)
            return -1;
        result = __Pyx_PyObject_IsTrue(py_result);
        Py_DECREF(py_result);
        return result;
    }
#endif
}

/* UnicodeEquals (used by fastcall) */
static CYTHON_INLINE int __Pyx_PyUnicode_Equals(PyObject* s1, PyObject* s2, int equals) {
#if CYTHON_COMPILING_IN_PYPY || CYTHON_COMPILING_IN_LIMITED_API || CYTHON_COMPILING_IN_GRAAL
    return PyObject_RichCompareBool(s1, s2, equals);
#else
    int s1_is_unicode, s2_is_unicode;
    if (s1 == s2) {
        goto return_eq;
    }
    s1_is_unicode = PyUnicode_CheckExact(s1);
    s2_is_unicode = PyUnicode_CheckExact(s2);
    if (s1_is_unicode & s2_is_unicode) {
        Py_ssize_t length, length2;
        int kind;
        void *data1, *data2;
        #if !CYTHON_COMPILING_IN_LIMITED_API
        if (unlikely(__Pyx_PyUnicode_READY(s1) < 0) || unlikely(__Pyx_PyUnicode_READY(s2) < 0))
            return -1;
        #endif
        length = __Pyx_PyUnicode_GET_LENGTH(s1);
        #if !CYTHON_ASSUME_SAFE_SIZE
        if (unlikely(length < 0)) return -1;
        #endif
        length2 = __Pyx_PyUnicode_GET_LENGTH(s2);
        #if !CYTHON_ASSUME_SAFE_SIZE
        if (unlikely(length2 < 0)) return -1;
        #endif
        if (length != length2) {
            goto return_ne;
        }
#if CYTHON_USE_UNICODE_INTERNALS
        {
            Py_hash_t hash1, hash2;
            hash1 = ((PyASCIIObject*)s1)->hash;
            hash2 = ((PyASCIIObject*)s2)->hash;
            if (hash1 != hash2 && hash1 != -1 && hash2 != -1) {
                goto return_ne;
            }
        }
#endif
        kind = __Pyx_PyUnicode_KIND(s1);
        if (kind != __Pyx_PyUnicode_KIND(s2)) {
            goto return_ne;
        }
        data1 = __Pyx_PyUnicode_DATA(s1);
        data2 = __Pyx_PyUnicode_DATA(s2);
        if (__Pyx_PyUnicode_READ(kind, data1, 0) != __Pyx_PyUnicode_READ(kind, data2, 0)) {
            goto return_ne;
        } else if (length == 1) {
            goto return_eq;
        } else {
            int result = memcmp(data1, data2, (size_t)(length * kind));
            return (equals == Py_EQ) ? (result == 0) : (result != 0);
        }
    } else if ((s1 == Py_None) & s2_is_unicode) {
        goto return_ne;
    } else if ((s2 == Py_None) & s1_is_unicode) {
        goto return_ne;
    } else {
        int result;
        PyObject* py_result = PyObject_RichCompare(s1, s2, equals);
        if (!py_result)
            return -1;
        result = __Pyx_PyObject_IsTrue(py_result);
        Py_DECREF(py_result);
        return result;
    }
return_eq:
    return (equals == Py_EQ);
return_ne:
    return (equals == Py_NE);
#endif
}

/* fastcall */
#if CYTHON_METH_FASTCALL
static CYTHON_INLINE PyObject * __Pyx_GetKwValue_FASTCALL(PyObject *kwnames, PyObject *const *kwvalues, PyObject *s)
{
    Py_ssize_t i, n = __Pyx_PyTuple_GET_SIZE(kwnames);
    #if !CYTHON_ASSUME_SAFE_SIZE
    if (unlikely(n == -1)) return NULL;
    #endif
    for (i = 0; i < n; i++)
    {
        PyObject *namei = __Pyx_PyTuple_GET_ITEM(kwnames, i);
        #if !CYTHON_ASSUME_SAFE_MACROS
        if (unlikely(!namei)) return NULL;
        #endif
        if (s == namei) return kwvalues[i];
    }
    for (i = 0; i < n; i++)
    {
        PyObject *namei = __Pyx_PyTuple_GET_ITEM(kwnames, i);
        #if !CYTHON_ASSUME_SAFE_MACROS
        if (unlikely(!namei)) return NULL;
        #endif
        int eq = __Pyx_PyUnicode_Equals(s, namei, Py_EQ);
        if (unlikely(eq != 0)) {
            if (unlikely(eq < 0)) return NULL;
            return kwvalues[i];
        }
    }
    return NULL;
}
#if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x030d0000 || CYTHON_COMPILING_IN_LIMITED_API
CYTHON_UNUSED static PyObject *__Pyx_KwargsAsDict_FASTCALL(PyObject *kwnames, PyObject *const *kwvalues) {
    Py_ssize_t i, nkwargs;
    PyObject *dict;
#if !CYTHON_ASSUME_SAFE_SIZE
    nkwargs = PyTuple_Size(kwnames);
    if (unlikely(nkwargs < 0)) return NULL;
#else
    nkwargs = PyTuple_GET_SIZE(kwnames);
#endif
    dict = PyDict_New();
    if (unlikely(!dict))
        return NULL;
    for (i=0; i<nkwargs; i++) {
#if !CYTHON_ASSUME_SAFE_MACROS
        PyObject *key = PyTuple_GetItem(kwnames, i);
        if (!key) goto bad;
#else
        PyObject *key = PyTuple_GET_ITEM(kwnames, i);
#endif
        if (unlikely(PyDict_SetItem(dict, key, kwvalues[i]) < 0))
            goto bad;
    }
    return dict;
bad:
    Py_DECREF(dict);
    return NULL;
}
#endif
#endif

/* PyObjectCallOneArg (used by CallUnboundCMethod0) */
static CYTHON_INLINE PyObject* __Pyx_PyObject_CallOneArg(PyObject *func, PyObject *arg) {
    PyObject *args[2] = {NULL, arg};
    return __Pyx_PyObject_FastCall(func, args+1, 1 | __Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET);
}

/* UnpackUnboundCMethod (used by CallUnboundCMethod0) */
#if CYTHON_COMPILING_IN_LIMITED_API && __PYX_LIMITED_VERSION_HEX < 0x030C0000
static PyObject *__Pyx_SelflessCall(PyObject *method, PyObject *args, PyObject *kwargs) {
    PyObject *result;
    PyObject *selfless_args = PyTuple_GetSlice(args, 1, PyTuple_Size(args));
    if (unlikely(!selfless_args)) return NULL;
    result = PyObject_Call(method, selfless_args, kwargs);
    Py_DECREF(selfless_args);
    return result;
}
#elif CYTHON_COMPILING_IN_PYPY && PY_VERSION_HEX < 0x03090000
static PyObject *__Pyx_SelflessCall(PyObject *method, PyObject **args, Py_ssize_t nargs, PyObject *kwnames) {
        return _PyObject_Vectorcall
            (method, args ? args+1 : NULL, nargs ? nargs-1 : 0, kwnames);
}
#else
static PyObject *__Pyx_SelflessCall(PyObject *method, PyObject *const *args, Py_ssize_t nargs, PyObject *kwnames) {
    return
#if PY_VERSION_HEX < 0x03090000
    _PyObject_Vectorcall
#else
    PyObject_Vectorcall
#endif
        (method, args ? args+1 : NULL, nargs ? (size_t) nargs-1 : 0, kwnames);
}
#endif
static PyMethodDef __Pyx_UnboundCMethod_Def = {
     "CythonUnboundCMethod",
     __PYX_REINTERPRET_FUNCION(PyCFunction, __Pyx_SelflessCall),
#if CYTHON_COMPILING_IN_LIMITED_API && __PYX_LIMITED_VERSION_HEX < 0x030C0000
     METH_VARARGS | METH_KEYWORDS,
#else
     METH_FASTCALL | METH_KEYWORDS,
#endif
     NULL
};
static int __Pyx_TryUnpackUnboundCMethod(__Pyx_CachedCFunction* target) {
    PyObject *method, *result=NULL;
    method = __Pyx_PyObject_GetAttrStr(target->type, *target->method_name);
    if (unlikely(!method))
        return -1;
    result = method;
#if CYTHON_COMPILING_IN_CPYTHON
    if (likely(__Pyx_TypeCheck(method, &PyMethodDescr_Type)))
    {
        PyMethodDescrObject *descr = (PyMethodDescrObject*) method;
        target->func = descr->d_method->ml_meth;
        target->flag = descr->d_method->ml_flags & ~(METH_CLASS | METH_STATIC | METH_COEXIST | METH_STACKLESS);
    } else
#endif
#if CYTHON_COMPILING_IN_PYPY
#else
    if (PyCFunction_Check(method))
#endif
    {
        PyObject *self;
        int self_found;
#if CYTHON_COMPILING_IN_LIMITED_API || CYTHON_COMPILING_IN_PYPY
        self = PyObject_GetAttrString(method, "__self__");
        if (!self) {
            PyErr_Clear();
        }
#else
        self = PyCFunction_GET_SELF(method);
#endif
        self_found = (self && self != Py_None);
#if CYTHON_COMPILING_IN_LIMITED_API || CYTHON_COMPILING_IN_PYPY
        Py_XDECREF(self);
#endif
        if (self_found) {
            PyObject *unbound_method = PyCFunction_New(&__Pyx_UnboundCMethod_Def, method);
            if (unlikely(!unbound_method)) return -1;
            Py_DECREF(method);
            result = unbound_method;
        }
    }
#if !CYTHON_COMPILING_IN_CPYTHON_FREETHREADING
    if (unlikely(target->method)) {
        Py_DECREF(result);
    } else
#endif
    target->method = result;
    return 0;
}

/* CallUnboundCMethod0 */
#if CYTHON_COMPILING_IN_CPYTHON
static CYTHON_INLINE PyObject* __Pyx_CallUnboundCMethod0(__Pyx_CachedCFunction* cfunc, PyObject* self) {
    int was_initialized = __Pyx_CachedCFunction_GetAndSetInitializing(cfunc);
    if (likely(was_initialized == 2 && cfunc->func)) {
        if (likely(cfunc->flag == METH_NOARGS))
            return __Pyx_CallCFunction(cfunc, self, NULL);
        if (likely(cfunc->flag == METH_FASTCALL))
            return __Pyx_CallCFunctionFast(cfunc, self, NULL, 0);
        if (cfunc->flag == (METH_FASTCALL | METH_KEYWORDS))
            return __Pyx_CallCFunctionFastWithKeywords(cfunc, self, NULL, 0, NULL);
        if (likely(cfunc->flag == (METH_VARARGS | METH_KEYWORDS)))
            return __Pyx_CallCFunctionWithKeywords(cfunc, self, __pyx_mstate_global->__pyx_empty_tuple, NULL);
        if (cfunc->flag == METH_VARARGS)
            return __Pyx_CallCFunction(cfunc, self, __pyx_mstate_global->__pyx_empty_tuple);
        return __Pyx__CallUnboundCMethod0(cfunc, self);
    }
#if CYTHON_COMPILING_IN_CPYTHON_FREETHREADING
    else if (unlikely(was_initialized == 1)) {
        __Pyx_CachedCFunction tmp_cfunc = {
#ifndef __cplusplus
            0
#endif
        };
        tmp_cfunc.type = cfunc->type;
        tmp_cfunc.method_name = cfunc->method_name;
        return __Pyx__CallUnboundCMethod0(&tmp_cfunc, self);
    }
#endif
    PyObject *result = __Pyx__CallUnboundCMethod0(cfunc, self);
    __Pyx_CachedCFunction_SetFinishedInitializing(cfunc);
    return result;
}
#endif
static PyObject* __Pyx__CallUnboundCMethod0(__Pyx_CachedCFunction* cfunc, PyObject* self) {
    PyObject *result;
    if (unlikely(!cfunc->method) && unlikely(__Pyx_TryUnpackUnboundCMethod(cfunc) < 0)) return NULL;
    result = __Pyx_PyObject_CallOneArg(cfunc->method, self);
    return result;
}

/* py_dict_items (used by OwnedDictNext) */
static CYTHON_INLINE PyObject* __Pyx_PyDict_Items(PyObject* d) {
    return __Pyx_CallUnboundCMethod0(&__pyx_mstate_global->__pyx_umethod_PyDict_Type_items, d);
}

/* py_dict_values (used by OwnedDictNext) */
static CYTHON_INLINE PyObject* __Pyx_PyDict_Values(PyObject* d) {
    return __Pyx_CallUnboundCMethod0(&__pyx_mstate_global->__pyx_umethod_PyDict_Type_values, d);
}

/* OwnedDictNext (used by ParseKeywordsImpl) */
#if CYTHON_AVOID_BORROWED_REFS
static int __Pyx_PyDict_NextRef(PyObject *p, PyObject **ppos, PyObject **pkey, PyObject **pvalue) {
    PyObject *next = NULL;
    if (!*ppos) {
        if (pvalue) {
            PyObject *dictview = pkey ? __Pyx_PyDict_Items(p) : __Pyx_PyDict_Values(p);
            if (unlikely(!dictview)) goto bad;
            *ppos = PyObject_GetIter(dictview);
            Py_DECREF(dictview);
        } else {
            *ppos = PyObject_GetIter(p);
        }
        if (unlikely(!*ppos)) goto bad;
    }
    next = PyIter_Next(*ppos);
    if (!next) {
        if (PyErr_Occurred()) goto bad;
        return 0;
    }
    if (pkey && pvalue) {
        *pkey = __Pyx_PySequence_ITEM(next, 0);
        if (unlikely(*pkey)) goto bad;
        *pvalue = __Pyx_PySequence_ITEM(next, 1);
        if (unlikely(*pvalue)) goto bad;
        Py_DECREF(next);
    } else if (pkey) {
        *pkey = next;
    } else {
        assert(pvalue);
        *pvalue = next;
    }
    return 1;
  bad:
    Py_XDECREF(next);
#if !CYTHON_COMPILING_IN_LIMITED_API && PY_VERSION_HEX >= 0x030d0000
    PyErr_FormatUnraisable("Exception ignored in __Pyx_PyDict_NextRef");
#else
    PyErr_WriteUnraisable(__pyx_mstate_global->__pyx_n_u_Pyx_PyDict_NextRef);
#endif
    if (pkey) *pkey = NULL;
    if (pvalue) *pvalue = NULL;
    return 0;
}
#else // !CYTHON_AVOID_BORROWED_REFS
static int __Pyx_PyDict_NextRef(PyObject *p, Py_ssize_t *ppos, PyObject **pkey, PyObject **pvalue) {
    int result = PyDict_Next(p, ppos, pkey, pvalue);
    if (likely(result == 1)) {
        if (pkey) Py_INCREF(*pkey);
        if (pvalue) Py_INCREF(*pvalue);
    }
    return result;
}
#endif

/* RaiseDoubleKeywords (used by ParseKeywordsImpl) */
static void __Pyx_RaiseDoubleKeywordsError(
    const char* func_name,
    PyObject* kw_name)
{
    PyErr_Format(PyExc_TypeError,
        "%s() got multiple values for keyword argument '%U'", func_name, kw_name);
}

/* CallUnboundCMethod2 */
#if CYTHON_COMPILING_IN_CPYTHON
static CYTHON_INLINE PyObject *__Pyx_CallUnboundCMethod2(__Pyx_CachedCFunction *cfunc, PyObject *self, PyObject *arg1, PyObject *arg2) {
    int was_initialized = __Pyx_CachedCFunction_GetAndSetInitializing(cfunc);
    if (likely(was_initialized == 2 && cfunc->func)) {
        PyObject *args[2] = {arg1, arg2};
        if (cfunc->flag == METH_FASTCALL) {
            return __Pyx_CallCFunctionFast(cfunc, self, args, 2);
        }
        if (cfunc->flag == (METH_FASTCALL | METH_KEYWORDS))
            return __Pyx_CallCFunctionFastWithKeywords(cfunc, self, args, 2, NULL);
    }
#if CYTHON_COMPILING_IN_CPYTHON_FREETHREADING
    else if (unlikely(was_initialized == 1)) {
        __Pyx_CachedCFunction tmp_cfunc = {
#ifndef __cplusplus
            0
#endif
        };
        tmp_cfunc.type = cfunc->type;
        tmp_cfunc.method_name = cfunc->method_name;
        return __Pyx__CallUnboundCMethod2(&tmp_cfunc, self, arg1, arg2);
    }
#endif
    PyObject *result = __Pyx__CallUnboundCMethod2(cfunc, self, arg1, arg2);
    __Pyx_CachedCFunction_SetFinishedInitializing(cfunc);
    return result;
}
#endif
static PyObject* __Pyx__CallUnboundCMethod2(__Pyx_CachedCFunction* cfunc, PyObject* self, PyObject* arg1, PyObject* arg2){
    if (unlikely(!cfunc->func && !cfunc->method) && unlikely(__Pyx_TryUnpackUnboundCMethod(cfunc) < 0)) return NULL;
#if CYTHON_COMPILING_IN_CPYTHON
    if (cfunc->func && (cfunc->flag & METH_VARARGS)) {
        PyObject *result = NULL;
        PyObject *args = PyTuple_New(2);
        if (unlikely(!args)) return NULL;
        Py_INCREF(arg1);
        PyTuple_SET_ITEM(args, 0, arg1);
        Py_INCREF(arg2);
        PyTuple_SET_ITEM(args, 1, arg2);
        if (cfunc->flag & METH_KEYWORDS)
            result = __Pyx_CallCFunctionWithKeywords(cfunc, self, args, NULL);
        else
            result = __Pyx_CallCFunction(cfunc, self, args);
        Py_DECREF(args);
        return result;
    }
#endif
    {
        PyObject *args[4] = {NULL, self, arg1, arg2};
        return __Pyx_PyObject_FastCall(cfunc->method, args+1, 3 | __Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET);
    }
}

/* ParseKeywordsImpl (used by ParseKeywords) */
static int __Pyx_ValidateDuplicatePosArgs(
    PyObject *kwds,
    PyObject ** const argnames[],
    PyObject ** const *first_kw_arg,
    const char* function_name)
{
    PyObject ** const *name = argnames;
    while (name != first_kw_arg) {
        PyObject *key = **name;
        int found = PyDict_Contains(kwds, key);
        if (unlikely(found)) {
            if (found == 1) __Pyx_RaiseDoubleKeywordsError(function_name, key);
            goto bad;
        }
        name++;
    }
    return 0;
bad:
    return -1;
}
#if CYTHON_USE_UNICODE_INTERNALS
static CYTHON_INLINE int __Pyx_UnicodeKeywordsEqual(PyObject *s1, PyObject *s2) {
    int kind;
    Py_ssize_t len = PyUnicode_GET_LENGTH(s1);
    if (len != PyUnicode_GET_LENGTH(s2)) return 0;
    kind = PyUnicode_KIND(s1);
    if (kind != PyUnicode_KIND(s2)) return 0;
    const void *data1 = PyUnicode_DATA(s1);
    const void *data2 = PyUnicode_DATA(s2);
    return (memcmp(data1, data2, (size_t) len * (size_t) kind) == 0);
}
#endif
static int __Pyx_MatchKeywordArg_str(
    PyObject *key,
    PyObject ** const argnames[],
    PyObject ** const *first_kw_arg,
    size_t *index_found,
    const char *function_name)
{
    PyObject ** const *name;
    #if CYTHON_USE_UNICODE_INTERNALS
    Py_hash_t key_hash = ((PyASCIIObject*)key)->hash;
    if (unlikely(key_hash == -1)) {
        key_hash = PyObject_Hash(key);
        if (unlikely(key_hash == -1))
            goto bad;
    }
    #endif
    name = first_kw_arg;
    while (*name) {
        PyObject *name_str = **name;
        #if CYTHON_USE_UNICODE_INTERNALS
        if (key_hash == ((PyASCIIObject*)name_str)->hash && __Pyx_UnicodeKeywordsEqual(name_str, key)) {
            *index_found = (size_t) (name - argnames);
            return 1;
        }
        #else
        #if CYTHON_ASSUME_SAFE_SIZE
        if (PyUnicode_GET_LENGTH(name_str) == PyUnicode_GET_LENGTH(key))
        #endif
        {
            int cmp = PyUnicode_Compare(name_str, key);
            if (cmp < 0 && unlikely(PyErr_Occurred())) goto bad;
            if (cmp == 0) {
                *index_found = (size_t) (name - argnames);
                return 1;
            }
        }
        #endif
        name++;
    }
    name = argnames;
    while (name != first_kw_arg) {
        PyObject *name_str = **name;
        #if CYTHON_USE_UNICODE_INTERNALS
        if (unlikely(key_hash == ((PyASCIIObject*)name_str)->hash)) {
            if (__Pyx_UnicodeKeywordsEqual(name_str, key))
                goto arg_passed_twice;
        }
        #else
        #if CYTHON_ASSUME_SAFE_SIZE
        if (PyUnicode_GET_LENGTH(name_str) == PyUnicode_GET_LENGTH(key))
        #endif
        {
            if (unlikely(name_str == key)) goto arg_passed_twice;
            int cmp = PyUnicode_Compare(name_str, key);
            if (cmp < 0 && unlikely(PyErr_Occurred())) goto bad;
            if (cmp == 0) goto arg_passed_twice;
        }
        #endif
        name++;
    }
    return 0;
arg_passed_twice:
    __Pyx_RaiseDoubleKeywordsError(function_name, key);
    goto bad;
bad:
    return -1;
}
static int __Pyx_MatchKeywordArg_nostr(
    PyObject *key,
    PyObject ** const argnames[],
    PyObject ** const *first_kw_arg,
    size_t *index_found,
    const char *function_name)
{
    PyObject ** const *name;
    if (unlikely(!PyUnicode_Check(key))) goto invalid_keyword_type;
    name = first_kw_arg;
    while (*name) {
        int cmp = PyObject_RichCompareBool(**name, key, Py_EQ);
        if (cmp == 1) {
            *index_found = (size_t) (name - argnames);
            return 1;
        }
        if (unlikely(cmp == -1)) goto bad;
        name++;
    }
    name = argnames;
    while (name != first_kw_arg) {
        int cmp = PyObject_RichCompareBool(**name, key, Py_EQ);
        if (unlikely(cmp != 0)) {
            if (cmp == 1) goto arg_passed_twice;
            else goto bad;
        }
        name++;
    }
    return 0;
arg_passed_twice:
    __Pyx_RaiseDoubleKeywordsError(function_name, key);
    goto bad;
invalid_keyword_type:
    PyErr_Format(PyExc_TypeError,
        "%.200s() keywords must be strings", function_name);
    goto bad;
bad:
    return -1;
}
static CYTHON_INLINE int __Pyx_MatchKeywordArg(
    PyObject *key,
    PyObject ** const argnames[],
    PyObject ** const *first_kw_arg,
    size_t *index_found,
    const char *function_name)
{
    return likely(PyUnicode_CheckExact(key)) ?
        __Pyx_MatchKeywordArg_str(key, argnames, first_kw_arg, index_found, function_name) :
        __Pyx_MatchKeywordArg_nostr(key, argnames, first_kw_arg, index_found, function_name);
}
static void __Pyx_RejectUnknownKeyword(
    PyObject *kwds,
    PyObject ** const argnames[],
    PyObject ** const *first_kw_arg,
    const char *function_name)
{
    #if CYTHON_AVOID_BORROWED_REFS
    PyObject *pos = NULL;
    #else
    Py_ssize_t pos = 0;
    #endif
    PyObject *key = NULL;
    __Pyx_BEGIN_CRITICAL_SECTION(kwds);
    while (
        #if CYTHON_AVOID_BORROWED_REFS
        __Pyx_PyDict_NextRef(kwds, &pos, &key, NULL)
        #else
        PyDict_Next(kwds, &pos, &key, NULL)
        #endif
    ) {
        PyObject** const *name = first_kw_arg;
        while (*name && (**name != key)) name++;
        if (!*name) {
            size_t index_found = 0;
            int cmp = __Pyx_MatchKeywordArg(key, argnames, first_kw_arg, &index_found, function_name);
            if (cmp != 1) {
                if (cmp == 0) {
                    PyErr_Format(PyExc_TypeError,
                        "%s() got an unexpected keyword argument '%U'",
                        function_name, key);
                }
                #if CYTHON_AVOID_BORROWED_REFS
                Py_DECREF(key);
                #endif
                break;
            }
        }
        #if CYTHON_AVOID_BORROWED_REFS
        Py_DECREF(key);
        #endif
    }
    __Pyx_END_CRITICAL_SECTION();
    #if CYTHON_AVOID_BORROWED_REFS
    Py_XDECREF(pos);
    #endif
    assert(PyErr_Occurred());
}
static int __Pyx_ParseKeywordDict(
    PyObject *kwds,
    PyObject ** const argnames[],
    PyObject *values[],
    Py_ssize_t num_pos_args,
    Py_ssize_t num_kwargs,
    const char* function_name,
    int ignore_unknown_kwargs)
{
    PyObject** const *name;
    PyObject** const *first_kw_arg = argnames + num_pos_args;
    Py_ssize_t extracted = 0;
#if !CYTHON_COMPILING_IN_PYPY || defined(PyArg_ValidateKeywordArguments)
    if (unlikely(!PyArg_ValidateKeywordArguments(kwds))) return -1;
#endif
    name = first_kw_arg;
    while (*name && num_kwargs > extracted) {
        PyObject * key = **name;
        PyObject *value;
        int found = 0;
        #if __PYX_LIMITED_VERSION_HEX >= 0x030d0000
        found = PyDict_GetItemRef(kwds, key, &value);
        #else
        value = PyDict_GetItemWithError(kwds, key);
        if (value) {
            Py_INCREF(value);
            found = 1;
        } else {
            if (unlikely(PyErr_Occurred())) goto bad;
        }
        #endif
        if (found) {
            if (unlikely(found < 0)) goto bad;
            values[name-argnames] = value;
            extracted++;
        }
        name++;
    }
    if (num_kwargs > extracted) {
        if (ignore_unknown_kwargs) {
            if (unlikely(__Pyx_ValidateDuplicatePosArgs(kwds, argnames, first_kw_arg, function_name) == -1))
                goto bad;
        } else {
            __Pyx_RejectUnknownKeyword(kwds, argnames, first_kw_arg, function_name);
            goto bad;
        }
    }
    return 0;
bad:
    return -1;
}
static int __Pyx_ParseKeywordDictToDict(
    PyObject *kwds,
    PyObject ** const argnames[],
    PyObject *kwds2,
    PyObject *values[],
    Py_ssize_t num_pos_args,
    const char* function_name)
{
    PyObject** const *name;
    PyObject** const *first_kw_arg = argnames + num_pos_args;
    Py_ssize_t len;
#if !CYTHON_COMPILING_IN_PYPY || defined(PyArg_ValidateKeywordArguments)
    if (unlikely(!PyArg_ValidateKeywordArguments(kwds))) return -1;
#endif
    if (PyDict_Update(kwds2, kwds) < 0) goto bad;
    name = first_kw_arg;
    while (*name) {
        PyObject *key = **name;
        PyObject *value;
#if !CYTHON_COMPILING_IN_LIMITED_API && (PY_VERSION_HEX >= 0x030d00A2 || defined(PyDict_Pop))
        int found = PyDict_Pop(kwds2, key, &value);
        if (found) {
            if (unlikely(found < 0)) goto bad;
            values[name-argnames] = value;
        }
#elif __PYX_LIMITED_VERSION_HEX >= 0x030d0000
        int found = PyDict_GetItemRef(kwds2, key, &value);
        if (found) {
            if (unlikely(found < 0)) goto bad;
            values[name-argnames] = value;
            if (unlikely(PyDict_DelItem(kwds2, key) < 0)) goto bad;
        }
#else
    #if CYTHON_COMPILING_IN_CPYTHON
        value = _PyDict_Pop(kwds2, key, kwds2);
    #else
        value = __Pyx_CallUnboundCMethod2(&__pyx_mstate_global->__pyx_umethod_PyDict_Type_pop, kwds2, key, kwds2);
    #endif
        if (value == kwds2) {
            Py_DECREF(value);
        } else {
            if (unlikely(!value)) goto bad;
            values[name-argnames] = value;
        }
#endif
        name++;
    }
    len = PyDict_Size(kwds2);
    if (len > 0) {
        return __Pyx_ValidateDuplicatePosArgs(kwds, argnames, first_kw_arg, function_name);
    } else if (unlikely(len == -1)) {
        goto bad;
    }
    return 0;
bad:
    return -1;
}
static int __Pyx_ParseKeywordsTuple(
    PyObject *kwds,
    PyObject * const *kwvalues,
    PyObject ** const argnames[],
    PyObject *kwds2,
    PyObject *values[],
    Py_ssize_t num_pos_args,
    Py_ssize_t num_kwargs,
    const char* function_name,
    int ignore_unknown_kwargs)
{
    PyObject *key = NULL;
    PyObject** const * name;
    PyObject** const *first_kw_arg = argnames + num_pos_args;
    for (Py_ssize_t pos = 0; pos < num_kwargs; pos++) {
#if CYTHON_AVOID_BORROWED_REFS
        key = __Pyx_PySequence_ITEM(kwds, pos);
#else
        key = __Pyx_PyTuple_GET_ITEM(kwds, pos);
#endif
#if !CYTHON_ASSUME_SAFE_MACROS
        if (unlikely(!key)) goto bad;
#endif
        name = first_kw_arg;
        while (*name && (**name != key)) name++;
        if (*name) {
            PyObject *value = kwvalues[pos];
            values[name-argnames] = __Pyx_NewRef(value);
        } else {
            size_t index_found = 0;
            int cmp = __Pyx_MatchKeywordArg(key, argnames, first_kw_arg, &index_found, function_name);
            if (cmp == 1) {
                PyObject *value = kwvalues[pos];
                values[index_found] = __Pyx_NewRef(value);
            } else {
                if (unlikely(cmp == -1)) goto bad;
                if (kwds2) {
                    PyObject *value = kwvalues[pos];
                    if (unlikely(PyDict_SetItem(kwds2, key, value))) goto bad;
                } else if (!ignore_unknown_kwargs) {
                    goto invalid_keyword;
                }
            }
        }
        #if CYTHON_AVOID_BORROWED_REFS
        Py_DECREF(key);
        key = NULL;
        #endif
    }
    return 0;
invalid_keyword:
    PyErr_Format(PyExc_TypeError,
        "%s() got an unexpected keyword argument '%U'",
        function_name, key);
    goto bad;
bad:
    #if CYTHON_AVOID_BORROWED_REFS
    Py_XDECREF(key);
    #endif
    return -1;
}

/* ParseKeywords */
static int __Pyx_ParseKeywords(
    PyObject *kwds,
    PyObject * const *kwvalues,
    PyObject ** const argnames[],
    PyObject *kwds2,
    PyObject *values[],
    Py_ssize_t num_pos_args,
    Py_ssize_t num_kwargs,
    const char* function_name,
    int ignore_unknown_kwargs)
{
    if (CYTHON_METH_FASTCALL && likely(PyTuple_Check(kwds)))
        return __Pyx_ParseKeywordsTuple(kwds, kwvalues, argnames, kwds2, values, num_pos_args, num_kwargs, function_name, ignore_unknown_kwargs);
    else if (kwds2)
        return __Pyx_ParseKeywordDictToDict(kwds, argnames, kwds2, values, num_pos_args, function_name);
    else
        return __Pyx_ParseKeywordDict(kwds, argnames, values, num_pos_args, num_kwargs, function_name, ignore_unknown_kwargs);
}

/* RaiseArgTupleInvalid */
static void __Pyx_RaiseArgtupleInvalid(
    const char* func_name,
    int exact,
    Py_ssize_t num_min,
    Py_ssize_t num_max,
    Py_ssize_t num_found)
{
    Py_ssize_t num_expected;
    const char *more_or_less;
    if (num_found < num_min) {
        num_expected = num_min;
        more_or_less = "at least";
    } else {
        num_expected = num_max;
        more_or_less = "at most";
    }
    if (exact) {
        more_or_less = "exactly";
    }
    PyErr_Format(PyExc_TypeError,
                 "%.200s() takes %.8s %" CYTHON_FORMAT_SSIZE_T "d positional argument%.1s (%" CYTHON_FORMAT_SSIZE_T "d given)",
                 func_name, more_or_less, num_expected,
                 (num_expected == 1) ? "" : "s", num_found);
}

/* GetBuiltinName (used by GetModuleGlobalName) */
static PyObject *__Pyx_GetBuiltinName(PyObject *name) {
    PyObject* result = __Pyx_PyObject_GetAttrStrNoError(__pyx_mstate_global->__pyx_b, name);
    if (unlikely(!result) && !PyErr_Occurred()) {
        PyErr_Format(PyExc_NameError,
            "name '%U' is not defined", name);
    }
    return result;
}

/* PyDictVersioning (used by GetModuleGlobalName) */
#if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_TYPE_SLOTS
static CYTHON_INLINE PY_UINT64_T __Pyx_get_tp_dict_version(PyObject *obj) {
    PyObject *dict = Py_TYPE(obj)->tp_dict;
    return likely(dict) ? __PYX_GET_DICT_VERSION(dict) : 0;
}
static CYTHON_INLINE PY_UINT64_T __Pyx_get_object_dict_version(PyObject *obj) {
    PyObject **dictptr = NULL;
    Py_ssize_t offset = Py_TYPE(obj)->tp_dictoffset;
    if (offset) {
#if CYTHON_COMPILING_IN_CPYTHON
        dictptr = (likely(offset > 0)) ? (PyObject **) ((char *)obj + offset) : _PyObject_GetDictPtr(obj);
#else
        dictptr = _PyObject_GetDictPtr(obj);
#endif
    }
    return (dictptr && *dictptr) ? __PYX_GET_DICT_VERSION(*dictptr) : 0;
}
static CYTHON_INLINE int __Pyx_object_dict_version_matches(PyObject* obj, PY_UINT64_T tp_dict_version, PY_UINT64_T obj_dict_version) {
    PyObject *dict = Py_TYPE(obj)->tp_dict;
    if (unlikely(!dict) || unlikely(tp_dict_version != __PYX_GET_DICT_VERSION(dict)))
        return 0;
    return obj_dict_version == __Pyx_get_object_dict_version(obj);
}
#endif

/* GetModuleGlobalName */
#if CYTHON_USE_DICT_VERSIONS
static PyObject *__Pyx__GetModuleGlobalName(PyObject *name, PY_UINT64_T *dict_version, PyObject **dict_cached_value)
#else
static CYTHON_INLINE PyObject *__Pyx__GetModuleGlobalName(PyObject *name)
#endif
{
    PyObject *result;
#if CYTHON_COMPILING_IN_LIMITED_API
    if (unlikely(!__pyx_m)) {
        if (!PyErr_Occurred())
            PyErr_SetNone(PyExc_NameError);
        return NULL;
    }
    result = PyObject_GetAttr(__pyx_m, name);
    if (likely(result)) {
        return result;
    }
    PyErr_Clear();
#elif CYTHON_AVOID_BORROWED_REFS || CYTHON_AVOID_THREAD_UNSAFE_BORROWED_REFS
    if (unlikely(__Pyx_PyDict_GetItemRef(__pyx_mstate_global->__pyx_d, name, &result) == -1)) PyErr_Clear();
    __PYX_UPDATE_DICT_CACHE(__pyx_mstate_global->__pyx_d, result, *dict_cached_value, *dict_version)
    if (likely(result)) {
        return result;
    }
#else
    result = _PyDict_GetItem_KnownHash(__pyx_mstate_global->__pyx_d, name, ((PyASCIIObject *) name)->hash);
    __PYX_UPDATE_DICT_CACHE(__pyx_mstate_global->__pyx_d, result, *dict_cached_value, *dict_version)
    if (likely(result)) {
        return __Pyx_NewRef(result);
    }
    PyErr_Clear();
#endif
    return __Pyx_GetBuiltinName(name);
}

/* ObjectGetItem */
#if CYTHON_USE_TYPE_SLOTS
static PyObject *__Pyx_PyObject_GetIndex(PyObject *obj, PyObject *index) {
    PyObject *runerr = NULL;
    Py_ssize_t key_value;
    key_value = __Pyx_PyIndex_AsSsize_t(index);
    if (likely(key_value != -1 || !(runerr = PyErr_Occurred()))) {
        return __Pyx_GetItemInt_Fast(obj, key_value, 0, 1, 1, 1);
    }
    if (PyErr_GivenExceptionMatches(runerr, PyExc_OverflowError)) {
        __Pyx_TypeName index_type_name = __Pyx_PyType_GetFullyQualifiedName(Py_TYPE(index));
        PyErr_Clear();
        PyErr_Format(PyExc_IndexError,
            "cannot fit '" __Pyx_FMT_TYPENAME "' into an index-sized integer", index_type_name);
        __Pyx_DECREF_TypeName(index_type_name);
    }
    return NULL;
}
static PyObject *__Pyx_PyObject_GetItem_Slow(PyObject *obj, PyObject *key) {
    __Pyx_TypeName obj_type_name;
    if (likely(PyType_Check(obj))) {
        PyObject *meth = __Pyx_PyObject_GetAttrStrNoError(obj, __pyx_mstate_global->__pyx_n_u_class_getitem);
        if (!meth) {
            PyErr_Clear();
        } else {
            PyObject *result = __Pyx_PyObject_CallOneArg(meth, key);
            Py_DECREF(meth);
            return result;
        }
    }
    obj_type_name = __Pyx_PyType_GetFullyQualifiedName(Py_TYPE(obj));
    PyErr_Format(PyExc_TypeError,
        "'" __Pyx_FMT_TYPENAME "' object is not subscriptable", obj_type_name);
    __Pyx_DECREF_TypeName(obj_type_name);
    return NULL;
}
static PyObject *__Pyx_PyObject_GetItem(PyObject *obj, PyObject *key) {
    PyTypeObject *tp = Py_TYPE(obj);
    PyMappingMethods *mm = tp->tp_as_mapping;
    PySequenceMethods *sm = tp->tp_as_sequence;
    if (likely(mm && mm->mp_subscript)) {
        return mm->mp_subscript(obj, key);
    }
    if (likely(sm && sm->sq_item)) {
        return __Pyx_PyObject_GetIndex(obj, key);
    }
    return __Pyx_PyObject_GetItem_Slow(obj, key);
}
#endif

/* RaiseTooManyValuesToUnpack */
static CYTHON_INLINE void __Pyx_RaiseTooManyValuesError(Py_ssize_t expected) {
    PyErr_Format(PyExc_ValueError,
                 "too many values to unpack (expected %" CYTHON_FORMAT_SSIZE_T "d)", expected);
}

/* RaiseNeedMoreValuesToUnpack */
static CYTHON_INLINE void __Pyx_RaiseNeedMoreValuesError(Py_ssize_t index) {
    PyErr_Format(PyExc_ValueError,
                 "need more than %" CYTHON_FORMAT_SSIZE_T "d value%.1s to unpack",
                 index, (index == 1) ? "" : "s");
}

/* IterFinish */
static CYTHON_INLINE int __Pyx_IterFinish(void) {
    PyObject* exc_type;
    __Pyx_PyThreadState_declare
    __Pyx_PyThreadState_assign
    exc_type = __Pyx_PyErr_CurrentExceptionType();
    if (unlikely(exc_type)) {
        if (unlikely(!__Pyx_PyErr_GivenExceptionMatches(exc_type, PyExc_StopIteration)))
            return -1;
        __Pyx_PyErr_Clear();
        return 0;
    }
    return 0;
}

/* UnpackItemEndCheck */
static int __Pyx_IternextUnpackEndCheck(PyObject *retval, Py_ssize_t expected) {
    if (unlikely(retval)) {
        Py_DECREF(retval);
        __Pyx_RaiseTooManyValuesError(expected);
        return -1;
    }
    return __Pyx_IterFinish();
}

/* PyLongCompare */
static CYTHON_INLINE int __Pyx_PyLong_BoolEqObjC(PyObject *op1, PyObject *op2, long intval, long inplace) {
    CYTHON_MAYBE_UNUSED_VAR(intval);
    CYTHON_UNUSED_VAR(inplace);
    if (op1 == op2) {
        return 1;
    }
    #if CYTHON_USE_PYLONG_INTERNALS
    if (likely(PyLong_CheckExact(op1))) {
        int unequal;
        unsigned long uintval;
        Py_ssize_t size = __Pyx_PyLong_DigitCount(op1);
        const digit* digits = __Pyx_PyLong_Digits(op1);
        if (intval == 0) {
            return (__Pyx_PyLong_IsZero(op1) == 1);
        } else if (intval < 0) {
            if (__Pyx_PyLong_IsNonNeg(op1))
                return 0;
            intval = -intval;
        } else {
            if (__Pyx_PyLong_IsNeg(op1))
                return 0;
        }
        uintval = (unsigned long) intval;
#if PyLong_SHIFT * 4 < SIZEOF_LONG*8
        if (uintval >> (PyLong_SHIFT * 4)) {
            unequal = (size != 5) || (digits[0] != (uintval & (unsigned long) PyLong_MASK))
                 | (digits[1] != ((uintval >> (1 * PyLong_SHIFT)) & (unsigned long) PyLong_MASK)) | (digits[2] != ((uintval >> (2 * PyLong_SHIFT)) & (unsigned long) PyLong_MASK)) | (digits[3] != ((uintval >> (3 * PyLong_SHIFT)) & (unsigned long) PyLong_MASK)) | (digits[4] != ((uintval >> (4 * PyLong_SHIFT)) & (unsigned long) PyLong_MASK));
        } else
#endif
#if PyLong_SHIFT * 3 < SIZEOF_LONG*8
        if (uintval >> (PyLong_SHIFT * 3)) {
            unequal = (size != 4) || (digits[0] != (uintval & (unsigned long) PyLong_MASK))
                 | (digits[1] != ((uintval >> (1 * PyLong_SHIFT)) & (unsigned long) PyLong_MASK)) | (digits[2] != ((uintval >> (2 * PyLong_SHIFT)) & (unsigned long) PyLong_MASK)) | (digits[3] != ((uintval >> (3 * PyLong_SHIFT)) & (unsigned long) PyLong_MASK));
        } else
#endif
#if PyLong_SHIFT * 2 < SIZEOF_LONG*8
        if (uintval >> (PyLong_SHIFT * 2)) {
            unequal = (size != 3) || (digits[0] != (uintval & (unsigned long) PyLong_MASK))
                 | (digits[1] != ((uintval >> (1 * PyLong_SHIFT)) & (unsigned long) PyLong_MASK)) | (digits[2] != ((uintval >> (2 * PyLong_SHIFT)) & (unsigned long) PyLong_MASK));
        } else
#endif
#if PyLong_SHIFT * 1 < SIZEOF_LONG*8
        if (uintval >> (PyLong_SHIFT * 1)) {
            unequal = (size != 2) || (digits[0] != (uintval & (unsigned long) PyLong_MASK))
                 | (digits[1] != ((uintval >> (1 * PyLong_SHIFT)) & (unsigned long) PyLong_MASK));
        } else
#endif
            unequal = (size != 1) || (((unsigned long) digits[0]) != (uintval & (unsigned long) PyLong_MASK));
        return (unequal == 0);
    }
    #endif
    if (PyFloat_CheckExact(op1)) {
        const long b = intval;
        double a = __Pyx_PyFloat_AS_DOUBLE(op1);
        return ((double)a == (double)b);
    }
    return __Pyx_PyObject_IsTrueAndDecref(
        PyObject_RichCompare(op1, op2, Py_EQ));
}

/* PyObjectVectorCallKwBuilder */
#if CYTHON_VECTORCALL
static int __Pyx_VectorcallBuilder_AddArg(PyObject *key, PyObject *value, PyObject *builder, PyObject **args, int n) {
    (void)__Pyx_PyObject_FastCallDict;
    Py_INCREF(key);
    if (__Pyx_PyTuple_SET_ITEM(builder, n, key) != (0)) return -1;
    args[n] = value;
    return 0;
}
CYTHON_UNUSED static int __Pyx_VectorcallBuilder_AddArg_Check(PyObject *key, PyObject *value, PyObject *builder, PyObject **args, int n) {
    (void)__Pyx_VectorcallBuilder_AddArgStr;
    if (unlikely(!PyUnicode_Check(key))) {
        PyErr_SetString(PyExc_TypeError, "keywords must be strings");
        return -1;
    }
    return __Pyx_VectorcallBuilder_AddArg(key, value, builder, args, n);
}
static int __Pyx_VectorcallBuilder_AddArgStr(const char *key, PyObject *value, PyObject *builder, PyObject **args, int n) {
    PyObject *pyKey = PyUnicode_FromString(key);
    if (!pyKey) return -1;
    return __Pyx_VectorcallBuilder_AddArg(pyKey, value, builder, args, n);
}
#else // CYTHON_VECTORCALL
CYTHON_UNUSED static int __Pyx_VectorcallBuilder_AddArg_Check(PyObject *key, PyObject *value, PyObject *builder, CYTHON_UNUSED PyObject **args, CYTHON_UNUSED int n) {
    if (unlikely(!PyUnicode_Check(key))) {
        PyErr_SetString(PyExc_TypeError, "keywords must be strings");
        return -1;
    }
    return PyDict_SetItem(builder, key, value);
}
#endif

/* RaiseUnexpectedTypeError */
static int
__Pyx_RaiseUnexpectedTypeError(const char *expected, PyObject *obj)
{
    __Pyx_TypeName obj_type_name = __Pyx_PyType_GetFullyQualifiedName(Py_TYPE(obj));
    PyErr_Format(PyExc_TypeError, "Expected %s, got " __Pyx_FMT_TYPENAME,
                 expected, obj_type_name);
    __Pyx_DECREF_TypeName(obj_type_name);
    return 0;
}

/* PyObjectVectorCallMethodKwBuilder */
#if !CYTHON_VECTORCALL || PY_VERSION_HEX < 0x03090000
static PyObject *__Pyx_Object_VectorcallMethod_CallFromBuilder(PyObject *name, PyObject *const *args, size_t nargsf, PyObject *kwnames) {
    PyObject *result;
    PyObject *obj = PyObject_GetAttr(args[0], name);
    if (unlikely(!obj))
        return NULL;
    result = __Pyx_Object_Vectorcall_CallFromBuilder(obj, args+1, nargsf-1, kwnames);
    Py_DECREF(obj);
    return result;
}
#endif

/* SliceObject */
static CYTHON_INLINE PyObject* __Pyx_PyObject_GetSlice(PyObject* obj,
        Py_ssize_t cstart, Py_ssize_t cstop,
        PyObject** _py_start, PyObject** _py_stop, PyObject** _py_slice,
        int has_cstart, int has_cstop, CYTHON_UNUSED int wraparound) {
    __Pyx_TypeName obj_type_name;
#if CYTHON_USE_TYPE_SLOTS
    PyMappingMethods* mp = Py_TYPE(obj)->tp_as_mapping;
    if (likely(mp && mp->mp_subscript))
#endif
    {
        PyObject* result;
        PyObject *py_slice, *py_start, *py_stop;
        if (_py_slice) {
            py_slice = *_py_slice;
        } else {
            PyObject* owned_start = NULL;
            PyObject* owned_stop = NULL;
            if (_py_start) {
                py_start = *_py_start;
            } else {
                if (has_cstart) {
                    owned_start = py_start = PyLong_FromSsize_t(cstart);
                    if (unlikely(!py_start)) goto bad;
                } else
                    py_start = Py_None;
            }
            if (_py_stop) {
                py_stop = *_py_stop;
            } else {
                if (has_cstop) {
                    owned_stop = py_stop = PyLong_FromSsize_t(cstop);
                    if (unlikely(!py_stop)) {
                        Py_XDECREF(owned_start);
                        goto bad;
                    }
                } else
                    py_stop = Py_None;
            }
            py_slice = PySlice_New(py_start, py_stop, Py_None);
            Py_XDECREF(owned_start);
            Py_XDECREF(owned_stop);
            if (unlikely(!py_slice)) goto bad;
        }
#if CYTHON_USE_TYPE_SLOTS
        result = mp->mp_subscript(obj, py_slice);
#else
        result = PyObject_GetItem(obj, py_slice);
#endif
        if (!_py_slice) {
            Py_DECREF(py_slice);
        }
        return result;
    }
    obj_type_name = __Pyx_PyType_GetFullyQualifiedName(Py_TYPE(obj));
    PyErr_Format(PyExc_TypeError,
        "'" __Pyx_FMT_TYPENAME "' object is unsliceable", obj_type_name);
    __Pyx_DECREF_TypeName(obj_type_name);
bad:
    return NULL;
}

/* RaiseUnboundLocalError */
static void __Pyx_RaiseUnboundLocalError(const char *varname) {
    PyErr_Format(PyExc_UnboundLocalError, "local variable '%s' referenced before assignment", varname);
}

/* PyLongCompare */
static CYTHON_INLINE int __Pyx_PyLong_BoolNeObjC(PyObject *op1, PyObject *op2, long intval, long inplace) {
    CYTHON_MAYBE_UNUSED_VAR(intval);
    CYTHON_UNUSED_VAR(inplace);
    if (op1 == op2) {
        return 0;
    }
    #if CYTHON_USE_PYLONG_INTERNALS
    if (likely(PyLong_CheckExact(op1))) {
        int unequal;
        unsigned long uintval;
        Py_ssize_t size = __Pyx_PyLong_DigitCount(op1);
        const digit* digits = __Pyx_PyLong_Digits(op1);
        if (intval == 0) {
            return (__Pyx_PyLong_IsZero(op1) != 1);
        } else if (intval < 0) {
            if (__Pyx_PyLong_IsNonNeg(op1))
                return 1;
            intval = -intval;
        } else {
            if (__Pyx_PyLong_IsNeg(op1))
                return 1;
        }
        uintval = (unsigned long) intval;
#if PyLong_SHIFT * 4 < SIZEOF_LONG*8
        if (uintval >> (PyLong_SHIFT * 4)) {
            unequal = (size != 5) || (digits[0] != (uintval & (unsigned long) PyLong_MASK))
                 | (digits[1] != ((uintval >> (1 * PyLong_SHIFT)) & (unsigned long) PyLong_MASK)) | (digits[2] != ((uintval >> (2 * PyLong_SHIFT)) & (unsigned long) PyLong_MASK)) | (digits[3] != ((uintval >> (3 * PyLong_SHIFT)) & (unsigned long) PyLong_MASK)) | (digits[4] != ((uintval >> (4 * PyLong_SHIFT)) & (unsigned long) PyLong_MASK));
        } else
#endif
#if PyLong_SHIFT * 3 < SIZEOF_LONG*8
        if (uintval >> (PyLong_SHIFT * 3)) {
            unequal = (size != 4) || (digits[0] != (uintval & (unsigned long) PyLong_MASK))
                 | (digits[1] != ((uintval >> (1 * PyLong_SHIFT)) & (unsigned long) PyLong_MASK)) | (digits[2] != ((uintval >> (2 * PyLong_SHIFT)) & (unsigned long) PyLong_MASK)) | (digits[3] != ((uintval >> (3 * PyLong_SHIFT)) & (unsigned long) PyLong_MASK));
        } else
#endif
#if PyLong_SHIFT * 2 < SIZEOF_LONG*8
        if (uintval >> (PyLong_SHIFT * 2)) {
            unequal = (size != 3) || (digits[0] != (uintval & (unsigned long) PyLong_MASK))
                 | (digits[1] != ((uintval >> (1 * PyLong_SHIFT)) & (unsigned long) PyLong_MASK)) | (digits[2] != ((uintval >> (2 * PyLong_SHIFT)) & (unsigned long) PyLong_MASK));
        } else
#endif
#if PyLong_SHIFT * 1 < SIZEOF_LONG*8
        if (uintval >> (PyLong_SHIFT * 1)) {
            unequal = (size != 2) || (digits[0] != (uintval & (unsigned long) PyLong_MASK))
                 | (digits[1] != ((uintval >> (1 * PyLong_SHIFT)) & (unsigned long) PyLong_MASK));
        } else
#endif
            unequal = (size != 1) || (((unsigned long) digits[0]) != (uintval & (unsigned long) PyLong_MASK));
        return (unequal != 0);
    }
    #endif
    if (PyFloat_CheckExact(op1)) {
        const long b = intval;
        double a = __Pyx_PyFloat_AS_DOUBLE(op1);
        return ((double)a != (double)b);
    }
    return __Pyx_PyObject_IsTrueAndDecref(
        PyObject_RichCompare(op1, op2, Py_NE));
}

/* UnicodeConcatInPlace */
# if CYTHON_COMPILING_IN_CPYTHON
static int
__Pyx_unicode_modifiable(PyObject *unicode, int unsafe_shared)
{
    if (!__Pyx_IS_UNIQUELY_REFERENCED(unicode, unsafe_shared))
        return 0;
#if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX > 0x030F0000
    if (PyUnstable_Unicode_GET_CACHED_HASH(unicode) != -1)
        return 0;
#endif
    if (!PyUnicode_CheckExact(unicode))
        return 0;
    if (PyUnicode_CHECK_INTERNED(unicode))
        return 0;
    return 1;
}
static CYTHON_INLINE PyObject *__Pyx_PyUnicode_ConcatInPlaceImpl(PyObject **p_left, PyObject *right, int unsafe_shared
        #if CYTHON_REFNANNY
        , void* __pyx_refnanny
        #endif
    ) {
    PyObject *left = *p_left;
    Py_ssize_t left_len, right_len, new_len;
    if (unlikely(__Pyx_PyUnicode_READY(left) == -1))
        return NULL;
    if (unlikely(__Pyx_PyUnicode_READY(right) == -1))
        return NULL;
    left_len = PyUnicode_GET_LENGTH(left);
    if (left_len == 0) {
        Py_INCREF(right);
        return right;
    }
    right_len = PyUnicode_GET_LENGTH(right);
    if (right_len == 0) {
        Py_INCREF(left);
        return left;
    }
    if (unlikely(left_len > PY_SSIZE_T_MAX - right_len)) {
        PyErr_SetString(PyExc_OverflowError,
                        "strings are too large to concat");
        return NULL;
    }
    new_len = left_len + right_len;
    if (__Pyx_unicode_modifiable(left, unsafe_shared)
            && PyUnicode_CheckExact(right)
            && PyUnicode_KIND(right) <= PyUnicode_KIND(left)
            && !(PyUnicode_IS_ASCII(left) && !PyUnicode_IS_ASCII(right))) {
        int ret;
        __Pyx_GIVEREF(*p_left);
        ret = PyUnicode_Resize(p_left, new_len);
        __Pyx_GOTREF(*p_left);
        if (unlikely(ret != 0))
            return NULL;
        #if PY_VERSION_HEX >= 0x030d0000
        if (unlikely(PyUnicode_CopyCharacters(*p_left, left_len, right, 0, right_len) < 0)) return NULL;
        #else
        _PyUnicode_FastCopyCharacters(*p_left, left_len, right, 0, right_len);
        #endif
        __Pyx_INCREF(*p_left);
        __Pyx_GIVEREF(*p_left);
        return *p_left;
    } else {
        return __Pyx_PyUnicode_Concat(left, right);
    }
  }
#endif

/* GetAttr */
static CYTHON_INLINE PyObject *__Pyx_GetAttr(PyObject *o, PyObject *n) {
#if CYTHON_USE_TYPE_SLOTS
    if (likely(PyUnicode_Check(n)))
        return __Pyx_PyObject_GetAttrStr(o, n);
#endif
    return PyObject_GetAttr(o, n);
}

/* TypeImport */
#ifndef __PYX_HAVE_RT_ImportType_3_2_5
#define __PYX_HAVE_RT_ImportType_3_2_5
static PyTypeObject *__Pyx_ImportType_3_2_5(PyObject *module, const char *module_name, const char *class_name,
    size_t size, size_t alignment, enum __Pyx_ImportType_CheckSize_3_2_5 check_size)
{
    PyObject *result = 0;
    Py_ssize_t basicsize;
    Py_ssize_t itemsize;
#if defined(Py_LIMITED_API) || (defined(CYTHON_COMPILING_IN_LIMITED_API) && CYTHON_COMPILING_IN_LIMITED_API)
    PyObject *py_basicsize;
    PyObject *py_itemsize;
#endif
    result = PyObject_GetAttrString(module, class_name);
    if (!result)
        goto bad;
    if (!PyType_Check(result)) {
        PyErr_Format(PyExc_TypeError,
            "%.200s.%.200s is not a type object",
            module_name, class_name);
        goto bad;
    }
#if !( defined(Py_LIMITED_API) || (defined(CYTHON_COMPILING_IN_LIMITED_API) && CYTHON_COMPILING_IN_LIMITED_API) )
    basicsize = ((PyTypeObject *)result)->tp_basicsize;
    itemsize = ((PyTypeObject *)result)->tp_itemsize;
#else
    if (size == 0) {
        return (PyTypeObject *)result;
    }
    py_basicsize = PyObject_GetAttrString(result, "__basicsize__");
    if (!py_basicsize)
        goto bad;
    basicsize = PyLong_AsSsize_t(py_basicsize);
    Py_DECREF(py_basicsize);
    py_basicsize = 0;
    if (basicsize == (Py_ssize_t)-1 && PyErr_Occurred())
        goto bad;
    py_itemsize = PyObject_GetAttrString(result, "__itemsize__");
    if (!py_itemsize)
        goto bad;
    itemsize = PyLong_AsSsize_t(py_itemsize);
    Py_DECREF(py_itemsize);
    py_itemsize = 0;
    if (itemsize == (Py_ssize_t)-1 && PyErr_Occurred())
        goto bad;
#endif
    if (itemsize) {
        if (size % alignment) {
            alignment = size % alignment;
        }
        if (itemsize < (Py_ssize_t)alignment)
            itemsize = (Py_ssize_t)alignment;
    }
    if ((size_t)(basicsize + itemsize) < size) {
        PyErr_Format(PyExc_ValueError,
            "%.200s.%.200s size changed, may indicate binary incompatibility. "
            "Expected %zd from C header, got %zd from PyObject",
            module_name, class_name, size, basicsize+itemsize);
        goto bad;
    }
    if (check_size == __Pyx_ImportType_CheckSize_Error_3_2_5 &&
            ((size_t)basicsize > size || (size_t)(basicsize + itemsize) < size)) {
        PyErr_Format(PyExc_ValueError,
            "%.200s.%.200s size changed, may indicate binary incompatibility. "
            "Expected %zd from C header, got %zd-%zd from PyObject",
            module_name, class_name, size, basicsize, basicsize+itemsize);
        goto bad;
    }
    else if (check_size == __Pyx_ImportType_CheckSize_Warn_3_2_5 && (size_t)basicsize > size) {
        if (PyErr_WarnFormat(NULL, 0,
                "%.200s.%.200s size changed, may indicate binary incompatibility. "
                "Expected %zd from C header, got %zd from PyObject",
                module_name, class_name, size, basicsize) < 0) {
            goto bad;
        }
    }
    return (PyTypeObject *)result;
bad:
    Py_XDECREF(result);
    return NULL;
}
#endif

/* GetVTable */
static void* __Pyx_GetVtable(PyTypeObject *type) {
    void* ptr;
#if CYTHON_COMPILING_IN_LIMITED_API
    PyObject *ob = PyObject_GetAttr((PyObject *)type, __pyx_mstate_global->__pyx_n_u_pyx_vtable);
#else
    PyObject *ob = PyObject_GetItem(type->tp_dict, __pyx_mstate_global->__pyx_n_u_pyx_vtable);
#endif
    if (!ob)
        goto bad;
    ptr = PyCapsule_GetPointer(ob, 0);
    if (!ptr && !PyErr_Occurred())
        PyErr_SetString(PyExc_RuntimeError, "invalid vtable found for imported type");
    Py_DECREF(ob);
    return ptr;
bad:
    Py_XDECREF(ob);
    return NULL;
}

/* PxdImportShared (used by VoidPtrImport) */
#ifndef __PYX_HAVE_RT_ImportFromPxd_3_2_5
#define __PYX_HAVE_RT_ImportFromPxd_3_2_5
static int __Pyx_ImportFromPxd_3_2_5(PyObject *module, const char *name, void **p, const char *sig, const char *what) {
    PyObject *d = 0;
    PyObject *cobj = 0;
    d = PyObject_GetAttrString(module, "__pyx_capi__");
    if (!d)
        goto bad;
#if (defined(Py_LIMITED_API) && Py_LIMITED_API >= 0x030d0000) || (!defined(Py_LIMITED_API) && PY_VERSION_HEX >= 0x030d0000)
    PyDict_GetItemStringRef(d, name, &cobj);
#else
    cobj = PyDict_GetItemString(d, name);
    Py_XINCREF(cobj);
#endif
    if (!cobj) {
        PyErr_Format(PyExc_ImportError,
            "%.200s does not export expected C %.8s %.200s",
                PyModule_GetName(module), what, name);
        goto bad;
    }
    if (!PyCapsule_IsValid(cobj, sig)) {
        PyErr_Format(PyExc_TypeError,
            "C %.8s %.200s.%.200s has wrong signature (expected %.500s, got %.500s)",
             what, PyModule_GetName(module), name, sig, PyCapsule_GetName(cobj));
        goto bad;
    }
    *p = PyCapsule_GetPointer(cobj, sig);
    if (!(*p))
        goto bad;
    Py_DECREF(d);
    Py_DECREF(cobj);
    return 0;
bad:
    Py_XDECREF(d);
    Py_XDECREF(cobj);
    return -1;
}
#endif

/* VoidPtrImport */
#ifndef __PYX_HAVE_RT_ImportVoidPtr_3_2_5
#define __PYX_HAVE_RT_ImportVoidPtr_3_2_5
static int __Pyx_ImportVoidPtr_3_2_5(PyObject *module, const char *name, void **p, const char *sig) {
    return __Pyx_ImportFromPxd_3_2_5(module, name, p, sig, "variable");
}
#endif

/* FunctionImport */
#ifndef __PYX_HAVE_RT_ImportFunction_3_2_5
#define __PYX_HAVE_RT_ImportFunction_3_2_5
static int __Pyx_ImportFunction_3_2_5(PyObject *module, const char *funcname, void (**f)(void), const char *sig) {
    union {
        void (*fp)(void);
        void *p;
    } tmp;
    int result = __Pyx_ImportFromPxd_3_2_5(module, funcname, &tmp.p, sig, "function");
    if (result == 0) {
        *f = tmp.fp;
    }
    return result;
}
#endif

/* ImportImpl (used by Import) */
static int __Pyx__Import_GetModule(PyObject *qualname, PyObject **module) {
    PyObject *imported_module = PyImport_GetModule(qualname);
    if (unlikely(!imported_module)) {
        *module = NULL;
        if (PyErr_Occurred()) {
            return -1;
        }
        return 0;
    }
    *module = imported_module;
    return 1;
}
static int __Pyx__Import_Lookup(PyObject *qualname, PyObject *const *imported_names, Py_ssize_t len_imported_names, PyObject **module) {
    PyObject *imported_module;
    PyObject *top_level_package_name;
    Py_ssize_t i;
    int status, module_found;
    Py_ssize_t dot_index;
    module_found = __Pyx__Import_GetModule(qualname, &imported_module);
    if (unlikely(!module_found || module_found == -1)) {
        *module = NULL;
        return module_found;
    }
    if (imported_names) {
        for (i = 0; i < len_imported_names; i++) {
            PyObject *imported_name = imported_names[i];
#if __PYX_LIMITED_VERSION_HEX < 0x030d0000
            int has_imported_attribute = PyObject_HasAttr(imported_module, imported_name);
#else
            int has_imported_attribute = PyObject_HasAttrWithError(imported_module, imported_name);
            if (unlikely(has_imported_attribute == -1)) goto error;
#endif
            if (!has_imported_attribute) {
                goto not_found;
            }
        }
        *module = imported_module;
        return 1;
    }
    dot_index = PyUnicode_FindChar(qualname, '.', 0, PY_SSIZE_T_MAX, 1);
    if (dot_index == -1) {
        *module = imported_module;
        return 1;
    }
    if (unlikely(dot_index == -2)) goto error;
    top_level_package_name = PyUnicode_Substring(qualname, 0, dot_index);
    if (unlikely(!top_level_package_name)) goto error;
    Py_DECREF(imported_module);
    status = __Pyx__Import_GetModule(top_level_package_name, module);
    Py_DECREF(top_level_package_name);
    return status;
error:
    Py_DECREF(imported_module);
    *module = NULL;
    return -1;
not_found:
    Py_DECREF(imported_module);
    *module = NULL;
    return 0;
}
static PyObject *__Pyx__Import(PyObject *name, PyObject *const *imported_names, Py_ssize_t len_imported_names, PyObject *qualname, PyObject *moddict, int level) {
    PyObject *module = 0;
    PyObject *empty_dict = 0;
    PyObject *from_list = 0;
    int module_found;
    if (!qualname) {
        qualname = name;
    }
    module_found = __Pyx__Import_Lookup(qualname, imported_names, len_imported_names, &module);
    if (likely(module_found == 1)) {
        return module;
    } else if (unlikely(module_found == -1)) {
        return NULL;
    }
    empty_dict = PyDict_New();
    if (unlikely(!empty_dict))
        goto bad;
    if (imported_names) {
#if CYTHON_COMPILING_IN_CPYTHON
        from_list = __Pyx_PyList_FromArray(imported_names, len_imported_names);
        if (unlikely(!from_list))
            goto bad;
#else
        from_list = PyList_New(len_imported_names);
        if (unlikely(!from_list)) goto bad;
        for (Py_ssize_t i=0; i<len_imported_names; ++i) {
            if (PyList_SetItem(from_list, i, __Pyx_NewRef(imported_names[i])) < 0) goto bad;
        }
#endif
    }
    if (level == -1) {
        const char* package_sep = strchr(__Pyx_MODULE_NAME, '.');
        if (package_sep != (0)) {
            module = PyImport_ImportModuleLevelObject(
                name, moddict, empty_dict, from_list, 1);
            if (unlikely(!module)) {
                if (unlikely(!PyErr_ExceptionMatches(PyExc_ImportError)))
                    goto bad;
                PyErr_Clear();
            }
        }
        level = 0;
    }
    if (!module) {
        module = PyImport_ImportModuleLevelObject(
            name, moddict, empty_dict, from_list, level);
    }
bad:
    Py_XDECREF(from_list);
    Py_XDECREF(empty_dict);
    return module;
}

/* Import */
static PyObject *__Pyx_Import(PyObject *name, PyObject *const *imported_names, Py_ssize_t len_imported_names, PyObject *qualname, int level) {
    return __Pyx__Import(name, imported_names, len_imported_names, qualname, __pyx_mstate_global->__pyx_d, level);
}

/* ImportFrom */
static PyObject* __Pyx_ImportFrom(PyObject* module, PyObject* name) {
    PyObject* value = __Pyx_PyObject_GetAttrStr(module, name);
    if (unlikely(!value) && PyErr_ExceptionMatches(PyExc_AttributeError)) {
        const char* module_name_str = 0;
        PyObject* module_name = 0;
        PyObject* module_dot = 0;
        PyObject* full_name = 0;
        PyErr_Clear();
        module_name_str = PyModule_GetName(module);
        if (unlikely(!module_name_str)) { goto modbad; }
        module_name = PyUnicode_FromString(module_name_str);
        if (unlikely(!module_name)) { goto modbad; }
        module_dot = PyUnicode_Concat(module_name, __pyx_mstate_global->__pyx_kp_u__2);
        if (unlikely(!module_dot)) { goto modbad; }
        full_name = PyUnicode_Concat(module_dot, name);
        if (unlikely(!full_name)) { goto modbad; }
        #if (CYTHON_COMPILING_IN_PYPY && PYPY_VERSION_NUM  < 0x07030400) ||\
                CYTHON_COMPILING_IN_GRAAL
        {
            PyObject *modules = PyImport_GetModuleDict();
            if (unlikely(!modules))
                goto modbad;
            value = PyObject_GetItem(modules, full_name);
        }
        #else
        value = PyImport_GetModule(full_name);
        #endif
      modbad:
        Py_XDECREF(full_name);
        Py_XDECREF(module_dot);
        Py_XDECREF(module_name);
    }
    if (unlikely(!value)) {
        PyErr_Format(PyExc_ImportError, "cannot import name %S", name);
    }
    return value;
}

/* dict_setdefault (used by CLineInTraceback) */
static CYTHON_INLINE PyObject *__Pyx_PyDict_SetDefault(PyObject *d, PyObject *key, PyObject *default_value) {
    PyObject* value;
#if __PYX_LIMITED_VERSION_HEX >= 0x030F0000 || (!CYTHON_COMPILING_IN_LIMITED_API && PY_VERSION_HEX >= 0x030d00A4)
    PyDict_SetDefaultRef(d, key, default_value, &value);
#elif CYTHON_COMPILING_IN_LIMITED_API && __PYX_LIMITED_VERSION_HEX >= 0x030C0000
    PyObject *args[] = {d, key, default_value};
    value = PyObject_VectorcallMethod(__pyx_mstate_global->__pyx_n_u_setdefault, args, 3 | PY_VECTORCALL_ARGUMENTS_OFFSET, NULL);
#elif CYTHON_COMPILING_IN_LIMITED_API
    value = PyObject_CallMethodObjArgs(d, __pyx_mstate_global->__pyx_n_u_setdefault, key, default_value, NULL);
#else
    value = PyDict_SetDefault(d, key, default_value);
    if (unlikely(!value)) return NULL;
    Py_INCREF(value);
#endif
    return value;
}

/* CLineInTraceback (used by AddTraceback) */
#if CYTHON_CLINE_IN_TRACEBACK && CYTHON_CLINE_IN_TRACEBACK_RUNTIME
#if CYTHON_COMPILING_IN_LIMITED_API && __PYX_LIMITED_VERSION_HEX < 0x030A0000
#define __Pyx_PyProbablyModule_GetDict(o) __Pyx_XNewRef(PyModule_GetDict(o))
#elif !CYTHON_COMPILING_IN_CPYTHON || CYTHON_COMPILING_IN_CPYTHON_FREETHREADING
#define __Pyx_PyProbablyModule_GetDict(o) PyObject_GenericGetDict(o, NULL);
#else
PyObject* __Pyx_PyProbablyModule_GetDict(PyObject *o) {
    PyObject **dict_ptr = _PyObject_GetDictPtr(o);
    return dict_ptr ? __Pyx_XNewRef(*dict_ptr) : NULL;
}
#endif
static int __Pyx_CLineForTraceback(PyThreadState *tstate, int c_line) {
    PyObject *use_cline = NULL;
    PyObject *ptype, *pvalue, *ptraceback;
    PyObject *cython_runtime_dict;
    CYTHON_MAYBE_UNUSED_VAR(tstate);
    if (unlikely(!__pyx_mstate_global->__pyx_cython_runtime)) {
        return c_line;
    }
    __Pyx_ErrFetchInState(tstate, &ptype, &pvalue, &ptraceback);
    cython_runtime_dict = __Pyx_PyProbablyModule_GetDict(__pyx_mstate_global->__pyx_cython_runtime);
    if (likely(cython_runtime_dict)) {
        __PYX_PY_DICT_LOOKUP_IF_MODIFIED(
            use_cline, cython_runtime_dict,
            __Pyx_PyDict_SetDefault(cython_runtime_dict, __pyx_mstate_global->__pyx_n_u_cline_in_traceback, Py_False))
    }
    if (use_cline == NULL || use_cline == Py_False || (use_cline != Py_True && PyObject_Not(use_cline) != 0)) {
        c_line = 0;
    }
    Py_XDECREF(use_cline);
    Py_XDECREF(cython_runtime_dict);
    __Pyx_ErrRestoreInState(tstate, ptype, pvalue, ptraceback);
    return c_line;
}
#endif

/* CodeObjectCache (used by AddTraceback) */
static int __pyx_bisect_code_objects(__Pyx_CodeObjectCacheEntry* entries, int count, int code_line) {
    int start = 0, mid = 0, end = count - 1;
    if (end >= 0 && code_line > entries[end].code_line) {
        return count;
    }
    while (start < end) {
        mid = start + (end - start) / 2;
        if (code_line < entries[mid].code_line) {
            end = mid;
        } else if (code_line > entries[mid].code_line) {
             start = mid + 1;
        } else {
            return mid;
        }
    }
    if (code_line <= entries[mid].code_line) {
        return mid;
    } else {
        return mid + 1;
    }
}
static __Pyx_CachedCodeObjectType *__pyx__find_code_object(struct __Pyx_CodeObjectCache *code_cache, int code_line) {
    __Pyx_CachedCodeObjectType* code_object;
    int pos;
    if (unlikely(!code_line) || unlikely(!code_cache->entries)) {
        return NULL;
    }
    pos = __pyx_bisect_code_objects(code_cache->entries, code_cache->count, code_line);
    if (unlikely(pos >= code_cache->count) || unlikely(code_cache->entries[pos].code_line != code_line)) {
        return NULL;
    }
    code_object = code_cache->entries[pos].code_object;
    Py_INCREF(code_object);
    return code_object;
}
static __Pyx_CachedCodeObjectType *__pyx_find_code_object(int code_line) {
#if CYTHON_COMPILING_IN_CPYTHON_FREETHREADING && !CYTHON_ATOMICS
    (void)__pyx__find_code_object;
    return NULL; // Most implementation should have atomics. But otherwise, don't make it thread-safe, just miss.
#else
    struct __Pyx_CodeObjectCache *code_cache = &__pyx_mstate_global->__pyx_code_cache;
#if CYTHON_COMPILING_IN_CPYTHON_FREETHREADING
    __pyx_nonatomic_int_type old_count = __pyx_atomic_incr_acq_rel(&code_cache->accessor_count);
    if (old_count < 0) {
        __pyx_atomic_decr_acq_rel(&code_cache->accessor_count);
        return NULL;
    }
#endif
    __Pyx_CachedCodeObjectType *result = __pyx__find_code_object(code_cache, code_line);
#if CYTHON_COMPILING_IN_CPYTHON_FREETHREADING
    __pyx_atomic_decr_acq_rel(&code_cache->accessor_count);
#endif
    return result;
#endif
}
static void __pyx__insert_code_object(struct __Pyx_CodeObjectCache *code_cache, int code_line, __Pyx_CachedCodeObjectType* code_object)
{
    int pos, i;
    __Pyx_CodeObjectCacheEntry* entries = code_cache->entries;
    if (unlikely(!code_line)) {
        return;
    }
    if (unlikely(!entries)) {
        entries = (__Pyx_CodeObjectCacheEntry*)PyMem_Malloc(64*sizeof(__Pyx_CodeObjectCacheEntry));
        if (likely(entries)) {
            code_cache->entries = entries;
            code_cache->max_count = 64;
            code_cache->count = 1;
            entries[0].code_line = code_line;
            entries[0].code_object = code_object;
            Py_INCREF(code_object);
        }
        return;
    }
    pos = __pyx_bisect_code_objects(code_cache->entries, code_cache->count, code_line);
    if ((pos < code_cache->count) && unlikely(code_cache->entries[pos].code_line == code_line)) {
        __Pyx_CachedCodeObjectType* tmp = entries[pos].code_object;
        entries[pos].code_object = code_object;
        Py_INCREF(code_object);
        Py_DECREF(tmp);
        return;
    }
    if (code_cache->count == code_cache->max_count) {
        int new_max = code_cache->max_count + 64;
        entries = (__Pyx_CodeObjectCacheEntry*)PyMem_Realloc(
            code_cache->entries, ((size_t)new_max) * sizeof(__Pyx_CodeObjectCacheEntry));
        if (unlikely(!entries)) {
            return;
        }
        code_cache->entries = entries;
        code_cache->max_count = new_max;
    }
    for (i=code_cache->count; i>pos; i--) {
        entries[i] = entries[i-1];
    }
    entries[pos].code_line = code_line;
    entries[pos].code_object = code_object;
    code_cache->count++;
    Py_INCREF(code_object);
}
static void __pyx_insert_code_object(int code_line, __Pyx_CachedCodeObjectType* code_object) {
#if CYTHON_COMPILING_IN_CPYTHON_FREETHREADING && !CYTHON_ATOMICS
    (void)__pyx__insert_code_object;
    return; // Most implementation should have atomics. But otherwise, don't make it thread-safe, just fail.
#else
    struct __Pyx_CodeObjectCache *code_cache = &__pyx_mstate_global->__pyx_code_cache;
#if CYTHON_COMPILING_IN_CPYTHON_FREETHREADING
    __pyx_nonatomic_int_type expected = 0;
    if (!__pyx_atomic_int_cmp_exchange(&code_cache->accessor_count, &expected, INT_MIN)) {
        return;
    }
#endif
    __pyx__insert_code_object(code_cache, code_line, code_object);
#if CYTHON_COMPILING_IN_CPYTHON_FREETHREADING
    __pyx_atomic_sub(&code_cache->accessor_count, INT_MIN);
#endif
#endif
}

/* AddTraceback */
#include "compile.h"
#include "frameobject.h"
#include "traceback.h"
#if PY_VERSION_HEX >= 0x030b00a6 && !CYTHON_COMPILING_IN_LIMITED_API && !defined(PYPY_VERSION)
  #ifndef Py_BUILD_CORE
    #define Py_BUILD_CORE 1
  #endif
  #include "internal/pycore_frame.h"
#endif
#if CYTHON_COMPILING_IN_LIMITED_API
static PyObject *__Pyx_PyCode_Replace_For_AddTraceback(PyObject *code, PyObject *scratch_dict,
                                                       PyObject *firstlineno, PyObject *name) {
    PyObject *replace = NULL;
    if (unlikely(PyDict_SetItemString(scratch_dict, "co_firstlineno", firstlineno))) return NULL;
    if (unlikely(PyDict_SetItemString(scratch_dict, "co_name", name))) return NULL;
    replace = PyObject_GetAttrString(code, "replace");
    if (likely(replace)) {
        PyObject *result = PyObject_Call(replace, __pyx_mstate_global->__pyx_empty_tuple, scratch_dict);
        Py_DECREF(replace);
        return result;
    }
    PyErr_Clear();
    return NULL;
}
static void __Pyx_AddTraceback(const char *funcname, int c_line,
                               int py_line, const char *filename) {
    PyObject *code_object = NULL, *py_py_line = NULL, *py_funcname = NULL, *dict = NULL;
    PyObject *replace = NULL, *getframe = NULL, *frame = NULL;
    PyObject *exc_type, *exc_value, *exc_traceback;
    int success = 0;
    if (c_line) {
        c_line = __Pyx_CLineForTraceback(__Pyx_PyThreadState_Current, c_line);
    }
    PyErr_Fetch(&exc_type, &exc_value, &exc_traceback);
    code_object = __pyx_find_code_object(c_line ? -c_line : py_line);
    if (!code_object) {
        code_object = Py_CompileString("_getframe()", filename, Py_eval_input);
        if (unlikely(!code_object)) goto bad;
        py_py_line = PyLong_FromLong(py_line);
        if (unlikely(!py_py_line)) goto bad;
        if (c_line) {
            py_funcname = PyUnicode_FromFormat( "%s (%s:%d)", funcname, __pyx_cfilenm, c_line);
        } else {
            py_funcname = PyUnicode_FromString(funcname);
        }
        if (unlikely(!py_funcname)) goto bad;
        dict = PyDict_New();
        if (unlikely(!dict)) goto bad;
        {
            PyObject *old_code_object = code_object;
            code_object = __Pyx_PyCode_Replace_For_AddTraceback(code_object, dict, py_py_line, py_funcname);
            Py_DECREF(old_code_object);
        }
        if (unlikely(!code_object)) goto bad;
        __pyx_insert_code_object(c_line ? -c_line : py_line, code_object);
    } else {
        dict = PyDict_New();
    }
    getframe = PySys_GetObject("_getframe");
    if (unlikely(!getframe)) goto bad;
    if (unlikely(PyDict_SetItemString(dict, "_getframe", getframe))) goto bad;
    frame = PyEval_EvalCode(code_object, dict, dict);
    if (unlikely(!frame) || frame == Py_None) goto bad;
    success = 1;
  bad:
    PyErr_Restore(exc_type, exc_value, exc_traceback);
    Py_XDECREF(code_object);
    Py_XDECREF(py_py_line);
    Py_XDECREF(py_funcname);
    Py_XDECREF(dict);
    Py_XDECREF(replace);
    if (success) {
        PyTraceBack_Here(
            (struct _frame*)frame);
    }
    Py_XDECREF(frame);
}
#else
static PyCodeObject* __Pyx_CreateCodeObjectForTraceback(
            const char *funcname, int c_line,
            int py_line, const char *filename) {
    PyCodeObject *py_code = NULL;
    PyObject *py_funcname = NULL;
    if (c_line) {
        py_funcname = PyUnicode_FromFormat( "%s (%s:%d)", funcname, __pyx_cfilenm, c_line);
        if (!py_funcname) goto bad;
        funcname = PyUnicode_AsUTF8(py_funcname);
        if (!funcname) goto bad;
    }
    py_code = PyCode_NewEmpty(filename, funcname, py_line);
    Py_XDECREF(py_funcname);
    return py_code;
bad:
    Py_XDECREF(py_funcname);
    return NULL;
}
static void __Pyx_AddTraceback(const char *funcname, int c_line,
                               int py_line, const char *filename) {
    PyCodeObject *py_code = 0;
    PyFrameObject *py_frame = 0;
    PyThreadState *tstate = __Pyx_PyThreadState_Current;
    PyObject *ptype, *pvalue, *ptraceback;
    if (c_line) {
        c_line = __Pyx_CLineForTraceback(tstate, c_line);
    }
    py_code = __pyx_find_code_object(c_line ? -c_line : py_line);
    if (!py_code) {
        __Pyx_ErrFetchInState(tstate, &ptype, &pvalue, &ptraceback);
        py_code = __Pyx_CreateCodeObjectForTraceback(
            funcname, c_line, py_line, filename);
        if (!py_code) {
            /* If the code object creation fails, then we should clear the
               fetched exception references and propagate the new exception */
            Py_XDECREF(ptype);
            Py_XDECREF(pvalue);
            Py_XDECREF(ptraceback);
            goto bad;
        }
        __Pyx_ErrRestoreInState(tstate, ptype, pvalue, ptraceback);
        __pyx_insert_code_object(c_line ? -c_line : py_line, py_code);
    }
    py_frame = PyFrame_New(
        tstate,            /*PyThreadState *tstate,*/
        py_code,           /*PyCodeObject *code,*/
        __pyx_mstate_global->__pyx_d,    /*PyObject *globals,*/
        0                  /*PyObject *locals*/
    );
    if (!py_frame) goto bad;
    __Pyx_PyFrame_SetLineNumber(py_frame, py_line);
    PyTraceBack_Here(py_frame);
bad:
    Py_XDECREF(py_code);
    Py_XDECREF(py_frame);
}
#endif

/* CIntFromPyVerify */
#define __PYX_VERIFY_RETURN_INT(target_type, func_type, func_value)\
    __PYX__VERIFY_RETURN_INT(target_type, func_type, func_value, 0)
#define __PYX_VERIFY_RETURN_INT_EXC(target_type, func_type, func_value)\
    __PYX__VERIFY_RETURN_INT(target_type, func_type, func_value, 1)
#define __PYX__VERIFY_RETURN_INT(target_type, func_type, func_value, exc)\
    {\
        func_type value = func_value;\
        if (sizeof(target_type) < sizeof(func_type)) {\
            if (unlikely(value != (func_type) (target_type) value)) {\
                func_type zero = 0;\
                if (exc && unlikely(value == (func_type)-1 && PyErr_Occurred()))\
                    return (target_type) -1;\
                if (is_unsigned && unlikely(value < zero))\
                    goto raise_neg_overflow;\
                else\
                    goto raise_overflow;\
            }\
        }\
        return (target_type) value;\
    }

/* CIntToPy */
static CYTHON_INLINE PyObject* __Pyx_PyLong_From_long(long value) {
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic push
#pragma GCC diagnostic ignored "-Wconversion"
#endif
    const long neg_one = (long) -1, const_zero = (long) 0;
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic pop
#endif
    const int is_unsigned = neg_one > const_zero;
    if (is_unsigned) {
        if (sizeof(long) < sizeof(long)) {
            return PyLong_FromLong((long) value);
        } else if (sizeof(long) <= sizeof(unsigned long)) {
            return PyLong_FromUnsignedLong((unsigned long) value);
#if !CYTHON_COMPILING_IN_PYPY
        } else if (sizeof(long) <= sizeof(unsigned PY_LONG_LONG)) {
            return PyLong_FromUnsignedLongLong((unsigned PY_LONG_LONG) value);
#endif
        }
    } else {
        if (sizeof(long) <= sizeof(long)) {
            return PyLong_FromLong((long) value);
        } else if (sizeof(long) <= sizeof(PY_LONG_LONG)) {
            return PyLong_FromLongLong((PY_LONG_LONG) value);
        }
    }
    {
        unsigned char *bytes = (unsigned char *)&value;
#if !CYTHON_COMPILING_IN_LIMITED_API && PY_VERSION_HEX >= 0x030d00A4
        if (is_unsigned) {
            return PyLong_FromUnsignedNativeBytes(bytes, sizeof(value), -1);
        } else {
            return PyLong_FromNativeBytes(bytes, sizeof(value), -1);
        }
#elif !CYTHON_COMPILING_IN_LIMITED_API && PY_VERSION_HEX < 0x030d0000
        int one = 1; int little = (int)*(unsigned char *)&one;
        return _PyLong_FromByteArray(bytes, sizeof(long),
                                     little, !is_unsigned);
#else
        int one = 1; int little = (int)*(unsigned char *)&one;
        PyObject *from_bytes, *result = NULL, *kwds = NULL;
        PyObject *py_bytes = NULL, *order_str = NULL;
        from_bytes = PyObject_GetAttrString((PyObject*)&PyLong_Type, "from_bytes");
        if (!from_bytes) return NULL;
        py_bytes = PyBytes_FromStringAndSize((char*)bytes, sizeof(long));
        if (!py_bytes) goto limited_bad;
        order_str = PyUnicode_FromString(little ? "little" : "big");
        if (!order_str) goto limited_bad;
        {
            PyObject *args[3+(CYTHON_VECTORCALL ? 1 : 0)] = { NULL, py_bytes, order_str };
            if (!is_unsigned) {
                kwds = __Pyx_MakeVectorcallBuilderKwds(1);
                if (!kwds) goto limited_bad;
                if (__Pyx_VectorcallBuilder_AddArgStr("signed", __Pyx_NewRef(Py_True), kwds, args+3, 0) < 0) goto limited_bad;
            }
            result = __Pyx_Object_Vectorcall_CallFromBuilder(from_bytes, args+1, 2 | __Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET, kwds);
        }
        limited_bad:
        Py_XDECREF(kwds);
        Py_XDECREF(order_str);
        Py_XDECREF(py_bytes);
        Py_XDECREF(from_bytes);
        return result;
#endif
    }
}

/* CIntFromPy */
static CYTHON_INLINE int __Pyx_PyLong_As_int(PyObject *x) {
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic push
#pragma GCC diagnostic ignored "-Wconversion"
#endif
    const int neg_one = (int) -1, const_zero = (int) 0;
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic pop
#endif
    const int is_unsigned = neg_one > const_zero;
    if (unlikely(!PyLong_Check(x))) {
        int val;
        PyObject *tmp = __Pyx_PyNumber_Long(x);
        if (!tmp) return (int) -1;
        val = __Pyx_PyLong_As_int(tmp);
        Py_DECREF(tmp);
        return val;
    }
    if (is_unsigned) {
#if CYTHON_USE_PYLONG_INTERNALS
        if (unlikely(__Pyx_PyLong_IsNeg(x))) {
            goto raise_neg_overflow;
        } else if (__Pyx_PyLong_IsCompact(x)) {
            __PYX_VERIFY_RETURN_INT(int, __Pyx_compact_upylong, __Pyx_PyLong_CompactValueUnsigned(x))
        } else {
            const digit* digits = __Pyx_PyLong_Digits(x);
            assert(__Pyx_PyLong_DigitCount(x) > 1);
            switch (__Pyx_PyLong_DigitCount(x)) {
                case 2:
                    if ((8 * sizeof(int) > 1 * PyLong_SHIFT)) {
                        if ((8 * sizeof(unsigned long) > 2 * PyLong_SHIFT)) {
                            __PYX_VERIFY_RETURN_INT(int, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if ((8 * sizeof(int) >= 2 * PyLong_SHIFT)) {
                            return (int) (((((int)digits[1]) << PyLong_SHIFT) | (int)digits[0]));
                        }
                    }
                    break;
                case 3:
                    if ((8 * sizeof(int) > 2 * PyLong_SHIFT)) {
                        if ((8 * sizeof(unsigned long) > 3 * PyLong_SHIFT)) {
                            __PYX_VERIFY_RETURN_INT(int, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if ((8 * sizeof(int) >= 3 * PyLong_SHIFT)) {
                            return (int) (((((((int)digits[2]) << PyLong_SHIFT) | (int)digits[1]) << PyLong_SHIFT) | (int)digits[0]));
                        }
                    }
                    break;
                case 4:
                    if ((8 * sizeof(int) > 3 * PyLong_SHIFT)) {
                        if ((8 * sizeof(unsigned long) > 4 * PyLong_SHIFT)) {
                            __PYX_VERIFY_RETURN_INT(int, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if ((8 * sizeof(int) >= 4 * PyLong_SHIFT)) {
                            return (int) (((((((((int)digits[3]) << PyLong_SHIFT) | (int)digits[2]) << PyLong_SHIFT) | (int)digits[1]) << PyLong_SHIFT) | (int)digits[0]));
                        }
                    }
                    break;
            }
        }
#endif
#if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX < 0x030C00A7
        if (unlikely(Py_SIZE(x) < 0)) {
            goto raise_neg_overflow;
        }
#else
        {
            int result = PyObject_RichCompareBool(x, Py_False, Py_LT);
            if (unlikely(result < 0))
                return (int) -1;
            if (unlikely(result == 1))
                goto raise_neg_overflow;
        }
#endif
        if ((sizeof(int) <= sizeof(unsigned long))) {
            __PYX_VERIFY_RETURN_INT_EXC(int, unsigned long, PyLong_AsUnsignedLong(x))
        } else if ((sizeof(int) <= sizeof(unsigned PY_LONG_LONG))) {
            __PYX_VERIFY_RETURN_INT_EXC(int, unsigned PY_LONG_LONG, PyLong_AsUnsignedLongLong(x))
        }
    } else {
#if CYTHON_USE_PYLONG_INTERNALS
        if (__Pyx_PyLong_IsCompact(x)) {
            __PYX_VERIFY_RETURN_INT(int, __Pyx_compact_pylong, __Pyx_PyLong_CompactValue(x))
        } else {
            const digit* digits = __Pyx_PyLong_Digits(x);
            assert(__Pyx_PyLong_DigitCount(x) > 1);
            switch (__Pyx_PyLong_SignedDigitCount(x)) {
                case -2:
                    if ((8 * sizeof(int) - 1 > 1 * PyLong_SHIFT)) {
                        if ((8 * sizeof(unsigned long) > 2 * PyLong_SHIFT)) {
                            __PYX_VERIFY_RETURN_INT(int, long, -(long) (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if ((8 * sizeof(int) - 1 > 2 * PyLong_SHIFT)) {
                            return (int) (((int)-1)*(((((int)digits[1]) << PyLong_SHIFT) | (int)digits[0])));
                        }
                    }
                    break;
                case 2:
                    if ((8 * sizeof(int) > 1 * PyLong_SHIFT)) {
                        if ((8 * sizeof(unsigned long) > 2 * PyLong_SHIFT)) {
                            __PYX_VERIFY_RETURN_INT(int, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if ((8 * sizeof(int) - 1 > 2 * PyLong_SHIFT)) {
                            return (int) ((((((int)digits[1]) << PyLong_SHIFT) | (int)digits[0])));
                        }
                    }
                    break;
                case -3:
                    if ((8 * sizeof(int) - 1 > 2 * PyLong_SHIFT)) {
                        if ((8 * sizeof(unsigned long) > 3 * PyLong_SHIFT)) {
                            __PYX_VERIFY_RETURN_INT(int, long, -(long) (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if ((8 * sizeof(int) - 1 > 3 * PyLong_SHIFT)) {
                            return (int) (((int)-1)*(((((((int)digits[2]) << PyLong_SHIFT) | (int)digits[1]) << PyLong_SHIFT) | (int)digits[0])));
                        }
                    }
                    break;
                case 3:
                    if ((8 * sizeof(int) > 2 * PyLong_SHIFT)) {
                        if ((8 * sizeof(unsigned long) > 3 * PyLong_SHIFT)) {
                            __PYX_VERIFY_RETURN_INT(int, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if ((8 * sizeof(int) - 1 > 3 * PyLong_SHIFT)) {
                            return (int) ((((((((int)digits[2]) << PyLong_SHIFT) | (int)digits[1]) << PyLong_SHIFT) | (int)digits[0])));
                        }
                    }
                    break;
                case -4:
                    if ((8 * sizeof(int) - 1 > 3 * PyLong_SHIFT)) {
                        if ((8 * sizeof(unsigned long) > 4 * PyLong_SHIFT)) {
                            __PYX_VERIFY_RETURN_INT(int, long, -(long) (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if ((8 * sizeof(int) - 1 > 4 * PyLong_SHIFT)) {
                            return (int) (((int)-1)*(((((((((int)digits[3]) << PyLong_SHIFT) | (int)digits[2]) << PyLong_SHIFT) | (int)digits[1]) << PyLong_SHIFT) | (int)digits[0])));
                        }
                    }
                    break;
                case 4:
                    if ((8 * sizeof(int) > 3 * PyLong_SHIFT)) {
                        if ((8 * sizeof(unsigned long) > 4 * PyLong_SHIFT)) {
                            __PYX_VERIFY_RETURN_INT(int, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if ((8 * sizeof(int) - 1 > 4 * PyLong_SHIFT)) {
                            return (int) ((((((((((int)digits[3]) << PyLong_SHIFT) | (int)digits[2]) << PyLong_SHIFT) | (int)digits[1]) << PyLong_SHIFT) | (int)digits[0])));
                        }
                    }
                    break;
            }
        }
#endif
        if ((sizeof(int) <= sizeof(long))) {
            __PYX_VERIFY_RETURN_INT_EXC(int, long, PyLong_AsLong(x))
        } else if ((sizeof(int) <= sizeof(PY_LONG_LONG))) {
            __PYX_VERIFY_RETURN_INT_EXC(int, PY_LONG_LONG, PyLong_AsLongLong(x))
        }
    }
    {
        int val;
        int ret = -1;
#if PY_VERSION_HEX >= 0x030d00A6 && !CYTHON_COMPILING_IN_LIMITED_API
        Py_ssize_t bytes_copied = PyLong_AsNativeBytes(
            x, &val, sizeof(val), Py_ASNATIVEBYTES_NATIVE_ENDIAN | (is_unsigned ? Py_ASNATIVEBYTES_UNSIGNED_BUFFER | Py_ASNATIVEBYTES_REJECT_NEGATIVE : 0));
        if (unlikely(bytes_copied == -1)) {
        } else if (unlikely(bytes_copied > (Py_ssize_t) sizeof(val))) {
            goto raise_overflow;
        } else {
            ret = 0;
        }
#elif PY_VERSION_HEX < 0x030d0000 && !(CYTHON_COMPILING_IN_PYPY || CYTHON_COMPILING_IN_LIMITED_API) || defined(_PyLong_AsByteArray)
        int one = 1; int is_little = (int)*(unsigned char *)&one;
        unsigned char *bytes = (unsigned char *)&val;
        ret = _PyLong_AsByteArray((PyLongObject *)x,
                                    bytes, sizeof(val),
                                    is_little, !is_unsigned);
#else
        PyObject *v;
        PyObject *stepval = NULL, *mask = NULL, *shift = NULL;
        int bits, remaining_bits, is_negative = 0;
        int chunk_size = (sizeof(long) < 8) ? 30 : 62;
        if (likely(PyLong_CheckExact(x))) {
            v = __Pyx_NewRef(x);
        } else {
            v = PyNumber_Long(x);
            if (unlikely(!v)) return (int) -1;
            assert(PyLong_CheckExact(v));
        }
        {
            int result = PyObject_RichCompareBool(v, Py_False, Py_LT);
            if (unlikely(result < 0)) {
                Py_DECREF(v);
                return (int) -1;
            }
            is_negative = result == 1;
        }
        if (is_unsigned && unlikely(is_negative)) {
            Py_DECREF(v);
            goto raise_neg_overflow;
        } else if (is_negative) {
            stepval = PyNumber_Invert(v);
            Py_DECREF(v);
            if (unlikely(!stepval))
                return (int) -1;
        } else {
            stepval = v;
        }
        v = NULL;
        val = (int) 0;
        mask = PyLong_FromLong((1L << chunk_size) - 1); if (unlikely(!mask)) goto done;
        shift = PyLong_FromLong(chunk_size); if (unlikely(!shift)) goto done;
        for (bits = 0; bits < (int) sizeof(int) * 8 - chunk_size; bits += chunk_size) {
            PyObject *tmp, *digit;
            long idigit;
            digit = PyNumber_And(stepval, mask);
            if (unlikely(!digit)) goto done;
            idigit = PyLong_AsLong(digit);
            Py_DECREF(digit);
            if (unlikely(idigit < 0)) goto done;
            val |= ((int) idigit) << bits;
            tmp = PyNumber_Rshift(stepval, shift);
            if (unlikely(!tmp)) goto done;
            Py_DECREF(stepval); stepval = tmp;
        }
        Py_DECREF(shift); shift = NULL;
        Py_DECREF(mask); mask = NULL;
        {
            long idigit = PyLong_AsLong(stepval);
            if (unlikely(idigit < 0)) goto done;
            remaining_bits = ((int) sizeof(int) * 8) - bits - (is_unsigned ? 0 : 1);
            if (unlikely(idigit >= (1L << remaining_bits)))
                goto raise_overflow;
            val |= ((int) idigit) << bits;
        }
        if (!is_unsigned) {
            if (unlikely(val & (((int) 1) << (sizeof(int) * 8 - 1))))
                goto raise_overflow;
            if (is_negative)
                val = ~val;
        }
        ret = 0;
    done:
        Py_XDECREF(shift);
        Py_XDECREF(mask);
        Py_XDECREF(stepval);
#endif
        if (unlikely(ret))
            return (int) -1;
        return val;
    }
raise_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "value too large to convert to int");
    return (int) -1;
raise_neg_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "can't convert negative value to int");
    return (int) -1;
}

/* CIntToPy */
static CYTHON_INLINE PyObject* __Pyx_PyLong_From_int(int value) {
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic push
#pragma GCC diagnostic ignored "-Wconversion"
#endif
    const int neg_one = (int) -1, const_zero = (int) 0;
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic pop
#endif
    const int is_unsigned = neg_one > const_zero;
    if (is_unsigned) {
        if (sizeof(int) < sizeof(long)) {
            return PyLong_FromLong((long) value);
        } else if (sizeof(int) <= sizeof(unsigned long)) {
            return PyLong_FromUnsignedLong((unsigned long) value);
#if !CYTHON_COMPILING_IN_PYPY
        } else if (sizeof(int) <= sizeof(unsigned PY_LONG_LONG)) {
            return PyLong_FromUnsignedLongLong((unsigned PY_LONG_LONG) value);
#endif
        }
    } else {
        if (sizeof(int) <= sizeof(long)) {
            return PyLong_FromLong((long) value);
        } else if (sizeof(int) <= sizeof(PY_LONG_LONG)) {
            return PyLong_FromLongLong((PY_LONG_LONG) value);
        }
    }
    {
        unsigned char *bytes = (unsigned char *)&value;
#if !CYTHON_COMPILING_IN_LIMITED_API && PY_VERSION_HEX >= 0x030d00A4
        if (is_unsigned) {
            return PyLong_FromUnsignedNativeBytes(bytes, sizeof(value), -1);
        } else {
            return PyLong_FromNativeBytes(bytes, sizeof(value), -1);
        }
#elif !CYTHON_COMPILING_IN_LIMITED_API && PY_VERSION_HEX < 0x030d0000
        int one = 1; int little = (int)*(unsigned char *)&one;
        return _PyLong_FromByteArray(bytes, sizeof(int),
                                     little, !is_unsigned);
#else
        int one = 1; int little = (int)*(unsigned char *)&one;
        PyObject *from_bytes, *result = NULL, *kwds = NULL;
        PyObject *py_bytes = NULL, *order_str = NULL;
        from_bytes = PyObject_GetAttrString((PyObject*)&PyLong_Type, "from_bytes");
        if (!from_bytes) return NULL;
        py_bytes = PyBytes_FromStringAndSize((char*)bytes, sizeof(int));
        if (!py_bytes) goto limited_bad;
        order_str = PyUnicode_FromString(little ? "little" : "big");
        if (!order_str) goto limited_bad;
        {
            PyObject *args[3+(CYTHON_VECTORCALL ? 1 : 0)] = { NULL, py_bytes, order_str };
            if (!is_unsigned) {
                kwds = __Pyx_MakeVectorcallBuilderKwds(1);
                if (!kwds) goto limited_bad;
                if (__Pyx_VectorcallBuilder_AddArgStr("signed", __Pyx_NewRef(Py_True), kwds, args+3, 0) < 0) goto limited_bad;
            }
            result = __Pyx_Object_Vectorcall_CallFromBuilder(from_bytes, args+1, 2 | __Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET, kwds);
        }
        limited_bad:
        Py_XDECREF(kwds);
        Py_XDECREF(order_str);
        Py_XDECREF(py_bytes);
        Py_XDECREF(from_bytes);
        return result;
#endif
    }
}

/* CIntFromPy */
static CYTHON_INLINE size_t __Pyx_PyLong_As_size_t(PyObject *x) {
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic push
#pragma GCC diagnostic ignored "-Wconversion"
#endif
    const size_t neg_one = (size_t) -1, const_zero = (size_t) 0;
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic pop
#endif
    const int is_unsigned = neg_one > const_zero;
    if (unlikely(!PyLong_Check(x))) {
        size_t val;
        PyObject *tmp = __Pyx_PyNumber_Long(x);
        if (!tmp) return (size_t) -1;
        val = __Pyx_PyLong_As_size_t(tmp);
        Py_DECREF(tmp);
        return val;
    }
    if (is_unsigned) {
#if CYTHON_USE_PYLONG_INTERNALS
        if (unlikely(__Pyx_PyLong_IsNeg(x))) {
            goto raise_neg_overflow;
        } else if (__Pyx_PyLong_IsCompact(x)) {
            __PYX_VERIFY_RETURN_INT(size_t, __Pyx_compact_upylong, __Pyx_PyLong_CompactValueUnsigned(x))
        } else {
            const digit* digits = __Pyx_PyLong_Digits(x);
            assert(__Pyx_PyLong_DigitCount(x) > 1);
            switch (__Pyx_PyLong_DigitCount(x)) {
                case 2:
                    if ((8 * sizeof(size_t) > 1 * PyLong_SHIFT)) {
                        if ((8 * sizeof(unsigned long) > 2 * PyLong_SHIFT)) {
                            __PYX_VERIFY_RETURN_INT(size_t, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if ((8 * sizeof(size_t) >= 2 * PyLong_SHIFT)) {
                            return (size_t) (((((size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0]));
                        }
                    }
                    break;
                case 3:
                    if ((8 * sizeof(size_t) > 2 * PyLong_SHIFT)) {
                        if ((8 * sizeof(unsigned long) > 3 * PyLong_SHIFT)) {
                            __PYX_VERIFY_RETURN_INT(size_t, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if ((8 * sizeof(size_t) >= 3 * PyLong_SHIFT)) {
                            return (size_t) (((((((size_t)digits[2]) << PyLong_SHIFT) | (size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0]));
                        }
                    }
                    break;
                case 4:
                    if ((8 * sizeof(size_t) > 3 * PyLong_SHIFT)) {
                        if ((8 * sizeof(unsigned long) > 4 * PyLong_SHIFT)) {
                            __PYX_VERIFY_RETURN_INT(size_t, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if ((8 * sizeof(size_t) >= 4 * PyLong_SHIFT)) {
                            return (size_t) (((((((((size_t)digits[3]) << PyLong_SHIFT) | (size_t)digits[2]) << PyLong_SHIFT) | (size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0]));
                        }
                    }
                    break;
            }
        }
#endif
#if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX < 0x030C00A7
        if (unlikely(Py_SIZE(x) < 0)) {
            goto raise_neg_overflow;
        }
#else
        {
            int result = PyObject_RichCompareBool(x, Py_False, Py_LT);
            if (unlikely(result < 0))
                return (size_t) -1;
            if (unlikely(result == 1))
                goto raise_neg_overflow;
        }
#endif
        if ((sizeof(size_t) <= sizeof(unsigned long))) {
            __PYX_VERIFY_RETURN_INT_EXC(size_t, unsigned long, PyLong_AsUnsignedLong(x))
        } else if ((sizeof(size_t) <= sizeof(unsigned PY_LONG_LONG))) {
            __PYX_VERIFY_RETURN_INT_EXC(size_t, unsigned PY_LONG_LONG, PyLong_AsUnsignedLongLong(x))
        }
    } else {
#if CYTHON_USE_PYLONG_INTERNALS
        if (__Pyx_PyLong_IsCompact(x)) {
            __PYX_VERIFY_RETURN_INT(size_t, __Pyx_compact_pylong, __Pyx_PyLong_CompactValue(x))
        } else {
            const digit* digits = __Pyx_PyLong_Digits(x);
            assert(__Pyx_PyLong_DigitCount(x) > 1);
            switch (__Pyx_PyLong_SignedDigitCount(x)) {
                case -2:
                    if ((8 * sizeof(size_t) - 1 > 1 * PyLong_SHIFT)) {
                        if ((8 * sizeof(unsigned long) > 2 * PyLong_SHIFT)) {
                            __PYX_VERIFY_RETURN_INT(size_t, long, -(long) (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if ((8 * sizeof(size_t) - 1 > 2 * PyLong_SHIFT)) {
                            return (size_t) (((size_t)-1)*(((((size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0])));
                        }
                    }
                    break;
                case 2:
                    if ((8 * sizeof(size_t) > 1 * PyLong_SHIFT)) {
                        if ((8 * sizeof(unsigned long) > 2 * PyLong_SHIFT)) {
                            __PYX_VERIFY_RETURN_INT(size_t, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if ((8 * sizeof(size_t) - 1 > 2 * PyLong_SHIFT)) {
                            return (size_t) ((((((size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0])));
                        }
                    }
                    break;
                case -3:
                    if ((8 * sizeof(size_t) - 1 > 2 * PyLong_SHIFT)) {
                        if ((8 * sizeof(unsigned long) > 3 * PyLong_SHIFT)) {
                            __PYX_VERIFY_RETURN_INT(size_t, long, -(long) (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if ((8 * sizeof(size_t) - 1 > 3 * PyLong_SHIFT)) {
                            return (size_t) (((size_t)-1)*(((((((size_t)digits[2]) << PyLong_SHIFT) | (size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0])));
                        }
                    }
                    break;
                case 3:
                    if ((8 * sizeof(size_t) > 2 * PyLong_SHIFT)) {
                        if ((8 * sizeof(unsigned long) > 3 * PyLong_SHIFT)) {
                            __PYX_VERIFY_RETURN_INT(size_t, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if ((8 * sizeof(size_t) - 1 > 3 * PyLong_SHIFT)) {
                            return (size_t) ((((((((size_t)digits[2]) << PyLong_SHIFT) | (size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0])));
                        }
                    }
                    break;
                case -4:
                    if ((8 * sizeof(size_t) - 1 > 3 * PyLong_SHIFT)) {
                        if ((8 * sizeof(unsigned long) > 4 * PyLong_SHIFT)) {
                            __PYX_VERIFY_RETURN_INT(size_t, long, -(long) (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if ((8 * sizeof(size_t) - 1 > 4 * PyLong_SHIFT)) {
                            return (size_t) (((size_t)-1)*(((((((((size_t)digits[3]) << PyLong_SHIFT) | (size_t)digits[2]) << PyLong_SHIFT) | (size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0])));
                        }
                    }
                    break;
                case 4:
                    if ((8 * sizeof(size_t) > 3 * PyLong_SHIFT)) {
                        if ((8 * sizeof(unsigned long) > 4 * PyLong_SHIFT)) {
                            __PYX_VERIFY_RETURN_INT(size_t, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if ((8 * sizeof(size_t) - 1 > 4 * PyLong_SHIFT)) {
                            return (size_t) ((((((((((size_t)digits[3]) << PyLong_SHIFT) | (size_t)digits[2]) << PyLong_SHIFT) | (size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0])));
                        }
                    }
                    break;
            }
        }
#endif
        if ((sizeof(size_t) <= sizeof(long))) {
            __PYX_VERIFY_RETURN_INT_EXC(size_t, long, PyLong_AsLong(x))
        } else if ((sizeof(size_t) <= sizeof(PY_LONG_LONG))) {
            __PYX_VERIFY_RETURN_INT_EXC(size_t, PY_LONG_LONG, PyLong_AsLongLong(x))
        }
    }
    {
        size_t val;
        int ret = -1;
#if PY_VERSION_HEX >= 0x030d00A6 && !CYTHON_COMPILING_IN_LIMITED_API
        Py_ssize_t bytes_copied = PyLong_AsNativeBytes(
            x, &val, sizeof(val), Py_ASNATIVEBYTES_NATIVE_ENDIAN | (is_unsigned ? Py_ASNATIVEBYTES_UNSIGNED_BUFFER | Py_ASNATIVEBYTES_REJECT_NEGATIVE : 0));
        if (unlikely(bytes_copied == -1)) {
        } else if (unlikely(bytes_copied > (Py_ssize_t) sizeof(val))) {
            goto raise_overflow;
        } else {
            ret = 0;
        }
#elif PY_VERSION_HEX < 0x030d0000 && !(CYTHON_COMPILING_IN_PYPY || CYTHON_COMPILING_IN_LIMITED_API) || defined(_PyLong_AsByteArray)
        int one = 1; int is_little = (int)*(unsigned char *)&one;
        unsigned char *bytes = (unsigned char *)&val;
        ret = _PyLong_AsByteArray((PyLongObject *)x,
                                    bytes, sizeof(val),
                                    is_little, !is_unsigned);
#else
        PyObject *v;
        PyObject *stepval = NULL, *mask = NULL, *shift = NULL;
        int bits, remaining_bits, is_negative = 0;
        int chunk_size = (sizeof(long) < 8) ? 30 : 62;
        if (likely(PyLong_CheckExact(x))) {
            v = __Pyx_NewRef(x);
        } else {
            v = PyNumber_Long(x);
            if (unlikely(!v)) return (size_t) -1;
            assert(PyLong_CheckExact(v));
        }
        {
            int result = PyObject_RichCompareBool(v, Py_False, Py_LT);
            if (unlikely(result < 0)) {
                Py_DECREF(v);
                return (size_t) -1;
            }
            is_negative = result == 1;
        }
        if (is_unsigned && unlikely(is_negative)) {
            Py_DECREF(v);
            goto raise_neg_overflow;
        } else if (is_negative) {
            stepval = PyNumber_Invert(v);
            Py_DECREF(v);
            if (unlikely(!stepval))
                return (size_t) -1;
        } else {
            stepval = v;
        }
        v = NULL;
        val = (size_t) 0;
        mask = PyLong_FromLong((1L << chunk_size) - 1); if (unlikely(!mask)) goto done;
        shift = PyLong_FromLong(chunk_size); if (unlikely(!shift)) goto done;
        for (bits = 0; bits < (int) sizeof(size_t) * 8 - chunk_size; bits += chunk_size) {
            PyObject *tmp, *digit;
            long idigit;
            digit = PyNumber_And(stepval, mask);
            if (unlikely(!digit)) goto done;
            idigit = PyLong_AsLong(digit);
            Py_DECREF(digit);
            if (unlikely(idigit < 0)) goto done;
            val |= ((size_t) idigit) << bits;
            tmp = PyNumber_Rshift(stepval, shift);
            if (unlikely(!tmp)) goto done;
            Py_DECREF(stepval); stepval = tmp;
        }
        Py_DECREF(shift); shift = NULL;
        Py_DECREF(mask); mask = NULL;
        {
            long idigit = PyLong_AsLong(stepval);
            if (unlikely(idigit < 0)) goto done;
            remaining_bits = ((int) sizeof(size_t) * 8) - bits - (is_unsigned ? 0 : 1);
            if (unlikely(idigit >= (1L << remaining_bits)))
                goto raise_overflow;
            val |= ((size_t) idigit) << bits;
        }
        if (!is_unsigned) {
            if (unlikely(val & (((size_t) 1) << (sizeof(size_t) * 8 - 1))))
                goto raise_overflow;
            if (is_negative)
                val = ~val;
        }
        ret = 0;
    done:
        Py_XDECREF(shift);
        Py_XDECREF(mask);
        Py_XDECREF(stepval);
#endif
        if (unlikely(ret))
            return (size_t) -1;
        return val;
    }
raise_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "value too large to convert to size_t");
    return (size_t) -1;
raise_neg_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "can't convert negative value to size_t");
    return (size_t) -1;
}

/* FormatTypeName */
#if CYTHON_COMPILING_IN_LIMITED_API && __PYX_LIMITED_VERSION_HEX < 0x030d0000
static __Pyx_TypeName
__Pyx_PyType_GetFullyQualifiedName(PyTypeObject* tp)
{
    PyObject *module = NULL, *name = NULL, *result = NULL;
    #if __PYX_LIMITED_VERSION_HEX < 0x030b0000
    name = __Pyx_PyObject_GetAttrStr((PyObject *)tp,
                                               __pyx_mstate_global->__pyx_n_u_qualname);
    #else
    name = PyType_GetQualName(tp);
    #endif
    if (unlikely(name == NULL) || unlikely(!PyUnicode_Check(name))) goto bad;
    module = __Pyx_PyObject_GetAttrStr((PyObject *)tp,
                                               __pyx_mstate_global->__pyx_n_u_module);
    if (unlikely(module == NULL) || unlikely(!PyUnicode_Check(module))) goto bad;
    if (PyUnicode_CompareWithASCIIString(module, "builtins") == 0) {
        result = name;
        name = NULL;
        goto done;
    }
    result = PyUnicode_FromFormat("%U.%U", module, name);
    if (unlikely(result == NULL)) goto bad;
  done:
    Py_XDECREF(name);
    Py_XDECREF(module);
    return result;
  bad:
    PyErr_Clear();
    if (name) {
        result = name;
        name = NULL;
    } else {
        result = __Pyx_NewRef(__pyx_mstate_global->__pyx_kp_u__3);
    }
    goto done;
}
#endif

/* CIntFromPy */
static CYTHON_INLINE long __Pyx_PyLong_As_long(PyObject *x) {
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic push
#pragma GCC diagnostic ignored "-Wconversion"
#endif
    const long neg_one = (long) -1, const_zero = (long) 0;
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic pop
#endif
    const int is_unsigned = neg_one > const_zero;
    if (unlikely(!PyLong_Check(x))) {
        long val;
        PyObject *tmp = __Pyx_PyNumber_Long(x);
        if (!tmp) return (long) -1;
        val = __Pyx_PyLong_As_long(tmp);
        Py_DECREF(tmp);
        return val;
    }
    if (is_unsigned) {
#if CYTHON_USE_PYLONG_INTERNALS
        if (unlikely(__Pyx_PyLong_IsNeg(x))) {
            goto raise_neg_overflow;
        } else if (__Pyx_PyLong_IsCompact(x)) {
            __PYX_VERIFY_RETURN_INT(long, __Pyx_compact_upylong, __Pyx_PyLong_CompactValueUnsigned(x))
        } else {
            const digit* digits = __Pyx_PyLong_Digits(x);
            assert(__Pyx_PyLong_DigitCount(x) > 1);
            switch (__Pyx_PyLong_DigitCount(x)) {
                case 2:
                    if ((8 * sizeof(long) > 1 * PyLong_SHIFT)) {
                        if ((8 * sizeof(unsigned long) > 2 * PyLong_SHIFT)) {
                            __PYX_VERIFY_RETURN_INT(long, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if ((8 * sizeof(long) >= 2 * PyLong_SHIFT)) {
                            return (long) (((((long)digits[1]) << PyLong_SHIFT) | (long)digits[0]));
                        }
                    }
                    break;
                case 3:
                    if ((8 * sizeof(long) > 2 * PyLong_SHIFT)) {
                        if ((8 * sizeof(unsigned long) > 3 * PyLong_SHIFT)) {
                            __PYX_VERIFY_RETURN_INT(long, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if ((8 * sizeof(long) >= 3 * PyLong_SHIFT)) {
                            return (long) (((((((long)digits[2]) << PyLong_SHIFT) | (long)digits[1]) << PyLong_SHIFT) | (long)digits[0]));
                        }
                    }
                    break;
                case 4:
                    if ((8 * sizeof(long) > 3 * PyLong_SHIFT)) {
                        if ((8 * sizeof(unsigned long) > 4 * PyLong_SHIFT)) {
                            __PYX_VERIFY_RETURN_INT(long, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if ((8 * sizeof(long) >= 4 * PyLong_SHIFT)) {
                            return (long) (((((((((long)digits[3]) << PyLong_SHIFT) | (long)digits[2]) << PyLong_SHIFT) | (long)digits[1]) << PyLong_SHIFT) | (long)digits[0]));
                        }
                    }
                    break;
            }
        }
#endif
#if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX < 0x030C00A7
        if (unlikely(Py_SIZE(x) < 0)) {
            goto raise_neg_overflow;
        }
#else
        {
            int result = PyObject_RichCompareBool(x, Py_False, Py_LT);
            if (unlikely(result < 0))
                return (long) -1;
            if (unlikely(result == 1))
                goto raise_neg_overflow;
        }
#endif
        if ((sizeof(long) <= sizeof(unsigned long))) {
            __PYX_VERIFY_RETURN_INT_EXC(long, unsigned long, PyLong_AsUnsignedLong(x))
        } else if ((sizeof(long) <= sizeof(unsigned PY_LONG_LONG))) {
            __PYX_VERIFY_RETURN_INT_EXC(long, unsigned PY_LONG_LONG, PyLong_AsUnsignedLongLong(x))
        }
    } else {
#if CYTHON_USE_PYLONG_INTERNALS
        if (__Pyx_PyLong_IsCompact(x)) {
            __PYX_VERIFY_RETURN_INT(long, __Pyx_compact_pylong, __Pyx_PyLong_CompactValue(x))
        } else {
            const digit* digits = __Pyx_PyLong_Digits(x);
            assert(__Pyx_PyLong_DigitCount(x) > 1);
            switch (__Pyx_PyLong_SignedDigitCount(x)) {
                case -2:
                    if ((8 * sizeof(long) - 1 > 1 * PyLong_SHIFT)) {
                        if ((8 * sizeof(unsigned long) > 2 * PyLong_SHIFT)) {
                            __PYX_VERIFY_RETURN_INT(long, long, -(long) (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if ((8 * sizeof(long) - 1 > 2 * PyLong_SHIFT)) {
                            return (long) (((long)-1)*(((((long)digits[1]) << PyLong_SHIFT) | (long)digits[0])));
                        }
                    }
                    break;
                case 2:
                    if ((8 * sizeof(long) > 1 * PyLong_SHIFT)) {
                        if ((8 * sizeof(unsigned long) > 2 * PyLong_SHIFT)) {
                            __PYX_VERIFY_RETURN_INT(long, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if ((8 * sizeof(long) - 1 > 2 * PyLong_SHIFT)) {
                            return (long) ((((((long)digits[1]) << PyLong_SHIFT) | (long)digits[0])));
                        }
                    }
                    break;
                case -3:
                    if ((8 * sizeof(long) - 1 > 2 * PyLong_SHIFT)) {
                        if ((8 * sizeof(unsigned long) > 3 * PyLong_SHIFT)) {
                            __PYX_VERIFY_RETURN_INT(long, long, -(long) (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if ((8 * sizeof(long) - 1 > 3 * PyLong_SHIFT)) {
                            return (long) (((long)-1)*(((((((long)digits[2]) << PyLong_SHIFT) | (long)digits[1]) << PyLong_SHIFT) | (long)digits[0])));
                        }
                    }
                    break;
                case 3:
                    if ((8 * sizeof(long) > 2 * PyLong_SHIFT)) {
                        if ((8 * sizeof(unsigned long) > 3 * PyLong_SHIFT)) {
                            __PYX_VERIFY_RETURN_INT(long, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if ((8 * sizeof(long) - 1 > 3 * PyLong_SHIFT)) {
                            return (long) ((((((((long)digits[2]) << PyLong_SHIFT) | (long)digits[1]) << PyLong_SHIFT) | (long)digits[0])));
                        }
                    }
                    break;
                case -4:
                    if ((8 * sizeof(long) - 1 > 3 * PyLong_SHIFT)) {
                        if ((8 * sizeof(unsigned long) > 4 * PyLong_SHIFT)) {
                            __PYX_VERIFY_RETURN_INT(long, long, -(long) (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if ((8 * sizeof(long) - 1 > 4 * PyLong_SHIFT)) {
                            return (long) (((long)-1)*(((((((((long)digits[3]) << PyLong_SHIFT) | (long)digits[2]) << PyLong_SHIFT) | (long)digits[1]) << PyLong_SHIFT) | (long)digits[0])));
                        }
                    }
                    break;
                case 4:
                    if ((8 * sizeof(long) > 3 * PyLong_SHIFT)) {
                        if ((8 * sizeof(unsigned long) > 4 * PyLong_SHIFT)) {
                            __PYX_VERIFY_RETURN_INT(long, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if ((8 * sizeof(long) - 1 > 4 * PyLong_SHIFT)) {
                            return (long) ((((((((((long)digits[3]) << PyLong_SHIFT) | (long)digits[2]) << PyLong_SHIFT) | (long)digits[1]) << PyLong_SHIFT) | (long)digits[0])));
                        }
                    }
                    break;
            }
        }
#endif
        if ((sizeof(long) <= sizeof(long))) {
            __PYX_VERIFY_RETURN_INT_EXC(long, long, PyLong_AsLong(x))
        } else if ((sizeof(long) <= sizeof(PY_LONG_LONG))) {
            __PYX_VERIFY_RETURN_INT_EXC(long, PY_LONG_LONG, PyLong_AsLongLong(x))
        }
    }
    {
        long val;
        int ret = -1;
#if PY_VERSION_HEX >= 0x030d00A6 && !CYTHON_COMPILING_IN_LIMITED_API
        Py_ssize_t bytes_copied = PyLong_AsNativeBytes(
            x, &val, sizeof(val), Py_ASNATIVEBYTES_NATIVE_ENDIAN | (is_unsigned ? Py_ASNATIVEBYTES_UNSIGNED_BUFFER | Py_ASNATIVEBYTES_REJECT_NEGATIVE : 0));
        if (unlikely(bytes_copied == -1)) {
        } else if (unlikely(bytes_copied > (Py_ssize_t) sizeof(val))) {
            goto raise_overflow;
        } else {
            ret = 0;
        }
#elif PY_VERSION_HEX < 0x030d0000 && !(CYTHON_COMPILING_IN_PYPY || CYTHON_COMPILING_IN_LIMITED_API) || defined(_PyLong_AsByteArray)
        int one = 1; int is_little = (int)*(unsigned char *)&one;
        unsigned char *bytes = (unsigned char *)&val;
        ret = _PyLong_AsByteArray((PyLongObject *)x,
                                    bytes, sizeof(val),
                                    is_little, !is_unsigned);
#else
        PyObject *v;
        PyObject *stepval = NULL, *mask = NULL, *shift = NULL;
        int bits, remaining_bits, is_negative = 0;
        int chunk_size = (sizeof(long) < 8) ? 30 : 62;
        if (likely(PyLong_CheckExact(x))) {
            v = __Pyx_NewRef(x);
        } else {
            v = PyNumber_Long(x);
            if (unlikely(!v)) return (long) -1;
            assert(PyLong_CheckExact(v));
        }
        {
            int result = PyObject_RichCompareBool(v, Py_False, Py_LT);
            if (unlikely(result < 0)) {
                Py_DECREF(v);
                return (long) -1;
            }
            is_negative = result == 1;
        }
        if (is_unsigned && unlikely(is_negative)) {
            Py_DECREF(v);
            goto raise_neg_overflow;
        } else if (is_negative) {
            stepval = PyNumber_Invert(v);
            Py_DECREF(v);
            if (unlikely(!stepval))
                return (long) -1;
        } else {
            stepval = v;
        }
        v = NULL;
        val = (long) 0;
        mask = PyLong_FromLong((1L << chunk_size) - 1); if (unlikely(!mask)) goto done;
        shift = PyLong_FromLong(chunk_size); if (unlikely(!shift)) goto done;
        for (bits = 0; bits < (int) sizeof(long) * 8 - chunk_size; bits += chunk_size) {
            PyObject *tmp, *digit;
            long idigit;
            digit = PyNumber_And(stepval, mask);
            if (unlikely(!digit)) goto done;
            idigit = PyLong_AsLong(digit);
            Py_DECREF(digit);
            if (unlikely(idigit < 0)) goto done;
            val |= ((long) idigit) << bits;
            tmp = PyNumber_Rshift(stepval, shift);
            if (unlikely(!tmp)) goto done;
            Py_DECREF(stepval); stepval = tmp;
        }
        Py_DECREF(shift); shift = NULL;
        Py_DECREF(mask); mask = NULL;
        {
            long idigit = PyLong_AsLong(stepval);
            if (unlikely(idigit < 0)) goto done;
            remaining_bits = ((int) sizeof(long) * 8) - bits - (is_unsigned ? 0 : 1);
            if (unlikely(idigit >= (1L << remaining_bits)))
                goto raise_overflow;
            val |= ((long) idigit) << bits;
        }
        if (!is_unsigned) {
            if (unlikely(val & (((long) 1) << (sizeof(long) * 8 - 1))))
                goto raise_overflow;
            if (is_negative)
                val = ~val;
        }
        ret = 0;
    done:
        Py_XDECREF(shift);
        Py_XDECREF(mask);
        Py_XDECREF(stepval);
#endif
        if (unlikely(ret))
            return (long) -1;
        return val;
    }
raise_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "value too large to convert to long");
    return (long) -1;
raise_neg_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "can't convert negative value to long");
    return (long) -1;
}

/* FastTypeChecks */
#if CYTHON_COMPILING_IN_CPYTHON
static int __Pyx_InBases(PyTypeObject *a, PyTypeObject *b) {
    while (a) {
        a = __Pyx_PyType_GetSlot(a, tp_base, PyTypeObject*);
        if (a == b)
            return 1;
    }
    return b == &PyBaseObject_Type;
}
static CYTHON_INLINE int __Pyx_IsSubtype(PyTypeObject *a, PyTypeObject *b) {
    PyObject *mro;
    if (a == b) return 1;
    mro = a->tp_mro;
    if (likely(mro)) {
        Py_ssize_t i, n;
        n = PyTuple_GET_SIZE(mro);
        for (i = 0; i < n; i++) {
            if (PyTuple_GET_ITEM(mro, i) == (PyObject *)b)
                return 1;
        }
        return 0;
    }
    return __Pyx_InBases(a, b);
}
static CYTHON_INLINE int __Pyx_IsAnySubtype2(PyTypeObject *cls, PyTypeObject *a, PyTypeObject *b) {
    PyObject *mro;
    if (cls == a || cls == b) return 1;
    mro = cls->tp_mro;
    if (likely(mro)) {
        Py_ssize_t i, n;
        n = PyTuple_GET_SIZE(mro);
        for (i = 0; i < n; i++) {
            PyObject *base = PyTuple_GET_ITEM(mro, i);
            if (base == (PyObject *)a || base == (PyObject *)b)
                return 1;
        }
        return 0;
    }
    return __Pyx_InBases(cls, a) || __Pyx_InBases(cls, b);
}
static CYTHON_INLINE int __Pyx_inner_PyErr_GivenExceptionMatches2(PyObject *err, PyObject* exc_type1, PyObject *exc_type2) {
    if (exc_type1) {
        return __Pyx_IsAnySubtype2((PyTypeObject*)err, (PyTypeObject*)exc_type1, (PyTypeObject*)exc_type2);
    } else {
        return __Pyx_IsSubtype((PyTypeObject*)err, (PyTypeObject*)exc_type2);
    }
}
static int __Pyx_PyErr_GivenExceptionMatchesTuple(PyObject *exc_type, PyObject *tuple) {
    Py_ssize_t i, n;
    assert(PyExceptionClass_Check(exc_type));
    n = PyTuple_GET_SIZE(tuple);
    for (i=0; i<n; i++) {
        if (exc_type == PyTuple_GET_ITEM(tuple, i)) return 1;
    }
    for (i=0; i<n; i++) {
        PyObject *t = PyTuple_GET_ITEM(tuple, i);
        if (likely(PyExceptionClass_Check(t))) {
            if (__Pyx_inner_PyErr_GivenExceptionMatches2(exc_type, NULL, t)) return 1;
        } else {
        }
    }
    return 0;
}
static CYTHON_INLINE int __Pyx_PyErr_GivenExceptionMatches(PyObject *err, PyObject* exc_type) {
    if (likely(err == exc_type)) return 1;
    if (likely(PyExceptionClass_Check(err))) {
        if (likely(PyExceptionClass_Check(exc_type))) {
            return __Pyx_inner_PyErr_GivenExceptionMatches2(err, NULL, exc_type);
        } else if (likely(PyTuple_Check(exc_type))) {
            return __Pyx_PyErr_GivenExceptionMatchesTuple(err, exc_type);
        } else {
        }
    }
    return PyErr_GivenExceptionMatches(err, exc_type);
}
static CYTHON_INLINE int __Pyx_PyErr_GivenExceptionMatches2(PyObject *err, PyObject *exc_type1, PyObject *exc_type2) {
    assert(PyExceptionClass_Check(exc_type1));
    assert(PyExceptionClass_Check(exc_type2));
    if (likely(err == exc_type1 || err == exc_type2)) return 1;
    if (likely(PyExceptionClass_Check(err))) {
        return __Pyx_inner_PyErr_GivenExceptionMatches2(err, exc_type1, exc_type2);
    }
    return (PyErr_GivenExceptionMatches(err, exc_type1) || PyErr_GivenExceptionMatches(err, exc_type2));
}
#endif

/* GetRuntimeVersion */
#if __PYX_LIMITED_VERSION_HEX < 0x030b0000
void __Pyx_init_runtime_version(void) {
    if (__Pyx_cached_runtime_version == 0) {
        const char* rt_version = Py_GetVersion();
        unsigned long version = 0;
        unsigned long factor = 0x01000000UL;
        unsigned int digit = 0;
        int i = 0;
        while (factor) {
            while ('0' <= rt_version[i] && rt_version[i] <= '9') {
                digit = digit * 10 + (unsigned int) (rt_version[i] - '0');
                ++i;
            }
            version += factor * digit;
            if (rt_version[i] != '.')
                break;
            digit = 0;
            factor >>= 8;
            ++i;
        }
        __Pyx_cached_runtime_version = version;
    }
}
#endif
static unsigned long __Pyx_get_runtime_version(void) {
#if __PYX_LIMITED_VERSION_HEX >= 0x030b0000
    return Py_Version & ~0xFFUL;
#else
    return __Pyx_cached_runtime_version;
#endif
}

/* AddModuleRef */
#if CYTHON_COMPILING_IN_CPYTHON_FREETHREADING
  static PyObject *__Pyx_PyImport_AddModuleObjectRef(PyObject *name) {
      PyObject *module_dict = PyImport_GetModuleDict();
      PyObject *m;
      if (PyMapping_GetOptionalItem(module_dict, name, &m) < 0) {
          return NULL;
      }
      if (m != NULL && PyModule_Check(m)) {
          return m;
      }
      Py_XDECREF(m);
      m = PyModule_NewObject(name);
      if (m == NULL)
          return NULL;
      if (PyDict_CheckExact(module_dict)) {
          PyObject *new_m;
          (void)PyDict_SetDefaultRef(module_dict, name, m, &new_m);
          Py_DECREF(m);
          return new_m;
      } else {
           if (PyObject_SetItem(module_dict, name, m) != 0) {
                Py_DECREF(m);
                return NULL;
            }
            return m;
      }
  }
  static PyObject *__Pyx_PyImport_AddModuleRef(const char *name) {
      PyObject *py_name = PyUnicode_FromString(name);
      if (!py_name) return NULL;
      PyObject *module = __Pyx_PyImport_AddModuleObjectRef(py_name);
      Py_DECREF(py_name);
      return module;
  }
#elif __PYX_LIMITED_VERSION_HEX >= 0x030d0000
  #define __Pyx_PyImport_AddModuleRef(name) PyImport_AddModuleRef(name)
#else
  static PyObject *__Pyx_PyImport_AddModuleRef(const char *name) {
      PyObject *module = PyImport_AddModule(name);
      Py_XINCREF(module);
      return module;
  }
#endif

/* CheckBinaryVersion */
static int __Pyx_check_binary_version(unsigned long ct_version, unsigned long rt_version, int allow_newer) {
    const unsigned long MAJOR_MINOR = 0xFFFF0000UL;
    if ((rt_version & MAJOR_MINOR) == (ct_version & MAJOR_MINOR))
        return 0;
    if (likely(allow_newer && (rt_version & MAJOR_MINOR) > (ct_version & MAJOR_MINOR)))
        return 1;
    {
        char message[200];
        PyOS_snprintf(message, sizeof(message),
                      "compile time Python version %d.%d "
                      "of module '%.100s' "
                      "%s "
                      "runtime version %d.%d",
                       (int) (ct_version >> 24), (int) ((ct_version >> 16) & 0xFF),
                       __Pyx_MODULE_NAME,
                       (allow_newer) ? "was newer than" : "does not match",
                       (int) (rt_version >> 24), (int) ((rt_version >> 16) & 0xFF)
       );
        return PyErr_WarnEx(NULL, message, 1);
    }
}

/* DecompressString */
static PyObject *__Pyx_DecompressString(const char *s, Py_ssize_t length, int algo) {
    PyObject *module = NULL, *decompress, *compressed_bytes, *decompressed;
    const char* module_name = algo == 3 ? "compression.zstd" : algo == 2 ? "bz2" : "zlib";
    PyObject *methodname = PyUnicode_FromString("decompress");
    if (unlikely(!methodname)) return NULL;
    #if __PYX_LIMITED_VERSION_HEX >= 0x030e0000
    if (algo == 3) {
        PyObject *fromlist = Py_BuildValue("[O]", methodname);
        if (unlikely(!fromlist)) goto bad;
        module = PyImport_ImportModuleLevel("compression.zstd", NULL, NULL, fromlist, 0);
        Py_DECREF(fromlist);
    } else
    #endif
        module = PyImport_ImportModule(module_name);
    if (unlikely(!module)) goto import_failed;
    decompress = PyObject_GetAttr(module, methodname);
    if (unlikely(!decompress)) goto import_failed;
    {
        #ifdef __cplusplus
            char *memview_bytes = const_cast<char*>(s);
        #else
            #if defined(__clang__)
              #pragma clang diagnostic push
              #pragma clang diagnostic ignored "-Wcast-qual"
            #elif !defined(__INTEL_COMPILER) && defined(__GNUC__)
              #pragma GCC diagnostic push
              #pragma GCC diagnostic ignored "-Wcast-qual"
            #endif
            char *memview_bytes = (char*) s;
            #if defined(__clang__)
              #pragma clang diagnostic pop
            #elif !defined(__INTEL_COMPILER) && defined(__GNUC__)
              #pragma GCC diagnostic pop
            #endif
        #endif
        #if CYTHON_COMPILING_IN_LIMITED_API && !defined(PyBUF_READ)
        int memview_flags = 0x100;
        #else
        int memview_flags = PyBUF_READ;
        #endif
        compressed_bytes = PyMemoryView_FromMemory(memview_bytes, length, memview_flags);
    }
    if (unlikely(!compressed_bytes)) {
        Py_DECREF(decompress);
        goto bad;
    }
    decompressed = PyObject_CallFunctionObjArgs(decompress, compressed_bytes, NULL);
    Py_DECREF(compressed_bytes);
    Py_DECREF(decompress);
    Py_DECREF(module);
    Py_DECREF(methodname);
    return decompressed;
import_failed:
    PyErr_Format(PyExc_ImportError,
        "Failed to import '%.20s.decompress' - cannot initialise module strings. "
        "String compression was configured with the C macro 'CYTHON_COMPRESS_STRINGS=%d'.",
        module_name, algo);
bad:
    Py_XDECREF(module);
    Py_DECREF(methodname);
    return NULL;
}

#include <string.h>
static CYTHON_INLINE Py_ssize_t __Pyx_ssize_strlen(const char *s) {
    size_t len = strlen(s);
    if (unlikely(len > (size_t) PY_SSIZE_T_MAX)) {
        PyErr_SetString(PyExc_OverflowError, "byte string is too long");
        return -1;
    }
    return (Py_ssize_t) len;
}
static CYTHON_INLINE PyObject* __Pyx_PyUnicode_FromString(const char* c_str) {
    Py_ssize_t len = __Pyx_ssize_strlen(c_str);
    if (unlikely(len < 0)) return NULL;
    return __Pyx_PyUnicode_FromStringAndSize(c_str, len);
}
static CYTHON_INLINE PyObject* __Pyx_PyByteArray_FromString(const char* c_str) {
    Py_ssize_t len = __Pyx_ssize_strlen(c_str);
    if (unlikely(len < 0)) return NULL;
    return PyByteArray_FromStringAndSize(c_str, len);
}
static CYTHON_INLINE const char* __Pyx_PyObject_AsString(PyObject* o) {
    Py_ssize_t ignore;
    return __Pyx_PyObject_AsStringAndSize(o, &ignore);
}
#if __PYX_DEFAULT_STRING_ENCODING_IS_ASCII || __PYX_DEFAULT_STRING_ENCODING_IS_UTF8
static CYTHON_INLINE const char* __Pyx_PyUnicode_AsStringAndSize(PyObject* o, Py_ssize_t *length) {
    if (unlikely(__Pyx_PyUnicode_READY(o) == -1)) return NULL;
#if CYTHON_COMPILING_IN_LIMITED_API
    {
        const char* result;
        Py_ssize_t unicode_length;
        CYTHON_MAYBE_UNUSED_VAR(unicode_length); // only for __PYX_DEFAULT_STRING_ENCODING_IS_ASCII
        #if __PYX_LIMITED_VERSION_HEX < 0x030A0000
        if (unlikely(PyArg_Parse(o, "s#", &result, length) < 0)) return NULL;
        #else
        result = PyUnicode_AsUTF8AndSize(o, length);
        #endif
        #if __PYX_DEFAULT_STRING_ENCODING_IS_ASCII
        unicode_length = PyUnicode_GetLength(o);
        if (unlikely(unicode_length < 0)) return NULL;
        if (unlikely(unicode_length != *length)) {
            PyUnicode_AsASCIIString(o);
            return NULL;
        }
        #endif
        return result;
    }
#else
#if __PYX_DEFAULT_STRING_ENCODING_IS_ASCII
    if (likely(PyUnicode_IS_ASCII(o))) {
        *length = PyUnicode_GET_LENGTH(o);
        return PyUnicode_AsUTF8(o);
    } else {
        PyUnicode_AsASCIIString(o);
        return NULL;
    }
#else
    return PyUnicode_AsUTF8AndSize(o, length);
#endif
#endif
}
#endif
static CYTHON_INLINE const char* __Pyx_PyObject_AsStringAndSize(PyObject* o, Py_ssize_t *length) {
#if __PYX_DEFAULT_STRING_ENCODING_IS_ASCII || __PYX_DEFAULT_STRING_ENCODING_IS_UTF8
    if (PyUnicode_Check(o)) {
        return __Pyx_PyUnicode_AsStringAndSize(o, length);
    } else
#endif
    if (PyByteArray_Check(o)) {
#if (CYTHON_ASSUME_SAFE_SIZE && CYTHON_ASSUME_SAFE_MACROS) || (CYTHON_COMPILING_IN_PYPY && (defined(PyByteArray_AS_STRING) && defined(PyByteArray_GET_SIZE)))
        *length = PyByteArray_GET_SIZE(o);
        return PyByteArray_AS_STRING(o);
#else
        *length = PyByteArray_Size(o);
        if (*length == -1) return NULL;
        return PyByteArray_AsString(o);
#endif
    } else
    {
        char* result;
        int r = PyBytes_AsStringAndSize(o, &result, length);
        if (unlikely(r < 0)) {
            return NULL;
        } else {
            return result;
        }
    }
}
static CYTHON_INLINE int __Pyx_PyObject_IsTrue(PyObject* x) {
   int is_true = x == Py_True;
   if (is_true | (x == Py_False) | (x == Py_None)) return is_true;
   else return PyObject_IsTrue(x);
}
static CYTHON_INLINE int __Pyx_PyObject_IsTrueAndDecref(PyObject* x) {
    int retval;
    if (unlikely(!x)) return -1;
    retval = __Pyx_PyObject_IsTrue(x);
    Py_DECREF(x);
    return retval;
}
static PyObject* __Pyx_PyNumber_LongWrongResultType(PyObject* result) {
    __Pyx_TypeName result_type_name = __Pyx_PyType_GetFullyQualifiedName(Py_TYPE(result));
    if (PyLong_Check(result)) {
        if (PyErr_WarnFormat(PyExc_DeprecationWarning, 1,
                "__int__ returned non-int (type " __Pyx_FMT_TYPENAME ").  "
                "The ability to return an instance of a strict subclass of int is deprecated, "
                "and may be removed in a future version of Python.",
                result_type_name)) {
            __Pyx_DECREF_TypeName(result_type_name);
            Py_DECREF(result);
            return NULL;
        }
        __Pyx_DECREF_TypeName(result_type_name);
        return result;
    }
    PyErr_Format(PyExc_TypeError,
                 "__int__ returned non-int (type " __Pyx_FMT_TYPENAME ")",
                 result_type_name);
    __Pyx_DECREF_TypeName(result_type_name);
    Py_DECREF(result);
    return NULL;
}
static CYTHON_INLINE PyObject* __Pyx_PyNumber_Long(PyObject* x) {
#if CYTHON_USE_TYPE_SLOTS
  PyNumberMethods *m;
#endif
  PyObject *res = NULL;
  if (likely(PyLong_Check(x)))
      return __Pyx_NewRef(x);
#if CYTHON_USE_TYPE_SLOTS
  m = Py_TYPE(x)->tp_as_number;
  if (likely(m && m->nb_int)) {
      res = m->nb_int(x);
  }
#else
  if (!PyBytes_CheckExact(x) && !PyUnicode_CheckExact(x)) {
      res = PyNumber_Long(x);
  }
#endif
  if (likely(res)) {
      if (unlikely(!PyLong_CheckExact(res))) {
          return __Pyx_PyNumber_LongWrongResultType(res);
      }
  }
  else if (!PyErr_Occurred()) {
      PyErr_SetString(PyExc_TypeError,
                      "an integer is required");
  }
  return res;
}
static CYTHON_INLINE Py_ssize_t __Pyx_PyIndex_AsSsize_t(PyObject* b) {
  Py_ssize_t ival;
  PyObject *x;
  if (likely(PyLong_CheckExact(b))) {
    #if CYTHON_USE_PYLONG_INTERNALS
    if (likely(__Pyx_PyLong_IsCompact(b))) {
        return __Pyx_PyLong_CompactValue(b);
    } else {
      const digit* digits = __Pyx_PyLong_Digits(b);
      const Py_ssize_t size = __Pyx_PyLong_SignedDigitCount(b);
      switch (size) {
         case 2:
           if (8 * sizeof(Py_ssize_t) > 2 * PyLong_SHIFT) {
             return (Py_ssize_t) (((((size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0]));
           }
           break;
         case -2:
           if (8 * sizeof(Py_ssize_t) > 2 * PyLong_SHIFT) {
             return -(Py_ssize_t) (((((size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0]));
           }
           break;
         case 3:
           if (8 * sizeof(Py_ssize_t) > 3 * PyLong_SHIFT) {
             return (Py_ssize_t) (((((((size_t)digits[2]) << PyLong_SHIFT) | (size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0]));
           }
           break;
         case -3:
           if (8 * sizeof(Py_ssize_t) > 3 * PyLong_SHIFT) {
             return -(Py_ssize_t) (((((((size_t)digits[2]) << PyLong_SHIFT) | (size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0]));
           }
           break;
         case 4:
           if (8 * sizeof(Py_ssize_t) > 4 * PyLong_SHIFT) {
             return (Py_ssize_t) (((((((((size_t)digits[3]) << PyLong_SHIFT) | (size_t)digits[2]) << PyLong_SHIFT) | (size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0]));
           }
           break;
         case -4:
           if (8 * sizeof(Py_ssize_t) > 4 * PyLong_SHIFT) {
             return -(Py_ssize_t) (((((((((size_t)digits[3]) << PyLong_SHIFT) | (size_t)digits[2]) << PyLong_SHIFT) | (size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0]));
           }
           break;
      }
    }
    #endif
    return PyLong_AsSsize_t(b);
  }
  x = PyNumber_Index(b);
  if (!x) return -1;
  ival = PyLong_AsSsize_t(x);
  Py_DECREF(x);
  return ival;
}
static CYTHON_INLINE Py_hash_t __Pyx_PyIndex_AsHash_t(PyObject* o) {
  if (sizeof(Py_hash_t) == sizeof(Py_ssize_t)) {
    return (Py_hash_t) __Pyx_PyIndex_AsSsize_t(o);
  } else {
    Py_ssize_t ival;
    PyObject *x;
    x = PyNumber_Index(o);
    if (!x) return -1;
    ival = PyLong_AsLong(x);
    Py_DECREF(x);
    return ival;
  }
}
static CYTHON_INLINE PyObject *__Pyx_Owned_Py_None(int b) {
    CYTHON_UNUSED_VAR(b);
    return __Pyx_NewRef(Py_None);
}
static CYTHON_INLINE PyObject * __Pyx_PyBool_FromLong(long b) {
  return __Pyx_NewRef(b ? Py_True: Py_False);
}
static CYTHON_INLINE PyObject * __Pyx_PyLong_FromSize_t(size_t ival) {
    return PyLong_FromSize_t(ival);
}


/* MultiPhaseInitModuleState */
#if CYTHON_PEP489_MULTI_PHASE_INIT && CYTHON_USE_MODULE_STATE
#ifndef CYTHON_MODULE_STATE_LOOKUP_THREAD_SAFE
#if (CYTHON_COMPILING_IN_LIMITED_API || PY_VERSION_HEX >= 0x030C0000)
  #define CYTHON_MODULE_STATE_LOOKUP_THREAD_SAFE 1
#else
  #define CYTHON_MODULE_STATE_LOOKUP_THREAD_SAFE 0
#endif
#endif
#if CYTHON_MODULE_STATE_LOOKUP_THREAD_SAFE && !CYTHON_ATOMICS
#error "Module state with PEP489 requires atomics. Currently that's one of\
 C11, C++11, gcc atomic intrinsics or MSVC atomic intrinsics"
#endif
#if !CYTHON_MODULE_STATE_LOOKUP_THREAD_SAFE
#define __Pyx_ModuleStateLookup_Lock()
#define __Pyx_ModuleStateLookup_Unlock()
#elif !CYTHON_COMPILING_IN_LIMITED_API && PY_VERSION_HEX >= 0x030d0000
static PyMutex __Pyx_ModuleStateLookup_mutex = {0};
#define __Pyx_ModuleStateLookup_Lock() PyMutex_Lock(&__Pyx_ModuleStateLookup_mutex)
#define __Pyx_ModuleStateLookup_Unlock() PyMutex_Unlock(&__Pyx_ModuleStateLookup_mutex)
#elif defined(__cplusplus) && __cplusplus >= 201103L
#include <mutex>
static std::mutex __Pyx_ModuleStateLookup_mutex;
#define __Pyx_ModuleStateLookup_Lock() __Pyx_ModuleStateLookup_mutex.lock()
#define __Pyx_ModuleStateLookup_Unlock() __Pyx_ModuleStateLookup_mutex.unlock()
#elif defined(__STDC_VERSION__) && (__STDC_VERSION__ > 201112L) && !defined(__STDC_NO_THREADS__)
#include <threads.h>
static mtx_t __Pyx_ModuleStateLookup_mutex;
static once_flag __Pyx_ModuleStateLookup_mutex_once_flag = ONCE_FLAG_INIT;
static void __Pyx_ModuleStateLookup_initialize_mutex(void) {
    mtx_init(&__Pyx_ModuleStateLookup_mutex, mtx_plain);
}
#define __Pyx_ModuleStateLookup_Lock()\
  call_once(&__Pyx_ModuleStateLookup_mutex_once_flag, __Pyx_ModuleStateLookup_initialize_mutex);\
  mtx_lock(&__Pyx_ModuleStateLookup_mutex)
#define __Pyx_ModuleStateLookup_Unlock() mtx_unlock(&__Pyx_ModuleStateLookup_mutex)
#elif defined(HAVE_PTHREAD_H)
#include <pthread.h>
static pthread_mutex_t __Pyx_ModuleStateLookup_mutex = PTHREAD_MUTEX_INITIALIZER;
#define __Pyx_ModuleStateLookup_Lock() pthread_mutex_lock(&__Pyx_ModuleStateLookup_mutex)
#define __Pyx_ModuleStateLookup_Unlock() pthread_mutex_unlock(&__Pyx_ModuleStateLookup_mutex)
#elif defined(_WIN32)
#include <Windows.h>  // synchapi.h on its own doesn't work
static SRWLOCK __Pyx_ModuleStateLookup_mutex = SRWLOCK_INIT;
#define __Pyx_ModuleStateLookup_Lock() AcquireSRWLockExclusive(&__Pyx_ModuleStateLookup_mutex)
#define __Pyx_ModuleStateLookup_Unlock() ReleaseSRWLockExclusive(&__Pyx_ModuleStateLookup_mutex)
#else
#error "No suitable lock available for CYTHON_MODULE_STATE_LOOKUP_THREAD_SAFE.\
 Requires C standard >= C11, or C++ standard >= C++11,\
 or pthreads, or the Windows 32 API, or Python >= 3.13."
#endif
typedef struct {
    int64_t id;
    PyObject *module;
} __Pyx_InterpreterIdAndModule;
typedef struct {
    char interpreter_id_as_index;
    Py_ssize_t count;
    Py_ssize_t allocated;
    __Pyx_InterpreterIdAndModule table[1];
} __Pyx_ModuleStateLookupData;
#define __PYX_MODULE_STATE_LOOKUP_SMALL_SIZE 32
#if CYTHON_MODULE_STATE_LOOKUP_THREAD_SAFE
static __pyx_atomic_int_type __Pyx_ModuleStateLookup_read_counter = 0;
#endif
#if CYTHON_MODULE_STATE_LOOKUP_THREAD_SAFE
static __pyx_atomic_ptr_type __Pyx_ModuleStateLookup_data = 0;
#else
static __Pyx_ModuleStateLookupData* __Pyx_ModuleStateLookup_data = NULL;
#endif
static __Pyx_InterpreterIdAndModule* __Pyx_State_FindModuleStateLookupTableLowerBound(
        __Pyx_InterpreterIdAndModule* table,
        Py_ssize_t count,
        int64_t interpreterId) {
    __Pyx_InterpreterIdAndModule* begin = table;
    __Pyx_InterpreterIdAndModule* end = begin + count;
    if (begin->id == interpreterId) {
        return begin;
    }
    while ((end - begin) > __PYX_MODULE_STATE_LOOKUP_SMALL_SIZE) {
        __Pyx_InterpreterIdAndModule* halfway = begin + (end - begin)/2;
        if (halfway->id == interpreterId) {
            return halfway;
        }
        if (halfway->id < interpreterId) {
            begin = halfway;
        } else {
            end = halfway;
        }
    }
    for (; begin < end; ++begin) {
        if (begin->id >= interpreterId) return begin;
    }
    return begin;
}
static PyObject *__Pyx_State_FindModule(CYTHON_UNUSED void* dummy) {
    int64_t interpreter_id = PyInterpreterState_GetID(__Pyx_PyInterpreterState_Get());
    if (interpreter_id == -1) return NULL;
#if CYTHON_MODULE_STATE_LOOKUP_THREAD_SAFE
    __Pyx_ModuleStateLookupData* data = (__Pyx_ModuleStateLookupData*)__pyx_atomic_pointer_load_relaxed(&__Pyx_ModuleStateLookup_data);
    {
        __pyx_atomic_incr_acq_rel(&__Pyx_ModuleStateLookup_read_counter);
        if (likely(data)) {
            __Pyx_ModuleStateLookupData* new_data = (__Pyx_ModuleStateLookupData*)__pyx_atomic_pointer_load_acquire(&__Pyx_ModuleStateLookup_data);
            if (likely(data == new_data)) {
                goto read_finished;
            }
        }
        __pyx_atomic_decr_acq_rel(&__Pyx_ModuleStateLookup_read_counter);
        __Pyx_ModuleStateLookup_Lock();
        __pyx_atomic_incr_relaxed(&__Pyx_ModuleStateLookup_read_counter);
        data = (__Pyx_ModuleStateLookupData*)__pyx_atomic_pointer_load_relaxed(&__Pyx_ModuleStateLookup_data);
        __Pyx_ModuleStateLookup_Unlock();
    }
  read_finished:;
#else
    __Pyx_ModuleStateLookupData* data = __Pyx_ModuleStateLookup_data;
#endif
    __Pyx_InterpreterIdAndModule* found = NULL;
    if (unlikely(!data)) goto end;
    if (data->interpreter_id_as_index) {
        if (interpreter_id < data->count) {
            found = data->table+interpreter_id;
        }
    } else {
        found = __Pyx_State_FindModuleStateLookupTableLowerBound(
            data->table, data->count, interpreter_id);
    }
  end:
    {
        PyObject *result=NULL;
        if (found && found->id == interpreter_id) {
            result = found->module;
        }
#if CYTHON_MODULE_STATE_LOOKUP_THREAD_SAFE
        __pyx_atomic_decr_acq_rel(&__Pyx_ModuleStateLookup_read_counter);
#endif
        return result;
    }
}
#if CYTHON_MODULE_STATE_LOOKUP_THREAD_SAFE
static void __Pyx_ModuleStateLookup_wait_until_no_readers(void) {
    while (__pyx_atomic_load(&__Pyx_ModuleStateLookup_read_counter) != 0);
}
#else
#define __Pyx_ModuleStateLookup_wait_until_no_readers()
#endif
static int __Pyx_State_AddModuleInterpIdAsIndex(__Pyx_ModuleStateLookupData **old_data, PyObject* module, int64_t interpreter_id) {
    Py_ssize_t to_allocate = (*old_data)->allocated;
    while (to_allocate <= interpreter_id) {
        if (to_allocate == 0) to_allocate = 1;
        else to_allocate *= 2;
    }
    __Pyx_ModuleStateLookupData *new_data = *old_data;
    if (to_allocate != (*old_data)->allocated) {
         new_data = (__Pyx_ModuleStateLookupData *)realloc(
            *old_data,
            sizeof(__Pyx_ModuleStateLookupData)+(to_allocate-1)*sizeof(__Pyx_InterpreterIdAndModule));
        if (!new_data) {
            PyErr_NoMemory();
            return -1;
        }
        for (Py_ssize_t i = new_data->allocated; i < to_allocate; ++i) {
            new_data->table[i].id = i;
            new_data->table[i].module = NULL;
        }
        new_data->allocated = to_allocate;
    }
    new_data->table[interpreter_id].module = module;
    if (new_data->count < interpreter_id+1) {
        new_data->count = interpreter_id+1;
    }
    *old_data = new_data;
    return 0;
}
static void __Pyx_State_ConvertFromInterpIdAsIndex(__Pyx_ModuleStateLookupData *data) {
    __Pyx_InterpreterIdAndModule *read = data->table;
    __Pyx_InterpreterIdAndModule *write = data->table;
    __Pyx_InterpreterIdAndModule *end = read + data->count;
    for (; read<end; ++read) {
        if (read->module) {
            write->id = read->id;
            write->module = read->module;
            ++write;
        }
    }
    data->count = write - data->table;
    for (; write<end; ++write) {
        write->id = 0;
        write->module = NULL;
    }
    data->interpreter_id_as_index = 0;
}
static int __Pyx_State_AddModule(PyObject* module, CYTHON_UNUSED void* dummy) {
    int64_t interpreter_id = PyInterpreterState_GetID(__Pyx_PyInterpreterState_Get());
    if (interpreter_id == -1) return -1;
    int result = 0;
    __Pyx_ModuleStateLookup_Lock();
#if CYTHON_MODULE_STATE_LOOKUP_THREAD_SAFE
    __Pyx_ModuleStateLookupData *old_data = (__Pyx_ModuleStateLookupData *)
            __pyx_atomic_pointer_exchange(&__Pyx_ModuleStateLookup_data, 0);
#else
    __Pyx_ModuleStateLookupData *old_data = __Pyx_ModuleStateLookup_data;
#endif
    __Pyx_ModuleStateLookupData *new_data = old_data;
    if (!new_data) {
        new_data = (__Pyx_ModuleStateLookupData *)calloc(1, sizeof(__Pyx_ModuleStateLookupData));
        if (!new_data) {
            result = -1;
            PyErr_NoMemory();
            goto end;
        }
        new_data->allocated = 1;
        new_data->interpreter_id_as_index = 1;
    }
    __Pyx_ModuleStateLookup_wait_until_no_readers();
    if (new_data->interpreter_id_as_index) {
        if (interpreter_id < __PYX_MODULE_STATE_LOOKUP_SMALL_SIZE) {
            result = __Pyx_State_AddModuleInterpIdAsIndex(&new_data, module, interpreter_id);
            goto end;
        }
        __Pyx_State_ConvertFromInterpIdAsIndex(new_data);
    }
    {
        Py_ssize_t insert_at = 0;
        {
            __Pyx_InterpreterIdAndModule* lower_bound = __Pyx_State_FindModuleStateLookupTableLowerBound(
                new_data->table, new_data->count, interpreter_id);
            assert(lower_bound);
            insert_at = lower_bound - new_data->table;
            if (unlikely(insert_at < new_data->count && lower_bound->id == interpreter_id)) {
                lower_bound->module = module;
                goto end;  // already in table, nothing more to do
            }
        }
        if (new_data->count+1 >= new_data->allocated) {
            Py_ssize_t to_allocate = (new_data->count+1)*2;
            new_data =
                (__Pyx_ModuleStateLookupData*)realloc(
                    new_data,
                    sizeof(__Pyx_ModuleStateLookupData) +
                    (to_allocate-1)*sizeof(__Pyx_InterpreterIdAndModule));
            if (!new_data) {
                result = -1;
                new_data = old_data;
                PyErr_NoMemory();
                goto end;
            }
            new_data->allocated = to_allocate;
        }
        ++new_data->count;
        int64_t last_id = interpreter_id;
        PyObject *last_module = module;
        for (Py_ssize_t i=insert_at; i<new_data->count; ++i) {
            int64_t current_id = new_data->table[i].id;
            new_data->table[i].id = last_id;
            last_id = current_id;
            PyObject *current_module = new_data->table[i].module;
            new_data->table[i].module = last_module;
            last_module = current_module;
        }
    }
  end:
#if CYTHON_MODULE_STATE_LOOKUP_THREAD_SAFE
    __pyx_atomic_pointer_exchange(&__Pyx_ModuleStateLookup_data, new_data);
#else
    __Pyx_ModuleStateLookup_data = new_data;
#endif
    __Pyx_ModuleStateLookup_Unlock();
    return result;
}
static int __Pyx_State_RemoveModule(CYTHON_UNUSED void* dummy) {
    int64_t interpreter_id = PyInterpreterState_GetID(__Pyx_PyInterpreterState_Get());
    if (interpreter_id == -1) return -1;
    __Pyx_ModuleStateLookup_Lock();
#if CYTHON_MODULE_STATE_LOOKUP_THREAD_SAFE
    __Pyx_ModuleStateLookupData *data = (__Pyx_ModuleStateLookupData *)
            __pyx_atomic_pointer_exchange(&__Pyx_ModuleStateLookup_data, 0);
#else
    __Pyx_ModuleStateLookupData *data = __Pyx_ModuleStateLookup_data;
#endif
    if (data->interpreter_id_as_index) {
        if (interpreter_id < data->count) {
            data->table[interpreter_id].module = NULL;
        }
        goto done;
    }
    {
        __Pyx_ModuleStateLookup_wait_until_no_readers();
        __Pyx_InterpreterIdAndModule* lower_bound = __Pyx_State_FindModuleStateLookupTableLowerBound(
            data->table, data->count, interpreter_id);
        if (!lower_bound) goto done;
        if (lower_bound->id != interpreter_id) goto done;
        __Pyx_InterpreterIdAndModule *end = data->table+data->count;
        for (;lower_bound<end-1; ++lower_bound) {
            lower_bound->id = (lower_bound+1)->id;
            lower_bound->module = (lower_bound+1)->module;
        }
    }
    --data->count;
    if (data->count == 0) {
        free(data);
        data = NULL;
    }
  done:
#if CYTHON_MODULE_STATE_LOOKUP_THREAD_SAFE
    __pyx_atomic_pointer_exchange(&__Pyx_ModuleStateLookup_data, data);
#else
    __Pyx_ModuleStateLookup_data = data;
#endif
    __Pyx_ModuleStateLookup_Unlock();
    return 0;
}
#endif

/* #### Code section: utility_code_pragmas_end ### */
#ifdef _MSC_VER
#pragma warning( pop )
#endif



/* #### Code section: end ### */
#endif /* Py_PYTHON_H */
