/* Generated by Cython 3.2.5 */

/* BEGIN: Cython Metadata
{
    "distutils": {
        "define_macros": [
            [
                "_FORCE_INLINES",
                "1"
            ],
            [
                "CYTHON_EXTERN_C",
                "extern \"C\""
            ],
            [
                "NPY_TARGET_VERSION",
                "NPY_2_0_API_VERSION"
            ],
            [
                "CUPY_USE_HIP",
                "1"
            ],
            [
                "__HIP_PLATFORM_AMD__",
                "1"
            ],
            [
                "__HIP_PLATFORM_HCC__",
                "1"
            ],
            [
                "THRUST_DEVICE_SYSTEM",
                "THRUST_DEVICE_SYSTEM_HIP"
            ],
            [
                "CUPY_CACHE_KEY",
                "28a6d0d64fe089c15e4ff802c7b0f8edb50d2a99"
            ],
            [
                "CUPY_CUB_VERSION_CODE",
                400200
            ]
        ],
        "depends": [],
        "extra_compile_args": [
            "-std=c++14"
        ],
        "extra_link_args": [
            "-Wl,--disable-new-dtags,-rpath,/opt/rocm/hipfft/lib,-rpath,/opt/rocm/lib"
        ],
        "include_dirs": [
            "/opt/rocm/hipfft/include",
            "/opt/rocm/include/hipcub",
            "/build/python-cupy/src/python-cupy-rocm-gfx11/cupy/_core/include",
            "/opt/rocm/include",
            "/opt/rocm/include/hip",
            "/opt/rocm/include/rocrand",
            "/opt/rocm/include/hiprand",
            "/opt/rocm/include/roctracer",
            "/opt/rocm/include/hipblas",
            "/opt/rocm/include/hipsparse",
            "/opt/rocm/include/hipfft",
            "/opt/rocm/include/rocsolver",
            "/opt/rocm/include/rccl",
            "/usr/lib/python3.14/site-packages/numpy/_core/include"
        ],
        "language": "c++",
        "libraries": [
            "amdhip64",
            "hipblas",
            "hipfft",
            "hiprand",
            "hipsparse",
            "rocfft",
            "roctx64",
            "rocblas",
            "rocsolver",
            "rocsparse"
        ],
        "library_dirs": [
            "/opt/rocm/hipfft/lib",
            "/opt/rocm/lib"
        ],
        "name": "cupy.cuda.memory",
        "sources": [
            "cupy/cuda/memory.pyx"
        ]
    },
    "module_name": "cupy.cuda.memory"
}
END: Cython Metadata */

#ifndef PY_SSIZE_T_CLEAN
#define PY_SSIZE_T_CLEAN
#endif /* PY_SSIZE_T_CLEAN */
/* InitLimitedAPI */
#if defined(Py_LIMITED_API)
  #if !defined(CYTHON_LIMITED_API)
  #define CYTHON_LIMITED_API 1
  #endif
#elif defined(CYTHON_LIMITED_API)
  #ifdef _MSC_VER
  #pragma message ("Limited API usage is enabled with 'CYTHON_LIMITED_API' but 'Py_LIMITED_API' does not define a Python target version. Consider setting 'Py_LIMITED_API' instead.")
  #else
  #warning Limited API usage is enabled with 'CYTHON_LIMITED_API' but 'Py_LIMITED_API' does not define a Python target version. Consider setting 'Py_LIMITED_API' instead.
  #endif
#endif

#include "Python.h"
#ifndef Py_PYTHON_H
    #error Python headers needed to compile C extensions, please install development version of Python.
#elif PY_VERSION_HEX < 0x03080000
    #error Cython requires Python 3.8+.
#else
#define __PYX_ABI_VERSION "3_2_5"
#define CYTHON_HEX_VERSION 0x030205F0
#define CYTHON_FUTURE_DIVISION 1
/* CModulePreamble */
#include <stddef.h>
#ifndef offsetof
  #define offsetof(type, member) ( (size_t) & ((type*)0) -> member )
#endif
#if !defined(_WIN32) && !defined(WIN32) && !defined(MS_WINDOWS)
  #ifndef __stdcall
    #define __stdcall
  #endif
  #ifndef __cdecl
    #define __cdecl
  #endif
  #ifndef __fastcall
    #define __fastcall
  #endif
#endif
#ifndef DL_IMPORT
  #define DL_IMPORT(t) t
#endif
#ifndef DL_EXPORT
  #define DL_EXPORT(t) t
#endif
#define __PYX_COMMA ,
#ifndef PY_LONG_LONG
  #define PY_LONG_LONG LONG_LONG
#endif
#ifndef Py_HUGE_VAL
  #define Py_HUGE_VAL HUGE_VAL
#endif
#define __PYX_LIMITED_VERSION_HEX PY_VERSION_HEX
#if defined(GRAALVM_PYTHON)
  /* For very preliminary testing purposes. Most variables are set the same as PyPy.
     The existence of this section does not imply that anything works or is even tested */
  #define CYTHON_COMPILING_IN_PYPY 0
  #define CYTHON_COMPILING_IN_CPYTHON 0
  #define CYTHON_COMPILING_IN_LIMITED_API 0
  #define CYTHON_COMPILING_IN_GRAAL 1
  #define CYTHON_COMPILING_IN_CPYTHON_FREETHREADING 0
  #undef CYTHON_USE_TYPE_SLOTS
  #define CYTHON_USE_TYPE_SLOTS 0
  #undef CYTHON_USE_TYPE_SPECS
  #define CYTHON_USE_TYPE_SPECS 0
  #undef CYTHON_USE_PYTYPE_LOOKUP
  #define CYTHON_USE_PYTYPE_LOOKUP 0
  #undef CYTHON_USE_PYLIST_INTERNALS
  #define CYTHON_USE_PYLIST_INTERNALS 0
  #undef CYTHON_USE_UNICODE_INTERNALS
  #define CYTHON_USE_UNICODE_INTERNALS 0
  #undef CYTHON_USE_UNICODE_WRITER
  #define CYTHON_USE_UNICODE_WRITER 0
  #undef CYTHON_USE_PYLONG_INTERNALS
  #define CYTHON_USE_PYLONG_INTERNALS 0
  #undef CYTHON_AVOID_BORROWED_REFS
  #define CYTHON_AVOID_BORROWED_REFS 1
  #undef CYTHON_AVOID_THREAD_UNSAFE_BORROWED_REFS
  #define CYTHON_AVOID_THREAD_UNSAFE_BORROWED_REFS 0
  #undef CYTHON_ASSUME_SAFE_MACROS
  #define CYTHON_ASSUME_SAFE_MACROS 0
  #undef CYTHON_ASSUME_SAFE_SIZE
  #define CYTHON_ASSUME_SAFE_SIZE 0
  #undef CYTHON_UNPACK_METHODS
  #define CYTHON_UNPACK_METHODS 0
  #undef CYTHON_FAST_THREAD_STATE
  #define CYTHON_FAST_THREAD_STATE 0
  #undef CYTHON_FAST_GIL
  #define CYTHON_FAST_GIL 0
  #undef CYTHON_METH_FASTCALL
  #define CYTHON_METH_FASTCALL 0
  #undef CYTHON_FAST_PYCALL
  #define CYTHON_FAST_PYCALL 0
  #ifndef CYTHON_PEP487_INIT_SUBCLASS
    #define CYTHON_PEP487_INIT_SUBCLASS 1
  #endif
  #undef CYTHON_PEP489_MULTI_PHASE_INIT
  #define CYTHON_PEP489_MULTI_PHASE_INIT 1
  #undef CYTHON_USE_MODULE_STATE
  #define CYTHON_USE_MODULE_STATE 0
  #undef CYTHON_USE_SYS_MONITORING
  #define CYTHON_USE_SYS_MONITORING 0
  #undef CYTHON_USE_TP_FINALIZE
  #define CYTHON_USE_TP_FINALIZE 0
  #undef CYTHON_USE_AM_SEND
  #define CYTHON_USE_AM_SEND 0
  #undef CYTHON_USE_DICT_VERSIONS
  #define CYTHON_USE_DICT_VERSIONS 0
  #undef CYTHON_USE_EXC_INFO_STACK
  #define CYTHON_USE_EXC_INFO_STACK 1
  #ifndef CYTHON_UPDATE_DESCRIPTOR_DOC
    #define CYTHON_UPDATE_DESCRIPTOR_DOC 0
  #endif
  #undef CYTHON_USE_FREELISTS
  #define CYTHON_USE_FREELISTS 0
  #undef CYTHON_IMMORTAL_CONSTANTS
  #define CYTHON_IMMORTAL_CONSTANTS 0
#elif defined(PYPY_VERSION)
  #define CYTHON_COMPILING_IN_PYPY 1
  #define CYTHON_COMPILING_IN_CPYTHON 0
  #define CYTHON_COMPILING_IN_LIMITED_API 0
  #define CYTHON_COMPILING_IN_GRAAL 0
  #define CYTHON_COMPILING_IN_CPYTHON_FREETHREADING 0
  #undef CYTHON_USE_TYPE_SLOTS
  #define CYTHON_USE_TYPE_SLOTS 1
  #ifndef CYTHON_USE_TYPE_SPECS
    #define CYTHON_USE_TYPE_SPECS 0
  #endif
  #undef CYTHON_USE_PYTYPE_LOOKUP
  #define CYTHON_USE_PYTYPE_LOOKUP 0
  #undef CYTHON_USE_PYLIST_INTERNALS
  #define CYTHON_USE_PYLIST_INTERNALS 0
  #undef CYTHON_USE_UNICODE_INTERNALS
  #define CYTHON_USE_UNICODE_INTERNALS 0
  #undef CYTHON_USE_UNICODE_WRITER
  #define CYTHON_USE_UNICODE_WRITER 0
  #undef CYTHON_USE_PYLONG_INTERNALS
  #define CYTHON_USE_PYLONG_INTERNALS 0
  #undef CYTHON_AVOID_BORROWED_REFS
  #define CYTHON_AVOID_BORROWED_REFS 1
  #undef CYTHON_AVOID_THREAD_UNSAFE_BORROWED_REFS
  #define CYTHON_AVOID_THREAD_UNSAFE_BORROWED_REFS 1
  #undef CYTHON_ASSUME_SAFE_MACROS
  #define CYTHON_ASSUME_SAFE_MACROS 0
  #ifndef CYTHON_ASSUME_SAFE_SIZE
    #define CYTHON_ASSUME_SAFE_SIZE 1
  #endif
  #undef CYTHON_UNPACK_METHODS
  #define CYTHON_UNPACK_METHODS 0
  #undef CYTHON_FAST_THREAD_STATE
  #define CYTHON_FAST_THREAD_STATE 0
  #undef CYTHON_FAST_GIL
  #define CYTHON_FAST_GIL 0
  #undef CYTHON_METH_FASTCALL
  #define CYTHON_METH_FASTCALL 0
  #undef CYTHON_FAST_PYCALL
  #define CYTHON_FAST_PYCALL 0
  #ifndef CYTHON_PEP487_INIT_SUBCLASS
    #define CYTHON_PEP487_INIT_SUBCLASS 1
  #endif
  #if PY_VERSION_HEX < 0x03090000
    #undef CYTHON_PEP489_MULTI_PHASE_INIT
    #define CYTHON_PEP489_MULTI_PHASE_INIT 0
  #elif !defined(CYTHON_PEP489_MULTI_PHASE_INIT)
    #define CYTHON_PEP489_MULTI_PHASE_INIT 1
  #endif
  #undef CYTHON_USE_MODULE_STATE
  #define CYTHON_USE_MODULE_STATE 0
  #undef CYTHON_USE_SYS_MONITORING
  #define CYTHON_USE_SYS_MONITORING 0
  #ifndef CYTHON_USE_TP_FINALIZE
    #define CYTHON_USE_TP_FINALIZE (PYPY_VERSION_NUM >= 0x07030C00)
  #endif
  #undef CYTHON_USE_AM_SEND
  #define CYTHON_USE_AM_SEND 0
  #undef CYTHON_USE_DICT_VERSIONS
  #define CYTHON_USE_DICT_VERSIONS 0
  #undef CYTHON_USE_EXC_INFO_STACK
  #define CYTHON_USE_EXC_INFO_STACK 0
  #ifndef CYTHON_UPDATE_DESCRIPTOR_DOC
    #define CYTHON_UPDATE_DESCRIPTOR_DOC (PYPY_VERSION_NUM >= 0x07031100)
  #endif
  #undef CYTHON_USE_FREELISTS
  #define CYTHON_USE_FREELISTS 0
  #undef CYTHON_IMMORTAL_CONSTANTS
  #define CYTHON_IMMORTAL_CONSTANTS 0
#elif defined(CYTHON_LIMITED_API)
  #ifdef Py_LIMITED_API
    #undef __PYX_LIMITED_VERSION_HEX
    #define __PYX_LIMITED_VERSION_HEX Py_LIMITED_API
  #endif
  #define CYTHON_COMPILING_IN_PYPY 0
  #define CYTHON_COMPILING_IN_CPYTHON 0
  #define CYTHON_COMPILING_IN_LIMITED_API 1
  #define CYTHON_COMPILING_IN_GRAAL 0
  #define CYTHON_COMPILING_IN_CPYTHON_FREETHREADING 0
  #undef CYTHON_USE_TYPE_SLOTS
  #define CYTHON_USE_TYPE_SLOTS 0
  #undef CYTHON_USE_TYPE_SPECS
  #define CYTHON_USE_TYPE_SPECS 1
  #undef CYTHON_USE_PYTYPE_LOOKUP
  #define CYTHON_USE_PYTYPE_LOOKUP 0
  #undef CYTHON_USE_PYLIST_INTERNALS
  #define CYTHON_USE_PYLIST_INTERNALS 0
  #undef CYTHON_USE_UNICODE_INTERNALS
  #define CYTHON_USE_UNICODE_INTERNALS 0
  #ifndef CYTHON_USE_UNICODE_WRITER
    #define CYTHON_USE_UNICODE_WRITER 0
  #endif
  #undef CYTHON_USE_PYLONG_INTERNALS
  #define CYTHON_USE_PYLONG_INTERNALS 0
  #ifndef CYTHON_AVOID_BORROWED_REFS
    #define CYTHON_AVOID_BORROWED_REFS 0
  #endif
  #ifndef CYTHON_AVOID_THREAD_UNSAFE_BORROWED_REFS
    #define CYTHON_AVOID_THREAD_UNSAFE_BORROWED_REFS 0
  #endif
  #undef CYTHON_ASSUME_SAFE_MACROS
  #define CYTHON_ASSUME_SAFE_MACROS 0
  #undef CYTHON_ASSUME_SAFE_SIZE
  #define CYTHON_ASSUME_SAFE_SIZE 0
  #undef CYTHON_UNPACK_METHODS
  #define CYTHON_UNPACK_METHODS 0
  #undef CYTHON_FAST_THREAD_STATE
  #define CYTHON_FAST_THREAD_STATE 0
  #undef CYTHON_FAST_GIL
  #define CYTHON_FAST_GIL 0
  #undef CYTHON_METH_FASTCALL
  #define CYTHON_METH_FASTCALL (__PYX_LIMITED_VERSION_HEX >= 0x030C0000)
  #undef CYTHON_FAST_PYCALL
  #define CYTHON_FAST_PYCALL 0
  #ifndef CYTHON_PEP487_INIT_SUBCLASS
    #define CYTHON_PEP487_INIT_SUBCLASS 1
  #endif
  #ifndef CYTHON_PEP489_MULTI_PHASE_INIT
    #define CYTHON_PEP489_MULTI_PHASE_INIT 1
  #endif
  #ifndef CYTHON_USE_MODULE_STATE
    #define CYTHON_USE_MODULE_STATE 0
  #endif
  #undef CYTHON_USE_SYS_MONITORING
  #define CYTHON_USE_SYS_MONITORING 0
  #ifndef CYTHON_USE_TP_FINALIZE
    #define CYTHON_USE_TP_FINALIZE 0
  #endif
  #ifndef CYTHON_USE_AM_SEND
    #define CYTHON_USE_AM_SEND (__PYX_LIMITED_VERSION_HEX >= 0x030A0000)
  #endif
  #undef CYTHON_USE_DICT_VERSIONS
  #define CYTHON_USE_DICT_VERSIONS 0
  #undef CYTHON_USE_EXC_INFO_STACK
  #define CYTHON_USE_EXC_INFO_STACK 0
  #ifndef CYTHON_UPDATE_DESCRIPTOR_DOC
    #define CYTHON_UPDATE_DESCRIPTOR_DOC 0
  #endif
  #ifndef CYTHON_USE_FREELISTS
  #define CYTHON_USE_FREELISTS 1
  #endif
  #undef CYTHON_IMMORTAL_CONSTANTS
  #define CYTHON_IMMORTAL_CONSTANTS 0
#else
  #define CYTHON_COMPILING_IN_PYPY 0
  #define CYTHON_COMPILING_IN_CPYTHON 1
  #define CYTHON_COMPILING_IN_LIMITED_API 0
  #define CYTHON_COMPILING_IN_GRAAL 0
  #ifdef Py_GIL_DISABLED
    #define CYTHON_COMPILING_IN_CPYTHON_FREETHREADING 1
  #else
    #define CYTHON_COMPILING_IN_CPYTHON_FREETHREADING 0
  #endif
  #if PY_VERSION_HEX < 0x030A0000
    #undef CYTHON_USE_TYPE_SLOTS
    #define CYTHON_USE_TYPE_SLOTS 1
  #elif !defined(CYTHON_USE_TYPE_SLOTS)
    #define CYTHON_USE_TYPE_SLOTS 1
  #endif
  #ifndef CYTHON_USE_TYPE_SPECS
    #define CYTHON_USE_TYPE_SPECS 0
  #endif
  #ifndef CYTHON_USE_PYTYPE_LOOKUP
    #define CYTHON_USE_PYTYPE_LOOKUP 1
  #endif
  #ifndef CYTHON_USE_PYLONG_INTERNALS
    #define CYTHON_USE_PYLONG_INTERNALS 1
  #endif
  #if CYTHON_COMPILING_IN_CPYTHON_FREETHREADING
    #undef CYTHON_USE_PYLIST_INTERNALS
    #define CYTHON_USE_PYLIST_INTERNALS 0
  #elif !defined(CYTHON_USE_PYLIST_INTERNALS)
    #define CYTHON_USE_PYLIST_INTERNALS 1
  #endif
  #ifndef CYTHON_USE_UNICODE_INTERNALS
    #define CYTHON_USE_UNICODE_INTERNALS 1
  #endif
  #if CYTHON_COMPILING_IN_CPYTHON_FREETHREADING || PY_VERSION_HEX >= 0x030B00A2
    #undef CYTHON_USE_UNICODE_WRITER
    #define CYTHON_USE_UNICODE_WRITER 0
  #elif !defined(CYTHON_USE_UNICODE_WRITER)
    #define CYTHON_USE_UNICODE_WRITER 1
  #endif
  #ifndef CYTHON_AVOID_BORROWED_REFS
    #define CYTHON_AVOID_BORROWED_REFS 0
  #endif
  #if CYTHON_COMPILING_IN_CPYTHON_FREETHREADING
    #undef CYTHON_AVOID_THREAD_UNSAFE_BORROWED_REFS
    #define CYTHON_AVOID_THREAD_UNSAFE_BORROWED_REFS 1
  #elif !defined(CYTHON_AVOID_THREAD_UNSAFE_BORROWED_REFS)
    #define CYTHON_AVOID_THREAD_UNSAFE_BORROWED_REFS 0
  #endif
  #ifndef CYTHON_ASSUME_SAFE_MACROS
    #define CYTHON_ASSUME_SAFE_MACROS 1
  #endif
  #ifndef CYTHON_ASSUME_SAFE_SIZE
    #define CYTHON_ASSUME_SAFE_SIZE 1
  #endif
  #ifndef CYTHON_UNPACK_METHODS
    #define CYTHON_UNPACK_METHODS 1
  #endif
  #ifndef CYTHON_FAST_THREAD_STATE
    #define CYTHON_FAST_THREAD_STATE 1
  #endif
  #if CYTHON_COMPILING_IN_CPYTHON_FREETHREADING
    #undef CYTHON_FAST_GIL
    #define CYTHON_FAST_GIL 0
  #elif !defined(CYTHON_FAST_GIL)
    #define CYTHON_FAST_GIL (PY_VERSION_HEX < 0x030C00A6)
  #endif
  #ifndef CYTHON_METH_FASTCALL
    #define CYTHON_METH_FASTCALL 1
  #endif
  #ifndef CYTHON_FAST_PYCALL
    #define CYTHON_FAST_PYCALL 1
  #endif
  #ifndef CYTHON_PEP487_INIT_SUBCLASS
    #define CYTHON_PEP487_INIT_SUBCLASS 1
  #endif
  #ifndef CYTHON_PEP489_MULTI_PHASE_INIT
    #define CYTHON_PEP489_MULTI_PHASE_INIT 1
  #endif
  #ifndef CYTHON_USE_MODULE_STATE
    #define CYTHON_USE_MODULE_STATE 0
  #endif
  #ifndef CYTHON_USE_SYS_MONITORING
    #define CYTHON_USE_SYS_MONITORING (PY_VERSION_HEX >= 0x030d00B1)
  #endif
  #ifndef CYTHON_USE_TP_FINALIZE
    #define CYTHON_USE_TP_FINALIZE 1
  #endif
  #ifndef CYTHON_USE_AM_SEND
    #define CYTHON_USE_AM_SEND 1
  #endif
  #if CYTHON_COMPILING_IN_CPYTHON_FREETHREADING
    #undef CYTHON_USE_DICT_VERSIONS
    #define CYTHON_USE_DICT_VERSIONS 0
  #elif !defined(CYTHON_USE_DICT_VERSIONS)
    #define CYTHON_USE_DICT_VERSIONS  (PY_VERSION_HEX < 0x030C00A5 && !CYTHON_USE_MODULE_STATE)
  #endif
  #ifndef CYTHON_USE_EXC_INFO_STACK
    #define CYTHON_USE_EXC_INFO_STACK 1
  #endif
  #ifndef CYTHON_UPDATE_DESCRIPTOR_DOC
    #define CYTHON_UPDATE_DESCRIPTOR_DOC 1
  #endif
  #ifndef CYTHON_USE_FREELISTS
    #define CYTHON_USE_FREELISTS (!CYTHON_COMPILING_IN_CPYTHON_FREETHREADING)
  #endif
  #if defined(CYTHON_IMMORTAL_CONSTANTS) && PY_VERSION_HEX < 0x030C0000
    #undef CYTHON_IMMORTAL_CONSTANTS
    #define CYTHON_IMMORTAL_CONSTANTS 0  // definitely won't work
  #elif !defined(CYTHON_IMMORTAL_CONSTANTS)
    #define CYTHON_IMMORTAL_CONSTANTS (PY_VERSION_HEX >= 0x030C0000 && !CYTHON_USE_MODULE_STATE && CYTHON_COMPILING_IN_CPYTHON_FREETHREADING)
  #endif
#endif
#ifndef CYTHON_COMPRESS_STRINGS
  #define CYTHON_COMPRESS_STRINGS 1
#endif
#ifndef CYTHON_FAST_PYCCALL
#define CYTHON_FAST_PYCCALL  CYTHON_FAST_PYCALL
#endif
#ifndef CYTHON_VECTORCALL
#if CYTHON_COMPILING_IN_LIMITED_API
#define CYTHON_VECTORCALL  (__PYX_LIMITED_VERSION_HEX >= 0x030C0000)
#else
#define CYTHON_VECTORCALL  (CYTHON_FAST_PYCCALL)
#endif
#endif
#if CYTHON_USE_PYLONG_INTERNALS
  #undef SHIFT
  #undef BASE
  #undef MASK
  #ifdef SIZEOF_VOID_P
    enum { __pyx_check_sizeof_voidp = 1 / (int)(SIZEOF_VOID_P == sizeof(void*)) };
  #endif
#endif
#ifndef __has_attribute
  #define __has_attribute(x) 0
#endif
#ifndef __has_cpp_attribute
  #define __has_cpp_attribute(x) 0
#endif
#ifndef CYTHON_RESTRICT
  #if defined(__GNUC__)
    #define CYTHON_RESTRICT __restrict__
  #elif defined(_MSC_VER) && _MSC_VER >= 1400
    #define CYTHON_RESTRICT __restrict
  #elif defined (__STDC_VERSION__) && __STDC_VERSION__ >= 199901L
    #define CYTHON_RESTRICT restrict
  #else
    #define CYTHON_RESTRICT
  #endif
#endif
#ifndef CYTHON_UNUSED
  #if defined(__cplusplus)
    /* for clang __has_cpp_attribute(maybe_unused) is true even before C++17
     * but leads to warnings with -pedantic, since it is a C++17 feature */
    #if ((defined(_MSVC_LANG) && _MSVC_LANG >= 201703L) || __cplusplus >= 201703L)
      #if __has_cpp_attribute(maybe_unused)
        #define CYTHON_UNUSED [[maybe_unused]]
      #endif
    #endif
  #endif
#endif
#ifndef CYTHON_UNUSED
# if defined(__GNUC__)
#   if !(defined(__cplusplus)) || (__GNUC__ > 3 || (__GNUC__ == 3 && __GNUC_MINOR__ >= 4))
#     define CYTHON_UNUSED __attribute__ ((__unused__))
#   else
#     define CYTHON_UNUSED
#   endif
# elif defined(__ICC) || (defined(__INTEL_COMPILER) && !defined(_MSC_VER))
#   define CYTHON_UNUSED __attribute__ ((__unused__))
# else
#   define CYTHON_UNUSED
# endif
#endif
#ifndef CYTHON_UNUSED_VAR
#  if defined(__cplusplus)
     template<class T> void CYTHON_UNUSED_VAR( const T& ) { }
#  else
#    define CYTHON_UNUSED_VAR(x) (void)(x)
#  endif
#endif
#ifndef CYTHON_MAYBE_UNUSED_VAR
  #define CYTHON_MAYBE_UNUSED_VAR(x) CYTHON_UNUSED_VAR(x)
#endif
#ifndef CYTHON_NCP_UNUSED
# if CYTHON_COMPILING_IN_CPYTHON && !CYTHON_COMPILING_IN_CPYTHON_FREETHREADING
#  define CYTHON_NCP_UNUSED
# else
#  define CYTHON_NCP_UNUSED CYTHON_UNUSED
# endif
#endif
#ifndef CYTHON_USE_CPP_STD_MOVE
  #if defined(__cplusplus) && (\
    __cplusplus >= 201103L || (defined(_MSC_VER) && _MSC_VER >= 1600))
    #define CYTHON_USE_CPP_STD_MOVE 1
  #else
    #define CYTHON_USE_CPP_STD_MOVE 0
  #endif
#endif
#define __Pyx_void_to_None(void_result) ((void)(void_result), Py_INCREF(Py_None), Py_None)
#include <stdint.h>
typedef uintptr_t  __pyx_uintptr_t;
#ifndef CYTHON_FALLTHROUGH
  #if defined(__cplusplus)
    /* for clang __has_cpp_attribute(fallthrough) is true even before C++17
     * but leads to warnings with -pedantic, since it is a C++17 feature */
    #if ((defined(_MSVC_LANG) && _MSVC_LANG >= 201703L) || __cplusplus >= 201703L)
      #if __has_cpp_attribute(fallthrough)
        #define CYTHON_FALLTHROUGH [[fallthrough]]
      #endif
    #endif
    #ifndef CYTHON_FALLTHROUGH
      #if __has_cpp_attribute(clang::fallthrough)
        #define CYTHON_FALLTHROUGH [[clang::fallthrough]]
      #elif __has_cpp_attribute(gnu::fallthrough)
        #define CYTHON_FALLTHROUGH [[gnu::fallthrough]]
      #endif
    #endif
  #endif
  #ifndef CYTHON_FALLTHROUGH
    #if __has_attribute(fallthrough)
      #define CYTHON_FALLTHROUGH __attribute__((fallthrough))
    #else
      #define CYTHON_FALLTHROUGH
    #endif
  #endif
  #if defined(__clang__) && defined(__apple_build_version__)
    #if __apple_build_version__ < 7000000
      #undef  CYTHON_FALLTHROUGH
      #define CYTHON_FALLTHROUGH
    #endif
  #endif
#endif
#ifndef Py_UNREACHABLE
  #define Py_UNREACHABLE()  assert(0); abort()
#endif
#ifdef __cplusplus
  template <typename T>
  struct __PYX_IS_UNSIGNED_IMPL {static const bool value = T(0) < T(-1);};
  #define __PYX_IS_UNSIGNED(type) (__PYX_IS_UNSIGNED_IMPL<type>::value)
#else
  #define __PYX_IS_UNSIGNED(type) (((type)-1) > 0)
#endif
#if CYTHON_COMPILING_IN_PYPY == 1
  #define __PYX_NEED_TP_PRINT_SLOT  (PY_VERSION_HEX < 0x030A0000)
#else
  #define __PYX_NEED_TP_PRINT_SLOT  (PY_VERSION_HEX < 0x03090000)
#endif
#define __PYX_REINTERPRET_FUNCION(func_pointer, other_pointer) ((func_pointer)(void(*)(void))(other_pointer))

/* CppInitCode */
#ifndef __cplusplus
  #error "Cython files generated with the C++ option must be compiled with a C++ compiler."
#endif
#ifndef CYTHON_INLINE
  #if defined(__clang__)
    #define CYTHON_INLINE __inline__ __attribute__ ((__unused__))
  #else
    #define CYTHON_INLINE inline
  #endif
#endif
template<typename T>
void __Pyx_call_destructor(T& x) {
    x.~T();
}
template<typename T>
class __Pyx_FakeReference {
  public:
    __Pyx_FakeReference() : ptr(NULL) { }
    __Pyx_FakeReference(const T& ref) : ptr(const_cast<T*>(&ref)) { }
    T *operator->() { return ptr; }
    T *operator&() { return ptr; }
    operator T&() { return *ptr; }
    template<typename U> bool operator ==(const U& other) const { return *ptr == other; }
    template<typename U> bool operator !=(const U& other) const { return *ptr != other; }
    template<typename U> bool operator==(const __Pyx_FakeReference<U>& other) const { return *ptr == *other.ptr; }
    template<typename U> bool operator!=(const __Pyx_FakeReference<U>& other) const { return *ptr != *other.ptr; }
  private:
    T *ptr;
};

/* PythonCompatibility */
#define __PYX_BUILD_PY_SSIZE_T "n"
#define CYTHON_FORMAT_SSIZE_T "z"
#define __Pyx_BUILTIN_MODULE_NAME "builtins"
#define __Pyx_DefaultClassType PyType_Type
#if CYTHON_COMPILING_IN_LIMITED_API
    #ifndef CO_OPTIMIZED
    static int CO_OPTIMIZED;
    #endif
    #ifndef CO_NEWLOCALS
    static int CO_NEWLOCALS;
    #endif
    #ifndef CO_VARARGS
    static int CO_VARARGS;
    #endif
    #ifndef CO_VARKEYWORDS
    static int CO_VARKEYWORDS;
    #endif
    #ifndef CO_ASYNC_GENERATOR
    static int CO_ASYNC_GENERATOR;
    #endif
    #ifndef CO_GENERATOR
    static int CO_GENERATOR;
    #endif
    #ifndef CO_COROUTINE
    static int CO_COROUTINE;
    #endif
#else
    #ifndef CO_COROUTINE
      #define CO_COROUTINE 0x80
    #endif
    #ifndef CO_ASYNC_GENERATOR
      #define CO_ASYNC_GENERATOR 0x200
    #endif
#endif
static int __Pyx_init_co_variables(void);
#if PY_VERSION_HEX >= 0x030900A4 || defined(Py_IS_TYPE)
  #define __Pyx_IS_TYPE(ob, type) Py_IS_TYPE(ob, type)
#else
  #define __Pyx_IS_TYPE(ob, type) (((const PyObject*)ob)->ob_type == (type))
#endif
#if PY_VERSION_HEX >= 0x030A00B1 || defined(Py_Is)
  #define __Pyx_Py_Is(x, y)  Py_Is(x, y)
#else
  #define __Pyx_Py_Is(x, y) ((x) == (y))
#endif
#if PY_VERSION_HEX >= 0x030A00B1 || defined(Py_IsNone)
  #define __Pyx_Py_IsNone(ob) Py_IsNone(ob)
#else
  #define __Pyx_Py_IsNone(ob) __Pyx_Py_Is((ob), Py_None)
#endif
#if PY_VERSION_HEX >= 0x030A00B1 || defined(Py_IsTrue)
  #define __Pyx_Py_IsTrue(ob) Py_IsTrue(ob)
#else
  #define __Pyx_Py_IsTrue(ob) __Pyx_Py_Is((ob), Py_True)
#endif
#if PY_VERSION_HEX >= 0x030A00B1 || defined(Py_IsFalse)
  #define __Pyx_Py_IsFalse(ob) Py_IsFalse(ob)
#else
  #define __Pyx_Py_IsFalse(ob) __Pyx_Py_Is((ob), Py_False)
#endif
#define __Pyx_NoneAsNull(obj)  (__Pyx_Py_IsNone(obj) ? NULL : (obj))
#if PY_VERSION_HEX >= 0x030900F0 && !CYTHON_COMPILING_IN_PYPY
  #define __Pyx_PyObject_GC_IsFinalized(o) PyObject_GC_IsFinalized(o)
#else
  #define __Pyx_PyObject_GC_IsFinalized(o) _PyGC_FINALIZED(o)
#endif
#ifndef Py_TPFLAGS_CHECKTYPES
  #define Py_TPFLAGS_CHECKTYPES 0
#endif
#ifndef Py_TPFLAGS_HAVE_INDEX
  #define Py_TPFLAGS_HAVE_INDEX 0
#endif
#ifndef Py_TPFLAGS_HAVE_NEWBUFFER
  #define Py_TPFLAGS_HAVE_NEWBUFFER 0
#endif
#ifndef Py_TPFLAGS_HAVE_FINALIZE
  #define Py_TPFLAGS_HAVE_FINALIZE 0
#endif
#ifndef Py_TPFLAGS_SEQUENCE
  #define Py_TPFLAGS_SEQUENCE 0
#endif
#ifndef Py_TPFLAGS_MAPPING
  #define Py_TPFLAGS_MAPPING 0
#endif
#ifndef Py_TPFLAGS_IMMUTABLETYPE
  #define Py_TPFLAGS_IMMUTABLETYPE (1UL << 8)
#endif
#ifndef Py_TPFLAGS_DISALLOW_INSTANTIATION
  #define Py_TPFLAGS_DISALLOW_INSTANTIATION (1UL << 7)
#endif
#ifndef METH_STACKLESS
  #define METH_STACKLESS 0
#endif
#ifndef METH_FASTCALL
  #ifndef METH_FASTCALL
     #define METH_FASTCALL 0x80
  #endif
  typedef PyObject *(*__Pyx_PyCFunctionFast) (PyObject *self, PyObject *const *args, Py_ssize_t nargs);
  typedef PyObject *(*__Pyx_PyCFunctionFastWithKeywords) (PyObject *self, PyObject *const *args,
                                                          Py_ssize_t nargs, PyObject *kwnames);
#else
  #if PY_VERSION_HEX >= 0x030d00A4
  #  define __Pyx_PyCFunctionFast PyCFunctionFast
  #  define __Pyx_PyCFunctionFastWithKeywords PyCFunctionFastWithKeywords
  #else
  #  define __Pyx_PyCFunctionFast _PyCFunctionFast
  #  define __Pyx_PyCFunctionFastWithKeywords _PyCFunctionFastWithKeywords
  #endif
#endif
#if CYTHON_METH_FASTCALL
  #define __Pyx_METH_FASTCALL METH_FASTCALL
  #define __Pyx_PyCFunction_FastCall __Pyx_PyCFunctionFast
  #define __Pyx_PyCFunction_FastCallWithKeywords __Pyx_PyCFunctionFastWithKeywords
#else
  #define __Pyx_METH_FASTCALL METH_VARARGS
  #define __Pyx_PyCFunction_FastCall PyCFunction
  #define __Pyx_PyCFunction_FastCallWithKeywords PyCFunctionWithKeywords
#endif
#if CYTHON_VECTORCALL
  #define __pyx_vectorcallfunc vectorcallfunc
  #define __Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET  PY_VECTORCALL_ARGUMENTS_OFFSET
  #define __Pyx_PyVectorcall_NARGS(n)  PyVectorcall_NARGS((size_t)(n))
#else
  #define __Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET  0
  #define __Pyx_PyVectorcall_NARGS(n)  ((Py_ssize_t)(n))
#endif
#if PY_VERSION_HEX >= 0x030900B1
#define __Pyx_PyCFunction_CheckExact(func)  PyCFunction_CheckExact(func)
#else
#define __Pyx_PyCFunction_CheckExact(func)  PyCFunction_Check(func)
#endif
#define __Pyx_CyOrPyCFunction_Check(func)  PyCFunction_Check(func)
#if CYTHON_COMPILING_IN_CPYTHON
#define __Pyx_CyOrPyCFunction_GET_FUNCTION(func)  (((PyCFunctionObject*)(func))->m_ml->ml_meth)
#elif !CYTHON_COMPILING_IN_LIMITED_API
#define __Pyx_CyOrPyCFunction_GET_FUNCTION(func)  PyCFunction_GET_FUNCTION(func)
#endif
#if CYTHON_COMPILING_IN_CPYTHON
#define __Pyx_CyOrPyCFunction_GET_FLAGS(func)  (((PyCFunctionObject*)(func))->m_ml->ml_flags)
static CYTHON_INLINE PyObject* __Pyx_CyOrPyCFunction_GET_SELF(PyObject *func) {
    return (__Pyx_CyOrPyCFunction_GET_FLAGS(func) & METH_STATIC) ? NULL : ((PyCFunctionObject*)func)->m_self;
}
#endif
static CYTHON_INLINE int __Pyx__IsSameCFunction(PyObject *func, void (*cfunc)(void)) {
#if CYTHON_COMPILING_IN_LIMITED_API
    return PyCFunction_Check(func) && PyCFunction_GetFunction(func) == (PyCFunction) cfunc;
#else
    return PyCFunction_Check(func) && PyCFunction_GET_FUNCTION(func) == (PyCFunction) cfunc;
#endif
}
#define __Pyx_IsSameCFunction(func, cfunc)   __Pyx__IsSameCFunction(func, cfunc)
#if PY_VERSION_HEX < 0x03090000 || (CYTHON_COMPILING_IN_LIMITED_API && __PYX_LIMITED_VERSION_HEX < 0x030A0000)
  #define __Pyx_PyType_FromModuleAndSpec(m, s, b)  ((void)m, PyType_FromSpecWithBases(s, b))
  typedef PyObject *(*__Pyx_PyCMethod)(PyObject *, PyTypeObject *, PyObject *const *, size_t, PyObject *);
#else
  #define __Pyx_PyType_FromModuleAndSpec(m, s, b)  PyType_FromModuleAndSpec(m, s, b)
  #define __Pyx_PyCMethod  PyCMethod
#endif
#ifndef METH_METHOD
  #define METH_METHOD 0x200
#endif
#if CYTHON_COMPILING_IN_PYPY && !defined(PyObject_Malloc)
  #define PyObject_Malloc(s)   PyMem_Malloc(s)
  #define PyObject_Free(p)     PyMem_Free(p)
  #define PyObject_Realloc(p)  PyMem_Realloc(p)
#endif
#if CYTHON_COMPILING_IN_LIMITED_API
  #define __Pyx_PyFrame_SetLineNumber(frame, lineno)
#elif CYTHON_COMPILING_IN_GRAAL && defined(GRAALPY_VERSION_NUM) && GRAALPY_VERSION_NUM > 0x19000000
  #define __Pyx_PyCode_HasFreeVars(co)  (PyCode_GetNumFree(co) > 0)
  #define __Pyx_PyFrame_SetLineNumber(frame, lineno) GraalPyFrame_SetLineNumber((frame), (lineno))
#elif CYTHON_COMPILING_IN_GRAAL
  #define __Pyx_PyCode_HasFreeVars(co)  (PyCode_GetNumFree(co) > 0)
  #define __Pyx_PyFrame_SetLineNumber(frame, lineno) _PyFrame_SetLineNumber((frame), (lineno))
#else
  #define __Pyx_PyCode_HasFreeVars(co)  (PyCode_GetNumFree(co) > 0)
  #define __Pyx_PyFrame_SetLineNumber(frame, lineno)  (frame)->f_lineno = (lineno)
#endif
#if CYTHON_COMPILING_IN_LIMITED_API
  #define __Pyx_PyThreadState_Current PyThreadState_Get()
#elif !CYTHON_FAST_THREAD_STATE
  #define __Pyx_PyThreadState_Current PyThreadState_GET()
#elif PY_VERSION_HEX >= 0x030d00A1
  #define __Pyx_PyThreadState_Current PyThreadState_GetUnchecked()
#else
  #define __Pyx_PyThreadState_Current _PyThreadState_UncheckedGet()
#endif
#if CYTHON_USE_MODULE_STATE
static CYTHON_INLINE void *__Pyx__PyModule_GetState(PyObject *op)
{
    void *result;
    result = PyModule_GetState(op);
    if (!result)
        Py_FatalError("Couldn't find the module state");
    return result;
}
#define __Pyx_PyModule_GetState(o) (__pyx_mstatetype *)__Pyx__PyModule_GetState(o)
#else
#define __Pyx_PyModule_GetState(op) ((void)op,__pyx_mstate_global)
#endif
#define __Pyx_PyObject_GetSlot(obj, name, func_ctype)  __Pyx_PyType_GetSlot(Py_TYPE((PyObject *) obj), name, func_ctype)
#define __Pyx_PyObject_TryGetSlot(obj, name, func_ctype) __Pyx_PyType_TryGetSlot(Py_TYPE(obj), name, func_ctype)
#define __Pyx_PyObject_GetSubSlot(obj, sub, name, func_ctype) __Pyx_PyType_GetSubSlot(Py_TYPE(obj), sub, name, func_ctype)
#define __Pyx_PyObject_TryGetSubSlot(obj, sub, name, func_ctype) __Pyx_PyType_TryGetSubSlot(Py_TYPE(obj), sub, name, func_ctype)
#if CYTHON_USE_TYPE_SLOTS
  #define __Pyx_PyType_GetSlot(type, name, func_ctype)  ((type)->name)
  #define __Pyx_PyType_TryGetSlot(type, name, func_ctype) __Pyx_PyType_GetSlot(type, name, func_ctype)
  #define __Pyx_PyType_GetSubSlot(type, sub, name, func_ctype) (((type)->sub) ? ((type)->sub->name) : NULL)
  #define __Pyx_PyType_TryGetSubSlot(type, sub, name, func_ctype) __Pyx_PyType_GetSubSlot(type, sub, name, func_ctype)
#else
  #define __Pyx_PyType_GetSlot(type, name, func_ctype)  ((func_ctype) PyType_GetSlot((type), Py_##name))
  #define __Pyx_PyType_TryGetSlot(type, name, func_ctype)\
    ((__PYX_LIMITED_VERSION_HEX >= 0x030A0000 ||\
     (PyType_GetFlags(type) & Py_TPFLAGS_HEAPTYPE) || __Pyx_get_runtime_version() >= 0x030A0000) ?\
     __Pyx_PyType_GetSlot(type, name, func_ctype) : NULL)
  #define __Pyx_PyType_GetSubSlot(obj, sub, name, func_ctype) __Pyx_PyType_GetSlot(obj, name, func_ctype)
  #define __Pyx_PyType_TryGetSubSlot(obj, sub, name, func_ctype) __Pyx_PyType_TryGetSlot(obj, name, func_ctype)
#endif
#if CYTHON_COMPILING_IN_CPYTHON || defined(_PyDict_NewPresized)
#define __Pyx_PyDict_NewPresized(n)  ((n <= 8) ? PyDict_New() : _PyDict_NewPresized(n))
#else
#define __Pyx_PyDict_NewPresized(n)  PyDict_New()
#endif
#define __Pyx_PyNumber_Divide(x,y)         PyNumber_TrueDivide(x,y)
#define __Pyx_PyNumber_InPlaceDivide(x,y)  PyNumber_InPlaceTrueDivide(x,y)
#if CYTHON_COMPILING_IN_CPYTHON && CYTHON_USE_UNICODE_INTERNALS
#define __Pyx_PyDict_GetItemStrWithError(dict, name)  _PyDict_GetItem_KnownHash(dict, name, ((PyASCIIObject *) name)->hash)
static CYTHON_INLINE PyObject * __Pyx_PyDict_GetItemStr(PyObject *dict, PyObject *name) {
    PyObject *res = __Pyx_PyDict_GetItemStrWithError(dict, name);
    if (res == NULL) PyErr_Clear();
    return res;
}
#elif !CYTHON_COMPILING_IN_PYPY || PYPY_VERSION_NUM >= 0x07020000
#define __Pyx_PyDict_GetItemStrWithError  PyDict_GetItemWithError
#define __Pyx_PyDict_GetItemStr           PyDict_GetItem
#else
static CYTHON_INLINE PyObject * __Pyx_PyDict_GetItemStrWithError(PyObject *dict, PyObject *name) {
#if CYTHON_COMPILING_IN_PYPY
    return PyDict_GetItem(dict, name);
#else
    PyDictEntry *ep;
    PyDictObject *mp = (PyDictObject*) dict;
    long hash = ((PyStringObject *) name)->ob_shash;
    assert(hash != -1);
    ep = (mp->ma_lookup)(mp, name, hash);
    if (ep == NULL) {
        return NULL;
    }
    return ep->me_value;
#endif
}
#define __Pyx_PyDict_GetItemStr           PyDict_GetItem
#endif
#if CYTHON_USE_TYPE_SLOTS
  #define __Pyx_PyType_GetFlags(tp)   (((PyTypeObject *)tp)->tp_flags)
  #define __Pyx_PyType_HasFeature(type, feature)  ((__Pyx_PyType_GetFlags(type) & (feature)) != 0)
#else
  #define __Pyx_PyType_GetFlags(tp)   (PyType_GetFlags((PyTypeObject *)tp))
  #define __Pyx_PyType_HasFeature(type, feature)  PyType_HasFeature(type, feature)
#endif
#define __Pyx_PyObject_GetIterNextFunc(iterator)  __Pyx_PyObject_GetSlot(iterator, tp_iternext, iternextfunc)
#if CYTHON_USE_TYPE_SPECS
#define __Pyx_PyHeapTypeObject_GC_Del(obj)  {\
    PyTypeObject *type = Py_TYPE((PyObject*)obj);\
    assert(__Pyx_PyType_HasFeature(type, Py_TPFLAGS_HEAPTYPE));\
    PyObject_GC_Del(obj);\
    Py_DECREF(type);\
}
#else
#define __Pyx_PyHeapTypeObject_GC_Del(obj)  PyObject_GC_Del(obj)
#endif
#if CYTHON_COMPILING_IN_LIMITED_API
  #define __Pyx_PyUnicode_READY(op)       (0)
  #define __Pyx_PyUnicode_READ_CHAR(u, i) PyUnicode_ReadChar(u, i)
  #define __Pyx_PyUnicode_MAX_CHAR_VALUE(u)   ((void)u, 1114111U)
  #define __Pyx_PyUnicode_KIND(u)         ((void)u, (0))
  #define __Pyx_PyUnicode_DATA(u)         ((void*)u)
  #define __Pyx_PyUnicode_READ(k, d, i)   ((void)k, PyUnicode_ReadChar((PyObject*)(d), i))
  #define __Pyx_PyUnicode_IS_TRUE(u)      (0 != PyUnicode_GetLength(u))
#else
  #if PY_VERSION_HEX >= 0x030C0000
    #define __Pyx_PyUnicode_READY(op)       (0)
  #else
    #define __Pyx_PyUnicode_READY(op)       (likely(PyUnicode_IS_READY(op)) ?\
                                                0 : _PyUnicode_Ready((PyObject *)(op)))
  #endif
  #define __Pyx_PyUnicode_READ_CHAR(u, i) PyUnicode_READ_CHAR(u, i)
  #define __Pyx_PyUnicode_MAX_CHAR_VALUE(u)   PyUnicode_MAX_CHAR_VALUE(u)
  #define __Pyx_PyUnicode_KIND(u)         ((int)PyUnicode_KIND(u))
  #define __Pyx_PyUnicode_DATA(u)         PyUnicode_DATA(u)
  #define __Pyx_PyUnicode_READ(k, d, i)   PyUnicode_READ(k, d, i)
  #define __Pyx_PyUnicode_WRITE(k, d, i, ch)  PyUnicode_WRITE(k, d, i, (Py_UCS4) ch)
  #if PY_VERSION_HEX >= 0x030C0000
    #define __Pyx_PyUnicode_IS_TRUE(u)      (0 != PyUnicode_GET_LENGTH(u))
  #else
    #if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x03090000
    #define __Pyx_PyUnicode_IS_TRUE(u)      (0 != (likely(PyUnicode_IS_READY(u)) ? PyUnicode_GET_LENGTH(u) : ((PyCompactUnicodeObject *)(u))->wstr_length))
    #else
    #define __Pyx_PyUnicode_IS_TRUE(u)      (0 != (likely(PyUnicode_IS_READY(u)) ? PyUnicode_GET_LENGTH(u) : PyUnicode_GET_SIZE(u)))
    #endif
  #endif
#endif
#if CYTHON_COMPILING_IN_PYPY
  #define __Pyx_PyUnicode_Concat(a, b)      PyNumber_Add(a, b)
  #define __Pyx_PyUnicode_ConcatSafe(a, b)  PyNumber_Add(a, b)
#else
  #define __Pyx_PyUnicode_Concat(a, b)      PyUnicode_Concat(a, b)
  #define __Pyx_PyUnicode_ConcatSafe(a, b)  ((unlikely((a) == Py_None) || unlikely((b) == Py_None)) ?\
      PyNumber_Add(a, b) : __Pyx_PyUnicode_Concat(a, b))
#endif
#if CYTHON_COMPILING_IN_PYPY
  #if !defined(PyUnicode_DecodeUnicodeEscape)
    #define PyUnicode_DecodeUnicodeEscape(s, size, errors)  PyUnicode_Decode(s, size, "unicode_escape", errors)
  #endif
  #if !defined(PyUnicode_Contains)
    #define PyUnicode_Contains(u, s)  PySequence_Contains(u, s)
  #endif
  #if !defined(PyByteArray_Check)
    #define PyByteArray_Check(obj)  PyObject_TypeCheck(obj, &PyByteArray_Type)
  #endif
  #if !defined(PyObject_Format)
    #define PyObject_Format(obj, fmt)  PyObject_CallMethod(obj, "__format__", "O", fmt)
  #endif
#endif
#define __Pyx_PyUnicode_FormatSafe(a, b)  ((unlikely((a) == Py_None || (PyUnicode_Check(b) && !PyUnicode_CheckExact(b)))) ? PyNumber_Remainder(a, b) : PyUnicode_Format(a, b))
#if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x030E0000
  #define __Pyx_PySequence_ListKeepNew(obj)\
    (likely(PyList_CheckExact(obj) && PyUnstable_Object_IsUniquelyReferenced(obj)) ? __Pyx_NewRef(obj) : PySequence_List(obj))
#elif CYTHON_COMPILING_IN_CPYTHON
  #define __Pyx_PySequence_ListKeepNew(obj)\
    (likely(PyList_CheckExact(obj) && Py_REFCNT(obj) == 1) ? __Pyx_NewRef(obj) : PySequence_List(obj))
#else
  #define __Pyx_PySequence_ListKeepNew(obj)  PySequence_List(obj)
#endif
#ifndef PySet_CheckExact
  #define PySet_CheckExact(obj)        __Pyx_IS_TYPE(obj, &PySet_Type)
#endif
#if PY_VERSION_HEX >= 0x030900A4
  #define __Pyx_SET_REFCNT(obj, refcnt) Py_SET_REFCNT(obj, refcnt)
  #define __Pyx_SET_SIZE(obj, size) Py_SET_SIZE(obj, size)
#else
  #define __Pyx_SET_REFCNT(obj, refcnt) Py_REFCNT(obj) = (refcnt)
  #define __Pyx_SET_SIZE(obj, size) Py_SIZE(obj) = (size)
#endif
enum __Pyx_ReferenceSharing {
  __Pyx_ReferenceSharing_DefinitelyUnique, // We created it so we know it's unshared - no need to check
  __Pyx_ReferenceSharing_OwnStrongReference,
  __Pyx_ReferenceSharing_FunctionArgument,
  __Pyx_ReferenceSharing_SharedReference, // Never trust it to be unshared because it's a global or similar
};
#if CYTHON_COMPILING_IN_CPYTHON_FREETHREADING && PY_VERSION_HEX >= 0x030E0000
#define __Pyx_IS_UNIQUELY_REFERENCED(o, sharing)\
    (sharing == __Pyx_ReferenceSharing_DefinitelyUnique ? 1 :\
      (sharing == __Pyx_ReferenceSharing_FunctionArgument ? PyUnstable_Object_IsUniqueReferencedTemporary(o) :\
      (sharing == __Pyx_ReferenceSharing_OwnStrongReference ? PyUnstable_Object_IsUniquelyReferenced(o) : 0)))
#elif (CYTHON_COMPILING_IN_CPYTHON && !CYTHON_COMPILING_IN_CPYTHON_FREETHREADING) || CYTHON_COMPILING_IN_LIMITED_API
#define __Pyx_IS_UNIQUELY_REFERENCED(o, sharing) (((void)sharing), Py_REFCNT(o) == 1)
#else
#define __Pyx_IS_UNIQUELY_REFERENCED(o, sharing) (((void)o), ((void)sharing), 0)
#endif
#if CYTHON_AVOID_BORROWED_REFS || CYTHON_AVOID_THREAD_UNSAFE_BORROWED_REFS
  #if __PYX_LIMITED_VERSION_HEX >= 0x030d0000
    #define __Pyx_PyList_GetItemRef(o, i) PyList_GetItemRef(o, i)
  #elif CYTHON_COMPILING_IN_LIMITED_API || !CYTHON_ASSUME_SAFE_MACROS
    #define __Pyx_PyList_GetItemRef(o, i) (likely((i) >= 0) ? PySequence_GetItem(o, i) : (PyErr_SetString(PyExc_IndexError, "list index out of range"), (PyObject*)NULL))
  #else
    #define __Pyx_PyList_GetItemRef(o, i) PySequence_ITEM(o, i)
  #endif
#elif CYTHON_COMPILING_IN_LIMITED_API || !CYTHON_ASSUME_SAFE_MACROS
  #if __PYX_LIMITED_VERSION_HEX >= 0x030d0000
    #define __Pyx_PyList_GetItemRef(o, i) PyList_GetItemRef(o, i)
  #else
    #define __Pyx_PyList_GetItemRef(o, i) __Pyx_XNewRef(PyList_GetItem(o, i))
  #endif
#else
  #define __Pyx_PyList_GetItemRef(o, i) __Pyx_NewRef(PyList_GET_ITEM(o, i))
#endif
#if CYTHON_AVOID_THREAD_UNSAFE_BORROWED_REFS && !CYTHON_COMPILING_IN_LIMITED_API && CYTHON_ASSUME_SAFE_MACROS
  #define __Pyx_PyList_GetItemRefFast(o, i, unsafe_shared) (__Pyx_IS_UNIQUELY_REFERENCED(o, unsafe_shared) ?\
    __Pyx_NewRef(PyList_GET_ITEM(o, i)) : __Pyx_PyList_GetItemRef(o, i))
#else
  #define __Pyx_PyList_GetItemRefFast(o, i, unsafe_shared) __Pyx_PyList_GetItemRef(o, i)
#endif
#if __PYX_LIMITED_VERSION_HEX >= 0x030d0000
#define __Pyx_PyDict_GetItemRef(dict, key, result) PyDict_GetItemRef(dict, key, result)
#elif CYTHON_AVOID_BORROWED_REFS || CYTHON_AVOID_THREAD_UNSAFE_BORROWED_REFS
static CYTHON_INLINE int __Pyx_PyDict_GetItemRef(PyObject *dict, PyObject *key, PyObject **result) {
  *result = PyObject_GetItem(dict, key);
  if (*result == NULL) {
    if (PyErr_ExceptionMatches(PyExc_KeyError)) {
      PyErr_Clear();
      return 0;
    }
    return -1;
  }
  return 1;
}
#else
static CYTHON_INLINE int __Pyx_PyDict_GetItemRef(PyObject *dict, PyObject *key, PyObject **result) {
  *result = PyDict_GetItemWithError(dict, key);
  if (*result == NULL) {
    return PyErr_Occurred() ? -1 : 0;
  }
  Py_INCREF(*result);
  return 1;
}
#endif
#if defined(CYTHON_DEBUG_VISIT_CONST) && CYTHON_DEBUG_VISIT_CONST
  #define __Pyx_VISIT_CONST(obj)  Py_VISIT(obj)
#else
  #define __Pyx_VISIT_CONST(obj)
#endif
#if CYTHON_ASSUME_SAFE_MACROS
  #define __Pyx_PySequence_ITEM(o, i) PySequence_ITEM(o, i)
  #define __Pyx_PySequence_SIZE(seq)  Py_SIZE(seq)
  #define __Pyx_PyTuple_SET_ITEM(o, i, v) (PyTuple_SET_ITEM(o, i, v), (0))
  #define __Pyx_PyTuple_GET_ITEM(o, i) PyTuple_GET_ITEM(o, i)
  #define __Pyx_PyList_SET_ITEM(o, i, v) (PyList_SET_ITEM(o, i, v), (0))
  #define __Pyx_PyList_GET_ITEM(o, i) PyList_GET_ITEM(o, i)
#else
  #define __Pyx_PySequence_ITEM(o, i) PySequence_GetItem(o, i)
  #define __Pyx_PySequence_SIZE(seq)  PySequence_Size(seq)
  #define __Pyx_PyTuple_SET_ITEM(o, i, v) PyTuple_SetItem(o, i, v)
  #define __Pyx_PyTuple_GET_ITEM(o, i) PyTuple_GetItem(o, i)
  #define __Pyx_PyList_SET_ITEM(o, i, v) PyList_SetItem(o, i, v)
  #define __Pyx_PyList_GET_ITEM(o, i) PyList_GetItem(o, i)
#endif
#if CYTHON_ASSUME_SAFE_SIZE
  #define __Pyx_PyTuple_GET_SIZE(o) PyTuple_GET_SIZE(o)
  #define __Pyx_PyList_GET_SIZE(o) PyList_GET_SIZE(o)
  #define __Pyx_PySet_GET_SIZE(o) PySet_GET_SIZE(o)
  #define __Pyx_PyBytes_GET_SIZE(o) PyBytes_GET_SIZE(o)
  #define __Pyx_PyByteArray_GET_SIZE(o) PyByteArray_GET_SIZE(o)
  #define __Pyx_PyUnicode_GET_LENGTH(o) PyUnicode_GET_LENGTH(o)
#else
  #define __Pyx_PyTuple_GET_SIZE(o) PyTuple_Size(o)
  #define __Pyx_PyList_GET_SIZE(o) PyList_Size(o)
  #define __Pyx_PySet_GET_SIZE(o) PySet_Size(o)
  #define __Pyx_PyBytes_GET_SIZE(o) PyBytes_Size(o)
  #define __Pyx_PyByteArray_GET_SIZE(o) PyByteArray_Size(o)
  #define __Pyx_PyUnicode_GET_LENGTH(o) PyUnicode_GetLength(o)
#endif
#if CYTHON_COMPILING_IN_PYPY && !defined(PyUnicode_InternFromString)
  #define PyUnicode_InternFromString(s) PyUnicode_FromString(s)
#endif
#define __Pyx_PyLong_FromHash_t PyLong_FromSsize_t
#define __Pyx_PyLong_AsHash_t   __Pyx_PyIndex_AsSsize_t
#if __PYX_LIMITED_VERSION_HEX >= 0x030A0000
    #define __Pyx_PySendResult PySendResult
#else
    typedef enum {
        PYGEN_RETURN = 0,
        PYGEN_ERROR = -1,
        PYGEN_NEXT = 1,
    } __Pyx_PySendResult;
#endif
#if CYTHON_COMPILING_IN_LIMITED_API || PY_VERSION_HEX < 0x030A00A3
  typedef __Pyx_PySendResult (*__Pyx_pyiter_sendfunc)(PyObject *iter, PyObject *value, PyObject **result);
#else
  #define __Pyx_pyiter_sendfunc sendfunc
#endif
#if !CYTHON_USE_AM_SEND
#define __PYX_HAS_PY_AM_SEND 0
#elif __PYX_LIMITED_VERSION_HEX >= 0x030A0000
#define __PYX_HAS_PY_AM_SEND 1
#else
#define __PYX_HAS_PY_AM_SEND 2  // our own backported implementation
#endif
#if __PYX_HAS_PY_AM_SEND < 2
    #define __Pyx_PyAsyncMethodsStruct PyAsyncMethods
#else
    typedef struct {
        unaryfunc am_await;
        unaryfunc am_aiter;
        unaryfunc am_anext;
        __Pyx_pyiter_sendfunc am_send;
    } __Pyx_PyAsyncMethodsStruct;
    #define __Pyx_SlotTpAsAsync(s) ((PyAsyncMethods*)(s))
#endif
#if CYTHON_USE_AM_SEND && PY_VERSION_HEX < 0x030A00F0
    #define __Pyx_TPFLAGS_HAVE_AM_SEND (1UL << 21)
#else
    #define __Pyx_TPFLAGS_HAVE_AM_SEND (0)
#endif
#if PY_VERSION_HEX >= 0x03090000
#define __Pyx_PyInterpreterState_Get() PyInterpreterState_Get()
#else
#define __Pyx_PyInterpreterState_Get() PyThreadState_Get()->interp
#endif
#if CYTHON_COMPILING_IN_LIMITED_API && PY_VERSION_HEX < 0x030A0000
#ifdef __cplusplus
extern "C"
#endif
PyAPI_FUNC(void *) PyMem_Calloc(size_t nelem, size_t elsize);
#endif
#if CYTHON_COMPILING_IN_LIMITED_API
static int __Pyx_init_co_variable(PyObject *inspect, const char* name, int *write_to) {
    int value;
    PyObject *py_value = PyObject_GetAttrString(inspect, name);
    if (!py_value) return 0;
    value = (int) PyLong_AsLong(py_value);
    Py_DECREF(py_value);
    *write_to = value;
    return value != -1 || !PyErr_Occurred();
}
static int __Pyx_init_co_variables(void) {
    PyObject *inspect;
    int result;
    inspect = PyImport_ImportModule("inspect");
    result =
#if !defined(CO_OPTIMIZED)
        __Pyx_init_co_variable(inspect, "CO_OPTIMIZED", &CO_OPTIMIZED) &&
#endif
#if !defined(CO_NEWLOCALS)
        __Pyx_init_co_variable(inspect, "CO_NEWLOCALS", &CO_NEWLOCALS) &&
#endif
#if !defined(CO_VARARGS)
        __Pyx_init_co_variable(inspect, "CO_VARARGS", &CO_VARARGS) &&
#endif
#if !defined(CO_VARKEYWORDS)
        __Pyx_init_co_variable(inspect, "CO_VARKEYWORDS", &CO_VARKEYWORDS) &&
#endif
#if !defined(CO_ASYNC_GENERATOR)
        __Pyx_init_co_variable(inspect, "CO_ASYNC_GENERATOR", &CO_ASYNC_GENERATOR) &&
#endif
#if !defined(CO_GENERATOR)
        __Pyx_init_co_variable(inspect, "CO_GENERATOR", &CO_GENERATOR) &&
#endif
#if !defined(CO_COROUTINE)
        __Pyx_init_co_variable(inspect, "CO_COROUTINE", &CO_COROUTINE) &&
#endif
        1;
    Py_DECREF(inspect);
    return result ? 0 : -1;
}
#else
static int __Pyx_init_co_variables(void) {
    return 0;  // It's a limited API-only feature
}
#endif

/* MathInitCode */
#if defined(_WIN32) || defined(WIN32) || defined(MS_WINDOWS)
  #ifndef _USE_MATH_DEFINES
    #define _USE_MATH_DEFINES
  #endif
#endif
#include <math.h>
#if defined(__CYGWIN__) && defined(_LDBL_EQ_DBL)
#define __Pyx_truncl trunc
#else
#define __Pyx_truncl truncl
#endif

#ifndef CYTHON_CLINE_IN_TRACEBACK_RUNTIME
#define CYTHON_CLINE_IN_TRACEBACK_RUNTIME 0
#endif
#ifndef CYTHON_CLINE_IN_TRACEBACK
#define CYTHON_CLINE_IN_TRACEBACK CYTHON_CLINE_IN_TRACEBACK_RUNTIME
#endif
#if CYTHON_CLINE_IN_TRACEBACK
#define __PYX_MARK_ERR_POS(f_index, lineno)  { __pyx_filename = __pyx_f[f_index]; (void) __pyx_filename; __pyx_lineno = lineno; (void) __pyx_lineno; __pyx_clineno = __LINE__; (void) __pyx_clineno; }
#else
#define __PYX_MARK_ERR_POS(f_index, lineno)  { __pyx_filename = __pyx_f[f_index]; (void) __pyx_filename; __pyx_lineno = lineno; (void) __pyx_lineno; (void) __pyx_clineno; }
#endif
#define __PYX_ERR(f_index, lineno, Ln_error) \
    { __PYX_MARK_ERR_POS(f_index, lineno) goto Ln_error; }

#ifdef CYTHON_EXTERN_C
    #undef __PYX_EXTERN_C
    #define __PYX_EXTERN_C CYTHON_EXTERN_C
#elif defined(__PYX_EXTERN_C)
    #ifdef _MSC_VER
    #pragma message ("Please do not define the '__PYX_EXTERN_C' macro externally. Use 'CYTHON_EXTERN_C' instead.")
    #else
    #warning Please do not define the '__PYX_EXTERN_C' macro externally. Use 'CYTHON_EXTERN_C' instead.
    #endif
#else
    #define __PYX_EXTERN_C extern "C++"
#endif

#define __PYX_HAVE__cupy__cuda__memory
#define __PYX_HAVE_API__cupy__cuda__memory
/* Early includes */
#include <stdint.h>
#include "ios"
#include "new"
#include "stdexcept"
#include "typeinfo"
#include <vector>
#include <utility>

    #if __cplusplus >= 201103L || (defined(_MSC_VER) && _MSC_VER >= 1600)
    // move should be defined for these versions of MSVC, but __cplusplus isn't set usefully
    #include <type_traits>

    namespace cython_std {
    template <typename T> typename std::remove_reference<T>::type&& move(T& t) noexcept { return std::move(t); }
    template <typename T> typename std::remove_reference<T>::type&& move(T&& t) noexcept { return std::move(t); }
    }

    #endif
    
#include <map>
#include <string.h>
#include <stdio.h>

    #if __PYX_LIMITED_VERSION_HEX < 0x030d0000
    static CYTHON_INLINE PyObject *
    __Pyx_CAPI_PyList_GetItemRef(PyObject *list, Py_ssize_t index)
    {
        PyObject *item = PyList_GetItem(list, index);
        Py_XINCREF(item);
        return item;
    }
    #else
    #define __Pyx_CAPI_PyList_GetItemRef PyList_GetItemRef
    #endif

    #if CYTHON_COMPILING_IN_LIMITED_API || PY_VERSION_HEX < 0x030d0000
    static CYTHON_INLINE int
    __Pyx_CAPI_PyList_Extend(PyObject *list, PyObject *iterable)
    {
        return PyList_SetSlice(list, PY_SSIZE_T_MAX, PY_SSIZE_T_MAX, iterable);
    }

    static CYTHON_INLINE int
    __Pyx_CAPI_PyList_Clear(PyObject *list)
    {
        return PyList_SetSlice(list, 0, PY_SSIZE_T_MAX, NULL);
    }
    #else
    #define __Pyx_CAPI_PyList_Extend PyList_Extend
    #define __Pyx_CAPI_PyList_Clear PyList_Clear
    #endif
    

    #if PY_MAJOR_VERSION >= 3
      #define __Pyx_PyFloat_FromString(obj)  PyFloat_FromString(obj)
    #else
      #define __Pyx_PyFloat_FromString(obj)  PyFloat_FromString(obj, NULL)
    #endif
    
#include <stddef.h>

    #if PY_MAJOR_VERSION <= 2
    #define PyDict_GetItemWithError _PyDict_GetItemWithError
    #endif

    #if __PYX_LIMITED_VERSION_HEX < 0x030d0000
    static CYTHON_INLINE int
    __Pyx_CAPI_PyDict_GetItemStringRef(PyObject *mp, const char *key, PyObject **result)
    {
        int res;
        PyObject *key_obj = PyUnicode_FromString(key);
        if (key_obj == NULL) {
            *result = NULL;
            return -1;
        }
        res = __Pyx_PyDict_GetItemRef(mp, key_obj, result);
        Py_DECREF(key_obj);
        return res;
    }
    #else
    #define __Pyx_CAPI_PyDict_GetItemStringRef PyDict_GetItemStringRef
    #endif
    #if PY_VERSION_HEX < 0x030d0000 || (CYTHON_COMPILING_IN_LIMITED_API && __PYX_LIMITED_VERSION_HEX < 0x030F0000)
    static CYTHON_INLINE int
    __Pyx_CAPI_PyDict_SetDefaultRef(PyObject *d, PyObject *key, PyObject *default_value,
                        PyObject **result)
    {
        PyObject *value;
        if (__Pyx_PyDict_GetItemRef(d, key, &value) < 0) {
            // get error
            if (result) {
                *result = NULL;
            }
            return -1;
        }
        if (value != NULL) {
            // present
            if (result) {
                *result = value;
            }
            else {
                Py_DECREF(value);
            }
            return 1;
        }

        // missing: set the item
        if (PyDict_SetItem(d, key, default_value) < 0) {
            // set error
            if (result) {
                *result = NULL;
            }
            return -1;
        }
        if (result) {
            Py_INCREF(default_value);
            *result = default_value;
        }
        return 0;
    }
    #else
    #define __Pyx_CAPI_PyDict_SetDefaultRef PyDict_SetDefaultRef
    #endif
    

    #if PY_VERSION_HEX < 0x030d0000
    static CYTHON_INLINE int __Pyx_PyWeakref_GetRef(PyObject *ref, PyObject **pobj)
    {
        PyObject *obj = PyWeakref_GetObject(ref);
        if (obj == NULL) {
            // SystemError if ref is NULL
            *pobj = NULL;
            return -1;
        }
        if (obj == Py_None) {
            *pobj = NULL;
            return 0;
        }
        Py_INCREF(obj);
        *pobj = obj;
        return 1;
    }
    #else
    #define __Pyx_PyWeakref_GetRef PyWeakref_GetRef
    #endif
    
#include "pythread.h"

    #if (CYTHON_COMPILING_IN_PYPY && PYPY_VERSION_NUM < 0x07030600) && !defined(PyContextVar_Get)
    #define PyContextVar_Get(var, d, v)         ((d) ?             ((void)(var), Py_INCREF(d), (v)[0] = (d), 0) :             ((v)[0] = NULL, 0)         )
    #endif
    
#include <stdlib.h>
#include <atomic>
#include <set>
#include <mutex>
#ifdef _OPENMP
#include <omp.h>
#endif /* _OPENMP */

#if defined(PYREX_WITHOUT_ASSERTIONS) && !defined(CYTHON_WITHOUT_ASSERTIONS)
#define CYTHON_WITHOUT_ASSERTIONS
#endif

#ifdef CYTHON_FREETHREADING_COMPATIBLE
#if CYTHON_FREETHREADING_COMPATIBLE
#define __Pyx_FREETHREADING_COMPATIBLE Py_MOD_GIL_NOT_USED
#else
#define __Pyx_FREETHREADING_COMPATIBLE Py_MOD_GIL_USED
#endif
#else
#define __Pyx_FREETHREADING_COMPATIBLE Py_MOD_GIL_NOT_USED
#endif
#define __PYX_DEFAULT_STRING_ENCODING_IS_ASCII 0
#define __PYX_DEFAULT_STRING_ENCODING_IS_UTF8 0
#define __PYX_DEFAULT_STRING_ENCODING ""
#define __Pyx_PyObject_FromString __Pyx_PyBytes_FromString
#define __Pyx_PyObject_FromStringAndSize __Pyx_PyBytes_FromStringAndSize
#define __Pyx_uchar_cast(c) ((unsigned char)c)
#define __Pyx_long_cast(x) ((long)x)
#define __Pyx_fits_Py_ssize_t(v, type, is_signed)  (\
    (sizeof(type) < sizeof(Py_ssize_t))  ||\
    (sizeof(type) > sizeof(Py_ssize_t) &&\
          likely(v < (type)PY_SSIZE_T_MAX ||\
                 v == (type)PY_SSIZE_T_MAX)  &&\
          (!is_signed || likely(v > (type)PY_SSIZE_T_MIN ||\
                                v == (type)PY_SSIZE_T_MIN)))  ||\
    (sizeof(type) == sizeof(Py_ssize_t) &&\
          (is_signed || likely(v < (type)PY_SSIZE_T_MAX ||\
                               v == (type)PY_SSIZE_T_MAX)))  )
static CYTHON_INLINE int __Pyx_is_valid_index(Py_ssize_t i, Py_ssize_t limit) {
    return (size_t) i < (size_t) limit;
}
#if defined (__cplusplus) && __cplusplus >= 201103L
    #include <cstdlib>
    #define __Pyx_sst_abs(value) std::abs(value)
#elif SIZEOF_INT >= SIZEOF_SIZE_T
    #define __Pyx_sst_abs(value) abs(value)
#elif SIZEOF_LONG >= SIZEOF_SIZE_T
    #define __Pyx_sst_abs(value) labs(value)
#elif defined (_MSC_VER)
    #define __Pyx_sst_abs(value) ((Py_ssize_t)_abs64(value))
#elif defined (__STDC_VERSION__) && __STDC_VERSION__ >= 199901L
    #define __Pyx_sst_abs(value) llabs(value)
#elif defined (__GNUC__)
    #define __Pyx_sst_abs(value) __builtin_llabs(value)
#else
    #define __Pyx_sst_abs(value) ((value<0) ? -value : value)
#endif
static CYTHON_INLINE Py_ssize_t __Pyx_ssize_strlen(const char *s);
static CYTHON_INLINE const char* __Pyx_PyObject_AsString(PyObject*);
static CYTHON_INLINE const char* __Pyx_PyObject_AsStringAndSize(PyObject*, Py_ssize_t* length);
static CYTHON_INLINE PyObject* __Pyx_PyByteArray_FromString(const char*);
#define __Pyx_PyByteArray_FromStringAndSize(s, l) PyByteArray_FromStringAndSize((const char*)s, l)
#define __Pyx_PyBytes_FromString        PyBytes_FromString
#define __Pyx_PyBytes_FromStringAndSize PyBytes_FromStringAndSize
static CYTHON_INLINE PyObject* __Pyx_PyUnicode_FromString(const char*);
#if CYTHON_ASSUME_SAFE_MACROS
    #define __Pyx_PyBytes_AsWritableString(s)     ((char*) PyBytes_AS_STRING(s))
    #define __Pyx_PyBytes_AsWritableSString(s)    ((signed char*) PyBytes_AS_STRING(s))
    #define __Pyx_PyBytes_AsWritableUString(s)    ((unsigned char*) PyBytes_AS_STRING(s))
    #define __Pyx_PyBytes_AsString(s)     ((const char*) PyBytes_AS_STRING(s))
    #define __Pyx_PyBytes_AsSString(s)    ((const signed char*) PyBytes_AS_STRING(s))
    #define __Pyx_PyBytes_AsUString(s)    ((const unsigned char*) PyBytes_AS_STRING(s))
    #define __Pyx_PyByteArray_AsString(s) PyByteArray_AS_STRING(s)
#else
    #define __Pyx_PyBytes_AsWritableString(s)     ((char*) PyBytes_AsString(s))
    #define __Pyx_PyBytes_AsWritableSString(s)    ((signed char*) PyBytes_AsString(s))
    #define __Pyx_PyBytes_AsWritableUString(s)    ((unsigned char*) PyBytes_AsString(s))
    #define __Pyx_PyBytes_AsString(s)     ((const char*) PyBytes_AsString(s))
    #define __Pyx_PyBytes_AsSString(s)    ((const signed char*) PyBytes_AsString(s))
    #define __Pyx_PyBytes_AsUString(s)    ((const unsigned char*) PyBytes_AsString(s))
    #define __Pyx_PyByteArray_AsString(s) PyByteArray_AsString(s)
#endif
#define __Pyx_PyObject_AsWritableString(s)    ((char*)(__pyx_uintptr_t) __Pyx_PyObject_AsString(s))
#define __Pyx_PyObject_AsWritableSString(s)    ((signed char*)(__pyx_uintptr_t) __Pyx_PyObject_AsString(s))
#define __Pyx_PyObject_AsWritableUString(s)    ((unsigned char*)(__pyx_uintptr_t) __Pyx_PyObject_AsString(s))
#define __Pyx_PyObject_AsSString(s)    ((const signed char*) __Pyx_PyObject_AsString(s))
#define __Pyx_PyObject_AsUString(s)    ((const unsigned char*) __Pyx_PyObject_AsString(s))
#define __Pyx_PyObject_FromCString(s)  __Pyx_PyObject_FromString((const char*)s)
#define __Pyx_PyBytes_FromCString(s)   __Pyx_PyBytes_FromString((const char*)s)
#define __Pyx_PyByteArray_FromCString(s)   __Pyx_PyByteArray_FromString((const char*)s)
#define __Pyx_PyUnicode_FromCString(s) __Pyx_PyUnicode_FromString((const char*)s)
#define __Pyx_PyUnicode_FromOrdinal(o)       PyUnicode_FromOrdinal((int)o)
#define __Pyx_PyUnicode_AsUnicode            PyUnicode_AsUnicode
static CYTHON_INLINE PyObject *__Pyx_NewRef(PyObject *obj) {
#if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x030a0000 || defined(Py_NewRef)
    return Py_NewRef(obj);
#else
    Py_INCREF(obj);
    return obj;
#endif
}
static CYTHON_INLINE PyObject *__Pyx_XNewRef(PyObject *obj) {
#if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x030a0000 || defined(Py_XNewRef)
    return Py_XNewRef(obj);
#else
    Py_XINCREF(obj);
    return obj;
#endif
}
static CYTHON_INLINE PyObject *__Pyx_Owned_Py_None(int b);
static CYTHON_INLINE PyObject * __Pyx_PyBool_FromLong(long b);
static CYTHON_INLINE int __Pyx_PyObject_IsTrue(PyObject*);
static CYTHON_INLINE int __Pyx_PyObject_IsTrueAndDecref(PyObject*);
static CYTHON_INLINE PyObject* __Pyx_PyNumber_Long(PyObject* x);
#define __Pyx_PySequence_Tuple(obj)\
    (likely(PyTuple_CheckExact(obj)) ? __Pyx_NewRef(obj) : PySequence_Tuple(obj))
static CYTHON_INLINE Py_ssize_t __Pyx_PyIndex_AsSsize_t(PyObject*);
static CYTHON_INLINE PyObject * __Pyx_PyLong_FromSize_t(size_t);
static CYTHON_INLINE Py_hash_t __Pyx_PyIndex_AsHash_t(PyObject*);
#if CYTHON_ASSUME_SAFE_MACROS
#define __Pyx_PyFloat_AsDouble(x) (PyFloat_CheckExact(x) ? PyFloat_AS_DOUBLE(x) : PyFloat_AsDouble(x))
#define __Pyx_PyFloat_AS_DOUBLE(x) PyFloat_AS_DOUBLE(x)
#else
#define __Pyx_PyFloat_AsDouble(x) PyFloat_AsDouble(x)
#define __Pyx_PyFloat_AS_DOUBLE(x) PyFloat_AsDouble(x)
#endif
#define __Pyx_PyFloat_AsFloat(x) ((float) __Pyx_PyFloat_AsDouble(x))
#define __Pyx_PyNumber_Int(x) (PyLong_CheckExact(x) ? __Pyx_NewRef(x) : PyNumber_Long(x))
#if CYTHON_USE_PYLONG_INTERNALS
  #if PY_VERSION_HEX >= 0x030C00A7
  #ifndef _PyLong_SIGN_MASK
    #define _PyLong_SIGN_MASK 3
  #endif
  #ifndef _PyLong_NON_SIZE_BITS
    #define _PyLong_NON_SIZE_BITS 3
  #endif
  #define __Pyx_PyLong_Sign(x)  (((PyLongObject*)x)->long_value.lv_tag & _PyLong_SIGN_MASK)
  #define __Pyx_PyLong_IsNeg(x)  ((__Pyx_PyLong_Sign(x) & 2) != 0)
  #define __Pyx_PyLong_IsNonNeg(x)  (!__Pyx_PyLong_IsNeg(x))
  #define __Pyx_PyLong_IsZero(x)  (__Pyx_PyLong_Sign(x) & 1)
  #define __Pyx_PyLong_IsPos(x)  (__Pyx_PyLong_Sign(x) == 0)
  #define __Pyx_PyLong_CompactValueUnsigned(x)  (__Pyx_PyLong_Digits(x)[0])
  #define __Pyx_PyLong_DigitCount(x)  ((Py_ssize_t) (((PyLongObject*)x)->long_value.lv_tag >> _PyLong_NON_SIZE_BITS))
  #define __Pyx_PyLong_SignedDigitCount(x)\
        ((1 - (Py_ssize_t) __Pyx_PyLong_Sign(x)) * __Pyx_PyLong_DigitCount(x))
  #if defined(PyUnstable_Long_IsCompact) && defined(PyUnstable_Long_CompactValue)
    #define __Pyx_PyLong_IsCompact(x)     PyUnstable_Long_IsCompact((PyLongObject*) x)
    #define __Pyx_PyLong_CompactValue(x)  PyUnstable_Long_CompactValue((PyLongObject*) x)
  #else
    #define __Pyx_PyLong_IsCompact(x)     (((PyLongObject*)x)->long_value.lv_tag < (2 << _PyLong_NON_SIZE_BITS))
    #define __Pyx_PyLong_CompactValue(x)  ((1 - (Py_ssize_t) __Pyx_PyLong_Sign(x)) * (Py_ssize_t) __Pyx_PyLong_Digits(x)[0])
  #endif
  typedef Py_ssize_t  __Pyx_compact_pylong;
  typedef size_t  __Pyx_compact_upylong;
  #else
  #define __Pyx_PyLong_IsNeg(x)  (Py_SIZE(x) < 0)
  #define __Pyx_PyLong_IsNonNeg(x)  (Py_SIZE(x) >= 0)
  #define __Pyx_PyLong_IsZero(x)  (Py_SIZE(x) == 0)
  #define __Pyx_PyLong_IsPos(x)  (Py_SIZE(x) > 0)
  #define __Pyx_PyLong_CompactValueUnsigned(x)  ((Py_SIZE(x) == 0) ? 0 : __Pyx_PyLong_Digits(x)[0])
  #define __Pyx_PyLong_DigitCount(x)  __Pyx_sst_abs(Py_SIZE(x))
  #define __Pyx_PyLong_SignedDigitCount(x)  Py_SIZE(x)
  #define __Pyx_PyLong_IsCompact(x)  (Py_SIZE(x) == 0 || Py_SIZE(x) == 1 || Py_SIZE(x) == -1)
  #define __Pyx_PyLong_CompactValue(x)\
        ((Py_SIZE(x) == 0) ? (sdigit) 0 : ((Py_SIZE(x) < 0) ? -(sdigit)__Pyx_PyLong_Digits(x)[0] : (sdigit)__Pyx_PyLong_Digits(x)[0]))
  typedef sdigit  __Pyx_compact_pylong;
  typedef digit  __Pyx_compact_upylong;
  #endif
  #if PY_VERSION_HEX >= 0x030C00A5
  #define __Pyx_PyLong_Digits(x)  (((PyLongObject*)x)->long_value.ob_digit)
  #else
  #define __Pyx_PyLong_Digits(x)  (((PyLongObject*)x)->ob_digit)
  #endif
#endif
#if __PYX_DEFAULT_STRING_ENCODING_IS_UTF8
  #define __Pyx_PyUnicode_FromStringAndSize(c_str, size) PyUnicode_DecodeUTF8(c_str, size, NULL)
#elif __PYX_DEFAULT_STRING_ENCODING_IS_ASCII
  #define __Pyx_PyUnicode_FromStringAndSize(c_str, size) PyUnicode_DecodeASCII(c_str, size, NULL)
#else
  #define __Pyx_PyUnicode_FromStringAndSize(c_str, size) PyUnicode_Decode(c_str, size, __PYX_DEFAULT_STRING_ENCODING, NULL)
#endif


/* Test for GCC > 2.95 */
#if defined(__GNUC__)     && (__GNUC__ > 2 || (__GNUC__ == 2 && (__GNUC_MINOR__ > 95)))
  #define likely(x)   __builtin_expect(!!(x), 1)
  #define unlikely(x) __builtin_expect(!!(x), 0)
#else /* !__GNUC__ or GCC < 2.95 */
  #define likely(x)   (x)
  #define unlikely(x) (x)
#endif /* __GNUC__ */
/* PretendToInitialize */
#ifdef __cplusplus
#if __cplusplus > 201103L
#include <type_traits>
#endif
template <typename T>
static void __Pyx_pretend_to_initialize(T* ptr) {
#if __cplusplus > 201103L
    if ((std::is_trivially_default_constructible<T>::value))
#endif
        *ptr = T();
    (void)ptr;
}
#else
static CYTHON_INLINE void __Pyx_pretend_to_initialize(void* ptr) { (void)ptr; }
#endif


#if !CYTHON_USE_MODULE_STATE
static PyObject *__pyx_m = NULL;
#endif
static int __pyx_lineno;
static int __pyx_clineno = 0;
static const char * const __pyx_cfilenm = __FILE__;
static const char *__pyx_filename;

/* #### Code section: filename_table ### */

static const char* const __pyx_f[] = {
  "cupy/cuda/memory.pyx",
  "cupy/cuda/memory.pxd",
  "cpython/contextvars.pxd",
  "(tree fragment)",
  "cupy/cuda/device.pxd",
  "cpython/type.pxd",
  "cpython/bool.pxd",
  "cpython/complex.pxd",
  "cupy_backends/cuda/api/runtime.pxd",
};
/* #### Code section: utility_code_proto_before_types ### */
/* CriticalSectionsDefinition.proto (used by CriticalSections) */
#if !CYTHON_COMPILING_IN_CPYTHON_FREETHREADING
#define __Pyx_PyCriticalSection void*
#define __Pyx_PyCriticalSection2 void*
#define __Pyx_PyCriticalSection_End(cs)
#define __Pyx_PyCriticalSection2_End(cs)
#else
#define __Pyx_PyCriticalSection PyCriticalSection
#define __Pyx_PyCriticalSection2 PyCriticalSection2
#define __Pyx_PyCriticalSection_End PyCriticalSection_End
#define __Pyx_PyCriticalSection2_End PyCriticalSection2_End
#endif

/* CriticalSections.proto */
#if !CYTHON_COMPILING_IN_CPYTHON_FREETHREADING
#define __Pyx_PyCriticalSection_Begin(cs, arg) (void)(cs)
#define __Pyx_PyCriticalSection2_Begin(cs, arg1, arg2) (void)(cs)
#else
#define __Pyx_PyCriticalSection_Begin PyCriticalSection_Begin
#define __Pyx_PyCriticalSection2_Begin PyCriticalSection2_Begin
#endif
#if PY_VERSION_HEX < 0x030d0000 || CYTHON_COMPILING_IN_LIMITED_API
#define __Pyx_BEGIN_CRITICAL_SECTION(o) {
#define __Pyx_END_CRITICAL_SECTION() }
#else
#define __Pyx_BEGIN_CRITICAL_SECTION Py_BEGIN_CRITICAL_SECTION
#define __Pyx_END_CRITICAL_SECTION Py_END_CRITICAL_SECTION
#endif

/* NoFastGil.proto */
#define __Pyx_PyGILState_Ensure PyGILState_Ensure
#define __Pyx_PyGILState_Release PyGILState_Release
#define __Pyx_FastGIL_Remember()
#define __Pyx_FastGIL_Forget()
#define __Pyx_FastGilFuncInit()

/* Atomics.proto (used by UnpackUnboundCMethod) */
#include <pythread.h>
#ifndef CYTHON_ATOMICS
    #define CYTHON_ATOMICS 1
#endif
#define __PYX_CYTHON_ATOMICS_ENABLED() CYTHON_ATOMICS
#define __PYX_GET_CYTHON_COMPILING_IN_CPYTHON_FREETHREADING() CYTHON_COMPILING_IN_CPYTHON_FREETHREADING
#define __pyx_atomic_int_type int
#define __pyx_nonatomic_int_type int
#if CYTHON_ATOMICS && (defined(__STDC_VERSION__) &&\
                        (__STDC_VERSION__ >= 201112L) &&\
                        !defined(__STDC_NO_ATOMICS__))
    #include <stdatomic.h>
#elif CYTHON_ATOMICS && (defined(__cplusplus) && (\
                    (__cplusplus >= 201103L) ||\
                    (defined(_MSC_VER) && _MSC_VER >= 1700)))
    #include <atomic>
#endif
#if CYTHON_ATOMICS && (defined(__STDC_VERSION__) &&\
                        (__STDC_VERSION__ >= 201112L) &&\
                        !defined(__STDC_NO_ATOMICS__) &&\
                       ATOMIC_INT_LOCK_FREE == 2)
    #undef __pyx_atomic_int_type
    #define __pyx_atomic_int_type atomic_int
    #define __pyx_atomic_ptr_type atomic_uintptr_t
    #define __pyx_nonatomic_ptr_type uintptr_t
    #define __pyx_atomic_incr_relaxed(value) atomic_fetch_add_explicit(value, 1, memory_order_relaxed)
    #define __pyx_atomic_incr_acq_rel(value) atomic_fetch_add_explicit(value, 1, memory_order_acq_rel)
    #define __pyx_atomic_decr_acq_rel(value) atomic_fetch_sub_explicit(value, 1, memory_order_acq_rel)
    #define __pyx_atomic_sub(value, arg) atomic_fetch_sub(value, arg)
    #define __pyx_atomic_int_cmp_exchange(value, expected, desired) atomic_compare_exchange_strong(value, expected, desired)
    #define __pyx_atomic_load(value) atomic_load(value)
    #define __pyx_atomic_store(value, new_value) atomic_store(value, new_value)
    #define __pyx_atomic_pointer_load_relaxed(value) atomic_load_explicit(value, memory_order_relaxed)
    #define __pyx_atomic_pointer_load_acquire(value) atomic_load_explicit(value, memory_order_acquire)
    #define __pyx_atomic_pointer_exchange(value, new_value) atomic_exchange(value, (__pyx_nonatomic_ptr_type)new_value)
    #define __pyx_atomic_pointer_cmp_exchange(value, expected, desired) atomic_compare_exchange_strong(value, expected, desired)
    #if defined(__PYX_DEBUG_ATOMICS) && defined(_MSC_VER)
        #pragma message ("Using standard C atomics")
    #elif defined(__PYX_DEBUG_ATOMICS)
        #warning "Using standard C atomics"
    #endif
#elif CYTHON_ATOMICS && (defined(__cplusplus) && (\
                    (__cplusplus >= 201103L) ||\
\
                    (defined(_MSC_VER) && _MSC_VER >= 1700)) &&\
                    ATOMIC_INT_LOCK_FREE == 2)
    #undef __pyx_atomic_int_type
    #define __pyx_atomic_int_type std::atomic_int
    #define __pyx_atomic_ptr_type std::atomic_uintptr_t
    #define __pyx_nonatomic_ptr_type uintptr_t
    #define __pyx_atomic_incr_relaxed(value) std::atomic_fetch_add_explicit(value, 1, std::memory_order_relaxed)
    #define __pyx_atomic_incr_acq_rel(value) std::atomic_fetch_add_explicit(value, 1, std::memory_order_acq_rel)
    #define __pyx_atomic_decr_acq_rel(value) std::atomic_fetch_sub_explicit(value, 1, std::memory_order_acq_rel)
    #define __pyx_atomic_sub(value, arg) std::atomic_fetch_sub(value, arg)
    #define __pyx_atomic_int_cmp_exchange(value, expected, desired) std::atomic_compare_exchange_strong(value, expected, desired)
    #define __pyx_atomic_load(value) std::atomic_load(value)
    #define __pyx_atomic_store(value, new_value) std::atomic_store(value, new_value)
    #define __pyx_atomic_pointer_load_relaxed(value) std::atomic_load_explicit(value, std::memory_order_relaxed)
    #define __pyx_atomic_pointer_load_acquire(value) std::atomic_load_explicit(value, std::memory_order_acquire)
    #define __pyx_atomic_pointer_exchange(value, new_value) std::atomic_exchange(value, (__pyx_nonatomic_ptr_type)new_value)
    #define __pyx_atomic_pointer_cmp_exchange(value, expected, desired) std::atomic_compare_exchange_strong(value, expected, desired)
    #if defined(__PYX_DEBUG_ATOMICS) && defined(_MSC_VER)
        #pragma message ("Using standard C++ atomics")
    #elif defined(__PYX_DEBUG_ATOMICS)
        #warning "Using standard C++ atomics"
    #endif
#elif CYTHON_ATOMICS && (__GNUC__ >= 5 || (__GNUC__ == 4 &&\
                    (__GNUC_MINOR__ > 1 ||\
                    (__GNUC_MINOR__ == 1 && __GNUC_PATCHLEVEL__ >= 2))))
    #define __pyx_atomic_ptr_type void*
    #define __pyx_nonatomic_ptr_type void*
    #define __pyx_atomic_incr_relaxed(value) __sync_fetch_and_add(value, 1)
    #define __pyx_atomic_incr_acq_rel(value) __sync_fetch_and_add(value, 1)
    #define __pyx_atomic_decr_acq_rel(value) __sync_fetch_and_sub(value, 1)
    #define __pyx_atomic_sub(value, arg) __sync_fetch_and_sub(value, arg)
    static CYTHON_INLINE int __pyx_atomic_int_cmp_exchange(__pyx_atomic_int_type* value, __pyx_nonatomic_int_type* expected, __pyx_nonatomic_int_type desired) {
        __pyx_nonatomic_int_type old = __sync_val_compare_and_swap(value, *expected, desired);
        int result = old == *expected;
        *expected = old;
        return result;
    }
    #define __pyx_atomic_load(value) __sync_fetch_and_add(value, 0)
    #define __pyx_atomic_store(value, new_value) __sync_lock_test_and_set(value, new_value)
    #define __pyx_atomic_pointer_load_relaxed(value) __sync_fetch_and_add(value, 0)
    #define __pyx_atomic_pointer_load_acquire(value) __sync_fetch_and_add(value, 0)
    #define __pyx_atomic_pointer_exchange(value, new_value) __sync_lock_test_and_set(value, (__pyx_atomic_ptr_type)new_value)
    static CYTHON_INLINE int __pyx_atomic_pointer_cmp_exchange(__pyx_atomic_ptr_type* value, __pyx_nonatomic_ptr_type* expected, __pyx_nonatomic_ptr_type desired) {
        __pyx_nonatomic_ptr_type old = __sync_val_compare_and_swap(value, *expected, desired);
        int result = old == *expected;
        *expected = old;
        return result;
    }
    #ifdef __PYX_DEBUG_ATOMICS
        #warning "Using GNU atomics"
    #endif
#elif CYTHON_ATOMICS && defined(_MSC_VER)
    #include <intrin.h>
    #undef __pyx_atomic_int_type
    #define __pyx_atomic_int_type long
    #define __pyx_atomic_ptr_type void*
    #undef __pyx_nonatomic_int_type
    #define __pyx_nonatomic_int_type long
    #define __pyx_nonatomic_ptr_type void*
    #pragma intrinsic (_InterlockedExchangeAdd, _InterlockedExchange, _InterlockedCompareExchange, _InterlockedCompareExchangePointer, _InterlockedExchangePointer)
    #define __pyx_atomic_incr_relaxed(value) _InterlockedExchangeAdd(value, 1)
    #define __pyx_atomic_incr_acq_rel(value) _InterlockedExchangeAdd(value, 1)
    #define __pyx_atomic_decr_acq_rel(value) _InterlockedExchangeAdd(value, -1)
    #define __pyx_atomic_sub(value, arg) _InterlockedExchangeAdd(value, -arg)
    static CYTHON_INLINE int __pyx_atomic_int_cmp_exchange(__pyx_atomic_int_type* value, __pyx_nonatomic_int_type* expected, __pyx_nonatomic_int_type desired) {
        __pyx_nonatomic_int_type old = _InterlockedCompareExchange(value, desired, *expected);
        int result = old == *expected;
        *expected = old;
        return result;
    }
    #define __pyx_atomic_load(value) _InterlockedExchangeAdd(value, 0)
    #define __pyx_atomic_store(value, new_value) _InterlockedExchange(value, new_value)
    #define __pyx_atomic_pointer_load_relaxed(value) *(void * volatile *)value
    #define __pyx_atomic_pointer_load_acquire(value) _InterlockedCompareExchangePointer(value, 0, 0)
    #define __pyx_atomic_pointer_exchange(value, new_value) _InterlockedExchangePointer(value, (__pyx_atomic_ptr_type)new_value)
    static CYTHON_INLINE int __pyx_atomic_pointer_cmp_exchange(__pyx_atomic_ptr_type* value, __pyx_nonatomic_ptr_type* expected, __pyx_nonatomic_ptr_type desired) {
        __pyx_atomic_ptr_type old = _InterlockedCompareExchangePointer(value, desired, *expected);
        int result = old == *expected;
        *expected = old;
        return result;
    }
    #ifdef __PYX_DEBUG_ATOMICS
        #pragma message ("Using MSVC atomics")
    #endif
#else
    #undef CYTHON_ATOMICS
    #define CYTHON_ATOMICS 0
    #ifdef __PYX_DEBUG_ATOMICS
        #warning "Not using atomics"
    #endif
#endif

/* ForceInitThreads.proto */
#ifndef __PYX_FORCE_INIT_THREADS
  #define __PYX_FORCE_INIT_THREADS 0
#endif

/* IncludeStructmemberH.proto */
#include <structmember.h>

/* #### Code section: numeric_typedefs ### */
/* #### Code section: complex_type_declarations ### */
/* #### Code section: type_declarations ### */

/*--- Type declarations ---*/
struct __pyx_obj_4cupy_4cuda_6device_Handle;
struct __pyx_obj_4cupy_4cuda_6device_Device;
struct __pyx_obj_13cupy_backends_4cuda_3api_7runtime_PointerAttributes;
struct __pyx_obj_13cupy_backends_4cuda_3api_7runtime_MemPoolProps;
struct __pyx_obj_4cupy_4cuda_6memory_BaseMemory;
struct __pyx_obj_4cupy_4cuda_6memory_SystemMemory;
struct __pyx_obj_4cupy_4cuda_6memory_MemoryPointer;
struct __pyx_obj_4cupy_4cuda_6memory_MemoryPool;
struct __pyx_obj_4cupy_4cuda_6memory_CFunctionAllocatorMemory;
struct __pyx_obj_4cupy_4cuda_6memory_CFunctionAllocator;
struct __pyx_obj_4cupy_4cuda_6memory_PythonFunctionAllocatorMemory;
struct __pyx_obj_4cupy_4cuda_6memory_PythonFunctionAllocator;
struct __pyx_obj_4cupy_4cuda_6memory_Memory;
struct __pyx_obj_4cupy_4cuda_6memory_MemoryAsync;
struct __pyx_obj_4cupy_4cuda_6memory_UnownedMemory;
struct __pyx_obj_4cupy_4cuda_6memory_ManagedMemory;
struct __pyx_obj_4cupy_4cuda_6memory__Chunk;
struct __pyx_obj_4cupy_4cuda_6memory_PooledMemory;
struct __pyx_obj_4cupy_4cuda_6memory__Arena;
struct __pyx_obj_4cupy_4cuda_6memory_SingleDeviceMemoryPool;
struct __pyx_obj_4cupy_4cuda_6memory_MemoryAsyncPool;
struct __pyx_obj_4cupy_4cuda_6memory___pyx_scope_struct___ensure_pools_and_return_device_pool;
struct __pyx_obj_4cupy_4cuda_6memory___pyx_scope_struct_1_genexpr;
struct __pyx_opt_args_7cpython_11contextvars_get_value;
struct __pyx_opt_args_7cpython_11contextvars_get_value_no_default;

/* "cpython/contextvars.pxd":116
 * 
 * @_cython.c_compile_guard("!CYTHON_COMPILING_IN_LIMITED_API")
 * cdef inline object get_value(var, default_value=None):             # <<<<<<<<<<<<<<
 *     """Return a new reference to the value of the context variable,
 *     or the default value of the context variable,
*/
struct __pyx_opt_args_7cpython_11contextvars_get_value {
  int __pyx_n;
  PyObject *default_value;
};

/* "cpython/contextvars.pxd":134
 * 
 * @_cython.c_compile_guard("!CYTHON_COMPILING_IN_LIMITED_API")
 * cdef inline object get_value_no_default(var, default_value=None):             # <<<<<<<<<<<<<<
 *     """Return a new reference to the value of the context variable,
 *     or the provided default value if no such value was found.
*/
struct __pyx_opt_args_7cpython_11contextvars_get_value_no_default {
  int __pyx_n;
  PyObject *default_value;
};
struct __pyx_opt_args_4cupy_4cuda_6stream_get_current_stream;

/* "stream.pxd":5
 * 
 * cdef intptr_t get_current_stream_ptr() except? -1
 * cpdef get_current_stream(int device_id=*)             # <<<<<<<<<<<<<<
*/
struct __pyx_opt_args_4cupy_4cuda_6stream_get_current_stream {
  int __pyx_n;
  int device_id;
};

/* "cupy_backends/cuda/api/_driver_enum.pxd":1
 * cpdef enum:             # <<<<<<<<<<<<<<
 *     # CUjitInputType
 *     CU_JIT_INPUT_CUBIN = 0
*/
enum  {
  __pyx_e_13cupy_backends_4cuda_3api_12_driver_enum_CU_JIT_INPUT_CUBIN = 0,
  __pyx_e_13cupy_backends_4cuda_3api_12_driver_enum_CU_JIT_INPUT_PTX = 1,
  __pyx_e_13cupy_backends_4cuda_3api_12_driver_enum_CU_JIT_INPUT_FATBINARY = 2,
  __pyx_e_13cupy_backends_4cuda_3api_12_driver_enum_CU_JIT_INPUT_OBJECT = 3,
  __pyx_e_13cupy_backends_4cuda_3api_12_driver_enum_CU_JIT_INPUT_LIBRARY = 4,
  __pyx_e_13cupy_backends_4cuda_3api_12_driver_enum_CU_JIT_INPUT_NVVM = 5,
  __pyx_e_13cupy_backends_4cuda_3api_12_driver_enum_CU_JIT_MAX_REGISTERS = 0,
  __pyx_e_13cupy_backends_4cuda_3api_12_driver_enum_CU_JIT_THREADS_PER_BLOCK = 1,
  __pyx_e_13cupy_backends_4cuda_3api_12_driver_enum_CU_JIT_WALL_TIME = 2,
  __pyx_e_13cupy_backends_4cuda_3api_12_driver_enum_CU_JIT_INFO_LOG_BUFFER = 3,
  __pyx_e_13cupy_backends_4cuda_3api_12_driver_enum_CU_JIT_INFO_LOG_BUFFER_SIZE_BYTES = 4,
  __pyx_e_13cupy_backends_4cuda_3api_12_driver_enum_CU_JIT_ERROR_LOG_BUFFER = 5,
  __pyx_e_13cupy_backends_4cuda_3api_12_driver_enum_CU_JIT_ERROR_LOG_BUFFER_SIZE_BYTES = 6,
  __pyx_e_13cupy_backends_4cuda_3api_12_driver_enum_CU_JIT_OPTIMIZATION_LEVEL = 7,
  __pyx_e_13cupy_backends_4cuda_3api_12_driver_enum_CU_JIT_TARGET_FROM_CUCONTEXT = 8,
  __pyx_e_13cupy_backends_4cuda_3api_12_driver_enum_CU_JIT_TARGET = 9,
  __pyx_e_13cupy_backends_4cuda_3api_12_driver_enum_CU_JIT_FALLBACK_STRATEGY = 10,
  __pyx_e_13cupy_backends_4cuda_3api_12_driver_enum_CU_JIT_GENERATE_DEBUG_INFO = 11,
  __pyx_e_13cupy_backends_4cuda_3api_12_driver_enum_CU_JIT_LOG_VERBOSE = 12,
  __pyx_e_13cupy_backends_4cuda_3api_12_driver_enum_CU_JIT_GENERATE_LINE_INFO = 13,
  __pyx_e_13cupy_backends_4cuda_3api_12_driver_enum_CU_JIT_CACHE_MODE = 14,
  __pyx_e_13cupy_backends_4cuda_3api_12_driver_enum_CU_JIT_NEW_SM3X_OPT = 15,
  __pyx_e_13cupy_backends_4cuda_3api_12_driver_enum_CU_JIT_FAST_COMPILE = 16,
  __pyx_e_13cupy_backends_4cuda_3api_12_driver_enum_CU_JIT_GLOBAL_SYMBOL_NAMES = 17,
  __pyx_e_13cupy_backends_4cuda_3api_12_driver_enum_CU_JIT_GLOBAL_SYMBOL_ADDRESSES = 18,
  __pyx_e_13cupy_backends_4cuda_3api_12_driver_enum_CU_JIT_GLOBAL_SYMBOL_COUNT = 19,
  __pyx_e_13cupy_backends_4cuda_3api_12_driver_enum_CU_JIT_LTO = 20,
  __pyx_e_13cupy_backends_4cuda_3api_12_driver_enum_CU_JIT_FTZ = 21,
  __pyx_e_13cupy_backends_4cuda_3api_12_driver_enum_CU_JIT_PREC_DIV = 22,
  __pyx_e_13cupy_backends_4cuda_3api_12_driver_enum_CU_JIT_PREC_SQRT = 23,
  __pyx_e_13cupy_backends_4cuda_3api_12_driver_enum_CU_JIT_FMA = 24,
  __pyx_e_13cupy_backends_4cuda_3api_12_driver_enum_CU_JIT_NUM_OPTIONS = 25,
  __pyx_e_13cupy_backends_4cuda_3api_12_driver_enum_CU_FUNC_ATTRIBUTE_MAX_THREADS_PER_BLOCK = 0,
  __pyx_e_13cupy_backends_4cuda_3api_12_driver_enum_CU_FUNC_ATTRIBUTE_SHARED_SIZE_BYTES = 1,
  __pyx_e_13cupy_backends_4cuda_3api_12_driver_enum_CU_FUNC_ATTRIBUTE_CONST_SIZE_BYTES = 2,
  __pyx_e_13cupy_backends_4cuda_3api_12_driver_enum_CU_FUNC_ATTRIBUTE_LOCAL_SIZE_BYTES = 3,
  __pyx_e_13cupy_backends_4cuda_3api_12_driver_enum_CU_FUNC_ATTRIBUTE_NUM_REGS = 4,
  __pyx_e_13cupy_backends_4cuda_3api_12_driver_enum_CU_FUNC_ATTRIBUTE_PTX_VERSION = 5,
  __pyx_e_13cupy_backends_4cuda_3api_12_driver_enum_CU_FUNC_ATTRIBUTE_BINARY_VERSION = 6,
  __pyx_e_13cupy_backends_4cuda_3api_12_driver_enum_CU_FUNC_ATTRIBUTE_CACHE_MODE_CA = 7,
  __pyx_e_13cupy_backends_4cuda_3api_12_driver_enum_CU_FUNC_ATTRIBUTE_MAX_DYNAMIC_SHARED_SIZE_BYTES = 8,
  __pyx_e_13cupy_backends_4cuda_3api_12_driver_enum_CU_FUNC_ATTRIBUTE_PREFERRED_SHARED_MEMORY_CARVEOUT = 9,
  __pyx_e_13cupy_backends_4cuda_3api_12_driver_enum_CUDA_ERROR_INVALID_VALUE = 1,
  __pyx_e_13cupy_backends_4cuda_3api_12_driver_enum_CU_AD_FORMAT_UNSIGNED_INT8 = 0x01,
  __pyx_e_13cupy_backends_4cuda_3api_12_driver_enum_CU_AD_FORMAT_UNSIGNED_INT16 = 0x02,
  __pyx_e_13cupy_backends_4cuda_3api_12_driver_enum_CU_AD_FORMAT_UNSIGNED_INT32 = 0x03,
  __pyx_e_13cupy_backends_4cuda_3api_12_driver_enum_CU_AD_FORMAT_SIGNED_INT8 = 0x08,
  __pyx_e_13cupy_backends_4cuda_3api_12_driver_enum_CU_AD_FORMAT_SIGNED_INT16 = 0x09,
  __pyx_e_13cupy_backends_4cuda_3api_12_driver_enum_CU_AD_FORMAT_SIGNED_INT32 = 0x0a,
  __pyx_e_13cupy_backends_4cuda_3api_12_driver_enum_CU_AD_FORMAT_HALF = 0x10,
  __pyx_e_13cupy_backends_4cuda_3api_12_driver_enum_CU_AD_FORMAT_FLOAT = 0x20,
  __pyx_e_13cupy_backends_4cuda_3api_12_driver_enum_CU_TR_ADDRESS_MODE_WRAP = 0,
  __pyx_e_13cupy_backends_4cuda_3api_12_driver_enum_CU_TR_ADDRESS_MODE_CLAMP = 1,
  __pyx_e_13cupy_backends_4cuda_3api_12_driver_enum_CU_TR_ADDRESS_MODE_MIRROR = 2,
  __pyx_e_13cupy_backends_4cuda_3api_12_driver_enum_CU_TR_ADDRESS_MODE_BORDER = 3,
  __pyx_e_13cupy_backends_4cuda_3api_12_driver_enum_CU_TR_FILTER_MODE_POINT = 0,
  __pyx_e_13cupy_backends_4cuda_3api_12_driver_enum_CU_TR_FILTER_MODE_LINEAR = 1,
  __pyx_e_13cupy_backends_4cuda_3api_12_driver_enum_CU_TRSA_OVERRIDE_FORMAT = 0x01,
  __pyx_e_13cupy_backends_4cuda_3api_12_driver_enum_CU_TRSF_READ_AS_INTEGER = 0x01,
  __pyx_e_13cupy_backends_4cuda_3api_12_driver_enum_CU_TRSF_NORMALIZED_COORDINATES = 0x02,
  __pyx_e_13cupy_backends_4cuda_3api_12_driver_enum_CU_TRSF_SRGB = 0x10
};

/* "cupy_backends/cuda/api/_runtime_enum.pxd":1
 * cpdef enum:             # <<<<<<<<<<<<<<
 *     # need to revisit this when cython supports C++ enums (in 3.0)
 *     # https://stackoverflow.com/a/67138945
*/
enum  {

  /* "cupy_backends/cuda/api/_runtime_enum.pxd":116
 *     cudaGraphDebugDotFlagsKernelNodeAttributes = 1<<9
 *     cudaGraphDebugDotFlagsHandles = 1<<10
 *     cudaGraphDebugDotFlagsConditionalNodeParams = 1<<15             # <<<<<<<<<<<<<<
 * 
 * # This was a legacy mistake: the prefix "cuda" should have been removed
*/
  __pyx_e_13cupy_backends_4cuda_3api_13_runtime_enum_cudaMemoryTypeHost = 1,
  __pyx_e_13cupy_backends_4cuda_3api_13_runtime_enum_cudaMemoryTypeDevice = 2,
  __pyx_e_13cupy_backends_4cuda_3api_13_runtime_enum_cudaIpcMemLazyEnablePeerAccess = 1,
  __pyx_e_13cupy_backends_4cuda_3api_13_runtime_enum_cudaMemAttachGlobal = 1,
  __pyx_e_13cupy_backends_4cuda_3api_13_runtime_enum_cudaMemAttachHost = 2,
  __pyx_e_13cupy_backends_4cuda_3api_13_runtime_enum_cudaMemAttachSingle = 4,
  __pyx_e_13cupy_backends_4cuda_3api_13_runtime_enum_cudaCpuDeviceId = -1L,
  __pyx_e_13cupy_backends_4cuda_3api_13_runtime_enum_cudaInvalidDeviceId = -2L,
  __pyx_e_13cupy_backends_4cuda_3api_13_runtime_enum_cudaMemAdviseSetReadMostly = 1,
  __pyx_e_13cupy_backends_4cuda_3api_13_runtime_enum_cudaMemAdviseUnsetReadMostly = 2,
  __pyx_e_13cupy_backends_4cuda_3api_13_runtime_enum_cudaMemAdviseSetPreferredLocation = 3,
  __pyx_e_13cupy_backends_4cuda_3api_13_runtime_enum_cudaMemAdviseUnsetPreferredLocation = 4,
  __pyx_e_13cupy_backends_4cuda_3api_13_runtime_enum_cudaMemAdviseSetAccessedBy = 5,
  __pyx_e_13cupy_backends_4cuda_3api_13_runtime_enum_cudaMemAdviseUnsetAccessedBy = 6,
  __pyx_e_13cupy_backends_4cuda_3api_13_runtime_enum_CUDA_R_32F = 0,
  __pyx_e_13cupy_backends_4cuda_3api_13_runtime_enum_CUDA_R_64F = 1,
  __pyx_e_13cupy_backends_4cuda_3api_13_runtime_enum_CUDA_R_16F = 2,
  __pyx_e_13cupy_backends_4cuda_3api_13_runtime_enum_CUDA_R_8I = 3,
  __pyx_e_13cupy_backends_4cuda_3api_13_runtime_enum_CUDA_C_32F = 4,
  __pyx_e_13cupy_backends_4cuda_3api_13_runtime_enum_CUDA_C_64F = 5,
  __pyx_e_13cupy_backends_4cuda_3api_13_runtime_enum_CUDA_C_16F = 6,
  __pyx_e_13cupy_backends_4cuda_3api_13_runtime_enum_CUDA_C_8I = 7,
  __pyx_e_13cupy_backends_4cuda_3api_13_runtime_enum_CUDA_R_8U = 8,
  __pyx_e_13cupy_backends_4cuda_3api_13_runtime_enum_CUDA_C_8U = 9,
  __pyx_e_13cupy_backends_4cuda_3api_13_runtime_enum_CUDA_R_16BF = 14,
  __pyx_e_13cupy_backends_4cuda_3api_13_runtime_enum_cudaLimitStackSize = 0x00,
  __pyx_e_13cupy_backends_4cuda_3api_13_runtime_enum_cudaLimitPrintfFifoSize = 0x01,
  __pyx_e_13cupy_backends_4cuda_3api_13_runtime_enum_cudaLimitMallocHeapSize = 0x02,
  __pyx_e_13cupy_backends_4cuda_3api_13_runtime_enum_cudaLimitDevRuntimeSyncDepth = 0x03,
  __pyx_e_13cupy_backends_4cuda_3api_13_runtime_enum_cudaLimitDevRuntimePendingLaunchCount = 0x04,
  __pyx_e_13cupy_backends_4cuda_3api_13_runtime_enum_cudaLimitMaxL2FetchGranularity = 0x05,
  __pyx_e_13cupy_backends_4cuda_3api_13_runtime_enum_cudaChannelFormatKindSigned = 0,
  __pyx_e_13cupy_backends_4cuda_3api_13_runtime_enum_cudaChannelFormatKindUnsigned = 1,
  __pyx_e_13cupy_backends_4cuda_3api_13_runtime_enum_cudaChannelFormatKindFloat = 2,
  __pyx_e_13cupy_backends_4cuda_3api_13_runtime_enum_cudaChannelFormatKindNone = 3,
  __pyx_e_13cupy_backends_4cuda_3api_13_runtime_enum_cudaArrayDefault = 0,
  __pyx_e_13cupy_backends_4cuda_3api_13_runtime_enum_cudaArraySurfaceLoadStore = 2,
  __pyx_e_13cupy_backends_4cuda_3api_13_runtime_enum_cudaResourceTypeArray = 0,
  __pyx_e_13cupy_backends_4cuda_3api_13_runtime_enum_cudaResourceTypeMipmappedArray = 1,
  __pyx_e_13cupy_backends_4cuda_3api_13_runtime_enum_cudaResourceTypeLinear = 2,
  __pyx_e_13cupy_backends_4cuda_3api_13_runtime_enum_cudaResourceTypePitch2D = 3,
  __pyx_e_13cupy_backends_4cuda_3api_13_runtime_enum_cudaAddressModeWrap = 0,
  __pyx_e_13cupy_backends_4cuda_3api_13_runtime_enum_cudaAddressModeClamp = 1,
  __pyx_e_13cupy_backends_4cuda_3api_13_runtime_enum_cudaAddressModeMirror = 2,
  __pyx_e_13cupy_backends_4cuda_3api_13_runtime_enum_cudaAddressModeBorder = 3,
  __pyx_e_13cupy_backends_4cuda_3api_13_runtime_enum_cudaFilterModePoint = 0,
  __pyx_e_13cupy_backends_4cuda_3api_13_runtime_enum_cudaFilterModeLinear = 1,
  __pyx_e_13cupy_backends_4cuda_3api_13_runtime_enum_cudaReadModeElementType = 0,
  __pyx_e_13cupy_backends_4cuda_3api_13_runtime_enum_cudaReadModeNormalizedFloat = 1,
  __pyx_e_13cupy_backends_4cuda_3api_13_runtime_enum_cudaMemPoolReuseFollowEventDependencies = 0x1,
  __pyx_e_13cupy_backends_4cuda_3api_13_runtime_enum_cudaMemPoolReuseAllowOpportunistic = 0x2,
  __pyx_e_13cupy_backends_4cuda_3api_13_runtime_enum_cudaMemPoolReuseAllowInternalDependencies = 0x3,
  __pyx_e_13cupy_backends_4cuda_3api_13_runtime_enum_cudaMemPoolAttrReleaseThreshold = 0x4,
  __pyx_e_13cupy_backends_4cuda_3api_13_runtime_enum_cudaMemPoolAttrReservedMemCurrent = 0x5,
  __pyx_e_13cupy_backends_4cuda_3api_13_runtime_enum_cudaMemPoolAttrReservedMemHigh = 0x6,
  __pyx_e_13cupy_backends_4cuda_3api_13_runtime_enum_cudaMemPoolAttrUsedMemCurrent = 0x7,
  __pyx_e_13cupy_backends_4cuda_3api_13_runtime_enum_cudaMemPoolAttrUsedMemHigh = 0x8,
  __pyx_e_13cupy_backends_4cuda_3api_13_runtime_enum_cudaMemAllocationTypePinned = 0x1,
  __pyx_e_13cupy_backends_4cuda_3api_13_runtime_enum_cudaMemHandleTypeNone = 0x0,
  __pyx_e_13cupy_backends_4cuda_3api_13_runtime_enum_cudaMemHandleTypePosixFileDescriptor = 0x1,
  __pyx_e_13cupy_backends_4cuda_3api_13_runtime_enum_cudaMemLocationTypeInvalid = 0,
  __pyx_e_13cupy_backends_4cuda_3api_13_runtime_enum_cudaMemLocationTypeDevice = 1,
  __pyx_e_13cupy_backends_4cuda_3api_13_runtime_enum_cudaMemLocationTypeHost = 2,
  __pyx_e_13cupy_backends_4cuda_3api_13_runtime_enum_cudaMemLocationTypeHostNuma = 3,
  __pyx_e_13cupy_backends_4cuda_3api_13_runtime_enum_cudaMemLocationTypeHostNumaCurrent = 4,
  __pyx_e_13cupy_backends_4cuda_3api_13_runtime_enum_cudaGraphDebugDotFlagsVerbose = (1 << 0),
  __pyx_e_13cupy_backends_4cuda_3api_13_runtime_enum_cudaGraphDebugDotFlagsKernelNodeParams = (1 << 2),
  __pyx_e_13cupy_backends_4cuda_3api_13_runtime_enum_cudaGraphDebugDotFlagsMemcpyNodeParams = (1 << 3),
  __pyx_e_13cupy_backends_4cuda_3api_13_runtime_enum_cudaGraphDebugDotFlagsMemsetNodeParams = (1 << 4),
  __pyx_e_13cupy_backends_4cuda_3api_13_runtime_enum_cudaGraphDebugDotFlagsHostNodeParams = (1 << 5),
  __pyx_e_13cupy_backends_4cuda_3api_13_runtime_enum_cudaGraphDebugDotFlagsEventNodeParams = (1 << 6),
  __pyx_e_13cupy_backends_4cuda_3api_13_runtime_enum_cudaGraphDebugDotFlagsExtSemasSignalNodeParams = (1 << 7),
  __pyx_e_13cupy_backends_4cuda_3api_13_runtime_enum_cudaGraphDebugDotFlagsExtSemasWaitNodeParams = (1 << 8),
  __pyx_e_13cupy_backends_4cuda_3api_13_runtime_enum_cudaGraphDebugDotFlagsKernelNodeAttributes = (1 << 9),
  __pyx_e_13cupy_backends_4cuda_3api_13_runtime_enum_cudaGraphDebugDotFlagsHandles = (1 << 10),
  __pyx_e_13cupy_backends_4cuda_3api_13_runtime_enum_cudaGraphDebugDotFlagsConditionalNodeParams = (1 << 15)
};

/* "cupy_backends/cuda/api/_runtime_enum.pxd":124
 * IF CUPY_HIP_VERSION > 0:
 *     # separate in groups of 10 for easier counting...
 *     cpdef enum:             # <<<<<<<<<<<<<<
 *         cudaDevAttrMaxThreadsPerBlock = 0
 *         cudaDevAttrMaxBlockDimX
*/
enum  {
  __pyx_e_13cupy_backends_4cuda_3api_13_runtime_enum_cudaDevAttrMaxThreadsPerBlock = 0,
  __pyx_e_13cupy_backends_4cuda_3api_13_runtime_enum_cudaDevAttrMaxBlockDimX,
  __pyx_e_13cupy_backends_4cuda_3api_13_runtime_enum_cudaDevAttrMaxBlockDimY,
  __pyx_e_13cupy_backends_4cuda_3api_13_runtime_enum_cudaDevAttrMaxBlockDimZ,
  __pyx_e_13cupy_backends_4cuda_3api_13_runtime_enum_cudaDevAttrMaxGridDimX,
  __pyx_e_13cupy_backends_4cuda_3api_13_runtime_enum_cudaDevAttrMaxGridDimY,
  __pyx_e_13cupy_backends_4cuda_3api_13_runtime_enum_cudaDevAttrMaxGridDimZ,
  __pyx_e_13cupy_backends_4cuda_3api_13_runtime_enum_cudaDevAttrMaxSharedMemoryPerBlock,
  __pyx_e_13cupy_backends_4cuda_3api_13_runtime_enum_cudaDevAttrTotalConstantMemory,
  __pyx_e_13cupy_backends_4cuda_3api_13_runtime_enum_cudaDevAttrWarpSize,
  __pyx_e_13cupy_backends_4cuda_3api_13_runtime_enum_cudaDevAttrMaxRegistersPerBlock,
  __pyx_e_13cupy_backends_4cuda_3api_13_runtime_enum_cudaDevAttrClockRate,
  __pyx_e_13cupy_backends_4cuda_3api_13_runtime_enum_cudaDevAttrMemoryClockRate,
  __pyx_e_13cupy_backends_4cuda_3api_13_runtime_enum_cudaDevAttrGlobalMemoryBusWidth,
  __pyx_e_13cupy_backends_4cuda_3api_13_runtime_enum_cudaDevAttrMultiProcessorCount,
  __pyx_e_13cupy_backends_4cuda_3api_13_runtime_enum_cudaDevAttrComputeMode,
  __pyx_e_13cupy_backends_4cuda_3api_13_runtime_enum_cudaDevAttrL2CacheSize,
  __pyx_e_13cupy_backends_4cuda_3api_13_runtime_enum_cudaDevAttrMaxThreadsPerMultiProcessor,
  __pyx_e_13cupy_backends_4cuda_3api_13_runtime_enum_cudaDevAttrConcurrentKernels = 20,
  __pyx_e_13cupy_backends_4cuda_3api_13_runtime_enum_cudaDevAttrPciBusId,
  __pyx_e_13cupy_backends_4cuda_3api_13_runtime_enum_cudaDevAttrPciDeviceId,
  __pyx_e_13cupy_backends_4cuda_3api_13_runtime_enum_cudaDevAttrMaxSharedMemoryPerMultiprocessor,
  __pyx_e_13cupy_backends_4cuda_3api_13_runtime_enum_cudaDevAttrIsMultiGpuBoard,
  __pyx_e_13cupy_backends_4cuda_3api_13_runtime_enum_cudaDevAttrIntegrated,
  __pyx_e_13cupy_backends_4cuda_3api_13_runtime_enum_cudaDevAttrCooperativeLaunch,
  __pyx_e_13cupy_backends_4cuda_3api_13_runtime_enum_cudaDevAttrCooperativeMultiDeviceLaunch,
  __pyx_e_13cupy_backends_4cuda_3api_13_runtime_enum_cudaDevAttrMaxTexture1DWidth,
  __pyx_e_13cupy_backends_4cuda_3api_13_runtime_enum_cudaDevAttrMaxTexture2DWidth,
  __pyx_e_13cupy_backends_4cuda_3api_13_runtime_enum_cudaDevAttrMaxTexture2DHeight,
  __pyx_e_13cupy_backends_4cuda_3api_13_runtime_enum_cudaDevAttrMaxTexture3DWidth,
  __pyx_e_13cupy_backends_4cuda_3api_13_runtime_enum_cudaDevAttrMaxTexture3DHeight,
  __pyx_e_13cupy_backends_4cuda_3api_13_runtime_enum_cudaDevAttrMaxTexture3DDepth,
  __pyx_e_13cupy_backends_4cuda_3api_13_runtime_enum_cudaDevAttrMaxPitch = 36,
  __pyx_e_13cupy_backends_4cuda_3api_13_runtime_enum_cudaDevAttrTextureAlignment,
  __pyx_e_13cupy_backends_4cuda_3api_13_runtime_enum_cudaDevAttrTexturePitchAlignment,
  __pyx_e_13cupy_backends_4cuda_3api_13_runtime_enum_cudaDevAttrKernelExecTimeout,
  __pyx_e_13cupy_backends_4cuda_3api_13_runtime_enum_cudaDevAttrCanMapHostMemory,
  __pyx_e_13cupy_backends_4cuda_3api_13_runtime_enum_cudaDevAttrEccEnabled,
  __pyx_e_13cupy_backends_4cuda_3api_13_runtime_enum_cudaDevAttrMemoryPoolsSupported = 0
};

/* "cupy_backends/cuda/api/_runtime_enum.pxd":240
 *         # cudaDevAttrHostRegisterSupported
 *     IF CUPY_HIP_VERSION >= 310:
 *         cpdef enum:             # <<<<<<<<<<<<<<
 *             # hipDeviceAttributeAsicRevision  # does not exist in CUDA
 *             cudaDevAttrManagedMemory = 47
*/
enum  {
  __pyx_e_13cupy_backends_4cuda_3api_13_runtime_enum_cudaDevAttrManagedMemory = 47,
  __pyx_e_13cupy_backends_4cuda_3api_13_runtime_enum_cudaDevAttrDirectManagedMemAccessFromHost,
  __pyx_e_13cupy_backends_4cuda_3api_13_runtime_enum_cudaDevAttrConcurrentManagedAccess,
  __pyx_e_13cupy_backends_4cuda_3api_13_runtime_enum_cudaDevAttrPageableMemoryAccess,
  __pyx_e_13cupy_backends_4cuda_3api_13_runtime_enum_cudaDevAttrPageableMemoryAccessUsesHostPageTables
};
struct __pyx_opt_args_13cupy_backends_4cuda_3api_7runtime_mallocManaged;
struct __pyx_opt_args_13cupy_backends_4cuda_3api_7runtime_malloc3DArray;
struct __pyx_opt_args_13cupy_backends_4cuda_3api_7runtime_mallocArray;
struct __pyx_opt_args_13cupy_backends_4cuda_3api_7runtime_streamAddCallback;
struct __pyx_opt_args_13cupy_backends_4cuda_3api_7runtime_streamWaitEvent;
struct __pyx_opt_args_13cupy_backends_4cuda_3api_7runtime_streamBeginCapture;

/* "runtime.pxd":99
 * 
 * # For backward compatibility, keep APIs not prefixed with "cuda".
 * cpdef enum:             # <<<<<<<<<<<<<<
 *     memcpyHostToHost = 0
 *     memcpyHostToDevice = 1
*/
enum  {
  __pyx_e_13cupy_backends_4cuda_3api_7runtime_memcpyHostToHost = 0,
  __pyx_e_13cupy_backends_4cuda_3api_7runtime_memcpyHostToDevice = 1,
  __pyx_e_13cupy_backends_4cuda_3api_7runtime_memcpyDeviceToHost = 2,
  __pyx_e_13cupy_backends_4cuda_3api_7runtime_memcpyDeviceToDevice = 3,
  __pyx_e_13cupy_backends_4cuda_3api_7runtime_memcpyDefault = 4,
  __pyx_e_13cupy_backends_4cuda_3api_7runtime_hostAllocDefault = 0,
  __pyx_e_13cupy_backends_4cuda_3api_7runtime_hostAllocPortable = 1,
  __pyx_e_13cupy_backends_4cuda_3api_7runtime_hostAllocMapped = 2,
  __pyx_e_13cupy_backends_4cuda_3api_7runtime_hostAllocWriteCombined = 4,
  __pyx_e_13cupy_backends_4cuda_3api_7runtime_streamDefault = 0,
  __pyx_e_13cupy_backends_4cuda_3api_7runtime_streamNonBlocking = 1,
  __pyx_e_13cupy_backends_4cuda_3api_7runtime_streamLegacy = 1,
  __pyx_e_13cupy_backends_4cuda_3api_7runtime_streamPerThread = 2,
  __pyx_e_13cupy_backends_4cuda_3api_7runtime_eventDefault = 0,
  __pyx_e_13cupy_backends_4cuda_3api_7runtime_eventBlockingSync = 1,
  __pyx_e_13cupy_backends_4cuda_3api_7runtime_eventDisableTiming = 2,
  __pyx_e_13cupy_backends_4cuda_3api_7runtime_eventInterprocess = 4,
  __pyx_e_13cupy_backends_4cuda_3api_7runtime_streamCaptureModeGlobal = 0,
  __pyx_e_13cupy_backends_4cuda_3api_7runtime_streamCaptureModeThreadLocal = 1,
  __pyx_e_13cupy_backends_4cuda_3api_7runtime_streamCaptureModeRelaxed = 2,
  __pyx_e_13cupy_backends_4cuda_3api_7runtime_streamCaptureStatusNone = 0,
  __pyx_e_13cupy_backends_4cuda_3api_7runtime_streamCaptureStatusActive = 1,
  __pyx_e_13cupy_backends_4cuda_3api_7runtime_streamCaptureStatusInvalidated = 2,
  __pyx_e_13cupy_backends_4cuda_3api_7runtime_memoryTypeUnregistered = 0,
  __pyx_e_13cupy_backends_4cuda_3api_7runtime_memoryTypeHost = 1,
  __pyx_e_13cupy_backends_4cuda_3api_7runtime_memoryTypeDevice = 2,
  __pyx_e_13cupy_backends_4cuda_3api_7runtime_memoryTypeManaged = 3
};

/* "runtime.pxd":205
 * 
 * cpdef intptr_t malloc(size_t size) except? 0
 * cpdef intptr_t mallocManaged(size_t size, unsigned int flags=*) except? 0             # <<<<<<<<<<<<<<
 * cpdef intptr_t malloc3DArray(intptr_t desc, size_t width, size_t height,
 *                              size_t depth, unsigned int flags=*) except? 0
*/
struct __pyx_opt_args_13cupy_backends_4cuda_3api_7runtime_mallocManaged {
  int __pyx_n;
  unsigned int flags;
};

/* "runtime.pxd":206
 * cpdef intptr_t malloc(size_t size) except? 0
 * cpdef intptr_t mallocManaged(size_t size, unsigned int flags=*) except? 0
 * cpdef intptr_t malloc3DArray(intptr_t desc, size_t width, size_t height,             # <<<<<<<<<<<<<<
 *                              size_t depth, unsigned int flags=*) except? 0
 * cpdef intptr_t mallocArray(intptr_t desc, size_t width, size_t height,
*/
struct __pyx_opt_args_13cupy_backends_4cuda_3api_7runtime_malloc3DArray {
  int __pyx_n;
  unsigned int flags;
};

/* "runtime.pxd":208
 * cpdef intptr_t malloc3DArray(intptr_t desc, size_t width, size_t height,
 *                              size_t depth, unsigned int flags=*) except? 0
 * cpdef intptr_t mallocArray(intptr_t desc, size_t width, size_t height,             # <<<<<<<<<<<<<<
 *                            unsigned int flags=*) except? 0
 * cpdef intptr_t mallocAsync(size_t size, intptr_t stream) except? 0
*/
struct __pyx_opt_args_13cupy_backends_4cuda_3api_7runtime_mallocArray {
  int __pyx_n;
  unsigned int flags;
};

/* "runtime.pxd":275
 * cpdef streamDestroy(intptr_t stream)
 * cpdef streamSynchronize(intptr_t stream)
 * cpdef streamAddCallback(intptr_t stream, callback, intptr_t arg,             # <<<<<<<<<<<<<<
 *                         unsigned int flags=*)
 * cpdef launchHostFunc(intptr_t stream, callback, intptr_t arg)
*/
struct __pyx_opt_args_13cupy_backends_4cuda_3api_7runtime_streamAddCallback {
  int __pyx_n;
  unsigned int flags;
};

/* "runtime.pxd":280
 * cpdef _launchHostFuncUnmanaged(intptr_t stream, tuple func_arg)
 * cpdef streamQuery(intptr_t stream)
 * cpdef streamWaitEvent(intptr_t stream, intptr_t event, unsigned int flags=*)             # <<<<<<<<<<<<<<
 * cpdef streamBeginCapture(intptr_t stream, int mode=*)
 * cpdef intptr_t streamEndCapture(intptr_t stream) except? 0
*/
struct __pyx_opt_args_13cupy_backends_4cuda_3api_7runtime_streamWaitEvent {
  int __pyx_n;
  unsigned int flags;
};

/* "runtime.pxd":281
 * cpdef streamQuery(intptr_t stream)
 * cpdef streamWaitEvent(intptr_t stream, intptr_t event, unsigned int flags=*)
 * cpdef streamBeginCapture(intptr_t stream, int mode=*)             # <<<<<<<<<<<<<<
 * cpdef intptr_t streamEndCapture(intptr_t stream) except? 0
 * cpdef bint streamIsCapturing(intptr_t stream) except*
*/
struct __pyx_opt_args_13cupy_backends_4cuda_3api_7runtime_streamBeginCapture {
  int __pyx_n;
  int mode;
};
struct __pyx_opt_args_4cupy_4cuda_6memory_13MemoryPointer_copy_from_device_async;
struct __pyx_opt_args_4cupy_4cuda_6memory_13MemoryPointer_copy_from_host_async;
struct __pyx_opt_args_4cupy_4cuda_6memory_13MemoryPointer_copy_from_async;
struct __pyx_opt_args_4cupy_4cuda_6memory_13MemoryPointer_copy_to_host_async;
struct __pyx_opt_args_4cupy_4cuda_6memory_13MemoryPointer_memset_async;
struct __pyx_opt_args_4cupy_4cuda_6memory_set_allocator;
struct __pyx_opt_args_4cupy_4cuda_6memory_10MemoryPool_free_all_blocks;
struct __pyx_opt_args_4cupy_4cuda_6memory_10MemoryPool_set_limit;
struct __pyx_opt_args_4cupy_4cuda_6memory_6_Arena_insert_chunk;
struct __pyx_opt_args_4cupy_4cuda_6memory__parse_limit_string;
struct __pyx_opt_args_4cupy_4cuda_6memory_22SingleDeviceMemoryPool_free_all_blocks;
struct __pyx_opt_args_4cupy_4cuda_6memory_22SingleDeviceMemoryPool_set_limit;
struct __pyx_opt_args_4cupy_4cuda_6memory_15MemoryAsyncPool_free_all_blocks;
struct __pyx_opt_args_4cupy_4cuda_6memory_15MemoryAsyncPool_set_limit;

/* "cupy/cuda/memory.pxd":39
 * 
 *     cpdef copy_from_device(self, MemoryPointer src, size_t size)
 *     cpdef copy_from_device_async(self, MemoryPointer src, size_t size,             # <<<<<<<<<<<<<<
 *                                  stream=?)
 *     cpdef copy_from_host(self, mem, size_t size)
*/
struct __pyx_opt_args_4cupy_4cuda_6memory_13MemoryPointer_copy_from_device_async {
  int __pyx_n;
  PyObject *stream;
};

/* "cupy/cuda/memory.pxd":42
 *                                  stream=?)
 *     cpdef copy_from_host(self, mem, size_t size)
 *     cpdef copy_from_host_async(self, mem, size_t size, stream=?)             # <<<<<<<<<<<<<<
 *     cpdef copy_from(self, mem, size_t size)
 *     cpdef copy_from_async(self, mem, size_t size, stream=?)
*/
struct __pyx_opt_args_4cupy_4cuda_6memory_13MemoryPointer_copy_from_host_async {
  int __pyx_n;
  PyObject *stream;
};

/* "cupy/cuda/memory.pxd":44
 *     cpdef copy_from_host_async(self, mem, size_t size, stream=?)
 *     cpdef copy_from(self, mem, size_t size)
 *     cpdef copy_from_async(self, mem, size_t size, stream=?)             # <<<<<<<<<<<<<<
 *     cpdef copy_to_host(self, mem, size_t size)
 *     cpdef copy_to_host_async(self, mem, size_t size, stream=?)
*/
struct __pyx_opt_args_4cupy_4cuda_6memory_13MemoryPointer_copy_from_async {
  int __pyx_n;
  PyObject *stream;
};

/* "cupy/cuda/memory.pxd":46
 *     cpdef copy_from_async(self, mem, size_t size, stream=?)
 *     cpdef copy_to_host(self, mem, size_t size)
 *     cpdef copy_to_host_async(self, mem, size_t size, stream=?)             # <<<<<<<<<<<<<<
 *     cpdef memset(self, int value, size_t size)
 *     cpdef memset_async(self, int value, size_t size, stream=?)
*/
struct __pyx_opt_args_4cupy_4cuda_6memory_13MemoryPointer_copy_to_host_async {
  int __pyx_n;
  PyObject *stream;
};

/* "cupy/cuda/memory.pxd":48
 *     cpdef copy_to_host_async(self, mem, size_t size, stream=?)
 *     cpdef memset(self, int value, size_t size)
 *     cpdef memset_async(self, int value, size_t size, stream=?)             # <<<<<<<<<<<<<<
 * 
 * 
*/
struct __pyx_opt_args_4cupy_4cuda_6memory_13MemoryPointer_memset_async {
  int __pyx_n;
  PyObject *stream;
};

/* "cupy/cuda/memory.pxd":54
 * 
 * 
 * cpdef set_allocator(allocator=*)             # <<<<<<<<<<<<<<
 * cpdef get_allocator()
 * 
*/
struct __pyx_opt_args_4cupy_4cuda_6memory_set_allocator {
  int __pyx_n;
  PyObject *allocator;
};

/* "cupy/cuda/memory.pxd":65
 * 
 *     cpdef MemoryPointer malloc(self, size_t size)
 *     cpdef free_all_blocks(self, stream=?)             # <<<<<<<<<<<<<<
 *     cpdef free_all_free(self)
 *     # they call device_pool so they may raise error.
*/
struct __pyx_opt_args_4cupy_4cuda_6memory_10MemoryPool_free_all_blocks {
  int __pyx_n;
  PyObject *stream;
};

/* "cupy/cuda/memory.pxd":72
 *     cpdef size_t free_bytes(self)
 *     cpdef size_t total_bytes(self)
 *     cpdef set_limit(self, size=?, fraction=?)             # <<<<<<<<<<<<<<
 *     cpdef size_t get_limit(self)
 * 
*/
struct __pyx_opt_args_4cupy_4cuda_6memory_10MemoryPool_set_limit {
  int __pyx_n;
  PyObject *size;
  PyObject *fraction;
};

/* "cupy/cuda/memory.pyx":1051
 * # The std::set contains the size, _Chunk pair (as uintptr_t)
 * # The uintptr_t means we can compare well defined (including to 0)
 * ctypedef std_pair[size_t, uintptr_t] index_type             # <<<<<<<<<<<<<<
 * 
 * 
*/
typedef std::pair<size_t,uintptr_t>  __pyx_t_4cupy_4cuda_6memory_index_type;

/* "cupy/cuda/memory.pyx":1107
 *         return chunk
 * 
 *     cdef insert_chunk(self, _Chunk chunk, bint merge=True):             # <<<<<<<<<<<<<<
 *         if merge:
 *             chunk = self.try_merge_chunk(chunk)
*/
struct __pyx_opt_args_4cupy_4cuda_6memory_6_Arena_insert_chunk {
  int __pyx_n;
  int merge;
};

/* "cupy/cuda/memory.pyx":1204
 * # cpdef because uint-tested
 * # module-level function can be inlined
 * cpdef inline dict _parse_limit_string(limit=None):             # <<<<<<<<<<<<<<
 *     if limit is None:
 *         limit = os.environ.get('CUPY_GPU_MEMORY_LIMIT')
*/
struct __pyx_opt_args_4cupy_4cuda_6memory__parse_limit_string {
  int __pyx_n;
  PyObject *limit;
};

/* "cupy/cuda/memory.pyx":1400
 *             chunk.arena.add_pending_free_atomic(chunk)
 * 
 *     cpdef free_all_blocks(self, stream=None):             # <<<<<<<<<<<<<<
 *         """Free all **non-split** blocks for one or all arenas.
 *         """
*/
struct __pyx_opt_args_4cupy_4cuda_6memory_22SingleDeviceMemoryPool_free_all_blocks {
  int __pyx_n;
  PyObject *stream;
};

/* "cupy/cuda/memory.pyx":1492
 *         return limit_ok
 * 
 *     cpdef set_limit(self, size=None, fraction=None):             # <<<<<<<<<<<<<<
 *         if size is None:
 *             if fraction is None:
*/
struct __pyx_opt_args_4cupy_4cuda_6memory_22SingleDeviceMemoryPool_set_limit {
  int __pyx_n;
  PyObject *size;
  PyObject *fraction;
};

/* "cupy/cuda/memory.pyx":1903
 *         return mem
 * 
 *     cpdef free_all_blocks(self, stream=None):             # <<<<<<<<<<<<<<
 *         """Releases free memory.
 * 
*/
struct __pyx_opt_args_4cupy_4cuda_6memory_15MemoryAsyncPool_free_all_blocks {
  int __pyx_n;
  PyObject *stream;
};

/* "cupy/cuda/memory.pyx":1966
 *             pool, runtime.cudaMemPoolAttrReservedMemCurrent)
 * 
 *     cpdef set_limit(self, size=None, fraction=None):             # <<<<<<<<<<<<<<
 *         """Sets the upper limit of memory allocation of the current device.
 * 
*/
struct __pyx_opt_args_4cupy_4cuda_6memory_15MemoryAsyncPool_set_limit {
  int __pyx_n;
  PyObject *size;
  PyObject *fraction;
};

/* "cupy/cuda/memory.pyx":2043
 * 
 * 
 * ctypedef void*(*malloc_func_type)(void*, size_t, int) except +             # <<<<<<<<<<<<<<
 * ctypedef void(*free_func_type)(void*, void*, int) noexcept
 * 
*/
typedef void *(*__pyx_t_4cupy_4cuda_6memory_malloc_func_type)(void *, size_t, int);

/* "cupy/cuda/memory.pyx":2044
 * 
 * ctypedef void*(*malloc_func_type)(void*, size_t, int) except +
 * ctypedef void(*free_func_type)(void*, void*, int) noexcept             # <<<<<<<<<<<<<<
 * 
 * 
*/
typedef void (*__pyx_t_4cupy_4cuda_6memory_free_func_type)(void *, void *, int);

/* "device.pxd":13
 * cdef bint _enable_peer_access(int device, int peer) except -1
 * 
 * cdef class Handle:             # <<<<<<<<<<<<<<
 *     cdef:
 *         public size_t handle
*/
struct __pyx_obj_4cupy_4cuda_6device_Handle {
  PyObject_HEAD
  size_t handle;
  PyObject *_destroy_func;
};


/* "device.pxd":18
 *         object _destroy_func
 * 
 * cdef class Device:             # <<<<<<<<<<<<<<
 *     cdef:
 *         public int id
*/
struct __pyx_obj_4cupy_4cuda_6device_Device {
  PyObject_HEAD
  struct __pyx_vtabstruct_4cupy_4cuda_6device_Device *__pyx_vtab;
  int id;
  PyObject *_device_stack;
};


/* "runtime.pxd":8
 * ###############################################################################
 * 
 * cdef class PointerAttributes:             # <<<<<<<<<<<<<<
 *     cdef:
 *         readonly int device
*/
struct __pyx_obj_13cupy_backends_4cuda_3api_7runtime_PointerAttributes {
  PyObject_HEAD
  int device;
  intptr_t devicePointer;
  intptr_t hostPointer;
  int type;
};


/* "runtime.pxd":15
 *         readonly int type
 * 
 * cdef class MemPoolProps:             # <<<<<<<<<<<<<<
 *     # flatten the struct & list meaningful members
 *     cdef:
*/
struct __pyx_obj_13cupy_backends_4cuda_3api_7runtime_MemPoolProps {
  PyObject_HEAD
  int allocType;
  int handleType;
  int locationType;
  int devId;
};


/* "cupy/cuda/memory.pxd":10
 * 
 * 
 * @cython.no_gc             # <<<<<<<<<<<<<<
 * cdef class BaseMemory:
 * 
*/
struct __pyx_obj_4cupy_4cuda_6memory_BaseMemory {
  PyObject_HEAD
  intptr_t ptr;
  size_t size;
  int device_id;
};


/* "cupy/cuda/memory.pxd":19
 * 
 * 
 * @cython.no_gc             # <<<<<<<<<<<<<<
 * cdef class SystemMemory(BaseMemory):
 * 
*/
struct __pyx_obj_4cupy_4cuda_6memory_SystemMemory {
  struct __pyx_obj_4cupy_4cuda_6memory_BaseMemory __pyx_base;
  struct __pyx_vtabstruct_4cupy_4cuda_6memory_SystemMemory *__pyx_vtab;
  PyObject *_owner;
};


/* "cupy/cuda/memory.pxd":29
 * 
 * 
 * cdef class MemoryPointer:             # <<<<<<<<<<<<<<
 * 
 *     cdef:
*/
struct __pyx_obj_4cupy_4cuda_6memory_MemoryPointer {
  PyObject_HEAD
  struct __pyx_vtabstruct_4cupy_4cuda_6memory_MemoryPointer *__pyx_vtab;
  intptr_t ptr;
  int device_id;
  struct __pyx_obj_4cupy_4cuda_6memory_BaseMemory *mem;
};


/* "cupy/cuda/memory.pxd":58
 * 
 * 
 * cdef class MemoryPool:             # <<<<<<<<<<<<<<
 * 
 *     cdef:
*/
struct __pyx_obj_4cupy_4cuda_6memory_MemoryPool {
  PyObject_HEAD
  struct __pyx_vtabstruct_4cupy_4cuda_6memory_MemoryPool *__pyx_vtab;
  PyObject *_pools;
  PyObject *_allocator;
};


/* "cupy/cuda/memory.pxd":89
 * 
 * 
 * @cython.no_gc             # <<<<<<<<<<<<<<
 * cdef class CFunctionAllocatorMemory(BaseMemory):
 * 
*/
struct __pyx_obj_4cupy_4cuda_6memory_CFunctionAllocatorMemory {
  struct __pyx_obj_4cupy_4cuda_6memory_BaseMemory __pyx_base;
  intptr_t _param;
  intptr_t _free_func;
};


/* "cupy/cuda/memory.pxd":97
 * 
 * 
 * cdef class CFunctionAllocator:             # <<<<<<<<<<<<<<
 * 
 *     cdef:
*/
struct __pyx_obj_4cupy_4cuda_6memory_CFunctionAllocator {
  PyObject_HEAD
  struct __pyx_vtabstruct_4cupy_4cuda_6memory_CFunctionAllocator *__pyx_vtab;
  intptr_t _param;
  intptr_t _malloc_func;
  intptr_t _free_func;
  PyObject *_owner;
};


/* "cupy/cuda/memory.pxd":108
 * 
 * 
 * cdef class PythonFunctionAllocatorMemory(BaseMemory):             # <<<<<<<<<<<<<<
 * 
 *     cdef:
*/
struct __pyx_obj_4cupy_4cuda_6memory_PythonFunctionAllocatorMemory {
  struct __pyx_obj_4cupy_4cuda_6memory_BaseMemory __pyx_base;
  PyObject *_free_func;
};


/* "cupy/cuda/memory.pxd":114
 * 
 * 
 * cdef class PythonFunctionAllocator:             # <<<<<<<<<<<<<<
 * 
 *     cdef:
*/
struct __pyx_obj_4cupy_4cuda_6memory_PythonFunctionAllocator {
  PyObject_HEAD
  struct __pyx_vtabstruct_4cupy_4cuda_6memory_PythonFunctionAllocator *__pyx_vtab;
  PyObject *_malloc_func;
  PyObject *_free_func;
};


/* "cupy/cuda/memory.pyx":104
 * 
 * 
 * @cython.no_gc             # <<<<<<<<<<<<<<
 * cdef class Memory(BaseMemory):
 *     """Memory allocation on a CUDA device.
*/
struct __pyx_obj_4cupy_4cuda_6memory_Memory {
  struct __pyx_obj_4cupy_4cuda_6memory_BaseMemory __pyx_base;
};


/* "cupy/cuda/memory.pyx":145
 * 
 * 
 * @cython.no_gc             # <<<<<<<<<<<<<<
 * cdef class MemoryAsync(BaseMemory):
 *     """Asynchronous memory allocation on a CUDA device.
*/
struct __pyx_obj_4cupy_4cuda_6memory_MemoryAsync {
  struct __pyx_obj_4cupy_4cuda_6memory_BaseMemory __pyx_base;
  PyObject *stream_ref;
};


/* "cupy/cuda/memory.pyx":181
 * 
 * 
 * cdef class UnownedMemory(BaseMemory):             # <<<<<<<<<<<<<<
 *     """CUDA memory that is not owned by CuPy.
 * 
*/
struct __pyx_obj_4cupy_4cuda_6memory_UnownedMemory {
  struct __pyx_obj_4cupy_4cuda_6memory_BaseMemory __pyx_base;
  PyObject *_owner;
};


/* "cupy/cuda/memory.pyx":222
 * 
 * 
 * @cython.no_gc             # <<<<<<<<<<<<<<
 * cdef class ManagedMemory(BaseMemory):
 *     """Managed memory (Unified memory) allocation on a CUDA device.
*/
struct __pyx_obj_4cupy_4cuda_6memory_ManagedMemory {
  struct __pyx_obj_4cupy_4cuda_6memory_BaseMemory __pyx_base;
};


/* "cupy/cuda/memory.pyx":357
 * 
 * 
 * @cython.final             # <<<<<<<<<<<<<<
 * @cython.no_gc  # reference cycle would be a bug
 * cdef class _Chunk:
*/
struct __pyx_obj_4cupy_4cuda_6memory__Chunk {
  PyObject_HEAD
  struct __pyx_vtabstruct_4cupy_4cuda_6memory__Chunk *__pyx_vtab;
  struct __pyx_obj_4cupy_4cuda_6memory_BaseMemory *mem;
  ptrdiff_t offset;
  size_t size;
  struct __pyx_obj_4cupy_4cuda_6memory__Arena *arena;
  struct __pyx_obj_4cupy_4cuda_6memory__Chunk *prev;
  struct __pyx_obj_4cupy_4cuda_6memory__Chunk *next;
};


/* "cupy/cuda/memory.pyx":943
 * 
 * 
 * @cython.final             # <<<<<<<<<<<<<<
 * @cython.no_gc
 * cdef class PooledMemory(BaseMemory):
*/
struct __pyx_obj_4cupy_4cuda_6memory_PooledMemory {
  struct __pyx_obj_4cupy_4cuda_6memory_BaseMemory __pyx_base;
  struct __pyx_vtabstruct_4cupy_4cuda_6memory_PooledMemory *__pyx_vtab;
  PyObject *pool;
  PyObject *identity;
  struct __pyx_obj_4cupy_4cuda_6memory__Chunk *chunk;
  PyObject *__dict__;
};


/* "cupy/cuda/memory.pyx":1054
 * 
 * 
 * @cython.final             # <<<<<<<<<<<<<<
 * cdef class _Arena:
 *     # Arena class managing all free chunks belonging to a single stream ident.
*/
struct __pyx_obj_4cupy_4cuda_6memory__Arena {
  PyObject_HEAD
  struct __pyx_vtabstruct_4cupy_4cuda_6memory__Arena *__pyx_vtab;
  PyObject *_pending_free;
  std::set<__pyx_t_4cupy_4cuda_6memory_index_type>  index;
  PyObject *__weakref__;
};


/* "cupy/cuda/memory.pyx":1217
 * 
 * 
 * @cython.final             # <<<<<<<<<<<<<<
 * cdef class SingleDeviceMemoryPool:
 *     """Memory pool implementation for single device.
*/
struct __pyx_obj_4cupy_4cuda_6memory_SingleDeviceMemoryPool {
  PyObject_HEAD
  struct __pyx_vtabstruct_4cupy_4cuda_6memory_SingleDeviceMemoryPool *__pyx_vtab;
  PyObject *_allocator;
  PyObject *_arenas;
  std::mutex _arena_mutex;
  std::atomic<size_t>  _total_bytes;
  std::atomic<size_t>  _in_use_bytes;
  std::atomic<size_t>  _total_bytes_limit;
  PyObject *__weakref__;
  PyObject *_weakref;
  int _device_id;
};


/* "cupy/cuda/memory.pyx":1740
 * 
 * 
 * cdef class MemoryAsyncPool:             # <<<<<<<<<<<<<<
 *     """CUDA memory pool for all GPU devices on the host.
 * 
*/
struct __pyx_obj_4cupy_4cuda_6memory_MemoryAsyncPool {
  PyObject_HEAD
  struct __pyx_vtabstruct_4cupy_4cuda_6memory_MemoryAsyncPool *__pyx_vtab;
  PyObject *_pools;
  int memoryAsyncHasStat;
};


/* "cupy/cuda/memory.pyx":1599
 *         self._allocator = allocator
 * 
 *     cdef _ensure_pools_and_return_device_pool(self):             # <<<<<<<<<<<<<<
 *         # assume we get to create the pools (we may not be the only one)
 *         n_gpu = runtime.getDeviceCount()
*/
struct __pyx_obj_4cupy_4cuda_6memory___pyx_scope_struct___ensure_pools_and_return_device_pool {
  PyObject_HEAD
  struct __pyx_obj_4cupy_4cuda_6memory_MemoryPool *__pyx_v_self;
};


/* "cupy/cuda/memory.pyx":1603
 *         n_gpu = runtime.getDeviceCount()
 *         pools = tuple(
 *             SingleDeviceMemoryPool(self._allocator) for i in range(n_gpu))             # <<<<<<<<<<<<<<
 * 
 *         # If no-one beat us to it, set _pools. Note that the above seems to
*/
struct __pyx_obj_4cupy_4cuda_6memory___pyx_scope_struct_1_genexpr {
  PyObject_HEAD
  struct __pyx_obj_4cupy_4cuda_6memory___pyx_scope_struct___ensure_pools_and_return_device_pool *__pyx_outer_scope;
  int __pyx_genexpr_arg_0;
  int __pyx_v_i;
  int __pyx_t_0;
  int __pyx_t_1;
  int __pyx_t_2;
};



/* "device.pxd":18
 *         object _destroy_func
 * 
 * cdef class Device:             # <<<<<<<<<<<<<<
 *     cdef:
 *         public int id
*/

struct __pyx_vtabstruct_4cupy_4cuda_6device_Device {
  PyObject *(*use)(struct __pyx_obj_4cupy_4cuda_6device_Device *, int __pyx_skip_dispatch);
  PyObject *(*synchronize)(struct __pyx_obj_4cupy_4cuda_6device_Device *, int __pyx_skip_dispatch);
};
static struct __pyx_vtabstruct_4cupy_4cuda_6device_Device *__pyx_vtabptr_4cupy_4cuda_6device_Device;


/* "cupy/cuda/memory.pyx":273
 * 
 * 
 * @cython.no_gc             # <<<<<<<<<<<<<<
 * cdef class SystemMemory(BaseMemory):
 *     """Memory allocation on an HMM/ATS enabled system.
*/

struct __pyx_vtabstruct_4cupy_4cuda_6memory_SystemMemory {
  PyObject *(*from_external)(intptr_t, size_t, PyObject *);
};
static struct __pyx_vtabstruct_4cupy_4cuda_6memory_SystemMemory *__pyx_vtabptr_4cupy_4cuda_6memory_SystemMemory;


/* "cupy/cuda/memory.pyx":458
 * 
 * 
 * cdef class MemoryPointer:             # <<<<<<<<<<<<<<
 *     """Pointer to a point on a device memory.
 * 
*/

struct __pyx_vtabstruct_4cupy_4cuda_6memory_MemoryPointer {
  PyObject *(*_init)(struct __pyx_obj_4cupy_4cuda_6memory_MemoryPointer *, struct __pyx_obj_4cupy_4cuda_6memory_BaseMemory *, ptrdiff_t);
  PyObject *(*copy_from_device)(struct __pyx_obj_4cupy_4cuda_6memory_MemoryPointer *, struct __pyx_obj_4cupy_4cuda_6memory_MemoryPointer *, size_t, int __pyx_skip_dispatch);
  PyObject *(*copy_from_device_async)(struct __pyx_obj_4cupy_4cuda_6memory_MemoryPointer *, struct __pyx_obj_4cupy_4cuda_6memory_MemoryPointer *, size_t, int __pyx_skip_dispatch, struct __pyx_opt_args_4cupy_4cuda_6memory_13MemoryPointer_copy_from_device_async *__pyx_optional_args);
  PyObject *(*copy_from_host)(struct __pyx_obj_4cupy_4cuda_6memory_MemoryPointer *, PyObject *, size_t, int __pyx_skip_dispatch);
  PyObject *(*copy_from_host_async)(struct __pyx_obj_4cupy_4cuda_6memory_MemoryPointer *, PyObject *, size_t, int __pyx_skip_dispatch, struct __pyx_opt_args_4cupy_4cuda_6memory_13MemoryPointer_copy_from_host_async *__pyx_optional_args);
  PyObject *(*copy_from)(struct __pyx_obj_4cupy_4cuda_6memory_MemoryPointer *, PyObject *, size_t, int __pyx_skip_dispatch);
  PyObject *(*copy_from_async)(struct __pyx_obj_4cupy_4cuda_6memory_MemoryPointer *, PyObject *, size_t, int __pyx_skip_dispatch, struct __pyx_opt_args_4cupy_4cuda_6memory_13MemoryPointer_copy_from_async *__pyx_optional_args);
  PyObject *(*copy_to_host)(struct __pyx_obj_4cupy_4cuda_6memory_MemoryPointer *, PyObject *, size_t, int __pyx_skip_dispatch);
  PyObject *(*copy_to_host_async)(struct __pyx_obj_4cupy_4cuda_6memory_MemoryPointer *, PyObject *, size_t, int __pyx_skip_dispatch, struct __pyx_opt_args_4cupy_4cuda_6memory_13MemoryPointer_copy_to_host_async *__pyx_optional_args);
  PyObject *(*memset)(struct __pyx_obj_4cupy_4cuda_6memory_MemoryPointer *, int, size_t, int __pyx_skip_dispatch);
  PyObject *(*memset_async)(struct __pyx_obj_4cupy_4cuda_6memory_MemoryPointer *, int, size_t, int __pyx_skip_dispatch, struct __pyx_opt_args_4cupy_4cuda_6memory_13MemoryPointer_memset_async *__pyx_optional_args);
};
static struct __pyx_vtabstruct_4cupy_4cuda_6memory_MemoryPointer *__pyx_vtabptr_4cupy_4cuda_6memory_MemoryPointer;


/* "cupy/cuda/memory.pyx":1566
 * 
 * 
 * cdef class MemoryPool:             # <<<<<<<<<<<<<<
 * 
 *     """Memory pool for all GPU devices on the host.
*/

struct __pyx_vtabstruct_4cupy_4cuda_6memory_MemoryPool {
  struct __pyx_obj_4cupy_4cuda_6memory_MemoryPointer *(*malloc)(struct __pyx_obj_4cupy_4cuda_6memory_MemoryPool *, size_t, int __pyx_skip_dispatch);
  PyObject *(*free_all_blocks)(struct __pyx_obj_4cupy_4cuda_6memory_MemoryPool *, int __pyx_skip_dispatch, struct __pyx_opt_args_4cupy_4cuda_6memory_10MemoryPool_free_all_blocks *__pyx_optional_args);
  PyObject *(*free_all_free)(struct __pyx_obj_4cupy_4cuda_6memory_MemoryPool *, int __pyx_skip_dispatch);
  size_t (*n_free_blocks)(struct __pyx_obj_4cupy_4cuda_6memory_MemoryPool *, int __pyx_skip_dispatch);
  size_t (*used_bytes)(struct __pyx_obj_4cupy_4cuda_6memory_MemoryPool *, int __pyx_skip_dispatch);
  size_t (*free_bytes)(struct __pyx_obj_4cupy_4cuda_6memory_MemoryPool *, int __pyx_skip_dispatch);
  size_t (*total_bytes)(struct __pyx_obj_4cupy_4cuda_6memory_MemoryPool *, int __pyx_skip_dispatch);
  PyObject *(*set_limit)(struct __pyx_obj_4cupy_4cuda_6memory_MemoryPool *, int __pyx_skip_dispatch, struct __pyx_opt_args_4cupy_4cuda_6memory_10MemoryPool_set_limit *__pyx_optional_args);
  size_t (*get_limit)(struct __pyx_obj_4cupy_4cuda_6memory_MemoryPool *, int __pyx_skip_dispatch);
  PyObject *(*_ensure_pools_and_return_device_pool)(struct __pyx_obj_4cupy_4cuda_6memory_MemoryPool *);
  PyObject *(*device_pool)(struct __pyx_obj_4cupy_4cuda_6memory_MemoryPool *);
};
static struct __pyx_vtabstruct_4cupy_4cuda_6memory_MemoryPool *__pyx_vtabptr_4cupy_4cuda_6memory_MemoryPool;
static CYTHON_INLINE PyObject *__pyx_f_4cupy_4cuda_6memory_10MemoryPool_device_pool(struct __pyx_obj_4cupy_4cuda_6memory_MemoryPool *);


/* "cupy/cuda/memory.pyx":2079
 * 
 * 
 * cdef class CFunctionAllocator:             # <<<<<<<<<<<<<<
 * 
 *     """Allocator with C function pointers to allocation routines.
*/

struct __pyx_vtabstruct_4cupy_4cuda_6memory_CFunctionAllocator {
  struct __pyx_obj_4cupy_4cuda_6memory_MemoryPointer *(*malloc)(struct __pyx_obj_4cupy_4cuda_6memory_CFunctionAllocator *, size_t, int __pyx_skip_dispatch);
};
static struct __pyx_vtabstruct_4cupy_4cuda_6memory_CFunctionAllocator *__pyx_vtabptr_4cupy_4cuda_6memory_CFunctionAllocator;


/* "cupy/cuda/memory.pyx":2136
 * 
 * 
 * cdef class PythonFunctionAllocator:             # <<<<<<<<<<<<<<
 * 
 *     """Allocator with python functions to perform memory allocation.
*/

struct __pyx_vtabstruct_4cupy_4cuda_6memory_PythonFunctionAllocator {
  struct __pyx_obj_4cupy_4cuda_6memory_MemoryPointer *(*malloc)(struct __pyx_obj_4cupy_4cuda_6memory_PythonFunctionAllocator *, size_t, int __pyx_skip_dispatch);
};
static struct __pyx_vtabstruct_4cupy_4cuda_6memory_PythonFunctionAllocator *__pyx_vtabptr_4cupy_4cuda_6memory_PythonFunctionAllocator;


/* "cupy/cuda/memory.pyx":357
 * 
 * 
 * @cython.final             # <<<<<<<<<<<<<<
 * @cython.no_gc  # reference cycle would be a bug
 * cdef class _Chunk:
*/

struct __pyx_vtabstruct_4cupy_4cuda_6memory__Chunk {
  PyObject *(*_init)(struct __pyx_obj_4cupy_4cuda_6memory__Chunk *, struct __pyx_obj_4cupy_4cuda_6memory_BaseMemory *, ptrdiff_t, size_t, struct __pyx_obj_4cupy_4cuda_6memory__Arena *);
  intptr_t (*ptr)(struct __pyx_obj_4cupy_4cuda_6memory__Chunk *, int __pyx_skip_dispatch);
  struct __pyx_obj_4cupy_4cuda_6memory__Chunk *(*split)(struct __pyx_obj_4cupy_4cuda_6memory__Chunk *, size_t, int __pyx_skip_dispatch);
  PyObject *(*merge_next)(struct __pyx_obj_4cupy_4cuda_6memory__Chunk *, int __pyx_skip_dispatch);
};
static struct __pyx_vtabstruct_4cupy_4cuda_6memory__Chunk *__pyx_vtabptr_4cupy_4cuda_6memory__Chunk;
static PyObject *__pyx_f_4cupy_4cuda_6memory_6_Chunk__init(struct __pyx_obj_4cupy_4cuda_6memory__Chunk *, struct __pyx_obj_4cupy_4cuda_6memory_BaseMemory *, ptrdiff_t, size_t, struct __pyx_obj_4cupy_4cuda_6memory__Arena *);
static intptr_t __pyx_f_4cupy_4cuda_6memory_6_Chunk_ptr(struct __pyx_obj_4cupy_4cuda_6memory__Chunk *, int __pyx_skip_dispatch);
static struct __pyx_obj_4cupy_4cuda_6memory__Chunk *__pyx_f_4cupy_4cuda_6memory_6_Chunk_split(struct __pyx_obj_4cupy_4cuda_6memory__Chunk *, size_t, int __pyx_skip_dispatch);
static PyObject *__pyx_f_4cupy_4cuda_6memory_6_Chunk_merge_next(struct __pyx_obj_4cupy_4cuda_6memory__Chunk *, int __pyx_skip_dispatch);


/* "cupy/cuda/memory.pyx":943
 * 
 * 
 * @cython.final             # <<<<<<<<<<<<<<
 * @cython.no_gc
 * cdef class PooledMemory(BaseMemory):
*/

struct __pyx_vtabstruct_4cupy_4cuda_6memory_PooledMemory {
  PyObject *(*_init)(struct __pyx_obj_4cupy_4cuda_6memory_PooledMemory *, struct __pyx_obj_4cupy_4cuda_6memory__Chunk *, PyObject *);
  PyObject *(*free)(struct __pyx_obj_4cupy_4cuda_6memory_PooledMemory *, int __pyx_skip_dispatch);
};
static struct __pyx_vtabstruct_4cupy_4cuda_6memory_PooledMemory *__pyx_vtabptr_4cupy_4cuda_6memory_PooledMemory;
static PyObject *__pyx_f_4cupy_4cuda_6memory_12PooledMemory__init(struct __pyx_obj_4cupy_4cuda_6memory_PooledMemory *, struct __pyx_obj_4cupy_4cuda_6memory__Chunk *, PyObject *);
static PyObject *__pyx_f_4cupy_4cuda_6memory_12PooledMemory_free(struct __pyx_obj_4cupy_4cuda_6memory_PooledMemory *, int __pyx_skip_dispatch);


/* "cupy/cuda/memory.pyx":1054
 * 
 * 
 * @cython.final             # <<<<<<<<<<<<<<
 * cdef class _Arena:
 *     # Arena class managing all free chunks belonging to a single stream ident.
*/

struct __pyx_vtabstruct_4cupy_4cuda_6memory__Arena {
  PyObject *(*add_pending_free_atomic)(struct __pyx_obj_4cupy_4cuda_6memory__Arena *, struct __pyx_obj_4cupy_4cuda_6memory__Chunk *);
  PyObject *(*_commit_pending_free)(struct __pyx_obj_4cupy_4cuda_6memory__Arena *);
  struct __pyx_obj_4cupy_4cuda_6memory__Chunk *(*try_merge_chunk)(struct __pyx_obj_4cupy_4cuda_6memory__Arena *, struct __pyx_obj_4cupy_4cuda_6memory__Chunk *);
  PyObject *(*insert_chunk)(struct __pyx_obj_4cupy_4cuda_6memory__Arena *, struct __pyx_obj_4cupy_4cuda_6memory__Chunk *, struct __pyx_opt_args_4cupy_4cuda_6memory_6_Arena_insert_chunk *__pyx_optional_args);
  size_t (*free_all)(struct __pyx_obj_4cupy_4cuda_6memory__Arena *);
  int (*try_remove_chunk)(struct __pyx_obj_4cupy_4cuda_6memory__Arena *, struct __pyx_obj_4cupy_4cuda_6memory__Chunk *);
  struct __pyx_obj_4cupy_4cuda_6memory__Chunk *(*get_chunk)(struct __pyx_obj_4cupy_4cuda_6memory__Arena *, size_t);
  PyObject *(*_get_size)(struct __pyx_obj_4cupy_4cuda_6memory__Arena *);
  PyObject *(*_index_to_python)(struct __pyx_obj_4cupy_4cuda_6memory__Arena *);
};
static struct __pyx_vtabstruct_4cupy_4cuda_6memory__Arena *__pyx_vtabptr_4cupy_4cuda_6memory__Arena;
static PyObject *__pyx_f_4cupy_4cuda_6memory_6_Arena_add_pending_free_atomic(struct __pyx_obj_4cupy_4cuda_6memory__Arena *, struct __pyx_obj_4cupy_4cuda_6memory__Chunk *);
static PyObject *__pyx_f_4cupy_4cuda_6memory_6_Arena__commit_pending_free(struct __pyx_obj_4cupy_4cuda_6memory__Arena *);
static struct __pyx_obj_4cupy_4cuda_6memory__Chunk *__pyx_f_4cupy_4cuda_6memory_6_Arena_try_merge_chunk(struct __pyx_obj_4cupy_4cuda_6memory__Arena *, struct __pyx_obj_4cupy_4cuda_6memory__Chunk *);
static PyObject *__pyx_f_4cupy_4cuda_6memory_6_Arena_insert_chunk(struct __pyx_obj_4cupy_4cuda_6memory__Arena *, struct __pyx_obj_4cupy_4cuda_6memory__Chunk *, struct __pyx_opt_args_4cupy_4cuda_6memory_6_Arena_insert_chunk *__pyx_optional_args);
static size_t __pyx_f_4cupy_4cuda_6memory_6_Arena_free_all(struct __pyx_obj_4cupy_4cuda_6memory__Arena *);
static int __pyx_f_4cupy_4cuda_6memory_6_Arena_try_remove_chunk(struct __pyx_obj_4cupy_4cuda_6memory__Arena *, struct __pyx_obj_4cupy_4cuda_6memory__Chunk *);
static struct __pyx_obj_4cupy_4cuda_6memory__Chunk *__pyx_f_4cupy_4cuda_6memory_6_Arena_get_chunk(struct __pyx_obj_4cupy_4cuda_6memory__Arena *, size_t);
static PyObject *__pyx_f_4cupy_4cuda_6memory_6_Arena__get_size(struct __pyx_obj_4cupy_4cuda_6memory__Arena *);
static PyObject *__pyx_f_4cupy_4cuda_6memory_6_Arena__index_to_python(struct __pyx_obj_4cupy_4cuda_6memory__Arena *);


/* "cupy/cuda/memory.pyx":1217
 * 
 * 
 * @cython.final             # <<<<<<<<<<<<<<
 * cdef class SingleDeviceMemoryPool:
 *     """Memory pool implementation for single device.
*/

struct __pyx_vtabstruct_4cupy_4cuda_6memory_SingleDeviceMemoryPool {
  struct __pyx_obj_4cupy_4cuda_6memory__Arena *(*_arena)(struct __pyx_obj_4cupy_4cuda_6memory_SingleDeviceMemoryPool *, intptr_t, int __pyx_skip_dispatch);
  struct __pyx_obj_4cupy_4cuda_6memory_MemoryPointer *(*_alloc)(struct __pyx_obj_4cupy_4cuda_6memory_SingleDeviceMemoryPool *, Py_ssize_t);
  struct __pyx_obj_4cupy_4cuda_6memory_MemoryPointer *(*malloc)(struct __pyx_obj_4cupy_4cuda_6memory_SingleDeviceMemoryPool *, size_t, int __pyx_skip_dispatch);
  struct __pyx_obj_4cupy_4cuda_6memory_MemoryPointer *(*_malloc)(struct __pyx_obj_4cupy_4cuda_6memory_SingleDeviceMemoryPool *, size_t);
  PyObject *(*free)(struct __pyx_obj_4cupy_4cuda_6memory_SingleDeviceMemoryPool *, struct __pyx_obj_4cupy_4cuda_6memory__Chunk *);
  PyObject *(*free_all_blocks)(struct __pyx_obj_4cupy_4cuda_6memory_SingleDeviceMemoryPool *, int __pyx_skip_dispatch, struct __pyx_opt_args_4cupy_4cuda_6memory_22SingleDeviceMemoryPool_free_all_blocks *__pyx_optional_args);
  PyObject *(*free_all_free)(struct __pyx_obj_4cupy_4cuda_6memory_SingleDeviceMemoryPool *, int __pyx_skip_dispatch);
  size_t (*n_free_blocks)(struct __pyx_obj_4cupy_4cuda_6memory_SingleDeviceMemoryPool *, int __pyx_skip_dispatch);
  size_t (*used_bytes)(struct __pyx_obj_4cupy_4cuda_6memory_SingleDeviceMemoryPool *, int __pyx_skip_dispatch);
  size_t (*free_bytes)(struct __pyx_obj_4cupy_4cuda_6memory_SingleDeviceMemoryPool *, int __pyx_skip_dispatch);
  size_t (*total_bytes)(struct __pyx_obj_4cupy_4cuda_6memory_SingleDeviceMemoryPool *, int __pyx_skip_dispatch);
  int (*_try_block_total_bytes)(struct __pyx_obj_4cupy_4cuda_6memory_SingleDeviceMemoryPool *, size_t);
  PyObject *(*set_limit)(struct __pyx_obj_4cupy_4cuda_6memory_SingleDeviceMemoryPool *, int __pyx_skip_dispatch, struct __pyx_opt_args_4cupy_4cuda_6memory_22SingleDeviceMemoryPool_set_limit *__pyx_optional_args);
  size_t (*get_limit)(struct __pyx_obj_4cupy_4cuda_6memory_SingleDeviceMemoryPool *, int __pyx_skip_dispatch);
  struct __pyx_obj_4cupy_4cuda_6memory_BaseMemory *(*_try_malloc)(struct __pyx_obj_4cupy_4cuda_6memory_SingleDeviceMemoryPool *, size_t);
};
static struct __pyx_vtabstruct_4cupy_4cuda_6memory_SingleDeviceMemoryPool *__pyx_vtabptr_4cupy_4cuda_6memory_SingleDeviceMemoryPool;
static struct __pyx_obj_4cupy_4cuda_6memory__Arena *__pyx_f_4cupy_4cuda_6memory_22SingleDeviceMemoryPool__arena(struct __pyx_obj_4cupy_4cuda_6memory_SingleDeviceMemoryPool *, intptr_t, int __pyx_skip_dispatch);
static struct __pyx_obj_4cupy_4cuda_6memory_MemoryPointer *__pyx_f_4cupy_4cuda_6memory_22SingleDeviceMemoryPool__alloc(struct __pyx_obj_4cupy_4cuda_6memory_SingleDeviceMemoryPool *, Py_ssize_t);
static struct __pyx_obj_4cupy_4cuda_6memory_MemoryPointer *__pyx_f_4cupy_4cuda_6memory_22SingleDeviceMemoryPool_malloc(struct __pyx_obj_4cupy_4cuda_6memory_SingleDeviceMemoryPool *, size_t, int __pyx_skip_dispatch);
static struct __pyx_obj_4cupy_4cuda_6memory_MemoryPointer *__pyx_f_4cupy_4cuda_6memory_22SingleDeviceMemoryPool__malloc(struct __pyx_obj_4cupy_4cuda_6memory_SingleDeviceMemoryPool *, size_t);
static PyObject *__pyx_f_4cupy_4cuda_6memory_22SingleDeviceMemoryPool_free(struct __pyx_obj_4cupy_4cuda_6memory_SingleDeviceMemoryPool *, struct __pyx_obj_4cupy_4cuda_6memory__Chunk *);
static PyObject *__pyx_f_4cupy_4cuda_6memory_22SingleDeviceMemoryPool_free_all_blocks(struct __pyx_obj_4cupy_4cuda_6memory_SingleDeviceMemoryPool *, int __pyx_skip_dispatch, struct __pyx_opt_args_4cupy_4cuda_6memory_22SingleDeviceMemoryPool_free_all_blocks *__pyx_optional_args);
static PyObject *__pyx_f_4cupy_4cuda_6memory_22SingleDeviceMemoryPool_free_all_free(struct __pyx_obj_4cupy_4cuda_6memory_SingleDeviceMemoryPool *, int __pyx_skip_dispatch);
static size_t __pyx_f_4cupy_4cuda_6memory_22SingleDeviceMemoryPool_n_free_blocks(struct __pyx_obj_4cupy_4cuda_6memory_SingleDeviceMemoryPool *, int __pyx_skip_dispatch);
static size_t __pyx_f_4cupy_4cuda_6memory_22SingleDeviceMemoryPool_used_bytes(struct __pyx_obj_4cupy_4cuda_6memory_SingleDeviceMemoryPool *, int __pyx_skip_dispatch);
static size_t __pyx_f_4cupy_4cuda_6memory_22SingleDeviceMemoryPool_free_bytes(struct __pyx_obj_4cupy_4cuda_6memory_SingleDeviceMemoryPool *, int __pyx_skip_dispatch);
static size_t __pyx_f_4cupy_4cuda_6memory_22SingleDeviceMemoryPool_total_bytes(struct __pyx_obj_4cupy_4cuda_6memory_SingleDeviceMemoryPool *, int __pyx_skip_dispatch);
static int __pyx_f_4cupy_4cuda_6memory_22SingleDeviceMemoryPool__try_block_total_bytes(struct __pyx_obj_4cupy_4cuda_6memory_SingleDeviceMemoryPool *, size_t);
static PyObject *__pyx_f_4cupy_4cuda_6memory_22SingleDeviceMemoryPool_set_limit(struct __pyx_obj_4cupy_4cuda_6memory_SingleDeviceMemoryPool *, int __pyx_skip_dispatch, struct __pyx_opt_args_4cupy_4cuda_6memory_22SingleDeviceMemoryPool_set_limit *__pyx_optional_args);
static size_t __pyx_f_4cupy_4cuda_6memory_22SingleDeviceMemoryPool_get_limit(struct __pyx_obj_4cupy_4cuda_6memory_SingleDeviceMemoryPool *, int __pyx_skip_dispatch);
static struct __pyx_obj_4cupy_4cuda_6memory_BaseMemory *__pyx_f_4cupy_4cuda_6memory_22SingleDeviceMemoryPool__try_malloc(struct __pyx_obj_4cupy_4cuda_6memory_SingleDeviceMemoryPool *, size_t);


/* "cupy/cuda/memory.pyx":1740
 * 
 * 
 * cdef class MemoryAsyncPool:             # <<<<<<<<<<<<<<
 *     """CUDA memory pool for all GPU devices on the host.
 * 
*/

struct __pyx_vtabstruct_4cupy_4cuda_6memory_MemoryAsyncPool {
  intptr_t (*set_pool)(struct __pyx_obj_4cupy_4cuda_6memory_MemoryAsyncPool *, PyObject *, int);
  struct __pyx_obj_4cupy_4cuda_6memory_MemoryPointer *(*malloc)(struct __pyx_obj_4cupy_4cuda_6memory_MemoryAsyncPool *, size_t, int __pyx_skip_dispatch);
  PyObject *(*free_all_blocks)(struct __pyx_obj_4cupy_4cuda_6memory_MemoryAsyncPool *, int __pyx_skip_dispatch, struct __pyx_opt_args_4cupy_4cuda_6memory_15MemoryAsyncPool_free_all_blocks *__pyx_optional_args);
  size_t (*n_free_blocks)(struct __pyx_obj_4cupy_4cuda_6memory_MemoryAsyncPool *, int __pyx_skip_dispatch);
  size_t (*used_bytes)(struct __pyx_obj_4cupy_4cuda_6memory_MemoryAsyncPool *, int __pyx_skip_dispatch);
  size_t (*free_bytes)(struct __pyx_obj_4cupy_4cuda_6memory_MemoryAsyncPool *, int __pyx_skip_dispatch);
  size_t (*total_bytes)(struct __pyx_obj_4cupy_4cuda_6memory_MemoryAsyncPool *, int __pyx_skip_dispatch);
  PyObject *(*set_limit)(struct __pyx_obj_4cupy_4cuda_6memory_MemoryAsyncPool *, int __pyx_skip_dispatch, struct __pyx_opt_args_4cupy_4cuda_6memory_15MemoryAsyncPool_set_limit *__pyx_optional_args);
  size_t (*get_limit)(struct __pyx_obj_4cupy_4cuda_6memory_MemoryAsyncPool *, int __pyx_skip_dispatch);
};
static struct __pyx_vtabstruct_4cupy_4cuda_6memory_MemoryAsyncPool *__pyx_vtabptr_4cupy_4cuda_6memory_MemoryAsyncPool;
/* #### Code section: utility_code_proto ### */

/* --- Runtime support code (head) --- */
/* Refnanny.proto */
#ifndef CYTHON_REFNANNY
  #define CYTHON_REFNANNY 0
#endif
#if CYTHON_REFNANNY
  typedef struct {
    void (*INCREF)(void*, PyObject*, Py_ssize_t);
    void (*DECREF)(void*, PyObject*, Py_ssize_t);
    void (*GOTREF)(void*, PyObject*, Py_ssize_t);
    void (*GIVEREF)(void*, PyObject*, Py_ssize_t);
    void* (*SetupContext)(const char*, Py_ssize_t, const char*);
    void (*FinishContext)(void**);
  } __Pyx_RefNannyAPIStruct;
  static __Pyx_RefNannyAPIStruct *__Pyx_RefNanny = NULL;
  static __Pyx_RefNannyAPIStruct *__Pyx_RefNannyImportAPI(const char *modname);
  #define __Pyx_RefNannyDeclarations void *__pyx_refnanny = NULL;
  #define __Pyx_RefNannySetupContext(name, acquire_gil)\
          if (acquire_gil) {\
              PyGILState_STATE __pyx_gilstate_save = PyGILState_Ensure();\
              __pyx_refnanny = __Pyx_RefNanny->SetupContext((name), (__LINE__), (__FILE__));\
              PyGILState_Release(__pyx_gilstate_save);\
          } else {\
              __pyx_refnanny = __Pyx_RefNanny->SetupContext((name), (__LINE__), (__FILE__));\
          }
  #define __Pyx_RefNannyFinishContextNogil() {\
              PyGILState_STATE __pyx_gilstate_save = PyGILState_Ensure();\
              __Pyx_RefNannyFinishContext();\
              PyGILState_Release(__pyx_gilstate_save);\
          }
  #define __Pyx_RefNannyFinishContextNogil() {\
              PyGILState_STATE __pyx_gilstate_save = PyGILState_Ensure();\
              __Pyx_RefNannyFinishContext();\
              PyGILState_Release(__pyx_gilstate_save);\
          }
  #define __Pyx_RefNannyFinishContext()\
          __Pyx_RefNanny->FinishContext(&__pyx_refnanny)
  #define __Pyx_INCREF(r)  __Pyx_RefNanny->INCREF(__pyx_refnanny, (PyObject *)(r), (__LINE__))
  #define __Pyx_DECREF(r)  __Pyx_RefNanny->DECREF(__pyx_refnanny, (PyObject *)(r), (__LINE__))
  #define __Pyx_GOTREF(r)  __Pyx_RefNanny->GOTREF(__pyx_refnanny, (PyObject *)(r), (__LINE__))
  #define __Pyx_GIVEREF(r) __Pyx_RefNanny->GIVEREF(__pyx_refnanny, (PyObject *)(r), (__LINE__))
  #define __Pyx_XINCREF(r)  do { if((r) == NULL); else {__Pyx_INCREF(r); }} while(0)
  #define __Pyx_XDECREF(r)  do { if((r) == NULL); else {__Pyx_DECREF(r); }} while(0)
  #define __Pyx_XGOTREF(r)  do { if((r) == NULL); else {__Pyx_GOTREF(r); }} while(0)
  #define __Pyx_XGIVEREF(r) do { if((r) == NULL); else {__Pyx_GIVEREF(r);}} while(0)
#else
  #define __Pyx_RefNannyDeclarations
  #define __Pyx_RefNannySetupContext(name, acquire_gil)
  #define __Pyx_RefNannyFinishContextNogil()
  #define __Pyx_RefNannyFinishContext()
  #define __Pyx_INCREF(r) Py_INCREF(r)
  #define __Pyx_DECREF(r) Py_DECREF(r)
  #define __Pyx_GOTREF(r)
  #define __Pyx_GIVEREF(r)
  #define __Pyx_XINCREF(r) Py_XINCREF(r)
  #define __Pyx_XDECREF(r) Py_XDECREF(r)
  #define __Pyx_XGOTREF(r)
  #define __Pyx_XGIVEREF(r)
#endif
#define __Pyx_Py_XDECREF_SET(r, v) do {\
        PyObject *tmp = (PyObject *) r;\
        r = v; Py_XDECREF(tmp);\
    } while (0)
#define __Pyx_XDECREF_SET(r, v) do {\
        PyObject *tmp = (PyObject *) r;\
        r = v; __Pyx_XDECREF(tmp);\
    } while (0)
#define __Pyx_DECREF_SET(r, v) do {\
        PyObject *tmp = (PyObject *) r;\
        r = v; __Pyx_DECREF(tmp);\
    } while (0)
#define __Pyx_CLEAR(r)    do { PyObject* tmp = ((PyObject*)(r)); r = NULL; __Pyx_DECREF(tmp);} while(0)
#define __Pyx_XCLEAR(r)   do { if((r) != NULL) {PyObject* tmp = ((PyObject*)(r)); r = NULL; __Pyx_DECREF(tmp);}} while(0)

/* PyErrExceptionMatches.proto (used by PyObjectGetAttrStrNoError) */
#if CYTHON_FAST_THREAD_STATE
#define __Pyx_PyErr_ExceptionMatches(err) __Pyx_PyErr_ExceptionMatchesInState(__pyx_tstate, err)
static CYTHON_INLINE int __Pyx_PyErr_ExceptionMatchesInState(PyThreadState* tstate, PyObject* err);
#else
#define __Pyx_PyErr_ExceptionMatches(err)  PyErr_ExceptionMatches(err)
#endif

/* PyThreadStateGet.proto (used by PyErrFetchRestore) */
#if CYTHON_FAST_THREAD_STATE
#define __Pyx_PyThreadState_declare  PyThreadState *__pyx_tstate;
#define __Pyx_PyThreadState_assign  __pyx_tstate = __Pyx_PyThreadState_Current;
#if PY_VERSION_HEX >= 0x030C00A6
#define __Pyx_PyErr_Occurred()  (__pyx_tstate->current_exception != NULL)
#define __Pyx_PyErr_CurrentExceptionType()  (__pyx_tstate->current_exception ? (PyObject*) Py_TYPE(__pyx_tstate->current_exception) : (PyObject*) NULL)
#else
#define __Pyx_PyErr_Occurred()  (__pyx_tstate->curexc_type != NULL)
#define __Pyx_PyErr_CurrentExceptionType()  (__pyx_tstate->curexc_type)
#endif
#else
#define __Pyx_PyThreadState_declare
#define __Pyx_PyThreadState_assign
#define __Pyx_PyErr_Occurred()  (PyErr_Occurred() != NULL)
#define __Pyx_PyErr_CurrentExceptionType()  PyErr_Occurred()
#endif

/* PyErrFetchRestore.proto (used by PyObjectGetAttrStrNoError) */
#if CYTHON_FAST_THREAD_STATE
#define __Pyx_PyErr_Clear() __Pyx_ErrRestore(NULL, NULL, NULL)
#define __Pyx_ErrRestoreWithState(type, value, tb)  __Pyx_ErrRestoreInState(PyThreadState_GET(), type, value, tb)
#define __Pyx_ErrFetchWithState(type, value, tb)    __Pyx_ErrFetchInState(PyThreadState_GET(), type, value, tb)
#define __Pyx_ErrRestore(type, value, tb)  __Pyx_ErrRestoreInState(__pyx_tstate, type, value, tb)
#define __Pyx_ErrFetch(type, value, tb)    __Pyx_ErrFetchInState(__pyx_tstate, type, value, tb)
static CYTHON_INLINE void __Pyx_ErrRestoreInState(PyThreadState *tstate, PyObject *type, PyObject *value, PyObject *tb);
static CYTHON_INLINE void __Pyx_ErrFetchInState(PyThreadState *tstate, PyObject **type, PyObject **value, PyObject **tb);
#if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX < 0x030C00A6
#define __Pyx_PyErr_SetNone(exc) (Py_INCREF(exc), __Pyx_ErrRestore((exc), NULL, NULL))
#else
#define __Pyx_PyErr_SetNone(exc) PyErr_SetNone(exc)
#endif
#else
#define __Pyx_PyErr_Clear() PyErr_Clear()
#define __Pyx_PyErr_SetNone(exc) PyErr_SetNone(exc)
#define __Pyx_ErrRestoreWithState(type, value, tb)  PyErr_Restore(type, value, tb)
#define __Pyx_ErrFetchWithState(type, value, tb)  PyErr_Fetch(type, value, tb)
#define __Pyx_ErrRestoreInState(tstate, type, value, tb)  PyErr_Restore(type, value, tb)
#define __Pyx_ErrFetchInState(tstate, type, value, tb)  PyErr_Fetch(type, value, tb)
#define __Pyx_ErrRestore(type, value, tb)  PyErr_Restore(type, value, tb)
#define __Pyx_ErrFetch(type, value, tb)  PyErr_Fetch(type, value, tb)
#endif

/* PyObjectGetAttrStr.proto (used by PyObjectGetAttrStrNoError) */
#if CYTHON_USE_TYPE_SLOTS
static CYTHON_INLINE PyObject* __Pyx_PyObject_GetAttrStr(PyObject* obj, PyObject* attr_name);
#else
#define __Pyx_PyObject_GetAttrStr(o,n) PyObject_GetAttr(o,n)
#endif

/* PyObjectGetAttrStrNoError.proto (used by GetBuiltinName) */
static CYTHON_INLINE PyObject* __Pyx_PyObject_GetAttrStrNoError(PyObject* obj, PyObject* attr_name);

/* GetBuiltinName.proto */
static PyObject *__Pyx_GetBuiltinName(PyObject *name);

/* GetItemInt.proto */
#define __Pyx_GetItemInt(o, i, type, is_signed, to_py_func, is_list, wraparound, boundscheck, has_gil, unsafe_shared)\
    (__Pyx_fits_Py_ssize_t(i, type, is_signed) ?\
    __Pyx_GetItemInt_Fast(o, (Py_ssize_t)i, is_list, wraparound, boundscheck, unsafe_shared) :\
    (is_list ? (PyErr_SetString(PyExc_IndexError, "list index out of range"), (PyObject*)NULL) :\
               __Pyx_GetItemInt_Generic(o, to_py_func(i))))
#define __Pyx_GetItemInt_List(o, i, type, is_signed, to_py_func, is_list, wraparound, boundscheck, has_gil, unsafe_shared)\
    (__Pyx_fits_Py_ssize_t(i, type, is_signed) ?\
    __Pyx_GetItemInt_List_Fast(o, (Py_ssize_t)i, wraparound, boundscheck, unsafe_shared) :\
    (PyErr_SetString(PyExc_IndexError, "list index out of range"), (PyObject*)NULL))
static CYTHON_INLINE PyObject *__Pyx_GetItemInt_List_Fast(PyObject *o, Py_ssize_t i,
                                                              int wraparound, int boundscheck, int unsafe_shared);
#define __Pyx_GetItemInt_Tuple(o, i, type, is_signed, to_py_func, is_list, wraparound, boundscheck, has_gil, unsafe_shared)\
    (__Pyx_fits_Py_ssize_t(i, type, is_signed) ?\
    __Pyx_GetItemInt_Tuple_Fast(o, (Py_ssize_t)i, wraparound, boundscheck, unsafe_shared) :\
    (PyErr_SetString(PyExc_IndexError, "tuple index out of range"), (PyObject*)NULL))
static CYTHON_INLINE PyObject *__Pyx_GetItemInt_Tuple_Fast(PyObject *o, Py_ssize_t i,
                                                              int wraparound, int boundscheck, int unsafe_shared);
static PyObject *__Pyx_GetItemInt_Generic(PyObject *o, PyObject* j);
static CYTHON_INLINE PyObject *__Pyx_GetItemInt_Fast(PyObject *o, Py_ssize_t i,
                                                     int is_list, int wraparound, int boundscheck, int unsafe_shared);

/* ReleaseUnknownGil.proto */
#if CYTHON_COMPILING_IN_LIMITED_API && __PYX_LIMITED_VERSION_HEX < 0x030d0000
typedef struct {
  PyThreadState* ts;
  PyGILState_STATE gil_state;
} __Pyx_UnknownThreadState;
#else
#define __Pyx_UnknownThreadState PyThreadState*
#endif
static __Pyx_UnknownThreadState __Pyx_SaveUnknownThread(void);
static void __Pyx_RestoreUnknownThread(__Pyx_UnknownThreadState state);
static CYTHON_INLINE int __Pyx_UnknownThreadStateDefinitelyHadGil(__Pyx_UnknownThreadState state);
static CYTHON_INLINE int __Pyx_UnknownThreadStateMayHaveHadGil(__Pyx_UnknownThreadState state);

/* TupleAndListFromArray.proto (used by fastcall) */
#if CYTHON_COMPILING_IN_CPYTHON
static CYTHON_INLINE PyObject* __Pyx_PyList_FromArray(PyObject *const *src, Py_ssize_t n);
#endif
#if CYTHON_COMPILING_IN_CPYTHON || CYTHON_METH_FASTCALL
static CYTHON_INLINE PyObject* __Pyx_PyTuple_FromArray(PyObject *const *src, Py_ssize_t n);
#endif

/* IncludeStringH.proto (used by BytesEquals) */
#include <string.h>

/* BytesEquals.proto (used by UnicodeEquals) */
static CYTHON_INLINE int __Pyx_PyBytes_Equals(PyObject* s1, PyObject* s2, int equals);

/* UnicodeEquals.proto (used by fastcall) */
static CYTHON_INLINE int __Pyx_PyUnicode_Equals(PyObject* s1, PyObject* s2, int equals);

/* fastcall.proto */
#if CYTHON_AVOID_BORROWED_REFS
    #define __Pyx_ArgRef_VARARGS(args, i) __Pyx_PySequence_ITEM(args, i)
#elif CYTHON_ASSUME_SAFE_MACROS
    #define __Pyx_ArgRef_VARARGS(args, i) __Pyx_NewRef(__Pyx_PyTuple_GET_ITEM(args, i))
#else
    #define __Pyx_ArgRef_VARARGS(args, i) __Pyx_XNewRef(PyTuple_GetItem(args, i))
#endif
#define __Pyx_NumKwargs_VARARGS(kwds) PyDict_Size(kwds)
#define __Pyx_KwValues_VARARGS(args, nargs) NULL
#define __Pyx_GetKwValue_VARARGS(kw, kwvalues, s) __Pyx_PyDict_GetItemStrWithError(kw, s)
#define __Pyx_KwargsAsDict_VARARGS(kw, kwvalues) PyDict_Copy(kw)
#if CYTHON_METH_FASTCALL
    #define __Pyx_ArgRef_FASTCALL(args, i) __Pyx_NewRef(args[i])
    #define __Pyx_NumKwargs_FASTCALL(kwds) __Pyx_PyTuple_GET_SIZE(kwds)
    #define __Pyx_KwValues_FASTCALL(args, nargs) ((args) + (nargs))
    static CYTHON_INLINE PyObject * __Pyx_GetKwValue_FASTCALL(PyObject *kwnames, PyObject *const *kwvalues, PyObject *s);
  #if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x030d0000 || CYTHON_COMPILING_IN_LIMITED_API
    CYTHON_UNUSED static PyObject *__Pyx_KwargsAsDict_FASTCALL(PyObject *kwnames, PyObject *const *kwvalues);
  #else
    #define __Pyx_KwargsAsDict_FASTCALL(kw, kwvalues) _PyStack_AsDict(kwvalues, kw)
  #endif
#else
    #define __Pyx_ArgRef_FASTCALL __Pyx_ArgRef_VARARGS
    #define __Pyx_NumKwargs_FASTCALL __Pyx_NumKwargs_VARARGS
    #define __Pyx_KwValues_FASTCALL __Pyx_KwValues_VARARGS
    #define __Pyx_GetKwValue_FASTCALL __Pyx_GetKwValue_VARARGS
    #define __Pyx_KwargsAsDict_FASTCALL __Pyx_KwargsAsDict_VARARGS
#endif
#define __Pyx_ArgsSlice_VARARGS(args, start, stop) PyTuple_GetSlice(args, start, stop)
#if CYTHON_METH_FASTCALL || (CYTHON_COMPILING_IN_CPYTHON && CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS)
#define __Pyx_ArgsSlice_FASTCALL(args, start, stop) __Pyx_PyTuple_FromArray(args + start, stop - start)
#else
#define __Pyx_ArgsSlice_FASTCALL(args, start, stop) PyTuple_GetSlice(args, start, stop)
#endif

/* py_dict_items.proto (used by OwnedDictNext) */
static CYTHON_INLINE PyObject* __Pyx_PyDict_Items(PyObject* d);

/* CallCFunction.proto (used by CallUnboundCMethod0) */
#define __Pyx_CallCFunction(cfunc, self, args)\
    ((PyCFunction)(void(*)(void))(cfunc)->func)(self, args)
#define __Pyx_CallCFunctionWithKeywords(cfunc, self, args, kwargs)\
    ((PyCFunctionWithKeywords)(void(*)(void))(cfunc)->func)(self, args, kwargs)
#define __Pyx_CallCFunctionFast(cfunc, self, args, nargs)\
    ((__Pyx_PyCFunctionFast)(void(*)(void))(PyCFunction)(cfunc)->func)(self, args, nargs)
#define __Pyx_CallCFunctionFastWithKeywords(cfunc, self, args, nargs, kwnames)\
    ((__Pyx_PyCFunctionFastWithKeywords)(void(*)(void))(PyCFunction)(cfunc)->func)(self, args, nargs, kwnames)

/* PyObjectCall.proto (used by PyObjectFastCall) */
#if CYTHON_COMPILING_IN_CPYTHON
static CYTHON_INLINE PyObject* __Pyx_PyObject_Call(PyObject *func, PyObject *arg, PyObject *kw);
#else
#define __Pyx_PyObject_Call(func, arg, kw) PyObject_Call(func, arg, kw)
#endif

/* PyObjectCallMethO.proto (used by PyObjectFastCall) */
#if CYTHON_COMPILING_IN_CPYTHON
static CYTHON_INLINE PyObject* __Pyx_PyObject_CallMethO(PyObject *func, PyObject *arg);
#endif

/* PyObjectFastCall.proto (used by PyObjectCallOneArg) */
#define __Pyx_PyObject_FastCall(func, args, nargs)  __Pyx_PyObject_FastCallDict(func, args, (size_t)(nargs), NULL)
static CYTHON_INLINE PyObject* __Pyx_PyObject_FastCallDict(PyObject *func, PyObject * const*args, size_t nargs, PyObject *kwargs);

/* PyObjectCallOneArg.proto (used by CallUnboundCMethod0) */
static CYTHON_INLINE PyObject* __Pyx_PyObject_CallOneArg(PyObject *func, PyObject *arg);

/* UnpackUnboundCMethod.proto (used by CallUnboundCMethod0) */
typedef struct {
    PyObject *type;
    PyObject **method_name;
#if CYTHON_COMPILING_IN_CPYTHON_FREETHREADING && CYTHON_ATOMICS
    __pyx_atomic_int_type initialized;
#endif
    PyCFunction func;
    PyObject *method;
    int flag;
} __Pyx_CachedCFunction;
#if CYTHON_COMPILING_IN_CPYTHON_FREETHREADING
static CYTHON_INLINE int __Pyx_CachedCFunction_GetAndSetInitializing(__Pyx_CachedCFunction *cfunc) {
#if !CYTHON_ATOMICS
    return 1;
#else
    __pyx_nonatomic_int_type expected = 0;
    if (__pyx_atomic_int_cmp_exchange(&cfunc->initialized, &expected, 1)) {
        return 0;
    }
    return expected;
#endif
}
static CYTHON_INLINE void __Pyx_CachedCFunction_SetFinishedInitializing(__Pyx_CachedCFunction *cfunc) {
#if CYTHON_ATOMICS
    __pyx_atomic_store(&cfunc->initialized, 2);
#endif
}
#else
#define __Pyx_CachedCFunction_GetAndSetInitializing(cfunc) 2
#define __Pyx_CachedCFunction_SetFinishedInitializing(cfunc)
#endif

/* CallUnboundCMethod0.proto */
CYTHON_UNUSED
static PyObject* __Pyx__CallUnboundCMethod0(__Pyx_CachedCFunction* cfunc, PyObject* self);
#if CYTHON_COMPILING_IN_CPYTHON
static CYTHON_INLINE PyObject* __Pyx_CallUnboundCMethod0(__Pyx_CachedCFunction* cfunc, PyObject* self);
#else
#define __Pyx_CallUnboundCMethod0(cfunc, self)  __Pyx__CallUnboundCMethod0(cfunc, self)
#endif

/* py_dict_values.proto (used by OwnedDictNext) */
static CYTHON_INLINE PyObject* __Pyx_PyDict_Values(PyObject* d);

/* OwnedDictNext.proto (used by ParseKeywordsImpl) */
#if CYTHON_AVOID_BORROWED_REFS
static int __Pyx_PyDict_NextRef(PyObject *p, PyObject **ppos, PyObject **pkey, PyObject **pvalue);
#else
CYTHON_INLINE
static int __Pyx_PyDict_NextRef(PyObject *p, Py_ssize_t *ppos, PyObject **pkey, PyObject **pvalue);
#endif

/* RaiseDoubleKeywords.proto (used by ParseKeywordsImpl) */
static void __Pyx_RaiseDoubleKeywordsError(const char* func_name, PyObject* kw_name);

/* ParseKeywordsImpl.export */
static int __Pyx_ParseKeywordsTuple(
    PyObject *kwds,
    PyObject * const *kwvalues,
    PyObject ** const argnames[],
    PyObject *kwds2,
    PyObject *values[],
    Py_ssize_t num_pos_args,
    Py_ssize_t num_kwargs,
    const char* function_name,
    int ignore_unknown_kwargs
);
static int __Pyx_ParseKeywordDictToDict(
    PyObject *kwds,
    PyObject ** const argnames[],
    PyObject *kwds2,
    PyObject *values[],
    Py_ssize_t num_pos_args,
    const char* function_name
);
static int __Pyx_ParseKeywordDict(
    PyObject *kwds,
    PyObject ** const argnames[],
    PyObject *values[],
    Py_ssize_t num_pos_args,
    Py_ssize_t num_kwargs,
    const char* function_name,
    int ignore_unknown_kwargs
);

/* CallUnboundCMethod2.proto */
CYTHON_UNUSED
static PyObject* __Pyx__CallUnboundCMethod2(__Pyx_CachedCFunction* cfunc, PyObject* self, PyObject* arg1, PyObject* arg2);
#if CYTHON_COMPILING_IN_CPYTHON
static CYTHON_INLINE PyObject *__Pyx_CallUnboundCMethod2(__Pyx_CachedCFunction *cfunc, PyObject *self, PyObject *arg1, PyObject *arg2);
#else
#define __Pyx_CallUnboundCMethod2(cfunc, self, arg1, arg2)  __Pyx__CallUnboundCMethod2(cfunc, self, arg1, arg2)
#endif

/* ParseKeywords.proto */
static CYTHON_INLINE int __Pyx_ParseKeywords(
    PyObject *kwds, PyObject *const *kwvalues, PyObject ** const argnames[],
    PyObject *kwds2, PyObject *values[],
    Py_ssize_t num_pos_args, Py_ssize_t num_kwargs,
    const char* function_name,
    int ignore_unknown_kwargs
);

/* RaiseArgTupleInvalid.proto */
static void __Pyx_RaiseArgtupleInvalid(const char* func_name, int exact,
    Py_ssize_t num_min, Py_ssize_t num_max, Py_ssize_t num_found);

/* PyObjectDelAttr.proto (used by PyObjectSetAttrStr) */
#if CYTHON_COMPILING_IN_LIMITED_API && __PYX_LIMITED_VERSION_HEX < 0x030d0000
#define __Pyx_PyObject_DelAttr(o, n) PyObject_SetAttr(o, n, NULL)
#else
#define __Pyx_PyObject_DelAttr(o, n) PyObject_DelAttr(o, n)
#endif

/* PyObjectSetAttrStr.proto */
#if CYTHON_USE_TYPE_SLOTS
#define __Pyx_PyObject_DelAttrStr(o,n) __Pyx_PyObject_SetAttrStr(o, n, NULL)
static CYTHON_INLINE int __Pyx_PyObject_SetAttrStr(PyObject* obj, PyObject* attr_name, PyObject* value);
#else
#define __Pyx_PyObject_DelAttrStr(o,n)   __Pyx_PyObject_DelAttr(o,n)
#define __Pyx_PyObject_SetAttrStr(o,n,v) PyObject_SetAttr(o,n,v)
#endif

/* PyLongCompare.proto */
static CYTHON_INLINE int __Pyx_PyLong_BoolEqObjC(PyObject *op1, PyObject *op2, long intval, long inplace);

/* PyObjectFastCallMethod.proto */
#if CYTHON_VECTORCALL && PY_VERSION_HEX >= 0x03090000
#define __Pyx_PyObject_FastCallMethod(name, args, nargsf) PyObject_VectorcallMethod(name, args, nargsf, NULL)
#else
static PyObject *__Pyx_PyObject_FastCallMethod(PyObject *name, PyObject *const *args, size_t nargsf);
#endif

/* PyLongCompare.proto */
static CYTHON_INLINE int __Pyx_PyLong_BoolNeObjC(PyObject *op1, PyObject *op2, long intval, long inplace);

/* UnicodeConcatInPlace.proto */
# if CYTHON_COMPILING_IN_CPYTHON
    #if CYTHON_REFNANNY
        #define __Pyx_PyUnicode_ConcatInPlace(left, right, unsafe_shared) __Pyx_PyUnicode_ConcatInPlaceImpl(&left, right, unsafe_shared, __pyx_refnanny)
    #else
        #define __Pyx_PyUnicode_ConcatInPlace(left, right, unsafe_shared) __Pyx_PyUnicode_ConcatInPlaceImpl(&left, right, unsafe_shared)
    #endif
    #define __Pyx_PyUnicode_Concat__Pyx_ReferenceSharing_DefinitelyUniqueInPlace(left, right) __Pyx_PyUnicode_ConcatInPlace(left, right, __Pyx_ReferenceSharing_DefinitelyUnique)
    #define __Pyx_PyUnicode_Concat__Pyx_ReferenceSharing_OwnStrongReferenceInPlace(left, right) __Pyx_PyUnicode_ConcatInPlace(left, right, __Pyx_ReferenceSharing_OwnStrongReference)
    #define __Pyx_PyUnicode_Concat__Pyx_ReferenceSharing_FunctionArgumentInPlace(left, right) __Pyx_PyUnicode_ConcatInPlace(left, right, __Pyx_ReferenceSharing_DefinitelyUnique)
    #define __Pyx_PyUnicode_Concat__Pyx_ReferenceSharing_SharedReferenceInPlace(left, right) __Pyx_PyUnicode_ConcatInPlace(left, right, __Pyx_ReferenceSharing_SharedReference)
    static CYTHON_INLINE PyObject *__Pyx_PyUnicode_ConcatInPlaceImpl(PyObject **p_left, PyObject *right, int unsafe_shared
        #if CYTHON_REFNANNY
        , void* __pyx_refnanny
        #endif
    );
#else
#define __Pyx_PyUnicode_Concat__Pyx_ReferenceSharing_DefinitelyUniqueInPlace __Pyx_PyUnicode_Concat
#define __Pyx_PyUnicode_Concat__Pyx_ReferenceSharing_OwnStrongReferenceInPlace __Pyx_PyUnicode_Concat
#define __Pyx_PyUnicode_Concat__Pyx_ReferenceSharing_FunctionArgumentInPlace __Pyx_PyUnicode_Concat
#define __Pyx_PyUnicode_Concat__Pyx_ReferenceSharing_SharedReferenceInPlace __Pyx_PyUnicode_Concat
#endif
#define __Pyx_PyUnicode_Concat__Pyx_ReferenceSharing_DefinitelyUniqueInPlaceSafe(left, right)\
    ((unlikely((left) == Py_None) || unlikely((right) == Py_None)) ?\
    PyNumber_InPlaceAdd(left, right) : __Pyx_PyUnicode_Concat__Pyx_ReferenceSharing_DefinitelyUniqueInPlace(left, right))
#define __Pyx_PyUnicode_Concat__Pyx_ReferenceSharing_OwnStrongReferenceInPlaceSafe(left, right)\
    ((unlikely((left) == Py_None) || unlikely((right) == Py_None)) ?\
    PyNumber_InPlaceAdd(left, right) : __Pyx_PyUnicode_Concat__Pyx_ReferenceSharing_OwnStrongReferenceInPlace(left, right))
#define __Pyx_PyUnicode_Concat__Pyx_ReferenceSharing_FunctionArgumentInPlaceSafe(left, right)\
    ((unlikely((left) == Py_None) || unlikely((right) == Py_None)) ?\
    PyNumber_InPlaceAdd(left, right) : __Pyx_PyUnicode_Concat__Pyx_ReferenceSharing_FunctionArgumentInPlace(left, right))
#define __Pyx_PyUnicode_Concat__Pyx_ReferenceSharing_SharedReferenceInPlaceSafe(left, right)\
    ((unlikely((left) == Py_None) || unlikely((right) == Py_None)) ?\
    PyNumber_InPlaceAdd(left, right) : __Pyx_PyUnicode_Concat__Pyx_ReferenceSharing_SharedReferenceInPlace(left, right))

/* PyDictVersioning.proto (used by GetModuleGlobalName) */
#if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_TYPE_SLOTS
#define __PYX_DICT_VERSION_INIT  ((PY_UINT64_T) -1)
#define __PYX_GET_DICT_VERSION(dict)  (((PyDictObject*)(dict))->ma_version_tag)
#define __PYX_UPDATE_DICT_CACHE(dict, value, cache_var, version_var)\
    (version_var) = __PYX_GET_DICT_VERSION(dict);\
    (cache_var) = (value);
#define __PYX_PY_DICT_LOOKUP_IF_MODIFIED(VAR, DICT, LOOKUP) {\
    static PY_UINT64_T __pyx_dict_version = 0;\
    static PyObject *__pyx_dict_cached_value = NULL;\
    if (likely(__PYX_GET_DICT_VERSION(DICT) == __pyx_dict_version)) {\
        (VAR) = __Pyx_XNewRef(__pyx_dict_cached_value);\
    } else {\
        (VAR) = __pyx_dict_cached_value = (LOOKUP);\
        __pyx_dict_version = __PYX_GET_DICT_VERSION(DICT);\
    }\
}
static CYTHON_INLINE PY_UINT64_T __Pyx_get_tp_dict_version(PyObject *obj);
static CYTHON_INLINE PY_UINT64_T __Pyx_get_object_dict_version(PyObject *obj);
static CYTHON_INLINE int __Pyx_object_dict_version_matches(PyObject* obj, PY_UINT64_T tp_dict_version, PY_UINT64_T obj_dict_version);
#else
#define __PYX_GET_DICT_VERSION(dict)  (0)
#define __PYX_UPDATE_DICT_CACHE(dict, value, cache_var, version_var)
#define __PYX_PY_DICT_LOOKUP_IF_MODIFIED(VAR, DICT, LOOKUP)  (VAR) = (LOOKUP);
#endif

/* GetModuleGlobalName.proto */
#if CYTHON_USE_DICT_VERSIONS
#define __Pyx_GetModuleGlobalName(var, name)  do {\
    static PY_UINT64_T __pyx_dict_version = 0;\
    static PyObject *__pyx_dict_cached_value = NULL;\
    (var) = (likely(__pyx_dict_version == __PYX_GET_DICT_VERSION(__pyx_mstate_global->__pyx_d))) ?\
        (likely(__pyx_dict_cached_value) ? __Pyx_NewRef(__pyx_dict_cached_value) : __Pyx_GetBuiltinName(name)) :\
        __Pyx__GetModuleGlobalName(name, &__pyx_dict_version, &__pyx_dict_cached_value);\
} while(0)
#define __Pyx_GetModuleGlobalNameUncached(var, name)  do {\
    PY_UINT64_T __pyx_dict_version;\
    PyObject *__pyx_dict_cached_value;\
    (var) = __Pyx__GetModuleGlobalName(name, &__pyx_dict_version, &__pyx_dict_cached_value);\
} while(0)
static PyObject *__Pyx__GetModuleGlobalName(PyObject *name, PY_UINT64_T *dict_version, PyObject **dict_cached_value);
#else
#define __Pyx_GetModuleGlobalName(var, name)  (var) = __Pyx__GetModuleGlobalName(name)
#define __Pyx_GetModuleGlobalNameUncached(var, name)  (var) = __Pyx__GetModuleGlobalName(name)
static CYTHON_INLINE PyObject *__Pyx__GetModuleGlobalName(PyObject *name);
#endif

/* RejectKeywords.export */
static void __Pyx_RejectKeywords(const char* function_name, PyObject *kwds);

/* GetAttr3.proto */
static CYTHON_INLINE PyObject *__Pyx_GetAttr3(PyObject *, PyObject *, PyObject *);

/* RaiseUnexpectedTypeError.proto */
static int __Pyx_RaiseUnexpectedTypeError(const char *expected, PyObject *obj);

/* WriteUnraisableException.proto */
static void __Pyx_WriteUnraisable(const char *name, int clineno,
                                  int lineno, const char *filename,
                                  int full_traceback, int nogil);

/* PyRuntimeError_Check.proto */
#define __Pyx_PyExc_RuntimeError_Check(obj)  __Pyx_TypeCheck(obj, PyExc_RuntimeError)

/* RaiseException.export */
static void __Pyx_Raise(PyObject *type, PyObject *value, PyObject *tb, PyObject *cause);

/* GetTopmostException.proto (used by SaveResetException) */
#if CYTHON_USE_EXC_INFO_STACK && CYTHON_FAST_THREAD_STATE
static _PyErr_StackItem * __Pyx_PyErr_GetTopmostException(PyThreadState *tstate);
#endif

/* SaveResetException.proto */
#if CYTHON_FAST_THREAD_STATE
#define __Pyx_ExceptionSave(type, value, tb)  __Pyx__ExceptionSave(__pyx_tstate, type, value, tb)
static CYTHON_INLINE void __Pyx__ExceptionSave(PyThreadState *tstate, PyObject **type, PyObject **value, PyObject **tb);
#define __Pyx_ExceptionReset(type, value, tb)  __Pyx__ExceptionReset(__pyx_tstate, type, value, tb)
static CYTHON_INLINE void __Pyx__ExceptionReset(PyThreadState *tstate, PyObject *type, PyObject *value, PyObject *tb);
#else
#define __Pyx_ExceptionSave(type, value, tb)   PyErr_GetExcInfo(type, value, tb)
#define __Pyx_ExceptionReset(type, value, tb)  PyErr_SetExcInfo(type, value, tb)
#endif

/* PyAttributeError_Check.proto */
#define __Pyx_PyExc_AttributeError_Check(obj)  __Pyx_TypeCheck(obj, PyExc_AttributeError)

/* GetException.proto */
#if CYTHON_FAST_THREAD_STATE
#define __Pyx_GetException(type, value, tb)  __Pyx__GetException(__pyx_tstate, type, value, tb)
static int __Pyx__GetException(PyThreadState *tstate, PyObject **type, PyObject **value, PyObject **tb);
#else
static int __Pyx_GetException(PyObject **type, PyObject **value, PyObject **tb);
#endif

/* ListCompAppend.proto */
#if CYTHON_USE_PYLIST_INTERNALS && CYTHON_ASSUME_SAFE_MACROS
static CYTHON_INLINE int __Pyx_ListComp_Append(PyObject* list, PyObject* x) {
    PyListObject* L = (PyListObject*) list;
    Py_ssize_t len = Py_SIZE(list);
    if (likely(L->allocated > len)) {
        Py_INCREF(x);
        #if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x030d0000
        L->ob_item[len] = x;
        #else
        PyList_SET_ITEM(list, len, x);
        #endif
        __Pyx_SET_SIZE(list, len + 1);
        return 0;
    }
    return PyList_Append(list, x);
}
#else
#define __Pyx_ListComp_Append(L,x) PyList_Append(L,x)
#endif

/* ArgTypeTestFunc.export */
static int __Pyx__ArgTypeTest(PyObject *obj, PyTypeObject *type, const char *name, int exact);

/* ArgTypeTest.proto */
#define __Pyx_ArgTypeTest(obj, type, none_allowed, name, exact)\
    ((likely(__Pyx_IS_TYPE(obj, type) | (none_allowed && (obj == Py_None)))) ? 1 :\
        __Pyx__ArgTypeTest(obj, type, name, exact))

/* AssertionsEnabled.proto */
#if CYTHON_COMPILING_IN_LIMITED_API  ||  PY_VERSION_HEX >= 0x030C0000
  static int __pyx_assertions_enabled_flag;
  #define __pyx_assertions_enabled() (__pyx_assertions_enabled_flag)
  #if __clang__ || __GNUC__
  __attribute__((no_sanitize("thread")))
  #endif
  static int __Pyx_init_assertions_enabled(void) {
    PyObject *builtins, *debug, *debug_str;
    int flag;
    builtins = PyEval_GetBuiltins();
    if (!builtins) goto bad;
    debug_str = PyUnicode_FromStringAndSize("__debug__", 9);
    if (!debug_str) goto bad;
    debug = PyObject_GetItem(builtins, debug_str);
    Py_DECREF(debug_str);
    if (!debug) goto bad;
    flag = PyObject_IsTrue(debug);
    Py_DECREF(debug);
    if (flag == -1) goto bad;
    __pyx_assertions_enabled_flag = flag;
    return 0;
  bad:
    __pyx_assertions_enabled_flag = 1;
    return -1;
  }
#else
  #define __Pyx_init_assertions_enabled()  (0)
  #define __pyx_assertions_enabled()  (!Py_OptimizeFlag)
#endif

/* PyAssertionError_Check.proto */
#define __Pyx_PyExc_AssertionError_Check(obj)  __Pyx_TypeCheck(obj, PyExc_AssertionError)

/* RaiseTooManyValuesToUnpack.proto */
static CYTHON_INLINE void __Pyx_RaiseTooManyValuesError(Py_ssize_t expected);

/* RaiseNeedMoreValuesToUnpack.proto */
static CYTHON_INLINE void __Pyx_RaiseNeedMoreValuesError(Py_ssize_t index);

/* ExtTypeTest.proto */
static CYTHON_INLINE int __Pyx_TypeTest(PyObject *obj, PyTypeObject *type);

/* PyNumber_Hex.proto */
#define __Pyx_PyNumber_Hex(obj) PyNumber_ToBase((obj), 16)

/* PyObjectFormatSimple.proto */
#if CYTHON_COMPILING_IN_PYPY
    #define __Pyx_PyObject_FormatSimple(s, f) (\
        likely(PyUnicode_CheckExact(s)) ? (Py_INCREF(s), s) :\
        PyObject_Format(s, f))
#elif CYTHON_USE_TYPE_SLOTS
    #define __Pyx_PyObject_FormatSimple(s, f) (\
        likely(PyUnicode_CheckExact(s)) ? (Py_INCREF(s), s) :\
        likely(PyLong_CheckExact(s)) ? PyLong_Type.tp_repr(s) :\
        likely(PyFloat_CheckExact(s)) ? PyFloat_Type.tp_repr(s) :\
        PyObject_Format(s, f))
#else
    #define __Pyx_PyObject_FormatSimple(s, f) (\
        likely(PyUnicode_CheckExact(s)) ? (Py_INCREF(s), s) :\
        PyObject_Format(s, f))
#endif

/* PyUnicode_Unicode.proto */
static CYTHON_INLINE PyObject* __Pyx_PyUnicode_Unicode(PyObject *obj);

/* PyObjectFormatAndDecref.proto */
static CYTHON_INLINE PyObject* __Pyx_PyObject_FormatSimpleAndDecref(PyObject* s, PyObject* f);
static CYTHON_INLINE PyObject* __Pyx_PyObject_FormatAndDecref(PyObject* s, PyObject* f);

/* BuildPyUnicode.proto (used by COrdinalToPyUnicode) */
static PyObject* __Pyx_PyUnicode_BuildFromAscii(Py_ssize_t ulength, const char* chars, int clength,
                                                int prepend_sign, char padding_char);

/* COrdinalToPyUnicode.proto (used by CIntToPyUnicode) */
static CYTHON_INLINE int __Pyx_CheckUnicodeValue(int value);
static CYTHON_INLINE PyObject* __Pyx_PyUnicode_FromOrdinal_Padded(int value, Py_ssize_t width, char padding_char);

/* GCCDiagnostics.proto (used by CIntToPyUnicode) */
#if !defined(__INTEL_COMPILER) && defined(__GNUC__) && (__GNUC__ > 4 || (__GNUC__ == 4 && __GNUC_MINOR__ >= 6))
#define __Pyx_HAS_GCC_DIAGNOSTIC
#endif

/* IncludeStdlibH.proto (used by CIntToPyUnicode) */
#include <stdlib.h>

/* CIntToPyUnicode.proto */
#define __Pyx_PyUnicode_From_ptrdiff_t(value, width, padding_char, format_char) (\
    ((format_char) == ('c')) ?\
        __Pyx_uchar___Pyx_PyUnicode_From_ptrdiff_t(value, width, padding_char) :\
        __Pyx____Pyx_PyUnicode_From_ptrdiff_t(value, width, padding_char, format_char)\
    )
static CYTHON_INLINE PyObject* __Pyx_uchar___Pyx_PyUnicode_From_ptrdiff_t(ptrdiff_t value, Py_ssize_t width, char padding_char);
static CYTHON_INLINE PyObject* __Pyx____Pyx_PyUnicode_From_ptrdiff_t(ptrdiff_t value, Py_ssize_t width, char padding_char, char format_char);

/* JoinPyUnicode.export */
static PyObject* __Pyx_PyUnicode_Join(PyObject** values, Py_ssize_t value_count, Py_ssize_t result_ulength,
                                      Py_UCS4 max_char);

/* PyValueError_Check.proto */
#define __Pyx_PyExc_ValueError_Check(obj)  __Pyx_TypeCheck(obj, PyExc_ValueError)

/* IterFinish.proto (used by dict_iter) */
static CYTHON_INLINE int __Pyx_IterFinish(void);

/* PyObjectCallNoArg.proto (used by PyObjectCallMethod0) */
static CYTHON_INLINE PyObject* __Pyx_PyObject_CallNoArg(PyObject *func);

/* PyObjectGetMethod.proto (used by PyObjectCallMethod0) */
#if !(CYTHON_VECTORCALL && (__PYX_LIMITED_VERSION_HEX >= 0x030C0000 || (!CYTHON_COMPILING_IN_LIMITED_API && PY_VERSION_HEX >= 0x03090000)))
static int __Pyx_PyObject_GetMethod(PyObject *obj, PyObject *name, PyObject **method);
#endif

/* PyObjectCallMethod0.proto (used by dict_iter) */
static PyObject* __Pyx_PyObject_CallMethod0(PyObject* obj, PyObject* method_name);

/* UnpackItemEndCheck.proto (used by UnpackTuple2) */
static int __Pyx_IternextUnpackEndCheck(PyObject *retval, Py_ssize_t expected);

/* RaiseNoneIterError.proto (used by UnpackTupleError) */
static CYTHON_INLINE void __Pyx_RaiseNoneNotIterableError(void);

/* UnpackTupleError.proto (used by UnpackTuple2) */
static void __Pyx_UnpackTupleError(PyObject *, Py_ssize_t index);

/* UnpackTuple2.proto (used by dict_iter) */
static CYTHON_INLINE int __Pyx_unpack_tuple2(
    PyObject* tuple, PyObject** value1, PyObject** value2, int is_tuple, int has_known_size, int decref_tuple);
static CYTHON_INLINE int __Pyx_unpack_tuple2_exact(
    PyObject* tuple, PyObject** value1, PyObject** value2, int decref_tuple);
static int __Pyx_unpack_tuple2_generic(
    PyObject* tuple, PyObject** value1, PyObject** value2, int has_known_size, int decref_tuple);

/* dict_iter.proto */
static CYTHON_INLINE PyObject* __Pyx_dict_iterator(PyObject* dict, int is_dict, PyObject* method_name,
                                                   Py_ssize_t* p_orig_length, int* p_is_dict);
static CYTHON_INLINE int __Pyx_dict_iter_next(PyObject* dict_or_iter, Py_ssize_t orig_length, Py_ssize_t* ppos,
                                              PyObject** pkey, PyObject** pvalue, PyObject** pitem, int is_dict);

/* PyObjectVectorCallKwBuilder.proto (used by PyObjectVectorCallMethodKwBuilder) */
CYTHON_UNUSED static int __Pyx_VectorcallBuilder_AddArg_Check(PyObject *key, PyObject *value, PyObject *builder, PyObject **args, int n);
#if CYTHON_VECTORCALL
#if PY_VERSION_HEX >= 0x03090000
#define __Pyx_Object_Vectorcall_CallFromBuilder PyObject_Vectorcall
#else
#define __Pyx_Object_Vectorcall_CallFromBuilder _PyObject_Vectorcall
#endif
#define __Pyx_MakeVectorcallBuilderKwds(n) PyTuple_New(n)
static int __Pyx_VectorcallBuilder_AddArg(PyObject *key, PyObject *value, PyObject *builder, PyObject **args, int n);
static int __Pyx_VectorcallBuilder_AddArgStr(const char *key, PyObject *value, PyObject *builder, PyObject **args, int n);
#else
#define __Pyx_Object_Vectorcall_CallFromBuilder __Pyx_PyObject_FastCallDict
#define __Pyx_MakeVectorcallBuilderKwds(n) __Pyx_PyDict_NewPresized(n)
#define __Pyx_VectorcallBuilder_AddArg(key, value, builder, args, n) PyDict_SetItem(builder, key, value)
#define __Pyx_VectorcallBuilder_AddArgStr(key, value, builder, args, n) PyDict_SetItemString(builder, key, value)
#endif

/* PyObjectVectorCallMethodKwBuilder.proto */
#if CYTHON_VECTORCALL && PY_VERSION_HEX >= 0x03090000
#define __Pyx_Object_VectorcallMethod_CallFromBuilder PyObject_VectorcallMethod
#else
static PyObject *__Pyx_Object_VectorcallMethod_CallFromBuilder(PyObject *name, PyObject *const *args, size_t nargsf, PyObject *kwnames);
#endif

/* SwapException.proto */
#if CYTHON_FAST_THREAD_STATE
#define __Pyx_ExceptionSwap(type, value, tb)  __Pyx__ExceptionSwap(__pyx_tstate, type, value, tb)
static CYTHON_INLINE void __Pyx__ExceptionSwap(PyThreadState *tstate, PyObject **type, PyObject **value, PyObject **tb);
#else
static CYTHON_INLINE void __Pyx_ExceptionSwap(PyObject **type, PyObject **value, PyObject **tb);
#endif

/* ListAppend.proto */
#if CYTHON_USE_PYLIST_INTERNALS && CYTHON_ASSUME_SAFE_MACROS
static CYTHON_INLINE int __Pyx_PyList_Append(PyObject* list, PyObject* x) {
    PyListObject* L = (PyListObject*) list;
    Py_ssize_t len = Py_SIZE(list);
    if (likely(L->allocated > len) & likely(len > (L->allocated >> 1))) {
        Py_INCREF(x);
        #if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x030d0000
        L->ob_item[len] = x;
        #else
        PyList_SET_ITEM(list, len, x);
        #endif
        __Pyx_SET_SIZE(list, len + 1);
        return 0;
    }
    return PyList_Append(list, x);
}
#else
#define __Pyx_PyList_Append(L,x) PyList_Append(L,x)
#endif

/* pop.proto */
static CYTHON_INLINE PyObject* __Pyx__PyObject_Pop(PyObject* L);
#if CYTHON_USE_PYLIST_INTERNALS && CYTHON_ASSUME_SAFE_MACROS && CYTHON_ASSUME_SAFE_SIZE
static CYTHON_INLINE PyObject* __Pyx_PyList_Pop(PyObject* L);
#define __Pyx_PyObject_Pop(L) (likely(PyList_CheckExact(L)) ?\
    __Pyx_PyList_Pop(L) : __Pyx__PyObject_Pop(L))
#else
#define __Pyx_PyList_Pop(L)  __Pyx__PyObject_Pop(L)
#define __Pyx_PyObject_Pop(L)  __Pyx__PyObject_Pop(L)
#endif

/* PyTypeError_Check.proto */
#define __Pyx_PyExc_TypeError_Check(obj)  __Pyx_TypeCheck(obj, PyExc_TypeError)

/* pybytes_as_double.proto (used by pyobject_as_double) */
static double __Pyx_SlowPyString_AsDouble(PyObject *obj);
static double __Pyx__PyBytes_AsDouble(PyObject *obj, const char* start, Py_ssize_t length);
static CYTHON_INLINE double __Pyx_PyBytes_AsDouble(PyObject *obj) {
    char* as_c_string;
    Py_ssize_t size;
#if CYTHON_ASSUME_SAFE_MACROS && CYTHON_ASSUME_SAFE_SIZE
    as_c_string = PyBytes_AS_STRING(obj);
    size = PyBytes_GET_SIZE(obj);
#else
    if (PyBytes_AsStringAndSize(obj, &as_c_string, &size) < 0) {
        return (double)-1;
    }
#endif
    return __Pyx__PyBytes_AsDouble(obj, as_c_string, size);
}
static CYTHON_INLINE double __Pyx_PyByteArray_AsDouble(PyObject *obj) {
    char* as_c_string;
    Py_ssize_t size;
#if CYTHON_ASSUME_SAFE_MACROS && CYTHON_ASSUME_SAFE_SIZE
    as_c_string = PyByteArray_AS_STRING(obj);
    size = PyByteArray_GET_SIZE(obj);
#else
    as_c_string = PyByteArray_AsString(obj);
    if (as_c_string == NULL) {
        return (double)-1;
    }
    size = PyByteArray_Size(obj);
#endif
    return __Pyx__PyBytes_AsDouble(obj, as_c_string, size);
}

/* pyunicode_as_double.proto (used by pyobject_as_double) */
#if !CYTHON_COMPILING_IN_PYPY && CYTHON_ASSUME_SAFE_MACROS
static const char* __Pyx__PyUnicode_AsDouble_Copy(const void* data, const int kind, char* buffer, Py_ssize_t start, Py_ssize_t end) {
    int last_was_punctuation;
    Py_ssize_t i;
    last_was_punctuation = 1;
    for (i=start; i <= end; i++) {
        Py_UCS4 chr = PyUnicode_READ(kind, data, i);
        int is_punctuation = (chr == '_') | (chr == '.');
        *buffer = (char)chr;
        buffer += (chr != '_');
        if (unlikely(chr > 127)) goto parse_failure;
        if (unlikely(last_was_punctuation & is_punctuation)) goto parse_failure;
        last_was_punctuation = is_punctuation;
    }
    if (unlikely(last_was_punctuation)) goto parse_failure;
    *buffer = '\0';
    return buffer;
parse_failure:
    return NULL;
}
static double __Pyx__PyUnicode_AsDouble_inf_nan(const void* data, int kind, Py_ssize_t start, Py_ssize_t length) {
    int matches = 1;
    Py_UCS4 chr;
    Py_UCS4 sign = PyUnicode_READ(kind, data, start);
    int is_signed = (sign == '-') | (sign == '+');
    start += is_signed;
    length -= is_signed;
    switch (PyUnicode_READ(kind, data, start)) {
        #ifdef Py_NAN
        case 'n':
        case 'N':
            if (unlikely(length != 3)) goto parse_failure;
            chr = PyUnicode_READ(kind, data, start+1);
            matches &= (chr == 'a') | (chr == 'A');
            chr = PyUnicode_READ(kind, data, start+2);
            matches &= (chr == 'n') | (chr == 'N');
            if (unlikely(!matches)) goto parse_failure;
            return (sign == '-') ? -Py_NAN : Py_NAN;
        #endif
        case 'i':
        case 'I':
            if (unlikely(length < 3)) goto parse_failure;
            chr = PyUnicode_READ(kind, data, start+1);
            matches &= (chr == 'n') | (chr == 'N');
            chr = PyUnicode_READ(kind, data, start+2);
            matches &= (chr == 'f') | (chr == 'F');
            if (likely(length == 3 && matches))
                return (sign == '-') ? -Py_HUGE_VAL : Py_HUGE_VAL;
            if (unlikely(length != 8)) goto parse_failure;
            chr = PyUnicode_READ(kind, data, start+3);
            matches &= (chr == 'i') | (chr == 'I');
            chr = PyUnicode_READ(kind, data, start+4);
            matches &= (chr == 'n') | (chr == 'N');
            chr = PyUnicode_READ(kind, data, start+5);
            matches &= (chr == 'i') | (chr == 'I');
            chr = PyUnicode_READ(kind, data, start+6);
            matches &= (chr == 't') | (chr == 'T');
            chr = PyUnicode_READ(kind, data, start+7);
            matches &= (chr == 'y') | (chr == 'Y');
            if (unlikely(!matches)) goto parse_failure;
            return (sign == '-') ? -Py_HUGE_VAL : Py_HUGE_VAL;
        case '.': case '0': case '1': case '2': case '3': case '4': case '5': case '6': case '7': case '8': case '9':
            break;
        default:
            goto parse_failure;
    }
    return 0.0;
parse_failure:
    return -1.0;
}
static double __Pyx_PyUnicode_AsDouble_WithSpaces(PyObject *obj) {
    double value;
    const char *last;
    char *end;
    Py_ssize_t start, length = PyUnicode_GET_LENGTH(obj);
    const int kind = PyUnicode_KIND(obj);
    const void* data = PyUnicode_DATA(obj);
    start = 0;
    while (Py_UNICODE_ISSPACE(PyUnicode_READ(kind, data, start)))
        start++;
    while (start < length - 1 && Py_UNICODE_ISSPACE(PyUnicode_READ(kind, data, length - 1)))
        length--;
    length -= start;
    if (unlikely(length <= 0)) goto fallback;
    value = __Pyx__PyUnicode_AsDouble_inf_nan(data, kind, start, length);
    if (unlikely(value == -1.0)) goto fallback;
    if (value != 0.0) return value;
    if (length < 40) {
        char number[40];
        last = __Pyx__PyUnicode_AsDouble_Copy(data, kind, number, start, start + length);
        if (unlikely(!last)) goto fallback;
        value = PyOS_string_to_double(number, &end, NULL);
    } else {
        char *number = (char*) PyMem_Malloc((length + 1) * sizeof(char));
        if (unlikely(!number)) goto fallback;
        last = __Pyx__PyUnicode_AsDouble_Copy(data, kind, number, start, start + length);
        if (unlikely(!last)) {
            PyMem_Free(number);
            goto fallback;
        }
        value = PyOS_string_to_double(number, &end, NULL);
        PyMem_Free(number);
    }
    if (likely(end == last) || (value == (double)-1 && PyErr_Occurred())) {
        return value;
    }
fallback:
    return __Pyx_SlowPyString_AsDouble(obj);
}
#endif
static CYTHON_INLINE double __Pyx_PyUnicode_AsDouble(PyObject *obj) {
#if !CYTHON_COMPILING_IN_PYPY && CYTHON_ASSUME_SAFE_MACROS
    if (unlikely(__Pyx_PyUnicode_READY(obj) == -1))
        return (double)-1;
    if (likely(PyUnicode_IS_ASCII(obj))) {
        const char *s;
        Py_ssize_t length;
        s = PyUnicode_AsUTF8AndSize(obj, &length);
        return __Pyx__PyBytes_AsDouble(obj, s, length);
    }
    return __Pyx_PyUnicode_AsDouble_WithSpaces(obj);
#else
    return __Pyx_SlowPyString_AsDouble(obj);
#endif
}

/* pyobject_as_double.proto */
static double __Pyx__PyObject_AsDouble(PyObject* obj);
#if CYTHON_COMPILING_IN_PYPY
#define __Pyx_PyObject_AsDouble(obj)\
(likely(PyFloat_CheckExact(obj)) ? PyFloat_AS_DOUBLE(obj) :\
 likely(PyLong_CheckExact(obj)) ?\
 PyFloat_AsDouble(obj) : __Pyx__PyObject_AsDouble(obj))
#else
#define __Pyx_PyObject_AsDouble(obj)\
((likely(PyFloat_CheckExact(obj))) ?  __Pyx_PyFloat_AS_DOUBLE(obj) :\
 likely(PyLong_CheckExact(obj)) ?\
 PyLong_AsDouble(obj) : __Pyx__PyObject_AsDouble(obj))
#endif

/* SliceObject.proto */
static CYTHON_INLINE PyObject* __Pyx_PyObject_GetSlice(
        PyObject* obj, Py_ssize_t cstart, Py_ssize_t cstop,
        PyObject** py_start, PyObject** py_stop, PyObject** py_slice,
        int has_cstart, int has_cstop, int wraparound);

/* FloatExceptionCheck.proto */
#define __PYX_CHECK_FLOAT_EXCEPTION(value, error_value)\
    ((error_value) == (error_value) ?\
     (value) == (error_value) :\
     (value) != (value))

/* dict_getitem_default.proto */
static PyObject* __Pyx_PyDict_GetItemDefault(PyObject* d, PyObject* key, PyObject* default_value);

/* PyObjectCall2Args.proto (used by CallUnboundCMethod1) */
static CYTHON_INLINE PyObject* __Pyx_PyObject_Call2Args(PyObject* function, PyObject* arg1, PyObject* arg2);

/* CallUnboundCMethod1.proto */
CYTHON_UNUSED
static PyObject* __Pyx__CallUnboundCMethod1(__Pyx_CachedCFunction* cfunc, PyObject* self, PyObject* arg);
#if CYTHON_COMPILING_IN_CPYTHON
static CYTHON_INLINE PyObject* __Pyx_CallUnboundCMethod1(__Pyx_CachedCFunction* cfunc, PyObject* self, PyObject* arg);
#else
#define __Pyx_CallUnboundCMethod1(cfunc, self, arg)  __Pyx__CallUnboundCMethod1(cfunc, self, arg)
#endif

/* DictGetItem.proto */
#if !CYTHON_COMPILING_IN_PYPY
static PyObject *__Pyx_PyDict_GetItem(PyObject *d, PyObject* key);
#define __Pyx_PyObject_Dict_GetItem(obj, name)\
    (likely(PyDict_CheckExact(obj)) ?\
     __Pyx_PyDict_GetItem(obj, name) : PyObject_GetItem(obj, name))
#else
#define __Pyx_PyDict_GetItem(d, key) PyObject_GetItem(d, key)
#define __Pyx_PyObject_Dict_GetItem(obj, name)  PyObject_GetItem(obj, name)
#endif

/* py_dict_keys.proto */
static CYTHON_INLINE PyObject* __Pyx_PyDict_Keys(PyObject* d);

/* PyDictContains.proto */
static CYTHON_INLINE int __Pyx_PyDict_ContainsTF(PyObject* item, PyObject* dict, int eq) {
    int result = PyDict_Contains(dict, item);
    return unlikely(result < 0) ? result : (result == (eq == Py_EQ));
}

/* PyDeprecationWarning_Check.proto */
#define __Pyx_PyExc_DeprecationWarning_Check(obj)  __Pyx_TypeCheck(obj, PyExc_DeprecationWarning)

/* FastTypeChecks.proto */
#if CYTHON_COMPILING_IN_CPYTHON
#define __Pyx_TypeCheck(obj, type) __Pyx_IsSubtype(Py_TYPE(obj), (PyTypeObject *)type)
#define __Pyx_TypeCheck2(obj, type1, type2) __Pyx_IsAnySubtype2(Py_TYPE(obj), (PyTypeObject *)type1, (PyTypeObject *)type2)
static CYTHON_INLINE int __Pyx_IsSubtype(PyTypeObject *a, PyTypeObject *b);
static CYTHON_INLINE int __Pyx_IsAnySubtype2(PyTypeObject *cls, PyTypeObject *a, PyTypeObject *b);
static CYTHON_INLINE int __Pyx_PyErr_GivenExceptionMatches(PyObject *err, PyObject *type);
static CYTHON_INLINE int __Pyx_PyErr_GivenExceptionMatches2(PyObject *err, PyObject *type1, PyObject *type2);
#else
#define __Pyx_TypeCheck(obj, type) PyObject_TypeCheck(obj, (PyTypeObject *)type)
#define __Pyx_TypeCheck2(obj, type1, type2) (PyObject_TypeCheck(obj, (PyTypeObject *)type1) || PyObject_TypeCheck(obj, (PyTypeObject *)type2))
#define __Pyx_PyErr_GivenExceptionMatches(err, type) PyErr_GivenExceptionMatches(err, type)
static CYTHON_INLINE int __Pyx_PyErr_GivenExceptionMatches2(PyObject *err, PyObject *type1, PyObject *type2) {
    return PyErr_GivenExceptionMatches(err, type1) || PyErr_GivenExceptionMatches(err, type2);
}
#endif
#define __Pyx_PyErr_ExceptionMatches2(err1, err2)  __Pyx_PyErr_GivenExceptionMatches2(__Pyx_PyErr_CurrentExceptionType(), err1, err2)
#define __Pyx_PyException_Check(obj) __Pyx_TypeCheck(obj, PyExc_Exception)
#ifdef PyExceptionInstance_Check
  #define __Pyx_PyBaseException_Check(obj) PyExceptionInstance_Check(obj)
#else
  #define __Pyx_PyBaseException_Check(obj) __Pyx_TypeCheck(obj, PyExc_BaseException)
#endif

/* RaiseClosureNameError.proto */
static void __Pyx_RaiseClosureNameError(const char *varname);

/* pep479.proto */
static void __Pyx_Generator_Replace_StopIteration(int in_async_gen);

/* RaiseUnboundLocalError.proto */
static void __Pyx_RaiseUnboundLocalError(const char *varname);

/* PyNotImplementedError_Check.proto */
#define __Pyx_PyExc_NotImplementedError_Check(obj)  __Pyx_TypeCheck(obj, PyExc_NotImplementedError)

/* AllocateExtensionType.proto */
static PyObject *__Pyx_AllocateExtensionType(PyTypeObject *t, int is_final);

/* CallTypeTraverse.proto */
#if !CYTHON_USE_TYPE_SPECS || (!CYTHON_COMPILING_IN_LIMITED_API && PY_VERSION_HEX < 0x03090000)
#define __Pyx_call_type_traverse(o, always_call, visit, arg) 0
#else
static int __Pyx_call_type_traverse(PyObject *o, int always_call, visitproc visit, void *arg);
#endif

/* CallNextTpTraverse.proto */
static int __Pyx_call_next_tp_traverse(PyObject* obj, visitproc v, void *a, traverseproc current_tp_traverse);

/* CallNextTpClear.proto */
static void __Pyx_call_next_tp_clear(PyObject* obj, inquiry current_tp_clear);

/* DefaultPlacementNew.proto */
#include <new>
template<typename T>
void __Pyx_default_placement_construct(T* x) {
    new (static_cast<void*>(x)) T();
}

/* CheckTypeForFreelists.proto */
#if CYTHON_USE_FREELISTS
#if CYTHON_USE_TYPE_SPECS
#define __PYX_CHECK_FINAL_TYPE_FOR_FREELISTS(t, expected_tp, expected_size) ((int) ((t) == (expected_tp)))
#define __PYX_CHECK_TYPE_FOR_FREELIST_FLAGS  Py_TPFLAGS_IS_ABSTRACT
#else
#define __PYX_CHECK_FINAL_TYPE_FOR_FREELISTS(t, expected_tp, expected_size) ((int) ((t)->tp_basicsize == (expected_size)))
#define __PYX_CHECK_TYPE_FOR_FREELIST_FLAGS  (Py_TPFLAGS_IS_ABSTRACT | Py_TPFLAGS_HEAPTYPE)
#endif
#define __PYX_CHECK_TYPE_FOR_FREELISTS(t, expected_tp, expected_size)\
    (__PYX_CHECK_FINAL_TYPE_FOR_FREELISTS((t), (expected_tp), (expected_size)) &\
     (int) (!__Pyx_PyType_HasFeature((t), __PYX_CHECK_TYPE_FOR_FREELIST_FLAGS)))
#endif

/* FunctionExport.proto */
static int __Pyx_ExportFunction(PyObject *api_dict, const char *name, void (*f)(void), const char *sig);

/* GetApiDict.proto */
static PyObject *__Pyx_ApiExport_GetApiDict(void);

/* LimitedApiGetTypeDict.proto (used by SetItemOnTypeDict) */
#if CYTHON_COMPILING_IN_LIMITED_API
static PyObject *__Pyx_GetTypeDict(PyTypeObject *tp);
#endif

/* SetItemOnTypeDict.proto (used by FixUpExtensionType) */
static int __Pyx__SetItemOnTypeDict(PyTypeObject *tp, PyObject *k, PyObject *v);
#define __Pyx_SetItemOnTypeDict(tp, k, v) __Pyx__SetItemOnTypeDict((PyTypeObject*)tp, k, v)

/* FixUpExtensionType.proto */
static CYTHON_INLINE int __Pyx_fix_up_extension_type_from_spec(PyType_Spec *spec, PyTypeObject *type);

/* ValidateBasesTuple.proto (used by PyType_Ready) */
#if CYTHON_COMPILING_IN_CPYTHON || CYTHON_COMPILING_IN_LIMITED_API || CYTHON_USE_TYPE_SPECS
static int __Pyx_validate_bases_tuple(const char *type_name, Py_ssize_t dictoffset, PyObject *bases);
#endif

/* PyType_Ready.proto */
CYTHON_UNUSED static int __Pyx_PyType_Ready(PyTypeObject *t);

/* DelItemOnTypeDict.proto (used by SetupReduce) */
static int __Pyx__DelItemOnTypeDict(PyTypeObject *tp, PyObject *k);
#define __Pyx_DelItemOnTypeDict(tp, k) __Pyx__DelItemOnTypeDict((PyTypeObject*)tp, k)

/* SetupReduce.proto */
static int __Pyx_setup_reduce(PyObject* type_obj);

/* SetVTable.proto */
static int __Pyx_SetVtable(PyTypeObject* typeptr , void* vtable);

/* GetVTable.proto (used by MergeVTables) */
static void* __Pyx_GetVtable(PyTypeObject *type);

/* MergeVTables.proto */
static int __Pyx_MergeVtables(PyTypeObject *type);

/* TypeImport.proto */
#ifndef __PYX_HAVE_RT_ImportType_proto_3_2_5
#define __PYX_HAVE_RT_ImportType_proto_3_2_5
#if defined (__STDC_VERSION__) && __STDC_VERSION__ >= 201112L
#include <stdalign.h>
#endif
#if (defined (__STDC_VERSION__) && __STDC_VERSION__ >= 201112L) || __cplusplus >= 201103L
#define __PYX_GET_STRUCT_ALIGNMENT_3_2_5(s) alignof(s)
#else
#define __PYX_GET_STRUCT_ALIGNMENT_3_2_5(s) sizeof(void*)
#endif
enum __Pyx_ImportType_CheckSize_3_2_5 {
   __Pyx_ImportType_CheckSize_Error_3_2_5 = 0,
   __Pyx_ImportType_CheckSize_Warn_3_2_5 = 1,
   __Pyx_ImportType_CheckSize_Ignore_3_2_5 = 2
};
static PyTypeObject *__Pyx_ImportType_3_2_5(PyObject* module, const char *module_name, const char *class_name, size_t size, size_t alignment, enum __Pyx_ImportType_CheckSize_3_2_5 check_size);
#endif

/* VoidPtrImport.proto */
static int __Pyx_ImportVoidPtr_3_2_5(PyObject *module, const char *name, void **p, const char *sig);

/* FunctionImport.proto */
static int __Pyx_ImportFunction_3_2_5(PyObject *module, const char *funcname, void (**f)(void), const char *sig);

/* HasAttr.proto (used by ImportImpl) */
#if __PYX_LIMITED_VERSION_HEX >= 0x030d0000
#define __Pyx_HasAttr(o, n)  PyObject_HasAttrWithError(o, n)
#else
static CYTHON_INLINE int __Pyx_HasAttr(PyObject *, PyObject *);
#endif

/* ImportImpl.export */
static PyObject *__Pyx__Import(PyObject *name, PyObject *const *imported_names, Py_ssize_t len_imported_names, PyObject *qualname, PyObject *moddict, int level);

/* Import.proto */
static CYTHON_INLINE PyObject *__Pyx_Import(PyObject *name, PyObject *const *imported_names, Py_ssize_t len_imported_names, PyObject *qualname, int level);

/* ImportFrom.proto */
static PyObject* __Pyx_ImportFrom(PyObject* module, PyObject* name);

/* PyLongCompare.proto */
static CYTHON_INLINE PyObject* __Pyx_PyLong_NeObjC(PyObject *op1, PyObject *op2, long intval, long inplace);

/* PyMemoryError_Check.proto */
#define __Pyx_PyExc_MemoryError_Check(obj)  __Pyx_TypeCheck(obj, PyExc_MemoryError)

/* Py3UpdateBases.proto */
static PyObject* __Pyx_PEP560_update_bases(PyObject *bases);

/* CalculateMetaclass.proto */
static PyObject *__Pyx_CalculateMetaclass(PyTypeObject *metaclass, PyObject *bases);

/* dict_setdefault.proto (used by FetchCommonType) */
static CYTHON_INLINE PyObject *__Pyx_PyDict_SetDefault(PyObject *d, PyObject *key, PyObject *default_value);

/* AddModuleRef.proto (used by FetchSharedCythonModule) */
#if ((CYTHON_COMPILING_IN_CPYTHON_FREETHREADING ) ||\
     __PYX_LIMITED_VERSION_HEX < 0x030d0000)
  static PyObject *__Pyx_PyImport_AddModuleRef(const char *name);
#else
  #define __Pyx_PyImport_AddModuleRef(name) PyImport_AddModuleRef(name)
#endif

/* FetchSharedCythonModule.proto (used by FetchCommonType) */
static PyObject *__Pyx_FetchSharedCythonABIModule(void);

/* FetchCommonType.proto (used by CommonTypesMetaclass) */
static PyTypeObject* __Pyx_FetchCommonTypeFromSpec(PyTypeObject *metaclass, PyObject *module, PyType_Spec *spec, PyObject *bases);

/* CommonTypesMetaclass.proto (used by CythonFunctionShared) */
static int __pyx_CommonTypesMetaclass_init(PyObject *module);
#define __Pyx_CommonTypesMetaclass_USED

/* PyMethodNew.proto (used by CythonFunctionShared) */
static PyObject *__Pyx_PyMethod_New(PyObject *func, PyObject *self, PyObject *typ);

/* PyVectorcallFastCallDict.proto (used by CythonFunctionShared) */
#if CYTHON_METH_FASTCALL && CYTHON_VECTORCALL
static CYTHON_INLINE PyObject *__Pyx_PyVectorcall_FastCallDict(PyObject *func, __pyx_vectorcallfunc vc, PyObject *const *args, size_t nargs, PyObject *kw);
#endif

/* CythonFunctionShared.proto (used by CythonFunction) */
#define __Pyx_CyFunction_USED
#define __Pyx_CYFUNCTION_STATICMETHOD  0x01
#define __Pyx_CYFUNCTION_CLASSMETHOD   0x02
#define __Pyx_CYFUNCTION_CCLASS        0x04
#define __Pyx_CYFUNCTION_COROUTINE     0x08
#define __Pyx_CyFunction_GetClosure(f)\
    (((__pyx_CyFunctionObject *) (f))->func_closure)
#if PY_VERSION_HEX < 0x030900B1 || CYTHON_COMPILING_IN_LIMITED_API
  #define __Pyx_CyFunction_GetClassObj(f)\
      (((__pyx_CyFunctionObject *) (f))->func_classobj)
#else
  #define __Pyx_CyFunction_GetClassObj(f)\
      ((PyObject*) ((PyCMethodObject *) (f))->mm_class)
#endif
#define __Pyx_CyFunction_SetClassObj(f, classobj)\
    __Pyx__CyFunction_SetClassObj((__pyx_CyFunctionObject *) (f), (classobj))
#define __Pyx_CyFunction_Defaults(type, f)\
    ((type *)(((__pyx_CyFunctionObject *) (f))->defaults))
#define __Pyx_CyFunction_SetDefaultsGetter(f, g)\
    ((__pyx_CyFunctionObject *) (f))->defaults_getter = (g)
typedef struct {
#if CYTHON_COMPILING_IN_LIMITED_API
    PyObject_HEAD
    PyObject *func;
#elif PY_VERSION_HEX < 0x030900B1
    PyCFunctionObject func;
#else
    PyCMethodObject func;
#endif
#if CYTHON_COMPILING_IN_LIMITED_API && CYTHON_METH_FASTCALL
    __pyx_vectorcallfunc func_vectorcall;
#endif
#if CYTHON_COMPILING_IN_LIMITED_API
    PyObject *func_weakreflist;
#endif
#if PY_VERSION_HEX < 0x030C0000 || CYTHON_COMPILING_IN_LIMITED_API
    PyObject *func_dict;
#endif
    PyObject *func_name;
    PyObject *func_qualname;
    PyObject *func_doc;
    PyObject *func_globals;
    PyObject *func_code;
    PyObject *func_closure;
#if PY_VERSION_HEX < 0x030900B1 || CYTHON_COMPILING_IN_LIMITED_API
    PyObject *func_classobj;
#endif
    PyObject *defaults;
    int flags;
    PyObject *defaults_tuple;
    PyObject *defaults_kwdict;
    PyObject *(*defaults_getter)(PyObject *);
    PyObject *func_annotations;
    PyObject *func_is_coroutine;
} __pyx_CyFunctionObject;
#undef __Pyx_CyOrPyCFunction_Check
#define __Pyx_CyFunction_Check(obj)  __Pyx_TypeCheck(obj, __pyx_mstate_global->__pyx_CyFunctionType)
#define __Pyx_CyOrPyCFunction_Check(obj)  __Pyx_TypeCheck2(obj, __pyx_mstate_global->__pyx_CyFunctionType, &PyCFunction_Type)
#define __Pyx_CyFunction_CheckExact(obj)  __Pyx_IS_TYPE(obj, __pyx_mstate_global->__pyx_CyFunctionType)
static CYTHON_INLINE int __Pyx__IsSameCyOrCFunction(PyObject *func, void (*cfunc)(void));
#undef __Pyx_IsSameCFunction
#define __Pyx_IsSameCFunction(func, cfunc)   __Pyx__IsSameCyOrCFunction(func, cfunc)
static PyObject *__Pyx_CyFunction_Init(__pyx_CyFunctionObject* op, PyMethodDef *ml,
                                      int flags, PyObject* qualname,
                                      PyObject *closure,
                                      PyObject *module, PyObject *globals,
                                      PyObject* code);
static CYTHON_INLINE void __Pyx__CyFunction_SetClassObj(__pyx_CyFunctionObject* f, PyObject* classobj);
static CYTHON_INLINE PyObject *__Pyx_CyFunction_InitDefaults(PyObject *func,
                                                         PyTypeObject *defaults_type);
static CYTHON_INLINE void __Pyx_CyFunction_SetDefaultsTuple(PyObject *m,
                                                            PyObject *tuple);
static CYTHON_INLINE void __Pyx_CyFunction_SetDefaultsKwDict(PyObject *m,
                                                             PyObject *dict);
static CYTHON_INLINE void __Pyx_CyFunction_SetAnnotationsDict(PyObject *m,
                                                              PyObject *dict);
static int __pyx_CyFunction_init(PyObject *module);
#if CYTHON_METH_FASTCALL
static PyObject * __Pyx_CyFunction_Vectorcall_NOARGS(PyObject *func, PyObject *const *args, size_t nargsf, PyObject *kwnames);
static PyObject * __Pyx_CyFunction_Vectorcall_O(PyObject *func, PyObject *const *args, size_t nargsf, PyObject *kwnames);
static PyObject * __Pyx_CyFunction_Vectorcall_FASTCALL_KEYWORDS(PyObject *func, PyObject *const *args, size_t nargsf, PyObject *kwnames);
static PyObject * __Pyx_CyFunction_Vectorcall_FASTCALL_KEYWORDS_METHOD(PyObject *func, PyObject *const *args, size_t nargsf, PyObject *kwnames);
#if CYTHON_COMPILING_IN_LIMITED_API
#define __Pyx_CyFunction_func_vectorcall(f) (((__pyx_CyFunctionObject*)f)->func_vectorcall)
#else
#define __Pyx_CyFunction_func_vectorcall(f) (((PyCFunctionObject*)f)->vectorcall)
#endif
#endif

/* CythonFunction.proto */
static PyObject *__Pyx_CyFunction_New(PyMethodDef *ml,
                                      int flags, PyObject* qualname,
                                      PyObject *closure,
                                      PyObject *module, PyObject *globals,
                                      PyObject* code);

/* SetNameInClass.proto */
#if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX < 0x030d0000
#define __Pyx_SetNameInClass(ns, name, value)\
    (likely(PyDict_CheckExact(ns)) ? _PyDict_SetItem_KnownHash(ns, name, value, ((PyASCIIObject *) name)->hash) : PyObject_SetItem(ns, name, value))
#elif CYTHON_COMPILING_IN_CPYTHON
#define __Pyx_SetNameInClass(ns, name, value)\
    (likely(PyDict_CheckExact(ns)) ? PyDict_SetItem(ns, name, value) : PyObject_SetItem(ns, name, value))
#else
#define __Pyx_SetNameInClass(ns, name, value)  PyObject_SetItem(ns, name, value)
#endif

/* PyObjectLookupSpecial.proto (used by Py3ClassCreate) */
#if CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
#define __Pyx_PyObject_LookupSpecialNoError(obj, attr_name)  __Pyx__PyObject_LookupSpecial(obj, attr_name, 0)
#define __Pyx_PyObject_LookupSpecial(obj, attr_name)  __Pyx__PyObject_LookupSpecial(obj, attr_name, 1)
static CYTHON_INLINE PyObject* __Pyx__PyObject_LookupSpecial(PyObject* obj, PyObject* attr_name, int with_error);
#else
#define __Pyx_PyObject_LookupSpecialNoError(o,n) __Pyx_PyObject_GetAttrStrNoError(o,n)
#define __Pyx_PyObject_LookupSpecial(o,n) __Pyx_PyObject_GetAttrStr(o,n)
#endif

/* Py3ClassCreate.proto */
static PyObject *__Pyx_Py3MetaclassPrepare(PyObject *metaclass, PyObject *bases, PyObject *name, PyObject *qualname,
                                           PyObject *mkw, PyObject *modname, PyObject *doc);
static PyObject *__Pyx_Py3ClassCreate(PyObject *metaclass, PyObject *name, PyObject *bases, PyObject *dict,
                                      PyObject *mkw, int calculate_metaclass, int allow_py2_metaclass);

/* CLineInTraceback.proto (used by AddTraceback) */
#if CYTHON_CLINE_IN_TRACEBACK && CYTHON_CLINE_IN_TRACEBACK_RUNTIME
static int __Pyx_CLineForTraceback(PyThreadState *tstate, int c_line);
#else
#define __Pyx_CLineForTraceback(tstate, c_line)  (((CYTHON_CLINE_IN_TRACEBACK)) ? c_line : 0)
#endif

/* CodeObjectCache.proto (used by AddTraceback) */
#if CYTHON_COMPILING_IN_LIMITED_API
typedef PyObject __Pyx_CachedCodeObjectType;
#else
typedef PyCodeObject __Pyx_CachedCodeObjectType;
#endif
typedef struct {
    __Pyx_CachedCodeObjectType* code_object;
    int code_line;
} __Pyx_CodeObjectCacheEntry;
struct __Pyx_CodeObjectCache {
    int count;
    int max_count;
    __Pyx_CodeObjectCacheEntry* entries;
  #if CYTHON_COMPILING_IN_CPYTHON_FREETHREADING
    __pyx_atomic_int_type accessor_count;
  #endif
};
static int __pyx_bisect_code_objects(__Pyx_CodeObjectCacheEntry* entries, int count, int code_line);
static __Pyx_CachedCodeObjectType *__pyx_find_code_object(int code_line);
static void __pyx_insert_code_object(int code_line, __Pyx_CachedCodeObjectType* code_object);

/* AddTraceback.proto */
static void __Pyx_AddTraceback(const char *funcname, int c_line,
                               int py_line, const char *filename);

/* CheckUnpickleChecksum.proto */
static CYTHON_INLINE int __Pyx_CheckUnpickleChecksum(long checksum, long checksum1, long checksum2, long checksum3, const char *members);

/* CppExceptionConversion.proto */
#ifndef __Pyx_CppExn2PyErr
#include <new>
#include <typeinfo>
#include <stdexcept>
#include <ios>
static void __Pyx_CppExn2PyErr() {
  try {
    if (PyErr_Occurred())
      ; // let the latest Python exn pass through and ignore the current one
    else
      throw;
  } catch (const std::bad_alloc& exn) {
    PyErr_SetString(PyExc_MemoryError, exn.what());
  } catch (const std::bad_cast& exn) {
    PyErr_SetString(PyExc_TypeError, exn.what());
  } catch (const std::bad_typeid& exn) {
    PyErr_SetString(PyExc_TypeError, exn.what());
  } catch (const std::domain_error& exn) {
    PyErr_SetString(PyExc_ValueError, exn.what());
  } catch (const std::invalid_argument& exn) {
    PyErr_SetString(PyExc_ValueError, exn.what());
  } catch (const std::ios_base::failure& exn) {
    PyErr_SetString(PyExc_IOError, exn.what());
  } catch (const std::out_of_range& exn) {
    PyErr_SetString(PyExc_IndexError, exn.what());
  } catch (const std::overflow_error& exn) {
    PyErr_SetString(PyExc_OverflowError, exn.what());
  } catch (const std::range_error& exn) {
    PyErr_SetString(PyExc_ArithmeticError, exn.what());
  } catch (const std::underflow_error& exn) {
    PyErr_SetString(PyExc_ArithmeticError, exn.what());
  } catch (const std::exception& exn) {
    PyErr_SetString(PyExc_RuntimeError, exn.what());
  }
  catch (...)
  {
    PyErr_SetString(PyExc_RuntimeError, "Unknown exception");
  }
}
#endif

/* CIntToPy.proto */
static CYTHON_INLINE PyObject* __Pyx_PyLong_From_int(int value);

/* CIntFromPy.proto */
static CYTHON_INLINE int __Pyx_PyLong_As_int(PyObject *);

/* CIntFromPy.proto */
static CYTHON_INLINE size_t __Pyx_PyLong_As_size_t(PyObject *);

/* CIntToPy.proto */
static CYTHON_INLINE PyObject* __Pyx_PyLong_From_ptrdiff_t(ptrdiff_t value);

/* CIntFromPy.proto */
static CYTHON_INLINE ptrdiff_t __Pyx_PyLong_As_ptrdiff_t(PyObject *);

/* CIntFromPy.proto */
static CYTHON_INLINE long __Pyx_PyLong_As_long(PyObject *);

/* CIntToPy.proto */
static CYTHON_INLINE PyObject* __Pyx_PyLong_From_uint64_t(uint64_t value);

/* CIntToPy.proto */
static CYTHON_INLINE PyObject* __Pyx_PyLong_From_long(long value);

/* PyObjectCallMethod1.proto */
static PyObject* __Pyx_PyObject_CallMethod1(PyObject* obj, PyObject* method_name, PyObject* arg);

/* UpdateUnpickledDict.proto */
static int __Pyx_UpdateUnpickledDict(PyObject *obj, PyObject *state, Py_ssize_t index);

/* FormatTypeName.proto */
#if CYTHON_COMPILING_IN_LIMITED_API
typedef PyObject *__Pyx_TypeName;
#define __Pyx_FMT_TYPENAME "%U"
#define __Pyx_DECREF_TypeName(obj) Py_XDECREF(obj)
#if __PYX_LIMITED_VERSION_HEX >= 0x030d0000
#define __Pyx_PyType_GetFullyQualifiedName PyType_GetFullyQualifiedName
#else
static __Pyx_TypeName __Pyx_PyType_GetFullyQualifiedName(PyTypeObject* tp);
#endif
#else  // !LIMITED_API
typedef const char *__Pyx_TypeName;
#define __Pyx_FMT_TYPENAME "%.200s"
#define __Pyx_PyType_GetFullyQualifiedName(tp) ((tp)->tp_name)
#define __Pyx_DECREF_TypeName(obj)
#endif

/* GetRuntimeVersion.proto */
#if __PYX_LIMITED_VERSION_HEX < 0x030b0000
static unsigned long __Pyx_cached_runtime_version = 0;
static void __Pyx_init_runtime_version(void);
#else
#define __Pyx_init_runtime_version()
#endif
static unsigned long __Pyx_get_runtime_version(void);

/* IterNextPlain.proto (used by CoroutineBase) */
static CYTHON_INLINE PyObject *__Pyx_PyIter_Next_Plain(PyObject *iterator);
#if CYTHON_COMPILING_IN_LIMITED_API && __PYX_LIMITED_VERSION_HEX < 0x030A0000
static PyObject *__Pyx_GetBuiltinNext_LimitedAPI(void);
#endif

/* ReturnWithStopIteration.proto (used by CoroutineBase) */
static CYTHON_INLINE void __Pyx_ReturnWithStopIteration(PyObject* value, int async, int iternext);

/* CoroutineBase.proto (used by Generator) */
struct __pyx_CoroutineObject;
typedef PyObject *(*__pyx_coroutine_body_t)(struct __pyx_CoroutineObject *, PyThreadState *, PyObject *);
#if CYTHON_USE_EXC_INFO_STACK
#define __Pyx_ExcInfoStruct  _PyErr_StackItem
#else
typedef struct {
    PyObject *exc_type;
    PyObject *exc_value;
    PyObject *exc_traceback;
} __Pyx_ExcInfoStruct;
#endif
typedef struct __pyx_CoroutineObject {
    PyObject_HEAD
    __pyx_coroutine_body_t body;
    PyObject *closure;
    __Pyx_ExcInfoStruct gi_exc_state;
#if PY_VERSION_HEX < 0x030C0000 || CYTHON_COMPILING_IN_LIMITED_API
    PyObject *gi_weakreflist;
#endif
    PyObject *classobj;
    PyObject *yieldfrom;
    __Pyx_pyiter_sendfunc yieldfrom_am_send;
    PyObject *gi_name;
    PyObject *gi_qualname;
    PyObject *gi_modulename;
    PyObject *gi_code;
    PyObject *gi_frame;
#if CYTHON_USE_SYS_MONITORING && (CYTHON_PROFILE || CYTHON_TRACE)
    PyMonitoringState __pyx_pymonitoring_state[__Pyx_MonitoringEventTypes_CyGen_count];
    uint64_t __pyx_pymonitoring_version;
#endif
    int resume_label;
    char is_running;
} __pyx_CoroutineObject;
static __pyx_CoroutineObject *__Pyx__Coroutine_New(
    PyTypeObject *type, __pyx_coroutine_body_t body, PyObject *code, PyObject *closure,
    PyObject *name, PyObject *qualname, PyObject *module_name);
static __pyx_CoroutineObject *__Pyx__Coroutine_NewInit(
            __pyx_CoroutineObject *gen, __pyx_coroutine_body_t body, PyObject *code, PyObject *closure,
            PyObject *name, PyObject *qualname, PyObject *module_name);
static CYTHON_INLINE void __Pyx_Coroutine_ExceptionClear(__Pyx_ExcInfoStruct *self);
static int __Pyx_Coroutine_clear(PyObject *self);
static __Pyx_PySendResult __Pyx_Coroutine_AmSend(PyObject *self, PyObject *value, PyObject **retval);
static PyObject *__Pyx_Coroutine_Send(PyObject *self, PyObject *value);
static __Pyx_PySendResult __Pyx_Coroutine_Close(PyObject *self, PyObject **retval);
static PyObject *__Pyx_Coroutine_Throw(PyObject *gen, PyObject *args);
#if CYTHON_USE_EXC_INFO_STACK
#define __Pyx_Coroutine_SwapException(self)
#define __Pyx_Coroutine_ResetAndClearException(self)  __Pyx_Coroutine_ExceptionClear(&(self)->gi_exc_state)
#else
#define __Pyx_Coroutine_SwapException(self) {\
    __Pyx_ExceptionSwap(&(self)->gi_exc_state.exc_type, &(self)->gi_exc_state.exc_value, &(self)->gi_exc_state.exc_traceback);\
    __Pyx_Coroutine_ResetFrameBackpointer(&(self)->gi_exc_state);\
    }
#define __Pyx_Coroutine_ResetAndClearException(self) {\
    __Pyx_ExceptionReset((self)->gi_exc_state.exc_type, (self)->gi_exc_state.exc_value, (self)->gi_exc_state.exc_traceback);\
    (self)->gi_exc_state.exc_type = (self)->gi_exc_state.exc_value = (self)->gi_exc_state.exc_traceback = NULL;\
    }
#endif
#if CYTHON_FAST_THREAD_STATE
#define __Pyx_PyGen_FetchStopIterationValue(pvalue)\
    __Pyx_PyGen__FetchStopIterationValue(__pyx_tstate, pvalue)
#else
#define __Pyx_PyGen_FetchStopIterationValue(pvalue)\
    __Pyx_PyGen__FetchStopIterationValue(__Pyx_PyThreadState_Current, pvalue)
#endif
static int __Pyx_PyGen__FetchStopIterationValue(PyThreadState *tstate, PyObject **pvalue);
static CYTHON_INLINE void __Pyx_Coroutine_ResetFrameBackpointer(__Pyx_ExcInfoStruct *exc_state);
static char __Pyx_Coroutine_test_and_set_is_running(__pyx_CoroutineObject *gen);
static void __Pyx_Coroutine_unset_is_running(__pyx_CoroutineObject *gen);
static char __Pyx_Coroutine_get_is_running(__pyx_CoroutineObject *gen);
static PyObject *__Pyx_Coroutine_get_is_running_getter(PyObject *gen, void *closure);
#if __PYX_HAS_PY_AM_SEND == 2
static void __Pyx_SetBackportTypeAmSend(PyTypeObject *type, __Pyx_PyAsyncMethodsStruct *static_amsend_methods, __Pyx_pyiter_sendfunc am_send);
#endif
static PyObject *__Pyx_Coroutine_fail_reduce_ex(PyObject *self, PyObject *arg);

/* Generator.proto */
#define __Pyx_Generator_USED
#define __Pyx_Generator_CheckExact(obj) __Pyx_IS_TYPE(obj, __pyx_mstate_global->__pyx_GeneratorType)
#define __Pyx_Generator_New(body, code, closure, name, qualname, module_name)\
    __Pyx__Coroutine_New(__pyx_mstate_global->__pyx_GeneratorType, body, code, closure, name, qualname, module_name)
static PyObject *__Pyx_Generator_Next(PyObject *self);
static int __pyx_Generator_init(PyObject *module);
static CYTHON_INLINE PyObject *__Pyx_Generator_GetInlinedResult(PyObject *self);

/* CheckBinaryVersion.proto */
static int __Pyx_check_binary_version(unsigned long ct_version, unsigned long rt_version, int allow_newer);

/* DecompressString.proto */
static PyObject *__Pyx_DecompressString(const char *s, Py_ssize_t length, int algo);

/* MultiPhaseInitModuleState.proto */
#if CYTHON_PEP489_MULTI_PHASE_INIT && CYTHON_USE_MODULE_STATE
static PyObject *__Pyx_State_FindModule(void*);
static int __Pyx_State_AddModule(PyObject* module, void*);
static int __Pyx_State_RemoveModule(void*);
#elif CYTHON_USE_MODULE_STATE
#define __Pyx_State_FindModule PyState_FindModule
#define __Pyx_State_AddModule PyState_AddModule
#define __Pyx_State_RemoveModule PyState_RemoveModule
#endif

/* #### Code section: module_declarations ### */
/* CythonABIVersion.proto */
#if CYTHON_COMPILING_IN_LIMITED_API
    #if CYTHON_METH_FASTCALL
        #define __PYX_FASTCALL_ABI_SUFFIX  "_fastcall"
    #else
        #define __PYX_FASTCALL_ABI_SUFFIX
    #endif
    #define __PYX_LIMITED_ABI_SUFFIX "limited" __PYX_FASTCALL_ABI_SUFFIX __PYX_AM_SEND_ABI_SUFFIX
#else
    #define __PYX_LIMITED_ABI_SUFFIX
#endif
#if __PYX_HAS_PY_AM_SEND == 1
    #define __PYX_AM_SEND_ABI_SUFFIX
#elif __PYX_HAS_PY_AM_SEND == 2
    #define __PYX_AM_SEND_ABI_SUFFIX "amsendbackport"
#else
    #define __PYX_AM_SEND_ABI_SUFFIX "noamsend"
#endif
#ifndef __PYX_MONITORING_ABI_SUFFIX
    #define __PYX_MONITORING_ABI_SUFFIX
#endif
#if CYTHON_USE_TP_FINALIZE
    #define __PYX_TP_FINALIZE_ABI_SUFFIX
#else
    #define __PYX_TP_FINALIZE_ABI_SUFFIX "nofinalize"
#endif
#if CYTHON_USE_FREELISTS || !defined(__Pyx_AsyncGen_USED)
    #define __PYX_FREELISTS_ABI_SUFFIX
#else
    #define __PYX_FREELISTS_ABI_SUFFIX "nofreelists"
#endif
#define CYTHON_ABI  __PYX_ABI_VERSION __PYX_LIMITED_ABI_SUFFIX __PYX_MONITORING_ABI_SUFFIX __PYX_TP_FINALIZE_ABI_SUFFIX __PYX_FREELISTS_ABI_SUFFIX __PYX_AM_SEND_ABI_SUFFIX
#define __PYX_ABI_MODULE_NAME "_cython_" CYTHON_ABI
#define __PYX_TYPE_MODULE_PREFIX __PYX_ABI_MODULE_NAME "."

static CYTHON_INLINE PyObject *__pyx_f_4cupy_4cuda_6memory_10MemoryPool_device_pool(struct __pyx_obj_4cupy_4cuda_6memory_MemoryPool *__pyx_v_self); /* proto*/
#if !CYTHON_COMPILING_IN_LIMITED_API
static CYTHON_INLINE double __pyx_f_7cpython_7complex_7complex_4real_real(PyComplexObject *__pyx_v_self); /* proto*/
#endif
#if !CYTHON_COMPILING_IN_LIMITED_API
static CYTHON_INLINE double __pyx_f_7cpython_7complex_7complex_4imag_imag(PyComplexObject *__pyx_v_self); /* proto*/
#endif
static PyObject *__pyx_f_4cupy_4cuda_6memory_12SystemMemory_from_external(intptr_t __pyx_v_ptr, size_t __pyx_v_size, PyObject *__pyx_v_owner); /* proto*/
static PyObject *__pyx_f_4cupy_4cuda_6memory_6_Chunk__init(struct __pyx_obj_4cupy_4cuda_6memory__Chunk *__pyx_v_self, struct __pyx_obj_4cupy_4cuda_6memory_BaseMemory *__pyx_v_mem, ptrdiff_t __pyx_v_offset, size_t __pyx_v_size, struct __pyx_obj_4cupy_4cuda_6memory__Arena *__pyx_v_arena); /* proto*/
static intptr_t __pyx_f_4cupy_4cuda_6memory_6_Chunk_ptr(struct __pyx_obj_4cupy_4cuda_6memory__Chunk *__pyx_v_self, CYTHON_UNUSED int __pyx_skip_dispatch); /* proto*/
static struct __pyx_obj_4cupy_4cuda_6memory__Chunk *__pyx_f_4cupy_4cuda_6memory_6_Chunk_split(struct __pyx_obj_4cupy_4cuda_6memory__Chunk *__pyx_v_self, size_t __pyx_v_size, CYTHON_UNUSED int __pyx_skip_dispatch); /* proto*/
static PyObject *__pyx_f_4cupy_4cuda_6memory_6_Chunk_merge_next(struct __pyx_obj_4cupy_4cuda_6memory__Chunk *__pyx_v_self, CYTHON_UNUSED int __pyx_skip_dispatch); /* proto*/
static PyObject *__pyx_f_4cupy_4cuda_6memory_13MemoryPointer__init(struct __pyx_obj_4cupy_4cuda_6memory_MemoryPointer *__pyx_v_self, struct __pyx_obj_4cupy_4cuda_6memory_BaseMemory *__pyx_v_mem, ptrdiff_t __pyx_v_offset); /* proto*/
static PyObject *__pyx_f_4cupy_4cuda_6memory_13MemoryPointer_copy_from_device(struct __pyx_obj_4cupy_4cuda_6memory_MemoryPointer *__pyx_v_self, struct __pyx_obj_4cupy_4cuda_6memory_MemoryPointer *__pyx_v_src, size_t __pyx_v_size, int __pyx_skip_dispatch); /* proto*/
static PyObject *__pyx_f_4cupy_4cuda_6memory_13MemoryPointer_copy_from_device_async(struct __pyx_obj_4cupy_4cuda_6memory_MemoryPointer *__pyx_v_self, struct __pyx_obj_4cupy_4cuda_6memory_MemoryPointer *__pyx_v_src, size_t __pyx_v_size, int __pyx_skip_dispatch, struct __pyx_opt_args_4cupy_4cuda_6memory_13MemoryPointer_copy_from_device_async *__pyx_optional_args); /* proto*/
static PyObject *__pyx_f_4cupy_4cuda_6memory_13MemoryPointer_copy_from_host(struct __pyx_obj_4cupy_4cuda_6memory_MemoryPointer *__pyx_v_self, PyObject *__pyx_v_mem, size_t __pyx_v_size, int __pyx_skip_dispatch); /* proto*/
static PyObject *__pyx_f_4cupy_4cuda_6memory_13MemoryPointer_copy_from_host_async(struct __pyx_obj_4cupy_4cuda_6memory_MemoryPointer *__pyx_v_self, PyObject *__pyx_v_mem, size_t __pyx_v_size, int __pyx_skip_dispatch, struct __pyx_opt_args_4cupy_4cuda_6memory_13MemoryPointer_copy_from_host_async *__pyx_optional_args); /* proto*/
static PyObject *__pyx_f_4cupy_4cuda_6memory_13MemoryPointer_copy_from(struct __pyx_obj_4cupy_4cuda_6memory_MemoryPointer *__pyx_v_self, PyObject *__pyx_v_mem, size_t __pyx_v_size, int __pyx_skip_dispatch); /* proto*/
static PyObject *__pyx_f_4cupy_4cuda_6memory_13MemoryPointer_copy_from_async(struct __pyx_obj_4cupy_4cuda_6memory_MemoryPointer *__pyx_v_self, PyObject *__pyx_v_mem, size_t __pyx_v_size, int __pyx_skip_dispatch, struct __pyx_opt_args_4cupy_4cuda_6memory_13MemoryPointer_copy_from_async *__pyx_optional_args); /* proto*/
static PyObject *__pyx_f_4cupy_4cuda_6memory_13MemoryPointer_copy_to_host(struct __pyx_obj_4cupy_4cuda_6memory_MemoryPointer *__pyx_v_self, PyObject *__pyx_v_mem, size_t __pyx_v_size, int __pyx_skip_dispatch); /* proto*/
static PyObject *__pyx_f_4cupy_4cuda_6memory_13MemoryPointer_copy_to_host_async(struct __pyx_obj_4cupy_4cuda_6memory_MemoryPointer *__pyx_v_self, PyObject *__pyx_v_mem, size_t __pyx_v_size, int __pyx_skip_dispatch, struct __pyx_opt_args_4cupy_4cuda_6memory_13MemoryPointer_copy_to_host_async *__pyx_optional_args); /* proto*/
static PyObject *__pyx_f_4cupy_4cuda_6memory_13MemoryPointer_memset(struct __pyx_obj_4cupy_4cuda_6memory_MemoryPointer *__pyx_v_self, int __pyx_v_value, size_t __pyx_v_size, int __pyx_skip_dispatch); /* proto*/
static PyObject *__pyx_f_4cupy_4cuda_6memory_13MemoryPointer_memset_async(struct __pyx_obj_4cupy_4cuda_6memory_MemoryPointer *__pyx_v_self, int __pyx_v_value, size_t __pyx_v_size, int __pyx_skip_dispatch, struct __pyx_opt_args_4cupy_4cuda_6memory_13MemoryPointer_memset_async *__pyx_optional_args); /* proto*/
static PyObject *__pyx_f_4cupy_4cuda_6memory_12PooledMemory__init(struct __pyx_obj_4cupy_4cuda_6memory_PooledMemory *__pyx_v_self, struct __pyx_obj_4cupy_4cuda_6memory__Chunk *__pyx_v_chunk, PyObject *__pyx_v_pool); /* proto*/
static PyObject *__pyx_f_4cupy_4cuda_6memory_12PooledMemory_free(struct __pyx_obj_4cupy_4cuda_6memory_PooledMemory *__pyx_v_self, CYTHON_UNUSED int __pyx_skip_dispatch); /* proto*/
static PyObject *__pyx_f_4cupy_4cuda_6memory_6_Arena_add_pending_free_atomic(struct __pyx_obj_4cupy_4cuda_6memory__Arena *__pyx_v_self, struct __pyx_obj_4cupy_4cuda_6memory__Chunk *__pyx_v_chunk); /* proto*/
static PyObject *__pyx_f_4cupy_4cuda_6memory_6_Arena__commit_pending_free(struct __pyx_obj_4cupy_4cuda_6memory__Arena *__pyx_v_self); /* proto*/
static struct __pyx_obj_4cupy_4cuda_6memory__Chunk *__pyx_f_4cupy_4cuda_6memory_6_Arena_try_merge_chunk(struct __pyx_obj_4cupy_4cuda_6memory__Arena *__pyx_v_self, struct __pyx_obj_4cupy_4cuda_6memory__Chunk *__pyx_v_chunk); /* proto*/
static PyObject *__pyx_f_4cupy_4cuda_6memory_6_Arena_insert_chunk(struct __pyx_obj_4cupy_4cuda_6memory__Arena *__pyx_v_self, struct __pyx_obj_4cupy_4cuda_6memory__Chunk *__pyx_v_chunk, struct __pyx_opt_args_4cupy_4cuda_6memory_6_Arena_insert_chunk *__pyx_optional_args); /* proto*/
static size_t __pyx_f_4cupy_4cuda_6memory_6_Arena_free_all(struct __pyx_obj_4cupy_4cuda_6memory__Arena *__pyx_v_self); /* proto*/
static int __pyx_f_4cupy_4cuda_6memory_6_Arena_try_remove_chunk(struct __pyx_obj_4cupy_4cuda_6memory__Arena *__pyx_v_self, struct __pyx_obj_4cupy_4cuda_6memory__Chunk *__pyx_v_chunk); /* proto*/
static struct __pyx_obj_4cupy_4cuda_6memory__Chunk *__pyx_f_4cupy_4cuda_6memory_6_Arena_get_chunk(struct __pyx_obj_4cupy_4cuda_6memory__Arena *__pyx_v_self, size_t __pyx_v_size); /* proto*/
static PyObject *__pyx_f_4cupy_4cuda_6memory_6_Arena__get_size(struct __pyx_obj_4cupy_4cuda_6memory__Arena *__pyx_v_self); /* proto*/
static PyObject *__pyx_f_4cupy_4cuda_6memory_6_Arena__index_to_python(struct __pyx_obj_4cupy_4cuda_6memory__Arena *__pyx_v_self); /* proto*/
static struct __pyx_obj_4cupy_4cuda_6memory__Arena *__pyx_f_4cupy_4cuda_6memory_22SingleDeviceMemoryPool__arena(struct __pyx_obj_4cupy_4cuda_6memory_SingleDeviceMemoryPool *__pyx_v_self, intptr_t __pyx_v_stream_ident, CYTHON_UNUSED int __pyx_skip_dispatch); /* proto*/
static struct __pyx_obj_4cupy_4cuda_6memory_MemoryPointer *__pyx_f_4cupy_4cuda_6memory_22SingleDeviceMemoryPool__alloc(struct __pyx_obj_4cupy_4cuda_6memory_SingleDeviceMemoryPool *__pyx_v_self, Py_ssize_t __pyx_v_rounded_size); /* proto*/
static struct __pyx_obj_4cupy_4cuda_6memory_MemoryPointer *__pyx_f_4cupy_4cuda_6memory_22SingleDeviceMemoryPool_malloc(struct __pyx_obj_4cupy_4cuda_6memory_SingleDeviceMemoryPool *__pyx_v_self, size_t __pyx_v_size, CYTHON_UNUSED int __pyx_skip_dispatch); /* proto*/
static struct __pyx_obj_4cupy_4cuda_6memory_MemoryPointer *__pyx_f_4cupy_4cuda_6memory_22SingleDeviceMemoryPool__malloc(struct __pyx_obj_4cupy_4cuda_6memory_SingleDeviceMemoryPool *__pyx_v_self, size_t __pyx_v_size); /* proto*/
static PyObject *__pyx_f_4cupy_4cuda_6memory_22SingleDeviceMemoryPool_free(struct __pyx_obj_4cupy_4cuda_6memory_SingleDeviceMemoryPool *__pyx_v_self, struct __pyx_obj_4cupy_4cuda_6memory__Chunk *__pyx_v_chunk); /* proto*/
static PyObject *__pyx_f_4cupy_4cuda_6memory_22SingleDeviceMemoryPool_free_all_blocks(struct __pyx_obj_4cupy_4cuda_6memory_SingleDeviceMemoryPool *__pyx_v_self, CYTHON_UNUSED int __pyx_skip_dispatch, struct __pyx_opt_args_4cupy_4cuda_6memory_22SingleDeviceMemoryPool_free_all_blocks *__pyx_optional_args); /* proto*/
static PyObject *__pyx_f_4cupy_4cuda_6memory_22SingleDeviceMemoryPool_free_all_free(struct __pyx_obj_4cupy_4cuda_6memory_SingleDeviceMemoryPool *__pyx_v_self, CYTHON_UNUSED int __pyx_skip_dispatch); /* proto*/
static size_t __pyx_f_4cupy_4cuda_6memory_22SingleDeviceMemoryPool_n_free_blocks(struct __pyx_obj_4cupy_4cuda_6memory_SingleDeviceMemoryPool *__pyx_v_self, CYTHON_UNUSED int __pyx_skip_dispatch); /* proto*/
static size_t __pyx_f_4cupy_4cuda_6memory_22SingleDeviceMemoryPool_used_bytes(struct __pyx_obj_4cupy_4cuda_6memory_SingleDeviceMemoryPool *__pyx_v_self, CYTHON_UNUSED int __pyx_skip_dispatch); /* proto*/
static size_t __pyx_f_4cupy_4cuda_6memory_22SingleDeviceMemoryPool_free_bytes(struct __pyx_obj_4cupy_4cuda_6memory_SingleDeviceMemoryPool *__pyx_v_self, CYTHON_UNUSED int __pyx_skip_dispatch); /* proto*/
static size_t __pyx_f_4cupy_4cuda_6memory_22SingleDeviceMemoryPool_total_bytes(struct __pyx_obj_4cupy_4cuda_6memory_SingleDeviceMemoryPool *__pyx_v_self, CYTHON_UNUSED int __pyx_skip_dispatch); /* proto*/
static int __pyx_f_4cupy_4cuda_6memory_22SingleDeviceMemoryPool__try_block_total_bytes(struct __pyx_obj_4cupy_4cuda_6memory_SingleDeviceMemoryPool *__pyx_v_self, size_t __pyx_v_size); /* proto*/
static PyObject *__pyx_f_4cupy_4cuda_6memory_22SingleDeviceMemoryPool_set_limit(struct __pyx_obj_4cupy_4cuda_6memory_SingleDeviceMemoryPool *__pyx_v_self, CYTHON_UNUSED int __pyx_skip_dispatch, struct __pyx_opt_args_4cupy_4cuda_6memory_22SingleDeviceMemoryPool_set_limit *__pyx_optional_args); /* proto*/
static size_t __pyx_f_4cupy_4cuda_6memory_22SingleDeviceMemoryPool_get_limit(struct __pyx_obj_4cupy_4cuda_6memory_SingleDeviceMemoryPool *__pyx_v_self, CYTHON_UNUSED int __pyx_skip_dispatch); /* proto*/
static struct __pyx_obj_4cupy_4cuda_6memory_BaseMemory *__pyx_f_4cupy_4cuda_6memory_22SingleDeviceMemoryPool__try_malloc(struct __pyx_obj_4cupy_4cuda_6memory_SingleDeviceMemoryPool *__pyx_v_self, size_t __pyx_v_size); /* proto*/
static PyObject *__pyx_f_4cupy_4cuda_6memory_10MemoryPool__ensure_pools_and_return_device_pool(struct __pyx_obj_4cupy_4cuda_6memory_MemoryPool *__pyx_v_self); /* proto*/
static struct __pyx_obj_4cupy_4cuda_6memory_MemoryPointer *__pyx_f_4cupy_4cuda_6memory_10MemoryPool_malloc(struct __pyx_obj_4cupy_4cuda_6memory_MemoryPool *__pyx_v_self, size_t __pyx_v_size, int __pyx_skip_dispatch); /* proto*/
static PyObject *__pyx_f_4cupy_4cuda_6memory_10MemoryPool_free_all_blocks(struct __pyx_obj_4cupy_4cuda_6memory_MemoryPool *__pyx_v_self, int __pyx_skip_dispatch, struct __pyx_opt_args_4cupy_4cuda_6memory_10MemoryPool_free_all_blocks *__pyx_optional_args); /* proto*/
static PyObject *__pyx_f_4cupy_4cuda_6memory_10MemoryPool_free_all_free(struct __pyx_obj_4cupy_4cuda_6memory_MemoryPool *__pyx_v_self, int __pyx_skip_dispatch); /* proto*/
static size_t __pyx_f_4cupy_4cuda_6memory_10MemoryPool_n_free_blocks(struct __pyx_obj_4cupy_4cuda_6memory_MemoryPool *__pyx_v_self, int __pyx_skip_dispatch); /* proto*/
static size_t __pyx_f_4cupy_4cuda_6memory_10MemoryPool_used_bytes(struct __pyx_obj_4cupy_4cuda_6memory_MemoryPool *__pyx_v_self, int __pyx_skip_dispatch); /* proto*/
static size_t __pyx_f_4cupy_4cuda_6memory_10MemoryPool_free_bytes(struct __pyx_obj_4cupy_4cuda_6memory_MemoryPool *__pyx_v_self, int __pyx_skip_dispatch); /* proto*/
static size_t __pyx_f_4cupy_4cuda_6memory_10MemoryPool_total_bytes(struct __pyx_obj_4cupy_4cuda_6memory_MemoryPool *__pyx_v_self, int __pyx_skip_dispatch); /* proto*/
static PyObject *__pyx_f_4cupy_4cuda_6memory_10MemoryPool_set_limit(struct __pyx_obj_4cupy_4cuda_6memory_MemoryPool *__pyx_v_self, int __pyx_skip_dispatch, struct __pyx_opt_args_4cupy_4cuda_6memory_10MemoryPool_set_limit *__pyx_optional_args); /* proto*/
static size_t __pyx_f_4cupy_4cuda_6memory_10MemoryPool_get_limit(struct __pyx_obj_4cupy_4cuda_6memory_MemoryPool *__pyx_v_self, int __pyx_skip_dispatch); /* proto*/
static intptr_t __pyx_f_4cupy_4cuda_6memory_15MemoryAsyncPool_set_pool(CYTHON_UNUSED struct __pyx_obj_4cupy_4cuda_6memory_MemoryAsyncPool *__pyx_v_self, PyObject *__pyx_v_handle, int __pyx_v_dev_id); /* proto*/
static struct __pyx_obj_4cupy_4cuda_6memory_MemoryPointer *__pyx_f_4cupy_4cuda_6memory_15MemoryAsyncPool_malloc(struct __pyx_obj_4cupy_4cuda_6memory_MemoryAsyncPool *__pyx_v_self, size_t __pyx_v_size, int __pyx_skip_dispatch); /* proto*/
static PyObject *__pyx_f_4cupy_4cuda_6memory_15MemoryAsyncPool_free_all_blocks(struct __pyx_obj_4cupy_4cuda_6memory_MemoryAsyncPool *__pyx_v_self, int __pyx_skip_dispatch, struct __pyx_opt_args_4cupy_4cuda_6memory_15MemoryAsyncPool_free_all_blocks *__pyx_optional_args); /* proto*/
static size_t __pyx_f_4cupy_4cuda_6memory_15MemoryAsyncPool_n_free_blocks(CYTHON_UNUSED struct __pyx_obj_4cupy_4cuda_6memory_MemoryAsyncPool *__pyx_v_self, int __pyx_skip_dispatch); /* proto*/
static size_t __pyx_f_4cupy_4cuda_6memory_15MemoryAsyncPool_used_bytes(struct __pyx_obj_4cupy_4cuda_6memory_MemoryAsyncPool *__pyx_v_self, int __pyx_skip_dispatch); /* proto*/
static size_t __pyx_f_4cupy_4cuda_6memory_15MemoryAsyncPool_free_bytes(struct __pyx_obj_4cupy_4cuda_6memory_MemoryAsyncPool *__pyx_v_self, int __pyx_skip_dispatch); /* proto*/
static size_t __pyx_f_4cupy_4cuda_6memory_15MemoryAsyncPool_total_bytes(struct __pyx_obj_4cupy_4cuda_6memory_MemoryAsyncPool *__pyx_v_self, int __pyx_skip_dispatch); /* proto*/
static PyObject *__pyx_f_4cupy_4cuda_6memory_15MemoryAsyncPool_set_limit(struct __pyx_obj_4cupy_4cuda_6memory_MemoryAsyncPool *__pyx_v_self, int __pyx_skip_dispatch, struct __pyx_opt_args_4cupy_4cuda_6memory_15MemoryAsyncPool_set_limit *__pyx_optional_args); /* proto*/
static size_t __pyx_f_4cupy_4cuda_6memory_15MemoryAsyncPool_get_limit(struct __pyx_obj_4cupy_4cuda_6memory_MemoryAsyncPool *__pyx_v_self, int __pyx_skip_dispatch); /* proto*/
static struct __pyx_obj_4cupy_4cuda_6memory_MemoryPointer *__pyx_f_4cupy_4cuda_6memory_18CFunctionAllocator_malloc(struct __pyx_obj_4cupy_4cuda_6memory_CFunctionAllocator *__pyx_v_self, size_t __pyx_v_size, int __pyx_skip_dispatch); /* proto*/
static struct __pyx_obj_4cupy_4cuda_6memory_MemoryPointer *__pyx_f_4cupy_4cuda_6memory_23PythonFunctionAllocator_malloc(struct __pyx_obj_4cupy_4cuda_6memory_PythonFunctionAllocator *__pyx_v_self, size_t __pyx_v_size, int __pyx_skip_dispatch); /* proto*/

/* Module declarations from "cython" */

/* Module declarations from "libc.stdint" */

/* Module declarations from "libcpp" */

/* Module declarations from "libcpp.vector" */

/* Module declarations from "libcpp.utility" */

/* Module declarations from "libcpp.map" */

/* Module declarations from "cupy.cuda" */

/* Module declarations from "cupy.cuda.device" */
static int (*__pyx_f_4cupy_4cuda_6device_get_device_id)(int __pyx_skip_dispatch); /*proto*/
static int (*__pyx_f_4cupy_4cuda_6device__enable_peer_access)(int, int); /*proto*/

/* Module declarations from "cpython.version" */

/* Module declarations from "__builtin__" */

/* Module declarations from "cpython.type" */

/* Module declarations from "libc.string" */

/* Module declarations from "libc.stdio" */

/* Module declarations from "cpython.object" */

/* Module declarations from "cpython.ref" */

/* Module declarations from "cpython.exc" */

/* Module declarations from "cpython.module" */

/* Module declarations from "cpython.mem" */

/* Module declarations from "cpython.tuple" */

/* Module declarations from "cpython.list" */

/* Module declarations from "cpython.sequence" */

/* Module declarations from "cpython.mapping" */

/* Module declarations from "cpython.iterator" */

/* Module declarations from "cpython.number" */

/* Module declarations from "__builtin__" */

/* Module declarations from "cpython.bool" */

/* Module declarations from "cpython.long" */

/* Module declarations from "cpython.float" */

/* Module declarations from "__builtin__" */

/* Module declarations from "cpython.complex" */

/* Module declarations from "libc.stddef" */

/* Module declarations from "cpython.unicode" */

/* Module declarations from "cpython.pyport" */

/* Module declarations from "cpython.dict" */

/* Module declarations from "cpython.instance" */

/* Module declarations from "cpython.function" */

/* Module declarations from "cpython.method" */

/* Module declarations from "cpython.weakref" */

/* Module declarations from "cpython.getargs" */

/* Module declarations from "cpython.pythread" */

/* Module declarations from "cpython.pystate" */

/* Module declarations from "cpython.set" */

/* Module declarations from "cpython.buffer" */

/* Module declarations from "cpython.bytes" */

/* Module declarations from "cpython.pycapsule" */

/* Module declarations from "cpython.contextvars" */

/* Module declarations from "cpython" */

/* Module declarations from "libc.stdlib" */

/* Module declarations from "libcpp.atomic" */

/* Module declarations from "libcpp.set" */

/* Module declarations from "libcpp.pair" */

/* Module declarations from "libcpp.mutex" */

/* Module declarations from "cupy.cuda.memory_hook" */
static int (*__pyx_f_4cupy_4cuda_11memory_hook__has_memory_hooks)(int __pyx_skip_dispatch); /*proto*/
static PyObject *(*__pyx_f_4cupy_4cuda_11memory_hook_get_memory_hooks)(int __pyx_skip_dispatch); /*proto*/

/* Module declarations from "cupy.cuda.stream" */
static intptr_t (*__pyx_f_4cupy_4cuda_6stream_get_current_stream_ptr)(void); /*proto*/
static PyObject *(*__pyx_f_4cupy_4cuda_6stream_get_current_stream)(int __pyx_skip_dispatch, struct __pyx_opt_args_4cupy_4cuda_6stream_get_current_stream *__pyx_optional_args); /*proto*/

/* Module declarations from "cupy_backends.cuda.api" */

/* Module declarations from "cupy_backends.cuda.api._driver_enum" */

/* Module declarations from "cupy_backends.cuda.api.driver" */
static PyObject *(*__pyx_f_13cupy_backends_4cuda_3api_6driver_get_build_version)(int __pyx_skip_dispatch); /*proto*/

/* Module declarations from "cupy_backends.cuda.api._runtime_enum" */

/* Module declarations from "cupy_backends.cuda.api.runtime" */
static int *__pyx_vp_13cupy_backends_4cuda_3api_7runtime_errorMemoryAllocation = 0;
#define __pyx_v_13cupy_backends_4cuda_3api_7runtime_errorMemoryAllocation (*__pyx_vp_13cupy_backends_4cuda_3api_7runtime_errorMemoryAllocation)
static int *__pyx_vp_13cupy_backends_4cuda_3api_7runtime_errorInvalidValue = 0;
#define __pyx_v_13cupy_backends_4cuda_3api_7runtime_errorInvalidValue (*__pyx_vp_13cupy_backends_4cuda_3api_7runtime_errorInvalidValue)
static int *__pyx_vp_13cupy_backends_4cuda_3api_7runtime_errorPeerAccessAlreadyEnabled = 0;
#define __pyx_v_13cupy_backends_4cuda_3api_7runtime_errorPeerAccessAlreadyEnabled (*__pyx_vp_13cupy_backends_4cuda_3api_7runtime_errorPeerAccessAlreadyEnabled)
static int *__pyx_vp_13cupy_backends_4cuda_3api_7runtime_errorContextIsDestroyed = 0;
#define __pyx_v_13cupy_backends_4cuda_3api_7runtime_errorContextIsDestroyed (*__pyx_vp_13cupy_backends_4cuda_3api_7runtime_errorContextIsDestroyed)
static int *__pyx_vp_13cupy_backends_4cuda_3api_7runtime_errorInvalidResourceHandle = 0;
#define __pyx_v_13cupy_backends_4cuda_3api_7runtime_errorInvalidResourceHandle (*__pyx_vp_13cupy_backends_4cuda_3api_7runtime_errorInvalidResourceHandle)
static int *__pyx_vp_13cupy_backends_4cuda_3api_7runtime_deviceAttributeComputeCapabilityMajor = 0;
#define __pyx_v_13cupy_backends_4cuda_3api_7runtime_deviceAttributeComputeCapabilityMajor (*__pyx_vp_13cupy_backends_4cuda_3api_7runtime_deviceAttributeComputeCapabilityMajor)
static int *__pyx_vp_13cupy_backends_4cuda_3api_7runtime_deviceAttributeComputeCapabilityMinor = 0;
#define __pyx_v_13cupy_backends_4cuda_3api_7runtime_deviceAttributeComputeCapabilityMinor (*__pyx_vp_13cupy_backends_4cuda_3api_7runtime_deviceAttributeComputeCapabilityMinor)
static int *__pyx_vp_13cupy_backends_4cuda_3api_7runtime__is_hip_environment = 0;
#define __pyx_v_13cupy_backends_4cuda_3api_7runtime__is_hip_environment (*__pyx_vp_13cupy_backends_4cuda_3api_7runtime__is_hip_environment)
static int (*__pyx_f_13cupy_backends_4cuda_3api_7runtime_driverGetVersion)(int __pyx_skip_dispatch); /*proto*/
static int (*__pyx_f_13cupy_backends_4cuda_3api_7runtime_getDevice)(int __pyx_skip_dispatch); /*proto*/
static int (*__pyx_f_13cupy_backends_4cuda_3api_7runtime_deviceGetAttribute)(int, int, int __pyx_skip_dispatch); /*proto*/
static int (*__pyx_f_13cupy_backends_4cuda_3api_7runtime_getDeviceCount)(int __pyx_skip_dispatch); /*proto*/
static PyObject *(*__pyx_f_13cupy_backends_4cuda_3api_7runtime_setDevice)(int, int __pyx_skip_dispatch); /*proto*/
static intptr_t (*__pyx_f_13cupy_backends_4cuda_3api_7runtime_malloc)(size_t, int __pyx_skip_dispatch); /*proto*/
static intptr_t (*__pyx_f_13cupy_backends_4cuda_3api_7runtime_mallocManaged)(size_t, int __pyx_skip_dispatch, struct __pyx_opt_args_13cupy_backends_4cuda_3api_7runtime_mallocManaged *__pyx_optional_args); /*proto*/
static intptr_t (*__pyx_f_13cupy_backends_4cuda_3api_7runtime_mallocAsync)(size_t, intptr_t, int __pyx_skip_dispatch); /*proto*/
static PyObject *(*__pyx_f_13cupy_backends_4cuda_3api_7runtime_free)(intptr_t, int __pyx_skip_dispatch); /*proto*/
static PyObject *(*__pyx_f_13cupy_backends_4cuda_3api_7runtime_freeAsync)(intptr_t, intptr_t, int __pyx_skip_dispatch); /*proto*/
static PyObject *(*__pyx_f_13cupy_backends_4cuda_3api_7runtime_memGetInfo)(int __pyx_skip_dispatch); /*proto*/
static PyObject *(*__pyx_f_13cupy_backends_4cuda_3api_7runtime_memcpy)(intptr_t, intptr_t, size_t, int, int __pyx_skip_dispatch); /*proto*/
static PyObject *(*__pyx_f_13cupy_backends_4cuda_3api_7runtime_memcpyAsync)(intptr_t, intptr_t, size_t, int, intptr_t, int __pyx_skip_dispatch); /*proto*/
static PyObject *(*__pyx_f_13cupy_backends_4cuda_3api_7runtime_memset)(intptr_t, int, size_t, int __pyx_skip_dispatch); /*proto*/
static PyObject *(*__pyx_f_13cupy_backends_4cuda_3api_7runtime_memsetAsync)(intptr_t, int, size_t, intptr_t, int __pyx_skip_dispatch); /*proto*/
static PyObject *(*__pyx_f_13cupy_backends_4cuda_3api_7runtime_memPrefetchAsync)(intptr_t, size_t, int, intptr_t, int __pyx_skip_dispatch); /*proto*/
static PyObject *(*__pyx_f_13cupy_backends_4cuda_3api_7runtime_memAdvise)(intptr_t, size_t, int, int, int __pyx_skip_dispatch); /*proto*/
static struct __pyx_obj_13cupy_backends_4cuda_3api_7runtime_PointerAttributes *(*__pyx_f_13cupy_backends_4cuda_3api_7runtime_pointerGetAttributes)(intptr_t, int __pyx_skip_dispatch); /*proto*/
static intptr_t (*__pyx_f_13cupy_backends_4cuda_3api_7runtime_deviceGetDefaultMemPool)(int, int __pyx_skip_dispatch); /*proto*/
static intptr_t (*__pyx_f_13cupy_backends_4cuda_3api_7runtime_deviceGetMemPool)(int, int __pyx_skip_dispatch); /*proto*/
static PyObject *(*__pyx_f_13cupy_backends_4cuda_3api_7runtime_deviceSetMemPool)(int, intptr_t, int __pyx_skip_dispatch); /*proto*/
static PyObject *(*__pyx_f_13cupy_backends_4cuda_3api_7runtime_memPoolTrimTo)(intptr_t, size_t, int __pyx_skip_dispatch); /*proto*/
static PyObject *(*__pyx_f_13cupy_backends_4cuda_3api_7runtime_memPoolGetAttribute)(intptr_t, int, int __pyx_skip_dispatch); /*proto*/
static PyObject *(*__pyx_f_13cupy_backends_4cuda_3api_7runtime_memPoolSetAttribute)(intptr_t, int, PyObject *, int __pyx_skip_dispatch); /*proto*/
static PyObject *(*__pyx_f_13cupy_backends_4cuda_3api_7runtime_streamSynchronize)(intptr_t, int __pyx_skip_dispatch); /*proto*/
static int (*__pyx_f_13cupy_backends_4cuda_3api_7runtime_streamIsCapturing)(intptr_t, int __pyx_skip_dispatch); /*proto*/
static PyObject *(*__pyx_f_13cupy_backends_4cuda_3api_7runtime__ensure_context)(void); /*proto*/

/* Module declarations from "cupy.cuda.memory" */
static int const __pyx_v_4cupy_4cuda_6memory_ALLOCATION_UNIT_SIZE = 512;
static int __pyx_v_4cupy_4cuda_6memory__exit_mode;
static int __pyx_v_4cupy_4cuda_6memory__is_ump_enabled;
static PyObject *__pyx_v_4cupy_4cuda_6memory__current_allocator = 0;
static PyObject *__pyx_v_4cupy_4cuda_6memory__thread_local = 0;
static struct __pyx_obj_4cupy_4cuda_6memory_MemoryPointer *__pyx_f_4cupy_4cuda_6memory_alloc(PyObject *, int __pyx_skip_dispatch); /*proto*/
static PyObject *__pyx_f_4cupy_4cuda_6memory_set_allocator(int __pyx_skip_dispatch, struct __pyx_opt_args_4cupy_4cuda_6memory_set_allocator *__pyx_optional_args); /*proto*/
static PyObject *__pyx_f_4cupy_4cuda_6memory_get_allocator(int __pyx_skip_dispatch); /*proto*/
static CYTHON_INLINE void __pyx_f_4cupy_4cuda_6memory_check_async_alloc_supported(int); /*proto*/
static struct __pyx_obj_4cupy_4cuda_6memory_MemoryPointer *__pyx_f_4cupy_4cuda_6memory__malloc(size_t, int __pyx_skip_dispatch); /*proto*/
static struct __pyx_obj_4cupy_4cuda_6memory_MemoryPointer *__pyx_f_4cupy_4cuda_6memory_malloc_async(size_t, int __pyx_skip_dispatch); /*proto*/
static struct __pyx_obj_4cupy_4cuda_6memory_MemoryPointer *__pyx_f_4cupy_4cuda_6memory_malloc_managed(size_t, int __pyx_skip_dispatch); /*proto*/
static struct __pyx_obj_4cupy_4cuda_6memory_MemoryPointer *__pyx_f_4cupy_4cuda_6memory_malloc_system(size_t, int __pyx_skip_dispatch); /*proto*/
static CYTHON_INLINE intptr_t __pyx_f_4cupy_4cuda_6memory__get_stream_identifier(intptr_t); /*proto*/
static CYTHON_INLINE size_t __pyx_f_4cupy_4cuda_6memory__round_size(size_t, int __pyx_skip_dispatch); /*proto*/
static CYTHON_INLINE PyObject *__pyx_f_4cupy_4cuda_6memory__parse_limit_string(int __pyx_skip_dispatch, struct __pyx_opt_args_4cupy_4cuda_6memory__parse_limit_string *__pyx_optional_args); /*proto*/
static intptr_t __pyx_f_4cupy_4cuda_6memory__call_malloc(intptr_t, intptr_t, Py_ssize_t, int); /*proto*/
static void __pyx_f_4cupy_4cuda_6memory__call_free(intptr_t, intptr_t, intptr_t, int); /*proto*/
static PyObject *__pyx_f_4cupy_4cuda_6memory___pyx_unpickle_BaseMemory__set_state(struct __pyx_obj_4cupy_4cuda_6memory_BaseMemory *, PyObject *); /*proto*/
static PyObject *__pyx_f_4cupy_4cuda_6memory___pyx_unpickle_Memory__set_state(struct __pyx_obj_4cupy_4cuda_6memory_Memory *, PyObject *); /*proto*/
static PyObject *__pyx_f_4cupy_4cuda_6memory___pyx_unpickle_MemoryAsync__set_state(struct __pyx_obj_4cupy_4cuda_6memory_MemoryAsync *, PyObject *); /*proto*/
static PyObject *__pyx_f_4cupy_4cuda_6memory___pyx_unpickle_UnownedMemory__set_state(struct __pyx_obj_4cupy_4cuda_6memory_UnownedMemory *, PyObject *); /*proto*/
static PyObject *__pyx_f_4cupy_4cuda_6memory___pyx_unpickle_ManagedMemory__set_state(struct __pyx_obj_4cupy_4cuda_6memory_ManagedMemory *, PyObject *); /*proto*/
static PyObject *__pyx_f_4cupy_4cuda_6memory___pyx_unpickle_SystemMemory__set_state(struct __pyx_obj_4cupy_4cuda_6memory_SystemMemory *, PyObject *); /*proto*/
static PyObject *__pyx_f_4cupy_4cuda_6memory___pyx_unpickle__Chunk__set_state(struct __pyx_obj_4cupy_4cuda_6memory__Chunk *, PyObject *); /*proto*/
static PyObject *__pyx_f_4cupy_4cuda_6memory___pyx_unpickle_MemoryPointer__set_state(struct __pyx_obj_4cupy_4cuda_6memory_MemoryPointer *, PyObject *); /*proto*/
static PyObject *__pyx_f_4cupy_4cuda_6memory___pyx_unpickle_PooledMemory__set_state(struct __pyx_obj_4cupy_4cuda_6memory_PooledMemory *, PyObject *); /*proto*/
static PyObject *__pyx_f_4cupy_4cuda_6memory___pyx_unpickle_MemoryPool__set_state(struct __pyx_obj_4cupy_4cuda_6memory_MemoryPool *, PyObject *); /*proto*/
static PyObject *__pyx_f_4cupy_4cuda_6memory___pyx_unpickle_MemoryAsyncPool__set_state(struct __pyx_obj_4cupy_4cuda_6memory_MemoryAsyncPool *, PyObject *); /*proto*/
static PyObject *__pyx_f_4cupy_4cuda_6memory___pyx_unpickle_CFunctionAllocatorMemory__set_state(struct __pyx_obj_4cupy_4cuda_6memory_CFunctionAllocatorMemory *, PyObject *); /*proto*/
static PyObject *__pyx_f_4cupy_4cuda_6memory___pyx_unpickle_CFunctionAllocator__set_state(struct __pyx_obj_4cupy_4cuda_6memory_CFunctionAllocator *, PyObject *); /*proto*/
static PyObject *__pyx_f_4cupy_4cuda_6memory___pyx_unpickle_PythonFunctionAllocatorMemory__set_state(struct __pyx_obj_4cupy_4cuda_6memory_PythonFunctionAllocatorMemory *, PyObject *); /*proto*/
static PyObject *__pyx_f_4cupy_4cuda_6memory___pyx_unpickle_PythonFunctionAllocator__set_state(struct __pyx_obj_4cupy_4cuda_6memory_PythonFunctionAllocator *, PyObject *); /*proto*/
/* #### Code section: typeinfo ### */
/* #### Code section: before_global_var ### */
#define __Pyx_MODULE_NAME "cupy.cuda.memory"
extern int __pyx_module_is_main_cupy__cuda__memory;
int __pyx_module_is_main_cupy__cuda__memory = 0;

/* Implementation of "cupy.cuda.memory" */
/* #### Code section: global_var ### */
static PyObject *__pyx_builtin_super;
static PyObject *__pyx_builtin_id;
static PyObject *__pyx_builtin_object;
/* #### Code section: string_decls ### */
static const char __pyx_k_allocator__pools[] = "_allocator, _pools";
static const char __pyx_k_device_id_mem_ptr[] = "device_id, mem, ptr";
static const char __pyx_k_device_id_ptr_size[] = "device_id, ptr, size";
static const char __pyx_k_free_func__malloc_func[] = "_free_func, _malloc_func";
static const char __pyx_k_owner_device_id_ptr_size[] = "_owner, device_id, ptr, size";
static const char __pyx_k_pools_memoryAsyncHasStat[] = "_pools, memoryAsyncHasStat";
static const char __pyx_k_free_func_device_id_ptr_size[] = "_free_func, device_id, ptr, size";
static const char __pyx_k_device_id_ptr_size_stream_ref[] = "device_id, ptr, size, stream_ref";
static const char __pyx_k_free_func__malloc_func__owner[] = "_free_func, _malloc_func, _owner, _param";
static const char __pyx_k_free_func__param_device_id_ptr[] = "_free_func, _param, device_id, ptr, size";
static const char __pyx_k_arena_mem_next_offset_prev_size[] = "arena, mem, next, offset, prev, size";
static const char __pyx_k_chunk_device_id_identity_pool_pt[] = "chunk, device_id, identity, pool, ptr, size";
/* #### Code section: decls ### */
static PyObject *__pyx_pf_4cupy_4cuda_6memory__exit(CYTHON_UNUSED PyObject *__pyx_self); /* proto */
static PyObject *__pyx_pf_4cupy_4cuda_6memory_16OutOfMemoryError___init__(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_self, PyObject *__pyx_v_size, PyObject *__pyx_v_total, PyObject *__pyx_v_limit); /* proto */
static PyObject *__pyx_pf_4cupy_4cuda_6memory_16OutOfMemoryError_2__reduce__(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_4cupy_4cuda_6memory_10BaseMemory___int__(struct __pyx_obj_4cupy_4cuda_6memory_BaseMemory *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_4cupy_4cuda_6memory_10BaseMemory_6device___get__(struct __pyx_obj_4cupy_4cuda_6memory_BaseMemory *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_4cupy_4cuda_6memory_10BaseMemory_3ptr___get__(struct __pyx_obj_4cupy_4cuda_6memory_BaseMemory *__pyx_v_self); /* proto */
static int __pyx_pf_4cupy_4cuda_6memory_10BaseMemory_3ptr_2__set__(struct __pyx_obj_4cupy_4cuda_6memory_BaseMemory *__pyx_v_self, PyObject *__pyx_v_value); /* proto */
static PyObject *__pyx_pf_4cupy_4cuda_6memory_10BaseMemory_4size___get__(struct __pyx_obj_4cupy_4cuda_6memory_BaseMemory *__pyx_v_self); /* proto */
static int __pyx_pf_4cupy_4cuda_6memory_10BaseMemory_4size_2__set__(struct __pyx_obj_4cupy_4cuda_6memory_BaseMemory *__pyx_v_self, PyObject *__pyx_v_value); /* proto */
static PyObject *__pyx_pf_4cupy_4cuda_6memory_10BaseMemory_9device_id___get__(struct __pyx_obj_4cupy_4cuda_6memory_BaseMemory *__pyx_v_self); /* proto */
static int __pyx_pf_4cupy_4cuda_6memory_10BaseMemory_9device_id_2__set__(struct __pyx_obj_4cupy_4cuda_6memory_BaseMemory *__pyx_v_self, PyObject *__pyx_v_value); /* proto */
static PyObject *__pyx_pf_4cupy_4cuda_6memory_10BaseMemory_2__reduce_cython__(struct __pyx_obj_4cupy_4cuda_6memory_BaseMemory *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_4cupy_4cuda_6memory_10BaseMemory_4__setstate_cython__(struct __pyx_obj_4cupy_4cuda_6memory_BaseMemory *__pyx_v_self, PyObject *__pyx_v___pyx_state); /* proto */
static int __pyx_pf_4cupy_4cuda_6memory_6Memory___init__(struct __pyx_obj_4cupy_4cuda_6memory_Memory *__pyx_v_self, size_t __pyx_v_size); /* proto */
static void __pyx_pf_4cupy_4cuda_6memory_6Memory_2__dealloc__(struct __pyx_obj_4cupy_4cuda_6memory_Memory *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_4cupy_4cuda_6memory_6Memory_4__reduce_cython__(struct __pyx_obj_4cupy_4cuda_6memory_Memory *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_4cupy_4cuda_6memory_6Memory_6__setstate_cython__(struct __pyx_obj_4cupy_4cuda_6memory_Memory *__pyx_v_self, PyObject *__pyx_v___pyx_state); /* proto */
static int __pyx_pf_4cupy_4cuda_6memory_11MemoryAsync___init__(struct __pyx_obj_4cupy_4cuda_6memory_MemoryAsync *__pyx_v_self, size_t __pyx_v_size, PyObject *__pyx_v_stream); /* proto */
static void __pyx_pf_4cupy_4cuda_6memory_11MemoryAsync_2__dealloc__(struct __pyx_obj_4cupy_4cuda_6memory_MemoryAsync *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_4cupy_4cuda_6memory_11MemoryAsync_10stream_ref___get__(struct __pyx_obj_4cupy_4cuda_6memory_MemoryAsync *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_4cupy_4cuda_6memory_11MemoryAsync_4__reduce_cython__(struct __pyx_obj_4cupy_4cuda_6memory_MemoryAsync *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_4cupy_4cuda_6memory_11MemoryAsync_6__setstate_cython__(struct __pyx_obj_4cupy_4cuda_6memory_MemoryAsync *__pyx_v_self, PyObject *__pyx_v___pyx_state); /* proto */
static int __pyx_pf_4cupy_4cuda_6memory_13UnownedMemory___init__(struct __pyx_obj_4cupy_4cuda_6memory_UnownedMemory *__pyx_v_self, intptr_t __pyx_v_ptr, size_t __pyx_v_size, PyObject *__pyx_v_owner, int __pyx_v_device_id); /* proto */
static PyObject *__pyx_pf_4cupy_4cuda_6memory_13UnownedMemory_6_owner___get__(struct __pyx_obj_4cupy_4cuda_6memory_UnownedMemory *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_4cupy_4cuda_6memory_13UnownedMemory_2__reduce_cython__(struct __pyx_obj_4cupy_4cuda_6memory_UnownedMemory *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_4cupy_4cuda_6memory_13UnownedMemory_4__setstate_cython__(struct __pyx_obj_4cupy_4cuda_6memory_UnownedMemory *__pyx_v_self, PyObject *__pyx_v___pyx_state); /* proto */
static int __pyx_pf_4cupy_4cuda_6memory_13ManagedMemory___init__(struct __pyx_obj_4cupy_4cuda_6memory_ManagedMemory *__pyx_v_self, size_t __pyx_v_size); /* proto */
static PyObject *__pyx_pf_4cupy_4cuda_6memory_13ManagedMemory_2prefetch(struct __pyx_obj_4cupy_4cuda_6memory_ManagedMemory *__pyx_v_self, PyObject *__pyx_v_stream, int __pyx_v_device_id); /* proto */
static PyObject *__pyx_pf_4cupy_4cuda_6memory_13ManagedMemory_4advise(struct __pyx_obj_4cupy_4cuda_6memory_ManagedMemory *__pyx_v_self, int __pyx_v_advise, struct __pyx_obj_4cupy_4cuda_6device_Device *__pyx_v_dev); /* proto */
static void __pyx_pf_4cupy_4cuda_6memory_13ManagedMemory_6__dealloc__(struct __pyx_obj_4cupy_4cuda_6memory_ManagedMemory *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_4cupy_4cuda_6memory_13ManagedMemory_8__reduce_cython__(struct __pyx_obj_4cupy_4cuda_6memory_ManagedMemory *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_4cupy_4cuda_6memory_13ManagedMemory_10__setstate_cython__(struct __pyx_obj_4cupy_4cuda_6memory_ManagedMemory *__pyx_v_self, PyObject *__pyx_v___pyx_state); /* proto */
static int __pyx_pf_4cupy_4cuda_6memory_12SystemMemory___init__(struct __pyx_obj_4cupy_4cuda_6memory_SystemMemory *__pyx_v_self, size_t __pyx_v_size); /* proto */
static PyObject *__pyx_pf_4cupy_4cuda_6memory_12SystemMemory_2prefetch(struct __pyx_obj_4cupy_4cuda_6memory_SystemMemory *__pyx_v_self, PyObject *__pyx_v_stream, int __pyx_v_device_id); /* proto */
static PyObject *__pyx_pf_4cupy_4cuda_6memory_12SystemMemory_4advise(struct __pyx_obj_4cupy_4cuda_6memory_SystemMemory *__pyx_v_self, int __pyx_v_advise, struct __pyx_obj_4cupy_4cuda_6device_Device *__pyx_v_dev); /* proto */
static void __pyx_pf_4cupy_4cuda_6memory_12SystemMemory_6__dealloc__(struct __pyx_obj_4cupy_4cuda_6memory_SystemMemory *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_4cupy_4cuda_6memory_12SystemMemory_6_owner___get__(struct __pyx_obj_4cupy_4cuda_6memory_SystemMemory *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_4cupy_4cuda_6memory_12SystemMemory_8__reduce_cython__(struct __pyx_obj_4cupy_4cuda_6memory_SystemMemory *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_4cupy_4cuda_6memory_12SystemMemory_10__setstate_cython__(struct __pyx_obj_4cupy_4cuda_6memory_SystemMemory *__pyx_v_self, PyObject *__pyx_v___pyx_state); /* proto */
static int __pyx_pf_4cupy_4cuda_6memory_6_Chunk___init__(struct __pyx_obj_4cupy_4cuda_6memory__Chunk *__pyx_v_self, PyObject *__pyx_v_args); /* proto */
static PyObject *__pyx_pf_4cupy_4cuda_6memory_6_Chunk_2__repr__(struct __pyx_obj_4cupy_4cuda_6memory__Chunk *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_4cupy_4cuda_6memory_6_Chunk_4ptr(struct __pyx_obj_4cupy_4cuda_6memory__Chunk *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_4cupy_4cuda_6memory_6_Chunk_6split(struct __pyx_obj_4cupy_4cuda_6memory__Chunk *__pyx_v_self, size_t __pyx_v_size); /* proto */
static PyObject *__pyx_pf_4cupy_4cuda_6memory_6_Chunk_8merge_next(struct __pyx_obj_4cupy_4cuda_6memory__Chunk *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_4cupy_4cuda_6memory_6_Chunk_3mem___get__(struct __pyx_obj_4cupy_4cuda_6memory__Chunk *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_4cupy_4cuda_6memory_6_Chunk_6offset___get__(struct __pyx_obj_4cupy_4cuda_6memory__Chunk *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_4cupy_4cuda_6memory_6_Chunk_4size___get__(struct __pyx_obj_4cupy_4cuda_6memory__Chunk *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_4cupy_4cuda_6memory_6_Chunk_5arena___get__(struct __pyx_obj_4cupy_4cuda_6memory__Chunk *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_4cupy_4cuda_6memory_6_Chunk_4prev___get__(struct __pyx_obj_4cupy_4cuda_6memory__Chunk *__pyx_v_self); /* proto */
static int __pyx_pf_4cupy_4cuda_6memory_6_Chunk_4prev_2__set__(struct __pyx_obj_4cupy_4cuda_6memory__Chunk *__pyx_v_self, PyObject *__pyx_v_value); /* proto */
static int __pyx_pf_4cupy_4cuda_6memory_6_Chunk_4prev_4__del__(struct __pyx_obj_4cupy_4cuda_6memory__Chunk *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_4cupy_4cuda_6memory_6_Chunk_4next___get__(struct __pyx_obj_4cupy_4cuda_6memory__Chunk *__pyx_v_self); /* proto */
static int __pyx_pf_4cupy_4cuda_6memory_6_Chunk_4next_2__set__(struct __pyx_obj_4cupy_4cuda_6memory__Chunk *__pyx_v_self, PyObject *__pyx_v_value); /* proto */
static int __pyx_pf_4cupy_4cuda_6memory_6_Chunk_4next_4__del__(struct __pyx_obj_4cupy_4cuda_6memory__Chunk *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_4cupy_4cuda_6memory_6_Chunk_10__reduce_cython__(struct __pyx_obj_4cupy_4cuda_6memory__Chunk *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_4cupy_4cuda_6memory_6_Chunk_12__setstate_cython__(struct __pyx_obj_4cupy_4cuda_6memory__Chunk *__pyx_v_self, PyObject *__pyx_v___pyx_state); /* proto */
static int __pyx_pf_4cupy_4cuda_6memory_13MemoryPointer___init__(struct __pyx_obj_4cupy_4cuda_6memory_MemoryPointer *__pyx_v_self, struct __pyx_obj_4cupy_4cuda_6memory_BaseMemory *__pyx_v_mem, ptrdiff_t __pyx_v_offset); /* proto */
static PyObject *__pyx_pf_4cupy_4cuda_6memory_13MemoryPointer_2__int__(struct __pyx_obj_4cupy_4cuda_6memory_MemoryPointer *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_4cupy_4cuda_6memory_13MemoryPointer_4__repr__(struct __pyx_obj_4cupy_4cuda_6memory_MemoryPointer *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_4cupy_4cuda_6memory_13MemoryPointer_6device___get__(struct __pyx_obj_4cupy_4cuda_6memory_MemoryPointer *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_4cupy_4cuda_6memory_13MemoryPointer_6__add__(PyObject *__pyx_v_x, PyObject *__pyx_v_y); /* proto */
static PyObject *__pyx_pf_4cupy_4cuda_6memory_13MemoryPointer_8__iadd__(struct __pyx_obj_4cupy_4cuda_6memory_MemoryPointer *__pyx_v_self, ptrdiff_t __pyx_v_offset); /* proto */
static PyObject *__pyx_pf_4cupy_4cuda_6memory_13MemoryPointer_10__sub__(PyObject *__pyx_v_self, PyObject *__pyx_v_offset); /* proto */
static PyObject *__pyx_pf_4cupy_4cuda_6memory_13MemoryPointer_12__isub__(struct __pyx_obj_4cupy_4cuda_6memory_MemoryPointer *__pyx_v_self, ptrdiff_t __pyx_v_offset); /* proto */
static PyObject *__pyx_pf_4cupy_4cuda_6memory_13MemoryPointer_14copy_from_device(struct __pyx_obj_4cupy_4cuda_6memory_MemoryPointer *__pyx_v_self, struct __pyx_obj_4cupy_4cuda_6memory_MemoryPointer *__pyx_v_src, size_t __pyx_v_size); /* proto */
static PyObject *__pyx_pf_4cupy_4cuda_6memory_13MemoryPointer_16copy_from_device_async(struct __pyx_obj_4cupy_4cuda_6memory_MemoryPointer *__pyx_v_self, struct __pyx_obj_4cupy_4cuda_6memory_MemoryPointer *__pyx_v_src, size_t __pyx_v_size, PyObject *__pyx_v_stream); /* proto */
static PyObject *__pyx_pf_4cupy_4cuda_6memory_13MemoryPointer_18copy_from_host(struct __pyx_obj_4cupy_4cuda_6memory_MemoryPointer *__pyx_v_self, PyObject *__pyx_v_mem, size_t __pyx_v_size); /* proto */
static PyObject *__pyx_pf_4cupy_4cuda_6memory_13MemoryPointer_20copy_from_host_async(struct __pyx_obj_4cupy_4cuda_6memory_MemoryPointer *__pyx_v_self, PyObject *__pyx_v_mem, size_t __pyx_v_size, PyObject *__pyx_v_stream); /* proto */
static PyObject *__pyx_pf_4cupy_4cuda_6memory_13MemoryPointer_22copy_from(struct __pyx_obj_4cupy_4cuda_6memory_MemoryPointer *__pyx_v_self, PyObject *__pyx_v_mem, size_t __pyx_v_size); /* proto */
static PyObject *__pyx_pf_4cupy_4cuda_6memory_13MemoryPointer_24copy_from_async(struct __pyx_obj_4cupy_4cuda_6memory_MemoryPointer *__pyx_v_self, PyObject *__pyx_v_mem, size_t __pyx_v_size, PyObject *__pyx_v_stream); /* proto */
static PyObject *__pyx_pf_4cupy_4cuda_6memory_13MemoryPointer_26copy_to_host(struct __pyx_obj_4cupy_4cuda_6memory_MemoryPointer *__pyx_v_self, PyObject *__pyx_v_mem, size_t __pyx_v_size); /* proto */
static PyObject *__pyx_pf_4cupy_4cuda_6memory_13MemoryPointer_28copy_to_host_async(struct __pyx_obj_4cupy_4cuda_6memory_MemoryPointer *__pyx_v_self, PyObject *__pyx_v_mem, size_t __pyx_v_size, PyObject *__pyx_v_stream); /* proto */
static PyObject *__pyx_pf_4cupy_4cuda_6memory_13MemoryPointer_30memset(struct __pyx_obj_4cupy_4cuda_6memory_MemoryPointer *__pyx_v_self, int __pyx_v_value, size_t __pyx_v_size); /* proto */
static PyObject *__pyx_pf_4cupy_4cuda_6memory_13MemoryPointer_32memset_async(struct __pyx_obj_4cupy_4cuda_6memory_MemoryPointer *__pyx_v_self, int __pyx_v_value, size_t __pyx_v_size, PyObject *__pyx_v_stream); /* proto */
static PyObject *__pyx_pf_4cupy_4cuda_6memory_13MemoryPointer_3ptr___get__(struct __pyx_obj_4cupy_4cuda_6memory_MemoryPointer *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_4cupy_4cuda_6memory_13MemoryPointer_9device_id___get__(struct __pyx_obj_4cupy_4cuda_6memory_MemoryPointer *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_4cupy_4cuda_6memory_13MemoryPointer_3mem___get__(struct __pyx_obj_4cupy_4cuda_6memory_MemoryPointer *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_4cupy_4cuda_6memory_13MemoryPointer_34__reduce_cython__(struct __pyx_obj_4cupy_4cuda_6memory_MemoryPointer *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_4cupy_4cuda_6memory_13MemoryPointer_36__setstate_cython__(struct __pyx_obj_4cupy_4cuda_6memory_MemoryPointer *__pyx_v_self, PyObject *__pyx_v___pyx_state); /* proto */
static PyObject *__pyx_pf_4cupy_4cuda_6memory_2_malloc(CYTHON_UNUSED PyObject *__pyx_self, size_t __pyx_v_size); /* proto */
static PyObject *__pyx_pf_4cupy_4cuda_6memory_4malloc_async(CYTHON_UNUSED PyObject *__pyx_self, size_t __pyx_v_size); /* proto */
static PyObject *__pyx_pf_4cupy_4cuda_6memory_6malloc_managed(CYTHON_UNUSED PyObject *__pyx_self, size_t __pyx_v_size); /* proto */
static PyObject *__pyx_pf_4cupy_4cuda_6memory_8malloc_system(CYTHON_UNUSED PyObject *__pyx_self, size_t __pyx_v_size); /* proto */
static PyObject *__pyx_pf_4cupy_4cuda_6memory_10_get_thread_local_allocator(CYTHON_UNUSED PyObject *__pyx_self); /* proto */
static PyObject *__pyx_pf_4cupy_4cuda_6memory_12_set_thread_local_allocator(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_allocator); /* proto */
static PyObject *__pyx_pf_4cupy_4cuda_6memory_14alloc(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_size); /* proto */
static PyObject *__pyx_pf_4cupy_4cuda_6memory_16set_allocator(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_allocator); /* proto */
static PyObject *__pyx_pf_4cupy_4cuda_6memory_18get_allocator(CYTHON_UNUSED PyObject *__pyx_self); /* proto */
static int __pyx_pf_4cupy_4cuda_6memory_12PooledMemory___init__(struct __pyx_obj_4cupy_4cuda_6memory_PooledMemory *__pyx_v_self, struct __pyx_obj_4cupy_4cuda_6memory__Chunk *__pyx_v_chunk, PyObject *__pyx_v_pool); /* proto */
static PyObject *__pyx_pf_4cupy_4cuda_6memory_12PooledMemory_2free(struct __pyx_obj_4cupy_4cuda_6memory_PooledMemory *__pyx_v_self); /* proto */
static void __pyx_pf_4cupy_4cuda_6memory_12PooledMemory_4__dealloc__(struct __pyx_obj_4cupy_4cuda_6memory_PooledMemory *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_4cupy_4cuda_6memory_12PooledMemory_4pool___get__(struct __pyx_obj_4cupy_4cuda_6memory_PooledMemory *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_4cupy_4cuda_6memory_12PooledMemory_8identity___get__(struct __pyx_obj_4cupy_4cuda_6memory_PooledMemory *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_4cupy_4cuda_6memory_12PooledMemory_6__reduce_cython__(struct __pyx_obj_4cupy_4cuda_6memory_PooledMemory *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_4cupy_4cuda_6memory_12PooledMemory_8__setstate_cython__(struct __pyx_obj_4cupy_4cuda_6memory_PooledMemory *__pyx_v_self, PyObject *__pyx_v___pyx_state); /* proto */
static PyObject *__pyx_pf_4cupy_4cuda_6memory_20_round_size(CYTHON_UNUSED PyObject *__pyx_self, size_t __pyx_v_size); /* proto */
static int __pyx_pf_4cupy_4cuda_6memory_6_Arena___cinit__(struct __pyx_obj_4cupy_4cuda_6memory__Arena *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_4cupy_4cuda_6memory_6_Arena_2__reduce_cython__(CYTHON_UNUSED struct __pyx_obj_4cupy_4cuda_6memory__Arena *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_4cupy_4cuda_6memory_6_Arena_4__setstate_cython__(CYTHON_UNUSED struct __pyx_obj_4cupy_4cuda_6memory__Arena *__pyx_v_self, CYTHON_UNUSED PyObject *__pyx_v___pyx_state); /* proto */
static PyObject *__pyx_pf_4cupy_4cuda_6memory_22_parse_limit_string(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_limit); /* proto */
static int __pyx_pf_4cupy_4cuda_6memory_22SingleDeviceMemoryPool___init__(struct __pyx_obj_4cupy_4cuda_6memory_SingleDeviceMemoryPool *__pyx_v_self, PyObject *__pyx_v_allocator); /* proto */
static PyObject *__pyx_pf_4cupy_4cuda_6memory_22SingleDeviceMemoryPool_2_arena(struct __pyx_obj_4cupy_4cuda_6memory_SingleDeviceMemoryPool *__pyx_v_self, intptr_t __pyx_v_stream_ident); /* proto */
static PyObject *__pyx_pf_4cupy_4cuda_6memory_22SingleDeviceMemoryPool_4_debug_arena_get_index(struct __pyx_obj_4cupy_4cuda_6memory_SingleDeviceMemoryPool *__pyx_v_self, intptr_t __pyx_v_stream_ident); /* proto */
static PyObject *__pyx_pf_4cupy_4cuda_6memory_22SingleDeviceMemoryPool_6malloc(struct __pyx_obj_4cupy_4cuda_6memory_SingleDeviceMemoryPool *__pyx_v_self, size_t __pyx_v_size); /* proto */
static PyObject *__pyx_pf_4cupy_4cuda_6memory_22SingleDeviceMemoryPool_8free_all_blocks(struct __pyx_obj_4cupy_4cuda_6memory_SingleDeviceMemoryPool *__pyx_v_self, PyObject *__pyx_v_stream); /* proto */
static PyObject *__pyx_pf_4cupy_4cuda_6memory_22SingleDeviceMemoryPool_10free_all_free(struct __pyx_obj_4cupy_4cuda_6memory_SingleDeviceMemoryPool *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_4cupy_4cuda_6memory_22SingleDeviceMemoryPool_12n_free_blocks(struct __pyx_obj_4cupy_4cuda_6memory_SingleDeviceMemoryPool *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_4cupy_4cuda_6memory_22SingleDeviceMemoryPool_14used_bytes(struct __pyx_obj_4cupy_4cuda_6memory_SingleDeviceMemoryPool *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_4cupy_4cuda_6memory_22SingleDeviceMemoryPool_16free_bytes(struct __pyx_obj_4cupy_4cuda_6memory_SingleDeviceMemoryPool *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_4cupy_4cuda_6memory_22SingleDeviceMemoryPool_18total_bytes(struct __pyx_obj_4cupy_4cuda_6memory_SingleDeviceMemoryPool *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_4cupy_4cuda_6memory_22SingleDeviceMemoryPool_20set_limit(struct __pyx_obj_4cupy_4cuda_6memory_SingleDeviceMemoryPool *__pyx_v_self, PyObject *__pyx_v_size, PyObject *__pyx_v_fraction); /* proto */
static PyObject *__pyx_pf_4cupy_4cuda_6memory_22SingleDeviceMemoryPool_22get_limit(struct __pyx_obj_4cupy_4cuda_6memory_SingleDeviceMemoryPool *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_4cupy_4cuda_6memory_22SingleDeviceMemoryPool_10_device_id___get__(struct __pyx_obj_4cupy_4cuda_6memory_SingleDeviceMemoryPool *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_4cupy_4cuda_6memory_22SingleDeviceMemoryPool_24__reduce_cython__(CYTHON_UNUSED struct __pyx_obj_4cupy_4cuda_6memory_SingleDeviceMemoryPool *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_4cupy_4cuda_6memory_22SingleDeviceMemoryPool_26__setstate_cython__(CYTHON_UNUSED struct __pyx_obj_4cupy_4cuda_6memory_SingleDeviceMemoryPool *__pyx_v_self, CYTHON_UNUSED PyObject *__pyx_v___pyx_state); /* proto */
static int __pyx_pf_4cupy_4cuda_6memory_10MemoryPool___init__(struct __pyx_obj_4cupy_4cuda_6memory_MemoryPool *__pyx_v_self, PyObject *__pyx_v_allocator); /* proto */
static PyObject *__pyx_pf_4cupy_4cuda_6memory_10MemoryPool_36_ensure_pools_and_return_device_pool_genexpr(PyObject *__pyx_self, int __pyx_genexpr_arg_0); /* proto */
static PyObject *__pyx_pf_4cupy_4cuda_6memory_10MemoryPool_2malloc(struct __pyx_obj_4cupy_4cuda_6memory_MemoryPool *__pyx_v_self, size_t __pyx_v_size); /* proto */
static PyObject *__pyx_pf_4cupy_4cuda_6memory_10MemoryPool_4free_all_blocks(struct __pyx_obj_4cupy_4cuda_6memory_MemoryPool *__pyx_v_self, PyObject *__pyx_v_stream); /* proto */
static PyObject *__pyx_pf_4cupy_4cuda_6memory_10MemoryPool_6free_all_free(struct __pyx_obj_4cupy_4cuda_6memory_MemoryPool *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_4cupy_4cuda_6memory_10MemoryPool_8n_free_blocks(struct __pyx_obj_4cupy_4cuda_6memory_MemoryPool *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_4cupy_4cuda_6memory_10MemoryPool_10used_bytes(struct __pyx_obj_4cupy_4cuda_6memory_MemoryPool *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_4cupy_4cuda_6memory_10MemoryPool_12free_bytes(struct __pyx_obj_4cupy_4cuda_6memory_MemoryPool *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_4cupy_4cuda_6memory_10MemoryPool_14total_bytes(struct __pyx_obj_4cupy_4cuda_6memory_MemoryPool *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_4cupy_4cuda_6memory_10MemoryPool_16set_limit(struct __pyx_obj_4cupy_4cuda_6memory_MemoryPool *__pyx_v_self, PyObject *__pyx_v_size, PyObject *__pyx_v_fraction); /* proto */
static PyObject *__pyx_pf_4cupy_4cuda_6memory_10MemoryPool_18get_limit(struct __pyx_obj_4cupy_4cuda_6memory_MemoryPool *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_4cupy_4cuda_6memory_10MemoryPool_20__reduce_cython__(struct __pyx_obj_4cupy_4cuda_6memory_MemoryPool *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_4cupy_4cuda_6memory_10MemoryPool_22__setstate_cython__(struct __pyx_obj_4cupy_4cuda_6memory_MemoryPool *__pyx_v_self, PyObject *__pyx_v___pyx_state); /* proto */
static int __pyx_pf_4cupy_4cuda_6memory_15MemoryAsyncPool___init__(struct __pyx_obj_4cupy_4cuda_6memory_MemoryAsyncPool *__pyx_v_self, PyObject *__pyx_v_pool_handles); /* proto */
static PyObject *__pyx_pf_4cupy_4cuda_6memory_15MemoryAsyncPool_2malloc(struct __pyx_obj_4cupy_4cuda_6memory_MemoryAsyncPool *__pyx_v_self, size_t __pyx_v_size); /* proto */
static PyObject *__pyx_pf_4cupy_4cuda_6memory_15MemoryAsyncPool_4free_all_blocks(struct __pyx_obj_4cupy_4cuda_6memory_MemoryAsyncPool *__pyx_v_self, PyObject *__pyx_v_stream); /* proto */
static PyObject *__pyx_pf_4cupy_4cuda_6memory_15MemoryAsyncPool_6n_free_blocks(struct __pyx_obj_4cupy_4cuda_6memory_MemoryAsyncPool *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_4cupy_4cuda_6memory_15MemoryAsyncPool_8used_bytes(struct __pyx_obj_4cupy_4cuda_6memory_MemoryAsyncPool *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_4cupy_4cuda_6memory_15MemoryAsyncPool_10free_bytes(struct __pyx_obj_4cupy_4cuda_6memory_MemoryAsyncPool *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_4cupy_4cuda_6memory_15MemoryAsyncPool_12total_bytes(struct __pyx_obj_4cupy_4cuda_6memory_MemoryAsyncPool *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_4cupy_4cuda_6memory_15MemoryAsyncPool_14set_limit(struct __pyx_obj_4cupy_4cuda_6memory_MemoryAsyncPool *__pyx_v_self, PyObject *__pyx_v_size, PyObject *__pyx_v_fraction); /* proto */
static PyObject *__pyx_pf_4cupy_4cuda_6memory_15MemoryAsyncPool_16get_limit(struct __pyx_obj_4cupy_4cuda_6memory_MemoryAsyncPool *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_4cupy_4cuda_6memory_15MemoryAsyncPool_6_pools___get__(struct __pyx_obj_4cupy_4cuda_6memory_MemoryAsyncPool *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_4cupy_4cuda_6memory_15MemoryAsyncPool_18memoryAsyncHasStat___get__(struct __pyx_obj_4cupy_4cuda_6memory_MemoryAsyncPool *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_4cupy_4cuda_6memory_15MemoryAsyncPool_18__reduce_cython__(struct __pyx_obj_4cupy_4cuda_6memory_MemoryAsyncPool *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_4cupy_4cuda_6memory_15MemoryAsyncPool_20__setstate_cython__(struct __pyx_obj_4cupy_4cuda_6memory_MemoryAsyncPool *__pyx_v_self, PyObject *__pyx_v___pyx_state); /* proto */
static int __pyx_pf_4cupy_4cuda_6memory_24CFunctionAllocatorMemory___init__(struct __pyx_obj_4cupy_4cuda_6memory_CFunctionAllocatorMemory *__pyx_v_self, size_t __pyx_v_size, intptr_t __pyx_v_param, intptr_t __pyx_v_malloc_func, intptr_t __pyx_v_free_func, int __pyx_v_device_id); /* proto */
static void __pyx_pf_4cupy_4cuda_6memory_24CFunctionAllocatorMemory_2__dealloc__(struct __pyx_obj_4cupy_4cuda_6memory_CFunctionAllocatorMemory *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_4cupy_4cuda_6memory_24CFunctionAllocatorMemory_4__reduce_cython__(struct __pyx_obj_4cupy_4cuda_6memory_CFunctionAllocatorMemory *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_4cupy_4cuda_6memory_24CFunctionAllocatorMemory_6__setstate_cython__(struct __pyx_obj_4cupy_4cuda_6memory_CFunctionAllocatorMemory *__pyx_v_self, PyObject *__pyx_v___pyx_state); /* proto */
static int __pyx_pf_4cupy_4cuda_6memory_18CFunctionAllocator___init__(struct __pyx_obj_4cupy_4cuda_6memory_CFunctionAllocator *__pyx_v_self, intptr_t __pyx_v_param, intptr_t __pyx_v_malloc_func, intptr_t __pyx_v_free_func, PyObject *__pyx_v_owner); /* proto */
static PyObject *__pyx_pf_4cupy_4cuda_6memory_18CFunctionAllocator_2malloc(struct __pyx_obj_4cupy_4cuda_6memory_CFunctionAllocator *__pyx_v_self, size_t __pyx_v_size); /* proto */
static PyObject *__pyx_pf_4cupy_4cuda_6memory_18CFunctionAllocator_4__reduce_cython__(struct __pyx_obj_4cupy_4cuda_6memory_CFunctionAllocator *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_4cupy_4cuda_6memory_18CFunctionAllocator_6__setstate_cython__(struct __pyx_obj_4cupy_4cuda_6memory_CFunctionAllocator *__pyx_v_self, PyObject *__pyx_v___pyx_state); /* proto */
static int __pyx_pf_4cupy_4cuda_6memory_29PythonFunctionAllocatorMemory___init__(struct __pyx_obj_4cupy_4cuda_6memory_PythonFunctionAllocatorMemory *__pyx_v_self, size_t __pyx_v_size, PyObject *__pyx_v_malloc_func, PyObject *__pyx_v_free_func, int __pyx_v_device_id); /* proto */
static void __pyx_pf_4cupy_4cuda_6memory_29PythonFunctionAllocatorMemory_2__dealloc__(struct __pyx_obj_4cupy_4cuda_6memory_PythonFunctionAllocatorMemory *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_4cupy_4cuda_6memory_29PythonFunctionAllocatorMemory_4__reduce_cython__(struct __pyx_obj_4cupy_4cuda_6memory_PythonFunctionAllocatorMemory *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_4cupy_4cuda_6memory_29PythonFunctionAllocatorMemory_6__setstate_cython__(struct __pyx_obj_4cupy_4cuda_6memory_PythonFunctionAllocatorMemory *__pyx_v_self, PyObject *__pyx_v___pyx_state); /* proto */
static int __pyx_pf_4cupy_4cuda_6memory_23PythonFunctionAllocator___init__(struct __pyx_obj_4cupy_4cuda_6memory_PythonFunctionAllocator *__pyx_v_self, PyObject *__pyx_v_malloc_func, PyObject *__pyx_v_free_func); /* proto */
static PyObject *__pyx_pf_4cupy_4cuda_6memory_23PythonFunctionAllocator_2malloc(struct __pyx_obj_4cupy_4cuda_6memory_PythonFunctionAllocator *__pyx_v_self, size_t __pyx_v_size); /* proto */
static PyObject *__pyx_pf_4cupy_4cuda_6memory_23PythonFunctionAllocator_4__reduce_cython__(struct __pyx_obj_4cupy_4cuda_6memory_PythonFunctionAllocator *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_4cupy_4cuda_6memory_23PythonFunctionAllocator_6__setstate_cython__(struct __pyx_obj_4cupy_4cuda_6memory_PythonFunctionAllocator *__pyx_v_self, PyObject *__pyx_v___pyx_state); /* proto */
static PyObject *__pyx_pf_4cupy_4cuda_6memory_24__pyx_unpickle_BaseMemory(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v___pyx_type, long __pyx_v___pyx_checksum, PyObject *__pyx_v___pyx_state); /* proto */
static PyObject *__pyx_pf_4cupy_4cuda_6memory_26__pyx_unpickle_Memory(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v___pyx_type, long __pyx_v___pyx_checksum, PyObject *__pyx_v___pyx_state); /* proto */
static PyObject *__pyx_pf_4cupy_4cuda_6memory_28__pyx_unpickle_MemoryAsync(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v___pyx_type, long __pyx_v___pyx_checksum, PyObject *__pyx_v___pyx_state); /* proto */
static PyObject *__pyx_pf_4cupy_4cuda_6memory_30__pyx_unpickle_UnownedMemory(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v___pyx_type, long __pyx_v___pyx_checksum, PyObject *__pyx_v___pyx_state); /* proto */
static PyObject *__pyx_pf_4cupy_4cuda_6memory_32__pyx_unpickle_ManagedMemory(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v___pyx_type, long __pyx_v___pyx_checksum, PyObject *__pyx_v___pyx_state); /* proto */
static PyObject *__pyx_pf_4cupy_4cuda_6memory_34__pyx_unpickle_SystemMemory(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v___pyx_type, long __pyx_v___pyx_checksum, PyObject *__pyx_v___pyx_state); /* proto */
static PyObject *__pyx_pf_4cupy_4cuda_6memory_36__pyx_unpickle__Chunk(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v___pyx_type, long __pyx_v___pyx_checksum, PyObject *__pyx_v___pyx_state); /* proto */
static PyObject *__pyx_pf_4cupy_4cuda_6memory_38__pyx_unpickle_MemoryPointer(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v___pyx_type, long __pyx_v___pyx_checksum, PyObject *__pyx_v___pyx_state); /* proto */
static PyObject *__pyx_pf_4cupy_4cuda_6memory_40__pyx_unpickle_PooledMemory(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v___pyx_type, long __pyx_v___pyx_checksum, PyObject *__pyx_v___pyx_state); /* proto */
static PyObject *__pyx_pf_4cupy_4cuda_6memory_42__pyx_unpickle_MemoryPool(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v___pyx_type, long __pyx_v___pyx_checksum, PyObject *__pyx_v___pyx_state); /* proto */
static PyObject *__pyx_pf_4cupy_4cuda_6memory_44__pyx_unpickle_MemoryAsyncPool(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v___pyx_type, long __pyx_v___pyx_checksum, PyObject *__pyx_v___pyx_state); /* proto */
static PyObject *__pyx_pf_4cupy_4cuda_6memory_46__pyx_unpickle_CFunctionAllocatorMemory(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v___pyx_type, long __pyx_v___pyx_checksum, PyObject *__pyx_v___pyx_state); /* proto */
static PyObject *__pyx_pf_4cupy_4cuda_6memory_48__pyx_unpickle_CFunctionAllocator(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v___pyx_type, long __pyx_v___pyx_checksum, PyObject *__pyx_v___pyx_state); /* proto */
static PyObject *__pyx_pf_4cupy_4cuda_6memory_50__pyx_unpickle_PythonFunctionAllocatorMemory(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v___pyx_type, long __pyx_v___pyx_checksum, PyObject *__pyx_v___pyx_state); /* proto */
static PyObject *__pyx_pf_4cupy_4cuda_6memory_52__pyx_unpickle_PythonFunctionAllocator(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v___pyx_type, long __pyx_v___pyx_checksum, PyObject *__pyx_v___pyx_state); /* proto */
static PyObject *__pyx_tp_new_4cupy_4cuda_6memory_BaseMemory(PyTypeObject *t, PyObject *a, PyObject *k); /*proto*/
static PyObject *__pyx_tp_new_4cupy_4cuda_6memory_SystemMemory(PyTypeObject *t, PyObject *a, PyObject *k); /*proto*/
static PyObject *__pyx_tp_new_4cupy_4cuda_6memory_MemoryPointer(PyTypeObject *t, PyObject *a, PyObject *k); /*proto*/
static PyObject *__pyx_tp_new_4cupy_4cuda_6memory_MemoryPool(PyTypeObject *t, PyObject *a, PyObject *k); /*proto*/
static PyObject *__pyx_tp_new_4cupy_4cuda_6memory_CFunctionAllocator(PyTypeObject *t, PyObject *a, PyObject *k); /*proto*/
static PyObject *__pyx_tp_new_4cupy_4cuda_6memory_PythonFunctionAllocatorMemory(PyTypeObject *t, PyObject *a, PyObject *k); /*proto*/
static PyObject *__pyx_tp_new_4cupy_4cuda_6memory_PythonFunctionAllocator(PyTypeObject *t, PyObject *a, PyObject *k); /*proto*/
static PyObject *__pyx_tp_new_4cupy_4cuda_6memory_MemoryAsync(PyTypeObject *t, PyObject *a, PyObject *k); /*proto*/
static PyObject *__pyx_tp_new_4cupy_4cuda_6memory_UnownedMemory(PyTypeObject *t, PyObject *a, PyObject *k); /*proto*/
static PyObject *__pyx_tp_new_4cupy_4cuda_6memory__Chunk(PyTypeObject *t, PyObject *a, PyObject *k); /*proto*/
static PyObject *__pyx_tp_new_4cupy_4cuda_6memory_PooledMemory(PyTypeObject *t, PyObject *a, PyObject *k); /*proto*/
static PyObject *__pyx_tp_new_4cupy_4cuda_6memory__Arena(PyTypeObject *t, PyObject *a, PyObject *k); /*proto*/
static PyObject *__pyx_tp_new_4cupy_4cuda_6memory_SingleDeviceMemoryPool(PyTypeObject *t, PyObject *a, PyObject *k); /*proto*/
static PyObject *__pyx_tp_new_4cupy_4cuda_6memory_MemoryAsyncPool(PyTypeObject *t, PyObject *a, PyObject *k); /*proto*/
static PyObject *__pyx_tp_new_4cupy_4cuda_6memory___pyx_scope_struct___ensure_pools_and_return_device_pool(PyTypeObject *t, PyObject *a, PyObject *k); /*proto*/
static PyObject *__pyx_tp_new_4cupy_4cuda_6memory___pyx_scope_struct_1_genexpr(PyTypeObject *t, PyObject *a, PyObject *k); /*proto*/
/* #### Code section: late_includes ### */

    #include <utility>
    #include <exception>

    namespace {

    CYTHON_UNUSED PyGILState_STATE __pyx_libcpp_mutex_limited_api_ensure_gil() {
        #if CYTHON_COMPILING_IN_LIMITED_API
        if ((__PYX_LIMITED_VERSION_HEX < 0x030d0000) && __Pyx_get_runtime_version() < 0x030d0000) {
            return PyGILState_Ensure();
        }
        #endif
        return PyGILState_LOCKED;  // Unused
    }

    #if CYTHON_COMPILING_IN_LIMITED_API && __PYX_LIMITED_VERSION_HEX < 0x030d0000
    CYTHON_UNUSED void __pyx_libcpp_mutex_limited_api_release_gil(PyGILState_STATE gil_state) {
        if (__Pyx_get_runtime_version() < 0x030d0000)
            PyGILState_Release(gil_state);
    }
    #else
    #define __pyx_libcpp_mutex_limited_api_release_gil(ignore) (void)ignore
    #endif

    CYTHON_UNUSED int __pyx_libcpp_mutex_has_gil() {
        #if CYTHON_COMPILING_IN_LIMITED_API
            if ((__PYX_LIMITED_VERSION_HEX >= 0x030d0000) || __Pyx_get_runtime_version() >= 0x030d0000) {
                // In 3.13+ we can temporarily give up the GIL to find out what the thread state was
                PyThreadState *ts = PyThreadState_Swap(NULL);
                if (ts) {
                    PyThreadState_Swap(ts);
                    return 1;
                }
                return 0;
            }
            /* There is no way to know if we have the GIL. Therefore the only
            * thing we can safely do is make absolutely sure that we have it
            * in (__pyx_libcpp_mutex_limited_api_ensure_gil).
            */
            return 1;
        #elif PY_VERSION_HEX >= 0x030d0000
            return PyThreadState_GetUnchecked() != NULL;
        #elif PY_VERSION_HEX >= 0x030C0000
            return _PyThreadState_UncheckedGet() != NULL;
        #else
            return PyGILState_Check();
        #endif
    }

    template <typename F>
    class __pyx_libcpp_mutex_cleanup_on_exit {
        F on_exit;
        bool invoke = true;

    public:
        explicit __pyx_libcpp_mutex_cleanup_on_exit(F f)
            : on_exit(f)
        {}
        __pyx_libcpp_mutex_cleanup_on_exit(__pyx_libcpp_mutex_cleanup_on_exit &&rhs)
            : on_exit(std::move(rhs.on_exit))
            , invoke(rhs.invoke)
        {
            rhs.invoke = false;
        }

        __pyx_libcpp_mutex_cleanup_on_exit(const __pyx_libcpp_mutex_cleanup_on_exit&) = delete;
        __pyx_libcpp_mutex_cleanup_on_exit& operator=(const __pyx_libcpp_mutex_cleanup_on_exit&) = delete;

        ~__pyx_libcpp_mutex_cleanup_on_exit() {
            if (invoke)
                on_exit();
        }
    };

    template<typename F>
    __pyx_libcpp_mutex_cleanup_on_exit<F> __pyx_make_libcpp_mutex_cleanup_on_exit(F f) {
        return __pyx_libcpp_mutex_cleanup_on_exit<F>(std::move(f));
    }

    template <typename Callable, typename ... Args>
    void __pyx_cpp_py_safe_call_once(std::once_flag& flag, Callable&& callable, Args&&... args) {
        class PyException : public std::exception {
        public:
            using std::exception::exception;
        };

        __Pyx_UnknownThreadState thread_state = __Pyx_SaveUnknownThread();
        auto on_exit = __pyx_make_libcpp_mutex_cleanup_on_exit(
            [&]() {
                __Pyx_RestoreUnknownThread(thread_state);
            });

        try {
            std::call_once(flag,
                [&](Args&& ...args) {
                    // Make sure we have the GIL
                    PyGILState_STATE gil_state;
                    int had_gil_on_call = __Pyx_UnknownThreadStateDefinitelyHadGil(thread_state);
                    if (had_gil_on_call) {
                        __Pyx_RestoreUnknownThread(thread_state);
                    } else {
                        gil_state = PyGILState_Ensure();
                    }
                    auto on_callable_exit = __pyx_make_libcpp_mutex_cleanup_on_exit(
                        [&]() {
                            if (had_gil_on_call) {
                                thread_state = __Pyx_SaveUnknownThread();
                            } else {
                                PyGILState_Release(gil_state);
                            }
                        });

                    std::forward<Callable>(callable)(std::forward<Args>(args)...);
                    if (PyErr_Occurred()) {
                        throw PyException();
                    }
                },
                std::forward<Args>(args)...
            );
        } catch (const PyException&) {
            // Do nothing
        }
    }

    CYTHON_UNUSED void __pyx_cpp_py_safe_call_object_once(std::once_flag& flag, PyObject *py_callable) {
        auto callable = [py_callable]() {
            auto result = PyObject_CallObject(py_callable, nullptr);
            Py_XDECREF(result);
        };
        __pyx_cpp_py_safe_call_once(flag, callable);
    }

    template <typename LockableT>
    void __pyx_std_lock_wrapper(LockableT& arg0) {
        // std::lock only handles 2 or more arguments.
        // So create a 1 argument version.
        arg0.lock();
    }

    template <typename Lockable0T, typename Lockable1T, typename ... Lockables>
    void __pyx_std_lock_wrapper(Lockable0T& arg0, Lockable1T& arg1, Lockables&... args) {
        std::lock(arg0, arg1, args...);
    }

    CYTHON_UNUSED inline void __pyx_libcpp_mutex_unlock() {} // no-op

    template <typename Lockable0T, typename ... Lockables>
    void __pyx_libcpp_mutex_unlock(Lockable0T& arg0, Lockables&... locks) {
        arg0.unlock();
        __pyx_libcpp_mutex_unlock(locks...);
    }

    template <typename LockableT>
    int __pyx_std_try_lock_wrapper(LockableT& arg0) {
        // std::try_lock only handles 2 or more arguments.
        // So create a 1 argument version.
        if (arg0.try_lock()) {
            return -1;
        }
        return 0;
    }

    template <typename Lockable0T, typename Lockable1T, typename ...Lockables>
    int __pyx_std_try_lock_wrapper(Lockable0T& arg0, Lockable1T& arg1, Lockables&... args) {
        return std::try_lock(arg0, arg1, args...);
    }

    template <typename... LockTs>
    void __pyx_py_safe_std_lock_release_lock_reacquire(LockTs& ...locks) {
        // Release the GIL, acquire the lock, then reacquire the GIL.
        // This is safe provided the user never holds the GIL while trying
        // to reacquire the lock (i.e. it's safe provided they always use
        // the py-safe wrappers).
        PyThreadState *_save;
        Py_UNBLOCK_THREADS
        try {
            __pyx_std_lock_wrapper(locks...);
        } catch (...) {
            // In this case, we probably can't reason about the state of the locks but we can at least
            // make sure the GIL is consistent.
            Py_BLOCK_THREADS
            throw;
        }
        Py_BLOCK_THREADS
        return;
    }

    template <typename... LockTs>
    void __pyx_py_safe_std_lock(LockTs& ...locks) {
        PyGILState_STATE gil_state = __pyx_libcpp_mutex_limited_api_ensure_gil();
        int had_gil_on_call = __pyx_libcpp_mutex_has_gil();
        auto on_exit = __pyx_make_libcpp_mutex_cleanup_on_exit(
            [&]() {
                __pyx_libcpp_mutex_limited_api_release_gil(gil_state);
            });
        if (!had_gil_on_call) {
            // Nothing special to do - just lock and quit
            __pyx_std_lock_wrapper(locks...);
            return;
        }

        // It's a real shame there's no try_lock for the GIL, otherwise
        // we could just defer this whole thing to c++ std::lock.
        if (__pyx_std_try_lock_wrapper(locks...) == -1) {
            // success!
            return;
        }
        __pyx_py_safe_std_lock_release_lock_reacquire(locks...);
    }

    template <typename MutexT>
    std::unique_lock<MutexT> __pyx_py_safe_construct_unique_lock(MutexT& mutex) {
        std::unique_lock<MutexT> l{mutex, std::defer_lock};
        __pyx_py_safe_std_lock(l);
        return l;
    }
    } // namespace
    
/* #### Code section: module_state ### */
/* SmallCodeConfig */
#ifndef CYTHON_SMALL_CODE
#if defined(__clang__)
    #define CYTHON_SMALL_CODE
#elif defined(__GNUC__) && (__GNUC__ > 4 || (__GNUC__ == 4 && __GNUC_MINOR__ >= 3))
    #define CYTHON_SMALL_CODE __attribute__((cold))
#else
    #define CYTHON_SMALL_CODE
#endif
#endif

typedef struct {
  PyObject *__pyx_d;
  PyObject *__pyx_b;
  PyObject *__pyx_cython_runtime;
  PyObject *__pyx_empty_tuple;
  PyObject *__pyx_empty_bytes;
  PyObject *__pyx_empty_unicode;
  PyTypeObject *__pyx_ptype_4cupy_4cuda_6device_Handle;
  PyTypeObject *__pyx_ptype_4cupy_4cuda_6device_Device;
  PyTypeObject *__pyx_ptype_7cpython_4type_type;
  PyTypeObject *__pyx_ptype_7cpython_4bool_bool;
  PyTypeObject *__pyx_ptype_7cpython_7complex_complex;
  PyTypeObject *__pyx_ptype_13cupy_backends_4cuda_3api_7runtime_PointerAttributes;
  PyTypeObject *__pyx_ptype_13cupy_backends_4cuda_3api_7runtime_MemPoolProps;
  PyObject *__pyx_type_4cupy_4cuda_6memory_BaseMemory;
  PyObject *__pyx_type_4cupy_4cuda_6memory_SystemMemory;
  PyObject *__pyx_type_4cupy_4cuda_6memory_MemoryPointer;
  PyObject *__pyx_type_4cupy_4cuda_6memory_MemoryPool;
  PyObject *__pyx_type_4cupy_4cuda_6memory_CFunctionAllocatorMemory;
  PyObject *__pyx_type_4cupy_4cuda_6memory_CFunctionAllocator;
  PyObject *__pyx_type_4cupy_4cuda_6memory_PythonFunctionAllocatorMemory;
  PyObject *__pyx_type_4cupy_4cuda_6memory_PythonFunctionAllocator;
  PyObject *__pyx_type_4cupy_4cuda_6memory_Memory;
  PyObject *__pyx_type_4cupy_4cuda_6memory_MemoryAsync;
  PyObject *__pyx_type_4cupy_4cuda_6memory_UnownedMemory;
  PyObject *__pyx_type_4cupy_4cuda_6memory_ManagedMemory;
  PyObject *__pyx_type_4cupy_4cuda_6memory__Chunk;
  PyObject *__pyx_type_4cupy_4cuda_6memory_PooledMemory;
  PyObject *__pyx_type_4cupy_4cuda_6memory__Arena;
  PyObject *__pyx_type_4cupy_4cuda_6memory_SingleDeviceMemoryPool;
  PyObject *__pyx_type_4cupy_4cuda_6memory_MemoryAsyncPool;
  PyObject *__pyx_type_4cupy_4cuda_6memory___pyx_scope_struct___ensure_pools_and_return_device_pool;
  PyObject *__pyx_type_4cupy_4cuda_6memory___pyx_scope_struct_1_genexpr;
  PyTypeObject *__pyx_ptype_4cupy_4cuda_6memory_BaseMemory;
  PyTypeObject *__pyx_ptype_4cupy_4cuda_6memory_SystemMemory;
  PyTypeObject *__pyx_ptype_4cupy_4cuda_6memory_MemoryPointer;
  PyTypeObject *__pyx_ptype_4cupy_4cuda_6memory_MemoryPool;
  PyTypeObject *__pyx_ptype_4cupy_4cuda_6memory_CFunctionAllocatorMemory;
  PyTypeObject *__pyx_ptype_4cupy_4cuda_6memory_CFunctionAllocator;
  PyTypeObject *__pyx_ptype_4cupy_4cuda_6memory_PythonFunctionAllocatorMemory;
  PyTypeObject *__pyx_ptype_4cupy_4cuda_6memory_PythonFunctionAllocator;
  PyTypeObject *__pyx_ptype_4cupy_4cuda_6memory_Memory;
  PyTypeObject *__pyx_ptype_4cupy_4cuda_6memory_MemoryAsync;
  PyTypeObject *__pyx_ptype_4cupy_4cuda_6memory_UnownedMemory;
  PyTypeObject *__pyx_ptype_4cupy_4cuda_6memory_ManagedMemory;
  PyTypeObject *__pyx_ptype_4cupy_4cuda_6memory__Chunk;
  PyTypeObject *__pyx_ptype_4cupy_4cuda_6memory_PooledMemory;
  PyTypeObject *__pyx_ptype_4cupy_4cuda_6memory__Arena;
  PyTypeObject *__pyx_ptype_4cupy_4cuda_6memory_SingleDeviceMemoryPool;
  PyTypeObject *__pyx_ptype_4cupy_4cuda_6memory_MemoryAsyncPool;
  PyTypeObject *__pyx_ptype_4cupy_4cuda_6memory___pyx_scope_struct___ensure_pools_and_return_device_pool;
  PyTypeObject *__pyx_ptype_4cupy_4cuda_6memory___pyx_scope_struct_1_genexpr;
  __Pyx_CachedCFunction __pyx_umethod_PyDict_Type_get;
  __Pyx_CachedCFunction __pyx_umethod_PyDict_Type_items;
  __Pyx_CachedCFunction __pyx_umethod_PyDict_Type_keys;
  __Pyx_CachedCFunction __pyx_umethod_PyDict_Type_pop;
  __Pyx_CachedCFunction __pyx_umethod_PyDict_Type_values;
  __Pyx_CachedCFunction __pyx_umethod_PyList_Type_pop;
  int __pyx_k_;
  int __pyx_k__2;
  PyObject *__pyx_slice[1];
  PyObject *__pyx_tuple[2];
  PyObject *__pyx_codeobj_tab[3];
  PyObject *__pyx_string_tab[227];
  PyObject *__pyx_number_tab[16];
/* #### Code section: module_state_contents ### */

#if CYTHON_USE_FREELISTS
struct __pyx_obj_4cupy_4cuda_6memory___pyx_scope_struct___ensure_pools_and_return_device_pool *__pyx_freelist_4cupy_4cuda_6memory___pyx_scope_struct___ensure_pools_and_return_device_pool[8];
int __pyx_freecount_4cupy_4cuda_6memory___pyx_scope_struct___ensure_pools_and_return_device_pool;
#endif

#if CYTHON_USE_FREELISTS
struct __pyx_obj_4cupy_4cuda_6memory___pyx_scope_struct_1_genexpr *__pyx_freelist_4cupy_4cuda_6memory___pyx_scope_struct_1_genexpr[8];
int __pyx_freecount_4cupy_4cuda_6memory___pyx_scope_struct_1_genexpr;
#endif
/* CommonTypesMetaclass.module_state_decls */
PyTypeObject *__pyx_CommonTypesMetaclassType;

/* CachedMethodType.module_state_decls */
#if CYTHON_COMPILING_IN_LIMITED_API
PyObject *__Pyx_CachedMethodType;
#endif

/* CythonFunctionShared.module_state_decls */
PyTypeObject *__pyx_CyFunctionType;

/* CodeObjectCache.module_state_decls */
struct __Pyx_CodeObjectCache __pyx_code_cache;

/* IterNextPlain.module_state_decls */
#if CYTHON_COMPILING_IN_LIMITED_API && __PYX_LIMITED_VERSION_HEX < 0x030A0000
PyObject *__Pyx_GetBuiltinNext_LimitedAPI_cache;
#endif

/* Generator.module_state_decls */
PyTypeObject *__pyx_GeneratorType;

/* #### Code section: module_state_end ### */
} __pyx_mstatetype;

#if CYTHON_USE_MODULE_STATE
#ifdef __cplusplus
namespace {
extern struct PyModuleDef __pyx_moduledef;
} /* anonymous namespace */
#else
static struct PyModuleDef __pyx_moduledef;
#endif

#define __pyx_mstate_global (__Pyx_PyModule_GetState(__Pyx_State_FindModule(&__pyx_moduledef)))

#define __pyx_m (__Pyx_State_FindModule(&__pyx_moduledef))
#else
static __pyx_mstatetype __pyx_mstate_global_static =
#ifdef __cplusplus
    {};
#else
    {0};
#endif
static __pyx_mstatetype * const __pyx_mstate_global = &__pyx_mstate_global_static;
#endif
/* #### Code section: constant_name_defines ### */
#define __pyx_kp_u_0 __pyx_string_tab[0]
#define __pyx_kp_u_0x_x_device_mem_r __pyx_string_tab[1]
#define __pyx_kp_u_Can_t_change_the_global_allocato __pyx_string_tab[2]
#define __pyx_kp_u_Device_does_not_support_malloc_a __pyx_string_tab[3]
#define __pyx_kp_u_HIP_does_not_support_memory_asyn __pyx_string_tab[4]
#define __pyx_kp_u_Managed_memory_requires_ROCm_4_3 __pyx_string_tab[5]
#define __pyx_kp_u_None __pyx_string_tab[6]
#define __pyx_kp_u_Note_that_Cython_is_deliberately __pyx_string_tab[7]
#define __pyx_kp_u_Out_of_memory_allocating_bytes __pyx_string_tab[8]
#define __pyx_kp_u_Out_of_memory_allocating_bytes_a __pyx_string_tab[9]
#define __pyx_kp_u_Out_of_memory_error_Args_size_in __pyx_string_tab[10]
#define __pyx_kp_u_The_driver_version_is_insufficie __pyx_string_tab[11]
#define __pyx_kp_u_This_function_is_not_supported_i __pyx_string_tab[12]
#define __pyx_kp_u_UnownedMemory_requires_explicit __pyx_string_tab[13]
#define __pyx_kp_u__3 __pyx_string_tab[14]
#define __pyx_kp_u__4 __pyx_string_tab[15]
#define __pyx_kp_u__5 __pyx_string_tab[16]
#define __pyx_kp_u__6 __pyx_string_tab[17]
#define __pyx_kp_u__7 __pyx_string_tab[18]
#define __pyx_kp_u_add_note __pyx_string_tab[19]
#define __pyx_kp_u_allocated_so_far_bytes __pyx_string_tab[20]
#define __pyx_kp_u_at __pyx_string_tab[21]
#define __pyx_kp_u_cudaMemPoolCreate_is_not_yet_sup __pyx_string_tab[22]
#define __pyx_kp_u_cupy_cuda_memory_pyx __pyx_string_tab[23]
#define __pyx_kp_u_disable __pyx_string_tab[24]
#define __pyx_kp_u_enable __pyx_string_tab[25]
#define __pyx_kp_u_free_all_free_is_deprecated_Use __pyx_string_tab[26]
#define __pyx_kp_u_gc __pyx_string_tab[27]
#define __pyx_kp_u_handle_must_be_default_for_the_d __pyx_string_tab[28]
#define __pyx_kp_u_isenabled __pyx_string_tab[29]
#define __pyx_kp_u_mem __pyx_string_tab[30]
#define __pyx_kp_u_memory_limit_fraction_out_of_ran __pyx_string_tab[31]
#define __pyx_kp_u_memory_limit_size_out_of_range __pyx_string_tab[32]
#define __pyx_kp_u_must_provide_an_owner __pyx_string_tab[33]
#define __pyx_kp_u_next __pyx_string_tab[34]
#define __pyx_kp_u_no_default___reduce___due_to_non __pyx_string_tab[35]
#define __pyx_kp_u_offset __pyx_string_tab[36]
#define __pyx_kp_u_prev __pyx_string_tab[37]
#define __pyx_kp_u_self__arena_mutex_self__in_use_b __pyx_string_tab[38]
#define __pyx_kp_u_size_and_fraction_cannot_be_spec __pyx_string_tab[39]
#define __pyx_kp_u_the_current_stream_is_capturing __pyx_string_tab[40]
#define __pyx_kp_u_the_current_stream_is_capturing_2 __pyx_string_tab[41]
#define __pyx_kp_u_the_current_stream_is_capturing_3 __pyx_string_tab[42]
#define __pyx_kp_u_unknown __pyx_string_tab[43]
#define __pyx_n_u_Arena __pyx_string_tab[44]
#define __pyx_n_u_BaseMemory __pyx_string_tab[45]
#define __pyx_n_u_CFunctionAllocator __pyx_string_tab[46]
#define __pyx_n_u_CFunctionAllocatorMemory __pyx_string_tab[47]
#define __pyx_n_u_CUDARuntimeError __pyx_string_tab[48]
#define __pyx_n_u_CUPY_ENABLE_UMP __pyx_string_tab[49]
#define __pyx_n_u_CUPY_GPU_MEMORY_LIMIT __pyx_string_tab[50]
#define __pyx_n_u_Chunk __pyx_string_tab[51]
#define __pyx_n_u_ManagedMemory __pyx_string_tab[52]
#define __pyx_n_u_Memory __pyx_string_tab[53]
#define __pyx_n_u_MemoryAsync __pyx_string_tab[54]
#define __pyx_n_u_MemoryAsyncPool __pyx_string_tab[55]
#define __pyx_n_u_MemoryPointer __pyx_string_tab[56]
#define __pyx_n_u_MemoryPool __pyx_string_tab[57]
#define __pyx_n_u_MemoryPool__ensure_pools_and_ret __pyx_string_tab[58]
#define __pyx_n_u_None __pyx_string_tab[59]
#define __pyx_n_u_OutOfMemoryError __pyx_string_tab[60]
#define __pyx_n_u_OutOfMemoryError___init __pyx_string_tab[61]
#define __pyx_n_u_OutOfMemoryError___reduce __pyx_string_tab[62]
#define __pyx_n_u_PooledMemory __pyx_string_tab[63]
#define __pyx_n_u_PythonFunctionAllocator __pyx_string_tab[64]
#define __pyx_n_u_PythonFunctionAllocatorMemory __pyx_string_tab[65]
#define __pyx_n_u_Pyx_PyDict_NextRef __pyx_string_tab[66]
#define __pyx_n_u_SingleDeviceMemoryPool __pyx_string_tab[67]
#define __pyx_n_u_SystemMemory __pyx_string_tab[68]
#define __pyx_n_u_UnownedMemory __pyx_string_tab[69]
#define __pyx_n_u_advise __pyx_string_tab[70]
#define __pyx_n_u_alloc_postprocess __pyx_string_tab[71]
#define __pyx_n_u_alloc_preprocess __pyx_string_tab[72]
#define __pyx_n_u_allocation_unit_size __pyx_string_tab[73]
#define __pyx_n_u_allocator __pyx_string_tab[74]
#define __pyx_n_u_asyncio_coroutines __pyx_string_tab[75]
#define __pyx_n_u_atexit __pyx_string_tab[76]
#define __pyx_n_u_chunk __pyx_string_tab[77]
#define __pyx_n_u_class __pyx_string_tab[78]
#define __pyx_n_u_cline_in_traceback __pyx_string_tab[79]
#define __pyx_n_u_close __pyx_string_tab[80]
#define __pyx_n_u_collect __pyx_string_tab[81]
#define __pyx_n_u_copy_from __pyx_string_tab[82]
#define __pyx_n_u_copy_from_async __pyx_string_tab[83]
#define __pyx_n_u_copy_from_device __pyx_string_tab[84]
#define __pyx_n_u_copy_from_device_async __pyx_string_tab[85]
#define __pyx_n_u_copy_from_host __pyx_string_tab[86]
#define __pyx_n_u_copy_from_host_async __pyx_string_tab[87]
#define __pyx_n_u_copy_to_host __pyx_string_tab[88]
#define __pyx_n_u_copy_to_host_async __pyx_string_tab[89]
#define __pyx_n_u_create __pyx_string_tab[90]
#define __pyx_n_u_cupy_backends_cuda_api_runtime __pyx_string_tab[91]
#define __pyx_n_u_cupy_cuda_memory __pyx_string_tab[92]
#define __pyx_n_u_current __pyx_string_tab[93]
#define __pyx_n_u_default __pyx_string_tab[94]
#define __pyx_n_u_dev __pyx_string_tab[95]
#define __pyx_n_u_device_id __pyx_string_tab[96]
#define __pyx_n_u_device_support_async_alloc __pyx_string_tab[97]
#define __pyx_n_u_dict __pyx_string_tab[98]
#define __pyx_n_u_doc __pyx_string_tab[99]
#define __pyx_n_u_endswith __pyx_string_tab[100]
#define __pyx_n_u_environ __pyx_string_tab[101]
#define __pyx_n_u_exit __pyx_string_tab[102]
#define __pyx_n_u_format __pyx_string_tab[103]
#define __pyx_n_u_fraction __pyx_string_tab[104]
#define __pyx_n_u_free_all_blocks __pyx_string_tab[105]
#define __pyx_n_u_free_all_free __pyx_string_tab[106]
#define __pyx_n_u_free_bytes __pyx_string_tab[107]
#define __pyx_n_u_free_func __pyx_string_tab[108]
#define __pyx_n_u_free_postprocess __pyx_string_tab[109]
#define __pyx_n_u_free_preprocess __pyx_string_tab[110]
#define __pyx_n_u_func __pyx_string_tab[111]
#define __pyx_n_u_gc __pyx_string_tab[112]
#define __pyx_n_u_genexpr __pyx_string_tab[113]
#define __pyx_n_u_get __pyx_string_tab[114]
#define __pyx_n_u_get_limit __pyx_string_tab[115]
#define __pyx_n_u_get_thread_local_allocator __pyx_string_tab[116]
#define __pyx_n_u_getstate __pyx_string_tab[117]
#define __pyx_n_u_i __pyx_string_tab[118]
#define __pyx_n_u_iadd __pyx_string_tab[119]
#define __pyx_n_u_id __pyx_string_tab[120]
#define __pyx_n_u_init __pyx_string_tab[121]
#define __pyx_n_u_is_coroutine __pyx_string_tab[122]
#define __pyx_n_u_items __pyx_string_tab[123]
#define __pyx_n_u_itervalues __pyx_string_tab[124]
#define __pyx_n_u_keys __pyx_string_tab[125]
#define __pyx_n_u_limit __pyx_string_tab[126]
#define __pyx_n_u_limit_2 __pyx_string_tab[127]
#define __pyx_n_u_local __pyx_string_tab[128]
#define __pyx_n_u_main __pyx_string_tab[129]
#define __pyx_n_u_malloc __pyx_string_tab[130]
#define __pyx_n_u_malloc_2 __pyx_string_tab[131]
#define __pyx_n_u_malloc_func __pyx_string_tab[132]
#define __pyx_n_u_malloc_postprocess __pyx_string_tab[133]
#define __pyx_n_u_malloc_preprocess __pyx_string_tab[134]
#define __pyx_n_u_mem_2 __pyx_string_tab[135]
#define __pyx_n_u_mem_ptr __pyx_string_tab[136]
#define __pyx_n_u_mem_size __pyx_string_tab[137]
#define __pyx_n_u_memset __pyx_string_tab[138]
#define __pyx_n_u_memset_async __pyx_string_tab[139]
#define __pyx_n_u_metaclass __pyx_string_tab[140]
#define __pyx_n_u_module __pyx_string_tab[141]
#define __pyx_n_u_mro_entries __pyx_string_tab[142]
#define __pyx_n_u_msg __pyx_string_tab[143]
#define __pyx_n_u_n_free_blocks __pyx_string_tab[144]
#define __pyx_n_u_name __pyx_string_tab[145]
#define __pyx_n_u_new __pyx_string_tab[146]
#define __pyx_n_u_next_2 __pyx_string_tab[147]
#define __pyx_n_u_object __pyx_string_tab[148]
#define __pyx_n_u_offset_2 __pyx_string_tab[149]
#define __pyx_n_u_os __pyx_string_tab[150]
#define __pyx_n_u_owner __pyx_string_tab[151]
#define __pyx_n_u_param __pyx_string_tab[152]
#define __pyx_n_u_pmem_id __pyx_string_tab[153]
#define __pyx_n_u_pool __pyx_string_tab[154]
#define __pyx_n_u_pool_handles __pyx_string_tab[155]
#define __pyx_n_u_pop __pyx_string_tab[156]
#define __pyx_n_u_prefetch __pyx_string_tab[157]
#define __pyx_n_u_prepare __pyx_string_tab[158]
#define __pyx_n_u_ptr __pyx_string_tab[159]
#define __pyx_n_u_pyx_capi __pyx_string_tab[160]
#define __pyx_n_u_pyx_checksum __pyx_string_tab[161]
#define __pyx_n_u_pyx_state __pyx_string_tab[162]
#define __pyx_n_u_pyx_type __pyx_string_tab[163]
#define __pyx_n_u_pyx_unpickle_BaseMemory __pyx_string_tab[164]
#define __pyx_n_u_pyx_unpickle_CFunctionAllocato __pyx_string_tab[165]
#define __pyx_n_u_pyx_unpickle_CFunctionAllocato_2 __pyx_string_tab[166]
#define __pyx_n_u_pyx_unpickle_ManagedMemory __pyx_string_tab[167]
#define __pyx_n_u_pyx_unpickle_Memory __pyx_string_tab[168]
#define __pyx_n_u_pyx_unpickle_MemoryAsync __pyx_string_tab[169]
#define __pyx_n_u_pyx_unpickle_MemoryAsyncPool __pyx_string_tab[170]
#define __pyx_n_u_pyx_unpickle_MemoryPointer __pyx_string_tab[171]
#define __pyx_n_u_pyx_unpickle_MemoryPool __pyx_string_tab[172]
#define __pyx_n_u_pyx_unpickle_PooledMemory __pyx_string_tab[173]
#define __pyx_n_u_pyx_unpickle_PythonFunctionAll __pyx_string_tab[174]
#define __pyx_n_u_pyx_unpickle_PythonFunctionAll_2 __pyx_string_tab[175]
#define __pyx_n_u_pyx_unpickle_SystemMemory __pyx_string_tab[176]
#define __pyx_n_u_pyx_unpickle_UnownedMemory __pyx_string_tab[177]
#define __pyx_n_u_pyx_unpickle__Chunk __pyx_string_tab[178]
#define __pyx_n_u_pyx_vtable __pyx_string_tab[179]
#define __pyx_n_u_qualname __pyx_string_tab[180]
#define __pyx_n_u_reduce __pyx_string_tab[181]
#define __pyx_n_u_reduce_cython __pyx_string_tab[182]
#define __pyx_n_u_reduce_ex __pyx_string_tab[183]
#define __pyx_n_u_ref __pyx_string_tab[184]
#define __pyx_n_u_register __pyx_string_tab[185]
#define __pyx_n_u_self __pyx_string_tab[186]
#define __pyx_n_u_send __pyx_string_tab[187]
#define __pyx_n_u_set_limit __pyx_string_tab[188]
#define __pyx_n_u_set_name __pyx_string_tab[189]
#define __pyx_n_u_set_thread_local_allocator __pyx_string_tab[190]
#define __pyx_n_u_setdefault __pyx_string_tab[191]
#define __pyx_n_u_setstate __pyx_string_tab[192]
#define __pyx_n_u_setstate_cython __pyx_string_tab[193]
#define __pyx_n_u_size __pyx_string_tab[194]
#define __pyx_n_u_size_2 __pyx_string_tab[195]
#define __pyx_n_u_src __pyx_string_tab[196]
#define __pyx_n_u_status __pyx_string_tab[197]
#define __pyx_n_u_stream __pyx_string_tab[198]
#define __pyx_n_u_stream_ident __pyx_string_tab[199]
#define __pyx_n_u_super __pyx_string_tab[200]
#define __pyx_n_u_synchronize __pyx_string_tab[201]
#define __pyx_n_u_test __pyx_string_tab[202]
#define __pyx_n_u_threading __pyx_string_tab[203]
#define __pyx_n_u_throw __pyx_string_tab[204]
#define __pyx_n_u_tid __pyx_string_tab[205]
#define __pyx_n_u_tid_obj __pyx_string_tab[206]
#define __pyx_n_u_total __pyx_string_tab[207]
#define __pyx_n_u_total_2 __pyx_string_tab[208]
#define __pyx_n_u_total_bytes __pyx_string_tab[209]
#define __pyx_n_u_update __pyx_string_tab[210]
#define __pyx_n_u_used_bytes __pyx_string_tab[211]
#define __pyx_n_u_value __pyx_string_tab[212]
#define __pyx_n_u_values __pyx_string_tab[213]
#define __pyx_n_u_warn __pyx_string_tab[214]
#define __pyx_n_u_warnings __pyx_string_tab[215]
#define __pyx_n_u_weakref __pyx_string_tab[216]
#define __pyx_kp_b_PyObject_int___pyx_skip_dispatch __pyx_string_tab[217]
#define __pyx_kp_b_PyObject_int___pyx_skip_dispatch_2 __pyx_string_tab[218]
#define __pyx_kp_b_PyObject_int___pyx_skip_dispatch_3 __pyx_string_tab[219]
#define __pyx_kp_b_PyObject_int___pyx_skip_dispatch_4 __pyx_string_tab[220]
#define __pyx_kp_b_PyObject_int___pyx_skip_dispatch_5 __pyx_string_tab[221]
#define __pyx_kp_b_int__is_hip_environment_deviceAt __pyx_string_tab[222]
#define __pyx_kp_b_int_int___pyx_skip_dispatch_int __pyx_string_tab[223]
#define __pyx_kp_b_iso88591_A_AXT_Yd __pyx_string_tab[224]
#define __pyx_kp_b_iso88591_A_IQ_Ja_Ja_6_A_7waq_vT_9_31F_Q __pyx_string_tab[225]
#define __pyx_kp_b_iso88591__8 __pyx_string_tab[226]
#define __pyx_int_0 __pyx_number_tab[0]
#define __pyx_int_neg_1 __pyx_number_tab[1]
#define __pyx_int_1 __pyx_number_tab[2]
#define __pyx_int_14780958 __pyx_number_tab[3]
#define __pyx_int_40300000 __pyx_number_tab[4]
#define __pyx_int_58206641 __pyx_number_tab[5]
#define __pyx_int_71104427 __pyx_number_tab[6]
#define __pyx_int_88874970 __pyx_number_tab[7]
#define __pyx_int_102542768 __pyx_number_tab[8]
#define __pyx_int_120372003 __pyx_number_tab[9]
#define __pyx_int_127029364 __pyx_number_tab[10]
#define __pyx_int_177953241 __pyx_number_tab[11]
#define __pyx_int_216617047 __pyx_number_tab[12]
#define __pyx_int_228764240 __pyx_number_tab[13]
#define __pyx_int_240196946 __pyx_number_tab[14]
#define __pyx_int_255047627 __pyx_number_tab[15]
/* #### Code section: module_state_clear ### */
#if CYTHON_USE_MODULE_STATE
static CYTHON_SMALL_CODE int __pyx_m_clear(PyObject *m) {
  __pyx_mstatetype *clear_module_state = __Pyx_PyModule_GetState(m);
  if (!clear_module_state) return 0;
  Py_CLEAR(clear_module_state->__pyx_d);
  Py_CLEAR(clear_module_state->__pyx_b);
  Py_CLEAR(clear_module_state->__pyx_cython_runtime);
  Py_CLEAR(clear_module_state->__pyx_empty_tuple);
  Py_CLEAR(clear_module_state->__pyx_empty_bytes);
  Py_CLEAR(clear_module_state->__pyx_empty_unicode);
  #if CYTHON_PEP489_MULTI_PHASE_INIT
  __Pyx_State_RemoveModule(NULL);
  #endif
  Py_CLEAR(clear_module_state->__pyx_ptype_4cupy_4cuda_6device_Handle);
  Py_CLEAR(clear_module_state->__pyx_ptype_4cupy_4cuda_6device_Device);
  Py_CLEAR(clear_module_state->__pyx_ptype_7cpython_4type_type);
  Py_CLEAR(clear_module_state->__pyx_ptype_7cpython_4bool_bool);
  Py_CLEAR(clear_module_state->__pyx_ptype_7cpython_7complex_complex);
  Py_CLEAR(clear_module_state->__pyx_ptype_13cupy_backends_4cuda_3api_7runtime_PointerAttributes);
  Py_CLEAR(clear_module_state->__pyx_ptype_13cupy_backends_4cuda_3api_7runtime_MemPoolProps);
  Py_CLEAR(clear_module_state->__pyx_ptype_4cupy_4cuda_6memory_BaseMemory);
  Py_CLEAR(clear_module_state->__pyx_type_4cupy_4cuda_6memory_BaseMemory);
  Py_CLEAR(clear_module_state->__pyx_ptype_4cupy_4cuda_6memory_SystemMemory);
  Py_CLEAR(clear_module_state->__pyx_type_4cupy_4cuda_6memory_SystemMemory);
  Py_CLEAR(clear_module_state->__pyx_ptype_4cupy_4cuda_6memory_MemoryPointer);
  Py_CLEAR(clear_module_state->__pyx_type_4cupy_4cuda_6memory_MemoryPointer);
  Py_CLEAR(clear_module_state->__pyx_ptype_4cupy_4cuda_6memory_MemoryPool);
  Py_CLEAR(clear_module_state->__pyx_type_4cupy_4cuda_6memory_MemoryPool);
  Py_CLEAR(clear_module_state->__pyx_ptype_4cupy_4cuda_6memory_CFunctionAllocatorMemory);
  Py_CLEAR(clear_module_state->__pyx_type_4cupy_4cuda_6memory_CFunctionAllocatorMemory);
  Py_CLEAR(clear_module_state->__pyx_ptype_4cupy_4cuda_6memory_CFunctionAllocator);
  Py_CLEAR(clear_module_state->__pyx_type_4cupy_4cuda_6memory_CFunctionAllocator);
  Py_CLEAR(clear_module_state->__pyx_ptype_4cupy_4cuda_6memory_PythonFunctionAllocatorMemory);
  Py_CLEAR(clear_module_state->__pyx_type_4cupy_4cuda_6memory_PythonFunctionAllocatorMemory);
  Py_CLEAR(clear_module_state->__pyx_ptype_4cupy_4cuda_6memory_PythonFunctionAllocator);
  Py_CLEAR(clear_module_state->__pyx_type_4cupy_4cuda_6memory_PythonFunctionAllocator);
  Py_CLEAR(clear_module_state->__pyx_ptype_4cupy_4cuda_6memory_Memory);
  Py_CLEAR(clear_module_state->__pyx_type_4cupy_4cuda_6memory_Memory);
  Py_CLEAR(clear_module_state->__pyx_ptype_4cupy_4cuda_6memory_MemoryAsync);
  Py_CLEAR(clear_module_state->__pyx_type_4cupy_4cuda_6memory_MemoryAsync);
  Py_CLEAR(clear_module_state->__pyx_ptype_4cupy_4cuda_6memory_UnownedMemory);
  Py_CLEAR(clear_module_state->__pyx_type_4cupy_4cuda_6memory_UnownedMemory);
  Py_CLEAR(clear_module_state->__pyx_ptype_4cupy_4cuda_6memory_ManagedMemory);
  Py_CLEAR(clear_module_state->__pyx_type_4cupy_4cuda_6memory_ManagedMemory);
  Py_CLEAR(clear_module_state->__pyx_ptype_4cupy_4cuda_6memory__Chunk);
  Py_CLEAR(clear_module_state->__pyx_type_4cupy_4cuda_6memory__Chunk);
  Py_CLEAR(clear_module_state->__pyx_ptype_4cupy_4cuda_6memory_PooledMemory);
  Py_CLEAR(clear_module_state->__pyx_type_4cupy_4cuda_6memory_PooledMemory);
  Py_CLEAR(clear_module_state->__pyx_ptype_4cupy_4cuda_6memory__Arena);
  Py_CLEAR(clear_module_state->__pyx_type_4cupy_4cuda_6memory__Arena);
  Py_CLEAR(clear_module_state->__pyx_ptype_4cupy_4cuda_6memory_SingleDeviceMemoryPool);
  Py_CLEAR(clear_module_state->__pyx_type_4cupy_4cuda_6memory_SingleDeviceMemoryPool);
  Py_CLEAR(clear_module_state->__pyx_ptype_4cupy_4cuda_6memory_MemoryAsyncPool);
  Py_CLEAR(clear_module_state->__pyx_type_4cupy_4cuda_6memory_MemoryAsyncPool);
  Py_CLEAR(clear_module_state->__pyx_ptype_4cupy_4cuda_6memory___pyx_scope_struct___ensure_pools_and_return_device_pool);
  Py_CLEAR(clear_module_state->__pyx_type_4cupy_4cuda_6memory___pyx_scope_struct___ensure_pools_and_return_device_pool);
  Py_CLEAR(clear_module_state->__pyx_ptype_4cupy_4cuda_6memory___pyx_scope_struct_1_genexpr);
  Py_CLEAR(clear_module_state->__pyx_type_4cupy_4cuda_6memory___pyx_scope_struct_1_genexpr);
  for (int i=0; i<1; ++i) { Py_CLEAR(clear_module_state->__pyx_slice[i]); }
  for (int i=0; i<2; ++i) { Py_CLEAR(clear_module_state->__pyx_tuple[i]); }
  for (int i=0; i<3; ++i) { Py_CLEAR(clear_module_state->__pyx_codeobj_tab[i]); }
  for (int i=0; i<227; ++i) { Py_CLEAR(clear_module_state->__pyx_string_tab[i]); }
  for (int i=0; i<16; ++i) { Py_CLEAR(clear_module_state->__pyx_number_tab[i]); }
/* #### Code section: module_state_clear_contents ### */
/* CommonTypesMetaclass.module_state_clear */
Py_CLEAR(clear_module_state->__pyx_CommonTypesMetaclassType);

/* CythonFunctionShared.module_state_clear */
Py_CLEAR(clear_module_state->__pyx_CyFunctionType);

/* Generator.module_state_clear */
Py_CLEAR(clear_module_state->__pyx_GeneratorType);

/* #### Code section: module_state_clear_end ### */
return 0;
}
#endif
/* #### Code section: module_state_traverse ### */
#if CYTHON_USE_MODULE_STATE
static CYTHON_SMALL_CODE int __pyx_m_traverse(PyObject *m, visitproc visit, void *arg) {
  __pyx_mstatetype *traverse_module_state = __Pyx_PyModule_GetState(m);
  if (!traverse_module_state) return 0;
  Py_VISIT(traverse_module_state->__pyx_d);
  Py_VISIT(traverse_module_state->__pyx_b);
  Py_VISIT(traverse_module_state->__pyx_cython_runtime);
  __Pyx_VISIT_CONST(traverse_module_state->__pyx_empty_tuple);
  __Pyx_VISIT_CONST(traverse_module_state->__pyx_empty_bytes);
  __Pyx_VISIT_CONST(traverse_module_state->__pyx_empty_unicode);
  Py_VISIT(traverse_module_state->__pyx_ptype_4cupy_4cuda_6device_Handle);
  Py_VISIT(traverse_module_state->__pyx_ptype_4cupy_4cuda_6device_Device);
  Py_VISIT(traverse_module_state->__pyx_ptype_7cpython_4type_type);
  Py_VISIT(traverse_module_state->__pyx_ptype_7cpython_4bool_bool);
  Py_VISIT(traverse_module_state->__pyx_ptype_7cpython_7complex_complex);
  Py_VISIT(traverse_module_state->__pyx_ptype_13cupy_backends_4cuda_3api_7runtime_PointerAttributes);
  Py_VISIT(traverse_module_state->__pyx_ptype_13cupy_backends_4cuda_3api_7runtime_MemPoolProps);
  Py_VISIT(traverse_module_state->__pyx_ptype_4cupy_4cuda_6memory_BaseMemory);
  Py_VISIT(traverse_module_state->__pyx_type_4cupy_4cuda_6memory_BaseMemory);
  Py_VISIT(traverse_module_state->__pyx_ptype_4cupy_4cuda_6memory_SystemMemory);
  Py_VISIT(traverse_module_state->__pyx_type_4cupy_4cuda_6memory_SystemMemory);
  Py_VISIT(traverse_module_state->__pyx_ptype_4cupy_4cuda_6memory_MemoryPointer);
  Py_VISIT(traverse_module_state->__pyx_type_4cupy_4cuda_6memory_MemoryPointer);
  Py_VISIT(traverse_module_state->__pyx_ptype_4cupy_4cuda_6memory_MemoryPool);
  Py_VISIT(traverse_module_state->__pyx_type_4cupy_4cuda_6memory_MemoryPool);
  Py_VISIT(traverse_module_state->__pyx_ptype_4cupy_4cuda_6memory_CFunctionAllocatorMemory);
  Py_VISIT(traverse_module_state->__pyx_type_4cupy_4cuda_6memory_CFunctionAllocatorMemory);
  Py_VISIT(traverse_module_state->__pyx_ptype_4cupy_4cuda_6memory_CFunctionAllocator);
  Py_VISIT(traverse_module_state->__pyx_type_4cupy_4cuda_6memory_CFunctionAllocator);
  Py_VISIT(traverse_module_state->__pyx_ptype_4cupy_4cuda_6memory_PythonFunctionAllocatorMemory);
  Py_VISIT(traverse_module_state->__pyx_type_4cupy_4cuda_6memory_PythonFunctionAllocatorMemory);
  Py_VISIT(traverse_module_state->__pyx_ptype_4cupy_4cuda_6memory_PythonFunctionAllocator);
  Py_VISIT(traverse_module_state->__pyx_type_4cupy_4cuda_6memory_PythonFunctionAllocator);
  Py_VISIT(traverse_module_state->__pyx_ptype_4cupy_4cuda_6memory_Memory);
  Py_VISIT(traverse_module_state->__pyx_type_4cupy_4cuda_6memory_Memory);
  Py_VISIT(traverse_module_state->__pyx_ptype_4cupy_4cuda_6memory_MemoryAsync);
  Py_VISIT(traverse_module_state->__pyx_type_4cupy_4cuda_6memory_MemoryAsync);
  Py_VISIT(traverse_module_state->__pyx_ptype_4cupy_4cuda_6memory_UnownedMemory);
  Py_VISIT(traverse_module_state->__pyx_type_4cupy_4cuda_6memory_UnownedMemory);
  Py_VISIT(traverse_module_state->__pyx_ptype_4cupy_4cuda_6memory_ManagedMemory);
  Py_VISIT(traverse_module_state->__pyx_type_4cupy_4cuda_6memory_ManagedMemory);
  Py_VISIT(traverse_module_state->__pyx_ptype_4cupy_4cuda_6memory__Chunk);
  Py_VISIT(traverse_module_state->__pyx_type_4cupy_4cuda_6memory__Chunk);
  Py_VISIT(traverse_module_state->__pyx_ptype_4cupy_4cuda_6memory_PooledMemory);
  Py_VISIT(traverse_module_state->__pyx_type_4cupy_4cuda_6memory_PooledMemory);
  Py_VISIT(traverse_module_state->__pyx_ptype_4cupy_4cuda_6memory__Arena);
  Py_VISIT(traverse_module_state->__pyx_type_4cupy_4cuda_6memory__Arena);
  Py_VISIT(traverse_module_state->__pyx_ptype_4cupy_4cuda_6memory_SingleDeviceMemoryPool);
  Py_VISIT(traverse_module_state->__pyx_type_4cupy_4cuda_6memory_SingleDeviceMemoryPool);
  Py_VISIT(traverse_module_state->__pyx_ptype_4cupy_4cuda_6memory_MemoryAsyncPool);
  Py_VISIT(traverse_module_state->__pyx_type_4cupy_4cuda_6memory_MemoryAsyncPool);
  Py_VISIT(traverse_module_state->__pyx_ptype_4cupy_4cuda_6memory___pyx_scope_struct___ensure_pools_and_return_device_pool);
  Py_VISIT(traverse_module_state->__pyx_type_4cupy_4cuda_6memory___pyx_scope_struct___ensure_pools_and_return_device_pool);
  Py_VISIT(traverse_module_state->__pyx_ptype_4cupy_4cuda_6memory___pyx_scope_struct_1_genexpr);
  Py_VISIT(traverse_module_state->__pyx_type_4cupy_4cuda_6memory___pyx_scope_struct_1_genexpr);
  for (int i=0; i<1; ++i) { __Pyx_VISIT_CONST(traverse_module_state->__pyx_slice[i]); }
  for (int i=0; i<2; ++i) { __Pyx_VISIT_CONST(traverse_module_state->__pyx_tuple[i]); }
  for (int i=0; i<3; ++i) { __Pyx_VISIT_CONST(traverse_module_state->__pyx_codeobj_tab[i]); }
  for (int i=0; i<227; ++i) { __Pyx_VISIT_CONST(traverse_module_state->__pyx_string_tab[i]); }
  for (int i=0; i<16; ++i) { __Pyx_VISIT_CONST(traverse_module_state->__pyx_number_tab[i]); }
/* #### Code section: module_state_traverse_contents ### */
/* CommonTypesMetaclass.module_state_traverse */
Py_VISIT(traverse_module_state->__pyx_CommonTypesMetaclassType);

/* CythonFunctionShared.module_state_traverse */
Py_VISIT(traverse_module_state->__pyx_CyFunctionType);

/* Generator.module_state_traverse */
Py_VISIT(traverse_module_state->__pyx_GeneratorType);

/* #### Code section: module_state_traverse_end ### */
return 0;
}
#endif
/* #### Code section: module_code ### */

/* "cupy/cuda/memory.pxd":77
 *     cdef _ensure_pools_and_return_device_pool(self)
 * 
 *     cdef inline device_pool(self):             # <<<<<<<<<<<<<<
 *         # This criticial section may not be needed in practice. But without it
 *         # `self._pools` could be in an inconsistent state (the `PyObject *`
*/

static CYTHON_INLINE PyObject *__pyx_f_4cupy_4cuda_6memory_10MemoryPool_device_pool(struct __pyx_obj_4cupy_4cuda_6memory_MemoryPool *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_t_2;
  int __pyx_t_3;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("device_pool", 0);

  /* "cupy/cuda/memory.pxd":82
 *         # or the tuple not fully written due to CPU/compiler optimizations).
 *         # Atomics would be another way to ensure complete safety here.
 *         with cython.critical_section(self):             # <<<<<<<<<<<<<<
 *             if self._pools is not None:
 *                 return self._pools[device.get_device_id()]
*/
  {
      __Pyx_PyCriticalSection __pyx_cs;
      __pyx_t_1 = ((PyObject *)__pyx_v_self);
      __Pyx_INCREF(__pyx_t_1);
      __Pyx_PyCriticalSection_Begin(&__pyx_cs, (PyObject*)__pyx_t_1);
      /*try:*/ {

        /* "cupy/cuda/memory.pxd":83
 *         # Atomics would be another way to ensure complete safety here.
 *         with cython.critical_section(self):
 *             if self._pools is not None:             # <<<<<<<<<<<<<<
 *                 return self._pools[device.get_device_id()]
 * 
*/
        __pyx_t_2 = (__pyx_v_self->_pools != ((PyObject*)Py_None));
        if (__pyx_t_2) {

          /* "cupy/cuda/memory.pxd":84
 *         with cython.critical_section(self):
 *             if self._pools is not None:
 *                 return self._pools[device.get_device_id()]             # <<<<<<<<<<<<<<
 * 
 *         return self._ensure_pools_and_return_device_pool()
*/
          __Pyx_XDECREF(__pyx_r);
          if (unlikely(__pyx_v_self->_pools == Py_None)) {
            PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
            __PYX_ERR(1, 84, __pyx_L4_error)
          }
          __pyx_t_3 = __pyx_f_4cupy_4cuda_6device_get_device_id(0); if (unlikely(__pyx_t_3 == ((int)-1) && PyErr_Occurred())) __PYX_ERR(1, 84, __pyx_L4_error)
          __pyx_t_4 = __Pyx_GetItemInt_Tuple(__pyx_v_self->_pools, __pyx_t_3, int, 1, __Pyx_PyLong_From_int, 0, 1, 1, 1, __Pyx_ReferenceSharing_SharedReference); if (unlikely(!__pyx_t_4)) __PYX_ERR(1, 84, __pyx_L4_error)
          __Pyx_GOTREF(__pyx_t_4);
          __pyx_r = __pyx_t_4;
          __pyx_t_4 = 0;
          __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
          goto __pyx_L3_return;

          /* "cupy/cuda/memory.pxd":83
 *         # Atomics would be another way to ensure complete safety here.
 *         with cython.critical_section(self):
 *             if self._pools is not None:             # <<<<<<<<<<<<<<
 *                 return self._pools[device.get_device_id()]
 * 
*/
        }
      }

      /* "cupy/cuda/memory.pxd":82
 *         # or the tuple not fully written due to CPU/compiler optimizations).
 *         # Atomics would be another way to ensure complete safety here.
 *         with cython.critical_section(self):             # <<<<<<<<<<<<<<
 *             if self._pools is not None:
 *                 return self._pools[device.get_device_id()]
*/
      /*finally:*/ {
        /*normal exit:*/{
          __Pyx_PyCriticalSection_End(&__pyx_cs);
          goto __pyx_L5;
        }
        __pyx_L3_return: {
          __pyx_t_5 = __pyx_r;
          __pyx_r = 0;
          __Pyx_PyCriticalSection_End(&__pyx_cs);
          __pyx_r = __pyx_t_5;
          __pyx_t_5 = 0;
          goto __pyx_L0;
        }
        __pyx_L4_error: {
          __Pyx_PyCriticalSection_End(&__pyx_cs);
          goto __pyx_L1_error;
        }
        __pyx_L5:;
      }
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  }

  /* "cupy/cuda/memory.pxd":86
 *                 return self._pools[device.get_device_id()]
 * 
 *         return self._ensure_pools_and_return_device_pool()             # <<<<<<<<<<<<<<
 * 
 * 
*/
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = ((struct __pyx_vtabstruct_4cupy_4cuda_6memory_MemoryPool *)__pyx_v_self->__pyx_vtab)->_ensure_pools_and_return_device_pool(__pyx_v_self); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 86, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "cupy/cuda/memory.pxd":77
 *     cdef _ensure_pools_and_return_device_pool(self)
 * 
 *     cdef inline device_pool(self):             # <<<<<<<<<<<<<<
 *         # This criticial section may not be needed in practice. But without it
 *         # `self._pools` could be in an inconsistent state (the `PyObject *`
*/

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_AddTraceback("cupy.cuda.memory.MemoryPool.device_pool", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "cpython/complex.pxd":20
 * 
 *         # unavailable in limited API
 *         @property             # <<<<<<<<<<<<<<
 *         @_cython.c_compile_guard("!CYTHON_COMPILING_IN_LIMITED_API")
 *         cdef inline double real(self) noexcept:
*/

#if !CYTHON_COMPILING_IN_LIMITED_API
static CYTHON_INLINE double __pyx_f_7cpython_7complex_7complex_4real_real(PyComplexObject *__pyx_v_self) {
  double __pyx_r;

  /* "cpython/complex.pxd":23
 *         @_cython.c_compile_guard("!CYTHON_COMPILING_IN_LIMITED_API")
 *         cdef inline double real(self) noexcept:
 *             return self.cval.real             # <<<<<<<<<<<<<<
 * 
 *         # unavailable in limited API
*/
  __pyx_r = __pyx_v_self->cval.real;
  goto __pyx_L0;

  /* "cpython/complex.pxd":20
 * 
 *         # unavailable in limited API
 *         @property             # <<<<<<<<<<<<<<
 *         @_cython.c_compile_guard("!CYTHON_COMPILING_IN_LIMITED_API")
 *         cdef inline double real(self) noexcept:
*/

  /* function exit code */
  __pyx_L0:;
  return __pyx_r;
}
#endif /*!(#if !CYTHON_COMPILING_IN_LIMITED_API)*/

/* "cpython/complex.pxd":26
 * 
 *         # unavailable in limited API
 *         @property             # <<<<<<<<<<<<<<
 *         @_cython.c_compile_guard("!CYTHON_COMPILING_IN_LIMITED_API")
 *         cdef inline double imag(self) noexcept:
*/

#if !CYTHON_COMPILING_IN_LIMITED_API
static CYTHON_INLINE double __pyx_f_7cpython_7complex_7complex_4imag_imag(PyComplexObject *__pyx_v_self) {
  double __pyx_r;

  /* "cpython/complex.pxd":29
 *         @_cython.c_compile_guard("!CYTHON_COMPILING_IN_LIMITED_API")
 *         cdef inline double imag(self) noexcept:
 *             return self.cval.imag             # <<<<<<<<<<<<<<
 * 
 *     # PyTypeObject PyComplex_Type
*/
  __pyx_r = __pyx_v_self->cval.imag;
  goto __pyx_L0;

  /* "cpython/complex.pxd":26
 * 
 *         # unavailable in limited API
 *         @property             # <<<<<<<<<<<<<<
 *         @_cython.c_compile_guard("!CYTHON_COMPILING_IN_LIMITED_API")
 *         cdef inline double imag(self) noexcept:
*/

  /* function exit code */
  __pyx_L0:;
  return __pyx_r;
}
#endif /*!(#if !CYTHON_COMPILING_IN_LIMITED_API)*/

/* "cpython/contextvars.pxd":115
 * 
 * 
 * @_cython.c_compile_guard("!CYTHON_COMPILING_IN_LIMITED_API")             # <<<<<<<<<<<<<<
 * cdef inline object get_value(var, default_value=None):
 *     """Return a new reference to the value of the context variable,
*/

#if !CYTHON_COMPILING_IN_LIMITED_API
static CYTHON_INLINE PyObject *__pyx_f_7cpython_11contextvars_get_value(PyObject *__pyx_v_var, struct __pyx_opt_args_7cpython_11contextvars_get_value *__pyx_optional_args) {

  /* "cpython/contextvars.pxd":116
 * 
 * @_cython.c_compile_guard("!CYTHON_COMPILING_IN_LIMITED_API")
 * cdef inline object get_value(var, default_value=None):             # <<<<<<<<<<<<<<
 *     """Return a new reference to the value of the context variable,
 *     or the default value of the context variable,
*/
  PyObject *__pyx_v_default_value = ((PyObject *)Py_None);
  PyObject *__pyx_v_value;
  PyObject *__pyx_v_pyvalue = NULL;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  int __pyx_t_2;
  PyObject *__pyx_t_3 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("get_value", 0);
  if (__pyx_optional_args) {
    if (__pyx_optional_args->__pyx_n > 0) {
      __pyx_v_default_value = __pyx_optional_args->default_value;
    }
  }

  /* "cpython/contextvars.pxd":121
 *     or None if no such value or default was found.
 *     """
 *     cdef PyObject *value = NULL             # <<<<<<<<<<<<<<
 *     PyContextVar_Get(var, NULL, &value)
 *     if value is NULL:
*/
  __pyx_v_value = NULL;

  /* "cpython/contextvars.pxd":122
 *     """
 *     cdef PyObject *value = NULL
 *     PyContextVar_Get(var, NULL, &value)             # <<<<<<<<<<<<<<
 *     if value is NULL:
 *         # context variable does not have a default
*/
  __pyx_t_1 = PyContextVar_Get(__pyx_v_var, NULL, (&__pyx_v_value)); if (unlikely(__pyx_t_1 == ((int)-1))) __PYX_ERR(2, 122, __pyx_L1_error)

  /* "cpython/contextvars.pxd":123
 *     cdef PyObject *value = NULL
 *     PyContextVar_Get(var, NULL, &value)
 *     if value is NULL:             # <<<<<<<<<<<<<<
 *         # context variable does not have a default
 *         pyvalue = default_value
*/
  __pyx_t_2 = (__pyx_v_value == NULL);
  if (__pyx_t_2) {

    /* "cpython/contextvars.pxd":125
 *     if value is NULL:
 *         # context variable does not have a default
 *         pyvalue = default_value             # <<<<<<<<<<<<<<
 *     else:
 *         # value or default value of context variable
*/
    __Pyx_INCREF(__pyx_v_default_value);
    __pyx_v_pyvalue = __pyx_v_default_value;

    /* "cpython/contextvars.pxd":123
 *     cdef PyObject *value = NULL
 *     PyContextVar_Get(var, NULL, &value)
 *     if value is NULL:             # <<<<<<<<<<<<<<
 *         # context variable does not have a default
 *         pyvalue = default_value
*/
    goto __pyx_L3;
  }

  /* "cpython/contextvars.pxd":128
 *     else:
 *         # value or default value of context variable
 *         pyvalue = <object>value             # <<<<<<<<<<<<<<
 *         Py_XDECREF(value)  # PyContextVar_Get() returned an owned reference as 'PyObject*'
 *     return pyvalue
*/
  /*else*/ {
    __pyx_t_3 = ((PyObject *)__pyx_v_value);
    __Pyx_INCREF(__pyx_t_3);
    __pyx_v_pyvalue = __pyx_t_3;
    __pyx_t_3 = 0;

    /* "cpython/contextvars.pxd":129
 *         # value or default value of context variable
 *         pyvalue = <object>value
 *         Py_XDECREF(value)  # PyContextVar_Get() returned an owned reference as 'PyObject*'             # <<<<<<<<<<<<<<
 *     return pyvalue
 * 
*/
    Py_XDECREF(__pyx_v_value);
  }
  __pyx_L3:;

  /* "cpython/contextvars.pxd":130
 *         pyvalue = <object>value
 *         Py_XDECREF(value)  # PyContextVar_Get() returned an owned reference as 'PyObject*'
 *     return pyvalue             # <<<<<<<<<<<<<<
 * 
 * 
*/
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_v_pyvalue);
  __pyx_r = __pyx_v_pyvalue;
  goto __pyx_L0;

  /* "cpython/contextvars.pxd":115
 * 
 * 
 * @_cython.c_compile_guard("!CYTHON_COMPILING_IN_LIMITED_API")             # <<<<<<<<<<<<<<
 * cdef inline object get_value(var, default_value=None):
 *     """Return a new reference to the value of the context variable,
*/

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_AddTraceback("cpython.contextvars.get_value", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_pyvalue);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}
#endif /*!(#if !CYTHON_COMPILING_IN_LIMITED_API)*/

/* "cpython/contextvars.pxd":133
 * 
 * 
 * @_cython.c_compile_guard("!CYTHON_COMPILING_IN_LIMITED_API")             # <<<<<<<<<<<<<<
 * cdef inline object get_value_no_default(var, default_value=None):
 *     """Return a new reference to the value of the context variable,
*/

#if !CYTHON_COMPILING_IN_LIMITED_API
static CYTHON_INLINE PyObject *__pyx_f_7cpython_11contextvars_get_value_no_default(PyObject *__pyx_v_var, struct __pyx_opt_args_7cpython_11contextvars_get_value_no_default *__pyx_optional_args) {

  /* "cpython/contextvars.pxd":134
 * 
 * @_cython.c_compile_guard("!CYTHON_COMPILING_IN_LIMITED_API")
 * cdef inline object get_value_no_default(var, default_value=None):             # <<<<<<<<<<<<<<
 *     """Return a new reference to the value of the context variable,
 *     or the provided default value if no such value was found.
*/
  PyObject *__pyx_v_default_value = ((PyObject *)Py_None);
  PyObject *__pyx_v_value;
  PyObject *__pyx_v_pyvalue = NULL;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("get_value_no_default", 0);
  if (__pyx_optional_args) {
    if (__pyx_optional_args->__pyx_n > 0) {
      __pyx_v_default_value = __pyx_optional_args->default_value;
    }
  }

  /* "cpython/contextvars.pxd":140
 *     Ignores the default value of the context variable, if any.
 *     """
 *     cdef PyObject *value = NULL             # <<<<<<<<<<<<<<
 *     PyContextVar_Get(var, <PyObject*>default_value, &value)
 *     # value of context variable or 'default_value'
*/
  __pyx_v_value = NULL;

  /* "cpython/contextvars.pxd":141
 *     """
 *     cdef PyObject *value = NULL
 *     PyContextVar_Get(var, <PyObject*>default_value, &value)             # <<<<<<<<<<<<<<
 *     # value of context variable or 'default_value'
 *     pyvalue = <object>value
*/
  __pyx_t_1 = PyContextVar_Get(__pyx_v_var, ((PyObject *)__pyx_v_default_value), (&__pyx_v_value)); if (unlikely(__pyx_t_1 == ((int)-1))) __PYX_ERR(2, 141, __pyx_L1_error)

  /* "cpython/contextvars.pxd":143
 *     PyContextVar_Get(var, <PyObject*>default_value, &value)
 *     # value of context variable or 'default_value'
 *     pyvalue = <object>value             # <<<<<<<<<<<<<<
 *     Py_XDECREF(value)  # PyContextVar_Get() returned an owned reference as 'PyObject*'
 *     return pyvalue
*/
  __pyx_t_2 = ((PyObject *)__pyx_v_value);
  __Pyx_INCREF(__pyx_t_2);
  __pyx_v_pyvalue = __pyx_t_2;
  __pyx_t_2 = 0;

  /* "cpython/contextvars.pxd":144
 *     # value of context variable or 'default_value'
 *     pyvalue = <object>value
 *     Py_XDECREF(value)  # PyContextVar_Get() returned an owned reference as 'PyObject*'             # <<<<<<<<<<<<<<
 *     return pyvalue
*/
  Py_XDECREF(__pyx_v_value);

  /* "cpython/contextvars.pxd":145
 *     pyvalue = <object>value
 *     Py_XDECREF(value)  # PyContextVar_Get() returned an owned reference as 'PyObject*'
 *     return pyvalue             # <<<<<<<<<<<<<<
*/
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_v_pyvalue);
  __pyx_r = __pyx_v_pyvalue;
  goto __pyx_L0;

  /* "cpython/contextvars.pxd":133
 * 
 * 
 * @_cython.c_compile_guard("!CYTHON_COMPILING_IN_LIMITED_API")             # <<<<<<<<<<<<<<
 * cdef inline object get_value_no_default(var, default_value=None):
 *     """Return a new reference to the value of the context variable,
*/

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_AddTraceback("cpython.contextvars.get_value_no_default", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_pyvalue);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}
#endif /*!(#if !CYTHON_COMPILING_IN_LIMITED_API)*/

/* "libcpp/mutex.pxd":142
 * 
 * 
 * cdef inline void _dummy_force_utility_code_inclusion() nogil:             # <<<<<<<<<<<<<<
 *     with nogil:
 *         pass
*/

static CYTHON_INLINE void __pyx_f_6libcpp_5mutex__dummy_force_utility_code_inclusion(void) {

  /* "libcpp/mutex.pxd":143
 * 
 * cdef inline void _dummy_force_utility_code_inclusion() nogil:
 *     with nogil:             # <<<<<<<<<<<<<<
 *         pass
 * 
*/
  {
      __Pyx_UnknownThreadState _save;
      _save = __Pyx_SaveUnknownThread();
      __Pyx_FastGIL_Remember();
      /*try:*/ {
      }
      /*finally:*/ {
        /*normal exit:*/{
          __Pyx_FastGIL_Forget();
          __Pyx_RestoreUnknownThread(_save);
          goto __pyx_L5;
        }
        __pyx_L5:;
      }
  }

  /* "libcpp/mutex.pxd":142
 * 
 * 
 * cdef inline void _dummy_force_utility_code_inclusion() nogil:             # <<<<<<<<<<<<<<
 *     with nogil:
 *         pass
*/

  /* function exit code */
}

/* "cupy/cuda/memory.pyx":50
 * 
 * 
 * @atexit.register             # <<<<<<<<<<<<<<
 * def _exit():
 *     _exit_mode = True
*/

/* Python wrapper */
static PyObject *__pyx_pw_4cupy_4cuda_6memory_1_exit(PyObject *__pyx_self, CYTHON_UNUSED PyObject *unused); /*proto*/
PyDoc_STRVAR(__pyx_doc_4cupy_4cuda_6memory__exit, "_exit()");
static PyMethodDef __pyx_mdef_4cupy_4cuda_6memory_1_exit = {"_exit", (PyCFunction)__pyx_pw_4cupy_4cuda_6memory_1_exit, METH_NOARGS, __pyx_doc_4cupy_4cuda_6memory__exit};
static PyObject *__pyx_pw_4cupy_4cuda_6memory_1_exit(PyObject *__pyx_self, CYTHON_UNUSED PyObject *unused) {
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("_exit (wrapper)", 0);
  __pyx_kwvalues = __Pyx_KwValues_VARARGS(__pyx_args, __pyx_nargs);
  __pyx_r = __pyx_pf_4cupy_4cuda_6memory__exit(__pyx_self);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4cupy_4cuda_6memory__exit(CYTHON_UNUSED PyObject *__pyx_self) {
  CYTHON_UNUSED int __pyx_v__exit_mode;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("_exit", 0);

  /* "cupy/cuda/memory.pyx":52
 * @atexit.register
 * def _exit():
 *     _exit_mode = True             # <<<<<<<<<<<<<<
 * 
 * 
*/
  __pyx_v__exit_mode = 1;

  /* "cupy/cuda/memory.pyx":50
 * 
 * 
 * @atexit.register             # <<<<<<<<<<<<<<
 * def _exit():
 *     _exit_mode = True
*/

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "cupy/cuda/memory.pyx":64
 *     """
 * 
 *     def __init__(self, size, total, limit=0):             # <<<<<<<<<<<<<<
 *         self._size = size
 *         self._total = total
*/

/* Python wrapper */
static PyObject *__pyx_pw_4cupy_4cuda_6memory_16OutOfMemoryError_1__init__(PyObject *__pyx_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
PyDoc_STRVAR(__pyx_doc_4cupy_4cuda_6memory_16OutOfMemoryError___init__, "OutOfMemoryError.__init__(self, size, total, limit=0)");
static PyMethodDef __pyx_mdef_4cupy_4cuda_6memory_16OutOfMemoryError_1__init__ = {"__init__", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_4cupy_4cuda_6memory_16OutOfMemoryError_1__init__, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_4cupy_4cuda_6memory_16OutOfMemoryError___init__};
static PyObject *__pyx_pw_4cupy_4cuda_6memory_16OutOfMemoryError_1__init__(PyObject *__pyx_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
) {
  PyObject *__pyx_v_self = 0;
  PyObject *__pyx_v_size = 0;
  PyObject *__pyx_v_total = 0;
  PyObject *__pyx_v_limit = 0;
  #if !CYTHON_METH_FASTCALL
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  #endif
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject* values[4] = {0,0,0,0};
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__init__ (wrapper)", 0);
  #if !CYTHON_METH_FASTCALL
  #if CYTHON_ASSUME_SAFE_SIZE
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return NULL;
  #endif
  #endif
  __pyx_kwvalues = __Pyx_KwValues_FASTCALL(__pyx_args, __pyx_nargs);
  {
    PyObject ** const __pyx_pyargnames[] = {&__pyx_mstate_global->__pyx_n_u_self,&__pyx_mstate_global->__pyx_n_u_size,&__pyx_mstate_global->__pyx_n_u_total,&__pyx_mstate_global->__pyx_n_u_limit,0};
    const Py_ssize_t __pyx_kwds_len = (__pyx_kwds) ? __Pyx_NumKwargs_FASTCALL(__pyx_kwds) : 0;
    if (unlikely(__pyx_kwds_len < 0)) __PYX_ERR(0, 64, __pyx_L3_error)
    if (__pyx_kwds_len > 0) {
      switch (__pyx_nargs) {
        case  4:
        values[3] = __Pyx_ArgRef_FASTCALL(__pyx_args, 3);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[3])) __PYX_ERR(0, 64, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  3:
        values[2] = __Pyx_ArgRef_FASTCALL(__pyx_args, 2);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[2])) __PYX_ERR(0, 64, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  2:
        values[1] = __Pyx_ArgRef_FASTCALL(__pyx_args, 1);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[1])) __PYX_ERR(0, 64, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  1:
        values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(0, 64, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      const Py_ssize_t kwd_pos_args = __pyx_nargs;
      if (__Pyx_ParseKeywords(__pyx_kwds, __pyx_kwvalues, __pyx_pyargnames, 0, values, kwd_pos_args, __pyx_kwds_len, "__init__", 0) < (0)) __PYX_ERR(0, 64, __pyx_L3_error)
      if (!values[3]) values[3] = __Pyx_NewRef(((PyObject *)((PyObject*)__pyx_mstate_global->__pyx_int_0)));
      for (Py_ssize_t i = __pyx_nargs; i < 3; i++) {
        if (unlikely(!values[i])) { __Pyx_RaiseArgtupleInvalid("__init__", 0, 3, 4, i); __PYX_ERR(0, 64, __pyx_L3_error) }
      }
    } else {
      switch (__pyx_nargs) {
        case  4:
        values[3] = __Pyx_ArgRef_FASTCALL(__pyx_args, 3);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[3])) __PYX_ERR(0, 64, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  3:
        values[2] = __Pyx_ArgRef_FASTCALL(__pyx_args, 2);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[2])) __PYX_ERR(0, 64, __pyx_L3_error)
        values[1] = __Pyx_ArgRef_FASTCALL(__pyx_args, 1);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[1])) __PYX_ERR(0, 64, __pyx_L3_error)
        values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(0, 64, __pyx_L3_error)
        break;
        default: goto __pyx_L5_argtuple_error;
      }
      if (!values[3]) values[3] = __Pyx_NewRef(((PyObject *)((PyObject*)__pyx_mstate_global->__pyx_int_0)));
    }
    __pyx_v_self = values[0];
    __pyx_v_size = values[1];
    __pyx_v_total = values[2];
    __pyx_v_limit = values[3];
  }
  goto __pyx_L6_skip;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("__init__", 0, 3, 4, __pyx_nargs); __PYX_ERR(0, 64, __pyx_L3_error)
  __pyx_L6_skip:;
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_AddTraceback("cupy.cuda.memory.OutOfMemoryError.__init__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_4cupy_4cuda_6memory_16OutOfMemoryError___init__(__pyx_self, __pyx_v_self, __pyx_v_size, __pyx_v_total, __pyx_v_limit);

  /* function exit code */
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4cupy_4cuda_6memory_16OutOfMemoryError___init__(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_self, PyObject *__pyx_v_size, PyObject *__pyx_v_total, PyObject *__pyx_v_limit) {
  PyObject *__pyx_v_msg = NULL;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  size_t __pyx_t_4;
  PyObject *__pyx_t_5 = NULL;
  PyObject *__pyx_t_6 = NULL;
  PyObject *__pyx_t_7 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__init__", 0);

  /* "cupy/cuda/memory.pyx":65
 * 
 *     def __init__(self, size, total, limit=0):
 *         self._size = size             # <<<<<<<<<<<<<<
 *         self._total = total
 *         self._limit = limit
*/
  if (__Pyx_PyObject_SetAttrStr(__pyx_v_self, __pyx_mstate_global->__pyx_n_u_size_2, __pyx_v_size) < (0)) __PYX_ERR(0, 65, __pyx_L1_error)

  /* "cupy/cuda/memory.pyx":66
 *     def __init__(self, size, total, limit=0):
 *         self._size = size
 *         self._total = total             # <<<<<<<<<<<<<<
 *         self._limit = limit
 * 
*/
  if (__Pyx_PyObject_SetAttrStr(__pyx_v_self, __pyx_mstate_global->__pyx_n_u_total_2, __pyx_v_total) < (0)) __PYX_ERR(0, 66, __pyx_L1_error)

  /* "cupy/cuda/memory.pyx":67
 *         self._size = size
 *         self._total = total
 *         self._limit = limit             # <<<<<<<<<<<<<<
 * 
 *         if limit == 0:
*/
  if (__Pyx_PyObject_SetAttrStr(__pyx_v_self, __pyx_mstate_global->__pyx_n_u_limit_2, __pyx_v_limit) < (0)) __PYX_ERR(0, 67, __pyx_L1_error)

  /* "cupy/cuda/memory.pyx":69
 *         self._limit = limit
 * 
 *         if limit == 0:             # <<<<<<<<<<<<<<
 *             msg = 'Out of memory allocating {:,} bytes'.format(size)
 *             if total != -1:
*/
  __pyx_t_1 = (__Pyx_PyLong_BoolEqObjC(__pyx_v_limit, __pyx_mstate_global->__pyx_int_0, 0, 0)); if (unlikely((__pyx_t_1 < 0))) __PYX_ERR(0, 69, __pyx_L1_error)
  if (__pyx_t_1) {

    /* "cupy/cuda/memory.pyx":70
 * 
 *         if limit == 0:
 *             msg = 'Out of memory allocating {:,} bytes'.format(size)             # <<<<<<<<<<<<<<
 *             if total != -1:
 *                 msg += ' (allocated so far: {:,} bytes).'.format(total)
*/
    __pyx_t_3 = __pyx_mstate_global->__pyx_kp_u_Out_of_memory_allocating_bytes;
    __Pyx_INCREF(__pyx_t_3);
    __pyx_t_4 = 0;
    {
      PyObject *__pyx_callargs[2] = {__pyx_t_3, __pyx_v_size};
      __pyx_t_2 = __Pyx_PyObject_FastCallMethod((PyObject*)__pyx_mstate_global->__pyx_n_u_format, __pyx_callargs+__pyx_t_4, (2-__pyx_t_4) | (1*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
      __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
      if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 70, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_2);
    }
    __pyx_v_msg = ((PyObject*)__pyx_t_2);
    __pyx_t_2 = 0;

    /* "cupy/cuda/memory.pyx":71
 *         if limit == 0:
 *             msg = 'Out of memory allocating {:,} bytes'.format(size)
 *             if total != -1:             # <<<<<<<<<<<<<<
 *                 msg += ' (allocated so far: {:,} bytes).'.format(total)
 *         else:
*/
    __pyx_t_1 = (__Pyx_PyLong_BoolNeObjC(__pyx_v_total, __pyx_mstate_global->__pyx_int_neg_1, -1L, 0)); if (unlikely((__pyx_t_1 < 0))) __PYX_ERR(0, 71, __pyx_L1_error)
    if (__pyx_t_1) {

      /* "cupy/cuda/memory.pyx":72
 *             msg = 'Out of memory allocating {:,} bytes'.format(size)
 *             if total != -1:
 *                 msg += ' (allocated so far: {:,} bytes).'.format(total)             # <<<<<<<<<<<<<<
 *         else:
 *             msg = (
*/
      __pyx_t_3 = __pyx_mstate_global->__pyx_kp_u_allocated_so_far_bytes;
      __Pyx_INCREF(__pyx_t_3);
      __pyx_t_4 = 0;
      {
        PyObject *__pyx_callargs[2] = {__pyx_t_3, __pyx_v_total};
        __pyx_t_2 = __Pyx_PyObject_FastCallMethod((PyObject*)__pyx_mstate_global->__pyx_n_u_format, __pyx_callargs+__pyx_t_4, (2-__pyx_t_4) | (1*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
        __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
        if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 72, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_2);
      }
      __pyx_t_3 = __Pyx_PyUnicode_Concat__Pyx_ReferenceSharing_OwnStrongReferenceInPlaceSafe(__pyx_v_msg, __pyx_t_2); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 72, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_3);
      __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
      __Pyx_DECREF_SET(__pyx_v_msg, ((PyObject*)__pyx_t_3));
      __pyx_t_3 = 0;

      /* "cupy/cuda/memory.pyx":71
 *         if limit == 0:
 *             msg = 'Out of memory allocating {:,} bytes'.format(size)
 *             if total != -1:             # <<<<<<<<<<<<<<
 *                 msg += ' (allocated so far: {:,} bytes).'.format(total)
 *         else:
*/
    }

    /* "cupy/cuda/memory.pyx":69
 *         self._limit = limit
 * 
 *         if limit == 0:             # <<<<<<<<<<<<<<
 *             msg = 'Out of memory allocating {:,} bytes'.format(size)
 *             if total != -1:
*/
    goto __pyx_L3;
  }

  /* "cupy/cuda/memory.pyx":77
 *                 'Out of memory allocating {:,} bytes '
 *                 '(allocated so far: {:,} bytes, '
 *                 'limit set to: {:,} bytes).'.format(size, total, limit))             # <<<<<<<<<<<<<<
 *         super(OutOfMemoryError, self).__init__(msg)
 * 
*/
  /*else*/ {
    __pyx_t_2 = __pyx_mstate_global->__pyx_kp_u_Out_of_memory_allocating_bytes_a;
    __Pyx_INCREF(__pyx_t_2);
    __pyx_t_4 = 0;
    {
      PyObject *__pyx_callargs[4] = {__pyx_t_2, __pyx_v_size, __pyx_v_total, __pyx_v_limit};
      __pyx_t_3 = __Pyx_PyObject_FastCallMethod((PyObject*)__pyx_mstate_global->__pyx_n_u_format, __pyx_callargs+__pyx_t_4, (4-__pyx_t_4) | (1*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
      __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
      if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 77, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_3);
    }
    __pyx_v_msg = ((PyObject*)__pyx_t_3);
    __pyx_t_3 = 0;
  }
  __pyx_L3:;

  /* "cupy/cuda/memory.pyx":78
 *                 '(allocated so far: {:,} bytes, '
 *                 'limit set to: {:,} bytes).'.format(size, total, limit))
 *         super(OutOfMemoryError, self).__init__(msg)             # <<<<<<<<<<<<<<
 * 
 *     def __reduce__(self):
*/
  __pyx_t_6 = NULL;
  __Pyx_GetModuleGlobalName(__pyx_t_7, __pyx_mstate_global->__pyx_n_u_OutOfMemoryError); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 78, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_7);
  __pyx_t_4 = 1;
  {
    PyObject *__pyx_callargs[3] = {__pyx_t_6, __pyx_t_7, __pyx_v_self};
    __pyx_t_5 = __Pyx_PyObject_FastCall((PyObject*)__pyx_builtin_super, __pyx_callargs+__pyx_t_4, (3-__pyx_t_4) | (__pyx_t_4*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
    __Pyx_XDECREF(__pyx_t_6); __pyx_t_6 = 0;
    __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
    if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 78, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
  }
  __pyx_t_2 = __pyx_t_5;
  __Pyx_INCREF(__pyx_t_2);
  __pyx_t_4 = 0;
  {
    PyObject *__pyx_callargs[2] = {__pyx_t_2, __pyx_v_msg};
    __pyx_t_3 = __Pyx_PyObject_FastCallMethod((PyObject*)__pyx_mstate_global->__pyx_n_u_init, __pyx_callargs+__pyx_t_4, (2-__pyx_t_4) | (1*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
    __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
    if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 78, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
  }
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "cupy/cuda/memory.pyx":64
 *     """
 * 
 *     def __init__(self, size, total, limit=0):             # <<<<<<<<<<<<<<
 *         self._size = size
 *         self._total = total
*/

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_XDECREF(__pyx_t_7);
  __Pyx_AddTraceback("cupy.cuda.memory.OutOfMemoryError.__init__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_msg);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "cupy/cuda/memory.pyx":80
 *         super(OutOfMemoryError, self).__init__(msg)
 * 
 *     def __reduce__(self):             # <<<<<<<<<<<<<<
 *         return (type(self), (self._size, self._total, self._limit))
 * 
*/

/* Python wrapper */
static PyObject *__pyx_pw_4cupy_4cuda_6memory_16OutOfMemoryError_3__reduce__(PyObject *__pyx_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
PyDoc_STRVAR(__pyx_doc_4cupy_4cuda_6memory_16OutOfMemoryError_2__reduce__, "OutOfMemoryError.__reduce__(self)");
static PyMethodDef __pyx_mdef_4cupy_4cuda_6memory_16OutOfMemoryError_3__reduce__ = {"__reduce__", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_4cupy_4cuda_6memory_16OutOfMemoryError_3__reduce__, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_4cupy_4cuda_6memory_16OutOfMemoryError_2__reduce__};
static PyObject *__pyx_pw_4cupy_4cuda_6memory_16OutOfMemoryError_3__reduce__(PyObject *__pyx_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
) {
  PyObject *__pyx_v_self = 0;
  #if !CYTHON_METH_FASTCALL
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  #endif
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject* values[1] = {0};
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__reduce__ (wrapper)", 0);
  #if !CYTHON_METH_FASTCALL
  #if CYTHON_ASSUME_SAFE_SIZE
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return NULL;
  #endif
  #endif
  __pyx_kwvalues = __Pyx_KwValues_FASTCALL(__pyx_args, __pyx_nargs);
  {
    PyObject ** const __pyx_pyargnames[] = {&__pyx_mstate_global->__pyx_n_u_self,0};
    const Py_ssize_t __pyx_kwds_len = (__pyx_kwds) ? __Pyx_NumKwargs_FASTCALL(__pyx_kwds) : 0;
    if (unlikely(__pyx_kwds_len < 0)) __PYX_ERR(0, 80, __pyx_L3_error)
    if (__pyx_kwds_len > 0) {
      switch (__pyx_nargs) {
        case  1:
        values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(0, 80, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      const Py_ssize_t kwd_pos_args = __pyx_nargs;
      if (__Pyx_ParseKeywords(__pyx_kwds, __pyx_kwvalues, __pyx_pyargnames, 0, values, kwd_pos_args, __pyx_kwds_len, "__reduce__", 0) < (0)) __PYX_ERR(0, 80, __pyx_L3_error)
      for (Py_ssize_t i = __pyx_nargs; i < 1; i++) {
        if (unlikely(!values[i])) { __Pyx_RaiseArgtupleInvalid("__reduce__", 1, 1, 1, i); __PYX_ERR(0, 80, __pyx_L3_error) }
      }
    } else if (unlikely(__pyx_nargs != 1)) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
      if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(0, 80, __pyx_L3_error)
    }
    __pyx_v_self = values[0];
  }
  goto __pyx_L6_skip;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("__reduce__", 1, 1, 1, __pyx_nargs); __PYX_ERR(0, 80, __pyx_L3_error)
  __pyx_L6_skip:;
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_AddTraceback("cupy.cuda.memory.OutOfMemoryError.__reduce__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_4cupy_4cuda_6memory_16OutOfMemoryError_2__reduce__(__pyx_self, __pyx_v_self);

  /* function exit code */
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4cupy_4cuda_6memory_16OutOfMemoryError_2__reduce__(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__reduce__", 0);

  /* "cupy/cuda/memory.pyx":81
 * 
 *     def __reduce__(self):
 *         return (type(self), (self._size, self._total, self._limit))             # <<<<<<<<<<<<<<
 * 
 * 
*/
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_v_self, __pyx_mstate_global->__pyx_n_u_size_2); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 81, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_v_self, __pyx_mstate_global->__pyx_n_u_total_2); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 81, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_v_self, __pyx_mstate_global->__pyx_n_u_limit_2); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 81, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_4 = PyTuple_New(3); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 81, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_GIVEREF(__pyx_t_1);
  if (__Pyx_PyTuple_SET_ITEM(__pyx_t_4, 0, __pyx_t_1) != (0)) __PYX_ERR(0, 81, __pyx_L1_error);
  __Pyx_GIVEREF(__pyx_t_2);
  if (__Pyx_PyTuple_SET_ITEM(__pyx_t_4, 1, __pyx_t_2) != (0)) __PYX_ERR(0, 81, __pyx_L1_error);
  __Pyx_GIVEREF(__pyx_t_3);
  if (__Pyx_PyTuple_SET_ITEM(__pyx_t_4, 2, __pyx_t_3) != (0)) __PYX_ERR(0, 81, __pyx_L1_error);
  __pyx_t_1 = 0;
  __pyx_t_2 = 0;
  __pyx_t_3 = 0;
  __pyx_t_3 = PyTuple_New(2); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 81, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_INCREF(((PyObject *)Py_TYPE(__pyx_v_self)));
  __Pyx_GIVEREF(((PyObject *)Py_TYPE(__pyx_v_self)));
  if (__Pyx_PyTuple_SET_ITEM(__pyx_t_3, 0, ((PyObject *)Py_TYPE(__pyx_v_self))) != (0)) __PYX_ERR(0, 81, __pyx_L1_error);
  __Pyx_GIVEREF(__pyx_t_4);
  if (__Pyx_PyTuple_SET_ITEM(__pyx_t_3, 1, __pyx_t_4) != (0)) __PYX_ERR(0, 81, __pyx_L1_error);
  __pyx_t_4 = 0;
  __pyx_r = __pyx_t_3;
  __pyx_t_3 = 0;
  goto __pyx_L0;

  /* "cupy/cuda/memory.pyx":80
 *         super(OutOfMemoryError, self).__init__(msg)
 * 
 *     def __reduce__(self):             # <<<<<<<<<<<<<<
 *         return (type(self), (self._size, self._total, self._limit))
 * 
*/

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_AddTraceback("cupy.cuda.memory.OutOfMemoryError.__reduce__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "cupy/cuda/memory.pyx":95
 *     """
 * 
 *     def __int__(self):             # <<<<<<<<<<<<<<
 *         """Returns the pointer value to the head of the allocation."""
 *         return self.ptr
*/

/* Python wrapper */
static PyObject *__pyx_pw_4cupy_4cuda_6memory_10BaseMemory_1__int__(PyObject *__pyx_v_self); /*proto*/
PyDoc_STRVAR(__pyx_doc_4cupy_4cuda_6memory_10BaseMemory___int__, "BaseMemory.__int__(self)\n\nReturns the pointer value to the head of the allocation.");
#if CYTHON_UPDATE_DESCRIPTOR_DOC
struct wrapperbase __pyx_wrapperbase_4cupy_4cuda_6memory_10BaseMemory___int__;
#endif
static PyObject *__pyx_pw_4cupy_4cuda_6memory_10BaseMemory_1__int__(PyObject *__pyx_v_self) {
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__int__ (wrapper)", 0);
  __pyx_kwvalues = __Pyx_KwValues_VARARGS(__pyx_args, __pyx_nargs);
  __pyx_r = __pyx_pf_4cupy_4cuda_6memory_10BaseMemory___int__(((struct __pyx_obj_4cupy_4cuda_6memory_BaseMemory *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4cupy_4cuda_6memory_10BaseMemory___int__(struct __pyx_obj_4cupy_4cuda_6memory_BaseMemory *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__int__", 0);

  /* "cupy/cuda/memory.pyx":97
 *     def __int__(self):
 *         """Returns the pointer value to the head of the allocation."""
 *         return self.ptr             # <<<<<<<<<<<<<<
 * 
 *     @property
*/
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = PyLong_FromSsize_t(__pyx_v_self->ptr); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 97, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "cupy/cuda/memory.pyx":95
 *     """
 * 
 *     def __int__(self):             # <<<<<<<<<<<<<<
 *         """Returns the pointer value to the head of the allocation."""
 *         return self.ptr
*/

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("cupy.cuda.memory.BaseMemory.__int__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "cupy/cuda/memory.pyx":99
 *         return self.ptr
 * 
 *     @property             # <<<<<<<<<<<<<<
 *     def device(self):
 *         return device.Device(self.device_id)
*/

/* Python wrapper */
static PyObject *__pyx_pw_4cupy_4cuda_6memory_10BaseMemory_6device_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_4cupy_4cuda_6memory_10BaseMemory_6device_1__get__(PyObject *__pyx_v_self) {
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_kwvalues = __Pyx_KwValues_VARARGS(__pyx_args, __pyx_nargs);
  __pyx_r = __pyx_pf_4cupy_4cuda_6memory_10BaseMemory_6device___get__(((struct __pyx_obj_4cupy_4cuda_6memory_BaseMemory *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4cupy_4cuda_6memory_10BaseMemory_6device___get__(struct __pyx_obj_4cupy_4cuda_6memory_BaseMemory *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  size_t __pyx_t_4;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__get__", 0);

  /* "cupy/cuda/memory.pyx":101
 *     @property
 *     def device(self):
 *         return device.Device(self.device_id)             # <<<<<<<<<<<<<<
 * 
 * 
*/
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_2 = NULL;
  __pyx_t_3 = __Pyx_PyLong_From_int(__pyx_v_self->device_id); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 101, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_4 = 1;
  {
    PyObject *__pyx_callargs[2] = {__pyx_t_2, __pyx_t_3};
    __pyx_t_1 = __Pyx_PyObject_FastCall((PyObject*)__pyx_mstate_global->__pyx_ptype_4cupy_4cuda_6device_Device, __pyx_callargs+__pyx_t_4, (2-__pyx_t_4) | (__pyx_t_4*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
    __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 101, __pyx_L1_error)
    __Pyx_GOTREF((PyObject *)__pyx_t_1);
  }
  __pyx_r = ((PyObject *)__pyx_t_1);
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "cupy/cuda/memory.pyx":99
 *         return self.ptr
 * 
 *     @property             # <<<<<<<<<<<<<<
 *     def device(self):
 *         return device.Device(self.device_id)
*/

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_AddTraceback("cupy.cuda.memory.BaseMemory.device.__get__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "cupy/cuda/memory.pxd":14
 * 
 *     cdef:
 *         public intptr_t ptr             # <<<<<<<<<<<<<<
 *         public size_t size
 *         public int device_id
*/

/* Python wrapper */
static PyObject *__pyx_pw_4cupy_4cuda_6memory_10BaseMemory_3ptr_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_4cupy_4cuda_6memory_10BaseMemory_3ptr_1__get__(PyObject *__pyx_v_self) {
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_kwvalues = __Pyx_KwValues_VARARGS(__pyx_args, __pyx_nargs);
  __pyx_r = __pyx_pf_4cupy_4cuda_6memory_10BaseMemory_3ptr___get__(((struct __pyx_obj_4cupy_4cuda_6memory_BaseMemory *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4cupy_4cuda_6memory_10BaseMemory_3ptr___get__(struct __pyx_obj_4cupy_4cuda_6memory_BaseMemory *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__get__", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = PyLong_FromSsize_t(__pyx_v_self->ptr); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 14, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("cupy.cuda.memory.BaseMemory.ptr.__get__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static int __pyx_pw_4cupy_4cuda_6memory_10BaseMemory_3ptr_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value); /*proto*/
static int __pyx_pw_4cupy_4cuda_6memory_10BaseMemory_3ptr_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value) {
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__ (wrapper)", 0);
  __pyx_kwvalues = __Pyx_KwValues_VARARGS(__pyx_args, __pyx_nargs);
  __pyx_r = __pyx_pf_4cupy_4cuda_6memory_10BaseMemory_3ptr_2__set__(((struct __pyx_obj_4cupy_4cuda_6memory_BaseMemory *)__pyx_v_self), ((PyObject *)__pyx_v_value));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_4cupy_4cuda_6memory_10BaseMemory_3ptr_2__set__(struct __pyx_obj_4cupy_4cuda_6memory_BaseMemory *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  intptr_t __pyx_t_1;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __pyx_t_1 = PyLong_AsSsize_t(__pyx_v_value); if (unlikely((__pyx_t_1 == ((intptr_t)-1)) && PyErr_Occurred())) __PYX_ERR(1, 14, __pyx_L1_error)
  __pyx_v_self->ptr = __pyx_t_1;

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_AddTraceback("cupy.cuda.memory.BaseMemory.ptr.__set__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  return __pyx_r;
}

/* "cupy/cuda/memory.pxd":15
 *     cdef:
 *         public intptr_t ptr
 *         public size_t size             # <<<<<<<<<<<<<<
 *         public int device_id
 * 
*/

/* Python wrapper */
static PyObject *__pyx_pw_4cupy_4cuda_6memory_10BaseMemory_4size_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_4cupy_4cuda_6memory_10BaseMemory_4size_1__get__(PyObject *__pyx_v_self) {
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_kwvalues = __Pyx_KwValues_VARARGS(__pyx_args, __pyx_nargs);
  __pyx_r = __pyx_pf_4cupy_4cuda_6memory_10BaseMemory_4size___get__(((struct __pyx_obj_4cupy_4cuda_6memory_BaseMemory *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4cupy_4cuda_6memory_10BaseMemory_4size___get__(struct __pyx_obj_4cupy_4cuda_6memory_BaseMemory *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__get__", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __Pyx_PyLong_FromSize_t(__pyx_v_self->size); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 15, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("cupy.cuda.memory.BaseMemory.size.__get__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static int __pyx_pw_4cupy_4cuda_6memory_10BaseMemory_4size_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value); /*proto*/
static int __pyx_pw_4cupy_4cuda_6memory_10BaseMemory_4size_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value) {
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__ (wrapper)", 0);
  __pyx_kwvalues = __Pyx_KwValues_VARARGS(__pyx_args, __pyx_nargs);
  __pyx_r = __pyx_pf_4cupy_4cuda_6memory_10BaseMemory_4size_2__set__(((struct __pyx_obj_4cupy_4cuda_6memory_BaseMemory *)__pyx_v_self), ((PyObject *)__pyx_v_value));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_4cupy_4cuda_6memory_10BaseMemory_4size_2__set__(struct __pyx_obj_4cupy_4cuda_6memory_BaseMemory *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  size_t __pyx_t_1;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __pyx_t_1 = __Pyx_PyLong_As_size_t(__pyx_v_value); if (unlikely((__pyx_t_1 == (size_t)-1) && PyErr_Occurred())) __PYX_ERR(1, 15, __pyx_L1_error)
  __pyx_v_self->size = __pyx_t_1;

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_AddTraceback("cupy.cuda.memory.BaseMemory.size.__set__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  return __pyx_r;
}

/* "cupy/cuda/memory.pxd":16
 *         public intptr_t ptr
 *         public size_t size
 *         public int device_id             # <<<<<<<<<<<<<<
 * 
 * 
*/

/* Python wrapper */
static PyObject *__pyx_pw_4cupy_4cuda_6memory_10BaseMemory_9device_id_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_4cupy_4cuda_6memory_10BaseMemory_9device_id_1__get__(PyObject *__pyx_v_self) {
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_kwvalues = __Pyx_KwValues_VARARGS(__pyx_args, __pyx_nargs);
  __pyx_r = __pyx_pf_4cupy_4cuda_6memory_10BaseMemory_9device_id___get__(((struct __pyx_obj_4cupy_4cuda_6memory_BaseMemory *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4cupy_4cuda_6memory_10BaseMemory_9device_id___get__(struct __pyx_obj_4cupy_4cuda_6memory_BaseMemory *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__get__", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __Pyx_PyLong_From_int(__pyx_v_self->device_id); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 16, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("cupy.cuda.memory.BaseMemory.device_id.__get__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static int __pyx_pw_4cupy_4cuda_6memory_10BaseMemory_9device_id_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value); /*proto*/
static int __pyx_pw_4cupy_4cuda_6memory_10BaseMemory_9device_id_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value) {
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__ (wrapper)", 0);
  __pyx_kwvalues = __Pyx_KwValues_VARARGS(__pyx_args, __pyx_nargs);
  __pyx_r = __pyx_pf_4cupy_4cuda_6memory_10BaseMemory_9device_id_2__set__(((struct __pyx_obj_4cupy_4cuda_6memory_BaseMemory *)__pyx_v_self), ((PyObject *)__pyx_v_value));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_4cupy_4cuda_6memory_10BaseMemory_9device_id_2__set__(struct __pyx_obj_4cupy_4cuda_6memory_BaseMemory *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  int __pyx_t_1;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __pyx_t_1 = __Pyx_PyLong_As_int(__pyx_v_value); if (unlikely((__pyx_t_1 == (int)-1) && PyErr_Occurred())) __PYX_ERR(1, 16, __pyx_L1_error)
  __pyx_v_self->device_id = __pyx_t_1;

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_AddTraceback("cupy.cuda.memory.BaseMemory.device_id.__set__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  return __pyx_r;
}

/* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     cdef tuple state
 *     cdef object _dict
*/

/* Python wrapper */
static PyObject *__pyx_pw_4cupy_4cuda_6memory_10BaseMemory_3__reduce_cython__(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
PyDoc_STRVAR(__pyx_doc_4cupy_4cuda_6memory_10BaseMemory_2__reduce_cython__, "BaseMemory.__reduce_cython__(self)");
static PyObject *__pyx_pw_4cupy_4cuda_6memory_10BaseMemory_3__reduce_cython__(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
) {
  #if !CYTHON_METH_FASTCALL
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  #endif
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__reduce_cython__ (wrapper)", 0);
  #if !CYTHON_METH_FASTCALL
  #if CYTHON_ASSUME_SAFE_SIZE
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return NULL;
  #endif
  #endif
  __pyx_kwvalues = __Pyx_KwValues_FASTCALL(__pyx_args, __pyx_nargs);
  if (unlikely(__pyx_nargs > 0)) { __Pyx_RaiseArgtupleInvalid("__reduce_cython__", 1, 0, 0, __pyx_nargs); return NULL; }
  const Py_ssize_t __pyx_kwds_len = unlikely(__pyx_kwds) ? __Pyx_NumKwargs_FASTCALL(__pyx_kwds) : 0;
  if (unlikely(__pyx_kwds_len < 0)) return NULL;
  if (unlikely(__pyx_kwds_len > 0)) {__Pyx_RejectKeywords("__reduce_cython__", __pyx_kwds); return NULL;}
  __pyx_r = __pyx_pf_4cupy_4cuda_6memory_10BaseMemory_2__reduce_cython__(((struct __pyx_obj_4cupy_4cuda_6memory_BaseMemory *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4cupy_4cuda_6memory_10BaseMemory_2__reduce_cython__(struct __pyx_obj_4cupy_4cuda_6memory_BaseMemory *__pyx_v_self) {
  PyObject *__pyx_v_state = 0;
  PyObject *__pyx_v__dict = 0;
  int __pyx_v_use_setstate;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  int __pyx_t_5;
  int __pyx_t_6;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__reduce_cython__", 0);

  /* "(tree fragment)":5
 *     cdef object _dict
 *     cdef bint use_setstate
 *     state = (self.device_id, self.ptr, self.size)             # <<<<<<<<<<<<<<
 *     _dict = getattr(self, '__dict__', None)
 *     if _dict is not None and _dict:
*/
  __pyx_t_1 = __Pyx_PyLong_From_int(__pyx_v_self->device_id); if (unlikely(!__pyx_t_1)) __PYX_ERR(3, 5, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = PyLong_FromSsize_t(__pyx_v_self->ptr); if (unlikely(!__pyx_t_2)) __PYX_ERR(3, 5, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = __Pyx_PyLong_FromSize_t(__pyx_v_self->size); if (unlikely(!__pyx_t_3)) __PYX_ERR(3, 5, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_4 = PyTuple_New(3); if (unlikely(!__pyx_t_4)) __PYX_ERR(3, 5, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_GIVEREF(__pyx_t_1);
  if (__Pyx_PyTuple_SET_ITEM(__pyx_t_4, 0, __pyx_t_1) != (0)) __PYX_ERR(3, 5, __pyx_L1_error);
  __Pyx_GIVEREF(__pyx_t_2);
  if (__Pyx_PyTuple_SET_ITEM(__pyx_t_4, 1, __pyx_t_2) != (0)) __PYX_ERR(3, 5, __pyx_L1_error);
  __Pyx_GIVEREF(__pyx_t_3);
  if (__Pyx_PyTuple_SET_ITEM(__pyx_t_4, 2, __pyx_t_3) != (0)) __PYX_ERR(3, 5, __pyx_L1_error);
  __pyx_t_1 = 0;
  __pyx_t_2 = 0;
  __pyx_t_3 = 0;
  __pyx_v_state = ((PyObject*)__pyx_t_4);
  __pyx_t_4 = 0;

  /* "(tree fragment)":6
 *     cdef bint use_setstate
 *     state = (self.device_id, self.ptr, self.size)
 *     _dict = getattr(self, '__dict__', None)             # <<<<<<<<<<<<<<
 *     if _dict is not None and _dict:
 *         state += (_dict,)
*/
  __pyx_t_4 = __Pyx_GetAttr3(((PyObject *)__pyx_v_self), __pyx_mstate_global->__pyx_n_u_dict, Py_None); if (unlikely(!__pyx_t_4)) __PYX_ERR(3, 6, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_v__dict = __pyx_t_4;
  __pyx_t_4 = 0;

  /* "(tree fragment)":7
 *     state = (self.device_id, self.ptr, self.size)
 *     _dict = getattr(self, '__dict__', None)
 *     if _dict is not None and _dict:             # <<<<<<<<<<<<<<
 *         state += (_dict,)
 *         use_setstate = True
*/
  __pyx_t_6 = (__pyx_v__dict != Py_None);
  if (__pyx_t_6) {
  } else {
    __pyx_t_5 = __pyx_t_6;
    goto __pyx_L4_bool_binop_done;
  }
  __pyx_t_6 = __Pyx_PyObject_IsTrue(__pyx_v__dict); if (unlikely((__pyx_t_6 < 0))) __PYX_ERR(3, 7, __pyx_L1_error)
  __pyx_t_5 = __pyx_t_6;
  __pyx_L4_bool_binop_done:;
  if (__pyx_t_5) {

    /* "(tree fragment)":8
 *     _dict = getattr(self, '__dict__', None)
 *     if _dict is not None and _dict:
 *         state += (_dict,)             # <<<<<<<<<<<<<<
 *         use_setstate = True
 *     else:
*/
    __pyx_t_4 = PyTuple_New(1); if (unlikely(!__pyx_t_4)) __PYX_ERR(3, 8, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_INCREF(__pyx_v__dict);
    __Pyx_GIVEREF(__pyx_v__dict);
    if (__Pyx_PyTuple_SET_ITEM(__pyx_t_4, 0, __pyx_v__dict) != (0)) __PYX_ERR(3, 8, __pyx_L1_error);
    __pyx_t_3 = PyNumber_InPlaceAdd(__pyx_v_state, __pyx_t_4); if (unlikely(!__pyx_t_3)) __PYX_ERR(3, 8, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __Pyx_DECREF_SET(__pyx_v_state, ((PyObject*)__pyx_t_3));
    __pyx_t_3 = 0;

    /* "(tree fragment)":9
 *     if _dict is not None and _dict:
 *         state += (_dict,)
 *         use_setstate = True             # <<<<<<<<<<<<<<
 *     else:
 *         use_setstate = False
*/
    __pyx_v_use_setstate = 1;

    /* "(tree fragment)":7
 *     state = (self.device_id, self.ptr, self.size)
 *     _dict = getattr(self, '__dict__', None)
 *     if _dict is not None and _dict:             # <<<<<<<<<<<<<<
 *         state += (_dict,)
 *         use_setstate = True
*/
    goto __pyx_L3;
  }

  /* "(tree fragment)":11
 *         use_setstate = True
 *     else:
 *         use_setstate = False             # <<<<<<<<<<<<<<
 *     if use_setstate:
 *         return __pyx_unpickle_BaseMemory, (type(self), 0x61cadb0, None), state
*/
  /*else*/ {
    __pyx_v_use_setstate = 0;
  }
  __pyx_L3:;

  /* "(tree fragment)":12
 *     else:
 *         use_setstate = False
 *     if use_setstate:             # <<<<<<<<<<<<<<
 *         return __pyx_unpickle_BaseMemory, (type(self), 0x61cadb0, None), state
 *     else:
*/
  if (__pyx_v_use_setstate) {

    /* "(tree fragment)":13
 *         use_setstate = False
 *     if use_setstate:
 *         return __pyx_unpickle_BaseMemory, (type(self), 0x61cadb0, None), state             # <<<<<<<<<<<<<<
 *     else:
 *         return __pyx_unpickle_BaseMemory, (type(self), 0x61cadb0, state)
*/
    __Pyx_XDECREF(__pyx_r);
    __Pyx_GetModuleGlobalName(__pyx_t_3, __pyx_mstate_global->__pyx_n_u_pyx_unpickle_BaseMemory); if (unlikely(!__pyx_t_3)) __PYX_ERR(3, 13, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_t_4 = PyTuple_New(3); if (unlikely(!__pyx_t_4)) __PYX_ERR(3, 13, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_INCREF(((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    __Pyx_GIVEREF(((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    if (__Pyx_PyTuple_SET_ITEM(__pyx_t_4, 0, ((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self)))) != (0)) __PYX_ERR(3, 13, __pyx_L1_error);
    __Pyx_INCREF(__pyx_mstate_global->__pyx_int_102542768);
    __Pyx_GIVEREF(__pyx_mstate_global->__pyx_int_102542768);
    if (__Pyx_PyTuple_SET_ITEM(__pyx_t_4, 1, __pyx_mstate_global->__pyx_int_102542768) != (0)) __PYX_ERR(3, 13, __pyx_L1_error);
    __Pyx_INCREF(Py_None);
    __Pyx_GIVEREF(Py_None);
    if (__Pyx_PyTuple_SET_ITEM(__pyx_t_4, 2, Py_None) != (0)) __PYX_ERR(3, 13, __pyx_L1_error);
    __pyx_t_2 = PyTuple_New(3); if (unlikely(!__pyx_t_2)) __PYX_ERR(3, 13, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_GIVEREF(__pyx_t_3);
    if (__Pyx_PyTuple_SET_ITEM(__pyx_t_2, 0, __pyx_t_3) != (0)) __PYX_ERR(3, 13, __pyx_L1_error);
    __Pyx_GIVEREF(__pyx_t_4);
    if (__Pyx_PyTuple_SET_ITEM(__pyx_t_2, 1, __pyx_t_4) != (0)) __PYX_ERR(3, 13, __pyx_L1_error);
    __Pyx_INCREF(__pyx_v_state);
    __Pyx_GIVEREF(__pyx_v_state);
    if (__Pyx_PyTuple_SET_ITEM(__pyx_t_2, 2, __pyx_v_state) != (0)) __PYX_ERR(3, 13, __pyx_L1_error);
    __pyx_t_3 = 0;
    __pyx_t_4 = 0;
    __pyx_r = __pyx_t_2;
    __pyx_t_2 = 0;
    goto __pyx_L0;

    /* "(tree fragment)":12
 *     else:
 *         use_setstate = False
 *     if use_setstate:             # <<<<<<<<<<<<<<
 *         return __pyx_unpickle_BaseMemory, (type(self), 0x61cadb0, None), state
 *     else:
*/
  }

  /* "(tree fragment)":15
 *         return __pyx_unpickle_BaseMemory, (type(self), 0x61cadb0, None), state
 *     else:
 *         return __pyx_unpickle_BaseMemory, (type(self), 0x61cadb0, state)             # <<<<<<<<<<<<<<
 * def __setstate_cython__(self, __pyx_state):
 *     __pyx_unpickle_BaseMemory__set_state(self, __pyx_state)
*/
  /*else*/ {
    __Pyx_XDECREF(__pyx_r);
    __Pyx_GetModuleGlobalName(__pyx_t_2, __pyx_mstate_global->__pyx_n_u_pyx_unpickle_BaseMemory); if (unlikely(!__pyx_t_2)) __PYX_ERR(3, 15, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_4 = PyTuple_New(3); if (unlikely(!__pyx_t_4)) __PYX_ERR(3, 15, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_INCREF(((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    __Pyx_GIVEREF(((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    if (__Pyx_PyTuple_SET_ITEM(__pyx_t_4, 0, ((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self)))) != (0)) __PYX_ERR(3, 15, __pyx_L1_error);
    __Pyx_INCREF(__pyx_mstate_global->__pyx_int_102542768);
    __Pyx_GIVEREF(__pyx_mstate_global->__pyx_int_102542768);
    if (__Pyx_PyTuple_SET_ITEM(__pyx_t_4, 1, __pyx_mstate_global->__pyx_int_102542768) != (0)) __PYX_ERR(3, 15, __pyx_L1_error);
    __Pyx_INCREF(__pyx_v_state);
    __Pyx_GIVEREF(__pyx_v_state);
    if (__Pyx_PyTuple_SET_ITEM(__pyx_t_4, 2, __pyx_v_state) != (0)) __PYX_ERR(3, 15, __pyx_L1_error);
    __pyx_t_3 = PyTuple_New(2); if (unlikely(!__pyx_t_3)) __PYX_ERR(3, 15, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_GIVEREF(__pyx_t_2);
    if (__Pyx_PyTuple_SET_ITEM(__pyx_t_3, 0, __pyx_t_2) != (0)) __PYX_ERR(3, 15, __pyx_L1_error);
    __Pyx_GIVEREF(__pyx_t_4);
    if (__Pyx_PyTuple_SET_ITEM(__pyx_t_3, 1, __pyx_t_4) != (0)) __PYX_ERR(3, 15, __pyx_L1_error);
    __pyx_t_2 = 0;
    __pyx_t_4 = 0;
    __pyx_r = __pyx_t_3;
    __pyx_t_3 = 0;
    goto __pyx_L0;
  }

  /* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     cdef tuple state
 *     cdef object _dict
*/

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_AddTraceback("cupy.cuda.memory.BaseMemory.__reduce_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_state);
  __Pyx_XDECREF(__pyx_v__dict);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":16
 *     else:
 *         return __pyx_unpickle_BaseMemory, (type(self), 0x61cadb0, state)
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     __pyx_unpickle_BaseMemory__set_state(self, __pyx_state)
*/

/* Python wrapper */
static PyObject *__pyx_pw_4cupy_4cuda_6memory_10BaseMemory_5__setstate_cython__(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
PyDoc_STRVAR(__pyx_doc_4cupy_4cuda_6memory_10BaseMemory_4__setstate_cython__, "BaseMemory.__setstate_cython__(self, __pyx_state)");
static PyObject *__pyx_pw_4cupy_4cuda_6memory_10BaseMemory_5__setstate_cython__(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
) {
  PyObject *__pyx_v___pyx_state = 0;
  #if !CYTHON_METH_FASTCALL
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  #endif
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject* values[1] = {0};
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__setstate_cython__ (wrapper)", 0);
  #if !CYTHON_METH_FASTCALL
  #if CYTHON_ASSUME_SAFE_SIZE
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return NULL;
  #endif
  #endif
  __pyx_kwvalues = __Pyx_KwValues_FASTCALL(__pyx_args, __pyx_nargs);
  {
    PyObject ** const __pyx_pyargnames[] = {&__pyx_mstate_global->__pyx_n_u_pyx_state,0};
    const Py_ssize_t __pyx_kwds_len = (__pyx_kwds) ? __Pyx_NumKwargs_FASTCALL(__pyx_kwds) : 0;
    if (unlikely(__pyx_kwds_len < 0)) __PYX_ERR(3, 16, __pyx_L3_error)
    if (__pyx_kwds_len > 0) {
      switch (__pyx_nargs) {
        case  1:
        values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(3, 16, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      const Py_ssize_t kwd_pos_args = __pyx_nargs;
      if (__Pyx_ParseKeywords(__pyx_kwds, __pyx_kwvalues, __pyx_pyargnames, 0, values, kwd_pos_args, __pyx_kwds_len, "__setstate_cython__", 0) < (0)) __PYX_ERR(3, 16, __pyx_L3_error)
      for (Py_ssize_t i = __pyx_nargs; i < 1; i++) {
        if (unlikely(!values[i])) { __Pyx_RaiseArgtupleInvalid("__setstate_cython__", 1, 1, 1, i); __PYX_ERR(3, 16, __pyx_L3_error) }
      }
    } else if (unlikely(__pyx_nargs != 1)) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
      if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(3, 16, __pyx_L3_error)
    }
    __pyx_v___pyx_state = values[0];
  }
  goto __pyx_L6_skip;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("__setstate_cython__", 1, 1, 1, __pyx_nargs); __PYX_ERR(3, 16, __pyx_L3_error)
  __pyx_L6_skip:;
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_AddTraceback("cupy.cuda.memory.BaseMemory.__setstate_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_4cupy_4cuda_6memory_10BaseMemory_4__setstate_cython__(((struct __pyx_obj_4cupy_4cuda_6memory_BaseMemory *)__pyx_v_self), __pyx_v___pyx_state);

  /* function exit code */
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4cupy_4cuda_6memory_10BaseMemory_4__setstate_cython__(struct __pyx_obj_4cupy_4cuda_6memory_BaseMemory *__pyx_v_self, PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__setstate_cython__", 0);

  /* "(tree fragment)":17
 *         return __pyx_unpickle_BaseMemory, (type(self), 0x61cadb0, state)
 * def __setstate_cython__(self, __pyx_state):
 *     __pyx_unpickle_BaseMemory__set_state(self, __pyx_state)             # <<<<<<<<<<<<<<
*/
  __pyx_t_1 = __pyx_v___pyx_state;
  __Pyx_INCREF(__pyx_t_1);
  if (!(likely(PyTuple_CheckExact(__pyx_t_1))||((__pyx_t_1) == Py_None) || __Pyx_RaiseUnexpectedTypeError("tuple", __pyx_t_1))) __PYX_ERR(3, 17, __pyx_L1_error)
  if (unlikely(__pyx_t_1 == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "cannot pass None into a C function argument that is declared 'not None'");
    __PYX_ERR(3, 17, __pyx_L1_error)
  }
  __pyx_t_2 = __pyx_f_4cupy_4cuda_6memory___pyx_unpickle_BaseMemory__set_state(__pyx_v_self, ((PyObject*)__pyx_t_1)); if (unlikely(!__pyx_t_2)) __PYX_ERR(3, 17, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "(tree fragment)":16
 *     else:
 *         return __pyx_unpickle_BaseMemory, (type(self), 0x61cadb0, state)
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     __pyx_unpickle_BaseMemory__set_state(self, __pyx_state)
*/

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_AddTraceback("cupy.cuda.memory.BaseMemory.__setstate_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "cupy/cuda/memory.pyx":114
 *     """
 * 
 *     def __init__(self, size_t size):             # <<<<<<<<<<<<<<
 *         self.size = size
 *         self.device_id = device.get_device_id()
*/

/* Python wrapper */
static int __pyx_pw_4cupy_4cuda_6memory_6Memory_1__init__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static int __pyx_pw_4cupy_4cuda_6memory_6Memory_1__init__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  size_t __pyx_v_size;
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject* values[1] = {0};
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__init__ (wrapper)", 0);
  #if CYTHON_ASSUME_SAFE_SIZE
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return -1;
  #endif
  __pyx_kwvalues = __Pyx_KwValues_VARARGS(__pyx_args, __pyx_nargs);
  {
    PyObject ** const __pyx_pyargnames[] = {&__pyx_mstate_global->__pyx_n_u_size,0};
    const Py_ssize_t __pyx_kwds_len = (__pyx_kwds) ? __Pyx_NumKwargs_VARARGS(__pyx_kwds) : 0;
    if (unlikely(__pyx_kwds_len < 0)) __PYX_ERR(0, 114, __pyx_L3_error)
    if (__pyx_kwds_len > 0) {
      switch (__pyx_nargs) {
        case  1:
        values[0] = __Pyx_ArgRef_VARARGS(__pyx_args, 0);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(0, 114, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      const Py_ssize_t kwd_pos_args = __pyx_nargs;
      if (__Pyx_ParseKeywords(__pyx_kwds, __pyx_kwvalues, __pyx_pyargnames, 0, values, kwd_pos_args, __pyx_kwds_len, "__init__", 0) < (0)) __PYX_ERR(0, 114, __pyx_L3_error)
      for (Py_ssize_t i = __pyx_nargs; i < 1; i++) {
        if (unlikely(!values[i])) { __Pyx_RaiseArgtupleInvalid("__init__", 1, 1, 1, i); __PYX_ERR(0, 114, __pyx_L3_error) }
      }
    } else if (unlikely(__pyx_nargs != 1)) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = __Pyx_ArgRef_VARARGS(__pyx_args, 0);
      if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(0, 114, __pyx_L3_error)
    }
    __pyx_v_size = __Pyx_PyLong_As_size_t(values[0]); if (unlikely((__pyx_v_size == (size_t)-1) && PyErr_Occurred())) __PYX_ERR(0, 114, __pyx_L3_error)
  }
  goto __pyx_L6_skip;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("__init__", 1, 1, 1, __pyx_nargs); __PYX_ERR(0, 114, __pyx_L3_error)
  __pyx_L6_skip:;
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_AddTraceback("cupy.cuda.memory.Memory.__init__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return -1;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_4cupy_4cuda_6memory_6Memory___init__(((struct __pyx_obj_4cupy_4cuda_6memory_Memory *)__pyx_v_self), __pyx_v_size);

  /* function exit code */
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_4cupy_4cuda_6memory_6Memory___init__(struct __pyx_obj_4cupy_4cuda_6memory_Memory *__pyx_v_self, size_t __pyx_v_size) {
  int __pyx_r;
  int __pyx_t_1;
  int __pyx_t_2;
  intptr_t __pyx_t_3;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;

  /* "cupy/cuda/memory.pyx":115
 * 
 *     def __init__(self, size_t size):
 *         self.size = size             # <<<<<<<<<<<<<<
 *         self.device_id = device.get_device_id()
 *         self.ptr = 0
*/
  __pyx_v_self->__pyx_base.size = __pyx_v_size;

  /* "cupy/cuda/memory.pyx":116
 *     def __init__(self, size_t size):
 *         self.size = size
 *         self.device_id = device.get_device_id()             # <<<<<<<<<<<<<<
 *         self.ptr = 0
 *         if size > 0:
*/
  __pyx_t_1 = __pyx_f_4cupy_4cuda_6device_get_device_id(0); if (unlikely(__pyx_t_1 == ((int)-1) && PyErr_Occurred())) __PYX_ERR(0, 116, __pyx_L1_error)
  __pyx_v_self->__pyx_base.device_id = __pyx_t_1;

  /* "cupy/cuda/memory.pyx":117
 *         self.size = size
 *         self.device_id = device.get_device_id()
 *         self.ptr = 0             # <<<<<<<<<<<<<<
 *         if size > 0:
 *             self.ptr = runtime.malloc(size)
*/
  __pyx_v_self->__pyx_base.ptr = 0;

  /* "cupy/cuda/memory.pyx":118
 *         self.device_id = device.get_device_id()
 *         self.ptr = 0
 *         if size > 0:             # <<<<<<<<<<<<<<
 *             self.ptr = runtime.malloc(size)
 * 
*/
  __pyx_t_2 = (__pyx_v_size > 0);
  if (__pyx_t_2) {

    /* "cupy/cuda/memory.pyx":119
 *         self.ptr = 0
 *         if size > 0:
 *             self.ptr = runtime.malloc(size)             # <<<<<<<<<<<<<<
 * 
 *     def __dealloc__(self):
*/
    __pyx_t_3 = __pyx_f_13cupy_backends_4cuda_3api_7runtime_malloc(__pyx_v_size, 0); if (unlikely(__pyx_t_3 == ((intptr_t)0) && PyErr_Occurred())) __PYX_ERR(0, 119, __pyx_L1_error)
    __pyx_v_self->__pyx_base.ptr = __pyx_t_3;

    /* "cupy/cuda/memory.pyx":118
 *         self.device_id = device.get_device_id()
 *         self.ptr = 0
 *         if size > 0:             # <<<<<<<<<<<<<<
 *             self.ptr = runtime.malloc(size)
 * 
*/
  }

  /* "cupy/cuda/memory.pyx":114
 *     """
 * 
 *     def __init__(self, size_t size):             # <<<<<<<<<<<<<<
 *         self.size = size
 *         self.device_id = device.get_device_id()
*/

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_AddTraceback("cupy.cuda.memory.Memory.__init__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  return __pyx_r;
}

/* "cupy/cuda/memory.pyx":121
 *             self.ptr = runtime.malloc(size)
 * 
 *     def __dealloc__(self):             # <<<<<<<<<<<<<<
 *         # Note: Cannot raise in the destructor! (cython/cython#1613)
 *         if self.ptr:
*/

/* Python wrapper */
static void __pyx_pw_4cupy_4cuda_6memory_6Memory_3__dealloc__(PyObject *__pyx_v_self); /*proto*/
static void __pyx_pw_4cupy_4cuda_6memory_6Memory_3__dealloc__(PyObject *__pyx_v_self) {
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__dealloc__ (wrapper)", 0);
  __pyx_kwvalues = __Pyx_KwValues_VARARGS(__pyx_args, __pyx_nargs);
  __pyx_pf_4cupy_4cuda_6memory_6Memory_2__dealloc__(((struct __pyx_obj_4cupy_4cuda_6memory_Memory *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
}

static void __pyx_pf_4cupy_4cuda_6memory_6Memory_2__dealloc__(struct __pyx_obj_4cupy_4cuda_6memory_Memory *__pyx_v_self) {
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__dealloc__", 0);

  /* "cupy/cuda/memory.pyx":123
 *     def __dealloc__(self):
 *         # Note: Cannot raise in the destructor! (cython/cython#1613)
 *         if self.ptr:             # <<<<<<<<<<<<<<
 *             runtime.free(self.ptr)
 * 
*/
  __pyx_t_1 = (__pyx_v_self->__pyx_base.ptr != 0);
  if (__pyx_t_1) {

    /* "cupy/cuda/memory.pyx":124
 *         # Note: Cannot raise in the destructor! (cython/cython#1613)
 *         if self.ptr:
 *             runtime.free(self.ptr)             # <<<<<<<<<<<<<<
 * 
 * 
*/
    __pyx_t_2 = __pyx_f_13cupy_backends_4cuda_3api_7runtime_free(__pyx_v_self->__pyx_base.ptr, 0); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 124, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

    /* "cupy/cuda/memory.pyx":123
 *     def __dealloc__(self):
 *         # Note: Cannot raise in the destructor! (cython/cython#1613)
 *         if self.ptr:             # <<<<<<<<<<<<<<
 *             runtime.free(self.ptr)
 * 
*/
  }

  /* "cupy/cuda/memory.pyx":121
 *             self.ptr = runtime.malloc(size)
 * 
 *     def __dealloc__(self):             # <<<<<<<<<<<<<<
 *         # Note: Cannot raise in the destructor! (cython/cython#1613)
 *         if self.ptr:
*/

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_WriteUnraisable("cupy.cuda.memory.Memory.__dealloc__", __pyx_clineno, __pyx_lineno, __pyx_filename, 1, 0);
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
}

/* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     cdef tuple state
 *     cdef object _dict
*/

/* Python wrapper */
static PyObject *__pyx_pw_4cupy_4cuda_6memory_6Memory_5__reduce_cython__(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
PyDoc_STRVAR(__pyx_doc_4cupy_4cuda_6memory_6Memory_4__reduce_cython__, "Memory.__reduce_cython__(self)");
static PyObject *__pyx_pw_4cupy_4cuda_6memory_6Memory_5__reduce_cython__(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
) {
  #if !CYTHON_METH_FASTCALL
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  #endif
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__reduce_cython__ (wrapper)", 0);
  #if !CYTHON_METH_FASTCALL
  #if CYTHON_ASSUME_SAFE_SIZE
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return NULL;
  #endif
  #endif
  __pyx_kwvalues = __Pyx_KwValues_FASTCALL(__pyx_args, __pyx_nargs);
  if (unlikely(__pyx_nargs > 0)) { __Pyx_RaiseArgtupleInvalid("__reduce_cython__", 1, 0, 0, __pyx_nargs); return NULL; }
  const Py_ssize_t __pyx_kwds_len = unlikely(__pyx_kwds) ? __Pyx_NumKwargs_FASTCALL(__pyx_kwds) : 0;
  if (unlikely(__pyx_kwds_len < 0)) return NULL;
  if (unlikely(__pyx_kwds_len > 0)) {__Pyx_RejectKeywords("__reduce_cython__", __pyx_kwds); return NULL;}
  __pyx_r = __pyx_pf_4cupy_4cuda_6memory_6Memory_4__reduce_cython__(((struct __pyx_obj_4cupy_4cuda_6memory_Memory *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4cupy_4cuda_6memory_6Memory_4__reduce_cython__(struct __pyx_obj_4cupy_4cuda_6memory_Memory *__pyx_v_self) {
  PyObject *__pyx_v_state = 0;
  PyObject *__pyx_v__dict = 0;
  int __pyx_v_use_setstate;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  int __pyx_t_5;
  int __pyx_t_6;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__reduce_cython__", 0);

  /* "(tree fragment)":5
 *     cdef object _dict
 *     cdef bint use_setstate
 *     state = (self.device_id, self.ptr, self.size)             # <<<<<<<<<<<<<<
 *     _dict = getattr(self, '__dict__', None)
 *     if _dict is not None and _dict:
*/
  __pyx_t_1 = __Pyx_PyLong_From_int(__pyx_v_self->__pyx_base.device_id); if (unlikely(!__pyx_t_1)) __PYX_ERR(3, 5, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = PyLong_FromSsize_t(__pyx_v_self->__pyx_base.ptr); if (unlikely(!__pyx_t_2)) __PYX_ERR(3, 5, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = __Pyx_PyLong_FromSize_t(__pyx_v_self->__pyx_base.size); if (unlikely(!__pyx_t_3)) __PYX_ERR(3, 5, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_4 = PyTuple_New(3); if (unlikely(!__pyx_t_4)) __PYX_ERR(3, 5, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_GIVEREF(__pyx_t_1);
  if (__Pyx_PyTuple_SET_ITEM(__pyx_t_4, 0, __pyx_t_1) != (0)) __PYX_ERR(3, 5, __pyx_L1_error);
  __Pyx_GIVEREF(__pyx_t_2);
  if (__Pyx_PyTuple_SET_ITEM(__pyx_t_4, 1, __pyx_t_2) != (0)) __PYX_ERR(3, 5, __pyx_L1_error);
  __Pyx_GIVEREF(__pyx_t_3);
  if (__Pyx_PyTuple_SET_ITEM(__pyx_t_4, 2, __pyx_t_3) != (0)) __PYX_ERR(3, 5, __pyx_L1_error);
  __pyx_t_1 = 0;
  __pyx_t_2 = 0;
  __pyx_t_3 = 0;
  __pyx_v_state = ((PyObject*)__pyx_t_4);
  __pyx_t_4 = 0;

  /* "(tree fragment)":6
 *     cdef bint use_setstate
 *     state = (self.device_id, self.ptr, self.size)
 *     _dict = getattr(self, '__dict__', None)             # <<<<<<<<<<<<<<
 *     if _dict is not None and _dict:
 *         state += (_dict,)
*/
  __pyx_t_4 = __Pyx_GetAttr3(((PyObject *)__pyx_v_self), __pyx_mstate_global->__pyx_n_u_dict, Py_None); if (unlikely(!__pyx_t_4)) __PYX_ERR(3, 6, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_v__dict = __pyx_t_4;
  __pyx_t_4 = 0;

  /* "(tree fragment)":7
 *     state = (self.device_id, self.ptr, self.size)
 *     _dict = getattr(self, '__dict__', None)
 *     if _dict is not None and _dict:             # <<<<<<<<<<<<<<
 *         state += (_dict,)
 *         use_setstate = True
*/
  __pyx_t_6 = (__pyx_v__dict != Py_None);
  if (__pyx_t_6) {
  } else {
    __pyx_t_5 = __pyx_t_6;
    goto __pyx_L4_bool_binop_done;
  }
  __pyx_t_6 = __Pyx_PyObject_IsTrue(__pyx_v__dict); if (unlikely((__pyx_t_6 < 0))) __PYX_ERR(3, 7, __pyx_L1_error)
  __pyx_t_5 = __pyx_t_6;
  __pyx_L4_bool_binop_done:;
  if (__pyx_t_5) {

    /* "(tree fragment)":8
 *     _dict = getattr(self, '__dict__', None)
 *     if _dict is not None and _dict:
 *         state += (_dict,)             # <<<<<<<<<<<<<<
 *         use_setstate = True
 *     else:
*/
    __pyx_t_4 = PyTuple_New(1); if (unlikely(!__pyx_t_4)) __PYX_ERR(3, 8, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_INCREF(__pyx_v__dict);
    __Pyx_GIVEREF(__pyx_v__dict);
    if (__Pyx_PyTuple_SET_ITEM(__pyx_t_4, 0, __pyx_v__dict) != (0)) __PYX_ERR(3, 8, __pyx_L1_error);
    __pyx_t_3 = PyNumber_InPlaceAdd(__pyx_v_state, __pyx_t_4); if (unlikely(!__pyx_t_3)) __PYX_ERR(3, 8, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __Pyx_DECREF_SET(__pyx_v_state, ((PyObject*)__pyx_t_3));
    __pyx_t_3 = 0;

    /* "(tree fragment)":9
 *     if _dict is not None and _dict:
 *         state += (_dict,)
 *         use_setstate = True             # <<<<<<<<<<<<<<
 *     else:
 *         use_setstate = False
*/
    __pyx_v_use_setstate = 1;

    /* "(tree fragment)":7
 *     state = (self.device_id, self.ptr, self.size)
 *     _dict = getattr(self, '__dict__', None)
 *     if _dict is not None and _dict:             # <<<<<<<<<<<<<<
 *         state += (_dict,)
 *         use_setstate = True
*/
    goto __pyx_L3;
  }

  /* "(tree fragment)":11
 *         use_setstate = True
 *     else:
 *         use_setstate = False             # <<<<<<<<<<<<<<
 *     if use_setstate:
 *         return __pyx_unpickle_Memory, (type(self), 0x61cadb0, None), state
*/
  /*else*/ {
    __pyx_v_use_setstate = 0;
  }
  __pyx_L3:;

  /* "(tree fragment)":12
 *     else:
 *         use_setstate = False
 *     if use_setstate:             # <<<<<<<<<<<<<<
 *         return __pyx_unpickle_Memory, (type(self), 0x61cadb0, None), state
 *     else:
*/
  if (__pyx_v_use_setstate) {

    /* "(tree fragment)":13
 *         use_setstate = False
 *     if use_setstate:
 *         return __pyx_unpickle_Memory, (type(self), 0x61cadb0, None), state             # <<<<<<<<<<<<<<
 *     else:
 *         return __pyx_unpickle_Memory, (type(self), 0x61cadb0, state)
*/
    __Pyx_XDECREF(__pyx_r);
    __Pyx_GetModuleGlobalName(__pyx_t_3, __pyx_mstate_global->__pyx_n_u_pyx_unpickle_Memory); if (unlikely(!__pyx_t_3)) __PYX_ERR(3, 13, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_t_4 = PyTuple_New(3); if (unlikely(!__pyx_t_4)) __PYX_ERR(3, 13, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_INCREF(((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    __Pyx_GIVEREF(((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    if (__Pyx_PyTuple_SET_ITEM(__pyx_t_4, 0, ((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self)))) != (0)) __PYX_ERR(3, 13, __pyx_L1_error);
    __Pyx_INCREF(__pyx_mstate_global->__pyx_int_102542768);
    __Pyx_GIVEREF(__pyx_mstate_global->__pyx_int_102542768);
    if (__Pyx_PyTuple_SET_ITEM(__pyx_t_4, 1, __pyx_mstate_global->__pyx_int_102542768) != (0)) __PYX_ERR(3, 13, __pyx_L1_error);
    __Pyx_INCREF(Py_None);
    __Pyx_GIVEREF(Py_None);
    if (__Pyx_PyTuple_SET_ITEM(__pyx_t_4, 2, Py_None) != (0)) __PYX_ERR(3, 13, __pyx_L1_error);
    __pyx_t_2 = PyTuple_New(3); if (unlikely(!__pyx_t_2)) __PYX_ERR(3, 13, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_GIVEREF(__pyx_t_3);
    if (__Pyx_PyTuple_SET_ITEM(__pyx_t_2, 0, __pyx_t_3) != (0)) __PYX_ERR(3, 13, __pyx_L1_error);
    __Pyx_GIVEREF(__pyx_t_4);
    if (__Pyx_PyTuple_SET_ITEM(__pyx_t_2, 1, __pyx_t_4) != (0)) __PYX_ERR(3, 13, __pyx_L1_error);
    __Pyx_INCREF(__pyx_v_state);
    __Pyx_GIVEREF(__pyx_v_state);
    if (__Pyx_PyTuple_SET_ITEM(__pyx_t_2, 2, __pyx_v_state) != (0)) __PYX_ERR(3, 13, __pyx_L1_error);
    __pyx_t_3 = 0;
    __pyx_t_4 = 0;
    __pyx_r = __pyx_t_2;
    __pyx_t_2 = 0;
    goto __pyx_L0;

    /* "(tree fragment)":12
 *     else:
 *         use_setstate = False
 *     if use_setstate:             # <<<<<<<<<<<<<<
 *         return __pyx_unpickle_Memory, (type(self), 0x61cadb0, None), state
 *     else:
*/
  }

  /* "(tree fragment)":15
 *         return __pyx_unpickle_Memory, (type(self), 0x61cadb0, None), state
 *     else:
 *         return __pyx_unpickle_Memory, (type(self), 0x61cadb0, state)             # <<<<<<<<<<<<<<
 * def __setstate_cython__(self, __pyx_state):
 *     __pyx_unpickle_Memory__set_state(self, __pyx_state)
*/
  /*else*/ {
    __Pyx_XDECREF(__pyx_r);
    __Pyx_GetModuleGlobalName(__pyx_t_2, __pyx_mstate_global->__pyx_n_u_pyx_unpickle_Memory); if (unlikely(!__pyx_t_2)) __PYX_ERR(3, 15, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_4 = PyTuple_New(3); if (unlikely(!__pyx_t_4)) __PYX_ERR(3, 15, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_INCREF(((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    __Pyx_GIVEREF(((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    if (__Pyx_PyTuple_SET_ITEM(__pyx_t_4, 0, ((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self)))) != (0)) __PYX_ERR(3, 15, __pyx_L1_error);
    __Pyx_INCREF(__pyx_mstate_global->__pyx_int_102542768);
    __Pyx_GIVEREF(__pyx_mstate_global->__pyx_int_102542768);
    if (__Pyx_PyTuple_SET_ITEM(__pyx_t_4, 1, __pyx_mstate_global->__pyx_int_102542768) != (0)) __PYX_ERR(3, 15, __pyx_L1_error);
    __Pyx_INCREF(__pyx_v_state);
    __Pyx_GIVEREF(__pyx_v_state);
    if (__Pyx_PyTuple_SET_ITEM(__pyx_t_4, 2, __pyx_v_state) != (0)) __PYX_ERR(3, 15, __pyx_L1_error);
    __pyx_t_3 = PyTuple_New(2); if (unlikely(!__pyx_t_3)) __PYX_ERR(3, 15, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_GIVEREF(__pyx_t_2);
    if (__Pyx_PyTuple_SET_ITEM(__pyx_t_3, 0, __pyx_t_2) != (0)) __PYX_ERR(3, 15, __pyx_L1_error);
    __Pyx_GIVEREF(__pyx_t_4);
    if (__Pyx_PyTuple_SET_ITEM(__pyx_t_3, 1, __pyx_t_4) != (0)) __PYX_ERR(3, 15, __pyx_L1_error);
    __pyx_t_2 = 0;
    __pyx_t_4 = 0;
    __pyx_r = __pyx_t_3;
    __pyx_t_3 = 0;
    goto __pyx_L0;
  }

  /* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     cdef tuple state
 *     cdef object _dict
*/

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_AddTraceback("cupy.cuda.memory.Memory.__reduce_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_state);
  __Pyx_XDECREF(__pyx_v__dict);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":16
 *     else:
 *         return __pyx_unpickle_Memory, (type(self), 0x61cadb0, state)
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     __pyx_unpickle_Memory__set_state(self, __pyx_state)
*/

/* Python wrapper */
static PyObject *__pyx_pw_4cupy_4cuda_6memory_6Memory_7__setstate_cython__(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
PyDoc_STRVAR(__pyx_doc_4cupy_4cuda_6memory_6Memory_6__setstate_cython__, "Memory.__setstate_cython__(self, __pyx_state)");
static PyObject *__pyx_pw_4cupy_4cuda_6memory_6Memory_7__setstate_cython__(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
) {
  PyObject *__pyx_v___pyx_state = 0;
  #if !CYTHON_METH_FASTCALL
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  #endif
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject* values[1] = {0};
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__setstate_cython__ (wrapper)", 0);
  #if !CYTHON_METH_FASTCALL
  #if CYTHON_ASSUME_SAFE_SIZE
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return NULL;
  #endif
  #endif
  __pyx_kwvalues = __Pyx_KwValues_FASTCALL(__pyx_args, __pyx_nargs);
  {
    PyObject ** const __pyx_pyargnames[] = {&__pyx_mstate_global->__pyx_n_u_pyx_state,0};
    const Py_ssize_t __pyx_kwds_len = (__pyx_kwds) ? __Pyx_NumKwargs_FASTCALL(__pyx_kwds) : 0;
    if (unlikely(__pyx_kwds_len < 0)) __PYX_ERR(3, 16, __pyx_L3_error)
    if (__pyx_kwds_len > 0) {
      switch (__pyx_nargs) {
        case  1:
        values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(3, 16, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      const Py_ssize_t kwd_pos_args = __pyx_nargs;
      if (__Pyx_ParseKeywords(__pyx_kwds, __pyx_kwvalues, __pyx_pyargnames, 0, values, kwd_pos_args, __pyx_kwds_len, "__setstate_cython__", 0) < (0)) __PYX_ERR(3, 16, __pyx_L3_error)
      for (Py_ssize_t i = __pyx_nargs; i < 1; i++) {
        if (unlikely(!values[i])) { __Pyx_RaiseArgtupleInvalid("__setstate_cython__", 1, 1, 1, i); __PYX_ERR(3, 16, __pyx_L3_error) }
      }
    } else if (unlikely(__pyx_nargs != 1)) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
      if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(3, 16, __pyx_L3_error)
    }
    __pyx_v___pyx_state = values[0];
  }
  goto __pyx_L6_skip;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("__setstate_cython__", 1, 1, 1, __pyx_nargs); __PYX_ERR(3, 16, __pyx_L3_error)
  __pyx_L6_skip:;
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_AddTraceback("cupy.cuda.memory.Memory.__setstate_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_4cupy_4cuda_6memory_6Memory_6__setstate_cython__(((struct __pyx_obj_4cupy_4cuda_6memory_Memory *)__pyx_v_self), __pyx_v___pyx_state);

  /* function exit code */
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4cupy_4cuda_6memory_6Memory_6__setstate_cython__(struct __pyx_obj_4cupy_4cuda_6memory_Memory *__pyx_v_self, PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__setstate_cython__", 0);

  /* "(tree fragment)":17
 *         return __pyx_unpickle_Memory, (type(self), 0x61cadb0, state)
 * def __setstate_cython__(self, __pyx_state):
 *     __pyx_unpickle_Memory__set_state(self, __pyx_state)             # <<<<<<<<<<<<<<
*/
  __pyx_t_1 = __pyx_v___pyx_state;
  __Pyx_INCREF(__pyx_t_1);
  if (!(likely(PyTuple_CheckExact(__pyx_t_1))||((__pyx_t_1) == Py_None) || __Pyx_RaiseUnexpectedTypeError("tuple", __pyx_t_1))) __PYX_ERR(3, 17, __pyx_L1_error)
  if (unlikely(__pyx_t_1 == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "cannot pass None into a C function argument that is declared 'not None'");
    __PYX_ERR(3, 17, __pyx_L1_error)
  }
  __pyx_t_2 = __pyx_f_4cupy_4cuda_6memory___pyx_unpickle_Memory__set_state(__pyx_v_self, ((PyObject*)__pyx_t_1)); if (unlikely(!__pyx_t_2)) __PYX_ERR(3, 17, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "(tree fragment)":16
 *     else:
 *         return __pyx_unpickle_Memory, (type(self), 0x61cadb0, state)
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     __pyx_unpickle_Memory__set_state(self, __pyx_state)
*/

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_AddTraceback("cupy.cuda.memory.Memory.__setstate_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "cupy/cuda/memory.pyx":127
 * 
 * 
 * cdef inline void check_async_alloc_supported(int device_id) except*:             # <<<<<<<<<<<<<<
 *     if runtime._is_hip_environment:
 *         raise RuntimeError('HIP does not support memory_async')
*/

static CYTHON_INLINE void __pyx_f_4cupy_4cuda_6memory_check_async_alloc_supported(int __pyx_v_device_id) {
  PyObject *__pyx_v_support = 0;
  PyObject *__pyx_v_is_supported = NULL;
  int __pyx_7genexpr__pyx_v_dev_id;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  size_t __pyx_t_3;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  PyObject *__pyx_t_6 = NULL;
  int __pyx_t_7;
  PyObject *__pyx_t_8 = NULL;
  PyObject *__pyx_t_9 = NULL;
  int __pyx_t_10;
  int __pyx_t_11;
  int __pyx_t_12;
  PyObject *__pyx_t_13 = NULL;
  int __pyx_t_14;
  int __pyx_t_15;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("check_async_alloc_supported", 0);

  /* "cupy/cuda/memory.pyx":128
 * 
 * cdef inline void check_async_alloc_supported(int device_id) except*:
 *     if runtime._is_hip_environment:             # <<<<<<<<<<<<<<
 *         raise RuntimeError('HIP does not support memory_async')
 *     cdef int dev_id
*/
  if (unlikely(__pyx_v_13cupy_backends_4cuda_3api_7runtime__is_hip_environment)) {

    /* "cupy/cuda/memory.pyx":129
 * cdef inline void check_async_alloc_supported(int device_id) except*:
 *     if runtime._is_hip_environment:
 *         raise RuntimeError('HIP does not support memory_async')             # <<<<<<<<<<<<<<
 *     cdef int dev_id
 *     cdef list support
*/
    __pyx_t_2 = NULL;
    __pyx_t_3 = 1;
    {
      PyObject *__pyx_callargs[2] = {__pyx_t_2, __pyx_mstate_global->__pyx_kp_u_HIP_does_not_support_memory_asyn};
      __pyx_t_1 = __Pyx_PyObject_FastCall((PyObject*)(((PyTypeObject*)PyExc_RuntimeError)), __pyx_callargs+__pyx_t_3, (2-__pyx_t_3) | (__pyx_t_3*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
      __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
      if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 129, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
    }
    __Pyx_Raise(__pyx_t_1, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __PYX_ERR(0, 129, __pyx_L1_error)

    /* "cupy/cuda/memory.pyx":128
 * 
 * cdef inline void check_async_alloc_supported(int device_id) except*:
 *     if runtime._is_hip_environment:             # <<<<<<<<<<<<<<
 *         raise RuntimeError('HIP does not support memory_async')
 *     cdef int dev_id
*/
  }

  /* "cupy/cuda/memory.pyx":132
 *     cdef int dev_id
 *     cdef list support
 *     try:             # <<<<<<<<<<<<<<
 *         is_supported = _thread_local.device_support_async_alloc[device_id]
 *     except AttributeError:
*/
  {
    __Pyx_PyThreadState_declare
    __Pyx_PyThreadState_assign
    __Pyx_ExceptionSave(&__pyx_t_4, &__pyx_t_5, &__pyx_t_6);
    __Pyx_XGOTREF(__pyx_t_4);
    __Pyx_XGOTREF(__pyx_t_5);
    __Pyx_XGOTREF(__pyx_t_6);
    /*try:*/ {

      /* "cupy/cuda/memory.pyx":133
 *     cdef list support
 *     try:
 *         is_supported = _thread_local.device_support_async_alloc[device_id]             # <<<<<<<<<<<<<<
 *     except AttributeError:
 *         support = [runtime.deviceGetAttribute(
*/
      __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_v_4cupy_4cuda_6memory__thread_local, __pyx_mstate_global->__pyx_n_u_device_support_async_alloc); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 133, __pyx_L4_error)
      __Pyx_GOTREF(__pyx_t_1);
      __pyx_t_2 = __Pyx_GetItemInt(__pyx_t_1, __pyx_v_device_id, int, 1, __Pyx_PyLong_From_int, 0, 1, 1, 1, __Pyx_ReferenceSharing_OwnStrongReference); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 133, __pyx_L4_error)
      __Pyx_GOTREF(__pyx_t_2);
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      __pyx_v_is_supported = __pyx_t_2;
      __pyx_t_2 = 0;

      /* "cupy/cuda/memory.pyx":132
 *     cdef int dev_id
 *     cdef list support
 *     try:             # <<<<<<<<<<<<<<
 *         is_supported = _thread_local.device_support_async_alloc[device_id]
 *     except AttributeError:
*/
    }
    __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
    __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
    __Pyx_XDECREF(__pyx_t_6); __pyx_t_6 = 0;
    goto __pyx_L9_try_end;
    __pyx_L4_error:;
    __Pyx_XDECREF(__pyx_t_1); __pyx_t_1 = 0;
    __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;

    /* "cupy/cuda/memory.pyx":134
 *     try:
 *         is_supported = _thread_local.device_support_async_alloc[device_id]
 *     except AttributeError:             # <<<<<<<<<<<<<<
 *         support = [runtime.deviceGetAttribute(
 *             runtime.cudaDevAttrMemoryPoolsSupported, dev_id)
*/
    __pyx_t_7 = __Pyx_PyErr_ExceptionMatches(((PyObject *)(((PyTypeObject*)PyExc_AttributeError))));
    if (__pyx_t_7) {
      __Pyx_AddTraceback("cupy.cuda.memory.check_async_alloc_supported", __pyx_clineno, __pyx_lineno, __pyx_filename);
      if (__Pyx_GetException(&__pyx_t_2, &__pyx_t_1, &__pyx_t_8) < 0) __PYX_ERR(0, 134, __pyx_L6_except_error)
      __Pyx_XGOTREF(__pyx_t_2);
      __Pyx_XGOTREF(__pyx_t_1);
      __Pyx_XGOTREF(__pyx_t_8);

      /* "cupy/cuda/memory.pyx":135
 *         is_supported = _thread_local.device_support_async_alloc[device_id]
 *     except AttributeError:
 *         support = [runtime.deviceGetAttribute(             # <<<<<<<<<<<<<<
 *             runtime.cudaDevAttrMemoryPoolsSupported, dev_id)
 *             for dev_id in range(runtime.getDeviceCount())]
*/
      { /* enter inner scope */
        __pyx_t_9 = PyList_New(0); if (unlikely(!__pyx_t_9)) __PYX_ERR(0, 135, __pyx_L6_except_error)
        __Pyx_GOTREF(__pyx_t_9);

        /* "cupy/cuda/memory.pyx":137
 *         support = [runtime.deviceGetAttribute(
 *             runtime.cudaDevAttrMemoryPoolsSupported, dev_id)
 *             for dev_id in range(runtime.getDeviceCount())]             # <<<<<<<<<<<<<<
 *         _thread_local.device_support_async_alloc = support
 *         is_supported = support[device_id]
*/
        __pyx_t_7 = __pyx_f_13cupy_backends_4cuda_3api_7runtime_getDeviceCount(0); if (unlikely(__pyx_t_7 == ((int)-1) && PyErr_Occurred())) __PYX_ERR(0, 137, __pyx_L6_except_error)
        __pyx_t_10 = __pyx_t_7;
        for (__pyx_t_11 = 0; __pyx_t_11 < __pyx_t_10; __pyx_t_11+=1) {
          __pyx_7genexpr__pyx_v_dev_id = __pyx_t_11;

          /* "cupy/cuda/memory.pyx":135
 *         is_supported = _thread_local.device_support_async_alloc[device_id]
 *     except AttributeError:
 *         support = [runtime.deviceGetAttribute(             # <<<<<<<<<<<<<<
 *             runtime.cudaDevAttrMemoryPoolsSupported, dev_id)
 *             for dev_id in range(runtime.getDeviceCount())]
*/
          __pyx_t_12 = __pyx_f_13cupy_backends_4cuda_3api_7runtime_deviceGetAttribute(__pyx_e_13cupy_backends_4cuda_3api_13_runtime_enum_cudaDevAttrMemoryPoolsSupported, __pyx_7genexpr__pyx_v_dev_id, 0); if (unlikely(__pyx_t_12 == ((int)-1) && PyErr_Occurred())) __PYX_ERR(0, 135, __pyx_L6_except_error)
          __pyx_t_13 = __Pyx_PyLong_From_int(__pyx_t_12); if (unlikely(!__pyx_t_13)) __PYX_ERR(0, 135, __pyx_L6_except_error)
          __Pyx_GOTREF(__pyx_t_13);
          if (unlikely(__Pyx_ListComp_Append(__pyx_t_9, (PyObject*)__pyx_t_13))) __PYX_ERR(0, 135, __pyx_L6_except_error)
          __Pyx_DECREF(__pyx_t_13); __pyx_t_13 = 0;
        }
      } /* exit inner scope */
      __pyx_v_support = ((PyObject*)__pyx_t_9);
      __pyx_t_9 = 0;

      /* "cupy/cuda/memory.pyx":138
 *             runtime.cudaDevAttrMemoryPoolsSupported, dev_id)
 *             for dev_id in range(runtime.getDeviceCount())]
 *         _thread_local.device_support_async_alloc = support             # <<<<<<<<<<<<<<
 *         is_supported = support[device_id]
 *     if not is_supported:
*/
      if (__Pyx_PyObject_SetAttrStr(__pyx_v_4cupy_4cuda_6memory__thread_local, __pyx_mstate_global->__pyx_n_u_device_support_async_alloc, __pyx_v_support) < (0)) __PYX_ERR(0, 138, __pyx_L6_except_error)

      /* "cupy/cuda/memory.pyx":139
 *             for dev_id in range(runtime.getDeviceCount())]
 *         _thread_local.device_support_async_alloc = support
 *         is_supported = support[device_id]             # <<<<<<<<<<<<<<
 *     if not is_supported:
 *         raise RuntimeError('Device {} does not support '
*/
      __pyx_t_9 = __Pyx_GetItemInt_List(__pyx_v_support, __pyx_v_device_id, int, 1, __Pyx_PyLong_From_int, 1, 1, 1, 1, __Pyx_ReferenceSharing_OwnStrongReference); if (unlikely(!__pyx_t_9)) __PYX_ERR(0, 139, __pyx_L6_except_error)
      __Pyx_GOTREF(__pyx_t_9);
      __Pyx_XDECREF_SET(__pyx_v_is_supported, __pyx_t_9);
      __pyx_t_9 = 0;
      __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
      __Pyx_XDECREF(__pyx_t_1); __pyx_t_1 = 0;
      __Pyx_XDECREF(__pyx_t_8); __pyx_t_8 = 0;
      goto __pyx_L5_exception_handled;
    }
    goto __pyx_L6_except_error;

    /* "cupy/cuda/memory.pyx":132
 *     cdef int dev_id
 *     cdef list support
 *     try:             # <<<<<<<<<<<<<<
 *         is_supported = _thread_local.device_support_async_alloc[device_id]
 *     except AttributeError:
*/
    __pyx_L6_except_error:;
    __Pyx_XGIVEREF(__pyx_t_4);
    __Pyx_XGIVEREF(__pyx_t_5);
    __Pyx_XGIVEREF(__pyx_t_6);
    __Pyx_ExceptionReset(__pyx_t_4, __pyx_t_5, __pyx_t_6);
    goto __pyx_L1_error;
    __pyx_L5_exception_handled:;
    __Pyx_XGIVEREF(__pyx_t_4);
    __Pyx_XGIVEREF(__pyx_t_5);
    __Pyx_XGIVEREF(__pyx_t_6);
    __Pyx_ExceptionReset(__pyx_t_4, __pyx_t_5, __pyx_t_6);
    __pyx_L9_try_end:;
  }

  /* "cupy/cuda/memory.pyx":140
 *         _thread_local.device_support_async_alloc = support
 *         is_supported = support[device_id]
 *     if not is_supported:             # <<<<<<<<<<<<<<
 *         raise RuntimeError('Device {} does not support '
 *                            'malloc_async'.format(device_id))
*/
  __pyx_t_14 = __Pyx_PyObject_IsTrue(__pyx_v_is_supported); if (unlikely((__pyx_t_14 < 0))) __PYX_ERR(0, 140, __pyx_L1_error)
  __pyx_t_15 = (!__pyx_t_14);
  if (unlikely(__pyx_t_15)) {

    /* "cupy/cuda/memory.pyx":141
 *         is_supported = support[device_id]
 *     if not is_supported:
 *         raise RuntimeError('Device {} does not support '             # <<<<<<<<<<<<<<
 *                            'malloc_async'.format(device_id))
 * 
*/
    __pyx_t_1 = NULL;

    /* "cupy/cuda/memory.pyx":142
 *     if not is_supported:
 *         raise RuntimeError('Device {} does not support '
 *                            'malloc_async'.format(device_id))             # <<<<<<<<<<<<<<
 * 
 * 
*/
    __pyx_t_9 = __pyx_mstate_global->__pyx_kp_u_Device_does_not_support_malloc_a;
    __Pyx_INCREF(__pyx_t_9);
    __pyx_t_13 = __Pyx_PyLong_From_int(__pyx_v_device_id); if (unlikely(!__pyx_t_13)) __PYX_ERR(0, 142, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_13);
    __pyx_t_3 = 0;
    {
      PyObject *__pyx_callargs[2] = {__pyx_t_9, __pyx_t_13};
      __pyx_t_2 = __Pyx_PyObject_FastCallMethod((PyObject*)__pyx_mstate_global->__pyx_n_u_format, __pyx_callargs+__pyx_t_3, (2-__pyx_t_3) | (1*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
      __Pyx_XDECREF(__pyx_t_9); __pyx_t_9 = 0;
      __Pyx_DECREF(__pyx_t_13); __pyx_t_13 = 0;
      if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 142, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_2);
    }
    __pyx_t_3 = 1;
    {
      PyObject *__pyx_callargs[2] = {__pyx_t_1, __pyx_t_2};
      __pyx_t_8 = __Pyx_PyObject_FastCall((PyObject*)(((PyTypeObject*)PyExc_RuntimeError)), __pyx_callargs+__pyx_t_3, (2-__pyx_t_3) | (__pyx_t_3*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
      __Pyx_XDECREF(__pyx_t_1); __pyx_t_1 = 0;
      __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
      if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 141, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_8);
    }
    __Pyx_Raise(__pyx_t_8, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
    __PYX_ERR(0, 141, __pyx_L1_error)

    /* "cupy/cuda/memory.pyx":140
 *         _thread_local.device_support_async_alloc = support
 *         is_supported = support[device_id]
 *     if not is_supported:             # <<<<<<<<<<<<<<
 *         raise RuntimeError('Device {} does not support '
 *                            'malloc_async'.format(device_id))
*/
  }

  /* "cupy/cuda/memory.pyx":127
 * 
 * 
 * cdef inline void check_async_alloc_supported(int device_id) except*:             # <<<<<<<<<<<<<<
 *     if runtime._is_hip_environment:
 *         raise RuntimeError('HIP does not support memory_async')
*/

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_8);
  __Pyx_XDECREF(__pyx_t_9);
  __Pyx_XDECREF(__pyx_t_13);
  __Pyx_AddTraceback("cupy.cuda.memory.check_async_alloc_supported", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_support);
  __Pyx_XDECREF(__pyx_v_is_supported);
  __Pyx_RefNannyFinishContext();
}

/* "cupy/cuda/memory.pyx":159
 *         readonly object stream_ref
 * 
 *     def __init__(self, size_t size, stream):             # <<<<<<<<<<<<<<
 *         self.size = size
 *         self.device_id = device.get_device_id()
*/

/* Python wrapper */
static int __pyx_pw_4cupy_4cuda_6memory_11MemoryAsync_1__init__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static int __pyx_pw_4cupy_4cuda_6memory_11MemoryAsync_1__init__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  size_t __pyx_v_size;
  PyObject *__pyx_v_stream = 0;
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject* values[2] = {0,0};
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__init__ (wrapper)", 0);
  #if CYTHON_ASSUME_SAFE_SIZE
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return -1;
  #endif
  __pyx_kwvalues = __Pyx_KwValues_VARARGS(__pyx_args, __pyx_nargs);
  {
    PyObject ** const __pyx_pyargnames[] = {&__pyx_mstate_global->__pyx_n_u_size,&__pyx_mstate_global->__pyx_n_u_stream,0};
    const Py_ssize_t __pyx_kwds_len = (__pyx_kwds) ? __Pyx_NumKwargs_VARARGS(__pyx_kwds) : 0;
    if (unlikely(__pyx_kwds_len < 0)) __PYX_ERR(0, 159, __pyx_L3_error)
    if (__pyx_kwds_len > 0) {
      switch (__pyx_nargs) {
        case  2:
        values[1] = __Pyx_ArgRef_VARARGS(__pyx_args, 1);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[1])) __PYX_ERR(0, 159, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  1:
        values[0] = __Pyx_ArgRef_VARARGS(__pyx_args, 0);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(0, 159, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      const Py_ssize_t kwd_pos_args = __pyx_nargs;
      if (__Pyx_ParseKeywords(__pyx_kwds, __pyx_kwvalues, __pyx_pyargnames, 0, values, kwd_pos_args, __pyx_kwds_len, "__init__", 0) < (0)) __PYX_ERR(0, 159, __pyx_L3_error)
      for (Py_ssize_t i = __pyx_nargs; i < 2; i++) {
        if (unlikely(!values[i])) { __Pyx_RaiseArgtupleInvalid("__init__", 1, 2, 2, i); __PYX_ERR(0, 159, __pyx_L3_error) }
      }
    } else if (unlikely(__pyx_nargs != 2)) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = __Pyx_ArgRef_VARARGS(__pyx_args, 0);
      if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(0, 159, __pyx_L3_error)
      values[1] = __Pyx_ArgRef_VARARGS(__pyx_args, 1);
      if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[1])) __PYX_ERR(0, 159, __pyx_L3_error)
    }
    __pyx_v_size = __Pyx_PyLong_As_size_t(values[0]); if (unlikely((__pyx_v_size == (size_t)-1) && PyErr_Occurred())) __PYX_ERR(0, 159, __pyx_L3_error)
    __pyx_v_stream = values[1];
  }
  goto __pyx_L6_skip;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("__init__", 1, 2, 2, __pyx_nargs); __PYX_ERR(0, 159, __pyx_L3_error)
  __pyx_L6_skip:;
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_AddTraceback("cupy.cuda.memory.MemoryAsync.__init__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return -1;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_4cupy_4cuda_6memory_11MemoryAsync___init__(((struct __pyx_obj_4cupy_4cuda_6memory_MemoryAsync *)__pyx_v_self), __pyx_v_size, __pyx_v_stream);

  /* function exit code */
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_4cupy_4cuda_6memory_11MemoryAsync___init__(struct __pyx_obj_4cupy_4cuda_6memory_MemoryAsync *__pyx_v_self, size_t __pyx_v_size, PyObject *__pyx_v_stream) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  size_t __pyx_t_6;
  int __pyx_t_7;
  intptr_t __pyx_t_8;
  intptr_t __pyx_t_9;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__init__", 0);

  /* "cupy/cuda/memory.pyx":160
 * 
 *     def __init__(self, size_t size, stream):
 *         self.size = size             # <<<<<<<<<<<<<<
 *         self.device_id = device.get_device_id()
 *         # The stream is allowed to be destroyed before the memory is freed, so
*/
  __pyx_v_self->__pyx_base.size = __pyx_v_size;

  /* "cupy/cuda/memory.pyx":161
 *     def __init__(self, size_t size, stream):
 *         self.size = size
 *         self.device_id = device.get_device_id()             # <<<<<<<<<<<<<<
 *         # The stream is allowed to be destroyed before the memory is freed, so
 *         # we don't need to hold a strong reference to the stream.
*/
  __pyx_t_1 = __pyx_f_4cupy_4cuda_6device_get_device_id(0); if (unlikely(__pyx_t_1 == ((int)-1) && PyErr_Occurred())) __PYX_ERR(0, 161, __pyx_L1_error)
  __pyx_v_self->__pyx_base.device_id = __pyx_t_1;

  /* "cupy/cuda/memory.pyx":164
 *         # The stream is allowed to be destroyed before the memory is freed, so
 *         # we don't need to hold a strong reference to the stream.
 *         self.stream_ref = weakref.ref(stream)             # <<<<<<<<<<<<<<
 *         check_async_alloc_supported(self.device_id)
 *         if size > 0:
*/
  __pyx_t_3 = NULL;
  __Pyx_GetModuleGlobalName(__pyx_t_4, __pyx_mstate_global->__pyx_n_u_weakref); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 164, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_t_5 = __Pyx_PyObject_GetAttrStr(__pyx_t_4, __pyx_mstate_global->__pyx_n_u_ref); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 164, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_5);
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __pyx_t_6 = 1;
  #if CYTHON_UNPACK_METHODS
  if (unlikely(PyMethod_Check(__pyx_t_5))) {
    __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_5);
    assert(__pyx_t_3);
    PyObject* __pyx__function = PyMethod_GET_FUNCTION(__pyx_t_5);
    __Pyx_INCREF(__pyx_t_3);
    __Pyx_INCREF(__pyx__function);
    __Pyx_DECREF_SET(__pyx_t_5, __pyx__function);
    __pyx_t_6 = 0;
  }
  #endif
  {
    PyObject *__pyx_callargs[2] = {__pyx_t_3, __pyx_v_stream};
    __pyx_t_2 = __Pyx_PyObject_FastCall((PyObject*)__pyx_t_5, __pyx_callargs+__pyx_t_6, (2-__pyx_t_6) | (__pyx_t_6*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
    __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
    if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 164, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
  }
  __Pyx_GIVEREF(__pyx_t_2);
  __Pyx_GOTREF(__pyx_v_self->stream_ref);
  __Pyx_DECREF(__pyx_v_self->stream_ref);
  __pyx_v_self->stream_ref = __pyx_t_2;
  __pyx_t_2 = 0;

  /* "cupy/cuda/memory.pyx":165
 *         # we don't need to hold a strong reference to the stream.
 *         self.stream_ref = weakref.ref(stream)
 *         check_async_alloc_supported(self.device_id)             # <<<<<<<<<<<<<<
 *         if size > 0:
 *             self.ptr = runtime.mallocAsync(size, stream.ptr)
*/
  __pyx_f_4cupy_4cuda_6memory_check_async_alloc_supported(__pyx_v_self->__pyx_base.device_id); if (unlikely(PyErr_Occurred())) __PYX_ERR(0, 165, __pyx_L1_error)

  /* "cupy/cuda/memory.pyx":166
 *         self.stream_ref = weakref.ref(stream)
 *         check_async_alloc_supported(self.device_id)
 *         if size > 0:             # <<<<<<<<<<<<<<
 *             self.ptr = runtime.mallocAsync(size, stream.ptr)
 * 
*/
  __pyx_t_7 = (__pyx_v_size > 0);
  if (__pyx_t_7) {

    /* "cupy/cuda/memory.pyx":167
 *         check_async_alloc_supported(self.device_id)
 *         if size > 0:
 *             self.ptr = runtime.mallocAsync(size, stream.ptr)             # <<<<<<<<<<<<<<
 * 
 *     def __dealloc__(self):
*/
    __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_v_stream, __pyx_mstate_global->__pyx_n_u_ptr); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 167, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_8 = PyLong_AsSsize_t(__pyx_t_2); if (unlikely((__pyx_t_8 == ((intptr_t)-1)) && PyErr_Occurred())) __PYX_ERR(0, 167, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __pyx_t_9 = __pyx_f_13cupy_backends_4cuda_3api_7runtime_mallocAsync(__pyx_v_size, __pyx_t_8, 0); if (unlikely(__pyx_t_9 == ((intptr_t)0) && PyErr_Occurred())) __PYX_ERR(0, 167, __pyx_L1_error)
    __pyx_v_self->__pyx_base.ptr = __pyx_t_9;

    /* "cupy/cuda/memory.pyx":166
 *         self.stream_ref = weakref.ref(stream)
 *         check_async_alloc_supported(self.device_id)
 *         if size > 0:             # <<<<<<<<<<<<<<
 *             self.ptr = runtime.mallocAsync(size, stream.ptr)
 * 
*/
  }

  /* "cupy/cuda/memory.pyx":159
 *         readonly object stream_ref
 * 
 *     def __init__(self, size_t size, stream):             # <<<<<<<<<<<<<<
 *         self.size = size
 *         self.device_id = device.get_device_id()
*/

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_AddTraceback("cupy.cuda.memory.MemoryAsync.__init__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "cupy/cuda/memory.pyx":169
 *             self.ptr = runtime.mallocAsync(size, stream.ptr)
 * 
 *     def __dealloc__(self):             # <<<<<<<<<<<<<<
 *         # Free on the stream on which this memory was allocated.
 *         # If the stream is already destroyed, free on the current stream. In
*/

/* Python wrapper */
static void __pyx_pw_4cupy_4cuda_6memory_11MemoryAsync_3__dealloc__(PyObject *__pyx_v_self); /*proto*/
static void __pyx_pw_4cupy_4cuda_6memory_11MemoryAsync_3__dealloc__(PyObject *__pyx_v_self) {
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__dealloc__ (wrapper)", 0);
  __pyx_kwvalues = __Pyx_KwValues_VARARGS(__pyx_args, __pyx_nargs);
  __pyx_pf_4cupy_4cuda_6memory_11MemoryAsync_2__dealloc__(((struct __pyx_obj_4cupy_4cuda_6memory_MemoryAsync *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
}

static void __pyx_pf_4cupy_4cuda_6memory_11MemoryAsync_2__dealloc__(struct __pyx_obj_4cupy_4cuda_6memory_MemoryAsync *__pyx_v_self) {
  PyObject *__pyx_v_stream = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  size_t __pyx_t_5;
  intptr_t __pyx_t_6;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__dealloc__", 0);

  /* "cupy/cuda/memory.pyx":173
 *         # If the stream is already destroyed, free on the current stream. In
 *         # this case, we trust the user has established a correct stream order.
 *         if self.ptr == 0:             # <<<<<<<<<<<<<<
 *             return
 *         stream = self.stream_ref()
*/
  __pyx_t_1 = (__pyx_v_self->__pyx_base.ptr == 0);
  if (__pyx_t_1) {

    /* "cupy/cuda/memory.pyx":174
 *         # this case, we trust the user has established a correct stream order.
 *         if self.ptr == 0:
 *             return             # <<<<<<<<<<<<<<
 *         stream = self.stream_ref()
 *         if stream is None:
*/
    goto __pyx_L0;

    /* "cupy/cuda/memory.pyx":173
 *         # If the stream is already destroyed, free on the current stream. In
 *         # this case, we trust the user has established a correct stream order.
 *         if self.ptr == 0:             # <<<<<<<<<<<<<<
 *             return
 *         stream = self.stream_ref()
*/
  }

  /* "cupy/cuda/memory.pyx":175
 *         if self.ptr == 0:
 *             return
 *         stream = self.stream_ref()             # <<<<<<<<<<<<<<
 *         if stream is None:
 *             stream = stream_module.get_current_stream()
*/
  __pyx_t_3 = NULL;
  __Pyx_INCREF(__pyx_v_self->stream_ref);
  __pyx_t_4 = __pyx_v_self->stream_ref; 
  __pyx_t_5 = 1;
  #if CYTHON_UNPACK_METHODS
  if (likely(PyMethod_Check(__pyx_t_4))) {
    __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_4);
    assert(__pyx_t_3);
    PyObject* __pyx__function = PyMethod_GET_FUNCTION(__pyx_t_4);
    __Pyx_INCREF(__pyx_t_3);
    __Pyx_INCREF(__pyx__function);
    __Pyx_DECREF_SET(__pyx_t_4, __pyx__function);
    __pyx_t_5 = 0;
  }
  #endif
  {
    PyObject *__pyx_callargs[2] = {__pyx_t_3, NULL};
    __pyx_t_2 = __Pyx_PyObject_FastCall((PyObject*)__pyx_t_4, __pyx_callargs+__pyx_t_5, (1-__pyx_t_5) | (__pyx_t_5*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
    __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 175, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
  }
  __pyx_v_stream = __pyx_t_2;
  __pyx_t_2 = 0;

  /* "cupy/cuda/memory.pyx":176
 *             return
 *         stream = self.stream_ref()
 *         if stream is None:             # <<<<<<<<<<<<<<
 *             stream = stream_module.get_current_stream()
 *         runtime.freeAsync(self.ptr, stream.ptr)
*/
  __pyx_t_1 = (__pyx_v_stream == Py_None);
  if (__pyx_t_1) {

    /* "cupy/cuda/memory.pyx":177
 *         stream = self.stream_ref()
 *         if stream is None:
 *             stream = stream_module.get_current_stream()             # <<<<<<<<<<<<<<
 *         runtime.freeAsync(self.ptr, stream.ptr)
 * 
*/
    __pyx_t_2 = __pyx_f_4cupy_4cuda_6stream_get_current_stream(0, NULL); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 177, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_DECREF_SET(__pyx_v_stream, __pyx_t_2);
    __pyx_t_2 = 0;

    /* "cupy/cuda/memory.pyx":176
 *             return
 *         stream = self.stream_ref()
 *         if stream is None:             # <<<<<<<<<<<<<<
 *             stream = stream_module.get_current_stream()
 *         runtime.freeAsync(self.ptr, stream.ptr)
*/
  }

  /* "cupy/cuda/memory.pyx":178
 *         if stream is None:
 *             stream = stream_module.get_current_stream()
 *         runtime.freeAsync(self.ptr, stream.ptr)             # <<<<<<<<<<<<<<
 * 
 * 
*/
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_v_stream, __pyx_mstate_global->__pyx_n_u_ptr); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 178, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_6 = PyLong_AsSsize_t(__pyx_t_2); if (unlikely((__pyx_t_6 == ((intptr_t)-1)) && PyErr_Occurred())) __PYX_ERR(0, 178, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_2 = __pyx_f_13cupy_backends_4cuda_3api_7runtime_freeAsync(__pyx_v_self->__pyx_base.ptr, __pyx_t_6, 0); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 178, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "cupy/cuda/memory.pyx":169
 *             self.ptr = runtime.mallocAsync(size, stream.ptr)
 * 
 *     def __dealloc__(self):             # <<<<<<<<<<<<<<
 *         # Free on the stream on which this memory was allocated.
 *         # If the stream is already destroyed, free on the current stream. In
*/

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_WriteUnraisable("cupy.cuda.memory.MemoryAsync.__dealloc__", __pyx_clineno, __pyx_lineno, __pyx_filename, 1, 0);
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_stream);
  __Pyx_RefNannyFinishContext();
}

/* "cupy/cuda/memory.pyx":157
 * 
 *     cdef:
 *         readonly object stream_ref             # <<<<<<<<<<<<<<
 * 
 *     def __init__(self, size_t size, stream):
*/

/* Python wrapper */
static PyObject *__pyx_pw_4cupy_4cuda_6memory_11MemoryAsync_10stream_ref_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_4cupy_4cuda_6memory_11MemoryAsync_10stream_ref_1__get__(PyObject *__pyx_v_self) {
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_kwvalues = __Pyx_KwValues_VARARGS(__pyx_args, __pyx_nargs);
  __pyx_r = __pyx_pf_4cupy_4cuda_6memory_11MemoryAsync_10stream_ref___get__(((struct __pyx_obj_4cupy_4cuda_6memory_MemoryAsync *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4cupy_4cuda_6memory_11MemoryAsync_10stream_ref___get__(struct __pyx_obj_4cupy_4cuda_6memory_MemoryAsync *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__", 0);
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_v_self->stream_ref);
  __pyx_r = __pyx_v_self->stream_ref;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     cdef tuple state
 *     cdef object _dict
*/

/* Python wrapper */
static PyObject *__pyx_pw_4cupy_4cuda_6memory_11MemoryAsync_5__reduce_cython__(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
PyDoc_STRVAR(__pyx_doc_4cupy_4cuda_6memory_11MemoryAsync_4__reduce_cython__, "MemoryAsync.__reduce_cython__(self)");
static PyObject *__pyx_pw_4cupy_4cuda_6memory_11MemoryAsync_5__reduce_cython__(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
) {
  #if !CYTHON_METH_FASTCALL
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  #endif
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__reduce_cython__ (wrapper)", 0);
  #if !CYTHON_METH_FASTCALL
  #if CYTHON_ASSUME_SAFE_SIZE
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return NULL;
  #endif
  #endif
  __pyx_kwvalues = __Pyx_KwValues_FASTCALL(__pyx_args, __pyx_nargs);
  if (unlikely(__pyx_nargs > 0)) { __Pyx_RaiseArgtupleInvalid("__reduce_cython__", 1, 0, 0, __pyx_nargs); return NULL; }
  const Py_ssize_t __pyx_kwds_len = unlikely(__pyx_kwds) ? __Pyx_NumKwargs_FASTCALL(__pyx_kwds) : 0;
  if (unlikely(__pyx_kwds_len < 0)) return NULL;
  if (unlikely(__pyx_kwds_len > 0)) {__Pyx_RejectKeywords("__reduce_cython__", __pyx_kwds); return NULL;}
  __pyx_r = __pyx_pf_4cupy_4cuda_6memory_11MemoryAsync_4__reduce_cython__(((struct __pyx_obj_4cupy_4cuda_6memory_MemoryAsync *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4cupy_4cuda_6memory_11MemoryAsync_4__reduce_cython__(struct __pyx_obj_4cupy_4cuda_6memory_MemoryAsync *__pyx_v_self) {
  PyObject *__pyx_v_state = 0;
  PyObject *__pyx_v__dict = 0;
  int __pyx_v_use_setstate;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  int __pyx_t_5;
  int __pyx_t_6;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__reduce_cython__", 0);

  /* "(tree fragment)":5
 *     cdef object _dict
 *     cdef bint use_setstate
 *     state = (self.device_id, self.ptr, self.size, self.stream_ref)             # <<<<<<<<<<<<<<
 *     _dict = getattr(self, '__dict__', None)
 *     if _dict is not None and _dict:
*/
  __pyx_t_1 = __Pyx_PyLong_From_int(__pyx_v_self->__pyx_base.device_id); if (unlikely(!__pyx_t_1)) __PYX_ERR(3, 5, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = PyLong_FromSsize_t(__pyx_v_self->__pyx_base.ptr); if (unlikely(!__pyx_t_2)) __PYX_ERR(3, 5, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = __Pyx_PyLong_FromSize_t(__pyx_v_self->__pyx_base.size); if (unlikely(!__pyx_t_3)) __PYX_ERR(3, 5, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_4 = PyTuple_New(4); if (unlikely(!__pyx_t_4)) __PYX_ERR(3, 5, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_GIVEREF(__pyx_t_1);
  if (__Pyx_PyTuple_SET_ITEM(__pyx_t_4, 0, __pyx_t_1) != (0)) __PYX_ERR(3, 5, __pyx_L1_error);
  __Pyx_GIVEREF(__pyx_t_2);
  if (__Pyx_PyTuple_SET_ITEM(__pyx_t_4, 1, __pyx_t_2) != (0)) __PYX_ERR(3, 5, __pyx_L1_error);
  __Pyx_GIVEREF(__pyx_t_3);
  if (__Pyx_PyTuple_SET_ITEM(__pyx_t_4, 2, __pyx_t_3) != (0)) __PYX_ERR(3, 5, __pyx_L1_error);
  __Pyx_INCREF(__pyx_v_self->stream_ref);
  __Pyx_GIVEREF(__pyx_v_self->stream_ref);
  if (__Pyx_PyTuple_SET_ITEM(__pyx_t_4, 3, __pyx_v_self->stream_ref) != (0)) __PYX_ERR(3, 5, __pyx_L1_error);
  __pyx_t_1 = 0;
  __pyx_t_2 = 0;
  __pyx_t_3 = 0;
  __pyx_v_state = ((PyObject*)__pyx_t_4);
  __pyx_t_4 = 0;

  /* "(tree fragment)":6
 *     cdef bint use_setstate
 *     state = (self.device_id, self.ptr, self.size, self.stream_ref)
 *     _dict = getattr(self, '__dict__', None)             # <<<<<<<<<<<<<<
 *     if _dict is not None and _dict:
 *         state += (_dict,)
*/
  __pyx_t_4 = __Pyx_GetAttr3(((PyObject *)__pyx_v_self), __pyx_mstate_global->__pyx_n_u_dict, Py_None); if (unlikely(!__pyx_t_4)) __PYX_ERR(3, 6, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_v__dict = __pyx_t_4;
  __pyx_t_4 = 0;

  /* "(tree fragment)":7
 *     state = (self.device_id, self.ptr, self.size, self.stream_ref)
 *     _dict = getattr(self, '__dict__', None)
 *     if _dict is not None and _dict:             # <<<<<<<<<<<<<<
 *         state += (_dict,)
 *         use_setstate = True
*/
  __pyx_t_6 = (__pyx_v__dict != Py_None);
  if (__pyx_t_6) {
  } else {
    __pyx_t_5 = __pyx_t_6;
    goto __pyx_L4_bool_binop_done;
  }
  __pyx_t_6 = __Pyx_PyObject_IsTrue(__pyx_v__dict); if (unlikely((__pyx_t_6 < 0))) __PYX_ERR(3, 7, __pyx_L1_error)
  __pyx_t_5 = __pyx_t_6;
  __pyx_L4_bool_binop_done:;
  if (__pyx_t_5) {

    /* "(tree fragment)":8
 *     _dict = getattr(self, '__dict__', None)
 *     if _dict is not None and _dict:
 *         state += (_dict,)             # <<<<<<<<<<<<<<
 *         use_setstate = True
 *     else:
*/
    __pyx_t_4 = PyTuple_New(1); if (unlikely(!__pyx_t_4)) __PYX_ERR(3, 8, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_INCREF(__pyx_v__dict);
    __Pyx_GIVEREF(__pyx_v__dict);
    if (__Pyx_PyTuple_SET_ITEM(__pyx_t_4, 0, __pyx_v__dict) != (0)) __PYX_ERR(3, 8, __pyx_L1_error);
    __pyx_t_3 = PyNumber_InPlaceAdd(__pyx_v_state, __pyx_t_4); if (unlikely(!__pyx_t_3)) __PYX_ERR(3, 8, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __Pyx_DECREF_SET(__pyx_v_state, ((PyObject*)__pyx_t_3));
    __pyx_t_3 = 0;

    /* "(tree fragment)":9
 *     if _dict is not None and _dict:
 *         state += (_dict,)
 *         use_setstate = True             # <<<<<<<<<<<<<<
 *     else:
 *         use_setstate = self.stream_ref is not None
*/
    __pyx_v_use_setstate = 1;

    /* "(tree fragment)":7
 *     state = (self.device_id, self.ptr, self.size, self.stream_ref)
 *     _dict = getattr(self, '__dict__', None)
 *     if _dict is not None and _dict:             # <<<<<<<<<<<<<<
 *         state += (_dict,)
 *         use_setstate = True
*/
    goto __pyx_L3;
  }

  /* "(tree fragment)":11
 *         use_setstate = True
 *     else:
 *         use_setstate = self.stream_ref is not None             # <<<<<<<<<<<<<<
 *     if use_setstate:
 *         return __pyx_unpickle_MemoryAsync, (type(self), 0x0e18a1e, None), state
*/
  /*else*/ {
    __pyx_t_5 = (__pyx_v_self->stream_ref != Py_None);
    __pyx_v_use_setstate = __pyx_t_5;
  }
  __pyx_L3:;

  /* "(tree fragment)":12
 *     else:
 *         use_setstate = self.stream_ref is not None
 *     if use_setstate:             # <<<<<<<<<<<<<<
 *         return __pyx_unpickle_MemoryAsync, (type(self), 0x0e18a1e, None), state
 *     else:
*/
  if (__pyx_v_use_setstate) {

    /* "(tree fragment)":13
 *         use_setstate = self.stream_ref is not None
 *     if use_setstate:
 *         return __pyx_unpickle_MemoryAsync, (type(self), 0x0e18a1e, None), state             # <<<<<<<<<<<<<<
 *     else:
 *         return __pyx_unpickle_MemoryAsync, (type(self), 0x0e18a1e, state)
*/
    __Pyx_XDECREF(__pyx_r);
    __Pyx_GetModuleGlobalName(__pyx_t_3, __pyx_mstate_global->__pyx_n_u_pyx_unpickle_MemoryAsync); if (unlikely(!__pyx_t_3)) __PYX_ERR(3, 13, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_t_4 = PyTuple_New(3); if (unlikely(!__pyx_t_4)) __PYX_ERR(3, 13, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_INCREF(((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    __Pyx_GIVEREF(((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    if (__Pyx_PyTuple_SET_ITEM(__pyx_t_4, 0, ((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self)))) != (0)) __PYX_ERR(3, 13, __pyx_L1_error);
    __Pyx_INCREF(__pyx_mstate_global->__pyx_int_14780958);
    __Pyx_GIVEREF(__pyx_mstate_global->__pyx_int_14780958);
    if (__Pyx_PyTuple_SET_ITEM(__pyx_t_4, 1, __pyx_mstate_global->__pyx_int_14780958) != (0)) __PYX_ERR(3, 13, __pyx_L1_error);
    __Pyx_INCREF(Py_None);
    __Pyx_GIVEREF(Py_None);
    if (__Pyx_PyTuple_SET_ITEM(__pyx_t_4, 2, Py_None) != (0)) __PYX_ERR(3, 13, __pyx_L1_error);
    __pyx_t_2 = PyTuple_New(3); if (unlikely(!__pyx_t_2)) __PYX_ERR(3, 13, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_GIVEREF(__pyx_t_3);
    if (__Pyx_PyTuple_SET_ITEM(__pyx_t_2, 0, __pyx_t_3) != (0)) __PYX_ERR(3, 13, __pyx_L1_error);
    __Pyx_GIVEREF(__pyx_t_4);
    if (__Pyx_PyTuple_SET_ITEM(__pyx_t_2, 1, __pyx_t_4) != (0)) __PYX_ERR(3, 13, __pyx_L1_error);
    __Pyx_INCREF(__pyx_v_state);
    __Pyx_GIVEREF(__pyx_v_state);
    if (__Pyx_PyTuple_SET_ITEM(__pyx_t_2, 2, __pyx_v_state) != (0)) __PYX_ERR(3, 13, __pyx_L1_error);
    __pyx_t_3 = 0;
    __pyx_t_4 = 0;
    __pyx_r = __pyx_t_2;
    __pyx_t_2 = 0;
    goto __pyx_L0;

    /* "(tree fragment)":12
 *     else:
 *         use_setstate = self.stream_ref is not None
 *     if use_setstate:             # <<<<<<<<<<<<<<
 *         return __pyx_unpickle_MemoryAsync, (type(self), 0x0e18a1e, None), state
 *     else:
*/
  }

  /* "(tree fragment)":15
 *         return __pyx_unpickle_MemoryAsync, (type(self), 0x0e18a1e, None), state
 *     else:
 *         return __pyx_unpickle_MemoryAsync, (type(self), 0x0e18a1e, state)             # <<<<<<<<<<<<<<
 * def __setstate_cython__(self, __pyx_state):
 *     __pyx_unpickle_MemoryAsync__set_state(self, __pyx_state)
*/
  /*else*/ {
    __Pyx_XDECREF(__pyx_r);
    __Pyx_GetModuleGlobalName(__pyx_t_2, __pyx_mstate_global->__pyx_n_u_pyx_unpickle_MemoryAsync); if (unlikely(!__pyx_t_2)) __PYX_ERR(3, 15, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_4 = PyTuple_New(3); if (unlikely(!__pyx_t_4)) __PYX_ERR(3, 15, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_INCREF(((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    __Pyx_GIVEREF(((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    if (__Pyx_PyTuple_SET_ITEM(__pyx_t_4, 0, ((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self)))) != (0)) __PYX_ERR(3, 15, __pyx_L1_error);
    __Pyx_INCREF(__pyx_mstate_global->__pyx_int_14780958);
    __Pyx_GIVEREF(__pyx_mstate_global->__pyx_int_14780958);
    if (__Pyx_PyTuple_SET_ITEM(__pyx_t_4, 1, __pyx_mstate_global->__pyx_int_14780958) != (0)) __PYX_ERR(3, 15, __pyx_L1_error);
    __Pyx_INCREF(__pyx_v_state);
    __Pyx_GIVEREF(__pyx_v_state);
    if (__Pyx_PyTuple_SET_ITEM(__pyx_t_4, 2, __pyx_v_state) != (0)) __PYX_ERR(3, 15, __pyx_L1_error);
    __pyx_t_3 = PyTuple_New(2); if (unlikely(!__pyx_t_3)) __PYX_ERR(3, 15, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_GIVEREF(__pyx_t_2);
    if (__Pyx_PyTuple_SET_ITEM(__pyx_t_3, 0, __pyx_t_2) != (0)) __PYX_ERR(3, 15, __pyx_L1_error);
    __Pyx_GIVEREF(__pyx_t_4);
    if (__Pyx_PyTuple_SET_ITEM(__pyx_t_3, 1, __pyx_t_4) != (0)) __PYX_ERR(3, 15, __pyx_L1_error);
    __pyx_t_2 = 0;
    __pyx_t_4 = 0;
    __pyx_r = __pyx_t_3;
    __pyx_t_3 = 0;
    goto __pyx_L0;
  }

  /* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     cdef tuple state
 *     cdef object _dict
*/

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_AddTraceback("cupy.cuda.memory.MemoryAsync.__reduce_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_state);
  __Pyx_XDECREF(__pyx_v__dict);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":16
 *     else:
 *         return __pyx_unpickle_MemoryAsync, (type(self), 0x0e18a1e, state)
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     __pyx_unpickle_MemoryAsync__set_state(self, __pyx_state)
*/

/* Python wrapper */
static PyObject *__pyx_pw_4cupy_4cuda_6memory_11MemoryAsync_7__setstate_cython__(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
PyDoc_STRVAR(__pyx_doc_4cupy_4cuda_6memory_11MemoryAsync_6__setstate_cython__, "MemoryAsync.__setstate_cython__(self, __pyx_state)");
static PyObject *__pyx_pw_4cupy_4cuda_6memory_11MemoryAsync_7__setstate_cython__(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
) {
  PyObject *__pyx_v___pyx_state = 0;
  #if !CYTHON_METH_FASTCALL
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  #endif
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject* values[1] = {0};
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__setstate_cython__ (wrapper)", 0);
  #if !CYTHON_METH_FASTCALL
  #if CYTHON_ASSUME_SAFE_SIZE
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return NULL;
  #endif
  #endif
  __pyx_kwvalues = __Pyx_KwValues_FASTCALL(__pyx_args, __pyx_nargs);
  {
    PyObject ** const __pyx_pyargnames[] = {&__pyx_mstate_global->__pyx_n_u_pyx_state,0};
    const Py_ssize_t __pyx_kwds_len = (__pyx_kwds) ? __Pyx_NumKwargs_FASTCALL(__pyx_kwds) : 0;
    if (unlikely(__pyx_kwds_len < 0)) __PYX_ERR(3, 16, __pyx_L3_error)
    if (__pyx_kwds_len > 0) {
      switch (__pyx_nargs) {
        case  1:
        values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(3, 16, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      const Py_ssize_t kwd_pos_args = __pyx_nargs;
      if (__Pyx_ParseKeywords(__pyx_kwds, __pyx_kwvalues, __pyx_pyargnames, 0, values, kwd_pos_args, __pyx_kwds_len, "__setstate_cython__", 0) < (0)) __PYX_ERR(3, 16, __pyx_L3_error)
      for (Py_ssize_t i = __pyx_nargs; i < 1; i++) {
        if (unlikely(!values[i])) { __Pyx_RaiseArgtupleInvalid("__setstate_cython__", 1, 1, 1, i); __PYX_ERR(3, 16, __pyx_L3_error) }
      }
    } else if (unlikely(__pyx_nargs != 1)) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
      if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(3, 16, __pyx_L3_error)
    }
    __pyx_v___pyx_state = values[0];
  }
  goto __pyx_L6_skip;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("__setstate_cython__", 1, 1, 1, __pyx_nargs); __PYX_ERR(3, 16, __pyx_L3_error)
  __pyx_L6_skip:;
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_AddTraceback("cupy.cuda.memory.MemoryAsync.__setstate_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_4cupy_4cuda_6memory_11MemoryAsync_6__setstate_cython__(((struct __pyx_obj_4cupy_4cuda_6memory_MemoryAsync *)__pyx_v_self), __pyx_v___pyx_state);

  /* function exit code */
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4cupy_4cuda_6memory_11MemoryAsync_6__setstate_cython__(struct __pyx_obj_4cupy_4cuda_6memory_MemoryAsync *__pyx_v_self, PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__setstate_cython__", 0);

  /* "(tree fragment)":17
 *         return __pyx_unpickle_MemoryAsync, (type(self), 0x0e18a1e, state)
 * def __setstate_cython__(self, __pyx_state):
 *     __pyx_unpickle_MemoryAsync__set_state(self, __pyx_state)             # <<<<<<<<<<<<<<
*/
  __pyx_t_1 = __pyx_v___pyx_state;
  __Pyx_INCREF(__pyx_t_1);
  if (!(likely(PyTuple_CheckExact(__pyx_t_1))||((__pyx_t_1) == Py_None) || __Pyx_RaiseUnexpectedTypeError("tuple", __pyx_t_1))) __PYX_ERR(3, 17, __pyx_L1_error)
  if (unlikely(__pyx_t_1 == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "cannot pass None into a C function argument that is declared 'not None'");
    __PYX_ERR(3, 17, __pyx_L1_error)
  }
  __pyx_t_2 = __pyx_f_4cupy_4cuda_6memory___pyx_unpickle_MemoryAsync__set_state(__pyx_v_self, ((PyObject*)__pyx_t_1)); if (unlikely(!__pyx_t_2)) __PYX_ERR(3, 17, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "(tree fragment)":16
 *     else:
 *         return __pyx_unpickle_MemoryAsync, (type(self), 0x0e18a1e, state)
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     __pyx_unpickle_MemoryAsync__set_state(self, __pyx_state)
*/

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_AddTraceback("cupy.cuda.memory.MemoryAsync.__setstate_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "cupy/cuda/memory.pyx":196
 *         readonly object _owner
 * 
 *     def __init__(self, intptr_t ptr, size_t size, object owner,             # <<<<<<<<<<<<<<
 *                  int device_id=-1):
 *         cdef runtime.PointerAttributes ptr_attrs
*/

/* Python wrapper */
static int __pyx_pw_4cupy_4cuda_6memory_13UnownedMemory_1__init__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static int __pyx_pw_4cupy_4cuda_6memory_13UnownedMemory_1__init__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  intptr_t __pyx_v_ptr;
  size_t __pyx_v_size;
  PyObject *__pyx_v_owner = 0;
  int __pyx_v_device_id;
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject* values[4] = {0,0,0,0};
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__init__ (wrapper)", 0);
  #if CYTHON_ASSUME_SAFE_SIZE
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return -1;
  #endif
  __pyx_kwvalues = __Pyx_KwValues_VARARGS(__pyx_args, __pyx_nargs);
  {
    PyObject ** const __pyx_pyargnames[] = {&__pyx_mstate_global->__pyx_n_u_ptr,&__pyx_mstate_global->__pyx_n_u_size,&__pyx_mstate_global->__pyx_n_u_owner,&__pyx_mstate_global->__pyx_n_u_device_id,0};
    const Py_ssize_t __pyx_kwds_len = (__pyx_kwds) ? __Pyx_NumKwargs_VARARGS(__pyx_kwds) : 0;
    if (unlikely(__pyx_kwds_len < 0)) __PYX_ERR(0, 196, __pyx_L3_error)
    if (__pyx_kwds_len > 0) {
      switch (__pyx_nargs) {
        case  4:
        values[3] = __Pyx_ArgRef_VARARGS(__pyx_args, 3);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[3])) __PYX_ERR(0, 196, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  3:
        values[2] = __Pyx_ArgRef_VARARGS(__pyx_args, 2);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[2])) __PYX_ERR(0, 196, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  2:
        values[1] = __Pyx_ArgRef_VARARGS(__pyx_args, 1);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[1])) __PYX_ERR(0, 196, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  1:
        values[0] = __Pyx_ArgRef_VARARGS(__pyx_args, 0);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(0, 196, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      const Py_ssize_t kwd_pos_args = __pyx_nargs;
      if (__Pyx_ParseKeywords(__pyx_kwds, __pyx_kwvalues, __pyx_pyargnames, 0, values, kwd_pos_args, __pyx_kwds_len, "__init__", 0) < (0)) __PYX_ERR(0, 196, __pyx_L3_error)
      for (Py_ssize_t i = __pyx_nargs; i < 3; i++) {
        if (unlikely(!values[i])) { __Pyx_RaiseArgtupleInvalid("__init__", 0, 3, 4, i); __PYX_ERR(0, 196, __pyx_L3_error) }
      }
    } else {
      switch (__pyx_nargs) {
        case  4:
        values[3] = __Pyx_ArgRef_VARARGS(__pyx_args, 3);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[3])) __PYX_ERR(0, 196, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  3:
        values[2] = __Pyx_ArgRef_VARARGS(__pyx_args, 2);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[2])) __PYX_ERR(0, 196, __pyx_L3_error)
        values[1] = __Pyx_ArgRef_VARARGS(__pyx_args, 1);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[1])) __PYX_ERR(0, 196, __pyx_L3_error)
        values[0] = __Pyx_ArgRef_VARARGS(__pyx_args, 0);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(0, 196, __pyx_L3_error)
        break;
        default: goto __pyx_L5_argtuple_error;
      }
    }
    __pyx_v_ptr = PyLong_AsSsize_t(values[0]); if (unlikely((__pyx_v_ptr == ((intptr_t)-1)) && PyErr_Occurred())) __PYX_ERR(0, 196, __pyx_L3_error)
    __pyx_v_size = __Pyx_PyLong_As_size_t(values[1]); if (unlikely((__pyx_v_size == (size_t)-1) && PyErr_Occurred())) __PYX_ERR(0, 196, __pyx_L3_error)
    __pyx_v_owner = values[2];
    if (values[3]) {
      __pyx_v_device_id = __Pyx_PyLong_As_int(values[3]); if (unlikely((__pyx_v_device_id == (int)-1) && PyErr_Occurred())) __PYX_ERR(0, 197, __pyx_L3_error)
    } else {
      __pyx_v_device_id = ((int)-1);
    }
  }
  goto __pyx_L6_skip;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("__init__", 0, 3, 4, __pyx_nargs); __PYX_ERR(0, 196, __pyx_L3_error)
  __pyx_L6_skip:;
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_AddTraceback("cupy.cuda.memory.UnownedMemory.__init__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return -1;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_4cupy_4cuda_6memory_13UnownedMemory___init__(((struct __pyx_obj_4cupy_4cuda_6memory_UnownedMemory *)__pyx_v_self), __pyx_v_ptr, __pyx_v_size, __pyx_v_owner, __pyx_v_device_id);

  /* function exit code */
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_4cupy_4cuda_6memory_13UnownedMemory___init__(struct __pyx_obj_4cupy_4cuda_6memory_UnownedMemory *__pyx_v_self, intptr_t __pyx_v_ptr, size_t __pyx_v_size, PyObject *__pyx_v_owner, int __pyx_v_device_id) {
  struct __pyx_obj_13cupy_backends_4cuda_3api_7runtime_PointerAttributes *__pyx_v_ptr_attrs = 0;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  size_t __pyx_t_4;
  int __pyx_t_5;
  int __pyx_t_6;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__init__", 0);

  /* "cupy/cuda/memory.pyx":201
 *         # ptr=0 for 0-size arrays from __cuda_array_interface__ v2:
 *         # we need a valid device id as null ptr can't be looked up
 *         if device_id < 0:             # <<<<<<<<<<<<<<
 *             if ptr == 0:
 *                 raise RuntimeError('UnownedMemory requires explicit'
*/
  __pyx_t_1 = (__pyx_v_device_id < 0);
  if (__pyx_t_1) {

    /* "cupy/cuda/memory.pyx":202
 *         # we need a valid device id as null ptr can't be looked up
 *         if device_id < 0:
 *             if ptr == 0:             # <<<<<<<<<<<<<<
 *                 raise RuntimeError('UnownedMemory requires explicit'
 *                                    ' device ID for a null pointer.')
*/
    __pyx_t_1 = (__pyx_v_ptr == 0);
    if (unlikely(__pyx_t_1)) {

      /* "cupy/cuda/memory.pyx":203
 *         if device_id < 0:
 *             if ptr == 0:
 *                 raise RuntimeError('UnownedMemory requires explicit'             # <<<<<<<<<<<<<<
 *                                    ' device ID for a null pointer.')
 *             # Initialize a context to workaround a bug in CUDA 10.2+. (#3991)
*/
      __pyx_t_3 = NULL;
      __pyx_t_4 = 1;
      {
        PyObject *__pyx_callargs[2] = {__pyx_t_3, __pyx_mstate_global->__pyx_kp_u_UnownedMemory_requires_explicit};
        __pyx_t_2 = __Pyx_PyObject_FastCall((PyObject*)(((PyTypeObject*)PyExc_RuntimeError)), __pyx_callargs+__pyx_t_4, (2-__pyx_t_4) | (__pyx_t_4*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
        __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
        if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 203, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_2);
      }
      __Pyx_Raise(__pyx_t_2, 0, 0, 0);
      __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
      __PYX_ERR(0, 203, __pyx_L1_error)

      /* "cupy/cuda/memory.pyx":202
 *         # we need a valid device id as null ptr can't be looked up
 *         if device_id < 0:
 *             if ptr == 0:             # <<<<<<<<<<<<<<
 *                 raise RuntimeError('UnownedMemory requires explicit'
 *                                    ' device ID for a null pointer.')
*/
    }

    /* "cupy/cuda/memory.pyx":206
 *                                    ' device ID for a null pointer.')
 *             # Initialize a context to workaround a bug in CUDA 10.2+. (#3991)
 *             runtime._ensure_context()             # <<<<<<<<<<<<<<
 *             ptr_attrs = runtime.pointerGetAttributes(ptr)
 *             device_id = ptr_attrs.device
*/
    __pyx_t_2 = __pyx_f_13cupy_backends_4cuda_3api_7runtime__ensure_context(); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 206, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

    /* "cupy/cuda/memory.pyx":207
 *             # Initialize a context to workaround a bug in CUDA 10.2+. (#3991)
 *             runtime._ensure_context()
 *             ptr_attrs = runtime.pointerGetAttributes(ptr)             # <<<<<<<<<<<<<<
 *             device_id = ptr_attrs.device
 *             if device_id == runtime.cudaCpuDeviceId:
*/
    __pyx_t_2 = ((PyObject *)__pyx_f_13cupy_backends_4cuda_3api_7runtime_pointerGetAttributes(__pyx_v_ptr, 0)); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 207, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_v_ptr_attrs = ((struct __pyx_obj_13cupy_backends_4cuda_3api_7runtime_PointerAttributes *)__pyx_t_2);
    __pyx_t_2 = 0;

    /* "cupy/cuda/memory.pyx":208
 *             runtime._ensure_context()
 *             ptr_attrs = runtime.pointerGetAttributes(ptr)
 *             device_id = ptr_attrs.device             # <<<<<<<<<<<<<<
 *             if device_id == runtime.cudaCpuDeviceId:
 *                 # this happens with SystemMemory...
*/
    __pyx_t_5 = __pyx_v_ptr_attrs->device;
    __pyx_v_device_id = __pyx_t_5;

    /* "cupy/cuda/memory.pyx":209
 *             ptr_attrs = runtime.pointerGetAttributes(ptr)
 *             device_id = ptr_attrs.device
 *             if device_id == runtime.cudaCpuDeviceId:             # <<<<<<<<<<<<<<
 *                 # this happens with SystemMemory...
 *                 device_id = device.get_device_id()
*/
    __pyx_t_1 = (__pyx_v_device_id == __pyx_e_13cupy_backends_4cuda_3api_13_runtime_enum_cudaCpuDeviceId);
    if (__pyx_t_1) {

      /* "cupy/cuda/memory.pyx":211
 *             if device_id == runtime.cudaCpuDeviceId:
 *                 # this happens with SystemMemory...
 *                 device_id = device.get_device_id()             # <<<<<<<<<<<<<<
 *             # CUDA doesn't track memory allocated through the system malloc
 *             if device_id == runtime.cudaInvalidDeviceId and _is_ump_enabled:
*/
      __pyx_t_5 = __pyx_f_4cupy_4cuda_6device_get_device_id(0); if (unlikely(__pyx_t_5 == ((int)-1) && PyErr_Occurred())) __PYX_ERR(0, 211, __pyx_L1_error)
      __pyx_v_device_id = __pyx_t_5;

      /* "cupy/cuda/memory.pyx":209
 *             ptr_attrs = runtime.pointerGetAttributes(ptr)
 *             device_id = ptr_attrs.device
 *             if device_id == runtime.cudaCpuDeviceId:             # <<<<<<<<<<<<<<
 *                 # this happens with SystemMemory...
 *                 device_id = device.get_device_id()
*/
    }

    /* "cupy/cuda/memory.pyx":213
 *                 device_id = device.get_device_id()
 *             # CUDA doesn't track memory allocated through the system malloc
 *             if device_id == runtime.cudaInvalidDeviceId and _is_ump_enabled:             # <<<<<<<<<<<<<<
 *                 device_id = device.get_device_id()
 * 
*/
    __pyx_t_6 = (__pyx_v_device_id == __pyx_e_13cupy_backends_4cuda_3api_13_runtime_enum_cudaInvalidDeviceId);
    if (__pyx_t_6) {
    } else {
      __pyx_t_1 = __pyx_t_6;
      goto __pyx_L7_bool_binop_done;
    }
    __pyx_t_1 = __pyx_v_4cupy_4cuda_6memory__is_ump_enabled;
    __pyx_L7_bool_binop_done:;
    if (__pyx_t_1) {

      /* "cupy/cuda/memory.pyx":214
 *             # CUDA doesn't track memory allocated through the system malloc
 *             if device_id == runtime.cudaInvalidDeviceId and _is_ump_enabled:
 *                 device_id = device.get_device_id()             # <<<<<<<<<<<<<<
 * 
 *         self.size = size
*/
      __pyx_t_5 = __pyx_f_4cupy_4cuda_6device_get_device_id(0); if (unlikely(__pyx_t_5 == ((int)-1) && PyErr_Occurred())) __PYX_ERR(0, 214, __pyx_L1_error)
      __pyx_v_device_id = __pyx_t_5;

      /* "cupy/cuda/memory.pyx":213
 *                 device_id = device.get_device_id()
 *             # CUDA doesn't track memory allocated through the system malloc
 *             if device_id == runtime.cudaInvalidDeviceId and _is_ump_enabled:             # <<<<<<<<<<<<<<
 *                 device_id = device.get_device_id()
 * 
*/
    }

    /* "cupy/cuda/memory.pyx":201
 *         # ptr=0 for 0-size arrays from __cuda_array_interface__ v2:
 *         # we need a valid device id as null ptr can't be looked up
 *         if device_id < 0:             # <<<<<<<<<<<<<<
 *             if ptr == 0:
 *                 raise RuntimeError('UnownedMemory requires explicit'
*/
  }

  /* "cupy/cuda/memory.pyx":216
 *                 device_id = device.get_device_id()
 * 
 *         self.size = size             # <<<<<<<<<<<<<<
 *         self.device_id = device_id
 *         self.ptr = ptr
*/
  __pyx_v_self->__pyx_base.size = __pyx_v_size;

  /* "cupy/cuda/memory.pyx":217
 * 
 *         self.size = size
 *         self.device_id = device_id             # <<<<<<<<<<<<<<
 *         self.ptr = ptr
 *         self._owner = owner
*/
  __pyx_v_self->__pyx_base.device_id = __pyx_v_device_id;

  /* "cupy/cuda/memory.pyx":218
 *         self.size = size
 *         self.device_id = device_id
 *         self.ptr = ptr             # <<<<<<<<<<<<<<
 *         self._owner = owner
 * 
*/
  __pyx_v_self->__pyx_base.ptr = __pyx_v_ptr;

  /* "cupy/cuda/memory.pyx":219
 *         self.device_id = device_id
 *         self.ptr = ptr
 *         self._owner = owner             # <<<<<<<<<<<<<<
 * 
 * 
*/
  __Pyx_INCREF(__pyx_v_owner);
  __Pyx_GIVEREF(__pyx_v_owner);
  __Pyx_GOTREF(__pyx_v_self->_owner);
  __Pyx_DECREF(__pyx_v_self->_owner);
  __pyx_v_self->_owner = __pyx_v_owner;

  /* "cupy/cuda/memory.pyx":196
 *         readonly object _owner
 * 
 *     def __init__(self, intptr_t ptr, size_t size, object owner,             # <<<<<<<<<<<<<<
 *                  int device_id=-1):
 *         cdef runtime.PointerAttributes ptr_attrs
*/

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_AddTraceback("cupy.cuda.memory.UnownedMemory.__init__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_XDECREF((PyObject *)__pyx_v_ptr_attrs);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "cupy/cuda/memory.pyx":194
 * 
 *     cdef:
 *         readonly object _owner             # <<<<<<<<<<<<<<
 * 
 *     def __init__(self, intptr_t ptr, size_t size, object owner,
*/

/* Python wrapper */
static PyObject *__pyx_pw_4cupy_4cuda_6memory_13UnownedMemory_6_owner_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_4cupy_4cuda_6memory_13UnownedMemory_6_owner_1__get__(PyObject *__pyx_v_self) {
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_kwvalues = __Pyx_KwValues_VARARGS(__pyx_args, __pyx_nargs);
  __pyx_r = __pyx_pf_4cupy_4cuda_6memory_13UnownedMemory_6_owner___get__(((struct __pyx_obj_4cupy_4cuda_6memory_UnownedMemory *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4cupy_4cuda_6memory_13UnownedMemory_6_owner___get__(struct __pyx_obj_4cupy_4cuda_6memory_UnownedMemory *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__", 0);
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_v_self->_owner);
  __pyx_r = __pyx_v_self->_owner;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     cdef tuple state
 *     cdef object _dict
*/

/* Python wrapper */
static PyObject *__pyx_pw_4cupy_4cuda_6memory_13UnownedMemory_3__reduce_cython__(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
PyDoc_STRVAR(__pyx_doc_4cupy_4cuda_6memory_13UnownedMemory_2__reduce_cython__, "UnownedMemory.__reduce_cython__(self)");
static PyObject *__pyx_pw_4cupy_4cuda_6memory_13UnownedMemory_3__reduce_cython__(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
) {
  #if !CYTHON_METH_FASTCALL
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  #endif
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__reduce_cython__ (wrapper)", 0);
  #if !CYTHON_METH_FASTCALL
  #if CYTHON_ASSUME_SAFE_SIZE
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return NULL;
  #endif
  #endif
  __pyx_kwvalues = __Pyx_KwValues_FASTCALL(__pyx_args, __pyx_nargs);
  if (unlikely(__pyx_nargs > 0)) { __Pyx_RaiseArgtupleInvalid("__reduce_cython__", 1, 0, 0, __pyx_nargs); return NULL; }
  const Py_ssize_t __pyx_kwds_len = unlikely(__pyx_kwds) ? __Pyx_NumKwargs_FASTCALL(__pyx_kwds) : 0;
  if (unlikely(__pyx_kwds_len < 0)) return NULL;
  if (unlikely(__pyx_kwds_len > 0)) {__Pyx_RejectKeywords("__reduce_cython__", __pyx_kwds); return NULL;}
  __pyx_r = __pyx_pf_4cupy_4cuda_6memory_13UnownedMemory_2__reduce_cython__(((struct __pyx_obj_4cupy_4cuda_6memory_UnownedMemory *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4cupy_4cuda_6memory_13UnownedMemory_2__reduce_cython__(struct __pyx_obj_4cupy_4cuda_6memory_UnownedMemory *__pyx_v_self) {
  PyObject *__pyx_v_state = 0;
  PyObject *__pyx_v__dict = 0;
  int __pyx_v_use_setstate;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  int __pyx_t_5;
  int __pyx_t_6;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__reduce_cython__", 0);

  /* "(tree fragment)":5
 *     cdef object _dict
 *     cdef bint use_setstate
 *     state = (self._owner, self.device_id, self.ptr, self.size)             # <<<<<<<<<<<<<<
 *     _dict = getattr(self, '__dict__', None)
 *     if _dict is not None and _dict:
*/
  __pyx_t_1 = __Pyx_PyLong_From_int(__pyx_v_self->__pyx_base.device_id); if (unlikely(!__pyx_t_1)) __PYX_ERR(3, 5, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = PyLong_FromSsize_t(__pyx_v_self->__pyx_base.ptr); if (unlikely(!__pyx_t_2)) __PYX_ERR(3, 5, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = __Pyx_PyLong_FromSize_t(__pyx_v_self->__pyx_base.size); if (unlikely(!__pyx_t_3)) __PYX_ERR(3, 5, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_4 = PyTuple_New(4); if (unlikely(!__pyx_t_4)) __PYX_ERR(3, 5, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_INCREF(__pyx_v_self->_owner);
  __Pyx_GIVEREF(__pyx_v_self->_owner);
  if (__Pyx_PyTuple_SET_ITEM(__pyx_t_4, 0, __pyx_v_self->_owner) != (0)) __PYX_ERR(3, 5, __pyx_L1_error);
  __Pyx_GIVEREF(__pyx_t_1);
  if (__Pyx_PyTuple_SET_ITEM(__pyx_t_4, 1, __pyx_t_1) != (0)) __PYX_ERR(3, 5, __pyx_L1_error);
  __Pyx_GIVEREF(__pyx_t_2);
  if (__Pyx_PyTuple_SET_ITEM(__pyx_t_4, 2, __pyx_t_2) != (0)) __PYX_ERR(3, 5, __pyx_L1_error);
  __Pyx_GIVEREF(__pyx_t_3);
  if (__Pyx_PyTuple_SET_ITEM(__pyx_t_4, 3, __pyx_t_3) != (0)) __PYX_ERR(3, 5, __pyx_L1_error);
  __pyx_t_1 = 0;
  __pyx_t_2 = 0;
  __pyx_t_3 = 0;
  __pyx_v_state = ((PyObject*)__pyx_t_4);
  __pyx_t_4 = 0;

  /* "(tree fragment)":6
 *     cdef bint use_setstate
 *     state = (self._owner, self.device_id, self.ptr, self.size)
 *     _dict = getattr(self, '__dict__', None)             # <<<<<<<<<<<<<<
 *     if _dict is not None and _dict:
 *         state += (_dict,)
*/
  __pyx_t_4 = __Pyx_GetAttr3(((PyObject *)__pyx_v_self), __pyx_mstate_global->__pyx_n_u_dict, Py_None); if (unlikely(!__pyx_t_4)) __PYX_ERR(3, 6, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_v__dict = __pyx_t_4;
  __pyx_t_4 = 0;

  /* "(tree fragment)":7
 *     state = (self._owner, self.device_id, self.ptr, self.size)
 *     _dict = getattr(self, '__dict__', None)
 *     if _dict is not None and _dict:             # <<<<<<<<<<<<<<
 *         state += (_dict,)
 *         use_setstate = True
*/
  __pyx_t_6 = (__pyx_v__dict != Py_None);
  if (__pyx_t_6) {
  } else {
    __pyx_t_5 = __pyx_t_6;
    goto __pyx_L4_bool_binop_done;
  }
  __pyx_t_6 = __Pyx_PyObject_IsTrue(__pyx_v__dict); if (unlikely((__pyx_t_6 < 0))) __PYX_ERR(3, 7, __pyx_L1_error)
  __pyx_t_5 = __pyx_t_6;
  __pyx_L4_bool_binop_done:;
  if (__pyx_t_5) {

    /* "(tree fragment)":8
 *     _dict = getattr(self, '__dict__', None)
 *     if _dict is not None and _dict:
 *         state += (_dict,)             # <<<<<<<<<<<<<<
 *         use_setstate = True
 *     else:
*/
    __pyx_t_4 = PyTuple_New(1); if (unlikely(!__pyx_t_4)) __PYX_ERR(3, 8, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_INCREF(__pyx_v__dict);
    __Pyx_GIVEREF(__pyx_v__dict);
    if (__Pyx_PyTuple_SET_ITEM(__pyx_t_4, 0, __pyx_v__dict) != (0)) __PYX_ERR(3, 8, __pyx_L1_error);
    __pyx_t_3 = PyNumber_InPlaceAdd(__pyx_v_state, __pyx_t_4); if (unlikely(!__pyx_t_3)) __PYX_ERR(3, 8, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __Pyx_DECREF_SET(__pyx_v_state, ((PyObject*)__pyx_t_3));
    __pyx_t_3 = 0;

    /* "(tree fragment)":9
 *     if _dict is not None and _dict:
 *         state += (_dict,)
 *         use_setstate = True             # <<<<<<<<<<<<<<
 *     else:
 *         use_setstate = self._owner is not None
*/
    __pyx_v_use_setstate = 1;

    /* "(tree fragment)":7
 *     state = (self._owner, self.device_id, self.ptr, self.size)
 *     _dict = getattr(self, '__dict__', None)
 *     if _dict is not None and _dict:             # <<<<<<<<<<<<<<
 *         state += (_dict,)
 *         use_setstate = True
*/
    goto __pyx_L3;
  }

  /* "(tree fragment)":11
 *         use_setstate = True
 *     else:
 *         use_setstate = self._owner is not None             # <<<<<<<<<<<<<<
 *     if use_setstate:
 *         return __pyx_unpickle_UnownedMemory, (type(self), 0xa9b59d9, None), state
*/
  /*else*/ {
    __pyx_t_5 = (__pyx_v_self->_owner != Py_None);
    __pyx_v_use_setstate = __pyx_t_5;
  }
  __pyx_L3:;

  /* "(tree fragment)":12
 *     else:
 *         use_setstate = self._owner is not None
 *     if use_setstate:             # <<<<<<<<<<<<<<
 *         return __pyx_unpickle_UnownedMemory, (type(self), 0xa9b59d9, None), state
 *     else:
*/
  if (__pyx_v_use_setstate) {

    /* "(tree fragment)":13
 *         use_setstate = self._owner is not None
 *     if use_setstate:
 *         return __pyx_unpickle_UnownedMemory, (type(self), 0xa9b59d9, None), state             # <<<<<<<<<<<<<<
 *     else:
 *         return __pyx_unpickle_UnownedMemory, (type(self), 0xa9b59d9, state)
*/
    __Pyx_XDECREF(__pyx_r);
    __Pyx_GetModuleGlobalName(__pyx_t_3, __pyx_mstate_global->__pyx_n_u_pyx_unpickle_UnownedMemory); if (unlikely(!__pyx_t_3)) __PYX_ERR(3, 13, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_t_4 = PyTuple_New(3); if (unlikely(!__pyx_t_4)) __PYX_ERR(3, 13, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_INCREF(((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    __Pyx_GIVEREF(((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    if (__Pyx_PyTuple_SET_ITEM(__pyx_t_4, 0, ((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self)))) != (0)) __PYX_ERR(3, 13, __pyx_L1_error);
    __Pyx_INCREF(__pyx_mstate_global->__pyx_int_177953241);
    __Pyx_GIVEREF(__pyx_mstate_global->__pyx_int_177953241);
    if (__Pyx_PyTuple_SET_ITEM(__pyx_t_4, 1, __pyx_mstate_global->__pyx_int_177953241) != (0)) __PYX_ERR(3, 13, __pyx_L1_error);
    __Pyx_INCREF(Py_None);
    __Pyx_GIVEREF(Py_None);
    if (__Pyx_PyTuple_SET_ITEM(__pyx_t_4, 2, Py_None) != (0)) __PYX_ERR(3, 13, __pyx_L1_error);
    __pyx_t_2 = PyTuple_New(3); if (unlikely(!__pyx_t_2)) __PYX_ERR(3, 13, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_GIVEREF(__pyx_t_3);
    if (__Pyx_PyTuple_SET_ITEM(__pyx_t_2, 0, __pyx_t_3) != (0)) __PYX_ERR(3, 13, __pyx_L1_error);
    __Pyx_GIVEREF(__pyx_t_4);
    if (__Pyx_PyTuple_SET_ITEM(__pyx_t_2, 1, __pyx_t_4) != (0)) __PYX_ERR(3, 13, __pyx_L1_error);
    __Pyx_INCREF(__pyx_v_state);
    __Pyx_GIVEREF(__pyx_v_state);
    if (__Pyx_PyTuple_SET_ITEM(__pyx_t_2, 2, __pyx_v_state) != (0)) __PYX_ERR(3, 13, __pyx_L1_error);
    __pyx_t_3 = 0;
    __pyx_t_4 = 0;
    __pyx_r = __pyx_t_2;
    __pyx_t_2 = 0;
    goto __pyx_L0;

    /* "(tree fragment)":12
 *     else:
 *         use_setstate = self._owner is not None
 *     if use_setstate:             # <<<<<<<<<<<<<<
 *         return __pyx_unpickle_UnownedMemory, (type(self), 0xa9b59d9, None), state
 *     else:
*/
  }

  /* "(tree fragment)":15
 *         return __pyx_unpickle_UnownedMemory, (type(self), 0xa9b59d9, None), state
 *     else:
 *         return __pyx_unpickle_UnownedMemory, (type(self), 0xa9b59d9, state)             # <<<<<<<<<<<<<<
 * def __setstate_cython__(self, __pyx_state):
 *     __pyx_unpickle_UnownedMemory__set_state(self, __pyx_state)
*/
  /*else*/ {
    __Pyx_XDECREF(__pyx_r);
    __Pyx_GetModuleGlobalName(__pyx_t_2, __pyx_mstate_global->__pyx_n_u_pyx_unpickle_UnownedMemory); if (unlikely(!__pyx_t_2)) __PYX_ERR(3, 15, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_4 = PyTuple_New(3); if (unlikely(!__pyx_t_4)) __PYX_ERR(3, 15, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_INCREF(((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    __Pyx_GIVEREF(((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    if (__Pyx_PyTuple_SET_ITEM(__pyx_t_4, 0, ((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self)))) != (0)) __PYX_ERR(3, 15, __pyx_L1_error);
    __Pyx_INCREF(__pyx_mstate_global->__pyx_int_177953241);
    __Pyx_GIVEREF(__pyx_mstate_global->__pyx_int_177953241);
    if (__Pyx_PyTuple_SET_ITEM(__pyx_t_4, 1, __pyx_mstate_global->__pyx_int_177953241) != (0)) __PYX_ERR(3, 15, __pyx_L1_error);
    __Pyx_INCREF(__pyx_v_state);
    __Pyx_GIVEREF(__pyx_v_state);
    if (__Pyx_PyTuple_SET_ITEM(__pyx_t_4, 2, __pyx_v_state) != (0)) __PYX_ERR(3, 15, __pyx_L1_error);
    __pyx_t_3 = PyTuple_New(2); if (unlikely(!__pyx_t_3)) __PYX_ERR(3, 15, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_GIVEREF(__pyx_t_2);
    if (__Pyx_PyTuple_SET_ITEM(__pyx_t_3, 0, __pyx_t_2) != (0)) __PYX_ERR(3, 15, __pyx_L1_error);
    __Pyx_GIVEREF(__pyx_t_4);
    if (__Pyx_PyTuple_SET_ITEM(__pyx_t_3, 1, __pyx_t_4) != (0)) __PYX_ERR(3, 15, __pyx_L1_error);
    __pyx_t_2 = 0;
    __pyx_t_4 = 0;
    __pyx_r = __pyx_t_3;
    __pyx_t_3 = 0;
    goto __pyx_L0;
  }

  /* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     cdef tuple state
 *     cdef object _dict
*/

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_AddTraceback("cupy.cuda.memory.UnownedMemory.__reduce_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_state);
  __Pyx_XDECREF(__pyx_v__dict);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":16
 *     else:
 *         return __pyx_unpickle_UnownedMemory, (type(self), 0xa9b59d9, state)
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     __pyx_unpickle_UnownedMemory__set_state(self, __pyx_state)
*/

/* Python wrapper */
static PyObject *__pyx_pw_4cupy_4cuda_6memory_13UnownedMemory_5__setstate_cython__(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
PyDoc_STRVAR(__pyx_doc_4cupy_4cuda_6memory_13UnownedMemory_4__setstate_cython__, "UnownedMemory.__setstate_cython__(self, __pyx_state)");
static PyObject *__pyx_pw_4cupy_4cuda_6memory_13UnownedMemory_5__setstate_cython__(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
) {
  PyObject *__pyx_v___pyx_state = 0;
  #if !CYTHON_METH_FASTCALL
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  #endif
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject* values[1] = {0};
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__setstate_cython__ (wrapper)", 0);
  #if !CYTHON_METH_FASTCALL
  #if CYTHON_ASSUME_SAFE_SIZE
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return NULL;
  #endif
  #endif
  __pyx_kwvalues = __Pyx_KwValues_FASTCALL(__pyx_args, __pyx_nargs);
  {
    PyObject ** const __pyx_pyargnames[] = {&__pyx_mstate_global->__pyx_n_u_pyx_state,0};
    const Py_ssize_t __pyx_kwds_len = (__pyx_kwds) ? __Pyx_NumKwargs_FASTCALL(__pyx_kwds) : 0;
    if (unlikely(__pyx_kwds_len < 0)) __PYX_ERR(3, 16, __pyx_L3_error)
    if (__pyx_kwds_len > 0) {
      switch (__pyx_nargs) {
        case  1:
        values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(3, 16, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      const Py_ssize_t kwd_pos_args = __pyx_nargs;
      if (__Pyx_ParseKeywords(__pyx_kwds, __pyx_kwvalues, __pyx_pyargnames, 0, values, kwd_pos_args, __pyx_kwds_len, "__setstate_cython__", 0) < (0)) __PYX_ERR(3, 16, __pyx_L3_error)
      for (Py_ssize_t i = __pyx_nargs; i < 1; i++) {
        if (unlikely(!values[i])) { __Pyx_RaiseArgtupleInvalid("__setstate_cython__", 1, 1, 1, i); __PYX_ERR(3, 16, __pyx_L3_error) }
      }
    } else if (unlikely(__pyx_nargs != 1)) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
      if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(3, 16, __pyx_L3_error)
    }
    __pyx_v___pyx_state = values[0];
  }
  goto __pyx_L6_skip;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("__setstate_cython__", 1, 1, 1, __pyx_nargs); __PYX_ERR(3, 16, __pyx_L3_error)
  __pyx_L6_skip:;
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_AddTraceback("cupy.cuda.memory.UnownedMemory.__setstate_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_4cupy_4cuda_6memory_13UnownedMemory_4__setstate_cython__(((struct __pyx_obj_4cupy_4cuda_6memory_UnownedMemory *)__pyx_v_self), __pyx_v___pyx_state);

  /* function exit code */
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4cupy_4cuda_6memory_13UnownedMemory_4__setstate_cython__(struct __pyx_obj_4cupy_4cuda_6memory_UnownedMemory *__pyx_v_self, PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__setstate_cython__", 0);

  /* "(tree fragment)":17
 *         return __pyx_unpickle_UnownedMemory, (type(self), 0xa9b59d9, state)
 * def __setstate_cython__(self, __pyx_state):
 *     __pyx_unpickle_UnownedMemory__set_state(self, __pyx_state)             # <<<<<<<<<<<<<<
*/
  __pyx_t_1 = __pyx_v___pyx_state;
  __Pyx_INCREF(__pyx_t_1);
  if (!(likely(PyTuple_CheckExact(__pyx_t_1))||((__pyx_t_1) == Py_None) || __Pyx_RaiseUnexpectedTypeError("tuple", __pyx_t_1))) __PYX_ERR(3, 17, __pyx_L1_error)
  if (unlikely(__pyx_t_1 == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "cannot pass None into a C function argument that is declared 'not None'");
    __PYX_ERR(3, 17, __pyx_L1_error)
  }
  __pyx_t_2 = __pyx_f_4cupy_4cuda_6memory___pyx_unpickle_UnownedMemory__set_state(__pyx_v_self, ((PyObject*)__pyx_t_1)); if (unlikely(!__pyx_t_2)) __PYX_ERR(3, 17, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "(tree fragment)":16
 *     else:
 *         return __pyx_unpickle_UnownedMemory, (type(self), 0xa9b59d9, state)
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     __pyx_unpickle_UnownedMemory__set_state(self, __pyx_state)
*/

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_AddTraceback("cupy.cuda.memory.UnownedMemory.__setstate_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "cupy/cuda/memory.pyx":234
 *     """
 * 
 *     def __init__(self, size_t size):             # <<<<<<<<<<<<<<
 *         if (
 *             runtime._is_hip_environment and
*/

/* Python wrapper */
static int __pyx_pw_4cupy_4cuda_6memory_13ManagedMemory_1__init__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static int __pyx_pw_4cupy_4cuda_6memory_13ManagedMemory_1__init__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  size_t __pyx_v_size;
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject* values[1] = {0};
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__init__ (wrapper)", 0);
  #if CYTHON_ASSUME_SAFE_SIZE
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return -1;
  #endif
  __pyx_kwvalues = __Pyx_KwValues_VARARGS(__pyx_args, __pyx_nargs);
  {
    PyObject ** const __pyx_pyargnames[] = {&__pyx_mstate_global->__pyx_n_u_size,0};
    const Py_ssize_t __pyx_kwds_len = (__pyx_kwds) ? __Pyx_NumKwargs_VARARGS(__pyx_kwds) : 0;
    if (unlikely(__pyx_kwds_len < 0)) __PYX_ERR(0, 234, __pyx_L3_error)
    if (__pyx_kwds_len > 0) {
      switch (__pyx_nargs) {
        case  1:
        values[0] = __Pyx_ArgRef_VARARGS(__pyx_args, 0);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(0, 234, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      const Py_ssize_t kwd_pos_args = __pyx_nargs;
      if (__Pyx_ParseKeywords(__pyx_kwds, __pyx_kwvalues, __pyx_pyargnames, 0, values, kwd_pos_args, __pyx_kwds_len, "__init__", 0) < (0)) __PYX_ERR(0, 234, __pyx_L3_error)
      for (Py_ssize_t i = __pyx_nargs; i < 1; i++) {
        if (unlikely(!values[i])) { __Pyx_RaiseArgtupleInvalid("__init__", 1, 1, 1, i); __PYX_ERR(0, 234, __pyx_L3_error) }
      }
    } else if (unlikely(__pyx_nargs != 1)) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = __Pyx_ArgRef_VARARGS(__pyx_args, 0);
      if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(0, 234, __pyx_L3_error)
    }
    __pyx_v_size = __Pyx_PyLong_As_size_t(values[0]); if (unlikely((__pyx_v_size == (size_t)-1) && PyErr_Occurred())) __PYX_ERR(0, 234, __pyx_L3_error)
  }
  goto __pyx_L6_skip;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("__init__", 1, 1, 1, __pyx_nargs); __PYX_ERR(0, 234, __pyx_L3_error)
  __pyx_L6_skip:;
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_AddTraceback("cupy.cuda.memory.ManagedMemory.__init__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return -1;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_4cupy_4cuda_6memory_13ManagedMemory___init__(((struct __pyx_obj_4cupy_4cuda_6memory_ManagedMemory *)__pyx_v_self), __pyx_v_size);

  /* function exit code */
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_4cupy_4cuda_6memory_13ManagedMemory___init__(struct __pyx_obj_4cupy_4cuda_6memory_ManagedMemory *__pyx_v_self, size_t __pyx_v_size) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  int __pyx_t_4;
  size_t __pyx_t_5;
  int __pyx_t_6;
  intptr_t __pyx_t_7;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__init__", 0);

  /* "cupy/cuda/memory.pyx":236
 *     def __init__(self, size_t size):
 *         if (
 *             runtime._is_hip_environment and             # <<<<<<<<<<<<<<
 *             driver.get_build_version() < 40300000
 *         ):
*/
  if (__pyx_v_13cupy_backends_4cuda_3api_7runtime__is_hip_environment) {
  } else {
    __pyx_t_1 = __pyx_v_13cupy_backends_4cuda_3api_7runtime__is_hip_environment;
    goto __pyx_L4_bool_binop_done;
  }

  /* "cupy/cuda/memory.pyx":237
 *         if (
 *             runtime._is_hip_environment and
 *             driver.get_build_version() < 40300000             # <<<<<<<<<<<<<<
 *         ):
 *             raise RuntimeError('Managed memory requires ROCm 4.3+')
*/
  __pyx_t_2 = __pyx_f_13cupy_backends_4cuda_3api_6driver_get_build_version(0); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 237, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = PyObject_RichCompare(__pyx_t_2, __pyx_mstate_global->__pyx_int_40300000, Py_LT); __Pyx_XGOTREF(__pyx_t_3); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 237, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_4 = __Pyx_PyObject_IsTrue(__pyx_t_3); if (unlikely((__pyx_t_4 < 0))) __PYX_ERR(0, 237, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_t_1 = __pyx_t_4;
  __pyx_L4_bool_binop_done:;

  /* "cupy/cuda/memory.pyx":235
 * 
 *     def __init__(self, size_t size):
 *         if (             # <<<<<<<<<<<<<<
 *             runtime._is_hip_environment and
 *             driver.get_build_version() < 40300000
*/
  if (unlikely(__pyx_t_1)) {

    /* "cupy/cuda/memory.pyx":239
 *             driver.get_build_version() < 40300000
 *         ):
 *             raise RuntimeError('Managed memory requires ROCm 4.3+')             # <<<<<<<<<<<<<<
 *         self.size = size
 *         self.device_id = device.get_device_id()
*/
    __pyx_t_2 = NULL;
    __pyx_t_5 = 1;
    {
      PyObject *__pyx_callargs[2] = {__pyx_t_2, __pyx_mstate_global->__pyx_kp_u_Managed_memory_requires_ROCm_4_3};
      __pyx_t_3 = __Pyx_PyObject_FastCall((PyObject*)(((PyTypeObject*)PyExc_RuntimeError)), __pyx_callargs+__pyx_t_5, (2-__pyx_t_5) | (__pyx_t_5*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
      __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
      if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 239, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_3);
    }
    __Pyx_Raise(__pyx_t_3, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __PYX_ERR(0, 239, __pyx_L1_error)

    /* "cupy/cuda/memory.pyx":235
 * 
 *     def __init__(self, size_t size):
 *         if (             # <<<<<<<<<<<<<<
 *             runtime._is_hip_environment and
 *             driver.get_build_version() < 40300000
*/
  }

  /* "cupy/cuda/memory.pyx":240
 *         ):
 *             raise RuntimeError('Managed memory requires ROCm 4.3+')
 *         self.size = size             # <<<<<<<<<<<<<<
 *         self.device_id = device.get_device_id()
 *         self.ptr = 0
*/
  __pyx_v_self->__pyx_base.size = __pyx_v_size;

  /* "cupy/cuda/memory.pyx":241
 *             raise RuntimeError('Managed memory requires ROCm 4.3+')
 *         self.size = size
 *         self.device_id = device.get_device_id()             # <<<<<<<<<<<<<<
 *         self.ptr = 0
 *         if size > 0:
*/
  __pyx_t_6 = __pyx_f_4cupy_4cuda_6device_get_device_id(0); if (unlikely(__pyx_t_6 == ((int)-1) && PyErr_Occurred())) __PYX_ERR(0, 241, __pyx_L1_error)
  __pyx_v_self->__pyx_base.device_id = __pyx_t_6;

  /* "cupy/cuda/memory.pyx":242
 *         self.size = size
 *         self.device_id = device.get_device_id()
 *         self.ptr = 0             # <<<<<<<<<<<<<<
 *         if size > 0:
 *             self.ptr = runtime.mallocManaged(size)
*/
  __pyx_v_self->__pyx_base.ptr = 0;

  /* "cupy/cuda/memory.pyx":243
 *         self.device_id = device.get_device_id()
 *         self.ptr = 0
 *         if size > 0:             # <<<<<<<<<<<<<<
 *             self.ptr = runtime.mallocManaged(size)
 * 
*/
  __pyx_t_1 = (__pyx_v_size > 0);
  if (__pyx_t_1) {

    /* "cupy/cuda/memory.pyx":244
 *         self.ptr = 0
 *         if size > 0:
 *             self.ptr = runtime.mallocManaged(size)             # <<<<<<<<<<<<<<
 * 
 *     def prefetch(self, stream, *, int device_id=runtime.cudaInvalidDeviceId):
*/
    __pyx_t_7 = __pyx_f_13cupy_backends_4cuda_3api_7runtime_mallocManaged(__pyx_v_size, 0, NULL); if (unlikely(__pyx_t_7 == ((intptr_t)0) && PyErr_Occurred())) __PYX_ERR(0, 244, __pyx_L1_error)
    __pyx_v_self->__pyx_base.ptr = __pyx_t_7;

    /* "cupy/cuda/memory.pyx":243
 *         self.device_id = device.get_device_id()
 *         self.ptr = 0
 *         if size > 0:             # <<<<<<<<<<<<<<
 *             self.ptr = runtime.mallocManaged(size)
 * 
*/
  }

  /* "cupy/cuda/memory.pyx":234
 *     """
 * 
 *     def __init__(self, size_t size):             # <<<<<<<<<<<<<<
 *         if (
 *             runtime._is_hip_environment and
*/

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_AddTraceback("cupy.cuda.memory.ManagedMemory.__init__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "cupy/cuda/memory.pyx":246
 *             self.ptr = runtime.mallocManaged(size)
 * 
 *     def prefetch(self, stream, *, int device_id=runtime.cudaInvalidDeviceId):             # <<<<<<<<<<<<<<
 *         """(experimental) Prefetch memory.
 * 
*/

/* Python wrapper */
static PyObject *__pyx_pw_4cupy_4cuda_6memory_13ManagedMemory_3prefetch(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
PyDoc_STRVAR(__pyx_doc_4cupy_4cuda_6memory_13ManagedMemory_2prefetch, "ManagedMemory.prefetch(self, stream, *, int device_id=runtime.cudaInvalidDeviceId)\n\n(experimental) Prefetch memory.\n\nArgs:\n    stream (cupy.cuda.Stream): CUDA stream.\n    device_id (int): CUDA device ID (-1 for CPU).");
static PyObject *__pyx_pw_4cupy_4cuda_6memory_13ManagedMemory_3prefetch(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
) {
  PyObject *__pyx_v_stream = 0;
  int __pyx_v_device_id;
  #if !CYTHON_METH_FASTCALL
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  #endif
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject* values[2] = {0,0};
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("prefetch (wrapper)", 0);
  #if !CYTHON_METH_FASTCALL
  #if CYTHON_ASSUME_SAFE_SIZE
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return NULL;
  #endif
  #endif
  __pyx_kwvalues = __Pyx_KwValues_FASTCALL(__pyx_args, __pyx_nargs);
  {
    PyObject ** const __pyx_pyargnames[] = {&__pyx_mstate_global->__pyx_n_u_stream,&__pyx_mstate_global->__pyx_n_u_device_id,0};
    const Py_ssize_t __pyx_kwds_len = (__pyx_kwds) ? __Pyx_NumKwargs_FASTCALL(__pyx_kwds) : 0;
    if (unlikely(__pyx_kwds_len < 0)) __PYX_ERR(0, 246, __pyx_L3_error)
    if (__pyx_kwds_len > 0) {
      switch (__pyx_nargs) {
        case  1:
        values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(0, 246, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      const Py_ssize_t kwd_pos_args = __pyx_nargs;
      if (__Pyx_ParseKeywords(__pyx_kwds, __pyx_kwvalues, __pyx_pyargnames, 0, values, kwd_pos_args, __pyx_kwds_len, "prefetch", 0) < (0)) __PYX_ERR(0, 246, __pyx_L3_error)
      for (Py_ssize_t i = __pyx_nargs; i < 1; i++) {
        if (unlikely(!values[i])) { __Pyx_RaiseArgtupleInvalid("prefetch", 1, 1, 1, i); __PYX_ERR(0, 246, __pyx_L3_error) }
      }
    } else if (unlikely(__pyx_nargs != 1)) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
      if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(0, 246, __pyx_L3_error)
    }
    __pyx_v_stream = values[0];
    if (values[1]) {
      __pyx_v_device_id = __Pyx_PyLong_As_int(values[1]); if (unlikely((__pyx_v_device_id == (int)-1) && PyErr_Occurred())) __PYX_ERR(0, 246, __pyx_L3_error)
    } else {
      __pyx_v_device_id = __pyx_mstate_global->__pyx_k_;
    }
  }
  goto __pyx_L6_skip;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("prefetch", 1, 1, 1, __pyx_nargs); __PYX_ERR(0, 246, __pyx_L3_error)
  __pyx_L6_skip:;
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_AddTraceback("cupy.cuda.memory.ManagedMemory.prefetch", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_4cupy_4cuda_6memory_13ManagedMemory_2prefetch(((struct __pyx_obj_4cupy_4cuda_6memory_ManagedMemory *)__pyx_v_self), __pyx_v_stream, __pyx_v_device_id);

  /* function exit code */
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4cupy_4cuda_6memory_13ManagedMemory_2prefetch(struct __pyx_obj_4cupy_4cuda_6memory_ManagedMemory *__pyx_v_self, PyObject *__pyx_v_stream, int __pyx_v_device_id) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  int __pyx_t_2;
  PyObject *__pyx_t_3 = NULL;
  intptr_t __pyx_t_4;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("prefetch", 0);

  /* "cupy/cuda/memory.pyx":253
 *             device_id (int): CUDA device ID (-1 for CPU).
 *         """
 *         if device_id == runtime.cudaInvalidDeviceId:             # <<<<<<<<<<<<<<
 *             device_id = self.device_id
 *         runtime.memPrefetchAsync(self.ptr, self.size, device_id, stream.ptr)
*/
  __pyx_t_1 = (__pyx_v_device_id == __pyx_e_13cupy_backends_4cuda_3api_13_runtime_enum_cudaInvalidDeviceId);
  if (__pyx_t_1) {

    /* "cupy/cuda/memory.pyx":254
 *         """
 *         if device_id == runtime.cudaInvalidDeviceId:
 *             device_id = self.device_id             # <<<<<<<<<<<<<<
 *         runtime.memPrefetchAsync(self.ptr, self.size, device_id, stream.ptr)
 * 
*/
    __pyx_t_2 = __pyx_v_self->__pyx_base.device_id;
    __pyx_v_device_id = __pyx_t_2;

    /* "cupy/cuda/memory.pyx":253
 *             device_id (int): CUDA device ID (-1 for CPU).
 *         """
 *         if device_id == runtime.cudaInvalidDeviceId:             # <<<<<<<<<<<<<<
 *             device_id = self.device_id
 *         runtime.memPrefetchAsync(self.ptr, self.size, device_id, stream.ptr)
*/
  }

  /* "cupy/cuda/memory.pyx":255
 *         if device_id == runtime.cudaInvalidDeviceId:
 *             device_id = self.device_id
 *         runtime.memPrefetchAsync(self.ptr, self.size, device_id, stream.ptr)             # <<<<<<<<<<<<<<
 * 
 *     def advise(self, int advise, device.Device dev):
*/
  __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_v_stream, __pyx_mstate_global->__pyx_n_u_ptr); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 255, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_4 = PyLong_AsSsize_t(__pyx_t_3); if (unlikely((__pyx_t_4 == ((intptr_t)-1)) && PyErr_Occurred())) __PYX_ERR(0, 255, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_t_3 = __pyx_f_13cupy_backends_4cuda_3api_7runtime_memPrefetchAsync(__pyx_v_self->__pyx_base.ptr, __pyx_v_self->__pyx_base.size, __pyx_v_device_id, __pyx_t_4, 0); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 255, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "cupy/cuda/memory.pyx":246
 *             self.ptr = runtime.mallocManaged(size)
 * 
 *     def prefetch(self, stream, *, int device_id=runtime.cudaInvalidDeviceId):             # <<<<<<<<<<<<<<
 *         """(experimental) Prefetch memory.
 * 
*/

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_AddTraceback("cupy.cuda.memory.ManagedMemory.prefetch", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "cupy/cuda/memory.pyx":257
 *         runtime.memPrefetchAsync(self.ptr, self.size, device_id, stream.ptr)
 * 
 *     def advise(self, int advise, device.Device dev):             # <<<<<<<<<<<<<<
 *         """(experimental) Advise about the usage of this memory.
 * 
*/

/* Python wrapper */
static PyObject *__pyx_pw_4cupy_4cuda_6memory_13ManagedMemory_5advise(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
PyDoc_STRVAR(__pyx_doc_4cupy_4cuda_6memory_13ManagedMemory_4advise, "ManagedMemory.advise(self, int advise, Device dev)\n\n(experimental) Advise about the usage of this memory.\n\nArgs:\n    advics (int): Advise to be applied for this memory.\n    dev (cupy.cuda.Device): Device to apply the advice for.");
static PyObject *__pyx_pw_4cupy_4cuda_6memory_13ManagedMemory_5advise(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
) {
  int __pyx_v_advise;
  struct __pyx_obj_4cupy_4cuda_6device_Device *__pyx_v_dev = 0;
  #if !CYTHON_METH_FASTCALL
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  #endif
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject* values[2] = {0,0};
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("advise (wrapper)", 0);
  #if !CYTHON_METH_FASTCALL
  #if CYTHON_ASSUME_SAFE_SIZE
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return NULL;
  #endif
  #endif
  __pyx_kwvalues = __Pyx_KwValues_FASTCALL(__pyx_args, __pyx_nargs);
  {
    PyObject ** const __pyx_pyargnames[] = {&__pyx_mstate_global->__pyx_n_u_advise,&__pyx_mstate_global->__pyx_n_u_dev,0};
    const Py_ssize_t __pyx_kwds_len = (__pyx_kwds) ? __Pyx_NumKwargs_FASTCALL(__pyx_kwds) : 0;
    if (unlikely(__pyx_kwds_len < 0)) __PYX_ERR(0, 257, __pyx_L3_error)
    if (__pyx_kwds_len > 0) {
      switch (__pyx_nargs) {
        case  2:
        values[1] = __Pyx_ArgRef_FASTCALL(__pyx_args, 1);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[1])) __PYX_ERR(0, 257, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  1:
        values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(0, 257, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      const Py_ssize_t kwd_pos_args = __pyx_nargs;
      if (__Pyx_ParseKeywords(__pyx_kwds, __pyx_kwvalues, __pyx_pyargnames, 0, values, kwd_pos_args, __pyx_kwds_len, "advise", 0) < (0)) __PYX_ERR(0, 257, __pyx_L3_error)
      for (Py_ssize_t i = __pyx_nargs; i < 2; i++) {
        if (unlikely(!values[i])) { __Pyx_RaiseArgtupleInvalid("advise", 1, 2, 2, i); __PYX_ERR(0, 257, __pyx_L3_error) }
      }
    } else if (unlikely(__pyx_nargs != 2)) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
      if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(0, 257, __pyx_L3_error)
      values[1] = __Pyx_ArgRef_FASTCALL(__pyx_args, 1);
      if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[1])) __PYX_ERR(0, 257, __pyx_L3_error)
    }
    __pyx_v_advise = __Pyx_PyLong_As_int(values[0]); if (unlikely((__pyx_v_advise == (int)-1) && PyErr_Occurred())) __PYX_ERR(0, 257, __pyx_L3_error)
    __pyx_v_dev = ((struct __pyx_obj_4cupy_4cuda_6device_Device *)values[1]);
  }
  goto __pyx_L6_skip;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("advise", 1, 2, 2, __pyx_nargs); __PYX_ERR(0, 257, __pyx_L3_error)
  __pyx_L6_skip:;
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_AddTraceback("cupy.cuda.memory.ManagedMemory.advise", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_dev), __pyx_mstate_global->__pyx_ptype_4cupy_4cuda_6device_Device, 1, "dev", 0))) __PYX_ERR(0, 257, __pyx_L1_error)
  __pyx_r = __pyx_pf_4cupy_4cuda_6memory_13ManagedMemory_4advise(((struct __pyx_obj_4cupy_4cuda_6memory_ManagedMemory *)__pyx_v_self), __pyx_v_advise, __pyx_v_dev);

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = NULL;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  goto __pyx_L7_cleaned_up;
  __pyx_L0:;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __pyx_L7_cleaned_up:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4cupy_4cuda_6memory_13ManagedMemory_4advise(struct __pyx_obj_4cupy_4cuda_6memory_ManagedMemory *__pyx_v_self, int __pyx_v_advise, struct __pyx_obj_4cupy_4cuda_6device_Device *__pyx_v_dev) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("advise", 0);

  /* "cupy/cuda/memory.pyx":265
 * 
 *         """
 *         runtime.memAdvise(self.ptr, self.size, advise, dev.id)             # <<<<<<<<<<<<<<
 * 
 *     def __dealloc__(self):
*/
  __pyx_t_1 = __pyx_f_13cupy_backends_4cuda_3api_7runtime_memAdvise(__pyx_v_self->__pyx_base.ptr, __pyx_v_self->__pyx_base.size, __pyx_v_advise, __pyx_v_dev->id, 0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 265, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "cupy/cuda/memory.pyx":257
 *         runtime.memPrefetchAsync(self.ptr, self.size, device_id, stream.ptr)
 * 
 *     def advise(self, int advise, device.Device dev):             # <<<<<<<<<<<<<<
 *         """(experimental) Advise about the usage of this memory.
 * 
*/

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("cupy.cuda.memory.ManagedMemory.advise", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "cupy/cuda/memory.pyx":267
 *         runtime.memAdvise(self.ptr, self.size, advise, dev.id)
 * 
 *     def __dealloc__(self):             # <<<<<<<<<<<<<<
 *         # Note: Cannot raise in the destructor! (cython/cython#1613)
 *         if self.ptr:
*/

/* Python wrapper */
static void __pyx_pw_4cupy_4cuda_6memory_13ManagedMemory_7__dealloc__(PyObject *__pyx_v_self); /*proto*/
static void __pyx_pw_4cupy_4cuda_6memory_13ManagedMemory_7__dealloc__(PyObject *__pyx_v_self) {
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__dealloc__ (wrapper)", 0);
  __pyx_kwvalues = __Pyx_KwValues_VARARGS(__pyx_args, __pyx_nargs);
  __pyx_pf_4cupy_4cuda_6memory_13ManagedMemory_6__dealloc__(((struct __pyx_obj_4cupy_4cuda_6memory_ManagedMemory *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
}

static void __pyx_pf_4cupy_4cuda_6memory_13ManagedMemory_6__dealloc__(struct __pyx_obj_4cupy_4cuda_6memory_ManagedMemory *__pyx_v_self) {
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__dealloc__", 0);

  /* "cupy/cuda/memory.pyx":269
 *     def __dealloc__(self):
 *         # Note: Cannot raise in the destructor! (cython/cython#1613)
 *         if self.ptr:             # <<<<<<<<<<<<<<
 *             runtime.free(self.ptr)
 * 
*/
  __pyx_t_1 = (__pyx_v_self->__pyx_base.ptr != 0);
  if (__pyx_t_1) {

    /* "cupy/cuda/memory.pyx":270
 *         # Note: Cannot raise in the destructor! (cython/cython#1613)
 *         if self.ptr:
 *             runtime.free(self.ptr)             # <<<<<<<<<<<<<<
 * 
 * 
*/
    __pyx_t_2 = __pyx_f_13cupy_backends_4cuda_3api_7runtime_free(__pyx_v_self->__pyx_base.ptr, 0); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 270, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

    /* "cupy/cuda/memory.pyx":269
 *     def __dealloc__(self):
 *         # Note: Cannot raise in the destructor! (cython/cython#1613)
 *         if self.ptr:             # <<<<<<<<<<<<<<
 *             runtime.free(self.ptr)
 * 
*/
  }

  /* "cupy/cuda/memory.pyx":267
 *         runtime.memAdvise(self.ptr, self.size, advise, dev.id)
 * 
 *     def __dealloc__(self):             # <<<<<<<<<<<<<<
 *         # Note: Cannot raise in the destructor! (cython/cython#1613)
 *         if self.ptr:
*/

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_WriteUnraisable("cupy.cuda.memory.ManagedMemory.__dealloc__", __pyx_clineno, __pyx_lineno, __pyx_filename, 1, 0);
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
}

/* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     cdef tuple state
 *     cdef object _dict
*/

/* Python wrapper */
static PyObject *__pyx_pw_4cupy_4cuda_6memory_13ManagedMemory_9__reduce_cython__(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
PyDoc_STRVAR(__pyx_doc_4cupy_4cuda_6memory_13ManagedMemory_8__reduce_cython__, "ManagedMemory.__reduce_cython__(self)");
static PyObject *__pyx_pw_4cupy_4cuda_6memory_13ManagedMemory_9__reduce_cython__(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
) {
  #if !CYTHON_METH_FASTCALL
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  #endif
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__reduce_cython__ (wrapper)", 0);
  #if !CYTHON_METH_FASTCALL
  #if CYTHON_ASSUME_SAFE_SIZE
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return NULL;
  #endif
  #endif
  __pyx_kwvalues = __Pyx_KwValues_FASTCALL(__pyx_args, __pyx_nargs);
  if (unlikely(__pyx_nargs > 0)) { __Pyx_RaiseArgtupleInvalid("__reduce_cython__", 1, 0, 0, __pyx_nargs); return NULL; }
  const Py_ssize_t __pyx_kwds_len = unlikely(__pyx_kwds) ? __Pyx_NumKwargs_FASTCALL(__pyx_kwds) : 0;
  if (unlikely(__pyx_kwds_len < 0)) return NULL;
  if (unlikely(__pyx_kwds_len > 0)) {__Pyx_RejectKeywords("__reduce_cython__", __pyx_kwds); return NULL;}
  __pyx_r = __pyx_pf_4cupy_4cuda_6memory_13ManagedMemory_8__reduce_cython__(((struct __pyx_obj_4cupy_4cuda_6memory_ManagedMemory *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4cupy_4cuda_6memory_13ManagedMemory_8__reduce_cython__(struct __pyx_obj_4cupy_4cuda_6memory_ManagedMemory *__pyx_v_self) {
  PyObject *__pyx_v_state = 0;
  PyObject *__pyx_v__dict = 0;
  int __pyx_v_use_setstate;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  int __pyx_t_5;
  int __pyx_t_6;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__reduce_cython__", 0);

  /* "(tree fragment)":5
 *     cdef object _dict
 *     cdef bint use_setstate
 *     state = (self.device_id, self.ptr, self.size)             # <<<<<<<<<<<<<<
 *     _dict = getattr(self, '__dict__', None)
 *     if _dict is not None and _dict:
*/
  __pyx_t_1 = __Pyx_PyLong_From_int(__pyx_v_self->__pyx_base.device_id); if (unlikely(!__pyx_t_1)) __PYX_ERR(3, 5, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = PyLong_FromSsize_t(__pyx_v_self->__pyx_base.ptr); if (unlikely(!__pyx_t_2)) __PYX_ERR(3, 5, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = __Pyx_PyLong_FromSize_t(__pyx_v_self->__pyx_base.size); if (unlikely(!__pyx_t_3)) __PYX_ERR(3, 5, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_4 = PyTuple_New(3); if (unlikely(!__pyx_t_4)) __PYX_ERR(3, 5, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_GIVEREF(__pyx_t_1);
  if (__Pyx_PyTuple_SET_ITEM(__pyx_t_4, 0, __pyx_t_1) != (0)) __PYX_ERR(3, 5, __pyx_L1_error);
  __Pyx_GIVEREF(__pyx_t_2);
  if (__Pyx_PyTuple_SET_ITEM(__pyx_t_4, 1, __pyx_t_2) != (0)) __PYX_ERR(3, 5, __pyx_L1_error);
  __Pyx_GIVEREF(__pyx_t_3);
  if (__Pyx_PyTuple_SET_ITEM(__pyx_t_4, 2, __pyx_t_3) != (0)) __PYX_ERR(3, 5, __pyx_L1_error);
  __pyx_t_1 = 0;
  __pyx_t_2 = 0;
  __pyx_t_3 = 0;
  __pyx_v_state = ((PyObject*)__pyx_t_4);
  __pyx_t_4 = 0;

  /* "(tree fragment)":6
 *     cdef bint use_setstate
 *     state = (self.device_id, self.ptr, self.size)
 *     _dict = getattr(self, '__dict__', None)             # <<<<<<<<<<<<<<
 *     if _dict is not None and _dict:
 *         state += (_dict,)
*/
  __pyx_t_4 = __Pyx_GetAttr3(((PyObject *)__pyx_v_self), __pyx_mstate_global->__pyx_n_u_dict, Py_None); if (unlikely(!__pyx_t_4)) __PYX_ERR(3, 6, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_v__dict = __pyx_t_4;
  __pyx_t_4 = 0;

  /* "(tree fragment)":7
 *     state = (self.device_id, self.ptr, self.size)
 *     _dict = getattr(self, '__dict__', None)
 *     if _dict is not None and _dict:             # <<<<<<<<<<<<<<
 *         state += (_dict,)
 *         use_setstate = True
*/
  __pyx_t_6 = (__pyx_v__dict != Py_None);
  if (__pyx_t_6) {
  } else {
    __pyx_t_5 = __pyx_t_6;
    goto __pyx_L4_bool_binop_done;
  }
  __pyx_t_6 = __Pyx_PyObject_IsTrue(__pyx_v__dict); if (unlikely((__pyx_t_6 < 0))) __PYX_ERR(3, 7, __pyx_L1_error)
  __pyx_t_5 = __pyx_t_6;
  __pyx_L4_bool_binop_done:;
  if (__pyx_t_5) {

    /* "(tree fragment)":8
 *     _dict = getattr(self, '__dict__', None)
 *     if _dict is not None and _dict:
 *         state += (_dict,)             # <<<<<<<<<<<<<<
 *         use_setstate = True
 *     else:
*/
    __pyx_t_4 = PyTuple_New(1); if (unlikely(!__pyx_t_4)) __PYX_ERR(3, 8, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_INCREF(__pyx_v__dict);
    __Pyx_GIVEREF(__pyx_v__dict);
    if (__Pyx_PyTuple_SET_ITEM(__pyx_t_4, 0, __pyx_v__dict) != (0)) __PYX_ERR(3, 8, __pyx_L1_error);
    __pyx_t_3 = PyNumber_InPlaceAdd(__pyx_v_state, __pyx_t_4); if (unlikely(!__pyx_t_3)) __PYX_ERR(3, 8, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __Pyx_DECREF_SET(__pyx_v_state, ((PyObject*)__pyx_t_3));
    __pyx_t_3 = 0;

    /* "(tree fragment)":9
 *     if _dict is not None and _dict:
 *         state += (_dict,)
 *         use_setstate = True             # <<<<<<<<<<<<<<
 *     else:
 *         use_setstate = False
*/
    __pyx_v_use_setstate = 1;

    /* "(tree fragment)":7
 *     state = (self.device_id, self.ptr, self.size)
 *     _dict = getattr(self, '__dict__', None)
 *     if _dict is not None and _dict:             # <<<<<<<<<<<<<<
 *         state += (_dict,)
 *         use_setstate = True
*/
    goto __pyx_L3;
  }

  /* "(tree fragment)":11
 *         use_setstate = True
 *     else:
 *         use_setstate = False             # <<<<<<<<<<<<<<
 *     if use_setstate:
 *         return __pyx_unpickle_ManagedMemory, (type(self), 0x61cadb0, None), state
*/
  /*else*/ {
    __pyx_v_use_setstate = 0;
  }
  __pyx_L3:;

  /* "(tree fragment)":12
 *     else:
 *         use_setstate = False
 *     if use_setstate:             # <<<<<<<<<<<<<<
 *         return __pyx_unpickle_ManagedMemory, (type(self), 0x61cadb0, None), state
 *     else:
*/
  if (__pyx_v_use_setstate) {

    /* "(tree fragment)":13
 *         use_setstate = False
 *     if use_setstate:
 *         return __pyx_unpickle_ManagedMemory, (type(self), 0x61cadb0, None), state             # <<<<<<<<<<<<<<
 *     else:
 *         return __pyx_unpickle_ManagedMemory, (type(self), 0x61cadb0, state)
*/
    __Pyx_XDECREF(__pyx_r);
    __Pyx_GetModuleGlobalName(__pyx_t_3, __pyx_mstate_global->__pyx_n_u_pyx_unpickle_ManagedMemory); if (unlikely(!__pyx_t_3)) __PYX_ERR(3, 13, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_t_4 = PyTuple_New(3); if (unlikely(!__pyx_t_4)) __PYX_ERR(3, 13, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_INCREF(((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    __Pyx_GIVEREF(((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    if (__Pyx_PyTuple_SET_ITEM(__pyx_t_4, 0, ((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self)))) != (0)) __PYX_ERR(3, 13, __pyx_L1_error);
    __Pyx_INCREF(__pyx_mstate_global->__pyx_int_102542768);
    __Pyx_GIVEREF(__pyx_mstate_global->__pyx_int_102542768);
    if (__Pyx_PyTuple_SET_ITEM(__pyx_t_4, 1, __pyx_mstate_global->__pyx_int_102542768) != (0)) __PYX_ERR(3, 13, __pyx_L1_error);
    __Pyx_INCREF(Py_None);
    __Pyx_GIVEREF(Py_None);
    if (__Pyx_PyTuple_SET_ITEM(__pyx_t_4, 2, Py_None) != (0)) __PYX_ERR(3, 13, __pyx_L1_error);
    __pyx_t_2 = PyTuple_New(3); if (unlikely(!__pyx_t_2)) __PYX_ERR(3, 13, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_GIVEREF(__pyx_t_3);
    if (__Pyx_PyTuple_SET_ITEM(__pyx_t_2, 0, __pyx_t_3) != (0)) __PYX_ERR(3, 13, __pyx_L1_error);
    __Pyx_GIVEREF(__pyx_t_4);
    if (__Pyx_PyTuple_SET_ITEM(__pyx_t_2, 1, __pyx_t_4) != (0)) __PYX_ERR(3, 13, __pyx_L1_error);
    __Pyx_INCREF(__pyx_v_state);
    __Pyx_GIVEREF(__pyx_v_state);
    if (__Pyx_PyTuple_SET_ITEM(__pyx_t_2, 2, __pyx_v_state) != (0)) __PYX_ERR(3, 13, __pyx_L1_error);
    __pyx_t_3 = 0;
    __pyx_t_4 = 0;
    __pyx_r = __pyx_t_2;
    __pyx_t_2 = 0;
    goto __pyx_L0;

    /* "(tree fragment)":12
 *     else:
 *         use_setstate = False
 *     if use_setstate:             # <<<<<<<<<<<<<<
 *         return __pyx_unpickle_ManagedMemory, (type(self), 0x61cadb0, None), state
 *     else:
*/
  }

  /* "(tree fragment)":15
 *         return __pyx_unpickle_ManagedMemory, (type(self), 0x61cadb0, None), state
 *     else:
 *         return __pyx_unpickle_ManagedMemory, (type(self), 0x61cadb0, state)             # <<<<<<<<<<<<<<
 * def __setstate_cython__(self, __pyx_state):
 *     __pyx_unpickle_ManagedMemory__set_state(self, __pyx_state)
*/
  /*else*/ {
    __Pyx_XDECREF(__pyx_r);
    __Pyx_GetModuleGlobalName(__pyx_t_2, __pyx_mstate_global->__pyx_n_u_pyx_unpickle_ManagedMemory); if (unlikely(!__pyx_t_2)) __PYX_ERR(3, 15, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_4 = PyTuple_New(3); if (unlikely(!__pyx_t_4)) __PYX_ERR(3, 15, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_INCREF(((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    __Pyx_GIVEREF(((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    if (__Pyx_PyTuple_SET_ITEM(__pyx_t_4, 0, ((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self)))) != (0)) __PYX_ERR(3, 15, __pyx_L1_error);
    __Pyx_INCREF(__pyx_mstate_global->__pyx_int_102542768);
    __Pyx_GIVEREF(__pyx_mstate_global->__pyx_int_102542768);
    if (__Pyx_PyTuple_SET_ITEM(__pyx_t_4, 1, __pyx_mstate_global->__pyx_int_102542768) != (0)) __PYX_ERR(3, 15, __pyx_L1_error);
    __Pyx_INCREF(__pyx_v_state);
    __Pyx_GIVEREF(__pyx_v_state);
    if (__Pyx_PyTuple_SET_ITEM(__pyx_t_4, 2, __pyx_v_state) != (0)) __PYX_ERR(3, 15, __pyx_L1_error);
    __pyx_t_3 = PyTuple_New(2); if (unlikely(!__pyx_t_3)) __PYX_ERR(3, 15, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_GIVEREF(__pyx_t_2);
    if (__Pyx_PyTuple_SET_ITEM(__pyx_t_3, 0, __pyx_t_2) != (0)) __PYX_ERR(3, 15, __pyx_L1_error);
    __Pyx_GIVEREF(__pyx_t_4);
    if (__Pyx_PyTuple_SET_ITEM(__pyx_t_3, 1, __pyx_t_4) != (0)) __PYX_ERR(3, 15, __pyx_L1_error);
    __pyx_t_2 = 0;
    __pyx_t_4 = 0;
    __pyx_r = __pyx_t_3;
    __pyx_t_3 = 0;
    goto __pyx_L0;
  }

  /* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     cdef tuple state
 *     cdef object _dict
*/

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_AddTraceback("cupy.cuda.memory.ManagedMemory.__reduce_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_state);
  __Pyx_XDECREF(__pyx_v__dict);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":16
 *     else:
 *         return __pyx_unpickle_ManagedMemory, (type(self), 0x61cadb0, state)
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     __pyx_unpickle_ManagedMemory__set_state(self, __pyx_state)
*/

/* Python wrapper */
static PyObject *__pyx_pw_4cupy_4cuda_6memory_13ManagedMemory_11__setstate_cython__(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
PyDoc_STRVAR(__pyx_doc_4cupy_4cuda_6memory_13ManagedMemory_10__setstate_cython__, "ManagedMemory.__setstate_cython__(self, __pyx_state)");
static PyObject *__pyx_pw_4cupy_4cuda_6memory_13ManagedMemory_11__setstate_cython__(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
) {
  PyObject *__pyx_v___pyx_state = 0;
  #if !CYTHON_METH_FASTCALL
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  #endif
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject* values[1] = {0};
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__setstate_cython__ (wrapper)", 0);
  #if !CYTHON_METH_FASTCALL
  #if CYTHON_ASSUME_SAFE_SIZE
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return NULL;
  #endif
  #endif
  __pyx_kwvalues = __Pyx_KwValues_FASTCALL(__pyx_args, __pyx_nargs);
  {
    PyObject ** const __pyx_pyargnames[] = {&__pyx_mstate_global->__pyx_n_u_pyx_state,0};
    const Py_ssize_t __pyx_kwds_len = (__pyx_kwds) ? __Pyx_NumKwargs_FASTCALL(__pyx_kwds) : 0;
    if (unlikely(__pyx_kwds_len < 0)) __PYX_ERR(3, 16, __pyx_L3_error)
    if (__pyx_kwds_len > 0) {
      switch (__pyx_nargs) {
        case  1:
        values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(3, 16, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      const Py_ssize_t kwd_pos_args = __pyx_nargs;
      if (__Pyx_ParseKeywords(__pyx_kwds, __pyx_kwvalues, __pyx_pyargnames, 0, values, kwd_pos_args, __pyx_kwds_len, "__setstate_cython__", 0) < (0)) __PYX_ERR(3, 16, __pyx_L3_error)
      for (Py_ssize_t i = __pyx_nargs; i < 1; i++) {
        if (unlikely(!values[i])) { __Pyx_RaiseArgtupleInvalid("__setstate_cython__", 1, 1, 1, i); __PYX_ERR(3, 16, __pyx_L3_error) }
      }
    } else if (unlikely(__pyx_nargs != 1)) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
      if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(3, 16, __pyx_L3_error)
    }
    __pyx_v___pyx_state = values[0];
  }
  goto __pyx_L6_skip;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("__setstate_cython__", 1, 1, 1, __pyx_nargs); __PYX_ERR(3, 16, __pyx_L3_error)
  __pyx_L6_skip:;
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_AddTraceback("cupy.cuda.memory.ManagedMemory.__setstate_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_4cupy_4cuda_6memory_13ManagedMemory_10__setstate_cython__(((struct __pyx_obj_4cupy_4cuda_6memory_ManagedMemory *)__pyx_v_self), __pyx_v___pyx_state);

  /* function exit code */
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4cupy_4cuda_6memory_13ManagedMemory_10__setstate_cython__(struct __pyx_obj_4cupy_4cuda_6memory_ManagedMemory *__pyx_v_self, PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__setstate_cython__", 0);

  /* "(tree fragment)":17
 *         return __pyx_unpickle_ManagedMemory, (type(self), 0x61cadb0, state)
 * def __setstate_cython__(self, __pyx_state):
 *     __pyx_unpickle_ManagedMemory__set_state(self, __pyx_state)             # <<<<<<<<<<<<<<
*/
  __pyx_t_1 = __pyx_v___pyx_state;
  __Pyx_INCREF(__pyx_t_1);
  if (!(likely(PyTuple_CheckExact(__pyx_t_1))||((__pyx_t_1) == Py_None) || __Pyx_RaiseUnexpectedTypeError("tuple", __pyx_t_1))) __PYX_ERR(3, 17, __pyx_L1_error)
  if (unlikely(__pyx_t_1 == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "cannot pass None into a C function argument that is declared 'not None'");
    __PYX_ERR(3, 17, __pyx_L1_error)
  }
  __pyx_t_2 = __pyx_f_4cupy_4cuda_6memory___pyx_unpickle_ManagedMemory__set_state(__pyx_v_self, ((PyObject*)__pyx_t_1)); if (unlikely(!__pyx_t_2)) __PYX_ERR(3, 17, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "(tree fragment)":16
 *     else:
 *         return __pyx_unpickle_ManagedMemory, (type(self), 0x61cadb0, state)
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     __pyx_unpickle_ManagedMemory__set_state(self, __pyx_state)
*/

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_AddTraceback("cupy.cuda.memory.ManagedMemory.__setstate_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "cupy/cuda/memory.pyx":292
 *     """
 * 
 *     def __init__(self, size_t size):             # <<<<<<<<<<<<<<
 *         self.size = size
 *         # TODO(leofang): using the GPU id may not be ideal, but setting it
*/

/* Python wrapper */
static int __pyx_pw_4cupy_4cuda_6memory_12SystemMemory_1__init__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static int __pyx_pw_4cupy_4cuda_6memory_12SystemMemory_1__init__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  size_t __pyx_v_size;
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject* values[1] = {0};
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__init__ (wrapper)", 0);
  #if CYTHON_ASSUME_SAFE_SIZE
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return -1;
  #endif
  __pyx_kwvalues = __Pyx_KwValues_VARARGS(__pyx_args, __pyx_nargs);
  {
    PyObject ** const __pyx_pyargnames[] = {&__pyx_mstate_global->__pyx_n_u_size,0};
    const Py_ssize_t __pyx_kwds_len = (__pyx_kwds) ? __Pyx_NumKwargs_VARARGS(__pyx_kwds) : 0;
    if (unlikely(__pyx_kwds_len < 0)) __PYX_ERR(0, 292, __pyx_L3_error)
    if (__pyx_kwds_len > 0) {
      switch (__pyx_nargs) {
        case  1:
        values[0] = __Pyx_ArgRef_VARARGS(__pyx_args, 0);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(0, 292, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      const Py_ssize_t kwd_pos_args = __pyx_nargs;
      if (__Pyx_ParseKeywords(__pyx_kwds, __pyx_kwvalues, __pyx_pyargnames, 0, values, kwd_pos_args, __pyx_kwds_len, "__init__", 0) < (0)) __PYX_ERR(0, 292, __pyx_L3_error)
      for (Py_ssize_t i = __pyx_nargs; i < 1; i++) {
        if (unlikely(!values[i])) { __Pyx_RaiseArgtupleInvalid("__init__", 1, 1, 1, i); __PYX_ERR(0, 292, __pyx_L3_error) }
      }
    } else if (unlikely(__pyx_nargs != 1)) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = __Pyx_ArgRef_VARARGS(__pyx_args, 0);
      if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(0, 292, __pyx_L3_error)
    }
    __pyx_v_size = __Pyx_PyLong_As_size_t(values[0]); if (unlikely((__pyx_v_size == (size_t)-1) && PyErr_Occurred())) __PYX_ERR(0, 292, __pyx_L3_error)
  }
  goto __pyx_L6_skip;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("__init__", 1, 1, 1, __pyx_nargs); __PYX_ERR(0, 292, __pyx_L3_error)
  __pyx_L6_skip:;
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_AddTraceback("cupy.cuda.memory.SystemMemory.__init__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return -1;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_4cupy_4cuda_6memory_12SystemMemory___init__(((struct __pyx_obj_4cupy_4cuda_6memory_SystemMemory *)__pyx_v_self), __pyx_v_size);

  /* function exit code */
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_4cupy_4cuda_6memory_12SystemMemory___init__(struct __pyx_obj_4cupy_4cuda_6memory_SystemMemory *__pyx_v_self, size_t __pyx_v_size) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  int __pyx_t_2;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__init__", 0);

  /* "cupy/cuda/memory.pyx":293
 * 
 *     def __init__(self, size_t size):
 *         self.size = size             # <<<<<<<<<<<<<<
 *         # TODO(leofang): using the GPU id may not be ideal, but setting it
 *         # to cudaCpuDeviceId (-1) would require a lot of changes
*/
  __pyx_v_self->__pyx_base.size = __pyx_v_size;

  /* "cupy/cuda/memory.pyx":296
 *         # TODO(leofang): using the GPU id may not be ideal, but setting it
 *         # to cudaCpuDeviceId (-1) would require a lot of changes
 *         self.device_id = device.get_device_id()             # <<<<<<<<<<<<<<
 *         self.ptr = 0
 *         if size > 0:
*/
  __pyx_t_1 = __pyx_f_4cupy_4cuda_6device_get_device_id(0); if (unlikely(__pyx_t_1 == ((int)-1) && PyErr_Occurred())) __PYX_ERR(0, 296, __pyx_L1_error)
  __pyx_v_self->__pyx_base.device_id = __pyx_t_1;

  /* "cupy/cuda/memory.pyx":297
 *         # to cudaCpuDeviceId (-1) would require a lot of changes
 *         self.device_id = device.get_device_id()
 *         self.ptr = 0             # <<<<<<<<<<<<<<
 *         if size > 0:
 *             self.ptr = <intptr_t>c_malloc(size)
*/
  __pyx_v_self->__pyx_base.ptr = 0;

  /* "cupy/cuda/memory.pyx":298
 *         self.device_id = device.get_device_id()
 *         self.ptr = 0
 *         if size > 0:             # <<<<<<<<<<<<<<
 *             self.ptr = <intptr_t>c_malloc(size)
 *         self._owner = None
*/
  __pyx_t_2 = (__pyx_v_size > 0);
  if (__pyx_t_2) {

    /* "cupy/cuda/memory.pyx":299
 *         self.ptr = 0
 *         if size > 0:
 *             self.ptr = <intptr_t>c_malloc(size)             # <<<<<<<<<<<<<<
 *         self._owner = None
 * 
*/
    __pyx_v_self->__pyx_base.ptr = ((intptr_t)malloc(__pyx_v_size));

    /* "cupy/cuda/memory.pyx":298
 *         self.device_id = device.get_device_id()
 *         self.ptr = 0
 *         if size > 0:             # <<<<<<<<<<<<<<
 *             self.ptr = <intptr_t>c_malloc(size)
 *         self._owner = None
*/
  }

  /* "cupy/cuda/memory.pyx":300
 *         if size > 0:
 *             self.ptr = <intptr_t>c_malloc(size)
 *         self._owner = None             # <<<<<<<<<<<<<<
 * 
 *     @staticmethod
*/
  __Pyx_INCREF(Py_None);
  __Pyx_GIVEREF(Py_None);
  __Pyx_GOTREF(__pyx_v_self->_owner);
  __Pyx_DECREF(__pyx_v_self->_owner);
  __pyx_v_self->_owner = Py_None;

  /* "cupy/cuda/memory.pyx":292
 *     """
 * 
 *     def __init__(self, size_t size):             # <<<<<<<<<<<<<<
 *         self.size = size
 *         # TODO(leofang): using the GPU id may not be ideal, but setting it
*/

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_AddTraceback("cupy.cuda.memory.SystemMemory.__init__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "cupy/cuda/memory.pyx":302
 *         self._owner = None
 * 
 *     @staticmethod             # <<<<<<<<<<<<<<
 *     cdef from_external(intptr_t ptr, size_t size, object owner):
 *         """Warp externally allocated (not owned by CuPy) system memory.
*/

static PyObject *__pyx_f_4cupy_4cuda_6memory_12SystemMemory_from_external(intptr_t __pyx_v_ptr, size_t __pyx_v_size, PyObject *__pyx_v_owner) {
  struct __pyx_obj_4cupy_4cuda_6memory_SystemMemory *__pyx_v_self = 0;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_t_2;
  int __pyx_t_3;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("from_external", 0);

  /* "cupy/cuda/memory.pyx":312
 *                 alive.
 *         """
 *         cdef SystemMemory self = SystemMemory.__new__(SystemMemory)             # <<<<<<<<<<<<<<
 *         self.size = size
 *         # TODO(leofang): using the GPU id may not be ideal, but setting it
*/
  __pyx_t_1 = ((PyObject *)__pyx_tp_new_4cupy_4cuda_6memory_SystemMemory(((PyTypeObject *)__pyx_mstate_global->__pyx_ptype_4cupy_4cuda_6memory_SystemMemory), __pyx_mstate_global->__pyx_empty_tuple, NULL)); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 312, __pyx_L1_error)
  __Pyx_GOTREF((PyObject *)__pyx_t_1);
  __pyx_v_self = ((struct __pyx_obj_4cupy_4cuda_6memory_SystemMemory *)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "cupy/cuda/memory.pyx":313
 *         """
 *         cdef SystemMemory self = SystemMemory.__new__(SystemMemory)
 *         self.size = size             # <<<<<<<<<<<<<<
 *         # TODO(leofang): using the GPU id may not be ideal, but setting it
 *         # to cudaCpuDeviceId (-1) would require a lot of changes
*/
  __pyx_v_self->__pyx_base.size = __pyx_v_size;

  /* "cupy/cuda/memory.pyx":316
 *         # TODO(leofang): using the GPU id may not be ideal, but setting it
 *         # to cudaCpuDeviceId (-1) would require a lot of changes
 *         self.device_id = device.get_device_id()             # <<<<<<<<<<<<<<
 *         self.ptr = ptr
 *         assert owner is not None, 'must provide an owner'
*/
  __pyx_t_2 = __pyx_f_4cupy_4cuda_6device_get_device_id(0); if (unlikely(__pyx_t_2 == ((int)-1) && PyErr_Occurred())) __PYX_ERR(0, 316, __pyx_L1_error)
  __pyx_v_self->__pyx_base.device_id = __pyx_t_2;

  /* "cupy/cuda/memory.pyx":317
 *         # to cudaCpuDeviceId (-1) would require a lot of changes
 *         self.device_id = device.get_device_id()
 *         self.ptr = ptr             # <<<<<<<<<<<<<<
 *         assert owner is not None, 'must provide an owner'
 *         self._owner = owner
*/
  __pyx_v_self->__pyx_base.ptr = __pyx_v_ptr;

  /* "cupy/cuda/memory.pyx":318
 *         self.device_id = device.get_device_id()
 *         self.ptr = ptr
 *         assert owner is not None, 'must provide an owner'             # <<<<<<<<<<<<<<
 *         self._owner = owner
 * 
*/
  #ifndef CYTHON_WITHOUT_ASSERTIONS
  if (unlikely(__pyx_assertions_enabled())) {
    __pyx_t_3 = (__pyx_v_owner != Py_None);
    if (unlikely(!__pyx_t_3)) {
      __Pyx_Raise(((PyObject *)(((PyTypeObject*)PyExc_AssertionError))), __pyx_mstate_global->__pyx_kp_u_must_provide_an_owner, 0, 0);
      __PYX_ERR(0, 318, __pyx_L1_error)
    }
  }
  #else
  if ((1)); else __PYX_ERR(0, 318, __pyx_L1_error)
  #endif

  /* "cupy/cuda/memory.pyx":319
 *         self.ptr = ptr
 *         assert owner is not None, 'must provide an owner'
 *         self._owner = owner             # <<<<<<<<<<<<<<
 * 
 *         return self
*/
  __Pyx_INCREF(__pyx_v_owner);
  __Pyx_GIVEREF(__pyx_v_owner);
  __Pyx_GOTREF(__pyx_v_self->_owner);
  __Pyx_DECREF(__pyx_v_self->_owner);
  __pyx_v_self->_owner = __pyx_v_owner;

  /* "cupy/cuda/memory.pyx":321
 *         self._owner = owner
 * 
 *         return self             # <<<<<<<<<<<<<<
 * 
 *     def prefetch(self, stream, *, int device_id=runtime.cudaInvalidDeviceId):
*/
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF((PyObject *)__pyx_v_self);
  __pyx_r = ((PyObject *)__pyx_v_self);
  goto __pyx_L0;

  /* "cupy/cuda/memory.pyx":302
 *         self._owner = None
 * 
 *     @staticmethod             # <<<<<<<<<<<<<<
 *     cdef from_external(intptr_t ptr, size_t size, object owner):
 *         """Warp externally allocated (not owned by CuPy) system memory.
*/

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("cupy.cuda.memory.SystemMemory.from_external", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XDECREF((PyObject *)__pyx_v_self);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "cupy/cuda/memory.pyx":323
 *         return self
 * 
 *     def prefetch(self, stream, *, int device_id=runtime.cudaInvalidDeviceId):             # <<<<<<<<<<<<<<
 *         """Prefetch memory.
 * 
*/

/* Python wrapper */
static PyObject *__pyx_pw_4cupy_4cuda_6memory_12SystemMemory_3prefetch(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
PyDoc_STRVAR(__pyx_doc_4cupy_4cuda_6memory_12SystemMemory_2prefetch, "SystemMemory.prefetch(self, stream, *, int device_id=runtime.cudaInvalidDeviceId)\n\nPrefetch memory.\n\nArgs:\n    stream (cupy.cuda.Stream): CUDA stream.\n    device_id (int): CUDA device ID (-1 for CPU).");
static PyObject *__pyx_pw_4cupy_4cuda_6memory_12SystemMemory_3prefetch(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
) {
  PyObject *__pyx_v_stream = 0;
  int __pyx_v_device_id;
  #if !CYTHON_METH_FASTCALL
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  #endif
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject* values[2] = {0,0};
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("prefetch (wrapper)", 0);
  #if !CYTHON_METH_FASTCALL
  #if CYTHON_ASSUME_SAFE_SIZE
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return NULL;
  #endif
  #endif
  __pyx_kwvalues = __Pyx_KwValues_FASTCALL(__pyx_args, __pyx_nargs);
  {
    PyObject ** const __pyx_pyargnames[] = {&__pyx_mstate_global->__pyx_n_u_stream,&__pyx_mstate_global->__pyx_n_u_device_id,0};
    const Py_ssize_t __pyx_kwds_len = (__pyx_kwds) ? __Pyx_NumKwargs_FASTCALL(__pyx_kwds) : 0;
    if (unlikely(__pyx_kwds_len < 0)) __PYX_ERR(0, 323, __pyx_L3_error)
    if (__pyx_kwds_len > 0) {
      switch (__pyx_nargs) {
        case  1:
        values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(0, 323, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      const Py_ssize_t kwd_pos_args = __pyx_nargs;
      if (__Pyx_ParseKeywords(__pyx_kwds, __pyx_kwvalues, __pyx_pyargnames, 0, values, kwd_pos_args, __pyx_kwds_len, "prefetch", 0) < (0)) __PYX_ERR(0, 323, __pyx_L3_error)
      for (Py_ssize_t i = __pyx_nargs; i < 1; i++) {
        if (unlikely(!values[i])) { __Pyx_RaiseArgtupleInvalid("prefetch", 1, 1, 1, i); __PYX_ERR(0, 323, __pyx_L3_error) }
      }
    } else if (unlikely(__pyx_nargs != 1)) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
      if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(0, 323, __pyx_L3_error)
    }
    __pyx_v_stream = values[0];
    if (values[1]) {
      __pyx_v_device_id = __Pyx_PyLong_As_int(values[1]); if (unlikely((__pyx_v_device_id == (int)-1) && PyErr_Occurred())) __PYX_ERR(0, 323, __pyx_L3_error)
    } else {
      __pyx_v_device_id = __pyx_mstate_global->__pyx_k__2;
    }
  }
  goto __pyx_L6_skip;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("prefetch", 1, 1, 1, __pyx_nargs); __PYX_ERR(0, 323, __pyx_L3_error)
  __pyx_L6_skip:;
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_AddTraceback("cupy.cuda.memory.SystemMemory.prefetch", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_4cupy_4cuda_6memory_12SystemMemory_2prefetch(((struct __pyx_obj_4cupy_4cuda_6memory_SystemMemory *)__pyx_v_self), __pyx_v_stream, __pyx_v_device_id);

  /* function exit code */
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4cupy_4cuda_6memory_12SystemMemory_2prefetch(struct __pyx_obj_4cupy_4cuda_6memory_SystemMemory *__pyx_v_self, PyObject *__pyx_v_stream, int __pyx_v_device_id) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  int __pyx_t_2;
  PyObject *__pyx_t_3 = NULL;
  intptr_t __pyx_t_4;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("prefetch", 0);

  /* "cupy/cuda/memory.pyx":330
 *             device_id (int): CUDA device ID (-1 for CPU).
 *         """
 *         if device_id == runtime.cudaInvalidDeviceId:             # <<<<<<<<<<<<<<
 *             device_id = self.device_id
 *         runtime.memPrefetchAsync(self.ptr, self.size, device_id, stream.ptr)
*/
  __pyx_t_1 = (__pyx_v_device_id == __pyx_e_13cupy_backends_4cuda_3api_13_runtime_enum_cudaInvalidDeviceId);
  if (__pyx_t_1) {

    /* "cupy/cuda/memory.pyx":331
 *         """
 *         if device_id == runtime.cudaInvalidDeviceId:
 *             device_id = self.device_id             # <<<<<<<<<<<<<<
 *         runtime.memPrefetchAsync(self.ptr, self.size, device_id, stream.ptr)
 * 
*/
    __pyx_t_2 = __pyx_v_self->__pyx_base.device_id;
    __pyx_v_device_id = __pyx_t_2;

    /* "cupy/cuda/memory.pyx":330
 *             device_id (int): CUDA device ID (-1 for CPU).
 *         """
 *         if device_id == runtime.cudaInvalidDeviceId:             # <<<<<<<<<<<<<<
 *             device_id = self.device_id
 *         runtime.memPrefetchAsync(self.ptr, self.size, device_id, stream.ptr)
*/
  }

  /* "cupy/cuda/memory.pyx":332
 *         if device_id == runtime.cudaInvalidDeviceId:
 *             device_id = self.device_id
 *         runtime.memPrefetchAsync(self.ptr, self.size, device_id, stream.ptr)             # <<<<<<<<<<<<<<
 * 
 *     def advise(self, int advise, device.Device dev):
*/
  __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_v_stream, __pyx_mstate_global->__pyx_n_u_ptr); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 332, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_4 = PyLong_AsSsize_t(__pyx_t_3); if (unlikely((__pyx_t_4 == ((intptr_t)-1)) && PyErr_Occurred())) __PYX_ERR(0, 332, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_t_3 = __pyx_f_13cupy_backends_4cuda_3api_7runtime_memPrefetchAsync(__pyx_v_self->__pyx_base.ptr, __pyx_v_self->__pyx_base.size, __pyx_v_device_id, __pyx_t_4, 0); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 332, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "cupy/cuda/memory.pyx":323
 *         return self
 * 
 *     def prefetch(self, stream, *, int device_id=runtime.cudaInvalidDeviceId):             # <<<<<<<<<<<<<<
 *         """Prefetch memory.
 * 
*/

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_AddTraceback("cupy.cuda.memory.SystemMemory.prefetch", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "cupy/cuda/memory.pyx":334
 *         runtime.memPrefetchAsync(self.ptr, self.size, device_id, stream.ptr)
 * 
 *     def advise(self, int advise, device.Device dev):             # <<<<<<<<<<<<<<
 *         """Advise about the usage of this memory.
 * 
*/

/* Python wrapper */
static PyObject *__pyx_pw_4cupy_4cuda_6memory_12SystemMemory_5advise(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
PyDoc_STRVAR(__pyx_doc_4cupy_4cuda_6memory_12SystemMemory_4advise, "SystemMemory.advise(self, int advise, Device dev)\n\nAdvise about the usage of this memory.\n\nArgs:\n    advics (int): Advise to be applied for this memory.\n    dev (cupy.cuda.Device): Device to apply the advice for.");
static PyObject *__pyx_pw_4cupy_4cuda_6memory_12SystemMemory_5advise(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
) {
  int __pyx_v_advise;
  struct __pyx_obj_4cupy_4cuda_6device_Device *__pyx_v_dev = 0;
  #if !CYTHON_METH_FASTCALL
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  #endif
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject* values[2] = {0,0};
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("advise (wrapper)", 0);
  #if !CYTHON_METH_FASTCALL
  #if CYTHON_ASSUME_SAFE_SIZE
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return NULL;
  #endif
  #endif
  __pyx_kwvalues = __Pyx_KwValues_FASTCALL(__pyx_args, __pyx_nargs);
  {
    PyObject ** const __pyx_pyargnames[] = {&__pyx_mstate_global->__pyx_n_u_advise,&__pyx_mstate_global->__pyx_n_u_dev,0};
    const Py_ssize_t __pyx_kwds_len = (__pyx_kwds) ? __Pyx_NumKwargs_FASTCALL(__pyx_kwds) : 0;
    if (unlikely(__pyx_kwds_len < 0)) __PYX_ERR(0, 334, __pyx_L3_error)
    if (__pyx_kwds_len > 0) {
      switch (__pyx_nargs) {
        case  2:
        values[1] = __Pyx_ArgRef_FASTCALL(__pyx_args, 1);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[1])) __PYX_ERR(0, 334, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  1:
        values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(0, 334, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      const Py_ssize_t kwd_pos_args = __pyx_nargs;
      if (__Pyx_ParseKeywords(__pyx_kwds, __pyx_kwvalues, __pyx_pyargnames, 0, values, kwd_pos_args, __pyx_kwds_len, "advise", 0) < (0)) __PYX_ERR(0, 334, __pyx_L3_error)
      for (Py_ssize_t i = __pyx_nargs; i < 2; i++) {
        if (unlikely(!values[i])) { __Pyx_RaiseArgtupleInvalid("advise", 1, 2, 2, i); __PYX_ERR(0, 334, __pyx_L3_error) }
      }
    } else if (unlikely(__pyx_nargs != 2)) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
      if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(0, 334, __pyx_L3_error)
      values[1] = __Pyx_ArgRef_FASTCALL(__pyx_args, 1);
      if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[1])) __PYX_ERR(0, 334, __pyx_L3_error)
    }
    __pyx_v_advise = __Pyx_PyLong_As_int(values[0]); if (unlikely((__pyx_v_advise == (int)-1) && PyErr_Occurred())) __PYX_ERR(0, 334, __pyx_L3_error)
    __pyx_v_dev = ((struct __pyx_obj_4cupy_4cuda_6device_Device *)values[1]);
  }
  goto __pyx_L6_skip;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("advise", 1, 2, 2, __pyx_nargs); __PYX_ERR(0, 334, __pyx_L3_error)
  __pyx_L6_skip:;
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_AddTraceback("cupy.cuda.memory.SystemMemory.advise", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_dev), __pyx_mstate_global->__pyx_ptype_4cupy_4cuda_6device_Device, 1, "dev", 0))) __PYX_ERR(0, 334, __pyx_L1_error)
  __pyx_r = __pyx_pf_4cupy_4cuda_6memory_12SystemMemory_4advise(((struct __pyx_obj_4cupy_4cuda_6memory_SystemMemory *)__pyx_v_self), __pyx_v_advise, __pyx_v_dev);

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = NULL;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  goto __pyx_L7_cleaned_up;
  __pyx_L0:;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __pyx_L7_cleaned_up:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4cupy_4cuda_6memory_12SystemMemory_4advise(struct __pyx_obj_4cupy_4cuda_6memory_SystemMemory *__pyx_v_self, int __pyx_v_advise, struct __pyx_obj_4cupy_4cuda_6device_Device *__pyx_v_dev) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("advise", 0);

  /* "cupy/cuda/memory.pyx":343
 *         """
 *         # TODO(leofang): switch to cudaMemAdvice_v2 from CUDA 12.2
 *         runtime.memAdvise(self.ptr, self.size, advise, dev.id)             # <<<<<<<<<<<<<<
 * 
 *     def __dealloc__(self):
*/
  __pyx_t_1 = __pyx_f_13cupy_backends_4cuda_3api_7runtime_memAdvise(__pyx_v_self->__pyx_base.ptr, __pyx_v_self->__pyx_base.size, __pyx_v_advise, __pyx_v_dev->id, 0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 343, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "cupy/cuda/memory.pyx":334
 *         runtime.memPrefetchAsync(self.ptr, self.size, device_id, stream.ptr)
 * 
 *     def advise(self, int advise, device.Device dev):             # <<<<<<<<<<<<<<
 *         """Advise about the usage of this memory.
 * 
*/

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("cupy.cuda.memory.SystemMemory.advise", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "cupy/cuda/memory.pyx":345
 *         runtime.memAdvise(self.ptr, self.size, advise, dev.id)
 * 
 *     def __dealloc__(self):             # <<<<<<<<<<<<<<
 *         # Note: Cannot raise in the destructor! (cython/cython#1613)
 *         if self._owner is not None:
*/

/* Python wrapper */
static void __pyx_pw_4cupy_4cuda_6memory_12SystemMemory_7__dealloc__(PyObject *__pyx_v_self); /*proto*/
static void __pyx_pw_4cupy_4cuda_6memory_12SystemMemory_7__dealloc__(PyObject *__pyx_v_self) {
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__dealloc__ (wrapper)", 0);
  __pyx_kwvalues = __Pyx_KwValues_VARARGS(__pyx_args, __pyx_nargs);
  __pyx_pf_4cupy_4cuda_6memory_12SystemMemory_6__dealloc__(((struct __pyx_obj_4cupy_4cuda_6memory_SystemMemory *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
}

static void __pyx_pf_4cupy_4cuda_6memory_12SystemMemory_6__dealloc__(struct __pyx_obj_4cupy_4cuda_6memory_SystemMemory *__pyx_v_self) {
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  intptr_t __pyx_t_2;
  PyObject *__pyx_t_3 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__dealloc__", 0);

  /* "cupy/cuda/memory.pyx":347
 *     def __dealloc__(self):
 *         # Note: Cannot raise in the destructor! (cython/cython#1613)
 *         if self._owner is not None:             # <<<<<<<<<<<<<<
 *             # if we don't own the memory, we must sync before free to avoid
 *             # any race condition
*/
  __pyx_t_1 = (__pyx_v_self->_owner != Py_None);
  if (__pyx_t_1) {

    /* "cupy/cuda/memory.pyx":350
 *             # if we don't own the memory, we must sync before free to avoid
 *             # any race condition
 *             runtime.streamSynchronize(stream_module.get_current_stream_ptr())             # <<<<<<<<<<<<<<
 *         elif self.ptr:
 *             # we don't need to sync because we assume SystemMemory is allocated
*/
    __pyx_t_2 = __pyx_f_4cupy_4cuda_6stream_get_current_stream_ptr(); if (unlikely(__pyx_t_2 == ((intptr_t)-1L) && PyErr_Occurred())) __PYX_ERR(0, 350, __pyx_L1_error)
    __pyx_t_3 = __pyx_f_13cupy_backends_4cuda_3api_7runtime_streamSynchronize(__pyx_t_2, 0); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 350, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

    /* "cupy/cuda/memory.pyx":347
 *     def __dealloc__(self):
 *         # Note: Cannot raise in the destructor! (cython/cython#1613)
 *         if self._owner is not None:             # <<<<<<<<<<<<<<
 *             # if we don't own the memory, we must sync before free to avoid
 *             # any race condition
*/
    goto __pyx_L3;
  }

  /* "cupy/cuda/memory.pyx":351
 *             # any race condition
 *             runtime.streamSynchronize(stream_module.get_current_stream_ptr())
 *         elif self.ptr:             # <<<<<<<<<<<<<<
 *             # we don't need to sync because we assume SystemMemory is allocated
 *             # and protected by the (stream-ordered) memory pool
*/
  __pyx_t_1 = (__pyx_v_self->__pyx_base.ptr != 0);
  if (__pyx_t_1) {

    /* "cupy/cuda/memory.pyx":354
 *             # we don't need to sync because we assume SystemMemory is allocated
 *             # and protected by the (stream-ordered) memory pool
 *             c_free(<void*>self.ptr)             # <<<<<<<<<<<<<<
 * 
 * 
*/
    free(((void *)__pyx_v_self->__pyx_base.ptr));

    /* "cupy/cuda/memory.pyx":351
 *             # any race condition
 *             runtime.streamSynchronize(stream_module.get_current_stream_ptr())
 *         elif self.ptr:             # <<<<<<<<<<<<<<
 *             # we don't need to sync because we assume SystemMemory is allocated
 *             # and protected by the (stream-ordered) memory pool
*/
  }
  __pyx_L3:;

  /* "cupy/cuda/memory.pyx":345
 *         runtime.memAdvise(self.ptr, self.size, advise, dev.id)
 * 
 *     def __dealloc__(self):             # <<<<<<<<<<<<<<
 *         # Note: Cannot raise in the destructor! (cython/cython#1613)
 *         if self._owner is not None:
*/

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_WriteUnraisable("cupy.cuda.memory.SystemMemory.__dealloc__", __pyx_clineno, __pyx_lineno, __pyx_filename, 1, 0);
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
}

/* "cupy/cuda/memory.pxd":23
 * 
 *     cdef:
 *         readonly object _owner             # <<<<<<<<<<<<<<
 * 
 *     @staticmethod
*/

/* Python wrapper */
static PyObject *__pyx_pw_4cupy_4cuda_6memory_12SystemMemory_6_owner_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_4cupy_4cuda_6memory_12SystemMemory_6_owner_1__get__(PyObject *__pyx_v_self) {
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_kwvalues = __Pyx_KwValues_VARARGS(__pyx_args, __pyx_nargs);
  __pyx_r = __pyx_pf_4cupy_4cuda_6memory_12SystemMemory_6_owner___get__(((struct __pyx_obj_4cupy_4cuda_6memory_SystemMemory *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4cupy_4cuda_6memory_12SystemMemory_6_owner___get__(struct __pyx_obj_4cupy_4cuda_6memory_SystemMemory *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__", 0);
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_v_self->_owner);
  __pyx_r = __pyx_v_self->_owner;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     cdef tuple state
 *     cdef object _dict
*/

/* Python wrapper */
static PyObject *__pyx_pw_4cupy_4cuda_6memory_12SystemMemory_9__reduce_cython__(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
PyDoc_STRVAR(__pyx_doc_4cupy_4cuda_6memory_12SystemMemory_8__reduce_cython__, "SystemMemory.__reduce_cython__(self)");
static PyObject *__pyx_pw_4cupy_4cuda_6memory_12SystemMemory_9__reduce_cython__(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
) {
  #if !CYTHON_METH_FASTCALL
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  #endif
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__reduce_cython__ (wrapper)", 0);
  #if !CYTHON_METH_FASTCALL
  #if CYTHON_ASSUME_SAFE_SIZE
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return NULL;
  #endif
  #endif
  __pyx_kwvalues = __Pyx_KwValues_FASTCALL(__pyx_args, __pyx_nargs);
  if (unlikely(__pyx_nargs > 0)) { __Pyx_RaiseArgtupleInvalid("__reduce_cython__", 1, 0, 0, __pyx_nargs); return NULL; }
  const Py_ssize_t __pyx_kwds_len = unlikely(__pyx_kwds) ? __Pyx_NumKwargs_FASTCALL(__pyx_kwds) : 0;
  if (unlikely(__pyx_kwds_len < 0)) return NULL;
  if (unlikely(__pyx_kwds_len > 0)) {__Pyx_RejectKeywords("__reduce_cython__", __pyx_kwds); return NULL;}
  __pyx_r = __pyx_pf_4cupy_4cuda_6memory_12SystemMemory_8__reduce_cython__(((struct __pyx_obj_4cupy_4cuda_6memory_SystemMemory *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4cupy_4cuda_6memory_12SystemMemory_8__reduce_cython__(struct __pyx_obj_4cupy_4cuda_6memory_SystemMemory *__pyx_v_self) {
  PyObject *__pyx_v_state = 0;
  PyObject *__pyx_v__dict = 0;
  int __pyx_v_use_setstate;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  int __pyx_t_5;
  int __pyx_t_6;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__reduce_cython__", 0);

  /* "(tree fragment)":5
 *     cdef object _dict
 *     cdef bint use_setstate
 *     state = (self._owner, self.device_id, self.ptr, self.size)             # <<<<<<<<<<<<<<
 *     _dict = getattr(self, '__dict__', None)
 *     if _dict is not None and _dict:
*/
  __pyx_t_1 = __Pyx_PyLong_From_int(__pyx_v_self->__pyx_base.device_id); if (unlikely(!__pyx_t_1)) __PYX_ERR(3, 5, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = PyLong_FromSsize_t(__pyx_v_self->__pyx_base.ptr); if (unlikely(!__pyx_t_2)) __PYX_ERR(3, 5, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = __Pyx_PyLong_FromSize_t(__pyx_v_self->__pyx_base.size); if (unlikely(!__pyx_t_3)) __PYX_ERR(3, 5, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_4 = PyTuple_New(4); if (unlikely(!__pyx_t_4)) __PYX_ERR(3, 5, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_INCREF(__pyx_v_self->_owner);
  __Pyx_GIVEREF(__pyx_v_self->_owner);
  if (__Pyx_PyTuple_SET_ITEM(__pyx_t_4, 0, __pyx_v_self->_owner) != (0)) __PYX_ERR(3, 5, __pyx_L1_error);
  __Pyx_GIVEREF(__pyx_t_1);
  if (__Pyx_PyTuple_SET_ITEM(__pyx_t_4, 1, __pyx_t_1) != (0)) __PYX_ERR(3, 5, __pyx_L1_error);
  __Pyx_GIVEREF(__pyx_t_2);
  if (__Pyx_PyTuple_SET_ITEM(__pyx_t_4, 2, __pyx_t_2) != (0)) __PYX_ERR(3, 5, __pyx_L1_error);
  __Pyx_GIVEREF(__pyx_t_3);
  if (__Pyx_PyTuple_SET_ITEM(__pyx_t_4, 3, __pyx_t_3) != (0)) __PYX_ERR(3, 5, __pyx_L1_error);
  __pyx_t_1 = 0;
  __pyx_t_2 = 0;
  __pyx_t_3 = 0;
  __pyx_v_state = ((PyObject*)__pyx_t_4);
  __pyx_t_4 = 0;

  /* "(tree fragment)":6
 *     cdef bint use_setstate
 *     state = (self._owner, self.device_id, self.ptr, self.size)
 *     _dict = getattr(self, '__dict__', None)             # <<<<<<<<<<<<<<
 *     if _dict is not None and _dict:
 *         state += (_dict,)
*/
  __pyx_t_4 = __Pyx_GetAttr3(((PyObject *)__pyx_v_self), __pyx_mstate_global->__pyx_n_u_dict, Py_None); if (unlikely(!__pyx_t_4)) __PYX_ERR(3, 6, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_v__dict = __pyx_t_4;
  __pyx_t_4 = 0;

  /* "(tree fragment)":7
 *     state = (self._owner, self.device_id, self.ptr, self.size)
 *     _dict = getattr(self, '__dict__', None)
 *     if _dict is not None and _dict:             # <<<<<<<<<<<<<<
 *         state += (_dict,)
 *         use_setstate = True
*/
  __pyx_t_6 = (__pyx_v__dict != Py_None);
  if (__pyx_t_6) {
  } else {
    __pyx_t_5 = __pyx_t_6;
    goto __pyx_L4_bool_binop_done;
  }
  __pyx_t_6 = __Pyx_PyObject_IsTrue(__pyx_v__dict); if (unlikely((__pyx_t_6 < 0))) __PYX_ERR(3, 7, __pyx_L1_error)
  __pyx_t_5 = __pyx_t_6;
  __pyx_L4_bool_binop_done:;
  if (__pyx_t_5) {

    /* "(tree fragment)":8
 *     _dict = getattr(self, '__dict__', None)
 *     if _dict is not None and _dict:
 *         state += (_dict,)             # <<<<<<<<<<<<<<
 *         use_setstate = True
 *     else:
*/
    __pyx_t_4 = PyTuple_New(1); if (unlikely(!__pyx_t_4)) __PYX_ERR(3, 8, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_INCREF(__pyx_v__dict);
    __Pyx_GIVEREF(__pyx_v__dict);
    if (__Pyx_PyTuple_SET_ITEM(__pyx_t_4, 0, __pyx_v__dict) != (0)) __PYX_ERR(3, 8, __pyx_L1_error);
    __pyx_t_3 = PyNumber_InPlaceAdd(__pyx_v_state, __pyx_t_4); if (unlikely(!__pyx_t_3)) __PYX_ERR(3, 8, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __Pyx_DECREF_SET(__pyx_v_state, ((PyObject*)__pyx_t_3));
    __pyx_t_3 = 0;

    /* "(tree fragment)":9
 *     if _dict is not None and _dict:
 *         state += (_dict,)
 *         use_setstate = True             # <<<<<<<<<<<<<<
 *     else:
 *         use_setstate = self._owner is not None
*/
    __pyx_v_use_setstate = 1;

    /* "(tree fragment)":7
 *     state = (self._owner, self.device_id, self.ptr, self.size)
 *     _dict = getattr(self, '__dict__', None)
 *     if _dict is not None and _dict:             # <<<<<<<<<<<<<<
 *         state += (_dict,)
 *         use_setstate = True
*/
    goto __pyx_L3;
  }

  /* "(tree fragment)":11
 *         use_setstate = True
 *     else:
 *         use_setstate = self._owner is not None             # <<<<<<<<<<<<<<
 *     if use_setstate:
 *         return __pyx_unpickle_SystemMemory, (type(self), 0xa9b59d9, None), state
*/
  /*else*/ {
    __pyx_t_5 = (__pyx_v_self->_owner != Py_None);
    __pyx_v_use_setstate = __pyx_t_5;
  }
  __pyx_L3:;

  /* "(tree fragment)":12
 *     else:
 *         use_setstate = self._owner is not None
 *     if use_setstate:             # <<<<<<<<<<<<<<
 *         return __pyx_unpickle_SystemMemory, (type(self), 0xa9b59d9, None), state
 *     else:
*/
  if (__pyx_v_use_setstate) {

    /* "(tree fragment)":13
 *         use_setstate = self._owner is not None
 *     if use_setstate:
 *         return __pyx_unpickle_SystemMemory, (type(self), 0xa9b59d9, None), state             # <<<<<<<<<<<<<<
 *     else:
 *         return __pyx_unpickle_SystemMemory, (type(self), 0xa9b59d9, state)
*/
    __Pyx_XDECREF(__pyx_r);
    __Pyx_GetModuleGlobalName(__pyx_t_3, __pyx_mstate_global->__pyx_n_u_pyx_unpickle_SystemMemory); if (unlikely(!__pyx_t_3)) __PYX_ERR(3, 13, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_t_4 = PyTuple_New(3); if (unlikely(!__pyx_t_4)) __PYX_ERR(3, 13, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_INCREF(((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    __Pyx_GIVEREF(((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    if (__Pyx_PyTuple_SET_ITEM(__pyx_t_4, 0, ((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self)))) != (0)) __PYX_ERR(3, 13, __pyx_L1_error);
    __Pyx_INCREF(__pyx_mstate_global->__pyx_int_177953241);
    __Pyx_GIVEREF(__pyx_mstate_global->__pyx_int_177953241);
    if (__Pyx_PyTuple_SET_ITEM(__pyx_t_4, 1, __pyx_mstate_global->__pyx_int_177953241) != (0)) __PYX_ERR(3, 13, __pyx_L1_error);
    __Pyx_INCREF(Py_None);
    __Pyx_GIVEREF(Py_None);
    if (__Pyx_PyTuple_SET_ITEM(__pyx_t_4, 2, Py_None) != (0)) __PYX_ERR(3, 13, __pyx_L1_error);
    __pyx_t_2 = PyTuple_New(3); if (unlikely(!__pyx_t_2)) __PYX_ERR(3, 13, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_GIVEREF(__pyx_t_3);
    if (__Pyx_PyTuple_SET_ITEM(__pyx_t_2, 0, __pyx_t_3) != (0)) __PYX_ERR(3, 13, __pyx_L1_error);
    __Pyx_GIVEREF(__pyx_t_4);
    if (__Pyx_PyTuple_SET_ITEM(__pyx_t_2, 1, __pyx_t_4) != (0)) __PYX_ERR(3, 13, __pyx_L1_error);
    __Pyx_INCREF(__pyx_v_state);
    __Pyx_GIVEREF(__pyx_v_state);
    if (__Pyx_PyTuple_SET_ITEM(__pyx_t_2, 2, __pyx_v_state) != (0)) __PYX_ERR(3, 13, __pyx_L1_error);
    __pyx_t_3 = 0;
    __pyx_t_4 = 0;
    __pyx_r = __pyx_t_2;
    __pyx_t_2 = 0;
    goto __pyx_L0;

    /* "(tree fragment)":12
 *     else:
 *         use_setstate = self._owner is not None
 *     if use_setstate:             # <<<<<<<<<<<<<<
 *         return __pyx_unpickle_SystemMemory, (type(self), 0xa9b59d9, None), state
 *     else:
*/
  }

  /* "(tree fragment)":15
 *         return __pyx_unpickle_SystemMemory, (type(self), 0xa9b59d9, None), state
 *     else:
 *         return __pyx_unpickle_SystemMemory, (type(self), 0xa9b59d9, state)             # <<<<<<<<<<<<<<
 * def __setstate_cython__(self, __pyx_state):
 *     __pyx_unpickle_SystemMemory__set_state(self, __pyx_state)
*/
  /*else*/ {
    __Pyx_XDECREF(__pyx_r);
    __Pyx_GetModuleGlobalName(__pyx_t_2, __pyx_mstate_global->__pyx_n_u_pyx_unpickle_SystemMemory); if (unlikely(!__pyx_t_2)) __PYX_ERR(3, 15, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_4 = PyTuple_New(3); if (unlikely(!__pyx_t_4)) __PYX_ERR(3, 15, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_INCREF(((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    __Pyx_GIVEREF(((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    if (__Pyx_PyTuple_SET_ITEM(__pyx_t_4, 0, ((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self)))) != (0)) __PYX_ERR(3, 15, __pyx_L1_error);
    __Pyx_INCREF(__pyx_mstate_global->__pyx_int_177953241);
    __Pyx_GIVEREF(__pyx_mstate_global->__pyx_int_177953241);
    if (__Pyx_PyTuple_SET_ITEM(__pyx_t_4, 1, __pyx_mstate_global->__pyx_int_177953241) != (0)) __PYX_ERR(3, 15, __pyx_L1_error);
    __Pyx_INCREF(__pyx_v_state);
    __Pyx_GIVEREF(__pyx_v_state);
    if (__Pyx_PyTuple_SET_ITEM(__pyx_t_4, 2, __pyx_v_state) != (0)) __PYX_ERR(3, 15, __pyx_L1_error);
    __pyx_t_3 = PyTuple_New(2); if (unlikely(!__pyx_t_3)) __PYX_ERR(3, 15, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_GIVEREF(__pyx_t_2);
    if (__Pyx_PyTuple_SET_ITEM(__pyx_t_3, 0, __pyx_t_2) != (0)) __PYX_ERR(3, 15, __pyx_L1_error);
    __Pyx_GIVEREF(__pyx_t_4);
    if (__Pyx_PyTuple_SET_ITEM(__pyx_t_3, 1, __pyx_t_4) != (0)) __PYX_ERR(3, 15, __pyx_L1_error);
    __pyx_t_2 = 0;
    __pyx_t_4 = 0;
    __pyx_r = __pyx_t_3;
    __pyx_t_3 = 0;
    goto __pyx_L0;
  }

  /* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     cdef tuple state
 *     cdef object _dict
*/

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_AddTraceback("cupy.cuda.memory.SystemMemory.__reduce_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_state);
  __Pyx_XDECREF(__pyx_v__dict);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":16
 *     else:
 *         return __pyx_unpickle_SystemMemory, (type(self), 0xa9b59d9, state)
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     __pyx_unpickle_SystemMemory__set_state(self, __pyx_state)
*/

/* Python wrapper */
static PyObject *__pyx_pw_4cupy_4cuda_6memory_12SystemMemory_11__setstate_cython__(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
PyDoc_STRVAR(__pyx_doc_4cupy_4cuda_6memory_12SystemMemory_10__setstate_cython__, "SystemMemory.__setstate_cython__(self, __pyx_state)");
static PyObject *__pyx_pw_4cupy_4cuda_6memory_12SystemMemory_11__setstate_cython__(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
) {
  PyObject *__pyx_v___pyx_state = 0;
  #if !CYTHON_METH_FASTCALL
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  #endif
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject* values[1] = {0};
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__setstate_cython__ (wrapper)", 0);
  #if !CYTHON_METH_FASTCALL
  #if CYTHON_ASSUME_SAFE_SIZE
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return NULL;
  #endif
  #endif
  __pyx_kwvalues = __Pyx_KwValues_FASTCALL(__pyx_args, __pyx_nargs);
  {
    PyObject ** const __pyx_pyargnames[] = {&__pyx_mstate_global->__pyx_n_u_pyx_state,0};
    const Py_ssize_t __pyx_kwds_len = (__pyx_kwds) ? __Pyx_NumKwargs_FASTCALL(__pyx_kwds) : 0;
    if (unlikely(__pyx_kwds_len < 0)) __PYX_ERR(3, 16, __pyx_L3_error)
    if (__pyx_kwds_len > 0) {
      switch (__pyx_nargs) {
        case  1:
        values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(3, 16, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      const Py_ssize_t kwd_pos_args = __pyx_nargs;
      if (__Pyx_ParseKeywords(__pyx_kwds, __pyx_kwvalues, __pyx_pyargnames, 0, values, kwd_pos_args, __pyx_kwds_len, "__setstate_cython__", 0) < (0)) __PYX_ERR(3, 16, __pyx_L3_error)
      for (Py_ssize_t i = __pyx_nargs; i < 1; i++) {
        if (unlikely(!values[i])) { __Pyx_RaiseArgtupleInvalid("__setstate_cython__", 1, 1, 1, i); __PYX_ERR(3, 16, __pyx_L3_error) }
      }
    } else if (unlikely(__pyx_nargs != 1)) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
      if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(3, 16, __pyx_L3_error)
    }
    __pyx_v___pyx_state = values[0];
  }
  goto __pyx_L6_skip;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("__setstate_cython__", 1, 1, 1, __pyx_nargs); __PYX_ERR(3, 16, __pyx_L3_error)
  __pyx_L6_skip:;
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_AddTraceback("cupy.cuda.memory.SystemMemory.__setstate_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_4cupy_4cuda_6memory_12SystemMemory_10__setstate_cython__(((struct __pyx_obj_4cupy_4cuda_6memory_SystemMemory *)__pyx_v_self), __pyx_v___pyx_state);

  /* function exit code */
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4cupy_4cuda_6memory_12SystemMemory_10__setstate_cython__(struct __pyx_obj_4cupy_4cuda_6memory_SystemMemory *__pyx_v_self, PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__setstate_cython__", 0);

  /* "(tree fragment)":17
 *         return __pyx_unpickle_SystemMemory, (type(self), 0xa9b59d9, state)
 * def __setstate_cython__(self, __pyx_state):
 *     __pyx_unpickle_SystemMemory__set_state(self, __pyx_state)             # <<<<<<<<<<<<<<
*/
  __pyx_t_1 = __pyx_v___pyx_state;
  __Pyx_INCREF(__pyx_t_1);
  if (!(likely(PyTuple_CheckExact(__pyx_t_1))||((__pyx_t_1) == Py_None) || __Pyx_RaiseUnexpectedTypeError("tuple", __pyx_t_1))) __PYX_ERR(3, 17, __pyx_L1_error)
  if (unlikely(__pyx_t_1 == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "cannot pass None into a C function argument that is declared 'not None'");
    __PYX_ERR(3, 17, __pyx_L1_error)
  }
  __pyx_t_2 = __pyx_f_4cupy_4cuda_6memory___pyx_unpickle_SystemMemory__set_state(__pyx_v_self, ((PyObject*)__pyx_t_1)); if (unlikely(!__pyx_t_2)) __PYX_ERR(3, 17, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "(tree fragment)":16
 *     else:
 *         return __pyx_unpickle_SystemMemory, (type(self), 0xa9b59d9, state)
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     __pyx_unpickle_SystemMemory__set_state(self, __pyx_state)
*/

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_AddTraceback("cupy.cuda.memory.SystemMemory.__setstate_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "cupy/cuda/memory.pyx":397
 *         public _Chunk next
 * 
 *     def __init__(self, *args):             # <<<<<<<<<<<<<<
 *         # For debug
 *         mem, offset, size, arena = args
*/

/* Python wrapper */
static int __pyx_pw_4cupy_4cuda_6memory_6_Chunk_1__init__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static int __pyx_pw_4cupy_4cuda_6memory_6_Chunk_1__init__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_args = 0;
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__init__ (wrapper)", 0);
  #if CYTHON_ASSUME_SAFE_SIZE
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return -1;
  #endif
  __pyx_kwvalues = __Pyx_KwValues_VARARGS(__pyx_args, __pyx_nargs);
  const Py_ssize_t __pyx_kwds_len = unlikely(__pyx_kwds) ? __Pyx_NumKwargs_VARARGS(__pyx_kwds) : 0;
  if (unlikely(__pyx_kwds_len < 0)) return -1;
  if (unlikely(__pyx_kwds_len > 0)) {__Pyx_RejectKeywords("__init__", __pyx_kwds); return -1;}
  __Pyx_INCREF(__pyx_args);
  __pyx_v_args = __pyx_args;
  __pyx_r = __pyx_pf_4cupy_4cuda_6memory_6_Chunk___init__(((struct __pyx_obj_4cupy_4cuda_6memory__Chunk *)__pyx_v_self), __pyx_v_args);

  /* function exit code */
  __Pyx_DECREF(__pyx_v_args);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_4cupy_4cuda_6memory_6_Chunk___init__(struct __pyx_obj_4cupy_4cuda_6memory__Chunk *__pyx_v_self, PyObject *__pyx_v_args) {
  PyObject *__pyx_v_mem = NULL;
  PyObject *__pyx_v_offset = NULL;
  PyObject *__pyx_v_size = NULL;
  PyObject *__pyx_v_arena = NULL;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  ptrdiff_t __pyx_t_5;
  size_t __pyx_t_6;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__init__", 0);

  /* "cupy/cuda/memory.pyx":399
 *     def __init__(self, *args):
 *         # For debug
 *         mem, offset, size, arena = args             # <<<<<<<<<<<<<<
 *         self._init(mem, offset, size, arena)
 * 
*/
  if (1) {
    PyObject* sequence = __pyx_v_args;
    Py_ssize_t size = __Pyx_PyTuple_GET_SIZE(sequence);
    if (unlikely(size != 4)) {
      if (size > 4) __Pyx_RaiseTooManyValuesError(4);
      else if (size >= 0) __Pyx_RaiseNeedMoreValuesError(size);
      __PYX_ERR(0, 399, __pyx_L1_error)
    }
    #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
    __pyx_t_1 = PyTuple_GET_ITEM(sequence, 0);
    __Pyx_INCREF(__pyx_t_1);
    __pyx_t_2 = PyTuple_GET_ITEM(sequence, 1);
    __Pyx_INCREF(__pyx_t_2);
    __pyx_t_3 = PyTuple_GET_ITEM(sequence, 2);
    __Pyx_INCREF(__pyx_t_3);
    __pyx_t_4 = PyTuple_GET_ITEM(sequence, 3);
    __Pyx_INCREF(__pyx_t_4);
    #else
    {
      Py_ssize_t i;
      PyObject** temps[4] = {&__pyx_t_1,&__pyx_t_2,&__pyx_t_3,&__pyx_t_4};
      for (i=0; i < 4; i++) {
        PyObject* item = __Pyx_PySequence_ITEM(sequence, i); if (unlikely(!item)) __PYX_ERR(0, 399, __pyx_L1_error)
        __Pyx_GOTREF(item);
        *(temps[i]) = item;
      }
    }
    #endif
  }
  __pyx_v_mem = __pyx_t_1;
  __pyx_t_1 = 0;
  __pyx_v_offset = __pyx_t_2;
  __pyx_t_2 = 0;
  __pyx_v_size = __pyx_t_3;
  __pyx_t_3 = 0;
  __pyx_v_arena = __pyx_t_4;
  __pyx_t_4 = 0;

  /* "cupy/cuda/memory.pyx":400
 *         # For debug
 *         mem, offset, size, arena = args
 *         self._init(mem, offset, size, arena)             # <<<<<<<<<<<<<<
 * 
 *     def __repr__(self):
*/
  if (!(likely(((__pyx_v_mem) == Py_None) || likely(__Pyx_TypeTest(__pyx_v_mem, __pyx_mstate_global->__pyx_ptype_4cupy_4cuda_6memory_BaseMemory))))) __PYX_ERR(0, 400, __pyx_L1_error)
  __pyx_t_5 = __Pyx_PyLong_As_ptrdiff_t(__pyx_v_offset); if (unlikely((__pyx_t_5 == (ptrdiff_t)-1) && PyErr_Occurred())) __PYX_ERR(0, 400, __pyx_L1_error)
  __pyx_t_6 = __Pyx_PyLong_As_size_t(__pyx_v_size); if (unlikely((__pyx_t_6 == (size_t)-1) && PyErr_Occurred())) __PYX_ERR(0, 400, __pyx_L1_error)
  if (!(likely(((__pyx_v_arena) == Py_None) || likely(__Pyx_TypeTest(__pyx_v_arena, __pyx_mstate_global->__pyx_ptype_4cupy_4cuda_6memory__Arena))))) __PYX_ERR(0, 400, __pyx_L1_error)
  __pyx_t_4 = __pyx_f_4cupy_4cuda_6memory_6_Chunk__init(__pyx_v_self, ((struct __pyx_obj_4cupy_4cuda_6memory_BaseMemory *)__pyx_v_mem), __pyx_t_5, __pyx_t_6, ((struct __pyx_obj_4cupy_4cuda_6memory__Arena *)__pyx_v_arena)); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 400, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

  /* "cupy/cuda/memory.pyx":397
 *         public _Chunk next
 * 
 *     def __init__(self, *args):             # <<<<<<<<<<<<<<
 *         # For debug
 *         mem, offset, size, arena = args
*/

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_AddTraceback("cupy.cuda.memory._Chunk.__init__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_mem);
  __Pyx_XDECREF(__pyx_v_offset);
  __Pyx_XDECREF(__pyx_v_size);
  __Pyx_XDECREF(__pyx_v_arena);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "cupy/cuda/memory.pyx":402
 *         self._init(mem, offset, size, arena)
 * 
 *     def __repr__(self):             # <<<<<<<<<<<<<<
 *         # To simplify debugging if needed.
 *         mem, offset, prev, next = self.mem, self.offset, self.prev, self.next
*/

/* Python wrapper */
static PyObject *__pyx_pw_4cupy_4cuda_6memory_6_Chunk_3__repr__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_4cupy_4cuda_6memory_6_Chunk_3__repr__(PyObject *__pyx_v_self) {
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__repr__ (wrapper)", 0);
  __pyx_kwvalues = __Pyx_KwValues_VARARGS(__pyx_args, __pyx_nargs);
  __pyx_r = __pyx_pf_4cupy_4cuda_6memory_6_Chunk_2__repr__(((struct __pyx_obj_4cupy_4cuda_6memory__Chunk *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4cupy_4cuda_6memory_6_Chunk_2__repr__(struct __pyx_obj_4cupy_4cuda_6memory__Chunk *__pyx_v_self) {
  struct __pyx_obj_4cupy_4cuda_6memory_BaseMemory *__pyx_v_mem = NULL;
  ptrdiff_t __pyx_v_offset;
  PyObject *__pyx_v_prev = NULL;
  PyObject *__pyx_v_next = NULL;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  ptrdiff_t __pyx_t_2;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  int __pyx_t_5;
  size_t __pyx_t_6;
  PyObject *__pyx_t_7 = NULL;
  PyObject *__pyx_t_8 = NULL;
  PyObject *__pyx_t_9 = NULL;
  PyObject *__pyx_t_10[13];
  PyObject *__pyx_t_11 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__repr__", 0);

  /* "cupy/cuda/memory.pyx":404
 *     def __repr__(self):
 *         # To simplify debugging if needed.
 *         mem, offset, prev, next = self.mem, self.offset, self.prev, self.next             # <<<<<<<<<<<<<<
 *         prev = "None" if prev is None else hex(id(prev))
 *         next = "None" if next is None else hex(id(next))
*/
  __pyx_t_1 = ((PyObject *)__pyx_v_self->mem);
  __Pyx_INCREF(__pyx_t_1);
  __pyx_t_2 = __pyx_v_self->offset;
  __pyx_t_3 = ((PyObject *)__pyx_v_self->prev);
  __Pyx_INCREF(__pyx_t_3);
  __pyx_t_4 = ((PyObject *)__pyx_v_self->next);
  __Pyx_INCREF(__pyx_t_4);
  __pyx_v_mem = ((struct __pyx_obj_4cupy_4cuda_6memory_BaseMemory *)__pyx_t_1);
  __pyx_t_1 = 0;
  __pyx_v_offset = __pyx_t_2;
  __pyx_v_prev = __pyx_t_3;
  __pyx_t_3 = 0;
  __pyx_v_next = __pyx_t_4;
  __pyx_t_4 = 0;

  /* "cupy/cuda/memory.pyx":405
 *         # To simplify debugging if needed.
 *         mem, offset, prev, next = self.mem, self.offset, self.prev, self.next
 *         prev = "None" if prev is None else hex(id(prev))             # <<<<<<<<<<<<<<
 *         next = "None" if next is None else hex(id(next))
 *         return (
*/
  __pyx_t_5 = (__pyx_v_prev == Py_None);
  if (__pyx_t_5) {
    __Pyx_INCREF(__pyx_mstate_global->__pyx_n_u_None);
    __pyx_t_4 = __pyx_mstate_global->__pyx_n_u_None;
  } else {
    __pyx_t_1 = NULL;
    __pyx_t_6 = 1;
    {
      PyObject *__pyx_callargs[2] = {__pyx_t_1, __pyx_v_prev};
      __pyx_t_3 = __Pyx_PyObject_FastCall((PyObject*)__pyx_builtin_id, __pyx_callargs+__pyx_t_6, (2-__pyx_t_6) | (__pyx_t_6*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
      __Pyx_XDECREF(__pyx_t_1); __pyx_t_1 = 0;
      if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 405, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_3);
    }
    __pyx_t_1 = __Pyx_PyNumber_Hex(__pyx_t_3); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 405, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __pyx_t_4 = __pyx_t_1;
    __pyx_t_1 = 0;
  }
  __Pyx_DECREF_SET(__pyx_v_prev, __pyx_t_4);
  __pyx_t_4 = 0;

  /* "cupy/cuda/memory.pyx":406
 *         mem, offset, prev, next = self.mem, self.offset, self.prev, self.next
 *         prev = "None" if prev is None else hex(id(prev))
 *         next = "None" if next is None else hex(id(next))             # <<<<<<<<<<<<<<
 *         return (
 *             f"<{type(self)} at {hex(id(self))}, {mem=}, {offset=}, "
*/
  __pyx_t_5 = (__pyx_v_next == Py_None);
  if (__pyx_t_5) {
    __Pyx_INCREF(__pyx_mstate_global->__pyx_n_u_None);
    __pyx_t_4 = __pyx_mstate_global->__pyx_n_u_None;
  } else {
    __pyx_t_3 = NULL;
    __pyx_t_6 = 1;
    {
      PyObject *__pyx_callargs[2] = {__pyx_t_3, __pyx_v_next};
      __pyx_t_1 = __Pyx_PyObject_FastCall((PyObject*)__pyx_builtin_id, __pyx_callargs+__pyx_t_6, (2-__pyx_t_6) | (__pyx_t_6*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
      __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
      if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 406, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
    }
    __pyx_t_3 = __Pyx_PyNumber_Hex(__pyx_t_1); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 406, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __pyx_t_4 = __pyx_t_3;
    __pyx_t_3 = 0;
  }
  __Pyx_DECREF_SET(__pyx_v_next, __pyx_t_4);
  __pyx_t_4 = 0;

  /* "cupy/cuda/memory.pyx":407
 *         prev = "None" if prev is None else hex(id(prev))
 *         next = "None" if next is None else hex(id(next))
 *         return (             # <<<<<<<<<<<<<<
 *             f"<{type(self)} at {hex(id(self))}, {mem=}, {offset=}, "
 *             f"prev={prev} next={next}>")
*/
  __Pyx_XDECREF(__pyx_r);

  /* "cupy/cuda/memory.pyx":408
 *         next = "None" if next is None else hex(id(next))
 *         return (
 *             f"<{type(self)} at {hex(id(self))}, {mem=}, {offset=}, "             # <<<<<<<<<<<<<<
 *             f"prev={prev} next={next}>")
 * 
*/
  __pyx_t_4 = __Pyx_PyObject_FormatSimple(((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))), __pyx_mstate_global->__pyx_empty_unicode); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 408, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_t_3 = __Pyx_PyObject_CallOneArg(__pyx_builtin_id, ((PyObject *)__pyx_v_self)); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 408, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_1 = __Pyx_PyNumber_Hex(__pyx_t_3); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 408, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_t_3 = __Pyx_PyUnicode_Unicode(__pyx_t_1); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 408, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_1 = __Pyx_PyObject_FormatSimpleAndDecref(PyObject_Repr(((PyObject *)__pyx_v_mem)), __pyx_mstate_global->__pyx_empty_unicode); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 408, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_7 = __Pyx_PyUnicode_From_ptrdiff_t(__pyx_v_offset, 0, ' ', 'd'); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 408, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_7);

  /* "cupy/cuda/memory.pyx":409
 *         return (
 *             f"<{type(self)} at {hex(id(self))}, {mem=}, {offset=}, "
 *             f"prev={prev} next={next}>")             # <<<<<<<<<<<<<<
 * 
 *     cdef _init(self, BaseMemory mem, ptrdiff_t offset,
*/
  __pyx_t_8 = __Pyx_PyObject_FormatSimple(__pyx_v_prev, __pyx_mstate_global->__pyx_empty_unicode); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 409, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_8);
  __pyx_t_9 = __Pyx_PyObject_FormatSimple(__pyx_v_next, __pyx_mstate_global->__pyx_empty_unicode); if (unlikely(!__pyx_t_9)) __PYX_ERR(0, 409, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_9);
  __pyx_t_10[0] = __pyx_mstate_global->__pyx_kp_u__3;
  __pyx_t_10[1] = __pyx_t_4;
  __pyx_t_10[2] = __pyx_mstate_global->__pyx_kp_u_at;
  __pyx_t_10[3] = __pyx_t_3;
  __pyx_t_10[4] = __pyx_mstate_global->__pyx_kp_u_mem;
  __pyx_t_10[5] = __pyx_t_1;
  __pyx_t_10[6] = __pyx_mstate_global->__pyx_kp_u_offset;
  __pyx_t_10[7] = __pyx_t_7;
  __pyx_t_10[8] = __pyx_mstate_global->__pyx_kp_u_prev;
  __pyx_t_10[9] = __pyx_t_8;
  __pyx_t_10[10] = __pyx_mstate_global->__pyx_kp_u_next;
  __pyx_t_10[11] = __pyx_t_9;
  __pyx_t_10[12] = __pyx_mstate_global->__pyx_kp_u__4;

  /* "cupy/cuda/memory.pyx":408
 *         next = "None" if next is None else hex(id(next))
 *         return (
 *             f"<{type(self)} at {hex(id(self))}, {mem=}, {offset=}, "             # <<<<<<<<<<<<<<
 *             f"prev={prev} next={next}>")
 * 
*/
  __pyx_t_11 = __Pyx_PyUnicode_Join(__pyx_t_10, 13, 1 * 2 + __Pyx_PyUnicode_GET_LENGTH(__pyx_t_4) + 4 + __Pyx_PyUnicode_GET_LENGTH(__pyx_t_3) + 6 * 2 + __Pyx_PyUnicode_GET_LENGTH(__pyx_t_1) + 9 + __Pyx_PyUnicode_GET_LENGTH(__pyx_t_7) + 7 + __Pyx_PyUnicode_GET_LENGTH(__pyx_t_8) + __Pyx_PyUnicode_GET_LENGTH(__pyx_t_9), 127 | __Pyx_PyUnicode_MAX_CHAR_VALUE(__pyx_t_4) | __Pyx_PyUnicode_MAX_CHAR_VALUE(__pyx_t_3) | __Pyx_PyUnicode_MAX_CHAR_VALUE(__pyx_t_1) | __Pyx_PyUnicode_MAX_CHAR_VALUE(__pyx_t_8) | __Pyx_PyUnicode_MAX_CHAR_VALUE(__pyx_t_9));
  if (unlikely(!__pyx_t_11)) __PYX_ERR(0, 408, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_11);
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
  __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
  __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
  __pyx_r = __pyx_t_11;
  __pyx_t_11 = 0;
  goto __pyx_L0;

  /* "cupy/cuda/memory.pyx":402
 *         self._init(mem, offset, size, arena)
 * 
 *     def __repr__(self):             # <<<<<<<<<<<<<<
 *         # To simplify debugging if needed.
 *         mem, offset, prev, next = self.mem, self.offset, self.prev, self.next
*/

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_7);
  __Pyx_XDECREF(__pyx_t_8);
  __Pyx_XDECREF(__pyx_t_9);
  __Pyx_XDECREF(__pyx_t_11);
  __Pyx_AddTraceback("cupy.cuda.memory._Chunk.__repr__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF((PyObject *)__pyx_v_mem);
  __Pyx_XDECREF(__pyx_v_prev);
  __Pyx_XDECREF(__pyx_v_next);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "cupy/cuda/memory.pyx":411
 *             f"prev={prev} next={next}>")
 * 
 *     cdef _init(self, BaseMemory mem, ptrdiff_t offset,             # <<<<<<<<<<<<<<
 *                size_t size, _Arena arena):
 *         assert mem.ptr != 0 or offset == 0
*/

static PyObject *__pyx_f_4cupy_4cuda_6memory_6_Chunk__init(struct __pyx_obj_4cupy_4cuda_6memory__Chunk *__pyx_v_self, struct __pyx_obj_4cupy_4cuda_6memory_BaseMemory *__pyx_v_mem, ptrdiff_t __pyx_v_offset, size_t __pyx_v_size, struct __pyx_obj_4cupy_4cuda_6memory__Arena *__pyx_v_arena) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  int __pyx_t_2;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("_init", 0);

  /* "cupy/cuda/memory.pyx":413
 *     cdef _init(self, BaseMemory mem, ptrdiff_t offset,
 *                size_t size, _Arena arena):
 *         assert mem.ptr != 0 or offset == 0             # <<<<<<<<<<<<<<
 *         self.mem = mem
 *         self.offset = offset
*/
  #ifndef CYTHON_WITHOUT_ASSERTIONS
  if (unlikely(__pyx_assertions_enabled())) {
    __pyx_t_2 = (__pyx_v_mem->ptr != 0);
    if (!__pyx_t_2) {
    } else {
      __pyx_t_1 = __pyx_t_2;
      goto __pyx_L3_bool_binop_done;
    }
    __pyx_t_2 = (__pyx_v_offset == 0);
    __pyx_t_1 = __pyx_t_2;
    __pyx_L3_bool_binop_done:;
    if (unlikely(!__pyx_t_1)) {
      __Pyx_Raise(((PyObject *)(((PyTypeObject*)PyExc_AssertionError))), 0, 0, 0);
      __PYX_ERR(0, 413, __pyx_L1_error)
    }
  }
  #else
  if ((1)); else __PYX_ERR(0, 413, __pyx_L1_error)
  #endif

  /* "cupy/cuda/memory.pyx":414
 *                size_t size, _Arena arena):
 *         assert mem.ptr != 0 or offset == 0
 *         self.mem = mem             # <<<<<<<<<<<<<<
 *         self.offset = offset
 *         self.size = size
*/
  __Pyx_INCREF((PyObject *)__pyx_v_mem);
  __Pyx_GIVEREF((PyObject *)__pyx_v_mem);
  __Pyx_GOTREF((PyObject *)__pyx_v_self->mem);
  __Pyx_DECREF((PyObject *)__pyx_v_self->mem);
  __pyx_v_self->mem = __pyx_v_mem;

  /* "cupy/cuda/memory.pyx":415
 *         assert mem.ptr != 0 or offset == 0
 *         self.mem = mem
 *         self.offset = offset             # <<<<<<<<<<<<<<
 *         self.size = size
 *         self.arena = arena
*/
  __pyx_v_self->offset = __pyx_v_offset;

  /* "cupy/cuda/memory.pyx":416
 *         self.mem = mem
 *         self.offset = offset
 *         self.size = size             # <<<<<<<<<<<<<<
 *         self.arena = arena
 * 
*/
  __pyx_v_self->size = __pyx_v_size;

  /* "cupy/cuda/memory.pyx":417
 *         self.offset = offset
 *         self.size = size
 *         self.arena = arena             # <<<<<<<<<<<<<<
 * 
 *     cpdef intptr_t ptr(self) noexcept:
*/
  __Pyx_INCREF((PyObject *)__pyx_v_arena);
  __Pyx_GIVEREF((PyObject *)__pyx_v_arena);
  __Pyx_GOTREF((PyObject *)__pyx_v_self->arena);
  __Pyx_DECREF((PyObject *)__pyx_v_self->arena);
  __pyx_v_self->arena = __pyx_v_arena;

  /* "cupy/cuda/memory.pyx":411
 *             f"prev={prev} next={next}>")
 * 
 *     cdef _init(self, BaseMemory mem, ptrdiff_t offset,             # <<<<<<<<<<<<<<
 *                size_t size, _Arena arena):
 *         assert mem.ptr != 0 or offset == 0
*/

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_AddTraceback("cupy.cuda.memory._Chunk._init", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "cupy/cuda/memory.pyx":419
 *         self.arena = arena
 * 
 *     cpdef intptr_t ptr(self) noexcept:             # <<<<<<<<<<<<<<
 *         return self.mem.ptr + self.offset
 * 
*/

static PyObject *__pyx_pw_4cupy_4cuda_6memory_6_Chunk_5ptr(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
static intptr_t __pyx_f_4cupy_4cuda_6memory_6_Chunk_ptr(struct __pyx_obj_4cupy_4cuda_6memory__Chunk *__pyx_v_self, CYTHON_UNUSED int __pyx_skip_dispatch) {
  intptr_t __pyx_r;

  /* "cupy/cuda/memory.pyx":420
 * 
 *     cpdef intptr_t ptr(self) noexcept:
 *         return self.mem.ptr + self.offset             # <<<<<<<<<<<<<<
 * 
 *     cpdef _Chunk split(self, size_t size):
*/
  __pyx_r = (__pyx_v_self->mem->ptr + __pyx_v_self->offset);
  goto __pyx_L0;

  /* "cupy/cuda/memory.pyx":419
 *         self.arena = arena
 * 
 *     cpdef intptr_t ptr(self) noexcept:             # <<<<<<<<<<<<<<
 *         return self.mem.ptr + self.offset
 * 
*/

  /* function exit code */
  __pyx_L0:;
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_pw_4cupy_4cuda_6memory_6_Chunk_5ptr(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
PyDoc_STRVAR(__pyx_doc_4cupy_4cuda_6memory_6_Chunk_4ptr, "_Chunk.ptr(self) -> intptr_t");
static PyObject *__pyx_pw_4cupy_4cuda_6memory_6_Chunk_5ptr(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
) {
  #if !CYTHON_METH_FASTCALL
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  #endif
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("ptr (wrapper)", 0);
  #if !CYTHON_METH_FASTCALL
  #if CYTHON_ASSUME_SAFE_SIZE
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return NULL;
  #endif
  #endif
  __pyx_kwvalues = __Pyx_KwValues_FASTCALL(__pyx_args, __pyx_nargs);
  if (unlikely(__pyx_nargs > 0)) { __Pyx_RaiseArgtupleInvalid("ptr", 1, 0, 0, __pyx_nargs); return NULL; }
  const Py_ssize_t __pyx_kwds_len = unlikely(__pyx_kwds) ? __Pyx_NumKwargs_FASTCALL(__pyx_kwds) : 0;
  if (unlikely(__pyx_kwds_len < 0)) return NULL;
  if (unlikely(__pyx_kwds_len > 0)) {__Pyx_RejectKeywords("ptr", __pyx_kwds); return NULL;}
  __pyx_r = __pyx_pf_4cupy_4cuda_6memory_6_Chunk_4ptr(((struct __pyx_obj_4cupy_4cuda_6memory__Chunk *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4cupy_4cuda_6memory_6_Chunk_4ptr(struct __pyx_obj_4cupy_4cuda_6memory__Chunk *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("ptr", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = PyLong_FromSsize_t(__pyx_f_4cupy_4cuda_6memory_6_Chunk_ptr(__pyx_v_self, 1)); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 419, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("cupy.cuda.memory._Chunk.ptr", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "cupy/cuda/memory.pyx":422
 *         return self.mem.ptr + self.offset
 * 
 *     cpdef _Chunk split(self, size_t size):             # <<<<<<<<<<<<<<
 *         """Split contiguous block of a larger allocation"""
 *         cdef _Chunk remaining
*/

static PyObject *__pyx_pw_4cupy_4cuda_6memory_6_Chunk_7split(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
static struct __pyx_obj_4cupy_4cuda_6memory__Chunk *__pyx_f_4cupy_4cuda_6memory_6_Chunk_split(struct __pyx_obj_4cupy_4cuda_6memory__Chunk *__pyx_v_self, size_t __pyx_v_size, CYTHON_UNUSED int __pyx_skip_dispatch) {
  struct __pyx_obj_4cupy_4cuda_6memory__Chunk *__pyx_v_remaining = 0;
  struct __pyx_obj_4cupy_4cuda_6memory__Chunk *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("split", 0);

  /* "cupy/cuda/memory.pyx":425
 *         """Split contiguous block of a larger allocation"""
 *         cdef _Chunk remaining
 *         assert self.size >= size             # <<<<<<<<<<<<<<
 *         if self.size == size:
 *             return None
*/
  #ifndef CYTHON_WITHOUT_ASSERTIONS
  if (unlikely(__pyx_assertions_enabled())) {
    __pyx_t_1 = (__pyx_v_self->size >= __pyx_v_size);
    if (unlikely(!__pyx_t_1)) {
      __Pyx_Raise(((PyObject *)(((PyTypeObject*)PyExc_AssertionError))), 0, 0, 0);
      __PYX_ERR(0, 425, __pyx_L1_error)
    }
  }
  #else
  if ((1)); else __PYX_ERR(0, 425, __pyx_L1_error)
  #endif

  /* "cupy/cuda/memory.pyx":426
 *         cdef _Chunk remaining
 *         assert self.size >= size
 *         if self.size == size:             # <<<<<<<<<<<<<<
 *             return None
 *         remaining = _Chunk.__new__(_Chunk)
*/
  __pyx_t_1 = (__pyx_v_self->size == __pyx_v_size);
  if (__pyx_t_1) {

    /* "cupy/cuda/memory.pyx":427
 *         assert self.size >= size
 *         if self.size == size:
 *             return None             # <<<<<<<<<<<<<<
 *         remaining = _Chunk.__new__(_Chunk)
 *         remaining._init(self.mem, self.offset + size, self.size - size,
*/
    __Pyx_XDECREF((PyObject *)__pyx_r);
    __pyx_r = ((struct __pyx_obj_4cupy_4cuda_6memory__Chunk *)Py_None); __Pyx_INCREF(Py_None);
    goto __pyx_L0;

    /* "cupy/cuda/memory.pyx":426
 *         cdef _Chunk remaining
 *         assert self.size >= size
 *         if self.size == size:             # <<<<<<<<<<<<<<
 *             return None
 *         remaining = _Chunk.__new__(_Chunk)
*/
  }

  /* "cupy/cuda/memory.pyx":428
 *         if self.size == size:
 *             return None
 *         remaining = _Chunk.__new__(_Chunk)             # <<<<<<<<<<<<<<
 *         remaining._init(self.mem, self.offset + size, self.size - size,
 *                         self.arena)
*/
  __pyx_t_2 = ((PyObject *)__pyx_tp_new_4cupy_4cuda_6memory__Chunk(((PyTypeObject *)__pyx_mstate_global->__pyx_ptype_4cupy_4cuda_6memory__Chunk), __pyx_mstate_global->__pyx_empty_tuple, NULL)); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 428, __pyx_L1_error)
  __Pyx_GOTREF((PyObject *)__pyx_t_2);
  __pyx_v_remaining = ((struct __pyx_obj_4cupy_4cuda_6memory__Chunk *)__pyx_t_2);
  __pyx_t_2 = 0;

  /* "cupy/cuda/memory.pyx":429
 *             return None
 *         remaining = _Chunk.__new__(_Chunk)
 *         remaining._init(self.mem, self.offset + size, self.size - size,             # <<<<<<<<<<<<<<
 *                         self.arena)
 *         self.size = size
*/
  __pyx_t_2 = ((PyObject *)__pyx_v_self->mem);
  __Pyx_INCREF(__pyx_t_2);

  /* "cupy/cuda/memory.pyx":430
 *         remaining = _Chunk.__new__(_Chunk)
 *         remaining._init(self.mem, self.offset + size, self.size - size,
 *                         self.arena)             # <<<<<<<<<<<<<<
 *         self.size = size
 * 
*/
  __pyx_t_3 = ((PyObject *)__pyx_v_self->arena);
  __Pyx_INCREF(__pyx_t_3);

  /* "cupy/cuda/memory.pyx":429
 *             return None
 *         remaining = _Chunk.__new__(_Chunk)
 *         remaining._init(self.mem, self.offset + size, self.size - size,             # <<<<<<<<<<<<<<
 *                         self.arena)
 *         self.size = size
*/
  __pyx_t_4 = __pyx_f_4cupy_4cuda_6memory_6_Chunk__init(__pyx_v_remaining, ((struct __pyx_obj_4cupy_4cuda_6memory_BaseMemory *)__pyx_t_2), (__pyx_v_self->offset + __pyx_v_size), (__pyx_v_self->size - __pyx_v_size), ((struct __pyx_obj_4cupy_4cuda_6memory__Arena *)__pyx_t_3)); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 429, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

  /* "cupy/cuda/memory.pyx":431
 *         remaining._init(self.mem, self.offset + size, self.size - size,
 *                         self.arena)
 *         self.size = size             # <<<<<<<<<<<<<<
 * 
 *         if self.next is not None:
*/
  __pyx_v_self->size = __pyx_v_size;

  /* "cupy/cuda/memory.pyx":433
 *         self.size = size
 * 
 *         if self.next is not None:             # <<<<<<<<<<<<<<
 *             remaining.next = self.next
 *             remaining.next.prev = remaining
*/
  __pyx_t_1 = (((PyObject *)__pyx_v_self->next) != Py_None);
  if (__pyx_t_1) {

    /* "cupy/cuda/memory.pyx":434
 * 
 *         if self.next is not None:
 *             remaining.next = self.next             # <<<<<<<<<<<<<<
 *             remaining.next.prev = remaining
 *         self.next = remaining
*/
    __pyx_t_4 = ((PyObject *)__pyx_v_self->next);
    __Pyx_INCREF(__pyx_t_4);
    __Pyx_GIVEREF(__pyx_t_4);
    __Pyx_GOTREF((PyObject *)__pyx_v_remaining->next);
    __Pyx_DECREF((PyObject *)__pyx_v_remaining->next);
    __pyx_v_remaining->next = ((struct __pyx_obj_4cupy_4cuda_6memory__Chunk *)__pyx_t_4);
    __pyx_t_4 = 0;

    /* "cupy/cuda/memory.pyx":435
 *         if self.next is not None:
 *             remaining.next = self.next
 *             remaining.next.prev = remaining             # <<<<<<<<<<<<<<
 *         self.next = remaining
 *         remaining.prev = self
*/
    __Pyx_INCREF((PyObject *)__pyx_v_remaining);
    __Pyx_GIVEREF((PyObject *)__pyx_v_remaining);
    __Pyx_GOTREF((PyObject *)__pyx_v_remaining->next->prev);
    __Pyx_DECREF((PyObject *)__pyx_v_remaining->next->prev);
    __pyx_v_remaining->next->prev = __pyx_v_remaining;

    /* "cupy/cuda/memory.pyx":433
 *         self.size = size
 * 
 *         if self.next is not None:             # <<<<<<<<<<<<<<
 *             remaining.next = self.next
 *             remaining.next.prev = remaining
*/
  }

  /* "cupy/cuda/memory.pyx":436
 *             remaining.next = self.next
 *             remaining.next.prev = remaining
 *         self.next = remaining             # <<<<<<<<<<<<<<
 *         remaining.prev = self
 *         return remaining
*/
  __Pyx_INCREF((PyObject *)__pyx_v_remaining);
  __Pyx_GIVEREF((PyObject *)__pyx_v_remaining);
  __Pyx_GOTREF((PyObject *)__pyx_v_self->next);
  __Pyx_DECREF((PyObject *)__pyx_v_self->next);
  __pyx_v_self->next = __pyx_v_remaining;

  /* "cupy/cuda/memory.pyx":437
 *             remaining.next.prev = remaining
 *         self.next = remaining
 *         remaining.prev = self             # <<<<<<<<<<<<<<
 *         return remaining
 * 
*/
  __Pyx_INCREF((PyObject *)__pyx_v_self);
  __Pyx_GIVEREF((PyObject *)__pyx_v_self);
  __Pyx_GOTREF((PyObject *)__pyx_v_remaining->prev);
  __Pyx_DECREF((PyObject *)__pyx_v_remaining->prev);
  __pyx_v_remaining->prev = __pyx_v_self;

  /* "cupy/cuda/memory.pyx":438
 *         self.next = remaining
 *         remaining.prev = self
 *         return remaining             # <<<<<<<<<<<<<<
 * 
 *     cpdef merge_next(self):
*/
  __Pyx_XDECREF((PyObject *)__pyx_r);
  __Pyx_INCREF((PyObject *)__pyx_v_remaining);
  __pyx_r = __pyx_v_remaining;
  goto __pyx_L0;

  /* "cupy/cuda/memory.pyx":422
 *         return self.mem.ptr + self.offset
 * 
 *     cpdef _Chunk split(self, size_t size):             # <<<<<<<<<<<<<<
 *         """Split contiguous block of a larger allocation"""
 *         cdef _Chunk remaining
*/

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_AddTraceback("cupy.cuda.memory._Chunk.split", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XDECREF((PyObject *)__pyx_v_remaining);
  __Pyx_XGIVEREF((PyObject *)__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_pw_4cupy_4cuda_6memory_6_Chunk_7split(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
PyDoc_STRVAR(__pyx_doc_4cupy_4cuda_6memory_6_Chunk_6split, "_Chunk.split(self, size_t size) -> _Chunk\n\nSplit contiguous block of a larger allocation");
static PyObject *__pyx_pw_4cupy_4cuda_6memory_6_Chunk_7split(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
) {
  size_t __pyx_v_size;
  #if !CYTHON_METH_FASTCALL
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  #endif
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject* values[1] = {0};
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("split (wrapper)", 0);
  #if !CYTHON_METH_FASTCALL
  #if CYTHON_ASSUME_SAFE_SIZE
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return NULL;
  #endif
  #endif
  __pyx_kwvalues = __Pyx_KwValues_FASTCALL(__pyx_args, __pyx_nargs);
  {
    PyObject ** const __pyx_pyargnames[] = {&__pyx_mstate_global->__pyx_n_u_size,0};
    const Py_ssize_t __pyx_kwds_len = (__pyx_kwds) ? __Pyx_NumKwargs_FASTCALL(__pyx_kwds) : 0;
    if (unlikely(__pyx_kwds_len < 0)) __PYX_ERR(0, 422, __pyx_L3_error)
    if (__pyx_kwds_len > 0) {
      switch (__pyx_nargs) {
        case  1:
        values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(0, 422, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      const Py_ssize_t kwd_pos_args = __pyx_nargs;
      if (__Pyx_ParseKeywords(__pyx_kwds, __pyx_kwvalues, __pyx_pyargnames, 0, values, kwd_pos_args, __pyx_kwds_len, "split", 0) < (0)) __PYX_ERR(0, 422, __pyx_L3_error)
      for (Py_ssize_t i = __pyx_nargs; i < 1; i++) {
        if (unlikely(!values[i])) { __Pyx_RaiseArgtupleInvalid("split", 1, 1, 1, i); __PYX_ERR(0, 422, __pyx_L3_error) }
      }
    } else if (unlikely(__pyx_nargs != 1)) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
      if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(0, 422, __pyx_L3_error)
    }
    __pyx_v_size = __Pyx_PyLong_As_size_t(values[0]); if (unlikely((__pyx_v_size == (size_t)-1) && PyErr_Occurred())) __PYX_ERR(0, 422, __pyx_L3_error)
  }
  goto __pyx_L6_skip;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("split", 1, 1, 1, __pyx_nargs); __PYX_ERR(0, 422, __pyx_L3_error)
  __pyx_L6_skip:;
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_AddTraceback("cupy.cuda.memory._Chunk.split", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_4cupy_4cuda_6memory_6_Chunk_6split(((struct __pyx_obj_4cupy_4cuda_6memory__Chunk *)__pyx_v_self), __pyx_v_size);

  /* function exit code */
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4cupy_4cuda_6memory_6_Chunk_6split(struct __pyx_obj_4cupy_4cuda_6memory__Chunk *__pyx_v_self, size_t __pyx_v_size) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("split", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = ((PyObject *)__pyx_f_4cupy_4cuda_6memory_6_Chunk_split(__pyx_v_self, __pyx_v_size, 1)); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 422, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("cupy.cuda.memory._Chunk.split", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "cupy/cuda/memory.pyx":440
 *         return remaining
 * 
 *     cpdef merge_next(self):             # <<<<<<<<<<<<<<
 *         """Merge previously splitted next block into this one"""
 *         self.next.arena = None  # chunk is free so no arena
*/

static PyObject *__pyx_pw_4cupy_4cuda_6memory_6_Chunk_9merge_next(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
static PyObject *__pyx_f_4cupy_4cuda_6memory_6_Chunk_merge_next(struct __pyx_obj_4cupy_4cuda_6memory__Chunk *__pyx_v_self, CYTHON_UNUSED int __pyx_skip_dispatch) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_t_2;
  __Pyx_RefNannySetupContext("merge_next", 0);

  /* "cupy/cuda/memory.pyx":442
 *     cpdef merge_next(self):
 *         """Merge previously splitted next block into this one"""
 *         self.next.arena = None  # chunk is free so no arena             # <<<<<<<<<<<<<<
 *         self.size += self.next.size
 *         self.next = self.next.next
*/
  __Pyx_INCREF(Py_None);
  __Pyx_GIVEREF(Py_None);
  __Pyx_GOTREF((PyObject *)__pyx_v_self->next->arena);
  __Pyx_DECREF((PyObject *)__pyx_v_self->next->arena);
  __pyx_v_self->next->arena = ((struct __pyx_obj_4cupy_4cuda_6memory__Arena *)Py_None);

  /* "cupy/cuda/memory.pyx":443
 *         """Merge previously splitted next block into this one"""
 *         self.next.arena = None  # chunk is free so no arena
 *         self.size += self.next.size             # <<<<<<<<<<<<<<
 *         self.next = self.next.next
 *         if self.next is not None:
*/
  __pyx_v_self->size = (__pyx_v_self->size + __pyx_v_self->next->size);

  /* "cupy/cuda/memory.pyx":444
 *         self.next.arena = None  # chunk is free so no arena
 *         self.size += self.next.size
 *         self.next = self.next.next             # <<<<<<<<<<<<<<
 *         if self.next is not None:
 *             self.next.prev = self
*/
  __pyx_t_1 = ((PyObject *)__pyx_v_self->next->next);
  __Pyx_INCREF(__pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF((PyObject *)__pyx_v_self->next);
  __Pyx_DECREF((PyObject *)__pyx_v_self->next);
  __pyx_v_self->next = ((struct __pyx_obj_4cupy_4cuda_6memory__Chunk *)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "cupy/cuda/memory.pyx":445
 *         self.size += self.next.size
 *         self.next = self.next.next
 *         if self.next is not None:             # <<<<<<<<<<<<<<
 *             self.next.prev = self
 * 
*/
  __pyx_t_2 = (((PyObject *)__pyx_v_self->next) != Py_None);
  if (__pyx_t_2) {

    /* "cupy/cuda/memory.pyx":446
 *         self.next = self.next.next
 *         if self.next is not None:
 *             self.next.prev = self             # <<<<<<<<<<<<<<
 * 
 *     # Note on __del__/__dealloc__ of a chunk.
*/
    __Pyx_INCREF((PyObject *)__pyx_v_self);
    __Pyx_GIVEREF((PyObject *)__pyx_v_self);
    __Pyx_GOTREF((PyObject *)__pyx_v_self->next->prev);
    __Pyx_DECREF((PyObject *)__pyx_v_self->next->prev);
    __pyx_v_self->next->prev = __pyx_v_self;

    /* "cupy/cuda/memory.pyx":445
 *         self.size += self.next.size
 *         self.next = self.next.next
 *         if self.next is not None:             # <<<<<<<<<<<<<<
 *             self.next.prev = self
 * 
*/
  }

  /* "cupy/cuda/memory.pyx":440
 *         return remaining
 * 
 *     cpdef merge_next(self):             # <<<<<<<<<<<<<<
 *         """Merge previously splitted next block into this one"""
 *         self.next.arena = None  # chunk is free so no arena
*/

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_pw_4cupy_4cuda_6memory_6_Chunk_9merge_next(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
PyDoc_STRVAR(__pyx_doc_4cupy_4cuda_6memory_6_Chunk_8merge_next, "_Chunk.merge_next(self)\n\nMerge previously splitted next block into this one");
static PyObject *__pyx_pw_4cupy_4cuda_6memory_6_Chunk_9merge_next(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
) {
  #if !CYTHON_METH_FASTCALL
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  #endif
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("merge_next (wrapper)", 0);
  #if !CYTHON_METH_FASTCALL
  #if CYTHON_ASSUME_SAFE_SIZE
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return NULL;
  #endif
  #endif
  __pyx_kwvalues = __Pyx_KwValues_FASTCALL(__pyx_args, __pyx_nargs);
  if (unlikely(__pyx_nargs > 0)) { __Pyx_RaiseArgtupleInvalid("merge_next", 1, 0, 0, __pyx_nargs); return NULL; }
  const Py_ssize_t __pyx_kwds_len = unlikely(__pyx_kwds) ? __Pyx_NumKwargs_FASTCALL(__pyx_kwds) : 0;
  if (unlikely(__pyx_kwds_len < 0)) return NULL;
  if (unlikely(__pyx_kwds_len > 0)) {__Pyx_RejectKeywords("merge_next", __pyx_kwds); return NULL;}
  __pyx_r = __pyx_pf_4cupy_4cuda_6memory_6_Chunk_8merge_next(((struct __pyx_obj_4cupy_4cuda_6memory__Chunk *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4cupy_4cuda_6memory_6_Chunk_8merge_next(struct __pyx_obj_4cupy_4cuda_6memory__Chunk *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("merge_next", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __pyx_f_4cupy_4cuda_6memory_6_Chunk_merge_next(__pyx_v_self, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 440, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("cupy.cuda.memory._Chunk.merge_next", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "cupy/cuda/memory.pyx":390
 * 
 *     cdef:
 *         readonly BaseMemory mem             # <<<<<<<<<<<<<<
 *         readonly ptrdiff_t offset
 *         readonly size_t size
*/

/* Python wrapper */
static PyObject *__pyx_pw_4cupy_4cuda_6memory_6_Chunk_3mem_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_4cupy_4cuda_6memory_6_Chunk_3mem_1__get__(PyObject *__pyx_v_self) {
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_kwvalues = __Pyx_KwValues_VARARGS(__pyx_args, __pyx_nargs);
  __pyx_r = __pyx_pf_4cupy_4cuda_6memory_6_Chunk_3mem___get__(((struct __pyx_obj_4cupy_4cuda_6memory__Chunk *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4cupy_4cuda_6memory_6_Chunk_3mem___get__(struct __pyx_obj_4cupy_4cuda_6memory__Chunk *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__", 0);
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF((PyObject *)__pyx_v_self->mem);
  __pyx_r = ((PyObject *)__pyx_v_self->mem);
  goto __pyx_L0;

  /* function exit code */
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "cupy/cuda/memory.pyx":391
 *     cdef:
 *         readonly BaseMemory mem
 *         readonly ptrdiff_t offset             # <<<<<<<<<<<<<<
 *         readonly size_t size
 *         readonly _Arena arena
*/

/* Python wrapper */
static PyObject *__pyx_pw_4cupy_4cuda_6memory_6_Chunk_6offset_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_4cupy_4cuda_6memory_6_Chunk_6offset_1__get__(PyObject *__pyx_v_self) {
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_kwvalues = __Pyx_KwValues_VARARGS(__pyx_args, __pyx_nargs);
  __pyx_r = __pyx_pf_4cupy_4cuda_6memory_6_Chunk_6offset___get__(((struct __pyx_obj_4cupy_4cuda_6memory__Chunk *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4cupy_4cuda_6memory_6_Chunk_6offset___get__(struct __pyx_obj_4cupy_4cuda_6memory__Chunk *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__get__", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __Pyx_PyLong_From_ptrdiff_t(__pyx_v_self->offset); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 391, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("cupy.cuda.memory._Chunk.offset.__get__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "cupy/cuda/memory.pyx":392
 *         readonly BaseMemory mem
 *         readonly ptrdiff_t offset
 *         readonly size_t size             # <<<<<<<<<<<<<<
 *         readonly _Arena arena
 *         public _Chunk prev
*/

/* Python wrapper */
static PyObject *__pyx_pw_4cupy_4cuda_6memory_6_Chunk_4size_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_4cupy_4cuda_6memory_6_Chunk_4size_1__get__(PyObject *__pyx_v_self) {
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_kwvalues = __Pyx_KwValues_VARARGS(__pyx_args, __pyx_nargs);
  __pyx_r = __pyx_pf_4cupy_4cuda_6memory_6_Chunk_4size___get__(((struct __pyx_obj_4cupy_4cuda_6memory__Chunk *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4cupy_4cuda_6memory_6_Chunk_4size___get__(struct __pyx_obj_4cupy_4cuda_6memory__Chunk *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__get__", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __Pyx_PyLong_FromSize_t(__pyx_v_self->size); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 392, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("cupy.cuda.memory._Chunk.size.__get__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "cupy/cuda/memory.pyx":393
 *         readonly ptrdiff_t offset
 *         readonly size_t size
 *         readonly _Arena arena             # <<<<<<<<<<<<<<
 *         public _Chunk prev
 *         public _Chunk next
*/

/* Python wrapper */
static PyObject *__pyx_pw_4cupy_4cuda_6memory_6_Chunk_5arena_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_4cupy_4cuda_6memory_6_Chunk_5arena_1__get__(PyObject *__pyx_v_self) {
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_kwvalues = __Pyx_KwValues_VARARGS(__pyx_args, __pyx_nargs);
  __pyx_r = __pyx_pf_4cupy_4cuda_6memory_6_Chunk_5arena___get__(((struct __pyx_obj_4cupy_4cuda_6memory__Chunk *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4cupy_4cuda_6memory_6_Chunk_5arena___get__(struct __pyx_obj_4cupy_4cuda_6memory__Chunk *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__", 0);
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF((PyObject *)__pyx_v_self->arena);
  __pyx_r = ((PyObject *)__pyx_v_self->arena);
  goto __pyx_L0;

  /* function exit code */
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "cupy/cuda/memory.pyx":394
 *         readonly size_t size
 *         readonly _Arena arena
 *         public _Chunk prev             # <<<<<<<<<<<<<<
 *         public _Chunk next
 * 
*/

/* Python wrapper */
static PyObject *__pyx_pw_4cupy_4cuda_6memory_6_Chunk_4prev_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_4cupy_4cuda_6memory_6_Chunk_4prev_1__get__(PyObject *__pyx_v_self) {
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_kwvalues = __Pyx_KwValues_VARARGS(__pyx_args, __pyx_nargs);
  __pyx_r = __pyx_pf_4cupy_4cuda_6memory_6_Chunk_4prev___get__(((struct __pyx_obj_4cupy_4cuda_6memory__Chunk *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4cupy_4cuda_6memory_6_Chunk_4prev___get__(struct __pyx_obj_4cupy_4cuda_6memory__Chunk *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__", 0);
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF((PyObject *)__pyx_v_self->prev);
  __pyx_r = ((PyObject *)__pyx_v_self->prev);
  goto __pyx_L0;

  /* function exit code */
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static int __pyx_pw_4cupy_4cuda_6memory_6_Chunk_4prev_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value); /*proto*/
static int __pyx_pw_4cupy_4cuda_6memory_6_Chunk_4prev_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value) {
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__ (wrapper)", 0);
  __pyx_kwvalues = __Pyx_KwValues_VARARGS(__pyx_args, __pyx_nargs);
  __pyx_r = __pyx_pf_4cupy_4cuda_6memory_6_Chunk_4prev_2__set__(((struct __pyx_obj_4cupy_4cuda_6memory__Chunk *)__pyx_v_self), ((PyObject *)__pyx_v_value));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_4cupy_4cuda_6memory_6_Chunk_4prev_2__set__(struct __pyx_obj_4cupy_4cuda_6memory__Chunk *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__set__", 0);
  __pyx_t_1 = __pyx_v_value;
  __Pyx_INCREF(__pyx_t_1);
  if (!(likely(((__pyx_t_1) == Py_None) || likely(__Pyx_TypeTest(__pyx_t_1, __pyx_mstate_global->__pyx_ptype_4cupy_4cuda_6memory__Chunk))))) __PYX_ERR(0, 394, __pyx_L1_error)
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF((PyObject *)__pyx_v_self->prev);
  __Pyx_DECREF((PyObject *)__pyx_v_self->prev);
  __pyx_v_self->prev = ((struct __pyx_obj_4cupy_4cuda_6memory__Chunk *)__pyx_t_1);
  __pyx_t_1 = 0;

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("cupy.cuda.memory._Chunk.prev.__set__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static int __pyx_pw_4cupy_4cuda_6memory_6_Chunk_4prev_5__del__(PyObject *__pyx_v_self); /*proto*/
static int __pyx_pw_4cupy_4cuda_6memory_6_Chunk_4prev_5__del__(PyObject *__pyx_v_self) {
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__del__ (wrapper)", 0);
  __pyx_kwvalues = __Pyx_KwValues_VARARGS(__pyx_args, __pyx_nargs);
  __pyx_r = __pyx_pf_4cupy_4cuda_6memory_6_Chunk_4prev_4__del__(((struct __pyx_obj_4cupy_4cuda_6memory__Chunk *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_4cupy_4cuda_6memory_6_Chunk_4prev_4__del__(struct __pyx_obj_4cupy_4cuda_6memory__Chunk *__pyx_v_self) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__del__", 0);
  __Pyx_INCREF(Py_None);
  __Pyx_GIVEREF(Py_None);
  __Pyx_GOTREF((PyObject *)__pyx_v_self->prev);
  __Pyx_DECREF((PyObject *)__pyx_v_self->prev);
  __pyx_v_self->prev = ((struct __pyx_obj_4cupy_4cuda_6memory__Chunk *)Py_None);

  /* function exit code */
  __pyx_r = 0;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "cupy/cuda/memory.pyx":395
 *         readonly _Arena arena
 *         public _Chunk prev
 *         public _Chunk next             # <<<<<<<<<<<<<<
 * 
 *     def __init__(self, *args):
*/

/* Python wrapper */
static PyObject *__pyx_pw_4cupy_4cuda_6memory_6_Chunk_4next_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_4cupy_4cuda_6memory_6_Chunk_4next_1__get__(PyObject *__pyx_v_self) {
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_kwvalues = __Pyx_KwValues_VARARGS(__pyx_args, __pyx_nargs);
  __pyx_r = __pyx_pf_4cupy_4cuda_6memory_6_Chunk_4next___get__(((struct __pyx_obj_4cupy_4cuda_6memory__Chunk *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4cupy_4cuda_6memory_6_Chunk_4next___get__(struct __pyx_obj_4cupy_4cuda_6memory__Chunk *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__", 0);
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF((PyObject *)__pyx_v_self->next);
  __pyx_r = ((PyObject *)__pyx_v_self->next);
  goto __pyx_L0;

  /* function exit code */
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static int __pyx_pw_4cupy_4cuda_6memory_6_Chunk_4next_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value); /*proto*/
static int __pyx_pw_4cupy_4cuda_6memory_6_Chunk_4next_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value) {
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__ (wrapper)", 0);
  __pyx_kwvalues = __Pyx_KwValues_VARARGS(__pyx_args, __pyx_nargs);
  __pyx_r = __pyx_pf_4cupy_4cuda_6memory_6_Chunk_4next_2__set__(((struct __pyx_obj_4cupy_4cuda_6memory__Chunk *)__pyx_v_self), ((PyObject *)__pyx_v_value));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_4cupy_4cuda_6memory_6_Chunk_4next_2__set__(struct __pyx_obj_4cupy_4cuda_6memory__Chunk *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__set__", 0);
  __pyx_t_1 = __pyx_v_value;
  __Pyx_INCREF(__pyx_t_1);
  if (!(likely(((__pyx_t_1) == Py_None) || likely(__Pyx_TypeTest(__pyx_t_1, __pyx_mstate_global->__pyx_ptype_4cupy_4cuda_6memory__Chunk))))) __PYX_ERR(0, 395, __pyx_L1_error)
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF((PyObject *)__pyx_v_self->next);
  __Pyx_DECREF((PyObject *)__pyx_v_self->next);
  __pyx_v_self->next = ((struct __pyx_obj_4cupy_4cuda_6memory__Chunk *)__pyx_t_1);
  __pyx_t_1 = 0;

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("cupy.cuda.memory._Chunk.next.__set__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static int __pyx_pw_4cupy_4cuda_6memory_6_Chunk_4next_5__del__(PyObject *__pyx_v_self); /*proto*/
static int __pyx_pw_4cupy_4cuda_6memory_6_Chunk_4next_5__del__(PyObject *__pyx_v_self) {
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__del__ (wrapper)", 0);
  __pyx_kwvalues = __Pyx_KwValues_VARARGS(__pyx_args, __pyx_nargs);
  __pyx_r = __pyx_pf_4cupy_4cuda_6memory_6_Chunk_4next_4__del__(((struct __pyx_obj_4cupy_4cuda_6memory__Chunk *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_4cupy_4cuda_6memory_6_Chunk_4next_4__del__(struct __pyx_obj_4cupy_4cuda_6memory__Chunk *__pyx_v_self) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__del__", 0);
  __Pyx_INCREF(Py_None);
  __Pyx_GIVEREF(Py_None);
  __Pyx_GOTREF((PyObject *)__pyx_v_self->next);
  __Pyx_DECREF((PyObject *)__pyx_v_self->next);
  __pyx_v_self->next = ((struct __pyx_obj_4cupy_4cuda_6memory__Chunk *)Py_None);

  /* function exit code */
  __pyx_r = 0;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     cdef tuple state
 *     cdef object _dict
*/

/* Python wrapper */
static PyObject *__pyx_pw_4cupy_4cuda_6memory_6_Chunk_11__reduce_cython__(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
PyDoc_STRVAR(__pyx_doc_4cupy_4cuda_6memory_6_Chunk_10__reduce_cython__, "_Chunk.__reduce_cython__(self)");
static PyObject *__pyx_pw_4cupy_4cuda_6memory_6_Chunk_11__reduce_cython__(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
) {
  #if !CYTHON_METH_FASTCALL
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  #endif
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__reduce_cython__ (wrapper)", 0);
  #if !CYTHON_METH_FASTCALL
  #if CYTHON_ASSUME_SAFE_SIZE
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return NULL;
  #endif
  #endif
  __pyx_kwvalues = __Pyx_KwValues_FASTCALL(__pyx_args, __pyx_nargs);
  if (unlikely(__pyx_nargs > 0)) { __Pyx_RaiseArgtupleInvalid("__reduce_cython__", 1, 0, 0, __pyx_nargs); return NULL; }
  const Py_ssize_t __pyx_kwds_len = unlikely(__pyx_kwds) ? __Pyx_NumKwargs_FASTCALL(__pyx_kwds) : 0;
  if (unlikely(__pyx_kwds_len < 0)) return NULL;
  if (unlikely(__pyx_kwds_len > 0)) {__Pyx_RejectKeywords("__reduce_cython__", __pyx_kwds); return NULL;}
  __pyx_r = __pyx_pf_4cupy_4cuda_6memory_6_Chunk_10__reduce_cython__(((struct __pyx_obj_4cupy_4cuda_6memory__Chunk *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4cupy_4cuda_6memory_6_Chunk_10__reduce_cython__(struct __pyx_obj_4cupy_4cuda_6memory__Chunk *__pyx_v_self) {
  PyObject *__pyx_v_state = 0;
  PyObject *__pyx_v__dict = 0;
  int __pyx_v_use_setstate;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  int __pyx_t_4;
  int __pyx_t_5;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__reduce_cython__", 0);

  /* "(tree fragment)":5
 *     cdef object _dict
 *     cdef bint use_setstate
 *     state = (self.arena, self.mem, self.next, self.offset, self.prev, self.size)             # <<<<<<<<<<<<<<
 *     _dict = getattr(self, '__dict__', None)
 *     if _dict is not None and _dict:
*/
  __pyx_t_1 = __Pyx_PyLong_From_ptrdiff_t(__pyx_v_self->offset); if (unlikely(!__pyx_t_1)) __PYX_ERR(3, 5, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyLong_FromSize_t(__pyx_v_self->size); if (unlikely(!__pyx_t_2)) __PYX_ERR(3, 5, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = PyTuple_New(6); if (unlikely(!__pyx_t_3)) __PYX_ERR(3, 5, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_INCREF((PyObject *)__pyx_v_self->arena);
  __Pyx_GIVEREF((PyObject *)__pyx_v_self->arena);
  if (__Pyx_PyTuple_SET_ITEM(__pyx_t_3, 0, ((PyObject *)__pyx_v_self->arena)) != (0)) __PYX_ERR(3, 5, __pyx_L1_error);
  __Pyx_INCREF((PyObject *)__pyx_v_self->mem);
  __Pyx_GIVEREF((PyObject *)__pyx_v_self->mem);
  if (__Pyx_PyTuple_SET_ITEM(__pyx_t_3, 1, ((PyObject *)__pyx_v_self->mem)) != (0)) __PYX_ERR(3, 5, __pyx_L1_error);
  __Pyx_INCREF((PyObject *)__pyx_v_self->next);
  __Pyx_GIVEREF((PyObject *)__pyx_v_self->next);
  if (__Pyx_PyTuple_SET_ITEM(__pyx_t_3, 2, ((PyObject *)__pyx_v_self->next)) != (0)) __PYX_ERR(3, 5, __pyx_L1_error);
  __Pyx_GIVEREF(__pyx_t_1);
  if (__Pyx_PyTuple_SET_ITEM(__pyx_t_3, 3, __pyx_t_1) != (0)) __PYX_ERR(3, 5, __pyx_L1_error);
  __Pyx_INCREF((PyObject *)__pyx_v_self->prev);
  __Pyx_GIVEREF((PyObject *)__pyx_v_self->prev);
  if (__Pyx_PyTuple_SET_ITEM(__pyx_t_3, 4, ((PyObject *)__pyx_v_self->prev)) != (0)) __PYX_ERR(3, 5, __pyx_L1_error);
  __Pyx_GIVEREF(__pyx_t_2);
  if (__Pyx_PyTuple_SET_ITEM(__pyx_t_3, 5, __pyx_t_2) != (0)) __PYX_ERR(3, 5, __pyx_L1_error);
  __pyx_t_1 = 0;
  __pyx_t_2 = 0;
  __pyx_v_state = ((PyObject*)__pyx_t_3);
  __pyx_t_3 = 0;

  /* "(tree fragment)":6
 *     cdef bint use_setstate
 *     state = (self.arena, self.mem, self.next, self.offset, self.prev, self.size)
 *     _dict = getattr(self, '__dict__', None)             # <<<<<<<<<<<<<<
 *     if _dict is not None and _dict:
 *         state += (_dict,)
*/
  __pyx_t_3 = __Pyx_GetAttr3(((PyObject *)__pyx_v_self), __pyx_mstate_global->__pyx_n_u_dict, Py_None); if (unlikely(!__pyx_t_3)) __PYX_ERR(3, 6, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_v__dict = __pyx_t_3;
  __pyx_t_3 = 0;

  /* "(tree fragment)":7
 *     state = (self.arena, self.mem, self.next, self.offset, self.prev, self.size)
 *     _dict = getattr(self, '__dict__', None)
 *     if _dict is not None and _dict:             # <<<<<<<<<<<<<<
 *         state += (_dict,)
 *         use_setstate = True
*/
  __pyx_t_5 = (__pyx_v__dict != Py_None);
  if (__pyx_t_5) {
  } else {
    __pyx_t_4 = __pyx_t_5;
    goto __pyx_L4_bool_binop_done;
  }
  __pyx_t_5 = __Pyx_PyObject_IsTrue(__pyx_v__dict); if (unlikely((__pyx_t_5 < 0))) __PYX_ERR(3, 7, __pyx_L1_error)
  __pyx_t_4 = __pyx_t_5;
  __pyx_L4_bool_binop_done:;
  if (__pyx_t_4) {

    /* "(tree fragment)":8
 *     _dict = getattr(self, '__dict__', None)
 *     if _dict is not None and _dict:
 *         state += (_dict,)             # <<<<<<<<<<<<<<
 *         use_setstate = True
 *     else:
*/
    __pyx_t_3 = PyTuple_New(1); if (unlikely(!__pyx_t_3)) __PYX_ERR(3, 8, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_INCREF(__pyx_v__dict);
    __Pyx_GIVEREF(__pyx_v__dict);
    if (__Pyx_PyTuple_SET_ITEM(__pyx_t_3, 0, __pyx_v__dict) != (0)) __PYX_ERR(3, 8, __pyx_L1_error);
    __pyx_t_2 = PyNumber_InPlaceAdd(__pyx_v_state, __pyx_t_3); if (unlikely(!__pyx_t_2)) __PYX_ERR(3, 8, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __Pyx_DECREF_SET(__pyx_v_state, ((PyObject*)__pyx_t_2));
    __pyx_t_2 = 0;

    /* "(tree fragment)":9
 *     if _dict is not None and _dict:
 *         state += (_dict,)
 *         use_setstate = True             # <<<<<<<<<<<<<<
 *     else:
 *         use_setstate = self.arena is not None or self.mem is not None or self.next is not None or self.prev is not None
*/
    __pyx_v_use_setstate = 1;

    /* "(tree fragment)":7
 *     state = (self.arena, self.mem, self.next, self.offset, self.prev, self.size)
 *     _dict = getattr(self, '__dict__', None)
 *     if _dict is not None and _dict:             # <<<<<<<<<<<<<<
 *         state += (_dict,)
 *         use_setstate = True
*/
    goto __pyx_L3;
  }

  /* "(tree fragment)":11
 *         use_setstate = True
 *     else:
 *         use_setstate = self.arena is not None or self.mem is not None or self.next is not None or self.prev is not None             # <<<<<<<<<<<<<<
 *     if use_setstate:
 *         return __pyx_unpickle__Chunk, (type(self), 0x37829b1, None), state
*/
  /*else*/ {
    __pyx_t_5 = (((PyObject *)__pyx_v_self->arena) != Py_None);
    if (!__pyx_t_5) {
    } else {
      __pyx_t_4 = __pyx_t_5;
      goto __pyx_L6_bool_binop_done;
    }
    __pyx_t_5 = (((PyObject *)__pyx_v_self->mem) != Py_None);
    if (!__pyx_t_5) {
    } else {
      __pyx_t_4 = __pyx_t_5;
      goto __pyx_L6_bool_binop_done;
    }
    __pyx_t_5 = (((PyObject *)__pyx_v_self->next) != Py_None);
    if (!__pyx_t_5) {
    } else {
      __pyx_t_4 = __pyx_t_5;
      goto __pyx_L6_bool_binop_done;
    }
    __pyx_t_5 = (((PyObject *)__pyx_v_self->prev) != Py_None);
    __pyx_t_4 = __pyx_t_5;
    __pyx_L6_bool_binop_done:;
    __pyx_v_use_setstate = __pyx_t_4;
  }
  __pyx_L3:;

  /* "(tree fragment)":12
 *     else:
 *         use_setstate = self.arena is not None or self.mem is not None or self.next is not None or self.prev is not None
 *     if use_setstate:             # <<<<<<<<<<<<<<
 *         return __pyx_unpickle__Chunk, (type(self), 0x37829b1, None), state
 *     else:
*/
  if (__pyx_v_use_setstate) {

    /* "(tree fragment)":13
 *         use_setstate = self.arena is not None or self.mem is not None or self.next is not None or self.prev is not None
 *     if use_setstate:
 *         return __pyx_unpickle__Chunk, (type(self), 0x37829b1, None), state             # <<<<<<<<<<<<<<
 *     else:
 *         return __pyx_unpickle__Chunk, (type(self), 0x37829b1, state)
*/
    __Pyx_XDECREF(__pyx_r);
    __Pyx_GetModuleGlobalName(__pyx_t_2, __pyx_mstate_global->__pyx_n_u_pyx_unpickle__Chunk); if (unlikely(!__pyx_t_2)) __PYX_ERR(3, 13, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_3 = PyTuple_New(3); if (unlikely(!__pyx_t_3)) __PYX_ERR(3, 13, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_INCREF(((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    __Pyx_GIVEREF(((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    if (__Pyx_PyTuple_SET_ITEM(__pyx_t_3, 0, ((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self)))) != (0)) __PYX_ERR(3, 13, __pyx_L1_error);
    __Pyx_INCREF(__pyx_mstate_global->__pyx_int_58206641);
    __Pyx_GIVEREF(__pyx_mstate_global->__pyx_int_58206641);
    if (__Pyx_PyTuple_SET_ITEM(__pyx_t_3, 1, __pyx_mstate_global->__pyx_int_58206641) != (0)) __PYX_ERR(3, 13, __pyx_L1_error);
    __Pyx_INCREF(Py_None);
    __Pyx_GIVEREF(Py_None);
    if (__Pyx_PyTuple_SET_ITEM(__pyx_t_3, 2, Py_None) != (0)) __PYX_ERR(3, 13, __pyx_L1_error);
    __pyx_t_1 = PyTuple_New(3); if (unlikely(!__pyx_t_1)) __PYX_ERR(3, 13, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_GIVEREF(__pyx_t_2);
    if (__Pyx_PyTuple_SET_ITEM(__pyx_t_1, 0, __pyx_t_2) != (0)) __PYX_ERR(3, 13, __pyx_L1_error);
    __Pyx_GIVEREF(__pyx_t_3);
    if (__Pyx_PyTuple_SET_ITEM(__pyx_t_1, 1, __pyx_t_3) != (0)) __PYX_ERR(3, 13, __pyx_L1_error);
    __Pyx_INCREF(__pyx_v_state);
    __Pyx_GIVEREF(__pyx_v_state);
    if (__Pyx_PyTuple_SET_ITEM(__pyx_t_1, 2, __pyx_v_state) != (0)) __PYX_ERR(3, 13, __pyx_L1_error);
    __pyx_t_2 = 0;
    __pyx_t_3 = 0;
    __pyx_r = __pyx_t_1;
    __pyx_t_1 = 0;
    goto __pyx_L0;

    /* "(tree fragment)":12
 *     else:
 *         use_setstate = self.arena is not None or self.mem is not None or self.next is not None or self.prev is not None
 *     if use_setstate:             # <<<<<<<<<<<<<<
 *         return __pyx_unpickle__Chunk, (type(self), 0x37829b1, None), state
 *     else:
*/
  }

  /* "(tree fragment)":15
 *         return __pyx_unpickle__Chunk, (type(self), 0x37829b1, None), state
 *     else:
 *         return __pyx_unpickle__Chunk, (type(self), 0x37829b1, state)             # <<<<<<<<<<<<<<
 * def __setstate_cython__(self, __pyx_state):
 *     __pyx_unpickle__Chunk__set_state(self, __pyx_state)
*/
  /*else*/ {
    __Pyx_XDECREF(__pyx_r);
    __Pyx_GetModuleGlobalName(__pyx_t_1, __pyx_mstate_global->__pyx_n_u_pyx_unpickle__Chunk); if (unlikely(!__pyx_t_1)) __PYX_ERR(3, 15, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_t_3 = PyTuple_New(3); if (unlikely(!__pyx_t_3)) __PYX_ERR(3, 15, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_INCREF(((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    __Pyx_GIVEREF(((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    if (__Pyx_PyTuple_SET_ITEM(__pyx_t_3, 0, ((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self)))) != (0)) __PYX_ERR(3, 15, __pyx_L1_error);
    __Pyx_INCREF(__pyx_mstate_global->__pyx_int_58206641);
    __Pyx_GIVEREF(__pyx_mstate_global->__pyx_int_58206641);
    if (__Pyx_PyTuple_SET_ITEM(__pyx_t_3, 1, __pyx_mstate_global->__pyx_int_58206641) != (0)) __PYX_ERR(3, 15, __pyx_L1_error);
    __Pyx_INCREF(__pyx_v_state);
    __Pyx_GIVEREF(__pyx_v_state);
    if (__Pyx_PyTuple_SET_ITEM(__pyx_t_3, 2, __pyx_v_state) != (0)) __PYX_ERR(3, 15, __pyx_L1_error);
    __pyx_t_2 = PyTuple_New(2); if (unlikely(!__pyx_t_2)) __PYX_ERR(3, 15, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_GIVEREF(__pyx_t_1);
    if (__Pyx_PyTuple_SET_ITEM(__pyx_t_2, 0, __pyx_t_1) != (0)) __PYX_ERR(3, 15, __pyx_L1_error);
    __Pyx_GIVEREF(__pyx_t_3);
    if (__Pyx_PyTuple_SET_ITEM(__pyx_t_2, 1, __pyx_t_3) != (0)) __PYX_ERR(3, 15, __pyx_L1_error);
    __pyx_t_1 = 0;
    __pyx_t_3 = 0;
    __pyx_r = __pyx_t_2;
    __pyx_t_2 = 0;
    goto __pyx_L0;
  }

  /* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     cdef tuple state
 *     cdef object _dict
*/

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_AddTraceback("cupy.cuda.memory._Chunk.__reduce_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_state);
  __Pyx_XDECREF(__pyx_v__dict);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":16
 *     else:
 *         return __pyx_unpickle__Chunk, (type(self), 0x37829b1, state)
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     __pyx_unpickle__Chunk__set_state(self, __pyx_state)
*/

/* Python wrapper */
static PyObject *__pyx_pw_4cupy_4cuda_6memory_6_Chunk_13__setstate_cython__(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
PyDoc_STRVAR(__pyx_doc_4cupy_4cuda_6memory_6_Chunk_12__setstate_cython__, "_Chunk.__setstate_cython__(self, __pyx_state)");
static PyObject *__pyx_pw_4cupy_4cuda_6memory_6_Chunk_13__setstate_cython__(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
) {
  PyObject *__pyx_v___pyx_state = 0;
  #if !CYTHON_METH_FASTCALL
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  #endif
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject* values[1] = {0};
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__setstate_cython__ (wrapper)", 0);
  #if !CYTHON_METH_FASTCALL
  #if CYTHON_ASSUME_SAFE_SIZE
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return NULL;
  #endif
  #endif
  __pyx_kwvalues = __Pyx_KwValues_FASTCALL(__pyx_args, __pyx_nargs);
  {
    PyObject ** const __pyx_pyargnames[] = {&__pyx_mstate_global->__pyx_n_u_pyx_state,0};
    const Py_ssize_t __pyx_kwds_len = (__pyx_kwds) ? __Pyx_NumKwargs_FASTCALL(__pyx_kwds) : 0;
    if (unlikely(__pyx_kwds_len < 0)) __PYX_ERR(3, 16, __pyx_L3_error)
    if (__pyx_kwds_len > 0) {
      switch (__pyx_nargs) {
        case  1:
        values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(3, 16, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      const Py_ssize_t kwd_pos_args = __pyx_nargs;
      if (__Pyx_ParseKeywords(__pyx_kwds, __pyx_kwvalues, __pyx_pyargnames, 0, values, kwd_pos_args, __pyx_kwds_len, "__setstate_cython__", 0) < (0)) __PYX_ERR(3, 16, __pyx_L3_error)
      for (Py_ssize_t i = __pyx_nargs; i < 1; i++) {
        if (unlikely(!values[i])) { __Pyx_RaiseArgtupleInvalid("__setstate_cython__", 1, 1, 1, i); __PYX_ERR(3, 16, __pyx_L3_error) }
      }
    } else if (unlikely(__pyx_nargs != 1)) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
      if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(3, 16, __pyx_L3_error)
    }
    __pyx_v___pyx_state = values[0];
  }
  goto __pyx_L6_skip;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("__setstate_cython__", 1, 1, 1, __pyx_nargs); __PYX_ERR(3, 16, __pyx_L3_error)
  __pyx_L6_skip:;
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_AddTraceback("cupy.cuda.memory._Chunk.__setstate_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_4cupy_4cuda_6memory_6_Chunk_12__setstate_cython__(((struct __pyx_obj_4cupy_4cuda_6memory__Chunk *)__pyx_v_self), __pyx_v___pyx_state);

  /* function exit code */
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4cupy_4cuda_6memory_6_Chunk_12__setstate_cython__(struct __pyx_obj_4cupy_4cuda_6memory__Chunk *__pyx_v_self, PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__setstate_cython__", 0);

  /* "(tree fragment)":17
 *         return __pyx_unpickle__Chunk, (type(self), 0x37829b1, state)
 * def __setstate_cython__(self, __pyx_state):
 *     __pyx_unpickle__Chunk__set_state(self, __pyx_state)             # <<<<<<<<<<<<<<
*/
  __pyx_t_1 = __pyx_v___pyx_state;
  __Pyx_INCREF(__pyx_t_1);
  if (!(likely(PyTuple_CheckExact(__pyx_t_1))||((__pyx_t_1) == Py_None) || __Pyx_RaiseUnexpectedTypeError("tuple", __pyx_t_1))) __PYX_ERR(3, 17, __pyx_L1_error)
  if (unlikely(__pyx_t_1 == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "cannot pass None into a C function argument that is declared 'not None'");
    __PYX_ERR(3, 17, __pyx_L1_error)
  }
  __pyx_t_2 = __pyx_f_4cupy_4cuda_6memory___pyx_unpickle__Chunk__set_state(__pyx_v_self, ((PyObject*)__pyx_t_1)); if (unlikely(!__pyx_t_2)) __PYX_ERR(3, 17, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "(tree fragment)":16
 *     else:
 *         return __pyx_unpickle__Chunk, (type(self), 0x37829b1, state)
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     __pyx_unpickle__Chunk__set_state(self, __pyx_state)
*/

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_AddTraceback("cupy.cuda.memory._Chunk.__setstate_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "cupy/cuda/memory.pyx":476
 *     """
 * 
 *     def __init__(self, BaseMemory mem, ptrdiff_t offset):             # <<<<<<<<<<<<<<
 *         self._init(mem, offset)
 * 
*/

/* Python wrapper */
static int __pyx_pw_4cupy_4cuda_6memory_13MemoryPointer_1__init__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static int __pyx_pw_4cupy_4cuda_6memory_13MemoryPointer_1__init__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  struct __pyx_obj_4cupy_4cuda_6memory_BaseMemory *__pyx_v_mem = 0;
  ptrdiff_t __pyx_v_offset;
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject* values[2] = {0,0};
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__init__ (wrapper)", 0);
  #if CYTHON_ASSUME_SAFE_SIZE
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return -1;
  #endif
  __pyx_kwvalues = __Pyx_KwValues_VARARGS(__pyx_args, __pyx_nargs);
  {
    PyObject ** const __pyx_pyargnames[] = {&__pyx_mstate_global->__pyx_n_u_mem_2,&__pyx_mstate_global->__pyx_n_u_offset_2,0};
    const Py_ssize_t __pyx_kwds_len = (__pyx_kwds) ? __Pyx_NumKwargs_VARARGS(__pyx_kwds) : 0;
    if (unlikely(__pyx_kwds_len < 0)) __PYX_ERR(0, 476, __pyx_L3_error)
    if (__pyx_kwds_len > 0) {
      switch (__pyx_nargs) {
        case  2:
        values[1] = __Pyx_ArgRef_VARARGS(__pyx_args, 1);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[1])) __PYX_ERR(0, 476, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  1:
        values[0] = __Pyx_ArgRef_VARARGS(__pyx_args, 0);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(0, 476, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      const Py_ssize_t kwd_pos_args = __pyx_nargs;
      if (__Pyx_ParseKeywords(__pyx_kwds, __pyx_kwvalues, __pyx_pyargnames, 0, values, kwd_pos_args, __pyx_kwds_len, "__init__", 0) < (0)) __PYX_ERR(0, 476, __pyx_L3_error)
      for (Py_ssize_t i = __pyx_nargs; i < 2; i++) {
        if (unlikely(!values[i])) { __Pyx_RaiseArgtupleInvalid("__init__", 1, 2, 2, i); __PYX_ERR(0, 476, __pyx_L3_error) }
      }
    } else if (unlikely(__pyx_nargs != 2)) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = __Pyx_ArgRef_VARARGS(__pyx_args, 0);
      if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(0, 476, __pyx_L3_error)
      values[1] = __Pyx_ArgRef_VARARGS(__pyx_args, 1);
      if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[1])) __PYX_ERR(0, 476, __pyx_L3_error)
    }
    __pyx_v_mem = ((struct __pyx_obj_4cupy_4cuda_6memory_BaseMemory *)values[0]);
    __pyx_v_offset = __Pyx_PyLong_As_ptrdiff_t(values[1]); if (unlikely((__pyx_v_offset == (ptrdiff_t)-1) && PyErr_Occurred())) __PYX_ERR(0, 476, __pyx_L3_error)
  }
  goto __pyx_L6_skip;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("__init__", 1, 2, 2, __pyx_nargs); __PYX_ERR(0, 476, __pyx_L3_error)
  __pyx_L6_skip:;
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_AddTraceback("cupy.cuda.memory.MemoryPointer.__init__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return -1;
  __pyx_L4_argument_unpacking_done:;
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_mem), __pyx_mstate_global->__pyx_ptype_4cupy_4cuda_6memory_BaseMemory, 1, "mem", 0))) __PYX_ERR(0, 476, __pyx_L1_error)
  __pyx_r = __pyx_pf_4cupy_4cuda_6memory_13MemoryPointer___init__(((struct __pyx_obj_4cupy_4cuda_6memory_MemoryPointer *)__pyx_v_self), __pyx_v_mem, __pyx_v_offset);

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = -1;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  goto __pyx_L7_cleaned_up;
  __pyx_L0:;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __pyx_L7_cleaned_up:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_4cupy_4cuda_6memory_13MemoryPointer___init__(struct __pyx_obj_4cupy_4cuda_6memory_MemoryPointer *__pyx_v_self, struct __pyx_obj_4cupy_4cuda_6memory_BaseMemory *__pyx_v_mem, ptrdiff_t __pyx_v_offset) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__init__", 0);

  /* "cupy/cuda/memory.pyx":477
 * 
 *     def __init__(self, BaseMemory mem, ptrdiff_t offset):
 *         self._init(mem, offset)             # <<<<<<<<<<<<<<
 * 
 *     cdef _init(self, BaseMemory mem, ptrdiff_t offset):
*/
  __pyx_t_1 = ((struct __pyx_vtabstruct_4cupy_4cuda_6memory_MemoryPointer *)__pyx_v_self->__pyx_vtab)->_init(__pyx_v_self, __pyx_v_mem, __pyx_v_offset); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 477, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "cupy/cuda/memory.pyx":476
 *     """
 * 
 *     def __init__(self, BaseMemory mem, ptrdiff_t offset):             # <<<<<<<<<<<<<<
 *         self._init(mem, offset)
 * 
*/

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("cupy.cuda.memory.MemoryPointer.__init__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "cupy/cuda/memory.pyx":479
 *         self._init(mem, offset)
 * 
 *     cdef _init(self, BaseMemory mem, ptrdiff_t offset):             # <<<<<<<<<<<<<<
 *         assert mem.ptr != 0 or offset == 0
 *         self.ptr = mem.ptr + offset
*/

static PyObject *__pyx_f_4cupy_4cuda_6memory_13MemoryPointer__init(struct __pyx_obj_4cupy_4cuda_6memory_MemoryPointer *__pyx_v_self, struct __pyx_obj_4cupy_4cuda_6memory_BaseMemory *__pyx_v_mem, ptrdiff_t __pyx_v_offset) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  int __pyx_t_2;
  int __pyx_t_3;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("_init", 0);

  /* "cupy/cuda/memory.pyx":480
 * 
 *     cdef _init(self, BaseMemory mem, ptrdiff_t offset):
 *         assert mem.ptr != 0 or offset == 0             # <<<<<<<<<<<<<<
 *         self.ptr = mem.ptr + offset
 *         self.device_id = mem.device_id
*/
  #ifndef CYTHON_WITHOUT_ASSERTIONS
  if (unlikely(__pyx_assertions_enabled())) {
    __pyx_t_2 = (__pyx_v_mem->ptr != 0);
    if (!__pyx_t_2) {
    } else {
      __pyx_t_1 = __pyx_t_2;
      goto __pyx_L3_bool_binop_done;
    }
    __pyx_t_2 = (__pyx_v_offset == 0);
    __pyx_t_1 = __pyx_t_2;
    __pyx_L3_bool_binop_done:;
    if (unlikely(!__pyx_t_1)) {
      __Pyx_Raise(((PyObject *)(((PyTypeObject*)PyExc_AssertionError))), 0, 0, 0);
      __PYX_ERR(0, 480, __pyx_L1_error)
    }
  }
  #else
  if ((1)); else __PYX_ERR(0, 480, __pyx_L1_error)
  #endif

  /* "cupy/cuda/memory.pyx":481
 *     cdef _init(self, BaseMemory mem, ptrdiff_t offset):
 *         assert mem.ptr != 0 or offset == 0
 *         self.ptr = mem.ptr + offset             # <<<<<<<<<<<<<<
 *         self.device_id = mem.device_id
 *         self.mem = mem
*/
  __pyx_v_self->ptr = (__pyx_v_mem->ptr + __pyx_v_offset);

  /* "cupy/cuda/memory.pyx":482
 *         assert mem.ptr != 0 or offset == 0
 *         self.ptr = mem.ptr + offset
 *         self.device_id = mem.device_id             # <<<<<<<<<<<<<<
 *         self.mem = mem
 * 
*/
  __pyx_t_3 = __pyx_v_mem->device_id;
  __pyx_v_self->device_id = __pyx_t_3;

  /* "cupy/cuda/memory.pyx":483
 *         self.ptr = mem.ptr + offset
 *         self.device_id = mem.device_id
 *         self.mem = mem             # <<<<<<<<<<<<<<
 * 
 *     def __int__(self):
*/
  __Pyx_INCREF((PyObject *)__pyx_v_mem);
  __Pyx_GIVEREF((PyObject *)__pyx_v_mem);
  __Pyx_GOTREF((PyObject *)__pyx_v_self->mem);
  __Pyx_DECREF((PyObject *)__pyx_v_self->mem);
  __pyx_v_self->mem = __pyx_v_mem;

  /* "cupy/cuda/memory.pyx":479
 *         self._init(mem, offset)
 * 
 *     cdef _init(self, BaseMemory mem, ptrdiff_t offset):             # <<<<<<<<<<<<<<
 *         assert mem.ptr != 0 or offset == 0
 *         self.ptr = mem.ptr + offset
*/

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_AddTraceback("cupy.cuda.memory.MemoryPointer._init", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "cupy/cuda/memory.pyx":485
 *         self.mem = mem
 * 
 *     def __int__(self):             # <<<<<<<<<<<<<<
 *         """Returns the pointer value."""
 *         return self.ptr
*/

/* Python wrapper */
static PyObject *__pyx_pw_4cupy_4cuda_6memory_13MemoryPointer_3__int__(PyObject *__pyx_v_self); /*proto*/
PyDoc_STRVAR(__pyx_doc_4cupy_4cuda_6memory_13MemoryPointer_2__int__, "MemoryPointer.__int__(self)\n\nReturns the pointer value.");
#if CYTHON_UPDATE_DESCRIPTOR_DOC
struct wrapperbase __pyx_wrapperbase_4cupy_4cuda_6memory_13MemoryPointer_2__int__;
#endif
static PyObject *__pyx_pw_4cupy_4cuda_6memory_13MemoryPointer_3__int__(PyObject *__pyx_v_self) {
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__int__ (wrapper)", 0);
  __pyx_kwvalues = __Pyx_KwValues_VARARGS(__pyx_args, __pyx_nargs);
  __pyx_r = __pyx_pf_4cupy_4cuda_6memory_13MemoryPointer_2__int__(((struct __pyx_obj_4cupy_4cuda_6memory_MemoryPointer *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4cupy_4cuda_6memory_13MemoryPointer_2__int__(struct __pyx_obj_4cupy_4cuda_6memory_MemoryPointer *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__int__", 0);

  /* "cupy/cuda/memory.pyx":487
 *     def __int__(self):
 *         """Returns the pointer value."""
 *         return self.ptr             # <<<<<<<<<<<<<<
 * 
 *     def __repr__(self):
*/
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = PyLong_FromSsize_t(__pyx_v_self->ptr); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 487, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "cupy/cuda/memory.pyx":485
 *         self.mem = mem
 * 
 *     def __int__(self):             # <<<<<<<<<<<<<<
 *         """Returns the pointer value."""
 *         return self.ptr
*/

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("cupy.cuda.memory.MemoryPointer.__int__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "cupy/cuda/memory.pyx":489
 *         return self.ptr
 * 
 *     def __repr__(self):             # <<<<<<<<<<<<<<
 *         return '<{} 0x{:x} device={} mem={!r}>'.format(
 *             self.__class__.__name__,
*/

/* Python wrapper */
static PyObject *__pyx_pw_4cupy_4cuda_6memory_13MemoryPointer_5__repr__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_4cupy_4cuda_6memory_13MemoryPointer_5__repr__(PyObject *__pyx_v_self) {
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__repr__ (wrapper)", 0);
  __pyx_kwvalues = __Pyx_KwValues_VARARGS(__pyx_args, __pyx_nargs);
  __pyx_r = __pyx_pf_4cupy_4cuda_6memory_13MemoryPointer_4__repr__(((struct __pyx_obj_4cupy_4cuda_6memory_MemoryPointer *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4cupy_4cuda_6memory_13MemoryPointer_4__repr__(struct __pyx_obj_4cupy_4cuda_6memory_MemoryPointer *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  size_t __pyx_t_6;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__repr__", 0);

  /* "cupy/cuda/memory.pyx":490
 * 
 *     def __repr__(self):
 *         return '<{} 0x{:x} device={} mem={!r}>'.format(             # <<<<<<<<<<<<<<
 *             self.__class__.__name__,
 *             self.ptr, self.device_id, self.mem)
*/
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_2 = __pyx_mstate_global->__pyx_kp_u_0x_x_device_mem_r;
  __Pyx_INCREF(__pyx_t_2);

  /* "cupy/cuda/memory.pyx":491
 *     def __repr__(self):
 *         return '<{} 0x{:x} device={} mem={!r}>'.format(
 *             self.__class__.__name__,             # <<<<<<<<<<<<<<
 *             self.ptr, self.device_id, self.mem)
 * 
*/
  __pyx_t_3 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_mstate_global->__pyx_n_u_class); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 491, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_4 = __Pyx_PyObject_GetAttrStr(__pyx_t_3, __pyx_mstate_global->__pyx_n_u_name); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 491, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "cupy/cuda/memory.pyx":492
 *         return '<{} 0x{:x} device={} mem={!r}>'.format(
 *             self.__class__.__name__,
 *             self.ptr, self.device_id, self.mem)             # <<<<<<<<<<<<<<
 * 
 *     @property
*/
  __pyx_t_3 = PyLong_FromSsize_t(__pyx_v_self->ptr); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 492, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_5 = __Pyx_PyLong_From_int(__pyx_v_self->device_id); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 492, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_5);
  __pyx_t_6 = 0;
  {
    PyObject *__pyx_callargs[5] = {__pyx_t_2, __pyx_t_4, __pyx_t_3, __pyx_t_5, ((PyObject *)__pyx_v_self->mem)};
    __pyx_t_1 = __Pyx_PyObject_FastCallMethod((PyObject*)__pyx_mstate_global->__pyx_n_u_format, __pyx_callargs+__pyx_t_6, (5-__pyx_t_6) | (1*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
    __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
    if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 490, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
  }
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "cupy/cuda/memory.pyx":489
 *         return self.ptr
 * 
 *     def __repr__(self):             # <<<<<<<<<<<<<<
 *         return '<{} 0x{:x} device={} mem={!r}>'.format(
 *             self.__class__.__name__,
*/

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_AddTraceback("cupy.cuda.memory.MemoryPointer.__repr__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "cupy/cuda/memory.pyx":494
 *             self.ptr, self.device_id, self.mem)
 * 
 *     @property             # <<<<<<<<<<<<<<
 *     def device(self):
 *         return device.Device(self.device_id)
*/

/* Python wrapper */
static PyObject *__pyx_pw_4cupy_4cuda_6memory_13MemoryPointer_6device_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_4cupy_4cuda_6memory_13MemoryPointer_6device_1__get__(PyObject *__pyx_v_self) {
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_kwvalues = __Pyx_KwValues_VARARGS(__pyx_args, __pyx_nargs);
  __pyx_r = __pyx_pf_4cupy_4cuda_6memory_13MemoryPointer_6device___get__(((struct __pyx_obj_4cupy_4cuda_6memory_MemoryPointer *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4cupy_4cuda_6memory_13MemoryPointer_6device___get__(struct __pyx_obj_4cupy_4cuda_6memory_MemoryPointer *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  size_t __pyx_t_4;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__get__", 0);

  /* "cupy/cuda/memory.pyx":496
 *     @property
 *     def device(self):
 *         return device.Device(self.device_id)             # <<<<<<<<<<<<<<
 * 
 *     def __add__(x, y):
*/
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_2 = NULL;
  __pyx_t_3 = __Pyx_PyLong_From_int(__pyx_v_self->device_id); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 496, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_4 = 1;
  {
    PyObject *__pyx_callargs[2] = {__pyx_t_2, __pyx_t_3};
    __pyx_t_1 = __Pyx_PyObject_FastCall((PyObject*)__pyx_mstate_global->__pyx_ptype_4cupy_4cuda_6device_Device, __pyx_callargs+__pyx_t_4, (2-__pyx_t_4) | (__pyx_t_4*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
    __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 496, __pyx_L1_error)
    __Pyx_GOTREF((PyObject *)__pyx_t_1);
  }
  __pyx_r = ((PyObject *)__pyx_t_1);
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "cupy/cuda/memory.pyx":494
 *             self.ptr, self.device_id, self.mem)
 * 
 *     @property             # <<<<<<<<<<<<<<
 *     def device(self):
 *         return device.Device(self.device_id)
*/

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_AddTraceback("cupy.cuda.memory.MemoryPointer.device.__get__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "cupy/cuda/memory.pyx":498
 *         return device.Device(self.device_id)
 * 
 *     def __add__(x, y):             # <<<<<<<<<<<<<<
 *         """Adds an offset to the pointer."""
 *         cdef MemoryPointer self
*/

/* Python wrapper */
static PyObject *__pyx_pw_4cupy_4cuda_6memory_13MemoryPointer_7__add__(PyObject *__pyx_v_x, PyObject *__pyx_v_y); /*proto*/
PyDoc_STRVAR(__pyx_doc_4cupy_4cuda_6memory_13MemoryPointer_6__add__, "MemoryPointer.__add__(x, y)\n\nAdds an offset to the pointer.");
#if CYTHON_UPDATE_DESCRIPTOR_DOC
struct wrapperbase __pyx_wrapperbase_4cupy_4cuda_6memory_13MemoryPointer_6__add__;
#endif
static PyObject *__pyx_pw_4cupy_4cuda_6memory_13MemoryPointer_7__add__(PyObject *__pyx_v_x, PyObject *__pyx_v_y) {
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__add__ (wrapper)", 0);
  __pyx_kwvalues = __Pyx_KwValues_VARARGS(__pyx_args, __pyx_nargs);
  __pyx_r = __pyx_pf_4cupy_4cuda_6memory_13MemoryPointer_6__add__(((PyObject *)__pyx_v_x), ((PyObject *)__pyx_v_y));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4cupy_4cuda_6memory_13MemoryPointer_6__add__(PyObject *__pyx_v_x, PyObject *__pyx_v_y) {
  struct __pyx_obj_4cupy_4cuda_6memory_MemoryPointer *__pyx_v_self = 0;
  ptrdiff_t __pyx_v_offset;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  ptrdiff_t __pyx_t_3;
  int __pyx_t_4;
  PyObject *__pyx_t_5 = NULL;
  PyObject *__pyx_t_6 = NULL;
  size_t __pyx_t_7;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__add__", 0);

  /* "cupy/cuda/memory.pyx":502
 *         cdef MemoryPointer self
 *         cdef ptrdiff_t offset
 *         if isinstance(x, MemoryPointer):             # <<<<<<<<<<<<<<
 *             self = x
 *             offset = <ptrdiff_t?>y
*/
  __pyx_t_1 = __Pyx_TypeCheck(__pyx_v_x, __pyx_mstate_global->__pyx_ptype_4cupy_4cuda_6memory_MemoryPointer); 
  if (__pyx_t_1) {

    /* "cupy/cuda/memory.pyx":503
 *         cdef ptrdiff_t offset
 *         if isinstance(x, MemoryPointer):
 *             self = x             # <<<<<<<<<<<<<<
 *             offset = <ptrdiff_t?>y
 *         else:
*/
    __pyx_t_2 = __pyx_v_x;
    __Pyx_INCREF(__pyx_t_2);
    if (!(likely(((__pyx_t_2) == Py_None) || likely(__Pyx_TypeTest(__pyx_t_2, __pyx_mstate_global->__pyx_ptype_4cupy_4cuda_6memory_MemoryPointer))))) __PYX_ERR(0, 503, __pyx_L1_error)
    __pyx_v_self = ((struct __pyx_obj_4cupy_4cuda_6memory_MemoryPointer *)__pyx_t_2);
    __pyx_t_2 = 0;

    /* "cupy/cuda/memory.pyx":504
 *         if isinstance(x, MemoryPointer):
 *             self = x
 *             offset = <ptrdiff_t?>y             # <<<<<<<<<<<<<<
 *         else:
 *             self = <MemoryPointer?>y
*/
    __pyx_t_3 = __Pyx_PyLong_As_ptrdiff_t(__pyx_v_y); if (unlikely((__pyx_t_3 == (ptrdiff_t)-1) && PyErr_Occurred())) __PYX_ERR(0, 504, __pyx_L1_error)
    __pyx_v_offset = ((ptrdiff_t)__pyx_t_3);

    /* "cupy/cuda/memory.pyx":502
 *         cdef MemoryPointer self
 *         cdef ptrdiff_t offset
 *         if isinstance(x, MemoryPointer):             # <<<<<<<<<<<<<<
 *             self = x
 *             offset = <ptrdiff_t?>y
*/
    goto __pyx_L3;
  }

  /* "cupy/cuda/memory.pyx":506
 *             offset = <ptrdiff_t?>y
 *         else:
 *             self = <MemoryPointer?>y             # <<<<<<<<<<<<<<
 *             offset = <ptrdiff_t?>x
 *         assert self.ptr != 0 or offset == 0
*/
  /*else*/ {
    if (!(likely(__Pyx_TypeTest(__pyx_v_y, __pyx_mstate_global->__pyx_ptype_4cupy_4cuda_6memory_MemoryPointer)))) __PYX_ERR(0, 506, __pyx_L1_error)
    __pyx_t_2 = __pyx_v_y;
    __Pyx_INCREF(__pyx_t_2);
    __pyx_v_self = ((struct __pyx_obj_4cupy_4cuda_6memory_MemoryPointer *)__pyx_t_2);
    __pyx_t_2 = 0;

    /* "cupy/cuda/memory.pyx":507
 *         else:
 *             self = <MemoryPointer?>y
 *             offset = <ptrdiff_t?>x             # <<<<<<<<<<<<<<
 *         assert self.ptr != 0 or offset == 0
 *         return MemoryPointer(self.mem,
*/
    __pyx_t_3 = __Pyx_PyLong_As_ptrdiff_t(__pyx_v_x); if (unlikely((__pyx_t_3 == (ptrdiff_t)-1) && PyErr_Occurred())) __PYX_ERR(0, 507, __pyx_L1_error)
    __pyx_v_offset = ((ptrdiff_t)__pyx_t_3);
  }
  __pyx_L3:;

  /* "cupy/cuda/memory.pyx":508
 *             self = <MemoryPointer?>y
 *             offset = <ptrdiff_t?>x
 *         assert self.ptr != 0 or offset == 0             # <<<<<<<<<<<<<<
 *         return MemoryPointer(self.mem,
 *                              self.ptr - self.mem.ptr + offset)
*/
  #ifndef CYTHON_WITHOUT_ASSERTIONS
  if (unlikely(__pyx_assertions_enabled())) {
    __pyx_t_4 = (__pyx_v_self->ptr != 0);
    if (!__pyx_t_4) {
    } else {
      __pyx_t_1 = __pyx_t_4;
      goto __pyx_L4_bool_binop_done;
    }
    __pyx_t_4 = (__pyx_v_offset == 0);
    __pyx_t_1 = __pyx_t_4;
    __pyx_L4_bool_binop_done:;
    if (unlikely(!__pyx_t_1)) {
      __Pyx_Raise(((PyObject *)(((PyTypeObject*)PyExc_AssertionError))), 0, 0, 0);
      __PYX_ERR(0, 508, __pyx_L1_error)
    }
  }
  #else
  if ((1)); else __PYX_ERR(0, 508, __pyx_L1_error)
  #endif

  /* "cupy/cuda/memory.pyx":509
 *             offset = <ptrdiff_t?>x
 *         assert self.ptr != 0 or offset == 0
 *         return MemoryPointer(self.mem,             # <<<<<<<<<<<<<<
 *                              self.ptr - self.mem.ptr + offset)
 * 
*/
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_5 = NULL;

  /* "cupy/cuda/memory.pyx":510
 *         assert self.ptr != 0 or offset == 0
 *         return MemoryPointer(self.mem,
 *                              self.ptr - self.mem.ptr + offset)             # <<<<<<<<<<<<<<
 * 
 *     def __iadd__(self, ptrdiff_t offset):
*/
  __pyx_t_6 = __Pyx_PyLong_From_ptrdiff_t(((__pyx_v_self->ptr - __pyx_v_self->mem->ptr) + __pyx_v_offset)); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 510, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_6);
  __pyx_t_7 = 1;
  {
    PyObject *__pyx_callargs[3] = {__pyx_t_5, ((PyObject *)__pyx_v_self->mem), __pyx_t_6};
    __pyx_t_2 = __Pyx_PyObject_FastCall((PyObject*)__pyx_mstate_global->__pyx_ptype_4cupy_4cuda_6memory_MemoryPointer, __pyx_callargs+__pyx_t_7, (3-__pyx_t_7) | (__pyx_t_7*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
    __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
    __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
    if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 509, __pyx_L1_error)
    __Pyx_GOTREF((PyObject *)__pyx_t_2);
  }
  __pyx_r = ((PyObject *)__pyx_t_2);
  __pyx_t_2 = 0;
  goto __pyx_L0;

  /* "cupy/cuda/memory.pyx":498
 *         return device.Device(self.device_id)
 * 
 *     def __add__(x, y):             # <<<<<<<<<<<<<<
 *         """Adds an offset to the pointer."""
 *         cdef MemoryPointer self
*/

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_AddTraceback("cupy.cuda.memory.MemoryPointer.__add__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF((PyObject *)__pyx_v_self);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "cupy/cuda/memory.pyx":512
 *                              self.ptr - self.mem.ptr + offset)
 * 
 *     def __iadd__(self, ptrdiff_t offset):             # <<<<<<<<<<<<<<
 *         """Adds an offset to the pointer in place."""
 *         assert self.ptr != 0 or offset == 0
*/

/* Python wrapper */
static PyObject *__pyx_pw_4cupy_4cuda_6memory_13MemoryPointer_9__iadd__(PyObject *__pyx_v_self, PyObject *__pyx_arg_offset); /*proto*/
PyDoc_STRVAR(__pyx_doc_4cupy_4cuda_6memory_13MemoryPointer_8__iadd__, "MemoryPointer.__iadd__(self, ptrdiff_t offset)\n\nAdds an offset to the pointer in place.");
#if CYTHON_UPDATE_DESCRIPTOR_DOC
struct wrapperbase __pyx_wrapperbase_4cupy_4cuda_6memory_13MemoryPointer_8__iadd__;
#endif
static PyObject *__pyx_pw_4cupy_4cuda_6memory_13MemoryPointer_9__iadd__(PyObject *__pyx_v_self, PyObject *__pyx_arg_offset) {
  ptrdiff_t __pyx_v_offset;
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__iadd__ (wrapper)", 0);
  __pyx_kwvalues = __Pyx_KwValues_VARARGS(__pyx_args, __pyx_nargs);
  assert(__pyx_arg_offset); {
    __pyx_v_offset = __Pyx_PyLong_As_ptrdiff_t(__pyx_arg_offset); if (unlikely((__pyx_v_offset == (ptrdiff_t)-1) && PyErr_Occurred())) __PYX_ERR(0, 512, __pyx_L3_error)
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  __Pyx_AddTraceback("cupy.cuda.memory.MemoryPointer.__iadd__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_4cupy_4cuda_6memory_13MemoryPointer_8__iadd__(((struct __pyx_obj_4cupy_4cuda_6memory_MemoryPointer *)__pyx_v_self), ((ptrdiff_t)__pyx_v_offset));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4cupy_4cuda_6memory_13MemoryPointer_8__iadd__(struct __pyx_obj_4cupy_4cuda_6memory_MemoryPointer *__pyx_v_self, ptrdiff_t __pyx_v_offset) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  int __pyx_t_2;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__iadd__", 0);

  /* "cupy/cuda/memory.pyx":514
 *     def __iadd__(self, ptrdiff_t offset):
 *         """Adds an offset to the pointer in place."""
 *         assert self.ptr != 0 or offset == 0             # <<<<<<<<<<<<<<
 *         self.ptr += offset
 *         return self
*/
  #ifndef CYTHON_WITHOUT_ASSERTIONS
  if (unlikely(__pyx_assertions_enabled())) {
    __pyx_t_2 = (__pyx_v_self->ptr != 0);
    if (!__pyx_t_2) {
    } else {
      __pyx_t_1 = __pyx_t_2;
      goto __pyx_L3_bool_binop_done;
    }
    __pyx_t_2 = (__pyx_v_offset == 0);
    __pyx_t_1 = __pyx_t_2;
    __pyx_L3_bool_binop_done:;
    if (unlikely(!__pyx_t_1)) {
      __Pyx_Raise(((PyObject *)(((PyTypeObject*)PyExc_AssertionError))), 0, 0, 0);
      __PYX_ERR(0, 514, __pyx_L1_error)
    }
  }
  #else
  if ((1)); else __PYX_ERR(0, 514, __pyx_L1_error)
  #endif

  /* "cupy/cuda/memory.pyx":515
 *         """Adds an offset to the pointer in place."""
 *         assert self.ptr != 0 or offset == 0
 *         self.ptr += offset             # <<<<<<<<<<<<<<
 *         return self
 * 
*/
  __pyx_v_self->ptr = (__pyx_v_self->ptr + __pyx_v_offset);

  /* "cupy/cuda/memory.pyx":516
 *         assert self.ptr != 0 or offset == 0
 *         self.ptr += offset
 *         return self             # <<<<<<<<<<<<<<
 * 
 *     def __sub__(self, offset):
*/
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF((PyObject *)__pyx_v_self);
  __pyx_r = ((PyObject *)__pyx_v_self);
  goto __pyx_L0;

  /* "cupy/cuda/memory.pyx":512
 *                              self.ptr - self.mem.ptr + offset)
 * 
 *     def __iadd__(self, ptrdiff_t offset):             # <<<<<<<<<<<<<<
 *         """Adds an offset to the pointer in place."""
 *         assert self.ptr != 0 or offset == 0
*/

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_AddTraceback("cupy.cuda.memory.MemoryPointer.__iadd__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "cupy/cuda/memory.pyx":518
 *         return self
 * 
 *     def __sub__(self, offset):             # <<<<<<<<<<<<<<
 *         """Subtracts an offset from the pointer."""
 *         return self + -offset
*/

/* Python wrapper */
static PyObject *__pyx_pw_4cupy_4cuda_6memory_13MemoryPointer_11__sub__(PyObject *__pyx_v_self, PyObject *__pyx_v_offset); /*proto*/
PyDoc_STRVAR(__pyx_doc_4cupy_4cuda_6memory_13MemoryPointer_10__sub__, "MemoryPointer.__sub__(self, offset)\n\nSubtracts an offset from the pointer.");
#if CYTHON_UPDATE_DESCRIPTOR_DOC
struct wrapperbase __pyx_wrapperbase_4cupy_4cuda_6memory_13MemoryPointer_10__sub__;
#endif
static PyObject *__pyx_pw_4cupy_4cuda_6memory_13MemoryPointer_11__sub__(PyObject *__pyx_v_self, PyObject *__pyx_v_offset) {
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__sub__ (wrapper)", 0);
  __pyx_kwvalues = __Pyx_KwValues_VARARGS(__pyx_args, __pyx_nargs);
  __pyx_r = __pyx_pf_4cupy_4cuda_6memory_13MemoryPointer_10__sub__(((PyObject *)__pyx_v_self), ((PyObject *)__pyx_v_offset));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4cupy_4cuda_6memory_13MemoryPointer_10__sub__(PyObject *__pyx_v_self, PyObject *__pyx_v_offset) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__sub__", 0);

  /* "cupy/cuda/memory.pyx":520
 *     def __sub__(self, offset):
 *         """Subtracts an offset from the pointer."""
 *         return self + -offset             # <<<<<<<<<<<<<<
 * 
 *     def __isub__(self, ptrdiff_t offset):
*/
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = PyNumber_Negative(__pyx_v_offset); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 520, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = PyNumber_Add(__pyx_v_self, __pyx_t_1); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 520, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_r = __pyx_t_2;
  __pyx_t_2 = 0;
  goto __pyx_L0;

  /* "cupy/cuda/memory.pyx":518
 *         return self
 * 
 *     def __sub__(self, offset):             # <<<<<<<<<<<<<<
 *         """Subtracts an offset from the pointer."""
 *         return self + -offset
*/

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_AddTraceback("cupy.cuda.memory.MemoryPointer.__sub__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "cupy/cuda/memory.pyx":522
 *         return self + -offset
 * 
 *     def __isub__(self, ptrdiff_t offset):             # <<<<<<<<<<<<<<
 *         """Subtracts an offset from the pointer in place."""
 *         return self.__iadd__(-offset)
*/

/* Python wrapper */
static PyObject *__pyx_pw_4cupy_4cuda_6memory_13MemoryPointer_13__isub__(PyObject *__pyx_v_self, PyObject *__pyx_arg_offset); /*proto*/
PyDoc_STRVAR(__pyx_doc_4cupy_4cuda_6memory_13MemoryPointer_12__isub__, "MemoryPointer.__isub__(self, ptrdiff_t offset)\n\nSubtracts an offset from the pointer in place.");
#if CYTHON_UPDATE_DESCRIPTOR_DOC
struct wrapperbase __pyx_wrapperbase_4cupy_4cuda_6memory_13MemoryPointer_12__isub__;
#endif
static PyObject *__pyx_pw_4cupy_4cuda_6memory_13MemoryPointer_13__isub__(PyObject *__pyx_v_self, PyObject *__pyx_arg_offset) {
  ptrdiff_t __pyx_v_offset;
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__isub__ (wrapper)", 0);
  __pyx_kwvalues = __Pyx_KwValues_VARARGS(__pyx_args, __pyx_nargs);
  assert(__pyx_arg_offset); {
    __pyx_v_offset = __Pyx_PyLong_As_ptrdiff_t(__pyx_arg_offset); if (unlikely((__pyx_v_offset == (ptrdiff_t)-1) && PyErr_Occurred())) __PYX_ERR(0, 522, __pyx_L3_error)
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  __Pyx_AddTraceback("cupy.cuda.memory.MemoryPointer.__isub__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_4cupy_4cuda_6memory_13MemoryPointer_12__isub__(((struct __pyx_obj_4cupy_4cuda_6memory_MemoryPointer *)__pyx_v_self), ((ptrdiff_t)__pyx_v_offset));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4cupy_4cuda_6memory_13MemoryPointer_12__isub__(struct __pyx_obj_4cupy_4cuda_6memory_MemoryPointer *__pyx_v_self, ptrdiff_t __pyx_v_offset) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  size_t __pyx_t_4;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__isub__", 0);

  /* "cupy/cuda/memory.pyx":524
 *     def __isub__(self, ptrdiff_t offset):
 *         """Subtracts an offset from the pointer in place."""
 *         return self.__iadd__(-offset)             # <<<<<<<<<<<<<<
 * 
 *     cpdef copy_from_device(self, MemoryPointer src, size_t size):
*/
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_2 = ((PyObject *)__pyx_v_self);
  __Pyx_INCREF(__pyx_t_2);
  __pyx_t_3 = __Pyx_PyLong_From_ptrdiff_t((-__pyx_v_offset)); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 524, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_4 = 0;
  {
    PyObject *__pyx_callargs[2] = {__pyx_t_2, __pyx_t_3};
    __pyx_t_1 = __Pyx_PyObject_FastCallMethod((PyObject*)__pyx_mstate_global->__pyx_n_u_iadd, __pyx_callargs+__pyx_t_4, (2-__pyx_t_4) | (1*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
    __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 524, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
  }
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "cupy/cuda/memory.pyx":522
 *         return self + -offset
 * 
 *     def __isub__(self, ptrdiff_t offset):             # <<<<<<<<<<<<<<
 *         """Subtracts an offset from the pointer in place."""
 *         return self.__iadd__(-offset)
*/

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_AddTraceback("cupy.cuda.memory.MemoryPointer.__isub__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "cupy/cuda/memory.pyx":526
 *         return self.__iadd__(-offset)
 * 
 *     cpdef copy_from_device(self, MemoryPointer src, size_t size):             # <<<<<<<<<<<<<<
 *         """Copies a memory sequence from a (possibly different) device.
 * 
*/

static PyObject *__pyx_pw_4cupy_4cuda_6memory_13MemoryPointer_15copy_from_device(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
static PyObject *__pyx_f_4cupy_4cuda_6memory_13MemoryPointer_copy_from_device(struct __pyx_obj_4cupy_4cuda_6memory_MemoryPointer *__pyx_v_self, struct __pyx_obj_4cupy_4cuda_6memory_MemoryPointer *__pyx_v_src, size_t __pyx_v_size, int __pyx_skip_dispatch) {
  intptr_t __pyx_v_stream_ptr;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  size_t __pyx_t_6;
  intptr_t __pyx_t_7;
  int __pyx_t_8;
  int __pyx_t_9;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("copy_from_device", 0);
  /* Check if called by wrapper */
  if (unlikely(__pyx_skip_dispatch)) ;
  /* Check if overridden in Python */
  else if (
  #if !CYTHON_USE_TYPE_SLOTS
  unlikely(Py_TYPE(((PyObject *)__pyx_v_self)) != __pyx_mstate_global->__pyx_ptype_4cupy_4cuda_6memory_MemoryPointer &&
  __Pyx_PyType_HasFeature(Py_TYPE(((PyObject *)__pyx_v_self)), Py_TPFLAGS_HAVE_GC))
  #else
  unlikely(Py_TYPE(((PyObject *)__pyx_v_self))->tp_dictoffset != 0 || __Pyx_PyType_HasFeature(Py_TYPE(((PyObject *)__pyx_v_self)), (Py_TPFLAGS_IS_ABSTRACT | Py_TPFLAGS_HEAPTYPE)))
  #endif
  ) {
    #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
    static PY_UINT64_T __pyx_tp_dict_version = __PYX_DICT_VERSION_INIT, __pyx_obj_dict_version = __PYX_DICT_VERSION_INIT;
    if (unlikely(!__Pyx_object_dict_version_matches(((PyObject *)__pyx_v_self), __pyx_tp_dict_version, __pyx_obj_dict_version))) {
      PY_UINT64_T __pyx_typedict_guard = __Pyx_get_tp_dict_version(((PyObject *)__pyx_v_self));
      #endif
      __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_mstate_global->__pyx_n_u_copy_from_device); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 526, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      if (!__Pyx_IsSameCFunction(__pyx_t_1, (void(*)(void)) __pyx_pw_4cupy_4cuda_6memory_13MemoryPointer_15copy_from_device)) {
        __Pyx_XDECREF(__pyx_r);
        __pyx_t_3 = NULL;
        __Pyx_INCREF(__pyx_t_1);
        __pyx_t_4 = __pyx_t_1; 
        __pyx_t_5 = __Pyx_PyLong_FromSize_t(__pyx_v_size); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 526, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_5);
        __pyx_t_6 = 1;
        #if CYTHON_UNPACK_METHODS
        if (unlikely(PyMethod_Check(__pyx_t_4))) {
          __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_4);
          assert(__pyx_t_3);
          PyObject* __pyx__function = PyMethod_GET_FUNCTION(__pyx_t_4);
          __Pyx_INCREF(__pyx_t_3);
          __Pyx_INCREF(__pyx__function);
          __Pyx_DECREF_SET(__pyx_t_4, __pyx__function);
          __pyx_t_6 = 0;
        }
        #endif
        {
          PyObject *__pyx_callargs[3] = {__pyx_t_3, ((PyObject *)__pyx_v_src), __pyx_t_5};
          __pyx_t_2 = __Pyx_PyObject_FastCall((PyObject*)__pyx_t_4, __pyx_callargs+__pyx_t_6, (3-__pyx_t_6) | (__pyx_t_6*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
          __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
          __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
          __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
          if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 526, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_2);
        }
        __pyx_r = __pyx_t_2;
        __pyx_t_2 = 0;
        __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
        goto __pyx_L0;
      }
      #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
      __pyx_tp_dict_version = __Pyx_get_tp_dict_version(((PyObject *)__pyx_v_self));
      __pyx_obj_dict_version = __Pyx_get_object_dict_version(((PyObject *)__pyx_v_self));
      if (unlikely(__pyx_typedict_guard != __pyx_tp_dict_version)) {
        __pyx_tp_dict_version = __pyx_obj_dict_version = __PYX_DICT_VERSION_INIT;
      }
      #endif
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
    }
    #endif
  }

  /* "cupy/cuda/memory.pyx":540
 * 
 *         """
 *         stream_ptr = stream_module.get_current_stream_ptr()             # <<<<<<<<<<<<<<
 *         if (
 *             not runtime._is_hip_environment
*/
  __pyx_t_7 = __pyx_f_4cupy_4cuda_6stream_get_current_stream_ptr(); if (unlikely(__pyx_t_7 == ((intptr_t)-1L) && PyErr_Occurred())) __PYX_ERR(0, 540, __pyx_L1_error)
  __pyx_v_stream_ptr = __pyx_t_7;

  /* "cupy/cuda/memory.pyx":542
 *         stream_ptr = stream_module.get_current_stream_ptr()
 *         if (
 *             not runtime._is_hip_environment             # <<<<<<<<<<<<<<
 *             and runtime.streamIsCapturing(stream_ptr)
 *         ):
*/
  __pyx_t_9 = (!__pyx_v_13cupy_backends_4cuda_3api_7runtime__is_hip_environment);
  if (__pyx_t_9) {
  } else {
    __pyx_t_8 = __pyx_t_9;
    goto __pyx_L4_bool_binop_done;
  }

  /* "cupy/cuda/memory.pyx":543
 *         if (
 *             not runtime._is_hip_environment
 *             and runtime.streamIsCapturing(stream_ptr)             # <<<<<<<<<<<<<<
 *         ):
 *             raise RuntimeError(
*/
  __pyx_t_9 = __pyx_f_13cupy_backends_4cuda_3api_7runtime_streamIsCapturing(__pyx_v_stream_ptr, 0); if (unlikely(PyErr_Occurred())) __PYX_ERR(0, 543, __pyx_L1_error)
  __pyx_t_8 = __pyx_t_9;
  __pyx_L4_bool_binop_done:;

  /* "cupy/cuda/memory.pyx":541
 *         """
 *         stream_ptr = stream_module.get_current_stream_ptr()
 *         if (             # <<<<<<<<<<<<<<
 *             not runtime._is_hip_environment
 *             and runtime.streamIsCapturing(stream_ptr)
*/
  if (unlikely(__pyx_t_8)) {

    /* "cupy/cuda/memory.pyx":545
 *             and runtime.streamIsCapturing(stream_ptr)
 *         ):
 *             raise RuntimeError(             # <<<<<<<<<<<<<<
 *                 'the current stream is capturing, so synchronous API calls '
 *                 'are disallowed')
*/
    __pyx_t_2 = NULL;
    __pyx_t_6 = 1;
    {
      PyObject *__pyx_callargs[2] = {__pyx_t_2, __pyx_mstate_global->__pyx_kp_u_the_current_stream_is_capturing};
      __pyx_t_1 = __Pyx_PyObject_FastCall((PyObject*)(((PyTypeObject*)PyExc_RuntimeError)), __pyx_callargs+__pyx_t_6, (2-__pyx_t_6) | (__pyx_t_6*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
      __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
      if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 545, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
    }
    __Pyx_Raise(__pyx_t_1, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __PYX_ERR(0, 545, __pyx_L1_error)

    /* "cupy/cuda/memory.pyx":541
 *         """
 *         stream_ptr = stream_module.get_current_stream_ptr()
 *         if (             # <<<<<<<<<<<<<<
 *             not runtime._is_hip_environment
 *             and runtime.streamIsCapturing(stream_ptr)
*/
  }

  /* "cupy/cuda/memory.pyx":548
 *                 'the current stream is capturing, so synchronous API calls '
 *                 'are disallowed')
 *         if size > 0:             # <<<<<<<<<<<<<<
 *             device._enable_peer_access(src.device_id, self.device_id)
 *             runtime.memcpy(self.ptr, src.ptr, size,
*/
  __pyx_t_8 = (__pyx_v_size > 0);
  if (__pyx_t_8) {

    /* "cupy/cuda/memory.pyx":549
 *                 'are disallowed')
 *         if size > 0:
 *             device._enable_peer_access(src.device_id, self.device_id)             # <<<<<<<<<<<<<<
 *             runtime.memcpy(self.ptr, src.ptr, size,
 *                            runtime.memcpyDefault)
*/
    __pyx_t_8 = __pyx_f_4cupy_4cuda_6device__enable_peer_access(__pyx_v_src->device_id, __pyx_v_self->device_id); if (unlikely(__pyx_t_8 == ((int)-1))) __PYX_ERR(0, 549, __pyx_L1_error)

    /* "cupy/cuda/memory.pyx":550
 *         if size > 0:
 *             device._enable_peer_access(src.device_id, self.device_id)
 *             runtime.memcpy(self.ptr, src.ptr, size,             # <<<<<<<<<<<<<<
 *                            runtime.memcpyDefault)
 * 
*/
    __pyx_t_1 = __pyx_f_13cupy_backends_4cuda_3api_7runtime_memcpy(__pyx_v_self->ptr, __pyx_v_src->ptr, __pyx_v_size, __pyx_e_13cupy_backends_4cuda_3api_7runtime_memcpyDefault, 0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 550, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

    /* "cupy/cuda/memory.pyx":548
 *                 'the current stream is capturing, so synchronous API calls '
 *                 'are disallowed')
 *         if size > 0:             # <<<<<<<<<<<<<<
 *             device._enable_peer_access(src.device_id, self.device_id)
 *             runtime.memcpy(self.ptr, src.ptr, size,
*/
  }

  /* "cupy/cuda/memory.pyx":526
 *         return self.__iadd__(-offset)
 * 
 *     cpdef copy_from_device(self, MemoryPointer src, size_t size):             # <<<<<<<<<<<<<<
 *         """Copies a memory sequence from a (possibly different) device.
 * 
*/

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_AddTraceback("cupy.cuda.memory.MemoryPointer.copy_from_device", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_pw_4cupy_4cuda_6memory_13MemoryPointer_15copy_from_device(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
PyDoc_STRVAR(__pyx_doc_4cupy_4cuda_6memory_13MemoryPointer_14copy_from_device, "MemoryPointer.copy_from_device(self, MemoryPointer src, size_t size)\n\nCopies a memory sequence from a (possibly different) device.\n\nArgs:\n    src (cupy.cuda.MemoryPointer): Source memory pointer.\n    size (int): Size of the sequence in bytes.\n\n.. warning::\n\n    This function always uses the legacy default stream and does not\n    honor the current stream. Use `copy_from_device_async` instead\n    if you are using streams in your code, or have PTDS enabled.");
static PyObject *__pyx_pw_4cupy_4cuda_6memory_13MemoryPointer_15copy_from_device(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
) {
  struct __pyx_obj_4cupy_4cuda_6memory_MemoryPointer *__pyx_v_src = 0;
  size_t __pyx_v_size;
  #if !CYTHON_METH_FASTCALL
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  #endif
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject* values[2] = {0,0};
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("copy_from_device (wrapper)", 0);
  #if !CYTHON_METH_FASTCALL
  #if CYTHON_ASSUME_SAFE_SIZE
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return NULL;
  #endif
  #endif
  __pyx_kwvalues = __Pyx_KwValues_FASTCALL(__pyx_args, __pyx_nargs);
  {
    PyObject ** const __pyx_pyargnames[] = {&__pyx_mstate_global->__pyx_n_u_src,&__pyx_mstate_global->__pyx_n_u_size,0};
    const Py_ssize_t __pyx_kwds_len = (__pyx_kwds) ? __Pyx_NumKwargs_FASTCALL(__pyx_kwds) : 0;
    if (unlikely(__pyx_kwds_len < 0)) __PYX_ERR(0, 526, __pyx_L3_error)
    if (__pyx_kwds_len > 0) {
      switch (__pyx_nargs) {
        case  2:
        values[1] = __Pyx_ArgRef_FASTCALL(__pyx_args, 1);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[1])) __PYX_ERR(0, 526, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  1:
        values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(0, 526, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      const Py_ssize_t kwd_pos_args = __pyx_nargs;
      if (__Pyx_ParseKeywords(__pyx_kwds, __pyx_kwvalues, __pyx_pyargnames, 0, values, kwd_pos_args, __pyx_kwds_len, "copy_from_device", 0) < (0)) __PYX_ERR(0, 526, __pyx_L3_error)
      for (Py_ssize_t i = __pyx_nargs; i < 2; i++) {
        if (unlikely(!values[i])) { __Pyx_RaiseArgtupleInvalid("copy_from_device", 1, 2, 2, i); __PYX_ERR(0, 526, __pyx_L3_error) }
      }
    } else if (unlikely(__pyx_nargs != 2)) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
      if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(0, 526, __pyx_L3_error)
      values[1] = __Pyx_ArgRef_FASTCALL(__pyx_args, 1);
      if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[1])) __PYX_ERR(0, 526, __pyx_L3_error)
    }
    __pyx_v_src = ((struct __pyx_obj_4cupy_4cuda_6memory_MemoryPointer *)values[0]);
    __pyx_v_size = __Pyx_PyLong_As_size_t(values[1]); if (unlikely((__pyx_v_size == (size_t)-1) && PyErr_Occurred())) __PYX_ERR(0, 526, __pyx_L3_error)
  }
  goto __pyx_L6_skip;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("copy_from_device", 1, 2, 2, __pyx_nargs); __PYX_ERR(0, 526, __pyx_L3_error)
  __pyx_L6_skip:;
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_AddTraceback("cupy.cuda.memory.MemoryPointer.copy_from_device", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_src), __pyx_mstate_global->__pyx_ptype_4cupy_4cuda_6memory_MemoryPointer, 1, "src", 0))) __PYX_ERR(0, 526, __pyx_L1_error)
  __pyx_r = __pyx_pf_4cupy_4cuda_6memory_13MemoryPointer_14copy_from_device(((struct __pyx_obj_4cupy_4cuda_6memory_MemoryPointer *)__pyx_v_self), __pyx_v_src, __pyx_v_size);

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = NULL;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  goto __pyx_L7_cleaned_up;
  __pyx_L0:;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __pyx_L7_cleaned_up:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4cupy_4cuda_6memory_13MemoryPointer_14copy_from_device(struct __pyx_obj_4cupy_4cuda_6memory_MemoryPointer *__pyx_v_self, struct __pyx_obj_4cupy_4cuda_6memory_MemoryPointer *__pyx_v_src, size_t __pyx_v_size) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("copy_from_device", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __pyx_f_4cupy_4cuda_6memory_13MemoryPointer_copy_from_device(__pyx_v_self, __pyx_v_src, __pyx_v_size, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 526, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("cupy.cuda.memory.MemoryPointer.copy_from_device", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "cupy/cuda/memory.pyx":553
 *                            runtime.memcpyDefault)
 * 
 *     cpdef copy_from_device_async(self, MemoryPointer src, size_t size,             # <<<<<<<<<<<<<<
 *                                  stream=None):
 *         """Copies a memory from a (possibly different) device asynchronously.
*/

static PyObject *__pyx_pw_4cupy_4cuda_6memory_13MemoryPointer_17copy_from_device_async(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
static PyObject *__pyx_f_4cupy_4cuda_6memory_13MemoryPointer_copy_from_device_async(struct __pyx_obj_4cupy_4cuda_6memory_MemoryPointer *__pyx_v_self, struct __pyx_obj_4cupy_4cuda_6memory_MemoryPointer *__pyx_v_src, size_t __pyx_v_size, int __pyx_skip_dispatch, struct __pyx_opt_args_4cupy_4cuda_6memory_13MemoryPointer_copy_from_device_async *__pyx_optional_args) {

  /* "cupy/cuda/memory.pyx":554
 * 
 *     cpdef copy_from_device_async(self, MemoryPointer src, size_t size,
 *                                  stream=None):             # <<<<<<<<<<<<<<
 *         """Copies a memory from a (possibly different) device asynchronously.
 * 
*/
  PyObject *__pyx_v_stream = ((PyObject *)Py_None);
  PyObject *__pyx_v_stream_ptr = NULL;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  size_t __pyx_t_6;
  int __pyx_t_7;
  intptr_t __pyx_t_8;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("copy_from_device_async", 0);
  if (__pyx_optional_args) {
    if (__pyx_optional_args->__pyx_n > 0) {
      __pyx_v_stream = __pyx_optional_args->stream;
    }
  }

  /* "cupy/cuda/memory.pyx":553
 *                            runtime.memcpyDefault)
 * 
 *     cpdef copy_from_device_async(self, MemoryPointer src, size_t size,             # <<<<<<<<<<<<<<
 *                                  stream=None):
 *         """Copies a memory from a (possibly different) device asynchronously.
*/
  /* Check if called by wrapper */
  if (unlikely(__pyx_skip_dispatch)) ;
  /* Check if overridden in Python */
  else if (
  #if !CYTHON_USE_TYPE_SLOTS
  unlikely(Py_TYPE(((PyObject *)__pyx_v_self)) != __pyx_mstate_global->__pyx_ptype_4cupy_4cuda_6memory_MemoryPointer &&
  __Pyx_PyType_HasFeature(Py_TYPE(((PyObject *)__pyx_v_self)), Py_TPFLAGS_HAVE_GC))
  #else
  unlikely(Py_TYPE(((PyObject *)__pyx_v_self))->tp_dictoffset != 0 || __Pyx_PyType_HasFeature(Py_TYPE(((PyObject *)__pyx_v_self)), (Py_TPFLAGS_IS_ABSTRACT | Py_TPFLAGS_HEAPTYPE)))
  #endif
  ) {
    #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
    static PY_UINT64_T __pyx_tp_dict_version = __PYX_DICT_VERSION_INIT, __pyx_obj_dict_version = __PYX_DICT_VERSION_INIT;
    if (unlikely(!__Pyx_object_dict_version_matches(((PyObject *)__pyx_v_self), __pyx_tp_dict_version, __pyx_obj_dict_version))) {
      PY_UINT64_T __pyx_typedict_guard = __Pyx_get_tp_dict_version(((PyObject *)__pyx_v_self));
      #endif
      __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_mstate_global->__pyx_n_u_copy_from_device_async); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 553, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      if (!__Pyx_IsSameCFunction(__pyx_t_1, (void(*)(void)) __pyx_pw_4cupy_4cuda_6memory_13MemoryPointer_17copy_from_device_async)) {
        __Pyx_XDECREF(__pyx_r);
        __pyx_t_3 = NULL;
        __Pyx_INCREF(__pyx_t_1);
        __pyx_t_4 = __pyx_t_1; 
        __pyx_t_5 = __Pyx_PyLong_FromSize_t(__pyx_v_size); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 553, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_5);
        __pyx_t_6 = 1;
        #if CYTHON_UNPACK_METHODS
        if (unlikely(PyMethod_Check(__pyx_t_4))) {
          __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_4);
          assert(__pyx_t_3);
          PyObject* __pyx__function = PyMethod_GET_FUNCTION(__pyx_t_4);
          __Pyx_INCREF(__pyx_t_3);
          __Pyx_INCREF(__pyx__function);
          __Pyx_DECREF_SET(__pyx_t_4, __pyx__function);
          __pyx_t_6 = 0;
        }
        #endif
        {
          PyObject *__pyx_callargs[4] = {__pyx_t_3, ((PyObject *)__pyx_v_src), __pyx_t_5, __pyx_v_stream};
          __pyx_t_2 = __Pyx_PyObject_FastCall((PyObject*)__pyx_t_4, __pyx_callargs+__pyx_t_6, (4-__pyx_t_6) | (__pyx_t_6*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
          __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
          __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
          __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
          if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 553, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_2);
        }
        __pyx_r = __pyx_t_2;
        __pyx_t_2 = 0;
        __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
        goto __pyx_L0;
      }
      #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
      __pyx_tp_dict_version = __Pyx_get_tp_dict_version(((PyObject *)__pyx_v_self));
      __pyx_obj_dict_version = __Pyx_get_object_dict_version(((PyObject *)__pyx_v_self));
      if (unlikely(__pyx_typedict_guard != __pyx_tp_dict_version)) {
        __pyx_tp_dict_version = __pyx_obj_dict_version = __PYX_DICT_VERSION_INIT;
      }
      #endif
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
    }
    #endif
  }

  /* "cupy/cuda/memory.pyx":564
 * 
 *         """
 *         if stream is None:             # <<<<<<<<<<<<<<
 *             stream_ptr = stream_module.get_current_stream_ptr()
 *         else:
*/
  __pyx_t_7 = (__pyx_v_stream == Py_None);
  if (__pyx_t_7) {

    /* "cupy/cuda/memory.pyx":565
 *         """
 *         if stream is None:
 *             stream_ptr = stream_module.get_current_stream_ptr()             # <<<<<<<<<<<<<<
 *         else:
 *             stream_ptr = stream.ptr
*/
    __pyx_t_8 = __pyx_f_4cupy_4cuda_6stream_get_current_stream_ptr(); if (unlikely(__pyx_t_8 == ((intptr_t)-1L) && PyErr_Occurred())) __PYX_ERR(0, 565, __pyx_L1_error)
    __pyx_t_1 = PyLong_FromSsize_t(__pyx_t_8); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 565, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_v_stream_ptr = __pyx_t_1;
    __pyx_t_1 = 0;

    /* "cupy/cuda/memory.pyx":564
 * 
 *         """
 *         if stream is None:             # <<<<<<<<<<<<<<
 *             stream_ptr = stream_module.get_current_stream_ptr()
 *         else:
*/
    goto __pyx_L3;
  }

  /* "cupy/cuda/memory.pyx":567
 *             stream_ptr = stream_module.get_current_stream_ptr()
 *         else:
 *             stream_ptr = stream.ptr             # <<<<<<<<<<<<<<
 *         if size > 0:
 *             device._enable_peer_access(src.device_id, self.device_id)
*/
  /*else*/ {
    __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_v_stream, __pyx_mstate_global->__pyx_n_u_ptr); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 567, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_v_stream_ptr = __pyx_t_1;
    __pyx_t_1 = 0;
  }
  __pyx_L3:;

  /* "cupy/cuda/memory.pyx":568
 *         else:
 *             stream_ptr = stream.ptr
 *         if size > 0:             # <<<<<<<<<<<<<<
 *             device._enable_peer_access(src.device_id, self.device_id)
 *             runtime.memcpyAsync(self.ptr, src.ptr, size,
*/
  __pyx_t_7 = (__pyx_v_size > 0);
  if (__pyx_t_7) {

    /* "cupy/cuda/memory.pyx":569
 *             stream_ptr = stream.ptr
 *         if size > 0:
 *             device._enable_peer_access(src.device_id, self.device_id)             # <<<<<<<<<<<<<<
 *             runtime.memcpyAsync(self.ptr, src.ptr, size,
 *                                 runtime.memcpyDefault, stream_ptr)
*/
    __pyx_t_7 = __pyx_f_4cupy_4cuda_6device__enable_peer_access(__pyx_v_src->device_id, __pyx_v_self->device_id); if (unlikely(__pyx_t_7 == ((int)-1))) __PYX_ERR(0, 569, __pyx_L1_error)

    /* "cupy/cuda/memory.pyx":571
 *             device._enable_peer_access(src.device_id, self.device_id)
 *             runtime.memcpyAsync(self.ptr, src.ptr, size,
 *                                 runtime.memcpyDefault, stream_ptr)             # <<<<<<<<<<<<<<
 * 
 *     cpdef copy_from_host(self, mem, size_t size):
*/
    __pyx_t_8 = PyLong_AsSsize_t(__pyx_v_stream_ptr); if (unlikely((__pyx_t_8 == ((intptr_t)-1)) && PyErr_Occurred())) __PYX_ERR(0, 571, __pyx_L1_error)

    /* "cupy/cuda/memory.pyx":570
 *         if size > 0:
 *             device._enable_peer_access(src.device_id, self.device_id)
 *             runtime.memcpyAsync(self.ptr, src.ptr, size,             # <<<<<<<<<<<<<<
 *                                 runtime.memcpyDefault, stream_ptr)
 * 
*/
    __pyx_t_1 = __pyx_f_13cupy_backends_4cuda_3api_7runtime_memcpyAsync(__pyx_v_self->ptr, __pyx_v_src->ptr, __pyx_v_size, __pyx_e_13cupy_backends_4cuda_3api_7runtime_memcpyDefault, __pyx_t_8, 0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 570, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

    /* "cupy/cuda/memory.pyx":568
 *         else:
 *             stream_ptr = stream.ptr
 *         if size > 0:             # <<<<<<<<<<<<<<
 *             device._enable_peer_access(src.device_id, self.device_id)
 *             runtime.memcpyAsync(self.ptr, src.ptr, size,
*/
  }

  /* "cupy/cuda/memory.pyx":553
 *                            runtime.memcpyDefault)
 * 
 *     cpdef copy_from_device_async(self, MemoryPointer src, size_t size,             # <<<<<<<<<<<<<<
 *                                  stream=None):
 *         """Copies a memory from a (possibly different) device asynchronously.
*/

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_AddTraceback("cupy.cuda.memory.MemoryPointer.copy_from_device_async", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_stream_ptr);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_pw_4cupy_4cuda_6memory_13MemoryPointer_17copy_from_device_async(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
PyDoc_STRVAR(__pyx_doc_4cupy_4cuda_6memory_13MemoryPointer_16copy_from_device_async, "MemoryPointer.copy_from_device_async(self, MemoryPointer src, size_t size, stream=None)\n\nCopies a memory from a (possibly different) device asynchronously.\n\nArgs:\n    src (cupy.cuda.MemoryPointer): Source memory pointer.\n    size (int): Size of the sequence in bytes.\n    stream (cupy.cuda.Stream): CUDA stream.\n        The default uses CUDA stream of the current context.");
static PyObject *__pyx_pw_4cupy_4cuda_6memory_13MemoryPointer_17copy_from_device_async(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
) {
  struct __pyx_obj_4cupy_4cuda_6memory_MemoryPointer *__pyx_v_src = 0;
  size_t __pyx_v_size;
  PyObject *__pyx_v_stream = 0;
  #if !CYTHON_METH_FASTCALL
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  #endif
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject* values[3] = {0,0,0};
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("copy_from_device_async (wrapper)", 0);
  #if !CYTHON_METH_FASTCALL
  #if CYTHON_ASSUME_SAFE_SIZE
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return NULL;
  #endif
  #endif
  __pyx_kwvalues = __Pyx_KwValues_FASTCALL(__pyx_args, __pyx_nargs);
  {
    PyObject ** const __pyx_pyargnames[] = {&__pyx_mstate_global->__pyx_n_u_src,&__pyx_mstate_global->__pyx_n_u_size,&__pyx_mstate_global->__pyx_n_u_stream,0};
    const Py_ssize_t __pyx_kwds_len = (__pyx_kwds) ? __Pyx_NumKwargs_FASTCALL(__pyx_kwds) : 0;
    if (unlikely(__pyx_kwds_len < 0)) __PYX_ERR(0, 553, __pyx_L3_error)
    if (__pyx_kwds_len > 0) {
      switch (__pyx_nargs) {
        case  3:
        values[2] = __Pyx_ArgRef_FASTCALL(__pyx_args, 2);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[2])) __PYX_ERR(0, 553, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  2:
        values[1] = __Pyx_ArgRef_FASTCALL(__pyx_args, 1);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[1])) __PYX_ERR(0, 553, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  1:
        values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(0, 553, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      const Py_ssize_t kwd_pos_args = __pyx_nargs;
      if (__Pyx_ParseKeywords(__pyx_kwds, __pyx_kwvalues, __pyx_pyargnames, 0, values, kwd_pos_args, __pyx_kwds_len, "copy_from_device_async", 0) < (0)) __PYX_ERR(0, 553, __pyx_L3_error)

      /* "cupy/cuda/memory.pyx":554
 * 
 *     cpdef copy_from_device_async(self, MemoryPointer src, size_t size,
 *                                  stream=None):             # <<<<<<<<<<<<<<
 *         """Copies a memory from a (possibly different) device asynchronously.
 * 
*/
      if (!values[2]) values[2] = __Pyx_NewRef(((PyObject *)Py_None));
      for (Py_ssize_t i = __pyx_nargs; i < 2; i++) {
        if (unlikely(!values[i])) { __Pyx_RaiseArgtupleInvalid("copy_from_device_async", 0, 2, 3, i); __PYX_ERR(0, 553, __pyx_L3_error) }
      }
    } else {
      switch (__pyx_nargs) {
        case  3:
        values[2] = __Pyx_ArgRef_FASTCALL(__pyx_args, 2);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[2])) __PYX_ERR(0, 553, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  2:
        values[1] = __Pyx_ArgRef_FASTCALL(__pyx_args, 1);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[1])) __PYX_ERR(0, 553, __pyx_L3_error)
        values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(0, 553, __pyx_L3_error)
        break;
        default: goto __pyx_L5_argtuple_error;
      }
      if (!values[2]) values[2] = __Pyx_NewRef(((PyObject *)Py_None));
    }
    __pyx_v_src = ((struct __pyx_obj_4cupy_4cuda_6memory_MemoryPointer *)values[0]);
    __pyx_v_size = __Pyx_PyLong_As_size_t(values[1]); if (unlikely((__pyx_v_size == (size_t)-1) && PyErr_Occurred())) __PYX_ERR(0, 553, __pyx_L3_error)
    __pyx_v_stream = values[2];
  }
  goto __pyx_L6_skip;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("copy_from_device_async", 0, 2, 3, __pyx_nargs); __PYX_ERR(0, 553, __pyx_L3_error)
  __pyx_L6_skip:;
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_AddTraceback("cupy.cuda.memory.MemoryPointer.copy_from_device_async", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_src), __pyx_mstate_global->__pyx_ptype_4cupy_4cuda_6memory_MemoryPointer, 1, "src", 0))) __PYX_ERR(0, 553, __pyx_L1_error)
  __pyx_r = __pyx_pf_4cupy_4cuda_6memory_13MemoryPointer_16copy_from_device_async(((struct __pyx_obj_4cupy_4cuda_6memory_MemoryPointer *)__pyx_v_self), __pyx_v_src, __pyx_v_size, __pyx_v_stream);

  /* "cupy/cuda/memory.pyx":553
 *                            runtime.memcpyDefault)
 * 
 *     cpdef copy_from_device_async(self, MemoryPointer src, size_t size,             # <<<<<<<<<<<<<<
 *                                  stream=None):
 *         """Copies a memory from a (possibly different) device asynchronously.
*/

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = NULL;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  goto __pyx_L7_cleaned_up;
  __pyx_L0:;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __pyx_L7_cleaned_up:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4cupy_4cuda_6memory_13MemoryPointer_16copy_from_device_async(struct __pyx_obj_4cupy_4cuda_6memory_MemoryPointer *__pyx_v_self, struct __pyx_obj_4cupy_4cuda_6memory_MemoryPointer *__pyx_v_src, size_t __pyx_v_size, PyObject *__pyx_v_stream) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  struct __pyx_opt_args_4cupy_4cuda_6memory_13MemoryPointer_copy_from_device_async __pyx_t_2;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("copy_from_device_async", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_2.__pyx_n = 1;
  __pyx_t_2.stream = __pyx_v_stream;
  __pyx_t_1 = __pyx_vtabptr_4cupy_4cuda_6memory_MemoryPointer->copy_from_device_async(__pyx_v_self, __pyx_v_src, __pyx_v_size, 1, &__pyx_t_2); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 553, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("cupy.cuda.memory.MemoryPointer.copy_from_device_async", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "cupy/cuda/memory.pyx":573
 *                                 runtime.memcpyDefault, stream_ptr)
 * 
 *     cpdef copy_from_host(self, mem, size_t size):             # <<<<<<<<<<<<<<
 *         """Copies a memory sequence from the host memory.
 * 
*/

static PyObject *__pyx_pw_4cupy_4cuda_6memory_13MemoryPointer_19copy_from_host(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
static PyObject *__pyx_f_4cupy_4cuda_6memory_13MemoryPointer_copy_from_host(struct __pyx_obj_4cupy_4cuda_6memory_MemoryPointer *__pyx_v_self, PyObject *__pyx_v_mem, size_t __pyx_v_size, int __pyx_skip_dispatch) {
  intptr_t __pyx_v_stream_ptr;
  PyObject *__pyx_v_ptr = NULL;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  size_t __pyx_t_6;
  intptr_t __pyx_t_7;
  int __pyx_t_8;
  int __pyx_t_9;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("copy_from_host", 0);
  /* Check if called by wrapper */
  if (unlikely(__pyx_skip_dispatch)) ;
  /* Check if overridden in Python */
  else if (
  #if !CYTHON_USE_TYPE_SLOTS
  unlikely(Py_TYPE(((PyObject *)__pyx_v_self)) != __pyx_mstate_global->__pyx_ptype_4cupy_4cuda_6memory_MemoryPointer &&
  __Pyx_PyType_HasFeature(Py_TYPE(((PyObject *)__pyx_v_self)), Py_TPFLAGS_HAVE_GC))
  #else
  unlikely(Py_TYPE(((PyObject *)__pyx_v_self))->tp_dictoffset != 0 || __Pyx_PyType_HasFeature(Py_TYPE(((PyObject *)__pyx_v_self)), (Py_TPFLAGS_IS_ABSTRACT | Py_TPFLAGS_HEAPTYPE)))
  #endif
  ) {
    #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
    static PY_UINT64_T __pyx_tp_dict_version = __PYX_DICT_VERSION_INIT, __pyx_obj_dict_version = __PYX_DICT_VERSION_INIT;
    if (unlikely(!__Pyx_object_dict_version_matches(((PyObject *)__pyx_v_self), __pyx_tp_dict_version, __pyx_obj_dict_version))) {
      PY_UINT64_T __pyx_typedict_guard = __Pyx_get_tp_dict_version(((PyObject *)__pyx_v_self));
      #endif
      __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_mstate_global->__pyx_n_u_copy_from_host); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 573, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      if (!__Pyx_IsSameCFunction(__pyx_t_1, (void(*)(void)) __pyx_pw_4cupy_4cuda_6memory_13MemoryPointer_19copy_from_host)) {
        __Pyx_XDECREF(__pyx_r);
        __pyx_t_3 = NULL;
        __Pyx_INCREF(__pyx_t_1);
        __pyx_t_4 = __pyx_t_1; 
        __pyx_t_5 = __Pyx_PyLong_FromSize_t(__pyx_v_size); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 573, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_5);
        __pyx_t_6 = 1;
        #if CYTHON_UNPACK_METHODS
        if (unlikely(PyMethod_Check(__pyx_t_4))) {
          __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_4);
          assert(__pyx_t_3);
          PyObject* __pyx__function = PyMethod_GET_FUNCTION(__pyx_t_4);
          __Pyx_INCREF(__pyx_t_3);
          __Pyx_INCREF(__pyx__function);
          __Pyx_DECREF_SET(__pyx_t_4, __pyx__function);
          __pyx_t_6 = 0;
        }
        #endif
        {
          PyObject *__pyx_callargs[3] = {__pyx_t_3, __pyx_v_mem, __pyx_t_5};
          __pyx_t_2 = __Pyx_PyObject_FastCall((PyObject*)__pyx_t_4, __pyx_callargs+__pyx_t_6, (3-__pyx_t_6) | (__pyx_t_6*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
          __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
          __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
          __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
          if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 573, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_2);
        }
        __pyx_r = __pyx_t_2;
        __pyx_t_2 = 0;
        __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
        goto __pyx_L0;
      }
      #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
      __pyx_tp_dict_version = __Pyx_get_tp_dict_version(((PyObject *)__pyx_v_self));
      __pyx_obj_dict_version = __Pyx_get_object_dict_version(((PyObject *)__pyx_v_self));
      if (unlikely(__pyx_typedict_guard != __pyx_tp_dict_version)) {
        __pyx_tp_dict_version = __pyx_obj_dict_version = __PYX_DICT_VERSION_INIT;
      }
      #endif
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
    }
    #endif
  }

  /* "cupy/cuda/memory.pyx":587
 * 
 *         """
 *         stream_ptr = stream_module.get_current_stream_ptr()             # <<<<<<<<<<<<<<
 *         if (
 *             not runtime._is_hip_environment
*/
  __pyx_t_7 = __pyx_f_4cupy_4cuda_6stream_get_current_stream_ptr(); if (unlikely(__pyx_t_7 == ((intptr_t)-1L) && PyErr_Occurred())) __PYX_ERR(0, 587, __pyx_L1_error)
  __pyx_v_stream_ptr = __pyx_t_7;

  /* "cupy/cuda/memory.pyx":589
 *         stream_ptr = stream_module.get_current_stream_ptr()
 *         if (
 *             not runtime._is_hip_environment             # <<<<<<<<<<<<<<
 *             and runtime.streamIsCapturing(stream_ptr)
 *         ):
*/
  __pyx_t_9 = (!__pyx_v_13cupy_backends_4cuda_3api_7runtime__is_hip_environment);
  if (__pyx_t_9) {
  } else {
    __pyx_t_8 = __pyx_t_9;
    goto __pyx_L4_bool_binop_done;
  }

  /* "cupy/cuda/memory.pyx":590
 *         if (
 *             not runtime._is_hip_environment
 *             and runtime.streamIsCapturing(stream_ptr)             # <<<<<<<<<<<<<<
 *         ):
 *             raise RuntimeError(
*/
  __pyx_t_9 = __pyx_f_13cupy_backends_4cuda_3api_7runtime_streamIsCapturing(__pyx_v_stream_ptr, 0); if (unlikely(PyErr_Occurred())) __PYX_ERR(0, 590, __pyx_L1_error)
  __pyx_t_8 = __pyx_t_9;
  __pyx_L4_bool_binop_done:;

  /* "cupy/cuda/memory.pyx":588
 *         """
 *         stream_ptr = stream_module.get_current_stream_ptr()
 *         if (             # <<<<<<<<<<<<<<
 *             not runtime._is_hip_environment
 *             and runtime.streamIsCapturing(stream_ptr)
*/
  if (unlikely(__pyx_t_8)) {

    /* "cupy/cuda/memory.pyx":592
 *             and runtime.streamIsCapturing(stream_ptr)
 *         ):
 *             raise RuntimeError(             # <<<<<<<<<<<<<<
 *                 'the current stream is capturing, so synchronous API calls '
 *                 'are disallowed')
*/
    __pyx_t_2 = NULL;
    __pyx_t_6 = 1;
    {
      PyObject *__pyx_callargs[2] = {__pyx_t_2, __pyx_mstate_global->__pyx_kp_u_the_current_stream_is_capturing};
      __pyx_t_1 = __Pyx_PyObject_FastCall((PyObject*)(((PyTypeObject*)PyExc_RuntimeError)), __pyx_callargs+__pyx_t_6, (2-__pyx_t_6) | (__pyx_t_6*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
      __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
      if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 592, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
    }
    __Pyx_Raise(__pyx_t_1, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __PYX_ERR(0, 592, __pyx_L1_error)

    /* "cupy/cuda/memory.pyx":588
 *         """
 *         stream_ptr = stream_module.get_current_stream_ptr()
 *         if (             # <<<<<<<<<<<<<<
 *             not runtime._is_hip_environment
 *             and runtime.streamIsCapturing(stream_ptr)
*/
  }

  /* "cupy/cuda/memory.pyx":595
 *                 'the current stream is capturing, so synchronous API calls '
 *                 'are disallowed')
 *         if size > 0:             # <<<<<<<<<<<<<<
 *             ptr = mem if isinstance(mem, int) else mem.value
 *             runtime.memcpy(self.ptr, ptr, size,
*/
  __pyx_t_8 = (__pyx_v_size > 0);
  if (__pyx_t_8) {

    /* "cupy/cuda/memory.pyx":596
 *                 'are disallowed')
 *         if size > 0:
 *             ptr = mem if isinstance(mem, int) else mem.value             # <<<<<<<<<<<<<<
 *             runtime.memcpy(self.ptr, ptr, size,
 *                            runtime.memcpyHostToDevice)
*/
    __pyx_t_8 = PyLong_Check(__pyx_v_mem); 
    if (__pyx_t_8) {
      __Pyx_INCREF(__pyx_v_mem);
      __pyx_t_1 = __pyx_v_mem;
    } else {
      __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_v_mem, __pyx_mstate_global->__pyx_n_u_value); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 596, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_2);
      __pyx_t_1 = __pyx_t_2;
      __pyx_t_2 = 0;
    }
    __pyx_v_ptr = __pyx_t_1;
    __pyx_t_1 = 0;

    /* "cupy/cuda/memory.pyx":597
 *         if size > 0:
 *             ptr = mem if isinstance(mem, int) else mem.value
 *             runtime.memcpy(self.ptr, ptr, size,             # <<<<<<<<<<<<<<
 *                            runtime.memcpyHostToDevice)
 * 
*/
    __pyx_t_7 = PyLong_AsSsize_t(__pyx_v_ptr); if (unlikely((__pyx_t_7 == ((intptr_t)-1)) && PyErr_Occurred())) __PYX_ERR(0, 597, __pyx_L1_error)

    /* "cupy/cuda/memory.pyx":598
 *             ptr = mem if isinstance(mem, int) else mem.value
 *             runtime.memcpy(self.ptr, ptr, size,
 *                            runtime.memcpyHostToDevice)             # <<<<<<<<<<<<<<
 * 
 *     cpdef copy_from_host_async(self, mem, size_t size, stream=None):
*/
    __pyx_t_1 = __pyx_f_13cupy_backends_4cuda_3api_7runtime_memcpy(__pyx_v_self->ptr, __pyx_t_7, __pyx_v_size, __pyx_e_13cupy_backends_4cuda_3api_7runtime_memcpyHostToDevice, 0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 597, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

    /* "cupy/cuda/memory.pyx":595
 *                 'the current stream is capturing, so synchronous API calls '
 *                 'are disallowed')
 *         if size > 0:             # <<<<<<<<<<<<<<
 *             ptr = mem if isinstance(mem, int) else mem.value
 *             runtime.memcpy(self.ptr, ptr, size,
*/
  }

  /* "cupy/cuda/memory.pyx":573
 *                                 runtime.memcpyDefault, stream_ptr)
 * 
 *     cpdef copy_from_host(self, mem, size_t size):             # <<<<<<<<<<<<<<
 *         """Copies a memory sequence from the host memory.
 * 
*/

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_AddTraceback("cupy.cuda.memory.MemoryPointer.copy_from_host", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_ptr);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_pw_4cupy_4cuda_6memory_13MemoryPointer_19copy_from_host(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
PyDoc_STRVAR(__pyx_doc_4cupy_4cuda_6memory_13MemoryPointer_18copy_from_host, "MemoryPointer.copy_from_host(self, mem, size_t size)\n\nCopies a memory sequence from the host memory.\n\nArgs:\n    mem (int or ctypes.c_void_p): Source memory pointer.\n    size (int): Size of the sequence in bytes.\n\n.. warning::\n\n    This function always uses the legacy default stream and does not\n    honor the current stream. Use `copy_from_host_async` instead\n    if you are using streams in your code, or have PTDS enabled.");
static PyObject *__pyx_pw_4cupy_4cuda_6memory_13MemoryPointer_19copy_from_host(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
) {
  PyObject *__pyx_v_mem = 0;
  size_t __pyx_v_size;
  #if !CYTHON_METH_FASTCALL
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  #endif
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject* values[2] = {0,0};
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("copy_from_host (wrapper)", 0);
  #if !CYTHON_METH_FASTCALL
  #if CYTHON_ASSUME_SAFE_SIZE
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return NULL;
  #endif
  #endif
  __pyx_kwvalues = __Pyx_KwValues_FASTCALL(__pyx_args, __pyx_nargs);
  {
    PyObject ** const __pyx_pyargnames[] = {&__pyx_mstate_global->__pyx_n_u_mem_2,&__pyx_mstate_global->__pyx_n_u_size,0};
    const Py_ssize_t __pyx_kwds_len = (__pyx_kwds) ? __Pyx_NumKwargs_FASTCALL(__pyx_kwds) : 0;
    if (unlikely(__pyx_kwds_len < 0)) __PYX_ERR(0, 573, __pyx_L3_error)
    if (__pyx_kwds_len > 0) {
      switch (__pyx_nargs) {
        case  2:
        values[1] = __Pyx_ArgRef_FASTCALL(__pyx_args, 1);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[1])) __PYX_ERR(0, 573, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  1:
        values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(0, 573, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      const Py_ssize_t kwd_pos_args = __pyx_nargs;
      if (__Pyx_ParseKeywords(__pyx_kwds, __pyx_kwvalues, __pyx_pyargnames, 0, values, kwd_pos_args, __pyx_kwds_len, "copy_from_host", 0) < (0)) __PYX_ERR(0, 573, __pyx_L3_error)
      for (Py_ssize_t i = __pyx_nargs; i < 2; i++) {
        if (unlikely(!values[i])) { __Pyx_RaiseArgtupleInvalid("copy_from_host", 1, 2, 2, i); __PYX_ERR(0, 573, __pyx_L3_error) }
      }
    } else if (unlikely(__pyx_nargs != 2)) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
      if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(0, 573, __pyx_L3_error)
      values[1] = __Pyx_ArgRef_FASTCALL(__pyx_args, 1);
      if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[1])) __PYX_ERR(0, 573, __pyx_L3_error)
    }
    __pyx_v_mem = values[0];
    __pyx_v_size = __Pyx_PyLong_As_size_t(values[1]); if (unlikely((__pyx_v_size == (size_t)-1) && PyErr_Occurred())) __PYX_ERR(0, 573, __pyx_L3_error)
  }
  goto __pyx_L6_skip;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("copy_from_host", 1, 2, 2, __pyx_nargs); __PYX_ERR(0, 573, __pyx_L3_error)
  __pyx_L6_skip:;
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_AddTraceback("cupy.cuda.memory.MemoryPointer.copy_from_host", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_4cupy_4cuda_6memory_13MemoryPointer_18copy_from_host(((struct __pyx_obj_4cupy_4cuda_6memory_MemoryPointer *)__pyx_v_self), __pyx_v_mem, __pyx_v_size);

  /* function exit code */
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4cupy_4cuda_6memory_13MemoryPointer_18copy_from_host(struct __pyx_obj_4cupy_4cuda_6memory_MemoryPointer *__pyx_v_self, PyObject *__pyx_v_mem, size_t __pyx_v_size) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("copy_from_host", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __pyx_f_4cupy_4cuda_6memory_13MemoryPointer_copy_from_host(__pyx_v_self, __pyx_v_mem, __pyx_v_size, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 573, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("cupy.cuda.memory.MemoryPointer.copy_from_host", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "cupy/cuda/memory.pyx":600
 *                            runtime.memcpyHostToDevice)
 * 
 *     cpdef copy_from_host_async(self, mem, size_t size, stream=None):             # <<<<<<<<<<<<<<
 *         """Copies a memory sequence from the host memory asynchronously.
 * 
*/

static PyObject *__pyx_pw_4cupy_4cuda_6memory_13MemoryPointer_21copy_from_host_async(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
static PyObject *__pyx_f_4cupy_4cuda_6memory_13MemoryPointer_copy_from_host_async(struct __pyx_obj_4cupy_4cuda_6memory_MemoryPointer *__pyx_v_self, PyObject *__pyx_v_mem, size_t __pyx_v_size, int __pyx_skip_dispatch, struct __pyx_opt_args_4cupy_4cuda_6memory_13MemoryPointer_copy_from_host_async *__pyx_optional_args) {
  PyObject *__pyx_v_stream = ((PyObject *)Py_None);
  PyObject *__pyx_v_stream_ptr = NULL;
  PyObject *__pyx_v_ptr = NULL;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  size_t __pyx_t_6;
  int __pyx_t_7;
  intptr_t __pyx_t_8;
  int __pyx_t_9;
  intptr_t __pyx_t_10;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("copy_from_host_async", 0);
  if (__pyx_optional_args) {
    if (__pyx_optional_args->__pyx_n > 0) {
      __pyx_v_stream = __pyx_optional_args->stream;
    }
  }
  /* Check if called by wrapper */
  if (unlikely(__pyx_skip_dispatch)) ;
  /* Check if overridden in Python */
  else if (
  #if !CYTHON_USE_TYPE_SLOTS
  unlikely(Py_TYPE(((PyObject *)__pyx_v_self)) != __pyx_mstate_global->__pyx_ptype_4cupy_4cuda_6memory_MemoryPointer &&
  __Pyx_PyType_HasFeature(Py_TYPE(((PyObject *)__pyx_v_self)), Py_TPFLAGS_HAVE_GC))
  #else
  unlikely(Py_TYPE(((PyObject *)__pyx_v_self))->tp_dictoffset != 0 || __Pyx_PyType_HasFeature(Py_TYPE(((PyObject *)__pyx_v_self)), (Py_TPFLAGS_IS_ABSTRACT | Py_TPFLAGS_HEAPTYPE)))
  #endif
  ) {
    #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
    static PY_UINT64_T __pyx_tp_dict_version = __PYX_DICT_VERSION_INIT, __pyx_obj_dict_version = __PYX_DICT_VERSION_INIT;
    if (unlikely(!__Pyx_object_dict_version_matches(((PyObject *)__pyx_v_self), __pyx_tp_dict_version, __pyx_obj_dict_version))) {
      PY_UINT64_T __pyx_typedict_guard = __Pyx_get_tp_dict_version(((PyObject *)__pyx_v_self));
      #endif
      __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_mstate_global->__pyx_n_u_copy_from_host_async); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 600, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      if (!__Pyx_IsSameCFunction(__pyx_t_1, (void(*)(void)) __pyx_pw_4cupy_4cuda_6memory_13MemoryPointer_21copy_from_host_async)) {
        __Pyx_XDECREF(__pyx_r);
        __pyx_t_3 = NULL;
        __Pyx_INCREF(__pyx_t_1);
        __pyx_t_4 = __pyx_t_1; 
        __pyx_t_5 = __Pyx_PyLong_FromSize_t(__pyx_v_size); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 600, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_5);
        __pyx_t_6 = 1;
        #if CYTHON_UNPACK_METHODS
        if (unlikely(PyMethod_Check(__pyx_t_4))) {
          __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_4);
          assert(__pyx_t_3);
          PyObject* __pyx__function = PyMethod_GET_FUNCTION(__pyx_t_4);
          __Pyx_INCREF(__pyx_t_3);
          __Pyx_INCREF(__pyx__function);
          __Pyx_DECREF_SET(__pyx_t_4, __pyx__function);
          __pyx_t_6 = 0;
        }
        #endif
        {
          PyObject *__pyx_callargs[4] = {__pyx_t_3, __pyx_v_mem, __pyx_t_5, __pyx_v_stream};
          __pyx_t_2 = __Pyx_PyObject_FastCall((PyObject*)__pyx_t_4, __pyx_callargs+__pyx_t_6, (4-__pyx_t_6) | (__pyx_t_6*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
          __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
          __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
          __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
          if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 600, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_2);
        }
        __pyx_r = __pyx_t_2;
        __pyx_t_2 = 0;
        __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
        goto __pyx_L0;
      }
      #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
      __pyx_tp_dict_version = __Pyx_get_tp_dict_version(((PyObject *)__pyx_v_self));
      __pyx_obj_dict_version = __Pyx_get_object_dict_version(((PyObject *)__pyx_v_self));
      if (unlikely(__pyx_typedict_guard != __pyx_tp_dict_version)) {
        __pyx_tp_dict_version = __pyx_obj_dict_version = __PYX_DICT_VERSION_INIT;
      }
      #endif
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
    }
    #endif
  }

  /* "cupy/cuda/memory.pyx":611
 * 
 *         """
 *         if stream is None:             # <<<<<<<<<<<<<<
 *             stream_ptr = stream_module.get_current_stream_ptr()
 *         else:
*/
  __pyx_t_7 = (__pyx_v_stream == Py_None);
  if (__pyx_t_7) {

    /* "cupy/cuda/memory.pyx":612
 *         """
 *         if stream is None:
 *             stream_ptr = stream_module.get_current_stream_ptr()             # <<<<<<<<<<<<<<
 *         else:
 *             stream_ptr = stream.ptr
*/
    __pyx_t_8 = __pyx_f_4cupy_4cuda_6stream_get_current_stream_ptr(); if (unlikely(__pyx_t_8 == ((intptr_t)-1L) && PyErr_Occurred())) __PYX_ERR(0, 612, __pyx_L1_error)
    __pyx_t_1 = PyLong_FromSsize_t(__pyx_t_8); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 612, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_v_stream_ptr = __pyx_t_1;
    __pyx_t_1 = 0;

    /* "cupy/cuda/memory.pyx":611
 * 
 *         """
 *         if stream is None:             # <<<<<<<<<<<<<<
 *             stream_ptr = stream_module.get_current_stream_ptr()
 *         else:
*/
    goto __pyx_L3;
  }

  /* "cupy/cuda/memory.pyx":614
 *             stream_ptr = stream_module.get_current_stream_ptr()
 *         else:
 *             stream_ptr = stream.ptr             # <<<<<<<<<<<<<<
 *         if (
 *             not runtime._is_hip_environment
*/
  /*else*/ {
    __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_v_stream, __pyx_mstate_global->__pyx_n_u_ptr); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 614, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_v_stream_ptr = __pyx_t_1;
    __pyx_t_1 = 0;
  }
  __pyx_L3:;

  /* "cupy/cuda/memory.pyx":616
 *             stream_ptr = stream.ptr
 *         if (
 *             not runtime._is_hip_environment             # <<<<<<<<<<<<<<
 *             and runtime.streamIsCapturing(stream_ptr)
 *         ):
*/
  __pyx_t_9 = (!__pyx_v_13cupy_backends_4cuda_3api_7runtime__is_hip_environment);
  if (__pyx_t_9) {
  } else {
    __pyx_t_7 = __pyx_t_9;
    goto __pyx_L5_bool_binop_done;
  }

  /* "cupy/cuda/memory.pyx":617
 *         if (
 *             not runtime._is_hip_environment
 *             and runtime.streamIsCapturing(stream_ptr)             # <<<<<<<<<<<<<<
 *         ):
 *             raise RuntimeError(
*/
  __pyx_t_8 = PyLong_AsSsize_t(__pyx_v_stream_ptr); if (unlikely((__pyx_t_8 == ((intptr_t)-1)) && PyErr_Occurred())) __PYX_ERR(0, 617, __pyx_L1_error)
  __pyx_t_9 = __pyx_f_13cupy_backends_4cuda_3api_7runtime_streamIsCapturing(__pyx_t_8, 0); if (unlikely(PyErr_Occurred())) __PYX_ERR(0, 617, __pyx_L1_error)
  __pyx_t_7 = __pyx_t_9;
  __pyx_L5_bool_binop_done:;

  /* "cupy/cuda/memory.pyx":615
 *         else:
 *             stream_ptr = stream.ptr
 *         if (             # <<<<<<<<<<<<<<
 *             not runtime._is_hip_environment
 *             and runtime.streamIsCapturing(stream_ptr)
*/
  if (unlikely(__pyx_t_7)) {

    /* "cupy/cuda/memory.pyx":619
 *             and runtime.streamIsCapturing(stream_ptr)
 *         ):
 *             raise RuntimeError(             # <<<<<<<<<<<<<<
 *                 'the current stream is capturing, so H2D transfers '
 *                 'are disallowed')
*/
    __pyx_t_2 = NULL;
    __pyx_t_6 = 1;
    {
      PyObject *__pyx_callargs[2] = {__pyx_t_2, __pyx_mstate_global->__pyx_kp_u_the_current_stream_is_capturing_2};
      __pyx_t_1 = __Pyx_PyObject_FastCall((PyObject*)(((PyTypeObject*)PyExc_RuntimeError)), __pyx_callargs+__pyx_t_6, (2-__pyx_t_6) | (__pyx_t_6*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
      __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
      if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 619, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
    }
    __Pyx_Raise(__pyx_t_1, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __PYX_ERR(0, 619, __pyx_L1_error)

    /* "cupy/cuda/memory.pyx":615
 *         else:
 *             stream_ptr = stream.ptr
 *         if (             # <<<<<<<<<<<<<<
 *             not runtime._is_hip_environment
 *             and runtime.streamIsCapturing(stream_ptr)
*/
  }

  /* "cupy/cuda/memory.pyx":622
 *                 'the current stream is capturing, so H2D transfers '
 *                 'are disallowed')
 *         if size > 0:             # <<<<<<<<<<<<<<
 *             ptr = mem if isinstance(mem, int) else mem.value
 *             runtime.memcpyAsync(self.ptr, ptr, size,
*/
  __pyx_t_7 = (__pyx_v_size > 0);
  if (__pyx_t_7) {

    /* "cupy/cuda/memory.pyx":623
 *                 'are disallowed')
 *         if size > 0:
 *             ptr = mem if isinstance(mem, int) else mem.value             # <<<<<<<<<<<<<<
 *             runtime.memcpyAsync(self.ptr, ptr, size,
 *                                 runtime.memcpyHostToDevice, stream_ptr)
*/
    __pyx_t_7 = PyLong_Check(__pyx_v_mem); 
    if (__pyx_t_7) {
      __Pyx_INCREF(__pyx_v_mem);
      __pyx_t_1 = __pyx_v_mem;
    } else {
      __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_v_mem, __pyx_mstate_global->__pyx_n_u_value); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 623, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_2);
      __pyx_t_1 = __pyx_t_2;
      __pyx_t_2 = 0;
    }
    __pyx_v_ptr = __pyx_t_1;
    __pyx_t_1 = 0;

    /* "cupy/cuda/memory.pyx":624
 *         if size > 0:
 *             ptr = mem if isinstance(mem, int) else mem.value
 *             runtime.memcpyAsync(self.ptr, ptr, size,             # <<<<<<<<<<<<<<
 *                                 runtime.memcpyHostToDevice, stream_ptr)
 * 
*/
    __pyx_t_8 = PyLong_AsSsize_t(__pyx_v_ptr); if (unlikely((__pyx_t_8 == ((intptr_t)-1)) && PyErr_Occurred())) __PYX_ERR(0, 624, __pyx_L1_error)

    /* "cupy/cuda/memory.pyx":625
 *             ptr = mem if isinstance(mem, int) else mem.value
 *             runtime.memcpyAsync(self.ptr, ptr, size,
 *                                 runtime.memcpyHostToDevice, stream_ptr)             # <<<<<<<<<<<<<<
 * 
 *     cpdef copy_from(self, mem, size_t size):
*/
    __pyx_t_10 = PyLong_AsSsize_t(__pyx_v_stream_ptr); if (unlikely((__pyx_t_10 == ((intptr_t)-1)) && PyErr_Occurred())) __PYX_ERR(0, 625, __pyx_L1_error)

    /* "cupy/cuda/memory.pyx":624
 *         if size > 0:
 *             ptr = mem if isinstance(mem, int) else mem.value
 *             runtime.memcpyAsync(self.ptr, ptr, size,             # <<<<<<<<<<<<<<
 *                                 runtime.memcpyHostToDevice, stream_ptr)
 * 
*/
    __pyx_t_1 = __pyx_f_13cupy_backends_4cuda_3api_7runtime_memcpyAsync(__pyx_v_self->ptr, __pyx_t_8, __pyx_v_size, __pyx_e_13cupy_backends_4cuda_3api_7runtime_memcpyHostToDevice, __pyx_t_10, 0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 624, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

    /* "cupy/cuda/memory.pyx":622
 *                 'the current stream is capturing, so H2D transfers '
 *                 'are disallowed')
 *         if size > 0:             # <<<<<<<<<<<<<<
 *             ptr = mem if isinstance(mem, int) else mem.value
 *             runtime.memcpyAsync(self.ptr, ptr, size,
*/
  }

  /* "cupy/cuda/memory.pyx":600
 *                            runtime.memcpyHostToDevice)
 * 
 *     cpdef copy_from_host_async(self, mem, size_t size, stream=None):             # <<<<<<<<<<<<<<
 *         """Copies a memory sequence from the host memory asynchronously.
 * 
*/

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_AddTraceback("cupy.cuda.memory.MemoryPointer.copy_from_host_async", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_stream_ptr);
  __Pyx_XDECREF(__pyx_v_ptr);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_pw_4cupy_4cuda_6memory_13MemoryPointer_21copy_from_host_async(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
PyDoc_STRVAR(__pyx_doc_4cupy_4cuda_6memory_13MemoryPointer_20copy_from_host_async, "MemoryPointer.copy_from_host_async(self, mem, size_t size, stream=None)\n\nCopies a memory sequence from the host memory asynchronously.\n\nArgs:\n    mem (int or ctypes.c_void_p): Source memory pointer. It must point\n        to pinned memory.\n    size (int): Size of the sequence in bytes.\n    stream (cupy.cuda.Stream): CUDA stream.\n        The default uses CUDA stream of the current context.");
static PyObject *__pyx_pw_4cupy_4cuda_6memory_13MemoryPointer_21copy_from_host_async(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
) {
  PyObject *__pyx_v_mem = 0;
  size_t __pyx_v_size;
  PyObject *__pyx_v_stream = 0;
  #if !CYTHON_METH_FASTCALL
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  #endif
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject* values[3] = {0,0,0};
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("copy_from_host_async (wrapper)", 0);
  #if !CYTHON_METH_FASTCALL
  #if CYTHON_ASSUME_SAFE_SIZE
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return NULL;
  #endif
  #endif
  __pyx_kwvalues = __Pyx_KwValues_FASTCALL(__pyx_args, __pyx_nargs);
  {
    PyObject ** const __pyx_pyargnames[] = {&__pyx_mstate_global->__pyx_n_u_mem_2,&__pyx_mstate_global->__pyx_n_u_size,&__pyx_mstate_global->__pyx_n_u_stream,0};
    const Py_ssize_t __pyx_kwds_len = (__pyx_kwds) ? __Pyx_NumKwargs_FASTCALL(__pyx_kwds) : 0;
    if (unlikely(__pyx_kwds_len < 0)) __PYX_ERR(0, 600, __pyx_L3_error)
    if (__pyx_kwds_len > 0) {
      switch (__pyx_nargs) {
        case  3:
        values[2] = __Pyx_ArgRef_FASTCALL(__pyx_args, 2);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[2])) __PYX_ERR(0, 600, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  2:
        values[1] = __Pyx_ArgRef_FASTCALL(__pyx_args, 1);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[1])) __PYX_ERR(0, 600, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  1:
        values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(0, 600, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      const Py_ssize_t kwd_pos_args = __pyx_nargs;
      if (__Pyx_ParseKeywords(__pyx_kwds, __pyx_kwvalues, __pyx_pyargnames, 0, values, kwd_pos_args, __pyx_kwds_len, "copy_from_host_async", 0) < (0)) __PYX_ERR(0, 600, __pyx_L3_error)
      if (!values[2]) values[2] = __Pyx_NewRef(((PyObject *)Py_None));
      for (Py_ssize_t i = __pyx_nargs; i < 2; i++) {
        if (unlikely(!values[i])) { __Pyx_RaiseArgtupleInvalid("copy_from_host_async", 0, 2, 3, i); __PYX_ERR(0, 600, __pyx_L3_error) }
      }
    } else {
      switch (__pyx_nargs) {
        case  3:
        values[2] = __Pyx_ArgRef_FASTCALL(__pyx_args, 2);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[2])) __PYX_ERR(0, 600, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  2:
        values[1] = __Pyx_ArgRef_FASTCALL(__pyx_args, 1);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[1])) __PYX_ERR(0, 600, __pyx_L3_error)
        values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(0, 600, __pyx_L3_error)
        break;
        default: goto __pyx_L5_argtuple_error;
      }
      if (!values[2]) values[2] = __Pyx_NewRef(((PyObject *)Py_None));
    }
    __pyx_v_mem = values[0];
    __pyx_v_size = __Pyx_PyLong_As_size_t(values[1]); if (unlikely((__pyx_v_size == (size_t)-1) && PyErr_Occurred())) __PYX_ERR(0, 600, __pyx_L3_error)
    __pyx_v_stream = values[2];
  }
  goto __pyx_L6_skip;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("copy_from_host_async", 0, 2, 3, __pyx_nargs); __PYX_ERR(0, 600, __pyx_L3_error)
  __pyx_L6_skip:;
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_AddTraceback("cupy.cuda.memory.MemoryPointer.copy_from_host_async", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_4cupy_4cuda_6memory_13MemoryPointer_20copy_from_host_async(((struct __pyx_obj_4cupy_4cuda_6memory_MemoryPointer *)__pyx_v_self), __pyx_v_mem, __pyx_v_size, __pyx_v_stream);

  /* function exit code */
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4cupy_4cuda_6memory_13MemoryPointer_20copy_from_host_async(struct __pyx_obj_4cupy_4cuda_6memory_MemoryPointer *__pyx_v_self, PyObject *__pyx_v_mem, size_t __pyx_v_size, PyObject *__pyx_v_stream) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  struct __pyx_opt_args_4cupy_4cuda_6memory_13MemoryPointer_copy_from_host_async __pyx_t_2;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("copy_from_host_async", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_2.__pyx_n = 1;
  __pyx_t_2.stream = __pyx_v_stream;
  __pyx_t_1 = __pyx_vtabptr_4cupy_4cuda_6memory_MemoryPointer->copy_from_host_async(__pyx_v_self, __pyx_v_mem, __pyx_v_size, 1, &__pyx_t_2); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 600, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("cupy.cuda.memory.MemoryPointer.copy_from_host_async", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "cupy/cuda/memory.pyx":627
 *                                 runtime.memcpyHostToDevice, stream_ptr)
 * 
 *     cpdef copy_from(self, mem, size_t size):             # <<<<<<<<<<<<<<
 *         """Copies a memory sequence from a (possibly different) device or host.
 * 
*/

static PyObject *__pyx_pw_4cupy_4cuda_6memory_13MemoryPointer_23copy_from(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
static PyObject *__pyx_f_4cupy_4cuda_6memory_13MemoryPointer_copy_from(struct __pyx_obj_4cupy_4cuda_6memory_MemoryPointer *__pyx_v_self, PyObject *__pyx_v_mem, size_t __pyx_v_size, int __pyx_skip_dispatch) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  size_t __pyx_t_6;
  int __pyx_t_7;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("copy_from", 0);
  /* Check if called by wrapper */
  if (unlikely(__pyx_skip_dispatch)) ;
  /* Check if overridden in Python */
  else if (
  #if !CYTHON_USE_TYPE_SLOTS
  unlikely(Py_TYPE(((PyObject *)__pyx_v_self)) != __pyx_mstate_global->__pyx_ptype_4cupy_4cuda_6memory_MemoryPointer &&
  __Pyx_PyType_HasFeature(Py_TYPE(((PyObject *)__pyx_v_self)), Py_TPFLAGS_HAVE_GC))
  #else
  unlikely(Py_TYPE(((PyObject *)__pyx_v_self))->tp_dictoffset != 0 || __Pyx_PyType_HasFeature(Py_TYPE(((PyObject *)__pyx_v_self)), (Py_TPFLAGS_IS_ABSTRACT | Py_TPFLAGS_HEAPTYPE)))
  #endif
  ) {
    #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
    static PY_UINT64_T __pyx_tp_dict_version = __PYX_DICT_VERSION_INIT, __pyx_obj_dict_version = __PYX_DICT_VERSION_INIT;
    if (unlikely(!__Pyx_object_dict_version_matches(((PyObject *)__pyx_v_self), __pyx_tp_dict_version, __pyx_obj_dict_version))) {
      PY_UINT64_T __pyx_typedict_guard = __Pyx_get_tp_dict_version(((PyObject *)__pyx_v_self));
      #endif
      __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_mstate_global->__pyx_n_u_copy_from); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 627, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      if (!__Pyx_IsSameCFunction(__pyx_t_1, (void(*)(void)) __pyx_pw_4cupy_4cuda_6memory_13MemoryPointer_23copy_from)) {
        __Pyx_XDECREF(__pyx_r);
        __pyx_t_3 = NULL;
        __Pyx_INCREF(__pyx_t_1);
        __pyx_t_4 = __pyx_t_1; 
        __pyx_t_5 = __Pyx_PyLong_FromSize_t(__pyx_v_size); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 627, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_5);
        __pyx_t_6 = 1;
        #if CYTHON_UNPACK_METHODS
        if (unlikely(PyMethod_Check(__pyx_t_4))) {
          __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_4);
          assert(__pyx_t_3);
          PyObject* __pyx__function = PyMethod_GET_FUNCTION(__pyx_t_4);
          __Pyx_INCREF(__pyx_t_3);
          __Pyx_INCREF(__pyx__function);
          __Pyx_DECREF_SET(__pyx_t_4, __pyx__function);
          __pyx_t_6 = 0;
        }
        #endif
        {
          PyObject *__pyx_callargs[3] = {__pyx_t_3, __pyx_v_mem, __pyx_t_5};
          __pyx_t_2 = __Pyx_PyObject_FastCall((PyObject*)__pyx_t_4, __pyx_callargs+__pyx_t_6, (3-__pyx_t_6) | (__pyx_t_6*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
          __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
          __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
          __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
          if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 627, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_2);
        }
        __pyx_r = __pyx_t_2;
        __pyx_t_2 = 0;
        __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
        goto __pyx_L0;
      }
      #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
      __pyx_tp_dict_version = __Pyx_get_tp_dict_version(((PyObject *)__pyx_v_self));
      __pyx_obj_dict_version = __Pyx_get_object_dict_version(((PyObject *)__pyx_v_self));
      if (unlikely(__pyx_typedict_guard != __pyx_tp_dict_version)) {
        __pyx_tp_dict_version = __pyx_obj_dict_version = __PYX_DICT_VERSION_INIT;
      }
      #endif
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
    }
    #endif
  }

  /* "cupy/cuda/memory.pyx":646
 * 
 *         """
 *         if isinstance(mem, MemoryPointer):             # <<<<<<<<<<<<<<
 *             self.copy_from_device(mem, size)
 *         else:
*/
  __pyx_t_7 = __Pyx_TypeCheck(__pyx_v_mem, __pyx_mstate_global->__pyx_ptype_4cupy_4cuda_6memory_MemoryPointer); 
  if (__pyx_t_7) {

    /* "cupy/cuda/memory.pyx":647
 *         """
 *         if isinstance(mem, MemoryPointer):
 *             self.copy_from_device(mem, size)             # <<<<<<<<<<<<<<
 *         else:
 *             self.copy_from_host(mem, size)
*/
    if (!(likely(((__pyx_v_mem) == Py_None) || likely(__Pyx_TypeTest(__pyx_v_mem, __pyx_mstate_global->__pyx_ptype_4cupy_4cuda_6memory_MemoryPointer))))) __PYX_ERR(0, 647, __pyx_L1_error)
    __pyx_t_1 = ((struct __pyx_vtabstruct_4cupy_4cuda_6memory_MemoryPointer *)__pyx_v_self->__pyx_vtab)->copy_from_device(__pyx_v_self, ((struct __pyx_obj_4cupy_4cuda_6memory_MemoryPointer *)__pyx_v_mem), __pyx_v_size, 0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 647, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

    /* "cupy/cuda/memory.pyx":646
 * 
 *         """
 *         if isinstance(mem, MemoryPointer):             # <<<<<<<<<<<<<<
 *             self.copy_from_device(mem, size)
 *         else:
*/
    goto __pyx_L3;
  }

  /* "cupy/cuda/memory.pyx":649
 *             self.copy_from_device(mem, size)
 *         else:
 *             self.copy_from_host(mem, size)             # <<<<<<<<<<<<<<
 * 
 *     cpdef copy_from_async(self, mem, size_t size, stream=None):
*/
  /*else*/ {
    __pyx_t_1 = ((struct __pyx_vtabstruct_4cupy_4cuda_6memory_MemoryPointer *)__pyx_v_self->__pyx_vtab)->copy_from_host(__pyx_v_self, __pyx_v_mem, __pyx_v_size, 0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 649, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  }
  __pyx_L3:;

  /* "cupy/cuda/memory.pyx":627
 *                                 runtime.memcpyHostToDevice, stream_ptr)
 * 
 *     cpdef copy_from(self, mem, size_t size):             # <<<<<<<<<<<<<<
 *         """Copies a memory sequence from a (possibly different) device or host.
 * 
*/

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_AddTraceback("cupy.cuda.memory.MemoryPointer.copy_from", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_pw_4cupy_4cuda_6memory_13MemoryPointer_23copy_from(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
PyDoc_STRVAR(__pyx_doc_4cupy_4cuda_6memory_13MemoryPointer_22copy_from, "MemoryPointer.copy_from(self, mem, size_t size)\n\nCopies a memory sequence from a (possibly different) device or host.\n\nThis function is a useful interface that selects appropriate one from\n:meth:`~cupy.cuda.MemoryPointer.copy_from_device` and\n:meth:`~cupy.cuda.MemoryPointer.copy_from_host`.\n\nArgs:\n    mem (int or ctypes.c_void_p or cupy.cuda.MemoryPointer):\n        Source memory pointer.\n    size (int): Size of the sequence in bytes.\n\n.. warning::\n\n    This function always uses the legacy default stream and does not\n    honor the current stream. Use `copy_from_async` instead\n    if you are using streams in your code, or have PTDS enabled.");
static PyObject *__pyx_pw_4cupy_4cuda_6memory_13MemoryPointer_23copy_from(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
) {
  PyObject *__pyx_v_mem = 0;
  size_t __pyx_v_size;
  #if !CYTHON_METH_FASTCALL
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  #endif
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject* values[2] = {0,0};
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("copy_from (wrapper)", 0);
  #if !CYTHON_METH_FASTCALL
  #if CYTHON_ASSUME_SAFE_SIZE
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return NULL;
  #endif
  #endif
  __pyx_kwvalues = __Pyx_KwValues_FASTCALL(__pyx_args, __pyx_nargs);
  {
    PyObject ** const __pyx_pyargnames[] = {&__pyx_mstate_global->__pyx_n_u_mem_2,&__pyx_mstate_global->__pyx_n_u_size,0};
    const Py_ssize_t __pyx_kwds_len = (__pyx_kwds) ? __Pyx_NumKwargs_FASTCALL(__pyx_kwds) : 0;
    if (unlikely(__pyx_kwds_len < 0)) __PYX_ERR(0, 627, __pyx_L3_error)
    if (__pyx_kwds_len > 0) {
      switch (__pyx_nargs) {
        case  2:
        values[1] = __Pyx_ArgRef_FASTCALL(__pyx_args, 1);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[1])) __PYX_ERR(0, 627, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  1:
        values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(0, 627, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      const Py_ssize_t kwd_pos_args = __pyx_nargs;
      if (__Pyx_ParseKeywords(__pyx_kwds, __pyx_kwvalues, __pyx_pyargnames, 0, values, kwd_pos_args, __pyx_kwds_len, "copy_from", 0) < (0)) __PYX_ERR(0, 627, __pyx_L3_error)
      for (Py_ssize_t i = __pyx_nargs; i < 2; i++) {
        if (unlikely(!values[i])) { __Pyx_RaiseArgtupleInvalid("copy_from", 1, 2, 2, i); __PYX_ERR(0, 627, __pyx_L3_error) }
      }
    } else if (unlikely(__pyx_nargs != 2)) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
      if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(0, 627, __pyx_L3_error)
      values[1] = __Pyx_ArgRef_FASTCALL(__pyx_args, 1);
      if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[1])) __PYX_ERR(0, 627, __pyx_L3_error)
    }
    __pyx_v_mem = values[0];
    __pyx_v_size = __Pyx_PyLong_As_size_t(values[1]); if (unlikely((__pyx_v_size == (size_t)-1) && PyErr_Occurred())) __PYX_ERR(0, 627, __pyx_L3_error)
  }
  goto __pyx_L6_skip;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("copy_from", 1, 2, 2, __pyx_nargs); __PYX_ERR(0, 627, __pyx_L3_error)
  __pyx_L6_skip:;
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_AddTraceback("cupy.cuda.memory.MemoryPointer.copy_from", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_4cupy_4cuda_6memory_13MemoryPointer_22copy_from(((struct __pyx_obj_4cupy_4cuda_6memory_MemoryPointer *)__pyx_v_self), __pyx_v_mem, __pyx_v_size);

  /* function exit code */
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4cupy_4cuda_6memory_13MemoryPointer_22copy_from(struct __pyx_obj_4cupy_4cuda_6memory_MemoryPointer *__pyx_v_self, PyObject *__pyx_v_mem, size_t __pyx_v_size) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("copy_from", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __pyx_f_4cupy_4cuda_6memory_13MemoryPointer_copy_from(__pyx_v_self, __pyx_v_mem, __pyx_v_size, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 627, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("cupy.cuda.memory.MemoryPointer.copy_from", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "cupy/cuda/memory.pyx":651
 *             self.copy_from_host(mem, size)
 * 
 *     cpdef copy_from_async(self, mem, size_t size, stream=None):             # <<<<<<<<<<<<<<
 *         """Copies a memory sequence from an arbitrary place asynchronously.
 * 
*/

static PyObject *__pyx_pw_4cupy_4cuda_6memory_13MemoryPointer_25copy_from_async(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
static PyObject *__pyx_f_4cupy_4cuda_6memory_13MemoryPointer_copy_from_async(struct __pyx_obj_4cupy_4cuda_6memory_MemoryPointer *__pyx_v_self, PyObject *__pyx_v_mem, size_t __pyx_v_size, int __pyx_skip_dispatch, struct __pyx_opt_args_4cupy_4cuda_6memory_13MemoryPointer_copy_from_async *__pyx_optional_args) {
  PyObject *__pyx_v_stream = ((PyObject *)Py_None);
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  size_t __pyx_t_6;
  int __pyx_t_7;
  struct __pyx_opt_args_4cupy_4cuda_6memory_13MemoryPointer_copy_from_device_async __pyx_t_8;
  struct __pyx_opt_args_4cupy_4cuda_6memory_13MemoryPointer_copy_from_host_async __pyx_t_9;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("copy_from_async", 0);
  if (__pyx_optional_args) {
    if (__pyx_optional_args->__pyx_n > 0) {
      __pyx_v_stream = __pyx_optional_args->stream;
    }
  }
  /* Check if called by wrapper */
  if (unlikely(__pyx_skip_dispatch)) ;
  /* Check if overridden in Python */
  else if (
  #if !CYTHON_USE_TYPE_SLOTS
  unlikely(Py_TYPE(((PyObject *)__pyx_v_self)) != __pyx_mstate_global->__pyx_ptype_4cupy_4cuda_6memory_MemoryPointer &&
  __Pyx_PyType_HasFeature(Py_TYPE(((PyObject *)__pyx_v_self)), Py_TPFLAGS_HAVE_GC))
  #else
  unlikely(Py_TYPE(((PyObject *)__pyx_v_self))->tp_dictoffset != 0 || __Pyx_PyType_HasFeature(Py_TYPE(((PyObject *)__pyx_v_self)), (Py_TPFLAGS_IS_ABSTRACT | Py_TPFLAGS_HEAPTYPE)))
  #endif
  ) {
    #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
    static PY_UINT64_T __pyx_tp_dict_version = __PYX_DICT_VERSION_INIT, __pyx_obj_dict_version = __PYX_DICT_VERSION_INIT;
    if (unlikely(!__Pyx_object_dict_version_matches(((PyObject *)__pyx_v_self), __pyx_tp_dict_version, __pyx_obj_dict_version))) {
      PY_UINT64_T __pyx_typedict_guard = __Pyx_get_tp_dict_version(((PyObject *)__pyx_v_self));
      #endif
      __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_mstate_global->__pyx_n_u_copy_from_async); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 651, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      if (!__Pyx_IsSameCFunction(__pyx_t_1, (void(*)(void)) __pyx_pw_4cupy_4cuda_6memory_13MemoryPointer_25copy_from_async)) {
        __Pyx_XDECREF(__pyx_r);
        __pyx_t_3 = NULL;
        __Pyx_INCREF(__pyx_t_1);
        __pyx_t_4 = __pyx_t_1; 
        __pyx_t_5 = __Pyx_PyLong_FromSize_t(__pyx_v_size); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 651, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_5);
        __pyx_t_6 = 1;
        #if CYTHON_UNPACK_METHODS
        if (unlikely(PyMethod_Check(__pyx_t_4))) {
          __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_4);
          assert(__pyx_t_3);
          PyObject* __pyx__function = PyMethod_GET_FUNCTION(__pyx_t_4);
          __Pyx_INCREF(__pyx_t_3);
          __Pyx_INCREF(__pyx__function);
          __Pyx_DECREF_SET(__pyx_t_4, __pyx__function);
          __pyx_t_6 = 0;
        }
        #endif
        {
          PyObject *__pyx_callargs[4] = {__pyx_t_3, __pyx_v_mem, __pyx_t_5, __pyx_v_stream};
          __pyx_t_2 = __Pyx_PyObject_FastCall((PyObject*)__pyx_t_4, __pyx_callargs+__pyx_t_6, (4-__pyx_t_6) | (__pyx_t_6*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
          __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
          __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
          __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
          if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 651, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_2);
        }
        __pyx_r = __pyx_t_2;
        __pyx_t_2 = 0;
        __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
        goto __pyx_L0;
      }
      #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
      __pyx_tp_dict_version = __Pyx_get_tp_dict_version(((PyObject *)__pyx_v_self));
      __pyx_obj_dict_version = __Pyx_get_object_dict_version(((PyObject *)__pyx_v_self));
      if (unlikely(__pyx_typedict_guard != __pyx_tp_dict_version)) {
        __pyx_tp_dict_version = __pyx_obj_dict_version = __PYX_DICT_VERSION_INIT;
      }
      #endif
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
    }
    #endif
  }

  /* "cupy/cuda/memory.pyx":666
 * 
 *         """
 *         if isinstance(mem, MemoryPointer):             # <<<<<<<<<<<<<<
 *             self.copy_from_device_async(mem, size, stream)
 *         else:
*/
  __pyx_t_7 = __Pyx_TypeCheck(__pyx_v_mem, __pyx_mstate_global->__pyx_ptype_4cupy_4cuda_6memory_MemoryPointer); 
  if (__pyx_t_7) {

    /* "cupy/cuda/memory.pyx":667
 *         """
 *         if isinstance(mem, MemoryPointer):
 *             self.copy_from_device_async(mem, size, stream)             # <<<<<<<<<<<<<<
 *         else:
 *             self.copy_from_host_async(mem, size, stream)
*/
    if (!(likely(((__pyx_v_mem) == Py_None) || likely(__Pyx_TypeTest(__pyx_v_mem, __pyx_mstate_global->__pyx_ptype_4cupy_4cuda_6memory_MemoryPointer))))) __PYX_ERR(0, 667, __pyx_L1_error)
    __pyx_t_8.__pyx_n = 1;
    __pyx_t_8.stream = __pyx_v_stream;
    __pyx_t_1 = ((struct __pyx_vtabstruct_4cupy_4cuda_6memory_MemoryPointer *)__pyx_v_self->__pyx_vtab)->copy_from_device_async(__pyx_v_self, ((struct __pyx_obj_4cupy_4cuda_6memory_MemoryPointer *)__pyx_v_mem), __pyx_v_size, 0, &__pyx_t_8); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 667, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

    /* "cupy/cuda/memory.pyx":666
 * 
 *         """
 *         if isinstance(mem, MemoryPointer):             # <<<<<<<<<<<<<<
 *             self.copy_from_device_async(mem, size, stream)
 *         else:
*/
    goto __pyx_L3;
  }

  /* "cupy/cuda/memory.pyx":669
 *             self.copy_from_device_async(mem, size, stream)
 *         else:
 *             self.copy_from_host_async(mem, size, stream)             # <<<<<<<<<<<<<<
 * 
 *     cpdef copy_to_host(self, mem, size_t size):
*/
  /*else*/ {
    __pyx_t_9.__pyx_n = 1;
    __pyx_t_9.stream = __pyx_v_stream;
    __pyx_t_1 = ((struct __pyx_vtabstruct_4cupy_4cuda_6memory_MemoryPointer *)__pyx_v_self->__pyx_vtab)->copy_from_host_async(__pyx_v_self, __pyx_v_mem, __pyx_v_size, 0, &__pyx_t_9); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 669, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  }
  __pyx_L3:;

  /* "cupy/cuda/memory.pyx":651
 *             self.copy_from_host(mem, size)
 * 
 *     cpdef copy_from_async(self, mem, size_t size, stream=None):             # <<<<<<<<<<<<<<
 *         """Copies a memory sequence from an arbitrary place asynchronously.
 * 
*/

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_AddTraceback("cupy.cuda.memory.MemoryPointer.copy_from_async", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_pw_4cupy_4cuda_6memory_13MemoryPointer_25copy_from_async(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
PyDoc_STRVAR(__pyx_doc_4cupy_4cuda_6memory_13MemoryPointer_24copy_from_async, "MemoryPointer.copy_from_async(self, mem, size_t size, stream=None)\n\nCopies a memory sequence from an arbitrary place asynchronously.\n\nThis function is a useful interface that selects appropriate one from\n:meth:`~cupy.cuda.MemoryPointer.copy_from_device_async` and\n:meth:`~cupy.cuda.MemoryPointer.copy_from_host_async`.\n\nArgs:\n    mem (int or ctypes.c_void_p or cupy.cuda.MemoryPointer):\n        Source memory pointer.\n    size (int): Size of the sequence in bytes.\n    stream (cupy.cuda.Stream): CUDA stream.\n        The default uses CUDA stream of the current context.");
static PyObject *__pyx_pw_4cupy_4cuda_6memory_13MemoryPointer_25copy_from_async(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
) {
  PyObject *__pyx_v_mem = 0;
  size_t __pyx_v_size;
  PyObject *__pyx_v_stream = 0;
  #if !CYTHON_METH_FASTCALL
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  #endif
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject* values[3] = {0,0,0};
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("copy_from_async (wrapper)", 0);
  #if !CYTHON_METH_FASTCALL
  #if CYTHON_ASSUME_SAFE_SIZE
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return NULL;
  #endif
  #endif
  __pyx_kwvalues = __Pyx_KwValues_FASTCALL(__pyx_args, __pyx_nargs);
  {
    PyObject ** const __pyx_pyargnames[] = {&__pyx_mstate_global->__pyx_n_u_mem_2,&__pyx_mstate_global->__pyx_n_u_size,&__pyx_mstate_global->__pyx_n_u_stream,0};
    const Py_ssize_t __pyx_kwds_len = (__pyx_kwds) ? __Pyx_NumKwargs_FASTCALL(__pyx_kwds) : 0;
    if (unlikely(__pyx_kwds_len < 0)) __PYX_ERR(0, 651, __pyx_L3_error)
    if (__pyx_kwds_len > 0) {
      switch (__pyx_nargs) {
        case  3:
        values[2] = __Pyx_ArgRef_FASTCALL(__pyx_args, 2);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[2])) __PYX_ERR(0, 651, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  2:
        values[1] = __Pyx_ArgRef_FASTCALL(__pyx_args, 1);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[1])) __PYX_ERR(0, 651, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  1:
        values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(0, 651, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      const Py_ssize_t kwd_pos_args = __pyx_nargs;
      if (__Pyx_ParseKeywords(__pyx_kwds, __pyx_kwvalues, __pyx_pyargnames, 0, values, kwd_pos_args, __pyx_kwds_len, "copy_from_async", 0) < (0)) __PYX_ERR(0, 651, __pyx_L3_error)
      if (!values[2]) values[2] = __Pyx_NewRef(((PyObject *)Py_None));
      for (Py_ssize_t i = __pyx_nargs; i < 2; i++) {
        if (unlikely(!values[i])) { __Pyx_RaiseArgtupleInvalid("copy_from_async", 0, 2, 3, i); __PYX_ERR(0, 651, __pyx_L3_error) }
      }
    } else {
      switch (__pyx_nargs) {
        case  3:
        values[2] = __Pyx_ArgRef_FASTCALL(__pyx_args, 2);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[2])) __PYX_ERR(0, 651, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  2:
        values[1] = __Pyx_ArgRef_FASTCALL(__pyx_args, 1);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[1])) __PYX_ERR(0, 651, __pyx_L3_error)
        values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(0, 651, __pyx_L3_error)
        break;
        default: goto __pyx_L5_argtuple_error;
      }
      if (!values[2]) values[2] = __Pyx_NewRef(((PyObject *)Py_None));
    }
    __pyx_v_mem = values[0];
    __pyx_v_size = __Pyx_PyLong_As_size_t(values[1]); if (unlikely((__pyx_v_size == (size_t)-1) && PyErr_Occurred())) __PYX_ERR(0, 651, __pyx_L3_error)
    __pyx_v_stream = values[2];
  }
  goto __pyx_L6_skip;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("copy_from_async", 0, 2, 3, __pyx_nargs); __PYX_ERR(0, 651, __pyx_L3_error)
  __pyx_L6_skip:;
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_AddTraceback("cupy.cuda.memory.MemoryPointer.copy_from_async", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_4cupy_4cuda_6memory_13MemoryPointer_24copy_from_async(((struct __pyx_obj_4cupy_4cuda_6memory_MemoryPointer *)__pyx_v_self), __pyx_v_mem, __pyx_v_size, __pyx_v_stream);

  /* function exit code */
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4cupy_4cuda_6memory_13MemoryPointer_24copy_from_async(struct __pyx_obj_4cupy_4cuda_6memory_MemoryPointer *__pyx_v_self, PyObject *__pyx_v_mem, size_t __pyx_v_size, PyObject *__pyx_v_stream) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  struct __pyx_opt_args_4cupy_4cuda_6memory_13MemoryPointer_copy_from_async __pyx_t_2;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("copy_from_async", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_2.__pyx_n = 1;
  __pyx_t_2.stream = __pyx_v_stream;
  __pyx_t_1 = __pyx_vtabptr_4cupy_4cuda_6memory_MemoryPointer->copy_from_async(__pyx_v_self, __pyx_v_mem, __pyx_v_size, 1, &__pyx_t_2); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 651, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("cupy.cuda.memory.MemoryPointer.copy_from_async", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "cupy/cuda/memory.pyx":671
 *             self.copy_from_host_async(mem, size, stream)
 * 
 *     cpdef copy_to_host(self, mem, size_t size):             # <<<<<<<<<<<<<<
 *         """Copies a memory sequence to the host memory.
 * 
*/

static PyObject *__pyx_pw_4cupy_4cuda_6memory_13MemoryPointer_27copy_to_host(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
static PyObject *__pyx_f_4cupy_4cuda_6memory_13MemoryPointer_copy_to_host(struct __pyx_obj_4cupy_4cuda_6memory_MemoryPointer *__pyx_v_self, PyObject *__pyx_v_mem, size_t __pyx_v_size, int __pyx_skip_dispatch) {
  intptr_t __pyx_v_stream_ptr;
  PyObject *__pyx_v_ptr = NULL;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  size_t __pyx_t_6;
  intptr_t __pyx_t_7;
  int __pyx_t_8;
  int __pyx_t_9;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("copy_to_host", 0);
  /* Check if called by wrapper */
  if (unlikely(__pyx_skip_dispatch)) ;
  /* Check if overridden in Python */
  else if (
  #if !CYTHON_USE_TYPE_SLOTS
  unlikely(Py_TYPE(((PyObject *)__pyx_v_self)) != __pyx_mstate_global->__pyx_ptype_4cupy_4cuda_6memory_MemoryPointer &&
  __Pyx_PyType_HasFeature(Py_TYPE(((PyObject *)__pyx_v_self)), Py_TPFLAGS_HAVE_GC))
  #else
  unlikely(Py_TYPE(((PyObject *)__pyx_v_self))->tp_dictoffset != 0 || __Pyx_PyType_HasFeature(Py_TYPE(((PyObject *)__pyx_v_self)), (Py_TPFLAGS_IS_ABSTRACT | Py_TPFLAGS_HEAPTYPE)))
  #endif
  ) {
    #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
    static PY_UINT64_T __pyx_tp_dict_version = __PYX_DICT_VERSION_INIT, __pyx_obj_dict_version = __PYX_DICT_VERSION_INIT;
    if (unlikely(!__Pyx_object_dict_version_matches(((PyObject *)__pyx_v_self), __pyx_tp_dict_version, __pyx_obj_dict_version))) {
      PY_UINT64_T __pyx_typedict_guard = __Pyx_get_tp_dict_version(((PyObject *)__pyx_v_self));
      #endif
      __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_mstate_global->__pyx_n_u_copy_to_host); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 671, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      if (!__Pyx_IsSameCFunction(__pyx_t_1, (void(*)(void)) __pyx_pw_4cupy_4cuda_6memory_13MemoryPointer_27copy_to_host)) {
        __Pyx_XDECREF(__pyx_r);
        __pyx_t_3 = NULL;
        __Pyx_INCREF(__pyx_t_1);
        __pyx_t_4 = __pyx_t_1; 
        __pyx_t_5 = __Pyx_PyLong_FromSize_t(__pyx_v_size); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 671, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_5);
        __pyx_t_6 = 1;
        #if CYTHON_UNPACK_METHODS
        if (unlikely(PyMethod_Check(__pyx_t_4))) {
          __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_4);
          assert(__pyx_t_3);
          PyObject* __pyx__function = PyMethod_GET_FUNCTION(__pyx_t_4);
          __Pyx_INCREF(__pyx_t_3);
          __Pyx_INCREF(__pyx__function);
          __Pyx_DECREF_SET(__pyx_t_4, __pyx__function);
          __pyx_t_6 = 0;
        }
        #endif
        {
          PyObject *__pyx_callargs[3] = {__pyx_t_3, __pyx_v_mem, __pyx_t_5};
          __pyx_t_2 = __Pyx_PyObject_FastCall((PyObject*)__pyx_t_4, __pyx_callargs+__pyx_t_6, (3-__pyx_t_6) | (__pyx_t_6*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
          __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
          __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
          __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
          if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 671, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_2);
        }
        __pyx_r = __pyx_t_2;
        __pyx_t_2 = 0;
        __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
        goto __pyx_L0;
      }
      #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
      __pyx_tp_dict_version = __Pyx_get_tp_dict_version(((PyObject *)__pyx_v_self));
      __pyx_obj_dict_version = __Pyx_get_object_dict_version(((PyObject *)__pyx_v_self));
      if (unlikely(__pyx_typedict_guard != __pyx_tp_dict_version)) {
        __pyx_tp_dict_version = __pyx_obj_dict_version = __PYX_DICT_VERSION_INIT;
      }
      #endif
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
    }
    #endif
  }

  /* "cupy/cuda/memory.pyx":685
 * 
 *         """
 *         stream_ptr = stream_module.get_current_stream_ptr()             # <<<<<<<<<<<<<<
 *         if (
 *             not runtime._is_hip_environment
*/
  __pyx_t_7 = __pyx_f_4cupy_4cuda_6stream_get_current_stream_ptr(); if (unlikely(__pyx_t_7 == ((intptr_t)-1L) && PyErr_Occurred())) __PYX_ERR(0, 685, __pyx_L1_error)
  __pyx_v_stream_ptr = __pyx_t_7;

  /* "cupy/cuda/memory.pyx":687
 *         stream_ptr = stream_module.get_current_stream_ptr()
 *         if (
 *             not runtime._is_hip_environment             # <<<<<<<<<<<<<<
 *             and runtime.streamIsCapturing(stream_ptr)
 *         ):
*/
  __pyx_t_9 = (!__pyx_v_13cupy_backends_4cuda_3api_7runtime__is_hip_environment);
  if (__pyx_t_9) {
  } else {
    __pyx_t_8 = __pyx_t_9;
    goto __pyx_L4_bool_binop_done;
  }

  /* "cupy/cuda/memory.pyx":688
 *         if (
 *             not runtime._is_hip_environment
 *             and runtime.streamIsCapturing(stream_ptr)             # <<<<<<<<<<<<<<
 *         ):
 *             raise RuntimeError(
*/
  __pyx_t_9 = __pyx_f_13cupy_backends_4cuda_3api_7runtime_streamIsCapturing(__pyx_v_stream_ptr, 0); if (unlikely(PyErr_Occurred())) __PYX_ERR(0, 688, __pyx_L1_error)
  __pyx_t_8 = __pyx_t_9;
  __pyx_L4_bool_binop_done:;

  /* "cupy/cuda/memory.pyx":686
 *         """
 *         stream_ptr = stream_module.get_current_stream_ptr()
 *         if (             # <<<<<<<<<<<<<<
 *             not runtime._is_hip_environment
 *             and runtime.streamIsCapturing(stream_ptr)
*/
  if (unlikely(__pyx_t_8)) {

    /* "cupy/cuda/memory.pyx":690
 *             and runtime.streamIsCapturing(stream_ptr)
 *         ):
 *             raise RuntimeError(             # <<<<<<<<<<<<<<
 *                 'the current stream is capturing, so synchronous API calls '
 *                 'are disallowed')
*/
    __pyx_t_2 = NULL;
    __pyx_t_6 = 1;
    {
      PyObject *__pyx_callargs[2] = {__pyx_t_2, __pyx_mstate_global->__pyx_kp_u_the_current_stream_is_capturing};
      __pyx_t_1 = __Pyx_PyObject_FastCall((PyObject*)(((PyTypeObject*)PyExc_RuntimeError)), __pyx_callargs+__pyx_t_6, (2-__pyx_t_6) | (__pyx_t_6*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
      __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
      if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 690, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
    }
    __Pyx_Raise(__pyx_t_1, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __PYX_ERR(0, 690, __pyx_L1_error)

    /* "cupy/cuda/memory.pyx":686
 *         """
 *         stream_ptr = stream_module.get_current_stream_ptr()
 *         if (             # <<<<<<<<<<<<<<
 *             not runtime._is_hip_environment
 *             and runtime.streamIsCapturing(stream_ptr)
*/
  }

  /* "cupy/cuda/memory.pyx":693
 *                 'the current stream is capturing, so synchronous API calls '
 *                 'are disallowed')
 *         if size > 0:             # <<<<<<<<<<<<<<
 *             ptr = mem if isinstance(mem, int) else mem.value
 *             runtime.memcpy(ptr, self.ptr, size,
*/
  __pyx_t_8 = (__pyx_v_size > 0);
  if (__pyx_t_8) {

    /* "cupy/cuda/memory.pyx":694
 *                 'are disallowed')
 *         if size > 0:
 *             ptr = mem if isinstance(mem, int) else mem.value             # <<<<<<<<<<<<<<
 *             runtime.memcpy(ptr, self.ptr, size,
 *                            runtime.memcpyDeviceToHost)
*/
    __pyx_t_8 = PyLong_Check(__pyx_v_mem); 
    if (__pyx_t_8) {
      __Pyx_INCREF(__pyx_v_mem);
      __pyx_t_1 = __pyx_v_mem;
    } else {
      __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_v_mem, __pyx_mstate_global->__pyx_n_u_value); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 694, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_2);
      __pyx_t_1 = __pyx_t_2;
      __pyx_t_2 = 0;
    }
    __pyx_v_ptr = __pyx_t_1;
    __pyx_t_1 = 0;

    /* "cupy/cuda/memory.pyx":695
 *         if size > 0:
 *             ptr = mem if isinstance(mem, int) else mem.value
 *             runtime.memcpy(ptr, self.ptr, size,             # <<<<<<<<<<<<<<
 *                            runtime.memcpyDeviceToHost)
 * 
*/
    __pyx_t_7 = PyLong_AsSsize_t(__pyx_v_ptr); if (unlikely((__pyx_t_7 == ((intptr_t)-1)) && PyErr_Occurred())) __PYX_ERR(0, 695, __pyx_L1_error)

    /* "cupy/cuda/memory.pyx":696
 *             ptr = mem if isinstance(mem, int) else mem.value
 *             runtime.memcpy(ptr, self.ptr, size,
 *                            runtime.memcpyDeviceToHost)             # <<<<<<<<<<<<<<
 * 
 *     cpdef copy_to_host_async(self, mem, size_t size, stream=None):
*/
    __pyx_t_1 = __pyx_f_13cupy_backends_4cuda_3api_7runtime_memcpy(__pyx_t_7, __pyx_v_self->ptr, __pyx_v_size, __pyx_e_13cupy_backends_4cuda_3api_7runtime_memcpyDeviceToHost, 0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 695, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

    /* "cupy/cuda/memory.pyx":693
 *                 'the current stream is capturing, so synchronous API calls '
 *                 'are disallowed')
 *         if size > 0:             # <<<<<<<<<<<<<<
 *             ptr = mem if isinstance(mem, int) else mem.value
 *             runtime.memcpy(ptr, self.ptr, size,
*/
  }

  /* "cupy/cuda/memory.pyx":671
 *             self.copy_from_host_async(mem, size, stream)
 * 
 *     cpdef copy_to_host(self, mem, size_t size):             # <<<<<<<<<<<<<<
 *         """Copies a memory sequence to the host memory.
 * 
*/

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_AddTraceback("cupy.cuda.memory.MemoryPointer.copy_to_host", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_ptr);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_pw_4cupy_4cuda_6memory_13MemoryPointer_27copy_to_host(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
PyDoc_STRVAR(__pyx_doc_4cupy_4cuda_6memory_13MemoryPointer_26copy_to_host, "MemoryPointer.copy_to_host(self, mem, size_t size)\n\nCopies a memory sequence to the host memory.\n\nArgs:\n    mem (int or ctypes.c_void_p): Target memory pointer.\n    size (int): Size of the sequence in bytes.\n\n.. warning::\n\n    This function always uses the legacy default stream and does not\n    honor the current stream. Use `copy_to_host_async` instead\n    if you are using streams in your code, or have PTDS enabled.");
static PyObject *__pyx_pw_4cupy_4cuda_6memory_13MemoryPointer_27copy_to_host(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
) {
  PyObject *__pyx_v_mem = 0;
  size_t __pyx_v_size;
  #if !CYTHON_METH_FASTCALL
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  #endif
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject* values[2] = {0,0};
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("copy_to_host (wrapper)", 0);
  #if !CYTHON_METH_FASTCALL
  #if CYTHON_ASSUME_SAFE_SIZE
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return NULL;
  #endif
  #endif
  __pyx_kwvalues = __Pyx_KwValues_FASTCALL(__pyx_args, __pyx_nargs);
  {
    PyObject ** const __pyx_pyargnames[] = {&__pyx_mstate_global->__pyx_n_u_mem_2,&__pyx_mstate_global->__pyx_n_u_size,0};
    const Py_ssize_t __pyx_kwds_len = (__pyx_kwds) ? __Pyx_NumKwargs_FASTCALL(__pyx_kwds) : 0;
    if (unlikely(__pyx_kwds_len < 0)) __PYX_ERR(0, 671, __pyx_L3_error)
    if (__pyx_kwds_len > 0) {
      switch (__pyx_nargs) {
        case  2:
        values[1] = __Pyx_ArgRef_FASTCALL(__pyx_args, 1);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[1])) __PYX_ERR(0, 671, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  1:
        values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(0, 671, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      const Py_ssize_t kwd_pos_args = __pyx_nargs;
      if (__Pyx_ParseKeywords(__pyx_kwds, __pyx_kwvalues, __pyx_pyargnames, 0, values, kwd_pos_args, __pyx_kwds_len, "copy_to_host", 0) < (0)) __PYX_ERR(0, 671, __pyx_L3_error)
      for (Py_ssize_t i = __pyx_nargs; i < 2; i++) {
        if (unlikely(!values[i])) { __Pyx_RaiseArgtupleInvalid("copy_to_host", 1, 2, 2, i); __PYX_ERR(0, 671, __pyx_L3_error) }
      }
    } else if (unlikely(__pyx_nargs != 2)) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
      if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(0, 671, __pyx_L3_error)
      values[1] = __Pyx_ArgRef_FASTCALL(__pyx_args, 1);
      if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[1])) __PYX_ERR(0, 671, __pyx_L3_error)
    }
    __pyx_v_mem = values[0];
    __pyx_v_size = __Pyx_PyLong_As_size_t(values[1]); if (unlikely((__pyx_v_size == (size_t)-1) && PyErr_Occurred())) __PYX_ERR(0, 671, __pyx_L3_error)
  }
  goto __pyx_L6_skip;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("copy_to_host", 1, 2, 2, __pyx_nargs); __PYX_ERR(0, 671, __pyx_L3_error)
  __pyx_L6_skip:;
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_AddTraceback("cupy.cuda.memory.MemoryPointer.copy_to_host", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_4cupy_4cuda_6memory_13MemoryPointer_26copy_to_host(((struct __pyx_obj_4cupy_4cuda_6memory_MemoryPointer *)__pyx_v_self), __pyx_v_mem, __pyx_v_size);

  /* function exit code */
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4cupy_4cuda_6memory_13MemoryPointer_26copy_to_host(struct __pyx_obj_4cupy_4cuda_6memory_MemoryPointer *__pyx_v_self, PyObject *__pyx_v_mem, size_t __pyx_v_size) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("copy_to_host", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __pyx_f_4cupy_4cuda_6memory_13MemoryPointer_copy_to_host(__pyx_v_self, __pyx_v_mem, __pyx_v_size, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 671, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("cupy.cuda.memory.MemoryPointer.copy_to_host", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "cupy/cuda/memory.pyx":698
 *                            runtime.memcpyDeviceToHost)
 * 
 *     cpdef copy_to_host_async(self, mem, size_t size, stream=None):             # <<<<<<<<<<<<<<
 *         """Copies a memory sequence to the host memory asynchronously.
 * 
*/

static PyObject *__pyx_pw_4cupy_4cuda_6memory_13MemoryPointer_29copy_to_host_async(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
static PyObject *__pyx_f_4cupy_4cuda_6memory_13MemoryPointer_copy_to_host_async(struct __pyx_obj_4cupy_4cuda_6memory_MemoryPointer *__pyx_v_self, PyObject *__pyx_v_mem, size_t __pyx_v_size, int __pyx_skip_dispatch, struct __pyx_opt_args_4cupy_4cuda_6memory_13MemoryPointer_copy_to_host_async *__pyx_optional_args) {
  PyObject *__pyx_v_stream = ((PyObject *)Py_None);
  PyObject *__pyx_v_stream_ptr = NULL;
  PyObject *__pyx_v_ptr = NULL;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  size_t __pyx_t_6;
  int __pyx_t_7;
  intptr_t __pyx_t_8;
  int __pyx_t_9;
  intptr_t __pyx_t_10;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("copy_to_host_async", 0);
  if (__pyx_optional_args) {
    if (__pyx_optional_args->__pyx_n > 0) {
      __pyx_v_stream = __pyx_optional_args->stream;
    }
  }
  /* Check if called by wrapper */
  if (unlikely(__pyx_skip_dispatch)) ;
  /* Check if overridden in Python */
  else if (
  #if !CYTHON_USE_TYPE_SLOTS
  unlikely(Py_TYPE(((PyObject *)__pyx_v_self)) != __pyx_mstate_global->__pyx_ptype_4cupy_4cuda_6memory_MemoryPointer &&
  __Pyx_PyType_HasFeature(Py_TYPE(((PyObject *)__pyx_v_self)), Py_TPFLAGS_HAVE_GC))
  #else
  unlikely(Py_TYPE(((PyObject *)__pyx_v_self))->tp_dictoffset != 0 || __Pyx_PyType_HasFeature(Py_TYPE(((PyObject *)__pyx_v_self)), (Py_TPFLAGS_IS_ABSTRACT | Py_TPFLAGS_HEAPTYPE)))
  #endif
  ) {
    #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
    static PY_UINT64_T __pyx_tp_dict_version = __PYX_DICT_VERSION_INIT, __pyx_obj_dict_version = __PYX_DICT_VERSION_INIT;
    if (unlikely(!__Pyx_object_dict_version_matches(((PyObject *)__pyx_v_self), __pyx_tp_dict_version, __pyx_obj_dict_version))) {
      PY_UINT64_T __pyx_typedict_guard = __Pyx_get_tp_dict_version(((PyObject *)__pyx_v_self));
      #endif
      __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_mstate_global->__pyx_n_u_copy_to_host_async); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 698, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      if (!__Pyx_IsSameCFunction(__pyx_t_1, (void(*)(void)) __pyx_pw_4cupy_4cuda_6memory_13MemoryPointer_29copy_to_host_async)) {
        __Pyx_XDECREF(__pyx_r);
        __pyx_t_3 = NULL;
        __Pyx_INCREF(__pyx_t_1);
        __pyx_t_4 = __pyx_t_1; 
        __pyx_t_5 = __Pyx_PyLong_FromSize_t(__pyx_v_size); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 698, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_5);
        __pyx_t_6 = 1;
        #if CYTHON_UNPACK_METHODS
        if (unlikely(PyMethod_Check(__pyx_t_4))) {
          __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_4);
          assert(__pyx_t_3);
          PyObject* __pyx__function = PyMethod_GET_FUNCTION(__pyx_t_4);
          __Pyx_INCREF(__pyx_t_3);
          __Pyx_INCREF(__pyx__function);
          __Pyx_DECREF_SET(__pyx_t_4, __pyx__function);
          __pyx_t_6 = 0;
        }
        #endif
        {
          PyObject *__pyx_callargs[4] = {__pyx_t_3, __pyx_v_mem, __pyx_t_5, __pyx_v_stream};
          __pyx_t_2 = __Pyx_PyObject_FastCall((PyObject*)__pyx_t_4, __pyx_callargs+__pyx_t_6, (4-__pyx_t_6) | (__pyx_t_6*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
          __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
          __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
          __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
          if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 698, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_2);
        }
        __pyx_r = __pyx_t_2;
        __pyx_t_2 = 0;
        __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
        goto __pyx_L0;
      }
      #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
      __pyx_tp_dict_version = __Pyx_get_tp_dict_version(((PyObject *)__pyx_v_self));
      __pyx_obj_dict_version = __Pyx_get_object_dict_version(((PyObject *)__pyx_v_self));
      if (unlikely(__pyx_typedict_guard != __pyx_tp_dict_version)) {
        __pyx_tp_dict_version = __pyx_obj_dict_version = __PYX_DICT_VERSION_INIT;
      }
      #endif
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
    }
    #endif
  }

  /* "cupy/cuda/memory.pyx":709
 * 
 *         """
 *         if stream is None:             # <<<<<<<<<<<<<<
 *             stream_ptr = stream_module.get_current_stream_ptr()
 *         else:
*/
  __pyx_t_7 = (__pyx_v_stream == Py_None);
  if (__pyx_t_7) {

    /* "cupy/cuda/memory.pyx":710
 *         """
 *         if stream is None:
 *             stream_ptr = stream_module.get_current_stream_ptr()             # <<<<<<<<<<<<<<
 *         else:
 *             stream_ptr = stream.ptr
*/
    __pyx_t_8 = __pyx_f_4cupy_4cuda_6stream_get_current_stream_ptr(); if (unlikely(__pyx_t_8 == ((intptr_t)-1L) && PyErr_Occurred())) __PYX_ERR(0, 710, __pyx_L1_error)
    __pyx_t_1 = PyLong_FromSsize_t(__pyx_t_8); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 710, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_v_stream_ptr = __pyx_t_1;
    __pyx_t_1 = 0;

    /* "cupy/cuda/memory.pyx":709
 * 
 *         """
 *         if stream is None:             # <<<<<<<<<<<<<<
 *             stream_ptr = stream_module.get_current_stream_ptr()
 *         else:
*/
    goto __pyx_L3;
  }

  /* "cupy/cuda/memory.pyx":712
 *             stream_ptr = stream_module.get_current_stream_ptr()
 *         else:
 *             stream_ptr = stream.ptr             # <<<<<<<<<<<<<<
 *         if (
 *             not runtime._is_hip_environment
*/
  /*else*/ {
    __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_v_stream, __pyx_mstate_global->__pyx_n_u_ptr); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 712, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_v_stream_ptr = __pyx_t_1;
    __pyx_t_1 = 0;
  }
  __pyx_L3:;

  /* "cupy/cuda/memory.pyx":714
 *             stream_ptr = stream.ptr
 *         if (
 *             not runtime._is_hip_environment             # <<<<<<<<<<<<<<
 *             and runtime.streamIsCapturing(stream_ptr)
 *         ):
*/
  __pyx_t_9 = (!__pyx_v_13cupy_backends_4cuda_3api_7runtime__is_hip_environment);
  if (__pyx_t_9) {
  } else {
    __pyx_t_7 = __pyx_t_9;
    goto __pyx_L5_bool_binop_done;
  }

  /* "cupy/cuda/memory.pyx":715
 *         if (
 *             not runtime._is_hip_environment
 *             and runtime.streamIsCapturing(stream_ptr)             # <<<<<<<<<<<<<<
 *         ):
 *             raise RuntimeError(
*/
  __pyx_t_8 = PyLong_AsSsize_t(__pyx_v_stream_ptr); if (unlikely((__pyx_t_8 == ((intptr_t)-1)) && PyErr_Occurred())) __PYX_ERR(0, 715, __pyx_L1_error)
  __pyx_t_9 = __pyx_f_13cupy_backends_4cuda_3api_7runtime_streamIsCapturing(__pyx_t_8, 0); if (unlikely(PyErr_Occurred())) __PYX_ERR(0, 715, __pyx_L1_error)
  __pyx_t_7 = __pyx_t_9;
  __pyx_L5_bool_binop_done:;

  /* "cupy/cuda/memory.pyx":713
 *         else:
 *             stream_ptr = stream.ptr
 *         if (             # <<<<<<<<<<<<<<
 *             not runtime._is_hip_environment
 *             and runtime.streamIsCapturing(stream_ptr)
*/
  if (unlikely(__pyx_t_7)) {

    /* "cupy/cuda/memory.pyx":717
 *             and runtime.streamIsCapturing(stream_ptr)
 *         ):
 *             raise RuntimeError(             # <<<<<<<<<<<<<<
 *                 'the current stream is capturing, so D2H transfers '
 *                 'are disallowed')
*/
    __pyx_t_2 = NULL;
    __pyx_t_6 = 1;
    {
      PyObject *__pyx_callargs[2] = {__pyx_t_2, __pyx_mstate_global->__pyx_kp_u_the_current_stream_is_capturing_3};
      __pyx_t_1 = __Pyx_PyObject_FastCall((PyObject*)(((PyTypeObject*)PyExc_RuntimeError)), __pyx_callargs+__pyx_t_6, (2-__pyx_t_6) | (__pyx_t_6*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
      __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
      if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 717, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
    }
    __Pyx_Raise(__pyx_t_1, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __PYX_ERR(0, 717, __pyx_L1_error)

    /* "cupy/cuda/memory.pyx":713
 *         else:
 *             stream_ptr = stream.ptr
 *         if (             # <<<<<<<<<<<<<<
 *             not runtime._is_hip_environment
 *             and runtime.streamIsCapturing(stream_ptr)
*/
  }

  /* "cupy/cuda/memory.pyx":720
 *                 'the current stream is capturing, so D2H transfers '
 *                 'are disallowed')
 *         if size > 0:             # <<<<<<<<<<<<<<
 *             ptr = mem if isinstance(mem, int) else mem.value
 *             runtime.memcpyAsync(ptr, self.ptr, size,
*/
  __pyx_t_7 = (__pyx_v_size > 0);
  if (__pyx_t_7) {

    /* "cupy/cuda/memory.pyx":721
 *                 'are disallowed')
 *         if size > 0:
 *             ptr = mem if isinstance(mem, int) else mem.value             # <<<<<<<<<<<<<<
 *             runtime.memcpyAsync(ptr, self.ptr, size,
 *                                 runtime.memcpyDeviceToHost, stream_ptr)
*/
    __pyx_t_7 = PyLong_Check(__pyx_v_mem); 
    if (__pyx_t_7) {
      __Pyx_INCREF(__pyx_v_mem);
      __pyx_t_1 = __pyx_v_mem;
    } else {
      __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_v_mem, __pyx_mstate_global->__pyx_n_u_value); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 721, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_2);
      __pyx_t_1 = __pyx_t_2;
      __pyx_t_2 = 0;
    }
    __pyx_v_ptr = __pyx_t_1;
    __pyx_t_1 = 0;

    /* "cupy/cuda/memory.pyx":722
 *         if size > 0:
 *             ptr = mem if isinstance(mem, int) else mem.value
 *             runtime.memcpyAsync(ptr, self.ptr, size,             # <<<<<<<<<<<<<<
 *                                 runtime.memcpyDeviceToHost, stream_ptr)
 * 
*/
    __pyx_t_8 = PyLong_AsSsize_t(__pyx_v_ptr); if (unlikely((__pyx_t_8 == ((intptr_t)-1)) && PyErr_Occurred())) __PYX_ERR(0, 722, __pyx_L1_error)

    /* "cupy/cuda/memory.pyx":723
 *             ptr = mem if isinstance(mem, int) else mem.value
 *             runtime.memcpyAsync(ptr, self.ptr, size,
 *                                 runtime.memcpyDeviceToHost, stream_ptr)             # <<<<<<<<<<<<<<
 * 
 *     cpdef memset(self, int value, size_t size):
*/
    __pyx_t_10 = PyLong_AsSsize_t(__pyx_v_stream_ptr); if (unlikely((__pyx_t_10 == ((intptr_t)-1)) && PyErr_Occurred())) __PYX_ERR(0, 723, __pyx_L1_error)

    /* "cupy/cuda/memory.pyx":722
 *         if size > 0:
 *             ptr = mem if isinstance(mem, int) else mem.value
 *             runtime.memcpyAsync(ptr, self.ptr, size,             # <<<<<<<<<<<<<<
 *                                 runtime.memcpyDeviceToHost, stream_ptr)
 * 
*/
    __pyx_t_1 = __pyx_f_13cupy_backends_4cuda_3api_7runtime_memcpyAsync(__pyx_t_8, __pyx_v_self->ptr, __pyx_v_size, __pyx_e_13cupy_backends_4cuda_3api_7runtime_memcpyDeviceToHost, __pyx_t_10, 0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 722, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

    /* "cupy/cuda/memory.pyx":720
 *                 'the current stream is capturing, so D2H transfers '
 *                 'are disallowed')
 *         if size > 0:             # <<<<<<<<<<<<<<
 *             ptr = mem if isinstance(mem, int) else mem.value
 *             runtime.memcpyAsync(ptr, self.ptr, size,
*/
  }

  /* "cupy/cuda/memory.pyx":698
 *                            runtime.memcpyDeviceToHost)
 * 
 *     cpdef copy_to_host_async(self, mem, size_t size, stream=None):             # <<<<<<<<<<<<<<
 *         """Copies a memory sequence to the host memory asynchronously.
 * 
*/

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_AddTraceback("cupy.cuda.memory.MemoryPointer.copy_to_host_async", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_stream_ptr);
  __Pyx_XDECREF(__pyx_v_ptr);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_pw_4cupy_4cuda_6memory_13MemoryPointer_29copy_to_host_async(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
PyDoc_STRVAR(__pyx_doc_4cupy_4cuda_6memory_13MemoryPointer_28copy_to_host_async, "MemoryPointer.copy_to_host_async(self, mem, size_t size, stream=None)\n\nCopies a memory sequence to the host memory asynchronously.\n\nArgs:\n    mem (int or ctypes.c_void_p): Target memory pointer. It must point\n        to pinned memory.\n    size (int): Size of the sequence in bytes.\n    stream (cupy.cuda.Stream): CUDA stream.\n        The default uses CUDA stream of the current context.");
static PyObject *__pyx_pw_4cupy_4cuda_6memory_13MemoryPointer_29copy_to_host_async(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
) {
  PyObject *__pyx_v_mem = 0;
  size_t __pyx_v_size;
  PyObject *__pyx_v_stream = 0;
  #if !CYTHON_METH_FASTCALL
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  #endif
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject* values[3] = {0,0,0};
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("copy_to_host_async (wrapper)", 0);
  #if !CYTHON_METH_FASTCALL
  #if CYTHON_ASSUME_SAFE_SIZE
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return NULL;
  #endif
  #endif
  __pyx_kwvalues = __Pyx_KwValues_FASTCALL(__pyx_args, __pyx_nargs);
  {
    PyObject ** const __pyx_pyargnames[] = {&__pyx_mstate_global->__pyx_n_u_mem_2,&__pyx_mstate_global->__pyx_n_u_size,&__pyx_mstate_global->__pyx_n_u_stream,0};
    const Py_ssize_t __pyx_kwds_len = (__pyx_kwds) ? __Pyx_NumKwargs_FASTCALL(__pyx_kwds) : 0;
    if (unlikely(__pyx_kwds_len < 0)) __PYX_ERR(0, 698, __pyx_L3_error)
    if (__pyx_kwds_len > 0) {
      switch (__pyx_nargs) {
        case  3:
        values[2] = __Pyx_ArgRef_FASTCALL(__pyx_args, 2);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[2])) __PYX_ERR(0, 698, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  2:
        values[1] = __Pyx_ArgRef_FASTCALL(__pyx_args, 1);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[1])) __PYX_ERR(0, 698, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  1:
        values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(0, 698, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      const Py_ssize_t kwd_pos_args = __pyx_nargs;
      if (__Pyx_ParseKeywords(__pyx_kwds, __pyx_kwvalues, __pyx_pyargnames, 0, values, kwd_pos_args, __pyx_kwds_len, "copy_to_host_async", 0) < (0)) __PYX_ERR(0, 698, __pyx_L3_error)
      if (!values[2]) values[2] = __Pyx_NewRef(((PyObject *)Py_None));
      for (Py_ssize_t i = __pyx_nargs; i < 2; i++) {
        if (unlikely(!values[i])) { __Pyx_RaiseArgtupleInvalid("copy_to_host_async", 0, 2, 3, i); __PYX_ERR(0, 698, __pyx_L3_error) }
      }
    } else {
      switch (__pyx_nargs) {
        case  3:
        values[2] = __Pyx_ArgRef_FASTCALL(__pyx_args, 2);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[2])) __PYX_ERR(0, 698, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  2:
        values[1] = __Pyx_ArgRef_FASTCALL(__pyx_args, 1);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[1])) __PYX_ERR(0, 698, __pyx_L3_error)
        values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(0, 698, __pyx_L3_error)
        break;
        default: goto __pyx_L5_argtuple_error;
      }
      if (!values[2]) values[2] = __Pyx_NewRef(((PyObject *)Py_None));
    }
    __pyx_v_mem = values[0];
    __pyx_v_size = __Pyx_PyLong_As_size_t(values[1]); if (unlikely((__pyx_v_size == (size_t)-1) && PyErr_Occurred())) __PYX_ERR(0, 698, __pyx_L3_error)
    __pyx_v_stream = values[2];
  }
  goto __pyx_L6_skip;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("copy_to_host_async", 0, 2, 3, __pyx_nargs); __PYX_ERR(0, 698, __pyx_L3_error)
  __pyx_L6_skip:;
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_AddTraceback("cupy.cuda.memory.MemoryPointer.copy_to_host_async", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_4cupy_4cuda_6memory_13MemoryPointer_28copy_to_host_async(((struct __pyx_obj_4cupy_4cuda_6memory_MemoryPointer *)__pyx_v_self), __pyx_v_mem, __pyx_v_size, __pyx_v_stream);

  /* function exit code */
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4cupy_4cuda_6memory_13MemoryPointer_28copy_to_host_async(struct __pyx_obj_4cupy_4cuda_6memory_MemoryPointer *__pyx_v_self, PyObject *__pyx_v_mem, size_t __pyx_v_size, PyObject *__pyx_v_stream) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  struct __pyx_opt_args_4cupy_4cuda_6memory_13MemoryPointer_copy_to_host_async __pyx_t_2;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("copy_to_host_async", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_2.__pyx_n = 1;
  __pyx_t_2.stream = __pyx_v_stream;
  __pyx_t_1 = __pyx_vtabptr_4cupy_4cuda_6memory_MemoryPointer->copy_to_host_async(__pyx_v_self, __pyx_v_mem, __pyx_v_size, 1, &__pyx_t_2); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 698, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("cupy.cuda.memory.MemoryPointer.copy_to_host_async", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "cupy/cuda/memory.pyx":725
 *                                 runtime.memcpyDeviceToHost, stream_ptr)
 * 
 *     cpdef memset(self, int value, size_t size):             # <<<<<<<<<<<<<<
 *         """Fills a memory sequence by constant byte value.
 * 
*/

static PyObject *__pyx_pw_4cupy_4cuda_6memory_13MemoryPointer_31memset(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
static PyObject *__pyx_f_4cupy_4cuda_6memory_13MemoryPointer_memset(struct __pyx_obj_4cupy_4cuda_6memory_MemoryPointer *__pyx_v_self, int __pyx_v_value, size_t __pyx_v_size, int __pyx_skip_dispatch) {
  intptr_t __pyx_v_stream_ptr;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  PyObject *__pyx_t_6 = NULL;
  size_t __pyx_t_7;
  intptr_t __pyx_t_8;
  int __pyx_t_9;
  int __pyx_t_10;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("memset", 0);
  /* Check if called by wrapper */
  if (unlikely(__pyx_skip_dispatch)) ;
  /* Check if overridden in Python */
  else if (
  #if !CYTHON_USE_TYPE_SLOTS
  unlikely(Py_TYPE(((PyObject *)__pyx_v_self)) != __pyx_mstate_global->__pyx_ptype_4cupy_4cuda_6memory_MemoryPointer &&
  __Pyx_PyType_HasFeature(Py_TYPE(((PyObject *)__pyx_v_self)), Py_TPFLAGS_HAVE_GC))
  #else
  unlikely(Py_TYPE(((PyObject *)__pyx_v_self))->tp_dictoffset != 0 || __Pyx_PyType_HasFeature(Py_TYPE(((PyObject *)__pyx_v_self)), (Py_TPFLAGS_IS_ABSTRACT | Py_TPFLAGS_HEAPTYPE)))
  #endif
  ) {
    #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
    static PY_UINT64_T __pyx_tp_dict_version = __PYX_DICT_VERSION_INIT, __pyx_obj_dict_version = __PYX_DICT_VERSION_INIT;
    if (unlikely(!__Pyx_object_dict_version_matches(((PyObject *)__pyx_v_self), __pyx_tp_dict_version, __pyx_obj_dict_version))) {
      PY_UINT64_T __pyx_typedict_guard = __Pyx_get_tp_dict_version(((PyObject *)__pyx_v_self));
      #endif
      __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_mstate_global->__pyx_n_u_memset); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 725, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      if (!__Pyx_IsSameCFunction(__pyx_t_1, (void(*)(void)) __pyx_pw_4cupy_4cuda_6memory_13MemoryPointer_31memset)) {
        __Pyx_XDECREF(__pyx_r);
        __pyx_t_3 = NULL;
        __Pyx_INCREF(__pyx_t_1);
        __pyx_t_4 = __pyx_t_1; 
        __pyx_t_5 = __Pyx_PyLong_From_int(__pyx_v_value); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 725, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_5);
        __pyx_t_6 = __Pyx_PyLong_FromSize_t(__pyx_v_size); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 725, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_6);
        __pyx_t_7 = 1;
        #if CYTHON_UNPACK_METHODS
        if (unlikely(PyMethod_Check(__pyx_t_4))) {
          __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_4);
          assert(__pyx_t_3);
          PyObject* __pyx__function = PyMethod_GET_FUNCTION(__pyx_t_4);
          __Pyx_INCREF(__pyx_t_3);
          __Pyx_INCREF(__pyx__function);
          __Pyx_DECREF_SET(__pyx_t_4, __pyx__function);
          __pyx_t_7 = 0;
        }
        #endif
        {
          PyObject *__pyx_callargs[3] = {__pyx_t_3, __pyx_t_5, __pyx_t_6};
          __pyx_t_2 = __Pyx_PyObject_FastCall((PyObject*)__pyx_t_4, __pyx_callargs+__pyx_t_7, (3-__pyx_t_7) | (__pyx_t_7*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
          __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
          __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
          __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
          __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
          if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 725, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_2);
        }
        __pyx_r = __pyx_t_2;
        __pyx_t_2 = 0;
        __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
        goto __pyx_L0;
      }
      #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
      __pyx_tp_dict_version = __Pyx_get_tp_dict_version(((PyObject *)__pyx_v_self));
      __pyx_obj_dict_version = __Pyx_get_object_dict_version(((PyObject *)__pyx_v_self));
      if (unlikely(__pyx_typedict_guard != __pyx_tp_dict_version)) {
        __pyx_tp_dict_version = __pyx_obj_dict_version = __PYX_DICT_VERSION_INIT;
      }
      #endif
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
    }
    #endif
  }

  /* "cupy/cuda/memory.pyx":739
 * 
 *         """
 *         stream_ptr = stream_module.get_current_stream_ptr()             # <<<<<<<<<<<<<<
 *         if (
 *             not runtime._is_hip_environment
*/
  __pyx_t_8 = __pyx_f_4cupy_4cuda_6stream_get_current_stream_ptr(); if (unlikely(__pyx_t_8 == ((intptr_t)-1L) && PyErr_Occurred())) __PYX_ERR(0, 739, __pyx_L1_error)
  __pyx_v_stream_ptr = __pyx_t_8;

  /* "cupy/cuda/memory.pyx":741
 *         stream_ptr = stream_module.get_current_stream_ptr()
 *         if (
 *             not runtime._is_hip_environment             # <<<<<<<<<<<<<<
 *             and runtime.streamIsCapturing(stream_ptr)
 *         ):
*/
  __pyx_t_10 = (!__pyx_v_13cupy_backends_4cuda_3api_7runtime__is_hip_environment);
  if (__pyx_t_10) {
  } else {
    __pyx_t_9 = __pyx_t_10;
    goto __pyx_L4_bool_binop_done;
  }

  /* "cupy/cuda/memory.pyx":742
 *         if (
 *             not runtime._is_hip_environment
 *             and runtime.streamIsCapturing(stream_ptr)             # <<<<<<<<<<<<<<
 *         ):
 *             raise RuntimeError(
*/
  __pyx_t_10 = __pyx_f_13cupy_backends_4cuda_3api_7runtime_streamIsCapturing(__pyx_v_stream_ptr, 0); if (unlikely(PyErr_Occurred())) __PYX_ERR(0, 742, __pyx_L1_error)
  __pyx_t_9 = __pyx_t_10;
  __pyx_L4_bool_binop_done:;

  /* "cupy/cuda/memory.pyx":740
 *         """
 *         stream_ptr = stream_module.get_current_stream_ptr()
 *         if (             # <<<<<<<<<<<<<<
 *             not runtime._is_hip_environment
 *             and runtime.streamIsCapturing(stream_ptr)
*/
  if (unlikely(__pyx_t_9)) {

    /* "cupy/cuda/memory.pyx":744
 *             and runtime.streamIsCapturing(stream_ptr)
 *         ):
 *             raise RuntimeError(             # <<<<<<<<<<<<<<
 *                 'the current stream is capturing, so synchronous API calls '
 *                 'are disallowed')
*/
    __pyx_t_2 = NULL;
    __pyx_t_7 = 1;
    {
      PyObject *__pyx_callargs[2] = {__pyx_t_2, __pyx_mstate_global->__pyx_kp_u_the_current_stream_is_capturing};
      __pyx_t_1 = __Pyx_PyObject_FastCall((PyObject*)(((PyTypeObject*)PyExc_RuntimeError)), __pyx_callargs+__pyx_t_7, (2-__pyx_t_7) | (__pyx_t_7*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
      __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
      if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 744, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
    }
    __Pyx_Raise(__pyx_t_1, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __PYX_ERR(0, 744, __pyx_L1_error)

    /* "cupy/cuda/memory.pyx":740
 *         """
 *         stream_ptr = stream_module.get_current_stream_ptr()
 *         if (             # <<<<<<<<<<<<<<
 *             not runtime._is_hip_environment
 *             and runtime.streamIsCapturing(stream_ptr)
*/
  }

  /* "cupy/cuda/memory.pyx":747
 *                 'the current stream is capturing, so synchronous API calls '
 *                 'are disallowed')
 *         if size > 0:             # <<<<<<<<<<<<<<
 *             runtime.memset(self.ptr, value, size)
 * 
*/
  __pyx_t_9 = (__pyx_v_size > 0);
  if (__pyx_t_9) {

    /* "cupy/cuda/memory.pyx":748
 *                 'are disallowed')
 *         if size > 0:
 *             runtime.memset(self.ptr, value, size)             # <<<<<<<<<<<<<<
 * 
 *     cpdef memset_async(self, int value, size_t size, stream=None):
*/
    __pyx_t_1 = __pyx_f_13cupy_backends_4cuda_3api_7runtime_memset(__pyx_v_self->ptr, __pyx_v_value, __pyx_v_size, 0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 748, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

    /* "cupy/cuda/memory.pyx":747
 *                 'the current stream is capturing, so synchronous API calls '
 *                 'are disallowed')
 *         if size > 0:             # <<<<<<<<<<<<<<
 *             runtime.memset(self.ptr, value, size)
 * 
*/
  }

  /* "cupy/cuda/memory.pyx":725
 *                                 runtime.memcpyDeviceToHost, stream_ptr)
 * 
 *     cpdef memset(self, int value, size_t size):             # <<<<<<<<<<<<<<
 *         """Fills a memory sequence by constant byte value.
 * 
*/

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_AddTraceback("cupy.cuda.memory.MemoryPointer.memset", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_pw_4cupy_4cuda_6memory_13MemoryPointer_31memset(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
PyDoc_STRVAR(__pyx_doc_4cupy_4cuda_6memory_13MemoryPointer_30memset, "MemoryPointer.memset(self, int value, size_t size)\n\nFills a memory sequence by constant byte value.\n\nArgs:\n    value (int): Value to fill.\n    size (int): Size of the sequence in bytes.\n\n.. warning::\n\n    This function always uses the legacy default stream and does not\n    honor the current stream. Use `memset_async` instead\n    if you are using streams in your code, or have PTDS enabled.");
static PyObject *__pyx_pw_4cupy_4cuda_6memory_13MemoryPointer_31memset(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
) {
  int __pyx_v_value;
  size_t __pyx_v_size;
  #if !CYTHON_METH_FASTCALL
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  #endif
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject* values[2] = {0,0};
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("memset (wrapper)", 0);
  #if !CYTHON_METH_FASTCALL
  #if CYTHON_ASSUME_SAFE_SIZE
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return NULL;
  #endif
  #endif
  __pyx_kwvalues = __Pyx_KwValues_FASTCALL(__pyx_args, __pyx_nargs);
  {
    PyObject ** const __pyx_pyargnames[] = {&__pyx_mstate_global->__pyx_n_u_value,&__pyx_mstate_global->__pyx_n_u_size,0};
    const Py_ssize_t __pyx_kwds_len = (__pyx_kwds) ? __Pyx_NumKwargs_FASTCALL(__pyx_kwds) : 0;
    if (unlikely(__pyx_kwds_len < 0)) __PYX_ERR(0, 725, __pyx_L3_error)
    if (__pyx_kwds_len > 0) {
      switch (__pyx_nargs) {
        case  2:
        values[1] = __Pyx_ArgRef_FASTCALL(__pyx_args, 1);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[1])) __PYX_ERR(0, 725, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  1:
        values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(0, 725, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      const Py_ssize_t kwd_pos_args = __pyx_nargs;
      if (__Pyx_ParseKeywords(__pyx_kwds, __pyx_kwvalues, __pyx_pyargnames, 0, values, kwd_pos_args, __pyx_kwds_len, "memset", 0) < (0)) __PYX_ERR(0, 725, __pyx_L3_error)
      for (Py_ssize_t i = __pyx_nargs; i < 2; i++) {
        if (unlikely(!values[i])) { __Pyx_RaiseArgtupleInvalid("memset", 1, 2, 2, i); __PYX_ERR(0, 725, __pyx_L3_error) }
      }
    } else if (unlikely(__pyx_nargs != 2)) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
      if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(0, 725, __pyx_L3_error)
      values[1] = __Pyx_ArgRef_FASTCALL(__pyx_args, 1);
      if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[1])) __PYX_ERR(0, 725, __pyx_L3_error)
    }
    __pyx_v_value = __Pyx_PyLong_As_int(values[0]); if (unlikely((__pyx_v_value == (int)-1) && PyErr_Occurred())) __PYX_ERR(0, 725, __pyx_L3_error)
    __pyx_v_size = __Pyx_PyLong_As_size_t(values[1]); if (unlikely((__pyx_v_size == (size_t)-1) && PyErr_Occurred())) __PYX_ERR(0, 725, __pyx_L3_error)
  }
  goto __pyx_L6_skip;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("memset", 1, 2, 2, __pyx_nargs); __PYX_ERR(0, 725, __pyx_L3_error)
  __pyx_L6_skip:;
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_AddTraceback("cupy.cuda.memory.MemoryPointer.memset", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_4cupy_4cuda_6memory_13MemoryPointer_30memset(((struct __pyx_obj_4cupy_4cuda_6memory_MemoryPointer *)__pyx_v_self), __pyx_v_value, __pyx_v_size);

  /* function exit code */
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4cupy_4cuda_6memory_13MemoryPointer_30memset(struct __pyx_obj_4cupy_4cuda_6memory_MemoryPointer *__pyx_v_self, int __pyx_v_value, size_t __pyx_v_size) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("memset", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __pyx_f_4cupy_4cuda_6memory_13MemoryPointer_memset(__pyx_v_self, __pyx_v_value, __pyx_v_size, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 725, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("cupy.cuda.memory.MemoryPointer.memset", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "cupy/cuda/memory.pyx":750
 *             runtime.memset(self.ptr, value, size)
 * 
 *     cpdef memset_async(self, int value, size_t size, stream=None):             # <<<<<<<<<<<<<<
 *         """Fills a memory sequence by constant byte value asynchronously.
 * 
*/

static PyObject *__pyx_pw_4cupy_4cuda_6memory_13MemoryPointer_33memset_async(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
static PyObject *__pyx_f_4cupy_4cuda_6memory_13MemoryPointer_memset_async(struct __pyx_obj_4cupy_4cuda_6memory_MemoryPointer *__pyx_v_self, int __pyx_v_value, size_t __pyx_v_size, int __pyx_skip_dispatch, struct __pyx_opt_args_4cupy_4cuda_6memory_13MemoryPointer_memset_async *__pyx_optional_args) {
  PyObject *__pyx_v_stream = ((PyObject *)Py_None);
  PyObject *__pyx_v_stream_ptr = NULL;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  PyObject *__pyx_t_6 = NULL;
  size_t __pyx_t_7;
  int __pyx_t_8;
  intptr_t __pyx_t_9;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("memset_async", 0);
  if (__pyx_optional_args) {
    if (__pyx_optional_args->__pyx_n > 0) {
      __pyx_v_stream = __pyx_optional_args->stream;
    }
  }
  /* Check if called by wrapper */
  if (unlikely(__pyx_skip_dispatch)) ;
  /* Check if overridden in Python */
  else if (
  #if !CYTHON_USE_TYPE_SLOTS
  unlikely(Py_TYPE(((PyObject *)__pyx_v_self)) != __pyx_mstate_global->__pyx_ptype_4cupy_4cuda_6memory_MemoryPointer &&
  __Pyx_PyType_HasFeature(Py_TYPE(((PyObject *)__pyx_v_self)), Py_TPFLAGS_HAVE_GC))
  #else
  unlikely(Py_TYPE(((PyObject *)__pyx_v_self))->tp_dictoffset != 0 || __Pyx_PyType_HasFeature(Py_TYPE(((PyObject *)__pyx_v_self)), (Py_TPFLAGS_IS_ABSTRACT | Py_TPFLAGS_HEAPTYPE)))
  #endif
  ) {
    #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
    static PY_UINT64_T __pyx_tp_dict_version = __PYX_DICT_VERSION_INIT, __pyx_obj_dict_version = __PYX_DICT_VERSION_INIT;
    if (unlikely(!__Pyx_object_dict_version_matches(((PyObject *)__pyx_v_self), __pyx_tp_dict_version, __pyx_obj_dict_version))) {
      PY_UINT64_T __pyx_typedict_guard = __Pyx_get_tp_dict_version(((PyObject *)__pyx_v_self));
      #endif
      __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_mstate_global->__pyx_n_u_memset_async); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 750, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      if (!__Pyx_IsSameCFunction(__pyx_t_1, (void(*)(void)) __pyx_pw_4cupy_4cuda_6memory_13MemoryPointer_33memset_async)) {
        __Pyx_XDECREF(__pyx_r);
        __pyx_t_3 = NULL;
        __Pyx_INCREF(__pyx_t_1);
        __pyx_t_4 = __pyx_t_1; 
        __pyx_t_5 = __Pyx_PyLong_From_int(__pyx_v_value); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 750, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_5);
        __pyx_t_6 = __Pyx_PyLong_FromSize_t(__pyx_v_size); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 750, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_6);
        __pyx_t_7 = 1;
        #if CYTHON_UNPACK_METHODS
        if (unlikely(PyMethod_Check(__pyx_t_4))) {
          __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_4);
          assert(__pyx_t_3);
          PyObject* __pyx__function = PyMethod_GET_FUNCTION(__pyx_t_4);
          __Pyx_INCREF(__pyx_t_3);
          __Pyx_INCREF(__pyx__function);
          __Pyx_DECREF_SET(__pyx_t_4, __pyx__function);
          __pyx_t_7 = 0;
        }
        #endif
        {
          PyObject *__pyx_callargs[4] = {__pyx_t_3, __pyx_t_5, __pyx_t_6, __pyx_v_stream};
          __pyx_t_2 = __Pyx_PyObject_FastCall((PyObject*)__pyx_t_4, __pyx_callargs+__pyx_t_7, (4-__pyx_t_7) | (__pyx_t_7*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
          __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
          __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
          __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
          __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
          if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 750, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_2);
        }
        __pyx_r = __pyx_t_2;
        __pyx_t_2 = 0;
        __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
        goto __pyx_L0;
      }
      #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
      __pyx_tp_dict_version = __Pyx_get_tp_dict_version(((PyObject *)__pyx_v_self));
      __pyx_obj_dict_version = __Pyx_get_object_dict_version(((PyObject *)__pyx_v_self));
      if (unlikely(__pyx_typedict_guard != __pyx_tp_dict_version)) {
        __pyx_tp_dict_version = __pyx_obj_dict_version = __PYX_DICT_VERSION_INIT;
      }
      #endif
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
    }
    #endif
  }

  /* "cupy/cuda/memory.pyx":760
 * 
 *         """
 *         if stream is None:             # <<<<<<<<<<<<<<
 *             stream_ptr = stream_module.get_current_stream_ptr()
 *         else:
*/
  __pyx_t_8 = (__pyx_v_stream == Py_None);
  if (__pyx_t_8) {

    /* "cupy/cuda/memory.pyx":761
 *         """
 *         if stream is None:
 *             stream_ptr = stream_module.get_current_stream_ptr()             # <<<<<<<<<<<<<<
 *         else:
 *             stream_ptr = stream.ptr
*/
    __pyx_t_9 = __pyx_f_4cupy_4cuda_6stream_get_current_stream_ptr(); if (unlikely(__pyx_t_9 == ((intptr_t)-1L) && PyErr_Occurred())) __PYX_ERR(0, 761, __pyx_L1_error)
    __pyx_t_1 = PyLong_FromSsize_t(__pyx_t_9); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 761, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_v_stream_ptr = __pyx_t_1;
    __pyx_t_1 = 0;

    /* "cupy/cuda/memory.pyx":760
 * 
 *         """
 *         if stream is None:             # <<<<<<<<<<<<<<
 *             stream_ptr = stream_module.get_current_stream_ptr()
 *         else:
*/
    goto __pyx_L3;
  }

  /* "cupy/cuda/memory.pyx":763
 *             stream_ptr = stream_module.get_current_stream_ptr()
 *         else:
 *             stream_ptr = stream.ptr             # <<<<<<<<<<<<<<
 *         if size > 0:
 *             runtime.memsetAsync(self.ptr, value, size, stream_ptr)
*/
  /*else*/ {
    __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_v_stream, __pyx_mstate_global->__pyx_n_u_ptr); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 763, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_v_stream_ptr = __pyx_t_1;
    __pyx_t_1 = 0;
  }
  __pyx_L3:;

  /* "cupy/cuda/memory.pyx":764
 *         else:
 *             stream_ptr = stream.ptr
 *         if size > 0:             # <<<<<<<<<<<<<<
 *             runtime.memsetAsync(self.ptr, value, size, stream_ptr)
 * 
*/
  __pyx_t_8 = (__pyx_v_size > 0);
  if (__pyx_t_8) {

    /* "cupy/cuda/memory.pyx":765
 *             stream_ptr = stream.ptr
 *         if size > 0:
 *             runtime.memsetAsync(self.ptr, value, size, stream_ptr)             # <<<<<<<<<<<<<<
 * 
 * 
*/
    __pyx_t_9 = PyLong_AsSsize_t(__pyx_v_stream_ptr); if (unlikely((__pyx_t_9 == ((intptr_t)-1)) && PyErr_Occurred())) __PYX_ERR(0, 765, __pyx_L1_error)
    __pyx_t_1 = __pyx_f_13cupy_backends_4cuda_3api_7runtime_memsetAsync(__pyx_v_self->ptr, __pyx_v_value, __pyx_v_size, __pyx_t_9, 0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 765, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

    /* "cupy/cuda/memory.pyx":764
 *         else:
 *             stream_ptr = stream.ptr
 *         if size > 0:             # <<<<<<<<<<<<<<
 *             runtime.memsetAsync(self.ptr, value, size, stream_ptr)
 * 
*/
  }

  /* "cupy/cuda/memory.pyx":750
 *             runtime.memset(self.ptr, value, size)
 * 
 *     cpdef memset_async(self, int value, size_t size, stream=None):             # <<<<<<<<<<<<<<
 *         """Fills a memory sequence by constant byte value asynchronously.
 * 
*/

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_AddTraceback("cupy.cuda.memory.MemoryPointer.memset_async", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_stream_ptr);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_pw_4cupy_4cuda_6memory_13MemoryPointer_33memset_async(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
PyDoc_STRVAR(__pyx_doc_4cupy_4cuda_6memory_13MemoryPointer_32memset_async, "MemoryPointer.memset_async(self, int value, size_t size, stream=None)\n\nFills a memory sequence by constant byte value asynchronously.\n\nArgs:\n    value (int): Value to fill.\n    size (int): Size of the sequence in bytes.\n    stream (cupy.cuda.Stream): CUDA stream.\n        The default uses CUDA stream of the current context.");
static PyObject *__pyx_pw_4cupy_4cuda_6memory_13MemoryPointer_33memset_async(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
) {
  int __pyx_v_value;
  size_t __pyx_v_size;
  PyObject *__pyx_v_stream = 0;
  #if !CYTHON_METH_FASTCALL
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  #endif
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject* values[3] = {0,0,0};
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("memset_async (wrapper)", 0);
  #if !CYTHON_METH_FASTCALL
  #if CYTHON_ASSUME_SAFE_SIZE
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return NULL;
  #endif
  #endif
  __pyx_kwvalues = __Pyx_KwValues_FASTCALL(__pyx_args, __pyx_nargs);
  {
    PyObject ** const __pyx_pyargnames[] = {&__pyx_mstate_global->__pyx_n_u_value,&__pyx_mstate_global->__pyx_n_u_size,&__pyx_mstate_global->__pyx_n_u_stream,0};
    const Py_ssize_t __pyx_kwds_len = (__pyx_kwds) ? __Pyx_NumKwargs_FASTCALL(__pyx_kwds) : 0;
    if (unlikely(__pyx_kwds_len < 0)) __PYX_ERR(0, 750, __pyx_L3_error)
    if (__pyx_kwds_len > 0) {
      switch (__pyx_nargs) {
        case  3:
        values[2] = __Pyx_ArgRef_FASTCALL(__pyx_args, 2);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[2])) __PYX_ERR(0, 750, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  2:
        values[1] = __Pyx_ArgRef_FASTCALL(__pyx_args, 1);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[1])) __PYX_ERR(0, 750, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  1:
        values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(0, 750, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      const Py_ssize_t kwd_pos_args = __pyx_nargs;
      if (__Pyx_ParseKeywords(__pyx_kwds, __pyx_kwvalues, __pyx_pyargnames, 0, values, kwd_pos_args, __pyx_kwds_len, "memset_async", 0) < (0)) __PYX_ERR(0, 750, __pyx_L3_error)
      if (!values[2]) values[2] = __Pyx_NewRef(((PyObject *)Py_None));
      for (Py_ssize_t i = __pyx_nargs; i < 2; i++) {
        if (unlikely(!values[i])) { __Pyx_RaiseArgtupleInvalid("memset_async", 0, 2, 3, i); __PYX_ERR(0, 750, __pyx_L3_error) }
      }
    } else {
      switch (__pyx_nargs) {
        case  3:
        values[2] = __Pyx_ArgRef_FASTCALL(__pyx_args, 2);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[2])) __PYX_ERR(0, 750, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  2:
        values[1] = __Pyx_ArgRef_FASTCALL(__pyx_args, 1);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[1])) __PYX_ERR(0, 750, __pyx_L3_error)
        values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(0, 750, __pyx_L3_error)
        break;
        default: goto __pyx_L5_argtuple_error;
      }
      if (!values[2]) values[2] = __Pyx_NewRef(((PyObject *)Py_None));
    }
    __pyx_v_value = __Pyx_PyLong_As_int(values[0]); if (unlikely((__pyx_v_value == (int)-1) && PyErr_Occurred())) __PYX_ERR(0, 750, __pyx_L3_error)
    __pyx_v_size = __Pyx_PyLong_As_size_t(values[1]); if (unlikely((__pyx_v_size == (size_t)-1) && PyErr_Occurred())) __PYX_ERR(0, 750, __pyx_L3_error)
    __pyx_v_stream = values[2];
  }
  goto __pyx_L6_skip;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("memset_async", 0, 2, 3, __pyx_nargs); __PYX_ERR(0, 750, __pyx_L3_error)
  __pyx_L6_skip:;
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_AddTraceback("cupy.cuda.memory.MemoryPointer.memset_async", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_4cupy_4cuda_6memory_13MemoryPointer_32memset_async(((struct __pyx_obj_4cupy_4cuda_6memory_MemoryPointer *)__pyx_v_self), __pyx_v_value, __pyx_v_size, __pyx_v_stream);

  /* function exit code */
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4cupy_4cuda_6memory_13MemoryPointer_32memset_async(struct __pyx_obj_4cupy_4cuda_6memory_MemoryPointer *__pyx_v_self, int __pyx_v_value, size_t __pyx_v_size, PyObject *__pyx_v_stream) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  struct __pyx_opt_args_4cupy_4cuda_6memory_13MemoryPointer_memset_async __pyx_t_2;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("memset_async", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_2.__pyx_n = 1;
  __pyx_t_2.stream = __pyx_v_stream;
  __pyx_t_1 = __pyx_vtabptr_4cupy_4cuda_6memory_MemoryPointer->memset_async(__pyx_v_self, __pyx_v_value, __pyx_v_size, 1, &__pyx_t_2); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 750, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("cupy.cuda.memory.MemoryPointer.memset_async", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "cupy/cuda/memory.pxd":32
 * 
 *     cdef:
 *         readonly intptr_t ptr             # <<<<<<<<<<<<<<
 *         readonly int device_id
 *         readonly BaseMemory mem
*/

/* Python wrapper */
static PyObject *__pyx_pw_4cupy_4cuda_6memory_13MemoryPointer_3ptr_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_4cupy_4cuda_6memory_13MemoryPointer_3ptr_1__get__(PyObject *__pyx_v_self) {
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_kwvalues = __Pyx_KwValues_VARARGS(__pyx_args, __pyx_nargs);
  __pyx_r = __pyx_pf_4cupy_4cuda_6memory_13MemoryPointer_3ptr___get__(((struct __pyx_obj_4cupy_4cuda_6memory_MemoryPointer *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4cupy_4cuda_6memory_13MemoryPointer_3ptr___get__(struct __pyx_obj_4cupy_4cuda_6memory_MemoryPointer *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__get__", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = PyLong_FromSsize_t(__pyx_v_self->ptr); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 32, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("cupy.cuda.memory.MemoryPointer.ptr.__get__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "cupy/cuda/memory.pxd":33
 *     cdef:
 *         readonly intptr_t ptr
 *         readonly int device_id             # <<<<<<<<<<<<<<
 *         readonly BaseMemory mem
 * 
*/

/* Python wrapper */
static PyObject *__pyx_pw_4cupy_4cuda_6memory_13MemoryPointer_9device_id_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_4cupy_4cuda_6memory_13MemoryPointer_9device_id_1__get__(PyObject *__pyx_v_self) {
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_kwvalues = __Pyx_KwValues_VARARGS(__pyx_args, __pyx_nargs);
  __pyx_r = __pyx_pf_4cupy_4cuda_6memory_13MemoryPointer_9device_id___get__(((struct __pyx_obj_4cupy_4cuda_6memory_MemoryPointer *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4cupy_4cuda_6memory_13MemoryPointer_9device_id___get__(struct __pyx_obj_4cupy_4cuda_6memory_MemoryPointer *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__get__", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __Pyx_PyLong_From_int(__pyx_v_self->device_id); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 33, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("cupy.cuda.memory.MemoryPointer.device_id.__get__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "cupy/cuda/memory.pxd":34
 *         readonly intptr_t ptr
 *         readonly int device_id
 *         readonly BaseMemory mem             # <<<<<<<<<<<<<<
 * 
 *     cdef _init(self, BaseMemory mem, ptrdiff_t offset)
*/

/* Python wrapper */
static PyObject *__pyx_pw_4cupy_4cuda_6memory_13MemoryPointer_3mem_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_4cupy_4cuda_6memory_13MemoryPointer_3mem_1__get__(PyObject *__pyx_v_self) {
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_kwvalues = __Pyx_KwValues_VARARGS(__pyx_args, __pyx_nargs);
  __pyx_r = __pyx_pf_4cupy_4cuda_6memory_13MemoryPointer_3mem___get__(((struct __pyx_obj_4cupy_4cuda_6memory_MemoryPointer *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4cupy_4cuda_6memory_13MemoryPointer_3mem___get__(struct __pyx_obj_4cupy_4cuda_6memory_MemoryPointer *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__", 0);
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF((PyObject *)__pyx_v_self->mem);
  __pyx_r = ((PyObject *)__pyx_v_self->mem);
  goto __pyx_L0;

  /* function exit code */
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     cdef tuple state
 *     cdef object _dict
*/

/* Python wrapper */
static PyObject *__pyx_pw_4cupy_4cuda_6memory_13MemoryPointer_35__reduce_cython__(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
PyDoc_STRVAR(__pyx_doc_4cupy_4cuda_6memory_13MemoryPointer_34__reduce_cython__, "MemoryPointer.__reduce_cython__(self)");
static PyObject *__pyx_pw_4cupy_4cuda_6memory_13MemoryPointer_35__reduce_cython__(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
) {
  #if !CYTHON_METH_FASTCALL
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  #endif
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__reduce_cython__ (wrapper)", 0);
  #if !CYTHON_METH_FASTCALL
  #if CYTHON_ASSUME_SAFE_SIZE
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return NULL;
  #endif
  #endif
  __pyx_kwvalues = __Pyx_KwValues_FASTCALL(__pyx_args, __pyx_nargs);
  if (unlikely(__pyx_nargs > 0)) { __Pyx_RaiseArgtupleInvalid("__reduce_cython__", 1, 0, 0, __pyx_nargs); return NULL; }
  const Py_ssize_t __pyx_kwds_len = unlikely(__pyx_kwds) ? __Pyx_NumKwargs_FASTCALL(__pyx_kwds) : 0;
  if (unlikely(__pyx_kwds_len < 0)) return NULL;
  if (unlikely(__pyx_kwds_len > 0)) {__Pyx_RejectKeywords("__reduce_cython__", __pyx_kwds); return NULL;}
  __pyx_r = __pyx_pf_4cupy_4cuda_6memory_13MemoryPointer_34__reduce_cython__(((struct __pyx_obj_4cupy_4cuda_6memory_MemoryPointer *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4cupy_4cuda_6memory_13MemoryPointer_34__reduce_cython__(struct __pyx_obj_4cupy_4cuda_6memory_MemoryPointer *__pyx_v_self) {
  PyObject *__pyx_v_state = 0;
  PyObject *__pyx_v__dict = 0;
  int __pyx_v_use_setstate;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  int __pyx_t_4;
  int __pyx_t_5;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__reduce_cython__", 0);

  /* "(tree fragment)":5
 *     cdef object _dict
 *     cdef bint use_setstate
 *     state = (self.device_id, self.mem, self.ptr)             # <<<<<<<<<<<<<<
 *     _dict = getattr(self, '__dict__', None)
 *     if _dict is not None and _dict:
*/
  __pyx_t_1 = __Pyx_PyLong_From_int(__pyx_v_self->device_id); if (unlikely(!__pyx_t_1)) __PYX_ERR(3, 5, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = PyLong_FromSsize_t(__pyx_v_self->ptr); if (unlikely(!__pyx_t_2)) __PYX_ERR(3, 5, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = PyTuple_New(3); if (unlikely(!__pyx_t_3)) __PYX_ERR(3, 5, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_GIVEREF(__pyx_t_1);
  if (__Pyx_PyTuple_SET_ITEM(__pyx_t_3, 0, __pyx_t_1) != (0)) __PYX_ERR(3, 5, __pyx_L1_error);
  __Pyx_INCREF((PyObject *)__pyx_v_self->mem);
  __Pyx_GIVEREF((PyObject *)__pyx_v_self->mem);
  if (__Pyx_PyTuple_SET_ITEM(__pyx_t_3, 1, ((PyObject *)__pyx_v_self->mem)) != (0)) __PYX_ERR(3, 5, __pyx_L1_error);
  __Pyx_GIVEREF(__pyx_t_2);
  if (__Pyx_PyTuple_SET_ITEM(__pyx_t_3, 2, __pyx_t_2) != (0)) __PYX_ERR(3, 5, __pyx_L1_error);
  __pyx_t_1 = 0;
  __pyx_t_2 = 0;
  __pyx_v_state = ((PyObject*)__pyx_t_3);
  __pyx_t_3 = 0;

  /* "(tree fragment)":6
 *     cdef bint use_setstate
 *     state = (self.device_id, self.mem, self.ptr)
 *     _dict = getattr(self, '__dict__', None)             # <<<<<<<<<<<<<<
 *     if _dict is not None and _dict:
 *         state += (_dict,)
*/
  __pyx_t_3 = __Pyx_GetAttr3(((PyObject *)__pyx_v_self), __pyx_mstate_global->__pyx_n_u_dict, Py_None); if (unlikely(!__pyx_t_3)) __PYX_ERR(3, 6, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_v__dict = __pyx_t_3;
  __pyx_t_3 = 0;

  /* "(tree fragment)":7
 *     state = (self.device_id, self.mem, self.ptr)
 *     _dict = getattr(self, '__dict__', None)
 *     if _dict is not None and _dict:             # <<<<<<<<<<<<<<
 *         state += (_dict,)
 *         use_setstate = True
*/
  __pyx_t_5 = (__pyx_v__dict != Py_None);
  if (__pyx_t_5) {
  } else {
    __pyx_t_4 = __pyx_t_5;
    goto __pyx_L4_bool_binop_done;
  }
  __pyx_t_5 = __Pyx_PyObject_IsTrue(__pyx_v__dict); if (unlikely((__pyx_t_5 < 0))) __PYX_ERR(3, 7, __pyx_L1_error)
  __pyx_t_4 = __pyx_t_5;
  __pyx_L4_bool_binop_done:;
  if (__pyx_t_4) {

    /* "(tree fragment)":8
 *     _dict = getattr(self, '__dict__', None)
 *     if _dict is not None and _dict:
 *         state += (_dict,)             # <<<<<<<<<<<<<<
 *         use_setstate = True
 *     else:
*/
    __pyx_t_3 = PyTuple_New(1); if (unlikely(!__pyx_t_3)) __PYX_ERR(3, 8, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_INCREF(__pyx_v__dict);
    __Pyx_GIVEREF(__pyx_v__dict);
    if (__Pyx_PyTuple_SET_ITEM(__pyx_t_3, 0, __pyx_v__dict) != (0)) __PYX_ERR(3, 8, __pyx_L1_error);
    __pyx_t_2 = PyNumber_InPlaceAdd(__pyx_v_state, __pyx_t_3); if (unlikely(!__pyx_t_2)) __PYX_ERR(3, 8, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __Pyx_DECREF_SET(__pyx_v_state, ((PyObject*)__pyx_t_2));
    __pyx_t_2 = 0;

    /* "(tree fragment)":9
 *     if _dict is not None and _dict:
 *         state += (_dict,)
 *         use_setstate = True             # <<<<<<<<<<<<<<
 *     else:
 *         use_setstate = self.mem is not None
*/
    __pyx_v_use_setstate = 1;

    /* "(tree fragment)":7
 *     state = (self.device_id, self.mem, self.ptr)
 *     _dict = getattr(self, '__dict__', None)
 *     if _dict is not None and _dict:             # <<<<<<<<<<<<<<
 *         state += (_dict,)
 *         use_setstate = True
*/
    goto __pyx_L3;
  }

  /* "(tree fragment)":11
 *         use_setstate = True
 *     else:
 *         use_setstate = self.mem is not None             # <<<<<<<<<<<<<<
 *     if use_setstate:
 *         return __pyx_unpickle_MemoryPointer, (type(self), 0x72cbb23, None), state
*/
  /*else*/ {
    __pyx_t_4 = (((PyObject *)__pyx_v_self->mem) != Py_None);
    __pyx_v_use_setstate = __pyx_t_4;
  }
  __pyx_L3:;

  /* "(tree fragment)":12
 *     else:
 *         use_setstate = self.mem is not None
 *     if use_setstate:             # <<<<<<<<<<<<<<
 *         return __pyx_unpickle_MemoryPointer, (type(self), 0x72cbb23, None), state
 *     else:
*/
  if (__pyx_v_use_setstate) {

    /* "(tree fragment)":13
 *         use_setstate = self.mem is not None
 *     if use_setstate:
 *         return __pyx_unpickle_MemoryPointer, (type(self), 0x72cbb23, None), state             # <<<<<<<<<<<<<<
 *     else:
 *         return __pyx_unpickle_MemoryPointer, (type(self), 0x72cbb23, state)
*/
    __Pyx_XDECREF(__pyx_r);
    __Pyx_GetModuleGlobalName(__pyx_t_2, __pyx_mstate_global->__pyx_n_u_pyx_unpickle_MemoryPointer); if (unlikely(!__pyx_t_2)) __PYX_ERR(3, 13, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_3 = PyTuple_New(3); if (unlikely(!__pyx_t_3)) __PYX_ERR(3, 13, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_INCREF(((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    __Pyx_GIVEREF(((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    if (__Pyx_PyTuple_SET_ITEM(__pyx_t_3, 0, ((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self)))) != (0)) __PYX_ERR(3, 13, __pyx_L1_error);
    __Pyx_INCREF(__pyx_mstate_global->__pyx_int_120372003);
    __Pyx_GIVEREF(__pyx_mstate_global->__pyx_int_120372003);
    if (__Pyx_PyTuple_SET_ITEM(__pyx_t_3, 1, __pyx_mstate_global->__pyx_int_120372003) != (0)) __PYX_ERR(3, 13, __pyx_L1_error);
    __Pyx_INCREF(Py_None);
    __Pyx_GIVEREF(Py_None);
    if (__Pyx_PyTuple_SET_ITEM(__pyx_t_3, 2, Py_None) != (0)) __PYX_ERR(3, 13, __pyx_L1_error);
    __pyx_t_1 = PyTuple_New(3); if (unlikely(!__pyx_t_1)) __PYX_ERR(3, 13, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_GIVEREF(__pyx_t_2);
    if (__Pyx_PyTuple_SET_ITEM(__pyx_t_1, 0, __pyx_t_2) != (0)) __PYX_ERR(3, 13, __pyx_L1_error);
    __Pyx_GIVEREF(__pyx_t_3);
    if (__Pyx_PyTuple_SET_ITEM(__pyx_t_1, 1, __pyx_t_3) != (0)) __PYX_ERR(3, 13, __pyx_L1_error);
    __Pyx_INCREF(__pyx_v_state);
    __Pyx_GIVEREF(__pyx_v_state);
    if (__Pyx_PyTuple_SET_ITEM(__pyx_t_1, 2, __pyx_v_state) != (0)) __PYX_ERR(3, 13, __pyx_L1_error);
    __pyx_t_2 = 0;
    __pyx_t_3 = 0;
    __pyx_r = __pyx_t_1;
    __pyx_t_1 = 0;
    goto __pyx_L0;

    /* "(tree fragment)":12
 *     else:
 *         use_setstate = self.mem is not None
 *     if use_setstate:             # <<<<<<<<<<<<<<
 *         return __pyx_unpickle_MemoryPointer, (type(self), 0x72cbb23, None), state
 *     else:
*/
  }

  /* "(tree fragment)":15
 *         return __pyx_unpickle_MemoryPointer, (type(self), 0x72cbb23, None), state
 *     else:
 *         return __pyx_unpickle_MemoryPointer, (type(self), 0x72cbb23, state)             # <<<<<<<<<<<<<<
 * def __setstate_cython__(self, __pyx_state):
 *     __pyx_unpickle_MemoryPointer__set_state(self, __pyx_state)
*/
  /*else*/ {
    __Pyx_XDECREF(__pyx_r);
    __Pyx_GetModuleGlobalName(__pyx_t_1, __pyx_mstate_global->__pyx_n_u_pyx_unpickle_MemoryPointer); if (unlikely(!__pyx_t_1)) __PYX_ERR(3, 15, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_t_3 = PyTuple_New(3); if (unlikely(!__pyx_t_3)) __PYX_ERR(3, 15, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_INCREF(((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    __Pyx_GIVEREF(((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    if (__Pyx_PyTuple_SET_ITEM(__pyx_t_3, 0, ((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self)))) != (0)) __PYX_ERR(3, 15, __pyx_L1_error);
    __Pyx_INCREF(__pyx_mstate_global->__pyx_int_120372003);
    __Pyx_GIVEREF(__pyx_mstate_global->__pyx_int_120372003);
    if (__Pyx_PyTuple_SET_ITEM(__pyx_t_3, 1, __pyx_mstate_global->__pyx_int_120372003) != (0)) __PYX_ERR(3, 15, __pyx_L1_error);
    __Pyx_INCREF(__pyx_v_state);
    __Pyx_GIVEREF(__pyx_v_state);
    if (__Pyx_PyTuple_SET_ITEM(__pyx_t_3, 2, __pyx_v_state) != (0)) __PYX_ERR(3, 15, __pyx_L1_error);
    __pyx_t_2 = PyTuple_New(2); if (unlikely(!__pyx_t_2)) __PYX_ERR(3, 15, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_GIVEREF(__pyx_t_1);
    if (__Pyx_PyTuple_SET_ITEM(__pyx_t_2, 0, __pyx_t_1) != (0)) __PYX_ERR(3, 15, __pyx_L1_error);
    __Pyx_GIVEREF(__pyx_t_3);
    if (__Pyx_PyTuple_SET_ITEM(__pyx_t_2, 1, __pyx_t_3) != (0)) __PYX_ERR(3, 15, __pyx_L1_error);
    __pyx_t_1 = 0;
    __pyx_t_3 = 0;
    __pyx_r = __pyx_t_2;
    __pyx_t_2 = 0;
    goto __pyx_L0;
  }

  /* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     cdef tuple state
 *     cdef object _dict
*/

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_AddTraceback("cupy.cuda.memory.MemoryPointer.__reduce_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_state);
  __Pyx_XDECREF(__pyx_v__dict);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":16
 *     else:
 *         return __pyx_unpickle_MemoryPointer, (type(self), 0x72cbb23, state)
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     __pyx_unpickle_MemoryPointer__set_state(self, __pyx_state)
*/

/* Python wrapper */
static PyObject *__pyx_pw_4cupy_4cuda_6memory_13MemoryPointer_37__setstate_cython__(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
PyDoc_STRVAR(__pyx_doc_4cupy_4cuda_6memory_13MemoryPointer_36__setstate_cython__, "MemoryPointer.__setstate_cython__(self, __pyx_state)");
static PyObject *__pyx_pw_4cupy_4cuda_6memory_13MemoryPointer_37__setstate_cython__(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
) {
  PyObject *__pyx_v___pyx_state = 0;
  #if !CYTHON_METH_FASTCALL
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  #endif
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject* values[1] = {0};
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__setstate_cython__ (wrapper)", 0);
  #if !CYTHON_METH_FASTCALL
  #if CYTHON_ASSUME_SAFE_SIZE
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return NULL;
  #endif
  #endif
  __pyx_kwvalues = __Pyx_KwValues_FASTCALL(__pyx_args, __pyx_nargs);
  {
    PyObject ** const __pyx_pyargnames[] = {&__pyx_mstate_global->__pyx_n_u_pyx_state,0};
    const Py_ssize_t __pyx_kwds_len = (__pyx_kwds) ? __Pyx_NumKwargs_FASTCALL(__pyx_kwds) : 0;
    if (unlikely(__pyx_kwds_len < 0)) __PYX_ERR(3, 16, __pyx_L3_error)
    if (__pyx_kwds_len > 0) {
      switch (__pyx_nargs) {
        case  1:
        values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(3, 16, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      const Py_ssize_t kwd_pos_args = __pyx_nargs;
      if (__Pyx_ParseKeywords(__pyx_kwds, __pyx_kwvalues, __pyx_pyargnames, 0, values, kwd_pos_args, __pyx_kwds_len, "__setstate_cython__", 0) < (0)) __PYX_ERR(3, 16, __pyx_L3_error)
      for (Py_ssize_t i = __pyx_nargs; i < 1; i++) {
        if (unlikely(!values[i])) { __Pyx_RaiseArgtupleInvalid("__setstate_cython__", 1, 1, 1, i); __PYX_ERR(3, 16, __pyx_L3_error) }
      }
    } else if (unlikely(__pyx_nargs != 1)) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
      if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(3, 16, __pyx_L3_error)
    }
    __pyx_v___pyx_state = values[0];
  }
  goto __pyx_L6_skip;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("__setstate_cython__", 1, 1, 1, __pyx_nargs); __PYX_ERR(3, 16, __pyx_L3_error)
  __pyx_L6_skip:;
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_AddTraceback("cupy.cuda.memory.MemoryPointer.__setstate_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_4cupy_4cuda_6memory_13MemoryPointer_36__setstate_cython__(((struct __pyx_obj_4cupy_4cuda_6memory_MemoryPointer *)__pyx_v_self), __pyx_v___pyx_state);

  /* function exit code */
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4cupy_4cuda_6memory_13MemoryPointer_36__setstate_cython__(struct __pyx_obj_4cupy_4cuda_6memory_MemoryPointer *__pyx_v_self, PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__setstate_cython__", 0);

  /* "(tree fragment)":17
 *         return __pyx_unpickle_MemoryPointer, (type(self), 0x72cbb23, state)
 * def __setstate_cython__(self, __pyx_state):
 *     __pyx_unpickle_MemoryPointer__set_state(self, __pyx_state)             # <<<<<<<<<<<<<<
*/
  __pyx_t_1 = __pyx_v___pyx_state;
  __Pyx_INCREF(__pyx_t_1);
  if (!(likely(PyTuple_CheckExact(__pyx_t_1))||((__pyx_t_1) == Py_None) || __Pyx_RaiseUnexpectedTypeError("tuple", __pyx_t_1))) __PYX_ERR(3, 17, __pyx_L1_error)
  if (unlikely(__pyx_t_1 == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "cannot pass None into a C function argument that is declared 'not None'");
    __PYX_ERR(3, 17, __pyx_L1_error)
  }
  __pyx_t_2 = __pyx_f_4cupy_4cuda_6memory___pyx_unpickle_MemoryPointer__set_state(__pyx_v_self, ((PyObject*)__pyx_t_1)); if (unlikely(!__pyx_t_2)) __PYX_ERR(3, 17, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "(tree fragment)":16
 *     else:
 *         return __pyx_unpickle_MemoryPointer, (type(self), 0x72cbb23, state)
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     __pyx_unpickle_MemoryPointer__set_state(self, __pyx_state)
*/

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_AddTraceback("cupy.cuda.memory.MemoryPointer.__setstate_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "cupy/cuda/memory.pyx":769
 * 
 * # cpdef because unit-tested
 * cpdef MemoryPointer _malloc(size_t size):             # <<<<<<<<<<<<<<
 *     mem = Memory(size)
 *     return MemoryPointer(mem, 0)
*/

static PyObject *__pyx_pw_4cupy_4cuda_6memory_3_malloc(PyObject *__pyx_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
static struct __pyx_obj_4cupy_4cuda_6memory_MemoryPointer *__pyx_f_4cupy_4cuda_6memory__malloc(size_t __pyx_v_size, CYTHON_UNUSED int __pyx_skip_dispatch) {
  struct __pyx_obj_4cupy_4cuda_6memory_Memory *__pyx_v_mem = NULL;
  struct __pyx_obj_4cupy_4cuda_6memory_MemoryPointer *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  size_t __pyx_t_4;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("_malloc", 0);

  /* "cupy/cuda/memory.pyx":770
 * # cpdef because unit-tested
 * cpdef MemoryPointer _malloc(size_t size):
 *     mem = Memory(size)             # <<<<<<<<<<<<<<
 *     return MemoryPointer(mem, 0)
 * 
*/
  __pyx_t_2 = NULL;
  __pyx_t_3 = __Pyx_PyLong_FromSize_t(__pyx_v_size); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 770, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_4 = 1;
  {
    PyObject *__pyx_callargs[2] = {__pyx_t_2, __pyx_t_3};
    __pyx_t_1 = __Pyx_PyObject_FastCall((PyObject*)__pyx_mstate_global->__pyx_ptype_4cupy_4cuda_6memory_Memory, __pyx_callargs+__pyx_t_4, (2-__pyx_t_4) | (__pyx_t_4*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
    __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 770, __pyx_L1_error)
    __Pyx_GOTREF((PyObject *)__pyx_t_1);
  }
  __pyx_v_mem = ((struct __pyx_obj_4cupy_4cuda_6memory_Memory *)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "cupy/cuda/memory.pyx":771
 * cpdef MemoryPointer _malloc(size_t size):
 *     mem = Memory(size)
 *     return MemoryPointer(mem, 0)             # <<<<<<<<<<<<<<
 * 
 * 
*/
  __Pyx_XDECREF((PyObject *)__pyx_r);
  __pyx_t_3 = NULL;
  __pyx_t_4 = 1;
  {
    PyObject *__pyx_callargs[3] = {__pyx_t_3, ((PyObject *)__pyx_v_mem), __pyx_mstate_global->__pyx_int_0};
    __pyx_t_1 = __Pyx_PyObject_FastCall((PyObject*)__pyx_mstate_global->__pyx_ptype_4cupy_4cuda_6memory_MemoryPointer, __pyx_callargs+__pyx_t_4, (3-__pyx_t_4) | (__pyx_t_4*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
    __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
    if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 771, __pyx_L1_error)
    __Pyx_GOTREF((PyObject *)__pyx_t_1);
  }
  __pyx_r = ((struct __pyx_obj_4cupy_4cuda_6memory_MemoryPointer *)__pyx_t_1);
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "cupy/cuda/memory.pyx":769
 * 
 * # cpdef because unit-tested
 * cpdef MemoryPointer _malloc(size_t size):             # <<<<<<<<<<<<<<
 *     mem = Memory(size)
 *     return MemoryPointer(mem, 0)
*/

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_AddTraceback("cupy.cuda.memory._malloc", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XDECREF((PyObject *)__pyx_v_mem);
  __Pyx_XGIVEREF((PyObject *)__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_pw_4cupy_4cuda_6memory_3_malloc(PyObject *__pyx_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
PyDoc_STRVAR(__pyx_doc_4cupy_4cuda_6memory_2_malloc, "_malloc(size_t size) -> MemoryPointer");
static PyObject *__pyx_pw_4cupy_4cuda_6memory_3_malloc(PyObject *__pyx_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
) {
  size_t __pyx_v_size;
  #if !CYTHON_METH_FASTCALL
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  #endif
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject* values[1] = {0};
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("_malloc (wrapper)", 0);
  #if !CYTHON_METH_FASTCALL
  #if CYTHON_ASSUME_SAFE_SIZE
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return NULL;
  #endif
  #endif
  __pyx_kwvalues = __Pyx_KwValues_FASTCALL(__pyx_args, __pyx_nargs);
  {
    PyObject ** const __pyx_pyargnames[] = {&__pyx_mstate_global->__pyx_n_u_size,0};
    const Py_ssize_t __pyx_kwds_len = (__pyx_kwds) ? __Pyx_NumKwargs_FASTCALL(__pyx_kwds) : 0;
    if (unlikely(__pyx_kwds_len < 0)) __PYX_ERR(0, 769, __pyx_L3_error)
    if (__pyx_kwds_len > 0) {
      switch (__pyx_nargs) {
        case  1:
        values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(0, 769, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      const Py_ssize_t kwd_pos_args = __pyx_nargs;
      if (__Pyx_ParseKeywords(__pyx_kwds, __pyx_kwvalues, __pyx_pyargnames, 0, values, kwd_pos_args, __pyx_kwds_len, "_malloc", 0) < (0)) __PYX_ERR(0, 769, __pyx_L3_error)
      for (Py_ssize_t i = __pyx_nargs; i < 1; i++) {
        if (unlikely(!values[i])) { __Pyx_RaiseArgtupleInvalid("_malloc", 1, 1, 1, i); __PYX_ERR(0, 769, __pyx_L3_error) }
      }
    } else if (unlikely(__pyx_nargs != 1)) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
      if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(0, 769, __pyx_L3_error)
    }
    __pyx_v_size = __Pyx_PyLong_As_size_t(values[0]); if (unlikely((__pyx_v_size == (size_t)-1) && PyErr_Occurred())) __PYX_ERR(0, 769, __pyx_L3_error)
  }
  goto __pyx_L6_skip;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("_malloc", 1, 1, 1, __pyx_nargs); __PYX_ERR(0, 769, __pyx_L3_error)
  __pyx_L6_skip:;
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_AddTraceback("cupy.cuda.memory._malloc", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_4cupy_4cuda_6memory_2_malloc(__pyx_self, __pyx_v_size);

  /* function exit code */
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4cupy_4cuda_6memory_2_malloc(CYTHON_UNUSED PyObject *__pyx_self, size_t __pyx_v_size) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("_malloc", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = ((PyObject *)__pyx_f_4cupy_4cuda_6memory__malloc(__pyx_v_size, 1)); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 769, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("cupy.cuda.memory._malloc", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "cupy/cuda/memory.pyx":774
 * 
 * 
 * cpdef MemoryPointer malloc_async(size_t size):             # <<<<<<<<<<<<<<
 *     """Allocate memory from Stream Ordered Memory Allocator.
 * 
*/

static PyObject *__pyx_pw_4cupy_4cuda_6memory_5malloc_async(PyObject *__pyx_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
static struct __pyx_obj_4cupy_4cuda_6memory_MemoryPointer *__pyx_f_4cupy_4cuda_6memory_malloc_async(size_t __pyx_v_size, CYTHON_UNUSED int __pyx_skip_dispatch) {
  struct __pyx_obj_4cupy_4cuda_6memory_MemoryAsync *__pyx_v_mem = NULL;
  struct __pyx_obj_4cupy_4cuda_6memory_MemoryPointer *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  size_t __pyx_t_5;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("malloc_async", 0);

  /* "cupy/cuda/memory.pyx":799
 *         https://docs.nvidia.com/cuda/cuda-c-programming-guide/index.html#stream-ordered-memory-allocator
 *     """
 *     mem = MemoryAsync(size, stream_module.get_current_stream())             # <<<<<<<<<<<<<<
 *     return MemoryPointer(mem, 0)
 * 
*/
  __pyx_t_2 = NULL;
  __pyx_t_3 = __Pyx_PyLong_FromSize_t(__pyx_v_size); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 799, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_4 = __pyx_f_4cupy_4cuda_6stream_get_current_stream(0, NULL); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 799, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_t_5 = 1;
  {
    PyObject *__pyx_callargs[3] = {__pyx_t_2, __pyx_t_3, __pyx_t_4};
    __pyx_t_1 = __Pyx_PyObject_FastCall((PyObject*)__pyx_mstate_global->__pyx_ptype_4cupy_4cuda_6memory_MemoryAsync, __pyx_callargs+__pyx_t_5, (3-__pyx_t_5) | (__pyx_t_5*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
    __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 799, __pyx_L1_error)
    __Pyx_GOTREF((PyObject *)__pyx_t_1);
  }
  __pyx_v_mem = ((struct __pyx_obj_4cupy_4cuda_6memory_MemoryAsync *)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "cupy/cuda/memory.pyx":800
 *     """
 *     mem = MemoryAsync(size, stream_module.get_current_stream())
 *     return MemoryPointer(mem, 0)             # <<<<<<<<<<<<<<
 * 
 * 
*/
  __Pyx_XDECREF((PyObject *)__pyx_r);
  __pyx_t_4 = NULL;
  __pyx_t_5 = 1;
  {
    PyObject *__pyx_callargs[3] = {__pyx_t_4, ((PyObject *)__pyx_v_mem), __pyx_mstate_global->__pyx_int_0};
    __pyx_t_1 = __Pyx_PyObject_FastCall((PyObject*)__pyx_mstate_global->__pyx_ptype_4cupy_4cuda_6memory_MemoryPointer, __pyx_callargs+__pyx_t_5, (3-__pyx_t_5) | (__pyx_t_5*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
    __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
    if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 800, __pyx_L1_error)
    __Pyx_GOTREF((PyObject *)__pyx_t_1);
  }
  __pyx_r = ((struct __pyx_obj_4cupy_4cuda_6memory_MemoryPointer *)__pyx_t_1);
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "cupy/cuda/memory.pyx":774
 * 
 * 
 * cpdef MemoryPointer malloc_async(size_t size):             # <<<<<<<<<<<<<<
 *     """Allocate memory from Stream Ordered Memory Allocator.
 * 
*/

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_AddTraceback("cupy.cuda.memory.malloc_async", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XDECREF((PyObject *)__pyx_v_mem);
  __Pyx_XGIVEREF((PyObject *)__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_pw_4cupy_4cuda_6memory_5malloc_async(PyObject *__pyx_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
PyDoc_STRVAR(__pyx_doc_4cupy_4cuda_6memory_4malloc_async, "malloc_async(size_t size) -> MemoryPointer\n\nAllocate memory from Stream Ordered Memory Allocator.\n\nThis method can be used as a CuPy memory allocator. The simplest way to\nuse CUDA's Stream Ordered Memory Allocator as the default allocator is\nthe following code::\n\n    set_allocator(malloc_async)\n\nUsing this feature requires CUDA >= 11.2 with a supported GPU and platform.\nIf it is not supported, an error will be raised.\n\nThe current CuPy stream is used to allocate/free the memory.\n\nArgs:\n    size (int): Size of the memory allocation in bytes.\n\nReturns:\n    ~cupy.cuda.MemoryPointer: Pointer to the allocated buffer.\n\n.. seealso:: `Stream Ordered Memory Allocator`_\n\n.. _Stream Ordered Memory Allocator:\n    https://docs.nvidia.com/cuda/cuda-c-programming-guide/index.html#stream-ordered-memory-allocator");
static PyObject *__pyx_pw_4cupy_4cuda_6memory_5malloc_async(PyObject *__pyx_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
) {
  size_t __pyx_v_size;
  #if !CYTHON_METH_FASTCALL
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  #endif
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject* values[1] = {0};
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("malloc_async (wrapper)", 0);
  #if !CYTHON_METH_FASTCALL
  #if CYTHON_ASSUME_SAFE_SIZE
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return NULL;
  #endif
  #endif
  __pyx_kwvalues = __Pyx_KwValues_FASTCALL(__pyx_args, __pyx_nargs);
  {
    PyObject ** const __pyx_pyargnames[] = {&__pyx_mstate_global->__pyx_n_u_size,0};
    const Py_ssize_t __pyx_kwds_len = (__pyx_kwds) ? __Pyx_NumKwargs_FASTCALL(__pyx_kwds) : 0;
    if (unlikely(__pyx_kwds_len < 0)) __PYX_ERR(0, 774, __pyx_L3_error)
    if (__pyx_kwds_len > 0) {
      switch (__pyx_nargs) {
        case  1:
        values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(0, 774, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      const Py_ssize_t kwd_pos_args = __pyx_nargs;
      if (__Pyx_ParseKeywords(__pyx_kwds, __pyx_kwvalues, __pyx_pyargnames, 0, values, kwd_pos_args, __pyx_kwds_len, "malloc_async", 0) < (0)) __PYX_ERR(0, 774, __pyx_L3_error)
      for (Py_ssize_t i = __pyx_nargs; i < 1; i++) {
        if (unlikely(!values[i])) { __Pyx_RaiseArgtupleInvalid("malloc_async", 1, 1, 1, i); __PYX_ERR(0, 774, __pyx_L3_error) }
      }
    } else if (unlikely(__pyx_nargs != 1)) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
      if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(0, 774, __pyx_L3_error)
    }
    __pyx_v_size = __Pyx_PyLong_As_size_t(values[0]); if (unlikely((__pyx_v_size == (size_t)-1) && PyErr_Occurred())) __PYX_ERR(0, 774, __pyx_L3_error)
  }
  goto __pyx_L6_skip;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("malloc_async", 1, 1, 1, __pyx_nargs); __PYX_ERR(0, 774, __pyx_L3_error)
  __pyx_L6_skip:;
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_AddTraceback("cupy.cuda.memory.malloc_async", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_4cupy_4cuda_6memory_4malloc_async(__pyx_self, __pyx_v_size);

  /* function exit code */
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4cupy_4cuda_6memory_4malloc_async(CYTHON_UNUSED PyObject *__pyx_self, size_t __pyx_v_size) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("malloc_async", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = ((PyObject *)__pyx_f_4cupy_4cuda_6memory_malloc_async(__pyx_v_size, 1)); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 774, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("cupy.cuda.memory.malloc_async", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "cupy/cuda/memory.pyx":803
 * 
 * 
 * cpdef MemoryPointer malloc_managed(size_t size):             # <<<<<<<<<<<<<<
 *     """Allocate managed memory (unified memory).
 * 
*/

static PyObject *__pyx_pw_4cupy_4cuda_6memory_7malloc_managed(PyObject *__pyx_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
static struct __pyx_obj_4cupy_4cuda_6memory_MemoryPointer *__pyx_f_4cupy_4cuda_6memory_malloc_managed(size_t __pyx_v_size, CYTHON_UNUSED int __pyx_skip_dispatch) {
  struct __pyx_obj_4cupy_4cuda_6memory_ManagedMemory *__pyx_v_mem = NULL;
  struct __pyx_obj_4cupy_4cuda_6memory_MemoryPointer *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  size_t __pyx_t_4;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("malloc_managed", 0);

  /* "cupy/cuda/memory.pyx":824
 *         ~cupy.cuda.MemoryPointer: Pointer to the allocated buffer.
 *     """
 *     mem = ManagedMemory(size)             # <<<<<<<<<<<<<<
 *     return MemoryPointer(mem, 0)
 * 
*/
  __pyx_t_2 = NULL;
  __pyx_t_3 = __Pyx_PyLong_FromSize_t(__pyx_v_size); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 824, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_4 = 1;
  {
    PyObject *__pyx_callargs[2] = {__pyx_t_2, __pyx_t_3};
    __pyx_t_1 = __Pyx_PyObject_FastCall((PyObject*)__pyx_mstate_global->__pyx_ptype_4cupy_4cuda_6memory_ManagedMemory, __pyx_callargs+__pyx_t_4, (2-__pyx_t_4) | (__pyx_t_4*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
    __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 824, __pyx_L1_error)
    __Pyx_GOTREF((PyObject *)__pyx_t_1);
  }
  __pyx_v_mem = ((struct __pyx_obj_4cupy_4cuda_6memory_ManagedMemory *)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "cupy/cuda/memory.pyx":825
 *     """
 *     mem = ManagedMemory(size)
 *     return MemoryPointer(mem, 0)             # <<<<<<<<<<<<<<
 * 
 * 
*/
  __Pyx_XDECREF((PyObject *)__pyx_r);
  __pyx_t_3 = NULL;
  __pyx_t_4 = 1;
  {
    PyObject *__pyx_callargs[3] = {__pyx_t_3, ((PyObject *)__pyx_v_mem), __pyx_mstate_global->__pyx_int_0};
    __pyx_t_1 = __Pyx_PyObject_FastCall((PyObject*)__pyx_mstate_global->__pyx_ptype_4cupy_4cuda_6memory_MemoryPointer, __pyx_callargs+__pyx_t_4, (3-__pyx_t_4) | (__pyx_t_4*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
    __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
    if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 825, __pyx_L1_error)
    __Pyx_GOTREF((PyObject *)__pyx_t_1);
  }
  __pyx_r = ((struct __pyx_obj_4cupy_4cuda_6memory_MemoryPointer *)__pyx_t_1);
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "cupy/cuda/memory.pyx":803
 * 
 * 
 * cpdef MemoryPointer malloc_managed(size_t size):             # <<<<<<<<<<<<<<
 *     """Allocate managed memory (unified memory).
 * 
*/

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_AddTraceback("cupy.cuda.memory.malloc_managed", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XDECREF((PyObject *)__pyx_v_mem);
  __Pyx_XGIVEREF((PyObject *)__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_pw_4cupy_4cuda_6memory_7malloc_managed(PyObject *__pyx_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
PyDoc_STRVAR(__pyx_doc_4cupy_4cuda_6memory_6malloc_managed, "malloc_managed(size_t size) -> MemoryPointer\n\nAllocate managed memory (unified memory).\n\nThis method can be used as a CuPy memory allocator. The simplest way to\nuse a managed memory as the default allocator is the following code::\n\n    set_allocator(malloc_managed)\n\nThe advantage using managed memory in CuPy is that device memory\noversubscription is possible for GPUs that have a non-zero value for the\ndevice attribute cudaDevAttrConcurrentManagedAccess.\nCUDA >= 8.0 with GPUs later than or equal to Pascal is preferable.\n\nRead more at: https://docs.nvidia.com/cuda/cuda-runtime-api/group__CUDART__MEMORY.html#axzz4qygc1Ry1  # NOQA\n\nArgs:\n    size (int): Size of the memory allocation in bytes.\n\nReturns:\n    ~cupy.cuda.MemoryPointer: Pointer to the allocated buffer.");
static PyObject *__pyx_pw_4cupy_4cuda_6memory_7malloc_managed(PyObject *__pyx_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
) {
  size_t __pyx_v_size;
  #if !CYTHON_METH_FASTCALL
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  #endif
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject* values[1] = {0};
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("malloc_managed (wrapper)", 0);
  #if !CYTHON_METH_FASTCALL
  #if CYTHON_ASSUME_SAFE_SIZE
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return NULL;
  #endif
  #endif
  __pyx_kwvalues = __Pyx_KwValues_FASTCALL(__pyx_args, __pyx_nargs);
  {
    PyObject ** const __pyx_pyargnames[] = {&__pyx_mstate_global->__pyx_n_u_size,0};
    const Py_ssize_t __pyx_kwds_len = (__pyx_kwds) ? __Pyx_NumKwargs_FASTCALL(__pyx_kwds) : 0;
    if (unlikely(__pyx_kwds_len < 0)) __PYX_ERR(0, 803, __pyx_L3_error)
    if (__pyx_kwds_len > 0) {
      switch (__pyx_nargs) {
        case  1:
        values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(0, 803, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      const Py_ssize_t kwd_pos_args = __pyx_nargs;
      if (__Pyx_ParseKeywords(__pyx_kwds, __pyx_kwvalues, __pyx_pyargnames, 0, values, kwd_pos_args, __pyx_kwds_len, "malloc_managed", 0) < (0)) __PYX_ERR(0, 803, __pyx_L3_error)
      for (Py_ssize_t i = __pyx_nargs; i < 1; i++) {
        if (unlikely(!values[i])) { __Pyx_RaiseArgtupleInvalid("malloc_managed", 1, 1, 1, i); __PYX_ERR(0, 803, __pyx_L3_error) }
      }
    } else if (unlikely(__pyx_nargs != 1)) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
      if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(0, 803, __pyx_L3_error)
    }
    __pyx_v_size = __Pyx_PyLong_As_size_t(values[0]); if (unlikely((__pyx_v_size == (size_t)-1) && PyErr_Occurred())) __PYX_ERR(0, 803, __pyx_L3_error)
  }
  goto __pyx_L6_skip;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("malloc_managed", 1, 1, 1, __pyx_nargs); __PYX_ERR(0, 803, __pyx_L3_error)
  __pyx_L6_skip:;
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_AddTraceback("cupy.cuda.memory.malloc_managed", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_4cupy_4cuda_6memory_6malloc_managed(__pyx_self, __pyx_v_size);

  /* function exit code */
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4cupy_4cuda_6memory_6malloc_managed(CYTHON_UNUSED PyObject *__pyx_self, size_t __pyx_v_size) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("malloc_managed", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = ((PyObject *)__pyx_f_4cupy_4cuda_6memory_malloc_managed(__pyx_v_size, 1)); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 803, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("cupy.cuda.memory.malloc_managed", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "cupy/cuda/memory.pyx":828
 * 
 * 
 * cpdef MemoryPointer malloc_system(size_t size):             # <<<<<<<<<<<<<<
 *     """Allocate memory on an HMM/ATS enabled system.
 * 
*/

static PyObject *__pyx_pw_4cupy_4cuda_6memory_9malloc_system(PyObject *__pyx_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
static struct __pyx_obj_4cupy_4cuda_6memory_MemoryPointer *__pyx_f_4cupy_4cuda_6memory_malloc_system(size_t __pyx_v_size, CYTHON_UNUSED int __pyx_skip_dispatch) {
  struct __pyx_obj_4cupy_4cuda_6memory_SystemMemory *__pyx_v_mem = NULL;
  struct __pyx_obj_4cupy_4cuda_6memory_MemoryPointer *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  size_t __pyx_t_4;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("malloc_system", 0);

  /* "cupy/cuda/memory.pyx":857
 *         ~cupy.cuda.MemoryPointer: Pointer to the allocated buffer.
 *     """
 *     mem = SystemMemory(size)             # <<<<<<<<<<<<<<
 *     return MemoryPointer(mem, 0)
 * 
*/
  __pyx_t_2 = NULL;
  __pyx_t_3 = __Pyx_PyLong_FromSize_t(__pyx_v_size); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 857, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_4 = 1;
  {
    PyObject *__pyx_callargs[2] = {__pyx_t_2, __pyx_t_3};
    __pyx_t_1 = __Pyx_PyObject_FastCall((PyObject*)__pyx_mstate_global->__pyx_ptype_4cupy_4cuda_6memory_SystemMemory, __pyx_callargs+__pyx_t_4, (2-__pyx_t_4) | (__pyx_t_4*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
    __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 857, __pyx_L1_error)
    __Pyx_GOTREF((PyObject *)__pyx_t_1);
  }
  __pyx_v_mem = ((struct __pyx_obj_4cupy_4cuda_6memory_SystemMemory *)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "cupy/cuda/memory.pyx":858
 *     """
 *     mem = SystemMemory(size)
 *     return MemoryPointer(mem, 0)             # <<<<<<<<<<<<<<
 * 
 * 
*/
  __Pyx_XDECREF((PyObject *)__pyx_r);
  __pyx_t_3 = NULL;
  __pyx_t_4 = 1;
  {
    PyObject *__pyx_callargs[3] = {__pyx_t_3, ((PyObject *)__pyx_v_mem), __pyx_mstate_global->__pyx_int_0};
    __pyx_t_1 = __Pyx_PyObject_FastCall((PyObject*)__pyx_mstate_global->__pyx_ptype_4cupy_4cuda_6memory_MemoryPointer, __pyx_callargs+__pyx_t_4, (3-__pyx_t_4) | (__pyx_t_4*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
    __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
    if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 858, __pyx_L1_error)
    __Pyx_GOTREF((PyObject *)__pyx_t_1);
  }
  __pyx_r = ((struct __pyx_obj_4cupy_4cuda_6memory_MemoryPointer *)__pyx_t_1);
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "cupy/cuda/memory.pyx":828
 * 
 * 
 * cpdef MemoryPointer malloc_system(size_t size):             # <<<<<<<<<<<<<<
 *     """Allocate memory on an HMM/ATS enabled system.
 * 
*/

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_AddTraceback("cupy.cuda.memory.malloc_system", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XDECREF((PyObject *)__pyx_v_mem);
  __Pyx_XGIVEREF((PyObject *)__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_pw_4cupy_4cuda_6memory_9malloc_system(PyObject *__pyx_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
PyDoc_STRVAR(__pyx_doc_4cupy_4cuda_6memory_8malloc_system, "malloc_system(size_t size) -> MemoryPointer\n\nAllocate memory on an HMM/ATS enabled system.\n\nThis method can be used as a CuPy memory allocator. The simplest way to\nuse system memory as the default allocator is the following code::\n\n    set_allocator(malloc_system)\n\nOr, to enable the memory pool support (recommended)::\n\n    set_allocator(MemoryPool(malloc_system).malloc)\n\nHMM stands for heterogeneous memory management. It is a kernel-level\nfeature allowing memory allocated via the system ``malloc`` to be\naccessible by both CPU and GPU. Read more at:\nhttps://developer.nvidia.com/blog/simplifying-gpu-application-development-with-heterogeneous-memory-management  # NOQA\n\nATS stands for Address Translation Services. It is a hardware/software\nfeature on Grace Hopper that enables the CPU and GPU to share a single\nper-process page table, allowing memory allocated by the system to be\naccessible by both CPU and GPU. Read more at:\nhttps://developer.nvidia.com/blog/nvidia-grace-hopper-superchip-architecture-in-depth/  # NOQA\n\nArgs:\n    size (int): Size of the memory allocation in bytes.\n\nReturns:\n    ~cupy.cuda.MemoryPointer: Pointer to the allocated buffer.");
static PyObject *__pyx_pw_4cupy_4cuda_6memory_9malloc_system(PyObject *__pyx_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
) {
  size_t __pyx_v_size;
  #if !CYTHON_METH_FASTCALL
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  #endif
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject* values[1] = {0};
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("malloc_system (wrapper)", 0);
  #if !CYTHON_METH_FASTCALL
  #if CYTHON_ASSUME_SAFE_SIZE
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return NULL;
  #endif
  #endif
  __pyx_kwvalues = __Pyx_KwValues_FASTCALL(__pyx_args, __pyx_nargs);
  {
    PyObject ** const __pyx_pyargnames[] = {&__pyx_mstate_global->__pyx_n_u_size,0};
    const Py_ssize_t __pyx_kwds_len = (__pyx_kwds) ? __Pyx_NumKwargs_FASTCALL(__pyx_kwds) : 0;
    if (unlikely(__pyx_kwds_len < 0)) __PYX_ERR(0, 828, __pyx_L3_error)
    if (__pyx_kwds_len > 0) {
      switch (__pyx_nargs) {
        case  1:
        values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(0, 828, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      const Py_ssize_t kwd_pos_args = __pyx_nargs;
      if (__Pyx_ParseKeywords(__pyx_kwds, __pyx_kwvalues, __pyx_pyargnames, 0, values, kwd_pos_args, __pyx_kwds_len, "malloc_system", 0) < (0)) __PYX_ERR(0, 828, __pyx_L3_error)
      for (Py_ssize_t i = __pyx_nargs; i < 1; i++) {
        if (unlikely(!values[i])) { __Pyx_RaiseArgtupleInvalid("malloc_system", 1, 1, 1, i); __PYX_ERR(0, 828, __pyx_L3_error) }
      }
    } else if (unlikely(__pyx_nargs != 1)) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
      if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(0, 828, __pyx_L3_error)
    }
    __pyx_v_size = __Pyx_PyLong_As_size_t(values[0]); if (unlikely((__pyx_v_size == (size_t)-1) && PyErr_Occurred())) __PYX_ERR(0, 828, __pyx_L3_error)
  }
  goto __pyx_L6_skip;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("malloc_system", 1, 1, 1, __pyx_nargs); __PYX_ERR(0, 828, __pyx_L3_error)
  __pyx_L6_skip:;
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_AddTraceback("cupy.cuda.memory.malloc_system", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_4cupy_4cuda_6memory_8malloc_system(__pyx_self, __pyx_v_size);

  /* function exit code */
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4cupy_4cuda_6memory_8malloc_system(CYTHON_UNUSED PyObject *__pyx_self, size_t __pyx_v_size) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("malloc_system", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = ((PyObject *)__pyx_f_4cupy_4cuda_6memory_malloc_system(__pyx_v_size, 1)); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 828, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("cupy.cuda.memory.malloc_system", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "cupy/cuda/memory.pyx":865
 * 
 * 
 * def _get_thread_local_allocator():             # <<<<<<<<<<<<<<
 *     try:
 *         allocator = _thread_local.allocator
*/

/* Python wrapper */
static PyObject *__pyx_pw_4cupy_4cuda_6memory_11_get_thread_local_allocator(PyObject *__pyx_self, CYTHON_UNUSED PyObject *unused); /*proto*/
PyDoc_STRVAR(__pyx_doc_4cupy_4cuda_6memory_10_get_thread_local_allocator, "_get_thread_local_allocator()");
static PyMethodDef __pyx_mdef_4cupy_4cuda_6memory_11_get_thread_local_allocator = {"_get_thread_local_allocator", (PyCFunction)__pyx_pw_4cupy_4cuda_6memory_11_get_thread_local_allocator, METH_NOARGS, __pyx_doc_4cupy_4cuda_6memory_10_get_thread_local_allocator};
static PyObject *__pyx_pw_4cupy_4cuda_6memory_11_get_thread_local_allocator(PyObject *__pyx_self, CYTHON_UNUSED PyObject *unused) {
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("_get_thread_local_allocator (wrapper)", 0);
  __pyx_kwvalues = __Pyx_KwValues_VARARGS(__pyx_args, __pyx_nargs);
  __pyx_r = __pyx_pf_4cupy_4cuda_6memory_10_get_thread_local_allocator(__pyx_self);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4cupy_4cuda_6memory_10_get_thread_local_allocator(CYTHON_UNUSED PyObject *__pyx_self) {
  PyObject *__pyx_v_allocator = NULL;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  int __pyx_t_5;
  PyObject *__pyx_t_6 = NULL;
  PyObject *__pyx_t_7 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("_get_thread_local_allocator", 0);

  /* "cupy/cuda/memory.pyx":866
 * 
 * def _get_thread_local_allocator():
 *     try:             # <<<<<<<<<<<<<<
 *         allocator = _thread_local.allocator
 *     except AttributeError:
*/
  {
    __Pyx_PyThreadState_declare
    __Pyx_PyThreadState_assign
    __Pyx_ExceptionSave(&__pyx_t_1, &__pyx_t_2, &__pyx_t_3);
    __Pyx_XGOTREF(__pyx_t_1);
    __Pyx_XGOTREF(__pyx_t_2);
    __Pyx_XGOTREF(__pyx_t_3);
    /*try:*/ {

      /* "cupy/cuda/memory.pyx":867
 * def _get_thread_local_allocator():
 *     try:
 *         allocator = _thread_local.allocator             # <<<<<<<<<<<<<<
 *     except AttributeError:
 *         allocator = _thread_local.allocator = None
*/
      __pyx_t_4 = __Pyx_PyObject_GetAttrStr(__pyx_v_4cupy_4cuda_6memory__thread_local, __pyx_mstate_global->__pyx_n_u_allocator); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 867, __pyx_L3_error)
      __Pyx_GOTREF(__pyx_t_4);
      __pyx_v_allocator = __pyx_t_4;
      __pyx_t_4 = 0;

      /* "cupy/cuda/memory.pyx":866
 * 
 * def _get_thread_local_allocator():
 *     try:             # <<<<<<<<<<<<<<
 *         allocator = _thread_local.allocator
 *     except AttributeError:
*/
    }
    __Pyx_XDECREF(__pyx_t_1); __pyx_t_1 = 0;
    __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
    goto __pyx_L8_try_end;
    __pyx_L3_error:;
    __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;

    /* "cupy/cuda/memory.pyx":868
 *     try:
 *         allocator = _thread_local.allocator
 *     except AttributeError:             # <<<<<<<<<<<<<<
 *         allocator = _thread_local.allocator = None
 *     return allocator
*/
    __pyx_t_5 = __Pyx_PyErr_ExceptionMatches(((PyObject *)(((PyTypeObject*)PyExc_AttributeError))));
    if (__pyx_t_5) {
      __Pyx_AddTraceback("cupy.cuda.memory._get_thread_local_allocator", __pyx_clineno, __pyx_lineno, __pyx_filename);
      if (__Pyx_GetException(&__pyx_t_4, &__pyx_t_6, &__pyx_t_7) < 0) __PYX_ERR(0, 868, __pyx_L5_except_error)
      __Pyx_XGOTREF(__pyx_t_4);
      __Pyx_XGOTREF(__pyx_t_6);
      __Pyx_XGOTREF(__pyx_t_7);

      /* "cupy/cuda/memory.pyx":869
 *         allocator = _thread_local.allocator
 *     except AttributeError:
 *         allocator = _thread_local.allocator = None             # <<<<<<<<<<<<<<
 *     return allocator
 * 
*/
      __Pyx_INCREF(Py_None);
      __Pyx_XDECREF_SET(__pyx_v_allocator, Py_None);
      if (__Pyx_PyObject_SetAttrStr(__pyx_v_4cupy_4cuda_6memory__thread_local, __pyx_mstate_global->__pyx_n_u_allocator, Py_None) < (0)) __PYX_ERR(0, 869, __pyx_L5_except_error)
      __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
      __Pyx_XDECREF(__pyx_t_6); __pyx_t_6 = 0;
      __Pyx_XDECREF(__pyx_t_7); __pyx_t_7 = 0;
      goto __pyx_L4_exception_handled;
    }
    goto __pyx_L5_except_error;

    /* "cupy/cuda/memory.pyx":866
 * 
 * def _get_thread_local_allocator():
 *     try:             # <<<<<<<<<<<<<<
 *         allocator = _thread_local.allocator
 *     except AttributeError:
*/
    __pyx_L5_except_error:;
    __Pyx_XGIVEREF(__pyx_t_1);
    __Pyx_XGIVEREF(__pyx_t_2);
    __Pyx_XGIVEREF(__pyx_t_3);
    __Pyx_ExceptionReset(__pyx_t_1, __pyx_t_2, __pyx_t_3);
    goto __pyx_L1_error;
    __pyx_L4_exception_handled:;
    __Pyx_XGIVEREF(__pyx_t_1);
    __Pyx_XGIVEREF(__pyx_t_2);
    __Pyx_XGIVEREF(__pyx_t_3);
    __Pyx_ExceptionReset(__pyx_t_1, __pyx_t_2, __pyx_t_3);
    __pyx_L8_try_end:;
  }

  /* "cupy/cuda/memory.pyx":870
 *     except AttributeError:
 *         allocator = _thread_local.allocator = None
 *     return allocator             # <<<<<<<<<<<<<<
 * 
 * 
*/
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_v_allocator);
  __pyx_r = __pyx_v_allocator;
  goto __pyx_L0;

  /* "cupy/cuda/memory.pyx":865
 * 
 * 
 * def _get_thread_local_allocator():             # <<<<<<<<<<<<<<
 *     try:
 *         allocator = _thread_local.allocator
*/

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_XDECREF(__pyx_t_7);
  __Pyx_AddTraceback("cupy.cuda.memory._get_thread_local_allocator", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_allocator);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "cupy/cuda/memory.pyx":873
 * 
 * 
 * def _set_thread_local_allocator(allocator):             # <<<<<<<<<<<<<<
 *     _thread_local.allocator = allocator
 * 
*/

/* Python wrapper */
static PyObject *__pyx_pw_4cupy_4cuda_6memory_13_set_thread_local_allocator(PyObject *__pyx_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
PyDoc_STRVAR(__pyx_doc_4cupy_4cuda_6memory_12_set_thread_local_allocator, "_set_thread_local_allocator(allocator)");
static PyMethodDef __pyx_mdef_4cupy_4cuda_6memory_13_set_thread_local_allocator = {"_set_thread_local_allocator", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_4cupy_4cuda_6memory_13_set_thread_local_allocator, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_4cupy_4cuda_6memory_12_set_thread_local_allocator};
static PyObject *__pyx_pw_4cupy_4cuda_6memory_13_set_thread_local_allocator(PyObject *__pyx_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
) {
  PyObject *__pyx_v_allocator = 0;
  #if !CYTHON_METH_FASTCALL
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  #endif
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject* values[1] = {0};
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("_set_thread_local_allocator (wrapper)", 0);
  #if !CYTHON_METH_FASTCALL
  #if CYTHON_ASSUME_SAFE_SIZE
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return NULL;
  #endif
  #endif
  __pyx_kwvalues = __Pyx_KwValues_FASTCALL(__pyx_args, __pyx_nargs);
  {
    PyObject ** const __pyx_pyargnames[] = {&__pyx_mstate_global->__pyx_n_u_allocator,0};
    const Py_ssize_t __pyx_kwds_len = (__pyx_kwds) ? __Pyx_NumKwargs_FASTCALL(__pyx_kwds) : 0;
    if (unlikely(__pyx_kwds_len < 0)) __PYX_ERR(0, 873, __pyx_L3_error)
    if (__pyx_kwds_len > 0) {
      switch (__pyx_nargs) {
        case  1:
        values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(0, 873, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      const Py_ssize_t kwd_pos_args = __pyx_nargs;
      if (__Pyx_ParseKeywords(__pyx_kwds, __pyx_kwvalues, __pyx_pyargnames, 0, values, kwd_pos_args, __pyx_kwds_len, "_set_thread_local_allocator", 0) < (0)) __PYX_ERR(0, 873, __pyx_L3_error)
      for (Py_ssize_t i = __pyx_nargs; i < 1; i++) {
        if (unlikely(!values[i])) { __Pyx_RaiseArgtupleInvalid("_set_thread_local_allocator", 1, 1, 1, i); __PYX_ERR(0, 873, __pyx_L3_error) }
      }
    } else if (unlikely(__pyx_nargs != 1)) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
      if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(0, 873, __pyx_L3_error)
    }
    __pyx_v_allocator = values[0];
  }
  goto __pyx_L6_skip;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("_set_thread_local_allocator", 1, 1, 1, __pyx_nargs); __PYX_ERR(0, 873, __pyx_L3_error)
  __pyx_L6_skip:;
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_AddTraceback("cupy.cuda.memory._set_thread_local_allocator", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_4cupy_4cuda_6memory_12_set_thread_local_allocator(__pyx_self, __pyx_v_allocator);

  /* function exit code */
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4cupy_4cuda_6memory_12_set_thread_local_allocator(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_allocator) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("_set_thread_local_allocator", 0);

  /* "cupy/cuda/memory.pyx":874
 * 
 * def _set_thread_local_allocator(allocator):
 *     _thread_local.allocator = allocator             # <<<<<<<<<<<<<<
 * 
 * 
*/
  if (__Pyx_PyObject_SetAttrStr(__pyx_v_4cupy_4cuda_6memory__thread_local, __pyx_mstate_global->__pyx_n_u_allocator, __pyx_v_allocator) < (0)) __PYX_ERR(0, 874, __pyx_L1_error)

  /* "cupy/cuda/memory.pyx":873
 * 
 * 
 * def _set_thread_local_allocator(allocator):             # <<<<<<<<<<<<<<
 *     _thread_local.allocator = allocator
 * 
*/

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_AddTraceback("cupy.cuda.memory._set_thread_local_allocator", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "cupy/cuda/memory.pyx":877
 * 
 * 
 * cdef inline intptr_t _get_stream_identifier(intptr_t stream_ptr) except? -1:             # <<<<<<<<<<<<<<
 *     # When PTDS is enabled, return an ID to uniquely identify the default
 *     # stream for each thread. (#5069)
*/

static CYTHON_INLINE intptr_t __pyx_f_4cupy_4cuda_6memory__get_stream_identifier(intptr_t __pyx_v_stream_ptr) {
  intptr_t __pyx_v_tid;
  PyObject *__pyx_v_tid_obj = NULL;
  intptr_t __pyx_r;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  intptr_t __pyx_t_6;
  int __pyx_t_7;
  PyObject *__pyx_t_8 = NULL;
  PyObject *__pyx_t_9 = NULL;
  PyObject *__pyx_t_10 = NULL;
  PyObject *__pyx_t_11 = NULL;
  size_t __pyx_t_12;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("_get_stream_identifier", 0);

  /* "cupy/cuda/memory.pyx":880
 *     # When PTDS is enabled, return an ID to uniquely identify the default
 *     # stream for each thread. (#5069)
 *     if stream_ptr != runtime.streamPerThread:             # <<<<<<<<<<<<<<
 *         return stream_ptr
 * 
*/
  __pyx_t_1 = (__pyx_v_stream_ptr != __pyx_e_13cupy_backends_4cuda_3api_7runtime_streamPerThread);
  if (__pyx_t_1) {

    /* "cupy/cuda/memory.pyx":881
 *     # stream for each thread. (#5069)
 *     if stream_ptr != runtime.streamPerThread:
 *         return stream_ptr             # <<<<<<<<<<<<<<
 * 
 *     cdef intptr_t tid
*/
    __pyx_r = __pyx_v_stream_ptr;
    goto __pyx_L0;

    /* "cupy/cuda/memory.pyx":880
 *     # When PTDS is enabled, return an ID to uniquely identify the default
 *     # stream for each thread. (#5069)
 *     if stream_ptr != runtime.streamPerThread:             # <<<<<<<<<<<<<<
 *         return stream_ptr
 * 
*/
  }

  /* "cupy/cuda/memory.pyx":884
 * 
 *     cdef intptr_t tid
 *     try:             # <<<<<<<<<<<<<<
 *         tid = _thread_local._tid
 *     except AttributeError:
*/
  {
    __Pyx_PyThreadState_declare
    __Pyx_PyThreadState_assign
    __Pyx_ExceptionSave(&__pyx_t_2, &__pyx_t_3, &__pyx_t_4);
    __Pyx_XGOTREF(__pyx_t_2);
    __Pyx_XGOTREF(__pyx_t_3);
    __Pyx_XGOTREF(__pyx_t_4);
    /*try:*/ {

      /* "cupy/cuda/memory.pyx":885
 *     cdef intptr_t tid
 *     try:
 *         tid = _thread_local._tid             # <<<<<<<<<<<<<<
 *     except AttributeError:
 *         _thread_local._tid_obj = tid_obj = object()
*/
      __pyx_t_5 = __Pyx_PyObject_GetAttrStr(__pyx_v_4cupy_4cuda_6memory__thread_local, __pyx_mstate_global->__pyx_n_u_tid); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 885, __pyx_L4_error)
      __Pyx_GOTREF(__pyx_t_5);
      __pyx_t_6 = PyLong_AsSsize_t(__pyx_t_5); if (unlikely((__pyx_t_6 == ((intptr_t)-1)) && PyErr_Occurred())) __PYX_ERR(0, 885, __pyx_L4_error)
      __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
      __pyx_v_tid = __pyx_t_6;

      /* "cupy/cuda/memory.pyx":884
 * 
 *     cdef intptr_t tid
 *     try:             # <<<<<<<<<<<<<<
 *         tid = _thread_local._tid
 *     except AttributeError:
*/
    }
    __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
    __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
    goto __pyx_L9_try_end;
    __pyx_L4_error:;
    __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;

    /* "cupy/cuda/memory.pyx":886
 *     try:
 *         tid = _thread_local._tid
 *     except AttributeError:             # <<<<<<<<<<<<<<
 *         _thread_local._tid_obj = tid_obj = object()
 *         _thread_local._tid = tid = id(tid_obj)
*/
    __pyx_t_7 = __Pyx_PyErr_ExceptionMatches(((PyObject *)(((PyTypeObject*)PyExc_AttributeError))));
    if (__pyx_t_7) {
      __Pyx_AddTraceback("cupy.cuda.memory._get_stream_identifier", __pyx_clineno, __pyx_lineno, __pyx_filename);
      if (__Pyx_GetException(&__pyx_t_5, &__pyx_t_8, &__pyx_t_9) < 0) __PYX_ERR(0, 886, __pyx_L6_except_error)
      __Pyx_XGOTREF(__pyx_t_5);
      __Pyx_XGOTREF(__pyx_t_8);
      __Pyx_XGOTREF(__pyx_t_9);

      /* "cupy/cuda/memory.pyx":887
 *         tid = _thread_local._tid
 *     except AttributeError:
 *         _thread_local._tid_obj = tid_obj = object()             # <<<<<<<<<<<<<<
 *         _thread_local._tid = tid = id(tid_obj)
 *     return -tid
*/
      __pyx_t_11 = NULL;
      __pyx_t_12 = 1;
      {
        PyObject *__pyx_callargs[2] = {__pyx_t_11, NULL};
        __pyx_t_10 = __Pyx_PyObject_FastCall((PyObject*)__pyx_builtin_object, __pyx_callargs+__pyx_t_12, (1-__pyx_t_12) | (__pyx_t_12*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
        __Pyx_XDECREF(__pyx_t_11); __pyx_t_11 = 0;
        if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 887, __pyx_L6_except_error)
        __Pyx_GOTREF(__pyx_t_10);
      }
      if (__Pyx_PyObject_SetAttrStr(__pyx_v_4cupy_4cuda_6memory__thread_local, __pyx_mstate_global->__pyx_n_u_tid_obj, __pyx_t_10) < (0)) __PYX_ERR(0, 887, __pyx_L6_except_error)
      __Pyx_INCREF(__pyx_t_10);
      __pyx_v_tid_obj = __pyx_t_10;
      __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;

      /* "cupy/cuda/memory.pyx":888
 *     except AttributeError:
 *         _thread_local._tid_obj = tid_obj = object()
 *         _thread_local._tid = tid = id(tid_obj)             # <<<<<<<<<<<<<<
 *     return -tid
 * 
*/
      __pyx_t_11 = NULL;
      __pyx_t_12 = 1;
      {
        PyObject *__pyx_callargs[2] = {__pyx_t_11, __pyx_v_tid_obj};
        __pyx_t_10 = __Pyx_PyObject_FastCall((PyObject*)__pyx_builtin_id, __pyx_callargs+__pyx_t_12, (2-__pyx_t_12) | (__pyx_t_12*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
        __Pyx_XDECREF(__pyx_t_11); __pyx_t_11 = 0;
        if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 888, __pyx_L6_except_error)
        __Pyx_GOTREF(__pyx_t_10);
      }
      __pyx_t_6 = PyLong_AsSsize_t(__pyx_t_10); if (unlikely((__pyx_t_6 == ((intptr_t)-1)) && PyErr_Occurred())) __PYX_ERR(0, 888, __pyx_L6_except_error)
      if (__Pyx_PyObject_SetAttrStr(__pyx_v_4cupy_4cuda_6memory__thread_local, __pyx_mstate_global->__pyx_n_u_tid, __pyx_t_10) < (0)) __PYX_ERR(0, 888, __pyx_L6_except_error)
      __pyx_v_tid = __pyx_t_6;
      __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
      __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
      __Pyx_XDECREF(__pyx_t_8); __pyx_t_8 = 0;
      __Pyx_XDECREF(__pyx_t_9); __pyx_t_9 = 0;
      goto __pyx_L5_exception_handled;
    }
    goto __pyx_L6_except_error;

    /* "cupy/cuda/memory.pyx":884
 * 
 *     cdef intptr_t tid
 *     try:             # <<<<<<<<<<<<<<
 *         tid = _thread_local._tid
 *     except AttributeError:
*/
    __pyx_L6_except_error:;
    __Pyx_XGIVEREF(__pyx_t_2);
    __Pyx_XGIVEREF(__pyx_t_3);
    __Pyx_XGIVEREF(__pyx_t_4);
    __Pyx_ExceptionReset(__pyx_t_2, __pyx_t_3, __pyx_t_4);
    goto __pyx_L1_error;
    __pyx_L5_exception_handled:;
    __Pyx_XGIVEREF(__pyx_t_2);
    __Pyx_XGIVEREF(__pyx_t_3);
    __Pyx_XGIVEREF(__pyx_t_4);
    __Pyx_ExceptionReset(__pyx_t_2, __pyx_t_3, __pyx_t_4);
    __pyx_L9_try_end:;
  }

  /* "cupy/cuda/memory.pyx":889
 *         _thread_local._tid_obj = tid_obj = object()
 *         _thread_local._tid = tid = id(tid_obj)
 *     return -tid             # <<<<<<<<<<<<<<
 * 
 * 
*/
  __pyx_r = (-__pyx_v_tid);
  goto __pyx_L0;

  /* "cupy/cuda/memory.pyx":877
 * 
 * 
 * cdef inline intptr_t _get_stream_identifier(intptr_t stream_ptr) except? -1:             # <<<<<<<<<<<<<<
 *     # When PTDS is enabled, return an ID to uniquely identify the default
 *     # stream for each thread. (#5069)
*/

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_XDECREF(__pyx_t_8);
  __Pyx_XDECREF(__pyx_t_9);
  __Pyx_XDECREF(__pyx_t_10);
  __Pyx_XDECREF(__pyx_t_11);
  __Pyx_AddTraceback("cupy.cuda.memory._get_stream_identifier", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1L;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_tid_obj);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "cupy/cuda/memory.pyx":892
 * 
 * 
 * cpdef MemoryPointer alloc(size):             # <<<<<<<<<<<<<<
 *     """Calls the current allocator.
 * 
*/

static PyObject *__pyx_pw_4cupy_4cuda_6memory_15alloc(PyObject *__pyx_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
static struct __pyx_obj_4cupy_4cuda_6memory_MemoryPointer *__pyx_f_4cupy_4cuda_6memory_alloc(PyObject *__pyx_v_size, CYTHON_UNUSED int __pyx_skip_dispatch) {
  struct __pyx_obj_4cupy_4cuda_6memory_MemoryPointer *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  size_t __pyx_t_4;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("alloc", 0);

  /* "cupy/cuda/memory.pyx":904
 * 
 *     """
 *     return get_allocator()(size)             # <<<<<<<<<<<<<<
 * 
 * 
*/
  __Pyx_XDECREF((PyObject *)__pyx_r);
  __pyx_t_2 = NULL;
  __pyx_t_3 = __pyx_f_4cupy_4cuda_6memory_get_allocator(0); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 904, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_4 = 1;
  #if CYTHON_UNPACK_METHODS
  if (unlikely(PyMethod_Check(__pyx_t_3))) {
    __pyx_t_2 = PyMethod_GET_SELF(__pyx_t_3);
    assert(__pyx_t_2);
    PyObject* __pyx__function = PyMethod_GET_FUNCTION(__pyx_t_3);
    __Pyx_INCREF(__pyx_t_2);
    __Pyx_INCREF(__pyx__function);
    __Pyx_DECREF_SET(__pyx_t_3, __pyx__function);
    __pyx_t_4 = 0;
  }
  #endif
  {
    PyObject *__pyx_callargs[2] = {__pyx_t_2, __pyx_v_size};
    __pyx_t_1 = __Pyx_PyObject_FastCall((PyObject*)__pyx_t_3, __pyx_callargs+__pyx_t_4, (2-__pyx_t_4) | (__pyx_t_4*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
    __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 904, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
  }
  if (!(likely(((__pyx_t_1) == Py_None) || likely(__Pyx_TypeTest(__pyx_t_1, __pyx_mstate_global->__pyx_ptype_4cupy_4cuda_6memory_MemoryPointer))))) __PYX_ERR(0, 904, __pyx_L1_error)
  __pyx_r = ((struct __pyx_obj_4cupy_4cuda_6memory_MemoryPointer *)__pyx_t_1);
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "cupy/cuda/memory.pyx":892
 * 
 * 
 * cpdef MemoryPointer alloc(size):             # <<<<<<<<<<<<<<
 *     """Calls the current allocator.
 * 
*/

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_AddTraceback("cupy.cuda.memory.alloc", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF((PyObject *)__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_pw_4cupy_4cuda_6memory_15alloc(PyObject *__pyx_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
PyDoc_STRVAR(__pyx_doc_4cupy_4cuda_6memory_14alloc, "alloc(size) -> MemoryPointer\n\nCalls the current allocator.\n\nUse :func:`~cupy.cuda.set_allocator` to change the current allocator.\n\nArgs:\n    size (int): Size of the memory allocation.\n\nReturns:\n    ~cupy.cuda.MemoryPointer: Pointer to the allocated buffer.");
static PyObject *__pyx_pw_4cupy_4cuda_6memory_15alloc(PyObject *__pyx_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
) {
  PyObject *__pyx_v_size = 0;
  #if !CYTHON_METH_FASTCALL
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  #endif
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject* values[1] = {0};
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("alloc (wrapper)", 0);
  #if !CYTHON_METH_FASTCALL
  #if CYTHON_ASSUME_SAFE_SIZE
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return NULL;
  #endif
  #endif
  __pyx_kwvalues = __Pyx_KwValues_FASTCALL(__pyx_args, __pyx_nargs);
  {
    PyObject ** const __pyx_pyargnames[] = {&__pyx_mstate_global->__pyx_n_u_size,0};
    const Py_ssize_t __pyx_kwds_len = (__pyx_kwds) ? __Pyx_NumKwargs_FASTCALL(__pyx_kwds) : 0;
    if (unlikely(__pyx_kwds_len < 0)) __PYX_ERR(0, 892, __pyx_L3_error)
    if (__pyx_kwds_len > 0) {
      switch (__pyx_nargs) {
        case  1:
        values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(0, 892, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      const Py_ssize_t kwd_pos_args = __pyx_nargs;
      if (__Pyx_ParseKeywords(__pyx_kwds, __pyx_kwvalues, __pyx_pyargnames, 0, values, kwd_pos_args, __pyx_kwds_len, "alloc", 0) < (0)) __PYX_ERR(0, 892, __pyx_L3_error)
      for (Py_ssize_t i = __pyx_nargs; i < 1; i++) {
        if (unlikely(!values[i])) { __Pyx_RaiseArgtupleInvalid("alloc", 1, 1, 1, i); __PYX_ERR(0, 892, __pyx_L3_error) }
      }
    } else if (unlikely(__pyx_nargs != 1)) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
      if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(0, 892, __pyx_L3_error)
    }
    __pyx_v_size = values[0];
  }
  goto __pyx_L6_skip;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("alloc", 1, 1, 1, __pyx_nargs); __PYX_ERR(0, 892, __pyx_L3_error)
  __pyx_L6_skip:;
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_AddTraceback("cupy.cuda.memory.alloc", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_4cupy_4cuda_6memory_14alloc(__pyx_self, __pyx_v_size);

  /* function exit code */
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4cupy_4cuda_6memory_14alloc(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_size) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("alloc", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = ((PyObject *)__pyx_f_4cupy_4cuda_6memory_alloc(__pyx_v_size, 1)); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 892, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("cupy.cuda.memory.alloc", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "cupy/cuda/memory.pyx":907
 * 
 * 
 * cpdef set_allocator(allocator=None):             # <<<<<<<<<<<<<<
 *     """Sets the current allocator for GPU memory.
 * 
*/

static PyObject *__pyx_pw_4cupy_4cuda_6memory_17set_allocator(PyObject *__pyx_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
static PyObject *__pyx_f_4cupy_4cuda_6memory_set_allocator(CYTHON_UNUSED int __pyx_skip_dispatch, struct __pyx_opt_args_4cupy_4cuda_6memory_set_allocator *__pyx_optional_args) {
  PyObject *__pyx_v_allocator = ((PyObject *)Py_None);
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  size_t __pyx_t_4;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("set_allocator", 0);
  if (__pyx_optional_args) {
    if (__pyx_optional_args->__pyx_n > 0) {
      __pyx_v_allocator = __pyx_optional_args->allocator;
    }
  }
  __Pyx_INCREF(__pyx_v_allocator);

  /* "cupy/cuda/memory.pyx":919
 *     """
 *     global _current_allocator
 *     if allocator is None:             # <<<<<<<<<<<<<<
 *         allocator = _malloc
 *     if getattr(_thread_local, 'allocator', None) is not None:
*/
  __pyx_t_1 = (__pyx_v_allocator == Py_None);
  if (__pyx_t_1) {

    /* "cupy/cuda/memory.pyx":920
 *     global _current_allocator
 *     if allocator is None:
 *         allocator = _malloc             # <<<<<<<<<<<<<<
 *     if getattr(_thread_local, 'allocator', None) is not None:
 *         raise ValueError('Can\'t change the global allocator inside '
*/
    __Pyx_GetModuleGlobalName(__pyx_t_2, __pyx_mstate_global->__pyx_n_u_malloc); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 920, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_DECREF_SET(__pyx_v_allocator, __pyx_t_2);
    __pyx_t_2 = 0;

    /* "cupy/cuda/memory.pyx":919
 *     """
 *     global _current_allocator
 *     if allocator is None:             # <<<<<<<<<<<<<<
 *         allocator = _malloc
 *     if getattr(_thread_local, 'allocator', None) is not None:
*/
  }

  /* "cupy/cuda/memory.pyx":921
 *     if allocator is None:
 *         allocator = _malloc
 *     if getattr(_thread_local, 'allocator', None) is not None:             # <<<<<<<<<<<<<<
 *         raise ValueError('Can\'t change the global allocator inside '
 *                          '`using_allocator` context manager')
*/
  __pyx_t_2 = __pyx_v_4cupy_4cuda_6memory__thread_local;
  __Pyx_INCREF(__pyx_t_2);
  __pyx_t_3 = __Pyx_GetAttr3(__pyx_t_2, __pyx_mstate_global->__pyx_n_u_allocator, Py_None); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 921, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_1 = (__pyx_t_3 != Py_None);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  if (unlikely(__pyx_t_1)) {

    /* "cupy/cuda/memory.pyx":922
 *         allocator = _malloc
 *     if getattr(_thread_local, 'allocator', None) is not None:
 *         raise ValueError('Can\'t change the global allocator inside '             # <<<<<<<<<<<<<<
 *                          '`using_allocator` context manager')
 *     _current_allocator = allocator
*/
    __pyx_t_2 = NULL;
    __pyx_t_4 = 1;
    {
      PyObject *__pyx_callargs[2] = {__pyx_t_2, __pyx_mstate_global->__pyx_kp_u_Can_t_change_the_global_allocato};
      __pyx_t_3 = __Pyx_PyObject_FastCall((PyObject*)(((PyTypeObject*)PyExc_ValueError)), __pyx_callargs+__pyx_t_4, (2-__pyx_t_4) | (__pyx_t_4*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
      __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
      if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 922, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_3);
    }
    __Pyx_Raise(__pyx_t_3, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __PYX_ERR(0, 922, __pyx_L1_error)

    /* "cupy/cuda/memory.pyx":921
 *     if allocator is None:
 *         allocator = _malloc
 *     if getattr(_thread_local, 'allocator', None) is not None:             # <<<<<<<<<<<<<<
 *         raise ValueError('Can\'t change the global allocator inside '
 *                          '`using_allocator` context manager')
*/
  }

  /* "cupy/cuda/memory.pyx":924
 *         raise ValueError('Can\'t change the global allocator inside '
 *                          '`using_allocator` context manager')
 *     _current_allocator = allocator             # <<<<<<<<<<<<<<
 * 
 * 
*/
  __Pyx_INCREF(__pyx_v_allocator);
  __Pyx_XGOTREF(__pyx_v_4cupy_4cuda_6memory__current_allocator);
  __Pyx_DECREF_SET(__pyx_v_4cupy_4cuda_6memory__current_allocator, __pyx_v_allocator);
  __Pyx_GIVEREF(__pyx_v_allocator);

  /* "cupy/cuda/memory.pyx":907
 * 
 * 
 * cpdef set_allocator(allocator=None):             # <<<<<<<<<<<<<<
 *     """Sets the current allocator for GPU memory.
 * 
*/

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_AddTraceback("cupy.cuda.memory.set_allocator", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_allocator);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_pw_4cupy_4cuda_6memory_17set_allocator(PyObject *__pyx_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
PyDoc_STRVAR(__pyx_doc_4cupy_4cuda_6memory_16set_allocator, "set_allocator(allocator=None)\n\nSets the current allocator for GPU memory.\n\nArgs:\n    allocator (function): CuPy memory allocator. It must have the same\n        interface as the :func:`cupy.cuda.alloc` function, which takes the\n        buffer size as an argument and returns the device buffer of that\n        size. When ``None`` is specified, raw memory allocator will be\n        used (i.e., memory pool is disabled).");
static PyObject *__pyx_pw_4cupy_4cuda_6memory_17set_allocator(PyObject *__pyx_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
) {
  PyObject *__pyx_v_allocator = 0;
  #if !CYTHON_METH_FASTCALL
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  #endif
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject* values[1] = {0};
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("set_allocator (wrapper)", 0);
  #if !CYTHON_METH_FASTCALL
  #if CYTHON_ASSUME_SAFE_SIZE
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return NULL;
  #endif
  #endif
  __pyx_kwvalues = __Pyx_KwValues_FASTCALL(__pyx_args, __pyx_nargs);
  {
    PyObject ** const __pyx_pyargnames[] = {&__pyx_mstate_global->__pyx_n_u_allocator,0};
    const Py_ssize_t __pyx_kwds_len = (__pyx_kwds) ? __Pyx_NumKwargs_FASTCALL(__pyx_kwds) : 0;
    if (unlikely(__pyx_kwds_len < 0)) __PYX_ERR(0, 907, __pyx_L3_error)
    if (__pyx_kwds_len > 0) {
      switch (__pyx_nargs) {
        case  1:
        values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(0, 907, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      const Py_ssize_t kwd_pos_args = __pyx_nargs;
      if (__Pyx_ParseKeywords(__pyx_kwds, __pyx_kwvalues, __pyx_pyargnames, 0, values, kwd_pos_args, __pyx_kwds_len, "set_allocator", 0) < (0)) __PYX_ERR(0, 907, __pyx_L3_error)
      if (!values[0]) values[0] = __Pyx_NewRef(((PyObject *)Py_None));
    } else {
      switch (__pyx_nargs) {
        case  1:
        values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(0, 907, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      if (!values[0]) values[0] = __Pyx_NewRef(((PyObject *)Py_None));
    }
    __pyx_v_allocator = values[0];
  }
  goto __pyx_L6_skip;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("set_allocator", 0, 0, 1, __pyx_nargs); __PYX_ERR(0, 907, __pyx_L3_error)
  __pyx_L6_skip:;
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_AddTraceback("cupy.cuda.memory.set_allocator", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_4cupy_4cuda_6memory_16set_allocator(__pyx_self, __pyx_v_allocator);

  /* function exit code */
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4cupy_4cuda_6memory_16set_allocator(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_allocator) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  struct __pyx_opt_args_4cupy_4cuda_6memory_set_allocator __pyx_t_2;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("set_allocator", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_2.__pyx_n = 1;
  __pyx_t_2.allocator = __pyx_v_allocator;
  __pyx_t_1 = __pyx_f_4cupy_4cuda_6memory_set_allocator(1, &__pyx_t_2); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 907, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("cupy.cuda.memory.set_allocator", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "cupy/cuda/memory.pyx":927
 * 
 * 
 * cpdef get_allocator():             # <<<<<<<<<<<<<<
 *     """Returns the current allocator for GPU memory.
 * 
*/

static PyObject *__pyx_pw_4cupy_4cuda_6memory_19get_allocator(PyObject *__pyx_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static PyObject *__pyx_f_4cupy_4cuda_6memory_get_allocator(CYTHON_UNUSED int __pyx_skip_dispatch) {
  PyObject *__pyx_v_allocator = NULL;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  int __pyx_t_5;
  PyObject *__pyx_t_6 = NULL;
  PyObject *__pyx_t_7 = NULL;
  int __pyx_t_8;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("get_allocator", 0);

  /* "cupy/cuda/memory.pyx":933
 *         function: CuPy memory allocator.
 *     """
 *     try:             # <<<<<<<<<<<<<<
 *         allocator = _thread_local.allocator
 *     except AttributeError:
*/
  {
    __Pyx_PyThreadState_declare
    __Pyx_PyThreadState_assign
    __Pyx_ExceptionSave(&__pyx_t_1, &__pyx_t_2, &__pyx_t_3);
    __Pyx_XGOTREF(__pyx_t_1);
    __Pyx_XGOTREF(__pyx_t_2);
    __Pyx_XGOTREF(__pyx_t_3);
    /*try:*/ {

      /* "cupy/cuda/memory.pyx":934
 *     """
 *     try:
 *         allocator = _thread_local.allocator             # <<<<<<<<<<<<<<
 *     except AttributeError:
 *         _thread_local.allocator = allocator = None
*/
      __pyx_t_4 = __Pyx_PyObject_GetAttrStr(__pyx_v_4cupy_4cuda_6memory__thread_local, __pyx_mstate_global->__pyx_n_u_allocator); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 934, __pyx_L3_error)
      __Pyx_GOTREF(__pyx_t_4);
      __pyx_v_allocator = __pyx_t_4;
      __pyx_t_4 = 0;

      /* "cupy/cuda/memory.pyx":933
 *         function: CuPy memory allocator.
 *     """
 *     try:             # <<<<<<<<<<<<<<
 *         allocator = _thread_local.allocator
 *     except AttributeError:
*/
    }
    __Pyx_XDECREF(__pyx_t_1); __pyx_t_1 = 0;
    __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
    goto __pyx_L8_try_end;
    __pyx_L3_error:;
    __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;

    /* "cupy/cuda/memory.pyx":935
 *     try:
 *         allocator = _thread_local.allocator
 *     except AttributeError:             # <<<<<<<<<<<<<<
 *         _thread_local.allocator = allocator = None
 *     if allocator is None:
*/
    __pyx_t_5 = __Pyx_PyErr_ExceptionMatches(((PyObject *)(((PyTypeObject*)PyExc_AttributeError))));
    if (__pyx_t_5) {
      __Pyx_AddTraceback("cupy.cuda.memory.get_allocator", __pyx_clineno, __pyx_lineno, __pyx_filename);
      if (__Pyx_GetException(&__pyx_t_4, &__pyx_t_6, &__pyx_t_7) < 0) __PYX_ERR(0, 935, __pyx_L5_except_error)
      __Pyx_XGOTREF(__pyx_t_4);
      __Pyx_XGOTREF(__pyx_t_6);
      __Pyx_XGOTREF(__pyx_t_7);

      /* "cupy/cuda/memory.pyx":936
 *         allocator = _thread_local.allocator
 *     except AttributeError:
 *         _thread_local.allocator = allocator = None             # <<<<<<<<<<<<<<
 *     if allocator is None:
 *         return _current_allocator
*/
      if (__Pyx_PyObject_SetAttrStr(__pyx_v_4cupy_4cuda_6memory__thread_local, __pyx_mstate_global->__pyx_n_u_allocator, Py_None) < (0)) __PYX_ERR(0, 936, __pyx_L5_except_error)
      __Pyx_INCREF(Py_None);
      __Pyx_XDECREF_SET(__pyx_v_allocator, Py_None);
      __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
      __Pyx_XDECREF(__pyx_t_6); __pyx_t_6 = 0;
      __Pyx_XDECREF(__pyx_t_7); __pyx_t_7 = 0;
      goto __pyx_L4_exception_handled;
    }
    goto __pyx_L5_except_error;

    /* "cupy/cuda/memory.pyx":933
 *         function: CuPy memory allocator.
 *     """
 *     try:             # <<<<<<<<<<<<<<
 *         allocator = _thread_local.allocator
 *     except AttributeError:
*/
    __pyx_L5_except_error:;
    __Pyx_XGIVEREF(__pyx_t_1);
    __Pyx_XGIVEREF(__pyx_t_2);
    __Pyx_XGIVEREF(__pyx_t_3);
    __Pyx_ExceptionReset(__pyx_t_1, __pyx_t_2, __pyx_t_3);
    goto __pyx_L1_error;
    __pyx_L4_exception_handled:;
    __Pyx_XGIVEREF(__pyx_t_1);
    __Pyx_XGIVEREF(__pyx_t_2);
    __Pyx_XGIVEREF(__pyx_t_3);
    __Pyx_ExceptionReset(__pyx_t_1, __pyx_t_2, __pyx_t_3);
    __pyx_L8_try_end:;
  }

  /* "cupy/cuda/memory.pyx":937
 *     except AttributeError:
 *         _thread_local.allocator = allocator = None
 *     if allocator is None:             # <<<<<<<<<<<<<<
 *         return _current_allocator
 *     else:
*/
  __pyx_t_8 = (__pyx_v_allocator == Py_None);
  if (__pyx_t_8) {

    /* "cupy/cuda/memory.pyx":938
 *         _thread_local.allocator = allocator = None
 *     if allocator is None:
 *         return _current_allocator             # <<<<<<<<<<<<<<
 *     else:
 *         return allocator
*/
    __Pyx_XDECREF(__pyx_r);
    __Pyx_INCREF(__pyx_v_4cupy_4cuda_6memory__current_allocator);
    __pyx_r = __pyx_v_4cupy_4cuda_6memory__current_allocator;
    goto __pyx_L0;

    /* "cupy/cuda/memory.pyx":937
 *     except AttributeError:
 *         _thread_local.allocator = allocator = None
 *     if allocator is None:             # <<<<<<<<<<<<<<
 *         return _current_allocator
 *     else:
*/
  }

  /* "cupy/cuda/memory.pyx":940
 *         return _current_allocator
 *     else:
 *         return allocator             # <<<<<<<<<<<<<<
 * 
 * 
*/
  /*else*/ {
    __Pyx_XDECREF(__pyx_r);
    __Pyx_INCREF(__pyx_v_allocator);
    __pyx_r = __pyx_v_allocator;
    goto __pyx_L0;
  }

  /* "cupy/cuda/memory.pyx":927
 * 
 * 
 * cpdef get_allocator():             # <<<<<<<<<<<<<<
 *     """Returns the current allocator for GPU memory.
 * 
*/

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_XDECREF(__pyx_t_7);
  __Pyx_AddTraceback("cupy.cuda.memory.get_allocator", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_allocator);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_pw_4cupy_4cuda_6memory_19get_allocator(PyObject *__pyx_self, CYTHON_UNUSED PyObject *unused); /*proto*/
PyDoc_STRVAR(__pyx_doc_4cupy_4cuda_6memory_18get_allocator, "get_allocator()\n\nReturns the current allocator for GPU memory.\n\nReturns:\n    function: CuPy memory allocator.");
static PyObject *__pyx_pw_4cupy_4cuda_6memory_19get_allocator(PyObject *__pyx_self, CYTHON_UNUSED PyObject *unused) {
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("get_allocator (wrapper)", 0);
  __pyx_kwvalues = __Pyx_KwValues_VARARGS(__pyx_args, __pyx_nargs);
  __pyx_r = __pyx_pf_4cupy_4cuda_6memory_18get_allocator(__pyx_self);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4cupy_4cuda_6memory_18get_allocator(CYTHON_UNUSED PyObject *__pyx_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("get_allocator", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __pyx_f_4cupy_4cuda_6memory_get_allocator(1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 927, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("cupy.cuda.memory.get_allocator", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "cupy/cuda/memory.pyx":960
 *         dict __dict__
 * 
 *     def __init__(self, _Chunk chunk, pool):             # <<<<<<<<<<<<<<
 *         self._init(chunk, pool)
 * 
*/

/* Python wrapper */
static int __pyx_pw_4cupy_4cuda_6memory_12PooledMemory_1__init__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static int __pyx_pw_4cupy_4cuda_6memory_12PooledMemory_1__init__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  struct __pyx_obj_4cupy_4cuda_6memory__Chunk *__pyx_v_chunk = 0;
  PyObject *__pyx_v_pool = 0;
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject* values[2] = {0,0};
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__init__ (wrapper)", 0);
  #if CYTHON_ASSUME_SAFE_SIZE
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return -1;
  #endif
  __pyx_kwvalues = __Pyx_KwValues_VARARGS(__pyx_args, __pyx_nargs);
  {
    PyObject ** const __pyx_pyargnames[] = {&__pyx_mstate_global->__pyx_n_u_chunk,&__pyx_mstate_global->__pyx_n_u_pool,0};
    const Py_ssize_t __pyx_kwds_len = (__pyx_kwds) ? __Pyx_NumKwargs_VARARGS(__pyx_kwds) : 0;
    if (unlikely(__pyx_kwds_len < 0)) __PYX_ERR(0, 960, __pyx_L3_error)
    if (__pyx_kwds_len > 0) {
      switch (__pyx_nargs) {
        case  2:
        values[1] = __Pyx_ArgRef_VARARGS(__pyx_args, 1);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[1])) __PYX_ERR(0, 960, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  1:
        values[0] = __Pyx_ArgRef_VARARGS(__pyx_args, 0);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(0, 960, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      const Py_ssize_t kwd_pos_args = __pyx_nargs;
      if (__Pyx_ParseKeywords(__pyx_kwds, __pyx_kwvalues, __pyx_pyargnames, 0, values, kwd_pos_args, __pyx_kwds_len, "__init__", 0) < (0)) __PYX_ERR(0, 960, __pyx_L3_error)
      for (Py_ssize_t i = __pyx_nargs; i < 2; i++) {
        if (unlikely(!values[i])) { __Pyx_RaiseArgtupleInvalid("__init__", 1, 2, 2, i); __PYX_ERR(0, 960, __pyx_L3_error) }
      }
    } else if (unlikely(__pyx_nargs != 2)) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = __Pyx_ArgRef_VARARGS(__pyx_args, 0);
      if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(0, 960, __pyx_L3_error)
      values[1] = __Pyx_ArgRef_VARARGS(__pyx_args, 1);
      if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[1])) __PYX_ERR(0, 960, __pyx_L3_error)
    }
    __pyx_v_chunk = ((struct __pyx_obj_4cupy_4cuda_6memory__Chunk *)values[0]);
    __pyx_v_pool = values[1];
  }
  goto __pyx_L6_skip;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("__init__", 1, 2, 2, __pyx_nargs); __PYX_ERR(0, 960, __pyx_L3_error)
  __pyx_L6_skip:;
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_AddTraceback("cupy.cuda.memory.PooledMemory.__init__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return -1;
  __pyx_L4_argument_unpacking_done:;
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_chunk), __pyx_mstate_global->__pyx_ptype_4cupy_4cuda_6memory__Chunk, 1, "chunk", 0))) __PYX_ERR(0, 960, __pyx_L1_error)
  __pyx_r = __pyx_pf_4cupy_4cuda_6memory_12PooledMemory___init__(((struct __pyx_obj_4cupy_4cuda_6memory_PooledMemory *)__pyx_v_self), __pyx_v_chunk, __pyx_v_pool);

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = -1;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  goto __pyx_L7_cleaned_up;
  __pyx_L0:;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __pyx_L7_cleaned_up:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_4cupy_4cuda_6memory_12PooledMemory___init__(struct __pyx_obj_4cupy_4cuda_6memory_PooledMemory *__pyx_v_self, struct __pyx_obj_4cupy_4cuda_6memory__Chunk *__pyx_v_chunk, PyObject *__pyx_v_pool) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__init__", 0);

  /* "cupy/cuda/memory.pyx":961
 * 
 *     def __init__(self, _Chunk chunk, pool):
 *         self._init(chunk, pool)             # <<<<<<<<<<<<<<
 * 
 *     cdef _init(self, _Chunk chunk, pool):
*/
  __pyx_t_1 = __pyx_f_4cupy_4cuda_6memory_12PooledMemory__init(__pyx_v_self, __pyx_v_chunk, __pyx_v_pool); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 961, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "cupy/cuda/memory.pyx":960
 *         dict __dict__
 * 
 *     def __init__(self, _Chunk chunk, pool):             # <<<<<<<<<<<<<<
 *         self._init(chunk, pool)
 * 
*/

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("cupy.cuda.memory.PooledMemory.__init__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "cupy/cuda/memory.pyx":963
 *         self._init(chunk, pool)
 * 
 *     cdef _init(self, _Chunk chunk, pool):             # <<<<<<<<<<<<<<
 *         self.chunk = chunk
 *         self.ptr = chunk.ptr()
*/

static PyObject *__pyx_f_4cupy_4cuda_6memory_12PooledMemory__init(struct __pyx_obj_4cupy_4cuda_6memory_PooledMemory *__pyx_v_self, struct __pyx_obj_4cupy_4cuda_6memory__Chunk *__pyx_v_chunk, PyObject *__pyx_v_pool) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  size_t __pyx_t_1;
  int __pyx_t_2;
  PyObject *__pyx_t_3 = NULL;
  int __pyx_t_4;
  PyObject *__pyx_t_5 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("_init", 0);

  /* "cupy/cuda/memory.pyx":964
 * 
 *     cdef _init(self, _Chunk chunk, pool):
 *         self.chunk = chunk             # <<<<<<<<<<<<<<
 *         self.ptr = chunk.ptr()
 *         self.size = chunk.size
*/
  __Pyx_INCREF((PyObject *)__pyx_v_chunk);
  __Pyx_GIVEREF((PyObject *)__pyx_v_chunk);
  __Pyx_GOTREF((PyObject *)__pyx_v_self->chunk);
  __Pyx_DECREF((PyObject *)__pyx_v_self->chunk);
  __pyx_v_self->chunk = __pyx_v_chunk;

  /* "cupy/cuda/memory.pyx":965
 *     cdef _init(self, _Chunk chunk, pool):
 *         self.chunk = chunk
 *         self.ptr = chunk.ptr()             # <<<<<<<<<<<<<<
 *         self.size = chunk.size
 *         self.device_id = chunk.mem.device_id
*/
  __pyx_v_self->__pyx_base.ptr = __pyx_f_4cupy_4cuda_6memory_6_Chunk_ptr(__pyx_v_chunk, 0);

  /* "cupy/cuda/memory.pyx":966
 *         self.chunk = chunk
 *         self.ptr = chunk.ptr()
 *         self.size = chunk.size             # <<<<<<<<<<<<<<
 *         self.device_id = chunk.mem.device_id
 *         self.pool = pool
*/
  __pyx_t_1 = __pyx_v_chunk->size;
  __pyx_v_self->__pyx_base.size = __pyx_t_1;

  /* "cupy/cuda/memory.pyx":967
 *         self.ptr = chunk.ptr()
 *         self.size = chunk.size
 *         self.device_id = chunk.mem.device_id             # <<<<<<<<<<<<<<
 *         self.pool = pool
 * 
*/
  __pyx_t_2 = __pyx_v_chunk->mem->device_id;
  __pyx_v_self->__pyx_base.device_id = __pyx_t_2;

  /* "cupy/cuda/memory.pyx":968
 *         self.size = chunk.size
 *         self.device_id = chunk.mem.device_id
 *         self.pool = pool             # <<<<<<<<<<<<<<
 * 
 *         # we need a way to know what the underlying memory is
*/
  __Pyx_INCREF(__pyx_v_pool);
  __Pyx_GIVEREF(__pyx_v_pool);
  __Pyx_GOTREF(__pyx_v_self->pool);
  __Pyx_DECREF(__pyx_v_self->pool);
  __pyx_v_self->pool = __pyx_v_pool;

  /* "cupy/cuda/memory.pyx":972
 *         # we need a way to know what the underlying memory is
 *         # TODO(leofang): would it be better to do this in MemoryPointer?
 *         if isinstance(chunk.mem, Memory):             # <<<<<<<<<<<<<<
 *             self.identity = "Memory"
 *         elif isinstance(chunk.mem, MemoryAsync):
*/
  __pyx_t_3 = ((PyObject *)__pyx_v_chunk->mem);
  __Pyx_INCREF(__pyx_t_3);
  __pyx_t_4 = __Pyx_TypeCheck(__pyx_t_3, __pyx_mstate_global->__pyx_ptype_4cupy_4cuda_6memory_Memory); 
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  if (__pyx_t_4) {

    /* "cupy/cuda/memory.pyx":973
 *         # TODO(leofang): would it be better to do this in MemoryPointer?
 *         if isinstance(chunk.mem, Memory):
 *             self.identity = "Memory"             # <<<<<<<<<<<<<<
 *         elif isinstance(chunk.mem, MemoryAsync):
 *             self.identity = "MemoryAsync"
*/
    __Pyx_INCREF(__pyx_mstate_global->__pyx_n_u_Memory);
    __Pyx_GIVEREF(__pyx_mstate_global->__pyx_n_u_Memory);
    __Pyx_GOTREF(__pyx_v_self->identity);
    __Pyx_DECREF(__pyx_v_self->identity);
    __pyx_v_self->identity = __pyx_mstate_global->__pyx_n_u_Memory;

    /* "cupy/cuda/memory.pyx":972
 *         # we need a way to know what the underlying memory is
 *         # TODO(leofang): would it be better to do this in MemoryPointer?
 *         if isinstance(chunk.mem, Memory):             # <<<<<<<<<<<<<<
 *             self.identity = "Memory"
 *         elif isinstance(chunk.mem, MemoryAsync):
*/
    goto __pyx_L3;
  }

  /* "cupy/cuda/memory.pyx":974
 *         if isinstance(chunk.mem, Memory):
 *             self.identity = "Memory"
 *         elif isinstance(chunk.mem, MemoryAsync):             # <<<<<<<<<<<<<<
 *             self.identity = "MemoryAsync"
 *         elif isinstance(chunk.mem, UnownedMemory):
*/
  __pyx_t_3 = ((PyObject *)__pyx_v_chunk->mem);
  __Pyx_INCREF(__pyx_t_3);
  __pyx_t_4 = __Pyx_TypeCheck(__pyx_t_3, __pyx_mstate_global->__pyx_ptype_4cupy_4cuda_6memory_MemoryAsync); 
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  if (__pyx_t_4) {

    /* "cupy/cuda/memory.pyx":975
 *             self.identity = "Memory"
 *         elif isinstance(chunk.mem, MemoryAsync):
 *             self.identity = "MemoryAsync"             # <<<<<<<<<<<<<<
 *         elif isinstance(chunk.mem, UnownedMemory):
 *             self.identity = "UnownedMemory"
*/
    __Pyx_INCREF(__pyx_mstate_global->__pyx_n_u_MemoryAsync);
    __Pyx_GIVEREF(__pyx_mstate_global->__pyx_n_u_MemoryAsync);
    __Pyx_GOTREF(__pyx_v_self->identity);
    __Pyx_DECREF(__pyx_v_self->identity);
    __pyx_v_self->identity = __pyx_mstate_global->__pyx_n_u_MemoryAsync;

    /* "cupy/cuda/memory.pyx":974
 *         if isinstance(chunk.mem, Memory):
 *             self.identity = "Memory"
 *         elif isinstance(chunk.mem, MemoryAsync):             # <<<<<<<<<<<<<<
 *             self.identity = "MemoryAsync"
 *         elif isinstance(chunk.mem, UnownedMemory):
*/
    goto __pyx_L3;
  }

  /* "cupy/cuda/memory.pyx":976
 *         elif isinstance(chunk.mem, MemoryAsync):
 *             self.identity = "MemoryAsync"
 *         elif isinstance(chunk.mem, UnownedMemory):             # <<<<<<<<<<<<<<
 *             self.identity = "UnownedMemory"
 *         elif isinstance(chunk.mem, SystemMemory):
*/
  __pyx_t_3 = ((PyObject *)__pyx_v_chunk->mem);
  __Pyx_INCREF(__pyx_t_3);
  __pyx_t_4 = __Pyx_TypeCheck(__pyx_t_3, __pyx_mstate_global->__pyx_ptype_4cupy_4cuda_6memory_UnownedMemory); 
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  if (__pyx_t_4) {

    /* "cupy/cuda/memory.pyx":977
 *             self.identity = "MemoryAsync"
 *         elif isinstance(chunk.mem, UnownedMemory):
 *             self.identity = "UnownedMemory"             # <<<<<<<<<<<<<<
 *         elif isinstance(chunk.mem, SystemMemory):
 *             self.identity = "SystemMemory"
*/
    __Pyx_INCREF(__pyx_mstate_global->__pyx_n_u_UnownedMemory);
    __Pyx_GIVEREF(__pyx_mstate_global->__pyx_n_u_UnownedMemory);
    __Pyx_GOTREF(__pyx_v_self->identity);
    __Pyx_DECREF(__pyx_v_self->identity);
    __pyx_v_self->identity = __pyx_mstate_global->__pyx_n_u_UnownedMemory;

    /* "cupy/cuda/memory.pyx":976
 *         elif isinstance(chunk.mem, MemoryAsync):
 *             self.identity = "MemoryAsync"
 *         elif isinstance(chunk.mem, UnownedMemory):             # <<<<<<<<<<<<<<
 *             self.identity = "UnownedMemory"
 *         elif isinstance(chunk.mem, SystemMemory):
*/
    goto __pyx_L3;
  }

  /* "cupy/cuda/memory.pyx":978
 *         elif isinstance(chunk.mem, UnownedMemory):
 *             self.identity = "UnownedMemory"
 *         elif isinstance(chunk.mem, SystemMemory):             # <<<<<<<<<<<<<<
 *             self.identity = "SystemMemory"
 *             self.prefetch = <SystemMemory>(chunk.mem).prefetch
*/
  __pyx_t_3 = ((PyObject *)__pyx_v_chunk->mem);
  __Pyx_INCREF(__pyx_t_3);
  __pyx_t_4 = __Pyx_TypeCheck(__pyx_t_3, __pyx_mstate_global->__pyx_ptype_4cupy_4cuda_6memory_SystemMemory); 
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  if (__pyx_t_4) {

    /* "cupy/cuda/memory.pyx":979
 *             self.identity = "UnownedMemory"
 *         elif isinstance(chunk.mem, SystemMemory):
 *             self.identity = "SystemMemory"             # <<<<<<<<<<<<<<
 *             self.prefetch = <SystemMemory>(chunk.mem).prefetch
 *             self.advise = <SystemMemory>(chunk.mem).advise
*/
    __Pyx_INCREF(__pyx_mstate_global->__pyx_n_u_SystemMemory);
    __Pyx_GIVEREF(__pyx_mstate_global->__pyx_n_u_SystemMemory);
    __Pyx_GOTREF(__pyx_v_self->identity);
    __Pyx_DECREF(__pyx_v_self->identity);
    __pyx_v_self->identity = __pyx_mstate_global->__pyx_n_u_SystemMemory;

    /* "cupy/cuda/memory.pyx":980
 *         elif isinstance(chunk.mem, SystemMemory):
 *             self.identity = "SystemMemory"
 *             self.prefetch = <SystemMemory>(chunk.mem).prefetch             # <<<<<<<<<<<<<<
 *             self.advise = <SystemMemory>(chunk.mem).advise
 *         elif isinstance(chunk.mem, ManagedMemory):
*/
    __pyx_t_3 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_chunk->mem), __pyx_mstate_global->__pyx_n_u_prefetch); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 980, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_t_5 = __pyx_t_3;
    __Pyx_INCREF(__pyx_t_5);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    if (__Pyx_PyObject_SetAttrStr(((PyObject *)__pyx_v_self), __pyx_mstate_global->__pyx_n_u_prefetch, __pyx_t_5) < (0)) __PYX_ERR(0, 980, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;

    /* "cupy/cuda/memory.pyx":981
 *             self.identity = "SystemMemory"
 *             self.prefetch = <SystemMemory>(chunk.mem).prefetch
 *             self.advise = <SystemMemory>(chunk.mem).advise             # <<<<<<<<<<<<<<
 *         elif isinstance(chunk.mem, ManagedMemory):
 *             self.identity = "ManagedMemory"
*/
    __pyx_t_5 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_chunk->mem), __pyx_mstate_global->__pyx_n_u_advise); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 981, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __pyx_t_3 = __pyx_t_5;
    __Pyx_INCREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
    if (__Pyx_PyObject_SetAttrStr(((PyObject *)__pyx_v_self), __pyx_mstate_global->__pyx_n_u_advise, __pyx_t_3) < (0)) __PYX_ERR(0, 981, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

    /* "cupy/cuda/memory.pyx":978
 *         elif isinstance(chunk.mem, UnownedMemory):
 *             self.identity = "UnownedMemory"
 *         elif isinstance(chunk.mem, SystemMemory):             # <<<<<<<<<<<<<<
 *             self.identity = "SystemMemory"
 *             self.prefetch = <SystemMemory>(chunk.mem).prefetch
*/
    goto __pyx_L3;
  }

  /* "cupy/cuda/memory.pyx":982
 *             self.prefetch = <SystemMemory>(chunk.mem).prefetch
 *             self.advise = <SystemMemory>(chunk.mem).advise
 *         elif isinstance(chunk.mem, ManagedMemory):             # <<<<<<<<<<<<<<
 *             self.identity = "ManagedMemory"
 *             self.prefetch = <ManagedMemory>(chunk.mem).prefetch
*/
  __pyx_t_3 = ((PyObject *)__pyx_v_chunk->mem);
  __Pyx_INCREF(__pyx_t_3);
  __pyx_t_4 = __Pyx_TypeCheck(__pyx_t_3, __pyx_mstate_global->__pyx_ptype_4cupy_4cuda_6memory_ManagedMemory); 
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  if (__pyx_t_4) {

    /* "cupy/cuda/memory.pyx":983
 *             self.advise = <SystemMemory>(chunk.mem).advise
 *         elif isinstance(chunk.mem, ManagedMemory):
 *             self.identity = "ManagedMemory"             # <<<<<<<<<<<<<<
 *             self.prefetch = <ManagedMemory>(chunk.mem).prefetch
 *             self.advise = <ManagedMemory>(chunk.mem).advise
*/
    __Pyx_INCREF(__pyx_mstate_global->__pyx_n_u_ManagedMemory);
    __Pyx_GIVEREF(__pyx_mstate_global->__pyx_n_u_ManagedMemory);
    __Pyx_GOTREF(__pyx_v_self->identity);
    __Pyx_DECREF(__pyx_v_self->identity);
    __pyx_v_self->identity = __pyx_mstate_global->__pyx_n_u_ManagedMemory;

    /* "cupy/cuda/memory.pyx":984
 *         elif isinstance(chunk.mem, ManagedMemory):
 *             self.identity = "ManagedMemory"
 *             self.prefetch = <ManagedMemory>(chunk.mem).prefetch             # <<<<<<<<<<<<<<
 *             self.advise = <ManagedMemory>(chunk.mem).advise
 *         elif isinstance(chunk.mem, CFunctionAllocatorMemory):
*/
    __pyx_t_3 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_chunk->mem), __pyx_mstate_global->__pyx_n_u_prefetch); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 984, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_t_5 = __pyx_t_3;
    __Pyx_INCREF(__pyx_t_5);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    if (__Pyx_PyObject_SetAttrStr(((PyObject *)__pyx_v_self), __pyx_mstate_global->__pyx_n_u_prefetch, __pyx_t_5) < (0)) __PYX_ERR(0, 984, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;

    /* "cupy/cuda/memory.pyx":985
 *             self.identity = "ManagedMemory"
 *             self.prefetch = <ManagedMemory>(chunk.mem).prefetch
 *             self.advise = <ManagedMemory>(chunk.mem).advise             # <<<<<<<<<<<<<<
 *         elif isinstance(chunk.mem, CFunctionAllocatorMemory):
 *             self.identity = "CFunctionAllocatorMemory"
*/
    __pyx_t_5 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_chunk->mem), __pyx_mstate_global->__pyx_n_u_advise); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 985, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __pyx_t_3 = __pyx_t_5;
    __Pyx_INCREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
    if (__Pyx_PyObject_SetAttrStr(((PyObject *)__pyx_v_self), __pyx_mstate_global->__pyx_n_u_advise, __pyx_t_3) < (0)) __PYX_ERR(0, 985, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

    /* "cupy/cuda/memory.pyx":982
 *             self.prefetch = <SystemMemory>(chunk.mem).prefetch
 *             self.advise = <SystemMemory>(chunk.mem).advise
 *         elif isinstance(chunk.mem, ManagedMemory):             # <<<<<<<<<<<<<<
 *             self.identity = "ManagedMemory"
 *             self.prefetch = <ManagedMemory>(chunk.mem).prefetch
*/
    goto __pyx_L3;
  }

  /* "cupy/cuda/memory.pyx":986
 *             self.prefetch = <ManagedMemory>(chunk.mem).prefetch
 *             self.advise = <ManagedMemory>(chunk.mem).advise
 *         elif isinstance(chunk.mem, CFunctionAllocatorMemory):             # <<<<<<<<<<<<<<
 *             self.identity = "CFunctionAllocatorMemory"
 *         elif isinstance(chunk.mem, PythonFunctionAllocatorMemory):
*/
  __pyx_t_3 = ((PyObject *)__pyx_v_chunk->mem);
  __Pyx_INCREF(__pyx_t_3);
  __pyx_t_4 = __Pyx_TypeCheck(__pyx_t_3, __pyx_mstate_global->__pyx_ptype_4cupy_4cuda_6memory_CFunctionAllocatorMemory); 
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  if (__pyx_t_4) {

    /* "cupy/cuda/memory.pyx":987
 *             self.advise = <ManagedMemory>(chunk.mem).advise
 *         elif isinstance(chunk.mem, CFunctionAllocatorMemory):
 *             self.identity = "CFunctionAllocatorMemory"             # <<<<<<<<<<<<<<
 *         elif isinstance(chunk.mem, PythonFunctionAllocatorMemory):
 *             self.identity = "PythonFunctionAllocatorMemory"
*/
    __Pyx_INCREF(__pyx_mstate_global->__pyx_n_u_CFunctionAllocatorMemory);
    __Pyx_GIVEREF(__pyx_mstate_global->__pyx_n_u_CFunctionAllocatorMemory);
    __Pyx_GOTREF(__pyx_v_self->identity);
    __Pyx_DECREF(__pyx_v_self->identity);
    __pyx_v_self->identity = __pyx_mstate_global->__pyx_n_u_CFunctionAllocatorMemory;

    /* "cupy/cuda/memory.pyx":986
 *             self.prefetch = <ManagedMemory>(chunk.mem).prefetch
 *             self.advise = <ManagedMemory>(chunk.mem).advise
 *         elif isinstance(chunk.mem, CFunctionAllocatorMemory):             # <<<<<<<<<<<<<<
 *             self.identity = "CFunctionAllocatorMemory"
 *         elif isinstance(chunk.mem, PythonFunctionAllocatorMemory):
*/
    goto __pyx_L3;
  }

  /* "cupy/cuda/memory.pyx":988
 *         elif isinstance(chunk.mem, CFunctionAllocatorMemory):
 *             self.identity = "CFunctionAllocatorMemory"
 *         elif isinstance(chunk.mem, PythonFunctionAllocatorMemory):             # <<<<<<<<<<<<<<
 *             self.identity = "PythonFunctionAllocatorMemory"
 *         else:
*/
  __pyx_t_3 = ((PyObject *)__pyx_v_chunk->mem);
  __Pyx_INCREF(__pyx_t_3);
  __pyx_t_4 = __Pyx_TypeCheck(__pyx_t_3, __pyx_mstate_global->__pyx_ptype_4cupy_4cuda_6memory_PythonFunctionAllocatorMemory); 
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  if (__pyx_t_4) {

    /* "cupy/cuda/memory.pyx":989
 *             self.identity = "CFunctionAllocatorMemory"
 *         elif isinstance(chunk.mem, PythonFunctionAllocatorMemory):
 *             self.identity = "PythonFunctionAllocatorMemory"             # <<<<<<<<<<<<<<
 *         else:
 *             self.identity = "<unknown>"
*/
    __Pyx_INCREF(__pyx_mstate_global->__pyx_n_u_PythonFunctionAllocatorMemory);
    __Pyx_GIVEREF(__pyx_mstate_global->__pyx_n_u_PythonFunctionAllocatorMemory);
    __Pyx_GOTREF(__pyx_v_self->identity);
    __Pyx_DECREF(__pyx_v_self->identity);
    __pyx_v_self->identity = __pyx_mstate_global->__pyx_n_u_PythonFunctionAllocatorMemory;

    /* "cupy/cuda/memory.pyx":988
 *         elif isinstance(chunk.mem, CFunctionAllocatorMemory):
 *             self.identity = "CFunctionAllocatorMemory"
 *         elif isinstance(chunk.mem, PythonFunctionAllocatorMemory):             # <<<<<<<<<<<<<<
 *             self.identity = "PythonFunctionAllocatorMemory"
 *         else:
*/
    goto __pyx_L3;
  }

  /* "cupy/cuda/memory.pyx":991
 *             self.identity = "PythonFunctionAllocatorMemory"
 *         else:
 *             self.identity = "<unknown>"             # <<<<<<<<<<<<<<
 * 
 *     cpdef free(self):
*/
  /*else*/ {
    __Pyx_INCREF(__pyx_mstate_global->__pyx_kp_u_unknown);
    __Pyx_GIVEREF(__pyx_mstate_global->__pyx_kp_u_unknown);
    __Pyx_GOTREF(__pyx_v_self->identity);
    __Pyx_DECREF(__pyx_v_self->identity);
    __pyx_v_self->identity = __pyx_mstate_global->__pyx_kp_u_unknown;
  }
  __pyx_L3:;

  /* "cupy/cuda/memory.pyx":963
 *         self._init(chunk, pool)
 * 
 *     cdef _init(self, _Chunk chunk, pool):             # <<<<<<<<<<<<<<
 *         self.chunk = chunk
 *         self.ptr = chunk.ptr()
*/

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_AddTraceback("cupy.cuda.memory.PooledMemory._init", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "cupy/cuda/memory.pyx":993
 *             self.identity = "<unknown>"
 * 
 *     cpdef free(self):             # <<<<<<<<<<<<<<
 *         """Frees the memory buffer and returns it to the memory pool.
 * 
*/

static PyObject *__pyx_pw_4cupy_4cuda_6memory_12PooledMemory_3free(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
static PyObject *__pyx_f_4cupy_4cuda_6memory_12PooledMemory_free(struct __pyx_obj_4cupy_4cuda_6memory_PooledMemory *__pyx_v_self, CYTHON_UNUSED int __pyx_skip_dispatch) {
  PyObject *__pyx_v_pool_ref;
  intptr_t __pyx_v_ptr;
  PyObject *__pyx_v_pool = NULL;
  size_t __pyx_v_size;
  PyObject *__pyx_v_hooks = NULL;
  int __pyx_v_device_id;
  PyObject *__pyx_v_pmem_id = NULL;
  PyObject *__pyx_v_hook = NULL;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  intptr_t __pyx_t_1;
  int __pyx_t_2;
  PyObject *__pyx_t_3 = NULL;
  size_t __pyx_t_4;
  int __pyx_t_5;
  PyObject *__pyx_t_6 = NULL;
  Py_ssize_t __pyx_t_7;
  Py_ssize_t __pyx_t_8;
  int __pyx_t_9;
  PyObject *__pyx_t_10 = NULL;
  PyObject *__pyx_t_11 = NULL;
  PyObject *__pyx_t_12 = NULL;
  PyObject *__pyx_t_13 = NULL;
  PyObject *__pyx_t_14 = NULL;
  char const *__pyx_t_15;
  PyObject *__pyx_t_16 = NULL;
  PyObject *__pyx_t_17 = NULL;
  PyObject *__pyx_t_18 = NULL;
  PyObject *__pyx_t_19 = NULL;
  PyObject *__pyx_t_20 = NULL;
  PyObject *__pyx_t_21 = NULL;
  int __pyx_t_22;
  int __pyx_t_23;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("free", 0);

  /* "cupy/cuda/memory.pyx":1002
 *         cdef cpython.PyObject *pool_ref
 *         cdef intptr_t ptr
 *         ptr = self.ptr             # <<<<<<<<<<<<<<
 *         if ptr == 0:
 *             return
*/
  __pyx_t_1 = __pyx_v_self->__pyx_base.ptr;
  __pyx_v_ptr = __pyx_t_1;

  /* "cupy/cuda/memory.pyx":1003
 *         cdef intptr_t ptr
 *         ptr = self.ptr
 *         if ptr == 0:             # <<<<<<<<<<<<<<
 *             return
 *         self.ptr = 0
*/
  __pyx_t_2 = (__pyx_v_ptr == 0);
  if (__pyx_t_2) {

    /* "cupy/cuda/memory.pyx":1004
 *         ptr = self.ptr
 *         if ptr == 0:
 *             return             # <<<<<<<<<<<<<<
 *         self.ptr = 0
 *         if cpython.PyWeakref_GetRef(self.pool, &pool_ref) == 1:
*/
    __Pyx_XDECREF(__pyx_r);
    __pyx_r = Py_None; __Pyx_INCREF(Py_None);
    goto __pyx_L0;

    /* "cupy/cuda/memory.pyx":1003
 *         cdef intptr_t ptr
 *         ptr = self.ptr
 *         if ptr == 0:             # <<<<<<<<<<<<<<
 *             return
 *         self.ptr = 0
*/
  }

  /* "cupy/cuda/memory.pyx":1005
 *         if ptr == 0:
 *             return
 *         self.ptr = 0             # <<<<<<<<<<<<<<
 *         if cpython.PyWeakref_GetRef(self.pool, &pool_ref) == 1:
 *             pool = <object>pool_ref
*/
  __pyx_v_self->__pyx_base.ptr = 0;

  /* "cupy/cuda/memory.pyx":1006
 *             return
 *         self.ptr = 0
 *         if cpython.PyWeakref_GetRef(self.pool, &pool_ref) == 1:             # <<<<<<<<<<<<<<
 *             pool = <object>pool_ref
 *             cpython.Py_DECREF(pool)  # pool_ref owned a reference
*/
  __pyx_t_3 = __pyx_v_self->pool;
  __Pyx_INCREF(__pyx_t_3);
  __pyx_t_2 = (__Pyx_PyWeakref_GetRef(__pyx_t_3, (&__pyx_v_pool_ref)) == 1);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  if (__pyx_t_2) {

    /* "cupy/cuda/memory.pyx":1007
 *         self.ptr = 0
 *         if cpython.PyWeakref_GetRef(self.pool, &pool_ref) == 1:
 *             pool = <object>pool_ref             # <<<<<<<<<<<<<<
 *             cpython.Py_DECREF(pool)  # pool_ref owned a reference
 *         else:
*/
    __pyx_t_3 = ((PyObject *)__pyx_v_pool_ref);
    __Pyx_INCREF(__pyx_t_3);
    __pyx_v_pool = __pyx_t_3;
    __pyx_t_3 = 0;

    /* "cupy/cuda/memory.pyx":1008
 *         if cpython.PyWeakref_GetRef(self.pool, &pool_ref) == 1:
 *             pool = <object>pool_ref
 *             cpython.Py_DECREF(pool)  # pool_ref owned a reference             # <<<<<<<<<<<<<<
 *         else:
 *             return  # The pool does not exist anymore
*/
    Py_DECREF(__pyx_v_pool);

    /* "cupy/cuda/memory.pyx":1006
 *             return
 *         self.ptr = 0
 *         if cpython.PyWeakref_GetRef(self.pool, &pool_ref) == 1:             # <<<<<<<<<<<<<<
 *             pool = <object>pool_ref
 *             cpython.Py_DECREF(pool)  # pool_ref owned a reference
*/
    goto __pyx_L4;
  }

  /* "cupy/cuda/memory.pyx":1010
 *             cpython.Py_DECREF(pool)  # pool_ref owned a reference
 *         else:
 *             return  # The pool does not exist anymore             # <<<<<<<<<<<<<<
 * 
 *         size = self.size
*/
  /*else*/ {
    __Pyx_XDECREF(__pyx_r);
    __pyx_r = Py_None; __Pyx_INCREF(Py_None);
    goto __pyx_L0;
  }
  __pyx_L4:;

  /* "cupy/cuda/memory.pyx":1012
 *             return  # The pool does not exist anymore
 * 
 *         size = self.size             # <<<<<<<<<<<<<<
 *         if memory_hook._has_memory_hooks():
 *             hooks = memory_hook.get_memory_hooks()
*/
  __pyx_t_4 = __pyx_v_self->__pyx_base.size;
  __pyx_v_size = __pyx_t_4;

  /* "cupy/cuda/memory.pyx":1013
 * 
 *         size = self.size
 *         if memory_hook._has_memory_hooks():             # <<<<<<<<<<<<<<
 *             hooks = memory_hook.get_memory_hooks()
 *             if hooks:
*/
  __pyx_t_2 = __pyx_f_4cupy_4cuda_11memory_hook__has_memory_hooks(0); if (unlikely(__pyx_t_2 == ((int)-1))) __PYX_ERR(0, 1013, __pyx_L1_error)
  if (__pyx_t_2) {

    /* "cupy/cuda/memory.pyx":1014
 *         size = self.size
 *         if memory_hook._has_memory_hooks():
 *             hooks = memory_hook.get_memory_hooks()             # <<<<<<<<<<<<<<
 *             if hooks:
 *                 device_id = self.device_id
*/
    __pyx_t_3 = __pyx_f_4cupy_4cuda_11memory_hook_get_memory_hooks(0); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 1014, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_v_hooks = __pyx_t_3;
    __pyx_t_3 = 0;

    /* "cupy/cuda/memory.pyx":1015
 *         if memory_hook._has_memory_hooks():
 *             hooks = memory_hook.get_memory_hooks()
 *             if hooks:             # <<<<<<<<<<<<<<
 *                 device_id = self.device_id
 *                 pmem_id = id(self)
*/
    __pyx_t_2 = __Pyx_PyObject_IsTrue(__pyx_v_hooks); if (unlikely((__pyx_t_2 < 0))) __PYX_ERR(0, 1015, __pyx_L1_error)
    if (__pyx_t_2) {

      /* "cupy/cuda/memory.pyx":1016
 *             hooks = memory_hook.get_memory_hooks()
 *             if hooks:
 *                 device_id = self.device_id             # <<<<<<<<<<<<<<
 *                 pmem_id = id(self)
 * 
*/
      __pyx_t_5 = __pyx_v_self->__pyx_base.device_id;
      __pyx_v_device_id = __pyx_t_5;

      /* "cupy/cuda/memory.pyx":1017
 *             if hooks:
 *                 device_id = self.device_id
 *                 pmem_id = id(self)             # <<<<<<<<<<<<<<
 * 
 *                 for hook in hooks.values():
*/
      __pyx_t_6 = NULL;
      __pyx_t_4 = 1;
      {
        PyObject *__pyx_callargs[2] = {__pyx_t_6, ((PyObject *)__pyx_v_self)};
        __pyx_t_3 = __Pyx_PyObject_FastCall((PyObject*)__pyx_builtin_id, __pyx_callargs+__pyx_t_4, (2-__pyx_t_4) | (__pyx_t_4*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
        __Pyx_XDECREF(__pyx_t_6); __pyx_t_6 = 0;
        if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 1017, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_3);
      }
      __pyx_v_pmem_id = __pyx_t_3;
      __pyx_t_3 = 0;

      /* "cupy/cuda/memory.pyx":1019
 *                 pmem_id = id(self)
 * 
 *                 for hook in hooks.values():             # <<<<<<<<<<<<<<
 *                     hook.free_preprocess(device_id=device_id,
 *                                          mem_size=size,
*/
      __pyx_t_7 = 0;
      if (unlikely(__pyx_v_hooks == Py_None)) {
        PyErr_Format(PyExc_AttributeError, "'NoneType' object has no attribute '%.30s'", "values");
        __PYX_ERR(0, 1019, __pyx_L1_error)
      }
      __pyx_t_6 = __Pyx_dict_iterator(__pyx_v_hooks, 0, __pyx_mstate_global->__pyx_n_u_values, (&__pyx_t_8), (&__pyx_t_5)); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 1019, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_6);
      __Pyx_XDECREF(__pyx_t_3);
      __pyx_t_3 = __pyx_t_6;
      __pyx_t_6 = 0;
      while (1) {
        __pyx_t_9 = __Pyx_dict_iter_next(__pyx_t_3, __pyx_t_8, &__pyx_t_7, NULL, &__pyx_t_6, NULL, __pyx_t_5);
        if (unlikely(__pyx_t_9 == 0)) break;
        if (unlikely(__pyx_t_9 == -1)) __PYX_ERR(0, 1019, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_6);
        __Pyx_XDECREF_SET(__pyx_v_hook, __pyx_t_6);
        __pyx_t_6 = 0;

        /* "cupy/cuda/memory.pyx":1020
 * 
 *                 for hook in hooks.values():
 *                     hook.free_preprocess(device_id=device_id,             # <<<<<<<<<<<<<<
 *                                          mem_size=size,
 *                                          mem_ptr=ptr,
*/
        __pyx_t_10 = __pyx_v_hook;
        __Pyx_INCREF(__pyx_t_10);
        __pyx_t_11 = __Pyx_PyLong_From_int(__pyx_v_device_id); if (unlikely(!__pyx_t_11)) __PYX_ERR(0, 1020, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_11);

        /* "cupy/cuda/memory.pyx":1021
 *                 for hook in hooks.values():
 *                     hook.free_preprocess(device_id=device_id,
 *                                          mem_size=size,             # <<<<<<<<<<<<<<
 *                                          mem_ptr=ptr,
 *                                          pmem_id=pmem_id)
*/
        __pyx_t_12 = __Pyx_PyLong_FromSize_t(__pyx_v_size); if (unlikely(!__pyx_t_12)) __PYX_ERR(0, 1021, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_12);

        /* "cupy/cuda/memory.pyx":1022
 *                     hook.free_preprocess(device_id=device_id,
 *                                          mem_size=size,
 *                                          mem_ptr=ptr,             # <<<<<<<<<<<<<<
 *                                          pmem_id=pmem_id)
 *                 try:
*/
        __pyx_t_13 = PyLong_FromSsize_t(__pyx_v_ptr); if (unlikely(!__pyx_t_13)) __PYX_ERR(0, 1022, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_13);

        /* "cupy/cuda/memory.pyx":1023
 *                                          mem_size=size,
 *                                          mem_ptr=ptr,
 *                                          pmem_id=pmem_id)             # <<<<<<<<<<<<<<
 *                 try:
 *                     (<SingleDeviceMemoryPool>pool).free(self.chunk)
*/
        __pyx_t_4 = 0;
        {
          PyObject *__pyx_callargs[2 + ((CYTHON_VECTORCALL) ? 4 : 0)] = {__pyx_t_10, NULL};
          __pyx_t_14 = __Pyx_MakeVectorcallBuilderKwds(4); if (unlikely(!__pyx_t_14)) __PYX_ERR(0, 1020, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_14);
          if (__Pyx_VectorcallBuilder_AddArg(__pyx_mstate_global->__pyx_n_u_device_id, __pyx_t_11, __pyx_t_14, __pyx_callargs+1, 0) < (0)) __PYX_ERR(0, 1020, __pyx_L1_error)
          if (__Pyx_VectorcallBuilder_AddArg(__pyx_mstate_global->__pyx_n_u_mem_size, __pyx_t_12, __pyx_t_14, __pyx_callargs+1, 1) < (0)) __PYX_ERR(0, 1020, __pyx_L1_error)
          if (__Pyx_VectorcallBuilder_AddArg(__pyx_mstate_global->__pyx_n_u_mem_ptr, __pyx_t_13, __pyx_t_14, __pyx_callargs+1, 2) < (0)) __PYX_ERR(0, 1020, __pyx_L1_error)
          if (__Pyx_VectorcallBuilder_AddArg(__pyx_mstate_global->__pyx_n_u_pmem_id, __pyx_v_pmem_id, __pyx_t_14, __pyx_callargs+1, 3) < (0)) __PYX_ERR(0, 1020, __pyx_L1_error)
          __pyx_t_6 = __Pyx_Object_VectorcallMethod_CallFromBuilder((PyObject*)__pyx_mstate_global->__pyx_n_u_free_preprocess, __pyx_callargs+__pyx_t_4, (1-__pyx_t_4) | (1*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET), __pyx_t_14);
          __Pyx_XDECREF(__pyx_t_10); __pyx_t_10 = 0;
          __Pyx_DECREF(__pyx_t_11); __pyx_t_11 = 0;
          __Pyx_DECREF(__pyx_t_12); __pyx_t_12 = 0;
          __Pyx_DECREF(__pyx_t_13); __pyx_t_13 = 0;
          __Pyx_DECREF(__pyx_t_14); __pyx_t_14 = 0;
          if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 1020, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_6);
        }
        __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
      }
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

      /* "cupy/cuda/memory.pyx":1024
 *                                          mem_ptr=ptr,
 *                                          pmem_id=pmem_id)
 *                 try:             # <<<<<<<<<<<<<<
 *                     (<SingleDeviceMemoryPool>pool).free(self.chunk)
 *                 finally:
*/
      /*try:*/ {

        /* "cupy/cuda/memory.pyx":1025
 *                                          pmem_id=pmem_id)
 *                 try:
 *                     (<SingleDeviceMemoryPool>pool).free(self.chunk)             # <<<<<<<<<<<<<<
 *                 finally:
 *                     for hook in hooks.values():
*/
        __pyx_t_3 = ((PyObject *)__pyx_v_self->chunk);
        __Pyx_INCREF(__pyx_t_3);
        __pyx_t_6 = __pyx_f_4cupy_4cuda_6memory_22SingleDeviceMemoryPool_free(((struct __pyx_obj_4cupy_4cuda_6memory_SingleDeviceMemoryPool *)__pyx_v_pool), ((struct __pyx_obj_4cupy_4cuda_6memory__Chunk *)__pyx_t_3)); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 1025, __pyx_L10_error)
        __Pyx_GOTREF(__pyx_t_6);
        __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
        __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
      }

      /* "cupy/cuda/memory.pyx":1027
 *                     (<SingleDeviceMemoryPool>pool).free(self.chunk)
 *                 finally:
 *                     for hook in hooks.values():             # <<<<<<<<<<<<<<
 *                         hook.free_postprocess(device_id=device_id,
 *                                               mem_size=size,
*/
      /*finally:*/ {
        /*normal exit:*/{
          __pyx_t_8 = 0;
          if (unlikely(__pyx_v_hooks == Py_None)) {
            PyErr_Format(PyExc_AttributeError, "'NoneType' object has no attribute '%.30s'", "values");
            __PYX_ERR(0, 1027, __pyx_L1_error)
          }
          __pyx_t_3 = __Pyx_dict_iterator(__pyx_v_hooks, 0, __pyx_mstate_global->__pyx_n_u_values, (&__pyx_t_7), (&__pyx_t_5)); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 1027, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_3);
          __Pyx_XDECREF(__pyx_t_6);
          __pyx_t_6 = __pyx_t_3;
          __pyx_t_3 = 0;
          while (1) {
            __pyx_t_9 = __Pyx_dict_iter_next(__pyx_t_6, __pyx_t_7, &__pyx_t_8, NULL, &__pyx_t_3, NULL, __pyx_t_5);
            if (unlikely(__pyx_t_9 == 0)) break;
            if (unlikely(__pyx_t_9 == -1)) __PYX_ERR(0, 1027, __pyx_L1_error)
            __Pyx_GOTREF(__pyx_t_3);
            __Pyx_XDECREF_SET(__pyx_v_hook, __pyx_t_3);
            __pyx_t_3 = 0;

            /* "cupy/cuda/memory.pyx":1028
 *                 finally:
 *                     for hook in hooks.values():
 *                         hook.free_postprocess(device_id=device_id,             # <<<<<<<<<<<<<<
 *                                               mem_size=size,
 *                                               mem_ptr=ptr,
*/
            __pyx_t_14 = __pyx_v_hook;
            __Pyx_INCREF(__pyx_t_14);
            __pyx_t_13 = __Pyx_PyLong_From_int(__pyx_v_device_id); if (unlikely(!__pyx_t_13)) __PYX_ERR(0, 1028, __pyx_L1_error)
            __Pyx_GOTREF(__pyx_t_13);

            /* "cupy/cuda/memory.pyx":1029
 *                     for hook in hooks.values():
 *                         hook.free_postprocess(device_id=device_id,
 *                                               mem_size=size,             # <<<<<<<<<<<<<<
 *                                               mem_ptr=ptr,
 *                                               pmem_id=pmem_id)
*/
            __pyx_t_12 = __Pyx_PyLong_FromSize_t(__pyx_v_size); if (unlikely(!__pyx_t_12)) __PYX_ERR(0, 1029, __pyx_L1_error)
            __Pyx_GOTREF(__pyx_t_12);

            /* "cupy/cuda/memory.pyx":1030
 *                         hook.free_postprocess(device_id=device_id,
 *                                               mem_size=size,
 *                                               mem_ptr=ptr,             # <<<<<<<<<<<<<<
 *                                               pmem_id=pmem_id)
 *                 return
*/
            __pyx_t_11 = PyLong_FromSsize_t(__pyx_v_ptr); if (unlikely(!__pyx_t_11)) __PYX_ERR(0, 1030, __pyx_L1_error)
            __Pyx_GOTREF(__pyx_t_11);

            /* "cupy/cuda/memory.pyx":1031
 *                                               mem_size=size,
 *                                               mem_ptr=ptr,
 *                                               pmem_id=pmem_id)             # <<<<<<<<<<<<<<
 *                 return
 *         (<SingleDeviceMemoryPool>pool).free(self.chunk)
*/
            __pyx_t_4 = 0;
            {
              PyObject *__pyx_callargs[2 + ((CYTHON_VECTORCALL) ? 4 : 0)] = {__pyx_t_14, NULL};
              __pyx_t_10 = __Pyx_MakeVectorcallBuilderKwds(4); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 1028, __pyx_L1_error)
              __Pyx_GOTREF(__pyx_t_10);
              if (__Pyx_VectorcallBuilder_AddArg(__pyx_mstate_global->__pyx_n_u_device_id, __pyx_t_13, __pyx_t_10, __pyx_callargs+1, 0) < (0)) __PYX_ERR(0, 1028, __pyx_L1_error)
              if (__Pyx_VectorcallBuilder_AddArg(__pyx_mstate_global->__pyx_n_u_mem_size, __pyx_t_12, __pyx_t_10, __pyx_callargs+1, 1) < (0)) __PYX_ERR(0, 1028, __pyx_L1_error)
              if (__Pyx_VectorcallBuilder_AddArg(__pyx_mstate_global->__pyx_n_u_mem_ptr, __pyx_t_11, __pyx_t_10, __pyx_callargs+1, 2) < (0)) __PYX_ERR(0, 1028, __pyx_L1_error)
              if (__Pyx_VectorcallBuilder_AddArg(__pyx_mstate_global->__pyx_n_u_pmem_id, __pyx_v_pmem_id, __pyx_t_10, __pyx_callargs+1, 3) < (0)) __PYX_ERR(0, 1028, __pyx_L1_error)
              __pyx_t_3 = __Pyx_Object_VectorcallMethod_CallFromBuilder((PyObject*)__pyx_mstate_global->__pyx_n_u_free_postprocess, __pyx_callargs+__pyx_t_4, (1-__pyx_t_4) | (1*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET), __pyx_t_10);
              __Pyx_XDECREF(__pyx_t_14); __pyx_t_14 = 0;
              __Pyx_DECREF(__pyx_t_13); __pyx_t_13 = 0;
              __Pyx_DECREF(__pyx_t_12); __pyx_t_12 = 0;
              __Pyx_DECREF(__pyx_t_11); __pyx_t_11 = 0;
              __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
              if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 1028, __pyx_L1_error)
              __Pyx_GOTREF(__pyx_t_3);
            }
            __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
          }
          __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
          goto __pyx_L11;
        }
        __pyx_L10_error:;
        /*exception exit:*/{
          __Pyx_PyThreadState_declare
          __Pyx_PyThreadState_assign
          __pyx_t_16 = 0; __pyx_t_17 = 0; __pyx_t_18 = 0; __pyx_t_19 = 0; __pyx_t_20 = 0; __pyx_t_21 = 0;
          __Pyx_XDECREF(__pyx_t_10); __pyx_t_10 = 0;
          __Pyx_XDECREF(__pyx_t_11); __pyx_t_11 = 0;
          __Pyx_XDECREF(__pyx_t_12); __pyx_t_12 = 0;
          __Pyx_XDECREF(__pyx_t_13); __pyx_t_13 = 0;
          __Pyx_XDECREF(__pyx_t_14); __pyx_t_14 = 0;
          __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
          __Pyx_XDECREF(__pyx_t_6); __pyx_t_6 = 0;
           __Pyx_ExceptionSwap(&__pyx_t_19, &__pyx_t_20, &__pyx_t_21);
          if ( unlikely(__Pyx_GetException(&__pyx_t_16, &__pyx_t_17, &__pyx_t_18) < 0)) __Pyx_ErrFetch(&__pyx_t_16, &__pyx_t_17, &__pyx_t_18);
          __Pyx_XGOTREF(__pyx_t_16);
          __Pyx_XGOTREF(__pyx_t_17);
          __Pyx_XGOTREF(__pyx_t_18);
          __Pyx_XGOTREF(__pyx_t_19);
          __Pyx_XGOTREF(__pyx_t_20);
          __Pyx_XGOTREF(__pyx_t_21);
          __pyx_t_5 = __pyx_lineno; __pyx_t_9 = __pyx_clineno; __pyx_t_15 = __pyx_filename;
          {

            /* "cupy/cuda/memory.pyx":1027
 *                     (<SingleDeviceMemoryPool>pool).free(self.chunk)
 *                 finally:
 *                     for hook in hooks.values():             # <<<<<<<<<<<<<<
 *                         hook.free_postprocess(device_id=device_id,
 *                                               mem_size=size,
*/
            __pyx_t_7 = 0;
            if (unlikely(__pyx_v_hooks == Py_None)) {
              PyErr_Format(PyExc_AttributeError, "'NoneType' object has no attribute '%.30s'", "values");
              __PYX_ERR(0, 1027, __pyx_L15_error)
            }
            __pyx_t_3 = __Pyx_dict_iterator(__pyx_v_hooks, 0, __pyx_mstate_global->__pyx_n_u_values, (&__pyx_t_8), (&__pyx_t_22)); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 1027, __pyx_L15_error)
            __Pyx_GOTREF(__pyx_t_3);
            __Pyx_XDECREF(__pyx_t_6);
            __pyx_t_6 = __pyx_t_3;
            __pyx_t_3 = 0;
            while (1) {
              __pyx_t_23 = __Pyx_dict_iter_next(__pyx_t_6, __pyx_t_8, &__pyx_t_7, NULL, &__pyx_t_3, NULL, __pyx_t_22);
              if (unlikely(__pyx_t_23 == 0)) break;
              if (unlikely(__pyx_t_23 == -1)) __PYX_ERR(0, 1027, __pyx_L15_error)
              __Pyx_GOTREF(__pyx_t_3);
              __Pyx_XDECREF_SET(__pyx_v_hook, __pyx_t_3);
              __pyx_t_3 = 0;

              /* "cupy/cuda/memory.pyx":1028
 *                 finally:
 *                     for hook in hooks.values():
 *                         hook.free_postprocess(device_id=device_id,             # <<<<<<<<<<<<<<
 *                                               mem_size=size,
 *                                               mem_ptr=ptr,
*/
              __pyx_t_10 = __pyx_v_hook;
              __Pyx_INCREF(__pyx_t_10);
              __pyx_t_11 = __Pyx_PyLong_From_int(__pyx_v_device_id); if (unlikely(!__pyx_t_11)) __PYX_ERR(0, 1028, __pyx_L15_error)
              __Pyx_GOTREF(__pyx_t_11);

              /* "cupy/cuda/memory.pyx":1029
 *                     for hook in hooks.values():
 *                         hook.free_postprocess(device_id=device_id,
 *                                               mem_size=size,             # <<<<<<<<<<<<<<
 *                                               mem_ptr=ptr,
 *                                               pmem_id=pmem_id)
*/
              __pyx_t_12 = __Pyx_PyLong_FromSize_t(__pyx_v_size); if (unlikely(!__pyx_t_12)) __PYX_ERR(0, 1029, __pyx_L15_error)
              __Pyx_GOTREF(__pyx_t_12);

              /* "cupy/cuda/memory.pyx":1030
 *                         hook.free_postprocess(device_id=device_id,
 *                                               mem_size=size,
 *                                               mem_ptr=ptr,             # <<<<<<<<<<<<<<
 *                                               pmem_id=pmem_id)
 *                 return
*/
              __pyx_t_13 = PyLong_FromSsize_t(__pyx_v_ptr); if (unlikely(!__pyx_t_13)) __PYX_ERR(0, 1030, __pyx_L15_error)
              __Pyx_GOTREF(__pyx_t_13);

              /* "cupy/cuda/memory.pyx":1031
 *                                               mem_size=size,
 *                                               mem_ptr=ptr,
 *                                               pmem_id=pmem_id)             # <<<<<<<<<<<<<<
 *                 return
 *         (<SingleDeviceMemoryPool>pool).free(self.chunk)
*/
              __pyx_t_4 = 0;
              {
                PyObject *__pyx_callargs[2 + ((CYTHON_VECTORCALL) ? 4 : 0)] = {__pyx_t_10, NULL};
                __pyx_t_14 = __Pyx_MakeVectorcallBuilderKwds(4); if (unlikely(!__pyx_t_14)) __PYX_ERR(0, 1028, __pyx_L15_error)
                __Pyx_GOTREF(__pyx_t_14);
                if (__Pyx_VectorcallBuilder_AddArg(__pyx_mstate_global->__pyx_n_u_device_id, __pyx_t_11, __pyx_t_14, __pyx_callargs+1, 0) < (0)) __PYX_ERR(0, 1028, __pyx_L15_error)
                if (__Pyx_VectorcallBuilder_AddArg(__pyx_mstate_global->__pyx_n_u_mem_size, __pyx_t_12, __pyx_t_14, __pyx_callargs+1, 1) < (0)) __PYX_ERR(0, 1028, __pyx_L15_error)
                if (__Pyx_VectorcallBuilder_AddArg(__pyx_mstate_global->__pyx_n_u_mem_ptr, __pyx_t_13, __pyx_t_14, __pyx_callargs+1, 2) < (0)) __PYX_ERR(0, 1028, __pyx_L15_error)
                if (__Pyx_VectorcallBuilder_AddArg(__pyx_mstate_global->__pyx_n_u_pmem_id, __pyx_v_pmem_id, __pyx_t_14, __pyx_callargs+1, 3) < (0)) __PYX_ERR(0, 1028, __pyx_L15_error)
                __pyx_t_3 = __Pyx_Object_VectorcallMethod_CallFromBuilder((PyObject*)__pyx_mstate_global->__pyx_n_u_free_postprocess, __pyx_callargs+__pyx_t_4, (1-__pyx_t_4) | (1*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET), __pyx_t_14);
                __Pyx_XDECREF(__pyx_t_10); __pyx_t_10 = 0;
                __Pyx_DECREF(__pyx_t_11); __pyx_t_11 = 0;
                __Pyx_DECREF(__pyx_t_12); __pyx_t_12 = 0;
                __Pyx_DECREF(__pyx_t_13); __pyx_t_13 = 0;
                __Pyx_DECREF(__pyx_t_14); __pyx_t_14 = 0;
                if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 1028, __pyx_L15_error)
                __Pyx_GOTREF(__pyx_t_3);
              }
              __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
            }
            __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
          }
          __Pyx_XGIVEREF(__pyx_t_19);
          __Pyx_XGIVEREF(__pyx_t_20);
          __Pyx_XGIVEREF(__pyx_t_21);
          __Pyx_ExceptionReset(__pyx_t_19, __pyx_t_20, __pyx_t_21);
          __Pyx_XGIVEREF(__pyx_t_16);
          __Pyx_XGIVEREF(__pyx_t_17);
          __Pyx_XGIVEREF(__pyx_t_18);
          __Pyx_ErrRestore(__pyx_t_16, __pyx_t_17, __pyx_t_18);
          __pyx_t_16 = 0; __pyx_t_17 = 0; __pyx_t_18 = 0; __pyx_t_19 = 0; __pyx_t_20 = 0; __pyx_t_21 = 0;
          __pyx_lineno = __pyx_t_5; __pyx_clineno = __pyx_t_9; __pyx_filename = __pyx_t_15;
          goto __pyx_L1_error;
          __pyx_L15_error:;
          __Pyx_XGIVEREF(__pyx_t_19);
          __Pyx_XGIVEREF(__pyx_t_20);
          __Pyx_XGIVEREF(__pyx_t_21);
          __Pyx_ExceptionReset(__pyx_t_19, __pyx_t_20, __pyx_t_21);
          __Pyx_XDECREF(__pyx_t_16); __pyx_t_16 = 0;
          __Pyx_XDECREF(__pyx_t_17); __pyx_t_17 = 0;
          __Pyx_XDECREF(__pyx_t_18); __pyx_t_18 = 0;
          __pyx_t_19 = 0; __pyx_t_20 = 0; __pyx_t_21 = 0;
          goto __pyx_L1_error;
        }
        __pyx_L11:;
      }

      /* "cupy/cuda/memory.pyx":1032
 *                                               mem_ptr=ptr,
 *                                               pmem_id=pmem_id)
 *                 return             # <<<<<<<<<<<<<<
 *         (<SingleDeviceMemoryPool>pool).free(self.chunk)
 * 
*/
      __Pyx_XDECREF(__pyx_r);
      __pyx_r = Py_None; __Pyx_INCREF(Py_None);
      goto __pyx_L0;

      /* "cupy/cuda/memory.pyx":1015
 *         if memory_hook._has_memory_hooks():
 *             hooks = memory_hook.get_memory_hooks()
 *             if hooks:             # <<<<<<<<<<<<<<
 *                 device_id = self.device_id
 *                 pmem_id = id(self)
*/
    }

    /* "cupy/cuda/memory.pyx":1013
 * 
 *         size = self.size
 *         if memory_hook._has_memory_hooks():             # <<<<<<<<<<<<<<
 *             hooks = memory_hook.get_memory_hooks()
 *             if hooks:
*/
  }

  /* "cupy/cuda/memory.pyx":1033
 *                                               pmem_id=pmem_id)
 *                 return
 *         (<SingleDeviceMemoryPool>pool).free(self.chunk)             # <<<<<<<<<<<<<<
 * 
 *     def __dealloc__(self):
*/
  __pyx_t_6 = ((PyObject *)__pyx_v_self->chunk);
  __Pyx_INCREF(__pyx_t_6);
  __pyx_t_3 = __pyx_f_4cupy_4cuda_6memory_22SingleDeviceMemoryPool_free(((struct __pyx_obj_4cupy_4cuda_6memory_SingleDeviceMemoryPool *)__pyx_v_pool), ((struct __pyx_obj_4cupy_4cuda_6memory__Chunk *)__pyx_t_6)); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 1033, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "cupy/cuda/memory.pyx":993
 *             self.identity = "<unknown>"
 * 
 *     cpdef free(self):             # <<<<<<<<<<<<<<
 *         """Frees the memory buffer and returns it to the memory pool.
 * 
*/

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_XDECREF(__pyx_t_10);
  __Pyx_XDECREF(__pyx_t_11);
  __Pyx_XDECREF(__pyx_t_12);
  __Pyx_XDECREF(__pyx_t_13);
  __Pyx_XDECREF(__pyx_t_14);
  __Pyx_AddTraceback("cupy.cuda.memory.PooledMemory.free", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_pool);
  __Pyx_XDECREF(__pyx_v_hooks);
  __Pyx_XDECREF(__pyx_v_pmem_id);
  __Pyx_XDECREF(__pyx_v_hook);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_pw_4cupy_4cuda_6memory_12PooledMemory_3free(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
PyDoc_STRVAR(__pyx_doc_4cupy_4cuda_6memory_12PooledMemory_2free, "PooledMemory.free(self)\n\nFrees the memory buffer and returns it to the memory pool.\n\nThis function actually does not free the buffer. It just returns the\nbuffer to the memory pool for reuse.");
static PyObject *__pyx_pw_4cupy_4cuda_6memory_12PooledMemory_3free(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
) {
  #if !CYTHON_METH_FASTCALL
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  #endif
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("free (wrapper)", 0);
  #if !CYTHON_METH_FASTCALL
  #if CYTHON_ASSUME_SAFE_SIZE
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return NULL;
  #endif
  #endif
  __pyx_kwvalues = __Pyx_KwValues_FASTCALL(__pyx_args, __pyx_nargs);
  if (unlikely(__pyx_nargs > 0)) { __Pyx_RaiseArgtupleInvalid("free", 1, 0, 0, __pyx_nargs); return NULL; }
  const Py_ssize_t __pyx_kwds_len = unlikely(__pyx_kwds) ? __Pyx_NumKwargs_FASTCALL(__pyx_kwds) : 0;
  if (unlikely(__pyx_kwds_len < 0)) return NULL;
  if (unlikely(__pyx_kwds_len > 0)) {__Pyx_RejectKeywords("free", __pyx_kwds); return NULL;}
  __pyx_r = __pyx_pf_4cupy_4cuda_6memory_12PooledMemory_2free(((struct __pyx_obj_4cupy_4cuda_6memory_PooledMemory *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4cupy_4cuda_6memory_12PooledMemory_2free(struct __pyx_obj_4cupy_4cuda_6memory_PooledMemory *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("free", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __pyx_f_4cupy_4cuda_6memory_12PooledMemory_free(__pyx_v_self, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 993, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("cupy.cuda.memory.PooledMemory.free", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "cupy/cuda/memory.pyx":1035
 *         (<SingleDeviceMemoryPool>pool).free(self.chunk)
 * 
 *     def __dealloc__(self):             # <<<<<<<<<<<<<<
 *         if _exit_mode:
 *             return  # To avoid error at exit
*/

/* Python wrapper */
static void __pyx_pw_4cupy_4cuda_6memory_12PooledMemory_5__dealloc__(PyObject *__pyx_v_self); /*proto*/
static void __pyx_pw_4cupy_4cuda_6memory_12PooledMemory_5__dealloc__(PyObject *__pyx_v_self) {
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__dealloc__ (wrapper)", 0);
  __pyx_kwvalues = __Pyx_KwValues_VARARGS(__pyx_args, __pyx_nargs);
  __pyx_pf_4cupy_4cuda_6memory_12PooledMemory_4__dealloc__(((struct __pyx_obj_4cupy_4cuda_6memory_PooledMemory *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
}

static void __pyx_pf_4cupy_4cuda_6memory_12PooledMemory_4__dealloc__(struct __pyx_obj_4cupy_4cuda_6memory_PooledMemory *__pyx_v_self) {
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__dealloc__", 0);

  /* "cupy/cuda/memory.pyx":1036
 * 
 *     def __dealloc__(self):
 *         if _exit_mode:             # <<<<<<<<<<<<<<
 *             return  # To avoid error at exit
 *         self.free()
*/
  if (__pyx_v_4cupy_4cuda_6memory__exit_mode) {

    /* "cupy/cuda/memory.pyx":1037
 *     def __dealloc__(self):
 *         if _exit_mode:
 *             return  # To avoid error at exit             # <<<<<<<<<<<<<<
 *         self.free()
 * 
*/
    goto __pyx_L0;

    /* "cupy/cuda/memory.pyx":1036
 * 
 *     def __dealloc__(self):
 *         if _exit_mode:             # <<<<<<<<<<<<<<
 *             return  # To avoid error at exit
 *         self.free()
*/
  }

  /* "cupy/cuda/memory.pyx":1038
 *         if _exit_mode:
 *             return  # To avoid error at exit
 *         self.free()             # <<<<<<<<<<<<<<
 * 
 * 
*/
  __pyx_t_1 = __pyx_f_4cupy_4cuda_6memory_12PooledMemory_free(__pyx_v_self, 0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1038, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "cupy/cuda/memory.pyx":1035
 *         (<SingleDeviceMemoryPool>pool).free(self.chunk)
 * 
 *     def __dealloc__(self):             # <<<<<<<<<<<<<<
 *         if _exit_mode:
 *             return  # To avoid error at exit
*/

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_WriteUnraisable("cupy.cuda.memory.PooledMemory.__dealloc__", __pyx_clineno, __pyx_lineno, __pyx_filename, 1, 0);
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
}

/* "cupy/cuda/memory.pyx":955
 * 
 *     cdef:
 *         readonly object pool             # <<<<<<<<<<<<<<
 *         readonly str identity
 *         _Chunk chunk
*/

/* Python wrapper */
static PyObject *__pyx_pw_4cupy_4cuda_6memory_12PooledMemory_4pool_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_4cupy_4cuda_6memory_12PooledMemory_4pool_1__get__(PyObject *__pyx_v_self) {
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_kwvalues = __Pyx_KwValues_VARARGS(__pyx_args, __pyx_nargs);
  __pyx_r = __pyx_pf_4cupy_4cuda_6memory_12PooledMemory_4pool___get__(((struct __pyx_obj_4cupy_4cuda_6memory_PooledMemory *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4cupy_4cuda_6memory_12PooledMemory_4pool___get__(struct __pyx_obj_4cupy_4cuda_6memory_PooledMemory *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__", 0);
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_v_self->pool);
  __pyx_r = __pyx_v_self->pool;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "cupy/cuda/memory.pyx":956
 *     cdef:
 *         readonly object pool
 *         readonly str identity             # <<<<<<<<<<<<<<
 *         _Chunk chunk
 *         dict __dict__
*/

/* Python wrapper */
static PyObject *__pyx_pw_4cupy_4cuda_6memory_12PooledMemory_8identity_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_4cupy_4cuda_6memory_12PooledMemory_8identity_1__get__(PyObject *__pyx_v_self) {
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_kwvalues = __Pyx_KwValues_VARARGS(__pyx_args, __pyx_nargs);
  __pyx_r = __pyx_pf_4cupy_4cuda_6memory_12PooledMemory_8identity___get__(((struct __pyx_obj_4cupy_4cuda_6memory_PooledMemory *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4cupy_4cuda_6memory_12PooledMemory_8identity___get__(struct __pyx_obj_4cupy_4cuda_6memory_PooledMemory *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__", 0);
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_v_self->identity);
  __pyx_r = __pyx_v_self->identity;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     cdef tuple state
 *     cdef object _dict
*/

/* Python wrapper */
static PyObject *__pyx_pw_4cupy_4cuda_6memory_12PooledMemory_7__reduce_cython__(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
PyDoc_STRVAR(__pyx_doc_4cupy_4cuda_6memory_12PooledMemory_6__reduce_cython__, "PooledMemory.__reduce_cython__(self)");
static PyObject *__pyx_pw_4cupy_4cuda_6memory_12PooledMemory_7__reduce_cython__(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
) {
  #if !CYTHON_METH_FASTCALL
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  #endif
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__reduce_cython__ (wrapper)", 0);
  #if !CYTHON_METH_FASTCALL
  #if CYTHON_ASSUME_SAFE_SIZE
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return NULL;
  #endif
  #endif
  __pyx_kwvalues = __Pyx_KwValues_FASTCALL(__pyx_args, __pyx_nargs);
  if (unlikely(__pyx_nargs > 0)) { __Pyx_RaiseArgtupleInvalid("__reduce_cython__", 1, 0, 0, __pyx_nargs); return NULL; }
  const Py_ssize_t __pyx_kwds_len = unlikely(__pyx_kwds) ? __Pyx_NumKwargs_FASTCALL(__pyx_kwds) : 0;
  if (unlikely(__pyx_kwds_len < 0)) return NULL;
  if (unlikely(__pyx_kwds_len > 0)) {__Pyx_RejectKeywords("__reduce_cython__", __pyx_kwds); return NULL;}
  __pyx_r = __pyx_pf_4cupy_4cuda_6memory_12PooledMemory_6__reduce_cython__(((struct __pyx_obj_4cupy_4cuda_6memory_PooledMemory *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4cupy_4cuda_6memory_12PooledMemory_6__reduce_cython__(struct __pyx_obj_4cupy_4cuda_6memory_PooledMemory *__pyx_v_self) {
  PyObject *__pyx_v_state = 0;
  PyObject *__pyx_v__dict = 0;
  int __pyx_v_use_setstate;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  int __pyx_t_5;
  int __pyx_t_6;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__reduce_cython__", 0);

  /* "(tree fragment)":5
 *     cdef object _dict
 *     cdef bint use_setstate
 *     state = (self.chunk, self.device_id, self.identity, self.pool, self.ptr, self.size)             # <<<<<<<<<<<<<<
 *     _dict = getattr(self, '__dict__', None)
 *     if _dict is not None and _dict:
*/
  __pyx_t_1 = __Pyx_PyLong_From_int(__pyx_v_self->__pyx_base.device_id); if (unlikely(!__pyx_t_1)) __PYX_ERR(3, 5, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = PyLong_FromSsize_t(__pyx_v_self->__pyx_base.ptr); if (unlikely(!__pyx_t_2)) __PYX_ERR(3, 5, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = __Pyx_PyLong_FromSize_t(__pyx_v_self->__pyx_base.size); if (unlikely(!__pyx_t_3)) __PYX_ERR(3, 5, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_4 = PyTuple_New(6); if (unlikely(!__pyx_t_4)) __PYX_ERR(3, 5, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_INCREF((PyObject *)__pyx_v_self->chunk);
  __Pyx_GIVEREF((PyObject *)__pyx_v_self->chunk);
  if (__Pyx_PyTuple_SET_ITEM(__pyx_t_4, 0, ((PyObject *)__pyx_v_self->chunk)) != (0)) __PYX_ERR(3, 5, __pyx_L1_error);
  __Pyx_GIVEREF(__pyx_t_1);
  if (__Pyx_PyTuple_SET_ITEM(__pyx_t_4, 1, __pyx_t_1) != (0)) __PYX_ERR(3, 5, __pyx_L1_error);
  __Pyx_INCREF(__pyx_v_self->identity);
  __Pyx_GIVEREF(__pyx_v_self->identity);
  if (__Pyx_PyTuple_SET_ITEM(__pyx_t_4, 2, __pyx_v_self->identity) != (0)) __PYX_ERR(3, 5, __pyx_L1_error);
  __Pyx_INCREF(__pyx_v_self->pool);
  __Pyx_GIVEREF(__pyx_v_self->pool);
  if (__Pyx_PyTuple_SET_ITEM(__pyx_t_4, 3, __pyx_v_self->pool) != (0)) __PYX_ERR(3, 5, __pyx_L1_error);
  __Pyx_GIVEREF(__pyx_t_2);
  if (__Pyx_PyTuple_SET_ITEM(__pyx_t_4, 4, __pyx_t_2) != (0)) __PYX_ERR(3, 5, __pyx_L1_error);
  __Pyx_GIVEREF(__pyx_t_3);
  if (__Pyx_PyTuple_SET_ITEM(__pyx_t_4, 5, __pyx_t_3) != (0)) __PYX_ERR(3, 5, __pyx_L1_error);
  __pyx_t_1 = 0;
  __pyx_t_2 = 0;
  __pyx_t_3 = 0;
  __pyx_v_state = ((PyObject*)__pyx_t_4);
  __pyx_t_4 = 0;

  /* "(tree fragment)":6
 *     cdef bint use_setstate
 *     state = (self.chunk, self.device_id, self.identity, self.pool, self.ptr, self.size)
 *     _dict = getattr(self, '__dict__', None)             # <<<<<<<<<<<<<<
 *     if _dict is not None and _dict:
 *         state += (_dict,)
*/
  __pyx_t_4 = __Pyx_GetAttr3(((PyObject *)__pyx_v_self), __pyx_mstate_global->__pyx_n_u_dict, Py_None); if (unlikely(!__pyx_t_4)) __PYX_ERR(3, 6, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_v__dict = __pyx_t_4;
  __pyx_t_4 = 0;

  /* "(tree fragment)":7
 *     state = (self.chunk, self.device_id, self.identity, self.pool, self.ptr, self.size)
 *     _dict = getattr(self, '__dict__', None)
 *     if _dict is not None and _dict:             # <<<<<<<<<<<<<<
 *         state += (_dict,)
 *         use_setstate = True
*/
  __pyx_t_6 = (__pyx_v__dict != Py_None);
  if (__pyx_t_6) {
  } else {
    __pyx_t_5 = __pyx_t_6;
    goto __pyx_L4_bool_binop_done;
  }
  __pyx_t_6 = __Pyx_PyObject_IsTrue(__pyx_v__dict); if (unlikely((__pyx_t_6 < 0))) __PYX_ERR(3, 7, __pyx_L1_error)
  __pyx_t_5 = __pyx_t_6;
  __pyx_L4_bool_binop_done:;
  if (__pyx_t_5) {

    /* "(tree fragment)":8
 *     _dict = getattr(self, '__dict__', None)
 *     if _dict is not None and _dict:
 *         state += (_dict,)             # <<<<<<<<<<<<<<
 *         use_setstate = True
 *     else:
*/
    __pyx_t_4 = PyTuple_New(1); if (unlikely(!__pyx_t_4)) __PYX_ERR(3, 8, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_INCREF(__pyx_v__dict);
    __Pyx_GIVEREF(__pyx_v__dict);
    if (__Pyx_PyTuple_SET_ITEM(__pyx_t_4, 0, __pyx_v__dict) != (0)) __PYX_ERR(3, 8, __pyx_L1_error);
    __pyx_t_3 = PyNumber_InPlaceAdd(__pyx_v_state, __pyx_t_4); if (unlikely(!__pyx_t_3)) __PYX_ERR(3, 8, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __Pyx_DECREF_SET(__pyx_v_state, ((PyObject*)__pyx_t_3));
    __pyx_t_3 = 0;

    /* "(tree fragment)":9
 *     if _dict is not None and _dict:
 *         state += (_dict,)
 *         use_setstate = True             # <<<<<<<<<<<<<<
 *     else:
 *         use_setstate = self.chunk is not None or self.identity is not None or self.pool is not None
*/
    __pyx_v_use_setstate = 1;

    /* "(tree fragment)":7
 *     state = (self.chunk, self.device_id, self.identity, self.pool, self.ptr, self.size)
 *     _dict = getattr(self, '__dict__', None)
 *     if _dict is not None and _dict:             # <<<<<<<<<<<<<<
 *         state += (_dict,)
 *         use_setstate = True
*/
    goto __pyx_L3;
  }

  /* "(tree fragment)":11
 *         use_setstate = True
 *     else:
 *         use_setstate = self.chunk is not None or self.identity is not None or self.pool is not None             # <<<<<<<<<<<<<<
 *     if use_setstate:
 *         return __pyx_unpickle_PooledMemory, (type(self), 0x43cf7ab, None), state
*/
  /*else*/ {
    __pyx_t_6 = (((PyObject *)__pyx_v_self->chunk) != Py_None);
    if (!__pyx_t_6) {
    } else {
      __pyx_t_5 = __pyx_t_6;
      goto __pyx_L6_bool_binop_done;
    }
    __pyx_t_6 = (__pyx_v_self->identity != ((PyObject*)Py_None));
    if (!__pyx_t_6) {
    } else {
      __pyx_t_5 = __pyx_t_6;
      goto __pyx_L6_bool_binop_done;
    }
    __pyx_t_6 = (__pyx_v_self->pool != Py_None);
    __pyx_t_5 = __pyx_t_6;
    __pyx_L6_bool_binop_done:;
    __pyx_v_use_setstate = __pyx_t_5;
  }
  __pyx_L3:;

  /* "(tree fragment)":12
 *     else:
 *         use_setstate = self.chunk is not None or self.identity is not None or self.pool is not None
 *     if use_setstate:             # <<<<<<<<<<<<<<
 *         return __pyx_unpickle_PooledMemory, (type(self), 0x43cf7ab, None), state
 *     else:
*/
  if (__pyx_v_use_setstate) {

    /* "(tree fragment)":13
 *         use_setstate = self.chunk is not None or self.identity is not None or self.pool is not None
 *     if use_setstate:
 *         return __pyx_unpickle_PooledMemory, (type(self), 0x43cf7ab, None), state             # <<<<<<<<<<<<<<
 *     else:
 *         return __pyx_unpickle_PooledMemory, (type(self), 0x43cf7ab, state)
*/
    __Pyx_XDECREF(__pyx_r);
    __Pyx_GetModuleGlobalName(__pyx_t_3, __pyx_mstate_global->__pyx_n_u_pyx_unpickle_PooledMemory); if (unlikely(!__pyx_t_3)) __PYX_ERR(3, 13, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_t_4 = PyTuple_New(3); if (unlikely(!__pyx_t_4)) __PYX_ERR(3, 13, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_INCREF(((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    __Pyx_GIVEREF(((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    if (__Pyx_PyTuple_SET_ITEM(__pyx_t_4, 0, ((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self)))) != (0)) __PYX_ERR(3, 13, __pyx_L1_error);
    __Pyx_INCREF(__pyx_mstate_global->__pyx_int_71104427);
    __Pyx_GIVEREF(__pyx_mstate_global->__pyx_int_71104427);
    if (__Pyx_PyTuple_SET_ITEM(__pyx_t_4, 1, __pyx_mstate_global->__pyx_int_71104427) != (0)) __PYX_ERR(3, 13, __pyx_L1_error);
    __Pyx_INCREF(Py_None);
    __Pyx_GIVEREF(Py_None);
    if (__Pyx_PyTuple_SET_ITEM(__pyx_t_4, 2, Py_None) != (0)) __PYX_ERR(3, 13, __pyx_L1_error);
    __pyx_t_2 = PyTuple_New(3); if (unlikely(!__pyx_t_2)) __PYX_ERR(3, 13, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_GIVEREF(__pyx_t_3);
    if (__Pyx_PyTuple_SET_ITEM(__pyx_t_2, 0, __pyx_t_3) != (0)) __PYX_ERR(3, 13, __pyx_L1_error);
    __Pyx_GIVEREF(__pyx_t_4);
    if (__Pyx_PyTuple_SET_ITEM(__pyx_t_2, 1, __pyx_t_4) != (0)) __PYX_ERR(3, 13, __pyx_L1_error);
    __Pyx_INCREF(__pyx_v_state);
    __Pyx_GIVEREF(__pyx_v_state);
    if (__Pyx_PyTuple_SET_ITEM(__pyx_t_2, 2, __pyx_v_state) != (0)) __PYX_ERR(3, 13, __pyx_L1_error);
    __pyx_t_3 = 0;
    __pyx_t_4 = 0;
    __pyx_r = __pyx_t_2;
    __pyx_t_2 = 0;
    goto __pyx_L0;

    /* "(tree fragment)":12
 *     else:
 *         use_setstate = self.chunk is not None or self.identity is not None or self.pool is not None
 *     if use_setstate:             # <<<<<<<<<<<<<<
 *         return __pyx_unpickle_PooledMemory, (type(self), 0x43cf7ab, None), state
 *     else:
*/
  }

  /* "(tree fragment)":15
 *         return __pyx_unpickle_PooledMemory, (type(self), 0x43cf7ab, None), state
 *     else:
 *         return __pyx_unpickle_PooledMemory, (type(self), 0x43cf7ab, state)             # <<<<<<<<<<<<<<
 * def __setstate_cython__(self, __pyx_state):
 *     __pyx_unpickle_PooledMemory__set_state(self, __pyx_state)
*/
  /*else*/ {
    __Pyx_XDECREF(__pyx_r);
    __Pyx_GetModuleGlobalName(__pyx_t_2, __pyx_mstate_global->__pyx_n_u_pyx_unpickle_PooledMemory); if (unlikely(!__pyx_t_2)) __PYX_ERR(3, 15, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_4 = PyTuple_New(3); if (unlikely(!__pyx_t_4)) __PYX_ERR(3, 15, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_INCREF(((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    __Pyx_GIVEREF(((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    if (__Pyx_PyTuple_SET_ITEM(__pyx_t_4, 0, ((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self)))) != (0)) __PYX_ERR(3, 15, __pyx_L1_error);
    __Pyx_INCREF(__pyx_mstate_global->__pyx_int_71104427);
    __Pyx_GIVEREF(__pyx_mstate_global->__pyx_int_71104427);
    if (__Pyx_PyTuple_SET_ITEM(__pyx_t_4, 1, __pyx_mstate_global->__pyx_int_71104427) != (0)) __PYX_ERR(3, 15, __pyx_L1_error);
    __Pyx_INCREF(__pyx_v_state);
    __Pyx_GIVEREF(__pyx_v_state);
    if (__Pyx_PyTuple_SET_ITEM(__pyx_t_4, 2, __pyx_v_state) != (0)) __PYX_ERR(3, 15, __pyx_L1_error);
    __pyx_t_3 = PyTuple_New(2); if (unlikely(!__pyx_t_3)) __PYX_ERR(3, 15, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_GIVEREF(__pyx_t_2);
    if (__Pyx_PyTuple_SET_ITEM(__pyx_t_3, 0, __pyx_t_2) != (0)) __PYX_ERR(3, 15, __pyx_L1_error);
    __Pyx_GIVEREF(__pyx_t_4);
    if (__Pyx_PyTuple_SET_ITEM(__pyx_t_3, 1, __pyx_t_4) != (0)) __PYX_ERR(3, 15, __pyx_L1_error);
    __pyx_t_2 = 0;
    __pyx_t_4 = 0;
    __pyx_r = __pyx_t_3;
    __pyx_t_3 = 0;
    goto __pyx_L0;
  }

  /* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     cdef tuple state
 *     cdef object _dict
*/

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_AddTraceback("cupy.cuda.memory.PooledMemory.__reduce_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_state);
  __Pyx_XDECREF(__pyx_v__dict);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":16
 *     else:
 *         return __pyx_unpickle_PooledMemory, (type(self), 0x43cf7ab, state)
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     __pyx_unpickle_PooledMemory__set_state(self, __pyx_state)
*/

/* Python wrapper */
static PyObject *__pyx_pw_4cupy_4cuda_6memory_12PooledMemory_9__setstate_cython__(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
PyDoc_STRVAR(__pyx_doc_4cupy_4cuda_6memory_12PooledMemory_8__setstate_cython__, "PooledMemory.__setstate_cython__(self, __pyx_state)");
static PyObject *__pyx_pw_4cupy_4cuda_6memory_12PooledMemory_9__setstate_cython__(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
) {
  PyObject *__pyx_v___pyx_state = 0;
  #if !CYTHON_METH_FASTCALL
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  #endif
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject* values[1] = {0};
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__setstate_cython__ (wrapper)", 0);
  #if !CYTHON_METH_FASTCALL
  #if CYTHON_ASSUME_SAFE_SIZE
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return NULL;
  #endif
  #endif
  __pyx_kwvalues = __Pyx_KwValues_FASTCALL(__pyx_args, __pyx_nargs);
  {
    PyObject ** const __pyx_pyargnames[] = {&__pyx_mstate_global->__pyx_n_u_pyx_state,0};
    const Py_ssize_t __pyx_kwds_len = (__pyx_kwds) ? __Pyx_NumKwargs_FASTCALL(__pyx_kwds) : 0;
    if (unlikely(__pyx_kwds_len < 0)) __PYX_ERR(3, 16, __pyx_L3_error)
    if (__pyx_kwds_len > 0) {
      switch (__pyx_nargs) {
        case  1:
        values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(3, 16, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      const Py_ssize_t kwd_pos_args = __pyx_nargs;
      if (__Pyx_ParseKeywords(__pyx_kwds, __pyx_kwvalues, __pyx_pyargnames, 0, values, kwd_pos_args, __pyx_kwds_len, "__setstate_cython__", 0) < (0)) __PYX_ERR(3, 16, __pyx_L3_error)
      for (Py_ssize_t i = __pyx_nargs; i < 1; i++) {
        if (unlikely(!values[i])) { __Pyx_RaiseArgtupleInvalid("__setstate_cython__", 1, 1, 1, i); __PYX_ERR(3, 16, __pyx_L3_error) }
      }
    } else if (unlikely(__pyx_nargs != 1)) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
      if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(3, 16, __pyx_L3_error)
    }
    __pyx_v___pyx_state = values[0];
  }
  goto __pyx_L6_skip;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("__setstate_cython__", 1, 1, 1, __pyx_nargs); __PYX_ERR(3, 16, __pyx_L3_error)
  __pyx_L6_skip:;
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_AddTraceback("cupy.cuda.memory.PooledMemory.__setstate_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_4cupy_4cuda_6memory_12PooledMemory_8__setstate_cython__(((struct __pyx_obj_4cupy_4cuda_6memory_PooledMemory *)__pyx_v_self), __pyx_v___pyx_state);

  /* function exit code */
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4cupy_4cuda_6memory_12PooledMemory_8__setstate_cython__(struct __pyx_obj_4cupy_4cuda_6memory_PooledMemory *__pyx_v_self, PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__setstate_cython__", 0);

  /* "(tree fragment)":17
 *         return __pyx_unpickle_PooledMemory, (type(self), 0x43cf7ab, state)
 * def __setstate_cython__(self, __pyx_state):
 *     __pyx_unpickle_PooledMemory__set_state(self, __pyx_state)             # <<<<<<<<<<<<<<
*/
  __pyx_t_1 = __pyx_v___pyx_state;
  __Pyx_INCREF(__pyx_t_1);
  if (!(likely(PyTuple_CheckExact(__pyx_t_1))||((__pyx_t_1) == Py_None) || __Pyx_RaiseUnexpectedTypeError("tuple", __pyx_t_1))) __PYX_ERR(3, 17, __pyx_L1_error)
  if (unlikely(__pyx_t_1 == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "cannot pass None into a C function argument that is declared 'not None'");
    __PYX_ERR(3, 17, __pyx_L1_error)
  }
  __pyx_t_2 = __pyx_f_4cupy_4cuda_6memory___pyx_unpickle_PooledMemory__set_state(__pyx_v_self, ((PyObject*)__pyx_t_1)); if (unlikely(!__pyx_t_2)) __PYX_ERR(3, 17, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "(tree fragment)":16
 *     else:
 *         return __pyx_unpickle_PooledMemory, (type(self), 0x43cf7ab, state)
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     __pyx_unpickle_PooledMemory__set_state(self, __pyx_state)
*/

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_AddTraceback("cupy.cuda.memory.PooledMemory.__setstate_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "cupy/cuda/memory.pyx":1041
 * 
 * 
 * @cython.cdivision(True)             # <<<<<<<<<<<<<<
 * cpdef inline size_t _round_size(size_t size) noexcept:
 *     """Rounds up the memory size to fit memory alignment of cudaMalloc."""
*/

static PyObject *__pyx_pw_4cupy_4cuda_6memory_21_round_size(PyObject *__pyx_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
static CYTHON_INLINE size_t __pyx_f_4cupy_4cuda_6memory__round_size(size_t __pyx_v_size, CYTHON_UNUSED int __pyx_skip_dispatch) {
  size_t __pyx_r;

  /* "cupy/cuda/memory.pyx":1045
 *     """Rounds up the memory size to fit memory alignment of cudaMalloc."""
 *     # avoid 0 div checking
 *     size = (size + ALLOCATION_UNIT_SIZE - 1) // ALLOCATION_UNIT_SIZE             # <<<<<<<<<<<<<<
 *     return size * ALLOCATION_UNIT_SIZE
 * 
*/
  __pyx_v_size = (((__pyx_v_size + __pyx_v_4cupy_4cuda_6memory_ALLOCATION_UNIT_SIZE) - 1) / __pyx_v_4cupy_4cuda_6memory_ALLOCATION_UNIT_SIZE);

  /* "cupy/cuda/memory.pyx":1046
 *     # avoid 0 div checking
 *     size = (size + ALLOCATION_UNIT_SIZE - 1) // ALLOCATION_UNIT_SIZE
 *     return size * ALLOCATION_UNIT_SIZE             # <<<<<<<<<<<<<<
 * 
 * 
*/
  __pyx_r = (__pyx_v_size * __pyx_v_4cupy_4cuda_6memory_ALLOCATION_UNIT_SIZE);
  goto __pyx_L0;

  /* "cupy/cuda/memory.pyx":1041
 * 
 * 
 * @cython.cdivision(True)             # <<<<<<<<<<<<<<
 * cpdef inline size_t _round_size(size_t size) noexcept:
 *     """Rounds up the memory size to fit memory alignment of cudaMalloc."""
*/

  /* function exit code */
  __pyx_L0:;
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_pw_4cupy_4cuda_6memory_21_round_size(PyObject *__pyx_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
PyDoc_STRVAR(__pyx_doc_4cupy_4cuda_6memory_20_round_size, "_round_size(size_t size) -> size_t\n\nRounds up the memory size to fit memory alignment of cudaMalloc.");
static PyObject *__pyx_pw_4cupy_4cuda_6memory_21_round_size(PyObject *__pyx_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
) {
  size_t __pyx_v_size;
  #if !CYTHON_METH_FASTCALL
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  #endif
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject* values[1] = {0};
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("_round_size (wrapper)", 0);
  #if !CYTHON_METH_FASTCALL
  #if CYTHON_ASSUME_SAFE_SIZE
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return NULL;
  #endif
  #endif
  __pyx_kwvalues = __Pyx_KwValues_FASTCALL(__pyx_args, __pyx_nargs);
  {
    PyObject ** const __pyx_pyargnames[] = {&__pyx_mstate_global->__pyx_n_u_size,0};
    const Py_ssize_t __pyx_kwds_len = (__pyx_kwds) ? __Pyx_NumKwargs_FASTCALL(__pyx_kwds) : 0;
    if (unlikely(__pyx_kwds_len < 0)) __PYX_ERR(0, 1041, __pyx_L3_error)
    if (__pyx_kwds_len > 0) {
      switch (__pyx_nargs) {
        case  1:
        values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(0, 1041, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      const Py_ssize_t kwd_pos_args = __pyx_nargs;
      if (__Pyx_ParseKeywords(__pyx_kwds, __pyx_kwvalues, __pyx_pyargnames, 0, values, kwd_pos_args, __pyx_kwds_len, "_round_size", 0) < (0)) __PYX_ERR(0, 1041, __pyx_L3_error)
      for (Py_ssize_t i = __pyx_nargs; i < 1; i++) {
        if (unlikely(!values[i])) { __Pyx_RaiseArgtupleInvalid("_round_size", 1, 1, 1, i); __PYX_ERR(0, 1041, __pyx_L3_error) }
      }
    } else if (unlikely(__pyx_nargs != 1)) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
      if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(0, 1041, __pyx_L3_error)
    }
    __pyx_v_size = __Pyx_PyLong_As_size_t(values[0]); if (unlikely((__pyx_v_size == (size_t)-1) && PyErr_Occurred())) __PYX_ERR(0, 1042, __pyx_L3_error)
  }
  goto __pyx_L6_skip;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("_round_size", 1, 1, 1, __pyx_nargs); __PYX_ERR(0, 1041, __pyx_L3_error)
  __pyx_L6_skip:;
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_AddTraceback("cupy.cuda.memory._round_size", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_4cupy_4cuda_6memory_20_round_size(__pyx_self, __pyx_v_size);

  /* function exit code */
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4cupy_4cuda_6memory_20_round_size(CYTHON_UNUSED PyObject *__pyx_self, size_t __pyx_v_size) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("_round_size", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __Pyx_PyLong_FromSize_t(__pyx_f_4cupy_4cuda_6memory__round_size(__pyx_v_size, 1)); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1041, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("cupy.cuda.memory._round_size", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "cupy/cuda/memory.pyx":1072
 *         cdef object __weakref__
 * 
 *     def __cinit__(self):             # <<<<<<<<<<<<<<
 *         self._pending_free = []
 * 
*/

/* Python wrapper */
static int __pyx_pw_4cupy_4cuda_6memory_6_Arena_1__cinit__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static int __pyx_pw_4cupy_4cuda_6memory_6_Arena_1__cinit__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__cinit__ (wrapper)", 0);
  #if CYTHON_ASSUME_SAFE_SIZE
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return -1;
  #endif
  __pyx_kwvalues = __Pyx_KwValues_VARARGS(__pyx_args, __pyx_nargs);
  if (unlikely(__pyx_nargs > 0)) { __Pyx_RaiseArgtupleInvalid("__cinit__", 1, 0, 0, __pyx_nargs); return -1; }
  const Py_ssize_t __pyx_kwds_len = unlikely(__pyx_kwds) ? __Pyx_NumKwargs_VARARGS(__pyx_kwds) : 0;
  if (unlikely(__pyx_kwds_len < 0)) return -1;
  if (unlikely(__pyx_kwds_len > 0)) {__Pyx_RejectKeywords("__cinit__", __pyx_kwds); return -1;}
  __pyx_r = __pyx_pf_4cupy_4cuda_6memory_6_Arena___cinit__(((struct __pyx_obj_4cupy_4cuda_6memory__Arena *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_4cupy_4cuda_6memory_6_Arena___cinit__(struct __pyx_obj_4cupy_4cuda_6memory__Arena *__pyx_v_self) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__cinit__", 0);

  /* "cupy/cuda/memory.pyx":1073
 * 
 *     def __cinit__(self):
 *         self._pending_free = []             # <<<<<<<<<<<<<<
 * 
 *     cdef add_pending_free_atomic(self, _Chunk chunk):
*/
  __pyx_t_1 = PyList_New(0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1073, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v_self->_pending_free);
  __Pyx_DECREF(__pyx_v_self->_pending_free);
  __pyx_v_self->_pending_free = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "cupy/cuda/memory.pyx":1072
 *         cdef object __weakref__
 * 
 *     def __cinit__(self):             # <<<<<<<<<<<<<<
 *         self._pending_free = []
 * 
*/

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("cupy.cuda.memory._Arena.__cinit__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "cupy/cuda/memory.pyx":1075
 *         self._pending_free = []
 * 
 *     cdef add_pending_free_atomic(self, _Chunk chunk):             # <<<<<<<<<<<<<<
 *         """Add a chunk back to the arena as free. If you are holding the lock,
 *         use `insert_chunk()` directly (it will merge unless `merge=False`).
*/

static PyObject *__pyx_f_4cupy_4cuda_6memory_6_Arena_add_pending_free_atomic(struct __pyx_obj_4cupy_4cuda_6memory__Arena *__pyx_v_self, struct __pyx_obj_4cupy_4cuda_6memory__Chunk *__pyx_v_chunk) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("add_pending_free_atomic", 0);

  /* "cupy/cuda/memory.pyx":1081
 *         # In theory we could try to lock here, but in practice the caller
 *         # should handle that when relevant.
 *         self._pending_free.append(chunk)             # <<<<<<<<<<<<<<
 * 
 *     cdef _commit_pending_free(self):
*/
  if (unlikely(__pyx_v_self->_pending_free == Py_None)) {
    PyErr_Format(PyExc_AttributeError, "'NoneType' object has no attribute '%.30s'", "append");
    __PYX_ERR(0, 1081, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_PyList_Append(__pyx_v_self->_pending_free, ((PyObject *)__pyx_v_chunk)); if (unlikely(__pyx_t_1 == ((int)-1))) __PYX_ERR(0, 1081, __pyx_L1_error)

  /* "cupy/cuda/memory.pyx":1075
 *         self._pending_free = []
 * 
 *     cdef add_pending_free_atomic(self, _Chunk chunk):             # <<<<<<<<<<<<<<
 *         """Add a chunk back to the arena as free. If you are holding the lock,
 *         use `insert_chunk()` directly (it will merge unless `merge=False`).
*/

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_AddTraceback("cupy.cuda.memory._Arena.add_pending_free_atomic", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "cupy/cuda/memory.pyx":1083
 *         self._pending_free.append(chunk)
 * 
 *     cdef _commit_pending_free(self):             # <<<<<<<<<<<<<<
 *         """Clean up the current free list. An exclusive lock must
 *         be held for this, since we may have to insert.
*/

static PyObject *__pyx_f_4cupy_4cuda_6memory_6_Arena__commit_pending_free(struct __pyx_obj_4cupy_4cuda_6memory__Arena *__pyx_v_self) {
  CYTHON_UNUSED Py_ssize_t __pyx_v_i;
  struct __pyx_obj_4cupy_4cuda_6memory__Chunk *__pyx_v_chunk = 0;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  Py_ssize_t __pyx_t_2;
  Py_ssize_t __pyx_t_3;
  Py_ssize_t __pyx_t_4;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("_commit_pending_free", 0);

  /* "cupy/cuda/memory.pyx":1092
 *         cdef _Chunk chunk
 * 
 *         for i in range(len(self._pending_free)):             # <<<<<<<<<<<<<<
 *             chunk = self._pending_free.pop()
 *             self.insert_chunk(chunk)
*/
  __pyx_t_1 = __pyx_v_self->_pending_free;
  __Pyx_INCREF(__pyx_t_1);
  if (unlikely(__pyx_t_1 == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "object of type 'NoneType' has no len()");
    __PYX_ERR(0, 1092, __pyx_L1_error)
  }
  __pyx_t_2 = __Pyx_PyList_GET_SIZE(__pyx_t_1); if (unlikely(__pyx_t_2 == ((Py_ssize_t)-1))) __PYX_ERR(0, 1092, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_3 = __pyx_t_2;
  for (__pyx_t_4 = 0; __pyx_t_4 < __pyx_t_3; __pyx_t_4+=1) {
    __pyx_v_i = __pyx_t_4;

    /* "cupy/cuda/memory.pyx":1093
 * 
 *         for i in range(len(self._pending_free)):
 *             chunk = self._pending_free.pop()             # <<<<<<<<<<<<<<
 *             self.insert_chunk(chunk)
 * 
*/
    if (unlikely(__pyx_v_self->_pending_free == Py_None)) {
      PyErr_Format(PyExc_AttributeError, "'NoneType' object has no attribute '%.30s'", "pop");
      __PYX_ERR(0, 1093, __pyx_L1_error)
    }
    __pyx_t_1 = __Pyx_PyList_Pop(__pyx_v_self->_pending_free); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1093, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    if (!(likely(((__pyx_t_1) == Py_None) || likely(__Pyx_TypeTest(__pyx_t_1, __pyx_mstate_global->__pyx_ptype_4cupy_4cuda_6memory__Chunk))))) __PYX_ERR(0, 1093, __pyx_L1_error)
    __Pyx_XDECREF_SET(__pyx_v_chunk, ((struct __pyx_obj_4cupy_4cuda_6memory__Chunk *)__pyx_t_1));
    __pyx_t_1 = 0;

    /* "cupy/cuda/memory.pyx":1094
 *         for i in range(len(self._pending_free)):
 *             chunk = self._pending_free.pop()
 *             self.insert_chunk(chunk)             # <<<<<<<<<<<<<<
 * 
 *     cdef _Chunk try_merge_chunk(self, _Chunk chunk):
*/
    __pyx_t_1 = __pyx_f_4cupy_4cuda_6memory_6_Arena_insert_chunk(__pyx_v_self, __pyx_v_chunk, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1094, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  }

  /* "cupy/cuda/memory.pyx":1083
 *         self._pending_free.append(chunk)
 * 
 *     cdef _commit_pending_free(self):             # <<<<<<<<<<<<<<
 *         """Clean up the current free list. An exclusive lock must
 *         be held for this, since we may have to insert.
*/

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("cupy.cuda.memory._Arena._commit_pending_free", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XDECREF((PyObject *)__pyx_v_chunk);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "cupy/cuda/memory.pyx":1096
 *             self.insert_chunk(chunk)
 * 
 *     cdef _Chunk try_merge_chunk(self, _Chunk chunk):             # <<<<<<<<<<<<<<
 *         # If this chunk was split try to merge it again.
 *         while chunk.next is not None and self.try_remove_chunk(chunk.next):
*/

static struct __pyx_obj_4cupy_4cuda_6memory__Chunk *__pyx_f_4cupy_4cuda_6memory_6_Arena_try_merge_chunk(struct __pyx_obj_4cupy_4cuda_6memory__Arena *__pyx_v_self, struct __pyx_obj_4cupy_4cuda_6memory__Chunk *__pyx_v_chunk) {
  struct __pyx_obj_4cupy_4cuda_6memory__Chunk *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  int __pyx_t_2;
  PyObject *__pyx_t_3 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("try_merge_chunk", 0);
  __Pyx_INCREF((PyObject *)__pyx_v_chunk);

  /* "cupy/cuda/memory.pyx":1098
 *     cdef _Chunk try_merge_chunk(self, _Chunk chunk):
 *         # If this chunk was split try to merge it again.
 *         while chunk.next is not None and self.try_remove_chunk(chunk.next):             # <<<<<<<<<<<<<<
 *             chunk.merge_next()
 * 
*/
  while (1) {
    __pyx_t_2 = (((PyObject *)__pyx_v_chunk->next) != Py_None);
    if (__pyx_t_2) {
    } else {
      __pyx_t_1 = __pyx_t_2;
      goto __pyx_L5_bool_binop_done;
    }
    __pyx_t_3 = ((PyObject *)__pyx_v_chunk->next);
    __Pyx_INCREF(__pyx_t_3);
    __pyx_t_2 = __pyx_f_4cupy_4cuda_6memory_6_Arena_try_remove_chunk(__pyx_v_self, ((struct __pyx_obj_4cupy_4cuda_6memory__Chunk *)__pyx_t_3)); if (unlikely(__pyx_t_2 == ((int)-1))) __PYX_ERR(0, 1098, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __pyx_t_1 = __pyx_t_2;
    __pyx_L5_bool_binop_done:;
    if (!__pyx_t_1) break;

    /* "cupy/cuda/memory.pyx":1099
 *         # If this chunk was split try to merge it again.
 *         while chunk.next is not None and self.try_remove_chunk(chunk.next):
 *             chunk.merge_next()             # <<<<<<<<<<<<<<
 * 
 *         while chunk.prev is not None and self.try_remove_chunk(chunk.prev):
*/
    __pyx_t_3 = __pyx_f_4cupy_4cuda_6memory_6_Chunk_merge_next(__pyx_v_chunk, 0); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 1099, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  }

  /* "cupy/cuda/memory.pyx":1101
 *             chunk.merge_next()
 * 
 *         while chunk.prev is not None and self.try_remove_chunk(chunk.prev):             # <<<<<<<<<<<<<<
 *             chunk = chunk.prev
 *             chunk.merge_next()
*/
  while (1) {
    __pyx_t_2 = (((PyObject *)__pyx_v_chunk->prev) != Py_None);
    if (__pyx_t_2) {
    } else {
      __pyx_t_1 = __pyx_t_2;
      goto __pyx_L9_bool_binop_done;
    }
    __pyx_t_3 = ((PyObject *)__pyx_v_chunk->prev);
    __Pyx_INCREF(__pyx_t_3);
    __pyx_t_2 = __pyx_f_4cupy_4cuda_6memory_6_Arena_try_remove_chunk(__pyx_v_self, ((struct __pyx_obj_4cupy_4cuda_6memory__Chunk *)__pyx_t_3)); if (unlikely(__pyx_t_2 == ((int)-1))) __PYX_ERR(0, 1101, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __pyx_t_1 = __pyx_t_2;
    __pyx_L9_bool_binop_done:;
    if (!__pyx_t_1) break;

    /* "cupy/cuda/memory.pyx":1102
 * 
 *         while chunk.prev is not None and self.try_remove_chunk(chunk.prev):
 *             chunk = chunk.prev             # <<<<<<<<<<<<<<
 *             chunk.merge_next()
 * 
*/
    __pyx_t_3 = ((PyObject *)__pyx_v_chunk->prev);
    __Pyx_INCREF(__pyx_t_3);
    __Pyx_DECREF_SET(__pyx_v_chunk, ((struct __pyx_obj_4cupy_4cuda_6memory__Chunk *)__pyx_t_3));
    __pyx_t_3 = 0;

    /* "cupy/cuda/memory.pyx":1103
 *         while chunk.prev is not None and self.try_remove_chunk(chunk.prev):
 *             chunk = chunk.prev
 *             chunk.merge_next()             # <<<<<<<<<<<<<<
 * 
 *         return chunk
*/
    __pyx_t_3 = __pyx_f_4cupy_4cuda_6memory_6_Chunk_merge_next(__pyx_v_chunk, 0); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 1103, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  }

  /* "cupy/cuda/memory.pyx":1105
 *             chunk.merge_next()
 * 
 *         return chunk             # <<<<<<<<<<<<<<
 * 
 *     cdef insert_chunk(self, _Chunk chunk, bint merge=True):
*/
  __Pyx_XDECREF((PyObject *)__pyx_r);
  __Pyx_INCREF((PyObject *)__pyx_v_chunk);
  __pyx_r = __pyx_v_chunk;
  goto __pyx_L0;

  /* "cupy/cuda/memory.pyx":1096
 *             self.insert_chunk(chunk)
 * 
 *     cdef _Chunk try_merge_chunk(self, _Chunk chunk):             # <<<<<<<<<<<<<<
 *         # If this chunk was split try to merge it again.
 *         while chunk.next is not None and self.try_remove_chunk(chunk.next):
*/

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_AddTraceback("cupy.cuda.memory._Arena.try_merge_chunk", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XDECREF((PyObject *)__pyx_v_chunk);
  __Pyx_XGIVEREF((PyObject *)__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "cupy/cuda/memory.pyx":1107
 *         return chunk
 * 
 *     cdef insert_chunk(self, _Chunk chunk, bint merge=True):             # <<<<<<<<<<<<<<
 *         if merge:
 *             chunk = self.try_merge_chunk(chunk)
*/

static PyObject *__pyx_f_4cupy_4cuda_6memory_6_Arena_insert_chunk(struct __pyx_obj_4cupy_4cuda_6memory__Arena *__pyx_v_self, struct __pyx_obj_4cupy_4cuda_6memory__Chunk *__pyx_v_chunk, struct __pyx_opt_args_4cupy_4cuda_6memory_6_Arena_insert_chunk *__pyx_optional_args) {
  int __pyx_v_merge = ((int)1);
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __pyx_t_4cupy_4cuda_6memory_index_type __pyx_t_2;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("insert_chunk", 0);
  if (__pyx_optional_args) {
    if (__pyx_optional_args->__pyx_n > 0) {
      __pyx_v_merge = __pyx_optional_args->merge;
    }
  }
  __Pyx_INCREF((PyObject *)__pyx_v_chunk);

  /* "cupy/cuda/memory.pyx":1108
 * 
 *     cdef insert_chunk(self, _Chunk chunk, bint merge=True):
 *         if merge:             # <<<<<<<<<<<<<<
 *             chunk = self.try_merge_chunk(chunk)
 * 
*/
  if (__pyx_v_merge) {

    /* "cupy/cuda/memory.pyx":1109
 *     cdef insert_chunk(self, _Chunk chunk, bint merge=True):
 *         if merge:
 *             chunk = self.try_merge_chunk(chunk)             # <<<<<<<<<<<<<<
 * 
 *         cpython.Py_INCREF(chunk)  # index holds a reference now
*/
    __pyx_t_1 = ((PyObject *)__pyx_f_4cupy_4cuda_6memory_6_Arena_try_merge_chunk(__pyx_v_self, __pyx_v_chunk)); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1109, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF_SET(__pyx_v_chunk, ((struct __pyx_obj_4cupy_4cuda_6memory__Chunk *)__pyx_t_1));
    __pyx_t_1 = 0;

    /* "cupy/cuda/memory.pyx":1108
 * 
 *     cdef insert_chunk(self, _Chunk chunk, bint merge=True):
 *         if merge:             # <<<<<<<<<<<<<<
 *             chunk = self.try_merge_chunk(chunk)
 * 
*/
  }

  /* "cupy/cuda/memory.pyx":1111
 *             chunk = self.try_merge_chunk(chunk)
 * 
 *         cpython.Py_INCREF(chunk)  # index holds a reference now             # <<<<<<<<<<<<<<
 *         self.index.insert(index_type(chunk.size, <uintptr_t><void *>chunk))
 * 
*/
  Py_INCREF(((PyObject *)__pyx_v_chunk));

  /* "cupy/cuda/memory.pyx":1112
 * 
 *         cpython.Py_INCREF(chunk)  # index holds a reference now
 *         self.index.insert(index_type(chunk.size, <uintptr_t><void *>chunk))             # <<<<<<<<<<<<<<
 * 
 *     cdef size_t free_all(self) except -1:
*/
  try {
    __pyx_t_2 = __pyx_t_4cupy_4cuda_6memory_index_type(__pyx_v_chunk->size, ((uintptr_t)((void *)__pyx_v_chunk)));
  } catch(...) {
    __Pyx_CppExn2PyErr();
    __PYX_ERR(0, 1112, __pyx_L1_error)
  }
  try {
    __pyx_v_self->index.insert(__pyx_t_2);
  } catch(...) {
    __Pyx_CppExn2PyErr();
    __PYX_ERR(0, 1112, __pyx_L1_error)
  }

  /* "cupy/cuda/memory.pyx":1107
 *         return chunk
 * 
 *     cdef insert_chunk(self, _Chunk chunk, bint merge=True):             # <<<<<<<<<<<<<<
 *         if merge:
 *             chunk = self.try_merge_chunk(chunk)
*/

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("cupy.cuda.memory._Arena.insert_chunk", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XDECREF((PyObject *)__pyx_v_chunk);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "cupy/cuda/memory.pyx":1114
 *         self.index.insert(index_type(chunk.size, <uintptr_t><void *>chunk))
 * 
 *     cdef size_t free_all(self) except -1:             # <<<<<<<<<<<<<<
 *         """Frees all chunks (that can be free'd, split ones cannot).
 *         """
*/

static size_t __pyx_f_4cupy_4cuda_6memory_6_Arena_free_all(struct __pyx_obj_4cupy_4cuda_6memory__Arena *__pyx_v_self) {
  struct __pyx_obj_4cupy_4cuda_6memory__Chunk *__pyx_v_chunk = 0;
  size_t __pyx_v_bytes_freed;
  std::set<__pyx_t_4cupy_4cuda_6memory_index_type> ::iterator __pyx_v_it;
  size_t __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_t_2;
  void *__pyx_t_3;
  int __pyx_t_4;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("free_all", 0);

  /* "cupy/cuda/memory.pyx":1118
 *         """
 *         cdef _Chunk chunk
 *         cdef size_t bytes_freed = 0             # <<<<<<<<<<<<<<
 * 
 *         self._commit_pending_free()
*/
  __pyx_v_bytes_freed = 0;

  /* "cupy/cuda/memory.pyx":1120
 *         cdef size_t bytes_freed = 0
 * 
 *         self._commit_pending_free()             # <<<<<<<<<<<<<<
 * 
 *         it = self.index.begin()
*/
  __pyx_t_1 = __pyx_f_4cupy_4cuda_6memory_6_Arena__commit_pending_free(__pyx_v_self); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1120, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "cupy/cuda/memory.pyx":1122
 *         self._commit_pending_free()
 * 
 *         it = self.index.begin()             # <<<<<<<<<<<<<<
 *         while it != self.index.end():
 *             chunk = <_Chunk><void *>deref(it).second
*/
  __pyx_v_it = __pyx_v_self->index.begin();

  /* "cupy/cuda/memory.pyx":1123
 * 
 *         it = self.index.begin()
 *         while it != self.index.end():             # <<<<<<<<<<<<<<
 *             chunk = <_Chunk><void *>deref(it).second
 *             if chunk.next is not None or chunk.prev is not None:
*/
  while (1) {
    __pyx_t_2 = (__pyx_v_it != __pyx_v_self->index.end());
    if (!__pyx_t_2) break;

    /* "cupy/cuda/memory.pyx":1124
 *         it = self.index.begin()
 *         while it != self.index.end():
 *             chunk = <_Chunk><void *>deref(it).second             # <<<<<<<<<<<<<<
 *             if chunk.next is not None or chunk.prev is not None:
 *                 # Cannot free this chunk, continue to next.
*/
    __pyx_t_3 = ((void *)(*__pyx_v_it).second);
    __pyx_t_1 = ((PyObject *)__pyx_t_3);
    __Pyx_INCREF(__pyx_t_1);
    __Pyx_XDECREF_SET(__pyx_v_chunk, ((struct __pyx_obj_4cupy_4cuda_6memory__Chunk *)__pyx_t_1));
    __pyx_t_1 = 0;

    /* "cupy/cuda/memory.pyx":1125
 *         while it != self.index.end():
 *             chunk = <_Chunk><void *>deref(it).second
 *             if chunk.next is not None or chunk.prev is not None:             # <<<<<<<<<<<<<<
 *                 # Cannot free this chunk, continue to next.
 *                 postincrement(it)
*/
    __pyx_t_4 = (((PyObject *)__pyx_v_chunk->next) != Py_None);
    if (!__pyx_t_4) {
    } else {
      __pyx_t_2 = __pyx_t_4;
      goto __pyx_L6_bool_binop_done;
    }
    __pyx_t_4 = (((PyObject *)__pyx_v_chunk->prev) != Py_None);
    __pyx_t_2 = __pyx_t_4;
    __pyx_L6_bool_binop_done:;
    if (__pyx_t_2) {

      /* "cupy/cuda/memory.pyx":1127
 *             if chunk.next is not None or chunk.prev is not None:
 *                 # Cannot free this chunk, continue to next.
 *                 postincrement(it)             # <<<<<<<<<<<<<<
 *                 continue
 * 
*/
      (void)((__pyx_v_it++));

      /* "cupy/cuda/memory.pyx":1128
 *                 # Cannot free this chunk, continue to next.
 *                 postincrement(it)
 *                 continue             # <<<<<<<<<<<<<<
 * 
 *             # Otherwise, advance iterator and remove chunk
*/
      goto __pyx_L3_continue;

      /* "cupy/cuda/memory.pyx":1125
 *         while it != self.index.end():
 *             chunk = <_Chunk><void *>deref(it).second
 *             if chunk.next is not None or chunk.prev is not None:             # <<<<<<<<<<<<<<
 *                 # Cannot free this chunk, continue to next.
 *                 postincrement(it)
*/
    }

    /* "cupy/cuda/memory.pyx":1131
 * 
 *             # Otherwise, advance iterator and remove chunk
 *             self.index.erase(postincrement(it))             # <<<<<<<<<<<<<<
 *             cpython.Py_DECREF(chunk)
 * 
*/
    (void)(__pyx_v_self->index.erase((__pyx_v_it++)));

    /* "cupy/cuda/memory.pyx":1132
 *             # Otherwise, advance iterator and remove chunk
 *             self.index.erase(postincrement(it))
 *             cpython.Py_DECREF(chunk)             # <<<<<<<<<<<<<<
 * 
 *             # Freeing means removing the chunks from this arena. Because
*/
    Py_DECREF(((PyObject *)__pyx_v_chunk));

    /* "cupy/cuda/memory.pyx":1136
 *             # Freeing means removing the chunks from this arena. Because
 *             # chunks hold on the arena, we need to break that cycle.
 *             chunk.arena = None             # <<<<<<<<<<<<<<
 *             bytes_freed += chunk.size
 *             del chunk
*/
    __Pyx_INCREF(Py_None);
    __Pyx_GIVEREF(Py_None);
    __Pyx_GOTREF((PyObject *)__pyx_v_chunk->arena);
    __Pyx_DECREF((PyObject *)__pyx_v_chunk->arena);
    __pyx_v_chunk->arena = ((struct __pyx_obj_4cupy_4cuda_6memory__Arena *)Py_None);

    /* "cupy/cuda/memory.pyx":1137
 *             # chunks hold on the arena, we need to break that cycle.
 *             chunk.arena = None
 *             bytes_freed += chunk.size             # <<<<<<<<<<<<<<
 *             del chunk
 * 
*/
    __pyx_v_bytes_freed = (__pyx_v_bytes_freed + __pyx_v_chunk->size);

    /* "cupy/cuda/memory.pyx":1138
 *             chunk.arena = None
 *             bytes_freed += chunk.size
 *             del chunk             # <<<<<<<<<<<<<<
 * 
 *         return bytes_freed
*/
    __Pyx_DECREF((PyObject *)__pyx_v_chunk); __pyx_v_chunk = 0;
    __pyx_L3_continue:;
  }

  /* "cupy/cuda/memory.pyx":1140
 *             del chunk
 * 
 *         return bytes_freed             # <<<<<<<<<<<<<<
 * 
 *     cdef bint try_remove_chunk(self, _Chunk chunk) except -1:
*/
  __pyx_r = __pyx_v_bytes_freed;
  goto __pyx_L0;

  /* "cupy/cuda/memory.pyx":1114
 *         self.index.insert(index_type(chunk.size, <uintptr_t><void *>chunk))
 * 
 *     cdef size_t free_all(self) except -1:             # <<<<<<<<<<<<<<
 *         """Frees all chunks (that can be free'd, split ones cannot).
 *         """
*/

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("cupy.cuda.memory._Arena.free_all", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1L;
  __pyx_L0:;
  __Pyx_XDECREF((PyObject *)__pyx_v_chunk);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "cupy/cuda/memory.pyx":1142
 *         return bytes_freed
 * 
 *     cdef bint try_remove_chunk(self, _Chunk chunk) except -1:             # <<<<<<<<<<<<<<
 *         """Checks if the chunk appears to be currently free and if yes
 *         unlinks (removes) it from the arena (free chunks).
*/

static int __pyx_f_4cupy_4cuda_6memory_6_Arena_try_remove_chunk(struct __pyx_obj_4cupy_4cuda_6memory__Arena *__pyx_v_self, struct __pyx_obj_4cupy_4cuda_6memory__Chunk *__pyx_v_chunk) {
  std::set<__pyx_t_4cupy_4cuda_6memory_index_type> ::iterator __pyx_v_it;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __pyx_t_4cupy_4cuda_6memory_index_type __pyx_t_1;
  int __pyx_t_2;
  void *__pyx_t_3;
  PyObject *__pyx_t_4 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("try_remove_chunk", 0);
  __Pyx_INCREF((PyObject *)__pyx_v_chunk);

  /* "cupy/cuda/memory.pyx":1153
 *         cdef std_set[index_type].iterator it
 * 
 *         it = self.index.find(index_type(chunk.size, <uintptr_t><void *>chunk))             # <<<<<<<<<<<<<<
 *         if it == self.index.end():
 *             return False  # chunk seems not to be free
*/
  try {
    __pyx_t_1 = __pyx_t_4cupy_4cuda_6memory_index_type(__pyx_v_chunk->size, ((uintptr_t)((void *)__pyx_v_chunk)));
  } catch(...) {
    __Pyx_CppExn2PyErr();
    __PYX_ERR(0, 1153, __pyx_L1_error)
  }
  __pyx_v_it = __pyx_v_self->index.find(__pyx_t_1);

  /* "cupy/cuda/memory.pyx":1154
 * 
 *         it = self.index.find(index_type(chunk.size, <uintptr_t><void *>chunk))
 *         if it == self.index.end():             # <<<<<<<<<<<<<<
 *             return False  # chunk seems not to be free
 * 
*/
  __pyx_t_2 = (__pyx_v_it == __pyx_v_self->index.end());
  if (__pyx_t_2) {

    /* "cupy/cuda/memory.pyx":1155
 *         it = self.index.find(index_type(chunk.size, <uintptr_t><void *>chunk))
 *         if it == self.index.end():
 *             return False  # chunk seems not to be free             # <<<<<<<<<<<<<<
 * 
 *         chunk = <_Chunk><void *>deref(it).second
*/
    __pyx_r = 0;
    goto __pyx_L0;

    /* "cupy/cuda/memory.pyx":1154
 * 
 *         it = self.index.find(index_type(chunk.size, <uintptr_t><void *>chunk))
 *         if it == self.index.end():             # <<<<<<<<<<<<<<
 *             return False  # chunk seems not to be free
 * 
*/
  }

  /* "cupy/cuda/memory.pyx":1157
 *             return False  # chunk seems not to be free
 * 
 *         chunk = <_Chunk><void *>deref(it).second             # <<<<<<<<<<<<<<
 *         self.index.erase(it)  # Remove and decref
 *         cpython.Py_DECREF(chunk)
*/
  __pyx_t_3 = ((void *)(*__pyx_v_it).second);
  __pyx_t_4 = ((PyObject *)__pyx_t_3);
  __Pyx_INCREF(__pyx_t_4);
  __Pyx_DECREF_SET(__pyx_v_chunk, ((struct __pyx_obj_4cupy_4cuda_6memory__Chunk *)__pyx_t_4));
  __pyx_t_4 = 0;

  /* "cupy/cuda/memory.pyx":1158
 * 
 *         chunk = <_Chunk><void *>deref(it).second
 *         self.index.erase(it)  # Remove and decref             # <<<<<<<<<<<<<<
 *         cpython.Py_DECREF(chunk)
 *         return True
*/
  (void)(__pyx_v_self->index.erase(__pyx_v_it));

  /* "cupy/cuda/memory.pyx":1159
 *         chunk = <_Chunk><void *>deref(it).second
 *         self.index.erase(it)  # Remove and decref
 *         cpython.Py_DECREF(chunk)             # <<<<<<<<<<<<<<
 *         return True
 * 
*/
  Py_DECREF(((PyObject *)__pyx_v_chunk));

  /* "cupy/cuda/memory.pyx":1160
 *         self.index.erase(it)  # Remove and decref
 *         cpython.Py_DECREF(chunk)
 *         return True             # <<<<<<<<<<<<<<
 * 
 *     cdef _Chunk get_chunk(self, size_t size):
*/
  __pyx_r = 1;
  goto __pyx_L0;

  /* "cupy/cuda/memory.pyx":1142
 *         return bytes_freed
 * 
 *     cdef bint try_remove_chunk(self, _Chunk chunk) except -1:             # <<<<<<<<<<<<<<
 *         """Checks if the chunk appears to be currently free and if yes
 *         unlinks (removes) it from the arena (free chunks).
*/

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_AddTraceback("cupy.cuda.memory._Arena.try_remove_chunk", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_XDECREF((PyObject *)__pyx_v_chunk);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "cupy/cuda/memory.pyx":1162
 *         return True
 * 
 *     cdef _Chunk get_chunk(self, size_t size):             # <<<<<<<<<<<<<<
 *         """Get a free chunk of at least the given size from the arena.
 *         """
*/

static struct __pyx_obj_4cupy_4cuda_6memory__Chunk *__pyx_f_4cupy_4cuda_6memory_6_Arena_get_chunk(struct __pyx_obj_4cupy_4cuda_6memory__Arena *__pyx_v_self, size_t __pyx_v_size) {
  std::set<__pyx_t_4cupy_4cuda_6memory_index_type> ::iterator __pyx_v_it;
  struct __pyx_obj_4cupy_4cuda_6memory__Chunk *__pyx_v_chunk = 0;
  struct __pyx_obj_4cupy_4cuda_6memory__Chunk *__pyx_v_remaining = NULL;
  struct __pyx_obj_4cupy_4cuda_6memory__Chunk *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  Py_ssize_t __pyx_t_2;
  int __pyx_t_3;
  __pyx_t_4cupy_4cuda_6memory_index_type __pyx_t_4;
  void *__pyx_t_5;
  struct __pyx_opt_args_4cupy_4cuda_6memory_6_Arena_insert_chunk __pyx_t_6;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("get_chunk", 0);

  /* "cupy/cuda/memory.pyx":1166
 *         """
 *         cdef std_set[index_type].iterator it
 *         cdef _Chunk chunk = None             # <<<<<<<<<<<<<<
 * 
 *         if len(self._pending_free):
*/
  __Pyx_INCREF(Py_None);
  __pyx_v_chunk = ((struct __pyx_obj_4cupy_4cuda_6memory__Chunk *)Py_None);

  /* "cupy/cuda/memory.pyx":1168
 *         cdef _Chunk chunk = None
 * 
 *         if len(self._pending_free):             # <<<<<<<<<<<<<<
 *             self._commit_pending_free()
 * 
*/
  __pyx_t_1 = __pyx_v_self->_pending_free;
  __Pyx_INCREF(__pyx_t_1);
  if (unlikely(__pyx_t_1 == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "object of type 'NoneType' has no len()");
    __PYX_ERR(0, 1168, __pyx_L1_error)
  }
  __pyx_t_2 = __Pyx_PyList_GET_SIZE(__pyx_t_1); if (unlikely(__pyx_t_2 == ((Py_ssize_t)-1))) __PYX_ERR(0, 1168, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_3 = (__pyx_t_2 != 0);
  if (__pyx_t_3) {

    /* "cupy/cuda/memory.pyx":1169
 * 
 *         if len(self._pending_free):
 *             self._commit_pending_free()             # <<<<<<<<<<<<<<
 * 
 *         it = self.index.lower_bound(index_type(size, 0))
*/
    __pyx_t_1 = __pyx_f_4cupy_4cuda_6memory_6_Arena__commit_pending_free(__pyx_v_self); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1169, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

    /* "cupy/cuda/memory.pyx":1168
 *         cdef _Chunk chunk = None
 * 
 *         if len(self._pending_free):             # <<<<<<<<<<<<<<
 *             self._commit_pending_free()
 * 
*/
  }

  /* "cupy/cuda/memory.pyx":1171
 *             self._commit_pending_free()
 * 
 *         it = self.index.lower_bound(index_type(size, 0))             # <<<<<<<<<<<<<<
 *         if it != self.index.end():
 *             chunk = <_Chunk><void *>deref(it).second
*/
  try {
    __pyx_t_4 = __pyx_t_4cupy_4cuda_6memory_index_type(__pyx_v_size, 0);
  } catch(...) {
    __Pyx_CppExn2PyErr();
    __PYX_ERR(0, 1171, __pyx_L1_error)
  }
  __pyx_v_it = __pyx_v_self->index.lower_bound(__pyx_t_4);

  /* "cupy/cuda/memory.pyx":1172
 * 
 *         it = self.index.lower_bound(index_type(size, 0))
 *         if it != self.index.end():             # <<<<<<<<<<<<<<
 *             chunk = <_Chunk><void *>deref(it).second
 *             self.index.erase(it)  # Remove and decref
*/
  __pyx_t_3 = (__pyx_v_it != __pyx_v_self->index.end());
  if (__pyx_t_3) {

    /* "cupy/cuda/memory.pyx":1173
 *         it = self.index.lower_bound(index_type(size, 0))
 *         if it != self.index.end():
 *             chunk = <_Chunk><void *>deref(it).second             # <<<<<<<<<<<<<<
 *             self.index.erase(it)  # Remove and decref
 *             cpython.Py_DECREF(chunk)
*/
    __pyx_t_5 = ((void *)(*__pyx_v_it).second);
    __pyx_t_1 = ((PyObject *)__pyx_t_5);
    __Pyx_INCREF(__pyx_t_1);
    __Pyx_DECREF_SET(__pyx_v_chunk, ((struct __pyx_obj_4cupy_4cuda_6memory__Chunk *)__pyx_t_1));
    __pyx_t_1 = 0;

    /* "cupy/cuda/memory.pyx":1174
 *         if it != self.index.end():
 *             chunk = <_Chunk><void *>deref(it).second
 *             self.index.erase(it)  # Remove and decref             # <<<<<<<<<<<<<<
 *             cpython.Py_DECREF(chunk)
 *         else:
*/
    (void)(__pyx_v_self->index.erase(__pyx_v_it));

    /* "cupy/cuda/memory.pyx":1175
 *             chunk = <_Chunk><void *>deref(it).second
 *             self.index.erase(it)  # Remove and decref
 *             cpython.Py_DECREF(chunk)             # <<<<<<<<<<<<<<
 *         else:
 *             return None
*/
    Py_DECREF(((PyObject *)__pyx_v_chunk));

    /* "cupy/cuda/memory.pyx":1172
 * 
 *         it = self.index.lower_bound(index_type(size, 0))
 *         if it != self.index.end():             # <<<<<<<<<<<<<<
 *             chunk = <_Chunk><void *>deref(it).second
 *             self.index.erase(it)  # Remove and decref
*/
    goto __pyx_L4;
  }

  /* "cupy/cuda/memory.pyx":1177
 *             cpython.Py_DECREF(chunk)
 *         else:
 *             return None             # <<<<<<<<<<<<<<
 * 
 *         remaining = chunk.split(size)
*/
  /*else*/ {
    __Pyx_XDECREF((PyObject *)__pyx_r);
    __pyx_r = ((struct __pyx_obj_4cupy_4cuda_6memory__Chunk *)Py_None); __Pyx_INCREF(Py_None);
    goto __pyx_L0;
  }
  __pyx_L4:;

  /* "cupy/cuda/memory.pyx":1179
 *             return None
 * 
 *         remaining = chunk.split(size)             # <<<<<<<<<<<<<<
 *         if remaining is not None:
 *             self.insert_chunk(remaining, merge=False)
*/
  __pyx_t_1 = ((PyObject *)__pyx_f_4cupy_4cuda_6memory_6_Chunk_split(__pyx_v_chunk, __pyx_v_size, 0)); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1179, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_v_remaining = ((struct __pyx_obj_4cupy_4cuda_6memory__Chunk *)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "cupy/cuda/memory.pyx":1180
 * 
 *         remaining = chunk.split(size)
 *         if remaining is not None:             # <<<<<<<<<<<<<<
 *             self.insert_chunk(remaining, merge=False)
 *         return chunk
*/
  __pyx_t_3 = (((PyObject *)__pyx_v_remaining) != Py_None);
  if (__pyx_t_3) {

    /* "cupy/cuda/memory.pyx":1181
 *         remaining = chunk.split(size)
 *         if remaining is not None:
 *             self.insert_chunk(remaining, merge=False)             # <<<<<<<<<<<<<<
 *         return chunk
 * 
*/
    __pyx_t_6.__pyx_n = 1;
    __pyx_t_6.merge = 0;
    __pyx_t_1 = __pyx_f_4cupy_4cuda_6memory_6_Arena_insert_chunk(__pyx_v_self, __pyx_v_remaining, &__pyx_t_6); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1181, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

    /* "cupy/cuda/memory.pyx":1180
 * 
 *         remaining = chunk.split(size)
 *         if remaining is not None:             # <<<<<<<<<<<<<<
 *             self.insert_chunk(remaining, merge=False)
 *         return chunk
*/
  }

  /* "cupy/cuda/memory.pyx":1182
 *         if remaining is not None:
 *             self.insert_chunk(remaining, merge=False)
 *         return chunk             # <<<<<<<<<<<<<<
 * 
 *     cdef _get_size(self):
*/
  __Pyx_XDECREF((PyObject *)__pyx_r);
  __Pyx_INCREF((PyObject *)__pyx_v_chunk);
  __pyx_r = __pyx_v_chunk;
  goto __pyx_L0;

  /* "cupy/cuda/memory.pyx":1162
 *         return True
 * 
 *     cdef _Chunk get_chunk(self, size_t size):             # <<<<<<<<<<<<<<
 *         """Get a free chunk of at least the given size from the arena.
 *         """
*/

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("cupy.cuda.memory._Arena.get_chunk", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XDECREF((PyObject *)__pyx_v_chunk);
  __Pyx_XDECREF((PyObject *)__pyx_v_remaining);
  __Pyx_XGIVEREF((PyObject *)__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "cupy/cuda/memory.pyx":1184
 *         return chunk
 * 
 *     cdef _get_size(self):             # <<<<<<<<<<<<<<
 *         return self.index.size()
 * 
*/

static PyObject *__pyx_f_4cupy_4cuda_6memory_6_Arena__get_size(struct __pyx_obj_4cupy_4cuda_6memory__Arena *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("_get_size", 0);

  /* "cupy/cuda/memory.pyx":1185
 * 
 *     cdef _get_size(self):
 *         return self.index.size()             # <<<<<<<<<<<<<<
 * 
 *     cdef _index_to_python(self):
*/
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __Pyx_PyLong_FromSize_t(__pyx_v_self->index.size()); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1185, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "cupy/cuda/memory.pyx":1184
 *         return chunk
 * 
 *     cdef _get_size(self):             # <<<<<<<<<<<<<<
 *         return self.index.size()
 * 
*/

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("cupy.cuda.memory._Arena._get_size", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "cupy/cuda/memory.pyx":1187
 *         return self.index.size()
 * 
 *     cdef _index_to_python(self):             # <<<<<<<<<<<<<<
 *         # For debug purpose, expose index into Python.
 *         cdef std_set[index_type].iterator it
*/

static PyObject *__pyx_f_4cupy_4cuda_6memory_6_Arena__index_to_python(struct __pyx_obj_4cupy_4cuda_6memory__Arena *__pyx_v_self) {
  std::set<__pyx_t_4cupy_4cuda_6memory_index_type> ::iterator __pyx_v_it;
  PyObject *__pyx_v_result = 0;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_t_2;
  void *__pyx_t_3;
  PyObject *__pyx_t_4 = NULL;
  int __pyx_t_5;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("_index_to_python", 0);

  /* "cupy/cuda/memory.pyx":1190
 *         # For debug purpose, expose index into Python.
 *         cdef std_set[index_type].iterator it
 *         cdef list result = []             # <<<<<<<<<<<<<<
 * 
 *         self._commit_pending_free()
*/
  __pyx_t_1 = PyList_New(0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1190, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_v_result = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "cupy/cuda/memory.pyx":1192
 *         cdef list result = []
 * 
 *         self._commit_pending_free()             # <<<<<<<<<<<<<<
 * 
 *         it = self.index.begin()
*/
  __pyx_t_1 = __pyx_f_4cupy_4cuda_6memory_6_Arena__commit_pending_free(__pyx_v_self); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1192, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "cupy/cuda/memory.pyx":1194
 *         self._commit_pending_free()
 * 
 *         it = self.index.begin()             # <<<<<<<<<<<<<<
 *         while it != self.index.end():
 *             result.append((deref(it).first, <object><void *>deref(it).second))
*/
  __pyx_v_it = __pyx_v_self->index.begin();

  /* "cupy/cuda/memory.pyx":1195
 * 
 *         it = self.index.begin()
 *         while it != self.index.end():             # <<<<<<<<<<<<<<
 *             result.append((deref(it).first, <object><void *>deref(it).second))
 *             postincrement(it)
*/
  while (1) {
    __pyx_t_2 = (__pyx_v_it != __pyx_v_self->index.end());
    if (!__pyx_t_2) break;

    /* "cupy/cuda/memory.pyx":1196
 *         it = self.index.begin()
 *         while it != self.index.end():
 *             result.append((deref(it).first, <object><void *>deref(it).second))             # <<<<<<<<<<<<<<
 *             postincrement(it)
 * 
*/
    __pyx_t_1 = __Pyx_PyLong_FromSize_t((*__pyx_v_it).first); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1196, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_t_3 = ((void *)(*__pyx_v_it).second);
    __pyx_t_4 = PyTuple_New(2); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 1196, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_GIVEREF(__pyx_t_1);
    if (__Pyx_PyTuple_SET_ITEM(__pyx_t_4, 0, __pyx_t_1) != (0)) __PYX_ERR(0, 1196, __pyx_L1_error);
    __Pyx_INCREF(((PyObject *)__pyx_t_3));
    __Pyx_GIVEREF(((PyObject *)__pyx_t_3));
    if (__Pyx_PyTuple_SET_ITEM(__pyx_t_4, 1, ((PyObject *)__pyx_t_3)) != (0)) __PYX_ERR(0, 1196, __pyx_L1_error);
    __pyx_t_1 = 0;
    __pyx_t_5 = __Pyx_PyList_Append(__pyx_v_result, __pyx_t_4); if (unlikely(__pyx_t_5 == ((int)-1))) __PYX_ERR(0, 1196, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

    /* "cupy/cuda/memory.pyx":1197
 *         while it != self.index.end():
 *             result.append((deref(it).first, <object><void *>deref(it).second))
 *             postincrement(it)             # <<<<<<<<<<<<<<
 * 
 *         return result
*/
    (void)((__pyx_v_it++));
  }

  /* "cupy/cuda/memory.pyx":1199
 *             postincrement(it)
 * 
 *         return result             # <<<<<<<<<<<<<<
 * 
 * 
*/
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_v_result);
  __pyx_r = __pyx_v_result;
  goto __pyx_L0;

  /* "cupy/cuda/memory.pyx":1187
 *         return self.index.size()
 * 
 *     cdef _index_to_python(self):             # <<<<<<<<<<<<<<
 *         # For debug purpose, expose index into Python.
 *         cdef std_set[index_type].iterator it
*/

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_AddTraceback("cupy.cuda.memory._Arena._index_to_python", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_result);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     raise TypeError, "no default __reduce__ due to non-trivial __cinit__"
 * def __setstate_cython__(self, __pyx_state):
*/

/* Python wrapper */
static PyObject *__pyx_pw_4cupy_4cuda_6memory_6_Arena_3__reduce_cython__(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
PyDoc_STRVAR(__pyx_doc_4cupy_4cuda_6memory_6_Arena_2__reduce_cython__, "_Arena.__reduce_cython__(self)");
static PyObject *__pyx_pw_4cupy_4cuda_6memory_6_Arena_3__reduce_cython__(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
) {
  #if !CYTHON_METH_FASTCALL
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  #endif
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__reduce_cython__ (wrapper)", 0);
  #if !CYTHON_METH_FASTCALL
  #if CYTHON_ASSUME_SAFE_SIZE
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return NULL;
  #endif
  #endif
  __pyx_kwvalues = __Pyx_KwValues_FASTCALL(__pyx_args, __pyx_nargs);
  if (unlikely(__pyx_nargs > 0)) { __Pyx_RaiseArgtupleInvalid("__reduce_cython__", 1, 0, 0, __pyx_nargs); return NULL; }
  const Py_ssize_t __pyx_kwds_len = unlikely(__pyx_kwds) ? __Pyx_NumKwargs_FASTCALL(__pyx_kwds) : 0;
  if (unlikely(__pyx_kwds_len < 0)) return NULL;
  if (unlikely(__pyx_kwds_len > 0)) {__Pyx_RejectKeywords("__reduce_cython__", __pyx_kwds); return NULL;}
  __pyx_r = __pyx_pf_4cupy_4cuda_6memory_6_Arena_2__reduce_cython__(((struct __pyx_obj_4cupy_4cuda_6memory__Arena *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4cupy_4cuda_6memory_6_Arena_2__reduce_cython__(CYTHON_UNUSED struct __pyx_obj_4cupy_4cuda_6memory__Arena *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__reduce_cython__", 0);

  /* "(tree fragment)":2
 * def __reduce_cython__(self):
 *     raise TypeError, "no default __reduce__ due to non-trivial __cinit__"             # <<<<<<<<<<<<<<
 * def __setstate_cython__(self, __pyx_state):
 *     raise TypeError, "no default __reduce__ due to non-trivial __cinit__"
*/
  __Pyx_Raise(((PyObject *)(((PyTypeObject*)PyExc_TypeError))), __pyx_mstate_global->__pyx_kp_u_no_default___reduce___due_to_non, 0, 0);
  __PYX_ERR(3, 2, __pyx_L1_error)

  /* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     raise TypeError, "no default __reduce__ due to non-trivial __cinit__"
 * def __setstate_cython__(self, __pyx_state):
*/

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_AddTraceback("cupy.cuda.memory._Arena.__reduce_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":3
 * def __reduce_cython__(self):
 *     raise TypeError, "no default __reduce__ due to non-trivial __cinit__"
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     raise TypeError, "no default __reduce__ due to non-trivial __cinit__"
*/

/* Python wrapper */
static PyObject *__pyx_pw_4cupy_4cuda_6memory_6_Arena_5__setstate_cython__(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
PyDoc_STRVAR(__pyx_doc_4cupy_4cuda_6memory_6_Arena_4__setstate_cython__, "_Arena.__setstate_cython__(self, __pyx_state)");
static PyObject *__pyx_pw_4cupy_4cuda_6memory_6_Arena_5__setstate_cython__(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
) {
  CYTHON_UNUSED PyObject *__pyx_v___pyx_state = 0;
  #if !CYTHON_METH_FASTCALL
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  #endif
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject* values[1] = {0};
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__setstate_cython__ (wrapper)", 0);
  #if !CYTHON_METH_FASTCALL
  #if CYTHON_ASSUME_SAFE_SIZE
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return NULL;
  #endif
  #endif
  __pyx_kwvalues = __Pyx_KwValues_FASTCALL(__pyx_args, __pyx_nargs);
  {
    PyObject ** const __pyx_pyargnames[] = {&__pyx_mstate_global->__pyx_n_u_pyx_state,0};
    const Py_ssize_t __pyx_kwds_len = (__pyx_kwds) ? __Pyx_NumKwargs_FASTCALL(__pyx_kwds) : 0;
    if (unlikely(__pyx_kwds_len < 0)) __PYX_ERR(3, 3, __pyx_L3_error)
    if (__pyx_kwds_len > 0) {
      switch (__pyx_nargs) {
        case  1:
        values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(3, 3, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      const Py_ssize_t kwd_pos_args = __pyx_nargs;
      if (__Pyx_ParseKeywords(__pyx_kwds, __pyx_kwvalues, __pyx_pyargnames, 0, values, kwd_pos_args, __pyx_kwds_len, "__setstate_cython__", 0) < (0)) __PYX_ERR(3, 3, __pyx_L3_error)
      for (Py_ssize_t i = __pyx_nargs; i < 1; i++) {
        if (unlikely(!values[i])) { __Pyx_RaiseArgtupleInvalid("__setstate_cython__", 1, 1, 1, i); __PYX_ERR(3, 3, __pyx_L3_error) }
      }
    } else if (unlikely(__pyx_nargs != 1)) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
      if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(3, 3, __pyx_L3_error)
    }
    __pyx_v___pyx_state = values[0];
  }
  goto __pyx_L6_skip;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("__setstate_cython__", 1, 1, 1, __pyx_nargs); __PYX_ERR(3, 3, __pyx_L3_error)
  __pyx_L6_skip:;
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_AddTraceback("cupy.cuda.memory._Arena.__setstate_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_4cupy_4cuda_6memory_6_Arena_4__setstate_cython__(((struct __pyx_obj_4cupy_4cuda_6memory__Arena *)__pyx_v_self), __pyx_v___pyx_state);

  /* function exit code */
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4cupy_4cuda_6memory_6_Arena_4__setstate_cython__(CYTHON_UNUSED struct __pyx_obj_4cupy_4cuda_6memory__Arena *__pyx_v_self, CYTHON_UNUSED PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__setstate_cython__", 0);

  /* "(tree fragment)":4
 *     raise TypeError, "no default __reduce__ due to non-trivial __cinit__"
 * def __setstate_cython__(self, __pyx_state):
 *     raise TypeError, "no default __reduce__ due to non-trivial __cinit__"             # <<<<<<<<<<<<<<
*/
  __Pyx_Raise(((PyObject *)(((PyTypeObject*)PyExc_TypeError))), __pyx_mstate_global->__pyx_kp_u_no_default___reduce___due_to_non, 0, 0);
  __PYX_ERR(3, 4, __pyx_L1_error)

  /* "(tree fragment)":3
 * def __reduce_cython__(self):
 *     raise TypeError, "no default __reduce__ due to non-trivial __cinit__"
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     raise TypeError, "no default __reduce__ due to non-trivial __cinit__"
*/

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_AddTraceback("cupy.cuda.memory._Arena.__setstate_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "cupy/cuda/memory.pyx":1204
 * # cpdef because uint-tested
 * # module-level function can be inlined
 * cpdef inline dict _parse_limit_string(limit=None):             # <<<<<<<<<<<<<<
 *     if limit is None:
 *         limit = os.environ.get('CUPY_GPU_MEMORY_LIMIT')
*/

static PyObject *__pyx_pw_4cupy_4cuda_6memory_23_parse_limit_string(PyObject *__pyx_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
static CYTHON_INLINE PyObject *__pyx_f_4cupy_4cuda_6memory__parse_limit_string(CYTHON_UNUSED int __pyx_skip_dispatch, struct __pyx_opt_args_4cupy_4cuda_6memory__parse_limit_string *__pyx_optional_args) {
  PyObject *__pyx_v_limit = ((PyObject *)Py_None);
  PyObject *__pyx_v_size = NULL;
  PyObject *__pyx_v_fraction = NULL;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  size_t __pyx_t_6;
  double __pyx_t_7;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("_parse_limit_string", 0);
  if (__pyx_optional_args) {
    if (__pyx_optional_args->__pyx_n > 0) {
      __pyx_v_limit = __pyx_optional_args->limit;
    }
  }
  __Pyx_INCREF(__pyx_v_limit);

  /* "cupy/cuda/memory.pyx":1205
 * # module-level function can be inlined
 * cpdef inline dict _parse_limit_string(limit=None):
 *     if limit is None:             # <<<<<<<<<<<<<<
 *         limit = os.environ.get('CUPY_GPU_MEMORY_LIMIT')
 *     size = None
*/
  __pyx_t_1 = (__pyx_v_limit == Py_None);
  if (__pyx_t_1) {

    /* "cupy/cuda/memory.pyx":1206
 * cpdef inline dict _parse_limit_string(limit=None):
 *     if limit is None:
 *         limit = os.environ.get('CUPY_GPU_MEMORY_LIMIT')             # <<<<<<<<<<<<<<
 *     size = None
 *     fraction = None
*/
    __Pyx_GetModuleGlobalName(__pyx_t_4, __pyx_mstate_global->__pyx_n_u_os); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 1206, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __pyx_t_5 = __Pyx_PyObject_GetAttrStr(__pyx_t_4, __pyx_mstate_global->__pyx_n_u_environ); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 1206, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __pyx_t_3 = __pyx_t_5;
    __Pyx_INCREF(__pyx_t_3);
    __pyx_t_6 = 0;
    {
      PyObject *__pyx_callargs[2] = {__pyx_t_3, __pyx_mstate_global->__pyx_n_u_CUPY_GPU_MEMORY_LIMIT};
      __pyx_t_2 = __Pyx_PyObject_FastCallMethod((PyObject*)__pyx_mstate_global->__pyx_n_u_get, __pyx_callargs+__pyx_t_6, (2-__pyx_t_6) | (1*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
      __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
      __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
      if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1206, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_2);
    }
    __Pyx_DECREF_SET(__pyx_v_limit, __pyx_t_2);
    __pyx_t_2 = 0;

    /* "cupy/cuda/memory.pyx":1205
 * # module-level function can be inlined
 * cpdef inline dict _parse_limit_string(limit=None):
 *     if limit is None:             # <<<<<<<<<<<<<<
 *         limit = os.environ.get('CUPY_GPU_MEMORY_LIMIT')
 *     size = None
*/
  }

  /* "cupy/cuda/memory.pyx":1207
 *     if limit is None:
 *         limit = os.environ.get('CUPY_GPU_MEMORY_LIMIT')
 *     size = None             # <<<<<<<<<<<<<<
 *     fraction = None
 *     if limit is not None:
*/
  __Pyx_INCREF(Py_None);
  __pyx_v_size = ((PyObject*)Py_None);

  /* "cupy/cuda/memory.pyx":1208
 *         limit = os.environ.get('CUPY_GPU_MEMORY_LIMIT')
 *     size = None
 *     fraction = None             # <<<<<<<<<<<<<<
 *     if limit is not None:
 *         if limit.endswith('%'):
*/
  __Pyx_INCREF(Py_None);
  __pyx_v_fraction = Py_None;

  /* "cupy/cuda/memory.pyx":1209
 *     size = None
 *     fraction = None
 *     if limit is not None:             # <<<<<<<<<<<<<<
 *         if limit.endswith('%'):
 *             fraction = float(limit[:-1]) / 100.0
*/
  __pyx_t_1 = (__pyx_v_limit != Py_None);
  if (__pyx_t_1) {

    /* "cupy/cuda/memory.pyx":1210
 *     fraction = None
 *     if limit is not None:
 *         if limit.endswith('%'):             # <<<<<<<<<<<<<<
 *             fraction = float(limit[:-1]) / 100.0
 *         else:
*/
    __pyx_t_5 = __pyx_v_limit;
    __Pyx_INCREF(__pyx_t_5);
    __pyx_t_6 = 0;
    {
      PyObject *__pyx_callargs[2] = {__pyx_t_5, __pyx_mstate_global->__pyx_kp_u__5};
      __pyx_t_2 = __Pyx_PyObject_FastCallMethod((PyObject*)__pyx_mstate_global->__pyx_n_u_endswith, __pyx_callargs+__pyx_t_6, (2-__pyx_t_6) | (1*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
      __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
      if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1210, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_2);
    }
    __pyx_t_1 = __Pyx_PyObject_IsTrue(__pyx_t_2); if (unlikely((__pyx_t_1 < 0))) __PYX_ERR(0, 1210, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    if (__pyx_t_1) {

      /* "cupy/cuda/memory.pyx":1211
 *     if limit is not None:
 *         if limit.endswith('%'):
 *             fraction = float(limit[:-1]) / 100.0             # <<<<<<<<<<<<<<
 *         else:
 *             size = int(limit)
*/
      __pyx_t_2 = __Pyx_PyObject_GetSlice(__pyx_v_limit, 0, -1L, NULL, NULL, &__pyx_mstate_global->__pyx_slice[0], 0, 1, 1); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1211, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_2);
      __pyx_t_7 = __Pyx_PyObject_AsDouble(__pyx_t_2); if (unlikely(__PYX_CHECK_FLOAT_EXCEPTION(__pyx_t_7, ((double)((double)-1))) && PyErr_Occurred())) __PYX_ERR(0, 1211, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
      __pyx_t_2 = PyFloat_FromDouble((__pyx_t_7 / 100.0)); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1211, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_2);
      __Pyx_DECREF_SET(__pyx_v_fraction, __pyx_t_2);
      __pyx_t_2 = 0;

      /* "cupy/cuda/memory.pyx":1210
 *     fraction = None
 *     if limit is not None:
 *         if limit.endswith('%'):             # <<<<<<<<<<<<<<
 *             fraction = float(limit[:-1]) / 100.0
 *         else:
*/
      goto __pyx_L5;
    }

    /* "cupy/cuda/memory.pyx":1213
 *             fraction = float(limit[:-1]) / 100.0
 *         else:
 *             size = int(limit)             # <<<<<<<<<<<<<<
 *     return {'size': size, 'fraction': fraction}
 * 
*/
    /*else*/ {
      __pyx_t_2 = __Pyx_PyNumber_Int(__pyx_v_limit); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1213, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_2);
      __Pyx_DECREF_SET(__pyx_v_size, ((PyObject*)__pyx_t_2));
      __pyx_t_2 = 0;
    }
    __pyx_L5:;

    /* "cupy/cuda/memory.pyx":1209
 *     size = None
 *     fraction = None
 *     if limit is not None:             # <<<<<<<<<<<<<<
 *         if limit.endswith('%'):
 *             fraction = float(limit[:-1]) / 100.0
*/
  }

  /* "cupy/cuda/memory.pyx":1214
 *         else:
 *             size = int(limit)
 *     return {'size': size, 'fraction': fraction}             # <<<<<<<<<<<<<<
 * 
 * 
*/
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_2 = __Pyx_PyDict_NewPresized(2); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1214, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_t_2, __pyx_mstate_global->__pyx_n_u_size, __pyx_v_size) < (0)) __PYX_ERR(0, 1214, __pyx_L1_error)
  if (PyDict_SetItem(__pyx_t_2, __pyx_mstate_global->__pyx_n_u_fraction, __pyx_v_fraction) < (0)) __PYX_ERR(0, 1214, __pyx_L1_error)
  __pyx_r = ((PyObject*)__pyx_t_2);
  __pyx_t_2 = 0;
  goto __pyx_L0;

  /* "cupy/cuda/memory.pyx":1204
 * # cpdef because uint-tested
 * # module-level function can be inlined
 * cpdef inline dict _parse_limit_string(limit=None):             # <<<<<<<<<<<<<<
 *     if limit is None:
 *         limit = os.environ.get('CUPY_GPU_MEMORY_LIMIT')
*/

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_AddTraceback("cupy.cuda.memory._parse_limit_string", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_size);
  __Pyx_XDECREF(__pyx_v_fraction);
  __Pyx_XDECREF(__pyx_v_limit);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_pw_4cupy_4cuda_6memory_23_parse_limit_string(PyObject *__pyx_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
PyDoc_STRVAR(__pyx_doc_4cupy_4cuda_6memory_22_parse_limit_string, "_parse_limit_string(limit=None) -> dict");
static PyObject *__pyx_pw_4cupy_4cuda_6memory_23_parse_limit_string(PyObject *__pyx_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
) {
  PyObject *__pyx_v_limit = 0;
  #if !CYTHON_METH_FASTCALL
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  #endif
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject* values[1] = {0};
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("_parse_limit_string (wrapper)", 0);
  #if !CYTHON_METH_FASTCALL
  #if CYTHON_ASSUME_SAFE_SIZE
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return NULL;
  #endif
  #endif
  __pyx_kwvalues = __Pyx_KwValues_FASTCALL(__pyx_args, __pyx_nargs);
  {
    PyObject ** const __pyx_pyargnames[] = {&__pyx_mstate_global->__pyx_n_u_limit,0};
    const Py_ssize_t __pyx_kwds_len = (__pyx_kwds) ? __Pyx_NumKwargs_FASTCALL(__pyx_kwds) : 0;
    if (unlikely(__pyx_kwds_len < 0)) __PYX_ERR(0, 1204, __pyx_L3_error)
    if (__pyx_kwds_len > 0) {
      switch (__pyx_nargs) {
        case  1:
        values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(0, 1204, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      const Py_ssize_t kwd_pos_args = __pyx_nargs;
      if (__Pyx_ParseKeywords(__pyx_kwds, __pyx_kwvalues, __pyx_pyargnames, 0, values, kwd_pos_args, __pyx_kwds_len, "_parse_limit_string", 0) < (0)) __PYX_ERR(0, 1204, __pyx_L3_error)
      if (!values[0]) values[0] = __Pyx_NewRef(((PyObject *)Py_None));
    } else {
      switch (__pyx_nargs) {
        case  1:
        values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(0, 1204, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      if (!values[0]) values[0] = __Pyx_NewRef(((PyObject *)Py_None));
    }
    __pyx_v_limit = values[0];
  }
  goto __pyx_L6_skip;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("_parse_limit_string", 0, 0, 1, __pyx_nargs); __PYX_ERR(0, 1204, __pyx_L3_error)
  __pyx_L6_skip:;
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_AddTraceback("cupy.cuda.memory._parse_limit_string", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_4cupy_4cuda_6memory_22_parse_limit_string(__pyx_self, __pyx_v_limit);

  /* function exit code */
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4cupy_4cuda_6memory_22_parse_limit_string(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_limit) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  struct __pyx_opt_args_4cupy_4cuda_6memory__parse_limit_string __pyx_t_2;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("_parse_limit_string", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_2.__pyx_n = 1;
  __pyx_t_2.limit = __pyx_v_limit;
  __pyx_t_1 = __pyx_f_4cupy_4cuda_6memory__parse_limit_string(1, &__pyx_t_2); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1204, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("cupy.cuda.memory._parse_limit_string", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "cupy/cuda/memory.pyx":1258
 *         readonly int _device_id
 * 
 *     def __init__(self, allocator=None):             # <<<<<<<<<<<<<<
 *         if allocator is None:
 *             allocator = _malloc
*/

/* Python wrapper */
static int __pyx_pw_4cupy_4cuda_6memory_22SingleDeviceMemoryPool_1__init__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static int __pyx_pw_4cupy_4cuda_6memory_22SingleDeviceMemoryPool_1__init__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_allocator = 0;
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject* values[1] = {0};
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__init__ (wrapper)", 0);
  #if CYTHON_ASSUME_SAFE_SIZE
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return -1;
  #endif
  __pyx_kwvalues = __Pyx_KwValues_VARARGS(__pyx_args, __pyx_nargs);
  {
    PyObject ** const __pyx_pyargnames[] = {&__pyx_mstate_global->__pyx_n_u_allocator,0};
    const Py_ssize_t __pyx_kwds_len = (__pyx_kwds) ? __Pyx_NumKwargs_VARARGS(__pyx_kwds) : 0;
    if (unlikely(__pyx_kwds_len < 0)) __PYX_ERR(0, 1258, __pyx_L3_error)
    if (__pyx_kwds_len > 0) {
      switch (__pyx_nargs) {
        case  1:
        values[0] = __Pyx_ArgRef_VARARGS(__pyx_args, 0);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(0, 1258, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      const Py_ssize_t kwd_pos_args = __pyx_nargs;
      if (__Pyx_ParseKeywords(__pyx_kwds, __pyx_kwvalues, __pyx_pyargnames, 0, values, kwd_pos_args, __pyx_kwds_len, "__init__", 0) < (0)) __PYX_ERR(0, 1258, __pyx_L3_error)
      if (!values[0]) values[0] = __Pyx_NewRef(((PyObject *)Py_None));
    } else {
      switch (__pyx_nargs) {
        case  1:
        values[0] = __Pyx_ArgRef_VARARGS(__pyx_args, 0);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(0, 1258, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      if (!values[0]) values[0] = __Pyx_NewRef(((PyObject *)Py_None));
    }
    __pyx_v_allocator = values[0];
  }
  goto __pyx_L6_skip;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("__init__", 0, 0, 1, __pyx_nargs); __PYX_ERR(0, 1258, __pyx_L3_error)
  __pyx_L6_skip:;
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_AddTraceback("cupy.cuda.memory.SingleDeviceMemoryPool.__init__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return -1;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_4cupy_4cuda_6memory_22SingleDeviceMemoryPool___init__(((struct __pyx_obj_4cupy_4cuda_6memory_SingleDeviceMemoryPool *)__pyx_v_self), __pyx_v_allocator);

  /* function exit code */
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_4cupy_4cuda_6memory_22SingleDeviceMemoryPool___init__(struct __pyx_obj_4cupy_4cuda_6memory_SingleDeviceMemoryPool *__pyx_v_self, PyObject *__pyx_v_allocator) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  size_t __pyx_t_6;
  int __pyx_t_7;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__init__", 0);
  __Pyx_INCREF(__pyx_v_allocator);

  /* "cupy/cuda/memory.pyx":1259
 * 
 *     def __init__(self, allocator=None):
 *         if allocator is None:             # <<<<<<<<<<<<<<
 *             allocator = _malloc
 *         self._arenas = {}
*/
  __pyx_t_1 = (__pyx_v_allocator == Py_None);
  if (__pyx_t_1) {

    /* "cupy/cuda/memory.pyx":1260
 *     def __init__(self, allocator=None):
 *         if allocator is None:
 *             allocator = _malloc             # <<<<<<<<<<<<<<
 *         self._arenas = {}
 *         self._allocator = allocator
*/
    __Pyx_GetModuleGlobalName(__pyx_t_2, __pyx_mstate_global->__pyx_n_u_malloc); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1260, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_DECREF_SET(__pyx_v_allocator, __pyx_t_2);
    __pyx_t_2 = 0;

    /* "cupy/cuda/memory.pyx":1259
 * 
 *     def __init__(self, allocator=None):
 *         if allocator is None:             # <<<<<<<<<<<<<<
 *             allocator = _malloc
 *         self._arenas = {}
*/
  }

  /* "cupy/cuda/memory.pyx":1261
 *         if allocator is None:
 *             allocator = _malloc
 *         self._arenas = {}             # <<<<<<<<<<<<<<
 *         self._allocator = allocator
 *         self._weakref = weakref.ref(self)
*/
  __pyx_t_2 = __Pyx_PyDict_NewPresized(0); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1261, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_GIVEREF(__pyx_t_2);
  __Pyx_GOTREF(__pyx_v_self->_arenas);
  __Pyx_DECREF(__pyx_v_self->_arenas);
  __pyx_v_self->_arenas = ((PyObject*)__pyx_t_2);
  __pyx_t_2 = 0;

  /* "cupy/cuda/memory.pyx":1262
 *             allocator = _malloc
 *         self._arenas = {}
 *         self._allocator = allocator             # <<<<<<<<<<<<<<
 *         self._weakref = weakref.ref(self)
 *         self._device_id = device.get_device_id()
*/
  __Pyx_INCREF(__pyx_v_allocator);
  __Pyx_GIVEREF(__pyx_v_allocator);
  __Pyx_GOTREF(__pyx_v_self->_allocator);
  __Pyx_DECREF(__pyx_v_self->_allocator);
  __pyx_v_self->_allocator = __pyx_v_allocator;

  /* "cupy/cuda/memory.pyx":1263
 *         self._arenas = {}
 *         self._allocator = allocator
 *         self._weakref = weakref.ref(self)             # <<<<<<<<<<<<<<
 *         self._device_id = device.get_device_id()
 * 
*/
  __pyx_t_3 = NULL;
  __Pyx_GetModuleGlobalName(__pyx_t_4, __pyx_mstate_global->__pyx_n_u_weakref); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 1263, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_t_5 = __Pyx_PyObject_GetAttrStr(__pyx_t_4, __pyx_mstate_global->__pyx_n_u_ref); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 1263, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_5);
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __pyx_t_6 = 1;
  #if CYTHON_UNPACK_METHODS
  if (unlikely(PyMethod_Check(__pyx_t_5))) {
    __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_5);
    assert(__pyx_t_3);
    PyObject* __pyx__function = PyMethod_GET_FUNCTION(__pyx_t_5);
    __Pyx_INCREF(__pyx_t_3);
    __Pyx_INCREF(__pyx__function);
    __Pyx_DECREF_SET(__pyx_t_5, __pyx__function);
    __pyx_t_6 = 0;
  }
  #endif
  {
    PyObject *__pyx_callargs[2] = {__pyx_t_3, ((PyObject *)__pyx_v_self)};
    __pyx_t_2 = __Pyx_PyObject_FastCall((PyObject*)__pyx_t_5, __pyx_callargs+__pyx_t_6, (2-__pyx_t_6) | (__pyx_t_6*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
    __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
    if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1263, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
  }
  __Pyx_GIVEREF(__pyx_t_2);
  __Pyx_GOTREF(__pyx_v_self->_weakref);
  __Pyx_DECREF(__pyx_v_self->_weakref);
  __pyx_v_self->_weakref = __pyx_t_2;
  __pyx_t_2 = 0;

  /* "cupy/cuda/memory.pyx":1264
 *         self._allocator = allocator
 *         self._weakref = weakref.ref(self)
 *         self._device_id = device.get_device_id()             # <<<<<<<<<<<<<<
 * 
 *         self.set_limit(**(_parse_limit_string()))
*/
  __pyx_t_7 = __pyx_f_4cupy_4cuda_6device_get_device_id(0); if (unlikely(__pyx_t_7 == ((int)-1) && PyErr_Occurred())) __PYX_ERR(0, 1264, __pyx_L1_error)
  __pyx_v_self->_device_id = __pyx_t_7;

  /* "cupy/cuda/memory.pyx":1266
 *         self._device_id = device.get_device_id()
 * 
 *         self.set_limit(**(_parse_limit_string()))             # <<<<<<<<<<<<<<
 * 
 *     cpdef _Arena _arena(self, intptr_t stream_ident):
*/
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_mstate_global->__pyx_n_u_set_limit); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1266, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = __pyx_f_4cupy_4cuda_6memory__parse_limit_string(0, NULL); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 1266, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  if (unlikely(__pyx_t_3 == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "argument after ** must be a mapping, not NoneType");
    __PYX_ERR(0, 1266, __pyx_L1_error)
  }
  #if CYTHON_COMPILING_IN_CPYTHON || CYTHON_COMPILING_IN_LIMITED_API
  if (__Pyx_IS_UNIQUELY_REFERENCED(__pyx_t_3, __Pyx_ReferenceSharing_OwnStrongReference)) {
    __pyx_t_5 = __pyx_t_3;
    __pyx_t_3 = 0;
  } else
  #endif
  {
    __pyx_t_5 = PyDict_Copy(__pyx_t_3); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 1266, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  }
  __pyx_t_3 = __Pyx_PyObject_Call(__pyx_t_2, __pyx_mstate_global->__pyx_empty_tuple, __pyx_t_5); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 1266, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "cupy/cuda/memory.pyx":1258
 *         readonly int _device_id
 * 
 *     def __init__(self, allocator=None):             # <<<<<<<<<<<<<<
 *         if allocator is None:
 *             allocator = _malloc
*/

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_AddTraceback("cupy.cuda.memory.SingleDeviceMemoryPool.__init__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_allocator);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "cupy/cuda/memory.pyx":1268
 *         self.set_limit(**(_parse_limit_string()))
 * 
 *     cpdef _Arena _arena(self, intptr_t stream_ident):             # <<<<<<<<<<<<<<
 *         """Returns appropriate arena of a given stream, you should hold the
 *         arena mutex when getting this (to ensure nobody else changes things)
*/

static PyObject *__pyx_pw_4cupy_4cuda_6memory_22SingleDeviceMemoryPool_3_arena(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
static struct __pyx_obj_4cupy_4cuda_6memory__Arena *__pyx_f_4cupy_4cuda_6memory_22SingleDeviceMemoryPool__arena(struct __pyx_obj_4cupy_4cuda_6memory_SingleDeviceMemoryPool *__pyx_v_self, intptr_t __pyx_v_stream_ident, CYTHON_UNUSED int __pyx_skip_dispatch) {
  PyObject *__pyx_v_ret;
  struct __pyx_obj_4cupy_4cuda_6memory__Arena *__pyx_v_arena = 0;
  PyObject *__pyx_v_ref = NULL;
  PyObject *__pyx_v_key = NULL;
  struct __pyx_obj_4cupy_4cuda_6memory__Arena *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  int __pyx_t_3;
  Py_ssize_t __pyx_t_4;
  PyObject *__pyx_t_5 = NULL;
  PyObject *__pyx_t_6 = NULL;
  size_t __pyx_t_7;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("_arena", 0);

  /* "cupy/cuda/memory.pyx":1276
 *         cdef cpython.PyObject *ret
 *         cdef _Arena arena
 *         ref = self._arenas.get(stream_ident, None)             # <<<<<<<<<<<<<<
 *         if ref is not None:
 *             if cpython.PyWeakref_GetRef(ref, &ret) == 1:
*/
  if (unlikely(__pyx_v_self->_arenas == Py_None)) {
    PyErr_Format(PyExc_AttributeError, "'NoneType' object has no attribute '%.30s'", "get");
    __PYX_ERR(0, 1276, __pyx_L1_error)
  }
  __pyx_t_1 = PyLong_FromSsize_t(__pyx_v_stream_ident); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1276, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyDict_GetItemDefault(__pyx_v_self->_arenas, __pyx_t_1, Py_None); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1276, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v_ref = __pyx_t_2;
  __pyx_t_2 = 0;

  /* "cupy/cuda/memory.pyx":1277
 *         cdef _Arena arena
 *         ref = self._arenas.get(stream_ident, None)
 *         if ref is not None:             # <<<<<<<<<<<<<<
 *             if cpython.PyWeakref_GetRef(ref, &ret) == 1:
 *                 arena = <_Arena>ret
*/
  __pyx_t_3 = (__pyx_v_ref != Py_None);
  if (__pyx_t_3) {

    /* "cupy/cuda/memory.pyx":1278
 *         ref = self._arenas.get(stream_ident, None)
 *         if ref is not None:
 *             if cpython.PyWeakref_GetRef(ref, &ret) == 1:             # <<<<<<<<<<<<<<
 *                 arena = <_Arena>ret
 *                 cpython.Py_DECREF(arena)  # ret owned a reference
*/
    __pyx_t_3 = (__Pyx_PyWeakref_GetRef(__pyx_v_ref, (&__pyx_v_ret)) == 1);
    if (__pyx_t_3) {

      /* "cupy/cuda/memory.pyx":1279
 *         if ref is not None:
 *             if cpython.PyWeakref_GetRef(ref, &ret) == 1:
 *                 arena = <_Arena>ret             # <<<<<<<<<<<<<<
 *                 cpython.Py_DECREF(arena)  # ret owned a reference
 *                 return arena
*/
      __pyx_t_2 = ((PyObject *)__pyx_v_ret);
      __Pyx_INCREF(__pyx_t_2);
      __pyx_v_arena = ((struct __pyx_obj_4cupy_4cuda_6memory__Arena *)__pyx_t_2);
      __pyx_t_2 = 0;

      /* "cupy/cuda/memory.pyx":1280
 *             if cpython.PyWeakref_GetRef(ref, &ret) == 1:
 *                 arena = <_Arena>ret
 *                 cpython.Py_DECREF(arena)  # ret owned a reference             # <<<<<<<<<<<<<<
 *                 return arena
 * 
*/
      Py_DECREF(((PyObject *)__pyx_v_arena));

      /* "cupy/cuda/memory.pyx":1281
 *                 arena = <_Arena>ret
 *                 cpython.Py_DECREF(arena)  # ret owned a reference
 *                 return arena             # <<<<<<<<<<<<<<
 * 
 *         # Assume there are not many arenas being created and deleted so
*/
      __Pyx_XDECREF((PyObject *)__pyx_r);
      __Pyx_INCREF((PyObject *)__pyx_v_arena);
      __pyx_r = __pyx_v_arena;
      goto __pyx_L0;

      /* "cupy/cuda/memory.pyx":1278
 *         ref = self._arenas.get(stream_ident, None)
 *         if ref is not None:
 *             if cpython.PyWeakref_GetRef(ref, &ret) == 1:             # <<<<<<<<<<<<<<
 *                 arena = <_Arena>ret
 *                 cpython.Py_DECREF(arena)  # ret owned a reference
*/
    }

    /* "cupy/cuda/memory.pyx":1277
 *         cdef _Arena arena
 *         ref = self._arenas.get(stream_ident, None)
 *         if ref is not None:             # <<<<<<<<<<<<<<
 *             if cpython.PyWeakref_GetRef(ref, &ret) == 1:
 *                 arena = <_Arena>ret
*/
  }

  /* "cupy/cuda/memory.pyx":1285
 *         # Assume there are not many arenas being created and deleted so
 *         # just clean up dead refs here.
 *         for key in list(self._arenas):             # <<<<<<<<<<<<<<
 *             if self._arenas[key]() is None:
 *                 del self._arenas[key]
*/
  __pyx_t_2 = PySequence_List(__pyx_v_self->_arenas); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1285, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_1 = __pyx_t_2; __Pyx_INCREF(__pyx_t_1);
  __pyx_t_4 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  for (;;) {
    {
      Py_ssize_t __pyx_temp = __Pyx_PyList_GET_SIZE(__pyx_t_1);
      #if !CYTHON_ASSUME_SAFE_SIZE
      if (unlikely((__pyx_temp < 0))) __PYX_ERR(0, 1285, __pyx_L1_error)
      #endif
      if (__pyx_t_4 >= __pyx_temp) break;
    }
    __pyx_t_2 = __Pyx_PyList_GetItemRefFast(__pyx_t_1, __pyx_t_4, __Pyx_ReferenceSharing_OwnStrongReference);
    ++__pyx_t_4;
    if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1285, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_XDECREF_SET(__pyx_v_key, __pyx_t_2);
    __pyx_t_2 = 0;

    /* "cupy/cuda/memory.pyx":1286
 *         # just clean up dead refs here.
 *         for key in list(self._arenas):
 *             if self._arenas[key]() is None:             # <<<<<<<<<<<<<<
 *                 del self._arenas[key]
 * 
*/
    __pyx_t_5 = NULL;
    if (unlikely(__pyx_v_self->_arenas == Py_None)) {
      PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
      __PYX_ERR(0, 1286, __pyx_L1_error)
    }
    __pyx_t_6 = __Pyx_PyDict_GetItem(__pyx_v_self->_arenas, __pyx_v_key); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 1286, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_6);
    __pyx_t_7 = 1;
    #if CYTHON_UNPACK_METHODS
    if (unlikely(PyMethod_Check(__pyx_t_6))) {
      __pyx_t_5 = PyMethod_GET_SELF(__pyx_t_6);
      assert(__pyx_t_5);
      PyObject* __pyx__function = PyMethod_GET_FUNCTION(__pyx_t_6);
      __Pyx_INCREF(__pyx_t_5);
      __Pyx_INCREF(__pyx__function);
      __Pyx_DECREF_SET(__pyx_t_6, __pyx__function);
      __pyx_t_7 = 0;
    }
    #endif
    {
      PyObject *__pyx_callargs[2] = {__pyx_t_5, NULL};
      __pyx_t_2 = __Pyx_PyObject_FastCall((PyObject*)__pyx_t_6, __pyx_callargs+__pyx_t_7, (1-__pyx_t_7) | (__pyx_t_7*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
      __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
      __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
      if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1286, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_2);
    }
    __pyx_t_3 = (__pyx_t_2 == Py_None);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    if (__pyx_t_3) {

      /* "cupy/cuda/memory.pyx":1287
 *         for key in list(self._arenas):
 *             if self._arenas[key]() is None:
 *                 del self._arenas[key]             # <<<<<<<<<<<<<<
 * 
 *         arena = _Arena()
*/
      if (unlikely(__pyx_v_self->_arenas == Py_None)) {
        PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
        __PYX_ERR(0, 1287, __pyx_L1_error)
      }
      if (unlikely((PyDict_DelItem(__pyx_v_self->_arenas, __pyx_v_key) < 0))) __PYX_ERR(0, 1287, __pyx_L1_error)

      /* "cupy/cuda/memory.pyx":1286
 *         # just clean up dead refs here.
 *         for key in list(self._arenas):
 *             if self._arenas[key]() is None:             # <<<<<<<<<<<<<<
 *                 del self._arenas[key]
 * 
*/
    }

    /* "cupy/cuda/memory.pyx":1285
 *         # Assume there are not many arenas being created and deleted so
 *         # just clean up dead refs here.
 *         for key in list(self._arenas):             # <<<<<<<<<<<<<<
 *             if self._arenas[key]() is None:
 *                 del self._arenas[key]
*/
  }
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "cupy/cuda/memory.pyx":1289
 *                 del self._arenas[key]
 * 
 *         arena = _Arena()             # <<<<<<<<<<<<<<
 *         ref = weakref.ref(arena)
 *         self._arenas[stream_ident] = ref
*/
  __pyx_t_2 = NULL;
  __pyx_t_7 = 1;
  {
    PyObject *__pyx_callargs[2] = {__pyx_t_2, NULL};
    __pyx_t_1 = __Pyx_PyObject_FastCall((PyObject*)__pyx_mstate_global->__pyx_ptype_4cupy_4cuda_6memory__Arena, __pyx_callargs+__pyx_t_7, (1-__pyx_t_7) | (__pyx_t_7*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
    __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
    if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1289, __pyx_L1_error)
    __Pyx_GOTREF((PyObject *)__pyx_t_1);
  }
  __pyx_v_arena = ((struct __pyx_obj_4cupy_4cuda_6memory__Arena *)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "cupy/cuda/memory.pyx":1290
 * 
 *         arena = _Arena()
 *         ref = weakref.ref(arena)             # <<<<<<<<<<<<<<
 *         self._arenas[stream_ident] = ref
 *         return arena
*/
  __pyx_t_2 = NULL;
  __Pyx_GetModuleGlobalName(__pyx_t_6, __pyx_mstate_global->__pyx_n_u_weakref); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 1290, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_6);
  __pyx_t_5 = __Pyx_PyObject_GetAttrStr(__pyx_t_6, __pyx_mstate_global->__pyx_n_u_ref); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 1290, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_5);
  __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
  __pyx_t_7 = 1;
  #if CYTHON_UNPACK_METHODS
  if (unlikely(PyMethod_Check(__pyx_t_5))) {
    __pyx_t_2 = PyMethod_GET_SELF(__pyx_t_5);
    assert(__pyx_t_2);
    PyObject* __pyx__function = PyMethod_GET_FUNCTION(__pyx_t_5);
    __Pyx_INCREF(__pyx_t_2);
    __Pyx_INCREF(__pyx__function);
    __Pyx_DECREF_SET(__pyx_t_5, __pyx__function);
    __pyx_t_7 = 0;
  }
  #endif
  {
    PyObject *__pyx_callargs[2] = {__pyx_t_2, ((PyObject *)__pyx_v_arena)};
    __pyx_t_1 = __Pyx_PyObject_FastCall((PyObject*)__pyx_t_5, __pyx_callargs+__pyx_t_7, (2-__pyx_t_7) | (__pyx_t_7*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
    __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
    if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1290, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
  }
  __Pyx_DECREF_SET(__pyx_v_ref, __pyx_t_1);
  __pyx_t_1 = 0;

  /* "cupy/cuda/memory.pyx":1291
 *         arena = _Arena()
 *         ref = weakref.ref(arena)
 *         self._arenas[stream_ident] = ref             # <<<<<<<<<<<<<<
 *         return arena
 * 
*/
  if (unlikely(__pyx_v_self->_arenas == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(0, 1291, __pyx_L1_error)
  }
  __pyx_t_1 = PyLong_FromSsize_t(__pyx_v_stream_ident); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1291, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (unlikely((PyDict_SetItem(__pyx_v_self->_arenas, __pyx_t_1, __pyx_v_ref) < 0))) __PYX_ERR(0, 1291, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "cupy/cuda/memory.pyx":1292
 *         ref = weakref.ref(arena)
 *         self._arenas[stream_ident] = ref
 *         return arena             # <<<<<<<<<<<<<<
 * 
 *     def _debug_arena_get_index(self, intptr_t stream_ident):
*/
  __Pyx_XDECREF((PyObject *)__pyx_r);
  __Pyx_INCREF((PyObject *)__pyx_v_arena);
  __pyx_r = __pyx_v_arena;
  goto __pyx_L0;

  /* "cupy/cuda/memory.pyx":1268
 *         self.set_limit(**(_parse_limit_string()))
 * 
 *     cpdef _Arena _arena(self, intptr_t stream_ident):             # <<<<<<<<<<<<<<
 *         """Returns appropriate arena of a given stream, you should hold the
 *         arena mutex when getting this (to ensure nobody else changes things)
*/

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_AddTraceback("cupy.cuda.memory.SingleDeviceMemoryPool._arena", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XDECREF((PyObject *)__pyx_v_arena);
  __Pyx_XDECREF(__pyx_v_ref);
  __Pyx_XDECREF(__pyx_v_key);
  __Pyx_XGIVEREF((PyObject *)__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_pw_4cupy_4cuda_6memory_22SingleDeviceMemoryPool_3_arena(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
PyDoc_STRVAR(__pyx_doc_4cupy_4cuda_6memory_22SingleDeviceMemoryPool_2_arena, "SingleDeviceMemoryPool._arena(self, intptr_t stream_ident) -> _Arena\n\nReturns appropriate arena of a given stream, you should hold the\narena mutex when getting this (to ensure nobody else changes things)\n\nAll free chunks in the stream belong to one of the bin in the arena.");
static PyObject *__pyx_pw_4cupy_4cuda_6memory_22SingleDeviceMemoryPool_3_arena(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
) {
  intptr_t __pyx_v_stream_ident;
  #if !CYTHON_METH_FASTCALL
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  #endif
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject* values[1] = {0};
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("_arena (wrapper)", 0);
  #if !CYTHON_METH_FASTCALL
  #if CYTHON_ASSUME_SAFE_SIZE
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return NULL;
  #endif
  #endif
  __pyx_kwvalues = __Pyx_KwValues_FASTCALL(__pyx_args, __pyx_nargs);
  {
    PyObject ** const __pyx_pyargnames[] = {&__pyx_mstate_global->__pyx_n_u_stream_ident,0};
    const Py_ssize_t __pyx_kwds_len = (__pyx_kwds) ? __Pyx_NumKwargs_FASTCALL(__pyx_kwds) : 0;
    if (unlikely(__pyx_kwds_len < 0)) __PYX_ERR(0, 1268, __pyx_L3_error)
    if (__pyx_kwds_len > 0) {
      switch (__pyx_nargs) {
        case  1:
        values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(0, 1268, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      const Py_ssize_t kwd_pos_args = __pyx_nargs;
      if (__Pyx_ParseKeywords(__pyx_kwds, __pyx_kwvalues, __pyx_pyargnames, 0, values, kwd_pos_args, __pyx_kwds_len, "_arena", 0) < (0)) __PYX_ERR(0, 1268, __pyx_L3_error)
      for (Py_ssize_t i = __pyx_nargs; i < 1; i++) {
        if (unlikely(!values[i])) { __Pyx_RaiseArgtupleInvalid("_arena", 1, 1, 1, i); __PYX_ERR(0, 1268, __pyx_L3_error) }
      }
    } else if (unlikely(__pyx_nargs != 1)) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
      if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(0, 1268, __pyx_L3_error)
    }
    __pyx_v_stream_ident = PyLong_AsSsize_t(values[0]); if (unlikely((__pyx_v_stream_ident == ((intptr_t)-1)) && PyErr_Occurred())) __PYX_ERR(0, 1268, __pyx_L3_error)
  }
  goto __pyx_L6_skip;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("_arena", 1, 1, 1, __pyx_nargs); __PYX_ERR(0, 1268, __pyx_L3_error)
  __pyx_L6_skip:;
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_AddTraceback("cupy.cuda.memory.SingleDeviceMemoryPool._arena", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_4cupy_4cuda_6memory_22SingleDeviceMemoryPool_2_arena(((struct __pyx_obj_4cupy_4cuda_6memory_SingleDeviceMemoryPool *)__pyx_v_self), __pyx_v_stream_ident);

  /* function exit code */
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4cupy_4cuda_6memory_22SingleDeviceMemoryPool_2_arena(struct __pyx_obj_4cupy_4cuda_6memory_SingleDeviceMemoryPool *__pyx_v_self, intptr_t __pyx_v_stream_ident) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("_arena", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = ((PyObject *)__pyx_f_4cupy_4cuda_6memory_22SingleDeviceMemoryPool__arena(__pyx_v_self, __pyx_v_stream_ident, 1)); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1268, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("cupy.cuda.memory.SingleDeviceMemoryPool._arena", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "cupy/cuda/memory.pyx":1294
 *         return arena
 * 
 *     def _debug_arena_get_index(self, intptr_t stream_ident):             # <<<<<<<<<<<<<<
 *         # Sets returned are not copies, mutating them will break things
 *         with nogil:
*/

/* Python wrapper */
static PyObject *__pyx_pw_4cupy_4cuda_6memory_22SingleDeviceMemoryPool_5_debug_arena_get_index(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
PyDoc_STRVAR(__pyx_doc_4cupy_4cuda_6memory_22SingleDeviceMemoryPool_4_debug_arena_get_index, "SingleDeviceMemoryPool._debug_arena_get_index(self, intptr_t stream_ident)");
static PyObject *__pyx_pw_4cupy_4cuda_6memory_22SingleDeviceMemoryPool_5_debug_arena_get_index(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
) {
  intptr_t __pyx_v_stream_ident;
  #if !CYTHON_METH_FASTCALL
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  #endif
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject* values[1] = {0};
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("_debug_arena_get_index (wrapper)", 0);
  #if !CYTHON_METH_FASTCALL
  #if CYTHON_ASSUME_SAFE_SIZE
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return NULL;
  #endif
  #endif
  __pyx_kwvalues = __Pyx_KwValues_FASTCALL(__pyx_args, __pyx_nargs);
  {
    PyObject ** const __pyx_pyargnames[] = {&__pyx_mstate_global->__pyx_n_u_stream_ident,0};
    const Py_ssize_t __pyx_kwds_len = (__pyx_kwds) ? __Pyx_NumKwargs_FASTCALL(__pyx_kwds) : 0;
    if (unlikely(__pyx_kwds_len < 0)) __PYX_ERR(0, 1294, __pyx_L3_error)
    if (__pyx_kwds_len > 0) {
      switch (__pyx_nargs) {
        case  1:
        values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(0, 1294, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      const Py_ssize_t kwd_pos_args = __pyx_nargs;
      if (__Pyx_ParseKeywords(__pyx_kwds, __pyx_kwvalues, __pyx_pyargnames, 0, values, kwd_pos_args, __pyx_kwds_len, "_debug_arena_get_index", 0) < (0)) __PYX_ERR(0, 1294, __pyx_L3_error)
      for (Py_ssize_t i = __pyx_nargs; i < 1; i++) {
        if (unlikely(!values[i])) { __Pyx_RaiseArgtupleInvalid("_debug_arena_get_index", 1, 1, 1, i); __PYX_ERR(0, 1294, __pyx_L3_error) }
      }
    } else if (unlikely(__pyx_nargs != 1)) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
      if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(0, 1294, __pyx_L3_error)
    }
    __pyx_v_stream_ident = PyLong_AsSsize_t(values[0]); if (unlikely((__pyx_v_stream_ident == ((intptr_t)-1)) && PyErr_Occurred())) __PYX_ERR(0, 1294, __pyx_L3_error)
  }
  goto __pyx_L6_skip;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("_debug_arena_get_index", 1, 1, 1, __pyx_nargs); __PYX_ERR(0, 1294, __pyx_L3_error)
  __pyx_L6_skip:;
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_AddTraceback("cupy.cuda.memory.SingleDeviceMemoryPool._debug_arena_get_index", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_4cupy_4cuda_6memory_22SingleDeviceMemoryPool_4_debug_arena_get_index(((struct __pyx_obj_4cupy_4cuda_6memory_SingleDeviceMemoryPool *)__pyx_v_self), __pyx_v_stream_ident);

  /* function exit code */
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4cupy_4cuda_6memory_22SingleDeviceMemoryPool_4_debug_arena_get_index(struct __pyx_obj_4cupy_4cuda_6memory_SingleDeviceMemoryPool *__pyx_v_self, intptr_t __pyx_v_stream_ident) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  int __pyx_t_3;
  int __pyx_t_4;
  char const *__pyx_t_5;
  PyObject *__pyx_t_6 = NULL;
  PyObject *__pyx_t_7 = NULL;
  PyObject *__pyx_t_8 = NULL;
  PyObject *__pyx_t_9 = NULL;
  PyObject *__pyx_t_10 = NULL;
  PyObject *__pyx_t_11 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("_debug_arena_get_index", 0);

  /* "cupy/cuda/memory.pyx":1296
 *     def _debug_arena_get_index(self, intptr_t stream_ident):
 *         # Sets returned are not copies, mutating them will break things
 *         with nogil:             # <<<<<<<<<<<<<<
 *             self._arena_mutex.lock()
 *         try:
*/
  {
      PyThreadState * _save;
      _save = PyEval_SaveThread();
      __Pyx_FastGIL_Remember();
      /*try:*/ {

        /* "cupy/cuda/memory.pyx":1297
 *         # Sets returned are not copies, mutating them will break things
 *         with nogil:
 *             self._arena_mutex.lock()             # <<<<<<<<<<<<<<
 *         try:
 *             return self._arena(stream_ident)._index_to_python()
*/
        try {
          __pyx_v_self->_arena_mutex.lock();
        } catch(...) {
          PyGILState_STATE __pyx_gilstate_save = __Pyx_PyGILState_Ensure();
          __Pyx_CppExn2PyErr();
          __Pyx_PyGILState_Release(__pyx_gilstate_save);
          __PYX_ERR(0, 1297, __pyx_L4_error)
        }
      }

      /* "cupy/cuda/memory.pyx":1296
 *     def _debug_arena_get_index(self, intptr_t stream_ident):
 *         # Sets returned are not copies, mutating them will break things
 *         with nogil:             # <<<<<<<<<<<<<<
 *             self._arena_mutex.lock()
 *         try:
*/
      /*finally:*/ {
        /*normal exit:*/{
          __Pyx_FastGIL_Forget();
          PyEval_RestoreThread(_save);
          goto __pyx_L5;
        }
        __pyx_L4_error: {
          __Pyx_FastGIL_Forget();
          PyEval_RestoreThread(_save);
          goto __pyx_L1_error;
        }
        __pyx_L5:;
      }
  }

  /* "cupy/cuda/memory.pyx":1298
 *         with nogil:
 *             self._arena_mutex.lock()
 *         try:             # <<<<<<<<<<<<<<
 *             return self._arena(stream_ident)._index_to_python()
 *         finally:
*/
  /*try:*/ {

    /* "cupy/cuda/memory.pyx":1299
 *             self._arena_mutex.lock()
 *         try:
 *             return self._arena(stream_ident)._index_to_python()             # <<<<<<<<<<<<<<
 *         finally:
 *             self._arena_mutex.unlock()
*/
    __Pyx_XDECREF(__pyx_r);
    __pyx_t_1 = ((PyObject *)__pyx_f_4cupy_4cuda_6memory_22SingleDeviceMemoryPool__arena(__pyx_v_self, __pyx_v_stream_ident, 0)); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1299, __pyx_L7_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_t_2 = __pyx_f_4cupy_4cuda_6memory_6_Arena__index_to_python(((struct __pyx_obj_4cupy_4cuda_6memory__Arena *)__pyx_t_1)); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1299, __pyx_L7_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __pyx_r = __pyx_t_2;
    __pyx_t_2 = 0;
    goto __pyx_L6_return;
  }

  /* "cupy/cuda/memory.pyx":1301
 *             return self._arena(stream_ident)._index_to_python()
 *         finally:
 *             self._arena_mutex.unlock()             # <<<<<<<<<<<<<<
 * 
 *     cdef MemoryPointer _alloc(self, Py_ssize_t rounded_size):
*/
  /*finally:*/ {
    __pyx_L7_error:;
    /*exception exit:*/{
      __Pyx_PyThreadState_declare
      __Pyx_PyThreadState_assign
      __pyx_t_6 = 0; __pyx_t_7 = 0; __pyx_t_8 = 0; __pyx_t_9 = 0; __pyx_t_10 = 0; __pyx_t_11 = 0;
      __Pyx_XDECREF(__pyx_t_1); __pyx_t_1 = 0;
      __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
       __Pyx_ExceptionSwap(&__pyx_t_9, &__pyx_t_10, &__pyx_t_11);
      if ( unlikely(__Pyx_GetException(&__pyx_t_6, &__pyx_t_7, &__pyx_t_8) < 0)) __Pyx_ErrFetch(&__pyx_t_6, &__pyx_t_7, &__pyx_t_8);
      __Pyx_XGOTREF(__pyx_t_6);
      __Pyx_XGOTREF(__pyx_t_7);
      __Pyx_XGOTREF(__pyx_t_8);
      __Pyx_XGOTREF(__pyx_t_9);
      __Pyx_XGOTREF(__pyx_t_10);
      __Pyx_XGOTREF(__pyx_t_11);
      __pyx_t_3 = __pyx_lineno; __pyx_t_4 = __pyx_clineno; __pyx_t_5 = __pyx_filename;
      {
        __pyx_v_self->_arena_mutex.unlock();
      }
      __Pyx_XGIVEREF(__pyx_t_9);
      __Pyx_XGIVEREF(__pyx_t_10);
      __Pyx_XGIVEREF(__pyx_t_11);
      __Pyx_ExceptionReset(__pyx_t_9, __pyx_t_10, __pyx_t_11);
      __Pyx_XGIVEREF(__pyx_t_6);
      __Pyx_XGIVEREF(__pyx_t_7);
      __Pyx_XGIVEREF(__pyx_t_8);
      __Pyx_ErrRestore(__pyx_t_6, __pyx_t_7, __pyx_t_8);
      __pyx_t_6 = 0; __pyx_t_7 = 0; __pyx_t_8 = 0; __pyx_t_9 = 0; __pyx_t_10 = 0; __pyx_t_11 = 0;
      __pyx_lineno = __pyx_t_3; __pyx_clineno = __pyx_t_4; __pyx_filename = __pyx_t_5;
      goto __pyx_L1_error;
    }
    __pyx_L6_return: {
      __pyx_t_11 = __pyx_r;
      __pyx_r = 0;
      __pyx_v_self->_arena_mutex.unlock();
      __pyx_r = __pyx_t_11;
      __pyx_t_11 = 0;
      goto __pyx_L0;
    }
  }

  /* "cupy/cuda/memory.pyx":1294
 *         return arena
 * 
 *     def _debug_arena_get_index(self, intptr_t stream_ident):             # <<<<<<<<<<<<<<
 *         # Sets returned are not copies, mutating them will break things
 *         with nogil:
*/

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_AddTraceback("cupy.cuda.memory.SingleDeviceMemoryPool._debug_arena_get_index", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "cupy/cuda/memory.pyx":1303
 *             self._arena_mutex.unlock()
 * 
 *     cdef MemoryPointer _alloc(self, Py_ssize_t rounded_size):             # <<<<<<<<<<<<<<
 *         if memory_hook._has_memory_hooks():
 *             hooks = memory_hook.get_memory_hooks()
*/

static struct __pyx_obj_4cupy_4cuda_6memory_MemoryPointer *__pyx_f_4cupy_4cuda_6memory_22SingleDeviceMemoryPool__alloc(struct __pyx_obj_4cupy_4cuda_6memory_SingleDeviceMemoryPool *__pyx_v_self, Py_ssize_t __pyx_v_rounded_size) {
  PyObject *__pyx_v_hooks = NULL;
  PyObject *__pyx_v_memptr = NULL;
  int __pyx_v_device_id;
  PyObject *__pyx_v_hook = NULL;
  PyObject *__pyx_v_mem_ptr = NULL;
  struct __pyx_obj_4cupy_4cuda_6memory_MemoryPointer *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  int __pyx_t_3;
  Py_ssize_t __pyx_t_4;
  Py_ssize_t __pyx_t_5;
  PyObject *__pyx_t_6 = NULL;
  int __pyx_t_7;
  PyObject *__pyx_t_8 = NULL;
  PyObject *__pyx_t_9 = NULL;
  PyObject *__pyx_t_10 = NULL;
  size_t __pyx_t_11;
  PyObject *__pyx_t_12 = NULL;
  char const *__pyx_t_13;
  PyObject *__pyx_t_14 = NULL;
  PyObject *__pyx_t_15 = NULL;
  PyObject *__pyx_t_16 = NULL;
  PyObject *__pyx_t_17 = NULL;
  PyObject *__pyx_t_18 = NULL;
  PyObject *__pyx_t_19 = NULL;
  int __pyx_t_20;
  int __pyx_t_21;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("_alloc", 0);

  /* "cupy/cuda/memory.pyx":1304
 * 
 *     cdef MemoryPointer _alloc(self, Py_ssize_t rounded_size):
 *         if memory_hook._has_memory_hooks():             # <<<<<<<<<<<<<<
 *             hooks = memory_hook.get_memory_hooks()
 *             if hooks:
*/
  __pyx_t_1 = __pyx_f_4cupy_4cuda_11memory_hook__has_memory_hooks(0); if (unlikely(__pyx_t_1 == ((int)-1))) __PYX_ERR(0, 1304, __pyx_L1_error)
  if (__pyx_t_1) {

    /* "cupy/cuda/memory.pyx":1305
 *     cdef MemoryPointer _alloc(self, Py_ssize_t rounded_size):
 *         if memory_hook._has_memory_hooks():
 *             hooks = memory_hook.get_memory_hooks()             # <<<<<<<<<<<<<<
 *             if hooks:
 *                 memptr = None
*/
    __pyx_t_2 = __pyx_f_4cupy_4cuda_11memory_hook_get_memory_hooks(0); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1305, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_v_hooks = __pyx_t_2;
    __pyx_t_2 = 0;

    /* "cupy/cuda/memory.pyx":1306
 *         if memory_hook._has_memory_hooks():
 *             hooks = memory_hook.get_memory_hooks()
 *             if hooks:             # <<<<<<<<<<<<<<
 *                 memptr = None
 *                 device_id = self._device_id
*/
    __pyx_t_1 = __Pyx_PyObject_IsTrue(__pyx_v_hooks); if (unlikely((__pyx_t_1 < 0))) __PYX_ERR(0, 1306, __pyx_L1_error)
    if (__pyx_t_1) {

      /* "cupy/cuda/memory.pyx":1307
 *             hooks = memory_hook.get_memory_hooks()
 *             if hooks:
 *                 memptr = None             # <<<<<<<<<<<<<<
 *                 device_id = self._device_id
 *                 for hook in hooks.values():
*/
      __Pyx_INCREF(Py_None);
      __pyx_v_memptr = Py_None;

      /* "cupy/cuda/memory.pyx":1308
 *             if hooks:
 *                 memptr = None
 *                 device_id = self._device_id             # <<<<<<<<<<<<<<
 *                 for hook in hooks.values():
 *                     hook.alloc_preprocess(device_id=device_id,
*/
      __pyx_t_3 = __pyx_v_self->_device_id;
      __pyx_v_device_id = __pyx_t_3;

      /* "cupy/cuda/memory.pyx":1309
 *                 memptr = None
 *                 device_id = self._device_id
 *                 for hook in hooks.values():             # <<<<<<<<<<<<<<
 *                     hook.alloc_preprocess(device_id=device_id,
 *                                           mem_size=rounded_size)
*/
      __pyx_t_4 = 0;
      if (unlikely(__pyx_v_hooks == Py_None)) {
        PyErr_Format(PyExc_AttributeError, "'NoneType' object has no attribute '%.30s'", "values");
        __PYX_ERR(0, 1309, __pyx_L1_error)
      }
      __pyx_t_6 = __Pyx_dict_iterator(__pyx_v_hooks, 0, __pyx_mstate_global->__pyx_n_u_values, (&__pyx_t_5), (&__pyx_t_3)); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 1309, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_6);
      __Pyx_XDECREF(__pyx_t_2);
      __pyx_t_2 = __pyx_t_6;
      __pyx_t_6 = 0;
      while (1) {
        __pyx_t_7 = __Pyx_dict_iter_next(__pyx_t_2, __pyx_t_5, &__pyx_t_4, NULL, &__pyx_t_6, NULL, __pyx_t_3);
        if (unlikely(__pyx_t_7 == 0)) break;
        if (unlikely(__pyx_t_7 == -1)) __PYX_ERR(0, 1309, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_6);
        __Pyx_XDECREF_SET(__pyx_v_hook, __pyx_t_6);
        __pyx_t_6 = 0;

        /* "cupy/cuda/memory.pyx":1310
 *                 device_id = self._device_id
 *                 for hook in hooks.values():
 *                     hook.alloc_preprocess(device_id=device_id,             # <<<<<<<<<<<<<<
 *                                           mem_size=rounded_size)
 *                 try:
*/
        __pyx_t_8 = __pyx_v_hook;
        __Pyx_INCREF(__pyx_t_8);
        __pyx_t_9 = __Pyx_PyLong_From_int(__pyx_v_device_id); if (unlikely(!__pyx_t_9)) __PYX_ERR(0, 1310, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_9);

        /* "cupy/cuda/memory.pyx":1311
 *                 for hook in hooks.values():
 *                     hook.alloc_preprocess(device_id=device_id,
 *                                           mem_size=rounded_size)             # <<<<<<<<<<<<<<
 *                 try:
 *                     memptr = self._allocator(rounded_size)
*/
        __pyx_t_10 = PyLong_FromSsize_t(__pyx_v_rounded_size); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 1311, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_10);
        __pyx_t_11 = 0;
        {
          PyObject *__pyx_callargs[2 + ((CYTHON_VECTORCALL) ? 2 : 0)] = {__pyx_t_8, NULL};
          __pyx_t_12 = __Pyx_MakeVectorcallBuilderKwds(2); if (unlikely(!__pyx_t_12)) __PYX_ERR(0, 1310, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_12);
          if (__Pyx_VectorcallBuilder_AddArg(__pyx_mstate_global->__pyx_n_u_device_id, __pyx_t_9, __pyx_t_12, __pyx_callargs+1, 0) < (0)) __PYX_ERR(0, 1310, __pyx_L1_error)
          if (__Pyx_VectorcallBuilder_AddArg(__pyx_mstate_global->__pyx_n_u_mem_size, __pyx_t_10, __pyx_t_12, __pyx_callargs+1, 1) < (0)) __PYX_ERR(0, 1310, __pyx_L1_error)
          __pyx_t_6 = __Pyx_Object_VectorcallMethod_CallFromBuilder((PyObject*)__pyx_mstate_global->__pyx_n_u_alloc_preprocess, __pyx_callargs+__pyx_t_11, (1-__pyx_t_11) | (1*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET), __pyx_t_12);
          __Pyx_XDECREF(__pyx_t_8); __pyx_t_8 = 0;
          __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
          __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
          __Pyx_DECREF(__pyx_t_12); __pyx_t_12 = 0;
          if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 1310, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_6);
        }
        __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
      }
      __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

      /* "cupy/cuda/memory.pyx":1312
 *                     hook.alloc_preprocess(device_id=device_id,
 *                                           mem_size=rounded_size)
 *                 try:             # <<<<<<<<<<<<<<
 *                     memptr = self._allocator(rounded_size)
 *                 finally:
*/
      /*try:*/ {

        /* "cupy/cuda/memory.pyx":1313
 *                                           mem_size=rounded_size)
 *                 try:
 *                     memptr = self._allocator(rounded_size)             # <<<<<<<<<<<<<<
 *                 finally:
 *                     for hook in hooks.values():
*/
        __pyx_t_6 = NULL;
        __Pyx_INCREF(__pyx_v_self->_allocator);
        __pyx_t_12 = __pyx_v_self->_allocator; 
        __pyx_t_10 = PyLong_FromSsize_t(__pyx_v_rounded_size); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 1313, __pyx_L8_error)
        __Pyx_GOTREF(__pyx_t_10);
        __pyx_t_11 = 1;
        #if CYTHON_UNPACK_METHODS
        if (likely(PyMethod_Check(__pyx_t_12))) {
          __pyx_t_6 = PyMethod_GET_SELF(__pyx_t_12);
          assert(__pyx_t_6);
          PyObject* __pyx__function = PyMethod_GET_FUNCTION(__pyx_t_12);
          __Pyx_INCREF(__pyx_t_6);
          __Pyx_INCREF(__pyx__function);
          __Pyx_DECREF_SET(__pyx_t_12, __pyx__function);
          __pyx_t_11 = 0;
        }
        #endif
        {
          PyObject *__pyx_callargs[2] = {__pyx_t_6, __pyx_t_10};
          __pyx_t_2 = __Pyx_PyObject_FastCall((PyObject*)__pyx_t_12, __pyx_callargs+__pyx_t_11, (2-__pyx_t_11) | (__pyx_t_11*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
          __Pyx_XDECREF(__pyx_t_6); __pyx_t_6 = 0;
          __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
          __Pyx_DECREF(__pyx_t_12); __pyx_t_12 = 0;
          if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1313, __pyx_L8_error)
          __Pyx_GOTREF(__pyx_t_2);
        }
        __Pyx_DECREF_SET(__pyx_v_memptr, __pyx_t_2);
        __pyx_t_2 = 0;
      }

      /* "cupy/cuda/memory.pyx":1315
 *                     memptr = self._allocator(rounded_size)
 *                 finally:
 *                     for hook in hooks.values():             # <<<<<<<<<<<<<<
 *                         mem_ptr = memptr.ptr if memptr is not None else 0
 *                         hook.alloc_postprocess(device_id=device_id,
*/
      /*finally:*/ {
        /*normal exit:*/{
          __pyx_t_5 = 0;
          if (unlikely(__pyx_v_hooks == Py_None)) {
            PyErr_Format(PyExc_AttributeError, "'NoneType' object has no attribute '%.30s'", "values");
            __PYX_ERR(0, 1315, __pyx_L1_error)
          }
          __pyx_t_12 = __Pyx_dict_iterator(__pyx_v_hooks, 0, __pyx_mstate_global->__pyx_n_u_values, (&__pyx_t_4), (&__pyx_t_3)); if (unlikely(!__pyx_t_12)) __PYX_ERR(0, 1315, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_12);
          __Pyx_XDECREF(__pyx_t_2);
          __pyx_t_2 = __pyx_t_12;
          __pyx_t_12 = 0;
          while (1) {
            __pyx_t_7 = __Pyx_dict_iter_next(__pyx_t_2, __pyx_t_4, &__pyx_t_5, NULL, &__pyx_t_12, NULL, __pyx_t_3);
            if (unlikely(__pyx_t_7 == 0)) break;
            if (unlikely(__pyx_t_7 == -1)) __PYX_ERR(0, 1315, __pyx_L1_error)
            __Pyx_GOTREF(__pyx_t_12);
            __Pyx_XDECREF_SET(__pyx_v_hook, __pyx_t_12);
            __pyx_t_12 = 0;

            /* "cupy/cuda/memory.pyx":1316
 *                 finally:
 *                     for hook in hooks.values():
 *                         mem_ptr = memptr.ptr if memptr is not None else 0             # <<<<<<<<<<<<<<
 *                         hook.alloc_postprocess(device_id=device_id,
 *                                                mem_size=rounded_size,
*/
            __pyx_t_1 = (__pyx_v_memptr != Py_None);
            if (__pyx_t_1) {
              __pyx_t_10 = __Pyx_PyObject_GetAttrStr(__pyx_v_memptr, __pyx_mstate_global->__pyx_n_u_ptr); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 1316, __pyx_L1_error)
              __Pyx_GOTREF(__pyx_t_10);
              __pyx_t_12 = __pyx_t_10;
              __pyx_t_10 = 0;
            } else {
              __Pyx_INCREF(__pyx_mstate_global->__pyx_int_0);
              __pyx_t_12 = __pyx_mstate_global->__pyx_int_0;
            }
            __Pyx_XDECREF_SET(__pyx_v_mem_ptr, __pyx_t_12);
            __pyx_t_12 = 0;

            /* "cupy/cuda/memory.pyx":1317
 *                     for hook in hooks.values():
 *                         mem_ptr = memptr.ptr if memptr is not None else 0
 *                         hook.alloc_postprocess(device_id=device_id,             # <<<<<<<<<<<<<<
 *                                                mem_size=rounded_size,
 *                                                mem_ptr=mem_ptr)
*/
            __pyx_t_10 = __pyx_v_hook;
            __Pyx_INCREF(__pyx_t_10);
            __pyx_t_6 = __Pyx_PyLong_From_int(__pyx_v_device_id); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 1317, __pyx_L1_error)
            __Pyx_GOTREF(__pyx_t_6);

            /* "cupy/cuda/memory.pyx":1318
 *                         mem_ptr = memptr.ptr if memptr is not None else 0
 *                         hook.alloc_postprocess(device_id=device_id,
 *                                                mem_size=rounded_size,             # <<<<<<<<<<<<<<
 *                                                mem_ptr=mem_ptr)
 *                 return memptr
*/
            __pyx_t_9 = PyLong_FromSsize_t(__pyx_v_rounded_size); if (unlikely(!__pyx_t_9)) __PYX_ERR(0, 1318, __pyx_L1_error)
            __Pyx_GOTREF(__pyx_t_9);

            /* "cupy/cuda/memory.pyx":1319
 *                         hook.alloc_postprocess(device_id=device_id,
 *                                                mem_size=rounded_size,
 *                                                mem_ptr=mem_ptr)             # <<<<<<<<<<<<<<
 *                 return memptr
 *         return self._allocator(rounded_size)
*/
            __pyx_t_11 = 0;
            {
              PyObject *__pyx_callargs[2 + ((CYTHON_VECTORCALL) ? 3 : 0)] = {__pyx_t_10, NULL};
              __pyx_t_8 = __Pyx_MakeVectorcallBuilderKwds(3); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 1317, __pyx_L1_error)
              __Pyx_GOTREF(__pyx_t_8);
              if (__Pyx_VectorcallBuilder_AddArg(__pyx_mstate_global->__pyx_n_u_device_id, __pyx_t_6, __pyx_t_8, __pyx_callargs+1, 0) < (0)) __PYX_ERR(0, 1317, __pyx_L1_error)
              if (__Pyx_VectorcallBuilder_AddArg(__pyx_mstate_global->__pyx_n_u_mem_size, __pyx_t_9, __pyx_t_8, __pyx_callargs+1, 1) < (0)) __PYX_ERR(0, 1317, __pyx_L1_error)
              if (__Pyx_VectorcallBuilder_AddArg(__pyx_mstate_global->__pyx_n_u_mem_ptr, __pyx_v_mem_ptr, __pyx_t_8, __pyx_callargs+1, 2) < (0)) __PYX_ERR(0, 1317, __pyx_L1_error)
              __pyx_t_12 = __Pyx_Object_VectorcallMethod_CallFromBuilder((PyObject*)__pyx_mstate_global->__pyx_n_u_alloc_postprocess, __pyx_callargs+__pyx_t_11, (1-__pyx_t_11) | (1*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET), __pyx_t_8);
              __Pyx_XDECREF(__pyx_t_10); __pyx_t_10 = 0;
              __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
              __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
              __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
              if (unlikely(!__pyx_t_12)) __PYX_ERR(0, 1317, __pyx_L1_error)
              __Pyx_GOTREF(__pyx_t_12);
            }
            __Pyx_DECREF(__pyx_t_12); __pyx_t_12 = 0;
          }
          __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
          goto __pyx_L9;
        }
        __pyx_L8_error:;
        /*exception exit:*/{
          __Pyx_PyThreadState_declare
          __Pyx_PyThreadState_assign
          __pyx_t_14 = 0; __pyx_t_15 = 0; __pyx_t_16 = 0; __pyx_t_17 = 0; __pyx_t_18 = 0; __pyx_t_19 = 0;
          __Pyx_XDECREF(__pyx_t_10); __pyx_t_10 = 0;
          __Pyx_XDECREF(__pyx_t_12); __pyx_t_12 = 0;
          __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
          __Pyx_XDECREF(__pyx_t_6); __pyx_t_6 = 0;
          __Pyx_XDECREF(__pyx_t_8); __pyx_t_8 = 0;
          __Pyx_XDECREF(__pyx_t_9); __pyx_t_9 = 0;
           __Pyx_ExceptionSwap(&__pyx_t_17, &__pyx_t_18, &__pyx_t_19);
          if ( unlikely(__Pyx_GetException(&__pyx_t_14, &__pyx_t_15, &__pyx_t_16) < 0)) __Pyx_ErrFetch(&__pyx_t_14, &__pyx_t_15, &__pyx_t_16);
          __Pyx_XGOTREF(__pyx_t_14);
          __Pyx_XGOTREF(__pyx_t_15);
          __Pyx_XGOTREF(__pyx_t_16);
          __Pyx_XGOTREF(__pyx_t_17);
          __Pyx_XGOTREF(__pyx_t_18);
          __Pyx_XGOTREF(__pyx_t_19);
          __pyx_t_3 = __pyx_lineno; __pyx_t_7 = __pyx_clineno; __pyx_t_13 = __pyx_filename;
          {

            /* "cupy/cuda/memory.pyx":1315
 *                     memptr = self._allocator(rounded_size)
 *                 finally:
 *                     for hook in hooks.values():             # <<<<<<<<<<<<<<
 *                         mem_ptr = memptr.ptr if memptr is not None else 0
 *                         hook.alloc_postprocess(device_id=device_id,
*/
            __pyx_t_4 = 0;
            if (unlikely(__pyx_v_hooks == Py_None)) {
              PyErr_Format(PyExc_AttributeError, "'NoneType' object has no attribute '%.30s'", "values");
              __PYX_ERR(0, 1315, __pyx_L13_error)
            }
            __pyx_t_12 = __Pyx_dict_iterator(__pyx_v_hooks, 0, __pyx_mstate_global->__pyx_n_u_values, (&__pyx_t_5), (&__pyx_t_20)); if (unlikely(!__pyx_t_12)) __PYX_ERR(0, 1315, __pyx_L13_error)
            __Pyx_GOTREF(__pyx_t_12);
            __Pyx_XDECREF(__pyx_t_2);
            __pyx_t_2 = __pyx_t_12;
            __pyx_t_12 = 0;
            while (1) {
              __pyx_t_21 = __Pyx_dict_iter_next(__pyx_t_2, __pyx_t_5, &__pyx_t_4, NULL, &__pyx_t_12, NULL, __pyx_t_20);
              if (unlikely(__pyx_t_21 == 0)) break;
              if (unlikely(__pyx_t_21 == -1)) __PYX_ERR(0, 1315, __pyx_L13_error)
              __Pyx_GOTREF(__pyx_t_12);
              __Pyx_XDECREF_SET(__pyx_v_hook, __pyx_t_12);
              __pyx_t_12 = 0;

              /* "cupy/cuda/memory.pyx":1316
 *                 finally:
 *                     for hook in hooks.values():
 *                         mem_ptr = memptr.ptr if memptr is not None else 0             # <<<<<<<<<<<<<<
 *                         hook.alloc_postprocess(device_id=device_id,
 *                                                mem_size=rounded_size,
*/
              __pyx_t_1 = (__pyx_v_memptr != Py_None);
              if (__pyx_t_1) {
                __pyx_t_8 = __Pyx_PyObject_GetAttrStr(__pyx_v_memptr, __pyx_mstate_global->__pyx_n_u_ptr); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 1316, __pyx_L13_error)
                __Pyx_GOTREF(__pyx_t_8);
                __pyx_t_12 = __pyx_t_8;
                __pyx_t_8 = 0;
              } else {
                __Pyx_INCREF(__pyx_mstate_global->__pyx_int_0);
                __pyx_t_12 = __pyx_mstate_global->__pyx_int_0;
              }
              __Pyx_XDECREF_SET(__pyx_v_mem_ptr, __pyx_t_12);
              __pyx_t_12 = 0;

              /* "cupy/cuda/memory.pyx":1317
 *                     for hook in hooks.values():
 *                         mem_ptr = memptr.ptr if memptr is not None else 0
 *                         hook.alloc_postprocess(device_id=device_id,             # <<<<<<<<<<<<<<
 *                                                mem_size=rounded_size,
 *                                                mem_ptr=mem_ptr)
*/
              __pyx_t_8 = __pyx_v_hook;
              __Pyx_INCREF(__pyx_t_8);
              __pyx_t_9 = __Pyx_PyLong_From_int(__pyx_v_device_id); if (unlikely(!__pyx_t_9)) __PYX_ERR(0, 1317, __pyx_L13_error)
              __Pyx_GOTREF(__pyx_t_9);

              /* "cupy/cuda/memory.pyx":1318
 *                         mem_ptr = memptr.ptr if memptr is not None else 0
 *                         hook.alloc_postprocess(device_id=device_id,
 *                                                mem_size=rounded_size,             # <<<<<<<<<<<<<<
 *                                                mem_ptr=mem_ptr)
 *                 return memptr
*/
              __pyx_t_6 = PyLong_FromSsize_t(__pyx_v_rounded_size); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 1318, __pyx_L13_error)
              __Pyx_GOTREF(__pyx_t_6);

              /* "cupy/cuda/memory.pyx":1319
 *                         hook.alloc_postprocess(device_id=device_id,
 *                                                mem_size=rounded_size,
 *                                                mem_ptr=mem_ptr)             # <<<<<<<<<<<<<<
 *                 return memptr
 *         return self._allocator(rounded_size)
*/
              __pyx_t_11 = 0;
              {
                PyObject *__pyx_callargs[2 + ((CYTHON_VECTORCALL) ? 3 : 0)] = {__pyx_t_8, NULL};
                __pyx_t_10 = __Pyx_MakeVectorcallBuilderKwds(3); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 1317, __pyx_L13_error)
                __Pyx_GOTREF(__pyx_t_10);
                if (__Pyx_VectorcallBuilder_AddArg(__pyx_mstate_global->__pyx_n_u_device_id, __pyx_t_9, __pyx_t_10, __pyx_callargs+1, 0) < (0)) __PYX_ERR(0, 1317, __pyx_L13_error)
                if (__Pyx_VectorcallBuilder_AddArg(__pyx_mstate_global->__pyx_n_u_mem_size, __pyx_t_6, __pyx_t_10, __pyx_callargs+1, 1) < (0)) __PYX_ERR(0, 1317, __pyx_L13_error)
                if (__Pyx_VectorcallBuilder_AddArg(__pyx_mstate_global->__pyx_n_u_mem_ptr, __pyx_v_mem_ptr, __pyx_t_10, __pyx_callargs+1, 2) < (0)) __PYX_ERR(0, 1317, __pyx_L13_error)
                __pyx_t_12 = __Pyx_Object_VectorcallMethod_CallFromBuilder((PyObject*)__pyx_mstate_global->__pyx_n_u_alloc_postprocess, __pyx_callargs+__pyx_t_11, (1-__pyx_t_11) | (1*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET), __pyx_t_10);
                __Pyx_XDECREF(__pyx_t_8); __pyx_t_8 = 0;
                __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
                __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
                __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
                if (unlikely(!__pyx_t_12)) __PYX_ERR(0, 1317, __pyx_L13_error)
                __Pyx_GOTREF(__pyx_t_12);
              }
              __Pyx_DECREF(__pyx_t_12); __pyx_t_12 = 0;
            }
            __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
          }
          __Pyx_XGIVEREF(__pyx_t_17);
          __Pyx_XGIVEREF(__pyx_t_18);
          __Pyx_XGIVEREF(__pyx_t_19);
          __Pyx_ExceptionReset(__pyx_t_17, __pyx_t_18, __pyx_t_19);
          __Pyx_XGIVEREF(__pyx_t_14);
          __Pyx_XGIVEREF(__pyx_t_15);
          __Pyx_XGIVEREF(__pyx_t_16);
          __Pyx_ErrRestore(__pyx_t_14, __pyx_t_15, __pyx_t_16);
          __pyx_t_14 = 0; __pyx_t_15 = 0; __pyx_t_16 = 0; __pyx_t_17 = 0; __pyx_t_18 = 0; __pyx_t_19 = 0;
          __pyx_lineno = __pyx_t_3; __pyx_clineno = __pyx_t_7; __pyx_filename = __pyx_t_13;
          goto __pyx_L1_error;
          __pyx_L13_error:;
          __Pyx_XGIVEREF(__pyx_t_17);
          __Pyx_XGIVEREF(__pyx_t_18);
          __Pyx_XGIVEREF(__pyx_t_19);
          __Pyx_ExceptionReset(__pyx_t_17, __pyx_t_18, __pyx_t_19);
          __Pyx_XDECREF(__pyx_t_14); __pyx_t_14 = 0;
          __Pyx_XDECREF(__pyx_t_15); __pyx_t_15 = 0;
          __Pyx_XDECREF(__pyx_t_16); __pyx_t_16 = 0;
          __pyx_t_17 = 0; __pyx_t_18 = 0; __pyx_t_19 = 0;
          goto __pyx_L1_error;
        }
        __pyx_L9:;
      }

      /* "cupy/cuda/memory.pyx":1320
 *                                                mem_size=rounded_size,
 *                                                mem_ptr=mem_ptr)
 *                 return memptr             # <<<<<<<<<<<<<<
 *         return self._allocator(rounded_size)
 * 
*/
      __Pyx_XDECREF((PyObject *)__pyx_r);
      if (!(likely(((__pyx_v_memptr) == Py_None) || likely(__Pyx_TypeTest(__pyx_v_memptr, __pyx_mstate_global->__pyx_ptype_4cupy_4cuda_6memory_MemoryPointer))))) __PYX_ERR(0, 1320, __pyx_L1_error)
      __Pyx_INCREF(__pyx_v_memptr);
      __pyx_r = ((struct __pyx_obj_4cupy_4cuda_6memory_MemoryPointer *)__pyx_v_memptr);
      goto __pyx_L0;

      /* "cupy/cuda/memory.pyx":1306
 *         if memory_hook._has_memory_hooks():
 *             hooks = memory_hook.get_memory_hooks()
 *             if hooks:             # <<<<<<<<<<<<<<
 *                 memptr = None
 *                 device_id = self._device_id
*/
    }

    /* "cupy/cuda/memory.pyx":1304
 * 
 *     cdef MemoryPointer _alloc(self, Py_ssize_t rounded_size):
 *         if memory_hook._has_memory_hooks():             # <<<<<<<<<<<<<<
 *             hooks = memory_hook.get_memory_hooks()
 *             if hooks:
*/
  }

  /* "cupy/cuda/memory.pyx":1321
 *                                                mem_ptr=mem_ptr)
 *                 return memptr
 *         return self._allocator(rounded_size)             # <<<<<<<<<<<<<<
 * 
 *     cpdef MemoryPointer malloc(self, size_t size):
*/
  __Pyx_XDECREF((PyObject *)__pyx_r);
  __pyx_t_12 = NULL;
  __Pyx_INCREF(__pyx_v_self->_allocator);
  __pyx_t_10 = __pyx_v_self->_allocator; 
  __pyx_t_6 = PyLong_FromSsize_t(__pyx_v_rounded_size); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 1321, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_6);
  __pyx_t_11 = 1;
  #if CYTHON_UNPACK_METHODS
  if (likely(PyMethod_Check(__pyx_t_10))) {
    __pyx_t_12 = PyMethod_GET_SELF(__pyx_t_10);
    assert(__pyx_t_12);
    PyObject* __pyx__function = PyMethod_GET_FUNCTION(__pyx_t_10);
    __Pyx_INCREF(__pyx_t_12);
    __Pyx_INCREF(__pyx__function);
    __Pyx_DECREF_SET(__pyx_t_10, __pyx__function);
    __pyx_t_11 = 0;
  }
  #endif
  {
    PyObject *__pyx_callargs[2] = {__pyx_t_12, __pyx_t_6};
    __pyx_t_2 = __Pyx_PyObject_FastCall((PyObject*)__pyx_t_10, __pyx_callargs+__pyx_t_11, (2-__pyx_t_11) | (__pyx_t_11*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
    __Pyx_XDECREF(__pyx_t_12); __pyx_t_12 = 0;
    __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
    __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
    if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1321, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
  }
  if (!(likely(((__pyx_t_2) == Py_None) || likely(__Pyx_TypeTest(__pyx_t_2, __pyx_mstate_global->__pyx_ptype_4cupy_4cuda_6memory_MemoryPointer))))) __PYX_ERR(0, 1321, __pyx_L1_error)
  __pyx_r = ((struct __pyx_obj_4cupy_4cuda_6memory_MemoryPointer *)__pyx_t_2);
  __pyx_t_2 = 0;
  goto __pyx_L0;

  /* "cupy/cuda/memory.pyx":1303
 *             self._arena_mutex.unlock()
 * 
 *     cdef MemoryPointer _alloc(self, Py_ssize_t rounded_size):             # <<<<<<<<<<<<<<
 *         if memory_hook._has_memory_hooks():
 *             hooks = memory_hook.get_memory_hooks()
*/

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_XDECREF(__pyx_t_8);
  __Pyx_XDECREF(__pyx_t_9);
  __Pyx_XDECREF(__pyx_t_10);
  __Pyx_XDECREF(__pyx_t_12);
  __Pyx_AddTraceback("cupy.cuda.memory.SingleDeviceMemoryPool._alloc", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_hooks);
  __Pyx_XDECREF(__pyx_v_memptr);
  __Pyx_XDECREF(__pyx_v_hook);
  __Pyx_XDECREF(__pyx_v_mem_ptr);
  __Pyx_XGIVEREF((PyObject *)__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "cupy/cuda/memory.pyx":1323
 *         return self._allocator(rounded_size)
 * 
 *     cpdef MemoryPointer malloc(self, size_t size):             # <<<<<<<<<<<<<<
 *         rounded_size = _round_size(size)
 *         if memory_hook._has_memory_hooks():
*/

static PyObject *__pyx_pw_4cupy_4cuda_6memory_22SingleDeviceMemoryPool_7malloc(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
static struct __pyx_obj_4cupy_4cuda_6memory_MemoryPointer *__pyx_f_4cupy_4cuda_6memory_22SingleDeviceMemoryPool_malloc(struct __pyx_obj_4cupy_4cuda_6memory_SingleDeviceMemoryPool *__pyx_v_self, size_t __pyx_v_size, CYTHON_UNUSED int __pyx_skip_dispatch) {
  size_t __pyx_v_rounded_size;
  PyObject *__pyx_v_hooks = NULL;
  struct __pyx_obj_4cupy_4cuda_6memory_MemoryPointer *__pyx_v_memptr = NULL;
  int __pyx_v_device_id;
  PyObject *__pyx_v_hook = NULL;
  PyObject *__pyx_v_mem_ptr = NULL;
  PyObject *__pyx_v_pmem_id = NULL;
  struct __pyx_obj_4cupy_4cuda_6memory_MemoryPointer *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  int __pyx_t_3;
  Py_ssize_t __pyx_t_4;
  Py_ssize_t __pyx_t_5;
  PyObject *__pyx_t_6 = NULL;
  int __pyx_t_7;
  PyObject *__pyx_t_8 = NULL;
  PyObject *__pyx_t_9 = NULL;
  PyObject *__pyx_t_10 = NULL;
  PyObject *__pyx_t_11 = NULL;
  size_t __pyx_t_12;
  PyObject *__pyx_t_13 = NULL;
  char const *__pyx_t_14;
  PyObject *__pyx_t_15 = NULL;
  PyObject *__pyx_t_16 = NULL;
  PyObject *__pyx_t_17 = NULL;
  PyObject *__pyx_t_18 = NULL;
  PyObject *__pyx_t_19 = NULL;
  PyObject *__pyx_t_20 = NULL;
  int __pyx_t_21;
  int __pyx_t_22;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("malloc", 0);

  /* "cupy/cuda/memory.pyx":1324
 * 
 *     cpdef MemoryPointer malloc(self, size_t size):
 *         rounded_size = _round_size(size)             # <<<<<<<<<<<<<<
 *         if memory_hook._has_memory_hooks():
 *             hooks = memory_hook.get_memory_hooks()
*/
  __pyx_v_rounded_size = __pyx_f_4cupy_4cuda_6memory__round_size(__pyx_v_size, 0);

  /* "cupy/cuda/memory.pyx":1325
 *     cpdef MemoryPointer malloc(self, size_t size):
 *         rounded_size = _round_size(size)
 *         if memory_hook._has_memory_hooks():             # <<<<<<<<<<<<<<
 *             hooks = memory_hook.get_memory_hooks()
 *             if hooks:
*/
  __pyx_t_1 = __pyx_f_4cupy_4cuda_11memory_hook__has_memory_hooks(0); if (unlikely(__pyx_t_1 == ((int)-1))) __PYX_ERR(0, 1325, __pyx_L1_error)
  if (__pyx_t_1) {

    /* "cupy/cuda/memory.pyx":1326
 *         rounded_size = _round_size(size)
 *         if memory_hook._has_memory_hooks():
 *             hooks = memory_hook.get_memory_hooks()             # <<<<<<<<<<<<<<
 *             if hooks:
 *                 memptr = None
*/
    __pyx_t_2 = __pyx_f_4cupy_4cuda_11memory_hook_get_memory_hooks(0); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1326, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_v_hooks = __pyx_t_2;
    __pyx_t_2 = 0;

    /* "cupy/cuda/memory.pyx":1327
 *         if memory_hook._has_memory_hooks():
 *             hooks = memory_hook.get_memory_hooks()
 *             if hooks:             # <<<<<<<<<<<<<<
 *                 memptr = None
 *                 device_id = self._device_id
*/
    __pyx_t_1 = __Pyx_PyObject_IsTrue(__pyx_v_hooks); if (unlikely((__pyx_t_1 < 0))) __PYX_ERR(0, 1327, __pyx_L1_error)
    if (__pyx_t_1) {

      /* "cupy/cuda/memory.pyx":1328
 *             hooks = memory_hook.get_memory_hooks()
 *             if hooks:
 *                 memptr = None             # <<<<<<<<<<<<<<
 *                 device_id = self._device_id
 *                 for hook in hooks.values():
*/
      __Pyx_INCREF(Py_None);
      __pyx_v_memptr = ((struct __pyx_obj_4cupy_4cuda_6memory_MemoryPointer *)Py_None);

      /* "cupy/cuda/memory.pyx":1329
 *             if hooks:
 *                 memptr = None
 *                 device_id = self._device_id             # <<<<<<<<<<<<<<
 *                 for hook in hooks.values():
 *                     hook.malloc_preprocess(device_id=device_id,
*/
      __pyx_t_3 = __pyx_v_self->_device_id;
      __pyx_v_device_id = __pyx_t_3;

      /* "cupy/cuda/memory.pyx":1330
 *                 memptr = None
 *                 device_id = self._device_id
 *                 for hook in hooks.values():             # <<<<<<<<<<<<<<
 *                     hook.malloc_preprocess(device_id=device_id,
 *                                            size=size,
*/
      __pyx_t_4 = 0;
      if (unlikely(__pyx_v_hooks == Py_None)) {
        PyErr_Format(PyExc_AttributeError, "'NoneType' object has no attribute '%.30s'", "values");
        __PYX_ERR(0, 1330, __pyx_L1_error)
      }
      __pyx_t_6 = __Pyx_dict_iterator(__pyx_v_hooks, 0, __pyx_mstate_global->__pyx_n_u_values, (&__pyx_t_5), (&__pyx_t_3)); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 1330, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_6);
      __Pyx_XDECREF(__pyx_t_2);
      __pyx_t_2 = __pyx_t_6;
      __pyx_t_6 = 0;
      while (1) {
        __pyx_t_7 = __Pyx_dict_iter_next(__pyx_t_2, __pyx_t_5, &__pyx_t_4, NULL, &__pyx_t_6, NULL, __pyx_t_3);
        if (unlikely(__pyx_t_7 == 0)) break;
        if (unlikely(__pyx_t_7 == -1)) __PYX_ERR(0, 1330, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_6);
        __Pyx_XDECREF_SET(__pyx_v_hook, __pyx_t_6);
        __pyx_t_6 = 0;

        /* "cupy/cuda/memory.pyx":1331
 *                 device_id = self._device_id
 *                 for hook in hooks.values():
 *                     hook.malloc_preprocess(device_id=device_id,             # <<<<<<<<<<<<<<
 *                                            size=size,
 *                                            mem_size=rounded_size)
*/
        __pyx_t_8 = __pyx_v_hook;
        __Pyx_INCREF(__pyx_t_8);
        __pyx_t_9 = __Pyx_PyLong_From_int(__pyx_v_device_id); if (unlikely(!__pyx_t_9)) __PYX_ERR(0, 1331, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_9);

        /* "cupy/cuda/memory.pyx":1332
 *                 for hook in hooks.values():
 *                     hook.malloc_preprocess(device_id=device_id,
 *                                            size=size,             # <<<<<<<<<<<<<<
 *                                            mem_size=rounded_size)
 *                 try:
*/
        __pyx_t_10 = __Pyx_PyLong_FromSize_t(__pyx_v_size); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 1332, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_10);

        /* "cupy/cuda/memory.pyx":1333
 *                     hook.malloc_preprocess(device_id=device_id,
 *                                            size=size,
 *                                            mem_size=rounded_size)             # <<<<<<<<<<<<<<
 *                 try:
 *                     memptr = self._malloc(rounded_size)
*/
        __pyx_t_11 = __Pyx_PyLong_FromSize_t(__pyx_v_rounded_size); if (unlikely(!__pyx_t_11)) __PYX_ERR(0, 1333, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_11);
        __pyx_t_12 = 0;
        {
          PyObject *__pyx_callargs[2 + ((CYTHON_VECTORCALL) ? 3 : 0)] = {__pyx_t_8, NULL};
          __pyx_t_13 = __Pyx_MakeVectorcallBuilderKwds(3); if (unlikely(!__pyx_t_13)) __PYX_ERR(0, 1331, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_13);
          if (__Pyx_VectorcallBuilder_AddArg(__pyx_mstate_global->__pyx_n_u_device_id, __pyx_t_9, __pyx_t_13, __pyx_callargs+1, 0) < (0)) __PYX_ERR(0, 1331, __pyx_L1_error)
          if (__Pyx_VectorcallBuilder_AddArg(__pyx_mstate_global->__pyx_n_u_size, __pyx_t_10, __pyx_t_13, __pyx_callargs+1, 1) < (0)) __PYX_ERR(0, 1331, __pyx_L1_error)
          if (__Pyx_VectorcallBuilder_AddArg(__pyx_mstate_global->__pyx_n_u_mem_size, __pyx_t_11, __pyx_t_13, __pyx_callargs+1, 2) < (0)) __PYX_ERR(0, 1331, __pyx_L1_error)
          __pyx_t_6 = __Pyx_Object_VectorcallMethod_CallFromBuilder((PyObject*)__pyx_mstate_global->__pyx_n_u_malloc_preprocess, __pyx_callargs+__pyx_t_12, (1-__pyx_t_12) | (1*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET), __pyx_t_13);
          __Pyx_XDECREF(__pyx_t_8); __pyx_t_8 = 0;
          __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
          __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
          __Pyx_DECREF(__pyx_t_11); __pyx_t_11 = 0;
          __Pyx_DECREF(__pyx_t_13); __pyx_t_13 = 0;
          if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 1331, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_6);
        }
        __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
      }
      __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

      /* "cupy/cuda/memory.pyx":1334
 *                                            size=size,
 *                                            mem_size=rounded_size)
 *                 try:             # <<<<<<<<<<<<<<
 *                     memptr = self._malloc(rounded_size)
 *                 finally:
*/
      /*try:*/ {

        /* "cupy/cuda/memory.pyx":1335
 *                                            mem_size=rounded_size)
 *                 try:
 *                     memptr = self._malloc(rounded_size)             # <<<<<<<<<<<<<<
 *                 finally:
 *                     if memptr is None:
*/
        __pyx_t_2 = ((PyObject *)__pyx_f_4cupy_4cuda_6memory_22SingleDeviceMemoryPool__malloc(__pyx_v_self, __pyx_v_rounded_size)); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1335, __pyx_L8_error)
        __Pyx_GOTREF(__pyx_t_2);
        __Pyx_DECREF_SET(__pyx_v_memptr, ((struct __pyx_obj_4cupy_4cuda_6memory_MemoryPointer *)__pyx_t_2));
        __pyx_t_2 = 0;
      }

      /* "cupy/cuda/memory.pyx":1337
 *                     memptr = self._malloc(rounded_size)
 *                 finally:
 *                     if memptr is None:             # <<<<<<<<<<<<<<
 *                         mem_ptr = 0
 *                         pmem_id = 0
*/
      /*finally:*/ {
        /*normal exit:*/{
          __pyx_t_1 = (((PyObject *)__pyx_v_memptr) == Py_None);
          if (__pyx_t_1) {

            /* "cupy/cuda/memory.pyx":1338
 *                 finally:
 *                     if memptr is None:
 *                         mem_ptr = 0             # <<<<<<<<<<<<<<
 *                         pmem_id = 0
 *                     else:
*/
            __Pyx_INCREF(__pyx_mstate_global->__pyx_int_0);
            __pyx_v_mem_ptr = __pyx_mstate_global->__pyx_int_0;

            /* "cupy/cuda/memory.pyx":1339
 *                     if memptr is None:
 *                         mem_ptr = 0
 *                         pmem_id = 0             # <<<<<<<<<<<<<<
 *                     else:
 *                         mem_ptr = memptr.ptr
*/
            __Pyx_INCREF(__pyx_mstate_global->__pyx_int_0);
            __pyx_v_pmem_id = __pyx_mstate_global->__pyx_int_0;

            /* "cupy/cuda/memory.pyx":1337
 *                     memptr = self._malloc(rounded_size)
 *                 finally:
 *                     if memptr is None:             # <<<<<<<<<<<<<<
 *                         mem_ptr = 0
 *                         pmem_id = 0
*/
            goto __pyx_L10;
          }

          /* "cupy/cuda/memory.pyx":1341
 *                         pmem_id = 0
 *                     else:
 *                         mem_ptr = memptr.ptr             # <<<<<<<<<<<<<<
 *                         pmem_id = id(memptr.mem)
 *                     for hook in hooks.values():
*/
          /*else*/ {
            __pyx_t_2 = PyLong_FromSsize_t(__pyx_v_memptr->ptr); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1341, __pyx_L1_error)
            __Pyx_GOTREF(__pyx_t_2);
            __pyx_v_mem_ptr = __pyx_t_2;
            __pyx_t_2 = 0;

            /* "cupy/cuda/memory.pyx":1342
 *                     else:
 *                         mem_ptr = memptr.ptr
 *                         pmem_id = id(memptr.mem)             # <<<<<<<<<<<<<<
 *                     for hook in hooks.values():
 *                         hook.malloc_postprocess(device_id=device_id,
*/
            __pyx_t_6 = NULL;
            __pyx_t_12 = 1;
            {
              PyObject *__pyx_callargs[2] = {__pyx_t_6, ((PyObject *)__pyx_v_memptr->mem)};
              __pyx_t_2 = __Pyx_PyObject_FastCall((PyObject*)__pyx_builtin_id, __pyx_callargs+__pyx_t_12, (2-__pyx_t_12) | (__pyx_t_12*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
              __Pyx_XDECREF(__pyx_t_6); __pyx_t_6 = 0;
              if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1342, __pyx_L1_error)
              __Pyx_GOTREF(__pyx_t_2);
            }
            __pyx_v_pmem_id = __pyx_t_2;
            __pyx_t_2 = 0;
          }
          __pyx_L10:;

          /* "cupy/cuda/memory.pyx":1343
 *                         mem_ptr = memptr.ptr
 *                         pmem_id = id(memptr.mem)
 *                     for hook in hooks.values():             # <<<<<<<<<<<<<<
 *                         hook.malloc_postprocess(device_id=device_id,
 *                                                 size=size,
*/
          __pyx_t_5 = 0;
          if (unlikely(__pyx_v_hooks == Py_None)) {
            PyErr_Format(PyExc_AttributeError, "'NoneType' object has no attribute '%.30s'", "values");
            __PYX_ERR(0, 1343, __pyx_L1_error)
          }
          __pyx_t_6 = __Pyx_dict_iterator(__pyx_v_hooks, 0, __pyx_mstate_global->__pyx_n_u_values, (&__pyx_t_4), (&__pyx_t_3)); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 1343, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_6);
          __Pyx_XDECREF(__pyx_t_2);
          __pyx_t_2 = __pyx_t_6;
          __pyx_t_6 = 0;
          while (1) {
            __pyx_t_7 = __Pyx_dict_iter_next(__pyx_t_2, __pyx_t_4, &__pyx_t_5, NULL, &__pyx_t_6, NULL, __pyx_t_3);
            if (unlikely(__pyx_t_7 == 0)) break;
            if (unlikely(__pyx_t_7 == -1)) __PYX_ERR(0, 1343, __pyx_L1_error)
            __Pyx_GOTREF(__pyx_t_6);
            __Pyx_XDECREF_SET(__pyx_v_hook, __pyx_t_6);
            __pyx_t_6 = 0;

            /* "cupy/cuda/memory.pyx":1344
 *                         pmem_id = id(memptr.mem)
 *                     for hook in hooks.values():
 *                         hook.malloc_postprocess(device_id=device_id,             # <<<<<<<<<<<<<<
 *                                                 size=size,
 *                                                 mem_size=rounded_size,
*/
            __pyx_t_13 = __pyx_v_hook;
            __Pyx_INCREF(__pyx_t_13);
            __pyx_t_11 = __Pyx_PyLong_From_int(__pyx_v_device_id); if (unlikely(!__pyx_t_11)) __PYX_ERR(0, 1344, __pyx_L1_error)
            __Pyx_GOTREF(__pyx_t_11);

            /* "cupy/cuda/memory.pyx":1345
 *                     for hook in hooks.values():
 *                         hook.malloc_postprocess(device_id=device_id,
 *                                                 size=size,             # <<<<<<<<<<<<<<
 *                                                 mem_size=rounded_size,
 *                                                 mem_ptr=mem_ptr,
*/
            __pyx_t_10 = __Pyx_PyLong_FromSize_t(__pyx_v_size); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 1345, __pyx_L1_error)
            __Pyx_GOTREF(__pyx_t_10);

            /* "cupy/cuda/memory.pyx":1346
 *                         hook.malloc_postprocess(device_id=device_id,
 *                                                 size=size,
 *                                                 mem_size=rounded_size,             # <<<<<<<<<<<<<<
 *                                                 mem_ptr=mem_ptr,
 *                                                 pmem_id=pmem_id)
*/
            __pyx_t_9 = __Pyx_PyLong_FromSize_t(__pyx_v_rounded_size); if (unlikely(!__pyx_t_9)) __PYX_ERR(0, 1346, __pyx_L1_error)
            __Pyx_GOTREF(__pyx_t_9);

            /* "cupy/cuda/memory.pyx":1348
 *                                                 mem_size=rounded_size,
 *                                                 mem_ptr=mem_ptr,
 *                                                 pmem_id=pmem_id)             # <<<<<<<<<<<<<<
 *                 return memptr
 *         return self._malloc(rounded_size)
*/
            __pyx_t_12 = 0;
            {
              PyObject *__pyx_callargs[2 + ((CYTHON_VECTORCALL) ? 5 : 0)] = {__pyx_t_13, NULL};
              __pyx_t_8 = __Pyx_MakeVectorcallBuilderKwds(5); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 1344, __pyx_L1_error)
              __Pyx_GOTREF(__pyx_t_8);
              if (__Pyx_VectorcallBuilder_AddArg(__pyx_mstate_global->__pyx_n_u_device_id, __pyx_t_11, __pyx_t_8, __pyx_callargs+1, 0) < (0)) __PYX_ERR(0, 1344, __pyx_L1_error)
              if (__Pyx_VectorcallBuilder_AddArg(__pyx_mstate_global->__pyx_n_u_size, __pyx_t_10, __pyx_t_8, __pyx_callargs+1, 1) < (0)) __PYX_ERR(0, 1344, __pyx_L1_error)
              if (__Pyx_VectorcallBuilder_AddArg(__pyx_mstate_global->__pyx_n_u_mem_size, __pyx_t_9, __pyx_t_8, __pyx_callargs+1, 2) < (0)) __PYX_ERR(0, 1344, __pyx_L1_error)
              if (__Pyx_VectorcallBuilder_AddArg(__pyx_mstate_global->__pyx_n_u_mem_ptr, __pyx_v_mem_ptr, __pyx_t_8, __pyx_callargs+1, 3) < (0)) __PYX_ERR(0, 1344, __pyx_L1_error)
              if (__Pyx_VectorcallBuilder_AddArg(__pyx_mstate_global->__pyx_n_u_pmem_id, __pyx_v_pmem_id, __pyx_t_8, __pyx_callargs+1, 4) < (0)) __PYX_ERR(0, 1344, __pyx_L1_error)
              __pyx_t_6 = __Pyx_Object_VectorcallMethod_CallFromBuilder((PyObject*)__pyx_mstate_global->__pyx_n_u_malloc_postprocess, __pyx_callargs+__pyx_t_12, (1-__pyx_t_12) | (1*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET), __pyx_t_8);
              __Pyx_XDECREF(__pyx_t_13); __pyx_t_13 = 0;
              __Pyx_DECREF(__pyx_t_11); __pyx_t_11 = 0;
              __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
              __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
              __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
              if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 1344, __pyx_L1_error)
              __Pyx_GOTREF(__pyx_t_6);
            }
            __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
          }
          __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
          goto __pyx_L9;
        }
        __pyx_L8_error:;
        /*exception exit:*/{
          __Pyx_PyThreadState_declare
          __Pyx_PyThreadState_assign
          __pyx_t_15 = 0; __pyx_t_16 = 0; __pyx_t_17 = 0; __pyx_t_18 = 0; __pyx_t_19 = 0; __pyx_t_20 = 0;
          __Pyx_XDECREF(__pyx_t_10); __pyx_t_10 = 0;
          __Pyx_XDECREF(__pyx_t_11); __pyx_t_11 = 0;
          __Pyx_XDECREF(__pyx_t_13); __pyx_t_13 = 0;
          __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
          __Pyx_XDECREF(__pyx_t_6); __pyx_t_6 = 0;
          __Pyx_XDECREF(__pyx_t_8); __pyx_t_8 = 0;
          __Pyx_XDECREF(__pyx_t_9); __pyx_t_9 = 0;
           __Pyx_ExceptionSwap(&__pyx_t_18, &__pyx_t_19, &__pyx_t_20);
          if ( unlikely(__Pyx_GetException(&__pyx_t_15, &__pyx_t_16, &__pyx_t_17) < 0)) __Pyx_ErrFetch(&__pyx_t_15, &__pyx_t_16, &__pyx_t_17);
          __Pyx_XGOTREF(__pyx_t_15);
          __Pyx_XGOTREF(__pyx_t_16);
          __Pyx_XGOTREF(__pyx_t_17);
          __Pyx_XGOTREF(__pyx_t_18);
          __Pyx_XGOTREF(__pyx_t_19);
          __Pyx_XGOTREF(__pyx_t_20);
          __pyx_t_3 = __pyx_lineno; __pyx_t_7 = __pyx_clineno; __pyx_t_14 = __pyx_filename;
          {

            /* "cupy/cuda/memory.pyx":1337
 *                     memptr = self._malloc(rounded_size)
 *                 finally:
 *                     if memptr is None:             # <<<<<<<<<<<<<<
 *                         mem_ptr = 0
 *                         pmem_id = 0
*/
            __pyx_t_1 = (((PyObject *)__pyx_v_memptr) == Py_None);
            if (__pyx_t_1) {

              /* "cupy/cuda/memory.pyx":1338
 *                 finally:
 *                     if memptr is None:
 *                         mem_ptr = 0             # <<<<<<<<<<<<<<
 *                         pmem_id = 0
 *                     else:
*/
              __Pyx_INCREF(__pyx_mstate_global->__pyx_int_0);
              __pyx_v_mem_ptr = __pyx_mstate_global->__pyx_int_0;

              /* "cupy/cuda/memory.pyx":1339
 *                     if memptr is None:
 *                         mem_ptr = 0
 *                         pmem_id = 0             # <<<<<<<<<<<<<<
 *                     else:
 *                         mem_ptr = memptr.ptr
*/
              __Pyx_INCREF(__pyx_mstate_global->__pyx_int_0);
              __pyx_v_pmem_id = __pyx_mstate_global->__pyx_int_0;

              /* "cupy/cuda/memory.pyx":1337
 *                     memptr = self._malloc(rounded_size)
 *                 finally:
 *                     if memptr is None:             # <<<<<<<<<<<<<<
 *                         mem_ptr = 0
 *                         pmem_id = 0
*/
              goto __pyx_L15;
            }

            /* "cupy/cuda/memory.pyx":1341
 *                         pmem_id = 0
 *                     else:
 *                         mem_ptr = memptr.ptr             # <<<<<<<<<<<<<<
 *                         pmem_id = id(memptr.mem)
 *                     for hook in hooks.values():
*/
            /*else*/ {
              __pyx_t_2 = PyLong_FromSsize_t(__pyx_v_memptr->ptr); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1341, __pyx_L14_error)
              __Pyx_GOTREF(__pyx_t_2);
              __pyx_v_mem_ptr = __pyx_t_2;
              __pyx_t_2 = 0;

              /* "cupy/cuda/memory.pyx":1342
 *                     else:
 *                         mem_ptr = memptr.ptr
 *                         pmem_id = id(memptr.mem)             # <<<<<<<<<<<<<<
 *                     for hook in hooks.values():
 *                         hook.malloc_postprocess(device_id=device_id,
*/
              __pyx_t_6 = NULL;
              __pyx_t_12 = 1;
              {
                PyObject *__pyx_callargs[2] = {__pyx_t_6, ((PyObject *)__pyx_v_memptr->mem)};
                __pyx_t_2 = __Pyx_PyObject_FastCall((PyObject*)__pyx_builtin_id, __pyx_callargs+__pyx_t_12, (2-__pyx_t_12) | (__pyx_t_12*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
                __Pyx_XDECREF(__pyx_t_6); __pyx_t_6 = 0;
                if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1342, __pyx_L14_error)
                __Pyx_GOTREF(__pyx_t_2);
              }
              __pyx_v_pmem_id = __pyx_t_2;
              __pyx_t_2 = 0;
            }
            __pyx_L15:;

            /* "cupy/cuda/memory.pyx":1343
 *                         mem_ptr = memptr.ptr
 *                         pmem_id = id(memptr.mem)
 *                     for hook in hooks.values():             # <<<<<<<<<<<<<<
 *                         hook.malloc_postprocess(device_id=device_id,
 *                                                 size=size,
*/
            __pyx_t_4 = 0;
            if (unlikely(__pyx_v_hooks == Py_None)) {
              PyErr_Format(PyExc_AttributeError, "'NoneType' object has no attribute '%.30s'", "values");
              __PYX_ERR(0, 1343, __pyx_L14_error)
            }
            __pyx_t_6 = __Pyx_dict_iterator(__pyx_v_hooks, 0, __pyx_mstate_global->__pyx_n_u_values, (&__pyx_t_5), (&__pyx_t_21)); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 1343, __pyx_L14_error)
            __Pyx_GOTREF(__pyx_t_6);
            __Pyx_XDECREF(__pyx_t_2);
            __pyx_t_2 = __pyx_t_6;
            __pyx_t_6 = 0;
            while (1) {
              __pyx_t_22 = __Pyx_dict_iter_next(__pyx_t_2, __pyx_t_5, &__pyx_t_4, NULL, &__pyx_t_6, NULL, __pyx_t_21);
              if (unlikely(__pyx_t_22 == 0)) break;
              if (unlikely(__pyx_t_22 == -1)) __PYX_ERR(0, 1343, __pyx_L14_error)
              __Pyx_GOTREF(__pyx_t_6);
              __Pyx_XDECREF_SET(__pyx_v_hook, __pyx_t_6);
              __pyx_t_6 = 0;

              /* "cupy/cuda/memory.pyx":1344
 *                         pmem_id = id(memptr.mem)
 *                     for hook in hooks.values():
 *                         hook.malloc_postprocess(device_id=device_id,             # <<<<<<<<<<<<<<
 *                                                 size=size,
 *                                                 mem_size=rounded_size,
*/
              __pyx_t_8 = __pyx_v_hook;
              __Pyx_INCREF(__pyx_t_8);
              __pyx_t_9 = __Pyx_PyLong_From_int(__pyx_v_device_id); if (unlikely(!__pyx_t_9)) __PYX_ERR(0, 1344, __pyx_L14_error)
              __Pyx_GOTREF(__pyx_t_9);

              /* "cupy/cuda/memory.pyx":1345
 *                     for hook in hooks.values():
 *                         hook.malloc_postprocess(device_id=device_id,
 *                                                 size=size,             # <<<<<<<<<<<<<<
 *                                                 mem_size=rounded_size,
 *                                                 mem_ptr=mem_ptr,
*/
              __pyx_t_10 = __Pyx_PyLong_FromSize_t(__pyx_v_size); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 1345, __pyx_L14_error)
              __Pyx_GOTREF(__pyx_t_10);

              /* "cupy/cuda/memory.pyx":1346
 *                         hook.malloc_postprocess(device_id=device_id,
 *                                                 size=size,
 *                                                 mem_size=rounded_size,             # <<<<<<<<<<<<<<
 *                                                 mem_ptr=mem_ptr,
 *                                                 pmem_id=pmem_id)
*/
              __pyx_t_11 = __Pyx_PyLong_FromSize_t(__pyx_v_rounded_size); if (unlikely(!__pyx_t_11)) __PYX_ERR(0, 1346, __pyx_L14_error)
              __Pyx_GOTREF(__pyx_t_11);

              /* "cupy/cuda/memory.pyx":1348
 *                                                 mem_size=rounded_size,
 *                                                 mem_ptr=mem_ptr,
 *                                                 pmem_id=pmem_id)             # <<<<<<<<<<<<<<
 *                 return memptr
 *         return self._malloc(rounded_size)
*/
              __pyx_t_12 = 0;
              {
                PyObject *__pyx_callargs[2 + ((CYTHON_VECTORCALL) ? 5 : 0)] = {__pyx_t_8, NULL};
                __pyx_t_13 = __Pyx_MakeVectorcallBuilderKwds(5); if (unlikely(!__pyx_t_13)) __PYX_ERR(0, 1344, __pyx_L14_error)
                __Pyx_GOTREF(__pyx_t_13);
                if (__Pyx_VectorcallBuilder_AddArg(__pyx_mstate_global->__pyx_n_u_device_id, __pyx_t_9, __pyx_t_13, __pyx_callargs+1, 0) < (0)) __PYX_ERR(0, 1344, __pyx_L14_error)
                if (__Pyx_VectorcallBuilder_AddArg(__pyx_mstate_global->__pyx_n_u_size, __pyx_t_10, __pyx_t_13, __pyx_callargs+1, 1) < (0)) __PYX_ERR(0, 1344, __pyx_L14_error)
                if (__Pyx_VectorcallBuilder_AddArg(__pyx_mstate_global->__pyx_n_u_mem_size, __pyx_t_11, __pyx_t_13, __pyx_callargs+1, 2) < (0)) __PYX_ERR(0, 1344, __pyx_L14_error)
                if (__Pyx_VectorcallBuilder_AddArg(__pyx_mstate_global->__pyx_n_u_mem_ptr, __pyx_v_mem_ptr, __pyx_t_13, __pyx_callargs+1, 3) < (0)) __PYX_ERR(0, 1344, __pyx_L14_error)
                if (__Pyx_VectorcallBuilder_AddArg(__pyx_mstate_global->__pyx_n_u_pmem_id, __pyx_v_pmem_id, __pyx_t_13, __pyx_callargs+1, 4) < (0)) __PYX_ERR(0, 1344, __pyx_L14_error)
                __pyx_t_6 = __Pyx_Object_VectorcallMethod_CallFromBuilder((PyObject*)__pyx_mstate_global->__pyx_n_u_malloc_postprocess, __pyx_callargs+__pyx_t_12, (1-__pyx_t_12) | (1*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET), __pyx_t_13);
                __Pyx_XDECREF(__pyx_t_8); __pyx_t_8 = 0;
                __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
                __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
                __Pyx_DECREF(__pyx_t_11); __pyx_t_11 = 0;
                __Pyx_DECREF(__pyx_t_13); __pyx_t_13 = 0;
                if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 1344, __pyx_L14_error)
                __Pyx_GOTREF(__pyx_t_6);
              }
              __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
            }
            __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
          }
          __Pyx_XGIVEREF(__pyx_t_18);
          __Pyx_XGIVEREF(__pyx_t_19);
          __Pyx_XGIVEREF(__pyx_t_20);
          __Pyx_ExceptionReset(__pyx_t_18, __pyx_t_19, __pyx_t_20);
          __Pyx_XGIVEREF(__pyx_t_15);
          __Pyx_XGIVEREF(__pyx_t_16);
          __Pyx_XGIVEREF(__pyx_t_17);
          __Pyx_ErrRestore(__pyx_t_15, __pyx_t_16, __pyx_t_17);
          __pyx_t_15 = 0; __pyx_t_16 = 0; __pyx_t_17 = 0; __pyx_t_18 = 0; __pyx_t_19 = 0; __pyx_t_20 = 0;
          __pyx_lineno = __pyx_t_3; __pyx_clineno = __pyx_t_7; __pyx_filename = __pyx_t_14;
          goto __pyx_L1_error;
          __pyx_L14_error:;
          __Pyx_XGIVEREF(__pyx_t_18);
          __Pyx_XGIVEREF(__pyx_t_19);
          __Pyx_XGIVEREF(__pyx_t_20);
          __Pyx_ExceptionReset(__pyx_t_18, __pyx_t_19, __pyx_t_20);
          __Pyx_XDECREF(__pyx_t_15); __pyx_t_15 = 0;
          __Pyx_XDECREF(__pyx_t_16); __pyx_t_16 = 0;
          __Pyx_XDECREF(__pyx_t_17); __pyx_t_17 = 0;
          __pyx_t_18 = 0; __pyx_t_19 = 0; __pyx_t_20 = 0;
          goto __pyx_L1_error;
        }
        __pyx_L9:;
      }

      /* "cupy/cuda/memory.pyx":1349
 *                                                 mem_ptr=mem_ptr,
 *                                                 pmem_id=pmem_id)
 *                 return memptr             # <<<<<<<<<<<<<<
 *         return self._malloc(rounded_size)
 * 
*/
      __Pyx_XDECREF((PyObject *)__pyx_r);
      __Pyx_INCREF((PyObject *)__pyx_v_memptr);
      __pyx_r = __pyx_v_memptr;
      goto __pyx_L0;

      /* "cupy/cuda/memory.pyx":1327
 *         if memory_hook._has_memory_hooks():
 *             hooks = memory_hook.get_memory_hooks()
 *             if hooks:             # <<<<<<<<<<<<<<
 *                 memptr = None
 *                 device_id = self._device_id
*/
    }

    /* "cupy/cuda/memory.pyx":1325
 *     cpdef MemoryPointer malloc(self, size_t size):
 *         rounded_size = _round_size(size)
 *         if memory_hook._has_memory_hooks():             # <<<<<<<<<<<<<<
 *             hooks = memory_hook.get_memory_hooks()
 *             if hooks:
*/
  }

  /* "cupy/cuda/memory.pyx":1350
 *                                                 pmem_id=pmem_id)
 *                 return memptr
 *         return self._malloc(rounded_size)             # <<<<<<<<<<<<<<
 * 
 *     cdef MemoryPointer _malloc(self, size_t size):
*/
  __Pyx_XDECREF((PyObject *)__pyx_r);
  __pyx_t_2 = ((PyObject *)__pyx_f_4cupy_4cuda_6memory_22SingleDeviceMemoryPool__malloc(__pyx_v_self, __pyx_v_rounded_size)); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1350, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_r = ((struct __pyx_obj_4cupy_4cuda_6memory_MemoryPointer *)__pyx_t_2);
  __pyx_t_2 = 0;
  goto __pyx_L0;

  /* "cupy/cuda/memory.pyx":1323
 *         return self._allocator(rounded_size)
 * 
 *     cpdef MemoryPointer malloc(self, size_t size):             # <<<<<<<<<<<<<<
 *         rounded_size = _round_size(size)
 *         if memory_hook._has_memory_hooks():
*/

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_XDECREF(__pyx_t_8);
  __Pyx_XDECREF(__pyx_t_9);
  __Pyx_XDECREF(__pyx_t_10);
  __Pyx_XDECREF(__pyx_t_11);
  __Pyx_XDECREF(__pyx_t_13);
  __Pyx_AddTraceback("cupy.cuda.memory.SingleDeviceMemoryPool.malloc", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_hooks);
  __Pyx_XDECREF((PyObject *)__pyx_v_memptr);
  __Pyx_XDECREF(__pyx_v_hook);
  __Pyx_XDECREF(__pyx_v_mem_ptr);
  __Pyx_XDECREF(__pyx_v_pmem_id);
  __Pyx_XGIVEREF((PyObject *)__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_pw_4cupy_4cuda_6memory_22SingleDeviceMemoryPool_7malloc(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
PyDoc_STRVAR(__pyx_doc_4cupy_4cuda_6memory_22SingleDeviceMemoryPool_6malloc, "SingleDeviceMemoryPool.malloc(self, size_t size) -> MemoryPointer");
static PyObject *__pyx_pw_4cupy_4cuda_6memory_22SingleDeviceMemoryPool_7malloc(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
) {
  size_t __pyx_v_size;
  #if !CYTHON_METH_FASTCALL
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  #endif
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject* values[1] = {0};
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("malloc (wrapper)", 0);
  #if !CYTHON_METH_FASTCALL
  #if CYTHON_ASSUME_SAFE_SIZE
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return NULL;
  #endif
  #endif
  __pyx_kwvalues = __Pyx_KwValues_FASTCALL(__pyx_args, __pyx_nargs);
  {
    PyObject ** const __pyx_pyargnames[] = {&__pyx_mstate_global->__pyx_n_u_size,0};
    const Py_ssize_t __pyx_kwds_len = (__pyx_kwds) ? __Pyx_NumKwargs_FASTCALL(__pyx_kwds) : 0;
    if (unlikely(__pyx_kwds_len < 0)) __PYX_ERR(0, 1323, __pyx_L3_error)
    if (__pyx_kwds_len > 0) {
      switch (__pyx_nargs) {
        case  1:
        values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(0, 1323, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      const Py_ssize_t kwd_pos_args = __pyx_nargs;
      if (__Pyx_ParseKeywords(__pyx_kwds, __pyx_kwvalues, __pyx_pyargnames, 0, values, kwd_pos_args, __pyx_kwds_len, "malloc", 0) < (0)) __PYX_ERR(0, 1323, __pyx_L3_error)
      for (Py_ssize_t i = __pyx_nargs; i < 1; i++) {
        if (unlikely(!values[i])) { __Pyx_RaiseArgtupleInvalid("malloc", 1, 1, 1, i); __PYX_ERR(0, 1323, __pyx_L3_error) }
      }
    } else if (unlikely(__pyx_nargs != 1)) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
      if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(0, 1323, __pyx_L3_error)
    }
    __pyx_v_size = __Pyx_PyLong_As_size_t(values[0]); if (unlikely((__pyx_v_size == (size_t)-1) && PyErr_Occurred())) __PYX_ERR(0, 1323, __pyx_L3_error)
  }
  goto __pyx_L6_skip;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("malloc", 1, 1, 1, __pyx_nargs); __PYX_ERR(0, 1323, __pyx_L3_error)
  __pyx_L6_skip:;
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_AddTraceback("cupy.cuda.memory.SingleDeviceMemoryPool.malloc", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_4cupy_4cuda_6memory_22SingleDeviceMemoryPool_6malloc(((struct __pyx_obj_4cupy_4cuda_6memory_SingleDeviceMemoryPool *)__pyx_v_self), __pyx_v_size);

  /* function exit code */
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4cupy_4cuda_6memory_22SingleDeviceMemoryPool_6malloc(struct __pyx_obj_4cupy_4cuda_6memory_SingleDeviceMemoryPool *__pyx_v_self, size_t __pyx_v_size) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("malloc", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = ((PyObject *)__pyx_f_4cupy_4cuda_6memory_22SingleDeviceMemoryPool_malloc(__pyx_v_self, __pyx_v_size, 1)); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1323, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("cupy.cuda.memory.SingleDeviceMemoryPool.malloc", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "cupy/cuda/memory.pyx":1352
 *         return self._malloc(rounded_size)
 * 
 *     cdef MemoryPointer _malloc(self, size_t size):             # <<<<<<<<<<<<<<
 *         cdef _Arena arena
 *         cdef _Chunk chunk
*/

static struct __pyx_obj_4cupy_4cuda_6memory_MemoryPointer *__pyx_f_4cupy_4cuda_6memory_22SingleDeviceMemoryPool__malloc(struct __pyx_obj_4cupy_4cuda_6memory_SingleDeviceMemoryPool *__pyx_v_self, size_t __pyx_v_size) {
  struct __pyx_obj_4cupy_4cuda_6memory__Arena *__pyx_v_arena = 0;
  struct __pyx_obj_4cupy_4cuda_6memory__Chunk *__pyx_v_chunk = 0;
  struct __pyx_obj_4cupy_4cuda_6memory_BaseMemory *__pyx_v_mem = 0;
  struct __pyx_obj_4cupy_4cuda_6memory_PooledMemory *__pyx_v_pmem = 0;
  struct __pyx_obj_4cupy_4cuda_6memory_MemoryPointer *__pyx_v_ret = 0;
  intptr_t __pyx_v_stream_ident;
  struct __pyx_obj_4cupy_4cuda_6memory_MemoryPointer *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  size_t __pyx_t_6;
  intptr_t __pyx_t_7;
  intptr_t __pyx_t_8;
  int __pyx_t_9;
  int __pyx_t_10;
  char const *__pyx_t_11;
  PyObject *__pyx_t_12 = NULL;
  PyObject *__pyx_t_13 = NULL;
  PyObject *__pyx_t_14 = NULL;
  PyObject *__pyx_t_15 = NULL;
  PyObject *__pyx_t_16 = NULL;
  PyObject *__pyx_t_17 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("_malloc", 0);

  /* "cupy/cuda/memory.pyx":1358
 *         cdef PooledMemory pmem
 *         cdef MemoryPointer ret
 *         if size == 0:             # <<<<<<<<<<<<<<
 *             return MemoryPointer(Memory(0), 0)
 * 
*/
  __pyx_t_1 = (__pyx_v_size == 0);
  if (__pyx_t_1) {

    /* "cupy/cuda/memory.pyx":1359
 *         cdef MemoryPointer ret
 *         if size == 0:
 *             return MemoryPointer(Memory(0), 0)             # <<<<<<<<<<<<<<
 * 
 *         stream_ident = _get_stream_identifier(
*/
    __Pyx_XDECREF((PyObject *)__pyx_r);
    __pyx_t_3 = NULL;
    __pyx_t_5 = NULL;
    __pyx_t_6 = 1;
    {
      PyObject *__pyx_callargs[2] = {__pyx_t_5, __pyx_mstate_global->__pyx_int_0};
      __pyx_t_4 = __Pyx_PyObject_FastCall((PyObject*)__pyx_mstate_global->__pyx_ptype_4cupy_4cuda_6memory_Memory, __pyx_callargs+__pyx_t_6, (2-__pyx_t_6) | (__pyx_t_6*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
      __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
      if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 1359, __pyx_L1_error)
      __Pyx_GOTREF((PyObject *)__pyx_t_4);
    }
    __pyx_t_6 = 1;
    {
      PyObject *__pyx_callargs[3] = {__pyx_t_3, ((PyObject *)__pyx_t_4), __pyx_mstate_global->__pyx_int_0};
      __pyx_t_2 = __Pyx_PyObject_FastCall((PyObject*)__pyx_mstate_global->__pyx_ptype_4cupy_4cuda_6memory_MemoryPointer, __pyx_callargs+__pyx_t_6, (3-__pyx_t_6) | (__pyx_t_6*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
      __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
      __Pyx_DECREF((PyObject *)__pyx_t_4); __pyx_t_4 = 0;
      if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1359, __pyx_L1_error)
      __Pyx_GOTREF((PyObject *)__pyx_t_2);
    }
    __pyx_r = ((struct __pyx_obj_4cupy_4cuda_6memory_MemoryPointer *)__pyx_t_2);
    __pyx_t_2 = 0;
    goto __pyx_L0;

    /* "cupy/cuda/memory.pyx":1358
 *         cdef PooledMemory pmem
 *         cdef MemoryPointer ret
 *         if size == 0:             # <<<<<<<<<<<<<<
 *             return MemoryPointer(Memory(0), 0)
 * 
*/
  }

  /* "cupy/cuda/memory.pyx":1362
 * 
 *         stream_ident = _get_stream_identifier(
 *             stream_module.get_current_stream_ptr())             # <<<<<<<<<<<<<<
 * 
 *         if not self._arena_mutex.try_lock():
*/
  __pyx_t_7 = __pyx_f_4cupy_4cuda_6stream_get_current_stream_ptr(); if (unlikely(__pyx_t_7 == ((intptr_t)-1L) && PyErr_Occurred())) __PYX_ERR(0, 1362, __pyx_L1_error)

  /* "cupy/cuda/memory.pyx":1361
 *             return MemoryPointer(Memory(0), 0)
 * 
 *         stream_ident = _get_stream_identifier(             # <<<<<<<<<<<<<<
 *             stream_module.get_current_stream_ptr())
 * 
*/
  __pyx_t_8 = __pyx_f_4cupy_4cuda_6memory__get_stream_identifier(__pyx_t_7); if (unlikely(__pyx_t_8 == ((intptr_t)-1L) && PyErr_Occurred())) __PYX_ERR(0, 1361, __pyx_L1_error)
  __pyx_v_stream_ident = __pyx_t_8;

  /* "cupy/cuda/memory.pyx":1364
 *             stream_module.get_current_stream_ptr())
 * 
 *         if not self._arena_mutex.try_lock():             # <<<<<<<<<<<<<<
 *             with nogil:
 *                 self._arena_mutex.lock()
*/
  __pyx_t_1 = (!(__pyx_v_self->_arena_mutex.try_lock() != 0));
  if (__pyx_t_1) {

    /* "cupy/cuda/memory.pyx":1365
 * 
 *         if not self._arena_mutex.try_lock():
 *             with nogil:             # <<<<<<<<<<<<<<
 *                 self._arena_mutex.lock()
 *         try:
*/
    {
        PyThreadState * _save;
        _save = PyEval_SaveThread();
        __Pyx_FastGIL_Remember();
        /*try:*/ {

          /* "cupy/cuda/memory.pyx":1366
 *         if not self._arena_mutex.try_lock():
 *             with nogil:
 *                 self._arena_mutex.lock()             # <<<<<<<<<<<<<<
 *         try:
 *             arena = self._arena(stream_ident)
*/
          try {
            __pyx_v_self->_arena_mutex.lock();
          } catch(...) {
            PyGILState_STATE __pyx_gilstate_save = __Pyx_PyGILState_Ensure();
            __Pyx_CppExn2PyErr();
            __Pyx_PyGILState_Release(__pyx_gilstate_save);
            __PYX_ERR(0, 1366, __pyx_L6_error)
          }
        }

        /* "cupy/cuda/memory.pyx":1365
 * 
 *         if not self._arena_mutex.try_lock():
 *             with nogil:             # <<<<<<<<<<<<<<
 *                 self._arena_mutex.lock()
 *         try:
*/
        /*finally:*/ {
          /*normal exit:*/{
            __Pyx_FastGIL_Forget();
            PyEval_RestoreThread(_save);
            goto __pyx_L7;
          }
          __pyx_L6_error: {
            __Pyx_FastGIL_Forget();
            PyEval_RestoreThread(_save);
            goto __pyx_L1_error;
          }
          __pyx_L7:;
        }
    }

    /* "cupy/cuda/memory.pyx":1364
 *             stream_module.get_current_stream_ptr())
 * 
 *         if not self._arena_mutex.try_lock():             # <<<<<<<<<<<<<<
 *             with nogil:
 *                 self._arena_mutex.lock()
*/
  }

  /* "cupy/cuda/memory.pyx":1367
 *             with nogil:
 *                 self._arena_mutex.lock()
 *         try:             # <<<<<<<<<<<<<<
 *             arena = self._arena(stream_ident)
 *             # find best-fit, or a smallest larger allocation
*/
  /*try:*/ {

    /* "cupy/cuda/memory.pyx":1368
 *                 self._arena_mutex.lock()
 *         try:
 *             arena = self._arena(stream_ident)             # <<<<<<<<<<<<<<
 *             # find best-fit, or a smallest larger allocation
 *             chunk = arena.get_chunk(size)
*/
    __pyx_t_2 = ((PyObject *)__pyx_f_4cupy_4cuda_6memory_22SingleDeviceMemoryPool__arena(__pyx_v_self, __pyx_v_stream_ident, 0)); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1368, __pyx_L9_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_v_arena = ((struct __pyx_obj_4cupy_4cuda_6memory__Arena *)__pyx_t_2);
    __pyx_t_2 = 0;

    /* "cupy/cuda/memory.pyx":1370
 *             arena = self._arena(stream_ident)
 *             # find best-fit, or a smallest larger allocation
 *             chunk = arena.get_chunk(size)             # <<<<<<<<<<<<<<
 *         finally:
 *             self._arena_mutex.unlock()
*/
    __pyx_t_2 = ((PyObject *)__pyx_f_4cupy_4cuda_6memory_6_Arena_get_chunk(__pyx_v_arena, __pyx_v_size)); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1370, __pyx_L9_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_v_chunk = ((struct __pyx_obj_4cupy_4cuda_6memory__Chunk *)__pyx_t_2);
    __pyx_t_2 = 0;
  }

  /* "cupy/cuda/memory.pyx":1372
 *             chunk = arena.get_chunk(size)
 *         finally:
 *             self._arena_mutex.unlock()             # <<<<<<<<<<<<<<
 * 
 *         if chunk is None:
*/
  /*finally:*/ {
    /*normal exit:*/{
      __pyx_v_self->_arena_mutex.unlock();
      goto __pyx_L10;
    }
    __pyx_L9_error:;
    /*exception exit:*/{
      __Pyx_PyThreadState_declare
      __Pyx_PyThreadState_assign
      __pyx_t_12 = 0; __pyx_t_13 = 0; __pyx_t_14 = 0; __pyx_t_15 = 0; __pyx_t_16 = 0; __pyx_t_17 = 0;
      __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
      __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
      __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
      __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
       __Pyx_ExceptionSwap(&__pyx_t_15, &__pyx_t_16, &__pyx_t_17);
      if ( unlikely(__Pyx_GetException(&__pyx_t_12, &__pyx_t_13, &__pyx_t_14) < 0)) __Pyx_ErrFetch(&__pyx_t_12, &__pyx_t_13, &__pyx_t_14);
      __Pyx_XGOTREF(__pyx_t_12);
      __Pyx_XGOTREF(__pyx_t_13);
      __Pyx_XGOTREF(__pyx_t_14);
      __Pyx_XGOTREF(__pyx_t_15);
      __Pyx_XGOTREF(__pyx_t_16);
      __Pyx_XGOTREF(__pyx_t_17);
      __pyx_t_9 = __pyx_lineno; __pyx_t_10 = __pyx_clineno; __pyx_t_11 = __pyx_filename;
      {
        __pyx_v_self->_arena_mutex.unlock();
      }
      __Pyx_XGIVEREF(__pyx_t_15);
      __Pyx_XGIVEREF(__pyx_t_16);
      __Pyx_XGIVEREF(__pyx_t_17);
      __Pyx_ExceptionReset(__pyx_t_15, __pyx_t_16, __pyx_t_17);
      __Pyx_XGIVEREF(__pyx_t_12);
      __Pyx_XGIVEREF(__pyx_t_13);
      __Pyx_XGIVEREF(__pyx_t_14);
      __Pyx_ErrRestore(__pyx_t_12, __pyx_t_13, __pyx_t_14);
      __pyx_t_12 = 0; __pyx_t_13 = 0; __pyx_t_14 = 0; __pyx_t_15 = 0; __pyx_t_16 = 0; __pyx_t_17 = 0;
      __pyx_lineno = __pyx_t_9; __pyx_clineno = __pyx_t_10; __pyx_filename = __pyx_t_11;
      goto __pyx_L1_error;
    }
    __pyx_L10:;
  }

  /* "cupy/cuda/memory.pyx":1374
 *             self._arena_mutex.unlock()
 * 
 *         if chunk is None:             # <<<<<<<<<<<<<<
 *             # cudaMalloc if a cache chunk is not found
 *             mem = self._try_malloc(size)
*/
  __pyx_t_1 = (((PyObject *)__pyx_v_chunk) == Py_None);
  if (__pyx_t_1) {

    /* "cupy/cuda/memory.pyx":1376
 *         if chunk is None:
 *             # cudaMalloc if a cache chunk is not found
 *             mem = self._try_malloc(size)             # <<<<<<<<<<<<<<
 *             chunk = _Chunk.__new__(_Chunk)
 *             chunk._init(mem, 0, size, arena)
*/
    __pyx_t_2 = ((PyObject *)__pyx_f_4cupy_4cuda_6memory_22SingleDeviceMemoryPool__try_malloc(__pyx_v_self, __pyx_v_size)); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1376, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_v_mem = ((struct __pyx_obj_4cupy_4cuda_6memory_BaseMemory *)__pyx_t_2);
    __pyx_t_2 = 0;

    /* "cupy/cuda/memory.pyx":1377
 *             # cudaMalloc if a cache chunk is not found
 *             mem = self._try_malloc(size)
 *             chunk = _Chunk.__new__(_Chunk)             # <<<<<<<<<<<<<<
 *             chunk._init(mem, 0, size, arena)
 * 
*/
    __pyx_t_2 = ((PyObject *)__pyx_tp_new_4cupy_4cuda_6memory__Chunk(((PyTypeObject *)__pyx_mstate_global->__pyx_ptype_4cupy_4cuda_6memory__Chunk), __pyx_mstate_global->__pyx_empty_tuple, NULL)); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1377, __pyx_L1_error)
    __Pyx_GOTREF((PyObject *)__pyx_t_2);
    __Pyx_DECREF_SET(__pyx_v_chunk, ((struct __pyx_obj_4cupy_4cuda_6memory__Chunk *)__pyx_t_2));
    __pyx_t_2 = 0;

    /* "cupy/cuda/memory.pyx":1378
 *             mem = self._try_malloc(size)
 *             chunk = _Chunk.__new__(_Chunk)
 *             chunk._init(mem, 0, size, arena)             # <<<<<<<<<<<<<<
 * 
 *         self._in_use_bytes += chunk.size
*/
    __pyx_t_2 = __pyx_f_4cupy_4cuda_6memory_6_Chunk__init(__pyx_v_chunk, __pyx_v_mem, 0, __pyx_v_size, __pyx_v_arena); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1378, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

    /* "cupy/cuda/memory.pyx":1374
 *             self._arena_mutex.unlock()
 * 
 *         if chunk is None:             # <<<<<<<<<<<<<<
 *             # cudaMalloc if a cache chunk is not found
 *             mem = self._try_malloc(size)
*/
  }

  /* "cupy/cuda/memory.pyx":1380
 *             chunk._init(mem, 0, size, arena)
 * 
 *         self._in_use_bytes += chunk.size             # <<<<<<<<<<<<<<
 * 
 *         pmem = PooledMemory.__new__(PooledMemory)
*/
  __pyx_v_self->_in_use_bytes += __pyx_v_chunk->size;

  /* "cupy/cuda/memory.pyx":1382
 *         self._in_use_bytes += chunk.size
 * 
 *         pmem = PooledMemory.__new__(PooledMemory)             # <<<<<<<<<<<<<<
 *         pmem._init(chunk, self._weakref)
 *         ret = MemoryPointer.__new__(MemoryPointer)
*/
  __pyx_t_2 = ((PyObject *)__pyx_tp_new_4cupy_4cuda_6memory_PooledMemory(((PyTypeObject *)__pyx_mstate_global->__pyx_ptype_4cupy_4cuda_6memory_PooledMemory), __pyx_mstate_global->__pyx_empty_tuple, NULL)); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1382, __pyx_L1_error)
  __Pyx_GOTREF((PyObject *)__pyx_t_2);
  __pyx_v_pmem = ((struct __pyx_obj_4cupy_4cuda_6memory_PooledMemory *)__pyx_t_2);
  __pyx_t_2 = 0;

  /* "cupy/cuda/memory.pyx":1383
 * 
 *         pmem = PooledMemory.__new__(PooledMemory)
 *         pmem._init(chunk, self._weakref)             # <<<<<<<<<<<<<<
 *         ret = MemoryPointer.__new__(MemoryPointer)
 *         ret._init(pmem, 0)
*/
  __pyx_t_2 = __pyx_v_self->_weakref;
  __Pyx_INCREF(__pyx_t_2);
  __pyx_t_4 = __pyx_f_4cupy_4cuda_6memory_12PooledMemory__init(__pyx_v_pmem, __pyx_v_chunk, __pyx_t_2); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 1383, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

  /* "cupy/cuda/memory.pyx":1384
 *         pmem = PooledMemory.__new__(PooledMemory)
 *         pmem._init(chunk, self._weakref)
 *         ret = MemoryPointer.__new__(MemoryPointer)             # <<<<<<<<<<<<<<
 *         ret._init(pmem, 0)
 *         return ret
*/
  __pyx_t_4 = ((PyObject *)__pyx_tp_new_4cupy_4cuda_6memory_MemoryPointer(((PyTypeObject *)__pyx_mstate_global->__pyx_ptype_4cupy_4cuda_6memory_MemoryPointer), __pyx_mstate_global->__pyx_empty_tuple, NULL)); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 1384, __pyx_L1_error)
  __Pyx_GOTREF((PyObject *)__pyx_t_4);
  __pyx_v_ret = ((struct __pyx_obj_4cupy_4cuda_6memory_MemoryPointer *)__pyx_t_4);
  __pyx_t_4 = 0;

  /* "cupy/cuda/memory.pyx":1385
 *         pmem._init(chunk, self._weakref)
 *         ret = MemoryPointer.__new__(MemoryPointer)
 *         ret._init(pmem, 0)             # <<<<<<<<<<<<<<
 *         return ret
 * 
*/
  __pyx_t_4 = ((struct __pyx_vtabstruct_4cupy_4cuda_6memory_MemoryPointer *)__pyx_v_ret->__pyx_vtab)->_init(__pyx_v_ret, ((struct __pyx_obj_4cupy_4cuda_6memory_BaseMemory *)__pyx_v_pmem), 0); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 1385, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

  /* "cupy/cuda/memory.pyx":1386
 *         ret = MemoryPointer.__new__(MemoryPointer)
 *         ret._init(pmem, 0)
 *         return ret             # <<<<<<<<<<<<<<
 * 
 *     cdef free(self, _Chunk chunk):
*/
  __Pyx_XDECREF((PyObject *)__pyx_r);
  __Pyx_INCREF((PyObject *)__pyx_v_ret);
  __pyx_r = __pyx_v_ret;
  goto __pyx_L0;

  /* "cupy/cuda/memory.pyx":1352
 *         return self._malloc(rounded_size)
 * 
 *     cdef MemoryPointer _malloc(self, size_t size):             # <<<<<<<<<<<<<<
 *         cdef _Arena arena
 *         cdef _Chunk chunk
*/

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_AddTraceback("cupy.cuda.memory.SingleDeviceMemoryPool._malloc", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XDECREF((PyObject *)__pyx_v_arena);
  __Pyx_XDECREF((PyObject *)__pyx_v_chunk);
  __Pyx_XDECREF((PyObject *)__pyx_v_mem);
  __Pyx_XDECREF((PyObject *)__pyx_v_pmem);
  __Pyx_XDECREF((PyObject *)__pyx_v_ret);
  __Pyx_XGIVEREF((PyObject *)__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "cupy/cuda/memory.pyx":1388
 *         return ret
 * 
 *     cdef free(self, _Chunk chunk):             # <<<<<<<<<<<<<<
 *         self._in_use_bytes -= chunk.size
 * 
*/

static PyObject *__pyx_f_4cupy_4cuda_6memory_22SingleDeviceMemoryPool_free(struct __pyx_obj_4cupy_4cuda_6memory_SingleDeviceMemoryPool *__pyx_v_self, struct __pyx_obj_4cupy_4cuda_6memory__Chunk *__pyx_v_chunk) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  int __pyx_t_3;
  int __pyx_t_4;
  char const *__pyx_t_5;
  PyObject *__pyx_t_6 = NULL;
  PyObject *__pyx_t_7 = NULL;
  PyObject *__pyx_t_8 = NULL;
  PyObject *__pyx_t_9 = NULL;
  PyObject *__pyx_t_10 = NULL;
  PyObject *__pyx_t_11 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("free", 0);

  /* "cupy/cuda/memory.pyx":1389
 * 
 *     cdef free(self, _Chunk chunk):
 *         self._in_use_bytes -= chunk.size             # <<<<<<<<<<<<<<
 * 
 *         # Make sure freeing is always safe, but if we can lock do it.
*/
  __pyx_v_self->_in_use_bytes -= __pyx_v_chunk->size;

  /* "cupy/cuda/memory.pyx":1392
 * 
 *         # Make sure freeing is always safe, but if we can lock do it.
 *         if self._arena_mutex.try_lock():             # <<<<<<<<<<<<<<
 *             try:
 *                 chunk.arena.insert_chunk(chunk)
*/
  __pyx_t_1 = (__pyx_v_self->_arena_mutex.try_lock() != 0);
  if (__pyx_t_1) {

    /* "cupy/cuda/memory.pyx":1393
 *         # Make sure freeing is always safe, but if we can lock do it.
 *         if self._arena_mutex.try_lock():
 *             try:             # <<<<<<<<<<<<<<
 *                 chunk.arena.insert_chunk(chunk)
 *             finally:
*/
    /*try:*/ {

      /* "cupy/cuda/memory.pyx":1394
 *         if self._arena_mutex.try_lock():
 *             try:
 *                 chunk.arena.insert_chunk(chunk)             # <<<<<<<<<<<<<<
 *             finally:
 *                 self._arena_mutex.unlock()
*/
      __pyx_t_2 = __pyx_f_4cupy_4cuda_6memory_6_Arena_insert_chunk(__pyx_v_chunk->arena, __pyx_v_chunk, NULL); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1394, __pyx_L5_error)
      __Pyx_GOTREF(__pyx_t_2);
      __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    }

    /* "cupy/cuda/memory.pyx":1396
 *                 chunk.arena.insert_chunk(chunk)
 *             finally:
 *                 self._arena_mutex.unlock()             # <<<<<<<<<<<<<<
 *         else:
 *             chunk.arena.add_pending_free_atomic(chunk)
*/
    /*finally:*/ {
      /*normal exit:*/{
        __pyx_v_self->_arena_mutex.unlock();
        goto __pyx_L6;
      }
      __pyx_L5_error:;
      /*exception exit:*/{
        __Pyx_PyThreadState_declare
        __Pyx_PyThreadState_assign
        __pyx_t_6 = 0; __pyx_t_7 = 0; __pyx_t_8 = 0; __pyx_t_9 = 0; __pyx_t_10 = 0; __pyx_t_11 = 0;
        __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
         __Pyx_ExceptionSwap(&__pyx_t_9, &__pyx_t_10, &__pyx_t_11);
        if ( unlikely(__Pyx_GetException(&__pyx_t_6, &__pyx_t_7, &__pyx_t_8) < 0)) __Pyx_ErrFetch(&__pyx_t_6, &__pyx_t_7, &__pyx_t_8);
        __Pyx_XGOTREF(__pyx_t_6);
        __Pyx_XGOTREF(__pyx_t_7);
        __Pyx_XGOTREF(__pyx_t_8);
        __Pyx_XGOTREF(__pyx_t_9);
        __Pyx_XGOTREF(__pyx_t_10);
        __Pyx_XGOTREF(__pyx_t_11);
        __pyx_t_3 = __pyx_lineno; __pyx_t_4 = __pyx_clineno; __pyx_t_5 = __pyx_filename;
        {
          __pyx_v_self->_arena_mutex.unlock();
        }
        __Pyx_XGIVEREF(__pyx_t_9);
        __Pyx_XGIVEREF(__pyx_t_10);
        __Pyx_XGIVEREF(__pyx_t_11);
        __Pyx_ExceptionReset(__pyx_t_9, __pyx_t_10, __pyx_t_11);
        __Pyx_XGIVEREF(__pyx_t_6);
        __Pyx_XGIVEREF(__pyx_t_7);
        __Pyx_XGIVEREF(__pyx_t_8);
        __Pyx_ErrRestore(__pyx_t_6, __pyx_t_7, __pyx_t_8);
        __pyx_t_6 = 0; __pyx_t_7 = 0; __pyx_t_8 = 0; __pyx_t_9 = 0; __pyx_t_10 = 0; __pyx_t_11 = 0;
        __pyx_lineno = __pyx_t_3; __pyx_clineno = __pyx_t_4; __pyx_filename = __pyx_t_5;
        goto __pyx_L1_error;
      }
      __pyx_L6:;
    }

    /* "cupy/cuda/memory.pyx":1392
 * 
 *         # Make sure freeing is always safe, but if we can lock do it.
 *         if self._arena_mutex.try_lock():             # <<<<<<<<<<<<<<
 *             try:
 *                 chunk.arena.insert_chunk(chunk)
*/
    goto __pyx_L3;
  }

  /* "cupy/cuda/memory.pyx":1398
 *                 self._arena_mutex.unlock()
 *         else:
 *             chunk.arena.add_pending_free_atomic(chunk)             # <<<<<<<<<<<<<<
 * 
 *     cpdef free_all_blocks(self, stream=None):
*/
  /*else*/ {
    __pyx_t_2 = __pyx_f_4cupy_4cuda_6memory_6_Arena_add_pending_free_atomic(__pyx_v_chunk->arena, __pyx_v_chunk); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1398, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  }
  __pyx_L3:;

  /* "cupy/cuda/memory.pyx":1388
 *         return ret
 * 
 *     cdef free(self, _Chunk chunk):             # <<<<<<<<<<<<<<
 *         self._in_use_bytes -= chunk.size
 * 
*/

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_AddTraceback("cupy.cuda.memory.SingleDeviceMemoryPool.free", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "cupy/cuda/memory.pyx":1400
 *             chunk.arena.add_pending_free_atomic(chunk)
 * 
 *     cpdef free_all_blocks(self, stream=None):             # <<<<<<<<<<<<<<
 *         """Free all **non-split** blocks for one or all arenas.
 *         """
*/

static PyObject *__pyx_pw_4cupy_4cuda_6memory_22SingleDeviceMemoryPool_9free_all_blocks(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
static PyObject *__pyx_f_4cupy_4cuda_6memory_22SingleDeviceMemoryPool_free_all_blocks(struct __pyx_obj_4cupy_4cuda_6memory_SingleDeviceMemoryPool *__pyx_v_self, CYTHON_UNUSED int __pyx_skip_dispatch, struct __pyx_opt_args_4cupy_4cuda_6memory_22SingleDeviceMemoryPool_free_all_blocks *__pyx_optional_args) {
  PyObject *__pyx_v_stream = ((PyObject *)Py_None);
  intptr_t __pyx_v_stream_ident;
  struct __pyx_obj_4cupy_4cuda_6memory__Arena *__pyx_v_arena = 0;
  PyObject *__pyx_v_idents = 0;
  size_t __pyx_v_bytes_freed;
  PyObject *__pyx_v_ident = NULL;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  intptr_t __pyx_t_4;
  intptr_t __pyx_t_5;
  Py_ssize_t __pyx_t_6;
  PyObject *__pyx_t_7 = NULL;
  PyObject *__pyx_t_8 = NULL;
  size_t __pyx_t_9;
  int __pyx_t_10;
  int __pyx_t_11;
  char const *__pyx_t_12;
  PyObject *__pyx_t_13 = NULL;
  PyObject *__pyx_t_14 = NULL;
  PyObject *__pyx_t_15 = NULL;
  PyObject *__pyx_t_16 = NULL;
  PyObject *__pyx_t_17 = NULL;
  PyObject *__pyx_t_18 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("free_all_blocks", 0);
  if (__pyx_optional_args) {
    if (__pyx_optional_args->__pyx_n > 0) {
      __pyx_v_stream = __pyx_optional_args->stream;
    }
  }

  /* "cupy/cuda/memory.pyx":1406
 *         cdef _Arena arena
 *         cdef tuple idents
 *         cdef size_t bytes_freed = 0             # <<<<<<<<<<<<<<
 * 
 *         if not self._arena_mutex.try_lock():
*/
  __pyx_v_bytes_freed = 0;

  /* "cupy/cuda/memory.pyx":1408
 *         cdef size_t bytes_freed = 0
 * 
 *         if not self._arena_mutex.try_lock():             # <<<<<<<<<<<<<<
 *             with nogil:
 *                 self._arena_mutex.lock()
*/
  __pyx_t_1 = (!(__pyx_v_self->_arena_mutex.try_lock() != 0));
  if (__pyx_t_1) {

    /* "cupy/cuda/memory.pyx":1409
 * 
 *         if not self._arena_mutex.try_lock():
 *             with nogil:             # <<<<<<<<<<<<<<
 *                 self._arena_mutex.lock()
 *         try:
*/
    {
        PyThreadState * _save;
        _save = PyEval_SaveThread();
        __Pyx_FastGIL_Remember();
        /*try:*/ {

          /* "cupy/cuda/memory.pyx":1410
 *         if not self._arena_mutex.try_lock():
 *             with nogil:
 *                 self._arena_mutex.lock()             # <<<<<<<<<<<<<<
 *         try:
 *             if stream is None:
*/
          try {
            __pyx_v_self->_arena_mutex.lock();
          } catch(...) {
            PyGILState_STATE __pyx_gilstate_save = __Pyx_PyGILState_Ensure();
            __Pyx_CppExn2PyErr();
            __Pyx_PyGILState_Release(__pyx_gilstate_save);
            __PYX_ERR(0, 1410, __pyx_L5_error)
          }
        }

        /* "cupy/cuda/memory.pyx":1409
 * 
 *         if not self._arena_mutex.try_lock():
 *             with nogil:             # <<<<<<<<<<<<<<
 *                 self._arena_mutex.lock()
 *         try:
*/
        /*finally:*/ {
          /*normal exit:*/{
            __Pyx_FastGIL_Forget();
            PyEval_RestoreThread(_save);
            goto __pyx_L6;
          }
          __pyx_L5_error: {
            __Pyx_FastGIL_Forget();
            PyEval_RestoreThread(_save);
            goto __pyx_L1_error;
          }
          __pyx_L6:;
        }
    }

    /* "cupy/cuda/memory.pyx":1408
 *         cdef size_t bytes_freed = 0
 * 
 *         if not self._arena_mutex.try_lock():             # <<<<<<<<<<<<<<
 *             with nogil:
 *                 self._arena_mutex.lock()
*/
  }

  /* "cupy/cuda/memory.pyx":1411
 *             with nogil:
 *                 self._arena_mutex.lock()
 *         try:             # <<<<<<<<<<<<<<
 *             if stream is None:
 *                 idents = tuple(self._arenas.keys())
*/
  /*try:*/ {

    /* "cupy/cuda/memory.pyx":1412
 *                 self._arena_mutex.lock()
 *         try:
 *             if stream is None:             # <<<<<<<<<<<<<<
 *                 idents = tuple(self._arenas.keys())
 *             else:
*/
    __pyx_t_1 = (__pyx_v_stream == Py_None);
    if (__pyx_t_1) {

      /* "cupy/cuda/memory.pyx":1413
 *         try:
 *             if stream is None:
 *                 idents = tuple(self._arenas.keys())             # <<<<<<<<<<<<<<
 *             else:
 *                 stream_ident = _get_stream_identifier(stream.ptr)
*/
      if (unlikely(__pyx_v_self->_arenas == Py_None)) {
        PyErr_Format(PyExc_AttributeError, "'NoneType' object has no attribute '%.30s'", "keys");
        __PYX_ERR(0, 1413, __pyx_L8_error)
      }
      __pyx_t_2 = __Pyx_PyDict_Keys(__pyx_v_self->_arenas); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1413, __pyx_L8_error)
      __Pyx_GOTREF(__pyx_t_2);
      __pyx_t_3 = __Pyx_PySequence_Tuple(__pyx_t_2); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 1413, __pyx_L8_error)
      __Pyx_GOTREF(__pyx_t_3);
      __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
      __pyx_v_idents = ((PyObject*)__pyx_t_3);
      __pyx_t_3 = 0;

      /* "cupy/cuda/memory.pyx":1412
 *                 self._arena_mutex.lock()
 *         try:
 *             if stream is None:             # <<<<<<<<<<<<<<
 *                 idents = tuple(self._arenas.keys())
 *             else:
*/
      goto __pyx_L10;
    }

    /* "cupy/cuda/memory.pyx":1415
 *                 idents = tuple(self._arenas.keys())
 *             else:
 *                 stream_ident = _get_stream_identifier(stream.ptr)             # <<<<<<<<<<<<<<
 *                 if stream_ident not in self._arenas:
 *                     return  # stream doesn't exist, just return
*/
    /*else*/ {
      __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_v_stream, __pyx_mstate_global->__pyx_n_u_ptr); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 1415, __pyx_L8_error)
      __Pyx_GOTREF(__pyx_t_3);
      __pyx_t_4 = PyLong_AsSsize_t(__pyx_t_3); if (unlikely((__pyx_t_4 == ((intptr_t)-1)) && PyErr_Occurred())) __PYX_ERR(0, 1415, __pyx_L8_error)
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
      __pyx_t_5 = __pyx_f_4cupy_4cuda_6memory__get_stream_identifier(__pyx_t_4); if (unlikely(__pyx_t_5 == ((intptr_t)-1L) && PyErr_Occurred())) __PYX_ERR(0, 1415, __pyx_L8_error)
      __pyx_v_stream_ident = __pyx_t_5;

      /* "cupy/cuda/memory.pyx":1416
 *             else:
 *                 stream_ident = _get_stream_identifier(stream.ptr)
 *                 if stream_ident not in self._arenas:             # <<<<<<<<<<<<<<
 *                     return  # stream doesn't exist, just return
 *                 idents = (stream_ident,)
*/
      __pyx_t_3 = PyLong_FromSsize_t(__pyx_v_stream_ident); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 1416, __pyx_L8_error)
      __Pyx_GOTREF(__pyx_t_3);
      if (unlikely(__pyx_v_self->_arenas == Py_None)) {
        PyErr_SetString(PyExc_TypeError, "'NoneType' object is not iterable");
        __PYX_ERR(0, 1416, __pyx_L8_error)
      }
      __pyx_t_1 = (__Pyx_PyDict_ContainsTF(__pyx_t_3, __pyx_v_self->_arenas, Py_NE)); if (unlikely((__pyx_t_1 < 0))) __PYX_ERR(0, 1416, __pyx_L8_error)
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
      if (__pyx_t_1) {

        /* "cupy/cuda/memory.pyx":1417
 *                 stream_ident = _get_stream_identifier(stream.ptr)
 *                 if stream_ident not in self._arenas:
 *                     return  # stream doesn't exist, just return             # <<<<<<<<<<<<<<
 *                 idents = (stream_ident,)
 * 
*/
        __Pyx_XDECREF(__pyx_r);
        __pyx_r = Py_None; __Pyx_INCREF(Py_None);
        goto __pyx_L7_return;

        /* "cupy/cuda/memory.pyx":1416
 *             else:
 *                 stream_ident = _get_stream_identifier(stream.ptr)
 *                 if stream_ident not in self._arenas:             # <<<<<<<<<<<<<<
 *                     return  # stream doesn't exist, just return
 *                 idents = (stream_ident,)
*/
      }

      /* "cupy/cuda/memory.pyx":1418
 *                 if stream_ident not in self._arenas:
 *                     return  # stream doesn't exist, just return
 *                 idents = (stream_ident,)             # <<<<<<<<<<<<<<
 * 
 *             for ident in idents:
*/
      __pyx_t_3 = PyLong_FromSsize_t(__pyx_v_stream_ident); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 1418, __pyx_L8_error)
      __Pyx_GOTREF(__pyx_t_3);
      __pyx_t_2 = PyTuple_New(1); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1418, __pyx_L8_error)
      __Pyx_GOTREF(__pyx_t_2);
      __Pyx_GIVEREF(__pyx_t_3);
      if (__Pyx_PyTuple_SET_ITEM(__pyx_t_2, 0, __pyx_t_3) != (0)) __PYX_ERR(0, 1418, __pyx_L8_error);
      __pyx_t_3 = 0;
      __pyx_v_idents = ((PyObject*)__pyx_t_2);
      __pyx_t_2 = 0;
    }
    __pyx_L10:;

    /* "cupy/cuda/memory.pyx":1420
 *                 idents = (stream_ident,)
 * 
 *             for ident in idents:             # <<<<<<<<<<<<<<
 *                 arena = self._arenas[ident]()
 *                 if arena is None:
*/
    __pyx_t_2 = __pyx_v_idents; __Pyx_INCREF(__pyx_t_2);
    __pyx_t_6 = 0;
    for (;;) {
      {
        Py_ssize_t __pyx_temp = __Pyx_PyTuple_GET_SIZE(__pyx_t_2);
        #if !CYTHON_ASSUME_SAFE_SIZE
        if (unlikely((__pyx_temp < 0))) __PYX_ERR(0, 1420, __pyx_L8_error)
        #endif
        if (__pyx_t_6 >= __pyx_temp) break;
      }
      #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
      __pyx_t_3 = __Pyx_NewRef(PyTuple_GET_ITEM(__pyx_t_2, __pyx_t_6));
      #else
      __pyx_t_3 = __Pyx_PySequence_ITEM(__pyx_t_2, __pyx_t_6);
      #endif
      ++__pyx_t_6;
      if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 1420, __pyx_L8_error)
      __Pyx_GOTREF(__pyx_t_3);
      __Pyx_XDECREF_SET(__pyx_v_ident, __pyx_t_3);
      __pyx_t_3 = 0;

      /* "cupy/cuda/memory.pyx":1421
 * 
 *             for ident in idents:
 *                 arena = self._arenas[ident]()             # <<<<<<<<<<<<<<
 *                 if arena is None:
 *                     del self._arenas[ident]
*/
      __pyx_t_7 = NULL;
      if (unlikely(__pyx_v_self->_arenas == Py_None)) {
        PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
        __PYX_ERR(0, 1421, __pyx_L8_error)
      }
      __pyx_t_8 = __Pyx_PyDict_GetItem(__pyx_v_self->_arenas, __pyx_v_ident); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 1421, __pyx_L8_error)
      __Pyx_GOTREF(__pyx_t_8);
      __pyx_t_9 = 1;
      #if CYTHON_UNPACK_METHODS
      if (unlikely(PyMethod_Check(__pyx_t_8))) {
        __pyx_t_7 = PyMethod_GET_SELF(__pyx_t_8);
        assert(__pyx_t_7);
        PyObject* __pyx__function = PyMethod_GET_FUNCTION(__pyx_t_8);
        __Pyx_INCREF(__pyx_t_7);
        __Pyx_INCREF(__pyx__function);
        __Pyx_DECREF_SET(__pyx_t_8, __pyx__function);
        __pyx_t_9 = 0;
      }
      #endif
      {
        PyObject *__pyx_callargs[2] = {__pyx_t_7, NULL};
        __pyx_t_3 = __Pyx_PyObject_FastCall((PyObject*)__pyx_t_8, __pyx_callargs+__pyx_t_9, (1-__pyx_t_9) | (__pyx_t_9*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
        __Pyx_XDECREF(__pyx_t_7); __pyx_t_7 = 0;
        __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
        if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 1421, __pyx_L8_error)
        __Pyx_GOTREF(__pyx_t_3);
      }
      if (!(likely(((__pyx_t_3) == Py_None) || likely(__Pyx_TypeTest(__pyx_t_3, __pyx_mstate_global->__pyx_ptype_4cupy_4cuda_6memory__Arena))))) __PYX_ERR(0, 1421, __pyx_L8_error)
      __Pyx_XDECREF_SET(__pyx_v_arena, ((struct __pyx_obj_4cupy_4cuda_6memory__Arena *)__pyx_t_3));
      __pyx_t_3 = 0;

      /* "cupy/cuda/memory.pyx":1422
 *             for ident in idents:
 *                 arena = self._arenas[ident]()
 *                 if arena is None:             # <<<<<<<<<<<<<<
 *                     del self._arenas[ident]
 *                 else:
*/
      __pyx_t_1 = (((PyObject *)__pyx_v_arena) == Py_None);
      if (__pyx_t_1) {

        /* "cupy/cuda/memory.pyx":1423
 *                 arena = self._arenas[ident]()
 *                 if arena is None:
 *                     del self._arenas[ident]             # <<<<<<<<<<<<<<
 *                 else:
 *                     bytes_freed += arena.free_all()
*/
        if (unlikely(__pyx_v_self->_arenas == Py_None)) {
          PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
          __PYX_ERR(0, 1423, __pyx_L8_error)
        }
        if (unlikely((PyDict_DelItem(__pyx_v_self->_arenas, __pyx_v_ident) < 0))) __PYX_ERR(0, 1423, __pyx_L8_error)

        /* "cupy/cuda/memory.pyx":1422
 *             for ident in idents:
 *                 arena = self._arenas[ident]()
 *                 if arena is None:             # <<<<<<<<<<<<<<
 *                     del self._arenas[ident]
 *                 else:
*/
        goto __pyx_L14;
      }

      /* "cupy/cuda/memory.pyx":1425
 *                     del self._arenas[ident]
 *                 else:
 *                     bytes_freed += arena.free_all()             # <<<<<<<<<<<<<<
 *         finally:
 *             self._arena_mutex.unlock()
*/
      /*else*/ {
        __pyx_t_9 = __pyx_f_4cupy_4cuda_6memory_6_Arena_free_all(__pyx_v_arena); if (unlikely(__pyx_t_9 == ((size_t)-1L))) __PYX_ERR(0, 1425, __pyx_L8_error)
        __pyx_v_bytes_freed = (__pyx_v_bytes_freed + __pyx_t_9);
      }
      __pyx_L14:;

      /* "cupy/cuda/memory.pyx":1420
 *                 idents = (stream_ident,)
 * 
 *             for ident in idents:             # <<<<<<<<<<<<<<
 *                 arena = self._arenas[ident]()
 *                 if arena is None:
*/
    }
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  }

  /* "cupy/cuda/memory.pyx":1427
 *                     bytes_freed += arena.free_all()
 *         finally:
 *             self._arena_mutex.unlock()             # <<<<<<<<<<<<<<
 *             self._total_bytes -= bytes_freed
 * 
*/
  /*finally:*/ {
    /*normal exit:*/{
      __pyx_v_self->_arena_mutex.unlock();

      /* "cupy/cuda/memory.pyx":1428
 *         finally:
 *             self._arena_mutex.unlock()
 *             self._total_bytes -= bytes_freed             # <<<<<<<<<<<<<<
 * 
 *     cpdef free_all_free(self):
*/
      __pyx_v_self->_total_bytes -= __pyx_v_bytes_freed;
      goto __pyx_L9;
    }
    __pyx_L8_error:;
    /*exception exit:*/{
      __Pyx_PyThreadState_declare
      __Pyx_PyThreadState_assign
      __pyx_t_13 = 0; __pyx_t_14 = 0; __pyx_t_15 = 0; __pyx_t_16 = 0; __pyx_t_17 = 0; __pyx_t_18 = 0;
      __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
      __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
      __Pyx_XDECREF(__pyx_t_7); __pyx_t_7 = 0;
      __Pyx_XDECREF(__pyx_t_8); __pyx_t_8 = 0;
       __Pyx_ExceptionSwap(&__pyx_t_16, &__pyx_t_17, &__pyx_t_18);
      if ( unlikely(__Pyx_GetException(&__pyx_t_13, &__pyx_t_14, &__pyx_t_15) < 0)) __Pyx_ErrFetch(&__pyx_t_13, &__pyx_t_14, &__pyx_t_15);
      __Pyx_XGOTREF(__pyx_t_13);
      __Pyx_XGOTREF(__pyx_t_14);
      __Pyx_XGOTREF(__pyx_t_15);
      __Pyx_XGOTREF(__pyx_t_16);
      __Pyx_XGOTREF(__pyx_t_17);
      __Pyx_XGOTREF(__pyx_t_18);
      __pyx_t_10 = __pyx_lineno; __pyx_t_11 = __pyx_clineno; __pyx_t_12 = __pyx_filename;
      {

        /* "cupy/cuda/memory.pyx":1427
 *                     bytes_freed += arena.free_all()
 *         finally:
 *             self._arena_mutex.unlock()             # <<<<<<<<<<<<<<
 *             self._total_bytes -= bytes_freed
 * 
*/
        __pyx_v_self->_arena_mutex.unlock();

        /* "cupy/cuda/memory.pyx":1428
 *         finally:
 *             self._arena_mutex.unlock()
 *             self._total_bytes -= bytes_freed             # <<<<<<<<<<<<<<
 * 
 *     cpdef free_all_free(self):
*/
        __pyx_v_self->_total_bytes -= __pyx_v_bytes_freed;
      }
      __Pyx_XGIVEREF(__pyx_t_16);
      __Pyx_XGIVEREF(__pyx_t_17);
      __Pyx_XGIVEREF(__pyx_t_18);
      __Pyx_ExceptionReset(__pyx_t_16, __pyx_t_17, __pyx_t_18);
      __Pyx_XGIVEREF(__pyx_t_13);
      __Pyx_XGIVEREF(__pyx_t_14);
      __Pyx_XGIVEREF(__pyx_t_15);
      __Pyx_ErrRestore(__pyx_t_13, __pyx_t_14, __pyx_t_15);
      __pyx_t_13 = 0; __pyx_t_14 = 0; __pyx_t_15 = 0; __pyx_t_16 = 0; __pyx_t_17 = 0; __pyx_t_18 = 0;
      __pyx_lineno = __pyx_t_10; __pyx_clineno = __pyx_t_11; __pyx_filename = __pyx_t_12;
      goto __pyx_L1_error;
    }
    __pyx_L7_return: {
      __pyx_t_18 = __pyx_r;
      __pyx_r = 0;

      /* "cupy/cuda/memory.pyx":1427
 *                     bytes_freed += arena.free_all()
 *         finally:
 *             self._arena_mutex.unlock()             # <<<<<<<<<<<<<<
 *             self._total_bytes -= bytes_freed
 * 
*/
      __pyx_v_self->_arena_mutex.unlock();

      /* "cupy/cuda/memory.pyx":1428
 *         finally:
 *             self._arena_mutex.unlock()
 *             self._total_bytes -= bytes_freed             # <<<<<<<<<<<<<<
 * 
 *     cpdef free_all_free(self):
*/
      __pyx_v_self->_total_bytes -= __pyx_v_bytes_freed;
      __pyx_r = __pyx_t_18;
      __pyx_t_18 = 0;
      goto __pyx_L0;
    }
    __pyx_L9:;
  }

  /* "cupy/cuda/memory.pyx":1400
 *             chunk.arena.add_pending_free_atomic(chunk)
 * 
 *     cpdef free_all_blocks(self, stream=None):             # <<<<<<<<<<<<<<
 *         """Free all **non-split** blocks for one or all arenas.
 *         """
*/

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_7);
  __Pyx_XDECREF(__pyx_t_8);
  __Pyx_AddTraceback("cupy.cuda.memory.SingleDeviceMemoryPool.free_all_blocks", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XDECREF((PyObject *)__pyx_v_arena);
  __Pyx_XDECREF(__pyx_v_idents);
  __Pyx_XDECREF(__pyx_v_ident);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_pw_4cupy_4cuda_6memory_22SingleDeviceMemoryPool_9free_all_blocks(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
PyDoc_STRVAR(__pyx_doc_4cupy_4cuda_6memory_22SingleDeviceMemoryPool_8free_all_blocks, "SingleDeviceMemoryPool.free_all_blocks(self, stream=None)\n\nFree all **non-split** blocks for one or all arenas.\n        ");
static PyObject *__pyx_pw_4cupy_4cuda_6memory_22SingleDeviceMemoryPool_9free_all_blocks(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
) {
  PyObject *__pyx_v_stream = 0;
  #if !CYTHON_METH_FASTCALL
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  #endif
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject* values[1] = {0};
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("free_all_blocks (wrapper)", 0);
  #if !CYTHON_METH_FASTCALL
  #if CYTHON_ASSUME_SAFE_SIZE
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return NULL;
  #endif
  #endif
  __pyx_kwvalues = __Pyx_KwValues_FASTCALL(__pyx_args, __pyx_nargs);
  {
    PyObject ** const __pyx_pyargnames[] = {&__pyx_mstate_global->__pyx_n_u_stream,0};
    const Py_ssize_t __pyx_kwds_len = (__pyx_kwds) ? __Pyx_NumKwargs_FASTCALL(__pyx_kwds) : 0;
    if (unlikely(__pyx_kwds_len < 0)) __PYX_ERR(0, 1400, __pyx_L3_error)
    if (__pyx_kwds_len > 0) {
      switch (__pyx_nargs) {
        case  1:
        values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(0, 1400, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      const Py_ssize_t kwd_pos_args = __pyx_nargs;
      if (__Pyx_ParseKeywords(__pyx_kwds, __pyx_kwvalues, __pyx_pyargnames, 0, values, kwd_pos_args, __pyx_kwds_len, "free_all_blocks", 0) < (0)) __PYX_ERR(0, 1400, __pyx_L3_error)
      if (!values[0]) values[0] = __Pyx_NewRef(((PyObject *)Py_None));
    } else {
      switch (__pyx_nargs) {
        case  1:
        values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(0, 1400, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      if (!values[0]) values[0] = __Pyx_NewRef(((PyObject *)Py_None));
    }
    __pyx_v_stream = values[0];
  }
  goto __pyx_L6_skip;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("free_all_blocks", 0, 0, 1, __pyx_nargs); __PYX_ERR(0, 1400, __pyx_L3_error)
  __pyx_L6_skip:;
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_AddTraceback("cupy.cuda.memory.SingleDeviceMemoryPool.free_all_blocks", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_4cupy_4cuda_6memory_22SingleDeviceMemoryPool_8free_all_blocks(((struct __pyx_obj_4cupy_4cuda_6memory_SingleDeviceMemoryPool *)__pyx_v_self), __pyx_v_stream);

  /* function exit code */
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4cupy_4cuda_6memory_22SingleDeviceMemoryPool_8free_all_blocks(struct __pyx_obj_4cupy_4cuda_6memory_SingleDeviceMemoryPool *__pyx_v_self, PyObject *__pyx_v_stream) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  struct __pyx_opt_args_4cupy_4cuda_6memory_22SingleDeviceMemoryPool_free_all_blocks __pyx_t_2;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("free_all_blocks", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_2.__pyx_n = 1;
  __pyx_t_2.stream = __pyx_v_stream;
  __pyx_t_1 = __pyx_vtabptr_4cupy_4cuda_6memory_SingleDeviceMemoryPool->free_all_blocks(__pyx_v_self, 1, &__pyx_t_2); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1400, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("cupy.cuda.memory.SingleDeviceMemoryPool.free_all_blocks", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "cupy/cuda/memory.pyx":1430
 *             self._total_bytes -= bytes_freed
 * 
 *     cpdef free_all_free(self):             # <<<<<<<<<<<<<<
 *         warnings.warn(
 *             'free_all_free is deprecated. Use free_all_blocks instead.',
*/

static PyObject *__pyx_pw_4cupy_4cuda_6memory_22SingleDeviceMemoryPool_11free_all_free(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
static PyObject *__pyx_f_4cupy_4cuda_6memory_22SingleDeviceMemoryPool_free_all_free(struct __pyx_obj_4cupy_4cuda_6memory_SingleDeviceMemoryPool *__pyx_v_self, CYTHON_UNUSED int __pyx_skip_dispatch) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  size_t __pyx_t_5;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("free_all_free", 0);

  /* "cupy/cuda/memory.pyx":1431
 * 
 *     cpdef free_all_free(self):
 *         warnings.warn(             # <<<<<<<<<<<<<<
 *             'free_all_free is deprecated. Use free_all_blocks instead.',
 *             DeprecationWarning)
*/
  __pyx_t_2 = NULL;
  __Pyx_GetModuleGlobalName(__pyx_t_3, __pyx_mstate_global->__pyx_n_u_warnings); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 1431, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_4 = __Pyx_PyObject_GetAttrStr(__pyx_t_3, __pyx_mstate_global->__pyx_n_u_warn); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 1431, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "cupy/cuda/memory.pyx":1433
 *         warnings.warn(
 *             'free_all_free is deprecated. Use free_all_blocks instead.',
 *             DeprecationWarning)             # <<<<<<<<<<<<<<
 *         self.free_all_blocks()
 * 
*/
  __pyx_t_5 = 1;
  #if CYTHON_UNPACK_METHODS
  if (unlikely(PyMethod_Check(__pyx_t_4))) {
    __pyx_t_2 = PyMethod_GET_SELF(__pyx_t_4);
    assert(__pyx_t_2);
    PyObject* __pyx__function = PyMethod_GET_FUNCTION(__pyx_t_4);
    __Pyx_INCREF(__pyx_t_2);
    __Pyx_INCREF(__pyx__function);
    __Pyx_DECREF_SET(__pyx_t_4, __pyx__function);
    __pyx_t_5 = 0;
  }
  #endif
  {
    PyObject *__pyx_callargs[3] = {__pyx_t_2, __pyx_mstate_global->__pyx_kp_u_free_all_free_is_deprecated_Use, ((PyObject *)(((PyTypeObject*)PyExc_DeprecationWarning)))};
    __pyx_t_1 = __Pyx_PyObject_FastCall((PyObject*)__pyx_t_4, __pyx_callargs+__pyx_t_5, (3-__pyx_t_5) | (__pyx_t_5*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
    __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1431, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
  }
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "cupy/cuda/memory.pyx":1434
 *             'free_all_free is deprecated. Use free_all_blocks instead.',
 *             DeprecationWarning)
 *         self.free_all_blocks()             # <<<<<<<<<<<<<<
 * 
 *     cpdef size_t n_free_blocks(self) except -1:
*/
  __pyx_t_1 = __pyx_f_4cupy_4cuda_6memory_22SingleDeviceMemoryPool_free_all_blocks(__pyx_v_self, 0, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1434, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "cupy/cuda/memory.pyx":1430
 *             self._total_bytes -= bytes_freed
 * 
 *     cpdef free_all_free(self):             # <<<<<<<<<<<<<<
 *         warnings.warn(
 *             'free_all_free is deprecated. Use free_all_blocks instead.',
*/

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_AddTraceback("cupy.cuda.memory.SingleDeviceMemoryPool.free_all_free", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_pw_4cupy_4cuda_6memory_22SingleDeviceMemoryPool_11free_all_free(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
PyDoc_STRVAR(__pyx_doc_4cupy_4cuda_6memory_22SingleDeviceMemoryPool_10free_all_free, "SingleDeviceMemoryPool.free_all_free(self)");
static PyObject *__pyx_pw_4cupy_4cuda_6memory_22SingleDeviceMemoryPool_11free_all_free(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
) {
  #if !CYTHON_METH_FASTCALL
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  #endif
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("free_all_free (wrapper)", 0);
  #if !CYTHON_METH_FASTCALL
  #if CYTHON_ASSUME_SAFE_SIZE
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return NULL;
  #endif
  #endif
  __pyx_kwvalues = __Pyx_KwValues_FASTCALL(__pyx_args, __pyx_nargs);
  if (unlikely(__pyx_nargs > 0)) { __Pyx_RaiseArgtupleInvalid("free_all_free", 1, 0, 0, __pyx_nargs); return NULL; }
  const Py_ssize_t __pyx_kwds_len = unlikely(__pyx_kwds) ? __Pyx_NumKwargs_FASTCALL(__pyx_kwds) : 0;
  if (unlikely(__pyx_kwds_len < 0)) return NULL;
  if (unlikely(__pyx_kwds_len > 0)) {__Pyx_RejectKeywords("free_all_free", __pyx_kwds); return NULL;}
  __pyx_r = __pyx_pf_4cupy_4cuda_6memory_22SingleDeviceMemoryPool_10free_all_free(((struct __pyx_obj_4cupy_4cuda_6memory_SingleDeviceMemoryPool *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4cupy_4cuda_6memory_22SingleDeviceMemoryPool_10free_all_free(struct __pyx_obj_4cupy_4cuda_6memory_SingleDeviceMemoryPool *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("free_all_free", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __pyx_f_4cupy_4cuda_6memory_22SingleDeviceMemoryPool_free_all_free(__pyx_v_self, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1430, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("cupy.cuda.memory.SingleDeviceMemoryPool.free_all_free", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "cupy/cuda/memory.pyx":1436
 *         self.free_all_blocks()
 * 
 *     cpdef size_t n_free_blocks(self) except -1:             # <<<<<<<<<<<<<<
 *         cdef size_t n = 0
 *         cdef _Arena arena
*/

static PyObject *__pyx_pw_4cupy_4cuda_6memory_22SingleDeviceMemoryPool_13n_free_blocks(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
static size_t __pyx_f_4cupy_4cuda_6memory_22SingleDeviceMemoryPool_n_free_blocks(struct __pyx_obj_4cupy_4cuda_6memory_SingleDeviceMemoryPool *__pyx_v_self, CYTHON_UNUSED int __pyx_skip_dispatch) {
  size_t __pyx_v_n;
  struct __pyx_obj_4cupy_4cuda_6memory__Arena *__pyx_v_arena = 0;
  PyObject *__pyx_v_ref = NULL;
  size_t __pyx_r;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  Py_ssize_t __pyx_t_3;
  Py_ssize_t __pyx_t_4;
  int __pyx_t_5;
  PyObject *__pyx_t_6 = NULL;
  int __pyx_t_7;
  PyObject *__pyx_t_8 = NULL;
  PyObject *__pyx_t_9 = NULL;
  size_t __pyx_t_10;
  char const *__pyx_t_11;
  PyObject *__pyx_t_12 = NULL;
  PyObject *__pyx_t_13 = NULL;
  PyObject *__pyx_t_14 = NULL;
  PyObject *__pyx_t_15 = NULL;
  PyObject *__pyx_t_16 = NULL;
  PyObject *__pyx_t_17 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("n_free_blocks", 0);

  /* "cupy/cuda/memory.pyx":1437
 * 
 *     cpdef size_t n_free_blocks(self) except -1:
 *         cdef size_t n = 0             # <<<<<<<<<<<<<<
 *         cdef _Arena arena
 * 
*/
  __pyx_v_n = 0;

  /* "cupy/cuda/memory.pyx":1440
 *         cdef _Arena arena
 * 
 *         if not self._arena_mutex.try_lock():             # <<<<<<<<<<<<<<
 *             with nogil:
 *                 self._arena_mutex.lock()
*/
  __pyx_t_1 = (!(__pyx_v_self->_arena_mutex.try_lock() != 0));
  if (__pyx_t_1) {

    /* "cupy/cuda/memory.pyx":1441
 * 
 *         if not self._arena_mutex.try_lock():
 *             with nogil:             # <<<<<<<<<<<<<<
 *                 self._arena_mutex.lock()
 *         try:
*/
    {
        PyThreadState * _save;
        _save = PyEval_SaveThread();
        __Pyx_FastGIL_Remember();
        /*try:*/ {

          /* "cupy/cuda/memory.pyx":1442
 *         if not self._arena_mutex.try_lock():
 *             with nogil:
 *                 self._arena_mutex.lock()             # <<<<<<<<<<<<<<
 *         try:
 *             for ref in self._arenas.itervalues():
*/
          try {
            __pyx_v_self->_arena_mutex.lock();
          } catch(...) {
            PyGILState_STATE __pyx_gilstate_save = __Pyx_PyGILState_Ensure();
            __Pyx_CppExn2PyErr();
            __Pyx_PyGILState_Release(__pyx_gilstate_save);
            __PYX_ERR(0, 1442, __pyx_L5_error)
          }
        }

        /* "cupy/cuda/memory.pyx":1441
 * 
 *         if not self._arena_mutex.try_lock():
 *             with nogil:             # <<<<<<<<<<<<<<
 *                 self._arena_mutex.lock()
 *         try:
*/
        /*finally:*/ {
          /*normal exit:*/{
            __Pyx_FastGIL_Forget();
            PyEval_RestoreThread(_save);
            goto __pyx_L6;
          }
          __pyx_L5_error: {
            __Pyx_FastGIL_Forget();
            PyEval_RestoreThread(_save);
            goto __pyx_L1_error;
          }
          __pyx_L6:;
        }
    }

    /* "cupy/cuda/memory.pyx":1440
 *         cdef _Arena arena
 * 
 *         if not self._arena_mutex.try_lock():             # <<<<<<<<<<<<<<
 *             with nogil:
 *                 self._arena_mutex.lock()
*/
  }

  /* "cupy/cuda/memory.pyx":1443
 *             with nogil:
 *                 self._arena_mutex.lock()
 *         try:             # <<<<<<<<<<<<<<
 *             for ref in self._arenas.itervalues():
 *                 arena = ref()
*/
  /*try:*/ {

    /* "cupy/cuda/memory.pyx":1444
 *                 self._arena_mutex.lock()
 *         try:
 *             for ref in self._arenas.itervalues():             # <<<<<<<<<<<<<<
 *                 arena = ref()
 *                 if arena is None:
*/
    __pyx_t_3 = 0;
    if (unlikely(__pyx_v_self->_arenas == Py_None)) {
      PyErr_Format(PyExc_AttributeError, "'NoneType' object has no attribute '%.30s'", "itervalues");
      __PYX_ERR(0, 1444, __pyx_L8_error)
    }
    __pyx_t_6 = __Pyx_dict_iterator(__pyx_v_self->_arenas, 1, __pyx_mstate_global->__pyx_n_u_itervalues, (&__pyx_t_4), (&__pyx_t_5)); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 1444, __pyx_L8_error)
    __Pyx_GOTREF(__pyx_t_6);
    __Pyx_XDECREF(__pyx_t_2);
    __pyx_t_2 = __pyx_t_6;
    __pyx_t_6 = 0;
    while (1) {
      __pyx_t_7 = __Pyx_dict_iter_next(__pyx_t_2, __pyx_t_4, &__pyx_t_3, NULL, &__pyx_t_6, NULL, __pyx_t_5);
      if (unlikely(__pyx_t_7 == 0)) break;
      if (unlikely(__pyx_t_7 == -1)) __PYX_ERR(0, 1444, __pyx_L8_error)
      __Pyx_GOTREF(__pyx_t_6);
      __Pyx_XDECREF_SET(__pyx_v_ref, __pyx_t_6);
      __pyx_t_6 = 0;

      /* "cupy/cuda/memory.pyx":1445
 *         try:
 *             for ref in self._arenas.itervalues():
 *                 arena = ref()             # <<<<<<<<<<<<<<
 *                 if arena is None:
 *                     continue
*/
      __pyx_t_8 = NULL;
      __Pyx_INCREF(__pyx_v_ref);
      __pyx_t_9 = __pyx_v_ref; 
      __pyx_t_10 = 1;
      #if CYTHON_UNPACK_METHODS
      if (unlikely(PyMethod_Check(__pyx_t_9))) {
        __pyx_t_8 = PyMethod_GET_SELF(__pyx_t_9);
        assert(__pyx_t_8);
        PyObject* __pyx__function = PyMethod_GET_FUNCTION(__pyx_t_9);
        __Pyx_INCREF(__pyx_t_8);
        __Pyx_INCREF(__pyx__function);
        __Pyx_DECREF_SET(__pyx_t_9, __pyx__function);
        __pyx_t_10 = 0;
      }
      #endif
      {
        PyObject *__pyx_callargs[2] = {__pyx_t_8, NULL};
        __pyx_t_6 = __Pyx_PyObject_FastCall((PyObject*)__pyx_t_9, __pyx_callargs+__pyx_t_10, (1-__pyx_t_10) | (__pyx_t_10*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
        __Pyx_XDECREF(__pyx_t_8); __pyx_t_8 = 0;
        __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
        if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 1445, __pyx_L8_error)
        __Pyx_GOTREF(__pyx_t_6);
      }
      if (!(likely(((__pyx_t_6) == Py_None) || likely(__Pyx_TypeTest(__pyx_t_6, __pyx_mstate_global->__pyx_ptype_4cupy_4cuda_6memory__Arena))))) __PYX_ERR(0, 1445, __pyx_L8_error)
      __Pyx_XDECREF_SET(__pyx_v_arena, ((struct __pyx_obj_4cupy_4cuda_6memory__Arena *)__pyx_t_6));
      __pyx_t_6 = 0;

      /* "cupy/cuda/memory.pyx":1446
 *             for ref in self._arenas.itervalues():
 *                 arena = ref()
 *                 if arena is None:             # <<<<<<<<<<<<<<
 *                     continue
 *                 n += arena._get_size()
*/
      __pyx_t_1 = (((PyObject *)__pyx_v_arena) == Py_None);
      if (__pyx_t_1) {

        /* "cupy/cuda/memory.pyx":1447
 *                 arena = ref()
 *                 if arena is None:
 *                     continue             # <<<<<<<<<<<<<<
 *                 n += arena._get_size()
 *         finally:
*/
        goto __pyx_L10_continue;

        /* "cupy/cuda/memory.pyx":1446
 *             for ref in self._arenas.itervalues():
 *                 arena = ref()
 *                 if arena is None:             # <<<<<<<<<<<<<<
 *                     continue
 *                 n += arena._get_size()
*/
      }

      /* "cupy/cuda/memory.pyx":1448
 *                 if arena is None:
 *                     continue
 *                 n += arena._get_size()             # <<<<<<<<<<<<<<
 *         finally:
 *             self._arena_mutex.unlock()
*/
      __pyx_t_6 = __Pyx_PyLong_FromSize_t(__pyx_v_n); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 1448, __pyx_L8_error)
      __Pyx_GOTREF(__pyx_t_6);
      __pyx_t_9 = __pyx_f_4cupy_4cuda_6memory_6_Arena__get_size(__pyx_v_arena); if (unlikely(!__pyx_t_9)) __PYX_ERR(0, 1448, __pyx_L8_error)
      __Pyx_GOTREF(__pyx_t_9);
      __pyx_t_8 = PyNumber_InPlaceAdd(__pyx_t_6, __pyx_t_9); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 1448, __pyx_L8_error)
      __Pyx_GOTREF(__pyx_t_8);
      __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
      __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
      __pyx_t_10 = __Pyx_PyLong_As_size_t(__pyx_t_8); if (unlikely((__pyx_t_10 == (size_t)-1) && PyErr_Occurred())) __PYX_ERR(0, 1448, __pyx_L8_error)
      __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
      __pyx_v_n = __pyx_t_10;
      __pyx_L10_continue:;
    }
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  }

  /* "cupy/cuda/memory.pyx":1450
 *                 n += arena._get_size()
 *         finally:
 *             self._arena_mutex.unlock()             # <<<<<<<<<<<<<<
 * 
 *         return n
*/
  /*finally:*/ {
    /*normal exit:*/{
      __pyx_v_self->_arena_mutex.unlock();
      goto __pyx_L9;
    }
    __pyx_L8_error:;
    /*exception exit:*/{
      __Pyx_PyThreadState_declare
      __Pyx_PyThreadState_assign
      __pyx_t_12 = 0; __pyx_t_13 = 0; __pyx_t_14 = 0; __pyx_t_15 = 0; __pyx_t_16 = 0; __pyx_t_17 = 0;
      __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
      __Pyx_XDECREF(__pyx_t_6); __pyx_t_6 = 0;
      __Pyx_XDECREF(__pyx_t_8); __pyx_t_8 = 0;
      __Pyx_XDECREF(__pyx_t_9); __pyx_t_9 = 0;
       __Pyx_ExceptionSwap(&__pyx_t_15, &__pyx_t_16, &__pyx_t_17);
      if ( unlikely(__Pyx_GetException(&__pyx_t_12, &__pyx_t_13, &__pyx_t_14) < 0)) __Pyx_ErrFetch(&__pyx_t_12, &__pyx_t_13, &__pyx_t_14);
      __Pyx_XGOTREF(__pyx_t_12);
      __Pyx_XGOTREF(__pyx_t_13);
      __Pyx_XGOTREF(__pyx_t_14);
      __Pyx_XGOTREF(__pyx_t_15);
      __Pyx_XGOTREF(__pyx_t_16);
      __Pyx_XGOTREF(__pyx_t_17);
      __pyx_t_5 = __pyx_lineno; __pyx_t_7 = __pyx_clineno; __pyx_t_11 = __pyx_filename;
      {
        __pyx_v_self->_arena_mutex.unlock();
      }
      __Pyx_XGIVEREF(__pyx_t_15);
      __Pyx_XGIVEREF(__pyx_t_16);
      __Pyx_XGIVEREF(__pyx_t_17);
      __Pyx_ExceptionReset(__pyx_t_15, __pyx_t_16, __pyx_t_17);
      __Pyx_XGIVEREF(__pyx_t_12);
      __Pyx_XGIVEREF(__pyx_t_13);
      __Pyx_XGIVEREF(__pyx_t_14);
      __Pyx_ErrRestore(__pyx_t_12, __pyx_t_13, __pyx_t_14);
      __pyx_t_12 = 0; __pyx_t_13 = 0; __pyx_t_14 = 0; __pyx_t_15 = 0; __pyx_t_16 = 0; __pyx_t_17 = 0;
      __pyx_lineno = __pyx_t_5; __pyx_clineno = __pyx_t_7; __pyx_filename = __pyx_t_11;
      goto __pyx_L1_error;
    }
    __pyx_L9:;
  }

  /* "cupy/cuda/memory.pyx":1452
 *             self._arena_mutex.unlock()
 * 
 *         return n             # <<<<<<<<<<<<<<
 * 
 *     cpdef size_t used_bytes(self):
*/
  __pyx_r = __pyx_v_n;
  goto __pyx_L0;

  /* "cupy/cuda/memory.pyx":1436
 *         self.free_all_blocks()
 * 
 *     cpdef size_t n_free_blocks(self) except -1:             # <<<<<<<<<<<<<<
 *         cdef size_t n = 0
 *         cdef _Arena arena
*/

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_XDECREF(__pyx_t_8);
  __Pyx_XDECREF(__pyx_t_9);
  __Pyx_AddTraceback("cupy.cuda.memory.SingleDeviceMemoryPool.n_free_blocks", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1L;
  __pyx_L0:;
  __Pyx_XDECREF((PyObject *)__pyx_v_arena);
  __Pyx_XDECREF(__pyx_v_ref);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_pw_4cupy_4cuda_6memory_22SingleDeviceMemoryPool_13n_free_blocks(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
PyDoc_STRVAR(__pyx_doc_4cupy_4cuda_6memory_22SingleDeviceMemoryPool_12n_free_blocks, "SingleDeviceMemoryPool.n_free_blocks(self) -> size_t");
static PyObject *__pyx_pw_4cupy_4cuda_6memory_22SingleDeviceMemoryPool_13n_free_blocks(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
) {
  #if !CYTHON_METH_FASTCALL
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  #endif
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("n_free_blocks (wrapper)", 0);
  #if !CYTHON_METH_FASTCALL
  #if CYTHON_ASSUME_SAFE_SIZE
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return NULL;
  #endif
  #endif
  __pyx_kwvalues = __Pyx_KwValues_FASTCALL(__pyx_args, __pyx_nargs);
  if (unlikely(__pyx_nargs > 0)) { __Pyx_RaiseArgtupleInvalid("n_free_blocks", 1, 0, 0, __pyx_nargs); return NULL; }
  const Py_ssize_t __pyx_kwds_len = unlikely(__pyx_kwds) ? __Pyx_NumKwargs_FASTCALL(__pyx_kwds) : 0;
  if (unlikely(__pyx_kwds_len < 0)) return NULL;
  if (unlikely(__pyx_kwds_len > 0)) {__Pyx_RejectKeywords("n_free_blocks", __pyx_kwds); return NULL;}
  __pyx_r = __pyx_pf_4cupy_4cuda_6memory_22SingleDeviceMemoryPool_12n_free_blocks(((struct __pyx_obj_4cupy_4cuda_6memory_SingleDeviceMemoryPool *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4cupy_4cuda_6memory_22SingleDeviceMemoryPool_12n_free_blocks(struct __pyx_obj_4cupy_4cuda_6memory_SingleDeviceMemoryPool *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  size_t __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("n_free_blocks", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __pyx_f_4cupy_4cuda_6memory_22SingleDeviceMemoryPool_n_free_blocks(__pyx_v_self, 1); if (unlikely(__pyx_t_1 == ((size_t)-1L))) __PYX_ERR(0, 1436, __pyx_L1_error)
  __pyx_t_2 = __Pyx_PyLong_FromSize_t(__pyx_t_1); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1436, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_r = __pyx_t_2;
  __pyx_t_2 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_AddTraceback("cupy.cuda.memory.SingleDeviceMemoryPool.n_free_blocks", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "cupy/cuda/memory.pyx":1454
 *         return n
 * 
 *     cpdef size_t used_bytes(self):             # <<<<<<<<<<<<<<
 *         """Currently used bytes."""
 *         return self._in_use_bytes.load()
*/

static PyObject *__pyx_pw_4cupy_4cuda_6memory_22SingleDeviceMemoryPool_15used_bytes(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
static size_t __pyx_f_4cupy_4cuda_6memory_22SingleDeviceMemoryPool_used_bytes(struct __pyx_obj_4cupy_4cuda_6memory_SingleDeviceMemoryPool *__pyx_v_self, CYTHON_UNUSED int __pyx_skip_dispatch) {
  size_t __pyx_r;

  /* "cupy/cuda/memory.pyx":1456
 *     cpdef size_t used_bytes(self):
 *         """Currently used bytes."""
 *         return self._in_use_bytes.load()             # <<<<<<<<<<<<<<
 * 
 *     cpdef size_t free_bytes(self):
*/
  __pyx_r = __pyx_v_self->_in_use_bytes.load();
  goto __pyx_L0;

  /* "cupy/cuda/memory.pyx":1454
 *         return n
 * 
 *     cpdef size_t used_bytes(self):             # <<<<<<<<<<<<<<
 *         """Currently used bytes."""
 *         return self._in_use_bytes.load()
*/

  /* function exit code */
  __pyx_L0:;
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_pw_4cupy_4cuda_6memory_22SingleDeviceMemoryPool_15used_bytes(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
PyDoc_STRVAR(__pyx_doc_4cupy_4cuda_6memory_22SingleDeviceMemoryPool_14used_bytes, "SingleDeviceMemoryPool.used_bytes(self) -> size_t\n\nCurrently used bytes.");
static PyObject *__pyx_pw_4cupy_4cuda_6memory_22SingleDeviceMemoryPool_15used_bytes(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
) {
  #if !CYTHON_METH_FASTCALL
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  #endif
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("used_bytes (wrapper)", 0);
  #if !CYTHON_METH_FASTCALL
  #if CYTHON_ASSUME_SAFE_SIZE
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return NULL;
  #endif
  #endif
  __pyx_kwvalues = __Pyx_KwValues_FASTCALL(__pyx_args, __pyx_nargs);
  if (unlikely(__pyx_nargs > 0)) { __Pyx_RaiseArgtupleInvalid("used_bytes", 1, 0, 0, __pyx_nargs); return NULL; }
  const Py_ssize_t __pyx_kwds_len = unlikely(__pyx_kwds) ? __Pyx_NumKwargs_FASTCALL(__pyx_kwds) : 0;
  if (unlikely(__pyx_kwds_len < 0)) return NULL;
  if (unlikely(__pyx_kwds_len > 0)) {__Pyx_RejectKeywords("used_bytes", __pyx_kwds); return NULL;}
  __pyx_r = __pyx_pf_4cupy_4cuda_6memory_22SingleDeviceMemoryPool_14used_bytes(((struct __pyx_obj_4cupy_4cuda_6memory_SingleDeviceMemoryPool *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4cupy_4cuda_6memory_22SingleDeviceMemoryPool_14used_bytes(struct __pyx_obj_4cupy_4cuda_6memory_SingleDeviceMemoryPool *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  size_t __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("used_bytes", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __pyx_f_4cupy_4cuda_6memory_22SingleDeviceMemoryPool_used_bytes(__pyx_v_self, 1); if (unlikely(PyErr_Occurred())) __PYX_ERR(0, 1454, __pyx_L1_error)
  __pyx_t_2 = __Pyx_PyLong_FromSize_t(__pyx_t_1); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1454, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_r = __pyx_t_2;
  __pyx_t_2 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_AddTraceback("cupy.cuda.memory.SingleDeviceMemoryPool.used_bytes", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "cupy/cuda/memory.pyx":1458
 *         return self._in_use_bytes.load()
 * 
 *     cpdef size_t free_bytes(self):             # <<<<<<<<<<<<<<
 *         return self._total_bytes.load() - self._in_use_bytes.load()
 * 
*/

static PyObject *__pyx_pw_4cupy_4cuda_6memory_22SingleDeviceMemoryPool_17free_bytes(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
static size_t __pyx_f_4cupy_4cuda_6memory_22SingleDeviceMemoryPool_free_bytes(struct __pyx_obj_4cupy_4cuda_6memory_SingleDeviceMemoryPool *__pyx_v_self, CYTHON_UNUSED int __pyx_skip_dispatch) {
  size_t __pyx_r;

  /* "cupy/cuda/memory.pyx":1459
 * 
 *     cpdef size_t free_bytes(self):
 *         return self._total_bytes.load() - self._in_use_bytes.load()             # <<<<<<<<<<<<<<
 * 
 *     cpdef size_t total_bytes(self):
*/
  __pyx_r = (__pyx_v_self->_total_bytes.load() - __pyx_v_self->_in_use_bytes.load());
  goto __pyx_L0;

  /* "cupy/cuda/memory.pyx":1458
 *         return self._in_use_bytes.load()
 * 
 *     cpdef size_t free_bytes(self):             # <<<<<<<<<<<<<<
 *         return self._total_bytes.load() - self._in_use_bytes.load()
 * 
*/

  /* function exit code */
  __pyx_L0:;
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_pw_4cupy_4cuda_6memory_22SingleDeviceMemoryPool_17free_bytes(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
PyDoc_STRVAR(__pyx_doc_4cupy_4cuda_6memory_22SingleDeviceMemoryPool_16free_bytes, "SingleDeviceMemoryPool.free_bytes(self) -> size_t");
static PyObject *__pyx_pw_4cupy_4cuda_6memory_22SingleDeviceMemoryPool_17free_bytes(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
) {
  #if !CYTHON_METH_FASTCALL
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  #endif
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("free_bytes (wrapper)", 0);
  #if !CYTHON_METH_FASTCALL
  #if CYTHON_ASSUME_SAFE_SIZE
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return NULL;
  #endif
  #endif
  __pyx_kwvalues = __Pyx_KwValues_FASTCALL(__pyx_args, __pyx_nargs);
  if (unlikely(__pyx_nargs > 0)) { __Pyx_RaiseArgtupleInvalid("free_bytes", 1, 0, 0, __pyx_nargs); return NULL; }
  const Py_ssize_t __pyx_kwds_len = unlikely(__pyx_kwds) ? __Pyx_NumKwargs_FASTCALL(__pyx_kwds) : 0;
  if (unlikely(__pyx_kwds_len < 0)) return NULL;
  if (unlikely(__pyx_kwds_len > 0)) {__Pyx_RejectKeywords("free_bytes", __pyx_kwds); return NULL;}
  __pyx_r = __pyx_pf_4cupy_4cuda_6memory_22SingleDeviceMemoryPool_16free_bytes(((struct __pyx_obj_4cupy_4cuda_6memory_SingleDeviceMemoryPool *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4cupy_4cuda_6memory_22SingleDeviceMemoryPool_16free_bytes(struct __pyx_obj_4cupy_4cuda_6memory_SingleDeviceMemoryPool *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  size_t __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("free_bytes", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __pyx_f_4cupy_4cuda_6memory_22SingleDeviceMemoryPool_free_bytes(__pyx_v_self, 1); if (unlikely(PyErr_Occurred())) __PYX_ERR(0, 1458, __pyx_L1_error)
  __pyx_t_2 = __Pyx_PyLong_FromSize_t(__pyx_t_1); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1458, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_r = __pyx_t_2;
  __pyx_t_2 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_AddTraceback("cupy.cuda.memory.SingleDeviceMemoryPool.free_bytes", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "cupy/cuda/memory.pyx":1461
 *         return self._total_bytes.load() - self._in_use_bytes.load()
 * 
 *     cpdef size_t total_bytes(self):             # <<<<<<<<<<<<<<
 *         return self._total_bytes.load()
 * 
*/

static PyObject *__pyx_pw_4cupy_4cuda_6memory_22SingleDeviceMemoryPool_19total_bytes(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
static size_t __pyx_f_4cupy_4cuda_6memory_22SingleDeviceMemoryPool_total_bytes(struct __pyx_obj_4cupy_4cuda_6memory_SingleDeviceMemoryPool *__pyx_v_self, CYTHON_UNUSED int __pyx_skip_dispatch) {
  size_t __pyx_r;

  /* "cupy/cuda/memory.pyx":1462
 * 
 *     cpdef size_t total_bytes(self):
 *         return self._total_bytes.load()             # <<<<<<<<<<<<<<
 * 
 *     cdef bint _try_block_total_bytes(self, size_t size) except -1:
*/
  __pyx_r = __pyx_v_self->_total_bytes.load();
  goto __pyx_L0;

  /* "cupy/cuda/memory.pyx":1461
 *         return self._total_bytes.load() - self._in_use_bytes.load()
 * 
 *     cpdef size_t total_bytes(self):             # <<<<<<<<<<<<<<
 *         return self._total_bytes.load()
 * 
*/

  /* function exit code */
  __pyx_L0:;
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_pw_4cupy_4cuda_6memory_22SingleDeviceMemoryPool_19total_bytes(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
PyDoc_STRVAR(__pyx_doc_4cupy_4cuda_6memory_22SingleDeviceMemoryPool_18total_bytes, "SingleDeviceMemoryPool.total_bytes(self) -> size_t");
static PyObject *__pyx_pw_4cupy_4cuda_6memory_22SingleDeviceMemoryPool_19total_bytes(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
) {
  #if !CYTHON_METH_FASTCALL
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  #endif
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("total_bytes (wrapper)", 0);
  #if !CYTHON_METH_FASTCALL
  #if CYTHON_ASSUME_SAFE_SIZE
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return NULL;
  #endif
  #endif
  __pyx_kwvalues = __Pyx_KwValues_FASTCALL(__pyx_args, __pyx_nargs);
  if (unlikely(__pyx_nargs > 0)) { __Pyx_RaiseArgtupleInvalid("total_bytes", 1, 0, 0, __pyx_nargs); return NULL; }
  const Py_ssize_t __pyx_kwds_len = unlikely(__pyx_kwds) ? __Pyx_NumKwargs_FASTCALL(__pyx_kwds) : 0;
  if (unlikely(__pyx_kwds_len < 0)) return NULL;
  if (unlikely(__pyx_kwds_len > 0)) {__Pyx_RejectKeywords("total_bytes", __pyx_kwds); return NULL;}
  __pyx_r = __pyx_pf_4cupy_4cuda_6memory_22SingleDeviceMemoryPool_18total_bytes(((struct __pyx_obj_4cupy_4cuda_6memory_SingleDeviceMemoryPool *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4cupy_4cuda_6memory_22SingleDeviceMemoryPool_18total_bytes(struct __pyx_obj_4cupy_4cuda_6memory_SingleDeviceMemoryPool *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  size_t __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("total_bytes", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __pyx_f_4cupy_4cuda_6memory_22SingleDeviceMemoryPool_total_bytes(__pyx_v_self, 1); if (unlikely(PyErr_Occurred())) __PYX_ERR(0, 1461, __pyx_L1_error)
  __pyx_t_2 = __Pyx_PyLong_FromSize_t(__pyx_t_1); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1461, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_r = __pyx_t_2;
  __pyx_t_2 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_AddTraceback("cupy.cuda.memory.SingleDeviceMemoryPool.total_bytes", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "cupy/cuda/memory.pyx":1464
 *         return self._total_bytes.load()
 * 
 *     cdef bint _try_block_total_bytes(self, size_t size) except -1:             # <<<<<<<<<<<<<<
 *         """Try to block off `size` bytes from the total pool size.
 *         Returns True if successfull (caller should try to allocate that many
*/

static int __pyx_f_4cupy_4cuda_6memory_22SingleDeviceMemoryPool__try_block_total_bytes(struct __pyx_obj_4cupy_4cuda_6memory_SingleDeviceMemoryPool *__pyx_v_self, size_t __pyx_v_size) {
  size_t __pyx_v_limit;
  size_t __pyx_v_curr_total_bytes;
  int __pyx_v_limit_ok;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  PyObject *__pyx_t_6 = NULL;
  PyObject *__pyx_t_7 = NULL;
  size_t __pyx_t_8;
  int __pyx_t_9;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("_try_block_total_bytes", 0);

  /* "cupy/cuda/memory.pyx":1470
 *         May raise `OutOfMemoryError` if `size` is too large for the pool.
 *         """
 *         cdef size_t limit = self._total_bytes_limit.load()             # <<<<<<<<<<<<<<
 *         cdef size_t curr_total_bytes = self._total_bytes.load()
 *         cdef bint limit_ok
*/
  __pyx_v_limit = __pyx_v_self->_total_bytes_limit.load();

  /* "cupy/cuda/memory.pyx":1471
 *         """
 *         cdef size_t limit = self._total_bytes_limit.load()
 *         cdef size_t curr_total_bytes = self._total_bytes.load()             # <<<<<<<<<<<<<<
 *         cdef bint limit_ok
 * 
*/
  __pyx_v_curr_total_bytes = __pyx_v_self->_total_bytes.load();

  /* "cupy/cuda/memory.pyx":1474
 *         cdef bint limit_ok
 * 
 *         if limit == 0:             # <<<<<<<<<<<<<<
 *             limit = UINT64_MAX
 * 
*/
  __pyx_t_1 = (__pyx_v_limit == 0);
  if (__pyx_t_1) {

    /* "cupy/cuda/memory.pyx":1475
 * 
 *         if limit == 0:
 *             limit = UINT64_MAX             # <<<<<<<<<<<<<<
 * 
 *         if size > limit:
*/
    __pyx_v_limit = UINT64_MAX;

    /* "cupy/cuda/memory.pyx":1474
 *         cdef bint limit_ok
 * 
 *         if limit == 0:             # <<<<<<<<<<<<<<
 *             limit = UINT64_MAX
 * 
*/
  }

  /* "cupy/cuda/memory.pyx":1477
 *             limit = UINT64_MAX
 * 
 *         if size > limit:             # <<<<<<<<<<<<<<
 *             # Check also protects against integer overflow below.
 *             raise OutOfMemoryError(size, self._total_bytes.load(), limit)
*/
  __pyx_t_1 = (__pyx_v_size > __pyx_v_limit);
  if (unlikely(__pyx_t_1)) {

    /* "cupy/cuda/memory.pyx":1479
 *         if size > limit:
 *             # Check also protects against integer overflow below.
 *             raise OutOfMemoryError(size, self._total_bytes.load(), limit)             # <<<<<<<<<<<<<<
 * 
 *         limit_ok = curr_total_bytes <= limit - size
*/
    __pyx_t_3 = NULL;
    __Pyx_GetModuleGlobalName(__pyx_t_4, __pyx_mstate_global->__pyx_n_u_OutOfMemoryError); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 1479, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __pyx_t_5 = __Pyx_PyLong_FromSize_t(__pyx_v_size); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 1479, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __pyx_t_6 = __Pyx_PyLong_FromSize_t(__pyx_v_self->_total_bytes.load()); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 1479, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_6);
    __pyx_t_7 = __Pyx_PyLong_FromSize_t(__pyx_v_limit); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 1479, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_7);
    __pyx_t_8 = 1;
    #if CYTHON_UNPACK_METHODS
    if (unlikely(PyMethod_Check(__pyx_t_4))) {
      __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_4);
      assert(__pyx_t_3);
      PyObject* __pyx__function = PyMethod_GET_FUNCTION(__pyx_t_4);
      __Pyx_INCREF(__pyx_t_3);
      __Pyx_INCREF(__pyx__function);
      __Pyx_DECREF_SET(__pyx_t_4, __pyx__function);
      __pyx_t_8 = 0;
    }
    #endif
    {
      PyObject *__pyx_callargs[4] = {__pyx_t_3, __pyx_t_5, __pyx_t_6, __pyx_t_7};
      __pyx_t_2 = __Pyx_PyObject_FastCall((PyObject*)__pyx_t_4, __pyx_callargs+__pyx_t_8, (4-__pyx_t_8) | (__pyx_t_8*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
      __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
      __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
      __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
      __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1479, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_2);
    }
    __Pyx_Raise(__pyx_t_2, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __PYX_ERR(0, 1479, __pyx_L1_error)

    /* "cupy/cuda/memory.pyx":1477
 *             limit = UINT64_MAX
 * 
 *         if size > limit:             # <<<<<<<<<<<<<<
 *             # Check also protects against integer overflow below.
 *             raise OutOfMemoryError(size, self._total_bytes.load(), limit)
*/
  }

  /* "cupy/cuda/memory.pyx":1481
 *             raise OutOfMemoryError(size, self._total_bytes.load(), limit)
 * 
 *         limit_ok = curr_total_bytes <= limit - size             # <<<<<<<<<<<<<<
 * 
 *         while limit_ok and not self._total_bytes.compare_exchange_weak(
*/
  __pyx_v_limit_ok = (__pyx_v_curr_total_bytes <= (__pyx_v_limit - __pyx_v_size));

  /* "cupy/cuda/memory.pyx":1483
 *         limit_ok = curr_total_bytes <= limit - size
 * 
 *         while limit_ok and not self._total_bytes.compare_exchange_weak(             # <<<<<<<<<<<<<<
 *                 curr_total_bytes, curr_total_bytes+size):
 *             # If we reach here, `_total_bytes` was changed by another thread.
*/
  while (1) {
    if (__pyx_v_limit_ok) {
    } else {
      __pyx_t_1 = __pyx_v_limit_ok;
      goto __pyx_L7_bool_binop_done;
    }

    /* "cupy/cuda/memory.pyx":1484
 * 
 *         while limit_ok and not self._total_bytes.compare_exchange_weak(
 *                 curr_total_bytes, curr_total_bytes+size):             # <<<<<<<<<<<<<<
 *             # If we reach here, `_total_bytes` was changed by another thread.
 *             # TODO(seberg): Presumably we could use weaker memory order.
*/
    __pyx_t_9 = (!__pyx_v_self->_total_bytes.compare_exchange_weak(__pyx_v_curr_total_bytes, (__pyx_v_curr_total_bytes + __pyx_v_size)));
    __pyx_t_1 = __pyx_t_9;
    __pyx_L7_bool_binop_done:;
    if (!__pyx_t_1) break;

    /* "cupy/cuda/memory.pyx":1487
 *             # If we reach here, `_total_bytes` was changed by another thread.
 *             # TODO(seberg): Presumably we could use weaker memory order.
 *             curr_total_bytes = self._total_bytes.load()             # <<<<<<<<<<<<<<
 *             limit_ok = curr_total_bytes <= limit - size
 * 
*/
    __pyx_v_curr_total_bytes = __pyx_v_self->_total_bytes.load();

    /* "cupy/cuda/memory.pyx":1488
 *             # TODO(seberg): Presumably we could use weaker memory order.
 *             curr_total_bytes = self._total_bytes.load()
 *             limit_ok = curr_total_bytes <= limit - size             # <<<<<<<<<<<<<<
 * 
 *         return limit_ok
*/
    __pyx_v_limit_ok = (__pyx_v_curr_total_bytes <= (__pyx_v_limit - __pyx_v_size));
  }

  /* "cupy/cuda/memory.pyx":1490
 *             limit_ok = curr_total_bytes <= limit - size
 * 
 *         return limit_ok             # <<<<<<<<<<<<<<
 * 
 *     cpdef set_limit(self, size=None, fraction=None):
*/
  __pyx_r = __pyx_v_limit_ok;
  goto __pyx_L0;

  /* "cupy/cuda/memory.pyx":1464
 *         return self._total_bytes.load()
 * 
 *     cdef bint _try_block_total_bytes(self, size_t size) except -1:             # <<<<<<<<<<<<<<
 *         """Try to block off `size` bytes from the total pool size.
 *         Returns True if successfull (caller should try to allocate that many
*/

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_XDECREF(__pyx_t_7);
  __Pyx_AddTraceback("cupy.cuda.memory.SingleDeviceMemoryPool._try_block_total_bytes", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "cupy/cuda/memory.pyx":1492
 *         return limit_ok
 * 
 *     cpdef set_limit(self, size=None, fraction=None):             # <<<<<<<<<<<<<<
 *         if size is None:
 *             if fraction is None:
*/

static PyObject *__pyx_pw_4cupy_4cuda_6memory_22SingleDeviceMemoryPool_21set_limit(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
static PyObject *__pyx_f_4cupy_4cuda_6memory_22SingleDeviceMemoryPool_set_limit(struct __pyx_obj_4cupy_4cuda_6memory_SingleDeviceMemoryPool *__pyx_v_self, CYTHON_UNUSED int __pyx_skip_dispatch, struct __pyx_opt_args_4cupy_4cuda_6memory_22SingleDeviceMemoryPool_set_limit *__pyx_optional_args) {
  PyObject *__pyx_v_size = ((PyObject *)Py_None);
  PyObject *__pyx_v_fraction = ((PyObject *)Py_None);
  CYTHON_UNUSED PyObject *__pyx_v__ = NULL;
  PyObject *__pyx_v_total = NULL;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  int __pyx_t_3;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  PyObject *__pyx_t_6 = NULL;
  size_t __pyx_t_7;
  PyObject *(*__pyx_t_8)(PyObject *);
  struct __pyx_opt_args_4cupy_4cuda_6memory_22SingleDeviceMemoryPool_set_limit __pyx_t_9;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("set_limit", 0);
  if (__pyx_optional_args) {
    if (__pyx_optional_args->__pyx_n > 0) {
      __pyx_v_size = __pyx_optional_args->size;
      if (__pyx_optional_args->__pyx_n > 1) {
        __pyx_v_fraction = __pyx_optional_args->fraction;
      }
    }
  }
  __Pyx_INCREF(__pyx_v_size);

  /* "cupy/cuda/memory.pyx":1493
 * 
 *     cpdef set_limit(self, size=None, fraction=None):
 *         if size is None:             # <<<<<<<<<<<<<<
 *             if fraction is None:
 *                 size = 0
*/
  __pyx_t_1 = (__pyx_v_size == Py_None);
  if (__pyx_t_1) {

    /* "cupy/cuda/memory.pyx":1494
 *     cpdef set_limit(self, size=None, fraction=None):
 *         if size is None:
 *             if fraction is None:             # <<<<<<<<<<<<<<
 *                 size = 0
 *             else:
*/
    __pyx_t_1 = (__pyx_v_fraction == Py_None);
    if (__pyx_t_1) {

      /* "cupy/cuda/memory.pyx":1495
 *         if size is None:
 *             if fraction is None:
 *                 size = 0             # <<<<<<<<<<<<<<
 *             else:
 *                 if not 0 <= fraction <= 1:
*/
      __Pyx_INCREF(__pyx_mstate_global->__pyx_int_0);
      __Pyx_DECREF_SET(__pyx_v_size, __pyx_mstate_global->__pyx_int_0);

      /* "cupy/cuda/memory.pyx":1494
 *     cpdef set_limit(self, size=None, fraction=None):
 *         if size is None:
 *             if fraction is None:             # <<<<<<<<<<<<<<
 *                 size = 0
 *             else:
*/
      goto __pyx_L4;
    }

    /* "cupy/cuda/memory.pyx":1497
 *                 size = 0
 *             else:
 *                 if not 0 <= fraction <= 1:             # <<<<<<<<<<<<<<
 *                     raise ValueError(
 *                         'memory limit fraction out of range: {}'.format(
*/
    /*else*/ {
      __pyx_t_2 = PyObject_RichCompare(__pyx_mstate_global->__pyx_int_0, __pyx_v_fraction, Py_LE); __Pyx_XGOTREF(__pyx_t_2); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1497, __pyx_L1_error)
      if (__Pyx_PyObject_IsTrue(__pyx_t_2)) {
        __Pyx_DECREF(__pyx_t_2);
        __pyx_t_2 = PyObject_RichCompare(__pyx_v_fraction, __pyx_mstate_global->__pyx_int_1, Py_LE); __Pyx_XGOTREF(__pyx_t_2); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1497, __pyx_L1_error)
      }
      __pyx_t_1 = __Pyx_PyObject_IsTrue(__pyx_t_2); if (unlikely((__pyx_t_1 < 0))) __PYX_ERR(0, 1497, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
      __pyx_t_3 = (!__pyx_t_1);
      if (unlikely(__pyx_t_3)) {

        /* "cupy/cuda/memory.pyx":1498
 *             else:
 *                 if not 0 <= fraction <= 1:
 *                     raise ValueError(             # <<<<<<<<<<<<<<
 *                         'memory limit fraction out of range: {}'.format(
 *                             fraction))
*/
        __pyx_t_4 = NULL;

        /* "cupy/cuda/memory.pyx":1499
 *                 if not 0 <= fraction <= 1:
 *                     raise ValueError(
 *                         'memory limit fraction out of range: {}'.format(             # <<<<<<<<<<<<<<
 *                             fraction))
 *                 _, total = runtime.memGetInfo()
*/
        __pyx_t_6 = __pyx_mstate_global->__pyx_kp_u_memory_limit_fraction_out_of_ran;
        __Pyx_INCREF(__pyx_t_6);

        /* "cupy/cuda/memory.pyx":1500
 *                     raise ValueError(
 *                         'memory limit fraction out of range: {}'.format(
 *                             fraction))             # <<<<<<<<<<<<<<
 *                 _, total = runtime.memGetInfo()
 *                 size = fraction * total
*/
        __pyx_t_7 = 0;
        {
          PyObject *__pyx_callargs[2] = {__pyx_t_6, __pyx_v_fraction};
          __pyx_t_5 = __Pyx_PyObject_FastCallMethod((PyObject*)__pyx_mstate_global->__pyx_n_u_format, __pyx_callargs+__pyx_t_7, (2-__pyx_t_7) | (1*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
          __Pyx_XDECREF(__pyx_t_6); __pyx_t_6 = 0;
          if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 1499, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_5);
        }
        __pyx_t_7 = 1;
        {
          PyObject *__pyx_callargs[2] = {__pyx_t_4, __pyx_t_5};
          __pyx_t_2 = __Pyx_PyObject_FastCall((PyObject*)(((PyTypeObject*)PyExc_ValueError)), __pyx_callargs+__pyx_t_7, (2-__pyx_t_7) | (__pyx_t_7*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
          __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
          __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
          if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1498, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_2);
        }
        __Pyx_Raise(__pyx_t_2, 0, 0, 0);
        __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
        __PYX_ERR(0, 1498, __pyx_L1_error)

        /* "cupy/cuda/memory.pyx":1497
 *                 size = 0
 *             else:
 *                 if not 0 <= fraction <= 1:             # <<<<<<<<<<<<<<
 *                     raise ValueError(
 *                         'memory limit fraction out of range: {}'.format(
*/
      }

      /* "cupy/cuda/memory.pyx":1501
 *                         'memory limit fraction out of range: {}'.format(
 *                             fraction))
 *                 _, total = runtime.memGetInfo()             # <<<<<<<<<<<<<<
 *                 size = fraction * total
 *             self.set_limit(size=size)
*/
      __pyx_t_2 = __pyx_f_13cupy_backends_4cuda_3api_7runtime_memGetInfo(0); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1501, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_2);
      if ((likely(PyTuple_CheckExact(__pyx_t_2))) || (PyList_CheckExact(__pyx_t_2))) {
        PyObject* sequence = __pyx_t_2;
        Py_ssize_t size = __Pyx_PySequence_SIZE(sequence);
        if (unlikely(size != 2)) {
          if (size > 2) __Pyx_RaiseTooManyValuesError(2);
          else if (size >= 0) __Pyx_RaiseNeedMoreValuesError(size);
          __PYX_ERR(0, 1501, __pyx_L1_error)
        }
        #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
        if (likely(PyTuple_CheckExact(sequence))) {
          __pyx_t_5 = PyTuple_GET_ITEM(sequence, 0);
          __Pyx_INCREF(__pyx_t_5);
          __pyx_t_4 = PyTuple_GET_ITEM(sequence, 1);
          __Pyx_INCREF(__pyx_t_4);
        } else {
          __pyx_t_5 = __Pyx_PyList_GetItemRefFast(sequence, 0, __Pyx_ReferenceSharing_SharedReference);
          if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 1501, __pyx_L1_error)
          __Pyx_XGOTREF(__pyx_t_5);
          __pyx_t_4 = __Pyx_PyList_GetItemRefFast(sequence, 1, __Pyx_ReferenceSharing_SharedReference);
          if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 1501, __pyx_L1_error)
          __Pyx_XGOTREF(__pyx_t_4);
        }
        #else
        __pyx_t_5 = __Pyx_PySequence_ITEM(sequence, 0); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 1501, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_5);
        __pyx_t_4 = __Pyx_PySequence_ITEM(sequence, 1); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 1501, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_4);
        #endif
        __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
      } else {
        Py_ssize_t index = -1;
        __pyx_t_6 = PyObject_GetIter(__pyx_t_2); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 1501, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_6);
        __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
        __pyx_t_8 = (CYTHON_COMPILING_IN_LIMITED_API) ? PyIter_Next : __Pyx_PyObject_GetIterNextFunc(__pyx_t_6);
        index = 0; __pyx_t_5 = __pyx_t_8(__pyx_t_6); if (unlikely(!__pyx_t_5)) goto __pyx_L6_unpacking_failed;
        __Pyx_GOTREF(__pyx_t_5);
        index = 1; __pyx_t_4 = __pyx_t_8(__pyx_t_6); if (unlikely(!__pyx_t_4)) goto __pyx_L6_unpacking_failed;
        __Pyx_GOTREF(__pyx_t_4);
        if (__Pyx_IternextUnpackEndCheck(__pyx_t_8(__pyx_t_6), 2) < (0)) __PYX_ERR(0, 1501, __pyx_L1_error)
        __pyx_t_8 = NULL;
        __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
        goto __pyx_L7_unpacking_done;
        __pyx_L6_unpacking_failed:;
        __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
        __pyx_t_8 = NULL;
        if (__Pyx_IterFinish() == 0) __Pyx_RaiseNeedMoreValuesError(index);
        __PYX_ERR(0, 1501, __pyx_L1_error)
        __pyx_L7_unpacking_done:;
      }
      __pyx_v__ = __pyx_t_5;
      __pyx_t_5 = 0;
      __pyx_v_total = __pyx_t_4;
      __pyx_t_4 = 0;

      /* "cupy/cuda/memory.pyx":1502
 *                             fraction))
 *                 _, total = runtime.memGetInfo()
 *                 size = fraction * total             # <<<<<<<<<<<<<<
 *             self.set_limit(size=size)
 *             return
*/
      __pyx_t_2 = PyNumber_Multiply(__pyx_v_fraction, __pyx_v_total); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1502, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_2);
      __Pyx_DECREF_SET(__pyx_v_size, __pyx_t_2);
      __pyx_t_2 = 0;
    }
    __pyx_L4:;

    /* "cupy/cuda/memory.pyx":1503
 *                 _, total = runtime.memGetInfo()
 *                 size = fraction * total
 *             self.set_limit(size=size)             # <<<<<<<<<<<<<<
 *             return
 * 
*/
    __pyx_t_9.__pyx_n = 1;
    __pyx_t_9.size = __pyx_v_size;
    __pyx_t_2 = __pyx_f_4cupy_4cuda_6memory_22SingleDeviceMemoryPool_set_limit(__pyx_v_self, 0, &__pyx_t_9); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1503, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

    /* "cupy/cuda/memory.pyx":1504
 *                 size = fraction * total
 *             self.set_limit(size=size)
 *             return             # <<<<<<<<<<<<<<
 * 
 *         if fraction is not None:
*/
    __Pyx_XDECREF(__pyx_r);
    __pyx_r = Py_None; __Pyx_INCREF(Py_None);
    goto __pyx_L0;

    /* "cupy/cuda/memory.pyx":1493
 * 
 *     cpdef set_limit(self, size=None, fraction=None):
 *         if size is None:             # <<<<<<<<<<<<<<
 *             if fraction is None:
 *                 size = 0
*/
  }

  /* "cupy/cuda/memory.pyx":1506
 *             return
 * 
 *         if fraction is not None:             # <<<<<<<<<<<<<<
 *             raise ValueError('size and fraction cannot be specified at '
 *                              'one time')
*/
  __pyx_t_3 = (__pyx_v_fraction != Py_None);
  if (unlikely(__pyx_t_3)) {

    /* "cupy/cuda/memory.pyx":1507
 * 
 *         if fraction is not None:
 *             raise ValueError('size and fraction cannot be specified at '             # <<<<<<<<<<<<<<
 *                              'one time')
 *         if size < 0:
*/
    __pyx_t_4 = NULL;
    __pyx_t_7 = 1;
    {
      PyObject *__pyx_callargs[2] = {__pyx_t_4, __pyx_mstate_global->__pyx_kp_u_size_and_fraction_cannot_be_spec};
      __pyx_t_2 = __Pyx_PyObject_FastCall((PyObject*)(((PyTypeObject*)PyExc_ValueError)), __pyx_callargs+__pyx_t_7, (2-__pyx_t_7) | (__pyx_t_7*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
      __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
      if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1507, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_2);
    }
    __Pyx_Raise(__pyx_t_2, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __PYX_ERR(0, 1507, __pyx_L1_error)

    /* "cupy/cuda/memory.pyx":1506
 *             return
 * 
 *         if fraction is not None:             # <<<<<<<<<<<<<<
 *             raise ValueError('size and fraction cannot be specified at '
 *                              'one time')
*/
  }

  /* "cupy/cuda/memory.pyx":1509
 *             raise ValueError('size and fraction cannot be specified at '
 *                              'one time')
 *         if size < 0:             # <<<<<<<<<<<<<<
 *             raise ValueError(
 *                 'memory limit size out of range: {}'.format(size))
*/
  __pyx_t_2 = PyObject_RichCompare(__pyx_v_size, __pyx_mstate_global->__pyx_int_0, Py_LT); __Pyx_XGOTREF(__pyx_t_2); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1509, __pyx_L1_error)
  __pyx_t_3 = __Pyx_PyObject_IsTrue(__pyx_t_2); if (unlikely((__pyx_t_3 < 0))) __PYX_ERR(0, 1509, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  if (unlikely(__pyx_t_3)) {

    /* "cupy/cuda/memory.pyx":1510
 *                              'one time')
 *         if size < 0:
 *             raise ValueError(             # <<<<<<<<<<<<<<
 *                 'memory limit size out of range: {}'.format(size))
 * 
*/
    __pyx_t_4 = NULL;

    /* "cupy/cuda/memory.pyx":1511
 *         if size < 0:
 *             raise ValueError(
 *                 'memory limit size out of range: {}'.format(size))             # <<<<<<<<<<<<<<
 * 
 *         self._total_bytes_limit.store(size)
*/
    __pyx_t_6 = __pyx_mstate_global->__pyx_kp_u_memory_limit_size_out_of_range;
    __Pyx_INCREF(__pyx_t_6);
    __pyx_t_7 = 0;
    {
      PyObject *__pyx_callargs[2] = {__pyx_t_6, __pyx_v_size};
      __pyx_t_5 = __Pyx_PyObject_FastCallMethod((PyObject*)__pyx_mstate_global->__pyx_n_u_format, __pyx_callargs+__pyx_t_7, (2-__pyx_t_7) | (1*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
      __Pyx_XDECREF(__pyx_t_6); __pyx_t_6 = 0;
      if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 1511, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_5);
    }
    __pyx_t_7 = 1;
    {
      PyObject *__pyx_callargs[2] = {__pyx_t_4, __pyx_t_5};
      __pyx_t_2 = __Pyx_PyObject_FastCall((PyObject*)(((PyTypeObject*)PyExc_ValueError)), __pyx_callargs+__pyx_t_7, (2-__pyx_t_7) | (__pyx_t_7*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
      __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
      __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
      if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1510, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_2);
    }
    __Pyx_Raise(__pyx_t_2, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __PYX_ERR(0, 1510, __pyx_L1_error)

    /* "cupy/cuda/memory.pyx":1509
 *             raise ValueError('size and fraction cannot be specified at '
 *                              'one time')
 *         if size < 0:             # <<<<<<<<<<<<<<
 *             raise ValueError(
 *                 'memory limit size out of range: {}'.format(size))
*/
  }

  /* "cupy/cuda/memory.pyx":1513
 *                 'memory limit size out of range: {}'.format(size))
 * 
 *         self._total_bytes_limit.store(size)             # <<<<<<<<<<<<<<
 * 
 *     cpdef size_t get_limit(self):
*/
  __pyx_t_7 = __Pyx_PyLong_As_size_t(__pyx_v_size); if (unlikely((__pyx_t_7 == (size_t)-1) && PyErr_Occurred())) __PYX_ERR(0, 1513, __pyx_L1_error)
  __pyx_v_self->_total_bytes_limit.store(__pyx_t_7);

  /* "cupy/cuda/memory.pyx":1492
 *         return limit_ok
 * 
 *     cpdef set_limit(self, size=None, fraction=None):             # <<<<<<<<<<<<<<
 *         if size is None:
 *             if fraction is None:
*/

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_AddTraceback("cupy.cuda.memory.SingleDeviceMemoryPool.set_limit", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v__);
  __Pyx_XDECREF(__pyx_v_total);
  __Pyx_XDECREF(__pyx_v_size);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_pw_4cupy_4cuda_6memory_22SingleDeviceMemoryPool_21set_limit(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
PyDoc_STRVAR(__pyx_doc_4cupy_4cuda_6memory_22SingleDeviceMemoryPool_20set_limit, "SingleDeviceMemoryPool.set_limit(self, size=None, fraction=None)");
static PyObject *__pyx_pw_4cupy_4cuda_6memory_22SingleDeviceMemoryPool_21set_limit(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
) {
  PyObject *__pyx_v_size = 0;
  PyObject *__pyx_v_fraction = 0;
  #if !CYTHON_METH_FASTCALL
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  #endif
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject* values[2] = {0,0};
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("set_limit (wrapper)", 0);
  #if !CYTHON_METH_FASTCALL
  #if CYTHON_ASSUME_SAFE_SIZE
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return NULL;
  #endif
  #endif
  __pyx_kwvalues = __Pyx_KwValues_FASTCALL(__pyx_args, __pyx_nargs);
  {
    PyObject ** const __pyx_pyargnames[] = {&__pyx_mstate_global->__pyx_n_u_size,&__pyx_mstate_global->__pyx_n_u_fraction,0};
    const Py_ssize_t __pyx_kwds_len = (__pyx_kwds) ? __Pyx_NumKwargs_FASTCALL(__pyx_kwds) : 0;
    if (unlikely(__pyx_kwds_len < 0)) __PYX_ERR(0, 1492, __pyx_L3_error)
    if (__pyx_kwds_len > 0) {
      switch (__pyx_nargs) {
        case  2:
        values[1] = __Pyx_ArgRef_FASTCALL(__pyx_args, 1);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[1])) __PYX_ERR(0, 1492, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  1:
        values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(0, 1492, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      const Py_ssize_t kwd_pos_args = __pyx_nargs;
      if (__Pyx_ParseKeywords(__pyx_kwds, __pyx_kwvalues, __pyx_pyargnames, 0, values, kwd_pos_args, __pyx_kwds_len, "set_limit", 0) < (0)) __PYX_ERR(0, 1492, __pyx_L3_error)
      if (!values[0]) values[0] = __Pyx_NewRef(((PyObject *)Py_None));
      if (!values[1]) values[1] = __Pyx_NewRef(((PyObject *)Py_None));
    } else {
      switch (__pyx_nargs) {
        case  2:
        values[1] = __Pyx_ArgRef_FASTCALL(__pyx_args, 1);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[1])) __PYX_ERR(0, 1492, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  1:
        values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(0, 1492, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      if (!values[0]) values[0] = __Pyx_NewRef(((PyObject *)Py_None));
      if (!values[1]) values[1] = __Pyx_NewRef(((PyObject *)Py_None));
    }
    __pyx_v_size = values[0];
    __pyx_v_fraction = values[1];
  }
  goto __pyx_L6_skip;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("set_limit", 0, 0, 2, __pyx_nargs); __PYX_ERR(0, 1492, __pyx_L3_error)
  __pyx_L6_skip:;
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_AddTraceback("cupy.cuda.memory.SingleDeviceMemoryPool.set_limit", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_4cupy_4cuda_6memory_22SingleDeviceMemoryPool_20set_limit(((struct __pyx_obj_4cupy_4cuda_6memory_SingleDeviceMemoryPool *)__pyx_v_self), __pyx_v_size, __pyx_v_fraction);

  /* function exit code */
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4cupy_4cuda_6memory_22SingleDeviceMemoryPool_20set_limit(struct __pyx_obj_4cupy_4cuda_6memory_SingleDeviceMemoryPool *__pyx_v_self, PyObject *__pyx_v_size, PyObject *__pyx_v_fraction) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  struct __pyx_opt_args_4cupy_4cuda_6memory_22SingleDeviceMemoryPool_set_limit __pyx_t_2;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("set_limit", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_2.__pyx_n = 2;
  __pyx_t_2.size = __pyx_v_size;
  __pyx_t_2.fraction = __pyx_v_fraction;
  __pyx_t_1 = __pyx_vtabptr_4cupy_4cuda_6memory_SingleDeviceMemoryPool->set_limit(__pyx_v_self, 1, &__pyx_t_2); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1492, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("cupy.cuda.memory.SingleDeviceMemoryPool.set_limit", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "cupy/cuda/memory.pyx":1515
 *         self._total_bytes_limit.store(size)
 * 
 *     cpdef size_t get_limit(self):             # <<<<<<<<<<<<<<
 *         return self._total_bytes_limit.load()
 * 
*/

static PyObject *__pyx_pw_4cupy_4cuda_6memory_22SingleDeviceMemoryPool_23get_limit(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
static size_t __pyx_f_4cupy_4cuda_6memory_22SingleDeviceMemoryPool_get_limit(struct __pyx_obj_4cupy_4cuda_6memory_SingleDeviceMemoryPool *__pyx_v_self, CYTHON_UNUSED int __pyx_skip_dispatch) {
  size_t __pyx_r;

  /* "cupy/cuda/memory.pyx":1516
 * 
 *     cpdef size_t get_limit(self):
 *         return self._total_bytes_limit.load()             # <<<<<<<<<<<<<<
 * 
 *     cdef BaseMemory _try_malloc(self, size_t size):
*/
  __pyx_r = __pyx_v_self->_total_bytes_limit.load();
  goto __pyx_L0;

  /* "cupy/cuda/memory.pyx":1515
 *         self._total_bytes_limit.store(size)
 * 
 *     cpdef size_t get_limit(self):             # <<<<<<<<<<<<<<
 *         return self._total_bytes_limit.load()
 * 
*/

  /* function exit code */
  __pyx_L0:;
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_pw_4cupy_4cuda_6memory_22SingleDeviceMemoryPool_23get_limit(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
PyDoc_STRVAR(__pyx_doc_4cupy_4cuda_6memory_22SingleDeviceMemoryPool_22get_limit, "SingleDeviceMemoryPool.get_limit(self) -> size_t");
static PyObject *__pyx_pw_4cupy_4cuda_6memory_22SingleDeviceMemoryPool_23get_limit(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
) {
  #if !CYTHON_METH_FASTCALL
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  #endif
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("get_limit (wrapper)", 0);
  #if !CYTHON_METH_FASTCALL
  #if CYTHON_ASSUME_SAFE_SIZE
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return NULL;
  #endif
  #endif
  __pyx_kwvalues = __Pyx_KwValues_FASTCALL(__pyx_args, __pyx_nargs);
  if (unlikely(__pyx_nargs > 0)) { __Pyx_RaiseArgtupleInvalid("get_limit", 1, 0, 0, __pyx_nargs); return NULL; }
  const Py_ssize_t __pyx_kwds_len = unlikely(__pyx_kwds) ? __Pyx_NumKwargs_FASTCALL(__pyx_kwds) : 0;
  if (unlikely(__pyx_kwds_len < 0)) return NULL;
  if (unlikely(__pyx_kwds_len > 0)) {__Pyx_RejectKeywords("get_limit", __pyx_kwds); return NULL;}
  __pyx_r = __pyx_pf_4cupy_4cuda_6memory_22SingleDeviceMemoryPool_22get_limit(((struct __pyx_obj_4cupy_4cuda_6memory_SingleDeviceMemoryPool *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4cupy_4cuda_6memory_22SingleDeviceMemoryPool_22get_limit(struct __pyx_obj_4cupy_4cuda_6memory_SingleDeviceMemoryPool *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  size_t __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("get_limit", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __pyx_f_4cupy_4cuda_6memory_22SingleDeviceMemoryPool_get_limit(__pyx_v_self, 1); if (unlikely(PyErr_Occurred())) __PYX_ERR(0, 1515, __pyx_L1_error)
  __pyx_t_2 = __Pyx_PyLong_FromSize_t(__pyx_t_1); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1515, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_r = __pyx_t_2;
  __pyx_t_2 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_AddTraceback("cupy.cuda.memory.SingleDeviceMemoryPool.get_limit", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "cupy/cuda/memory.pyx":1518
 *         return self._total_bytes_limit.load()
 * 
 *     cdef BaseMemory _try_malloc(self, size_t size):             # <<<<<<<<<<<<<<
 *         cdef size_t limit = self._total_bytes_limit.load()
 *         cdef bint ok
*/

static struct __pyx_obj_4cupy_4cuda_6memory_BaseMemory *__pyx_f_4cupy_4cuda_6memory_22SingleDeviceMemoryPool__try_malloc(struct __pyx_obj_4cupy_4cuda_6memory_SingleDeviceMemoryPool *__pyx_v_self, size_t __pyx_v_size) {
  size_t __pyx_v_limit;
  int __pyx_v_ok;
  struct __pyx_obj_4cupy_4cuda_6memory_BaseMemory *__pyx_v_mem = NULL;
  int __pyx_v_oom_error;
  PyObject *__pyx_v_e = NULL;
  struct __pyx_obj_4cupy_4cuda_6memory_BaseMemory *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  int __pyx_t_2;
  PyObject *__pyx_t_3 = NULL;
  struct __pyx_opt_args_4cupy_4cuda_6memory_22SingleDeviceMemoryPool_free_all_blocks __pyx_t_4;
  int __pyx_t_5;
  PyObject *__pyx_t_6 = NULL;
  PyObject *__pyx_t_7 = NULL;
  PyObject *__pyx_t_8 = NULL;
  size_t __pyx_t_9;
  PyObject *__pyx_t_10 = NULL;
  PyObject *__pyx_t_11 = NULL;
  PyObject *__pyx_t_12 = NULL;
  PyObject *__pyx_t_13 = NULL;
  PyObject *__pyx_t_14 = NULL;
  int __pyx_t_15;
  PyObject *__pyx_t_16 = NULL;
  PyObject *__pyx_t_17 = NULL;
  PyObject *__pyx_t_18 = NULL;
  PyObject *__pyx_t_19 = NULL;
  PyObject *__pyx_t_20 = NULL;
  PyObject *__pyx_t_21 = NULL;
  PyObject *__pyx_t_22 = NULL;
  PyObject *__pyx_t_23 = NULL;
  PyObject *__pyx_t_24 = NULL;
  PyObject *__pyx_t_25 = NULL;
  PyObject *__pyx_t_26 = NULL;
  PyObject *__pyx_t_27 = NULL;
  int __pyx_t_28;
  char const *__pyx_t_29;
  PyObject *__pyx_t_30 = NULL;
  PyObject *__pyx_t_31 = NULL;
  PyObject *__pyx_t_32 = NULL;
  PyObject *__pyx_t_33 = NULL;
  PyObject *__pyx_t_34 = NULL;
  PyObject *__pyx_t_35 = NULL;
  char const *__pyx_t_36;
  char const *__pyx_t_37;
  char const *__pyx_t_38;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("_try_malloc", 0);

  /* "cupy/cuda/memory.pyx":1519
 * 
 *     cdef BaseMemory _try_malloc(self, size_t size):
 *         cdef size_t limit = self._total_bytes_limit.load()             # <<<<<<<<<<<<<<
 *         cdef bint ok
 * 
*/
  __pyx_v_limit = __pyx_v_self->_total_bytes_limit.load();

  /* "cupy/cuda/memory.pyx":1522
 *         cdef bint ok
 * 
 *         if not (ok := self._try_block_total_bytes(size)):             # <<<<<<<<<<<<<<
 *             self.free_all_blocks(None)
 *         if not ok and not (ok := self._try_block_total_bytes(size)):
*/
  __pyx_t_1 = __pyx_f_4cupy_4cuda_6memory_22SingleDeviceMemoryPool__try_block_total_bytes(__pyx_v_self, __pyx_v_size); if (unlikely(__pyx_t_1 == ((int)-1))) __PYX_ERR(0, 1522, __pyx_L1_error)
  __pyx_v_ok = __pyx_t_1;
  __pyx_t_2 = (!__pyx_t_1);
  if (__pyx_t_2) {

    /* "cupy/cuda/memory.pyx":1523
 * 
 *         if not (ok := self._try_block_total_bytes(size)):
 *             self.free_all_blocks(None)             # <<<<<<<<<<<<<<
 *         if not ok and not (ok := self._try_block_total_bytes(size)):
 *             gc.collect()
*/
    __pyx_t_4.__pyx_n = 1;
    __pyx_t_4.stream = Py_None;
    __pyx_t_3 = __pyx_f_4cupy_4cuda_6memory_22SingleDeviceMemoryPool_free_all_blocks(__pyx_v_self, 0, &__pyx_t_4); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 1523, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

    /* "cupy/cuda/memory.pyx":1522
 *         cdef bint ok
 * 
 *         if not (ok := self._try_block_total_bytes(size)):             # <<<<<<<<<<<<<<
 *             self.free_all_blocks(None)
 *         if not ok and not (ok := self._try_block_total_bytes(size)):
*/
  }

  /* "cupy/cuda/memory.pyx":1524
 *         if not (ok := self._try_block_total_bytes(size)):
 *             self.free_all_blocks(None)
 *         if not ok and not (ok := self._try_block_total_bytes(size)):             # <<<<<<<<<<<<<<
 *             gc.collect()
 *             self.free_all_blocks(None)
*/
  __pyx_t_1 = (!__pyx_v_ok);
  if (__pyx_t_1) {
  } else {
    __pyx_t_2 = __pyx_t_1;
    goto __pyx_L5_bool_binop_done;
  }
  __pyx_t_1 = __pyx_f_4cupy_4cuda_6memory_22SingleDeviceMemoryPool__try_block_total_bytes(__pyx_v_self, __pyx_v_size); if (unlikely(__pyx_t_1 == ((int)-1))) __PYX_ERR(0, 1524, __pyx_L1_error)
  __pyx_v_ok = __pyx_t_1;
  __pyx_t_5 = (!__pyx_t_1);
  __pyx_t_2 = __pyx_t_5;
  __pyx_L5_bool_binop_done:;
  if (__pyx_t_2) {

    /* "cupy/cuda/memory.pyx":1525
 *             self.free_all_blocks(None)
 *         if not ok and not (ok := self._try_block_total_bytes(size)):
 *             gc.collect()             # <<<<<<<<<<<<<<
 *             self.free_all_blocks(None)
 *         if not ok and not (ok := self._try_block_total_bytes(size)):
*/
    __pyx_t_6 = NULL;
    __Pyx_GetModuleGlobalName(__pyx_t_7, __pyx_mstate_global->__pyx_n_u_gc); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 1525, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_7);
    __pyx_t_8 = __Pyx_PyObject_GetAttrStr(__pyx_t_7, __pyx_mstate_global->__pyx_n_u_collect); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 1525, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_8);
    __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
    __pyx_t_9 = 1;
    #if CYTHON_UNPACK_METHODS
    if (unlikely(PyMethod_Check(__pyx_t_8))) {
      __pyx_t_6 = PyMethod_GET_SELF(__pyx_t_8);
      assert(__pyx_t_6);
      PyObject* __pyx__function = PyMethod_GET_FUNCTION(__pyx_t_8);
      __Pyx_INCREF(__pyx_t_6);
      __Pyx_INCREF(__pyx__function);
      __Pyx_DECREF_SET(__pyx_t_8, __pyx__function);
      __pyx_t_9 = 0;
    }
    #endif
    {
      PyObject *__pyx_callargs[2] = {__pyx_t_6, NULL};
      __pyx_t_3 = __Pyx_PyObject_FastCall((PyObject*)__pyx_t_8, __pyx_callargs+__pyx_t_9, (1-__pyx_t_9) | (__pyx_t_9*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
      __Pyx_XDECREF(__pyx_t_6); __pyx_t_6 = 0;
      __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
      if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 1525, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_3);
    }
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

    /* "cupy/cuda/memory.pyx":1526
 *         if not ok and not (ok := self._try_block_total_bytes(size)):
 *             gc.collect()
 *             self.free_all_blocks(None)             # <<<<<<<<<<<<<<
 *         if not ok and not (ok := self._try_block_total_bytes(size)):
 *             raise OutOfMemoryError(size, self._total_bytes.load(), limit)
*/
    __pyx_t_4.__pyx_n = 1;
    __pyx_t_4.stream = Py_None;
    __pyx_t_3 = __pyx_f_4cupy_4cuda_6memory_22SingleDeviceMemoryPool_free_all_blocks(__pyx_v_self, 0, &__pyx_t_4); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 1526, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

    /* "cupy/cuda/memory.pyx":1524
 *         if not (ok := self._try_block_total_bytes(size)):
 *             self.free_all_blocks(None)
 *         if not ok and not (ok := self._try_block_total_bytes(size)):             # <<<<<<<<<<<<<<
 *             gc.collect()
 *             self.free_all_blocks(None)
*/
  }

  /* "cupy/cuda/memory.pyx":1527
 *             gc.collect()
 *             self.free_all_blocks(None)
 *         if not ok and not (ok := self._try_block_total_bytes(size)):             # <<<<<<<<<<<<<<
 *             raise OutOfMemoryError(size, self._total_bytes.load(), limit)
 * 
*/
  __pyx_t_5 = (!__pyx_v_ok);
  if (__pyx_t_5) {
  } else {
    __pyx_t_2 = __pyx_t_5;
    goto __pyx_L8_bool_binop_done;
  }
  __pyx_t_5 = __pyx_f_4cupy_4cuda_6memory_22SingleDeviceMemoryPool__try_block_total_bytes(__pyx_v_self, __pyx_v_size); if (unlikely(__pyx_t_5 == ((int)-1))) __PYX_ERR(0, 1527, __pyx_L1_error)
  __pyx_v_ok = __pyx_t_5;
  __pyx_t_1 = (!__pyx_t_5);
  __pyx_t_2 = __pyx_t_1;
  __pyx_L8_bool_binop_done:;
  if (unlikely(__pyx_t_2)) {

    /* "cupy/cuda/memory.pyx":1528
 *             self.free_all_blocks(None)
 *         if not ok and not (ok := self._try_block_total_bytes(size)):
 *             raise OutOfMemoryError(size, self._total_bytes.load(), limit)             # <<<<<<<<<<<<<<
 * 
 *         # If we reach here, we are allowed to allocate size bytes based on
*/
    __pyx_t_8 = NULL;
    __Pyx_GetModuleGlobalName(__pyx_t_6, __pyx_mstate_global->__pyx_n_u_OutOfMemoryError); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 1528, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_6);
    __pyx_t_7 = __Pyx_PyLong_FromSize_t(__pyx_v_size); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 1528, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_7);
    __pyx_t_10 = __Pyx_PyLong_FromSize_t(__pyx_v_self->_total_bytes.load()); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 1528, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_10);
    __pyx_t_11 = __Pyx_PyLong_FromSize_t(__pyx_v_limit); if (unlikely(!__pyx_t_11)) __PYX_ERR(0, 1528, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_11);
    __pyx_t_9 = 1;
    #if CYTHON_UNPACK_METHODS
    if (unlikely(PyMethod_Check(__pyx_t_6))) {
      __pyx_t_8 = PyMethod_GET_SELF(__pyx_t_6);
      assert(__pyx_t_8);
      PyObject* __pyx__function = PyMethod_GET_FUNCTION(__pyx_t_6);
      __Pyx_INCREF(__pyx_t_8);
      __Pyx_INCREF(__pyx__function);
      __Pyx_DECREF_SET(__pyx_t_6, __pyx__function);
      __pyx_t_9 = 0;
    }
    #endif
    {
      PyObject *__pyx_callargs[4] = {__pyx_t_8, __pyx_t_7, __pyx_t_10, __pyx_t_11};
      __pyx_t_3 = __Pyx_PyObject_FastCall((PyObject*)__pyx_t_6, __pyx_callargs+__pyx_t_9, (4-__pyx_t_9) | (__pyx_t_9*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
      __Pyx_XDECREF(__pyx_t_8); __pyx_t_8 = 0;
      __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
      __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
      __Pyx_DECREF(__pyx_t_11); __pyx_t_11 = 0;
      __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
      if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 1528, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_3);
    }
    __Pyx_Raise(__pyx_t_3, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __PYX_ERR(0, 1528, __pyx_L1_error)

    /* "cupy/cuda/memory.pyx":1527
 *             gc.collect()
 *             self.free_all_blocks(None)
 *         if not ok and not (ok := self._try_block_total_bytes(size)):             # <<<<<<<<<<<<<<
 *             raise OutOfMemoryError(size, self._total_bytes.load(), limit)
 * 
*/
  }

  /* "cupy/cuda/memory.pyx":1533
 *         # the limit. But the actual allocation may still fail (and we must
 *         # reduce _total_bytes again if it does).
 *         mem = None             # <<<<<<<<<<<<<<
 *         oom_error = False
 *         try:
*/
  __Pyx_INCREF(Py_None);
  __pyx_v_mem = ((struct __pyx_obj_4cupy_4cuda_6memory_BaseMemory *)Py_None);

  /* "cupy/cuda/memory.pyx":1534
 *         # reduce _total_bytes again if it does).
 *         mem = None
 *         oom_error = False             # <<<<<<<<<<<<<<
 *         try:
 *             mem = self._alloc(size).mem
*/
  __pyx_v_oom_error = 0;

  /* "cupy/cuda/memory.pyx":1535
 *         mem = None
 *         oom_error = False
 *         try:             # <<<<<<<<<<<<<<
 *             mem = self._alloc(size).mem
 *         except CUDARuntimeError as e:
*/
  /*try:*/ {
    {
      __Pyx_PyThreadState_declare
      __Pyx_PyThreadState_assign
      __Pyx_ExceptionSave(&__pyx_t_12, &__pyx_t_13, &__pyx_t_14);
      __Pyx_XGOTREF(__pyx_t_12);
      __Pyx_XGOTREF(__pyx_t_13);
      __Pyx_XGOTREF(__pyx_t_14);
      /*try:*/ {

        /* "cupy/cuda/memory.pyx":1536
 *         oom_error = False
 *         try:
 *             mem = self._alloc(size).mem             # <<<<<<<<<<<<<<
 *         except CUDARuntimeError as e:
 *             if e.status != runtime.errorMemoryAllocation:
*/
        __pyx_t_3 = ((PyObject *)__pyx_f_4cupy_4cuda_6memory_22SingleDeviceMemoryPool__alloc(__pyx_v_self, __pyx_v_size)); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 1536, __pyx_L13_error)
        __Pyx_GOTREF(__pyx_t_3);
        __pyx_t_6 = ((PyObject *)((struct __pyx_obj_4cupy_4cuda_6memory_MemoryPointer *)__pyx_t_3)->mem);
        __Pyx_INCREF(__pyx_t_6);
        __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
        __Pyx_DECREF_SET(__pyx_v_mem, ((struct __pyx_obj_4cupy_4cuda_6memory_BaseMemory *)__pyx_t_6));
        __pyx_t_6 = 0;

        /* "cupy/cuda/memory.pyx":1535
 *         mem = None
 *         oom_error = False
 *         try:             # <<<<<<<<<<<<<<
 *             mem = self._alloc(size).mem
 *         except CUDARuntimeError as e:
*/
      }
      __Pyx_XDECREF(__pyx_t_12); __pyx_t_12 = 0;
      __Pyx_XDECREF(__pyx_t_13); __pyx_t_13 = 0;
      __Pyx_XDECREF(__pyx_t_14); __pyx_t_14 = 0;
      goto __pyx_L18_try_end;
      __pyx_L13_error:;
      __Pyx_XDECREF(__pyx_t_10); __pyx_t_10 = 0;
      __Pyx_XDECREF(__pyx_t_11); __pyx_t_11 = 0;
      __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
      __Pyx_XDECREF(__pyx_t_6); __pyx_t_6 = 0;
      __Pyx_XDECREF(__pyx_t_7); __pyx_t_7 = 0;
      __Pyx_XDECREF(__pyx_t_8); __pyx_t_8 = 0;

      /* "cupy/cuda/memory.pyx":1537
 *         try:
 *             mem = self._alloc(size).mem
 *         except CUDARuntimeError as e:             # <<<<<<<<<<<<<<
 *             if e.status != runtime.errorMemoryAllocation:
 *                 raise
*/
      __Pyx_ErrFetch(&__pyx_t_6, &__pyx_t_3, &__pyx_t_11);
      __Pyx_GetModuleGlobalName(__pyx_t_10, __pyx_mstate_global->__pyx_n_u_CUDARuntimeError); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 1537, __pyx_L15_except_error)
      __Pyx_GOTREF(__pyx_t_10);
      __pyx_t_15 = __Pyx_PyErr_GivenExceptionMatches(__pyx_t_6, __pyx_t_10);
      __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
      __Pyx_ErrRestore(__pyx_t_6, __pyx_t_3, __pyx_t_11);
      __pyx_t_6 = 0; __pyx_t_3 = 0; __pyx_t_11 = 0;
      if (__pyx_t_15) {
        __Pyx_AddTraceback("cupy.cuda.memory.SingleDeviceMemoryPool._try_malloc", __pyx_clineno, __pyx_lineno, __pyx_filename);
        if (__Pyx_GetException(&__pyx_t_11, &__pyx_t_3, &__pyx_t_6) < 0) __PYX_ERR(0, 1537, __pyx_L15_except_error)
        __Pyx_XGOTREF(__pyx_t_11);
        __Pyx_XGOTREF(__pyx_t_3);
        __Pyx_XGOTREF(__pyx_t_6);
        __Pyx_INCREF(__pyx_t_3);
        __pyx_v_e = __pyx_t_3;
        /*try:*/ {

          /* "cupy/cuda/memory.pyx":1538
 *             mem = self._alloc(size).mem
 *         except CUDARuntimeError as e:
 *             if e.status != runtime.errorMemoryAllocation:             # <<<<<<<<<<<<<<
 *                 raise
 * 
*/
          __pyx_t_10 = __Pyx_PyObject_GetAttrStr(__pyx_v_e, __pyx_mstate_global->__pyx_n_u_status); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 1538, __pyx_L24_error)
          __Pyx_GOTREF(__pyx_t_10);
          __pyx_t_7 = __Pyx_PyLong_From_int(__pyx_v_13cupy_backends_4cuda_3api_7runtime_errorMemoryAllocation); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 1538, __pyx_L24_error)
          __Pyx_GOTREF(__pyx_t_7);
          __pyx_t_8 = PyObject_RichCompare(__pyx_t_10, __pyx_t_7, Py_NE); __Pyx_XGOTREF(__pyx_t_8); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 1538, __pyx_L24_error)
          __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
          __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
          __pyx_t_2 = __Pyx_PyObject_IsTrue(__pyx_t_8); if (unlikely((__pyx_t_2 < 0))) __PYX_ERR(0, 1538, __pyx_L24_error)
          __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
          if (unlikely(__pyx_t_2)) {

            /* "cupy/cuda/memory.pyx":1539
 *         except CUDARuntimeError as e:
 *             if e.status != runtime.errorMemoryAllocation:
 *                 raise             # <<<<<<<<<<<<<<
 * 
 *             self.free_all_blocks(None)
*/
            __Pyx_GIVEREF(__pyx_t_11);
            __Pyx_GIVEREF(__pyx_t_3);
            __Pyx_XGIVEREF(__pyx_t_6);
            __Pyx_ErrRestoreWithState(__pyx_t_11, __pyx_t_3, __pyx_t_6);
            __pyx_t_11 = 0;  __pyx_t_3 = 0;  __pyx_t_6 = 0; 
            __PYX_ERR(0, 1539, __pyx_L24_error)

            /* "cupy/cuda/memory.pyx":1538
 *             mem = self._alloc(size).mem
 *         except CUDARuntimeError as e:
 *             if e.status != runtime.errorMemoryAllocation:             # <<<<<<<<<<<<<<
 *                 raise
 * 
*/
          }

          /* "cupy/cuda/memory.pyx":1541
 *                 raise
 * 
 *             self.free_all_blocks(None)             # <<<<<<<<<<<<<<
 *             try:
 *                 mem = self._alloc(size).mem
*/
          __pyx_t_4.__pyx_n = 1;
          __pyx_t_4.stream = Py_None;
          __pyx_t_8 = __pyx_f_4cupy_4cuda_6memory_22SingleDeviceMemoryPool_free_all_blocks(__pyx_v_self, 0, &__pyx_t_4); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 1541, __pyx_L24_error)
          __Pyx_GOTREF(__pyx_t_8);
          __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;

          /* "cupy/cuda/memory.pyx":1542
 * 
 *             self.free_all_blocks(None)
 *             try:             # <<<<<<<<<<<<<<
 *                 mem = self._alloc(size).mem
 *             except CUDARuntimeError as e:
*/
          {
            __Pyx_PyThreadState_declare
            __Pyx_PyThreadState_assign
            __Pyx_ExceptionSave(&__pyx_t_16, &__pyx_t_17, &__pyx_t_18);
            __Pyx_XGOTREF(__pyx_t_16);
            __Pyx_XGOTREF(__pyx_t_17);
            __Pyx_XGOTREF(__pyx_t_18);
            /*try:*/ {

              /* "cupy/cuda/memory.pyx":1543
 *             self.free_all_blocks(None)
 *             try:
 *                 mem = self._alloc(size).mem             # <<<<<<<<<<<<<<
 *             except CUDARuntimeError as e:
 *                 if e.status != runtime.errorMemoryAllocation:
*/
              __pyx_t_8 = ((PyObject *)__pyx_f_4cupy_4cuda_6memory_22SingleDeviceMemoryPool__alloc(__pyx_v_self, __pyx_v_size)); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 1543, __pyx_L27_error)
              __Pyx_GOTREF(__pyx_t_8);
              __pyx_t_7 = ((PyObject *)((struct __pyx_obj_4cupy_4cuda_6memory_MemoryPointer *)__pyx_t_8)->mem);
              __Pyx_INCREF(__pyx_t_7);
              __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
              __Pyx_DECREF_SET(__pyx_v_mem, ((struct __pyx_obj_4cupy_4cuda_6memory_BaseMemory *)__pyx_t_7));
              __pyx_t_7 = 0;

              /* "cupy/cuda/memory.pyx":1542
 * 
 *             self.free_all_blocks(None)
 *             try:             # <<<<<<<<<<<<<<
 *                 mem = self._alloc(size).mem
 *             except CUDARuntimeError as e:
*/
            }
            __Pyx_XDECREF(__pyx_t_16); __pyx_t_16 = 0;
            __Pyx_XDECREF(__pyx_t_17); __pyx_t_17 = 0;
            __Pyx_XDECREF(__pyx_t_18); __pyx_t_18 = 0;
            goto __pyx_L34_try_end;
            __pyx_L27_error:;
            __Pyx_XDECREF(__pyx_t_10); __pyx_t_10 = 0;
            __Pyx_XDECREF(__pyx_t_7); __pyx_t_7 = 0;
            __Pyx_XDECREF(__pyx_t_8); __pyx_t_8 = 0;

            /* "cupy/cuda/memory.pyx":1544
 *             try:
 *                 mem = self._alloc(size).mem
 *             except CUDARuntimeError as e:             # <<<<<<<<<<<<<<
 *                 if e.status != runtime.errorMemoryAllocation:
 *                     raise
*/
            __Pyx_ErrFetch(&__pyx_t_7, &__pyx_t_8, &__pyx_t_10);
            __Pyx_GetModuleGlobalName(__pyx_t_19, __pyx_mstate_global->__pyx_n_u_CUDARuntimeError); if (unlikely(!__pyx_t_19)) __PYX_ERR(0, 1544, __pyx_L29_except_error)
            __Pyx_GOTREF(__pyx_t_19);
            __pyx_t_15 = __Pyx_PyErr_GivenExceptionMatches(__pyx_t_7, __pyx_t_19);
            __Pyx_DECREF(__pyx_t_19); __pyx_t_19 = 0;
            __Pyx_ErrRestore(__pyx_t_7, __pyx_t_8, __pyx_t_10);
            __pyx_t_7 = 0; __pyx_t_8 = 0; __pyx_t_10 = 0;
            if (__pyx_t_15) {
              __Pyx_AddTraceback("cupy.cuda.memory.SingleDeviceMemoryPool._try_malloc", __pyx_clineno, __pyx_lineno, __pyx_filename);
              if (__Pyx_GetException(&__pyx_t_10, &__pyx_t_8, &__pyx_t_7) < 0) __PYX_ERR(0, 1544, __pyx_L29_except_error)
              __Pyx_XGOTREF(__pyx_t_10);
              __Pyx_XGOTREF(__pyx_t_8);
              __Pyx_XGOTREF(__pyx_t_7);
              __Pyx_INCREF(__pyx_t_8);
              __Pyx_DECREF_SET(__pyx_v_e, __pyx_t_8);
              /*try:*/ {

                /* "cupy/cuda/memory.pyx":1545
 *                 mem = self._alloc(size).mem
 *             except CUDARuntimeError as e:
 *                 if e.status != runtime.errorMemoryAllocation:             # <<<<<<<<<<<<<<
 *                     raise
 *                 gc.collect()
*/
                __pyx_t_19 = __Pyx_PyObject_GetAttrStr(__pyx_v_e, __pyx_mstate_global->__pyx_n_u_status); if (unlikely(!__pyx_t_19)) __PYX_ERR(0, 1545, __pyx_L40_error)
                __Pyx_GOTREF(__pyx_t_19);
                __pyx_t_20 = __Pyx_PyLong_From_int(__pyx_v_13cupy_backends_4cuda_3api_7runtime_errorMemoryAllocation); if (unlikely(!__pyx_t_20)) __PYX_ERR(0, 1545, __pyx_L40_error)
                __Pyx_GOTREF(__pyx_t_20);
                __pyx_t_21 = PyObject_RichCompare(__pyx_t_19, __pyx_t_20, Py_NE); __Pyx_XGOTREF(__pyx_t_21); if (unlikely(!__pyx_t_21)) __PYX_ERR(0, 1545, __pyx_L40_error)
                __Pyx_DECREF(__pyx_t_19); __pyx_t_19 = 0;
                __Pyx_DECREF(__pyx_t_20); __pyx_t_20 = 0;
                __pyx_t_2 = __Pyx_PyObject_IsTrue(__pyx_t_21); if (unlikely((__pyx_t_2 < 0))) __PYX_ERR(0, 1545, __pyx_L40_error)
                __Pyx_DECREF(__pyx_t_21); __pyx_t_21 = 0;
                if (unlikely(__pyx_t_2)) {

                  /* "cupy/cuda/memory.pyx":1546
 *             except CUDARuntimeError as e:
 *                 if e.status != runtime.errorMemoryAllocation:
 *                     raise             # <<<<<<<<<<<<<<
 *                 gc.collect()
 *                 gc.collect()
*/
                  __Pyx_GIVEREF(__pyx_t_10);
                  __Pyx_GIVEREF(__pyx_t_8);
                  __Pyx_XGIVEREF(__pyx_t_7);
                  __Pyx_ErrRestoreWithState(__pyx_t_10, __pyx_t_8, __pyx_t_7);
                  __pyx_t_10 = 0;  __pyx_t_8 = 0;  __pyx_t_7 = 0; 
                  __PYX_ERR(0, 1546, __pyx_L40_error)

                  /* "cupy/cuda/memory.pyx":1545
 *                 mem = self._alloc(size).mem
 *             except CUDARuntimeError as e:
 *                 if e.status != runtime.errorMemoryAllocation:             # <<<<<<<<<<<<<<
 *                     raise
 *                 gc.collect()
*/
                }

                /* "cupy/cuda/memory.pyx":1547
 *                 if e.status != runtime.errorMemoryAllocation:
 *                     raise
 *                 gc.collect()             # <<<<<<<<<<<<<<
 *                 gc.collect()
 *                 self.free_all_blocks(None)
*/
                __pyx_t_20 = NULL;
                __Pyx_GetModuleGlobalName(__pyx_t_19, __pyx_mstate_global->__pyx_n_u_gc); if (unlikely(!__pyx_t_19)) __PYX_ERR(0, 1547, __pyx_L40_error)
                __Pyx_GOTREF(__pyx_t_19);
                __pyx_t_22 = __Pyx_PyObject_GetAttrStr(__pyx_t_19, __pyx_mstate_global->__pyx_n_u_collect); if (unlikely(!__pyx_t_22)) __PYX_ERR(0, 1547, __pyx_L40_error)
                __Pyx_GOTREF(__pyx_t_22);
                __Pyx_DECREF(__pyx_t_19); __pyx_t_19 = 0;
                __pyx_t_9 = 1;
                #if CYTHON_UNPACK_METHODS
                if (unlikely(PyMethod_Check(__pyx_t_22))) {
                  __pyx_t_20 = PyMethod_GET_SELF(__pyx_t_22);
                  assert(__pyx_t_20);
                  PyObject* __pyx__function = PyMethod_GET_FUNCTION(__pyx_t_22);
                  __Pyx_INCREF(__pyx_t_20);
                  __Pyx_INCREF(__pyx__function);
                  __Pyx_DECREF_SET(__pyx_t_22, __pyx__function);
                  __pyx_t_9 = 0;
                }
                #endif
                {
                  PyObject *__pyx_callargs[2] = {__pyx_t_20, NULL};
                  __pyx_t_21 = __Pyx_PyObject_FastCall((PyObject*)__pyx_t_22, __pyx_callargs+__pyx_t_9, (1-__pyx_t_9) | (__pyx_t_9*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
                  __Pyx_XDECREF(__pyx_t_20); __pyx_t_20 = 0;
                  __Pyx_DECREF(__pyx_t_22); __pyx_t_22 = 0;
                  if (unlikely(!__pyx_t_21)) __PYX_ERR(0, 1547, __pyx_L40_error)
                  __Pyx_GOTREF(__pyx_t_21);
                }
                __Pyx_DECREF(__pyx_t_21); __pyx_t_21 = 0;

                /* "cupy/cuda/memory.pyx":1548
 *                     raise
 *                 gc.collect()
 *                 gc.collect()             # <<<<<<<<<<<<<<
 *                 self.free_all_blocks(None)
 *                 try:
*/
                __pyx_t_22 = NULL;
                __Pyx_GetModuleGlobalName(__pyx_t_20, __pyx_mstate_global->__pyx_n_u_gc); if (unlikely(!__pyx_t_20)) __PYX_ERR(0, 1548, __pyx_L40_error)
                __Pyx_GOTREF(__pyx_t_20);
                __pyx_t_19 = __Pyx_PyObject_GetAttrStr(__pyx_t_20, __pyx_mstate_global->__pyx_n_u_collect); if (unlikely(!__pyx_t_19)) __PYX_ERR(0, 1548, __pyx_L40_error)
                __Pyx_GOTREF(__pyx_t_19);
                __Pyx_DECREF(__pyx_t_20); __pyx_t_20 = 0;
                __pyx_t_9 = 1;
                #if CYTHON_UNPACK_METHODS
                if (unlikely(PyMethod_Check(__pyx_t_19))) {
                  __pyx_t_22 = PyMethod_GET_SELF(__pyx_t_19);
                  assert(__pyx_t_22);
                  PyObject* __pyx__function = PyMethod_GET_FUNCTION(__pyx_t_19);
                  __Pyx_INCREF(__pyx_t_22);
                  __Pyx_INCREF(__pyx__function);
                  __Pyx_DECREF_SET(__pyx_t_19, __pyx__function);
                  __pyx_t_9 = 0;
                }
                #endif
                {
                  PyObject *__pyx_callargs[2] = {__pyx_t_22, NULL};
                  __pyx_t_21 = __Pyx_PyObject_FastCall((PyObject*)__pyx_t_19, __pyx_callargs+__pyx_t_9, (1-__pyx_t_9) | (__pyx_t_9*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
                  __Pyx_XDECREF(__pyx_t_22); __pyx_t_22 = 0;
                  __Pyx_DECREF(__pyx_t_19); __pyx_t_19 = 0;
                  if (unlikely(!__pyx_t_21)) __PYX_ERR(0, 1548, __pyx_L40_error)
                  __Pyx_GOTREF(__pyx_t_21);
                }
                __Pyx_DECREF(__pyx_t_21); __pyx_t_21 = 0;

                /* "cupy/cuda/memory.pyx":1549
 *                 gc.collect()
 *                 gc.collect()
 *                 self.free_all_blocks(None)             # <<<<<<<<<<<<<<
 *                 try:
 *                     mem = self._alloc(size).mem
*/
                __pyx_t_4.__pyx_n = 1;
                __pyx_t_4.stream = Py_None;
                __pyx_t_21 = __pyx_f_4cupy_4cuda_6memory_22SingleDeviceMemoryPool_free_all_blocks(__pyx_v_self, 0, &__pyx_t_4); if (unlikely(!__pyx_t_21)) __PYX_ERR(0, 1549, __pyx_L40_error)
                __Pyx_GOTREF(__pyx_t_21);
                __Pyx_DECREF(__pyx_t_21); __pyx_t_21 = 0;

                /* "cupy/cuda/memory.pyx":1550
 *                 gc.collect()
 *                 self.free_all_blocks(None)
 *                 try:             # <<<<<<<<<<<<<<
 *                     mem = self._alloc(size).mem
 *                 except CUDARuntimeError as e:
*/
                {
                  __Pyx_PyThreadState_declare
                  __Pyx_PyThreadState_assign
                  __Pyx_ExceptionSave(&__pyx_t_23, &__pyx_t_24, &__pyx_t_25);
                  __Pyx_XGOTREF(__pyx_t_23);
                  __Pyx_XGOTREF(__pyx_t_24);
                  __Pyx_XGOTREF(__pyx_t_25);
                  /*try:*/ {

                    /* "cupy/cuda/memory.pyx":1551
 *                 self.free_all_blocks(None)
 *                 try:
 *                     mem = self._alloc(size).mem             # <<<<<<<<<<<<<<
 *                 except CUDARuntimeError as e:
 *                     if e.status != runtime.errorMemoryAllocation:
*/
                    __pyx_t_21 = ((PyObject *)__pyx_f_4cupy_4cuda_6memory_22SingleDeviceMemoryPool__alloc(__pyx_v_self, __pyx_v_size)); if (unlikely(!__pyx_t_21)) __PYX_ERR(0, 1551, __pyx_L43_error)
                    __Pyx_GOTREF(__pyx_t_21);
                    __pyx_t_19 = ((PyObject *)((struct __pyx_obj_4cupy_4cuda_6memory_MemoryPointer *)__pyx_t_21)->mem);
                    __Pyx_INCREF(__pyx_t_19);
                    __Pyx_DECREF(__pyx_t_21); __pyx_t_21 = 0;
                    __Pyx_DECREF_SET(__pyx_v_mem, ((struct __pyx_obj_4cupy_4cuda_6memory_BaseMemory *)__pyx_t_19));
                    __pyx_t_19 = 0;

                    /* "cupy/cuda/memory.pyx":1550
 *                 gc.collect()
 *                 self.free_all_blocks(None)
 *                 try:             # <<<<<<<<<<<<<<
 *                     mem = self._alloc(size).mem
 *                 except CUDARuntimeError as e:
*/
                  }
                  __Pyx_XDECREF(__pyx_t_23); __pyx_t_23 = 0;
                  __Pyx_XDECREF(__pyx_t_24); __pyx_t_24 = 0;
                  __Pyx_XDECREF(__pyx_t_25); __pyx_t_25 = 0;
                  goto __pyx_L50_try_end;
                  __pyx_L43_error:;
                  __Pyx_XDECREF(__pyx_t_19); __pyx_t_19 = 0;
                  __Pyx_XDECREF(__pyx_t_20); __pyx_t_20 = 0;
                  __Pyx_XDECREF(__pyx_t_21); __pyx_t_21 = 0;
                  __Pyx_XDECREF(__pyx_t_22); __pyx_t_22 = 0;

                  /* "cupy/cuda/memory.pyx":1552
 *                 try:
 *                     mem = self._alloc(size).mem
 *                 except CUDARuntimeError as e:             # <<<<<<<<<<<<<<
 *                     if e.status != runtime.errorMemoryAllocation:
 *                         raise
*/
                  __Pyx_ErrFetch(&__pyx_t_19, &__pyx_t_21, &__pyx_t_22);
                  __Pyx_GetModuleGlobalName(__pyx_t_20, __pyx_mstate_global->__pyx_n_u_CUDARuntimeError); if (unlikely(!__pyx_t_20)) __PYX_ERR(0, 1552, __pyx_L45_except_error)
                  __Pyx_GOTREF(__pyx_t_20);
                  __pyx_t_15 = __Pyx_PyErr_GivenExceptionMatches(__pyx_t_19, __pyx_t_20);
                  __Pyx_DECREF(__pyx_t_20); __pyx_t_20 = 0;
                  __Pyx_ErrRestore(__pyx_t_19, __pyx_t_21, __pyx_t_22);
                  __pyx_t_19 = 0; __pyx_t_21 = 0; __pyx_t_22 = 0;
                  if (__pyx_t_15) {
                    __Pyx_AddTraceback("cupy.cuda.memory.SingleDeviceMemoryPool._try_malloc", __pyx_clineno, __pyx_lineno, __pyx_filename);
                    if (__Pyx_GetException(&__pyx_t_22, &__pyx_t_21, &__pyx_t_19) < 0) __PYX_ERR(0, 1552, __pyx_L45_except_error)
                    __Pyx_XGOTREF(__pyx_t_22);
                    __Pyx_XGOTREF(__pyx_t_21);
                    __Pyx_XGOTREF(__pyx_t_19);
                    __Pyx_INCREF(__pyx_t_21);
                    __Pyx_DECREF_SET(__pyx_v_e, __pyx_t_21);
                    /*try:*/ {

                      /* "cupy/cuda/memory.pyx":1553
 *                     mem = self._alloc(size).mem
 *                 except CUDARuntimeError as e:
 *                     if e.status != runtime.errorMemoryAllocation:             # <<<<<<<<<<<<<<
 *                         raise
 *                     oom_error = True
*/
                      __pyx_t_20 = __Pyx_PyObject_GetAttrStr(__pyx_v_e, __pyx_mstate_global->__pyx_n_u_status); if (unlikely(!__pyx_t_20)) __PYX_ERR(0, 1553, __pyx_L56_error)
                      __Pyx_GOTREF(__pyx_t_20);
                      __pyx_t_26 = __Pyx_PyLong_From_int(__pyx_v_13cupy_backends_4cuda_3api_7runtime_errorMemoryAllocation); if (unlikely(!__pyx_t_26)) __PYX_ERR(0, 1553, __pyx_L56_error)
                      __Pyx_GOTREF(__pyx_t_26);
                      __pyx_t_27 = PyObject_RichCompare(__pyx_t_20, __pyx_t_26, Py_NE); __Pyx_XGOTREF(__pyx_t_27); if (unlikely(!__pyx_t_27)) __PYX_ERR(0, 1553, __pyx_L56_error)
                      __Pyx_DECREF(__pyx_t_20); __pyx_t_20 = 0;
                      __Pyx_DECREF(__pyx_t_26); __pyx_t_26 = 0;
                      __pyx_t_2 = __Pyx_PyObject_IsTrue(__pyx_t_27); if (unlikely((__pyx_t_2 < 0))) __PYX_ERR(0, 1553, __pyx_L56_error)
                      __Pyx_DECREF(__pyx_t_27); __pyx_t_27 = 0;
                      if (unlikely(__pyx_t_2)) {

                        /* "cupy/cuda/memory.pyx":1554
 *                 except CUDARuntimeError as e:
 *                     if e.status != runtime.errorMemoryAllocation:
 *                         raise             # <<<<<<<<<<<<<<
 *                     oom_error = True
 *         finally:
*/
                        __Pyx_GIVEREF(__pyx_t_22);
                        __Pyx_GIVEREF(__pyx_t_21);
                        __Pyx_XGIVEREF(__pyx_t_19);
                        __Pyx_ErrRestoreWithState(__pyx_t_22, __pyx_t_21, __pyx_t_19);
                        __pyx_t_22 = 0;  __pyx_t_21 = 0;  __pyx_t_19 = 0; 
                        __PYX_ERR(0, 1554, __pyx_L56_error)

                        /* "cupy/cuda/memory.pyx":1553
 *                     mem = self._alloc(size).mem
 *                 except CUDARuntimeError as e:
 *                     if e.status != runtime.errorMemoryAllocation:             # <<<<<<<<<<<<<<
 *                         raise
 *                     oom_error = True
*/
                      }

                      /* "cupy/cuda/memory.pyx":1555
 *                     if e.status != runtime.errorMemoryAllocation:
 *                         raise
 *                     oom_error = True             # <<<<<<<<<<<<<<
 *         finally:
 *             if mem is None:
*/
                      __pyx_v_oom_error = 1;
                    }

                    /* "cupy/cuda/memory.pyx":1552
 *                 try:
 *                     mem = self._alloc(size).mem
 *                 except CUDARuntimeError as e:             # <<<<<<<<<<<<<<
 *                     if e.status != runtime.errorMemoryAllocation:
 *                         raise
*/
                    /*finally:*/ {
                      /*normal exit:*/{
                        __Pyx_DECREF(__pyx_v_e); __pyx_v_e = 0;
                        goto __pyx_L57;
                      }
                      __pyx_L56_error:;
                      /*exception exit:*/{
                        __Pyx_PyThreadState_declare
                        __Pyx_PyThreadState_assign
                        __pyx_t_30 = 0; __pyx_t_31 = 0; __pyx_t_32 = 0; __pyx_t_33 = 0; __pyx_t_34 = 0; __pyx_t_35 = 0;
                        __Pyx_XDECREF(__pyx_t_20); __pyx_t_20 = 0;
                        __Pyx_XDECREF(__pyx_t_26); __pyx_t_26 = 0;
                        __Pyx_XDECREF(__pyx_t_27); __pyx_t_27 = 0;
                         __Pyx_ExceptionSwap(&__pyx_t_33, &__pyx_t_34, &__pyx_t_35);
                        if ( unlikely(__Pyx_GetException(&__pyx_t_30, &__pyx_t_31, &__pyx_t_32) < 0)) __Pyx_ErrFetch(&__pyx_t_30, &__pyx_t_31, &__pyx_t_32);
                        __Pyx_XGOTREF(__pyx_t_30);
                        __Pyx_XGOTREF(__pyx_t_31);
                        __Pyx_XGOTREF(__pyx_t_32);
                        __Pyx_XGOTREF(__pyx_t_33);
                        __Pyx_XGOTREF(__pyx_t_34);
                        __Pyx_XGOTREF(__pyx_t_35);
                        __pyx_t_15 = __pyx_lineno; __pyx_t_28 = __pyx_clineno; __pyx_t_29 = __pyx_filename;
                        {
                          __Pyx_DECREF(__pyx_v_e); __pyx_v_e = 0;
                        }
                        __Pyx_XGIVEREF(__pyx_t_33);
                        __Pyx_XGIVEREF(__pyx_t_34);
                        __Pyx_XGIVEREF(__pyx_t_35);
                        __Pyx_ExceptionReset(__pyx_t_33, __pyx_t_34, __pyx_t_35);
                        __Pyx_XGIVEREF(__pyx_t_30);
                        __Pyx_XGIVEREF(__pyx_t_31);
                        __Pyx_XGIVEREF(__pyx_t_32);
                        __Pyx_ErrRestore(__pyx_t_30, __pyx_t_31, __pyx_t_32);
                        __pyx_t_30 = 0; __pyx_t_31 = 0; __pyx_t_32 = 0; __pyx_t_33 = 0; __pyx_t_34 = 0; __pyx_t_35 = 0;
                        __pyx_lineno = __pyx_t_15; __pyx_clineno = __pyx_t_28; __pyx_filename = __pyx_t_29;
                        goto __pyx_L45_except_error;
                      }
                      __pyx_L57:;
                    }
                    __Pyx_XDECREF(__pyx_t_22); __pyx_t_22 = 0;
                    __Pyx_XDECREF(__pyx_t_21); __pyx_t_21 = 0;
                    __Pyx_XDECREF(__pyx_t_19); __pyx_t_19 = 0;
                    goto __pyx_L44_exception_handled;
                  }
                  goto __pyx_L45_except_error;

                  /* "cupy/cuda/memory.pyx":1550
 *                 gc.collect()
 *                 self.free_all_blocks(None)
 *                 try:             # <<<<<<<<<<<<<<
 *                     mem = self._alloc(size).mem
 *                 except CUDARuntimeError as e:
*/
                  __pyx_L45_except_error:;
                  __Pyx_XGIVEREF(__pyx_t_23);
                  __Pyx_XGIVEREF(__pyx_t_24);
                  __Pyx_XGIVEREF(__pyx_t_25);
                  __Pyx_ExceptionReset(__pyx_t_23, __pyx_t_24, __pyx_t_25);
                  goto __pyx_L40_error;
                  __pyx_L44_exception_handled:;
                  __Pyx_XGIVEREF(__pyx_t_23);
                  __Pyx_XGIVEREF(__pyx_t_24);
                  __Pyx_XGIVEREF(__pyx_t_25);
                  __Pyx_ExceptionReset(__pyx_t_23, __pyx_t_24, __pyx_t_25);
                  __pyx_L50_try_end:;
                }
              }

              /* "cupy/cuda/memory.pyx":1544
 *             try:
 *                 mem = self._alloc(size).mem
 *             except CUDARuntimeError as e:             # <<<<<<<<<<<<<<
 *                 if e.status != runtime.errorMemoryAllocation:
 *                     raise
*/
              /*finally:*/ {
                /*normal exit:*/{
                  __Pyx_XDECREF(__pyx_v_e); __pyx_v_e = 0;
                  goto __pyx_L41;
                }
                __pyx_L40_error:;
                /*exception exit:*/{
                  __Pyx_PyThreadState_declare
                  __Pyx_PyThreadState_assign
                  __pyx_t_25 = 0; __pyx_t_24 = 0; __pyx_t_23 = 0; __pyx_t_35 = 0; __pyx_t_34 = 0; __pyx_t_33 = 0;
                  __Pyx_XDECREF(__pyx_t_19); __pyx_t_19 = 0;
                  __Pyx_XDECREF(__pyx_t_20); __pyx_t_20 = 0;
                  __Pyx_XDECREF(__pyx_t_21); __pyx_t_21 = 0;
                  __Pyx_XDECREF(__pyx_t_22); __pyx_t_22 = 0;
                  __Pyx_XDECREF(__pyx_t_26); __pyx_t_26 = 0;
                  __Pyx_XDECREF(__pyx_t_27); __pyx_t_27 = 0;
                   __Pyx_ExceptionSwap(&__pyx_t_35, &__pyx_t_34, &__pyx_t_33);
                  if ( unlikely(__Pyx_GetException(&__pyx_t_25, &__pyx_t_24, &__pyx_t_23) < 0)) __Pyx_ErrFetch(&__pyx_t_25, &__pyx_t_24, &__pyx_t_23);
                  __Pyx_XGOTREF(__pyx_t_25);
                  __Pyx_XGOTREF(__pyx_t_24);
                  __Pyx_XGOTREF(__pyx_t_23);
                  __Pyx_XGOTREF(__pyx_t_35);
                  __Pyx_XGOTREF(__pyx_t_34);
                  __Pyx_XGOTREF(__pyx_t_33);
                  __pyx_t_28 = __pyx_lineno; __pyx_t_15 = __pyx_clineno; __pyx_t_36 = __pyx_filename;
                  {
                    __Pyx_XDECREF(__pyx_v_e); __pyx_v_e = 0;
                  }
                  __Pyx_XGIVEREF(__pyx_t_35);
                  __Pyx_XGIVEREF(__pyx_t_34);
                  __Pyx_XGIVEREF(__pyx_t_33);
                  __Pyx_ExceptionReset(__pyx_t_35, __pyx_t_34, __pyx_t_33);
                  __Pyx_XGIVEREF(__pyx_t_25);
                  __Pyx_XGIVEREF(__pyx_t_24);
                  __Pyx_XGIVEREF(__pyx_t_23);
                  __Pyx_ErrRestore(__pyx_t_25, __pyx_t_24, __pyx_t_23);
                  __pyx_t_25 = 0; __pyx_t_24 = 0; __pyx_t_23 = 0; __pyx_t_35 = 0; __pyx_t_34 = 0; __pyx_t_33 = 0;
                  __pyx_lineno = __pyx_t_28; __pyx_clineno = __pyx_t_15; __pyx_filename = __pyx_t_36;
                  goto __pyx_L29_except_error;
                }
                __pyx_L41:;
              }
              __Pyx_XDECREF(__pyx_t_10); __pyx_t_10 = 0;
              __Pyx_XDECREF(__pyx_t_8); __pyx_t_8 = 0;
              __Pyx_XDECREF(__pyx_t_7); __pyx_t_7 = 0;
              goto __pyx_L28_exception_handled;
            }
            goto __pyx_L29_except_error;

            /* "cupy/cuda/memory.pyx":1542
 * 
 *             self.free_all_blocks(None)
 *             try:             # <<<<<<<<<<<<<<
 *                 mem = self._alloc(size).mem
 *             except CUDARuntimeError as e:
*/
            __pyx_L29_except_error:;
            __Pyx_XGIVEREF(__pyx_t_16);
            __Pyx_XGIVEREF(__pyx_t_17);
            __Pyx_XGIVEREF(__pyx_t_18);
            __Pyx_ExceptionReset(__pyx_t_16, __pyx_t_17, __pyx_t_18);
            goto __pyx_L24_error;
            __pyx_L28_exception_handled:;
            __Pyx_XGIVEREF(__pyx_t_16);
            __Pyx_XGIVEREF(__pyx_t_17);
            __Pyx_XGIVEREF(__pyx_t_18);
            __Pyx_ExceptionReset(__pyx_t_16, __pyx_t_17, __pyx_t_18);
            __pyx_L34_try_end:;
          }
        }

        /* "cupy/cuda/memory.pyx":1537
 *         try:
 *             mem = self._alloc(size).mem
 *         except CUDARuntimeError as e:             # <<<<<<<<<<<<<<
 *             if e.status != runtime.errorMemoryAllocation:
 *                 raise
*/
        /*finally:*/ {
          /*normal exit:*/{
            __Pyx_XDECREF(__pyx_v_e); __pyx_v_e = 0;
            goto __pyx_L25;
          }
          __pyx_L24_error:;
          /*exception exit:*/{
            __Pyx_PyThreadState_declare
            __Pyx_PyThreadState_assign
            __pyx_t_18 = 0; __pyx_t_17 = 0; __pyx_t_16 = 0; __pyx_t_33 = 0; __pyx_t_34 = 0; __pyx_t_35 = 0;
            __Pyx_XDECREF(__pyx_t_10); __pyx_t_10 = 0;
            __Pyx_XDECREF(__pyx_t_19); __pyx_t_19 = 0;
            __Pyx_XDECREF(__pyx_t_20); __pyx_t_20 = 0;
            __Pyx_XDECREF(__pyx_t_21); __pyx_t_21 = 0;
            __Pyx_XDECREF(__pyx_t_22); __pyx_t_22 = 0;
            __Pyx_XDECREF(__pyx_t_26); __pyx_t_26 = 0;
            __Pyx_XDECREF(__pyx_t_27); __pyx_t_27 = 0;
            __Pyx_XDECREF(__pyx_t_7); __pyx_t_7 = 0;
            __Pyx_XDECREF(__pyx_t_8); __pyx_t_8 = 0;
             __Pyx_ExceptionSwap(&__pyx_t_33, &__pyx_t_34, &__pyx_t_35);
            if ( unlikely(__Pyx_GetException(&__pyx_t_18, &__pyx_t_17, &__pyx_t_16) < 0)) __Pyx_ErrFetch(&__pyx_t_18, &__pyx_t_17, &__pyx_t_16);
            __Pyx_XGOTREF(__pyx_t_18);
            __Pyx_XGOTREF(__pyx_t_17);
            __Pyx_XGOTREF(__pyx_t_16);
            __Pyx_XGOTREF(__pyx_t_33);
            __Pyx_XGOTREF(__pyx_t_34);
            __Pyx_XGOTREF(__pyx_t_35);
            __pyx_t_15 = __pyx_lineno; __pyx_t_28 = __pyx_clineno; __pyx_t_37 = __pyx_filename;
            {
              __Pyx_XDECREF(__pyx_v_e); __pyx_v_e = 0;
            }
            __Pyx_XGIVEREF(__pyx_t_33);
            __Pyx_XGIVEREF(__pyx_t_34);
            __Pyx_XGIVEREF(__pyx_t_35);
            __Pyx_ExceptionReset(__pyx_t_33, __pyx_t_34, __pyx_t_35);
            __Pyx_XGIVEREF(__pyx_t_18);
            __Pyx_XGIVEREF(__pyx_t_17);
            __Pyx_XGIVEREF(__pyx_t_16);
            __Pyx_ErrRestore(__pyx_t_18, __pyx_t_17, __pyx_t_16);
            __pyx_t_18 = 0; __pyx_t_17 = 0; __pyx_t_16 = 0; __pyx_t_33 = 0; __pyx_t_34 = 0; __pyx_t_35 = 0;
            __pyx_lineno = __pyx_t_15; __pyx_clineno = __pyx_t_28; __pyx_filename = __pyx_t_37;
            goto __pyx_L15_except_error;
          }
          __pyx_L25:;
        }
        __Pyx_XDECREF(__pyx_t_11); __pyx_t_11 = 0;
        __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
        __Pyx_XDECREF(__pyx_t_6); __pyx_t_6 = 0;
        goto __pyx_L14_exception_handled;
      }
      goto __pyx_L15_except_error;

      /* "cupy/cuda/memory.pyx":1535
 *         mem = None
 *         oom_error = False
 *         try:             # <<<<<<<<<<<<<<
 *             mem = self._alloc(size).mem
 *         except CUDARuntimeError as e:
*/
      __pyx_L15_except_error:;
      __Pyx_XGIVEREF(__pyx_t_12);
      __Pyx_XGIVEREF(__pyx_t_13);
      __Pyx_XGIVEREF(__pyx_t_14);
      __Pyx_ExceptionReset(__pyx_t_12, __pyx_t_13, __pyx_t_14);
      goto __pyx_L11_error;
      __pyx_L14_exception_handled:;
      __Pyx_XGIVEREF(__pyx_t_12);
      __Pyx_XGIVEREF(__pyx_t_13);
      __Pyx_XGIVEREF(__pyx_t_14);
      __Pyx_ExceptionReset(__pyx_t_12, __pyx_t_13, __pyx_t_14);
      __pyx_L18_try_end:;
    }
  }

  /* "cupy/cuda/memory.pyx":1557
 *                     oom_error = True
 *         finally:
 *             if mem is None:             # <<<<<<<<<<<<<<
 *                 self._total_bytes -= size
 *                 if oom_error:
*/
  /*finally:*/ {
    /*normal exit:*/{
      __pyx_t_2 = (((PyObject *)__pyx_v_mem) == Py_None);
      if (__pyx_t_2) {

        /* "cupy/cuda/memory.pyx":1558
 *         finally:
 *             if mem is None:
 *                 self._total_bytes -= size             # <<<<<<<<<<<<<<
 *                 if oom_error:
 *                     raise OutOfMemoryError(
*/
        __pyx_v_self->_total_bytes -= __pyx_v_size;

        /* "cupy/cuda/memory.pyx":1559
 *             if mem is None:
 *                 self._total_bytes -= size
 *                 if oom_error:             # <<<<<<<<<<<<<<
 *                     raise OutOfMemoryError(
 *                         size, self._total_bytes.load(), limit)
*/
        if (unlikely(__pyx_v_oom_error)) {

          /* "cupy/cuda/memory.pyx":1560
 *                 self._total_bytes -= size
 *                 if oom_error:
 *                     raise OutOfMemoryError(             # <<<<<<<<<<<<<<
 *                         size, self._total_bytes.load(), limit)
 * 
*/
          __pyx_t_3 = NULL;
          __Pyx_GetModuleGlobalName(__pyx_t_11, __pyx_mstate_global->__pyx_n_u_OutOfMemoryError); if (unlikely(!__pyx_t_11)) __PYX_ERR(0, 1560, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_11);

          /* "cupy/cuda/memory.pyx":1561
 *                 if oom_error:
 *                     raise OutOfMemoryError(
 *                         size, self._total_bytes.load(), limit)             # <<<<<<<<<<<<<<
 * 
 *         return mem
*/
          __pyx_t_7 = __Pyx_PyLong_FromSize_t(__pyx_v_size); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 1561, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_7);
          __pyx_t_8 = __Pyx_PyLong_FromSize_t(__pyx_v_self->_total_bytes.load()); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 1561, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_8);
          __pyx_t_10 = __Pyx_PyLong_FromSize_t(__pyx_v_limit); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 1561, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_10);
          __pyx_t_9 = 1;
          #if CYTHON_UNPACK_METHODS
          if (unlikely(PyMethod_Check(__pyx_t_11))) {
            __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_11);
            assert(__pyx_t_3);
            PyObject* __pyx__function = PyMethod_GET_FUNCTION(__pyx_t_11);
            __Pyx_INCREF(__pyx_t_3);
            __Pyx_INCREF(__pyx__function);
            __Pyx_DECREF_SET(__pyx_t_11, __pyx__function);
            __pyx_t_9 = 0;
          }
          #endif
          {
            PyObject *__pyx_callargs[4] = {__pyx_t_3, __pyx_t_7, __pyx_t_8, __pyx_t_10};
            __pyx_t_6 = __Pyx_PyObject_FastCall((PyObject*)__pyx_t_11, __pyx_callargs+__pyx_t_9, (4-__pyx_t_9) | (__pyx_t_9*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
            __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
            __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
            __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
            __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
            __Pyx_DECREF(__pyx_t_11); __pyx_t_11 = 0;
            if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 1560, __pyx_L1_error)
            __Pyx_GOTREF(__pyx_t_6);
          }
          __Pyx_Raise(__pyx_t_6, 0, 0, 0);
          __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
          __PYX_ERR(0, 1560, __pyx_L1_error)

          /* "cupy/cuda/memory.pyx":1559
 *             if mem is None:
 *                 self._total_bytes -= size
 *                 if oom_error:             # <<<<<<<<<<<<<<
 *                     raise OutOfMemoryError(
 *                         size, self._total_bytes.load(), limit)
*/
        }

        /* "cupy/cuda/memory.pyx":1557
 *                     oom_error = True
 *         finally:
 *             if mem is None:             # <<<<<<<<<<<<<<
 *                 self._total_bytes -= size
 *                 if oom_error:
*/
      }
      goto __pyx_L12;
    }
    __pyx_L11_error:;
    /*exception exit:*/{
      __Pyx_PyThreadState_declare
      __Pyx_PyThreadState_assign
      __pyx_t_14 = 0; __pyx_t_13 = 0; __pyx_t_12 = 0; __pyx_t_35 = 0; __pyx_t_34 = 0; __pyx_t_33 = 0;
      __Pyx_XDECREF(__pyx_t_10); __pyx_t_10 = 0;
      __Pyx_XDECREF(__pyx_t_11); __pyx_t_11 = 0;
      __Pyx_XDECREF(__pyx_t_19); __pyx_t_19 = 0;
      __Pyx_XDECREF(__pyx_t_20); __pyx_t_20 = 0;
      __Pyx_XDECREF(__pyx_t_21); __pyx_t_21 = 0;
      __Pyx_XDECREF(__pyx_t_22); __pyx_t_22 = 0;
      __Pyx_XDECREF(__pyx_t_26); __pyx_t_26 = 0;
      __Pyx_XDECREF(__pyx_t_27); __pyx_t_27 = 0;
      __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
      __Pyx_XDECREF(__pyx_t_6); __pyx_t_6 = 0;
      __Pyx_XDECREF(__pyx_t_7); __pyx_t_7 = 0;
      __Pyx_XDECREF(__pyx_t_8); __pyx_t_8 = 0;
       __Pyx_ExceptionSwap(&__pyx_t_35, &__pyx_t_34, &__pyx_t_33);
      if ( unlikely(__Pyx_GetException(&__pyx_t_14, &__pyx_t_13, &__pyx_t_12) < 0)) __Pyx_ErrFetch(&__pyx_t_14, &__pyx_t_13, &__pyx_t_12);
      __Pyx_XGOTREF(__pyx_t_14);
      __Pyx_XGOTREF(__pyx_t_13);
      __Pyx_XGOTREF(__pyx_t_12);
      __Pyx_XGOTREF(__pyx_t_35);
      __Pyx_XGOTREF(__pyx_t_34);
      __Pyx_XGOTREF(__pyx_t_33);
      __pyx_t_28 = __pyx_lineno; __pyx_t_15 = __pyx_clineno; __pyx_t_38 = __pyx_filename;
      {
        __pyx_t_2 = (((PyObject *)__pyx_v_mem) == Py_None);
        if (__pyx_t_2) {

          /* "cupy/cuda/memory.pyx":1558
 *         finally:
 *             if mem is None:
 *                 self._total_bytes -= size             # <<<<<<<<<<<<<<
 *                 if oom_error:
 *                     raise OutOfMemoryError(
*/
          __pyx_v_self->_total_bytes -= __pyx_v_size;

          /* "cupy/cuda/memory.pyx":1559
 *             if mem is None:
 *                 self._total_bytes -= size
 *                 if oom_error:             # <<<<<<<<<<<<<<
 *                     raise OutOfMemoryError(
 *                         size, self._total_bytes.load(), limit)
*/
          if (unlikely(__pyx_v_oom_error)) {

            /* "cupy/cuda/memory.pyx":1560
 *                 self._total_bytes -= size
 *                 if oom_error:
 *                     raise OutOfMemoryError(             # <<<<<<<<<<<<<<
 *                         size, self._total_bytes.load(), limit)
 * 
*/
            __pyx_t_11 = NULL;
            __Pyx_GetModuleGlobalName(__pyx_t_10, __pyx_mstate_global->__pyx_n_u_OutOfMemoryError); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 1560, __pyx_L74_error)
            __Pyx_GOTREF(__pyx_t_10);

            /* "cupy/cuda/memory.pyx":1561
 *                 if oom_error:
 *                     raise OutOfMemoryError(
 *                         size, self._total_bytes.load(), limit)             # <<<<<<<<<<<<<<
 * 
 *         return mem
*/
            __pyx_t_8 = __Pyx_PyLong_FromSize_t(__pyx_v_size); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 1561, __pyx_L74_error)
            __Pyx_GOTREF(__pyx_t_8);
            __pyx_t_7 = __Pyx_PyLong_FromSize_t(__pyx_v_self->_total_bytes.load()); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 1561, __pyx_L74_error)
            __Pyx_GOTREF(__pyx_t_7);
            __pyx_t_3 = __Pyx_PyLong_FromSize_t(__pyx_v_limit); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 1561, __pyx_L74_error)
            __Pyx_GOTREF(__pyx_t_3);
            __pyx_t_9 = 1;
            #if CYTHON_UNPACK_METHODS
            if (unlikely(PyMethod_Check(__pyx_t_10))) {
              __pyx_t_11 = PyMethod_GET_SELF(__pyx_t_10);
              assert(__pyx_t_11);
              PyObject* __pyx__function = PyMethod_GET_FUNCTION(__pyx_t_10);
              __Pyx_INCREF(__pyx_t_11);
              __Pyx_INCREF(__pyx__function);
              __Pyx_DECREF_SET(__pyx_t_10, __pyx__function);
              __pyx_t_9 = 0;
            }
            #endif
            {
              PyObject *__pyx_callargs[4] = {__pyx_t_11, __pyx_t_8, __pyx_t_7, __pyx_t_3};
              __pyx_t_6 = __Pyx_PyObject_FastCall((PyObject*)__pyx_t_10, __pyx_callargs+__pyx_t_9, (4-__pyx_t_9) | (__pyx_t_9*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
              __Pyx_XDECREF(__pyx_t_11); __pyx_t_11 = 0;
              __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
              __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
              __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
              __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
              if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 1560, __pyx_L74_error)
              __Pyx_GOTREF(__pyx_t_6);
            }
            __Pyx_Raise(__pyx_t_6, 0, 0, 0);
            __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
            __PYX_ERR(0, 1560, __pyx_L74_error)

            /* "cupy/cuda/memory.pyx":1559
 *             if mem is None:
 *                 self._total_bytes -= size
 *                 if oom_error:             # <<<<<<<<<<<<<<
 *                     raise OutOfMemoryError(
 *                         size, self._total_bytes.load(), limit)
*/
          }

          /* "cupy/cuda/memory.pyx":1557
 *                     oom_error = True
 *         finally:
 *             if mem is None:             # <<<<<<<<<<<<<<
 *                 self._total_bytes -= size
 *                 if oom_error:
*/
        }
      }
      __Pyx_XGIVEREF(__pyx_t_35);
      __Pyx_XGIVEREF(__pyx_t_34);
      __Pyx_XGIVEREF(__pyx_t_33);
      __Pyx_ExceptionReset(__pyx_t_35, __pyx_t_34, __pyx_t_33);
      __Pyx_XGIVEREF(__pyx_t_14);
      __Pyx_XGIVEREF(__pyx_t_13);
      __Pyx_XGIVEREF(__pyx_t_12);
      __Pyx_ErrRestore(__pyx_t_14, __pyx_t_13, __pyx_t_12);
      __pyx_t_14 = 0; __pyx_t_13 = 0; __pyx_t_12 = 0; __pyx_t_35 = 0; __pyx_t_34 = 0; __pyx_t_33 = 0;
      __pyx_lineno = __pyx_t_28; __pyx_clineno = __pyx_t_15; __pyx_filename = __pyx_t_38;
      goto __pyx_L1_error;
      __pyx_L74_error:;
      __Pyx_XGIVEREF(__pyx_t_35);
      __Pyx_XGIVEREF(__pyx_t_34);
      __Pyx_XGIVEREF(__pyx_t_33);
      __Pyx_ExceptionReset(__pyx_t_35, __pyx_t_34, __pyx_t_33);
      __Pyx_XDECREF(__pyx_t_14); __pyx_t_14 = 0;
      __Pyx_XDECREF(__pyx_t_13); __pyx_t_13 = 0;
      __Pyx_XDECREF(__pyx_t_12); __pyx_t_12 = 0;
      __pyx_t_35 = 0; __pyx_t_34 = 0; __pyx_t_33 = 0;
      goto __pyx_L1_error;
    }
    __pyx_L12:;
  }

  /* "cupy/cuda/memory.pyx":1563
 *                         size, self._total_bytes.load(), limit)
 * 
 *         return mem             # <<<<<<<<<<<<<<
 * 
 * 
*/
  __Pyx_XDECREF((PyObject *)__pyx_r);
  __Pyx_INCREF((PyObject *)__pyx_v_mem);
  __pyx_r = __pyx_v_mem;
  goto __pyx_L0;

  /* "cupy/cuda/memory.pyx":1518
 *         return self._total_bytes_limit.load()
 * 
 *     cdef BaseMemory _try_malloc(self, size_t size):             # <<<<<<<<<<<<<<
 *         cdef size_t limit = self._total_bytes_limit.load()
 *         cdef bint ok
*/

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_XDECREF(__pyx_t_7);
  __Pyx_XDECREF(__pyx_t_8);
  __Pyx_XDECREF(__pyx_t_10);
  __Pyx_XDECREF(__pyx_t_11);
  __Pyx_XDECREF(__pyx_t_19);
  __Pyx_XDECREF(__pyx_t_20);
  __Pyx_XDECREF(__pyx_t_21);
  __Pyx_XDECREF(__pyx_t_22);
  __Pyx_XDECREF(__pyx_t_26);
  __Pyx_XDECREF(__pyx_t_27);
  __Pyx_AddTraceback("cupy.cuda.memory.SingleDeviceMemoryPool._try_malloc", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XDECREF((PyObject *)__pyx_v_mem);
  __Pyx_XDECREF(__pyx_v_e);
  __Pyx_XGIVEREF((PyObject *)__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "cupy/cuda/memory.pyx":1256
 *         object __weakref__
 *         object _weakref
 *         readonly int _device_id             # <<<<<<<<<<<<<<
 * 
 *     def __init__(self, allocator=None):
*/

/* Python wrapper */
static PyObject *__pyx_pw_4cupy_4cuda_6memory_22SingleDeviceMemoryPool_10_device_id_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_4cupy_4cuda_6memory_22SingleDeviceMemoryPool_10_device_id_1__get__(PyObject *__pyx_v_self) {
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_kwvalues = __Pyx_KwValues_VARARGS(__pyx_args, __pyx_nargs);
  __pyx_r = __pyx_pf_4cupy_4cuda_6memory_22SingleDeviceMemoryPool_10_device_id___get__(((struct __pyx_obj_4cupy_4cuda_6memory_SingleDeviceMemoryPool *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4cupy_4cuda_6memory_22SingleDeviceMemoryPool_10_device_id___get__(struct __pyx_obj_4cupy_4cuda_6memory_SingleDeviceMemoryPool *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__get__", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __Pyx_PyLong_From_int(__pyx_v_self->_device_id); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1256, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("cupy.cuda.memory.SingleDeviceMemoryPool._device_id.__get__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     raise TypeError, "self._arena_mutex,self._in_use_bytes,self._total_bytes,self._total_bytes_limit cannot be converted to a Python object for pickling"
 * def __setstate_cython__(self, __pyx_state):
*/

/* Python wrapper */
static PyObject *__pyx_pw_4cupy_4cuda_6memory_22SingleDeviceMemoryPool_25__reduce_cython__(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
PyDoc_STRVAR(__pyx_doc_4cupy_4cuda_6memory_22SingleDeviceMemoryPool_24__reduce_cython__, "SingleDeviceMemoryPool.__reduce_cython__(self)");
static PyObject *__pyx_pw_4cupy_4cuda_6memory_22SingleDeviceMemoryPool_25__reduce_cython__(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
) {
  #if !CYTHON_METH_FASTCALL
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  #endif
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__reduce_cython__ (wrapper)", 0);
  #if !CYTHON_METH_FASTCALL
  #if CYTHON_ASSUME_SAFE_SIZE
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return NULL;
  #endif
  #endif
  __pyx_kwvalues = __Pyx_KwValues_FASTCALL(__pyx_args, __pyx_nargs);
  if (unlikely(__pyx_nargs > 0)) { __Pyx_RaiseArgtupleInvalid("__reduce_cython__", 1, 0, 0, __pyx_nargs); return NULL; }
  const Py_ssize_t __pyx_kwds_len = unlikely(__pyx_kwds) ? __Pyx_NumKwargs_FASTCALL(__pyx_kwds) : 0;
  if (unlikely(__pyx_kwds_len < 0)) return NULL;
  if (unlikely(__pyx_kwds_len > 0)) {__Pyx_RejectKeywords("__reduce_cython__", __pyx_kwds); return NULL;}
  __pyx_r = __pyx_pf_4cupy_4cuda_6memory_22SingleDeviceMemoryPool_24__reduce_cython__(((struct __pyx_obj_4cupy_4cuda_6memory_SingleDeviceMemoryPool *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4cupy_4cuda_6memory_22SingleDeviceMemoryPool_24__reduce_cython__(CYTHON_UNUSED struct __pyx_obj_4cupy_4cuda_6memory_SingleDeviceMemoryPool *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__reduce_cython__", 0);

  /* "(tree fragment)":2
 * def __reduce_cython__(self):
 *     raise TypeError, "self._arena_mutex,self._in_use_bytes,self._total_bytes,self._total_bytes_limit cannot be converted to a Python object for pickling"             # <<<<<<<<<<<<<<
 * def __setstate_cython__(self, __pyx_state):
 *     raise TypeError, "self._arena_mutex,self._in_use_bytes,self._total_bytes,self._total_bytes_limit cannot be converted to a Python object for pickling"
*/
  __Pyx_Raise(((PyObject *)(((PyTypeObject*)PyExc_TypeError))), __pyx_mstate_global->__pyx_kp_u_self__arena_mutex_self__in_use_b, 0, 0);
  __PYX_ERR(3, 2, __pyx_L1_error)

  /* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     raise TypeError, "self._arena_mutex,self._in_use_bytes,self._total_bytes,self._total_bytes_limit cannot be converted to a Python object for pickling"
 * def __setstate_cython__(self, __pyx_state):
*/

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_AddTraceback("cupy.cuda.memory.SingleDeviceMemoryPool.__reduce_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":3
 * def __reduce_cython__(self):
 *     raise TypeError, "self._arena_mutex,self._in_use_bytes,self._total_bytes,self._total_bytes_limit cannot be converted to a Python object for pickling"
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     raise TypeError, "self._arena_mutex,self._in_use_bytes,self._total_bytes,self._total_bytes_limit cannot be converted to a Python object for pickling"
*/

/* Python wrapper */
static PyObject *__pyx_pw_4cupy_4cuda_6memory_22SingleDeviceMemoryPool_27__setstate_cython__(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
PyDoc_STRVAR(__pyx_doc_4cupy_4cuda_6memory_22SingleDeviceMemoryPool_26__setstate_cython__, "SingleDeviceMemoryPool.__setstate_cython__(self, __pyx_state)");
static PyObject *__pyx_pw_4cupy_4cuda_6memory_22SingleDeviceMemoryPool_27__setstate_cython__(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
) {
  CYTHON_UNUSED PyObject *__pyx_v___pyx_state = 0;
  #if !CYTHON_METH_FASTCALL
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  #endif
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject* values[1] = {0};
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__setstate_cython__ (wrapper)", 0);
  #if !CYTHON_METH_FASTCALL
  #if CYTHON_ASSUME_SAFE_SIZE
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return NULL;
  #endif
  #endif
  __pyx_kwvalues = __Pyx_KwValues_FASTCALL(__pyx_args, __pyx_nargs);
  {
    PyObject ** const __pyx_pyargnames[] = {&__pyx_mstate_global->__pyx_n_u_pyx_state,0};
    const Py_ssize_t __pyx_kwds_len = (__pyx_kwds) ? __Pyx_NumKwargs_FASTCALL(__pyx_kwds) : 0;
    if (unlikely(__pyx_kwds_len < 0)) __PYX_ERR(3, 3, __pyx_L3_error)
    if (__pyx_kwds_len > 0) {
      switch (__pyx_nargs) {
        case  1:
        values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(3, 3, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      const Py_ssize_t kwd_pos_args = __pyx_nargs;
      if (__Pyx_ParseKeywords(__pyx_kwds, __pyx_kwvalues, __pyx_pyargnames, 0, values, kwd_pos_args, __pyx_kwds_len, "__setstate_cython__", 0) < (0)) __PYX_ERR(3, 3, __pyx_L3_error)
      for (Py_ssize_t i = __pyx_nargs; i < 1; i++) {
        if (unlikely(!values[i])) { __Pyx_RaiseArgtupleInvalid("__setstate_cython__", 1, 1, 1, i); __PYX_ERR(3, 3, __pyx_L3_error) }
      }
    } else if (unlikely(__pyx_nargs != 1)) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
      if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(3, 3, __pyx_L3_error)
    }
    __pyx_v___pyx_state = values[0];
  }
  goto __pyx_L6_skip;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("__setstate_cython__", 1, 1, 1, __pyx_nargs); __PYX_ERR(3, 3, __pyx_L3_error)
  __pyx_L6_skip:;
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_AddTraceback("cupy.cuda.memory.SingleDeviceMemoryPool.__setstate_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_4cupy_4cuda_6memory_22SingleDeviceMemoryPool_26__setstate_cython__(((struct __pyx_obj_4cupy_4cuda_6memory_SingleDeviceMemoryPool *)__pyx_v_self), __pyx_v___pyx_state);

  /* function exit code */
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4cupy_4cuda_6memory_22SingleDeviceMemoryPool_26__setstate_cython__(CYTHON_UNUSED struct __pyx_obj_4cupy_4cuda_6memory_SingleDeviceMemoryPool *__pyx_v_self, CYTHON_UNUSED PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__setstate_cython__", 0);

  /* "(tree fragment)":4
 *     raise TypeError, "self._arena_mutex,self._in_use_bytes,self._total_bytes,self._total_bytes_limit cannot be converted to a Python object for pickling"
 * def __setstate_cython__(self, __pyx_state):
 *     raise TypeError, "self._arena_mutex,self._in_use_bytes,self._total_bytes,self._total_bytes_limit cannot be converted to a Python object for pickling"             # <<<<<<<<<<<<<<
*/
  __Pyx_Raise(((PyObject *)(((PyTypeObject*)PyExc_TypeError))), __pyx_mstate_global->__pyx_kp_u_self__arena_mutex_self__in_use_b, 0, 0);
  __PYX_ERR(3, 4, __pyx_L1_error)

  /* "(tree fragment)":3
 * def __reduce_cython__(self):
 *     raise TypeError, "self._arena_mutex,self._in_use_bytes,self._total_bytes,self._total_bytes_limit cannot be converted to a Python object for pickling"
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     raise TypeError, "self._arena_mutex,self._in_use_bytes,self._total_bytes,self._total_bytes_limit cannot be converted to a Python object for pickling"
*/

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_AddTraceback("cupy.cuda.memory.SingleDeviceMemoryPool.__setstate_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "cupy/cuda/memory.pyx":1594
 *     """
 * 
 *     def __init__(self, allocator=None):             # <<<<<<<<<<<<<<
 *         if allocator is None:
 *             allocator = _malloc
*/

/* Python wrapper */
static int __pyx_pw_4cupy_4cuda_6memory_10MemoryPool_1__init__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static int __pyx_pw_4cupy_4cuda_6memory_10MemoryPool_1__init__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_allocator = 0;
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject* values[1] = {0};
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__init__ (wrapper)", 0);
  #if CYTHON_ASSUME_SAFE_SIZE
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return -1;
  #endif
  __pyx_kwvalues = __Pyx_KwValues_VARARGS(__pyx_args, __pyx_nargs);
  {
    PyObject ** const __pyx_pyargnames[] = {&__pyx_mstate_global->__pyx_n_u_allocator,0};
    const Py_ssize_t __pyx_kwds_len = (__pyx_kwds) ? __Pyx_NumKwargs_VARARGS(__pyx_kwds) : 0;
    if (unlikely(__pyx_kwds_len < 0)) __PYX_ERR(0, 1594, __pyx_L3_error)
    if (__pyx_kwds_len > 0) {
      switch (__pyx_nargs) {
        case  1:
        values[0] = __Pyx_ArgRef_VARARGS(__pyx_args, 0);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(0, 1594, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      const Py_ssize_t kwd_pos_args = __pyx_nargs;
      if (__Pyx_ParseKeywords(__pyx_kwds, __pyx_kwvalues, __pyx_pyargnames, 0, values, kwd_pos_args, __pyx_kwds_len, "__init__", 0) < (0)) __PYX_ERR(0, 1594, __pyx_L3_error)
      if (!values[0]) values[0] = __Pyx_NewRef(((PyObject *)Py_None));
    } else {
      switch (__pyx_nargs) {
        case  1:
        values[0] = __Pyx_ArgRef_VARARGS(__pyx_args, 0);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(0, 1594, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      if (!values[0]) values[0] = __Pyx_NewRef(((PyObject *)Py_None));
    }
    __pyx_v_allocator = values[0];
  }
  goto __pyx_L6_skip;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("__init__", 0, 0, 1, __pyx_nargs); __PYX_ERR(0, 1594, __pyx_L3_error)
  __pyx_L6_skip:;
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_AddTraceback("cupy.cuda.memory.MemoryPool.__init__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return -1;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_4cupy_4cuda_6memory_10MemoryPool___init__(((struct __pyx_obj_4cupy_4cuda_6memory_MemoryPool *)__pyx_v_self), __pyx_v_allocator);

  /* function exit code */
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_4cupy_4cuda_6memory_10MemoryPool___init__(struct __pyx_obj_4cupy_4cuda_6memory_MemoryPool *__pyx_v_self, PyObject *__pyx_v_allocator) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__init__", 0);
  __Pyx_INCREF(__pyx_v_allocator);

  /* "cupy/cuda/memory.pyx":1595
 * 
 *     def __init__(self, allocator=None):
 *         if allocator is None:             # <<<<<<<<<<<<<<
 *             allocator = _malloc
 *         self._allocator = allocator
*/
  __pyx_t_1 = (__pyx_v_allocator == Py_None);
  if (__pyx_t_1) {

    /* "cupy/cuda/memory.pyx":1596
 *     def __init__(self, allocator=None):
 *         if allocator is None:
 *             allocator = _malloc             # <<<<<<<<<<<<<<
 *         self._allocator = allocator
 * 
*/
    __Pyx_GetModuleGlobalName(__pyx_t_2, __pyx_mstate_global->__pyx_n_u_malloc); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1596, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_DECREF_SET(__pyx_v_allocator, __pyx_t_2);
    __pyx_t_2 = 0;

    /* "cupy/cuda/memory.pyx":1595
 * 
 *     def __init__(self, allocator=None):
 *         if allocator is None:             # <<<<<<<<<<<<<<
 *             allocator = _malloc
 *         self._allocator = allocator
*/
  }

  /* "cupy/cuda/memory.pyx":1597
 *         if allocator is None:
 *             allocator = _malloc
 *         self._allocator = allocator             # <<<<<<<<<<<<<<
 * 
 *     cdef _ensure_pools_and_return_device_pool(self):
*/
  __Pyx_INCREF(__pyx_v_allocator);
  __Pyx_GIVEREF(__pyx_v_allocator);
  __Pyx_GOTREF(__pyx_v_self->_allocator);
  __Pyx_DECREF(__pyx_v_self->_allocator);
  __pyx_v_self->_allocator = __pyx_v_allocator;

  /* "cupy/cuda/memory.pyx":1594
 *     """
 * 
 *     def __init__(self, allocator=None):             # <<<<<<<<<<<<<<
 *         if allocator is None:
 *             allocator = _malloc
*/

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_AddTraceback("cupy.cuda.memory.MemoryPool.__init__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_allocator);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}
static PyObject *__pyx_gb_4cupy_4cuda_6memory_10MemoryPool_36_ensure_pools_and_return_device_pool_2generator(__pyx_CoroutineObject *__pyx_generator, CYTHON_UNUSED PyThreadState *__pyx_tstate, PyObject *__pyx_sent_value); /* proto */

/* "cupy/cuda/memory.pyx":1603
 *         n_gpu = runtime.getDeviceCount()
 *         pools = tuple(
 *             SingleDeviceMemoryPool(self._allocator) for i in range(n_gpu))             # <<<<<<<<<<<<<<
 * 
 *         # If no-one beat us to it, set _pools. Note that the above seems to
*/

static PyObject *__pyx_pf_4cupy_4cuda_6memory_10MemoryPool_36_ensure_pools_and_return_device_pool_genexpr(PyObject *__pyx_self, int __pyx_genexpr_arg_0) {
  struct __pyx_obj_4cupy_4cuda_6memory___pyx_scope_struct_1_genexpr *__pyx_cur_scope;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("genexpr", 0);
  __pyx_cur_scope = (struct __pyx_obj_4cupy_4cuda_6memory___pyx_scope_struct_1_genexpr *)__pyx_tp_new_4cupy_4cuda_6memory___pyx_scope_struct_1_genexpr(__pyx_mstate_global->__pyx_ptype_4cupy_4cuda_6memory___pyx_scope_struct_1_genexpr, __pyx_mstate_global->__pyx_empty_tuple, NULL);
  if (unlikely(!__pyx_cur_scope)) {
    __pyx_cur_scope = ((struct __pyx_obj_4cupy_4cuda_6memory___pyx_scope_struct_1_genexpr *)Py_None);
    __Pyx_INCREF(Py_None);
    __PYX_ERR(0, 1603, __pyx_L1_error)
  } else {
    __Pyx_GOTREF((PyObject *)__pyx_cur_scope);
  }
  __pyx_cur_scope->__pyx_outer_scope = (struct __pyx_obj_4cupy_4cuda_6memory___pyx_scope_struct___ensure_pools_and_return_device_pool *) __pyx_self;
  __Pyx_INCREF((PyObject *)__pyx_cur_scope->__pyx_outer_scope);
  __Pyx_GIVEREF((PyObject *)__pyx_cur_scope->__pyx_outer_scope);
  __pyx_cur_scope->__pyx_genexpr_arg_0 = __pyx_genexpr_arg_0;
  {
    __pyx_CoroutineObject *gen = __Pyx_Generator_New((__pyx_coroutine_body_t) __pyx_gb_4cupy_4cuda_6memory_10MemoryPool_36_ensure_pools_and_return_device_pool_2generator, ((PyObject *)__pyx_mstate_global->__pyx_codeobj_tab[0]), (PyObject *) __pyx_cur_scope, __pyx_mstate_global->__pyx_n_u_genexpr, __pyx_mstate_global->__pyx_n_u_MemoryPool__ensure_pools_and_ret, __pyx_mstate_global->__pyx_n_u_cupy_cuda_memory); if (unlikely(!gen)) __PYX_ERR(0, 1603, __pyx_L1_error)
    __Pyx_DECREF(__pyx_cur_scope);
    __Pyx_RefNannyFinishContext();
    return (PyObject *) gen;
  }

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_AddTraceback("cupy.cuda.memory.MemoryPool._ensure_pools_and_return_device_pool.genexpr", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __Pyx_DECREF((PyObject *)__pyx_cur_scope);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_gb_4cupy_4cuda_6memory_10MemoryPool_36_ensure_pools_and_return_device_pool_2generator(__pyx_CoroutineObject *__pyx_generator, CYTHON_UNUSED PyThreadState *__pyx_tstate, PyObject *__pyx_sent_value) /* generator body */
{
  struct __pyx_obj_4cupy_4cuda_6memory___pyx_scope_struct_1_genexpr *__pyx_cur_scope = ((struct __pyx_obj_4cupy_4cuda_6memory___pyx_scope_struct_1_genexpr *)__pyx_generator->closure);
  PyObject *__pyx_r = NULL;
  int __pyx_t_1;
  int __pyx_t_2;
  int __pyx_t_3;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  size_t __pyx_t_6;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("genexpr", 0);
  switch (__pyx_generator->resume_label) {
    case 0: goto __pyx_L3_first_run;
    case 1: goto __pyx_L6_resume_from_yield;
    default: /* CPython raises the right error here */
    __Pyx_RefNannyFinishContext();
    return NULL;
  }
  __pyx_L3_first_run:;
  if (unlikely(__pyx_sent_value != Py_None)) {
    if (unlikely(__pyx_sent_value)) PyErr_SetString(PyExc_TypeError, "can't send non-None value to a just-started generator");
    __PYX_ERR(0, 1603, __pyx_L1_error)
  }
  __pyx_t_1 = __pyx_cur_scope->__pyx_genexpr_arg_0;
  __pyx_t_2 = __pyx_t_1;
  for (__pyx_t_3 = 0; __pyx_t_3 < __pyx_t_2; __pyx_t_3+=1) {
    __pyx_cur_scope->__pyx_v_i = __pyx_t_3;
    __pyx_t_5 = NULL;
    if (unlikely(!__pyx_cur_scope->__pyx_outer_scope->__pyx_v_self)) { __Pyx_RaiseClosureNameError("self"); __PYX_ERR(0, 1603, __pyx_L1_error) }
    __pyx_t_6 = 1;
    {
      PyObject *__pyx_callargs[2] = {__pyx_t_5, __pyx_cur_scope->__pyx_outer_scope->__pyx_v_self->_allocator};
      __pyx_t_4 = __Pyx_PyObject_FastCall((PyObject*)__pyx_mstate_global->__pyx_ptype_4cupy_4cuda_6memory_SingleDeviceMemoryPool, __pyx_callargs+__pyx_t_6, (2-__pyx_t_6) | (__pyx_t_6*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
      __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
      if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 1603, __pyx_L1_error)
      __Pyx_GOTREF((PyObject *)__pyx_t_4);
    }
    __pyx_r = ((PyObject *)__pyx_t_4);
    __pyx_t_4 = 0;
    __pyx_cur_scope->__pyx_t_0 = __pyx_t_1;
    __pyx_cur_scope->__pyx_t_1 = __pyx_t_2;
    __pyx_cur_scope->__pyx_t_2 = __pyx_t_3;
    __Pyx_XGIVEREF(__pyx_r);
    __Pyx_RefNannyFinishContext();
    __Pyx_Coroutine_ResetAndClearException(__pyx_generator);
    /* return from generator, yielding value */
    __pyx_generator->resume_label = 1;
    return __pyx_r;
    __pyx_L6_resume_from_yield:;
    __pyx_t_1 = __pyx_cur_scope->__pyx_t_0;
    __pyx_t_2 = __pyx_cur_scope->__pyx_t_1;
    __pyx_t_3 = __pyx_cur_scope->__pyx_t_2;
    if (unlikely(!__pyx_sent_value)) __PYX_ERR(0, 1603, __pyx_L1_error)
  }
  CYTHON_MAYBE_UNUSED_VAR(__pyx_cur_scope);

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  if (__Pyx_PyErr_Occurred()) {
    __Pyx_Generator_Replace_StopIteration(0);
    __Pyx_AddTraceback("genexpr", __pyx_clineno, __pyx_lineno, __pyx_filename);
  }
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  #if !CYTHON_USE_EXC_INFO_STACK
  __Pyx_Coroutine_ResetAndClearException(__pyx_generator);
  #endif
  __pyx_generator->resume_label = -1;
  __Pyx_Coroutine_clear((PyObject*)__pyx_generator);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "cupy/cuda/memory.pyx":1599
 *         self._allocator = allocator
 * 
 *     cdef _ensure_pools_and_return_device_pool(self):             # <<<<<<<<<<<<<<
 *         # assume we get to create the pools (we may not be the only one)
 *         n_gpu = runtime.getDeviceCount()
*/

static PyObject *__pyx_f_4cupy_4cuda_6memory_10MemoryPool__ensure_pools_and_return_device_pool(struct __pyx_obj_4cupy_4cuda_6memory_MemoryPool *__pyx_v_self) {
  struct __pyx_obj_4cupy_4cuda_6memory___pyx_scope_struct___ensure_pools_and_return_device_pool *__pyx_cur_scope;
  int __pyx_v_n_gpu;
  PyObject *__pyx_v_pools = NULL;
  PyObject *__pyx_gb_4cupy_4cuda_6memory_10MemoryPool_36_ensure_pools_and_return_device_pool_2generator = 0;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  int __pyx_t_4;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("_ensure_pools_and_return_device_pool", 0);
  __pyx_cur_scope = (struct __pyx_obj_4cupy_4cuda_6memory___pyx_scope_struct___ensure_pools_and_return_device_pool *)__pyx_tp_new_4cupy_4cuda_6memory___pyx_scope_struct___ensure_pools_and_return_device_pool(__pyx_mstate_global->__pyx_ptype_4cupy_4cuda_6memory___pyx_scope_struct___ensure_pools_and_return_device_pool, __pyx_mstate_global->__pyx_empty_tuple, NULL);
  if (unlikely(!__pyx_cur_scope)) {
    __pyx_cur_scope = ((struct __pyx_obj_4cupy_4cuda_6memory___pyx_scope_struct___ensure_pools_and_return_device_pool *)Py_None);
    __Pyx_INCREF(Py_None);
    __PYX_ERR(0, 1599, __pyx_L1_error)
  } else {
    __Pyx_GOTREF((PyObject *)__pyx_cur_scope);
  }
  __pyx_cur_scope->__pyx_v_self = __pyx_v_self;
  __Pyx_INCREF((PyObject *)__pyx_cur_scope->__pyx_v_self);
  __Pyx_GIVEREF((PyObject *)__pyx_cur_scope->__pyx_v_self);

  /* "cupy/cuda/memory.pyx":1601
 *     cdef _ensure_pools_and_return_device_pool(self):
 *         # assume we get to create the pools (we may not be the only one)
 *         n_gpu = runtime.getDeviceCount()             # <<<<<<<<<<<<<<
 *         pools = tuple(
 *             SingleDeviceMemoryPool(self._allocator) for i in range(n_gpu))
*/
  __pyx_t_1 = __pyx_f_13cupy_backends_4cuda_3api_7runtime_getDeviceCount(0); if (unlikely(__pyx_t_1 == ((int)-1) && PyErr_Occurred())) __PYX_ERR(0, 1601, __pyx_L1_error)
  __pyx_v_n_gpu = __pyx_t_1;

  /* "cupy/cuda/memory.pyx":1603
 *         n_gpu = runtime.getDeviceCount()
 *         pools = tuple(
 *             SingleDeviceMemoryPool(self._allocator) for i in range(n_gpu))             # <<<<<<<<<<<<<<
 * 
 *         # If no-one beat us to it, set _pools. Note that the above seems to
*/
  __pyx_t_2 = __pyx_pf_4cupy_4cuda_6memory_10MemoryPool_36_ensure_pools_and_return_device_pool_genexpr(((PyObject*)__pyx_cur_scope), __pyx_v_n_gpu); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1603, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);

  /* "cupy/cuda/memory.pyx":1602
 *         # assume we get to create the pools (we may not be the only one)
 *         n_gpu = runtime.getDeviceCount()
 *         pools = tuple(             # <<<<<<<<<<<<<<
 *             SingleDeviceMemoryPool(self._allocator) for i in range(n_gpu))
 * 
*/
  __pyx_t_3 = __Pyx_PySequence_Tuple(__pyx_t_2); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 1602, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_v_pools = ((PyObject*)__pyx_t_3);
  __pyx_t_3 = 0;

  /* "cupy/cuda/memory.pyx":1608
 *         # release the critical section, so including it doesn't work and
 *         # would require a proper mutex.
 *         with cython.critical_section(self):             # <<<<<<<<<<<<<<
 *             if self._pools is None:
 *                 self._pools = pools
*/
  {
      __Pyx_PyCriticalSection __pyx_cs;
      if (unlikely(!__pyx_cur_scope->__pyx_v_self)) { __Pyx_RaiseUnboundLocalError("self"); __PYX_ERR(0, 1608, __pyx_L1_error) }
      __pyx_t_3 = ((PyObject *)__pyx_cur_scope->__pyx_v_self);
      __Pyx_INCREF(__pyx_t_3);
      __Pyx_PyCriticalSection_Begin(&__pyx_cs, (PyObject*)__pyx_t_3);
      /*try:*/ {

        /* "cupy/cuda/memory.pyx":1609
 *         # would require a proper mutex.
 *         with cython.critical_section(self):
 *             if self._pools is None:             # <<<<<<<<<<<<<<
 *                 self._pools = pools
 * 
*/
        __pyx_t_4 = (__pyx_cur_scope->__pyx_v_self->_pools == ((PyObject*)Py_None));
        if (__pyx_t_4) {

          /* "cupy/cuda/memory.pyx":1610
 *         with cython.critical_section(self):
 *             if self._pools is None:
 *                 self._pools = pools             # <<<<<<<<<<<<<<
 * 
 *         return self._pools[device.get_device_id()]
*/
          __Pyx_INCREF(__pyx_v_pools);
          __Pyx_GIVEREF(__pyx_v_pools);
          __Pyx_GOTREF(__pyx_cur_scope->__pyx_v_self->_pools);
          __Pyx_DECREF(__pyx_cur_scope->__pyx_v_self->_pools);
          __pyx_cur_scope->__pyx_v_self->_pools = __pyx_v_pools;

          /* "cupy/cuda/memory.pyx":1609
 *         # would require a proper mutex.
 *         with cython.critical_section(self):
 *             if self._pools is None:             # <<<<<<<<<<<<<<
 *                 self._pools = pools
 * 
*/
        }
      }

      /* "cupy/cuda/memory.pyx":1608
 *         # release the critical section, so including it doesn't work and
 *         # would require a proper mutex.
 *         with cython.critical_section(self):             # <<<<<<<<<<<<<<
 *             if self._pools is None:
 *                 self._pools = pools
*/
      /*finally:*/ {
        /*normal exit:*/{
          __Pyx_PyCriticalSection_End(&__pyx_cs);
          goto __pyx_L5;
        }
        __pyx_L5:;
      }
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  }

  /* "cupy/cuda/memory.pyx":1612
 *                 self._pools = pools
 * 
 *         return self._pools[device.get_device_id()]             # <<<<<<<<<<<<<<
 * 
 *     cpdef MemoryPointer malloc(self, size_t size):
*/
  __Pyx_XDECREF(__pyx_r);
  if (unlikely(__pyx_cur_scope->__pyx_v_self->_pools == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(0, 1612, __pyx_L1_error)
  }
  __pyx_t_1 = __pyx_f_4cupy_4cuda_6device_get_device_id(0); if (unlikely(__pyx_t_1 == ((int)-1) && PyErr_Occurred())) __PYX_ERR(0, 1612, __pyx_L1_error)
  __pyx_t_3 = __Pyx_GetItemInt_Tuple(__pyx_cur_scope->__pyx_v_self->_pools, __pyx_t_1, int, 1, __Pyx_PyLong_From_int, 0, 1, 1, 1, __Pyx_ReferenceSharing_SharedReference); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 1612, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_r = __pyx_t_3;
  __pyx_t_3 = 0;
  goto __pyx_L0;

  /* "cupy/cuda/memory.pyx":1599
 *         self._allocator = allocator
 * 
 *     cdef _ensure_pools_and_return_device_pool(self):             # <<<<<<<<<<<<<<
 *         # assume we get to create the pools (we may not be the only one)
 *         n_gpu = runtime.getDeviceCount()
*/

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_AddTraceback("cupy.cuda.memory.MemoryPool._ensure_pools_and_return_device_pool", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_pools);
  __Pyx_XDECREF(__pyx_gb_4cupy_4cuda_6memory_10MemoryPool_36_ensure_pools_and_return_device_pool_2generator);
  __Pyx_DECREF((PyObject *)__pyx_cur_scope);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "cupy/cuda/memory.pyx":1614
 *         return self._pools[device.get_device_id()]
 * 
 *     cpdef MemoryPointer malloc(self, size_t size):             # <<<<<<<<<<<<<<
 *         """Allocates the memory, from the pool if possible.
 * 
*/

static PyObject *__pyx_pw_4cupy_4cuda_6memory_10MemoryPool_3malloc(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
static struct __pyx_obj_4cupy_4cuda_6memory_MemoryPointer *__pyx_f_4cupy_4cuda_6memory_10MemoryPool_malloc(struct __pyx_obj_4cupy_4cuda_6memory_MemoryPool *__pyx_v_self, size_t __pyx_v_size, int __pyx_skip_dispatch) {
  struct __pyx_obj_4cupy_4cuda_6memory_SingleDeviceMemoryPool *__pyx_v_mp = NULL;
  struct __pyx_obj_4cupy_4cuda_6memory_MemoryPointer *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  size_t __pyx_t_6;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("malloc", 0);
  /* Check if called by wrapper */
  if (unlikely(__pyx_skip_dispatch)) ;
  /* Check if overridden in Python */
  else if (
  #if !CYTHON_USE_TYPE_SLOTS
  unlikely(Py_TYPE(((PyObject *)__pyx_v_self)) != __pyx_mstate_global->__pyx_ptype_4cupy_4cuda_6memory_MemoryPool &&
  __Pyx_PyType_HasFeature(Py_TYPE(((PyObject *)__pyx_v_self)), Py_TPFLAGS_HAVE_GC))
  #else
  unlikely(Py_TYPE(((PyObject *)__pyx_v_self))->tp_dictoffset != 0 || __Pyx_PyType_HasFeature(Py_TYPE(((PyObject *)__pyx_v_self)), (Py_TPFLAGS_IS_ABSTRACT | Py_TPFLAGS_HEAPTYPE)))
  #endif
  ) {
    #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
    static PY_UINT64_T __pyx_tp_dict_version = __PYX_DICT_VERSION_INIT, __pyx_obj_dict_version = __PYX_DICT_VERSION_INIT;
    if (unlikely(!__Pyx_object_dict_version_matches(((PyObject *)__pyx_v_self), __pyx_tp_dict_version, __pyx_obj_dict_version))) {
      PY_UINT64_T __pyx_typedict_guard = __Pyx_get_tp_dict_version(((PyObject *)__pyx_v_self));
      #endif
      __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_mstate_global->__pyx_n_u_malloc_2); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1614, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      if (!__Pyx_IsSameCFunction(__pyx_t_1, (void(*)(void)) __pyx_pw_4cupy_4cuda_6memory_10MemoryPool_3malloc)) {
        __Pyx_XDECREF((PyObject *)__pyx_r);
        __pyx_t_3 = NULL;
        __Pyx_INCREF(__pyx_t_1);
        __pyx_t_4 = __pyx_t_1; 
        __pyx_t_5 = __Pyx_PyLong_FromSize_t(__pyx_v_size); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 1614, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_5);
        __pyx_t_6 = 1;
        #if CYTHON_UNPACK_METHODS
        if (unlikely(PyMethod_Check(__pyx_t_4))) {
          __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_4);
          assert(__pyx_t_3);
          PyObject* __pyx__function = PyMethod_GET_FUNCTION(__pyx_t_4);
          __Pyx_INCREF(__pyx_t_3);
          __Pyx_INCREF(__pyx__function);
          __Pyx_DECREF_SET(__pyx_t_4, __pyx__function);
          __pyx_t_6 = 0;
        }
        #endif
        {
          PyObject *__pyx_callargs[2] = {__pyx_t_3, __pyx_t_5};
          __pyx_t_2 = __Pyx_PyObject_FastCall((PyObject*)__pyx_t_4, __pyx_callargs+__pyx_t_6, (2-__pyx_t_6) | (__pyx_t_6*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
          __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
          __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
          __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
          if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1614, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_2);
        }
        if (!(likely(((__pyx_t_2) == Py_None) || likely(__Pyx_TypeTest(__pyx_t_2, __pyx_mstate_global->__pyx_ptype_4cupy_4cuda_6memory_MemoryPointer))))) __PYX_ERR(0, 1614, __pyx_L1_error)
        __pyx_r = ((struct __pyx_obj_4cupy_4cuda_6memory_MemoryPointer *)__pyx_t_2);
        __pyx_t_2 = 0;
        __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
        goto __pyx_L0;
      }
      #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
      __pyx_tp_dict_version = __Pyx_get_tp_dict_version(((PyObject *)__pyx_v_self));
      __pyx_obj_dict_version = __Pyx_get_object_dict_version(((PyObject *)__pyx_v_self));
      if (unlikely(__pyx_typedict_guard != __pyx_tp_dict_version)) {
        __pyx_tp_dict_version = __pyx_obj_dict_version = __PYX_DICT_VERSION_INIT;
      }
      #endif
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
    }
    #endif
  }

  /* "cupy/cuda/memory.pyx":1634
 * 
 *         """
 *         mp = <SingleDeviceMemoryPool>self.device_pool()             # <<<<<<<<<<<<<<
 *         return mp.malloc(size)
 * 
*/
  __pyx_t_1 = __pyx_f_4cupy_4cuda_6memory_10MemoryPool_device_pool(__pyx_v_self); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1634, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __pyx_t_1;
  __Pyx_INCREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v_mp = ((struct __pyx_obj_4cupy_4cuda_6memory_SingleDeviceMemoryPool *)__pyx_t_2);
  __pyx_t_2 = 0;

  /* "cupy/cuda/memory.pyx":1635
 *         """
 *         mp = <SingleDeviceMemoryPool>self.device_pool()
 *         return mp.malloc(size)             # <<<<<<<<<<<<<<
 * 
 *     cpdef free_all_blocks(self, stream=None):
*/
  __Pyx_XDECREF((PyObject *)__pyx_r);
  __pyx_t_2 = ((PyObject *)__pyx_f_4cupy_4cuda_6memory_22SingleDeviceMemoryPool_malloc(__pyx_v_mp, __pyx_v_size, 0)); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1635, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_r = ((struct __pyx_obj_4cupy_4cuda_6memory_MemoryPointer *)__pyx_t_2);
  __pyx_t_2 = 0;
  goto __pyx_L0;

  /* "cupy/cuda/memory.pyx":1614
 *         return self._pools[device.get_device_id()]
 * 
 *     cpdef MemoryPointer malloc(self, size_t size):             # <<<<<<<<<<<<<<
 *         """Allocates the memory, from the pool if possible.
 * 
*/

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_AddTraceback("cupy.cuda.memory.MemoryPool.malloc", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XDECREF((PyObject *)__pyx_v_mp);
  __Pyx_XGIVEREF((PyObject *)__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_pw_4cupy_4cuda_6memory_10MemoryPool_3malloc(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
PyDoc_STRVAR(__pyx_doc_4cupy_4cuda_6memory_10MemoryPool_2malloc, "MemoryPool.malloc(self, size_t size) -> MemoryPointer\n\nAllocates the memory, from the pool if possible.\n\nThis method can be used as a CuPy memory allocator. The simplest way to\nuse a memory pool as the default allocator is the following code::\n\n    set_allocator(MemoryPool().malloc)\n\nAlso, the way to use a memory pool of Managed memory (Unified memory)\nas the default allocator is the following code::\n\n    set_allocator(MemoryPool(malloc_managed).malloc)\n\nArgs:\n    size (int): Size of the memory buffer to allocate in bytes.\n\nReturns:\n    ~cupy.cuda.MemoryPointer: Pointer to the allocated buffer.");
static PyObject *__pyx_pw_4cupy_4cuda_6memory_10MemoryPool_3malloc(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
) {
  size_t __pyx_v_size;
  #if !CYTHON_METH_FASTCALL
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  #endif
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject* values[1] = {0};
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("malloc (wrapper)", 0);
  #if !CYTHON_METH_FASTCALL
  #if CYTHON_ASSUME_SAFE_SIZE
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return NULL;
  #endif
  #endif
  __pyx_kwvalues = __Pyx_KwValues_FASTCALL(__pyx_args, __pyx_nargs);
  {
    PyObject ** const __pyx_pyargnames[] = {&__pyx_mstate_global->__pyx_n_u_size,0};
    const Py_ssize_t __pyx_kwds_len = (__pyx_kwds) ? __Pyx_NumKwargs_FASTCALL(__pyx_kwds) : 0;
    if (unlikely(__pyx_kwds_len < 0)) __PYX_ERR(0, 1614, __pyx_L3_error)
    if (__pyx_kwds_len > 0) {
      switch (__pyx_nargs) {
        case  1:
        values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(0, 1614, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      const Py_ssize_t kwd_pos_args = __pyx_nargs;
      if (__Pyx_ParseKeywords(__pyx_kwds, __pyx_kwvalues, __pyx_pyargnames, 0, values, kwd_pos_args, __pyx_kwds_len, "malloc", 0) < (0)) __PYX_ERR(0, 1614, __pyx_L3_error)
      for (Py_ssize_t i = __pyx_nargs; i < 1; i++) {
        if (unlikely(!values[i])) { __Pyx_RaiseArgtupleInvalid("malloc", 1, 1, 1, i); __PYX_ERR(0, 1614, __pyx_L3_error) }
      }
    } else if (unlikely(__pyx_nargs != 1)) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
      if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(0, 1614, __pyx_L3_error)
    }
    __pyx_v_size = __Pyx_PyLong_As_size_t(values[0]); if (unlikely((__pyx_v_size == (size_t)-1) && PyErr_Occurred())) __PYX_ERR(0, 1614, __pyx_L3_error)
  }
  goto __pyx_L6_skip;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("malloc", 1, 1, 1, __pyx_nargs); __PYX_ERR(0, 1614, __pyx_L3_error)
  __pyx_L6_skip:;
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_AddTraceback("cupy.cuda.memory.MemoryPool.malloc", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_4cupy_4cuda_6memory_10MemoryPool_2malloc(((struct __pyx_obj_4cupy_4cuda_6memory_MemoryPool *)__pyx_v_self), __pyx_v_size);

  /* function exit code */
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4cupy_4cuda_6memory_10MemoryPool_2malloc(struct __pyx_obj_4cupy_4cuda_6memory_MemoryPool *__pyx_v_self, size_t __pyx_v_size) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("malloc", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = ((PyObject *)__pyx_f_4cupy_4cuda_6memory_10MemoryPool_malloc(__pyx_v_self, __pyx_v_size, 1)); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1614, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("cupy.cuda.memory.MemoryPool.malloc", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "cupy/cuda/memory.pyx":1637
 *         return mp.malloc(size)
 * 
 *     cpdef free_all_blocks(self, stream=None):             # <<<<<<<<<<<<<<
 *         """Releases free blocks.
 * 
*/

static PyObject *__pyx_pw_4cupy_4cuda_6memory_10MemoryPool_5free_all_blocks(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
static PyObject *__pyx_f_4cupy_4cuda_6memory_10MemoryPool_free_all_blocks(struct __pyx_obj_4cupy_4cuda_6memory_MemoryPool *__pyx_v_self, int __pyx_skip_dispatch, struct __pyx_opt_args_4cupy_4cuda_6memory_10MemoryPool_free_all_blocks *__pyx_optional_args) {
  PyObject *__pyx_v_stream = ((PyObject *)Py_None);
  struct __pyx_obj_4cupy_4cuda_6memory_SingleDeviceMemoryPool *__pyx_v_mp = NULL;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  size_t __pyx_t_5;
  struct __pyx_opt_args_4cupy_4cuda_6memory_22SingleDeviceMemoryPool_free_all_blocks __pyx_t_6;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("free_all_blocks", 0);
  if (__pyx_optional_args) {
    if (__pyx_optional_args->__pyx_n > 0) {
      __pyx_v_stream = __pyx_optional_args->stream;
    }
  }
  /* Check if called by wrapper */
  if (unlikely(__pyx_skip_dispatch)) ;
  /* Check if overridden in Python */
  else if (
  #if !CYTHON_USE_TYPE_SLOTS
  unlikely(Py_TYPE(((PyObject *)__pyx_v_self)) != __pyx_mstate_global->__pyx_ptype_4cupy_4cuda_6memory_MemoryPool &&
  __Pyx_PyType_HasFeature(Py_TYPE(((PyObject *)__pyx_v_self)), Py_TPFLAGS_HAVE_GC))
  #else
  unlikely(Py_TYPE(((PyObject *)__pyx_v_self))->tp_dictoffset != 0 || __Pyx_PyType_HasFeature(Py_TYPE(((PyObject *)__pyx_v_self)), (Py_TPFLAGS_IS_ABSTRACT | Py_TPFLAGS_HEAPTYPE)))
  #endif
  ) {
    #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
    static PY_UINT64_T __pyx_tp_dict_version = __PYX_DICT_VERSION_INIT, __pyx_obj_dict_version = __PYX_DICT_VERSION_INIT;
    if (unlikely(!__Pyx_object_dict_version_matches(((PyObject *)__pyx_v_self), __pyx_tp_dict_version, __pyx_obj_dict_version))) {
      PY_UINT64_T __pyx_typedict_guard = __Pyx_get_tp_dict_version(((PyObject *)__pyx_v_self));
      #endif
      __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_mstate_global->__pyx_n_u_free_all_blocks); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1637, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      if (!__Pyx_IsSameCFunction(__pyx_t_1, (void(*)(void)) __pyx_pw_4cupy_4cuda_6memory_10MemoryPool_5free_all_blocks)) {
        __Pyx_XDECREF(__pyx_r);
        __pyx_t_3 = NULL;
        __Pyx_INCREF(__pyx_t_1);
        __pyx_t_4 = __pyx_t_1; 
        __pyx_t_5 = 1;
        #if CYTHON_UNPACK_METHODS
        if (unlikely(PyMethod_Check(__pyx_t_4))) {
          __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_4);
          assert(__pyx_t_3);
          PyObject* __pyx__function = PyMethod_GET_FUNCTION(__pyx_t_4);
          __Pyx_INCREF(__pyx_t_3);
          __Pyx_INCREF(__pyx__function);
          __Pyx_DECREF_SET(__pyx_t_4, __pyx__function);
          __pyx_t_5 = 0;
        }
        #endif
        {
          PyObject *__pyx_callargs[2] = {__pyx_t_3, __pyx_v_stream};
          __pyx_t_2 = __Pyx_PyObject_FastCall((PyObject*)__pyx_t_4, __pyx_callargs+__pyx_t_5, (2-__pyx_t_5) | (__pyx_t_5*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
          __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
          __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
          if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1637, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_2);
        }
        __pyx_r = __pyx_t_2;
        __pyx_t_2 = 0;
        __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
        goto __pyx_L0;
      }
      #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
      __pyx_tp_dict_version = __Pyx_get_tp_dict_version(((PyObject *)__pyx_v_self));
      __pyx_obj_dict_version = __Pyx_get_object_dict_version(((PyObject *)__pyx_v_self));
      if (unlikely(__pyx_typedict_guard != __pyx_tp_dict_version)) {
        __pyx_tp_dict_version = __pyx_obj_dict_version = __PYX_DICT_VERSION_INIT;
      }
      #endif
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
    }
    #endif
  }

  /* "cupy/cuda/memory.pyx":1650
 *             even if :meth:`free_all_blocks` is called.
 *         """
 *         mp = <SingleDeviceMemoryPool>self.device_pool()             # <<<<<<<<<<<<<<
 *         mp.free_all_blocks(stream=stream)
 * 
*/
  __pyx_t_1 = __pyx_f_4cupy_4cuda_6memory_10MemoryPool_device_pool(__pyx_v_self); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1650, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __pyx_t_1;
  __Pyx_INCREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v_mp = ((struct __pyx_obj_4cupy_4cuda_6memory_SingleDeviceMemoryPool *)__pyx_t_2);
  __pyx_t_2 = 0;

  /* "cupy/cuda/memory.pyx":1651
 *         """
 *         mp = <SingleDeviceMemoryPool>self.device_pool()
 *         mp.free_all_blocks(stream=stream)             # <<<<<<<<<<<<<<
 * 
 *     cpdef free_all_free(self):
*/
  __pyx_t_6.__pyx_n = 1;
  __pyx_t_6.stream = __pyx_v_stream;
  __pyx_t_2 = __pyx_f_4cupy_4cuda_6memory_22SingleDeviceMemoryPool_free_all_blocks(__pyx_v_mp, 0, &__pyx_t_6); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1651, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "cupy/cuda/memory.pyx":1637
 *         return mp.malloc(size)
 * 
 *     cpdef free_all_blocks(self, stream=None):             # <<<<<<<<<<<<<<
 *         """Releases free blocks.
 * 
*/

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_AddTraceback("cupy.cuda.memory.MemoryPool.free_all_blocks", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XDECREF((PyObject *)__pyx_v_mp);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_pw_4cupy_4cuda_6memory_10MemoryPool_5free_all_blocks(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
PyDoc_STRVAR(__pyx_doc_4cupy_4cuda_6memory_10MemoryPool_4free_all_blocks, "MemoryPool.free_all_blocks(self, stream=None)\n\nReleases free blocks.\n\nArgs:\n    stream (cupy.cuda.Stream): Release free blocks in the arena\n        of the given stream. The default releases blocks in all\n        arenas.\n\n.. note::\n    A memory pool may split a free block for space efficiency. A split\n    block is not released until all its parts are merged back into one\n    even if :meth:`free_all_blocks` is called.");
static PyObject *__pyx_pw_4cupy_4cuda_6memory_10MemoryPool_5free_all_blocks(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
) {
  PyObject *__pyx_v_stream = 0;
  #if !CYTHON_METH_FASTCALL
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  #endif
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject* values[1] = {0};
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("free_all_blocks (wrapper)", 0);
  #if !CYTHON_METH_FASTCALL
  #if CYTHON_ASSUME_SAFE_SIZE
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return NULL;
  #endif
  #endif
  __pyx_kwvalues = __Pyx_KwValues_FASTCALL(__pyx_args, __pyx_nargs);
  {
    PyObject ** const __pyx_pyargnames[] = {&__pyx_mstate_global->__pyx_n_u_stream,0};
    const Py_ssize_t __pyx_kwds_len = (__pyx_kwds) ? __Pyx_NumKwargs_FASTCALL(__pyx_kwds) : 0;
    if (unlikely(__pyx_kwds_len < 0)) __PYX_ERR(0, 1637, __pyx_L3_error)
    if (__pyx_kwds_len > 0) {
      switch (__pyx_nargs) {
        case  1:
        values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(0, 1637, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      const Py_ssize_t kwd_pos_args = __pyx_nargs;
      if (__Pyx_ParseKeywords(__pyx_kwds, __pyx_kwvalues, __pyx_pyargnames, 0, values, kwd_pos_args, __pyx_kwds_len, "free_all_blocks", 0) < (0)) __PYX_ERR(0, 1637, __pyx_L3_error)
      if (!values[0]) values[0] = __Pyx_NewRef(((PyObject *)Py_None));
    } else {
      switch (__pyx_nargs) {
        case  1:
        values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(0, 1637, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      if (!values[0]) values[0] = __Pyx_NewRef(((PyObject *)Py_None));
    }
    __pyx_v_stream = values[0];
  }
  goto __pyx_L6_skip;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("free_all_blocks", 0, 0, 1, __pyx_nargs); __PYX_ERR(0, 1637, __pyx_L3_error)
  __pyx_L6_skip:;
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_AddTraceback("cupy.cuda.memory.MemoryPool.free_all_blocks", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_4cupy_4cuda_6memory_10MemoryPool_4free_all_blocks(((struct __pyx_obj_4cupy_4cuda_6memory_MemoryPool *)__pyx_v_self), __pyx_v_stream);

  /* function exit code */
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4cupy_4cuda_6memory_10MemoryPool_4free_all_blocks(struct __pyx_obj_4cupy_4cuda_6memory_MemoryPool *__pyx_v_self, PyObject *__pyx_v_stream) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  struct __pyx_opt_args_4cupy_4cuda_6memory_10MemoryPool_free_all_blocks __pyx_t_2;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("free_all_blocks", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_2.__pyx_n = 1;
  __pyx_t_2.stream = __pyx_v_stream;
  __pyx_t_1 = __pyx_vtabptr_4cupy_4cuda_6memory_MemoryPool->free_all_blocks(__pyx_v_self, 1, &__pyx_t_2); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1637, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("cupy.cuda.memory.MemoryPool.free_all_blocks", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "cupy/cuda/memory.pyx":1653
 *         mp.free_all_blocks(stream=stream)
 * 
 *     cpdef free_all_free(self):             # <<<<<<<<<<<<<<
 *         """(Deprecated) Use :meth:`free_all_blocks` instead."""
 *         warnings.warn(
*/

static PyObject *__pyx_pw_4cupy_4cuda_6memory_10MemoryPool_7free_all_free(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
static PyObject *__pyx_f_4cupy_4cuda_6memory_10MemoryPool_free_all_free(struct __pyx_obj_4cupy_4cuda_6memory_MemoryPool *__pyx_v_self, int __pyx_skip_dispatch) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  size_t __pyx_t_5;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("free_all_free", 0);
  /* Check if called by wrapper */
  if (unlikely(__pyx_skip_dispatch)) ;
  /* Check if overridden in Python */
  else if (
  #if !CYTHON_USE_TYPE_SLOTS
  unlikely(Py_TYPE(((PyObject *)__pyx_v_self)) != __pyx_mstate_global->__pyx_ptype_4cupy_4cuda_6memory_MemoryPool &&
  __Pyx_PyType_HasFeature(Py_TYPE(((PyObject *)__pyx_v_self)), Py_TPFLAGS_HAVE_GC))
  #else
  unlikely(Py_TYPE(((PyObject *)__pyx_v_self))->tp_dictoffset != 0 || __Pyx_PyType_HasFeature(Py_TYPE(((PyObject *)__pyx_v_self)), (Py_TPFLAGS_IS_ABSTRACT | Py_TPFLAGS_HEAPTYPE)))
  #endif
  ) {
    #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
    static PY_UINT64_T __pyx_tp_dict_version = __PYX_DICT_VERSION_INIT, __pyx_obj_dict_version = __PYX_DICT_VERSION_INIT;
    if (unlikely(!__Pyx_object_dict_version_matches(((PyObject *)__pyx_v_self), __pyx_tp_dict_version, __pyx_obj_dict_version))) {
      PY_UINT64_T __pyx_typedict_guard = __Pyx_get_tp_dict_version(((PyObject *)__pyx_v_self));
      #endif
      __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_mstate_global->__pyx_n_u_free_all_free); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1653, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      if (!__Pyx_IsSameCFunction(__pyx_t_1, (void(*)(void)) __pyx_pw_4cupy_4cuda_6memory_10MemoryPool_7free_all_free)) {
        __Pyx_XDECREF(__pyx_r);
        __pyx_t_3 = NULL;
        __Pyx_INCREF(__pyx_t_1);
        __pyx_t_4 = __pyx_t_1; 
        __pyx_t_5 = 1;
        #if CYTHON_UNPACK_METHODS
        if (unlikely(PyMethod_Check(__pyx_t_4))) {
          __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_4);
          assert(__pyx_t_3);
          PyObject* __pyx__function = PyMethod_GET_FUNCTION(__pyx_t_4);
          __Pyx_INCREF(__pyx_t_3);
          __Pyx_INCREF(__pyx__function);
          __Pyx_DECREF_SET(__pyx_t_4, __pyx__function);
          __pyx_t_5 = 0;
        }
        #endif
        {
          PyObject *__pyx_callargs[2] = {__pyx_t_3, NULL};
          __pyx_t_2 = __Pyx_PyObject_FastCall((PyObject*)__pyx_t_4, __pyx_callargs+__pyx_t_5, (1-__pyx_t_5) | (__pyx_t_5*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
          __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
          __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
          if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1653, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_2);
        }
        __pyx_r = __pyx_t_2;
        __pyx_t_2 = 0;
        __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
        goto __pyx_L0;
      }
      #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
      __pyx_tp_dict_version = __Pyx_get_tp_dict_version(((PyObject *)__pyx_v_self));
      __pyx_obj_dict_version = __Pyx_get_object_dict_version(((PyObject *)__pyx_v_self));
      if (unlikely(__pyx_typedict_guard != __pyx_tp_dict_version)) {
        __pyx_tp_dict_version = __pyx_obj_dict_version = __PYX_DICT_VERSION_INIT;
      }
      #endif
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
    }
    #endif
  }

  /* "cupy/cuda/memory.pyx":1655
 *     cpdef free_all_free(self):
 *         """(Deprecated) Use :meth:`free_all_blocks` instead."""
 *         warnings.warn(             # <<<<<<<<<<<<<<
 *             'free_all_free is deprecated. Use free_all_blocks instead.',
 *             DeprecationWarning)
*/
  __pyx_t_2 = NULL;
  __Pyx_GetModuleGlobalName(__pyx_t_4, __pyx_mstate_global->__pyx_n_u_warnings); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 1655, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_t_4, __pyx_mstate_global->__pyx_n_u_warn); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 1655, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

  /* "cupy/cuda/memory.pyx":1657
 *         warnings.warn(
 *             'free_all_free is deprecated. Use free_all_blocks instead.',
 *             DeprecationWarning)             # <<<<<<<<<<<<<<
 *         self.free_all_blocks()
 * 
*/
  __pyx_t_5 = 1;
  #if CYTHON_UNPACK_METHODS
  if (unlikely(PyMethod_Check(__pyx_t_3))) {
    __pyx_t_2 = PyMethod_GET_SELF(__pyx_t_3);
    assert(__pyx_t_2);
    PyObject* __pyx__function = PyMethod_GET_FUNCTION(__pyx_t_3);
    __Pyx_INCREF(__pyx_t_2);
    __Pyx_INCREF(__pyx__function);
    __Pyx_DECREF_SET(__pyx_t_3, __pyx__function);
    __pyx_t_5 = 0;
  }
  #endif
  {
    PyObject *__pyx_callargs[3] = {__pyx_t_2, __pyx_mstate_global->__pyx_kp_u_free_all_free_is_deprecated_Use, ((PyObject *)(((PyTypeObject*)PyExc_DeprecationWarning)))};
    __pyx_t_1 = __Pyx_PyObject_FastCall((PyObject*)__pyx_t_3, __pyx_callargs+__pyx_t_5, (3-__pyx_t_5) | (__pyx_t_5*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
    __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1655, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
  }
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "cupy/cuda/memory.pyx":1658
 *             'free_all_free is deprecated. Use free_all_blocks instead.',
 *             DeprecationWarning)
 *         self.free_all_blocks()             # <<<<<<<<<<<<<<
 * 
 *     cpdef size_t n_free_blocks(self):
*/
  __pyx_t_1 = ((struct __pyx_vtabstruct_4cupy_4cuda_6memory_MemoryPool *)__pyx_v_self->__pyx_vtab)->free_all_blocks(__pyx_v_self, 0, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1658, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "cupy/cuda/memory.pyx":1653
 *         mp.free_all_blocks(stream=stream)
 * 
 *     cpdef free_all_free(self):             # <<<<<<<<<<<<<<
 *         """(Deprecated) Use :meth:`free_all_blocks` instead."""
 *         warnings.warn(
*/

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_AddTraceback("cupy.cuda.memory.MemoryPool.free_all_free", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_pw_4cupy_4cuda_6memory_10MemoryPool_7free_all_free(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
PyDoc_STRVAR(__pyx_doc_4cupy_4cuda_6memory_10MemoryPool_6free_all_free, "MemoryPool.free_all_free(self)\n\n(Deprecated) Use :meth:`free_all_blocks` instead.");
static PyObject *__pyx_pw_4cupy_4cuda_6memory_10MemoryPool_7free_all_free(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
) {
  #if !CYTHON_METH_FASTCALL
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  #endif
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("free_all_free (wrapper)", 0);
  #if !CYTHON_METH_FASTCALL
  #if CYTHON_ASSUME_SAFE_SIZE
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return NULL;
  #endif
  #endif
  __pyx_kwvalues = __Pyx_KwValues_FASTCALL(__pyx_args, __pyx_nargs);
  if (unlikely(__pyx_nargs > 0)) { __Pyx_RaiseArgtupleInvalid("free_all_free", 1, 0, 0, __pyx_nargs); return NULL; }
  const Py_ssize_t __pyx_kwds_len = unlikely(__pyx_kwds) ? __Pyx_NumKwargs_FASTCALL(__pyx_kwds) : 0;
  if (unlikely(__pyx_kwds_len < 0)) return NULL;
  if (unlikely(__pyx_kwds_len > 0)) {__Pyx_RejectKeywords("free_all_free", __pyx_kwds); return NULL;}
  __pyx_r = __pyx_pf_4cupy_4cuda_6memory_10MemoryPool_6free_all_free(((struct __pyx_obj_4cupy_4cuda_6memory_MemoryPool *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4cupy_4cuda_6memory_10MemoryPool_6free_all_free(struct __pyx_obj_4cupy_4cuda_6memory_MemoryPool *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("free_all_free", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __pyx_f_4cupy_4cuda_6memory_10MemoryPool_free_all_free(__pyx_v_self, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1653, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("cupy.cuda.memory.MemoryPool.free_all_free", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "cupy/cuda/memory.pyx":1660
 *         self.free_all_blocks()
 * 
 *     cpdef size_t n_free_blocks(self):             # <<<<<<<<<<<<<<
 *         """Counts the total number of free blocks.
 * 
*/

static PyObject *__pyx_pw_4cupy_4cuda_6memory_10MemoryPool_9n_free_blocks(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
static size_t __pyx_f_4cupy_4cuda_6memory_10MemoryPool_n_free_blocks(struct __pyx_obj_4cupy_4cuda_6memory_MemoryPool *__pyx_v_self, int __pyx_skip_dispatch) {
  struct __pyx_obj_4cupy_4cuda_6memory_SingleDeviceMemoryPool *__pyx_v_mp = NULL;
  size_t __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  size_t __pyx_t_5;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("n_free_blocks", 0);
  /* Check if called by wrapper */
  if (unlikely(__pyx_skip_dispatch)) ;
  /* Check if overridden in Python */
  else if (
  #if !CYTHON_USE_TYPE_SLOTS
  unlikely(Py_TYPE(((PyObject *)__pyx_v_self)) != __pyx_mstate_global->__pyx_ptype_4cupy_4cuda_6memory_MemoryPool &&
  __Pyx_PyType_HasFeature(Py_TYPE(((PyObject *)__pyx_v_self)), Py_TPFLAGS_HAVE_GC))
  #else
  unlikely(Py_TYPE(((PyObject *)__pyx_v_self))->tp_dictoffset != 0 || __Pyx_PyType_HasFeature(Py_TYPE(((PyObject *)__pyx_v_self)), (Py_TPFLAGS_IS_ABSTRACT | Py_TPFLAGS_HEAPTYPE)))
  #endif
  ) {
    #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
    static PY_UINT64_T __pyx_tp_dict_version = __PYX_DICT_VERSION_INIT, __pyx_obj_dict_version = __PYX_DICT_VERSION_INIT;
    if (unlikely(!__Pyx_object_dict_version_matches(((PyObject *)__pyx_v_self), __pyx_tp_dict_version, __pyx_obj_dict_version))) {
      PY_UINT64_T __pyx_typedict_guard = __Pyx_get_tp_dict_version(((PyObject *)__pyx_v_self));
      #endif
      __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_mstate_global->__pyx_n_u_n_free_blocks); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1660, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      if (!__Pyx_IsSameCFunction(__pyx_t_1, (void(*)(void)) __pyx_pw_4cupy_4cuda_6memory_10MemoryPool_9n_free_blocks)) {
        __pyx_t_3 = NULL;
        __Pyx_INCREF(__pyx_t_1);
        __pyx_t_4 = __pyx_t_1; 
        __pyx_t_5 = 1;
        #if CYTHON_UNPACK_METHODS
        if (unlikely(PyMethod_Check(__pyx_t_4))) {
          __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_4);
          assert(__pyx_t_3);
          PyObject* __pyx__function = PyMethod_GET_FUNCTION(__pyx_t_4);
          __Pyx_INCREF(__pyx_t_3);
          __Pyx_INCREF(__pyx__function);
          __Pyx_DECREF_SET(__pyx_t_4, __pyx__function);
          __pyx_t_5 = 0;
        }
        #endif
        {
          PyObject *__pyx_callargs[2] = {__pyx_t_3, NULL};
          __pyx_t_2 = __Pyx_PyObject_FastCall((PyObject*)__pyx_t_4, __pyx_callargs+__pyx_t_5, (1-__pyx_t_5) | (__pyx_t_5*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
          __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
          __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
          if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1660, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_2);
        }
        __pyx_t_5 = __Pyx_PyLong_As_size_t(__pyx_t_2); if (unlikely((__pyx_t_5 == (size_t)-1) && PyErr_Occurred())) __PYX_ERR(0, 1660, __pyx_L1_error)
        __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
        __pyx_r = __pyx_t_5;
        __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
        goto __pyx_L0;
      }
      #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
      __pyx_tp_dict_version = __Pyx_get_tp_dict_version(((PyObject *)__pyx_v_self));
      __pyx_obj_dict_version = __Pyx_get_object_dict_version(((PyObject *)__pyx_v_self));
      if (unlikely(__pyx_typedict_guard != __pyx_tp_dict_version)) {
        __pyx_tp_dict_version = __pyx_obj_dict_version = __PYX_DICT_VERSION_INIT;
      }
      #endif
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
    }
    #endif
  }

  /* "cupy/cuda/memory.pyx":1666
 *             int: The total number of free blocks.
 *         """
 *         mp = <SingleDeviceMemoryPool>self.device_pool()             # <<<<<<<<<<<<<<
 *         return mp.n_free_blocks()
 * 
*/
  __pyx_t_1 = __pyx_f_4cupy_4cuda_6memory_10MemoryPool_device_pool(__pyx_v_self); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1666, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __pyx_t_1;
  __Pyx_INCREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v_mp = ((struct __pyx_obj_4cupy_4cuda_6memory_SingleDeviceMemoryPool *)__pyx_t_2);
  __pyx_t_2 = 0;

  /* "cupy/cuda/memory.pyx":1667
 *         """
 *         mp = <SingleDeviceMemoryPool>self.device_pool()
 *         return mp.n_free_blocks()             # <<<<<<<<<<<<<<
 * 
 *     cpdef size_t used_bytes(self):
*/
  __pyx_t_5 = __pyx_f_4cupy_4cuda_6memory_22SingleDeviceMemoryPool_n_free_blocks(__pyx_v_mp, 0); if (unlikely(__pyx_t_5 == ((size_t)-1L))) __PYX_ERR(0, 1667, __pyx_L1_error)
  __pyx_r = __pyx_t_5;
  goto __pyx_L0;

  /* "cupy/cuda/memory.pyx":1660
 *         self.free_all_blocks()
 * 
 *     cpdef size_t n_free_blocks(self):             # <<<<<<<<<<<<<<
 *         """Counts the total number of free blocks.
 * 
*/

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_AddTraceback("cupy.cuda.memory.MemoryPool.n_free_blocks", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XDECREF((PyObject *)__pyx_v_mp);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_pw_4cupy_4cuda_6memory_10MemoryPool_9n_free_blocks(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
PyDoc_STRVAR(__pyx_doc_4cupy_4cuda_6memory_10MemoryPool_8n_free_blocks, "MemoryPool.n_free_blocks(self) -> size_t\n\nCounts the total number of free blocks.\n\nReturns:\n    int: The total number of free blocks.");
static PyObject *__pyx_pw_4cupy_4cuda_6memory_10MemoryPool_9n_free_blocks(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
) {
  #if !CYTHON_METH_FASTCALL
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  #endif
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("n_free_blocks (wrapper)", 0);
  #if !CYTHON_METH_FASTCALL
  #if CYTHON_ASSUME_SAFE_SIZE
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return NULL;
  #endif
  #endif
  __pyx_kwvalues = __Pyx_KwValues_FASTCALL(__pyx_args, __pyx_nargs);
  if (unlikely(__pyx_nargs > 0)) { __Pyx_RaiseArgtupleInvalid("n_free_blocks", 1, 0, 0, __pyx_nargs); return NULL; }
  const Py_ssize_t __pyx_kwds_len = unlikely(__pyx_kwds) ? __Pyx_NumKwargs_FASTCALL(__pyx_kwds) : 0;
  if (unlikely(__pyx_kwds_len < 0)) return NULL;
  if (unlikely(__pyx_kwds_len > 0)) {__Pyx_RejectKeywords("n_free_blocks", __pyx_kwds); return NULL;}
  __pyx_r = __pyx_pf_4cupy_4cuda_6memory_10MemoryPool_8n_free_blocks(((struct __pyx_obj_4cupy_4cuda_6memory_MemoryPool *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4cupy_4cuda_6memory_10MemoryPool_8n_free_blocks(struct __pyx_obj_4cupy_4cuda_6memory_MemoryPool *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  size_t __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("n_free_blocks", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __pyx_f_4cupy_4cuda_6memory_10MemoryPool_n_free_blocks(__pyx_v_self, 1); if (unlikely(PyErr_Occurred())) __PYX_ERR(0, 1660, __pyx_L1_error)
  __pyx_t_2 = __Pyx_PyLong_FromSize_t(__pyx_t_1); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1660, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_r = __pyx_t_2;
  __pyx_t_2 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_AddTraceback("cupy.cuda.memory.MemoryPool.n_free_blocks", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "cupy/cuda/memory.pyx":1669
 *         return mp.n_free_blocks()
 * 
 *     cpdef size_t used_bytes(self):             # <<<<<<<<<<<<<<
 *         """Gets the total number of bytes used by the pool.
 * 
*/

static PyObject *__pyx_pw_4cupy_4cuda_6memory_10MemoryPool_11used_bytes(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
static size_t __pyx_f_4cupy_4cuda_6memory_10MemoryPool_used_bytes(struct __pyx_obj_4cupy_4cuda_6memory_MemoryPool *__pyx_v_self, int __pyx_skip_dispatch) {
  struct __pyx_obj_4cupy_4cuda_6memory_SingleDeviceMemoryPool *__pyx_v_mp = NULL;
  size_t __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  size_t __pyx_t_5;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("used_bytes", 0);
  /* Check if called by wrapper */
  if (unlikely(__pyx_skip_dispatch)) ;
  /* Check if overridden in Python */
  else if (
  #if !CYTHON_USE_TYPE_SLOTS
  unlikely(Py_TYPE(((PyObject *)__pyx_v_self)) != __pyx_mstate_global->__pyx_ptype_4cupy_4cuda_6memory_MemoryPool &&
  __Pyx_PyType_HasFeature(Py_TYPE(((PyObject *)__pyx_v_self)), Py_TPFLAGS_HAVE_GC))
  #else
  unlikely(Py_TYPE(((PyObject *)__pyx_v_self))->tp_dictoffset != 0 || __Pyx_PyType_HasFeature(Py_TYPE(((PyObject *)__pyx_v_self)), (Py_TPFLAGS_IS_ABSTRACT | Py_TPFLAGS_HEAPTYPE)))
  #endif
  ) {
    #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
    static PY_UINT64_T __pyx_tp_dict_version = __PYX_DICT_VERSION_INIT, __pyx_obj_dict_version = __PYX_DICT_VERSION_INIT;
    if (unlikely(!__Pyx_object_dict_version_matches(((PyObject *)__pyx_v_self), __pyx_tp_dict_version, __pyx_obj_dict_version))) {
      PY_UINT64_T __pyx_typedict_guard = __Pyx_get_tp_dict_version(((PyObject *)__pyx_v_self));
      #endif
      __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_mstate_global->__pyx_n_u_used_bytes); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1669, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      if (!__Pyx_IsSameCFunction(__pyx_t_1, (void(*)(void)) __pyx_pw_4cupy_4cuda_6memory_10MemoryPool_11used_bytes)) {
        __pyx_t_3 = NULL;
        __Pyx_INCREF(__pyx_t_1);
        __pyx_t_4 = __pyx_t_1; 
        __pyx_t_5 = 1;
        #if CYTHON_UNPACK_METHODS
        if (unlikely(PyMethod_Check(__pyx_t_4))) {
          __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_4);
          assert(__pyx_t_3);
          PyObject* __pyx__function = PyMethod_GET_FUNCTION(__pyx_t_4);
          __Pyx_INCREF(__pyx_t_3);
          __Pyx_INCREF(__pyx__function);
          __Pyx_DECREF_SET(__pyx_t_4, __pyx__function);
          __pyx_t_5 = 0;
        }
        #endif
        {
          PyObject *__pyx_callargs[2] = {__pyx_t_3, NULL};
          __pyx_t_2 = __Pyx_PyObject_FastCall((PyObject*)__pyx_t_4, __pyx_callargs+__pyx_t_5, (1-__pyx_t_5) | (__pyx_t_5*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
          __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
          __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
          if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1669, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_2);
        }
        __pyx_t_5 = __Pyx_PyLong_As_size_t(__pyx_t_2); if (unlikely((__pyx_t_5 == (size_t)-1) && PyErr_Occurred())) __PYX_ERR(0, 1669, __pyx_L1_error)
        __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
        __pyx_r = __pyx_t_5;
        __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
        goto __pyx_L0;
      }
      #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
      __pyx_tp_dict_version = __Pyx_get_tp_dict_version(((PyObject *)__pyx_v_self));
      __pyx_obj_dict_version = __Pyx_get_object_dict_version(((PyObject *)__pyx_v_self));
      if (unlikely(__pyx_typedict_guard != __pyx_tp_dict_version)) {
        __pyx_tp_dict_version = __pyx_obj_dict_version = __PYX_DICT_VERSION_INIT;
      }
      #endif
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
    }
    #endif
  }

  /* "cupy/cuda/memory.pyx":1675
 *             int: The total number of bytes used by the pool.
 *         """
 *         mp = <SingleDeviceMemoryPool>self.device_pool()             # <<<<<<<<<<<<<<
 *         return mp.used_bytes()
 * 
*/
  __pyx_t_1 = __pyx_f_4cupy_4cuda_6memory_10MemoryPool_device_pool(__pyx_v_self); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1675, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __pyx_t_1;
  __Pyx_INCREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v_mp = ((struct __pyx_obj_4cupy_4cuda_6memory_SingleDeviceMemoryPool *)__pyx_t_2);
  __pyx_t_2 = 0;

  /* "cupy/cuda/memory.pyx":1676
 *         """
 *         mp = <SingleDeviceMemoryPool>self.device_pool()
 *         return mp.used_bytes()             # <<<<<<<<<<<<<<
 * 
 *     cpdef size_t free_bytes(self):
*/
  __pyx_t_5 = __pyx_f_4cupy_4cuda_6memory_22SingleDeviceMemoryPool_used_bytes(__pyx_v_mp, 0); if (unlikely(PyErr_Occurred())) __PYX_ERR(0, 1676, __pyx_L1_error)
  __pyx_r = __pyx_t_5;
  goto __pyx_L0;

  /* "cupy/cuda/memory.pyx":1669
 *         return mp.n_free_blocks()
 * 
 *     cpdef size_t used_bytes(self):             # <<<<<<<<<<<<<<
 *         """Gets the total number of bytes used by the pool.
 * 
*/

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_AddTraceback("cupy.cuda.memory.MemoryPool.used_bytes", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XDECREF((PyObject *)__pyx_v_mp);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_pw_4cupy_4cuda_6memory_10MemoryPool_11used_bytes(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
PyDoc_STRVAR(__pyx_doc_4cupy_4cuda_6memory_10MemoryPool_10used_bytes, "MemoryPool.used_bytes(self) -> size_t\n\nGets the total number of bytes used by the pool.\n\nReturns:\n    int: The total number of bytes used by the pool.");
static PyObject *__pyx_pw_4cupy_4cuda_6memory_10MemoryPool_11used_bytes(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
) {
  #if !CYTHON_METH_FASTCALL
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  #endif
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("used_bytes (wrapper)", 0);
  #if !CYTHON_METH_FASTCALL
  #if CYTHON_ASSUME_SAFE_SIZE
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return NULL;
  #endif
  #endif
  __pyx_kwvalues = __Pyx_KwValues_FASTCALL(__pyx_args, __pyx_nargs);
  if (unlikely(__pyx_nargs > 0)) { __Pyx_RaiseArgtupleInvalid("used_bytes", 1, 0, 0, __pyx_nargs); return NULL; }
  const Py_ssize_t __pyx_kwds_len = unlikely(__pyx_kwds) ? __Pyx_NumKwargs_FASTCALL(__pyx_kwds) : 0;
  if (unlikely(__pyx_kwds_len < 0)) return NULL;
  if (unlikely(__pyx_kwds_len > 0)) {__Pyx_RejectKeywords("used_bytes", __pyx_kwds); return NULL;}
  __pyx_r = __pyx_pf_4cupy_4cuda_6memory_10MemoryPool_10used_bytes(((struct __pyx_obj_4cupy_4cuda_6memory_MemoryPool *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4cupy_4cuda_6memory_10MemoryPool_10used_bytes(struct __pyx_obj_4cupy_4cuda_6memory_MemoryPool *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  size_t __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("used_bytes", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __pyx_f_4cupy_4cuda_6memory_10MemoryPool_used_bytes(__pyx_v_self, 1); if (unlikely(PyErr_Occurred())) __PYX_ERR(0, 1669, __pyx_L1_error)
  __pyx_t_2 = __Pyx_PyLong_FromSize_t(__pyx_t_1); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1669, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_r = __pyx_t_2;
  __pyx_t_2 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_AddTraceback("cupy.cuda.memory.MemoryPool.used_bytes", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "cupy/cuda/memory.pyx":1678
 *         return mp.used_bytes()
 * 
 *     cpdef size_t free_bytes(self):             # <<<<<<<<<<<<<<
 *         """Gets the total number of bytes acquired but not used by the pool.
 * 
*/

static PyObject *__pyx_pw_4cupy_4cuda_6memory_10MemoryPool_13free_bytes(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
static size_t __pyx_f_4cupy_4cuda_6memory_10MemoryPool_free_bytes(struct __pyx_obj_4cupy_4cuda_6memory_MemoryPool *__pyx_v_self, int __pyx_skip_dispatch) {
  struct __pyx_obj_4cupy_4cuda_6memory_SingleDeviceMemoryPool *__pyx_v_mp = NULL;
  size_t __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  size_t __pyx_t_5;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("free_bytes", 0);
  /* Check if called by wrapper */
  if (unlikely(__pyx_skip_dispatch)) ;
  /* Check if overridden in Python */
  else if (
  #if !CYTHON_USE_TYPE_SLOTS
  unlikely(Py_TYPE(((PyObject *)__pyx_v_self)) != __pyx_mstate_global->__pyx_ptype_4cupy_4cuda_6memory_MemoryPool &&
  __Pyx_PyType_HasFeature(Py_TYPE(((PyObject *)__pyx_v_self)), Py_TPFLAGS_HAVE_GC))
  #else
  unlikely(Py_TYPE(((PyObject *)__pyx_v_self))->tp_dictoffset != 0 || __Pyx_PyType_HasFeature(Py_TYPE(((PyObject *)__pyx_v_self)), (Py_TPFLAGS_IS_ABSTRACT | Py_TPFLAGS_HEAPTYPE)))
  #endif
  ) {
    #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
    static PY_UINT64_T __pyx_tp_dict_version = __PYX_DICT_VERSION_INIT, __pyx_obj_dict_version = __PYX_DICT_VERSION_INIT;
    if (unlikely(!__Pyx_object_dict_version_matches(((PyObject *)__pyx_v_self), __pyx_tp_dict_version, __pyx_obj_dict_version))) {
      PY_UINT64_T __pyx_typedict_guard = __Pyx_get_tp_dict_version(((PyObject *)__pyx_v_self));
      #endif
      __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_mstate_global->__pyx_n_u_free_bytes); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1678, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      if (!__Pyx_IsSameCFunction(__pyx_t_1, (void(*)(void)) __pyx_pw_4cupy_4cuda_6memory_10MemoryPool_13free_bytes)) {
        __pyx_t_3 = NULL;
        __Pyx_INCREF(__pyx_t_1);
        __pyx_t_4 = __pyx_t_1; 
        __pyx_t_5 = 1;
        #if CYTHON_UNPACK_METHODS
        if (unlikely(PyMethod_Check(__pyx_t_4))) {
          __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_4);
          assert(__pyx_t_3);
          PyObject* __pyx__function = PyMethod_GET_FUNCTION(__pyx_t_4);
          __Pyx_INCREF(__pyx_t_3);
          __Pyx_INCREF(__pyx__function);
          __Pyx_DECREF_SET(__pyx_t_4, __pyx__function);
          __pyx_t_5 = 0;
        }
        #endif
        {
          PyObject *__pyx_callargs[2] = {__pyx_t_3, NULL};
          __pyx_t_2 = __Pyx_PyObject_FastCall((PyObject*)__pyx_t_4, __pyx_callargs+__pyx_t_5, (1-__pyx_t_5) | (__pyx_t_5*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
          __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
          __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
          if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1678, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_2);
        }
        __pyx_t_5 = __Pyx_PyLong_As_size_t(__pyx_t_2); if (unlikely((__pyx_t_5 == (size_t)-1) && PyErr_Occurred())) __PYX_ERR(0, 1678, __pyx_L1_error)
        __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
        __pyx_r = __pyx_t_5;
        __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
        goto __pyx_L0;
      }
      #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
      __pyx_tp_dict_version = __Pyx_get_tp_dict_version(((PyObject *)__pyx_v_self));
      __pyx_obj_dict_version = __Pyx_get_object_dict_version(((PyObject *)__pyx_v_self));
      if (unlikely(__pyx_typedict_guard != __pyx_tp_dict_version)) {
        __pyx_tp_dict_version = __pyx_obj_dict_version = __PYX_DICT_VERSION_INIT;
      }
      #endif
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
    }
    #endif
  }

  /* "cupy/cuda/memory.pyx":1684
 *             int: The total number of bytes acquired but not used by the pool.
 *         """
 *         mp = <SingleDeviceMemoryPool>self.device_pool()             # <<<<<<<<<<<<<<
 *         return mp.free_bytes()
 * 
*/
  __pyx_t_1 = __pyx_f_4cupy_4cuda_6memory_10MemoryPool_device_pool(__pyx_v_self); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1684, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __pyx_t_1;
  __Pyx_INCREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v_mp = ((struct __pyx_obj_4cupy_4cuda_6memory_SingleDeviceMemoryPool *)__pyx_t_2);
  __pyx_t_2 = 0;

  /* "cupy/cuda/memory.pyx":1685
 *         """
 *         mp = <SingleDeviceMemoryPool>self.device_pool()
 *         return mp.free_bytes()             # <<<<<<<<<<<<<<
 * 
 *     cpdef size_t total_bytes(self):
*/
  __pyx_t_5 = __pyx_f_4cupy_4cuda_6memory_22SingleDeviceMemoryPool_free_bytes(__pyx_v_mp, 0); if (unlikely(PyErr_Occurred())) __PYX_ERR(0, 1685, __pyx_L1_error)
  __pyx_r = __pyx_t_5;
  goto __pyx_L0;

  /* "cupy/cuda/memory.pyx":1678
 *         return mp.used_bytes()
 * 
 *     cpdef size_t free_bytes(self):             # <<<<<<<<<<<<<<
 *         """Gets the total number of bytes acquired but not used by the pool.
 * 
*/

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_AddTraceback("cupy.cuda.memory.MemoryPool.free_bytes", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XDECREF((PyObject *)__pyx_v_mp);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_pw_4cupy_4cuda_6memory_10MemoryPool_13free_bytes(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
PyDoc_STRVAR(__pyx_doc_4cupy_4cuda_6memory_10MemoryPool_12free_bytes, "MemoryPool.free_bytes(self) -> size_t\n\nGets the total number of bytes acquired but not used by the pool.\n\nReturns:\n    int: The total number of bytes acquired but not used by the pool.");
static PyObject *__pyx_pw_4cupy_4cuda_6memory_10MemoryPool_13free_bytes(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
) {
  #if !CYTHON_METH_FASTCALL
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  #endif
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("free_bytes (wrapper)", 0);
  #if !CYTHON_METH_FASTCALL
  #if CYTHON_ASSUME_SAFE_SIZE
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return NULL;
  #endif
  #endif
  __pyx_kwvalues = __Pyx_KwValues_FASTCALL(__pyx_args, __pyx_nargs);
  if (unlikely(__pyx_nargs > 0)) { __Pyx_RaiseArgtupleInvalid("free_bytes", 1, 0, 0, __pyx_nargs); return NULL; }
  const Py_ssize_t __pyx_kwds_len = unlikely(__pyx_kwds) ? __Pyx_NumKwargs_FASTCALL(__pyx_kwds) : 0;
  if (unlikely(__pyx_kwds_len < 0)) return NULL;
  if (unlikely(__pyx_kwds_len > 0)) {__Pyx_RejectKeywords("free_bytes", __pyx_kwds); return NULL;}
  __pyx_r = __pyx_pf_4cupy_4cuda_6memory_10MemoryPool_12free_bytes(((struct __pyx_obj_4cupy_4cuda_6memory_MemoryPool *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4cupy_4cuda_6memory_10MemoryPool_12free_bytes(struct __pyx_obj_4cupy_4cuda_6memory_MemoryPool *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  size_t __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("free_bytes", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __pyx_f_4cupy_4cuda_6memory_10MemoryPool_free_bytes(__pyx_v_self, 1); if (unlikely(PyErr_Occurred())) __PYX_ERR(0, 1678, __pyx_L1_error)
  __pyx_t_2 = __Pyx_PyLong_FromSize_t(__pyx_t_1); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1678, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_r = __pyx_t_2;
  __pyx_t_2 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_AddTraceback("cupy.cuda.memory.MemoryPool.free_bytes", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "cupy/cuda/memory.pyx":1687
 *         return mp.free_bytes()
 * 
 *     cpdef size_t total_bytes(self):             # <<<<<<<<<<<<<<
 *         """Gets the total number of bytes acquired by the pool.
 * 
*/

static PyObject *__pyx_pw_4cupy_4cuda_6memory_10MemoryPool_15total_bytes(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
static size_t __pyx_f_4cupy_4cuda_6memory_10MemoryPool_total_bytes(struct __pyx_obj_4cupy_4cuda_6memory_MemoryPool *__pyx_v_self, int __pyx_skip_dispatch) {
  struct __pyx_obj_4cupy_4cuda_6memory_SingleDeviceMemoryPool *__pyx_v_mp = NULL;
  size_t __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  size_t __pyx_t_5;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("total_bytes", 0);
  /* Check if called by wrapper */
  if (unlikely(__pyx_skip_dispatch)) ;
  /* Check if overridden in Python */
  else if (
  #if !CYTHON_USE_TYPE_SLOTS
  unlikely(Py_TYPE(((PyObject *)__pyx_v_self)) != __pyx_mstate_global->__pyx_ptype_4cupy_4cuda_6memory_MemoryPool &&
  __Pyx_PyType_HasFeature(Py_TYPE(((PyObject *)__pyx_v_self)), Py_TPFLAGS_HAVE_GC))
  #else
  unlikely(Py_TYPE(((PyObject *)__pyx_v_self))->tp_dictoffset != 0 || __Pyx_PyType_HasFeature(Py_TYPE(((PyObject *)__pyx_v_self)), (Py_TPFLAGS_IS_ABSTRACT | Py_TPFLAGS_HEAPTYPE)))
  #endif
  ) {
    #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
    static PY_UINT64_T __pyx_tp_dict_version = __PYX_DICT_VERSION_INIT, __pyx_obj_dict_version = __PYX_DICT_VERSION_INIT;
    if (unlikely(!__Pyx_object_dict_version_matches(((PyObject *)__pyx_v_self), __pyx_tp_dict_version, __pyx_obj_dict_version))) {
      PY_UINT64_T __pyx_typedict_guard = __Pyx_get_tp_dict_version(((PyObject *)__pyx_v_self));
      #endif
      __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_mstate_global->__pyx_n_u_total_bytes); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1687, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      if (!__Pyx_IsSameCFunction(__pyx_t_1, (void(*)(void)) __pyx_pw_4cupy_4cuda_6memory_10MemoryPool_15total_bytes)) {
        __pyx_t_3 = NULL;
        __Pyx_INCREF(__pyx_t_1);
        __pyx_t_4 = __pyx_t_1; 
        __pyx_t_5 = 1;
        #if CYTHON_UNPACK_METHODS
        if (unlikely(PyMethod_Check(__pyx_t_4))) {
          __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_4);
          assert(__pyx_t_3);
          PyObject* __pyx__function = PyMethod_GET_FUNCTION(__pyx_t_4);
          __Pyx_INCREF(__pyx_t_3);
          __Pyx_INCREF(__pyx__function);
          __Pyx_DECREF_SET(__pyx_t_4, __pyx__function);
          __pyx_t_5 = 0;
        }
        #endif
        {
          PyObject *__pyx_callargs[2] = {__pyx_t_3, NULL};
          __pyx_t_2 = __Pyx_PyObject_FastCall((PyObject*)__pyx_t_4, __pyx_callargs+__pyx_t_5, (1-__pyx_t_5) | (__pyx_t_5*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
          __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
          __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
          if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1687, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_2);
        }
        __pyx_t_5 = __Pyx_PyLong_As_size_t(__pyx_t_2); if (unlikely((__pyx_t_5 == (size_t)-1) && PyErr_Occurred())) __PYX_ERR(0, 1687, __pyx_L1_error)
        __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
        __pyx_r = __pyx_t_5;
        __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
        goto __pyx_L0;
      }
      #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
      __pyx_tp_dict_version = __Pyx_get_tp_dict_version(((PyObject *)__pyx_v_self));
      __pyx_obj_dict_version = __Pyx_get_object_dict_version(((PyObject *)__pyx_v_self));
      if (unlikely(__pyx_typedict_guard != __pyx_tp_dict_version)) {
        __pyx_tp_dict_version = __pyx_obj_dict_version = __PYX_DICT_VERSION_INIT;
      }
      #endif
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
    }
    #endif
  }

  /* "cupy/cuda/memory.pyx":1693
 *             int: The total number of bytes acquired by the pool.
 *         """
 *         mp = <SingleDeviceMemoryPool>self.device_pool()             # <<<<<<<<<<<<<<
 *         return mp.total_bytes()
 * 
*/
  __pyx_t_1 = __pyx_f_4cupy_4cuda_6memory_10MemoryPool_device_pool(__pyx_v_self); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1693, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __pyx_t_1;
  __Pyx_INCREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v_mp = ((struct __pyx_obj_4cupy_4cuda_6memory_SingleDeviceMemoryPool *)__pyx_t_2);
  __pyx_t_2 = 0;

  /* "cupy/cuda/memory.pyx":1694
 *         """
 *         mp = <SingleDeviceMemoryPool>self.device_pool()
 *         return mp.total_bytes()             # <<<<<<<<<<<<<<
 * 
 *     cpdef set_limit(self, size=None, fraction=None):
*/
  __pyx_t_5 = __pyx_f_4cupy_4cuda_6memory_22SingleDeviceMemoryPool_total_bytes(__pyx_v_mp, 0); if (unlikely(PyErr_Occurred())) __PYX_ERR(0, 1694, __pyx_L1_error)
  __pyx_r = __pyx_t_5;
  goto __pyx_L0;

  /* "cupy/cuda/memory.pyx":1687
 *         return mp.free_bytes()
 * 
 *     cpdef size_t total_bytes(self):             # <<<<<<<<<<<<<<
 *         """Gets the total number of bytes acquired by the pool.
 * 
*/

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_AddTraceback("cupy.cuda.memory.MemoryPool.total_bytes", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XDECREF((PyObject *)__pyx_v_mp);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_pw_4cupy_4cuda_6memory_10MemoryPool_15total_bytes(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
PyDoc_STRVAR(__pyx_doc_4cupy_4cuda_6memory_10MemoryPool_14total_bytes, "MemoryPool.total_bytes(self) -> size_t\n\nGets the total number of bytes acquired by the pool.\n\nReturns:\n    int: The total number of bytes acquired by the pool.");
static PyObject *__pyx_pw_4cupy_4cuda_6memory_10MemoryPool_15total_bytes(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
) {
  #if !CYTHON_METH_FASTCALL
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  #endif
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("total_bytes (wrapper)", 0);
  #if !CYTHON_METH_FASTCALL
  #if CYTHON_ASSUME_SAFE_SIZE
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return NULL;
  #endif
  #endif
  __pyx_kwvalues = __Pyx_KwValues_FASTCALL(__pyx_args, __pyx_nargs);
  if (unlikely(__pyx_nargs > 0)) { __Pyx_RaiseArgtupleInvalid("total_bytes", 1, 0, 0, __pyx_nargs); return NULL; }
  const Py_ssize_t __pyx_kwds_len = unlikely(__pyx_kwds) ? __Pyx_NumKwargs_FASTCALL(__pyx_kwds) : 0;
  if (unlikely(__pyx_kwds_len < 0)) return NULL;
  if (unlikely(__pyx_kwds_len > 0)) {__Pyx_RejectKeywords("total_bytes", __pyx_kwds); return NULL;}
  __pyx_r = __pyx_pf_4cupy_4cuda_6memory_10MemoryPool_14total_bytes(((struct __pyx_obj_4cupy_4cuda_6memory_MemoryPool *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4cupy_4cuda_6memory_10MemoryPool_14total_bytes(struct __pyx_obj_4cupy_4cuda_6memory_MemoryPool *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  size_t __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("total_bytes", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __pyx_f_4cupy_4cuda_6memory_10MemoryPool_total_bytes(__pyx_v_self, 1); if (unlikely(PyErr_Occurred())) __PYX_ERR(0, 1687, __pyx_L1_error)
  __pyx_t_2 = __Pyx_PyLong_FromSize_t(__pyx_t_1); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1687, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_r = __pyx_t_2;
  __pyx_t_2 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_AddTraceback("cupy.cuda.memory.MemoryPool.total_bytes", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "cupy/cuda/memory.pyx":1696
 *         return mp.total_bytes()
 * 
 *     cpdef set_limit(self, size=None, fraction=None):             # <<<<<<<<<<<<<<
 *         """Sets the upper limit of memory allocation of the current device.
 * 
*/

static PyObject *__pyx_pw_4cupy_4cuda_6memory_10MemoryPool_17set_limit(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
static PyObject *__pyx_f_4cupy_4cuda_6memory_10MemoryPool_set_limit(struct __pyx_obj_4cupy_4cuda_6memory_MemoryPool *__pyx_v_self, int __pyx_skip_dispatch, struct __pyx_opt_args_4cupy_4cuda_6memory_10MemoryPool_set_limit *__pyx_optional_args) {
  PyObject *__pyx_v_size = ((PyObject *)Py_None);
  PyObject *__pyx_v_fraction = ((PyObject *)Py_None);
  struct __pyx_obj_4cupy_4cuda_6memory_SingleDeviceMemoryPool *__pyx_v_mp = NULL;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  size_t __pyx_t_5;
  struct __pyx_opt_args_4cupy_4cuda_6memory_22SingleDeviceMemoryPool_set_limit __pyx_t_6;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("set_limit", 0);
  if (__pyx_optional_args) {
    if (__pyx_optional_args->__pyx_n > 0) {
      __pyx_v_size = __pyx_optional_args->size;
      if (__pyx_optional_args->__pyx_n > 1) {
        __pyx_v_fraction = __pyx_optional_args->fraction;
      }
    }
  }
  /* Check if called by wrapper */
  if (unlikely(__pyx_skip_dispatch)) ;
  /* Check if overridden in Python */
  else if (
  #if !CYTHON_USE_TYPE_SLOTS
  unlikely(Py_TYPE(((PyObject *)__pyx_v_self)) != __pyx_mstate_global->__pyx_ptype_4cupy_4cuda_6memory_MemoryPool &&
  __Pyx_PyType_HasFeature(Py_TYPE(((PyObject *)__pyx_v_self)), Py_TPFLAGS_HAVE_GC))
  #else
  unlikely(Py_TYPE(((PyObject *)__pyx_v_self))->tp_dictoffset != 0 || __Pyx_PyType_HasFeature(Py_TYPE(((PyObject *)__pyx_v_self)), (Py_TPFLAGS_IS_ABSTRACT | Py_TPFLAGS_HEAPTYPE)))
  #endif
  ) {
    #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
    static PY_UINT64_T __pyx_tp_dict_version = __PYX_DICT_VERSION_INIT, __pyx_obj_dict_version = __PYX_DICT_VERSION_INIT;
    if (unlikely(!__Pyx_object_dict_version_matches(((PyObject *)__pyx_v_self), __pyx_tp_dict_version, __pyx_obj_dict_version))) {
      PY_UINT64_T __pyx_typedict_guard = __Pyx_get_tp_dict_version(((PyObject *)__pyx_v_self));
      #endif
      __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_mstate_global->__pyx_n_u_set_limit); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1696, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      if (!__Pyx_IsSameCFunction(__pyx_t_1, (void(*)(void)) __pyx_pw_4cupy_4cuda_6memory_10MemoryPool_17set_limit)) {
        __Pyx_XDECREF(__pyx_r);
        __pyx_t_3 = NULL;
        __Pyx_INCREF(__pyx_t_1);
        __pyx_t_4 = __pyx_t_1; 
        __pyx_t_5 = 1;
        #if CYTHON_UNPACK_METHODS
        if (unlikely(PyMethod_Check(__pyx_t_4))) {
          __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_4);
          assert(__pyx_t_3);
          PyObject* __pyx__function = PyMethod_GET_FUNCTION(__pyx_t_4);
          __Pyx_INCREF(__pyx_t_3);
          __Pyx_INCREF(__pyx__function);
          __Pyx_DECREF_SET(__pyx_t_4, __pyx__function);
          __pyx_t_5 = 0;
        }
        #endif
        {
          PyObject *__pyx_callargs[3] = {__pyx_t_3, __pyx_v_size, __pyx_v_fraction};
          __pyx_t_2 = __Pyx_PyObject_FastCall((PyObject*)__pyx_t_4, __pyx_callargs+__pyx_t_5, (3-__pyx_t_5) | (__pyx_t_5*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
          __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
          __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
          if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1696, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_2);
        }
        __pyx_r = __pyx_t_2;
        __pyx_t_2 = 0;
        __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
        goto __pyx_L0;
      }
      #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
      __pyx_tp_dict_version = __Pyx_get_tp_dict_version(((PyObject *)__pyx_v_self));
      __pyx_obj_dict_version = __Pyx_get_object_dict_version(((PyObject *)__pyx_v_self));
      if (unlikely(__pyx_typedict_guard != __pyx_tp_dict_version)) {
        __pyx_tp_dict_version = __pyx_obj_dict_version = __PYX_DICT_VERSION_INIT;
      }
      #endif
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
    }
    #endif
  }

  /* "cupy/cuda/memory.pyx":1727
 *             fraction (float): Fraction in the range of ``[0, 1]``.
 *         """
 *         mp = <SingleDeviceMemoryPool>self.device_pool()             # <<<<<<<<<<<<<<
 *         mp.set_limit(size, fraction)
 * 
*/
  __pyx_t_1 = __pyx_f_4cupy_4cuda_6memory_10MemoryPool_device_pool(__pyx_v_self); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1727, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __pyx_t_1;
  __Pyx_INCREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v_mp = ((struct __pyx_obj_4cupy_4cuda_6memory_SingleDeviceMemoryPool *)__pyx_t_2);
  __pyx_t_2 = 0;

  /* "cupy/cuda/memory.pyx":1728
 *         """
 *         mp = <SingleDeviceMemoryPool>self.device_pool()
 *         mp.set_limit(size, fraction)             # <<<<<<<<<<<<<<
 * 
 *     cpdef size_t get_limit(self):
*/
  __pyx_t_6.__pyx_n = 2;
  __pyx_t_6.size = __pyx_v_size;
  __pyx_t_6.fraction = __pyx_v_fraction;
  __pyx_t_2 = __pyx_f_4cupy_4cuda_6memory_22SingleDeviceMemoryPool_set_limit(__pyx_v_mp, 0, &__pyx_t_6); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1728, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "cupy/cuda/memory.pyx":1696
 *         return mp.total_bytes()
 * 
 *     cpdef set_limit(self, size=None, fraction=None):             # <<<<<<<<<<<<<<
 *         """Sets the upper limit of memory allocation of the current device.
 * 
*/

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_AddTraceback("cupy.cuda.memory.MemoryPool.set_limit", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XDECREF((PyObject *)__pyx_v_mp);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_pw_4cupy_4cuda_6memory_10MemoryPool_17set_limit(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
PyDoc_STRVAR(__pyx_doc_4cupy_4cuda_6memory_10MemoryPool_16set_limit, "MemoryPool.set_limit(self, size=None, fraction=None)\n\nSets the upper limit of memory allocation of the current device.\n\nWhen `fraction` is specified, its value will become a fraction of the\namount of GPU memory that is available for allocation.\nFor example, if you have a GPU with 2 GiB memory, you can either use\n``set_limit(fraction=0.5)`` or ``set_limit(size=1024**3)`` to limit\nthe memory size to 1 GiB.\n\n``size`` and ``fraction`` cannot be specified at the same time.\nIf both of them are **not** specified or ``0`` is specified, the\nlimit will be disabled.\n\n.. note::\n    You can also set the limit by using ``CUPY_GPU_MEMORY_LIMIT``\n    environment variable, see :ref:`environment` for the details.\n    The limit set by this method supersedes the value specified in\n    the environment variable.\n\n    Also note that this method only changes the limit for the current\n    device, whereas the environment variable sets the default limit for\n    all devices.\n\n.. note::\n    Changing the limit is not thread-safe. Other threads may use an\n    outdated value for example if currently performing allocations.\n\nArgs:\n    size (int): Limit size in bytes.\n    fraction (float): Fraction in the range of ``[0, 1]``.");
static PyObject *__pyx_pw_4cupy_4cuda_6memory_10MemoryPool_17set_limit(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
) {
  PyObject *__pyx_v_size = 0;
  PyObject *__pyx_v_fraction = 0;
  #if !CYTHON_METH_FASTCALL
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  #endif
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject* values[2] = {0,0};
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("set_limit (wrapper)", 0);
  #if !CYTHON_METH_FASTCALL
  #if CYTHON_ASSUME_SAFE_SIZE
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return NULL;
  #endif
  #endif
  __pyx_kwvalues = __Pyx_KwValues_FASTCALL(__pyx_args, __pyx_nargs);
  {
    PyObject ** const __pyx_pyargnames[] = {&__pyx_mstate_global->__pyx_n_u_size,&__pyx_mstate_global->__pyx_n_u_fraction,0};
    const Py_ssize_t __pyx_kwds_len = (__pyx_kwds) ? __Pyx_NumKwargs_FASTCALL(__pyx_kwds) : 0;
    if (unlikely(__pyx_kwds_len < 0)) __PYX_ERR(0, 1696, __pyx_L3_error)
    if (__pyx_kwds_len > 0) {
      switch (__pyx_nargs) {
        case  2:
        values[1] = __Pyx_ArgRef_FASTCALL(__pyx_args, 1);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[1])) __PYX_ERR(0, 1696, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  1:
        values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(0, 1696, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      const Py_ssize_t kwd_pos_args = __pyx_nargs;
      if (__Pyx_ParseKeywords(__pyx_kwds, __pyx_kwvalues, __pyx_pyargnames, 0, values, kwd_pos_args, __pyx_kwds_len, "set_limit", 0) < (0)) __PYX_ERR(0, 1696, __pyx_L3_error)
      if (!values[0]) values[0] = __Pyx_NewRef(((PyObject *)Py_None));
      if (!values[1]) values[1] = __Pyx_NewRef(((PyObject *)Py_None));
    } else {
      switch (__pyx_nargs) {
        case  2:
        values[1] = __Pyx_ArgRef_FASTCALL(__pyx_args, 1);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[1])) __PYX_ERR(0, 1696, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  1:
        values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(0, 1696, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      if (!values[0]) values[0] = __Pyx_NewRef(((PyObject *)Py_None));
      if (!values[1]) values[1] = __Pyx_NewRef(((PyObject *)Py_None));
    }
    __pyx_v_size = values[0];
    __pyx_v_fraction = values[1];
  }
  goto __pyx_L6_skip;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("set_limit", 0, 0, 2, __pyx_nargs); __PYX_ERR(0, 1696, __pyx_L3_error)
  __pyx_L6_skip:;
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_AddTraceback("cupy.cuda.memory.MemoryPool.set_limit", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_4cupy_4cuda_6memory_10MemoryPool_16set_limit(((struct __pyx_obj_4cupy_4cuda_6memory_MemoryPool *)__pyx_v_self), __pyx_v_size, __pyx_v_fraction);

  /* function exit code */
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4cupy_4cuda_6memory_10MemoryPool_16set_limit(struct __pyx_obj_4cupy_4cuda_6memory_MemoryPool *__pyx_v_self, PyObject *__pyx_v_size, PyObject *__pyx_v_fraction) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  struct __pyx_opt_args_4cupy_4cuda_6memory_10MemoryPool_set_limit __pyx_t_2;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("set_limit", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_2.__pyx_n = 2;
  __pyx_t_2.size = __pyx_v_size;
  __pyx_t_2.fraction = __pyx_v_fraction;
  __pyx_t_1 = __pyx_vtabptr_4cupy_4cuda_6memory_MemoryPool->set_limit(__pyx_v_self, 1, &__pyx_t_2); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1696, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("cupy.cuda.memory.MemoryPool.set_limit", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "cupy/cuda/memory.pyx":1730
 *         mp.set_limit(size, fraction)
 * 
 *     cpdef size_t get_limit(self):             # <<<<<<<<<<<<<<
 *         """Gets the upper limit of memory allocation of the current device.
 * 
*/

static PyObject *__pyx_pw_4cupy_4cuda_6memory_10MemoryPool_19get_limit(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
static size_t __pyx_f_4cupy_4cuda_6memory_10MemoryPool_get_limit(struct __pyx_obj_4cupy_4cuda_6memory_MemoryPool *__pyx_v_self, int __pyx_skip_dispatch) {
  struct __pyx_obj_4cupy_4cuda_6memory_SingleDeviceMemoryPool *__pyx_v_mp = NULL;
  size_t __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  size_t __pyx_t_5;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("get_limit", 0);
  /* Check if called by wrapper */
  if (unlikely(__pyx_skip_dispatch)) ;
  /* Check if overridden in Python */
  else if (
  #if !CYTHON_USE_TYPE_SLOTS
  unlikely(Py_TYPE(((PyObject *)__pyx_v_self)) != __pyx_mstate_global->__pyx_ptype_4cupy_4cuda_6memory_MemoryPool &&
  __Pyx_PyType_HasFeature(Py_TYPE(((PyObject *)__pyx_v_self)), Py_TPFLAGS_HAVE_GC))
  #else
  unlikely(Py_TYPE(((PyObject *)__pyx_v_self))->tp_dictoffset != 0 || __Pyx_PyType_HasFeature(Py_TYPE(((PyObject *)__pyx_v_self)), (Py_TPFLAGS_IS_ABSTRACT | Py_TPFLAGS_HEAPTYPE)))
  #endif
  ) {
    #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
    static PY_UINT64_T __pyx_tp_dict_version = __PYX_DICT_VERSION_INIT, __pyx_obj_dict_version = __PYX_DICT_VERSION_INIT;
    if (unlikely(!__Pyx_object_dict_version_matches(((PyObject *)__pyx_v_self), __pyx_tp_dict_version, __pyx_obj_dict_version))) {
      PY_UINT64_T __pyx_typedict_guard = __Pyx_get_tp_dict_version(((PyObject *)__pyx_v_self));
      #endif
      __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_mstate_global->__pyx_n_u_get_limit); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1730, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      if (!__Pyx_IsSameCFunction(__pyx_t_1, (void(*)(void)) __pyx_pw_4cupy_4cuda_6memory_10MemoryPool_19get_limit)) {
        __pyx_t_3 = NULL;
        __Pyx_INCREF(__pyx_t_1);
        __pyx_t_4 = __pyx_t_1; 
        __pyx_t_5 = 1;
        #if CYTHON_UNPACK_METHODS
        if (unlikely(PyMethod_Check(__pyx_t_4))) {
          __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_4);
          assert(__pyx_t_3);
          PyObject* __pyx__function = PyMethod_GET_FUNCTION(__pyx_t_4);
          __Pyx_INCREF(__pyx_t_3);
          __Pyx_INCREF(__pyx__function);
          __Pyx_DECREF_SET(__pyx_t_4, __pyx__function);
          __pyx_t_5 = 0;
        }
        #endif
        {
          PyObject *__pyx_callargs[2] = {__pyx_t_3, NULL};
          __pyx_t_2 = __Pyx_PyObject_FastCall((PyObject*)__pyx_t_4, __pyx_callargs+__pyx_t_5, (1-__pyx_t_5) | (__pyx_t_5*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
          __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
          __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
          if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1730, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_2);
        }
        __pyx_t_5 = __Pyx_PyLong_As_size_t(__pyx_t_2); if (unlikely((__pyx_t_5 == (size_t)-1) && PyErr_Occurred())) __PYX_ERR(0, 1730, __pyx_L1_error)
        __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
        __pyx_r = __pyx_t_5;
        __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
        goto __pyx_L0;
      }
      #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
      __pyx_tp_dict_version = __Pyx_get_tp_dict_version(((PyObject *)__pyx_v_self));
      __pyx_obj_dict_version = __Pyx_get_object_dict_version(((PyObject *)__pyx_v_self));
      if (unlikely(__pyx_typedict_guard != __pyx_tp_dict_version)) {
        __pyx_tp_dict_version = __pyx_obj_dict_version = __PYX_DICT_VERSION_INIT;
      }
      #endif
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
    }
    #endif
  }

  /* "cupy/cuda/memory.pyx":1736
 *             int: The number of bytes
 *         """
 *         mp = <SingleDeviceMemoryPool>self.device_pool()             # <<<<<<<<<<<<<<
 *         return mp.get_limit()
 * 
*/
  __pyx_t_1 = __pyx_f_4cupy_4cuda_6memory_10MemoryPool_device_pool(__pyx_v_self); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1736, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __pyx_t_1;
  __Pyx_INCREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v_mp = ((struct __pyx_obj_4cupy_4cuda_6memory_SingleDeviceMemoryPool *)__pyx_t_2);
  __pyx_t_2 = 0;

  /* "cupy/cuda/memory.pyx":1737
 *         """
 *         mp = <SingleDeviceMemoryPool>self.device_pool()
 *         return mp.get_limit()             # <<<<<<<<<<<<<<
 * 
 * 
*/
  __pyx_t_5 = __pyx_f_4cupy_4cuda_6memory_22SingleDeviceMemoryPool_get_limit(__pyx_v_mp, 0); if (unlikely(PyErr_Occurred())) __PYX_ERR(0, 1737, __pyx_L1_error)
  __pyx_r = __pyx_t_5;
  goto __pyx_L0;

  /* "cupy/cuda/memory.pyx":1730
 *         mp.set_limit(size, fraction)
 * 
 *     cpdef size_t get_limit(self):             # <<<<<<<<<<<<<<
 *         """Gets the upper limit of memory allocation of the current device.
 * 
*/

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_AddTraceback("cupy.cuda.memory.MemoryPool.get_limit", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XDECREF((PyObject *)__pyx_v_mp);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_pw_4cupy_4cuda_6memory_10MemoryPool_19get_limit(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
PyDoc_STRVAR(__pyx_doc_4cupy_4cuda_6memory_10MemoryPool_18get_limit, "MemoryPool.get_limit(self) -> size_t\n\nGets the upper limit of memory allocation of the current device.\n\nReturns:\n    int: The number of bytes");
static PyObject *__pyx_pw_4cupy_4cuda_6memory_10MemoryPool_19get_limit(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
) {
  #if !CYTHON_METH_FASTCALL
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  #endif
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("get_limit (wrapper)", 0);
  #if !CYTHON_METH_FASTCALL
  #if CYTHON_ASSUME_SAFE_SIZE
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return NULL;
  #endif
  #endif
  __pyx_kwvalues = __Pyx_KwValues_FASTCALL(__pyx_args, __pyx_nargs);
  if (unlikely(__pyx_nargs > 0)) { __Pyx_RaiseArgtupleInvalid("get_limit", 1, 0, 0, __pyx_nargs); return NULL; }
  const Py_ssize_t __pyx_kwds_len = unlikely(__pyx_kwds) ? __Pyx_NumKwargs_FASTCALL(__pyx_kwds) : 0;
  if (unlikely(__pyx_kwds_len < 0)) return NULL;
  if (unlikely(__pyx_kwds_len > 0)) {__Pyx_RejectKeywords("get_limit", __pyx_kwds); return NULL;}
  __pyx_r = __pyx_pf_4cupy_4cuda_6memory_10MemoryPool_18get_limit(((struct __pyx_obj_4cupy_4cuda_6memory_MemoryPool *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4cupy_4cuda_6memory_10MemoryPool_18get_limit(struct __pyx_obj_4cupy_4cuda_6memory_MemoryPool *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  size_t __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("get_limit", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __pyx_f_4cupy_4cuda_6memory_10MemoryPool_get_limit(__pyx_v_self, 1); if (unlikely(PyErr_Occurred())) __PYX_ERR(0, 1730, __pyx_L1_error)
  __pyx_t_2 = __Pyx_PyLong_FromSize_t(__pyx_t_1); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1730, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_r = __pyx_t_2;
  __pyx_t_2 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_AddTraceback("cupy.cuda.memory.MemoryPool.get_limit", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     cdef tuple state
 *     cdef object _dict
*/

/* Python wrapper */
static PyObject *__pyx_pw_4cupy_4cuda_6memory_10MemoryPool_21__reduce_cython__(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
PyDoc_STRVAR(__pyx_doc_4cupy_4cuda_6memory_10MemoryPool_20__reduce_cython__, "MemoryPool.__reduce_cython__(self)");
static PyObject *__pyx_pw_4cupy_4cuda_6memory_10MemoryPool_21__reduce_cython__(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
) {
  #if !CYTHON_METH_FASTCALL
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  #endif
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__reduce_cython__ (wrapper)", 0);
  #if !CYTHON_METH_FASTCALL
  #if CYTHON_ASSUME_SAFE_SIZE
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return NULL;
  #endif
  #endif
  __pyx_kwvalues = __Pyx_KwValues_FASTCALL(__pyx_args, __pyx_nargs);
  if (unlikely(__pyx_nargs > 0)) { __Pyx_RaiseArgtupleInvalid("__reduce_cython__", 1, 0, 0, __pyx_nargs); return NULL; }
  const Py_ssize_t __pyx_kwds_len = unlikely(__pyx_kwds) ? __Pyx_NumKwargs_FASTCALL(__pyx_kwds) : 0;
  if (unlikely(__pyx_kwds_len < 0)) return NULL;
  if (unlikely(__pyx_kwds_len > 0)) {__Pyx_RejectKeywords("__reduce_cython__", __pyx_kwds); return NULL;}
  __pyx_r = __pyx_pf_4cupy_4cuda_6memory_10MemoryPool_20__reduce_cython__(((struct __pyx_obj_4cupy_4cuda_6memory_MemoryPool *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4cupy_4cuda_6memory_10MemoryPool_20__reduce_cython__(struct __pyx_obj_4cupy_4cuda_6memory_MemoryPool *__pyx_v_self) {
  PyObject *__pyx_v_state = 0;
  PyObject *__pyx_v__dict = 0;
  int __pyx_v_use_setstate;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_t_2;
  int __pyx_t_3;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__reduce_cython__", 0);

  /* "(tree fragment)":5
 *     cdef object _dict
 *     cdef bint use_setstate
 *     state = (self._allocator, self._pools)             # <<<<<<<<<<<<<<
 *     _dict = getattr(self, '__dict__', None)
 *     if _dict is not None and _dict:
*/
  __pyx_t_1 = PyTuple_New(2); if (unlikely(!__pyx_t_1)) __PYX_ERR(3, 5, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_INCREF(__pyx_v_self->_allocator);
  __Pyx_GIVEREF(__pyx_v_self->_allocator);
  if (__Pyx_PyTuple_SET_ITEM(__pyx_t_1, 0, __pyx_v_self->_allocator) != (0)) __PYX_ERR(3, 5, __pyx_L1_error);
  __Pyx_INCREF(__pyx_v_self->_pools);
  __Pyx_GIVEREF(__pyx_v_self->_pools);
  if (__Pyx_PyTuple_SET_ITEM(__pyx_t_1, 1, __pyx_v_self->_pools) != (0)) __PYX_ERR(3, 5, __pyx_L1_error);
  __pyx_v_state = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "(tree fragment)":6
 *     cdef bint use_setstate
 *     state = (self._allocator, self._pools)
 *     _dict = getattr(self, '__dict__', None)             # <<<<<<<<<<<<<<
 *     if _dict is not None and _dict:
 *         state += (_dict,)
*/
  __pyx_t_1 = __Pyx_GetAttr3(((PyObject *)__pyx_v_self), __pyx_mstate_global->__pyx_n_u_dict, Py_None); if (unlikely(!__pyx_t_1)) __PYX_ERR(3, 6, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_v__dict = __pyx_t_1;
  __pyx_t_1 = 0;

  /* "(tree fragment)":7
 *     state = (self._allocator, self._pools)
 *     _dict = getattr(self, '__dict__', None)
 *     if _dict is not None and _dict:             # <<<<<<<<<<<<<<
 *         state += (_dict,)
 *         use_setstate = True
*/
  __pyx_t_3 = (__pyx_v__dict != Py_None);
  if (__pyx_t_3) {
  } else {
    __pyx_t_2 = __pyx_t_3;
    goto __pyx_L4_bool_binop_done;
  }
  __pyx_t_3 = __Pyx_PyObject_IsTrue(__pyx_v__dict); if (unlikely((__pyx_t_3 < 0))) __PYX_ERR(3, 7, __pyx_L1_error)
  __pyx_t_2 = __pyx_t_3;
  __pyx_L4_bool_binop_done:;
  if (__pyx_t_2) {

    /* "(tree fragment)":8
 *     _dict = getattr(self, '__dict__', None)
 *     if _dict is not None and _dict:
 *         state += (_dict,)             # <<<<<<<<<<<<<<
 *         use_setstate = True
 *     else:
*/
    __pyx_t_1 = PyTuple_New(1); if (unlikely(!__pyx_t_1)) __PYX_ERR(3, 8, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_INCREF(__pyx_v__dict);
    __Pyx_GIVEREF(__pyx_v__dict);
    if (__Pyx_PyTuple_SET_ITEM(__pyx_t_1, 0, __pyx_v__dict) != (0)) __PYX_ERR(3, 8, __pyx_L1_error);
    __pyx_t_4 = PyNumber_InPlaceAdd(__pyx_v_state, __pyx_t_1); if (unlikely(!__pyx_t_4)) __PYX_ERR(3, 8, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __Pyx_DECREF_SET(__pyx_v_state, ((PyObject*)__pyx_t_4));
    __pyx_t_4 = 0;

    /* "(tree fragment)":9
 *     if _dict is not None and _dict:
 *         state += (_dict,)
 *         use_setstate = True             # <<<<<<<<<<<<<<
 *     else:
 *         use_setstate = self._allocator is not None or self._pools is not None
*/
    __pyx_v_use_setstate = 1;

    /* "(tree fragment)":7
 *     state = (self._allocator, self._pools)
 *     _dict = getattr(self, '__dict__', None)
 *     if _dict is not None and _dict:             # <<<<<<<<<<<<<<
 *         state += (_dict,)
 *         use_setstate = True
*/
    goto __pyx_L3;
  }

  /* "(tree fragment)":11
 *         use_setstate = True
 *     else:
 *         use_setstate = self._allocator is not None or self._pools is not None             # <<<<<<<<<<<<<<
 *     if use_setstate:
 *         return __pyx_unpickle_MemoryPool, (type(self), 0x7925074, None), state
*/
  /*else*/ {
    __pyx_t_3 = (__pyx_v_self->_allocator != Py_None);
    if (!__pyx_t_3) {
    } else {
      __pyx_t_2 = __pyx_t_3;
      goto __pyx_L6_bool_binop_done;
    }
    __pyx_t_3 = (__pyx_v_self->_pools != ((PyObject*)Py_None));
    __pyx_t_2 = __pyx_t_3;
    __pyx_L6_bool_binop_done:;
    __pyx_v_use_setstate = __pyx_t_2;
  }
  __pyx_L3:;

  /* "(tree fragment)":12
 *     else:
 *         use_setstate = self._allocator is not None or self._pools is not None
 *     if use_setstate:             # <<<<<<<<<<<<<<
 *         return __pyx_unpickle_MemoryPool, (type(self), 0x7925074, None), state
 *     else:
*/
  if (__pyx_v_use_setstate) {

    /* "(tree fragment)":13
 *         use_setstate = self._allocator is not None or self._pools is not None
 *     if use_setstate:
 *         return __pyx_unpickle_MemoryPool, (type(self), 0x7925074, None), state             # <<<<<<<<<<<<<<
 *     else:
 *         return __pyx_unpickle_MemoryPool, (type(self), 0x7925074, state)
*/
    __Pyx_XDECREF(__pyx_r);
    __Pyx_GetModuleGlobalName(__pyx_t_4, __pyx_mstate_global->__pyx_n_u_pyx_unpickle_MemoryPool); if (unlikely(!__pyx_t_4)) __PYX_ERR(3, 13, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __pyx_t_1 = PyTuple_New(3); if (unlikely(!__pyx_t_1)) __PYX_ERR(3, 13, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_INCREF(((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    __Pyx_GIVEREF(((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    if (__Pyx_PyTuple_SET_ITEM(__pyx_t_1, 0, ((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self)))) != (0)) __PYX_ERR(3, 13, __pyx_L1_error);
    __Pyx_INCREF(__pyx_mstate_global->__pyx_int_127029364);
    __Pyx_GIVEREF(__pyx_mstate_global->__pyx_int_127029364);
    if (__Pyx_PyTuple_SET_ITEM(__pyx_t_1, 1, __pyx_mstate_global->__pyx_int_127029364) != (0)) __PYX_ERR(3, 13, __pyx_L1_error);
    __Pyx_INCREF(Py_None);
    __Pyx_GIVEREF(Py_None);
    if (__Pyx_PyTuple_SET_ITEM(__pyx_t_1, 2, Py_None) != (0)) __PYX_ERR(3, 13, __pyx_L1_error);
    __pyx_t_5 = PyTuple_New(3); if (unlikely(!__pyx_t_5)) __PYX_ERR(3, 13, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __Pyx_GIVEREF(__pyx_t_4);
    if (__Pyx_PyTuple_SET_ITEM(__pyx_t_5, 0, __pyx_t_4) != (0)) __PYX_ERR(3, 13, __pyx_L1_error);
    __Pyx_GIVEREF(__pyx_t_1);
    if (__Pyx_PyTuple_SET_ITEM(__pyx_t_5, 1, __pyx_t_1) != (0)) __PYX_ERR(3, 13, __pyx_L1_error);
    __Pyx_INCREF(__pyx_v_state);
    __Pyx_GIVEREF(__pyx_v_state);
    if (__Pyx_PyTuple_SET_ITEM(__pyx_t_5, 2, __pyx_v_state) != (0)) __PYX_ERR(3, 13, __pyx_L1_error);
    __pyx_t_4 = 0;
    __pyx_t_1 = 0;
    __pyx_r = __pyx_t_5;
    __pyx_t_5 = 0;
    goto __pyx_L0;

    /* "(tree fragment)":12
 *     else:
 *         use_setstate = self._allocator is not None or self._pools is not None
 *     if use_setstate:             # <<<<<<<<<<<<<<
 *         return __pyx_unpickle_MemoryPool, (type(self), 0x7925074, None), state
 *     else:
*/
  }

  /* "(tree fragment)":15
 *         return __pyx_unpickle_MemoryPool, (type(self), 0x7925074, None), state
 *     else:
 *         return __pyx_unpickle_MemoryPool, (type(self), 0x7925074, state)             # <<<<<<<<<<<<<<
 * def __setstate_cython__(self, __pyx_state):
 *     __pyx_unpickle_MemoryPool__set_state(self, __pyx_state)
*/
  /*else*/ {
    __Pyx_XDECREF(__pyx_r);
    __Pyx_GetModuleGlobalName(__pyx_t_5, __pyx_mstate_global->__pyx_n_u_pyx_unpickle_MemoryPool); if (unlikely(!__pyx_t_5)) __PYX_ERR(3, 15, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __pyx_t_1 = PyTuple_New(3); if (unlikely(!__pyx_t_1)) __PYX_ERR(3, 15, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_INCREF(((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    __Pyx_GIVEREF(((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    if (__Pyx_PyTuple_SET_ITEM(__pyx_t_1, 0, ((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self)))) != (0)) __PYX_ERR(3, 15, __pyx_L1_error);
    __Pyx_INCREF(__pyx_mstate_global->__pyx_int_127029364);
    __Pyx_GIVEREF(__pyx_mstate_global->__pyx_int_127029364);
    if (__Pyx_PyTuple_SET_ITEM(__pyx_t_1, 1, __pyx_mstate_global->__pyx_int_127029364) != (0)) __PYX_ERR(3, 15, __pyx_L1_error);
    __Pyx_INCREF(__pyx_v_state);
    __Pyx_GIVEREF(__pyx_v_state);
    if (__Pyx_PyTuple_SET_ITEM(__pyx_t_1, 2, __pyx_v_state) != (0)) __PYX_ERR(3, 15, __pyx_L1_error);
    __pyx_t_4 = PyTuple_New(2); if (unlikely(!__pyx_t_4)) __PYX_ERR(3, 15, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_GIVEREF(__pyx_t_5);
    if (__Pyx_PyTuple_SET_ITEM(__pyx_t_4, 0, __pyx_t_5) != (0)) __PYX_ERR(3, 15, __pyx_L1_error);
    __Pyx_GIVEREF(__pyx_t_1);
    if (__Pyx_PyTuple_SET_ITEM(__pyx_t_4, 1, __pyx_t_1) != (0)) __PYX_ERR(3, 15, __pyx_L1_error);
    __pyx_t_5 = 0;
    __pyx_t_1 = 0;
    __pyx_r = __pyx_t_4;
    __pyx_t_4 = 0;
    goto __pyx_L0;
  }

  /* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     cdef tuple state
 *     cdef object _dict
*/

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_AddTraceback("cupy.cuda.memory.MemoryPool.__reduce_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_state);
  __Pyx_XDECREF(__pyx_v__dict);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":16
 *     else:
 *         return __pyx_unpickle_MemoryPool, (type(self), 0x7925074, state)
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     __pyx_unpickle_MemoryPool__set_state(self, __pyx_state)
*/

/* Python wrapper */
static PyObject *__pyx_pw_4cupy_4cuda_6memory_10MemoryPool_23__setstate_cython__(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
PyDoc_STRVAR(__pyx_doc_4cupy_4cuda_6memory_10MemoryPool_22__setstate_cython__, "MemoryPool.__setstate_cython__(self, __pyx_state)");
static PyObject *__pyx_pw_4cupy_4cuda_6memory_10MemoryPool_23__setstate_cython__(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
) {
  PyObject *__pyx_v___pyx_state = 0;
  #if !CYTHON_METH_FASTCALL
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  #endif
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject* values[1] = {0};
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__setstate_cython__ (wrapper)", 0);
  #if !CYTHON_METH_FASTCALL
  #if CYTHON_ASSUME_SAFE_SIZE
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return NULL;
  #endif
  #endif
  __pyx_kwvalues = __Pyx_KwValues_FASTCALL(__pyx_args, __pyx_nargs);
  {
    PyObject ** const __pyx_pyargnames[] = {&__pyx_mstate_global->__pyx_n_u_pyx_state,0};
    const Py_ssize_t __pyx_kwds_len = (__pyx_kwds) ? __Pyx_NumKwargs_FASTCALL(__pyx_kwds) : 0;
    if (unlikely(__pyx_kwds_len < 0)) __PYX_ERR(3, 16, __pyx_L3_error)
    if (__pyx_kwds_len > 0) {
      switch (__pyx_nargs) {
        case  1:
        values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(3, 16, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      const Py_ssize_t kwd_pos_args = __pyx_nargs;
      if (__Pyx_ParseKeywords(__pyx_kwds, __pyx_kwvalues, __pyx_pyargnames, 0, values, kwd_pos_args, __pyx_kwds_len, "__setstate_cython__", 0) < (0)) __PYX_ERR(3, 16, __pyx_L3_error)
      for (Py_ssize_t i = __pyx_nargs; i < 1; i++) {
        if (unlikely(!values[i])) { __Pyx_RaiseArgtupleInvalid("__setstate_cython__", 1, 1, 1, i); __PYX_ERR(3, 16, __pyx_L3_error) }
      }
    } else if (unlikely(__pyx_nargs != 1)) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
      if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(3, 16, __pyx_L3_error)
    }
    __pyx_v___pyx_state = values[0];
  }
  goto __pyx_L6_skip;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("__setstate_cython__", 1, 1, 1, __pyx_nargs); __PYX_ERR(3, 16, __pyx_L3_error)
  __pyx_L6_skip:;
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_AddTraceback("cupy.cuda.memory.MemoryPool.__setstate_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_4cupy_4cuda_6memory_10MemoryPool_22__setstate_cython__(((struct __pyx_obj_4cupy_4cuda_6memory_MemoryPool *)__pyx_v_self), __pyx_v___pyx_state);

  /* function exit code */
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4cupy_4cuda_6memory_10MemoryPool_22__setstate_cython__(struct __pyx_obj_4cupy_4cuda_6memory_MemoryPool *__pyx_v_self, PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__setstate_cython__", 0);

  /* "(tree fragment)":17
 *         return __pyx_unpickle_MemoryPool, (type(self), 0x7925074, state)
 * def __setstate_cython__(self, __pyx_state):
 *     __pyx_unpickle_MemoryPool__set_state(self, __pyx_state)             # <<<<<<<<<<<<<<
*/
  __pyx_t_1 = __pyx_v___pyx_state;
  __Pyx_INCREF(__pyx_t_1);
  if (!(likely(PyTuple_CheckExact(__pyx_t_1))||((__pyx_t_1) == Py_None) || __Pyx_RaiseUnexpectedTypeError("tuple", __pyx_t_1))) __PYX_ERR(3, 17, __pyx_L1_error)
  if (unlikely(__pyx_t_1 == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "cannot pass None into a C function argument that is declared 'not None'");
    __PYX_ERR(3, 17, __pyx_L1_error)
  }
  __pyx_t_2 = __pyx_f_4cupy_4cuda_6memory___pyx_unpickle_MemoryPool__set_state(__pyx_v_self, ((PyObject*)__pyx_t_1)); if (unlikely(!__pyx_t_2)) __PYX_ERR(3, 17, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "(tree fragment)":16
 *     else:
 *         return __pyx_unpickle_MemoryPool, (type(self), 0x7925074, state)
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     __pyx_unpickle_MemoryPool__set_state(self, __pyx_state)
*/

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_AddTraceback("cupy.cuda.memory.MemoryPool.__setstate_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "cupy/cuda/memory.pyx":1795
 *         readonly bint memoryAsyncHasStat
 * 
 *     def __init__(self, pool_handles='current'):             # <<<<<<<<<<<<<<
 *         cdef int dev_id, prev_dev_id, dev_counts
 *         cdef dict limit = _parse_limit_string()
*/

/* Python wrapper */
static int __pyx_pw_4cupy_4cuda_6memory_15MemoryAsyncPool_1__init__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static int __pyx_pw_4cupy_4cuda_6memory_15MemoryAsyncPool_1__init__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_pool_handles = 0;
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject* values[1] = {0};
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__init__ (wrapper)", 0);
  #if CYTHON_ASSUME_SAFE_SIZE
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return -1;
  #endif
  __pyx_kwvalues = __Pyx_KwValues_VARARGS(__pyx_args, __pyx_nargs);
  {
    PyObject ** const __pyx_pyargnames[] = {&__pyx_mstate_global->__pyx_n_u_pool_handles,0};
    const Py_ssize_t __pyx_kwds_len = (__pyx_kwds) ? __Pyx_NumKwargs_VARARGS(__pyx_kwds) : 0;
    if (unlikely(__pyx_kwds_len < 0)) __PYX_ERR(0, 1795, __pyx_L3_error)
    if (__pyx_kwds_len > 0) {
      switch (__pyx_nargs) {
        case  1:
        values[0] = __Pyx_ArgRef_VARARGS(__pyx_args, 0);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(0, 1795, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      const Py_ssize_t kwd_pos_args = __pyx_nargs;
      if (__Pyx_ParseKeywords(__pyx_kwds, __pyx_kwvalues, __pyx_pyargnames, 0, values, kwd_pos_args, __pyx_kwds_len, "__init__", 0) < (0)) __PYX_ERR(0, 1795, __pyx_L3_error)
      if (!values[0]) values[0] = __Pyx_NewRef(((PyObject *)__pyx_mstate_global->__pyx_n_u_current));
    } else {
      switch (__pyx_nargs) {
        case  1:
        values[0] = __Pyx_ArgRef_VARARGS(__pyx_args, 0);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(0, 1795, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      if (!values[0]) values[0] = __Pyx_NewRef(((PyObject *)__pyx_mstate_global->__pyx_n_u_current));
    }
    __pyx_v_pool_handles = values[0];
  }
  goto __pyx_L6_skip;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("__init__", 0, 0, 1, __pyx_nargs); __PYX_ERR(0, 1795, __pyx_L3_error)
  __pyx_L6_skip:;
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_AddTraceback("cupy.cuda.memory.MemoryAsyncPool.__init__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return -1;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_4cupy_4cuda_6memory_15MemoryAsyncPool___init__(((struct __pyx_obj_4cupy_4cuda_6memory_MemoryAsyncPool *)__pyx_v_self), __pyx_v_pool_handles);

  /* function exit code */
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_4cupy_4cuda_6memory_15MemoryAsyncPool___init__(struct __pyx_obj_4cupy_4cuda_6memory_MemoryAsyncPool *__pyx_v_self, PyObject *__pyx_v_pool_handles) {
  int __pyx_v_dev_id;
  int __pyx_v_prev_dev_id;
  int __pyx_v_dev_counts;
  PyObject *__pyx_v_limit = 0;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_t_2;
  int __pyx_t_3;
  int __pyx_t_4;
  int __pyx_t_5;
  int __pyx_t_6;
  int __pyx_t_7;
  intptr_t __pyx_t_8;
  int __pyx_t_9;
  PyObject *__pyx_t_10 = NULL;
  PyObject *__pyx_t_11 = NULL;
  int __pyx_t_12;
  int __pyx_t_13;
  char const *__pyx_t_14;
  PyObject *__pyx_t_15 = NULL;
  PyObject *__pyx_t_16 = NULL;
  PyObject *__pyx_t_17 = NULL;
  PyObject *__pyx_t_18 = NULL;
  PyObject *__pyx_t_19 = NULL;
  PyObject *__pyx_t_20 = NULL;
  char const *__pyx_t_21;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__init__", 0);

  /* "cupy/cuda/memory.pyx":1797
 *     def __init__(self, pool_handles='current'):
 *         cdef int dev_id, prev_dev_id, dev_counts
 *         cdef dict limit = _parse_limit_string()             # <<<<<<<<<<<<<<
 *         dev_counts = runtime.getDeviceCount()
 *         self._pools = []
*/
  __pyx_t_1 = __pyx_f_4cupy_4cuda_6memory__parse_limit_string(0, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1797, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_v_limit = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "cupy/cuda/memory.pyx":1798
 *         cdef int dev_id, prev_dev_id, dev_counts
 *         cdef dict limit = _parse_limit_string()
 *         dev_counts = runtime.getDeviceCount()             # <<<<<<<<<<<<<<
 *         self._pools = []
 *         self.memoryAsyncHasStat = (runtime.driverGetVersion() >= 11030)
*/
  __pyx_t_2 = __pyx_f_13cupy_backends_4cuda_3api_7runtime_getDeviceCount(0); if (unlikely(__pyx_t_2 == ((int)-1) && PyErr_Occurred())) __PYX_ERR(0, 1798, __pyx_L1_error)
  __pyx_v_dev_counts = __pyx_t_2;

  /* "cupy/cuda/memory.pyx":1799
 *         cdef dict limit = _parse_limit_string()
 *         dev_counts = runtime.getDeviceCount()
 *         self._pools = []             # <<<<<<<<<<<<<<
 *         self.memoryAsyncHasStat = (runtime.driverGetVersion() >= 11030)
 *         prev_dev_id = runtime.getDevice()
*/
  __pyx_t_1 = PyList_New(0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1799, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v_self->_pools);
  __Pyx_DECREF(__pyx_v_self->_pools);
  __pyx_v_self->_pools = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "cupy/cuda/memory.pyx":1800
 *         dev_counts = runtime.getDeviceCount()
 *         self._pools = []
 *         self.memoryAsyncHasStat = (runtime.driverGetVersion() >= 11030)             # <<<<<<<<<<<<<<
 *         prev_dev_id = runtime.getDevice()
 *         if (cpython.PySequence_Check(pool_handles)
*/
  __pyx_t_2 = __pyx_f_13cupy_backends_4cuda_3api_7runtime_driverGetVersion(0); if (unlikely(__pyx_t_2 == ((int)-1) && PyErr_Occurred())) __PYX_ERR(0, 1800, __pyx_L1_error)
  __pyx_v_self->memoryAsyncHasStat = (__pyx_t_2 >= 0x2B16);

  /* "cupy/cuda/memory.pyx":1801
 *         self._pools = []
 *         self.memoryAsyncHasStat = (runtime.driverGetVersion() >= 11030)
 *         prev_dev_id = runtime.getDevice()             # <<<<<<<<<<<<<<
 *         if (cpython.PySequence_Check(pool_handles)
 *                 and not isinstance(pool_handles, str)):
*/
  __pyx_t_2 = __pyx_f_13cupy_backends_4cuda_3api_7runtime_getDevice(0); if (unlikely(__pyx_t_2 == ((int)-1) && PyErr_Occurred())) __PYX_ERR(0, 1801, __pyx_L1_error)
  __pyx_v_prev_dev_id = __pyx_t_2;

  /* "cupy/cuda/memory.pyx":1802
 *         self.memoryAsyncHasStat = (runtime.driverGetVersion() >= 11030)
 *         prev_dev_id = runtime.getDevice()
 *         if (cpython.PySequence_Check(pool_handles)             # <<<<<<<<<<<<<<
 *                 and not isinstance(pool_handles, str)):
 *             # allow different kinds of handles on each device
*/
  __pyx_t_4 = PySequence_Check(__pyx_v_pool_handles);
  if (__pyx_t_4) {
  } else {
    __pyx_t_3 = __pyx_t_4;
    goto __pyx_L4_bool_binop_done;
  }

  /* "cupy/cuda/memory.pyx":1803
 *         prev_dev_id = runtime.getDevice()
 *         if (cpython.PySequence_Check(pool_handles)
 *                 and not isinstance(pool_handles, str)):             # <<<<<<<<<<<<<<
 *             # allow different kinds of handles on each device
 *             for dev_id in range(dev_counts):
*/
  __pyx_t_4 = PyUnicode_Check(__pyx_v_pool_handles); 
  __pyx_t_5 = (!__pyx_t_4);
  __pyx_t_3 = __pyx_t_5;
  __pyx_L4_bool_binop_done:;

  /* "cupy/cuda/memory.pyx":1802
 *         self.memoryAsyncHasStat = (runtime.driverGetVersion() >= 11030)
 *         prev_dev_id = runtime.getDevice()
 *         if (cpython.PySequence_Check(pool_handles)             # <<<<<<<<<<<<<<
 *                 and not isinstance(pool_handles, str)):
 *             # allow different kinds of handles on each device
*/
  if (__pyx_t_3) {

    /* "cupy/cuda/memory.pyx":1805
 *                 and not isinstance(pool_handles, str)):
 *             # allow different kinds of handles on each device
 *             for dev_id in range(dev_counts):             # <<<<<<<<<<<<<<
 *                 try:
 *                     runtime.setDevice(dev_id)
*/
    __pyx_t_2 = __pyx_v_dev_counts;
    __pyx_t_6 = __pyx_t_2;
    for (__pyx_t_7 = 0; __pyx_t_7 < __pyx_t_6; __pyx_t_7+=1) {
      __pyx_v_dev_id = __pyx_t_7;

      /* "cupy/cuda/memory.pyx":1806
 *             # allow different kinds of handles on each device
 *             for dev_id in range(dev_counts):
 *                 try:             # <<<<<<<<<<<<<<
 *                     runtime.setDevice(dev_id)
 *                     self._pools.append(self.set_pool(
*/
      /*try:*/ {

        /* "cupy/cuda/memory.pyx":1807
 *             for dev_id in range(dev_counts):
 *                 try:
 *                     runtime.setDevice(dev_id)             # <<<<<<<<<<<<<<
 *                     self._pools.append(self.set_pool(
 *                         pool_handles[dev_id], dev_id))
*/
        __pyx_t_1 = __pyx_f_13cupy_backends_4cuda_3api_7runtime_setDevice(__pyx_v_dev_id, 0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1807, __pyx_L11_error)
        __Pyx_GOTREF(__pyx_t_1);
        __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

        /* "cupy/cuda/memory.pyx":1808
 *                 try:
 *                     runtime.setDevice(dev_id)
 *                     self._pools.append(self.set_pool(             # <<<<<<<<<<<<<<
 *                         pool_handles[dev_id], dev_id))
 *                     self.set_limit(**limit)
*/
        if (unlikely(__pyx_v_self->_pools == Py_None)) {
          PyErr_Format(PyExc_AttributeError, "'NoneType' object has no attribute '%.30s'", "append");
          __PYX_ERR(0, 1808, __pyx_L11_error)
        }

        /* "cupy/cuda/memory.pyx":1809
 *                     runtime.setDevice(dev_id)
 *                     self._pools.append(self.set_pool(
 *                         pool_handles[dev_id], dev_id))             # <<<<<<<<<<<<<<
 *                     self.set_limit(**limit)
 *                 finally:
*/
        __pyx_t_1 = __Pyx_GetItemInt(__pyx_v_pool_handles, __pyx_v_dev_id, int, 1, __Pyx_PyLong_From_int, 0, 1, 1, 1, __Pyx_ReferenceSharing_FunctionArgument); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1809, __pyx_L11_error)
        __Pyx_GOTREF(__pyx_t_1);

        /* "cupy/cuda/memory.pyx":1808
 *                 try:
 *                     runtime.setDevice(dev_id)
 *                     self._pools.append(self.set_pool(             # <<<<<<<<<<<<<<
 *                         pool_handles[dev_id], dev_id))
 *                     self.set_limit(**limit)
*/
        __pyx_t_8 = ((struct __pyx_vtabstruct_4cupy_4cuda_6memory_MemoryAsyncPool *)__pyx_v_self->__pyx_vtab)->set_pool(__pyx_v_self, __pyx_t_1, __pyx_v_dev_id); if (unlikely(__pyx_t_8 == ((intptr_t)0) && PyErr_Occurred())) __PYX_ERR(0, 1808, __pyx_L11_error)
        __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
        __pyx_t_1 = PyLong_FromSsize_t(__pyx_t_8); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1808, __pyx_L11_error)
        __Pyx_GOTREF(__pyx_t_1);
        __pyx_t_9 = __Pyx_PyList_Append(__pyx_v_self->_pools, __pyx_t_1); if (unlikely(__pyx_t_9 == ((int)-1))) __PYX_ERR(0, 1808, __pyx_L11_error)
        __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

        /* "cupy/cuda/memory.pyx":1810
 *                     self._pools.append(self.set_pool(
 *                         pool_handles[dev_id], dev_id))
 *                     self.set_limit(**limit)             # <<<<<<<<<<<<<<
 *                 finally:
 *                     runtime.setDevice(prev_dev_id)
*/
        __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_mstate_global->__pyx_n_u_set_limit); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1810, __pyx_L11_error)
        __Pyx_GOTREF(__pyx_t_1);
        if (unlikely(__pyx_v_limit == Py_None)) {
          PyErr_SetString(PyExc_TypeError, "argument after ** must be a mapping, not NoneType");
          __PYX_ERR(0, 1810, __pyx_L11_error)
        }
        __pyx_t_10 = PyDict_Copy(__pyx_v_limit); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 1810, __pyx_L11_error)
        __Pyx_GOTREF(__pyx_t_10);
        __pyx_t_11 = __Pyx_PyObject_Call(__pyx_t_1, __pyx_mstate_global->__pyx_empty_tuple, __pyx_t_10); if (unlikely(!__pyx_t_11)) __PYX_ERR(0, 1810, __pyx_L11_error)
        __Pyx_GOTREF(__pyx_t_11);
        __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
        __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
        __Pyx_DECREF(__pyx_t_11); __pyx_t_11 = 0;
      }

      /* "cupy/cuda/memory.pyx":1812
 *                     self.set_limit(**limit)
 *                 finally:
 *                     runtime.setDevice(prev_dev_id)             # <<<<<<<<<<<<<<
 *         else:
 *             # use the same argument for all devices
*/
      /*finally:*/ {
        /*normal exit:*/{
          __pyx_t_11 = __pyx_f_13cupy_backends_4cuda_3api_7runtime_setDevice(__pyx_v_prev_dev_id, 0); if (unlikely(!__pyx_t_11)) __PYX_ERR(0, 1812, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_11);
          __Pyx_DECREF(__pyx_t_11); __pyx_t_11 = 0;
          goto __pyx_L12;
        }
        __pyx_L11_error:;
        /*exception exit:*/{
          __Pyx_PyThreadState_declare
          __Pyx_PyThreadState_assign
          __pyx_t_15 = 0; __pyx_t_16 = 0; __pyx_t_17 = 0; __pyx_t_18 = 0; __pyx_t_19 = 0; __pyx_t_20 = 0;
          __Pyx_XDECREF(__pyx_t_1); __pyx_t_1 = 0;
          __Pyx_XDECREF(__pyx_t_10); __pyx_t_10 = 0;
          __Pyx_XDECREF(__pyx_t_11); __pyx_t_11 = 0;
           __Pyx_ExceptionSwap(&__pyx_t_18, &__pyx_t_19, &__pyx_t_20);
          if ( unlikely(__Pyx_GetException(&__pyx_t_15, &__pyx_t_16, &__pyx_t_17) < 0)) __Pyx_ErrFetch(&__pyx_t_15, &__pyx_t_16, &__pyx_t_17);
          __Pyx_XGOTREF(__pyx_t_15);
          __Pyx_XGOTREF(__pyx_t_16);
          __Pyx_XGOTREF(__pyx_t_17);
          __Pyx_XGOTREF(__pyx_t_18);
          __Pyx_XGOTREF(__pyx_t_19);
          __Pyx_XGOTREF(__pyx_t_20);
          __pyx_t_12 = __pyx_lineno; __pyx_t_13 = __pyx_clineno; __pyx_t_14 = __pyx_filename;
          {
            __pyx_t_11 = __pyx_f_13cupy_backends_4cuda_3api_7runtime_setDevice(__pyx_v_prev_dev_id, 0); if (unlikely(!__pyx_t_11)) __PYX_ERR(0, 1812, __pyx_L16_error)
            __Pyx_GOTREF(__pyx_t_11);
            __Pyx_DECREF(__pyx_t_11); __pyx_t_11 = 0;
          }
          __Pyx_XGIVEREF(__pyx_t_18);
          __Pyx_XGIVEREF(__pyx_t_19);
          __Pyx_XGIVEREF(__pyx_t_20);
          __Pyx_ExceptionReset(__pyx_t_18, __pyx_t_19, __pyx_t_20);
          __Pyx_XGIVEREF(__pyx_t_15);
          __Pyx_XGIVEREF(__pyx_t_16);
          __Pyx_XGIVEREF(__pyx_t_17);
          __Pyx_ErrRestore(__pyx_t_15, __pyx_t_16, __pyx_t_17);
          __pyx_t_15 = 0; __pyx_t_16 = 0; __pyx_t_17 = 0; __pyx_t_18 = 0; __pyx_t_19 = 0; __pyx_t_20 = 0;
          __pyx_lineno = __pyx_t_12; __pyx_clineno = __pyx_t_13; __pyx_filename = __pyx_t_14;
          goto __pyx_L1_error;
          __pyx_L16_error:;
          __Pyx_XGIVEREF(__pyx_t_18);
          __Pyx_XGIVEREF(__pyx_t_19);
          __Pyx_XGIVEREF(__pyx_t_20);
          __Pyx_ExceptionReset(__pyx_t_18, __pyx_t_19, __pyx_t_20);
          __Pyx_XDECREF(__pyx_t_15); __pyx_t_15 = 0;
          __Pyx_XDECREF(__pyx_t_16); __pyx_t_16 = 0;
          __Pyx_XDECREF(__pyx_t_17); __pyx_t_17 = 0;
          __pyx_t_18 = 0; __pyx_t_19 = 0; __pyx_t_20 = 0;
          goto __pyx_L1_error;
        }
        __pyx_L12:;
      }
    }

    /* "cupy/cuda/memory.pyx":1802
 *         self.memoryAsyncHasStat = (runtime.driverGetVersion() >= 11030)
 *         prev_dev_id = runtime.getDevice()
 *         if (cpython.PySequence_Check(pool_handles)             # <<<<<<<<<<<<<<
 *                 and not isinstance(pool_handles, str)):
 *             # allow different kinds of handles on each device
*/
    goto __pyx_L3;
  }

  /* "cupy/cuda/memory.pyx":1815
 *         else:
 *             # use the same argument for all devices
 *             for dev_id in range(dev_counts):             # <<<<<<<<<<<<<<
 *                 try:
 *                     runtime.setDevice(dev_id)
*/
  /*else*/ {
    __pyx_t_2 = __pyx_v_dev_counts;
    __pyx_t_6 = __pyx_t_2;
    for (__pyx_t_7 = 0; __pyx_t_7 < __pyx_t_6; __pyx_t_7+=1) {
      __pyx_v_dev_id = __pyx_t_7;

      /* "cupy/cuda/memory.pyx":1816
 *             # use the same argument for all devices
 *             for dev_id in range(dev_counts):
 *                 try:             # <<<<<<<<<<<<<<
 *                     runtime.setDevice(dev_id)
 *                     self._pools.append(self.set_pool(pool_handles, dev_id))
*/
      /*try:*/ {

        /* "cupy/cuda/memory.pyx":1817
 *             for dev_id in range(dev_counts):
 *                 try:
 *                     runtime.setDevice(dev_id)             # <<<<<<<<<<<<<<
 *                     self._pools.append(self.set_pool(pool_handles, dev_id))
 *                     self.set_limit(**limit)
*/
        __pyx_t_11 = __pyx_f_13cupy_backends_4cuda_3api_7runtime_setDevice(__pyx_v_dev_id, 0); if (unlikely(!__pyx_t_11)) __PYX_ERR(0, 1817, __pyx_L22_error)
        __Pyx_GOTREF(__pyx_t_11);
        __Pyx_DECREF(__pyx_t_11); __pyx_t_11 = 0;

        /* "cupy/cuda/memory.pyx":1818
 *                 try:
 *                     runtime.setDevice(dev_id)
 *                     self._pools.append(self.set_pool(pool_handles, dev_id))             # <<<<<<<<<<<<<<
 *                     self.set_limit(**limit)
 *                 finally:
*/
        if (unlikely(__pyx_v_self->_pools == Py_None)) {
          PyErr_Format(PyExc_AttributeError, "'NoneType' object has no attribute '%.30s'", "append");
          __PYX_ERR(0, 1818, __pyx_L22_error)
        }
        __pyx_t_8 = ((struct __pyx_vtabstruct_4cupy_4cuda_6memory_MemoryAsyncPool *)__pyx_v_self->__pyx_vtab)->set_pool(__pyx_v_self, __pyx_v_pool_handles, __pyx_v_dev_id); if (unlikely(__pyx_t_8 == ((intptr_t)0) && PyErr_Occurred())) __PYX_ERR(0, 1818, __pyx_L22_error)
        __pyx_t_11 = PyLong_FromSsize_t(__pyx_t_8); if (unlikely(!__pyx_t_11)) __PYX_ERR(0, 1818, __pyx_L22_error)
        __Pyx_GOTREF(__pyx_t_11);
        __pyx_t_9 = __Pyx_PyList_Append(__pyx_v_self->_pools, __pyx_t_11); if (unlikely(__pyx_t_9 == ((int)-1))) __PYX_ERR(0, 1818, __pyx_L22_error)
        __Pyx_DECREF(__pyx_t_11); __pyx_t_11 = 0;

        /* "cupy/cuda/memory.pyx":1819
 *                     runtime.setDevice(dev_id)
 *                     self._pools.append(self.set_pool(pool_handles, dev_id))
 *                     self.set_limit(**limit)             # <<<<<<<<<<<<<<
 *                 finally:
 *                     runtime.setDevice(prev_dev_id)
*/
        __pyx_t_11 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_mstate_global->__pyx_n_u_set_limit); if (unlikely(!__pyx_t_11)) __PYX_ERR(0, 1819, __pyx_L22_error)
        __Pyx_GOTREF(__pyx_t_11);
        if (unlikely(__pyx_v_limit == Py_None)) {
          PyErr_SetString(PyExc_TypeError, "argument after ** must be a mapping, not NoneType");
          __PYX_ERR(0, 1819, __pyx_L22_error)
        }
        __pyx_t_10 = PyDict_Copy(__pyx_v_limit); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 1819, __pyx_L22_error)
        __Pyx_GOTREF(__pyx_t_10);
        __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_11, __pyx_mstate_global->__pyx_empty_tuple, __pyx_t_10); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1819, __pyx_L22_error)
        __Pyx_GOTREF(__pyx_t_1);
        __Pyx_DECREF(__pyx_t_11); __pyx_t_11 = 0;
        __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
        __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      }

      /* "cupy/cuda/memory.pyx":1821
 *                     self.set_limit(**limit)
 *                 finally:
 *                     runtime.setDevice(prev_dev_id)             # <<<<<<<<<<<<<<
 * 
 *     cdef intptr_t set_pool(self, handle, int dev_id) except? 0:
*/
      /*finally:*/ {
        /*normal exit:*/{
          __pyx_t_1 = __pyx_f_13cupy_backends_4cuda_3api_7runtime_setDevice(__pyx_v_prev_dev_id, 0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1821, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_1);
          __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
          goto __pyx_L23;
        }
        __pyx_L22_error:;
        /*exception exit:*/{
          __Pyx_PyThreadState_declare
          __Pyx_PyThreadState_assign
          __pyx_t_20 = 0; __pyx_t_19 = 0; __pyx_t_18 = 0; __pyx_t_17 = 0; __pyx_t_16 = 0; __pyx_t_15 = 0;
          __Pyx_XDECREF(__pyx_t_1); __pyx_t_1 = 0;
          __Pyx_XDECREF(__pyx_t_10); __pyx_t_10 = 0;
          __Pyx_XDECREF(__pyx_t_11); __pyx_t_11 = 0;
           __Pyx_ExceptionSwap(&__pyx_t_17, &__pyx_t_16, &__pyx_t_15);
          if ( unlikely(__Pyx_GetException(&__pyx_t_20, &__pyx_t_19, &__pyx_t_18) < 0)) __Pyx_ErrFetch(&__pyx_t_20, &__pyx_t_19, &__pyx_t_18);
          __Pyx_XGOTREF(__pyx_t_20);
          __Pyx_XGOTREF(__pyx_t_19);
          __Pyx_XGOTREF(__pyx_t_18);
          __Pyx_XGOTREF(__pyx_t_17);
          __Pyx_XGOTREF(__pyx_t_16);
          __Pyx_XGOTREF(__pyx_t_15);
          __pyx_t_13 = __pyx_lineno; __pyx_t_12 = __pyx_clineno; __pyx_t_21 = __pyx_filename;
          {
            __pyx_t_1 = __pyx_f_13cupy_backends_4cuda_3api_7runtime_setDevice(__pyx_v_prev_dev_id, 0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1821, __pyx_L27_error)
            __Pyx_GOTREF(__pyx_t_1);
            __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
          }
          __Pyx_XGIVEREF(__pyx_t_17);
          __Pyx_XGIVEREF(__pyx_t_16);
          __Pyx_XGIVEREF(__pyx_t_15);
          __Pyx_ExceptionReset(__pyx_t_17, __pyx_t_16, __pyx_t_15);
          __Pyx_XGIVEREF(__pyx_t_20);
          __Pyx_XGIVEREF(__pyx_t_19);
          __Pyx_XGIVEREF(__pyx_t_18);
          __Pyx_ErrRestore(__pyx_t_20, __pyx_t_19, __pyx_t_18);
          __pyx_t_20 = 0; __pyx_t_19 = 0; __pyx_t_18 = 0; __pyx_t_17 = 0; __pyx_t_16 = 0; __pyx_t_15 = 0;
          __pyx_lineno = __pyx_t_13; __pyx_clineno = __pyx_t_12; __pyx_filename = __pyx_t_21;
          goto __pyx_L1_error;
          __pyx_L27_error:;
          __Pyx_XGIVEREF(__pyx_t_17);
          __Pyx_XGIVEREF(__pyx_t_16);
          __Pyx_XGIVEREF(__pyx_t_15);
          __Pyx_ExceptionReset(__pyx_t_17, __pyx_t_16, __pyx_t_15);
          __Pyx_XDECREF(__pyx_t_20); __pyx_t_20 = 0;
          __Pyx_XDECREF(__pyx_t_19); __pyx_t_19 = 0;
          __Pyx_XDECREF(__pyx_t_18); __pyx_t_18 = 0;
          __pyx_t_17 = 0; __pyx_t_16 = 0; __pyx_t_15 = 0;
          goto __pyx_L1_error;
        }
        __pyx_L23:;
      }
    }
  }
  __pyx_L3:;

  /* "cupy/cuda/memory.pyx":1795
 *         readonly bint memoryAsyncHasStat
 * 
 *     def __init__(self, pool_handles='current'):             # <<<<<<<<<<<<<<
 *         cdef int dev_id, prev_dev_id, dev_counts
 *         cdef dict limit = _parse_limit_string()
*/

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_10);
  __Pyx_XDECREF(__pyx_t_11);
  __Pyx_AddTraceback("cupy.cuda.memory.MemoryAsyncPool.__init__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_limit);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "cupy/cuda/memory.pyx":1823
 *                     runtime.setDevice(prev_dev_id)
 * 
 *     cdef intptr_t set_pool(self, handle, int dev_id) except? 0:             # <<<<<<<<<<<<<<
 *         cdef intptr_t pool
 *         if handle == 'default':
*/

static intptr_t __pyx_f_4cupy_4cuda_6memory_15MemoryAsyncPool_set_pool(CYTHON_UNUSED struct __pyx_obj_4cupy_4cuda_6memory_MemoryAsyncPool *__pyx_v_self, PyObject *__pyx_v_handle, int __pyx_v_dev_id) {
  intptr_t __pyx_v_pool;
  intptr_t __pyx_r;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  intptr_t __pyx_t_2;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  size_t __pyx_t_5;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("set_pool", 0);

  /* "cupy/cuda/memory.pyx":1825
 *     cdef intptr_t set_pool(self, handle, int dev_id) except? 0:
 *         cdef intptr_t pool
 *         if handle == 'default':             # <<<<<<<<<<<<<<
 *             # Use the device's default pool
 *             pool = runtime.deviceGetDefaultMemPool(dev_id)
*/
  __pyx_t_1 = (__Pyx_PyUnicode_Equals(__pyx_v_handle, __pyx_mstate_global->__pyx_n_u_default, Py_EQ)); if (unlikely((__pyx_t_1 < 0))) __PYX_ERR(0, 1825, __pyx_L1_error)
  if (__pyx_t_1) {

    /* "cupy/cuda/memory.pyx":1827
 *         if handle == 'default':
 *             # Use the device's default pool
 *             pool = runtime.deviceGetDefaultMemPool(dev_id)             # <<<<<<<<<<<<<<
 *         elif handle == 'current':
 *             # Use the device's current pool
*/
    __pyx_t_2 = __pyx_f_13cupy_backends_4cuda_3api_7runtime_deviceGetDefaultMemPool(__pyx_v_dev_id, 0); if (unlikely(__pyx_t_2 == ((intptr_t)0) && PyErr_Occurred())) __PYX_ERR(0, 1827, __pyx_L1_error)
    __pyx_v_pool = __pyx_t_2;

    /* "cupy/cuda/memory.pyx":1825
 *     cdef intptr_t set_pool(self, handle, int dev_id) except? 0:
 *         cdef intptr_t pool
 *         if handle == 'default':             # <<<<<<<<<<<<<<
 *             # Use the device's default pool
 *             pool = runtime.deviceGetDefaultMemPool(dev_id)
*/
    goto __pyx_L3;
  }

  /* "cupy/cuda/memory.pyx":1828
 *             # Use the device's default pool
 *             pool = runtime.deviceGetDefaultMemPool(dev_id)
 *         elif handle == 'current':             # <<<<<<<<<<<<<<
 *             # Use the device's current pool
 *             pool = runtime.deviceGetMemPool(dev_id)
*/
  __pyx_t_1 = (__Pyx_PyUnicode_Equals(__pyx_v_handle, __pyx_mstate_global->__pyx_n_u_current, Py_EQ)); if (unlikely((__pyx_t_1 < 0))) __PYX_ERR(0, 1828, __pyx_L1_error)
  if (__pyx_t_1) {

    /* "cupy/cuda/memory.pyx":1830
 *         elif handle == 'current':
 *             # Use the device's current pool
 *             pool = runtime.deviceGetMemPool(dev_id)             # <<<<<<<<<<<<<<
 *         elif handle == 'create':
 *             # TODO(leofang): Support cudaMemPoolCreate
*/
    __pyx_t_2 = __pyx_f_13cupy_backends_4cuda_3api_7runtime_deviceGetMemPool(__pyx_v_dev_id, 0); if (unlikely(__pyx_t_2 == ((intptr_t)0) && PyErr_Occurred())) __PYX_ERR(0, 1830, __pyx_L1_error)
    __pyx_v_pool = __pyx_t_2;

    /* "cupy/cuda/memory.pyx":1828
 *             # Use the device's default pool
 *             pool = runtime.deviceGetDefaultMemPool(dev_id)
 *         elif handle == 'current':             # <<<<<<<<<<<<<<
 *             # Use the device's current pool
 *             pool = runtime.deviceGetMemPool(dev_id)
*/
    goto __pyx_L3;
  }

  /* "cupy/cuda/memory.pyx":1831
 *             # Use the device's current pool
 *             pool = runtime.deviceGetMemPool(dev_id)
 *         elif handle == 'create':             # <<<<<<<<<<<<<<
 *             # TODO(leofang): Support cudaMemPoolCreate
 *             raise NotImplementedError('cudaMemPoolCreate is not yet supported')
*/
  __pyx_t_1 = (__Pyx_PyUnicode_Equals(__pyx_v_handle, __pyx_mstate_global->__pyx_n_u_create, Py_EQ)); if (unlikely((__pyx_t_1 < 0))) __PYX_ERR(0, 1831, __pyx_L1_error)
  if (unlikely(__pyx_t_1)) {

    /* "cupy/cuda/memory.pyx":1833
 *         elif handle == 'create':
 *             # TODO(leofang): Support cudaMemPoolCreate
 *             raise NotImplementedError('cudaMemPoolCreate is not yet supported')             # <<<<<<<<<<<<<<
 *         elif isinstance(handle, int):
 *             # Use an existing pool (likely from other applications?)
*/
    __pyx_t_4 = NULL;
    __pyx_t_5 = 1;
    {
      PyObject *__pyx_callargs[2] = {__pyx_t_4, __pyx_mstate_global->__pyx_kp_u_cudaMemPoolCreate_is_not_yet_sup};
      __pyx_t_3 = __Pyx_PyObject_FastCall((PyObject*)(((PyTypeObject*)PyExc_NotImplementedError)), __pyx_callargs+__pyx_t_5, (2-__pyx_t_5) | (__pyx_t_5*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
      __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
      if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 1833, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_3);
    }
    __Pyx_Raise(__pyx_t_3, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __PYX_ERR(0, 1833, __pyx_L1_error)

    /* "cupy/cuda/memory.pyx":1831
 *             # Use the device's current pool
 *             pool = runtime.deviceGetMemPool(dev_id)
 *         elif handle == 'create':             # <<<<<<<<<<<<<<
 *             # TODO(leofang): Support cudaMemPoolCreate
 *             raise NotImplementedError('cudaMemPoolCreate is not yet supported')
*/
  }

  /* "cupy/cuda/memory.pyx":1834
 *             # TODO(leofang): Support cudaMemPoolCreate
 *             raise NotImplementedError('cudaMemPoolCreate is not yet supported')
 *         elif isinstance(handle, int):             # <<<<<<<<<<<<<<
 *             # Use an existing pool (likely from other applications?)
 *             pool = <intptr_t>(handle)
*/
  __pyx_t_1 = PyLong_Check(__pyx_v_handle); 
  if (likely(__pyx_t_1)) {

    /* "cupy/cuda/memory.pyx":1836
 *         elif isinstance(handle, int):
 *             # Use an existing pool (likely from other applications?)
 *             pool = <intptr_t>(handle)             # <<<<<<<<<<<<<<
 *         else:
 *             raise ValueError("handle must be "
*/
    __pyx_t_2 = PyLong_AsSsize_t(__pyx_v_handle); if (unlikely((__pyx_t_2 == ((intptr_t)-1)) && PyErr_Occurred())) __PYX_ERR(0, 1836, __pyx_L1_error)
    __pyx_v_pool = ((intptr_t)__pyx_t_2);

    /* "cupy/cuda/memory.pyx":1834
 *             # TODO(leofang): Support cudaMemPoolCreate
 *             raise NotImplementedError('cudaMemPoolCreate is not yet supported')
 *         elif isinstance(handle, int):             # <<<<<<<<<<<<<<
 *             # Use an existing pool (likely from other applications?)
 *             pool = <intptr_t>(handle)
*/
    goto __pyx_L3;
  }

  /* "cupy/cuda/memory.pyx":1838
 *             pool = <intptr_t>(handle)
 *         else:
 *             raise ValueError("handle must be "             # <<<<<<<<<<<<<<
 *                              "'default' (for the device's default pool), "
 *                              "'current' (for the device's current pool), "
*/
  /*else*/ {
    __pyx_t_4 = NULL;
    __pyx_t_5 = 1;
    {
      PyObject *__pyx_callargs[2] = {__pyx_t_4, __pyx_mstate_global->__pyx_kp_u_handle_must_be_default_for_the_d};
      __pyx_t_3 = __Pyx_PyObject_FastCall((PyObject*)(((PyTypeObject*)PyExc_ValueError)), __pyx_callargs+__pyx_t_5, (2-__pyx_t_5) | (__pyx_t_5*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
      __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
      if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 1838, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_3);
    }
    __Pyx_Raise(__pyx_t_3, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __PYX_ERR(0, 1838, __pyx_L1_error)
  }
  __pyx_L3:;

  /* "cupy/cuda/memory.pyx":1842
 *                              "'current' (for the device's current pool), "
 *                              "or int (a pointer to cudaMemPool_t)")
 *         runtime.deviceSetMemPool(dev_id, pool)             # <<<<<<<<<<<<<<
 *         return pool
 * 
*/
  __pyx_t_3 = __pyx_f_13cupy_backends_4cuda_3api_7runtime_deviceSetMemPool(__pyx_v_dev_id, __pyx_v_pool, 0); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 1842, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "cupy/cuda/memory.pyx":1843
 *                              "or int (a pointer to cudaMemPool_t)")
 *         runtime.deviceSetMemPool(dev_id, pool)
 *         return pool             # <<<<<<<<<<<<<<
 * 
 *     cpdef MemoryPointer malloc(self, size_t size):
*/
  __pyx_r = __pyx_v_pool;
  goto __pyx_L0;

  /* "cupy/cuda/memory.pyx":1823
 *                     runtime.setDevice(prev_dev_id)
 * 
 *     cdef intptr_t set_pool(self, handle, int dev_id) except? 0:             # <<<<<<<<<<<<<<
 *         cdef intptr_t pool
 *         if handle == 'default':
*/

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_AddTraceback("cupy.cuda.memory.MemoryAsyncPool.set_pool", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "cupy/cuda/memory.pyx":1845
 *         return pool
 * 
 *     cpdef MemoryPointer malloc(self, size_t size):             # <<<<<<<<<<<<<<
 *         """Allocate memory from the current device's pool on the current
 *         stream.
*/

static PyObject *__pyx_pw_4cupy_4cuda_6memory_15MemoryAsyncPool_3malloc(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
static struct __pyx_obj_4cupy_4cuda_6memory_MemoryPointer *__pyx_f_4cupy_4cuda_6memory_15MemoryAsyncPool_malloc(struct __pyx_obj_4cupy_4cuda_6memory_MemoryAsyncPool *__pyx_v_self, size_t __pyx_v_size, int __pyx_skip_dispatch) {
  size_t __pyx_v_rounded_size;
  struct __pyx_obj_4cupy_4cuda_6memory_MemoryPointer *__pyx_v_mem = NULL;
  int __pyx_v_oom_error;
  size_t __pyx_v_curr_total;
  size_t __pyx_v_curr_free;
  size_t __pyx_v_total_limit;
  PyObject *__pyx_v_e = NULL;
  PyObject *__pyx_v_stream = NULL;
  struct __pyx_obj_4cupy_4cuda_6memory_MemoryPointer *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  size_t __pyx_t_6;
  int __pyx_t_7;
  size_t __pyx_t_8;
  size_t __pyx_t_9;
  PyObject *__pyx_t_10 = NULL;
  PyObject *__pyx_t_11 = NULL;
  PyObject *__pyx_t_12 = NULL;
  PyObject *__pyx_t_13 = NULL;
  int __pyx_t_14;
  PyObject *__pyx_t_15 = NULL;
  PyObject *__pyx_t_16 = NULL;
  PyObject *__pyx_t_17 = NULL;
  PyObject *__pyx_t_18 = NULL;
  PyObject *__pyx_t_19 = NULL;
  PyObject *__pyx_t_20 = NULL;
  PyObject *__pyx_t_21 = NULL;
  PyObject *__pyx_t_22 = NULL;
  PyObject *__pyx_t_23 = NULL;
  PyObject *__pyx_t_24 = NULL;
  PyObject *__pyx_t_25 = NULL;
  PyObject *__pyx_t_26 = NULL;
  int __pyx_t_27;
  char const *__pyx_t_28;
  PyObject *__pyx_t_29 = NULL;
  PyObject *__pyx_t_30 = NULL;
  PyObject *__pyx_t_31 = NULL;
  PyObject *__pyx_t_32 = NULL;
  PyObject *__pyx_t_33 = NULL;
  PyObject *__pyx_t_34 = NULL;
  char const *__pyx_t_35;
  char const *__pyx_t_36;
  char const *__pyx_t_37;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("malloc", 0);
  /* Check if called by wrapper */
  if (unlikely(__pyx_skip_dispatch)) ;
  /* Check if overridden in Python */
  else if (
  #if !CYTHON_USE_TYPE_SLOTS
  unlikely(Py_TYPE(((PyObject *)__pyx_v_self)) != __pyx_mstate_global->__pyx_ptype_4cupy_4cuda_6memory_MemoryAsyncPool &&
  __Pyx_PyType_HasFeature(Py_TYPE(((PyObject *)__pyx_v_self)), Py_TPFLAGS_HAVE_GC))
  #else
  unlikely(Py_TYPE(((PyObject *)__pyx_v_self))->tp_dictoffset != 0 || __Pyx_PyType_HasFeature(Py_TYPE(((PyObject *)__pyx_v_self)), (Py_TPFLAGS_IS_ABSTRACT | Py_TPFLAGS_HEAPTYPE)))
  #endif
  ) {
    #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
    static PY_UINT64_T __pyx_tp_dict_version = __PYX_DICT_VERSION_INIT, __pyx_obj_dict_version = __PYX_DICT_VERSION_INIT;
    if (unlikely(!__Pyx_object_dict_version_matches(((PyObject *)__pyx_v_self), __pyx_tp_dict_version, __pyx_obj_dict_version))) {
      PY_UINT64_T __pyx_typedict_guard = __Pyx_get_tp_dict_version(((PyObject *)__pyx_v_self));
      #endif
      __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_mstate_global->__pyx_n_u_malloc_2); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1845, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      if (!__Pyx_IsSameCFunction(__pyx_t_1, (void(*)(void)) __pyx_pw_4cupy_4cuda_6memory_15MemoryAsyncPool_3malloc)) {
        __Pyx_XDECREF((PyObject *)__pyx_r);
        __pyx_t_3 = NULL;
        __Pyx_INCREF(__pyx_t_1);
        __pyx_t_4 = __pyx_t_1; 
        __pyx_t_5 = __Pyx_PyLong_FromSize_t(__pyx_v_size); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 1845, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_5);
        __pyx_t_6 = 1;
        #if CYTHON_UNPACK_METHODS
        if (unlikely(PyMethod_Check(__pyx_t_4))) {
          __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_4);
          assert(__pyx_t_3);
          PyObject* __pyx__function = PyMethod_GET_FUNCTION(__pyx_t_4);
          __Pyx_INCREF(__pyx_t_3);
          __Pyx_INCREF(__pyx__function);
          __Pyx_DECREF_SET(__pyx_t_4, __pyx__function);
          __pyx_t_6 = 0;
        }
        #endif
        {
          PyObject *__pyx_callargs[2] = {__pyx_t_3, __pyx_t_5};
          __pyx_t_2 = __Pyx_PyObject_FastCall((PyObject*)__pyx_t_4, __pyx_callargs+__pyx_t_6, (2-__pyx_t_6) | (__pyx_t_6*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
          __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
          __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
          __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
          if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1845, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_2);
        }
        if (!(likely(((__pyx_t_2) == Py_None) || likely(__Pyx_TypeTest(__pyx_t_2, __pyx_mstate_global->__pyx_ptype_4cupy_4cuda_6memory_MemoryPointer))))) __PYX_ERR(0, 1845, __pyx_L1_error)
        __pyx_r = ((struct __pyx_obj_4cupy_4cuda_6memory_MemoryPointer *)__pyx_t_2);
        __pyx_t_2 = 0;
        __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
        goto __pyx_L0;
      }
      #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
      __pyx_tp_dict_version = __Pyx_get_tp_dict_version(((PyObject *)__pyx_v_self));
      __pyx_obj_dict_version = __Pyx_get_object_dict_version(((PyObject *)__pyx_v_self));
      if (unlikely(__pyx_typedict_guard != __pyx_tp_dict_version)) {
        __pyx_tp_dict_version = __pyx_obj_dict_version = __PYX_DICT_VERSION_INIT;
      }
      #endif
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
    }
    #endif
  }

  /* "cupy/cuda/memory.pyx":1861
 * 
 *         """
 *         cdef size_t rounded_size = _round_size(size)             # <<<<<<<<<<<<<<
 *         mem = None
 *         oom_error = False
*/
  __pyx_v_rounded_size = __pyx_f_4cupy_4cuda_6memory__round_size(__pyx_v_size, 0);

  /* "cupy/cuda/memory.pyx":1862
 *         """
 *         cdef size_t rounded_size = _round_size(size)
 *         mem = None             # <<<<<<<<<<<<<<
 *         oom_error = False
 * 
*/
  __Pyx_INCREF(Py_None);
  __pyx_v_mem = ((struct __pyx_obj_4cupy_4cuda_6memory_MemoryPointer *)Py_None);

  /* "cupy/cuda/memory.pyx":1863
 *         cdef size_t rounded_size = _round_size(size)
 *         mem = None
 *         oom_error = False             # <<<<<<<<<<<<<<
 * 
 *         # CUDA does not allow us to set a hard limit, so the best we can do is
*/
  __pyx_v_oom_error = 0;

  /* "cupy/cuda/memory.pyx":1868
 *         # to prevent CuPy from drawing too much memory from the pool; we cannot
 *         # do anything if other applications oversubscribe the pool.
 *         cdef size_t curr_total=-1, curr_free=0, total_limit=0             # <<<<<<<<<<<<<<
 *         if self.memoryAsyncHasStat:
 *             curr_total = self.total_bytes()
*/
  __pyx_v_curr_total = -1L;
  __pyx_v_curr_free = 0;
  __pyx_v_total_limit = 0;

  /* "cupy/cuda/memory.pyx":1869
 *         # do anything if other applications oversubscribe the pool.
 *         cdef size_t curr_total=-1, curr_free=0, total_limit=0
 *         if self.memoryAsyncHasStat:             # <<<<<<<<<<<<<<
 *             curr_total = self.total_bytes()
 *             curr_free = curr_total - self.used_bytes()
*/
  if (__pyx_v_self->memoryAsyncHasStat) {

    /* "cupy/cuda/memory.pyx":1870
 *         cdef size_t curr_total=-1, curr_free=0, total_limit=0
 *         if self.memoryAsyncHasStat:
 *             curr_total = self.total_bytes()             # <<<<<<<<<<<<<<
 *             curr_free = curr_total - self.used_bytes()
 *             if curr_free < rounded_size:  # need to increase pool size
*/
    __pyx_t_6 = ((struct __pyx_vtabstruct_4cupy_4cuda_6memory_MemoryAsyncPool *)__pyx_v_self->__pyx_vtab)->total_bytes(__pyx_v_self, 0); if (unlikely(PyErr_Occurred())) __PYX_ERR(0, 1870, __pyx_L1_error)
    __pyx_v_curr_total = __pyx_t_6;

    /* "cupy/cuda/memory.pyx":1871
 *         if self.memoryAsyncHasStat:
 *             curr_total = self.total_bytes()
 *             curr_free = curr_total - self.used_bytes()             # <<<<<<<<<<<<<<
 *             if curr_free < rounded_size:  # need to increase pool size
 *                 total_limit = self.get_limit()
*/
    __pyx_t_6 = ((struct __pyx_vtabstruct_4cupy_4cuda_6memory_MemoryAsyncPool *)__pyx_v_self->__pyx_vtab)->used_bytes(__pyx_v_self, 0); if (unlikely(PyErr_Occurred())) __PYX_ERR(0, 1871, __pyx_L1_error)
    __pyx_v_curr_free = (__pyx_v_curr_total - __pyx_t_6);

    /* "cupy/cuda/memory.pyx":1872
 *             curr_total = self.total_bytes()
 *             curr_free = curr_total - self.used_bytes()
 *             if curr_free < rounded_size:  # need to increase pool size             # <<<<<<<<<<<<<<
 *                 total_limit = self.get_limit()
 *                 if max(total_limit, curr_total) < curr_total + rounded_size:
*/
    __pyx_t_7 = (__pyx_v_curr_free < __pyx_v_rounded_size);
    if (__pyx_t_7) {

      /* "cupy/cuda/memory.pyx":1873
 *             curr_free = curr_total - self.used_bytes()
 *             if curr_free < rounded_size:  # need to increase pool size
 *                 total_limit = self.get_limit()             # <<<<<<<<<<<<<<
 *                 if max(total_limit, curr_total) < curr_total + rounded_size:
 *                     raise OutOfMemoryError(size, curr_total, total_limit)
*/
      __pyx_t_6 = ((struct __pyx_vtabstruct_4cupy_4cuda_6memory_MemoryAsyncPool *)__pyx_v_self->__pyx_vtab)->get_limit(__pyx_v_self, 0); if (unlikely(PyErr_Occurred())) __PYX_ERR(0, 1873, __pyx_L1_error)
      __pyx_v_total_limit = __pyx_t_6;

      /* "cupy/cuda/memory.pyx":1874
 *             if curr_free < rounded_size:  # need to increase pool size
 *                 total_limit = self.get_limit()
 *                 if max(total_limit, curr_total) < curr_total + rounded_size:             # <<<<<<<<<<<<<<
 *                     raise OutOfMemoryError(size, curr_total, total_limit)
 * 
*/
      __pyx_t_6 = __pyx_v_curr_total;
      __pyx_t_8 = __pyx_v_total_limit;
      __pyx_t_7 = (__pyx_t_6 > __pyx_t_8);
      if (__pyx_t_7) {
        __pyx_t_9 = __pyx_t_6;
      } else {
        __pyx_t_9 = __pyx_t_8;
      }
      __pyx_t_7 = (__pyx_t_9 < (__pyx_v_curr_total + __pyx_v_rounded_size));
      if (unlikely(__pyx_t_7)) {

        /* "cupy/cuda/memory.pyx":1875
 *                 total_limit = self.get_limit()
 *                 if max(total_limit, curr_total) < curr_total + rounded_size:
 *                     raise OutOfMemoryError(size, curr_total, total_limit)             # <<<<<<<<<<<<<<
 * 
 *         try:
*/
        __pyx_t_2 = NULL;
        __Pyx_GetModuleGlobalName(__pyx_t_4, __pyx_mstate_global->__pyx_n_u_OutOfMemoryError); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 1875, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_4);
        __pyx_t_5 = __Pyx_PyLong_FromSize_t(__pyx_v_size); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 1875, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_5);
        __pyx_t_3 = __Pyx_PyLong_FromSize_t(__pyx_v_curr_total); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 1875, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_3);
        __pyx_t_10 = __Pyx_PyLong_FromSize_t(__pyx_v_total_limit); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 1875, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_10);
        __pyx_t_9 = 1;
        #if CYTHON_UNPACK_METHODS
        if (unlikely(PyMethod_Check(__pyx_t_4))) {
          __pyx_t_2 = PyMethod_GET_SELF(__pyx_t_4);
          assert(__pyx_t_2);
          PyObject* __pyx__function = PyMethod_GET_FUNCTION(__pyx_t_4);
          __Pyx_INCREF(__pyx_t_2);
          __Pyx_INCREF(__pyx__function);
          __Pyx_DECREF_SET(__pyx_t_4, __pyx__function);
          __pyx_t_9 = 0;
        }
        #endif
        {
          PyObject *__pyx_callargs[4] = {__pyx_t_2, __pyx_t_5, __pyx_t_3, __pyx_t_10};
          __pyx_t_1 = __Pyx_PyObject_FastCall((PyObject*)__pyx_t_4, __pyx_callargs+__pyx_t_9, (4-__pyx_t_9) | (__pyx_t_9*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
          __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
          __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
          __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
          __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
          __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
          if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1875, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_1);
        }
        __Pyx_Raise(__pyx_t_1, 0, 0, 0);
        __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
        __PYX_ERR(0, 1875, __pyx_L1_error)

        /* "cupy/cuda/memory.pyx":1874
 *             if curr_free < rounded_size:  # need to increase pool size
 *                 total_limit = self.get_limit()
 *                 if max(total_limit, curr_total) < curr_total + rounded_size:             # <<<<<<<<<<<<<<
 *                     raise OutOfMemoryError(size, curr_total, total_limit)
 * 
*/
      }

      /* "cupy/cuda/memory.pyx":1872
 *             curr_total = self.total_bytes()
 *             curr_free = curr_total - self.used_bytes()
 *             if curr_free < rounded_size:  # need to increase pool size             # <<<<<<<<<<<<<<
 *                 total_limit = self.get_limit()
 *                 if max(total_limit, curr_total) < curr_total + rounded_size:
*/
    }

    /* "cupy/cuda/memory.pyx":1869
 *         # do anything if other applications oversubscribe the pool.
 *         cdef size_t curr_total=-1, curr_free=0, total_limit=0
 *         if self.memoryAsyncHasStat:             # <<<<<<<<<<<<<<
 *             curr_total = self.total_bytes()
 *             curr_free = curr_total - self.used_bytes()
*/
  }

  /* "cupy/cuda/memory.pyx":1877
 *                     raise OutOfMemoryError(size, curr_total, total_limit)
 * 
 *         try:             # <<<<<<<<<<<<<<
 *             mem = malloc_async(rounded_size)
 *         except CUDARuntimeError as e:
*/
  /*try:*/ {
    {
      __Pyx_PyThreadState_declare
      __Pyx_PyThreadState_assign
      __Pyx_ExceptionSave(&__pyx_t_11, &__pyx_t_12, &__pyx_t_13);
      __Pyx_XGOTREF(__pyx_t_11);
      __Pyx_XGOTREF(__pyx_t_12);
      __Pyx_XGOTREF(__pyx_t_13);
      /*try:*/ {

        /* "cupy/cuda/memory.pyx":1878
 * 
 *         try:
 *             mem = malloc_async(rounded_size)             # <<<<<<<<<<<<<<
 *         except CUDARuntimeError as e:
 *             if e.status != runtime.errorMemoryAllocation:
*/
        __pyx_t_1 = ((PyObject *)__pyx_f_4cupy_4cuda_6memory_malloc_async(__pyx_v_rounded_size, 0)); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1878, __pyx_L9_error)
        __Pyx_GOTREF(__pyx_t_1);
        __Pyx_DECREF_SET(__pyx_v_mem, ((struct __pyx_obj_4cupy_4cuda_6memory_MemoryPointer *)__pyx_t_1));
        __pyx_t_1 = 0;

        /* "cupy/cuda/memory.pyx":1877
 *                     raise OutOfMemoryError(size, curr_total, total_limit)
 * 
 *         try:             # <<<<<<<<<<<<<<
 *             mem = malloc_async(rounded_size)
 *         except CUDARuntimeError as e:
*/
      }
      __Pyx_XDECREF(__pyx_t_11); __pyx_t_11 = 0;
      __Pyx_XDECREF(__pyx_t_12); __pyx_t_12 = 0;
      __Pyx_XDECREF(__pyx_t_13); __pyx_t_13 = 0;
      goto __pyx_L14_try_end;
      __pyx_L9_error:;
      __Pyx_XDECREF(__pyx_t_1); __pyx_t_1 = 0;
      __Pyx_XDECREF(__pyx_t_10); __pyx_t_10 = 0;
      __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
      __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
      __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
      __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;

      /* "cupy/cuda/memory.pyx":1879
 *         try:
 *             mem = malloc_async(rounded_size)
 *         except CUDARuntimeError as e:             # <<<<<<<<<<<<<<
 *             if e.status != runtime.errorMemoryAllocation:
 *                 raise
*/
      __Pyx_ErrFetch(&__pyx_t_1, &__pyx_t_4, &__pyx_t_10);
      __Pyx_GetModuleGlobalName(__pyx_t_3, __pyx_mstate_global->__pyx_n_u_CUDARuntimeError); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 1879, __pyx_L11_except_error)
      __Pyx_GOTREF(__pyx_t_3);
      __pyx_t_14 = __Pyx_PyErr_GivenExceptionMatches(__pyx_t_1, __pyx_t_3);
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
      __Pyx_ErrRestore(__pyx_t_1, __pyx_t_4, __pyx_t_10);
      __pyx_t_1 = 0; __pyx_t_4 = 0; __pyx_t_10 = 0;
      if (__pyx_t_14) {
        __Pyx_AddTraceback("cupy.cuda.memory.MemoryAsyncPool.malloc", __pyx_clineno, __pyx_lineno, __pyx_filename);
        if (__Pyx_GetException(&__pyx_t_10, &__pyx_t_4, &__pyx_t_1) < 0) __PYX_ERR(0, 1879, __pyx_L11_except_error)
        __Pyx_XGOTREF(__pyx_t_10);
        __Pyx_XGOTREF(__pyx_t_4);
        __Pyx_XGOTREF(__pyx_t_1);
        __Pyx_INCREF(__pyx_t_4);
        __pyx_v_e = __pyx_t_4;
        /*try:*/ {

          /* "cupy/cuda/memory.pyx":1880
 *             mem = malloc_async(rounded_size)
 *         except CUDARuntimeError as e:
 *             if e.status != runtime.errorMemoryAllocation:             # <<<<<<<<<<<<<<
 *                 raise
 *             stream = stream_module.get_current_stream()
*/
          __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_v_e, __pyx_mstate_global->__pyx_n_u_status); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 1880, __pyx_L20_error)
          __Pyx_GOTREF(__pyx_t_3);
          __pyx_t_5 = __Pyx_PyLong_From_int(__pyx_v_13cupy_backends_4cuda_3api_7runtime_errorMemoryAllocation); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 1880, __pyx_L20_error)
          __Pyx_GOTREF(__pyx_t_5);
          __pyx_t_2 = PyObject_RichCompare(__pyx_t_3, __pyx_t_5, Py_NE); __Pyx_XGOTREF(__pyx_t_2); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1880, __pyx_L20_error)
          __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
          __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
          __pyx_t_7 = __Pyx_PyObject_IsTrue(__pyx_t_2); if (unlikely((__pyx_t_7 < 0))) __PYX_ERR(0, 1880, __pyx_L20_error)
          __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
          if (unlikely(__pyx_t_7)) {

            /* "cupy/cuda/memory.pyx":1881
 *         except CUDARuntimeError as e:
 *             if e.status != runtime.errorMemoryAllocation:
 *                 raise             # <<<<<<<<<<<<<<
 *             stream = stream_module.get_current_stream()
 *             stream.synchronize()
*/
            __Pyx_GIVEREF(__pyx_t_10);
            __Pyx_GIVEREF(__pyx_t_4);
            __Pyx_XGIVEREF(__pyx_t_1);
            __Pyx_ErrRestoreWithState(__pyx_t_10, __pyx_t_4, __pyx_t_1);
            __pyx_t_10 = 0;  __pyx_t_4 = 0;  __pyx_t_1 = 0; 
            __PYX_ERR(0, 1881, __pyx_L20_error)

            /* "cupy/cuda/memory.pyx":1880
 *             mem = malloc_async(rounded_size)
 *         except CUDARuntimeError as e:
 *             if e.status != runtime.errorMemoryAllocation:             # <<<<<<<<<<<<<<
 *                 raise
 *             stream = stream_module.get_current_stream()
*/
          }

          /* "cupy/cuda/memory.pyx":1882
 *             if e.status != runtime.errorMemoryAllocation:
 *                 raise
 *             stream = stream_module.get_current_stream()             # <<<<<<<<<<<<<<
 *             stream.synchronize()
 *             try:
*/
          __pyx_t_2 = __pyx_f_4cupy_4cuda_6stream_get_current_stream(0, NULL); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1882, __pyx_L20_error)
          __Pyx_GOTREF(__pyx_t_2);
          __pyx_v_stream = __pyx_t_2;
          __pyx_t_2 = 0;

          /* "cupy/cuda/memory.pyx":1883
 *                 raise
 *             stream = stream_module.get_current_stream()
 *             stream.synchronize()             # <<<<<<<<<<<<<<
 *             try:
 *                 mem = malloc_async(rounded_size)
*/
          __pyx_t_5 = __pyx_v_stream;
          __Pyx_INCREF(__pyx_t_5);
          __pyx_t_9 = 0;
          {
            PyObject *__pyx_callargs[2] = {__pyx_t_5, NULL};
            __pyx_t_2 = __Pyx_PyObject_FastCallMethod((PyObject*)__pyx_mstate_global->__pyx_n_u_synchronize, __pyx_callargs+__pyx_t_9, (1-__pyx_t_9) | (1*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
            __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
            if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1883, __pyx_L20_error)
            __Pyx_GOTREF(__pyx_t_2);
          }
          __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

          /* "cupy/cuda/memory.pyx":1884
 *             stream = stream_module.get_current_stream()
 *             stream.synchronize()
 *             try:             # <<<<<<<<<<<<<<
 *                 mem = malloc_async(rounded_size)
 *             except CUDARuntimeError as e:
*/
          {
            __Pyx_PyThreadState_declare
            __Pyx_PyThreadState_assign
            __Pyx_ExceptionSave(&__pyx_t_15, &__pyx_t_16, &__pyx_t_17);
            __Pyx_XGOTREF(__pyx_t_15);
            __Pyx_XGOTREF(__pyx_t_16);
            __Pyx_XGOTREF(__pyx_t_17);
            /*try:*/ {

              /* "cupy/cuda/memory.pyx":1885
 *             stream.synchronize()
 *             try:
 *                 mem = malloc_async(rounded_size)             # <<<<<<<<<<<<<<
 *             except CUDARuntimeError as e:
 *                 if e.status != runtime.errorMemoryAllocation:
*/
              __pyx_t_2 = ((PyObject *)__pyx_f_4cupy_4cuda_6memory_malloc_async(__pyx_v_rounded_size, 0)); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1885, __pyx_L23_error)
              __Pyx_GOTREF(__pyx_t_2);
              __Pyx_DECREF_SET(__pyx_v_mem, ((struct __pyx_obj_4cupy_4cuda_6memory_MemoryPointer *)__pyx_t_2));
              __pyx_t_2 = 0;

              /* "cupy/cuda/memory.pyx":1884
 *             stream = stream_module.get_current_stream()
 *             stream.synchronize()
 *             try:             # <<<<<<<<<<<<<<
 *                 mem = malloc_async(rounded_size)
 *             except CUDARuntimeError as e:
*/
            }
            __Pyx_XDECREF(__pyx_t_15); __pyx_t_15 = 0;
            __Pyx_XDECREF(__pyx_t_16); __pyx_t_16 = 0;
            __Pyx_XDECREF(__pyx_t_17); __pyx_t_17 = 0;
            goto __pyx_L30_try_end;
            __pyx_L23_error:;
            __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
            __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
            __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;

            /* "cupy/cuda/memory.pyx":1886
 *             try:
 *                 mem = malloc_async(rounded_size)
 *             except CUDARuntimeError as e:             # <<<<<<<<<<<<<<
 *                 if e.status != runtime.errorMemoryAllocation:
 *                     raise
*/
            __Pyx_ErrFetch(&__pyx_t_2, &__pyx_t_5, &__pyx_t_3);
            __Pyx_GetModuleGlobalName(__pyx_t_18, __pyx_mstate_global->__pyx_n_u_CUDARuntimeError); if (unlikely(!__pyx_t_18)) __PYX_ERR(0, 1886, __pyx_L25_except_error)
            __Pyx_GOTREF(__pyx_t_18);
            __pyx_t_14 = __Pyx_PyErr_GivenExceptionMatches(__pyx_t_2, __pyx_t_18);
            __Pyx_DECREF(__pyx_t_18); __pyx_t_18 = 0;
            __Pyx_ErrRestore(__pyx_t_2, __pyx_t_5, __pyx_t_3);
            __pyx_t_2 = 0; __pyx_t_5 = 0; __pyx_t_3 = 0;
            if (__pyx_t_14) {
              __Pyx_AddTraceback("cupy.cuda.memory.MemoryAsyncPool.malloc", __pyx_clineno, __pyx_lineno, __pyx_filename);
              if (__Pyx_GetException(&__pyx_t_3, &__pyx_t_5, &__pyx_t_2) < 0) __PYX_ERR(0, 1886, __pyx_L25_except_error)
              __Pyx_XGOTREF(__pyx_t_3);
              __Pyx_XGOTREF(__pyx_t_5);
              __Pyx_XGOTREF(__pyx_t_2);
              __Pyx_INCREF(__pyx_t_5);
              __Pyx_DECREF_SET(__pyx_v_e, __pyx_t_5);
              /*try:*/ {

                /* "cupy/cuda/memory.pyx":1887
 *                 mem = malloc_async(rounded_size)
 *             except CUDARuntimeError as e:
 *                 if e.status != runtime.errorMemoryAllocation:             # <<<<<<<<<<<<<<
 *                     raise
 *                 stream.synchronize()
*/
                __pyx_t_18 = __Pyx_PyObject_GetAttrStr(__pyx_v_e, __pyx_mstate_global->__pyx_n_u_status); if (unlikely(!__pyx_t_18)) __PYX_ERR(0, 1887, __pyx_L36_error)
                __Pyx_GOTREF(__pyx_t_18);
                __pyx_t_19 = __Pyx_PyLong_From_int(__pyx_v_13cupy_backends_4cuda_3api_7runtime_errorMemoryAllocation); if (unlikely(!__pyx_t_19)) __PYX_ERR(0, 1887, __pyx_L36_error)
                __Pyx_GOTREF(__pyx_t_19);
                __pyx_t_20 = PyObject_RichCompare(__pyx_t_18, __pyx_t_19, Py_NE); __Pyx_XGOTREF(__pyx_t_20); if (unlikely(!__pyx_t_20)) __PYX_ERR(0, 1887, __pyx_L36_error)
                __Pyx_DECREF(__pyx_t_18); __pyx_t_18 = 0;
                __Pyx_DECREF(__pyx_t_19); __pyx_t_19 = 0;
                __pyx_t_7 = __Pyx_PyObject_IsTrue(__pyx_t_20); if (unlikely((__pyx_t_7 < 0))) __PYX_ERR(0, 1887, __pyx_L36_error)
                __Pyx_DECREF(__pyx_t_20); __pyx_t_20 = 0;
                if (unlikely(__pyx_t_7)) {

                  /* "cupy/cuda/memory.pyx":1888
 *             except CUDARuntimeError as e:
 *                 if e.status != runtime.errorMemoryAllocation:
 *                     raise             # <<<<<<<<<<<<<<
 *                 stream.synchronize()
 *                 try:
*/
                  __Pyx_GIVEREF(__pyx_t_3);
                  __Pyx_GIVEREF(__pyx_t_5);
                  __Pyx_XGIVEREF(__pyx_t_2);
                  __Pyx_ErrRestoreWithState(__pyx_t_3, __pyx_t_5, __pyx_t_2);
                  __pyx_t_3 = 0;  __pyx_t_5 = 0;  __pyx_t_2 = 0; 
                  __PYX_ERR(0, 1888, __pyx_L36_error)

                  /* "cupy/cuda/memory.pyx":1887
 *                 mem = malloc_async(rounded_size)
 *             except CUDARuntimeError as e:
 *                 if e.status != runtime.errorMemoryAllocation:             # <<<<<<<<<<<<<<
 *                     raise
 *                 stream.synchronize()
*/
                }

                /* "cupy/cuda/memory.pyx":1889
 *                 if e.status != runtime.errorMemoryAllocation:
 *                     raise
 *                 stream.synchronize()             # <<<<<<<<<<<<<<
 *                 try:
 *                     mem = malloc_async(rounded_size)
*/
                __pyx_t_19 = __pyx_v_stream;
                __Pyx_INCREF(__pyx_t_19);
                __pyx_t_9 = 0;
                {
                  PyObject *__pyx_callargs[2] = {__pyx_t_19, NULL};
                  __pyx_t_20 = __Pyx_PyObject_FastCallMethod((PyObject*)__pyx_mstate_global->__pyx_n_u_synchronize, __pyx_callargs+__pyx_t_9, (1-__pyx_t_9) | (1*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
                  __Pyx_XDECREF(__pyx_t_19); __pyx_t_19 = 0;
                  if (unlikely(!__pyx_t_20)) __PYX_ERR(0, 1889, __pyx_L36_error)
                  __Pyx_GOTREF(__pyx_t_20);
                }
                __Pyx_DECREF(__pyx_t_20); __pyx_t_20 = 0;

                /* "cupy/cuda/memory.pyx":1890
 *                     raise
 *                 stream.synchronize()
 *                 try:             # <<<<<<<<<<<<<<
 *                     mem = malloc_async(rounded_size)
 *                 except CUDARuntimeError as e:
*/
                {
                  __Pyx_PyThreadState_declare
                  __Pyx_PyThreadState_assign
                  __Pyx_ExceptionSave(&__pyx_t_21, &__pyx_t_22, &__pyx_t_23);
                  __Pyx_XGOTREF(__pyx_t_21);
                  __Pyx_XGOTREF(__pyx_t_22);
                  __Pyx_XGOTREF(__pyx_t_23);
                  /*try:*/ {

                    /* "cupy/cuda/memory.pyx":1891
 *                 stream.synchronize()
 *                 try:
 *                     mem = malloc_async(rounded_size)             # <<<<<<<<<<<<<<
 *                 except CUDARuntimeError as e:
 *                     if e.status != runtime.errorMemoryAllocation:
*/
                    __pyx_t_20 = ((PyObject *)__pyx_f_4cupy_4cuda_6memory_malloc_async(__pyx_v_rounded_size, 0)); if (unlikely(!__pyx_t_20)) __PYX_ERR(0, 1891, __pyx_L39_error)
                    __Pyx_GOTREF(__pyx_t_20);
                    __Pyx_DECREF_SET(__pyx_v_mem, ((struct __pyx_obj_4cupy_4cuda_6memory_MemoryPointer *)__pyx_t_20));
                    __pyx_t_20 = 0;

                    /* "cupy/cuda/memory.pyx":1890
 *                     raise
 *                 stream.synchronize()
 *                 try:             # <<<<<<<<<<<<<<
 *                     mem = malloc_async(rounded_size)
 *                 except CUDARuntimeError as e:
*/
                  }
                  __Pyx_XDECREF(__pyx_t_21); __pyx_t_21 = 0;
                  __Pyx_XDECREF(__pyx_t_22); __pyx_t_22 = 0;
                  __Pyx_XDECREF(__pyx_t_23); __pyx_t_23 = 0;
                  goto __pyx_L46_try_end;
                  __pyx_L39_error:;
                  __Pyx_XDECREF(__pyx_t_18); __pyx_t_18 = 0;
                  __Pyx_XDECREF(__pyx_t_19); __pyx_t_19 = 0;
                  __Pyx_XDECREF(__pyx_t_20); __pyx_t_20 = 0;

                  /* "cupy/cuda/memory.pyx":1892
 *                 try:
 *                     mem = malloc_async(rounded_size)
 *                 except CUDARuntimeError as e:             # <<<<<<<<<<<<<<
 *                     if e.status != runtime.errorMemoryAllocation:
 *                         raise
*/
                  __Pyx_ErrFetch(&__pyx_t_20, &__pyx_t_19, &__pyx_t_18);
                  __Pyx_GetModuleGlobalName(__pyx_t_24, __pyx_mstate_global->__pyx_n_u_CUDARuntimeError); if (unlikely(!__pyx_t_24)) __PYX_ERR(0, 1892, __pyx_L41_except_error)
                  __Pyx_GOTREF(__pyx_t_24);
                  __pyx_t_14 = __Pyx_PyErr_GivenExceptionMatches(__pyx_t_20, __pyx_t_24);
                  __Pyx_DECREF(__pyx_t_24); __pyx_t_24 = 0;
                  __Pyx_ErrRestore(__pyx_t_20, __pyx_t_19, __pyx_t_18);
                  __pyx_t_20 = 0; __pyx_t_19 = 0; __pyx_t_18 = 0;
                  if (__pyx_t_14) {
                    __Pyx_AddTraceback("cupy.cuda.memory.MemoryAsyncPool.malloc", __pyx_clineno, __pyx_lineno, __pyx_filename);
                    if (__Pyx_GetException(&__pyx_t_18, &__pyx_t_19, &__pyx_t_20) < 0) __PYX_ERR(0, 1892, __pyx_L41_except_error)
                    __Pyx_XGOTREF(__pyx_t_18);
                    __Pyx_XGOTREF(__pyx_t_19);
                    __Pyx_XGOTREF(__pyx_t_20);
                    __Pyx_INCREF(__pyx_t_19);
                    __Pyx_DECREF_SET(__pyx_v_e, __pyx_t_19);
                    /*try:*/ {

                      /* "cupy/cuda/memory.pyx":1893
 *                     mem = malloc_async(rounded_size)
 *                 except CUDARuntimeError as e:
 *                     if e.status != runtime.errorMemoryAllocation:             # <<<<<<<<<<<<<<
 *                         raise
 *                     oom_error = True
*/
                      __pyx_t_24 = __Pyx_PyObject_GetAttrStr(__pyx_v_e, __pyx_mstate_global->__pyx_n_u_status); if (unlikely(!__pyx_t_24)) __PYX_ERR(0, 1893, __pyx_L52_error)
                      __Pyx_GOTREF(__pyx_t_24);
                      __pyx_t_25 = __Pyx_PyLong_From_int(__pyx_v_13cupy_backends_4cuda_3api_7runtime_errorMemoryAllocation); if (unlikely(!__pyx_t_25)) __PYX_ERR(0, 1893, __pyx_L52_error)
                      __Pyx_GOTREF(__pyx_t_25);
                      __pyx_t_26 = PyObject_RichCompare(__pyx_t_24, __pyx_t_25, Py_NE); __Pyx_XGOTREF(__pyx_t_26); if (unlikely(!__pyx_t_26)) __PYX_ERR(0, 1893, __pyx_L52_error)
                      __Pyx_DECREF(__pyx_t_24); __pyx_t_24 = 0;
                      __Pyx_DECREF(__pyx_t_25); __pyx_t_25 = 0;
                      __pyx_t_7 = __Pyx_PyObject_IsTrue(__pyx_t_26); if (unlikely((__pyx_t_7 < 0))) __PYX_ERR(0, 1893, __pyx_L52_error)
                      __Pyx_DECREF(__pyx_t_26); __pyx_t_26 = 0;
                      if (unlikely(__pyx_t_7)) {

                        /* "cupy/cuda/memory.pyx":1894
 *                 except CUDARuntimeError as e:
 *                     if e.status != runtime.errorMemoryAllocation:
 *                         raise             # <<<<<<<<<<<<<<
 *                     oom_error = True
 *         finally:
*/
                        __Pyx_GIVEREF(__pyx_t_18);
                        __Pyx_GIVEREF(__pyx_t_19);
                        __Pyx_XGIVEREF(__pyx_t_20);
                        __Pyx_ErrRestoreWithState(__pyx_t_18, __pyx_t_19, __pyx_t_20);
                        __pyx_t_18 = 0;  __pyx_t_19 = 0;  __pyx_t_20 = 0; 
                        __PYX_ERR(0, 1894, __pyx_L52_error)

                        /* "cupy/cuda/memory.pyx":1893
 *                     mem = malloc_async(rounded_size)
 *                 except CUDARuntimeError as e:
 *                     if e.status != runtime.errorMemoryAllocation:             # <<<<<<<<<<<<<<
 *                         raise
 *                     oom_error = True
*/
                      }

                      /* "cupy/cuda/memory.pyx":1895
 *                     if e.status != runtime.errorMemoryAllocation:
 *                         raise
 *                     oom_error = True             # <<<<<<<<<<<<<<
 *         finally:
 *             if mem is None:
*/
                      __pyx_v_oom_error = 1;
                    }

                    /* "cupy/cuda/memory.pyx":1892
 *                 try:
 *                     mem = malloc_async(rounded_size)
 *                 except CUDARuntimeError as e:             # <<<<<<<<<<<<<<
 *                     if e.status != runtime.errorMemoryAllocation:
 *                         raise
*/
                    /*finally:*/ {
                      /*normal exit:*/{
                        __Pyx_DECREF(__pyx_v_e); __pyx_v_e = 0;
                        goto __pyx_L53;
                      }
                      __pyx_L52_error:;
                      /*exception exit:*/{
                        __Pyx_PyThreadState_declare
                        __Pyx_PyThreadState_assign
                        __pyx_t_29 = 0; __pyx_t_30 = 0; __pyx_t_31 = 0; __pyx_t_32 = 0; __pyx_t_33 = 0; __pyx_t_34 = 0;
                        __Pyx_XDECREF(__pyx_t_24); __pyx_t_24 = 0;
                        __Pyx_XDECREF(__pyx_t_25); __pyx_t_25 = 0;
                        __Pyx_XDECREF(__pyx_t_26); __pyx_t_26 = 0;
                         __Pyx_ExceptionSwap(&__pyx_t_32, &__pyx_t_33, &__pyx_t_34);
                        if ( unlikely(__Pyx_GetException(&__pyx_t_29, &__pyx_t_30, &__pyx_t_31) < 0)) __Pyx_ErrFetch(&__pyx_t_29, &__pyx_t_30, &__pyx_t_31);
                        __Pyx_XGOTREF(__pyx_t_29);
                        __Pyx_XGOTREF(__pyx_t_30);
                        __Pyx_XGOTREF(__pyx_t_31);
                        __Pyx_XGOTREF(__pyx_t_32);
                        __Pyx_XGOTREF(__pyx_t_33);
                        __Pyx_XGOTREF(__pyx_t_34);
                        __pyx_t_14 = __pyx_lineno; __pyx_t_27 = __pyx_clineno; __pyx_t_28 = __pyx_filename;
                        {
                          __Pyx_DECREF(__pyx_v_e); __pyx_v_e = 0;
                        }
                        __Pyx_XGIVEREF(__pyx_t_32);
                        __Pyx_XGIVEREF(__pyx_t_33);
                        __Pyx_XGIVEREF(__pyx_t_34);
                        __Pyx_ExceptionReset(__pyx_t_32, __pyx_t_33, __pyx_t_34);
                        __Pyx_XGIVEREF(__pyx_t_29);
                        __Pyx_XGIVEREF(__pyx_t_30);
                        __Pyx_XGIVEREF(__pyx_t_31);
                        __Pyx_ErrRestore(__pyx_t_29, __pyx_t_30, __pyx_t_31);
                        __pyx_t_29 = 0; __pyx_t_30 = 0; __pyx_t_31 = 0; __pyx_t_32 = 0; __pyx_t_33 = 0; __pyx_t_34 = 0;
                        __pyx_lineno = __pyx_t_14; __pyx_clineno = __pyx_t_27; __pyx_filename = __pyx_t_28;
                        goto __pyx_L41_except_error;
                      }
                      __pyx_L53:;
                    }
                    __Pyx_XDECREF(__pyx_t_18); __pyx_t_18 = 0;
                    __Pyx_XDECREF(__pyx_t_19); __pyx_t_19 = 0;
                    __Pyx_XDECREF(__pyx_t_20); __pyx_t_20 = 0;
                    goto __pyx_L40_exception_handled;
                  }
                  goto __pyx_L41_except_error;

                  /* "cupy/cuda/memory.pyx":1890
 *                     raise
 *                 stream.synchronize()
 *                 try:             # <<<<<<<<<<<<<<
 *                     mem = malloc_async(rounded_size)
 *                 except CUDARuntimeError as e:
*/
                  __pyx_L41_except_error:;
                  __Pyx_XGIVEREF(__pyx_t_21);
                  __Pyx_XGIVEREF(__pyx_t_22);
                  __Pyx_XGIVEREF(__pyx_t_23);
                  __Pyx_ExceptionReset(__pyx_t_21, __pyx_t_22, __pyx_t_23);
                  goto __pyx_L36_error;
                  __pyx_L40_exception_handled:;
                  __Pyx_XGIVEREF(__pyx_t_21);
                  __Pyx_XGIVEREF(__pyx_t_22);
                  __Pyx_XGIVEREF(__pyx_t_23);
                  __Pyx_ExceptionReset(__pyx_t_21, __pyx_t_22, __pyx_t_23);
                  __pyx_L46_try_end:;
                }
              }

              /* "cupy/cuda/memory.pyx":1886
 *             try:
 *                 mem = malloc_async(rounded_size)
 *             except CUDARuntimeError as e:             # <<<<<<<<<<<<<<
 *                 if e.status != runtime.errorMemoryAllocation:
 *                     raise
*/
              /*finally:*/ {
                /*normal exit:*/{
                  __Pyx_XDECREF(__pyx_v_e); __pyx_v_e = 0;
                  goto __pyx_L37;
                }
                __pyx_L36_error:;
                /*exception exit:*/{
                  __Pyx_PyThreadState_declare
                  __Pyx_PyThreadState_assign
                  __pyx_t_23 = 0; __pyx_t_22 = 0; __pyx_t_21 = 0; __pyx_t_34 = 0; __pyx_t_33 = 0; __pyx_t_32 = 0;
                  __Pyx_XDECREF(__pyx_t_18); __pyx_t_18 = 0;
                  __Pyx_XDECREF(__pyx_t_19); __pyx_t_19 = 0;
                  __Pyx_XDECREF(__pyx_t_20); __pyx_t_20 = 0;
                  __Pyx_XDECREF(__pyx_t_24); __pyx_t_24 = 0;
                  __Pyx_XDECREF(__pyx_t_25); __pyx_t_25 = 0;
                  __Pyx_XDECREF(__pyx_t_26); __pyx_t_26 = 0;
                   __Pyx_ExceptionSwap(&__pyx_t_34, &__pyx_t_33, &__pyx_t_32);
                  if ( unlikely(__Pyx_GetException(&__pyx_t_23, &__pyx_t_22, &__pyx_t_21) < 0)) __Pyx_ErrFetch(&__pyx_t_23, &__pyx_t_22, &__pyx_t_21);
                  __Pyx_XGOTREF(__pyx_t_23);
                  __Pyx_XGOTREF(__pyx_t_22);
                  __Pyx_XGOTREF(__pyx_t_21);
                  __Pyx_XGOTREF(__pyx_t_34);
                  __Pyx_XGOTREF(__pyx_t_33);
                  __Pyx_XGOTREF(__pyx_t_32);
                  __pyx_t_27 = __pyx_lineno; __pyx_t_14 = __pyx_clineno; __pyx_t_35 = __pyx_filename;
                  {
                    __Pyx_XDECREF(__pyx_v_e); __pyx_v_e = 0;
                  }
                  __Pyx_XGIVEREF(__pyx_t_34);
                  __Pyx_XGIVEREF(__pyx_t_33);
                  __Pyx_XGIVEREF(__pyx_t_32);
                  __Pyx_ExceptionReset(__pyx_t_34, __pyx_t_33, __pyx_t_32);
                  __Pyx_XGIVEREF(__pyx_t_23);
                  __Pyx_XGIVEREF(__pyx_t_22);
                  __Pyx_XGIVEREF(__pyx_t_21);
                  __Pyx_ErrRestore(__pyx_t_23, __pyx_t_22, __pyx_t_21);
                  __pyx_t_23 = 0; __pyx_t_22 = 0; __pyx_t_21 = 0; __pyx_t_34 = 0; __pyx_t_33 = 0; __pyx_t_32 = 0;
                  __pyx_lineno = __pyx_t_27; __pyx_clineno = __pyx_t_14; __pyx_filename = __pyx_t_35;
                  goto __pyx_L25_except_error;
                }
                __pyx_L37:;
              }
              __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
              __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
              __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
              goto __pyx_L24_exception_handled;
            }
            goto __pyx_L25_except_error;

            /* "cupy/cuda/memory.pyx":1884
 *             stream = stream_module.get_current_stream()
 *             stream.synchronize()
 *             try:             # <<<<<<<<<<<<<<
 *                 mem = malloc_async(rounded_size)
 *             except CUDARuntimeError as e:
*/
            __pyx_L25_except_error:;
            __Pyx_XGIVEREF(__pyx_t_15);
            __Pyx_XGIVEREF(__pyx_t_16);
            __Pyx_XGIVEREF(__pyx_t_17);
            __Pyx_ExceptionReset(__pyx_t_15, __pyx_t_16, __pyx_t_17);
            goto __pyx_L20_error;
            __pyx_L24_exception_handled:;
            __Pyx_XGIVEREF(__pyx_t_15);
            __Pyx_XGIVEREF(__pyx_t_16);
            __Pyx_XGIVEREF(__pyx_t_17);
            __Pyx_ExceptionReset(__pyx_t_15, __pyx_t_16, __pyx_t_17);
            __pyx_L30_try_end:;
          }
        }

        /* "cupy/cuda/memory.pyx":1879
 *         try:
 *             mem = malloc_async(rounded_size)
 *         except CUDARuntimeError as e:             # <<<<<<<<<<<<<<
 *             if e.status != runtime.errorMemoryAllocation:
 *                 raise
*/
        /*finally:*/ {
          /*normal exit:*/{
            __Pyx_XDECREF(__pyx_v_e); __pyx_v_e = 0;
            goto __pyx_L21;
          }
          __pyx_L20_error:;
          /*exception exit:*/{
            __Pyx_PyThreadState_declare
            __Pyx_PyThreadState_assign
            __pyx_t_17 = 0; __pyx_t_16 = 0; __pyx_t_15 = 0; __pyx_t_32 = 0; __pyx_t_33 = 0; __pyx_t_34 = 0;
            __Pyx_XDECREF(__pyx_t_18); __pyx_t_18 = 0;
            __Pyx_XDECREF(__pyx_t_19); __pyx_t_19 = 0;
            __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
            __Pyx_XDECREF(__pyx_t_20); __pyx_t_20 = 0;
            __Pyx_XDECREF(__pyx_t_24); __pyx_t_24 = 0;
            __Pyx_XDECREF(__pyx_t_25); __pyx_t_25 = 0;
            __Pyx_XDECREF(__pyx_t_26); __pyx_t_26 = 0;
            __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
            __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
             __Pyx_ExceptionSwap(&__pyx_t_32, &__pyx_t_33, &__pyx_t_34);
            if ( unlikely(__Pyx_GetException(&__pyx_t_17, &__pyx_t_16, &__pyx_t_15) < 0)) __Pyx_ErrFetch(&__pyx_t_17, &__pyx_t_16, &__pyx_t_15);
            __Pyx_XGOTREF(__pyx_t_17);
            __Pyx_XGOTREF(__pyx_t_16);
            __Pyx_XGOTREF(__pyx_t_15);
            __Pyx_XGOTREF(__pyx_t_32);
            __Pyx_XGOTREF(__pyx_t_33);
            __Pyx_XGOTREF(__pyx_t_34);
            __pyx_t_14 = __pyx_lineno; __pyx_t_27 = __pyx_clineno; __pyx_t_36 = __pyx_filename;
            {
              __Pyx_XDECREF(__pyx_v_e); __pyx_v_e = 0;
            }
            __Pyx_XGIVEREF(__pyx_t_32);
            __Pyx_XGIVEREF(__pyx_t_33);
            __Pyx_XGIVEREF(__pyx_t_34);
            __Pyx_ExceptionReset(__pyx_t_32, __pyx_t_33, __pyx_t_34);
            __Pyx_XGIVEREF(__pyx_t_17);
            __Pyx_XGIVEREF(__pyx_t_16);
            __Pyx_XGIVEREF(__pyx_t_15);
            __Pyx_ErrRestore(__pyx_t_17, __pyx_t_16, __pyx_t_15);
            __pyx_t_17 = 0; __pyx_t_16 = 0; __pyx_t_15 = 0; __pyx_t_32 = 0; __pyx_t_33 = 0; __pyx_t_34 = 0;
            __pyx_lineno = __pyx_t_14; __pyx_clineno = __pyx_t_27; __pyx_filename = __pyx_t_36;
            goto __pyx_L11_except_error;
          }
          __pyx_L21:;
        }
        __Pyx_XDECREF(__pyx_t_10); __pyx_t_10 = 0;
        __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
        __Pyx_XDECREF(__pyx_t_1); __pyx_t_1 = 0;
        goto __pyx_L10_exception_handled;
      }
      goto __pyx_L11_except_error;

      /* "cupy/cuda/memory.pyx":1877
 *                     raise OutOfMemoryError(size, curr_total, total_limit)
 * 
 *         try:             # <<<<<<<<<<<<<<
 *             mem = malloc_async(rounded_size)
 *         except CUDARuntimeError as e:
*/
      __pyx_L11_except_error:;
      __Pyx_XGIVEREF(__pyx_t_11);
      __Pyx_XGIVEREF(__pyx_t_12);
      __Pyx_XGIVEREF(__pyx_t_13);
      __Pyx_ExceptionReset(__pyx_t_11, __pyx_t_12, __pyx_t_13);
      goto __pyx_L7_error;
      __pyx_L10_exception_handled:;
      __Pyx_XGIVEREF(__pyx_t_11);
      __Pyx_XGIVEREF(__pyx_t_12);
      __Pyx_XGIVEREF(__pyx_t_13);
      __Pyx_ExceptionReset(__pyx_t_11, __pyx_t_12, __pyx_t_13);
      __pyx_L14_try_end:;
    }
  }

  /* "cupy/cuda/memory.pyx":1897
 *                     oom_error = True
 *         finally:
 *             if mem is None:             # <<<<<<<<<<<<<<
 *                 assert oom_error
 *                 # Set total to -1 as we do not have access to the mempool usage
*/
  /*finally:*/ {
    /*normal exit:*/{
      __pyx_t_7 = (((PyObject *)__pyx_v_mem) == Py_None);
      if (unlikely(__pyx_t_7)) {

        /* "cupy/cuda/memory.pyx":1898
 *         finally:
 *             if mem is None:
 *                 assert oom_error             # <<<<<<<<<<<<<<
 *                 # Set total to -1 as we do not have access to the mempool usage
 *                 raise OutOfMemoryError(size, curr_total, total_limit)
*/
        #ifndef CYTHON_WITHOUT_ASSERTIONS
        if (unlikely(__pyx_assertions_enabled())) {
          if (unlikely(!__pyx_v_oom_error)) {
            __Pyx_Raise(((PyObject *)(((PyTypeObject*)PyExc_AssertionError))), 0, 0, 0);
            __PYX_ERR(0, 1898, __pyx_L1_error)
          }
        }
        #else
        if ((1)); else __PYX_ERR(0, 1898, __pyx_L1_error)
        #endif

        /* "cupy/cuda/memory.pyx":1900
 *                 assert oom_error
 *                 # Set total to -1 as we do not have access to the mempool usage
 *                 raise OutOfMemoryError(size, curr_total, total_limit)             # <<<<<<<<<<<<<<
 *         return mem
 * 
*/
        __pyx_t_4 = NULL;
        __Pyx_GetModuleGlobalName(__pyx_t_10, __pyx_mstate_global->__pyx_n_u_OutOfMemoryError); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 1900, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_10);
        __pyx_t_2 = __Pyx_PyLong_FromSize_t(__pyx_v_size); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1900, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_2);
        __pyx_t_5 = __Pyx_PyLong_FromSize_t(__pyx_v_curr_total); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 1900, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_5);
        __pyx_t_3 = __Pyx_PyLong_FromSize_t(__pyx_v_total_limit); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 1900, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_3);
        __pyx_t_9 = 1;
        #if CYTHON_UNPACK_METHODS
        if (unlikely(PyMethod_Check(__pyx_t_10))) {
          __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_10);
          assert(__pyx_t_4);
          PyObject* __pyx__function = PyMethod_GET_FUNCTION(__pyx_t_10);
          __Pyx_INCREF(__pyx_t_4);
          __Pyx_INCREF(__pyx__function);
          __Pyx_DECREF_SET(__pyx_t_10, __pyx__function);
          __pyx_t_9 = 0;
        }
        #endif
        {
          PyObject *__pyx_callargs[4] = {__pyx_t_4, __pyx_t_2, __pyx_t_5, __pyx_t_3};
          __pyx_t_1 = __Pyx_PyObject_FastCall((PyObject*)__pyx_t_10, __pyx_callargs+__pyx_t_9, (4-__pyx_t_9) | (__pyx_t_9*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
          __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
          __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
          __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
          __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
          __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
          if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1900, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_1);
        }
        __Pyx_Raise(__pyx_t_1, 0, 0, 0);
        __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
        __PYX_ERR(0, 1900, __pyx_L1_error)

        /* "cupy/cuda/memory.pyx":1897
 *                     oom_error = True
 *         finally:
 *             if mem is None:             # <<<<<<<<<<<<<<
 *                 assert oom_error
 *                 # Set total to -1 as we do not have access to the mempool usage
*/
      }
      goto __pyx_L8;
    }
    __pyx_L7_error:;
    /*exception exit:*/{
      __Pyx_PyThreadState_declare
      __Pyx_PyThreadState_assign
      __pyx_t_13 = 0; __pyx_t_12 = 0; __pyx_t_11 = 0; __pyx_t_34 = 0; __pyx_t_33 = 0; __pyx_t_32 = 0;
      __Pyx_XDECREF(__pyx_t_1); __pyx_t_1 = 0;
      __Pyx_XDECREF(__pyx_t_10); __pyx_t_10 = 0;
      __Pyx_XDECREF(__pyx_t_18); __pyx_t_18 = 0;
      __Pyx_XDECREF(__pyx_t_19); __pyx_t_19 = 0;
      __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
      __Pyx_XDECREF(__pyx_t_20); __pyx_t_20 = 0;
      __Pyx_XDECREF(__pyx_t_24); __pyx_t_24 = 0;
      __Pyx_XDECREF(__pyx_t_25); __pyx_t_25 = 0;
      __Pyx_XDECREF(__pyx_t_26); __pyx_t_26 = 0;
      __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
      __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
      __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
       __Pyx_ExceptionSwap(&__pyx_t_34, &__pyx_t_33, &__pyx_t_32);
      if ( unlikely(__Pyx_GetException(&__pyx_t_13, &__pyx_t_12, &__pyx_t_11) < 0)) __Pyx_ErrFetch(&__pyx_t_13, &__pyx_t_12, &__pyx_t_11);
      __Pyx_XGOTREF(__pyx_t_13);
      __Pyx_XGOTREF(__pyx_t_12);
      __Pyx_XGOTREF(__pyx_t_11);
      __Pyx_XGOTREF(__pyx_t_34);
      __Pyx_XGOTREF(__pyx_t_33);
      __Pyx_XGOTREF(__pyx_t_32);
      __pyx_t_27 = __pyx_lineno; __pyx_t_14 = __pyx_clineno; __pyx_t_37 = __pyx_filename;
      {
        __pyx_t_7 = (((PyObject *)__pyx_v_mem) == Py_None);
        if (unlikely(__pyx_t_7)) {

          /* "cupy/cuda/memory.pyx":1898
 *         finally:
 *             if mem is None:
 *                 assert oom_error             # <<<<<<<<<<<<<<
 *                 # Set total to -1 as we do not have access to the mempool usage
 *                 raise OutOfMemoryError(size, curr_total, total_limit)
*/
          #ifndef CYTHON_WITHOUT_ASSERTIONS
          if (unlikely(__pyx_assertions_enabled())) {
            if (unlikely(!__pyx_v_oom_error)) {
              __Pyx_Raise(((PyObject *)(((PyTypeObject*)PyExc_AssertionError))), 0, 0, 0);
              __PYX_ERR(0, 1898, __pyx_L69_error)
            }
          }
          #else
          if ((1)); else __PYX_ERR(0, 1898, __pyx_L69_error)
          #endif

          /* "cupy/cuda/memory.pyx":1900
 *                 assert oom_error
 *                 # Set total to -1 as we do not have access to the mempool usage
 *                 raise OutOfMemoryError(size, curr_total, total_limit)             # <<<<<<<<<<<<<<
 *         return mem
 * 
*/
          __pyx_t_10 = NULL;
          __Pyx_GetModuleGlobalName(__pyx_t_3, __pyx_mstate_global->__pyx_n_u_OutOfMemoryError); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 1900, __pyx_L69_error)
          __Pyx_GOTREF(__pyx_t_3);
          __pyx_t_5 = __Pyx_PyLong_FromSize_t(__pyx_v_size); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 1900, __pyx_L69_error)
          __Pyx_GOTREF(__pyx_t_5);
          __pyx_t_2 = __Pyx_PyLong_FromSize_t(__pyx_v_curr_total); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1900, __pyx_L69_error)
          __Pyx_GOTREF(__pyx_t_2);
          __pyx_t_4 = __Pyx_PyLong_FromSize_t(__pyx_v_total_limit); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 1900, __pyx_L69_error)
          __Pyx_GOTREF(__pyx_t_4);
          __pyx_t_9 = 1;
          #if CYTHON_UNPACK_METHODS
          if (unlikely(PyMethod_Check(__pyx_t_3))) {
            __pyx_t_10 = PyMethod_GET_SELF(__pyx_t_3);
            assert(__pyx_t_10);
            PyObject* __pyx__function = PyMethod_GET_FUNCTION(__pyx_t_3);
            __Pyx_INCREF(__pyx_t_10);
            __Pyx_INCREF(__pyx__function);
            __Pyx_DECREF_SET(__pyx_t_3, __pyx__function);
            __pyx_t_9 = 0;
          }
          #endif
          {
            PyObject *__pyx_callargs[4] = {__pyx_t_10, __pyx_t_5, __pyx_t_2, __pyx_t_4};
            __pyx_t_1 = __Pyx_PyObject_FastCall((PyObject*)__pyx_t_3, __pyx_callargs+__pyx_t_9, (4-__pyx_t_9) | (__pyx_t_9*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
            __Pyx_XDECREF(__pyx_t_10); __pyx_t_10 = 0;
            __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
            __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
            __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
            __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
            if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1900, __pyx_L69_error)
            __Pyx_GOTREF(__pyx_t_1);
          }
          __Pyx_Raise(__pyx_t_1, 0, 0, 0);
          __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
          __PYX_ERR(0, 1900, __pyx_L69_error)

          /* "cupy/cuda/memory.pyx":1897
 *                     oom_error = True
 *         finally:
 *             if mem is None:             # <<<<<<<<<<<<<<
 *                 assert oom_error
 *                 # Set total to -1 as we do not have access to the mempool usage
*/
        }
      }
      __Pyx_XGIVEREF(__pyx_t_34);
      __Pyx_XGIVEREF(__pyx_t_33);
      __Pyx_XGIVEREF(__pyx_t_32);
      __Pyx_ExceptionReset(__pyx_t_34, __pyx_t_33, __pyx_t_32);
      __Pyx_XGIVEREF(__pyx_t_13);
      __Pyx_XGIVEREF(__pyx_t_12);
      __Pyx_XGIVEREF(__pyx_t_11);
      __Pyx_ErrRestore(__pyx_t_13, __pyx_t_12, __pyx_t_11);
      __pyx_t_13 = 0; __pyx_t_12 = 0; __pyx_t_11 = 0; __pyx_t_34 = 0; __pyx_t_33 = 0; __pyx_t_32 = 0;
      __pyx_lineno = __pyx_t_27; __pyx_clineno = __pyx_t_14; __pyx_filename = __pyx_t_37;
      goto __pyx_L1_error;
      __pyx_L69_error:;
      __Pyx_XGIVEREF(__pyx_t_34);
      __Pyx_XGIVEREF(__pyx_t_33);
      __Pyx_XGIVEREF(__pyx_t_32);
      __Pyx_ExceptionReset(__pyx_t_34, __pyx_t_33, __pyx_t_32);
      __Pyx_XDECREF(__pyx_t_13); __pyx_t_13 = 0;
      __Pyx_XDECREF(__pyx_t_12); __pyx_t_12 = 0;
      __Pyx_XDECREF(__pyx_t_11); __pyx_t_11 = 0;
      __pyx_t_34 = 0; __pyx_t_33 = 0; __pyx_t_32 = 0;
      goto __pyx_L1_error;
    }
    __pyx_L8:;
  }

  /* "cupy/cuda/memory.pyx":1901
 *                 # Set total to -1 as we do not have access to the mempool usage
 *                 raise OutOfMemoryError(size, curr_total, total_limit)
 *         return mem             # <<<<<<<<<<<<<<
 * 
 *     cpdef free_all_blocks(self, stream=None):
*/
  __Pyx_XDECREF((PyObject *)__pyx_r);
  __Pyx_INCREF((PyObject *)__pyx_v_mem);
  __pyx_r = __pyx_v_mem;
  goto __pyx_L0;

  /* "cupy/cuda/memory.pyx":1845
 *         return pool
 * 
 *     cpdef MemoryPointer malloc(self, size_t size):             # <<<<<<<<<<<<<<
 *         """Allocate memory from the current device's pool on the current
 *         stream.
*/

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_XDECREF(__pyx_t_10);
  __Pyx_XDECREF(__pyx_t_18);
  __Pyx_XDECREF(__pyx_t_19);
  __Pyx_XDECREF(__pyx_t_20);
  __Pyx_XDECREF(__pyx_t_24);
  __Pyx_XDECREF(__pyx_t_25);
  __Pyx_XDECREF(__pyx_t_26);
  __Pyx_AddTraceback("cupy.cuda.memory.MemoryAsyncPool.malloc", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XDECREF((PyObject *)__pyx_v_mem);
  __Pyx_XDECREF(__pyx_v_e);
  __Pyx_XDECREF(__pyx_v_stream);
  __Pyx_XGIVEREF((PyObject *)__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_pw_4cupy_4cuda_6memory_15MemoryAsyncPool_3malloc(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
PyDoc_STRVAR(__pyx_doc_4cupy_4cuda_6memory_15MemoryAsyncPool_2malloc, "MemoryAsyncPool.malloc(self, size_t size) -> MemoryPointer\n\nAllocate memory from the current device's pool on the current\nstream.\n\nThis method can be used as a CuPy memory allocator. The simplest way to\nuse a memory pool as the default allocator is the following code::\n\n    set_allocator(MemoryAsyncPool().malloc)\n\nArgs:\n    size (int): Size of the memory buffer to allocate in bytes.\n\nReturns:\n    ~cupy.cuda.MemoryPointer: Pointer to the allocated buffer.");
static PyObject *__pyx_pw_4cupy_4cuda_6memory_15MemoryAsyncPool_3malloc(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
) {
  size_t __pyx_v_size;
  #if !CYTHON_METH_FASTCALL
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  #endif
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject* values[1] = {0};
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("malloc (wrapper)", 0);
  #if !CYTHON_METH_FASTCALL
  #if CYTHON_ASSUME_SAFE_SIZE
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return NULL;
  #endif
  #endif
  __pyx_kwvalues = __Pyx_KwValues_FASTCALL(__pyx_args, __pyx_nargs);
  {
    PyObject ** const __pyx_pyargnames[] = {&__pyx_mstate_global->__pyx_n_u_size,0};
    const Py_ssize_t __pyx_kwds_len = (__pyx_kwds) ? __Pyx_NumKwargs_FASTCALL(__pyx_kwds) : 0;
    if (unlikely(__pyx_kwds_len < 0)) __PYX_ERR(0, 1845, __pyx_L3_error)
    if (__pyx_kwds_len > 0) {
      switch (__pyx_nargs) {
        case  1:
        values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(0, 1845, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      const Py_ssize_t kwd_pos_args = __pyx_nargs;
      if (__Pyx_ParseKeywords(__pyx_kwds, __pyx_kwvalues, __pyx_pyargnames, 0, values, kwd_pos_args, __pyx_kwds_len, "malloc", 0) < (0)) __PYX_ERR(0, 1845, __pyx_L3_error)
      for (Py_ssize_t i = __pyx_nargs; i < 1; i++) {
        if (unlikely(!values[i])) { __Pyx_RaiseArgtupleInvalid("malloc", 1, 1, 1, i); __PYX_ERR(0, 1845, __pyx_L3_error) }
      }
    } else if (unlikely(__pyx_nargs != 1)) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
      if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(0, 1845, __pyx_L3_error)
    }
    __pyx_v_size = __Pyx_PyLong_As_size_t(values[0]); if (unlikely((__pyx_v_size == (size_t)-1) && PyErr_Occurred())) __PYX_ERR(0, 1845, __pyx_L3_error)
  }
  goto __pyx_L6_skip;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("malloc", 1, 1, 1, __pyx_nargs); __PYX_ERR(0, 1845, __pyx_L3_error)
  __pyx_L6_skip:;
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_AddTraceback("cupy.cuda.memory.MemoryAsyncPool.malloc", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_4cupy_4cuda_6memory_15MemoryAsyncPool_2malloc(((struct __pyx_obj_4cupy_4cuda_6memory_MemoryAsyncPool *)__pyx_v_self), __pyx_v_size);

  /* function exit code */
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4cupy_4cuda_6memory_15MemoryAsyncPool_2malloc(struct __pyx_obj_4cupy_4cuda_6memory_MemoryAsyncPool *__pyx_v_self, size_t __pyx_v_size) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("malloc", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = ((PyObject *)__pyx_f_4cupy_4cuda_6memory_15MemoryAsyncPool_malloc(__pyx_v_self, __pyx_v_size, 1)); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1845, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("cupy.cuda.memory.MemoryAsyncPool.malloc", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "cupy/cuda/memory.pyx":1903
 *         return mem
 * 
 *     cpdef free_all_blocks(self, stream=None):             # <<<<<<<<<<<<<<
 *         """Releases free memory.
 * 
*/

static PyObject *__pyx_pw_4cupy_4cuda_6memory_15MemoryAsyncPool_5free_all_blocks(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
static PyObject *__pyx_f_4cupy_4cuda_6memory_15MemoryAsyncPool_free_all_blocks(struct __pyx_obj_4cupy_4cuda_6memory_MemoryAsyncPool *__pyx_v_self, int __pyx_skip_dispatch, struct __pyx_opt_args_4cupy_4cuda_6memory_15MemoryAsyncPool_free_all_blocks *__pyx_optional_args) {
  PyObject *__pyx_v_stream = ((PyObject *)Py_None);
  intptr_t __pyx_v_pool;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  size_t __pyx_t_5;
  int __pyx_t_6;
  int __pyx_t_7;
  intptr_t __pyx_t_8;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("free_all_blocks", 0);
  if (__pyx_optional_args) {
    if (__pyx_optional_args->__pyx_n > 0) {
      __pyx_v_stream = __pyx_optional_args->stream;
    }
  }
  __Pyx_INCREF(__pyx_v_stream);
  /* Check if called by wrapper */
  if (unlikely(__pyx_skip_dispatch)) ;
  /* Check if overridden in Python */
  else if (
  #if !CYTHON_USE_TYPE_SLOTS
  unlikely(Py_TYPE(((PyObject *)__pyx_v_self)) != __pyx_mstate_global->__pyx_ptype_4cupy_4cuda_6memory_MemoryAsyncPool &&
  __Pyx_PyType_HasFeature(Py_TYPE(((PyObject *)__pyx_v_self)), Py_TPFLAGS_HAVE_GC))
  #else
  unlikely(Py_TYPE(((PyObject *)__pyx_v_self))->tp_dictoffset != 0 || __Pyx_PyType_HasFeature(Py_TYPE(((PyObject *)__pyx_v_self)), (Py_TPFLAGS_IS_ABSTRACT | Py_TPFLAGS_HEAPTYPE)))
  #endif
  ) {
    #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
    static PY_UINT64_T __pyx_tp_dict_version = __PYX_DICT_VERSION_INIT, __pyx_obj_dict_version = __PYX_DICT_VERSION_INIT;
    if (unlikely(!__Pyx_object_dict_version_matches(((PyObject *)__pyx_v_self), __pyx_tp_dict_version, __pyx_obj_dict_version))) {
      PY_UINT64_T __pyx_typedict_guard = __Pyx_get_tp_dict_version(((PyObject *)__pyx_v_self));
      #endif
      __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_mstate_global->__pyx_n_u_free_all_blocks); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1903, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      if (!__Pyx_IsSameCFunction(__pyx_t_1, (void(*)(void)) __pyx_pw_4cupy_4cuda_6memory_15MemoryAsyncPool_5free_all_blocks)) {
        __Pyx_XDECREF(__pyx_r);
        __pyx_t_3 = NULL;
        __Pyx_INCREF(__pyx_t_1);
        __pyx_t_4 = __pyx_t_1; 
        __pyx_t_5 = 1;
        #if CYTHON_UNPACK_METHODS
        if (unlikely(PyMethod_Check(__pyx_t_4))) {
          __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_4);
          assert(__pyx_t_3);
          PyObject* __pyx__function = PyMethod_GET_FUNCTION(__pyx_t_4);
          __Pyx_INCREF(__pyx_t_3);
          __Pyx_INCREF(__pyx__function);
          __Pyx_DECREF_SET(__pyx_t_4, __pyx__function);
          __pyx_t_5 = 0;
        }
        #endif
        {
          PyObject *__pyx_callargs[2] = {__pyx_t_3, __pyx_v_stream};
          __pyx_t_2 = __Pyx_PyObject_FastCall((PyObject*)__pyx_t_4, __pyx_callargs+__pyx_t_5, (2-__pyx_t_5) | (__pyx_t_5*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
          __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
          __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
          if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1903, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_2);
        }
        __pyx_r = __pyx_t_2;
        __pyx_t_2 = 0;
        __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
        goto __pyx_L0;
      }
      #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
      __pyx_tp_dict_version = __Pyx_get_tp_dict_version(((PyObject *)__pyx_v_self));
      __pyx_obj_dict_version = __Pyx_get_object_dict_version(((PyObject *)__pyx_v_self));
      if (unlikely(__pyx_typedict_guard != __pyx_tp_dict_version)) {
        __pyx_tp_dict_version = __pyx_obj_dict_version = __PYX_DICT_VERSION_INIT;
      }
      #endif
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
    }
    #endif
  }

  /* "cupy/cuda/memory.pyx":1920
 *         # they become visible (to both cudaMalloc and cudaMallocAsync). See
 *         # https://github.com/cupy/cupy/issues/3777#issuecomment-758890450
 *         if stream is None:             # <<<<<<<<<<<<<<
 *             stream = stream_module.get_current_stream()
 *         stream.synchronize()
*/
  __pyx_t_6 = (__pyx_v_stream == Py_None);
  if (__pyx_t_6) {

    /* "cupy/cuda/memory.pyx":1921
 *         # https://github.com/cupy/cupy/issues/3777#issuecomment-758890450
 *         if stream is None:
 *             stream = stream_module.get_current_stream()             # <<<<<<<<<<<<<<
 *         stream.synchronize()
 *         cdef intptr_t pool = self._pools[device.get_device_id()]
*/
    __pyx_t_1 = __pyx_f_4cupy_4cuda_6stream_get_current_stream(0, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1921, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF_SET(__pyx_v_stream, __pyx_t_1);
    __pyx_t_1 = 0;

    /* "cupy/cuda/memory.pyx":1920
 *         # they become visible (to both cudaMalloc and cudaMallocAsync). See
 *         # https://github.com/cupy/cupy/issues/3777#issuecomment-758890450
 *         if stream is None:             # <<<<<<<<<<<<<<
 *             stream = stream_module.get_current_stream()
 *         stream.synchronize()
*/
  }

  /* "cupy/cuda/memory.pyx":1922
 *         if stream is None:
 *             stream = stream_module.get_current_stream()
 *         stream.synchronize()             # <<<<<<<<<<<<<<
 *         cdef intptr_t pool = self._pools[device.get_device_id()]
 *         # We don't care the actual limit; putting 0 here means we guarantee
*/
  __pyx_t_2 = __pyx_v_stream;
  __Pyx_INCREF(__pyx_t_2);
  __pyx_t_5 = 0;
  {
    PyObject *__pyx_callargs[2] = {__pyx_t_2, NULL};
    __pyx_t_1 = __Pyx_PyObject_FastCallMethod((PyObject*)__pyx_mstate_global->__pyx_n_u_synchronize, __pyx_callargs+__pyx_t_5, (1-__pyx_t_5) | (1*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
    __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
    if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1922, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
  }
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "cupy/cuda/memory.pyx":1923
 *             stream = stream_module.get_current_stream()
 *         stream.synchronize()
 *         cdef intptr_t pool = self._pools[device.get_device_id()]             # <<<<<<<<<<<<<<
 *         # We don't care the actual limit; putting 0 here means we guarantee
 *         # to reserve at least 0 bytes
*/
  if (unlikely(__pyx_v_self->_pools == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(0, 1923, __pyx_L1_error)
  }
  __pyx_t_7 = __pyx_f_4cupy_4cuda_6device_get_device_id(0); if (unlikely(__pyx_t_7 == ((int)-1) && PyErr_Occurred())) __PYX_ERR(0, 1923, __pyx_L1_error)
  __pyx_t_1 = __Pyx_GetItemInt_List(__pyx_v_self->_pools, __pyx_t_7, int, 1, __Pyx_PyLong_From_int, 1, 1, 1, 1, __Pyx_ReferenceSharing_SharedReference); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1923, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_8 = PyLong_AsSsize_t(__pyx_t_1); if (unlikely((__pyx_t_8 == ((intptr_t)-1)) && PyErr_Occurred())) __PYX_ERR(0, 1923, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v_pool = __pyx_t_8;

  /* "cupy/cuda/memory.pyx":1926
 *         # We don't care the actual limit; putting 0 here means we guarantee
 *         # to reserve at least 0 bytes
 *         runtime.memPoolTrimTo(pool, 0)             # <<<<<<<<<<<<<<
 * 
 *     cpdef size_t n_free_blocks(self):
*/
  __pyx_t_1 = __pyx_f_13cupy_backends_4cuda_3api_7runtime_memPoolTrimTo(__pyx_v_pool, 0, 0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1926, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "cupy/cuda/memory.pyx":1903
 *         return mem
 * 
 *     cpdef free_all_blocks(self, stream=None):             # <<<<<<<<<<<<<<
 *         """Releases free memory.
 * 
*/

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_AddTraceback("cupy.cuda.memory.MemoryAsyncPool.free_all_blocks", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_stream);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_pw_4cupy_4cuda_6memory_15MemoryAsyncPool_5free_all_blocks(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
PyDoc_STRVAR(__pyx_doc_4cupy_4cuda_6memory_15MemoryAsyncPool_4free_all_blocks, "MemoryAsyncPool.free_all_blocks(self, stream=None)\n\nReleases free memory.\n\nArgs:\n    stream (cupy.cuda.Stream): Release memory freed on the given\n        ``stream``. If ``stream`` is ``None``, the current stream is\n        used.\n\n.. seealso:: `Physical Page Caching Behavior`_\n\n.. _Physical Page Caching Behavior:\n    https://docs.nvidia.com/cuda/cuda-c-programming-guide/index.html#stream-ordered-physical-page-caching-behavior");
static PyObject *__pyx_pw_4cupy_4cuda_6memory_15MemoryAsyncPool_5free_all_blocks(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
) {
  PyObject *__pyx_v_stream = 0;
  #if !CYTHON_METH_FASTCALL
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  #endif
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject* values[1] = {0};
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("free_all_blocks (wrapper)", 0);
  #if !CYTHON_METH_FASTCALL
  #if CYTHON_ASSUME_SAFE_SIZE
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return NULL;
  #endif
  #endif
  __pyx_kwvalues = __Pyx_KwValues_FASTCALL(__pyx_args, __pyx_nargs);
  {
    PyObject ** const __pyx_pyargnames[] = {&__pyx_mstate_global->__pyx_n_u_stream,0};
    const Py_ssize_t __pyx_kwds_len = (__pyx_kwds) ? __Pyx_NumKwargs_FASTCALL(__pyx_kwds) : 0;
    if (unlikely(__pyx_kwds_len < 0)) __PYX_ERR(0, 1903, __pyx_L3_error)
    if (__pyx_kwds_len > 0) {
      switch (__pyx_nargs) {
        case  1:
        values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(0, 1903, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      const Py_ssize_t kwd_pos_args = __pyx_nargs;
      if (__Pyx_ParseKeywords(__pyx_kwds, __pyx_kwvalues, __pyx_pyargnames, 0, values, kwd_pos_args, __pyx_kwds_len, "free_all_blocks", 0) < (0)) __PYX_ERR(0, 1903, __pyx_L3_error)
      if (!values[0]) values[0] = __Pyx_NewRef(((PyObject *)Py_None));
    } else {
      switch (__pyx_nargs) {
        case  1:
        values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(0, 1903, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      if (!values[0]) values[0] = __Pyx_NewRef(((PyObject *)Py_None));
    }
    __pyx_v_stream = values[0];
  }
  goto __pyx_L6_skip;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("free_all_blocks", 0, 0, 1, __pyx_nargs); __PYX_ERR(0, 1903, __pyx_L3_error)
  __pyx_L6_skip:;
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_AddTraceback("cupy.cuda.memory.MemoryAsyncPool.free_all_blocks", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_4cupy_4cuda_6memory_15MemoryAsyncPool_4free_all_blocks(((struct __pyx_obj_4cupy_4cuda_6memory_MemoryAsyncPool *)__pyx_v_self), __pyx_v_stream);

  /* function exit code */
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4cupy_4cuda_6memory_15MemoryAsyncPool_4free_all_blocks(struct __pyx_obj_4cupy_4cuda_6memory_MemoryAsyncPool *__pyx_v_self, PyObject *__pyx_v_stream) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  struct __pyx_opt_args_4cupy_4cuda_6memory_15MemoryAsyncPool_free_all_blocks __pyx_t_2;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("free_all_blocks", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_2.__pyx_n = 1;
  __pyx_t_2.stream = __pyx_v_stream;
  __pyx_t_1 = __pyx_vtabptr_4cupy_4cuda_6memory_MemoryAsyncPool->free_all_blocks(__pyx_v_self, 1, &__pyx_t_2); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1903, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("cupy.cuda.memory.MemoryAsyncPool.free_all_blocks", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "cupy/cuda/memory.pyx":1928
 *         runtime.memPoolTrimTo(pool, 0)
 * 
 *     cpdef size_t n_free_blocks(self):             # <<<<<<<<<<<<<<
 *         raise NotImplementedError(
 *             'This function is not supported in MemoryAsyncPool')
*/

static PyObject *__pyx_pw_4cupy_4cuda_6memory_15MemoryAsyncPool_7n_free_blocks(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
static size_t __pyx_f_4cupy_4cuda_6memory_15MemoryAsyncPool_n_free_blocks(CYTHON_UNUSED struct __pyx_obj_4cupy_4cuda_6memory_MemoryAsyncPool *__pyx_v_self, int __pyx_skip_dispatch) {
  size_t __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  size_t __pyx_t_5;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("n_free_blocks", 0);
  /* Check if called by wrapper */
  if (unlikely(__pyx_skip_dispatch)) ;
  /* Check if overridden in Python */
  else if (
  #if !CYTHON_USE_TYPE_SLOTS
  unlikely(Py_TYPE(((PyObject *)__pyx_v_self)) != __pyx_mstate_global->__pyx_ptype_4cupy_4cuda_6memory_MemoryAsyncPool &&
  __Pyx_PyType_HasFeature(Py_TYPE(((PyObject *)__pyx_v_self)), Py_TPFLAGS_HAVE_GC))
  #else
  unlikely(Py_TYPE(((PyObject *)__pyx_v_self))->tp_dictoffset != 0 || __Pyx_PyType_HasFeature(Py_TYPE(((PyObject *)__pyx_v_self)), (Py_TPFLAGS_IS_ABSTRACT | Py_TPFLAGS_HEAPTYPE)))
  #endif
  ) {
    #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
    static PY_UINT64_T __pyx_tp_dict_version = __PYX_DICT_VERSION_INIT, __pyx_obj_dict_version = __PYX_DICT_VERSION_INIT;
    if (unlikely(!__Pyx_object_dict_version_matches(((PyObject *)__pyx_v_self), __pyx_tp_dict_version, __pyx_obj_dict_version))) {
      PY_UINT64_T __pyx_typedict_guard = __Pyx_get_tp_dict_version(((PyObject *)__pyx_v_self));
      #endif
      __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_mstate_global->__pyx_n_u_n_free_blocks); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1928, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      if (!__Pyx_IsSameCFunction(__pyx_t_1, (void(*)(void)) __pyx_pw_4cupy_4cuda_6memory_15MemoryAsyncPool_7n_free_blocks)) {
        __pyx_t_3 = NULL;
        __Pyx_INCREF(__pyx_t_1);
        __pyx_t_4 = __pyx_t_1; 
        __pyx_t_5 = 1;
        #if CYTHON_UNPACK_METHODS
        if (unlikely(PyMethod_Check(__pyx_t_4))) {
          __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_4);
          assert(__pyx_t_3);
          PyObject* __pyx__function = PyMethod_GET_FUNCTION(__pyx_t_4);
          __Pyx_INCREF(__pyx_t_3);
          __Pyx_INCREF(__pyx__function);
          __Pyx_DECREF_SET(__pyx_t_4, __pyx__function);
          __pyx_t_5 = 0;
        }
        #endif
        {
          PyObject *__pyx_callargs[2] = {__pyx_t_3, NULL};
          __pyx_t_2 = __Pyx_PyObject_FastCall((PyObject*)__pyx_t_4, __pyx_callargs+__pyx_t_5, (1-__pyx_t_5) | (__pyx_t_5*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
          __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
          __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
          if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1928, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_2);
        }
        __pyx_t_5 = __Pyx_PyLong_As_size_t(__pyx_t_2); if (unlikely((__pyx_t_5 == (size_t)-1) && PyErr_Occurred())) __PYX_ERR(0, 1928, __pyx_L1_error)
        __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
        __pyx_r = __pyx_t_5;
        __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
        goto __pyx_L0;
      }
      #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
      __pyx_tp_dict_version = __Pyx_get_tp_dict_version(((PyObject *)__pyx_v_self));
      __pyx_obj_dict_version = __Pyx_get_object_dict_version(((PyObject *)__pyx_v_self));
      if (unlikely(__pyx_typedict_guard != __pyx_tp_dict_version)) {
        __pyx_tp_dict_version = __pyx_obj_dict_version = __PYX_DICT_VERSION_INIT;
      }
      #endif
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
    }
    #endif
  }

  /* "cupy/cuda/memory.pyx":1929
 * 
 *     cpdef size_t n_free_blocks(self):
 *         raise NotImplementedError(             # <<<<<<<<<<<<<<
 *             'This function is not supported in MemoryAsyncPool')
 * 
*/
  __pyx_t_2 = NULL;
  __pyx_t_5 = 1;
  {
    PyObject *__pyx_callargs[2] = {__pyx_t_2, __pyx_mstate_global->__pyx_kp_u_This_function_is_not_supported_i};
    __pyx_t_1 = __Pyx_PyObject_FastCall((PyObject*)(((PyTypeObject*)PyExc_NotImplementedError)), __pyx_callargs+__pyx_t_5, (2-__pyx_t_5) | (__pyx_t_5*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
    __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
    if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1929, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
  }
  __Pyx_Raise(__pyx_t_1, 0, 0, 0);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __PYX_ERR(0, 1929, __pyx_L1_error)

  /* "cupy/cuda/memory.pyx":1928
 *         runtime.memPoolTrimTo(pool, 0)
 * 
 *     cpdef size_t n_free_blocks(self):             # <<<<<<<<<<<<<<
 *         raise NotImplementedError(
 *             'This function is not supported in MemoryAsyncPool')
*/

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_AddTraceback("cupy.cuda.memory.MemoryAsyncPool.n_free_blocks", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_pw_4cupy_4cuda_6memory_15MemoryAsyncPool_7n_free_blocks(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
PyDoc_STRVAR(__pyx_doc_4cupy_4cuda_6memory_15MemoryAsyncPool_6n_free_blocks, "MemoryAsyncPool.n_free_blocks(self) -> size_t");
static PyObject *__pyx_pw_4cupy_4cuda_6memory_15MemoryAsyncPool_7n_free_blocks(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
) {
  #if !CYTHON_METH_FASTCALL
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  #endif
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("n_free_blocks (wrapper)", 0);
  #if !CYTHON_METH_FASTCALL
  #if CYTHON_ASSUME_SAFE_SIZE
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return NULL;
  #endif
  #endif
  __pyx_kwvalues = __Pyx_KwValues_FASTCALL(__pyx_args, __pyx_nargs);
  if (unlikely(__pyx_nargs > 0)) { __Pyx_RaiseArgtupleInvalid("n_free_blocks", 1, 0, 0, __pyx_nargs); return NULL; }
  const Py_ssize_t __pyx_kwds_len = unlikely(__pyx_kwds) ? __Pyx_NumKwargs_FASTCALL(__pyx_kwds) : 0;
  if (unlikely(__pyx_kwds_len < 0)) return NULL;
  if (unlikely(__pyx_kwds_len > 0)) {__Pyx_RejectKeywords("n_free_blocks", __pyx_kwds); return NULL;}
  __pyx_r = __pyx_pf_4cupy_4cuda_6memory_15MemoryAsyncPool_6n_free_blocks(((struct __pyx_obj_4cupy_4cuda_6memory_MemoryAsyncPool *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4cupy_4cuda_6memory_15MemoryAsyncPool_6n_free_blocks(struct __pyx_obj_4cupy_4cuda_6memory_MemoryAsyncPool *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  size_t __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("n_free_blocks", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __pyx_f_4cupy_4cuda_6memory_15MemoryAsyncPool_n_free_blocks(__pyx_v_self, 1); if (unlikely(PyErr_Occurred())) __PYX_ERR(0, 1928, __pyx_L1_error)
  __pyx_t_2 = __Pyx_PyLong_FromSize_t(__pyx_t_1); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1928, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_r = __pyx_t_2;
  __pyx_t_2 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_AddTraceback("cupy.cuda.memory.MemoryAsyncPool.n_free_blocks", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "cupy/cuda/memory.pyx":1932
 *             'This function is not supported in MemoryAsyncPool')
 * 
 *     cpdef size_t used_bytes(self):             # <<<<<<<<<<<<<<
 *         """Gets the total number of bytes used by the pool.
 * 
*/

static PyObject *__pyx_pw_4cupy_4cuda_6memory_15MemoryAsyncPool_9used_bytes(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
static size_t __pyx_f_4cupy_4cuda_6memory_15MemoryAsyncPool_used_bytes(struct __pyx_obj_4cupy_4cuda_6memory_MemoryAsyncPool *__pyx_v_self, int __pyx_skip_dispatch) {
  intptr_t __pyx_v_pool;
  size_t __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  size_t __pyx_t_5;
  int __pyx_t_6;
  int __pyx_t_7;
  intptr_t __pyx_t_8;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("used_bytes", 0);
  /* Check if called by wrapper */
  if (unlikely(__pyx_skip_dispatch)) ;
  /* Check if overridden in Python */
  else if (
  #if !CYTHON_USE_TYPE_SLOTS
  unlikely(Py_TYPE(((PyObject *)__pyx_v_self)) != __pyx_mstate_global->__pyx_ptype_4cupy_4cuda_6memory_MemoryAsyncPool &&
  __Pyx_PyType_HasFeature(Py_TYPE(((PyObject *)__pyx_v_self)), Py_TPFLAGS_HAVE_GC))
  #else
  unlikely(Py_TYPE(((PyObject *)__pyx_v_self))->tp_dictoffset != 0 || __Pyx_PyType_HasFeature(Py_TYPE(((PyObject *)__pyx_v_self)), (Py_TPFLAGS_IS_ABSTRACT | Py_TPFLAGS_HEAPTYPE)))
  #endif
  ) {
    #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
    static PY_UINT64_T __pyx_tp_dict_version = __PYX_DICT_VERSION_INIT, __pyx_obj_dict_version = __PYX_DICT_VERSION_INIT;
    if (unlikely(!__Pyx_object_dict_version_matches(((PyObject *)__pyx_v_self), __pyx_tp_dict_version, __pyx_obj_dict_version))) {
      PY_UINT64_T __pyx_typedict_guard = __Pyx_get_tp_dict_version(((PyObject *)__pyx_v_self));
      #endif
      __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_mstate_global->__pyx_n_u_used_bytes); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1932, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      if (!__Pyx_IsSameCFunction(__pyx_t_1, (void(*)(void)) __pyx_pw_4cupy_4cuda_6memory_15MemoryAsyncPool_9used_bytes)) {
        __pyx_t_3 = NULL;
        __Pyx_INCREF(__pyx_t_1);
        __pyx_t_4 = __pyx_t_1; 
        __pyx_t_5 = 1;
        #if CYTHON_UNPACK_METHODS
        if (unlikely(PyMethod_Check(__pyx_t_4))) {
          __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_4);
          assert(__pyx_t_3);
          PyObject* __pyx__function = PyMethod_GET_FUNCTION(__pyx_t_4);
          __Pyx_INCREF(__pyx_t_3);
          __Pyx_INCREF(__pyx__function);
          __Pyx_DECREF_SET(__pyx_t_4, __pyx__function);
          __pyx_t_5 = 0;
        }
        #endif
        {
          PyObject *__pyx_callargs[2] = {__pyx_t_3, NULL};
          __pyx_t_2 = __Pyx_PyObject_FastCall((PyObject*)__pyx_t_4, __pyx_callargs+__pyx_t_5, (1-__pyx_t_5) | (__pyx_t_5*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
          __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
          __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
          if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1932, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_2);
        }
        __pyx_t_5 = __Pyx_PyLong_As_size_t(__pyx_t_2); if (unlikely((__pyx_t_5 == (size_t)-1) && PyErr_Occurred())) __PYX_ERR(0, 1932, __pyx_L1_error)
        __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
        __pyx_r = __pyx_t_5;
        __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
        goto __pyx_L0;
      }
      #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
      __pyx_tp_dict_version = __Pyx_get_tp_dict_version(((PyObject *)__pyx_v_self));
      __pyx_obj_dict_version = __Pyx_get_object_dict_version(((PyObject *)__pyx_v_self));
      if (unlikely(__pyx_typedict_guard != __pyx_tp_dict_version)) {
        __pyx_tp_dict_version = __pyx_obj_dict_version = __PYX_DICT_VERSION_INIT;
      }
      #endif
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
    }
    #endif
  }

  /* "cupy/cuda/memory.pyx":1938
 *             int: The total number of bytes used by the pool.
 *         """
 *         if not self.memoryAsyncHasStat:             # <<<<<<<<<<<<<<
 *             raise RuntimeError(
 *                 'The driver version is insufficient for this query')
*/
  __pyx_t_6 = (!__pyx_v_self->memoryAsyncHasStat);
  if (unlikely(__pyx_t_6)) {

    /* "cupy/cuda/memory.pyx":1939
 *         """
 *         if not self.memoryAsyncHasStat:
 *             raise RuntimeError(             # <<<<<<<<<<<<<<
 *                 'The driver version is insufficient for this query')
 *         cdef intptr_t pool = self._pools[device.get_device_id()]
*/
    __pyx_t_2 = NULL;
    __pyx_t_5 = 1;
    {
      PyObject *__pyx_callargs[2] = {__pyx_t_2, __pyx_mstate_global->__pyx_kp_u_The_driver_version_is_insufficie};
      __pyx_t_1 = __Pyx_PyObject_FastCall((PyObject*)(((PyTypeObject*)PyExc_RuntimeError)), __pyx_callargs+__pyx_t_5, (2-__pyx_t_5) | (__pyx_t_5*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
      __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
      if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1939, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
    }
    __Pyx_Raise(__pyx_t_1, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __PYX_ERR(0, 1939, __pyx_L1_error)

    /* "cupy/cuda/memory.pyx":1938
 *             int: The total number of bytes used by the pool.
 *         """
 *         if not self.memoryAsyncHasStat:             # <<<<<<<<<<<<<<
 *             raise RuntimeError(
 *                 'The driver version is insufficient for this query')
*/
  }

  /* "cupy/cuda/memory.pyx":1941
 *             raise RuntimeError(
 *                 'The driver version is insufficient for this query')
 *         cdef intptr_t pool = self._pools[device.get_device_id()]             # <<<<<<<<<<<<<<
 *         return runtime.memPoolGetAttribute(
 *             pool, runtime.cudaMemPoolAttrUsedMemCurrent)
*/
  if (unlikely(__pyx_v_self->_pools == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(0, 1941, __pyx_L1_error)
  }
  __pyx_t_7 = __pyx_f_4cupy_4cuda_6device_get_device_id(0); if (unlikely(__pyx_t_7 == ((int)-1) && PyErr_Occurred())) __PYX_ERR(0, 1941, __pyx_L1_error)
  __pyx_t_1 = __Pyx_GetItemInt_List(__pyx_v_self->_pools, __pyx_t_7, int, 1, __Pyx_PyLong_From_int, 1, 1, 1, 1, __Pyx_ReferenceSharing_SharedReference); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1941, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_8 = PyLong_AsSsize_t(__pyx_t_1); if (unlikely((__pyx_t_8 == ((intptr_t)-1)) && PyErr_Occurred())) __PYX_ERR(0, 1941, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v_pool = __pyx_t_8;

  /* "cupy/cuda/memory.pyx":1942
 *                 'The driver version is insufficient for this query')
 *         cdef intptr_t pool = self._pools[device.get_device_id()]
 *         return runtime.memPoolGetAttribute(             # <<<<<<<<<<<<<<
 *             pool, runtime.cudaMemPoolAttrUsedMemCurrent)
 * 
*/
  __pyx_t_1 = __pyx_f_13cupy_backends_4cuda_3api_7runtime_memPoolGetAttribute(__pyx_v_pool, __pyx_e_13cupy_backends_4cuda_3api_13_runtime_enum_cudaMemPoolAttrUsedMemCurrent, 0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1942, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_5 = __Pyx_PyLong_As_size_t(__pyx_t_1); if (unlikely((__pyx_t_5 == (size_t)-1) && PyErr_Occurred())) __PYX_ERR(0, 1942, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_r = __pyx_t_5;
  goto __pyx_L0;

  /* "cupy/cuda/memory.pyx":1932
 *             'This function is not supported in MemoryAsyncPool')
 * 
 *     cpdef size_t used_bytes(self):             # <<<<<<<<<<<<<<
 *         """Gets the total number of bytes used by the pool.
 * 
*/

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_AddTraceback("cupy.cuda.memory.MemoryAsyncPool.used_bytes", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_pw_4cupy_4cuda_6memory_15MemoryAsyncPool_9used_bytes(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
PyDoc_STRVAR(__pyx_doc_4cupy_4cuda_6memory_15MemoryAsyncPool_8used_bytes, "MemoryAsyncPool.used_bytes(self) -> size_t\n\nGets the total number of bytes used by the pool.\n\nReturns:\n    int: The total number of bytes used by the pool.");
static PyObject *__pyx_pw_4cupy_4cuda_6memory_15MemoryAsyncPool_9used_bytes(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
) {
  #if !CYTHON_METH_FASTCALL
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  #endif
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("used_bytes (wrapper)", 0);
  #if !CYTHON_METH_FASTCALL
  #if CYTHON_ASSUME_SAFE_SIZE
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return NULL;
  #endif
  #endif
  __pyx_kwvalues = __Pyx_KwValues_FASTCALL(__pyx_args, __pyx_nargs);
  if (unlikely(__pyx_nargs > 0)) { __Pyx_RaiseArgtupleInvalid("used_bytes", 1, 0, 0, __pyx_nargs); return NULL; }
  const Py_ssize_t __pyx_kwds_len = unlikely(__pyx_kwds) ? __Pyx_NumKwargs_FASTCALL(__pyx_kwds) : 0;
  if (unlikely(__pyx_kwds_len < 0)) return NULL;
  if (unlikely(__pyx_kwds_len > 0)) {__Pyx_RejectKeywords("used_bytes", __pyx_kwds); return NULL;}
  __pyx_r = __pyx_pf_4cupy_4cuda_6memory_15MemoryAsyncPool_8used_bytes(((struct __pyx_obj_4cupy_4cuda_6memory_MemoryAsyncPool *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4cupy_4cuda_6memory_15MemoryAsyncPool_8used_bytes(struct __pyx_obj_4cupy_4cuda_6memory_MemoryAsyncPool *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  size_t __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("used_bytes", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __pyx_f_4cupy_4cuda_6memory_15MemoryAsyncPool_used_bytes(__pyx_v_self, 1); if (unlikely(PyErr_Occurred())) __PYX_ERR(0, 1932, __pyx_L1_error)
  __pyx_t_2 = __Pyx_PyLong_FromSize_t(__pyx_t_1); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1932, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_r = __pyx_t_2;
  __pyx_t_2 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_AddTraceback("cupy.cuda.memory.MemoryAsyncPool.used_bytes", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "cupy/cuda/memory.pyx":1945
 *             pool, runtime.cudaMemPoolAttrUsedMemCurrent)
 * 
 *     cpdef size_t free_bytes(self):             # <<<<<<<<<<<<<<
 *         """Gets the total number of bytes acquired but not used by the pool.
 * 
*/

static PyObject *__pyx_pw_4cupy_4cuda_6memory_15MemoryAsyncPool_11free_bytes(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
static size_t __pyx_f_4cupy_4cuda_6memory_15MemoryAsyncPool_free_bytes(struct __pyx_obj_4cupy_4cuda_6memory_MemoryAsyncPool *__pyx_v_self, int __pyx_skip_dispatch) {
  size_t __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  size_t __pyx_t_5;
  size_t __pyx_t_6;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("free_bytes", 0);
  /* Check if called by wrapper */
  if (unlikely(__pyx_skip_dispatch)) ;
  /* Check if overridden in Python */
  else if (
  #if !CYTHON_USE_TYPE_SLOTS
  unlikely(Py_TYPE(((PyObject *)__pyx_v_self)) != __pyx_mstate_global->__pyx_ptype_4cupy_4cuda_6memory_MemoryAsyncPool &&
  __Pyx_PyType_HasFeature(Py_TYPE(((PyObject *)__pyx_v_self)), Py_TPFLAGS_HAVE_GC))
  #else
  unlikely(Py_TYPE(((PyObject *)__pyx_v_self))->tp_dictoffset != 0 || __Pyx_PyType_HasFeature(Py_TYPE(((PyObject *)__pyx_v_self)), (Py_TPFLAGS_IS_ABSTRACT | Py_TPFLAGS_HEAPTYPE)))
  #endif
  ) {
    #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
    static PY_UINT64_T __pyx_tp_dict_version = __PYX_DICT_VERSION_INIT, __pyx_obj_dict_version = __PYX_DICT_VERSION_INIT;
    if (unlikely(!__Pyx_object_dict_version_matches(((PyObject *)__pyx_v_self), __pyx_tp_dict_version, __pyx_obj_dict_version))) {
      PY_UINT64_T __pyx_typedict_guard = __Pyx_get_tp_dict_version(((PyObject *)__pyx_v_self));
      #endif
      __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_mstate_global->__pyx_n_u_free_bytes); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1945, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      if (!__Pyx_IsSameCFunction(__pyx_t_1, (void(*)(void)) __pyx_pw_4cupy_4cuda_6memory_15MemoryAsyncPool_11free_bytes)) {
        __pyx_t_3 = NULL;
        __Pyx_INCREF(__pyx_t_1);
        __pyx_t_4 = __pyx_t_1; 
        __pyx_t_5 = 1;
        #if CYTHON_UNPACK_METHODS
        if (unlikely(PyMethod_Check(__pyx_t_4))) {
          __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_4);
          assert(__pyx_t_3);
          PyObject* __pyx__function = PyMethod_GET_FUNCTION(__pyx_t_4);
          __Pyx_INCREF(__pyx_t_3);
          __Pyx_INCREF(__pyx__function);
          __Pyx_DECREF_SET(__pyx_t_4, __pyx__function);
          __pyx_t_5 = 0;
        }
        #endif
        {
          PyObject *__pyx_callargs[2] = {__pyx_t_3, NULL};
          __pyx_t_2 = __Pyx_PyObject_FastCall((PyObject*)__pyx_t_4, __pyx_callargs+__pyx_t_5, (1-__pyx_t_5) | (__pyx_t_5*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
          __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
          __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
          if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1945, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_2);
        }
        __pyx_t_5 = __Pyx_PyLong_As_size_t(__pyx_t_2); if (unlikely((__pyx_t_5 == (size_t)-1) && PyErr_Occurred())) __PYX_ERR(0, 1945, __pyx_L1_error)
        __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
        __pyx_r = __pyx_t_5;
        __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
        goto __pyx_L0;
      }
      #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
      __pyx_tp_dict_version = __Pyx_get_tp_dict_version(((PyObject *)__pyx_v_self));
      __pyx_obj_dict_version = __Pyx_get_object_dict_version(((PyObject *)__pyx_v_self));
      if (unlikely(__pyx_typedict_guard != __pyx_tp_dict_version)) {
        __pyx_tp_dict_version = __pyx_obj_dict_version = __PYX_DICT_VERSION_INIT;
      }
      #endif
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
    }
    #endif
  }

  /* "cupy/cuda/memory.pyx":1951
 *             int: The total number of bytes acquired but not used by the pool.
 *         """
 *         return self.total_bytes() - self.used_bytes()             # <<<<<<<<<<<<<<
 * 
 *     cpdef size_t total_bytes(self):
*/
  __pyx_t_5 = ((struct __pyx_vtabstruct_4cupy_4cuda_6memory_MemoryAsyncPool *)__pyx_v_self->__pyx_vtab)->total_bytes(__pyx_v_self, 0); if (unlikely(PyErr_Occurred())) __PYX_ERR(0, 1951, __pyx_L1_error)
  __pyx_t_6 = ((struct __pyx_vtabstruct_4cupy_4cuda_6memory_MemoryAsyncPool *)__pyx_v_self->__pyx_vtab)->used_bytes(__pyx_v_self, 0); if (unlikely(PyErr_Occurred())) __PYX_ERR(0, 1951, __pyx_L1_error)
  __pyx_r = (__pyx_t_5 - __pyx_t_6);
  goto __pyx_L0;

  /* "cupy/cuda/memory.pyx":1945
 *             pool, runtime.cudaMemPoolAttrUsedMemCurrent)
 * 
 *     cpdef size_t free_bytes(self):             # <<<<<<<<<<<<<<
 *         """Gets the total number of bytes acquired but not used by the pool.
 * 
*/

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_AddTraceback("cupy.cuda.memory.MemoryAsyncPool.free_bytes", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_pw_4cupy_4cuda_6memory_15MemoryAsyncPool_11free_bytes(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
PyDoc_STRVAR(__pyx_doc_4cupy_4cuda_6memory_15MemoryAsyncPool_10free_bytes, "MemoryAsyncPool.free_bytes(self) -> size_t\n\nGets the total number of bytes acquired but not used by the pool.\n\nReturns:\n    int: The total number of bytes acquired but not used by the pool.");
static PyObject *__pyx_pw_4cupy_4cuda_6memory_15MemoryAsyncPool_11free_bytes(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
) {
  #if !CYTHON_METH_FASTCALL
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  #endif
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("free_bytes (wrapper)", 0);
  #if !CYTHON_METH_FASTCALL
  #if CYTHON_ASSUME_SAFE_SIZE
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return NULL;
  #endif
  #endif
  __pyx_kwvalues = __Pyx_KwValues_FASTCALL(__pyx_args, __pyx_nargs);
  if (unlikely(__pyx_nargs > 0)) { __Pyx_RaiseArgtupleInvalid("free_bytes", 1, 0, 0, __pyx_nargs); return NULL; }
  const Py_ssize_t __pyx_kwds_len = unlikely(__pyx_kwds) ? __Pyx_NumKwargs_FASTCALL(__pyx_kwds) : 0;
  if (unlikely(__pyx_kwds_len < 0)) return NULL;
  if (unlikely(__pyx_kwds_len > 0)) {__Pyx_RejectKeywords("free_bytes", __pyx_kwds); return NULL;}
  __pyx_r = __pyx_pf_4cupy_4cuda_6memory_15MemoryAsyncPool_10free_bytes(((struct __pyx_obj_4cupy_4cuda_6memory_MemoryAsyncPool *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4cupy_4cuda_6memory_15MemoryAsyncPool_10free_bytes(struct __pyx_obj_4cupy_4cuda_6memory_MemoryAsyncPool *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  size_t __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("free_bytes", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __pyx_f_4cupy_4cuda_6memory_15MemoryAsyncPool_free_bytes(__pyx_v_self, 1); if (unlikely(PyErr_Occurred())) __PYX_ERR(0, 1945, __pyx_L1_error)
  __pyx_t_2 = __Pyx_PyLong_FromSize_t(__pyx_t_1); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1945, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_r = __pyx_t_2;
  __pyx_t_2 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_AddTraceback("cupy.cuda.memory.MemoryAsyncPool.free_bytes", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "cupy/cuda/memory.pyx":1953
 *         return self.total_bytes() - self.used_bytes()
 * 
 *     cpdef size_t total_bytes(self):             # <<<<<<<<<<<<<<
 *         """Gets the total number of bytes acquired by the pool.
 * 
*/

static PyObject *__pyx_pw_4cupy_4cuda_6memory_15MemoryAsyncPool_13total_bytes(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
static size_t __pyx_f_4cupy_4cuda_6memory_15MemoryAsyncPool_total_bytes(struct __pyx_obj_4cupy_4cuda_6memory_MemoryAsyncPool *__pyx_v_self, int __pyx_skip_dispatch) {
  intptr_t __pyx_v_pool;
  size_t __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  size_t __pyx_t_5;
  int __pyx_t_6;
  int __pyx_t_7;
  intptr_t __pyx_t_8;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("total_bytes", 0);
  /* Check if called by wrapper */
  if (unlikely(__pyx_skip_dispatch)) ;
  /* Check if overridden in Python */
  else if (
  #if !CYTHON_USE_TYPE_SLOTS
  unlikely(Py_TYPE(((PyObject *)__pyx_v_self)) != __pyx_mstate_global->__pyx_ptype_4cupy_4cuda_6memory_MemoryAsyncPool &&
  __Pyx_PyType_HasFeature(Py_TYPE(((PyObject *)__pyx_v_self)), Py_TPFLAGS_HAVE_GC))
  #else
  unlikely(Py_TYPE(((PyObject *)__pyx_v_self))->tp_dictoffset != 0 || __Pyx_PyType_HasFeature(Py_TYPE(((PyObject *)__pyx_v_self)), (Py_TPFLAGS_IS_ABSTRACT | Py_TPFLAGS_HEAPTYPE)))
  #endif
  ) {
    #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
    static PY_UINT64_T __pyx_tp_dict_version = __PYX_DICT_VERSION_INIT, __pyx_obj_dict_version = __PYX_DICT_VERSION_INIT;
    if (unlikely(!__Pyx_object_dict_version_matches(((PyObject *)__pyx_v_self), __pyx_tp_dict_version, __pyx_obj_dict_version))) {
      PY_UINT64_T __pyx_typedict_guard = __Pyx_get_tp_dict_version(((PyObject *)__pyx_v_self));
      #endif
      __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_mstate_global->__pyx_n_u_total_bytes); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1953, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      if (!__Pyx_IsSameCFunction(__pyx_t_1, (void(*)(void)) __pyx_pw_4cupy_4cuda_6memory_15MemoryAsyncPool_13total_bytes)) {
        __pyx_t_3 = NULL;
        __Pyx_INCREF(__pyx_t_1);
        __pyx_t_4 = __pyx_t_1; 
        __pyx_t_5 = 1;
        #if CYTHON_UNPACK_METHODS
        if (unlikely(PyMethod_Check(__pyx_t_4))) {
          __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_4);
          assert(__pyx_t_3);
          PyObject* __pyx__function = PyMethod_GET_FUNCTION(__pyx_t_4);
          __Pyx_INCREF(__pyx_t_3);
          __Pyx_INCREF(__pyx__function);
          __Pyx_DECREF_SET(__pyx_t_4, __pyx__function);
          __pyx_t_5 = 0;
        }
        #endif
        {
          PyObject *__pyx_callargs[2] = {__pyx_t_3, NULL};
          __pyx_t_2 = __Pyx_PyObject_FastCall((PyObject*)__pyx_t_4, __pyx_callargs+__pyx_t_5, (1-__pyx_t_5) | (__pyx_t_5*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
          __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
          __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
          if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1953, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_2);
        }
        __pyx_t_5 = __Pyx_PyLong_As_size_t(__pyx_t_2); if (unlikely((__pyx_t_5 == (size_t)-1) && PyErr_Occurred())) __PYX_ERR(0, 1953, __pyx_L1_error)
        __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
        __pyx_r = __pyx_t_5;
        __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
        goto __pyx_L0;
      }
      #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
      __pyx_tp_dict_version = __Pyx_get_tp_dict_version(((PyObject *)__pyx_v_self));
      __pyx_obj_dict_version = __Pyx_get_object_dict_version(((PyObject *)__pyx_v_self));
      if (unlikely(__pyx_typedict_guard != __pyx_tp_dict_version)) {
        __pyx_tp_dict_version = __pyx_obj_dict_version = __PYX_DICT_VERSION_INIT;
      }
      #endif
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
    }
    #endif
  }

  /* "cupy/cuda/memory.pyx":1959
 *             int: The total number of bytes acquired by the pool.
 *         """
 *         if not self.memoryAsyncHasStat:             # <<<<<<<<<<<<<<
 *             raise RuntimeError(
 *                 'The driver version is insufficient for this query')
*/
  __pyx_t_6 = (!__pyx_v_self->memoryAsyncHasStat);
  if (unlikely(__pyx_t_6)) {

    /* "cupy/cuda/memory.pyx":1960
 *         """
 *         if not self.memoryAsyncHasStat:
 *             raise RuntimeError(             # <<<<<<<<<<<<<<
 *                 'The driver version is insufficient for this query')
 *         cdef intptr_t pool = self._pools[device.get_device_id()]
*/
    __pyx_t_2 = NULL;
    __pyx_t_5 = 1;
    {
      PyObject *__pyx_callargs[2] = {__pyx_t_2, __pyx_mstate_global->__pyx_kp_u_The_driver_version_is_insufficie};
      __pyx_t_1 = __Pyx_PyObject_FastCall((PyObject*)(((PyTypeObject*)PyExc_RuntimeError)), __pyx_callargs+__pyx_t_5, (2-__pyx_t_5) | (__pyx_t_5*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
      __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
      if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1960, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
    }
    __Pyx_Raise(__pyx_t_1, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __PYX_ERR(0, 1960, __pyx_L1_error)

    /* "cupy/cuda/memory.pyx":1959
 *             int: The total number of bytes acquired by the pool.
 *         """
 *         if not self.memoryAsyncHasStat:             # <<<<<<<<<<<<<<
 *             raise RuntimeError(
 *                 'The driver version is insufficient for this query')
*/
  }

  /* "cupy/cuda/memory.pyx":1962
 *             raise RuntimeError(
 *                 'The driver version is insufficient for this query')
 *         cdef intptr_t pool = self._pools[device.get_device_id()]             # <<<<<<<<<<<<<<
 *         return runtime.memPoolGetAttribute(
 *             pool, runtime.cudaMemPoolAttrReservedMemCurrent)
*/
  if (unlikely(__pyx_v_self->_pools == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(0, 1962, __pyx_L1_error)
  }
  __pyx_t_7 = __pyx_f_4cupy_4cuda_6device_get_device_id(0); if (unlikely(__pyx_t_7 == ((int)-1) && PyErr_Occurred())) __PYX_ERR(0, 1962, __pyx_L1_error)
  __pyx_t_1 = __Pyx_GetItemInt_List(__pyx_v_self->_pools, __pyx_t_7, int, 1, __Pyx_PyLong_From_int, 1, 1, 1, 1, __Pyx_ReferenceSharing_SharedReference); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1962, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_8 = PyLong_AsSsize_t(__pyx_t_1); if (unlikely((__pyx_t_8 == ((intptr_t)-1)) && PyErr_Occurred())) __PYX_ERR(0, 1962, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v_pool = __pyx_t_8;

  /* "cupy/cuda/memory.pyx":1963
 *                 'The driver version is insufficient for this query')
 *         cdef intptr_t pool = self._pools[device.get_device_id()]
 *         return runtime.memPoolGetAttribute(             # <<<<<<<<<<<<<<
 *             pool, runtime.cudaMemPoolAttrReservedMemCurrent)
 * 
*/
  __pyx_t_1 = __pyx_f_13cupy_backends_4cuda_3api_7runtime_memPoolGetAttribute(__pyx_v_pool, __pyx_e_13cupy_backends_4cuda_3api_13_runtime_enum_cudaMemPoolAttrReservedMemCurrent, 0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1963, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_5 = __Pyx_PyLong_As_size_t(__pyx_t_1); if (unlikely((__pyx_t_5 == (size_t)-1) && PyErr_Occurred())) __PYX_ERR(0, 1963, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_r = __pyx_t_5;
  goto __pyx_L0;

  /* "cupy/cuda/memory.pyx":1953
 *         return self.total_bytes() - self.used_bytes()
 * 
 *     cpdef size_t total_bytes(self):             # <<<<<<<<<<<<<<
 *         """Gets the total number of bytes acquired by the pool.
 * 
*/

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_AddTraceback("cupy.cuda.memory.MemoryAsyncPool.total_bytes", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_pw_4cupy_4cuda_6memory_15MemoryAsyncPool_13total_bytes(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
PyDoc_STRVAR(__pyx_doc_4cupy_4cuda_6memory_15MemoryAsyncPool_12total_bytes, "MemoryAsyncPool.total_bytes(self) -> size_t\n\nGets the total number of bytes acquired by the pool.\n\nReturns:\n    int: The total number of bytes acquired by the pool.");
static PyObject *__pyx_pw_4cupy_4cuda_6memory_15MemoryAsyncPool_13total_bytes(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
) {
  #if !CYTHON_METH_FASTCALL
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  #endif
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("total_bytes (wrapper)", 0);
  #if !CYTHON_METH_FASTCALL
  #if CYTHON_ASSUME_SAFE_SIZE
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return NULL;
  #endif
  #endif
  __pyx_kwvalues = __Pyx_KwValues_FASTCALL(__pyx_args, __pyx_nargs);
  if (unlikely(__pyx_nargs > 0)) { __Pyx_RaiseArgtupleInvalid("total_bytes", 1, 0, 0, __pyx_nargs); return NULL; }
  const Py_ssize_t __pyx_kwds_len = unlikely(__pyx_kwds) ? __Pyx_NumKwargs_FASTCALL(__pyx_kwds) : 0;
  if (unlikely(__pyx_kwds_len < 0)) return NULL;
  if (unlikely(__pyx_kwds_len > 0)) {__Pyx_RejectKeywords("total_bytes", __pyx_kwds); return NULL;}
  __pyx_r = __pyx_pf_4cupy_4cuda_6memory_15MemoryAsyncPool_12total_bytes(((struct __pyx_obj_4cupy_4cuda_6memory_MemoryAsyncPool *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4cupy_4cuda_6memory_15MemoryAsyncPool_12total_bytes(struct __pyx_obj_4cupy_4cuda_6memory_MemoryAsyncPool *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  size_t __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("total_bytes", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __pyx_f_4cupy_4cuda_6memory_15MemoryAsyncPool_total_bytes(__pyx_v_self, 1); if (unlikely(PyErr_Occurred())) __PYX_ERR(0, 1953, __pyx_L1_error)
  __pyx_t_2 = __Pyx_PyLong_FromSize_t(__pyx_t_1); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1953, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_r = __pyx_t_2;
  __pyx_t_2 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_AddTraceback("cupy.cuda.memory.MemoryAsyncPool.total_bytes", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "cupy/cuda/memory.pyx":1966
 *             pool, runtime.cudaMemPoolAttrReservedMemCurrent)
 * 
 *     cpdef set_limit(self, size=None, fraction=None):             # <<<<<<<<<<<<<<
 *         """Sets the upper limit of memory allocation of the current device.
 * 
*/

static PyObject *__pyx_pw_4cupy_4cuda_6memory_15MemoryAsyncPool_15set_limit(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
static PyObject *__pyx_f_4cupy_4cuda_6memory_15MemoryAsyncPool_set_limit(struct __pyx_obj_4cupy_4cuda_6memory_MemoryAsyncPool *__pyx_v_self, int __pyx_skip_dispatch, struct __pyx_opt_args_4cupy_4cuda_6memory_15MemoryAsyncPool_set_limit *__pyx_optional_args) {
  PyObject *__pyx_v_size = ((PyObject *)Py_None);
  PyObject *__pyx_v_fraction = ((PyObject *)Py_None);
  CYTHON_UNUSED PyObject *__pyx_v__ = NULL;
  PyObject *__pyx_v_total = NULL;
  intptr_t __pyx_v_pool;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  size_t __pyx_t_5;
  int __pyx_t_6;
  int __pyx_t_7;
  PyObject *(*__pyx_t_8)(PyObject *);
  struct __pyx_opt_args_4cupy_4cuda_6memory_15MemoryAsyncPool_set_limit __pyx_t_9;
  int __pyx_t_10;
  intptr_t __pyx_t_11;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("set_limit", 0);
  if (__pyx_optional_args) {
    if (__pyx_optional_args->__pyx_n > 0) {
      __pyx_v_size = __pyx_optional_args->size;
      if (__pyx_optional_args->__pyx_n > 1) {
        __pyx_v_fraction = __pyx_optional_args->fraction;
      }
    }
  }
  __Pyx_INCREF(__pyx_v_size);
  /* Check if called by wrapper */
  if (unlikely(__pyx_skip_dispatch)) ;
  /* Check if overridden in Python */
  else if (
  #if !CYTHON_USE_TYPE_SLOTS
  unlikely(Py_TYPE(((PyObject *)__pyx_v_self)) != __pyx_mstate_global->__pyx_ptype_4cupy_4cuda_6memory_MemoryAsyncPool &&
  __Pyx_PyType_HasFeature(Py_TYPE(((PyObject *)__pyx_v_self)), Py_TPFLAGS_HAVE_GC))
  #else
  unlikely(Py_TYPE(((PyObject *)__pyx_v_self))->tp_dictoffset != 0 || __Pyx_PyType_HasFeature(Py_TYPE(((PyObject *)__pyx_v_self)), (Py_TPFLAGS_IS_ABSTRACT | Py_TPFLAGS_HEAPTYPE)))
  #endif
  ) {
    #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
    static PY_UINT64_T __pyx_tp_dict_version = __PYX_DICT_VERSION_INIT, __pyx_obj_dict_version = __PYX_DICT_VERSION_INIT;
    if (unlikely(!__Pyx_object_dict_version_matches(((PyObject *)__pyx_v_self), __pyx_tp_dict_version, __pyx_obj_dict_version))) {
      PY_UINT64_T __pyx_typedict_guard = __Pyx_get_tp_dict_version(((PyObject *)__pyx_v_self));
      #endif
      __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_mstate_global->__pyx_n_u_set_limit); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1966, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      if (!__Pyx_IsSameCFunction(__pyx_t_1, (void(*)(void)) __pyx_pw_4cupy_4cuda_6memory_15MemoryAsyncPool_15set_limit)) {
        __Pyx_XDECREF(__pyx_r);
        __pyx_t_3 = NULL;
        __Pyx_INCREF(__pyx_t_1);
        __pyx_t_4 = __pyx_t_1; 
        __pyx_t_5 = 1;
        #if CYTHON_UNPACK_METHODS
        if (unlikely(PyMethod_Check(__pyx_t_4))) {
          __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_4);
          assert(__pyx_t_3);
          PyObject* __pyx__function = PyMethod_GET_FUNCTION(__pyx_t_4);
          __Pyx_INCREF(__pyx_t_3);
          __Pyx_INCREF(__pyx__function);
          __Pyx_DECREF_SET(__pyx_t_4, __pyx__function);
          __pyx_t_5 = 0;
        }
        #endif
        {
          PyObject *__pyx_callargs[3] = {__pyx_t_3, __pyx_v_size, __pyx_v_fraction};
          __pyx_t_2 = __Pyx_PyObject_FastCall((PyObject*)__pyx_t_4, __pyx_callargs+__pyx_t_5, (3-__pyx_t_5) | (__pyx_t_5*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
          __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
          __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
          if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1966, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_2);
        }
        __pyx_r = __pyx_t_2;
        __pyx_t_2 = 0;
        __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
        goto __pyx_L0;
      }
      #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
      __pyx_tp_dict_version = __Pyx_get_tp_dict_version(((PyObject *)__pyx_v_self));
      __pyx_obj_dict_version = __Pyx_get_object_dict_version(((PyObject *)__pyx_v_self));
      if (unlikely(__pyx_typedict_guard != __pyx_tp_dict_version)) {
        __pyx_tp_dict_version = __pyx_obj_dict_version = __PYX_DICT_VERSION_INIT;
      }
      #endif
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
    }
    #endif
  }

  /* "cupy/cuda/memory.pyx":2000
 *             fraction (float): Fraction in the range of ``[0, 1]``.
 *         """
 *         if size is None:             # <<<<<<<<<<<<<<
 *             if fraction is None:
 *                 size = 0
*/
  __pyx_t_6 = (__pyx_v_size == Py_None);
  if (__pyx_t_6) {

    /* "cupy/cuda/memory.pyx":2001
 *         """
 *         if size is None:
 *             if fraction is None:             # <<<<<<<<<<<<<<
 *                 size = 0
 *             else:
*/
    __pyx_t_6 = (__pyx_v_fraction == Py_None);
    if (__pyx_t_6) {

      /* "cupy/cuda/memory.pyx":2002
 *         if size is None:
 *             if fraction is None:
 *                 size = 0             # <<<<<<<<<<<<<<
 *             else:
 *                 if not 0 <= fraction <= 1:
*/
      __Pyx_INCREF(__pyx_mstate_global->__pyx_int_0);
      __Pyx_DECREF_SET(__pyx_v_size, __pyx_mstate_global->__pyx_int_0);

      /* "cupy/cuda/memory.pyx":2001
 *         """
 *         if size is None:
 *             if fraction is None:             # <<<<<<<<<<<<<<
 *                 size = 0
 *             else:
*/
      goto __pyx_L4;
    }

    /* "cupy/cuda/memory.pyx":2004
 *                 size = 0
 *             else:
 *                 if not 0 <= fraction <= 1:             # <<<<<<<<<<<<<<
 *                     raise ValueError(
 *                         'memory limit fraction out of range: {}'.format(
*/
    /*else*/ {
      __pyx_t_1 = PyObject_RichCompare(__pyx_mstate_global->__pyx_int_0, __pyx_v_fraction, Py_LE); __Pyx_XGOTREF(__pyx_t_1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 2004, __pyx_L1_error)
      if (__Pyx_PyObject_IsTrue(__pyx_t_1)) {
        __Pyx_DECREF(__pyx_t_1);
        __pyx_t_1 = PyObject_RichCompare(__pyx_v_fraction, __pyx_mstate_global->__pyx_int_1, Py_LE); __Pyx_XGOTREF(__pyx_t_1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 2004, __pyx_L1_error)
      }
      __pyx_t_6 = __Pyx_PyObject_IsTrue(__pyx_t_1); if (unlikely((__pyx_t_6 < 0))) __PYX_ERR(0, 2004, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      __pyx_t_7 = (!__pyx_t_6);
      if (unlikely(__pyx_t_7)) {

        /* "cupy/cuda/memory.pyx":2005
 *             else:
 *                 if not 0 <= fraction <= 1:
 *                     raise ValueError(             # <<<<<<<<<<<<<<
 *                         'memory limit fraction out of range: {}'.format(
 *                             fraction))
*/
        __pyx_t_2 = NULL;

        /* "cupy/cuda/memory.pyx":2006
 *                 if not 0 <= fraction <= 1:
 *                     raise ValueError(
 *                         'memory limit fraction out of range: {}'.format(             # <<<<<<<<<<<<<<
 *                             fraction))
 *                 _, total = runtime.memGetInfo()
*/
        __pyx_t_3 = __pyx_mstate_global->__pyx_kp_u_memory_limit_fraction_out_of_ran;
        __Pyx_INCREF(__pyx_t_3);

        /* "cupy/cuda/memory.pyx":2007
 *                     raise ValueError(
 *                         'memory limit fraction out of range: {}'.format(
 *                             fraction))             # <<<<<<<<<<<<<<
 *                 _, total = runtime.memGetInfo()
 *                 size = fraction * total
*/
        __pyx_t_5 = 0;
        {
          PyObject *__pyx_callargs[2] = {__pyx_t_3, __pyx_v_fraction};
          __pyx_t_4 = __Pyx_PyObject_FastCallMethod((PyObject*)__pyx_mstate_global->__pyx_n_u_format, __pyx_callargs+__pyx_t_5, (2-__pyx_t_5) | (1*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
          __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
          if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 2006, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_4);
        }
        __pyx_t_5 = 1;
        {
          PyObject *__pyx_callargs[2] = {__pyx_t_2, __pyx_t_4};
          __pyx_t_1 = __Pyx_PyObject_FastCall((PyObject*)(((PyTypeObject*)PyExc_ValueError)), __pyx_callargs+__pyx_t_5, (2-__pyx_t_5) | (__pyx_t_5*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
          __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
          __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
          if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 2005, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_1);
        }
        __Pyx_Raise(__pyx_t_1, 0, 0, 0);
        __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
        __PYX_ERR(0, 2005, __pyx_L1_error)

        /* "cupy/cuda/memory.pyx":2004
 *                 size = 0
 *             else:
 *                 if not 0 <= fraction <= 1:             # <<<<<<<<<<<<<<
 *                     raise ValueError(
 *                         'memory limit fraction out of range: {}'.format(
*/
      }

      /* "cupy/cuda/memory.pyx":2008
 *                         'memory limit fraction out of range: {}'.format(
 *                             fraction))
 *                 _, total = runtime.memGetInfo()             # <<<<<<<<<<<<<<
 *                 size = fraction * total
 *             self.set_limit(size=size)
*/
      __pyx_t_1 = __pyx_f_13cupy_backends_4cuda_3api_7runtime_memGetInfo(0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 2008, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      if ((likely(PyTuple_CheckExact(__pyx_t_1))) || (PyList_CheckExact(__pyx_t_1))) {
        PyObject* sequence = __pyx_t_1;
        Py_ssize_t size = __Pyx_PySequence_SIZE(sequence);
        if (unlikely(size != 2)) {
          if (size > 2) __Pyx_RaiseTooManyValuesError(2);
          else if (size >= 0) __Pyx_RaiseNeedMoreValuesError(size);
          __PYX_ERR(0, 2008, __pyx_L1_error)
        }
        #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
        if (likely(PyTuple_CheckExact(sequence))) {
          __pyx_t_4 = PyTuple_GET_ITEM(sequence, 0);
          __Pyx_INCREF(__pyx_t_4);
          __pyx_t_2 = PyTuple_GET_ITEM(sequence, 1);
          __Pyx_INCREF(__pyx_t_2);
        } else {
          __pyx_t_4 = __Pyx_PyList_GetItemRefFast(sequence, 0, __Pyx_ReferenceSharing_SharedReference);
          if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 2008, __pyx_L1_error)
          __Pyx_XGOTREF(__pyx_t_4);
          __pyx_t_2 = __Pyx_PyList_GetItemRefFast(sequence, 1, __Pyx_ReferenceSharing_SharedReference);
          if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 2008, __pyx_L1_error)
          __Pyx_XGOTREF(__pyx_t_2);
        }
        #else
        __pyx_t_4 = __Pyx_PySequence_ITEM(sequence, 0); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 2008, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_4);
        __pyx_t_2 = __Pyx_PySequence_ITEM(sequence, 1); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 2008, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_2);
        #endif
        __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      } else {
        Py_ssize_t index = -1;
        __pyx_t_3 = PyObject_GetIter(__pyx_t_1); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 2008, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_3);
        __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
        __pyx_t_8 = (CYTHON_COMPILING_IN_LIMITED_API) ? PyIter_Next : __Pyx_PyObject_GetIterNextFunc(__pyx_t_3);
        index = 0; __pyx_t_4 = __pyx_t_8(__pyx_t_3); if (unlikely(!__pyx_t_4)) goto __pyx_L6_unpacking_failed;
        __Pyx_GOTREF(__pyx_t_4);
        index = 1; __pyx_t_2 = __pyx_t_8(__pyx_t_3); if (unlikely(!__pyx_t_2)) goto __pyx_L6_unpacking_failed;
        __Pyx_GOTREF(__pyx_t_2);
        if (__Pyx_IternextUnpackEndCheck(__pyx_t_8(__pyx_t_3), 2) < (0)) __PYX_ERR(0, 2008, __pyx_L1_error)
        __pyx_t_8 = NULL;
        __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
        goto __pyx_L7_unpacking_done;
        __pyx_L6_unpacking_failed:;
        __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
        __pyx_t_8 = NULL;
        if (__Pyx_IterFinish() == 0) __Pyx_RaiseNeedMoreValuesError(index);
        __PYX_ERR(0, 2008, __pyx_L1_error)
        __pyx_L7_unpacking_done:;
      }
      __pyx_v__ = __pyx_t_4;
      __pyx_t_4 = 0;
      __pyx_v_total = __pyx_t_2;
      __pyx_t_2 = 0;

      /* "cupy/cuda/memory.pyx":2009
 *                             fraction))
 *                 _, total = runtime.memGetInfo()
 *                 size = fraction * total             # <<<<<<<<<<<<<<
 *             self.set_limit(size=size)
 *             return
*/
      __pyx_t_1 = PyNumber_Multiply(__pyx_v_fraction, __pyx_v_total); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 2009, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      __Pyx_DECREF_SET(__pyx_v_size, __pyx_t_1);
      __pyx_t_1 = 0;
    }
    __pyx_L4:;

    /* "cupy/cuda/memory.pyx":2010
 *                 _, total = runtime.memGetInfo()
 *                 size = fraction * total
 *             self.set_limit(size=size)             # <<<<<<<<<<<<<<
 *             return
 * 
*/
    __pyx_t_9.__pyx_n = 1;
    __pyx_t_9.size = __pyx_v_size;
    __pyx_t_1 = ((struct __pyx_vtabstruct_4cupy_4cuda_6memory_MemoryAsyncPool *)__pyx_v_self->__pyx_vtab)->set_limit(__pyx_v_self, 0, &__pyx_t_9); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 2010, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

    /* "cupy/cuda/memory.pyx":2011
 *                 size = fraction * total
 *             self.set_limit(size=size)
 *             return             # <<<<<<<<<<<<<<
 * 
 *         if fraction is not None:
*/
    __Pyx_XDECREF(__pyx_r);
    __pyx_r = Py_None; __Pyx_INCREF(Py_None);
    goto __pyx_L0;

    /* "cupy/cuda/memory.pyx":2000
 *             fraction (float): Fraction in the range of ``[0, 1]``.
 *         """
 *         if size is None:             # <<<<<<<<<<<<<<
 *             if fraction is None:
 *                 size = 0
*/
  }

  /* "cupy/cuda/memory.pyx":2013
 *             return
 * 
 *         if fraction is not None:             # <<<<<<<<<<<<<<
 *             raise ValueError('size and fraction cannot be specified at '
 *                              'one time')
*/
  __pyx_t_7 = (__pyx_v_fraction != Py_None);
  if (unlikely(__pyx_t_7)) {

    /* "cupy/cuda/memory.pyx":2014
 * 
 *         if fraction is not None:
 *             raise ValueError('size and fraction cannot be specified at '             # <<<<<<<<<<<<<<
 *                              'one time')
 *         if size < 0:
*/
    __pyx_t_2 = NULL;
    __pyx_t_5 = 1;
    {
      PyObject *__pyx_callargs[2] = {__pyx_t_2, __pyx_mstate_global->__pyx_kp_u_size_and_fraction_cannot_be_spec};
      __pyx_t_1 = __Pyx_PyObject_FastCall((PyObject*)(((PyTypeObject*)PyExc_ValueError)), __pyx_callargs+__pyx_t_5, (2-__pyx_t_5) | (__pyx_t_5*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
      __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
      if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 2014, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
    }
    __Pyx_Raise(__pyx_t_1, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __PYX_ERR(0, 2014, __pyx_L1_error)

    /* "cupy/cuda/memory.pyx":2013
 *             return
 * 
 *         if fraction is not None:             # <<<<<<<<<<<<<<
 *             raise ValueError('size and fraction cannot be specified at '
 *                              'one time')
*/
  }

  /* "cupy/cuda/memory.pyx":2016
 *             raise ValueError('size and fraction cannot be specified at '
 *                              'one time')
 *         if size < 0:             # <<<<<<<<<<<<<<
 *             raise ValueError(
 *                 'memory limit size out of range: {}'.format(size))
*/
  __pyx_t_1 = PyObject_RichCompare(__pyx_v_size, __pyx_mstate_global->__pyx_int_0, Py_LT); __Pyx_XGOTREF(__pyx_t_1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 2016, __pyx_L1_error)
  __pyx_t_7 = __Pyx_PyObject_IsTrue(__pyx_t_1); if (unlikely((__pyx_t_7 < 0))) __PYX_ERR(0, 2016, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  if (unlikely(__pyx_t_7)) {

    /* "cupy/cuda/memory.pyx":2017
 *                              'one time')
 *         if size < 0:
 *             raise ValueError(             # <<<<<<<<<<<<<<
 *                 'memory limit size out of range: {}'.format(size))
 * 
*/
    __pyx_t_2 = NULL;

    /* "cupy/cuda/memory.pyx":2018
 *         if size < 0:
 *             raise ValueError(
 *                 'memory limit size out of range: {}'.format(size))             # <<<<<<<<<<<<<<
 * 
 *         if size == 0:
*/
    __pyx_t_3 = __pyx_mstate_global->__pyx_kp_u_memory_limit_size_out_of_range;
    __Pyx_INCREF(__pyx_t_3);
    __pyx_t_5 = 0;
    {
      PyObject *__pyx_callargs[2] = {__pyx_t_3, __pyx_v_size};
      __pyx_t_4 = __Pyx_PyObject_FastCallMethod((PyObject*)__pyx_mstate_global->__pyx_n_u_format, __pyx_callargs+__pyx_t_5, (2-__pyx_t_5) | (1*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
      __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
      if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 2018, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_4);
    }
    __pyx_t_5 = 1;
    {
      PyObject *__pyx_callargs[2] = {__pyx_t_2, __pyx_t_4};
      __pyx_t_1 = __Pyx_PyObject_FastCall((PyObject*)(((PyTypeObject*)PyExc_ValueError)), __pyx_callargs+__pyx_t_5, (2-__pyx_t_5) | (__pyx_t_5*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
      __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 2017, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
    }
    __Pyx_Raise(__pyx_t_1, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __PYX_ERR(0, 2017, __pyx_L1_error)

    /* "cupy/cuda/memory.pyx":2016
 *             raise ValueError('size and fraction cannot be specified at '
 *                              'one time')
 *         if size < 0:             # <<<<<<<<<<<<<<
 *             raise ValueError(
 *                 'memory limit size out of range: {}'.format(size))
*/
  }

  /* "cupy/cuda/memory.pyx":2020
 *                 'memory limit size out of range: {}'.format(size))
 * 
 *         if size == 0:             # <<<<<<<<<<<<<<
 *             size = UINT64_MAX  # ensure pool size is never shrunk
 *         cdef intptr_t pool = self._pools[device.get_device_id()]
*/
  __pyx_t_7 = (__Pyx_PyLong_BoolEqObjC(__pyx_v_size, __pyx_mstate_global->__pyx_int_0, 0, 0)); if (unlikely((__pyx_t_7 < 0))) __PYX_ERR(0, 2020, __pyx_L1_error)
  if (__pyx_t_7) {

    /* "cupy/cuda/memory.pyx":2021
 * 
 *         if size == 0:
 *             size = UINT64_MAX  # ensure pool size is never shrunk             # <<<<<<<<<<<<<<
 *         cdef intptr_t pool = self._pools[device.get_device_id()]
 *         runtime.memPoolSetAttribute(
*/
    __pyx_t_1 = __Pyx_PyLong_From_uint64_t(UINT64_MAX); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 2021, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF_SET(__pyx_v_size, __pyx_t_1);
    __pyx_t_1 = 0;

    /* "cupy/cuda/memory.pyx":2020
 *                 'memory limit size out of range: {}'.format(size))
 * 
 *         if size == 0:             # <<<<<<<<<<<<<<
 *             size = UINT64_MAX  # ensure pool size is never shrunk
 *         cdef intptr_t pool = self._pools[device.get_device_id()]
*/
  }

  /* "cupy/cuda/memory.pyx":2022
 *         if size == 0:
 *             size = UINT64_MAX  # ensure pool size is never shrunk
 *         cdef intptr_t pool = self._pools[device.get_device_id()]             # <<<<<<<<<<<<<<
 *         runtime.memPoolSetAttribute(
 *             pool, runtime.cudaMemPoolAttrReleaseThreshold, size)
*/
  if (unlikely(__pyx_v_self->_pools == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(0, 2022, __pyx_L1_error)
  }
  __pyx_t_10 = __pyx_f_4cupy_4cuda_6device_get_device_id(0); if (unlikely(__pyx_t_10 == ((int)-1) && PyErr_Occurred())) __PYX_ERR(0, 2022, __pyx_L1_error)
  __pyx_t_1 = __Pyx_GetItemInt_List(__pyx_v_self->_pools, __pyx_t_10, int, 1, __Pyx_PyLong_From_int, 1, 1, 1, 1, __Pyx_ReferenceSharing_SharedReference); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 2022, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_11 = PyLong_AsSsize_t(__pyx_t_1); if (unlikely((__pyx_t_11 == ((intptr_t)-1)) && PyErr_Occurred())) __PYX_ERR(0, 2022, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v_pool = __pyx_t_11;

  /* "cupy/cuda/memory.pyx":2023
 *             size = UINT64_MAX  # ensure pool size is never shrunk
 *         cdef intptr_t pool = self._pools[device.get_device_id()]
 *         runtime.memPoolSetAttribute(             # <<<<<<<<<<<<<<
 *             pool, runtime.cudaMemPoolAttrReleaseThreshold, size)
 * 
*/
  __pyx_t_1 = __pyx_f_13cupy_backends_4cuda_3api_7runtime_memPoolSetAttribute(__pyx_v_pool, __pyx_e_13cupy_backends_4cuda_3api_13_runtime_enum_cudaMemPoolAttrReleaseThreshold, __pyx_v_size, 0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 2023, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "cupy/cuda/memory.pyx":1966
 *             pool, runtime.cudaMemPoolAttrReservedMemCurrent)
 * 
 *     cpdef set_limit(self, size=None, fraction=None):             # <<<<<<<<<<<<<<
 *         """Sets the upper limit of memory allocation of the current device.
 * 
*/

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_AddTraceback("cupy.cuda.memory.MemoryAsyncPool.set_limit", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v__);
  __Pyx_XDECREF(__pyx_v_total);
  __Pyx_XDECREF(__pyx_v_size);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_pw_4cupy_4cuda_6memory_15MemoryAsyncPool_15set_limit(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
PyDoc_STRVAR(__pyx_doc_4cupy_4cuda_6memory_15MemoryAsyncPool_14set_limit, "MemoryAsyncPool.set_limit(self, size=None, fraction=None)\n\nSets the upper limit of memory allocation of the current device.\n\nWhen `fraction` is specified, its value will become a fraction of the\namount of GPU memory that is available for allocation.\nFor example, if you have a GPU with 2 GiB memory, you can either use\n``set_limit(fraction=0.5)`` or ``set_limit(size=1024**3)`` to limit\nthe memory size to 1 GiB.\n\n``size`` and ``fraction`` cannot be specified at the same time.\nIf both of them are **not** specified or ``0`` is specified, the\nlimit will be disabled.\n\n.. note::\n    Unlike with :class:`MemoryPool`, :class:`MemoryAsyncPool`'s\n    :meth:`set_limit` method can only impose a *soft* limit. If other\n    (non-CuPy) applications are also allocating memory from the same\n    mempool, this limit may not be respected. Internally, this limit\n    is set via the ``cudaMemPoolAttrReleaseThreshold`` attribute.\n\n.. note::\n    You can also set the limit by using ``CUPY_GPU_MEMORY_LIMIT``\n    environment variable, see :ref:`environment` for the details.\n    The limit set by this method supersedes the value specified in\n    the environment variable.\n\n    Also note that this method only changes the limit for the current\n    device, whereas the environment variable sets the default limit for\n    all devices.\n\nArgs:\n    size (int): Limit size in bytes.\n    fraction (float): Fraction in the range of ``[0, 1]``.");
static PyObject *__pyx_pw_4cupy_4cuda_6memory_15MemoryAsyncPool_15set_limit(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
) {
  PyObject *__pyx_v_size = 0;
  PyObject *__pyx_v_fraction = 0;
  #if !CYTHON_METH_FASTCALL
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  #endif
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject* values[2] = {0,0};
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("set_limit (wrapper)", 0);
  #if !CYTHON_METH_FASTCALL
  #if CYTHON_ASSUME_SAFE_SIZE
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return NULL;
  #endif
  #endif
  __pyx_kwvalues = __Pyx_KwValues_FASTCALL(__pyx_args, __pyx_nargs);
  {
    PyObject ** const __pyx_pyargnames[] = {&__pyx_mstate_global->__pyx_n_u_size,&__pyx_mstate_global->__pyx_n_u_fraction,0};
    const Py_ssize_t __pyx_kwds_len = (__pyx_kwds) ? __Pyx_NumKwargs_FASTCALL(__pyx_kwds) : 0;
    if (unlikely(__pyx_kwds_len < 0)) __PYX_ERR(0, 1966, __pyx_L3_error)
    if (__pyx_kwds_len > 0) {
      switch (__pyx_nargs) {
        case  2:
        values[1] = __Pyx_ArgRef_FASTCALL(__pyx_args, 1);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[1])) __PYX_ERR(0, 1966, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  1:
        values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(0, 1966, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      const Py_ssize_t kwd_pos_args = __pyx_nargs;
      if (__Pyx_ParseKeywords(__pyx_kwds, __pyx_kwvalues, __pyx_pyargnames, 0, values, kwd_pos_args, __pyx_kwds_len, "set_limit", 0) < (0)) __PYX_ERR(0, 1966, __pyx_L3_error)
      if (!values[0]) values[0] = __Pyx_NewRef(((PyObject *)Py_None));
      if (!values[1]) values[1] = __Pyx_NewRef(((PyObject *)Py_None));
    } else {
      switch (__pyx_nargs) {
        case  2:
        values[1] = __Pyx_ArgRef_FASTCALL(__pyx_args, 1);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[1])) __PYX_ERR(0, 1966, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  1:
        values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(0, 1966, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      if (!values[0]) values[0] = __Pyx_NewRef(((PyObject *)Py_None));
      if (!values[1]) values[1] = __Pyx_NewRef(((PyObject *)Py_None));
    }
    __pyx_v_size = values[0];
    __pyx_v_fraction = values[1];
  }
  goto __pyx_L6_skip;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("set_limit", 0, 0, 2, __pyx_nargs); __PYX_ERR(0, 1966, __pyx_L3_error)
  __pyx_L6_skip:;
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_AddTraceback("cupy.cuda.memory.MemoryAsyncPool.set_limit", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_4cupy_4cuda_6memory_15MemoryAsyncPool_14set_limit(((struct __pyx_obj_4cupy_4cuda_6memory_MemoryAsyncPool *)__pyx_v_self), __pyx_v_size, __pyx_v_fraction);

  /* function exit code */
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4cupy_4cuda_6memory_15MemoryAsyncPool_14set_limit(struct __pyx_obj_4cupy_4cuda_6memory_MemoryAsyncPool *__pyx_v_self, PyObject *__pyx_v_size, PyObject *__pyx_v_fraction) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  struct __pyx_opt_args_4cupy_4cuda_6memory_15MemoryAsyncPool_set_limit __pyx_t_2;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("set_limit", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_2.__pyx_n = 2;
  __pyx_t_2.size = __pyx_v_size;
  __pyx_t_2.fraction = __pyx_v_fraction;
  __pyx_t_1 = __pyx_vtabptr_4cupy_4cuda_6memory_MemoryAsyncPool->set_limit(__pyx_v_self, 1, &__pyx_t_2); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1966, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("cupy.cuda.memory.MemoryAsyncPool.set_limit", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "cupy/cuda/memory.pyx":2026
 *             pool, runtime.cudaMemPoolAttrReleaseThreshold, size)
 * 
 *     cpdef size_t get_limit(self):             # <<<<<<<<<<<<<<
 *         """Gets the upper limit of memory allocation of the current device.
 * 
*/

static PyObject *__pyx_pw_4cupy_4cuda_6memory_15MemoryAsyncPool_17get_limit(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
static size_t __pyx_f_4cupy_4cuda_6memory_15MemoryAsyncPool_get_limit(struct __pyx_obj_4cupy_4cuda_6memory_MemoryAsyncPool *__pyx_v_self, int __pyx_skip_dispatch) {
  intptr_t __pyx_v_pool;
  size_t __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  size_t __pyx_t_5;
  int __pyx_t_6;
  intptr_t __pyx_t_7;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("get_limit", 0);
  /* Check if called by wrapper */
  if (unlikely(__pyx_skip_dispatch)) ;
  /* Check if overridden in Python */
  else if (
  #if !CYTHON_USE_TYPE_SLOTS
  unlikely(Py_TYPE(((PyObject *)__pyx_v_self)) != __pyx_mstate_global->__pyx_ptype_4cupy_4cuda_6memory_MemoryAsyncPool &&
  __Pyx_PyType_HasFeature(Py_TYPE(((PyObject *)__pyx_v_self)), Py_TPFLAGS_HAVE_GC))
  #else
  unlikely(Py_TYPE(((PyObject *)__pyx_v_self))->tp_dictoffset != 0 || __Pyx_PyType_HasFeature(Py_TYPE(((PyObject *)__pyx_v_self)), (Py_TPFLAGS_IS_ABSTRACT | Py_TPFLAGS_HEAPTYPE)))
  #endif
  ) {
    #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
    static PY_UINT64_T __pyx_tp_dict_version = __PYX_DICT_VERSION_INIT, __pyx_obj_dict_version = __PYX_DICT_VERSION_INIT;
    if (unlikely(!__Pyx_object_dict_version_matches(((PyObject *)__pyx_v_self), __pyx_tp_dict_version, __pyx_obj_dict_version))) {
      PY_UINT64_T __pyx_typedict_guard = __Pyx_get_tp_dict_version(((PyObject *)__pyx_v_self));
      #endif
      __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_mstate_global->__pyx_n_u_get_limit); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 2026, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      if (!__Pyx_IsSameCFunction(__pyx_t_1, (void(*)(void)) __pyx_pw_4cupy_4cuda_6memory_15MemoryAsyncPool_17get_limit)) {
        __pyx_t_3 = NULL;
        __Pyx_INCREF(__pyx_t_1);
        __pyx_t_4 = __pyx_t_1; 
        __pyx_t_5 = 1;
        #if CYTHON_UNPACK_METHODS
        if (unlikely(PyMethod_Check(__pyx_t_4))) {
          __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_4);
          assert(__pyx_t_3);
          PyObject* __pyx__function = PyMethod_GET_FUNCTION(__pyx_t_4);
          __Pyx_INCREF(__pyx_t_3);
          __Pyx_INCREF(__pyx__function);
          __Pyx_DECREF_SET(__pyx_t_4, __pyx__function);
          __pyx_t_5 = 0;
        }
        #endif
        {
          PyObject *__pyx_callargs[2] = {__pyx_t_3, NULL};
          __pyx_t_2 = __Pyx_PyObject_FastCall((PyObject*)__pyx_t_4, __pyx_callargs+__pyx_t_5, (1-__pyx_t_5) | (__pyx_t_5*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
          __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
          __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
          if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 2026, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_2);
        }
        __pyx_t_5 = __Pyx_PyLong_As_size_t(__pyx_t_2); if (unlikely((__pyx_t_5 == (size_t)-1) && PyErr_Occurred())) __PYX_ERR(0, 2026, __pyx_L1_error)
        __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
        __pyx_r = __pyx_t_5;
        __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
        goto __pyx_L0;
      }
      #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
      __pyx_tp_dict_version = __Pyx_get_tp_dict_version(((PyObject *)__pyx_v_self));
      __pyx_obj_dict_version = __Pyx_get_object_dict_version(((PyObject *)__pyx_v_self));
      if (unlikely(__pyx_typedict_guard != __pyx_tp_dict_version)) {
        __pyx_tp_dict_version = __pyx_obj_dict_version = __PYX_DICT_VERSION_INIT;
      }
      #endif
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
    }
    #endif
  }

  /* "cupy/cuda/memory.pyx":2038
 *             mempool, this limit may not be respected.
 *         """
 *         cdef intptr_t pool = self._pools[device.get_device_id()]             # <<<<<<<<<<<<<<
 *         return runtime.memPoolGetAttribute(
 *             pool, runtime.cudaMemPoolAttrReleaseThreshold)
*/
  if (unlikely(__pyx_v_self->_pools == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(0, 2038, __pyx_L1_error)
  }
  __pyx_t_6 = __pyx_f_4cupy_4cuda_6device_get_device_id(0); if (unlikely(__pyx_t_6 == ((int)-1) && PyErr_Occurred())) __PYX_ERR(0, 2038, __pyx_L1_error)
  __pyx_t_1 = __Pyx_GetItemInt_List(__pyx_v_self->_pools, __pyx_t_6, int, 1, __Pyx_PyLong_From_int, 1, 1, 1, 1, __Pyx_ReferenceSharing_SharedReference); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 2038, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_7 = PyLong_AsSsize_t(__pyx_t_1); if (unlikely((__pyx_t_7 == ((intptr_t)-1)) && PyErr_Occurred())) __PYX_ERR(0, 2038, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v_pool = __pyx_t_7;

  /* "cupy/cuda/memory.pyx":2039
 *         """
 *         cdef intptr_t pool = self._pools[device.get_device_id()]
 *         return runtime.memPoolGetAttribute(             # <<<<<<<<<<<<<<
 *             pool, runtime.cudaMemPoolAttrReleaseThreshold)
 * 
*/
  __pyx_t_1 = __pyx_f_13cupy_backends_4cuda_3api_7runtime_memPoolGetAttribute(__pyx_v_pool, __pyx_e_13cupy_backends_4cuda_3api_13_runtime_enum_cudaMemPoolAttrReleaseThreshold, 0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 2039, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_5 = __Pyx_PyLong_As_size_t(__pyx_t_1); if (unlikely((__pyx_t_5 == (size_t)-1) && PyErr_Occurred())) __PYX_ERR(0, 2039, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_r = __pyx_t_5;
  goto __pyx_L0;

  /* "cupy/cuda/memory.pyx":2026
 *             pool, runtime.cudaMemPoolAttrReleaseThreshold, size)
 * 
 *     cpdef size_t get_limit(self):             # <<<<<<<<<<<<<<
 *         """Gets the upper limit of memory allocation of the current device.
 * 
*/

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_AddTraceback("cupy.cuda.memory.MemoryAsyncPool.get_limit", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_pw_4cupy_4cuda_6memory_15MemoryAsyncPool_17get_limit(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
PyDoc_STRVAR(__pyx_doc_4cupy_4cuda_6memory_15MemoryAsyncPool_16get_limit, "MemoryAsyncPool.get_limit(self) -> size_t\n\nGets the upper limit of memory allocation of the current device.\n\nReturns:\n    int: The number of bytes\n\n.. note::\n    Unlike with :class:`MemoryPool`, :class:`MemoryAsyncPool`'s\n    :meth:`set_limit` method can only impose a *soft* limit. If other\n    (non-CuPy) applications are also allocating memory from the same\n    mempool, this limit may not be respected.");
static PyObject *__pyx_pw_4cupy_4cuda_6memory_15MemoryAsyncPool_17get_limit(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
) {
  #if !CYTHON_METH_FASTCALL
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  #endif
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("get_limit (wrapper)", 0);
  #if !CYTHON_METH_FASTCALL
  #if CYTHON_ASSUME_SAFE_SIZE
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return NULL;
  #endif
  #endif
  __pyx_kwvalues = __Pyx_KwValues_FASTCALL(__pyx_args, __pyx_nargs);
  if (unlikely(__pyx_nargs > 0)) { __Pyx_RaiseArgtupleInvalid("get_limit", 1, 0, 0, __pyx_nargs); return NULL; }
  const Py_ssize_t __pyx_kwds_len = unlikely(__pyx_kwds) ? __Pyx_NumKwargs_FASTCALL(__pyx_kwds) : 0;
  if (unlikely(__pyx_kwds_len < 0)) return NULL;
  if (unlikely(__pyx_kwds_len > 0)) {__Pyx_RejectKeywords("get_limit", __pyx_kwds); return NULL;}
  __pyx_r = __pyx_pf_4cupy_4cuda_6memory_15MemoryAsyncPool_16get_limit(((struct __pyx_obj_4cupy_4cuda_6memory_MemoryAsyncPool *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4cupy_4cuda_6memory_15MemoryAsyncPool_16get_limit(struct __pyx_obj_4cupy_4cuda_6memory_MemoryAsyncPool *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  size_t __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("get_limit", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __pyx_f_4cupy_4cuda_6memory_15MemoryAsyncPool_get_limit(__pyx_v_self, 1); if (unlikely(PyErr_Occurred())) __PYX_ERR(0, 2026, __pyx_L1_error)
  __pyx_t_2 = __Pyx_PyLong_FromSize_t(__pyx_t_1); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 2026, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_r = __pyx_t_2;
  __pyx_t_2 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_AddTraceback("cupy.cuda.memory.MemoryAsyncPool.get_limit", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "cupy/cuda/memory.pyx":1792
 *     cdef:
 *         # A list of cudaMemPool_t to each device's mempool
 *         readonly list _pools             # <<<<<<<<<<<<<<
 *         readonly bint memoryAsyncHasStat
 * 
*/

/* Python wrapper */
static PyObject *__pyx_pw_4cupy_4cuda_6memory_15MemoryAsyncPool_6_pools_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_4cupy_4cuda_6memory_15MemoryAsyncPool_6_pools_1__get__(PyObject *__pyx_v_self) {
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_kwvalues = __Pyx_KwValues_VARARGS(__pyx_args, __pyx_nargs);
  __pyx_r = __pyx_pf_4cupy_4cuda_6memory_15MemoryAsyncPool_6_pools___get__(((struct __pyx_obj_4cupy_4cuda_6memory_MemoryAsyncPool *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4cupy_4cuda_6memory_15MemoryAsyncPool_6_pools___get__(struct __pyx_obj_4cupy_4cuda_6memory_MemoryAsyncPool *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__", 0);
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_v_self->_pools);
  __pyx_r = __pyx_v_self->_pools;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "cupy/cuda/memory.pyx":1793
 *         # A list of cudaMemPool_t to each device's mempool
 *         readonly list _pools
 *         readonly bint memoryAsyncHasStat             # <<<<<<<<<<<<<<
 * 
 *     def __init__(self, pool_handles='current'):
*/

/* Python wrapper */
static PyObject *__pyx_pw_4cupy_4cuda_6memory_15MemoryAsyncPool_18memoryAsyncHasStat_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_4cupy_4cuda_6memory_15MemoryAsyncPool_18memoryAsyncHasStat_1__get__(PyObject *__pyx_v_self) {
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_kwvalues = __Pyx_KwValues_VARARGS(__pyx_args, __pyx_nargs);
  __pyx_r = __pyx_pf_4cupy_4cuda_6memory_15MemoryAsyncPool_18memoryAsyncHasStat___get__(((struct __pyx_obj_4cupy_4cuda_6memory_MemoryAsyncPool *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4cupy_4cuda_6memory_15MemoryAsyncPool_18memoryAsyncHasStat___get__(struct __pyx_obj_4cupy_4cuda_6memory_MemoryAsyncPool *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__get__", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __Pyx_PyBool_FromLong(__pyx_v_self->memoryAsyncHasStat); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1793, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("cupy.cuda.memory.MemoryAsyncPool.memoryAsyncHasStat.__get__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     cdef tuple state
 *     cdef object _dict
*/

/* Python wrapper */
static PyObject *__pyx_pw_4cupy_4cuda_6memory_15MemoryAsyncPool_19__reduce_cython__(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
PyDoc_STRVAR(__pyx_doc_4cupy_4cuda_6memory_15MemoryAsyncPool_18__reduce_cython__, "MemoryAsyncPool.__reduce_cython__(self)");
static PyObject *__pyx_pw_4cupy_4cuda_6memory_15MemoryAsyncPool_19__reduce_cython__(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
) {
  #if !CYTHON_METH_FASTCALL
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  #endif
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__reduce_cython__ (wrapper)", 0);
  #if !CYTHON_METH_FASTCALL
  #if CYTHON_ASSUME_SAFE_SIZE
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return NULL;
  #endif
  #endif
  __pyx_kwvalues = __Pyx_KwValues_FASTCALL(__pyx_args, __pyx_nargs);
  if (unlikely(__pyx_nargs > 0)) { __Pyx_RaiseArgtupleInvalid("__reduce_cython__", 1, 0, 0, __pyx_nargs); return NULL; }
  const Py_ssize_t __pyx_kwds_len = unlikely(__pyx_kwds) ? __Pyx_NumKwargs_FASTCALL(__pyx_kwds) : 0;
  if (unlikely(__pyx_kwds_len < 0)) return NULL;
  if (unlikely(__pyx_kwds_len > 0)) {__Pyx_RejectKeywords("__reduce_cython__", __pyx_kwds); return NULL;}
  __pyx_r = __pyx_pf_4cupy_4cuda_6memory_15MemoryAsyncPool_18__reduce_cython__(((struct __pyx_obj_4cupy_4cuda_6memory_MemoryAsyncPool *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4cupy_4cuda_6memory_15MemoryAsyncPool_18__reduce_cython__(struct __pyx_obj_4cupy_4cuda_6memory_MemoryAsyncPool *__pyx_v_self) {
  PyObject *__pyx_v_state = 0;
  PyObject *__pyx_v__dict = 0;
  int __pyx_v_use_setstate;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  int __pyx_t_3;
  int __pyx_t_4;
  PyObject *__pyx_t_5 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__reduce_cython__", 0);

  /* "(tree fragment)":5
 *     cdef object _dict
 *     cdef bint use_setstate
 *     state = (self._pools, self.memoryAsyncHasStat)             # <<<<<<<<<<<<<<
 *     _dict = getattr(self, '__dict__', None)
 *     if _dict is not None and _dict:
*/
  __pyx_t_1 = __Pyx_PyBool_FromLong(__pyx_v_self->memoryAsyncHasStat); if (unlikely(!__pyx_t_1)) __PYX_ERR(3, 5, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = PyTuple_New(2); if (unlikely(!__pyx_t_2)) __PYX_ERR(3, 5, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_INCREF(__pyx_v_self->_pools);
  __Pyx_GIVEREF(__pyx_v_self->_pools);
  if (__Pyx_PyTuple_SET_ITEM(__pyx_t_2, 0, __pyx_v_self->_pools) != (0)) __PYX_ERR(3, 5, __pyx_L1_error);
  __Pyx_GIVEREF(__pyx_t_1);
  if (__Pyx_PyTuple_SET_ITEM(__pyx_t_2, 1, __pyx_t_1) != (0)) __PYX_ERR(3, 5, __pyx_L1_error);
  __pyx_t_1 = 0;
  __pyx_v_state = ((PyObject*)__pyx_t_2);
  __pyx_t_2 = 0;

  /* "(tree fragment)":6
 *     cdef bint use_setstate
 *     state = (self._pools, self.memoryAsyncHasStat)
 *     _dict = getattr(self, '__dict__', None)             # <<<<<<<<<<<<<<
 *     if _dict is not None and _dict:
 *         state += (_dict,)
*/
  __pyx_t_2 = __Pyx_GetAttr3(((PyObject *)__pyx_v_self), __pyx_mstate_global->__pyx_n_u_dict, Py_None); if (unlikely(!__pyx_t_2)) __PYX_ERR(3, 6, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_v__dict = __pyx_t_2;
  __pyx_t_2 = 0;

  /* "(tree fragment)":7
 *     state = (self._pools, self.memoryAsyncHasStat)
 *     _dict = getattr(self, '__dict__', None)
 *     if _dict is not None and _dict:             # <<<<<<<<<<<<<<
 *         state += (_dict,)
 *         use_setstate = True
*/
  __pyx_t_4 = (__pyx_v__dict != Py_None);
  if (__pyx_t_4) {
  } else {
    __pyx_t_3 = __pyx_t_4;
    goto __pyx_L4_bool_binop_done;
  }
  __pyx_t_4 = __Pyx_PyObject_IsTrue(__pyx_v__dict); if (unlikely((__pyx_t_4 < 0))) __PYX_ERR(3, 7, __pyx_L1_error)
  __pyx_t_3 = __pyx_t_4;
  __pyx_L4_bool_binop_done:;
  if (__pyx_t_3) {

    /* "(tree fragment)":8
 *     _dict = getattr(self, '__dict__', None)
 *     if _dict is not None and _dict:
 *         state += (_dict,)             # <<<<<<<<<<<<<<
 *         use_setstate = True
 *     else:
*/
    __pyx_t_2 = PyTuple_New(1); if (unlikely(!__pyx_t_2)) __PYX_ERR(3, 8, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_INCREF(__pyx_v__dict);
    __Pyx_GIVEREF(__pyx_v__dict);
    if (__Pyx_PyTuple_SET_ITEM(__pyx_t_2, 0, __pyx_v__dict) != (0)) __PYX_ERR(3, 8, __pyx_L1_error);
    __pyx_t_1 = PyNumber_InPlaceAdd(__pyx_v_state, __pyx_t_2); if (unlikely(!__pyx_t_1)) __PYX_ERR(3, 8, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_DECREF_SET(__pyx_v_state, ((PyObject*)__pyx_t_1));
    __pyx_t_1 = 0;

    /* "(tree fragment)":9
 *     if _dict is not None and _dict:
 *         state += (_dict,)
 *         use_setstate = True             # <<<<<<<<<<<<<<
 *     else:
 *         use_setstate = self._pools is not None
*/
    __pyx_v_use_setstate = 1;

    /* "(tree fragment)":7
 *     state = (self._pools, self.memoryAsyncHasStat)
 *     _dict = getattr(self, '__dict__', None)
 *     if _dict is not None and _dict:             # <<<<<<<<<<<<<<
 *         state += (_dict,)
 *         use_setstate = True
*/
    goto __pyx_L3;
  }

  /* "(tree fragment)":11
 *         use_setstate = True
 *     else:
 *         use_setstate = self._pools is not None             # <<<<<<<<<<<<<<
 *     if use_setstate:
 *         return __pyx_unpickle_MemoryAsyncPool, (type(self), 0xe511d52, None), state
*/
  /*else*/ {
    __pyx_t_3 = (__pyx_v_self->_pools != ((PyObject*)Py_None));
    __pyx_v_use_setstate = __pyx_t_3;
  }
  __pyx_L3:;

  /* "(tree fragment)":12
 *     else:
 *         use_setstate = self._pools is not None
 *     if use_setstate:             # <<<<<<<<<<<<<<
 *         return __pyx_unpickle_MemoryAsyncPool, (type(self), 0xe511d52, None), state
 *     else:
*/
  if (__pyx_v_use_setstate) {

    /* "(tree fragment)":13
 *         use_setstate = self._pools is not None
 *     if use_setstate:
 *         return __pyx_unpickle_MemoryAsyncPool, (type(self), 0xe511d52, None), state             # <<<<<<<<<<<<<<
 *     else:
 *         return __pyx_unpickle_MemoryAsyncPool, (type(self), 0xe511d52, state)
*/
    __Pyx_XDECREF(__pyx_r);
    __Pyx_GetModuleGlobalName(__pyx_t_1, __pyx_mstate_global->__pyx_n_u_pyx_unpickle_MemoryAsyncPool); if (unlikely(!__pyx_t_1)) __PYX_ERR(3, 13, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_t_2 = PyTuple_New(3); if (unlikely(!__pyx_t_2)) __PYX_ERR(3, 13, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_INCREF(((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    __Pyx_GIVEREF(((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    if (__Pyx_PyTuple_SET_ITEM(__pyx_t_2, 0, ((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self)))) != (0)) __PYX_ERR(3, 13, __pyx_L1_error);
    __Pyx_INCREF(__pyx_mstate_global->__pyx_int_240196946);
    __Pyx_GIVEREF(__pyx_mstate_global->__pyx_int_240196946);
    if (__Pyx_PyTuple_SET_ITEM(__pyx_t_2, 1, __pyx_mstate_global->__pyx_int_240196946) != (0)) __PYX_ERR(3, 13, __pyx_L1_error);
    __Pyx_INCREF(Py_None);
    __Pyx_GIVEREF(Py_None);
    if (__Pyx_PyTuple_SET_ITEM(__pyx_t_2, 2, Py_None) != (0)) __PYX_ERR(3, 13, __pyx_L1_error);
    __pyx_t_5 = PyTuple_New(3); if (unlikely(!__pyx_t_5)) __PYX_ERR(3, 13, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __Pyx_GIVEREF(__pyx_t_1);
    if (__Pyx_PyTuple_SET_ITEM(__pyx_t_5, 0, __pyx_t_1) != (0)) __PYX_ERR(3, 13, __pyx_L1_error);
    __Pyx_GIVEREF(__pyx_t_2);
    if (__Pyx_PyTuple_SET_ITEM(__pyx_t_5, 1, __pyx_t_2) != (0)) __PYX_ERR(3, 13, __pyx_L1_error);
    __Pyx_INCREF(__pyx_v_state);
    __Pyx_GIVEREF(__pyx_v_state);
    if (__Pyx_PyTuple_SET_ITEM(__pyx_t_5, 2, __pyx_v_state) != (0)) __PYX_ERR(3, 13, __pyx_L1_error);
    __pyx_t_1 = 0;
    __pyx_t_2 = 0;
    __pyx_r = __pyx_t_5;
    __pyx_t_5 = 0;
    goto __pyx_L0;

    /* "(tree fragment)":12
 *     else:
 *         use_setstate = self._pools is not None
 *     if use_setstate:             # <<<<<<<<<<<<<<
 *         return __pyx_unpickle_MemoryAsyncPool, (type(self), 0xe511d52, None), state
 *     else:
*/
  }

  /* "(tree fragment)":15
 *         return __pyx_unpickle_MemoryAsyncPool, (type(self), 0xe511d52, None), state
 *     else:
 *         return __pyx_unpickle_MemoryAsyncPool, (type(self), 0xe511d52, state)             # <<<<<<<<<<<<<<
 * def __setstate_cython__(self, __pyx_state):
 *     __pyx_unpickle_MemoryAsyncPool__set_state(self, __pyx_state)
*/
  /*else*/ {
    __Pyx_XDECREF(__pyx_r);
    __Pyx_GetModuleGlobalName(__pyx_t_5, __pyx_mstate_global->__pyx_n_u_pyx_unpickle_MemoryAsyncPool); if (unlikely(!__pyx_t_5)) __PYX_ERR(3, 15, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __pyx_t_2 = PyTuple_New(3); if (unlikely(!__pyx_t_2)) __PYX_ERR(3, 15, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_INCREF(((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    __Pyx_GIVEREF(((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    if (__Pyx_PyTuple_SET_ITEM(__pyx_t_2, 0, ((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self)))) != (0)) __PYX_ERR(3, 15, __pyx_L1_error);
    __Pyx_INCREF(__pyx_mstate_global->__pyx_int_240196946);
    __Pyx_GIVEREF(__pyx_mstate_global->__pyx_int_240196946);
    if (__Pyx_PyTuple_SET_ITEM(__pyx_t_2, 1, __pyx_mstate_global->__pyx_int_240196946) != (0)) __PYX_ERR(3, 15, __pyx_L1_error);
    __Pyx_INCREF(__pyx_v_state);
    __Pyx_GIVEREF(__pyx_v_state);
    if (__Pyx_PyTuple_SET_ITEM(__pyx_t_2, 2, __pyx_v_state) != (0)) __PYX_ERR(3, 15, __pyx_L1_error);
    __pyx_t_1 = PyTuple_New(2); if (unlikely(!__pyx_t_1)) __PYX_ERR(3, 15, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_GIVEREF(__pyx_t_5);
    if (__Pyx_PyTuple_SET_ITEM(__pyx_t_1, 0, __pyx_t_5) != (0)) __PYX_ERR(3, 15, __pyx_L1_error);
    __Pyx_GIVEREF(__pyx_t_2);
    if (__Pyx_PyTuple_SET_ITEM(__pyx_t_1, 1, __pyx_t_2) != (0)) __PYX_ERR(3, 15, __pyx_L1_error);
    __pyx_t_5 = 0;
    __pyx_t_2 = 0;
    __pyx_r = __pyx_t_1;
    __pyx_t_1 = 0;
    goto __pyx_L0;
  }

  /* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     cdef tuple state
 *     cdef object _dict
*/

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_AddTraceback("cupy.cuda.memory.MemoryAsyncPool.__reduce_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_state);
  __Pyx_XDECREF(__pyx_v__dict);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":16
 *     else:
 *         return __pyx_unpickle_MemoryAsyncPool, (type(self), 0xe511d52, state)
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     __pyx_unpickle_MemoryAsyncPool__set_state(self, __pyx_state)
*/

/* Python wrapper */
static PyObject *__pyx_pw_4cupy_4cuda_6memory_15MemoryAsyncPool_21__setstate_cython__(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
PyDoc_STRVAR(__pyx_doc_4cupy_4cuda_6memory_15MemoryAsyncPool_20__setstate_cython__, "MemoryAsyncPool.__setstate_cython__(self, __pyx_state)");
static PyObject *__pyx_pw_4cupy_4cuda_6memory_15MemoryAsyncPool_21__setstate_cython__(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
) {
  PyObject *__pyx_v___pyx_state = 0;
  #if !CYTHON_METH_FASTCALL
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  #endif
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject* values[1] = {0};
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__setstate_cython__ (wrapper)", 0);
  #if !CYTHON_METH_FASTCALL
  #if CYTHON_ASSUME_SAFE_SIZE
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return NULL;
  #endif
  #endif
  __pyx_kwvalues = __Pyx_KwValues_FASTCALL(__pyx_args, __pyx_nargs);
  {
    PyObject ** const __pyx_pyargnames[] = {&__pyx_mstate_global->__pyx_n_u_pyx_state,0};
    const Py_ssize_t __pyx_kwds_len = (__pyx_kwds) ? __Pyx_NumKwargs_FASTCALL(__pyx_kwds) : 0;
    if (unlikely(__pyx_kwds_len < 0)) __PYX_ERR(3, 16, __pyx_L3_error)
    if (__pyx_kwds_len > 0) {
      switch (__pyx_nargs) {
        case  1:
        values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(3, 16, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      const Py_ssize_t kwd_pos_args = __pyx_nargs;
      if (__Pyx_ParseKeywords(__pyx_kwds, __pyx_kwvalues, __pyx_pyargnames, 0, values, kwd_pos_args, __pyx_kwds_len, "__setstate_cython__", 0) < (0)) __PYX_ERR(3, 16, __pyx_L3_error)
      for (Py_ssize_t i = __pyx_nargs; i < 1; i++) {
        if (unlikely(!values[i])) { __Pyx_RaiseArgtupleInvalid("__setstate_cython__", 1, 1, 1, i); __PYX_ERR(3, 16, __pyx_L3_error) }
      }
    } else if (unlikely(__pyx_nargs != 1)) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
      if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(3, 16, __pyx_L3_error)
    }
    __pyx_v___pyx_state = values[0];
  }
  goto __pyx_L6_skip;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("__setstate_cython__", 1, 1, 1, __pyx_nargs); __PYX_ERR(3, 16, __pyx_L3_error)
  __pyx_L6_skip:;
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_AddTraceback("cupy.cuda.memory.MemoryAsyncPool.__setstate_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_4cupy_4cuda_6memory_15MemoryAsyncPool_20__setstate_cython__(((struct __pyx_obj_4cupy_4cuda_6memory_MemoryAsyncPool *)__pyx_v_self), __pyx_v___pyx_state);

  /* function exit code */
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4cupy_4cuda_6memory_15MemoryAsyncPool_20__setstate_cython__(struct __pyx_obj_4cupy_4cuda_6memory_MemoryAsyncPool *__pyx_v_self, PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__setstate_cython__", 0);

  /* "(tree fragment)":17
 *         return __pyx_unpickle_MemoryAsyncPool, (type(self), 0xe511d52, state)
 * def __setstate_cython__(self, __pyx_state):
 *     __pyx_unpickle_MemoryAsyncPool__set_state(self, __pyx_state)             # <<<<<<<<<<<<<<
*/
  __pyx_t_1 = __pyx_v___pyx_state;
  __Pyx_INCREF(__pyx_t_1);
  if (!(likely(PyTuple_CheckExact(__pyx_t_1))||((__pyx_t_1) == Py_None) || __Pyx_RaiseUnexpectedTypeError("tuple", __pyx_t_1))) __PYX_ERR(3, 17, __pyx_L1_error)
  if (unlikely(__pyx_t_1 == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "cannot pass None into a C function argument that is declared 'not None'");
    __PYX_ERR(3, 17, __pyx_L1_error)
  }
  __pyx_t_2 = __pyx_f_4cupy_4cuda_6memory___pyx_unpickle_MemoryAsyncPool__set_state(__pyx_v_self, ((PyObject*)__pyx_t_1)); if (unlikely(!__pyx_t_2)) __PYX_ERR(3, 17, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "(tree fragment)":16
 *     else:
 *         return __pyx_unpickle_MemoryAsyncPool, (type(self), 0xe511d52, state)
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     __pyx_unpickle_MemoryAsyncPool__set_state(self, __pyx_state)
*/

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_AddTraceback("cupy.cuda.memory.MemoryAsyncPool.__setstate_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "cupy/cuda/memory.pyx":2047
 * 
 * 
 * cdef intptr_t _call_malloc(             # <<<<<<<<<<<<<<
 *         intptr_t param, intptr_t malloc_func, Py_ssize_t size,
 *         int device_id) except *:
*/

static intptr_t __pyx_f_4cupy_4cuda_6memory__call_malloc(intptr_t __pyx_v_param, intptr_t __pyx_v_malloc_func, Py_ssize_t __pyx_v_size, int __pyx_v_device_id) {
  intptr_t __pyx_r;
  void *__pyx_t_1;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;

  /* "cupy/cuda/memory.pyx":2051
 *         int device_id) except *:
 *     return <intptr_t>(
 *         (<malloc_func_type>malloc_func)(<void*>param, size, device_id))             # <<<<<<<<<<<<<<
 * 
 * 
*/
  try {
    __pyx_t_1 = ((__pyx_t_4cupy_4cuda_6memory_malloc_func_type)__pyx_v_malloc_func)(((void *)__pyx_v_param), __pyx_v_size, __pyx_v_device_id);
  } catch(...) {
    __Pyx_CppExn2PyErr();
    __PYX_ERR(0, 2051, __pyx_L1_error)
  }

  /* "cupy/cuda/memory.pyx":2050
 *         intptr_t param, intptr_t malloc_func, Py_ssize_t size,
 *         int device_id) except *:
 *     return <intptr_t>(             # <<<<<<<<<<<<<<
 *         (<malloc_func_type>malloc_func)(<void*>param, size, device_id))
 * 
*/
  __pyx_r = ((intptr_t)__pyx_t_1);
  goto __pyx_L0;

  /* "cupy/cuda/memory.pyx":2047
 * 
 * 
 * cdef intptr_t _call_malloc(             # <<<<<<<<<<<<<<
 *         intptr_t param, intptr_t malloc_func, Py_ssize_t size,
 *         int device_id) except *:
*/

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_AddTraceback("cupy.cuda.memory._call_malloc", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1L;
  __pyx_L0:;
  return __pyx_r;
}

/* "cupy/cuda/memory.pyx":2054
 * 
 * 
 * cdef void _call_free(             # <<<<<<<<<<<<<<
 *         intptr_t param, intptr_t free_func, intptr_t ptr,
 *         int device_id) noexcept:
*/

static void __pyx_f_4cupy_4cuda_6memory__call_free(intptr_t __pyx_v_param, intptr_t __pyx_v_free_func, intptr_t __pyx_v_ptr, int __pyx_v_device_id) {

  /* "cupy/cuda/memory.pyx":2057
 *         intptr_t param, intptr_t free_func, intptr_t ptr,
 *         int device_id) noexcept:
 *     (<free_func_type>free_func)(<void*>param, <void*>ptr, device_id)             # <<<<<<<<<<<<<<
 * 
 * 
*/
  ((__pyx_t_4cupy_4cuda_6memory_free_func_type)__pyx_v_free_func)(((void *)__pyx_v_param), ((void *)__pyx_v_ptr), __pyx_v_device_id);

  /* "cupy/cuda/memory.pyx":2054
 * 
 * 
 * cdef void _call_free(             # <<<<<<<<<<<<<<
 *         intptr_t param, intptr_t free_func, intptr_t ptr,
 *         int device_id) noexcept:
*/

  /* function exit code */
}

/* "cupy/cuda/memory.pyx":2063
 * cdef class CFunctionAllocatorMemory(BaseMemory):
 * 
 *     def __init__(self, size_t size, intptr_t param,             # <<<<<<<<<<<<<<
 *                  intptr_t malloc_func, intptr_t free_func,
 *                  int device_id):
*/

/* Python wrapper */
static int __pyx_pw_4cupy_4cuda_6memory_24CFunctionAllocatorMemory_1__init__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static int __pyx_pw_4cupy_4cuda_6memory_24CFunctionAllocatorMemory_1__init__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  size_t __pyx_v_size;
  intptr_t __pyx_v_param;
  intptr_t __pyx_v_malloc_func;
  intptr_t __pyx_v_free_func;
  int __pyx_v_device_id;
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject* values[5] = {0,0,0,0,0};
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__init__ (wrapper)", 0);
  #if CYTHON_ASSUME_SAFE_SIZE
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return -1;
  #endif
  __pyx_kwvalues = __Pyx_KwValues_VARARGS(__pyx_args, __pyx_nargs);
  {
    PyObject ** const __pyx_pyargnames[] = {&__pyx_mstate_global->__pyx_n_u_size,&__pyx_mstate_global->__pyx_n_u_param,&__pyx_mstate_global->__pyx_n_u_malloc_func,&__pyx_mstate_global->__pyx_n_u_free_func,&__pyx_mstate_global->__pyx_n_u_device_id,0};
    const Py_ssize_t __pyx_kwds_len = (__pyx_kwds) ? __Pyx_NumKwargs_VARARGS(__pyx_kwds) : 0;
    if (unlikely(__pyx_kwds_len < 0)) __PYX_ERR(0, 2063, __pyx_L3_error)
    if (__pyx_kwds_len > 0) {
      switch (__pyx_nargs) {
        case  5:
        values[4] = __Pyx_ArgRef_VARARGS(__pyx_args, 4);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[4])) __PYX_ERR(0, 2063, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  4:
        values[3] = __Pyx_ArgRef_VARARGS(__pyx_args, 3);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[3])) __PYX_ERR(0, 2063, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  3:
        values[2] = __Pyx_ArgRef_VARARGS(__pyx_args, 2);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[2])) __PYX_ERR(0, 2063, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  2:
        values[1] = __Pyx_ArgRef_VARARGS(__pyx_args, 1);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[1])) __PYX_ERR(0, 2063, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  1:
        values[0] = __Pyx_ArgRef_VARARGS(__pyx_args, 0);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(0, 2063, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      const Py_ssize_t kwd_pos_args = __pyx_nargs;
      if (__Pyx_ParseKeywords(__pyx_kwds, __pyx_kwvalues, __pyx_pyargnames, 0, values, kwd_pos_args, __pyx_kwds_len, "__init__", 0) < (0)) __PYX_ERR(0, 2063, __pyx_L3_error)
      for (Py_ssize_t i = __pyx_nargs; i < 5; i++) {
        if (unlikely(!values[i])) { __Pyx_RaiseArgtupleInvalid("__init__", 1, 5, 5, i); __PYX_ERR(0, 2063, __pyx_L3_error) }
      }
    } else if (unlikely(__pyx_nargs != 5)) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = __Pyx_ArgRef_VARARGS(__pyx_args, 0);
      if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(0, 2063, __pyx_L3_error)
      values[1] = __Pyx_ArgRef_VARARGS(__pyx_args, 1);
      if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[1])) __PYX_ERR(0, 2063, __pyx_L3_error)
      values[2] = __Pyx_ArgRef_VARARGS(__pyx_args, 2);
      if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[2])) __PYX_ERR(0, 2063, __pyx_L3_error)
      values[3] = __Pyx_ArgRef_VARARGS(__pyx_args, 3);
      if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[3])) __PYX_ERR(0, 2063, __pyx_L3_error)
      values[4] = __Pyx_ArgRef_VARARGS(__pyx_args, 4);
      if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[4])) __PYX_ERR(0, 2063, __pyx_L3_error)
    }
    __pyx_v_size = __Pyx_PyLong_As_size_t(values[0]); if (unlikely((__pyx_v_size == (size_t)-1) && PyErr_Occurred())) __PYX_ERR(0, 2063, __pyx_L3_error)
    __pyx_v_param = PyLong_AsSsize_t(values[1]); if (unlikely((__pyx_v_param == ((intptr_t)-1)) && PyErr_Occurred())) __PYX_ERR(0, 2063, __pyx_L3_error)
    __pyx_v_malloc_func = PyLong_AsSsize_t(values[2]); if (unlikely((__pyx_v_malloc_func == ((intptr_t)-1)) && PyErr_Occurred())) __PYX_ERR(0, 2064, __pyx_L3_error)
    __pyx_v_free_func = PyLong_AsSsize_t(values[3]); if (unlikely((__pyx_v_free_func == ((intptr_t)-1)) && PyErr_Occurred())) __PYX_ERR(0, 2064, __pyx_L3_error)
    __pyx_v_device_id = __Pyx_PyLong_As_int(values[4]); if (unlikely((__pyx_v_device_id == (int)-1) && PyErr_Occurred())) __PYX_ERR(0, 2065, __pyx_L3_error)
  }
  goto __pyx_L6_skip;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("__init__", 1, 5, 5, __pyx_nargs); __PYX_ERR(0, 2063, __pyx_L3_error)
  __pyx_L6_skip:;
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_AddTraceback("cupy.cuda.memory.CFunctionAllocatorMemory.__init__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return -1;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_4cupy_4cuda_6memory_24CFunctionAllocatorMemory___init__(((struct __pyx_obj_4cupy_4cuda_6memory_CFunctionAllocatorMemory *)__pyx_v_self), __pyx_v_size, __pyx_v_param, __pyx_v_malloc_func, __pyx_v_free_func, __pyx_v_device_id);

  /* function exit code */
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_4cupy_4cuda_6memory_24CFunctionAllocatorMemory___init__(struct __pyx_obj_4cupy_4cuda_6memory_CFunctionAllocatorMemory *__pyx_v_self, size_t __pyx_v_size, intptr_t __pyx_v_param, intptr_t __pyx_v_malloc_func, intptr_t __pyx_v_free_func, int __pyx_v_device_id) {
  int __pyx_r;
  int __pyx_t_1;
  intptr_t __pyx_t_2;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;

  /* "cupy/cuda/memory.pyx":2066
 *                  intptr_t malloc_func, intptr_t free_func,
 *                  int device_id):
 *         self._param = param             # <<<<<<<<<<<<<<
 *         self._free_func = free_func
 *         self.device_id = device_id
*/
  __pyx_v_self->_param = __pyx_v_param;

  /* "cupy/cuda/memory.pyx":2067
 *                  int device_id):
 *         self._param = param
 *         self._free_func = free_func             # <<<<<<<<<<<<<<
 *         self.device_id = device_id
 *         self.size = size
*/
  __pyx_v_self->_free_func = __pyx_v_free_func;

  /* "cupy/cuda/memory.pyx":2068
 *         self._param = param
 *         self._free_func = free_func
 *         self.device_id = device_id             # <<<<<<<<<<<<<<
 *         self.size = size
 *         self.ptr = 0
*/
  __pyx_v_self->__pyx_base.device_id = __pyx_v_device_id;

  /* "cupy/cuda/memory.pyx":2069
 *         self._free_func = free_func
 *         self.device_id = device_id
 *         self.size = size             # <<<<<<<<<<<<<<
 *         self.ptr = 0
 *         if size > 0:
*/
  __pyx_v_self->__pyx_base.size = __pyx_v_size;

  /* "cupy/cuda/memory.pyx":2070
 *         self.device_id = device_id
 *         self.size = size
 *         self.ptr = 0             # <<<<<<<<<<<<<<
 *         if size > 0:
 *             self.ptr = _call_malloc(param, malloc_func, size, device_id)
*/
  __pyx_v_self->__pyx_base.ptr = 0;

  /* "cupy/cuda/memory.pyx":2071
 *         self.size = size
 *         self.ptr = 0
 *         if size > 0:             # <<<<<<<<<<<<<<
 *             self.ptr = _call_malloc(param, malloc_func, size, device_id)
 * 
*/
  __pyx_t_1 = (__pyx_v_size > 0);
  if (__pyx_t_1) {

    /* "cupy/cuda/memory.pyx":2072
 *         self.ptr = 0
 *         if size > 0:
 *             self.ptr = _call_malloc(param, malloc_func, size, device_id)             # <<<<<<<<<<<<<<
 * 
 *     def __dealloc__(self):
*/
    __pyx_t_2 = __pyx_f_4cupy_4cuda_6memory__call_malloc(__pyx_v_param, __pyx_v_malloc_func, __pyx_v_size, __pyx_v_device_id); if (unlikely(__pyx_t_2 == ((intptr_t)-1L) && PyErr_Occurred())) __PYX_ERR(0, 2072, __pyx_L1_error)
    __pyx_v_self->__pyx_base.ptr = __pyx_t_2;

    /* "cupy/cuda/memory.pyx":2071
 *         self.size = size
 *         self.ptr = 0
 *         if size > 0:             # <<<<<<<<<<<<<<
 *             self.ptr = _call_malloc(param, malloc_func, size, device_id)
 * 
*/
  }

  /* "cupy/cuda/memory.pyx":2063
 * cdef class CFunctionAllocatorMemory(BaseMemory):
 * 
 *     def __init__(self, size_t size, intptr_t param,             # <<<<<<<<<<<<<<
 *                  intptr_t malloc_func, intptr_t free_func,
 *                  int device_id):
*/

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_AddTraceback("cupy.cuda.memory.CFunctionAllocatorMemory.__init__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  return __pyx_r;
}

/* "cupy/cuda/memory.pyx":2074
 *             self.ptr = _call_malloc(param, malloc_func, size, device_id)
 * 
 *     def __dealloc__(self):             # <<<<<<<<<<<<<<
 *         if self.ptr:
 *             _call_free(self._param, self._free_func, self.ptr, self.device_id)
*/

/* Python wrapper */
static void __pyx_pw_4cupy_4cuda_6memory_24CFunctionAllocatorMemory_3__dealloc__(PyObject *__pyx_v_self); /*proto*/
static void __pyx_pw_4cupy_4cuda_6memory_24CFunctionAllocatorMemory_3__dealloc__(PyObject *__pyx_v_self) {
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__dealloc__ (wrapper)", 0);
  __pyx_kwvalues = __Pyx_KwValues_VARARGS(__pyx_args, __pyx_nargs);
  __pyx_pf_4cupy_4cuda_6memory_24CFunctionAllocatorMemory_2__dealloc__(((struct __pyx_obj_4cupy_4cuda_6memory_CFunctionAllocatorMemory *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
}

static void __pyx_pf_4cupy_4cuda_6memory_24CFunctionAllocatorMemory_2__dealloc__(struct __pyx_obj_4cupy_4cuda_6memory_CFunctionAllocatorMemory *__pyx_v_self) {
  int __pyx_t_1;

  /* "cupy/cuda/memory.pyx":2075
 * 
 *     def __dealloc__(self):
 *         if self.ptr:             # <<<<<<<<<<<<<<
 *             _call_free(self._param, self._free_func, self.ptr, self.device_id)
 * 
*/
  __pyx_t_1 = (__pyx_v_self->__pyx_base.ptr != 0);
  if (__pyx_t_1) {

    /* "cupy/cuda/memory.pyx":2076
 *     def __dealloc__(self):
 *         if self.ptr:
 *             _call_free(self._param, self._free_func, self.ptr, self.device_id)             # <<<<<<<<<<<<<<
 * 
 * 
*/
    __pyx_f_4cupy_4cuda_6memory__call_free(__pyx_v_self->_param, __pyx_v_self->_free_func, __pyx_v_self->__pyx_base.ptr, __pyx_v_self->__pyx_base.device_id);

    /* "cupy/cuda/memory.pyx":2075
 * 
 *     def __dealloc__(self):
 *         if self.ptr:             # <<<<<<<<<<<<<<
 *             _call_free(self._param, self._free_func, self.ptr, self.device_id)
 * 
*/
  }

  /* "cupy/cuda/memory.pyx":2074
 *             self.ptr = _call_malloc(param, malloc_func, size, device_id)
 * 
 *     def __dealloc__(self):             # <<<<<<<<<<<<<<
 *         if self.ptr:
 *             _call_free(self._param, self._free_func, self.ptr, self.device_id)
*/

  /* function exit code */
}

/* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     cdef tuple state
 *     cdef object _dict
*/

/* Python wrapper */
static PyObject *__pyx_pw_4cupy_4cuda_6memory_24CFunctionAllocatorMemory_5__reduce_cython__(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
PyDoc_STRVAR(__pyx_doc_4cupy_4cuda_6memory_24CFunctionAllocatorMemory_4__reduce_cython__, "CFunctionAllocatorMemory.__reduce_cython__(self)");
static PyObject *__pyx_pw_4cupy_4cuda_6memory_24CFunctionAllocatorMemory_5__reduce_cython__(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
) {
  #if !CYTHON_METH_FASTCALL
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  #endif
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__reduce_cython__ (wrapper)", 0);
  #if !CYTHON_METH_FASTCALL
  #if CYTHON_ASSUME_SAFE_SIZE
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return NULL;
  #endif
  #endif
  __pyx_kwvalues = __Pyx_KwValues_FASTCALL(__pyx_args, __pyx_nargs);
  if (unlikely(__pyx_nargs > 0)) { __Pyx_RaiseArgtupleInvalid("__reduce_cython__", 1, 0, 0, __pyx_nargs); return NULL; }
  const Py_ssize_t __pyx_kwds_len = unlikely(__pyx_kwds) ? __Pyx_NumKwargs_FASTCALL(__pyx_kwds) : 0;
  if (unlikely(__pyx_kwds_len < 0)) return NULL;
  if (unlikely(__pyx_kwds_len > 0)) {__Pyx_RejectKeywords("__reduce_cython__", __pyx_kwds); return NULL;}
  __pyx_r = __pyx_pf_4cupy_4cuda_6memory_24CFunctionAllocatorMemory_4__reduce_cython__(((struct __pyx_obj_4cupy_4cuda_6memory_CFunctionAllocatorMemory *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4cupy_4cuda_6memory_24CFunctionAllocatorMemory_4__reduce_cython__(struct __pyx_obj_4cupy_4cuda_6memory_CFunctionAllocatorMemory *__pyx_v_self) {
  PyObject *__pyx_v_state = 0;
  PyObject *__pyx_v__dict = 0;
  int __pyx_v_use_setstate;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  PyObject *__pyx_t_6 = NULL;
  int __pyx_t_7;
  int __pyx_t_8;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__reduce_cython__", 0);

  /* "(tree fragment)":5
 *     cdef object _dict
 *     cdef bint use_setstate
 *     state = (self._free_func, self._param, self.device_id, self.ptr, self.size)             # <<<<<<<<<<<<<<
 *     _dict = getattr(self, '__dict__', None)
 *     if _dict is not None and _dict:
*/
  __pyx_t_1 = PyLong_FromSsize_t(__pyx_v_self->_free_func); if (unlikely(!__pyx_t_1)) __PYX_ERR(3, 5, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = PyLong_FromSsize_t(__pyx_v_self->_param); if (unlikely(!__pyx_t_2)) __PYX_ERR(3, 5, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = __Pyx_PyLong_From_int(__pyx_v_self->__pyx_base.device_id); if (unlikely(!__pyx_t_3)) __PYX_ERR(3, 5, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_4 = PyLong_FromSsize_t(__pyx_v_self->__pyx_base.ptr); if (unlikely(!__pyx_t_4)) __PYX_ERR(3, 5, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_t_5 = __Pyx_PyLong_FromSize_t(__pyx_v_self->__pyx_base.size); if (unlikely(!__pyx_t_5)) __PYX_ERR(3, 5, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_5);
  __pyx_t_6 = PyTuple_New(5); if (unlikely(!__pyx_t_6)) __PYX_ERR(3, 5, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_6);
  __Pyx_GIVEREF(__pyx_t_1);
  if (__Pyx_PyTuple_SET_ITEM(__pyx_t_6, 0, __pyx_t_1) != (0)) __PYX_ERR(3, 5, __pyx_L1_error);
  __Pyx_GIVEREF(__pyx_t_2);
  if (__Pyx_PyTuple_SET_ITEM(__pyx_t_6, 1, __pyx_t_2) != (0)) __PYX_ERR(3, 5, __pyx_L1_error);
  __Pyx_GIVEREF(__pyx_t_3);
  if (__Pyx_PyTuple_SET_ITEM(__pyx_t_6, 2, __pyx_t_3) != (0)) __PYX_ERR(3, 5, __pyx_L1_error);
  __Pyx_GIVEREF(__pyx_t_4);
  if (__Pyx_PyTuple_SET_ITEM(__pyx_t_6, 3, __pyx_t_4) != (0)) __PYX_ERR(3, 5, __pyx_L1_error);
  __Pyx_GIVEREF(__pyx_t_5);
  if (__Pyx_PyTuple_SET_ITEM(__pyx_t_6, 4, __pyx_t_5) != (0)) __PYX_ERR(3, 5, __pyx_L1_error);
  __pyx_t_1 = 0;
  __pyx_t_2 = 0;
  __pyx_t_3 = 0;
  __pyx_t_4 = 0;
  __pyx_t_5 = 0;
  __pyx_v_state = ((PyObject*)__pyx_t_6);
  __pyx_t_6 = 0;

  /* "(tree fragment)":6
 *     cdef bint use_setstate
 *     state = (self._free_func, self._param, self.device_id, self.ptr, self.size)
 *     _dict = getattr(self, '__dict__', None)             # <<<<<<<<<<<<<<
 *     if _dict is not None and _dict:
 *         state += (_dict,)
*/
  __pyx_t_6 = __Pyx_GetAttr3(((PyObject *)__pyx_v_self), __pyx_mstate_global->__pyx_n_u_dict, Py_None); if (unlikely(!__pyx_t_6)) __PYX_ERR(3, 6, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_6);
  __pyx_v__dict = __pyx_t_6;
  __pyx_t_6 = 0;

  /* "(tree fragment)":7
 *     state = (self._free_func, self._param, self.device_id, self.ptr, self.size)
 *     _dict = getattr(self, '__dict__', None)
 *     if _dict is not None and _dict:             # <<<<<<<<<<<<<<
 *         state += (_dict,)
 *         use_setstate = True
*/
  __pyx_t_8 = (__pyx_v__dict != Py_None);
  if (__pyx_t_8) {
  } else {
    __pyx_t_7 = __pyx_t_8;
    goto __pyx_L4_bool_binop_done;
  }
  __pyx_t_8 = __Pyx_PyObject_IsTrue(__pyx_v__dict); if (unlikely((__pyx_t_8 < 0))) __PYX_ERR(3, 7, __pyx_L1_error)
  __pyx_t_7 = __pyx_t_8;
  __pyx_L4_bool_binop_done:;
  if (__pyx_t_7) {

    /* "(tree fragment)":8
 *     _dict = getattr(self, '__dict__', None)
 *     if _dict is not None and _dict:
 *         state += (_dict,)             # <<<<<<<<<<<<<<
 *         use_setstate = True
 *     else:
*/
    __pyx_t_6 = PyTuple_New(1); if (unlikely(!__pyx_t_6)) __PYX_ERR(3, 8, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_6);
    __Pyx_INCREF(__pyx_v__dict);
    __Pyx_GIVEREF(__pyx_v__dict);
    if (__Pyx_PyTuple_SET_ITEM(__pyx_t_6, 0, __pyx_v__dict) != (0)) __PYX_ERR(3, 8, __pyx_L1_error);
    __pyx_t_5 = PyNumber_InPlaceAdd(__pyx_v_state, __pyx_t_6); if (unlikely(!__pyx_t_5)) __PYX_ERR(3, 8, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
    __Pyx_DECREF_SET(__pyx_v_state, ((PyObject*)__pyx_t_5));
    __pyx_t_5 = 0;

    /* "(tree fragment)":9
 *     if _dict is not None and _dict:
 *         state += (_dict,)
 *         use_setstate = True             # <<<<<<<<<<<<<<
 *     else:
 *         use_setstate = False
*/
    __pyx_v_use_setstate = 1;

    /* "(tree fragment)":7
 *     state = (self._free_func, self._param, self.device_id, self.ptr, self.size)
 *     _dict = getattr(self, '__dict__', None)
 *     if _dict is not None and _dict:             # <<<<<<<<<<<<<<
 *         state += (_dict,)
 *         use_setstate = True
*/
    goto __pyx_L3;
  }

  /* "(tree fragment)":11
 *         use_setstate = True
 *     else:
 *         use_setstate = False             # <<<<<<<<<<<<<<
 *     if use_setstate:
 *         return __pyx_unpickle_CFunctionAllocatorMemory, (type(self), 0xce95057, None), state
*/
  /*else*/ {
    __pyx_v_use_setstate = 0;
  }
  __pyx_L3:;

  /* "(tree fragment)":12
 *     else:
 *         use_setstate = False
 *     if use_setstate:             # <<<<<<<<<<<<<<
 *         return __pyx_unpickle_CFunctionAllocatorMemory, (type(self), 0xce95057, None), state
 *     else:
*/
  if (__pyx_v_use_setstate) {

    /* "(tree fragment)":13
 *         use_setstate = False
 *     if use_setstate:
 *         return __pyx_unpickle_CFunctionAllocatorMemory, (type(self), 0xce95057, None), state             # <<<<<<<<<<<<<<
 *     else:
 *         return __pyx_unpickle_CFunctionAllocatorMemory, (type(self), 0xce95057, state)
*/
    __Pyx_XDECREF(__pyx_r);
    __Pyx_GetModuleGlobalName(__pyx_t_5, __pyx_mstate_global->__pyx_n_u_pyx_unpickle_CFunctionAllocato); if (unlikely(!__pyx_t_5)) __PYX_ERR(3, 13, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __pyx_t_6 = PyTuple_New(3); if (unlikely(!__pyx_t_6)) __PYX_ERR(3, 13, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_6);
    __Pyx_INCREF(((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    __Pyx_GIVEREF(((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    if (__Pyx_PyTuple_SET_ITEM(__pyx_t_6, 0, ((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self)))) != (0)) __PYX_ERR(3, 13, __pyx_L1_error);
    __Pyx_INCREF(__pyx_mstate_global->__pyx_int_216617047);
    __Pyx_GIVEREF(__pyx_mstate_global->__pyx_int_216617047);
    if (__Pyx_PyTuple_SET_ITEM(__pyx_t_6, 1, __pyx_mstate_global->__pyx_int_216617047) != (0)) __PYX_ERR(3, 13, __pyx_L1_error);
    __Pyx_INCREF(Py_None);
    __Pyx_GIVEREF(Py_None);
    if (__Pyx_PyTuple_SET_ITEM(__pyx_t_6, 2, Py_None) != (0)) __PYX_ERR(3, 13, __pyx_L1_error);
    __pyx_t_4 = PyTuple_New(3); if (unlikely(!__pyx_t_4)) __PYX_ERR(3, 13, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_GIVEREF(__pyx_t_5);
    if (__Pyx_PyTuple_SET_ITEM(__pyx_t_4, 0, __pyx_t_5) != (0)) __PYX_ERR(3, 13, __pyx_L1_error);
    __Pyx_GIVEREF(__pyx_t_6);
    if (__Pyx_PyTuple_SET_ITEM(__pyx_t_4, 1, __pyx_t_6) != (0)) __PYX_ERR(3, 13, __pyx_L1_error);
    __Pyx_INCREF(__pyx_v_state);
    __Pyx_GIVEREF(__pyx_v_state);
    if (__Pyx_PyTuple_SET_ITEM(__pyx_t_4, 2, __pyx_v_state) != (0)) __PYX_ERR(3, 13, __pyx_L1_error);
    __pyx_t_5 = 0;
    __pyx_t_6 = 0;
    __pyx_r = __pyx_t_4;
    __pyx_t_4 = 0;
    goto __pyx_L0;

    /* "(tree fragment)":12
 *     else:
 *         use_setstate = False
 *     if use_setstate:             # <<<<<<<<<<<<<<
 *         return __pyx_unpickle_CFunctionAllocatorMemory, (type(self), 0xce95057, None), state
 *     else:
*/
  }

  /* "(tree fragment)":15
 *         return __pyx_unpickle_CFunctionAllocatorMemory, (type(self), 0xce95057, None), state
 *     else:
 *         return __pyx_unpickle_CFunctionAllocatorMemory, (type(self), 0xce95057, state)             # <<<<<<<<<<<<<<
 * def __setstate_cython__(self, __pyx_state):
 *     __pyx_unpickle_CFunctionAllocatorMemory__set_state(self, __pyx_state)
*/
  /*else*/ {
    __Pyx_XDECREF(__pyx_r);
    __Pyx_GetModuleGlobalName(__pyx_t_4, __pyx_mstate_global->__pyx_n_u_pyx_unpickle_CFunctionAllocato); if (unlikely(!__pyx_t_4)) __PYX_ERR(3, 15, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __pyx_t_6 = PyTuple_New(3); if (unlikely(!__pyx_t_6)) __PYX_ERR(3, 15, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_6);
    __Pyx_INCREF(((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    __Pyx_GIVEREF(((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    if (__Pyx_PyTuple_SET_ITEM(__pyx_t_6, 0, ((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self)))) != (0)) __PYX_ERR(3, 15, __pyx_L1_error);
    __Pyx_INCREF(__pyx_mstate_global->__pyx_int_216617047);
    __Pyx_GIVEREF(__pyx_mstate_global->__pyx_int_216617047);
    if (__Pyx_PyTuple_SET_ITEM(__pyx_t_6, 1, __pyx_mstate_global->__pyx_int_216617047) != (0)) __PYX_ERR(3, 15, __pyx_L1_error);
    __Pyx_INCREF(__pyx_v_state);
    __Pyx_GIVEREF(__pyx_v_state);
    if (__Pyx_PyTuple_SET_ITEM(__pyx_t_6, 2, __pyx_v_state) != (0)) __PYX_ERR(3, 15, __pyx_L1_error);
    __pyx_t_5 = PyTuple_New(2); if (unlikely(!__pyx_t_5)) __PYX_ERR(3, 15, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __Pyx_GIVEREF(__pyx_t_4);
    if (__Pyx_PyTuple_SET_ITEM(__pyx_t_5, 0, __pyx_t_4) != (0)) __PYX_ERR(3, 15, __pyx_L1_error);
    __Pyx_GIVEREF(__pyx_t_6);
    if (__Pyx_PyTuple_SET_ITEM(__pyx_t_5, 1, __pyx_t_6) != (0)) __PYX_ERR(3, 15, __pyx_L1_error);
    __pyx_t_4 = 0;
    __pyx_t_6 = 0;
    __pyx_r = __pyx_t_5;
    __pyx_t_5 = 0;
    goto __pyx_L0;
  }

  /* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     cdef tuple state
 *     cdef object _dict
*/

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_AddTraceback("cupy.cuda.memory.CFunctionAllocatorMemory.__reduce_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_state);
  __Pyx_XDECREF(__pyx_v__dict);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":16
 *     else:
 *         return __pyx_unpickle_CFunctionAllocatorMemory, (type(self), 0xce95057, state)
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     __pyx_unpickle_CFunctionAllocatorMemory__set_state(self, __pyx_state)
*/

/* Python wrapper */
static PyObject *__pyx_pw_4cupy_4cuda_6memory_24CFunctionAllocatorMemory_7__setstate_cython__(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
PyDoc_STRVAR(__pyx_doc_4cupy_4cuda_6memory_24CFunctionAllocatorMemory_6__setstate_cython__, "CFunctionAllocatorMemory.__setstate_cython__(self, __pyx_state)");
static PyObject *__pyx_pw_4cupy_4cuda_6memory_24CFunctionAllocatorMemory_7__setstate_cython__(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
) {
  PyObject *__pyx_v___pyx_state = 0;
  #if !CYTHON_METH_FASTCALL
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  #endif
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject* values[1] = {0};
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__setstate_cython__ (wrapper)", 0);
  #if !CYTHON_METH_FASTCALL
  #if CYTHON_ASSUME_SAFE_SIZE
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return NULL;
  #endif
  #endif
  __pyx_kwvalues = __Pyx_KwValues_FASTCALL(__pyx_args, __pyx_nargs);
  {
    PyObject ** const __pyx_pyargnames[] = {&__pyx_mstate_global->__pyx_n_u_pyx_state,0};
    const Py_ssize_t __pyx_kwds_len = (__pyx_kwds) ? __Pyx_NumKwargs_FASTCALL(__pyx_kwds) : 0;
    if (unlikely(__pyx_kwds_len < 0)) __PYX_ERR(3, 16, __pyx_L3_error)
    if (__pyx_kwds_len > 0) {
      switch (__pyx_nargs) {
        case  1:
        values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(3, 16, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      const Py_ssize_t kwd_pos_args = __pyx_nargs;
      if (__Pyx_ParseKeywords(__pyx_kwds, __pyx_kwvalues, __pyx_pyargnames, 0, values, kwd_pos_args, __pyx_kwds_len, "__setstate_cython__", 0) < (0)) __PYX_ERR(3, 16, __pyx_L3_error)
      for (Py_ssize_t i = __pyx_nargs; i < 1; i++) {
        if (unlikely(!values[i])) { __Pyx_RaiseArgtupleInvalid("__setstate_cython__", 1, 1, 1, i); __PYX_ERR(3, 16, __pyx_L3_error) }
      }
    } else if (unlikely(__pyx_nargs != 1)) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
      if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(3, 16, __pyx_L3_error)
    }
    __pyx_v___pyx_state = values[0];
  }
  goto __pyx_L6_skip;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("__setstate_cython__", 1, 1, 1, __pyx_nargs); __PYX_ERR(3, 16, __pyx_L3_error)
  __pyx_L6_skip:;
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_AddTraceback("cupy.cuda.memory.CFunctionAllocatorMemory.__setstate_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_4cupy_4cuda_6memory_24CFunctionAllocatorMemory_6__setstate_cython__(((struct __pyx_obj_4cupy_4cuda_6memory_CFunctionAllocatorMemory *)__pyx_v_self), __pyx_v___pyx_state);

  /* function exit code */
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4cupy_4cuda_6memory_24CFunctionAllocatorMemory_6__setstate_cython__(struct __pyx_obj_4cupy_4cuda_6memory_CFunctionAllocatorMemory *__pyx_v_self, PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__setstate_cython__", 0);

  /* "(tree fragment)":17
 *         return __pyx_unpickle_CFunctionAllocatorMemory, (type(self), 0xce95057, state)
 * def __setstate_cython__(self, __pyx_state):
 *     __pyx_unpickle_CFunctionAllocatorMemory__set_state(self, __pyx_state)             # <<<<<<<<<<<<<<
*/
  __pyx_t_1 = __pyx_v___pyx_state;
  __Pyx_INCREF(__pyx_t_1);
  if (!(likely(PyTuple_CheckExact(__pyx_t_1))||((__pyx_t_1) == Py_None) || __Pyx_RaiseUnexpectedTypeError("tuple", __pyx_t_1))) __PYX_ERR(3, 17, __pyx_L1_error)
  if (unlikely(__pyx_t_1 == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "cannot pass None into a C function argument that is declared 'not None'");
    __PYX_ERR(3, 17, __pyx_L1_error)
  }
  __pyx_t_2 = __pyx_f_4cupy_4cuda_6memory___pyx_unpickle_CFunctionAllocatorMemory__set_state(__pyx_v_self, ((PyObject*)__pyx_t_1)); if (unlikely(!__pyx_t_2)) __PYX_ERR(3, 17, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "(tree fragment)":16
 *     else:
 *         return __pyx_unpickle_CFunctionAllocatorMemory, (type(self), 0xce95057, state)
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     __pyx_unpickle_CFunctionAllocatorMemory__set_state(self, __pyx_state)
*/

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_AddTraceback("cupy.cuda.memory.CFunctionAllocatorMemory.__setstate_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "cupy/cuda/memory.pyx":2107
 *     """
 * 
 *     def __init__(self, intptr_t param, intptr_t malloc_func,             # <<<<<<<<<<<<<<
 *                  intptr_t free_func, object owner):
 *         self._param = param
*/

/* Python wrapper */
static int __pyx_pw_4cupy_4cuda_6memory_18CFunctionAllocator_1__init__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static int __pyx_pw_4cupy_4cuda_6memory_18CFunctionAllocator_1__init__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  intptr_t __pyx_v_param;
  intptr_t __pyx_v_malloc_func;
  intptr_t __pyx_v_free_func;
  PyObject *__pyx_v_owner = 0;
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject* values[4] = {0,0,0,0};
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__init__ (wrapper)", 0);
  #if CYTHON_ASSUME_SAFE_SIZE
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return -1;
  #endif
  __pyx_kwvalues = __Pyx_KwValues_VARARGS(__pyx_args, __pyx_nargs);
  {
    PyObject ** const __pyx_pyargnames[] = {&__pyx_mstate_global->__pyx_n_u_param,&__pyx_mstate_global->__pyx_n_u_malloc_func,&__pyx_mstate_global->__pyx_n_u_free_func,&__pyx_mstate_global->__pyx_n_u_owner,0};
    const Py_ssize_t __pyx_kwds_len = (__pyx_kwds) ? __Pyx_NumKwargs_VARARGS(__pyx_kwds) : 0;
    if (unlikely(__pyx_kwds_len < 0)) __PYX_ERR(0, 2107, __pyx_L3_error)
    if (__pyx_kwds_len > 0) {
      switch (__pyx_nargs) {
        case  4:
        values[3] = __Pyx_ArgRef_VARARGS(__pyx_args, 3);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[3])) __PYX_ERR(0, 2107, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  3:
        values[2] = __Pyx_ArgRef_VARARGS(__pyx_args, 2);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[2])) __PYX_ERR(0, 2107, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  2:
        values[1] = __Pyx_ArgRef_VARARGS(__pyx_args, 1);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[1])) __PYX_ERR(0, 2107, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  1:
        values[0] = __Pyx_ArgRef_VARARGS(__pyx_args, 0);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(0, 2107, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      const Py_ssize_t kwd_pos_args = __pyx_nargs;
      if (__Pyx_ParseKeywords(__pyx_kwds, __pyx_kwvalues, __pyx_pyargnames, 0, values, kwd_pos_args, __pyx_kwds_len, "__init__", 0) < (0)) __PYX_ERR(0, 2107, __pyx_L3_error)
      for (Py_ssize_t i = __pyx_nargs; i < 4; i++) {
        if (unlikely(!values[i])) { __Pyx_RaiseArgtupleInvalid("__init__", 1, 4, 4, i); __PYX_ERR(0, 2107, __pyx_L3_error) }
      }
    } else if (unlikely(__pyx_nargs != 4)) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = __Pyx_ArgRef_VARARGS(__pyx_args, 0);
      if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(0, 2107, __pyx_L3_error)
      values[1] = __Pyx_ArgRef_VARARGS(__pyx_args, 1);
      if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[1])) __PYX_ERR(0, 2107, __pyx_L3_error)
      values[2] = __Pyx_ArgRef_VARARGS(__pyx_args, 2);
      if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[2])) __PYX_ERR(0, 2107, __pyx_L3_error)
      values[3] = __Pyx_ArgRef_VARARGS(__pyx_args, 3);
      if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[3])) __PYX_ERR(0, 2107, __pyx_L3_error)
    }
    __pyx_v_param = PyLong_AsSsize_t(values[0]); if (unlikely((__pyx_v_param == ((intptr_t)-1)) && PyErr_Occurred())) __PYX_ERR(0, 2107, __pyx_L3_error)
    __pyx_v_malloc_func = PyLong_AsSsize_t(values[1]); if (unlikely((__pyx_v_malloc_func == ((intptr_t)-1)) && PyErr_Occurred())) __PYX_ERR(0, 2107, __pyx_L3_error)
    __pyx_v_free_func = PyLong_AsSsize_t(values[2]); if (unlikely((__pyx_v_free_func == ((intptr_t)-1)) && PyErr_Occurred())) __PYX_ERR(0, 2108, __pyx_L3_error)
    __pyx_v_owner = values[3];
  }
  goto __pyx_L6_skip;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("__init__", 1, 4, 4, __pyx_nargs); __PYX_ERR(0, 2107, __pyx_L3_error)
  __pyx_L6_skip:;
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_AddTraceback("cupy.cuda.memory.CFunctionAllocator.__init__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return -1;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_4cupy_4cuda_6memory_18CFunctionAllocator___init__(((struct __pyx_obj_4cupy_4cuda_6memory_CFunctionAllocator *)__pyx_v_self), __pyx_v_param, __pyx_v_malloc_func, __pyx_v_free_func, __pyx_v_owner);

  /* function exit code */
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_4cupy_4cuda_6memory_18CFunctionAllocator___init__(struct __pyx_obj_4cupy_4cuda_6memory_CFunctionAllocator *__pyx_v_self, intptr_t __pyx_v_param, intptr_t __pyx_v_malloc_func, intptr_t __pyx_v_free_func, PyObject *__pyx_v_owner) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__init__", 0);

  /* "cupy/cuda/memory.pyx":2109
 *     def __init__(self, intptr_t param, intptr_t malloc_func,
 *                  intptr_t free_func, object owner):
 *         self._param = param             # <<<<<<<<<<<<<<
 *         self._malloc_func = malloc_func
 *         self._free_func = free_func
*/
  __pyx_v_self->_param = __pyx_v_param;

  /* "cupy/cuda/memory.pyx":2110
 *                  intptr_t free_func, object owner):
 *         self._param = param
 *         self._malloc_func = malloc_func             # <<<<<<<<<<<<<<
 *         self._free_func = free_func
 *         self._owner = owner
*/
  __pyx_v_self->_malloc_func = __pyx_v_malloc_func;

  /* "cupy/cuda/memory.pyx":2111
 *         self._param = param
 *         self._malloc_func = malloc_func
 *         self._free_func = free_func             # <<<<<<<<<<<<<<
 *         self._owner = owner
 * 
*/
  __pyx_v_self->_free_func = __pyx_v_free_func;

  /* "cupy/cuda/memory.pyx":2112
 *         self._malloc_func = malloc_func
 *         self._free_func = free_func
 *         self._owner = owner             # <<<<<<<<<<<<<<
 * 
 *     cpdef MemoryPointer malloc(self, size_t size):
*/
  __Pyx_INCREF(__pyx_v_owner);
  __Pyx_GIVEREF(__pyx_v_owner);
  __Pyx_GOTREF(__pyx_v_self->_owner);
  __Pyx_DECREF(__pyx_v_self->_owner);
  __pyx_v_self->_owner = __pyx_v_owner;

  /* "cupy/cuda/memory.pyx":2107
 *     """
 * 
 *     def __init__(self, intptr_t param, intptr_t malloc_func,             # <<<<<<<<<<<<<<
 *                  intptr_t free_func, object owner):
 *         self._param = param
*/

  /* function exit code */
  __pyx_r = 0;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "cupy/cuda/memory.pyx":2114
 *         self._owner = owner
 * 
 *     cpdef MemoryPointer malloc(self, size_t size):             # <<<<<<<<<<<<<<
 *         mem = CFunctionAllocatorMemory(size, self._param, self._malloc_func,
 *                                        self._free_func, device.get_device_id())
*/

static PyObject *__pyx_pw_4cupy_4cuda_6memory_18CFunctionAllocator_3malloc(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
static struct __pyx_obj_4cupy_4cuda_6memory_MemoryPointer *__pyx_f_4cupy_4cuda_6memory_18CFunctionAllocator_malloc(struct __pyx_obj_4cupy_4cuda_6memory_CFunctionAllocator *__pyx_v_self, size_t __pyx_v_size, int __pyx_skip_dispatch) {
  struct __pyx_obj_4cupy_4cuda_6memory_CFunctionAllocatorMemory *__pyx_v_mem = NULL;
  struct __pyx_obj_4cupy_4cuda_6memory_MemoryPointer *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  size_t __pyx_t_6;
  PyObject *__pyx_t_7 = NULL;
  int __pyx_t_8;
  PyObject *__pyx_t_9 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("malloc", 0);
  /* Check if called by wrapper */
  if (unlikely(__pyx_skip_dispatch)) ;
  /* Check if overridden in Python */
  else if (
  #if !CYTHON_USE_TYPE_SLOTS
  unlikely(Py_TYPE(((PyObject *)__pyx_v_self)) != __pyx_mstate_global->__pyx_ptype_4cupy_4cuda_6memory_CFunctionAllocator &&
  __Pyx_PyType_HasFeature(Py_TYPE(((PyObject *)__pyx_v_self)), Py_TPFLAGS_HAVE_GC))
  #else
  unlikely(Py_TYPE(((PyObject *)__pyx_v_self))->tp_dictoffset != 0 || __Pyx_PyType_HasFeature(Py_TYPE(((PyObject *)__pyx_v_self)), (Py_TPFLAGS_IS_ABSTRACT | Py_TPFLAGS_HEAPTYPE)))
  #endif
  ) {
    #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
    static PY_UINT64_T __pyx_tp_dict_version = __PYX_DICT_VERSION_INIT, __pyx_obj_dict_version = __PYX_DICT_VERSION_INIT;
    if (unlikely(!__Pyx_object_dict_version_matches(((PyObject *)__pyx_v_self), __pyx_tp_dict_version, __pyx_obj_dict_version))) {
      PY_UINT64_T __pyx_typedict_guard = __Pyx_get_tp_dict_version(((PyObject *)__pyx_v_self));
      #endif
      __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_mstate_global->__pyx_n_u_malloc_2); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 2114, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      if (!__Pyx_IsSameCFunction(__pyx_t_1, (void(*)(void)) __pyx_pw_4cupy_4cuda_6memory_18CFunctionAllocator_3malloc)) {
        __Pyx_XDECREF((PyObject *)__pyx_r);
        __pyx_t_3 = NULL;
        __Pyx_INCREF(__pyx_t_1);
        __pyx_t_4 = __pyx_t_1; 
        __pyx_t_5 = __Pyx_PyLong_FromSize_t(__pyx_v_size); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 2114, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_5);
        __pyx_t_6 = 1;
        #if CYTHON_UNPACK_METHODS
        if (unlikely(PyMethod_Check(__pyx_t_4))) {
          __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_4);
          assert(__pyx_t_3);
          PyObject* __pyx__function = PyMethod_GET_FUNCTION(__pyx_t_4);
          __Pyx_INCREF(__pyx_t_3);
          __Pyx_INCREF(__pyx__function);
          __Pyx_DECREF_SET(__pyx_t_4, __pyx__function);
          __pyx_t_6 = 0;
        }
        #endif
        {
          PyObject *__pyx_callargs[2] = {__pyx_t_3, __pyx_t_5};
          __pyx_t_2 = __Pyx_PyObject_FastCall((PyObject*)__pyx_t_4, __pyx_callargs+__pyx_t_6, (2-__pyx_t_6) | (__pyx_t_6*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
          __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
          __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
          __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
          if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 2114, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_2);
        }
        if (!(likely(((__pyx_t_2) == Py_None) || likely(__Pyx_TypeTest(__pyx_t_2, __pyx_mstate_global->__pyx_ptype_4cupy_4cuda_6memory_MemoryPointer))))) __PYX_ERR(0, 2114, __pyx_L1_error)
        __pyx_r = ((struct __pyx_obj_4cupy_4cuda_6memory_MemoryPointer *)__pyx_t_2);
        __pyx_t_2 = 0;
        __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
        goto __pyx_L0;
      }
      #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
      __pyx_tp_dict_version = __Pyx_get_tp_dict_version(((PyObject *)__pyx_v_self));
      __pyx_obj_dict_version = __Pyx_get_object_dict_version(((PyObject *)__pyx_v_self));
      if (unlikely(__pyx_typedict_guard != __pyx_tp_dict_version)) {
        __pyx_tp_dict_version = __pyx_obj_dict_version = __PYX_DICT_VERSION_INIT;
      }
      #endif
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
    }
    #endif
  }

  /* "cupy/cuda/memory.pyx":2115
 * 
 *     cpdef MemoryPointer malloc(self, size_t size):
 *         mem = CFunctionAllocatorMemory(size, self._param, self._malloc_func,             # <<<<<<<<<<<<<<
 *                                        self._free_func, device.get_device_id())
 *         return MemoryPointer(mem, 0)
*/
  __pyx_t_2 = NULL;
  __pyx_t_4 = __Pyx_PyLong_FromSize_t(__pyx_v_size); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 2115, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_t_5 = PyLong_FromSsize_t(__pyx_v_self->_param); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 2115, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_5);
  __pyx_t_3 = PyLong_FromSsize_t(__pyx_v_self->_malloc_func); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 2115, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);

  /* "cupy/cuda/memory.pyx":2116
 *     cpdef MemoryPointer malloc(self, size_t size):
 *         mem = CFunctionAllocatorMemory(size, self._param, self._malloc_func,
 *                                        self._free_func, device.get_device_id())             # <<<<<<<<<<<<<<
 *         return MemoryPointer(mem, 0)
 * 
*/
  __pyx_t_7 = PyLong_FromSsize_t(__pyx_v_self->_free_func); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 2116, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_7);
  __pyx_t_8 = __pyx_f_4cupy_4cuda_6device_get_device_id(0); if (unlikely(__pyx_t_8 == ((int)-1) && PyErr_Occurred())) __PYX_ERR(0, 2116, __pyx_L1_error)
  __pyx_t_9 = __Pyx_PyLong_From_int(__pyx_t_8); if (unlikely(!__pyx_t_9)) __PYX_ERR(0, 2116, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_9);
  __pyx_t_6 = 1;
  {
    PyObject *__pyx_callargs[6] = {__pyx_t_2, __pyx_t_4, __pyx_t_5, __pyx_t_3, __pyx_t_7, __pyx_t_9};
    __pyx_t_1 = __Pyx_PyObject_FastCall((PyObject*)__pyx_mstate_global->__pyx_ptype_4cupy_4cuda_6memory_CFunctionAllocatorMemory, __pyx_callargs+__pyx_t_6, (6-__pyx_t_6) | (__pyx_t_6*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
    __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
    __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
    if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 2115, __pyx_L1_error)
    __Pyx_GOTREF((PyObject *)__pyx_t_1);
  }
  __pyx_v_mem = ((struct __pyx_obj_4cupy_4cuda_6memory_CFunctionAllocatorMemory *)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "cupy/cuda/memory.pyx":2117
 *         mem = CFunctionAllocatorMemory(size, self._param, self._malloc_func,
 *                                        self._free_func, device.get_device_id())
 *         return MemoryPointer(mem, 0)             # <<<<<<<<<<<<<<
 * 
 * 
*/
  __Pyx_XDECREF((PyObject *)__pyx_r);
  __pyx_t_9 = NULL;
  __pyx_t_6 = 1;
  {
    PyObject *__pyx_callargs[3] = {__pyx_t_9, ((PyObject *)__pyx_v_mem), __pyx_mstate_global->__pyx_int_0};
    __pyx_t_1 = __Pyx_PyObject_FastCall((PyObject*)__pyx_mstate_global->__pyx_ptype_4cupy_4cuda_6memory_MemoryPointer, __pyx_callargs+__pyx_t_6, (3-__pyx_t_6) | (__pyx_t_6*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
    __Pyx_XDECREF(__pyx_t_9); __pyx_t_9 = 0;
    if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 2117, __pyx_L1_error)
    __Pyx_GOTREF((PyObject *)__pyx_t_1);
  }
  __pyx_r = ((struct __pyx_obj_4cupy_4cuda_6memory_MemoryPointer *)__pyx_t_1);
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "cupy/cuda/memory.pyx":2114
 *         self._owner = owner
 * 
 *     cpdef MemoryPointer malloc(self, size_t size):             # <<<<<<<<<<<<<<
 *         mem = CFunctionAllocatorMemory(size, self._param, self._malloc_func,
 *                                        self._free_func, device.get_device_id())
*/

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_XDECREF(__pyx_t_7);
  __Pyx_XDECREF(__pyx_t_9);
  __Pyx_AddTraceback("cupy.cuda.memory.CFunctionAllocator.malloc", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XDECREF((PyObject *)__pyx_v_mem);
  __Pyx_XGIVEREF((PyObject *)__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_pw_4cupy_4cuda_6memory_18CFunctionAllocator_3malloc(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
PyDoc_STRVAR(__pyx_doc_4cupy_4cuda_6memory_18CFunctionAllocator_2malloc, "CFunctionAllocator.malloc(self, size_t size) -> MemoryPointer");
static PyObject *__pyx_pw_4cupy_4cuda_6memory_18CFunctionAllocator_3malloc(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
) {
  size_t __pyx_v_size;
  #if !CYTHON_METH_FASTCALL
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  #endif
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject* values[1] = {0};
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("malloc (wrapper)", 0);
  #if !CYTHON_METH_FASTCALL
  #if CYTHON_ASSUME_SAFE_SIZE
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return NULL;
  #endif
  #endif
  __pyx_kwvalues = __Pyx_KwValues_FASTCALL(__pyx_args, __pyx_nargs);
  {
    PyObject ** const __pyx_pyargnames[] = {&__pyx_mstate_global->__pyx_n_u_size,0};
    const Py_ssize_t __pyx_kwds_len = (__pyx_kwds) ? __Pyx_NumKwargs_FASTCALL(__pyx_kwds) : 0;
    if (unlikely(__pyx_kwds_len < 0)) __PYX_ERR(0, 2114, __pyx_L3_error)
    if (__pyx_kwds_len > 0) {
      switch (__pyx_nargs) {
        case  1:
        values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(0, 2114, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      const Py_ssize_t kwd_pos_args = __pyx_nargs;
      if (__Pyx_ParseKeywords(__pyx_kwds, __pyx_kwvalues, __pyx_pyargnames, 0, values, kwd_pos_args, __pyx_kwds_len, "malloc", 0) < (0)) __PYX_ERR(0, 2114, __pyx_L3_error)
      for (Py_ssize_t i = __pyx_nargs; i < 1; i++) {
        if (unlikely(!values[i])) { __Pyx_RaiseArgtupleInvalid("malloc", 1, 1, 1, i); __PYX_ERR(0, 2114, __pyx_L3_error) }
      }
    } else if (unlikely(__pyx_nargs != 1)) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
      if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(0, 2114, __pyx_L3_error)
    }
    __pyx_v_size = __Pyx_PyLong_As_size_t(values[0]); if (unlikely((__pyx_v_size == (size_t)-1) && PyErr_Occurred())) __PYX_ERR(0, 2114, __pyx_L3_error)
  }
  goto __pyx_L6_skip;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("malloc", 1, 1, 1, __pyx_nargs); __PYX_ERR(0, 2114, __pyx_L3_error)
  __pyx_L6_skip:;
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_AddTraceback("cupy.cuda.memory.CFunctionAllocator.malloc", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_4cupy_4cuda_6memory_18CFunctionAllocator_2malloc(((struct __pyx_obj_4cupy_4cuda_6memory_CFunctionAllocator *)__pyx_v_self), __pyx_v_size);

  /* function exit code */
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4cupy_4cuda_6memory_18CFunctionAllocator_2malloc(struct __pyx_obj_4cupy_4cuda_6memory_CFunctionAllocator *__pyx_v_self, size_t __pyx_v_size) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("malloc", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = ((PyObject *)__pyx_f_4cupy_4cuda_6memory_18CFunctionAllocator_malloc(__pyx_v_self, __pyx_v_size, 1)); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 2114, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("cupy.cuda.memory.CFunctionAllocator.malloc", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     cdef tuple state
 *     cdef object _dict
*/

/* Python wrapper */
static PyObject *__pyx_pw_4cupy_4cuda_6memory_18CFunctionAllocator_5__reduce_cython__(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
PyDoc_STRVAR(__pyx_doc_4cupy_4cuda_6memory_18CFunctionAllocator_4__reduce_cython__, "CFunctionAllocator.__reduce_cython__(self)");
static PyObject *__pyx_pw_4cupy_4cuda_6memory_18CFunctionAllocator_5__reduce_cython__(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
) {
  #if !CYTHON_METH_FASTCALL
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  #endif
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__reduce_cython__ (wrapper)", 0);
  #if !CYTHON_METH_FASTCALL
  #if CYTHON_ASSUME_SAFE_SIZE
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return NULL;
  #endif
  #endif
  __pyx_kwvalues = __Pyx_KwValues_FASTCALL(__pyx_args, __pyx_nargs);
  if (unlikely(__pyx_nargs > 0)) { __Pyx_RaiseArgtupleInvalid("__reduce_cython__", 1, 0, 0, __pyx_nargs); return NULL; }
  const Py_ssize_t __pyx_kwds_len = unlikely(__pyx_kwds) ? __Pyx_NumKwargs_FASTCALL(__pyx_kwds) : 0;
  if (unlikely(__pyx_kwds_len < 0)) return NULL;
  if (unlikely(__pyx_kwds_len > 0)) {__Pyx_RejectKeywords("__reduce_cython__", __pyx_kwds); return NULL;}
  __pyx_r = __pyx_pf_4cupy_4cuda_6memory_18CFunctionAllocator_4__reduce_cython__(((struct __pyx_obj_4cupy_4cuda_6memory_CFunctionAllocator *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4cupy_4cuda_6memory_18CFunctionAllocator_4__reduce_cython__(struct __pyx_obj_4cupy_4cuda_6memory_CFunctionAllocator *__pyx_v_self) {
  PyObject *__pyx_v_state = 0;
  PyObject *__pyx_v__dict = 0;
  int __pyx_v_use_setstate;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  int __pyx_t_5;
  int __pyx_t_6;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__reduce_cython__", 0);

  /* "(tree fragment)":5
 *     cdef object _dict
 *     cdef bint use_setstate
 *     state = (self._free_func, self._malloc_func, self._owner, self._param)             # <<<<<<<<<<<<<<
 *     _dict = getattr(self, '__dict__', None)
 *     if _dict is not None and _dict:
*/
  __pyx_t_1 = PyLong_FromSsize_t(__pyx_v_self->_free_func); if (unlikely(!__pyx_t_1)) __PYX_ERR(3, 5, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = PyLong_FromSsize_t(__pyx_v_self->_malloc_func); if (unlikely(!__pyx_t_2)) __PYX_ERR(3, 5, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = PyLong_FromSsize_t(__pyx_v_self->_param); if (unlikely(!__pyx_t_3)) __PYX_ERR(3, 5, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_4 = PyTuple_New(4); if (unlikely(!__pyx_t_4)) __PYX_ERR(3, 5, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_GIVEREF(__pyx_t_1);
  if (__Pyx_PyTuple_SET_ITEM(__pyx_t_4, 0, __pyx_t_1) != (0)) __PYX_ERR(3, 5, __pyx_L1_error);
  __Pyx_GIVEREF(__pyx_t_2);
  if (__Pyx_PyTuple_SET_ITEM(__pyx_t_4, 1, __pyx_t_2) != (0)) __PYX_ERR(3, 5, __pyx_L1_error);
  __Pyx_INCREF(__pyx_v_self->_owner);
  __Pyx_GIVEREF(__pyx_v_self->_owner);
  if (__Pyx_PyTuple_SET_ITEM(__pyx_t_4, 2, __pyx_v_self->_owner) != (0)) __PYX_ERR(3, 5, __pyx_L1_error);
  __Pyx_GIVEREF(__pyx_t_3);
  if (__Pyx_PyTuple_SET_ITEM(__pyx_t_4, 3, __pyx_t_3) != (0)) __PYX_ERR(3, 5, __pyx_L1_error);
  __pyx_t_1 = 0;
  __pyx_t_2 = 0;
  __pyx_t_3 = 0;
  __pyx_v_state = ((PyObject*)__pyx_t_4);
  __pyx_t_4 = 0;

  /* "(tree fragment)":6
 *     cdef bint use_setstate
 *     state = (self._free_func, self._malloc_func, self._owner, self._param)
 *     _dict = getattr(self, '__dict__', None)             # <<<<<<<<<<<<<<
 *     if _dict is not None and _dict:
 *         state += (_dict,)
*/
  __pyx_t_4 = __Pyx_GetAttr3(((PyObject *)__pyx_v_self), __pyx_mstate_global->__pyx_n_u_dict, Py_None); if (unlikely(!__pyx_t_4)) __PYX_ERR(3, 6, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_v__dict = __pyx_t_4;
  __pyx_t_4 = 0;

  /* "(tree fragment)":7
 *     state = (self._free_func, self._malloc_func, self._owner, self._param)
 *     _dict = getattr(self, '__dict__', None)
 *     if _dict is not None and _dict:             # <<<<<<<<<<<<<<
 *         state += (_dict,)
 *         use_setstate = True
*/
  __pyx_t_6 = (__pyx_v__dict != Py_None);
  if (__pyx_t_6) {
  } else {
    __pyx_t_5 = __pyx_t_6;
    goto __pyx_L4_bool_binop_done;
  }
  __pyx_t_6 = __Pyx_PyObject_IsTrue(__pyx_v__dict); if (unlikely((__pyx_t_6 < 0))) __PYX_ERR(3, 7, __pyx_L1_error)
  __pyx_t_5 = __pyx_t_6;
  __pyx_L4_bool_binop_done:;
  if (__pyx_t_5) {

    /* "(tree fragment)":8
 *     _dict = getattr(self, '__dict__', None)
 *     if _dict is not None and _dict:
 *         state += (_dict,)             # <<<<<<<<<<<<<<
 *         use_setstate = True
 *     else:
*/
    __pyx_t_4 = PyTuple_New(1); if (unlikely(!__pyx_t_4)) __PYX_ERR(3, 8, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_INCREF(__pyx_v__dict);
    __Pyx_GIVEREF(__pyx_v__dict);
    if (__Pyx_PyTuple_SET_ITEM(__pyx_t_4, 0, __pyx_v__dict) != (0)) __PYX_ERR(3, 8, __pyx_L1_error);
    __pyx_t_3 = PyNumber_InPlaceAdd(__pyx_v_state, __pyx_t_4); if (unlikely(!__pyx_t_3)) __PYX_ERR(3, 8, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __Pyx_DECREF_SET(__pyx_v_state, ((PyObject*)__pyx_t_3));
    __pyx_t_3 = 0;

    /* "(tree fragment)":9
 *     if _dict is not None and _dict:
 *         state += (_dict,)
 *         use_setstate = True             # <<<<<<<<<<<<<<
 *     else:
 *         use_setstate = self._owner is not None
*/
    __pyx_v_use_setstate = 1;

    /* "(tree fragment)":7
 *     state = (self._free_func, self._malloc_func, self._owner, self._param)
 *     _dict = getattr(self, '__dict__', None)
 *     if _dict is not None and _dict:             # <<<<<<<<<<<<<<
 *         state += (_dict,)
 *         use_setstate = True
*/
    goto __pyx_L3;
  }

  /* "(tree fragment)":11
 *         use_setstate = True
 *     else:
 *         use_setstate = self._owner is not None             # <<<<<<<<<<<<<<
 *     if use_setstate:
 *         return __pyx_unpickle_CFunctionAllocator, (type(self), 0x54c1fda, None), state
*/
  /*else*/ {
    __pyx_t_5 = (__pyx_v_self->_owner != Py_None);
    __pyx_v_use_setstate = __pyx_t_5;
  }
  __pyx_L3:;

  /* "(tree fragment)":12
 *     else:
 *         use_setstate = self._owner is not None
 *     if use_setstate:             # <<<<<<<<<<<<<<
 *         return __pyx_unpickle_CFunctionAllocator, (type(self), 0x54c1fda, None), state
 *     else:
*/
  if (__pyx_v_use_setstate) {

    /* "(tree fragment)":13
 *         use_setstate = self._owner is not None
 *     if use_setstate:
 *         return __pyx_unpickle_CFunctionAllocator, (type(self), 0x54c1fda, None), state             # <<<<<<<<<<<<<<
 *     else:
 *         return __pyx_unpickle_CFunctionAllocator, (type(self), 0x54c1fda, state)
*/
    __Pyx_XDECREF(__pyx_r);
    __Pyx_GetModuleGlobalName(__pyx_t_3, __pyx_mstate_global->__pyx_n_u_pyx_unpickle_CFunctionAllocato_2); if (unlikely(!__pyx_t_3)) __PYX_ERR(3, 13, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_t_4 = PyTuple_New(3); if (unlikely(!__pyx_t_4)) __PYX_ERR(3, 13, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_INCREF(((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    __Pyx_GIVEREF(((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    if (__Pyx_PyTuple_SET_ITEM(__pyx_t_4, 0, ((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self)))) != (0)) __PYX_ERR(3, 13, __pyx_L1_error);
    __Pyx_INCREF(__pyx_mstate_global->__pyx_int_88874970);
    __Pyx_GIVEREF(__pyx_mstate_global->__pyx_int_88874970);
    if (__Pyx_PyTuple_SET_ITEM(__pyx_t_4, 1, __pyx_mstate_global->__pyx_int_88874970) != (0)) __PYX_ERR(3, 13, __pyx_L1_error);
    __Pyx_INCREF(Py_None);
    __Pyx_GIVEREF(Py_None);
    if (__Pyx_PyTuple_SET_ITEM(__pyx_t_4, 2, Py_None) != (0)) __PYX_ERR(3, 13, __pyx_L1_error);
    __pyx_t_2 = PyTuple_New(3); if (unlikely(!__pyx_t_2)) __PYX_ERR(3, 13, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_GIVEREF(__pyx_t_3);
    if (__Pyx_PyTuple_SET_ITEM(__pyx_t_2, 0, __pyx_t_3) != (0)) __PYX_ERR(3, 13, __pyx_L1_error);
    __Pyx_GIVEREF(__pyx_t_4);
    if (__Pyx_PyTuple_SET_ITEM(__pyx_t_2, 1, __pyx_t_4) != (0)) __PYX_ERR(3, 13, __pyx_L1_error);
    __Pyx_INCREF(__pyx_v_state);
    __Pyx_GIVEREF(__pyx_v_state);
    if (__Pyx_PyTuple_SET_ITEM(__pyx_t_2, 2, __pyx_v_state) != (0)) __PYX_ERR(3, 13, __pyx_L1_error);
    __pyx_t_3 = 0;
    __pyx_t_4 = 0;
    __pyx_r = __pyx_t_2;
    __pyx_t_2 = 0;
    goto __pyx_L0;

    /* "(tree fragment)":12
 *     else:
 *         use_setstate = self._owner is not None
 *     if use_setstate:             # <<<<<<<<<<<<<<
 *         return __pyx_unpickle_CFunctionAllocator, (type(self), 0x54c1fda, None), state
 *     else:
*/
  }

  /* "(tree fragment)":15
 *         return __pyx_unpickle_CFunctionAllocator, (type(self), 0x54c1fda, None), state
 *     else:
 *         return __pyx_unpickle_CFunctionAllocator, (type(self), 0x54c1fda, state)             # <<<<<<<<<<<<<<
 * def __setstate_cython__(self, __pyx_state):
 *     __pyx_unpickle_CFunctionAllocator__set_state(self, __pyx_state)
*/
  /*else*/ {
    __Pyx_XDECREF(__pyx_r);
    __Pyx_GetModuleGlobalName(__pyx_t_2, __pyx_mstate_global->__pyx_n_u_pyx_unpickle_CFunctionAllocato_2); if (unlikely(!__pyx_t_2)) __PYX_ERR(3, 15, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_4 = PyTuple_New(3); if (unlikely(!__pyx_t_4)) __PYX_ERR(3, 15, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_INCREF(((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    __Pyx_GIVEREF(((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    if (__Pyx_PyTuple_SET_ITEM(__pyx_t_4, 0, ((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self)))) != (0)) __PYX_ERR(3, 15, __pyx_L1_error);
    __Pyx_INCREF(__pyx_mstate_global->__pyx_int_88874970);
    __Pyx_GIVEREF(__pyx_mstate_global->__pyx_int_88874970);
    if (__Pyx_PyTuple_SET_ITEM(__pyx_t_4, 1, __pyx_mstate_global->__pyx_int_88874970) != (0)) __PYX_ERR(3, 15, __pyx_L1_error);
    __Pyx_INCREF(__pyx_v_state);
    __Pyx_GIVEREF(__pyx_v_state);
    if (__Pyx_PyTuple_SET_ITEM(__pyx_t_4, 2, __pyx_v_state) != (0)) __PYX_ERR(3, 15, __pyx_L1_error);
    __pyx_t_3 = PyTuple_New(2); if (unlikely(!__pyx_t_3)) __PYX_ERR(3, 15, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_GIVEREF(__pyx_t_2);
    if (__Pyx_PyTuple_SET_ITEM(__pyx_t_3, 0, __pyx_t_2) != (0)) __PYX_ERR(3, 15, __pyx_L1_error);
    __Pyx_GIVEREF(__pyx_t_4);
    if (__Pyx_PyTuple_SET_ITEM(__pyx_t_3, 1, __pyx_t_4) != (0)) __PYX_ERR(3, 15, __pyx_L1_error);
    __pyx_t_2 = 0;
    __pyx_t_4 = 0;
    __pyx_r = __pyx_t_3;
    __pyx_t_3 = 0;
    goto __pyx_L0;
  }

  /* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     cdef tuple state
 *     cdef object _dict
*/

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_AddTraceback("cupy.cuda.memory.CFunctionAllocator.__reduce_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_state);
  __Pyx_XDECREF(__pyx_v__dict);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":16
 *     else:
 *         return __pyx_unpickle_CFunctionAllocator, (type(self), 0x54c1fda, state)
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     __pyx_unpickle_CFunctionAllocator__set_state(self, __pyx_state)
*/

/* Python wrapper */
static PyObject *__pyx_pw_4cupy_4cuda_6memory_18CFunctionAllocator_7__setstate_cython__(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
PyDoc_STRVAR(__pyx_doc_4cupy_4cuda_6memory_18CFunctionAllocator_6__setstate_cython__, "CFunctionAllocator.__setstate_cython__(self, __pyx_state)");
static PyObject *__pyx_pw_4cupy_4cuda_6memory_18CFunctionAllocator_7__setstate_cython__(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
) {
  PyObject *__pyx_v___pyx_state = 0;
  #if !CYTHON_METH_FASTCALL
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  #endif
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject* values[1] = {0};
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__setstate_cython__ (wrapper)", 0);
  #if !CYTHON_METH_FASTCALL
  #if CYTHON_ASSUME_SAFE_SIZE
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return NULL;
  #endif
  #endif
  __pyx_kwvalues = __Pyx_KwValues_FASTCALL(__pyx_args, __pyx_nargs);
  {
    PyObject ** const __pyx_pyargnames[] = {&__pyx_mstate_global->__pyx_n_u_pyx_state,0};
    const Py_ssize_t __pyx_kwds_len = (__pyx_kwds) ? __Pyx_NumKwargs_FASTCALL(__pyx_kwds) : 0;
    if (unlikely(__pyx_kwds_len < 0)) __PYX_ERR(3, 16, __pyx_L3_error)
    if (__pyx_kwds_len > 0) {
      switch (__pyx_nargs) {
        case  1:
        values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(3, 16, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      const Py_ssize_t kwd_pos_args = __pyx_nargs;
      if (__Pyx_ParseKeywords(__pyx_kwds, __pyx_kwvalues, __pyx_pyargnames, 0, values, kwd_pos_args, __pyx_kwds_len, "__setstate_cython__", 0) < (0)) __PYX_ERR(3, 16, __pyx_L3_error)
      for (Py_ssize_t i = __pyx_nargs; i < 1; i++) {
        if (unlikely(!values[i])) { __Pyx_RaiseArgtupleInvalid("__setstate_cython__", 1, 1, 1, i); __PYX_ERR(3, 16, __pyx_L3_error) }
      }
    } else if (unlikely(__pyx_nargs != 1)) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
      if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(3, 16, __pyx_L3_error)
    }
    __pyx_v___pyx_state = values[0];
  }
  goto __pyx_L6_skip;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("__setstate_cython__", 1, 1, 1, __pyx_nargs); __PYX_ERR(3, 16, __pyx_L3_error)
  __pyx_L6_skip:;
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_AddTraceback("cupy.cuda.memory.CFunctionAllocator.__setstate_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_4cupy_4cuda_6memory_18CFunctionAllocator_6__setstate_cython__(((struct __pyx_obj_4cupy_4cuda_6memory_CFunctionAllocator *)__pyx_v_self), __pyx_v___pyx_state);

  /* function exit code */
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4cupy_4cuda_6memory_18CFunctionAllocator_6__setstate_cython__(struct __pyx_obj_4cupy_4cuda_6memory_CFunctionAllocator *__pyx_v_self, PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__setstate_cython__", 0);

  /* "(tree fragment)":17
 *         return __pyx_unpickle_CFunctionAllocator, (type(self), 0x54c1fda, state)
 * def __setstate_cython__(self, __pyx_state):
 *     __pyx_unpickle_CFunctionAllocator__set_state(self, __pyx_state)             # <<<<<<<<<<<<<<
*/
  __pyx_t_1 = __pyx_v___pyx_state;
  __Pyx_INCREF(__pyx_t_1);
  if (!(likely(PyTuple_CheckExact(__pyx_t_1))||((__pyx_t_1) == Py_None) || __Pyx_RaiseUnexpectedTypeError("tuple", __pyx_t_1))) __PYX_ERR(3, 17, __pyx_L1_error)
  if (unlikely(__pyx_t_1 == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "cannot pass None into a C function argument that is declared 'not None'");
    __PYX_ERR(3, 17, __pyx_L1_error)
  }
  __pyx_t_2 = __pyx_f_4cupy_4cuda_6memory___pyx_unpickle_CFunctionAllocator__set_state(__pyx_v_self, ((PyObject*)__pyx_t_1)); if (unlikely(!__pyx_t_2)) __PYX_ERR(3, 17, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "(tree fragment)":16
 *     else:
 *         return __pyx_unpickle_CFunctionAllocator, (type(self), 0x54c1fda, state)
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     __pyx_unpickle_CFunctionAllocator__set_state(self, __pyx_state)
*/

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_AddTraceback("cupy.cuda.memory.CFunctionAllocator.__setstate_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "cupy/cuda/memory.pyx":2122
 * cdef class PythonFunctionAllocatorMemory(BaseMemory):
 * 
 *     def __init__(self, size_t size, malloc_func, free_func,             # <<<<<<<<<<<<<<
 *                  int device_id):
 *         self._free_func = free_func
*/

/* Python wrapper */
static int __pyx_pw_4cupy_4cuda_6memory_29PythonFunctionAllocatorMemory_1__init__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static int __pyx_pw_4cupy_4cuda_6memory_29PythonFunctionAllocatorMemory_1__init__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  size_t __pyx_v_size;
  PyObject *__pyx_v_malloc_func = 0;
  PyObject *__pyx_v_free_func = 0;
  int __pyx_v_device_id;
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject* values[4] = {0,0,0,0};
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__init__ (wrapper)", 0);
  #if CYTHON_ASSUME_SAFE_SIZE
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return -1;
  #endif
  __pyx_kwvalues = __Pyx_KwValues_VARARGS(__pyx_args, __pyx_nargs);
  {
    PyObject ** const __pyx_pyargnames[] = {&__pyx_mstate_global->__pyx_n_u_size,&__pyx_mstate_global->__pyx_n_u_malloc_func,&__pyx_mstate_global->__pyx_n_u_free_func,&__pyx_mstate_global->__pyx_n_u_device_id,0};
    const Py_ssize_t __pyx_kwds_len = (__pyx_kwds) ? __Pyx_NumKwargs_VARARGS(__pyx_kwds) : 0;
    if (unlikely(__pyx_kwds_len < 0)) __PYX_ERR(0, 2122, __pyx_L3_error)
    if (__pyx_kwds_len > 0) {
      switch (__pyx_nargs) {
        case  4:
        values[3] = __Pyx_ArgRef_VARARGS(__pyx_args, 3);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[3])) __PYX_ERR(0, 2122, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  3:
        values[2] = __Pyx_ArgRef_VARARGS(__pyx_args, 2);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[2])) __PYX_ERR(0, 2122, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  2:
        values[1] = __Pyx_ArgRef_VARARGS(__pyx_args, 1);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[1])) __PYX_ERR(0, 2122, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  1:
        values[0] = __Pyx_ArgRef_VARARGS(__pyx_args, 0);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(0, 2122, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      const Py_ssize_t kwd_pos_args = __pyx_nargs;
      if (__Pyx_ParseKeywords(__pyx_kwds, __pyx_kwvalues, __pyx_pyargnames, 0, values, kwd_pos_args, __pyx_kwds_len, "__init__", 0) < (0)) __PYX_ERR(0, 2122, __pyx_L3_error)
      for (Py_ssize_t i = __pyx_nargs; i < 4; i++) {
        if (unlikely(!values[i])) { __Pyx_RaiseArgtupleInvalid("__init__", 1, 4, 4, i); __PYX_ERR(0, 2122, __pyx_L3_error) }
      }
    } else if (unlikely(__pyx_nargs != 4)) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = __Pyx_ArgRef_VARARGS(__pyx_args, 0);
      if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(0, 2122, __pyx_L3_error)
      values[1] = __Pyx_ArgRef_VARARGS(__pyx_args, 1);
      if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[1])) __PYX_ERR(0, 2122, __pyx_L3_error)
      values[2] = __Pyx_ArgRef_VARARGS(__pyx_args, 2);
      if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[2])) __PYX_ERR(0, 2122, __pyx_L3_error)
      values[3] = __Pyx_ArgRef_VARARGS(__pyx_args, 3);
      if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[3])) __PYX_ERR(0, 2122, __pyx_L3_error)
    }
    __pyx_v_size = __Pyx_PyLong_As_size_t(values[0]); if (unlikely((__pyx_v_size == (size_t)-1) && PyErr_Occurred())) __PYX_ERR(0, 2122, __pyx_L3_error)
    __pyx_v_malloc_func = values[1];
    __pyx_v_free_func = values[2];
    __pyx_v_device_id = __Pyx_PyLong_As_int(values[3]); if (unlikely((__pyx_v_device_id == (int)-1) && PyErr_Occurred())) __PYX_ERR(0, 2123, __pyx_L3_error)
  }
  goto __pyx_L6_skip;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("__init__", 1, 4, 4, __pyx_nargs); __PYX_ERR(0, 2122, __pyx_L3_error)
  __pyx_L6_skip:;
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_AddTraceback("cupy.cuda.memory.PythonFunctionAllocatorMemory.__init__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return -1;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_4cupy_4cuda_6memory_29PythonFunctionAllocatorMemory___init__(((struct __pyx_obj_4cupy_4cuda_6memory_PythonFunctionAllocatorMemory *)__pyx_v_self), __pyx_v_size, __pyx_v_malloc_func, __pyx_v_free_func, __pyx_v_device_id);

  /* function exit code */
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_4cupy_4cuda_6memory_29PythonFunctionAllocatorMemory___init__(struct __pyx_obj_4cupy_4cuda_6memory_PythonFunctionAllocatorMemory *__pyx_v_self, size_t __pyx_v_size, PyObject *__pyx_v_malloc_func, PyObject *__pyx_v_free_func, int __pyx_v_device_id) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  PyObject *__pyx_t_6 = NULL;
  size_t __pyx_t_7;
  intptr_t __pyx_t_8;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__init__", 0);

  /* "cupy/cuda/memory.pyx":2124
 *     def __init__(self, size_t size, malloc_func, free_func,
 *                  int device_id):
 *         self._free_func = free_func             # <<<<<<<<<<<<<<
 *         self.device_id = device_id
 *         self.size = size
*/
  __Pyx_INCREF(__pyx_v_free_func);
  __Pyx_GIVEREF(__pyx_v_free_func);
  __Pyx_GOTREF(__pyx_v_self->_free_func);
  __Pyx_DECREF(__pyx_v_self->_free_func);
  __pyx_v_self->_free_func = __pyx_v_free_func;

  /* "cupy/cuda/memory.pyx":2125
 *                  int device_id):
 *         self._free_func = free_func
 *         self.device_id = device_id             # <<<<<<<<<<<<<<
 *         self.size = size
 *         self.ptr = 0
*/
  __pyx_v_self->__pyx_base.device_id = __pyx_v_device_id;

  /* "cupy/cuda/memory.pyx":2126
 *         self._free_func = free_func
 *         self.device_id = device_id
 *         self.size = size             # <<<<<<<<<<<<<<
 *         self.ptr = 0
 *         if size > 0:
*/
  __pyx_v_self->__pyx_base.size = __pyx_v_size;

  /* "cupy/cuda/memory.pyx":2127
 *         self.device_id = device_id
 *         self.size = size
 *         self.ptr = 0             # <<<<<<<<<<<<<<
 *         if size > 0:
 *             self.ptr = malloc_func(size, device_id)
*/
  __pyx_v_self->__pyx_base.ptr = 0;

  /* "cupy/cuda/memory.pyx":2128
 *         self.size = size
 *         self.ptr = 0
 *         if size > 0:             # <<<<<<<<<<<<<<
 *             self.ptr = malloc_func(size, device_id)
 * 
*/
  __pyx_t_1 = (__pyx_v_size > 0);
  if (__pyx_t_1) {

    /* "cupy/cuda/memory.pyx":2129
 *         self.ptr = 0
 *         if size > 0:
 *             self.ptr = malloc_func(size, device_id)             # <<<<<<<<<<<<<<
 * 
 *     def __dealloc__(self):
*/
    __pyx_t_3 = NULL;
    __Pyx_INCREF(__pyx_v_malloc_func);
    __pyx_t_4 = __pyx_v_malloc_func; 
    __pyx_t_5 = __Pyx_PyLong_FromSize_t(__pyx_v_size); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 2129, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __pyx_t_6 = __Pyx_PyLong_From_int(__pyx_v_device_id); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 2129, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_6);
    __pyx_t_7 = 1;
    #if CYTHON_UNPACK_METHODS
    if (unlikely(PyMethod_Check(__pyx_t_4))) {
      __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_4);
      assert(__pyx_t_3);
      PyObject* __pyx__function = PyMethod_GET_FUNCTION(__pyx_t_4);
      __Pyx_INCREF(__pyx_t_3);
      __Pyx_INCREF(__pyx__function);
      __Pyx_DECREF_SET(__pyx_t_4, __pyx__function);
      __pyx_t_7 = 0;
    }
    #endif
    {
      PyObject *__pyx_callargs[3] = {__pyx_t_3, __pyx_t_5, __pyx_t_6};
      __pyx_t_2 = __Pyx_PyObject_FastCall((PyObject*)__pyx_t_4, __pyx_callargs+__pyx_t_7, (3-__pyx_t_7) | (__pyx_t_7*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
      __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
      __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
      __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 2129, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_2);
    }
    __pyx_t_8 = PyLong_AsSsize_t(__pyx_t_2); if (unlikely((__pyx_t_8 == ((intptr_t)-1)) && PyErr_Occurred())) __PYX_ERR(0, 2129, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __pyx_v_self->__pyx_base.ptr = __pyx_t_8;

    /* "cupy/cuda/memory.pyx":2128
 *         self.size = size
 *         self.ptr = 0
 *         if size > 0:             # <<<<<<<<<<<<<<
 *             self.ptr = malloc_func(size, device_id)
 * 
*/
  }

  /* "cupy/cuda/memory.pyx":2122
 * cdef class PythonFunctionAllocatorMemory(BaseMemory):
 * 
 *     def __init__(self, size_t size, malloc_func, free_func,             # <<<<<<<<<<<<<<
 *                  int device_id):
 *         self._free_func = free_func
*/

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_AddTraceback("cupy.cuda.memory.PythonFunctionAllocatorMemory.__init__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "cupy/cuda/memory.pyx":2131
 *             self.ptr = malloc_func(size, device_id)
 * 
 *     def __dealloc__(self):             # <<<<<<<<<<<<<<
 *         if self.ptr:
 *             self._free_func(self.ptr, self.device_id)
*/

/* Python wrapper */
static void __pyx_pw_4cupy_4cuda_6memory_29PythonFunctionAllocatorMemory_3__dealloc__(PyObject *__pyx_v_self); /*proto*/
static void __pyx_pw_4cupy_4cuda_6memory_29PythonFunctionAllocatorMemory_3__dealloc__(PyObject *__pyx_v_self) {
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__dealloc__ (wrapper)", 0);
  __pyx_kwvalues = __Pyx_KwValues_VARARGS(__pyx_args, __pyx_nargs);
  __pyx_pf_4cupy_4cuda_6memory_29PythonFunctionAllocatorMemory_2__dealloc__(((struct __pyx_obj_4cupy_4cuda_6memory_PythonFunctionAllocatorMemory *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
}

static void __pyx_pf_4cupy_4cuda_6memory_29PythonFunctionAllocatorMemory_2__dealloc__(struct __pyx_obj_4cupy_4cuda_6memory_PythonFunctionAllocatorMemory *__pyx_v_self) {
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  PyObject *__pyx_t_6 = NULL;
  size_t __pyx_t_7;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__dealloc__", 0);

  /* "cupy/cuda/memory.pyx":2132
 * 
 *     def __dealloc__(self):
 *         if self.ptr:             # <<<<<<<<<<<<<<
 *             self._free_func(self.ptr, self.device_id)
 * 
*/
  __pyx_t_1 = (__pyx_v_self->__pyx_base.ptr != 0);
  if (__pyx_t_1) {

    /* "cupy/cuda/memory.pyx":2133
 *     def __dealloc__(self):
 *         if self.ptr:
 *             self._free_func(self.ptr, self.device_id)             # <<<<<<<<<<<<<<
 * 
 * 
*/
    __pyx_t_3 = NULL;
    __Pyx_INCREF(__pyx_v_self->_free_func);
    __pyx_t_4 = __pyx_v_self->_free_func; 
    __pyx_t_5 = PyLong_FromSsize_t(__pyx_v_self->__pyx_base.ptr); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 2133, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __pyx_t_6 = __Pyx_PyLong_From_int(__pyx_v_self->__pyx_base.device_id); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 2133, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_6);
    __pyx_t_7 = 1;
    #if CYTHON_UNPACK_METHODS
    if (likely(PyMethod_Check(__pyx_t_4))) {
      __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_4);
      assert(__pyx_t_3);
      PyObject* __pyx__function = PyMethod_GET_FUNCTION(__pyx_t_4);
      __Pyx_INCREF(__pyx_t_3);
      __Pyx_INCREF(__pyx__function);
      __Pyx_DECREF_SET(__pyx_t_4, __pyx__function);
      __pyx_t_7 = 0;
    }
    #endif
    {
      PyObject *__pyx_callargs[3] = {__pyx_t_3, __pyx_t_5, __pyx_t_6};
      __pyx_t_2 = __Pyx_PyObject_FastCall((PyObject*)__pyx_t_4, __pyx_callargs+__pyx_t_7, (3-__pyx_t_7) | (__pyx_t_7*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
      __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
      __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
      __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 2133, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_2);
    }
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

    /* "cupy/cuda/memory.pyx":2132
 * 
 *     def __dealloc__(self):
 *         if self.ptr:             # <<<<<<<<<<<<<<
 *             self._free_func(self.ptr, self.device_id)
 * 
*/
  }

  /* "cupy/cuda/memory.pyx":2131
 *             self.ptr = malloc_func(size, device_id)
 * 
 *     def __dealloc__(self):             # <<<<<<<<<<<<<<
 *         if self.ptr:
 *             self._free_func(self.ptr, self.device_id)
*/

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_WriteUnraisable("cupy.cuda.memory.PythonFunctionAllocatorMemory.__dealloc__", __pyx_clineno, __pyx_lineno, __pyx_filename, 1, 0);
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
}

/* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     cdef tuple state
 *     cdef object _dict
*/

/* Python wrapper */
static PyObject *__pyx_pw_4cupy_4cuda_6memory_29PythonFunctionAllocatorMemory_5__reduce_cython__(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
PyDoc_STRVAR(__pyx_doc_4cupy_4cuda_6memory_29PythonFunctionAllocatorMemory_4__reduce_cython__, "PythonFunctionAllocatorMemory.__reduce_cython__(self)");
static PyObject *__pyx_pw_4cupy_4cuda_6memory_29PythonFunctionAllocatorMemory_5__reduce_cython__(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
) {
  #if !CYTHON_METH_FASTCALL
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  #endif
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__reduce_cython__ (wrapper)", 0);
  #if !CYTHON_METH_FASTCALL
  #if CYTHON_ASSUME_SAFE_SIZE
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return NULL;
  #endif
  #endif
  __pyx_kwvalues = __Pyx_KwValues_FASTCALL(__pyx_args, __pyx_nargs);
  if (unlikely(__pyx_nargs > 0)) { __Pyx_RaiseArgtupleInvalid("__reduce_cython__", 1, 0, 0, __pyx_nargs); return NULL; }
  const Py_ssize_t __pyx_kwds_len = unlikely(__pyx_kwds) ? __Pyx_NumKwargs_FASTCALL(__pyx_kwds) : 0;
  if (unlikely(__pyx_kwds_len < 0)) return NULL;
  if (unlikely(__pyx_kwds_len > 0)) {__Pyx_RejectKeywords("__reduce_cython__", __pyx_kwds); return NULL;}
  __pyx_r = __pyx_pf_4cupy_4cuda_6memory_29PythonFunctionAllocatorMemory_4__reduce_cython__(((struct __pyx_obj_4cupy_4cuda_6memory_PythonFunctionAllocatorMemory *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4cupy_4cuda_6memory_29PythonFunctionAllocatorMemory_4__reduce_cython__(struct __pyx_obj_4cupy_4cuda_6memory_PythonFunctionAllocatorMemory *__pyx_v_self) {
  PyObject *__pyx_v_state = 0;
  PyObject *__pyx_v__dict = 0;
  int __pyx_v_use_setstate;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  int __pyx_t_5;
  int __pyx_t_6;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__reduce_cython__", 0);

  /* "(tree fragment)":5
 *     cdef object _dict
 *     cdef bint use_setstate
 *     state = (self._free_func, self.device_id, self.ptr, self.size)             # <<<<<<<<<<<<<<
 *     _dict = getattr(self, '__dict__', None)
 *     if _dict is not None and _dict:
*/
  __pyx_t_1 = __Pyx_PyLong_From_int(__pyx_v_self->__pyx_base.device_id); if (unlikely(!__pyx_t_1)) __PYX_ERR(3, 5, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = PyLong_FromSsize_t(__pyx_v_self->__pyx_base.ptr); if (unlikely(!__pyx_t_2)) __PYX_ERR(3, 5, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = __Pyx_PyLong_FromSize_t(__pyx_v_self->__pyx_base.size); if (unlikely(!__pyx_t_3)) __PYX_ERR(3, 5, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_4 = PyTuple_New(4); if (unlikely(!__pyx_t_4)) __PYX_ERR(3, 5, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_INCREF(__pyx_v_self->_free_func);
  __Pyx_GIVEREF(__pyx_v_self->_free_func);
  if (__Pyx_PyTuple_SET_ITEM(__pyx_t_4, 0, __pyx_v_self->_free_func) != (0)) __PYX_ERR(3, 5, __pyx_L1_error);
  __Pyx_GIVEREF(__pyx_t_1);
  if (__Pyx_PyTuple_SET_ITEM(__pyx_t_4, 1, __pyx_t_1) != (0)) __PYX_ERR(3, 5, __pyx_L1_error);
  __Pyx_GIVEREF(__pyx_t_2);
  if (__Pyx_PyTuple_SET_ITEM(__pyx_t_4, 2, __pyx_t_2) != (0)) __PYX_ERR(3, 5, __pyx_L1_error);
  __Pyx_GIVEREF(__pyx_t_3);
  if (__Pyx_PyTuple_SET_ITEM(__pyx_t_4, 3, __pyx_t_3) != (0)) __PYX_ERR(3, 5, __pyx_L1_error);
  __pyx_t_1 = 0;
  __pyx_t_2 = 0;
  __pyx_t_3 = 0;
  __pyx_v_state = ((PyObject*)__pyx_t_4);
  __pyx_t_4 = 0;

  /* "(tree fragment)":6
 *     cdef bint use_setstate
 *     state = (self._free_func, self.device_id, self.ptr, self.size)
 *     _dict = getattr(self, '__dict__', None)             # <<<<<<<<<<<<<<
 *     if _dict is not None and _dict:
 *         state += (_dict,)
*/
  __pyx_t_4 = __Pyx_GetAttr3(((PyObject *)__pyx_v_self), __pyx_mstate_global->__pyx_n_u_dict, Py_None); if (unlikely(!__pyx_t_4)) __PYX_ERR(3, 6, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_v__dict = __pyx_t_4;
  __pyx_t_4 = 0;

  /* "(tree fragment)":7
 *     state = (self._free_func, self.device_id, self.ptr, self.size)
 *     _dict = getattr(self, '__dict__', None)
 *     if _dict is not None and _dict:             # <<<<<<<<<<<<<<
 *         state += (_dict,)
 *         use_setstate = True
*/
  __pyx_t_6 = (__pyx_v__dict != Py_None);
  if (__pyx_t_6) {
  } else {
    __pyx_t_5 = __pyx_t_6;
    goto __pyx_L4_bool_binop_done;
  }
  __pyx_t_6 = __Pyx_PyObject_IsTrue(__pyx_v__dict); if (unlikely((__pyx_t_6 < 0))) __PYX_ERR(3, 7, __pyx_L1_error)
  __pyx_t_5 = __pyx_t_6;
  __pyx_L4_bool_binop_done:;
  if (__pyx_t_5) {

    /* "(tree fragment)":8
 *     _dict = getattr(self, '__dict__', None)
 *     if _dict is not None and _dict:
 *         state += (_dict,)             # <<<<<<<<<<<<<<
 *         use_setstate = True
 *     else:
*/
    __pyx_t_4 = PyTuple_New(1); if (unlikely(!__pyx_t_4)) __PYX_ERR(3, 8, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_INCREF(__pyx_v__dict);
    __Pyx_GIVEREF(__pyx_v__dict);
    if (__Pyx_PyTuple_SET_ITEM(__pyx_t_4, 0, __pyx_v__dict) != (0)) __PYX_ERR(3, 8, __pyx_L1_error);
    __pyx_t_3 = PyNumber_InPlaceAdd(__pyx_v_state, __pyx_t_4); if (unlikely(!__pyx_t_3)) __PYX_ERR(3, 8, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __Pyx_DECREF_SET(__pyx_v_state, ((PyObject*)__pyx_t_3));
    __pyx_t_3 = 0;

    /* "(tree fragment)":9
 *     if _dict is not None and _dict:
 *         state += (_dict,)
 *         use_setstate = True             # <<<<<<<<<<<<<<
 *     else:
 *         use_setstate = self._free_func is not None
*/
    __pyx_v_use_setstate = 1;

    /* "(tree fragment)":7
 *     state = (self._free_func, self.device_id, self.ptr, self.size)
 *     _dict = getattr(self, '__dict__', None)
 *     if _dict is not None and _dict:             # <<<<<<<<<<<<<<
 *         state += (_dict,)
 *         use_setstate = True
*/
    goto __pyx_L3;
  }

  /* "(tree fragment)":11
 *         use_setstate = True
 *     else:
 *         use_setstate = self._free_func is not None             # <<<<<<<<<<<<<<
 *     if use_setstate:
 *         return __pyx_unpickle_PythonFunctionAllocatorMemory, (type(self), 0xf33b7cb, None), state
*/
  /*else*/ {
    __pyx_t_5 = (__pyx_v_self->_free_func != Py_None);
    __pyx_v_use_setstate = __pyx_t_5;
  }
  __pyx_L3:;

  /* "(tree fragment)":12
 *     else:
 *         use_setstate = self._free_func is not None
 *     if use_setstate:             # <<<<<<<<<<<<<<
 *         return __pyx_unpickle_PythonFunctionAllocatorMemory, (type(self), 0xf33b7cb, None), state
 *     else:
*/
  if (__pyx_v_use_setstate) {

    /* "(tree fragment)":13
 *         use_setstate = self._free_func is not None
 *     if use_setstate:
 *         return __pyx_unpickle_PythonFunctionAllocatorMemory, (type(self), 0xf33b7cb, None), state             # <<<<<<<<<<<<<<
 *     else:
 *         return __pyx_unpickle_PythonFunctionAllocatorMemory, (type(self), 0xf33b7cb, state)
*/
    __Pyx_XDECREF(__pyx_r);
    __Pyx_GetModuleGlobalName(__pyx_t_3, __pyx_mstate_global->__pyx_n_u_pyx_unpickle_PythonFunctionAll); if (unlikely(!__pyx_t_3)) __PYX_ERR(3, 13, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_t_4 = PyTuple_New(3); if (unlikely(!__pyx_t_4)) __PYX_ERR(3, 13, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_INCREF(((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    __Pyx_GIVEREF(((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    if (__Pyx_PyTuple_SET_ITEM(__pyx_t_4, 0, ((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self)))) != (0)) __PYX_ERR(3, 13, __pyx_L1_error);
    __Pyx_INCREF(__pyx_mstate_global->__pyx_int_255047627);
    __Pyx_GIVEREF(__pyx_mstate_global->__pyx_int_255047627);
    if (__Pyx_PyTuple_SET_ITEM(__pyx_t_4, 1, __pyx_mstate_global->__pyx_int_255047627) != (0)) __PYX_ERR(3, 13, __pyx_L1_error);
    __Pyx_INCREF(Py_None);
    __Pyx_GIVEREF(Py_None);
    if (__Pyx_PyTuple_SET_ITEM(__pyx_t_4, 2, Py_None) != (0)) __PYX_ERR(3, 13, __pyx_L1_error);
    __pyx_t_2 = PyTuple_New(3); if (unlikely(!__pyx_t_2)) __PYX_ERR(3, 13, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_GIVEREF(__pyx_t_3);
    if (__Pyx_PyTuple_SET_ITEM(__pyx_t_2, 0, __pyx_t_3) != (0)) __PYX_ERR(3, 13, __pyx_L1_error);
    __Pyx_GIVEREF(__pyx_t_4);
    if (__Pyx_PyTuple_SET_ITEM(__pyx_t_2, 1, __pyx_t_4) != (0)) __PYX_ERR(3, 13, __pyx_L1_error);
    __Pyx_INCREF(__pyx_v_state);
    __Pyx_GIVEREF(__pyx_v_state);
    if (__Pyx_PyTuple_SET_ITEM(__pyx_t_2, 2, __pyx_v_state) != (0)) __PYX_ERR(3, 13, __pyx_L1_error);
    __pyx_t_3 = 0;
    __pyx_t_4 = 0;
    __pyx_r = __pyx_t_2;
    __pyx_t_2 = 0;
    goto __pyx_L0;

    /* "(tree fragment)":12
 *     else:
 *         use_setstate = self._free_func is not None
 *     if use_setstate:             # <<<<<<<<<<<<<<
 *         return __pyx_unpickle_PythonFunctionAllocatorMemory, (type(self), 0xf33b7cb, None), state
 *     else:
*/
  }

  /* "(tree fragment)":15
 *         return __pyx_unpickle_PythonFunctionAllocatorMemory, (type(self), 0xf33b7cb, None), state
 *     else:
 *         return __pyx_unpickle_PythonFunctionAllocatorMemory, (type(self), 0xf33b7cb, state)             # <<<<<<<<<<<<<<
 * def __setstate_cython__(self, __pyx_state):
 *     __pyx_unpickle_PythonFunctionAllocatorMemory__set_state(self, __pyx_state)
*/
  /*else*/ {
    __Pyx_XDECREF(__pyx_r);
    __Pyx_GetModuleGlobalName(__pyx_t_2, __pyx_mstate_global->__pyx_n_u_pyx_unpickle_PythonFunctionAll); if (unlikely(!__pyx_t_2)) __PYX_ERR(3, 15, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_4 = PyTuple_New(3); if (unlikely(!__pyx_t_4)) __PYX_ERR(3, 15, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_INCREF(((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    __Pyx_GIVEREF(((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    if (__Pyx_PyTuple_SET_ITEM(__pyx_t_4, 0, ((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self)))) != (0)) __PYX_ERR(3, 15, __pyx_L1_error);
    __Pyx_INCREF(__pyx_mstate_global->__pyx_int_255047627);
    __Pyx_GIVEREF(__pyx_mstate_global->__pyx_int_255047627);
    if (__Pyx_PyTuple_SET_ITEM(__pyx_t_4, 1, __pyx_mstate_global->__pyx_int_255047627) != (0)) __PYX_ERR(3, 15, __pyx_L1_error);
    __Pyx_INCREF(__pyx_v_state);
    __Pyx_GIVEREF(__pyx_v_state);
    if (__Pyx_PyTuple_SET_ITEM(__pyx_t_4, 2, __pyx_v_state) != (0)) __PYX_ERR(3, 15, __pyx_L1_error);
    __pyx_t_3 = PyTuple_New(2); if (unlikely(!__pyx_t_3)) __PYX_ERR(3, 15, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_GIVEREF(__pyx_t_2);
    if (__Pyx_PyTuple_SET_ITEM(__pyx_t_3, 0, __pyx_t_2) != (0)) __PYX_ERR(3, 15, __pyx_L1_error);
    __Pyx_GIVEREF(__pyx_t_4);
    if (__Pyx_PyTuple_SET_ITEM(__pyx_t_3, 1, __pyx_t_4) != (0)) __PYX_ERR(3, 15, __pyx_L1_error);
    __pyx_t_2 = 0;
    __pyx_t_4 = 0;
    __pyx_r = __pyx_t_3;
    __pyx_t_3 = 0;
    goto __pyx_L0;
  }

  /* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     cdef tuple state
 *     cdef object _dict
*/

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_AddTraceback("cupy.cuda.memory.PythonFunctionAllocatorMemory.__reduce_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_state);
  __Pyx_XDECREF(__pyx_v__dict);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":16
 *     else:
 *         return __pyx_unpickle_PythonFunctionAllocatorMemory, (type(self), 0xf33b7cb, state)
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     __pyx_unpickle_PythonFunctionAllocatorMemory__set_state(self, __pyx_state)
*/

/* Python wrapper */
static PyObject *__pyx_pw_4cupy_4cuda_6memory_29PythonFunctionAllocatorMemory_7__setstate_cython__(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
PyDoc_STRVAR(__pyx_doc_4cupy_4cuda_6memory_29PythonFunctionAllocatorMemory_6__setstate_cython__, "PythonFunctionAllocatorMemory.__setstate_cython__(self, __pyx_state)");
static PyObject *__pyx_pw_4cupy_4cuda_6memory_29PythonFunctionAllocatorMemory_7__setstate_cython__(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
) {
  PyObject *__pyx_v___pyx_state = 0;
  #if !CYTHON_METH_FASTCALL
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  #endif
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject* values[1] = {0};
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__setstate_cython__ (wrapper)", 0);
  #if !CYTHON_METH_FASTCALL
  #if CYTHON_ASSUME_SAFE_SIZE
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return NULL;
  #endif
  #endif
  __pyx_kwvalues = __Pyx_KwValues_FASTCALL(__pyx_args, __pyx_nargs);
  {
    PyObject ** const __pyx_pyargnames[] = {&__pyx_mstate_global->__pyx_n_u_pyx_state,0};
    const Py_ssize_t __pyx_kwds_len = (__pyx_kwds) ? __Pyx_NumKwargs_FASTCALL(__pyx_kwds) : 0;
    if (unlikely(__pyx_kwds_len < 0)) __PYX_ERR(3, 16, __pyx_L3_error)
    if (__pyx_kwds_len > 0) {
      switch (__pyx_nargs) {
        case  1:
        values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(3, 16, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      const Py_ssize_t kwd_pos_args = __pyx_nargs;
      if (__Pyx_ParseKeywords(__pyx_kwds, __pyx_kwvalues, __pyx_pyargnames, 0, values, kwd_pos_args, __pyx_kwds_len, "__setstate_cython__", 0) < (0)) __PYX_ERR(3, 16, __pyx_L3_error)
      for (Py_ssize_t i = __pyx_nargs; i < 1; i++) {
        if (unlikely(!values[i])) { __Pyx_RaiseArgtupleInvalid("__setstate_cython__", 1, 1, 1, i); __PYX_ERR(3, 16, __pyx_L3_error) }
      }
    } else if (unlikely(__pyx_nargs != 1)) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
      if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(3, 16, __pyx_L3_error)
    }
    __pyx_v___pyx_state = values[0];
  }
  goto __pyx_L6_skip;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("__setstate_cython__", 1, 1, 1, __pyx_nargs); __PYX_ERR(3, 16, __pyx_L3_error)
  __pyx_L6_skip:;
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_AddTraceback("cupy.cuda.memory.PythonFunctionAllocatorMemory.__setstate_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_4cupy_4cuda_6memory_29PythonFunctionAllocatorMemory_6__setstate_cython__(((struct __pyx_obj_4cupy_4cuda_6memory_PythonFunctionAllocatorMemory *)__pyx_v_self), __pyx_v___pyx_state);

  /* function exit code */
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4cupy_4cuda_6memory_29PythonFunctionAllocatorMemory_6__setstate_cython__(struct __pyx_obj_4cupy_4cuda_6memory_PythonFunctionAllocatorMemory *__pyx_v_self, PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__setstate_cython__", 0);

  /* "(tree fragment)":17
 *         return __pyx_unpickle_PythonFunctionAllocatorMemory, (type(self), 0xf33b7cb, state)
 * def __setstate_cython__(self, __pyx_state):
 *     __pyx_unpickle_PythonFunctionAllocatorMemory__set_state(self, __pyx_state)             # <<<<<<<<<<<<<<
*/
  __pyx_t_1 = __pyx_v___pyx_state;
  __Pyx_INCREF(__pyx_t_1);
  if (!(likely(PyTuple_CheckExact(__pyx_t_1))||((__pyx_t_1) == Py_None) || __Pyx_RaiseUnexpectedTypeError("tuple", __pyx_t_1))) __PYX_ERR(3, 17, __pyx_L1_error)
  if (unlikely(__pyx_t_1 == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "cannot pass None into a C function argument that is declared 'not None'");
    __PYX_ERR(3, 17, __pyx_L1_error)
  }
  __pyx_t_2 = __pyx_f_4cupy_4cuda_6memory___pyx_unpickle_PythonFunctionAllocatorMemory__set_state(__pyx_v_self, ((PyObject*)__pyx_t_1)); if (unlikely(!__pyx_t_2)) __PYX_ERR(3, 17, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "(tree fragment)":16
 *     else:
 *         return __pyx_unpickle_PythonFunctionAllocatorMemory, (type(self), 0xf33b7cb, state)
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     __pyx_unpickle_PythonFunctionAllocatorMemory__set_state(self, __pyx_state)
*/

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_AddTraceback("cupy.cuda.memory.PythonFunctionAllocatorMemory.__setstate_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "cupy/cuda/memory.pyx":2164
 *     """
 * 
 *     def __init__(self, malloc_func, free_func):             # <<<<<<<<<<<<<<
 *         self._malloc_func = malloc_func
 *         self._free_func = free_func
*/

/* Python wrapper */
static int __pyx_pw_4cupy_4cuda_6memory_23PythonFunctionAllocator_1__init__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static int __pyx_pw_4cupy_4cuda_6memory_23PythonFunctionAllocator_1__init__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_malloc_func = 0;
  PyObject *__pyx_v_free_func = 0;
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject* values[2] = {0,0};
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__init__ (wrapper)", 0);
  #if CYTHON_ASSUME_SAFE_SIZE
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return -1;
  #endif
  __pyx_kwvalues = __Pyx_KwValues_VARARGS(__pyx_args, __pyx_nargs);
  {
    PyObject ** const __pyx_pyargnames[] = {&__pyx_mstate_global->__pyx_n_u_malloc_func,&__pyx_mstate_global->__pyx_n_u_free_func,0};
    const Py_ssize_t __pyx_kwds_len = (__pyx_kwds) ? __Pyx_NumKwargs_VARARGS(__pyx_kwds) : 0;
    if (unlikely(__pyx_kwds_len < 0)) __PYX_ERR(0, 2164, __pyx_L3_error)
    if (__pyx_kwds_len > 0) {
      switch (__pyx_nargs) {
        case  2:
        values[1] = __Pyx_ArgRef_VARARGS(__pyx_args, 1);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[1])) __PYX_ERR(0, 2164, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  1:
        values[0] = __Pyx_ArgRef_VARARGS(__pyx_args, 0);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(0, 2164, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      const Py_ssize_t kwd_pos_args = __pyx_nargs;
      if (__Pyx_ParseKeywords(__pyx_kwds, __pyx_kwvalues, __pyx_pyargnames, 0, values, kwd_pos_args, __pyx_kwds_len, "__init__", 0) < (0)) __PYX_ERR(0, 2164, __pyx_L3_error)
      for (Py_ssize_t i = __pyx_nargs; i < 2; i++) {
        if (unlikely(!values[i])) { __Pyx_RaiseArgtupleInvalid("__init__", 1, 2, 2, i); __PYX_ERR(0, 2164, __pyx_L3_error) }
      }
    } else if (unlikely(__pyx_nargs != 2)) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = __Pyx_ArgRef_VARARGS(__pyx_args, 0);
      if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(0, 2164, __pyx_L3_error)
      values[1] = __Pyx_ArgRef_VARARGS(__pyx_args, 1);
      if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[1])) __PYX_ERR(0, 2164, __pyx_L3_error)
    }
    __pyx_v_malloc_func = values[0];
    __pyx_v_free_func = values[1];
  }
  goto __pyx_L6_skip;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("__init__", 1, 2, 2, __pyx_nargs); __PYX_ERR(0, 2164, __pyx_L3_error)
  __pyx_L6_skip:;
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_AddTraceback("cupy.cuda.memory.PythonFunctionAllocator.__init__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return -1;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_4cupy_4cuda_6memory_23PythonFunctionAllocator___init__(((struct __pyx_obj_4cupy_4cuda_6memory_PythonFunctionAllocator *)__pyx_v_self), __pyx_v_malloc_func, __pyx_v_free_func);

  /* function exit code */
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_4cupy_4cuda_6memory_23PythonFunctionAllocator___init__(struct __pyx_obj_4cupy_4cuda_6memory_PythonFunctionAllocator *__pyx_v_self, PyObject *__pyx_v_malloc_func, PyObject *__pyx_v_free_func) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__init__", 0);

  /* "cupy/cuda/memory.pyx":2165
 * 
 *     def __init__(self, malloc_func, free_func):
 *         self._malloc_func = malloc_func             # <<<<<<<<<<<<<<
 *         self._free_func = free_func
 * 
*/
  __Pyx_INCREF(__pyx_v_malloc_func);
  __Pyx_GIVEREF(__pyx_v_malloc_func);
  __Pyx_GOTREF(__pyx_v_self->_malloc_func);
  __Pyx_DECREF(__pyx_v_self->_malloc_func);
  __pyx_v_self->_malloc_func = __pyx_v_malloc_func;

  /* "cupy/cuda/memory.pyx":2166
 *     def __init__(self, malloc_func, free_func):
 *         self._malloc_func = malloc_func
 *         self._free_func = free_func             # <<<<<<<<<<<<<<
 * 
 *     cpdef MemoryPointer malloc(self, size_t size):
*/
  __Pyx_INCREF(__pyx_v_free_func);
  __Pyx_GIVEREF(__pyx_v_free_func);
  __Pyx_GOTREF(__pyx_v_self->_free_func);
  __Pyx_DECREF(__pyx_v_self->_free_func);
  __pyx_v_self->_free_func = __pyx_v_free_func;

  /* "cupy/cuda/memory.pyx":2164
 *     """
 * 
 *     def __init__(self, malloc_func, free_func):             # <<<<<<<<<<<<<<
 *         self._malloc_func = malloc_func
 *         self._free_func = free_func
*/

  /* function exit code */
  __pyx_r = 0;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "cupy/cuda/memory.pyx":2168
 *         self._free_func = free_func
 * 
 *     cpdef MemoryPointer malloc(self, size_t size):             # <<<<<<<<<<<<<<
 *         mem = PythonFunctionAllocatorMemory(
 *             size, self._malloc_func,
*/

static PyObject *__pyx_pw_4cupy_4cuda_6memory_23PythonFunctionAllocator_3malloc(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
static struct __pyx_obj_4cupy_4cuda_6memory_MemoryPointer *__pyx_f_4cupy_4cuda_6memory_23PythonFunctionAllocator_malloc(struct __pyx_obj_4cupy_4cuda_6memory_PythonFunctionAllocator *__pyx_v_self, size_t __pyx_v_size, int __pyx_skip_dispatch) {
  struct __pyx_obj_4cupy_4cuda_6memory_PythonFunctionAllocatorMemory *__pyx_v_mem = NULL;
  struct __pyx_obj_4cupy_4cuda_6memory_MemoryPointer *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  size_t __pyx_t_6;
  int __pyx_t_7;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("malloc", 0);
  /* Check if called by wrapper */
  if (unlikely(__pyx_skip_dispatch)) ;
  /* Check if overridden in Python */
  else if (
  #if !CYTHON_USE_TYPE_SLOTS
  unlikely(Py_TYPE(((PyObject *)__pyx_v_self)) != __pyx_mstate_global->__pyx_ptype_4cupy_4cuda_6memory_PythonFunctionAllocator &&
  __Pyx_PyType_HasFeature(Py_TYPE(((PyObject *)__pyx_v_self)), Py_TPFLAGS_HAVE_GC))
  #else
  unlikely(Py_TYPE(((PyObject *)__pyx_v_self))->tp_dictoffset != 0 || __Pyx_PyType_HasFeature(Py_TYPE(((PyObject *)__pyx_v_self)), (Py_TPFLAGS_IS_ABSTRACT | Py_TPFLAGS_HEAPTYPE)))
  #endif
  ) {
    #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
    static PY_UINT64_T __pyx_tp_dict_version = __PYX_DICT_VERSION_INIT, __pyx_obj_dict_version = __PYX_DICT_VERSION_INIT;
    if (unlikely(!__Pyx_object_dict_version_matches(((PyObject *)__pyx_v_self), __pyx_tp_dict_version, __pyx_obj_dict_version))) {
      PY_UINT64_T __pyx_typedict_guard = __Pyx_get_tp_dict_version(((PyObject *)__pyx_v_self));
      #endif
      __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_mstate_global->__pyx_n_u_malloc_2); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 2168, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      if (!__Pyx_IsSameCFunction(__pyx_t_1, (void(*)(void)) __pyx_pw_4cupy_4cuda_6memory_23PythonFunctionAllocator_3malloc)) {
        __Pyx_XDECREF((PyObject *)__pyx_r);
        __pyx_t_3 = NULL;
        __Pyx_INCREF(__pyx_t_1);
        __pyx_t_4 = __pyx_t_1; 
        __pyx_t_5 = __Pyx_PyLong_FromSize_t(__pyx_v_size); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 2168, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_5);
        __pyx_t_6 = 1;
        #if CYTHON_UNPACK_METHODS
        if (unlikely(PyMethod_Check(__pyx_t_4))) {
          __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_4);
          assert(__pyx_t_3);
          PyObject* __pyx__function = PyMethod_GET_FUNCTION(__pyx_t_4);
          __Pyx_INCREF(__pyx_t_3);
          __Pyx_INCREF(__pyx__function);
          __Pyx_DECREF_SET(__pyx_t_4, __pyx__function);
          __pyx_t_6 = 0;
        }
        #endif
        {
          PyObject *__pyx_callargs[2] = {__pyx_t_3, __pyx_t_5};
          __pyx_t_2 = __Pyx_PyObject_FastCall((PyObject*)__pyx_t_4, __pyx_callargs+__pyx_t_6, (2-__pyx_t_6) | (__pyx_t_6*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
          __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
          __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
          __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
          if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 2168, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_2);
        }
        if (!(likely(((__pyx_t_2) == Py_None) || likely(__Pyx_TypeTest(__pyx_t_2, __pyx_mstate_global->__pyx_ptype_4cupy_4cuda_6memory_MemoryPointer))))) __PYX_ERR(0, 2168, __pyx_L1_error)
        __pyx_r = ((struct __pyx_obj_4cupy_4cuda_6memory_MemoryPointer *)__pyx_t_2);
        __pyx_t_2 = 0;
        __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
        goto __pyx_L0;
      }
      #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
      __pyx_tp_dict_version = __Pyx_get_tp_dict_version(((PyObject *)__pyx_v_self));
      __pyx_obj_dict_version = __Pyx_get_object_dict_version(((PyObject *)__pyx_v_self));
      if (unlikely(__pyx_typedict_guard != __pyx_tp_dict_version)) {
        __pyx_tp_dict_version = __pyx_obj_dict_version = __PYX_DICT_VERSION_INIT;
      }
      #endif
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
    }
    #endif
  }

  /* "cupy/cuda/memory.pyx":2169
 * 
 *     cpdef MemoryPointer malloc(self, size_t size):
 *         mem = PythonFunctionAllocatorMemory(             # <<<<<<<<<<<<<<
 *             size, self._malloc_func,
 *             self._free_func, device.get_device_id())
*/
  __pyx_t_2 = NULL;

  /* "cupy/cuda/memory.pyx":2170
 *     cpdef MemoryPointer malloc(self, size_t size):
 *         mem = PythonFunctionAllocatorMemory(
 *             size, self._malloc_func,             # <<<<<<<<<<<<<<
 *             self._free_func, device.get_device_id())
 *         return MemoryPointer(mem, 0)
*/
  __pyx_t_4 = __Pyx_PyLong_FromSize_t(__pyx_v_size); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 2170, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);

  /* "cupy/cuda/memory.pyx":2171
 *         mem = PythonFunctionAllocatorMemory(
 *             size, self._malloc_func,
 *             self._free_func, device.get_device_id())             # <<<<<<<<<<<<<<
 *         return MemoryPointer(mem, 0)
*/
  __pyx_t_7 = __pyx_f_4cupy_4cuda_6device_get_device_id(0); if (unlikely(__pyx_t_7 == ((int)-1) && PyErr_Occurred())) __PYX_ERR(0, 2171, __pyx_L1_error)
  __pyx_t_5 = __Pyx_PyLong_From_int(__pyx_t_7); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 2171, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_5);
  __pyx_t_6 = 1;
  {
    PyObject *__pyx_callargs[5] = {__pyx_t_2, __pyx_t_4, __pyx_v_self->_malloc_func, __pyx_v_self->_free_func, __pyx_t_5};
    __pyx_t_1 = __Pyx_PyObject_FastCall((PyObject*)__pyx_mstate_global->__pyx_ptype_4cupy_4cuda_6memory_PythonFunctionAllocatorMemory, __pyx_callargs+__pyx_t_6, (5-__pyx_t_6) | (__pyx_t_6*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
    __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
    if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 2169, __pyx_L1_error)
    __Pyx_GOTREF((PyObject *)__pyx_t_1);
  }
  __pyx_v_mem = ((struct __pyx_obj_4cupy_4cuda_6memory_PythonFunctionAllocatorMemory *)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "cupy/cuda/memory.pyx":2172
 *             size, self._malloc_func,
 *             self._free_func, device.get_device_id())
 *         return MemoryPointer(mem, 0)             # <<<<<<<<<<<<<<
*/
  __Pyx_XDECREF((PyObject *)__pyx_r);
  __pyx_t_5 = NULL;
  __pyx_t_6 = 1;
  {
    PyObject *__pyx_callargs[3] = {__pyx_t_5, ((PyObject *)__pyx_v_mem), __pyx_mstate_global->__pyx_int_0};
    __pyx_t_1 = __Pyx_PyObject_FastCall((PyObject*)__pyx_mstate_global->__pyx_ptype_4cupy_4cuda_6memory_MemoryPointer, __pyx_callargs+__pyx_t_6, (3-__pyx_t_6) | (__pyx_t_6*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
    __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
    if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 2172, __pyx_L1_error)
    __Pyx_GOTREF((PyObject *)__pyx_t_1);
  }
  __pyx_r = ((struct __pyx_obj_4cupy_4cuda_6memory_MemoryPointer *)__pyx_t_1);
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "cupy/cuda/memory.pyx":2168
 *         self._free_func = free_func
 * 
 *     cpdef MemoryPointer malloc(self, size_t size):             # <<<<<<<<<<<<<<
 *         mem = PythonFunctionAllocatorMemory(
 *             size, self._malloc_func,
*/

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_AddTraceback("cupy.cuda.memory.PythonFunctionAllocator.malloc", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XDECREF((PyObject *)__pyx_v_mem);
  __Pyx_XGIVEREF((PyObject *)__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_pw_4cupy_4cuda_6memory_23PythonFunctionAllocator_3malloc(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
PyDoc_STRVAR(__pyx_doc_4cupy_4cuda_6memory_23PythonFunctionAllocator_2malloc, "PythonFunctionAllocator.malloc(self, size_t size) -> MemoryPointer");
static PyObject *__pyx_pw_4cupy_4cuda_6memory_23PythonFunctionAllocator_3malloc(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
) {
  size_t __pyx_v_size;
  #if !CYTHON_METH_FASTCALL
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  #endif
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject* values[1] = {0};
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("malloc (wrapper)", 0);
  #if !CYTHON_METH_FASTCALL
  #if CYTHON_ASSUME_SAFE_SIZE
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return NULL;
  #endif
  #endif
  __pyx_kwvalues = __Pyx_KwValues_FASTCALL(__pyx_args, __pyx_nargs);
  {
    PyObject ** const __pyx_pyargnames[] = {&__pyx_mstate_global->__pyx_n_u_size,0};
    const Py_ssize_t __pyx_kwds_len = (__pyx_kwds) ? __Pyx_NumKwargs_FASTCALL(__pyx_kwds) : 0;
    if (unlikely(__pyx_kwds_len < 0)) __PYX_ERR(0, 2168, __pyx_L3_error)
    if (__pyx_kwds_len > 0) {
      switch (__pyx_nargs) {
        case  1:
        values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(0, 2168, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      const Py_ssize_t kwd_pos_args = __pyx_nargs;
      if (__Pyx_ParseKeywords(__pyx_kwds, __pyx_kwvalues, __pyx_pyargnames, 0, values, kwd_pos_args, __pyx_kwds_len, "malloc", 0) < (0)) __PYX_ERR(0, 2168, __pyx_L3_error)
      for (Py_ssize_t i = __pyx_nargs; i < 1; i++) {
        if (unlikely(!values[i])) { __Pyx_RaiseArgtupleInvalid("malloc", 1, 1, 1, i); __PYX_ERR(0, 2168, __pyx_L3_error) }
      }
    } else if (unlikely(__pyx_nargs != 1)) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
      if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(0, 2168, __pyx_L3_error)
    }
    __pyx_v_size = __Pyx_PyLong_As_size_t(values[0]); if (unlikely((__pyx_v_size == (size_t)-1) && PyErr_Occurred())) __PYX_ERR(0, 2168, __pyx_L3_error)
  }
  goto __pyx_L6_skip;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("malloc", 1, 1, 1, __pyx_nargs); __PYX_ERR(0, 2168, __pyx_L3_error)
  __pyx_L6_skip:;
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_AddTraceback("cupy.cuda.memory.PythonFunctionAllocator.malloc", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_4cupy_4cuda_6memory_23PythonFunctionAllocator_2malloc(((struct __pyx_obj_4cupy_4cuda_6memory_PythonFunctionAllocator *)__pyx_v_self), __pyx_v_size);

  /* function exit code */
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4cupy_4cuda_6memory_23PythonFunctionAllocator_2malloc(struct __pyx_obj_4cupy_4cuda_6memory_PythonFunctionAllocator *__pyx_v_self, size_t __pyx_v_size) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("malloc", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = ((PyObject *)__pyx_f_4cupy_4cuda_6memory_23PythonFunctionAllocator_malloc(__pyx_v_self, __pyx_v_size, 1)); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 2168, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("cupy.cuda.memory.PythonFunctionAllocator.malloc", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     cdef tuple state
 *     cdef object _dict
*/

/* Python wrapper */
static PyObject *__pyx_pw_4cupy_4cuda_6memory_23PythonFunctionAllocator_5__reduce_cython__(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
PyDoc_STRVAR(__pyx_doc_4cupy_4cuda_6memory_23PythonFunctionAllocator_4__reduce_cython__, "PythonFunctionAllocator.__reduce_cython__(self)");
static PyObject *__pyx_pw_4cupy_4cuda_6memory_23PythonFunctionAllocator_5__reduce_cython__(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
) {
  #if !CYTHON_METH_FASTCALL
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  #endif
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__reduce_cython__ (wrapper)", 0);
  #if !CYTHON_METH_FASTCALL
  #if CYTHON_ASSUME_SAFE_SIZE
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return NULL;
  #endif
  #endif
  __pyx_kwvalues = __Pyx_KwValues_FASTCALL(__pyx_args, __pyx_nargs);
  if (unlikely(__pyx_nargs > 0)) { __Pyx_RaiseArgtupleInvalid("__reduce_cython__", 1, 0, 0, __pyx_nargs); return NULL; }
  const Py_ssize_t __pyx_kwds_len = unlikely(__pyx_kwds) ? __Pyx_NumKwargs_FASTCALL(__pyx_kwds) : 0;
  if (unlikely(__pyx_kwds_len < 0)) return NULL;
  if (unlikely(__pyx_kwds_len > 0)) {__Pyx_RejectKeywords("__reduce_cython__", __pyx_kwds); return NULL;}
  __pyx_r = __pyx_pf_4cupy_4cuda_6memory_23PythonFunctionAllocator_4__reduce_cython__(((struct __pyx_obj_4cupy_4cuda_6memory_PythonFunctionAllocator *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4cupy_4cuda_6memory_23PythonFunctionAllocator_4__reduce_cython__(struct __pyx_obj_4cupy_4cuda_6memory_PythonFunctionAllocator *__pyx_v_self) {
  PyObject *__pyx_v_state = 0;
  PyObject *__pyx_v__dict = 0;
  int __pyx_v_use_setstate;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_t_2;
  int __pyx_t_3;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__reduce_cython__", 0);

  /* "(tree fragment)":5
 *     cdef object _dict
 *     cdef bint use_setstate
 *     state = (self._free_func, self._malloc_func)             # <<<<<<<<<<<<<<
 *     _dict = getattr(self, '__dict__', None)
 *     if _dict is not None and _dict:
*/
  __pyx_t_1 = PyTuple_New(2); if (unlikely(!__pyx_t_1)) __PYX_ERR(3, 5, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_INCREF(__pyx_v_self->_free_func);
  __Pyx_GIVEREF(__pyx_v_self->_free_func);
  if (__Pyx_PyTuple_SET_ITEM(__pyx_t_1, 0, __pyx_v_self->_free_func) != (0)) __PYX_ERR(3, 5, __pyx_L1_error);
  __Pyx_INCREF(__pyx_v_self->_malloc_func);
  __Pyx_GIVEREF(__pyx_v_self->_malloc_func);
  if (__Pyx_PyTuple_SET_ITEM(__pyx_t_1, 1, __pyx_v_self->_malloc_func) != (0)) __PYX_ERR(3, 5, __pyx_L1_error);
  __pyx_v_state = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "(tree fragment)":6
 *     cdef bint use_setstate
 *     state = (self._free_func, self._malloc_func)
 *     _dict = getattr(self, '__dict__', None)             # <<<<<<<<<<<<<<
 *     if _dict is not None and _dict:
 *         state += (_dict,)
*/
  __pyx_t_1 = __Pyx_GetAttr3(((PyObject *)__pyx_v_self), __pyx_mstate_global->__pyx_n_u_dict, Py_None); if (unlikely(!__pyx_t_1)) __PYX_ERR(3, 6, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_v__dict = __pyx_t_1;
  __pyx_t_1 = 0;

  /* "(tree fragment)":7
 *     state = (self._free_func, self._malloc_func)
 *     _dict = getattr(self, '__dict__', None)
 *     if _dict is not None and _dict:             # <<<<<<<<<<<<<<
 *         state += (_dict,)
 *         use_setstate = True
*/
  __pyx_t_3 = (__pyx_v__dict != Py_None);
  if (__pyx_t_3) {
  } else {
    __pyx_t_2 = __pyx_t_3;
    goto __pyx_L4_bool_binop_done;
  }
  __pyx_t_3 = __Pyx_PyObject_IsTrue(__pyx_v__dict); if (unlikely((__pyx_t_3 < 0))) __PYX_ERR(3, 7, __pyx_L1_error)
  __pyx_t_2 = __pyx_t_3;
  __pyx_L4_bool_binop_done:;
  if (__pyx_t_2) {

    /* "(tree fragment)":8
 *     _dict = getattr(self, '__dict__', None)
 *     if _dict is not None and _dict:
 *         state += (_dict,)             # <<<<<<<<<<<<<<
 *         use_setstate = True
 *     else:
*/
    __pyx_t_1 = PyTuple_New(1); if (unlikely(!__pyx_t_1)) __PYX_ERR(3, 8, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_INCREF(__pyx_v__dict);
    __Pyx_GIVEREF(__pyx_v__dict);
    if (__Pyx_PyTuple_SET_ITEM(__pyx_t_1, 0, __pyx_v__dict) != (0)) __PYX_ERR(3, 8, __pyx_L1_error);
    __pyx_t_4 = PyNumber_InPlaceAdd(__pyx_v_state, __pyx_t_1); if (unlikely(!__pyx_t_4)) __PYX_ERR(3, 8, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __Pyx_DECREF_SET(__pyx_v_state, ((PyObject*)__pyx_t_4));
    __pyx_t_4 = 0;

    /* "(tree fragment)":9
 *     if _dict is not None and _dict:
 *         state += (_dict,)
 *         use_setstate = True             # <<<<<<<<<<<<<<
 *     else:
 *         use_setstate = self._free_func is not None or self._malloc_func is not None
*/
    __pyx_v_use_setstate = 1;

    /* "(tree fragment)":7
 *     state = (self._free_func, self._malloc_func)
 *     _dict = getattr(self, '__dict__', None)
 *     if _dict is not None and _dict:             # <<<<<<<<<<<<<<
 *         state += (_dict,)
 *         use_setstate = True
*/
    goto __pyx_L3;
  }

  /* "(tree fragment)":11
 *         use_setstate = True
 *     else:
 *         use_setstate = self._free_func is not None or self._malloc_func is not None             # <<<<<<<<<<<<<<
 *     if use_setstate:
 *         return __pyx_unpickle_PythonFunctionAllocator, (type(self), 0xda2aa50, None), state
*/
  /*else*/ {
    __pyx_t_3 = (__pyx_v_self->_free_func != Py_None);
    if (!__pyx_t_3) {
    } else {
      __pyx_t_2 = __pyx_t_3;
      goto __pyx_L6_bool_binop_done;
    }
    __pyx_t_3 = (__pyx_v_self->_malloc_func != Py_None);
    __pyx_t_2 = __pyx_t_3;
    __pyx_L6_bool_binop_done:;
    __pyx_v_use_setstate = __pyx_t_2;
  }
  __pyx_L3:;

  /* "(tree fragment)":12
 *     else:
 *         use_setstate = self._free_func is not None or self._malloc_func is not None
 *     if use_setstate:             # <<<<<<<<<<<<<<
 *         return __pyx_unpickle_PythonFunctionAllocator, (type(self), 0xda2aa50, None), state
 *     else:
*/
  if (__pyx_v_use_setstate) {

    /* "(tree fragment)":13
 *         use_setstate = self._free_func is not None or self._malloc_func is not None
 *     if use_setstate:
 *         return __pyx_unpickle_PythonFunctionAllocator, (type(self), 0xda2aa50, None), state             # <<<<<<<<<<<<<<
 *     else:
 *         return __pyx_unpickle_PythonFunctionAllocator, (type(self), 0xda2aa50, state)
*/
    __Pyx_XDECREF(__pyx_r);
    __Pyx_GetModuleGlobalName(__pyx_t_4, __pyx_mstate_global->__pyx_n_u_pyx_unpickle_PythonFunctionAll_2); if (unlikely(!__pyx_t_4)) __PYX_ERR(3, 13, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __pyx_t_1 = PyTuple_New(3); if (unlikely(!__pyx_t_1)) __PYX_ERR(3, 13, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_INCREF(((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    __Pyx_GIVEREF(((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    if (__Pyx_PyTuple_SET_ITEM(__pyx_t_1, 0, ((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self)))) != (0)) __PYX_ERR(3, 13, __pyx_L1_error);
    __Pyx_INCREF(__pyx_mstate_global->__pyx_int_228764240);
    __Pyx_GIVEREF(__pyx_mstate_global->__pyx_int_228764240);
    if (__Pyx_PyTuple_SET_ITEM(__pyx_t_1, 1, __pyx_mstate_global->__pyx_int_228764240) != (0)) __PYX_ERR(3, 13, __pyx_L1_error);
    __Pyx_INCREF(Py_None);
    __Pyx_GIVEREF(Py_None);
    if (__Pyx_PyTuple_SET_ITEM(__pyx_t_1, 2, Py_None) != (0)) __PYX_ERR(3, 13, __pyx_L1_error);
    __pyx_t_5 = PyTuple_New(3); if (unlikely(!__pyx_t_5)) __PYX_ERR(3, 13, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __Pyx_GIVEREF(__pyx_t_4);
    if (__Pyx_PyTuple_SET_ITEM(__pyx_t_5, 0, __pyx_t_4) != (0)) __PYX_ERR(3, 13, __pyx_L1_error);
    __Pyx_GIVEREF(__pyx_t_1);
    if (__Pyx_PyTuple_SET_ITEM(__pyx_t_5, 1, __pyx_t_1) != (0)) __PYX_ERR(3, 13, __pyx_L1_error);
    __Pyx_INCREF(__pyx_v_state);
    __Pyx_GIVEREF(__pyx_v_state);
    if (__Pyx_PyTuple_SET_ITEM(__pyx_t_5, 2, __pyx_v_state) != (0)) __PYX_ERR(3, 13, __pyx_L1_error);
    __pyx_t_4 = 0;
    __pyx_t_1 = 0;
    __pyx_r = __pyx_t_5;
    __pyx_t_5 = 0;
    goto __pyx_L0;

    /* "(tree fragment)":12
 *     else:
 *         use_setstate = self._free_func is not None or self._malloc_func is not None
 *     if use_setstate:             # <<<<<<<<<<<<<<
 *         return __pyx_unpickle_PythonFunctionAllocator, (type(self), 0xda2aa50, None), state
 *     else:
*/
  }

  /* "(tree fragment)":15
 *         return __pyx_unpickle_PythonFunctionAllocator, (type(self), 0xda2aa50, None), state
 *     else:
 *         return __pyx_unpickle_PythonFunctionAllocator, (type(self), 0xda2aa50, state)             # <<<<<<<<<<<<<<
 * def __setstate_cython__(self, __pyx_state):
 *     __pyx_unpickle_PythonFunctionAllocator__set_state(self, __pyx_state)
*/
  /*else*/ {
    __Pyx_XDECREF(__pyx_r);
    __Pyx_GetModuleGlobalName(__pyx_t_5, __pyx_mstate_global->__pyx_n_u_pyx_unpickle_PythonFunctionAll_2); if (unlikely(!__pyx_t_5)) __PYX_ERR(3, 15, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __pyx_t_1 = PyTuple_New(3); if (unlikely(!__pyx_t_1)) __PYX_ERR(3, 15, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_INCREF(((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    __Pyx_GIVEREF(((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    if (__Pyx_PyTuple_SET_ITEM(__pyx_t_1, 0, ((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self)))) != (0)) __PYX_ERR(3, 15, __pyx_L1_error);
    __Pyx_INCREF(__pyx_mstate_global->__pyx_int_228764240);
    __Pyx_GIVEREF(__pyx_mstate_global->__pyx_int_228764240);
    if (__Pyx_PyTuple_SET_ITEM(__pyx_t_1, 1, __pyx_mstate_global->__pyx_int_228764240) != (0)) __PYX_ERR(3, 15, __pyx_L1_error);
    __Pyx_INCREF(__pyx_v_state);
    __Pyx_GIVEREF(__pyx_v_state);
    if (__Pyx_PyTuple_SET_ITEM(__pyx_t_1, 2, __pyx_v_state) != (0)) __PYX_ERR(3, 15, __pyx_L1_error);
    __pyx_t_4 = PyTuple_New(2); if (unlikely(!__pyx_t_4)) __PYX_ERR(3, 15, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_GIVEREF(__pyx_t_5);
    if (__Pyx_PyTuple_SET_ITEM(__pyx_t_4, 0, __pyx_t_5) != (0)) __PYX_ERR(3, 15, __pyx_L1_error);
    __Pyx_GIVEREF(__pyx_t_1);
    if (__Pyx_PyTuple_SET_ITEM(__pyx_t_4, 1, __pyx_t_1) != (0)) __PYX_ERR(3, 15, __pyx_L1_error);
    __pyx_t_5 = 0;
    __pyx_t_1 = 0;
    __pyx_r = __pyx_t_4;
    __pyx_t_4 = 0;
    goto __pyx_L0;
  }

  /* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     cdef tuple state
 *     cdef object _dict
*/

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_AddTraceback("cupy.cuda.memory.PythonFunctionAllocator.__reduce_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_state);
  __Pyx_XDECREF(__pyx_v__dict);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":16
 *     else:
 *         return __pyx_unpickle_PythonFunctionAllocator, (type(self), 0xda2aa50, state)
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     __pyx_unpickle_PythonFunctionAllocator__set_state(self, __pyx_state)
*/

/* Python wrapper */
static PyObject *__pyx_pw_4cupy_4cuda_6memory_23PythonFunctionAllocator_7__setstate_cython__(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
PyDoc_STRVAR(__pyx_doc_4cupy_4cuda_6memory_23PythonFunctionAllocator_6__setstate_cython__, "PythonFunctionAllocator.__setstate_cython__(self, __pyx_state)");
static PyObject *__pyx_pw_4cupy_4cuda_6memory_23PythonFunctionAllocator_7__setstate_cython__(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
) {
  PyObject *__pyx_v___pyx_state = 0;
  #if !CYTHON_METH_FASTCALL
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  #endif
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject* values[1] = {0};
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__setstate_cython__ (wrapper)", 0);
  #if !CYTHON_METH_FASTCALL
  #if CYTHON_ASSUME_SAFE_SIZE
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return NULL;
  #endif
  #endif
  __pyx_kwvalues = __Pyx_KwValues_FASTCALL(__pyx_args, __pyx_nargs);
  {
    PyObject ** const __pyx_pyargnames[] = {&__pyx_mstate_global->__pyx_n_u_pyx_state,0};
    const Py_ssize_t __pyx_kwds_len = (__pyx_kwds) ? __Pyx_NumKwargs_FASTCALL(__pyx_kwds) : 0;
    if (unlikely(__pyx_kwds_len < 0)) __PYX_ERR(3, 16, __pyx_L3_error)
    if (__pyx_kwds_len > 0) {
      switch (__pyx_nargs) {
        case  1:
        values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(3, 16, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      const Py_ssize_t kwd_pos_args = __pyx_nargs;
      if (__Pyx_ParseKeywords(__pyx_kwds, __pyx_kwvalues, __pyx_pyargnames, 0, values, kwd_pos_args, __pyx_kwds_len, "__setstate_cython__", 0) < (0)) __PYX_ERR(3, 16, __pyx_L3_error)
      for (Py_ssize_t i = __pyx_nargs; i < 1; i++) {
        if (unlikely(!values[i])) { __Pyx_RaiseArgtupleInvalid("__setstate_cython__", 1, 1, 1, i); __PYX_ERR(3, 16, __pyx_L3_error) }
      }
    } else if (unlikely(__pyx_nargs != 1)) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
      if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(3, 16, __pyx_L3_error)
    }
    __pyx_v___pyx_state = values[0];
  }
  goto __pyx_L6_skip;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("__setstate_cython__", 1, 1, 1, __pyx_nargs); __PYX_ERR(3, 16, __pyx_L3_error)
  __pyx_L6_skip:;
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_AddTraceback("cupy.cuda.memory.PythonFunctionAllocator.__setstate_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_4cupy_4cuda_6memory_23PythonFunctionAllocator_6__setstate_cython__(((struct __pyx_obj_4cupy_4cuda_6memory_PythonFunctionAllocator *)__pyx_v_self), __pyx_v___pyx_state);

  /* function exit code */
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4cupy_4cuda_6memory_23PythonFunctionAllocator_6__setstate_cython__(struct __pyx_obj_4cupy_4cuda_6memory_PythonFunctionAllocator *__pyx_v_self, PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__setstate_cython__", 0);

  /* "(tree fragment)":17
 *         return __pyx_unpickle_PythonFunctionAllocator, (type(self), 0xda2aa50, state)
 * def __setstate_cython__(self, __pyx_state):
 *     __pyx_unpickle_PythonFunctionAllocator__set_state(self, __pyx_state)             # <<<<<<<<<<<<<<
*/
  __pyx_t_1 = __pyx_v___pyx_state;
  __Pyx_INCREF(__pyx_t_1);
  if (!(likely(PyTuple_CheckExact(__pyx_t_1))||((__pyx_t_1) == Py_None) || __Pyx_RaiseUnexpectedTypeError("tuple", __pyx_t_1))) __PYX_ERR(3, 17, __pyx_L1_error)
  if (unlikely(__pyx_t_1 == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "cannot pass None into a C function argument that is declared 'not None'");
    __PYX_ERR(3, 17, __pyx_L1_error)
  }
  __pyx_t_2 = __pyx_f_4cupy_4cuda_6memory___pyx_unpickle_PythonFunctionAllocator__set_state(__pyx_v_self, ((PyObject*)__pyx_t_1)); if (unlikely(!__pyx_t_2)) __PYX_ERR(3, 17, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "(tree fragment)":16
 *     else:
 *         return __pyx_unpickle_PythonFunctionAllocator, (type(self), 0xda2aa50, state)
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     __pyx_unpickle_PythonFunctionAllocator__set_state(self, __pyx_state)
*/

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_AddTraceback("cupy.cuda.memory.PythonFunctionAllocator.__setstate_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":4
 *     int __Pyx_CheckUnpickleChecksum(long, long, long, long, const char*) except -1
 *     int __Pyx_UpdateUnpickledDict(object, object, Py_ssize_t) except -1
 * def __pyx_unpickle_BaseMemory(__pyx_type, long __pyx_checksum, tuple __pyx_state):             # <<<<<<<<<<<<<<
 *     cdef object __pyx_result
 *     __Pyx_CheckUnpickleChecksum(__pyx_checksum, 0x61cadb0, 0xe1d5c2a, 0xeeb5490, b'device_id, ptr, size')
*/

/* Python wrapper */
static PyObject *__pyx_pw_4cupy_4cuda_6memory_25__pyx_unpickle_BaseMemory(PyObject *__pyx_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
PyDoc_STRVAR(__pyx_doc_4cupy_4cuda_6memory_24__pyx_unpickle_BaseMemory, "__pyx_unpickle_BaseMemory(__pyx_type, long __pyx_checksum, tuple __pyx_state)");
static PyMethodDef __pyx_mdef_4cupy_4cuda_6memory_25__pyx_unpickle_BaseMemory = {"__pyx_unpickle_BaseMemory", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_4cupy_4cuda_6memory_25__pyx_unpickle_BaseMemory, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_4cupy_4cuda_6memory_24__pyx_unpickle_BaseMemory};
static PyObject *__pyx_pw_4cupy_4cuda_6memory_25__pyx_unpickle_BaseMemory(PyObject *__pyx_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
) {
  PyObject *__pyx_v___pyx_type = 0;
  long __pyx_v___pyx_checksum;
  PyObject *__pyx_v___pyx_state = 0;
  #if !CYTHON_METH_FASTCALL
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  #endif
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject* values[3] = {0,0,0};
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__pyx_unpickle_BaseMemory (wrapper)", 0);
  #if !CYTHON_METH_FASTCALL
  #if CYTHON_ASSUME_SAFE_SIZE
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return NULL;
  #endif
  #endif
  __pyx_kwvalues = __Pyx_KwValues_FASTCALL(__pyx_args, __pyx_nargs);
  {
    PyObject ** const __pyx_pyargnames[] = {&__pyx_mstate_global->__pyx_n_u_pyx_type,&__pyx_mstate_global->__pyx_n_u_pyx_checksum,&__pyx_mstate_global->__pyx_n_u_pyx_state,0};
    const Py_ssize_t __pyx_kwds_len = (__pyx_kwds) ? __Pyx_NumKwargs_FASTCALL(__pyx_kwds) : 0;
    if (unlikely(__pyx_kwds_len < 0)) __PYX_ERR(3, 4, __pyx_L3_error)
    if (__pyx_kwds_len > 0) {
      switch (__pyx_nargs) {
        case  3:
        values[2] = __Pyx_ArgRef_FASTCALL(__pyx_args, 2);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[2])) __PYX_ERR(3, 4, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  2:
        values[1] = __Pyx_ArgRef_FASTCALL(__pyx_args, 1);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[1])) __PYX_ERR(3, 4, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  1:
        values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(3, 4, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      const Py_ssize_t kwd_pos_args = __pyx_nargs;
      if (__Pyx_ParseKeywords(__pyx_kwds, __pyx_kwvalues, __pyx_pyargnames, 0, values, kwd_pos_args, __pyx_kwds_len, "__pyx_unpickle_BaseMemory", 0) < (0)) __PYX_ERR(3, 4, __pyx_L3_error)
      for (Py_ssize_t i = __pyx_nargs; i < 3; i++) {
        if (unlikely(!values[i])) { __Pyx_RaiseArgtupleInvalid("__pyx_unpickle_BaseMemory", 1, 3, 3, i); __PYX_ERR(3, 4, __pyx_L3_error) }
      }
    } else if (unlikely(__pyx_nargs != 3)) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
      if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(3, 4, __pyx_L3_error)
      values[1] = __Pyx_ArgRef_FASTCALL(__pyx_args, 1);
      if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[1])) __PYX_ERR(3, 4, __pyx_L3_error)
      values[2] = __Pyx_ArgRef_FASTCALL(__pyx_args, 2);
      if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[2])) __PYX_ERR(3, 4, __pyx_L3_error)
    }
    __pyx_v___pyx_type = values[0];
    __pyx_v___pyx_checksum = __Pyx_PyLong_As_long(values[1]); if (unlikely((__pyx_v___pyx_checksum == (long)-1) && PyErr_Occurred())) __PYX_ERR(3, 4, __pyx_L3_error)
    __pyx_v___pyx_state = ((PyObject*)values[2]);
  }
  goto __pyx_L6_skip;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("__pyx_unpickle_BaseMemory", 1, 3, 3, __pyx_nargs); __PYX_ERR(3, 4, __pyx_L3_error)
  __pyx_L6_skip:;
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_AddTraceback("cupy.cuda.memory.__pyx_unpickle_BaseMemory", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v___pyx_state), (&PyTuple_Type), 1, "__pyx_state", 1))) __PYX_ERR(3, 4, __pyx_L1_error)
  __pyx_r = __pyx_pf_4cupy_4cuda_6memory_24__pyx_unpickle_BaseMemory(__pyx_self, __pyx_v___pyx_type, __pyx_v___pyx_checksum, __pyx_v___pyx_state);

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = NULL;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  goto __pyx_L7_cleaned_up;
  __pyx_L0:;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __pyx_L7_cleaned_up:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4cupy_4cuda_6memory_24__pyx_unpickle_BaseMemory(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v___pyx_type, long __pyx_v___pyx_checksum, PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_v___pyx_result = 0;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  size_t __pyx_t_4;
  int __pyx_t_5;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__pyx_unpickle_BaseMemory", 0);

  /* "(tree fragment)":6
 * def __pyx_unpickle_BaseMemory(__pyx_type, long __pyx_checksum, tuple __pyx_state):
 *     cdef object __pyx_result
 *     __Pyx_CheckUnpickleChecksum(__pyx_checksum, 0x61cadb0, 0xe1d5c2a, 0xeeb5490, b'device_id, ptr, size')             # <<<<<<<<<<<<<<
 *     __pyx_result = BaseMemory.__new__(__pyx_type)
 *     if __pyx_state is not None:
*/
  __pyx_t_1 = __Pyx_CheckUnpickleChecksum(__pyx_v___pyx_checksum, 0x61cadb0, 0xe1d5c2a, 0xeeb5490, __pyx_k_device_id_ptr_size); if (unlikely(__pyx_t_1 == ((int)-1))) __PYX_ERR(3, 6, __pyx_L1_error)

  /* "(tree fragment)":7
 *     cdef object __pyx_result
 *     __Pyx_CheckUnpickleChecksum(__pyx_checksum, 0x61cadb0, 0xe1d5c2a, 0xeeb5490, b'device_id, ptr, size')
 *     __pyx_result = BaseMemory.__new__(__pyx_type)             # <<<<<<<<<<<<<<
 *     if __pyx_state is not None:
 *         __pyx_unpickle_BaseMemory__set_state(<BaseMemory> __pyx_result, __pyx_state)
*/
  __pyx_t_3 = ((PyObject *)__pyx_mstate_global->__pyx_ptype_4cupy_4cuda_6memory_BaseMemory);
  __Pyx_INCREF(__pyx_t_3);
  __pyx_t_4 = 0;
  {
    PyObject *__pyx_callargs[2] = {__pyx_t_3, __pyx_v___pyx_type};
    __pyx_t_2 = __Pyx_PyObject_FastCallMethod((PyObject*)__pyx_mstate_global->__pyx_n_u_new, __pyx_callargs+__pyx_t_4, (2-__pyx_t_4) | (1*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
    __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
    if (unlikely(!__pyx_t_2)) __PYX_ERR(3, 7, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
  }
  __pyx_v___pyx_result = __pyx_t_2;
  __pyx_t_2 = 0;

  /* "(tree fragment)":8
 *     __Pyx_CheckUnpickleChecksum(__pyx_checksum, 0x61cadb0, 0xe1d5c2a, 0xeeb5490, b'device_id, ptr, size')
 *     __pyx_result = BaseMemory.__new__(__pyx_type)
 *     if __pyx_state is not None:             # <<<<<<<<<<<<<<
 *         __pyx_unpickle_BaseMemory__set_state(<BaseMemory> __pyx_result, __pyx_state)
 *     return __pyx_result
*/
  __pyx_t_5 = (__pyx_v___pyx_state != ((PyObject*)Py_None));
  if (__pyx_t_5) {

    /* "(tree fragment)":9
 *     __pyx_result = BaseMemory.__new__(__pyx_type)
 *     if __pyx_state is not None:
 *         __pyx_unpickle_BaseMemory__set_state(<BaseMemory> __pyx_result, __pyx_state)             # <<<<<<<<<<<<<<
 *     return __pyx_result
 * cdef __pyx_unpickle_BaseMemory__set_state(BaseMemory __pyx_result, __pyx_state: tuple):
*/
    if (unlikely(__pyx_v___pyx_state == Py_None)) {
      PyErr_SetString(PyExc_TypeError, "cannot pass None into a C function argument that is declared 'not None'");
      __PYX_ERR(3, 9, __pyx_L1_error)
    }
    __pyx_t_2 = __pyx_f_4cupy_4cuda_6memory___pyx_unpickle_BaseMemory__set_state(((struct __pyx_obj_4cupy_4cuda_6memory_BaseMemory *)__pyx_v___pyx_result), __pyx_v___pyx_state); if (unlikely(!__pyx_t_2)) __PYX_ERR(3, 9, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

    /* "(tree fragment)":8
 *     __Pyx_CheckUnpickleChecksum(__pyx_checksum, 0x61cadb0, 0xe1d5c2a, 0xeeb5490, b'device_id, ptr, size')
 *     __pyx_result = BaseMemory.__new__(__pyx_type)
 *     if __pyx_state is not None:             # <<<<<<<<<<<<<<
 *         __pyx_unpickle_BaseMemory__set_state(<BaseMemory> __pyx_result, __pyx_state)
 *     return __pyx_result
*/
  }

  /* "(tree fragment)":10
 *     if __pyx_state is not None:
 *         __pyx_unpickle_BaseMemory__set_state(<BaseMemory> __pyx_result, __pyx_state)
 *     return __pyx_result             # <<<<<<<<<<<<<<
 * cdef __pyx_unpickle_BaseMemory__set_state(BaseMemory __pyx_result, __pyx_state: tuple):
 *     __pyx_result.device_id = __pyx_state[0]; __pyx_result.ptr = __pyx_state[1]; __pyx_result.size = __pyx_state[2]
*/
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_v___pyx_result);
  __pyx_r = __pyx_v___pyx_result;
  goto __pyx_L0;

  /* "(tree fragment)":4
 *     int __Pyx_CheckUnpickleChecksum(long, long, long, long, const char*) except -1
 *     int __Pyx_UpdateUnpickledDict(object, object, Py_ssize_t) except -1
 * def __pyx_unpickle_BaseMemory(__pyx_type, long __pyx_checksum, tuple __pyx_state):             # <<<<<<<<<<<<<<
 *     cdef object __pyx_result
 *     __Pyx_CheckUnpickleChecksum(__pyx_checksum, 0x61cadb0, 0xe1d5c2a, 0xeeb5490, b'device_id, ptr, size')
*/

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_AddTraceback("cupy.cuda.memory.__pyx_unpickle_BaseMemory", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v___pyx_result);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":11
 *         __pyx_unpickle_BaseMemory__set_state(<BaseMemory> __pyx_result, __pyx_state)
 *     return __pyx_result
 * cdef __pyx_unpickle_BaseMemory__set_state(BaseMemory __pyx_result, __pyx_state: tuple):             # <<<<<<<<<<<<<<
 *     __pyx_result.device_id = __pyx_state[0]; __pyx_result.ptr = __pyx_state[1]; __pyx_result.size = __pyx_state[2]
 *     __Pyx_UpdateUnpickledDict(__pyx_result, __pyx_state, 3)
*/

static PyObject *__pyx_f_4cupy_4cuda_6memory___pyx_unpickle_BaseMemory__set_state(struct __pyx_obj_4cupy_4cuda_6memory_BaseMemory *__pyx_v___pyx_result, PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_t_2;
  intptr_t __pyx_t_3;
  size_t __pyx_t_4;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__pyx_unpickle_BaseMemory__set_state", 0);

  /* "(tree fragment)":12
 *     return __pyx_result
 * cdef __pyx_unpickle_BaseMemory__set_state(BaseMemory __pyx_result, __pyx_state: tuple):
 *     __pyx_result.device_id = __pyx_state[0]; __pyx_result.ptr = __pyx_state[1]; __pyx_result.size = __pyx_state[2]             # <<<<<<<<<<<<<<
 *     __Pyx_UpdateUnpickledDict(__pyx_result, __pyx_state, 3)
*/
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 0, long, 1, __Pyx_PyLong_From_long, 0, 0, 1, 1, __Pyx_ReferenceSharing_FunctionArgument); if (unlikely(!__pyx_t_1)) __PYX_ERR(3, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyLong_As_int(__pyx_t_1); if (unlikely((__pyx_t_2 == (int)-1) && PyErr_Occurred())) __PYX_ERR(3, 12, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v___pyx_result->device_id = __pyx_t_2;
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 1, long, 1, __Pyx_PyLong_From_long, 0, 0, 1, 1, __Pyx_ReferenceSharing_FunctionArgument); if (unlikely(!__pyx_t_1)) __PYX_ERR(3, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_3 = PyLong_AsSsize_t(__pyx_t_1); if (unlikely((__pyx_t_3 == ((intptr_t)-1)) && PyErr_Occurred())) __PYX_ERR(3, 12, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v___pyx_result->ptr = __pyx_t_3;
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 2, long, 1, __Pyx_PyLong_From_long, 0, 0, 1, 1, __Pyx_ReferenceSharing_FunctionArgument); if (unlikely(!__pyx_t_1)) __PYX_ERR(3, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_4 = __Pyx_PyLong_As_size_t(__pyx_t_1); if (unlikely((__pyx_t_4 == (size_t)-1) && PyErr_Occurred())) __PYX_ERR(3, 12, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v___pyx_result->size = __pyx_t_4;

  /* "(tree fragment)":13
 * cdef __pyx_unpickle_BaseMemory__set_state(BaseMemory __pyx_result, __pyx_state: tuple):
 *     __pyx_result.device_id = __pyx_state[0]; __pyx_result.ptr = __pyx_state[1]; __pyx_result.size = __pyx_state[2]
 *     __Pyx_UpdateUnpickledDict(__pyx_result, __pyx_state, 3)             # <<<<<<<<<<<<<<
*/
  __pyx_t_2 = __Pyx_UpdateUnpickledDict(((PyObject *)__pyx_v___pyx_result), __pyx_v___pyx_state, 3); if (unlikely(__pyx_t_2 == ((int)-1))) __PYX_ERR(3, 13, __pyx_L1_error)

  /* "(tree fragment)":11
 *         __pyx_unpickle_BaseMemory__set_state(<BaseMemory> __pyx_result, __pyx_state)
 *     return __pyx_result
 * cdef __pyx_unpickle_BaseMemory__set_state(BaseMemory __pyx_result, __pyx_state: tuple):             # <<<<<<<<<<<<<<
 *     __pyx_result.device_id = __pyx_state[0]; __pyx_result.ptr = __pyx_state[1]; __pyx_result.size = __pyx_state[2]
 *     __Pyx_UpdateUnpickledDict(__pyx_result, __pyx_state, 3)
*/

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("cupy.cuda.memory.__pyx_unpickle_BaseMemory__set_state", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":4
 *     int __Pyx_CheckUnpickleChecksum(long, long, long, long, const char*) except -1
 *     int __Pyx_UpdateUnpickledDict(object, object, Py_ssize_t) except -1
 * def __pyx_unpickle_Memory(__pyx_type, long __pyx_checksum, tuple __pyx_state):             # <<<<<<<<<<<<<<
 *     cdef object __pyx_result
 *     __Pyx_CheckUnpickleChecksum(__pyx_checksum, 0x61cadb0, 0xe1d5c2a, 0xeeb5490, b'device_id, ptr, size')
*/

/* Python wrapper */
static PyObject *__pyx_pw_4cupy_4cuda_6memory_27__pyx_unpickle_Memory(PyObject *__pyx_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
PyDoc_STRVAR(__pyx_doc_4cupy_4cuda_6memory_26__pyx_unpickle_Memory, "__pyx_unpickle_Memory(__pyx_type, long __pyx_checksum, tuple __pyx_state)");
static PyMethodDef __pyx_mdef_4cupy_4cuda_6memory_27__pyx_unpickle_Memory = {"__pyx_unpickle_Memory", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_4cupy_4cuda_6memory_27__pyx_unpickle_Memory, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_4cupy_4cuda_6memory_26__pyx_unpickle_Memory};
static PyObject *__pyx_pw_4cupy_4cuda_6memory_27__pyx_unpickle_Memory(PyObject *__pyx_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
) {
  PyObject *__pyx_v___pyx_type = 0;
  long __pyx_v___pyx_checksum;
  PyObject *__pyx_v___pyx_state = 0;
  #if !CYTHON_METH_FASTCALL
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  #endif
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject* values[3] = {0,0,0};
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__pyx_unpickle_Memory (wrapper)", 0);
  #if !CYTHON_METH_FASTCALL
  #if CYTHON_ASSUME_SAFE_SIZE
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return NULL;
  #endif
  #endif
  __pyx_kwvalues = __Pyx_KwValues_FASTCALL(__pyx_args, __pyx_nargs);
  {
    PyObject ** const __pyx_pyargnames[] = {&__pyx_mstate_global->__pyx_n_u_pyx_type,&__pyx_mstate_global->__pyx_n_u_pyx_checksum,&__pyx_mstate_global->__pyx_n_u_pyx_state,0};
    const Py_ssize_t __pyx_kwds_len = (__pyx_kwds) ? __Pyx_NumKwargs_FASTCALL(__pyx_kwds) : 0;
    if (unlikely(__pyx_kwds_len < 0)) __PYX_ERR(3, 4, __pyx_L3_error)
    if (__pyx_kwds_len > 0) {
      switch (__pyx_nargs) {
        case  3:
        values[2] = __Pyx_ArgRef_FASTCALL(__pyx_args, 2);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[2])) __PYX_ERR(3, 4, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  2:
        values[1] = __Pyx_ArgRef_FASTCALL(__pyx_args, 1);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[1])) __PYX_ERR(3, 4, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  1:
        values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(3, 4, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      const Py_ssize_t kwd_pos_args = __pyx_nargs;
      if (__Pyx_ParseKeywords(__pyx_kwds, __pyx_kwvalues, __pyx_pyargnames, 0, values, kwd_pos_args, __pyx_kwds_len, "__pyx_unpickle_Memory", 0) < (0)) __PYX_ERR(3, 4, __pyx_L3_error)
      for (Py_ssize_t i = __pyx_nargs; i < 3; i++) {
        if (unlikely(!values[i])) { __Pyx_RaiseArgtupleInvalid("__pyx_unpickle_Memory", 1, 3, 3, i); __PYX_ERR(3, 4, __pyx_L3_error) }
      }
    } else if (unlikely(__pyx_nargs != 3)) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
      if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(3, 4, __pyx_L3_error)
      values[1] = __Pyx_ArgRef_FASTCALL(__pyx_args, 1);
      if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[1])) __PYX_ERR(3, 4, __pyx_L3_error)
      values[2] = __Pyx_ArgRef_FASTCALL(__pyx_args, 2);
      if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[2])) __PYX_ERR(3, 4, __pyx_L3_error)
    }
    __pyx_v___pyx_type = values[0];
    __pyx_v___pyx_checksum = __Pyx_PyLong_As_long(values[1]); if (unlikely((__pyx_v___pyx_checksum == (long)-1) && PyErr_Occurred())) __PYX_ERR(3, 4, __pyx_L3_error)
    __pyx_v___pyx_state = ((PyObject*)values[2]);
  }
  goto __pyx_L6_skip;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("__pyx_unpickle_Memory", 1, 3, 3, __pyx_nargs); __PYX_ERR(3, 4, __pyx_L3_error)
  __pyx_L6_skip:;
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_AddTraceback("cupy.cuda.memory.__pyx_unpickle_Memory", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v___pyx_state), (&PyTuple_Type), 1, "__pyx_state", 1))) __PYX_ERR(3, 4, __pyx_L1_error)
  __pyx_r = __pyx_pf_4cupy_4cuda_6memory_26__pyx_unpickle_Memory(__pyx_self, __pyx_v___pyx_type, __pyx_v___pyx_checksum, __pyx_v___pyx_state);

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = NULL;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  goto __pyx_L7_cleaned_up;
  __pyx_L0:;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __pyx_L7_cleaned_up:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4cupy_4cuda_6memory_26__pyx_unpickle_Memory(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v___pyx_type, long __pyx_v___pyx_checksum, PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_v___pyx_result = 0;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  size_t __pyx_t_4;
  int __pyx_t_5;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__pyx_unpickle_Memory", 0);

  /* "(tree fragment)":6
 * def __pyx_unpickle_Memory(__pyx_type, long __pyx_checksum, tuple __pyx_state):
 *     cdef object __pyx_result
 *     __Pyx_CheckUnpickleChecksum(__pyx_checksum, 0x61cadb0, 0xe1d5c2a, 0xeeb5490, b'device_id, ptr, size')             # <<<<<<<<<<<<<<
 *     __pyx_result = Memory.__new__(__pyx_type)
 *     if __pyx_state is not None:
*/
  __pyx_t_1 = __Pyx_CheckUnpickleChecksum(__pyx_v___pyx_checksum, 0x61cadb0, 0xe1d5c2a, 0xeeb5490, __pyx_k_device_id_ptr_size); if (unlikely(__pyx_t_1 == ((int)-1))) __PYX_ERR(3, 6, __pyx_L1_error)

  /* "(tree fragment)":7
 *     cdef object __pyx_result
 *     __Pyx_CheckUnpickleChecksum(__pyx_checksum, 0x61cadb0, 0xe1d5c2a, 0xeeb5490, b'device_id, ptr, size')
 *     __pyx_result = Memory.__new__(__pyx_type)             # <<<<<<<<<<<<<<
 *     if __pyx_state is not None:
 *         __pyx_unpickle_Memory__set_state(<Memory> __pyx_result, __pyx_state)
*/
  __pyx_t_3 = ((PyObject *)__pyx_mstate_global->__pyx_ptype_4cupy_4cuda_6memory_Memory);
  __Pyx_INCREF(__pyx_t_3);
  __pyx_t_4 = 0;
  {
    PyObject *__pyx_callargs[2] = {__pyx_t_3, __pyx_v___pyx_type};
    __pyx_t_2 = __Pyx_PyObject_FastCallMethod((PyObject*)__pyx_mstate_global->__pyx_n_u_new, __pyx_callargs+__pyx_t_4, (2-__pyx_t_4) | (1*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
    __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
    if (unlikely(!__pyx_t_2)) __PYX_ERR(3, 7, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
  }
  __pyx_v___pyx_result = __pyx_t_2;
  __pyx_t_2 = 0;

  /* "(tree fragment)":8
 *     __Pyx_CheckUnpickleChecksum(__pyx_checksum, 0x61cadb0, 0xe1d5c2a, 0xeeb5490, b'device_id, ptr, size')
 *     __pyx_result = Memory.__new__(__pyx_type)
 *     if __pyx_state is not None:             # <<<<<<<<<<<<<<
 *         __pyx_unpickle_Memory__set_state(<Memory> __pyx_result, __pyx_state)
 *     return __pyx_result
*/
  __pyx_t_5 = (__pyx_v___pyx_state != ((PyObject*)Py_None));
  if (__pyx_t_5) {

    /* "(tree fragment)":9
 *     __pyx_result = Memory.__new__(__pyx_type)
 *     if __pyx_state is not None:
 *         __pyx_unpickle_Memory__set_state(<Memory> __pyx_result, __pyx_state)             # <<<<<<<<<<<<<<
 *     return __pyx_result
 * cdef __pyx_unpickle_Memory__set_state(Memory __pyx_result, __pyx_state: tuple):
*/
    if (unlikely(__pyx_v___pyx_state == Py_None)) {
      PyErr_SetString(PyExc_TypeError, "cannot pass None into a C function argument that is declared 'not None'");
      __PYX_ERR(3, 9, __pyx_L1_error)
    }
    __pyx_t_2 = __pyx_f_4cupy_4cuda_6memory___pyx_unpickle_Memory__set_state(((struct __pyx_obj_4cupy_4cuda_6memory_Memory *)__pyx_v___pyx_result), __pyx_v___pyx_state); if (unlikely(!__pyx_t_2)) __PYX_ERR(3, 9, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

    /* "(tree fragment)":8
 *     __Pyx_CheckUnpickleChecksum(__pyx_checksum, 0x61cadb0, 0xe1d5c2a, 0xeeb5490, b'device_id, ptr, size')
 *     __pyx_result = Memory.__new__(__pyx_type)
 *     if __pyx_state is not None:             # <<<<<<<<<<<<<<
 *         __pyx_unpickle_Memory__set_state(<Memory> __pyx_result, __pyx_state)
 *     return __pyx_result
*/
  }

  /* "(tree fragment)":10
 *     if __pyx_state is not None:
 *         __pyx_unpickle_Memory__set_state(<Memory> __pyx_result, __pyx_state)
 *     return __pyx_result             # <<<<<<<<<<<<<<
 * cdef __pyx_unpickle_Memory__set_state(Memory __pyx_result, __pyx_state: tuple):
 *     __pyx_result.device_id = __pyx_state[0]; __pyx_result.ptr = __pyx_state[1]; __pyx_result.size = __pyx_state[2]
*/
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_v___pyx_result);
  __pyx_r = __pyx_v___pyx_result;
  goto __pyx_L0;

  /* "(tree fragment)":4
 *     int __Pyx_CheckUnpickleChecksum(long, long, long, long, const char*) except -1
 *     int __Pyx_UpdateUnpickledDict(object, object, Py_ssize_t) except -1
 * def __pyx_unpickle_Memory(__pyx_type, long __pyx_checksum, tuple __pyx_state):             # <<<<<<<<<<<<<<
 *     cdef object __pyx_result
 *     __Pyx_CheckUnpickleChecksum(__pyx_checksum, 0x61cadb0, 0xe1d5c2a, 0xeeb5490, b'device_id, ptr, size')
*/

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_AddTraceback("cupy.cuda.memory.__pyx_unpickle_Memory", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v___pyx_result);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":11
 *         __pyx_unpickle_Memory__set_state(<Memory> __pyx_result, __pyx_state)
 *     return __pyx_result
 * cdef __pyx_unpickle_Memory__set_state(Memory __pyx_result, __pyx_state: tuple):             # <<<<<<<<<<<<<<
 *     __pyx_result.device_id = __pyx_state[0]; __pyx_result.ptr = __pyx_state[1]; __pyx_result.size = __pyx_state[2]
 *     __Pyx_UpdateUnpickledDict(__pyx_result, __pyx_state, 3)
*/

static PyObject *__pyx_f_4cupy_4cuda_6memory___pyx_unpickle_Memory__set_state(struct __pyx_obj_4cupy_4cuda_6memory_Memory *__pyx_v___pyx_result, PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_t_2;
  intptr_t __pyx_t_3;
  size_t __pyx_t_4;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__pyx_unpickle_Memory__set_state", 0);

  /* "(tree fragment)":12
 *     return __pyx_result
 * cdef __pyx_unpickle_Memory__set_state(Memory __pyx_result, __pyx_state: tuple):
 *     __pyx_result.device_id = __pyx_state[0]; __pyx_result.ptr = __pyx_state[1]; __pyx_result.size = __pyx_state[2]             # <<<<<<<<<<<<<<
 *     __Pyx_UpdateUnpickledDict(__pyx_result, __pyx_state, 3)
*/
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 0, long, 1, __Pyx_PyLong_From_long, 0, 0, 1, 1, __Pyx_ReferenceSharing_FunctionArgument); if (unlikely(!__pyx_t_1)) __PYX_ERR(3, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyLong_As_int(__pyx_t_1); if (unlikely((__pyx_t_2 == (int)-1) && PyErr_Occurred())) __PYX_ERR(3, 12, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v___pyx_result->__pyx_base.device_id = __pyx_t_2;
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 1, long, 1, __Pyx_PyLong_From_long, 0, 0, 1, 1, __Pyx_ReferenceSharing_FunctionArgument); if (unlikely(!__pyx_t_1)) __PYX_ERR(3, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_3 = PyLong_AsSsize_t(__pyx_t_1); if (unlikely((__pyx_t_3 == ((intptr_t)-1)) && PyErr_Occurred())) __PYX_ERR(3, 12, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v___pyx_result->__pyx_base.ptr = __pyx_t_3;
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 2, long, 1, __Pyx_PyLong_From_long, 0, 0, 1, 1, __Pyx_ReferenceSharing_FunctionArgument); if (unlikely(!__pyx_t_1)) __PYX_ERR(3, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_4 = __Pyx_PyLong_As_size_t(__pyx_t_1); if (unlikely((__pyx_t_4 == (size_t)-1) && PyErr_Occurred())) __PYX_ERR(3, 12, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v___pyx_result->__pyx_base.size = __pyx_t_4;

  /* "(tree fragment)":13
 * cdef __pyx_unpickle_Memory__set_state(Memory __pyx_result, __pyx_state: tuple):
 *     __pyx_result.device_id = __pyx_state[0]; __pyx_result.ptr = __pyx_state[1]; __pyx_result.size = __pyx_state[2]
 *     __Pyx_UpdateUnpickledDict(__pyx_result, __pyx_state, 3)             # <<<<<<<<<<<<<<
*/
  __pyx_t_2 = __Pyx_UpdateUnpickledDict(((PyObject *)__pyx_v___pyx_result), __pyx_v___pyx_state, 3); if (unlikely(__pyx_t_2 == ((int)-1))) __PYX_ERR(3, 13, __pyx_L1_error)

  /* "(tree fragment)":11
 *         __pyx_unpickle_Memory__set_state(<Memory> __pyx_result, __pyx_state)
 *     return __pyx_result
 * cdef __pyx_unpickle_Memory__set_state(Memory __pyx_result, __pyx_state: tuple):             # <<<<<<<<<<<<<<
 *     __pyx_result.device_id = __pyx_state[0]; __pyx_result.ptr = __pyx_state[1]; __pyx_result.size = __pyx_state[2]
 *     __Pyx_UpdateUnpickledDict(__pyx_result, __pyx_state, 3)
*/

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("cupy.cuda.memory.__pyx_unpickle_Memory__set_state", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":4
 *     int __Pyx_CheckUnpickleChecksum(long, long, long, long, const char*) except -1
 *     int __Pyx_UpdateUnpickledDict(object, object, Py_ssize_t) except -1
 * def __pyx_unpickle_MemoryAsync(__pyx_type, long __pyx_checksum, tuple __pyx_state):             # <<<<<<<<<<<<<<
 *     cdef object __pyx_result
 *     __Pyx_CheckUnpickleChecksum(__pyx_checksum, 0x0e18a1e, 0xd8681f6, 0xf0c20a1, b'device_id, ptr, size, stream_ref')
*/

/* Python wrapper */
static PyObject *__pyx_pw_4cupy_4cuda_6memory_29__pyx_unpickle_MemoryAsync(PyObject *__pyx_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
PyDoc_STRVAR(__pyx_doc_4cupy_4cuda_6memory_28__pyx_unpickle_MemoryAsync, "__pyx_unpickle_MemoryAsync(__pyx_type, long __pyx_checksum, tuple __pyx_state)");
static PyMethodDef __pyx_mdef_4cupy_4cuda_6memory_29__pyx_unpickle_MemoryAsync = {"__pyx_unpickle_MemoryAsync", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_4cupy_4cuda_6memory_29__pyx_unpickle_MemoryAsync, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_4cupy_4cuda_6memory_28__pyx_unpickle_MemoryAsync};
static PyObject *__pyx_pw_4cupy_4cuda_6memory_29__pyx_unpickle_MemoryAsync(PyObject *__pyx_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
) {
  PyObject *__pyx_v___pyx_type = 0;
  long __pyx_v___pyx_checksum;
  PyObject *__pyx_v___pyx_state = 0;
  #if !CYTHON_METH_FASTCALL
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  #endif
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject* values[3] = {0,0,0};
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__pyx_unpickle_MemoryAsync (wrapper)", 0);
  #if !CYTHON_METH_FASTCALL
  #if CYTHON_ASSUME_SAFE_SIZE
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return NULL;
  #endif
  #endif
  __pyx_kwvalues = __Pyx_KwValues_FASTCALL(__pyx_args, __pyx_nargs);
  {
    PyObject ** const __pyx_pyargnames[] = {&__pyx_mstate_global->__pyx_n_u_pyx_type,&__pyx_mstate_global->__pyx_n_u_pyx_checksum,&__pyx_mstate_global->__pyx_n_u_pyx_state,0};
    const Py_ssize_t __pyx_kwds_len = (__pyx_kwds) ? __Pyx_NumKwargs_FASTCALL(__pyx_kwds) : 0;
    if (unlikely(__pyx_kwds_len < 0)) __PYX_ERR(3, 4, __pyx_L3_error)
    if (__pyx_kwds_len > 0) {
      switch (__pyx_nargs) {
        case  3:
        values[2] = __Pyx_ArgRef_FASTCALL(__pyx_args, 2);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[2])) __PYX_ERR(3, 4, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  2:
        values[1] = __Pyx_ArgRef_FASTCALL(__pyx_args, 1);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[1])) __PYX_ERR(3, 4, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  1:
        values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(3, 4, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      const Py_ssize_t kwd_pos_args = __pyx_nargs;
      if (__Pyx_ParseKeywords(__pyx_kwds, __pyx_kwvalues, __pyx_pyargnames, 0, values, kwd_pos_args, __pyx_kwds_len, "__pyx_unpickle_MemoryAsync", 0) < (0)) __PYX_ERR(3, 4, __pyx_L3_error)
      for (Py_ssize_t i = __pyx_nargs; i < 3; i++) {
        if (unlikely(!values[i])) { __Pyx_RaiseArgtupleInvalid("__pyx_unpickle_MemoryAsync", 1, 3, 3, i); __PYX_ERR(3, 4, __pyx_L3_error) }
      }
    } else if (unlikely(__pyx_nargs != 3)) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
      if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(3, 4, __pyx_L3_error)
      values[1] = __Pyx_ArgRef_FASTCALL(__pyx_args, 1);
      if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[1])) __PYX_ERR(3, 4, __pyx_L3_error)
      values[2] = __Pyx_ArgRef_FASTCALL(__pyx_args, 2);
      if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[2])) __PYX_ERR(3, 4, __pyx_L3_error)
    }
    __pyx_v___pyx_type = values[0];
    __pyx_v___pyx_checksum = __Pyx_PyLong_As_long(values[1]); if (unlikely((__pyx_v___pyx_checksum == (long)-1) && PyErr_Occurred())) __PYX_ERR(3, 4, __pyx_L3_error)
    __pyx_v___pyx_state = ((PyObject*)values[2]);
  }
  goto __pyx_L6_skip;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("__pyx_unpickle_MemoryAsync", 1, 3, 3, __pyx_nargs); __PYX_ERR(3, 4, __pyx_L3_error)
  __pyx_L6_skip:;
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_AddTraceback("cupy.cuda.memory.__pyx_unpickle_MemoryAsync", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v___pyx_state), (&PyTuple_Type), 1, "__pyx_state", 1))) __PYX_ERR(3, 4, __pyx_L1_error)
  __pyx_r = __pyx_pf_4cupy_4cuda_6memory_28__pyx_unpickle_MemoryAsync(__pyx_self, __pyx_v___pyx_type, __pyx_v___pyx_checksum, __pyx_v___pyx_state);

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = NULL;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  goto __pyx_L7_cleaned_up;
  __pyx_L0:;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __pyx_L7_cleaned_up:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4cupy_4cuda_6memory_28__pyx_unpickle_MemoryAsync(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v___pyx_type, long __pyx_v___pyx_checksum, PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_v___pyx_result = 0;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  size_t __pyx_t_4;
  int __pyx_t_5;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__pyx_unpickle_MemoryAsync", 0);

  /* "(tree fragment)":6
 * def __pyx_unpickle_MemoryAsync(__pyx_type, long __pyx_checksum, tuple __pyx_state):
 *     cdef object __pyx_result
 *     __Pyx_CheckUnpickleChecksum(__pyx_checksum, 0x0e18a1e, 0xd8681f6, 0xf0c20a1, b'device_id, ptr, size, stream_ref')             # <<<<<<<<<<<<<<
 *     __pyx_result = MemoryAsync.__new__(__pyx_type)
 *     if __pyx_state is not None:
*/
  __pyx_t_1 = __Pyx_CheckUnpickleChecksum(__pyx_v___pyx_checksum, 0x0e18a1e, 0xd8681f6, 0xf0c20a1, __pyx_k_device_id_ptr_size_stream_ref); if (unlikely(__pyx_t_1 == ((int)-1))) __PYX_ERR(3, 6, __pyx_L1_error)

  /* "(tree fragment)":7
 *     cdef object __pyx_result
 *     __Pyx_CheckUnpickleChecksum(__pyx_checksum, 0x0e18a1e, 0xd8681f6, 0xf0c20a1, b'device_id, ptr, size, stream_ref')
 *     __pyx_result = MemoryAsync.__new__(__pyx_type)             # <<<<<<<<<<<<<<
 *     if __pyx_state is not None:
 *         __pyx_unpickle_MemoryAsync__set_state(<MemoryAsync> __pyx_result, __pyx_state)
*/
  __pyx_t_3 = ((PyObject *)__pyx_mstate_global->__pyx_ptype_4cupy_4cuda_6memory_MemoryAsync);
  __Pyx_INCREF(__pyx_t_3);
  __pyx_t_4 = 0;
  {
    PyObject *__pyx_callargs[2] = {__pyx_t_3, __pyx_v___pyx_type};
    __pyx_t_2 = __Pyx_PyObject_FastCallMethod((PyObject*)__pyx_mstate_global->__pyx_n_u_new, __pyx_callargs+__pyx_t_4, (2-__pyx_t_4) | (1*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
    __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
    if (unlikely(!__pyx_t_2)) __PYX_ERR(3, 7, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
  }
  __pyx_v___pyx_result = __pyx_t_2;
  __pyx_t_2 = 0;

  /* "(tree fragment)":8
 *     __Pyx_CheckUnpickleChecksum(__pyx_checksum, 0x0e18a1e, 0xd8681f6, 0xf0c20a1, b'device_id, ptr, size, stream_ref')
 *     __pyx_result = MemoryAsync.__new__(__pyx_type)
 *     if __pyx_state is not None:             # <<<<<<<<<<<<<<
 *         __pyx_unpickle_MemoryAsync__set_state(<MemoryAsync> __pyx_result, __pyx_state)
 *     return __pyx_result
*/
  __pyx_t_5 = (__pyx_v___pyx_state != ((PyObject*)Py_None));
  if (__pyx_t_5) {

    /* "(tree fragment)":9
 *     __pyx_result = MemoryAsync.__new__(__pyx_type)
 *     if __pyx_state is not None:
 *         __pyx_unpickle_MemoryAsync__set_state(<MemoryAsync> __pyx_result, __pyx_state)             # <<<<<<<<<<<<<<
 *     return __pyx_result
 * cdef __pyx_unpickle_MemoryAsync__set_state(MemoryAsync __pyx_result, __pyx_state: tuple):
*/
    if (unlikely(__pyx_v___pyx_state == Py_None)) {
      PyErr_SetString(PyExc_TypeError, "cannot pass None into a C function argument that is declared 'not None'");
      __PYX_ERR(3, 9, __pyx_L1_error)
    }
    __pyx_t_2 = __pyx_f_4cupy_4cuda_6memory___pyx_unpickle_MemoryAsync__set_state(((struct __pyx_obj_4cupy_4cuda_6memory_MemoryAsync *)__pyx_v___pyx_result), __pyx_v___pyx_state); if (unlikely(!__pyx_t_2)) __PYX_ERR(3, 9, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

    /* "(tree fragment)":8
 *     __Pyx_CheckUnpickleChecksum(__pyx_checksum, 0x0e18a1e, 0xd8681f6, 0xf0c20a1, b'device_id, ptr, size, stream_ref')
 *     __pyx_result = MemoryAsync.__new__(__pyx_type)
 *     if __pyx_state is not None:             # <<<<<<<<<<<<<<
 *         __pyx_unpickle_MemoryAsync__set_state(<MemoryAsync> __pyx_result, __pyx_state)
 *     return __pyx_result
*/
  }

  /* "(tree fragment)":10
 *     if __pyx_state is not None:
 *         __pyx_unpickle_MemoryAsync__set_state(<MemoryAsync> __pyx_result, __pyx_state)
 *     return __pyx_result             # <<<<<<<<<<<<<<
 * cdef __pyx_unpickle_MemoryAsync__set_state(MemoryAsync __pyx_result, __pyx_state: tuple):
 *     __pyx_result.device_id = __pyx_state[0]; __pyx_result.ptr = __pyx_state[1]; __pyx_result.size = __pyx_state[2]; __pyx_result.stream_ref = __pyx_state[3]
*/
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_v___pyx_result);
  __pyx_r = __pyx_v___pyx_result;
  goto __pyx_L0;

  /* "(tree fragment)":4
 *     int __Pyx_CheckUnpickleChecksum(long, long, long, long, const char*) except -1
 *     int __Pyx_UpdateUnpickledDict(object, object, Py_ssize_t) except -1
 * def __pyx_unpickle_MemoryAsync(__pyx_type, long __pyx_checksum, tuple __pyx_state):             # <<<<<<<<<<<<<<
 *     cdef object __pyx_result
 *     __Pyx_CheckUnpickleChecksum(__pyx_checksum, 0x0e18a1e, 0xd8681f6, 0xf0c20a1, b'device_id, ptr, size, stream_ref')
*/

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_AddTraceback("cupy.cuda.memory.__pyx_unpickle_MemoryAsync", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v___pyx_result);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":11
 *         __pyx_unpickle_MemoryAsync__set_state(<MemoryAsync> __pyx_result, __pyx_state)
 *     return __pyx_result
 * cdef __pyx_unpickle_MemoryAsync__set_state(MemoryAsync __pyx_result, __pyx_state: tuple):             # <<<<<<<<<<<<<<
 *     __pyx_result.device_id = __pyx_state[0]; __pyx_result.ptr = __pyx_state[1]; __pyx_result.size = __pyx_state[2]; __pyx_result.stream_ref = __pyx_state[3]
 *     __Pyx_UpdateUnpickledDict(__pyx_result, __pyx_state, 4)
*/

static PyObject *__pyx_f_4cupy_4cuda_6memory___pyx_unpickle_MemoryAsync__set_state(struct __pyx_obj_4cupy_4cuda_6memory_MemoryAsync *__pyx_v___pyx_result, PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_t_2;
  intptr_t __pyx_t_3;
  size_t __pyx_t_4;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__pyx_unpickle_MemoryAsync__set_state", 0);

  /* "(tree fragment)":12
 *     return __pyx_result
 * cdef __pyx_unpickle_MemoryAsync__set_state(MemoryAsync __pyx_result, __pyx_state: tuple):
 *     __pyx_result.device_id = __pyx_state[0]; __pyx_result.ptr = __pyx_state[1]; __pyx_result.size = __pyx_state[2]; __pyx_result.stream_ref = __pyx_state[3]             # <<<<<<<<<<<<<<
 *     __Pyx_UpdateUnpickledDict(__pyx_result, __pyx_state, 4)
*/
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 0, long, 1, __Pyx_PyLong_From_long, 0, 0, 1, 1, __Pyx_ReferenceSharing_FunctionArgument); if (unlikely(!__pyx_t_1)) __PYX_ERR(3, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyLong_As_int(__pyx_t_1); if (unlikely((__pyx_t_2 == (int)-1) && PyErr_Occurred())) __PYX_ERR(3, 12, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v___pyx_result->__pyx_base.device_id = __pyx_t_2;
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 1, long, 1, __Pyx_PyLong_From_long, 0, 0, 1, 1, __Pyx_ReferenceSharing_FunctionArgument); if (unlikely(!__pyx_t_1)) __PYX_ERR(3, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_3 = PyLong_AsSsize_t(__pyx_t_1); if (unlikely((__pyx_t_3 == ((intptr_t)-1)) && PyErr_Occurred())) __PYX_ERR(3, 12, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v___pyx_result->__pyx_base.ptr = __pyx_t_3;
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 2, long, 1, __Pyx_PyLong_From_long, 0, 0, 1, 1, __Pyx_ReferenceSharing_FunctionArgument); if (unlikely(!__pyx_t_1)) __PYX_ERR(3, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_4 = __Pyx_PyLong_As_size_t(__pyx_t_1); if (unlikely((__pyx_t_4 == (size_t)-1) && PyErr_Occurred())) __PYX_ERR(3, 12, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v___pyx_result->__pyx_base.size = __pyx_t_4;
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 3, long, 1, __Pyx_PyLong_From_long, 0, 0, 1, 1, __Pyx_ReferenceSharing_FunctionArgument); if (unlikely(!__pyx_t_1)) __PYX_ERR(3, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->stream_ref);
  __Pyx_DECREF(__pyx_v___pyx_result->stream_ref);
  __pyx_v___pyx_result->stream_ref = __pyx_t_1;
  __pyx_t_1 = 0;

  /* "(tree fragment)":13
 * cdef __pyx_unpickle_MemoryAsync__set_state(MemoryAsync __pyx_result, __pyx_state: tuple):
 *     __pyx_result.device_id = __pyx_state[0]; __pyx_result.ptr = __pyx_state[1]; __pyx_result.size = __pyx_state[2]; __pyx_result.stream_ref = __pyx_state[3]
 *     __Pyx_UpdateUnpickledDict(__pyx_result, __pyx_state, 4)             # <<<<<<<<<<<<<<
*/
  __pyx_t_2 = __Pyx_UpdateUnpickledDict(((PyObject *)__pyx_v___pyx_result), __pyx_v___pyx_state, 4); if (unlikely(__pyx_t_2 == ((int)-1))) __PYX_ERR(3, 13, __pyx_L1_error)

  /* "(tree fragment)":11
 *         __pyx_unpickle_MemoryAsync__set_state(<MemoryAsync> __pyx_result, __pyx_state)
 *     return __pyx_result
 * cdef __pyx_unpickle_MemoryAsync__set_state(MemoryAsync __pyx_result, __pyx_state: tuple):             # <<<<<<<<<<<<<<
 *     __pyx_result.device_id = __pyx_state[0]; __pyx_result.ptr = __pyx_state[1]; __pyx_result.size = __pyx_state[2]; __pyx_result.stream_ref = __pyx_state[3]
 *     __Pyx_UpdateUnpickledDict(__pyx_result, __pyx_state, 4)
*/

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("cupy.cuda.memory.__pyx_unpickle_MemoryAsync__set_state", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":4
 *     int __Pyx_CheckUnpickleChecksum(long, long, long, long, const char*) except -1
 *     int __Pyx_UpdateUnpickledDict(object, object, Py_ssize_t) except -1
 * def __pyx_unpickle_UnownedMemory(__pyx_type, long __pyx_checksum, tuple __pyx_state):             # <<<<<<<<<<<<<<
 *     cdef object __pyx_result
 *     __Pyx_CheckUnpickleChecksum(__pyx_checksum, 0xa9b59d9, 0xa91bd4c, 0x383c807, b'_owner, device_id, ptr, size')
*/

/* Python wrapper */
static PyObject *__pyx_pw_4cupy_4cuda_6memory_31__pyx_unpickle_UnownedMemory(PyObject *__pyx_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
PyDoc_STRVAR(__pyx_doc_4cupy_4cuda_6memory_30__pyx_unpickle_UnownedMemory, "__pyx_unpickle_UnownedMemory(__pyx_type, long __pyx_checksum, tuple __pyx_state)");
static PyMethodDef __pyx_mdef_4cupy_4cuda_6memory_31__pyx_unpickle_UnownedMemory = {"__pyx_unpickle_UnownedMemory", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_4cupy_4cuda_6memory_31__pyx_unpickle_UnownedMemory, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_4cupy_4cuda_6memory_30__pyx_unpickle_UnownedMemory};
static PyObject *__pyx_pw_4cupy_4cuda_6memory_31__pyx_unpickle_UnownedMemory(PyObject *__pyx_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
) {
  PyObject *__pyx_v___pyx_type = 0;
  long __pyx_v___pyx_checksum;
  PyObject *__pyx_v___pyx_state = 0;
  #if !CYTHON_METH_FASTCALL
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  #endif
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject* values[3] = {0,0,0};
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__pyx_unpickle_UnownedMemory (wrapper)", 0);
  #if !CYTHON_METH_FASTCALL
  #if CYTHON_ASSUME_SAFE_SIZE
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return NULL;
  #endif
  #endif
  __pyx_kwvalues = __Pyx_KwValues_FASTCALL(__pyx_args, __pyx_nargs);
  {
    PyObject ** const __pyx_pyargnames[] = {&__pyx_mstate_global->__pyx_n_u_pyx_type,&__pyx_mstate_global->__pyx_n_u_pyx_checksum,&__pyx_mstate_global->__pyx_n_u_pyx_state,0};
    const Py_ssize_t __pyx_kwds_len = (__pyx_kwds) ? __Pyx_NumKwargs_FASTCALL(__pyx_kwds) : 0;
    if (unlikely(__pyx_kwds_len < 0)) __PYX_ERR(3, 4, __pyx_L3_error)
    if (__pyx_kwds_len > 0) {
      switch (__pyx_nargs) {
        case  3:
        values[2] = __Pyx_ArgRef_FASTCALL(__pyx_args, 2);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[2])) __PYX_ERR(3, 4, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  2:
        values[1] = __Pyx_ArgRef_FASTCALL(__pyx_args, 1);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[1])) __PYX_ERR(3, 4, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  1:
        values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(3, 4, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      const Py_ssize_t kwd_pos_args = __pyx_nargs;
      if (__Pyx_ParseKeywords(__pyx_kwds, __pyx_kwvalues, __pyx_pyargnames, 0, values, kwd_pos_args, __pyx_kwds_len, "__pyx_unpickle_UnownedMemory", 0) < (0)) __PYX_ERR(3, 4, __pyx_L3_error)
      for (Py_ssize_t i = __pyx_nargs; i < 3; i++) {
        if (unlikely(!values[i])) { __Pyx_RaiseArgtupleInvalid("__pyx_unpickle_UnownedMemory", 1, 3, 3, i); __PYX_ERR(3, 4, __pyx_L3_error) }
      }
    } else if (unlikely(__pyx_nargs != 3)) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
      if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(3, 4, __pyx_L3_error)
      values[1] = __Pyx_ArgRef_FASTCALL(__pyx_args, 1);
      if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[1])) __PYX_ERR(3, 4, __pyx_L3_error)
      values[2] = __Pyx_ArgRef_FASTCALL(__pyx_args, 2);
      if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[2])) __PYX_ERR(3, 4, __pyx_L3_error)
    }
    __pyx_v___pyx_type = values[0];
    __pyx_v___pyx_checksum = __Pyx_PyLong_As_long(values[1]); if (unlikely((__pyx_v___pyx_checksum == (long)-1) && PyErr_Occurred())) __PYX_ERR(3, 4, __pyx_L3_error)
    __pyx_v___pyx_state = ((PyObject*)values[2]);
  }
  goto __pyx_L6_skip;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("__pyx_unpickle_UnownedMemory", 1, 3, 3, __pyx_nargs); __PYX_ERR(3, 4, __pyx_L3_error)
  __pyx_L6_skip:;
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_AddTraceback("cupy.cuda.memory.__pyx_unpickle_UnownedMemory", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v___pyx_state), (&PyTuple_Type), 1, "__pyx_state", 1))) __PYX_ERR(3, 4, __pyx_L1_error)
  __pyx_r = __pyx_pf_4cupy_4cuda_6memory_30__pyx_unpickle_UnownedMemory(__pyx_self, __pyx_v___pyx_type, __pyx_v___pyx_checksum, __pyx_v___pyx_state);

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = NULL;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  goto __pyx_L7_cleaned_up;
  __pyx_L0:;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __pyx_L7_cleaned_up:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4cupy_4cuda_6memory_30__pyx_unpickle_UnownedMemory(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v___pyx_type, long __pyx_v___pyx_checksum, PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_v___pyx_result = 0;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  size_t __pyx_t_4;
  int __pyx_t_5;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__pyx_unpickle_UnownedMemory", 0);

  /* "(tree fragment)":6
 * def __pyx_unpickle_UnownedMemory(__pyx_type, long __pyx_checksum, tuple __pyx_state):
 *     cdef object __pyx_result
 *     __Pyx_CheckUnpickleChecksum(__pyx_checksum, 0xa9b59d9, 0xa91bd4c, 0x383c807, b'_owner, device_id, ptr, size')             # <<<<<<<<<<<<<<
 *     __pyx_result = UnownedMemory.__new__(__pyx_type)
 *     if __pyx_state is not None:
*/
  __pyx_t_1 = __Pyx_CheckUnpickleChecksum(__pyx_v___pyx_checksum, 0xa9b59d9, 0xa91bd4c, 0x383c807, __pyx_k_owner_device_id_ptr_size); if (unlikely(__pyx_t_1 == ((int)-1))) __PYX_ERR(3, 6, __pyx_L1_error)

  /* "(tree fragment)":7
 *     cdef object __pyx_result
 *     __Pyx_CheckUnpickleChecksum(__pyx_checksum, 0xa9b59d9, 0xa91bd4c, 0x383c807, b'_owner, device_id, ptr, size')
 *     __pyx_result = UnownedMemory.__new__(__pyx_type)             # <<<<<<<<<<<<<<
 *     if __pyx_state is not None:
 *         __pyx_unpickle_UnownedMemory__set_state(<UnownedMemory> __pyx_result, __pyx_state)
*/
  __pyx_t_3 = ((PyObject *)__pyx_mstate_global->__pyx_ptype_4cupy_4cuda_6memory_UnownedMemory);
  __Pyx_INCREF(__pyx_t_3);
  __pyx_t_4 = 0;
  {
    PyObject *__pyx_callargs[2] = {__pyx_t_3, __pyx_v___pyx_type};
    __pyx_t_2 = __Pyx_PyObject_FastCallMethod((PyObject*)__pyx_mstate_global->__pyx_n_u_new, __pyx_callargs+__pyx_t_4, (2-__pyx_t_4) | (1*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
    __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
    if (unlikely(!__pyx_t_2)) __PYX_ERR(3, 7, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
  }
  __pyx_v___pyx_result = __pyx_t_2;
  __pyx_t_2 = 0;

  /* "(tree fragment)":8
 *     __Pyx_CheckUnpickleChecksum(__pyx_checksum, 0xa9b59d9, 0xa91bd4c, 0x383c807, b'_owner, device_id, ptr, size')
 *     __pyx_result = UnownedMemory.__new__(__pyx_type)
 *     if __pyx_state is not None:             # <<<<<<<<<<<<<<
 *         __pyx_unpickle_UnownedMemory__set_state(<UnownedMemory> __pyx_result, __pyx_state)
 *     return __pyx_result
*/
  __pyx_t_5 = (__pyx_v___pyx_state != ((PyObject*)Py_None));
  if (__pyx_t_5) {

    /* "(tree fragment)":9
 *     __pyx_result = UnownedMemory.__new__(__pyx_type)
 *     if __pyx_state is not None:
 *         __pyx_unpickle_UnownedMemory__set_state(<UnownedMemory> __pyx_result, __pyx_state)             # <<<<<<<<<<<<<<
 *     return __pyx_result
 * cdef __pyx_unpickle_UnownedMemory__set_state(UnownedMemory __pyx_result, __pyx_state: tuple):
*/
    if (unlikely(__pyx_v___pyx_state == Py_None)) {
      PyErr_SetString(PyExc_TypeError, "cannot pass None into a C function argument that is declared 'not None'");
      __PYX_ERR(3, 9, __pyx_L1_error)
    }
    __pyx_t_2 = __pyx_f_4cupy_4cuda_6memory___pyx_unpickle_UnownedMemory__set_state(((struct __pyx_obj_4cupy_4cuda_6memory_UnownedMemory *)__pyx_v___pyx_result), __pyx_v___pyx_state); if (unlikely(!__pyx_t_2)) __PYX_ERR(3, 9, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

    /* "(tree fragment)":8
 *     __Pyx_CheckUnpickleChecksum(__pyx_checksum, 0xa9b59d9, 0xa91bd4c, 0x383c807, b'_owner, device_id, ptr, size')
 *     __pyx_result = UnownedMemory.__new__(__pyx_type)
 *     if __pyx_state is not None:             # <<<<<<<<<<<<<<
 *         __pyx_unpickle_UnownedMemory__set_state(<UnownedMemory> __pyx_result, __pyx_state)
 *     return __pyx_result
*/
  }

  /* "(tree fragment)":10
 *     if __pyx_state is not None:
 *         __pyx_unpickle_UnownedMemory__set_state(<UnownedMemory> __pyx_result, __pyx_state)
 *     return __pyx_result             # <<<<<<<<<<<<<<
 * cdef __pyx_unpickle_UnownedMemory__set_state(UnownedMemory __pyx_result, __pyx_state: tuple):
 *     __pyx_result._owner = __pyx_state[0]; __pyx_result.device_id = __pyx_state[1]; __pyx_result.ptr = __pyx_state[2]; __pyx_result.size = __pyx_state[3]
*/
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_v___pyx_result);
  __pyx_r = __pyx_v___pyx_result;
  goto __pyx_L0;

  /* "(tree fragment)":4
 *     int __Pyx_CheckUnpickleChecksum(long, long, long, long, const char*) except -1
 *     int __Pyx_UpdateUnpickledDict(object, object, Py_ssize_t) except -1
 * def __pyx_unpickle_UnownedMemory(__pyx_type, long __pyx_checksum, tuple __pyx_state):             # <<<<<<<<<<<<<<
 *     cdef object __pyx_result
 *     __Pyx_CheckUnpickleChecksum(__pyx_checksum, 0xa9b59d9, 0xa91bd4c, 0x383c807, b'_owner, device_id, ptr, size')
*/

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_AddTraceback("cupy.cuda.memory.__pyx_unpickle_UnownedMemory", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v___pyx_result);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":11
 *         __pyx_unpickle_UnownedMemory__set_state(<UnownedMemory> __pyx_result, __pyx_state)
 *     return __pyx_result
 * cdef __pyx_unpickle_UnownedMemory__set_state(UnownedMemory __pyx_result, __pyx_state: tuple):             # <<<<<<<<<<<<<<
 *     __pyx_result._owner = __pyx_state[0]; __pyx_result.device_id = __pyx_state[1]; __pyx_result.ptr = __pyx_state[2]; __pyx_result.size = __pyx_state[3]
 *     __Pyx_UpdateUnpickledDict(__pyx_result, __pyx_state, 4)
*/

static PyObject *__pyx_f_4cupy_4cuda_6memory___pyx_unpickle_UnownedMemory__set_state(struct __pyx_obj_4cupy_4cuda_6memory_UnownedMemory *__pyx_v___pyx_result, PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_t_2;
  intptr_t __pyx_t_3;
  size_t __pyx_t_4;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__pyx_unpickle_UnownedMemory__set_state", 0);

  /* "(tree fragment)":12
 *     return __pyx_result
 * cdef __pyx_unpickle_UnownedMemory__set_state(UnownedMemory __pyx_result, __pyx_state: tuple):
 *     __pyx_result._owner = __pyx_state[0]; __pyx_result.device_id = __pyx_state[1]; __pyx_result.ptr = __pyx_state[2]; __pyx_result.size = __pyx_state[3]             # <<<<<<<<<<<<<<
 *     __Pyx_UpdateUnpickledDict(__pyx_result, __pyx_state, 4)
*/
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 0, long, 1, __Pyx_PyLong_From_long, 0, 0, 1, 1, __Pyx_ReferenceSharing_FunctionArgument); if (unlikely(!__pyx_t_1)) __PYX_ERR(3, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->_owner);
  __Pyx_DECREF(__pyx_v___pyx_result->_owner);
  __pyx_v___pyx_result->_owner = __pyx_t_1;
  __pyx_t_1 = 0;
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 1, long, 1, __Pyx_PyLong_From_long, 0, 0, 1, 1, __Pyx_ReferenceSharing_FunctionArgument); if (unlikely(!__pyx_t_1)) __PYX_ERR(3, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyLong_As_int(__pyx_t_1); if (unlikely((__pyx_t_2 == (int)-1) && PyErr_Occurred())) __PYX_ERR(3, 12, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v___pyx_result->__pyx_base.device_id = __pyx_t_2;
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 2, long, 1, __Pyx_PyLong_From_long, 0, 0, 1, 1, __Pyx_ReferenceSharing_FunctionArgument); if (unlikely(!__pyx_t_1)) __PYX_ERR(3, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_3 = PyLong_AsSsize_t(__pyx_t_1); if (unlikely((__pyx_t_3 == ((intptr_t)-1)) && PyErr_Occurred())) __PYX_ERR(3, 12, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v___pyx_result->__pyx_base.ptr = __pyx_t_3;
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 3, long, 1, __Pyx_PyLong_From_long, 0, 0, 1, 1, __Pyx_ReferenceSharing_FunctionArgument); if (unlikely(!__pyx_t_1)) __PYX_ERR(3, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_4 = __Pyx_PyLong_As_size_t(__pyx_t_1); if (unlikely((__pyx_t_4 == (size_t)-1) && PyErr_Occurred())) __PYX_ERR(3, 12, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v___pyx_result->__pyx_base.size = __pyx_t_4;

  /* "(tree fragment)":13
 * cdef __pyx_unpickle_UnownedMemory__set_state(UnownedMemory __pyx_result, __pyx_state: tuple):
 *     __pyx_result._owner = __pyx_state[0]; __pyx_result.device_id = __pyx_state[1]; __pyx_result.ptr = __pyx_state[2]; __pyx_result.size = __pyx_state[3]
 *     __Pyx_UpdateUnpickledDict(__pyx_result, __pyx_state, 4)             # <<<<<<<<<<<<<<
*/
  __pyx_t_2 = __Pyx_UpdateUnpickledDict(((PyObject *)__pyx_v___pyx_result), __pyx_v___pyx_state, 4); if (unlikely(__pyx_t_2 == ((int)-1))) __PYX_ERR(3, 13, __pyx_L1_error)

  /* "(tree fragment)":11
 *         __pyx_unpickle_UnownedMemory__set_state(<UnownedMemory> __pyx_result, __pyx_state)
 *     return __pyx_result
 * cdef __pyx_unpickle_UnownedMemory__set_state(UnownedMemory __pyx_result, __pyx_state: tuple):             # <<<<<<<<<<<<<<
 *     __pyx_result._owner = __pyx_state[0]; __pyx_result.device_id = __pyx_state[1]; __pyx_result.ptr = __pyx_state[2]; __pyx_result.size = __pyx_state[3]
 *     __Pyx_UpdateUnpickledDict(__pyx_result, __pyx_state, 4)
*/

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("cupy.cuda.memory.__pyx_unpickle_UnownedMemory__set_state", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":4
 *     int __Pyx_CheckUnpickleChecksum(long, long, long, long, const char*) except -1
 *     int __Pyx_UpdateUnpickledDict(object, object, Py_ssize_t) except -1
 * def __pyx_unpickle_ManagedMemory(__pyx_type, long __pyx_checksum, tuple __pyx_state):             # <<<<<<<<<<<<<<
 *     cdef object __pyx_result
 *     __Pyx_CheckUnpickleChecksum(__pyx_checksum, 0x61cadb0, 0xe1d5c2a, 0xeeb5490, b'device_id, ptr, size')
*/

/* Python wrapper */
static PyObject *__pyx_pw_4cupy_4cuda_6memory_33__pyx_unpickle_ManagedMemory(PyObject *__pyx_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
PyDoc_STRVAR(__pyx_doc_4cupy_4cuda_6memory_32__pyx_unpickle_ManagedMemory, "__pyx_unpickle_ManagedMemory(__pyx_type, long __pyx_checksum, tuple __pyx_state)");
static PyMethodDef __pyx_mdef_4cupy_4cuda_6memory_33__pyx_unpickle_ManagedMemory = {"__pyx_unpickle_ManagedMemory", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_4cupy_4cuda_6memory_33__pyx_unpickle_ManagedMemory, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_4cupy_4cuda_6memory_32__pyx_unpickle_ManagedMemory};
static PyObject *__pyx_pw_4cupy_4cuda_6memory_33__pyx_unpickle_ManagedMemory(PyObject *__pyx_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
) {
  PyObject *__pyx_v___pyx_type = 0;
  long __pyx_v___pyx_checksum;
  PyObject *__pyx_v___pyx_state = 0;
  #if !CYTHON_METH_FASTCALL
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  #endif
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject* values[3] = {0,0,0};
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__pyx_unpickle_ManagedMemory (wrapper)", 0);
  #if !CYTHON_METH_FASTCALL
  #if CYTHON_ASSUME_SAFE_SIZE
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return NULL;
  #endif
  #endif
  __pyx_kwvalues = __Pyx_KwValues_FASTCALL(__pyx_args, __pyx_nargs);
  {
    PyObject ** const __pyx_pyargnames[] = {&__pyx_mstate_global->__pyx_n_u_pyx_type,&__pyx_mstate_global->__pyx_n_u_pyx_checksum,&__pyx_mstate_global->__pyx_n_u_pyx_state,0};
    const Py_ssize_t __pyx_kwds_len = (__pyx_kwds) ? __Pyx_NumKwargs_FASTCALL(__pyx_kwds) : 0;
    if (unlikely(__pyx_kwds_len < 0)) __PYX_ERR(3, 4, __pyx_L3_error)
    if (__pyx_kwds_len > 0) {
      switch (__pyx_nargs) {
        case  3:
        values[2] = __Pyx_ArgRef_FASTCALL(__pyx_args, 2);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[2])) __PYX_ERR(3, 4, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  2:
        values[1] = __Pyx_ArgRef_FASTCALL(__pyx_args, 1);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[1])) __PYX_ERR(3, 4, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  1:
        values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(3, 4, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      const Py_ssize_t kwd_pos_args = __pyx_nargs;
      if (__Pyx_ParseKeywords(__pyx_kwds, __pyx_kwvalues, __pyx_pyargnames, 0, values, kwd_pos_args, __pyx_kwds_len, "__pyx_unpickle_ManagedMemory", 0) < (0)) __PYX_ERR(3, 4, __pyx_L3_error)
      for (Py_ssize_t i = __pyx_nargs; i < 3; i++) {
        if (unlikely(!values[i])) { __Pyx_RaiseArgtupleInvalid("__pyx_unpickle_ManagedMemory", 1, 3, 3, i); __PYX_ERR(3, 4, __pyx_L3_error) }
      }
    } else if (unlikely(__pyx_nargs != 3)) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
      if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(3, 4, __pyx_L3_error)
      values[1] = __Pyx_ArgRef_FASTCALL(__pyx_args, 1);
      if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[1])) __PYX_ERR(3, 4, __pyx_L3_error)
      values[2] = __Pyx_ArgRef_FASTCALL(__pyx_args, 2);
      if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[2])) __PYX_ERR(3, 4, __pyx_L3_error)
    }
    __pyx_v___pyx_type = values[0];
    __pyx_v___pyx_checksum = __Pyx_PyLong_As_long(values[1]); if (unlikely((__pyx_v___pyx_checksum == (long)-1) && PyErr_Occurred())) __PYX_ERR(3, 4, __pyx_L3_error)
    __pyx_v___pyx_state = ((PyObject*)values[2]);
  }
  goto __pyx_L6_skip;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("__pyx_unpickle_ManagedMemory", 1, 3, 3, __pyx_nargs); __PYX_ERR(3, 4, __pyx_L3_error)
  __pyx_L6_skip:;
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_AddTraceback("cupy.cuda.memory.__pyx_unpickle_ManagedMemory", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v___pyx_state), (&PyTuple_Type), 1, "__pyx_state", 1))) __PYX_ERR(3, 4, __pyx_L1_error)
  __pyx_r = __pyx_pf_4cupy_4cuda_6memory_32__pyx_unpickle_ManagedMemory(__pyx_self, __pyx_v___pyx_type, __pyx_v___pyx_checksum, __pyx_v___pyx_state);

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = NULL;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  goto __pyx_L7_cleaned_up;
  __pyx_L0:;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __pyx_L7_cleaned_up:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4cupy_4cuda_6memory_32__pyx_unpickle_ManagedMemory(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v___pyx_type, long __pyx_v___pyx_checksum, PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_v___pyx_result = 0;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  size_t __pyx_t_4;
  int __pyx_t_5;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__pyx_unpickle_ManagedMemory", 0);

  /* "(tree fragment)":6
 * def __pyx_unpickle_ManagedMemory(__pyx_type, long __pyx_checksum, tuple __pyx_state):
 *     cdef object __pyx_result
 *     __Pyx_CheckUnpickleChecksum(__pyx_checksum, 0x61cadb0, 0xe1d5c2a, 0xeeb5490, b'device_id, ptr, size')             # <<<<<<<<<<<<<<
 *     __pyx_result = ManagedMemory.__new__(__pyx_type)
 *     if __pyx_state is not None:
*/
  __pyx_t_1 = __Pyx_CheckUnpickleChecksum(__pyx_v___pyx_checksum, 0x61cadb0, 0xe1d5c2a, 0xeeb5490, __pyx_k_device_id_ptr_size); if (unlikely(__pyx_t_1 == ((int)-1))) __PYX_ERR(3, 6, __pyx_L1_error)

  /* "(tree fragment)":7
 *     cdef object __pyx_result
 *     __Pyx_CheckUnpickleChecksum(__pyx_checksum, 0x61cadb0, 0xe1d5c2a, 0xeeb5490, b'device_id, ptr, size')
 *     __pyx_result = ManagedMemory.__new__(__pyx_type)             # <<<<<<<<<<<<<<
 *     if __pyx_state is not None:
 *         __pyx_unpickle_ManagedMemory__set_state(<ManagedMemory> __pyx_result, __pyx_state)
*/
  __pyx_t_3 = ((PyObject *)__pyx_mstate_global->__pyx_ptype_4cupy_4cuda_6memory_ManagedMemory);
  __Pyx_INCREF(__pyx_t_3);
  __pyx_t_4 = 0;
  {
    PyObject *__pyx_callargs[2] = {__pyx_t_3, __pyx_v___pyx_type};
    __pyx_t_2 = __Pyx_PyObject_FastCallMethod((PyObject*)__pyx_mstate_global->__pyx_n_u_new, __pyx_callargs+__pyx_t_4, (2-__pyx_t_4) | (1*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
    __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
    if (unlikely(!__pyx_t_2)) __PYX_ERR(3, 7, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
  }
  __pyx_v___pyx_result = __pyx_t_2;
  __pyx_t_2 = 0;

  /* "(tree fragment)":8
 *     __Pyx_CheckUnpickleChecksum(__pyx_checksum, 0x61cadb0, 0xe1d5c2a, 0xeeb5490, b'device_id, ptr, size')
 *     __pyx_result = ManagedMemory.__new__(__pyx_type)
 *     if __pyx_state is not None:             # <<<<<<<<<<<<<<
 *         __pyx_unpickle_ManagedMemory__set_state(<ManagedMemory> __pyx_result, __pyx_state)
 *     return __pyx_result
*/
  __pyx_t_5 = (__pyx_v___pyx_state != ((PyObject*)Py_None));
  if (__pyx_t_5) {

    /* "(tree fragment)":9
 *     __pyx_result = ManagedMemory.__new__(__pyx_type)
 *     if __pyx_state is not None:
 *         __pyx_unpickle_ManagedMemory__set_state(<ManagedMemory> __pyx_result, __pyx_state)             # <<<<<<<<<<<<<<
 *     return __pyx_result
 * cdef __pyx_unpickle_ManagedMemory__set_state(ManagedMemory __pyx_result, __pyx_state: tuple):
*/
    if (unlikely(__pyx_v___pyx_state == Py_None)) {
      PyErr_SetString(PyExc_TypeError, "cannot pass None into a C function argument that is declared 'not None'");
      __PYX_ERR(3, 9, __pyx_L1_error)
    }
    __pyx_t_2 = __pyx_f_4cupy_4cuda_6memory___pyx_unpickle_ManagedMemory__set_state(((struct __pyx_obj_4cupy_4cuda_6memory_ManagedMemory *)__pyx_v___pyx_result), __pyx_v___pyx_state); if (unlikely(!__pyx_t_2)) __PYX_ERR(3, 9, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

    /* "(tree fragment)":8
 *     __Pyx_CheckUnpickleChecksum(__pyx_checksum, 0x61cadb0, 0xe1d5c2a, 0xeeb5490, b'device_id, ptr, size')
 *     __pyx_result = ManagedMemory.__new__(__pyx_type)
 *     if __pyx_state is not None:             # <<<<<<<<<<<<<<
 *         __pyx_unpickle_ManagedMemory__set_state(<ManagedMemory> __pyx_result, __pyx_state)
 *     return __pyx_result
*/
  }

  /* "(tree fragment)":10
 *     if __pyx_state is not None:
 *         __pyx_unpickle_ManagedMemory__set_state(<ManagedMemory> __pyx_result, __pyx_state)
 *     return __pyx_result             # <<<<<<<<<<<<<<
 * cdef __pyx_unpickle_ManagedMemory__set_state(ManagedMemory __pyx_result, __pyx_state: tuple):
 *     __pyx_result.device_id = __pyx_state[0]; __pyx_result.ptr = __pyx_state[1]; __pyx_result.size = __pyx_state[2]
*/
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_v___pyx_result);
  __pyx_r = __pyx_v___pyx_result;
  goto __pyx_L0;

  /* "(tree fragment)":4
 *     int __Pyx_CheckUnpickleChecksum(long, long, long, long, const char*) except -1
 *     int __Pyx_UpdateUnpickledDict(object, object, Py_ssize_t) except -1
 * def __pyx_unpickle_ManagedMemory(__pyx_type, long __pyx_checksum, tuple __pyx_state):             # <<<<<<<<<<<<<<
 *     cdef object __pyx_result
 *     __Pyx_CheckUnpickleChecksum(__pyx_checksum, 0x61cadb0, 0xe1d5c2a, 0xeeb5490, b'device_id, ptr, size')
*/

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_AddTraceback("cupy.cuda.memory.__pyx_unpickle_ManagedMemory", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v___pyx_result);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":11
 *         __pyx_unpickle_ManagedMemory__set_state(<ManagedMemory> __pyx_result, __pyx_state)
 *     return __pyx_result
 * cdef __pyx_unpickle_ManagedMemory__set_state(ManagedMemory __pyx_result, __pyx_state: tuple):             # <<<<<<<<<<<<<<
 *     __pyx_result.device_id = __pyx_state[0]; __pyx_result.ptr = __pyx_state[1]; __pyx_result.size = __pyx_state[2]
 *     __Pyx_UpdateUnpickledDict(__pyx_result, __pyx_state, 3)
*/

static PyObject *__pyx_f_4cupy_4cuda_6memory___pyx_unpickle_ManagedMemory__set_state(struct __pyx_obj_4cupy_4cuda_6memory_ManagedMemory *__pyx_v___pyx_result, PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_t_2;
  intptr_t __pyx_t_3;
  size_t __pyx_t_4;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__pyx_unpickle_ManagedMemory__set_state", 0);

  /* "(tree fragment)":12
 *     return __pyx_result
 * cdef __pyx_unpickle_ManagedMemory__set_state(ManagedMemory __pyx_result, __pyx_state: tuple):
 *     __pyx_result.device_id = __pyx_state[0]; __pyx_result.ptr = __pyx_state[1]; __pyx_result.size = __pyx_state[2]             # <<<<<<<<<<<<<<
 *     __Pyx_UpdateUnpickledDict(__pyx_result, __pyx_state, 3)
*/
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 0, long, 1, __Pyx_PyLong_From_long, 0, 0, 1, 1, __Pyx_ReferenceSharing_FunctionArgument); if (unlikely(!__pyx_t_1)) __PYX_ERR(3, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyLong_As_int(__pyx_t_1); if (unlikely((__pyx_t_2 == (int)-1) && PyErr_Occurred())) __PYX_ERR(3, 12, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v___pyx_result->__pyx_base.device_id = __pyx_t_2;
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 1, long, 1, __Pyx_PyLong_From_long, 0, 0, 1, 1, __Pyx_ReferenceSharing_FunctionArgument); if (unlikely(!__pyx_t_1)) __PYX_ERR(3, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_3 = PyLong_AsSsize_t(__pyx_t_1); if (unlikely((__pyx_t_3 == ((intptr_t)-1)) && PyErr_Occurred())) __PYX_ERR(3, 12, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v___pyx_result->__pyx_base.ptr = __pyx_t_3;
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 2, long, 1, __Pyx_PyLong_From_long, 0, 0, 1, 1, __Pyx_ReferenceSharing_FunctionArgument); if (unlikely(!__pyx_t_1)) __PYX_ERR(3, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_4 = __Pyx_PyLong_As_size_t(__pyx_t_1); if (unlikely((__pyx_t_4 == (size_t)-1) && PyErr_Occurred())) __PYX_ERR(3, 12, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v___pyx_result->__pyx_base.size = __pyx_t_4;

  /* "(tree fragment)":13
 * cdef __pyx_unpickle_ManagedMemory__set_state(ManagedMemory __pyx_result, __pyx_state: tuple):
 *     __pyx_result.device_id = __pyx_state[0]; __pyx_result.ptr = __pyx_state[1]; __pyx_result.size = __pyx_state[2]
 *     __Pyx_UpdateUnpickledDict(__pyx_result, __pyx_state, 3)             # <<<<<<<<<<<<<<
*/
  __pyx_t_2 = __Pyx_UpdateUnpickledDict(((PyObject *)__pyx_v___pyx_result), __pyx_v___pyx_state, 3); if (unlikely(__pyx_t_2 == ((int)-1))) __PYX_ERR(3, 13, __pyx_L1_error)

  /* "(tree fragment)":11
 *         __pyx_unpickle_ManagedMemory__set_state(<ManagedMemory> __pyx_result, __pyx_state)
 *     return __pyx_result
 * cdef __pyx_unpickle_ManagedMemory__set_state(ManagedMemory __pyx_result, __pyx_state: tuple):             # <<<<<<<<<<<<<<
 *     __pyx_result.device_id = __pyx_state[0]; __pyx_result.ptr = __pyx_state[1]; __pyx_result.size = __pyx_state[2]
 *     __Pyx_UpdateUnpickledDict(__pyx_result, __pyx_state, 3)
*/

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("cupy.cuda.memory.__pyx_unpickle_ManagedMemory__set_state", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":4
 *     int __Pyx_CheckUnpickleChecksum(long, long, long, long, const char*) except -1
 *     int __Pyx_UpdateUnpickledDict(object, object, Py_ssize_t) except -1
 * def __pyx_unpickle_SystemMemory(__pyx_type, long __pyx_checksum, tuple __pyx_state):             # <<<<<<<<<<<<<<
 *     cdef object __pyx_result
 *     __Pyx_CheckUnpickleChecksum(__pyx_checksum, 0xa9b59d9, 0xa91bd4c, 0x383c807, b'_owner, device_id, ptr, size')
*/

/* Python wrapper */
static PyObject *__pyx_pw_4cupy_4cuda_6memory_35__pyx_unpickle_SystemMemory(PyObject *__pyx_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
PyDoc_STRVAR(__pyx_doc_4cupy_4cuda_6memory_34__pyx_unpickle_SystemMemory, "__pyx_unpickle_SystemMemory(__pyx_type, long __pyx_checksum, tuple __pyx_state)");
static PyMethodDef __pyx_mdef_4cupy_4cuda_6memory_35__pyx_unpickle_SystemMemory = {"__pyx_unpickle_SystemMemory", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_4cupy_4cuda_6memory_35__pyx_unpickle_SystemMemory, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_4cupy_4cuda_6memory_34__pyx_unpickle_SystemMemory};
static PyObject *__pyx_pw_4cupy_4cuda_6memory_35__pyx_unpickle_SystemMemory(PyObject *__pyx_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
) {
  PyObject *__pyx_v___pyx_type = 0;
  long __pyx_v___pyx_checksum;
  PyObject *__pyx_v___pyx_state = 0;
  #if !CYTHON_METH_FASTCALL
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  #endif
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject* values[3] = {0,0,0};
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__pyx_unpickle_SystemMemory (wrapper)", 0);
  #if !CYTHON_METH_FASTCALL
  #if CYTHON_ASSUME_SAFE_SIZE
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return NULL;
  #endif
  #endif
  __pyx_kwvalues = __Pyx_KwValues_FASTCALL(__pyx_args, __pyx_nargs);
  {
    PyObject ** const __pyx_pyargnames[] = {&__pyx_mstate_global->__pyx_n_u_pyx_type,&__pyx_mstate_global->__pyx_n_u_pyx_checksum,&__pyx_mstate_global->__pyx_n_u_pyx_state,0};
    const Py_ssize_t __pyx_kwds_len = (__pyx_kwds) ? __Pyx_NumKwargs_FASTCALL(__pyx_kwds) : 0;
    if (unlikely(__pyx_kwds_len < 0)) __PYX_ERR(3, 4, __pyx_L3_error)
    if (__pyx_kwds_len > 0) {
      switch (__pyx_nargs) {
        case  3:
        values[2] = __Pyx_ArgRef_FASTCALL(__pyx_args, 2);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[2])) __PYX_ERR(3, 4, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  2:
        values[1] = __Pyx_ArgRef_FASTCALL(__pyx_args, 1);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[1])) __PYX_ERR(3, 4, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  1:
        values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(3, 4, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      const Py_ssize_t kwd_pos_args = __pyx_nargs;
      if (__Pyx_ParseKeywords(__pyx_kwds, __pyx_kwvalues, __pyx_pyargnames, 0, values, kwd_pos_args, __pyx_kwds_len, "__pyx_unpickle_SystemMemory", 0) < (0)) __PYX_ERR(3, 4, __pyx_L3_error)
      for (Py_ssize_t i = __pyx_nargs; i < 3; i++) {
        if (unlikely(!values[i])) { __Pyx_RaiseArgtupleInvalid("__pyx_unpickle_SystemMemory", 1, 3, 3, i); __PYX_ERR(3, 4, __pyx_L3_error) }
      }
    } else if (unlikely(__pyx_nargs != 3)) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
      if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(3, 4, __pyx_L3_error)
      values[1] = __Pyx_ArgRef_FASTCALL(__pyx_args, 1);
      if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[1])) __PYX_ERR(3, 4, __pyx_L3_error)
      values[2] = __Pyx_ArgRef_FASTCALL(__pyx_args, 2);
      if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[2])) __PYX_ERR(3, 4, __pyx_L3_error)
    }
    __pyx_v___pyx_type = values[0];
    __pyx_v___pyx_checksum = __Pyx_PyLong_As_long(values[1]); if (unlikely((__pyx_v___pyx_checksum == (long)-1) && PyErr_Occurred())) __PYX_ERR(3, 4, __pyx_L3_error)
    __pyx_v___pyx_state = ((PyObject*)values[2]);
  }
  goto __pyx_L6_skip;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("__pyx_unpickle_SystemMemory", 1, 3, 3, __pyx_nargs); __PYX_ERR(3, 4, __pyx_L3_error)
  __pyx_L6_skip:;
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_AddTraceback("cupy.cuda.memory.__pyx_unpickle_SystemMemory", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v___pyx_state), (&PyTuple_Type), 1, "__pyx_state", 1))) __PYX_ERR(3, 4, __pyx_L1_error)
  __pyx_r = __pyx_pf_4cupy_4cuda_6memory_34__pyx_unpickle_SystemMemory(__pyx_self, __pyx_v___pyx_type, __pyx_v___pyx_checksum, __pyx_v___pyx_state);

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = NULL;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  goto __pyx_L7_cleaned_up;
  __pyx_L0:;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __pyx_L7_cleaned_up:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4cupy_4cuda_6memory_34__pyx_unpickle_SystemMemory(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v___pyx_type, long __pyx_v___pyx_checksum, PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_v___pyx_result = 0;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  size_t __pyx_t_4;
  int __pyx_t_5;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__pyx_unpickle_SystemMemory", 0);

  /* "(tree fragment)":6
 * def __pyx_unpickle_SystemMemory(__pyx_type, long __pyx_checksum, tuple __pyx_state):
 *     cdef object __pyx_result
 *     __Pyx_CheckUnpickleChecksum(__pyx_checksum, 0xa9b59d9, 0xa91bd4c, 0x383c807, b'_owner, device_id, ptr, size')             # <<<<<<<<<<<<<<
 *     __pyx_result = SystemMemory.__new__(__pyx_type)
 *     if __pyx_state is not None:
*/
  __pyx_t_1 = __Pyx_CheckUnpickleChecksum(__pyx_v___pyx_checksum, 0xa9b59d9, 0xa91bd4c, 0x383c807, __pyx_k_owner_device_id_ptr_size); if (unlikely(__pyx_t_1 == ((int)-1))) __PYX_ERR(3, 6, __pyx_L1_error)

  /* "(tree fragment)":7
 *     cdef object __pyx_result
 *     __Pyx_CheckUnpickleChecksum(__pyx_checksum, 0xa9b59d9, 0xa91bd4c, 0x383c807, b'_owner, device_id, ptr, size')
 *     __pyx_result = SystemMemory.__new__(__pyx_type)             # <<<<<<<<<<<<<<
 *     if __pyx_state is not None:
 *         __pyx_unpickle_SystemMemory__set_state(<SystemMemory> __pyx_result, __pyx_state)
*/
  __pyx_t_3 = ((PyObject *)__pyx_mstate_global->__pyx_ptype_4cupy_4cuda_6memory_SystemMemory);
  __Pyx_INCREF(__pyx_t_3);
  __pyx_t_4 = 0;
  {
    PyObject *__pyx_callargs[2] = {__pyx_t_3, __pyx_v___pyx_type};
    __pyx_t_2 = __Pyx_PyObject_FastCallMethod((PyObject*)__pyx_mstate_global->__pyx_n_u_new, __pyx_callargs+__pyx_t_4, (2-__pyx_t_4) | (1*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
    __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
    if (unlikely(!__pyx_t_2)) __PYX_ERR(3, 7, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
  }
  __pyx_v___pyx_result = __pyx_t_2;
  __pyx_t_2 = 0;

  /* "(tree fragment)":8
 *     __Pyx_CheckUnpickleChecksum(__pyx_checksum, 0xa9b59d9, 0xa91bd4c, 0x383c807, b'_owner, device_id, ptr, size')
 *     __pyx_result = SystemMemory.__new__(__pyx_type)
 *     if __pyx_state is not None:             # <<<<<<<<<<<<<<
 *         __pyx_unpickle_SystemMemory__set_state(<SystemMemory> __pyx_result, __pyx_state)
 *     return __pyx_result
*/
  __pyx_t_5 = (__pyx_v___pyx_state != ((PyObject*)Py_None));
  if (__pyx_t_5) {

    /* "(tree fragment)":9
 *     __pyx_result = SystemMemory.__new__(__pyx_type)
 *     if __pyx_state is not None:
 *         __pyx_unpickle_SystemMemory__set_state(<SystemMemory> __pyx_result, __pyx_state)             # <<<<<<<<<<<<<<
 *     return __pyx_result
 * cdef __pyx_unpickle_SystemMemory__set_state(SystemMemory __pyx_result, __pyx_state: tuple):
*/
    if (unlikely(__pyx_v___pyx_state == Py_None)) {
      PyErr_SetString(PyExc_TypeError, "cannot pass None into a C function argument that is declared 'not None'");
      __PYX_ERR(3, 9, __pyx_L1_error)
    }
    __pyx_t_2 = __pyx_f_4cupy_4cuda_6memory___pyx_unpickle_SystemMemory__set_state(((struct __pyx_obj_4cupy_4cuda_6memory_SystemMemory *)__pyx_v___pyx_result), __pyx_v___pyx_state); if (unlikely(!__pyx_t_2)) __PYX_ERR(3, 9, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

    /* "(tree fragment)":8
 *     __Pyx_CheckUnpickleChecksum(__pyx_checksum, 0xa9b59d9, 0xa91bd4c, 0x383c807, b'_owner, device_id, ptr, size')
 *     __pyx_result = SystemMemory.__new__(__pyx_type)
 *     if __pyx_state is not None:             # <<<<<<<<<<<<<<
 *         __pyx_unpickle_SystemMemory__set_state(<SystemMemory> __pyx_result, __pyx_state)
 *     return __pyx_result
*/
  }

  /* "(tree fragment)":10
 *     if __pyx_state is not None:
 *         __pyx_unpickle_SystemMemory__set_state(<SystemMemory> __pyx_result, __pyx_state)
 *     return __pyx_result             # <<<<<<<<<<<<<<
 * cdef __pyx_unpickle_SystemMemory__set_state(SystemMemory __pyx_result, __pyx_state: tuple):
 *     __pyx_result._owner = __pyx_state[0]; __pyx_result.device_id = __pyx_state[1]; __pyx_result.ptr = __pyx_state[2]; __pyx_result.size = __pyx_state[3]
*/
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_v___pyx_result);
  __pyx_r = __pyx_v___pyx_result;
  goto __pyx_L0;

  /* "(tree fragment)":4
 *     int __Pyx_CheckUnpickleChecksum(long, long, long, long, const char*) except -1
 *     int __Pyx_UpdateUnpickledDict(object, object, Py_ssize_t) except -1
 * def __pyx_unpickle_SystemMemory(__pyx_type, long __pyx_checksum, tuple __pyx_state):             # <<<<<<<<<<<<<<
 *     cdef object __pyx_result
 *     __Pyx_CheckUnpickleChecksum(__pyx_checksum, 0xa9b59d9, 0xa91bd4c, 0x383c807, b'_owner, device_id, ptr, size')
*/

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_AddTraceback("cupy.cuda.memory.__pyx_unpickle_SystemMemory", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v___pyx_result);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":11
 *         __pyx_unpickle_SystemMemory__set_state(<SystemMemory> __pyx_result, __pyx_state)
 *     return __pyx_result
 * cdef __pyx_unpickle_SystemMemory__set_state(SystemMemory __pyx_result, __pyx_state: tuple):             # <<<<<<<<<<<<<<
 *     __pyx_result._owner = __pyx_state[0]; __pyx_result.device_id = __pyx_state[1]; __pyx_result.ptr = __pyx_state[2]; __pyx_result.size = __pyx_state[3]
 *     __Pyx_UpdateUnpickledDict(__pyx_result, __pyx_state, 4)
*/

static PyObject *__pyx_f_4cupy_4cuda_6memory___pyx_unpickle_SystemMemory__set_state(struct __pyx_obj_4cupy_4cuda_6memory_SystemMemory *__pyx_v___pyx_result, PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_t_2;
  intptr_t __pyx_t_3;
  size_t __pyx_t_4;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__pyx_unpickle_SystemMemory__set_state", 0);

  /* "(tree fragment)":12
 *     return __pyx_result
 * cdef __pyx_unpickle_SystemMemory__set_state(SystemMemory __pyx_result, __pyx_state: tuple):
 *     __pyx_result._owner = __pyx_state[0]; __pyx_result.device_id = __pyx_state[1]; __pyx_result.ptr = __pyx_state[2]; __pyx_result.size = __pyx_state[3]             # <<<<<<<<<<<<<<
 *     __Pyx_UpdateUnpickledDict(__pyx_result, __pyx_state, 4)
*/
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 0, long, 1, __Pyx_PyLong_From_long, 0, 0, 1, 1, __Pyx_ReferenceSharing_FunctionArgument); if (unlikely(!__pyx_t_1)) __PYX_ERR(3, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->_owner);
  __Pyx_DECREF(__pyx_v___pyx_result->_owner);
  __pyx_v___pyx_result->_owner = __pyx_t_1;
  __pyx_t_1 = 0;
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 1, long, 1, __Pyx_PyLong_From_long, 0, 0, 1, 1, __Pyx_ReferenceSharing_FunctionArgument); if (unlikely(!__pyx_t_1)) __PYX_ERR(3, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyLong_As_int(__pyx_t_1); if (unlikely((__pyx_t_2 == (int)-1) && PyErr_Occurred())) __PYX_ERR(3, 12, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v___pyx_result->__pyx_base.device_id = __pyx_t_2;
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 2, long, 1, __Pyx_PyLong_From_long, 0, 0, 1, 1, __Pyx_ReferenceSharing_FunctionArgument); if (unlikely(!__pyx_t_1)) __PYX_ERR(3, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_3 = PyLong_AsSsize_t(__pyx_t_1); if (unlikely((__pyx_t_3 == ((intptr_t)-1)) && PyErr_Occurred())) __PYX_ERR(3, 12, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v___pyx_result->__pyx_base.ptr = __pyx_t_3;
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 3, long, 1, __Pyx_PyLong_From_long, 0, 0, 1, 1, __Pyx_ReferenceSharing_FunctionArgument); if (unlikely(!__pyx_t_1)) __PYX_ERR(3, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_4 = __Pyx_PyLong_As_size_t(__pyx_t_1); if (unlikely((__pyx_t_4 == (size_t)-1) && PyErr_Occurred())) __PYX_ERR(3, 12, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v___pyx_result->__pyx_base.size = __pyx_t_4;

  /* "(tree fragment)":13
 * cdef __pyx_unpickle_SystemMemory__set_state(SystemMemory __pyx_result, __pyx_state: tuple):
 *     __pyx_result._owner = __pyx_state[0]; __pyx_result.device_id = __pyx_state[1]; __pyx_result.ptr = __pyx_state[2]; __pyx_result.size = __pyx_state[3]
 *     __Pyx_UpdateUnpickledDict(__pyx_result, __pyx_state, 4)             # <<<<<<<<<<<<<<
*/
  __pyx_t_2 = __Pyx_UpdateUnpickledDict(((PyObject *)__pyx_v___pyx_result), __pyx_v___pyx_state, 4); if (unlikely(__pyx_t_2 == ((int)-1))) __PYX_ERR(3, 13, __pyx_L1_error)

  /* "(tree fragment)":11
 *         __pyx_unpickle_SystemMemory__set_state(<SystemMemory> __pyx_result, __pyx_state)
 *     return __pyx_result
 * cdef __pyx_unpickle_SystemMemory__set_state(SystemMemory __pyx_result, __pyx_state: tuple):             # <<<<<<<<<<<<<<
 *     __pyx_result._owner = __pyx_state[0]; __pyx_result.device_id = __pyx_state[1]; __pyx_result.ptr = __pyx_state[2]; __pyx_result.size = __pyx_state[3]
 *     __Pyx_UpdateUnpickledDict(__pyx_result, __pyx_state, 4)
*/

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("cupy.cuda.memory.__pyx_unpickle_SystemMemory__set_state", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":4
 *     int __Pyx_CheckUnpickleChecksum(long, long, long, long, const char*) except -1
 *     int __Pyx_UpdateUnpickledDict(object, object, Py_ssize_t) except -1
 * def __pyx_unpickle__Chunk(__pyx_type, long __pyx_checksum, tuple __pyx_state):             # <<<<<<<<<<<<<<
 *     cdef object __pyx_result
 *     __Pyx_CheckUnpickleChecksum(__pyx_checksum, 0x37829b1, 0x2c2d147, 0x0239ab7, b'arena, mem, next, offset, prev, size')
*/

/* Python wrapper */
static PyObject *__pyx_pw_4cupy_4cuda_6memory_37__pyx_unpickle__Chunk(PyObject *__pyx_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
PyDoc_STRVAR(__pyx_doc_4cupy_4cuda_6memory_36__pyx_unpickle__Chunk, "__pyx_unpickle__Chunk(__pyx_type, long __pyx_checksum, tuple __pyx_state)");
static PyMethodDef __pyx_mdef_4cupy_4cuda_6memory_37__pyx_unpickle__Chunk = {"__pyx_unpickle__Chunk", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_4cupy_4cuda_6memory_37__pyx_unpickle__Chunk, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_4cupy_4cuda_6memory_36__pyx_unpickle__Chunk};
static PyObject *__pyx_pw_4cupy_4cuda_6memory_37__pyx_unpickle__Chunk(PyObject *__pyx_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
) {
  PyObject *__pyx_v___pyx_type = 0;
  long __pyx_v___pyx_checksum;
  PyObject *__pyx_v___pyx_state = 0;
  #if !CYTHON_METH_FASTCALL
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  #endif
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject* values[3] = {0,0,0};
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__pyx_unpickle__Chunk (wrapper)", 0);
  #if !CYTHON_METH_FASTCALL
  #if CYTHON_ASSUME_SAFE_SIZE
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return NULL;
  #endif
  #endif
  __pyx_kwvalues = __Pyx_KwValues_FASTCALL(__pyx_args, __pyx_nargs);
  {
    PyObject ** const __pyx_pyargnames[] = {&__pyx_mstate_global->__pyx_n_u_pyx_type,&__pyx_mstate_global->__pyx_n_u_pyx_checksum,&__pyx_mstate_global->__pyx_n_u_pyx_state,0};
    const Py_ssize_t __pyx_kwds_len = (__pyx_kwds) ? __Pyx_NumKwargs_FASTCALL(__pyx_kwds) : 0;
    if (unlikely(__pyx_kwds_len < 0)) __PYX_ERR(3, 4, __pyx_L3_error)
    if (__pyx_kwds_len > 0) {
      switch (__pyx_nargs) {
        case  3:
        values[2] = __Pyx_ArgRef_FASTCALL(__pyx_args, 2);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[2])) __PYX_ERR(3, 4, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  2:
        values[1] = __Pyx_ArgRef_FASTCALL(__pyx_args, 1);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[1])) __PYX_ERR(3, 4, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  1:
        values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(3, 4, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      const Py_ssize_t kwd_pos_args = __pyx_nargs;
      if (__Pyx_ParseKeywords(__pyx_kwds, __pyx_kwvalues, __pyx_pyargnames, 0, values, kwd_pos_args, __pyx_kwds_len, "__pyx_unpickle__Chunk", 0) < (0)) __PYX_ERR(3, 4, __pyx_L3_error)
      for (Py_ssize_t i = __pyx_nargs; i < 3; i++) {
        if (unlikely(!values[i])) { __Pyx_RaiseArgtupleInvalid("__pyx_unpickle__Chunk", 1, 3, 3, i); __PYX_ERR(3, 4, __pyx_L3_error) }
      }
    } else if (unlikely(__pyx_nargs != 3)) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
      if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(3, 4, __pyx_L3_error)
      values[1] = __Pyx_ArgRef_FASTCALL(__pyx_args, 1);
      if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[1])) __PYX_ERR(3, 4, __pyx_L3_error)
      values[2] = __Pyx_ArgRef_FASTCALL(__pyx_args, 2);
      if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[2])) __PYX_ERR(3, 4, __pyx_L3_error)
    }
    __pyx_v___pyx_type = values[0];
    __pyx_v___pyx_checksum = __Pyx_PyLong_As_long(values[1]); if (unlikely((__pyx_v___pyx_checksum == (long)-1) && PyErr_Occurred())) __PYX_ERR(3, 4, __pyx_L3_error)
    __pyx_v___pyx_state = ((PyObject*)values[2]);
  }
  goto __pyx_L6_skip;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("__pyx_unpickle__Chunk", 1, 3, 3, __pyx_nargs); __PYX_ERR(3, 4, __pyx_L3_error)
  __pyx_L6_skip:;
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_AddTraceback("cupy.cuda.memory.__pyx_unpickle__Chunk", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v___pyx_state), (&PyTuple_Type), 1, "__pyx_state", 1))) __PYX_ERR(3, 4, __pyx_L1_error)
  __pyx_r = __pyx_pf_4cupy_4cuda_6memory_36__pyx_unpickle__Chunk(__pyx_self, __pyx_v___pyx_type, __pyx_v___pyx_checksum, __pyx_v___pyx_state);

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = NULL;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  goto __pyx_L7_cleaned_up;
  __pyx_L0:;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __pyx_L7_cleaned_up:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4cupy_4cuda_6memory_36__pyx_unpickle__Chunk(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v___pyx_type, long __pyx_v___pyx_checksum, PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_v___pyx_result = 0;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  size_t __pyx_t_4;
  int __pyx_t_5;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__pyx_unpickle__Chunk", 0);

  /* "(tree fragment)":6
 * def __pyx_unpickle__Chunk(__pyx_type, long __pyx_checksum, tuple __pyx_state):
 *     cdef object __pyx_result
 *     __Pyx_CheckUnpickleChecksum(__pyx_checksum, 0x37829b1, 0x2c2d147, 0x0239ab7, b'arena, mem, next, offset, prev, size')             # <<<<<<<<<<<<<<
 *     __pyx_result = _Chunk.__new__(__pyx_type)
 *     if __pyx_state is not None:
*/
  __pyx_t_1 = __Pyx_CheckUnpickleChecksum(__pyx_v___pyx_checksum, 0x37829b1, 0x2c2d147, 0x0239ab7, __pyx_k_arena_mem_next_offset_prev_size); if (unlikely(__pyx_t_1 == ((int)-1))) __PYX_ERR(3, 6, __pyx_L1_error)

  /* "(tree fragment)":7
 *     cdef object __pyx_result
 *     __Pyx_CheckUnpickleChecksum(__pyx_checksum, 0x37829b1, 0x2c2d147, 0x0239ab7, b'arena, mem, next, offset, prev, size')
 *     __pyx_result = _Chunk.__new__(__pyx_type)             # <<<<<<<<<<<<<<
 *     if __pyx_state is not None:
 *         __pyx_unpickle__Chunk__set_state(<_Chunk> __pyx_result, __pyx_state)
*/
  __pyx_t_3 = ((PyObject *)__pyx_mstate_global->__pyx_ptype_4cupy_4cuda_6memory__Chunk);
  __Pyx_INCREF(__pyx_t_3);
  __pyx_t_4 = 0;
  {
    PyObject *__pyx_callargs[2] = {__pyx_t_3, __pyx_v___pyx_type};
    __pyx_t_2 = __Pyx_PyObject_FastCallMethod((PyObject*)__pyx_mstate_global->__pyx_n_u_new, __pyx_callargs+__pyx_t_4, (2-__pyx_t_4) | (1*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
    __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
    if (unlikely(!__pyx_t_2)) __PYX_ERR(3, 7, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
  }
  __pyx_v___pyx_result = __pyx_t_2;
  __pyx_t_2 = 0;

  /* "(tree fragment)":8
 *     __Pyx_CheckUnpickleChecksum(__pyx_checksum, 0x37829b1, 0x2c2d147, 0x0239ab7, b'arena, mem, next, offset, prev, size')
 *     __pyx_result = _Chunk.__new__(__pyx_type)
 *     if __pyx_state is not None:             # <<<<<<<<<<<<<<
 *         __pyx_unpickle__Chunk__set_state(<_Chunk> __pyx_result, __pyx_state)
 *     return __pyx_result
*/
  __pyx_t_5 = (__pyx_v___pyx_state != ((PyObject*)Py_None));
  if (__pyx_t_5) {

    /* "(tree fragment)":9
 *     __pyx_result = _Chunk.__new__(__pyx_type)
 *     if __pyx_state is not None:
 *         __pyx_unpickle__Chunk__set_state(<_Chunk> __pyx_result, __pyx_state)             # <<<<<<<<<<<<<<
 *     return __pyx_result
 * cdef __pyx_unpickle__Chunk__set_state(_Chunk __pyx_result, __pyx_state: tuple):
*/
    if (unlikely(__pyx_v___pyx_state == Py_None)) {
      PyErr_SetString(PyExc_TypeError, "cannot pass None into a C function argument that is declared 'not None'");
      __PYX_ERR(3, 9, __pyx_L1_error)
    }
    __pyx_t_2 = __pyx_f_4cupy_4cuda_6memory___pyx_unpickle__Chunk__set_state(((struct __pyx_obj_4cupy_4cuda_6memory__Chunk *)__pyx_v___pyx_result), __pyx_v___pyx_state); if (unlikely(!__pyx_t_2)) __PYX_ERR(3, 9, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

    /* "(tree fragment)":8
 *     __Pyx_CheckUnpickleChecksum(__pyx_checksum, 0x37829b1, 0x2c2d147, 0x0239ab7, b'arena, mem, next, offset, prev, size')
 *     __pyx_result = _Chunk.__new__(__pyx_type)
 *     if __pyx_state is not None:             # <<<<<<<<<<<<<<
 *         __pyx_unpickle__Chunk__set_state(<_Chunk> __pyx_result, __pyx_state)
 *     return __pyx_result
*/
  }

  /* "(tree fragment)":10
 *     if __pyx_state is not None:
 *         __pyx_unpickle__Chunk__set_state(<_Chunk> __pyx_result, __pyx_state)
 *     return __pyx_result             # <<<<<<<<<<<<<<
 * cdef __pyx_unpickle__Chunk__set_state(_Chunk __pyx_result, __pyx_state: tuple):
 *     __pyx_result.arena = __pyx_state[0]; __pyx_result.mem = __pyx_state[1]; __pyx_result.next = __pyx_state[2]; __pyx_result.offset = __pyx_state[3]; __pyx_result.prev = __pyx_state[4]; __pyx_result.size = __pyx_state[5]
*/
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_v___pyx_result);
  __pyx_r = __pyx_v___pyx_result;
  goto __pyx_L0;

  /* "(tree fragment)":4
 *     int __Pyx_CheckUnpickleChecksum(long, long, long, long, const char*) except -1
 *     int __Pyx_UpdateUnpickledDict(object, object, Py_ssize_t) except -1
 * def __pyx_unpickle__Chunk(__pyx_type, long __pyx_checksum, tuple __pyx_state):             # <<<<<<<<<<<<<<
 *     cdef object __pyx_result
 *     __Pyx_CheckUnpickleChecksum(__pyx_checksum, 0x37829b1, 0x2c2d147, 0x0239ab7, b'arena, mem, next, offset, prev, size')
*/

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_AddTraceback("cupy.cuda.memory.__pyx_unpickle__Chunk", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v___pyx_result);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":11
 *         __pyx_unpickle__Chunk__set_state(<_Chunk> __pyx_result, __pyx_state)
 *     return __pyx_result
 * cdef __pyx_unpickle__Chunk__set_state(_Chunk __pyx_result, __pyx_state: tuple):             # <<<<<<<<<<<<<<
 *     __pyx_result.arena = __pyx_state[0]; __pyx_result.mem = __pyx_state[1]; __pyx_result.next = __pyx_state[2]; __pyx_result.offset = __pyx_state[3]; __pyx_result.prev = __pyx_state[4]; __pyx_result.size = __pyx_state[5]
 *     __Pyx_UpdateUnpickledDict(__pyx_result, __pyx_state, 6)
*/

static PyObject *__pyx_f_4cupy_4cuda_6memory___pyx_unpickle__Chunk__set_state(struct __pyx_obj_4cupy_4cuda_6memory__Chunk *__pyx_v___pyx_result, PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  ptrdiff_t __pyx_t_2;
  size_t __pyx_t_3;
  int __pyx_t_4;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__pyx_unpickle__Chunk__set_state", 0);

  /* "(tree fragment)":12
 *     return __pyx_result
 * cdef __pyx_unpickle__Chunk__set_state(_Chunk __pyx_result, __pyx_state: tuple):
 *     __pyx_result.arena = __pyx_state[0]; __pyx_result.mem = __pyx_state[1]; __pyx_result.next = __pyx_state[2]; __pyx_result.offset = __pyx_state[3]; __pyx_result.prev = __pyx_state[4]; __pyx_result.size = __pyx_state[5]             # <<<<<<<<<<<<<<
 *     __Pyx_UpdateUnpickledDict(__pyx_result, __pyx_state, 6)
*/
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 0, long, 1, __Pyx_PyLong_From_long, 0, 0, 1, 1, __Pyx_ReferenceSharing_FunctionArgument); if (unlikely(!__pyx_t_1)) __PYX_ERR(3, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (!(likely(((__pyx_t_1) == Py_None) || likely(__Pyx_TypeTest(__pyx_t_1, __pyx_mstate_global->__pyx_ptype_4cupy_4cuda_6memory__Arena))))) __PYX_ERR(3, 12, __pyx_L1_error)
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF((PyObject *)__pyx_v___pyx_result->arena);
  __Pyx_DECREF((PyObject *)__pyx_v___pyx_result->arena);
  __pyx_v___pyx_result->arena = ((struct __pyx_obj_4cupy_4cuda_6memory__Arena *)__pyx_t_1);
  __pyx_t_1 = 0;
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 1, long, 1, __Pyx_PyLong_From_long, 0, 0, 1, 1, __Pyx_ReferenceSharing_FunctionArgument); if (unlikely(!__pyx_t_1)) __PYX_ERR(3, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (!(likely(((__pyx_t_1) == Py_None) || likely(__Pyx_TypeTest(__pyx_t_1, __pyx_mstate_global->__pyx_ptype_4cupy_4cuda_6memory_BaseMemory))))) __PYX_ERR(3, 12, __pyx_L1_error)
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF((PyObject *)__pyx_v___pyx_result->mem);
  __Pyx_DECREF((PyObject *)__pyx_v___pyx_result->mem);
  __pyx_v___pyx_result->mem = ((struct __pyx_obj_4cupy_4cuda_6memory_BaseMemory *)__pyx_t_1);
  __pyx_t_1 = 0;
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 2, long, 1, __Pyx_PyLong_From_long, 0, 0, 1, 1, __Pyx_ReferenceSharing_FunctionArgument); if (unlikely(!__pyx_t_1)) __PYX_ERR(3, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (!(likely(((__pyx_t_1) == Py_None) || likely(__Pyx_TypeTest(__pyx_t_1, __pyx_mstate_global->__pyx_ptype_4cupy_4cuda_6memory__Chunk))))) __PYX_ERR(3, 12, __pyx_L1_error)
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF((PyObject *)__pyx_v___pyx_result->next);
  __Pyx_DECREF((PyObject *)__pyx_v___pyx_result->next);
  __pyx_v___pyx_result->next = ((struct __pyx_obj_4cupy_4cuda_6memory__Chunk *)__pyx_t_1);
  __pyx_t_1 = 0;
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 3, long, 1, __Pyx_PyLong_From_long, 0, 0, 1, 1, __Pyx_ReferenceSharing_FunctionArgument); if (unlikely(!__pyx_t_1)) __PYX_ERR(3, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyLong_As_ptrdiff_t(__pyx_t_1); if (unlikely((__pyx_t_2 == (ptrdiff_t)-1) && PyErr_Occurred())) __PYX_ERR(3, 12, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v___pyx_result->offset = __pyx_t_2;
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 4, long, 1, __Pyx_PyLong_From_long, 0, 0, 1, 1, __Pyx_ReferenceSharing_FunctionArgument); if (unlikely(!__pyx_t_1)) __PYX_ERR(3, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (!(likely(((__pyx_t_1) == Py_None) || likely(__Pyx_TypeTest(__pyx_t_1, __pyx_mstate_global->__pyx_ptype_4cupy_4cuda_6memory__Chunk))))) __PYX_ERR(3, 12, __pyx_L1_error)
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF((PyObject *)__pyx_v___pyx_result->prev);
  __Pyx_DECREF((PyObject *)__pyx_v___pyx_result->prev);
  __pyx_v___pyx_result->prev = ((struct __pyx_obj_4cupy_4cuda_6memory__Chunk *)__pyx_t_1);
  __pyx_t_1 = 0;
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 5, long, 1, __Pyx_PyLong_From_long, 0, 0, 1, 1, __Pyx_ReferenceSharing_FunctionArgument); if (unlikely(!__pyx_t_1)) __PYX_ERR(3, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_3 = __Pyx_PyLong_As_size_t(__pyx_t_1); if (unlikely((__pyx_t_3 == (size_t)-1) && PyErr_Occurred())) __PYX_ERR(3, 12, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v___pyx_result->size = __pyx_t_3;

  /* "(tree fragment)":13
 * cdef __pyx_unpickle__Chunk__set_state(_Chunk __pyx_result, __pyx_state: tuple):
 *     __pyx_result.arena = __pyx_state[0]; __pyx_result.mem = __pyx_state[1]; __pyx_result.next = __pyx_state[2]; __pyx_result.offset = __pyx_state[3]; __pyx_result.prev = __pyx_state[4]; __pyx_result.size = __pyx_state[5]
 *     __Pyx_UpdateUnpickledDict(__pyx_result, __pyx_state, 6)             # <<<<<<<<<<<<<<
*/
  __pyx_t_4 = __Pyx_UpdateUnpickledDict(((PyObject *)__pyx_v___pyx_result), __pyx_v___pyx_state, 6); if (unlikely(__pyx_t_4 == ((int)-1))) __PYX_ERR(3, 13, __pyx_L1_error)

  /* "(tree fragment)":11
 *         __pyx_unpickle__Chunk__set_state(<_Chunk> __pyx_result, __pyx_state)
 *     return __pyx_result
 * cdef __pyx_unpickle__Chunk__set_state(_Chunk __pyx_result, __pyx_state: tuple):             # <<<<<<<<<<<<<<
 *     __pyx_result.arena = __pyx_state[0]; __pyx_result.mem = __pyx_state[1]; __pyx_result.next = __pyx_state[2]; __pyx_result.offset = __pyx_state[3]; __pyx_result.prev = __pyx_state[4]; __pyx_result.size = __pyx_state[5]
 *     __Pyx_UpdateUnpickledDict(__pyx_result, __pyx_state, 6)
*/

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("cupy.cuda.memory.__pyx_unpickle__Chunk__set_state", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":4
 *     int __Pyx_CheckUnpickleChecksum(long, long, long, long, const char*) except -1
 *     int __Pyx_UpdateUnpickledDict(object, object, Py_ssize_t) except -1
 * def __pyx_unpickle_MemoryPointer(__pyx_type, long __pyx_checksum, tuple __pyx_state):             # <<<<<<<<<<<<<<
 *     cdef object __pyx_result
 *     __Pyx_CheckUnpickleChecksum(__pyx_checksum, 0x72cbb23, 0x6f47a22, 0x80a7757, b'device_id, mem, ptr')
*/

/* Python wrapper */
static PyObject *__pyx_pw_4cupy_4cuda_6memory_39__pyx_unpickle_MemoryPointer(PyObject *__pyx_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
PyDoc_STRVAR(__pyx_doc_4cupy_4cuda_6memory_38__pyx_unpickle_MemoryPointer, "__pyx_unpickle_MemoryPointer(__pyx_type, long __pyx_checksum, tuple __pyx_state)");
static PyMethodDef __pyx_mdef_4cupy_4cuda_6memory_39__pyx_unpickle_MemoryPointer = {"__pyx_unpickle_MemoryPointer", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_4cupy_4cuda_6memory_39__pyx_unpickle_MemoryPointer, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_4cupy_4cuda_6memory_38__pyx_unpickle_MemoryPointer};
static PyObject *__pyx_pw_4cupy_4cuda_6memory_39__pyx_unpickle_MemoryPointer(PyObject *__pyx_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
) {
  PyObject *__pyx_v___pyx_type = 0;
  long __pyx_v___pyx_checksum;
  PyObject *__pyx_v___pyx_state = 0;
  #if !CYTHON_METH_FASTCALL
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  #endif
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject* values[3] = {0,0,0};
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__pyx_unpickle_MemoryPointer (wrapper)", 0);
  #if !CYTHON_METH_FASTCALL
  #if CYTHON_ASSUME_SAFE_SIZE
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return NULL;
  #endif
  #endif
  __pyx_kwvalues = __Pyx_KwValues_FASTCALL(__pyx_args, __pyx_nargs);
  {
    PyObject ** const __pyx_pyargnames[] = {&__pyx_mstate_global->__pyx_n_u_pyx_type,&__pyx_mstate_global->__pyx_n_u_pyx_checksum,&__pyx_mstate_global->__pyx_n_u_pyx_state,0};
    const Py_ssize_t __pyx_kwds_len = (__pyx_kwds) ? __Pyx_NumKwargs_FASTCALL(__pyx_kwds) : 0;
    if (unlikely(__pyx_kwds_len < 0)) __PYX_ERR(3, 4, __pyx_L3_error)
    if (__pyx_kwds_len > 0) {
      switch (__pyx_nargs) {
        case  3:
        values[2] = __Pyx_ArgRef_FASTCALL(__pyx_args, 2);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[2])) __PYX_ERR(3, 4, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  2:
        values[1] = __Pyx_ArgRef_FASTCALL(__pyx_args, 1);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[1])) __PYX_ERR(3, 4, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  1:
        values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(3, 4, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      const Py_ssize_t kwd_pos_args = __pyx_nargs;
      if (__Pyx_ParseKeywords(__pyx_kwds, __pyx_kwvalues, __pyx_pyargnames, 0, values, kwd_pos_args, __pyx_kwds_len, "__pyx_unpickle_MemoryPointer", 0) < (0)) __PYX_ERR(3, 4, __pyx_L3_error)
      for (Py_ssize_t i = __pyx_nargs; i < 3; i++) {
        if (unlikely(!values[i])) { __Pyx_RaiseArgtupleInvalid("__pyx_unpickle_MemoryPointer", 1, 3, 3, i); __PYX_ERR(3, 4, __pyx_L3_error) }
      }
    } else if (unlikely(__pyx_nargs != 3)) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
      if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(3, 4, __pyx_L3_error)
      values[1] = __Pyx_ArgRef_FASTCALL(__pyx_args, 1);
      if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[1])) __PYX_ERR(3, 4, __pyx_L3_error)
      values[2] = __Pyx_ArgRef_FASTCALL(__pyx_args, 2);
      if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[2])) __PYX_ERR(3, 4, __pyx_L3_error)
    }
    __pyx_v___pyx_type = values[0];
    __pyx_v___pyx_checksum = __Pyx_PyLong_As_long(values[1]); if (unlikely((__pyx_v___pyx_checksum == (long)-1) && PyErr_Occurred())) __PYX_ERR(3, 4, __pyx_L3_error)
    __pyx_v___pyx_state = ((PyObject*)values[2]);
  }
  goto __pyx_L6_skip;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("__pyx_unpickle_MemoryPointer", 1, 3, 3, __pyx_nargs); __PYX_ERR(3, 4, __pyx_L3_error)
  __pyx_L6_skip:;
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_AddTraceback("cupy.cuda.memory.__pyx_unpickle_MemoryPointer", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v___pyx_state), (&PyTuple_Type), 1, "__pyx_state", 1))) __PYX_ERR(3, 4, __pyx_L1_error)
  __pyx_r = __pyx_pf_4cupy_4cuda_6memory_38__pyx_unpickle_MemoryPointer(__pyx_self, __pyx_v___pyx_type, __pyx_v___pyx_checksum, __pyx_v___pyx_state);

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = NULL;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  goto __pyx_L7_cleaned_up;
  __pyx_L0:;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __pyx_L7_cleaned_up:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4cupy_4cuda_6memory_38__pyx_unpickle_MemoryPointer(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v___pyx_type, long __pyx_v___pyx_checksum, PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_v___pyx_result = 0;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  size_t __pyx_t_4;
  int __pyx_t_5;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__pyx_unpickle_MemoryPointer", 0);

  /* "(tree fragment)":6
 * def __pyx_unpickle_MemoryPointer(__pyx_type, long __pyx_checksum, tuple __pyx_state):
 *     cdef object __pyx_result
 *     __Pyx_CheckUnpickleChecksum(__pyx_checksum, 0x72cbb23, 0x6f47a22, 0x80a7757, b'device_id, mem, ptr')             # <<<<<<<<<<<<<<
 *     __pyx_result = MemoryPointer.__new__(__pyx_type)
 *     if __pyx_state is not None:
*/
  __pyx_t_1 = __Pyx_CheckUnpickleChecksum(__pyx_v___pyx_checksum, 0x72cbb23, 0x6f47a22, 0x80a7757, __pyx_k_device_id_mem_ptr); if (unlikely(__pyx_t_1 == ((int)-1))) __PYX_ERR(3, 6, __pyx_L1_error)

  /* "(tree fragment)":7
 *     cdef object __pyx_result
 *     __Pyx_CheckUnpickleChecksum(__pyx_checksum, 0x72cbb23, 0x6f47a22, 0x80a7757, b'device_id, mem, ptr')
 *     __pyx_result = MemoryPointer.__new__(__pyx_type)             # <<<<<<<<<<<<<<
 *     if __pyx_state is not None:
 *         __pyx_unpickle_MemoryPointer__set_state(<MemoryPointer> __pyx_result, __pyx_state)
*/
  __pyx_t_3 = ((PyObject *)__pyx_mstate_global->__pyx_ptype_4cupy_4cuda_6memory_MemoryPointer);
  __Pyx_INCREF(__pyx_t_3);
  __pyx_t_4 = 0;
  {
    PyObject *__pyx_callargs[2] = {__pyx_t_3, __pyx_v___pyx_type};
    __pyx_t_2 = __Pyx_PyObject_FastCallMethod((PyObject*)__pyx_mstate_global->__pyx_n_u_new, __pyx_callargs+__pyx_t_4, (2-__pyx_t_4) | (1*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
    __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
    if (unlikely(!__pyx_t_2)) __PYX_ERR(3, 7, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
  }
  __pyx_v___pyx_result = __pyx_t_2;
  __pyx_t_2 = 0;

  /* "(tree fragment)":8
 *     __Pyx_CheckUnpickleChecksum(__pyx_checksum, 0x72cbb23, 0x6f47a22, 0x80a7757, b'device_id, mem, ptr')
 *     __pyx_result = MemoryPointer.__new__(__pyx_type)
 *     if __pyx_state is not None:             # <<<<<<<<<<<<<<
 *         __pyx_unpickle_MemoryPointer__set_state(<MemoryPointer> __pyx_result, __pyx_state)
 *     return __pyx_result
*/
  __pyx_t_5 = (__pyx_v___pyx_state != ((PyObject*)Py_None));
  if (__pyx_t_5) {

    /* "(tree fragment)":9
 *     __pyx_result = MemoryPointer.__new__(__pyx_type)
 *     if __pyx_state is not None:
 *         __pyx_unpickle_MemoryPointer__set_state(<MemoryPointer> __pyx_result, __pyx_state)             # <<<<<<<<<<<<<<
 *     return __pyx_result
 * cdef __pyx_unpickle_MemoryPointer__set_state(MemoryPointer __pyx_result, __pyx_state: tuple):
*/
    if (unlikely(__pyx_v___pyx_state == Py_None)) {
      PyErr_SetString(PyExc_TypeError, "cannot pass None into a C function argument that is declared 'not None'");
      __PYX_ERR(3, 9, __pyx_L1_error)
    }
    __pyx_t_2 = __pyx_f_4cupy_4cuda_6memory___pyx_unpickle_MemoryPointer__set_state(((struct __pyx_obj_4cupy_4cuda_6memory_MemoryPointer *)__pyx_v___pyx_result), __pyx_v___pyx_state); if (unlikely(!__pyx_t_2)) __PYX_ERR(3, 9, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

    /* "(tree fragment)":8
 *     __Pyx_CheckUnpickleChecksum(__pyx_checksum, 0x72cbb23, 0x6f47a22, 0x80a7757, b'device_id, mem, ptr')
 *     __pyx_result = MemoryPointer.__new__(__pyx_type)
 *     if __pyx_state is not None:             # <<<<<<<<<<<<<<
 *         __pyx_unpickle_MemoryPointer__set_state(<MemoryPointer> __pyx_result, __pyx_state)
 *     return __pyx_result
*/
  }

  /* "(tree fragment)":10
 *     if __pyx_state is not None:
 *         __pyx_unpickle_MemoryPointer__set_state(<MemoryPointer> __pyx_result, __pyx_state)
 *     return __pyx_result             # <<<<<<<<<<<<<<
 * cdef __pyx_unpickle_MemoryPointer__set_state(MemoryPointer __pyx_result, __pyx_state: tuple):
 *     __pyx_result.device_id = __pyx_state[0]; __pyx_result.mem = __pyx_state[1]; __pyx_result.ptr = __pyx_state[2]
*/
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_v___pyx_result);
  __pyx_r = __pyx_v___pyx_result;
  goto __pyx_L0;

  /* "(tree fragment)":4
 *     int __Pyx_CheckUnpickleChecksum(long, long, long, long, const char*) except -1
 *     int __Pyx_UpdateUnpickledDict(object, object, Py_ssize_t) except -1
 * def __pyx_unpickle_MemoryPointer(__pyx_type, long __pyx_checksum, tuple __pyx_state):             # <<<<<<<<<<<<<<
 *     cdef object __pyx_result
 *     __Pyx_CheckUnpickleChecksum(__pyx_checksum, 0x72cbb23, 0x6f47a22, 0x80a7757, b'device_id, mem, ptr')
*/

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_AddTraceback("cupy.cuda.memory.__pyx_unpickle_MemoryPointer", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v___pyx_result);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":11
 *         __pyx_unpickle_MemoryPointer__set_state(<MemoryPointer> __pyx_result, __pyx_state)
 *     return __pyx_result
 * cdef __pyx_unpickle_MemoryPointer__set_state(MemoryPointer __pyx_result, __pyx_state: tuple):             # <<<<<<<<<<<<<<
 *     __pyx_result.device_id = __pyx_state[0]; __pyx_result.mem = __pyx_state[1]; __pyx_result.ptr = __pyx_state[2]
 *     __Pyx_UpdateUnpickledDict(__pyx_result, __pyx_state, 3)
*/

static PyObject *__pyx_f_4cupy_4cuda_6memory___pyx_unpickle_MemoryPointer__set_state(struct __pyx_obj_4cupy_4cuda_6memory_MemoryPointer *__pyx_v___pyx_result, PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_t_2;
  intptr_t __pyx_t_3;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__pyx_unpickle_MemoryPointer__set_state", 0);

  /* "(tree fragment)":12
 *     return __pyx_result
 * cdef __pyx_unpickle_MemoryPointer__set_state(MemoryPointer __pyx_result, __pyx_state: tuple):
 *     __pyx_result.device_id = __pyx_state[0]; __pyx_result.mem = __pyx_state[1]; __pyx_result.ptr = __pyx_state[2]             # <<<<<<<<<<<<<<
 *     __Pyx_UpdateUnpickledDict(__pyx_result, __pyx_state, 3)
*/
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 0, long, 1, __Pyx_PyLong_From_long, 0, 0, 1, 1, __Pyx_ReferenceSharing_FunctionArgument); if (unlikely(!__pyx_t_1)) __PYX_ERR(3, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyLong_As_int(__pyx_t_1); if (unlikely((__pyx_t_2 == (int)-1) && PyErr_Occurred())) __PYX_ERR(3, 12, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v___pyx_result->device_id = __pyx_t_2;
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 1, long, 1, __Pyx_PyLong_From_long, 0, 0, 1, 1, __Pyx_ReferenceSharing_FunctionArgument); if (unlikely(!__pyx_t_1)) __PYX_ERR(3, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (!(likely(((__pyx_t_1) == Py_None) || likely(__Pyx_TypeTest(__pyx_t_1, __pyx_mstate_global->__pyx_ptype_4cupy_4cuda_6memory_BaseMemory))))) __PYX_ERR(3, 12, __pyx_L1_error)
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF((PyObject *)__pyx_v___pyx_result->mem);
  __Pyx_DECREF((PyObject *)__pyx_v___pyx_result->mem);
  __pyx_v___pyx_result->mem = ((struct __pyx_obj_4cupy_4cuda_6memory_BaseMemory *)__pyx_t_1);
  __pyx_t_1 = 0;
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 2, long, 1, __Pyx_PyLong_From_long, 0, 0, 1, 1, __Pyx_ReferenceSharing_FunctionArgument); if (unlikely(!__pyx_t_1)) __PYX_ERR(3, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_3 = PyLong_AsSsize_t(__pyx_t_1); if (unlikely((__pyx_t_3 == ((intptr_t)-1)) && PyErr_Occurred())) __PYX_ERR(3, 12, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v___pyx_result->ptr = __pyx_t_3;

  /* "(tree fragment)":13
 * cdef __pyx_unpickle_MemoryPointer__set_state(MemoryPointer __pyx_result, __pyx_state: tuple):
 *     __pyx_result.device_id = __pyx_state[0]; __pyx_result.mem = __pyx_state[1]; __pyx_result.ptr = __pyx_state[2]
 *     __Pyx_UpdateUnpickledDict(__pyx_result, __pyx_state, 3)             # <<<<<<<<<<<<<<
*/
  __pyx_t_2 = __Pyx_UpdateUnpickledDict(((PyObject *)__pyx_v___pyx_result), __pyx_v___pyx_state, 3); if (unlikely(__pyx_t_2 == ((int)-1))) __PYX_ERR(3, 13, __pyx_L1_error)

  /* "(tree fragment)":11
 *         __pyx_unpickle_MemoryPointer__set_state(<MemoryPointer> __pyx_result, __pyx_state)
 *     return __pyx_result
 * cdef __pyx_unpickle_MemoryPointer__set_state(MemoryPointer __pyx_result, __pyx_state: tuple):             # <<<<<<<<<<<<<<
 *     __pyx_result.device_id = __pyx_state[0]; __pyx_result.mem = __pyx_state[1]; __pyx_result.ptr = __pyx_state[2]
 *     __Pyx_UpdateUnpickledDict(__pyx_result, __pyx_state, 3)
*/

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("cupy.cuda.memory.__pyx_unpickle_MemoryPointer__set_state", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":4
 *     int __Pyx_CheckUnpickleChecksum(long, long, long, long, const char*) except -1
 *     int __Pyx_UpdateUnpickledDict(object, object, Py_ssize_t) except -1
 * def __pyx_unpickle_PooledMemory(__pyx_type, long __pyx_checksum, tuple __pyx_state):             # <<<<<<<<<<<<<<
 *     cdef object __pyx_result
 *     __Pyx_CheckUnpickleChecksum(__pyx_checksum, 0x43cf7ab, 0xe717094, 0x15db62e, b'chunk, device_id, identity, pool, ptr, size')
*/

/* Python wrapper */
static PyObject *__pyx_pw_4cupy_4cuda_6memory_41__pyx_unpickle_PooledMemory(PyObject *__pyx_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
PyDoc_STRVAR(__pyx_doc_4cupy_4cuda_6memory_40__pyx_unpickle_PooledMemory, "__pyx_unpickle_PooledMemory(__pyx_type, long __pyx_checksum, tuple __pyx_state)");
static PyMethodDef __pyx_mdef_4cupy_4cuda_6memory_41__pyx_unpickle_PooledMemory = {"__pyx_unpickle_PooledMemory", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_4cupy_4cuda_6memory_41__pyx_unpickle_PooledMemory, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_4cupy_4cuda_6memory_40__pyx_unpickle_PooledMemory};
static PyObject *__pyx_pw_4cupy_4cuda_6memory_41__pyx_unpickle_PooledMemory(PyObject *__pyx_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
) {
  PyObject *__pyx_v___pyx_type = 0;
  long __pyx_v___pyx_checksum;
  PyObject *__pyx_v___pyx_state = 0;
  #if !CYTHON_METH_FASTCALL
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  #endif
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject* values[3] = {0,0,0};
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__pyx_unpickle_PooledMemory (wrapper)", 0);
  #if !CYTHON_METH_FASTCALL
  #if CYTHON_ASSUME_SAFE_SIZE
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return NULL;
  #endif
  #endif
  __pyx_kwvalues = __Pyx_KwValues_FASTCALL(__pyx_args, __pyx_nargs);
  {
    PyObject ** const __pyx_pyargnames[] = {&__pyx_mstate_global->__pyx_n_u_pyx_type,&__pyx_mstate_global->__pyx_n_u_pyx_checksum,&__pyx_mstate_global->__pyx_n_u_pyx_state,0};
    const Py_ssize_t __pyx_kwds_len = (__pyx_kwds) ? __Pyx_NumKwargs_FASTCALL(__pyx_kwds) : 0;
    if (unlikely(__pyx_kwds_len < 0)) __PYX_ERR(3, 4, __pyx_L3_error)
    if (__pyx_kwds_len > 0) {
      switch (__pyx_nargs) {
        case  3:
        values[2] = __Pyx_ArgRef_FASTCALL(__pyx_args, 2);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[2])) __PYX_ERR(3, 4, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  2:
        values[1] = __Pyx_ArgRef_FASTCALL(__pyx_args, 1);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[1])) __PYX_ERR(3, 4, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  1:
        values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(3, 4, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      const Py_ssize_t kwd_pos_args = __pyx_nargs;
      if (__Pyx_ParseKeywords(__pyx_kwds, __pyx_kwvalues, __pyx_pyargnames, 0, values, kwd_pos_args, __pyx_kwds_len, "__pyx_unpickle_PooledMemory", 0) < (0)) __PYX_ERR(3, 4, __pyx_L3_error)
      for (Py_ssize_t i = __pyx_nargs; i < 3; i++) {
        if (unlikely(!values[i])) { __Pyx_RaiseArgtupleInvalid("__pyx_unpickle_PooledMemory", 1, 3, 3, i); __PYX_ERR(3, 4, __pyx_L3_error) }
      }
    } else if (unlikely(__pyx_nargs != 3)) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
      if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(3, 4, __pyx_L3_error)
      values[1] = __Pyx_ArgRef_FASTCALL(__pyx_args, 1);
      if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[1])) __PYX_ERR(3, 4, __pyx_L3_error)
      values[2] = __Pyx_ArgRef_FASTCALL(__pyx_args, 2);
      if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[2])) __PYX_ERR(3, 4, __pyx_L3_error)
    }
    __pyx_v___pyx_type = values[0];
    __pyx_v___pyx_checksum = __Pyx_PyLong_As_long(values[1]); if (unlikely((__pyx_v___pyx_checksum == (long)-1) && PyErr_Occurred())) __PYX_ERR(3, 4, __pyx_L3_error)
    __pyx_v___pyx_state = ((PyObject*)values[2]);
  }
  goto __pyx_L6_skip;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("__pyx_unpickle_PooledMemory", 1, 3, 3, __pyx_nargs); __PYX_ERR(3, 4, __pyx_L3_error)
  __pyx_L6_skip:;
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_AddTraceback("cupy.cuda.memory.__pyx_unpickle_PooledMemory", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v___pyx_state), (&PyTuple_Type), 1, "__pyx_state", 1))) __PYX_ERR(3, 4, __pyx_L1_error)
  __pyx_r = __pyx_pf_4cupy_4cuda_6memory_40__pyx_unpickle_PooledMemory(__pyx_self, __pyx_v___pyx_type, __pyx_v___pyx_checksum, __pyx_v___pyx_state);

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = NULL;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  goto __pyx_L7_cleaned_up;
  __pyx_L0:;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __pyx_L7_cleaned_up:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4cupy_4cuda_6memory_40__pyx_unpickle_PooledMemory(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v___pyx_type, long __pyx_v___pyx_checksum, PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_v___pyx_result = 0;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  size_t __pyx_t_4;
  int __pyx_t_5;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__pyx_unpickle_PooledMemory", 0);

  /* "(tree fragment)":6
 * def __pyx_unpickle_PooledMemory(__pyx_type, long __pyx_checksum, tuple __pyx_state):
 *     cdef object __pyx_result
 *     __Pyx_CheckUnpickleChecksum(__pyx_checksum, 0x43cf7ab, 0xe717094, 0x15db62e, b'chunk, device_id, identity, pool, ptr, size')             # <<<<<<<<<<<<<<
 *     __pyx_result = PooledMemory.__new__(__pyx_type)
 *     if __pyx_state is not None:
*/
  __pyx_t_1 = __Pyx_CheckUnpickleChecksum(__pyx_v___pyx_checksum, 0x43cf7ab, 0xe717094, 0x15db62e, __pyx_k_chunk_device_id_identity_pool_pt); if (unlikely(__pyx_t_1 == ((int)-1))) __PYX_ERR(3, 6, __pyx_L1_error)

  /* "(tree fragment)":7
 *     cdef object __pyx_result
 *     __Pyx_CheckUnpickleChecksum(__pyx_checksum, 0x43cf7ab, 0xe717094, 0x15db62e, b'chunk, device_id, identity, pool, ptr, size')
 *     __pyx_result = PooledMemory.__new__(__pyx_type)             # <<<<<<<<<<<<<<
 *     if __pyx_state is not None:
 *         __pyx_unpickle_PooledMemory__set_state(<PooledMemory> __pyx_result, __pyx_state)
*/
  __pyx_t_3 = ((PyObject *)__pyx_mstate_global->__pyx_ptype_4cupy_4cuda_6memory_PooledMemory);
  __Pyx_INCREF(__pyx_t_3);
  __pyx_t_4 = 0;
  {
    PyObject *__pyx_callargs[2] = {__pyx_t_3, __pyx_v___pyx_type};
    __pyx_t_2 = __Pyx_PyObject_FastCallMethod((PyObject*)__pyx_mstate_global->__pyx_n_u_new, __pyx_callargs+__pyx_t_4, (2-__pyx_t_4) | (1*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
    __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
    if (unlikely(!__pyx_t_2)) __PYX_ERR(3, 7, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
  }
  __pyx_v___pyx_result = __pyx_t_2;
  __pyx_t_2 = 0;

  /* "(tree fragment)":8
 *     __Pyx_CheckUnpickleChecksum(__pyx_checksum, 0x43cf7ab, 0xe717094, 0x15db62e, b'chunk, device_id, identity, pool, ptr, size')
 *     __pyx_result = PooledMemory.__new__(__pyx_type)
 *     if __pyx_state is not None:             # <<<<<<<<<<<<<<
 *         __pyx_unpickle_PooledMemory__set_state(<PooledMemory> __pyx_result, __pyx_state)
 *     return __pyx_result
*/
  __pyx_t_5 = (__pyx_v___pyx_state != ((PyObject*)Py_None));
  if (__pyx_t_5) {

    /* "(tree fragment)":9
 *     __pyx_result = PooledMemory.__new__(__pyx_type)
 *     if __pyx_state is not None:
 *         __pyx_unpickle_PooledMemory__set_state(<PooledMemory> __pyx_result, __pyx_state)             # <<<<<<<<<<<<<<
 *     return __pyx_result
 * cdef __pyx_unpickle_PooledMemory__set_state(PooledMemory __pyx_result, __pyx_state: tuple):
*/
    if (unlikely(__pyx_v___pyx_state == Py_None)) {
      PyErr_SetString(PyExc_TypeError, "cannot pass None into a C function argument that is declared 'not None'");
      __PYX_ERR(3, 9, __pyx_L1_error)
    }
    __pyx_t_2 = __pyx_f_4cupy_4cuda_6memory___pyx_unpickle_PooledMemory__set_state(((struct __pyx_obj_4cupy_4cuda_6memory_PooledMemory *)__pyx_v___pyx_result), __pyx_v___pyx_state); if (unlikely(!__pyx_t_2)) __PYX_ERR(3, 9, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

    /* "(tree fragment)":8
 *     __Pyx_CheckUnpickleChecksum(__pyx_checksum, 0x43cf7ab, 0xe717094, 0x15db62e, b'chunk, device_id, identity, pool, ptr, size')
 *     __pyx_result = PooledMemory.__new__(__pyx_type)
 *     if __pyx_state is not None:             # <<<<<<<<<<<<<<
 *         __pyx_unpickle_PooledMemory__set_state(<PooledMemory> __pyx_result, __pyx_state)
 *     return __pyx_result
*/
  }

  /* "(tree fragment)":10
 *     if __pyx_state is not None:
 *         __pyx_unpickle_PooledMemory__set_state(<PooledMemory> __pyx_result, __pyx_state)
 *     return __pyx_result             # <<<<<<<<<<<<<<
 * cdef __pyx_unpickle_PooledMemory__set_state(PooledMemory __pyx_result, __pyx_state: tuple):
 *     __pyx_result.chunk = __pyx_state[0]; __pyx_result.device_id = __pyx_state[1]; __pyx_result.identity = __pyx_state[2]; __pyx_result.pool = __pyx_state[3]; __pyx_result.ptr = __pyx_state[4]; __pyx_result.size = __pyx_state[5]
*/
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_v___pyx_result);
  __pyx_r = __pyx_v___pyx_result;
  goto __pyx_L0;

  /* "(tree fragment)":4
 *     int __Pyx_CheckUnpickleChecksum(long, long, long, long, const char*) except -1
 *     int __Pyx_UpdateUnpickledDict(object, object, Py_ssize_t) except -1
 * def __pyx_unpickle_PooledMemory(__pyx_type, long __pyx_checksum, tuple __pyx_state):             # <<<<<<<<<<<<<<
 *     cdef object __pyx_result
 *     __Pyx_CheckUnpickleChecksum(__pyx_checksum, 0x43cf7ab, 0xe717094, 0x15db62e, b'chunk, device_id, identity, pool, ptr, size')
*/

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_AddTraceback("cupy.cuda.memory.__pyx_unpickle_PooledMemory", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v___pyx_result);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":11
 *         __pyx_unpickle_PooledMemory__set_state(<PooledMemory> __pyx_result, __pyx_state)
 *     return __pyx_result
 * cdef __pyx_unpickle_PooledMemory__set_state(PooledMemory __pyx_result, __pyx_state: tuple):             # <<<<<<<<<<<<<<
 *     __pyx_result.chunk = __pyx_state[0]; __pyx_result.device_id = __pyx_state[1]; __pyx_result.identity = __pyx_state[2]; __pyx_result.pool = __pyx_state[3]; __pyx_result.ptr = __pyx_state[4]; __pyx_result.size = __pyx_state[5]
 *     __Pyx_UpdateUnpickledDict(__pyx_result, __pyx_state, 6)
*/

static PyObject *__pyx_f_4cupy_4cuda_6memory___pyx_unpickle_PooledMemory__set_state(struct __pyx_obj_4cupy_4cuda_6memory_PooledMemory *__pyx_v___pyx_result, PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_t_2;
  intptr_t __pyx_t_3;
  size_t __pyx_t_4;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__pyx_unpickle_PooledMemory__set_state", 0);

  /* "(tree fragment)":12
 *     return __pyx_result
 * cdef __pyx_unpickle_PooledMemory__set_state(PooledMemory __pyx_result, __pyx_state: tuple):
 *     __pyx_result.chunk = __pyx_state[0]; __pyx_result.device_id = __pyx_state[1]; __pyx_result.identity = __pyx_state[2]; __pyx_result.pool = __pyx_state[3]; __pyx_result.ptr = __pyx_state[4]; __pyx_result.size = __pyx_state[5]             # <<<<<<<<<<<<<<
 *     __Pyx_UpdateUnpickledDict(__pyx_result, __pyx_state, 6)
*/
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 0, long, 1, __Pyx_PyLong_From_long, 0, 0, 1, 1, __Pyx_ReferenceSharing_FunctionArgument); if (unlikely(!__pyx_t_1)) __PYX_ERR(3, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (!(likely(((__pyx_t_1) == Py_None) || likely(__Pyx_TypeTest(__pyx_t_1, __pyx_mstate_global->__pyx_ptype_4cupy_4cuda_6memory__Chunk))))) __PYX_ERR(3, 12, __pyx_L1_error)
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF((PyObject *)__pyx_v___pyx_result->chunk);
  __Pyx_DECREF((PyObject *)__pyx_v___pyx_result->chunk);
  __pyx_v___pyx_result->chunk = ((struct __pyx_obj_4cupy_4cuda_6memory__Chunk *)__pyx_t_1);
  __pyx_t_1 = 0;
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 1, long, 1, __Pyx_PyLong_From_long, 0, 0, 1, 1, __Pyx_ReferenceSharing_FunctionArgument); if (unlikely(!__pyx_t_1)) __PYX_ERR(3, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyLong_As_int(__pyx_t_1); if (unlikely((__pyx_t_2 == (int)-1) && PyErr_Occurred())) __PYX_ERR(3, 12, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v___pyx_result->__pyx_base.device_id = __pyx_t_2;
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 2, long, 1, __Pyx_PyLong_From_long, 0, 0, 1, 1, __Pyx_ReferenceSharing_FunctionArgument); if (unlikely(!__pyx_t_1)) __PYX_ERR(3, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (!(likely(PyUnicode_CheckExact(__pyx_t_1))||((__pyx_t_1) == Py_None) || __Pyx_RaiseUnexpectedTypeError("str", __pyx_t_1))) __PYX_ERR(3, 12, __pyx_L1_error)
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->identity);
  __Pyx_DECREF(__pyx_v___pyx_result->identity);
  __pyx_v___pyx_result->identity = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 3, long, 1, __Pyx_PyLong_From_long, 0, 0, 1, 1, __Pyx_ReferenceSharing_FunctionArgument); if (unlikely(!__pyx_t_1)) __PYX_ERR(3, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->pool);
  __Pyx_DECREF(__pyx_v___pyx_result->pool);
  __pyx_v___pyx_result->pool = __pyx_t_1;
  __pyx_t_1 = 0;
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 4, long, 1, __Pyx_PyLong_From_long, 0, 0, 1, 1, __Pyx_ReferenceSharing_FunctionArgument); if (unlikely(!__pyx_t_1)) __PYX_ERR(3, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_3 = PyLong_AsSsize_t(__pyx_t_1); if (unlikely((__pyx_t_3 == ((intptr_t)-1)) && PyErr_Occurred())) __PYX_ERR(3, 12, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v___pyx_result->__pyx_base.ptr = __pyx_t_3;
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 5, long, 1, __Pyx_PyLong_From_long, 0, 0, 1, 1, __Pyx_ReferenceSharing_FunctionArgument); if (unlikely(!__pyx_t_1)) __PYX_ERR(3, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_4 = __Pyx_PyLong_As_size_t(__pyx_t_1); if (unlikely((__pyx_t_4 == (size_t)-1) && PyErr_Occurred())) __PYX_ERR(3, 12, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v___pyx_result->__pyx_base.size = __pyx_t_4;

  /* "(tree fragment)":13
 * cdef __pyx_unpickle_PooledMemory__set_state(PooledMemory __pyx_result, __pyx_state: tuple):
 *     __pyx_result.chunk = __pyx_state[0]; __pyx_result.device_id = __pyx_state[1]; __pyx_result.identity = __pyx_state[2]; __pyx_result.pool = __pyx_state[3]; __pyx_result.ptr = __pyx_state[4]; __pyx_result.size = __pyx_state[5]
 *     __Pyx_UpdateUnpickledDict(__pyx_result, __pyx_state, 6)             # <<<<<<<<<<<<<<
*/
  __pyx_t_2 = __Pyx_UpdateUnpickledDict(((PyObject *)__pyx_v___pyx_result), __pyx_v___pyx_state, 6); if (unlikely(__pyx_t_2 == ((int)-1))) __PYX_ERR(3, 13, __pyx_L1_error)

  /* "(tree fragment)":11
 *         __pyx_unpickle_PooledMemory__set_state(<PooledMemory> __pyx_result, __pyx_state)
 *     return __pyx_result
 * cdef __pyx_unpickle_PooledMemory__set_state(PooledMemory __pyx_result, __pyx_state: tuple):             # <<<<<<<<<<<<<<
 *     __pyx_result.chunk = __pyx_state[0]; __pyx_result.device_id = __pyx_state[1]; __pyx_result.identity = __pyx_state[2]; __pyx_result.pool = __pyx_state[3]; __pyx_result.ptr = __pyx_state[4]; __pyx_result.size = __pyx_state[5]
 *     __Pyx_UpdateUnpickledDict(__pyx_result, __pyx_state, 6)
*/

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("cupy.cuda.memory.__pyx_unpickle_PooledMemory__set_state", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":4
 *     int __Pyx_CheckUnpickleChecksum(long, long, long, long, const char*) except -1
 *     int __Pyx_UpdateUnpickledDict(object, object, Py_ssize_t) except -1
 * def __pyx_unpickle_MemoryPool(__pyx_type, long __pyx_checksum, tuple __pyx_state):             # <<<<<<<<<<<<<<
 *     cdef object __pyx_result
 *     __Pyx_CheckUnpickleChecksum(__pyx_checksum, 0x7925074, 0xdd80965, 0xbe76b6e, b'_allocator, _pools')
*/

/* Python wrapper */
static PyObject *__pyx_pw_4cupy_4cuda_6memory_43__pyx_unpickle_MemoryPool(PyObject *__pyx_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
PyDoc_STRVAR(__pyx_doc_4cupy_4cuda_6memory_42__pyx_unpickle_MemoryPool, "__pyx_unpickle_MemoryPool(__pyx_type, long __pyx_checksum, tuple __pyx_state)");
static PyMethodDef __pyx_mdef_4cupy_4cuda_6memory_43__pyx_unpickle_MemoryPool = {"__pyx_unpickle_MemoryPool", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_4cupy_4cuda_6memory_43__pyx_unpickle_MemoryPool, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_4cupy_4cuda_6memory_42__pyx_unpickle_MemoryPool};
static PyObject *__pyx_pw_4cupy_4cuda_6memory_43__pyx_unpickle_MemoryPool(PyObject *__pyx_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
) {
  PyObject *__pyx_v___pyx_type = 0;
  long __pyx_v___pyx_checksum;
  PyObject *__pyx_v___pyx_state = 0;
  #if !CYTHON_METH_FASTCALL
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  #endif
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject* values[3] = {0,0,0};
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__pyx_unpickle_MemoryPool (wrapper)", 0);
  #if !CYTHON_METH_FASTCALL
  #if CYTHON_ASSUME_SAFE_SIZE
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return NULL;
  #endif
  #endif
  __pyx_kwvalues = __Pyx_KwValues_FASTCALL(__pyx_args, __pyx_nargs);
  {
    PyObject ** const __pyx_pyargnames[] = {&__pyx_mstate_global->__pyx_n_u_pyx_type,&__pyx_mstate_global->__pyx_n_u_pyx_checksum,&__pyx_mstate_global->__pyx_n_u_pyx_state,0};
    const Py_ssize_t __pyx_kwds_len = (__pyx_kwds) ? __Pyx_NumKwargs_FASTCALL(__pyx_kwds) : 0;
    if (unlikely(__pyx_kwds_len < 0)) __PYX_ERR(3, 4, __pyx_L3_error)
    if (__pyx_kwds_len > 0) {
      switch (__pyx_nargs) {
        case  3:
        values[2] = __Pyx_ArgRef_FASTCALL(__pyx_args, 2);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[2])) __PYX_ERR(3, 4, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  2:
        values[1] = __Pyx_ArgRef_FASTCALL(__pyx_args, 1);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[1])) __PYX_ERR(3, 4, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  1:
        values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(3, 4, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      const Py_ssize_t kwd_pos_args = __pyx_nargs;
      if (__Pyx_ParseKeywords(__pyx_kwds, __pyx_kwvalues, __pyx_pyargnames, 0, values, kwd_pos_args, __pyx_kwds_len, "__pyx_unpickle_MemoryPool", 0) < (0)) __PYX_ERR(3, 4, __pyx_L3_error)
      for (Py_ssize_t i = __pyx_nargs; i < 3; i++) {
        if (unlikely(!values[i])) { __Pyx_RaiseArgtupleInvalid("__pyx_unpickle_MemoryPool", 1, 3, 3, i); __PYX_ERR(3, 4, __pyx_L3_error) }
      }
    } else if (unlikely(__pyx_nargs != 3)) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
      if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(3, 4, __pyx_L3_error)
      values[1] = __Pyx_ArgRef_FASTCALL(__pyx_args, 1);
      if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[1])) __PYX_ERR(3, 4, __pyx_L3_error)
      values[2] = __Pyx_ArgRef_FASTCALL(__pyx_args, 2);
      if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[2])) __PYX_ERR(3, 4, __pyx_L3_error)
    }
    __pyx_v___pyx_type = values[0];
    __pyx_v___pyx_checksum = __Pyx_PyLong_As_long(values[1]); if (unlikely((__pyx_v___pyx_checksum == (long)-1) && PyErr_Occurred())) __PYX_ERR(3, 4, __pyx_L3_error)
    __pyx_v___pyx_state = ((PyObject*)values[2]);
  }
  goto __pyx_L6_skip;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("__pyx_unpickle_MemoryPool", 1, 3, 3, __pyx_nargs); __PYX_ERR(3, 4, __pyx_L3_error)
  __pyx_L6_skip:;
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_AddTraceback("cupy.cuda.memory.__pyx_unpickle_MemoryPool", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v___pyx_state), (&PyTuple_Type), 1, "__pyx_state", 1))) __PYX_ERR(3, 4, __pyx_L1_error)
  __pyx_r = __pyx_pf_4cupy_4cuda_6memory_42__pyx_unpickle_MemoryPool(__pyx_self, __pyx_v___pyx_type, __pyx_v___pyx_checksum, __pyx_v___pyx_state);

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = NULL;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  goto __pyx_L7_cleaned_up;
  __pyx_L0:;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __pyx_L7_cleaned_up:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4cupy_4cuda_6memory_42__pyx_unpickle_MemoryPool(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v___pyx_type, long __pyx_v___pyx_checksum, PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_v___pyx_result = 0;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  size_t __pyx_t_4;
  int __pyx_t_5;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__pyx_unpickle_MemoryPool", 0);

  /* "(tree fragment)":6
 * def __pyx_unpickle_MemoryPool(__pyx_type, long __pyx_checksum, tuple __pyx_state):
 *     cdef object __pyx_result
 *     __Pyx_CheckUnpickleChecksum(__pyx_checksum, 0x7925074, 0xdd80965, 0xbe76b6e, b'_allocator, _pools')             # <<<<<<<<<<<<<<
 *     __pyx_result = MemoryPool.__new__(__pyx_type)
 *     if __pyx_state is not None:
*/
  __pyx_t_1 = __Pyx_CheckUnpickleChecksum(__pyx_v___pyx_checksum, 0x7925074, 0xdd80965, 0xbe76b6e, __pyx_k_allocator__pools); if (unlikely(__pyx_t_1 == ((int)-1))) __PYX_ERR(3, 6, __pyx_L1_error)

  /* "(tree fragment)":7
 *     cdef object __pyx_result
 *     __Pyx_CheckUnpickleChecksum(__pyx_checksum, 0x7925074, 0xdd80965, 0xbe76b6e, b'_allocator, _pools')
 *     __pyx_result = MemoryPool.__new__(__pyx_type)             # <<<<<<<<<<<<<<
 *     if __pyx_state is not None:
 *         __pyx_unpickle_MemoryPool__set_state(<MemoryPool> __pyx_result, __pyx_state)
*/
  __pyx_t_3 = ((PyObject *)__pyx_mstate_global->__pyx_ptype_4cupy_4cuda_6memory_MemoryPool);
  __Pyx_INCREF(__pyx_t_3);
  __pyx_t_4 = 0;
  {
    PyObject *__pyx_callargs[2] = {__pyx_t_3, __pyx_v___pyx_type};
    __pyx_t_2 = __Pyx_PyObject_FastCallMethod((PyObject*)__pyx_mstate_global->__pyx_n_u_new, __pyx_callargs+__pyx_t_4, (2-__pyx_t_4) | (1*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
    __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
    if (unlikely(!__pyx_t_2)) __PYX_ERR(3, 7, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
  }
  __pyx_v___pyx_result = __pyx_t_2;
  __pyx_t_2 = 0;

  /* "(tree fragment)":8
 *     __Pyx_CheckUnpickleChecksum(__pyx_checksum, 0x7925074, 0xdd80965, 0xbe76b6e, b'_allocator, _pools')
 *     __pyx_result = MemoryPool.__new__(__pyx_type)
 *     if __pyx_state is not None:             # <<<<<<<<<<<<<<
 *         __pyx_unpickle_MemoryPool__set_state(<MemoryPool> __pyx_result, __pyx_state)
 *     return __pyx_result
*/
  __pyx_t_5 = (__pyx_v___pyx_state != ((PyObject*)Py_None));
  if (__pyx_t_5) {

    /* "(tree fragment)":9
 *     __pyx_result = MemoryPool.__new__(__pyx_type)
 *     if __pyx_state is not None:
 *         __pyx_unpickle_MemoryPool__set_state(<MemoryPool> __pyx_result, __pyx_state)             # <<<<<<<<<<<<<<
 *     return __pyx_result
 * cdef __pyx_unpickle_MemoryPool__set_state(MemoryPool __pyx_result, __pyx_state: tuple):
*/
    if (unlikely(__pyx_v___pyx_state == Py_None)) {
      PyErr_SetString(PyExc_TypeError, "cannot pass None into a C function argument that is declared 'not None'");
      __PYX_ERR(3, 9, __pyx_L1_error)
    }
    __pyx_t_2 = __pyx_f_4cupy_4cuda_6memory___pyx_unpickle_MemoryPool__set_state(((struct __pyx_obj_4cupy_4cuda_6memory_MemoryPool *)__pyx_v___pyx_result), __pyx_v___pyx_state); if (unlikely(!__pyx_t_2)) __PYX_ERR(3, 9, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

    /* "(tree fragment)":8
 *     __Pyx_CheckUnpickleChecksum(__pyx_checksum, 0x7925074, 0xdd80965, 0xbe76b6e, b'_allocator, _pools')
 *     __pyx_result = MemoryPool.__new__(__pyx_type)
 *     if __pyx_state is not None:             # <<<<<<<<<<<<<<
 *         __pyx_unpickle_MemoryPool__set_state(<MemoryPool> __pyx_result, __pyx_state)
 *     return __pyx_result
*/
  }

  /* "(tree fragment)":10
 *     if __pyx_state is not None:
 *         __pyx_unpickle_MemoryPool__set_state(<MemoryPool> __pyx_result, __pyx_state)
 *     return __pyx_result             # <<<<<<<<<<<<<<
 * cdef __pyx_unpickle_MemoryPool__set_state(MemoryPool __pyx_result, __pyx_state: tuple):
 *     __pyx_result._allocator = __pyx_state[0]; __pyx_result._pools = __pyx_state[1]
*/
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_v___pyx_result);
  __pyx_r = __pyx_v___pyx_result;
  goto __pyx_L0;

  /* "(tree fragment)":4
 *     int __Pyx_CheckUnpickleChecksum(long, long, long, long, const char*) except -1
 *     int __Pyx_UpdateUnpickledDict(object, object, Py_ssize_t) except -1
 * def __pyx_unpickle_MemoryPool(__pyx_type, long __pyx_checksum, tuple __pyx_state):             # <<<<<<<<<<<<<<
 *     cdef object __pyx_result
 *     __Pyx_CheckUnpickleChecksum(__pyx_checksum, 0x7925074, 0xdd80965, 0xbe76b6e, b'_allocator, _pools')
*/

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_AddTraceback("cupy.cuda.memory.__pyx_unpickle_MemoryPool", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v___pyx_result);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":11
 *         __pyx_unpickle_MemoryPool__set_state(<MemoryPool> __pyx_result, __pyx_state)
 *     return __pyx_result
 * cdef __pyx_unpickle_MemoryPool__set_state(MemoryPool __pyx_result, __pyx_state: tuple):             # <<<<<<<<<<<<<<
 *     __pyx_result._allocator = __pyx_state[0]; __pyx_result._pools = __pyx_state[1]
 *     __Pyx_UpdateUnpickledDict(__pyx_result, __pyx_state, 2)
*/

static PyObject *__pyx_f_4cupy_4cuda_6memory___pyx_unpickle_MemoryPool__set_state(struct __pyx_obj_4cupy_4cuda_6memory_MemoryPool *__pyx_v___pyx_result, PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_t_2;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__pyx_unpickle_MemoryPool__set_state", 0);

  /* "(tree fragment)":12
 *     return __pyx_result
 * cdef __pyx_unpickle_MemoryPool__set_state(MemoryPool __pyx_result, __pyx_state: tuple):
 *     __pyx_result._allocator = __pyx_state[0]; __pyx_result._pools = __pyx_state[1]             # <<<<<<<<<<<<<<
 *     __Pyx_UpdateUnpickledDict(__pyx_result, __pyx_state, 2)
*/
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 0, long, 1, __Pyx_PyLong_From_long, 0, 0, 1, 1, __Pyx_ReferenceSharing_FunctionArgument); if (unlikely(!__pyx_t_1)) __PYX_ERR(3, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->_allocator);
  __Pyx_DECREF(__pyx_v___pyx_result->_allocator);
  __pyx_v___pyx_result->_allocator = __pyx_t_1;
  __pyx_t_1 = 0;
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 1, long, 1, __Pyx_PyLong_From_long, 0, 0, 1, 1, __Pyx_ReferenceSharing_FunctionArgument); if (unlikely(!__pyx_t_1)) __PYX_ERR(3, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (!(likely(PyTuple_CheckExact(__pyx_t_1))||((__pyx_t_1) == Py_None) || __Pyx_RaiseUnexpectedTypeError("tuple", __pyx_t_1))) __PYX_ERR(3, 12, __pyx_L1_error)
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->_pools);
  __Pyx_DECREF(__pyx_v___pyx_result->_pools);
  __pyx_v___pyx_result->_pools = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "(tree fragment)":13
 * cdef __pyx_unpickle_MemoryPool__set_state(MemoryPool __pyx_result, __pyx_state: tuple):
 *     __pyx_result._allocator = __pyx_state[0]; __pyx_result._pools = __pyx_state[1]
 *     __Pyx_UpdateUnpickledDict(__pyx_result, __pyx_state, 2)             # <<<<<<<<<<<<<<
*/
  __pyx_t_2 = __Pyx_UpdateUnpickledDict(((PyObject *)__pyx_v___pyx_result), __pyx_v___pyx_state, 2); if (unlikely(__pyx_t_2 == ((int)-1))) __PYX_ERR(3, 13, __pyx_L1_error)

  /* "(tree fragment)":11
 *         __pyx_unpickle_MemoryPool__set_state(<MemoryPool> __pyx_result, __pyx_state)
 *     return __pyx_result
 * cdef __pyx_unpickle_MemoryPool__set_state(MemoryPool __pyx_result, __pyx_state: tuple):             # <<<<<<<<<<<<<<
 *     __pyx_result._allocator = __pyx_state[0]; __pyx_result._pools = __pyx_state[1]
 *     __Pyx_UpdateUnpickledDict(__pyx_result, __pyx_state, 2)
*/

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("cupy.cuda.memory.__pyx_unpickle_MemoryPool__set_state", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":4
 *     int __Pyx_CheckUnpickleChecksum(long, long, long, long, const char*) except -1
 *     int __Pyx_UpdateUnpickledDict(object, object, Py_ssize_t) except -1
 * def __pyx_unpickle_MemoryAsyncPool(__pyx_type, long __pyx_checksum, tuple __pyx_state):             # <<<<<<<<<<<<<<
 *     cdef object __pyx_result
 *     __Pyx_CheckUnpickleChecksum(__pyx_checksum, 0xe511d52, 0x96a91a4, 0xc327f55, b'_pools, memoryAsyncHasStat')
*/

/* Python wrapper */
static PyObject *__pyx_pw_4cupy_4cuda_6memory_45__pyx_unpickle_MemoryAsyncPool(PyObject *__pyx_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
PyDoc_STRVAR(__pyx_doc_4cupy_4cuda_6memory_44__pyx_unpickle_MemoryAsyncPool, "__pyx_unpickle_MemoryAsyncPool(__pyx_type, long __pyx_checksum, tuple __pyx_state)");
static PyMethodDef __pyx_mdef_4cupy_4cuda_6memory_45__pyx_unpickle_MemoryAsyncPool = {"__pyx_unpickle_MemoryAsyncPool", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_4cupy_4cuda_6memory_45__pyx_unpickle_MemoryAsyncPool, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_4cupy_4cuda_6memory_44__pyx_unpickle_MemoryAsyncPool};
static PyObject *__pyx_pw_4cupy_4cuda_6memory_45__pyx_unpickle_MemoryAsyncPool(PyObject *__pyx_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
) {
  PyObject *__pyx_v___pyx_type = 0;
  long __pyx_v___pyx_checksum;
  PyObject *__pyx_v___pyx_state = 0;
  #if !CYTHON_METH_FASTCALL
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  #endif
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject* values[3] = {0,0,0};
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__pyx_unpickle_MemoryAsyncPool (wrapper)", 0);
  #if !CYTHON_METH_FASTCALL
  #if CYTHON_ASSUME_SAFE_SIZE
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return NULL;
  #endif
  #endif
  __pyx_kwvalues = __Pyx_KwValues_FASTCALL(__pyx_args, __pyx_nargs);
  {
    PyObject ** const __pyx_pyargnames[] = {&__pyx_mstate_global->__pyx_n_u_pyx_type,&__pyx_mstate_global->__pyx_n_u_pyx_checksum,&__pyx_mstate_global->__pyx_n_u_pyx_state,0};
    const Py_ssize_t __pyx_kwds_len = (__pyx_kwds) ? __Pyx_NumKwargs_FASTCALL(__pyx_kwds) : 0;
    if (unlikely(__pyx_kwds_len < 0)) __PYX_ERR(3, 4, __pyx_L3_error)
    if (__pyx_kwds_len > 0) {
      switch (__pyx_nargs) {
        case  3:
        values[2] = __Pyx_ArgRef_FASTCALL(__pyx_args, 2);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[2])) __PYX_ERR(3, 4, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  2:
        values[1] = __Pyx_ArgRef_FASTCALL(__pyx_args, 1);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[1])) __PYX_ERR(3, 4, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  1:
        values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(3, 4, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      const Py_ssize_t kwd_pos_args = __pyx_nargs;
      if (__Pyx_ParseKeywords(__pyx_kwds, __pyx_kwvalues, __pyx_pyargnames, 0, values, kwd_pos_args, __pyx_kwds_len, "__pyx_unpickle_MemoryAsyncPool", 0) < (0)) __PYX_ERR(3, 4, __pyx_L3_error)
      for (Py_ssize_t i = __pyx_nargs; i < 3; i++) {
        if (unlikely(!values[i])) { __Pyx_RaiseArgtupleInvalid("__pyx_unpickle_MemoryAsyncPool", 1, 3, 3, i); __PYX_ERR(3, 4, __pyx_L3_error) }
      }
    } else if (unlikely(__pyx_nargs != 3)) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
      if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(3, 4, __pyx_L3_error)
      values[1] = __Pyx_ArgRef_FASTCALL(__pyx_args, 1);
      if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[1])) __PYX_ERR(3, 4, __pyx_L3_error)
      values[2] = __Pyx_ArgRef_FASTCALL(__pyx_args, 2);
      if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[2])) __PYX_ERR(3, 4, __pyx_L3_error)
    }
    __pyx_v___pyx_type = values[0];
    __pyx_v___pyx_checksum = __Pyx_PyLong_As_long(values[1]); if (unlikely((__pyx_v___pyx_checksum == (long)-1) && PyErr_Occurred())) __PYX_ERR(3, 4, __pyx_L3_error)
    __pyx_v___pyx_state = ((PyObject*)values[2]);
  }
  goto __pyx_L6_skip;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("__pyx_unpickle_MemoryAsyncPool", 1, 3, 3, __pyx_nargs); __PYX_ERR(3, 4, __pyx_L3_error)
  __pyx_L6_skip:;
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_AddTraceback("cupy.cuda.memory.__pyx_unpickle_MemoryAsyncPool", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v___pyx_state), (&PyTuple_Type), 1, "__pyx_state", 1))) __PYX_ERR(3, 4, __pyx_L1_error)
  __pyx_r = __pyx_pf_4cupy_4cuda_6memory_44__pyx_unpickle_MemoryAsyncPool(__pyx_self, __pyx_v___pyx_type, __pyx_v___pyx_checksum, __pyx_v___pyx_state);

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = NULL;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  goto __pyx_L7_cleaned_up;
  __pyx_L0:;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __pyx_L7_cleaned_up:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4cupy_4cuda_6memory_44__pyx_unpickle_MemoryAsyncPool(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v___pyx_type, long __pyx_v___pyx_checksum, PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_v___pyx_result = 0;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  size_t __pyx_t_4;
  int __pyx_t_5;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__pyx_unpickle_MemoryAsyncPool", 0);

  /* "(tree fragment)":6
 * def __pyx_unpickle_MemoryAsyncPool(__pyx_type, long __pyx_checksum, tuple __pyx_state):
 *     cdef object __pyx_result
 *     __Pyx_CheckUnpickleChecksum(__pyx_checksum, 0xe511d52, 0x96a91a4, 0xc327f55, b'_pools, memoryAsyncHasStat')             # <<<<<<<<<<<<<<
 *     __pyx_result = MemoryAsyncPool.__new__(__pyx_type)
 *     if __pyx_state is not None:
*/
  __pyx_t_1 = __Pyx_CheckUnpickleChecksum(__pyx_v___pyx_checksum, 0xe511d52, 0x96a91a4, 0xc327f55, __pyx_k_pools_memoryAsyncHasStat); if (unlikely(__pyx_t_1 == ((int)-1))) __PYX_ERR(3, 6, __pyx_L1_error)

  /* "(tree fragment)":7
 *     cdef object __pyx_result
 *     __Pyx_CheckUnpickleChecksum(__pyx_checksum, 0xe511d52, 0x96a91a4, 0xc327f55, b'_pools, memoryAsyncHasStat')
 *     __pyx_result = MemoryAsyncPool.__new__(__pyx_type)             # <<<<<<<<<<<<<<
 *     if __pyx_state is not None:
 *         __pyx_unpickle_MemoryAsyncPool__set_state(<MemoryAsyncPool> __pyx_result, __pyx_state)
*/
  __pyx_t_3 = ((PyObject *)__pyx_mstate_global->__pyx_ptype_4cupy_4cuda_6memory_MemoryAsyncPool);
  __Pyx_INCREF(__pyx_t_3);
  __pyx_t_4 = 0;
  {
    PyObject *__pyx_callargs[2] = {__pyx_t_3, __pyx_v___pyx_type};
    __pyx_t_2 = __Pyx_PyObject_FastCallMethod((PyObject*)__pyx_mstate_global->__pyx_n_u_new, __pyx_callargs+__pyx_t_4, (2-__pyx_t_4) | (1*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
    __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
    if (unlikely(!__pyx_t_2)) __PYX_ERR(3, 7, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
  }
  __pyx_v___pyx_result = __pyx_t_2;
  __pyx_t_2 = 0;

  /* "(tree fragment)":8
 *     __Pyx_CheckUnpickleChecksum(__pyx_checksum, 0xe511d52, 0x96a91a4, 0xc327f55, b'_pools, memoryAsyncHasStat')
 *     __pyx_result = MemoryAsyncPool.__new__(__pyx_type)
 *     if __pyx_state is not None:             # <<<<<<<<<<<<<<
 *         __pyx_unpickle_MemoryAsyncPool__set_state(<MemoryAsyncPool> __pyx_result, __pyx_state)
 *     return __pyx_result
*/
  __pyx_t_5 = (__pyx_v___pyx_state != ((PyObject*)Py_None));
  if (__pyx_t_5) {

    /* "(tree fragment)":9
 *     __pyx_result = MemoryAsyncPool.__new__(__pyx_type)
 *     if __pyx_state is not None:
 *         __pyx_unpickle_MemoryAsyncPool__set_state(<MemoryAsyncPool> __pyx_result, __pyx_state)             # <<<<<<<<<<<<<<
 *     return __pyx_result
 * cdef __pyx_unpickle_MemoryAsyncPool__set_state(MemoryAsyncPool __pyx_result, __pyx_state: tuple):
*/
    if (unlikely(__pyx_v___pyx_state == Py_None)) {
      PyErr_SetString(PyExc_TypeError, "cannot pass None into a C function argument that is declared 'not None'");
      __PYX_ERR(3, 9, __pyx_L1_error)
    }
    __pyx_t_2 = __pyx_f_4cupy_4cuda_6memory___pyx_unpickle_MemoryAsyncPool__set_state(((struct __pyx_obj_4cupy_4cuda_6memory_MemoryAsyncPool *)__pyx_v___pyx_result), __pyx_v___pyx_state); if (unlikely(!__pyx_t_2)) __PYX_ERR(3, 9, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

    /* "(tree fragment)":8
 *     __Pyx_CheckUnpickleChecksum(__pyx_checksum, 0xe511d52, 0x96a91a4, 0xc327f55, b'_pools, memoryAsyncHasStat')
 *     __pyx_result = MemoryAsyncPool.__new__(__pyx_type)
 *     if __pyx_state is not None:             # <<<<<<<<<<<<<<
 *         __pyx_unpickle_MemoryAsyncPool__set_state(<MemoryAsyncPool> __pyx_result, __pyx_state)
 *     return __pyx_result
*/
  }

  /* "(tree fragment)":10
 *     if __pyx_state is not None:
 *         __pyx_unpickle_MemoryAsyncPool__set_state(<MemoryAsyncPool> __pyx_result, __pyx_state)
 *     return __pyx_result             # <<<<<<<<<<<<<<
 * cdef __pyx_unpickle_MemoryAsyncPool__set_state(MemoryAsyncPool __pyx_result, __pyx_state: tuple):
 *     __pyx_result._pools = __pyx_state[0]; __pyx_result.memoryAsyncHasStat = __pyx_state[1]
*/
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_v___pyx_result);
  __pyx_r = __pyx_v___pyx_result;
  goto __pyx_L0;

  /* "(tree fragment)":4
 *     int __Pyx_CheckUnpickleChecksum(long, long, long, long, const char*) except -1
 *     int __Pyx_UpdateUnpickledDict(object, object, Py_ssize_t) except -1
 * def __pyx_unpickle_MemoryAsyncPool(__pyx_type, long __pyx_checksum, tuple __pyx_state):             # <<<<<<<<<<<<<<
 *     cdef object __pyx_result
 *     __Pyx_CheckUnpickleChecksum(__pyx_checksum, 0xe511d52, 0x96a91a4, 0xc327f55, b'_pools, memoryAsyncHasStat')
*/

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_AddTraceback("cupy.cuda.memory.__pyx_unpickle_MemoryAsyncPool", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v___pyx_result);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":11
 *         __pyx_unpickle_MemoryAsyncPool__set_state(<MemoryAsyncPool> __pyx_result, __pyx_state)
 *     return __pyx_result
 * cdef __pyx_unpickle_MemoryAsyncPool__set_state(MemoryAsyncPool __pyx_result, __pyx_state: tuple):             # <<<<<<<<<<<<<<
 *     __pyx_result._pools = __pyx_state[0]; __pyx_result.memoryAsyncHasStat = __pyx_state[1]
 *     __Pyx_UpdateUnpickledDict(__pyx_result, __pyx_state, 2)
*/

static PyObject *__pyx_f_4cupy_4cuda_6memory___pyx_unpickle_MemoryAsyncPool__set_state(struct __pyx_obj_4cupy_4cuda_6memory_MemoryAsyncPool *__pyx_v___pyx_result, PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_t_2;
  int __pyx_t_3;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__pyx_unpickle_MemoryAsyncPool__set_state", 0);

  /* "(tree fragment)":12
 *     return __pyx_result
 * cdef __pyx_unpickle_MemoryAsyncPool__set_state(MemoryAsyncPool __pyx_result, __pyx_state: tuple):
 *     __pyx_result._pools = __pyx_state[0]; __pyx_result.memoryAsyncHasStat = __pyx_state[1]             # <<<<<<<<<<<<<<
 *     __Pyx_UpdateUnpickledDict(__pyx_result, __pyx_state, 2)
*/
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 0, long, 1, __Pyx_PyLong_From_long, 0, 0, 1, 1, __Pyx_ReferenceSharing_FunctionArgument); if (unlikely(!__pyx_t_1)) __PYX_ERR(3, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (!(likely(PyList_CheckExact(__pyx_t_1))||((__pyx_t_1) == Py_None) || __Pyx_RaiseUnexpectedTypeError("list", __pyx_t_1))) __PYX_ERR(3, 12, __pyx_L1_error)
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->_pools);
  __Pyx_DECREF(__pyx_v___pyx_result->_pools);
  __pyx_v___pyx_result->_pools = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 1, long, 1, __Pyx_PyLong_From_long, 0, 0, 1, 1, __Pyx_ReferenceSharing_FunctionArgument); if (unlikely(!__pyx_t_1)) __PYX_ERR(3, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyObject_IsTrue(__pyx_t_1); if (unlikely((__pyx_t_2 == (int)-1) && PyErr_Occurred())) __PYX_ERR(3, 12, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v___pyx_result->memoryAsyncHasStat = __pyx_t_2;

  /* "(tree fragment)":13
 * cdef __pyx_unpickle_MemoryAsyncPool__set_state(MemoryAsyncPool __pyx_result, __pyx_state: tuple):
 *     __pyx_result._pools = __pyx_state[0]; __pyx_result.memoryAsyncHasStat = __pyx_state[1]
 *     __Pyx_UpdateUnpickledDict(__pyx_result, __pyx_state, 2)             # <<<<<<<<<<<<<<
*/
  __pyx_t_3 = __Pyx_UpdateUnpickledDict(((PyObject *)__pyx_v___pyx_result), __pyx_v___pyx_state, 2); if (unlikely(__pyx_t_3 == ((int)-1))) __PYX_ERR(3, 13, __pyx_L1_error)

  /* "(tree fragment)":11
 *         __pyx_unpickle_MemoryAsyncPool__set_state(<MemoryAsyncPool> __pyx_result, __pyx_state)
 *     return __pyx_result
 * cdef __pyx_unpickle_MemoryAsyncPool__set_state(MemoryAsyncPool __pyx_result, __pyx_state: tuple):             # <<<<<<<<<<<<<<
 *     __pyx_result._pools = __pyx_state[0]; __pyx_result.memoryAsyncHasStat = __pyx_state[1]
 *     __Pyx_UpdateUnpickledDict(__pyx_result, __pyx_state, 2)
*/

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("cupy.cuda.memory.__pyx_unpickle_MemoryAsyncPool__set_state", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":4
 *     int __Pyx_CheckUnpickleChecksum(long, long, long, long, const char*) except -1
 *     int __Pyx_UpdateUnpickledDict(object, object, Py_ssize_t) except -1
 * def __pyx_unpickle_CFunctionAllocatorMemory(__pyx_type, long __pyx_checksum, tuple __pyx_state):             # <<<<<<<<<<<<<<
 *     cdef object __pyx_result
 *     __Pyx_CheckUnpickleChecksum(__pyx_checksum, 0xce95057, 0x22054d2, 0xfa3fd60, b'_free_func, _param, device_id, ptr, size')
*/

/* Python wrapper */
static PyObject *__pyx_pw_4cupy_4cuda_6memory_47__pyx_unpickle_CFunctionAllocatorMemory(PyObject *__pyx_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
PyDoc_STRVAR(__pyx_doc_4cupy_4cuda_6memory_46__pyx_unpickle_CFunctionAllocatorMemory, "__pyx_unpickle_CFunctionAllocatorMemory(__pyx_type, long __pyx_checksum, tuple __pyx_state)");
static PyMethodDef __pyx_mdef_4cupy_4cuda_6memory_47__pyx_unpickle_CFunctionAllocatorMemory = {"__pyx_unpickle_CFunctionAllocatorMemory", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_4cupy_4cuda_6memory_47__pyx_unpickle_CFunctionAllocatorMemory, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_4cupy_4cuda_6memory_46__pyx_unpickle_CFunctionAllocatorMemory};
static PyObject *__pyx_pw_4cupy_4cuda_6memory_47__pyx_unpickle_CFunctionAllocatorMemory(PyObject *__pyx_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
) {
  PyObject *__pyx_v___pyx_type = 0;
  long __pyx_v___pyx_checksum;
  PyObject *__pyx_v___pyx_state = 0;
  #if !CYTHON_METH_FASTCALL
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  #endif
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject* values[3] = {0,0,0};
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__pyx_unpickle_CFunctionAllocatorMemory (wrapper)", 0);
  #if !CYTHON_METH_FASTCALL
  #if CYTHON_ASSUME_SAFE_SIZE
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return NULL;
  #endif
  #endif
  __pyx_kwvalues = __Pyx_KwValues_FASTCALL(__pyx_args, __pyx_nargs);
  {
    PyObject ** const __pyx_pyargnames[] = {&__pyx_mstate_global->__pyx_n_u_pyx_type,&__pyx_mstate_global->__pyx_n_u_pyx_checksum,&__pyx_mstate_global->__pyx_n_u_pyx_state,0};
    const Py_ssize_t __pyx_kwds_len = (__pyx_kwds) ? __Pyx_NumKwargs_FASTCALL(__pyx_kwds) : 0;
    if (unlikely(__pyx_kwds_len < 0)) __PYX_ERR(3, 4, __pyx_L3_error)
    if (__pyx_kwds_len > 0) {
      switch (__pyx_nargs) {
        case  3:
        values[2] = __Pyx_ArgRef_FASTCALL(__pyx_args, 2);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[2])) __PYX_ERR(3, 4, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  2:
        values[1] = __Pyx_ArgRef_FASTCALL(__pyx_args, 1);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[1])) __PYX_ERR(3, 4, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  1:
        values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(3, 4, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      const Py_ssize_t kwd_pos_args = __pyx_nargs;
      if (__Pyx_ParseKeywords(__pyx_kwds, __pyx_kwvalues, __pyx_pyargnames, 0, values, kwd_pos_args, __pyx_kwds_len, "__pyx_unpickle_CFunctionAllocatorMemory", 0) < (0)) __PYX_ERR(3, 4, __pyx_L3_error)
      for (Py_ssize_t i = __pyx_nargs; i < 3; i++) {
        if (unlikely(!values[i])) { __Pyx_RaiseArgtupleInvalid("__pyx_unpickle_CFunctionAllocatorMemory", 1, 3, 3, i); __PYX_ERR(3, 4, __pyx_L3_error) }
      }
    } else if (unlikely(__pyx_nargs != 3)) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
      if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(3, 4, __pyx_L3_error)
      values[1] = __Pyx_ArgRef_FASTCALL(__pyx_args, 1);
      if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[1])) __PYX_ERR(3, 4, __pyx_L3_error)
      values[2] = __Pyx_ArgRef_FASTCALL(__pyx_args, 2);
      if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[2])) __PYX_ERR(3, 4, __pyx_L3_error)
    }
    __pyx_v___pyx_type = values[0];
    __pyx_v___pyx_checksum = __Pyx_PyLong_As_long(values[1]); if (unlikely((__pyx_v___pyx_checksum == (long)-1) && PyErr_Occurred())) __PYX_ERR(3, 4, __pyx_L3_error)
    __pyx_v___pyx_state = ((PyObject*)values[2]);
  }
  goto __pyx_L6_skip;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("__pyx_unpickle_CFunctionAllocatorMemory", 1, 3, 3, __pyx_nargs); __PYX_ERR(3, 4, __pyx_L3_error)
  __pyx_L6_skip:;
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_AddTraceback("cupy.cuda.memory.__pyx_unpickle_CFunctionAllocatorMemory", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v___pyx_state), (&PyTuple_Type), 1, "__pyx_state", 1))) __PYX_ERR(3, 4, __pyx_L1_error)
  __pyx_r = __pyx_pf_4cupy_4cuda_6memory_46__pyx_unpickle_CFunctionAllocatorMemory(__pyx_self, __pyx_v___pyx_type, __pyx_v___pyx_checksum, __pyx_v___pyx_state);

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = NULL;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  goto __pyx_L7_cleaned_up;
  __pyx_L0:;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __pyx_L7_cleaned_up:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4cupy_4cuda_6memory_46__pyx_unpickle_CFunctionAllocatorMemory(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v___pyx_type, long __pyx_v___pyx_checksum, PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_v___pyx_result = 0;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  size_t __pyx_t_4;
  int __pyx_t_5;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__pyx_unpickle_CFunctionAllocatorMemory", 0);

  /* "(tree fragment)":6
 * def __pyx_unpickle_CFunctionAllocatorMemory(__pyx_type, long __pyx_checksum, tuple __pyx_state):
 *     cdef object __pyx_result
 *     __Pyx_CheckUnpickleChecksum(__pyx_checksum, 0xce95057, 0x22054d2, 0xfa3fd60, b'_free_func, _param, device_id, ptr, size')             # <<<<<<<<<<<<<<
 *     __pyx_result = CFunctionAllocatorMemory.__new__(__pyx_type)
 *     if __pyx_state is not None:
*/
  __pyx_t_1 = __Pyx_CheckUnpickleChecksum(__pyx_v___pyx_checksum, 0xce95057, 0x22054d2, 0xfa3fd60, __pyx_k_free_func__param_device_id_ptr); if (unlikely(__pyx_t_1 == ((int)-1))) __PYX_ERR(3, 6, __pyx_L1_error)

  /* "(tree fragment)":7
 *     cdef object __pyx_result
 *     __Pyx_CheckUnpickleChecksum(__pyx_checksum, 0xce95057, 0x22054d2, 0xfa3fd60, b'_free_func, _param, device_id, ptr, size')
 *     __pyx_result = CFunctionAllocatorMemory.__new__(__pyx_type)             # <<<<<<<<<<<<<<
 *     if __pyx_state is not None:
 *         __pyx_unpickle_CFunctionAllocatorMemory__set_state(<CFunctionAllocatorMemory> __pyx_result, __pyx_state)
*/
  __pyx_t_3 = ((PyObject *)__pyx_mstate_global->__pyx_ptype_4cupy_4cuda_6memory_CFunctionAllocatorMemory);
  __Pyx_INCREF(__pyx_t_3);
  __pyx_t_4 = 0;
  {
    PyObject *__pyx_callargs[2] = {__pyx_t_3, __pyx_v___pyx_type};
    __pyx_t_2 = __Pyx_PyObject_FastCallMethod((PyObject*)__pyx_mstate_global->__pyx_n_u_new, __pyx_callargs+__pyx_t_4, (2-__pyx_t_4) | (1*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
    __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
    if (unlikely(!__pyx_t_2)) __PYX_ERR(3, 7, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
  }
  __pyx_v___pyx_result = __pyx_t_2;
  __pyx_t_2 = 0;

  /* "(tree fragment)":8
 *     __Pyx_CheckUnpickleChecksum(__pyx_checksum, 0xce95057, 0x22054d2, 0xfa3fd60, b'_free_func, _param, device_id, ptr, size')
 *     __pyx_result = CFunctionAllocatorMemory.__new__(__pyx_type)
 *     if __pyx_state is not None:             # <<<<<<<<<<<<<<
 *         __pyx_unpickle_CFunctionAllocatorMemory__set_state(<CFunctionAllocatorMemory> __pyx_result, __pyx_state)
 *     return __pyx_result
*/
  __pyx_t_5 = (__pyx_v___pyx_state != ((PyObject*)Py_None));
  if (__pyx_t_5) {

    /* "(tree fragment)":9
 *     __pyx_result = CFunctionAllocatorMemory.__new__(__pyx_type)
 *     if __pyx_state is not None:
 *         __pyx_unpickle_CFunctionAllocatorMemory__set_state(<CFunctionAllocatorMemory> __pyx_result, __pyx_state)             # <<<<<<<<<<<<<<
 *     return __pyx_result
 * cdef __pyx_unpickle_CFunctionAllocatorMemory__set_state(CFunctionAllocatorMemory __pyx_result, __pyx_state: tuple):
*/
    if (unlikely(__pyx_v___pyx_state == Py_None)) {
      PyErr_SetString(PyExc_TypeError, "cannot pass None into a C function argument that is declared 'not None'");
      __PYX_ERR(3, 9, __pyx_L1_error)
    }
    __pyx_t_2 = __pyx_f_4cupy_4cuda_6memory___pyx_unpickle_CFunctionAllocatorMemory__set_state(((struct __pyx_obj_4cupy_4cuda_6memory_CFunctionAllocatorMemory *)__pyx_v___pyx_result), __pyx_v___pyx_state); if (unlikely(!__pyx_t_2)) __PYX_ERR(3, 9, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

    /* "(tree fragment)":8
 *     __Pyx_CheckUnpickleChecksum(__pyx_checksum, 0xce95057, 0x22054d2, 0xfa3fd60, b'_free_func, _param, device_id, ptr, size')
 *     __pyx_result = CFunctionAllocatorMemory.__new__(__pyx_type)
 *     if __pyx_state is not None:             # <<<<<<<<<<<<<<
 *         __pyx_unpickle_CFunctionAllocatorMemory__set_state(<CFunctionAllocatorMemory> __pyx_result, __pyx_state)
 *     return __pyx_result
*/
  }

  /* "(tree fragment)":10
 *     if __pyx_state is not None:
 *         __pyx_unpickle_CFunctionAllocatorMemory__set_state(<CFunctionAllocatorMemory> __pyx_result, __pyx_state)
 *     return __pyx_result             # <<<<<<<<<<<<<<
 * cdef __pyx_unpickle_CFunctionAllocatorMemory__set_state(CFunctionAllocatorMemory __pyx_result, __pyx_state: tuple):
 *     __pyx_result._free_func = __pyx_state[0]; __pyx_result._param = __pyx_state[1]; __pyx_result.device_id = __pyx_state[2]; __pyx_result.ptr = __pyx_state[3]; __pyx_result.size = __pyx_state[4]
*/
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_v___pyx_result);
  __pyx_r = __pyx_v___pyx_result;
  goto __pyx_L0;

  /* "(tree fragment)":4
 *     int __Pyx_CheckUnpickleChecksum(long, long, long, long, const char*) except -1
 *     int __Pyx_UpdateUnpickledDict(object, object, Py_ssize_t) except -1
 * def __pyx_unpickle_CFunctionAllocatorMemory(__pyx_type, long __pyx_checksum, tuple __pyx_state):             # <<<<<<<<<<<<<<
 *     cdef object __pyx_result
 *     __Pyx_CheckUnpickleChecksum(__pyx_checksum, 0xce95057, 0x22054d2, 0xfa3fd60, b'_free_func, _param, device_id, ptr, size')
*/

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_AddTraceback("cupy.cuda.memory.__pyx_unpickle_CFunctionAllocatorMemory", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v___pyx_result);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":11
 *         __pyx_unpickle_CFunctionAllocatorMemory__set_state(<CFunctionAllocatorMemory> __pyx_result, __pyx_state)
 *     return __pyx_result
 * cdef __pyx_unpickle_CFunctionAllocatorMemory__set_state(CFunctionAllocatorMemory __pyx_result, __pyx_state: tuple):             # <<<<<<<<<<<<<<
 *     __pyx_result._free_func = __pyx_state[0]; __pyx_result._param = __pyx_state[1]; __pyx_result.device_id = __pyx_state[2]; __pyx_result.ptr = __pyx_state[3]; __pyx_result.size = __pyx_state[4]
 *     __Pyx_UpdateUnpickledDict(__pyx_result, __pyx_state, 5)
*/

static PyObject *__pyx_f_4cupy_4cuda_6memory___pyx_unpickle_CFunctionAllocatorMemory__set_state(struct __pyx_obj_4cupy_4cuda_6memory_CFunctionAllocatorMemory *__pyx_v___pyx_result, PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  intptr_t __pyx_t_2;
  int __pyx_t_3;
  size_t __pyx_t_4;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__pyx_unpickle_CFunctionAllocatorMemory__set_state", 0);

  /* "(tree fragment)":12
 *     return __pyx_result
 * cdef __pyx_unpickle_CFunctionAllocatorMemory__set_state(CFunctionAllocatorMemory __pyx_result, __pyx_state: tuple):
 *     __pyx_result._free_func = __pyx_state[0]; __pyx_result._param = __pyx_state[1]; __pyx_result.device_id = __pyx_state[2]; __pyx_result.ptr = __pyx_state[3]; __pyx_result.size = __pyx_state[4]             # <<<<<<<<<<<<<<
 *     __Pyx_UpdateUnpickledDict(__pyx_result, __pyx_state, 5)
*/
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 0, long, 1, __Pyx_PyLong_From_long, 0, 0, 1, 1, __Pyx_ReferenceSharing_FunctionArgument); if (unlikely(!__pyx_t_1)) __PYX_ERR(3, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = PyLong_AsSsize_t(__pyx_t_1); if (unlikely((__pyx_t_2 == ((intptr_t)-1)) && PyErr_Occurred())) __PYX_ERR(3, 12, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v___pyx_result->_free_func = __pyx_t_2;
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 1, long, 1, __Pyx_PyLong_From_long, 0, 0, 1, 1, __Pyx_ReferenceSharing_FunctionArgument); if (unlikely(!__pyx_t_1)) __PYX_ERR(3, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = PyLong_AsSsize_t(__pyx_t_1); if (unlikely((__pyx_t_2 == ((intptr_t)-1)) && PyErr_Occurred())) __PYX_ERR(3, 12, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v___pyx_result->_param = __pyx_t_2;
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 2, long, 1, __Pyx_PyLong_From_long, 0, 0, 1, 1, __Pyx_ReferenceSharing_FunctionArgument); if (unlikely(!__pyx_t_1)) __PYX_ERR(3, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_3 = __Pyx_PyLong_As_int(__pyx_t_1); if (unlikely((__pyx_t_3 == (int)-1) && PyErr_Occurred())) __PYX_ERR(3, 12, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v___pyx_result->__pyx_base.device_id = __pyx_t_3;
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 3, long, 1, __Pyx_PyLong_From_long, 0, 0, 1, 1, __Pyx_ReferenceSharing_FunctionArgument); if (unlikely(!__pyx_t_1)) __PYX_ERR(3, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = PyLong_AsSsize_t(__pyx_t_1); if (unlikely((__pyx_t_2 == ((intptr_t)-1)) && PyErr_Occurred())) __PYX_ERR(3, 12, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v___pyx_result->__pyx_base.ptr = __pyx_t_2;
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 4, long, 1, __Pyx_PyLong_From_long, 0, 0, 1, 1, __Pyx_ReferenceSharing_FunctionArgument); if (unlikely(!__pyx_t_1)) __PYX_ERR(3, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_4 = __Pyx_PyLong_As_size_t(__pyx_t_1); if (unlikely((__pyx_t_4 == (size_t)-1) && PyErr_Occurred())) __PYX_ERR(3, 12, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v___pyx_result->__pyx_base.size = __pyx_t_4;

  /* "(tree fragment)":13
 * cdef __pyx_unpickle_CFunctionAllocatorMemory__set_state(CFunctionAllocatorMemory __pyx_result, __pyx_state: tuple):
 *     __pyx_result._free_func = __pyx_state[0]; __pyx_result._param = __pyx_state[1]; __pyx_result.device_id = __pyx_state[2]; __pyx_result.ptr = __pyx_state[3]; __pyx_result.size = __pyx_state[4]
 *     __Pyx_UpdateUnpickledDict(__pyx_result, __pyx_state, 5)             # <<<<<<<<<<<<<<
*/
  __pyx_t_3 = __Pyx_UpdateUnpickledDict(((PyObject *)__pyx_v___pyx_result), __pyx_v___pyx_state, 5); if (unlikely(__pyx_t_3 == ((int)-1))) __PYX_ERR(3, 13, __pyx_L1_error)

  /* "(tree fragment)":11
 *         __pyx_unpickle_CFunctionAllocatorMemory__set_state(<CFunctionAllocatorMemory> __pyx_result, __pyx_state)
 *     return __pyx_result
 * cdef __pyx_unpickle_CFunctionAllocatorMemory__set_state(CFunctionAllocatorMemory __pyx_result, __pyx_state: tuple):             # <<<<<<<<<<<<<<
 *     __pyx_result._free_func = __pyx_state[0]; __pyx_result._param = __pyx_state[1]; __pyx_result.device_id = __pyx_state[2]; __pyx_result.ptr = __pyx_state[3]; __pyx_result.size = __pyx_state[4]
 *     __Pyx_UpdateUnpickledDict(__pyx_result, __pyx_state, 5)
*/

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("cupy.cuda.memory.__pyx_unpickle_CFunctionAllocatorMemory__set_state", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":4
 *     int __Pyx_CheckUnpickleChecksum(long, long, long, long, const char*) except -1
 *     int __Pyx_UpdateUnpickledDict(object, object, Py_ssize_t) except -1
 * def __pyx_unpickle_CFunctionAllocator(__pyx_type, long __pyx_checksum, tuple __pyx_state):             # <<<<<<<<<<<<<<
 *     cdef object __pyx_result
 *     __Pyx_CheckUnpickleChecksum(__pyx_checksum, 0x54c1fda, 0xc66c958, 0xced2061, b'_free_func, _malloc_func, _owner, _param')
*/

/* Python wrapper */
static PyObject *__pyx_pw_4cupy_4cuda_6memory_49__pyx_unpickle_CFunctionAllocator(PyObject *__pyx_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
PyDoc_STRVAR(__pyx_doc_4cupy_4cuda_6memory_48__pyx_unpickle_CFunctionAllocator, "__pyx_unpickle_CFunctionAllocator(__pyx_type, long __pyx_checksum, tuple __pyx_state)");
static PyMethodDef __pyx_mdef_4cupy_4cuda_6memory_49__pyx_unpickle_CFunctionAllocator = {"__pyx_unpickle_CFunctionAllocator", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_4cupy_4cuda_6memory_49__pyx_unpickle_CFunctionAllocator, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_4cupy_4cuda_6memory_48__pyx_unpickle_CFunctionAllocator};
static PyObject *__pyx_pw_4cupy_4cuda_6memory_49__pyx_unpickle_CFunctionAllocator(PyObject *__pyx_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
) {
  PyObject *__pyx_v___pyx_type = 0;
  long __pyx_v___pyx_checksum;
  PyObject *__pyx_v___pyx_state = 0;
  #if !CYTHON_METH_FASTCALL
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  #endif
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject* values[3] = {0,0,0};
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__pyx_unpickle_CFunctionAllocator (wrapper)", 0);
  #if !CYTHON_METH_FASTCALL
  #if CYTHON_ASSUME_SAFE_SIZE
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return NULL;
  #endif
  #endif
  __pyx_kwvalues = __Pyx_KwValues_FASTCALL(__pyx_args, __pyx_nargs);
  {
    PyObject ** const __pyx_pyargnames[] = {&__pyx_mstate_global->__pyx_n_u_pyx_type,&__pyx_mstate_global->__pyx_n_u_pyx_checksum,&__pyx_mstate_global->__pyx_n_u_pyx_state,0};
    const Py_ssize_t __pyx_kwds_len = (__pyx_kwds) ? __Pyx_NumKwargs_FASTCALL(__pyx_kwds) : 0;
    if (unlikely(__pyx_kwds_len < 0)) __PYX_ERR(3, 4, __pyx_L3_error)
    if (__pyx_kwds_len > 0) {
      switch (__pyx_nargs) {
        case  3:
        values[2] = __Pyx_ArgRef_FASTCALL(__pyx_args, 2);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[2])) __PYX_ERR(3, 4, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  2:
        values[1] = __Pyx_ArgRef_FASTCALL(__pyx_args, 1);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[1])) __PYX_ERR(3, 4, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  1:
        values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(3, 4, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      const Py_ssize_t kwd_pos_args = __pyx_nargs;
      if (__Pyx_ParseKeywords(__pyx_kwds, __pyx_kwvalues, __pyx_pyargnames, 0, values, kwd_pos_args, __pyx_kwds_len, "__pyx_unpickle_CFunctionAllocator", 0) < (0)) __PYX_ERR(3, 4, __pyx_L3_error)
      for (Py_ssize_t i = __pyx_nargs; i < 3; i++) {
        if (unlikely(!values[i])) { __Pyx_RaiseArgtupleInvalid("__pyx_unpickle_CFunctionAllocator", 1, 3, 3, i); __PYX_ERR(3, 4, __pyx_L3_error) }
      }
    } else if (unlikely(__pyx_nargs != 3)) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
      if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(3, 4, __pyx_L3_error)
      values[1] = __Pyx_ArgRef_FASTCALL(__pyx_args, 1);
      if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[1])) __PYX_ERR(3, 4, __pyx_L3_error)
      values[2] = __Pyx_ArgRef_FASTCALL(__pyx_args, 2);
      if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[2])) __PYX_ERR(3, 4, __pyx_L3_error)
    }
    __pyx_v___pyx_type = values[0];
    __pyx_v___pyx_checksum = __Pyx_PyLong_As_long(values[1]); if (unlikely((__pyx_v___pyx_checksum == (long)-1) && PyErr_Occurred())) __PYX_ERR(3, 4, __pyx_L3_error)
    __pyx_v___pyx_state = ((PyObject*)values[2]);
  }
  goto __pyx_L6_skip;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("__pyx_unpickle_CFunctionAllocator", 1, 3, 3, __pyx_nargs); __PYX_ERR(3, 4, __pyx_L3_error)
  __pyx_L6_skip:;
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_AddTraceback("cupy.cuda.memory.__pyx_unpickle_CFunctionAllocator", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v___pyx_state), (&PyTuple_Type), 1, "__pyx_state", 1))) __PYX_ERR(3, 4, __pyx_L1_error)
  __pyx_r = __pyx_pf_4cupy_4cuda_6memory_48__pyx_unpickle_CFunctionAllocator(__pyx_self, __pyx_v___pyx_type, __pyx_v___pyx_checksum, __pyx_v___pyx_state);

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = NULL;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  goto __pyx_L7_cleaned_up;
  __pyx_L0:;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __pyx_L7_cleaned_up:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4cupy_4cuda_6memory_48__pyx_unpickle_CFunctionAllocator(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v___pyx_type, long __pyx_v___pyx_checksum, PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_v___pyx_result = 0;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  size_t __pyx_t_4;
  int __pyx_t_5;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__pyx_unpickle_CFunctionAllocator", 0);

  /* "(tree fragment)":6
 * def __pyx_unpickle_CFunctionAllocator(__pyx_type, long __pyx_checksum, tuple __pyx_state):
 *     cdef object __pyx_result
 *     __Pyx_CheckUnpickleChecksum(__pyx_checksum, 0x54c1fda, 0xc66c958, 0xced2061, b'_free_func, _malloc_func, _owner, _param')             # <<<<<<<<<<<<<<
 *     __pyx_result = CFunctionAllocator.__new__(__pyx_type)
 *     if __pyx_state is not None:
*/
  __pyx_t_1 = __Pyx_CheckUnpickleChecksum(__pyx_v___pyx_checksum, 0x54c1fda, 0xc66c958, 0xced2061, __pyx_k_free_func__malloc_func__owner); if (unlikely(__pyx_t_1 == ((int)-1))) __PYX_ERR(3, 6, __pyx_L1_error)

  /* "(tree fragment)":7
 *     cdef object __pyx_result
 *     __Pyx_CheckUnpickleChecksum(__pyx_checksum, 0x54c1fda, 0xc66c958, 0xced2061, b'_free_func, _malloc_func, _owner, _param')
 *     __pyx_result = CFunctionAllocator.__new__(__pyx_type)             # <<<<<<<<<<<<<<
 *     if __pyx_state is not None:
 *         __pyx_unpickle_CFunctionAllocator__set_state(<CFunctionAllocator> __pyx_result, __pyx_state)
*/
  __pyx_t_3 = ((PyObject *)__pyx_mstate_global->__pyx_ptype_4cupy_4cuda_6memory_CFunctionAllocator);
  __Pyx_INCREF(__pyx_t_3);
  __pyx_t_4 = 0;
  {
    PyObject *__pyx_callargs[2] = {__pyx_t_3, __pyx_v___pyx_type};
    __pyx_t_2 = __Pyx_PyObject_FastCallMethod((PyObject*)__pyx_mstate_global->__pyx_n_u_new, __pyx_callargs+__pyx_t_4, (2-__pyx_t_4) | (1*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
    __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
    if (unlikely(!__pyx_t_2)) __PYX_ERR(3, 7, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
  }
  __pyx_v___pyx_result = __pyx_t_2;
  __pyx_t_2 = 0;

  /* "(tree fragment)":8
 *     __Pyx_CheckUnpickleChecksum(__pyx_checksum, 0x54c1fda, 0xc66c958, 0xced2061, b'_free_func, _malloc_func, _owner, _param')
 *     __pyx_result = CFunctionAllocator.__new__(__pyx_type)
 *     if __pyx_state is not None:             # <<<<<<<<<<<<<<
 *         __pyx_unpickle_CFunctionAllocator__set_state(<CFunctionAllocator> __pyx_result, __pyx_state)
 *     return __pyx_result
*/
  __pyx_t_5 = (__pyx_v___pyx_state != ((PyObject*)Py_None));
  if (__pyx_t_5) {

    /* "(tree fragment)":9
 *     __pyx_result = CFunctionAllocator.__new__(__pyx_type)
 *     if __pyx_state is not None:
 *         __pyx_unpickle_CFunctionAllocator__set_state(<CFunctionAllocator> __pyx_result, __pyx_state)             # <<<<<<<<<<<<<<
 *     return __pyx_result
 * cdef __pyx_unpickle_CFunctionAllocator__set_state(CFunctionAllocator __pyx_result, __pyx_state: tuple):
*/
    if (unlikely(__pyx_v___pyx_state == Py_None)) {
      PyErr_SetString(PyExc_TypeError, "cannot pass None into a C function argument that is declared 'not None'");
      __PYX_ERR(3, 9, __pyx_L1_error)
    }
    __pyx_t_2 = __pyx_f_4cupy_4cuda_6memory___pyx_unpickle_CFunctionAllocator__set_state(((struct __pyx_obj_4cupy_4cuda_6memory_CFunctionAllocator *)__pyx_v___pyx_result), __pyx_v___pyx_state); if (unlikely(!__pyx_t_2)) __PYX_ERR(3, 9, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

    /* "(tree fragment)":8
 *     __Pyx_CheckUnpickleChecksum(__pyx_checksum, 0x54c1fda, 0xc66c958, 0xced2061, b'_free_func, _malloc_func, _owner, _param')
 *     __pyx_result = CFunctionAllocator.__new__(__pyx_type)
 *     if __pyx_state is not None:             # <<<<<<<<<<<<<<
 *         __pyx_unpickle_CFunctionAllocator__set_state(<CFunctionAllocator> __pyx_result, __pyx_state)
 *     return __pyx_result
*/
  }

  /* "(tree fragment)":10
 *     if __pyx_state is not None:
 *         __pyx_unpickle_CFunctionAllocator__set_state(<CFunctionAllocator> __pyx_result, __pyx_state)
 *     return __pyx_result             # <<<<<<<<<<<<<<
 * cdef __pyx_unpickle_CFunctionAllocator__set_state(CFunctionAllocator __pyx_result, __pyx_state: tuple):
 *     __pyx_result._free_func = __pyx_state[0]; __pyx_result._malloc_func = __pyx_state[1]; __pyx_result._owner = __pyx_state[2]; __pyx_result._param = __pyx_state[3]
*/
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_v___pyx_result);
  __pyx_r = __pyx_v___pyx_result;
  goto __pyx_L0;

  /* "(tree fragment)":4
 *     int __Pyx_CheckUnpickleChecksum(long, long, long, long, const char*) except -1
 *     int __Pyx_UpdateUnpickledDict(object, object, Py_ssize_t) except -1
 * def __pyx_unpickle_CFunctionAllocator(__pyx_type, long __pyx_checksum, tuple __pyx_state):             # <<<<<<<<<<<<<<
 *     cdef object __pyx_result
 *     __Pyx_CheckUnpickleChecksum(__pyx_checksum, 0x54c1fda, 0xc66c958, 0xced2061, b'_free_func, _malloc_func, _owner, _param')
*/

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_AddTraceback("cupy.cuda.memory.__pyx_unpickle_CFunctionAllocator", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v___pyx_result);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":11
 *         __pyx_unpickle_CFunctionAllocator__set_state(<CFunctionAllocator> __pyx_result, __pyx_state)
 *     return __pyx_result
 * cdef __pyx_unpickle_CFunctionAllocator__set_state(CFunctionAllocator __pyx_result, __pyx_state: tuple):             # <<<<<<<<<<<<<<
 *     __pyx_result._free_func = __pyx_state[0]; __pyx_result._malloc_func = __pyx_state[1]; __pyx_result._owner = __pyx_state[2]; __pyx_result._param = __pyx_state[3]
 *     __Pyx_UpdateUnpickledDict(__pyx_result, __pyx_state, 4)
*/

static PyObject *__pyx_f_4cupy_4cuda_6memory___pyx_unpickle_CFunctionAllocator__set_state(struct __pyx_obj_4cupy_4cuda_6memory_CFunctionAllocator *__pyx_v___pyx_result, PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  intptr_t __pyx_t_2;
  int __pyx_t_3;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__pyx_unpickle_CFunctionAllocator__set_state", 0);

  /* "(tree fragment)":12
 *     return __pyx_result
 * cdef __pyx_unpickle_CFunctionAllocator__set_state(CFunctionAllocator __pyx_result, __pyx_state: tuple):
 *     __pyx_result._free_func = __pyx_state[0]; __pyx_result._malloc_func = __pyx_state[1]; __pyx_result._owner = __pyx_state[2]; __pyx_result._param = __pyx_state[3]             # <<<<<<<<<<<<<<
 *     __Pyx_UpdateUnpickledDict(__pyx_result, __pyx_state, 4)
*/
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 0, long, 1, __Pyx_PyLong_From_long, 0, 0, 1, 1, __Pyx_ReferenceSharing_FunctionArgument); if (unlikely(!__pyx_t_1)) __PYX_ERR(3, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = PyLong_AsSsize_t(__pyx_t_1); if (unlikely((__pyx_t_2 == ((intptr_t)-1)) && PyErr_Occurred())) __PYX_ERR(3, 12, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v___pyx_result->_free_func = __pyx_t_2;
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 1, long, 1, __Pyx_PyLong_From_long, 0, 0, 1, 1, __Pyx_ReferenceSharing_FunctionArgument); if (unlikely(!__pyx_t_1)) __PYX_ERR(3, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = PyLong_AsSsize_t(__pyx_t_1); if (unlikely((__pyx_t_2 == ((intptr_t)-1)) && PyErr_Occurred())) __PYX_ERR(3, 12, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v___pyx_result->_malloc_func = __pyx_t_2;
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 2, long, 1, __Pyx_PyLong_From_long, 0, 0, 1, 1, __Pyx_ReferenceSharing_FunctionArgument); if (unlikely(!__pyx_t_1)) __PYX_ERR(3, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->_owner);
  __Pyx_DECREF(__pyx_v___pyx_result->_owner);
  __pyx_v___pyx_result->_owner = __pyx_t_1;
  __pyx_t_1 = 0;
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 3, long, 1, __Pyx_PyLong_From_long, 0, 0, 1, 1, __Pyx_ReferenceSharing_FunctionArgument); if (unlikely(!__pyx_t_1)) __PYX_ERR(3, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = PyLong_AsSsize_t(__pyx_t_1); if (unlikely((__pyx_t_2 == ((intptr_t)-1)) && PyErr_Occurred())) __PYX_ERR(3, 12, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v___pyx_result->_param = __pyx_t_2;

  /* "(tree fragment)":13
 * cdef __pyx_unpickle_CFunctionAllocator__set_state(CFunctionAllocator __pyx_result, __pyx_state: tuple):
 *     __pyx_result._free_func = __pyx_state[0]; __pyx_result._malloc_func = __pyx_state[1]; __pyx_result._owner = __pyx_state[2]; __pyx_result._param = __pyx_state[3]
 *     __Pyx_UpdateUnpickledDict(__pyx_result, __pyx_state, 4)             # <<<<<<<<<<<<<<
*/
  __pyx_t_3 = __Pyx_UpdateUnpickledDict(((PyObject *)__pyx_v___pyx_result), __pyx_v___pyx_state, 4); if (unlikely(__pyx_t_3 == ((int)-1))) __PYX_ERR(3, 13, __pyx_L1_error)

  /* "(tree fragment)":11
 *         __pyx_unpickle_CFunctionAllocator__set_state(<CFunctionAllocator> __pyx_result, __pyx_state)
 *     return __pyx_result
 * cdef __pyx_unpickle_CFunctionAllocator__set_state(CFunctionAllocator __pyx_result, __pyx_state: tuple):             # <<<<<<<<<<<<<<
 *     __pyx_result._free_func = __pyx_state[0]; __pyx_result._malloc_func = __pyx_state[1]; __pyx_result._owner = __pyx_state[2]; __pyx_result._param = __pyx_state[3]
 *     __Pyx_UpdateUnpickledDict(__pyx_result, __pyx_state, 4)
*/

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("cupy.cuda.memory.__pyx_unpickle_CFunctionAllocator__set_state", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":4
 *     int __Pyx_CheckUnpickleChecksum(long, long, long, long, const char*) except -1
 *     int __Pyx_UpdateUnpickledDict(object, object, Py_ssize_t) except -1
 * def __pyx_unpickle_PythonFunctionAllocatorMemory(__pyx_type, long __pyx_checksum, tuple __pyx_state):             # <<<<<<<<<<<<<<
 *     cdef object __pyx_result
 *     __Pyx_CheckUnpickleChecksum(__pyx_checksum, 0xf33b7cb, 0xd854a2e, 0x4f1a007, b'_free_func, device_id, ptr, size')
*/

/* Python wrapper */
static PyObject *__pyx_pw_4cupy_4cuda_6memory_51__pyx_unpickle_PythonFunctionAllocatorMemory(PyObject *__pyx_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
PyDoc_STRVAR(__pyx_doc_4cupy_4cuda_6memory_50__pyx_unpickle_PythonFunctionAllocatorMemory, "__pyx_unpickle_PythonFunctionAllocatorMemory(__pyx_type, long __pyx_checksum, tuple __pyx_state)");
static PyMethodDef __pyx_mdef_4cupy_4cuda_6memory_51__pyx_unpickle_PythonFunctionAllocatorMemory = {"__pyx_unpickle_PythonFunctionAllocatorMemory", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_4cupy_4cuda_6memory_51__pyx_unpickle_PythonFunctionAllocatorMemory, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_4cupy_4cuda_6memory_50__pyx_unpickle_PythonFunctionAllocatorMemory};
static PyObject *__pyx_pw_4cupy_4cuda_6memory_51__pyx_unpickle_PythonFunctionAllocatorMemory(PyObject *__pyx_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
) {
  PyObject *__pyx_v___pyx_type = 0;
  long __pyx_v___pyx_checksum;
  PyObject *__pyx_v___pyx_state = 0;
  #if !CYTHON_METH_FASTCALL
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  #endif
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject* values[3] = {0,0,0};
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__pyx_unpickle_PythonFunctionAllocatorMemory (wrapper)", 0);
  #if !CYTHON_METH_FASTCALL
  #if CYTHON_ASSUME_SAFE_SIZE
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return NULL;
  #endif
  #endif
  __pyx_kwvalues = __Pyx_KwValues_FASTCALL(__pyx_args, __pyx_nargs);
  {
    PyObject ** const __pyx_pyargnames[] = {&__pyx_mstate_global->__pyx_n_u_pyx_type,&__pyx_mstate_global->__pyx_n_u_pyx_checksum,&__pyx_mstate_global->__pyx_n_u_pyx_state,0};
    const Py_ssize_t __pyx_kwds_len = (__pyx_kwds) ? __Pyx_NumKwargs_FASTCALL(__pyx_kwds) : 0;
    if (unlikely(__pyx_kwds_len < 0)) __PYX_ERR(3, 4, __pyx_L3_error)
    if (__pyx_kwds_len > 0) {
      switch (__pyx_nargs) {
        case  3:
        values[2] = __Pyx_ArgRef_FASTCALL(__pyx_args, 2);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[2])) __PYX_ERR(3, 4, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  2:
        values[1] = __Pyx_ArgRef_FASTCALL(__pyx_args, 1);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[1])) __PYX_ERR(3, 4, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  1:
        values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(3, 4, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      const Py_ssize_t kwd_pos_args = __pyx_nargs;
      if (__Pyx_ParseKeywords(__pyx_kwds, __pyx_kwvalues, __pyx_pyargnames, 0, values, kwd_pos_args, __pyx_kwds_len, "__pyx_unpickle_PythonFunctionAllocatorMemory", 0) < (0)) __PYX_ERR(3, 4, __pyx_L3_error)
      for (Py_ssize_t i = __pyx_nargs; i < 3; i++) {
        if (unlikely(!values[i])) { __Pyx_RaiseArgtupleInvalid("__pyx_unpickle_PythonFunctionAllocatorMemory", 1, 3, 3, i); __PYX_ERR(3, 4, __pyx_L3_error) }
      }
    } else if (unlikely(__pyx_nargs != 3)) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
      if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(3, 4, __pyx_L3_error)
      values[1] = __Pyx_ArgRef_FASTCALL(__pyx_args, 1);
      if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[1])) __PYX_ERR(3, 4, __pyx_L3_error)
      values[2] = __Pyx_ArgRef_FASTCALL(__pyx_args, 2);
      if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[2])) __PYX_ERR(3, 4, __pyx_L3_error)
    }
    __pyx_v___pyx_type = values[0];
    __pyx_v___pyx_checksum = __Pyx_PyLong_As_long(values[1]); if (unlikely((__pyx_v___pyx_checksum == (long)-1) && PyErr_Occurred())) __PYX_ERR(3, 4, __pyx_L3_error)
    __pyx_v___pyx_state = ((PyObject*)values[2]);
  }
  goto __pyx_L6_skip;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("__pyx_unpickle_PythonFunctionAllocatorMemory", 1, 3, 3, __pyx_nargs); __PYX_ERR(3, 4, __pyx_L3_error)
  __pyx_L6_skip:;
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_AddTraceback("cupy.cuda.memory.__pyx_unpickle_PythonFunctionAllocatorMemory", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v___pyx_state), (&PyTuple_Type), 1, "__pyx_state", 1))) __PYX_ERR(3, 4, __pyx_L1_error)
  __pyx_r = __pyx_pf_4cupy_4cuda_6memory_50__pyx_unpickle_PythonFunctionAllocatorMemory(__pyx_self, __pyx_v___pyx_type, __pyx_v___pyx_checksum, __pyx_v___pyx_state);

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = NULL;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  goto __pyx_L7_cleaned_up;
  __pyx_L0:;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __pyx_L7_cleaned_up:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4cupy_4cuda_6memory_50__pyx_unpickle_PythonFunctionAllocatorMemory(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v___pyx_type, long __pyx_v___pyx_checksum, PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_v___pyx_result = 0;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  size_t __pyx_t_4;
  int __pyx_t_5;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__pyx_unpickle_PythonFunctionAllocatorMemory", 0);

  /* "(tree fragment)":6
 * def __pyx_unpickle_PythonFunctionAllocatorMemory(__pyx_type, long __pyx_checksum, tuple __pyx_state):
 *     cdef object __pyx_result
 *     __Pyx_CheckUnpickleChecksum(__pyx_checksum, 0xf33b7cb, 0xd854a2e, 0x4f1a007, b'_free_func, device_id, ptr, size')             # <<<<<<<<<<<<<<
 *     __pyx_result = PythonFunctionAllocatorMemory.__new__(__pyx_type)
 *     if __pyx_state is not None:
*/
  __pyx_t_1 = __Pyx_CheckUnpickleChecksum(__pyx_v___pyx_checksum, 0xf33b7cb, 0xd854a2e, 0x4f1a007, __pyx_k_free_func_device_id_ptr_size); if (unlikely(__pyx_t_1 == ((int)-1))) __PYX_ERR(3, 6, __pyx_L1_error)

  /* "(tree fragment)":7
 *     cdef object __pyx_result
 *     __Pyx_CheckUnpickleChecksum(__pyx_checksum, 0xf33b7cb, 0xd854a2e, 0x4f1a007, b'_free_func, device_id, ptr, size')
 *     __pyx_result = PythonFunctionAllocatorMemory.__new__(__pyx_type)             # <<<<<<<<<<<<<<
 *     if __pyx_state is not None:
 *         __pyx_unpickle_PythonFunctionAllocatorMemory__set_state(<PythonFunctionAllocatorMemory> __pyx_result, __pyx_state)
*/
  __pyx_t_3 = ((PyObject *)__pyx_mstate_global->__pyx_ptype_4cupy_4cuda_6memory_PythonFunctionAllocatorMemory);
  __Pyx_INCREF(__pyx_t_3);
  __pyx_t_4 = 0;
  {
    PyObject *__pyx_callargs[2] = {__pyx_t_3, __pyx_v___pyx_type};
    __pyx_t_2 = __Pyx_PyObject_FastCallMethod((PyObject*)__pyx_mstate_global->__pyx_n_u_new, __pyx_callargs+__pyx_t_4, (2-__pyx_t_4) | (1*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
    __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
    if (unlikely(!__pyx_t_2)) __PYX_ERR(3, 7, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
  }
  __pyx_v___pyx_result = __pyx_t_2;
  __pyx_t_2 = 0;

  /* "(tree fragment)":8
 *     __Pyx_CheckUnpickleChecksum(__pyx_checksum, 0xf33b7cb, 0xd854a2e, 0x4f1a007, b'_free_func, device_id, ptr, size')
 *     __pyx_result = PythonFunctionAllocatorMemory.__new__(__pyx_type)
 *     if __pyx_state is not None:             # <<<<<<<<<<<<<<
 *         __pyx_unpickle_PythonFunctionAllocatorMemory__set_state(<PythonFunctionAllocatorMemory> __pyx_result, __pyx_state)
 *     return __pyx_result
*/
  __pyx_t_5 = (__pyx_v___pyx_state != ((PyObject*)Py_None));
  if (__pyx_t_5) {

    /* "(tree fragment)":9
 *     __pyx_result = PythonFunctionAllocatorMemory.__new__(__pyx_type)
 *     if __pyx_state is not None:
 *         __pyx_unpickle_PythonFunctionAllocatorMemory__set_state(<PythonFunctionAllocatorMemory> __pyx_result, __pyx_state)             # <<<<<<<<<<<<<<
 *     return __pyx_result
 * cdef __pyx_unpickle_PythonFunctionAllocatorMemory__set_state(PythonFunctionAllocatorMemory __pyx_result, __pyx_state: tuple):
*/
    if (unlikely(__pyx_v___pyx_state == Py_None)) {
      PyErr_SetString(PyExc_TypeError, "cannot pass None into a C function argument that is declared 'not None'");
      __PYX_ERR(3, 9, __pyx_L1_error)
    }
    __pyx_t_2 = __pyx_f_4cupy_4cuda_6memory___pyx_unpickle_PythonFunctionAllocatorMemory__set_state(((struct __pyx_obj_4cupy_4cuda_6memory_PythonFunctionAllocatorMemory *)__pyx_v___pyx_result), __pyx_v___pyx_state); if (unlikely(!__pyx_t_2)) __PYX_ERR(3, 9, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

    /* "(tree fragment)":8
 *     __Pyx_CheckUnpickleChecksum(__pyx_checksum, 0xf33b7cb, 0xd854a2e, 0x4f1a007, b'_free_func, device_id, ptr, size')
 *     __pyx_result = PythonFunctionAllocatorMemory.__new__(__pyx_type)
 *     if __pyx_state is not None:             # <<<<<<<<<<<<<<
 *         __pyx_unpickle_PythonFunctionAllocatorMemory__set_state(<PythonFunctionAllocatorMemory> __pyx_result, __pyx_state)
 *     return __pyx_result
*/
  }

  /* "(tree fragment)":10
 *     if __pyx_state is not None:
 *         __pyx_unpickle_PythonFunctionAllocatorMemory__set_state(<PythonFunctionAllocatorMemory> __pyx_result, __pyx_state)
 *     return __pyx_result             # <<<<<<<<<<<<<<
 * cdef __pyx_unpickle_PythonFunctionAllocatorMemory__set_state(PythonFunctionAllocatorMemory __pyx_result, __pyx_state: tuple):
 *     __pyx_result._free_func = __pyx_state[0]; __pyx_result.device_id = __pyx_state[1]; __pyx_result.ptr = __pyx_state[2]; __pyx_result.size = __pyx_state[3]
*/
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_v___pyx_result);
  __pyx_r = __pyx_v___pyx_result;
  goto __pyx_L0;

  /* "(tree fragment)":4
 *     int __Pyx_CheckUnpickleChecksum(long, long, long, long, const char*) except -1
 *     int __Pyx_UpdateUnpickledDict(object, object, Py_ssize_t) except -1
 * def __pyx_unpickle_PythonFunctionAllocatorMemory(__pyx_type, long __pyx_checksum, tuple __pyx_state):             # <<<<<<<<<<<<<<
 *     cdef object __pyx_result
 *     __Pyx_CheckUnpickleChecksum(__pyx_checksum, 0xf33b7cb, 0xd854a2e, 0x4f1a007, b'_free_func, device_id, ptr, size')
*/

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_AddTraceback("cupy.cuda.memory.__pyx_unpickle_PythonFunctionAllocatorMemory", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v___pyx_result);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":11
 *         __pyx_unpickle_PythonFunctionAllocatorMemory__set_state(<PythonFunctionAllocatorMemory> __pyx_result, __pyx_state)
 *     return __pyx_result
 * cdef __pyx_unpickle_PythonFunctionAllocatorMemory__set_state(PythonFunctionAllocatorMemory __pyx_result, __pyx_state: tuple):             # <<<<<<<<<<<<<<
 *     __pyx_result._free_func = __pyx_state[0]; __pyx_result.device_id = __pyx_state[1]; __pyx_result.ptr = __pyx_state[2]; __pyx_result.size = __pyx_state[3]
 *     __Pyx_UpdateUnpickledDict(__pyx_result, __pyx_state, 4)
*/

static PyObject *__pyx_f_4cupy_4cuda_6memory___pyx_unpickle_PythonFunctionAllocatorMemory__set_state(struct __pyx_obj_4cupy_4cuda_6memory_PythonFunctionAllocatorMemory *__pyx_v___pyx_result, PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_t_2;
  intptr_t __pyx_t_3;
  size_t __pyx_t_4;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__pyx_unpickle_PythonFunctionAllocatorMemory__set_state", 0);

  /* "(tree fragment)":12
 *     return __pyx_result
 * cdef __pyx_unpickle_PythonFunctionAllocatorMemory__set_state(PythonFunctionAllocatorMemory __pyx_result, __pyx_state: tuple):
 *     __pyx_result._free_func = __pyx_state[0]; __pyx_result.device_id = __pyx_state[1]; __pyx_result.ptr = __pyx_state[2]; __pyx_result.size = __pyx_state[3]             # <<<<<<<<<<<<<<
 *     __Pyx_UpdateUnpickledDict(__pyx_result, __pyx_state, 4)
*/
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 0, long, 1, __Pyx_PyLong_From_long, 0, 0, 1, 1, __Pyx_ReferenceSharing_FunctionArgument); if (unlikely(!__pyx_t_1)) __PYX_ERR(3, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->_free_func);
  __Pyx_DECREF(__pyx_v___pyx_result->_free_func);
  __pyx_v___pyx_result->_free_func = __pyx_t_1;
  __pyx_t_1 = 0;
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 1, long, 1, __Pyx_PyLong_From_long, 0, 0, 1, 1, __Pyx_ReferenceSharing_FunctionArgument); if (unlikely(!__pyx_t_1)) __PYX_ERR(3, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyLong_As_int(__pyx_t_1); if (unlikely((__pyx_t_2 == (int)-1) && PyErr_Occurred())) __PYX_ERR(3, 12, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v___pyx_result->__pyx_base.device_id = __pyx_t_2;
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 2, long, 1, __Pyx_PyLong_From_long, 0, 0, 1, 1, __Pyx_ReferenceSharing_FunctionArgument); if (unlikely(!__pyx_t_1)) __PYX_ERR(3, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_3 = PyLong_AsSsize_t(__pyx_t_1); if (unlikely((__pyx_t_3 == ((intptr_t)-1)) && PyErr_Occurred())) __PYX_ERR(3, 12, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v___pyx_result->__pyx_base.ptr = __pyx_t_3;
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 3, long, 1, __Pyx_PyLong_From_long, 0, 0, 1, 1, __Pyx_ReferenceSharing_FunctionArgument); if (unlikely(!__pyx_t_1)) __PYX_ERR(3, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_4 = __Pyx_PyLong_As_size_t(__pyx_t_1); if (unlikely((__pyx_t_4 == (size_t)-1) && PyErr_Occurred())) __PYX_ERR(3, 12, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v___pyx_result->__pyx_base.size = __pyx_t_4;

  /* "(tree fragment)":13
 * cdef __pyx_unpickle_PythonFunctionAllocatorMemory__set_state(PythonFunctionAllocatorMemory __pyx_result, __pyx_state: tuple):
 *     __pyx_result._free_func = __pyx_state[0]; __pyx_result.device_id = __pyx_state[1]; __pyx_result.ptr = __pyx_state[2]; __pyx_result.size = __pyx_state[3]
 *     __Pyx_UpdateUnpickledDict(__pyx_result, __pyx_state, 4)             # <<<<<<<<<<<<<<
*/
  __pyx_t_2 = __Pyx_UpdateUnpickledDict(((PyObject *)__pyx_v___pyx_result), __pyx_v___pyx_state, 4); if (unlikely(__pyx_t_2 == ((int)-1))) __PYX_ERR(3, 13, __pyx_L1_error)

  /* "(tree fragment)":11
 *         __pyx_unpickle_PythonFunctionAllocatorMemory__set_state(<PythonFunctionAllocatorMemory> __pyx_result, __pyx_state)
 *     return __pyx_result
 * cdef __pyx_unpickle_PythonFunctionAllocatorMemory__set_state(PythonFunctionAllocatorMemory __pyx_result, __pyx_state: tuple):             # <<<<<<<<<<<<<<
 *     __pyx_result._free_func = __pyx_state[0]; __pyx_result.device_id = __pyx_state[1]; __pyx_result.ptr = __pyx_state[2]; __pyx_result.size = __pyx_state[3]
 *     __Pyx_UpdateUnpickledDict(__pyx_result, __pyx_state, 4)
*/

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("cupy.cuda.memory.__pyx_unpickle_PythonFunctionAllocatorMemory__set_state", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":4
 *     int __Pyx_CheckUnpickleChecksum(long, long, long, long, const char*) except -1
 *     int __Pyx_UpdateUnpickledDict(object, object, Py_ssize_t) except -1
 * def __pyx_unpickle_PythonFunctionAllocator(__pyx_type, long __pyx_checksum, tuple __pyx_state):             # <<<<<<<<<<<<<<
 *     cdef object __pyx_result
 *     __Pyx_CheckUnpickleChecksum(__pyx_checksum, 0xda2aa50, 0xb921491, 0x048ae08, b'_free_func, _malloc_func')
*/

/* Python wrapper */
static PyObject *__pyx_pw_4cupy_4cuda_6memory_53__pyx_unpickle_PythonFunctionAllocator(PyObject *__pyx_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
PyDoc_STRVAR(__pyx_doc_4cupy_4cuda_6memory_52__pyx_unpickle_PythonFunctionAllocator, "__pyx_unpickle_PythonFunctionAllocator(__pyx_type, long __pyx_checksum, tuple __pyx_state)");
static PyMethodDef __pyx_mdef_4cupy_4cuda_6memory_53__pyx_unpickle_PythonFunctionAllocator = {"__pyx_unpickle_PythonFunctionAllocator", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_4cupy_4cuda_6memory_53__pyx_unpickle_PythonFunctionAllocator, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_4cupy_4cuda_6memory_52__pyx_unpickle_PythonFunctionAllocator};
static PyObject *__pyx_pw_4cupy_4cuda_6memory_53__pyx_unpickle_PythonFunctionAllocator(PyObject *__pyx_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
) {
  PyObject *__pyx_v___pyx_type = 0;
  long __pyx_v___pyx_checksum;
  PyObject *__pyx_v___pyx_state = 0;
  #if !CYTHON_METH_FASTCALL
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  #endif
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject* values[3] = {0,0,0};
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__pyx_unpickle_PythonFunctionAllocator (wrapper)", 0);
  #if !CYTHON_METH_FASTCALL
  #if CYTHON_ASSUME_SAFE_SIZE
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return NULL;
  #endif
  #endif
  __pyx_kwvalues = __Pyx_KwValues_FASTCALL(__pyx_args, __pyx_nargs);
  {
    PyObject ** const __pyx_pyargnames[] = {&__pyx_mstate_global->__pyx_n_u_pyx_type,&__pyx_mstate_global->__pyx_n_u_pyx_checksum,&__pyx_mstate_global->__pyx_n_u_pyx_state,0};
    const Py_ssize_t __pyx_kwds_len = (__pyx_kwds) ? __Pyx_NumKwargs_FASTCALL(__pyx_kwds) : 0;
    if (unlikely(__pyx_kwds_len < 0)) __PYX_ERR(3, 4, __pyx_L3_error)
    if (__pyx_kwds_len > 0) {
      switch (__pyx_nargs) {
        case  3:
        values[2] = __Pyx_ArgRef_FASTCALL(__pyx_args, 2);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[2])) __PYX_ERR(3, 4, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  2:
        values[1] = __Pyx_ArgRef_FASTCALL(__pyx_args, 1);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[1])) __PYX_ERR(3, 4, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  1:
        values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(3, 4, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      const Py_ssize_t kwd_pos_args = __pyx_nargs;
      if (__Pyx_ParseKeywords(__pyx_kwds, __pyx_kwvalues, __pyx_pyargnames, 0, values, kwd_pos_args, __pyx_kwds_len, "__pyx_unpickle_PythonFunctionAllocator", 0) < (0)) __PYX_ERR(3, 4, __pyx_L3_error)
      for (Py_ssize_t i = __pyx_nargs; i < 3; i++) {
        if (unlikely(!values[i])) { __Pyx_RaiseArgtupleInvalid("__pyx_unpickle_PythonFunctionAllocator", 1, 3, 3, i); __PYX_ERR(3, 4, __pyx_L3_error) }
      }
    } else if (unlikely(__pyx_nargs != 3)) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
      if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(3, 4, __pyx_L3_error)
      values[1] = __Pyx_ArgRef_FASTCALL(__pyx_args, 1);
      if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[1])) __PYX_ERR(3, 4, __pyx_L3_error)
      values[2] = __Pyx_ArgRef_FASTCALL(__pyx_args, 2);
      if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[2])) __PYX_ERR(3, 4, __pyx_L3_error)
    }
    __pyx_v___pyx_type = values[0];
    __pyx_v___pyx_checksum = __Pyx_PyLong_As_long(values[1]); if (unlikely((__pyx_v___pyx_checksum == (long)-1) && PyErr_Occurred())) __PYX_ERR(3, 4, __pyx_L3_error)
    __pyx_v___pyx_state = ((PyObject*)values[2]);
  }
  goto __pyx_L6_skip;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("__pyx_unpickle_PythonFunctionAllocator", 1, 3, 3, __pyx_nargs); __PYX_ERR(3, 4, __pyx_L3_error)
  __pyx_L6_skip:;
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_AddTraceback("cupy.cuda.memory.__pyx_unpickle_PythonFunctionAllocator", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v___pyx_state), (&PyTuple_Type), 1, "__pyx_state", 1))) __PYX_ERR(3, 4, __pyx_L1_error)
  __pyx_r = __pyx_pf_4cupy_4cuda_6memory_52__pyx_unpickle_PythonFunctionAllocator(__pyx_self, __pyx_v___pyx_type, __pyx_v___pyx_checksum, __pyx_v___pyx_state);

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = NULL;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  goto __pyx_L7_cleaned_up;
  __pyx_L0:;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __pyx_L7_cleaned_up:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4cupy_4cuda_6memory_52__pyx_unpickle_PythonFunctionAllocator(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v___pyx_type, long __pyx_v___pyx_checksum, PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_v___pyx_result = 0;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  size_t __pyx_t_4;
  int __pyx_t_5;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__pyx_unpickle_PythonFunctionAllocator", 0);

  /* "(tree fragment)":6
 * def __pyx_unpickle_PythonFunctionAllocator(__pyx_type, long __pyx_checksum, tuple __pyx_state):
 *     cdef object __pyx_result
 *     __Pyx_CheckUnpickleChecksum(__pyx_checksum, 0xda2aa50, 0xb921491, 0x048ae08, b'_free_func, _malloc_func')             # <<<<<<<<<<<<<<
 *     __pyx_result = PythonFunctionAllocator.__new__(__pyx_type)
 *     if __pyx_state is not None:
*/
  __pyx_t_1 = __Pyx_CheckUnpickleChecksum(__pyx_v___pyx_checksum, 0xda2aa50, 0xb921491, 0x048ae08, __pyx_k_free_func__malloc_func); if (unlikely(__pyx_t_1 == ((int)-1))) __PYX_ERR(3, 6, __pyx_L1_error)

  /* "(tree fragment)":7
 *     cdef object __pyx_result
 *     __Pyx_CheckUnpickleChecksum(__pyx_checksum, 0xda2aa50, 0xb921491, 0x048ae08, b'_free_func, _malloc_func')
 *     __pyx_result = PythonFunctionAllocator.__new__(__pyx_type)             # <<<<<<<<<<<<<<
 *     if __pyx_state is not None:
 *         __pyx_unpickle_PythonFunctionAllocator__set_state(<PythonFunctionAllocator> __pyx_result, __pyx_state)
*/
  __pyx_t_3 = ((PyObject *)__pyx_mstate_global->__pyx_ptype_4cupy_4cuda_6memory_PythonFunctionAllocator);
  __Pyx_INCREF(__pyx_t_3);
  __pyx_t_4 = 0;
  {
    PyObject *__pyx_callargs[2] = {__pyx_t_3, __pyx_v___pyx_type};
    __pyx_t_2 = __Pyx_PyObject_FastCallMethod((PyObject*)__pyx_mstate_global->__pyx_n_u_new, __pyx_callargs+__pyx_t_4, (2-__pyx_t_4) | (1*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
    __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
    if (unlikely(!__pyx_t_2)) __PYX_ERR(3, 7, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
  }
  __pyx_v___pyx_result = __pyx_t_2;
  __pyx_t_2 = 0;

  /* "(tree fragment)":8
 *     __Pyx_CheckUnpickleChecksum(__pyx_checksum, 0xda2aa50, 0xb921491, 0x048ae08, b'_free_func, _malloc_func')
 *     __pyx_result = PythonFunctionAllocator.__new__(__pyx_type)
 *     if __pyx_state is not None:             # <<<<<<<<<<<<<<
 *         __pyx_unpickle_PythonFunctionAllocator__set_state(<PythonFunctionAllocator> __pyx_result, __pyx_state)
 *     return __pyx_result
*/
  __pyx_t_5 = (__pyx_v___pyx_state != ((PyObject*)Py_None));
  if (__pyx_t_5) {

    /* "(tree fragment)":9
 *     __pyx_result = PythonFunctionAllocator.__new__(__pyx_type)
 *     if __pyx_state is not None:
 *         __pyx_unpickle_PythonFunctionAllocator__set_state(<PythonFunctionAllocator> __pyx_result, __pyx_state)             # <<<<<<<<<<<<<<
 *     return __pyx_result
 * cdef __pyx_unpickle_PythonFunctionAllocator__set_state(PythonFunctionAllocator __pyx_result, __pyx_state: tuple):
*/
    if (unlikely(__pyx_v___pyx_state == Py_None)) {
      PyErr_SetString(PyExc_TypeError, "cannot pass None into a C function argument that is declared 'not None'");
      __PYX_ERR(3, 9, __pyx_L1_error)
    }
    __pyx_t_2 = __pyx_f_4cupy_4cuda_6memory___pyx_unpickle_PythonFunctionAllocator__set_state(((struct __pyx_obj_4cupy_4cuda_6memory_PythonFunctionAllocator *)__pyx_v___pyx_result), __pyx_v___pyx_state); if (unlikely(!__pyx_t_2)) __PYX_ERR(3, 9, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

    /* "(tree fragment)":8
 *     __Pyx_CheckUnpickleChecksum(__pyx_checksum, 0xda2aa50, 0xb921491, 0x048ae08, b'_free_func, _malloc_func')
 *     __pyx_result = PythonFunctionAllocator.__new__(__pyx_type)
 *     if __pyx_state is not None:             # <<<<<<<<<<<<<<
 *         __pyx_unpickle_PythonFunctionAllocator__set_state(<PythonFunctionAllocator> __pyx_result, __pyx_state)
 *     return __pyx_result
*/
  }

  /* "(tree fragment)":10
 *     if __pyx_state is not None:
 *         __pyx_unpickle_PythonFunctionAllocator__set_state(<PythonFunctionAllocator> __pyx_result, __pyx_state)
 *     return __pyx_result             # <<<<<<<<<<<<<<
 * cdef __pyx_unpickle_PythonFunctionAllocator__set_state(PythonFunctionAllocator __pyx_result, __pyx_state: tuple):
 *     __pyx_result._free_func = __pyx_state[0]; __pyx_result._malloc_func = __pyx_state[1]
*/
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_v___pyx_result);
  __pyx_r = __pyx_v___pyx_result;
  goto __pyx_L0;

  /* "(tree fragment)":4
 *     int __Pyx_CheckUnpickleChecksum(long, long, long, long, const char*) except -1
 *     int __Pyx_UpdateUnpickledDict(object, object, Py_ssize_t) except -1
 * def __pyx_unpickle_PythonFunctionAllocator(__pyx_type, long __pyx_checksum, tuple __pyx_state):             # <<<<<<<<<<<<<<
 *     cdef object __pyx_result
 *     __Pyx_CheckUnpickleChecksum(__pyx_checksum, 0xda2aa50, 0xb921491, 0x048ae08, b'_free_func, _malloc_func')
*/

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_AddTraceback("cupy.cuda.memory.__pyx_unpickle_PythonFunctionAllocator", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v___pyx_result);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":11
 *         __pyx_unpickle_PythonFunctionAllocator__set_state(<PythonFunctionAllocator> __pyx_result, __pyx_state)
 *     return __pyx_result
 * cdef __pyx_unpickle_PythonFunctionAllocator__set_state(PythonFunctionAllocator __pyx_result, __pyx_state: tuple):             # <<<<<<<<<<<<<<
 *     __pyx_result._free_func = __pyx_state[0]; __pyx_result._malloc_func = __pyx_state[1]
 *     __Pyx_UpdateUnpickledDict(__pyx_result, __pyx_state, 2)
*/

static PyObject *__pyx_f_4cupy_4cuda_6memory___pyx_unpickle_PythonFunctionAllocator__set_state(struct __pyx_obj_4cupy_4cuda_6memory_PythonFunctionAllocator *__pyx_v___pyx_result, PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_t_2;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__pyx_unpickle_PythonFunctionAllocator__set_state", 0);

  /* "(tree fragment)":12
 *     return __pyx_result
 * cdef __pyx_unpickle_PythonFunctionAllocator__set_state(PythonFunctionAllocator __pyx_result, __pyx_state: tuple):
 *     __pyx_result._free_func = __pyx_state[0]; __pyx_result._malloc_func = __pyx_state[1]             # <<<<<<<<<<<<<<
 *     __Pyx_UpdateUnpickledDict(__pyx_result, __pyx_state, 2)
*/
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 0, long, 1, __Pyx_PyLong_From_long, 0, 0, 1, 1, __Pyx_ReferenceSharing_FunctionArgument); if (unlikely(!__pyx_t_1)) __PYX_ERR(3, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->_free_func);
  __Pyx_DECREF(__pyx_v___pyx_result->_free_func);
  __pyx_v___pyx_result->_free_func = __pyx_t_1;
  __pyx_t_1 = 0;
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 1, long, 1, __Pyx_PyLong_From_long, 0, 0, 1, 1, __Pyx_ReferenceSharing_FunctionArgument); if (unlikely(!__pyx_t_1)) __PYX_ERR(3, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->_malloc_func);
  __Pyx_DECREF(__pyx_v___pyx_result->_malloc_func);
  __pyx_v___pyx_result->_malloc_func = __pyx_t_1;
  __pyx_t_1 = 0;

  /* "(tree fragment)":13
 * cdef __pyx_unpickle_PythonFunctionAllocator__set_state(PythonFunctionAllocator __pyx_result, __pyx_state: tuple):
 *     __pyx_result._free_func = __pyx_state[0]; __pyx_result._malloc_func = __pyx_state[1]
 *     __Pyx_UpdateUnpickledDict(__pyx_result, __pyx_state, 2)             # <<<<<<<<<<<<<<
*/
  __pyx_t_2 = __Pyx_UpdateUnpickledDict(((PyObject *)__pyx_v___pyx_result), __pyx_v___pyx_state, 2); if (unlikely(__pyx_t_2 == ((int)-1))) __PYX_ERR(3, 13, __pyx_L1_error)

  /* "(tree fragment)":11
 *         __pyx_unpickle_PythonFunctionAllocator__set_state(<PythonFunctionAllocator> __pyx_result, __pyx_state)
 *     return __pyx_result
 * cdef __pyx_unpickle_PythonFunctionAllocator__set_state(PythonFunctionAllocator __pyx_result, __pyx_state: tuple):             # <<<<<<<<<<<<<<
 *     __pyx_result._free_func = __pyx_state[0]; __pyx_result._malloc_func = __pyx_state[1]
 *     __Pyx_UpdateUnpickledDict(__pyx_result, __pyx_state, 2)
*/

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("cupy.cuda.memory.__pyx_unpickle_PythonFunctionAllocator__set_state", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}
/* #### Code section: module_exttypes ### */

static PyObject *__pyx_tp_new_4cupy_4cuda_6memory_BaseMemory(PyTypeObject *t, CYTHON_UNUSED PyObject *a, CYTHON_UNUSED PyObject *k) {
  PyObject *o;
  o = __Pyx_AllocateExtensionType(t, 0);
  if (unlikely(!o)) return 0;
  return o;
}

static void __pyx_tp_dealloc_4cupy_4cuda_6memory_BaseMemory(PyObject *o) {
  #if CYTHON_USE_TP_FINALIZE
  if (unlikely(__Pyx_PyObject_GetSlot(o, tp_finalize, destructor)) && (!PyType_IS_GC(Py_TYPE(o)) || !__Pyx_PyObject_GC_IsFinalized(o))) {
    if (__Pyx_PyObject_GetSlot(o, tp_dealloc, destructor) == __pyx_tp_dealloc_4cupy_4cuda_6memory_BaseMemory) {
      if (PyObject_CallFinalizerFromDealloc(o)) return;
    }
  }
  #endif
  PyTypeObject *tp = Py_TYPE(o);
  #if CYTHON_USE_TYPE_SLOTS
  (*tp->tp_free)(o);
  #else
  {
    freefunc tp_free = (freefunc)PyType_GetSlot(tp, Py_tp_free);
    if (tp_free) tp_free(o);
  }
  #endif
  #if CYTHON_USE_TYPE_SPECS
  Py_DECREF(tp);
  #endif
}

static PyObject *__pyx_getprop_4cupy_4cuda_6memory_10BaseMemory_device(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_4cupy_4cuda_6memory_10BaseMemory_6device_1__get__(o);
}

static PyObject *__pyx_getprop_4cupy_4cuda_6memory_10BaseMemory_ptr(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_4cupy_4cuda_6memory_10BaseMemory_3ptr_1__get__(o);
}

static int __pyx_setprop_4cupy_4cuda_6memory_10BaseMemory_ptr(PyObject *o, PyObject *v, CYTHON_UNUSED void *x) {
  if (v) {
    return __pyx_pw_4cupy_4cuda_6memory_10BaseMemory_3ptr_3__set__(o, v);
  }
  else {
    PyErr_SetString(PyExc_NotImplementedError, "__del__");
    return -1;
  }
}

static PyObject *__pyx_getprop_4cupy_4cuda_6memory_10BaseMemory_size(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_4cupy_4cuda_6memory_10BaseMemory_4size_1__get__(o);
}

static int __pyx_setprop_4cupy_4cuda_6memory_10BaseMemory_size(PyObject *o, PyObject *v, CYTHON_UNUSED void *x) {
  if (v) {
    return __pyx_pw_4cupy_4cuda_6memory_10BaseMemory_4size_3__set__(o, v);
  }
  else {
    PyErr_SetString(PyExc_NotImplementedError, "__del__");
    return -1;
  }
}

static PyObject *__pyx_getprop_4cupy_4cuda_6memory_10BaseMemory_device_id(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_4cupy_4cuda_6memory_10BaseMemory_9device_id_1__get__(o);
}

static int __pyx_setprop_4cupy_4cuda_6memory_10BaseMemory_device_id(PyObject *o, PyObject *v, CYTHON_UNUSED void *x) {
  if (v) {
    return __pyx_pw_4cupy_4cuda_6memory_10BaseMemory_9device_id_3__set__(o, v);
  }
  else {
    PyErr_SetString(PyExc_NotImplementedError, "__del__");
    return -1;
  }
}

static PyMethodDef __pyx_methods_4cupy_4cuda_6memory_BaseMemory[] = {
  {"__reduce_cython__", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_4cupy_4cuda_6memory_10BaseMemory_3__reduce_cython__, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_4cupy_4cuda_6memory_10BaseMemory_2__reduce_cython__},
  {"__setstate_cython__", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_4cupy_4cuda_6memory_10BaseMemory_5__setstate_cython__, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_4cupy_4cuda_6memory_10BaseMemory_4__setstate_cython__},
  {0, 0, 0, 0}
};

static struct PyGetSetDef __pyx_getsets_4cupy_4cuda_6memory_BaseMemory[] = {
  {"device", __pyx_getprop_4cupy_4cuda_6memory_10BaseMemory_device, 0, 0, 0},
  {"ptr", __pyx_getprop_4cupy_4cuda_6memory_10BaseMemory_ptr, __pyx_setprop_4cupy_4cuda_6memory_10BaseMemory_ptr, PyDoc_STR("ptr: 'intptr_t'"), 0},
  {"size", __pyx_getprop_4cupy_4cuda_6memory_10BaseMemory_size, __pyx_setprop_4cupy_4cuda_6memory_10BaseMemory_size, PyDoc_STR("size: 'size_t'"), 0},
  {"device_id", __pyx_getprop_4cupy_4cuda_6memory_10BaseMemory_device_id, __pyx_setprop_4cupy_4cuda_6memory_10BaseMemory_device_id, PyDoc_STR("device_id: 'int'"), 0},
  {0, 0, 0, 0, 0}
};
#if CYTHON_USE_TYPE_SPECS
static PyType_Slot __pyx_type_4cupy_4cuda_6memory_BaseMemory_slots[] = {
  {Py_tp_dealloc, (void *)__pyx_tp_dealloc_4cupy_4cuda_6memory_BaseMemory},
  {Py_nb_int, (void *)__pyx_pw_4cupy_4cuda_6memory_10BaseMemory_1__int__},
  {Py_tp_doc, (void *)PyDoc_STR("Memory on a CUDA device.\n\n    Attributes:\n        ~Memory.ptr (int): Pointer to the place within the buffer.\n        ~Memory.size (int): Size of the memory allocation in bytes.\n        ~Memory.device (~cupy.cuda.Device): Device whose memory the pointer\n            refers to.\n    ")},
  {Py_tp_methods, (void *)__pyx_methods_4cupy_4cuda_6memory_BaseMemory},
  {Py_tp_getset, (void *)__pyx_getsets_4cupy_4cuda_6memory_BaseMemory},
  {Py_tp_new, (void *)__pyx_tp_new_4cupy_4cuda_6memory_BaseMemory},
  {0, 0},
};
static PyType_Spec __pyx_type_4cupy_4cuda_6memory_BaseMemory_spec = {
  "cupy.cuda.memory.BaseMemory",
  sizeof(struct __pyx_obj_4cupy_4cuda_6memory_BaseMemory),
  0,
  Py_TPFLAGS_DEFAULT|Py_TPFLAGS_HAVE_VERSION_TAG|Py_TPFLAGS_CHECKTYPES|Py_TPFLAGS_HAVE_NEWBUFFER|Py_TPFLAGS_BASETYPE,
  __pyx_type_4cupy_4cuda_6memory_BaseMemory_slots,
};
#else

static PyNumberMethods __pyx_tp_as_number_BaseMemory = {
  0, /*nb_add*/
  0, /*nb_subtract*/
  0, /*nb_multiply*/
  0, /*nb_remainder*/
  0, /*nb_divmod*/
  0, /*nb_power*/
  0, /*nb_negative*/
  0, /*nb_positive*/
  0, /*nb_absolute*/
  0, /*nb_bool*/
  0, /*nb_invert*/
  0, /*nb_lshift*/
  0, /*nb_rshift*/
  0, /*nb_and*/
  0, /*nb_xor*/
  0, /*nb_or*/
  __pyx_pw_4cupy_4cuda_6memory_10BaseMemory_1__int__, /*nb_int*/
  0, /*nb_long (reserved)*/
  0, /*nb_float*/
  0, /*nb_inplace_add*/
  0, /*nb_inplace_subtract*/
  0, /*nb_inplace_multiply*/
  0, /*nb_inplace_remainder*/
  0, /*nb_inplace_power*/
  0, /*nb_inplace_lshift*/
  0, /*nb_inplace_rshift*/
  0, /*nb_inplace_and*/
  0, /*nb_inplace_xor*/
  0, /*nb_inplace_or*/
  0, /*nb_floor_divide*/
  0, /*nb_true_divide*/
  0, /*nb_inplace_floor_divide*/
  0, /*nb_inplace_true_divide*/
  0, /*nb_index*/
  0, /*nb_matrix_multiply*/
  0, /*nb_inplace_matrix_multiply*/
};

static PyTypeObject __pyx_type_4cupy_4cuda_6memory_BaseMemory = {
  PyVarObject_HEAD_INIT(0, 0)
  "cupy.cuda.memory.""BaseMemory", /*tp_name*/
  sizeof(struct __pyx_obj_4cupy_4cuda_6memory_BaseMemory), /*tp_basicsize*/
  0, /*tp_itemsize*/
  __pyx_tp_dealloc_4cupy_4cuda_6memory_BaseMemory, /*tp_dealloc*/
  0, /*tp_vectorcall_offset*/
  0, /*tp_getattr*/
  0, /*tp_setattr*/
  0, /*tp_as_async*/
  0, /*tp_repr*/
  &__pyx_tp_as_number_BaseMemory, /*tp_as_number*/
  0, /*tp_as_sequence*/
  0, /*tp_as_mapping*/
  0, /*tp_hash*/
  0, /*tp_call*/
  0, /*tp_str*/
  0, /*tp_getattro*/
  0, /*tp_setattro*/
  0, /*tp_as_buffer*/
  Py_TPFLAGS_DEFAULT|Py_TPFLAGS_HAVE_VERSION_TAG|Py_TPFLAGS_CHECKTYPES|Py_TPFLAGS_HAVE_NEWBUFFER|Py_TPFLAGS_BASETYPE, /*tp_flags*/
  PyDoc_STR("Memory on a CUDA device.\n\n    Attributes:\n        ~Memory.ptr (int): Pointer to the place within the buffer.\n        ~Memory.size (int): Size of the memory allocation in bytes.\n        ~Memory.device (~cupy.cuda.Device): Device whose memory the pointer\n            refers to.\n    "), /*tp_doc*/
  0, /*tp_traverse*/
  0, /*tp_clear*/
  0, /*tp_richcompare*/
  0, /*tp_weaklistoffset*/
  0, /*tp_iter*/
  0, /*tp_iternext*/
  __pyx_methods_4cupy_4cuda_6memory_BaseMemory, /*tp_methods*/
  0, /*tp_members*/
  __pyx_getsets_4cupy_4cuda_6memory_BaseMemory, /*tp_getset*/
  0, /*tp_base*/
  0, /*tp_dict*/
  0, /*tp_descr_get*/
  0, /*tp_descr_set*/
  #if !CYTHON_USE_TYPE_SPECS
  0, /*tp_dictoffset*/
  #endif
  0, /*tp_init*/
  0, /*tp_alloc*/
  __pyx_tp_new_4cupy_4cuda_6memory_BaseMemory, /*tp_new*/
  0, /*tp_free*/
  0, /*tp_is_gc*/
  0, /*tp_bases*/
  0, /*tp_mro*/
  0, /*tp_cache*/
  0, /*tp_subclasses*/
  0, /*tp_weaklist*/
  0, /*tp_del*/
  0, /*tp_version_tag*/
  #if CYTHON_USE_TP_FINALIZE
  0, /*tp_finalize*/
  #else
  NULL, /*tp_finalize*/
  #endif
  #if !CYTHON_COMPILING_IN_PYPY || PYPY_VERSION_NUM >= 0x07030800
  0, /*tp_vectorcall*/
  #endif
  #if __PYX_NEED_TP_PRINT_SLOT == 1
  0, /*tp_print*/
  #endif
  #if PY_VERSION_HEX >= 0x030C0000
  0, /*tp_watched*/
  #endif
  #if PY_VERSION_HEX >= 0x030d00A4
  0, /*tp_versions_used*/
  #endif
  #if CYTHON_COMPILING_IN_PYPY && PY_VERSION_HEX >= 0x03090000 && PY_VERSION_HEX < 0x030a0000
  0, /*tp_pypy_flags*/
  #endif
};
#endif
static struct __pyx_vtabstruct_4cupy_4cuda_6memory_SystemMemory __pyx_vtable_4cupy_4cuda_6memory_SystemMemory;

static PyObject *__pyx_tp_new_4cupy_4cuda_6memory_SystemMemory(PyTypeObject *t, PyObject *a, PyObject *k) {
  struct __pyx_obj_4cupy_4cuda_6memory_SystemMemory *p;
  PyObject *o = __pyx_tp_new_4cupy_4cuda_6memory_BaseMemory(t, a, k);
  if (unlikely(!o)) return 0;
  p = ((struct __pyx_obj_4cupy_4cuda_6memory_SystemMemory *)o);
  p->__pyx_vtab = __pyx_vtabptr_4cupy_4cuda_6memory_SystemMemory;
  p->_owner = Py_None; Py_INCREF(Py_None);
  return o;
}

static void __pyx_tp_dealloc_4cupy_4cuda_6memory_SystemMemory(PyObject *o) {
  struct __pyx_obj_4cupy_4cuda_6memory_SystemMemory *p = (struct __pyx_obj_4cupy_4cuda_6memory_SystemMemory *)o;
  #if CYTHON_USE_TP_FINALIZE
  if (unlikely(__Pyx_PyObject_GetSlot(o, tp_finalize, destructor)) && (!PyType_IS_GC(Py_TYPE(o)) || !__Pyx_PyObject_GC_IsFinalized(o))) {
    if (__Pyx_PyObject_GetSlot(o, tp_dealloc, destructor) == __pyx_tp_dealloc_4cupy_4cuda_6memory_SystemMemory) {
      if (PyObject_CallFinalizerFromDealloc(o)) return;
    }
  }
  #endif
  {
    PyObject *etype, *eval, *etb;
    PyErr_Fetch(&etype, &eval, &etb);
    __Pyx_SET_REFCNT(o, Py_REFCNT(o) + 1);
    __pyx_pw_4cupy_4cuda_6memory_12SystemMemory_7__dealloc__(o);
    __Pyx_SET_REFCNT(o, Py_REFCNT(o) - 1);
    PyErr_Restore(etype, eval, etb);
  }
  Py_CLEAR(p->_owner);
  __pyx_tp_dealloc_4cupy_4cuda_6memory_BaseMemory(o);
}

static PyObject *__pyx_getprop_4cupy_4cuda_6memory_12SystemMemory__owner(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_4cupy_4cuda_6memory_12SystemMemory_6_owner_1__get__(o);
}

static PyMethodDef __pyx_methods_4cupy_4cuda_6memory_SystemMemory[] = {
  {"prefetch", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_4cupy_4cuda_6memory_12SystemMemory_3prefetch, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_4cupy_4cuda_6memory_12SystemMemory_2prefetch},
  {"advise", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_4cupy_4cuda_6memory_12SystemMemory_5advise, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_4cupy_4cuda_6memory_12SystemMemory_4advise},
  {"__reduce_cython__", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_4cupy_4cuda_6memory_12SystemMemory_9__reduce_cython__, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_4cupy_4cuda_6memory_12SystemMemory_8__reduce_cython__},
  {"__setstate_cython__", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_4cupy_4cuda_6memory_12SystemMemory_11__setstate_cython__, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_4cupy_4cuda_6memory_12SystemMemory_10__setstate_cython__},
  {0, 0, 0, 0}
};

static struct PyGetSetDef __pyx_getsets_4cupy_4cuda_6memory_SystemMemory[] = {
  {"_owner", __pyx_getprop_4cupy_4cuda_6memory_12SystemMemory__owner, 0, 0, 0},
  {0, 0, 0, 0, 0}
};
#if CYTHON_USE_TYPE_SPECS
static PyType_Slot __pyx_type_4cupy_4cuda_6memory_SystemMemory_slots[] = {
  {Py_tp_dealloc, (void *)__pyx_tp_dealloc_4cupy_4cuda_6memory_SystemMemory},
  {Py_tp_doc, (void *)PyDoc_STR("SystemMemory(size_t size)\n\nMemory allocation on an HMM/ATS enabled system.\n\nHMM stands for heterogeneous memory management. It is a kernel-level\nfeature allowing memory allocated via the system ``malloc`` to be\naccessible by both CPU and GPU.\n\nATS stands for Address Translation Services. It is a hardware/software\nfeature on Grace Hopper that enables the CPU and GPU to share a single\nper-process page table, allowing memory allocated by the system to be\naccessible by both CPU and GPU.\n\nThis class provides an RAII interface of the memory allocation.\n\nArgs:\n    size (int): Size of the memory allocation in bytes.")},
  {Py_tp_methods, (void *)__pyx_methods_4cupy_4cuda_6memory_SystemMemory},
  {Py_tp_getset, (void *)__pyx_getsets_4cupy_4cuda_6memory_SystemMemory},
  {Py_tp_init, (void *)__pyx_pw_4cupy_4cuda_6memory_12SystemMemory_1__init__},
  {Py_tp_new, (void *)__pyx_tp_new_4cupy_4cuda_6memory_SystemMemory},
  {0, 0},
};
static PyType_Spec __pyx_type_4cupy_4cuda_6memory_SystemMemory_spec = {
  "cupy.cuda.memory.SystemMemory",
  sizeof(struct __pyx_obj_4cupy_4cuda_6memory_SystemMemory),
  0,
  Py_TPFLAGS_DEFAULT|Py_TPFLAGS_HAVE_VERSION_TAG|Py_TPFLAGS_CHECKTYPES|Py_TPFLAGS_HAVE_NEWBUFFER|Py_TPFLAGS_BASETYPE,
  __pyx_type_4cupy_4cuda_6memory_SystemMemory_slots,
};
#else

static PyTypeObject __pyx_type_4cupy_4cuda_6memory_SystemMemory = {
  PyVarObject_HEAD_INIT(0, 0)
  "cupy.cuda.memory.""SystemMemory", /*tp_name*/
  sizeof(struct __pyx_obj_4cupy_4cuda_6memory_SystemMemory), /*tp_basicsize*/
  0, /*tp_itemsize*/
  __pyx_tp_dealloc_4cupy_4cuda_6memory_SystemMemory, /*tp_dealloc*/
  0, /*tp_vectorcall_offset*/
  0, /*tp_getattr*/
  0, /*tp_setattr*/
  0, /*tp_as_async*/
  0, /*tp_repr*/
  0, /*tp_as_number*/
  0, /*tp_as_sequence*/
  0, /*tp_as_mapping*/
  0, /*tp_hash*/
  0, /*tp_call*/
  0, /*tp_str*/
  0, /*tp_getattro*/
  0, /*tp_setattro*/
  0, /*tp_as_buffer*/
  Py_TPFLAGS_DEFAULT|Py_TPFLAGS_HAVE_VERSION_TAG|Py_TPFLAGS_CHECKTYPES|Py_TPFLAGS_HAVE_NEWBUFFER|Py_TPFLAGS_BASETYPE, /*tp_flags*/
  PyDoc_STR("SystemMemory(size_t size)\n\nMemory allocation on an HMM/ATS enabled system.\n\nHMM stands for heterogeneous memory management. It is a kernel-level\nfeature allowing memory allocated via the system ``malloc`` to be\naccessible by both CPU and GPU.\n\nATS stands for Address Translation Services. It is a hardware/software\nfeature on Grace Hopper that enables the CPU and GPU to share a single\nper-process page table, allowing memory allocated by the system to be\naccessible by both CPU and GPU.\n\nThis class provides an RAII interface of the memory allocation.\n\nArgs:\n    size (int): Size of the memory allocation in bytes."), /*tp_doc*/
  0, /*tp_traverse*/
  0, /*tp_clear*/
  0, /*tp_richcompare*/
  0, /*tp_weaklistoffset*/
  0, /*tp_iter*/
  0, /*tp_iternext*/
  __pyx_methods_4cupy_4cuda_6memory_SystemMemory, /*tp_methods*/
  0, /*tp_members*/
  __pyx_getsets_4cupy_4cuda_6memory_SystemMemory, /*tp_getset*/
  0, /*tp_base*/
  0, /*tp_dict*/
  0, /*tp_descr_get*/
  0, /*tp_descr_set*/
  #if !CYTHON_USE_TYPE_SPECS
  0, /*tp_dictoffset*/
  #endif
  __pyx_pw_4cupy_4cuda_6memory_12SystemMemory_1__init__, /*tp_init*/
  0, /*tp_alloc*/
  __pyx_tp_new_4cupy_4cuda_6memory_SystemMemory, /*tp_new*/
  0, /*tp_free*/
  0, /*tp_is_gc*/
  0, /*tp_bases*/
  0, /*tp_mro*/
  0, /*tp_cache*/
  0, /*tp_subclasses*/
  0, /*tp_weaklist*/
  0, /*tp_del*/
  0, /*tp_version_tag*/
  #if CYTHON_USE_TP_FINALIZE
  0, /*tp_finalize*/
  #else
  NULL, /*tp_finalize*/
  #endif
  #if !CYTHON_COMPILING_IN_PYPY || PYPY_VERSION_NUM >= 0x07030800
  0, /*tp_vectorcall*/
  #endif
  #if __PYX_NEED_TP_PRINT_SLOT == 1
  0, /*tp_print*/
  #endif
  #if PY_VERSION_HEX >= 0x030C0000
  0, /*tp_watched*/
  #endif
  #if PY_VERSION_HEX >= 0x030d00A4
  0, /*tp_versions_used*/
  #endif
  #if CYTHON_COMPILING_IN_PYPY && PY_VERSION_HEX >= 0x03090000 && PY_VERSION_HEX < 0x030a0000
  0, /*tp_pypy_flags*/
  #endif
};
#endif
static struct __pyx_vtabstruct_4cupy_4cuda_6memory_MemoryPointer __pyx_vtable_4cupy_4cuda_6memory_MemoryPointer;

static PyObject *__pyx_tp_new_4cupy_4cuda_6memory_MemoryPointer(PyTypeObject *t, CYTHON_UNUSED PyObject *a, CYTHON_UNUSED PyObject *k) {
  struct __pyx_obj_4cupy_4cuda_6memory_MemoryPointer *p;
  PyObject *o;
  o = __Pyx_AllocateExtensionType(t, 0);
  if (unlikely(!o)) return 0;
  p = ((struct __pyx_obj_4cupy_4cuda_6memory_MemoryPointer *)o);
  p->__pyx_vtab = __pyx_vtabptr_4cupy_4cuda_6memory_MemoryPointer;
  p->mem = ((struct __pyx_obj_4cupy_4cuda_6memory_BaseMemory *)Py_None); Py_INCREF(Py_None);
  return o;
}

static void __pyx_tp_dealloc_4cupy_4cuda_6memory_MemoryPointer(PyObject *o) {
  struct __pyx_obj_4cupy_4cuda_6memory_MemoryPointer *p = (struct __pyx_obj_4cupy_4cuda_6memory_MemoryPointer *)o;
  #if CYTHON_USE_TP_FINALIZE
  if (unlikely(__Pyx_PyObject_GetSlot(o, tp_finalize, destructor)) && !__Pyx_PyObject_GC_IsFinalized(o)) {
    if (__Pyx_PyObject_GetSlot(o, tp_dealloc, destructor) == __pyx_tp_dealloc_4cupy_4cuda_6memory_MemoryPointer) {
      if (PyObject_CallFinalizerFromDealloc(o)) return;
    }
  }
  #endif
  PyObject_GC_UnTrack(o);
  Py_CLEAR(p->mem);
  PyTypeObject *tp = Py_TYPE(o);
  #if CYTHON_USE_TYPE_SLOTS
  (*tp->tp_free)(o);
  #else
  {
    freefunc tp_free = (freefunc)PyType_GetSlot(tp, Py_tp_free);
    if (tp_free) tp_free(o);
  }
  #endif
  #if CYTHON_USE_TYPE_SPECS
  Py_DECREF(tp);
  #endif
}

static int __pyx_tp_traverse_4cupy_4cuda_6memory_MemoryPointer(PyObject *o, visitproc v, void *a) {
  int e;
  struct __pyx_obj_4cupy_4cuda_6memory_MemoryPointer *p = (struct __pyx_obj_4cupy_4cuda_6memory_MemoryPointer *)o;
  {
    e = __Pyx_call_type_traverse(o, 1, v, a);
    if (e) return e;
  }
  if (p->mem) {
    e = (*v)(((PyObject *)p->mem), a); if (e) return e;
  }
  return 0;
}

static int __pyx_tp_clear_4cupy_4cuda_6memory_MemoryPointer(PyObject *o) {
  PyObject* tmp;
  struct __pyx_obj_4cupy_4cuda_6memory_MemoryPointer *p = (struct __pyx_obj_4cupy_4cuda_6memory_MemoryPointer *)o;
  tmp = ((PyObject*)p->mem);
  p->mem = ((struct __pyx_obj_4cupy_4cuda_6memory_BaseMemory *)Py_None); Py_INCREF(Py_None);
  Py_XDECREF(tmp);
  return 0;
}
#define __pyx_nb_add_4cupy_4cuda_6memory_MemoryPointer __pyx_pw_4cupy_4cuda_6memory_13MemoryPointer_7__add__
#define __pyx_nb_subtract_4cupy_4cuda_6memory_MemoryPointer __pyx_pw_4cupy_4cuda_6memory_13MemoryPointer_11__sub__

static PyObject *__pyx_getprop_4cupy_4cuda_6memory_13MemoryPointer_device(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_4cupy_4cuda_6memory_13MemoryPointer_6device_1__get__(o);
}

static PyObject *__pyx_getprop_4cupy_4cuda_6memory_13MemoryPointer_ptr(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_4cupy_4cuda_6memory_13MemoryPointer_3ptr_1__get__(o);
}

static PyObject *__pyx_getprop_4cupy_4cuda_6memory_13MemoryPointer_device_id(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_4cupy_4cuda_6memory_13MemoryPointer_9device_id_1__get__(o);
}

static PyObject *__pyx_getprop_4cupy_4cuda_6memory_13MemoryPointer_mem(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_4cupy_4cuda_6memory_13MemoryPointer_3mem_1__get__(o);
}

static PyMethodDef __pyx_methods_4cupy_4cuda_6memory_MemoryPointer[] = {
  {"__add__", (PyCFunction)__pyx_pw_4cupy_4cuda_6memory_13MemoryPointer_7__add__, METH_O|METH_COEXIST, __pyx_doc_4cupy_4cuda_6memory_13MemoryPointer_6__add__},
  {"__sub__", (PyCFunction)__pyx_pw_4cupy_4cuda_6memory_13MemoryPointer_11__sub__, METH_O|METH_COEXIST, __pyx_doc_4cupy_4cuda_6memory_13MemoryPointer_10__sub__},
  {"copy_from_device", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_4cupy_4cuda_6memory_13MemoryPointer_15copy_from_device, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_4cupy_4cuda_6memory_13MemoryPointer_14copy_from_device},
  {"copy_from_device_async", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_4cupy_4cuda_6memory_13MemoryPointer_17copy_from_device_async, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_4cupy_4cuda_6memory_13MemoryPointer_16copy_from_device_async},
  {"copy_from_host", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_4cupy_4cuda_6memory_13MemoryPointer_19copy_from_host, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_4cupy_4cuda_6memory_13MemoryPointer_18copy_from_host},
  {"copy_from_host_async", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_4cupy_4cuda_6memory_13MemoryPointer_21copy_from_host_async, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_4cupy_4cuda_6memory_13MemoryPointer_20copy_from_host_async},
  {"copy_from", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_4cupy_4cuda_6memory_13MemoryPointer_23copy_from, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_4cupy_4cuda_6memory_13MemoryPointer_22copy_from},
  {"copy_from_async", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_4cupy_4cuda_6memory_13MemoryPointer_25copy_from_async, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_4cupy_4cuda_6memory_13MemoryPointer_24copy_from_async},
  {"copy_to_host", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_4cupy_4cuda_6memory_13MemoryPointer_27copy_to_host, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_4cupy_4cuda_6memory_13MemoryPointer_26copy_to_host},
  {"copy_to_host_async", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_4cupy_4cuda_6memory_13MemoryPointer_29copy_to_host_async, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_4cupy_4cuda_6memory_13MemoryPointer_28copy_to_host_async},
  {"memset", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_4cupy_4cuda_6memory_13MemoryPointer_31memset, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_4cupy_4cuda_6memory_13MemoryPointer_30memset},
  {"memset_async", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_4cupy_4cuda_6memory_13MemoryPointer_33memset_async, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_4cupy_4cuda_6memory_13MemoryPointer_32memset_async},
  {"__reduce_cython__", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_4cupy_4cuda_6memory_13MemoryPointer_35__reduce_cython__, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_4cupy_4cuda_6memory_13MemoryPointer_34__reduce_cython__},
  {"__setstate_cython__", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_4cupy_4cuda_6memory_13MemoryPointer_37__setstate_cython__, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_4cupy_4cuda_6memory_13MemoryPointer_36__setstate_cython__},
  {0, 0, 0, 0}
};

static struct PyGetSetDef __pyx_getsets_4cupy_4cuda_6memory_MemoryPointer[] = {
  {"device", __pyx_getprop_4cupy_4cuda_6memory_13MemoryPointer_device, 0, 0, 0},
  {"ptr", __pyx_getprop_4cupy_4cuda_6memory_13MemoryPointer_ptr, 0, 0, 0},
  {"device_id", __pyx_getprop_4cupy_4cuda_6memory_13MemoryPointer_device_id, 0, 0, 0},
  {"mem", __pyx_getprop_4cupy_4cuda_6memory_13MemoryPointer_mem, 0, 0, 0},
  {0, 0, 0, 0, 0}
};
#if CYTHON_USE_TYPE_SPECS
static PyType_Slot __pyx_type_4cupy_4cuda_6memory_MemoryPointer_slots[] = {
  {Py_tp_dealloc, (void *)__pyx_tp_dealloc_4cupy_4cuda_6memory_MemoryPointer},
  {Py_tp_repr, (void *)__pyx_pw_4cupy_4cuda_6memory_13MemoryPointer_5__repr__},
  {Py_nb_add, (void *)__pyx_nb_add_4cupy_4cuda_6memory_MemoryPointer},
  {Py_nb_subtract, (void *)__pyx_nb_subtract_4cupy_4cuda_6memory_MemoryPointer},
  {Py_nb_int, (void *)__pyx_pw_4cupy_4cuda_6memory_13MemoryPointer_3__int__},
  {Py_nb_inplace_add, (void *)__pyx_pw_4cupy_4cuda_6memory_13MemoryPointer_9__iadd__},
  {Py_nb_inplace_subtract, (void *)__pyx_pw_4cupy_4cuda_6memory_13MemoryPointer_13__isub__},
  {Py_tp_doc, (void *)PyDoc_STR("MemoryPointer(BaseMemory mem, ptrdiff_t offset)\n\nPointer to a point on a device memory.\n\nAn instance of this class holds a reference to the original memory buffer\nand a pointer to a place within this buffer.\n\nArgs:\n    mem (~cupy.cuda.BaseMemory): The device memory buffer.\n    offset (int): An offset from the head of the buffer to the place this\n        pointer refers.\n\nAttributes:\n    ~MemoryPointer.device (~cupy.cuda.Device): Device whose memory the\n        pointer refers to.\n    ~MemoryPointer.mem (~cupy.cuda.BaseMemory): The device memory buffer.\n    ~MemoryPointer.ptr (int): Pointer to the place within the buffer.")},
  {Py_tp_traverse, (void *)__pyx_tp_traverse_4cupy_4cuda_6memory_MemoryPointer},
  {Py_tp_clear, (void *)__pyx_tp_clear_4cupy_4cuda_6memory_MemoryPointer},
  {Py_tp_methods, (void *)__pyx_methods_4cupy_4cuda_6memory_MemoryPointer},
  {Py_tp_getset, (void *)__pyx_getsets_4cupy_4cuda_6memory_MemoryPointer},
  {Py_tp_init, (void *)__pyx_pw_4cupy_4cuda_6memory_13MemoryPointer_1__init__},
  {Py_tp_new, (void *)__pyx_tp_new_4cupy_4cuda_6memory_MemoryPointer},
  {0, 0},
};
static PyType_Spec __pyx_type_4cupy_4cuda_6memory_MemoryPointer_spec = {
  "cupy.cuda.memory.MemoryPointer",
  sizeof(struct __pyx_obj_4cupy_4cuda_6memory_MemoryPointer),
  0,
  Py_TPFLAGS_DEFAULT|Py_TPFLAGS_HAVE_VERSION_TAG|Py_TPFLAGS_CHECKTYPES|Py_TPFLAGS_HAVE_NEWBUFFER|Py_TPFLAGS_BASETYPE|Py_TPFLAGS_HAVE_GC,
  __pyx_type_4cupy_4cuda_6memory_MemoryPointer_slots,
};
#else

static PyNumberMethods __pyx_tp_as_number_MemoryPointer = {
  __pyx_nb_add_4cupy_4cuda_6memory_MemoryPointer, /*nb_add*/
  __pyx_nb_subtract_4cupy_4cuda_6memory_MemoryPointer, /*nb_subtract*/
  0, /*nb_multiply*/
  0, /*nb_remainder*/
  0, /*nb_divmod*/
  0, /*nb_power*/
  0, /*nb_negative*/
  0, /*nb_positive*/
  0, /*nb_absolute*/
  0, /*nb_bool*/
  0, /*nb_invert*/
  0, /*nb_lshift*/
  0, /*nb_rshift*/
  0, /*nb_and*/
  0, /*nb_xor*/
  0, /*nb_or*/
  __pyx_pw_4cupy_4cuda_6memory_13MemoryPointer_3__int__, /*nb_int*/
  0, /*nb_long (reserved)*/
  0, /*nb_float*/
  __pyx_pw_4cupy_4cuda_6memory_13MemoryPointer_9__iadd__, /*nb_inplace_add*/
  __pyx_pw_4cupy_4cuda_6memory_13MemoryPointer_13__isub__, /*nb_inplace_subtract*/
  0, /*nb_inplace_multiply*/
  0, /*nb_inplace_remainder*/
  0, /*nb_inplace_power*/
  0, /*nb_inplace_lshift*/
  0, /*nb_inplace_rshift*/
  0, /*nb_inplace_and*/
  0, /*nb_inplace_xor*/
  0, /*nb_inplace_or*/
  0, /*nb_floor_divide*/
  0, /*nb_true_divide*/
  0, /*nb_inplace_floor_divide*/
  0, /*nb_inplace_true_divide*/
  0, /*nb_index*/
  0, /*nb_matrix_multiply*/
  0, /*nb_inplace_matrix_multiply*/
};

static PyTypeObject __pyx_type_4cupy_4cuda_6memory_MemoryPointer = {
  PyVarObject_HEAD_INIT(0, 0)
  "cupy.cuda.memory.""MemoryPointer", /*tp_name*/
  sizeof(struct __pyx_obj_4cupy_4cuda_6memory_MemoryPointer), /*tp_basicsize*/
  0, /*tp_itemsize*/
  __pyx_tp_dealloc_4cupy_4cuda_6memory_MemoryPointer, /*tp_dealloc*/
  0, /*tp_vectorcall_offset*/
  0, /*tp_getattr*/
  0, /*tp_setattr*/
  0, /*tp_as_async*/
  __pyx_pw_4cupy_4cuda_6memory_13MemoryPointer_5__repr__, /*tp_repr*/
  &__pyx_tp_as_number_MemoryPointer, /*tp_as_number*/
  0, /*tp_as_sequence*/
  0, /*tp_as_mapping*/
  0, /*tp_hash*/
  0, /*tp_call*/
  0, /*tp_str*/
  0, /*tp_getattro*/
  0, /*tp_setattro*/
  0, /*tp_as_buffer*/
  Py_TPFLAGS_DEFAULT|Py_TPFLAGS_HAVE_VERSION_TAG|Py_TPFLAGS_CHECKTYPES|Py_TPFLAGS_HAVE_NEWBUFFER|Py_TPFLAGS_BASETYPE|Py_TPFLAGS_HAVE_GC, /*tp_flags*/
  PyDoc_STR("MemoryPointer(BaseMemory mem, ptrdiff_t offset)\n\nPointer to a point on a device memory.\n\nAn instance of this class holds a reference to the original memory buffer\nand a pointer to a place within this buffer.\n\nArgs:\n    mem (~cupy.cuda.BaseMemory): The device memory buffer.\n    offset (int): An offset from the head of the buffer to the place this\n        pointer refers.\n\nAttributes:\n    ~MemoryPointer.device (~cupy.cuda.Device): Device whose memory the\n        pointer refers to.\n    ~MemoryPointer.mem (~cupy.cuda.BaseMemory): The device memory buffer.\n    ~MemoryPointer.ptr (int): Pointer to the place within the buffer."), /*tp_doc*/
  __pyx_tp_traverse_4cupy_4cuda_6memory_MemoryPointer, /*tp_traverse*/
  __pyx_tp_clear_4cupy_4cuda_6memory_MemoryPointer, /*tp_clear*/
  0, /*tp_richcompare*/
  0, /*tp_weaklistoffset*/
  0, /*tp_iter*/
  0, /*tp_iternext*/
  __pyx_methods_4cupy_4cuda_6memory_MemoryPointer, /*tp_methods*/
  0, /*tp_members*/
  __pyx_getsets_4cupy_4cuda_6memory_MemoryPointer, /*tp_getset*/
  0, /*tp_base*/
  0, /*tp_dict*/
  0, /*tp_descr_get*/
  0, /*tp_descr_set*/
  #if !CYTHON_USE_TYPE_SPECS
  0, /*tp_dictoffset*/
  #endif
  __pyx_pw_4cupy_4cuda_6memory_13MemoryPointer_1__init__, /*tp_init*/
  0, /*tp_alloc*/
  __pyx_tp_new_4cupy_4cuda_6memory_MemoryPointer, /*tp_new*/
  0, /*tp_free*/
  0, /*tp_is_gc*/
  0, /*tp_bases*/
  0, /*tp_mro*/
  0, /*tp_cache*/
  0, /*tp_subclasses*/
  0, /*tp_weaklist*/
  0, /*tp_del*/
  0, /*tp_version_tag*/
  #if CYTHON_USE_TP_FINALIZE
  0, /*tp_finalize*/
  #else
  NULL, /*tp_finalize*/
  #endif
  #if !CYTHON_COMPILING_IN_PYPY || PYPY_VERSION_NUM >= 0x07030800
  0, /*tp_vectorcall*/
  #endif
  #if __PYX_NEED_TP_PRINT_SLOT == 1
  0, /*tp_print*/
  #endif
  #if PY_VERSION_HEX >= 0x030C0000
  0, /*tp_watched*/
  #endif
  #if PY_VERSION_HEX >= 0x030d00A4
  0, /*tp_versions_used*/
  #endif
  #if CYTHON_COMPILING_IN_PYPY && PY_VERSION_HEX >= 0x03090000 && PY_VERSION_HEX < 0x030a0000
  0, /*tp_pypy_flags*/
  #endif
};
#endif
static struct __pyx_vtabstruct_4cupy_4cuda_6memory_MemoryPool __pyx_vtable_4cupy_4cuda_6memory_MemoryPool;

static PyObject *__pyx_tp_new_4cupy_4cuda_6memory_MemoryPool(PyTypeObject *t, CYTHON_UNUSED PyObject *a, CYTHON_UNUSED PyObject *k) {
  struct __pyx_obj_4cupy_4cuda_6memory_MemoryPool *p;
  PyObject *o;
  o = __Pyx_AllocateExtensionType(t, 0);
  if (unlikely(!o)) return 0;
  p = ((struct __pyx_obj_4cupy_4cuda_6memory_MemoryPool *)o);
  p->__pyx_vtab = __pyx_vtabptr_4cupy_4cuda_6memory_MemoryPool;
  p->_pools = ((PyObject*)Py_None); Py_INCREF(Py_None);
  p->_allocator = Py_None; Py_INCREF(Py_None);
  return o;
}

static void __pyx_tp_dealloc_4cupy_4cuda_6memory_MemoryPool(PyObject *o) {
  struct __pyx_obj_4cupy_4cuda_6memory_MemoryPool *p = (struct __pyx_obj_4cupy_4cuda_6memory_MemoryPool *)o;
  #if CYTHON_USE_TP_FINALIZE
  if (unlikely(__Pyx_PyObject_GetSlot(o, tp_finalize, destructor)) && !__Pyx_PyObject_GC_IsFinalized(o)) {
    if (__Pyx_PyObject_GetSlot(o, tp_dealloc, destructor) == __pyx_tp_dealloc_4cupy_4cuda_6memory_MemoryPool) {
      if (PyObject_CallFinalizerFromDealloc(o)) return;
    }
  }
  #endif
  PyObject_GC_UnTrack(o);
  Py_CLEAR(p->_pools);
  Py_CLEAR(p->_allocator);
  PyTypeObject *tp = Py_TYPE(o);
  #if CYTHON_USE_TYPE_SLOTS
  (*tp->tp_free)(o);
  #else
  {
    freefunc tp_free = (freefunc)PyType_GetSlot(tp, Py_tp_free);
    if (tp_free) tp_free(o);
  }
  #endif
  #if CYTHON_USE_TYPE_SPECS
  Py_DECREF(tp);
  #endif
}

static int __pyx_tp_traverse_4cupy_4cuda_6memory_MemoryPool(PyObject *o, visitproc v, void *a) {
  int e;
  struct __pyx_obj_4cupy_4cuda_6memory_MemoryPool *p = (struct __pyx_obj_4cupy_4cuda_6memory_MemoryPool *)o;
  {
    e = __Pyx_call_type_traverse(o, 1, v, a);
    if (e) return e;
  }
  if (p->_pools) {
    e = (*v)(p->_pools, a); if (e) return e;
  }
  if (p->_allocator) {
    e = (*v)(p->_allocator, a); if (e) return e;
  }
  return 0;
}

static int __pyx_tp_clear_4cupy_4cuda_6memory_MemoryPool(PyObject *o) {
  PyObject* tmp;
  struct __pyx_obj_4cupy_4cuda_6memory_MemoryPool *p = (struct __pyx_obj_4cupy_4cuda_6memory_MemoryPool *)o;
  tmp = ((PyObject*)p->_pools);
  p->_pools = ((PyObject*)Py_None); Py_INCREF(Py_None);
  Py_XDECREF(tmp);
  tmp = ((PyObject*)p->_allocator);
  p->_allocator = Py_None; Py_INCREF(Py_None);
  Py_XDECREF(tmp);
  return 0;
}

static PyMethodDef __pyx_methods_4cupy_4cuda_6memory_MemoryPool[] = {
  {"malloc", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_4cupy_4cuda_6memory_10MemoryPool_3malloc, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_4cupy_4cuda_6memory_10MemoryPool_2malloc},
  {"free_all_blocks", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_4cupy_4cuda_6memory_10MemoryPool_5free_all_blocks, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_4cupy_4cuda_6memory_10MemoryPool_4free_all_blocks},
  {"free_all_free", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_4cupy_4cuda_6memory_10MemoryPool_7free_all_free, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_4cupy_4cuda_6memory_10MemoryPool_6free_all_free},
  {"n_free_blocks", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_4cupy_4cuda_6memory_10MemoryPool_9n_free_blocks, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_4cupy_4cuda_6memory_10MemoryPool_8n_free_blocks},
  {"used_bytes", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_4cupy_4cuda_6memory_10MemoryPool_11used_bytes, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_4cupy_4cuda_6memory_10MemoryPool_10used_bytes},
  {"free_bytes", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_4cupy_4cuda_6memory_10MemoryPool_13free_bytes, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_4cupy_4cuda_6memory_10MemoryPool_12free_bytes},
  {"total_bytes", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_4cupy_4cuda_6memory_10MemoryPool_15total_bytes, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_4cupy_4cuda_6memory_10MemoryPool_14total_bytes},
  {"set_limit", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_4cupy_4cuda_6memory_10MemoryPool_17set_limit, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_4cupy_4cuda_6memory_10MemoryPool_16set_limit},
  {"get_limit", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_4cupy_4cuda_6memory_10MemoryPool_19get_limit, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_4cupy_4cuda_6memory_10MemoryPool_18get_limit},
  {"__reduce_cython__", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_4cupy_4cuda_6memory_10MemoryPool_21__reduce_cython__, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_4cupy_4cuda_6memory_10MemoryPool_20__reduce_cython__},
  {"__setstate_cython__", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_4cupy_4cuda_6memory_10MemoryPool_23__setstate_cython__, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_4cupy_4cuda_6memory_10MemoryPool_22__setstate_cython__},
  {0, 0, 0, 0}
};
#if CYTHON_USE_TYPE_SPECS
static PyType_Slot __pyx_type_4cupy_4cuda_6memory_MemoryPool_slots[] = {
  {Py_tp_dealloc, (void *)__pyx_tp_dealloc_4cupy_4cuda_6memory_MemoryPool},
  {Py_tp_doc, (void *)PyDoc_STR("MemoryPool(allocator=None)\n\nMemory pool for all GPU devices on the host.\n\nA memory pool preserves any allocations even if they are freed by the user.\nFreed memory buffers are held by the memory pool as *free blocks*, and they\nare reused for further memory allocations of the same sizes. The allocated\nblocks are managed for each device, so one instance of this class can be\nused for multiple devices.\n\n.. note::\n   When the allocation is skipped by reusing the pre-allocated block, it\n   does not call ``cudaMalloc`` and therefore CPU-GPU synchronization does\n   not occur. It makes interleaves of memory allocations and kernel\n   invocations very fast.\n\n.. note::\n   The memory pool holds allocated blocks without freeing as much as\n   possible. It makes the program hold most of the device memory, which may\n   make other CUDA programs running in parallel out-of-memory situation.\n\nArgs:\n    allocator (function): The base CuPy memory allocator. It is used for\n        allocating new blocks when the blocks of the required size are all\n        in use.")},
  {Py_tp_traverse, (void *)__pyx_tp_traverse_4cupy_4cuda_6memory_MemoryPool},
  {Py_tp_clear, (void *)__pyx_tp_clear_4cupy_4cuda_6memory_MemoryPool},
  {Py_tp_methods, (void *)__pyx_methods_4cupy_4cuda_6memory_MemoryPool},
  {Py_tp_init, (void *)__pyx_pw_4cupy_4cuda_6memory_10MemoryPool_1__init__},
  {Py_tp_new, (void *)__pyx_tp_new_4cupy_4cuda_6memory_MemoryPool},
  {0, 0},
};
static PyType_Spec __pyx_type_4cupy_4cuda_6memory_MemoryPool_spec = {
  "cupy.cuda.memory.MemoryPool",
  sizeof(struct __pyx_obj_4cupy_4cuda_6memory_MemoryPool),
  0,
  Py_TPFLAGS_DEFAULT|Py_TPFLAGS_HAVE_VERSION_TAG|Py_TPFLAGS_CHECKTYPES|Py_TPFLAGS_HAVE_NEWBUFFER|Py_TPFLAGS_BASETYPE|Py_TPFLAGS_HAVE_GC,
  __pyx_type_4cupy_4cuda_6memory_MemoryPool_slots,
};
#else

static PyTypeObject __pyx_type_4cupy_4cuda_6memory_MemoryPool = {
  PyVarObject_HEAD_INIT(0, 0)
  "cupy.cuda.memory.""MemoryPool", /*tp_name*/
  sizeof(struct __pyx_obj_4cupy_4cuda_6memory_MemoryPool), /*tp_basicsize*/
  0, /*tp_itemsize*/
  __pyx_tp_dealloc_4cupy_4cuda_6memory_MemoryPool, /*tp_dealloc*/
  0, /*tp_vectorcall_offset*/
  0, /*tp_getattr*/
  0, /*tp_setattr*/
  0, /*tp_as_async*/
  0, /*tp_repr*/
  0, /*tp_as_number*/
  0, /*tp_as_sequence*/
  0, /*tp_as_mapping*/
  0, /*tp_hash*/
  0, /*tp_call*/
  0, /*tp_str*/
  0, /*tp_getattro*/
  0, /*tp_setattro*/
  0, /*tp_as_buffer*/
  Py_TPFLAGS_DEFAULT|Py_TPFLAGS_HAVE_VERSION_TAG|Py_TPFLAGS_CHECKTYPES|Py_TPFLAGS_HAVE_NEWBUFFER|Py_TPFLAGS_BASETYPE|Py_TPFLAGS_HAVE_GC, /*tp_flags*/
  PyDoc_STR("MemoryPool(allocator=None)\n\nMemory pool for all GPU devices on the host.\n\nA memory pool preserves any allocations even if they are freed by the user.\nFreed memory buffers are held by the memory pool as *free blocks*, and they\nare reused for further memory allocations of the same sizes. The allocated\nblocks are managed for each device, so one instance of this class can be\nused for multiple devices.\n\n.. note::\n   When the allocation is skipped by reusing the pre-allocated block, it\n   does not call ``cudaMalloc`` and therefore CPU-GPU synchronization does\n   not occur. It makes interleaves of memory allocations and kernel\n   invocations very fast.\n\n.. note::\n   The memory pool holds allocated blocks without freeing as much as\n   possible. It makes the program hold most of the device memory, which may\n   make other CUDA programs running in parallel out-of-memory situation.\n\nArgs:\n    allocator (function): The base CuPy memory allocator. It is used for\n        allocating new blocks when the blocks of the required size are all\n        in use."), /*tp_doc*/
  __pyx_tp_traverse_4cupy_4cuda_6memory_MemoryPool, /*tp_traverse*/
  __pyx_tp_clear_4cupy_4cuda_6memory_MemoryPool, /*tp_clear*/
  0, /*tp_richcompare*/
  0, /*tp_weaklistoffset*/
  0, /*tp_iter*/
  0, /*tp_iternext*/
  __pyx_methods_4cupy_4cuda_6memory_MemoryPool, /*tp_methods*/
  0, /*tp_members*/
  0, /*tp_getset*/
  0, /*tp_base*/
  0, /*tp_dict*/
  0, /*tp_descr_get*/
  0, /*tp_descr_set*/
  #if !CYTHON_USE_TYPE_SPECS
  0, /*tp_dictoffset*/
  #endif
  __pyx_pw_4cupy_4cuda_6memory_10MemoryPool_1__init__, /*tp_init*/
  0, /*tp_alloc*/
  __pyx_tp_new_4cupy_4cuda_6memory_MemoryPool, /*tp_new*/
  0, /*tp_free*/
  0, /*tp_is_gc*/
  0, /*tp_bases*/
  0, /*tp_mro*/
  0, /*tp_cache*/
  0, /*tp_subclasses*/
  0, /*tp_weaklist*/
  0, /*tp_del*/
  0, /*tp_version_tag*/
  #if CYTHON_USE_TP_FINALIZE
  0, /*tp_finalize*/
  #else
  NULL, /*tp_finalize*/
  #endif
  #if !CYTHON_COMPILING_IN_PYPY || PYPY_VERSION_NUM >= 0x07030800
  0, /*tp_vectorcall*/
  #endif
  #if __PYX_NEED_TP_PRINT_SLOT == 1
  0, /*tp_print*/
  #endif
  #if PY_VERSION_HEX >= 0x030C0000
  0, /*tp_watched*/
  #endif
  #if PY_VERSION_HEX >= 0x030d00A4
  0, /*tp_versions_used*/
  #endif
  #if CYTHON_COMPILING_IN_PYPY && PY_VERSION_HEX >= 0x03090000 && PY_VERSION_HEX < 0x030a0000
  0, /*tp_pypy_flags*/
  #endif
};
#endif

static void __pyx_tp_dealloc_4cupy_4cuda_6memory_CFunctionAllocatorMemory(PyObject *o) {
  #if CYTHON_USE_TP_FINALIZE
  if (unlikely(__Pyx_PyObject_GetSlot(o, tp_finalize, destructor)) && (!PyType_IS_GC(Py_TYPE(o)) || !__Pyx_PyObject_GC_IsFinalized(o))) {
    if (__Pyx_PyObject_GetSlot(o, tp_dealloc, destructor) == __pyx_tp_dealloc_4cupy_4cuda_6memory_CFunctionAllocatorMemory) {
      if (PyObject_CallFinalizerFromDealloc(o)) return;
    }
  }
  #endif
  {
    PyObject *etype, *eval, *etb;
    PyErr_Fetch(&etype, &eval, &etb);
    __Pyx_SET_REFCNT(o, Py_REFCNT(o) + 1);
    __pyx_pw_4cupy_4cuda_6memory_24CFunctionAllocatorMemory_3__dealloc__(o);
    __Pyx_SET_REFCNT(o, Py_REFCNT(o) - 1);
    PyErr_Restore(etype, eval, etb);
  }
  __pyx_tp_dealloc_4cupy_4cuda_6memory_BaseMemory(o);
}

static PyMethodDef __pyx_methods_4cupy_4cuda_6memory_CFunctionAllocatorMemory[] = {
  {"__reduce_cython__", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_4cupy_4cuda_6memory_24CFunctionAllocatorMemory_5__reduce_cython__, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_4cupy_4cuda_6memory_24CFunctionAllocatorMemory_4__reduce_cython__},
  {"__setstate_cython__", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_4cupy_4cuda_6memory_24CFunctionAllocatorMemory_7__setstate_cython__, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_4cupy_4cuda_6memory_24CFunctionAllocatorMemory_6__setstate_cython__},
  {0, 0, 0, 0}
};
#if CYTHON_USE_TYPE_SPECS
static PyType_Slot __pyx_type_4cupy_4cuda_6memory_CFunctionAllocatorMemory_slots[] = {
  {Py_tp_dealloc, (void *)__pyx_tp_dealloc_4cupy_4cuda_6memory_CFunctionAllocatorMemory},
  {Py_tp_doc, (void *)PyDoc_STR("CFunctionAllocatorMemory(size_t size, intptr_t param, intptr_t malloc_func, intptr_t free_func, int device_id)")},
  {Py_tp_methods, (void *)__pyx_methods_4cupy_4cuda_6memory_CFunctionAllocatorMemory},
  {Py_tp_init, (void *)__pyx_pw_4cupy_4cuda_6memory_24CFunctionAllocatorMemory_1__init__},
  {Py_tp_new, (void *)__pyx_tp_new_4cupy_4cuda_6memory_BaseMemory},
  {0, 0},
};
static PyType_Spec __pyx_type_4cupy_4cuda_6memory_CFunctionAllocatorMemory_spec = {
  "cupy.cuda.memory.CFunctionAllocatorMemory",
  sizeof(struct __pyx_obj_4cupy_4cuda_6memory_CFunctionAllocatorMemory),
  0,
  Py_TPFLAGS_DEFAULT|Py_TPFLAGS_HAVE_VERSION_TAG|Py_TPFLAGS_CHECKTYPES|Py_TPFLAGS_HAVE_NEWBUFFER|Py_TPFLAGS_BASETYPE,
  __pyx_type_4cupy_4cuda_6memory_CFunctionAllocatorMemory_slots,
};
#else

static PyTypeObject __pyx_type_4cupy_4cuda_6memory_CFunctionAllocatorMemory = {
  PyVarObject_HEAD_INIT(0, 0)
  "cupy.cuda.memory.""CFunctionAllocatorMemory", /*tp_name*/
  sizeof(struct __pyx_obj_4cupy_4cuda_6memory_CFunctionAllocatorMemory), /*tp_basicsize*/
  0, /*tp_itemsize*/
  __pyx_tp_dealloc_4cupy_4cuda_6memory_CFunctionAllocatorMemory, /*tp_dealloc*/
  0, /*tp_vectorcall_offset*/
  0, /*tp_getattr*/
  0, /*tp_setattr*/
  0, /*tp_as_async*/
  0, /*tp_repr*/
  0, /*tp_as_number*/
  0, /*tp_as_sequence*/
  0, /*tp_as_mapping*/
  0, /*tp_hash*/
  0, /*tp_call*/
  0, /*tp_str*/
  0, /*tp_getattro*/
  0, /*tp_setattro*/
  0, /*tp_as_buffer*/
  Py_TPFLAGS_DEFAULT|Py_TPFLAGS_HAVE_VERSION_TAG|Py_TPFLAGS_CHECKTYPES|Py_TPFLAGS_HAVE_NEWBUFFER|Py_TPFLAGS_BASETYPE, /*tp_flags*/
  PyDoc_STR("CFunctionAllocatorMemory(size_t size, intptr_t param, intptr_t malloc_func, intptr_t free_func, int device_id)"), /*tp_doc*/
  0, /*tp_traverse*/
  0, /*tp_clear*/
  0, /*tp_richcompare*/
  0, /*tp_weaklistoffset*/
  0, /*tp_iter*/
  0, /*tp_iternext*/
  __pyx_methods_4cupy_4cuda_6memory_CFunctionAllocatorMemory, /*tp_methods*/
  0, /*tp_members*/
  0, /*tp_getset*/
  0, /*tp_base*/
  0, /*tp_dict*/
  0, /*tp_descr_get*/
  0, /*tp_descr_set*/
  #if !CYTHON_USE_TYPE_SPECS
  0, /*tp_dictoffset*/
  #endif
  __pyx_pw_4cupy_4cuda_6memory_24CFunctionAllocatorMemory_1__init__, /*tp_init*/
  0, /*tp_alloc*/
  __pyx_tp_new_4cupy_4cuda_6memory_BaseMemory, /*tp_new*/
  0, /*tp_free*/
  0, /*tp_is_gc*/
  0, /*tp_bases*/
  0, /*tp_mro*/
  0, /*tp_cache*/
  0, /*tp_subclasses*/
  0, /*tp_weaklist*/
  0, /*tp_del*/
  0, /*tp_version_tag*/
  #if CYTHON_USE_TP_FINALIZE
  0, /*tp_finalize*/
  #else
  NULL, /*tp_finalize*/
  #endif
  #if !CYTHON_COMPILING_IN_PYPY || PYPY_VERSION_NUM >= 0x07030800
  0, /*tp_vectorcall*/
  #endif
  #if __PYX_NEED_TP_PRINT_SLOT == 1
  0, /*tp_print*/
  #endif
  #if PY_VERSION_HEX >= 0x030C0000
  0, /*tp_watched*/
  #endif
  #if PY_VERSION_HEX >= 0x030d00A4
  0, /*tp_versions_used*/
  #endif
  #if CYTHON_COMPILING_IN_PYPY && PY_VERSION_HEX >= 0x03090000 && PY_VERSION_HEX < 0x030a0000
  0, /*tp_pypy_flags*/
  #endif
};
#endif
static struct __pyx_vtabstruct_4cupy_4cuda_6memory_CFunctionAllocator __pyx_vtable_4cupy_4cuda_6memory_CFunctionAllocator;

static PyObject *__pyx_tp_new_4cupy_4cuda_6memory_CFunctionAllocator(PyTypeObject *t, CYTHON_UNUSED PyObject *a, CYTHON_UNUSED PyObject *k) {
  struct __pyx_obj_4cupy_4cuda_6memory_CFunctionAllocator *p;
  PyObject *o;
  o = __Pyx_AllocateExtensionType(t, 0);
  if (unlikely(!o)) return 0;
  p = ((struct __pyx_obj_4cupy_4cuda_6memory_CFunctionAllocator *)o);
  p->__pyx_vtab = __pyx_vtabptr_4cupy_4cuda_6memory_CFunctionAllocator;
  p->_owner = Py_None; Py_INCREF(Py_None);
  return o;
}

static void __pyx_tp_dealloc_4cupy_4cuda_6memory_CFunctionAllocator(PyObject *o) {
  struct __pyx_obj_4cupy_4cuda_6memory_CFunctionAllocator *p = (struct __pyx_obj_4cupy_4cuda_6memory_CFunctionAllocator *)o;
  #if CYTHON_USE_TP_FINALIZE
  if (unlikely(__Pyx_PyObject_GetSlot(o, tp_finalize, destructor)) && !__Pyx_PyObject_GC_IsFinalized(o)) {
    if (__Pyx_PyObject_GetSlot(o, tp_dealloc, destructor) == __pyx_tp_dealloc_4cupy_4cuda_6memory_CFunctionAllocator) {
      if (PyObject_CallFinalizerFromDealloc(o)) return;
    }
  }
  #endif
  PyObject_GC_UnTrack(o);
  Py_CLEAR(p->_owner);
  PyTypeObject *tp = Py_TYPE(o);
  #if CYTHON_USE_TYPE_SLOTS
  (*tp->tp_free)(o);
  #else
  {
    freefunc tp_free = (freefunc)PyType_GetSlot(tp, Py_tp_free);
    if (tp_free) tp_free(o);
  }
  #endif
  #if CYTHON_USE_TYPE_SPECS
  Py_DECREF(tp);
  #endif
}

static int __pyx_tp_traverse_4cupy_4cuda_6memory_CFunctionAllocator(PyObject *o, visitproc v, void *a) {
  int e;
  struct __pyx_obj_4cupy_4cuda_6memory_CFunctionAllocator *p = (struct __pyx_obj_4cupy_4cuda_6memory_CFunctionAllocator *)o;
  {
    e = __Pyx_call_type_traverse(o, 1, v, a);
    if (e) return e;
  }
  if (p->_owner) {
    e = (*v)(p->_owner, a); if (e) return e;
  }
  return 0;
}

static int __pyx_tp_clear_4cupy_4cuda_6memory_CFunctionAllocator(PyObject *o) {
  PyObject* tmp;
  struct __pyx_obj_4cupy_4cuda_6memory_CFunctionAllocator *p = (struct __pyx_obj_4cupy_4cuda_6memory_CFunctionAllocator *)o;
  tmp = ((PyObject*)p->_owner);
  p->_owner = Py_None; Py_INCREF(Py_None);
  Py_XDECREF(tmp);
  return 0;
}

static PyMethodDef __pyx_methods_4cupy_4cuda_6memory_CFunctionAllocator[] = {
  {"malloc", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_4cupy_4cuda_6memory_18CFunctionAllocator_3malloc, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_4cupy_4cuda_6memory_18CFunctionAllocator_2malloc},
  {"__reduce_cython__", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_4cupy_4cuda_6memory_18CFunctionAllocator_5__reduce_cython__, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_4cupy_4cuda_6memory_18CFunctionAllocator_4__reduce_cython__},
  {"__setstate_cython__", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_4cupy_4cuda_6memory_18CFunctionAllocator_7__setstate_cython__, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_4cupy_4cuda_6memory_18CFunctionAllocator_6__setstate_cython__},
  {0, 0, 0, 0}
};
#if CYTHON_USE_TYPE_SPECS
static PyType_Slot __pyx_type_4cupy_4cuda_6memory_CFunctionAllocator_slots[] = {
  {Py_tp_dealloc, (void *)__pyx_tp_dealloc_4cupy_4cuda_6memory_CFunctionAllocator},
  {Py_tp_doc, (void *)PyDoc_STR("CFunctionAllocator(intptr_t param, intptr_t malloc_func, intptr_t free_func, owner)\n\nAllocator with C function pointers to allocation routines.\n\nThis allocator keeps raw pointers to a *param* object along with functions\npointers to *malloc* and *free*, delegating the actual allocation to\nexternal sources while only handling the timing of the resource allocation\nand deallocation.\n\n*malloc* should follow the signature ``void*(*malloc)(void*, size_t, int)``\nreturning the pointer to the allocated memory given the pointer to\n*param*, the number of bytes to allocate and the device id on which the\nallocation should take place.\n\nSimilarly, *free* should follow the signature\n``void(*free)(void*, void*, int)`` with no return, taking the pointer to\n*param*, the pointer to the allocated memory and the device id on which the\nmemory was allocated.\n\nArgs:\n    param (int): Address of *param*.\n    malloc_func (int): Address of *malloc*.\n    free_func (int): Address of *free*.\n    owner (object): Reference to the owner object to keep the param and\n        the functions alive.")},
  {Py_tp_traverse, (void *)__pyx_tp_traverse_4cupy_4cuda_6memory_CFunctionAllocator},
  {Py_tp_clear, (void *)__pyx_tp_clear_4cupy_4cuda_6memory_CFunctionAllocator},
  {Py_tp_methods, (void *)__pyx_methods_4cupy_4cuda_6memory_CFunctionAllocator},
  {Py_tp_init, (void *)__pyx_pw_4cupy_4cuda_6memory_18CFunctionAllocator_1__init__},
  {Py_tp_new, (void *)__pyx_tp_new_4cupy_4cuda_6memory_CFunctionAllocator},
  {0, 0},
};
static PyType_Spec __pyx_type_4cupy_4cuda_6memory_CFunctionAllocator_spec = {
  "cupy.cuda.memory.CFunctionAllocator",
  sizeof(struct __pyx_obj_4cupy_4cuda_6memory_CFunctionAllocator),
  0,
  Py_TPFLAGS_DEFAULT|Py_TPFLAGS_HAVE_VERSION_TAG|Py_TPFLAGS_CHECKTYPES|Py_TPFLAGS_HAVE_NEWBUFFER|Py_TPFLAGS_BASETYPE|Py_TPFLAGS_HAVE_GC,
  __pyx_type_4cupy_4cuda_6memory_CFunctionAllocator_slots,
};
#else

static PyTypeObject __pyx_type_4cupy_4cuda_6memory_CFunctionAllocator = {
  PyVarObject_HEAD_INIT(0, 0)
  "cupy.cuda.memory.""CFunctionAllocator", /*tp_name*/
  sizeof(struct __pyx_obj_4cupy_4cuda_6memory_CFunctionAllocator), /*tp_basicsize*/
  0, /*tp_itemsize*/
  __pyx_tp_dealloc_4cupy_4cuda_6memory_CFunctionAllocator, /*tp_dealloc*/
  0, /*tp_vectorcall_offset*/
  0, /*tp_getattr*/
  0, /*tp_setattr*/
  0, /*tp_as_async*/
  0, /*tp_repr*/
  0, /*tp_as_number*/
  0, /*tp_as_sequence*/
  0, /*tp_as_mapping*/
  0, /*tp_hash*/
  0, /*tp_call*/
  0, /*tp_str*/
  0, /*tp_getattro*/
  0, /*tp_setattro*/
  0, /*tp_as_buffer*/
  Py_TPFLAGS_DEFAULT|Py_TPFLAGS_HAVE_VERSION_TAG|Py_TPFLAGS_CHECKTYPES|Py_TPFLAGS_HAVE_NEWBUFFER|Py_TPFLAGS_BASETYPE|Py_TPFLAGS_HAVE_GC, /*tp_flags*/
  PyDoc_STR("CFunctionAllocator(intptr_t param, intptr_t malloc_func, intptr_t free_func, owner)\n\nAllocator with C function pointers to allocation routines.\n\nThis allocator keeps raw pointers to a *param* object along with functions\npointers to *malloc* and *free*, delegating the actual allocation to\nexternal sources while only handling the timing of the resource allocation\nand deallocation.\n\n*malloc* should follow the signature ``void*(*malloc)(void*, size_t, int)``\nreturning the pointer to the allocated memory given the pointer to\n*param*, the number of bytes to allocate and the device id on which the\nallocation should take place.\n\nSimilarly, *free* should follow the signature\n``void(*free)(void*, void*, int)`` with no return, taking the pointer to\n*param*, the pointer to the allocated memory and the device id on which the\nmemory was allocated.\n\nArgs:\n    param (int): Address of *param*.\n    malloc_func (int): Address of *malloc*.\n    free_func (int): Address of *free*.\n    owner (object): Reference to the owner object to keep the param and\n        the functions alive."), /*tp_doc*/
  __pyx_tp_traverse_4cupy_4cuda_6memory_CFunctionAllocator, /*tp_traverse*/
  __pyx_tp_clear_4cupy_4cuda_6memory_CFunctionAllocator, /*tp_clear*/
  0, /*tp_richcompare*/
  0, /*tp_weaklistoffset*/
  0, /*tp_iter*/
  0, /*tp_iternext*/
  __pyx_methods_4cupy_4cuda_6memory_CFunctionAllocator, /*tp_methods*/
  0, /*tp_members*/
  0, /*tp_getset*/
  0, /*tp_base*/
  0, /*tp_dict*/
  0, /*tp_descr_get*/
  0, /*tp_descr_set*/
  #if !CYTHON_USE_TYPE_SPECS
  0, /*tp_dictoffset*/
  #endif
  __pyx_pw_4cupy_4cuda_6memory_18CFunctionAllocator_1__init__, /*tp_init*/
  0, /*tp_alloc*/
  __pyx_tp_new_4cupy_4cuda_6memory_CFunctionAllocator, /*tp_new*/
  0, /*tp_free*/
  0, /*tp_is_gc*/
  0, /*tp_bases*/
  0, /*tp_mro*/
  0, /*tp_cache*/
  0, /*tp_subclasses*/
  0, /*tp_weaklist*/
  0, /*tp_del*/
  0, /*tp_version_tag*/
  #if CYTHON_USE_TP_FINALIZE
  0, /*tp_finalize*/
  #else
  NULL, /*tp_finalize*/
  #endif
  #if !CYTHON_COMPILING_IN_PYPY || PYPY_VERSION_NUM >= 0x07030800
  0, /*tp_vectorcall*/
  #endif
  #if __PYX_NEED_TP_PRINT_SLOT == 1
  0, /*tp_print*/
  #endif
  #if PY_VERSION_HEX >= 0x030C0000
  0, /*tp_watched*/
  #endif
  #if PY_VERSION_HEX >= 0x030d00A4
  0, /*tp_versions_used*/
  #endif
  #if CYTHON_COMPILING_IN_PYPY && PY_VERSION_HEX >= 0x03090000 && PY_VERSION_HEX < 0x030a0000
  0, /*tp_pypy_flags*/
  #endif
};
#endif

static PyObject *__pyx_tp_new_4cupy_4cuda_6memory_PythonFunctionAllocatorMemory(PyTypeObject *t, PyObject *a, PyObject *k) {
  struct __pyx_obj_4cupy_4cuda_6memory_PythonFunctionAllocatorMemory *p;
  PyObject *o = __pyx_tp_new_4cupy_4cuda_6memory_BaseMemory(t, a, k);
  if (unlikely(!o)) return 0;
  p = ((struct __pyx_obj_4cupy_4cuda_6memory_PythonFunctionAllocatorMemory *)o);
  p->_free_func = Py_None; Py_INCREF(Py_None);
  return o;
}

static void __pyx_tp_dealloc_4cupy_4cuda_6memory_PythonFunctionAllocatorMemory(PyObject *o) {
  struct __pyx_obj_4cupy_4cuda_6memory_PythonFunctionAllocatorMemory *p = (struct __pyx_obj_4cupy_4cuda_6memory_PythonFunctionAllocatorMemory *)o;
  #if CYTHON_USE_TP_FINALIZE
  if (unlikely(__Pyx_PyObject_GetSlot(o, tp_finalize, destructor)) && !__Pyx_PyObject_GC_IsFinalized(o)) {
    if (__Pyx_PyObject_GetSlot(o, tp_dealloc, destructor) == __pyx_tp_dealloc_4cupy_4cuda_6memory_PythonFunctionAllocatorMemory) {
      if (PyObject_CallFinalizerFromDealloc(o)) return;
    }
  }
  #endif
  PyObject_GC_UnTrack(o);
  {
    PyObject *etype, *eval, *etb;
    PyErr_Fetch(&etype, &eval, &etb);
    __Pyx_SET_REFCNT(o, Py_REFCNT(o) + 1);
    __pyx_pw_4cupy_4cuda_6memory_29PythonFunctionAllocatorMemory_3__dealloc__(o);
    __Pyx_SET_REFCNT(o, Py_REFCNT(o) - 1);
    PyErr_Restore(etype, eval, etb);
  }
  Py_CLEAR(p->_free_func);
  __pyx_tp_dealloc_4cupy_4cuda_6memory_BaseMemory(o);
}

static int __pyx_tp_traverse_4cupy_4cuda_6memory_PythonFunctionAllocatorMemory(PyObject *o, visitproc v, void *a) {
  int e;
  struct __pyx_obj_4cupy_4cuda_6memory_PythonFunctionAllocatorMemory *p = (struct __pyx_obj_4cupy_4cuda_6memory_PythonFunctionAllocatorMemory *)o;
  #if !CYTHON_USE_MODULE_STATE
  e = 0;
  if (likely(__pyx_mstate_global->__pyx_ptype_4cupy_4cuda_6memory_BaseMemory)) {
    traverseproc traverse = __Pyx_PyType_GetSlot(__pyx_mstate_global->__pyx_ptype_4cupy_4cuda_6memory_BaseMemory, tp_traverse, traverseproc);
    if (traverse) { e = traverse(o, v, a); }
  } else
  #endif
  { e = __Pyx_call_next_tp_traverse(o, v, a, __pyx_tp_traverse_4cupy_4cuda_6memory_PythonFunctionAllocatorMemory); }
  if (e) return e;
  {
    e = __Pyx_call_type_traverse(o, 0, v, a);
    if (e) return e;
  }
  if (p->_free_func) {
    e = (*v)(p->_free_func, a); if (e) return e;
  }
  return 0;
}

static int __pyx_tp_clear_4cupy_4cuda_6memory_PythonFunctionAllocatorMemory(PyObject *o) {
  PyObject* tmp;
  struct __pyx_obj_4cupy_4cuda_6memory_PythonFunctionAllocatorMemory *p = (struct __pyx_obj_4cupy_4cuda_6memory_PythonFunctionAllocatorMemory *)o;
  #if !CYTHON_USE_MODULE_STATE
  if (likely(__pyx_mstate_global->__pyx_ptype_4cupy_4cuda_6memory_BaseMemory)) {
    inquiry clear = __Pyx_PyType_GetSlot(__pyx_mstate_global->__pyx_ptype_4cupy_4cuda_6memory_BaseMemory, tp_clear, inquiry);
    if (clear) clear(o);
  } else
  #endif
  { __Pyx_call_next_tp_clear(o, __pyx_tp_clear_4cupy_4cuda_6memory_PythonFunctionAllocatorMemory); }
  tmp = ((PyObject*)p->_free_func);
  p->_free_func = Py_None; Py_INCREF(Py_None);
  Py_XDECREF(tmp);
  return 0;
}

static PyMethodDef __pyx_methods_4cupy_4cuda_6memory_PythonFunctionAllocatorMemory[] = {
  {"__reduce_cython__", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_4cupy_4cuda_6memory_29PythonFunctionAllocatorMemory_5__reduce_cython__, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_4cupy_4cuda_6memory_29PythonFunctionAllocatorMemory_4__reduce_cython__},
  {"__setstate_cython__", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_4cupy_4cuda_6memory_29PythonFunctionAllocatorMemory_7__setstate_cython__, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_4cupy_4cuda_6memory_29PythonFunctionAllocatorMemory_6__setstate_cython__},
  {0, 0, 0, 0}
};
#if CYTHON_USE_TYPE_SPECS
static PyType_Slot __pyx_type_4cupy_4cuda_6memory_PythonFunctionAllocatorMemory_slots[] = {
  {Py_tp_dealloc, (void *)__pyx_tp_dealloc_4cupy_4cuda_6memory_PythonFunctionAllocatorMemory},
  {Py_tp_doc, (void *)PyDoc_STR("PythonFunctionAllocatorMemory(size_t size, malloc_func, free_func, int device_id)")},
  {Py_tp_traverse, (void *)__pyx_tp_traverse_4cupy_4cuda_6memory_PythonFunctionAllocatorMemory},
  {Py_tp_clear, (void *)__pyx_tp_clear_4cupy_4cuda_6memory_PythonFunctionAllocatorMemory},
  {Py_tp_methods, (void *)__pyx_methods_4cupy_4cuda_6memory_PythonFunctionAllocatorMemory},
  {Py_tp_init, (void *)__pyx_pw_4cupy_4cuda_6memory_29PythonFunctionAllocatorMemory_1__init__},
  {Py_tp_new, (void *)__pyx_tp_new_4cupy_4cuda_6memory_PythonFunctionAllocatorMemory},
  {0, 0},
};
static PyType_Spec __pyx_type_4cupy_4cuda_6memory_PythonFunctionAllocatorMemory_spec = {
  "cupy.cuda.memory.PythonFunctionAllocatorMemory",
  sizeof(struct __pyx_obj_4cupy_4cuda_6memory_PythonFunctionAllocatorMemory),
  0,
  Py_TPFLAGS_DEFAULT|Py_TPFLAGS_HAVE_VERSION_TAG|Py_TPFLAGS_CHECKTYPES|Py_TPFLAGS_HAVE_NEWBUFFER|Py_TPFLAGS_BASETYPE|Py_TPFLAGS_HAVE_GC,
  __pyx_type_4cupy_4cuda_6memory_PythonFunctionAllocatorMemory_slots,
};
#else

static PyTypeObject __pyx_type_4cupy_4cuda_6memory_PythonFunctionAllocatorMemory = {
  PyVarObject_HEAD_INIT(0, 0)
  "cupy.cuda.memory.""PythonFunctionAllocatorMemory", /*tp_name*/
  sizeof(struct __pyx_obj_4cupy_4cuda_6memory_PythonFunctionAllocatorMemory), /*tp_basicsize*/
  0, /*tp_itemsize*/
  __pyx_tp_dealloc_4cupy_4cuda_6memory_PythonFunctionAllocatorMemory, /*tp_dealloc*/
  0, /*tp_vectorcall_offset*/
  0, /*tp_getattr*/
  0, /*tp_setattr*/
  0, /*tp_as_async*/
  0, /*tp_repr*/
  0, /*tp_as_number*/
  0, /*tp_as_sequence*/
  0, /*tp_as_mapping*/
  0, /*tp_hash*/
  0, /*tp_call*/
  0, /*tp_str*/
  0, /*tp_getattro*/
  0, /*tp_setattro*/
  0, /*tp_as_buffer*/
  Py_TPFLAGS_DEFAULT|Py_TPFLAGS_HAVE_VERSION_TAG|Py_TPFLAGS_CHECKTYPES|Py_TPFLAGS_HAVE_NEWBUFFER|Py_TPFLAGS_BASETYPE|Py_TPFLAGS_HAVE_GC, /*tp_flags*/
  PyDoc_STR("PythonFunctionAllocatorMemory(size_t size, malloc_func, free_func, int device_id)"), /*tp_doc*/
  __pyx_tp_traverse_4cupy_4cuda_6memory_PythonFunctionAllocatorMemory, /*tp_traverse*/
  __pyx_tp_clear_4cupy_4cuda_6memory_PythonFunctionAllocatorMemory, /*tp_clear*/
  0, /*tp_richcompare*/
  0, /*tp_weaklistoffset*/
  0, /*tp_iter*/
  0, /*tp_iternext*/
  __pyx_methods_4cupy_4cuda_6memory_PythonFunctionAllocatorMemory, /*tp_methods*/
  0, /*tp_members*/
  0, /*tp_getset*/
  0, /*tp_base*/
  0, /*tp_dict*/
  0, /*tp_descr_get*/
  0, /*tp_descr_set*/
  #if !CYTHON_USE_TYPE_SPECS
  0, /*tp_dictoffset*/
  #endif
  __pyx_pw_4cupy_4cuda_6memory_29PythonFunctionAllocatorMemory_1__init__, /*tp_init*/
  0, /*tp_alloc*/
  __pyx_tp_new_4cupy_4cuda_6memory_PythonFunctionAllocatorMemory, /*tp_new*/
  0, /*tp_free*/
  0, /*tp_is_gc*/
  0, /*tp_bases*/
  0, /*tp_mro*/
  0, /*tp_cache*/
  0, /*tp_subclasses*/
  0, /*tp_weaklist*/
  0, /*tp_del*/
  0, /*tp_version_tag*/
  #if CYTHON_USE_TP_FINALIZE
  0, /*tp_finalize*/
  #else
  NULL, /*tp_finalize*/
  #endif
  #if !CYTHON_COMPILING_IN_PYPY || PYPY_VERSION_NUM >= 0x07030800
  0, /*tp_vectorcall*/
  #endif
  #if __PYX_NEED_TP_PRINT_SLOT == 1
  0, /*tp_print*/
  #endif
  #if PY_VERSION_HEX >= 0x030C0000
  0, /*tp_watched*/
  #endif
  #if PY_VERSION_HEX >= 0x030d00A4
  0, /*tp_versions_used*/
  #endif
  #if CYTHON_COMPILING_IN_PYPY && PY_VERSION_HEX >= 0x03090000 && PY_VERSION_HEX < 0x030a0000
  0, /*tp_pypy_flags*/
  #endif
};
#endif
static struct __pyx_vtabstruct_4cupy_4cuda_6memory_PythonFunctionAllocator __pyx_vtable_4cupy_4cuda_6memory_PythonFunctionAllocator;

static PyObject *__pyx_tp_new_4cupy_4cuda_6memory_PythonFunctionAllocator(PyTypeObject *t, CYTHON_UNUSED PyObject *a, CYTHON_UNUSED PyObject *k) {
  struct __pyx_obj_4cupy_4cuda_6memory_PythonFunctionAllocator *p;
  PyObject *o;
  o = __Pyx_AllocateExtensionType(t, 0);
  if (unlikely(!o)) return 0;
  p = ((struct __pyx_obj_4cupy_4cuda_6memory_PythonFunctionAllocator *)o);
  p->__pyx_vtab = __pyx_vtabptr_4cupy_4cuda_6memory_PythonFunctionAllocator;
  p->_malloc_func = Py_None; Py_INCREF(Py_None);
  p->_free_func = Py_None; Py_INCREF(Py_None);
  return o;
}

static void __pyx_tp_dealloc_4cupy_4cuda_6memory_PythonFunctionAllocator(PyObject *o) {
  struct __pyx_obj_4cupy_4cuda_6memory_PythonFunctionAllocator *p = (struct __pyx_obj_4cupy_4cuda_6memory_PythonFunctionAllocator *)o;
  #if CYTHON_USE_TP_FINALIZE
  if (unlikely(__Pyx_PyObject_GetSlot(o, tp_finalize, destructor)) && !__Pyx_PyObject_GC_IsFinalized(o)) {
    if (__Pyx_PyObject_GetSlot(o, tp_dealloc, destructor) == __pyx_tp_dealloc_4cupy_4cuda_6memory_PythonFunctionAllocator) {
      if (PyObject_CallFinalizerFromDealloc(o)) return;
    }
  }
  #endif
  PyObject_GC_UnTrack(o);
  Py_CLEAR(p->_malloc_func);
  Py_CLEAR(p->_free_func);
  PyTypeObject *tp = Py_TYPE(o);
  #if CYTHON_USE_TYPE_SLOTS
  (*tp->tp_free)(o);
  #else
  {
    freefunc tp_free = (freefunc)PyType_GetSlot(tp, Py_tp_free);
    if (tp_free) tp_free(o);
  }
  #endif
  #if CYTHON_USE_TYPE_SPECS
  Py_DECREF(tp);
  #endif
}

static int __pyx_tp_traverse_4cupy_4cuda_6memory_PythonFunctionAllocator(PyObject *o, visitproc v, void *a) {
  int e;
  struct __pyx_obj_4cupy_4cuda_6memory_PythonFunctionAllocator *p = (struct __pyx_obj_4cupy_4cuda_6memory_PythonFunctionAllocator *)o;
  {
    e = __Pyx_call_type_traverse(o, 1, v, a);
    if (e) return e;
  }
  if (p->_malloc_func) {
    e = (*v)(p->_malloc_func, a); if (e) return e;
  }
  if (p->_free_func) {
    e = (*v)(p->_free_func, a); if (e) return e;
  }
  return 0;
}

static int __pyx_tp_clear_4cupy_4cuda_6memory_PythonFunctionAllocator(PyObject *o) {
  PyObject* tmp;
  struct __pyx_obj_4cupy_4cuda_6memory_PythonFunctionAllocator *p = (struct __pyx_obj_4cupy_4cuda_6memory_PythonFunctionAllocator *)o;
  tmp = ((PyObject*)p->_malloc_func);
  p->_malloc_func = Py_None; Py_INCREF(Py_None);
  Py_XDECREF(tmp);
  tmp = ((PyObject*)p->_free_func);
  p->_free_func = Py_None; Py_INCREF(Py_None);
  Py_XDECREF(tmp);
  return 0;
}

static PyMethodDef __pyx_methods_4cupy_4cuda_6memory_PythonFunctionAllocator[] = {
  {"malloc", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_4cupy_4cuda_6memory_23PythonFunctionAllocator_3malloc, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_4cupy_4cuda_6memory_23PythonFunctionAllocator_2malloc},
  {"__reduce_cython__", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_4cupy_4cuda_6memory_23PythonFunctionAllocator_5__reduce_cython__, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_4cupy_4cuda_6memory_23PythonFunctionAllocator_4__reduce_cython__},
  {"__setstate_cython__", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_4cupy_4cuda_6memory_23PythonFunctionAllocator_7__setstate_cython__, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_4cupy_4cuda_6memory_23PythonFunctionAllocator_6__setstate_cython__},
  {0, 0, 0, 0}
};
#if CYTHON_USE_TYPE_SPECS
static PyType_Slot __pyx_type_4cupy_4cuda_6memory_PythonFunctionAllocator_slots[] = {
  {Py_tp_dealloc, (void *)__pyx_tp_dealloc_4cupy_4cuda_6memory_PythonFunctionAllocator},
  {Py_tp_doc, (void *)PyDoc_STR("PythonFunctionAllocator(malloc_func, free_func)\n\nAllocator with python functions to perform memory allocation.\n\nThis allocator keeps functions corresponding to *malloc* and *free*,\ndelegating the actual allocation to external sources while only\nhandling the timing of the resource allocation and deallocation.\n\n*malloc* should follow the signature ``malloc(int, int) -> int``\nreturning the pointer to the allocated memory given the *param* object,\nthe number of bytes to allocate and the device id on which the\nallocation should take place.\n\nSimilarly, *free* should follow the signature\n``free(int, int)`` with no return, taking the pointer to the\nallocated memory and the device id on which the memory was allocated.\n\nIf the external memory management supports asynchronous operations,\nthe current CuPy stream can be retrieved inside ``malloc_func`` and\n``free_func`` by calling :func:`cupy.cuda.get_current_stream()`. To\nuse external streams, wrap them with :func:`cupy.cuda.ExternalStream`.\n\nArgs:\n    malloc_func (function): *malloc* function to be called.\n    free_func (function): *free* function to be called.")},
  {Py_tp_traverse, (void *)__pyx_tp_traverse_4cupy_4cuda_6memory_PythonFunctionAllocator},
  {Py_tp_clear, (void *)__pyx_tp_clear_4cupy_4cuda_6memory_PythonFunctionAllocator},
  {Py_tp_methods, (void *)__pyx_methods_4cupy_4cuda_6memory_PythonFunctionAllocator},
  {Py_tp_init, (void *)__pyx_pw_4cupy_4cuda_6memory_23PythonFunctionAllocator_1__init__},
  {Py_tp_new, (void *)__pyx_tp_new_4cupy_4cuda_6memory_PythonFunctionAllocator},
  {0, 0},
};
static PyType_Spec __pyx_type_4cupy_4cuda_6memory_PythonFunctionAllocator_spec = {
  "cupy.cuda.memory.PythonFunctionAllocator",
  sizeof(struct __pyx_obj_4cupy_4cuda_6memory_PythonFunctionAllocator),
  0,
  Py_TPFLAGS_DEFAULT|Py_TPFLAGS_HAVE_VERSION_TAG|Py_TPFLAGS_CHECKTYPES|Py_TPFLAGS_HAVE_NEWBUFFER|Py_TPFLAGS_BASETYPE|Py_TPFLAGS_HAVE_GC,
  __pyx_type_4cupy_4cuda_6memory_PythonFunctionAllocator_slots,
};
#else

static PyTypeObject __pyx_type_4cupy_4cuda_6memory_PythonFunctionAllocator = {
  PyVarObject_HEAD_INIT(0, 0)
  "cupy.cuda.memory.""PythonFunctionAllocator", /*tp_name*/
  sizeof(struct __pyx_obj_4cupy_4cuda_6memory_PythonFunctionAllocator), /*tp_basicsize*/
  0, /*tp_itemsize*/
  __pyx_tp_dealloc_4cupy_4cuda_6memory_PythonFunctionAllocator, /*tp_dealloc*/
  0, /*tp_vectorcall_offset*/
  0, /*tp_getattr*/
  0, /*tp_setattr*/
  0, /*tp_as_async*/
  0, /*tp_repr*/
  0, /*tp_as_number*/
  0, /*tp_as_sequence*/
  0, /*tp_as_mapping*/
  0, /*tp_hash*/
  0, /*tp_call*/
  0, /*tp_str*/
  0, /*tp_getattro*/
  0, /*tp_setattro*/
  0, /*tp_as_buffer*/
  Py_TPFLAGS_DEFAULT|Py_TPFLAGS_HAVE_VERSION_TAG|Py_TPFLAGS_CHECKTYPES|Py_TPFLAGS_HAVE_NEWBUFFER|Py_TPFLAGS_BASETYPE|Py_TPFLAGS_HAVE_GC, /*tp_flags*/
  PyDoc_STR("PythonFunctionAllocator(malloc_func, free_func)\n\nAllocator with python functions to perform memory allocation.\n\nThis allocator keeps functions corresponding to *malloc* and *free*,\ndelegating the actual allocation to external sources while only\nhandling the timing of the resource allocation and deallocation.\n\n*malloc* should follow the signature ``malloc(int, int) -> int``\nreturning the pointer to the allocated memory given the *param* object,\nthe number of bytes to allocate and the device id on which the\nallocation should take place.\n\nSimilarly, *free* should follow the signature\n``free(int, int)`` with no return, taking the pointer to the\nallocated memory and the device id on which the memory was allocated.\n\nIf the external memory management supports asynchronous operations,\nthe current CuPy stream can be retrieved inside ``malloc_func`` and\n``free_func`` by calling :func:`cupy.cuda.get_current_stream()`. To\nuse external streams, wrap them with :func:`cupy.cuda.ExternalStream`.\n\nArgs:\n    malloc_func (function): *malloc* function to be called.\n    free_func (function): *free* function to be called."), /*tp_doc*/
  __pyx_tp_traverse_4cupy_4cuda_6memory_PythonFunctionAllocator, /*tp_traverse*/
  __pyx_tp_clear_4cupy_4cuda_6memory_PythonFunctionAllocator, /*tp_clear*/
  0, /*tp_richcompare*/
  0, /*tp_weaklistoffset*/
  0, /*tp_iter*/
  0, /*tp_iternext*/
  __pyx_methods_4cupy_4cuda_6memory_PythonFunctionAllocator, /*tp_methods*/
  0, /*tp_members*/
  0, /*tp_getset*/
  0, /*tp_base*/
  0, /*tp_dict*/
  0, /*tp_descr_get*/
  0, /*tp_descr_set*/
  #if !CYTHON_USE_TYPE_SPECS
  0, /*tp_dictoffset*/
  #endif
  __pyx_pw_4cupy_4cuda_6memory_23PythonFunctionAllocator_1__init__, /*tp_init*/
  0, /*tp_alloc*/
  __pyx_tp_new_4cupy_4cuda_6memory_PythonFunctionAllocator, /*tp_new*/
  0, /*tp_free*/
  0, /*tp_is_gc*/
  0, /*tp_bases*/
  0, /*tp_mro*/
  0, /*tp_cache*/
  0, /*tp_subclasses*/
  0, /*tp_weaklist*/
  0, /*tp_del*/
  0, /*tp_version_tag*/
  #if CYTHON_USE_TP_FINALIZE
  0, /*tp_finalize*/
  #else
  NULL, /*tp_finalize*/
  #endif
  #if !CYTHON_COMPILING_IN_PYPY || PYPY_VERSION_NUM >= 0x07030800
  0, /*tp_vectorcall*/
  #endif
  #if __PYX_NEED_TP_PRINT_SLOT == 1
  0, /*tp_print*/
  #endif
  #if PY_VERSION_HEX >= 0x030C0000
  0, /*tp_watched*/
  #endif
  #if PY_VERSION_HEX >= 0x030d00A4
  0, /*tp_versions_used*/
  #endif
  #if CYTHON_COMPILING_IN_PYPY && PY_VERSION_HEX >= 0x03090000 && PY_VERSION_HEX < 0x030a0000
  0, /*tp_pypy_flags*/
  #endif
};
#endif

static void __pyx_tp_dealloc_4cupy_4cuda_6memory_Memory(PyObject *o) {
  #if CYTHON_USE_TP_FINALIZE
  if (unlikely(__Pyx_PyObject_GetSlot(o, tp_finalize, destructor)) && (!PyType_IS_GC(Py_TYPE(o)) || !__Pyx_PyObject_GC_IsFinalized(o))) {
    if (__Pyx_PyObject_GetSlot(o, tp_dealloc, destructor) == __pyx_tp_dealloc_4cupy_4cuda_6memory_Memory) {
      if (PyObject_CallFinalizerFromDealloc(o)) return;
    }
  }
  #endif
  {
    PyObject *etype, *eval, *etb;
    PyErr_Fetch(&etype, &eval, &etb);
    __Pyx_SET_REFCNT(o, Py_REFCNT(o) + 1);
    __pyx_pw_4cupy_4cuda_6memory_6Memory_3__dealloc__(o);
    __Pyx_SET_REFCNT(o, Py_REFCNT(o) - 1);
    PyErr_Restore(etype, eval, etb);
  }
  __pyx_tp_dealloc_4cupy_4cuda_6memory_BaseMemory(o);
}

static PyMethodDef __pyx_methods_4cupy_4cuda_6memory_Memory[] = {
  {"__reduce_cython__", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_4cupy_4cuda_6memory_6Memory_5__reduce_cython__, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_4cupy_4cuda_6memory_6Memory_4__reduce_cython__},
  {"__setstate_cython__", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_4cupy_4cuda_6memory_6Memory_7__setstate_cython__, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_4cupy_4cuda_6memory_6Memory_6__setstate_cython__},
  {0, 0, 0, 0}
};
#if CYTHON_USE_TYPE_SPECS
static PyType_Slot __pyx_type_4cupy_4cuda_6memory_Memory_slots[] = {
  {Py_tp_dealloc, (void *)__pyx_tp_dealloc_4cupy_4cuda_6memory_Memory},
  {Py_tp_doc, (void *)PyDoc_STR("Memory(size_t size)\n\nMemory allocation on a CUDA device.\n\nThis class provides an RAII interface of the CUDA memory allocation.\n\nArgs:\n    size (int): Size of the memory allocation in bytes.")},
  {Py_tp_methods, (void *)__pyx_methods_4cupy_4cuda_6memory_Memory},
  {Py_tp_init, (void *)__pyx_pw_4cupy_4cuda_6memory_6Memory_1__init__},
  {Py_tp_new, (void *)__pyx_tp_new_4cupy_4cuda_6memory_BaseMemory},
  {0, 0},
};
static PyType_Spec __pyx_type_4cupy_4cuda_6memory_Memory_spec = {
  "cupy.cuda.memory.Memory",
  sizeof(struct __pyx_obj_4cupy_4cuda_6memory_Memory),
  0,
  Py_TPFLAGS_DEFAULT|Py_TPFLAGS_HAVE_VERSION_TAG|Py_TPFLAGS_CHECKTYPES|Py_TPFLAGS_HAVE_NEWBUFFER|Py_TPFLAGS_BASETYPE,
  __pyx_type_4cupy_4cuda_6memory_Memory_slots,
};
#else

static PyTypeObject __pyx_type_4cupy_4cuda_6memory_Memory = {
  PyVarObject_HEAD_INIT(0, 0)
  "cupy.cuda.memory.""Memory", /*tp_name*/
  sizeof(struct __pyx_obj_4cupy_4cuda_6memory_Memory), /*tp_basicsize*/
  0, /*tp_itemsize*/
  __pyx_tp_dealloc_4cupy_4cuda_6memory_Memory, /*tp_dealloc*/
  0, /*tp_vectorcall_offset*/
  0, /*tp_getattr*/
  0, /*tp_setattr*/
  0, /*tp_as_async*/
  0, /*tp_repr*/
  0, /*tp_as_number*/
  0, /*tp_as_sequence*/
  0, /*tp_as_mapping*/
  0, /*tp_hash*/
  0, /*tp_call*/
  0, /*tp_str*/
  0, /*tp_getattro*/
  0, /*tp_setattro*/
  0, /*tp_as_buffer*/
  Py_TPFLAGS_DEFAULT|Py_TPFLAGS_HAVE_VERSION_TAG|Py_TPFLAGS_CHECKTYPES|Py_TPFLAGS_HAVE_NEWBUFFER|Py_TPFLAGS_BASETYPE, /*tp_flags*/
  PyDoc_STR("Memory(size_t size)\n\nMemory allocation on a CUDA device.\n\nThis class provides an RAII interface of the CUDA memory allocation.\n\nArgs:\n    size (int): Size of the memory allocation in bytes."), /*tp_doc*/
  0, /*tp_traverse*/
  0, /*tp_clear*/
  0, /*tp_richcompare*/
  0, /*tp_weaklistoffset*/
  0, /*tp_iter*/
  0, /*tp_iternext*/
  __pyx_methods_4cupy_4cuda_6memory_Memory, /*tp_methods*/
  0, /*tp_members*/
  0, /*tp_getset*/
  0, /*tp_base*/
  0, /*tp_dict*/
  0, /*tp_descr_get*/
  0, /*tp_descr_set*/
  #if !CYTHON_USE_TYPE_SPECS
  0, /*tp_dictoffset*/
  #endif
  __pyx_pw_4cupy_4cuda_6memory_6Memory_1__init__, /*tp_init*/
  0, /*tp_alloc*/
  __pyx_tp_new_4cupy_4cuda_6memory_BaseMemory, /*tp_new*/
  0, /*tp_free*/
  0, /*tp_is_gc*/
  0, /*tp_bases*/
  0, /*tp_mro*/
  0, /*tp_cache*/
  0, /*tp_subclasses*/
  0, /*tp_weaklist*/
  0, /*tp_del*/
  0, /*tp_version_tag*/
  #if CYTHON_USE_TP_FINALIZE
  0, /*tp_finalize*/
  #else
  NULL, /*tp_finalize*/
  #endif
  #if !CYTHON_COMPILING_IN_PYPY || PYPY_VERSION_NUM >= 0x07030800
  0, /*tp_vectorcall*/
  #endif
  #if __PYX_NEED_TP_PRINT_SLOT == 1
  0, /*tp_print*/
  #endif
  #if PY_VERSION_HEX >= 0x030C0000
  0, /*tp_watched*/
  #endif
  #if PY_VERSION_HEX >= 0x030d00A4
  0, /*tp_versions_used*/
  #endif
  #if CYTHON_COMPILING_IN_PYPY && PY_VERSION_HEX >= 0x03090000 && PY_VERSION_HEX < 0x030a0000
  0, /*tp_pypy_flags*/
  #endif
};
#endif

static PyObject *__pyx_tp_new_4cupy_4cuda_6memory_MemoryAsync(PyTypeObject *t, PyObject *a, PyObject *k) {
  struct __pyx_obj_4cupy_4cuda_6memory_MemoryAsync *p;
  PyObject *o = __pyx_tp_new_4cupy_4cuda_6memory_BaseMemory(t, a, k);
  if (unlikely(!o)) return 0;
  p = ((struct __pyx_obj_4cupy_4cuda_6memory_MemoryAsync *)o);
  p->stream_ref = Py_None; Py_INCREF(Py_None);
  return o;
}

static void __pyx_tp_dealloc_4cupy_4cuda_6memory_MemoryAsync(PyObject *o) {
  struct __pyx_obj_4cupy_4cuda_6memory_MemoryAsync *p = (struct __pyx_obj_4cupy_4cuda_6memory_MemoryAsync *)o;
  #if CYTHON_USE_TP_FINALIZE
  if (unlikely(__Pyx_PyObject_GetSlot(o, tp_finalize, destructor)) && (!PyType_IS_GC(Py_TYPE(o)) || !__Pyx_PyObject_GC_IsFinalized(o))) {
    if (__Pyx_PyObject_GetSlot(o, tp_dealloc, destructor) == __pyx_tp_dealloc_4cupy_4cuda_6memory_MemoryAsync) {
      if (PyObject_CallFinalizerFromDealloc(o)) return;
    }
  }
  #endif
  {
    PyObject *etype, *eval, *etb;
    PyErr_Fetch(&etype, &eval, &etb);
    __Pyx_SET_REFCNT(o, Py_REFCNT(o) + 1);
    __pyx_pw_4cupy_4cuda_6memory_11MemoryAsync_3__dealloc__(o);
    __Pyx_SET_REFCNT(o, Py_REFCNT(o) - 1);
    PyErr_Restore(etype, eval, etb);
  }
  Py_CLEAR(p->stream_ref);
  __pyx_tp_dealloc_4cupy_4cuda_6memory_BaseMemory(o);
}

static PyObject *__pyx_getprop_4cupy_4cuda_6memory_11MemoryAsync_stream_ref(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_4cupy_4cuda_6memory_11MemoryAsync_10stream_ref_1__get__(o);
}

static PyMethodDef __pyx_methods_4cupy_4cuda_6memory_MemoryAsync[] = {
  {"__reduce_cython__", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_4cupy_4cuda_6memory_11MemoryAsync_5__reduce_cython__, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_4cupy_4cuda_6memory_11MemoryAsync_4__reduce_cython__},
  {"__setstate_cython__", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_4cupy_4cuda_6memory_11MemoryAsync_7__setstate_cython__, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_4cupy_4cuda_6memory_11MemoryAsync_6__setstate_cython__},
  {0, 0, 0, 0}
};

static struct PyGetSetDef __pyx_getsets_4cupy_4cuda_6memory_MemoryAsync[] = {
  {"stream_ref", __pyx_getprop_4cupy_4cuda_6memory_11MemoryAsync_stream_ref, 0, 0, 0},
  {0, 0, 0, 0, 0}
};
#if CYTHON_USE_TYPE_SPECS
static PyType_Slot __pyx_type_4cupy_4cuda_6memory_MemoryAsync_slots[] = {
  {Py_tp_dealloc, (void *)__pyx_tp_dealloc_4cupy_4cuda_6memory_MemoryAsync},
  {Py_tp_doc, (void *)PyDoc_STR("MemoryAsync(size_t size, stream)\n\nAsynchronous memory allocation on a CUDA device.\n\nThis class provides an RAII interface of the CUDA memory allocation.\n\nArgs:\n    size (int): Size of the memory allocation in bytes.\n    stream (Stream): The stream on which the memory is allocated and freed.")},
  {Py_tp_methods, (void *)__pyx_methods_4cupy_4cuda_6memory_MemoryAsync},
  {Py_tp_getset, (void *)__pyx_getsets_4cupy_4cuda_6memory_MemoryAsync},
  {Py_tp_init, (void *)__pyx_pw_4cupy_4cuda_6memory_11MemoryAsync_1__init__},
  {Py_tp_new, (void *)__pyx_tp_new_4cupy_4cuda_6memory_MemoryAsync},
  {0, 0},
};
static PyType_Spec __pyx_type_4cupy_4cuda_6memory_MemoryAsync_spec = {
  "cupy.cuda.memory.MemoryAsync",
  sizeof(struct __pyx_obj_4cupy_4cuda_6memory_MemoryAsync),
  0,
  Py_TPFLAGS_DEFAULT|Py_TPFLAGS_HAVE_VERSION_TAG|Py_TPFLAGS_CHECKTYPES|Py_TPFLAGS_HAVE_NEWBUFFER|Py_TPFLAGS_BASETYPE,
  __pyx_type_4cupy_4cuda_6memory_MemoryAsync_slots,
};
#else

static PyTypeObject __pyx_type_4cupy_4cuda_6memory_MemoryAsync = {
  PyVarObject_HEAD_INIT(0, 0)
  "cupy.cuda.memory.""MemoryAsync", /*tp_name*/
  sizeof(struct __pyx_obj_4cupy_4cuda_6memory_MemoryAsync), /*tp_basicsize*/
  0, /*tp_itemsize*/
  __pyx_tp_dealloc_4cupy_4cuda_6memory_MemoryAsync, /*tp_dealloc*/
  0, /*tp_vectorcall_offset*/
  0, /*tp_getattr*/
  0, /*tp_setattr*/
  0, /*tp_as_async*/
  0, /*tp_repr*/
  0, /*tp_as_number*/
  0, /*tp_as_sequence*/
  0, /*tp_as_mapping*/
  0, /*tp_hash*/
  0, /*tp_call*/
  0, /*tp_str*/
  0, /*tp_getattro*/
  0, /*tp_setattro*/
  0, /*tp_as_buffer*/
  Py_TPFLAGS_DEFAULT|Py_TPFLAGS_HAVE_VERSION_TAG|Py_TPFLAGS_CHECKTYPES|Py_TPFLAGS_HAVE_NEWBUFFER|Py_TPFLAGS_BASETYPE, /*tp_flags*/
  PyDoc_STR("MemoryAsync(size_t size, stream)\n\nAsynchronous memory allocation on a CUDA device.\n\nThis class provides an RAII interface of the CUDA memory allocation.\n\nArgs:\n    size (int): Size of the memory allocation in bytes.\n    stream (Stream): The stream on which the memory is allocated and freed."), /*tp_doc*/
  0, /*tp_traverse*/
  0, /*tp_clear*/
  0, /*tp_richcompare*/
  0, /*tp_weaklistoffset*/
  0, /*tp_iter*/
  0, /*tp_iternext*/
  __pyx_methods_4cupy_4cuda_6memory_MemoryAsync, /*tp_methods*/
  0, /*tp_members*/
  __pyx_getsets_4cupy_4cuda_6memory_MemoryAsync, /*tp_getset*/
  0, /*tp_base*/
  0, /*tp_dict*/
  0, /*tp_descr_get*/
  0, /*tp_descr_set*/
  #if !CYTHON_USE_TYPE_SPECS
  0, /*tp_dictoffset*/
  #endif
  __pyx_pw_4cupy_4cuda_6memory_11MemoryAsync_1__init__, /*tp_init*/
  0, /*tp_alloc*/
  __pyx_tp_new_4cupy_4cuda_6memory_MemoryAsync, /*tp_new*/
  0, /*tp_free*/
  0, /*tp_is_gc*/
  0, /*tp_bases*/
  0, /*tp_mro*/
  0, /*tp_cache*/
  0, /*tp_subclasses*/
  0, /*tp_weaklist*/
  0, /*tp_del*/
  0, /*tp_version_tag*/
  #if CYTHON_USE_TP_FINALIZE
  0, /*tp_finalize*/
  #else
  NULL, /*tp_finalize*/
  #endif
  #if !CYTHON_COMPILING_IN_PYPY || PYPY_VERSION_NUM >= 0x07030800
  0, /*tp_vectorcall*/
  #endif
  #if __PYX_NEED_TP_PRINT_SLOT == 1
  0, /*tp_print*/
  #endif
  #if PY_VERSION_HEX >= 0x030C0000
  0, /*tp_watched*/
  #endif
  #if PY_VERSION_HEX >= 0x030d00A4
  0, /*tp_versions_used*/
  #endif
  #if CYTHON_COMPILING_IN_PYPY && PY_VERSION_HEX >= 0x03090000 && PY_VERSION_HEX < 0x030a0000
  0, /*tp_pypy_flags*/
  #endif
};
#endif

static PyObject *__pyx_tp_new_4cupy_4cuda_6memory_UnownedMemory(PyTypeObject *t, PyObject *a, PyObject *k) {
  struct __pyx_obj_4cupy_4cuda_6memory_UnownedMemory *p;
  PyObject *o = __pyx_tp_new_4cupy_4cuda_6memory_BaseMemory(t, a, k);
  if (unlikely(!o)) return 0;
  p = ((struct __pyx_obj_4cupy_4cuda_6memory_UnownedMemory *)o);
  p->_owner = Py_None; Py_INCREF(Py_None);
  return o;
}

static void __pyx_tp_dealloc_4cupy_4cuda_6memory_UnownedMemory(PyObject *o) {
  struct __pyx_obj_4cupy_4cuda_6memory_UnownedMemory *p = (struct __pyx_obj_4cupy_4cuda_6memory_UnownedMemory *)o;
  #if CYTHON_USE_TP_FINALIZE
  if (unlikely(__Pyx_PyObject_GetSlot(o, tp_finalize, destructor)) && !__Pyx_PyObject_GC_IsFinalized(o)) {
    if (__Pyx_PyObject_GetSlot(o, tp_dealloc, destructor) == __pyx_tp_dealloc_4cupy_4cuda_6memory_UnownedMemory) {
      if (PyObject_CallFinalizerFromDealloc(o)) return;
    }
  }
  #endif
  PyObject_GC_UnTrack(o);
  Py_CLEAR(p->_owner);
  __pyx_tp_dealloc_4cupy_4cuda_6memory_BaseMemory(o);
}

static int __pyx_tp_traverse_4cupy_4cuda_6memory_UnownedMemory(PyObject *o, visitproc v, void *a) {
  int e;
  struct __pyx_obj_4cupy_4cuda_6memory_UnownedMemory *p = (struct __pyx_obj_4cupy_4cuda_6memory_UnownedMemory *)o;
  #if !CYTHON_USE_MODULE_STATE
  e = 0;
  if (likely(__pyx_mstate_global->__pyx_ptype_4cupy_4cuda_6memory_BaseMemory)) {
    traverseproc traverse = __Pyx_PyType_GetSlot(__pyx_mstate_global->__pyx_ptype_4cupy_4cuda_6memory_BaseMemory, tp_traverse, traverseproc);
    if (traverse) { e = traverse(o, v, a); }
  } else
  #endif
  { e = __Pyx_call_next_tp_traverse(o, v, a, __pyx_tp_traverse_4cupy_4cuda_6memory_UnownedMemory); }
  if (e) return e;
  {
    e = __Pyx_call_type_traverse(o, 0, v, a);
    if (e) return e;
  }
  if (p->_owner) {
    e = (*v)(p->_owner, a); if (e) return e;
  }
  return 0;
}

static int __pyx_tp_clear_4cupy_4cuda_6memory_UnownedMemory(PyObject *o) {
  PyObject* tmp;
  struct __pyx_obj_4cupy_4cuda_6memory_UnownedMemory *p = (struct __pyx_obj_4cupy_4cuda_6memory_UnownedMemory *)o;
  #if !CYTHON_USE_MODULE_STATE
  if (likely(__pyx_mstate_global->__pyx_ptype_4cupy_4cuda_6memory_BaseMemory)) {
    inquiry clear = __Pyx_PyType_GetSlot(__pyx_mstate_global->__pyx_ptype_4cupy_4cuda_6memory_BaseMemory, tp_clear, inquiry);
    if (clear) clear(o);
  } else
  #endif
  { __Pyx_call_next_tp_clear(o, __pyx_tp_clear_4cupy_4cuda_6memory_UnownedMemory); }
  tmp = ((PyObject*)p->_owner);
  p->_owner = Py_None; Py_INCREF(Py_None);
  Py_XDECREF(tmp);
  return 0;
}

static PyObject *__pyx_getprop_4cupy_4cuda_6memory_13UnownedMemory__owner(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_4cupy_4cuda_6memory_13UnownedMemory_6_owner_1__get__(o);
}

static PyMethodDef __pyx_methods_4cupy_4cuda_6memory_UnownedMemory[] = {
  {"__reduce_cython__", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_4cupy_4cuda_6memory_13UnownedMemory_3__reduce_cython__, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_4cupy_4cuda_6memory_13UnownedMemory_2__reduce_cython__},
  {"__setstate_cython__", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_4cupy_4cuda_6memory_13UnownedMemory_5__setstate_cython__, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_4cupy_4cuda_6memory_13UnownedMemory_4__setstate_cython__},
  {0, 0, 0, 0}
};

static struct PyGetSetDef __pyx_getsets_4cupy_4cuda_6memory_UnownedMemory[] = {
  {"_owner", __pyx_getprop_4cupy_4cuda_6memory_13UnownedMemory__owner, 0, 0, 0},
  {0, 0, 0, 0, 0}
};
#if CYTHON_USE_TYPE_SPECS
static PyType_Slot __pyx_type_4cupy_4cuda_6memory_UnownedMemory_slots[] = {
  {Py_tp_dealloc, (void *)__pyx_tp_dealloc_4cupy_4cuda_6memory_UnownedMemory},
  {Py_tp_doc, (void *)PyDoc_STR("UnownedMemory(intptr_t ptr, size_t size, owner, int device_id=-1)\n\nCUDA memory that is not owned by CuPy.\n\nArgs:\n    ptr (int): Pointer to the buffer.\n    size (int): Size of the buffer.\n    owner (object): Reference to the owner object to keep the memory\n        alive.\n    device_id (int): CUDA device ID of the buffer. If omitted, the device\n        associated to the pointer is retrieved.")},
  {Py_tp_traverse, (void *)__pyx_tp_traverse_4cupy_4cuda_6memory_UnownedMemory},
  {Py_tp_clear, (void *)__pyx_tp_clear_4cupy_4cuda_6memory_UnownedMemory},
  {Py_tp_methods, (void *)__pyx_methods_4cupy_4cuda_6memory_UnownedMemory},
  {Py_tp_getset, (void *)__pyx_getsets_4cupy_4cuda_6memory_UnownedMemory},
  {Py_tp_init, (void *)__pyx_pw_4cupy_4cuda_6memory_13UnownedMemory_1__init__},
  {Py_tp_new, (void *)__pyx_tp_new_4cupy_4cuda_6memory_UnownedMemory},
  {0, 0},
};
static PyType_Spec __pyx_type_4cupy_4cuda_6memory_UnownedMemory_spec = {
  "cupy.cuda.memory.UnownedMemory",
  sizeof(struct __pyx_obj_4cupy_4cuda_6memory_UnownedMemory),
  0,
  Py_TPFLAGS_DEFAULT|Py_TPFLAGS_HAVE_VERSION_TAG|Py_TPFLAGS_CHECKTYPES|Py_TPFLAGS_HAVE_NEWBUFFER|Py_TPFLAGS_BASETYPE|Py_TPFLAGS_HAVE_GC,
  __pyx_type_4cupy_4cuda_6memory_UnownedMemory_slots,
};
#else

static PyTypeObject __pyx_type_4cupy_4cuda_6memory_UnownedMemory = {
  PyVarObject_HEAD_INIT(0, 0)
  "cupy.cuda.memory.""UnownedMemory", /*tp_name*/
  sizeof(struct __pyx_obj_4cupy_4cuda_6memory_UnownedMemory), /*tp_basicsize*/
  0, /*tp_itemsize*/
  __pyx_tp_dealloc_4cupy_4cuda_6memory_UnownedMemory, /*tp_dealloc*/
  0, /*tp_vectorcall_offset*/
  0, /*tp_getattr*/
  0, /*tp_setattr*/
  0, /*tp_as_async*/
  0, /*tp_repr*/
  0, /*tp_as_number*/
  0, /*tp_as_sequence*/
  0, /*tp_as_mapping*/
  0, /*tp_hash*/
  0, /*tp_call*/
  0, /*tp_str*/
  0, /*tp_getattro*/
  0, /*tp_setattro*/
  0, /*tp_as_buffer*/
  Py_TPFLAGS_DEFAULT|Py_TPFLAGS_HAVE_VERSION_TAG|Py_TPFLAGS_CHECKTYPES|Py_TPFLAGS_HAVE_NEWBUFFER|Py_TPFLAGS_BASETYPE|Py_TPFLAGS_HAVE_GC, /*tp_flags*/
  PyDoc_STR("UnownedMemory(intptr_t ptr, size_t size, owner, int device_id=-1)\n\nCUDA memory that is not owned by CuPy.\n\nArgs:\n    ptr (int): Pointer to the buffer.\n    size (int): Size of the buffer.\n    owner (object): Reference to the owner object to keep the memory\n        alive.\n    device_id (int): CUDA device ID of the buffer. If omitted, the device\n        associated to the pointer is retrieved."), /*tp_doc*/
  __pyx_tp_traverse_4cupy_4cuda_6memory_UnownedMemory, /*tp_traverse*/
  __pyx_tp_clear_4cupy_4cuda_6memory_UnownedMemory, /*tp_clear*/
  0, /*tp_richcompare*/
  0, /*tp_weaklistoffset*/
  0, /*tp_iter*/
  0, /*tp_iternext*/
  __pyx_methods_4cupy_4cuda_6memory_UnownedMemory, /*tp_methods*/
  0, /*tp_members*/
  __pyx_getsets_4cupy_4cuda_6memory_UnownedMemory, /*tp_getset*/
  0, /*tp_base*/
  0, /*tp_dict*/
  0, /*tp_descr_get*/
  0, /*tp_descr_set*/
  #if !CYTHON_USE_TYPE_SPECS
  0, /*tp_dictoffset*/
  #endif
  __pyx_pw_4cupy_4cuda_6memory_13UnownedMemory_1__init__, /*tp_init*/
  0, /*tp_alloc*/
  __pyx_tp_new_4cupy_4cuda_6memory_UnownedMemory, /*tp_new*/
  0, /*tp_free*/
  0, /*tp_is_gc*/
  0, /*tp_bases*/
  0, /*tp_mro*/
  0, /*tp_cache*/
  0, /*tp_subclasses*/
  0, /*tp_weaklist*/
  0, /*tp_del*/
  0, /*tp_version_tag*/
  #if CYTHON_USE_TP_FINALIZE
  0, /*tp_finalize*/
  #else
  NULL, /*tp_finalize*/
  #endif
  #if !CYTHON_COMPILING_IN_PYPY || PYPY_VERSION_NUM >= 0x07030800
  0, /*tp_vectorcall*/
  #endif
  #if __PYX_NEED_TP_PRINT_SLOT == 1
  0, /*tp_print*/
  #endif
  #if PY_VERSION_HEX >= 0x030C0000
  0, /*tp_watched*/
  #endif
  #if PY_VERSION_HEX >= 0x030d00A4
  0, /*tp_versions_used*/
  #endif
  #if CYTHON_COMPILING_IN_PYPY && PY_VERSION_HEX >= 0x03090000 && PY_VERSION_HEX < 0x030a0000
  0, /*tp_pypy_flags*/
  #endif
};
#endif

static void __pyx_tp_dealloc_4cupy_4cuda_6memory_ManagedMemory(PyObject *o) {
  #if CYTHON_USE_TP_FINALIZE
  if (unlikely(__Pyx_PyObject_GetSlot(o, tp_finalize, destructor)) && (!PyType_IS_GC(Py_TYPE(o)) || !__Pyx_PyObject_GC_IsFinalized(o))) {
    if (__Pyx_PyObject_GetSlot(o, tp_dealloc, destructor) == __pyx_tp_dealloc_4cupy_4cuda_6memory_ManagedMemory) {
      if (PyObject_CallFinalizerFromDealloc(o)) return;
    }
  }
  #endif
  {
    PyObject *etype, *eval, *etb;
    PyErr_Fetch(&etype, &eval, &etb);
    __Pyx_SET_REFCNT(o, Py_REFCNT(o) + 1);
    __pyx_pw_4cupy_4cuda_6memory_13ManagedMemory_7__dealloc__(o);
    __Pyx_SET_REFCNT(o, Py_REFCNT(o) - 1);
    PyErr_Restore(etype, eval, etb);
  }
  __pyx_tp_dealloc_4cupy_4cuda_6memory_BaseMemory(o);
}

static PyMethodDef __pyx_methods_4cupy_4cuda_6memory_ManagedMemory[] = {
  {"prefetch", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_4cupy_4cuda_6memory_13ManagedMemory_3prefetch, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_4cupy_4cuda_6memory_13ManagedMemory_2prefetch},
  {"advise", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_4cupy_4cuda_6memory_13ManagedMemory_5advise, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_4cupy_4cuda_6memory_13ManagedMemory_4advise},
  {"__reduce_cython__", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_4cupy_4cuda_6memory_13ManagedMemory_9__reduce_cython__, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_4cupy_4cuda_6memory_13ManagedMemory_8__reduce_cython__},
  {"__setstate_cython__", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_4cupy_4cuda_6memory_13ManagedMemory_11__setstate_cython__, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_4cupy_4cuda_6memory_13ManagedMemory_10__setstate_cython__},
  {0, 0, 0, 0}
};
#if CYTHON_USE_TYPE_SPECS
static PyType_Slot __pyx_type_4cupy_4cuda_6memory_ManagedMemory_slots[] = {
  {Py_tp_dealloc, (void *)__pyx_tp_dealloc_4cupy_4cuda_6memory_ManagedMemory},
  {Py_tp_doc, (void *)PyDoc_STR("ManagedMemory(size_t size)\n\nManaged memory (Unified memory) allocation on a CUDA device.\n\nThis class provides an RAII interface of the CUDA managed memory\nallocation.\n\nArgs:\n    size (int): Size of the memory allocation in bytes.")},
  {Py_tp_methods, (void *)__pyx_methods_4cupy_4cuda_6memory_ManagedMemory},
  {Py_tp_init, (void *)__pyx_pw_4cupy_4cuda_6memory_13ManagedMemory_1__init__},
  {Py_tp_new, (void *)__pyx_tp_new_4cupy_4cuda_6memory_BaseMemory},
  {0, 0},
};
static PyType_Spec __pyx_type_4cupy_4cuda_6memory_ManagedMemory_spec = {
  "cupy.cuda.memory.ManagedMemory",
  sizeof(struct __pyx_obj_4cupy_4cuda_6memory_ManagedMemory),
  0,
  Py_TPFLAGS_DEFAULT|Py_TPFLAGS_HAVE_VERSION_TAG|Py_TPFLAGS_CHECKTYPES|Py_TPFLAGS_HAVE_NEWBUFFER|Py_TPFLAGS_BASETYPE,
  __pyx_type_4cupy_4cuda_6memory_ManagedMemory_slots,
};
#else

static PyTypeObject __pyx_type_4cupy_4cuda_6memory_ManagedMemory = {
  PyVarObject_HEAD_INIT(0, 0)
  "cupy.cuda.memory.""ManagedMemory", /*tp_name*/
  sizeof(struct __pyx_obj_4cupy_4cuda_6memory_ManagedMemory), /*tp_basicsize*/
  0, /*tp_itemsize*/
  __pyx_tp_dealloc_4cupy_4cuda_6memory_ManagedMemory, /*tp_dealloc*/
  0, /*tp_vectorcall_offset*/
  0, /*tp_getattr*/
  0, /*tp_setattr*/
  0, /*tp_as_async*/
  0, /*tp_repr*/
  0, /*tp_as_number*/
  0, /*tp_as_sequence*/
  0, /*tp_as_mapping*/
  0, /*tp_hash*/
  0, /*tp_call*/
  0, /*tp_str*/
  0, /*tp_getattro*/
  0, /*tp_setattro*/
  0, /*tp_as_buffer*/
  Py_TPFLAGS_DEFAULT|Py_TPFLAGS_HAVE_VERSION_TAG|Py_TPFLAGS_CHECKTYPES|Py_TPFLAGS_HAVE_NEWBUFFER|Py_TPFLAGS_BASETYPE, /*tp_flags*/
  PyDoc_STR("ManagedMemory(size_t size)\n\nManaged memory (Unified memory) allocation on a CUDA device.\n\nThis class provides an RAII interface of the CUDA managed memory\nallocation.\n\nArgs:\n    size (int): Size of the memory allocation in bytes."), /*tp_doc*/
  0, /*tp_traverse*/
  0, /*tp_clear*/
  0, /*tp_richcompare*/
  0, /*tp_weaklistoffset*/
  0, /*tp_iter*/
  0, /*tp_iternext*/
  __pyx_methods_4cupy_4cuda_6memory_ManagedMemory, /*tp_methods*/
  0, /*tp_members*/
  0, /*tp_getset*/
  0, /*tp_base*/
  0, /*tp_dict*/
  0, /*tp_descr_get*/
  0, /*tp_descr_set*/
  #if !CYTHON_USE_TYPE_SPECS
  0, /*tp_dictoffset*/
  #endif
  __pyx_pw_4cupy_4cuda_6memory_13ManagedMemory_1__init__, /*tp_init*/
  0, /*tp_alloc*/
  __pyx_tp_new_4cupy_4cuda_6memory_BaseMemory, /*tp_new*/
  0, /*tp_free*/
  0, /*tp_is_gc*/
  0, /*tp_bases*/
  0, /*tp_mro*/
  0, /*tp_cache*/
  0, /*tp_subclasses*/
  0, /*tp_weaklist*/
  0, /*tp_del*/
  0, /*tp_version_tag*/
  #if CYTHON_USE_TP_FINALIZE
  0, /*tp_finalize*/
  #else
  NULL, /*tp_finalize*/
  #endif
  #if !CYTHON_COMPILING_IN_PYPY || PYPY_VERSION_NUM >= 0x07030800
  0, /*tp_vectorcall*/
  #endif
  #if __PYX_NEED_TP_PRINT_SLOT == 1
  0, /*tp_print*/
  #endif
  #if PY_VERSION_HEX >= 0x030C0000
  0, /*tp_watched*/
  #endif
  #if PY_VERSION_HEX >= 0x030d00A4
  0, /*tp_versions_used*/
  #endif
  #if CYTHON_COMPILING_IN_PYPY && PY_VERSION_HEX >= 0x03090000 && PY_VERSION_HEX < 0x030a0000
  0, /*tp_pypy_flags*/
  #endif
};
#endif
static struct __pyx_vtabstruct_4cupy_4cuda_6memory__Chunk __pyx_vtable_4cupy_4cuda_6memory__Chunk;

static PyObject *__pyx_tp_new_4cupy_4cuda_6memory__Chunk(PyTypeObject *t, CYTHON_UNUSED PyObject *a, CYTHON_UNUSED PyObject *k) {
  struct __pyx_obj_4cupy_4cuda_6memory__Chunk *p;
  PyObject *o;
  o = __Pyx_AllocateExtensionType(t, 1);
  if (unlikely(!o)) return 0;
  p = ((struct __pyx_obj_4cupy_4cuda_6memory__Chunk *)o);
  p->__pyx_vtab = __pyx_vtabptr_4cupy_4cuda_6memory__Chunk;
  p->mem = ((struct __pyx_obj_4cupy_4cuda_6memory_BaseMemory *)Py_None); Py_INCREF(Py_None);
  p->arena = ((struct __pyx_obj_4cupy_4cuda_6memory__Arena *)Py_None); Py_INCREF(Py_None);
  p->prev = ((struct __pyx_obj_4cupy_4cuda_6memory__Chunk *)Py_None); Py_INCREF(Py_None);
  p->next = ((struct __pyx_obj_4cupy_4cuda_6memory__Chunk *)Py_None); Py_INCREF(Py_None);
  return o;
}

static void __pyx_tp_dealloc_4cupy_4cuda_6memory__Chunk(PyObject *o) {
  struct __pyx_obj_4cupy_4cuda_6memory__Chunk *p = (struct __pyx_obj_4cupy_4cuda_6memory__Chunk *)o;
  Py_CLEAR(p->mem);
  Py_CLEAR(p->arena);
  Py_CLEAR(p->prev);
  Py_CLEAR(p->next);
  PyTypeObject *tp = Py_TYPE(o);
  #if CYTHON_USE_TYPE_SLOTS
  (*tp->tp_free)(o);
  #else
  {
    freefunc tp_free = (freefunc)PyType_GetSlot(tp, Py_tp_free);
    if (tp_free) tp_free(o);
  }
  #endif
  #if CYTHON_USE_TYPE_SPECS
  Py_DECREF(tp);
  #endif
}

static PyObject *__pyx_getprop_4cupy_4cuda_6memory_6_Chunk_mem(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_4cupy_4cuda_6memory_6_Chunk_3mem_1__get__(o);
}

static PyObject *__pyx_getprop_4cupy_4cuda_6memory_6_Chunk_offset(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_4cupy_4cuda_6memory_6_Chunk_6offset_1__get__(o);
}

static PyObject *__pyx_getprop_4cupy_4cuda_6memory_6_Chunk_size(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_4cupy_4cuda_6memory_6_Chunk_4size_1__get__(o);
}

static PyObject *__pyx_getprop_4cupy_4cuda_6memory_6_Chunk_arena(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_4cupy_4cuda_6memory_6_Chunk_5arena_1__get__(o);
}

static PyObject *__pyx_getprop_4cupy_4cuda_6memory_6_Chunk_prev(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_4cupy_4cuda_6memory_6_Chunk_4prev_1__get__(o);
}

static int __pyx_setprop_4cupy_4cuda_6memory_6_Chunk_prev(PyObject *o, PyObject *v, CYTHON_UNUSED void *x) {
  if (v) {
    return __pyx_pw_4cupy_4cuda_6memory_6_Chunk_4prev_3__set__(o, v);
  }
  else {
    return __pyx_pw_4cupy_4cuda_6memory_6_Chunk_4prev_5__del__(o);
  }
}

static PyObject *__pyx_getprop_4cupy_4cuda_6memory_6_Chunk_next(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_4cupy_4cuda_6memory_6_Chunk_4next_1__get__(o);
}

static int __pyx_setprop_4cupy_4cuda_6memory_6_Chunk_next(PyObject *o, PyObject *v, CYTHON_UNUSED void *x) {
  if (v) {
    return __pyx_pw_4cupy_4cuda_6memory_6_Chunk_4next_3__set__(o, v);
  }
  else {
    return __pyx_pw_4cupy_4cuda_6memory_6_Chunk_4next_5__del__(o);
  }
}

static PyMethodDef __pyx_methods_4cupy_4cuda_6memory__Chunk[] = {
  {"ptr", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_4cupy_4cuda_6memory_6_Chunk_5ptr, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_4cupy_4cuda_6memory_6_Chunk_4ptr},
  {"split", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_4cupy_4cuda_6memory_6_Chunk_7split, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_4cupy_4cuda_6memory_6_Chunk_6split},
  {"merge_next", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_4cupy_4cuda_6memory_6_Chunk_9merge_next, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_4cupy_4cuda_6memory_6_Chunk_8merge_next},
  {"__reduce_cython__", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_4cupy_4cuda_6memory_6_Chunk_11__reduce_cython__, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_4cupy_4cuda_6memory_6_Chunk_10__reduce_cython__},
  {"__setstate_cython__", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_4cupy_4cuda_6memory_6_Chunk_13__setstate_cython__, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_4cupy_4cuda_6memory_6_Chunk_12__setstate_cython__},
  {0, 0, 0, 0}
};

static struct PyGetSetDef __pyx_getsets_4cupy_4cuda_6memory__Chunk[] = {
  {"mem", __pyx_getprop_4cupy_4cuda_6memory_6_Chunk_mem, 0, 0, 0},
  {"offset", __pyx_getprop_4cupy_4cuda_6memory_6_Chunk_offset, 0, 0, 0},
  {"size", __pyx_getprop_4cupy_4cuda_6memory_6_Chunk_size, 0, 0, 0},
  {"arena", __pyx_getprop_4cupy_4cuda_6memory_6_Chunk_arena, 0, 0, 0},
  {"prev", __pyx_getprop_4cupy_4cuda_6memory_6_Chunk_prev, __pyx_setprop_4cupy_4cuda_6memory_6_Chunk_prev, PyDoc_STR("prev: cupy.cuda.memory._Chunk"), 0},
  {"next", __pyx_getprop_4cupy_4cuda_6memory_6_Chunk_next, __pyx_setprop_4cupy_4cuda_6memory_6_Chunk_next, PyDoc_STR("next: cupy.cuda.memory._Chunk"), 0},
  {0, 0, 0, 0, 0}
};
#if CYTHON_USE_TYPE_SPECS
static PyType_Slot __pyx_type_4cupy_4cuda_6memory__Chunk_slots[] = {
  {Py_tp_dealloc, (void *)__pyx_tp_dealloc_4cupy_4cuda_6memory__Chunk},
  {Py_tp_repr, (void *)__pyx_pw_4cupy_4cuda_6memory_6_Chunk_3__repr__},
  {Py_tp_doc, (void *)PyDoc_STR("_Chunk(*args)\n\nA chunk points to a device memory.\n\nA chunk might be a splitted memory block from a larger allocation.\nThe prev/next pointers construct a doubly-linked list of memory addresses\nsorted by base address that must be contiguous.\n\nArgs:\n    mem (~cupy.cuda.Memory): The device memory buffer.\n    offset (int): An offset bytes from the head of the buffer.\n    size (int): Chunk size in bytes.\n    arena (_Arena): The _Arena this chunk is associated with.\n\nAttributes:\n    mem (Memory): The device memory buffer.\n    ptr (int): Memory address.\n    offset (int): An offset bytes from the head of the buffer.\n    size (int): Chunk size in bytes.\n    arena (_Arena): The _Arena this chunk is associated with (or None\n        if this chunk is free'd). Allows us to find the arena on free\n        without the need of locking and ensures the arena isn't deleted\n        while chunks still exists.\n        But, we should set arena=None when we are done with the chunk.\n\nNotes:\n    Mutating chunks is only safe if the arena mutex is held since\n    otherwise another thread may mutate it (e.g. split or merge).")},
  {Py_tp_methods, (void *)__pyx_methods_4cupy_4cuda_6memory__Chunk},
  {Py_tp_getset, (void *)__pyx_getsets_4cupy_4cuda_6memory__Chunk},
  {Py_tp_init, (void *)__pyx_pw_4cupy_4cuda_6memory_6_Chunk_1__init__},
  {Py_tp_new, (void *)__pyx_tp_new_4cupy_4cuda_6memory__Chunk},
  {0, 0},
};
static PyType_Spec __pyx_type_4cupy_4cuda_6memory__Chunk_spec = {
  "cupy.cuda.memory._Chunk",
  sizeof(struct __pyx_obj_4cupy_4cuda_6memory__Chunk),
  0,
  Py_TPFLAGS_DEFAULT|Py_TPFLAGS_HAVE_VERSION_TAG|Py_TPFLAGS_CHECKTYPES|Py_TPFLAGS_HAVE_NEWBUFFER,
  __pyx_type_4cupy_4cuda_6memory__Chunk_slots,
};
#else

static PyTypeObject __pyx_type_4cupy_4cuda_6memory__Chunk = {
  PyVarObject_HEAD_INIT(0, 0)
  "cupy.cuda.memory.""_Chunk", /*tp_name*/
  sizeof(struct __pyx_obj_4cupy_4cuda_6memory__Chunk), /*tp_basicsize*/
  0, /*tp_itemsize*/
  __pyx_tp_dealloc_4cupy_4cuda_6memory__Chunk, /*tp_dealloc*/
  0, /*tp_vectorcall_offset*/
  0, /*tp_getattr*/
  0, /*tp_setattr*/
  0, /*tp_as_async*/
  __pyx_pw_4cupy_4cuda_6memory_6_Chunk_3__repr__, /*tp_repr*/
  0, /*tp_as_number*/
  0, /*tp_as_sequence*/
  0, /*tp_as_mapping*/
  0, /*tp_hash*/
  0, /*tp_call*/
  0, /*tp_str*/
  0, /*tp_getattro*/
  0, /*tp_setattro*/
  0, /*tp_as_buffer*/
  Py_TPFLAGS_DEFAULT|Py_TPFLAGS_HAVE_VERSION_TAG|Py_TPFLAGS_CHECKTYPES|Py_TPFLAGS_HAVE_NEWBUFFER, /*tp_flags*/
  PyDoc_STR("_Chunk(*args)\n\nA chunk points to a device memory.\n\nA chunk might be a splitted memory block from a larger allocation.\nThe prev/next pointers construct a doubly-linked list of memory addresses\nsorted by base address that must be contiguous.\n\nArgs:\n    mem (~cupy.cuda.Memory): The device memory buffer.\n    offset (int): An offset bytes from the head of the buffer.\n    size (int): Chunk size in bytes.\n    arena (_Arena): The _Arena this chunk is associated with.\n\nAttributes:\n    mem (Memory): The device memory buffer.\n    ptr (int): Memory address.\n    offset (int): An offset bytes from the head of the buffer.\n    size (int): Chunk size in bytes.\n    arena (_Arena): The _Arena this chunk is associated with (or None\n        if this chunk is free'd). Allows us to find the arena on free\n        without the need of locking and ensures the arena isn't deleted\n        while chunks still exists.\n        But, we should set arena=None when we are done with the chunk.\n\nNotes:\n    Mutating chunks is only safe if the arena mutex is held since\n    otherwise another thread may mutate it (e.g. split or merge)."), /*tp_doc*/
  0, /*tp_traverse*/
  0, /*tp_clear*/
  0, /*tp_richcompare*/
  0, /*tp_weaklistoffset*/
  0, /*tp_iter*/
  0, /*tp_iternext*/
  __pyx_methods_4cupy_4cuda_6memory__Chunk, /*tp_methods*/
  0, /*tp_members*/
  __pyx_getsets_4cupy_4cuda_6memory__Chunk, /*tp_getset*/
  0, /*tp_base*/
  0, /*tp_dict*/
  0, /*tp_descr_get*/
  0, /*tp_descr_set*/
  #if !CYTHON_USE_TYPE_SPECS
  0, /*tp_dictoffset*/
  #endif
  __pyx_pw_4cupy_4cuda_6memory_6_Chunk_1__init__, /*tp_init*/
  0, /*tp_alloc*/
  __pyx_tp_new_4cupy_4cuda_6memory__Chunk, /*tp_new*/
  0, /*tp_free*/
  0, /*tp_is_gc*/
  0, /*tp_bases*/
  0, /*tp_mro*/
  0, /*tp_cache*/
  0, /*tp_subclasses*/
  0, /*tp_weaklist*/
  0, /*tp_del*/
  0, /*tp_version_tag*/
  #if CYTHON_USE_TP_FINALIZE
  0, /*tp_finalize*/
  #else
  NULL, /*tp_finalize*/
  #endif
  #if !CYTHON_COMPILING_IN_PYPY || PYPY_VERSION_NUM >= 0x07030800
  0, /*tp_vectorcall*/
  #endif
  #if __PYX_NEED_TP_PRINT_SLOT == 1
  0, /*tp_print*/
  #endif
  #if PY_VERSION_HEX >= 0x030C0000
  0, /*tp_watched*/
  #endif
  #if PY_VERSION_HEX >= 0x030d00A4
  0, /*tp_versions_used*/
  #endif
  #if CYTHON_COMPILING_IN_PYPY && PY_VERSION_HEX >= 0x03090000 && PY_VERSION_HEX < 0x030a0000
  0, /*tp_pypy_flags*/
  #endif
};
#endif
static struct __pyx_vtabstruct_4cupy_4cuda_6memory_PooledMemory __pyx_vtable_4cupy_4cuda_6memory_PooledMemory;

static PyObject *__pyx_tp_new_4cupy_4cuda_6memory_PooledMemory(PyTypeObject *t, PyObject *a, PyObject *k) {
  struct __pyx_obj_4cupy_4cuda_6memory_PooledMemory *p;
  PyObject *o = __pyx_tp_new_4cupy_4cuda_6memory_BaseMemory(t, a, k);
  if (unlikely(!o)) return 0;
  p = ((struct __pyx_obj_4cupy_4cuda_6memory_PooledMemory *)o);
  p->__pyx_vtab = __pyx_vtabptr_4cupy_4cuda_6memory_PooledMemory;
  p->pool = Py_None; Py_INCREF(Py_None);
  p->identity = ((PyObject*)Py_None); Py_INCREF(Py_None);
  p->chunk = ((struct __pyx_obj_4cupy_4cuda_6memory__Chunk *)Py_None); Py_INCREF(Py_None);
  p->__dict__ = PyDict_New(); if (unlikely(!p->__dict__)) goto bad;return o;
  bad:
  Py_DECREF(o); o = 0;
  return NULL;
}

static void __pyx_tp_dealloc_4cupy_4cuda_6memory_PooledMemory(PyObject *o) {
  struct __pyx_obj_4cupy_4cuda_6memory_PooledMemory *p = (struct __pyx_obj_4cupy_4cuda_6memory_PooledMemory *)o;
  {
    PyObject *etype, *eval, *etb;
    PyErr_Fetch(&etype, &eval, &etb);
    __Pyx_SET_REFCNT(o, Py_REFCNT(o) + 1);
    __pyx_pw_4cupy_4cuda_6memory_12PooledMemory_5__dealloc__(o);
    __Pyx_SET_REFCNT(o, Py_REFCNT(o) - 1);
    PyErr_Restore(etype, eval, etb);
  }
  if (p->__dict__) PyDict_Clear(p->__dict__);
  Py_CLEAR(p->pool);
  Py_CLEAR(p->identity);
  Py_CLEAR(p->chunk);
  Py_CLEAR(p->__dict__);
  __pyx_tp_dealloc_4cupy_4cuda_6memory_BaseMemory(o);
}

#if CYTHON_COMPILING_IN_LIMITED_API && __PYX_LIMITED_VERSION_HEX < 0x030A0000
static PyObject *__pyx___dict__getter_4cupy_4cuda_6memory_PooledMemory(PyObject *o, CYTHON_UNUSED void *x) {
  struct __pyx_obj_4cupy_4cuda_6memory_PooledMemory *p = (struct __pyx_obj_4cupy_4cuda_6memory_PooledMemory *)o;
  if (unlikely(!p->__dict__)){
    p->__dict__ = PyDict_New();
  }
  Py_XINCREF(p->__dict__);
  return p->__dict__;
}
#else
#define __pyx___dict__getter_4cupy_4cuda_6memory_PooledMemory PyObject_GenericGetDict
#endif

static PyObject *__pyx_getprop_4cupy_4cuda_6memory_12PooledMemory_pool(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_4cupy_4cuda_6memory_12PooledMemory_4pool_1__get__(o);
}

static PyObject *__pyx_getprop_4cupy_4cuda_6memory_12PooledMemory_identity(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_4cupy_4cuda_6memory_12PooledMemory_8identity_1__get__(o);
}

static PyMethodDef __pyx_methods_4cupy_4cuda_6memory_PooledMemory[] = {
  {"free", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_4cupy_4cuda_6memory_12PooledMemory_3free, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_4cupy_4cuda_6memory_12PooledMemory_2free},
  {"__reduce_cython__", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_4cupy_4cuda_6memory_12PooledMemory_7__reduce_cython__, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_4cupy_4cuda_6memory_12PooledMemory_6__reduce_cython__},
  {"__setstate_cython__", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_4cupy_4cuda_6memory_12PooledMemory_9__setstate_cython__, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_4cupy_4cuda_6memory_12PooledMemory_8__setstate_cython__},
  {0, 0, 0, 0}
};

static struct PyGetSetDef __pyx_getsets_4cupy_4cuda_6memory_PooledMemory[] = {
  {"__dict__", __pyx___dict__getter_4cupy_4cuda_6memory_PooledMemory, 0, 0, 0},
  {"pool", __pyx_getprop_4cupy_4cuda_6memory_12PooledMemory_pool, 0, 0, 0},
  {"identity", __pyx_getprop_4cupy_4cuda_6memory_12PooledMemory_identity, 0, 0, 0},
  {0, 0, 0, 0, 0}
};
#if CYTHON_USE_TYPE_SPECS
static struct PyMemberDef __pyx_tp_members_PooledMemory[] = {
  {"__dictoffset__", T_PYSSIZET, offsetof(struct __pyx_obj_4cupy_4cuda_6memory_PooledMemory, __dict__), READONLY, NULL},
  {NULL, 0, 0, 0, NULL}
};
static PyType_Slot __pyx_type_4cupy_4cuda_6memory_PooledMemory_slots[] = {
  {Py_tp_dealloc, (void *)__pyx_tp_dealloc_4cupy_4cuda_6memory_PooledMemory},
  {Py_tp_doc, (void *)PyDoc_STR("PooledMemory(_Chunk chunk, pool)\n\nMemory allocation for a memory pool.\n\nThe instance of this class is created by memory pool allocator, so user\nshould not instantiate it by hand.")},
  {Py_tp_methods, (void *)__pyx_methods_4cupy_4cuda_6memory_PooledMemory},
  {Py_tp_members, (void *)__pyx_tp_members_PooledMemory},
  {Py_tp_getset, (void *)__pyx_getsets_4cupy_4cuda_6memory_PooledMemory},
  #if !CYTHON_USE_TYPE_SPECS
  {Py_tp_dictoffset, (void *)offsetof(struct __pyx_obj_4cupy_4cuda_6memory_PooledMemory, __dict__)},
  #endif
  {Py_tp_init, (void *)__pyx_pw_4cupy_4cuda_6memory_12PooledMemory_1__init__},
  {Py_tp_new, (void *)__pyx_tp_new_4cupy_4cuda_6memory_PooledMemory},
  {0, 0},
};
static PyType_Spec __pyx_type_4cupy_4cuda_6memory_PooledMemory_spec = {
  "cupy.cuda.memory.PooledMemory",
  sizeof(struct __pyx_obj_4cupy_4cuda_6memory_PooledMemory),
  0,
  Py_TPFLAGS_DEFAULT|Py_TPFLAGS_HAVE_VERSION_TAG|Py_TPFLAGS_CHECKTYPES|Py_TPFLAGS_HAVE_NEWBUFFER,
  __pyx_type_4cupy_4cuda_6memory_PooledMemory_slots,
};
#else

static PyTypeObject __pyx_type_4cupy_4cuda_6memory_PooledMemory = {
  PyVarObject_HEAD_INIT(0, 0)
  "cupy.cuda.memory.""PooledMemory", /*tp_name*/
  sizeof(struct __pyx_obj_4cupy_4cuda_6memory_PooledMemory), /*tp_basicsize*/
  0, /*tp_itemsize*/
  __pyx_tp_dealloc_4cupy_4cuda_6memory_PooledMemory, /*tp_dealloc*/
  0, /*tp_vectorcall_offset*/
  0, /*tp_getattr*/
  0, /*tp_setattr*/
  0, /*tp_as_async*/
  0, /*tp_repr*/
  0, /*tp_as_number*/
  0, /*tp_as_sequence*/
  0, /*tp_as_mapping*/
  0, /*tp_hash*/
  0, /*tp_call*/
  0, /*tp_str*/
  0, /*tp_getattro*/
  0, /*tp_setattro*/
  0, /*tp_as_buffer*/
  Py_TPFLAGS_DEFAULT|Py_TPFLAGS_HAVE_VERSION_TAG|Py_TPFLAGS_CHECKTYPES|Py_TPFLAGS_HAVE_NEWBUFFER, /*tp_flags*/
  PyDoc_STR("PooledMemory(_Chunk chunk, pool)\n\nMemory allocation for a memory pool.\n\nThe instance of this class is created by memory pool allocator, so user\nshould not instantiate it by hand."), /*tp_doc*/
  0, /*tp_traverse*/
  0, /*tp_clear*/
  0, /*tp_richcompare*/
  0, /*tp_weaklistoffset*/
  0, /*tp_iter*/
  0, /*tp_iternext*/
  __pyx_methods_4cupy_4cuda_6memory_PooledMemory, /*tp_methods*/
  0, /*tp_members*/
  __pyx_getsets_4cupy_4cuda_6memory_PooledMemory, /*tp_getset*/
  0, /*tp_base*/
  0, /*tp_dict*/
  0, /*tp_descr_get*/
  0, /*tp_descr_set*/
  #if !CYTHON_USE_TYPE_SPECS
  offsetof(struct __pyx_obj_4cupy_4cuda_6memory_PooledMemory, __dict__), /*tp_dictoffset*/
  #endif
  __pyx_pw_4cupy_4cuda_6memory_12PooledMemory_1__init__, /*tp_init*/
  0, /*tp_alloc*/
  __pyx_tp_new_4cupy_4cuda_6memory_PooledMemory, /*tp_new*/
  0, /*tp_free*/
  0, /*tp_is_gc*/
  0, /*tp_bases*/
  0, /*tp_mro*/
  0, /*tp_cache*/
  0, /*tp_subclasses*/
  0, /*tp_weaklist*/
  0, /*tp_del*/
  0, /*tp_version_tag*/
  #if CYTHON_USE_TP_FINALIZE
  0, /*tp_finalize*/
  #else
  NULL, /*tp_finalize*/
  #endif
  #if !CYTHON_COMPILING_IN_PYPY || PYPY_VERSION_NUM >= 0x07030800
  0, /*tp_vectorcall*/
  #endif
  #if __PYX_NEED_TP_PRINT_SLOT == 1
  0, /*tp_print*/
  #endif
  #if PY_VERSION_HEX >= 0x030C0000
  0, /*tp_watched*/
  #endif
  #if PY_VERSION_HEX >= 0x030d00A4
  0, /*tp_versions_used*/
  #endif
  #if CYTHON_COMPILING_IN_PYPY && PY_VERSION_HEX >= 0x03090000 && PY_VERSION_HEX < 0x030a0000
  0, /*tp_pypy_flags*/
  #endif
};
#endif
static struct __pyx_vtabstruct_4cupy_4cuda_6memory__Arena __pyx_vtable_4cupy_4cuda_6memory__Arena;

static PyObject *__pyx_tp_new_4cupy_4cuda_6memory__Arena(PyTypeObject *t, CYTHON_UNUSED PyObject *a, CYTHON_UNUSED PyObject *k) {
  struct __pyx_obj_4cupy_4cuda_6memory__Arena *p;
  PyObject *o;
  o = __Pyx_AllocateExtensionType(t, 1);
  if (unlikely(!o)) return 0;
  p = ((struct __pyx_obj_4cupy_4cuda_6memory__Arena *)o);
  p->__pyx_vtab = __pyx_vtabptr_4cupy_4cuda_6memory__Arena;
  __Pyx_default_placement_construct(&(p->index));
  p->_pending_free = ((PyObject*)Py_None); Py_INCREF(Py_None);
  if (unlikely(__pyx_pw_4cupy_4cuda_6memory_6_Arena_1__cinit__(o, __pyx_mstate_global->__pyx_empty_tuple, NULL) < 0)) goto bad;
  return o;
  bad:
  Py_DECREF(o); o = 0;
  return NULL;
}

static void __pyx_tp_dealloc_4cupy_4cuda_6memory__Arena(PyObject *o) {
  struct __pyx_obj_4cupy_4cuda_6memory__Arena *p = (struct __pyx_obj_4cupy_4cuda_6memory__Arena *)o;
  PyObject_GC_UnTrack(o);
  if (p->__weakref__) PyObject_ClearWeakRefs(o);
  __Pyx_call_destructor(p->index);
  Py_CLEAR(p->_pending_free);
  PyTypeObject *tp = Py_TYPE(o);
  #if CYTHON_USE_TYPE_SLOTS
  (*tp->tp_free)(o);
  #else
  {
    freefunc tp_free = (freefunc)PyType_GetSlot(tp, Py_tp_free);
    if (tp_free) tp_free(o);
  }
  #endif
  #if CYTHON_USE_TYPE_SPECS
  Py_DECREF(tp);
  #endif
}

static int __pyx_tp_traverse_4cupy_4cuda_6memory__Arena(PyObject *o, visitproc v, void *a) {
  int e;
  struct __pyx_obj_4cupy_4cuda_6memory__Arena *p = (struct __pyx_obj_4cupy_4cuda_6memory__Arena *)o;
  {
    e = __Pyx_call_type_traverse(o, 1, v, a);
    if (e) return e;
  }
  if (p->_pending_free) {
    e = (*v)(p->_pending_free, a); if (e) return e;
  }
  return 0;
}

static int __pyx_tp_clear_4cupy_4cuda_6memory__Arena(PyObject *o) {
  PyObject* tmp;
  struct __pyx_obj_4cupy_4cuda_6memory__Arena *p = (struct __pyx_obj_4cupy_4cuda_6memory__Arena *)o;
  tmp = ((PyObject*)p->_pending_free);
  p->_pending_free = ((PyObject*)Py_None); Py_INCREF(Py_None);
  Py_XDECREF(tmp);
  return 0;
}

static PyMethodDef __pyx_methods_4cupy_4cuda_6memory__Arena[] = {
  {"__reduce_cython__", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_4cupy_4cuda_6memory_6_Arena_3__reduce_cython__, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_4cupy_4cuda_6memory_6_Arena_2__reduce_cython__},
  {"__setstate_cython__", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_4cupy_4cuda_6memory_6_Arena_5__setstate_cython__, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_4cupy_4cuda_6memory_6_Arena_4__setstate_cython__},
  {0, 0, 0, 0}
};
#if CYTHON_USE_TYPE_SPECS
static PyMemberDef __pyx_type_4cupy_4cuda_6memory__Arena_members[] = {
  #if !CYTHON_USE_TYPE_SLOTS
  {"__weaklistoffset__", T_PYSSIZET, offsetof(struct __pyx_obj_4cupy_4cuda_6memory__Arena, __weakref__), READONLY, 0},
  #endif
  {0, 0, 0, 0, 0}
};
#if CYTHON_COMPILING_IN_LIMITED_API && __PYX_LIMITED_VERSION_HEX < 0x03090000
#if defined(__GNUC__) || defined(__clang__)
#warning "cupy/cuda/memory.pyx:1070:20: : __weakref__ is unsupported in the Limited API when running on Python <3.9."
#elif defined(_MSC_VER)
#pragma message("cupy/cuda/memory.pyx:1070:20: : __weakref__ is unsupported in the Limited API when running on Python <3.9.")
#endif
#endif
static PyType_Slot __pyx_type_4cupy_4cuda_6memory__Arena_slots[] = {
  {Py_tp_dealloc, (void *)__pyx_tp_dealloc_4cupy_4cuda_6memory__Arena},
  {Py_tp_traverse, (void *)__pyx_tp_traverse_4cupy_4cuda_6memory__Arena},
  {Py_tp_clear, (void *)__pyx_tp_clear_4cupy_4cuda_6memory__Arena},
  {Py_tp_methods, (void *)__pyx_methods_4cupy_4cuda_6memory__Arena},
  {Py_tp_new, (void *)__pyx_tp_new_4cupy_4cuda_6memory__Arena},
  {Py_tp_members, (void*)__pyx_type_4cupy_4cuda_6memory__Arena_members},
  {0, 0},
};
static PyType_Spec __pyx_type_4cupy_4cuda_6memory__Arena_spec = {
  "cupy.cuda.memory._Arena",
  sizeof(struct __pyx_obj_4cupy_4cuda_6memory__Arena),
  0,
  Py_TPFLAGS_DEFAULT|Py_TPFLAGS_HAVE_VERSION_TAG|Py_TPFLAGS_CHECKTYPES|Py_TPFLAGS_HAVE_NEWBUFFER|Py_TPFLAGS_HAVE_GC,
  __pyx_type_4cupy_4cuda_6memory__Arena_slots,
};
#else

static PyTypeObject __pyx_type_4cupy_4cuda_6memory__Arena = {
  PyVarObject_HEAD_INIT(0, 0)
  "cupy.cuda.memory.""_Arena", /*tp_name*/
  sizeof(struct __pyx_obj_4cupy_4cuda_6memory__Arena), /*tp_basicsize*/
  0, /*tp_itemsize*/
  __pyx_tp_dealloc_4cupy_4cuda_6memory__Arena, /*tp_dealloc*/
  0, /*tp_vectorcall_offset*/
  0, /*tp_getattr*/
  0, /*tp_setattr*/
  0, /*tp_as_async*/
  0, /*tp_repr*/
  0, /*tp_as_number*/
  0, /*tp_as_sequence*/
  0, /*tp_as_mapping*/
  0, /*tp_hash*/
  0, /*tp_call*/
  0, /*tp_str*/
  0, /*tp_getattro*/
  0, /*tp_setattro*/
  0, /*tp_as_buffer*/
  Py_TPFLAGS_DEFAULT|Py_TPFLAGS_HAVE_VERSION_TAG|Py_TPFLAGS_CHECKTYPES|Py_TPFLAGS_HAVE_NEWBUFFER|Py_TPFLAGS_HAVE_GC, /*tp_flags*/
  0, /*tp_doc*/
  __pyx_tp_traverse_4cupy_4cuda_6memory__Arena, /*tp_traverse*/
  __pyx_tp_clear_4cupy_4cuda_6memory__Arena, /*tp_clear*/
  0, /*tp_richcompare*/
  0, /*tp_weaklistoffset*/
  0, /*tp_iter*/
  0, /*tp_iternext*/
  __pyx_methods_4cupy_4cuda_6memory__Arena, /*tp_methods*/
  0, /*tp_members*/
  0, /*tp_getset*/
  0, /*tp_base*/
  0, /*tp_dict*/
  0, /*tp_descr_get*/
  0, /*tp_descr_set*/
  #if !CYTHON_USE_TYPE_SPECS
  0, /*tp_dictoffset*/
  #endif
  0, /*tp_init*/
  0, /*tp_alloc*/
  __pyx_tp_new_4cupy_4cuda_6memory__Arena, /*tp_new*/
  0, /*tp_free*/
  0, /*tp_is_gc*/
  0, /*tp_bases*/
  0, /*tp_mro*/
  0, /*tp_cache*/
  0, /*tp_subclasses*/
  0, /*tp_weaklist*/
  0, /*tp_del*/
  0, /*tp_version_tag*/
  #if CYTHON_USE_TP_FINALIZE
  0, /*tp_finalize*/
  #else
  NULL, /*tp_finalize*/
  #endif
  #if !CYTHON_COMPILING_IN_PYPY || PYPY_VERSION_NUM >= 0x07030800
  0, /*tp_vectorcall*/
  #endif
  #if __PYX_NEED_TP_PRINT_SLOT == 1
  0, /*tp_print*/
  #endif
  #if PY_VERSION_HEX >= 0x030C0000
  0, /*tp_watched*/
  #endif
  #if PY_VERSION_HEX >= 0x030d00A4
  0, /*tp_versions_used*/
  #endif
  #if CYTHON_COMPILING_IN_PYPY && PY_VERSION_HEX >= 0x03090000 && PY_VERSION_HEX < 0x030a0000
  0, /*tp_pypy_flags*/
  #endif
};
#endif
static struct __pyx_vtabstruct_4cupy_4cuda_6memory_SingleDeviceMemoryPool __pyx_vtable_4cupy_4cuda_6memory_SingleDeviceMemoryPool;

static PyObject *__pyx_tp_new_4cupy_4cuda_6memory_SingleDeviceMemoryPool(PyTypeObject *t, CYTHON_UNUSED PyObject *a, CYTHON_UNUSED PyObject *k) {
  struct __pyx_obj_4cupy_4cuda_6memory_SingleDeviceMemoryPool *p;
  PyObject *o;
  o = __Pyx_AllocateExtensionType(t, 1);
  if (unlikely(!o)) return 0;
  p = ((struct __pyx_obj_4cupy_4cuda_6memory_SingleDeviceMemoryPool *)o);
  p->__pyx_vtab = __pyx_vtabptr_4cupy_4cuda_6memory_SingleDeviceMemoryPool;
  __Pyx_default_placement_construct(&(p->_arena_mutex));
  __Pyx_default_placement_construct(&(p->_total_bytes));
  __Pyx_default_placement_construct(&(p->_in_use_bytes));
  __Pyx_default_placement_construct(&(p->_total_bytes_limit));
  p->_allocator = Py_None; Py_INCREF(Py_None);
  p->_arenas = ((PyObject*)Py_None); Py_INCREF(Py_None);
  p->_weakref = Py_None; Py_INCREF(Py_None);
  return o;
}

static void __pyx_tp_dealloc_4cupy_4cuda_6memory_SingleDeviceMemoryPool(PyObject *o) {
  struct __pyx_obj_4cupy_4cuda_6memory_SingleDeviceMemoryPool *p = (struct __pyx_obj_4cupy_4cuda_6memory_SingleDeviceMemoryPool *)o;
  PyObject_GC_UnTrack(o);
  if (p->__weakref__) PyObject_ClearWeakRefs(o);
  __Pyx_call_destructor(p->_arena_mutex);
  __Pyx_call_destructor(p->_total_bytes);
  __Pyx_call_destructor(p->_in_use_bytes);
  __Pyx_call_destructor(p->_total_bytes_limit);
  Py_CLEAR(p->_allocator);
  Py_CLEAR(p->_arenas);
  Py_CLEAR(p->_weakref);
  PyTypeObject *tp = Py_TYPE(o);
  #if CYTHON_USE_TYPE_SLOTS
  (*tp->tp_free)(o);
  #else
  {
    freefunc tp_free = (freefunc)PyType_GetSlot(tp, Py_tp_free);
    if (tp_free) tp_free(o);
  }
  #endif
  #if CYTHON_USE_TYPE_SPECS
  Py_DECREF(tp);
  #endif
}

static int __pyx_tp_traverse_4cupy_4cuda_6memory_SingleDeviceMemoryPool(PyObject *o, visitproc v, void *a) {
  int e;
  struct __pyx_obj_4cupy_4cuda_6memory_SingleDeviceMemoryPool *p = (struct __pyx_obj_4cupy_4cuda_6memory_SingleDeviceMemoryPool *)o;
  {
    e = __Pyx_call_type_traverse(o, 1, v, a);
    if (e) return e;
  }
  if (p->_allocator) {
    e = (*v)(p->_allocator, a); if (e) return e;
  }
  if (p->_arenas) {
    e = (*v)(p->_arenas, a); if (e) return e;
  }
  if (p->_weakref) {
    e = (*v)(p->_weakref, a); if (e) return e;
  }
  return 0;
}

static int __pyx_tp_clear_4cupy_4cuda_6memory_SingleDeviceMemoryPool(PyObject *o) {
  PyObject* tmp;
  struct __pyx_obj_4cupy_4cuda_6memory_SingleDeviceMemoryPool *p = (struct __pyx_obj_4cupy_4cuda_6memory_SingleDeviceMemoryPool *)o;
  tmp = ((PyObject*)p->_allocator);
  p->_allocator = Py_None; Py_INCREF(Py_None);
  Py_XDECREF(tmp);
  tmp = ((PyObject*)p->_arenas);
  p->_arenas = ((PyObject*)Py_None); Py_INCREF(Py_None);
  Py_XDECREF(tmp);
  tmp = ((PyObject*)p->_weakref);
  p->_weakref = Py_None; Py_INCREF(Py_None);
  Py_XDECREF(tmp);
  return 0;
}

static PyObject *__pyx_getprop_4cupy_4cuda_6memory_22SingleDeviceMemoryPool__device_id(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_4cupy_4cuda_6memory_22SingleDeviceMemoryPool_10_device_id_1__get__(o);
}

static PyMethodDef __pyx_methods_4cupy_4cuda_6memory_SingleDeviceMemoryPool[] = {
  {"_arena", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_4cupy_4cuda_6memory_22SingleDeviceMemoryPool_3_arena, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_4cupy_4cuda_6memory_22SingleDeviceMemoryPool_2_arena},
  {"_debug_arena_get_index", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_4cupy_4cuda_6memory_22SingleDeviceMemoryPool_5_debug_arena_get_index, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_4cupy_4cuda_6memory_22SingleDeviceMemoryPool_4_debug_arena_get_index},
  {"malloc", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_4cupy_4cuda_6memory_22SingleDeviceMemoryPool_7malloc, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_4cupy_4cuda_6memory_22SingleDeviceMemoryPool_6malloc},
  {"free_all_blocks", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_4cupy_4cuda_6memory_22SingleDeviceMemoryPool_9free_all_blocks, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_4cupy_4cuda_6memory_22SingleDeviceMemoryPool_8free_all_blocks},
  {"free_all_free", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_4cupy_4cuda_6memory_22SingleDeviceMemoryPool_11free_all_free, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_4cupy_4cuda_6memory_22SingleDeviceMemoryPool_10free_all_free},
  {"n_free_blocks", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_4cupy_4cuda_6memory_22SingleDeviceMemoryPool_13n_free_blocks, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_4cupy_4cuda_6memory_22SingleDeviceMemoryPool_12n_free_blocks},
  {"used_bytes", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_4cupy_4cuda_6memory_22SingleDeviceMemoryPool_15used_bytes, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_4cupy_4cuda_6memory_22SingleDeviceMemoryPool_14used_bytes},
  {"free_bytes", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_4cupy_4cuda_6memory_22SingleDeviceMemoryPool_17free_bytes, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_4cupy_4cuda_6memory_22SingleDeviceMemoryPool_16free_bytes},
  {"total_bytes", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_4cupy_4cuda_6memory_22SingleDeviceMemoryPool_19total_bytes, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_4cupy_4cuda_6memory_22SingleDeviceMemoryPool_18total_bytes},
  {"set_limit", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_4cupy_4cuda_6memory_22SingleDeviceMemoryPool_21set_limit, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_4cupy_4cuda_6memory_22SingleDeviceMemoryPool_20set_limit},
  {"get_limit", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_4cupy_4cuda_6memory_22SingleDeviceMemoryPool_23get_limit, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_4cupy_4cuda_6memory_22SingleDeviceMemoryPool_22get_limit},
  {"__reduce_cython__", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_4cupy_4cuda_6memory_22SingleDeviceMemoryPool_25__reduce_cython__, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_4cupy_4cuda_6memory_22SingleDeviceMemoryPool_24__reduce_cython__},
  {"__setstate_cython__", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_4cupy_4cuda_6memory_22SingleDeviceMemoryPool_27__setstate_cython__, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_4cupy_4cuda_6memory_22SingleDeviceMemoryPool_26__setstate_cython__},
  {0, 0, 0, 0}
};

static struct PyGetSetDef __pyx_getsets_4cupy_4cuda_6memory_SingleDeviceMemoryPool[] = {
  {"_device_id", __pyx_getprop_4cupy_4cuda_6memory_22SingleDeviceMemoryPool__device_id, 0, 0, 0},
  {0, 0, 0, 0, 0}
};
#if CYTHON_USE_TYPE_SPECS
static PyMemberDef __pyx_type_4cupy_4cuda_6memory_SingleDeviceMemoryPool_members[] = {
  #if !CYTHON_USE_TYPE_SLOTS
  {"__weaklistoffset__", T_PYSSIZET, offsetof(struct __pyx_obj_4cupy_4cuda_6memory_SingleDeviceMemoryPool, __weakref__), READONLY, 0},
  #endif
  {0, 0, 0, 0, 0}
};
#if CYTHON_COMPILING_IN_LIMITED_API && __PYX_LIMITED_VERSION_HEX < 0x03090000
#if defined(__GNUC__) || defined(__clang__)
#warning "cupy/cuda/memory.pyx:1254:15: : __weakref__ is unsupported in the Limited API when running on Python <3.9."
#elif defined(_MSC_VER)
#pragma message("cupy/cuda/memory.pyx:1254:15: : __weakref__ is unsupported in the Limited API when running on Python <3.9.")
#endif
#endif
static PyType_Slot __pyx_type_4cupy_4cuda_6memory_SingleDeviceMemoryPool_slots[] = {
  {Py_tp_dealloc, (void *)__pyx_tp_dealloc_4cupy_4cuda_6memory_SingleDeviceMemoryPool},
  {Py_tp_doc, (void *)PyDoc_STR("SingleDeviceMemoryPool(allocator=None)\n\nMemory pool implementation for single device.\n\n- The allocator attempts to find the smallest cached block that will fit\n  the requested size. If the block is larger than the requested size,\n  it may be split. If no block is found, the allocator will delegate to\n  cudaMalloc.\n- If the cudaMalloc fails, the allocator will free all cached blocks that\n  are not split and retry the allocation.")},
  {Py_tp_traverse, (void *)__pyx_tp_traverse_4cupy_4cuda_6memory_SingleDeviceMemoryPool},
  {Py_tp_clear, (void *)__pyx_tp_clear_4cupy_4cuda_6memory_SingleDeviceMemoryPool},
  {Py_tp_methods, (void *)__pyx_methods_4cupy_4cuda_6memory_SingleDeviceMemoryPool},
  {Py_tp_getset, (void *)__pyx_getsets_4cupy_4cuda_6memory_SingleDeviceMemoryPool},
  {Py_tp_init, (void *)__pyx_pw_4cupy_4cuda_6memory_22SingleDeviceMemoryPool_1__init__},
  {Py_tp_new, (void *)__pyx_tp_new_4cupy_4cuda_6memory_SingleDeviceMemoryPool},
  {Py_tp_members, (void*)__pyx_type_4cupy_4cuda_6memory_SingleDeviceMemoryPool_members},
  {0, 0},
};
static PyType_Spec __pyx_type_4cupy_4cuda_6memory_SingleDeviceMemoryPool_spec = {
  "cupy.cuda.memory.SingleDeviceMemoryPool",
  sizeof(struct __pyx_obj_4cupy_4cuda_6memory_SingleDeviceMemoryPool),
  0,
  Py_TPFLAGS_DEFAULT|Py_TPFLAGS_HAVE_VERSION_TAG|Py_TPFLAGS_CHECKTYPES|Py_TPFLAGS_HAVE_NEWBUFFER|Py_TPFLAGS_HAVE_GC,
  __pyx_type_4cupy_4cuda_6memory_SingleDeviceMemoryPool_slots,
};
#else

static PyTypeObject __pyx_type_4cupy_4cuda_6memory_SingleDeviceMemoryPool = {
  PyVarObject_HEAD_INIT(0, 0)
  "cupy.cuda.memory.""SingleDeviceMemoryPool", /*tp_name*/
  sizeof(struct __pyx_obj_4cupy_4cuda_6memory_SingleDeviceMemoryPool), /*tp_basicsize*/
  0, /*tp_itemsize*/
  __pyx_tp_dealloc_4cupy_4cuda_6memory_SingleDeviceMemoryPool, /*tp_dealloc*/
  0, /*tp_vectorcall_offset*/
  0, /*tp_getattr*/
  0, /*tp_setattr*/
  0, /*tp_as_async*/
  0, /*tp_repr*/
  0, /*tp_as_number*/
  0, /*tp_as_sequence*/
  0, /*tp_as_mapping*/
  0, /*tp_hash*/
  0, /*tp_call*/
  0, /*tp_str*/
  0, /*tp_getattro*/
  0, /*tp_setattro*/
  0, /*tp_as_buffer*/
  Py_TPFLAGS_DEFAULT|Py_TPFLAGS_HAVE_VERSION_TAG|Py_TPFLAGS_CHECKTYPES|Py_TPFLAGS_HAVE_NEWBUFFER|Py_TPFLAGS_HAVE_GC, /*tp_flags*/
  PyDoc_STR("SingleDeviceMemoryPool(allocator=None)\n\nMemory pool implementation for single device.\n\n- The allocator attempts to find the smallest cached block that will fit\n  the requested size. If the block is larger than the requested size,\n  it may be split. If no block is found, the allocator will delegate to\n  cudaMalloc.\n- If the cudaMalloc fails, the allocator will free all cached blocks that\n  are not split and retry the allocation."), /*tp_doc*/
  __pyx_tp_traverse_4cupy_4cuda_6memory_SingleDeviceMemoryPool, /*tp_traverse*/
  __pyx_tp_clear_4cupy_4cuda_6memory_SingleDeviceMemoryPool, /*tp_clear*/
  0, /*tp_richcompare*/
  0, /*tp_weaklistoffset*/
  0, /*tp_iter*/
  0, /*tp_iternext*/
  __pyx_methods_4cupy_4cuda_6memory_SingleDeviceMemoryPool, /*tp_methods*/
  0, /*tp_members*/
  __pyx_getsets_4cupy_4cuda_6memory_SingleDeviceMemoryPool, /*tp_getset*/
  0, /*tp_base*/
  0, /*tp_dict*/
  0, /*tp_descr_get*/
  0, /*tp_descr_set*/
  #if !CYTHON_USE_TYPE_SPECS
  0, /*tp_dictoffset*/
  #endif
  __pyx_pw_4cupy_4cuda_6memory_22SingleDeviceMemoryPool_1__init__, /*tp_init*/
  0, /*tp_alloc*/
  __pyx_tp_new_4cupy_4cuda_6memory_SingleDeviceMemoryPool, /*tp_new*/
  0, /*tp_free*/
  0, /*tp_is_gc*/
  0, /*tp_bases*/
  0, /*tp_mro*/
  0, /*tp_cache*/
  0, /*tp_subclasses*/
  0, /*tp_weaklist*/
  0, /*tp_del*/
  0, /*tp_version_tag*/
  #if CYTHON_USE_TP_FINALIZE
  0, /*tp_finalize*/
  #else
  NULL, /*tp_finalize*/
  #endif
  #if !CYTHON_COMPILING_IN_PYPY || PYPY_VERSION_NUM >= 0x07030800
  0, /*tp_vectorcall*/
  #endif
  #if __PYX_NEED_TP_PRINT_SLOT == 1
  0, /*tp_print*/
  #endif
  #if PY_VERSION_HEX >= 0x030C0000
  0, /*tp_watched*/
  #endif
  #if PY_VERSION_HEX >= 0x030d00A4
  0, /*tp_versions_used*/
  #endif
  #if CYTHON_COMPILING_IN_PYPY && PY_VERSION_HEX >= 0x03090000 && PY_VERSION_HEX < 0x030a0000
  0, /*tp_pypy_flags*/
  #endif
};
#endif
static struct __pyx_vtabstruct_4cupy_4cuda_6memory_MemoryAsyncPool __pyx_vtable_4cupy_4cuda_6memory_MemoryAsyncPool;

static PyObject *__pyx_tp_new_4cupy_4cuda_6memory_MemoryAsyncPool(PyTypeObject *t, CYTHON_UNUSED PyObject *a, CYTHON_UNUSED PyObject *k) {
  struct __pyx_obj_4cupy_4cuda_6memory_MemoryAsyncPool *p;
  PyObject *o;
  o = __Pyx_AllocateExtensionType(t, 0);
  if (unlikely(!o)) return 0;
  p = ((struct __pyx_obj_4cupy_4cuda_6memory_MemoryAsyncPool *)o);
  p->__pyx_vtab = __pyx_vtabptr_4cupy_4cuda_6memory_MemoryAsyncPool;
  p->_pools = ((PyObject*)Py_None); Py_INCREF(Py_None);
  return o;
}

static void __pyx_tp_dealloc_4cupy_4cuda_6memory_MemoryAsyncPool(PyObject *o) {
  struct __pyx_obj_4cupy_4cuda_6memory_MemoryAsyncPool *p = (struct __pyx_obj_4cupy_4cuda_6memory_MemoryAsyncPool *)o;
  #if CYTHON_USE_TP_FINALIZE
  if (unlikely(__Pyx_PyObject_GetSlot(o, tp_finalize, destructor)) && !__Pyx_PyObject_GC_IsFinalized(o)) {
    if (__Pyx_PyObject_GetSlot(o, tp_dealloc, destructor) == __pyx_tp_dealloc_4cupy_4cuda_6memory_MemoryAsyncPool) {
      if (PyObject_CallFinalizerFromDealloc(o)) return;
    }
  }
  #endif
  PyObject_GC_UnTrack(o);
  Py_CLEAR(p->_pools);
  PyTypeObject *tp = Py_TYPE(o);
  #if CYTHON_USE_TYPE_SLOTS
  (*tp->tp_free)(o);
  #else
  {
    freefunc tp_free = (freefunc)PyType_GetSlot(tp, Py_tp_free);
    if (tp_free) tp_free(o);
  }
  #endif
  #if CYTHON_USE_TYPE_SPECS
  Py_DECREF(tp);
  #endif
}

static int __pyx_tp_traverse_4cupy_4cuda_6memory_MemoryAsyncPool(PyObject *o, visitproc v, void *a) {
  int e;
  struct __pyx_obj_4cupy_4cuda_6memory_MemoryAsyncPool *p = (struct __pyx_obj_4cupy_4cuda_6memory_MemoryAsyncPool *)o;
  {
    e = __Pyx_call_type_traverse(o, 1, v, a);
    if (e) return e;
  }
  if (p->_pools) {
    e = (*v)(p->_pools, a); if (e) return e;
  }
  return 0;
}

static int __pyx_tp_clear_4cupy_4cuda_6memory_MemoryAsyncPool(PyObject *o) {
  PyObject* tmp;
  struct __pyx_obj_4cupy_4cuda_6memory_MemoryAsyncPool *p = (struct __pyx_obj_4cupy_4cuda_6memory_MemoryAsyncPool *)o;
  tmp = ((PyObject*)p->_pools);
  p->_pools = ((PyObject*)Py_None); Py_INCREF(Py_None);
  Py_XDECREF(tmp);
  return 0;
}

static PyObject *__pyx_getprop_4cupy_4cuda_6memory_15MemoryAsyncPool__pools(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_4cupy_4cuda_6memory_15MemoryAsyncPool_6_pools_1__get__(o);
}

static PyObject *__pyx_getprop_4cupy_4cuda_6memory_15MemoryAsyncPool_memoryAsyncHasStat(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_4cupy_4cuda_6memory_15MemoryAsyncPool_18memoryAsyncHasStat_1__get__(o);
}

static PyMethodDef __pyx_methods_4cupy_4cuda_6memory_MemoryAsyncPool[] = {
  {"malloc", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_4cupy_4cuda_6memory_15MemoryAsyncPool_3malloc, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_4cupy_4cuda_6memory_15MemoryAsyncPool_2malloc},
  {"free_all_blocks", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_4cupy_4cuda_6memory_15MemoryAsyncPool_5free_all_blocks, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_4cupy_4cuda_6memory_15MemoryAsyncPool_4free_all_blocks},
  {"n_free_blocks", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_4cupy_4cuda_6memory_15MemoryAsyncPool_7n_free_blocks, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_4cupy_4cuda_6memory_15MemoryAsyncPool_6n_free_blocks},
  {"used_bytes", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_4cupy_4cuda_6memory_15MemoryAsyncPool_9used_bytes, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_4cupy_4cuda_6memory_15MemoryAsyncPool_8used_bytes},
  {"free_bytes", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_4cupy_4cuda_6memory_15MemoryAsyncPool_11free_bytes, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_4cupy_4cuda_6memory_15MemoryAsyncPool_10free_bytes},
  {"total_bytes", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_4cupy_4cuda_6memory_15MemoryAsyncPool_13total_bytes, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_4cupy_4cuda_6memory_15MemoryAsyncPool_12total_bytes},
  {"set_limit", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_4cupy_4cuda_6memory_15MemoryAsyncPool_15set_limit, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_4cupy_4cuda_6memory_15MemoryAsyncPool_14set_limit},
  {"get_limit", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_4cupy_4cuda_6memory_15MemoryAsyncPool_17get_limit, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_4cupy_4cuda_6memory_15MemoryAsyncPool_16get_limit},
  {"__reduce_cython__", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_4cupy_4cuda_6memory_15MemoryAsyncPool_19__reduce_cython__, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_4cupy_4cuda_6memory_15MemoryAsyncPool_18__reduce_cython__},
  {"__setstate_cython__", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_4cupy_4cuda_6memory_15MemoryAsyncPool_21__setstate_cython__, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_4cupy_4cuda_6memory_15MemoryAsyncPool_20__setstate_cython__},
  {0, 0, 0, 0}
};

static struct PyGetSetDef __pyx_getsets_4cupy_4cuda_6memory_MemoryAsyncPool[] = {
  {"_pools", __pyx_getprop_4cupy_4cuda_6memory_15MemoryAsyncPool__pools, 0, 0, 0},
  {"memoryAsyncHasStat", __pyx_getprop_4cupy_4cuda_6memory_15MemoryAsyncPool_memoryAsyncHasStat, 0, 0, 0},
  {0, 0, 0, 0, 0}
};
#if CYTHON_USE_TYPE_SPECS
static PyType_Slot __pyx_type_4cupy_4cuda_6memory_MemoryAsyncPool_slots[] = {
  {Py_tp_dealloc, (void *)__pyx_tp_dealloc_4cupy_4cuda_6memory_MemoryAsyncPool},
  {Py_tp_doc, (void *)PyDoc_STR("MemoryAsyncPool(pool_handles='current')\n\nCUDA memory pool for all GPU devices on the host.\n\nA memory pool preserves any allocations even if they are freed by the user.\nOne instance of this class can be used for multiple devices. This class\nuses CUDA's Stream Ordered Memory Allocator (supported on CUDA 11.2+).\nThe simplest way to use this pool as CuPy's default allocator is the\nfollowing code::\n\n    set_allocator(MemoryAsyncPool().malloc)\n\nUsing this feature requires CUDA >= 11.2 with a supported GPU and platform.\nIf it is not supported, an error will be raised.\n\nThe current CuPy stream is used to allocate/free the memory.\n\nArgs:\n    pool_handles (str or int): A flag to indicate which mempool to use.\n        `'default'` is for the device's default mempool, `'current'` is for\n        the current mempool (which could be the default one), and an `int`\n        that represents ``cudaMemPool_t`` created from elsewhere for an\n        external mempool. A list consisting of these flags can also be\n        accepted, in which case the list length must equal to the total\n        number of visible devices so that the mempools for each device can\n        be set independently.\n\n.. note::\n    :class:`MemoryAsyncPool` currently cannot work with memory hooks.\n\n.. seealso:: `Stream Ordered Memory Allocator`_\n\n.. _Stream Ordered Memory Allocator:\n    https://docs.nvidia.com/cuda/cuda-c-programming-guide/index.html#stream-ordered-memory-allocator")},
  {Py_tp_traverse, (void *)__pyx_tp_traverse_4cupy_4cuda_6memory_MemoryAsyncPool},
  {Py_tp_clear, (void *)__pyx_tp_clear_4cupy_4cuda_6memory_MemoryAsyncPool},
  {Py_tp_methods, (void *)__pyx_methods_4cupy_4cuda_6memory_MemoryAsyncPool},
  {Py_tp_getset, (void *)__pyx_getsets_4cupy_4cuda_6memory_MemoryAsyncPool},
  {Py_tp_init, (void *)__pyx_pw_4cupy_4cuda_6memory_15MemoryAsyncPool_1__init__},
  {Py_tp_new, (void *)__pyx_tp_new_4cupy_4cuda_6memory_MemoryAsyncPool},
  {0, 0},
};
static PyType_Spec __pyx_type_4cupy_4cuda_6memory_MemoryAsyncPool_spec = {
  "cupy.cuda.memory.MemoryAsyncPool",
  sizeof(struct __pyx_obj_4cupy_4cuda_6memory_MemoryAsyncPool),
  0,
  Py_TPFLAGS_DEFAULT|Py_TPFLAGS_HAVE_VERSION_TAG|Py_TPFLAGS_CHECKTYPES|Py_TPFLAGS_HAVE_NEWBUFFER|Py_TPFLAGS_BASETYPE|Py_TPFLAGS_HAVE_GC,
  __pyx_type_4cupy_4cuda_6memory_MemoryAsyncPool_slots,
};
#else

static PyTypeObject __pyx_type_4cupy_4cuda_6memory_MemoryAsyncPool = {
  PyVarObject_HEAD_INIT(0, 0)
  "cupy.cuda.memory.""MemoryAsyncPool", /*tp_name*/
  sizeof(struct __pyx_obj_4cupy_4cuda_6memory_MemoryAsyncPool), /*tp_basicsize*/
  0, /*tp_itemsize*/
  __pyx_tp_dealloc_4cupy_4cuda_6memory_MemoryAsyncPool, /*tp_dealloc*/
  0, /*tp_vectorcall_offset*/
  0, /*tp_getattr*/
  0, /*tp_setattr*/
  0, /*tp_as_async*/
  0, /*tp_repr*/
  0, /*tp_as_number*/
  0, /*tp_as_sequence*/
  0, /*tp_as_mapping*/
  0, /*tp_hash*/
  0, /*tp_call*/
  0, /*tp_str*/
  0, /*tp_getattro*/
  0, /*tp_setattro*/
  0, /*tp_as_buffer*/
  Py_TPFLAGS_DEFAULT|Py_TPFLAGS_HAVE_VERSION_TAG|Py_TPFLAGS_CHECKTYPES|Py_TPFLAGS_HAVE_NEWBUFFER|Py_TPFLAGS_BASETYPE|Py_TPFLAGS_HAVE_GC, /*tp_flags*/
  PyDoc_STR("MemoryAsyncPool(pool_handles='current')\n\nCUDA memory pool for all GPU devices on the host.\n\nA memory pool preserves any allocations even if they are freed by the user.\nOne instance of this class can be used for multiple devices. This class\nuses CUDA's Stream Ordered Memory Allocator (supported on CUDA 11.2+).\nThe simplest way to use this pool as CuPy's default allocator is the\nfollowing code::\n\n    set_allocator(MemoryAsyncPool().malloc)\n\nUsing this feature requires CUDA >= 11.2 with a supported GPU and platform.\nIf it is not supported, an error will be raised.\n\nThe current CuPy stream is used to allocate/free the memory.\n\nArgs:\n    pool_handles (str or int): A flag to indicate which mempool to use.\n        `'default'` is for the device's default mempool, `'current'` is for\n        the current mempool (which could be the default one), and an `int`\n        that represents ``cudaMemPool_t`` created from elsewhere for an\n        external mempool. A list consisting of these flags can also be\n        accepted, in which case the list length must equal to the total\n        number of visible devices so that the mempools for each device can\n        be set independently.\n\n.. note::\n    :class:`MemoryAsyncPool` currently cannot work with memory hooks.\n\n.. seealso:: `Stream Ordered Memory Allocator`_\n\n.. _Stream Ordered Memory Allocator:\n    https://docs.nvidia.com/cuda/cuda-c-programming-guide/index.html#stream-ordered-memory-allocator"), /*tp_doc*/
  __pyx_tp_traverse_4cupy_4cuda_6memory_MemoryAsyncPool, /*tp_traverse*/
  __pyx_tp_clear_4cupy_4cuda_6memory_MemoryAsyncPool, /*tp_clear*/
  0, /*tp_richcompare*/
  0, /*tp_weaklistoffset*/
  0, /*tp_iter*/
  0, /*tp_iternext*/
  __pyx_methods_4cupy_4cuda_6memory_MemoryAsyncPool, /*tp_methods*/
  0, /*tp_members*/
  __pyx_getsets_4cupy_4cuda_6memory_MemoryAsyncPool, /*tp_getset*/
  0, /*tp_base*/
  0, /*tp_dict*/
  0, /*tp_descr_get*/
  0, /*tp_descr_set*/
  #if !CYTHON_USE_TYPE_SPECS
  0, /*tp_dictoffset*/
  #endif
  __pyx_pw_4cupy_4cuda_6memory_15MemoryAsyncPool_1__init__, /*tp_init*/
  0, /*tp_alloc*/
  __pyx_tp_new_4cupy_4cuda_6memory_MemoryAsyncPool, /*tp_new*/
  0, /*tp_free*/
  0, /*tp_is_gc*/
  0, /*tp_bases*/
  0, /*tp_mro*/
  0, /*tp_cache*/
  0, /*tp_subclasses*/
  0, /*tp_weaklist*/
  0, /*tp_del*/
  0, /*tp_version_tag*/
  #if CYTHON_USE_TP_FINALIZE
  0, /*tp_finalize*/
  #else
  NULL, /*tp_finalize*/
  #endif
  #if !CYTHON_COMPILING_IN_PYPY || PYPY_VERSION_NUM >= 0x07030800
  0, /*tp_vectorcall*/
  #endif
  #if __PYX_NEED_TP_PRINT_SLOT == 1
  0, /*tp_print*/
  #endif
  #if PY_VERSION_HEX >= 0x030C0000
  0, /*tp_watched*/
  #endif
  #if PY_VERSION_HEX >= 0x030d00A4
  0, /*tp_versions_used*/
  #endif
  #if CYTHON_COMPILING_IN_PYPY && PY_VERSION_HEX >= 0x03090000 && PY_VERSION_HEX < 0x030a0000
  0, /*tp_pypy_flags*/
  #endif
};
#endif

static PyObject *__pyx_tp_new_4cupy_4cuda_6memory___pyx_scope_struct___ensure_pools_and_return_device_pool(PyTypeObject *t, CYTHON_UNUSED PyObject *a, CYTHON_UNUSED PyObject *k) {
  PyObject *o;
  #if CYTHON_USE_FREELISTS
  if (likely((int)(__pyx_mstate_global->__pyx_freecount_4cupy_4cuda_6memory___pyx_scope_struct___ensure_pools_and_return_device_pool > 0) & __PYX_CHECK_FINAL_TYPE_FOR_FREELISTS(t, __pyx_mstate_global->__pyx_ptype_4cupy_4cuda_6memory___pyx_scope_struct___ensure_pools_and_return_device_pool, sizeof(struct __pyx_obj_4cupy_4cuda_6memory___pyx_scope_struct___ensure_pools_and_return_device_pool))))
  {
    o = (PyObject*)__pyx_mstate_global->__pyx_freelist_4cupy_4cuda_6memory___pyx_scope_struct___ensure_pools_and_return_device_pool[--__pyx_mstate_global->__pyx_freecount_4cupy_4cuda_6memory___pyx_scope_struct___ensure_pools_and_return_device_pool];
    #if CYTHON_USE_TYPE_SPECS
    Py_DECREF(Py_TYPE(o));
    #endif
    memset(o, 0, sizeof(struct __pyx_obj_4cupy_4cuda_6memory___pyx_scope_struct___ensure_pools_and_return_device_pool));
    #if CYTHON_COMPILING_IN_LIMITED_API
    (void) PyObject_Init(o, t);
    #else
    (void) PyObject_INIT(o, t);
    #endif
    PyObject_GC_Track(o);
  } else
  #endif
  {
    o = __Pyx_AllocateExtensionType(t, 1);
    if (unlikely(!o)) return 0;
  }
  return o;
}

static void __pyx_tp_dealloc_4cupy_4cuda_6memory___pyx_scope_struct___ensure_pools_and_return_device_pool(PyObject *o) {
  struct __pyx_obj_4cupy_4cuda_6memory___pyx_scope_struct___ensure_pools_and_return_device_pool *p = (struct __pyx_obj_4cupy_4cuda_6memory___pyx_scope_struct___ensure_pools_and_return_device_pool *)o;
  #if CYTHON_USE_TP_FINALIZE
  if (unlikely(__Pyx_PyObject_GetSlot(o, tp_finalize, destructor)) && !__Pyx_PyObject_GC_IsFinalized(o)) {
    if (__Pyx_PyObject_GetSlot(o, tp_dealloc, destructor) == __pyx_tp_dealloc_4cupy_4cuda_6memory___pyx_scope_struct___ensure_pools_and_return_device_pool) {
      if (PyObject_CallFinalizerFromDealloc(o)) return;
    }
  }
  #endif
  PyObject_GC_UnTrack(o);
  Py_CLEAR(p->__pyx_v_self);
  #if CYTHON_USE_FREELISTS
  if (likely((int)(__pyx_mstate_global->__pyx_freecount_4cupy_4cuda_6memory___pyx_scope_struct___ensure_pools_and_return_device_pool < 8) & __PYX_CHECK_FINAL_TYPE_FOR_FREELISTS(Py_TYPE(o), __pyx_mstate_global->__pyx_ptype_4cupy_4cuda_6memory___pyx_scope_struct___ensure_pools_and_return_device_pool, sizeof(struct __pyx_obj_4cupy_4cuda_6memory___pyx_scope_struct___ensure_pools_and_return_device_pool))))
  {
    __pyx_mstate_global->__pyx_freelist_4cupy_4cuda_6memory___pyx_scope_struct___ensure_pools_and_return_device_pool[__pyx_mstate_global->__pyx_freecount_4cupy_4cuda_6memory___pyx_scope_struct___ensure_pools_and_return_device_pool++] = ((struct __pyx_obj_4cupy_4cuda_6memory___pyx_scope_struct___ensure_pools_and_return_device_pool *)o);
  } else
  #endif
  {
    PyTypeObject *tp = Py_TYPE(o);
    #if CYTHON_USE_TYPE_SLOTS
    (*tp->tp_free)(o);
    #else
    {
      freefunc tp_free = (freefunc)PyType_GetSlot(tp, Py_tp_free);
      if (tp_free) tp_free(o);
    }
    #endif
    #if CYTHON_USE_TYPE_SPECS
    Py_DECREF(tp);
    #endif
  }
}

static int __pyx_tp_traverse_4cupy_4cuda_6memory___pyx_scope_struct___ensure_pools_and_return_device_pool(PyObject *o, visitproc v, void *a) {
  int e;
  struct __pyx_obj_4cupy_4cuda_6memory___pyx_scope_struct___ensure_pools_and_return_device_pool *p = (struct __pyx_obj_4cupy_4cuda_6memory___pyx_scope_struct___ensure_pools_and_return_device_pool *)o;
  {
    e = __Pyx_call_type_traverse(o, 1, v, a);
    if (e) return e;
  }
  if (p->__pyx_v_self) {
    e = (*v)(((PyObject *)p->__pyx_v_self), a); if (e) return e;
  }
  return 0;
}

static int __pyx_tp_clear_4cupy_4cuda_6memory___pyx_scope_struct___ensure_pools_and_return_device_pool(PyObject *o) {
  PyObject* tmp;
  struct __pyx_obj_4cupy_4cuda_6memory___pyx_scope_struct___ensure_pools_and_return_device_pool *p = (struct __pyx_obj_4cupy_4cuda_6memory___pyx_scope_struct___ensure_pools_and_return_device_pool *)o;
  tmp = ((PyObject*)p->__pyx_v_self);
  p->__pyx_v_self = ((struct __pyx_obj_4cupy_4cuda_6memory_MemoryPool *)Py_None); Py_INCREF(Py_None);
  Py_XDECREF(tmp);
  return 0;
}
#if CYTHON_USE_TYPE_SPECS
static PyType_Slot __pyx_type_4cupy_4cuda_6memory___pyx_scope_struct___ensure_pools_and_return_device_pool_slots[] = {
  {Py_tp_dealloc, (void *)__pyx_tp_dealloc_4cupy_4cuda_6memory___pyx_scope_struct___ensure_pools_and_return_device_pool},
  {Py_tp_traverse, (void *)__pyx_tp_traverse_4cupy_4cuda_6memory___pyx_scope_struct___ensure_pools_and_return_device_pool},
  {Py_tp_clear, (void *)__pyx_tp_clear_4cupy_4cuda_6memory___pyx_scope_struct___ensure_pools_and_return_device_pool},
  {Py_tp_new, (void *)__pyx_tp_new_4cupy_4cuda_6memory___pyx_scope_struct___ensure_pools_and_return_device_pool},
  {0, 0},
};
static PyType_Spec __pyx_type_4cupy_4cuda_6memory___pyx_scope_struct___ensure_pools_and_return_device_pool_spec = {
  "cupy.cuda.memory.__pyx_scope_struct___ensure_pools_and_return_device_pool",
  sizeof(struct __pyx_obj_4cupy_4cuda_6memory___pyx_scope_struct___ensure_pools_and_return_device_pool),
  0,
  Py_TPFLAGS_DEFAULT|Py_TPFLAGS_HAVE_VERSION_TAG|Py_TPFLAGS_CHECKTYPES|Py_TPFLAGS_HAVE_NEWBUFFER|Py_TPFLAGS_HAVE_GC,
  __pyx_type_4cupy_4cuda_6memory___pyx_scope_struct___ensure_pools_and_return_device_pool_slots,
};
#else

static PyTypeObject __pyx_type_4cupy_4cuda_6memory___pyx_scope_struct___ensure_pools_and_return_device_pool = {
  PyVarObject_HEAD_INIT(0, 0)
  "cupy.cuda.memory.""__pyx_scope_struct___ensure_pools_and_return_device_pool", /*tp_name*/
  sizeof(struct __pyx_obj_4cupy_4cuda_6memory___pyx_scope_struct___ensure_pools_and_return_device_pool), /*tp_basicsize*/
  0, /*tp_itemsize*/
  __pyx_tp_dealloc_4cupy_4cuda_6memory___pyx_scope_struct___ensure_pools_and_return_device_pool, /*tp_dealloc*/
  0, /*tp_vectorcall_offset*/
  0, /*tp_getattr*/
  0, /*tp_setattr*/
  0, /*tp_as_async*/
  0, /*tp_repr*/
  0, /*tp_as_number*/
  0, /*tp_as_sequence*/
  0, /*tp_as_mapping*/
  0, /*tp_hash*/
  0, /*tp_call*/
  0, /*tp_str*/
  0, /*tp_getattro*/
  0, /*tp_setattro*/
  0, /*tp_as_buffer*/
  Py_TPFLAGS_DEFAULT|Py_TPFLAGS_HAVE_VERSION_TAG|Py_TPFLAGS_CHECKTYPES|Py_TPFLAGS_HAVE_NEWBUFFER|Py_TPFLAGS_HAVE_GC, /*tp_flags*/
  0, /*tp_doc*/
  __pyx_tp_traverse_4cupy_4cuda_6memory___pyx_scope_struct___ensure_pools_and_return_device_pool, /*tp_traverse*/
  __pyx_tp_clear_4cupy_4cuda_6memory___pyx_scope_struct___ensure_pools_and_return_device_pool, /*tp_clear*/
  0, /*tp_richcompare*/
  0, /*tp_weaklistoffset*/
  0, /*tp_iter*/
  0, /*tp_iternext*/
  0, /*tp_methods*/
  0, /*tp_members*/
  0, /*tp_getset*/
  0, /*tp_base*/
  0, /*tp_dict*/
  0, /*tp_descr_get*/
  0, /*tp_descr_set*/
  #if !CYTHON_USE_TYPE_SPECS
  0, /*tp_dictoffset*/
  #endif
  0, /*tp_init*/
  0, /*tp_alloc*/
  __pyx_tp_new_4cupy_4cuda_6memory___pyx_scope_struct___ensure_pools_and_return_device_pool, /*tp_new*/
  0, /*tp_free*/
  0, /*tp_is_gc*/
  0, /*tp_bases*/
  0, /*tp_mro*/
  0, /*tp_cache*/
  0, /*tp_subclasses*/
  0, /*tp_weaklist*/
  0, /*tp_del*/
  0, /*tp_version_tag*/
  #if CYTHON_USE_TP_FINALIZE
  0, /*tp_finalize*/
  #else
  NULL, /*tp_finalize*/
  #endif
  #if !CYTHON_COMPILING_IN_PYPY || PYPY_VERSION_NUM >= 0x07030800
  0, /*tp_vectorcall*/
  #endif
  #if __PYX_NEED_TP_PRINT_SLOT == 1
  0, /*tp_print*/
  #endif
  #if PY_VERSION_HEX >= 0x030C0000
  0, /*tp_watched*/
  #endif
  #if PY_VERSION_HEX >= 0x030d00A4
  0, /*tp_versions_used*/
  #endif
  #if CYTHON_COMPILING_IN_PYPY && PY_VERSION_HEX >= 0x03090000 && PY_VERSION_HEX < 0x030a0000
  0, /*tp_pypy_flags*/
  #endif
};
#endif

static PyObject *__pyx_tp_new_4cupy_4cuda_6memory___pyx_scope_struct_1_genexpr(PyTypeObject *t, CYTHON_UNUSED PyObject *a, CYTHON_UNUSED PyObject *k) {
  PyObject *o;
  #if CYTHON_USE_FREELISTS
  if (likely((int)(__pyx_mstate_global->__pyx_freecount_4cupy_4cuda_6memory___pyx_scope_struct_1_genexpr > 0) & __PYX_CHECK_FINAL_TYPE_FOR_FREELISTS(t, __pyx_mstate_global->__pyx_ptype_4cupy_4cuda_6memory___pyx_scope_struct_1_genexpr, sizeof(struct __pyx_obj_4cupy_4cuda_6memory___pyx_scope_struct_1_genexpr))))
  {
    o = (PyObject*)__pyx_mstate_global->__pyx_freelist_4cupy_4cuda_6memory___pyx_scope_struct_1_genexpr[--__pyx_mstate_global->__pyx_freecount_4cupy_4cuda_6memory___pyx_scope_struct_1_genexpr];
    #if CYTHON_USE_TYPE_SPECS
    Py_DECREF(Py_TYPE(o));
    #endif
    memset(o, 0, sizeof(struct __pyx_obj_4cupy_4cuda_6memory___pyx_scope_struct_1_genexpr));
    #if CYTHON_COMPILING_IN_LIMITED_API
    (void) PyObject_Init(o, t);
    #else
    (void) PyObject_INIT(o, t);
    #endif
    PyObject_GC_Track(o);
  } else
  #endif
  {
    o = __Pyx_AllocateExtensionType(t, 1);
    if (unlikely(!o)) return 0;
  }
  return o;
}

static void __pyx_tp_dealloc_4cupy_4cuda_6memory___pyx_scope_struct_1_genexpr(PyObject *o) {
  struct __pyx_obj_4cupy_4cuda_6memory___pyx_scope_struct_1_genexpr *p = (struct __pyx_obj_4cupy_4cuda_6memory___pyx_scope_struct_1_genexpr *)o;
  #if CYTHON_USE_TP_FINALIZE
  if (unlikely(__Pyx_PyObject_GetSlot(o, tp_finalize, destructor)) && !__Pyx_PyObject_GC_IsFinalized(o)) {
    if (__Pyx_PyObject_GetSlot(o, tp_dealloc, destructor) == __pyx_tp_dealloc_4cupy_4cuda_6memory___pyx_scope_struct_1_genexpr) {
      if (PyObject_CallFinalizerFromDealloc(o)) return;
    }
  }
  #endif
  PyObject_GC_UnTrack(o);
  Py_CLEAR(p->__pyx_outer_scope);
  #if CYTHON_USE_FREELISTS
  if (likely((int)(__pyx_mstate_global->__pyx_freecount_4cupy_4cuda_6memory___pyx_scope_struct_1_genexpr < 8) & __PYX_CHECK_FINAL_TYPE_FOR_FREELISTS(Py_TYPE(o), __pyx_mstate_global->__pyx_ptype_4cupy_4cuda_6memory___pyx_scope_struct_1_genexpr, sizeof(struct __pyx_obj_4cupy_4cuda_6memory___pyx_scope_struct_1_genexpr))))
  {
    __pyx_mstate_global->__pyx_freelist_4cupy_4cuda_6memory___pyx_scope_struct_1_genexpr[__pyx_mstate_global->__pyx_freecount_4cupy_4cuda_6memory___pyx_scope_struct_1_genexpr++] = ((struct __pyx_obj_4cupy_4cuda_6memory___pyx_scope_struct_1_genexpr *)o);
  } else
  #endif
  {
    PyTypeObject *tp = Py_TYPE(o);
    #if CYTHON_USE_TYPE_SLOTS
    (*tp->tp_free)(o);
    #else
    {
      freefunc tp_free = (freefunc)PyType_GetSlot(tp, Py_tp_free);
      if (tp_free) tp_free(o);
    }
    #endif
    #if CYTHON_USE_TYPE_SPECS
    Py_DECREF(tp);
    #endif
  }
}

static int __pyx_tp_traverse_4cupy_4cuda_6memory___pyx_scope_struct_1_genexpr(PyObject *o, visitproc v, void *a) {
  int e;
  struct __pyx_obj_4cupy_4cuda_6memory___pyx_scope_struct_1_genexpr *p = (struct __pyx_obj_4cupy_4cuda_6memory___pyx_scope_struct_1_genexpr *)o;
  {
    e = __Pyx_call_type_traverse(o, 1, v, a);
    if (e) return e;
  }
  if (p->__pyx_outer_scope) {
    e = (*v)(((PyObject *)p->__pyx_outer_scope), a); if (e) return e;
  }
  return 0;
}
#if CYTHON_USE_TYPE_SPECS
static PyType_Slot __pyx_type_4cupy_4cuda_6memory___pyx_scope_struct_1_genexpr_slots[] = {
  {Py_tp_dealloc, (void *)__pyx_tp_dealloc_4cupy_4cuda_6memory___pyx_scope_struct_1_genexpr},
  {Py_tp_traverse, (void *)__pyx_tp_traverse_4cupy_4cuda_6memory___pyx_scope_struct_1_genexpr},
  {Py_tp_new, (void *)__pyx_tp_new_4cupy_4cuda_6memory___pyx_scope_struct_1_genexpr},
  {0, 0},
};
static PyType_Spec __pyx_type_4cupy_4cuda_6memory___pyx_scope_struct_1_genexpr_spec = {
  "cupy.cuda.memory.__pyx_scope_struct_1_genexpr",
  sizeof(struct __pyx_obj_4cupy_4cuda_6memory___pyx_scope_struct_1_genexpr),
  0,
  Py_TPFLAGS_DEFAULT|Py_TPFLAGS_HAVE_VERSION_TAG|Py_TPFLAGS_CHECKTYPES|Py_TPFLAGS_HAVE_NEWBUFFER|Py_TPFLAGS_HAVE_GC,
  __pyx_type_4cupy_4cuda_6memory___pyx_scope_struct_1_genexpr_slots,
};
#else

static PyTypeObject __pyx_type_4cupy_4cuda_6memory___pyx_scope_struct_1_genexpr = {
  PyVarObject_HEAD_INIT(0, 0)
  "cupy.cuda.memory.""__pyx_scope_struct_1_genexpr", /*tp_name*/
  sizeof(struct __pyx_obj_4cupy_4cuda_6memory___pyx_scope_struct_1_genexpr), /*tp_basicsize*/
  0, /*tp_itemsize*/
  __pyx_tp_dealloc_4cupy_4cuda_6memory___pyx_scope_struct_1_genexpr, /*tp_dealloc*/
  0, /*tp_vectorcall_offset*/
  0, /*tp_getattr*/
  0, /*tp_setattr*/
  0, /*tp_as_async*/
  0, /*tp_repr*/
  0, /*tp_as_number*/
  0, /*tp_as_sequence*/
  0, /*tp_as_mapping*/
  0, /*tp_hash*/
  0, /*tp_call*/
  0, /*tp_str*/
  0, /*tp_getattro*/
  0, /*tp_setattro*/
  0, /*tp_as_buffer*/
  Py_TPFLAGS_DEFAULT|Py_TPFLAGS_HAVE_VERSION_TAG|Py_TPFLAGS_CHECKTYPES|Py_TPFLAGS_HAVE_NEWBUFFER|Py_TPFLAGS_HAVE_GC, /*tp_flags*/
  0, /*tp_doc*/
  __pyx_tp_traverse_4cupy_4cuda_6memory___pyx_scope_struct_1_genexpr, /*tp_traverse*/
  0, /*tp_clear*/
  0, /*tp_richcompare*/
  0, /*tp_weaklistoffset*/
  0, /*tp_iter*/
  0, /*tp_iternext*/
  0, /*tp_methods*/
  0, /*tp_members*/
  0, /*tp_getset*/
  0, /*tp_base*/
  0, /*tp_dict*/
  0, /*tp_descr_get*/
  0, /*tp_descr_set*/
  #if !CYTHON_USE_TYPE_SPECS
  0, /*tp_dictoffset*/
  #endif
  0, /*tp_init*/
  0, /*tp_alloc*/
  __pyx_tp_new_4cupy_4cuda_6memory___pyx_scope_struct_1_genexpr, /*tp_new*/
  0, /*tp_free*/
  0, /*tp_is_gc*/
  0, /*tp_bases*/
  0, /*tp_mro*/
  0, /*tp_cache*/
  0, /*tp_subclasses*/
  0, /*tp_weaklist*/
  0, /*tp_del*/
  0, /*tp_version_tag*/
  #if CYTHON_USE_TP_FINALIZE
  0, /*tp_finalize*/
  #else
  NULL, /*tp_finalize*/
  #endif
  #if !CYTHON_COMPILING_IN_PYPY || PYPY_VERSION_NUM >= 0x07030800
  0, /*tp_vectorcall*/
  #endif
  #if __PYX_NEED_TP_PRINT_SLOT == 1
  0, /*tp_print*/
  #endif
  #if PY_VERSION_HEX >= 0x030C0000
  0, /*tp_watched*/
  #endif
  #if PY_VERSION_HEX >= 0x030d00A4
  0, /*tp_versions_used*/
  #endif
  #if CYTHON_COMPILING_IN_PYPY && PY_VERSION_HEX >= 0x03090000 && PY_VERSION_HEX < 0x030a0000
  0, /*tp_pypy_flags*/
  #endif
};
#endif

static PyMethodDef __pyx_methods[] = {
  {"_malloc", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_4cupy_4cuda_6memory_3_malloc, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_4cupy_4cuda_6memory_2_malloc},
  {"malloc_async", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_4cupy_4cuda_6memory_5malloc_async, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_4cupy_4cuda_6memory_4malloc_async},
  {"malloc_managed", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_4cupy_4cuda_6memory_7malloc_managed, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_4cupy_4cuda_6memory_6malloc_managed},
  {"malloc_system", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_4cupy_4cuda_6memory_9malloc_system, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_4cupy_4cuda_6memory_8malloc_system},
  {"alloc", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_4cupy_4cuda_6memory_15alloc, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_4cupy_4cuda_6memory_14alloc},
  {"set_allocator", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_4cupy_4cuda_6memory_17set_allocator, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_4cupy_4cuda_6memory_16set_allocator},
  {"get_allocator", (PyCFunction)__pyx_pw_4cupy_4cuda_6memory_19get_allocator, METH_NOARGS, __pyx_doc_4cupy_4cuda_6memory_18get_allocator},
  {"_round_size", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_4cupy_4cuda_6memory_21_round_size, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_4cupy_4cuda_6memory_20_round_size},
  {"_parse_limit_string", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_4cupy_4cuda_6memory_23_parse_limit_string, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_4cupy_4cuda_6memory_22_parse_limit_string},
  {0, 0, 0, 0}
};
/* #### Code section: initfunc_declarations ### */
static CYTHON_SMALL_CODE int __Pyx_InitCachedBuiltins(__pyx_mstatetype *__pyx_mstate); /*proto*/
static CYTHON_SMALL_CODE int __Pyx_InitCachedConstants(__pyx_mstatetype *__pyx_mstate); /*proto*/
static CYTHON_SMALL_CODE int __Pyx_InitGlobals(void); /*proto*/
static CYTHON_SMALL_CODE int __Pyx_InitConstants(__pyx_mstatetype *__pyx_mstate); /*proto*/
static CYTHON_SMALL_CODE int __Pyx_modinit_global_init_code(__pyx_mstatetype *__pyx_mstate); /*proto*/
static CYTHON_SMALL_CODE int __Pyx_modinit_variable_export_code(__pyx_mstatetype *__pyx_mstate); /*proto*/
static CYTHON_SMALL_CODE int __Pyx_modinit_function_export_code(__pyx_mstatetype *__pyx_mstate); /*proto*/
static CYTHON_SMALL_CODE int __Pyx_modinit_type_init_code(__pyx_mstatetype *__pyx_mstate); /*proto*/
static CYTHON_SMALL_CODE int __Pyx_modinit_type_import_code(__pyx_mstatetype *__pyx_mstate); /*proto*/
static CYTHON_SMALL_CODE int __Pyx_modinit_variable_import_code(__pyx_mstatetype *__pyx_mstate); /*proto*/
static CYTHON_SMALL_CODE int __Pyx_modinit_function_import_code(__pyx_mstatetype *__pyx_mstate); /*proto*/
static CYTHON_SMALL_CODE int __Pyx_CreateCodeObjects(__pyx_mstatetype *__pyx_mstate); /*proto*/
/* #### Code section: init_module ### */

static int __Pyx_modinit_global_init_code(__pyx_mstatetype *__pyx_mstate) {
  __Pyx_RefNannyDeclarations
  CYTHON_UNUSED_VAR(__pyx_mstate);
  __Pyx_RefNannySetupContext("__Pyx_modinit_global_init_code", 0);
  /*--- Global init code ---*/
  __pyx_v_4cupy_4cuda_6memory__current_allocator = Py_None; Py_INCREF(Py_None);
  __pyx_v_4cupy_4cuda_6memory__thread_local = Py_None; Py_INCREF(Py_None);
  __Pyx_RefNannyFinishContext();
  return 0;
}

static int __Pyx_modinit_variable_export_code(__pyx_mstatetype *__pyx_mstate) {
  __Pyx_RefNannyDeclarations
  CYTHON_UNUSED_VAR(__pyx_mstate);
  __Pyx_RefNannySetupContext("__Pyx_modinit_variable_export_code", 0);
  /*--- Variable export code ---*/
  __Pyx_RefNannyFinishContext();
  return 0;
}

static int __Pyx_modinit_function_export_code(__pyx_mstatetype *__pyx_mstate) {
  __Pyx_RefNannyDeclarations
  CYTHON_UNUSED_VAR(__pyx_mstate);
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__Pyx_modinit_function_export_code", 0);
  /*--- Function export code ---*/
  {
    __pyx_t_1 = __Pyx_ApiExport_GetApiDict(); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    const char * __pyx_export_signature = __Pyx_PyBytes_AsString(__pyx_mstate_global->__pyx_kp_b_PyObject_int___pyx_skip_dispatch);
    #if !CYTHON_ASSUME_SAFE_MACROS
    if (unlikely(!__pyx_export_signature)) __PYX_ERR(0, 1, __pyx_L1_error)
    #endif
    const char * __pyx_export_name = __pyx_export_signature + 240;
    void (*const __pyx_export_pointers[])(void) = {(void (*)(void))&__pyx_f_4cupy_4cuda_6memory_get_allocator, (void (*)(void))&__pyx_f_4cupy_4cuda_6memory_set_allocator, (void (*)(void))&__pyx_f_4cupy_4cuda_6memory_alloc, (void (*)(void)) NULL};
    void (*const *__pyx_export_pointer)(void) = __pyx_export_pointers;
    const char *__pyx_export_current_signature = __pyx_export_signature;
    while (*__pyx_export_pointer) {
      if (__Pyx_ExportFunction(__pyx_t_1, __pyx_export_name, *__pyx_export_pointer, __pyx_export_current_signature) < (0)) __PYX_ERR(0, 1, __pyx_L1_error)
      ++__pyx_export_pointer;
      __pyx_export_name = strchr(__pyx_export_name, '\0') + 1;
      __pyx_export_signature = strchr(__pyx_export_signature, '\0') + 1;
      if (*__pyx_export_signature != '\0') __pyx_export_current_signature = __pyx_export_signature;
    }
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  }
  __Pyx_RefNannyFinishContext();
  return 0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_RefNannyFinishContext();
  return -1;
}

static int __Pyx_modinit_type_init_code(__pyx_mstatetype *__pyx_mstate) {
  __Pyx_RefNannyDeclarations
  CYTHON_UNUSED_VAR(__pyx_mstate);
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__Pyx_modinit_type_init_code", 0);
  /*--- Type init code ---*/
  #if CYTHON_USE_TYPE_SPECS
  __pyx_mstate->__pyx_ptype_4cupy_4cuda_6memory_BaseMemory = (PyTypeObject *) __Pyx_PyType_FromModuleAndSpec(__pyx_m, &__pyx_type_4cupy_4cuda_6memory_BaseMemory_spec, NULL); if (unlikely(!__pyx_mstate->__pyx_ptype_4cupy_4cuda_6memory_BaseMemory)) __PYX_ERR(0, 84, __pyx_L1_error)
  if (__Pyx_fix_up_extension_type_from_spec(&__pyx_type_4cupy_4cuda_6memory_BaseMemory_spec, __pyx_mstate->__pyx_ptype_4cupy_4cuda_6memory_BaseMemory) < (0)) __PYX_ERR(0, 84, __pyx_L1_error)
  #else
  __pyx_mstate->__pyx_ptype_4cupy_4cuda_6memory_BaseMemory = &__pyx_type_4cupy_4cuda_6memory_BaseMemory;
  #endif
  #if !CYTHON_COMPILING_IN_LIMITED_API
  #endif
  #if !CYTHON_USE_TYPE_SPECS
  if (__Pyx_PyType_Ready(__pyx_mstate->__pyx_ptype_4cupy_4cuda_6memory_BaseMemory) < (0)) __PYX_ERR(0, 84, __pyx_L1_error)
  #endif
  #if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x030E0000
  PyUnstable_Object_EnableDeferredRefcount((PyObject*)__pyx_mstate->__pyx_ptype_4cupy_4cuda_6memory_BaseMemory);
  #endif
  #if !CYTHON_COMPILING_IN_LIMITED_API
  if ((CYTHON_USE_TYPE_SLOTS && CYTHON_USE_PYTYPE_LOOKUP) && likely(!__pyx_mstate->__pyx_ptype_4cupy_4cuda_6memory_BaseMemory->tp_dictoffset && __pyx_mstate->__pyx_ptype_4cupy_4cuda_6memory_BaseMemory->tp_getattro == PyObject_GenericGetAttr)) {
    __pyx_mstate->__pyx_ptype_4cupy_4cuda_6memory_BaseMemory->tp_getattro = PyObject_GenericGetAttr;
  }
  #endif
  #if CYTHON_UPDATE_DESCRIPTOR_DOC
  {
    PyObject *wrapper = PyObject_GetAttrString((PyObject *)__pyx_mstate->__pyx_ptype_4cupy_4cuda_6memory_BaseMemory, "__int__"); if (unlikely(!wrapper)) __PYX_ERR(0, 84, __pyx_L1_error)
    if (__Pyx_IS_TYPE(wrapper, &PyWrapperDescr_Type)) {
      __pyx_wrapperbase_4cupy_4cuda_6memory_10BaseMemory___int__ = *((PyWrapperDescrObject *)wrapper)->d_base;
      __pyx_wrapperbase_4cupy_4cuda_6memory_10BaseMemory___int__.doc = __pyx_doc_4cupy_4cuda_6memory_10BaseMemory___int__;
      ((PyWrapperDescrObject *)wrapper)->d_base = &__pyx_wrapperbase_4cupy_4cuda_6memory_10BaseMemory___int__;
    }
  }
  #endif
  if (PyObject_SetAttr(__pyx_m, __pyx_mstate_global->__pyx_n_u_BaseMemory, (PyObject *) __pyx_mstate->__pyx_ptype_4cupy_4cuda_6memory_BaseMemory) < (0)) __PYX_ERR(0, 84, __pyx_L1_error)
  if (__Pyx_setup_reduce((PyObject *) __pyx_mstate->__pyx_ptype_4cupy_4cuda_6memory_BaseMemory) < (0)) __PYX_ERR(0, 84, __pyx_L1_error)
  __pyx_vtabptr_4cupy_4cuda_6memory_SystemMemory = &__pyx_vtable_4cupy_4cuda_6memory_SystemMemory;
  __pyx_vtable_4cupy_4cuda_6memory_SystemMemory.from_external = (PyObject *(*)(intptr_t, size_t, PyObject *))__pyx_f_4cupy_4cuda_6memory_12SystemMemory_from_external;
  #if CYTHON_USE_TYPE_SPECS
  __pyx_t_1 = PyTuple_Pack(1, (PyObject *)__pyx_mstate_global->__pyx_ptype_4cupy_4cuda_6memory_BaseMemory); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 273, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_mstate->__pyx_ptype_4cupy_4cuda_6memory_SystemMemory = (PyTypeObject *) __Pyx_PyType_FromModuleAndSpec(__pyx_m, &__pyx_type_4cupy_4cuda_6memory_SystemMemory_spec, __pyx_t_1);
  __Pyx_XDECREF(__pyx_t_1); __pyx_t_1 = 0;
  if (unlikely(!__pyx_mstate->__pyx_ptype_4cupy_4cuda_6memory_SystemMemory)) __PYX_ERR(0, 273, __pyx_L1_error)
  if (__Pyx_fix_up_extension_type_from_spec(&__pyx_type_4cupy_4cuda_6memory_SystemMemory_spec, __pyx_mstate->__pyx_ptype_4cupy_4cuda_6memory_SystemMemory) < (0)) __PYX_ERR(0, 273, __pyx_L1_error)
  #else
  __pyx_mstate->__pyx_ptype_4cupy_4cuda_6memory_SystemMemory = &__pyx_type_4cupy_4cuda_6memory_SystemMemory;
  #endif
  #if !CYTHON_COMPILING_IN_LIMITED_API
  __pyx_mstate_global->__pyx_ptype_4cupy_4cuda_6memory_SystemMemory->tp_base = __pyx_mstate_global->__pyx_ptype_4cupy_4cuda_6memory_BaseMemory;
  #endif
  #if !CYTHON_USE_TYPE_SPECS
  if (__Pyx_PyType_Ready(__pyx_mstate->__pyx_ptype_4cupy_4cuda_6memory_SystemMemory) < (0)) __PYX_ERR(0, 273, __pyx_L1_error)
  #endif
  #if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x030E0000
  PyUnstable_Object_EnableDeferredRefcount((PyObject*)__pyx_mstate->__pyx_ptype_4cupy_4cuda_6memory_SystemMemory);
  #endif
  #if !CYTHON_COMPILING_IN_LIMITED_API
  if ((CYTHON_USE_TYPE_SLOTS && CYTHON_USE_PYTYPE_LOOKUP) && likely(!__pyx_mstate->__pyx_ptype_4cupy_4cuda_6memory_SystemMemory->tp_dictoffset && __pyx_mstate->__pyx_ptype_4cupy_4cuda_6memory_SystemMemory->tp_getattro == PyObject_GenericGetAttr)) {
    __pyx_mstate->__pyx_ptype_4cupy_4cuda_6memory_SystemMemory->tp_getattro = PyObject_GenericGetAttr;
  }
  #endif
  if (__Pyx_SetVtable(__pyx_mstate->__pyx_ptype_4cupy_4cuda_6memory_SystemMemory, __pyx_vtabptr_4cupy_4cuda_6memory_SystemMemory) < (0)) __PYX_ERR(0, 273, __pyx_L1_error)
  if (__Pyx_MergeVtables(__pyx_mstate->__pyx_ptype_4cupy_4cuda_6memory_SystemMemory) < (0)) __PYX_ERR(0, 273, __pyx_L1_error)
  if (PyObject_SetAttr(__pyx_m, __pyx_mstate_global->__pyx_n_u_SystemMemory, (PyObject *) __pyx_mstate->__pyx_ptype_4cupy_4cuda_6memory_SystemMemory) < (0)) __PYX_ERR(0, 273, __pyx_L1_error)
  if (__Pyx_setup_reduce((PyObject *) __pyx_mstate->__pyx_ptype_4cupy_4cuda_6memory_SystemMemory) < (0)) __PYX_ERR(0, 273, __pyx_L1_error)
  __pyx_vtabptr_4cupy_4cuda_6memory_MemoryPointer = &__pyx_vtable_4cupy_4cuda_6memory_MemoryPointer;
  __pyx_vtable_4cupy_4cuda_6memory_MemoryPointer._init = (PyObject *(*)(struct __pyx_obj_4cupy_4cuda_6memory_MemoryPointer *, struct __pyx_obj_4cupy_4cuda_6memory_BaseMemory *, ptrdiff_t))__pyx_f_4cupy_4cuda_6memory_13MemoryPointer__init;
  __pyx_vtable_4cupy_4cuda_6memory_MemoryPointer.copy_from_device = (PyObject *(*)(struct __pyx_obj_4cupy_4cuda_6memory_MemoryPointer *, struct __pyx_obj_4cupy_4cuda_6memory_MemoryPointer *, size_t, int __pyx_skip_dispatch))__pyx_f_4cupy_4cuda_6memory_13MemoryPointer_copy_from_device;
  __pyx_vtable_4cupy_4cuda_6memory_MemoryPointer.copy_from_device_async = (PyObject *(*)(struct __pyx_obj_4cupy_4cuda_6memory_MemoryPointer *, struct __pyx_obj_4cupy_4cuda_6memory_MemoryPointer *, size_t, int __pyx_skip_dispatch, struct __pyx_opt_args_4cupy_4cuda_6memory_13MemoryPointer_copy_from_device_async *__pyx_optional_args))__pyx_f_4cupy_4cuda_6memory_13MemoryPointer_copy_from_device_async;
  __pyx_vtable_4cupy_4cuda_6memory_MemoryPointer.copy_from_host = (PyObject *(*)(struct __pyx_obj_4cupy_4cuda_6memory_MemoryPointer *, PyObject *, size_t, int __pyx_skip_dispatch))__pyx_f_4cupy_4cuda_6memory_13MemoryPointer_copy_from_host;
  __pyx_vtable_4cupy_4cuda_6memory_MemoryPointer.copy_from_host_async = (PyObject *(*)(struct __pyx_obj_4cupy_4cuda_6memory_MemoryPointer *, PyObject *, size_t, int __pyx_skip_dispatch, struct __pyx_opt_args_4cupy_4cuda_6memory_13MemoryPointer_copy_from_host_async *__pyx_optional_args))__pyx_f_4cupy_4cuda_6memory_13MemoryPointer_copy_from_host_async;
  __pyx_vtable_4cupy_4cuda_6memory_MemoryPointer.copy_from = (PyObject *(*)(struct __pyx_obj_4cupy_4cuda_6memory_MemoryPointer *, PyObject *, size_t, int __pyx_skip_dispatch))__pyx_f_4cupy_4cuda_6memory_13MemoryPointer_copy_from;
  __pyx_vtable_4cupy_4cuda_6memory_MemoryPointer.copy_from_async = (PyObject *(*)(struct __pyx_obj_4cupy_4cuda_6memory_MemoryPointer *, PyObject *, size_t, int __pyx_skip_dispatch, struct __pyx_opt_args_4cupy_4cuda_6memory_13MemoryPointer_copy_from_async *__pyx_optional_args))__pyx_f_4cupy_4cuda_6memory_13MemoryPointer_copy_from_async;
  __pyx_vtable_4cupy_4cuda_6memory_MemoryPointer.copy_to_host = (PyObject *(*)(struct __pyx_obj_4cupy_4cuda_6memory_MemoryPointer *, PyObject *, size_t, int __pyx_skip_dispatch))__pyx_f_4cupy_4cuda_6memory_13MemoryPointer_copy_to_host;
  __pyx_vtable_4cupy_4cuda_6memory_MemoryPointer.copy_to_host_async = (PyObject *(*)(struct __pyx_obj_4cupy_4cuda_6memory_MemoryPointer *, PyObject *, size_t, int __pyx_skip_dispatch, struct __pyx_opt_args_4cupy_4cuda_6memory_13MemoryPointer_copy_to_host_async *__pyx_optional_args))__pyx_f_4cupy_4cuda_6memory_13MemoryPointer_copy_to_host_async;
  __pyx_vtable_4cupy_4cuda_6memory_MemoryPointer.memset = (PyObject *(*)(struct __pyx_obj_4cupy_4cuda_6memory_MemoryPointer *, int, size_t, int __pyx_skip_dispatch))__pyx_f_4cupy_4cuda_6memory_13MemoryPointer_memset;
  __pyx_vtable_4cupy_4cuda_6memory_MemoryPointer.memset_async = (PyObject *(*)(struct __pyx_obj_4cupy_4cuda_6memory_MemoryPointer *, int, size_t, int __pyx_skip_dispatch, struct __pyx_opt_args_4cupy_4cuda_6memory_13MemoryPointer_memset_async *__pyx_optional_args))__pyx_f_4cupy_4cuda_6memory_13MemoryPointer_memset_async;
  #if CYTHON_USE_TYPE_SPECS
  __pyx_mstate->__pyx_ptype_4cupy_4cuda_6memory_MemoryPointer = (PyTypeObject *) __Pyx_PyType_FromModuleAndSpec(__pyx_m, &__pyx_type_4cupy_4cuda_6memory_MemoryPointer_spec, NULL); if (unlikely(!__pyx_mstate->__pyx_ptype_4cupy_4cuda_6memory_MemoryPointer)) __PYX_ERR(0, 458, __pyx_L1_error)
  if (__Pyx_fix_up_extension_type_from_spec(&__pyx_type_4cupy_4cuda_6memory_MemoryPointer_spec, __pyx_mstate->__pyx_ptype_4cupy_4cuda_6memory_MemoryPointer) < (0)) __PYX_ERR(0, 458, __pyx_L1_error)
  #else
  __pyx_mstate->__pyx_ptype_4cupy_4cuda_6memory_MemoryPointer = &__pyx_type_4cupy_4cuda_6memory_MemoryPointer;
  #endif
  #if !CYTHON_COMPILING_IN_LIMITED_API
  #endif
  #if !CYTHON_USE_TYPE_SPECS
  if (__Pyx_PyType_Ready(__pyx_mstate->__pyx_ptype_4cupy_4cuda_6memory_MemoryPointer) < (0)) __PYX_ERR(0, 458, __pyx_L1_error)
  #endif
  #if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x030E0000
  PyUnstable_Object_EnableDeferredRefcount((PyObject*)__pyx_mstate->__pyx_ptype_4cupy_4cuda_6memory_MemoryPointer);
  #endif
  #if !CYTHON_COMPILING_IN_LIMITED_API
  if ((CYTHON_USE_TYPE_SLOTS && CYTHON_USE_PYTYPE_LOOKUP) && likely(!__pyx_mstate->__pyx_ptype_4cupy_4cuda_6memory_MemoryPointer->tp_dictoffset && __pyx_mstate->__pyx_ptype_4cupy_4cuda_6memory_MemoryPointer->tp_getattro == PyObject_GenericGetAttr)) {
    __pyx_mstate->__pyx_ptype_4cupy_4cuda_6memory_MemoryPointer->tp_getattro = PyObject_GenericGetAttr;
  }
  #endif
  #if CYTHON_UPDATE_DESCRIPTOR_DOC
  {
    PyObject *wrapper = PyObject_GetAttrString((PyObject *)__pyx_mstate->__pyx_ptype_4cupy_4cuda_6memory_MemoryPointer, "__int__"); if (unlikely(!wrapper)) __PYX_ERR(0, 458, __pyx_L1_error)
    if (__Pyx_IS_TYPE(wrapper, &PyWrapperDescr_Type)) {
      __pyx_wrapperbase_4cupy_4cuda_6memory_13MemoryPointer_2__int__ = *((PyWrapperDescrObject *)wrapper)->d_base;
      __pyx_wrapperbase_4cupy_4cuda_6memory_13MemoryPointer_2__int__.doc = __pyx_doc_4cupy_4cuda_6memory_13MemoryPointer_2__int__;
      ((PyWrapperDescrObject *)wrapper)->d_base = &__pyx_wrapperbase_4cupy_4cuda_6memory_13MemoryPointer_2__int__;
    }
  }
  #endif
  #if CYTHON_UPDATE_DESCRIPTOR_DOC
  {
    PyObject *wrapper = PyObject_GetAttrString((PyObject *)__pyx_mstate->__pyx_ptype_4cupy_4cuda_6memory_MemoryPointer, "__add__"); if (unlikely(!wrapper)) __PYX_ERR(0, 458, __pyx_L1_error)
    if (__Pyx_IS_TYPE(wrapper, &PyWrapperDescr_Type)) {
      __pyx_wrapperbase_4cupy_4cuda_6memory_13MemoryPointer_6__add__ = *((PyWrapperDescrObject *)wrapper)->d_base;
      __pyx_wrapperbase_4cupy_4cuda_6memory_13MemoryPointer_6__add__.doc = __pyx_doc_4cupy_4cuda_6memory_13MemoryPointer_6__add__;
      ((PyWrapperDescrObject *)wrapper)->d_base = &__pyx_wrapperbase_4cupy_4cuda_6memory_13MemoryPointer_6__add__;
    }
  }
  #endif
  #if CYTHON_UPDATE_DESCRIPTOR_DOC
  {
    PyObject *wrapper = PyObject_GetAttrString((PyObject *)__pyx_mstate->__pyx_ptype_4cupy_4cuda_6memory_MemoryPointer, "__iadd__"); if (unlikely(!wrapper)) __PYX_ERR(0, 458, __pyx_L1_error)
    if (__Pyx_IS_TYPE(wrapper, &PyWrapperDescr_Type)) {
      __pyx_wrapperbase_4cupy_4cuda_6memory_13MemoryPointer_8__iadd__ = *((PyWrapperDescrObject *)wrapper)->d_base;
      __pyx_wrapperbase_4cupy_4cuda_6memory_13MemoryPointer_8__iadd__.doc = __pyx_doc_4cupy_4cuda_6memory_13MemoryPointer_8__iadd__;
      ((PyWrapperDescrObject *)wrapper)->d_base = &__pyx_wrapperbase_4cupy_4cuda_6memory_13MemoryPointer_8__iadd__;
    }
  }
  #endif
  #if CYTHON_UPDATE_DESCRIPTOR_DOC
  {
    PyObject *wrapper = PyObject_GetAttrString((PyObject *)__pyx_mstate->__pyx_ptype_4cupy_4cuda_6memory_MemoryPointer, "__sub__"); if (unlikely(!wrapper)) __PYX_ERR(0, 458, __pyx_L1_error)
    if (__Pyx_IS_TYPE(wrapper, &PyWrapperDescr_Type)) {
      __pyx_wrapperbase_4cupy_4cuda_6memory_13MemoryPointer_10__sub__ = *((PyWrapperDescrObject *)wrapper)->d_base;
      __pyx_wrapperbase_4cupy_4cuda_6memory_13MemoryPointer_10__sub__.doc = __pyx_doc_4cupy_4cuda_6memory_13MemoryPointer_10__sub__;
      ((PyWrapperDescrObject *)wrapper)->d_base = &__pyx_wrapperbase_4cupy_4cuda_6memory_13MemoryPointer_10__sub__;
    }
  }
  #endif
  #if CYTHON_UPDATE_DESCRIPTOR_DOC
  {
    PyObject *wrapper = PyObject_GetAttrString((PyObject *)__pyx_mstate->__pyx_ptype_4cupy_4cuda_6memory_MemoryPointer, "__isub__"); if (unlikely(!wrapper)) __PYX_ERR(0, 458, __pyx_L1_error)
    if (__Pyx_IS_TYPE(wrapper, &PyWrapperDescr_Type)) {
      __pyx_wrapperbase_4cupy_4cuda_6memory_13MemoryPointer_12__isub__ = *((PyWrapperDescrObject *)wrapper)->d_base;
      __pyx_wrapperbase_4cupy_4cuda_6memory_13MemoryPointer_12__isub__.doc = __pyx_doc_4cupy_4cuda_6memory_13MemoryPointer_12__isub__;
      ((PyWrapperDescrObject *)wrapper)->d_base = &__pyx_wrapperbase_4cupy_4cuda_6memory_13MemoryPointer_12__isub__;
    }
  }
  #endif
  if (__Pyx_SetVtable(__pyx_mstate->__pyx_ptype_4cupy_4cuda_6memory_MemoryPointer, __pyx_vtabptr_4cupy_4cuda_6memory_MemoryPointer) < (0)) __PYX_ERR(0, 458, __pyx_L1_error)
  if (__Pyx_MergeVtables(__pyx_mstate->__pyx_ptype_4cupy_4cuda_6memory_MemoryPointer) < (0)) __PYX_ERR(0, 458, __pyx_L1_error)
  if (PyObject_SetAttr(__pyx_m, __pyx_mstate_global->__pyx_n_u_MemoryPointer, (PyObject *) __pyx_mstate->__pyx_ptype_4cupy_4cuda_6memory_MemoryPointer) < (0)) __PYX_ERR(0, 458, __pyx_L1_error)
  if (__Pyx_setup_reduce((PyObject *) __pyx_mstate->__pyx_ptype_4cupy_4cuda_6memory_MemoryPointer) < (0)) __PYX_ERR(0, 458, __pyx_L1_error)
  __pyx_vtabptr_4cupy_4cuda_6memory_MemoryPool = &__pyx_vtable_4cupy_4cuda_6memory_MemoryPool;
  __pyx_vtable_4cupy_4cuda_6memory_MemoryPool.malloc = (struct __pyx_obj_4cupy_4cuda_6memory_MemoryPointer *(*)(struct __pyx_obj_4cupy_4cuda_6memory_MemoryPool *, size_t, int __pyx_skip_dispatch))__pyx_f_4cupy_4cuda_6memory_10MemoryPool_malloc;
  __pyx_vtable_4cupy_4cuda_6memory_MemoryPool.free_all_blocks = (PyObject *(*)(struct __pyx_obj_4cupy_4cuda_6memory_MemoryPool *, int __pyx_skip_dispatch, struct __pyx_opt_args_4cupy_4cuda_6memory_10MemoryPool_free_all_blocks *__pyx_optional_args))__pyx_f_4cupy_4cuda_6memory_10MemoryPool_free_all_blocks;
  __pyx_vtable_4cupy_4cuda_6memory_MemoryPool.free_all_free = (PyObject *(*)(struct __pyx_obj_4cupy_4cuda_6memory_MemoryPool *, int __pyx_skip_dispatch))__pyx_f_4cupy_4cuda_6memory_10MemoryPool_free_all_free;
  __pyx_vtable_4cupy_4cuda_6memory_MemoryPool.n_free_blocks = (size_t (*)(struct __pyx_obj_4cupy_4cuda_6memory_MemoryPool *, int __pyx_skip_dispatch))__pyx_f_4cupy_4cuda_6memory_10MemoryPool_n_free_blocks;
  __pyx_vtable_4cupy_4cuda_6memory_MemoryPool.used_bytes = (size_t (*)(struct __pyx_obj_4cupy_4cuda_6memory_MemoryPool *, int __pyx_skip_dispatch))__pyx_f_4cupy_4cuda_6memory_10MemoryPool_used_bytes;
  __pyx_vtable_4cupy_4cuda_6memory_MemoryPool.free_bytes = (size_t (*)(struct __pyx_obj_4cupy_4cuda_6memory_MemoryPool *, int __pyx_skip_dispatch))__pyx_f_4cupy_4cuda_6memory_10MemoryPool_free_bytes;
  __pyx_vtable_4cupy_4cuda_6memory_MemoryPool.total_bytes = (size_t (*)(struct __pyx_obj_4cupy_4cuda_6memory_MemoryPool *, int __pyx_skip_dispatch))__pyx_f_4cupy_4cuda_6memory_10MemoryPool_total_bytes;
  __pyx_vtable_4cupy_4cuda_6memory_MemoryPool.set_limit = (PyObject *(*)(struct __pyx_obj_4cupy_4cuda_6memory_MemoryPool *, int __pyx_skip_dispatch, struct __pyx_opt_args_4cupy_4cuda_6memory_10MemoryPool_set_limit *__pyx_optional_args))__pyx_f_4cupy_4cuda_6memory_10MemoryPool_set_limit;
  __pyx_vtable_4cupy_4cuda_6memory_MemoryPool.get_limit = (size_t (*)(struct __pyx_obj_4cupy_4cuda_6memory_MemoryPool *, int __pyx_skip_dispatch))__pyx_f_4cupy_4cuda_6memory_10MemoryPool_get_limit;
  __pyx_vtable_4cupy_4cuda_6memory_MemoryPool._ensure_pools_and_return_device_pool = (PyObject *(*)(struct __pyx_obj_4cupy_4cuda_6memory_MemoryPool *))__pyx_f_4cupy_4cuda_6memory_10MemoryPool__ensure_pools_and_return_device_pool;
  __pyx_vtable_4cupy_4cuda_6memory_MemoryPool.device_pool = (PyObject *(*)(struct __pyx_obj_4cupy_4cuda_6memory_MemoryPool *))__pyx_f_4cupy_4cuda_6memory_10MemoryPool_device_pool;
  #if CYTHON_USE_TYPE_SPECS
  __pyx_mstate->__pyx_ptype_4cupy_4cuda_6memory_MemoryPool = (PyTypeObject *) __Pyx_PyType_FromModuleAndSpec(__pyx_m, &__pyx_type_4cupy_4cuda_6memory_MemoryPool_spec, NULL); if (unlikely(!__pyx_mstate->__pyx_ptype_4cupy_4cuda_6memory_MemoryPool)) __PYX_ERR(0, 1566, __pyx_L1_error)
  if (__Pyx_fix_up_extension_type_from_spec(&__pyx_type_4cupy_4cuda_6memory_MemoryPool_spec, __pyx_mstate->__pyx_ptype_4cupy_4cuda_6memory_MemoryPool) < (0)) __PYX_ERR(0, 1566, __pyx_L1_error)
  #else
  __pyx_mstate->__pyx_ptype_4cupy_4cuda_6memory_MemoryPool = &__pyx_type_4cupy_4cuda_6memory_MemoryPool;
  #endif
  #if !CYTHON_COMPILING_IN_LIMITED_API
  #endif
  #if !CYTHON_USE_TYPE_SPECS
  if (__Pyx_PyType_Ready(__pyx_mstate->__pyx_ptype_4cupy_4cuda_6memory_MemoryPool) < (0)) __PYX_ERR(0, 1566, __pyx_L1_error)
  #endif
  #if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x030E0000
  PyUnstable_Object_EnableDeferredRefcount((PyObject*)__pyx_mstate->__pyx_ptype_4cupy_4cuda_6memory_MemoryPool);
  #endif
  #if !CYTHON_COMPILING_IN_LIMITED_API
  if ((CYTHON_USE_TYPE_SLOTS && CYTHON_USE_PYTYPE_LOOKUP) && likely(!__pyx_mstate->__pyx_ptype_4cupy_4cuda_6memory_MemoryPool->tp_dictoffset && __pyx_mstate->__pyx_ptype_4cupy_4cuda_6memory_MemoryPool->tp_getattro == PyObject_GenericGetAttr)) {
    __pyx_mstate->__pyx_ptype_4cupy_4cuda_6memory_MemoryPool->tp_getattro = PyObject_GenericGetAttr;
  }
  #endif
  if (__Pyx_SetVtable(__pyx_mstate->__pyx_ptype_4cupy_4cuda_6memory_MemoryPool, __pyx_vtabptr_4cupy_4cuda_6memory_MemoryPool) < (0)) __PYX_ERR(0, 1566, __pyx_L1_error)
  if (__Pyx_MergeVtables(__pyx_mstate->__pyx_ptype_4cupy_4cuda_6memory_MemoryPool) < (0)) __PYX_ERR(0, 1566, __pyx_L1_error)
  if (PyObject_SetAttr(__pyx_m, __pyx_mstate_global->__pyx_n_u_MemoryPool, (PyObject *) __pyx_mstate->__pyx_ptype_4cupy_4cuda_6memory_MemoryPool) < (0)) __PYX_ERR(0, 1566, __pyx_L1_error)
  if (__Pyx_setup_reduce((PyObject *) __pyx_mstate->__pyx_ptype_4cupy_4cuda_6memory_MemoryPool) < (0)) __PYX_ERR(0, 1566, __pyx_L1_error)
  #if CYTHON_USE_TYPE_SPECS
  __pyx_t_1 = PyTuple_Pack(1, (PyObject *)__pyx_mstate_global->__pyx_ptype_4cupy_4cuda_6memory_BaseMemory); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 2060, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_mstate->__pyx_ptype_4cupy_4cuda_6memory_CFunctionAllocatorMemory = (PyTypeObject *) __Pyx_PyType_FromModuleAndSpec(__pyx_m, &__pyx_type_4cupy_4cuda_6memory_CFunctionAllocatorMemory_spec, __pyx_t_1);
  __Pyx_XDECREF(__pyx_t_1); __pyx_t_1 = 0;
  if (unlikely(!__pyx_mstate->__pyx_ptype_4cupy_4cuda_6memory_CFunctionAllocatorMemory)) __PYX_ERR(0, 2060, __pyx_L1_error)
  if (__Pyx_fix_up_extension_type_from_spec(&__pyx_type_4cupy_4cuda_6memory_CFunctionAllocatorMemory_spec, __pyx_mstate->__pyx_ptype_4cupy_4cuda_6memory_CFunctionAllocatorMemory) < (0)) __PYX_ERR(0, 2060, __pyx_L1_error)
  #else
  __pyx_mstate->__pyx_ptype_4cupy_4cuda_6memory_CFunctionAllocatorMemory = &__pyx_type_4cupy_4cuda_6memory_CFunctionAllocatorMemory;
  #endif
  #if !CYTHON_COMPILING_IN_LIMITED_API
  __pyx_mstate_global->__pyx_ptype_4cupy_4cuda_6memory_CFunctionAllocatorMemory->tp_base = __pyx_mstate_global->__pyx_ptype_4cupy_4cuda_6memory_BaseMemory;
  #endif
  #if !CYTHON_USE_TYPE_SPECS
  if (__Pyx_PyType_Ready(__pyx_mstate->__pyx_ptype_4cupy_4cuda_6memory_CFunctionAllocatorMemory) < (0)) __PYX_ERR(0, 2060, __pyx_L1_error)
  #endif
  #if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x030E0000
  PyUnstable_Object_EnableDeferredRefcount((PyObject*)__pyx_mstate->__pyx_ptype_4cupy_4cuda_6memory_CFunctionAllocatorMemory);
  #endif
  #if !CYTHON_COMPILING_IN_LIMITED_API
  if ((CYTHON_USE_TYPE_SLOTS && CYTHON_USE_PYTYPE_LOOKUP) && likely(!__pyx_mstate->__pyx_ptype_4cupy_4cuda_6memory_CFunctionAllocatorMemory->tp_dictoffset && __pyx_mstate->__pyx_ptype_4cupy_4cuda_6memory_CFunctionAllocatorMemory->tp_getattro == PyObject_GenericGetAttr)) {
    __pyx_mstate->__pyx_ptype_4cupy_4cuda_6memory_CFunctionAllocatorMemory->tp_getattro = PyObject_GenericGetAttr;
  }
  #endif
  if (PyObject_SetAttr(__pyx_m, __pyx_mstate_global->__pyx_n_u_CFunctionAllocatorMemory, (PyObject *) __pyx_mstate->__pyx_ptype_4cupy_4cuda_6memory_CFunctionAllocatorMemory) < (0)) __PYX_ERR(0, 2060, __pyx_L1_error)
  if (__Pyx_setup_reduce((PyObject *) __pyx_mstate->__pyx_ptype_4cupy_4cuda_6memory_CFunctionAllocatorMemory) < (0)) __PYX_ERR(0, 2060, __pyx_L1_error)
  __pyx_vtabptr_4cupy_4cuda_6memory_CFunctionAllocator = &__pyx_vtable_4cupy_4cuda_6memory_CFunctionAllocator;
  __pyx_vtable_4cupy_4cuda_6memory_CFunctionAllocator.malloc = (struct __pyx_obj_4cupy_4cuda_6memory_MemoryPointer *(*)(struct __pyx_obj_4cupy_4cuda_6memory_CFunctionAllocator *, size_t, int __pyx_skip_dispatch))__pyx_f_4cupy_4cuda_6memory_18CFunctionAllocator_malloc;
  #if CYTHON_USE_TYPE_SPECS
  __pyx_mstate->__pyx_ptype_4cupy_4cuda_6memory_CFunctionAllocator = (PyTypeObject *) __Pyx_PyType_FromModuleAndSpec(__pyx_m, &__pyx_type_4cupy_4cuda_6memory_CFunctionAllocator_spec, NULL); if (unlikely(!__pyx_mstate->__pyx_ptype_4cupy_4cuda_6memory_CFunctionAllocator)) __PYX_ERR(0, 2079, __pyx_L1_error)
  if (__Pyx_fix_up_extension_type_from_spec(&__pyx_type_4cupy_4cuda_6memory_CFunctionAllocator_spec, __pyx_mstate->__pyx_ptype_4cupy_4cuda_6memory_CFunctionAllocator) < (0)) __PYX_ERR(0, 2079, __pyx_L1_error)
  #else
  __pyx_mstate->__pyx_ptype_4cupy_4cuda_6memory_CFunctionAllocator = &__pyx_type_4cupy_4cuda_6memory_CFunctionAllocator;
  #endif
  #if !CYTHON_COMPILING_IN_LIMITED_API
  #endif
  #if !CYTHON_USE_TYPE_SPECS
  if (__Pyx_PyType_Ready(__pyx_mstate->__pyx_ptype_4cupy_4cuda_6memory_CFunctionAllocator) < (0)) __PYX_ERR(0, 2079, __pyx_L1_error)
  #endif
  #if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x030E0000
  PyUnstable_Object_EnableDeferredRefcount((PyObject*)__pyx_mstate->__pyx_ptype_4cupy_4cuda_6memory_CFunctionAllocator);
  #endif
  #if !CYTHON_COMPILING_IN_LIMITED_API
  if ((CYTHON_USE_TYPE_SLOTS && CYTHON_USE_PYTYPE_LOOKUP) && likely(!__pyx_mstate->__pyx_ptype_4cupy_4cuda_6memory_CFunctionAllocator->tp_dictoffset && __pyx_mstate->__pyx_ptype_4cupy_4cuda_6memory_CFunctionAllocator->tp_getattro == PyObject_GenericGetAttr)) {
    __pyx_mstate->__pyx_ptype_4cupy_4cuda_6memory_CFunctionAllocator->tp_getattro = PyObject_GenericGetAttr;
  }
  #endif
  if (__Pyx_SetVtable(__pyx_mstate->__pyx_ptype_4cupy_4cuda_6memory_CFunctionAllocator, __pyx_vtabptr_4cupy_4cuda_6memory_CFunctionAllocator) < (0)) __PYX_ERR(0, 2079, __pyx_L1_error)
  if (__Pyx_MergeVtables(__pyx_mstate->__pyx_ptype_4cupy_4cuda_6memory_CFunctionAllocator) < (0)) __PYX_ERR(0, 2079, __pyx_L1_error)
  if (PyObject_SetAttr(__pyx_m, __pyx_mstate_global->__pyx_n_u_CFunctionAllocator, (PyObject *) __pyx_mstate->__pyx_ptype_4cupy_4cuda_6memory_CFunctionAllocator) < (0)) __PYX_ERR(0, 2079, __pyx_L1_error)
  if (__Pyx_setup_reduce((PyObject *) __pyx_mstate->__pyx_ptype_4cupy_4cuda_6memory_CFunctionAllocator) < (0)) __PYX_ERR(0, 2079, __pyx_L1_error)
  #if CYTHON_USE_TYPE_SPECS
  __pyx_t_1 = PyTuple_Pack(1, (PyObject *)__pyx_mstate_global->__pyx_ptype_4cupy_4cuda_6memory_BaseMemory); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 2120, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_mstate->__pyx_ptype_4cupy_4cuda_6memory_PythonFunctionAllocatorMemory = (PyTypeObject *) __Pyx_PyType_FromModuleAndSpec(__pyx_m, &__pyx_type_4cupy_4cuda_6memory_PythonFunctionAllocatorMemory_spec, __pyx_t_1);
  __Pyx_XDECREF(__pyx_t_1); __pyx_t_1 = 0;
  if (unlikely(!__pyx_mstate->__pyx_ptype_4cupy_4cuda_6memory_PythonFunctionAllocatorMemory)) __PYX_ERR(0, 2120, __pyx_L1_error)
  if (__Pyx_fix_up_extension_type_from_spec(&__pyx_type_4cupy_4cuda_6memory_PythonFunctionAllocatorMemory_spec, __pyx_mstate->__pyx_ptype_4cupy_4cuda_6memory_PythonFunctionAllocatorMemory) < (0)) __PYX_ERR(0, 2120, __pyx_L1_error)
  #else
  __pyx_mstate->__pyx_ptype_4cupy_4cuda_6memory_PythonFunctionAllocatorMemory = &__pyx_type_4cupy_4cuda_6memory_PythonFunctionAllocatorMemory;
  #endif
  #if !CYTHON_COMPILING_IN_LIMITED_API
  __pyx_mstate_global->__pyx_ptype_4cupy_4cuda_6memory_PythonFunctionAllocatorMemory->tp_base = __pyx_mstate_global->__pyx_ptype_4cupy_4cuda_6memory_BaseMemory;
  #endif
  #if !CYTHON_USE_TYPE_SPECS
  if (__Pyx_PyType_Ready(__pyx_mstate->__pyx_ptype_4cupy_4cuda_6memory_PythonFunctionAllocatorMemory) < (0)) __PYX_ERR(0, 2120, __pyx_L1_error)
  #endif
  #if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x030E0000
  PyUnstable_Object_EnableDeferredRefcount((PyObject*)__pyx_mstate->__pyx_ptype_4cupy_4cuda_6memory_PythonFunctionAllocatorMemory);
  #endif
  #if !CYTHON_COMPILING_IN_LIMITED_API
  if ((CYTHON_USE_TYPE_SLOTS && CYTHON_USE_PYTYPE_LOOKUP) && likely(!__pyx_mstate->__pyx_ptype_4cupy_4cuda_6memory_PythonFunctionAllocatorMemory->tp_dictoffset && __pyx_mstate->__pyx_ptype_4cupy_4cuda_6memory_PythonFunctionAllocatorMemory->tp_getattro == PyObject_GenericGetAttr)) {
    __pyx_mstate->__pyx_ptype_4cupy_4cuda_6memory_PythonFunctionAllocatorMemory->tp_getattro = PyObject_GenericGetAttr;
  }
  #endif
  if (PyObject_SetAttr(__pyx_m, __pyx_mstate_global->__pyx_n_u_PythonFunctionAllocatorMemory, (PyObject *) __pyx_mstate->__pyx_ptype_4cupy_4cuda_6memory_PythonFunctionAllocatorMemory) < (0)) __PYX_ERR(0, 2120, __pyx_L1_error)
  if (__Pyx_setup_reduce((PyObject *) __pyx_mstate->__pyx_ptype_4cupy_4cuda_6memory_PythonFunctionAllocatorMemory) < (0)) __PYX_ERR(0, 2120, __pyx_L1_error)
  __pyx_vtabptr_4cupy_4cuda_6memory_PythonFunctionAllocator = &__pyx_vtable_4cupy_4cuda_6memory_PythonFunctionAllocator;
  __pyx_vtable_4cupy_4cuda_6memory_PythonFunctionAllocator.malloc = (struct __pyx_obj_4cupy_4cuda_6memory_MemoryPointer *(*)(struct __pyx_obj_4cupy_4cuda_6memory_PythonFunctionAllocator *, size_t, int __pyx_skip_dispatch))__pyx_f_4cupy_4cuda_6memory_23PythonFunctionAllocator_malloc;
  #if CYTHON_USE_TYPE_SPECS
  __pyx_mstate->__pyx_ptype_4cupy_4cuda_6memory_PythonFunctionAllocator = (PyTypeObject *) __Pyx_PyType_FromModuleAndSpec(__pyx_m, &__pyx_type_4cupy_4cuda_6memory_PythonFunctionAllocator_spec, NULL); if (unlikely(!__pyx_mstate->__pyx_ptype_4cupy_4cuda_6memory_PythonFunctionAllocator)) __PYX_ERR(0, 2136, __pyx_L1_error)
  if (__Pyx_fix_up_extension_type_from_spec(&__pyx_type_4cupy_4cuda_6memory_PythonFunctionAllocator_spec, __pyx_mstate->__pyx_ptype_4cupy_4cuda_6memory_PythonFunctionAllocator) < (0)) __PYX_ERR(0, 2136, __pyx_L1_error)
  #else
  __pyx_mstate->__pyx_ptype_4cupy_4cuda_6memory_PythonFunctionAllocator = &__pyx_type_4cupy_4cuda_6memory_PythonFunctionAllocator;
  #endif
  #if !CYTHON_COMPILING_IN_LIMITED_API
  #endif
  #if !CYTHON_USE_TYPE_SPECS
  if (__Pyx_PyType_Ready(__pyx_mstate->__pyx_ptype_4cupy_4cuda_6memory_PythonFunctionAllocator) < (0)) __PYX_ERR(0, 2136, __pyx_L1_error)
  #endif
  #if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x030E0000
  PyUnstable_Object_EnableDeferredRefcount((PyObject*)__pyx_mstate->__pyx_ptype_4cupy_4cuda_6memory_PythonFunctionAllocator);
  #endif
  #if !CYTHON_COMPILING_IN_LIMITED_API
  if ((CYTHON_USE_TYPE_SLOTS && CYTHON_USE_PYTYPE_LOOKUP) && likely(!__pyx_mstate->__pyx_ptype_4cupy_4cuda_6memory_PythonFunctionAllocator->tp_dictoffset && __pyx_mstate->__pyx_ptype_4cupy_4cuda_6memory_PythonFunctionAllocator->tp_getattro == PyObject_GenericGetAttr)) {
    __pyx_mstate->__pyx_ptype_4cupy_4cuda_6memory_PythonFunctionAllocator->tp_getattro = PyObject_GenericGetAttr;
  }
  #endif
  if (__Pyx_SetVtable(__pyx_mstate->__pyx_ptype_4cupy_4cuda_6memory_PythonFunctionAllocator, __pyx_vtabptr_4cupy_4cuda_6memory_PythonFunctionAllocator) < (0)) __PYX_ERR(0, 2136, __pyx_L1_error)
  if (__Pyx_MergeVtables(__pyx_mstate->__pyx_ptype_4cupy_4cuda_6memory_PythonFunctionAllocator) < (0)) __PYX_ERR(0, 2136, __pyx_L1_error)
  if (PyObject_SetAttr(__pyx_m, __pyx_mstate_global->__pyx_n_u_PythonFunctionAllocator, (PyObject *) __pyx_mstate->__pyx_ptype_4cupy_4cuda_6memory_PythonFunctionAllocator) < (0)) __PYX_ERR(0, 2136, __pyx_L1_error)
  if (__Pyx_setup_reduce((PyObject *) __pyx_mstate->__pyx_ptype_4cupy_4cuda_6memory_PythonFunctionAllocator) < (0)) __PYX_ERR(0, 2136, __pyx_L1_error)
  #if CYTHON_USE_TYPE_SPECS
  __pyx_t_1 = PyTuple_Pack(1, (PyObject *)__pyx_mstate_global->__pyx_ptype_4cupy_4cuda_6memory_BaseMemory); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 104, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_mstate->__pyx_ptype_4cupy_4cuda_6memory_Memory = (PyTypeObject *) __Pyx_PyType_FromModuleAndSpec(__pyx_m, &__pyx_type_4cupy_4cuda_6memory_Memory_spec, __pyx_t_1);
  __Pyx_XDECREF(__pyx_t_1); __pyx_t_1 = 0;
  if (unlikely(!__pyx_mstate->__pyx_ptype_4cupy_4cuda_6memory_Memory)) __PYX_ERR(0, 104, __pyx_L1_error)
  if (__Pyx_fix_up_extension_type_from_spec(&__pyx_type_4cupy_4cuda_6memory_Memory_spec, __pyx_mstate->__pyx_ptype_4cupy_4cuda_6memory_Memory) < (0)) __PYX_ERR(0, 104, __pyx_L1_error)
  #else
  __pyx_mstate->__pyx_ptype_4cupy_4cuda_6memory_Memory = &__pyx_type_4cupy_4cuda_6memory_Memory;
  #endif
  #if !CYTHON_COMPILING_IN_LIMITED_API
  __pyx_mstate_global->__pyx_ptype_4cupy_4cuda_6memory_Memory->tp_base = __pyx_mstate_global->__pyx_ptype_4cupy_4cuda_6memory_BaseMemory;
  #endif
  #if !CYTHON_USE_TYPE_SPECS
  if (__Pyx_PyType_Ready(__pyx_mstate->__pyx_ptype_4cupy_4cuda_6memory_Memory) < (0)) __PYX_ERR(0, 104, __pyx_L1_error)
  #endif
  #if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x030E0000
  PyUnstable_Object_EnableDeferredRefcount((PyObject*)__pyx_mstate->__pyx_ptype_4cupy_4cuda_6memory_Memory);
  #endif
  #if !CYTHON_COMPILING_IN_LIMITED_API
  if ((CYTHON_USE_TYPE_SLOTS && CYTHON_USE_PYTYPE_LOOKUP) && likely(!__pyx_mstate->__pyx_ptype_4cupy_4cuda_6memory_Memory->tp_dictoffset && __pyx_mstate->__pyx_ptype_4cupy_4cuda_6memory_Memory->tp_getattro == PyObject_GenericGetAttr)) {
    __pyx_mstate->__pyx_ptype_4cupy_4cuda_6memory_Memory->tp_getattro = PyObject_GenericGetAttr;
  }
  #endif
  if (PyObject_SetAttr(__pyx_m, __pyx_mstate_global->__pyx_n_u_Memory, (PyObject *) __pyx_mstate->__pyx_ptype_4cupy_4cuda_6memory_Memory) < (0)) __PYX_ERR(0, 104, __pyx_L1_error)
  if (__Pyx_setup_reduce((PyObject *) __pyx_mstate->__pyx_ptype_4cupy_4cuda_6memory_Memory) < (0)) __PYX_ERR(0, 104, __pyx_L1_error)
  #if CYTHON_USE_TYPE_SPECS
  __pyx_t_1 = PyTuple_Pack(1, (PyObject *)__pyx_mstate_global->__pyx_ptype_4cupy_4cuda_6memory_BaseMemory); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 145, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_mstate->__pyx_ptype_4cupy_4cuda_6memory_MemoryAsync = (PyTypeObject *) __Pyx_PyType_FromModuleAndSpec(__pyx_m, &__pyx_type_4cupy_4cuda_6memory_MemoryAsync_spec, __pyx_t_1);
  __Pyx_XDECREF(__pyx_t_1); __pyx_t_1 = 0;
  if (unlikely(!__pyx_mstate->__pyx_ptype_4cupy_4cuda_6memory_MemoryAsync)) __PYX_ERR(0, 145, __pyx_L1_error)
  if (__Pyx_fix_up_extension_type_from_spec(&__pyx_type_4cupy_4cuda_6memory_MemoryAsync_spec, __pyx_mstate->__pyx_ptype_4cupy_4cuda_6memory_MemoryAsync) < (0)) __PYX_ERR(0, 145, __pyx_L1_error)
  #else
  __pyx_mstate->__pyx_ptype_4cupy_4cuda_6memory_MemoryAsync = &__pyx_type_4cupy_4cuda_6memory_MemoryAsync;
  #endif
  #if !CYTHON_COMPILING_IN_LIMITED_API
  __pyx_mstate_global->__pyx_ptype_4cupy_4cuda_6memory_MemoryAsync->tp_base = __pyx_mstate_global->__pyx_ptype_4cupy_4cuda_6memory_BaseMemory;
  #endif
  #if !CYTHON_USE_TYPE_SPECS
  if (__Pyx_PyType_Ready(__pyx_mstate->__pyx_ptype_4cupy_4cuda_6memory_MemoryAsync) < (0)) __PYX_ERR(0, 145, __pyx_L1_error)
  #endif
  #if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x030E0000
  PyUnstable_Object_EnableDeferredRefcount((PyObject*)__pyx_mstate->__pyx_ptype_4cupy_4cuda_6memory_MemoryAsync);
  #endif
  #if !CYTHON_COMPILING_IN_LIMITED_API
  if ((CYTHON_USE_TYPE_SLOTS && CYTHON_USE_PYTYPE_LOOKUP) && likely(!__pyx_mstate->__pyx_ptype_4cupy_4cuda_6memory_MemoryAsync->tp_dictoffset && __pyx_mstate->__pyx_ptype_4cupy_4cuda_6memory_MemoryAsync->tp_getattro == PyObject_GenericGetAttr)) {
    __pyx_mstate->__pyx_ptype_4cupy_4cuda_6memory_MemoryAsync->tp_getattro = PyObject_GenericGetAttr;
  }
  #endif
  if (PyObject_SetAttr(__pyx_m, __pyx_mstate_global->__pyx_n_u_MemoryAsync, (PyObject *) __pyx_mstate->__pyx_ptype_4cupy_4cuda_6memory_MemoryAsync) < (0)) __PYX_ERR(0, 145, __pyx_L1_error)
  if (__Pyx_setup_reduce((PyObject *) __pyx_mstate->__pyx_ptype_4cupy_4cuda_6memory_MemoryAsync) < (0)) __PYX_ERR(0, 145, __pyx_L1_error)
  #if CYTHON_USE_TYPE_SPECS
  __pyx_t_1 = PyTuple_Pack(1, (PyObject *)__pyx_mstate_global->__pyx_ptype_4cupy_4cuda_6memory_BaseMemory); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 181, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_mstate->__pyx_ptype_4cupy_4cuda_6memory_UnownedMemory = (PyTypeObject *) __Pyx_PyType_FromModuleAndSpec(__pyx_m, &__pyx_type_4cupy_4cuda_6memory_UnownedMemory_spec, __pyx_t_1);
  __Pyx_XDECREF(__pyx_t_1); __pyx_t_1 = 0;
  if (unlikely(!__pyx_mstate->__pyx_ptype_4cupy_4cuda_6memory_UnownedMemory)) __PYX_ERR(0, 181, __pyx_L1_error)
  if (__Pyx_fix_up_extension_type_from_spec(&__pyx_type_4cupy_4cuda_6memory_UnownedMemory_spec, __pyx_mstate->__pyx_ptype_4cupy_4cuda_6memory_UnownedMemory) < (0)) __PYX_ERR(0, 181, __pyx_L1_error)
  #else
  __pyx_mstate->__pyx_ptype_4cupy_4cuda_6memory_UnownedMemory = &__pyx_type_4cupy_4cuda_6memory_UnownedMemory;
  #endif
  #if !CYTHON_COMPILING_IN_LIMITED_API
  __pyx_mstate_global->__pyx_ptype_4cupy_4cuda_6memory_UnownedMemory->tp_base = __pyx_mstate_global->__pyx_ptype_4cupy_4cuda_6memory_BaseMemory;
  #endif
  #if !CYTHON_USE_TYPE_SPECS
  if (__Pyx_PyType_Ready(__pyx_mstate->__pyx_ptype_4cupy_4cuda_6memory_UnownedMemory) < (0)) __PYX_ERR(0, 181, __pyx_L1_error)
  #endif
  #if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x030E0000
  PyUnstable_Object_EnableDeferredRefcount((PyObject*)__pyx_mstate->__pyx_ptype_4cupy_4cuda_6memory_UnownedMemory);
  #endif
  #if !CYTHON_COMPILING_IN_LIMITED_API
  if ((CYTHON_USE_TYPE_SLOTS && CYTHON_USE_PYTYPE_LOOKUP) && likely(!__pyx_mstate->__pyx_ptype_4cupy_4cuda_6memory_UnownedMemory->tp_dictoffset && __pyx_mstate->__pyx_ptype_4cupy_4cuda_6memory_UnownedMemory->tp_getattro == PyObject_GenericGetAttr)) {
    __pyx_mstate->__pyx_ptype_4cupy_4cuda_6memory_UnownedMemory->tp_getattro = PyObject_GenericGetAttr;
  }
  #endif
  if (PyObject_SetAttr(__pyx_m, __pyx_mstate_global->__pyx_n_u_UnownedMemory, (PyObject *) __pyx_mstate->__pyx_ptype_4cupy_4cuda_6memory_UnownedMemory) < (0)) __PYX_ERR(0, 181, __pyx_L1_error)
  if (__Pyx_setup_reduce((PyObject *) __pyx_mstate->__pyx_ptype_4cupy_4cuda_6memory_UnownedMemory) < (0)) __PYX_ERR(0, 181, __pyx_L1_error)
  #if CYTHON_USE_TYPE_SPECS
  __pyx_t_1 = PyTuple_Pack(1, (PyObject *)__pyx_mstate_global->__pyx_ptype_4cupy_4cuda_6memory_BaseMemory); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 222, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_mstate->__pyx_ptype_4cupy_4cuda_6memory_ManagedMemory = (PyTypeObject *) __Pyx_PyType_FromModuleAndSpec(__pyx_m, &__pyx_type_4cupy_4cuda_6memory_ManagedMemory_spec, __pyx_t_1);
  __Pyx_XDECREF(__pyx_t_1); __pyx_t_1 = 0;
  if (unlikely(!__pyx_mstate->__pyx_ptype_4cupy_4cuda_6memory_ManagedMemory)) __PYX_ERR(0, 222, __pyx_L1_error)
  if (__Pyx_fix_up_extension_type_from_spec(&__pyx_type_4cupy_4cuda_6memory_ManagedMemory_spec, __pyx_mstate->__pyx_ptype_4cupy_4cuda_6memory_ManagedMemory) < (0)) __PYX_ERR(0, 222, __pyx_L1_error)
  #else
  __pyx_mstate->__pyx_ptype_4cupy_4cuda_6memory_ManagedMemory = &__pyx_type_4cupy_4cuda_6memory_ManagedMemory;
  #endif
  #if !CYTHON_COMPILING_IN_LIMITED_API
  __pyx_mstate_global->__pyx_ptype_4cupy_4cuda_6memory_ManagedMemory->tp_base = __pyx_mstate_global->__pyx_ptype_4cupy_4cuda_6memory_BaseMemory;
  #endif
  #if !CYTHON_USE_TYPE_SPECS
  if (__Pyx_PyType_Ready(__pyx_mstate->__pyx_ptype_4cupy_4cuda_6memory_ManagedMemory) < (0)) __PYX_ERR(0, 222, __pyx_L1_error)
  #endif
  #if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x030E0000
  PyUnstable_Object_EnableDeferredRefcount((PyObject*)__pyx_mstate->__pyx_ptype_4cupy_4cuda_6memory_ManagedMemory);
  #endif
  #if !CYTHON_COMPILING_IN_LIMITED_API
  if ((CYTHON_USE_TYPE_SLOTS && CYTHON_USE_PYTYPE_LOOKUP) && likely(!__pyx_mstate->__pyx_ptype_4cupy_4cuda_6memory_ManagedMemory->tp_dictoffset && __pyx_mstate->__pyx_ptype_4cupy_4cuda_6memory_ManagedMemory->tp_getattro == PyObject_GenericGetAttr)) {
    __pyx_mstate->__pyx_ptype_4cupy_4cuda_6memory_ManagedMemory->tp_getattro = PyObject_GenericGetAttr;
  }
  #endif
  if (PyObject_SetAttr(__pyx_m, __pyx_mstate_global->__pyx_n_u_ManagedMemory, (PyObject *) __pyx_mstate->__pyx_ptype_4cupy_4cuda_6memory_ManagedMemory) < (0)) __PYX_ERR(0, 222, __pyx_L1_error)
  if (__Pyx_setup_reduce((PyObject *) __pyx_mstate->__pyx_ptype_4cupy_4cuda_6memory_ManagedMemory) < (0)) __PYX_ERR(0, 222, __pyx_L1_error)
  __pyx_vtabptr_4cupy_4cuda_6memory__Chunk = &__pyx_vtable_4cupy_4cuda_6memory__Chunk;
  __pyx_vtable_4cupy_4cuda_6memory__Chunk._init = (PyObject *(*)(struct __pyx_obj_4cupy_4cuda_6memory__Chunk *, struct __pyx_obj_4cupy_4cuda_6memory_BaseMemory *, ptrdiff_t, size_t, struct __pyx_obj_4cupy_4cuda_6memory__Arena *))__pyx_f_4cupy_4cuda_6memory_6_Chunk__init;
  __pyx_vtable_4cupy_4cuda_6memory__Chunk.ptr = (intptr_t (*)(struct __pyx_obj_4cupy_4cuda_6memory__Chunk *, int __pyx_skip_dispatch))__pyx_f_4cupy_4cuda_6memory_6_Chunk_ptr;
  __pyx_vtable_4cupy_4cuda_6memory__Chunk.split = (struct __pyx_obj_4cupy_4cuda_6memory__Chunk *(*)(struct __pyx_obj_4cupy_4cuda_6memory__Chunk *, size_t, int __pyx_skip_dispatch))__pyx_f_4cupy_4cuda_6memory_6_Chunk_split;
  __pyx_vtable_4cupy_4cuda_6memory__Chunk.merge_next = (PyObject *(*)(struct __pyx_obj_4cupy_4cuda_6memory__Chunk *, int __pyx_skip_dispatch))__pyx_f_4cupy_4cuda_6memory_6_Chunk_merge_next;
  #if CYTHON_USE_TYPE_SPECS
  __pyx_mstate->__pyx_ptype_4cupy_4cuda_6memory__Chunk = (PyTypeObject *) __Pyx_PyType_FromModuleAndSpec(__pyx_m, &__pyx_type_4cupy_4cuda_6memory__Chunk_spec, NULL); if (unlikely(!__pyx_mstate->__pyx_ptype_4cupy_4cuda_6memory__Chunk)) __PYX_ERR(0, 357, __pyx_L1_error)
  if (__Pyx_fix_up_extension_type_from_spec(&__pyx_type_4cupy_4cuda_6memory__Chunk_spec, __pyx_mstate->__pyx_ptype_4cupy_4cuda_6memory__Chunk) < (0)) __PYX_ERR(0, 357, __pyx_L1_error)
  #else
  __pyx_mstate->__pyx_ptype_4cupy_4cuda_6memory__Chunk = &__pyx_type_4cupy_4cuda_6memory__Chunk;
  #endif
  #if !CYTHON_COMPILING_IN_LIMITED_API
  #endif
  #if !CYTHON_USE_TYPE_SPECS
  if (__Pyx_PyType_Ready(__pyx_mstate->__pyx_ptype_4cupy_4cuda_6memory__Chunk) < (0)) __PYX_ERR(0, 357, __pyx_L1_error)
  #endif
  #if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x030E0000
  PyUnstable_Object_EnableDeferredRefcount((PyObject*)__pyx_mstate->__pyx_ptype_4cupy_4cuda_6memory__Chunk);
  #endif
  #if !CYTHON_COMPILING_IN_LIMITED_API
  if ((CYTHON_USE_TYPE_SLOTS && CYTHON_USE_PYTYPE_LOOKUP) && likely(!__pyx_mstate->__pyx_ptype_4cupy_4cuda_6memory__Chunk->tp_dictoffset && __pyx_mstate->__pyx_ptype_4cupy_4cuda_6memory__Chunk->tp_getattro == PyObject_GenericGetAttr)) {
    __pyx_mstate->__pyx_ptype_4cupy_4cuda_6memory__Chunk->tp_getattro = PyObject_GenericGetAttr;
  }
  #endif
  if (__Pyx_SetVtable(__pyx_mstate->__pyx_ptype_4cupy_4cuda_6memory__Chunk, __pyx_vtabptr_4cupy_4cuda_6memory__Chunk) < (0)) __PYX_ERR(0, 357, __pyx_L1_error)
  if (__Pyx_MergeVtables(__pyx_mstate->__pyx_ptype_4cupy_4cuda_6memory__Chunk) < (0)) __PYX_ERR(0, 357, __pyx_L1_error)
  if (PyObject_SetAttr(__pyx_m, __pyx_mstate_global->__pyx_n_u_Chunk, (PyObject *) __pyx_mstate->__pyx_ptype_4cupy_4cuda_6memory__Chunk) < (0)) __PYX_ERR(0, 357, __pyx_L1_error)
  if (__Pyx_setup_reduce((PyObject *) __pyx_mstate->__pyx_ptype_4cupy_4cuda_6memory__Chunk) < (0)) __PYX_ERR(0, 357, __pyx_L1_error)
  __pyx_vtabptr_4cupy_4cuda_6memory_PooledMemory = &__pyx_vtable_4cupy_4cuda_6memory_PooledMemory;
  __pyx_vtable_4cupy_4cuda_6memory_PooledMemory._init = (PyObject *(*)(struct __pyx_obj_4cupy_4cuda_6memory_PooledMemory *, struct __pyx_obj_4cupy_4cuda_6memory__Chunk *, PyObject *))__pyx_f_4cupy_4cuda_6memory_12PooledMemory__init;
  __pyx_vtable_4cupy_4cuda_6memory_PooledMemory.free = (PyObject *(*)(struct __pyx_obj_4cupy_4cuda_6memory_PooledMemory *, int __pyx_skip_dispatch))__pyx_f_4cupy_4cuda_6memory_12PooledMemory_free;
  #if CYTHON_USE_TYPE_SPECS
  __pyx_t_1 = PyTuple_Pack(1, (PyObject *)__pyx_mstate_global->__pyx_ptype_4cupy_4cuda_6memory_BaseMemory); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 943, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_mstate->__pyx_ptype_4cupy_4cuda_6memory_PooledMemory = (PyTypeObject *) __Pyx_PyType_FromModuleAndSpec(__pyx_m, &__pyx_type_4cupy_4cuda_6memory_PooledMemory_spec, __pyx_t_1);
  __Pyx_XDECREF(__pyx_t_1); __pyx_t_1 = 0;
  if (unlikely(!__pyx_mstate->__pyx_ptype_4cupy_4cuda_6memory_PooledMemory)) __PYX_ERR(0, 943, __pyx_L1_error)
  if (__Pyx_fix_up_extension_type_from_spec(&__pyx_type_4cupy_4cuda_6memory_PooledMemory_spec, __pyx_mstate->__pyx_ptype_4cupy_4cuda_6memory_PooledMemory) < (0)) __PYX_ERR(0, 943, __pyx_L1_error)
  #else
  __pyx_mstate->__pyx_ptype_4cupy_4cuda_6memory_PooledMemory = &__pyx_type_4cupy_4cuda_6memory_PooledMemory;
  #endif
  #if !CYTHON_COMPILING_IN_LIMITED_API
  __pyx_mstate_global->__pyx_ptype_4cupy_4cuda_6memory_PooledMemory->tp_base = __pyx_mstate_global->__pyx_ptype_4cupy_4cuda_6memory_BaseMemory;
  #endif
  #if !CYTHON_USE_TYPE_SPECS
  if (__Pyx_PyType_Ready(__pyx_mstate->__pyx_ptype_4cupy_4cuda_6memory_PooledMemory) < (0)) __PYX_ERR(0, 943, __pyx_L1_error)
  #endif
  #if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x030E0000
  PyUnstable_Object_EnableDeferredRefcount((PyObject*)__pyx_mstate->__pyx_ptype_4cupy_4cuda_6memory_PooledMemory);
  #endif
  if (__Pyx_SetVtable(__pyx_mstate->__pyx_ptype_4cupy_4cuda_6memory_PooledMemory, __pyx_vtabptr_4cupy_4cuda_6memory_PooledMemory) < (0)) __PYX_ERR(0, 943, __pyx_L1_error)
  if (__Pyx_MergeVtables(__pyx_mstate->__pyx_ptype_4cupy_4cuda_6memory_PooledMemory) < (0)) __PYX_ERR(0, 943, __pyx_L1_error)
  if (PyObject_SetAttr(__pyx_m, __pyx_mstate_global->__pyx_n_u_PooledMemory, (PyObject *) __pyx_mstate->__pyx_ptype_4cupy_4cuda_6memory_PooledMemory) < (0)) __PYX_ERR(0, 943, __pyx_L1_error)
  if (__Pyx_setup_reduce((PyObject *) __pyx_mstate->__pyx_ptype_4cupy_4cuda_6memory_PooledMemory) < (0)) __PYX_ERR(0, 943, __pyx_L1_error)
  __pyx_vtabptr_4cupy_4cuda_6memory__Arena = &__pyx_vtable_4cupy_4cuda_6memory__Arena;
  __pyx_vtable_4cupy_4cuda_6memory__Arena.add_pending_free_atomic = (PyObject *(*)(struct __pyx_obj_4cupy_4cuda_6memory__Arena *, struct __pyx_obj_4cupy_4cuda_6memory__Chunk *))__pyx_f_4cupy_4cuda_6memory_6_Arena_add_pending_free_atomic;
  __pyx_vtable_4cupy_4cuda_6memory__Arena._commit_pending_free = (PyObject *(*)(struct __pyx_obj_4cupy_4cuda_6memory__Arena *))__pyx_f_4cupy_4cuda_6memory_6_Arena__commit_pending_free;
  __pyx_vtable_4cupy_4cuda_6memory__Arena.try_merge_chunk = (struct __pyx_obj_4cupy_4cuda_6memory__Chunk *(*)(struct __pyx_obj_4cupy_4cuda_6memory__Arena *, struct __pyx_obj_4cupy_4cuda_6memory__Chunk *))__pyx_f_4cupy_4cuda_6memory_6_Arena_try_merge_chunk;
  __pyx_vtable_4cupy_4cuda_6memory__Arena.insert_chunk = (PyObject *(*)(struct __pyx_obj_4cupy_4cuda_6memory__Arena *, struct __pyx_obj_4cupy_4cuda_6memory__Chunk *, struct __pyx_opt_args_4cupy_4cuda_6memory_6_Arena_insert_chunk *__pyx_optional_args))__pyx_f_4cupy_4cuda_6memory_6_Arena_insert_chunk;
  __pyx_vtable_4cupy_4cuda_6memory__Arena.free_all = (size_t (*)(struct __pyx_obj_4cupy_4cuda_6memory__Arena *))__pyx_f_4cupy_4cuda_6memory_6_Arena_free_all;
  __pyx_vtable_4cupy_4cuda_6memory__Arena.try_remove_chunk = (int (*)(struct __pyx_obj_4cupy_4cuda_6memory__Arena *, struct __pyx_obj_4cupy_4cuda_6memory__Chunk *))__pyx_f_4cupy_4cuda_6memory_6_Arena_try_remove_chunk;
  __pyx_vtable_4cupy_4cuda_6memory__Arena.get_chunk = (struct __pyx_obj_4cupy_4cuda_6memory__Chunk *(*)(struct __pyx_obj_4cupy_4cuda_6memory__Arena *, size_t))__pyx_f_4cupy_4cuda_6memory_6_Arena_get_chunk;
  __pyx_vtable_4cupy_4cuda_6memory__Arena._get_size = (PyObject *(*)(struct __pyx_obj_4cupy_4cuda_6memory__Arena *))__pyx_f_4cupy_4cuda_6memory_6_Arena__get_size;
  __pyx_vtable_4cupy_4cuda_6memory__Arena._index_to_python = (PyObject *(*)(struct __pyx_obj_4cupy_4cuda_6memory__Arena *))__pyx_f_4cupy_4cuda_6memory_6_Arena__index_to_python;
  #if CYTHON_USE_TYPE_SPECS
  __pyx_mstate->__pyx_ptype_4cupy_4cuda_6memory__Arena = (PyTypeObject *) __Pyx_PyType_FromModuleAndSpec(__pyx_m, &__pyx_type_4cupy_4cuda_6memory__Arena_spec, NULL); if (unlikely(!__pyx_mstate->__pyx_ptype_4cupy_4cuda_6memory__Arena)) __PYX_ERR(0, 1054, __pyx_L1_error)
  if (__Pyx_fix_up_extension_type_from_spec(&__pyx_type_4cupy_4cuda_6memory__Arena_spec, __pyx_mstate->__pyx_ptype_4cupy_4cuda_6memory__Arena) < (0)) __PYX_ERR(0, 1054, __pyx_L1_error)
  #else
  __pyx_mstate->__pyx_ptype_4cupy_4cuda_6memory__Arena = &__pyx_type_4cupy_4cuda_6memory__Arena;
  #endif
  #if !CYTHON_COMPILING_IN_LIMITED_API
  #endif
  #if !CYTHON_USE_TYPE_SPECS
  if (__Pyx_PyType_Ready(__pyx_mstate->__pyx_ptype_4cupy_4cuda_6memory__Arena) < (0)) __PYX_ERR(0, 1054, __pyx_L1_error)
  #endif
  #if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x030E0000
  PyUnstable_Object_EnableDeferredRefcount((PyObject*)__pyx_mstate->__pyx_ptype_4cupy_4cuda_6memory__Arena);
  #endif
  #if !CYTHON_COMPILING_IN_LIMITED_API
  if ((CYTHON_USE_TYPE_SLOTS && CYTHON_USE_PYTYPE_LOOKUP) && likely(!__pyx_mstate->__pyx_ptype_4cupy_4cuda_6memory__Arena->tp_dictoffset && __pyx_mstate->__pyx_ptype_4cupy_4cuda_6memory__Arena->tp_getattro == PyObject_GenericGetAttr)) {
    __pyx_mstate->__pyx_ptype_4cupy_4cuda_6memory__Arena->tp_getattro = PyObject_GenericGetAttr;
  }
  #endif
  if (__Pyx_SetVtable(__pyx_mstate->__pyx_ptype_4cupy_4cuda_6memory__Arena, __pyx_vtabptr_4cupy_4cuda_6memory__Arena) < (0)) __PYX_ERR(0, 1054, __pyx_L1_error)
  if (__Pyx_MergeVtables(__pyx_mstate->__pyx_ptype_4cupy_4cuda_6memory__Arena) < (0)) __PYX_ERR(0, 1054, __pyx_L1_error)
  if (PyObject_SetAttr(__pyx_m, __pyx_mstate_global->__pyx_n_u_Arena, (PyObject *) __pyx_mstate->__pyx_ptype_4cupy_4cuda_6memory__Arena) < (0)) __PYX_ERR(0, 1054, __pyx_L1_error)
  #if CYTHON_USE_TYPE_SLOTS
  if (__pyx_mstate->__pyx_ptype_4cupy_4cuda_6memory__Arena->tp_weaklistoffset == 0) __pyx_mstate->__pyx_ptype_4cupy_4cuda_6memory__Arena->tp_weaklistoffset = offsetof(struct __pyx_obj_4cupy_4cuda_6memory__Arena, __weakref__);
  #endif
  if (__Pyx_setup_reduce((PyObject *) __pyx_mstate->__pyx_ptype_4cupy_4cuda_6memory__Arena) < (0)) __PYX_ERR(0, 1054, __pyx_L1_error)
  __pyx_vtabptr_4cupy_4cuda_6memory_SingleDeviceMemoryPool = &__pyx_vtable_4cupy_4cuda_6memory_SingleDeviceMemoryPool;
  __pyx_vtable_4cupy_4cuda_6memory_SingleDeviceMemoryPool._arena = (struct __pyx_obj_4cupy_4cuda_6memory__Arena *(*)(struct __pyx_obj_4cupy_4cuda_6memory_SingleDeviceMemoryPool *, intptr_t, int __pyx_skip_dispatch))__pyx_f_4cupy_4cuda_6memory_22SingleDeviceMemoryPool__arena;
  __pyx_vtable_4cupy_4cuda_6memory_SingleDeviceMemoryPool._alloc = (struct __pyx_obj_4cupy_4cuda_6memory_MemoryPointer *(*)(struct __pyx_obj_4cupy_4cuda_6memory_SingleDeviceMemoryPool *, Py_ssize_t))__pyx_f_4cupy_4cuda_6memory_22SingleDeviceMemoryPool__alloc;
  __pyx_vtable_4cupy_4cuda_6memory_SingleDeviceMemoryPool.malloc = (struct __pyx_obj_4cupy_4cuda_6memory_MemoryPointer *(*)(struct __pyx_obj_4cupy_4cuda_6memory_SingleDeviceMemoryPool *, size_t, int __pyx_skip_dispatch))__pyx_f_4cupy_4cuda_6memory_22SingleDeviceMemoryPool_malloc;
  __pyx_vtable_4cupy_4cuda_6memory_SingleDeviceMemoryPool._malloc = (struct __pyx_obj_4cupy_4cuda_6memory_MemoryPointer *(*)(struct __pyx_obj_4cupy_4cuda_6memory_SingleDeviceMemoryPool *, size_t))__pyx_f_4cupy_4cuda_6memory_22SingleDeviceMemoryPool__malloc;
  __pyx_vtable_4cupy_4cuda_6memory_SingleDeviceMemoryPool.free = (PyObject *(*)(struct __pyx_obj_4cupy_4cuda_6memory_SingleDeviceMemoryPool *, struct __pyx_obj_4cupy_4cuda_6memory__Chunk *))__pyx_f_4cupy_4cuda_6memory_22SingleDeviceMemoryPool_free;
  __pyx_vtable_4cupy_4cuda_6memory_SingleDeviceMemoryPool.free_all_blocks = (PyObject *(*)(struct __pyx_obj_4cupy_4cuda_6memory_SingleDeviceMemoryPool *, int __pyx_skip_dispatch, struct __pyx_opt_args_4cupy_4cuda_6memory_22SingleDeviceMemoryPool_free_all_blocks *__pyx_optional_args))__pyx_f_4cupy_4cuda_6memory_22SingleDeviceMemoryPool_free_all_blocks;
  __pyx_vtable_4cupy_4cuda_6memory_SingleDeviceMemoryPool.free_all_free = (PyObject *(*)(struct __pyx_obj_4cupy_4cuda_6memory_SingleDeviceMemoryPool *, int __pyx_skip_dispatch))__pyx_f_4cupy_4cuda_6memory_22SingleDeviceMemoryPool_free_all_free;
  __pyx_vtable_4cupy_4cuda_6memory_SingleDeviceMemoryPool.n_free_blocks = (size_t (*)(struct __pyx_obj_4cupy_4cuda_6memory_SingleDeviceMemoryPool *, int __pyx_skip_dispatch))__pyx_f_4cupy_4cuda_6memory_22SingleDeviceMemoryPool_n_free_blocks;
  __pyx_vtable_4cupy_4cuda_6memory_SingleDeviceMemoryPool.used_bytes = (size_t (*)(struct __pyx_obj_4cupy_4cuda_6memory_SingleDeviceMemoryPool *, int __pyx_skip_dispatch))__pyx_f_4cupy_4cuda_6memory_22SingleDeviceMemoryPool_used_bytes;
  __pyx_vtable_4cupy_4cuda_6memory_SingleDeviceMemoryPool.free_bytes = (size_t (*)(struct __pyx_obj_4cupy_4cuda_6memory_SingleDeviceMemoryPool *, int __pyx_skip_dispatch))__pyx_f_4cupy_4cuda_6memory_22SingleDeviceMemoryPool_free_bytes;
  __pyx_vtable_4cupy_4cuda_6memory_SingleDeviceMemoryPool.total_bytes = (size_t (*)(struct __pyx_obj_4cupy_4cuda_6memory_SingleDeviceMemoryPool *, int __pyx_skip_dispatch))__pyx_f_4cupy_4cuda_6memory_22SingleDeviceMemoryPool_total_bytes;
  __pyx_vtable_4cupy_4cuda_6memory_SingleDeviceMemoryPool._try_block_total_bytes = (int (*)(struct __pyx_obj_4cupy_4cuda_6memory_SingleDeviceMemoryPool *, size_t))__pyx_f_4cupy_4cuda_6memory_22SingleDeviceMemoryPool__try_block_total_bytes;
  __pyx_vtable_4cupy_4cuda_6memory_SingleDeviceMemoryPool.set_limit = (PyObject *(*)(struct __pyx_obj_4cupy_4cuda_6memory_SingleDeviceMemoryPool *, int __pyx_skip_dispatch, struct __pyx_opt_args_4cupy_4cuda_6memory_22SingleDeviceMemoryPool_set_limit *__pyx_optional_args))__pyx_f_4cupy_4cuda_6memory_22SingleDeviceMemoryPool_set_limit;
  __pyx_vtable_4cupy_4cuda_6memory_SingleDeviceMemoryPool.get_limit = (size_t (*)(struct __pyx_obj_4cupy_4cuda_6memory_SingleDeviceMemoryPool *, int __pyx_skip_dispatch))__pyx_f_4cupy_4cuda_6memory_22SingleDeviceMemoryPool_get_limit;
  __pyx_vtable_4cupy_4cuda_6memory_SingleDeviceMemoryPool._try_malloc = (struct __pyx_obj_4cupy_4cuda_6memory_BaseMemory *(*)(struct __pyx_obj_4cupy_4cuda_6memory_SingleDeviceMemoryPool *, size_t))__pyx_f_4cupy_4cuda_6memory_22SingleDeviceMemoryPool__try_malloc;
  #if CYTHON_USE_TYPE_SPECS
  __pyx_mstate->__pyx_ptype_4cupy_4cuda_6memory_SingleDeviceMemoryPool = (PyTypeObject *) __Pyx_PyType_FromModuleAndSpec(__pyx_m, &__pyx_type_4cupy_4cuda_6memory_SingleDeviceMemoryPool_spec, NULL); if (unlikely(!__pyx_mstate->__pyx_ptype_4cupy_4cuda_6memory_SingleDeviceMemoryPool)) __PYX_ERR(0, 1217, __pyx_L1_error)
  if (__Pyx_fix_up_extension_type_from_spec(&__pyx_type_4cupy_4cuda_6memory_SingleDeviceMemoryPool_spec, __pyx_mstate->__pyx_ptype_4cupy_4cuda_6memory_SingleDeviceMemoryPool) < (0)) __PYX_ERR(0, 1217, __pyx_L1_error)
  #else
  __pyx_mstate->__pyx_ptype_4cupy_4cuda_6memory_SingleDeviceMemoryPool = &__pyx_type_4cupy_4cuda_6memory_SingleDeviceMemoryPool;
  #endif
  #if !CYTHON_COMPILING_IN_LIMITED_API
  #endif
  #if !CYTHON_USE_TYPE_SPECS
  if (__Pyx_PyType_Ready(__pyx_mstate->__pyx_ptype_4cupy_4cuda_6memory_SingleDeviceMemoryPool) < (0)) __PYX_ERR(0, 1217, __pyx_L1_error)
  #endif
  #if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x030E0000
  PyUnstable_Object_EnableDeferredRefcount((PyObject*)__pyx_mstate->__pyx_ptype_4cupy_4cuda_6memory_SingleDeviceMemoryPool);
  #endif
  #if !CYTHON_COMPILING_IN_LIMITED_API
  if ((CYTHON_USE_TYPE_SLOTS && CYTHON_USE_PYTYPE_LOOKUP) && likely(!__pyx_mstate->__pyx_ptype_4cupy_4cuda_6memory_SingleDeviceMemoryPool->tp_dictoffset && __pyx_mstate->__pyx_ptype_4cupy_4cuda_6memory_SingleDeviceMemoryPool->tp_getattro == PyObject_GenericGetAttr)) {
    __pyx_mstate->__pyx_ptype_4cupy_4cuda_6memory_SingleDeviceMemoryPool->tp_getattro = PyObject_GenericGetAttr;
  }
  #endif
  if (__Pyx_SetVtable(__pyx_mstate->__pyx_ptype_4cupy_4cuda_6memory_SingleDeviceMemoryPool, __pyx_vtabptr_4cupy_4cuda_6memory_SingleDeviceMemoryPool) < (0)) __PYX_ERR(0, 1217, __pyx_L1_error)
  if (__Pyx_MergeVtables(__pyx_mstate->__pyx_ptype_4cupy_4cuda_6memory_SingleDeviceMemoryPool) < (0)) __PYX_ERR(0, 1217, __pyx_L1_error)
  if (PyObject_SetAttr(__pyx_m, __pyx_mstate_global->__pyx_n_u_SingleDeviceMemoryPool, (PyObject *) __pyx_mstate->__pyx_ptype_4cupy_4cuda_6memory_SingleDeviceMemoryPool) < (0)) __PYX_ERR(0, 1217, __pyx_L1_error)
  #if CYTHON_USE_TYPE_SLOTS
  if (__pyx_mstate->__pyx_ptype_4cupy_4cuda_6memory_SingleDeviceMemoryPool->tp_weaklistoffset == 0) __pyx_mstate->__pyx_ptype_4cupy_4cuda_6memory_SingleDeviceMemoryPool->tp_weaklistoffset = offsetof(struct __pyx_obj_4cupy_4cuda_6memory_SingleDeviceMemoryPool, __weakref__);
  #endif
  if (__Pyx_setup_reduce((PyObject *) __pyx_mstate->__pyx_ptype_4cupy_4cuda_6memory_SingleDeviceMemoryPool) < (0)) __PYX_ERR(0, 1217, __pyx_L1_error)
  __pyx_vtabptr_4cupy_4cuda_6memory_MemoryAsyncPool = &__pyx_vtable_4cupy_4cuda_6memory_MemoryAsyncPool;
  __pyx_vtable_4cupy_4cuda_6memory_MemoryAsyncPool.set_pool = (intptr_t (*)(struct __pyx_obj_4cupy_4cuda_6memory_MemoryAsyncPool *, PyObject *, int))__pyx_f_4cupy_4cuda_6memory_15MemoryAsyncPool_set_pool;
  __pyx_vtable_4cupy_4cuda_6memory_MemoryAsyncPool.malloc = (struct __pyx_obj_4cupy_4cuda_6memory_MemoryPointer *(*)(struct __pyx_obj_4cupy_4cuda_6memory_MemoryAsyncPool *, size_t, int __pyx_skip_dispatch))__pyx_f_4cupy_4cuda_6memory_15MemoryAsyncPool_malloc;
  __pyx_vtable_4cupy_4cuda_6memory_MemoryAsyncPool.free_all_blocks = (PyObject *(*)(struct __pyx_obj_4cupy_4cuda_6memory_MemoryAsyncPool *, int __pyx_skip_dispatch, struct __pyx_opt_args_4cupy_4cuda_6memory_15MemoryAsyncPool_free_all_blocks *__pyx_optional_args))__pyx_f_4cupy_4cuda_6memory_15MemoryAsyncPool_free_all_blocks;
  __pyx_vtable_4cupy_4cuda_6memory_MemoryAsyncPool.n_free_blocks = (size_t (*)(struct __pyx_obj_4cupy_4cuda_6memory_MemoryAsyncPool *, int __pyx_skip_dispatch))__pyx_f_4cupy_4cuda_6memory_15MemoryAsyncPool_n_free_blocks;
  __pyx_vtable_4cupy_4cuda_6memory_MemoryAsyncPool.used_bytes = (size_t (*)(struct __pyx_obj_4cupy_4cuda_6memory_MemoryAsyncPool *, int __pyx_skip_dispatch))__pyx_f_4cupy_4cuda_6memory_15MemoryAsyncPool_used_bytes;
  __pyx_vtable_4cupy_4cuda_6memory_MemoryAsyncPool.free_bytes = (size_t (*)(struct __pyx_obj_4cupy_4cuda_6memory_MemoryAsyncPool *, int __pyx_skip_dispatch))__pyx_f_4cupy_4cuda_6memory_15MemoryAsyncPool_free_bytes;
  __pyx_vtable_4cupy_4cuda_6memory_MemoryAsyncPool.total_bytes = (size_t (*)(struct __pyx_obj_4cupy_4cuda_6memory_MemoryAsyncPool *, int __pyx_skip_dispatch))__pyx_f_4cupy_4cuda_6memory_15MemoryAsyncPool_total_bytes;
  __pyx_vtable_4cupy_4cuda_6memory_MemoryAsyncPool.set_limit = (PyObject *(*)(struct __pyx_obj_4cupy_4cuda_6memory_MemoryAsyncPool *, int __pyx_skip_dispatch, struct __pyx_opt_args_4cupy_4cuda_6memory_15MemoryAsyncPool_set_limit *__pyx_optional_args))__pyx_f_4cupy_4cuda_6memory_15MemoryAsyncPool_set_limit;
  __pyx_vtable_4cupy_4cuda_6memory_MemoryAsyncPool.get_limit = (size_t (*)(struct __pyx_obj_4cupy_4cuda_6memory_MemoryAsyncPool *, int __pyx_skip_dispatch))__pyx_f_4cupy_4cuda_6memory_15MemoryAsyncPool_get_limit;
  #if CYTHON_USE_TYPE_SPECS
  __pyx_mstate->__pyx_ptype_4cupy_4cuda_6memory_MemoryAsyncPool = (PyTypeObject *) __Pyx_PyType_FromModuleAndSpec(__pyx_m, &__pyx_type_4cupy_4cuda_6memory_MemoryAsyncPool_spec, NULL); if (unlikely(!__pyx_mstate->__pyx_ptype_4cupy_4cuda_6memory_MemoryAsyncPool)) __PYX_ERR(0, 1740, __pyx_L1_error)
  if (__Pyx_fix_up_extension_type_from_spec(&__pyx_type_4cupy_4cuda_6memory_MemoryAsyncPool_spec, __pyx_mstate->__pyx_ptype_4cupy_4cuda_6memory_MemoryAsyncPool) < (0)) __PYX_ERR(0, 1740, __pyx_L1_error)
  #else
  __pyx_mstate->__pyx_ptype_4cupy_4cuda_6memory_MemoryAsyncPool = &__pyx_type_4cupy_4cuda_6memory_MemoryAsyncPool;
  #endif
  #if !CYTHON_COMPILING_IN_LIMITED_API
  #endif
  #if !CYTHON_USE_TYPE_SPECS
  if (__Pyx_PyType_Ready(__pyx_mstate->__pyx_ptype_4cupy_4cuda_6memory_MemoryAsyncPool) < (0)) __PYX_ERR(0, 1740, __pyx_L1_error)
  #endif
  #if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x030E0000
  PyUnstable_Object_EnableDeferredRefcount((PyObject*)__pyx_mstate->__pyx_ptype_4cupy_4cuda_6memory_MemoryAsyncPool);
  #endif
  #if !CYTHON_COMPILING_IN_LIMITED_API
  if ((CYTHON_USE_TYPE_SLOTS && CYTHON_USE_PYTYPE_LOOKUP) && likely(!__pyx_mstate->__pyx_ptype_4cupy_4cuda_6memory_MemoryAsyncPool->tp_dictoffset && __pyx_mstate->__pyx_ptype_4cupy_4cuda_6memory_MemoryAsyncPool->tp_getattro == PyObject_GenericGetAttr)) {
    __pyx_mstate->__pyx_ptype_4cupy_4cuda_6memory_MemoryAsyncPool->tp_getattro = PyObject_GenericGetAttr;
  }
  #endif
  if (__Pyx_SetVtable(__pyx_mstate->__pyx_ptype_4cupy_4cuda_6memory_MemoryAsyncPool, __pyx_vtabptr_4cupy_4cuda_6memory_MemoryAsyncPool) < (0)) __PYX_ERR(0, 1740, __pyx_L1_error)
  if (__Pyx_MergeVtables(__pyx_mstate->__pyx_ptype_4cupy_4cuda_6memory_MemoryAsyncPool) < (0)) __PYX_ERR(0, 1740, __pyx_L1_error)
  if (PyObject_SetAttr(__pyx_m, __pyx_mstate_global->__pyx_n_u_MemoryAsyncPool, (PyObject *) __pyx_mstate->__pyx_ptype_4cupy_4cuda_6memory_MemoryAsyncPool) < (0)) __PYX_ERR(0, 1740, __pyx_L1_error)
  if (__Pyx_setup_reduce((PyObject *) __pyx_mstate->__pyx_ptype_4cupy_4cuda_6memory_MemoryAsyncPool) < (0)) __PYX_ERR(0, 1740, __pyx_L1_error)
  #if CYTHON_USE_TYPE_SPECS
  __pyx_mstate->__pyx_ptype_4cupy_4cuda_6memory___pyx_scope_struct___ensure_pools_and_return_device_pool = (PyTypeObject *) __Pyx_PyType_FromModuleAndSpec(__pyx_m, &__pyx_type_4cupy_4cuda_6memory___pyx_scope_struct___ensure_pools_and_return_device_pool_spec, NULL); if (unlikely(!__pyx_mstate->__pyx_ptype_4cupy_4cuda_6memory___pyx_scope_struct___ensure_pools_and_return_device_pool)) __PYX_ERR(0, 1599, __pyx_L1_error)
  if (__Pyx_fix_up_extension_type_from_spec(&__pyx_type_4cupy_4cuda_6memory___pyx_scope_struct___ensure_pools_and_return_device_pool_spec, __pyx_mstate->__pyx_ptype_4cupy_4cuda_6memory___pyx_scope_struct___ensure_pools_and_return_device_pool) < (0)) __PYX_ERR(0, 1599, __pyx_L1_error)
  #else
  __pyx_mstate->__pyx_ptype_4cupy_4cuda_6memory___pyx_scope_struct___ensure_pools_and_return_device_pool = &__pyx_type_4cupy_4cuda_6memory___pyx_scope_struct___ensure_pools_and_return_device_pool;
  #endif
  #if !CYTHON_COMPILING_IN_LIMITED_API
  #endif
  #if !CYTHON_USE_TYPE_SPECS
  if (__Pyx_PyType_Ready(__pyx_mstate->__pyx_ptype_4cupy_4cuda_6memory___pyx_scope_struct___ensure_pools_and_return_device_pool) < (0)) __PYX_ERR(0, 1599, __pyx_L1_error)
  #endif
  #if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x030E0000
  PyUnstable_Object_EnableDeferredRefcount((PyObject*)__pyx_mstate->__pyx_ptype_4cupy_4cuda_6memory___pyx_scope_struct___ensure_pools_and_return_device_pool);
  #endif
  #if !CYTHON_COMPILING_IN_LIMITED_API
  if ((CYTHON_USE_TYPE_SLOTS && CYTHON_USE_PYTYPE_LOOKUP) && likely(!__pyx_mstate->__pyx_ptype_4cupy_4cuda_6memory___pyx_scope_struct___ensure_pools_and_return_device_pool->tp_dictoffset && __pyx_mstate->__pyx_ptype_4cupy_4cuda_6memory___pyx_scope_struct___ensure_pools_and_return_device_pool->tp_getattro == PyObject_GenericGetAttr)) {
    __pyx_mstate->__pyx_ptype_4cupy_4cuda_6memory___pyx_scope_struct___ensure_pools_and_return_device_pool->tp_getattro = PyObject_GenericGetAttr;
  }
  #endif
  #if CYTHON_USE_TYPE_SPECS
  __pyx_mstate->__pyx_ptype_4cupy_4cuda_6memory___pyx_scope_struct_1_genexpr = (PyTypeObject *) __Pyx_PyType_FromModuleAndSpec(__pyx_m, &__pyx_type_4cupy_4cuda_6memory___pyx_scope_struct_1_genexpr_spec, NULL); if (unlikely(!__pyx_mstate->__pyx_ptype_4cupy_4cuda_6memory___pyx_scope_struct_1_genexpr)) __PYX_ERR(0, 1603, __pyx_L1_error)
  if (__Pyx_fix_up_extension_type_from_spec(&__pyx_type_4cupy_4cuda_6memory___pyx_scope_struct_1_genexpr_spec, __pyx_mstate->__pyx_ptype_4cupy_4cuda_6memory___pyx_scope_struct_1_genexpr) < (0)) __PYX_ERR(0, 1603, __pyx_L1_error)
  #else
  __pyx_mstate->__pyx_ptype_4cupy_4cuda_6memory___pyx_scope_struct_1_genexpr = &__pyx_type_4cupy_4cuda_6memory___pyx_scope_struct_1_genexpr;
  #endif
  #if !CYTHON_COMPILING_IN_LIMITED_API
  #endif
  #if !CYTHON_USE_TYPE_SPECS
  if (__Pyx_PyType_Ready(__pyx_mstate->__pyx_ptype_4cupy_4cuda_6memory___pyx_scope_struct_1_genexpr) < (0)) __PYX_ERR(0, 1603, __pyx_L1_error)
  #endif
  #if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x030E0000
  PyUnstable_Object_EnableDeferredRefcount((PyObject*)__pyx_mstate->__pyx_ptype_4cupy_4cuda_6memory___pyx_scope_struct_1_genexpr);
  #endif
  #if !CYTHON_COMPILING_IN_LIMITED_API
  if ((CYTHON_USE_TYPE_SLOTS && CYTHON_USE_PYTYPE_LOOKUP) && likely(!__pyx_mstate->__pyx_ptype_4cupy_4cuda_6memory___pyx_scope_struct_1_genexpr->tp_dictoffset && __pyx_mstate->__pyx_ptype_4cupy_4cuda_6memory___pyx_scope_struct_1_genexpr->tp_getattro == PyObject_GenericGetAttr)) {
    __pyx_mstate->__pyx_ptype_4cupy_4cuda_6memory___pyx_scope_struct_1_genexpr->tp_getattro = PyObject_GenericGetAttr;
  }
  #endif
  __Pyx_RefNannyFinishContext();
  return 0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_RefNannyFinishContext();
  return -1;
}

static int __Pyx_modinit_type_import_code(__pyx_mstatetype *__pyx_mstate) {
  __Pyx_RefNannyDeclarations
  CYTHON_UNUSED_VAR(__pyx_mstate);
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__Pyx_modinit_type_import_code", 0);
  /*--- Type import code ---*/
  __pyx_t_1 = PyImport_ImportModule("cupy.cuda.device"); if (unlikely(!__pyx_t_1)) __PYX_ERR(4, 13, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_mstate->__pyx_ptype_4cupy_4cuda_6device_Handle = __Pyx_ImportType_3_2_5(__pyx_t_1, "cupy.cuda.device", "Handle",
  #if defined(PYPY_VERSION_NUM) && PYPY_VERSION_NUM < 0x050B0000
  sizeof(struct __pyx_obj_4cupy_4cuda_6device_Handle), __PYX_GET_STRUCT_ALIGNMENT_3_2_5(struct __pyx_obj_4cupy_4cuda_6device_Handle),
  #elif CYTHON_COMPILING_IN_LIMITED_API
  sizeof(struct __pyx_obj_4cupy_4cuda_6device_Handle), __PYX_GET_STRUCT_ALIGNMENT_3_2_5(struct __pyx_obj_4cupy_4cuda_6device_Handle),
  #else
  sizeof(struct __pyx_obj_4cupy_4cuda_6device_Handle), __PYX_GET_STRUCT_ALIGNMENT_3_2_5(struct __pyx_obj_4cupy_4cuda_6device_Handle),
  #endif
  __Pyx_ImportType_CheckSize_Warn_3_2_5); if (!__pyx_mstate->__pyx_ptype_4cupy_4cuda_6device_Handle) __PYX_ERR(4, 13, __pyx_L1_error)
  __pyx_mstate->__pyx_ptype_4cupy_4cuda_6device_Device = __Pyx_ImportType_3_2_5(__pyx_t_1, "cupy.cuda.device", "Device",
  #if defined(PYPY_VERSION_NUM) && PYPY_VERSION_NUM < 0x050B0000
  sizeof(struct __pyx_obj_4cupy_4cuda_6device_Device), __PYX_GET_STRUCT_ALIGNMENT_3_2_5(struct __pyx_obj_4cupy_4cuda_6device_Device),
  #elif CYTHON_COMPILING_IN_LIMITED_API
  sizeof(struct __pyx_obj_4cupy_4cuda_6device_Device), __PYX_GET_STRUCT_ALIGNMENT_3_2_5(struct __pyx_obj_4cupy_4cuda_6device_Device),
  #else
  sizeof(struct __pyx_obj_4cupy_4cuda_6device_Device), __PYX_GET_STRUCT_ALIGNMENT_3_2_5(struct __pyx_obj_4cupy_4cuda_6device_Device),
  #endif
  __Pyx_ImportType_CheckSize_Warn_3_2_5); if (!__pyx_mstate->__pyx_ptype_4cupy_4cuda_6device_Device) __PYX_ERR(4, 18, __pyx_L1_error)
  __pyx_vtabptr_4cupy_4cuda_6device_Device = (struct __pyx_vtabstruct_4cupy_4cuda_6device_Device*)__Pyx_GetVtable(__pyx_mstate->__pyx_ptype_4cupy_4cuda_6device_Device); if (unlikely(!__pyx_vtabptr_4cupy_4cuda_6device_Device)) __PYX_ERR(4, 18, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_1 = PyImport_ImportModule(__Pyx_BUILTIN_MODULE_NAME); if (unlikely(!__pyx_t_1)) __PYX_ERR(5, 9, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_mstate->__pyx_ptype_7cpython_4type_type = __Pyx_ImportType_3_2_5(__pyx_t_1, __Pyx_BUILTIN_MODULE_NAME, "type",
  #if defined(PYPY_VERSION_NUM) && PYPY_VERSION_NUM < 0x050B0000
  sizeof(PyTypeObject), __PYX_GET_STRUCT_ALIGNMENT_3_2_5(PyTypeObject),
  #elif CYTHON_COMPILING_IN_LIMITED_API
  0, 0,
  #else
  sizeof(PyHeapTypeObject), __PYX_GET_STRUCT_ALIGNMENT_3_2_5(PyHeapTypeObject),
  #endif
  __Pyx_ImportType_CheckSize_Warn_3_2_5); if (!__pyx_mstate->__pyx_ptype_7cpython_4type_type) __PYX_ERR(5, 9, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_1 = PyImport_ImportModule(__Pyx_BUILTIN_MODULE_NAME); if (unlikely(!__pyx_t_1)) __PYX_ERR(6, 8, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_mstate->__pyx_ptype_7cpython_4bool_bool = __Pyx_ImportType_3_2_5(__pyx_t_1, __Pyx_BUILTIN_MODULE_NAME, "bool",
  #if defined(PYPY_VERSION_NUM) && PYPY_VERSION_NUM < 0x050B0000
  sizeof(PyLongObject), __PYX_GET_STRUCT_ALIGNMENT_3_2_5(PyLongObject),
  #elif CYTHON_COMPILING_IN_LIMITED_API
  0, 0,
  #else
  sizeof(PyLongObject), __PYX_GET_STRUCT_ALIGNMENT_3_2_5(PyLongObject),
  #endif
  __Pyx_ImportType_CheckSize_Warn_3_2_5); if (!__pyx_mstate->__pyx_ptype_7cpython_4bool_bool) __PYX_ERR(6, 8, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_1 = PyImport_ImportModule(__Pyx_BUILTIN_MODULE_NAME); if (unlikely(!__pyx_t_1)) __PYX_ERR(7, 16, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_mstate->__pyx_ptype_7cpython_7complex_complex = __Pyx_ImportType_3_2_5(__pyx_t_1, __Pyx_BUILTIN_MODULE_NAME, "complex",
  #if defined(PYPY_VERSION_NUM) && PYPY_VERSION_NUM < 0x050B0000
  sizeof(PyComplexObject), __PYX_GET_STRUCT_ALIGNMENT_3_2_5(PyComplexObject),
  #elif CYTHON_COMPILING_IN_LIMITED_API
  0, 0,
  #else
  sizeof(PyComplexObject), __PYX_GET_STRUCT_ALIGNMENT_3_2_5(PyComplexObject),
  #endif
  __Pyx_ImportType_CheckSize_Warn_3_2_5); if (!__pyx_mstate->__pyx_ptype_7cpython_7complex_complex) __PYX_ERR(7, 16, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_1 = PyImport_ImportModule("cupy_backends.cuda.api.runtime"); if (unlikely(!__pyx_t_1)) __PYX_ERR(8, 8, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_mstate->__pyx_ptype_13cupy_backends_4cuda_3api_7runtime_PointerAttributes = __Pyx_ImportType_3_2_5(__pyx_t_1, "cupy_backends.cuda.api.runtime", "PointerAttributes",
  #if defined(PYPY_VERSION_NUM) && PYPY_VERSION_NUM < 0x050B0000
  sizeof(struct __pyx_obj_13cupy_backends_4cuda_3api_7runtime_PointerAttributes), __PYX_GET_STRUCT_ALIGNMENT_3_2_5(struct __pyx_obj_13cupy_backends_4cuda_3api_7runtime_PointerAttributes),
  #elif CYTHON_COMPILING_IN_LIMITED_API
  sizeof(struct __pyx_obj_13cupy_backends_4cuda_3api_7runtime_PointerAttributes), __PYX_GET_STRUCT_ALIGNMENT_3_2_5(struct __pyx_obj_13cupy_backends_4cuda_3api_7runtime_PointerAttributes),
  #else
  sizeof(struct __pyx_obj_13cupy_backends_4cuda_3api_7runtime_PointerAttributes), __PYX_GET_STRUCT_ALIGNMENT_3_2_5(struct __pyx_obj_13cupy_backends_4cuda_3api_7runtime_PointerAttributes),
  #endif
  __Pyx_ImportType_CheckSize_Warn_3_2_5); if (!__pyx_mstate->__pyx_ptype_13cupy_backends_4cuda_3api_7runtime_PointerAttributes) __PYX_ERR(8, 8, __pyx_L1_error)
  __pyx_mstate->__pyx_ptype_13cupy_backends_4cuda_3api_7runtime_MemPoolProps = __Pyx_ImportType_3_2_5(__pyx_t_1, "cupy_backends.cuda.api.runtime", "MemPoolProps",
  #if defined(PYPY_VERSION_NUM) && PYPY_VERSION_NUM < 0x050B0000
  sizeof(struct __pyx_obj_13cupy_backends_4cuda_3api_7runtime_MemPoolProps), __PYX_GET_STRUCT_ALIGNMENT_3_2_5(struct __pyx_obj_13cupy_backends_4cuda_3api_7runtime_MemPoolProps),
  #elif CYTHON_COMPILING_IN_LIMITED_API
  sizeof(struct __pyx_obj_13cupy_backends_4cuda_3api_7runtime_MemPoolProps), __PYX_GET_STRUCT_ALIGNMENT_3_2_5(struct __pyx_obj_13cupy_backends_4cuda_3api_7runtime_MemPoolProps),
  #else
  sizeof(struct __pyx_obj_13cupy_backends_4cuda_3api_7runtime_MemPoolProps), __PYX_GET_STRUCT_ALIGNMENT_3_2_5(struct __pyx_obj_13cupy_backends_4cuda_3api_7runtime_MemPoolProps),
  #endif
  __Pyx_ImportType_CheckSize_Warn_3_2_5); if (!__pyx_mstate->__pyx_ptype_13cupy_backends_4cuda_3api_7runtime_MemPoolProps) __PYX_ERR(8, 15, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_RefNannyFinishContext();
  return 0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_RefNannyFinishContext();
  return -1;
}

static int __Pyx_modinit_variable_import_code(__pyx_mstatetype *__pyx_mstate) {
  __Pyx_RefNannyDeclarations
  CYTHON_UNUSED_VAR(__pyx_mstate);
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__Pyx_modinit_variable_import_code", 0);
  /*--- Variable import code ---*/
  {
    __pyx_t_1 = PyImport_ImportModule("cupy_backends.cuda.api.runtime"); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    const char * __pyx_import_signature = __Pyx_PyBytes_AsString(__pyx_mstate_global->__pyx_kp_b_int__is_hip_environment_deviceAt);
    #if !CYTHON_ASSUME_SAFE_MACROS
    if (unlikely(!__pyx_import_signature)) __PYX_ERR(0, 1, __pyx_L1_error)
    #endif
    const char * __pyx_import_name = __pyx_import_signature + 11;
    void **const __pyx_import_pointers[] = {(void **)&__pyx_vp_13cupy_backends_4cuda_3api_7runtime__is_hip_environment, (void **)&__pyx_vp_13cupy_backends_4cuda_3api_7runtime_deviceAttributeComputeCapabilityMajor, (void **)&__pyx_vp_13cupy_backends_4cuda_3api_7runtime_deviceAttributeComputeCapabilityMinor, (void **)&__pyx_vp_13cupy_backends_4cuda_3api_7runtime_errorContextIsDestroyed, (void **)&__pyx_vp_13cupy_backends_4cuda_3api_7runtime_errorInvalidResourceHandle, (void **)&__pyx_vp_13cupy_backends_4cuda_3api_7runtime_errorInvalidValue, (void **)&__pyx_vp_13cupy_backends_4cuda_3api_7runtime_errorMemoryAllocation, (void **)&__pyx_vp_13cupy_backends_4cuda_3api_7runtime_errorPeerAccessAlreadyEnabled, (void **) NULL};
    void **const *__pyx_import_pointer = __pyx_import_pointers;
    const char *__pyx_import_current_signature = __pyx_import_signature;
    while (*__pyx_import_pointer) {
      if (__Pyx_ImportVoidPtr_3_2_5(__pyx_t_1, __pyx_import_name, *__pyx_import_pointer, __pyx_import_current_signature) < (0)) __PYX_ERR(0, 1, __pyx_L1_error)
      ++__pyx_import_pointer;
      __pyx_import_name = strchr(__pyx_import_name, '\0') + 1;
      __pyx_import_signature = strchr(__pyx_import_signature, '\0') + 1;
      if (*__pyx_import_signature != '\0') __pyx_import_current_signature = __pyx_import_signature;
    }
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  }
  __Pyx_RefNannyFinishContext();
  return 0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_RefNannyFinishContext();
  return -1;
}

static int __Pyx_modinit_function_import_code(__pyx_mstatetype *__pyx_mstate) {
  __Pyx_RefNannyDeclarations
  CYTHON_UNUSED_VAR(__pyx_mstate);
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__Pyx_modinit_function_import_code", 0);
  /*--- Function import code ---*/
  {
    __pyx_t_1 = PyImport_ImportModule("cupy.cuda.device"); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    const char * __pyx_import_signature = __Pyx_PyBytes_AsString(__pyx_mstate_global->__pyx_kp_b_int_int___pyx_skip_dispatch_int);
    #if !CYTHON_ASSUME_SAFE_MACROS
    if (unlikely(!__pyx_import_signature)) __PYX_ERR(0, 1, __pyx_L1_error)
    #endif
    const char * __pyx_import_name = __pyx_import_signature + 45;
    void (**const __pyx_import_pointers[])(void) = {(void (**)(void))&__pyx_f_4cupy_4cuda_6device_get_device_id, (void (**)(void))&__pyx_f_4cupy_4cuda_6device__enable_peer_access, (void (**)(void)) NULL};
    void (**const *__pyx_import_pointer)(void) = __pyx_import_pointers;
    const char *__pyx_import_current_signature = __pyx_import_signature;
    while (*__pyx_import_pointer) {
      if (__Pyx_ImportFunction_3_2_5(__pyx_t_1, __pyx_import_name, *__pyx_import_pointer, __pyx_import_current_signature) < (0)) __PYX_ERR(0, 1, __pyx_L1_error)
      ++__pyx_import_pointer;
      __pyx_import_name = strchr(__pyx_import_name, '\0') + 1;
      __pyx_import_signature = strchr(__pyx_import_signature, '\0') + 1;
      if (*__pyx_import_signature != '\0') __pyx_import_current_signature = __pyx_import_signature;
    }
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  }
  {
    __pyx_t_1 = PyImport_ImportModule("cupy.cuda.memory_hook"); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    const char * __pyx_import_signature = __Pyx_PyBytes_AsString(__pyx_mstate_global->__pyx_kp_b_PyObject_int___pyx_skip_dispatch_2);
    #if !CYTHON_ASSUME_SAFE_MACROS
    if (unlikely(!__pyx_import_signature)) __PYX_ERR(0, 1, __pyx_L1_error)
    #endif
    const char * __pyx_import_name = __pyx_import_signature + 66;
    void (**const __pyx_import_pointers[])(void) = {(void (**)(void))&__pyx_f_4cupy_4cuda_11memory_hook_get_memory_hooks, (void (**)(void))&__pyx_f_4cupy_4cuda_11memory_hook__has_memory_hooks, (void (**)(void)) NULL};
    void (**const *__pyx_import_pointer)(void) = __pyx_import_pointers;
    const char *__pyx_import_current_signature = __pyx_import_signature;
    while (*__pyx_import_pointer) {
      if (__Pyx_ImportFunction_3_2_5(__pyx_t_1, __pyx_import_name, *__pyx_import_pointer, __pyx_import_current_signature) < (0)) __PYX_ERR(0, 1, __pyx_L1_error)
      ++__pyx_import_pointer;
      __pyx_import_name = strchr(__pyx_import_name, '\0') + 1;
      __pyx_import_signature = strchr(__pyx_import_signature, '\0') + 1;
      if (*__pyx_import_signature != '\0') __pyx_import_current_signature = __pyx_import_signature;
    }
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  }
  {
    __pyx_t_1 = PyImport_ImportModule("cupy.cuda.stream"); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    const char * __pyx_import_signature = __Pyx_PyBytes_AsString(__pyx_mstate_global->__pyx_kp_b_PyObject_int___pyx_skip_dispatch_3);
    #if !CYTHON_ASSUME_SAFE_MACROS
    if (unlikely(!__pyx_import_signature)) __PYX_ERR(0, 1, __pyx_L1_error)
    #endif
    const char * __pyx_import_name = __pyx_import_signature + 135;
    void (**const __pyx_import_pointers[])(void) = {(void (**)(void))&__pyx_f_4cupy_4cuda_6stream_get_current_stream, (void (**)(void))&__pyx_f_4cupy_4cuda_6stream_get_current_stream_ptr, (void (**)(void)) NULL};
    void (**const *__pyx_import_pointer)(void) = __pyx_import_pointers;
    const char *__pyx_import_current_signature = __pyx_import_signature;
    while (*__pyx_import_pointer) {
      if (__Pyx_ImportFunction_3_2_5(__pyx_t_1, __pyx_import_name, *__pyx_import_pointer, __pyx_import_current_signature) < (0)) __PYX_ERR(0, 1, __pyx_L1_error)
      ++__pyx_import_pointer;
      __pyx_import_name = strchr(__pyx_import_name, '\0') + 1;
      __pyx_import_signature = strchr(__pyx_import_signature, '\0') + 1;
      if (*__pyx_import_signature != '\0') __pyx_import_current_signature = __pyx_import_signature;
    }
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  }
  {
    __pyx_t_1 = PyImport_ImportModule("cupy_backends.cuda.api.driver"); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    const char * __pyx_import_signature = __Pyx_PyBytes_AsString(__pyx_mstate_global->__pyx_kp_b_PyObject_int___pyx_skip_dispatch_4);
    #if !CYTHON_ASSUME_SAFE_MACROS
    if (unlikely(!__pyx_import_signature)) __PYX_ERR(0, 1, __pyx_L1_error)
    #endif
    const char * __pyx_import_name = __pyx_import_signature + 36;
    void (**const __pyx_import_pointers[])(void) = {(void (**)(void))&__pyx_f_13cupy_backends_4cuda_3api_6driver_get_build_version, (void (**)(void)) NULL};
    void (**const *__pyx_import_pointer)(void) = __pyx_import_pointers;
    const char *__pyx_import_current_signature = __pyx_import_signature;
    while (*__pyx_import_pointer) {
      if (__Pyx_ImportFunction_3_2_5(__pyx_t_1, __pyx_import_name, *__pyx_import_pointer, __pyx_import_current_signature) < (0)) __PYX_ERR(0, 1, __pyx_L1_error)
      ++__pyx_import_pointer;
      __pyx_import_name = strchr(__pyx_import_name, '\0') + 1;
      __pyx_import_signature = strchr(__pyx_import_signature, '\0') + 1;
      if (*__pyx_import_signature != '\0') __pyx_import_current_signature = __pyx_import_signature;
    }
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  }
  {
    __pyx_t_1 = PyImport_ImportModule("cupy_backends.cuda.api.runtime"); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    const char * __pyx_import_signature = __Pyx_PyBytes_AsString(__pyx_mstate_global->__pyx_kp_b_PyObject_int___pyx_skip_dispatch_5);
    #if !CYTHON_ASSUME_SAFE_MACROS
    if (unlikely(!__pyx_import_signature)) __PYX_ERR(0, 1, __pyx_L1_error)
    #endif
    const char * __pyx_import_name = __pyx_import_signature + 1319;
    void (**const __pyx_import_pointers[])(void) = {(void (**)(void))&__pyx_f_13cupy_backends_4cuda_3api_7runtime_memGetInfo, (void (**)(void))&__pyx_f_13cupy_backends_4cuda_3api_7runtime_setDevice, (void (**)(void))&__pyx_f_13cupy_backends_4cuda_3api_7runtime_deviceSetMemPool, (void (**)(void))&__pyx_f_13cupy_backends_4cuda_3api_7runtime_free, (void (**)(void))&__pyx_f_13cupy_backends_4cuda_3api_7runtime_streamSynchronize, (void (**)(void))&__pyx_f_13cupy_backends_4cuda_3api_7runtime_memPoolSetAttribute, (void (**)(void))&__pyx_f_13cupy_backends_4cuda_3api_7runtime_memPoolGetAttribute, (void (**)(void))&__pyx_f_13cupy_backends_4cuda_3api_7runtime_memset, (void (**)(void))&__pyx_f_13cupy_backends_4cuda_3api_7runtime_memsetAsync, (void (**)(void))&__pyx_f_13cupy_backends_4cuda_3api_7runtime_freeAsync, (void (**)(void))&__pyx_f_13cupy_backends_4cuda_3api_7runtime_memcpy, (void (**)(void))&__pyx_f_13cupy_backends_4cuda_3api_7runtime_memcpyAsync, (void (**)(void))&__pyx_f_13cupy_backends_4cuda_3api_7runtime_memPoolTrimTo, (void (**)(void))&__pyx_f_13cupy_backends_4cuda_3api_7runtime_memAdvise, (void (**)(void))&__pyx_f_13cupy_backends_4cuda_3api_7runtime_memPrefetchAsync, (void (**)(void))&__pyx_f_13cupy_backends_4cuda_3api_7runtime__ensure_context, (void (**)(void))&__pyx_f_13cupy_backends_4cuda_3api_7runtime_driverGetVersion, (void (**)(void))&__pyx_f_13cupy_backends_4cuda_3api_7runtime_getDevice, (void (**)(void))&__pyx_f_13cupy_backends_4cuda_3api_7runtime_getDeviceCount, (void (**)(void))&__pyx_f_13cupy_backends_4cuda_3api_7runtime_deviceGetAttribute, (void (**)(void))&__pyx_f_13cupy_backends_4cuda_3api_7runtime_streamIsCapturing, (void (**)(void))&__pyx_f_13cupy_backends_4cuda_3api_7runtime_deviceGetDefaultMemPool, (void (**)(void))&__pyx_f_13cupy_backends_4cuda_3api_7runtime_deviceGetMemPool, (void (**)(void))&__pyx_f_13cupy_backends_4cuda_3api_7runtime_malloc, (void (**)(void))&__pyx_f_13cupy_backends_4cuda_3api_7runtime_mallocManaged, (void (**)(void))&__pyx_f_13cupy_backends_4cuda_3api_7runtime_mallocAsync, (void (**)(void))&__pyx_f_13cupy_backends_4cuda_3api_7runtime_pointerGetAttributes, (void (**)(void)) NULL};
    void (**const *__pyx_import_pointer)(void) = __pyx_import_pointers;
    const char *__pyx_import_current_signature = __pyx_import_signature;
    while (*__pyx_import_pointer) {
      if (__Pyx_ImportFunction_3_2_5(__pyx_t_1, __pyx_import_name, *__pyx_import_pointer, __pyx_import_current_signature) < (0)) __PYX_ERR(0, 1, __pyx_L1_error)
      ++__pyx_import_pointer;
      __pyx_import_name = strchr(__pyx_import_name, '\0') + 1;
      __pyx_import_signature = strchr(__pyx_import_signature, '\0') + 1;
      if (*__pyx_import_signature != '\0') __pyx_import_current_signature = __pyx_import_signature;
    }
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  }
  __Pyx_RefNannyFinishContext();
  return 0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_RefNannyFinishContext();
  return -1;
}

#if CYTHON_PEP489_MULTI_PHASE_INIT
static PyObject* __pyx_pymod_create(PyObject *spec, PyModuleDef *def); /*proto*/
static int __pyx_pymod_exec_memory(PyObject* module); /*proto*/
static PyModuleDef_Slot __pyx_moduledef_slots[] = {
  {Py_mod_create, (void*)__pyx_pymod_create},
  {Py_mod_exec, (void*)__pyx_pymod_exec_memory},
  #if CYTHON_COMPILING_IN_CPYTHON_FREETHREADING
  {Py_mod_gil, __Pyx_FREETHREADING_COMPATIBLE},
  #endif
  #if PY_VERSION_HEX >= 0x030C0000 && CYTHON_USE_MODULE_STATE
  {Py_mod_multiple_interpreters, Py_MOD_MULTIPLE_INTERPRETERS_NOT_SUPPORTED},
  #endif
  {0, NULL}
};
#endif

#ifdef __cplusplus
namespace {
  struct PyModuleDef __pyx_moduledef =
  #else
  static struct PyModuleDef __pyx_moduledef =
  #endif
  {
      PyModuleDef_HEAD_INIT,
      "memory",
      0, /* m_doc */
    #if CYTHON_USE_MODULE_STATE
      sizeof(__pyx_mstatetype), /* m_size */
    #else
      (CYTHON_PEP489_MULTI_PHASE_INIT) ? 0 : -1, /* m_size */
    #endif
      __pyx_methods /* m_methods */,
    #if CYTHON_PEP489_MULTI_PHASE_INIT
      __pyx_moduledef_slots, /* m_slots */
    #else
      NULL, /* m_reload */
    #endif
    #if CYTHON_USE_MODULE_STATE
      __pyx_m_traverse, /* m_traverse */
      __pyx_m_clear, /* m_clear */
      NULL /* m_free */
    #else
      NULL, /* m_traverse */
      NULL, /* m_clear */
      NULL /* m_free */
    #endif
  };
  #ifdef __cplusplus
} /* anonymous namespace */
#endif

/* PyModInitFuncType */
#ifndef CYTHON_NO_PYINIT_EXPORT
  #define __Pyx_PyMODINIT_FUNC PyMODINIT_FUNC
#else
  #ifdef __cplusplus
  #define __Pyx_PyMODINIT_FUNC extern "C" PyObject *
  #else
  #define __Pyx_PyMODINIT_FUNC PyObject *
  #endif
#endif

__Pyx_PyMODINIT_FUNC PyInit_memory(void) CYTHON_SMALL_CODE; /*proto*/
__Pyx_PyMODINIT_FUNC PyInit_memory(void)
#if CYTHON_PEP489_MULTI_PHASE_INIT
{
  return PyModuleDef_Init(&__pyx_moduledef);
}
/* ModuleCreationPEP489 */
#if CYTHON_COMPILING_IN_LIMITED_API && (__PYX_LIMITED_VERSION_HEX < 0x03090000\
      || ((defined(_WIN32) || defined(WIN32) || defined(MS_WINDOWS)) && __PYX_LIMITED_VERSION_HEX < 0x030A0000))
static PY_INT64_T __Pyx_GetCurrentInterpreterId(void) {
    {
        PyObject *module = PyImport_ImportModule("_interpreters"); // 3.13+ I think
        if (!module) {
            PyErr_Clear(); // just try the 3.8-3.12 version
            module = PyImport_ImportModule("_xxsubinterpreters");
            if (!module) goto bad;
        }
        PyObject *current = PyObject_CallMethod(module, "get_current", NULL);
        Py_DECREF(module);
        if (!current) goto bad;
        if (PyTuple_Check(current)) {
            PyObject *new_current = PySequence_GetItem(current, 0);
            Py_DECREF(current);
            current = new_current;
            if (!new_current) goto bad;
        }
        long long as_c_int = PyLong_AsLongLong(current);
        Py_DECREF(current);
        return as_c_int;
    }
  bad:
    PySys_WriteStderr("__Pyx_GetCurrentInterpreterId failed. Try setting the C define CYTHON_PEP489_MULTI_PHASE_INIT=0\n");
    return -1;
}
#endif
#if !CYTHON_USE_MODULE_STATE
static CYTHON_SMALL_CODE int __Pyx_check_single_interpreter(void) {
    static PY_INT64_T main_interpreter_id = -1;
#if CYTHON_COMPILING_IN_GRAAL && defined(GRAALPY_VERSION_NUM) && GRAALPY_VERSION_NUM > 0x19000000
    PY_INT64_T current_id = GraalPyInterpreterState_GetIDFromThreadState(PyThreadState_Get());
#elif CYTHON_COMPILING_IN_GRAAL
    PY_INT64_T current_id = PyInterpreterState_GetIDFromThreadState(PyThreadState_Get());
#elif CYTHON_COMPILING_IN_LIMITED_API && (__PYX_LIMITED_VERSION_HEX < 0x03090000\
      || ((defined(_WIN32) || defined(WIN32) || defined(MS_WINDOWS)) && __PYX_LIMITED_VERSION_HEX < 0x030A0000))
    PY_INT64_T current_id = __Pyx_GetCurrentInterpreterId();
#elif CYTHON_COMPILING_IN_LIMITED_API
    PY_INT64_T current_id = PyInterpreterState_GetID(PyInterpreterState_Get());
#else
    PY_INT64_T current_id = PyInterpreterState_GetID(PyThreadState_Get()->interp);
#endif
    if (unlikely(current_id == -1)) {
        return -1;
    }
    if (main_interpreter_id == -1) {
        main_interpreter_id = current_id;
        return 0;
    } else if (unlikely(main_interpreter_id != current_id)) {
        PyErr_SetString(
            PyExc_ImportError,
            "Interpreter change detected - this module can only be loaded into one interpreter per process.");
        return -1;
    }
    return 0;
}
#endif
static CYTHON_SMALL_CODE int __Pyx_copy_spec_to_module(PyObject *spec, PyObject *moddict, const char* from_name, const char* to_name, int allow_none)
{
    PyObject *value = PyObject_GetAttrString(spec, from_name);
    int result = 0;
    if (likely(value)) {
        if (allow_none || value != Py_None) {
            result = PyDict_SetItemString(moddict, to_name, value);
        }
        Py_DECREF(value);
    } else if (PyErr_ExceptionMatches(PyExc_AttributeError)) {
        PyErr_Clear();
    } else {
        result = -1;
    }
    return result;
}
static CYTHON_SMALL_CODE PyObject* __pyx_pymod_create(PyObject *spec, PyModuleDef *def) {
    PyObject *module = NULL, *moddict, *modname;
    CYTHON_UNUSED_VAR(def);
    #if !CYTHON_USE_MODULE_STATE
    if (__Pyx_check_single_interpreter())
        return NULL;
    #endif
    if (__pyx_m)
        return __Pyx_NewRef(__pyx_m);
    modname = PyObject_GetAttrString(spec, "name");
    if (unlikely(!modname)) goto bad;
    module = PyModule_NewObject(modname);
    Py_DECREF(modname);
    if (unlikely(!module)) goto bad;
    moddict = PyModule_GetDict(module);
    if (unlikely(!moddict)) goto bad;
    if (unlikely(__Pyx_copy_spec_to_module(spec, moddict, "loader", "__loader__", 1) < 0)) goto bad;
    if (unlikely(__Pyx_copy_spec_to_module(spec, moddict, "origin", "__file__", 1) < 0)) goto bad;
    if (unlikely(__Pyx_copy_spec_to_module(spec, moddict, "parent", "__package__", 1) < 0)) goto bad;
    if (unlikely(__Pyx_copy_spec_to_module(spec, moddict, "submodule_search_locations", "__path__", 0) < 0)) goto bad;
    return module;
bad:
    Py_XDECREF(module);
    return NULL;
}


static CYTHON_SMALL_CODE int __pyx_pymod_exec_memory(PyObject *__pyx_pyinit_module)
#endif
{
  int stringtab_initialized = 0;
  #if CYTHON_USE_MODULE_STATE
  int pystate_addmodule_run = 0;
  #endif
  __pyx_mstatetype *__pyx_mstate = NULL;
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  Py_ssize_t __pyx_t_3;
  PyObject *__pyx_t_4 = NULL;
  int __pyx_t_5;
  PyObject *__pyx_t_6 = NULL;
  PyObject *__pyx_t_7 = NULL;
  size_t __pyx_t_8;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannyDeclarations
  #if CYTHON_PEP489_MULTI_PHASE_INIT
  if (__pyx_m) {
    if (__pyx_m == __pyx_pyinit_module) return 0;
    PyErr_SetString(PyExc_RuntimeError, "Module 'memory' has already been imported. Re-initialisation is not supported.");
    return -1;
  }
  #else
  if (__pyx_m) return __Pyx_NewRef(__pyx_m);
  #endif
  /*--- Module creation code ---*/
  #if CYTHON_PEP489_MULTI_PHASE_INIT
  __pyx_t_1 = __pyx_pyinit_module;
  Py_INCREF(__pyx_t_1);
  #else
  __pyx_t_1 = PyModule_Create(&__pyx_moduledef); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1, __pyx_L1_error)
  #endif
  #if CYTHON_USE_MODULE_STATE
  {
    int add_module_result = __Pyx_State_AddModule(__pyx_t_1, &__pyx_moduledef);
    __pyx_t_1 = 0; /* transfer ownership from __pyx_t_1 to "memory" pseudovariable */
    if (unlikely((add_module_result < 0))) __PYX_ERR(0, 1, __pyx_L1_error)
    pystate_addmodule_run = 1;
  }
  #else
  __pyx_m = __pyx_t_1;
  #endif
  #if CYTHON_COMPILING_IN_CPYTHON_FREETHREADING
  PyUnstable_Module_SetGIL(__pyx_m, Py_MOD_GIL_NOT_USED);
  #endif
  __pyx_mstate = __pyx_mstate_global;
  CYTHON_UNUSED_VAR(__pyx_t_1);
  __pyx_mstate->__pyx_d = PyModule_GetDict(__pyx_m); if (unlikely(!__pyx_mstate->__pyx_d)) __PYX_ERR(0, 1, __pyx_L1_error)
  Py_INCREF(__pyx_mstate->__pyx_d);
  __pyx_mstate->__pyx_b = __Pyx_PyImport_AddModuleRef(__Pyx_BUILTIN_MODULE_NAME); if (unlikely(!__pyx_mstate->__pyx_b)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_mstate->__pyx_cython_runtime = __Pyx_PyImport_AddModuleRef("cython_runtime"); if (unlikely(!__pyx_mstate->__pyx_cython_runtime)) __PYX_ERR(0, 1, __pyx_L1_error)
  if (PyObject_SetAttrString(__pyx_m, "__builtins__", __pyx_mstate->__pyx_b) < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  /* ImportRefnannyAPI */
  #if CYTHON_REFNANNY
  __Pyx_RefNanny = __Pyx_RefNannyImportAPI("refnanny");
  if (!__Pyx_RefNanny) {
    PyErr_Clear();
    __Pyx_RefNanny = __Pyx_RefNannyImportAPI("Cython.Runtime.refnanny");
    if (!__Pyx_RefNanny)
        Py_FatalError("failed to import 'refnanny' module");
  }
  #endif
  
__Pyx_RefNannySetupContext("PyInit_memory", 0);
  __Pyx_init_runtime_version();
  if (__Pyx_check_binary_version(__PYX_LIMITED_VERSION_HEX, __Pyx_get_runtime_version(), CYTHON_COMPILING_IN_LIMITED_API) < (0)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_mstate->__pyx_empty_tuple = PyTuple_New(0); if (unlikely(!__pyx_mstate->__pyx_empty_tuple)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_mstate->__pyx_empty_bytes = PyBytes_FromStringAndSize("", 0); if (unlikely(!__pyx_mstate->__pyx_empty_bytes)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_mstate->__pyx_empty_unicode = PyUnicode_FromStringAndSize("", 0); if (unlikely(!__pyx_mstate->__pyx_empty_unicode)) __PYX_ERR(0, 1, __pyx_L1_error)
  /*--- Library function declarations ---*/
  /*--- Initialize various global constants etc. ---*/
  if (__Pyx_InitConstants(__pyx_mstate) < (0)) __PYX_ERR(0, 1, __pyx_L1_error)
  stringtab_initialized = 1;
  if (__Pyx_InitGlobals() < (0)) __PYX_ERR(0, 1, __pyx_L1_error)
  if (__pyx_module_is_main_cupy__cuda__memory) {
    if (PyObject_SetAttr(__pyx_m, __pyx_mstate_global->__pyx_n_u_name, __pyx_mstate_global->__pyx_n_u_main) < (0)) __PYX_ERR(0, 1, __pyx_L1_error)
  }
  {
    PyObject *modules = PyImport_GetModuleDict(); if (unlikely(!modules)) __PYX_ERR(0, 1, __pyx_L1_error)
    if (!PyDict_GetItemString(modules, "cupy.cuda.memory")) {
      if (unlikely((PyDict_SetItemString(modules, "cupy.cuda.memory", __pyx_m) < 0))) __PYX_ERR(0, 1, __pyx_L1_error)
    }
  }
  /*--- Builtin init code ---*/
  if (__Pyx_InitCachedBuiltins(__pyx_mstate) < (0)) __PYX_ERR(0, 1, __pyx_L1_error)
  /*--- Constants init code ---*/
  if (__Pyx_InitCachedConstants(__pyx_mstate) < (0)) __PYX_ERR(0, 1, __pyx_L1_error)
  if (__Pyx_CreateCodeObjects(__pyx_mstate) < (0)) __PYX_ERR(0, 1, __pyx_L1_error)
  /*--- Global type/function init code ---*/
  (void)__Pyx_modinit_global_init_code(__pyx_mstate);
  (void)__Pyx_modinit_variable_export_code(__pyx_mstate);
  if (unlikely((__Pyx_modinit_function_export_code(__pyx_mstate) < 0))) __PYX_ERR(0, 1, __pyx_L1_error)
  if (unlikely((__Pyx_modinit_type_init_code(__pyx_mstate) < 0))) __PYX_ERR(0, 1, __pyx_L1_error)
  if (unlikely((__Pyx_modinit_type_import_code(__pyx_mstate) < 0))) __PYX_ERR(0, 1, __pyx_L1_error)
  if (unlikely((__Pyx_modinit_variable_import_code(__pyx_mstate) < 0))) __PYX_ERR(0, 1, __pyx_L1_error)
  if (unlikely((__Pyx_modinit_function_import_code(__pyx_mstate) < 0))) __PYX_ERR(0, 1, __pyx_L1_error)
  /*--- Execution code ---*/

  /* "cupy/cuda/memory.pyx":7
 * from cython.operator cimport dereference as deref, postincrement
 * 
 * import atexit             # <<<<<<<<<<<<<<
 * import gc
 * import os
*/
  __pyx_t_1 = __Pyx_Import(__pyx_mstate_global->__pyx_n_u_atexit, 0, 0, NULL, 0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 7, __pyx_L1_error)
  __pyx_t_2 = __pyx_t_1;
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_mstate_global->__pyx_d, __pyx_mstate_global->__pyx_n_u_atexit, __pyx_t_2) < (0)) __PYX_ERR(0, 7, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "cupy/cuda/memory.pyx":8
 * 
 * import atexit
 * import gc             # <<<<<<<<<<<<<<
 * import os
 * import threading
*/
  __pyx_t_1 = __Pyx_Import(__pyx_mstate_global->__pyx_n_u_gc, 0, 0, NULL, 0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 8, __pyx_L1_error)
  __pyx_t_2 = __pyx_t_1;
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_mstate_global->__pyx_d, __pyx_mstate_global->__pyx_n_u_gc, __pyx_t_2) < (0)) __PYX_ERR(0, 8, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "cupy/cuda/memory.pyx":9
 * import atexit
 * import gc
 * import os             # <<<<<<<<<<<<<<
 * import threading
 * import warnings
*/
  __pyx_t_1 = __Pyx_Import(__pyx_mstate_global->__pyx_n_u_os, 0, 0, NULL, 0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 9, __pyx_L1_error)
  __pyx_t_2 = __pyx_t_1;
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_mstate_global->__pyx_d, __pyx_mstate_global->__pyx_n_u_os, __pyx_t_2) < (0)) __PYX_ERR(0, 9, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "cupy/cuda/memory.pyx":10
 * import gc
 * import os
 * import threading             # <<<<<<<<<<<<<<
 * import warnings
 * import weakref
*/
  __pyx_t_1 = __Pyx_Import(__pyx_mstate_global->__pyx_n_u_threading, 0, 0, NULL, 0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 10, __pyx_L1_error)
  __pyx_t_2 = __pyx_t_1;
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_mstate_global->__pyx_d, __pyx_mstate_global->__pyx_n_u_threading, __pyx_t_2) < (0)) __PYX_ERR(0, 10, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "cupy/cuda/memory.pyx":11
 * import os
 * import threading
 * import warnings             # <<<<<<<<<<<<<<
 * import weakref
 * 
*/
  __pyx_t_1 = __Pyx_Import(__pyx_mstate_global->__pyx_n_u_warnings, 0, 0, NULL, 0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 11, __pyx_L1_error)
  __pyx_t_2 = __pyx_t_1;
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_mstate_global->__pyx_d, __pyx_mstate_global->__pyx_n_u_warnings, __pyx_t_2) < (0)) __PYX_ERR(0, 11, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "cupy/cuda/memory.pyx":12
 * import threading
 * import warnings
 * import weakref             # <<<<<<<<<<<<<<
 * 
 * from libc.stdint cimport intptr_t, uintptr_t
*/
  __pyx_t_1 = __Pyx_Import(__pyx_mstate_global->__pyx_n_u_weakref, 0, 0, NULL, 0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 12, __pyx_L1_error)
  __pyx_t_2 = __pyx_t_1;
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_mstate_global->__pyx_d, __pyx_mstate_global->__pyx_n_u_weakref, __pyx_t_2) < (0)) __PYX_ERR(0, 12, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "cupy/cuda/memory.pyx":29
 * from cupy_backends.cuda.api cimport runtime
 * 
 * from cupy_backends.cuda.api.runtime import CUDARuntimeError             # <<<<<<<<<<<<<<
 * 
 * 
*/
  {
    PyObject* const __pyx_imported_names[] = {__pyx_mstate_global->__pyx_n_u_CUDARuntimeError};
    __pyx_t_1 = __Pyx_Import(__pyx_mstate_global->__pyx_n_u_cupy_backends_cuda_api_runtime, __pyx_imported_names, 1, NULL, 0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 29, __pyx_L1_error)
  }
  __pyx_t_2 = __pyx_t_1;
  __Pyx_GOTREF(__pyx_t_2);
  {
    PyObject* const __pyx_imported_names[] = {__pyx_mstate_global->__pyx_n_u_CUDARuntimeError};
    __pyx_t_3 = 0; {
      __pyx_t_4 = __Pyx_ImportFrom(__pyx_t_2, __pyx_imported_names[__pyx_t_3]); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 29, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_4);
      if (PyDict_SetItem(__pyx_mstate_global->__pyx_d, __pyx_imported_names[__pyx_t_3], __pyx_t_4) < (0)) __PYX_ERR(0, 29, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    }
  }
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "cupy/cuda/memory.pyx":42
 * cdef const int ALLOCATION_UNIT_SIZE = 512
 * # for test
 * _allocation_unit_size = ALLOCATION_UNIT_SIZE             # <<<<<<<<<<<<<<
 * 
 * 
*/
  __pyx_t_2 = __Pyx_PyLong_From_int(__pyx_v_4cupy_4cuda_6memory_ALLOCATION_UNIT_SIZE); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 42, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_mstate_global->__pyx_d, __pyx_mstate_global->__pyx_n_u_allocation_unit_size, __pyx_t_2) < (0)) __PYX_ERR(0, 42, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "cupy/cuda/memory.pyx":45
 * 
 * 
 * cdef bint _exit_mode = False             # <<<<<<<<<<<<<<
 * 
 * cdef bint _is_ump_enabled = (int(os.environ.get('CUPY_ENABLE_UMP', '0')) != 0)
*/
  __pyx_v_4cupy_4cuda_6memory__exit_mode = 0;

  /* "cupy/cuda/memory.pyx":47
 * cdef bint _exit_mode = False
 * 
 * cdef bint _is_ump_enabled = (int(os.environ.get('CUPY_ENABLE_UMP', '0')) != 0)             # <<<<<<<<<<<<<<
 * 
 * 
*/
  __Pyx_GetModuleGlobalName(__pyx_t_2, __pyx_mstate_global->__pyx_n_u_os); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 47, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_4 = __Pyx_PyObject_GetAttrStr(__pyx_t_2, __pyx_mstate_global->__pyx_n_u_environ); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 47, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_t_4, __pyx_mstate_global->__pyx_n_u_get); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 47, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __pyx_t_4 = __Pyx_PyObject_Call(__pyx_t_2, __pyx_mstate_global->__pyx_tuple[0], NULL); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 47, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_2 = __Pyx_PyNumber_Int(__pyx_t_4); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 47, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __pyx_t_4 = __Pyx_PyLong_NeObjC(__pyx_t_2, __pyx_mstate_global->__pyx_int_0, 0, 0); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 47, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_5 = __Pyx_PyObject_IsTrue(__pyx_t_4); if (unlikely((__pyx_t_5 == (int)-1) && PyErr_Occurred())) __PYX_ERR(0, 47, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __pyx_v_4cupy_4cuda_6memory__is_ump_enabled = __pyx_t_5;

  /* "cupy/cuda/memory.pyx":50
 * 
 * 
 * @atexit.register             # <<<<<<<<<<<<<<
 * def _exit():
 *     _exit_mode = True
*/
  __pyx_t_2 = NULL;
  __Pyx_GetModuleGlobalName(__pyx_t_6, __pyx_mstate_global->__pyx_n_u_atexit); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 50, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_6);
  __pyx_t_7 = __Pyx_PyObject_GetAttrStr(__pyx_t_6, __pyx_mstate_global->__pyx_n_u_register); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 50, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_7);
  __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
  __pyx_t_6 = PyCFunction_NewEx(&__pyx_mdef_4cupy_4cuda_6memory_1_exit, NULL, __pyx_mstate_global->__pyx_n_u_cupy_cuda_memory); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 50, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_6);
  __pyx_t_8 = 1;
  {
    PyObject *__pyx_callargs[2] = {__pyx_t_2, __pyx_t_6};
    __pyx_t_4 = __Pyx_PyObject_FastCall((PyObject*)__pyx_t_7, __pyx_callargs+__pyx_t_8, (2-__pyx_t_8) | (__pyx_t_8*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
    __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
    __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
    if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 50, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
  }
  if (PyDict_SetItem(__pyx_mstate_global->__pyx_d, __pyx_mstate_global->__pyx_n_u_exit, __pyx_t_4) < (0)) __PYX_ERR(0, 50, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

  /* "cupy/cuda/memory.pyx":55
 * 
 * 
 * class OutOfMemoryError(MemoryError):             # <<<<<<<<<<<<<<
 *     """Out-of-memory error.
 * 
*/
  __pyx_t_4 = PyTuple_Pack(1, ((PyObject *)(((PyTypeObject*)PyExc_MemoryError)))); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 55, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_t_7 = __Pyx_PEP560_update_bases(__pyx_t_4); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 55, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_7);
  __pyx_t_6 = __Pyx_CalculateMetaclass(NULL, __pyx_t_7); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 55, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_6);
  __pyx_t_2 = __Pyx_Py3MetaclassPrepare(__pyx_t_6, __pyx_t_7, __pyx_mstate_global->__pyx_n_u_OutOfMemoryError, __pyx_mstate_global->__pyx_n_u_OutOfMemoryError, (PyObject *) NULL, __pyx_mstate_global->__pyx_n_u_cupy_cuda_memory, __pyx_mstate_global->__pyx_kp_u_Out_of_memory_error_Args_size_in); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 55, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (__pyx_t_7 != __pyx_t_4) {
    if (unlikely((PyDict_SetItemString(__pyx_t_2, "__orig_bases__", __pyx_t_4) < 0))) __PYX_ERR(0, 55, __pyx_L1_error)
  }
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

  /* "cupy/cuda/memory.pyx":64
 *     """
 * 
 *     def __init__(self, size, total, limit=0):             # <<<<<<<<<<<<<<
 *         self._size = size
 *         self._total = total
*/
  __pyx_t_4 = __Pyx_CyFunction_New(&__pyx_mdef_4cupy_4cuda_6memory_16OutOfMemoryError_1__init__, 0, __pyx_mstate_global->__pyx_n_u_OutOfMemoryError___init, NULL, __pyx_mstate_global->__pyx_n_u_cupy_cuda_memory, __pyx_mstate_global->__pyx_d, ((PyObject *)__pyx_mstate_global->__pyx_codeobj_tab[1])); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 64, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  #if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x030E0000
  PyUnstable_Object_EnableDeferredRefcount(__pyx_t_4);
  #endif
  __Pyx_CyFunction_SetDefaultsTuple(__pyx_t_4, __pyx_mstate_global->__pyx_tuple[1]);
  if (__Pyx_SetNameInClass(__pyx_t_2, __pyx_mstate_global->__pyx_n_u_init, __pyx_t_4) < (0)) __PYX_ERR(0, 64, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

  /* "cupy/cuda/memory.pyx":80
 *         super(OutOfMemoryError, self).__init__(msg)
 * 
 *     def __reduce__(self):             # <<<<<<<<<<<<<<
 *         return (type(self), (self._size, self._total, self._limit))
 * 
*/
  __pyx_t_4 = __Pyx_CyFunction_New(&__pyx_mdef_4cupy_4cuda_6memory_16OutOfMemoryError_3__reduce__, 0, __pyx_mstate_global->__pyx_n_u_OutOfMemoryError___reduce, NULL, __pyx_mstate_global->__pyx_n_u_cupy_cuda_memory, __pyx_mstate_global->__pyx_d, ((PyObject *)__pyx_mstate_global->__pyx_codeobj_tab[2])); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 80, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  #if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x030E0000
  PyUnstable_Object_EnableDeferredRefcount(__pyx_t_4);
  #endif
  if (__Pyx_SetNameInClass(__pyx_t_2, __pyx_mstate_global->__pyx_n_u_reduce, __pyx_t_4) < (0)) __PYX_ERR(0, 80, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

  /* "cupy/cuda/memory.pyx":55
 * 
 * 
 * class OutOfMemoryError(MemoryError):             # <<<<<<<<<<<<<<
 *     """Out-of-memory error.
 * 
*/
  __pyx_t_4 = __Pyx_Py3ClassCreate(__pyx_t_6, __pyx_mstate_global->__pyx_n_u_OutOfMemoryError, __pyx_t_7, __pyx_t_2, NULL, 0, 0); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 55, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  #if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x030E0000
  PyUnstable_Object_EnableDeferredRefcount(__pyx_t_4);
  #endif
  if (PyDict_SetItem(__pyx_mstate_global->__pyx_d, __pyx_mstate_global->__pyx_n_u_OutOfMemoryError, __pyx_t_4) < (0)) __PYX_ERR(0, 55, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
  __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;

  /* "cupy/cuda/memory.pyx":246
 *             self.ptr = runtime.mallocManaged(size)
 * 
 *     def prefetch(self, stream, *, int device_id=runtime.cudaInvalidDeviceId):             # <<<<<<<<<<<<<<
 *         """(experimental) Prefetch memory.
 * 
*/
  __pyx_mstate_global->__pyx_k_ = __pyx_e_13cupy_backends_4cuda_3api_13_runtime_enum_cudaInvalidDeviceId;

  /* "cupy/cuda/memory.pyx":323
 *         return self
 * 
 *     def prefetch(self, stream, *, int device_id=runtime.cudaInvalidDeviceId):             # <<<<<<<<<<<<<<
 *         """Prefetch memory.
 * 
*/
  __pyx_mstate_global->__pyx_k__2 = __pyx_e_13cupy_backends_4cuda_3api_13_runtime_enum_cudaInvalidDeviceId;

  /* "cupy/cuda/memory.pyx":861
 * 
 * 
 * cdef object _current_allocator = _malloc             # <<<<<<<<<<<<<<
 * cdef object _thread_local = threading.local()
 * 
*/
  __Pyx_GetModuleGlobalName(__pyx_t_7, __pyx_mstate_global->__pyx_n_u_malloc); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 861, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_7);
  __Pyx_XGOTREF(__pyx_v_4cupy_4cuda_6memory__current_allocator);
  __Pyx_DECREF_SET(__pyx_v_4cupy_4cuda_6memory__current_allocator, __pyx_t_7);
  __Pyx_GIVEREF(__pyx_t_7);
  __pyx_t_7 = 0;

  /* "cupy/cuda/memory.pyx":862
 * 
 * cdef object _current_allocator = _malloc
 * cdef object _thread_local = threading.local()             # <<<<<<<<<<<<<<
 * 
 * 
*/
  __pyx_t_6 = NULL;
  __Pyx_GetModuleGlobalName(__pyx_t_2, __pyx_mstate_global->__pyx_n_u_threading); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 862, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_4 = __Pyx_PyObject_GetAttrStr(__pyx_t_2, __pyx_mstate_global->__pyx_n_u_local); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 862, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_8 = 1;
  {
    PyObject *__pyx_callargs[2] = {__pyx_t_6, NULL};
    __pyx_t_7 = __Pyx_PyObject_FastCall((PyObject*)__pyx_t_4, __pyx_callargs+__pyx_t_8, (1-__pyx_t_8) | (__pyx_t_8*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
    __Pyx_XDECREF(__pyx_t_6); __pyx_t_6 = 0;
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 862, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_7);
  }
  __Pyx_XGOTREF(__pyx_v_4cupy_4cuda_6memory__thread_local);
  __Pyx_DECREF_SET(__pyx_v_4cupy_4cuda_6memory__thread_local, __pyx_t_7);
  __Pyx_GIVEREF(__pyx_t_7);
  __pyx_t_7 = 0;

  /* "cupy/cuda/memory.pyx":865
 * 
 * 
 * def _get_thread_local_allocator():             # <<<<<<<<<<<<<<
 *     try:
 *         allocator = _thread_local.allocator
*/
  __pyx_t_7 = PyCFunction_NewEx(&__pyx_mdef_4cupy_4cuda_6memory_11_get_thread_local_allocator, NULL, __pyx_mstate_global->__pyx_n_u_cupy_cuda_memory); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 865, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_7);
  if (PyDict_SetItem(__pyx_mstate_global->__pyx_d, __pyx_mstate_global->__pyx_n_u_get_thread_local_allocator, __pyx_t_7) < (0)) __PYX_ERR(0, 865, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;

  /* "cupy/cuda/memory.pyx":873
 * 
 * 
 * def _set_thread_local_allocator(allocator):             # <<<<<<<<<<<<<<
 *     _thread_local.allocator = allocator
 * 
*/
  __pyx_t_7 = PyCFunction_NewEx(&__pyx_mdef_4cupy_4cuda_6memory_13_set_thread_local_allocator, NULL, __pyx_mstate_global->__pyx_n_u_cupy_cuda_memory); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 873, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_7);
  if (PyDict_SetItem(__pyx_mstate_global->__pyx_d, __pyx_mstate_global->__pyx_n_u_set_thread_local_allocator, __pyx_t_7) < (0)) __PYX_ERR(0, 873, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;

  /* "(tree fragment)":4
 *     int __Pyx_CheckUnpickleChecksum(long, long, long, long, const char*) except -1
 *     int __Pyx_UpdateUnpickledDict(object, object, Py_ssize_t) except -1
 * def __pyx_unpickle_BaseMemory(__pyx_type, long __pyx_checksum, tuple __pyx_state):             # <<<<<<<<<<<<<<
 *     cdef object __pyx_result
 *     __Pyx_CheckUnpickleChecksum(__pyx_checksum, 0x61cadb0, 0xe1d5c2a, 0xeeb5490, b'device_id, ptr, size')
*/
  __pyx_t_7 = PyCFunction_NewEx(&__pyx_mdef_4cupy_4cuda_6memory_25__pyx_unpickle_BaseMemory, NULL, __pyx_mstate_global->__pyx_n_u_cupy_cuda_memory); if (unlikely(!__pyx_t_7)) __PYX_ERR(3, 4, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_7);
  if (PyDict_SetItem(__pyx_mstate_global->__pyx_d, __pyx_mstate_global->__pyx_n_u_pyx_unpickle_BaseMemory, __pyx_t_7) < (0)) __PYX_ERR(3, 4, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;

  /* "(tree fragment)":1
 * cdef extern from *:             # <<<<<<<<<<<<<<
 *     int __Pyx_CheckUnpickleChecksum(long, long, long, long, const char*) except -1
 *     int __Pyx_UpdateUnpickledDict(object, object, Py_ssize_t) except -1
*/
  __pyx_t_7 = PyCFunction_NewEx(&__pyx_mdef_4cupy_4cuda_6memory_27__pyx_unpickle_Memory, NULL, __pyx_mstate_global->__pyx_n_u_cupy_cuda_memory); if (unlikely(!__pyx_t_7)) __PYX_ERR(3, 4, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_7);
  if (PyDict_SetItem(__pyx_mstate_global->__pyx_d, __pyx_mstate_global->__pyx_n_u_pyx_unpickle_Memory, __pyx_t_7) < (0)) __PYX_ERR(3, 4, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;

  /* "(tree fragment)":4
 *     int __Pyx_CheckUnpickleChecksum(long, long, long, long, const char*) except -1
 *     int __Pyx_UpdateUnpickledDict(object, object, Py_ssize_t) except -1
 * def __pyx_unpickle_MemoryAsync(__pyx_type, long __pyx_checksum, tuple __pyx_state):             # <<<<<<<<<<<<<<
 *     cdef object __pyx_result
 *     __Pyx_CheckUnpickleChecksum(__pyx_checksum, 0x0e18a1e, 0xd8681f6, 0xf0c20a1, b'device_id, ptr, size, stream_ref')
*/
  __pyx_t_7 = PyCFunction_NewEx(&__pyx_mdef_4cupy_4cuda_6memory_29__pyx_unpickle_MemoryAsync, NULL, __pyx_mstate_global->__pyx_n_u_cupy_cuda_memory); if (unlikely(!__pyx_t_7)) __PYX_ERR(3, 4, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_7);
  if (PyDict_SetItem(__pyx_mstate_global->__pyx_d, __pyx_mstate_global->__pyx_n_u_pyx_unpickle_MemoryAsync, __pyx_t_7) < (0)) __PYX_ERR(3, 4, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;

  /* "(tree fragment)":1
 * cdef extern from *:             # <<<<<<<<<<<<<<
 *     int __Pyx_CheckUnpickleChecksum(long, long, long, long, const char*) except -1
 *     int __Pyx_UpdateUnpickledDict(object, object, Py_ssize_t) except -1
*/
  __pyx_t_7 = PyCFunction_NewEx(&__pyx_mdef_4cupy_4cuda_6memory_31__pyx_unpickle_UnownedMemory, NULL, __pyx_mstate_global->__pyx_n_u_cupy_cuda_memory); if (unlikely(!__pyx_t_7)) __PYX_ERR(3, 4, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_7);
  if (PyDict_SetItem(__pyx_mstate_global->__pyx_d, __pyx_mstate_global->__pyx_n_u_pyx_unpickle_UnownedMemory, __pyx_t_7) < (0)) __PYX_ERR(3, 4, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;

  /* "(tree fragment)":4
 *     int __Pyx_CheckUnpickleChecksum(long, long, long, long, const char*) except -1
 *     int __Pyx_UpdateUnpickledDict(object, object, Py_ssize_t) except -1
 * def __pyx_unpickle_ManagedMemory(__pyx_type, long __pyx_checksum, tuple __pyx_state):             # <<<<<<<<<<<<<<
 *     cdef object __pyx_result
 *     __Pyx_CheckUnpickleChecksum(__pyx_checksum, 0x61cadb0, 0xe1d5c2a, 0xeeb5490, b'device_id, ptr, size')
*/
  __pyx_t_7 = PyCFunction_NewEx(&__pyx_mdef_4cupy_4cuda_6memory_33__pyx_unpickle_ManagedMemory, NULL, __pyx_mstate_global->__pyx_n_u_cupy_cuda_memory); if (unlikely(!__pyx_t_7)) __PYX_ERR(3, 4, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_7);
  if (PyDict_SetItem(__pyx_mstate_global->__pyx_d, __pyx_mstate_global->__pyx_n_u_pyx_unpickle_ManagedMemory, __pyx_t_7) < (0)) __PYX_ERR(3, 4, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;

  /* "(tree fragment)":1
 * cdef extern from *:             # <<<<<<<<<<<<<<
 *     int __Pyx_CheckUnpickleChecksum(long, long, long, long, const char*) except -1
 *     int __Pyx_UpdateUnpickledDict(object, object, Py_ssize_t) except -1
*/
  __pyx_t_7 = PyCFunction_NewEx(&__pyx_mdef_4cupy_4cuda_6memory_35__pyx_unpickle_SystemMemory, NULL, __pyx_mstate_global->__pyx_n_u_cupy_cuda_memory); if (unlikely(!__pyx_t_7)) __PYX_ERR(3, 4, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_7);
  if (PyDict_SetItem(__pyx_mstate_global->__pyx_d, __pyx_mstate_global->__pyx_n_u_pyx_unpickle_SystemMemory, __pyx_t_7) < (0)) __PYX_ERR(3, 4, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;

  /* "(tree fragment)":4
 *     int __Pyx_CheckUnpickleChecksum(long, long, long, long, const char*) except -1
 *     int __Pyx_UpdateUnpickledDict(object, object, Py_ssize_t) except -1
 * def __pyx_unpickle__Chunk(__pyx_type, long __pyx_checksum, tuple __pyx_state):             # <<<<<<<<<<<<<<
 *     cdef object __pyx_result
 *     __Pyx_CheckUnpickleChecksum(__pyx_checksum, 0x37829b1, 0x2c2d147, 0x0239ab7, b'arena, mem, next, offset, prev, size')
*/
  __pyx_t_7 = PyCFunction_NewEx(&__pyx_mdef_4cupy_4cuda_6memory_37__pyx_unpickle__Chunk, NULL, __pyx_mstate_global->__pyx_n_u_cupy_cuda_memory); if (unlikely(!__pyx_t_7)) __PYX_ERR(3, 4, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_7);
  if (PyDict_SetItem(__pyx_mstate_global->__pyx_d, __pyx_mstate_global->__pyx_n_u_pyx_unpickle__Chunk, __pyx_t_7) < (0)) __PYX_ERR(3, 4, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;

  /* "(tree fragment)":1
 * cdef extern from *:             # <<<<<<<<<<<<<<
 *     int __Pyx_CheckUnpickleChecksum(long, long, long, long, const char*) except -1
 *     int __Pyx_UpdateUnpickledDict(object, object, Py_ssize_t) except -1
*/
  __pyx_t_7 = PyCFunction_NewEx(&__pyx_mdef_4cupy_4cuda_6memory_39__pyx_unpickle_MemoryPointer, NULL, __pyx_mstate_global->__pyx_n_u_cupy_cuda_memory); if (unlikely(!__pyx_t_7)) __PYX_ERR(3, 4, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_7);
  if (PyDict_SetItem(__pyx_mstate_global->__pyx_d, __pyx_mstate_global->__pyx_n_u_pyx_unpickle_MemoryPointer, __pyx_t_7) < (0)) __PYX_ERR(3, 4, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;

  /* "(tree fragment)":4
 *     int __Pyx_CheckUnpickleChecksum(long, long, long, long, const char*) except -1
 *     int __Pyx_UpdateUnpickledDict(object, object, Py_ssize_t) except -1
 * def __pyx_unpickle_PooledMemory(__pyx_type, long __pyx_checksum, tuple __pyx_state):             # <<<<<<<<<<<<<<
 *     cdef object __pyx_result
 *     __Pyx_CheckUnpickleChecksum(__pyx_checksum, 0x43cf7ab, 0xe717094, 0x15db62e, b'chunk, device_id, identity, pool, ptr, size')
*/
  __pyx_t_7 = PyCFunction_NewEx(&__pyx_mdef_4cupy_4cuda_6memory_41__pyx_unpickle_PooledMemory, NULL, __pyx_mstate_global->__pyx_n_u_cupy_cuda_memory); if (unlikely(!__pyx_t_7)) __PYX_ERR(3, 4, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_7);
  if (PyDict_SetItem(__pyx_mstate_global->__pyx_d, __pyx_mstate_global->__pyx_n_u_pyx_unpickle_PooledMemory, __pyx_t_7) < (0)) __PYX_ERR(3, 4, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;

  /* "(tree fragment)":1
 * cdef extern from *:             # <<<<<<<<<<<<<<
 *     int __Pyx_CheckUnpickleChecksum(long, long, long, long, const char*) except -1
 *     int __Pyx_UpdateUnpickledDict(object, object, Py_ssize_t) except -1
*/
  __pyx_t_7 = PyCFunction_NewEx(&__pyx_mdef_4cupy_4cuda_6memory_43__pyx_unpickle_MemoryPool, NULL, __pyx_mstate_global->__pyx_n_u_cupy_cuda_memory); if (unlikely(!__pyx_t_7)) __PYX_ERR(3, 4, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_7);
  if (PyDict_SetItem(__pyx_mstate_global->__pyx_d, __pyx_mstate_global->__pyx_n_u_pyx_unpickle_MemoryPool, __pyx_t_7) < (0)) __PYX_ERR(3, 4, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;

  /* "(tree fragment)":4
 *     int __Pyx_CheckUnpickleChecksum(long, long, long, long, const char*) except -1
 *     int __Pyx_UpdateUnpickledDict(object, object, Py_ssize_t) except -1
 * def __pyx_unpickle_MemoryAsyncPool(__pyx_type, long __pyx_checksum, tuple __pyx_state):             # <<<<<<<<<<<<<<
 *     cdef object __pyx_result
 *     __Pyx_CheckUnpickleChecksum(__pyx_checksum, 0xe511d52, 0x96a91a4, 0xc327f55, b'_pools, memoryAsyncHasStat')
*/
  __pyx_t_7 = PyCFunction_NewEx(&__pyx_mdef_4cupy_4cuda_6memory_45__pyx_unpickle_MemoryAsyncPool, NULL, __pyx_mstate_global->__pyx_n_u_cupy_cuda_memory); if (unlikely(!__pyx_t_7)) __PYX_ERR(3, 4, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_7);
  if (PyDict_SetItem(__pyx_mstate_global->__pyx_d, __pyx_mstate_global->__pyx_n_u_pyx_unpickle_MemoryAsyncPool, __pyx_t_7) < (0)) __PYX_ERR(3, 4, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;

  /* "(tree fragment)":1
 * cdef extern from *:             # <<<<<<<<<<<<<<
 *     int __Pyx_CheckUnpickleChecksum(long, long, long, long, const char*) except -1
 *     int __Pyx_UpdateUnpickledDict(object, object, Py_ssize_t) except -1
*/
  __pyx_t_7 = PyCFunction_NewEx(&__pyx_mdef_4cupy_4cuda_6memory_47__pyx_unpickle_CFunctionAllocatorMemory, NULL, __pyx_mstate_global->__pyx_n_u_cupy_cuda_memory); if (unlikely(!__pyx_t_7)) __PYX_ERR(3, 4, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_7);
  if (PyDict_SetItem(__pyx_mstate_global->__pyx_d, __pyx_mstate_global->__pyx_n_u_pyx_unpickle_CFunctionAllocato, __pyx_t_7) < (0)) __PYX_ERR(3, 4, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;

  /* "(tree fragment)":4
 *     int __Pyx_CheckUnpickleChecksum(long, long, long, long, const char*) except -1
 *     int __Pyx_UpdateUnpickledDict(object, object, Py_ssize_t) except -1
 * def __pyx_unpickle_CFunctionAllocator(__pyx_type, long __pyx_checksum, tuple __pyx_state):             # <<<<<<<<<<<<<<
 *     cdef object __pyx_result
 *     __Pyx_CheckUnpickleChecksum(__pyx_checksum, 0x54c1fda, 0xc66c958, 0xced2061, b'_free_func, _malloc_func, _owner, _param')
*/
  __pyx_t_7 = PyCFunction_NewEx(&__pyx_mdef_4cupy_4cuda_6memory_49__pyx_unpickle_CFunctionAllocator, NULL, __pyx_mstate_global->__pyx_n_u_cupy_cuda_memory); if (unlikely(!__pyx_t_7)) __PYX_ERR(3, 4, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_7);
  if (PyDict_SetItem(__pyx_mstate_global->__pyx_d, __pyx_mstate_global->__pyx_n_u_pyx_unpickle_CFunctionAllocato_2, __pyx_t_7) < (0)) __PYX_ERR(3, 4, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;

  /* "(tree fragment)":1
 * cdef extern from *:             # <<<<<<<<<<<<<<
 *     int __Pyx_CheckUnpickleChecksum(long, long, long, long, const char*) except -1
 *     int __Pyx_UpdateUnpickledDict(object, object, Py_ssize_t) except -1
*/
  __pyx_t_7 = PyCFunction_NewEx(&__pyx_mdef_4cupy_4cuda_6memory_51__pyx_unpickle_PythonFunctionAllocatorMemory, NULL, __pyx_mstate_global->__pyx_n_u_cupy_cuda_memory); if (unlikely(!__pyx_t_7)) __PYX_ERR(3, 4, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_7);
  if (PyDict_SetItem(__pyx_mstate_global->__pyx_d, __pyx_mstate_global->__pyx_n_u_pyx_unpickle_PythonFunctionAll, __pyx_t_7) < (0)) __PYX_ERR(3, 4, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;

  /* "(tree fragment)":4
 *     int __Pyx_CheckUnpickleChecksum(long, long, long, long, const char*) except -1
 *     int __Pyx_UpdateUnpickledDict(object, object, Py_ssize_t) except -1
 * def __pyx_unpickle_PythonFunctionAllocator(__pyx_type, long __pyx_checksum, tuple __pyx_state):             # <<<<<<<<<<<<<<
 *     cdef object __pyx_result
 *     __Pyx_CheckUnpickleChecksum(__pyx_checksum, 0xda2aa50, 0xb921491, 0x048ae08, b'_free_func, _malloc_func')
*/
  __pyx_t_7 = PyCFunction_NewEx(&__pyx_mdef_4cupy_4cuda_6memory_53__pyx_unpickle_PythonFunctionAllocator, NULL, __pyx_mstate_global->__pyx_n_u_cupy_cuda_memory); if (unlikely(!__pyx_t_7)) __PYX_ERR(3, 4, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_7);
  if (PyDict_SetItem(__pyx_mstate_global->__pyx_d, __pyx_mstate_global->__pyx_n_u_pyx_unpickle_PythonFunctionAll_2, __pyx_t_7) < (0)) __PYX_ERR(3, 4, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;

  /* "cupy/cuda/memory.pyx":1
 * # distutils: language = c++             # <<<<<<<<<<<<<<
 * cimport cpython  # NOQA
 * cimport cython  # NOQA
*/
  __pyx_t_7 = __Pyx_PyDict_NewPresized(0); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 1, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_7);
  if (PyDict_SetItem(__pyx_mstate_global->__pyx_d, __pyx_mstate_global->__pyx_n_u_test, __pyx_t_7) < (0)) __PYX_ERR(0, 1, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;

  /*--- Wrapped vars code ---*/

  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_XDECREF(__pyx_t_7);
  if (__pyx_m) {
    if (__pyx_mstate->__pyx_d && stringtab_initialized) {
      __Pyx_AddTraceback("init cupy.cuda.memory", __pyx_clineno, __pyx_lineno, __pyx_filename);
    }
    #if !CYTHON_USE_MODULE_STATE
    Py_CLEAR(__pyx_m);
    #else
    Py_DECREF(__pyx_m);
    if (pystate_addmodule_run) {
      PyObject *tp, *value, *tb;
      PyErr_Fetch(&tp, &value, &tb);
      PyState_RemoveModule(&__pyx_moduledef);
      PyErr_Restore(tp, value, tb);
    }
    #endif
  } else if (!PyErr_Occurred()) {
    PyErr_SetString(PyExc_ImportError, "init cupy.cuda.memory");
  }
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  #if CYTHON_PEP489_MULTI_PHASE_INIT
  return (__pyx_m != NULL) ? 0 : -1;
  #else
  return __pyx_m;
  #endif
}
/* #### Code section: pystring_table ### */
/* #### Code section: cached_builtins ### */

static int __Pyx_InitCachedBuiltins(__pyx_mstatetype *__pyx_mstate) {
  CYTHON_UNUSED_VAR(__pyx_mstate);
  __pyx_builtin_super = __Pyx_GetBuiltinName(__pyx_mstate->__pyx_n_u_super); if (!__pyx_builtin_super) __PYX_ERR(0, 78, __pyx_L1_error)
  __pyx_builtin_id = __Pyx_GetBuiltinName(__pyx_mstate->__pyx_n_u_id); if (!__pyx_builtin_id) __PYX_ERR(0, 405, __pyx_L1_error)
  __pyx_builtin_object = __Pyx_GetBuiltinName(__pyx_mstate->__pyx_n_u_object); if (!__pyx_builtin_object) __PYX_ERR(0, 887, __pyx_L1_error)

  /* Cached unbound methods */
  __pyx_mstate->__pyx_umethod_PyDict_Type_get.type = (PyObject*)&PyDict_Type;
  __pyx_mstate->__pyx_umethod_PyDict_Type_get.method_name = &__pyx_mstate->__pyx_n_u_get;
  __pyx_mstate->__pyx_umethod_PyDict_Type_items.type = (PyObject*)&PyDict_Type;
  __pyx_mstate->__pyx_umethod_PyDict_Type_items.method_name = &__pyx_mstate->__pyx_n_u_items;
  __pyx_mstate->__pyx_umethod_PyDict_Type_keys.type = (PyObject*)&PyDict_Type;
  __pyx_mstate->__pyx_umethod_PyDict_Type_keys.method_name = &__pyx_mstate->__pyx_n_u_keys;
  __pyx_mstate->__pyx_umethod_PyDict_Type_pop.type = (PyObject*)&PyDict_Type;
  __pyx_mstate->__pyx_umethod_PyDict_Type_pop.method_name = &__pyx_mstate->__pyx_n_u_pop;
  __pyx_mstate->__pyx_umethod_PyDict_Type_values.type = (PyObject*)&PyDict_Type;
  __pyx_mstate->__pyx_umethod_PyDict_Type_values.method_name = &__pyx_mstate->__pyx_n_u_values;
  __pyx_mstate->__pyx_umethod_PyList_Type_pop.type = (PyObject*)&PyList_Type;
  __pyx_mstate->__pyx_umethod_PyList_Type_pop.method_name = &__pyx_mstate->__pyx_n_u_pop;
  return 0;
  __pyx_L1_error:;
  return -1;
}
/* #### Code section: cached_constants ### */

static int __Pyx_InitCachedConstants(__pyx_mstatetype *__pyx_mstate) {
  __Pyx_RefNannyDeclarations
  CYTHON_UNUSED_VAR(__pyx_mstate);
  __Pyx_RefNannySetupContext("__Pyx_InitCachedConstants", 0);

  /* "cupy/cuda/memory.pyx":1211
 *     if limit is not None:
 *         if limit.endswith('%'):
 *             fraction = float(limit[:-1]) / 100.0             # <<<<<<<<<<<<<<
 *         else:
 *             size = int(limit)
*/
  __pyx_mstate_global->__pyx_slice[0] = PySlice_New(Py_None, __pyx_mstate_global->__pyx_int_neg_1, Py_None); if (unlikely(!__pyx_mstate_global->__pyx_slice[0])) __PYX_ERR(0, 1211, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_mstate_global->__pyx_slice[0]);
  __Pyx_GIVEREF(__pyx_mstate_global->__pyx_slice[0]);

  /* "cupy/cuda/memory.pyx":47
 * cdef bint _exit_mode = False
 * 
 * cdef bint _is_ump_enabled = (int(os.environ.get('CUPY_ENABLE_UMP', '0')) != 0)             # <<<<<<<<<<<<<<
 * 
 * 
*/
  __pyx_mstate_global->__pyx_tuple[0] = PyTuple_Pack(2, __pyx_mstate_global->__pyx_n_u_CUPY_ENABLE_UMP, __pyx_mstate_global->__pyx_kp_u_0); if (unlikely(!__pyx_mstate_global->__pyx_tuple[0])) __PYX_ERR(0, 47, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_mstate_global->__pyx_tuple[0]);
  __Pyx_GIVEREF(__pyx_mstate_global->__pyx_tuple[0]);

  /* "cupy/cuda/memory.pyx":64
 *     """
 * 
 *     def __init__(self, size, total, limit=0):             # <<<<<<<<<<<<<<
 *         self._size = size
 *         self._total = total
*/
  __pyx_mstate_global->__pyx_tuple[1] = PyTuple_Pack(1, ((PyObject*)__pyx_mstate_global->__pyx_int_0)); if (unlikely(!__pyx_mstate_global->__pyx_tuple[1])) __PYX_ERR(0, 64, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_mstate_global->__pyx_tuple[1]);
  __Pyx_GIVEREF(__pyx_mstate_global->__pyx_tuple[1]);
  #if CYTHON_IMMORTAL_CONSTANTS
  {
    PyObject **table = __pyx_mstate->__pyx_tuple;
    for (Py_ssize_t i=0; i<2; ++i) {
      #if CYTHON_COMPILING_IN_CPYTHON_FREETHREADING
      #if PY_VERSION_HEX < 0x030E0000
      if (_Py_IsOwnedByCurrentThread(table[i]) && Py_REFCNT(table[i]) == 1)
      #else
      if (PyUnstable_Object_IsUniquelyReferenced(table[i]))
      #endif
      {
        Py_SET_REFCNT(table[i], _Py_IMMORTAL_REFCNT_LOCAL);
      }
      #else
      Py_SET_REFCNT(table[i], _Py_IMMORTAL_INITIAL_REFCNT);
      #endif
    }
  }
  #endif
  #if CYTHON_IMMORTAL_CONSTANTS
  {
    PyObject **table = __pyx_mstate->__pyx_slice;
    for (Py_ssize_t i=0; i<1; ++i) {
      #if CYTHON_COMPILING_IN_CPYTHON_FREETHREADING
      #if PY_VERSION_HEX < 0x030E0000
      if (_Py_IsOwnedByCurrentThread(table[i]) && Py_REFCNT(table[i]) == 1)
      #else
      if (PyUnstable_Object_IsUniquelyReferenced(table[i]))
      #endif
      {
        Py_SET_REFCNT(table[i], _Py_IMMORTAL_REFCNT_LOCAL);
      }
      #else
      Py_SET_REFCNT(table[i], _Py_IMMORTAL_INITIAL_REFCNT);
      #endif
    }
  }
  #endif
  __Pyx_RefNannyFinishContext();
  return 0;
  __pyx_L1_error:;
  __Pyx_RefNannyFinishContext();
  return -1;
}
/* #### Code section: init_constants ### */

static int __Pyx_InitConstants(__pyx_mstatetype *__pyx_mstate) {
  CYTHON_UNUSED_VAR(__pyx_mstate);
  {
    const struct { const unsigned int length: 11; } index[] = {{1},{30},{74},{39},{33},{33},{4},{179},{35},{93},{200},{49},{49},{61},{1},{1},{1},{1},{1},{8},{32},{4},{38},{20},{7},{6},{57},{2},{136},{9},{6},{38},{34},{21},{6},{50},{9},{7},{130},{49},{72},{64},{64},{9},{6},{10},{18},{24},{16},{15},{21},{6},{13},{6},{11},{15},{13},{10},{64},{4},{16},{25},{27},{12},{23},{29},{20},{22},{12},{13},{6},{17},{16},{21},{9},{18},{6},{5},{9},{18},{5},{7},{9},{15},{16},{22},{14},{20},{12},{18},{6},{30},{16},{7},{7},{3},{9},{26},{8},{7},{8},{7},{5},{6},{8},{15},{13},{10},{9},{16},{15},{8},{2},{7},{3},{9},{27},{12},{1},{8},{2},{8},{13},{5},{10},{4},{5},{6},{5},{8},{7},{6},{11},{18},{17},{3},{7},{8},{6},{12},{13},{10},{15},{3},{13},{8},{7},{4},{6},{6},{2},{5},{5},{7},{4},{12},{3},{8},{11},{3},{12},{14},{11},{10},{25},{39},{33},{28},{21},{26},{30},{28},{25},{27},{44},{38},{27},{28},{21},{14},{12},{10},{17},{13},{3},{8},{4},{4},{9},{12},{27},{10},{12},{19},{4},{5},{3},{6},{6},{12},{5},{11},{8},{9},{5},{4},{8},{5},{6},{11},{6},{10},{5},{6},{4},{8},{7},{273},{100},{176},{53},{1698},{227},{78},{23},{101},{2}};
    #if (CYTHON_COMPRESS_STRINGS) == 3 && __PYX_LIMITED_VERSION_HEX >= 0x030e0000 /* compression: zstd (2297 bytes) */
const char* const cstring = "(\265/\375`\355\030}G\000j\\\000\021=\020\223\323\001\000\250\005\000\010\010\244\002\005P\013\010\244\002Q\352,!\"S\322\227R\256(\244\265\235D\226\330\346\361\340\331#\213\036F\343\003&BUl\253\300\247g\222jwwO\026\236\004\366\000\360\000\364\000\323\337~|I\371\3633~N\024\344\343\241\243\240,\3039\210\203z\243\252J\200\200\324\261\233/L\361\302G|G\271\247\032\220\336I\243SP\347\257>\332\242\001\231\241?K$\366\267&X\344\236q\273\246\036\013F\367\373\021\302\365xv\205\276R\343kc\206:c\245SG%gY\275B\367\317\315\217NQ\235\335\367\261\317t\365g\034\203[\033\\\355\000\242\303\005\317\210Ga\3633\327\357Z=\376m\376\270\307M\335)\214\241\2578\372\373q\005\303\270\302\002\007\251\203R\234\022\023un7W\270\265\221T\035)\366tP0X\371\256\216RN\327\217P\235cS\221f\320\302\245\364G\37402\205\261@\320\371\204\241\334\234C\247\035\177Y|o\365\355'>\220r\350\225ApCL\337\003.gDKG\230\021B1\025\302\314\211\270i\313\010d\306(\203\266\314\020\221!S\004\212\r\"\315\223\303\355W\221\r(\032\263\270k\312\275\352\357u<\033\365\031a\354\220\246\347KwJ-\367;\207\236)\367\3120\352\257\233\034c\351 \350\236\ns\362\023=\272\317\326b\237\013:\315\210\177Rk!\367.iz\354!\"\326\204<\373\244Y\353\035t\354\255\373\0130\302\331t-f\263q\257\025a\310\217\030j+\\Ql\331\322q5\315\335\307\257\370\220\273'\363P\315\207Tu\017\246\240\036\377\346N\334<\336'\245\374\032Hj\313\257\302\342\373({\212r\313\305\334\023d~\237u\327F\225\2240`\250\344d\\qs\266M\246\277\312\253\265%!M\030!Bj~\350\242T\250+\322\321\177r\304$\215!K\204)/F\317\206\344\031\234\023J\325<W\342\244\017\33117M\323\241s\256\335M9\204\002\343F\253\367\346\344\316\214\005\313\233\337\035h\304\354\355\\\371\354\213\026\303)\253\247&\207\026\367\256\020{\236\270\245\214\322Ws\222\332^\351\361G,\014\251\205\020\344\316\256\037\362\212\315o\266\270\317\317\336/\027\265\240[jy\305*\316\363\255\250\235\342w\313\335Z\374\373\276V[\356\320\257g\020\003\255\365K-\030\333\211\034\313jk5\327\222\223+I\336 \257\370\3571D\305\236V%#{\306\357E\306\036\037Sa\373a""\266\375k\261w#\376u\206\356g\354\310\244\350\346\342vg\026[\276G\311\356Dm)~\257N\370\235Ll\362#^\261\2369\221Q\265\2264\225\264\0341d\2204\302\237\010i\301\236\317\275\032\334\345\213{:\344%yB\234\205\322!\351\250X/LJHFHV\212\347\204\207$e\215\250(\261Y1\262^xR\272G\3404\331\210\244t4\301a\337\022\2335B\005\006\021^Vn\354\215\027/\002^\024\201\016\0265\016\014\210Y\220\303\003\306\004\016\r88hv,P\020s\300\300\216\0059\000\344x\000\023\342\201\216\007\nb\026\330@q\021\325H` f\005faXp|`\240\203\201\201\230\215K\003\027\0175\n\014\330\261.)X\270tj\030x\221\020@G\004\230\206\343\305\r\0077 \274\240\330p\341\262q\t\340\242`@\314\206\306\256f\226#U[\356\267\3314\263g\276m\201\202Y\317\350l\322\334[\021\3544\256b\277\263\210w\030\272\2161\226\244z\367\334g\345J*r\230Z\322\205\234\213p\274\363pFN\213\314&\242P\256Q*\311N\234d\230{\312<\237\264L\234`\347\371\204\363\317\217\353\377\315\007\016\0344dT\263ZsQ\264|C+\343*1\206<\003RM\346\343-\214\256\202\207\265\255\303rP\357z\031\331\354w\325d>B\354W\272kK:\025\255PRy&n\225e\026\330\313\322\330'\37769\233\373\305\362\353\235\1775\313\351\017\331'kg\367\202\"\2502l\3241\245\314\310\214\214$)H\206\003B\020\014\026\220\207\201\232\206{2\2110\0264\316\030f\2101\"\022H0\"\241\210\004\031I\n\232\003hP\313\334\236\341;\n6\273\367\310\261O\214\007\325\3043\236 \014\300Qk\027\321\252\324O\027h\314_{K+\207l\224\223A7\014\245H\020F\323\356\266$\224PQ\t\214\022\325P\214\022\326\360\340]\361\336\014\341\346TX\311\300\256\220Q\247\003\000\032\333\204\004\370j\301>2\334\225\022\3658d\352\240f\027E^1\r\317Dp\365\217\347\227\264\2269\365{_\303`\362\026\336\302\031\033\304\301p\374M\303\212\244\240RUS\215\350)\005\253\365\370\307\264\336eh\242\t\364\\&\300\214\262^;$J\327\311\225\321\024iu\303\024i>\014\214\312\177$\225\2022FG\035\357\031\256\nN\267\"ya\304\264\023\263\007\314gaFv\360\3460\251\213\034\243\200)\273\006\246\020|+^!\252G\023apj\376u\237\216C\342\360\274\014\307?G;H\233\202\201\306v\313$\013\253\237\226AeY\036\021\353\330lh\244""\204^\364[\353q\002r\331:\336%\"^\037\267\006mH\266\331\364\252 [b\034@,\312\037NR\346\233h\3264\273\256,\236\213\203\200\204\250\326\366\3650\375\252\t\373>\343\245\230.\262\213\230\006r\302\347\241*\317\347}\345\220\301\224\332\327\031e\254.I%\004\341gR>\253.:%\234\256\010\364\036\274\007/'\317\335dQ\316\213\027W\216\254\307\025\374&0\333d\010.\315\354\226%n\346\203\376w\016\352|`\377;y\356<Q\027\307L\000/\340qo!\361\347\t~\360\335^Ti\277N\222T\333\201\311,\316\241\232\317\n\304L\213\317w\027XJZ4\301\265\276\177&v%\00196\324M\345\370)\205B\303\377\317\004p\242\344C%JH\200t\026k\214\345W\321L\3400XL\221\366\343\252&\026\364/{\005\003\311_]\200V.\234_\327\332\247\323\036\026ei\031\373\200\212-z\377\221\306\026\374l\034$t\321\2016\310\351\023\231\225\023\304\361w\250l9\350\215\000&\017\241\253F\315\314\002H\312\222@:l\005\277\020(\036\335v\027M\321\260H\344\257\333\375\033\006\026\224\200/\005\213\222\250\205\260|\216v\271gV\225xY\354\360\177K\202\346>kSm{\267\034\337\032w\022\263\364\362\264|\020\327\350\217\315\270\010\177e\366\340Y\276\237h\033\243Hd\304`\026\316\032\201\344\207+\352f\000\023\007\213\330\020^30q\242\370J\212pK\356\310\266\023B\301Gb\3130\215\352\376\014\34048\301\212\225'\305\341`\357\335\020\270\"\020\017\306\230\255(p\343X\311\345aW\352\026[&\2446$`4\271\017\344\n\360\224\005\355\032!T\261\204\005`/-\316\355\005(\337fU\210\260*\260\232V\334\307\223\265y.\020v\305\035\371wA\335\337\351\2562\333\0329\2354\005\"\364\212\300\341\316+\341\020\027#1\273\3540\307pc?\232]8\251\326\033\246X\355\212\007\263\366!\262\203\212\277$b#\253\354\251\216n\371\303N\244X\271\220ao\364\273\344D\020\006\276S\266\345'X\320\326\324\363\000Q3\030-]\324\340\2327\206]\207\251[\231xRN\330D\005\272\026&\207\2002Q\026W\220\373\230\034\367\300\000\253s\354 \034qbX\323D\370O\360\021W\255\326D\232MK\231\225\336\2658\313\\<\211\202\347\330\320\375\272a\234\252\252\340\021U&q\013oY\374\025x\262B\245KK\320\223\004\r\333\301\260?\220]\321(\233r\352\354\017\215\320\226\")2\233\310""\325\244\022\245\271\0021mx<\365$3\371\307n@\000\340_5\365L\306,=J<\236\202\t]o\222?\225\022d\316\257b\320$\243\033\344\325\353:]^\001\261\204L\272/\226\260\331G\224J\222\374\n\"\304g \215\236h\370\233\301\211>ix\376\257Ln\220\375U\000\304~>\364*\225\325\233\017\001\234\210p\246\321G6\032\376\274\225\273\005)pZ\r\255\234\302c\305\203YY\010\355\301D[\310\036\321q\020s\374\324RZL\033\026\226@\332\356\205\275\324\2402\305\361\010\205T\343a\321\013\0264\336\022j\016\312\264\270(%\325\217\266\2218\362T*R\nG\201\306\273\334\254M\371\347.";
    PyObject *data = __Pyx_DecompressString(cstring, 2297, 3);
    if (unlikely(!data)) __PYX_ERR(0, 1, __pyx_L1_error)
    const char* const bytes = __Pyx_PyBytes_AsString(data);
    #if !CYTHON_ASSUME_SAFE_MACROS
    if (likely(bytes)); else { Py_DECREF(data); __PYX_ERR(0, 1, __pyx_L1_error) }
    #endif
    #elif (CYTHON_COMPRESS_STRINGS) == 2 /* compression: bz2 (2504 bytes) */
const char* const cstring = "BZh91AY&SY!\215\233\234\000\001l\177\377f_\374\300f\377\375\367\277\367\377`\377\377\377\372@@@@@@@@@\000@@@\000`\n\235>\343\234blv\204\207l\245*U@\221%i\335\250+\357\206\204jhi455?FD\364\207\245='\250\017Q\352\000\320z\200\r\000\000\000\014\322~\2504\321&\322d\322hF\243 h\006\200\320\000\000\000\000\000\321\220\000\003\236\247\250\000h\000\001\240\000\000\000\003@\000\000\000\001\247\251\352\t5$&!\032\0042OI\262\233Ph\321\243F\203@\000h4\006\215\000\000\000\205M\000\000\000\000\000\r\000\000\000\000\000\000\000\000\t\024\320\215\001\001OM\024\360\223Sd\306\221\032mG\250\365\032\000h\003@\000\000\031\016\363\302\252\357\320RB\273}\276\367{B\367>~\347\301u\370\334wp\312\255\246wXkE(\034\357k\\\030g\364G\310M2\020H\212\0345pIt\277\233\326XC>\023\035C\304\252\020\n\236l\014\223Q\327\004\022p2\305\226\331\021[!6\203\300x\363\206\000\315t\000Pd\002\232\027_d\3769S\235z\325\272{\214\377}\367\224\316\300\202\373e\341!\224\221\364\254\257\326\326f\266\010<M\204\221~\024YA\221U\031\203\345T_J\356\360y'\202\321y{\013d`s\300\0103&(\220g\002\322\r\\i3\007\373N\267\001\260\242\305\301\"\204\301\214]\3638s)\212TN\301\024\002\010h\311c#\205`)H\262\205\261T<\031\231\274\2733\302\355=^\030\345G_\217\373\351\370\376\376f\\\2233^\217N\3421TADY\004\037\312@\335\306\251\206\360\217<K\203\t\t\355\365\355\014X\013\002,5.\352\221JO,\373\361\t\310m`\206j\020\345q\332\333n\266\231\325\24252\331\307\036\303\351V~M4\341\240Y]|f\217\000\313\343\207\276\244B\303\035\232{\227\250*/\235#0+\202\217\025\\\243\324\216\363\311\313\256?\224\364\333\217\033\\\000\212~\307\026\343\255\351B\014&\007\246\346\213\023\327\226H\241@\211\307\314\361\212#Y\204O\007/?\314\277L\223\"FU\327\272\375x%\206z\243@\220N\244\260{\216\310\256\026\350B\255]LM\205\242k\300M[\306\177\2776~\t\372\215\251\2424\352\357{\214\266rR/fbl\213C\034 a\000\260Q\033\354\366E\376y\316\021\006$\331\357C\333,\352[\206\227\3070\266\263#\032\022/\"\031\240D`3\275T\020e\272`\212O\201\2541\230\303\221k\236\211\251\277u\302f\276\375\311Umy2Bci""X\350\345\267\023\206-\275\330\200-\374\033 \304\204\304\014-\301\232r\375\303\274[\r,\331\262\372_Y)\3524A\271[\354B\332\2564\263\225\272\253\203\302\303\300\027\226\371lw\206\232%\344?\272\226\316\344\035\244!\0214\2401\213-Z2\251\nF'\277.LG%\026\325\002\230A.\354\226\232\244\022\312\250\276J\031,b\312\240\234\225\025HN(\242KU\263\370\021\023r\246\274\350I\356&\241%\013\326tX\322\351\010\320\230\213I`nd\357]\202\324\235\\NG[\207M\203b\252\321\271\312F\020\330$\251\224\202\351\326\351$\264\255\330m\2451E\024A\025d\302\341\307\237\221\264c\271\2324\223\254\255\301y\346\250\322|\014\244\266\272\347\177\310\025\222\325t\377\211lv\026\357v/\272\334\025\312a\3532\013\0079H\263\325\353\270j\200\331G2\213\017\304\377\316?\036J\3023[\345\335\237~\364i\006\277:\252\363C\342\221c\330\374e!\257yub\236)v\230\372\211\367\005\000\312\227\024\022[x\203\305\025\355\347\276aV\200z\013\321\371=\343\r\027\217\214\333\016Nb\343\262H2\211\204[\357\221\n\0248\313\330\250\301\2100,\245\231A\213\010vr\307@c\t\274\227E\020\271\331\240\033\035|\273@\007\216SS\273)\316,\014\002\312\221Oh~j\\\353;\034#,[\035;(F\265\0225C\320oL\233\001\302\274\376\371\037q\023\347%.\342a\341\325yY\225C&6\211\rd\300\221\203\224\021\023}\272\010\354\272\342\213\023\035\344\241\311\207\224Y\0367\233/\265|\355LIh\257\203\007(ER\216Xb\035l\023L\227,\255\003\362T\305\203Y\246\304R\030\210\3418a\215\304\346c\273=\253D\365\t\032h\007>\225\006\013\221W@\271\205+KW\323\255\341\327\220`\344\341l\0342\327\332+B\370b.\230x\3558B\243\303ZO\322\2220\254\214p\36649\304\230i|\2155\244\333\221\225\"\206(\n8\365k\036\256+\3532\n\345\"\367\212)\247l\023\254\025\024\233\350\024\211 \260\020X[\177A\322\307>iQ\270e\373\356A\031ZQ\r\030f\316\207\223\301\303#\235;;Q\346+\356^rq\326\314\217\t\274\204U\356y2\356}\035\035\210)\322\312\341\326\303\344\370\002\\\211C\"\030\301Ez;\345Z\325)\335\311\313\207H\254\026,\233/\342^\205\213\013\373\022\353\344\227\300\370\207z\354,\230\265\031\245M\334\361\270\367\307\243\037f\307\024\303\032J""\\\334C\026\230\223\205u\300K\321%\344D\255,\021Fb\345\340y_l(\224+\000\030\272\026\244\3556KZ\240Q\031h\366)\351\205\307&\314\244\250CdP\003\247\246\267q*K\021\351\204\2142\275\370<C\213\321a\033e\302\304\340\2435\251\253\277\277{f\274\216mS\027\234`\316\205\204a\020H\213\024AQ\r\351-\013Z\010\240*\343\265\241Y\005'|I9\262\3358\014\2416\236e\002g\303!%\335\3412a\2654\373\372\365\227\226\234M\274\225\367\034\207&I\304Bg\305WMA|\222 \025wB\313\031S:\tH\302\2052D\240;\266\0339$\271bY\330j\271\013\253\023\325\313\2303c\327=M/\ri\003\2540H\243\002\316\315\2017\363=\273@\233\26153<\353+8\013\031\"{dxZ\2206\220\347J\250c\233f=\211\355\304\332\3429_q\201d\312\026\027\016\213\340\306\213\357\376o\206\017O\372\226\244\275\273\035\033:\353\261\"(\300\026\335\304@Q\317\014\247R\352I'X\224v\261({_THu\017\250\356\024\325\374\337\270\221\315\244\3301\024\025QU~+9HzP\235\273\356\250\027\246\0325\204\306\016k\215\364\332H\035\253\333\365U\222V/Y\r\304\220\202Bi\003\006\225T\004\204(-\221H2\357\363\365rmk%!\341*\263\206\303(\253\023\250\310\005\344a\307c\t8\335\036\031\330\351\212\233\222C\2214\223\224\332\235V.\342\005\002\227T4\371\005X148\031L\343 \261\233\204\270\014\375\374\244py\3101P\320\301\214R\251+\321\270\001o\214\202j\302\2013^\335x\3173\311\240b\246\023\244\367$\227Q.t\240\222\352\306\\\".\013$\253\036}\320w2\374\372\245\216\364Id\216xiJ\005\2473\202qQP\230\204\355\206\263\235\230\244\300\013\366\220$\346\254\263;]\255\001d\025b:j\212(\250\304\026m&\264[-\020\224\361K\361,]~\255\230\005\034K}\311\034\366\345\321\270\332\265B\366z\353I\355\363k\324\202\2413\002\350(\252\326h<;C\244\216.(s\274Y\213\036\003z\010\261H\267\203\004\017\306\2306\025`\202\335\017\344\350\335t\222,\003\345\340\261\213\244\341\327=\001.phE\323\325\036d\305\302W\317\310R\372Te\024\347*\336O\214\246\324?n\2243\236\262\362\374Ke9<`\304\030\225\rt\231\004,V/\014M4a\315\225\261\023s\336\264\363\032\001\2204'\030$\274\230\224\024\367\355\022Q\007\006\306\230\3036\307C\240\263\361A\263\023\010U0[\217wl""\364\246U$,7\3179\2272\204\315\274|H\3340m\243\215\322\312\365\247j\201\216\212\204\014\241\0209@\301\0214e\224>\321\325\335n\241\314\004g\031\270Y\234\324hj\330!\251QX\242\254\026#\005\021\213\007J\327\006!\326D\324K\032\r\224\264\327\211<\240\222(\306\307e#\275\344K\301u\222K\030[6\3323\3251\305\240\240\301\315\212-\031h\311$GE\314\007\244\213\241\325RX\354[\211H&\356\002\365\275\227\204\364\300\301d6\201\233\024\2141\271+\033ip\005:ha\337-\2131\013 J1&\221\340I\323\321X\026X\252rJ\201\031\352c\213\343\377\261\\Q\340Fb6\305$\251\350 \236\275\271\212\262\017\256\006^U\363d\222\323\327\252\275-@\234\217\002!\004W\177\353\247\262}\022\251\206\365\310a9\227n2\226O\222\345\235bhE\320q=\242\277z\265\255\212\212OR\356U\220\221\220\356ww6\370\331\026\225Y;\033u8\021\261\021!\023M!\240i\214\307vb\017[R\271e3\212G\247\004s\036\" \"\240@@\223\202\016\027\220T\240b\200 \\\200FP\021s\334=:\357\212Oy\021 |\t\337\222\221\014_\004-S2u\t_\240O\274\177\245q[\037s\213\202xaV\002Bl\006?\213\271\"\234(H\020\306\315\316\000";
    PyObject *data = __Pyx_DecompressString(cstring, 2504, 2);
    if (unlikely(!data)) __PYX_ERR(0, 1, __pyx_L1_error)
    const char* const bytes = __Pyx_PyBytes_AsString(data);
    #if !CYTHON_ASSUME_SAFE_MACROS
    if (likely(bytes)); else { Py_DECREF(data); __PYX_ERR(0, 1, __pyx_L1_error) }
    #endif
    #elif (CYTHON_COMPRESS_STRINGS) != 0 /* compression: zlib (2323 bytes) */
const char* const cstring = "x\332\255XKo\034\271\021\236\000\033\300\330x\023{\023\3009\322@\002Y\233\361\354\303\302n\"X\016fG\262=\301\312\232\325c\261>q9\335\354\031Z\335d\213d\217\3241\004\344\350\243\216{\324\321G\037u\364O\360QG\377\224T\221\354\236\2074\222,\257\240\351&\213\305b\261X\365U\261\277z\370\362\220|u\360r\371\340\220\304|$\"\276\002\204\214g+/\357\352\303G\035&\027,\211\206L\0168\261CN\006\251\352\263\224\2604U\021\263J\023!\215\2109\371\2450B\016hM\377\205DJZ~`I\306$\033p\275\352\204\023\020\036+n\210T\226\230\"\317\225F\016\234E\231)e\364\264\333;\207\201gJ\227\236a\335\211\213\003\215h\276W\010\r\374\233\033\235\214,\265\036\374\343\231\222\374\231\262\250.\263\244S\332\241\222D\030\330]*\372\\3\313\323\222\030\253Ed\271F&Izk\275\373K\377\\\"L\306 \360\005\217\254\201\265\373Q\312\214\001\321*!\375B\244VHb\313\234\233\026\351&\244T\005\221\034\024\261\212\344\30079\001\014%\211\341\326Yl\201I\330\013\263BI\n\323\301J\013$\006\225#+F\034g?f\251\341\255\215\302\342Ba[\301\216\300L^.7\017I\277\264\334\\\201\205\334\013dP\314(\2220\275<1\332$\251\310\204\365\252\251\311\221E\\\377\276J\356\007\341\\k\245[\237~J\340\257\255\007f\331\265\360\317\210\377rrOH\273\270L\266\260=\241Q_\025(\230\3649\251\325h\3253-X!\2353\325\024Q\304\215I\2124-\311\354\026\306\"\274\372AD;\030\000\216\327\321=\3336\230<\326`ZM\340g\204?|\360\322\"ID$\270\264$Qx\356@\335+\270.\267\261\225\0242\262\201w\302\363@\0058\364u\247c\033\275\257\247T\272#\325\276\344\361\372\214\003\362\203<\205\005l\010#\322]u\0131\"aS$W\2404\327\255\207\217\376\336\3727\213c*\321E/<\255\305\026\001\007\216\212\230\301R\270pGs`\255T,\371\204\232Q\221\227_\"\353\227\336\240\255\274<\210\205a\375\224s\211\317Ds\216\321I\261\341\303!\007\037t'Dv\014'5C\0374\332u\026\263\234\305\255\001\006\177\234r\222\025\306\342\311.\304<aEj\027\310=oH\036v\274\200B\335\020\354V\245\213M\262\020\025Z\203\305\317c\rC\025\253C\0228ZVY\n\375hb\357\324.\n\343\267\0227\035>\005\317\361.\221h\346\317O\371\030\321\010X`\314\303).\347\273g8p_\271V#""\2041\000\003<\\\r\241}`W\244\252wD\251\346q\021qJI\\\270\260\225J\336\007\024\031\tpjJ#!\205\245\0246\222$\020^+M\020\311G+\206\247I\2132\330)\243Y\001x\330\364\024!ia8\365Q\351I.<\346Q\250\337@\344\260\004O\001\320\025\334\333z\374a\244\347aN\365\021\274\234\337\345\"\332M\001\034\334\236\021\330j\023\215\205\230\234G\"\021 \004\034\r`\223X\221q<\245\352p\000&9\313\320]\"\226\333B\203\274&z*\206\302P+\251\nC\332\275.\214\246\251!\260M\202N\007>\275\317\343\253\310y\372\315*\261p\022&\201P\275\306\374\325o\236\316\235\377\260\220\273\030\251\217h\033\315\377=3\334\207l\347q\010\366v\225\254\316R\002\347\316j{\263\220h\2255\004\304\316N\3579]{\326\376\376\2075\272\263\336s\335'\275\035\272\276\266\276\261\371\234\376\320]\357n\323\316\020\026\016Y\312\213\231|:\024\231\001\024\337\355y\277\257:c\262J[\224\003~iN1X\014\205\323\004o\0043H\352\303\311\321[\017Q\367\324<j\r8xo\2561\021\002\252o$^\220\333\301l\277E\251w\334s\006*\207G\r\252\255x?;c\2569d?\007$\224\007\360[\205\214K\237A\\m\362d\013N0\345\276&\030\357s\253\004\314\311|\177\neY<\202\350\367eB\256\214\205p\305t\021\010\232\207~U~`\252-p[\350\374uI\342\312\007\241Z\221\322\200\001Br\003\350w l\204\007\006\021\214\311\033_0\202\361\t~\025\361>\213v\243T\031\036\2514\205\320\212T^\002\204\252\254n\370\252d\334\365'2\333\237\345\032\302&\246{\023\034V\215\307C;\214\272\004\200XOQ1.c\323B\214l\261\\\264\264wT\034\365D\017}!\204\002\216\2016A!\021\207F\310!~\005oAJc<+\370\213\261\203\353\354\013,jF\002\202\236\242\321\000b2f+L\231I\036S\311\306u\034\210\271\026fZ\327\2308G\337\037\037\243c\242t\020\005O\036p\013\377\036\003)\266\354\020,\021S\347\360\343\232\223\342\230\201R\013\234V\300?\246Y\330g\345\342T\030Z\237\275\000G3\360\320#\226\026\334\354\362\322x\361\356\351\005\323\214\201\037\340\013W\230|:\005\2633\376\230\315:$\034\001\374\323\334j|\2417\302\033\322\203\177\006\223S\300\\\026\274\017:*.R\356ZZ\201\351!\305\000\374\323\314\014$\365\246t&\246T\262\314\261I\276\217\217\003\353\301\337\347\037e""\\\032\313\231fY\216k\213\0301\002\177\324\347s\223\253\034\024M8\370?u*\003|RT\025\036\020\257\000\264\002\305\273\366\220\303\222E\346{\301\302\330\304r\330\267\n\351\022\016\247c\234\235\031\350\314\205\207K\330f\030\246\240uvl>\261\355m=o\304\325\030\347\215\006X\2363vf\322$X\316\016]\014\222W\341\235\341\232\304\313\231\241)\370\234\031\363)\312\023G\026K*<\350\275\202\245\225OU\320_\267\"\247\317\004\201\037`\023\034r @\007\215\005\213A\234\250\303\224b3\3103sC\026F\00229\256\340Z\023\355ja\214\035\3670:\302\221\302\370\232\300?\301\275!R\000\311@\223P\232 ?\005\320\001\301~i\3108\320P\373\324\002$\340\017\342\305WX\3569Ql\025y\014KC}\026\373\276\203\010\217\023\373LK\374aa\265\317\331.\230\240Wn\370\252\353\013\274\227\220\020#\273\"\007\02059\203\360Zl\\\316\323\304\032\247\210\2521\225\003:\300\275\213.9\264_B@\247\337\206{\260\203\216\372\002\376E=\003<\005m\013\323\026\033\323\322\372/\316\0254\345\341\240\333X\315&\231\273\227\301\344\352\215)]\032\256u\025\203\270Z\377\302%\202\212C\245vM\0030\313LQ~#\203\006\357\301\365B\236\244\241\324<\337\252\260\026\002$\350>R\"\366\212NO<\207\204\230z\025\233\340L\374\316\020\323pq\375p\317\272\340\330\316at[\271\362\224\313\270\347\2627\311\225\374j\376\364kLqp\361\321S\257k\240\217\2347\241\301o'\344Z:]\317\214\263K\177\354\374\017P\335\207\345\305\350\322\250\307/\323\256\342\273L\2011,\\,o\202\361R\313^\231u\036\320}\375`\352\242\020 \357\001\226u\337\205\273B\250l\253O\252\227a\336\007D\306\231\364s\025mB\"j[\250y\373\005~\315\274\022\360@^x\302mW&\n\263\221\277S6\374\345f\213\333\360\375\250\201\265s\303\003\362\326\270D\300\311\356\346\311m\275lE{2C\003\341\341\345*F'\321\267\200\032\345ex\325$\224\261\255E\266\255\260\327vWXG\017U\267g\254\256\366\341sy\303\177\273\204\265\177\362I\000\323B\330S\335\352(\260X\330\342\224\226~\177]\323\251\276\222\214\231V}\231U\231\243\246W\004\357\n\215)\217\010=\257h\370.7\271\236\001J#\374\341\355j\010\347\022.\211\031\2575\254\331;*\313\361\305r\326\027\251\260\345:{""\001E\303\245\\B\002\227\373\"\335\361F\352\232U(\354\264*AEG\357J(\317D\274\311\215*t\304\237\272\033\316\324\320OX\276yJ\250\372\353O\005\236\332\343\340y\356#t;\305r\261\\\363\037\033/\306\222)$\361)\274\276_\343\321\272\342:\007\321\2249\331\377k\237\336\270u\364\311Q\373\350\347_\267\217\377\372\372/\257\237\277\211O\356\276\373\344o\3070r\363U\367\350G|\375\347\210\205\327\373\033\177x\365\355\321\237\217\332\2477?\177\367\371wo\366O\330\311\336\351\315?\275\032\035m\377z\373\364\326\235ww\376ur\347\355\357\336\336~\177\363\263\323[\267\3377\037\274\371\372\315\343\223\205\267\267Oo|\366\352\307w\177$\307\277?~\376\232\275\336;\276\373\177\004\342\342\366";
    PyObject *data = __Pyx_DecompressString(cstring, 2323, 1);
    if (unlikely(!data)) __PYX_ERR(0, 1, __pyx_L1_error)
    const char* const bytes = __Pyx_PyBytes_AsString(data);
    #if !CYTHON_ASSUME_SAFE_MACROS
    if (likely(bytes)); else { Py_DECREF(data); __PYX_ERR(0, 1, __pyx_L1_error) }
    #endif
    #else /* compression: none (6637 bytes) */
const char* const bytes = "0<{} 0x{:x} device={} mem={!r}>Can't change the global allocator inside `using_allocator` context managerDevice {} does not support malloc_asyncHIP does not support memory_asyncManaged memory requires ROCm 4.3+NoneNote that Cython is deliberately stricter than PEP-484 and rejects subclasses of builtin types. If you need to pass subclasses then set the 'annotation_typing' directive to False.Out of memory allocating {:,} bytesOut of memory allocating {:,} bytes (allocated so far: {:,} bytes, limit set to: {:,} bytes).Out-of-memory error.\n\n    Args:\n        size (int): Size of memory about to be allocated.\n        total (int): Size of memory successfully allocated so far.\n        limit (int): Allocation limit.\n    The driver version is insufficient for this queryThis function is not supported in MemoryAsyncPoolUnownedMemory requires explicit device ID for a null pointer.<>%.?add_note (allocated so far: {:,} bytes). at cudaMemPoolCreate is not yet supportedcupy/cuda/memory.pyxdisableenablefree_all_free is deprecated. Use free_all_blocks instead.gchandle must be 'default' (for the device's default pool), 'current' (for the device's current pool), or int (a pointer to cudaMemPool_t)isenabled, mem=memory limit fraction out of range: {}memory limit size out of range: {}must provide an owner next=no default __reduce__ due to non-trivial __cinit__, offset=, prev=self._arena_mutex,self._in_use_bytes,self._total_bytes,self._total_bytes_limit cannot be converted to a Python object for picklingsize and fraction cannot be specified at one timethe current stream is capturing, so synchronous API calls are disallowedthe current stream is capturing, so H2D transfers are disallowedthe current stream is capturing, so D2H transfers are disallowed<unknown>_ArenaBaseMemoryCFunctionAllocatorCFunctionAllocatorMemoryCUDARuntimeErrorCUPY_ENABLE_UMPCUPY_GPU_MEMORY_LIMIT_ChunkManagedMemoryMemoryMemoryAsyncMemoryAsyncPoolMemoryPointerMemoryPoolMemoryPool._ensure_pools_and_return_device_pool"".<locals>.genexprNoneOutOfMemoryErrorOutOfMemoryError.__init__OutOfMemoryError.__reduce__PooledMemoryPythonFunctionAllocatorPythonFunctionAllocatorMemory__Pyx_PyDict_NextRefSingleDeviceMemoryPoolSystemMemoryUnownedMemoryadvisealloc_postprocessalloc_preprocess_allocation_unit_sizeallocatorasyncio.coroutinesatexitchunk__class__cline_in_tracebackclosecollectcopy_fromcopy_from_asynccopy_from_devicecopy_from_device_asynccopy_from_hostcopy_from_host_asynccopy_to_hostcopy_to_host_asynccreatecupy_backends.cuda.api.runtimecupy.cuda.memorycurrentdefaultdevdevice_iddevice_support_async_alloc__dict____doc__endswithenviron_exitformatfractionfree_all_blocksfree_all_freefree_bytesfree_funcfree_postprocessfree_preprocess__func__gcgenexprgetget_limit_get_thread_local_allocator__getstate__i__iadd__id__init___is_coroutineitemsitervalueskeyslimit_limitlocal__main___mallocmallocmalloc_funcmalloc_postprocessmalloc_preprocessmemmem_ptrmem_sizememsetmemset_async__metaclass____module____mro_entries__msgn_free_blocks__name____new__nextobjectoffsetosownerparampmem_idpoolpool_handlespopprefetch__prepare__ptr__pyx_capi____pyx_checksum__pyx_state__pyx_type__pyx_unpickle_BaseMemory__pyx_unpickle_CFunctionAllocatorMemory__pyx_unpickle_CFunctionAllocator__pyx_unpickle_ManagedMemory__pyx_unpickle_Memory__pyx_unpickle_MemoryAsync__pyx_unpickle_MemoryAsyncPool__pyx_unpickle_MemoryPointer__pyx_unpickle_MemoryPool__pyx_unpickle_PooledMemory__pyx_unpickle_PythonFunctionAllocatorMemory__pyx_unpickle_PythonFunctionAllocator__pyx_unpickle_SystemMemory__pyx_unpickle_UnownedMemory__pyx_unpickle__Chunk__pyx_vtable____qualname____reduce____reduce_cython____reduce_ex__refregisterselfsendset_limit__set_name___set_thread_local_allocatorsetdefault__setstate____setstate_cython__size_sizesrcstatusstreamstream_identsupersynchronize__test__threadingthrow_tid_tid_objtotal_totaltotal_bytesupdateused_bytesvaluevalueswarnwarningsweakrefPyObject *(int __pyx_skip_dispatch)\000PyObject *(int __pyx_skip_dispatch, struct __pyx_""opt_args_4cupy_4cuda_6memory_set_allocator *__pyx_optional_args)\000struct __pyx_obj_4cupy_4cuda_6memory_MemoryPointer *(PyObject *, int __pyx_skip_dispatch)\000get_allocator\000set_allocator\000allocPyObject *(int __pyx_skip_dispatch)\000int (int __pyx_skip_dispatch)\000get_memory_hooks\000_has_memory_hooksPyObject *(int __pyx_skip_dispatch, struct __pyx_opt_args_4cupy_4cuda_6stream_get_current_stream *__pyx_optional_args)\000intptr_t (void)\000get_current_stream\000get_current_stream_ptrPyObject *(int __pyx_skip_dispatch)\000get_build_versionPyObject *(int __pyx_skip_dispatch)\000PyObject *(int, int __pyx_skip_dispatch)\000PyObject *(int, intptr_t, int __pyx_skip_dispatch)\000PyObject *(intptr_t, int __pyx_skip_dispatch)\000\000PyObject *(intptr_t, int, PyObject *, int __pyx_skip_dispatch)\000PyObject *(intptr_t, int, int __pyx_skip_dispatch)\000PyObject *(intptr_t, int, size_t, int __pyx_skip_dispatch)\000PyObject *(intptr_t, int, size_t, intptr_t, int __pyx_skip_dispatch)\000PyObject *(intptr_t, intptr_t, int __pyx_skip_dispatch)\000PyObject *(intptr_t, intptr_t, size_t, int, int __pyx_skip_dispatch)\000PyObject *(intptr_t, intptr_t, size_t, int, intptr_t, int __pyx_skip_dispatch)\000PyObject *(intptr_t, size_t, int __pyx_skip_dispatch)\000PyObject *(intptr_t, size_t, int, int, int __pyx_skip_dispatch)\000PyObject *(intptr_t, size_t, int, intptr_t, int __pyx_skip_dispatch)\000PyObject *(void)\000int (int __pyx_skip_dispatch)\000\000\000int (int, int, int __pyx_skip_dispatch)\000int (intptr_t, int __pyx_skip_dispatch)\000intptr_t (int, int __pyx_skip_dispatch)\000\000intptr_t (size_t, int __pyx_skip_dispatch)\000intptr_t (size_t, int __pyx_skip_dispatch, struct __pyx_opt_args_13cupy_backends_4cuda_3api_7runtime_mallocManaged *__pyx_optional_args)\000intptr_t (size_t, intptr_t, int __pyx_skip_dispatch)\000struct __pyx_obj_13cupy_backends_4cuda_3api_7runtime_PointerAttributes *(intptr_t, int __pyx_skip_dispatch)\000memGetInfo\000setDevice\000deviceSetMemPool\000fre""e\000streamSynchronize\000memPoolSetAttribute\000memPoolGetAttribute\000memset\000memsetAsync\000freeAsync\000memcpy\000memcpyAsync\000memPoolTrimTo\000memAdvise\000memPrefetchAsync\000_ensure_context\000driverGetVersion\000getDevice\000getDeviceCount\000deviceGetAttribute\000streamIsCapturing\000deviceGetDefaultMemPool\000deviceGetMemPool\000malloc\000mallocManaged\000mallocAsync\000pointerGetAttributesint\000\000\000\000\000\000\000\000_is_hip_environment\000deviceAttributeComputeCapabilityMajor\000deviceAttributeComputeCapabilityMinor\000errorContextIsDestroyed\000errorInvalidResourceHandle\000errorInvalidValue\000errorMemoryAllocation\000errorPeerAccessAlreadyEnabledint (int __pyx_skip_dispatch)\000int (int, int)\000get_device_id\000_enable_peer_access\200A\330\010\020\220\004\220A\220X\230T\240\030\250\024\250Y\260d\270!\320\004$\240A\330\010\014\210I\220Q\330\010\014\210J\220a\330\010\014\210J\220a\340\010\013\2106\220\023\220A\330\014\022\320\0227\260w\270a\270q\330\014\017\210v\220T\230\021\330\020\027\320\0279\270\027\300\001\300\021\340\014\r\330\020\021\340,3\2601\260F\270'\300\021\330\010\r\210Q\320\016 \240\005\240Y\250a\250q\240!";
    PyObject *data = NULL;
    CYTHON_UNUSED_VAR(__Pyx_DecompressString);
    #endif
    PyObject **stringtab = __pyx_mstate->__pyx_string_tab;
    Py_ssize_t pos = 0;
    for (int i = 0; i < 217; i++) {
      Py_ssize_t bytes_length = index[i].length;
      PyObject *string = PyUnicode_DecodeUTF8(bytes + pos, bytes_length, NULL);
      if (likely(string) && i >= 44) PyUnicode_InternInPlace(&string);
      if (unlikely(!string)) {
        Py_XDECREF(data);
        __PYX_ERR(0, 1, __pyx_L1_error)
      }
      stringtab[i] = string;
      pos += bytes_length;
    }
    for (int i = 217; i < 227; i++) {
      Py_ssize_t bytes_length = index[i].length;
      PyObject *string = PyBytes_FromStringAndSize(bytes + pos, bytes_length);
      stringtab[i] = string;
      pos += bytes_length;
      if (unlikely(!string)) {
        Py_XDECREF(data);
        __PYX_ERR(0, 1, __pyx_L1_error)
      }
    }
    Py_XDECREF(data);
    for (Py_ssize_t i = 0; i < 227; i++) {
      if (unlikely(PyObject_Hash(stringtab[i]) == -1)) {
        __PYX_ERR(0, 1, __pyx_L1_error)
      }
    }
    #if CYTHON_IMMORTAL_CONSTANTS
    {
      PyObject **table = stringtab + 217;
      for (Py_ssize_t i=0; i<10; ++i) {
        #if CYTHON_COMPILING_IN_CPYTHON_FREETHREADING
        #if PY_VERSION_HEX < 0x030E0000
        if (_Py_IsOwnedByCurrentThread(table[i]) && Py_REFCNT(table[i]) == 1)
        #else
        if (PyUnstable_Object_IsUniquelyReferenced(table[i]))
        #endif
        {
          Py_SET_REFCNT(table[i], _Py_IMMORTAL_REFCNT_LOCAL);
        }
        #else
        Py_SET_REFCNT(table[i], _Py_IMMORTAL_INITIAL_REFCNT);
        #endif
      }
    }
    #endif
  }
  {
    PyObject **numbertab = __pyx_mstate->__pyx_number_tab + 0;
    int8_t const cint_constants_1[] = {0,-1,1};
    int32_t const cint_constants_4[] = {14780958L,40300000L,58206641L,71104427L,88874970L,102542768L,120372003L,127029364L,177953241L,216617047L,228764240L,240196946L,255047627L};
    for (int i = 0; i < 16; i++) {
      numbertab[i] = PyLong_FromLong((i < 3 ? cint_constants_1[i - 0] : cint_constants_4[i - 3]));
      if (unlikely(!numbertab[i])) __PYX_ERR(0, 1, __pyx_L1_error)
    }
  }
  #if CYTHON_IMMORTAL_CONSTANTS
  {
    PyObject **table = __pyx_mstate->__pyx_number_tab;
    for (Py_ssize_t i=0; i<16; ++i) {
      #if CYTHON_COMPILING_IN_CPYTHON_FREETHREADING
      #if PY_VERSION_HEX < 0x030E0000
      if (_Py_IsOwnedByCurrentThread(table[i]) && Py_REFCNT(table[i]) == 1)
      #else
      if (PyUnstable_Object_IsUniquelyReferenced(table[i]))
      #endif
      {
        Py_SET_REFCNT(table[i], _Py_IMMORTAL_REFCNT_LOCAL);
      }
      #else
      Py_SET_REFCNT(table[i], _Py_IMMORTAL_INITIAL_REFCNT);
      #endif
    }
  }
  #endif
  return 0;
  __pyx_L1_error:;
  return -1;
}
/* #### Code section: init_codeobjects ### */
typedef struct {
    unsigned int argcount : 3;
    unsigned int num_posonly_args : 1;
    unsigned int num_kwonly_args : 1;
    unsigned int nlocals : 3;
    unsigned int flags : 10;
    unsigned int first_line : 11;
} __Pyx_PyCode_New_function_description;
/* NewCodeObj.proto */
static PyObject* __Pyx_PyCode_New(
        const __Pyx_PyCode_New_function_description descr,
        PyObject * const *varnames,
        PyObject *filename,
        PyObject *funcname,
        PyObject *line_table,
        PyObject *tuple_dedup_map
);


static int __Pyx_CreateCodeObjects(__pyx_mstatetype *__pyx_mstate) {
  PyObject* tuple_dedup_map = PyDict_New();
  if (unlikely(!tuple_dedup_map)) return -1;
  {
    const __Pyx_PyCode_New_function_description descr = {0, 0, 0, 1, (unsigned int)(CO_OPTIMIZED|CO_NEWLOCALS|CO_GENERATOR), 1603};
    PyObject* const varnames[] = {__pyx_mstate->__pyx_n_u_i};
    __pyx_mstate_global->__pyx_codeobj_tab[0] = __Pyx_PyCode_New(descr, varnames, __pyx_mstate->__pyx_kp_u_cupy_cuda_memory_pyx, __pyx_mstate->__pyx_n_u_genexpr, __pyx_mstate->__pyx_kp_b_iso88591__8, tuple_dedup_map); if (unlikely(!__pyx_mstate_global->__pyx_codeobj_tab[0])) goto bad;
  }
  {
    const __Pyx_PyCode_New_function_description descr = {4, 0, 0, 5, (unsigned int)(CO_OPTIMIZED|CO_NEWLOCALS), 64};
    PyObject* const varnames[] = {__pyx_mstate->__pyx_n_u_self, __pyx_mstate->__pyx_n_u_size, __pyx_mstate->__pyx_n_u_total, __pyx_mstate->__pyx_n_u_limit, __pyx_mstate->__pyx_n_u_msg};
    __pyx_mstate_global->__pyx_codeobj_tab[1] = __Pyx_PyCode_New(descr, varnames, __pyx_mstate->__pyx_kp_u_cupy_cuda_memory_pyx, __pyx_mstate->__pyx_n_u_init, __pyx_mstate->__pyx_kp_b_iso88591_A_IQ_Ja_Ja_6_A_7waq_vT_9_31F_Q, tuple_dedup_map); if (unlikely(!__pyx_mstate_global->__pyx_codeobj_tab[1])) goto bad;
  }
  {
    const __Pyx_PyCode_New_function_description descr = {1, 0, 0, 1, (unsigned int)(CO_OPTIMIZED|CO_NEWLOCALS), 80};
    PyObject* const varnames[] = {__pyx_mstate->__pyx_n_u_self};
    __pyx_mstate_global->__pyx_codeobj_tab[2] = __Pyx_PyCode_New(descr, varnames, __pyx_mstate->__pyx_kp_u_cupy_cuda_memory_pyx, __pyx_mstate->__pyx_n_u_reduce, __pyx_mstate->__pyx_kp_b_iso88591_A_AXT_Yd, tuple_dedup_map); if (unlikely(!__pyx_mstate_global->__pyx_codeobj_tab[2])) goto bad;
  }
  Py_DECREF(tuple_dedup_map);
  return 0;
  bad:
  Py_DECREF(tuple_dedup_map);
  return -1;
}
/* #### Code section: init_globals ### */

static int __Pyx_InitGlobals(void) {
  /* PythonCompatibility.init */
  if (likely(__Pyx_init_co_variables() == 0)); else
  
  if (unlikely(PyErr_Occurred())) __PYX_ERR(0, 1, __pyx_L1_error)

  /* AssertionsEnabled.init */
  if (likely(__Pyx_init_assertions_enabled() == 0)); else
  
  if (unlikely(PyErr_Occurred())) __PYX_ERR(0, 1, __pyx_L1_error)

  /* CommonTypesMetaclass.init */
  if (likely(__pyx_CommonTypesMetaclass_init(__pyx_m) == 0)); else
  
  if (unlikely(PyErr_Occurred())) __PYX_ERR(0, 1, __pyx_L1_error)

  /* CachedMethodType.init */
  #if CYTHON_COMPILING_IN_LIMITED_API
  {
      PyObject *typesModule=NULL;
      typesModule = PyImport_ImportModule("types");
      if (typesModule) {
          __pyx_mstate_global->__Pyx_CachedMethodType = PyObject_GetAttrString(typesModule, "MethodType");
          Py_DECREF(typesModule);
      }
  } // error handling follows
  #endif
  
  if (unlikely(PyErr_Occurred())) __PYX_ERR(0, 1, __pyx_L1_error)

  /* CythonFunctionShared.init */
  if (likely(__pyx_CyFunction_init(__pyx_m) == 0)); else
  
  if (unlikely(PyErr_Occurred())) __PYX_ERR(0, 1, __pyx_L1_error)

  /* Generator.init */
  if (likely(__pyx_Generator_init(__pyx_m) == 0)); else
  
  if (unlikely(PyErr_Occurred())) __PYX_ERR(0, 1, __pyx_L1_error)

  return 0;
  __pyx_L1_error:;
  return -1;
}
/* #### Code section: cleanup_globals ### */
/* #### Code section: cleanup_module ### */
/* #### Code section: main_method ### */
/* #### Code section: utility_code_pragmas ### */
#ifdef _MSC_VER
#pragma warning( push )
/* Warning 4127: conditional expression is constant
 * Cython uses constant conditional expressions to allow in inline functions to be optimized at
 * compile-time, so this warning is not useful
 */
#pragma warning( disable : 4127 )
#endif



/* #### Code section: utility_code_def ### */

/* --- Runtime support code --- */
/* Refnanny */
#if CYTHON_REFNANNY
static __Pyx_RefNannyAPIStruct *__Pyx_RefNannyImportAPI(const char *modname) {
    PyObject *m = NULL, *p = NULL;
    void *r = NULL;
    m = PyImport_ImportModule(modname);
    if (!m) goto end;
    p = PyObject_GetAttrString(m, "RefNannyAPI");
    if (!p) goto end;
    r = PyLong_AsVoidPtr(p);
end:
    Py_XDECREF(p);
    Py_XDECREF(m);
    return (__Pyx_RefNannyAPIStruct *)r;
}
#endif

/* PyErrExceptionMatches (used by PyObjectGetAttrStrNoError) */
#if CYTHON_FAST_THREAD_STATE
static int __Pyx_PyErr_ExceptionMatchesTuple(PyObject *exc_type, PyObject *tuple) {
    Py_ssize_t i, n;
    n = PyTuple_GET_SIZE(tuple);
    for (i=0; i<n; i++) {
        if (exc_type == PyTuple_GET_ITEM(tuple, i)) return 1;
    }
    for (i=0; i<n; i++) {
        if (__Pyx_PyErr_GivenExceptionMatches(exc_type, PyTuple_GET_ITEM(tuple, i))) return 1;
    }
    return 0;
}
static CYTHON_INLINE int __Pyx_PyErr_ExceptionMatchesInState(PyThreadState* tstate, PyObject* err) {
    int result;
    PyObject *exc_type;
#if PY_VERSION_HEX >= 0x030C00A6
    PyObject *current_exception = tstate->current_exception;
    if (unlikely(!current_exception)) return 0;
    exc_type = (PyObject*) Py_TYPE(current_exception);
    if (exc_type == err) return 1;
#else
    exc_type = tstate->curexc_type;
    if (exc_type == err) return 1;
    if (unlikely(!exc_type)) return 0;
#endif
    #if CYTHON_AVOID_BORROWED_REFS
    Py_INCREF(exc_type);
    #endif
    if (unlikely(PyTuple_Check(err))) {
        result = __Pyx_PyErr_ExceptionMatchesTuple(exc_type, err);
    } else {
        result = __Pyx_PyErr_GivenExceptionMatches(exc_type, err);
    }
    #if CYTHON_AVOID_BORROWED_REFS
    Py_DECREF(exc_type);
    #endif
    return result;
}
#endif

/* PyErrFetchRestore (used by PyObjectGetAttrStrNoError) */
#if CYTHON_FAST_THREAD_STATE
static CYTHON_INLINE void __Pyx_ErrRestoreInState(PyThreadState *tstate, PyObject *type, PyObject *value, PyObject *tb) {
#if PY_VERSION_HEX >= 0x030C00A6
    PyObject *tmp_value;
    assert(type == NULL || (value != NULL && type == (PyObject*) Py_TYPE(value)));
    if (value) {
        #if CYTHON_COMPILING_IN_CPYTHON
        if (unlikely(((PyBaseExceptionObject*) value)->traceback != tb))
        #endif
            PyException_SetTraceback(value, tb);
    }
    tmp_value = tstate->current_exception;
    tstate->current_exception = value;
    Py_XDECREF(tmp_value);
    Py_XDECREF(type);
    Py_XDECREF(tb);
#else
    PyObject *tmp_type, *tmp_value, *tmp_tb;
    tmp_type = tstate->curexc_type;
    tmp_value = tstate->curexc_value;
    tmp_tb = tstate->curexc_traceback;
    tstate->curexc_type = type;
    tstate->curexc_value = value;
    tstate->curexc_traceback = tb;
    Py_XDECREF(tmp_type);
    Py_XDECREF(tmp_value);
    Py_XDECREF(tmp_tb);
#endif
}
static CYTHON_INLINE void __Pyx_ErrFetchInState(PyThreadState *tstate, PyObject **type, PyObject **value, PyObject **tb) {
#if PY_VERSION_HEX >= 0x030C00A6
    PyObject* exc_value;
    exc_value = tstate->current_exception;
    tstate->current_exception = 0;
    *value = exc_value;
    *type = NULL;
    *tb = NULL;
    if (exc_value) {
        *type = (PyObject*) Py_TYPE(exc_value);
        Py_INCREF(*type);
        #if CYTHON_COMPILING_IN_CPYTHON
        *tb = ((PyBaseExceptionObject*) exc_value)->traceback;
        Py_XINCREF(*tb);
        #else
        *tb = PyException_GetTraceback(exc_value);
        #endif
    }
#else
    *type = tstate->curexc_type;
    *value = tstate->curexc_value;
    *tb = tstate->curexc_traceback;
    tstate->curexc_type = 0;
    tstate->curexc_value = 0;
    tstate->curexc_traceback = 0;
#endif
}
#endif

/* PyObjectGetAttrStr (used by PyObjectGetAttrStrNoError) */
#if CYTHON_USE_TYPE_SLOTS
static CYTHON_INLINE PyObject* __Pyx_PyObject_GetAttrStr(PyObject* obj, PyObject* attr_name) {
    PyTypeObject* tp = Py_TYPE(obj);
    if (likely(tp->tp_getattro))
        return tp->tp_getattro(obj, attr_name);
    return PyObject_GetAttr(obj, attr_name);
}
#endif

/* PyObjectGetAttrStrNoError (used by GetBuiltinName) */
#if __PYX_LIMITED_VERSION_HEX < 0x030d0000
static void __Pyx_PyObject_GetAttrStr_ClearAttributeError(void) {
    __Pyx_PyThreadState_declare
    __Pyx_PyThreadState_assign
    if (likely(__Pyx_PyErr_ExceptionMatches(PyExc_AttributeError)))
        __Pyx_PyErr_Clear();
}
#endif
static CYTHON_INLINE PyObject* __Pyx_PyObject_GetAttrStrNoError(PyObject* obj, PyObject* attr_name) {
    PyObject *result;
#if __PYX_LIMITED_VERSION_HEX >= 0x030d0000
    (void) PyObject_GetOptionalAttr(obj, attr_name, &result);
    return result;
#else
#if CYTHON_COMPILING_IN_CPYTHON && CYTHON_USE_TYPE_SLOTS
    PyTypeObject* tp = Py_TYPE(obj);
    if (likely(tp->tp_getattro == PyObject_GenericGetAttr)) {
        return _PyObject_GenericGetAttrWithDict(obj, attr_name, NULL, 1);
    }
#endif
    result = __Pyx_PyObject_GetAttrStr(obj, attr_name);
    if (unlikely(!result)) {
        __Pyx_PyObject_GetAttrStr_ClearAttributeError();
    }
    return result;
#endif
}

/* GetBuiltinName */
static PyObject *__Pyx_GetBuiltinName(PyObject *name) {
    PyObject* result = __Pyx_PyObject_GetAttrStrNoError(__pyx_mstate_global->__pyx_b, name);
    if (unlikely(!result) && !PyErr_Occurred()) {
        PyErr_Format(PyExc_NameError,
            "name '%U' is not defined", name);
    }
    return result;
}

/* GetItemInt */
static PyObject *__Pyx_GetItemInt_Generic(PyObject *o, PyObject* j) {
    PyObject *r;
    if (unlikely(!j)) return NULL;
    r = PyObject_GetItem(o, j);
    Py_DECREF(j);
    return r;
}
static CYTHON_INLINE PyObject *__Pyx_GetItemInt_List_Fast(PyObject *o, Py_ssize_t i,
                                                              int wraparound, int boundscheck, int unsafe_shared) {
    CYTHON_MAYBE_UNUSED_VAR(unsafe_shared);
#if CYTHON_ASSUME_SAFE_SIZE
    Py_ssize_t wrapped_i = i;
    if (wraparound & unlikely(i < 0)) {
        wrapped_i += PyList_GET_SIZE(o);
    }
    if ((CYTHON_AVOID_BORROWED_REFS || CYTHON_AVOID_THREAD_UNSAFE_BORROWED_REFS || !CYTHON_ASSUME_SAFE_MACROS)) {
        return __Pyx_PyList_GetItemRefFast(o, wrapped_i, unsafe_shared);
    } else
    if ((!boundscheck) || likely(__Pyx_is_valid_index(wrapped_i, PyList_GET_SIZE(o)))) {
        return __Pyx_NewRef(PyList_GET_ITEM(o, wrapped_i));
    }
    return __Pyx_GetItemInt_Generic(o, PyLong_FromSsize_t(i));
#else
    (void)wraparound;
    (void)boundscheck;
    return PySequence_GetItem(o, i);
#endif
}
static CYTHON_INLINE PyObject *__Pyx_GetItemInt_Tuple_Fast(PyObject *o, Py_ssize_t i,
                                                              int wraparound, int boundscheck, int unsafe_shared) {
    CYTHON_MAYBE_UNUSED_VAR(unsafe_shared);
#if CYTHON_ASSUME_SAFE_SIZE && CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
    Py_ssize_t wrapped_i = i;
    if (wraparound & unlikely(i < 0)) {
        wrapped_i += PyTuple_GET_SIZE(o);
    }
    if ((!boundscheck) || likely(__Pyx_is_valid_index(wrapped_i, PyTuple_GET_SIZE(o)))) {
        return __Pyx_NewRef(PyTuple_GET_ITEM(o, wrapped_i));
    }
    return __Pyx_GetItemInt_Generic(o, PyLong_FromSsize_t(i));
#else
    (void)wraparound;
    (void)boundscheck;
    return PySequence_GetItem(o, i);
#endif
}
static CYTHON_INLINE PyObject *__Pyx_GetItemInt_Fast(PyObject *o, Py_ssize_t i, int is_list,
                                                     int wraparound, int boundscheck, int unsafe_shared) {
    CYTHON_MAYBE_UNUSED_VAR(unsafe_shared);
#if CYTHON_ASSUME_SAFE_MACROS && CYTHON_ASSUME_SAFE_SIZE
    if (is_list || PyList_CheckExact(o)) {
        Py_ssize_t n = ((!wraparound) | likely(i >= 0)) ? i : i + PyList_GET_SIZE(o);
        if ((CYTHON_AVOID_BORROWED_REFS || CYTHON_AVOID_THREAD_UNSAFE_BORROWED_REFS)) {
            return __Pyx_PyList_GetItemRefFast(o, n, unsafe_shared);
        } else if ((!boundscheck) || (likely(__Pyx_is_valid_index(n, PyList_GET_SIZE(o))))) {
            return __Pyx_NewRef(PyList_GET_ITEM(o, n));
        }
    } else
    #if !CYTHON_AVOID_BORROWED_REFS
    if (PyTuple_CheckExact(o)) {
        Py_ssize_t n = ((!wraparound) | likely(i >= 0)) ? i : i + PyTuple_GET_SIZE(o);
        if ((!boundscheck) || likely(__Pyx_is_valid_index(n, PyTuple_GET_SIZE(o)))) {
            return __Pyx_NewRef(PyTuple_GET_ITEM(o, n));
        }
    } else
    #endif
#endif
#if CYTHON_USE_TYPE_SLOTS && !CYTHON_COMPILING_IN_PYPY
    {
        PyMappingMethods *mm = Py_TYPE(o)->tp_as_mapping;
        PySequenceMethods *sm = Py_TYPE(o)->tp_as_sequence;
        if (!is_list && mm && mm->mp_subscript) {
            PyObject *r, *key = PyLong_FromSsize_t(i);
            if (unlikely(!key)) return NULL;
            r = mm->mp_subscript(o, key);
            Py_DECREF(key);
            return r;
        }
        if (is_list || likely(sm && sm->sq_item)) {
            if (wraparound && unlikely(i < 0) && likely(sm->sq_length)) {
                Py_ssize_t l = sm->sq_length(o);
                if (likely(l >= 0)) {
                    i += l;
                } else {
                    if (!PyErr_ExceptionMatches(PyExc_OverflowError))
                        return NULL;
                    PyErr_Clear();
                }
            }
            return sm->sq_item(o, i);
        }
    }
#else
    if (is_list || !PyMapping_Check(o)) {
        return PySequence_GetItem(o, i);
    }
#endif
    (void)wraparound;
    (void)boundscheck;
    return __Pyx_GetItemInt_Generic(o, PyLong_FromSsize_t(i));
}

/* ReleaseUnknownGil */
static __Pyx_UnknownThreadState __Pyx_SaveUnknownThread(void) {
#if CYTHON_COMPILING_IN_LIMITED_API && __PYX_LIMITED_VERSION_HEX < 0x030d0000
    if (__Pyx_get_runtime_version() >= 0x030d0000) {
        PyThreadState *ts = PyThreadState_Swap(NULL);
        __Pyx_UnknownThreadState out = { ts, PyGILState_UNLOCKED  };
        return out;
    } else {
        PyGILState_STATE gil_state = PyGILState_Ensure();
        PyThreadState *ts = PyEval_SaveThread();
        __Pyx_UnknownThreadState out = { ts, gil_state };
        return out;
    }
#elif __PYX_LIMITED_VERSION_HEX >= 0x030d0000
    return PyThreadState_Swap(NULL);
#else
  #if CYTHON_COMPILING_IN_PYPY || PY_VERSION_HEX < 0x030C0000
    if (PyGILState_Check())
  #else
    if (_PyThreadState_UncheckedGet())  // UncheckedGet is a reliable check for the GIL from 3.12 upwards
  #endif
    {
      return PyEval_SaveThread();
    }
    return NULL; // Nothing to release - we don't have the GIL
#endif
}
static void __Pyx_RestoreUnknownThread(__Pyx_UnknownThreadState state) {
#if CYTHON_COMPILING_IN_LIMITED_API && __PYX_LIMITED_VERSION_HEX < 0x030d0000
    if (!state.ts) return;
    PyEval_RestoreThread(state.ts);
    if (__Pyx_get_runtime_version() < 0x030d0000) {
        PyGILState_Release(state.gil_state);
    }
#else
    if (state) {
        PyEval_RestoreThread(state);
    }
#endif
}
static CYTHON_INLINE int __Pyx_UnknownThreadStateDefinitelyHadGil(__Pyx_UnknownThreadState state) {
  #if CYTHON_COMPILING_IN_LIMITED_API && __PYX_LIMITED_VERSION_HEX < 0x030d0000
  return ((state.ts != NULL) && (__Pyx_get_runtime_version() >= 0x030d0000));
  #else
  return state != NULL;
  #endif
}
static CYTHON_INLINE int __Pyx_UnknownThreadStateMayHaveHadGil(__Pyx_UnknownThreadState state) {
  #if CYTHON_COMPILING_IN_LIMITED_API && __PYX_LIMITED_VERSION_HEX < 0x030d0000
  return state.ts != NULL;
  #else
  return state != NULL;
  #endif
}

/* TupleAndListFromArray (used by fastcall) */
#if !CYTHON_COMPILING_IN_CPYTHON && CYTHON_METH_FASTCALL
static CYTHON_INLINE PyObject *
__Pyx_PyTuple_FromArray(PyObject *const *src, Py_ssize_t n)
{
    PyObject *res;
    Py_ssize_t i;
    if (n <= 0) {
        return __Pyx_NewRef(__pyx_mstate_global->__pyx_empty_tuple);
    }
    res = PyTuple_New(n);
    if (unlikely(res == NULL)) return NULL;
    for (i = 0; i < n; i++) {
        Py_INCREF(src[i]);
        if (unlikely(__Pyx_PyTuple_SET_ITEM(res, i, src[i]) < (0))) {
            Py_DECREF(res);
            return NULL;
        }
    }
    return res;
}
#elif CYTHON_COMPILING_IN_CPYTHON
static CYTHON_INLINE void __Pyx_copy_object_array(PyObject *const *CYTHON_RESTRICT src, PyObject** CYTHON_RESTRICT dest, Py_ssize_t length) {
    PyObject *v;
    Py_ssize_t i;
    for (i = 0; i < length; i++) {
        v = dest[i] = src[i];
        Py_INCREF(v);
    }
}
static CYTHON_INLINE PyObject *
__Pyx_PyTuple_FromArray(PyObject *const *src, Py_ssize_t n)
{
    PyObject *res;
    if (n <= 0) {
        return __Pyx_NewRef(__pyx_mstate_global->__pyx_empty_tuple);
    }
    res = PyTuple_New(n);
    if (unlikely(res == NULL)) return NULL;
    __Pyx_copy_object_array(src, ((PyTupleObject*)res)->ob_item, n);
    return res;
}
static CYTHON_INLINE PyObject *
__Pyx_PyList_FromArray(PyObject *const *src, Py_ssize_t n)
{
    PyObject *res;
    if (n <= 0) {
        return PyList_New(0);
    }
    res = PyList_New(n);
    if (unlikely(res == NULL)) return NULL;
    __Pyx_copy_object_array(src, ((PyListObject*)res)->ob_item, n);
    return res;
}
#endif

/* BytesEquals (used by UnicodeEquals) */
static CYTHON_INLINE int __Pyx_PyBytes_Equals(PyObject* s1, PyObject* s2, int equals) {
#if CYTHON_COMPILING_IN_PYPY || CYTHON_COMPILING_IN_LIMITED_API || CYTHON_COMPILING_IN_GRAAL ||\
        !(CYTHON_ASSUME_SAFE_SIZE && CYTHON_ASSUME_SAFE_MACROS)
    return PyObject_RichCompareBool(s1, s2, equals);
#else
    if (s1 == s2) {
        return (equals == Py_EQ);
    } else if (PyBytes_CheckExact(s1) & PyBytes_CheckExact(s2)) {
        const char *ps1, *ps2;
        Py_ssize_t length = PyBytes_GET_SIZE(s1);
        if (length != PyBytes_GET_SIZE(s2))
            return (equals == Py_NE);
        ps1 = PyBytes_AS_STRING(s1);
        ps2 = PyBytes_AS_STRING(s2);
        if (ps1[0] != ps2[0]) {
            return (equals == Py_NE);
        } else if (length == 1) {
            return (equals == Py_EQ);
        } else {
            int result;
#if CYTHON_USE_UNICODE_INTERNALS && (PY_VERSION_HEX < 0x030B0000)
            Py_hash_t hash1, hash2;
            hash1 = ((PyBytesObject*)s1)->ob_shash;
            hash2 = ((PyBytesObject*)s2)->ob_shash;
            if (hash1 != hash2 && hash1 != -1 && hash2 != -1) {
                return (equals == Py_NE);
            }
#endif
            result = memcmp(ps1, ps2, (size_t)length);
            return (equals == Py_EQ) ? (result == 0) : (result != 0);
        }
    } else if ((s1 == Py_None) & PyBytes_CheckExact(s2)) {
        return (equals == Py_NE);
    } else if ((s2 == Py_None) & PyBytes_CheckExact(s1)) {
        return (equals == Py_NE);
    } else {
        int result;
        PyObject* py_result = PyObject_RichCompare(s1, s2, equals);
        if (!py_result)
            return -1;
        result = __Pyx_PyObject_IsTrue(py_result);
        Py_DECREF(py_result);
        return result;
    }
#endif
}

/* UnicodeEquals (used by fastcall) */
static CYTHON_INLINE int __Pyx_PyUnicode_Equals(PyObject* s1, PyObject* s2, int equals) {
#if CYTHON_COMPILING_IN_PYPY || CYTHON_COMPILING_IN_LIMITED_API || CYTHON_COMPILING_IN_GRAAL
    return PyObject_RichCompareBool(s1, s2, equals);
#else
    int s1_is_unicode, s2_is_unicode;
    if (s1 == s2) {
        goto return_eq;
    }
    s1_is_unicode = PyUnicode_CheckExact(s1);
    s2_is_unicode = PyUnicode_CheckExact(s2);
    if (s1_is_unicode & s2_is_unicode) {
        Py_ssize_t length, length2;
        int kind;
        void *data1, *data2;
        #if !CYTHON_COMPILING_IN_LIMITED_API
        if (unlikely(__Pyx_PyUnicode_READY(s1) < 0) || unlikely(__Pyx_PyUnicode_READY(s2) < 0))
            return -1;
        #endif
        length = __Pyx_PyUnicode_GET_LENGTH(s1);
        #if !CYTHON_ASSUME_SAFE_SIZE
        if (unlikely(length < 0)) return -1;
        #endif
        length2 = __Pyx_PyUnicode_GET_LENGTH(s2);
        #if !CYTHON_ASSUME_SAFE_SIZE
        if (unlikely(length2 < 0)) return -1;
        #endif
        if (length != length2) {
            goto return_ne;
        }
#if CYTHON_USE_UNICODE_INTERNALS
        {
            Py_hash_t hash1, hash2;
            hash1 = ((PyASCIIObject*)s1)->hash;
            hash2 = ((PyASCIIObject*)s2)->hash;
            if (hash1 != hash2 && hash1 != -1 && hash2 != -1) {
                goto return_ne;
            }
        }
#endif
        kind = __Pyx_PyUnicode_KIND(s1);
        if (kind != __Pyx_PyUnicode_KIND(s2)) {
            goto return_ne;
        }
        data1 = __Pyx_PyUnicode_DATA(s1);
        data2 = __Pyx_PyUnicode_DATA(s2);
        if (__Pyx_PyUnicode_READ(kind, data1, 0) != __Pyx_PyUnicode_READ(kind, data2, 0)) {
            goto return_ne;
        } else if (length == 1) {
            goto return_eq;
        } else {
            int result = memcmp(data1, data2, (size_t)(length * kind));
            return (equals == Py_EQ) ? (result == 0) : (result != 0);
        }
    } else if ((s1 == Py_None) & s2_is_unicode) {
        goto return_ne;
    } else if ((s2 == Py_None) & s1_is_unicode) {
        goto return_ne;
    } else {
        int result;
        PyObject* py_result = PyObject_RichCompare(s1, s2, equals);
        if (!py_result)
            return -1;
        result = __Pyx_PyObject_IsTrue(py_result);
        Py_DECREF(py_result);
        return result;
    }
return_eq:
    return (equals == Py_EQ);
return_ne:
    return (equals == Py_NE);
#endif
}

/* fastcall */
#if CYTHON_METH_FASTCALL
static CYTHON_INLINE PyObject * __Pyx_GetKwValue_FASTCALL(PyObject *kwnames, PyObject *const *kwvalues, PyObject *s)
{
    Py_ssize_t i, n = __Pyx_PyTuple_GET_SIZE(kwnames);
    #if !CYTHON_ASSUME_SAFE_SIZE
    if (unlikely(n == -1)) return NULL;
    #endif
    for (i = 0; i < n; i++)
    {
        PyObject *namei = __Pyx_PyTuple_GET_ITEM(kwnames, i);
        #if !CYTHON_ASSUME_SAFE_MACROS
        if (unlikely(!namei)) return NULL;
        #endif
        if (s == namei) return kwvalues[i];
    }
    for (i = 0; i < n; i++)
    {
        PyObject *namei = __Pyx_PyTuple_GET_ITEM(kwnames, i);
        #if !CYTHON_ASSUME_SAFE_MACROS
        if (unlikely(!namei)) return NULL;
        #endif
        int eq = __Pyx_PyUnicode_Equals(s, namei, Py_EQ);
        if (unlikely(eq != 0)) {
            if (unlikely(eq < 0)) return NULL;
            return kwvalues[i];
        }
    }
    return NULL;
}
#if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x030d0000 || CYTHON_COMPILING_IN_LIMITED_API
CYTHON_UNUSED static PyObject *__Pyx_KwargsAsDict_FASTCALL(PyObject *kwnames, PyObject *const *kwvalues) {
    Py_ssize_t i, nkwargs;
    PyObject *dict;
#if !CYTHON_ASSUME_SAFE_SIZE
    nkwargs = PyTuple_Size(kwnames);
    if (unlikely(nkwargs < 0)) return NULL;
#else
    nkwargs = PyTuple_GET_SIZE(kwnames);
#endif
    dict = PyDict_New();
    if (unlikely(!dict))
        return NULL;
    for (i=0; i<nkwargs; i++) {
#if !CYTHON_ASSUME_SAFE_MACROS
        PyObject *key = PyTuple_GetItem(kwnames, i);
        if (!key) goto bad;
#else
        PyObject *key = PyTuple_GET_ITEM(kwnames, i);
#endif
        if (unlikely(PyDict_SetItem(dict, key, kwvalues[i]) < 0))
            goto bad;
    }
    return dict;
bad:
    Py_DECREF(dict);
    return NULL;
}
#endif
#endif

/* PyObjectCall (used by PyObjectFastCall) */
#if CYTHON_COMPILING_IN_CPYTHON
static CYTHON_INLINE PyObject* __Pyx_PyObject_Call(PyObject *func, PyObject *arg, PyObject *kw) {
    PyObject *result;
    ternaryfunc call = Py_TYPE(func)->tp_call;
    if (unlikely(!call))
        return PyObject_Call(func, arg, kw);
    if (unlikely(Py_EnterRecursiveCall(" while calling a Python object")))
        return NULL;
    result = (*call)(func, arg, kw);
    Py_LeaveRecursiveCall();
    if (unlikely(!result) && unlikely(!PyErr_Occurred())) {
        PyErr_SetString(
            PyExc_SystemError,
            "NULL result without error in PyObject_Call");
    }
    return result;
}
#endif

/* PyObjectCallMethO (used by PyObjectFastCall) */
#if CYTHON_COMPILING_IN_CPYTHON
static CYTHON_INLINE PyObject* __Pyx_PyObject_CallMethO(PyObject *func, PyObject *arg) {
    PyObject *self, *result;
    PyCFunction cfunc;
    cfunc = __Pyx_CyOrPyCFunction_GET_FUNCTION(func);
    self = __Pyx_CyOrPyCFunction_GET_SELF(func);
    if (unlikely(Py_EnterRecursiveCall(" while calling a Python object")))
        return NULL;
    result = cfunc(self, arg);
    Py_LeaveRecursiveCall();
    if (unlikely(!result) && unlikely(!PyErr_Occurred())) {
        PyErr_SetString(
            PyExc_SystemError,
            "NULL result without error in PyObject_Call");
    }
    return result;
}
#endif

/* PyObjectFastCall (used by PyObjectCallOneArg) */
#if PY_VERSION_HEX < 0x03090000 || CYTHON_COMPILING_IN_LIMITED_API
static PyObject* __Pyx_PyObject_FastCall_fallback(PyObject *func, PyObject * const*args, size_t nargs, PyObject *kwargs) {
    PyObject *argstuple;
    PyObject *result = 0;
    size_t i;
    argstuple = PyTuple_New((Py_ssize_t)nargs);
    if (unlikely(!argstuple)) return NULL;
    for (i = 0; i < nargs; i++) {
        Py_INCREF(args[i]);
        if (__Pyx_PyTuple_SET_ITEM(argstuple, (Py_ssize_t)i, args[i]) != (0)) goto bad;
    }
    result = __Pyx_PyObject_Call(func, argstuple, kwargs);
  bad:
    Py_DECREF(argstuple);
    return result;
}
#endif
#if CYTHON_VECTORCALL && !CYTHON_COMPILING_IN_LIMITED_API
  #if PY_VERSION_HEX < 0x03090000
    #define __Pyx_PyVectorcall_Function(callable) _PyVectorcall_Function(callable)
  #elif CYTHON_COMPILING_IN_CPYTHON
static CYTHON_INLINE vectorcallfunc __Pyx_PyVectorcall_Function(PyObject *callable) {
    PyTypeObject *tp = Py_TYPE(callable);
    #if defined(__Pyx_CyFunction_USED)
    if (__Pyx_CyFunction_CheckExact(callable)) {
        return __Pyx_CyFunction_func_vectorcall(callable);
    }
    #endif
    if (!PyType_HasFeature(tp, Py_TPFLAGS_HAVE_VECTORCALL)) {
        return NULL;
    }
    assert(PyCallable_Check(callable));
    Py_ssize_t offset = tp->tp_vectorcall_offset;
    assert(offset > 0);
    vectorcallfunc ptr;
    memcpy(&ptr, (char *) callable + offset, sizeof(ptr));
    return ptr;
}
  #else
    #define __Pyx_PyVectorcall_Function(callable) PyVectorcall_Function(callable)
  #endif
#endif
static CYTHON_INLINE PyObject* __Pyx_PyObject_FastCallDict(PyObject *func, PyObject *const *args, size_t _nargs, PyObject *kwargs) {
    Py_ssize_t nargs = __Pyx_PyVectorcall_NARGS(_nargs);
#if CYTHON_COMPILING_IN_CPYTHON
    if (nargs == 0 && kwargs == NULL) {
        if (__Pyx_CyOrPyCFunction_Check(func) && likely( __Pyx_CyOrPyCFunction_GET_FLAGS(func) & METH_NOARGS))
            return __Pyx_PyObject_CallMethO(func, NULL);
    }
    else if (nargs == 1 && kwargs == NULL) {
        if (__Pyx_CyOrPyCFunction_Check(func) && likely( __Pyx_CyOrPyCFunction_GET_FLAGS(func) & METH_O))
            return __Pyx_PyObject_CallMethO(func, args[0]);
    }
#endif
    if (kwargs == NULL) {
        #if CYTHON_VECTORCALL
          #if CYTHON_COMPILING_IN_LIMITED_API
            return PyObject_Vectorcall(func, args, _nargs, NULL);
          #else
            vectorcallfunc f = __Pyx_PyVectorcall_Function(func);
            if (f) {
                return f(func, args, _nargs, NULL);
            }
          #endif
        #endif
    }
    if (nargs == 0) {
        return __Pyx_PyObject_Call(func, __pyx_mstate_global->__pyx_empty_tuple, kwargs);
    }
    #if PY_VERSION_HEX >= 0x03090000 && !CYTHON_COMPILING_IN_LIMITED_API
    return PyObject_VectorcallDict(func, args, (size_t)nargs, kwargs);
    #else
    return __Pyx_PyObject_FastCall_fallback(func, args, (size_t)nargs, kwargs);
    #endif
}

/* PyObjectCallOneArg (used by CallUnboundCMethod0) */
static CYTHON_INLINE PyObject* __Pyx_PyObject_CallOneArg(PyObject *func, PyObject *arg) {
    PyObject *args[2] = {NULL, arg};
    return __Pyx_PyObject_FastCall(func, args+1, 1 | __Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET);
}

/* UnpackUnboundCMethod (used by CallUnboundCMethod0) */
#if CYTHON_COMPILING_IN_LIMITED_API && __PYX_LIMITED_VERSION_HEX < 0x030C0000
static PyObject *__Pyx_SelflessCall(PyObject *method, PyObject *args, PyObject *kwargs) {
    PyObject *result;
    PyObject *selfless_args = PyTuple_GetSlice(args, 1, PyTuple_Size(args));
    if (unlikely(!selfless_args)) return NULL;
    result = PyObject_Call(method, selfless_args, kwargs);
    Py_DECREF(selfless_args);
    return result;
}
#elif CYTHON_COMPILING_IN_PYPY && PY_VERSION_HEX < 0x03090000
static PyObject *__Pyx_SelflessCall(PyObject *method, PyObject **args, Py_ssize_t nargs, PyObject *kwnames) {
        return _PyObject_Vectorcall
            (method, args ? args+1 : NULL, nargs ? nargs-1 : 0, kwnames);
}
#else
static PyObject *__Pyx_SelflessCall(PyObject *method, PyObject *const *args, Py_ssize_t nargs, PyObject *kwnames) {
    return
#if PY_VERSION_HEX < 0x03090000
    _PyObject_Vectorcall
#else
    PyObject_Vectorcall
#endif
        (method, args ? args+1 : NULL, nargs ? (size_t) nargs-1 : 0, kwnames);
}
#endif
static PyMethodDef __Pyx_UnboundCMethod_Def = {
     "CythonUnboundCMethod",
     __PYX_REINTERPRET_FUNCION(PyCFunction, __Pyx_SelflessCall),
#if CYTHON_COMPILING_IN_LIMITED_API && __PYX_LIMITED_VERSION_HEX < 0x030C0000
     METH_VARARGS | METH_KEYWORDS,
#else
     METH_FASTCALL | METH_KEYWORDS,
#endif
     NULL
};
static int __Pyx_TryUnpackUnboundCMethod(__Pyx_CachedCFunction* target) {
    PyObject *method, *result=NULL;
    method = __Pyx_PyObject_GetAttrStr(target->type, *target->method_name);
    if (unlikely(!method))
        return -1;
    result = method;
#if CYTHON_COMPILING_IN_CPYTHON
    if (likely(__Pyx_TypeCheck(method, &PyMethodDescr_Type)))
    {
        PyMethodDescrObject *descr = (PyMethodDescrObject*) method;
        target->func = descr->d_method->ml_meth;
        target->flag = descr->d_method->ml_flags & ~(METH_CLASS | METH_STATIC | METH_COEXIST | METH_STACKLESS);
    } else
#endif
#if CYTHON_COMPILING_IN_PYPY
#else
    if (PyCFunction_Check(method))
#endif
    {
        PyObject *self;
        int self_found;
#if CYTHON_COMPILING_IN_LIMITED_API || CYTHON_COMPILING_IN_PYPY
        self = PyObject_GetAttrString(method, "__self__");
        if (!self) {
            PyErr_Clear();
        }
#else
        self = PyCFunction_GET_SELF(method);
#endif
        self_found = (self && self != Py_None);
#if CYTHON_COMPILING_IN_LIMITED_API || CYTHON_COMPILING_IN_PYPY
        Py_XDECREF(self);
#endif
        if (self_found) {
            PyObject *unbound_method = PyCFunction_New(&__Pyx_UnboundCMethod_Def, method);
            if (unlikely(!unbound_method)) return -1;
            Py_DECREF(method);
            result = unbound_method;
        }
    }
#if !CYTHON_COMPILING_IN_CPYTHON_FREETHREADING
    if (unlikely(target->method)) {
        Py_DECREF(result);
    } else
#endif
    target->method = result;
    return 0;
}

/* CallUnboundCMethod0 */
#if CYTHON_COMPILING_IN_CPYTHON
static CYTHON_INLINE PyObject* __Pyx_CallUnboundCMethod0(__Pyx_CachedCFunction* cfunc, PyObject* self) {
    int was_initialized = __Pyx_CachedCFunction_GetAndSetInitializing(cfunc);
    if (likely(was_initialized == 2 && cfunc->func)) {
        if (likely(cfunc->flag == METH_NOARGS))
            return __Pyx_CallCFunction(cfunc, self, NULL);
        if (likely(cfunc->flag == METH_FASTCALL))
            return __Pyx_CallCFunctionFast(cfunc, self, NULL, 0);
        if (cfunc->flag == (METH_FASTCALL | METH_KEYWORDS))
            return __Pyx_CallCFunctionFastWithKeywords(cfunc, self, NULL, 0, NULL);
        if (likely(cfunc->flag == (METH_VARARGS | METH_KEYWORDS)))
            return __Pyx_CallCFunctionWithKeywords(cfunc, self, __pyx_mstate_global->__pyx_empty_tuple, NULL);
        if (cfunc->flag == METH_VARARGS)
            return __Pyx_CallCFunction(cfunc, self, __pyx_mstate_global->__pyx_empty_tuple);
        return __Pyx__CallUnboundCMethod0(cfunc, self);
    }
#if CYTHON_COMPILING_IN_CPYTHON_FREETHREADING
    else if (unlikely(was_initialized == 1)) {
        __Pyx_CachedCFunction tmp_cfunc = {
#ifndef __cplusplus
            0
#endif
        };
        tmp_cfunc.type = cfunc->type;
        tmp_cfunc.method_name = cfunc->method_name;
        return __Pyx__CallUnboundCMethod0(&tmp_cfunc, self);
    }
#endif
    PyObject *result = __Pyx__CallUnboundCMethod0(cfunc, self);
    __Pyx_CachedCFunction_SetFinishedInitializing(cfunc);
    return result;
}
#endif
static PyObject* __Pyx__CallUnboundCMethod0(__Pyx_CachedCFunction* cfunc, PyObject* self) {
    PyObject *result;
    if (unlikely(!cfunc->method) && unlikely(__Pyx_TryUnpackUnboundCMethod(cfunc) < 0)) return NULL;
    result = __Pyx_PyObject_CallOneArg(cfunc->method, self);
    return result;
}

/* py_dict_items (used by OwnedDictNext) */
static CYTHON_INLINE PyObject* __Pyx_PyDict_Items(PyObject* d) {
    return __Pyx_CallUnboundCMethod0(&__pyx_mstate_global->__pyx_umethod_PyDict_Type_items, d);
}

/* py_dict_values (used by OwnedDictNext) */
static CYTHON_INLINE PyObject* __Pyx_PyDict_Values(PyObject* d) {
    return __Pyx_CallUnboundCMethod0(&__pyx_mstate_global->__pyx_umethod_PyDict_Type_values, d);
}

/* OwnedDictNext (used by ParseKeywordsImpl) */
#if CYTHON_AVOID_BORROWED_REFS
static int __Pyx_PyDict_NextRef(PyObject *p, PyObject **ppos, PyObject **pkey, PyObject **pvalue) {
    PyObject *next = NULL;
    if (!*ppos) {
        if (pvalue) {
            PyObject *dictview = pkey ? __Pyx_PyDict_Items(p) : __Pyx_PyDict_Values(p);
            if (unlikely(!dictview)) goto bad;
            *ppos = PyObject_GetIter(dictview);
            Py_DECREF(dictview);
        } else {
            *ppos = PyObject_GetIter(p);
        }
        if (unlikely(!*ppos)) goto bad;
    }
    next = PyIter_Next(*ppos);
    if (!next) {
        if (PyErr_Occurred()) goto bad;
        return 0;
    }
    if (pkey && pvalue) {
        *pkey = __Pyx_PySequence_ITEM(next, 0);
        if (unlikely(*pkey)) goto bad;
        *pvalue = __Pyx_PySequence_ITEM(next, 1);
        if (unlikely(*pvalue)) goto bad;
        Py_DECREF(next);
    } else if (pkey) {
        *pkey = next;
    } else {
        assert(pvalue);
        *pvalue = next;
    }
    return 1;
  bad:
    Py_XDECREF(next);
#if !CYTHON_COMPILING_IN_LIMITED_API && PY_VERSION_HEX >= 0x030d0000
    PyErr_FormatUnraisable("Exception ignored in __Pyx_PyDict_NextRef");
#else
    PyErr_WriteUnraisable(__pyx_mstate_global->__pyx_n_u_Pyx_PyDict_NextRef);
#endif
    if (pkey) *pkey = NULL;
    if (pvalue) *pvalue = NULL;
    return 0;
}
#else // !CYTHON_AVOID_BORROWED_REFS
static int __Pyx_PyDict_NextRef(PyObject *p, Py_ssize_t *ppos, PyObject **pkey, PyObject **pvalue) {
    int result = PyDict_Next(p, ppos, pkey, pvalue);
    if (likely(result == 1)) {
        if (pkey) Py_INCREF(*pkey);
        if (pvalue) Py_INCREF(*pvalue);
    }
    return result;
}
#endif

/* RaiseDoubleKeywords (used by ParseKeywordsImpl) */
static void __Pyx_RaiseDoubleKeywordsError(
    const char* func_name,
    PyObject* kw_name)
{
    PyErr_Format(PyExc_TypeError,
        "%s() got multiple values for keyword argument '%U'", func_name, kw_name);
}

/* CallUnboundCMethod2 */
#if CYTHON_COMPILING_IN_CPYTHON
static CYTHON_INLINE PyObject *__Pyx_CallUnboundCMethod2(__Pyx_CachedCFunction *cfunc, PyObject *self, PyObject *arg1, PyObject *arg2) {
    int was_initialized = __Pyx_CachedCFunction_GetAndSetInitializing(cfunc);
    if (likely(was_initialized == 2 && cfunc->func)) {
        PyObject *args[2] = {arg1, arg2};
        if (cfunc->flag == METH_FASTCALL) {
            return __Pyx_CallCFunctionFast(cfunc, self, args, 2);
        }
        if (cfunc->flag == (METH_FASTCALL | METH_KEYWORDS))
            return __Pyx_CallCFunctionFastWithKeywords(cfunc, self, args, 2, NULL);
    }
#if CYTHON_COMPILING_IN_CPYTHON_FREETHREADING
    else if (unlikely(was_initialized == 1)) {
        __Pyx_CachedCFunction tmp_cfunc = {
#ifndef __cplusplus
            0
#endif
        };
        tmp_cfunc.type = cfunc->type;
        tmp_cfunc.method_name = cfunc->method_name;
        return __Pyx__CallUnboundCMethod2(&tmp_cfunc, self, arg1, arg2);
    }
#endif
    PyObject *result = __Pyx__CallUnboundCMethod2(cfunc, self, arg1, arg2);
    __Pyx_CachedCFunction_SetFinishedInitializing(cfunc);
    return result;
}
#endif
static PyObject* __Pyx__CallUnboundCMethod2(__Pyx_CachedCFunction* cfunc, PyObject* self, PyObject* arg1, PyObject* arg2){
    if (unlikely(!cfunc->func && !cfunc->method) && unlikely(__Pyx_TryUnpackUnboundCMethod(cfunc) < 0)) return NULL;
#if CYTHON_COMPILING_IN_CPYTHON
    if (cfunc->func && (cfunc->flag & METH_VARARGS)) {
        PyObject *result = NULL;
        PyObject *args = PyTuple_New(2);
        if (unlikely(!args)) return NULL;
        Py_INCREF(arg1);
        PyTuple_SET_ITEM(args, 0, arg1);
        Py_INCREF(arg2);
        PyTuple_SET_ITEM(args, 1, arg2);
        if (cfunc->flag & METH_KEYWORDS)
            result = __Pyx_CallCFunctionWithKeywords(cfunc, self, args, NULL);
        else
            result = __Pyx_CallCFunction(cfunc, self, args);
        Py_DECREF(args);
        return result;
    }
#endif
    {
        PyObject *args[4] = {NULL, self, arg1, arg2};
        return __Pyx_PyObject_FastCall(cfunc->method, args+1, 3 | __Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET);
    }
}

/* ParseKeywordsImpl (used by ParseKeywords) */
static int __Pyx_ValidateDuplicatePosArgs(
    PyObject *kwds,
    PyObject ** const argnames[],
    PyObject ** const *first_kw_arg,
    const char* function_name)
{
    PyObject ** const *name = argnames;
    while (name != first_kw_arg) {
        PyObject *key = **name;
        int found = PyDict_Contains(kwds, key);
        if (unlikely(found)) {
            if (found == 1) __Pyx_RaiseDoubleKeywordsError(function_name, key);
            goto bad;
        }
        name++;
    }
    return 0;
bad:
    return -1;
}
#if CYTHON_USE_UNICODE_INTERNALS
static CYTHON_INLINE int __Pyx_UnicodeKeywordsEqual(PyObject *s1, PyObject *s2) {
    int kind;
    Py_ssize_t len = PyUnicode_GET_LENGTH(s1);
    if (len != PyUnicode_GET_LENGTH(s2)) return 0;
    kind = PyUnicode_KIND(s1);
    if (kind != PyUnicode_KIND(s2)) return 0;
    const void *data1 = PyUnicode_DATA(s1);
    const void *data2 = PyUnicode_DATA(s2);
    return (memcmp(data1, data2, (size_t) len * (size_t) kind) == 0);
}
#endif
static int __Pyx_MatchKeywordArg_str(
    PyObject *key,
    PyObject ** const argnames[],
    PyObject ** const *first_kw_arg,
    size_t *index_found,
    const char *function_name)
{
    PyObject ** const *name;
    #if CYTHON_USE_UNICODE_INTERNALS
    Py_hash_t key_hash = ((PyASCIIObject*)key)->hash;
    if (unlikely(key_hash == -1)) {
        key_hash = PyObject_Hash(key);
        if (unlikely(key_hash == -1))
            goto bad;
    }
    #endif
    name = first_kw_arg;
    while (*name) {
        PyObject *name_str = **name;
        #if CYTHON_USE_UNICODE_INTERNALS
        if (key_hash == ((PyASCIIObject*)name_str)->hash && __Pyx_UnicodeKeywordsEqual(name_str, key)) {
            *index_found = (size_t) (name - argnames);
            return 1;
        }
        #else
        #if CYTHON_ASSUME_SAFE_SIZE
        if (PyUnicode_GET_LENGTH(name_str) == PyUnicode_GET_LENGTH(key))
        #endif
        {
            int cmp = PyUnicode_Compare(name_str, key);
            if (cmp < 0 && unlikely(PyErr_Occurred())) goto bad;
            if (cmp == 0) {
                *index_found = (size_t) (name - argnames);
                return 1;
            }
        }
        #endif
        name++;
    }
    name = argnames;
    while (name != first_kw_arg) {
        PyObject *name_str = **name;
        #if CYTHON_USE_UNICODE_INTERNALS
        if (unlikely(key_hash == ((PyASCIIObject*)name_str)->hash)) {
            if (__Pyx_UnicodeKeywordsEqual(name_str, key))
                goto arg_passed_twice;
        }
        #else
        #if CYTHON_ASSUME_SAFE_SIZE
        if (PyUnicode_GET_LENGTH(name_str) == PyUnicode_GET_LENGTH(key))
        #endif
        {
            if (unlikely(name_str == key)) goto arg_passed_twice;
            int cmp = PyUnicode_Compare(name_str, key);
            if (cmp < 0 && unlikely(PyErr_Occurred())) goto bad;
            if (cmp == 0) goto arg_passed_twice;
        }
        #endif
        name++;
    }
    return 0;
arg_passed_twice:
    __Pyx_RaiseDoubleKeywordsError(function_name, key);
    goto bad;
bad:
    return -1;
}
static int __Pyx_MatchKeywordArg_nostr(
    PyObject *key,
    PyObject ** const argnames[],
    PyObject ** const *first_kw_arg,
    size_t *index_found,
    const char *function_name)
{
    PyObject ** const *name;
    if (unlikely(!PyUnicode_Check(key))) goto invalid_keyword_type;
    name = first_kw_arg;
    while (*name) {
        int cmp = PyObject_RichCompareBool(**name, key, Py_EQ);
        if (cmp == 1) {
            *index_found = (size_t) (name - argnames);
            return 1;
        }
        if (unlikely(cmp == -1)) goto bad;
        name++;
    }
    name = argnames;
    while (name != first_kw_arg) {
        int cmp = PyObject_RichCompareBool(**name, key, Py_EQ);
        if (unlikely(cmp != 0)) {
            if (cmp == 1) goto arg_passed_twice;
            else goto bad;
        }
        name++;
    }
    return 0;
arg_passed_twice:
    __Pyx_RaiseDoubleKeywordsError(function_name, key);
    goto bad;
invalid_keyword_type:
    PyErr_Format(PyExc_TypeError,
        "%.200s() keywords must be strings", function_name);
    goto bad;
bad:
    return -1;
}
static CYTHON_INLINE int __Pyx_MatchKeywordArg(
    PyObject *key,
    PyObject ** const argnames[],
    PyObject ** const *first_kw_arg,
    size_t *index_found,
    const char *function_name)
{
    return likely(PyUnicode_CheckExact(key)) ?
        __Pyx_MatchKeywordArg_str(key, argnames, first_kw_arg, index_found, function_name) :
        __Pyx_MatchKeywordArg_nostr(key, argnames, first_kw_arg, index_found, function_name);
}
static void __Pyx_RejectUnknownKeyword(
    PyObject *kwds,
    PyObject ** const argnames[],
    PyObject ** const *first_kw_arg,
    const char *function_name)
{
    #if CYTHON_AVOID_BORROWED_REFS
    PyObject *pos = NULL;
    #else
    Py_ssize_t pos = 0;
    #endif
    PyObject *key = NULL;
    __Pyx_BEGIN_CRITICAL_SECTION(kwds);
    while (
        #if CYTHON_AVOID_BORROWED_REFS
        __Pyx_PyDict_NextRef(kwds, &pos, &key, NULL)
        #else
        PyDict_Next(kwds, &pos, &key, NULL)
        #endif
    ) {
        PyObject** const *name = first_kw_arg;
        while (*name && (**name != key)) name++;
        if (!*name) {
            size_t index_found = 0;
            int cmp = __Pyx_MatchKeywordArg(key, argnames, first_kw_arg, &index_found, function_name);
            if (cmp != 1) {
                if (cmp == 0) {
                    PyErr_Format(PyExc_TypeError,
                        "%s() got an unexpected keyword argument '%U'",
                        function_name, key);
                }
                #if CYTHON_AVOID_BORROWED_REFS
                Py_DECREF(key);
                #endif
                break;
            }
        }
        #if CYTHON_AVOID_BORROWED_REFS
        Py_DECREF(key);
        #endif
    }
    __Pyx_END_CRITICAL_SECTION();
    #if CYTHON_AVOID_BORROWED_REFS
    Py_XDECREF(pos);
    #endif
    assert(PyErr_Occurred());
}
static int __Pyx_ParseKeywordDict(
    PyObject *kwds,
    PyObject ** const argnames[],
    PyObject *values[],
    Py_ssize_t num_pos_args,
    Py_ssize_t num_kwargs,
    const char* function_name,
    int ignore_unknown_kwargs)
{
    PyObject** const *name;
    PyObject** const *first_kw_arg = argnames + num_pos_args;
    Py_ssize_t extracted = 0;
#if !CYTHON_COMPILING_IN_PYPY || defined(PyArg_ValidateKeywordArguments)
    if (unlikely(!PyArg_ValidateKeywordArguments(kwds))) return -1;
#endif
    name = first_kw_arg;
    while (*name && num_kwargs > extracted) {
        PyObject * key = **name;
        PyObject *value;
        int found = 0;
        #if __PYX_LIMITED_VERSION_HEX >= 0x030d0000
        found = PyDict_GetItemRef(kwds, key, &value);
        #else
        value = PyDict_GetItemWithError(kwds, key);
        if (value) {
            Py_INCREF(value);
            found = 1;
        } else {
            if (unlikely(PyErr_Occurred())) goto bad;
        }
        #endif
        if (found) {
            if (unlikely(found < 0)) goto bad;
            values[name-argnames] = value;
            extracted++;
        }
        name++;
    }
    if (num_kwargs > extracted) {
        if (ignore_unknown_kwargs) {
            if (unlikely(__Pyx_ValidateDuplicatePosArgs(kwds, argnames, first_kw_arg, function_name) == -1))
                goto bad;
        } else {
            __Pyx_RejectUnknownKeyword(kwds, argnames, first_kw_arg, function_name);
            goto bad;
        }
    }
    return 0;
bad:
    return -1;
}
static int __Pyx_ParseKeywordDictToDict(
    PyObject *kwds,
    PyObject ** const argnames[],
    PyObject *kwds2,
    PyObject *values[],
    Py_ssize_t num_pos_args,
    const char* function_name)
{
    PyObject** const *name;
    PyObject** const *first_kw_arg = argnames + num_pos_args;
    Py_ssize_t len;
#if !CYTHON_COMPILING_IN_PYPY || defined(PyArg_ValidateKeywordArguments)
    if (unlikely(!PyArg_ValidateKeywordArguments(kwds))) return -1;
#endif
    if (PyDict_Update(kwds2, kwds) < 0) goto bad;
    name = first_kw_arg;
    while (*name) {
        PyObject *key = **name;
        PyObject *value;
#if !CYTHON_COMPILING_IN_LIMITED_API && (PY_VERSION_HEX >= 0x030d00A2 || defined(PyDict_Pop))
        int found = PyDict_Pop(kwds2, key, &value);
        if (found) {
            if (unlikely(found < 0)) goto bad;
            values[name-argnames] = value;
        }
#elif __PYX_LIMITED_VERSION_HEX >= 0x030d0000
        int found = PyDict_GetItemRef(kwds2, key, &value);
        if (found) {
            if (unlikely(found < 0)) goto bad;
            values[name-argnames] = value;
            if (unlikely(PyDict_DelItem(kwds2, key) < 0)) goto bad;
        }
#else
    #if CYTHON_COMPILING_IN_CPYTHON
        value = _PyDict_Pop(kwds2, key, kwds2);
    #else
        value = __Pyx_CallUnboundCMethod2(&__pyx_mstate_global->__pyx_umethod_PyDict_Type_pop, kwds2, key, kwds2);
    #endif
        if (value == kwds2) {
            Py_DECREF(value);
        } else {
            if (unlikely(!value)) goto bad;
            values[name-argnames] = value;
        }
#endif
        name++;
    }
    len = PyDict_Size(kwds2);
    if (len > 0) {
        return __Pyx_ValidateDuplicatePosArgs(kwds, argnames, first_kw_arg, function_name);
    } else if (unlikely(len == -1)) {
        goto bad;
    }
    return 0;
bad:
    return -1;
}
static int __Pyx_ParseKeywordsTuple(
    PyObject *kwds,
    PyObject * const *kwvalues,
    PyObject ** const argnames[],
    PyObject *kwds2,
    PyObject *values[],
    Py_ssize_t num_pos_args,
    Py_ssize_t num_kwargs,
    const char* function_name,
    int ignore_unknown_kwargs)
{
    PyObject *key = NULL;
    PyObject** const * name;
    PyObject** const *first_kw_arg = argnames + num_pos_args;
    for (Py_ssize_t pos = 0; pos < num_kwargs; pos++) {
#if CYTHON_AVOID_BORROWED_REFS
        key = __Pyx_PySequence_ITEM(kwds, pos);
#else
        key = __Pyx_PyTuple_GET_ITEM(kwds, pos);
#endif
#if !CYTHON_ASSUME_SAFE_MACROS
        if (unlikely(!key)) goto bad;
#endif
        name = first_kw_arg;
        while (*name && (**name != key)) name++;
        if (*name) {
            PyObject *value = kwvalues[pos];
            values[name-argnames] = __Pyx_NewRef(value);
        } else {
            size_t index_found = 0;
            int cmp = __Pyx_MatchKeywordArg(key, argnames, first_kw_arg, &index_found, function_name);
            if (cmp == 1) {
                PyObject *value = kwvalues[pos];
                values[index_found] = __Pyx_NewRef(value);
            } else {
                if (unlikely(cmp == -1)) goto bad;
                if (kwds2) {
                    PyObject *value = kwvalues[pos];
                    if (unlikely(PyDict_SetItem(kwds2, key, value))) goto bad;
                } else if (!ignore_unknown_kwargs) {
                    goto invalid_keyword;
                }
            }
        }
        #if CYTHON_AVOID_BORROWED_REFS
        Py_DECREF(key);
        key = NULL;
        #endif
    }
    return 0;
invalid_keyword:
    PyErr_Format(PyExc_TypeError,
        "%s() got an unexpected keyword argument '%U'",
        function_name, key);
    goto bad;
bad:
    #if CYTHON_AVOID_BORROWED_REFS
    Py_XDECREF(key);
    #endif
    return -1;
}

/* ParseKeywords */
static int __Pyx_ParseKeywords(
    PyObject *kwds,
    PyObject * const *kwvalues,
    PyObject ** const argnames[],
    PyObject *kwds2,
    PyObject *values[],
    Py_ssize_t num_pos_args,
    Py_ssize_t num_kwargs,
    const char* function_name,
    int ignore_unknown_kwargs)
{
    if (CYTHON_METH_FASTCALL && likely(PyTuple_Check(kwds)))
        return __Pyx_ParseKeywordsTuple(kwds, kwvalues, argnames, kwds2, values, num_pos_args, num_kwargs, function_name, ignore_unknown_kwargs);
    else if (kwds2)
        return __Pyx_ParseKeywordDictToDict(kwds, argnames, kwds2, values, num_pos_args, function_name);
    else
        return __Pyx_ParseKeywordDict(kwds, argnames, values, num_pos_args, num_kwargs, function_name, ignore_unknown_kwargs);
}

/* RaiseArgTupleInvalid */
static void __Pyx_RaiseArgtupleInvalid(
    const char* func_name,
    int exact,
    Py_ssize_t num_min,
    Py_ssize_t num_max,
    Py_ssize_t num_found)
{
    Py_ssize_t num_expected;
    const char *more_or_less;
    if (num_found < num_min) {
        num_expected = num_min;
        more_or_less = "at least";
    } else {
        num_expected = num_max;
        more_or_less = "at most";
    }
    if (exact) {
        more_or_less = "exactly";
    }
    PyErr_Format(PyExc_TypeError,
                 "%.200s() takes %.8s %" CYTHON_FORMAT_SSIZE_T "d positional argument%.1s (%" CYTHON_FORMAT_SSIZE_T "d given)",
                 func_name, more_or_less, num_expected,
                 (num_expected == 1) ? "" : "s", num_found);
}

/* PyObjectSetAttrStr */
#if CYTHON_USE_TYPE_SLOTS
static CYTHON_INLINE int __Pyx_PyObject_SetAttrStr(PyObject* obj, PyObject* attr_name, PyObject* value) {
    PyTypeObject* tp = Py_TYPE(obj);
    if (likely(tp->tp_setattro))
        return tp->tp_setattro(obj, attr_name, value);
    return PyObject_SetAttr(obj, attr_name, value);
}
#endif

/* PyLongCompare */
static CYTHON_INLINE int __Pyx_PyLong_BoolEqObjC(PyObject *op1, PyObject *op2, long intval, long inplace) {
    CYTHON_MAYBE_UNUSED_VAR(intval);
    CYTHON_UNUSED_VAR(inplace);
    if (op1 == op2) {
        return 1;
    }
    #if CYTHON_USE_PYLONG_INTERNALS
    if (likely(PyLong_CheckExact(op1))) {
        int unequal;
        unsigned long uintval;
        Py_ssize_t size = __Pyx_PyLong_DigitCount(op1);
        const digit* digits = __Pyx_PyLong_Digits(op1);
        if (intval == 0) {
            return (__Pyx_PyLong_IsZero(op1) == 1);
        } else if (intval < 0) {
            if (__Pyx_PyLong_IsNonNeg(op1))
                return 0;
            intval = -intval;
        } else {
            if (__Pyx_PyLong_IsNeg(op1))
                return 0;
        }
        uintval = (unsigned long) intval;
#if PyLong_SHIFT * 4 < SIZEOF_LONG*8
        if (uintval >> (PyLong_SHIFT * 4)) {
            unequal = (size != 5) || (digits[0] != (uintval & (unsigned long) PyLong_MASK))
                 | (digits[1] != ((uintval >> (1 * PyLong_SHIFT)) & (unsigned long) PyLong_MASK)) | (digits[2] != ((uintval >> (2 * PyLong_SHIFT)) & (unsigned long) PyLong_MASK)) | (digits[3] != ((uintval >> (3 * PyLong_SHIFT)) & (unsigned long) PyLong_MASK)) | (digits[4] != ((uintval >> (4 * PyLong_SHIFT)) & (unsigned long) PyLong_MASK));
        } else
#endif
#if PyLong_SHIFT * 3 < SIZEOF_LONG*8
        if (uintval >> (PyLong_SHIFT * 3)) {
            unequal = (size != 4) || (digits[0] != (uintval & (unsigned long) PyLong_MASK))
                 | (digits[1] != ((uintval >> (1 * PyLong_SHIFT)) & (unsigned long) PyLong_MASK)) | (digits[2] != ((uintval >> (2 * PyLong_SHIFT)) & (unsigned long) PyLong_MASK)) | (digits[3] != ((uintval >> (3 * PyLong_SHIFT)) & (unsigned long) PyLong_MASK));
        } else
#endif
#if PyLong_SHIFT * 2 < SIZEOF_LONG*8
        if (uintval >> (PyLong_SHIFT * 2)) {
            unequal = (size != 3) || (digits[0] != (uintval & (unsigned long) PyLong_MASK))
                 | (digits[1] != ((uintval >> (1 * PyLong_SHIFT)) & (unsigned long) PyLong_MASK)) | (digits[2] != ((uintval >> (2 * PyLong_SHIFT)) & (unsigned long) PyLong_MASK));
        } else
#endif
#if PyLong_SHIFT * 1 < SIZEOF_LONG*8
        if (uintval >> (PyLong_SHIFT * 1)) {
            unequal = (size != 2) || (digits[0] != (uintval & (unsigned long) PyLong_MASK))
                 | (digits[1] != ((uintval >> (1 * PyLong_SHIFT)) & (unsigned long) PyLong_MASK));
        } else
#endif
            unequal = (size != 1) || (((unsigned long) digits[0]) != (uintval & (unsigned long) PyLong_MASK));
        return (unequal == 0);
    }
    #endif
    if (PyFloat_CheckExact(op1)) {
        const long b = intval;
        double a = __Pyx_PyFloat_AS_DOUBLE(op1);
        return ((double)a == (double)b);
    }
    return __Pyx_PyObject_IsTrueAndDecref(
        PyObject_RichCompare(op1, op2, Py_EQ));
}

/* PyObjectFastCallMethod */
#if !CYTHON_VECTORCALL || PY_VERSION_HEX < 0x03090000
static PyObject *__Pyx_PyObject_FastCallMethod(PyObject *name, PyObject *const *args, size_t nargsf) {
    PyObject *result;
    PyObject *attr = PyObject_GetAttr(args[0], name);
    if (unlikely(!attr))
        return NULL;
    result = __Pyx_PyObject_FastCall(attr, args+1, nargsf - 1);
    Py_DECREF(attr);
    return result;
}
#endif

/* PyLongCompare */
static CYTHON_INLINE int __Pyx_PyLong_BoolNeObjC(PyObject *op1, PyObject *op2, long intval, long inplace) {
    CYTHON_MAYBE_UNUSED_VAR(intval);
    CYTHON_UNUSED_VAR(inplace);
    if (op1 == op2) {
        return 0;
    }
    #if CYTHON_USE_PYLONG_INTERNALS
    if (likely(PyLong_CheckExact(op1))) {
        int unequal;
        unsigned long uintval;
        Py_ssize_t size = __Pyx_PyLong_DigitCount(op1);
        const digit* digits = __Pyx_PyLong_Digits(op1);
        if (intval == 0) {
            return (__Pyx_PyLong_IsZero(op1) != 1);
        } else if (intval < 0) {
            if (__Pyx_PyLong_IsNonNeg(op1))
                return 1;
            intval = -intval;
        } else {
            if (__Pyx_PyLong_IsNeg(op1))
                return 1;
        }
        uintval = (unsigned long) intval;
#if PyLong_SHIFT * 4 < SIZEOF_LONG*8
        if (uintval >> (PyLong_SHIFT * 4)) {
            unequal = (size != 5) || (digits[0] != (uintval & (unsigned long) PyLong_MASK))
                 | (digits[1] != ((uintval >> (1 * PyLong_SHIFT)) & (unsigned long) PyLong_MASK)) | (digits[2] != ((uintval >> (2 * PyLong_SHIFT)) & (unsigned long) PyLong_MASK)) | (digits[3] != ((uintval >> (3 * PyLong_SHIFT)) & (unsigned long) PyLong_MASK)) | (digits[4] != ((uintval >> (4 * PyLong_SHIFT)) & (unsigned long) PyLong_MASK));
        } else
#endif
#if PyLong_SHIFT * 3 < SIZEOF_LONG*8
        if (uintval >> (PyLong_SHIFT * 3)) {
            unequal = (size != 4) || (digits[0] != (uintval & (unsigned long) PyLong_MASK))
                 | (digits[1] != ((uintval >> (1 * PyLong_SHIFT)) & (unsigned long) PyLong_MASK)) | (digits[2] != ((uintval >> (2 * PyLong_SHIFT)) & (unsigned long) PyLong_MASK)) | (digits[3] != ((uintval >> (3 * PyLong_SHIFT)) & (unsigned long) PyLong_MASK));
        } else
#endif
#if PyLong_SHIFT * 2 < SIZEOF_LONG*8
        if (uintval >> (PyLong_SHIFT * 2)) {
            unequal = (size != 3) || (digits[0] != (uintval & (unsigned long) PyLong_MASK))
                 | (digits[1] != ((uintval >> (1 * PyLong_SHIFT)) & (unsigned long) PyLong_MASK)) | (digits[2] != ((uintval >> (2 * PyLong_SHIFT)) & (unsigned long) PyLong_MASK));
        } else
#endif
#if PyLong_SHIFT * 1 < SIZEOF_LONG*8
        if (uintval >> (PyLong_SHIFT * 1)) {
            unequal = (size != 2) || (digits[0] != (uintval & (unsigned long) PyLong_MASK))
                 | (digits[1] != ((uintval >> (1 * PyLong_SHIFT)) & (unsigned long) PyLong_MASK));
        } else
#endif
            unequal = (size != 1) || (((unsigned long) digits[0]) != (uintval & (unsigned long) PyLong_MASK));
        return (unequal != 0);
    }
    #endif
    if (PyFloat_CheckExact(op1)) {
        const long b = intval;
        double a = __Pyx_PyFloat_AS_DOUBLE(op1);
        return ((double)a != (double)b);
    }
    return __Pyx_PyObject_IsTrueAndDecref(
        PyObject_RichCompare(op1, op2, Py_NE));
}

/* UnicodeConcatInPlace */
# if CYTHON_COMPILING_IN_CPYTHON
static int
__Pyx_unicode_modifiable(PyObject *unicode, int unsafe_shared)
{
    if (!__Pyx_IS_UNIQUELY_REFERENCED(unicode, unsafe_shared))
        return 0;
#if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX > 0x030F0000
    if (PyUnstable_Unicode_GET_CACHED_HASH(unicode) != -1)
        return 0;
#endif
    if (!PyUnicode_CheckExact(unicode))
        return 0;
    if (PyUnicode_CHECK_INTERNED(unicode))
        return 0;
    return 1;
}
static CYTHON_INLINE PyObject *__Pyx_PyUnicode_ConcatInPlaceImpl(PyObject **p_left, PyObject *right, int unsafe_shared
        #if CYTHON_REFNANNY
        , void* __pyx_refnanny
        #endif
    ) {
    PyObject *left = *p_left;
    Py_ssize_t left_len, right_len, new_len;
    if (unlikely(__Pyx_PyUnicode_READY(left) == -1))
        return NULL;
    if (unlikely(__Pyx_PyUnicode_READY(right) == -1))
        return NULL;
    left_len = PyUnicode_GET_LENGTH(left);
    if (left_len == 0) {
        Py_INCREF(right);
        return right;
    }
    right_len = PyUnicode_GET_LENGTH(right);
    if (right_len == 0) {
        Py_INCREF(left);
        return left;
    }
    if (unlikely(left_len > PY_SSIZE_T_MAX - right_len)) {
        PyErr_SetString(PyExc_OverflowError,
                        "strings are too large to concat");
        return NULL;
    }
    new_len = left_len + right_len;
    if (__Pyx_unicode_modifiable(left, unsafe_shared)
            && PyUnicode_CheckExact(right)
            && PyUnicode_KIND(right) <= PyUnicode_KIND(left)
            && !(PyUnicode_IS_ASCII(left) && !PyUnicode_IS_ASCII(right))) {
        int ret;
        __Pyx_GIVEREF(*p_left);
        ret = PyUnicode_Resize(p_left, new_len);
        __Pyx_GOTREF(*p_left);
        if (unlikely(ret != 0))
            return NULL;
        #if PY_VERSION_HEX >= 0x030d0000
        if (unlikely(PyUnicode_CopyCharacters(*p_left, left_len, right, 0, right_len) < 0)) return NULL;
        #else
        _PyUnicode_FastCopyCharacters(*p_left, left_len, right, 0, right_len);
        #endif
        __Pyx_INCREF(*p_left);
        __Pyx_GIVEREF(*p_left);
        return *p_left;
    } else {
        return __Pyx_PyUnicode_Concat(left, right);
    }
  }
#endif

/* PyDictVersioning (used by GetModuleGlobalName) */
#if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_TYPE_SLOTS
static CYTHON_INLINE PY_UINT64_T __Pyx_get_tp_dict_version(PyObject *obj) {
    PyObject *dict = Py_TYPE(obj)->tp_dict;
    return likely(dict) ? __PYX_GET_DICT_VERSION(dict) : 0;
}
static CYTHON_INLINE PY_UINT64_T __Pyx_get_object_dict_version(PyObject *obj) {
    PyObject **dictptr = NULL;
    Py_ssize_t offset = Py_TYPE(obj)->tp_dictoffset;
    if (offset) {
#if CYTHON_COMPILING_IN_CPYTHON
        dictptr = (likely(offset > 0)) ? (PyObject **) ((char *)obj + offset) : _PyObject_GetDictPtr(obj);
#else
        dictptr = _PyObject_GetDictPtr(obj);
#endif
    }
    return (dictptr && *dictptr) ? __PYX_GET_DICT_VERSION(*dictptr) : 0;
}
static CYTHON_INLINE int __Pyx_object_dict_version_matches(PyObject* obj, PY_UINT64_T tp_dict_version, PY_UINT64_T obj_dict_version) {
    PyObject *dict = Py_TYPE(obj)->tp_dict;
    if (unlikely(!dict) || unlikely(tp_dict_version != __PYX_GET_DICT_VERSION(dict)))
        return 0;
    return obj_dict_version == __Pyx_get_object_dict_version(obj);
}
#endif

/* GetModuleGlobalName */
#if CYTHON_USE_DICT_VERSIONS
static PyObject *__Pyx__GetModuleGlobalName(PyObject *name, PY_UINT64_T *dict_version, PyObject **dict_cached_value)
#else
static CYTHON_INLINE PyObject *__Pyx__GetModuleGlobalName(PyObject *name)
#endif
{
    PyObject *result;
#if CYTHON_COMPILING_IN_LIMITED_API
    if (unlikely(!__pyx_m)) {
        if (!PyErr_Occurred())
            PyErr_SetNone(PyExc_NameError);
        return NULL;
    }
    result = PyObject_GetAttr(__pyx_m, name);
    if (likely(result)) {
        return result;
    }
    PyErr_Clear();
#elif CYTHON_AVOID_BORROWED_REFS || CYTHON_AVOID_THREAD_UNSAFE_BORROWED_REFS
    if (unlikely(__Pyx_PyDict_GetItemRef(__pyx_mstate_global->__pyx_d, name, &result) == -1)) PyErr_Clear();
    __PYX_UPDATE_DICT_CACHE(__pyx_mstate_global->__pyx_d, result, *dict_cached_value, *dict_version)
    if (likely(result)) {
        return result;
    }
#else
    result = _PyDict_GetItem_KnownHash(__pyx_mstate_global->__pyx_d, name, ((PyASCIIObject *) name)->hash);
    __PYX_UPDATE_DICT_CACHE(__pyx_mstate_global->__pyx_d, result, *dict_cached_value, *dict_version)
    if (likely(result)) {
        return __Pyx_NewRef(result);
    }
    PyErr_Clear();
#endif
    return __Pyx_GetBuiltinName(name);
}

/* RejectKeywords */
static void __Pyx_RejectKeywords(const char* function_name, PyObject *kwds) {
    PyObject *key = NULL;
    if (CYTHON_METH_FASTCALL && likely(PyTuple_Check(kwds))) {
        key = __Pyx_PySequence_ITEM(kwds, 0);
    } else {
#if CYTHON_AVOID_BORROWED_REFS
        PyObject *pos = NULL;
#else
        Py_ssize_t pos = 0;
#endif
#if !CYTHON_COMPILING_IN_PYPY || defined(PyArg_ValidateKeywordArguments)
        if (unlikely(!PyArg_ValidateKeywordArguments(kwds))) return;
#endif
        __Pyx_PyDict_NextRef(kwds, &pos, &key, NULL);
#if CYTHON_AVOID_BORROWED_REFS
        Py_XDECREF(pos);
#endif
    }
    if (likely(key)) {
        PyErr_Format(PyExc_TypeError,
            "%s() got an unexpected keyword argument '%U'",
            function_name, key);
        Py_DECREF(key);
    }
}

/* GetAttr3 */
#if __PYX_LIMITED_VERSION_HEX < 0x030d0000
static PyObject *__Pyx_GetAttr3Default(PyObject *d) {
    __Pyx_PyThreadState_declare
    __Pyx_PyThreadState_assign
    if (unlikely(!__Pyx_PyErr_ExceptionMatches(PyExc_AttributeError)))
        return NULL;
    __Pyx_PyErr_Clear();
    Py_INCREF(d);
    return d;
}
#endif
static CYTHON_INLINE PyObject *__Pyx_GetAttr3(PyObject *o, PyObject *n, PyObject *d) {
    PyObject *r;
#if __PYX_LIMITED_VERSION_HEX >= 0x030d0000
    int res = PyObject_GetOptionalAttr(o, n, &r);
    return (res != 0) ? r : __Pyx_NewRef(d);
#else
  #if CYTHON_USE_TYPE_SLOTS
    if (likely(PyUnicode_Check(n))) {
        r = __Pyx_PyObject_GetAttrStrNoError(o, n);
        if (unlikely(!r) && likely(!PyErr_Occurred())) {
            r = __Pyx_NewRef(d);
        }
        return r;
    }
  #endif
    r = PyObject_GetAttr(o, n);
    return (likely(r)) ? r : __Pyx_GetAttr3Default(d);
#endif
}

/* RaiseUnexpectedTypeError */
static int
__Pyx_RaiseUnexpectedTypeError(const char *expected, PyObject *obj)
{
    __Pyx_TypeName obj_type_name = __Pyx_PyType_GetFullyQualifiedName(Py_TYPE(obj));
    PyErr_Format(PyExc_TypeError, "Expected %s, got " __Pyx_FMT_TYPENAME,
                 expected, obj_type_name);
    __Pyx_DECREF_TypeName(obj_type_name);
    return 0;
}

/* WriteUnraisableException */
static void __Pyx_WriteUnraisable(const char *name, int clineno,
                                  int lineno, const char *filename,
                                  int full_traceback, int nogil) {
    PyObject *old_exc, *old_val, *old_tb;
    PyObject *ctx;
    __Pyx_PyThreadState_declare
    PyGILState_STATE state;
    if (nogil)
        state = PyGILState_Ensure();
    else state = (PyGILState_STATE)0;
    CYTHON_UNUSED_VAR(clineno);
    CYTHON_UNUSED_VAR(lineno);
    CYTHON_UNUSED_VAR(filename);
    CYTHON_MAYBE_UNUSED_VAR(nogil);
    __Pyx_PyThreadState_assign
    __Pyx_ErrFetch(&old_exc, &old_val, &old_tb);
    if (full_traceback) {
        Py_XINCREF(old_exc);
        Py_XINCREF(old_val);
        Py_XINCREF(old_tb);
        __Pyx_ErrRestore(old_exc, old_val, old_tb);
        PyErr_PrintEx(0);
    }
    ctx = PyUnicode_FromString(name);
    __Pyx_ErrRestore(old_exc, old_val, old_tb);
    if (!ctx) {
        PyErr_WriteUnraisable(Py_None);
    } else {
        PyErr_WriteUnraisable(ctx);
        Py_DECREF(ctx);
    }
    if (nogil)
        PyGILState_Release(state);
}

/* RaiseException */
static void __Pyx_Raise(PyObject *type, PyObject *value, PyObject *tb, PyObject *cause) {
    PyObject* owned_instance = NULL;
    if (tb == Py_None) {
        tb = 0;
    } else if (tb && !PyTraceBack_Check(tb)) {
        PyErr_SetString(PyExc_TypeError,
            "raise: arg 3 must be a traceback or None");
        goto bad;
    }
    if (value == Py_None)
        value = 0;
    if (PyExceptionInstance_Check(type)) {
        if (value) {
            PyErr_SetString(PyExc_TypeError,
                "instance exception may not have a separate value");
            goto bad;
        }
        value = type;
        type = (PyObject*) Py_TYPE(value);
    } else if (PyExceptionClass_Check(type)) {
        PyObject *instance_class = NULL;
        if (value && PyExceptionInstance_Check(value)) {
            instance_class = (PyObject*) Py_TYPE(value);
            if (instance_class != type) {
                int is_subclass = PyObject_IsSubclass(instance_class, type);
                if (!is_subclass) {
                    instance_class = NULL;
                } else if (unlikely(is_subclass == -1)) {
                    goto bad;
                } else {
                    type = instance_class;
                }
            }
        }
        if (!instance_class) {
            PyObject *args;
            if (!value)
                args = PyTuple_New(0);
            else if (PyTuple_Check(value)) {
                Py_INCREF(value);
                args = value;
            } else
                args = PyTuple_Pack(1, value);
            if (!args)
                goto bad;
            owned_instance = PyObject_Call(type, args, NULL);
            Py_DECREF(args);
            if (!owned_instance)
                goto bad;
            value = owned_instance;
            if (!PyExceptionInstance_Check(value)) {
                PyErr_Format(PyExc_TypeError,
                             "calling %R should have returned an instance of "
                             "BaseException, not %R",
                             type, Py_TYPE(value));
                goto bad;
            }
        }
    } else {
        PyErr_SetString(PyExc_TypeError,
            "raise: exception class must be a subclass of BaseException");
        goto bad;
    }
    if (cause) {
        PyObject *fixed_cause;
        if (cause == Py_None) {
            fixed_cause = NULL;
        } else if (PyExceptionClass_Check(cause)) {
            fixed_cause = PyObject_CallObject(cause, NULL);
            if (fixed_cause == NULL)
                goto bad;
        } else if (PyExceptionInstance_Check(cause)) {
            fixed_cause = cause;
            Py_INCREF(fixed_cause);
        } else {
            PyErr_SetString(PyExc_TypeError,
                            "exception causes must derive from "
                            "BaseException");
            goto bad;
        }
        PyException_SetCause(value, fixed_cause);
    }
    PyErr_SetObject(type, value);
    if (tb) {
#if PY_VERSION_HEX >= 0x030C00A6
        PyException_SetTraceback(value, tb);
#elif CYTHON_FAST_THREAD_STATE
        PyThreadState *tstate = __Pyx_PyThreadState_Current;
        PyObject* tmp_tb = tstate->curexc_traceback;
        if (tb != tmp_tb) {
            Py_INCREF(tb);
            tstate->curexc_traceback = tb;
            Py_XDECREF(tmp_tb);
        }
#else
        PyObject *tmp_type, *tmp_value, *tmp_tb;
        PyErr_Fetch(&tmp_type, &tmp_value, &tmp_tb);
        Py_INCREF(tb);
        PyErr_Restore(tmp_type, tmp_value, tb);
        Py_XDECREF(tmp_tb);
#endif
    }
bad:
    Py_XDECREF(owned_instance);
    return;
}

/* GetTopmostException (used by SaveResetException) */
#if CYTHON_USE_EXC_INFO_STACK && CYTHON_FAST_THREAD_STATE
static _PyErr_StackItem *
__Pyx_PyErr_GetTopmostException(PyThreadState *tstate)
{
    _PyErr_StackItem *exc_info = tstate->exc_info;
    while ((exc_info->exc_value == NULL || exc_info->exc_value == Py_None) &&
           exc_info->previous_item != NULL)
    {
        exc_info = exc_info->previous_item;
    }
    return exc_info;
}
#endif

/* SaveResetException */
#if CYTHON_FAST_THREAD_STATE
static CYTHON_INLINE void __Pyx__ExceptionSave(PyThreadState *tstate, PyObject **type, PyObject **value, PyObject **tb) {
  #if CYTHON_USE_EXC_INFO_STACK && PY_VERSION_HEX >= 0x030B00a4
    _PyErr_StackItem *exc_info = __Pyx_PyErr_GetTopmostException(tstate);
    PyObject *exc_value = exc_info->exc_value;
    if (exc_value == NULL || exc_value == Py_None) {
        *value = NULL;
        *type = NULL;
        *tb = NULL;
    } else {
        *value = exc_value;
        Py_INCREF(*value);
        *type = (PyObject*) Py_TYPE(exc_value);
        Py_INCREF(*type);
        *tb = PyException_GetTraceback(exc_value);
    }
  #elif CYTHON_USE_EXC_INFO_STACK
    _PyErr_StackItem *exc_info = __Pyx_PyErr_GetTopmostException(tstate);
    *type = exc_info->exc_type;
    *value = exc_info->exc_value;
    *tb = exc_info->exc_traceback;
    Py_XINCREF(*type);
    Py_XINCREF(*value);
    Py_XINCREF(*tb);
  #else
    *type = tstate->exc_type;
    *value = tstate->exc_value;
    *tb = tstate->exc_traceback;
    Py_XINCREF(*type);
    Py_XINCREF(*value);
    Py_XINCREF(*tb);
  #endif
}
static CYTHON_INLINE void __Pyx__ExceptionReset(PyThreadState *tstate, PyObject *type, PyObject *value, PyObject *tb) {
  #if CYTHON_USE_EXC_INFO_STACK && PY_VERSION_HEX >= 0x030B00a4
    _PyErr_StackItem *exc_info = tstate->exc_info;
    PyObject *tmp_value = exc_info->exc_value;
    exc_info->exc_value = value;
    Py_XDECREF(tmp_value);
    Py_XDECREF(type);
    Py_XDECREF(tb);
  #else
    PyObject *tmp_type, *tmp_value, *tmp_tb;
    #if CYTHON_USE_EXC_INFO_STACK
    _PyErr_StackItem *exc_info = tstate->exc_info;
    tmp_type = exc_info->exc_type;
    tmp_value = exc_info->exc_value;
    tmp_tb = exc_info->exc_traceback;
    exc_info->exc_type = type;
    exc_info->exc_value = value;
    exc_info->exc_traceback = tb;
    #else
    tmp_type = tstate->exc_type;
    tmp_value = tstate->exc_value;
    tmp_tb = tstate->exc_traceback;
    tstate->exc_type = type;
    tstate->exc_value = value;
    tstate->exc_traceback = tb;
    #endif
    Py_XDECREF(tmp_type);
    Py_XDECREF(tmp_value);
    Py_XDECREF(tmp_tb);
  #endif
}
#endif

/* GetException */
#if CYTHON_FAST_THREAD_STATE
static int __Pyx__GetException(PyThreadState *tstate, PyObject **type, PyObject **value, PyObject **tb)
#else
static int __Pyx_GetException(PyObject **type, PyObject **value, PyObject **tb)
#endif
{
    PyObject *local_type = NULL, *local_value, *local_tb = NULL;
#if CYTHON_FAST_THREAD_STATE
    PyObject *tmp_type, *tmp_value, *tmp_tb;
  #if PY_VERSION_HEX >= 0x030C0000
    local_value = tstate->current_exception;
    tstate->current_exception = 0;
  #else
    local_type = tstate->curexc_type;
    local_value = tstate->curexc_value;
    local_tb = tstate->curexc_traceback;
    tstate->curexc_type = 0;
    tstate->curexc_value = 0;
    tstate->curexc_traceback = 0;
  #endif
#elif __PYX_LIMITED_VERSION_HEX > 0x030C0000
    local_value = PyErr_GetRaisedException();
#else
    PyErr_Fetch(&local_type, &local_value, &local_tb);
#endif
#if __PYX_LIMITED_VERSION_HEX > 0x030C0000
    if (likely(local_value)) {
        local_type = (PyObject*) Py_TYPE(local_value);
        Py_INCREF(local_type);
        local_tb = PyException_GetTraceback(local_value);
    }
#else
    PyErr_NormalizeException(&local_type, &local_value, &local_tb);
#if CYTHON_FAST_THREAD_STATE
    if (unlikely(tstate->curexc_type))
#else
    if (unlikely(PyErr_Occurred()))
#endif
        goto bad;
    if (local_tb) {
        if (unlikely(PyException_SetTraceback(local_value, local_tb) < 0))
            goto bad;
    }
#endif // __PYX_LIMITED_VERSION_HEX > 0x030C0000
    Py_XINCREF(local_tb);
    Py_XINCREF(local_type);
    Py_XINCREF(local_value);
    *type = local_type;
    *value = local_value;
    *tb = local_tb;
#if CYTHON_FAST_THREAD_STATE
    #if CYTHON_USE_EXC_INFO_STACK
    {
        _PyErr_StackItem *exc_info = tstate->exc_info;
      #if PY_VERSION_HEX >= 0x030B00a4
        tmp_value = exc_info->exc_value;
        exc_info->exc_value = local_value;
        tmp_type = NULL;
        tmp_tb = NULL;
        Py_XDECREF(local_type);
        Py_XDECREF(local_tb);
      #else
        tmp_type = exc_info->exc_type;
        tmp_value = exc_info->exc_value;
        tmp_tb = exc_info->exc_traceback;
        exc_info->exc_type = local_type;
        exc_info->exc_value = local_value;
        exc_info->exc_traceback = local_tb;
      #endif
    }
    #else
    tmp_type = tstate->exc_type;
    tmp_value = tstate->exc_value;
    tmp_tb = tstate->exc_traceback;
    tstate->exc_type = local_type;
    tstate->exc_value = local_value;
    tstate->exc_traceback = local_tb;
    #endif
    Py_XDECREF(tmp_type);
    Py_XDECREF(tmp_value);
    Py_XDECREF(tmp_tb);
#elif __PYX_LIMITED_VERSION_HEX >= 0x030b0000
    PyErr_SetHandledException(local_value);
    Py_XDECREF(local_value);
    Py_XDECREF(local_type);
    Py_XDECREF(local_tb);
#else
    PyErr_SetExcInfo(local_type, local_value, local_tb);
#endif
    return 0;
#if __PYX_LIMITED_VERSION_HEX <= 0x030C0000
bad:
    *type = 0;
    *value = 0;
    *tb = 0;
    Py_XDECREF(local_type);
    Py_XDECREF(local_value);
    Py_XDECREF(local_tb);
    return -1;
#endif
}

/* ArgTypeTestFunc (used by ArgTypeTest) */
static int __Pyx__ArgTypeTest(PyObject *obj, PyTypeObject *type, const char *name, int exact)
{
    __Pyx_TypeName type_name;
    __Pyx_TypeName obj_type_name;
    PyObject *extra_info = __pyx_mstate_global->__pyx_empty_unicode;
    int from_annotation_subclass = 0;
    if (unlikely(!type)) {
        PyErr_SetString(PyExc_SystemError, "Missing type object");
        return 0;
    }
    else if (!exact) {
        if (likely(__Pyx_TypeCheck(obj, type))) return 1;
    } else if (exact == 2) {
        if (__Pyx_TypeCheck(obj, type)) {
            from_annotation_subclass = 1;
            extra_info = __pyx_mstate_global->__pyx_kp_u_Note_that_Cython_is_deliberately;
        }
    }
    type_name = __Pyx_PyType_GetFullyQualifiedName(type);
    obj_type_name = __Pyx_PyType_GetFullyQualifiedName(Py_TYPE(obj));
    PyErr_Format(PyExc_TypeError,
        "Argument '%.200s' has incorrect type (expected " __Pyx_FMT_TYPENAME
        ", got " __Pyx_FMT_TYPENAME ")"
#if __PYX_LIMITED_VERSION_HEX < 0x030C0000
        "%s%U"
#endif
        , name, type_name, obj_type_name
#if __PYX_LIMITED_VERSION_HEX < 0x030C0000
        , (from_annotation_subclass ? ". " : ""), extra_info
#endif
        );
#if __PYX_LIMITED_VERSION_HEX >= 0x030C0000
    if (exact == 2 && from_annotation_subclass) {
        PyObject *res;
        PyObject *vargs[2];
        vargs[0] = PyErr_GetRaisedException();
        vargs[1] = extra_info;
        res = PyObject_VectorcallMethod(__pyx_mstate_global->__pyx_kp_u_add_note, vargs, 2, NULL);
        Py_XDECREF(res);
        PyErr_SetRaisedException(vargs[0]);
    }
#endif
    __Pyx_DECREF_TypeName(type_name);
    __Pyx_DECREF_TypeName(obj_type_name);
    return 0;
}

/* RaiseTooManyValuesToUnpack */
static CYTHON_INLINE void __Pyx_RaiseTooManyValuesError(Py_ssize_t expected) {
    PyErr_Format(PyExc_ValueError,
                 "too many values to unpack (expected %" CYTHON_FORMAT_SSIZE_T "d)", expected);
}

/* RaiseNeedMoreValuesToUnpack */
static CYTHON_INLINE void __Pyx_RaiseNeedMoreValuesError(Py_ssize_t index) {
    PyErr_Format(PyExc_ValueError,
                 "need more than %" CYTHON_FORMAT_SSIZE_T "d value%.1s to unpack",
                 index, (index == 1) ? "" : "s");
}

/* ExtTypeTest */
static CYTHON_INLINE int __Pyx_TypeTest(PyObject *obj, PyTypeObject *type) {
    __Pyx_TypeName obj_type_name;
    __Pyx_TypeName type_name;
    if (unlikely(!type)) {
        PyErr_SetString(PyExc_SystemError, "Missing type object");
        return 0;
    }
    if (likely(__Pyx_TypeCheck(obj, type)))
        return 1;
    obj_type_name = __Pyx_PyType_GetFullyQualifiedName(Py_TYPE(obj));
    type_name = __Pyx_PyType_GetFullyQualifiedName(type);
    PyErr_Format(PyExc_TypeError,
                 "Cannot convert " __Pyx_FMT_TYPENAME " to " __Pyx_FMT_TYPENAME,
                 obj_type_name, type_name);
    __Pyx_DECREF_TypeName(obj_type_name);
    __Pyx_DECREF_TypeName(type_name);
    return 0;
}

/* PyUnicode_Unicode */
static CYTHON_INLINE PyObject* __Pyx_PyUnicode_Unicode(PyObject *obj) {
    if (unlikely(obj == Py_None))
        obj = __pyx_mstate_global->__pyx_kp_u_None;
    return __Pyx_NewRef(obj);
}

/* PyObjectFormatAndDecref */
static CYTHON_INLINE PyObject* __Pyx_PyObject_FormatSimpleAndDecref(PyObject* s, PyObject* f) {
    if (unlikely(!s)) return NULL;
    if (likely(PyUnicode_CheckExact(s))) return s;
    return __Pyx_PyObject_FormatAndDecref(s, f);
}
static CYTHON_INLINE PyObject* __Pyx_PyObject_FormatAndDecref(PyObject* s, PyObject* f) {
    PyObject *result;
    if (unlikely(!s)) return NULL;
    result = PyObject_Format(s, f);
    Py_DECREF(s);
    return result;
}

/* CIntToDigits (used by CIntToPyUnicode) */
static const char DIGIT_PAIRS_10[2*10*10+1] = {
    "00010203040506070809"
    "10111213141516171819"
    "20212223242526272829"
    "30313233343536373839"
    "40414243444546474849"
    "50515253545556575859"
    "60616263646566676869"
    "70717273747576777879"
    "80818283848586878889"
    "90919293949596979899"
};
static const char DIGIT_PAIRS_8[2*8*8+1] = {
    "0001020304050607"
    "1011121314151617"
    "2021222324252627"
    "3031323334353637"
    "4041424344454647"
    "5051525354555657"
    "6061626364656667"
    "7071727374757677"
};
static const char DIGITS_HEX[2*16+1] = {
    "0123456789abcdef"
    "0123456789ABCDEF"
};

/* BuildPyUnicode (used by COrdinalToPyUnicode) */
static PyObject* __Pyx_PyUnicode_BuildFromAscii(Py_ssize_t ulength, const char* chars, int clength,
                                                int prepend_sign, char padding_char) {
    PyObject *uval;
    Py_ssize_t uoffset = ulength - clength;
#if CYTHON_USE_UNICODE_INTERNALS
    Py_ssize_t i;
    void *udata;
    uval = PyUnicode_New(ulength, 127);
    if (unlikely(!uval)) return NULL;
    udata = PyUnicode_DATA(uval);
    if (uoffset > 0) {
        i = 0;
        if (prepend_sign) {
            __Pyx_PyUnicode_WRITE(PyUnicode_1BYTE_KIND, udata, 0, '-');
            i++;
        }
        for (; i < uoffset; i++) {
            __Pyx_PyUnicode_WRITE(PyUnicode_1BYTE_KIND, udata, i, padding_char);
        }
    }
    for (i=0; i < clength; i++) {
        __Pyx_PyUnicode_WRITE(PyUnicode_1BYTE_KIND, udata, uoffset+i, chars[i]);
    }
#else
    {
        PyObject *sign = NULL, *padding = NULL;
        uval = NULL;
        if (uoffset > 0) {
            prepend_sign = !!prepend_sign;
            if (uoffset > prepend_sign) {
                padding = PyUnicode_FromOrdinal(padding_char);
                if (likely(padding) && uoffset > prepend_sign + 1) {
                    PyObject *tmp = PySequence_Repeat(padding, uoffset - prepend_sign);
                    Py_DECREF(padding);
                    padding = tmp;
                }
                if (unlikely(!padding)) goto done_or_error;
            }
            if (prepend_sign) {
                sign = PyUnicode_FromOrdinal('-');
                if (unlikely(!sign)) goto done_or_error;
            }
        }
        uval = PyUnicode_DecodeASCII(chars, clength, NULL);
        if (likely(uval) && padding) {
            PyObject *tmp = PyUnicode_Concat(padding, uval);
            Py_DECREF(uval);
            uval = tmp;
        }
        if (likely(uval) && sign) {
            PyObject *tmp = PyUnicode_Concat(sign, uval);
            Py_DECREF(uval);
            uval = tmp;
        }
done_or_error:
        Py_XDECREF(padding);
        Py_XDECREF(sign);
    }
#endif
    return uval;
}

/* COrdinalToPyUnicode (used by CIntToPyUnicode) */
static CYTHON_INLINE int __Pyx_CheckUnicodeValue(int value) {
    return value <= 1114111;
}
static PyObject* __Pyx_PyUnicode_FromOrdinal_Padded(int value, Py_ssize_t ulength, char padding_char) {
    Py_ssize_t padding_length = ulength - 1;
    if (likely((padding_length <= 250) && (value < 0xD800 || value > 0xDFFF))) {
        char chars[256];
        if (value <= 255) {
            memset(chars, padding_char, (size_t) padding_length);
            chars[ulength-1] = (char) value;
            return PyUnicode_DecodeLatin1(chars, ulength, NULL);
        }
        char *cpos = chars + sizeof(chars);
        if (value < 0x800) {
            *--cpos = (char) (0x80 | (value & 0x3f));
            value >>= 6;
            *--cpos = (char) (0xc0 | (value & 0x1f));
        } else if (value < 0x10000) {
            *--cpos = (char) (0x80 | (value & 0x3f));
            value >>= 6;
            *--cpos = (char) (0x80 | (value & 0x3f));
            value >>= 6;
            *--cpos = (char) (0xe0 | (value & 0x0f));
        } else {
            *--cpos = (char) (0x80 | (value & 0x3f));
            value >>= 6;
            *--cpos = (char) (0x80 | (value & 0x3f));
            value >>= 6;
            *--cpos = (char) (0x80 | (value & 0x3f));
            value >>= 6;
            *--cpos = (char) (0xf0 | (value & 0x07));
        }
        cpos -= padding_length;
        memset(cpos, padding_char, (size_t) padding_length);
        return PyUnicode_DecodeUTF8(cpos, chars + sizeof(chars) - cpos, NULL);
    }
    if (value <= 127 && CYTHON_USE_UNICODE_INTERNALS) {
        const char chars[1] = {(char) value};
        return __Pyx_PyUnicode_BuildFromAscii(ulength, chars, 1, 0, padding_char);
    }
    {
        PyObject *uchar, *padding_uchar, *padding, *result;
        padding_uchar = PyUnicode_FromOrdinal(padding_char);
        if (unlikely(!padding_uchar)) return NULL;
        padding = PySequence_Repeat(padding_uchar, padding_length);
        Py_DECREF(padding_uchar);
        if (unlikely(!padding)) return NULL;
        uchar = PyUnicode_FromOrdinal(value);
        if (unlikely(!uchar)) {
            Py_DECREF(padding);
            return NULL;
        }
        result = PyUnicode_Concat(padding, uchar);
        Py_DECREF(padding);
        Py_DECREF(uchar);
        return result;
    }
}

/* CIntToPyUnicode */
static CYTHON_INLINE PyObject* __Pyx_uchar___Pyx_PyUnicode_From_ptrdiff_t(ptrdiff_t value, Py_ssize_t width, char padding_char) {
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic push
#pragma GCC diagnostic ignored "-Wconversion"
#endif
    const ptrdiff_t neg_one = (ptrdiff_t) -1, const_zero = (ptrdiff_t) 0;
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic pop
#endif
    const int is_unsigned = neg_one > const_zero;
    if (unlikely(!(is_unsigned || value == 0 || value > 0) ||
                    !(sizeof(value) <= 2 || value & ~ (ptrdiff_t) 0x01fffff || __Pyx_CheckUnicodeValue((int) value)))) {
        PyErr_SetString(PyExc_OverflowError, "%c arg not in range(0x110000)");
        return NULL;
    }
    if (width <= 1) {
        return PyUnicode_FromOrdinal((int) value);
    }
    return __Pyx_PyUnicode_FromOrdinal_Padded((int) value, width, padding_char);
}
static CYTHON_INLINE PyObject* __Pyx____Pyx_PyUnicode_From_ptrdiff_t(ptrdiff_t value, Py_ssize_t width, char padding_char, char format_char) {
    char digits[sizeof(ptrdiff_t)*3+2];
    char *dpos, *end = digits + sizeof(ptrdiff_t)*3+2;
    const char *hex_digits = DIGITS_HEX;
    Py_ssize_t length, ulength;
    int prepend_sign, last_one_off;
    ptrdiff_t remaining;
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic push
#pragma GCC diagnostic ignored "-Wconversion"
#endif
    const ptrdiff_t neg_one = (ptrdiff_t) -1, const_zero = (ptrdiff_t) 0;
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic pop
#endif
    const int is_unsigned = neg_one > const_zero;
    if (format_char == 'X') {
        hex_digits += 16;
        format_char = 'x';
    }
    remaining = value;
    last_one_off = 0;
    dpos = end;
    do {
        int digit_pos;
        switch (format_char) {
        case 'o':
            digit_pos = abs((int)(remaining % (8*8)));
            remaining = (ptrdiff_t) (remaining / (8*8));
            dpos -= 2;
            memcpy(dpos, DIGIT_PAIRS_8 + digit_pos * 2, 2);
            last_one_off = (digit_pos < 8);
            break;
        case 'd':
            digit_pos = abs((int)(remaining % (10*10)));
            remaining = (ptrdiff_t) (remaining / (10*10));
            dpos -= 2;
            memcpy(dpos, DIGIT_PAIRS_10 + digit_pos * 2, 2);
            last_one_off = (digit_pos < 10);
            break;
        case 'x':
            *(--dpos) = hex_digits[abs((int)(remaining % 16))];
            remaining = (ptrdiff_t) (remaining / 16);
            break;
        default:
            assert(0);
            break;
        }
    } while (unlikely(remaining != 0));
    assert(!last_one_off || *dpos == '0');
    dpos += last_one_off;
    length = end - dpos;
    ulength = length;
    prepend_sign = 0;
    if (!is_unsigned && value <= neg_one) {
        if (padding_char == ' ' || width <= length + 1) {
            *(--dpos) = '-';
            ++length;
        } else {
            prepend_sign = 1;
        }
        ++ulength;
    }
    if (width > ulength) {
        ulength = width;
    }
    if (ulength == 1) {
        return PyUnicode_FromOrdinal(*dpos);
    }
    return __Pyx_PyUnicode_BuildFromAscii(ulength, dpos, (int) length, prepend_sign, padding_char);
}

/* JoinPyUnicode */
static PyObject* __Pyx_PyUnicode_Join(PyObject** values, Py_ssize_t value_count, Py_ssize_t result_ulength,
                                      Py_UCS4 max_char) {
#if CYTHON_USE_UNICODE_INTERNALS && CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
    PyObject *result_uval;
    int result_ukind, kind_shift;
    Py_ssize_t i, char_pos;
    void *result_udata;
    if (max_char > 1114111) max_char = 1114111;
    result_uval = PyUnicode_New(result_ulength, max_char);
    if (unlikely(!result_uval)) return NULL;
    result_ukind = (max_char <= 255) ? PyUnicode_1BYTE_KIND : (max_char <= 65535) ? PyUnicode_2BYTE_KIND : PyUnicode_4BYTE_KIND;
    kind_shift = (result_ukind == PyUnicode_4BYTE_KIND) ? 2 : result_ukind - 1;
    result_udata = PyUnicode_DATA(result_uval);
    assert(kind_shift == 2 || kind_shift == 1 || kind_shift == 0);
    if (unlikely((PY_SSIZE_T_MAX >> kind_shift) - result_ulength < 0))
        goto overflow;
    char_pos = 0;
    for (i=0; i < value_count; i++) {
        int ukind;
        Py_ssize_t ulength;
        void *udata;
        PyObject *uval = values[i];
        #if !CYTHON_COMPILING_IN_LIMITED_API
        if (__Pyx_PyUnicode_READY(uval) == (-1))
            goto bad;
        #endif
        ulength = __Pyx_PyUnicode_GET_LENGTH(uval);
        #if !CYTHON_ASSUME_SAFE_SIZE
        if (unlikely(ulength < 0)) goto bad;
        #endif
        if (unlikely(!ulength))
            continue;
        if (unlikely((PY_SSIZE_T_MAX >> kind_shift) - ulength < char_pos))
            goto overflow;
        ukind = __Pyx_PyUnicode_KIND(uval);
        udata = __Pyx_PyUnicode_DATA(uval);
        if (ukind == result_ukind) {
            memcpy((char *)result_udata + (char_pos << kind_shift), udata, (size_t) (ulength << kind_shift));
        } else {
            #if PY_VERSION_HEX >= 0x030d0000
            if (unlikely(PyUnicode_CopyCharacters(result_uval, char_pos, uval, 0, ulength) < 0)) goto bad;
            #elif CYTHON_COMPILING_IN_CPYTHON || defined(_PyUnicode_FastCopyCharacters)
            _PyUnicode_FastCopyCharacters(result_uval, char_pos, uval, 0, ulength);
            #else
            Py_ssize_t j;
            for (j=0; j < ulength; j++) {
                Py_UCS4 uchar = __Pyx_PyUnicode_READ(ukind, udata, j);
                __Pyx_PyUnicode_WRITE(result_ukind, result_udata, char_pos+j, uchar);
            }
            #endif
        }
        char_pos += ulength;
    }
    return result_uval;
overflow:
    PyErr_SetString(PyExc_OverflowError, "join() result is too long for a Python string");
bad:
    Py_DECREF(result_uval);
    return NULL;
#else
    Py_ssize_t i;
    PyObject *result = NULL;
    PyObject *value_tuple = PyTuple_New(value_count);
    if (unlikely(!value_tuple)) return NULL;
    CYTHON_UNUSED_VAR(max_char);
    CYTHON_UNUSED_VAR(result_ulength);
    for (i=0; i<value_count; i++) {
        Py_INCREF(values[i]);
        if (__Pyx_PyTuple_SET_ITEM(value_tuple, i, values[i]) != (0)) goto bad;
    }
    result = PyUnicode_Join(__pyx_mstate_global->__pyx_empty_unicode, value_tuple);
bad:
    Py_DECREF(value_tuple);
    return result;
#endif
}

/* IterFinish (used by dict_iter) */
static CYTHON_INLINE int __Pyx_IterFinish(void) {
    PyObject* exc_type;
    __Pyx_PyThreadState_declare
    __Pyx_PyThreadState_assign
    exc_type = __Pyx_PyErr_CurrentExceptionType();
    if (unlikely(exc_type)) {
        if (unlikely(!__Pyx_PyErr_GivenExceptionMatches(exc_type, PyExc_StopIteration)))
            return -1;
        __Pyx_PyErr_Clear();
        return 0;
    }
    return 0;
}

/* PyObjectCallNoArg (used by PyObjectCallMethod0) */
static CYTHON_INLINE PyObject* __Pyx_PyObject_CallNoArg(PyObject *func) {
    PyObject *arg[2] = {NULL, NULL};
    return __Pyx_PyObject_FastCall(func, arg + 1, 0 | __Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET);
}

/* PyObjectGetMethod (used by PyObjectCallMethod0) */
#if !(CYTHON_VECTORCALL && (__PYX_LIMITED_VERSION_HEX >= 0x030C0000 || (!CYTHON_COMPILING_IN_LIMITED_API && PY_VERSION_HEX >= 0x03090000)))
static int __Pyx_PyObject_GetMethod(PyObject *obj, PyObject *name, PyObject **method) {
    PyObject *attr;
#if CYTHON_UNPACK_METHODS && CYTHON_COMPILING_IN_CPYTHON && CYTHON_USE_PYTYPE_LOOKUP
    __Pyx_TypeName type_name;
    PyTypeObject *tp = Py_TYPE(obj);
    PyObject *descr;
    descrgetfunc f = NULL;
    PyObject **dictptr, *dict;
    int meth_found = 0;
    assert (*method == NULL);
    if (unlikely(tp->tp_getattro != PyObject_GenericGetAttr)) {
        attr = __Pyx_PyObject_GetAttrStr(obj, name);
        goto try_unpack;
    }
    if (unlikely(tp->tp_dict == NULL) && unlikely(PyType_Ready(tp) < 0)) {
        return 0;
    }
    descr = _PyType_Lookup(tp, name);
    if (likely(descr != NULL)) {
        Py_INCREF(descr);
#if defined(Py_TPFLAGS_METHOD_DESCRIPTOR) && Py_TPFLAGS_METHOD_DESCRIPTOR
        if (__Pyx_PyType_HasFeature(Py_TYPE(descr), Py_TPFLAGS_METHOD_DESCRIPTOR))
#else
        #ifdef __Pyx_CyFunction_USED
        if (likely(PyFunction_Check(descr) || __Pyx_IS_TYPE(descr, &PyMethodDescr_Type) || __Pyx_CyFunction_Check(descr)))
        #else
        if (likely(PyFunction_Check(descr) || __Pyx_IS_TYPE(descr, &PyMethodDescr_Type)))
        #endif
#endif
        {
            meth_found = 1;
        } else {
            f = Py_TYPE(descr)->tp_descr_get;
            if (f != NULL && PyDescr_IsData(descr)) {
                attr = f(descr, obj, (PyObject *)Py_TYPE(obj));
                Py_DECREF(descr);
                goto try_unpack;
            }
        }
    }
    dictptr = _PyObject_GetDictPtr(obj);
    if (dictptr != NULL && (dict = *dictptr) != NULL) {
        Py_INCREF(dict);
        attr = __Pyx_PyDict_GetItemStr(dict, name);
        if (attr != NULL) {
            Py_INCREF(attr);
            Py_DECREF(dict);
            Py_XDECREF(descr);
            goto try_unpack;
        }
        Py_DECREF(dict);
    }
    if (meth_found) {
        *method = descr;
        return 1;
    }
    if (f != NULL) {
        attr = f(descr, obj, (PyObject *)Py_TYPE(obj));
        Py_DECREF(descr);
        goto try_unpack;
    }
    if (likely(descr != NULL)) {
        *method = descr;
        return 0;
    }
    type_name = __Pyx_PyType_GetFullyQualifiedName(tp);
    PyErr_Format(PyExc_AttributeError,
                 "'" __Pyx_FMT_TYPENAME "' object has no attribute '%U'",
                 type_name, name);
    __Pyx_DECREF_TypeName(type_name);
    return 0;
#else
    attr = __Pyx_PyObject_GetAttrStr(obj, name);
    goto try_unpack;
#endif
try_unpack:
#if CYTHON_UNPACK_METHODS
    if (likely(attr) && PyMethod_Check(attr) && likely(PyMethod_GET_SELF(attr) == obj)) {
        PyObject *function = PyMethod_GET_FUNCTION(attr);
        Py_INCREF(function);
        Py_DECREF(attr);
        *method = function;
        return 1;
    }
#endif
    *method = attr;
    return 0;
}
#endif

/* PyObjectCallMethod0 (used by dict_iter) */
static PyObject* __Pyx_PyObject_CallMethod0(PyObject* obj, PyObject* method_name) {
#if CYTHON_VECTORCALL && (__PYX_LIMITED_VERSION_HEX >= 0x030C0000 || (!CYTHON_COMPILING_IN_LIMITED_API && PY_VERSION_HEX >= 0x03090000))
    PyObject *args[1] = {obj};
    (void) __Pyx_PyObject_CallOneArg;
    (void) __Pyx_PyObject_CallNoArg;
    return PyObject_VectorcallMethod(method_name, args, 1 | PY_VECTORCALL_ARGUMENTS_OFFSET, NULL);
#else
    PyObject *method = NULL, *result = NULL;
    int is_method = __Pyx_PyObject_GetMethod(obj, method_name, &method);
    if (likely(is_method)) {
        result = __Pyx_PyObject_CallOneArg(method, obj);
        Py_DECREF(method);
        return result;
    }
    if (unlikely(!method)) goto bad;
    result = __Pyx_PyObject_CallNoArg(method);
    Py_DECREF(method);
bad:
    return result;
#endif
}

/* UnpackItemEndCheck (used by UnpackTuple2) */
static int __Pyx_IternextUnpackEndCheck(PyObject *retval, Py_ssize_t expected) {
    if (unlikely(retval)) {
        Py_DECREF(retval);
        __Pyx_RaiseTooManyValuesError(expected);
        return -1;
    }
    return __Pyx_IterFinish();
}

/* RaiseNoneIterError (used by UnpackTupleError) */
static CYTHON_INLINE void __Pyx_RaiseNoneNotIterableError(void) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not iterable");
}

/* UnpackTupleError (used by UnpackTuple2) */
static void __Pyx_UnpackTupleError(PyObject *t, Py_ssize_t index) {
    if (t == Py_None) {
      __Pyx_RaiseNoneNotIterableError();
    } else {
      Py_ssize_t size = __Pyx_PyTuple_GET_SIZE(t);
 #if !CYTHON_ASSUME_SAFE_SIZE
      if (unlikely(size < 0)) return;
 #endif
      if (size < index) {
        __Pyx_RaiseNeedMoreValuesError(size);
      } else {
        __Pyx_RaiseTooManyValuesError(index);
      }
    }
}

/* UnpackTuple2 (used by dict_iter) */
static CYTHON_INLINE int __Pyx_unpack_tuple2(
        PyObject* tuple, PyObject** value1, PyObject** value2, int is_tuple, int has_known_size, int decref_tuple) {
    if (likely(is_tuple || PyTuple_Check(tuple))) {
        Py_ssize_t size;
        if (has_known_size) {
            return __Pyx_unpack_tuple2_exact(tuple, value1, value2, decref_tuple);
        }
        size = __Pyx_PyTuple_GET_SIZE(tuple);
        if (likely(size == 2)) {
            return __Pyx_unpack_tuple2_exact(tuple, value1, value2, decref_tuple);
        }
        if (size >= 0) {
            __Pyx_UnpackTupleError(tuple, 2);
        }
        return -1;
    } else {
        return __Pyx_unpack_tuple2_generic(tuple, value1, value2, has_known_size, decref_tuple);
    }
}
static CYTHON_INLINE int __Pyx_unpack_tuple2_exact(
        PyObject* tuple, PyObject** pvalue1, PyObject** pvalue2, int decref_tuple) {
    PyObject *value1 = NULL, *value2 = NULL;
#if CYTHON_AVOID_BORROWED_REFS || !CYTHON_ASSUME_SAFE_MACROS
    value1 = __Pyx_PySequence_ITEM(tuple, 0);  if (unlikely(!value1)) goto bad;
    value2 = __Pyx_PySequence_ITEM(tuple, 1);  if (unlikely(!value2)) goto bad;
#else
    value1 = PyTuple_GET_ITEM(tuple, 0);  Py_INCREF(value1);
    value2 = PyTuple_GET_ITEM(tuple, 1);  Py_INCREF(value2);
#endif
    if (decref_tuple) {
        Py_DECREF(tuple);
    }
    *pvalue1 = value1;
    *pvalue2 = value2;
    return 0;
#if CYTHON_AVOID_BORROWED_REFS || !CYTHON_ASSUME_SAFE_MACROS
bad:
    Py_XDECREF(value1);
    Py_XDECREF(value2);
    if (decref_tuple) { Py_XDECREF(tuple); }
    return -1;
#endif
}
static int __Pyx_unpack_tuple2_generic(PyObject* tuple, PyObject** pvalue1, PyObject** pvalue2,
                                       int has_known_size, int decref_tuple) {
    Py_ssize_t index;
    PyObject *value1 = NULL, *value2 = NULL, *iter = NULL;
    iternextfunc iternext;
    iter = PyObject_GetIter(tuple);
    if (unlikely(!iter)) goto bad;
    if (decref_tuple) { Py_DECREF(tuple); tuple = NULL; }
    iternext = __Pyx_PyObject_GetIterNextFunc(iter);
    value1 = iternext(iter); if (unlikely(!value1)) { index = 0; goto unpacking_failed; }
    value2 = iternext(iter); if (unlikely(!value2)) { index = 1; goto unpacking_failed; }
    if (!has_known_size && unlikely(__Pyx_IternextUnpackEndCheck(iternext(iter), 2))) goto bad;
    Py_DECREF(iter);
    *pvalue1 = value1;
    *pvalue2 = value2;
    return 0;
unpacking_failed:
    if (!has_known_size && __Pyx_IterFinish() == 0)
        __Pyx_RaiseNeedMoreValuesError(index);
bad:
    Py_XDECREF(iter);
    Py_XDECREF(value1);
    Py_XDECREF(value2);
    if (decref_tuple) { Py_XDECREF(tuple); }
    return -1;
}

/* dict_iter */
#if CYTHON_AVOID_BORROWED_REFS
#include <string.h>
#endif
static CYTHON_INLINE PyObject* __Pyx_dict_iterator(PyObject* iterable, int is_dict, PyObject* method_name,
                                                   Py_ssize_t* p_orig_length, int* p_source_is_dict) {
    is_dict = is_dict || likely(PyDict_CheckExact(iterable));
    *p_source_is_dict = is_dict;
    if (is_dict) {
#if !CYTHON_AVOID_BORROWED_REFS
        *p_orig_length = PyDict_Size(iterable);
        Py_INCREF(iterable);
        return iterable;
#else
        static PyObject *py_items = NULL, *py_keys = NULL, *py_values = NULL;
        PyObject **pp = NULL;
        if (method_name) {
            const char *name = PyUnicode_AsUTF8(method_name);
            if (strcmp(name, "iteritems") == 0) pp = &py_items;
            else if (strcmp(name, "iterkeys") == 0) pp = &py_keys;
            else if (strcmp(name, "itervalues") == 0) pp = &py_values;
            if (pp) {
                if (!*pp) {
                    *pp = PyUnicode_FromString(name + 4);
                    if (!*pp)
                        return NULL;
                }
                method_name = *pp;
            }
        }
#endif
    }
    *p_orig_length = 0;
    if (method_name) {
        PyObject* iter;
        iterable = __Pyx_PyObject_CallMethod0(iterable, method_name);
        if (!iterable)
            return NULL;
#if !CYTHON_AVOID_BORROWED_REFS
        if (PyTuple_CheckExact(iterable) || PyList_CheckExact(iterable))
            return iterable;
#endif
        iter = PyObject_GetIter(iterable);
        Py_DECREF(iterable);
        return iter;
    }
    return PyObject_GetIter(iterable);
}
#if !CYTHON_AVOID_BORROWED_REFS
static CYTHON_INLINE int __Pyx_dict_iter_next_source_is_dict(
        PyObject* iter_obj, CYTHON_NCP_UNUSED Py_ssize_t orig_length, CYTHON_NCP_UNUSED Py_ssize_t* ppos,
        PyObject** pkey, PyObject** pvalue, PyObject** pitem) {
    PyObject *key, *value;
    if (unlikely(orig_length != PyDict_Size(iter_obj))) {
        PyErr_SetString(PyExc_RuntimeError, "dictionary changed size during iteration");
        return -1;
    }
    if (unlikely(!PyDict_Next(iter_obj, ppos, &key, &value))) {
        return 0;
    }
    if (pitem) {
        PyObject* tuple = PyTuple_New(2);
        if (unlikely(!tuple)) {
            return -1;
        }
        Py_INCREF(key);
        Py_INCREF(value);
        #if CYTHON_ASSUME_SAFE_MACROS
        PyTuple_SET_ITEM(tuple, 0, key);
        PyTuple_SET_ITEM(tuple, 1, value);
        #else
        if (unlikely(PyTuple_SetItem(tuple, 0, key) < 0)) {
            Py_DECREF(value);
            Py_DECREF(tuple);
            return -1;
        }
        if (unlikely(PyTuple_SetItem(tuple, 1, value) < 0)) {
            Py_DECREF(tuple);
            return -1;
        }
        #endif
        *pitem = tuple;
    } else {
        if (pkey) {
            Py_INCREF(key);
            *pkey = key;
        }
        if (pvalue) {
            Py_INCREF(value);
            *pvalue = value;
        }
    }
    return 1;
}
#endif
static CYTHON_INLINE int __Pyx_dict_iter_next(
        PyObject* iter_obj, CYTHON_NCP_UNUSED Py_ssize_t orig_length, CYTHON_NCP_UNUSED Py_ssize_t* ppos,
        PyObject** pkey, PyObject** pvalue, PyObject** pitem, int source_is_dict) {
    PyObject* next_item;
#if !CYTHON_AVOID_BORROWED_REFS
    if (source_is_dict) {
        int result;
#if PY_VERSION_HEX >= 0x030d0000 && !CYTHON_COMPILING_IN_LIMITED_API
        Py_BEGIN_CRITICAL_SECTION(iter_obj);
#endif
        result = __Pyx_dict_iter_next_source_is_dict(iter_obj, orig_length, ppos, pkey, pvalue, pitem);
#if PY_VERSION_HEX >= 0x030d0000 && !CYTHON_COMPILING_IN_LIMITED_API
        Py_END_CRITICAL_SECTION();
#endif
        return result;
    } else if (PyTuple_CheckExact(iter_obj)) {
        Py_ssize_t pos = *ppos;
        Py_ssize_t tuple_size = __Pyx_PyTuple_GET_SIZE(iter_obj);
        #if !CYTHON_ASSUME_SAFE_SIZE
        if (unlikely(tuple_size < 0)) return -1;
        #endif
        if (unlikely(pos >= tuple_size)) return 0;
        *ppos = pos + 1;
        #if CYTHON_ASSUME_SAFE_MACROS
        next_item = PyTuple_GET_ITEM(iter_obj, pos);
        #else
        next_item = PyTuple_GetItem(iter_obj, pos);
        if (unlikely(!next_item)) return -1;
        #endif
        Py_INCREF(next_item);
    } else if (PyList_CheckExact(iter_obj)) {
        Py_ssize_t pos = *ppos;
        Py_ssize_t list_size = __Pyx_PyList_GET_SIZE(iter_obj);
        #if !CYTHON_ASSUME_SAFE_SIZE
        if (unlikely(list_size < 0)) return -1;
        #endif
        if (unlikely(pos >= list_size)) return 0;
        *ppos = pos + 1;
        next_item = __Pyx_PyList_GetItemRef(iter_obj, pos);
        if (unlikely(!next_item)) return -1;
    } else
#endif
    {
        next_item = PyIter_Next(iter_obj);
        if (unlikely(!next_item)) {
            return __Pyx_IterFinish();
        }
    }
    if (pitem) {
        *pitem = next_item;
    } else if (pkey && pvalue) {
        if (__Pyx_unpack_tuple2(next_item, pkey, pvalue, source_is_dict, source_is_dict, 1))
            return -1;
    } else if (pkey) {
        *pkey = next_item;
    } else {
        *pvalue = next_item;
    }
    return 1;
}

/* PyObjectVectorCallKwBuilder (used by PyObjectVectorCallMethodKwBuilder) */
#if CYTHON_VECTORCALL
static int __Pyx_VectorcallBuilder_AddArg(PyObject *key, PyObject *value, PyObject *builder, PyObject **args, int n) {
    (void)__Pyx_PyObject_FastCallDict;
    Py_INCREF(key);
    if (__Pyx_PyTuple_SET_ITEM(builder, n, key) != (0)) return -1;
    args[n] = value;
    return 0;
}
CYTHON_UNUSED static int __Pyx_VectorcallBuilder_AddArg_Check(PyObject *key, PyObject *value, PyObject *builder, PyObject **args, int n) {
    (void)__Pyx_VectorcallBuilder_AddArgStr;
    if (unlikely(!PyUnicode_Check(key))) {
        PyErr_SetString(PyExc_TypeError, "keywords must be strings");
        return -1;
    }
    return __Pyx_VectorcallBuilder_AddArg(key, value, builder, args, n);
}
static int __Pyx_VectorcallBuilder_AddArgStr(const char *key, PyObject *value, PyObject *builder, PyObject **args, int n) {
    PyObject *pyKey = PyUnicode_FromString(key);
    if (!pyKey) return -1;
    return __Pyx_VectorcallBuilder_AddArg(pyKey, value, builder, args, n);
}
#else // CYTHON_VECTORCALL
CYTHON_UNUSED static int __Pyx_VectorcallBuilder_AddArg_Check(PyObject *key, PyObject *value, PyObject *builder, CYTHON_UNUSED PyObject **args, CYTHON_UNUSED int n) {
    if (unlikely(!PyUnicode_Check(key))) {
        PyErr_SetString(PyExc_TypeError, "keywords must be strings");
        return -1;
    }
    return PyDict_SetItem(builder, key, value);
}
#endif

/* PyObjectVectorCallMethodKwBuilder */
#if !CYTHON_VECTORCALL || PY_VERSION_HEX < 0x03090000
static PyObject *__Pyx_Object_VectorcallMethod_CallFromBuilder(PyObject *name, PyObject *const *args, size_t nargsf, PyObject *kwnames) {
    PyObject *result;
    PyObject *obj = PyObject_GetAttr(args[0], name);
    if (unlikely(!obj))
        return NULL;
    result = __Pyx_Object_Vectorcall_CallFromBuilder(obj, args+1, nargsf-1, kwnames);
    Py_DECREF(obj);
    return result;
}
#endif

/* SwapException */
#if CYTHON_FAST_THREAD_STATE
static CYTHON_INLINE void __Pyx__ExceptionSwap(PyThreadState *tstate, PyObject **type, PyObject **value, PyObject **tb) {
    PyObject *tmp_type, *tmp_value, *tmp_tb;
  #if CYTHON_USE_EXC_INFO_STACK && PY_VERSION_HEX >= 0x030B00a4
    _PyErr_StackItem *exc_info = tstate->exc_info;
    tmp_value = exc_info->exc_value;
    exc_info->exc_value = *value;
    if (tmp_value == NULL || tmp_value == Py_None) {
        Py_XDECREF(tmp_value);
        tmp_value = NULL;
        tmp_type = NULL;
        tmp_tb = NULL;
    } else {
        tmp_type = (PyObject*) Py_TYPE(tmp_value);
        Py_INCREF(tmp_type);
        #if CYTHON_COMPILING_IN_CPYTHON
        tmp_tb = ((PyBaseExceptionObject*) tmp_value)->traceback;
        Py_XINCREF(tmp_tb);
        #else
        tmp_tb = PyException_GetTraceback(tmp_value);
        #endif
    }
  #elif CYTHON_USE_EXC_INFO_STACK
    _PyErr_StackItem *exc_info = tstate->exc_info;
    tmp_type = exc_info->exc_type;
    tmp_value = exc_info->exc_value;
    tmp_tb = exc_info->exc_traceback;
    exc_info->exc_type = *type;
    exc_info->exc_value = *value;
    exc_info->exc_traceback = *tb;
  #else
    tmp_type = tstate->exc_type;
    tmp_value = tstate->exc_value;
    tmp_tb = tstate->exc_traceback;
    tstate->exc_type = *type;
    tstate->exc_value = *value;
    tstate->exc_traceback = *tb;
  #endif
    *type = tmp_type;
    *value = tmp_value;
    *tb = tmp_tb;
}
#else
static CYTHON_INLINE void __Pyx_ExceptionSwap(PyObject **type, PyObject **value, PyObject **tb) {
    PyObject *tmp_type, *tmp_value, *tmp_tb;
    PyErr_GetExcInfo(&tmp_type, &tmp_value, &tmp_tb);
    PyErr_SetExcInfo(*type, *value, *tb);
    *type = tmp_type;
    *value = tmp_value;
    *tb = tmp_tb;
}
#endif

/* pop */
static CYTHON_INLINE PyObject* __Pyx__PyObject_Pop(PyObject* L) {
    if (__Pyx_IS_TYPE(L, &PySet_Type)) {
        return PySet_Pop(L);
    }
    return __Pyx_PyObject_CallMethod0(L, __pyx_mstate_global->__pyx_n_u_pop);
}
#if CYTHON_USE_PYLIST_INTERNALS && CYTHON_ASSUME_SAFE_MACROS && CYTHON_ASSUME_SAFE_SIZE
static CYTHON_INLINE PyObject* __Pyx_PyList_Pop(PyObject* L) {
    if (likely(PyList_GET_SIZE(L) > (((PyListObject*)L)->allocated >> 1))) {
        __Pyx_SET_SIZE(L, Py_SIZE(L) - 1);
        return PyList_GET_ITEM(L, PyList_GET_SIZE(L));
    }
    return __Pyx_CallUnboundCMethod0(&__pyx_mstate_global->__pyx_umethod_PyList_Type_pop, L);
}
#endif

/* pybytes_as_double (used by pyobject_as_double) */
static double __Pyx_SlowPyString_AsDouble(PyObject *obj) {
    PyObject *float_value = PyFloat_FromString(obj);
    if (likely(float_value)) {
        double value = __Pyx_PyFloat_AS_DOUBLE(float_value);
        Py_DECREF(float_value);
        return value;
    }
    return (double)-1;
}
static const char* __Pyx__PyBytes_AsDouble_Copy(const char* start, char* buffer, Py_ssize_t length) {
    int last_was_punctuation = 1;
    int parse_error_found = 0;
    Py_ssize_t i;
    for (i=0; i < length; i++) {
        char chr = start[i];
        int is_punctuation = (chr == '_') | (chr == '.') | (chr == 'e') | (chr == 'E');
        *buffer = chr;
        buffer += (chr != '_');
        parse_error_found |= last_was_punctuation & is_punctuation;
        last_was_punctuation = is_punctuation;
    }
    parse_error_found |= last_was_punctuation;
    *buffer = '\0';
    return unlikely(parse_error_found) ? NULL : buffer;
}
static double __Pyx__PyBytes_AsDouble_inf_nan(const char* start, Py_ssize_t length) {
    int matches = 1;
    char sign = start[0];
    int is_signed = (sign == '+') | (sign == '-');
    start += is_signed;
    length -= is_signed;
    switch (start[0]) {
        #ifdef Py_NAN
        case 'n':
        case 'N':
            if (unlikely(length != 3)) goto parse_failure;
            matches &= (start[1] == 'a' || start[1] == 'A');
            matches &= (start[2] == 'n' || start[2] == 'N');
            if (unlikely(!matches)) goto parse_failure;
            return (sign == '-') ? -Py_NAN : Py_NAN;
        #endif
        case 'i':
        case 'I':
            if (unlikely(length < 3)) goto parse_failure;
            matches &= (start[1] == 'n' || start[1] == 'N');
            matches &= (start[2] == 'f' || start[2] == 'F');
            if (likely(length == 3 && matches))
                return (sign == '-') ? -Py_HUGE_VAL : Py_HUGE_VAL;
            if (unlikely(length != 8)) goto parse_failure;
            matches &= (start[3] == 'i' || start[3] == 'I');
            matches &= (start[4] == 'n' || start[4] == 'N');
            matches &= (start[5] == 'i' || start[5] == 'I');
            matches &= (start[6] == 't' || start[6] == 'T');
            matches &= (start[7] == 'y' || start[7] == 'Y');
            if (unlikely(!matches)) goto parse_failure;
            return (sign == '-') ? -Py_HUGE_VAL : Py_HUGE_VAL;
        case '.': case '0': case '1': case '2': case '3': case '4': case '5': case '6': case '7': case '8': case '9':
            break;
        default:
            goto parse_failure;
    }
    return 0.0;
parse_failure:
    return -1.0;
}
static CYTHON_INLINE int __Pyx__PyBytes_AsDouble_IsSpace(char ch) {
    return (ch == 0x20) | !((ch < 0x9) | (ch > 0xd));
}
CYTHON_UNUSED static double __Pyx__PyBytes_AsDouble(PyObject *obj, const char* start, Py_ssize_t length) {
    double value;
    Py_ssize_t i, digits;
    const char *last = start + length;
    char *end;
    while (__Pyx__PyBytes_AsDouble_IsSpace(*start))
        start++;
    while (start < last - 1 && __Pyx__PyBytes_AsDouble_IsSpace(last[-1]))
        last--;
    length = last - start;
    if (unlikely(length <= 0)) goto fallback;
    value = __Pyx__PyBytes_AsDouble_inf_nan(start, length);
    if (unlikely(value == -1.0)) goto fallback;
    if (value != 0.0) return value;
    digits = 0;
    for (i=0; i < length; digits += start[i++] != '_');
    if (likely(digits == length)) {
        value = PyOS_string_to_double(start, &end, NULL);
    } else if (digits < 40) {
        char number[40];
        last = __Pyx__PyBytes_AsDouble_Copy(start, number, length);
        if (unlikely(!last)) goto fallback;
        value = PyOS_string_to_double(number, &end, NULL);
    } else {
        char *number = (char*) PyMem_Malloc((digits + 1) * sizeof(char));
        if (unlikely(!number)) goto fallback;
        last = __Pyx__PyBytes_AsDouble_Copy(start, number, length);
        if (unlikely(!last)) {
            PyMem_Free(number);
            goto fallback;
        }
        value = PyOS_string_to_double(number, &end, NULL);
        PyMem_Free(number);
    }
    if (likely(end == last) || (value == (double)-1 && PyErr_Occurred())) {
        return value;
    }
fallback:
    return __Pyx_SlowPyString_AsDouble(obj);
}

/* pyobject_as_double */
static double __Pyx__PyObject_AsDouble(PyObject* obj) {
    if (PyUnicode_CheckExact(obj)) {
        return __Pyx_PyUnicode_AsDouble(obj);
    } else if (PyBytes_CheckExact(obj)) {
        return __Pyx_PyBytes_AsDouble(obj);
    } else if (PyByteArray_CheckExact(obj)) {
        return __Pyx_PyByteArray_AsDouble(obj);
    } else {
        PyObject* float_value;
#if !CYTHON_USE_TYPE_SLOTS
        float_value = PyNumber_Float(obj);  if ((0)) goto bad;
        (void)__Pyx_PyObject_CallOneArg;
#else
        PyNumberMethods *nb = Py_TYPE(obj)->tp_as_number;
        if (likely(nb) && likely(nb->nb_float)) {
            float_value = nb->nb_float(obj);
            if (likely(float_value) && unlikely(!PyFloat_Check(float_value))) {
                __Pyx_TypeName float_value_type_name = __Pyx_PyType_GetFullyQualifiedName(Py_TYPE(float_value));
                PyErr_Format(PyExc_TypeError,
                    "__float__ returned non-float (type " __Pyx_FMT_TYPENAME ")",
                    float_value_type_name);
                __Pyx_DECREF_TypeName(float_value_type_name);
                Py_DECREF(float_value);
                goto bad;
            }
        } else {
            float_value = __Pyx_PyObject_CallOneArg((PyObject*)&PyFloat_Type, obj);
        }
#endif
        if (likely(float_value)) {
            double value = __Pyx_PyFloat_AS_DOUBLE(float_value);
            Py_DECREF(float_value);
            return value;
        }
    }
bad:
    return (double)-1;
}

/* SliceObject */
static CYTHON_INLINE PyObject* __Pyx_PyObject_GetSlice(PyObject* obj,
        Py_ssize_t cstart, Py_ssize_t cstop,
        PyObject** _py_start, PyObject** _py_stop, PyObject** _py_slice,
        int has_cstart, int has_cstop, CYTHON_UNUSED int wraparound) {
    __Pyx_TypeName obj_type_name;
#if CYTHON_USE_TYPE_SLOTS
    PyMappingMethods* mp = Py_TYPE(obj)->tp_as_mapping;
    if (likely(mp && mp->mp_subscript))
#endif
    {
        PyObject* result;
        PyObject *py_slice, *py_start, *py_stop;
        if (_py_slice) {
            py_slice = *_py_slice;
        } else {
            PyObject* owned_start = NULL;
            PyObject* owned_stop = NULL;
            if (_py_start) {
                py_start = *_py_start;
            } else {
                if (has_cstart) {
                    owned_start = py_start = PyLong_FromSsize_t(cstart);
                    if (unlikely(!py_start)) goto bad;
                } else
                    py_start = Py_None;
            }
            if (_py_stop) {
                py_stop = *_py_stop;
            } else {
                if (has_cstop) {
                    owned_stop = py_stop = PyLong_FromSsize_t(cstop);
                    if (unlikely(!py_stop)) {
                        Py_XDECREF(owned_start);
                        goto bad;
                    }
                } else
                    py_stop = Py_None;
            }
            py_slice = PySlice_New(py_start, py_stop, Py_None);
            Py_XDECREF(owned_start);
            Py_XDECREF(owned_stop);
            if (unlikely(!py_slice)) goto bad;
        }
#if CYTHON_USE_TYPE_SLOTS
        result = mp->mp_subscript(obj, py_slice);
#else
        result = PyObject_GetItem(obj, py_slice);
#endif
        if (!_py_slice) {
            Py_DECREF(py_slice);
        }
        return result;
    }
    obj_type_name = __Pyx_PyType_GetFullyQualifiedName(Py_TYPE(obj));
    PyErr_Format(PyExc_TypeError,
        "'" __Pyx_FMT_TYPENAME "' object is unsliceable", obj_type_name);
    __Pyx_DECREF_TypeName(obj_type_name);
bad:
    return NULL;
}

/* PyObjectCall2Args (used by CallUnboundCMethod1) */
static CYTHON_INLINE PyObject* __Pyx_PyObject_Call2Args(PyObject* function, PyObject* arg1, PyObject* arg2) {
    PyObject *args[3] = {NULL, arg1, arg2};
    return __Pyx_PyObject_FastCall(function, args+1, 2 | __Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET);
}

/* CallUnboundCMethod1 */
#if CYTHON_COMPILING_IN_CPYTHON
static CYTHON_INLINE PyObject* __Pyx_CallUnboundCMethod1(__Pyx_CachedCFunction* cfunc, PyObject* self, PyObject* arg) {
    int was_initialized =  __Pyx_CachedCFunction_GetAndSetInitializing(cfunc);
    if (likely(was_initialized == 2 && cfunc->func)) {
        int flag = cfunc->flag;
        if (flag == METH_O) {
            return __Pyx_CallCFunction(cfunc, self, arg);
        } else if (flag == METH_FASTCALL) {
            return __Pyx_CallCFunctionFast(cfunc, self, &arg, 1);
        } else if (flag == (METH_FASTCALL | METH_KEYWORDS)) {
            return __Pyx_CallCFunctionFastWithKeywords(cfunc, self, &arg, 1, NULL);
        }
    }
#if CYTHON_COMPILING_IN_CPYTHON_FREETHREADING
    else if (unlikely(was_initialized == 1)) {
        __Pyx_CachedCFunction tmp_cfunc = {
#ifndef __cplusplus
            0
#endif
        };
        tmp_cfunc.type = cfunc->type;
        tmp_cfunc.method_name = cfunc->method_name;
        return __Pyx__CallUnboundCMethod1(&tmp_cfunc, self, arg);
    }
#endif
    PyObject* result = __Pyx__CallUnboundCMethod1(cfunc, self, arg);
    __Pyx_CachedCFunction_SetFinishedInitializing(cfunc);
    return result;
}
#endif
static PyObject* __Pyx__CallUnboundCMethod1(__Pyx_CachedCFunction* cfunc, PyObject* self, PyObject* arg){
    PyObject *result = NULL;
    if (unlikely(!cfunc->func && !cfunc->method) && unlikely(__Pyx_TryUnpackUnboundCMethod(cfunc) < 0)) return NULL;
#if CYTHON_COMPILING_IN_CPYTHON
    if (cfunc->func && (cfunc->flag & METH_VARARGS)) {
        PyObject *args = PyTuple_New(1);
        if (unlikely(!args)) return NULL;
        Py_INCREF(arg);
        PyTuple_SET_ITEM(args, 0, arg);
        if (cfunc->flag & METH_KEYWORDS)
            result = __Pyx_CallCFunctionWithKeywords(cfunc, self, args, NULL);
        else
            result = __Pyx_CallCFunction(cfunc, self, args);
        Py_DECREF(args);
    } else
#endif
    {
        result = __Pyx_PyObject_Call2Args(cfunc->method, self, arg);
    }
    return result;
}

/* dict_getitem_default */
static PyObject* __Pyx_PyDict_GetItemDefault(PyObject* d, PyObject* key, PyObject* default_value) {
    PyObject* value;
#if !CYTHON_COMPILING_IN_PYPY || PYPY_VERSION_NUM >= 0x07020000
    value = PyDict_GetItemWithError(d, key);
    if (unlikely(!value)) {
        if (unlikely(PyErr_Occurred()))
            return NULL;
        value = default_value;
    }
    Py_INCREF(value);
    if ((1));
#else
    if (PyBytes_CheckExact(key) || PyUnicode_CheckExact(key) || PyLong_CheckExact(key)) {
        value = PyDict_GetItem(d, key);
        if (unlikely(!value)) {
            value = default_value;
        }
        Py_INCREF(value);
    }
#endif
    else {
        if (default_value == Py_None)
            value = __Pyx_CallUnboundCMethod1(&__pyx_mstate_global->__pyx_umethod_PyDict_Type_get, d, key);
        else
            value = __Pyx_CallUnboundCMethod2(&__pyx_mstate_global->__pyx_umethod_PyDict_Type_get, d, key, default_value);
    }
    return value;
}

/* DictGetItem */
#if !CYTHON_COMPILING_IN_PYPY
static PyObject *__Pyx_PyDict_GetItem(PyObject *d, PyObject* key) {
    PyObject *value;
    if (unlikely(__Pyx_PyDict_GetItemRef(d, key, &value) == 0)) { // no value, no error
        if (unlikely(PyTuple_Check(key))) {
            PyObject* args = PyTuple_Pack(1, key);
            if (likely(args)) {
                PyErr_SetObject(PyExc_KeyError, args);
                Py_DECREF(args);
            }
        } else {
            PyErr_SetObject(PyExc_KeyError, key);
        }
    }
    return value;
}
#endif

/* py_dict_keys */
static CYTHON_INLINE PyObject* __Pyx_PyDict_Keys(PyObject* d) {
    return __Pyx_CallUnboundCMethod0(&__pyx_mstate_global->__pyx_umethod_PyDict_Type_keys, d);
}

/* FastTypeChecks */
#if CYTHON_COMPILING_IN_CPYTHON
static int __Pyx_InBases(PyTypeObject *a, PyTypeObject *b) {
    while (a) {
        a = __Pyx_PyType_GetSlot(a, tp_base, PyTypeObject*);
        if (a == b)
            return 1;
    }
    return b == &PyBaseObject_Type;
}
static CYTHON_INLINE int __Pyx_IsSubtype(PyTypeObject *a, PyTypeObject *b) {
    PyObject *mro;
    if (a == b) return 1;
    mro = a->tp_mro;
    if (likely(mro)) {
        Py_ssize_t i, n;
        n = PyTuple_GET_SIZE(mro);
        for (i = 0; i < n; i++) {
            if (PyTuple_GET_ITEM(mro, i) == (PyObject *)b)
                return 1;
        }
        return 0;
    }
    return __Pyx_InBases(a, b);
}
static CYTHON_INLINE int __Pyx_IsAnySubtype2(PyTypeObject *cls, PyTypeObject *a, PyTypeObject *b) {
    PyObject *mro;
    if (cls == a || cls == b) return 1;
    mro = cls->tp_mro;
    if (likely(mro)) {
        Py_ssize_t i, n;
        n = PyTuple_GET_SIZE(mro);
        for (i = 0; i < n; i++) {
            PyObject *base = PyTuple_GET_ITEM(mro, i);
            if (base == (PyObject *)a || base == (PyObject *)b)
                return 1;
        }
        return 0;
    }
    return __Pyx_InBases(cls, a) || __Pyx_InBases(cls, b);
}
static CYTHON_INLINE int __Pyx_inner_PyErr_GivenExceptionMatches2(PyObject *err, PyObject* exc_type1, PyObject *exc_type2) {
    if (exc_type1) {
        return __Pyx_IsAnySubtype2((PyTypeObject*)err, (PyTypeObject*)exc_type1, (PyTypeObject*)exc_type2);
    } else {
        return __Pyx_IsSubtype((PyTypeObject*)err, (PyTypeObject*)exc_type2);
    }
}
static int __Pyx_PyErr_GivenExceptionMatchesTuple(PyObject *exc_type, PyObject *tuple) {
    Py_ssize_t i, n;
    assert(PyExceptionClass_Check(exc_type));
    n = PyTuple_GET_SIZE(tuple);
    for (i=0; i<n; i++) {
        if (exc_type == PyTuple_GET_ITEM(tuple, i)) return 1;
    }
    for (i=0; i<n; i++) {
        PyObject *t = PyTuple_GET_ITEM(tuple, i);
        if (likely(PyExceptionClass_Check(t))) {
            if (__Pyx_inner_PyErr_GivenExceptionMatches2(exc_type, NULL, t)) return 1;
        } else {
        }
    }
    return 0;
}
static CYTHON_INLINE int __Pyx_PyErr_GivenExceptionMatches(PyObject *err, PyObject* exc_type) {
    if (likely(err == exc_type)) return 1;
    if (likely(PyExceptionClass_Check(err))) {
        if (likely(PyExceptionClass_Check(exc_type))) {
            return __Pyx_inner_PyErr_GivenExceptionMatches2(err, NULL, exc_type);
        } else if (likely(PyTuple_Check(exc_type))) {
            return __Pyx_PyErr_GivenExceptionMatchesTuple(err, exc_type);
        } else {
        }
    }
    return PyErr_GivenExceptionMatches(err, exc_type);
}
static CYTHON_INLINE int __Pyx_PyErr_GivenExceptionMatches2(PyObject *err, PyObject *exc_type1, PyObject *exc_type2) {
    assert(PyExceptionClass_Check(exc_type1));
    assert(PyExceptionClass_Check(exc_type2));
    if (likely(err == exc_type1 || err == exc_type2)) return 1;
    if (likely(PyExceptionClass_Check(err))) {
        return __Pyx_inner_PyErr_GivenExceptionMatches2(err, exc_type1, exc_type2);
    }
    return (PyErr_GivenExceptionMatches(err, exc_type1) || PyErr_GivenExceptionMatches(err, exc_type2));
}
#endif

/* RaiseClosureNameError */
static void __Pyx_RaiseClosureNameError(const char *varname) {
    PyErr_Format(PyExc_NameError, "free variable '%s' referenced before assignment in enclosing scope", varname);
}

/* pep479 */
static void __Pyx_Generator_Replace_StopIteration(int in_async_gen) {
    PyObject *exc, *val, *tb, *cur_exc, *new_exc;
    __Pyx_PyThreadState_declare
    int is_async_stopiteration = 0;
    CYTHON_MAYBE_UNUSED_VAR(in_async_gen);
    __Pyx_PyThreadState_assign
    cur_exc = __Pyx_PyErr_CurrentExceptionType();
    if (likely(!__Pyx_PyErr_GivenExceptionMatches(cur_exc, PyExc_StopIteration))) {
        if (in_async_gen && unlikely(__Pyx_PyErr_GivenExceptionMatches(cur_exc, PyExc_StopAsyncIteration))) {
            is_async_stopiteration = 1;
        } else {
            return;
        }
    }
    __Pyx_GetException(&exc, &val, &tb);
    Py_XDECREF(exc);
    Py_XDECREF(tb);
    new_exc = PyObject_CallFunction(PyExc_RuntimeError, "s",
        is_async_stopiteration ? "async generator raised StopAsyncIteration" :
        in_async_gen ? "async generator raised StopIteration" :
        "generator raised StopIteration");
    if (!new_exc) {
        Py_XDECREF(val);
        return;
    }
    PyException_SetCause(new_exc, val); // steals ref to val
    PyErr_SetObject(PyExc_RuntimeError, new_exc);
    Py_DECREF(new_exc);
}

/* RaiseUnboundLocalError */
static void __Pyx_RaiseUnboundLocalError(const char *varname) {
    PyErr_Format(PyExc_UnboundLocalError, "local variable '%s' referenced before assignment", varname);
}

/* AllocateExtensionType */
static PyObject *__Pyx_AllocateExtensionType(PyTypeObject *t, int is_final) {
    if (is_final || likely(!__Pyx_PyType_HasFeature(t, Py_TPFLAGS_IS_ABSTRACT))) {
        allocfunc alloc_func = __Pyx_PyType_GetSlot(t, tp_alloc, allocfunc);
        return alloc_func(t, 0);
    } else {
        newfunc tp_new = __Pyx_PyType_TryGetSlot(&PyBaseObject_Type, tp_new, newfunc);
    #if CYTHON_COMPILING_IN_LIMITED_API && __PYX_LIMITED_VERSION_HEX < 0x030A0000
        if (!tp_new) {
            PyObject *new_str = PyUnicode_FromString("__new__");
            if (likely(new_str)) {
                PyObject *o = PyObject_CallMethodObjArgs((PyObject *)&PyBaseObject_Type, new_str, t, NULL);
                Py_DECREF(new_str);
                return o;
            } else
                return NULL;
        } else
    #endif
        return tp_new(t, __pyx_mstate_global->__pyx_empty_tuple, 0);
    }
}

/* CallTypeTraverse */
#if !CYTHON_USE_TYPE_SPECS || (!CYTHON_COMPILING_IN_LIMITED_API && PY_VERSION_HEX < 0x03090000)
#else
static int __Pyx_call_type_traverse(PyObject *o, int always_call, visitproc visit, void *arg) {
    #if CYTHON_COMPILING_IN_LIMITED_API && __PYX_LIMITED_VERSION_HEX < 0x03090000
    if (__Pyx_get_runtime_version() < 0x03090000) return 0;
    #endif
    if (!always_call) {
        PyTypeObject *base = __Pyx_PyObject_GetSlot(o, tp_base, PyTypeObject*);
        unsigned long flags = PyType_GetFlags(base);
        if (flags & Py_TPFLAGS_HEAPTYPE) {
            return 0;
        }
    }
    Py_VISIT((PyObject*)Py_TYPE(o));
    return 0;
}
#endif

/* CallNextTpTraverse */
static int __Pyx_call_next_tp_traverse(PyObject* obj, visitproc v, void *a, traverseproc current_tp_traverse) {
    PyTypeObject* type = Py_TYPE(obj);
    traverseproc tp_traverse = NULL;
    while (type && __Pyx_PyType_GetSlot(type, tp_traverse, traverseproc) != current_tp_traverse)
        type = __Pyx_PyType_GetSlot(type, tp_base, PyTypeObject*);
    while (type && (tp_traverse = __Pyx_PyType_GetSlot(type, tp_traverse, traverseproc)) == current_tp_traverse)
        type = __Pyx_PyType_GetSlot(type, tp_base, PyTypeObject*);
    if (type && tp_traverse)
        return tp_traverse(obj, v, a);
    return 0;
}

/* CallNextTpClear */
static void __Pyx_call_next_tp_clear(PyObject* obj, inquiry current_tp_clear) {
    PyTypeObject* type = Py_TYPE(obj);
    inquiry tp_clear = NULL;
    while (type && __Pyx_PyType_GetSlot(type, tp_clear, inquiry) != current_tp_clear)
        type = __Pyx_PyType_GetSlot(type, tp_base, PyTypeObject*);
    while (type && (tp_clear = __Pyx_PyType_GetSlot(type, tp_clear, inquiry)) == current_tp_clear)
        type = __Pyx_PyType_GetSlot(type, tp_base, PyTypeObject*);
    if (type && tp_clear)
        tp_clear(obj);
}

/* FunctionExport */
static int __Pyx_ExportFunction(PyObject *api_dict, const char *name, void (*f)(void), const char *sig) {
    PyObject *cobj;
    union {
        void (*fp)(void);
        void *p;
    } tmp;
    tmp.fp = f;
    cobj = PyCapsule_New(tmp.p, sig, 0);
    if (!cobj)
        goto bad;
    if (PyDict_SetItemString(api_dict, name, cobj) < 0)
        goto bad;
    Py_DECREF(cobj);
    return 0;
bad:
    Py_XDECREF(cobj);
    return -1;
}

/* GetApiDict */
static PyObject *__Pyx_ApiExport_GetApiDict(void) {
    PyObject *d;
    if (__Pyx_PyDict_GetItemRef(__pyx_mstate_global->__pyx_d, __pyx_mstate_global->__pyx_n_u_pyx_capi, &d) == -1)
        return NULL;
    if (!d) {
        d = PyDict_New();
        if (!d)
            goto bad;
        if (PyObject_SetAttr(__pyx_m, __pyx_mstate_global->__pyx_n_u_pyx_capi, d) < 0)
            goto bad;
    }
    return d;
bad:
    Py_XDECREF(d);
    return NULL;
}

/* LimitedApiGetTypeDict (used by SetItemOnTypeDict) */
#if CYTHON_COMPILING_IN_LIMITED_API
static Py_ssize_t __Pyx_GetTypeDictOffset(void) {
    PyObject *tp_dictoffset_o;
    Py_ssize_t tp_dictoffset;
    tp_dictoffset_o = PyObject_GetAttrString((PyObject*)(&PyType_Type), "__dictoffset__");
    if (unlikely(!tp_dictoffset_o)) return -1;
    tp_dictoffset = PyLong_AsSsize_t(tp_dictoffset_o);
    Py_DECREF(tp_dictoffset_o);
    if (unlikely(tp_dictoffset == 0)) {
        PyErr_SetString(
            PyExc_TypeError,
            "'type' doesn't have a dictoffset");
        return -1;
    } else if (unlikely(tp_dictoffset < 0)) {
        PyErr_SetString(
            PyExc_TypeError,
            "'type' has an unexpected negative dictoffset. "
            "Please report this as Cython bug");
        return -1;
    }
    return tp_dictoffset;
}
static PyObject *__Pyx_GetTypeDict(PyTypeObject *tp) {
    static Py_ssize_t tp_dictoffset = 0;
    if (unlikely(tp_dictoffset == 0)) {
        tp_dictoffset = __Pyx_GetTypeDictOffset();
        if (unlikely(tp_dictoffset == -1 && PyErr_Occurred())) {
            tp_dictoffset = 0; // try again next time?
            return NULL;
        }
    }
    return *(PyObject**)((char*)tp + tp_dictoffset);
}
#endif

/* SetItemOnTypeDict (used by FixUpExtensionType) */
static int __Pyx__SetItemOnTypeDict(PyTypeObject *tp, PyObject *k, PyObject *v) {
    int result;
    PyObject *tp_dict;
#if CYTHON_COMPILING_IN_LIMITED_API
    tp_dict = __Pyx_GetTypeDict(tp);
    if (unlikely(!tp_dict)) return -1;
#else
    tp_dict = tp->tp_dict;
#endif
    result = PyDict_SetItem(tp_dict, k, v);
    if (likely(!result)) {
        PyType_Modified(tp);
        if (unlikely(PyObject_HasAttr(v, __pyx_mstate_global->__pyx_n_u_set_name))) {
            PyObject *setNameResult = PyObject_CallMethodObjArgs(v, __pyx_mstate_global->__pyx_n_u_set_name,  (PyObject *) tp, k, NULL);
            if (!setNameResult) return -1;
            Py_DECREF(setNameResult);
        }
    }
    return result;
}

/* FixUpExtensionType */
static int __Pyx_fix_up_extension_type_from_spec(PyType_Spec *spec, PyTypeObject *type) {
#if __PYX_LIMITED_VERSION_HEX > 0x030900B1
    CYTHON_UNUSED_VAR(spec);
    CYTHON_UNUSED_VAR(type);
    CYTHON_UNUSED_VAR(__Pyx__SetItemOnTypeDict);
#else
    const PyType_Slot *slot = spec->slots;
    int changed = 0;
#if !CYTHON_COMPILING_IN_LIMITED_API
    while (slot && slot->slot && slot->slot != Py_tp_members)
        slot++;
    if (slot && slot->slot == Py_tp_members) {
#if !CYTHON_COMPILING_IN_CPYTHON
        const
#endif  // !CYTHON_COMPILING_IN_CPYTHON)
            PyMemberDef *memb = (PyMemberDef*) slot->pfunc;
        while (memb && memb->name) {
            if (memb->name[0] == '_' && memb->name[1] == '_') {
                if (strcmp(memb->name, "__weaklistoffset__") == 0) {
                    assert(memb->type == T_PYSSIZET);
                    assert(memb->flags == READONLY);
                    type->tp_weaklistoffset = memb->offset;
                    changed = 1;
                }
                else if (strcmp(memb->name, "__dictoffset__") == 0) {
                    assert(memb->type == T_PYSSIZET);
                    assert(memb->flags == READONLY);
                    type->tp_dictoffset = memb->offset;
                    changed = 1;
                }
#if CYTHON_METH_FASTCALL
                else if (strcmp(memb->name, "__vectorcalloffset__") == 0) {
                    assert(memb->type == T_PYSSIZET);
                    assert(memb->flags == READONLY);
                    type->tp_vectorcall_offset = memb->offset;
                    changed = 1;
                }
#endif  // CYTHON_METH_FASTCALL
#if !CYTHON_COMPILING_IN_PYPY
                else if (strcmp(memb->name, "__module__") == 0) {
                    PyObject *descr;
                    assert(memb->type == T_OBJECT);
                    assert(memb->flags == 0 || memb->flags == READONLY);
                    descr = PyDescr_NewMember(type, memb);
                    if (unlikely(!descr))
                        return -1;
                    int set_item_result = PyDict_SetItem(type->tp_dict, PyDescr_NAME(descr), descr);
                    Py_DECREF(descr);
                    if (unlikely(set_item_result < 0)) {
                        return -1;
                    }
                    changed = 1;
                }
#endif  // !CYTHON_COMPILING_IN_PYPY
            }
            memb++;
        }
    }
#endif  // !CYTHON_COMPILING_IN_LIMITED_API
#if !CYTHON_COMPILING_IN_PYPY
    slot = spec->slots;
    while (slot && slot->slot && slot->slot != Py_tp_getset)
        slot++;
    if (slot && slot->slot == Py_tp_getset) {
        PyGetSetDef *getset = (PyGetSetDef*) slot->pfunc;
        while (getset && getset->name) {
            if (getset->name[0] == '_' && getset->name[1] == '_' && strcmp(getset->name, "__module__") == 0) {
                PyObject *descr = PyDescr_NewGetSet(type, getset);
                if (unlikely(!descr))
                    return -1;
                #if CYTHON_COMPILING_IN_LIMITED_API
                PyObject *pyname = PyUnicode_FromString(getset->name);
                if (unlikely(!pyname)) {
                    Py_DECREF(descr);
                    return -1;
                }
                int set_item_result = __Pyx_SetItemOnTypeDict(type, pyname, descr);
                Py_DECREF(pyname);
                #else
                CYTHON_UNUSED_VAR(__Pyx__SetItemOnTypeDict);
                int set_item_result = PyDict_SetItem(type->tp_dict, PyDescr_NAME(descr), descr);
                #endif
                Py_DECREF(descr);
                if (unlikely(set_item_result < 0)) {
                    return -1;
                }
                changed = 1;
            }
            ++getset;
        }
    }
#else
    CYTHON_UNUSED_VAR(__Pyx__SetItemOnTypeDict);
#endif  // !CYTHON_COMPILING_IN_PYPY
    if (changed)
        PyType_Modified(type);
#endif  // PY_VERSION_HEX > 0x030900B1
    return 0;
}

/* ValidateBasesTuple (used by PyType_Ready) */
#if CYTHON_COMPILING_IN_CPYTHON || CYTHON_COMPILING_IN_LIMITED_API || CYTHON_USE_TYPE_SPECS
static int __Pyx_validate_bases_tuple(const char *type_name, Py_ssize_t dictoffset, PyObject *bases) {
    Py_ssize_t i, n;
#if CYTHON_ASSUME_SAFE_SIZE
    n = PyTuple_GET_SIZE(bases);
#else
    n = PyTuple_Size(bases);
    if (unlikely(n < 0)) return -1;
#endif
    for (i = 1; i < n; i++)
    {
        PyTypeObject *b;
#if CYTHON_AVOID_BORROWED_REFS
        PyObject *b0 = PySequence_GetItem(bases, i);
        if (!b0) return -1;
#elif CYTHON_ASSUME_SAFE_MACROS
        PyObject *b0 = PyTuple_GET_ITEM(bases, i);
#else
        PyObject *b0 = PyTuple_GetItem(bases, i);
        if (!b0) return -1;
#endif
        b = (PyTypeObject*) b0;
        if (!__Pyx_PyType_HasFeature(b, Py_TPFLAGS_HEAPTYPE))
        {
            __Pyx_TypeName b_name = __Pyx_PyType_GetFullyQualifiedName(b);
            PyErr_Format(PyExc_TypeError,
                "base class '" __Pyx_FMT_TYPENAME "' is not a heap type", b_name);
            __Pyx_DECREF_TypeName(b_name);
#if CYTHON_AVOID_BORROWED_REFS
            Py_DECREF(b0);
#endif
            return -1;
        }
        if (dictoffset == 0)
        {
            Py_ssize_t b_dictoffset = 0;
#if CYTHON_USE_TYPE_SLOTS
            b_dictoffset = b->tp_dictoffset;
#else
            PyObject *py_b_dictoffset = PyObject_GetAttrString((PyObject*)b, "__dictoffset__");
            if (!py_b_dictoffset) goto dictoffset_return;
            b_dictoffset = PyLong_AsSsize_t(py_b_dictoffset);
            Py_DECREF(py_b_dictoffset);
            if (b_dictoffset == -1 && PyErr_Occurred()) goto dictoffset_return;
#endif
            if (b_dictoffset) {
                {
                    __Pyx_TypeName b_name = __Pyx_PyType_GetFullyQualifiedName(b);
                    PyErr_Format(PyExc_TypeError,
                        "extension type '%.200s' has no __dict__ slot, "
                        "but base type '" __Pyx_FMT_TYPENAME "' has: "
                        "either add 'cdef dict __dict__' to the extension type "
                        "or add '__slots__ = [...]' to the base type",
                        type_name, b_name);
                    __Pyx_DECREF_TypeName(b_name);
                }
#if !CYTHON_USE_TYPE_SLOTS
              dictoffset_return:
#endif
#if CYTHON_AVOID_BORROWED_REFS
                Py_DECREF(b0);
#endif
                return -1;
            }
        }
#if CYTHON_AVOID_BORROWED_REFS
        Py_DECREF(b0);
#endif
    }
    return 0;
}
#endif

/* PyType_Ready */
CYTHON_UNUSED static int __Pyx_PyType_HasMultipleInheritance(PyTypeObject *t) {
    while (t) {
        PyObject *bases = __Pyx_PyType_GetSlot(t, tp_bases, PyObject*);
        if (bases) {
            return 1;
        }
        t = __Pyx_PyType_GetSlot(t, tp_base, PyTypeObject*);
    }
    return 0;
}
static int __Pyx_PyType_Ready(PyTypeObject *t) {
#if CYTHON_USE_TYPE_SPECS || !CYTHON_COMPILING_IN_CPYTHON || defined(PYSTON_MAJOR_VERSION)
    (void)__Pyx_PyObject_CallMethod0;
#if CYTHON_USE_TYPE_SPECS
    (void)__Pyx_validate_bases_tuple;
#endif
    return PyType_Ready(t);
#else
    int r;
    if (!__Pyx_PyType_HasMultipleInheritance(t)) {
        return PyType_Ready(t);
    }
    PyObject *bases = __Pyx_PyType_GetSlot(t, tp_bases, PyObject*);
    if (bases && unlikely(__Pyx_validate_bases_tuple(t->tp_name, t->tp_dictoffset, bases) == -1))
        return -1;
#if !defined(PYSTON_MAJOR_VERSION)
    {
        int gc_was_enabled;
    #if PY_VERSION_HEX >= 0x030A00b1
        gc_was_enabled = PyGC_Disable();
        (void)__Pyx_PyObject_CallMethod0;
    #else
        PyObject *ret, *py_status;
        PyObject *gc = NULL;
        #if (!CYTHON_COMPILING_IN_PYPY || PYPY_VERSION_NUM+0 >= 0x07030400) &&\
                !CYTHON_COMPILING_IN_GRAAL
        gc = PyImport_GetModule(__pyx_mstate_global->__pyx_kp_u_gc);
        #endif
        if (unlikely(!gc)) gc = PyImport_Import(__pyx_mstate_global->__pyx_kp_u_gc);
        if (unlikely(!gc)) return -1;
        py_status = __Pyx_PyObject_CallMethod0(gc, __pyx_mstate_global->__pyx_kp_u_isenabled);
        if (unlikely(!py_status)) {
            Py_DECREF(gc);
            return -1;
        }
        gc_was_enabled = __Pyx_PyObject_IsTrue(py_status);
        Py_DECREF(py_status);
        if (gc_was_enabled > 0) {
            ret = __Pyx_PyObject_CallMethod0(gc, __pyx_mstate_global->__pyx_kp_u_disable);
            if (unlikely(!ret)) {
                Py_DECREF(gc);
                return -1;
            }
            Py_DECREF(ret);
        } else if (unlikely(gc_was_enabled == -1)) {
            Py_DECREF(gc);
            return -1;
        }
    #endif
        t->tp_flags |= Py_TPFLAGS_HEAPTYPE;
#if PY_VERSION_HEX >= 0x030A0000
        t->tp_flags |= Py_TPFLAGS_IMMUTABLETYPE;
#endif
#else
        (void)__Pyx_PyObject_CallMethod0;
#endif
    r = PyType_Ready(t);
#if !defined(PYSTON_MAJOR_VERSION)
        t->tp_flags &= ~Py_TPFLAGS_HEAPTYPE;
    #if PY_VERSION_HEX >= 0x030A00b1
        if (gc_was_enabled)
            PyGC_Enable();
    #else
        if (gc_was_enabled) {
            PyObject *tp, *v, *tb;
            PyErr_Fetch(&tp, &v, &tb);
            ret = __Pyx_PyObject_CallMethod0(gc, __pyx_mstate_global->__pyx_kp_u_enable);
            if (likely(ret || r == -1)) {
                Py_XDECREF(ret);
                PyErr_Restore(tp, v, tb);
            } else {
                Py_XDECREF(tp);
                Py_XDECREF(v);
                Py_XDECREF(tb);
                r = -1;
            }
        }
        Py_DECREF(gc);
    #endif
    }
#endif
    return r;
#endif
}

/* DelItemOnTypeDict (used by SetupReduce) */
static int __Pyx__DelItemOnTypeDict(PyTypeObject *tp, PyObject *k) {
    int result;
    PyObject *tp_dict;
#if CYTHON_COMPILING_IN_LIMITED_API
    tp_dict = __Pyx_GetTypeDict(tp);
    if (unlikely(!tp_dict)) return -1;
#else
    tp_dict = tp->tp_dict;
#endif
    result = PyDict_DelItem(tp_dict, k);
    if (likely(!result)) PyType_Modified(tp);
    return result;
}

/* SetupReduce */
static int __Pyx_setup_reduce_is_named(PyObject* meth, PyObject* name) {
  int ret;
  PyObject *name_attr;
  name_attr = __Pyx_PyObject_GetAttrStrNoError(meth, __pyx_mstate_global->__pyx_n_u_name);
  if (likely(name_attr)) {
      ret = PyObject_RichCompareBool(name_attr, name, Py_EQ);
  } else {
      ret = -1;
  }
  if (unlikely(ret < 0)) {
      PyErr_Clear();
      ret = 0;
  }
  Py_XDECREF(name_attr);
  return ret;
}
static int __Pyx_setup_reduce(PyObject* type_obj) {
    int ret = 0;
    PyObject *object_reduce = NULL;
    PyObject *object_getstate = NULL;
    PyObject *object_reduce_ex = NULL;
    PyObject *reduce = NULL;
    PyObject *reduce_ex = NULL;
    PyObject *reduce_cython = NULL;
    PyObject *setstate = NULL;
    PyObject *setstate_cython = NULL;
    PyObject *getstate = NULL;
#if CYTHON_USE_PYTYPE_LOOKUP
    getstate = _PyType_Lookup((PyTypeObject*)type_obj, __pyx_mstate_global->__pyx_n_u_getstate);
#else
    getstate = __Pyx_PyObject_GetAttrStrNoError(type_obj, __pyx_mstate_global->__pyx_n_u_getstate);
    if (!getstate && PyErr_Occurred()) {
        goto __PYX_BAD;
    }
#endif
    if (getstate) {
#if CYTHON_USE_PYTYPE_LOOKUP
        object_getstate = _PyType_Lookup(&PyBaseObject_Type, __pyx_mstate_global->__pyx_n_u_getstate);
#else
        object_getstate = __Pyx_PyObject_GetAttrStrNoError((PyObject*)&PyBaseObject_Type, __pyx_mstate_global->__pyx_n_u_getstate);
        if (!object_getstate && PyErr_Occurred()) {
            goto __PYX_BAD;
        }
#endif
        if (object_getstate != getstate) {
            goto __PYX_GOOD;
        }
    }
#if CYTHON_USE_PYTYPE_LOOKUP
    object_reduce_ex = _PyType_Lookup(&PyBaseObject_Type, __pyx_mstate_global->__pyx_n_u_reduce_ex); if (!object_reduce_ex) goto __PYX_BAD;
#else
    object_reduce_ex = __Pyx_PyObject_GetAttrStr((PyObject*)&PyBaseObject_Type, __pyx_mstate_global->__pyx_n_u_reduce_ex); if (!object_reduce_ex) goto __PYX_BAD;
#endif
    reduce_ex = __Pyx_PyObject_GetAttrStr(type_obj, __pyx_mstate_global->__pyx_n_u_reduce_ex); if (unlikely(!reduce_ex)) goto __PYX_BAD;
    if (reduce_ex == object_reduce_ex) {
#if CYTHON_USE_PYTYPE_LOOKUP
        object_reduce = _PyType_Lookup(&PyBaseObject_Type, __pyx_mstate_global->__pyx_n_u_reduce); if (!object_reduce) goto __PYX_BAD;
#else
        object_reduce = __Pyx_PyObject_GetAttrStr((PyObject*)&PyBaseObject_Type, __pyx_mstate_global->__pyx_n_u_reduce); if (!object_reduce) goto __PYX_BAD;
#endif
        reduce = __Pyx_PyObject_GetAttrStr(type_obj, __pyx_mstate_global->__pyx_n_u_reduce); if (unlikely(!reduce)) goto __PYX_BAD;
        if (reduce == object_reduce || __Pyx_setup_reduce_is_named(reduce, __pyx_mstate_global->__pyx_n_u_reduce_cython)) {
            reduce_cython = __Pyx_PyObject_GetAttrStrNoError(type_obj, __pyx_mstate_global->__pyx_n_u_reduce_cython);
            if (likely(reduce_cython)) {
                ret = __Pyx_SetItemOnTypeDict((PyTypeObject*)type_obj, __pyx_mstate_global->__pyx_n_u_reduce, reduce_cython); if (unlikely(ret < 0)) goto __PYX_BAD;
                ret = __Pyx_DelItemOnTypeDict((PyTypeObject*)type_obj, __pyx_mstate_global->__pyx_n_u_reduce_cython); if (unlikely(ret < 0)) goto __PYX_BAD;
            } else if (reduce == object_reduce || PyErr_Occurred()) {
                goto __PYX_BAD;
            }
            setstate = __Pyx_PyObject_GetAttrStrNoError(type_obj, __pyx_mstate_global->__pyx_n_u_setstate);
            if (!setstate) PyErr_Clear();
            if (!setstate || __Pyx_setup_reduce_is_named(setstate, __pyx_mstate_global->__pyx_n_u_setstate_cython)) {
                setstate_cython = __Pyx_PyObject_GetAttrStrNoError(type_obj, __pyx_mstate_global->__pyx_n_u_setstate_cython);
                if (likely(setstate_cython)) {
                    ret = __Pyx_SetItemOnTypeDict((PyTypeObject*)type_obj, __pyx_mstate_global->__pyx_n_u_setstate, setstate_cython); if (unlikely(ret < 0)) goto __PYX_BAD;
                    ret = __Pyx_DelItemOnTypeDict((PyTypeObject*)type_obj, __pyx_mstate_global->__pyx_n_u_setstate_cython); if (unlikely(ret < 0)) goto __PYX_BAD;
                } else if (!setstate || PyErr_Occurred()) {
                    goto __PYX_BAD;
                }
            }
            PyType_Modified((PyTypeObject*)type_obj);
        }
    }
    goto __PYX_GOOD;
__PYX_BAD:
    if (!PyErr_Occurred()) {
        __Pyx_TypeName type_obj_name =
            __Pyx_PyType_GetFullyQualifiedName((PyTypeObject*)type_obj);
        PyErr_Format(PyExc_RuntimeError,
            "Unable to initialize pickling for " __Pyx_FMT_TYPENAME, type_obj_name);
        __Pyx_DECREF_TypeName(type_obj_name);
    }
    ret = -1;
__PYX_GOOD:
#if !CYTHON_USE_PYTYPE_LOOKUP
    Py_XDECREF(object_reduce);
    Py_XDECREF(object_reduce_ex);
    Py_XDECREF(object_getstate);
    Py_XDECREF(getstate);
#endif
    Py_XDECREF(reduce);
    Py_XDECREF(reduce_ex);
    Py_XDECREF(reduce_cython);
    Py_XDECREF(setstate);
    Py_XDECREF(setstate_cython);
    return ret;
}

/* SetVTable */
static int __Pyx_SetVtable(PyTypeObject *type, void *vtable) {
    PyObject *ob = PyCapsule_New(vtable, 0, 0);
    if (unlikely(!ob))
        goto bad;
#if CYTHON_COMPILING_IN_LIMITED_API
    if (unlikely(PyObject_SetAttr((PyObject *) type, __pyx_mstate_global->__pyx_n_u_pyx_vtable, ob) < 0))
#else
    if (unlikely(PyDict_SetItem(type->tp_dict, __pyx_mstate_global->__pyx_n_u_pyx_vtable, ob) < 0))
#endif
        goto bad;
    Py_DECREF(ob);
    return 0;
bad:
    Py_XDECREF(ob);
    return -1;
}

/* GetVTable (used by MergeVTables) */
static void* __Pyx_GetVtable(PyTypeObject *type) {
    void* ptr;
#if CYTHON_COMPILING_IN_LIMITED_API
    PyObject *ob = PyObject_GetAttr((PyObject *)type, __pyx_mstate_global->__pyx_n_u_pyx_vtable);
#else
    PyObject *ob = PyObject_GetItem(type->tp_dict, __pyx_mstate_global->__pyx_n_u_pyx_vtable);
#endif
    if (!ob)
        goto bad;
    ptr = PyCapsule_GetPointer(ob, 0);
    if (!ptr && !PyErr_Occurred())
        PyErr_SetString(PyExc_RuntimeError, "invalid vtable found for imported type");
    Py_DECREF(ob);
    return ptr;
bad:
    Py_XDECREF(ob);
    return NULL;
}

/* MergeVTables */
static int __Pyx_MergeVtables(PyTypeObject *type) {
    int i=0;
    Py_ssize_t size;
    void** base_vtables;
    __Pyx_TypeName tp_base_name = NULL;
    __Pyx_TypeName base_name = NULL;
    void* unknown = (void*)-1;
    PyObject* bases = __Pyx_PyType_GetSlot(type, tp_bases, PyObject*);
    int base_depth = 0;
    {
        PyTypeObject* base = __Pyx_PyType_GetSlot(type, tp_base, PyTypeObject*);
        while (base) {
            base_depth += 1;
            base = __Pyx_PyType_GetSlot(base, tp_base, PyTypeObject*);
        }
    }
    base_vtables = (void**) PyMem_Malloc(sizeof(void*) * (size_t)(base_depth + 1));
    base_vtables[0] = unknown;
#if CYTHON_COMPILING_IN_LIMITED_API
    size = PyTuple_Size(bases);
    if (size < 0) goto other_failure;
#else
    size = PyTuple_GET_SIZE(bases);
#endif
    for (i = 1; i < size; i++) {
        PyObject *basei;
        void* base_vtable;
#if CYTHON_AVOID_BORROWED_REFS
        basei = PySequence_GetItem(bases, i);
        if (unlikely(!basei)) goto other_failure;
#elif !CYTHON_ASSUME_SAFE_MACROS
        basei = PyTuple_GetItem(bases, i);
        if (unlikely(!basei)) goto other_failure;
#else
        basei = PyTuple_GET_ITEM(bases, i);
#endif
        base_vtable = __Pyx_GetVtable((PyTypeObject*)basei);
#if CYTHON_AVOID_BORROWED_REFS
        Py_DECREF(basei);
#endif
        if (base_vtable != NULL) {
            int j;
            PyTypeObject* base = __Pyx_PyType_GetSlot(type, tp_base, PyTypeObject*);
            for (j = 0; j < base_depth; j++) {
                if (base_vtables[j] == unknown) {
                    base_vtables[j] = __Pyx_GetVtable(base);
                    base_vtables[j + 1] = unknown;
                }
                if (base_vtables[j] == base_vtable) {
                    break;
                } else if (base_vtables[j] == NULL) {
                    goto bad;
                }
                base = __Pyx_PyType_GetSlot(base, tp_base, PyTypeObject*);
            }
        }
    }
    PyErr_Clear();
    PyMem_Free(base_vtables);
    return 0;
bad:
    {
        PyTypeObject* basei = NULL;
        PyTypeObject* tp_base = __Pyx_PyType_GetSlot(type, tp_base, PyTypeObject*);
        tp_base_name = __Pyx_PyType_GetFullyQualifiedName(tp_base);
#if CYTHON_AVOID_BORROWED_REFS
        basei = (PyTypeObject*)PySequence_GetItem(bases, i);
        if (unlikely(!basei)) goto really_bad;
#elif !CYTHON_ASSUME_SAFE_MACROS
        basei = (PyTypeObject*)PyTuple_GetItem(bases, i);
        if (unlikely(!basei)) goto really_bad;
#else
        basei = (PyTypeObject*)PyTuple_GET_ITEM(bases, i);
#endif
        base_name = __Pyx_PyType_GetFullyQualifiedName(basei);
#if CYTHON_AVOID_BORROWED_REFS
        Py_DECREF(basei);
#endif
    }
    PyErr_Format(PyExc_TypeError,
        "multiple bases have vtable conflict: '" __Pyx_FMT_TYPENAME "' and '" __Pyx_FMT_TYPENAME "'", tp_base_name, base_name);
#if CYTHON_AVOID_BORROWED_REFS || !CYTHON_ASSUME_SAFE_MACROS
really_bad: // bad has failed!
#endif
    __Pyx_DECREF_TypeName(tp_base_name);
    __Pyx_DECREF_TypeName(base_name);
#if CYTHON_COMPILING_IN_LIMITED_API || CYTHON_AVOID_BORROWED_REFS || !CYTHON_ASSUME_SAFE_MACROS
other_failure:
#endif
    PyMem_Free(base_vtables);
    return -1;
}

/* TypeImport */
#ifndef __PYX_HAVE_RT_ImportType_3_2_5
#define __PYX_HAVE_RT_ImportType_3_2_5
static PyTypeObject *__Pyx_ImportType_3_2_5(PyObject *module, const char *module_name, const char *class_name,
    size_t size, size_t alignment, enum __Pyx_ImportType_CheckSize_3_2_5 check_size)
{
    PyObject *result = 0;
    Py_ssize_t basicsize;
    Py_ssize_t itemsize;
#if defined(Py_LIMITED_API) || (defined(CYTHON_COMPILING_IN_LIMITED_API) && CYTHON_COMPILING_IN_LIMITED_API)
    PyObject *py_basicsize;
    PyObject *py_itemsize;
#endif
    result = PyObject_GetAttrString(module, class_name);
    if (!result)
        goto bad;
    if (!PyType_Check(result)) {
        PyErr_Format(PyExc_TypeError,
            "%.200s.%.200s is not a type object",
            module_name, class_name);
        goto bad;
    }
#if !( defined(Py_LIMITED_API) || (defined(CYTHON_COMPILING_IN_LIMITED_API) && CYTHON_COMPILING_IN_LIMITED_API) )
    basicsize = ((PyTypeObject *)result)->tp_basicsize;
    itemsize = ((PyTypeObject *)result)->tp_itemsize;
#else
    if (size == 0) {
        return (PyTypeObject *)result;
    }
    py_basicsize = PyObject_GetAttrString(result, "__basicsize__");
    if (!py_basicsize)
        goto bad;
    basicsize = PyLong_AsSsize_t(py_basicsize);
    Py_DECREF(py_basicsize);
    py_basicsize = 0;
    if (basicsize == (Py_ssize_t)-1 && PyErr_Occurred())
        goto bad;
    py_itemsize = PyObject_GetAttrString(result, "__itemsize__");
    if (!py_itemsize)
        goto bad;
    itemsize = PyLong_AsSsize_t(py_itemsize);
    Py_DECREF(py_itemsize);
    py_itemsize = 0;
    if (itemsize == (Py_ssize_t)-1 && PyErr_Occurred())
        goto bad;
#endif
    if (itemsize) {
        if (size % alignment) {
            alignment = size % alignment;
        }
        if (itemsize < (Py_ssize_t)alignment)
            itemsize = (Py_ssize_t)alignment;
    }
    if ((size_t)(basicsize + itemsize) < size) {
        PyErr_Format(PyExc_ValueError,
            "%.200s.%.200s size changed, may indicate binary incompatibility. "
            "Expected %zd from C header, got %zd from PyObject",
            module_name, class_name, size, basicsize+itemsize);
        goto bad;
    }
    if (check_size == __Pyx_ImportType_CheckSize_Error_3_2_5 &&
            ((size_t)basicsize > size || (size_t)(basicsize + itemsize) < size)) {
        PyErr_Format(PyExc_ValueError,
            "%.200s.%.200s size changed, may indicate binary incompatibility. "
            "Expected %zd from C header, got %zd-%zd from PyObject",
            module_name, class_name, size, basicsize, basicsize+itemsize);
        goto bad;
    }
    else if (check_size == __Pyx_ImportType_CheckSize_Warn_3_2_5 && (size_t)basicsize > size) {
        if (PyErr_WarnFormat(NULL, 0,
                "%.200s.%.200s size changed, may indicate binary incompatibility. "
                "Expected %zd from C header, got %zd from PyObject",
                module_name, class_name, size, basicsize) < 0) {
            goto bad;
        }
    }
    return (PyTypeObject *)result;
bad:
    Py_XDECREF(result);
    return NULL;
}
#endif

/* PxdImportShared (used by VoidPtrImport) */
#ifndef __PYX_HAVE_RT_ImportFromPxd_3_2_5
#define __PYX_HAVE_RT_ImportFromPxd_3_2_5
static int __Pyx_ImportFromPxd_3_2_5(PyObject *module, const char *name, void **p, const char *sig, const char *what) {
    PyObject *d = 0;
    PyObject *cobj = 0;
    d = PyObject_GetAttrString(module, "__pyx_capi__");
    if (!d)
        goto bad;
#if (defined(Py_LIMITED_API) && Py_LIMITED_API >= 0x030d0000) || (!defined(Py_LIMITED_API) && PY_VERSION_HEX >= 0x030d0000)
    PyDict_GetItemStringRef(d, name, &cobj);
#else
    cobj = PyDict_GetItemString(d, name);
    Py_XINCREF(cobj);
#endif
    if (!cobj) {
        PyErr_Format(PyExc_ImportError,
            "%.200s does not export expected C %.8s %.200s",
                PyModule_GetName(module), what, name);
        goto bad;
    }
    if (!PyCapsule_IsValid(cobj, sig)) {
        PyErr_Format(PyExc_TypeError,
            "C %.8s %.200s.%.200s has wrong signature (expected %.500s, got %.500s)",
             what, PyModule_GetName(module), name, sig, PyCapsule_GetName(cobj));
        goto bad;
    }
    *p = PyCapsule_GetPointer(cobj, sig);
    if (!(*p))
        goto bad;
    Py_DECREF(d);
    Py_DECREF(cobj);
    return 0;
bad:
    Py_XDECREF(d);
    Py_XDECREF(cobj);
    return -1;
}
#endif

/* VoidPtrImport */
#ifndef __PYX_HAVE_RT_ImportVoidPtr_3_2_5
#define __PYX_HAVE_RT_ImportVoidPtr_3_2_5
static int __Pyx_ImportVoidPtr_3_2_5(PyObject *module, const char *name, void **p, const char *sig) {
    return __Pyx_ImportFromPxd_3_2_5(module, name, p, sig, "variable");
}
#endif

/* FunctionImport */
#ifndef __PYX_HAVE_RT_ImportFunction_3_2_5
#define __PYX_HAVE_RT_ImportFunction_3_2_5
static int __Pyx_ImportFunction_3_2_5(PyObject *module, const char *funcname, void (**f)(void), const char *sig) {
    union {
        void (*fp)(void);
        void *p;
    } tmp;
    int result = __Pyx_ImportFromPxd_3_2_5(module, funcname, &tmp.p, sig, "function");
    if (result == 0) {
        *f = tmp.fp;
    }
    return result;
}
#endif

/* HasAttr (used by ImportImpl) */
#if __PYX_LIMITED_VERSION_HEX < 0x030d0000
static CYTHON_INLINE int __Pyx_HasAttr(PyObject *o, PyObject *n) {
    PyObject *r;
    if (unlikely(!PyUnicode_Check(n))) {
        PyErr_SetString(PyExc_TypeError,
                        "hasattr(): attribute name must be string");
        return -1;
    }
    r = __Pyx_PyObject_GetAttrStrNoError(o, n);
    if (!r) {
        return (unlikely(PyErr_Occurred())) ? -1 : 0;
    } else {
        Py_DECREF(r);
        return 1;
    }
}
#endif

/* ImportImpl (used by Import) */
static int __Pyx__Import_GetModule(PyObject *qualname, PyObject **module) {
    PyObject *imported_module = PyImport_GetModule(qualname);
    if (unlikely(!imported_module)) {
        *module = NULL;
        if (PyErr_Occurred()) {
            return -1;
        }
        return 0;
    }
    *module = imported_module;
    return 1;
}
static int __Pyx__Import_Lookup(PyObject *qualname, PyObject *const *imported_names, Py_ssize_t len_imported_names, PyObject **module) {
    PyObject *imported_module;
    PyObject *top_level_package_name;
    Py_ssize_t i;
    int status, module_found;
    Py_ssize_t dot_index;
    module_found = __Pyx__Import_GetModule(qualname, &imported_module);
    if (unlikely(!module_found || module_found == -1)) {
        *module = NULL;
        return module_found;
    }
    if (imported_names) {
        for (i = 0; i < len_imported_names; i++) {
            PyObject *imported_name = imported_names[i];
#if __PYX_LIMITED_VERSION_HEX < 0x030d0000
            int has_imported_attribute = PyObject_HasAttr(imported_module, imported_name);
#else
            int has_imported_attribute = PyObject_HasAttrWithError(imported_module, imported_name);
            if (unlikely(has_imported_attribute == -1)) goto error;
#endif
            if (!has_imported_attribute) {
                goto not_found;
            }
        }
        *module = imported_module;
        return 1;
    }
    dot_index = PyUnicode_FindChar(qualname, '.', 0, PY_SSIZE_T_MAX, 1);
    if (dot_index == -1) {
        *module = imported_module;
        return 1;
    }
    if (unlikely(dot_index == -2)) goto error;
    top_level_package_name = PyUnicode_Substring(qualname, 0, dot_index);
    if (unlikely(!top_level_package_name)) goto error;
    Py_DECREF(imported_module);
    status = __Pyx__Import_GetModule(top_level_package_name, module);
    Py_DECREF(top_level_package_name);
    return status;
error:
    Py_DECREF(imported_module);
    *module = NULL;
    return -1;
not_found:
    Py_DECREF(imported_module);
    *module = NULL;
    return 0;
}
static PyObject *__Pyx__Import(PyObject *name, PyObject *const *imported_names, Py_ssize_t len_imported_names, PyObject *qualname, PyObject *moddict, int level) {
    PyObject *module = 0;
    PyObject *empty_dict = 0;
    PyObject *from_list = 0;
    int module_found;
    if (!qualname) {
        qualname = name;
    }
    module_found = __Pyx__Import_Lookup(qualname, imported_names, len_imported_names, &module);
    if (likely(module_found == 1)) {
        return module;
    } else if (unlikely(module_found == -1)) {
        return NULL;
    }
    empty_dict = PyDict_New();
    if (unlikely(!empty_dict))
        goto bad;
    if (imported_names) {
#if CYTHON_COMPILING_IN_CPYTHON
        from_list = __Pyx_PyList_FromArray(imported_names, len_imported_names);
        if (unlikely(!from_list))
            goto bad;
#else
        from_list = PyList_New(len_imported_names);
        if (unlikely(!from_list)) goto bad;
        for (Py_ssize_t i=0; i<len_imported_names; ++i) {
            if (PyList_SetItem(from_list, i, __Pyx_NewRef(imported_names[i])) < 0) goto bad;
        }
#endif
    }
    if (level == -1) {
        const char* package_sep = strchr(__Pyx_MODULE_NAME, '.');
        if (package_sep != (0)) {
            module = PyImport_ImportModuleLevelObject(
                name, moddict, empty_dict, from_list, 1);
            if (unlikely(!module)) {
                if (unlikely(!PyErr_ExceptionMatches(PyExc_ImportError)))
                    goto bad;
                PyErr_Clear();
            }
        }
        level = 0;
    }
    if (!module) {
        module = PyImport_ImportModuleLevelObject(
            name, moddict, empty_dict, from_list, level);
    }
bad:
    Py_XDECREF(from_list);
    Py_XDECREF(empty_dict);
    return module;
}

/* Import */
static PyObject *__Pyx_Import(PyObject *name, PyObject *const *imported_names, Py_ssize_t len_imported_names, PyObject *qualname, int level) {
    return __Pyx__Import(name, imported_names, len_imported_names, qualname, __pyx_mstate_global->__pyx_d, level);
}

/* ImportFrom */
static PyObject* __Pyx_ImportFrom(PyObject* module, PyObject* name) {
    PyObject* value = __Pyx_PyObject_GetAttrStr(module, name);
    if (unlikely(!value) && PyErr_ExceptionMatches(PyExc_AttributeError)) {
        const char* module_name_str = 0;
        PyObject* module_name = 0;
        PyObject* module_dot = 0;
        PyObject* full_name = 0;
        PyErr_Clear();
        module_name_str = PyModule_GetName(module);
        if (unlikely(!module_name_str)) { goto modbad; }
        module_name = PyUnicode_FromString(module_name_str);
        if (unlikely(!module_name)) { goto modbad; }
        module_dot = PyUnicode_Concat(module_name, __pyx_mstate_global->__pyx_kp_u__6);
        if (unlikely(!module_dot)) { goto modbad; }
        full_name = PyUnicode_Concat(module_dot, name);
        if (unlikely(!full_name)) { goto modbad; }
        #if (CYTHON_COMPILING_IN_PYPY && PYPY_VERSION_NUM  < 0x07030400) ||\
                CYTHON_COMPILING_IN_GRAAL
        {
            PyObject *modules = PyImport_GetModuleDict();
            if (unlikely(!modules))
                goto modbad;
            value = PyObject_GetItem(modules, full_name);
        }
        #else
        value = PyImport_GetModule(full_name);
        #endif
      modbad:
        Py_XDECREF(full_name);
        Py_XDECREF(module_dot);
        Py_XDECREF(module_name);
    }
    if (unlikely(!value)) {
        PyErr_Format(PyExc_ImportError, "cannot import name %S", name);
    }
    return value;
}

/* PyLongCompare */
static CYTHON_INLINE PyObject* __Pyx_PyLong_NeObjC(PyObject *op1, PyObject *op2, long intval, long inplace) {
    CYTHON_MAYBE_UNUSED_VAR(intval);
    CYTHON_UNUSED_VAR(inplace);
    if (op1 == op2) {
        Py_RETURN_FALSE;
    }
    #if CYTHON_USE_PYLONG_INTERNALS
    if (likely(PyLong_CheckExact(op1))) {
        int unequal;
        unsigned long uintval;
        Py_ssize_t size = __Pyx_PyLong_DigitCount(op1);
        const digit* digits = __Pyx_PyLong_Digits(op1);
        if (intval == 0) {
            if (__Pyx_PyLong_IsZero(op1) != 1) Py_RETURN_TRUE; else Py_RETURN_FALSE;
        } else if (intval < 0) {
            if (__Pyx_PyLong_IsNonNeg(op1))
                Py_RETURN_TRUE;
            intval = -intval;
        } else {
            if (__Pyx_PyLong_IsNeg(op1))
                Py_RETURN_TRUE;
        }
        uintval = (unsigned long) intval;
#if PyLong_SHIFT * 4 < SIZEOF_LONG*8
        if (uintval >> (PyLong_SHIFT * 4)) {
            unequal = (size != 5) || (digits[0] != (uintval & (unsigned long) PyLong_MASK))
                 | (digits[1] != ((uintval >> (1 * PyLong_SHIFT)) & (unsigned long) PyLong_MASK)) | (digits[2] != ((uintval >> (2 * PyLong_SHIFT)) & (unsigned long) PyLong_MASK)) | (digits[3] != ((uintval >> (3 * PyLong_SHIFT)) & (unsigned long) PyLong_MASK)) | (digits[4] != ((uintval >> (4 * PyLong_SHIFT)) & (unsigned long) PyLong_MASK));
        } else
#endif
#if PyLong_SHIFT * 3 < SIZEOF_LONG*8
        if (uintval >> (PyLong_SHIFT * 3)) {
            unequal = (size != 4) || (digits[0] != (uintval & (unsigned long) PyLong_MASK))
                 | (digits[1] != ((uintval >> (1 * PyLong_SHIFT)) & (unsigned long) PyLong_MASK)) | (digits[2] != ((uintval >> (2 * PyLong_SHIFT)) & (unsigned long) PyLong_MASK)) | (digits[3] != ((uintval >> (3 * PyLong_SHIFT)) & (unsigned long) PyLong_MASK));
        } else
#endif
#if PyLong_SHIFT * 2 < SIZEOF_LONG*8
        if (uintval >> (PyLong_SHIFT * 2)) {
            unequal = (size != 3) || (digits[0] != (uintval & (unsigned long) PyLong_MASK))
                 | (digits[1] != ((uintval >> (1 * PyLong_SHIFT)) & (unsigned long) PyLong_MASK)) | (digits[2] != ((uintval >> (2 * PyLong_SHIFT)) & (unsigned long) PyLong_MASK));
        } else
#endif
#if PyLong_SHIFT * 1 < SIZEOF_LONG*8
        if (uintval >> (PyLong_SHIFT * 1)) {
            unequal = (size != 2) || (digits[0] != (uintval & (unsigned long) PyLong_MASK))
                 | (digits[1] != ((uintval >> (1 * PyLong_SHIFT)) & (unsigned long) PyLong_MASK));
        } else
#endif
            unequal = (size != 1) || (((unsigned long) digits[0]) != (uintval & (unsigned long) PyLong_MASK));
        if (unequal != 0) Py_RETURN_TRUE; else Py_RETURN_FALSE;
    }
    #endif
    if (PyFloat_CheckExact(op1)) {
        const long b = intval;
        double a = __Pyx_PyFloat_AS_DOUBLE(op1);
        if ((double)a != (double)b) Py_RETURN_TRUE; else Py_RETURN_FALSE;
    }
    return (
        PyObject_RichCompare(op1, op2, Py_NE));
}

/* Py3UpdateBases */
static PyObject*
__Pyx_PEP560_update_bases(PyObject *bases)
{
    Py_ssize_t i, j, size_bases;
    PyObject *base = NULL, *meth, *new_base, *result, *new_bases = NULL;
#if CYTHON_ASSUME_SAFE_SIZE
    size_bases = PyTuple_GET_SIZE(bases);
#else
    size_bases = PyTuple_Size(bases);
    if (size_bases < 0) return NULL;
#endif
    for (i = 0; i < size_bases; i++) {
#if CYTHON_AVOID_BORROWED_REFS
        Py_CLEAR(base);
#endif
#if CYTHON_ASSUME_SAFE_MACROS
        base = PyTuple_GET_ITEM(bases, i);
#else
        base = PyTuple_GetItem(bases, i);
        if (!base) goto error;
#endif
#if CYTHON_AVOID_BORROWED_REFS
        Py_INCREF(base);
#endif
        if (PyType_Check(base)) {
            if (new_bases) {
                if (PyList_Append(new_bases, base) < 0) {
                    goto error;
                }
            }
            continue;
        }
        meth = __Pyx_PyObject_GetAttrStrNoError(base, __pyx_mstate_global->__pyx_n_u_mro_entries);
        if (!meth && PyErr_Occurred()) {
            goto error;
        }
        if (!meth) {
            if (new_bases) {
                if (PyList_Append(new_bases, base) < 0) {
                    goto error;
                }
            }
            continue;
        }
        new_base = __Pyx_PyObject_CallOneArg(meth, bases);
        Py_DECREF(meth);
        if (!new_base) {
            goto error;
        }
        if (!PyTuple_Check(new_base)) {
            PyErr_SetString(PyExc_TypeError,
                            "__mro_entries__ must return a tuple");
            Py_DECREF(new_base);
            goto error;
        }
        if (!new_bases) {
            if (!(new_bases = PyList_New(i))) {
                goto error;
            }
            for (j = 0; j < i; j++) {
                PyObject *base_from_list;
#if CYTHON_ASSUME_SAFE_MACROS
                base_from_list = PyTuple_GET_ITEM(bases, j);
                PyList_SET_ITEM(new_bases, j, base_from_list);
                Py_INCREF(base_from_list);
#else
                base_from_list = PyTuple_GetItem(bases, j);
                if (!base_from_list) goto error;
                Py_INCREF(base_from_list);
                if (PyList_SetItem(new_bases, j, base_from_list) < 0) goto error;
#endif
            }
        }
#if CYTHON_ASSUME_SAFE_SIZE
        j = PyList_GET_SIZE(new_bases);
#else
        j = PyList_Size(new_bases);
        if (j < 0) goto error;
#endif
        if (PyList_SetSlice(new_bases, j, j, new_base) < 0) {
            goto error;
        }
        Py_DECREF(new_base);
    }
    if (!new_bases) {
        Py_INCREF(bases);
        return bases;
    }
    result = PyList_AsTuple(new_bases);
    Py_DECREF(new_bases);
#if CYTHON_AVOID_BORROWED_REFS
    Py_XDECREF(base);
#endif
    return result;
error:
    Py_XDECREF(new_bases);
#if CYTHON_AVOID_BORROWED_REFS
    Py_XDECREF(base);
#endif
    return NULL;
}

/* CalculateMetaclass */
static PyObject *__Pyx_CalculateMetaclass(PyTypeObject *metaclass, PyObject *bases) {
    Py_ssize_t i, nbases;
#if CYTHON_ASSUME_SAFE_SIZE
    nbases = PyTuple_GET_SIZE(bases);
#else
    nbases = PyTuple_Size(bases);
    if (nbases < 0) return NULL;
#endif
    for (i=0; i < nbases; i++) {
        PyTypeObject *tmptype;
#if CYTHON_ASSUME_SAFE_MACROS
        PyObject *tmp = PyTuple_GET_ITEM(bases, i);
#else
        PyObject *tmp = PyTuple_GetItem(bases, i);
        if (!tmp) return NULL;
#endif
        tmptype = Py_TYPE(tmp);
        if (!metaclass) {
            metaclass = tmptype;
            continue;
        }
        if (PyType_IsSubtype(metaclass, tmptype))
            continue;
        if (PyType_IsSubtype(tmptype, metaclass)) {
            metaclass = tmptype;
            continue;
        }
        PyErr_SetString(PyExc_TypeError,
                        "metaclass conflict: "
                        "the metaclass of a derived class "
                        "must be a (non-strict) subclass "
                        "of the metaclasses of all its bases");
        return NULL;
    }
    if (!metaclass) {
        metaclass = &PyType_Type;
    }
    Py_INCREF((PyObject*) metaclass);
    return (PyObject*) metaclass;
}

/* dict_setdefault (used by FetchCommonType) */
static CYTHON_INLINE PyObject *__Pyx_PyDict_SetDefault(PyObject *d, PyObject *key, PyObject *default_value) {
    PyObject* value;
#if __PYX_LIMITED_VERSION_HEX >= 0x030F0000 || (!CYTHON_COMPILING_IN_LIMITED_API && PY_VERSION_HEX >= 0x030d00A4)
    PyDict_SetDefaultRef(d, key, default_value, &value);
#elif CYTHON_COMPILING_IN_LIMITED_API && __PYX_LIMITED_VERSION_HEX >= 0x030C0000
    PyObject *args[] = {d, key, default_value};
    value = PyObject_VectorcallMethod(__pyx_mstate_global->__pyx_n_u_setdefault, args, 3 | PY_VECTORCALL_ARGUMENTS_OFFSET, NULL);
#elif CYTHON_COMPILING_IN_LIMITED_API
    value = PyObject_CallMethodObjArgs(d, __pyx_mstate_global->__pyx_n_u_setdefault, key, default_value, NULL);
#else
    value = PyDict_SetDefault(d, key, default_value);
    if (unlikely(!value)) return NULL;
    Py_INCREF(value);
#endif
    return value;
}

/* AddModuleRef (used by FetchSharedCythonModule) */
#if CYTHON_COMPILING_IN_CPYTHON_FREETHREADING
  static PyObject *__Pyx_PyImport_AddModuleObjectRef(PyObject *name) {
      PyObject *module_dict = PyImport_GetModuleDict();
      PyObject *m;
      if (PyMapping_GetOptionalItem(module_dict, name, &m) < 0) {
          return NULL;
      }
      if (m != NULL && PyModule_Check(m)) {
          return m;
      }
      Py_XDECREF(m);
      m = PyModule_NewObject(name);
      if (m == NULL)
          return NULL;
      if (PyDict_CheckExact(module_dict)) {
          PyObject *new_m;
          (void)PyDict_SetDefaultRef(module_dict, name, m, &new_m);
          Py_DECREF(m);
          return new_m;
      } else {
           if (PyObject_SetItem(module_dict, name, m) != 0) {
                Py_DECREF(m);
                return NULL;
            }
            return m;
      }
  }
  static PyObject *__Pyx_PyImport_AddModuleRef(const char *name) {
      PyObject *py_name = PyUnicode_FromString(name);
      if (!py_name) return NULL;
      PyObject *module = __Pyx_PyImport_AddModuleObjectRef(py_name);
      Py_DECREF(py_name);
      return module;
  }
#elif __PYX_LIMITED_VERSION_HEX >= 0x030d0000
  #define __Pyx_PyImport_AddModuleRef(name) PyImport_AddModuleRef(name)
#else
  static PyObject *__Pyx_PyImport_AddModuleRef(const char *name) {
      PyObject *module = PyImport_AddModule(name);
      Py_XINCREF(module);
      return module;
  }
#endif

/* FetchSharedCythonModule (used by FetchCommonType) */
static PyObject *__Pyx_FetchSharedCythonABIModule(void) {
    return __Pyx_PyImport_AddModuleRef(__PYX_ABI_MODULE_NAME);
}

/* FetchCommonType (used by CommonTypesMetaclass) */
#if __PYX_LIMITED_VERSION_HEX < 0x030C0000
static PyObject* __Pyx_PyType_FromMetaclass(PyTypeObject *metaclass, PyObject *module, PyType_Spec *spec, PyObject *bases) {
    PyObject *result = __Pyx_PyType_FromModuleAndSpec(module, spec, bases);
    if (result && metaclass) {
        PyObject *old_tp = (PyObject*)Py_TYPE(result);
    Py_INCREF((PyObject*)metaclass);
#if __PYX_LIMITED_VERSION_HEX >= 0x03090000
        Py_SET_TYPE(result, metaclass);
#else
        result->ob_type = metaclass;
#endif
        Py_DECREF(old_tp);
    }
    return result;
}
#else
#define __Pyx_PyType_FromMetaclass(me, mo, s, b) PyType_FromMetaclass(me, mo, s, b)
#endif
static int __Pyx_VerifyCachedType(PyObject *cached_type,
                               const char *name,
                               Py_ssize_t expected_basicsize) {
    Py_ssize_t basicsize;
    if (!PyType_Check(cached_type)) {
        PyErr_Format(PyExc_TypeError,
            "Shared Cython type %.200s is not a type object", name);
        return -1;
    }
    if (expected_basicsize == 0) {
        return 0; // size is inherited, nothing useful to check
    }
#if CYTHON_COMPILING_IN_LIMITED_API
    PyObject *py_basicsize;
    py_basicsize = PyObject_GetAttrString(cached_type, "__basicsize__");
    if (unlikely(!py_basicsize)) return -1;
    basicsize = PyLong_AsSsize_t(py_basicsize);
    Py_DECREF(py_basicsize);
    py_basicsize = NULL;
    if (unlikely(basicsize == (Py_ssize_t)-1) && PyErr_Occurred()) return -1;
#else
    basicsize = ((PyTypeObject*) cached_type)->tp_basicsize;
#endif
    if (basicsize != expected_basicsize) {
        PyErr_Format(PyExc_TypeError,
            "Shared Cython type %.200s has the wrong size, try recompiling",
            name);
        return -1;
    }
    return 0;
}
static PyTypeObject *__Pyx_FetchCommonTypeFromSpec(PyTypeObject *metaclass, PyObject *module, PyType_Spec *spec, PyObject *bases) {
    PyObject *abi_module = NULL, *cached_type = NULL, *abi_module_dict, *new_cached_type, *py_object_name;
    int get_item_ref_result;
    const char* object_name = strrchr(spec->name, '.');
    object_name = object_name ? object_name+1 : spec->name;
    py_object_name = PyUnicode_FromString(object_name);
    if (!py_object_name) return NULL;
    abi_module = __Pyx_FetchSharedCythonABIModule();
    if (!abi_module) goto done;
    abi_module_dict = PyModule_GetDict(abi_module);
    if (!abi_module_dict) goto done;
    get_item_ref_result = __Pyx_PyDict_GetItemRef(abi_module_dict, py_object_name, &cached_type);
    if (get_item_ref_result == 1) {
        if (__Pyx_VerifyCachedType(
              cached_type,
              object_name,
              spec->basicsize) < 0) {
            goto bad;
        }
        goto done;
    } else if (unlikely(get_item_ref_result == -1)) {
        goto bad;
    }
    cached_type = __Pyx_PyType_FromMetaclass(
        metaclass,
        CYTHON_USE_MODULE_STATE ? module : abi_module,
        spec, bases);
    if (unlikely(!cached_type)) goto bad;
    if (unlikely(__Pyx_fix_up_extension_type_from_spec(spec, (PyTypeObject *) cached_type) < 0)) goto bad;
    new_cached_type = __Pyx_PyDict_SetDefault(abi_module_dict, py_object_name, cached_type);
    if (unlikely(new_cached_type != cached_type)) {
        if (unlikely(!new_cached_type)) goto bad;
        Py_DECREF(cached_type);
        cached_type = new_cached_type;
        if (__Pyx_VerifyCachedType(
                cached_type,
                object_name,
                spec->basicsize) < 0) {
            goto bad;
        }
        goto done;
    } else {
        Py_DECREF(new_cached_type);
    }
done:
    Py_XDECREF(abi_module);
    Py_DECREF(py_object_name);
    assert(cached_type == NULL || PyType_Check(cached_type));
    return (PyTypeObject *) cached_type;
bad:
    Py_XDECREF(cached_type);
    cached_type = NULL;
    goto done;
}

/* CommonTypesMetaclass (used by CythonFunctionShared) */
static PyObject* __pyx_CommonTypesMetaclass_get_module(CYTHON_UNUSED PyObject *self, CYTHON_UNUSED void* context) {
    return PyUnicode_FromString(__PYX_ABI_MODULE_NAME);
}
#if __PYX_LIMITED_VERSION_HEX < 0x030A0000
static PyObject* __pyx_CommonTypesMetaclass_call(CYTHON_UNUSED PyObject *self, CYTHON_UNUSED PyObject *args, CYTHON_UNUSED PyObject *kwds) {
    PyErr_SetString(PyExc_TypeError, "Cannot instantiate Cython internal types");
    return NULL;
}
static int __pyx_CommonTypesMetaclass_setattr(CYTHON_UNUSED PyObject *self, CYTHON_UNUSED PyObject *attr, CYTHON_UNUSED PyObject *value) {
    PyErr_SetString(PyExc_TypeError, "Cython internal types are immutable");
    return -1;
}
#endif
static PyGetSetDef __pyx_CommonTypesMetaclass_getset[] = {
    {"__module__", __pyx_CommonTypesMetaclass_get_module, NULL, NULL, NULL},
    {0, 0, 0, 0, 0}
};
static PyType_Slot __pyx_CommonTypesMetaclass_slots[] = {
    {Py_tp_getset, (void *)__pyx_CommonTypesMetaclass_getset},
    #if __PYX_LIMITED_VERSION_HEX < 0x030A0000
    {Py_tp_call, (void*)__pyx_CommonTypesMetaclass_call},
    {Py_tp_new, (void*)__pyx_CommonTypesMetaclass_call},
    {Py_tp_setattro, (void*)__pyx_CommonTypesMetaclass_setattr},
    #endif
    {0, 0}
};
static PyType_Spec __pyx_CommonTypesMetaclass_spec = {
    __PYX_TYPE_MODULE_PREFIX "_common_types_metatype",
    0,
    0,
    Py_TPFLAGS_IMMUTABLETYPE |
    Py_TPFLAGS_DISALLOW_INSTANTIATION |
    Py_TPFLAGS_DEFAULT,
    __pyx_CommonTypesMetaclass_slots
};
static int __pyx_CommonTypesMetaclass_init(PyObject *module) {
    __pyx_mstatetype *mstate = __Pyx_PyModule_GetState(module);
    PyObject *bases = PyTuple_Pack(1, &PyType_Type);
    if (unlikely(!bases)) {
        return -1;
    }
    mstate->__pyx_CommonTypesMetaclassType = __Pyx_FetchCommonTypeFromSpec(NULL, module, &__pyx_CommonTypesMetaclass_spec, bases);
    Py_DECREF(bases);
    if (unlikely(mstate->__pyx_CommonTypesMetaclassType == NULL)) {
        return -1;
    }
    return 0;
}

/* PyMethodNew (used by CythonFunctionShared) */
#if CYTHON_COMPILING_IN_LIMITED_API
static PyObject *__Pyx_PyMethod_New(PyObject *func, PyObject *self, PyObject *typ) {
    PyObject *result;
    CYTHON_UNUSED_VAR(typ);
    if (!self)
        return __Pyx_NewRef(func);
    #if __PYX_LIMITED_VERSION_HEX >= 0x030C0000
    {
        PyObject *args[] = {func, self};
        result = PyObject_Vectorcall(__pyx_mstate_global->__Pyx_CachedMethodType, args, 2, NULL);
    }
    #else
    result = PyObject_CallFunctionObjArgs(__pyx_mstate_global->__Pyx_CachedMethodType, func, self, NULL);
    #endif
    return result;
}
#else
static PyObject *__Pyx_PyMethod_New(PyObject *func, PyObject *self, PyObject *typ) {
    CYTHON_UNUSED_VAR(typ);
    if (!self)
        return __Pyx_NewRef(func);
    return PyMethod_New(func, self);
}
#endif

/* PyVectorcallFastCallDict (used by CythonFunctionShared) */
#if CYTHON_METH_FASTCALL && CYTHON_VECTORCALL
static PyObject *__Pyx_PyVectorcall_FastCallDict_kw(PyObject *func, __pyx_vectorcallfunc vc, PyObject *const *args, size_t nargs, PyObject *kw)
{
    PyObject *res = NULL;
    PyObject *kwnames;
    PyObject **newargs;
    PyObject **kwvalues;
    Py_ssize_t i;
    #if CYTHON_AVOID_BORROWED_REFS
    PyObject *pos;
    #else
    Py_ssize_t pos;
    #endif
    size_t j;
    PyObject *key, *value;
    unsigned long keys_are_strings;
    #if !CYTHON_ASSUME_SAFE_SIZE
    Py_ssize_t nkw = PyDict_Size(kw);
    if (unlikely(nkw == -1)) return NULL;
    #else
    Py_ssize_t nkw = PyDict_GET_SIZE(kw);
    #endif
    newargs = (PyObject **)PyMem_Malloc((nargs + (size_t)nkw) * sizeof(args[0]));
    if (unlikely(newargs == NULL)) {
        PyErr_NoMemory();
        return NULL;
    }
    for (j = 0; j < nargs; j++) newargs[j] = args[j];
    kwnames = PyTuple_New(nkw);
    if (unlikely(kwnames == NULL)) {
        PyMem_Free(newargs);
        return NULL;
    }
    kwvalues = newargs + nargs;
    pos = 0;
    i = 0;
    keys_are_strings = Py_TPFLAGS_UNICODE_SUBCLASS;
    while (__Pyx_PyDict_NextRef(kw, &pos, &key, &value)) {
        keys_are_strings &=
        #if CYTHON_COMPILING_IN_LIMITED_API
            PyType_GetFlags(Py_TYPE(key));
        #else
            Py_TYPE(key)->tp_flags;
        #endif
        #if !CYTHON_ASSUME_SAFE_MACROS
        if (unlikely(PyTuple_SetItem(kwnames, i, key) < 0)) goto cleanup;
        #else
        PyTuple_SET_ITEM(kwnames, i, key);
        #endif
        kwvalues[i] = value;
        i++;
    }
    if (unlikely(!keys_are_strings)) {
        PyErr_SetString(PyExc_TypeError, "keywords must be strings");
        goto cleanup;
    }
    res = vc(func, newargs, nargs, kwnames);
cleanup:
    #if CYTHON_AVOID_BORROWED_REFS
    Py_DECREF(pos);
    #endif
    Py_DECREF(kwnames);
    for (i = 0; i < nkw; i++)
        Py_DECREF(kwvalues[i]);
    PyMem_Free(newargs);
    return res;
}
static CYTHON_INLINE PyObject *__Pyx_PyVectorcall_FastCallDict(PyObject *func, __pyx_vectorcallfunc vc, PyObject *const *args, size_t nargs, PyObject *kw)
{
    Py_ssize_t kw_size =
        likely(kw == NULL) ?
        0 :
#if !CYTHON_ASSUME_SAFE_SIZE
        PyDict_Size(kw);
#else
        PyDict_GET_SIZE(kw);
#endif
    if (kw_size == 0) {
        return vc(func, args, nargs, NULL);
    }
#if !CYTHON_ASSUME_SAFE_SIZE
    else if (unlikely(kw_size == -1)) {
        return NULL;
    }
#endif
    return __Pyx_PyVectorcall_FastCallDict_kw(func, vc, args, nargs, kw);
}
#endif

/* CythonFunctionShared (used by CythonFunction) */
#if CYTHON_COMPILING_IN_LIMITED_API
static CYTHON_INLINE int __Pyx__IsSameCyOrCFunctionNoMethod(PyObject *func, void (*cfunc)(void)) {
    if (__Pyx_CyFunction_Check(func)) {
        return PyCFunction_GetFunction(((__pyx_CyFunctionObject*)func)->func) == (PyCFunction) cfunc;
    } else if (PyCFunction_Check(func)) {
        return PyCFunction_GetFunction(func) == (PyCFunction) cfunc;
    }
    return 0;
}
static CYTHON_INLINE int __Pyx__IsSameCyOrCFunction(PyObject *func, void (*cfunc)(void)) {
    if ((PyObject*)Py_TYPE(func) == __pyx_mstate_global->__Pyx_CachedMethodType) {
        int result;
        PyObject *newFunc = PyObject_GetAttr(func, __pyx_mstate_global->__pyx_n_u_func);
        if (unlikely(!newFunc)) {
            PyErr_Clear(); // It's only an optimization, so don't throw an error
            return 0;
        }
        result = __Pyx__IsSameCyOrCFunctionNoMethod(newFunc, cfunc);
        Py_DECREF(newFunc);
        return result;
    }
    return __Pyx__IsSameCyOrCFunctionNoMethod(func, cfunc);
}
#else
static CYTHON_INLINE int __Pyx__IsSameCyOrCFunction(PyObject *func, void (*cfunc)(void)) {
    if (PyMethod_Check(func)) {
        func = PyMethod_GET_FUNCTION(func);
    }
    return __Pyx_CyOrPyCFunction_Check(func) && __Pyx_CyOrPyCFunction_GET_FUNCTION(func) == (PyCFunction) cfunc;
}
#endif
static CYTHON_INLINE void __Pyx__CyFunction_SetClassObj(__pyx_CyFunctionObject* f, PyObject* classobj) {
#if PY_VERSION_HEX < 0x030900B1 || CYTHON_COMPILING_IN_LIMITED_API
    __Pyx_Py_XDECREF_SET(
        __Pyx_CyFunction_GetClassObj(f),
            ((classobj) ? __Pyx_NewRef(classobj) : NULL));
#else
    __Pyx_Py_XDECREF_SET(
        ((PyCMethodObject *) (f))->mm_class,
        (PyTypeObject*)((classobj) ? __Pyx_NewRef(classobj) : NULL));
#endif
}
static PyObject *
__Pyx_CyFunction_get_doc_locked(__pyx_CyFunctionObject *op)
{
    if (unlikely(op->func_doc == NULL)) {
#if CYTHON_COMPILING_IN_LIMITED_API
        op->func_doc = PyObject_GetAttrString(op->func, "__doc__");
        if (unlikely(!op->func_doc)) return NULL;
#else
        if (((PyCFunctionObject*)op)->m_ml->ml_doc) {
            op->func_doc = PyUnicode_FromString(((PyCFunctionObject*)op)->m_ml->ml_doc);
            if (unlikely(op->func_doc == NULL))
                return NULL;
        } else {
            Py_INCREF(Py_None);
            return Py_None;
        }
#endif
    }
    Py_INCREF(op->func_doc);
    return op->func_doc;
}
static PyObject *
__Pyx_CyFunction_get_doc(__pyx_CyFunctionObject *op, void *closure) {
    PyObject *result;
    CYTHON_UNUSED_VAR(closure);
    __Pyx_BEGIN_CRITICAL_SECTION(op);
    result = __Pyx_CyFunction_get_doc_locked(op);
    __Pyx_END_CRITICAL_SECTION();
    return result;
}
static int
__Pyx_CyFunction_set_doc(__pyx_CyFunctionObject *op, PyObject *value, void *context)
{
    CYTHON_UNUSED_VAR(context);
    if (value == NULL) {
        value = Py_None;
    }
    Py_INCREF(value);
    __Pyx_BEGIN_CRITICAL_SECTION(op);
    __Pyx_Py_XDECREF_SET(op->func_doc, value);
    __Pyx_END_CRITICAL_SECTION();
    return 0;
}
static PyObject *
__Pyx_CyFunction_get_name_locked(__pyx_CyFunctionObject *op)
{
    if (unlikely(op->func_name == NULL)) {
#if CYTHON_COMPILING_IN_LIMITED_API
        op->func_name = PyObject_GetAttrString(op->func, "__name__");
#else
        op->func_name = PyUnicode_InternFromString(((PyCFunctionObject*)op)->m_ml->ml_name);
#endif
        if (unlikely(op->func_name == NULL))
            return NULL;
    }
    Py_INCREF(op->func_name);
    return op->func_name;
}
static PyObject *
__Pyx_CyFunction_get_name(__pyx_CyFunctionObject *op, void *context)
{
    PyObject *result = NULL;
    CYTHON_UNUSED_VAR(context);
    __Pyx_BEGIN_CRITICAL_SECTION(op);
    result = __Pyx_CyFunction_get_name_locked(op);
    __Pyx_END_CRITICAL_SECTION();
    return result;
}
static int
__Pyx_CyFunction_set_name(__pyx_CyFunctionObject *op, PyObject *value, void *context)
{
    CYTHON_UNUSED_VAR(context);
    if (unlikely(value == NULL || !PyUnicode_Check(value))) {
        PyErr_SetString(PyExc_TypeError,
                        "__name__ must be set to a string object");
        return -1;
    }
    Py_INCREF(value);
    __Pyx_BEGIN_CRITICAL_SECTION(op);
    __Pyx_Py_XDECREF_SET(op->func_name, value);
    __Pyx_END_CRITICAL_SECTION();
    return 0;
}
static PyObject *
__Pyx_CyFunction_get_qualname(__pyx_CyFunctionObject *op, void *context)
{
    CYTHON_UNUSED_VAR(context);
    PyObject *result;
    __Pyx_BEGIN_CRITICAL_SECTION(op);
    Py_INCREF(op->func_qualname);
    result = op->func_qualname;
    __Pyx_END_CRITICAL_SECTION();
    return result;
}
static int
__Pyx_CyFunction_set_qualname(__pyx_CyFunctionObject *op, PyObject *value, void *context)
{
    CYTHON_UNUSED_VAR(context);
    if (unlikely(value == NULL || !PyUnicode_Check(value))) {
        PyErr_SetString(PyExc_TypeError,
                        "__qualname__ must be set to a string object");
        return -1;
    }
    Py_INCREF(value);
    __Pyx_BEGIN_CRITICAL_SECTION(op);
    __Pyx_Py_XDECREF_SET(op->func_qualname, value);
    __Pyx_END_CRITICAL_SECTION();
    return 0;
}
#if CYTHON_COMPILING_IN_LIMITED_API && __PYX_LIMITED_VERSION_HEX < 0x030A0000
static PyObject *
__Pyx_CyFunction_get_dict(__pyx_CyFunctionObject *op, void *context)
{
    CYTHON_UNUSED_VAR(context);
    if (unlikely(op->func_dict == NULL)) {
        op->func_dict = PyDict_New();
        if (unlikely(op->func_dict == NULL))
            return NULL;
    }
    Py_INCREF(op->func_dict);
    return op->func_dict;
}
#endif
static PyObject *
__Pyx_CyFunction_get_globals(__pyx_CyFunctionObject *op, void *context)
{
    CYTHON_UNUSED_VAR(context);
    Py_INCREF(op->func_globals);
    return op->func_globals;
}
static PyObject *
__Pyx_CyFunction_get_closure(__pyx_CyFunctionObject *op, void *context)
{
    CYTHON_UNUSED_VAR(op);
    CYTHON_UNUSED_VAR(context);
    Py_INCREF(Py_None);
    return Py_None;
}
static PyObject *
__Pyx_CyFunction_get_code(__pyx_CyFunctionObject *op, void *context)
{
    PyObject* result = (op->func_code) ? op->func_code : Py_None;
    CYTHON_UNUSED_VAR(context);
    Py_INCREF(result);
    return result;
}
static int
__Pyx_CyFunction_init_defaults(__pyx_CyFunctionObject *op) {
    int result = 0;
    PyObject *res = op->defaults_getter((PyObject *) op);
    if (unlikely(!res))
        return -1;
    #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
    op->defaults_tuple = PyTuple_GET_ITEM(res, 0);
    Py_INCREF(op->defaults_tuple);
    op->defaults_kwdict = PyTuple_GET_ITEM(res, 1);
    Py_INCREF(op->defaults_kwdict);
    #else
    op->defaults_tuple = __Pyx_PySequence_ITEM(res, 0);
    if (unlikely(!op->defaults_tuple)) result = -1;
    else {
        op->defaults_kwdict = __Pyx_PySequence_ITEM(res, 1);
        if (unlikely(!op->defaults_kwdict)) result = -1;
    }
    #endif
    Py_DECREF(res);
    return result;
}
static int
__Pyx_CyFunction_set_defaults(__pyx_CyFunctionObject *op, PyObject* value, void *context) {
    CYTHON_UNUSED_VAR(context);
    if (!value) {
        value = Py_None;
    } else if (unlikely(value != Py_None && !PyTuple_Check(value))) {
        PyErr_SetString(PyExc_TypeError,
                        "__defaults__ must be set to a tuple object");
        return -1;
    }
    PyErr_WarnEx(PyExc_RuntimeWarning, "changes to cyfunction.__defaults__ will not "
                 "currently affect the values used in function calls", 1);
    Py_INCREF(value);
    __Pyx_BEGIN_CRITICAL_SECTION(op);
    __Pyx_Py_XDECREF_SET(op->defaults_tuple, value);
    __Pyx_END_CRITICAL_SECTION();
    return 0;
}
static PyObject *
__Pyx_CyFunction_get_defaults_locked(__pyx_CyFunctionObject *op) {
    PyObject* result = op->defaults_tuple;
    if (unlikely(!result)) {
        if (op->defaults_getter) {
            if (unlikely(__Pyx_CyFunction_init_defaults(op) < 0)) return NULL;
            result = op->defaults_tuple;
        } else {
            result = Py_None;
        }
    }
    Py_INCREF(result);
    return result;
}
static PyObject *
__Pyx_CyFunction_get_defaults(__pyx_CyFunctionObject *op, void *context) {
    PyObject* result = NULL;
    CYTHON_UNUSED_VAR(context);
    __Pyx_BEGIN_CRITICAL_SECTION(op);
    result = __Pyx_CyFunction_get_defaults_locked(op);
    __Pyx_END_CRITICAL_SECTION();
    return result;
}
static int
__Pyx_CyFunction_set_kwdefaults(__pyx_CyFunctionObject *op, PyObject* value, void *context) {
    CYTHON_UNUSED_VAR(context);
    if (!value) {
        value = Py_None;
    } else if (unlikely(value != Py_None && !PyDict_Check(value))) {
        PyErr_SetString(PyExc_TypeError,
                        "__kwdefaults__ must be set to a dict object");
        return -1;
    }
    PyErr_WarnEx(PyExc_RuntimeWarning, "changes to cyfunction.__kwdefaults__ will not "
                 "currently affect the values used in function calls", 1);
    Py_INCREF(value);
    __Pyx_BEGIN_CRITICAL_SECTION(op);
    __Pyx_Py_XDECREF_SET(op->defaults_kwdict, value);
    __Pyx_END_CRITICAL_SECTION();
    return 0;
}
static PyObject *
__Pyx_CyFunction_get_kwdefaults_locked(__pyx_CyFunctionObject *op) {
    PyObject* result = op->defaults_kwdict;
    if (unlikely(!result)) {
        if (op->defaults_getter) {
            if (unlikely(__Pyx_CyFunction_init_defaults(op) < 0)) return NULL;
            result = op->defaults_kwdict;
        } else {
            result = Py_None;
        }
    }
    Py_INCREF(result);
    return result;
}
static PyObject *
__Pyx_CyFunction_get_kwdefaults(__pyx_CyFunctionObject *op, void *context) {
    PyObject* result;
    CYTHON_UNUSED_VAR(context);
    __Pyx_BEGIN_CRITICAL_SECTION(op);
    result = __Pyx_CyFunction_get_kwdefaults_locked(op);
    __Pyx_END_CRITICAL_SECTION();
    return result;
}
static int
__Pyx_CyFunction_set_annotations(__pyx_CyFunctionObject *op, PyObject* value, void *context) {
    CYTHON_UNUSED_VAR(context);
    if (!value || value == Py_None) {
        value = NULL;
    } else if (unlikely(!PyDict_Check(value))) {
        PyErr_SetString(PyExc_TypeError,
                        "__annotations__ must be set to a dict object");
        return -1;
    }
    Py_XINCREF(value);
    __Pyx_BEGIN_CRITICAL_SECTION(op);
    __Pyx_Py_XDECREF_SET(op->func_annotations, value);
    __Pyx_END_CRITICAL_SECTION();
    return 0;
}
static PyObject *
__Pyx_CyFunction_get_annotations_locked(__pyx_CyFunctionObject *op) {
    PyObject* result = op->func_annotations;
    if (unlikely(!result)) {
        result = PyDict_New();
        if (unlikely(!result)) return NULL;
        op->func_annotations = result;
    }
    Py_INCREF(result);
    return result;
}
static PyObject *
__Pyx_CyFunction_get_annotations(__pyx_CyFunctionObject *op, void *context) {
    PyObject *result;
    CYTHON_UNUSED_VAR(context);
    __Pyx_BEGIN_CRITICAL_SECTION(op);
    result = __Pyx_CyFunction_get_annotations_locked(op);
    __Pyx_END_CRITICAL_SECTION();
    return result;
}
static PyObject *
__Pyx_CyFunction_get_is_coroutine_value(__pyx_CyFunctionObject *op) {
    int is_coroutine = op->flags & __Pyx_CYFUNCTION_COROUTINE;
    if (is_coroutine) {
        PyObject *is_coroutine_value, *module, *fromlist, *marker = __pyx_mstate_global->__pyx_n_u_is_coroutine;
        fromlist = PyList_New(1);
        if (unlikely(!fromlist)) return NULL;
        Py_INCREF(marker);
#if CYTHON_ASSUME_SAFE_MACROS
        PyList_SET_ITEM(fromlist, 0, marker);
#else
        if (unlikely(PyList_SetItem(fromlist, 0, marker) < 0)) {
            Py_DECREF(fromlist);
            return NULL;
        }
#endif
        module = PyImport_ImportModuleLevelObject(__pyx_mstate_global->__pyx_n_u_asyncio_coroutines, NULL, NULL, fromlist, 0);
        Py_DECREF(fromlist);
        if (unlikely(!module)) goto ignore;
        is_coroutine_value = __Pyx_PyObject_GetAttrStr(module, marker);
        Py_DECREF(module);
        if (likely(is_coroutine_value)) {
            return is_coroutine_value;
        }
ignore:
        PyErr_Clear();
    }
    return __Pyx_PyBool_FromLong(is_coroutine);
}
static PyObject *
__Pyx_CyFunction_get_is_coroutine(__pyx_CyFunctionObject *op, void *context) {
    PyObject *result;
    CYTHON_UNUSED_VAR(context);
    if (op->func_is_coroutine) {
        return __Pyx_NewRef(op->func_is_coroutine);
    }
    result = __Pyx_CyFunction_get_is_coroutine_value(op);
    if (unlikely(!result))
        return NULL;
    __Pyx_BEGIN_CRITICAL_SECTION(op);
    if (op->func_is_coroutine) {
        Py_DECREF(result);
        result = __Pyx_NewRef(op->func_is_coroutine);
    } else {
        op->func_is_coroutine = __Pyx_NewRef(result);
    }
    __Pyx_END_CRITICAL_SECTION();
    return result;
}
static void __Pyx_CyFunction_raise_argument_count_error(__pyx_CyFunctionObject *func, const char* message, Py_ssize_t size) {
#if CYTHON_COMPILING_IN_LIMITED_API
    PyObject *py_name = __Pyx_CyFunction_get_name(func, NULL);
    if (!py_name) return;
    PyErr_Format(PyExc_TypeError,
        "%.200S() %s (%" CYTHON_FORMAT_SSIZE_T "d given)",
        py_name, message, size);
    Py_DECREF(py_name);
#else
    const char* name = ((PyCFunctionObject*)func)->m_ml->ml_name;
    PyErr_Format(PyExc_TypeError,
        "%.200s() %s (%" CYTHON_FORMAT_SSIZE_T "d given)",
        name, message, size);
#endif
}
static void __Pyx_CyFunction_raise_type_error(__pyx_CyFunctionObject *func, const char* message) {
#if CYTHON_COMPILING_IN_LIMITED_API
    PyObject *py_name = __Pyx_CyFunction_get_name(func, NULL);
    if (!py_name) return;
    PyErr_Format(PyExc_TypeError,
        "%.200S() %s",
        py_name, message);
    Py_DECREF(py_name);
#else
    const char* name = ((PyCFunctionObject*)func)->m_ml->ml_name;
    PyErr_Format(PyExc_TypeError,
        "%.200s() %s",
        name, message);
#endif
}
#if CYTHON_COMPILING_IN_LIMITED_API
static PyObject *
__Pyx_CyFunction_get_module(__pyx_CyFunctionObject *op, void *context) {
    CYTHON_UNUSED_VAR(context);
    return PyObject_GetAttrString(op->func, "__module__");
}
static int
__Pyx_CyFunction_set_module(__pyx_CyFunctionObject *op, PyObject* value, void *context) {
    CYTHON_UNUSED_VAR(context);
    return PyObject_SetAttrString(op->func, "__module__", value);
}
#endif
static PyGetSetDef __pyx_CyFunction_getsets[] = {
    {"func_doc", (getter)__Pyx_CyFunction_get_doc, (setter)__Pyx_CyFunction_set_doc, 0, 0},
    {"__doc__",  (getter)__Pyx_CyFunction_get_doc, (setter)__Pyx_CyFunction_set_doc, 0, 0},
    {"func_name", (getter)__Pyx_CyFunction_get_name, (setter)__Pyx_CyFunction_set_name, 0, 0},
    {"__name__", (getter)__Pyx_CyFunction_get_name, (setter)__Pyx_CyFunction_set_name, 0, 0},
    {"__qualname__", (getter)__Pyx_CyFunction_get_qualname, (setter)__Pyx_CyFunction_set_qualname, 0, 0},
#if CYTHON_COMPILING_IN_LIMITED_API && __PYX_LIMITED_VERSION_HEX < 0x030A0000
    {"func_dict", (getter)__Pyx_CyFunction_get_dict, (setter)PyObject_GenericSetDict, 0, 0},
    {"__dict__", (getter)__Pyx_CyFunction_get_dict, (setter)PyObject_GenericSetDict, 0, 0},
#else
    {"func_dict", (getter)PyObject_GenericGetDict, (setter)PyObject_GenericSetDict, 0, 0},
    {"__dict__", (getter)PyObject_GenericGetDict, (setter)PyObject_GenericSetDict, 0, 0},
#endif
    {"func_globals", (getter)__Pyx_CyFunction_get_globals, 0, 0, 0},
    {"__globals__", (getter)__Pyx_CyFunction_get_globals, 0, 0, 0},
    {"func_closure", (getter)__Pyx_CyFunction_get_closure, 0, 0, 0},
    {"__closure__", (getter)__Pyx_CyFunction_get_closure, 0, 0, 0},
    {"func_code", (getter)__Pyx_CyFunction_get_code, 0, 0, 0},
    {"__code__", (getter)__Pyx_CyFunction_get_code, 0, 0, 0},
    {"func_defaults", (getter)__Pyx_CyFunction_get_defaults, (setter)__Pyx_CyFunction_set_defaults, 0, 0},
    {"__defaults__", (getter)__Pyx_CyFunction_get_defaults, (setter)__Pyx_CyFunction_set_defaults, 0, 0},
    {"__kwdefaults__", (getter)__Pyx_CyFunction_get_kwdefaults, (setter)__Pyx_CyFunction_set_kwdefaults, 0, 0},
    {"__annotations__", (getter)__Pyx_CyFunction_get_annotations, (setter)__Pyx_CyFunction_set_annotations, 0, 0},
    {"_is_coroutine", (getter)__Pyx_CyFunction_get_is_coroutine, 0, 0, 0},
#if CYTHON_COMPILING_IN_LIMITED_API
    {"__module__", (getter)__Pyx_CyFunction_get_module, (setter)__Pyx_CyFunction_set_module, 0, 0},
#endif
    {0, 0, 0, 0, 0}
};
static PyMemberDef __pyx_CyFunction_members[] = {
#if !CYTHON_COMPILING_IN_LIMITED_API
    {"__module__", T_OBJECT, offsetof(PyCFunctionObject, m_module), 0, 0},
#endif
#if PY_VERSION_HEX < 0x030C0000 || CYTHON_COMPILING_IN_LIMITED_API
    {"__dictoffset__", T_PYSSIZET, offsetof(__pyx_CyFunctionObject, func_dict), READONLY, 0},
#endif
#if CYTHON_METH_FASTCALL
#if CYTHON_COMPILING_IN_LIMITED_API
    {"__vectorcalloffset__", T_PYSSIZET, offsetof(__pyx_CyFunctionObject, func_vectorcall), READONLY, 0},
#else
    {"__vectorcalloffset__", T_PYSSIZET, offsetof(PyCFunctionObject, vectorcall), READONLY, 0},
#endif
#if CYTHON_COMPILING_IN_LIMITED_API
    {"__weaklistoffset__", T_PYSSIZET, offsetof(__pyx_CyFunctionObject, func_weakreflist), READONLY, 0},
#else
    {"__weaklistoffset__", T_PYSSIZET, offsetof(PyCFunctionObject, m_weakreflist), READONLY, 0},
#endif
#endif
    {0, 0, 0,  0, 0}
};
static PyObject *
__Pyx_CyFunction_reduce(__pyx_CyFunctionObject *m, PyObject *args)
{
    PyObject *result = NULL;
    CYTHON_UNUSED_VAR(args);
    __Pyx_BEGIN_CRITICAL_SECTION(m);
    Py_INCREF(m->func_qualname);
    result = m->func_qualname;
    __Pyx_END_CRITICAL_SECTION();
    return result;
}
static PyMethodDef __pyx_CyFunction_methods[] = {
    {"__reduce__", (PyCFunction)__Pyx_CyFunction_reduce, METH_VARARGS, 0},
    {0, 0, 0, 0}
};
#if CYTHON_COMPILING_IN_LIMITED_API
#define __Pyx_CyFunction_weakreflist(cyfunc) ((cyfunc)->func_weakreflist)
#else
#define __Pyx_CyFunction_weakreflist(cyfunc) (((PyCFunctionObject*)cyfunc)->m_weakreflist)
#endif
static PyObject *__Pyx_CyFunction_Init(__pyx_CyFunctionObject *op, PyMethodDef *ml, int flags, PyObject* qualname,
                                       PyObject *closure, PyObject *module, PyObject* globals, PyObject* code) {
#if !CYTHON_COMPILING_IN_LIMITED_API
    PyCFunctionObject *cf = (PyCFunctionObject*) op;
#endif
    if (unlikely(op == NULL))
        return NULL;
#if CYTHON_COMPILING_IN_LIMITED_API
    op->func = PyCFunction_NewEx(ml, (PyObject*)op, module);
    if (unlikely(!op->func)) return NULL;
#endif
    op->flags = flags;
    __Pyx_CyFunction_weakreflist(op) = NULL;
#if !CYTHON_COMPILING_IN_LIMITED_API
    cf->m_ml = ml;
    cf->m_self = (PyObject *) op;
#endif
    Py_XINCREF(closure);
    op->func_closure = closure;
#if !CYTHON_COMPILING_IN_LIMITED_API
    Py_XINCREF(module);
    cf->m_module = module;
#endif
#if PY_VERSION_HEX < 0x030C0000 || CYTHON_COMPILING_IN_LIMITED_API
    op->func_dict = NULL;
#endif
    op->func_name = NULL;
    Py_INCREF(qualname);
    op->func_qualname = qualname;
    op->func_doc = NULL;
#if PY_VERSION_HEX < 0x030900B1 || CYTHON_COMPILING_IN_LIMITED_API
    op->func_classobj = NULL;
#else
    ((PyCMethodObject*)op)->mm_class = NULL;
#endif
    op->func_globals = globals;
    Py_INCREF(op->func_globals);
    Py_XINCREF(code);
    op->func_code = code;
    op->defaults = NULL;
    op->defaults_tuple = NULL;
    op->defaults_kwdict = NULL;
    op->defaults_getter = NULL;
    op->func_annotations = NULL;
    op->func_is_coroutine = NULL;
#if CYTHON_METH_FASTCALL
    switch (ml->ml_flags & (METH_VARARGS | METH_FASTCALL | METH_NOARGS | METH_O | METH_KEYWORDS | METH_METHOD)) {
    case METH_NOARGS:
        __Pyx_CyFunction_func_vectorcall(op) = __Pyx_CyFunction_Vectorcall_NOARGS;
        break;
    case METH_O:
        __Pyx_CyFunction_func_vectorcall(op) = __Pyx_CyFunction_Vectorcall_O;
        break;
    case METH_METHOD | METH_FASTCALL | METH_KEYWORDS:
        __Pyx_CyFunction_func_vectorcall(op) = __Pyx_CyFunction_Vectorcall_FASTCALL_KEYWORDS_METHOD;
        break;
    case METH_FASTCALL | METH_KEYWORDS:
        __Pyx_CyFunction_func_vectorcall(op) = __Pyx_CyFunction_Vectorcall_FASTCALL_KEYWORDS;
        break;
    case METH_VARARGS | METH_KEYWORDS:
        __Pyx_CyFunction_func_vectorcall(op) = NULL;
        break;
    default:
        PyErr_SetString(PyExc_SystemError, "Bad call flags for CyFunction");
        Py_DECREF(op);
        return NULL;
    }
#endif
    return (PyObject *) op;
}
static int
__Pyx_CyFunction_clear(__pyx_CyFunctionObject *m)
{
    Py_CLEAR(m->func_closure);
#if CYTHON_COMPILING_IN_LIMITED_API
    Py_CLEAR(m->func);
#else
    Py_CLEAR(((PyCFunctionObject*)m)->m_module);
#endif
#if PY_VERSION_HEX < 0x030C0000 || CYTHON_COMPILING_IN_LIMITED_API
    Py_CLEAR(m->func_dict);
#elif PY_VERSION_HEX < 0x030d0000
    _PyObject_ClearManagedDict((PyObject*)m);
#else
    PyObject_ClearManagedDict((PyObject*)m);
#endif
    Py_CLEAR(m->func_name);
    Py_CLEAR(m->func_qualname);
    Py_CLEAR(m->func_doc);
    Py_CLEAR(m->func_globals);
    Py_CLEAR(m->func_code);
#if PY_VERSION_HEX < 0x030900B1 || CYTHON_COMPILING_IN_LIMITED_API
    Py_CLEAR(__Pyx_CyFunction_GetClassObj(m));
#else
    {
        PyObject *cls = (PyObject*) ((PyCMethodObject *) (m))->mm_class;
        ((PyCMethodObject *) (m))->mm_class = NULL;
        Py_XDECREF(cls);
    }
#endif
    Py_CLEAR(m->defaults_tuple);
    Py_CLEAR(m->defaults_kwdict);
    Py_CLEAR(m->func_annotations);
    Py_CLEAR(m->func_is_coroutine);
    Py_CLEAR(m->defaults);
    return 0;
}
static void __Pyx__CyFunction_dealloc(__pyx_CyFunctionObject *m)
{
    if (__Pyx_CyFunction_weakreflist(m) != NULL)
        PyObject_ClearWeakRefs((PyObject *) m);
    __Pyx_CyFunction_clear(m);
    __Pyx_PyHeapTypeObject_GC_Del(m);
}
static void __Pyx_CyFunction_dealloc(__pyx_CyFunctionObject *m)
{
    PyObject_GC_UnTrack(m);
    __Pyx__CyFunction_dealloc(m);
}
static int __Pyx_CyFunction_traverse(__pyx_CyFunctionObject *m, visitproc visit, void *arg)
{
    {
        int e = __Pyx_call_type_traverse((PyObject*)m, 1, visit, arg);
        if (e) return e;
    }
    Py_VISIT(m->func_closure);
#if CYTHON_COMPILING_IN_LIMITED_API
    Py_VISIT(m->func);
#else
    Py_VISIT(((PyCFunctionObject*)m)->m_module);
#endif
#if PY_VERSION_HEX < 0x030C0000 || CYTHON_COMPILING_IN_LIMITED_API
    Py_VISIT(m->func_dict);
#else
    {
        int e =
#if PY_VERSION_HEX < 0x030d0000
            _PyObject_VisitManagedDict
#else
            PyObject_VisitManagedDict
#endif
                ((PyObject*)m, visit, arg);
        if (e != 0) return e;
    }
#endif
    __Pyx_VISIT_CONST(m->func_name);
    __Pyx_VISIT_CONST(m->func_qualname);
    Py_VISIT(m->func_doc);
    Py_VISIT(m->func_globals);
    __Pyx_VISIT_CONST(m->func_code);
    Py_VISIT(__Pyx_CyFunction_GetClassObj(m));
    Py_VISIT(m->defaults_tuple);
    Py_VISIT(m->defaults_kwdict);
    Py_VISIT(m->func_annotations);
    Py_VISIT(m->func_is_coroutine);
    Py_VISIT(m->defaults);
    return 0;
}
static PyObject*
__Pyx_CyFunction_repr(__pyx_CyFunctionObject *op)
{
    PyObject *repr;
    __Pyx_BEGIN_CRITICAL_SECTION(op);
    repr = PyUnicode_FromFormat("<cyfunction %U at %p>",
                                op->func_qualname, (void *)op);
    __Pyx_END_CRITICAL_SECTION();
    return repr;
}
static PyObject * __Pyx_CyFunction_CallMethod(PyObject *func, PyObject *self, PyObject *arg, PyObject *kw) {
#if CYTHON_COMPILING_IN_LIMITED_API
    PyObject *f = ((__pyx_CyFunctionObject*)func)->func;
    PyCFunction meth;
    int flags;
    meth = PyCFunction_GetFunction(f);
    if (unlikely(!meth)) return NULL;
    flags = PyCFunction_GetFlags(f);
    if (unlikely(flags < 0)) return NULL;
#else
    PyCFunctionObject* f = (PyCFunctionObject*)func;
    PyCFunction meth = f->m_ml->ml_meth;
    int flags = f->m_ml->ml_flags;
#endif
    Py_ssize_t size;
    switch (flags & (METH_VARARGS | METH_KEYWORDS | METH_NOARGS | METH_O)) {
    case METH_VARARGS:
        if (likely(kw == NULL || PyDict_Size(kw) == 0))
            return (*meth)(self, arg);
        break;
    case METH_VARARGS | METH_KEYWORDS:
        return (*(PyCFunctionWithKeywords)(void(*)(void))meth)(self, arg, kw);
    case METH_NOARGS:
        if (likely(kw == NULL || PyDict_Size(kw) == 0)) {
#if CYTHON_ASSUME_SAFE_SIZE
            size = PyTuple_GET_SIZE(arg);
#else
            size = PyTuple_Size(arg);
            if (unlikely(size < 0)) return NULL;
#endif
            if (likely(size == 0))
                return (*meth)(self, NULL);
            __Pyx_CyFunction_raise_argument_count_error(
                (__pyx_CyFunctionObject*)func,
                "takes no arguments", size);
            return NULL;
        }
        break;
    case METH_O:
        if (likely(kw == NULL || PyDict_Size(kw) == 0)) {
#if CYTHON_ASSUME_SAFE_SIZE
            size = PyTuple_GET_SIZE(arg);
#else
            size = PyTuple_Size(arg);
            if (unlikely(size < 0)) return NULL;
#endif
            if (likely(size == 1)) {
                PyObject *result, *arg0;
                #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
                arg0 = PyTuple_GET_ITEM(arg, 0);
                #else
                arg0 = __Pyx_PySequence_ITEM(arg, 0); if (unlikely(!arg0)) return NULL;
                #endif
                result = (*meth)(self, arg0);
                #if !(CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS)
                Py_DECREF(arg0);
                #endif
                return result;
            }
            __Pyx_CyFunction_raise_argument_count_error(
                (__pyx_CyFunctionObject*)func,
                "takes exactly one argument", size);
            return NULL;
        }
        break;
    default:
        PyErr_SetString(PyExc_SystemError, "Bad call flags for CyFunction");
        return NULL;
    }
    __Pyx_CyFunction_raise_type_error(
        (__pyx_CyFunctionObject*)func, "takes no keyword arguments");
    return NULL;
}
static CYTHON_INLINE PyObject *__Pyx_CyFunction_Call(PyObject *func, PyObject *arg, PyObject *kw) {
    PyObject *self, *result;
#if CYTHON_COMPILING_IN_LIMITED_API
    self = PyCFunction_GetSelf(((__pyx_CyFunctionObject*)func)->func);
    if (unlikely(!self) && PyErr_Occurred()) return NULL;
#else
    self = ((PyCFunctionObject*)func)->m_self;
#endif
    result = __Pyx_CyFunction_CallMethod(func, self, arg, kw);
    return result;
}
static PyObject *__Pyx_CyFunction_CallAsMethod(PyObject *func, PyObject *args, PyObject *kw) {
    PyObject *result;
    __pyx_CyFunctionObject *cyfunc = (__pyx_CyFunctionObject *) func;
#if CYTHON_METH_FASTCALL && CYTHON_VECTORCALL
     __pyx_vectorcallfunc vc = __Pyx_CyFunction_func_vectorcall(cyfunc);
    if (vc) {
#if CYTHON_ASSUME_SAFE_MACROS && CYTHON_ASSUME_SAFE_SIZE
        return __Pyx_PyVectorcall_FastCallDict(func, vc, &PyTuple_GET_ITEM(args, 0), (size_t)PyTuple_GET_SIZE(args), kw);
#else
        (void) &__Pyx_PyVectorcall_FastCallDict;
        return PyVectorcall_Call(func, args, kw);
#endif
    }
#endif
    if ((cyfunc->flags & __Pyx_CYFUNCTION_CCLASS) && !(cyfunc->flags & __Pyx_CYFUNCTION_STATICMETHOD)) {
        Py_ssize_t argc;
        PyObject *new_args;
        PyObject *self;
#if CYTHON_ASSUME_SAFE_SIZE
        argc = PyTuple_GET_SIZE(args);
#else
        argc = PyTuple_Size(args);
        if (unlikely(argc < 0)) return NULL;
#endif
        new_args = PyTuple_GetSlice(args, 1, argc);
        if (unlikely(!new_args))
            return NULL;
        self = PyTuple_GetItem(args, 0);
        if (unlikely(!self)) {
            Py_DECREF(new_args);
            PyErr_Format(PyExc_TypeError,
                         "unbound method %.200S() needs an argument",
                         cyfunc->func_qualname);
            return NULL;
        }
        result = __Pyx_CyFunction_CallMethod(func, self, new_args, kw);
        Py_DECREF(new_args);
    } else {
        result = __Pyx_CyFunction_Call(func, args, kw);
    }
    return result;
}
#if CYTHON_METH_FASTCALL && CYTHON_VECTORCALL
static CYTHON_INLINE int __Pyx_CyFunction_Vectorcall_CheckArgs(__pyx_CyFunctionObject *cyfunc, Py_ssize_t nargs, PyObject *kwnames)
{
    int ret = 0;
    if ((cyfunc->flags & __Pyx_CYFUNCTION_CCLASS) && !(cyfunc->flags & __Pyx_CYFUNCTION_STATICMETHOD)) {
        if (unlikely(nargs < 1)) {
            __Pyx_CyFunction_raise_type_error(
                cyfunc, "needs an argument");
            return -1;
        }
        ret = 1;
    }
    if (unlikely(kwnames) && unlikely(__Pyx_PyTuple_GET_SIZE(kwnames))) {
        __Pyx_CyFunction_raise_type_error(
            cyfunc, "takes no keyword arguments");
        return -1;
    }
    return ret;
}
static PyObject * __Pyx_CyFunction_Vectorcall_NOARGS(PyObject *func, PyObject *const *args, size_t nargsf, PyObject *kwnames)
{
    __pyx_CyFunctionObject *cyfunc = (__pyx_CyFunctionObject *)func;
    Py_ssize_t nargs = PyVectorcall_NARGS(nargsf);
    PyObject *self;
#if CYTHON_COMPILING_IN_LIMITED_API
    PyCFunction meth = PyCFunction_GetFunction(cyfunc->func);
    if (unlikely(!meth)) return NULL;
#else
    PyCFunction meth = ((PyCFunctionObject*)cyfunc)->m_ml->ml_meth;
#endif
    switch (__Pyx_CyFunction_Vectorcall_CheckArgs(cyfunc, nargs, kwnames)) {
    case 1:
        self = args[0];
        args += 1;
        nargs -= 1;
        break;
    case 0:
#if CYTHON_COMPILING_IN_LIMITED_API
        self = PyCFunction_GetSelf(((__pyx_CyFunctionObject*)cyfunc)->func);
        if (unlikely(!self) && PyErr_Occurred()) return NULL;
#else
        self = ((PyCFunctionObject*)cyfunc)->m_self;
#endif
        break;
    default:
        return NULL;
    }
    if (unlikely(nargs != 0)) {
        __Pyx_CyFunction_raise_argument_count_error(
            cyfunc, "takes no arguments", nargs);
        return NULL;
    }
    return meth(self, NULL);
}
static PyObject * __Pyx_CyFunction_Vectorcall_O(PyObject *func, PyObject *const *args, size_t nargsf, PyObject *kwnames)
{
    __pyx_CyFunctionObject *cyfunc = (__pyx_CyFunctionObject *)func;
    Py_ssize_t nargs = PyVectorcall_NARGS(nargsf);
    PyObject *self;
#if CYTHON_COMPILING_IN_LIMITED_API
    PyCFunction meth = PyCFunction_GetFunction(cyfunc->func);
    if (unlikely(!meth)) return NULL;
#else
    PyCFunction meth = ((PyCFunctionObject*)cyfunc)->m_ml->ml_meth;
#endif
    switch (__Pyx_CyFunction_Vectorcall_CheckArgs(cyfunc, nargs, kwnames)) {
    case 1:
        self = args[0];
        args += 1;
        nargs -= 1;
        break;
    case 0:
#if CYTHON_COMPILING_IN_LIMITED_API
        self = PyCFunction_GetSelf(((__pyx_CyFunctionObject*)cyfunc)->func);
        if (unlikely(!self) && PyErr_Occurred()) return NULL;
#else
        self = ((PyCFunctionObject*)cyfunc)->m_self;
#endif
        break;
    default:
        return NULL;
    }
    if (unlikely(nargs != 1)) {
        __Pyx_CyFunction_raise_argument_count_error(
            cyfunc, "takes exactly one argument", nargs);
        return NULL;
    }
    return meth(self, args[0]);
}
static PyObject * __Pyx_CyFunction_Vectorcall_FASTCALL_KEYWORDS(PyObject *func, PyObject *const *args, size_t nargsf, PyObject *kwnames)
{
    __pyx_CyFunctionObject *cyfunc = (__pyx_CyFunctionObject *)func;
    Py_ssize_t nargs = PyVectorcall_NARGS(nargsf);
    PyObject *self;
#if CYTHON_COMPILING_IN_LIMITED_API
    PyCFunction meth = PyCFunction_GetFunction(cyfunc->func);
    if (unlikely(!meth)) return NULL;
#else
    PyCFunction meth = ((PyCFunctionObject*)cyfunc)->m_ml->ml_meth;
#endif
    switch (__Pyx_CyFunction_Vectorcall_CheckArgs(cyfunc, nargs, NULL)) {
    case 1:
        self = args[0];
        args += 1;
        nargs -= 1;
        break;
    case 0:
#if CYTHON_COMPILING_IN_LIMITED_API
        self = PyCFunction_GetSelf(((__pyx_CyFunctionObject*)cyfunc)->func);
        if (unlikely(!self) && PyErr_Occurred()) return NULL;
#else
        self = ((PyCFunctionObject*)cyfunc)->m_self;
#endif
        break;
    default:
        return NULL;
    }
    return ((__Pyx_PyCFunctionFastWithKeywords)(void(*)(void))meth)(self, args, nargs, kwnames);
}
static PyObject * __Pyx_CyFunction_Vectorcall_FASTCALL_KEYWORDS_METHOD(PyObject *func, PyObject *const *args, size_t nargsf, PyObject *kwnames)
{
    __pyx_CyFunctionObject *cyfunc = (__pyx_CyFunctionObject *)func;
    PyTypeObject *cls = (PyTypeObject *) __Pyx_CyFunction_GetClassObj(cyfunc);
    Py_ssize_t nargs = PyVectorcall_NARGS(nargsf);
    PyObject *self;
#if CYTHON_COMPILING_IN_LIMITED_API
    PyCFunction meth = PyCFunction_GetFunction(cyfunc->func);
    if (unlikely(!meth)) return NULL;
#else
    PyCFunction meth = ((PyCFunctionObject*)cyfunc)->m_ml->ml_meth;
#endif
    switch (__Pyx_CyFunction_Vectorcall_CheckArgs(cyfunc, nargs, NULL)) {
    case 1:
        self = args[0];
        args += 1;
        nargs -= 1;
        break;
    case 0:
#if CYTHON_COMPILING_IN_LIMITED_API
        self = PyCFunction_GetSelf(((__pyx_CyFunctionObject*)cyfunc)->func);
        if (unlikely(!self) && PyErr_Occurred()) return NULL;
#else
        self = ((PyCFunctionObject*)cyfunc)->m_self;
#endif
        break;
    default:
        return NULL;
    }
    #if PY_VERSION_HEX < 0x030e00A6
    size_t nargs_value = (size_t) nargs;
    #else
    Py_ssize_t nargs_value = nargs;
    #endif
    return ((__Pyx_PyCMethod)(void(*)(void))meth)(self, cls, args, nargs_value, kwnames);
}
#endif
static PyType_Slot __pyx_CyFunctionType_slots[] = {
    {Py_tp_dealloc, (void *)__Pyx_CyFunction_dealloc},
    {Py_tp_repr, (void *)__Pyx_CyFunction_repr},
    {Py_tp_call, (void *)__Pyx_CyFunction_CallAsMethod},
    {Py_tp_traverse, (void *)__Pyx_CyFunction_traverse},
    {Py_tp_clear, (void *)__Pyx_CyFunction_clear},
    {Py_tp_methods, (void *)__pyx_CyFunction_methods},
    {Py_tp_members, (void *)__pyx_CyFunction_members},
    {Py_tp_getset, (void *)__pyx_CyFunction_getsets},
    {Py_tp_descr_get, (void *)__Pyx_PyMethod_New},
    {0, 0},
};
static PyType_Spec __pyx_CyFunctionType_spec = {
    __PYX_TYPE_MODULE_PREFIX "cython_function_or_method",
    sizeof(__pyx_CyFunctionObject),
    0,
#ifdef Py_TPFLAGS_METHOD_DESCRIPTOR
    Py_TPFLAGS_METHOD_DESCRIPTOR |
#endif
#if CYTHON_METH_FASTCALL
#if defined(Py_TPFLAGS_HAVE_VECTORCALL)
    Py_TPFLAGS_HAVE_VECTORCALL |
#elif defined(_Py_TPFLAGS_HAVE_VECTORCALL)
    _Py_TPFLAGS_HAVE_VECTORCALL |
#endif
#endif // CYTHON_METH_FASTCALL
#if PY_VERSION_HEX >= 0x030C0000 && !CYTHON_COMPILING_IN_LIMITED_API
    Py_TPFLAGS_MANAGED_DICT |
#endif
    Py_TPFLAGS_IMMUTABLETYPE | Py_TPFLAGS_DISALLOW_INSTANTIATION |
    Py_TPFLAGS_DEFAULT | Py_TPFLAGS_HAVE_GC | Py_TPFLAGS_BASETYPE,
    __pyx_CyFunctionType_slots
};
static int __pyx_CyFunction_init(PyObject *module) {
    __pyx_mstatetype *mstate = __Pyx_PyModule_GetState(module);
    mstate->__pyx_CyFunctionType = __Pyx_FetchCommonTypeFromSpec(
        mstate->__pyx_CommonTypesMetaclassType, module, &__pyx_CyFunctionType_spec, NULL);
    if (unlikely(mstate->__pyx_CyFunctionType == NULL)) {
        return -1;
    }
    return 0;
}
static CYTHON_INLINE PyObject *__Pyx_CyFunction_InitDefaults(PyObject *func, PyTypeObject *defaults_type) {
    __pyx_CyFunctionObject *m = (__pyx_CyFunctionObject *) func;
    m->defaults = PyObject_CallObject((PyObject*)defaults_type, NULL); // _PyObject_New(defaults_type);
    if (unlikely(!m->defaults))
        return NULL;
    return m->defaults;
}
static CYTHON_INLINE void __Pyx_CyFunction_SetDefaultsTuple(PyObject *func, PyObject *tuple) {
    __pyx_CyFunctionObject *m = (__pyx_CyFunctionObject *) func;
    m->defaults_tuple = tuple;
    Py_INCREF(tuple);
}
static CYTHON_INLINE void __Pyx_CyFunction_SetDefaultsKwDict(PyObject *func, PyObject *dict) {
    __pyx_CyFunctionObject *m = (__pyx_CyFunctionObject *) func;
    m->defaults_kwdict = dict;
    Py_INCREF(dict);
}
static CYTHON_INLINE void __Pyx_CyFunction_SetAnnotationsDict(PyObject *func, PyObject *dict) {
    __pyx_CyFunctionObject *m = (__pyx_CyFunctionObject *) func;
    m->func_annotations = dict;
    Py_INCREF(dict);
}

/* CythonFunction */
static PyObject *__Pyx_CyFunction_New(PyMethodDef *ml, int flags, PyObject* qualname,
                                      PyObject *closure, PyObject *module, PyObject* globals, PyObject* code) {
    PyObject *op = __Pyx_CyFunction_Init(
        PyObject_GC_New(__pyx_CyFunctionObject, __pyx_mstate_global->__pyx_CyFunctionType),
        ml, flags, qualname, closure, module, globals, code
    );
    if (likely(op)) {
        PyObject_GC_Track(op);
    }
    return op;
}

/* PyObjectLookupSpecial (used by Py3ClassCreate) */
#if CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
static CYTHON_INLINE PyObject* __Pyx__PyObject_LookupSpecial(PyObject* obj, PyObject* attr_name, int with_error) {
    PyObject *res;
    PyTypeObject *tp = Py_TYPE(obj);
    res = _PyType_Lookup(tp, attr_name);
    if (likely(res)) {
        descrgetfunc f = Py_TYPE(res)->tp_descr_get;
        if (!f) {
            Py_INCREF(res);
        } else {
            res = f(res, obj, (PyObject *)tp);
        }
    } else if (with_error) {
        PyErr_SetObject(PyExc_AttributeError, attr_name);
    }
    return res;
}
#endif

/* Py3ClassCreate */
static PyObject *__Pyx_Py3MetaclassPrepare(PyObject *metaclass, PyObject *bases, PyObject *name,
                                           PyObject *qualname, PyObject *mkw, PyObject *modname, PyObject *doc) {
    PyObject *ns;
    if (metaclass) {
        PyObject *prep = __Pyx_PyObject_GetAttrStrNoError(metaclass, __pyx_mstate_global->__pyx_n_u_prepare);
        if (prep) {
            PyObject *pargs[3] = {NULL, name, bases};
            ns = __Pyx_PyObject_FastCallDict(prep, pargs+1, 2 | __Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET, mkw);
            Py_DECREF(prep);
        } else {
            if (unlikely(PyErr_Occurred()))
                return NULL;
            ns = PyDict_New();
        }
    } else {
        ns = PyDict_New();
    }
    if (unlikely(!ns))
        return NULL;
    if (unlikely(PyObject_SetItem(ns, __pyx_mstate_global->__pyx_n_u_module, modname) < 0)) goto bad;
    if (unlikely(PyObject_SetItem(ns, __pyx_mstate_global->__pyx_n_u_qualname, qualname) < 0)) goto bad;
    if (unlikely(doc && PyObject_SetItem(ns, __pyx_mstate_global->__pyx_n_u_doc, doc) < 0)) goto bad;
    return ns;
bad:
    Py_DECREF(ns);
    return NULL;
}
static PyObject *__Pyx_Py3ClassCreate(PyObject *metaclass, PyObject *name, PyObject *bases,
                                      PyObject *dict, PyObject *mkw,
                                      int calculate_metaclass, int allow_py2_metaclass) {
    PyObject *result;
    PyObject *owned_metaclass = NULL;
    PyObject *margs[4] = {NULL, name, bases, dict};
    if (allow_py2_metaclass) {
        owned_metaclass = PyObject_GetItem(dict, __pyx_mstate_global->__pyx_n_u_metaclass);
        if (owned_metaclass) {
            metaclass = owned_metaclass;
        } else if (likely(PyErr_ExceptionMatches(PyExc_KeyError))) {
            PyErr_Clear();
        } else {
            return NULL;
        }
    }
    if (calculate_metaclass && (!metaclass || PyType_Check(metaclass))) {
        metaclass = __Pyx_CalculateMetaclass((PyTypeObject*) metaclass, bases);
        Py_XDECREF(owned_metaclass);
        if (unlikely(!metaclass))
            return NULL;
        owned_metaclass = metaclass;
    }
    result = __Pyx_PyObject_FastCallDict(metaclass, margs+1, 3 | __Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET, mkw);
    Py_XDECREF(owned_metaclass);
    return result;
}

/* CLineInTraceback (used by AddTraceback) */
#if CYTHON_CLINE_IN_TRACEBACK && CYTHON_CLINE_IN_TRACEBACK_RUNTIME
#if CYTHON_COMPILING_IN_LIMITED_API && __PYX_LIMITED_VERSION_HEX < 0x030A0000
#define __Pyx_PyProbablyModule_GetDict(o) __Pyx_XNewRef(PyModule_GetDict(o))
#elif !CYTHON_COMPILING_IN_CPYTHON || CYTHON_COMPILING_IN_CPYTHON_FREETHREADING
#define __Pyx_PyProbablyModule_GetDict(o) PyObject_GenericGetDict(o, NULL);
#else
PyObject* __Pyx_PyProbablyModule_GetDict(PyObject *o) {
    PyObject **dict_ptr = _PyObject_GetDictPtr(o);
    return dict_ptr ? __Pyx_XNewRef(*dict_ptr) : NULL;
}
#endif
static int __Pyx_CLineForTraceback(PyThreadState *tstate, int c_line) {
    PyObject *use_cline = NULL;
    PyObject *ptype, *pvalue, *ptraceback;
    PyObject *cython_runtime_dict;
    CYTHON_MAYBE_UNUSED_VAR(tstate);
    if (unlikely(!__pyx_mstate_global->__pyx_cython_runtime)) {
        return c_line;
    }
    __Pyx_ErrFetchInState(tstate, &ptype, &pvalue, &ptraceback);
    cython_runtime_dict = __Pyx_PyProbablyModule_GetDict(__pyx_mstate_global->__pyx_cython_runtime);
    if (likely(cython_runtime_dict)) {
        __PYX_PY_DICT_LOOKUP_IF_MODIFIED(
            use_cline, cython_runtime_dict,
            __Pyx_PyDict_SetDefault(cython_runtime_dict, __pyx_mstate_global->__pyx_n_u_cline_in_traceback, Py_False))
    }
    if (use_cline == NULL || use_cline == Py_False || (use_cline != Py_True && PyObject_Not(use_cline) != 0)) {
        c_line = 0;
    }
    Py_XDECREF(use_cline);
    Py_XDECREF(cython_runtime_dict);
    __Pyx_ErrRestoreInState(tstate, ptype, pvalue, ptraceback);
    return c_line;
}
#endif

/* CodeObjectCache (used by AddTraceback) */
static int __pyx_bisect_code_objects(__Pyx_CodeObjectCacheEntry* entries, int count, int code_line) {
    int start = 0, mid = 0, end = count - 1;
    if (end >= 0 && code_line > entries[end].code_line) {
        return count;
    }
    while (start < end) {
        mid = start + (end - start) / 2;
        if (code_line < entries[mid].code_line) {
            end = mid;
        } else if (code_line > entries[mid].code_line) {
             start = mid + 1;
        } else {
            return mid;
        }
    }
    if (code_line <= entries[mid].code_line) {
        return mid;
    } else {
        return mid + 1;
    }
}
static __Pyx_CachedCodeObjectType *__pyx__find_code_object(struct __Pyx_CodeObjectCache *code_cache, int code_line) {
    __Pyx_CachedCodeObjectType* code_object;
    int pos;
    if (unlikely(!code_line) || unlikely(!code_cache->entries)) {
        return NULL;
    }
    pos = __pyx_bisect_code_objects(code_cache->entries, code_cache->count, code_line);
    if (unlikely(pos >= code_cache->count) || unlikely(code_cache->entries[pos].code_line != code_line)) {
        return NULL;
    }
    code_object = code_cache->entries[pos].code_object;
    Py_INCREF(code_object);
    return code_object;
}
static __Pyx_CachedCodeObjectType *__pyx_find_code_object(int code_line) {
#if CYTHON_COMPILING_IN_CPYTHON_FREETHREADING && !CYTHON_ATOMICS
    (void)__pyx__find_code_object;
    return NULL; // Most implementation should have atomics. But otherwise, don't make it thread-safe, just miss.
#else
    struct __Pyx_CodeObjectCache *code_cache = &__pyx_mstate_global->__pyx_code_cache;
#if CYTHON_COMPILING_IN_CPYTHON_FREETHREADING
    __pyx_nonatomic_int_type old_count = __pyx_atomic_incr_acq_rel(&code_cache->accessor_count);
    if (old_count < 0) {
        __pyx_atomic_decr_acq_rel(&code_cache->accessor_count);
        return NULL;
    }
#endif
    __Pyx_CachedCodeObjectType *result = __pyx__find_code_object(code_cache, code_line);
#if CYTHON_COMPILING_IN_CPYTHON_FREETHREADING
    __pyx_atomic_decr_acq_rel(&code_cache->accessor_count);
#endif
    return result;
#endif
}
static void __pyx__insert_code_object(struct __Pyx_CodeObjectCache *code_cache, int code_line, __Pyx_CachedCodeObjectType* code_object)
{
    int pos, i;
    __Pyx_CodeObjectCacheEntry* entries = code_cache->entries;
    if (unlikely(!code_line)) {
        return;
    }
    if (unlikely(!entries)) {
        entries = (__Pyx_CodeObjectCacheEntry*)PyMem_Malloc(64*sizeof(__Pyx_CodeObjectCacheEntry));
        if (likely(entries)) {
            code_cache->entries = entries;
            code_cache->max_count = 64;
            code_cache->count = 1;
            entries[0].code_line = code_line;
            entries[0].code_object = code_object;
            Py_INCREF(code_object);
        }
        return;
    }
    pos = __pyx_bisect_code_objects(code_cache->entries, code_cache->count, code_line);
    if ((pos < code_cache->count) && unlikely(code_cache->entries[pos].code_line == code_line)) {
        __Pyx_CachedCodeObjectType* tmp = entries[pos].code_object;
        entries[pos].code_object = code_object;
        Py_INCREF(code_object);
        Py_DECREF(tmp);
        return;
    }
    if (code_cache->count == code_cache->max_count) {
        int new_max = code_cache->max_count + 64;
        entries = (__Pyx_CodeObjectCacheEntry*)PyMem_Realloc(
            code_cache->entries, ((size_t)new_max) * sizeof(__Pyx_CodeObjectCacheEntry));
        if (unlikely(!entries)) {
            return;
        }
        code_cache->entries = entries;
        code_cache->max_count = new_max;
    }
    for (i=code_cache->count; i>pos; i--) {
        entries[i] = entries[i-1];
    }
    entries[pos].code_line = code_line;
    entries[pos].code_object = code_object;
    code_cache->count++;
    Py_INCREF(code_object);
}
static void __pyx_insert_code_object(int code_line, __Pyx_CachedCodeObjectType* code_object) {
#if CYTHON_COMPILING_IN_CPYTHON_FREETHREADING && !CYTHON_ATOMICS
    (void)__pyx__insert_code_object;
    return; // Most implementation should have atomics. But otherwise, don't make it thread-safe, just fail.
#else
    struct __Pyx_CodeObjectCache *code_cache = &__pyx_mstate_global->__pyx_code_cache;
#if CYTHON_COMPILING_IN_CPYTHON_FREETHREADING
    __pyx_nonatomic_int_type expected = 0;
    if (!__pyx_atomic_int_cmp_exchange(&code_cache->accessor_count, &expected, INT_MIN)) {
        return;
    }
#endif
    __pyx__insert_code_object(code_cache, code_line, code_object);
#if CYTHON_COMPILING_IN_CPYTHON_FREETHREADING
    __pyx_atomic_sub(&code_cache->accessor_count, INT_MIN);
#endif
#endif
}

/* AddTraceback */
#include "compile.h"
#include "frameobject.h"
#include "traceback.h"
#if PY_VERSION_HEX >= 0x030b00a6 && !CYTHON_COMPILING_IN_LIMITED_API && !defined(PYPY_VERSION)
  #ifndef Py_BUILD_CORE
    #define Py_BUILD_CORE 1
  #endif
  #include "internal/pycore_frame.h"
#endif
#if CYTHON_COMPILING_IN_LIMITED_API
static PyObject *__Pyx_PyCode_Replace_For_AddTraceback(PyObject *code, PyObject *scratch_dict,
                                                       PyObject *firstlineno, PyObject *name) {
    PyObject *replace = NULL;
    if (unlikely(PyDict_SetItemString(scratch_dict, "co_firstlineno", firstlineno))) return NULL;
    if (unlikely(PyDict_SetItemString(scratch_dict, "co_name", name))) return NULL;
    replace = PyObject_GetAttrString(code, "replace");
    if (likely(replace)) {
        PyObject *result = PyObject_Call(replace, __pyx_mstate_global->__pyx_empty_tuple, scratch_dict);
        Py_DECREF(replace);
        return result;
    }
    PyErr_Clear();
    return NULL;
}
static void __Pyx_AddTraceback(const char *funcname, int c_line,
                               int py_line, const char *filename) {
    PyObject *code_object = NULL, *py_py_line = NULL, *py_funcname = NULL, *dict = NULL;
    PyObject *replace = NULL, *getframe = NULL, *frame = NULL;
    PyObject *exc_type, *exc_value, *exc_traceback;
    int success = 0;
    if (c_line) {
        c_line = __Pyx_CLineForTraceback(__Pyx_PyThreadState_Current, c_line);
    }
    PyErr_Fetch(&exc_type, &exc_value, &exc_traceback);
    code_object = __pyx_find_code_object(c_line ? -c_line : py_line);
    if (!code_object) {
        code_object = Py_CompileString("_getframe()", filename, Py_eval_input);
        if (unlikely(!code_object)) goto bad;
        py_py_line = PyLong_FromLong(py_line);
        if (unlikely(!py_py_line)) goto bad;
        if (c_line) {
            py_funcname = PyUnicode_FromFormat( "%s (%s:%d)", funcname, __pyx_cfilenm, c_line);
        } else {
            py_funcname = PyUnicode_FromString(funcname);
        }
        if (unlikely(!py_funcname)) goto bad;
        dict = PyDict_New();
        if (unlikely(!dict)) goto bad;
        {
            PyObject *old_code_object = code_object;
            code_object = __Pyx_PyCode_Replace_For_AddTraceback(code_object, dict, py_py_line, py_funcname);
            Py_DECREF(old_code_object);
        }
        if (unlikely(!code_object)) goto bad;
        __pyx_insert_code_object(c_line ? -c_line : py_line, code_object);
    } else {
        dict = PyDict_New();
    }
    getframe = PySys_GetObject("_getframe");
    if (unlikely(!getframe)) goto bad;
    if (unlikely(PyDict_SetItemString(dict, "_getframe", getframe))) goto bad;
    frame = PyEval_EvalCode(code_object, dict, dict);
    if (unlikely(!frame) || frame == Py_None) goto bad;
    success = 1;
  bad:
    PyErr_Restore(exc_type, exc_value, exc_traceback);
    Py_XDECREF(code_object);
    Py_XDECREF(py_py_line);
    Py_XDECREF(py_funcname);
    Py_XDECREF(dict);
    Py_XDECREF(replace);
    if (success) {
        PyTraceBack_Here(
            (struct _frame*)frame);
    }
    Py_XDECREF(frame);
}
#else
static PyCodeObject* __Pyx_CreateCodeObjectForTraceback(
            const char *funcname, int c_line,
            int py_line, const char *filename) {
    PyCodeObject *py_code = NULL;
    PyObject *py_funcname = NULL;
    if (c_line) {
        py_funcname = PyUnicode_FromFormat( "%s (%s:%d)", funcname, __pyx_cfilenm, c_line);
        if (!py_funcname) goto bad;
        funcname = PyUnicode_AsUTF8(py_funcname);
        if (!funcname) goto bad;
    }
    py_code = PyCode_NewEmpty(filename, funcname, py_line);
    Py_XDECREF(py_funcname);
    return py_code;
bad:
    Py_XDECREF(py_funcname);
    return NULL;
}
static void __Pyx_AddTraceback(const char *funcname, int c_line,
                               int py_line, const char *filename) {
    PyCodeObject *py_code = 0;
    PyFrameObject *py_frame = 0;
    PyThreadState *tstate = __Pyx_PyThreadState_Current;
    PyObject *ptype, *pvalue, *ptraceback;
    if (c_line) {
        c_line = __Pyx_CLineForTraceback(tstate, c_line);
    }
    py_code = __pyx_find_code_object(c_line ? -c_line : py_line);
    if (!py_code) {
        __Pyx_ErrFetchInState(tstate, &ptype, &pvalue, &ptraceback);
        py_code = __Pyx_CreateCodeObjectForTraceback(
            funcname, c_line, py_line, filename);
        if (!py_code) {
            /* If the code object creation fails, then we should clear the
               fetched exception references and propagate the new exception */
            Py_XDECREF(ptype);
            Py_XDECREF(pvalue);
            Py_XDECREF(ptraceback);
            goto bad;
        }
        __Pyx_ErrRestoreInState(tstate, ptype, pvalue, ptraceback);
        __pyx_insert_code_object(c_line ? -c_line : py_line, py_code);
    }
    py_frame = PyFrame_New(
        tstate,            /*PyThreadState *tstate,*/
        py_code,           /*PyCodeObject *code,*/
        __pyx_mstate_global->__pyx_d,    /*PyObject *globals,*/
        0                  /*PyObject *locals*/
    );
    if (!py_frame) goto bad;
    __Pyx_PyFrame_SetLineNumber(py_frame, py_line);
    PyTraceBack_Here(py_frame);
bad:
    Py_XDECREF(py_code);
    Py_XDECREF(py_frame);
}
#endif

/* CIntFromPyVerify */
#define __PYX_VERIFY_RETURN_INT(target_type, func_type, func_value)\
    __PYX__VERIFY_RETURN_INT(target_type, func_type, func_value, 0)
#define __PYX_VERIFY_RETURN_INT_EXC(target_type, func_type, func_value)\
    __PYX__VERIFY_RETURN_INT(target_type, func_type, func_value, 1)
#define __PYX__VERIFY_RETURN_INT(target_type, func_type, func_value, exc)\
    {\
        func_type value = func_value;\
        if (sizeof(target_type) < sizeof(func_type)) {\
            if (unlikely(value != (func_type) (target_type) value)) {\
                func_type zero = 0;\
                if (exc && unlikely(value == (func_type)-1 && PyErr_Occurred()))\
                    return (target_type) -1;\
                if (is_unsigned && unlikely(value < zero))\
                    goto raise_neg_overflow;\
                else\
                    goto raise_overflow;\
            }\
        }\
        return (target_type) value;\
    }

/* CheckUnpickleChecksum */
static void __Pyx_RaiseUnpickleChecksumError(long checksum, long checksum1, long checksum2, long checksum3, const char *members) {
    PyObject *pickle_module = PyImport_ImportModule("pickle");
    if (unlikely(!pickle_module)) return;
    PyObject *pickle_error = PyObject_GetAttrString(pickle_module, "PickleError");
    Py_DECREF(pickle_module);
    if (unlikely(!pickle_error)) return;
    if (checksum2 == checksum1) {
        PyErr_Format(pickle_error, "Incompatible checksums (0x%x vs (0x%x) = (%s))",
            checksum, checksum1, members);
    } else if (checksum3 == checksum2) {
        PyErr_Format(pickle_error, "Incompatible checksums (0x%x vs (0x%x, 0x%x) = (%s))",
            checksum, checksum1, checksum2, members);
    } else {
        PyErr_Format(pickle_error, "Incompatible checksums (0x%x vs (0x%x, 0x%x, 0x%x) = (%s))",
            checksum, checksum1, checksum2, checksum3, members);
    }
    Py_DECREF(pickle_error);
}
static int __Pyx_CheckUnpickleChecksum(long checksum, long checksum1, long checksum2, long checksum3, const char *members) {
    int found = 0;
    found |= checksum1 == checksum;
    found |= checksum2 == checksum;
    found |= checksum3 == checksum;
    if (likely(found))
        return 0;
    __Pyx_RaiseUnpickleChecksumError(checksum, checksum1, checksum2, checksum3, members);
    return -1;
}

/* CIntToPy */
static CYTHON_INLINE PyObject* __Pyx_PyLong_From_int(int value) {
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic push
#pragma GCC diagnostic ignored "-Wconversion"
#endif
    const int neg_one = (int) -1, const_zero = (int) 0;
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic pop
#endif
    const int is_unsigned = neg_one > const_zero;
    if (is_unsigned) {
        if (sizeof(int) < sizeof(long)) {
            return PyLong_FromLong((long) value);
        } else if (sizeof(int) <= sizeof(unsigned long)) {
            return PyLong_FromUnsignedLong((unsigned long) value);
#if !CYTHON_COMPILING_IN_PYPY
        } else if (sizeof(int) <= sizeof(unsigned PY_LONG_LONG)) {
            return PyLong_FromUnsignedLongLong((unsigned PY_LONG_LONG) value);
#endif
        }
    } else {
        if (sizeof(int) <= sizeof(long)) {
            return PyLong_FromLong((long) value);
        } else if (sizeof(int) <= sizeof(PY_LONG_LONG)) {
            return PyLong_FromLongLong((PY_LONG_LONG) value);
        }
    }
    {
        unsigned char *bytes = (unsigned char *)&value;
#if !CYTHON_COMPILING_IN_LIMITED_API && PY_VERSION_HEX >= 0x030d00A4
        if (is_unsigned) {
            return PyLong_FromUnsignedNativeBytes(bytes, sizeof(value), -1);
        } else {
            return PyLong_FromNativeBytes(bytes, sizeof(value), -1);
        }
#elif !CYTHON_COMPILING_IN_LIMITED_API && PY_VERSION_HEX < 0x030d0000
        int one = 1; int little = (int)*(unsigned char *)&one;
        return _PyLong_FromByteArray(bytes, sizeof(int),
                                     little, !is_unsigned);
#else
        int one = 1; int little = (int)*(unsigned char *)&one;
        PyObject *from_bytes, *result = NULL, *kwds = NULL;
        PyObject *py_bytes = NULL, *order_str = NULL;
        from_bytes = PyObject_GetAttrString((PyObject*)&PyLong_Type, "from_bytes");
        if (!from_bytes) return NULL;
        py_bytes = PyBytes_FromStringAndSize((char*)bytes, sizeof(int));
        if (!py_bytes) goto limited_bad;
        order_str = PyUnicode_FromString(little ? "little" : "big");
        if (!order_str) goto limited_bad;
        {
            PyObject *args[3+(CYTHON_VECTORCALL ? 1 : 0)] = { NULL, py_bytes, order_str };
            if (!is_unsigned) {
                kwds = __Pyx_MakeVectorcallBuilderKwds(1);
                if (!kwds) goto limited_bad;
                if (__Pyx_VectorcallBuilder_AddArgStr("signed", __Pyx_NewRef(Py_True), kwds, args+3, 0) < 0) goto limited_bad;
            }
            result = __Pyx_Object_Vectorcall_CallFromBuilder(from_bytes, args+1, 2 | __Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET, kwds);
        }
        limited_bad:
        Py_XDECREF(kwds);
        Py_XDECREF(order_str);
        Py_XDECREF(py_bytes);
        Py_XDECREF(from_bytes);
        return result;
#endif
    }
}

/* CIntFromPy */
static CYTHON_INLINE int __Pyx_PyLong_As_int(PyObject *x) {
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic push
#pragma GCC diagnostic ignored "-Wconversion"
#endif
    const int neg_one = (int) -1, const_zero = (int) 0;
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic pop
#endif
    const int is_unsigned = neg_one > const_zero;
    if (unlikely(!PyLong_Check(x))) {
        int val;
        PyObject *tmp = __Pyx_PyNumber_Long(x);
        if (!tmp) return (int) -1;
        val = __Pyx_PyLong_As_int(tmp);
        Py_DECREF(tmp);
        return val;
    }
    if (is_unsigned) {
#if CYTHON_USE_PYLONG_INTERNALS
        if (unlikely(__Pyx_PyLong_IsNeg(x))) {
            goto raise_neg_overflow;
        } else if (__Pyx_PyLong_IsCompact(x)) {
            __PYX_VERIFY_RETURN_INT(int, __Pyx_compact_upylong, __Pyx_PyLong_CompactValueUnsigned(x))
        } else {
            const digit* digits = __Pyx_PyLong_Digits(x);
            assert(__Pyx_PyLong_DigitCount(x) > 1);
            switch (__Pyx_PyLong_DigitCount(x)) {
                case 2:
                    if ((8 * sizeof(int) > 1 * PyLong_SHIFT)) {
                        if ((8 * sizeof(unsigned long) > 2 * PyLong_SHIFT)) {
                            __PYX_VERIFY_RETURN_INT(int, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if ((8 * sizeof(int) >= 2 * PyLong_SHIFT)) {
                            return (int) (((((int)digits[1]) << PyLong_SHIFT) | (int)digits[0]));
                        }
                    }
                    break;
                case 3:
                    if ((8 * sizeof(int) > 2 * PyLong_SHIFT)) {
                        if ((8 * sizeof(unsigned long) > 3 * PyLong_SHIFT)) {
                            __PYX_VERIFY_RETURN_INT(int, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if ((8 * sizeof(int) >= 3 * PyLong_SHIFT)) {
                            return (int) (((((((int)digits[2]) << PyLong_SHIFT) | (int)digits[1]) << PyLong_SHIFT) | (int)digits[0]));
                        }
                    }
                    break;
                case 4:
                    if ((8 * sizeof(int) > 3 * PyLong_SHIFT)) {
                        if ((8 * sizeof(unsigned long) > 4 * PyLong_SHIFT)) {
                            __PYX_VERIFY_RETURN_INT(int, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if ((8 * sizeof(int) >= 4 * PyLong_SHIFT)) {
                            return (int) (((((((((int)digits[3]) << PyLong_SHIFT) | (int)digits[2]) << PyLong_SHIFT) | (int)digits[1]) << PyLong_SHIFT) | (int)digits[0]));
                        }
                    }
                    break;
            }
        }
#endif
#if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX < 0x030C00A7
        if (unlikely(Py_SIZE(x) < 0)) {
            goto raise_neg_overflow;
        }
#else
        {
            int result = PyObject_RichCompareBool(x, Py_False, Py_LT);
            if (unlikely(result < 0))
                return (int) -1;
            if (unlikely(result == 1))
                goto raise_neg_overflow;
        }
#endif
        if ((sizeof(int) <= sizeof(unsigned long))) {
            __PYX_VERIFY_RETURN_INT_EXC(int, unsigned long, PyLong_AsUnsignedLong(x))
        } else if ((sizeof(int) <= sizeof(unsigned PY_LONG_LONG))) {
            __PYX_VERIFY_RETURN_INT_EXC(int, unsigned PY_LONG_LONG, PyLong_AsUnsignedLongLong(x))
        }
    } else {
#if CYTHON_USE_PYLONG_INTERNALS
        if (__Pyx_PyLong_IsCompact(x)) {
            __PYX_VERIFY_RETURN_INT(int, __Pyx_compact_pylong, __Pyx_PyLong_CompactValue(x))
        } else {
            const digit* digits = __Pyx_PyLong_Digits(x);
            assert(__Pyx_PyLong_DigitCount(x) > 1);
            switch (__Pyx_PyLong_SignedDigitCount(x)) {
                case -2:
                    if ((8 * sizeof(int) - 1 > 1 * PyLong_SHIFT)) {
                        if ((8 * sizeof(unsigned long) > 2 * PyLong_SHIFT)) {
                            __PYX_VERIFY_RETURN_INT(int, long, -(long) (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if ((8 * sizeof(int) - 1 > 2 * PyLong_SHIFT)) {
                            return (int) (((int)-1)*(((((int)digits[1]) << PyLong_SHIFT) | (int)digits[0])));
                        }
                    }
                    break;
                case 2:
                    if ((8 * sizeof(int) > 1 * PyLong_SHIFT)) {
                        if ((8 * sizeof(unsigned long) > 2 * PyLong_SHIFT)) {
                            __PYX_VERIFY_RETURN_INT(int, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if ((8 * sizeof(int) - 1 > 2 * PyLong_SHIFT)) {
                            return (int) ((((((int)digits[1]) << PyLong_SHIFT) | (int)digits[0])));
                        }
                    }
                    break;
                case -3:
                    if ((8 * sizeof(int) - 1 > 2 * PyLong_SHIFT)) {
                        if ((8 * sizeof(unsigned long) > 3 * PyLong_SHIFT)) {
                            __PYX_VERIFY_RETURN_INT(int, long, -(long) (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if ((8 * sizeof(int) - 1 > 3 * PyLong_SHIFT)) {
                            return (int) (((int)-1)*(((((((int)digits[2]) << PyLong_SHIFT) | (int)digits[1]) << PyLong_SHIFT) | (int)digits[0])));
                        }
                    }
                    break;
                case 3:
                    if ((8 * sizeof(int) > 2 * PyLong_SHIFT)) {
                        if ((8 * sizeof(unsigned long) > 3 * PyLong_SHIFT)) {
                            __PYX_VERIFY_RETURN_INT(int, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if ((8 * sizeof(int) - 1 > 3 * PyLong_SHIFT)) {
                            return (int) ((((((((int)digits[2]) << PyLong_SHIFT) | (int)digits[1]) << PyLong_SHIFT) | (int)digits[0])));
                        }
                    }
                    break;
                case -4:
                    if ((8 * sizeof(int) - 1 > 3 * PyLong_SHIFT)) {
                        if ((8 * sizeof(unsigned long) > 4 * PyLong_SHIFT)) {
                            __PYX_VERIFY_RETURN_INT(int, long, -(long) (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if ((8 * sizeof(int) - 1 > 4 * PyLong_SHIFT)) {
                            return (int) (((int)-1)*(((((((((int)digits[3]) << PyLong_SHIFT) | (int)digits[2]) << PyLong_SHIFT) | (int)digits[1]) << PyLong_SHIFT) | (int)digits[0])));
                        }
                    }
                    break;
                case 4:
                    if ((8 * sizeof(int) > 3 * PyLong_SHIFT)) {
                        if ((8 * sizeof(unsigned long) > 4 * PyLong_SHIFT)) {
                            __PYX_VERIFY_RETURN_INT(int, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if ((8 * sizeof(int) - 1 > 4 * PyLong_SHIFT)) {
                            return (int) ((((((((((int)digits[3]) << PyLong_SHIFT) | (int)digits[2]) << PyLong_SHIFT) | (int)digits[1]) << PyLong_SHIFT) | (int)digits[0])));
                        }
                    }
                    break;
            }
        }
#endif
        if ((sizeof(int) <= sizeof(long))) {
            __PYX_VERIFY_RETURN_INT_EXC(int, long, PyLong_AsLong(x))
        } else if ((sizeof(int) <= sizeof(PY_LONG_LONG))) {
            __PYX_VERIFY_RETURN_INT_EXC(int, PY_LONG_LONG, PyLong_AsLongLong(x))
        }
    }
    {
        int val;
        int ret = -1;
#if PY_VERSION_HEX >= 0x030d00A6 && !CYTHON_COMPILING_IN_LIMITED_API
        Py_ssize_t bytes_copied = PyLong_AsNativeBytes(
            x, &val, sizeof(val), Py_ASNATIVEBYTES_NATIVE_ENDIAN | (is_unsigned ? Py_ASNATIVEBYTES_UNSIGNED_BUFFER | Py_ASNATIVEBYTES_REJECT_NEGATIVE : 0));
        if (unlikely(bytes_copied == -1)) {
        } else if (unlikely(bytes_copied > (Py_ssize_t) sizeof(val))) {
            goto raise_overflow;
        } else {
            ret = 0;
        }
#elif PY_VERSION_HEX < 0x030d0000 && !(CYTHON_COMPILING_IN_PYPY || CYTHON_COMPILING_IN_LIMITED_API) || defined(_PyLong_AsByteArray)
        int one = 1; int is_little = (int)*(unsigned char *)&one;
        unsigned char *bytes = (unsigned char *)&val;
        ret = _PyLong_AsByteArray((PyLongObject *)x,
                                    bytes, sizeof(val),
                                    is_little, !is_unsigned);
#else
        PyObject *v;
        PyObject *stepval = NULL, *mask = NULL, *shift = NULL;
        int bits, remaining_bits, is_negative = 0;
        int chunk_size = (sizeof(long) < 8) ? 30 : 62;
        if (likely(PyLong_CheckExact(x))) {
            v = __Pyx_NewRef(x);
        } else {
            v = PyNumber_Long(x);
            if (unlikely(!v)) return (int) -1;
            assert(PyLong_CheckExact(v));
        }
        {
            int result = PyObject_RichCompareBool(v, Py_False, Py_LT);
            if (unlikely(result < 0)) {
                Py_DECREF(v);
                return (int) -1;
            }
            is_negative = result == 1;
        }
        if (is_unsigned && unlikely(is_negative)) {
            Py_DECREF(v);
            goto raise_neg_overflow;
        } else if (is_negative) {
            stepval = PyNumber_Invert(v);
            Py_DECREF(v);
            if (unlikely(!stepval))
                return (int) -1;
        } else {
            stepval = v;
        }
        v = NULL;
        val = (int) 0;
        mask = PyLong_FromLong((1L << chunk_size) - 1); if (unlikely(!mask)) goto done;
        shift = PyLong_FromLong(chunk_size); if (unlikely(!shift)) goto done;
        for (bits = 0; bits < (int) sizeof(int) * 8 - chunk_size; bits += chunk_size) {
            PyObject *tmp, *digit;
            long idigit;
            digit = PyNumber_And(stepval, mask);
            if (unlikely(!digit)) goto done;
            idigit = PyLong_AsLong(digit);
            Py_DECREF(digit);
            if (unlikely(idigit < 0)) goto done;
            val |= ((int) idigit) << bits;
            tmp = PyNumber_Rshift(stepval, shift);
            if (unlikely(!tmp)) goto done;
            Py_DECREF(stepval); stepval = tmp;
        }
        Py_DECREF(shift); shift = NULL;
        Py_DECREF(mask); mask = NULL;
        {
            long idigit = PyLong_AsLong(stepval);
            if (unlikely(idigit < 0)) goto done;
            remaining_bits = ((int) sizeof(int) * 8) - bits - (is_unsigned ? 0 : 1);
            if (unlikely(idigit >= (1L << remaining_bits)))
                goto raise_overflow;
            val |= ((int) idigit) << bits;
        }
        if (!is_unsigned) {
            if (unlikely(val & (((int) 1) << (sizeof(int) * 8 - 1))))
                goto raise_overflow;
            if (is_negative)
                val = ~val;
        }
        ret = 0;
    done:
        Py_XDECREF(shift);
        Py_XDECREF(mask);
        Py_XDECREF(stepval);
#endif
        if (unlikely(ret))
            return (int) -1;
        return val;
    }
raise_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "value too large to convert to int");
    return (int) -1;
raise_neg_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "can't convert negative value to int");
    return (int) -1;
}

/* CIntFromPy */
static CYTHON_INLINE size_t __Pyx_PyLong_As_size_t(PyObject *x) {
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic push
#pragma GCC diagnostic ignored "-Wconversion"
#endif
    const size_t neg_one = (size_t) -1, const_zero = (size_t) 0;
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic pop
#endif
    const int is_unsigned = neg_one > const_zero;
    if (unlikely(!PyLong_Check(x))) {
        size_t val;
        PyObject *tmp = __Pyx_PyNumber_Long(x);
        if (!tmp) return (size_t) -1;
        val = __Pyx_PyLong_As_size_t(tmp);
        Py_DECREF(tmp);
        return val;
    }
    if (is_unsigned) {
#if CYTHON_USE_PYLONG_INTERNALS
        if (unlikely(__Pyx_PyLong_IsNeg(x))) {
            goto raise_neg_overflow;
        } else if (__Pyx_PyLong_IsCompact(x)) {
            __PYX_VERIFY_RETURN_INT(size_t, __Pyx_compact_upylong, __Pyx_PyLong_CompactValueUnsigned(x))
        } else {
            const digit* digits = __Pyx_PyLong_Digits(x);
            assert(__Pyx_PyLong_DigitCount(x) > 1);
            switch (__Pyx_PyLong_DigitCount(x)) {
                case 2:
                    if ((8 * sizeof(size_t) > 1 * PyLong_SHIFT)) {
                        if ((8 * sizeof(unsigned long) > 2 * PyLong_SHIFT)) {
                            __PYX_VERIFY_RETURN_INT(size_t, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if ((8 * sizeof(size_t) >= 2 * PyLong_SHIFT)) {
                            return (size_t) (((((size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0]));
                        }
                    }
                    break;
                case 3:
                    if ((8 * sizeof(size_t) > 2 * PyLong_SHIFT)) {
                        if ((8 * sizeof(unsigned long) > 3 * PyLong_SHIFT)) {
                            __PYX_VERIFY_RETURN_INT(size_t, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if ((8 * sizeof(size_t) >= 3 * PyLong_SHIFT)) {
                            return (size_t) (((((((size_t)digits[2]) << PyLong_SHIFT) | (size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0]));
                        }
                    }
                    break;
                case 4:
                    if ((8 * sizeof(size_t) > 3 * PyLong_SHIFT)) {
                        if ((8 * sizeof(unsigned long) > 4 * PyLong_SHIFT)) {
                            __PYX_VERIFY_RETURN_INT(size_t, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if ((8 * sizeof(size_t) >= 4 * PyLong_SHIFT)) {
                            return (size_t) (((((((((size_t)digits[3]) << PyLong_SHIFT) | (size_t)digits[2]) << PyLong_SHIFT) | (size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0]));
                        }
                    }
                    break;
            }
        }
#endif
#if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX < 0x030C00A7
        if (unlikely(Py_SIZE(x) < 0)) {
            goto raise_neg_overflow;
        }
#else
        {
            int result = PyObject_RichCompareBool(x, Py_False, Py_LT);
            if (unlikely(result < 0))
                return (size_t) -1;
            if (unlikely(result == 1))
                goto raise_neg_overflow;
        }
#endif
        if ((sizeof(size_t) <= sizeof(unsigned long))) {
            __PYX_VERIFY_RETURN_INT_EXC(size_t, unsigned long, PyLong_AsUnsignedLong(x))
        } else if ((sizeof(size_t) <= sizeof(unsigned PY_LONG_LONG))) {
            __PYX_VERIFY_RETURN_INT_EXC(size_t, unsigned PY_LONG_LONG, PyLong_AsUnsignedLongLong(x))
        }
    } else {
#if CYTHON_USE_PYLONG_INTERNALS
        if (__Pyx_PyLong_IsCompact(x)) {
            __PYX_VERIFY_RETURN_INT(size_t, __Pyx_compact_pylong, __Pyx_PyLong_CompactValue(x))
        } else {
            const digit* digits = __Pyx_PyLong_Digits(x);
            assert(__Pyx_PyLong_DigitCount(x) > 1);
            switch (__Pyx_PyLong_SignedDigitCount(x)) {
                case -2:
                    if ((8 * sizeof(size_t) - 1 > 1 * PyLong_SHIFT)) {
                        if ((8 * sizeof(unsigned long) > 2 * PyLong_SHIFT)) {
                            __PYX_VERIFY_RETURN_INT(size_t, long, -(long) (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if ((8 * sizeof(size_t) - 1 > 2 * PyLong_SHIFT)) {
                            return (size_t) (((size_t)-1)*(((((size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0])));
                        }
                    }
                    break;
                case 2:
                    if ((8 * sizeof(size_t) > 1 * PyLong_SHIFT)) {
                        if ((8 * sizeof(unsigned long) > 2 * PyLong_SHIFT)) {
                            __PYX_VERIFY_RETURN_INT(size_t, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if ((8 * sizeof(size_t) - 1 > 2 * PyLong_SHIFT)) {
                            return (size_t) ((((((size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0])));
                        }
                    }
                    break;
                case -3:
                    if ((8 * sizeof(size_t) - 1 > 2 * PyLong_SHIFT)) {
                        if ((8 * sizeof(unsigned long) > 3 * PyLong_SHIFT)) {
                            __PYX_VERIFY_RETURN_INT(size_t, long, -(long) (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if ((8 * sizeof(size_t) - 1 > 3 * PyLong_SHIFT)) {
                            return (size_t) (((size_t)-1)*(((((((size_t)digits[2]) << PyLong_SHIFT) | (size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0])));
                        }
                    }
                    break;
                case 3:
                    if ((8 * sizeof(size_t) > 2 * PyLong_SHIFT)) {
                        if ((8 * sizeof(unsigned long) > 3 * PyLong_SHIFT)) {
                            __PYX_VERIFY_RETURN_INT(size_t, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if ((8 * sizeof(size_t) - 1 > 3 * PyLong_SHIFT)) {
                            return (size_t) ((((((((size_t)digits[2]) << PyLong_SHIFT) | (size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0])));
                        }
                    }
                    break;
                case -4:
                    if ((8 * sizeof(size_t) - 1 > 3 * PyLong_SHIFT)) {
                        if ((8 * sizeof(unsigned long) > 4 * PyLong_SHIFT)) {
                            __PYX_VERIFY_RETURN_INT(size_t, long, -(long) (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if ((8 * sizeof(size_t) - 1 > 4 * PyLong_SHIFT)) {
                            return (size_t) (((size_t)-1)*(((((((((size_t)digits[3]) << PyLong_SHIFT) | (size_t)digits[2]) << PyLong_SHIFT) | (size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0])));
                        }
                    }
                    break;
                case 4:
                    if ((8 * sizeof(size_t) > 3 * PyLong_SHIFT)) {
                        if ((8 * sizeof(unsigned long) > 4 * PyLong_SHIFT)) {
                            __PYX_VERIFY_RETURN_INT(size_t, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if ((8 * sizeof(size_t) - 1 > 4 * PyLong_SHIFT)) {
                            return (size_t) ((((((((((size_t)digits[3]) << PyLong_SHIFT) | (size_t)digits[2]) << PyLong_SHIFT) | (size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0])));
                        }
                    }
                    break;
            }
        }
#endif
        if ((sizeof(size_t) <= sizeof(long))) {
            __PYX_VERIFY_RETURN_INT_EXC(size_t, long, PyLong_AsLong(x))
        } else if ((sizeof(size_t) <= sizeof(PY_LONG_LONG))) {
            __PYX_VERIFY_RETURN_INT_EXC(size_t, PY_LONG_LONG, PyLong_AsLongLong(x))
        }
    }
    {
        size_t val;
        int ret = -1;
#if PY_VERSION_HEX >= 0x030d00A6 && !CYTHON_COMPILING_IN_LIMITED_API
        Py_ssize_t bytes_copied = PyLong_AsNativeBytes(
            x, &val, sizeof(val), Py_ASNATIVEBYTES_NATIVE_ENDIAN | (is_unsigned ? Py_ASNATIVEBYTES_UNSIGNED_BUFFER | Py_ASNATIVEBYTES_REJECT_NEGATIVE : 0));
        if (unlikely(bytes_copied == -1)) {
        } else if (unlikely(bytes_copied > (Py_ssize_t) sizeof(val))) {
            goto raise_overflow;
        } else {
            ret = 0;
        }
#elif PY_VERSION_HEX < 0x030d0000 && !(CYTHON_COMPILING_IN_PYPY || CYTHON_COMPILING_IN_LIMITED_API) || defined(_PyLong_AsByteArray)
        int one = 1; int is_little = (int)*(unsigned char *)&one;
        unsigned char *bytes = (unsigned char *)&val;
        ret = _PyLong_AsByteArray((PyLongObject *)x,
                                    bytes, sizeof(val),
                                    is_little, !is_unsigned);
#else
        PyObject *v;
        PyObject *stepval = NULL, *mask = NULL, *shift = NULL;
        int bits, remaining_bits, is_negative = 0;
        int chunk_size = (sizeof(long) < 8) ? 30 : 62;
        if (likely(PyLong_CheckExact(x))) {
            v = __Pyx_NewRef(x);
        } else {
            v = PyNumber_Long(x);
            if (unlikely(!v)) return (size_t) -1;
            assert(PyLong_CheckExact(v));
        }
        {
            int result = PyObject_RichCompareBool(v, Py_False, Py_LT);
            if (unlikely(result < 0)) {
                Py_DECREF(v);
                return (size_t) -1;
            }
            is_negative = result == 1;
        }
        if (is_unsigned && unlikely(is_negative)) {
            Py_DECREF(v);
            goto raise_neg_overflow;
        } else if (is_negative) {
            stepval = PyNumber_Invert(v);
            Py_DECREF(v);
            if (unlikely(!stepval))
                return (size_t) -1;
        } else {
            stepval = v;
        }
        v = NULL;
        val = (size_t) 0;
        mask = PyLong_FromLong((1L << chunk_size) - 1); if (unlikely(!mask)) goto done;
        shift = PyLong_FromLong(chunk_size); if (unlikely(!shift)) goto done;
        for (bits = 0; bits < (int) sizeof(size_t) * 8 - chunk_size; bits += chunk_size) {
            PyObject *tmp, *digit;
            long idigit;
            digit = PyNumber_And(stepval, mask);
            if (unlikely(!digit)) goto done;
            idigit = PyLong_AsLong(digit);
            Py_DECREF(digit);
            if (unlikely(idigit < 0)) goto done;
            val |= ((size_t) idigit) << bits;
            tmp = PyNumber_Rshift(stepval, shift);
            if (unlikely(!tmp)) goto done;
            Py_DECREF(stepval); stepval = tmp;
        }
        Py_DECREF(shift); shift = NULL;
        Py_DECREF(mask); mask = NULL;
        {
            long idigit = PyLong_AsLong(stepval);
            if (unlikely(idigit < 0)) goto done;
            remaining_bits = ((int) sizeof(size_t) * 8) - bits - (is_unsigned ? 0 : 1);
            if (unlikely(idigit >= (1L << remaining_bits)))
                goto raise_overflow;
            val |= ((size_t) idigit) << bits;
        }
        if (!is_unsigned) {
            if (unlikely(val & (((size_t) 1) << (sizeof(size_t) * 8 - 1))))
                goto raise_overflow;
            if (is_negative)
                val = ~val;
        }
        ret = 0;
    done:
        Py_XDECREF(shift);
        Py_XDECREF(mask);
        Py_XDECREF(stepval);
#endif
        if (unlikely(ret))
            return (size_t) -1;
        return val;
    }
raise_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "value too large to convert to size_t");
    return (size_t) -1;
raise_neg_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "can't convert negative value to size_t");
    return (size_t) -1;
}

/* CIntToPy */
static CYTHON_INLINE PyObject* __Pyx_PyLong_From_ptrdiff_t(ptrdiff_t value) {
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic push
#pragma GCC diagnostic ignored "-Wconversion"
#endif
    const ptrdiff_t neg_one = (ptrdiff_t) -1, const_zero = (ptrdiff_t) 0;
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic pop
#endif
    const int is_unsigned = neg_one > const_zero;
    if (is_unsigned) {
        if (sizeof(ptrdiff_t) < sizeof(long)) {
            return PyLong_FromLong((long) value);
        } else if (sizeof(ptrdiff_t) <= sizeof(unsigned long)) {
            return PyLong_FromUnsignedLong((unsigned long) value);
#if !CYTHON_COMPILING_IN_PYPY
        } else if (sizeof(ptrdiff_t) <= sizeof(unsigned PY_LONG_LONG)) {
            return PyLong_FromUnsignedLongLong((unsigned PY_LONG_LONG) value);
#endif
        }
    } else {
        if (sizeof(ptrdiff_t) <= sizeof(long)) {
            return PyLong_FromLong((long) value);
        } else if (sizeof(ptrdiff_t) <= sizeof(PY_LONG_LONG)) {
            return PyLong_FromLongLong((PY_LONG_LONG) value);
        }
    }
    {
        unsigned char *bytes = (unsigned char *)&value;
#if !CYTHON_COMPILING_IN_LIMITED_API && PY_VERSION_HEX >= 0x030d00A4
        if (is_unsigned) {
            return PyLong_FromUnsignedNativeBytes(bytes, sizeof(value), -1);
        } else {
            return PyLong_FromNativeBytes(bytes, sizeof(value), -1);
        }
#elif !CYTHON_COMPILING_IN_LIMITED_API && PY_VERSION_HEX < 0x030d0000
        int one = 1; int little = (int)*(unsigned char *)&one;
        return _PyLong_FromByteArray(bytes, sizeof(ptrdiff_t),
                                     little, !is_unsigned);
#else
        int one = 1; int little = (int)*(unsigned char *)&one;
        PyObject *from_bytes, *result = NULL, *kwds = NULL;
        PyObject *py_bytes = NULL, *order_str = NULL;
        from_bytes = PyObject_GetAttrString((PyObject*)&PyLong_Type, "from_bytes");
        if (!from_bytes) return NULL;
        py_bytes = PyBytes_FromStringAndSize((char*)bytes, sizeof(ptrdiff_t));
        if (!py_bytes) goto limited_bad;
        order_str = PyUnicode_FromString(little ? "little" : "big");
        if (!order_str) goto limited_bad;
        {
            PyObject *args[3+(CYTHON_VECTORCALL ? 1 : 0)] = { NULL, py_bytes, order_str };
            if (!is_unsigned) {
                kwds = __Pyx_MakeVectorcallBuilderKwds(1);
                if (!kwds) goto limited_bad;
                if (__Pyx_VectorcallBuilder_AddArgStr("signed", __Pyx_NewRef(Py_True), kwds, args+3, 0) < 0) goto limited_bad;
            }
            result = __Pyx_Object_Vectorcall_CallFromBuilder(from_bytes, args+1, 2 | __Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET, kwds);
        }
        limited_bad:
        Py_XDECREF(kwds);
        Py_XDECREF(order_str);
        Py_XDECREF(py_bytes);
        Py_XDECREF(from_bytes);
        return result;
#endif
    }
}

/* CIntFromPy */
static CYTHON_INLINE ptrdiff_t __Pyx_PyLong_As_ptrdiff_t(PyObject *x) {
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic push
#pragma GCC diagnostic ignored "-Wconversion"
#endif
    const ptrdiff_t neg_one = (ptrdiff_t) -1, const_zero = (ptrdiff_t) 0;
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic pop
#endif
    const int is_unsigned = neg_one > const_zero;
    if (unlikely(!PyLong_Check(x))) {
        ptrdiff_t val;
        PyObject *tmp = __Pyx_PyNumber_Long(x);
        if (!tmp) return (ptrdiff_t) -1;
        val = __Pyx_PyLong_As_ptrdiff_t(tmp);
        Py_DECREF(tmp);
        return val;
    }
    if (is_unsigned) {
#if CYTHON_USE_PYLONG_INTERNALS
        if (unlikely(__Pyx_PyLong_IsNeg(x))) {
            goto raise_neg_overflow;
        } else if (__Pyx_PyLong_IsCompact(x)) {
            __PYX_VERIFY_RETURN_INT(ptrdiff_t, __Pyx_compact_upylong, __Pyx_PyLong_CompactValueUnsigned(x))
        } else {
            const digit* digits = __Pyx_PyLong_Digits(x);
            assert(__Pyx_PyLong_DigitCount(x) > 1);
            switch (__Pyx_PyLong_DigitCount(x)) {
                case 2:
                    if ((8 * sizeof(ptrdiff_t) > 1 * PyLong_SHIFT)) {
                        if ((8 * sizeof(unsigned long) > 2 * PyLong_SHIFT)) {
                            __PYX_VERIFY_RETURN_INT(ptrdiff_t, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if ((8 * sizeof(ptrdiff_t) >= 2 * PyLong_SHIFT)) {
                            return (ptrdiff_t) (((((ptrdiff_t)digits[1]) << PyLong_SHIFT) | (ptrdiff_t)digits[0]));
                        }
                    }
                    break;
                case 3:
                    if ((8 * sizeof(ptrdiff_t) > 2 * PyLong_SHIFT)) {
                        if ((8 * sizeof(unsigned long) > 3 * PyLong_SHIFT)) {
                            __PYX_VERIFY_RETURN_INT(ptrdiff_t, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if ((8 * sizeof(ptrdiff_t) >= 3 * PyLong_SHIFT)) {
                            return (ptrdiff_t) (((((((ptrdiff_t)digits[2]) << PyLong_SHIFT) | (ptrdiff_t)digits[1]) << PyLong_SHIFT) | (ptrdiff_t)digits[0]));
                        }
                    }
                    break;
                case 4:
                    if ((8 * sizeof(ptrdiff_t) > 3 * PyLong_SHIFT)) {
                        if ((8 * sizeof(unsigned long) > 4 * PyLong_SHIFT)) {
                            __PYX_VERIFY_RETURN_INT(ptrdiff_t, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if ((8 * sizeof(ptrdiff_t) >= 4 * PyLong_SHIFT)) {
                            return (ptrdiff_t) (((((((((ptrdiff_t)digits[3]) << PyLong_SHIFT) | (ptrdiff_t)digits[2]) << PyLong_SHIFT) | (ptrdiff_t)digits[1]) << PyLong_SHIFT) | (ptrdiff_t)digits[0]));
                        }
                    }
                    break;
            }
        }
#endif
#if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX < 0x030C00A7
        if (unlikely(Py_SIZE(x) < 0)) {
            goto raise_neg_overflow;
        }
#else
        {
            int result = PyObject_RichCompareBool(x, Py_False, Py_LT);
            if (unlikely(result < 0))
                return (ptrdiff_t) -1;
            if (unlikely(result == 1))
                goto raise_neg_overflow;
        }
#endif
        if ((sizeof(ptrdiff_t) <= sizeof(unsigned long))) {
            __PYX_VERIFY_RETURN_INT_EXC(ptrdiff_t, unsigned long, PyLong_AsUnsignedLong(x))
        } else if ((sizeof(ptrdiff_t) <= sizeof(unsigned PY_LONG_LONG))) {
            __PYX_VERIFY_RETURN_INT_EXC(ptrdiff_t, unsigned PY_LONG_LONG, PyLong_AsUnsignedLongLong(x))
        }
    } else {
#if CYTHON_USE_PYLONG_INTERNALS
        if (__Pyx_PyLong_IsCompact(x)) {
            __PYX_VERIFY_RETURN_INT(ptrdiff_t, __Pyx_compact_pylong, __Pyx_PyLong_CompactValue(x))
        } else {
            const digit* digits = __Pyx_PyLong_Digits(x);
            assert(__Pyx_PyLong_DigitCount(x) > 1);
            switch (__Pyx_PyLong_SignedDigitCount(x)) {
                case -2:
                    if ((8 * sizeof(ptrdiff_t) - 1 > 1 * PyLong_SHIFT)) {
                        if ((8 * sizeof(unsigned long) > 2 * PyLong_SHIFT)) {
                            __PYX_VERIFY_RETURN_INT(ptrdiff_t, long, -(long) (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if ((8 * sizeof(ptrdiff_t) - 1 > 2 * PyLong_SHIFT)) {
                            return (ptrdiff_t) (((ptrdiff_t)-1)*(((((ptrdiff_t)digits[1]) << PyLong_SHIFT) | (ptrdiff_t)digits[0])));
                        }
                    }
                    break;
                case 2:
                    if ((8 * sizeof(ptrdiff_t) > 1 * PyLong_SHIFT)) {
                        if ((8 * sizeof(unsigned long) > 2 * PyLong_SHIFT)) {
                            __PYX_VERIFY_RETURN_INT(ptrdiff_t, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if ((8 * sizeof(ptrdiff_t) - 1 > 2 * PyLong_SHIFT)) {
                            return (ptrdiff_t) ((((((ptrdiff_t)digits[1]) << PyLong_SHIFT) | (ptrdiff_t)digits[0])));
                        }
                    }
                    break;
                case -3:
                    if ((8 * sizeof(ptrdiff_t) - 1 > 2 * PyLong_SHIFT)) {
                        if ((8 * sizeof(unsigned long) > 3 * PyLong_SHIFT)) {
                            __PYX_VERIFY_RETURN_INT(ptrdiff_t, long, -(long) (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if ((8 * sizeof(ptrdiff_t) - 1 > 3 * PyLong_SHIFT)) {
                            return (ptrdiff_t) (((ptrdiff_t)-1)*(((((((ptrdiff_t)digits[2]) << PyLong_SHIFT) | (ptrdiff_t)digits[1]) << PyLong_SHIFT) | (ptrdiff_t)digits[0])));
                        }
                    }
                    break;
                case 3:
                    if ((8 * sizeof(ptrdiff_t) > 2 * PyLong_SHIFT)) {
                        if ((8 * sizeof(unsigned long) > 3 * PyLong_SHIFT)) {
                            __PYX_VERIFY_RETURN_INT(ptrdiff_t, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if ((8 * sizeof(ptrdiff_t) - 1 > 3 * PyLong_SHIFT)) {
                            return (ptrdiff_t) ((((((((ptrdiff_t)digits[2]) << PyLong_SHIFT) | (ptrdiff_t)digits[1]) << PyLong_SHIFT) | (ptrdiff_t)digits[0])));
                        }
                    }
                    break;
                case -4:
                    if ((8 * sizeof(ptrdiff_t) - 1 > 3 * PyLong_SHIFT)) {
                        if ((8 * sizeof(unsigned long) > 4 * PyLong_SHIFT)) {
                            __PYX_VERIFY_RETURN_INT(ptrdiff_t, long, -(long) (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if ((8 * sizeof(ptrdiff_t) - 1 > 4 * PyLong_SHIFT)) {
                            return (ptrdiff_t) (((ptrdiff_t)-1)*(((((((((ptrdiff_t)digits[3]) << PyLong_SHIFT) | (ptrdiff_t)digits[2]) << PyLong_SHIFT) | (ptrdiff_t)digits[1]) << PyLong_SHIFT) | (ptrdiff_t)digits[0])));
                        }
                    }
                    break;
                case 4:
                    if ((8 * sizeof(ptrdiff_t) > 3 * PyLong_SHIFT)) {
                        if ((8 * sizeof(unsigned long) > 4 * PyLong_SHIFT)) {
                            __PYX_VERIFY_RETURN_INT(ptrdiff_t, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if ((8 * sizeof(ptrdiff_t) - 1 > 4 * PyLong_SHIFT)) {
                            return (ptrdiff_t) ((((((((((ptrdiff_t)digits[3]) << PyLong_SHIFT) | (ptrdiff_t)digits[2]) << PyLong_SHIFT) | (ptrdiff_t)digits[1]) << PyLong_SHIFT) | (ptrdiff_t)digits[0])));
                        }
                    }
                    break;
            }
        }
#endif
        if ((sizeof(ptrdiff_t) <= sizeof(long))) {
            __PYX_VERIFY_RETURN_INT_EXC(ptrdiff_t, long, PyLong_AsLong(x))
        } else if ((sizeof(ptrdiff_t) <= sizeof(PY_LONG_LONG))) {
            __PYX_VERIFY_RETURN_INT_EXC(ptrdiff_t, PY_LONG_LONG, PyLong_AsLongLong(x))
        }
    }
    {
        ptrdiff_t val;
        int ret = -1;
#if PY_VERSION_HEX >= 0x030d00A6 && !CYTHON_COMPILING_IN_LIMITED_API
        Py_ssize_t bytes_copied = PyLong_AsNativeBytes(
            x, &val, sizeof(val), Py_ASNATIVEBYTES_NATIVE_ENDIAN | (is_unsigned ? Py_ASNATIVEBYTES_UNSIGNED_BUFFER | Py_ASNATIVEBYTES_REJECT_NEGATIVE : 0));
        if (unlikely(bytes_copied == -1)) {
        } else if (unlikely(bytes_copied > (Py_ssize_t) sizeof(val))) {
            goto raise_overflow;
        } else {
            ret = 0;
        }
#elif PY_VERSION_HEX < 0x030d0000 && !(CYTHON_COMPILING_IN_PYPY || CYTHON_COMPILING_IN_LIMITED_API) || defined(_PyLong_AsByteArray)
        int one = 1; int is_little = (int)*(unsigned char *)&one;
        unsigned char *bytes = (unsigned char *)&val;
        ret = _PyLong_AsByteArray((PyLongObject *)x,
                                    bytes, sizeof(val),
                                    is_little, !is_unsigned);
#else
        PyObject *v;
        PyObject *stepval = NULL, *mask = NULL, *shift = NULL;
        int bits, remaining_bits, is_negative = 0;
        int chunk_size = (sizeof(long) < 8) ? 30 : 62;
        if (likely(PyLong_CheckExact(x))) {
            v = __Pyx_NewRef(x);
        } else {
            v = PyNumber_Long(x);
            if (unlikely(!v)) return (ptrdiff_t) -1;
            assert(PyLong_CheckExact(v));
        }
        {
            int result = PyObject_RichCompareBool(v, Py_False, Py_LT);
            if (unlikely(result < 0)) {
                Py_DECREF(v);
                return (ptrdiff_t) -1;
            }
            is_negative = result == 1;
        }
        if (is_unsigned && unlikely(is_negative)) {
            Py_DECREF(v);
            goto raise_neg_overflow;
        } else if (is_negative) {
            stepval = PyNumber_Invert(v);
            Py_DECREF(v);
            if (unlikely(!stepval))
                return (ptrdiff_t) -1;
        } else {
            stepval = v;
        }
        v = NULL;
        val = (ptrdiff_t) 0;
        mask = PyLong_FromLong((1L << chunk_size) - 1); if (unlikely(!mask)) goto done;
        shift = PyLong_FromLong(chunk_size); if (unlikely(!shift)) goto done;
        for (bits = 0; bits < (int) sizeof(ptrdiff_t) * 8 - chunk_size; bits += chunk_size) {
            PyObject *tmp, *digit;
            long idigit;
            digit = PyNumber_And(stepval, mask);
            if (unlikely(!digit)) goto done;
            idigit = PyLong_AsLong(digit);
            Py_DECREF(digit);
            if (unlikely(idigit < 0)) goto done;
            val |= ((ptrdiff_t) idigit) << bits;
            tmp = PyNumber_Rshift(stepval, shift);
            if (unlikely(!tmp)) goto done;
            Py_DECREF(stepval); stepval = tmp;
        }
        Py_DECREF(shift); shift = NULL;
        Py_DECREF(mask); mask = NULL;
        {
            long idigit = PyLong_AsLong(stepval);
            if (unlikely(idigit < 0)) goto done;
            remaining_bits = ((int) sizeof(ptrdiff_t) * 8) - bits - (is_unsigned ? 0 : 1);
            if (unlikely(idigit >= (1L << remaining_bits)))
                goto raise_overflow;
            val |= ((ptrdiff_t) idigit) << bits;
        }
        if (!is_unsigned) {
            if (unlikely(val & (((ptrdiff_t) 1) << (sizeof(ptrdiff_t) * 8 - 1))))
                goto raise_overflow;
            if (is_negative)
                val = ~val;
        }
        ret = 0;
    done:
        Py_XDECREF(shift);
        Py_XDECREF(mask);
        Py_XDECREF(stepval);
#endif
        if (unlikely(ret))
            return (ptrdiff_t) -1;
        return val;
    }
raise_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "value too large to convert to ptrdiff_t");
    return (ptrdiff_t) -1;
raise_neg_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "can't convert negative value to ptrdiff_t");
    return (ptrdiff_t) -1;
}

/* CIntFromPy */
static CYTHON_INLINE long __Pyx_PyLong_As_long(PyObject *x) {
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic push
#pragma GCC diagnostic ignored "-Wconversion"
#endif
    const long neg_one = (long) -1, const_zero = (long) 0;
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic pop
#endif
    const int is_unsigned = neg_one > const_zero;
    if (unlikely(!PyLong_Check(x))) {
        long val;
        PyObject *tmp = __Pyx_PyNumber_Long(x);
        if (!tmp) return (long) -1;
        val = __Pyx_PyLong_As_long(tmp);
        Py_DECREF(tmp);
        return val;
    }
    if (is_unsigned) {
#if CYTHON_USE_PYLONG_INTERNALS
        if (unlikely(__Pyx_PyLong_IsNeg(x))) {
            goto raise_neg_overflow;
        } else if (__Pyx_PyLong_IsCompact(x)) {
            __PYX_VERIFY_RETURN_INT(long, __Pyx_compact_upylong, __Pyx_PyLong_CompactValueUnsigned(x))
        } else {
            const digit* digits = __Pyx_PyLong_Digits(x);
            assert(__Pyx_PyLong_DigitCount(x) > 1);
            switch (__Pyx_PyLong_DigitCount(x)) {
                case 2:
                    if ((8 * sizeof(long) > 1 * PyLong_SHIFT)) {
                        if ((8 * sizeof(unsigned long) > 2 * PyLong_SHIFT)) {
                            __PYX_VERIFY_RETURN_INT(long, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if ((8 * sizeof(long) >= 2 * PyLong_SHIFT)) {
                            return (long) (((((long)digits[1]) << PyLong_SHIFT) | (long)digits[0]));
                        }
                    }
                    break;
                case 3:
                    if ((8 * sizeof(long) > 2 * PyLong_SHIFT)) {
                        if ((8 * sizeof(unsigned long) > 3 * PyLong_SHIFT)) {
                            __PYX_VERIFY_RETURN_INT(long, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if ((8 * sizeof(long) >= 3 * PyLong_SHIFT)) {
                            return (long) (((((((long)digits[2]) << PyLong_SHIFT) | (long)digits[1]) << PyLong_SHIFT) | (long)digits[0]));
                        }
                    }
                    break;
                case 4:
                    if ((8 * sizeof(long) > 3 * PyLong_SHIFT)) {
                        if ((8 * sizeof(unsigned long) > 4 * PyLong_SHIFT)) {
                            __PYX_VERIFY_RETURN_INT(long, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if ((8 * sizeof(long) >= 4 * PyLong_SHIFT)) {
                            return (long) (((((((((long)digits[3]) << PyLong_SHIFT) | (long)digits[2]) << PyLong_SHIFT) | (long)digits[1]) << PyLong_SHIFT) | (long)digits[0]));
                        }
                    }
                    break;
            }
        }
#endif
#if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX < 0x030C00A7
        if (unlikely(Py_SIZE(x) < 0)) {
            goto raise_neg_overflow;
        }
#else
        {
            int result = PyObject_RichCompareBool(x, Py_False, Py_LT);
            if (unlikely(result < 0))
                return (long) -1;
            if (unlikely(result == 1))
                goto raise_neg_overflow;
        }
#endif
        if ((sizeof(long) <= sizeof(unsigned long))) {
            __PYX_VERIFY_RETURN_INT_EXC(long, unsigned long, PyLong_AsUnsignedLong(x))
        } else if ((sizeof(long) <= sizeof(unsigned PY_LONG_LONG))) {
            __PYX_VERIFY_RETURN_INT_EXC(long, unsigned PY_LONG_LONG, PyLong_AsUnsignedLongLong(x))
        }
    } else {
#if CYTHON_USE_PYLONG_INTERNALS
        if (__Pyx_PyLong_IsCompact(x)) {
            __PYX_VERIFY_RETURN_INT(long, __Pyx_compact_pylong, __Pyx_PyLong_CompactValue(x))
        } else {
            const digit* digits = __Pyx_PyLong_Digits(x);
            assert(__Pyx_PyLong_DigitCount(x) > 1);
            switch (__Pyx_PyLong_SignedDigitCount(x)) {
                case -2:
                    if ((8 * sizeof(long) - 1 > 1 * PyLong_SHIFT)) {
                        if ((8 * sizeof(unsigned long) > 2 * PyLong_SHIFT)) {
                            __PYX_VERIFY_RETURN_INT(long, long, -(long) (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if ((8 * sizeof(long) - 1 > 2 * PyLong_SHIFT)) {
                            return (long) (((long)-1)*(((((long)digits[1]) << PyLong_SHIFT) | (long)digits[0])));
                        }
                    }
                    break;
                case 2:
                    if ((8 * sizeof(long) > 1 * PyLong_SHIFT)) {
                        if ((8 * sizeof(unsigned long) > 2 * PyLong_SHIFT)) {
                            __PYX_VERIFY_RETURN_INT(long, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if ((8 * sizeof(long) - 1 > 2 * PyLong_SHIFT)) {
                            return (long) ((((((long)digits[1]) << PyLong_SHIFT) | (long)digits[0])));
                        }
                    }
                    break;
                case -3:
                    if ((8 * sizeof(long) - 1 > 2 * PyLong_SHIFT)) {
                        if ((8 * sizeof(unsigned long) > 3 * PyLong_SHIFT)) {
                            __PYX_VERIFY_RETURN_INT(long, long, -(long) (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if ((8 * sizeof(long) - 1 > 3 * PyLong_SHIFT)) {
                            return (long) (((long)-1)*(((((((long)digits[2]) << PyLong_SHIFT) | (long)digits[1]) << PyLong_SHIFT) | (long)digits[0])));
                        }
                    }
                    break;
                case 3:
                    if ((8 * sizeof(long) > 2 * PyLong_SHIFT)) {
                        if ((8 * sizeof(unsigned long) > 3 * PyLong_SHIFT)) {
                            __PYX_VERIFY_RETURN_INT(long, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if ((8 * sizeof(long) - 1 > 3 * PyLong_SHIFT)) {
                            return (long) ((((((((long)digits[2]) << PyLong_SHIFT) | (long)digits[1]) << PyLong_SHIFT) | (long)digits[0])));
                        }
                    }
                    break;
                case -4:
                    if ((8 * sizeof(long) - 1 > 3 * PyLong_SHIFT)) {
                        if ((8 * sizeof(unsigned long) > 4 * PyLong_SHIFT)) {
                            __PYX_VERIFY_RETURN_INT(long, long, -(long) (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if ((8 * sizeof(long) - 1 > 4 * PyLong_SHIFT)) {
                            return (long) (((long)-1)*(((((((((long)digits[3]) << PyLong_SHIFT) | (long)digits[2]) << PyLong_SHIFT) | (long)digits[1]) << PyLong_SHIFT) | (long)digits[0])));
                        }
                    }
                    break;
                case 4:
                    if ((8 * sizeof(long) > 3 * PyLong_SHIFT)) {
                        if ((8 * sizeof(unsigned long) > 4 * PyLong_SHIFT)) {
                            __PYX_VERIFY_RETURN_INT(long, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if ((8 * sizeof(long) - 1 > 4 * PyLong_SHIFT)) {
                            return (long) ((((((((((long)digits[3]) << PyLong_SHIFT) | (long)digits[2]) << PyLong_SHIFT) | (long)digits[1]) << PyLong_SHIFT) | (long)digits[0])));
                        }
                    }
                    break;
            }
        }
#endif
        if ((sizeof(long) <= sizeof(long))) {
            __PYX_VERIFY_RETURN_INT_EXC(long, long, PyLong_AsLong(x))
        } else if ((sizeof(long) <= sizeof(PY_LONG_LONG))) {
            __PYX_VERIFY_RETURN_INT_EXC(long, PY_LONG_LONG, PyLong_AsLongLong(x))
        }
    }
    {
        long val;
        int ret = -1;
#if PY_VERSION_HEX >= 0x030d00A6 && !CYTHON_COMPILING_IN_LIMITED_API
        Py_ssize_t bytes_copied = PyLong_AsNativeBytes(
            x, &val, sizeof(val), Py_ASNATIVEBYTES_NATIVE_ENDIAN | (is_unsigned ? Py_ASNATIVEBYTES_UNSIGNED_BUFFER | Py_ASNATIVEBYTES_REJECT_NEGATIVE : 0));
        if (unlikely(bytes_copied == -1)) {
        } else if (unlikely(bytes_copied > (Py_ssize_t) sizeof(val))) {
            goto raise_overflow;
        } else {
            ret = 0;
        }
#elif PY_VERSION_HEX < 0x030d0000 && !(CYTHON_COMPILING_IN_PYPY || CYTHON_COMPILING_IN_LIMITED_API) || defined(_PyLong_AsByteArray)
        int one = 1; int is_little = (int)*(unsigned char *)&one;
        unsigned char *bytes = (unsigned char *)&val;
        ret = _PyLong_AsByteArray((PyLongObject *)x,
                                    bytes, sizeof(val),
                                    is_little, !is_unsigned);
#else
        PyObject *v;
        PyObject *stepval = NULL, *mask = NULL, *shift = NULL;
        int bits, remaining_bits, is_negative = 0;
        int chunk_size = (sizeof(long) < 8) ? 30 : 62;
        if (likely(PyLong_CheckExact(x))) {
            v = __Pyx_NewRef(x);
        } else {
            v = PyNumber_Long(x);
            if (unlikely(!v)) return (long) -1;
            assert(PyLong_CheckExact(v));
        }
        {
            int result = PyObject_RichCompareBool(v, Py_False, Py_LT);
            if (unlikely(result < 0)) {
                Py_DECREF(v);
                return (long) -1;
            }
            is_negative = result == 1;
        }
        if (is_unsigned && unlikely(is_negative)) {
            Py_DECREF(v);
            goto raise_neg_overflow;
        } else if (is_negative) {
            stepval = PyNumber_Invert(v);
            Py_DECREF(v);
            if (unlikely(!stepval))
                return (long) -1;
        } else {
            stepval = v;
        }
        v = NULL;
        val = (long) 0;
        mask = PyLong_FromLong((1L << chunk_size) - 1); if (unlikely(!mask)) goto done;
        shift = PyLong_FromLong(chunk_size); if (unlikely(!shift)) goto done;
        for (bits = 0; bits < (int) sizeof(long) * 8 - chunk_size; bits += chunk_size) {
            PyObject *tmp, *digit;
            long idigit;
            digit = PyNumber_And(stepval, mask);
            if (unlikely(!digit)) goto done;
            idigit = PyLong_AsLong(digit);
            Py_DECREF(digit);
            if (unlikely(idigit < 0)) goto done;
            val |= ((long) idigit) << bits;
            tmp = PyNumber_Rshift(stepval, shift);
            if (unlikely(!tmp)) goto done;
            Py_DECREF(stepval); stepval = tmp;
        }
        Py_DECREF(shift); shift = NULL;
        Py_DECREF(mask); mask = NULL;
        {
            long idigit = PyLong_AsLong(stepval);
            if (unlikely(idigit < 0)) goto done;
            remaining_bits = ((int) sizeof(long) * 8) - bits - (is_unsigned ? 0 : 1);
            if (unlikely(idigit >= (1L << remaining_bits)))
                goto raise_overflow;
            val |= ((long) idigit) << bits;
        }
        if (!is_unsigned) {
            if (unlikely(val & (((long) 1) << (sizeof(long) * 8 - 1))))
                goto raise_overflow;
            if (is_negative)
                val = ~val;
        }
        ret = 0;
    done:
        Py_XDECREF(shift);
        Py_XDECREF(mask);
        Py_XDECREF(stepval);
#endif
        if (unlikely(ret))
            return (long) -1;
        return val;
    }
raise_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "value too large to convert to long");
    return (long) -1;
raise_neg_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "can't convert negative value to long");
    return (long) -1;
}

/* CIntToPy */
static CYTHON_INLINE PyObject* __Pyx_PyLong_From_uint64_t(uint64_t value) {
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic push
#pragma GCC diagnostic ignored "-Wconversion"
#endif
    const uint64_t neg_one = (uint64_t) -1, const_zero = (uint64_t) 0;
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic pop
#endif
    const int is_unsigned = neg_one > const_zero;
    if (is_unsigned) {
        if (sizeof(uint64_t) < sizeof(long)) {
            return PyLong_FromLong((long) value);
        } else if (sizeof(uint64_t) <= sizeof(unsigned long)) {
            return PyLong_FromUnsignedLong((unsigned long) value);
#if !CYTHON_COMPILING_IN_PYPY
        } else if (sizeof(uint64_t) <= sizeof(unsigned PY_LONG_LONG)) {
            return PyLong_FromUnsignedLongLong((unsigned PY_LONG_LONG) value);
#endif
        }
    } else {
        if (sizeof(uint64_t) <= sizeof(long)) {
            return PyLong_FromLong((long) value);
        } else if (sizeof(uint64_t) <= sizeof(PY_LONG_LONG)) {
            return PyLong_FromLongLong((PY_LONG_LONG) value);
        }
    }
    {
        unsigned char *bytes = (unsigned char *)&value;
#if !CYTHON_COMPILING_IN_LIMITED_API && PY_VERSION_HEX >= 0x030d00A4
        if (is_unsigned) {
            return PyLong_FromUnsignedNativeBytes(bytes, sizeof(value), -1);
        } else {
            return PyLong_FromNativeBytes(bytes, sizeof(value), -1);
        }
#elif !CYTHON_COMPILING_IN_LIMITED_API && PY_VERSION_HEX < 0x030d0000
        int one = 1; int little = (int)*(unsigned char *)&one;
        return _PyLong_FromByteArray(bytes, sizeof(uint64_t),
                                     little, !is_unsigned);
#else
        int one = 1; int little = (int)*(unsigned char *)&one;
        PyObject *from_bytes, *result = NULL, *kwds = NULL;
        PyObject *py_bytes = NULL, *order_str = NULL;
        from_bytes = PyObject_GetAttrString((PyObject*)&PyLong_Type, "from_bytes");
        if (!from_bytes) return NULL;
        py_bytes = PyBytes_FromStringAndSize((char*)bytes, sizeof(uint64_t));
        if (!py_bytes) goto limited_bad;
        order_str = PyUnicode_FromString(little ? "little" : "big");
        if (!order_str) goto limited_bad;
        {
            PyObject *args[3+(CYTHON_VECTORCALL ? 1 : 0)] = { NULL, py_bytes, order_str };
            if (!is_unsigned) {
                kwds = __Pyx_MakeVectorcallBuilderKwds(1);
                if (!kwds) goto limited_bad;
                if (__Pyx_VectorcallBuilder_AddArgStr("signed", __Pyx_NewRef(Py_True), kwds, args+3, 0) < 0) goto limited_bad;
            }
            result = __Pyx_Object_Vectorcall_CallFromBuilder(from_bytes, args+1, 2 | __Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET, kwds);
        }
        limited_bad:
        Py_XDECREF(kwds);
        Py_XDECREF(order_str);
        Py_XDECREF(py_bytes);
        Py_XDECREF(from_bytes);
        return result;
#endif
    }
}

/* CIntToPy */
static CYTHON_INLINE PyObject* __Pyx_PyLong_From_long(long value) {
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic push
#pragma GCC diagnostic ignored "-Wconversion"
#endif
    const long neg_one = (long) -1, const_zero = (long) 0;
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic pop
#endif
    const int is_unsigned = neg_one > const_zero;
    if (is_unsigned) {
        if (sizeof(long) < sizeof(long)) {
            return PyLong_FromLong((long) value);
        } else if (sizeof(long) <= sizeof(unsigned long)) {
            return PyLong_FromUnsignedLong((unsigned long) value);
#if !CYTHON_COMPILING_IN_PYPY
        } else if (sizeof(long) <= sizeof(unsigned PY_LONG_LONG)) {
            return PyLong_FromUnsignedLongLong((unsigned PY_LONG_LONG) value);
#endif
        }
    } else {
        if (sizeof(long) <= sizeof(long)) {
            return PyLong_FromLong((long) value);
        } else if (sizeof(long) <= sizeof(PY_LONG_LONG)) {
            return PyLong_FromLongLong((PY_LONG_LONG) value);
        }
    }
    {
        unsigned char *bytes = (unsigned char *)&value;
#if !CYTHON_COMPILING_IN_LIMITED_API && PY_VERSION_HEX >= 0x030d00A4
        if (is_unsigned) {
            return PyLong_FromUnsignedNativeBytes(bytes, sizeof(value), -1);
        } else {
            return PyLong_FromNativeBytes(bytes, sizeof(value), -1);
        }
#elif !CYTHON_COMPILING_IN_LIMITED_API && PY_VERSION_HEX < 0x030d0000
        int one = 1; int little = (int)*(unsigned char *)&one;
        return _PyLong_FromByteArray(bytes, sizeof(long),
                                     little, !is_unsigned);
#else
        int one = 1; int little = (int)*(unsigned char *)&one;
        PyObject *from_bytes, *result = NULL, *kwds = NULL;
        PyObject *py_bytes = NULL, *order_str = NULL;
        from_bytes = PyObject_GetAttrString((PyObject*)&PyLong_Type, "from_bytes");
        if (!from_bytes) return NULL;
        py_bytes = PyBytes_FromStringAndSize((char*)bytes, sizeof(long));
        if (!py_bytes) goto limited_bad;
        order_str = PyUnicode_FromString(little ? "little" : "big");
        if (!order_str) goto limited_bad;
        {
            PyObject *args[3+(CYTHON_VECTORCALL ? 1 : 0)] = { NULL, py_bytes, order_str };
            if (!is_unsigned) {
                kwds = __Pyx_MakeVectorcallBuilderKwds(1);
                if (!kwds) goto limited_bad;
                if (__Pyx_VectorcallBuilder_AddArgStr("signed", __Pyx_NewRef(Py_True), kwds, args+3, 0) < 0) goto limited_bad;
            }
            result = __Pyx_Object_Vectorcall_CallFromBuilder(from_bytes, args+1, 2 | __Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET, kwds);
        }
        limited_bad:
        Py_XDECREF(kwds);
        Py_XDECREF(order_str);
        Py_XDECREF(py_bytes);
        Py_XDECREF(from_bytes);
        return result;
#endif
    }
}

/* PyObjectCallMethod1 */
#if !(CYTHON_VECTORCALL && (__PYX_LIMITED_VERSION_HEX >= 0x030C0000 || (!CYTHON_COMPILING_IN_LIMITED_API && PY_VERSION_HEX >= 0x03090000)))
static PyObject* __Pyx__PyObject_CallMethod1(PyObject* method, PyObject* arg) {
    PyObject *result = __Pyx_PyObject_CallOneArg(method, arg);
    Py_DECREF(method);
    return result;
}
#endif
static PyObject* __Pyx_PyObject_CallMethod1(PyObject* obj, PyObject* method_name, PyObject* arg) {
#if CYTHON_VECTORCALL && (__PYX_LIMITED_VERSION_HEX >= 0x030C0000 || (!CYTHON_COMPILING_IN_LIMITED_API && PY_VERSION_HEX >= 0x03090000))
    PyObject *args[2] = {obj, arg};
    (void) __Pyx_PyObject_CallOneArg;
    (void) __Pyx_PyObject_Call2Args;
    return PyObject_VectorcallMethod(method_name, args, 2 | PY_VECTORCALL_ARGUMENTS_OFFSET, NULL);
#else
    PyObject *method = NULL, *result;
    int is_method = __Pyx_PyObject_GetMethod(obj, method_name, &method);
    if (likely(is_method)) {
        result = __Pyx_PyObject_Call2Args(method, obj, arg);
        Py_DECREF(method);
        return result;
    }
    if (unlikely(!method)) return NULL;
    return __Pyx__PyObject_CallMethod1(method, arg);
#endif
}

/* UpdateUnpickledDict */
static int __Pyx__UpdateUnpickledDict(PyObject *obj, PyObject *state, Py_ssize_t index) {
    PyObject *state_dict = __Pyx_PySequence_ITEM(state, index);
    if (unlikely(!state_dict)) {
        return -1;
    }
    int non_empty = PyObject_IsTrue(state_dict);
    if (non_empty == 0) {
        Py_DECREF(state_dict);
        return 0;
    } else if (unlikely(non_empty == -1)) {
        return -1;
    }
    PyObject *dict;
    #if CYTHON_COMPILING_IN_LIMITED_API && __PYX_LIMITED_VERSION_HEX < 0x030A0000
    dict = PyObject_GetAttrString(obj, "__dict__");
    #else
    dict = PyObject_GenericGetDict(obj, NULL);
    #endif
    if (unlikely(!dict)) {
        Py_DECREF(state_dict);
        return -1;
    }
    int result;
    if (likely(PyDict_CheckExact(dict))) {
        result = PyDict_Update(dict, state_dict);
    } else {
        PyObject *obj_result = __Pyx_PyObject_CallMethod1(dict, __pyx_mstate_global->__pyx_n_u_update, state_dict);
        if (likely(obj_result)) {
            Py_DECREF(obj_result);
            result = 0;
        } else {
            result = -1;
        }
    }
    Py_DECREF(state_dict);
    Py_DECREF(dict);
    return result;
}
static int __Pyx_UpdateUnpickledDict(PyObject *obj, PyObject *state, Py_ssize_t index) {
    Py_ssize_t state_size = __Pyx_PyTuple_GET_SIZE(state);
    #if !CYTHON_ASSUME_SAFE_SIZE
    if (unlikely(state_size == -1)) return -1;
    #endif
    if (state_size <= index) {
        return 0;
    }
    return __Pyx__UpdateUnpickledDict(obj, state, index);
}

/* FormatTypeName */
#if CYTHON_COMPILING_IN_LIMITED_API && __PYX_LIMITED_VERSION_HEX < 0x030d0000
static __Pyx_TypeName
__Pyx_PyType_GetFullyQualifiedName(PyTypeObject* tp)
{
    PyObject *module = NULL, *name = NULL, *result = NULL;
    #if __PYX_LIMITED_VERSION_HEX < 0x030b0000
    name = __Pyx_PyObject_GetAttrStr((PyObject *)tp,
                                               __pyx_mstate_global->__pyx_n_u_qualname);
    #else
    name = PyType_GetQualName(tp);
    #endif
    if (unlikely(name == NULL) || unlikely(!PyUnicode_Check(name))) goto bad;
    module = __Pyx_PyObject_GetAttrStr((PyObject *)tp,
                                               __pyx_mstate_global->__pyx_n_u_module);
    if (unlikely(module == NULL) || unlikely(!PyUnicode_Check(module))) goto bad;
    if (PyUnicode_CompareWithASCIIString(module, "builtins") == 0) {
        result = name;
        name = NULL;
        goto done;
    }
    result = PyUnicode_FromFormat("%U.%U", module, name);
    if (unlikely(result == NULL)) goto bad;
  done:
    Py_XDECREF(name);
    Py_XDECREF(module);
    return result;
  bad:
    PyErr_Clear();
    if (name) {
        result = name;
        name = NULL;
    } else {
        result = __Pyx_NewRef(__pyx_mstate_global->__pyx_kp_u__7);
    }
    goto done;
}
#endif

/* GetRuntimeVersion */
#if __PYX_LIMITED_VERSION_HEX < 0x030b0000
void __Pyx_init_runtime_version(void) {
    if (__Pyx_cached_runtime_version == 0) {
        const char* rt_version = Py_GetVersion();
        unsigned long version = 0;
        unsigned long factor = 0x01000000UL;
        unsigned int digit = 0;
        int i = 0;
        while (factor) {
            while ('0' <= rt_version[i] && rt_version[i] <= '9') {
                digit = digit * 10 + (unsigned int) (rt_version[i] - '0');
                ++i;
            }
            version += factor * digit;
            if (rt_version[i] != '.')
                break;
            digit = 0;
            factor >>= 8;
            ++i;
        }
        __Pyx_cached_runtime_version = version;
    }
}
#endif
static unsigned long __Pyx_get_runtime_version(void) {
#if __PYX_LIMITED_VERSION_HEX >= 0x030b0000
    return Py_Version & ~0xFFUL;
#else
    return __Pyx_cached_runtime_version;
#endif
}

/* IterNextPlain (used by CoroutineBase) */
#if CYTHON_COMPILING_IN_LIMITED_API && __PYX_LIMITED_VERSION_HEX < 0x030A0000
static PyObject *__Pyx_GetBuiltinNext_LimitedAPI(void) {
    if (unlikely(!__pyx_mstate_global->__Pyx_GetBuiltinNext_LimitedAPI_cache))
        __pyx_mstate_global->__Pyx_GetBuiltinNext_LimitedAPI_cache = __Pyx_GetBuiltinName(__pyx_mstate_global->__pyx_n_u_next_2);
    return __pyx_mstate_global->__Pyx_GetBuiltinNext_LimitedAPI_cache;
}
#endif
static CYTHON_INLINE PyObject *__Pyx_PyIter_Next_Plain(PyObject *iterator) {
#if CYTHON_COMPILING_IN_LIMITED_API && __PYX_LIMITED_VERSION_HEX < 0x030A0000
    PyObject *result;
    PyObject *next = __Pyx_GetBuiltinNext_LimitedAPI();
    if (unlikely(!next)) return NULL;
    result = PyObject_CallFunctionObjArgs(next, iterator, NULL);
    return result;
#else
    (void)__Pyx_GetBuiltinName; // only for early limited API
    iternextfunc iternext = __Pyx_PyObject_GetIterNextFunc(iterator);
    assert(iternext);
    return iternext(iterator);
#endif
}

/* ReturnWithStopIteration (used by CoroutineBase) */
static void __Pyx__ReturnWithStopIteration(PyObject* value, int async);
static CYTHON_INLINE void __Pyx_ReturnWithStopIteration(PyObject* value, int async, int iternext) {
    if (value == Py_None) {
        if (async || !iternext)
            PyErr_SetNone(async ? PyExc_StopAsyncIteration : PyExc_StopIteration);
        return;
    }
    __Pyx__ReturnWithStopIteration(value, async);
}
static void __Pyx__ReturnWithStopIteration(PyObject* value, int async) {
#if CYTHON_COMPILING_IN_CPYTHON
    __Pyx_PyThreadState_declare
#endif
    PyObject *exc;
    PyObject *exc_type = async ? PyExc_StopAsyncIteration : PyExc_StopIteration;
#if CYTHON_COMPILING_IN_CPYTHON
    if ((PY_VERSION_HEX >= (0x030C00A6)) || unlikely(PyTuple_Check(value) || PyExceptionInstance_Check(value))) {
        if (PY_VERSION_HEX >= (0x030e00A1)) {
            exc = __Pyx_PyObject_CallOneArg(exc_type, value);
        } else {
            PyObject *args_tuple = PyTuple_New(1);
            if (unlikely(!args_tuple)) return;
            Py_INCREF(value);
            PyTuple_SET_ITEM(args_tuple, 0, value);
            exc = PyObject_Call(exc_type, args_tuple, NULL);
            Py_DECREF(args_tuple);
        }
        if (unlikely(!exc)) return;
    } else {
        Py_INCREF(value);
        exc = value;
    }
    #if CYTHON_FAST_THREAD_STATE
    __Pyx_PyThreadState_assign
    #if CYTHON_USE_EXC_INFO_STACK
    if (!__pyx_tstate->exc_info->exc_value)
    #else
    if (!__pyx_tstate->exc_type)
    #endif
    {
        Py_INCREF(exc_type);
        __Pyx_ErrRestore(exc_type, exc, NULL);
        return;
    }
    #endif
#else
    exc = __Pyx_PyObject_CallOneArg(exc_type, value);
    if (unlikely(!exc)) return;
#endif
    PyErr_SetObject(exc_type, exc);
    Py_DECREF(exc);
}

/* CoroutineBase (used by Generator) */
#if !CYTHON_COMPILING_IN_LIMITED_API
#include <frameobject.h>
#if PY_VERSION_HEX >= 0x030b00a6 && !defined(PYPY_VERSION)
  #ifndef Py_BUILD_CORE
    #define Py_BUILD_CORE 1
  #endif
  #include "internal/pycore_frame.h"
#endif
#endif // CYTHON_COMPILING_IN_LIMITED_API
static CYTHON_INLINE void
__Pyx_Coroutine_Undelegate(__pyx_CoroutineObject *gen) {
#if CYTHON_USE_AM_SEND
    gen->yieldfrom_am_send = NULL;
#endif
    Py_CLEAR(gen->yieldfrom);
}
static int __Pyx_PyGen__FetchStopIterationValue(PyThreadState *__pyx_tstate, PyObject **pvalue) {
    PyObject *et, *ev, *tb;
    PyObject *value = NULL;
    CYTHON_UNUSED_VAR(__pyx_tstate);
    __Pyx_ErrFetch(&et, &ev, &tb);
    if (!et) {
        Py_XDECREF(tb);
        Py_XDECREF(ev);
        Py_INCREF(Py_None);
        *pvalue = Py_None;
        return 0;
    }
    if (likely(et == PyExc_StopIteration)) {
        if (!ev) {
            Py_INCREF(Py_None);
            value = Py_None;
        }
        else if (likely(__Pyx_IS_TYPE(ev, (PyTypeObject*)PyExc_StopIteration))) {
            #if CYTHON_COMPILING_IN_LIMITED_API || CYTHON_COMPILING_IN_GRAAL
            value = PyObject_GetAttr(ev, __pyx_mstate_global->__pyx_n_u_value);
            if (unlikely(!value)) goto limited_api_failure;
            #else
            value = ((PyStopIterationObject *)ev)->value;
            Py_INCREF(value);
            #endif
            Py_DECREF(ev);
        }
        else if (unlikely(PyTuple_Check(ev))) {
            Py_ssize_t tuple_size = __Pyx_PyTuple_GET_SIZE(ev);
            #if !CYTHON_ASSUME_SAFE_SIZE
            if (unlikely(tuple_size < 0)) {
                Py_XDECREF(tb);
                Py_DECREF(ev);
                Py_DECREF(et);
                return -1;
            }
            #endif
            if (tuple_size >= 1) {
#if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
                value = PyTuple_GET_ITEM(ev, 0);
                Py_INCREF(value);
#elif CYTHON_ASSUME_SAFE_MACROS
                value = PySequence_ITEM(ev, 0);
#else
                value = PySequence_GetItem(ev, 0);
                if (!value) goto limited_api_failure;
#endif
            } else {
                Py_INCREF(Py_None);
                value = Py_None;
            }
            Py_DECREF(ev);
        }
        else if (!__Pyx_TypeCheck(ev, (PyTypeObject*)PyExc_StopIteration)) {
            value = ev;
        }
        if (likely(value)) {
            Py_XDECREF(tb);
            Py_DECREF(et);
            *pvalue = value;
            return 0;
        }
    } else if (!__Pyx_PyErr_GivenExceptionMatches(et, PyExc_StopIteration)) {
        __Pyx_ErrRestore(et, ev, tb);
        return -1;
    }
    PyErr_NormalizeException(&et, &ev, &tb);
    if (unlikely(!PyObject_TypeCheck(ev, (PyTypeObject*)PyExc_StopIteration))) {
        __Pyx_ErrRestore(et, ev, tb);
        return -1;
    }
    Py_XDECREF(tb);
    Py_DECREF(et);
#if CYTHON_COMPILING_IN_LIMITED_API
    value = PyObject_GetAttr(ev, __pyx_mstate_global->__pyx_n_u_value);
#else
    value = ((PyStopIterationObject *)ev)->value;
    Py_INCREF(value);
#endif
    Py_DECREF(ev);
#if CYTHON_COMPILING_IN_LIMITED_API
    if (unlikely(!value)) return -1;
#endif
    *pvalue = value;
    return 0;
#if CYTHON_COMPILING_IN_LIMITED_API || CYTHON_COMPILING_IN_GRAAL || !CYTHON_ASSUME_SAFE_MACROS
  limited_api_failure:
    Py_XDECREF(et);
    Py_XDECREF(tb);
    Py_XDECREF(ev);
    return -1;
#endif
}
static CYTHON_INLINE
__Pyx_PySendResult __Pyx_Coroutine_status_from_result(PyObject **retval) {
    if (*retval) {
        return PYGEN_NEXT;
    } else if (likely(__Pyx_PyGen__FetchStopIterationValue(__Pyx_PyThreadState_Current, retval) == 0)) {
        return PYGEN_RETURN;
    } else {
        return PYGEN_ERROR;
    }
}
static CYTHON_INLINE
void __Pyx_Coroutine_ExceptionClear(__Pyx_ExcInfoStruct *exc_state) {
#if PY_VERSION_HEX >= 0x030B00a4
    Py_CLEAR(exc_state->exc_value);
#else
    PyObject *t, *v, *tb;
    t = exc_state->exc_type;
    v = exc_state->exc_value;
    tb = exc_state->exc_traceback;
    exc_state->exc_type = NULL;
    exc_state->exc_value = NULL;
    exc_state->exc_traceback = NULL;
    Py_XDECREF(t);
    Py_XDECREF(v);
    Py_XDECREF(tb);
#endif
}
#define __Pyx_Coroutine_AlreadyRunningError(gen)  (__Pyx__Coroutine_AlreadyRunningError(gen), (PyObject*)NULL)
static void __Pyx__Coroutine_AlreadyRunningError(__pyx_CoroutineObject *gen) {
    const char *msg;
    CYTHON_MAYBE_UNUSED_VAR(gen);
    if ((0)) {
    #ifdef __Pyx_Coroutine_USED
    } else if (__Pyx_Coroutine_Check((PyObject*)gen)) {
        msg = "coroutine already executing";
    #endif
    #ifdef __Pyx_AsyncGen_USED
    } else if (__Pyx_AsyncGen_CheckExact((PyObject*)gen)) {
        msg = "async generator already executing";
    #endif
    } else {
        msg = "generator already executing";
    }
    PyErr_SetString(PyExc_ValueError, msg);
}
static void __Pyx_Coroutine_AlreadyTerminatedError(PyObject *gen, PyObject *value, int closing) {
    CYTHON_MAYBE_UNUSED_VAR(gen);
    CYTHON_MAYBE_UNUSED_VAR(closing);
    #ifdef __Pyx_Coroutine_USED
    if (!closing && __Pyx_Coroutine_Check(gen)) {
        PyErr_SetString(PyExc_RuntimeError, "cannot reuse already awaited coroutine");
    } else
    #endif
    if (value) {
        #ifdef __Pyx_AsyncGen_USED
        if (__Pyx_AsyncGen_CheckExact(gen))
            PyErr_SetNone(PyExc_StopAsyncIteration);
        else
        #endif
        PyErr_SetNone(PyExc_StopIteration);
    }
}
static
__Pyx_PySendResult __Pyx_Coroutine_SendEx(__pyx_CoroutineObject *self, PyObject *value, PyObject **result, int closing) {
    __Pyx_PyThreadState_declare
    PyThreadState *tstate;
    __Pyx_ExcInfoStruct *exc_state;
    PyObject *retval;
    assert(__Pyx_Coroutine_get_is_running(self));  // Callers should ensure is_running
    if (unlikely(self->resume_label == -1)) {
        __Pyx_Coroutine_AlreadyTerminatedError((PyObject*)self, value, closing);
        return PYGEN_ERROR;
    }
#if CYTHON_FAST_THREAD_STATE
    __Pyx_PyThreadState_assign
    tstate = __pyx_tstate;
#else
    tstate = __Pyx_PyThreadState_Current;
#endif
    exc_state = &self->gi_exc_state;
    if (exc_state->exc_value) {
        #if CYTHON_COMPILING_IN_LIMITED_API || CYTHON_COMPILING_IN_PYPY
        #else
        PyObject *exc_tb;
        #if PY_VERSION_HEX >= 0x030B00a4 && !CYTHON_COMPILING_IN_CPYTHON
        exc_tb = PyException_GetTraceback(exc_state->exc_value);
        #elif PY_VERSION_HEX >= 0x030B00a4
        exc_tb = ((PyBaseExceptionObject*) exc_state->exc_value)->traceback;
        #else
        exc_tb = exc_state->exc_traceback;
        #endif
        if (exc_tb) {
            PyTracebackObject *tb = (PyTracebackObject *) exc_tb;
            PyFrameObject *f = tb->tb_frame;
            assert(f->f_back == NULL);
            #if PY_VERSION_HEX >= 0x030B00A1
            f->f_back = PyThreadState_GetFrame(tstate);
            #else
            Py_XINCREF(tstate->frame);
            f->f_back = tstate->frame;
            #endif
            #if PY_VERSION_HEX >= 0x030B00a4 && !CYTHON_COMPILING_IN_CPYTHON
            Py_DECREF(exc_tb);
            #endif
        }
        #endif
    }
#if CYTHON_USE_EXC_INFO_STACK
    exc_state->previous_item = tstate->exc_info;
    tstate->exc_info = exc_state;
#else
    if (exc_state->exc_type) {
        __Pyx_ExceptionSwap(&exc_state->exc_type, &exc_state->exc_value, &exc_state->exc_traceback);
    } else {
        __Pyx_Coroutine_ExceptionClear(exc_state);
        __Pyx_ExceptionSave(&exc_state->exc_type, &exc_state->exc_value, &exc_state->exc_traceback);
    }
#endif
    retval = self->body(self, tstate, value);
#if CYTHON_USE_EXC_INFO_STACK
    exc_state = &self->gi_exc_state;
    tstate->exc_info = exc_state->previous_item;
    exc_state->previous_item = NULL;
    __Pyx_Coroutine_ResetFrameBackpointer(exc_state);
#endif
    *result = retval;
    if (self->resume_label == -1) {
        return likely(retval) ? PYGEN_RETURN : PYGEN_ERROR;
    }
    return PYGEN_NEXT;
}
static CYTHON_INLINE void __Pyx_Coroutine_ResetFrameBackpointer(__Pyx_ExcInfoStruct *exc_state) {
#if CYTHON_COMPILING_IN_PYPY || CYTHON_COMPILING_IN_LIMITED_API
    CYTHON_UNUSED_VAR(exc_state);
#else
    PyObject *exc_tb;
    #if PY_VERSION_HEX >= 0x030B00a4
    if (!exc_state->exc_value) return;
    exc_tb = PyException_GetTraceback(exc_state->exc_value);
    #else
    exc_tb = exc_state->exc_traceback;
    #endif
    if (likely(exc_tb)) {
        PyTracebackObject *tb = (PyTracebackObject *) exc_tb;
        PyFrameObject *f = tb->tb_frame;
        Py_CLEAR(f->f_back);
        #if PY_VERSION_HEX >= 0x030B00a4
        Py_DECREF(exc_tb);
        #endif
    }
#endif
}
#define __Pyx_Coroutine_MethodReturnFromResult(gen, result, retval, iternext)\
    ((result) == PYGEN_NEXT ? (retval) : __Pyx__Coroutine_MethodReturnFromResult(gen, result, retval, iternext))
static PyObject *
__Pyx__Coroutine_MethodReturnFromResult(PyObject* gen, __Pyx_PySendResult result, PyObject *retval, int iternext) {
    CYTHON_MAYBE_UNUSED_VAR(gen);
    if (likely(result == PYGEN_RETURN)) {
        int is_async = 0;
        #ifdef __Pyx_AsyncGen_USED
        is_async = __Pyx_AsyncGen_CheckExact(gen);
        #endif
        __Pyx_ReturnWithStopIteration(retval, is_async, iternext);
        Py_XDECREF(retval);
    }
    return NULL;
}
#if CYTHON_COMPILING_IN_CPYTHON
static CYTHON_INLINE
PyObject *__Pyx_PyGen_Send(PyGenObject *gen, PyObject *arg) {
#if PY_VERSION_HEX <= 0x030A00A1
    return _PyGen_Send(gen, arg);
#else
    PyObject *result;
    if (PyIter_Send((PyObject*)gen, arg ? arg : Py_None, &result) == PYGEN_RETURN) {
        if (PyAsyncGen_CheckExact(gen)) {
            assert(result == Py_None);
            PyErr_SetNone(PyExc_StopAsyncIteration);
        }
        else if (result == Py_None) {
            PyErr_SetNone(PyExc_StopIteration);
        }
        else {
#if PY_VERSION_HEX < 0x030d00A1
            _PyGen_SetStopIterationValue(result);
#else
            if (!PyTuple_Check(result) && !PyExceptionInstance_Check(result)) {
                PyErr_SetObject(PyExc_StopIteration, result);
            } else {
                PyObject *exc = __Pyx_PyObject_CallOneArg(PyExc_StopIteration, result);
                if (likely(exc != NULL)) {
                    PyErr_SetObject(PyExc_StopIteration, exc);
                    Py_DECREF(exc);
                }
            }
#endif
        }
        Py_DECREF(result);
        result = NULL;
    }
    return result;
#endif
}
#endif
static CYTHON_INLINE __Pyx_PySendResult
__Pyx_Coroutine_FinishDelegation(__pyx_CoroutineObject *gen, PyObject** retval) {
    __Pyx_PySendResult result;
    PyObject *val = NULL;
    assert(__Pyx_Coroutine_get_is_running(gen));
    __Pyx_Coroutine_Undelegate(gen);
    __Pyx_PyGen__FetchStopIterationValue(__Pyx_PyThreadState_Current, &val);
    result = __Pyx_Coroutine_SendEx(gen, val, retval, 0);
    Py_XDECREF(val);
    return result;
}
#if CYTHON_USE_AM_SEND
static __Pyx_PySendResult
__Pyx_Coroutine_SendToDelegate(__pyx_CoroutineObject *gen, __Pyx_pyiter_sendfunc gen_am_send, PyObject *value, PyObject **retval) {
    PyObject *ret = NULL;
    __Pyx_PySendResult delegate_result, result;
    assert(__Pyx_Coroutine_get_is_running(gen));
    delegate_result = gen_am_send(gen->yieldfrom, value, &ret);
    if (delegate_result == PYGEN_NEXT) {
        assert (ret != NULL);
        *retval = ret;
        return PYGEN_NEXT;
    }
    assert (delegate_result != PYGEN_ERROR || ret == NULL);
    __Pyx_Coroutine_Undelegate(gen);
    result = __Pyx_Coroutine_SendEx(gen, ret, retval, 0);
    Py_XDECREF(ret);
    return result;
}
#endif
static PyObject *__Pyx_Coroutine_Send(PyObject *self, PyObject *value) {
    PyObject *retval = NULL;
    __Pyx_PySendResult result = __Pyx_Coroutine_AmSend(self, value, &retval);
    return __Pyx_Coroutine_MethodReturnFromResult(self, result, retval, 0);
}
static __Pyx_PySendResult
__Pyx_Coroutine_AmSend(PyObject *self, PyObject *value, PyObject **retval) {
    __Pyx_PySendResult result;
    __pyx_CoroutineObject *gen = (__pyx_CoroutineObject*) self;
    if (unlikely(__Pyx_Coroutine_test_and_set_is_running(gen))) {
        *retval = __Pyx_Coroutine_AlreadyRunningError(gen);
        return PYGEN_ERROR;
    }
    #if CYTHON_USE_AM_SEND
    if (gen->yieldfrom_am_send) {
        result = __Pyx_Coroutine_SendToDelegate(gen, gen->yieldfrom_am_send, value, retval);
    } else
    #endif
    if (gen->yieldfrom) {
        PyObject *yf = gen->yieldfrom;
        PyObject *ret;
      #if !CYTHON_USE_AM_SEND
        #ifdef __Pyx_Generator_USED
        if (__Pyx_Generator_CheckExact(yf)) {
            ret = __Pyx_Coroutine_Send(yf, value);
        } else
        #endif
        #ifdef __Pyx_Coroutine_USED
        if (__Pyx_Coroutine_Check(yf)) {
            ret = __Pyx_Coroutine_Send(yf, value);
        } else
        #endif
        #ifdef __Pyx_AsyncGen_USED
        if (__pyx_PyAsyncGenASend_CheckExact(yf)) {
            ret = __Pyx_async_gen_asend_send(yf, value);
        } else
        #endif
        #if CYTHON_COMPILING_IN_CPYTHON
        if (PyGen_CheckExact(yf)) {
            ret = __Pyx_PyGen_Send((PyGenObject*)yf, value == Py_None ? NULL : value);
        } else
        if (PyCoro_CheckExact(yf)) {
            ret = __Pyx_PyGen_Send((PyGenObject*)yf, value == Py_None ? NULL : value);
        } else
        #endif
      #endif
        {
            #if !CYTHON_COMPILING_IN_LIMITED_API || __PYX_LIMITED_VERSION_HEX >= 0x03080000
            if (value == Py_None && PyIter_Check(yf))
                ret = __Pyx_PyIter_Next_Plain(yf);
            else
            #endif
                ret = __Pyx_PyObject_CallMethod1(yf, __pyx_mstate_global->__pyx_n_u_send, value);
        }
        if (likely(ret)) {
            __Pyx_Coroutine_unset_is_running(gen);
            *retval = ret;
            return PYGEN_NEXT;
        }
        result = __Pyx_Coroutine_FinishDelegation(gen, retval);
    } else {
        result = __Pyx_Coroutine_SendEx(gen, value, retval, 0);
    }
    __Pyx_Coroutine_unset_is_running(gen);
    return result;
}
static int __Pyx_Coroutine_CloseIter(__pyx_CoroutineObject *gen, PyObject *yf) {
    __Pyx_PySendResult result;
    PyObject *retval = NULL;
    CYTHON_UNUSED_VAR(gen);
    assert(__Pyx_Coroutine_get_is_running(gen));
    #ifdef __Pyx_Generator_USED
    if (__Pyx_Generator_CheckExact(yf)) {
        result = __Pyx_Coroutine_Close(yf, &retval);
    } else
    #endif
    #ifdef __Pyx_Coroutine_USED
    if (__Pyx_Coroutine_Check(yf)) {
        result = __Pyx_Coroutine_Close(yf, &retval);
    } else
    if (__Pyx_CoroutineAwait_CheckExact(yf)) {
        result = __Pyx_CoroutineAwait_Close((__pyx_CoroutineAwaitObject*)yf);
    } else
    #endif
    #ifdef __Pyx_AsyncGen_USED
    if (__pyx_PyAsyncGenASend_CheckExact(yf)) {
        retval = __Pyx_async_gen_asend_close(yf, NULL);
        result = PYGEN_RETURN;
    } else
    if (__pyx_PyAsyncGenAThrow_CheckExact(yf)) {
        retval = __Pyx_async_gen_athrow_close(yf, NULL);
        result = PYGEN_RETURN;
    } else
    #endif
    {
        PyObject *meth;
        result = PYGEN_RETURN;
        meth = __Pyx_PyObject_GetAttrStrNoError(yf, __pyx_mstate_global->__pyx_n_u_close);
        if (unlikely(!meth)) {
            if (unlikely(PyErr_Occurred())) {
                PyErr_WriteUnraisable(yf);
            }
        } else {
            retval = __Pyx_PyObject_CallNoArg(meth);
            Py_DECREF(meth);
            if (unlikely(!retval)) {
                result = PYGEN_ERROR;
            }
        }
    }
    Py_XDECREF(retval);
    return result == PYGEN_ERROR ? -1 : 0;
}
static PyObject *__Pyx_Generator_Next(PyObject *self) {
    __Pyx_PySendResult result;
    PyObject *retval = NULL;
    __pyx_CoroutineObject *gen = (__pyx_CoroutineObject*) self;
    if (unlikely(__Pyx_Coroutine_test_and_set_is_running(gen))) {
        return __Pyx_Coroutine_AlreadyRunningError(gen);
    }
    #if CYTHON_USE_AM_SEND
    if (gen->yieldfrom_am_send) {
        result = __Pyx_Coroutine_SendToDelegate(gen, gen->yieldfrom_am_send, Py_None, &retval);
    } else
    #endif
    if (gen->yieldfrom) {
        PyObject *yf = gen->yieldfrom;
        PyObject *ret;
        #ifdef __Pyx_Generator_USED
        if (__Pyx_Generator_CheckExact(yf)) {
            ret = __Pyx_Generator_Next(yf);
        } else
        #endif
        #ifdef __Pyx_Coroutine_USED
        if (__Pyx_Coroutine_CheckExact(yf)) {
            ret = __Pyx_Coroutine_Send(yf, Py_None);
        } else
        #endif
        #if CYTHON_COMPILING_IN_CPYTHON && (PY_VERSION_HEX < 0x030A00A3 || !CYTHON_USE_AM_SEND)
        if (PyGen_CheckExact(yf)) {
            ret = __Pyx_PyGen_Send((PyGenObject*)yf, NULL);
        } else
        #endif
            ret = __Pyx_PyIter_Next_Plain(yf);
        if (likely(ret)) {
            __Pyx_Coroutine_unset_is_running(gen);
            return ret;
        }
        result = __Pyx_Coroutine_FinishDelegation(gen, &retval);
    } else {
        result = __Pyx_Coroutine_SendEx(gen, Py_None, &retval, 0);
    }
    __Pyx_Coroutine_unset_is_running(gen);
    return __Pyx_Coroutine_MethodReturnFromResult(self, result, retval, 1);
}
static PyObject *__Pyx_Coroutine_Close_Method(PyObject *self, PyObject *arg) {
    PyObject *retval = NULL;
    __Pyx_PySendResult result;
    CYTHON_UNUSED_VAR(arg);
    result = __Pyx_Coroutine_Close(self, &retval);
    if (unlikely(result == PYGEN_ERROR))
        return NULL;
    Py_XDECREF(retval);
    Py_RETURN_NONE;
}
static __Pyx_PySendResult
__Pyx_Coroutine_Close(PyObject *self, PyObject **retval) {
    __pyx_CoroutineObject *gen = (__pyx_CoroutineObject *) self;
    __Pyx_PySendResult result;
    PyObject *yf;
    int err = 0;
    if (unlikely(__Pyx_Coroutine_test_and_set_is_running(gen))) {
        *retval = __Pyx_Coroutine_AlreadyRunningError(gen);
        return PYGEN_ERROR;
    }
    yf = gen->yieldfrom;
    if (yf) {
        Py_INCREF(yf);
        err = __Pyx_Coroutine_CloseIter(gen, yf);
        __Pyx_Coroutine_Undelegate(gen);
        Py_DECREF(yf);
    }
    if (err == 0)
        PyErr_SetNone(PyExc_GeneratorExit);
    result = __Pyx_Coroutine_SendEx(gen, NULL, retval, 1);
    if (result == PYGEN_ERROR) {
        __Pyx_PyThreadState_declare
        __Pyx_PyThreadState_assign
        __Pyx_Coroutine_unset_is_running(gen);
        if (!__Pyx_PyErr_Occurred()) {
            return PYGEN_RETURN;
        } else if (likely(__Pyx_PyErr_ExceptionMatches2(PyExc_GeneratorExit, PyExc_StopIteration))) {
            __Pyx_PyErr_Clear();
            return PYGEN_RETURN;
        }
        return PYGEN_ERROR;
    } else if (likely(result == PYGEN_RETURN && *retval == Py_None)) {
        __Pyx_Coroutine_unset_is_running(gen);
        return PYGEN_RETURN;
    } else {
        const char *msg;
        Py_DECREF(*retval);
        *retval = NULL;
        if ((0)) {
        #ifdef __Pyx_Coroutine_USED
        } else if (__Pyx_Coroutine_Check(self)) {
            msg = "coroutine ignored GeneratorExit";
        #endif
        #ifdef __Pyx_AsyncGen_USED
        } else if (__Pyx_AsyncGen_CheckExact(self)) {
            msg = "async generator ignored GeneratorExit";
        #endif
        } else {
            msg = "generator ignored GeneratorExit";
        }
        PyErr_SetString(PyExc_RuntimeError, msg);
        __Pyx_Coroutine_unset_is_running(gen);
        return PYGEN_ERROR;
    }
}
static PyObject *__Pyx__Coroutine_Throw(PyObject *self, PyObject *typ, PyObject *val, PyObject *tb,
                                        PyObject *args, int close_on_genexit) {
    __pyx_CoroutineObject *gen = (__pyx_CoroutineObject *) self;
    PyObject *yf;
    if (unlikely(__Pyx_Coroutine_test_and_set_is_running(gen)))
        return __Pyx_Coroutine_AlreadyRunningError(gen);
    yf = gen->yieldfrom;
    if (yf) {
        __Pyx_PySendResult result;
        PyObject *ret;
        Py_INCREF(yf);
        if (__Pyx_PyErr_GivenExceptionMatches(typ, PyExc_GeneratorExit) && close_on_genexit) {
            int err = __Pyx_Coroutine_CloseIter(gen, yf);
            Py_DECREF(yf);
            __Pyx_Coroutine_Undelegate(gen);
            if (err < 0)
                goto propagate_exception;
            goto throw_here;
        }
        if (0
        #ifdef __Pyx_Generator_USED
            || __Pyx_Generator_CheckExact(yf)
        #endif
        #ifdef __Pyx_Coroutine_USED
            || __Pyx_Coroutine_Check(yf)
        #endif
            ) {
            ret = __Pyx__Coroutine_Throw(yf, typ, val, tb, args, close_on_genexit);
        #ifdef __Pyx_Coroutine_USED
        } else if (__Pyx_CoroutineAwait_CheckExact(yf)) {
            ret = __Pyx__Coroutine_Throw(((__pyx_CoroutineAwaitObject*)yf)->coroutine, typ, val, tb, args, close_on_genexit);
        #endif
        } else {
            PyObject *meth = __Pyx_PyObject_GetAttrStrNoError(yf, __pyx_mstate_global->__pyx_n_u_throw);
            if (unlikely(!meth)) {
                Py_DECREF(yf);
                if (unlikely(PyErr_Occurred())) {
                    __Pyx_Coroutine_unset_is_running(gen);
                    return NULL;
                }
                __Pyx_Coroutine_Undelegate(gen);
                goto throw_here;
            }
            if (likely(args)) {
                ret = __Pyx_PyObject_Call(meth, args, NULL);
            } else {
                PyObject *cargs[4] = {NULL, typ, val, tb};
                size_t nargs = 1U + (val != NULL) + (tb != NULL);
                ret = __Pyx_PyObject_FastCall(meth, cargs+1, nargs | __Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET);
            }
            Py_DECREF(meth);
        }
        Py_DECREF(yf);
        if (ret) {
            __Pyx_Coroutine_unset_is_running(gen);
            return ret;
        }
        result = __Pyx_Coroutine_FinishDelegation(gen, &ret);
        __Pyx_Coroutine_unset_is_running(gen);
        return __Pyx_Coroutine_MethodReturnFromResult(self, result, ret, 0);
    }
throw_here:
    __Pyx_Raise(typ, val, tb, NULL);
propagate_exception:
    {
        PyObject *retval = NULL;
        __Pyx_PySendResult result = __Pyx_Coroutine_SendEx(gen, NULL, &retval, 0);
        __Pyx_Coroutine_unset_is_running(gen);
        return __Pyx_Coroutine_MethodReturnFromResult(self, result, retval, 0);
    }
}
static PyObject *__Pyx_Coroutine_Throw(PyObject *self, PyObject *args) {
    PyObject *typ;
    PyObject *val = NULL;
    PyObject *tb = NULL;
    if (unlikely(!PyArg_UnpackTuple(args, "throw", 1, 3, &typ, &val, &tb)))
        return NULL;
    return __Pyx__Coroutine_Throw(self, typ, val, tb, args, 1);
}
static CYTHON_INLINE int __Pyx_Coroutine_traverse_excstate(__Pyx_ExcInfoStruct *exc_state, visitproc visit, void *arg) {
#if PY_VERSION_HEX >= 0x030B00a4
    Py_VISIT(exc_state->exc_value);
#else
    Py_VISIT(exc_state->exc_type);
    Py_VISIT(exc_state->exc_value);
    Py_VISIT(exc_state->exc_traceback);
#endif
    return 0;
}
static int __Pyx_Coroutine_traverse(__pyx_CoroutineObject *gen, visitproc visit, void *arg) {
    {
        int e = __Pyx_call_type_traverse((PyObject*)gen, 1, visit, arg);
        if (e) return e;
    }
    Py_VISIT(gen->closure);
    Py_VISIT(gen->classobj);
    Py_VISIT(gen->yieldfrom);
    return __Pyx_Coroutine_traverse_excstate(&gen->gi_exc_state, visit, arg);
}
static int __Pyx_Coroutine_clear(PyObject *self) {
    __pyx_CoroutineObject *gen = (__pyx_CoroutineObject *) self;
    Py_CLEAR(gen->closure);
    Py_CLEAR(gen->classobj);
    __Pyx_Coroutine_Undelegate(gen);
    __Pyx_Coroutine_ExceptionClear(&gen->gi_exc_state);
#ifdef __Pyx_AsyncGen_USED
    if (__Pyx_AsyncGen_CheckExact(self)) {
        Py_CLEAR(((__pyx_PyAsyncGenObject*)gen)->ag_finalizer);
    }
#endif
    Py_CLEAR(gen->gi_code);
    Py_CLEAR(gen->gi_frame);
    Py_CLEAR(gen->gi_name);
    Py_CLEAR(gen->gi_qualname);
    Py_CLEAR(gen->gi_modulename);
    return 0;
}
static void __Pyx_Coroutine_dealloc(PyObject *self) {
    __pyx_CoroutineObject *gen = (__pyx_CoroutineObject *) self;
    PyObject_GC_UnTrack(gen);
    #if PY_VERSION_HEX < 0x030C0000 || CYTHON_COMPILING_IN_LIMITED_API
    if (gen->gi_weakreflist != NULL)
    #endif
        PyObject_ClearWeakRefs(self);
    if (gen->resume_label >= 0) {
        PyObject_GC_Track(self);
#if CYTHON_USE_TP_FINALIZE
        if (unlikely(PyObject_CallFinalizerFromDealloc(self)))
#else
        {
            destructor del = __Pyx_PyObject_GetSlot(gen, tp_del, destructor);
            if (del) del(self);
        }
        if (unlikely(Py_REFCNT(self) > 0))
#endif
        {
            return;
        }
        PyObject_GC_UnTrack(self);
    }
#ifdef __Pyx_AsyncGen_USED
    if (__Pyx_AsyncGen_CheckExact(self)) {
        /* We have to handle this case for asynchronous generators
           right here, because this code has to be between UNTRACK
           and GC_Del. */
        Py_CLEAR(((__pyx_PyAsyncGenObject*)self)->ag_finalizer);
    }
#endif
    __Pyx_Coroutine_clear(self);
    __Pyx_PyHeapTypeObject_GC_Del(gen);
}
#if CYTHON_USE_TP_FINALIZE
static void __Pyx_Coroutine_del(PyObject *self) {
    PyObject *error_type, *error_value, *error_traceback;
    __pyx_CoroutineObject *gen = (__pyx_CoroutineObject *) self;
    __Pyx_PyThreadState_declare
    if (gen->resume_label < 0) {
        return;
    }
    __Pyx_PyThreadState_assign
    __Pyx_ErrFetch(&error_type, &error_value, &error_traceback);
#ifdef __Pyx_AsyncGen_USED
    if (__Pyx_AsyncGen_CheckExact(self)) {
        __pyx_PyAsyncGenObject *agen = (__pyx_PyAsyncGenObject*)self;
        PyObject *finalizer = agen->ag_finalizer;
        if (finalizer && !agen->ag_closed) {
            PyObject *res = __Pyx_PyObject_CallOneArg(finalizer, self);
            if (unlikely(!res)) {
                PyErr_WriteUnraisable(self);
            } else {
                Py_DECREF(res);
            }
            __Pyx_ErrRestore(error_type, error_value, error_traceback);
            return;
        }
    }
#endif
    if (unlikely(gen->resume_label == 0 && !error_value)) {
#ifdef __Pyx_Coroutine_USED
#ifdef __Pyx_Generator_USED
    if (!__Pyx_Generator_CheckExact(self))
#endif
        {
        PyObject_GC_UnTrack(self);
        if (unlikely(PyErr_WarnFormat(PyExc_RuntimeWarning, 1, "coroutine '%.50S' was never awaited", gen->gi_qualname) < 0))
            PyErr_WriteUnraisable(self);
        PyObject_GC_Track(self);
        }
#endif
    } else {
        PyObject *retval = NULL;
        __Pyx_PySendResult result = __Pyx_Coroutine_Close(self, &retval);
        if (result == PYGEN_ERROR) {
            PyErr_WriteUnraisable(self);
        } else {
            Py_XDECREF(retval);
        }
    }
    __Pyx_ErrRestore(error_type, error_value, error_traceback);
}
#endif
static PyObject *
__Pyx_Coroutine_get_name(__pyx_CoroutineObject *self, void *context)
{
    PyObject *name = self->gi_name;
    CYTHON_UNUSED_VAR(context);
    if (unlikely(!name)) name = Py_None;
    Py_INCREF(name);
    return name;
}
static int
__Pyx_Coroutine_set_name(__pyx_CoroutineObject *self, PyObject *value, void *context)
{
    CYTHON_UNUSED_VAR(context);
    if (unlikely(value == NULL || !PyUnicode_Check(value))) {
        PyErr_SetString(PyExc_TypeError,
                        "__name__ must be set to a string object");
        return -1;
    }
    Py_INCREF(value);
    __Pyx_Py_XDECREF_SET(self->gi_name, value);
    return 0;
}
static PyObject *
__Pyx_Coroutine_get_qualname(__pyx_CoroutineObject *self, void *context)
{
    PyObject *name = self->gi_qualname;
    CYTHON_UNUSED_VAR(context);
    if (unlikely(!name)) name = Py_None;
    Py_INCREF(name);
    return name;
}
static int
__Pyx_Coroutine_set_qualname(__pyx_CoroutineObject *self, PyObject *value, void *context)
{
    CYTHON_UNUSED_VAR(context);
    if (unlikely(value == NULL || !PyUnicode_Check(value))) {
        PyErr_SetString(PyExc_TypeError,
                        "__qualname__ must be set to a string object");
        return -1;
    }
    Py_INCREF(value);
    __Pyx_Py_XDECREF_SET(self->gi_qualname, value);
    return 0;
}
#if !CYTHON_COMPILING_IN_LIMITED_API
static PyObject *
__Pyx__Coroutine_get_frame_locked(__pyx_CoroutineObject *self) {
    PyObject *frame;
    frame = self->gi_frame;
    if (!frame) {
        if (unlikely(!self->gi_code)) {
            Py_RETURN_NONE;
        }
        PyObject *globals = PyDict_New();
        if (unlikely(!globals)) return NULL;
        frame = (PyObject *) PyFrame_New(
            PyThreadState_Get(),            /*PyThreadState *tstate,*/
            (PyCodeObject*) self->gi_code,  /*PyCodeObject *code,*/
            globals,                        /*PyObject *globals,*/
            0                               /*PyObject *locals*/
        );
        Py_DECREF(globals);
        if (unlikely(!frame))
            return NULL;
        if (unlikely(self->gi_frame)) {
            Py_DECREF(frame);
            frame = self->gi_frame;
        } else {
            self->gi_frame = frame;
        }
    }
    Py_INCREF(frame);
    return frame;
}
#endif
static PyObject *
__Pyx__Coroutine_get_frame(__pyx_CoroutineObject *self)
{
#if !CYTHON_COMPILING_IN_LIMITED_API
    PyObject *frame;
    __Pyx_BEGIN_CRITICAL_SECTION((PyObject*)self);
    frame = __Pyx__Coroutine_get_frame_locked(self);
    __Pyx_END_CRITICAL_SECTION();
    return frame;
#else
    CYTHON_UNUSED_VAR(self);
    Py_RETURN_NONE;
#endif
}
static PyObject *
__Pyx_Coroutine_get_frame(__pyx_CoroutineObject *self, void *context) {
    CYTHON_UNUSED_VAR(context);
    PyObject *frame = self->gi_frame;
    if (frame)
        return __Pyx_NewRef(frame);
    return __Pyx__Coroutine_get_frame(self);
}
static __pyx_CoroutineObject *__Pyx__Coroutine_New(
            PyTypeObject* type, __pyx_coroutine_body_t body, PyObject *code, PyObject *closure,
            PyObject *name, PyObject *qualname, PyObject *module_name) {
    __pyx_CoroutineObject *gen = PyObject_GC_New(__pyx_CoroutineObject, type);
    if (unlikely(!gen))
        return NULL;
    return __Pyx__Coroutine_NewInit(gen, body, code, closure, name, qualname, module_name);
}
static __pyx_CoroutineObject *__Pyx__Coroutine_NewInit(
            __pyx_CoroutineObject *gen, __pyx_coroutine_body_t body, PyObject *code, PyObject *closure,
            PyObject *name, PyObject *qualname, PyObject *module_name) {
    gen->body = body;
    gen->closure = closure;
    Py_XINCREF(closure);
    gen->is_running = 0;
    gen->resume_label = 0;
    gen->classobj = NULL;
    gen->yieldfrom = NULL;
    gen->yieldfrom_am_send = NULL;
    #if PY_VERSION_HEX >= 0x030B00a4 && !CYTHON_COMPILING_IN_LIMITED_API
    gen->gi_exc_state.exc_value = NULL;
    #else
    gen->gi_exc_state.exc_type = NULL;
    gen->gi_exc_state.exc_value = NULL;
    gen->gi_exc_state.exc_traceback = NULL;
    #endif
#if CYTHON_USE_EXC_INFO_STACK
    gen->gi_exc_state.previous_item = NULL;
#endif
#if PY_VERSION_HEX < 0x030C0000 || CYTHON_COMPILING_IN_LIMITED_API
    gen->gi_weakreflist = NULL;
#endif
    Py_XINCREF(qualname);
    gen->gi_qualname = qualname;
    Py_XINCREF(name);
    gen->gi_name = name;
    Py_XINCREF(module_name);
    gen->gi_modulename = module_name;
    Py_XINCREF(code);
    gen->gi_code = code;
    gen->gi_frame = NULL;
    PyObject_GC_Track(gen);
    return gen;
}
static char __Pyx_Coroutine_test_and_set_is_running(__pyx_CoroutineObject *gen) {
    char result;
    __Pyx_BEGIN_CRITICAL_SECTION((PyObject*)gen);
    result = gen->is_running;
    gen->is_running = 1;
    __Pyx_END_CRITICAL_SECTION();
    return result;
}
static void __Pyx_Coroutine_unset_is_running(__pyx_CoroutineObject *gen) {
    __Pyx_BEGIN_CRITICAL_SECTION((PyObject*)gen);
    assert(gen->is_running);
    gen->is_running = 0;
    __Pyx_END_CRITICAL_SECTION();
}
static char __Pyx_Coroutine_get_is_running(__pyx_CoroutineObject *gen) {
    char result;
    __Pyx_BEGIN_CRITICAL_SECTION((PyObject*)gen);
    result = gen->is_running;
    __Pyx_END_CRITICAL_SECTION();
    return result;
}
static PyObject *__Pyx_Coroutine_get_is_running_getter(PyObject *gen, void *closure) {
    CYTHON_UNUSED_VAR(closure);
    char result = __Pyx_Coroutine_get_is_running((__pyx_CoroutineObject*)gen);
    if (result) Py_RETURN_TRUE;
    else Py_RETURN_FALSE;
}
#if __PYX_HAS_PY_AM_SEND == 2
static void __Pyx_SetBackportTypeAmSend(PyTypeObject *type, __Pyx_PyAsyncMethodsStruct *static_amsend_methods, __Pyx_pyiter_sendfunc am_send) {
    Py_ssize_t ptr_offset = (char*)(type->tp_as_async) - (char*)type;
    if (ptr_offset < 0 || ptr_offset > type->tp_basicsize) {
        return;
    }
    memcpy((void*)static_amsend_methods, (void*)(type->tp_as_async), sizeof(*type->tp_as_async));
    static_amsend_methods->am_send = am_send;
    type->tp_as_async = __Pyx_SlotTpAsAsync(static_amsend_methods);
}
#endif
static PyObject *__Pyx_Coroutine_fail_reduce_ex(PyObject *self, PyObject *arg) {
    CYTHON_UNUSED_VAR(arg);
    __Pyx_TypeName self_type_name = __Pyx_PyType_GetFullyQualifiedName(Py_TYPE((PyObject*)self));
    PyErr_Format(PyExc_TypeError, "cannot pickle '" __Pyx_FMT_TYPENAME "' object",
                         self_type_name);
    __Pyx_DECREF_TypeName(self_type_name);
    return NULL;
}

/* Generator */
static PyMethodDef __pyx_Generator_methods[] = {
    {"send", (PyCFunction) __Pyx_Coroutine_Send, METH_O,
     PyDoc_STR("send(arg) -> send 'arg' into generator,\nreturn next yielded value or raise StopIteration.")},
    {"throw", (PyCFunction) __Pyx_Coroutine_Throw, METH_VARARGS,
     PyDoc_STR("throw(typ[,val[,tb]]) -> raise exception in generator,\nreturn next yielded value or raise StopIteration.")},
    {"close", (PyCFunction) __Pyx_Coroutine_Close_Method, METH_NOARGS,
     PyDoc_STR("close() -> raise GeneratorExit inside generator.")},
    {"__reduce_ex__", (PyCFunction) __Pyx_Coroutine_fail_reduce_ex, METH_O, 0},
    {"__reduce__", (PyCFunction) __Pyx_Coroutine_fail_reduce_ex, METH_NOARGS, 0},
    {0, 0, 0, 0}
};
static PyMemberDef __pyx_Generator_memberlist[] = {
    {"gi_yieldfrom", T_OBJECT, offsetof(__pyx_CoroutineObject, yieldfrom), READONLY,
     PyDoc_STR("object being iterated by 'yield from', or None")},
    {"gi_code", T_OBJECT, offsetof(__pyx_CoroutineObject, gi_code), READONLY, NULL},
    {"__module__", T_OBJECT, offsetof(__pyx_CoroutineObject, gi_modulename), 0, 0},
#if PY_VERSION_HEX < 0x030C0000 || CYTHON_COMPILING_IN_LIMITED_API
    {"__weaklistoffset__", T_PYSSIZET, offsetof(__pyx_CoroutineObject, gi_weakreflist), READONLY, 0},
#endif
    {0, 0, 0, 0, 0}
};
static PyGetSetDef __pyx_Generator_getsets[] = {
    {"__name__", (getter)__Pyx_Coroutine_get_name, (setter)__Pyx_Coroutine_set_name,
     PyDoc_STR("name of the generator"), 0},
    {"__qualname__", (getter)__Pyx_Coroutine_get_qualname, (setter)__Pyx_Coroutine_set_qualname,
     PyDoc_STR("qualified name of the generator"), 0},
    {"gi_frame", (getter)__Pyx_Coroutine_get_frame, NULL,
     PyDoc_STR("Frame of the generator"), 0},
    {"gi_running", __Pyx_Coroutine_get_is_running_getter, NULL, NULL, NULL},
    {0, 0, 0, 0, 0}
};
static PyType_Slot __pyx_GeneratorType_slots[] = {
    {Py_tp_dealloc, (void *)__Pyx_Coroutine_dealloc},
    {Py_tp_traverse, (void *)__Pyx_Coroutine_traverse},
    {Py_tp_iter, (void *)PyObject_SelfIter},
    {Py_tp_iternext, (void *)__Pyx_Generator_Next},
    {Py_tp_methods, (void *)__pyx_Generator_methods},
    {Py_tp_members, (void *)__pyx_Generator_memberlist},
    {Py_tp_getset, (void *)__pyx_Generator_getsets},
    {Py_tp_getattro, (void *) PyObject_GenericGetAttr},
#if CYTHON_USE_TP_FINALIZE
    {Py_tp_finalize, (void *)__Pyx_Coroutine_del},
#endif
#if __PYX_HAS_PY_AM_SEND == 1
    {Py_am_send, (void *)__Pyx_Coroutine_AmSend},
#endif
    {0, 0},
};
static PyType_Spec __pyx_GeneratorType_spec = {
    __PYX_TYPE_MODULE_PREFIX "generator",
    sizeof(__pyx_CoroutineObject),
    0,
#if PY_VERSION_HEX >= 0x030C0000 && !CYTHON_COMPILING_IN_LIMITED_API
    Py_TPFLAGS_MANAGED_WEAKREF |
#endif
    Py_TPFLAGS_IMMUTABLETYPE | Py_TPFLAGS_DISALLOW_INSTANTIATION |
    Py_TPFLAGS_DEFAULT | Py_TPFLAGS_HAVE_GC | __Pyx_TPFLAGS_HAVE_AM_SEND,
    __pyx_GeneratorType_slots
};
#if __PYX_HAS_PY_AM_SEND == 2
static __Pyx_PyAsyncMethodsStruct __pyx_Generator_as_async;
#endif
static int __pyx_Generator_init(PyObject *module) {
    __pyx_mstatetype *mstate = __Pyx_PyModule_GetState(module);
    mstate->__pyx_GeneratorType = __Pyx_FetchCommonTypeFromSpec(
        mstate->__pyx_CommonTypesMetaclassType, module, &__pyx_GeneratorType_spec, NULL);
    if (unlikely(!mstate->__pyx_GeneratorType)) {
        return -1;
    }
#if __PYX_HAS_PY_AM_SEND == 2
    __Pyx_SetBackportTypeAmSend(mstate->__pyx_GeneratorType, &__pyx_Generator_as_async, &__Pyx_Coroutine_AmSend);
#endif
    return 0;
}
static PyObject *__Pyx_Generator_GetInlinedResult(PyObject *self) {
    __pyx_CoroutineObject *gen = (__pyx_CoroutineObject*) self;
    PyObject *retval = NULL;
    if (unlikely(__Pyx_Coroutine_test_and_set_is_running(gen))) {
        return __Pyx_Coroutine_AlreadyRunningError(gen);
    }
    __Pyx_PySendResult result = __Pyx_Coroutine_SendEx(gen, Py_None, &retval, 0);
    __Pyx_Coroutine_unset_is_running(gen);
    (void) result;
    assert (result == PYGEN_RETURN || result == PYGEN_ERROR);
    assert ((result == PYGEN_RETURN && retval != NULL) || (result == PYGEN_ERROR && retval == NULL));
    return retval;
}

/* CheckBinaryVersion */
static int __Pyx_check_binary_version(unsigned long ct_version, unsigned long rt_version, int allow_newer) {
    const unsigned long MAJOR_MINOR = 0xFFFF0000UL;
    if ((rt_version & MAJOR_MINOR) == (ct_version & MAJOR_MINOR))
        return 0;
    if (likely(allow_newer && (rt_version & MAJOR_MINOR) > (ct_version & MAJOR_MINOR)))
        return 1;
    {
        char message[200];
        PyOS_snprintf(message, sizeof(message),
                      "compile time Python version %d.%d "
                      "of module '%.100s' "
                      "%s "
                      "runtime version %d.%d",
                       (int) (ct_version >> 24), (int) ((ct_version >> 16) & 0xFF),
                       __Pyx_MODULE_NAME,
                       (allow_newer) ? "was newer than" : "does not match",
                       (int) (rt_version >> 24), (int) ((rt_version >> 16) & 0xFF)
       );
        return PyErr_WarnEx(NULL, message, 1);
    }
}

/* NewCodeObj */
#if CYTHON_COMPILING_IN_LIMITED_API
    static PyObject* __Pyx__PyCode_New(int a, int p, int k, int l, int s, int f,
                                       PyObject *code, PyObject *c, PyObject* n, PyObject *v,
                                       PyObject *fv, PyObject *cell, PyObject* fn,
                                       PyObject *name, int fline, PyObject *lnos) {
        PyObject *exception_table = NULL;
        PyObject *types_module=NULL, *code_type=NULL, *result=NULL;
        #if __PYX_LIMITED_VERSION_HEX < 0x030b0000
        PyObject *version_info;
        PyObject *py_minor_version = NULL;
        #endif
        long minor_version = 0;
        PyObject *type, *value, *traceback;
        PyErr_Fetch(&type, &value, &traceback);
        #if __PYX_LIMITED_VERSION_HEX >= 0x030b0000
        minor_version = 11;
        #else
        if (!(version_info = PySys_GetObject("version_info"))) goto end;
        if (!(py_minor_version = PySequence_GetItem(version_info, 1))) goto end;
        minor_version = PyLong_AsLong(py_minor_version);
        Py_DECREF(py_minor_version);
        if (minor_version == -1 && PyErr_Occurred()) goto end;
        #endif
        if (!(types_module = PyImport_ImportModule("types"))) goto end;
        if (!(code_type = PyObject_GetAttrString(types_module, "CodeType"))) goto end;
        if (minor_version <= 7) {
            (void)p;
            result = PyObject_CallFunction(code_type, "iiiiiOOOOOOiOOO", a, k, l, s, f, code,
                          c, n, v, fn, name, fline, lnos, fv, cell);
        } else if (minor_version <= 10) {
            result = PyObject_CallFunction(code_type, "iiiiiiOOOOOOiOOO", a,p, k, l, s, f, code,
                          c, n, v, fn, name, fline, lnos, fv, cell);
        } else {
            if (!(exception_table = PyBytes_FromStringAndSize(NULL, 0))) goto end;
            result = PyObject_CallFunction(code_type, "iiiiiiOOOOOOOiOOOO", a,p, k, l, s, f, code,
                          c, n, v, fn, name, name, fline, lnos, exception_table, fv, cell);
        }
    end:
        Py_XDECREF(code_type);
        Py_XDECREF(exception_table);
        Py_XDECREF(types_module);
        if (type) {
            PyErr_Restore(type, value, traceback);
        }
        return result;
    }
#elif PY_VERSION_HEX >= 0x030B0000
  static PyCodeObject* __Pyx__PyCode_New(int a, int p, int k, int l, int s, int f,
                                         PyObject *code, PyObject *c, PyObject* n, PyObject *v,
                                         PyObject *fv, PyObject *cell, PyObject* fn,
                                         PyObject *name, int fline, PyObject *lnos) {
    PyCodeObject *result;
    result =
      #if PY_VERSION_HEX >= 0x030C0000
        PyUnstable_Code_NewWithPosOnlyArgs
      #else
        PyCode_NewWithPosOnlyArgs
      #endif
        (a, p, k, l, s, f, code, c, n, v, fv, cell, fn, name, name, fline, lnos, __pyx_mstate_global->__pyx_empty_bytes);
    #if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x030c00A1
    if (likely(result))
        result->_co_firsttraceable = 0;
    #endif
    return result;
  }
#elif !CYTHON_COMPILING_IN_PYPY
  #define __Pyx__PyCode_New(a, p, k, l, s, f, code, c, n, v, fv, cell, fn, name, fline, lnos)\
          PyCode_NewWithPosOnlyArgs(a, p, k, l, s, f, code, c, n, v, fv, cell, fn, name, fline, lnos)
#else
  #define __Pyx__PyCode_New(a, p, k, l, s, f, code, c, n, v, fv, cell, fn, name, fline, lnos)\
          PyCode_New(a, k, l, s, f, code, c, n, v, fv, cell, fn, name, fline, lnos)
#endif
static PyObject* __Pyx_PyCode_New(
        const __Pyx_PyCode_New_function_description descr,
        PyObject * const *varnames,
        PyObject *filename,
        PyObject *funcname,
        PyObject *line_table,
        PyObject *tuple_dedup_map
) {
    PyObject *code_obj = NULL, *varnames_tuple_dedup = NULL, *code_bytes = NULL;
    Py_ssize_t var_count = (Py_ssize_t) descr.nlocals;
    PyObject *varnames_tuple = PyTuple_New(var_count);
    if (unlikely(!varnames_tuple)) return NULL;
    for (Py_ssize_t i=0; i < var_count; i++) {
        Py_INCREF(varnames[i]);
        if (__Pyx_PyTuple_SET_ITEM(varnames_tuple, i, varnames[i]) != (0)) goto done;
    }
    #if CYTHON_COMPILING_IN_LIMITED_API
    varnames_tuple_dedup = PyDict_GetItem(tuple_dedup_map, varnames_tuple);
    if (!varnames_tuple_dedup) {
        if (unlikely(PyDict_SetItem(tuple_dedup_map, varnames_tuple, varnames_tuple) < 0)) goto done;
        varnames_tuple_dedup = varnames_tuple;
    }
    #else
    varnames_tuple_dedup = PyDict_SetDefault(tuple_dedup_map, varnames_tuple, varnames_tuple);
    if (unlikely(!varnames_tuple_dedup)) goto done;
    #endif
    #if CYTHON_AVOID_BORROWED_REFS
    Py_INCREF(varnames_tuple_dedup);
    #endif
    if (__PYX_LIMITED_VERSION_HEX >= (0x030b0000) && line_table != NULL && !CYTHON_COMPILING_IN_GRAAL) {
        Py_ssize_t line_table_length = __Pyx_PyBytes_GET_SIZE(line_table);
        #if !CYTHON_ASSUME_SAFE_SIZE
        if (unlikely(line_table_length == -1)) goto done;
        #endif
        Py_ssize_t code_len = (line_table_length * 2 + 4) & ~3LL;
        code_bytes = PyBytes_FromStringAndSize(NULL, code_len);
        if (unlikely(!code_bytes)) goto done;
        char* c_code_bytes = PyBytes_AsString(code_bytes);
        if (unlikely(!c_code_bytes)) goto done;
        memset(c_code_bytes, 0, (size_t) code_len);
    }
    code_obj = (PyObject*) __Pyx__PyCode_New(
        (int) descr.argcount,
        (int) descr.num_posonly_args,
        (int) descr.num_kwonly_args,
        (int) descr.nlocals,
        0,
        (int) descr.flags,
        code_bytes ? code_bytes : __pyx_mstate_global->__pyx_empty_bytes,
        __pyx_mstate_global->__pyx_empty_tuple,
        __pyx_mstate_global->__pyx_empty_tuple,
        varnames_tuple_dedup,
        __pyx_mstate_global->__pyx_empty_tuple,
        __pyx_mstate_global->__pyx_empty_tuple,
        filename,
        funcname,
        (int) descr.first_line,
        (__PYX_LIMITED_VERSION_HEX >= (0x030b0000) && line_table) ? line_table : __pyx_mstate_global->__pyx_empty_bytes
    );
done:
    Py_XDECREF(code_bytes);
    #if CYTHON_AVOID_BORROWED_REFS
    Py_XDECREF(varnames_tuple_dedup);
    #endif
    Py_DECREF(varnames_tuple);
    return code_obj;
}

/* DecompressString */
static PyObject *__Pyx_DecompressString(const char *s, Py_ssize_t length, int algo) {
    PyObject *module = NULL, *decompress, *compressed_bytes, *decompressed;
    const char* module_name = algo == 3 ? "compression.zstd" : algo == 2 ? "bz2" : "zlib";
    PyObject *methodname = PyUnicode_FromString("decompress");
    if (unlikely(!methodname)) return NULL;
    #if __PYX_LIMITED_VERSION_HEX >= 0x030e0000
    if (algo == 3) {
        PyObject *fromlist = Py_BuildValue("[O]", methodname);
        if (unlikely(!fromlist)) goto bad;
        module = PyImport_ImportModuleLevel("compression.zstd", NULL, NULL, fromlist, 0);
        Py_DECREF(fromlist);
    } else
    #endif
        module = PyImport_ImportModule(module_name);
    if (unlikely(!module)) goto import_failed;
    decompress = PyObject_GetAttr(module, methodname);
    if (unlikely(!decompress)) goto import_failed;
    {
        #ifdef __cplusplus
            char *memview_bytes = const_cast<char*>(s);
        #else
            #if defined(__clang__)
              #pragma clang diagnostic push
              #pragma clang diagnostic ignored "-Wcast-qual"
            #elif !defined(__INTEL_COMPILER) && defined(__GNUC__)
              #pragma GCC diagnostic push
              #pragma GCC diagnostic ignored "-Wcast-qual"
            #endif
            char *memview_bytes = (char*) s;
            #if defined(__clang__)
              #pragma clang diagnostic pop
            #elif !defined(__INTEL_COMPILER) && defined(__GNUC__)
              #pragma GCC diagnostic pop
            #endif
        #endif
        #if CYTHON_COMPILING_IN_LIMITED_API && !defined(PyBUF_READ)
        int memview_flags = 0x100;
        #else
        int memview_flags = PyBUF_READ;
        #endif
        compressed_bytes = PyMemoryView_FromMemory(memview_bytes, length, memview_flags);
    }
    if (unlikely(!compressed_bytes)) {
        Py_DECREF(decompress);
        goto bad;
    }
    decompressed = PyObject_CallFunctionObjArgs(decompress, compressed_bytes, NULL);
    Py_DECREF(compressed_bytes);
    Py_DECREF(decompress);
    Py_DECREF(module);
    Py_DECREF(methodname);
    return decompressed;
import_failed:
    PyErr_Format(PyExc_ImportError,
        "Failed to import '%.20s.decompress' - cannot initialise module strings. "
        "String compression was configured with the C macro 'CYTHON_COMPRESS_STRINGS=%d'.",
        module_name, algo);
bad:
    Py_XDECREF(module);
    Py_DECREF(methodname);
    return NULL;
}

#include <string.h>
static CYTHON_INLINE Py_ssize_t __Pyx_ssize_strlen(const char *s) {
    size_t len = strlen(s);
    if (unlikely(len > (size_t) PY_SSIZE_T_MAX)) {
        PyErr_SetString(PyExc_OverflowError, "byte string is too long");
        return -1;
    }
    return (Py_ssize_t) len;
}
static CYTHON_INLINE PyObject* __Pyx_PyUnicode_FromString(const char* c_str) {
    Py_ssize_t len = __Pyx_ssize_strlen(c_str);
    if (unlikely(len < 0)) return NULL;
    return __Pyx_PyUnicode_FromStringAndSize(c_str, len);
}
static CYTHON_INLINE PyObject* __Pyx_PyByteArray_FromString(const char* c_str) {
    Py_ssize_t len = __Pyx_ssize_strlen(c_str);
    if (unlikely(len < 0)) return NULL;
    return PyByteArray_FromStringAndSize(c_str, len);
}
static CYTHON_INLINE const char* __Pyx_PyObject_AsString(PyObject* o) {
    Py_ssize_t ignore;
    return __Pyx_PyObject_AsStringAndSize(o, &ignore);
}
#if __PYX_DEFAULT_STRING_ENCODING_IS_ASCII || __PYX_DEFAULT_STRING_ENCODING_IS_UTF8
static CYTHON_INLINE const char* __Pyx_PyUnicode_AsStringAndSize(PyObject* o, Py_ssize_t *length) {
    if (unlikely(__Pyx_PyUnicode_READY(o) == -1)) return NULL;
#if CYTHON_COMPILING_IN_LIMITED_API
    {
        const char* result;
        Py_ssize_t unicode_length;
        CYTHON_MAYBE_UNUSED_VAR(unicode_length); // only for __PYX_DEFAULT_STRING_ENCODING_IS_ASCII
        #if __PYX_LIMITED_VERSION_HEX < 0x030A0000
        if (unlikely(PyArg_Parse(o, "s#", &result, length) < 0)) return NULL;
        #else
        result = PyUnicode_AsUTF8AndSize(o, length);
        #endif
        #if __PYX_DEFAULT_STRING_ENCODING_IS_ASCII
        unicode_length = PyUnicode_GetLength(o);
        if (unlikely(unicode_length < 0)) return NULL;
        if (unlikely(unicode_length != *length)) {
            PyUnicode_AsASCIIString(o);
            return NULL;
        }
        #endif
        return result;
    }
#else
#if __PYX_DEFAULT_STRING_ENCODING_IS_ASCII
    if (likely(PyUnicode_IS_ASCII(o))) {
        *length = PyUnicode_GET_LENGTH(o);
        return PyUnicode_AsUTF8(o);
    } else {
        PyUnicode_AsASCIIString(o);
        return NULL;
    }
#else
    return PyUnicode_AsUTF8AndSize(o, length);
#endif
#endif
}
#endif
static CYTHON_INLINE const char* __Pyx_PyObject_AsStringAndSize(PyObject* o, Py_ssize_t *length) {
#if __PYX_DEFAULT_STRING_ENCODING_IS_ASCII || __PYX_DEFAULT_STRING_ENCODING_IS_UTF8
    if (PyUnicode_Check(o)) {
        return __Pyx_PyUnicode_AsStringAndSize(o, length);
    } else
#endif
    if (PyByteArray_Check(o)) {
#if (CYTHON_ASSUME_SAFE_SIZE && CYTHON_ASSUME_SAFE_MACROS) || (CYTHON_COMPILING_IN_PYPY && (defined(PyByteArray_AS_STRING) && defined(PyByteArray_GET_SIZE)))
        *length = PyByteArray_GET_SIZE(o);
        return PyByteArray_AS_STRING(o);
#else
        *length = PyByteArray_Size(o);
        if (*length == -1) return NULL;
        return PyByteArray_AsString(o);
#endif
    } else
    {
        char* result;
        int r = PyBytes_AsStringAndSize(o, &result, length);
        if (unlikely(r < 0)) {
            return NULL;
        } else {
            return result;
        }
    }
}
static CYTHON_INLINE int __Pyx_PyObject_IsTrue(PyObject* x) {
   int is_true = x == Py_True;
   if (is_true | (x == Py_False) | (x == Py_None)) return is_true;
   else return PyObject_IsTrue(x);
}
static CYTHON_INLINE int __Pyx_PyObject_IsTrueAndDecref(PyObject* x) {
    int retval;
    if (unlikely(!x)) return -1;
    retval = __Pyx_PyObject_IsTrue(x);
    Py_DECREF(x);
    return retval;
}
static PyObject* __Pyx_PyNumber_LongWrongResultType(PyObject* result) {
    __Pyx_TypeName result_type_name = __Pyx_PyType_GetFullyQualifiedName(Py_TYPE(result));
    if (PyLong_Check(result)) {
        if (PyErr_WarnFormat(PyExc_DeprecationWarning, 1,
                "__int__ returned non-int (type " __Pyx_FMT_TYPENAME ").  "
                "The ability to return an instance of a strict subclass of int is deprecated, "
                "and may be removed in a future version of Python.",
                result_type_name)) {
            __Pyx_DECREF_TypeName(result_type_name);
            Py_DECREF(result);
            return NULL;
        }
        __Pyx_DECREF_TypeName(result_type_name);
        return result;
    }
    PyErr_Format(PyExc_TypeError,
                 "__int__ returned non-int (type " __Pyx_FMT_TYPENAME ")",
                 result_type_name);
    __Pyx_DECREF_TypeName(result_type_name);
    Py_DECREF(result);
    return NULL;
}
static CYTHON_INLINE PyObject* __Pyx_PyNumber_Long(PyObject* x) {
#if CYTHON_USE_TYPE_SLOTS
  PyNumberMethods *m;
#endif
  PyObject *res = NULL;
  if (likely(PyLong_Check(x)))
      return __Pyx_NewRef(x);
#if CYTHON_USE_TYPE_SLOTS
  m = Py_TYPE(x)->tp_as_number;
  if (likely(m && m->nb_int)) {
      res = m->nb_int(x);
  }
#else
  if (!PyBytes_CheckExact(x) && !PyUnicode_CheckExact(x)) {
      res = PyNumber_Long(x);
  }
#endif
  if (likely(res)) {
      if (unlikely(!PyLong_CheckExact(res))) {
          return __Pyx_PyNumber_LongWrongResultType(res);
      }
  }
  else if (!PyErr_Occurred()) {
      PyErr_SetString(PyExc_TypeError,
                      "an integer is required");
  }
  return res;
}
static CYTHON_INLINE Py_ssize_t __Pyx_PyIndex_AsSsize_t(PyObject* b) {
  Py_ssize_t ival;
  PyObject *x;
  if (likely(PyLong_CheckExact(b))) {
    #if CYTHON_USE_PYLONG_INTERNALS
    if (likely(__Pyx_PyLong_IsCompact(b))) {
        return __Pyx_PyLong_CompactValue(b);
    } else {
      const digit* digits = __Pyx_PyLong_Digits(b);
      const Py_ssize_t size = __Pyx_PyLong_SignedDigitCount(b);
      switch (size) {
         case 2:
           if (8 * sizeof(Py_ssize_t) > 2 * PyLong_SHIFT) {
             return (Py_ssize_t) (((((size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0]));
           }
           break;
         case -2:
           if (8 * sizeof(Py_ssize_t) > 2 * PyLong_SHIFT) {
             return -(Py_ssize_t) (((((size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0]));
           }
           break;
         case 3:
           if (8 * sizeof(Py_ssize_t) > 3 * PyLong_SHIFT) {
             return (Py_ssize_t) (((((((size_t)digits[2]) << PyLong_SHIFT) | (size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0]));
           }
           break;
         case -3:
           if (8 * sizeof(Py_ssize_t) > 3 * PyLong_SHIFT) {
             return -(Py_ssize_t) (((((((size_t)digits[2]) << PyLong_SHIFT) | (size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0]));
           }
           break;
         case 4:
           if (8 * sizeof(Py_ssize_t) > 4 * PyLong_SHIFT) {
             return (Py_ssize_t) (((((((((size_t)digits[3]) << PyLong_SHIFT) | (size_t)digits[2]) << PyLong_SHIFT) | (size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0]));
           }
           break;
         case -4:
           if (8 * sizeof(Py_ssize_t) > 4 * PyLong_SHIFT) {
             return -(Py_ssize_t) (((((((((size_t)digits[3]) << PyLong_SHIFT) | (size_t)digits[2]) << PyLong_SHIFT) | (size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0]));
           }
           break;
      }
    }
    #endif
    return PyLong_AsSsize_t(b);
  }
  x = PyNumber_Index(b);
  if (!x) return -1;
  ival = PyLong_AsSsize_t(x);
  Py_DECREF(x);
  return ival;
}
static CYTHON_INLINE Py_hash_t __Pyx_PyIndex_AsHash_t(PyObject* o) {
  if (sizeof(Py_hash_t) == sizeof(Py_ssize_t)) {
    return (Py_hash_t) __Pyx_PyIndex_AsSsize_t(o);
  } else {
    Py_ssize_t ival;
    PyObject *x;
    x = PyNumber_Index(o);
    if (!x) return -1;
    ival = PyLong_AsLong(x);
    Py_DECREF(x);
    return ival;
  }
}
static CYTHON_INLINE PyObject *__Pyx_Owned_Py_None(int b) {
    CYTHON_UNUSED_VAR(b);
    return __Pyx_NewRef(Py_None);
}
static CYTHON_INLINE PyObject * __Pyx_PyBool_FromLong(long b) {
  return __Pyx_NewRef(b ? Py_True: Py_False);
}
static CYTHON_INLINE PyObject * __Pyx_PyLong_FromSize_t(size_t ival) {
    return PyLong_FromSize_t(ival);
}


/* MultiPhaseInitModuleState */
#if CYTHON_PEP489_MULTI_PHASE_INIT && CYTHON_USE_MODULE_STATE
#ifndef CYTHON_MODULE_STATE_LOOKUP_THREAD_SAFE
#if (CYTHON_COMPILING_IN_LIMITED_API || PY_VERSION_HEX >= 0x030C0000)
  #define CYTHON_MODULE_STATE_LOOKUP_THREAD_SAFE 1
#else
  #define CYTHON_MODULE_STATE_LOOKUP_THREAD_SAFE 0
#endif
#endif
#if CYTHON_MODULE_STATE_LOOKUP_THREAD_SAFE && !CYTHON_ATOMICS
#error "Module state with PEP489 requires atomics. Currently that's one of\
 C11, C++11, gcc atomic intrinsics or MSVC atomic intrinsics"
#endif
#if !CYTHON_MODULE_STATE_LOOKUP_THREAD_SAFE
#define __Pyx_ModuleStateLookup_Lock()
#define __Pyx_ModuleStateLookup_Unlock()
#elif !CYTHON_COMPILING_IN_LIMITED_API && PY_VERSION_HEX >= 0x030d0000
static PyMutex __Pyx_ModuleStateLookup_mutex = {0};
#define __Pyx_ModuleStateLookup_Lock() PyMutex_Lock(&__Pyx_ModuleStateLookup_mutex)
#define __Pyx_ModuleStateLookup_Unlock() PyMutex_Unlock(&__Pyx_ModuleStateLookup_mutex)
#elif defined(__cplusplus) && __cplusplus >= 201103L
#include <mutex>
static std::mutex __Pyx_ModuleStateLookup_mutex;
#define __Pyx_ModuleStateLookup_Lock() __Pyx_ModuleStateLookup_mutex.lock()
#define __Pyx_ModuleStateLookup_Unlock() __Pyx_ModuleStateLookup_mutex.unlock()
#elif defined(__STDC_VERSION__) && (__STDC_VERSION__ > 201112L) && !defined(__STDC_NO_THREADS__)
#include <threads.h>
static mtx_t __Pyx_ModuleStateLookup_mutex;
static once_flag __Pyx_ModuleStateLookup_mutex_once_flag = ONCE_FLAG_INIT;
static void __Pyx_ModuleStateLookup_initialize_mutex(void) {
    mtx_init(&__Pyx_ModuleStateLookup_mutex, mtx_plain);
}
#define __Pyx_ModuleStateLookup_Lock()\
  call_once(&__Pyx_ModuleStateLookup_mutex_once_flag, __Pyx_ModuleStateLookup_initialize_mutex);\
  mtx_lock(&__Pyx_ModuleStateLookup_mutex)
#define __Pyx_ModuleStateLookup_Unlock() mtx_unlock(&__Pyx_ModuleStateLookup_mutex)
#elif defined(HAVE_PTHREAD_H)
#include <pthread.h>
static pthread_mutex_t __Pyx_ModuleStateLookup_mutex = PTHREAD_MUTEX_INITIALIZER;
#define __Pyx_ModuleStateLookup_Lock() pthread_mutex_lock(&__Pyx_ModuleStateLookup_mutex)
#define __Pyx_ModuleStateLookup_Unlock() pthread_mutex_unlock(&__Pyx_ModuleStateLookup_mutex)
#elif defined(_WIN32)
#include <Windows.h>  // synchapi.h on its own doesn't work
static SRWLOCK __Pyx_ModuleStateLookup_mutex = SRWLOCK_INIT;
#define __Pyx_ModuleStateLookup_Lock() AcquireSRWLockExclusive(&__Pyx_ModuleStateLookup_mutex)
#define __Pyx_ModuleStateLookup_Unlock() ReleaseSRWLockExclusive(&__Pyx_ModuleStateLookup_mutex)
#else
#error "No suitable lock available for CYTHON_MODULE_STATE_LOOKUP_THREAD_SAFE.\
 Requires C standard >= C11, or C++ standard >= C++11,\
 or pthreads, or the Windows 32 API, or Python >= 3.13."
#endif
typedef struct {
    int64_t id;
    PyObject *module;
} __Pyx_InterpreterIdAndModule;
typedef struct {
    char interpreter_id_as_index;
    Py_ssize_t count;
    Py_ssize_t allocated;
    __Pyx_InterpreterIdAndModule table[1];
} __Pyx_ModuleStateLookupData;
#define __PYX_MODULE_STATE_LOOKUP_SMALL_SIZE 32
#if CYTHON_MODULE_STATE_LOOKUP_THREAD_SAFE
static __pyx_atomic_int_type __Pyx_ModuleStateLookup_read_counter = 0;
#endif
#if CYTHON_MODULE_STATE_LOOKUP_THREAD_SAFE
static __pyx_atomic_ptr_type __Pyx_ModuleStateLookup_data = 0;
#else
static __Pyx_ModuleStateLookupData* __Pyx_ModuleStateLookup_data = NULL;
#endif
static __Pyx_InterpreterIdAndModule* __Pyx_State_FindModuleStateLookupTableLowerBound(
        __Pyx_InterpreterIdAndModule* table,
        Py_ssize_t count,
        int64_t interpreterId) {
    __Pyx_InterpreterIdAndModule* begin = table;
    __Pyx_InterpreterIdAndModule* end = begin + count;
    if (begin->id == interpreterId) {
        return begin;
    }
    while ((end - begin) > __PYX_MODULE_STATE_LOOKUP_SMALL_SIZE) {
        __Pyx_InterpreterIdAndModule* halfway = begin + (end - begin)/2;
        if (halfway->id == interpreterId) {
            return halfway;
        }
        if (halfway->id < interpreterId) {
            begin = halfway;
        } else {
            end = halfway;
        }
    }
    for (; begin < end; ++begin) {
        if (begin->id >= interpreterId) return begin;
    }
    return begin;
}
static PyObject *__Pyx_State_FindModule(CYTHON_UNUSED void* dummy) {
    int64_t interpreter_id = PyInterpreterState_GetID(__Pyx_PyInterpreterState_Get());
    if (interpreter_id == -1) return NULL;
#if CYTHON_MODULE_STATE_LOOKUP_THREAD_SAFE
    __Pyx_ModuleStateLookupData* data = (__Pyx_ModuleStateLookupData*)__pyx_atomic_pointer_load_relaxed(&__Pyx_ModuleStateLookup_data);
    {
        __pyx_atomic_incr_acq_rel(&__Pyx_ModuleStateLookup_read_counter);
        if (likely(data)) {
            __Pyx_ModuleStateLookupData* new_data = (__Pyx_ModuleStateLookupData*)__pyx_atomic_pointer_load_acquire(&__Pyx_ModuleStateLookup_data);
            if (likely(data == new_data)) {
                goto read_finished;
            }
        }
        __pyx_atomic_decr_acq_rel(&__Pyx_ModuleStateLookup_read_counter);
        __Pyx_ModuleStateLookup_Lock();
        __pyx_atomic_incr_relaxed(&__Pyx_ModuleStateLookup_read_counter);
        data = (__Pyx_ModuleStateLookupData*)__pyx_atomic_pointer_load_relaxed(&__Pyx_ModuleStateLookup_data);
        __Pyx_ModuleStateLookup_Unlock();
    }
  read_finished:;
#else
    __Pyx_ModuleStateLookupData* data = __Pyx_ModuleStateLookup_data;
#endif
    __Pyx_InterpreterIdAndModule* found = NULL;
    if (unlikely(!data)) goto end;
    if (data->interpreter_id_as_index) {
        if (interpreter_id < data->count) {
            found = data->table+interpreter_id;
        }
    } else {
        found = __Pyx_State_FindModuleStateLookupTableLowerBound(
            data->table, data->count, interpreter_id);
    }
  end:
    {
        PyObject *result=NULL;
        if (found && found->id == interpreter_id) {
            result = found->module;
        }
#if CYTHON_MODULE_STATE_LOOKUP_THREAD_SAFE
        __pyx_atomic_decr_acq_rel(&__Pyx_ModuleStateLookup_read_counter);
#endif
        return result;
    }
}
#if CYTHON_MODULE_STATE_LOOKUP_THREAD_SAFE
static void __Pyx_ModuleStateLookup_wait_until_no_readers(void) {
    while (__pyx_atomic_load(&__Pyx_ModuleStateLookup_read_counter) != 0);
}
#else
#define __Pyx_ModuleStateLookup_wait_until_no_readers()
#endif
static int __Pyx_State_AddModuleInterpIdAsIndex(__Pyx_ModuleStateLookupData **old_data, PyObject* module, int64_t interpreter_id) {
    Py_ssize_t to_allocate = (*old_data)->allocated;
    while (to_allocate <= interpreter_id) {
        if (to_allocate == 0) to_allocate = 1;
        else to_allocate *= 2;
    }
    __Pyx_ModuleStateLookupData *new_data = *old_data;
    if (to_allocate != (*old_data)->allocated) {
         new_data = (__Pyx_ModuleStateLookupData *)realloc(
            *old_data,
            sizeof(__Pyx_ModuleStateLookupData)+(to_allocate-1)*sizeof(__Pyx_InterpreterIdAndModule));
        if (!new_data) {
            PyErr_NoMemory();
            return -1;
        }
        for (Py_ssize_t i = new_data->allocated; i < to_allocate; ++i) {
            new_data->table[i].id = i;
            new_data->table[i].module = NULL;
        }
        new_data->allocated = to_allocate;
    }
    new_data->table[interpreter_id].module = module;
    if (new_data->count < interpreter_id+1) {
        new_data->count = interpreter_id+1;
    }
    *old_data = new_data;
    return 0;
}
static void __Pyx_State_ConvertFromInterpIdAsIndex(__Pyx_ModuleStateLookupData *data) {
    __Pyx_InterpreterIdAndModule *read = data->table;
    __Pyx_InterpreterIdAndModule *write = data->table;
    __Pyx_InterpreterIdAndModule *end = read + data->count;
    for (; read<end; ++read) {
        if (read->module) {
            write->id = read->id;
            write->module = read->module;
            ++write;
        }
    }
    data->count = write - data->table;
    for (; write<end; ++write) {
        write->id = 0;
        write->module = NULL;
    }
    data->interpreter_id_as_index = 0;
}
static int __Pyx_State_AddModule(PyObject* module, CYTHON_UNUSED void* dummy) {
    int64_t interpreter_id = PyInterpreterState_GetID(__Pyx_PyInterpreterState_Get());
    if (interpreter_id == -1) return -1;
    int result = 0;
    __Pyx_ModuleStateLookup_Lock();
#if CYTHON_MODULE_STATE_LOOKUP_THREAD_SAFE
    __Pyx_ModuleStateLookupData *old_data = (__Pyx_ModuleStateLookupData *)
            __pyx_atomic_pointer_exchange(&__Pyx_ModuleStateLookup_data, 0);
#else
    __Pyx_ModuleStateLookupData *old_data = __Pyx_ModuleStateLookup_data;
#endif
    __Pyx_ModuleStateLookupData *new_data = old_data;
    if (!new_data) {
        new_data = (__Pyx_ModuleStateLookupData *)calloc(1, sizeof(__Pyx_ModuleStateLookupData));
        if (!new_data) {
            result = -1;
            PyErr_NoMemory();
            goto end;
        }
        new_data->allocated = 1;
        new_data->interpreter_id_as_index = 1;
    }
    __Pyx_ModuleStateLookup_wait_until_no_readers();
    if (new_data->interpreter_id_as_index) {
        if (interpreter_id < __PYX_MODULE_STATE_LOOKUP_SMALL_SIZE) {
            result = __Pyx_State_AddModuleInterpIdAsIndex(&new_data, module, interpreter_id);
            goto end;
        }
        __Pyx_State_ConvertFromInterpIdAsIndex(new_data);
    }
    {
        Py_ssize_t insert_at = 0;
        {
            __Pyx_InterpreterIdAndModule* lower_bound = __Pyx_State_FindModuleStateLookupTableLowerBound(
                new_data->table, new_data->count, interpreter_id);
            assert(lower_bound);
            insert_at = lower_bound - new_data->table;
            if (unlikely(insert_at < new_data->count && lower_bound->id == interpreter_id)) {
                lower_bound->module = module;
                goto end;  // already in table, nothing more to do
            }
        }
        if (new_data->count+1 >= new_data->allocated) {
            Py_ssize_t to_allocate = (new_data->count+1)*2;
            new_data =
                (__Pyx_ModuleStateLookupData*)realloc(
                    new_data,
                    sizeof(__Pyx_ModuleStateLookupData) +
                    (to_allocate-1)*sizeof(__Pyx_InterpreterIdAndModule));
            if (!new_data) {
                result = -1;
                new_data = old_data;
                PyErr_NoMemory();
                goto end;
            }
            new_data->allocated = to_allocate;
        }
        ++new_data->count;
        int64_t last_id = interpreter_id;
        PyObject *last_module = module;
        for (Py_ssize_t i=insert_at; i<new_data->count; ++i) {
            int64_t current_id = new_data->table[i].id;
            new_data->table[i].id = last_id;
            last_id = current_id;
            PyObject *current_module = new_data->table[i].module;
            new_data->table[i].module = last_module;
            last_module = current_module;
        }
    }
  end:
#if CYTHON_MODULE_STATE_LOOKUP_THREAD_SAFE
    __pyx_atomic_pointer_exchange(&__Pyx_ModuleStateLookup_data, new_data);
#else
    __Pyx_ModuleStateLookup_data = new_data;
#endif
    __Pyx_ModuleStateLookup_Unlock();
    return result;
}
static int __Pyx_State_RemoveModule(CYTHON_UNUSED void* dummy) {
    int64_t interpreter_id = PyInterpreterState_GetID(__Pyx_PyInterpreterState_Get());
    if (interpreter_id == -1) return -1;
    __Pyx_ModuleStateLookup_Lock();
#if CYTHON_MODULE_STATE_LOOKUP_THREAD_SAFE
    __Pyx_ModuleStateLookupData *data = (__Pyx_ModuleStateLookupData *)
            __pyx_atomic_pointer_exchange(&__Pyx_ModuleStateLookup_data, 0);
#else
    __Pyx_ModuleStateLookupData *data = __Pyx_ModuleStateLookup_data;
#endif
    if (data->interpreter_id_as_index) {
        if (interpreter_id < data->count) {
            data->table[interpreter_id].module = NULL;
        }
        goto done;
    }
    {
        __Pyx_ModuleStateLookup_wait_until_no_readers();
        __Pyx_InterpreterIdAndModule* lower_bound = __Pyx_State_FindModuleStateLookupTableLowerBound(
            data->table, data->count, interpreter_id);
        if (!lower_bound) goto done;
        if (lower_bound->id != interpreter_id) goto done;
        __Pyx_InterpreterIdAndModule *end = data->table+data->count;
        for (;lower_bound<end-1; ++lower_bound) {
            lower_bound->id = (lower_bound+1)->id;
            lower_bound->module = (lower_bound+1)->module;
        }
    }
    --data->count;
    if (data->count == 0) {
        free(data);
        data = NULL;
    }
  done:
#if CYTHON_MODULE_STATE_LOOKUP_THREAD_SAFE
    __pyx_atomic_pointer_exchange(&__Pyx_ModuleStateLookup_data, data);
#else
    __Pyx_ModuleStateLookup_data = data;
#endif
    __Pyx_ModuleStateLookup_Unlock();
    return 0;
}
#endif

/* #### Code section: utility_code_pragmas_end ### */
#ifdef _MSC_VER
#pragma warning( pop )
#endif



/* #### Code section: end ### */
#endif /* Py_PYTHON_H */
